/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * BoTest.h	(S.Makhmali)
 * include/Bo/BoTest.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BOTEST__
# define __BOTEST__

/******************************************************************************
 * %% BeginSection: includes
 */

#include <Bo/libc/Stdio.h>
#include <Bo/libc/Fmtio.h>
#include <Bo/Funcjmp.h>

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -
# pragma mark [ definitions ]
# pragma mark -

# if __myc_support(NVM)
#  define BO_TEST_PRINTF(aFormat...) fprintf(stderr,aFormat)
#  define BO_TEST_PRINTF_NOTHING(aFormat...) ((void)0)
#  define BO_TEST_WPRINTF(aFormat...) fwprintf(stderr,aFormat)
#  define BO_TEST_WPRINTF_NOTHING(aFormat...) ((void)0)
# else
# if __myc_support(AVM)
#  define BO_TEST_PRINTF(/*aFormat,*/...) fprintf(stderr,/*aFormat,*/__VA_ARGS__)
#  define BO_TEST_PRINTF_NOTHING(/*aFormat,*/...) ((void)0)
#  define BO_TEST_WPRINTF(/*aFormat,*/...) fwprintf(stderr,/*aFormat,*/__VA_ARGS__)
#  define BO_TEST_WPRINTF_NOTHING(/*aFormat,*/...) ((void)0)
# endif /* __myc_support(AVM) */
# endif /* __myc_support(NVM) */

# ifdef _DEBUG
#  define BO_TEST_DBG_PRINTF BO_TEST_PRINTF
#  define BO_TEST_DBG_WPRINTF BO_TEST_WPRINTF
# else
#  define BO_TEST_DBG_PRINTF BO_TEST_PRINTF_NOTHING
#  define BO_TEST_DBG_WPRINTF BO_TEST_WPRINTF_NOTHING
# endif


# define _BO_TEST_MAKE_BIT(n) (1 << ((n) - 1))
# define _BO_TEST_SET(t,n) ((t) |= _BO_TEST_MAKE_BIT(n))
# define _BO_TEST_CLR(t,n) ((t) &= ~_BO_TEST_MAKE_BIT(n))
# define _BO_TEST_ISSET(t,n) (_BO_TEST_MAKE_BIT(n) & (t))
# define _BO_TEST_ZERO(t) ((t) = 0)


# define BO_TEST_DECLS(N) \
	u_int32_t tests; \
	bo_status_t testStatus; \
	unsigned testIndex; \
	bo_status_t testStatusv[N]

# define BO_TEST_SET(aTestNum) _BO_TEST_SET(tests, aTestNum)
# define BO_TEST_CLR(aTestNum) _BO_TEST_CLR(tests, aTestNum)
# define BO_TEST_ISSET(aTestNum) _BO_TEST_ISSET(tests, aTestNum)
# define BO_TEST_ZERO() _BO_TEST_ZERO(tests)
# define BO_TEST_SETALL() (tests = ~0L)

# define BO_TEST_INIT() \
	testStatus = 0; \
	testIndex = 1

# define BO_CURRENT_TEST testIndex

# define BO_TEST_BEGIN \
	testStatusv[testIndex - 1] = 0; \
	if (BO_TEST_ISSET(testIndex)) do {

# define BO_TEST_PRINTSELF() \
	fprintf(stderr, "test %d\n", testIndex)

# ifdef _DEBUG
#  define BO_TEST_DBG_PRINTSELF() BO_TEST_PRINTSELF()
# else
#  define BO_TEST_DBG_PRINTSELF() ((void)0)
# endif

# define BO_TEST_BREAKIFFAIL(anExpression,aText) \
	if (!(anExpression)) { \
		testStatusv[testIndex - 1] = 1; \
		BO_TEST_PRINTF( \
			"%s:%d: test %d: %s: %s\n\n", __FILE__, __LINE__, testIndex, \
			"failure", aText \
		); \
		break; \
	}; ((void)0)

# define BO_TEST_FUNCJMP_BREAKIFFAIL(anExpression,aText) \
	if (!(anExpression)) { \
		testStatusv[testIndex - 1] = 1; \
		BO_TEST_PRINTF( \
			"%s:%d: test %d: %s: %s\n\n", __FILE__, __LINE__, testIndex, \
			"failure", aText \
		); \
		BO_FUNCJMP_BREAK(1,break); \
	}; ((void)0)

# if __myc_support(NVM)
#  define BO_TEST_BREAKIFFAIL_PRINTF(anExpression,aText,aFormat...) \
	if (!(anExpression)) { \
		testStatusv[testIndex - 1] = 1; \
		BO_TEST_PRINTF(aFormat); \
		BO_TEST_PRINTF( \
			"%s:%d: test %d: %s: %s\n\n", __FILE__, __LINE__, testIndex, \
			"failure", aText \
		); \
		break; \
	}; ((void)0)

#  define BO_TEST_FUNCJMP_BREAKIFFAIL_PRINTF(anExpression,aText,aFormat...) \
	if (!(anExpression)) { \
		testStatusv[testIndex - 1] = 1; \
		BO_TEST_PRINTF(aFormat); \
		BO_TEST_PRINTF( \
			"%s:%d: test %d: %s: %s\n\n", __FILE__, __LINE__, testIndex, \
			"failure", aText \
		); \
		BO_FUNCJMP_BREAK(1,break); \
	}; ((void)0)
# else
# if __myc_support(AVM)
#  define BO_TEST_BREAKIFFAIL_PRINTF(anExpression,aText,/*aFormat,*/...) \
	if (!(anExpression)) { \
		testStatusv[testIndex - 1] = 1; \
		BO_TEST_PRINTF(/*aFormat,*/__VA_ARGS__); \
		BO_TEST_PRINTF( \
			"%s:%d: test %d: %s: %s\n\n", __FILE__, __LINE__, testIndex, \
			"failure", aText \
		); \
		break; \
	}; ((void)0)

#  define BO_TEST_FUNCJMP_BREAKIFFAIL_PRINTF(anExpression,aText,/*aFormat,*/...) \
	if (!(anExpression)) { \
		testStatusv[testIndex - 1] = 1; \
		BO_TEST_PRINTF(/*aFormat,*/__VA_ARGS__); \
		BO_TEST_PRINTF( \
			"%s:%d: test %d: %s: %s\n\n", __FILE__, __LINE__, testIndex, \
			"failure", aText \
		); \
		BO_FUNCJMP_BREAK(1,break); \
	}; ((void)0)
# endif /* __myc_support(AVM) */
# endif /* __myc_support(NVM) */

# define BO_TEST_BREAKFAIL_NEXT() \
	if (0 != testStatusv[testIndex - 1]) { \
		break; \
	}; ((void)0)

# define BO_TEST_FUNCJMP_BREAKFAIL_NEXT() \
	if (0 != testStatusv[testIndex - 1]) { \
		BO_FUNCJMP_BREAK(1,break); \
	}; ((void)0)

# define BO_TEST_END \
		BO_TEST_PRINTF("%s:%d: test %d: %s\n\n", __FILE__, __LINE__, testIndex, "success"); \
	} while (0); \
	testStatus += testStatusv[testIndex - 1]; \
	++testIndex;

# define BO_TEST_EXIT() \
	return (testStatus)

#endif /* !__BOTEST__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
