/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Crtdbg.h	(V.Sokur)
 * include/Bo/Crtdbg.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_CRTDBG__
# define __BO_CRTDBG__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# ifndef NDEBUG
#  include <Bo/libc/Stdio.h>
# endif

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -
# pragma mark [ definitions ]
# pragma mark -

# ifndef _BoCrtDbg_Printf

#  ifdef _BO_CRT_DEBUG0
#   define _BoCrtDbg_Printf _im_BoCrtDbg_Printf
#   define _BoCrtDbg_Wprintf _im_BoCrtDbg_Wprintf
#  else
#   define _BoCrtDbg_Printf _im_BoCrtDbg_PrintfNothing
#   define _BoCrtDbg_Wprintf _im_BoCrtDbg_WprintfNothing
#  endif

#  if __myc_support(NVM)
#   define _im_BoCrtDbg_Printf(aFormat...) fprintf(stderr,aFormat)
#   define _im_BoCrtDbg_PrintfNothing(aFormat...) ((void)0)
#   define _im_BoCrtDbg_Wprintf(aFormat...) fwprintf(stderr,aFormat)
#   define _im_BoCrtDbg_WprintfNothing(aFormat...) ((void)0)
#  else
#  if __myc_support(AVM)
#   define _im_BoCrtDbg_Printf(/*aFormat,*/...) fprintf(stderr,/*aFormat,*/__VA_ARGS__)
#   define _im_BoCrtDbg_PrintfNothing(/*aFormat,*/...) ((void)0)
#   define _im_BoCrtDbg_Wprintf(/*aFormat,*/...) fwprintf(stderr,/*aFormat,*/__VA_ARGS__)
#   define _im_BoCrtDbg_WprintfNothing(/*aFormat,*/...) ((void)0)
#  endif /* __myc_support(AVM) */
#  endif /* __myc_support(NVM) */

# endif /* !_BoCrtDbg_Printf */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * _BoCrtDbg_EnterFunc
 */
# pragma mark -

# ifndef _BoCrtDbg_EnterFunc
#  define _BoCrtDbg_EnterFunc() __BoCrtDbg_EnterFunc(__func__)
# endif

# ifndef __BoCrtDbg_EnterFunc

__myc_flinkage__(public,libBo) void
__BoCrtDbg_EnterFunc(
# if __myc_require(prototypes)
	char const *aFunc
# endif
	);

#  define __BoCrtDbg_EnterFunc(aFunc) \
	_BoCrtDbg_Printf("Enter %s.\n", aFunc)
# endif

/* ----------------------------------------------------------------------------
 * _BoCrtDbg_LeaveFunc0
 */
# pragma mark -

# ifndef _BoCrtDbg_LeaveFunc0
#  define _BoCrtDbg_LeaveFunc0() __BoCrtDbg_LeaveFunc0(__func__)
# endif

# ifndef __BoCrtDbg_LeaveFunc0

__myc_flinkage__(public,libBo) void
__BoCrtDbg_LeaveFunc0(
# if __myc_require(prototypes)
	char const *aFunc
# endif
	);

#  define __BoCrtDbg_LeaveFunc0(aFunc) \
	_BoCrtDbg_Printf("Leave %s.\n", aFunc)
# endif

/* ----------------------------------------------------------------------------
 * _BoCrtDbg_LeaveFunc
 */
# pragma mark -

# ifndef _BoCrtDbg_LeaveFunc
#  define _BoCrtDbg_LeaveFunc(aStatus) \
	__BoCrtDbg_LeaveFunc(__func__, (BoS_SUCCESS(aStatus) ? "Success" : "Failure"))
# endif

# ifndef __BoCrtDbg_LeaveFunc

__myc_flinkage__(public,libBo) void
__BoCrtDbg_LeaveFunc(
# if __myc_require(prototypes)
	char const *aFunc,
	char const *aMessage
# endif
	);

#  define __BoCrtDbg_LeaveFunc(aFunc,aMessage) \
	_BoCrtDbg_Printf("Leave %s. %s.\n", aFunc,aMessage)
# endif

__MYC_END_CDECLS

#endif /* !__BO_CRTDBG__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
