/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Debug.h	(V.Sokur)
 * include/Bo/Debug.h
 *
 * $Id$
 */

/******************************************************************************
 * %% BeginSection: includes
 */

#ifndef __BO_SYS_CDEFS__
# include <Bo/sys/Cdefs.h>
#endif

#ifndef NDEBUG
# ifndef __BO_LIBC_STDIO__
#  include <Bo/libc/Stdio.h>
# endif
#endif

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -
# pragma mark [ definitions ]
# pragma mark -

# ifdef _BoDbg_Eval
#  undef _BoDbg_Eval
# endif
# ifndef NDEBUG
#  define _BoDbg_Eval(anExpr) (anExpr)
# else
#  define _BoDbg_Eval(anExpr) ((void)0)
# endif


# ifdef _BoDbg_StaticAssert
#  undef _BoDbg_StaticAssert
# endif
# if 0
# ifndef NDEBUG
#  define _BoDbg_StaticAssert(aName,anExpr) \
	static char aName[(anExpr) ? 1 : -1];
# else
#  define _BoDbg_StaticAssert(aName,anExpr)
# endif
# endif
# define _BoDbg_StaticAssert(aName,anExpr) \
	typedef char __myc_xconcat(_A_test_,aName)[(anExpr) ? 1 : -1]

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

# ifndef _BoDbg_Printf

#  ifdef _DEBUG
#   define _BoDbg_Printf _im_BoDbg_Printf
#   define _BoDbg_Wprintf _im_BoDbg_Wprintf
#  else
#   define _BoDbg_Printf _im_BoDbg_PrintfNothing
#   define _BoDbg_Wprintf _im_BoDbg_WprintfNothing
#  endif

#  if __myc_support(NVM)
#   define _im_BoDbg_Printf(aFormat...) fprintf(stderr,aFormat)
#   define _im_BoDbg_PrintfNothing(aFormat...) ((void)0)
#   define _im_BoDbg_Wprintf(aFormat...) fwprintf(stderr,aFormat)
#   define _im_BoDbg_WprintfNothing(aFormat...) ((void)0)
#  else
#  if __myc_support(AVM)
#   define _im_BoDbg_Printf(/*aFormat,*/...) fprintf(stderr,/*aFormat,*/__VA_ARGS__)
#   define _im_BoDbg_PrintfNothing(/*aFormat,*/...) ((void)0)
#   define _im_BoDbg_Wprintf(/*aFormat,*/...) fwprintf(stderr,/*aFormat,*/__VA_ARGS__)
#   define _im_BoDbg_WprintfNothing(/*aFormat,*/...) ((void)0)
#  endif /* __myc_support(AVM) */
#  endif /* __myc_support(NVM) */

# endif /* !_BoDbg_Printf */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

#ifdef __MWERKS__
# ifndef NDEBUG
#  include __MYC_LIBC_HEADER(stdio.h)
# endif
#endif
#include <assert.h>

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
