/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Errors.h	(V.Sokur)
 * include/Bo/Errors.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_ERRORS__
# define __BO_ERRORS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Cdefs.h>

# if 0
# ifndef BO_SYSTYPE_WIN32
#  include <errno.h>
# endif
# endif

/******************************************************************************
 * %% BeginSection: constant declarations
 */
# pragma mark -
# pragma mark [ constant declarations ]
# pragma mark -

# define BO_MAKE_ERRCODE2(aCategory,aProblem) \
	((((aCategory) & 0xff) << 8) | ((aProblem) & 0xff))

# define BO_MAKE_ERRCODE3(aCategory,aType,aProblem) \
	((((aCategory) & 0xff) << 8) | (((aType) & 0xf) << 4) | ((aProblem) & 0xf))

# define _BoERRCODE2(aCategory,aProblem) BO_MAKE_ERRCODE2(aCategory,aProblem)
# define _BoERRCODE3(aCategory,aType,aProblem) BO_MAKE_ERRCODE3(aCategory,aType,aProblem)

# define BoERRCODE_CATEGORY(anErrcode) ((unsigned)(((anErrcode) >> 8) & 0xff))
# define BoERRCODE2_PROBLEM(anErrcode) ((unsigned)((anErrcode) & 0xff))
# define BoERRCODE3_TYPE(anErrcode) ((unsigned)(((anErrcode) >> 4) & 0xf))
# define BoERRCODE3_PROBLEM(anErrcode) ((unsigned)((anErrcode) & 0xf))

/*
 * Well known categories
 */

# define BoK_ERROR_ARG_CATEGORY (1)
# define BoK_ERROR_RAC_CATEGORY (2)
# define BoK_ERROR_STATE_CATEGORY (3)
# define BoK_ERROR_OP_CATEGORY (4)
# define BoK_ERROR_TEXT_CATEGORY (5)
# define BoK_ERROR_AUTH_CATEGORY (6)
# define BoK_ERROR_MATH_CATEGORY (7)

# define BoK_ERROR_SYS_CATEGORY (128)
# define BoK_ERROR_IO_CATEGORY (129)
# define BoK_ERROR_FILE_CATEGORY (130)
# define BoK_ERROR_IPC_CATEGORY (131)

# define BoK_ERROR_AIB_CATEGORY (192)
# define BoK_ERROR_DIR_CATEGORY (193)

/* ============================================================================
 * ARGUMENT ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ argument errors ]
# pragma mark -

/*
 * ArgListLimitExceeded
 * An argument list is too long.
 */
# define _BoK_ArgErrorProblem_ArgListLimitExceeded (1)

# define BoK_ArgError_ArgListLimit /* E2BIG */ \
	_BoERRCODE2(BoK_ERROR_ARG_CATEGORY,_BoK_ArgErrorProblem_ArgListLimitExceeded)

/* BadAddress */
# define _BoK_ArgErrorProblem_BadAddress (2)

# define BoK_ArgError_BadAddress /* EFAULT */ \
	_BoERRCODE2(BoK_ERROR_ARG_CATEGORY,_BoK_ArgErrorProblem_BadAddress)

/* InvalidArgument */
# define _BoK_ArgErrorProblem_InvalidArgument (3)

# define BoK_ArgError_InvalidArgument /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_ARG_CATEGORY,_BoK_ArgErrorProblem_InvalidArgument)

/*
 * InvalidString
 * The input string is empty (or NUL).
 */
# define _BoK_ArgErrorProblem_InvalidString (4)

# define BoK_ArgError_InvalidString /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_ARG_CATEGORY,_BoK_ArgErrorProblem_InvalidString)

/*
 * Overflow
 * The provided buffer is too small.
 */
# define _BoK_ArgErrorProblem_Overflow (5)

# define BoK_ArgError_Overflow /* EOVERFLOW */ \
	_BoERRCODE2(BoK_ERROR_ARG_CATEGORY,_BoK_ArgErrorProblem_Overflow)
/* @} */

/* ============================================================================
 * RESOURCE ACCESS ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ resource access errors ]
# pragma mark -

/*
 * OperationNotPermitted
 * Permision denied: operation not permitted [not a super user], process has
 * not appropriate privileges to access a resource.
 */
# define _BoK_RacErrorProblem_OperationNotPermitted (1)

# define BoK_RacError_OperationNotPermitted /* EPERM */ \
	_BoERRCODE2(BoK_ERROR_RAC_CATEGORY,_BoK_RacErrorProblem_OperationNotPermitted)

/*
 * AccessViolation
 * Permision denied:
 * 1. access rights violation;
 * 2. access a resource [file] in a way that is forbidden by its
 *    [file] access permissions.
 */
# define _BoK_RacErrorProblem_AccessViolation (2)

# define BoK_RacError_AccessViolation /* EACCES */ \
	_BoERRCODE2(BoK_ERROR_RAC_CATEGORY,_BoK_RacErrorProblem_AccessViolation)

/*
 * DeadLockDetected
 * An attempt to lock the resource that would have resulted in a deadlock
 * situation.
 */
# define _BoK_RacErrorProblem_DeadLockDetected (3)

# define BoK_RacError_DeadLock /* EDEADLK */ \
	_BoERRCODE2(BoK_ERROR_RAC_CATEGORY,_BoK_RacErrorProblem_DeadLockDetected)

/*
 * Unavailable (WouldBlock)
 * The resource temporarily unavailable. The operation would block.
 * This error is returned from operations on non-blocking device
 * (sockets, ...) that cannot be completed immediately.
 */
# define _BoK_RacErrorProblem_Unavailable (4)

# define BoK_RacError_WouldBlock /* EWOULDBLOCK, EAGAIN */ \
	_BoERRCODE2(BoK_ERROR_RAC_CATEGORY,_BoK_RacErrorProblem_Unavailable)

/*
 * Busy
 * The resource is used [by another process] in a conflicting manner.
 */
# define _BoK_RacErrorProblem_Busy (5)

# define BoK_RacError_Busy /* EBUSY */ \
	_BoERRCODE2(BoK_ERROR_RAC_CATEGORY,_BoK_RacErrorProblem_Busy)

/*
 * NotConfigured
 * The resource not ready to use or no such resource.
 */
# define _BoK_RacErrorProblem_NotConfigured (6)

# define BoK_RacError_NotConfigured /* ENXIO */ \
	_BoERRCODE2(BoK_ERROR_RAC_CATEGORY,_BoK_RacErrorProblem_NotConfigured)

/*
 * NoResourceAvail
 * The free resources to use.
 */
# define _BoK_RacErrorProblem_NoResourceAvail (7)

# define BoK_RacError_NoRsrcAvail /* EAGAIN */ \
	_BoERRCODE2(BoK_ERROR_RAC_CATEGORY,_BoK_RacErrorProblem_NoResourceAvail)
/* @} */

/* ============================================================================
 * [Object] STATE ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ state errors ]
# pragma mark -

/*
 * Invalid
 */
# define _BoK_StateErrorProblem_Invalid (1)

# define BoK_StateError_Invalid /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_STATE_CATEGORY,_BoK_StateErrorProblem_Invalid)

/*
 * NotConfigured
 * The object is in the unconfigured state.
 */
# define _BoK_StateErrorProblem_NotConfigured (2)

# define BoK_StateError_NotConfigured /* ENXIO */ \
	_BoERRCODE2(BoK_ERROR_STATE_CATEGORY,_BoK_StateErrorProblem_NotConfigured)

/*
 * WrongStateToPerform
 * The object is in the wrong state to perform operation.
 */
# define _BoK_StateErrorProblem_WrongStateToPerform (3)

# define BoK_StateError_WrongState /* ENXIO, EINVAL */ \
	_BoERRCODE2(BoK_ERROR_STATE_CATEGORY,_BoK_StateErrorProblem_WrongStateToPerform)

/*
 * AlreadyInState
 * An operation brings the [object] state to the new state,
 * and the object is already in that state.
 */
# define _BoK_StateErrorProblem_AlreadyInState (4)

# define BoK_StateError_Already /* EALREADY */ \
	_BoERRCODE2(BoK_ERROR_STATE_CATEGORY,_BoK_StateErrorProblem_AlreadyInState)

/*
 * Destructible
 * The object is in the state, when the requested operation can be only
 * performed in destructive manner.
 */
# define _BoK_StateErrorProblem_Destructible (5)

# define BoK_StateError_Destructible /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_STATE_CATEGORY,_BoK_StateErrorProblem_Destructible)
/* @} */

/* ============================================================================
 * OPERATIONAL ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ operational errors ]
# pragma mark -

/*
 * Inappropriate
 * The inappropriate operation to an object.
 */
# define _BoK_OpErrorProblem_Inappropriate (1)

# define BoK_OpError_Inappropriate /* ENODEV */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_Inappropriate)

/*
 * NotImpl
 * The function (operation) is not implemented [in this implementation].
 */
# define _BoK_OpErrorProblem_NotImpl (2)

# define BoK_OpError_NotImpl /* ENOSYS */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_NotImpl)

/*
 * NotAvail
 * The inappropriate operation to an object implementation.
 * For example, trying to read a write-only device such as a printer.
 */
# define _BoK_OpErrorProblem_NotAvail (3)

# define BoK_OpError_NotAvail /* ENODEV */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_NotAvail)

/*
 * NotSupp
 * The implementation [current build] does not support
 * the operation [of the standard].
 */
# define _BoK_OpErrorProblem_NotSupp (4)

# define BoK_OpError_NotSupp /* EOPNOTSUPP */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_NotSupp)

/*
 * FeatureNotSupp
 * The implementation does not support the option, parameter or the feature
 * [of the standard].
 *
 * From IEEE Std 1003.1-2001
 *    Base, Realtime, Threads or Thread Priority Scheduling OPTION errors.
 */
# define _BoK_OpErrorProblem_FeatureNotSupp (5)

# define BoK_OpError_FeatureNotSupp /* ENOTSUP */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_FeatureNotSupp)

/*
 * InProgress
 * The blocking operation is now in progress [the asynchronous operation
 * has not yet completed].
 */
# define _BoK_OpErrorProblem_InProgress (6)

# define BoK_OpError_InProgress /* EINPROGRESS */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_InProgress)

/*
 * Already
 * The operation [on non-blocking device] is already in progress.
 */
# define _BoK_OpErrorProblem_Already (7)

# define BoK_OpError_Already /* EALREADY */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_Already)

/*
 * Canceled
 * The asynchronous operation was cancelled before completion.
 */
# define _BoK_OpErrorProblem_Canceled (8)

# define BoK_OpError_Canceled /* ECANCELED */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_Canceled)

/*
 * Interrupted
 * The execution of an interruptible function was interrupted by
 * an asynchronous signal (such as quit or terminate signal).
 */
# define _BoK_OpErrorProblem_Interrupted (9)

# define BoK_OpError_Interrupted /* EINTR */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_Interrupted)

/*
 * TimedOut
 * The operation timed out.
 */
# define _BoK_OpErrorProblem_TimedOut (10)

# define BoK_OpError_TimedOut /* ETIMEDOUT */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_TimedOut)

/*
 * UnwillingToPerform
 * @todo: doc.
 */
# define _BoK_OpErrorProblem_UnwillingToPerform (11)

# define BoK_OpError_UnwillingToPerform /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_UnwillingToPerform)

/*
 * NotAvailIf
 * @todo: doc.
 */
# define _BoK_OpErrorProblem_NotAvailIf (12)

# define BoK_OpError_NotAvailIf /* ENODEV */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_NotAvailIf)

/*
 * NotSuppIf
 * @todo: doc.
 */
# define _BoK_OpErrorProblem_NotSuppIf (13)

# define BoK_OpError_NotSuppIf /* EOPNOTSUPP */ \
	_BoERRCODE2(BoK_ERROR_OP_CATEGORY,_BoK_OpErrorProblem_NotSuppIf)
/* @} */

/* ============================================================================
 * TEXT ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ text errors ]
# pragma mark -

/*
 * IllegalMBSequence
 * Illegal multibyte sequence, ISO/IEC 9899/AMD1:1995.
 */
# define _BoK_TextErrorProblem_IllegalMBSequence (1)

# define BoK_TextError_IllegalMBSequence /* EILSEQ */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_IllegalMBSequence)

/*
 * IncompleteMBSequence
 * Incomplete multibyte sequence.
 */
# define _BoK_TextErrorProblem_IncompleteMBSequence (2)

# define BoK_TextError_IncompleteMBSequence /* EILSEQ */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_IncompleteMBSequence)

/*
 * BadUnicodeCharacter
 * Bad UNICODE character.
 */
# define _BoK_TextErrorProblem_BadUnicodeCharacter (3)

# define BoK_TextError_BadUnicodeCharacter /* EILSEQ */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_BadUnicodeCharacter)

/*
 * MBOverflow
 * A buffer for a multibyte character is too small.
 */
# define _BoK_TextErrorProblem_MBOverflow (4)

# define BoK_TextError_MBOverflow /* EOVERFLOW */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_MBOverflow)

/*
 * TokenMaxLenExceeded
 * A token max len exceeded.
 */
# define _BoK_TextErrorProblem_TokenMaxLenExceeded (5)

# define BoK_TextError_TokenMaxLen /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_TokenMaxLenExceeded)

/*
 * InvalidSyntax
 * Generic syntax error
 */
# define _BoK_TextErrorProblem_InvalidSyntax (6)

# define BoK_TextError_InvalidSyntax /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_InvalidSyntax)

/*
 * EndOfText
 * Syntax error: end of text reached
 */
# define _BoK_TextErrorProblem_EndOfText (7)

# define BoK_TextError_EndOfText /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_EndOfText)

/*
 * UnmatchedCharacter
 * Syntax error: unmatched character
 */
# define _BoK_TextErrorProblem_UnmatchedCharacter (8)

# define BoK_TextError_UnmatchedCharacter /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_UnmatchedCharacter)

/*
 * EmptyToken
 * Syntax error: a token (identifier,keyword,...) is empty
 */
# define _BoK_TextErrorProblem_EmptyToken (9)

# define BoK_TextError_EmptyToken /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_EmptyToken)

/*
 * UnexpectedToken
 * Syntax error: unexpected token (identifier,keyword,...)
 */
# define _BoK_TextErrorProblem_UnexpectedToken (10)

# define BoK_TextError_UnexpectedToken /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_UnexpectedToken)

/*
 * MissingToken
 * Syntax error: missing token (identifier,keyword,...)
 */
# define _BoK_TextErrorProblem_MissingToken (11)

# define BoK_TextError_MissingToken /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_MissingToken)

/*
 * BadToken
 * Syntax error: bad token (identifier,keyword,...)
 */
# define _BoK_TextErrorProblem_BadToken (12)

# define BoK_TextError_BadToken /* EINVAL */ \
	_BoERRCODE2(BoK_ERROR_TEXT_CATEGORY,_BoK_TextErrorProblem_BadToken)
/* @} */

/* ============================================================================
 * AUTHENTICATION ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ authentication errors ]
# pragma mark -

/*
 * NoInformation
 * Authentication runtime error.
 */
# define _BoK_AuthErrorProblem_NoInformation (1)

# define BoK_AuthError_NoInformation /* EAUTH */ \
	_BoERRCODE2(BoK_ERROR_AUTH_CATEGORY,_BoK_AuthErrorProblem_NoInformation)

/*
 * NeedAuth
 * Need authenticator.
 */
# define _BoK_AuthErrorProblem_NeedAuth (2)

# define BoK_AuthError_NeedAuth /* ENEEDAUTH */ \
	_BoERRCODE2(BoK_ERROR_AUTH_CATEGORY,_BoK_AuthErrorProblem_NeedAuth)
/* @} */

/* ============================================================================
 * MATH ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ math errors ]
# pragma mark -

/*
 * OutOfDomain
 * Numerical argument out of domain.
 */
# define _BoK_MathErrorProblem_OutOfDomain (1)

# define BoK_MathError_OutOfDomain /* EDOM */ \
	_BoERRCODE2(BoK_ERROR_MATH_CATEGORY,_BoK_MathErrorProblem_OutOfDomain)

/*
 * ResultRange
 * Result too large or too small.
 */
# define _BoK_MathErrorProblem_ResultRange (2)

# define BoK_MathError_ResultRange /* ERANGE */ \
	_BoERRCODE2(BoK_ERROR_MATH_CATEGORY,_BoK_MathErrorProblem_ResultRange)
/* @} */

/* ============================================================================
 * COMMON SYSTEM ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ common system errors ]
# pragma mark -

/*
 * NoInformation
 * Unknown (unqualified, ...) runtime error.
 */
# define _BoK_SysErrorProblem_NoInformation (1)

# define BoK_SysError_NoInformation /*  */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_NoInformation)

/*
 * NoSuchProcess
 * No such process [with gived ID].
 */
# define _BoK_SysErrorProblem_NoSuchProcess (2)

# define BoK_SysError_NoSuchProcess /* ESRCH */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_NoSuchProcess)

/*
 * InvalidExecFormat
 * The file is not in the format required by the implementation for
 * executable files.
 */
# define _BoK_SysErrorProblem_InvalidExecFormat (3)

# define BoK_SysError_InvalidExecFormat /* ENOEXEC */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_InvalidExecFormat)

/*
 * NoChildToWait
 * No child processes to wait.
 */
# define _BoK_SysErrorProblem_NoChildToWait (4)

# define BoK_SysError_NoChildToWait /* ECHILD */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_NoChildToWait)

/*
 * NoMemory
 * Cannot allocate memory.
 */
# define _BoK_SysErrorProblem_NoMemory (5)

# define BoK_SysError_NoMemory /* ENOMEM */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_NoMemory)

/*
 * ProcsLimitExceeded
 * Too many processes.
 */
# define _BoK_SysErrorProblem_ProcsLimitExceeded (6)

# define BoK_SysError_ProcsLimit /* EPROCLIM */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_ProcsLimitExceeded)

/*
 * UsersLimitExceeded
 * Too many users.
 */
# define _BoK_SysErrorProblem_UsersLimitExceeded (7)

# define BoK_SysError_UsersLimit /* EUSERS */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_UsersLimitExceeded)

/*
 * DiskQuotaExceeded
 * Disc quota exceeded.
 */
# define _BoK_SysErrorProblem_DiskQuotaExceeded (8)

# define BoK_SysError_DiskQuota /* EDQUOT */ \
	_BoERRCODE2(BoK_ERROR_SYS_CATEGORY,_BoK_SysErrorProblem_DiskQuotaExceeded)
/* @} */

/* ============================================================================
 * I/O ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ I/O errors ]
# pragma mark -

/*
 * NoInformation
 * [Physical] Input/output runtime error.
 */
# define _BoK_IoErrorProblem_NoInformation (1)

# define BoK_IoError_NoInformation /* EIO */ \
	_BoERRCODE2(BoK_ERROR_IO_CATEGORY,_BoK_IoErrorProblem_NoInformation)

/*
 * NotConfigured
 * Device not configured, no such device or address.
 */
# define _BoK_IoErrorProblem_NotConfigured (2)

# define BoK_IoError_NotConfigured /* ENXIO */ \
	_BoERRCODE2(BoK_ERROR_IO_CATEGORY,_BoK_IoErrorProblem_NotConfigured)

/*
 * NotBlockDevice
 * Block device required.
 */
# define _BoK_IoErrorProblem_NotBlockDevice (3)

# define BoK_IoError_NotBlockDevice /* ENOTBLK */ \
	_BoERRCODE2(BoK_ERROR_IO_CATEGORY,_BoK_IoErrorProblem_NotBlockDevice)

/*
 * NotSeekable
 * The device does not support seek operation.
 */
# define _BoK_IoErrorProblem_NotSeekable (4)

# define BoK_IoError_NotSeekable /* ESPIPE */ \
	_BoERRCODE2(BoK_ERROR_IO_CATEGORY,_BoK_IoErrorProblem_NotSeekable)

/*
 * NotSuppDevice
 * Device [of the class] not supported.
 */
# define _BoK_IoErrorProblem_NotSuppDevice (5)

# define BoK_IoError_NotSuppDevice /* ENOTBLK */ \
	_BoERRCODE2(BoK_ERROR_IO_CATEGORY,_BoK_IoErrorProblem_NotSuppDevice)

/*
 * NoPortAvail
 * @todo: doc
 */
# define _BoK_IoErrorProblem_NoPortAvail (6)

# define BoK_IoError_NoPortAvail /* ENOSPC */ \
	_BoERRCODE2(BoK_ERROR_IO_CATEGORY,_BoK_IoErrorProblem_NoPortAvail)
/* @} */

/* ============================================================================
 * FILE ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ file errors ]
# pragma mark -

/*
 * NoSuchEntry
 * No such file or directory.
 */
# define _BoK_FileErrorProblem_NoSuchEntry (1)

# define BoK_FileError_NoSuchEntry /* ENOENT */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_NoSuchEntry)

/*
 * BadDescriptor
 * The resource [file] descriptor is out of range, referres to no open file.
 */
# define _BoK_FileErrorProblem_BadDescriptor (2)

# define BoK_FileError_BadDescriptor /* EBADF */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_BadDescriptor)

/*
 * Busy
 * Mount device busy.
 */
# define _BoK_FileErrorProblem_Busy (3)

# define BoK_FileError_Busy /* EBUSY */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_Busy)

/*
 * AlreadyExists
 * File already exists.
 */
# define _BoK_FileErrorProblem_AlreadyExists (4)

# define BoK_FileError_AlreadyExists /* EEXIST */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_AlreadyExists)

/*
 * CrossDeviceLink
 * Cross-device link.
 */
# define _BoK_FileErrorProblem_CrossDeviceLink (5)

# define BoK_FileError_CrossDeviceLink /* EXDEV */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_CrossDeviceLink)

/*
 * OperationNotAvail
 * The inappropriate function to a device implementation.
 * Operation not supported by device.
 * For example, trying to read a write-only device such as a printer.
 */
# define _BoK_FileErrorProblem_OperationNotAvail (6)

# define BoK_FileError_OperationNotAvail /* ENODEV */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_OperationNotAvail)

/*
 * NotDir
 * Not a directory.
 */
# define _BoK_FileErrorProblem_NotDir (7)

# define BoK_FileError_NotDir /* ENOTDIR */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_NotDir)

/*
 * IsDir
 * Is a directory.
 */
# define _BoK_FileErrorProblem_IsDir (8)

# define BoK_FileError_IsDir /* EISDIR */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_IsDir)

/*
 * SysDescLimitExceeded
 * Too many open files in system. File table overflow.
 */
# define _BoK_FileErrorProblem_SysDescLimitExceeded (9)

# define BoK_FileError_SysDescLimit /* ENFILE */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_SysDescLimitExceeded)

/*
 * ProcessDescLimitExceeded
 * Too many open files.
 * (Maximum number of file descriptors allowed in this process).
 */
# define _BoK_FileErrorProblem_ProcessDescLimitExceeded (10)

# define BoK_FileError_ProcessDescLimit /* EMFILE */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_ProcessDescLimitExceeded)

/*
 * IoctlOperationNotAvail
 * An inappropriate [I/O] control function for a file or special file.
 */
# define _BoK_FileErrorProblem_IoctlOperationNotAvail (11)

# define BoK_FileError_IoctlOperationNotAvail /* ENOTTY */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_IoctlOperationNotAvail)

/*
 * TextBusy
 * Text file busy.
 * The file is a pure procedure (shared text) file that is
 * being executed and the open() call requests write access.
 */
# define _BoK_FileErrorProblem_TextBusy (12)

# define BoK_FileError_TextBusy /* ETXTBSY */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_TextBusy)

/*
 * FileSizeLimitExceeded
 * File too large.
 */
# define _BoK_FileErrorProblem_FileSizeLimitExceeded (13)

# define BoK_FileError_FileSizeLimit /* EFBIG */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_FileSizeLimitExceeded)

/*
 * NoFreeSpaceAvail
 * No free space left on the device during a 'write' operation
 * on a regular file, or when extending a directory.
 */
# define _BoK_FileErrorProblem_NoFreeSpaceAvail (14)

# define BoK_FileError_NoFreeSpace /* ENOSPC */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_NoFreeSpaceAvail)

/*
 * IllegalSeekOnPipe
 * Illegal seek operation on a pipe or FIFO.
 */
# define _BoK_FileErrorProblem_IllegalSeekOnPipe (15)

# define BoK_FileError_SeekOnPipe /* ESPIPE */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_IllegalSeekOnPipe)

/*
 * ReadOnlyFS
 * Read-only file system.
 */
# define _BoK_FileErrorProblem_ReadOnlyFS (16)

# define BoK_FileError_ReadOnlyFS /* EROFS */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_ReadOnlyFS)

/*
 * LinksLimitExceeded
 * Too many links of a single file.
 */
# define _BoK_FileErrorProblem_LinksLimitExceeded (17)

# define BoK_FileError_LinksLimit /* EMLINK */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_LinksLimitExceeded)

/*
 * LoopDetected
 * Too many levels of symbolic links.
 */
# define _BoK_FileErrorProblem_LoopDetected (18)

# define BoK_FileError_Loop /* ELOOP */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_LoopDetected)

/*
 * NameSizeLimitExceeded
 * File name too long.
 */
# define _BoK_FileErrorProblem_NameSizeLimitExceeded (19)

# define BoK_FileError_NameSizeLimit /* ENAMETOOLONG */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_NameSizeLimitExceeded)

/*
 * DirNotEmpty
 * Directory not empty.
 */
# define _BoK_FileErrorProblem_DirNotEmpty (20)

# define BoK_FileError_DirNotEmpty /* ENOTEMPTY */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_DirNotEmpty)

/*
 * NoLockAvail
 * No locks available.
 */
# define _BoK_FileErrorProblem_NoLockAvail (21)

# define BoK_FileError_NoLockAvail /* ENOLCK */ \
	_BoERRCODE2(BoK_ERROR_FILE_CATEGORY,_BoK_FileErrorProblem_NoLockAvail)
/* @} */

/* ============================================================================
 * IPC (Interprocess Communication) ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ IPC errors ]
# pragma mark -

/* ------------------------------------------------------------------------- */
/* Argument errors */
# pragma mark = IPC: argument errors =
/* ------------------------------------------------------------------------- */

/*
 * NotSocket
 * Socket operation on a non-socket. Socket required.
 */
# define _BoK_IpcErrorProblem_NotSocket (1)

# define BoK_IpcError_NotSocket /* ENOTSOCK */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_NotSocket)

/*
 * DestAddrRequired
 * Destination address required.
 */
# define _BoK_IpcErrorProblem_DestAddrRequired (2)

# define BoK_IpcError_DestAddrRequired /* EDESTADDRREQ */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_DestAddrRequired)

/*
 * MsgSizeLimitExceeded
 * Message too long.
 */
# define _BoK_IpcErrorProblem_MsgSizeLimitExceeded (3)

# define BoK_IpcError_MsgSizeLimit /* EMSGSIZE */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_MsgSizeLimitExceeded)

/*
 * ProtoWrongType
 * Protocol wrong type for socket.
 */
# define _BoK_IpcErrorProblem_ProtoWrongType (4)

# define BoK_IpcError_Prototype /* EPROTOTYPE */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_ProtoWrongType)

/*
 * ProtoNotAvail
 * Protocol not available.
 */
# define _BoK_IpcErrorProblem_ProtoNotAvail (5)

# define BoK_IpcError_ProtoNotAvail /* ENOPROTOOPT */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_ProtoNotAvail)

/*
 * ProtoNotSupp
 * Protocol not supported.
 */
# define _BoK_IpcErrorProblem_ProtoNotSupp (6)

# define BoK_IpcError_ProtoNotSupp /* EPROTONOSUPPORT */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_ProtoNotSupp)

/*
 * SockTypeNotSupp
 * Socket type not supported.
 */
# define _BoK_IpcErrorProblem_SockTypeNotSupp (7)

# define BoK_IpcError_SockTypeNotSupp /* ESOCKTNOSUPPORT */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_SockTypeNotSupp)

/*
 * OperationNotAvailOnSock
 * Operation not available on socket.
 */
# define _BoK_IpcErrorProblem_OperationNotAvailOnSock (8)

# define BoK_IpcError_OperationNotAvailOnSock /* EOPNOTSUPP */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_OperationNotAvailOnSock)

/*
 * PFNotSupp
 * Protocol family not supported.
 */
# define _BoK_IpcErrorProblem_PFNotSupp (9)

# define BoK_IpcError_PFNotSupp /* EPFNOSUPPORT */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_PFNotSupp)

/*
 * AFNotSupp
 * Address family not supported by protocol family.
 */
# define _BoK_IpcErrorProblem_AFNotSupp (10)

# define BoK_IpcError_AFNotSupp /* EAFNOSUPPORT */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_AFNotSupp)

/*
 * AddrInUse
 * Address already in use.
 */
# define _BoK_IpcErrorProblem_AddrInUse (11)

# define BoK_IpcError_AddrInUse /* EADDRINUSE */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_AddrInUse)

/*
 * AddrNotAvail
 * Cannot assign requested address.
 */
# define _BoK_IpcErrorProblem_AddrNotAvail (12)

# define BoK_IpcError_AddrNotAvail /* EADDRNOTAVAIL */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_AddrNotAvail)

/* ------------------------------------------------------------------------- */
/* Operational errors */
# pragma mark = IPC: operational errors =
/* ------------------------------------------------------------------------- */

/*
 * NetDown
 * Network is down. There is a serious failure of network system.
 */
# define _BoK_IpcErrorProblem_NetDown (13)

# define BoK_IpcError_NetDown /* ENETDOWN */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_NetDown)

/*
 * NetUnreach
 * Network is unreachable.
 */
# define _BoK_IpcErrorProblem_NetUnreach (14)

# define BoK_IpcError_NetUnreach /* ENETUNREACH */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_NetUnreach)

/*
 * NetReset
 * Network dropped connection on reset.
 */
# define _BoK_IpcErrorProblem_NetReset (15)

# define BoK_IpcError_NetReset /* ENETRESET */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_NetReset)

/*
 * ConnectionAborted
 * Software caused connection abort.
 */
# define _BoK_IpcErrorProblem_ConnectionAborted (16)

# define BoK_IpcError_ConnectionAborted /* ECONNABORTED */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_ConnectionAborted)

/*
 * ConnectionReset
 * Connection reset by peer.
 */
# define _BoK_IpcErrorProblem_ConnectionReset (17)

# define BoK_IpcError_ConnectionReset /* ECONNRESET */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_ConnectionReset)

/*
 * NoBufferSpaceAvail
 * No internal buffer space available.
 */
# define _BoK_IpcErrorProblem_NoBufferSpaceAvail (18)

# define BoK_IpcError_NoBufferSpace /* ENOBUFS */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_NoBufferSpaceAvail)

/*
 * IsConnected
 * Socket already connected.
 */
# define _BoK_IpcErrorProblem_IsConnected (19)

# define BoK_IpcError_IsConnected /* EISCONN */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_IsConnected)

/*
 * NotConnected
 * Socket not connected.
 */
# define _BoK_IpcErrorProblem_NotConnected (20)

# define BoK_IpcError_NotConnected /* ENOTCONN */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_NotConnected)

/*
 * Shutdown
 * Cannot send after socket shutdown.
 */
# define _BoK_IpcErrorProblem_Shutdown (21)

# define BoK_IpcError_Shutdown /* ESHUTDOWN */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_Shutdown)

/*
 * ConnectionRefused
 * Connection refused.
 */
# define _BoK_IpcErrorProblem_ConnectionRefused (22)

# define BoK_IpcError_ConnectionRefused /* ECONNREFUSED */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_ConnectionRefused)

/*
 * HostDown
 * Host is down.
 */
# define _BoK_IpcErrorProblem_HostDown (23)

# define BoK_IpcError_HostDown /* EHOSTDOWN */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_HostDown)

/*
 * NoRouteToHost
 * No route to host.
 */
# define _BoK_IpcErrorProblem_NoRouteToHost (24)

# define BoK_IpcError_HostUnreach /* EHOSTUNREACH */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_NoRouteToHost)

/*
 * BrokenPipe
 * Broken pipe.
 */
# define _BoK_IpcErrorProblem_BrokenPipe (25)

# define BoK_IpcError_BrokenPipe /* EPIPE */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_BrokenPipe)

/*
 * BadMsg
 * An implementation detects a corrupted message.
 */
# define _BoK_IpcErrorProblem_BadMsg (26)

# define BoK_IpcError_BadMsg /* EBADMSG */ \
	_BoERRCODE2(BoK_ERROR_IPC_CATEGORY,_BoK_IpcErrorProblem_BadMsg)
/* @} */

/* ============================================================================
 * AIB (Abstract Information Base) ERRORS
 * @{
 */
# pragma mark -
# pragma mark [ AIB errors ]
# pragma mark -

/* Type */
# define BoK_AIB_NAME_ERROR (1)
# define BoK_AIB_ATTRIBUTE_ERROR (2)
# define BoK_AIB_UPDATE_ERROR (3)
# define BoK_AIB_SERVICE_ERROR (4)

/* ------------------------------------------------------------------------- */
# pragma mark = AIB: name errors =
/* ------------------------------------------------------------------------- */
/* Name Error Problem */
/* @{ */

/*
 * InvalidSyntax
 * @todo: doc
 */
# define _BoK_AIBNameErrorProblem_InvalidSyntax (1)

# define BoK_AIBNameError_InvalidSyntax /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_NAME_ERROR,_BoK_AIBNameErrorProblem_InvalidSyntax)

/*
 * NoSuchEntry
 * The name does not match the name of any entry.
 */
# define _BoK_AIBNameErrorProblem_NoSuchEntry (2)

# define BoK_AIBNameError_NoSuchEntry /* ENOENT */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_NAME_ERROR,_BoK_AIBNameErrorProblem_NoSuchEntry)

/*
 * SizeLimitExceeded
 * @todo: doc
 */
# define _BoK_AIBNameErrorProblem_SizeLimitExceeded (3)

# define BoK_AIBNameError_SizeLimit /* ENAMETOOLONG */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_NAME_ERROR,_BoK_AIBNameErrorProblem_SizeLimitExceeded)
/* @} */

/* ------------------------------------------------------------------------- */
# pragma mark = AIB: attribute errors =
/* ------------------------------------------------------------------------- */
/* Attribute Error Problem */
/* @{ */

/*
 * InvalidSyntax
 * @todo: doc
 */
# define _BoK_AIBAttrErrorProblem_InvalidSyntax (1)

# define BoK_AIBAttrError_InvalidSyntax /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_ATTRIBUTE_ERROR,_BoK_AIBAttrErrorProblem_InvalidSyntax)

/*
 * InvalidSyntax
 * No such attribute or attribute value.
 */
# define _BoK_AIBAttrErrorProblem_NoSuchAttribute (2)

# define BoK_AIBAttrError_NoSuchAttribute /* ENOENT */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_ATTRIBUTE_ERROR,_BoK_AIBAttrErrorProblem_NoSuchAttribute)

/*
 * AlreadyExists
 * @todo: doc
 */
# define _BoK_AIBAttrErrorProblem_AlreadyExists (3)

# define BoK_AIBAttrError_AlreadyExists /* EEXIST */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_ATTRIBUTE_ERROR,_BoK_AIBAttrErrorProblem_AlreadyExists)
/* @} */

/* ------------------------------------------------------------------------- */
# pragma mark = AIB: update errors =
/* ------------------------------------------------------------------------- */
/* Update Error Problem */
/* @{ */

/*
 * AlreadyExists
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_AlreadyExists (1)

# define BoK_AIBUpdateError_AlreadyExists /* EEXIST */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_AlreadyExists)

/*
 * NoSuchParent
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NoSuchParent (2)

# define BoK_AIBUpdateError_NoSuchParent /* ENOENT */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NoSuchParent)

/*
 * NotAllowedOnTheEntry
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NotAllowedOnTheEntry (3)

# define BoK_AIBUpdateError_NotAllowedOnTheEntry /* ENODEV */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NotAllowedOnTheEntry)

/*
 * IsCompound
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_IsCompound (4)

# define BoK_AIBUpdateError_IsCompound /* EISDIR */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_IsCompound)

/*
 * NotCompound
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NotCompound (5)

# define BoK_AIBUpdateError_NotCompound /* ENOTDIR */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NotCompound)

/*
 * IsEmptyCompound
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_IsEmptyCompound (6)

# define BoK_AIBUpdateError_IsEmptyCompound /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_IsEmptyCompound)

/*
 * NotEmptyCompound
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NotEmptyCompound (7)

# define BoK_AIBUpdateError_NotEmptyCompound /* ENOTEMPTY */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NotEmptyCompound)

/*
 * CompoundOverflow
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_CompoundOverflow (8)

# define BoK_AIBUpdateError_CompoundOverflow /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_CompoundOverflow)

/*
 * IsInserted
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_IsInserted (9)

# define BoK_AIBUpdateError_IsInserted /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_IsInserted)

/*
 * NotInserted
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NotInserted (10)

# define BoK_AIBUpdateError_NotInserted /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NotInserted)

/*
 * IsMounted
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_IsMounted (11)

# define BoK_AIBUpdateError_IsMounted /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_IsMounted)

/*
 * NotMounted
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NotMounted (12)

# define BoK_AIBUpdateError_NotMounted /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NotMounted)

/*
 * MultipleMountNotSupp
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_MultipleMountNotSupp (13)

# define BoK_AIBUpdateError_MultipleMountNotSupp /* ENOTSUP */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_MultipleMountNotSupp)

/*
 * NotMountpoint
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NotMountpoint (14)

# define BoK_AIBUpdateError_NotMountpoint /* ENODEV */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NotMountpoint)

/*
 * NotMountable
 * @todo: doc
 */
# define _BoK_AIBUpdateErrorProblem_NotMountable (15)

# define BoK_AIBUpdateError_NotMountable /* ENODEV */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_UPDATE_ERROR,_BoK_AIBUpdateErrorProblem_NotMountable)
/* @} */

/* ------------------------------------------------------------------------- */
# pragma mark = AIB: service errors =
/* ------------------------------------------------------------------------- */
/* Service Error Problem */
/* @{ */

/*
 * NoInformation
 * @todo: doc
 */
# define _BoK_AIBServiceErrorProblem_NoInformation (1)

# define BoK_AIBServiceError_NoInformation /* EINVAL */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_SERVICE_ERROR,_BoK_AIBServiceErrorProblem_NoInformation)

/*
 * Busy
 * @todo: doc
 */
# define _BoK_AIBServiceErrorProblem_Busy (2)

# define BoK_AIBServiceError_Busy /* EBUSY */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_SERVICE_ERROR,_BoK_AIBServiceErrorProblem_Busy)

/*
 * NestLevelLimitExceeded
 * @todo: doc
 */
# define _BoK_AIBServiceErrorProblem_NestLevelLimitExceeded (3)

# define BoK_AIBServiceError_NestLevelLimit /* ENOMEM */ \
	_BoERRCODE3(BoK_ERROR_AIB_CATEGORY,BoK_AIB_SERVICE_ERROR,_BoK_AIBServiceErrorProblem_NestLevelLimitExceeded)
/* @} */

/* ============================================================================
 * DIRECTORY ERRORS
 * (From X.511, Open Systems Interconnection - The Directory: Abstract Service Definitions )
 * @{
 */
# pragma mark -
# pragma mark [ directory errors ]
# pragma mark -

/* Type */
# define BoK_DIR_NAME_ERROR (1)
# define BoK_DIR_UPDATE_ERROR (2)
# define BoK_DIR_ATTRIBUTE_ERROR (3)
# define BoK_DIR_SECURITY_ERROR (4)
# define BoK_DIR_SERVICE_ERROR (5)

/* Name Error Problem */
/* @{ */

/* The name does not match the name of any object. */
# define _BoK_DirNameErrorProblem_NoSuchObject (1)

# define BoK_DirNameError_NoSuchObject \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_NAME_ERROR,_BoK_DirNameErrorProblem_NoSuchObject)

/* An alias has been dereferenced which names no object. */
# define _BoK_DirNameErrorProblem_AliasProblem (2)

# define BoK_DirNameError_Alias \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_NAME_ERROR,_BoK_DirNameErrorProblem_AliasProblem)

/* An attribute type and its accompanying attribute value in an AVA
   (attribute value assertion) in the name are incompatible. */
# define _BoK_DirNameErrorProblem_InvalidAttributeSyntax (3)

# define BoK_DirNameError_InvalidAttributeSyntax \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_NAME_ERROR,_BoK_DirNameErrorProblem_InvalidAttributeSyntax)

/* An alias was encountered in a situation where it was not allowed or
   where access was denied. */
# define _BoK_DirNameErrorProblem_AliasDereferencingProblem (4)

# define BoK_DirNameError_AliasDereferencing \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_NAME_ERROR,_BoK_DirNameErrorProblem_AliasDereferencingProblem)

/* A context type or value used in a name is not understood or is invalid,
   the use of a context variant name is not acceptable, or during name
   resolution a purported name matches the names of more than one DIT entry. */
# define _BoK_DirNameErrorProblem_ContextProblem (5)

# define BoK_DirNameError_ContextProblem \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_NAME_ERROR,_BoK_DirNameErrorProblem_ContextProblem)
/* @} */

/* Update Error Problem */
/* @{ */

/* The attempted addition or modification would violate the structure rules
   of the DIT as defined in the Directory schema and
   ITU-T Rec. X.501 | ISO/IEC 9594-2. That is, it would place an entry
   as the subordinate of an alias entry, or in a region of the DIT not
   permitted to a member of its object class, or would define an RDN for
   an entry to include a forbidden attribute type. */
# define _BoK_DirUpdateErrorProblem_NamingViolation (1)

# define BoK_DirUpdateError_NamingViolation \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_NamingViolation)

/* The attempted update would produce an entry inconsistent with the rules
   for entry content; for example, its object class definition, the DIT
   content rules, or with the definitions of ITU-T Rec. X.501 | ISO/IEC 9594-2
   as they pertain to object classes. */
# define _BoK_DirUpdateErrorProblem_ObjectClassViolation (2)

# define BoK_DirUpdateError_ObjectClassViolation \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_ObjectClassViolation)

/* The attempted operation is only allowed on leaf entries of the DIT. */
# define _BoK_DirUpdateErrorProblem_NotAllowedOnNonLeaf (3)

# define BoK_DirUpdateError_NotAllowedOnNonLeaf \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_NotAllowedOnNonLeaf)

/* The attempted operation would affect the RDN (e.g. removal of
   an attribute which is a part of the RDN). */
# define _BoK_DirUpdateErrorProblem_NotAllowedOnRDN (4)

# define BoK_DirUpdateError_NotAllowedOnRDN \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_NotAllowedOnRDN)

/* An attempted addEntry or modifyDN operation names an entry which already
   exists. */
# define _BoK_DirUpdateErrorProblem_EntryAlreadyExists (5)

# define BoK_DirUpdateError_EntryAlreadyExists \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_EntryAlreadyExists)

/* An attempted update would need to operate on multiple DSAs where this
   operation is not permitted. */
# define _BoK_DirUpdateErrorProblem_AffectsMultipleDSAs (6)

# define BoK_DirUpdateError_AffectsMultipleDSAs \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_AffectsMultipleDSAs)

/* An operation attempted to modify the structural object class of an entry. */
# define _BoK_DirUpdateErrorProblem_ObjectClassModificationProhibited (7)

# define BoK_DirUpdateError_ObjectClassModificationProhibited \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_ObjectClassModificationProhibited)

/* An attempted modifyDN operation names a new superior entry that does not exist. */
# define _BoK_DirUpdateErrorProblem_NoSuchSuperior (8)

# define BoK_DirUpdateError_NoSuchSuperior \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_NoSuchSuperior)

/* An operation attempted to delete a compound entry without specifying
   the ancestor as the object. */
# define _BoK_DirUpdateErrorProblem_NotAncestor (9)

# define BoK_DirUpdateError_NotAncestor \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_NotAncestor)

/* An operation attempted to establish an entry as an immediately
   hierarchical child under a family member that is not the ancestor. */
# define _BoK_DirUpdateErrorProblem_ParentNotAncestor (10)

# define BoK_DirUpdateError_ParentNotAncestor \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_ParentNotAncestor)

/* An operation attempted to break a rule applicable to a hierarchical group:
   a hierarchical group has to be completely outside any service specific
       administrative area or has to be completely contained within a service
       specific administrative area;
   hierarchical group is confined to a single DSA. */
# define _BoK_DirUpdateErrorProblem_HierarchyRuleViolation (11)

# define BoK_DirUpdateError_HierarchyRuleViolation \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_HierarchyRuleViolation)

/* An operation attempted to break a rule applicable to families within
   a compound entry. */
# define _BoK_DirUpdateErrorProblem_FamilyRuleViolation (12)

# define BoK_DirUpdateError_FamilyRuleViolation \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_UPDATE_ERROR,_BoK_DirUpdateErrorProblem_FamilyRuleViolation)
/* @} */

/* Attribute Error Problem */
/* @{ */

/* The named entry lacks one of the attributes or attribute values
   specified as an argument of the operation. */
# define _BoK_DirAttrErrorProblem_NoSuchAttribute (1)

# define BoK_DirAttrError_NoSuchAttribute \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_ATTRIBUTE_ERROR,_BoK_DirAttrErrorProblem_NoSuchAttribute)

/* A purported attribute value, specified as an argument of the operation,
   does not conform to the attribute syntax of the attribute type. */
# define _BoK_DirAttrErrorProblem_InvalidSyntax (2)

# define BoK_DirAttrError_InvalidSyntax \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_ATTRIBUTE_ERROR,_BoK_DirAttrErrorProblem_InvalidSyntax)

/* An undefined attribute type was provided as an argument to the operation.
   This error may occur only in relation to addEntry or modifyEntry operations. */
# define _BoK_DirAttrErrorProblem_UndefinedType (3)

# define BoK_DirAttrError_UndefinedType \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_ATTRIBUTE_ERROR,_BoK_DirAttrErrorProblem_UndefinedType)

/* An attempt was made, e.g. in a filter, to use a matching rule not defined
   for the attribute type concerned. */
# define _BoK_DirAttrErrorProblem_InappropriateMatching (4)

# define BoK_DirAttrError_InappropriateMatching \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_ATTRIBUTE_ERROR,_BoK_DirAttrErrorProblem_InappropriateMatching)

/* An attribute value supplied in the argument of an operation does not
   conform to the constraints imposed by ITU-T Rec. X.501 | ISO/IEC 9594-2
   or by the attribute definition (e.g. the value exceeds the maximum
   size allowed). */
# define _BoK_DirAttrErrorProblem_ConstraintViolation (5)

# define BoK_DirAttrError_ConstraintViolation \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_ATTRIBUTE_ERROR,_BoK_DirAttrErrorProblem_ConstraintViolation)

/* An attempt was made to add an attribute which already existed in the
   entry, or a value which already existed in the attribute. */
# define _BoK_DirAttrErrorProblem_AlreadyExists (6)

# define BoK_DirAttrError_AlreadyExists \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_ATTRIBUTE_ERROR,_BoK_DirAttrErrorProblem_AlreadyExists)

/* A context list or context supplied with an attribute value in the argument
   of an operation does not conform to the constraints imposed by
   ITU-T Rec. X.501 | ISO/IEC 9594-2, by the context definition
   (e.g. the context value is not of the correct syntax),
   or the DIT Context Use. */
# define _BoK_DirAttrErrorProblem_ContextViolation (7)

# define BoK_DirAttrError_ContextViolation \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_ATTRIBUTE_ERROR,_BoK_DirAttrErrorProblem_ContextViolation)
/* @} */

/* Security Error Problem */
/* @{ */

/* The level of security associated with the requestor's credentials is
   inconsistent with the level of protection requested, e.g. simple
   credentials were supplied while strong credentials were required. */
# define _BoK_DirSecurityErrorProblem_InappropriateAuthentication (1)

# define BoK_DirSecurityError_InappropriateAuthentication \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_InappropriateAuthentication)

/* The supplied credentials were invalid. */
# define _BoK_DirSecurityErrorProblem_InvalidCredentials (2)

# define BoK_DirSecurityError_InvalidCredentials \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_InvalidCredentials)

/* The requestor does not have the right to carry out the requested operation. */
# define _BoK_DirSecurityErrorProblem_InsufficientAccessRights (3)

# define BoK_DirSecurityError_InsufficientAccessRights \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_InsufficientAccessRights)

/* The signature of the request was found to be invalid. */
# define _BoK_DirSecurityErrorProblem_InvalidSignature (4)

# define BoK_DirSecurityError_InvalidSignature \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_InvalidSignature)

/* The Directory was unwilling to carry out the requested operation because
   the argument was not signed. */
# define _BoK_DirSecurityErrorProblem_ProtectionRequired (5)

# define BoK_DirSecurityError_ProtectionRequired \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_ProtectionRequired)

/* The requested operation produced a security error for which
   no information is available. */
# define _BoK_DirSecurityErrorProblem_NoInformation (6)

# define BoK_DirSecurityError_NoInformation \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_NoInformation)

/* The credentials are blocked from consideration for security reasons
   (e.g. because an invalid password has been presented too many times
   in succession). The decision to return this error is governed by
   the security policy in effect for the DSA. */
# define _BoK_DirSecurityErrorProblem_BlockedCredentials (7)

# define BoK_DirSecurityError_BlockedCredentials \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_BlockedCredentials)

/* The two entities have differing protection parameters defined for
   the respective security services. */
# define _BoK_DirSecurityErrorProblem_InvalidQOPMatch (8)

# define BoK_DirSecurityError_InvalidQOPMatch \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_InvalidQOPMatch)

/* The supplied SPKM token was found to be invalid. The spkmInfo parameter
   contains an indication that this is an SPKM error token and the identifier
   of the SPKM context with which this error is associated. */
# define _BoK_DirSecurityErrorProblem_spkmError (9)

# define BoK_DirSecurityError_spkmError \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SECURITY_ERROR,_BoK_DirSecurityErrorProblem_spkmError)
/* @} */

/* Service Error Problem */
/* @{ */

/* The Directory, or some part of it, is presently too busy to perform
   the requested operation, but may be able to do so after a short while. */
# define _BoK_DirServiceErrorProblem_Busy (1)

# define BoK_DirServiceError_Busy \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_Busy)

/* The Directory, or some part of it, is currently unavailable. */
# define _BoK_DirServiceErrorProblem_Unavailable (2)

# define BoK_DirServiceError_Unavailable \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_Unavailable)

/* The Directory, or some part of it, is not prepared to execute this request,
   e.g. because it would lead to excessive consumption of resources or
   violates the policy of an Administrative Authority involved. */
# define _BoK_DirServiceErrorProblem_UnwillingToPerform (3)

# define BoK_DirServiceError_UnwillingToPerform \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_UnwillingToPerform)

/* The Directory is unable to accomplish the request other than by chaining;
   however, chaining was prohibited by means of the chainingProhibited
   service control option. */
# define _BoK_DirServiceErrorProblem_ChainingRequired (4)

# define BoK_DirServiceError_ChainingRequired \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_ChainingRequired)

/* The DSA returning this error did not have administrative authority for the
   appropriate naming context and as a consequence was not able to participate
   in name resolution. */
# define _BoK_DirServiceErrorProblem_UnableToProceed (5)

# define BoK_DirServiceError_UnableToProceed \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_UnableToProceed)

/* The DSA was unable to perform the request as directed by the DUA, (via
   OperationProgress). This may have arisen due to using an invalid referral. */
# define _BoK_DirServiceErrorProblem_InvalidReference (6)

# define BoK_DirServiceError_InvalidReference \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_InvalidReference)

/* The Directory has reached the limit of time set by the user in a service
   control. No partial results are available to return to the user. */
# define _BoK_DirServiceErrorProblem_TimeLimitExceeded (7)

# define BoK_DirServiceError_TimeLimitExceeded \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_TimeLimitExceeded)

/* The Directory has reached some limit set by an administrative authority,
   and no partial results are available to return to the user. */
# define _BoK_DirServiceErrorProblem_AdministrativeLimitExceeded (8)

# define BoK_DirServiceError_AdministrativeLimitExceeded \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_AdministrativeLimitExceeded)

/* The Directory is unable to accomplish this request due to an internal loop. */
# define _BoK_DirServiceErrorProblem_LoopDetected (9)

# define BoK_DirServiceError_LoopDetected \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_LoopDetected)

/* The Directory was unable to satisfy the request because one or more
   critical extensions were not available. */
# define _BoK_DirServiceErrorProblem_UnavailableCriticalExtension (10)

# define BoK_DirServiceError_UnavailableCriticalExtension \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_UnavailableCriticalExtension)

/* No referrals were available within the requested scope. */
# define _BoK_DirServiceErrorProblem_OutOfScope (11)

# define BoK_DirServiceError_OutOfScope \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_OutOfScope)

/* The Directory is unable to accomplish the request due to a DIT
   consistency problem. */
# define _BoK_DirServiceErrorProblem_DITError (12)

# define BoK_DirServiceError_DITError \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_DITError)

/* The parameters of the requested operation are invalid. This problem is
   reported if the queryReference in paged results is invalid. */
# define _BoK_DirServiceErrorProblem_InvalidQueryReference (13)

# define BoK_DirServiceError_InvalidQueryReference \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_InvalidQueryReference)

/* A search request failed within a service specific administrative area
   because no search-rule was available for the search or because the search
   violated an applicable search-rule. Additional diagnostic information may
   be returned together with this service problem. */
# define _BoK_DirServiceErrorProblem_RequestedServiceNotAvail (14)

# define BoK_DirServiceError_RequestedServiceNotAvail \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_RequestedServiceNotAvail)

/* An attempt was made, e.g. in a filter, to use a matching rule not supported
   by the DSA when the performExactly search option is set. */
# define _BoK_DirServiceErrorProblem_MatchRuleNotSupp (15)

# define BoK_DirServiceError_MatchRuleNotSupp \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_MatchRuleNotSupp)

/* A mapping-based matching rule was selected, but the mappable filter items
   provided multiple matches against the relevant mapping table. This error
   situation is accompanied by a notification attribute as indicated by
   the relevant matching-based matching rule. */
# define _BoK_DirServiceErrorProblem_AmbiguousKeyAttributes (16)

# define BoK_DirServiceError_AmbiguousKeyAttributes \
	_BoERRCODE3(BoK_ERROR_DIR_CATEGORY,BoK_DIR_SERVICE_ERROR,_BoK_DirServiceErrorProblem_AmbiguousKeyAttributes)
/* @} */

/* @} */


///* Operation deadlock condition */
//# define BoK_EDEADLOCK 1109         /* EDEADLK */
//
///* The character converter not supported by cconv impl */
//# define BoK_ENOCCONV 1202          /* ENOSYS */
//
///* The URI scheme not supported by uri impl */
//# define BoK_ENOURISCH 1203         /* ENOSYS */
//
///* The PLIST class not supported by plist impl */
//# define BoK_ENOPLCLASS 1204        /* ENOSYS */

///* An object is already inserted */
//# define BoK_DAM_EISINSERT 7012
//
///* A "standalone" object */
//# define BoK_DAM_ENOTINSERT 7013

//	/* Library not supported */
//	BoK_LIBNOTSUPP,
//# define BoK_LIBNOTSUPP BoK_LIBNOTSUPP
//
//	/* Successful function not yet performed */
//	BoK_NOTINITIALISED,
//# define BoK_NOTINITIALISED BoK_NOTINITIALISED
//
//	/* Unexpected error code */
//	BoK_ERRUNEXPECTED,
//# define BoK_ERRUNEXPECTED BoK_ERRUNEXPECTED
//
//	/* Host not found */
//	BoK_HOSTNOTFOUND,
//# define BoK_HOSTNOTFOUND BoK_HOSTNOTFOUND
//
//	/* Class type not found */
//	BoK_TYPENOTFOUND,
//# define BoK_TYPENOTFOUND BoK_TYPENOTFOUND
//
//	/* A nonrecoverable error occurred */
//	BoK_NORECOVERABLE,
//# define BoK_NORECOVERABLE BoK_NORECOVERABLE
//
//	/* Valid name, no data record of requested type */
//	BoK_NODATA,
//# define BoK_NODATA BoK_NODATA

#endif /* !__BO_ERRORS__ */

/******************************************************************************
 * %% BeginSection: UNIX (FreeBSD) ERRORS: [0x0-0x500)
 * @{
 */
#pragma mark -
#pragma mark [ UNIX (FreeBSD) errors ]
#pragma mark -

#ifdef __NEED_UNIX_ERRORS
# undef __NEED_UNIX_ERRORS
# include <Bo/sys/_Errors.h>
#endif
/* @} */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
