/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Jmptab.h	(V.Sokur)
 * include/Bo/Jmptab.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_JMPTAB__
# define __BO_JMPTAB__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Cdefs.h>

# ifndef __USE_DEFAULT_JMPTAB
#  include _BO_COMPILER_HEADER(_Jmptab.h)
# endif

# ifndef BoL_JMPTAB_SIZE
#  define BoL_JMPTAB_SIZE 99
# endif

# ifndef BO_JMPTAB_DEFINED
#  define BO_JMPTAB_DEFINED

/*
 * Usage sample
 *

BO_BEGIN_JMPTAB(R,3)
	BO_JALINK(R,first)
	BO_JALINK(R,second)
	BO_JALINK(R,third)
BO_END_JMPTAB(R)

	for (;;) {
		x = get_code();
		if (x == -1)
			break;

		BO_JMPSW(R,x)

			BO_JMPCASE(R,first):
				// do something
				BO_JMPBREAKSW(R)

			BO_JMPCASE(R,second):
				// do something
				BO_JMPBREAKSW(R)

			BO_JMPCASE(R,third):
				// do something
				BO_JMPBREAKSW(R)

		BO_JMPENDSW(R)
	}


 */

/* you may redefine the macro if you need */
#  define BO_JANAME(aTabName,anActionName) anActionName


#  define BO_BEGIN_JMPTAB(aTabName,N)
#  define BO_JALINK(T,A)
#  define BO_END_JMPTAB(aTabName)


/* x = [0,N], default: 0 */
#  define BO_JMPSW(T,x) switch((x)) {
#  define BO_JMPCASE_DEFAULT(T) default
#  define BO_JMPCASE(T,A) case BO_JANAME(T,A)
#  define BO_JMPBREAKSW(T) goto __myc_xconcat3(_L_jmptab_,T,_finish);
#  define BO_JMPENDSW(T) } __myc_xconcat3(_L_jmptab_,T,_finish):;


/* @{ */
#if 0
#  define BO_JMPTAB_LINK1(T,A) 0
#  define BO_JMPTAB_LINK2(T,A2,A1) 0
#  define BO_JMPTAB_LINK3(T,A3,A2,A1) 0
#  define BO_JMPTAB_LINK4(T,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK5(T,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK6(T,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK7(T,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK8(T,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK9(T,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK10(T,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK11(T,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK12(T,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK13(T,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK14(T,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK15(T,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK16(T,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK17(T,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK18(T,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK19(T,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK20(T,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK21(T,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK22(T,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK23(T,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK24(T,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK25(T,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK26(T,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK27(T,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK28(T,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK29(T,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK30(T,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK31(T,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK32(T,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK33(T,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK34(T,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK35(T,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK36(T,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK37(T,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK38(T,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK39(T,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK40(T,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK41(T,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK42(T,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK43(T,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK44(T,A44,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK45(T,A45,A44,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK46(T,A46,A45,A44,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK47(T,A47,A46,A45,A44,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK48(T,A48,A47,A46,A45,A44,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK49(T,A49,A48,A47,A46,A45,A44,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#  define BO_JMPTAB_LINK50(T,A50,A49,A48,A47,A46,A45,A44,A43,A42,A41,A40,A39,A38,A37,A36,A35,A34,A33,A32,A31,A30,A29,A28,A27,A26,A25,A24,A23,A22,A21,A20,A19,A18,A17,A16,A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1) 0
#endif
/* @} */

# endif /* BO_JMPTAB_DEFINED */

#endif /* !__BO_JMPTAB__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
