/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Noticedefs.h	(N.Panasiuk)
 * include/Bo/Noticedefs.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_NOTICEDEFS__
# define __BO_NOTICEDEFS__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SYS_TYPES__
#  include <Bo/sys/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define BoK_NEVENT_DOMAIN_SHIFT (16)
# define BoK_NEVENT_DOMAIN_MASK (0xffff)
# define BoK_NEVENT_DOMAIN_MASK_S (0xffff0000)
# define BoK_NEVENT_ID_MASK (0xffff)

# define BO_MAKE_NEVENT(aDomain,anId) \
	((((aDomain) & BoK_NEVENT_DOMAIN_MASK) << BoK_NEVENT_DOMAIN_SHIFT) | \
		((anId) & BoK_NEVENT_ID_MASK))

/*
 * domain functionality of event
 */
# define BoK_NEVENT_ANY_DOMAIN (0)

# define BoNEVENT_DOMAIN(anEvent) \
	((u_int32_t) (((anEvent) >> BoK_NEVENT_DOMAIN_SHIFT) & BoK_NEVENT_DOMAIN_MASK))

# define BoNEVENT_CLEAR_DOMAIN(anEvent) \
	((void) ((anEvent) &= ~BoK_NEVENT_DOMAIN_MASK_S))

# define BoNEVENT_SET_DOMAIN(anEvent,aDomain) \
	do { \
		BoNEVENT_CLEAR_DOMAIN(anEvent); \
		(anEvent) |= ((aDomain) & BoK_NEVENT_DOMAIN_MASK) << BoK_NEVENT_DOMAIN_SHIFT; \
	} while (0)

/*
 * "id" functionality of event
 */
# define BoK_NEVENT_ANY_ID (0)

# define BoNEVENT_ID(anEvent) ((anEvent) & BoK_NEVENT_ID_MASK)

# define BoNEVENT_CLEAR_ID(anEvent) \
	((void) ((anEvent) &= ~BoK_NEVENT_ID_MASK))

# define BoNEVENT_SET_ID(anEvent,anId) \
	do { \
		BoNEVENT_CLEAR_ID(anEvent); \
		(anEvent) |= (anId) & BoK_NEVENT_ID_MASK; \
	} while (0)

/*
 * common functionality of event
 */

/* bo_mismatch_t BO_NEVENT_Compare(int32_t aLhs, int32_t aRhs); */
# if __myc_support(macro_retval)
#  define BO_NEVENT_Compare(aLhs, aRhs) \
	__extension__ ({ \
		int32_t ___mismatch; \
		register int32_t ___lhs, ___rhs; \
		do { \
			___lhs = (int32_t)BoNEVENT_DOMAIN(aLhs); \
			___rhs = (int32_t)BoNEVENT_DOMAIN(aRhs); \
			if (0 != (___mismatch = ___lhs - ___rhs)) { \
				if (BoK_NEVENT_ANY_DOMAIN != ___lhs && BoK_NEVENT_ANY_DOMAIN != ___rhs) \
					break; \
			} \
			___lhs = (int32_t)BoNEVENT_ID(aLhs); \
			___rhs = (int32_t)BoNEVENT_ID(aRhs); \
			if (0 != (___mismatch = ___lhs - ___rhs)) { \
				if (BoK_NEVENT_ANY_ID != ___lhs && BoK_NEVENT_ANY_ID != ___rhs) \
					break; \
				___mismatch = 0; \
			} \
		} while (0); \
		/* return */ (___mismatch); \
	})
# else
__myc_extern_inline__ bo_mismatch_t
BO_NEVENT_Compare(
#  if __myc_require(prototypes)
	int32_t aLhs,
	int32_t aRhs
#  else
	aLhs, aRhs
#  endif
	)
#  if !__myc_require(prototypes)
	int32_t aLhs, aRhs;
#  endif
{
	bo_mismatch_t retval;
	register int32_t lhs, rhs;

	lhs = BoNEVENT_DOMAIN(aLhs);
	rhs = BoNEVENT_DOMAIN(aRhs);
	if (0 != (retval = lhs - rhs)) {
		if (BoK_NEVENT_ANY_DOMAIN != lhs && BoK_NEVENT_ANY_DOMAIN != rhs) {
			return (retval);
			/* NOTREACHED */
		}
	}
	lhs = BoNEVENT_ID(aLhs);
	rhs = BoNEVENT_ID(aRhs);
	if (0 != (retval = lhs - rhs)) {
		if (BoK_NEVENT_ANY_ID != lhs && BoK_NEVENT_ANY_ID != rhs) {
			return (retval);
			/* NOTREACHED */
		}
	}
	return (0);
}
# endif


/* "NULL" event */
# define BoK_NULL_NEVENT (0)

/******************************************************************************
 * %% BeginSection: NOTICE definitions
 */
# pragma mark -
# pragma mark [ NOTICE definitions ]
# pragma mark -

struct bo_notice_s {
	u_int32_t     n_event;
};
typedef struct bo_notice_s *bo_notice_t;

struct bo_vnotice {
	u_int32_t     n_event;
	union {
		int32_t   n_val32;
		int32_t   n_val32v[2];
		u_int64_t n_val64;
	} u;
};

struct bo_exnotice {
	u_int32_t     n_event;
	union {
		int32_t   n_val32;
		int32_t   n_val32v[2];
		u_int64_t n_val64;
	} u;
	bo_pointer_t  n_sender;
	bo_pointer_t  n_uctx;
};

# define bo_notice_p(x) ((struct bo_notice_s *)(x))
# define bo_vnotice_p(x) ((struct bo_vnotice *)(x))
# define bo_exnotice_p(x) ((struct bo_exnotice *)(x))

# define BO_NOTICE_Event(x) (bo_notice_p(x)->n_event)
# define BO_NOTICE_Val32(x) (bo_vnotice_p(x)->u.n_val32)
# define BO_NOTICE_Val64(x) (bo_vnotice_p(x)->u.n_val64)
# define BO_NOTICE_Val32v0(x) (bo_vnotice_p(x)->u.n_val32v[0])
# define BO_NOTICE_Val32v1(x) (bo_vnotice_p(x)->u.n_val32v[1])
# define BO_NOTICE_Val32v(x,i) (bo_vnotice_p(x)->u.n_val32v[i])
# define BO_NOTICE_Sender(x) (bo_exnotice_p(x)->n_sender)
# define BO_NOTICE_Context(x) (bo_exnotice_p(x)->n_uctx)

#endif /* !__BO_NOTICEDEFS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
