/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Optdefs.h	(V.Sokur)
 * include/Bo/Optdefs.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_OPTDEFS__
# define __BO_OPTDEFS__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_TYPES__
#  include <Bo/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# ifndef BO_NO_NAMELESS_UNION
#  define _BO_OPTIONS_ACS_NAME
# else
#  define _BO_OPTIONS_ACS_NAME u
# endif

struct bo_options {
	struct bo_snabsbuf  opts;
		/* 1. (0 <= opts.size) => (bo_opthdr, ...) (total size in bytes);
		   2. (opts.size < 0) => bo_optb[] (count = -opts.size);
		       NOTE!!! currently not implemented */

	union {
		bo_scalar_t     __action;
		bo_scalar_t     __status;
	} _BO_OPTIONS_ACS_NAME;
};
# define bo_options_p(x) ((struct bo_options *)(x))

# define BO_OPTIONS_GUTS \
	struct bo_snabsbuf    opts; \
	union { \
		bo_scalar_t     __action; \
		bo_scalar_t     __status; \
	} _BO_OPTIONS_ACS_NAME


# ifndef BO_NO_NAMELESS_UNION
#  define BO_OPTIONS_Action(x) (bo_options_p(x)->__action)
#  define BO_OPTIONS_Status(x) (bo_options_p(x)->__status)
# else
#  define BO_OPTIONS_Action(x) (bo_options_p(x)->u.__action)
#  define BO_OPTIONS_Status(x) (bo_options_p(x)->u.__status)
# endif

# define BO_OPTIONS_NBytes(x) \
	((0 <= bo_options_p(x)->opts.size) ? (bo_options_p(x)->opts.size) : -1)

/* @todo: implement */
# define BO_OPTIONS_Count(x) \
	((bo_options_p(x)->opts.size < 0) ? (-bo_options_p(x)->opts.size) : -1)

/*
 * values for bo_options.action
 */

# define BoK_OPT_GETDEFAULT 0
# define BoK_OPT_GETCURRENT 1
# define BoK_OPT_CHECK 2 /* are options supported? */
# define BoK_OPT_NEGOTIATE 3

/*
 * OPTCTL OPERATION ONLY
 *
 * values for an option status returned at the field 'status' of
 * the struct bo_opthdr
 *
 * BoS_OK: successful result
 * BoS_NORMAL: partial success result
 * BoS_ERR: failure result
 * BoS_NOTSUPP: not supported result
 * BoS_READONLY: read-only result
 */
/*
 * status returned from the operation:
 *		1. BoS_OK: all options are processed successfully
 *		2. BoS_NORMAL: at least 1 option is not processed successfully, is unknown.
 *
 * status assigned to the 'status' field of the struct bo_options:
 *		1. BoS_OK: all options are processed successfully
 *		2. BoS_NORMAL:
 *			1) there is at least one unknown option in the options;
 *			   for that option: opthdr->status = BoS_NOTSUPP
 *		3. BoS_ERR:
 *			1) option argument is unknown; opthdr->status = BoS_NOTSUPP
 *			2) option handling failure; opthdr->status = BoS_ERR or BoS_READONLY
 */

/*
 * optctl example:
 *

	struct bo_options *Options;

	bo_status_t status, optsStatus, optStatus;
	struct bo_opthdr *ohdr;

	...

	status = BoS_OK;

	...

	optsStatus = BoS_OK;
	for (ohdr = BO_OPTIONS_FIRSTHDR(Options);
			NULL != ohdr;
				ohdr = BO_OPTIONS_NEXTHDR(Options, ohdr))
	{
		...

		if (MyOptionToHandle(ohdr)) {
			optStatus = MyOptionHandler(..., _BO_OPT_DATA(ohdr), ...);
			BO_OPT_Status(ohdr) = (bo_scalar_t)optStatus;
			if (BoS_FAILURE(optStatus)) {
				optsStatus = BoS_ERR;
				status = BoS_NORMAL;
			}
			continue;
		}

		...

		BO_OPT_Status(ohdr) = BoS_NOTSUPP;
		if (BoS_SUCCESS(optsStatus))
			optsStatus = BoS_NORMAL;
		status = BoS_NORMAL;
	}
	BO_OPTIONS_Status(Options) = (bo_scalar_t)optsStatus;

	...

	return (status);

 */

/*
 * control example:
 *

	int aRq;
	struct bo_options *Options;

	bo_status_t status;

	...

	status = BoS_ERR;
	switch (aRq) {
		...
		case MyControlCommand: {
			struct bo_opthdr *opt;
			struct my_optdata *optdata;

			opt = NULL;
			BO_OPTIONS_FIND_LAST(opt, Options, MyOptDomain, MyOptName);
			if (NULL != opt) {
				optdata = _BO_OPT_DATA(opt);
				status = MyControlHandler(..., optdata, ...);
				BO_OPT_Status(opt) = (bo_scalar_t)status;
				if (BoS_SUCCESS(status)) {
					// do something;
				}
			}
			break;
		}
		...
	}

	...

	return (status);

 */

struct bo_opthdr {
	bo_uscalar_t  total_len; /* sizeof(struct bo_opthdr) + length(opt-value) */
	bo_uscalar_t  domain;    /* level, protocol, ... */
	bo_uscalar_t  name;
	bo_scalar_t   status;
	/* data */
};
# define bo_opthdr_p(x) ((struct bo_opthdr *)(x))

struct bo_nmoptdata {
	unsigned  nam;
};

struct bo_optb {
	bo_uscalar_t  total_len; /* sizeof(struct bo_optb) */
	bo_uscalar_t  domain;    /* level, protocol, ... */
	bo_uscalar_t  name;
	bo_scalar_t   status;
	bo_pointer_t  data;
};
# define bo_optb_p(x) ((struct bo_optb *)(x))

/*
 * [well known] values for bo_opthdr.domain
 */

# ifndef BoK_MEM_OPT
#  define BoK_MEM_OPT __myc_mk4ccs("memo")
#  define _BoK_MEM_OPT __myc_mk4cc('m','e','m','o')
# endif
# ifndef BoK_FIOS_OPT
#  define BoK_FIOS_OPT __myc_mk4ccs("fios")
#  define _BoK_FIOS_OPT __myc_mk4cc('f','i','o','s')
# endif

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/* Calculate the value to store in .total_len */
# define BO_OPT_TOTAL_LEN(aDatalen) \
	(_BO_OPTDATA_ALIGN(sizeof(struct bo_opthdr)) + aDatalen)

/* Calculate the size suitable for option memory allocation */
# define BO_OPT_SIZE(aDatalen) \
	(_BO_OPTDATA_ALIGN(sizeof(struct bo_opthdr) + _BO_OPTHDR_ALIGN(aDatalen)))
# define BO_OPT_SIZE2(aLen1,aLen2) \
	(BO_OPT_SIZE(aLen2) + BO_OPT_SIZE(aLen1))
# define BO_OPT_SIZE3(aLen1,aLen2,aLen3) \
	(BO_OPT_SIZE(aLen3) + BO_OPT_SIZE2(aLen1,aLen2))
# define BO_OPT_SIZE4(aLen1,aLen2,aLen3,aLen4) \
	(BO_OPT_SIZE(aLen4) + BO_OPT_SIZE3(aLen1,aLen2,aLen3))
# define BO_OPT_SIZE5(aLen1,aLen2,aLen3,aLen4,aLen5) \
	(BO_OPT_SIZE(aLen5) + BO_OPT_SIZE3(aLen1,aLen2,aLen3,aLen4))


# define BO_OPT_TOTAL_LEN_OF(_Tp) BO_OPT_TOTAL_LEN(sizeof(_Tp))

# define BO_OPT_SIZE_OF(_Tp) \
	BO_OPT_SIZE(sizeof(_Tp))
# define BO_OPT_SIZE_OF2(_Tp1,_Tp2) \
	(BO_OPT_SIZE_OF(_Tp2) + BO_OPT_SIZE_OF(_Tp1))
# define BO_OPT_SIZE_OF3(_Tp1,_Tp2,_Tp3) \
	(BO_OPT_SIZE_OF(_Tp3) + BO_OPT_SIZE_OF2(_Tp1,_Tp2))
# define BO_OPT_SIZE_OF4(_Tp1,_Tp2,_Tp3,_Tp4) \
	(BO_OPT_SIZE_OF(_Tp4) + BO_OPT_SIZE_OF3(_Tp1,_Tp2,_Tp3))
# define BO_OPT_SIZE_OF5(_Tp1,_Tp2,_Tp3,_Tp4,_Tp5) \
	(BO_OPT_SIZE_OF(_Tp5) + BO_OPT_SIZE_OF3(_Tp1,_Tp2,_Tp3,_Tp4))

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/*
struct my_opt {
	struct bo_opthdr  hdr;
	struct my_optdata data;
};
 */

# define BO_OPT_TAG(aName) aName
# define BO_OPT_TYPENAME(aName) struct BO_OPT_TAG(aName)
# define BO_OPT_STRUCT(aName,_Tp) \
	BO_OPT_TYPENAME(aName) { \
		unsigned char  opaque[BO_OPT_SIZE_OF(_Tp)]; \
	}

/* ------------------------------------------------------------------------- */

# define BO_OPT_TotalLen(anOpthdr) bo_opthdr_p(anOpthdr)->total_len
# define BO_OPT_Domain(anOpthdr) bo_opthdr_p(anOpthdr)->domain
# define BO_OPT_Name(anOpthdr) bo_opthdr_p(anOpthdr)->name
# define BO_OPT_Status(anOpthdr) bo_opthdr_p(anOpthdr)->status

# define BO_OPT_DATA(_Tp, anOpthdr) \
	((_Tp *) _BO_OPT_DATA(anOpthdr))

# if __myc_support(macro_retval)
#  define _BO_OPT_DATA(anOpthdr) \
	__extension__ ({ \
		bo_pointer_t ___ohp = (bo_pointer_t)((bo_byte_t *)(anOpthdr) + sizeof(struct bo_opthdr)); \
		___ohp = _BO_OPTDATA_ALIGN_PTR(___ohp); \
		/* return */ (___ohp); \
	})
# else
#  define _BO_OPT_DATA(anOpthdr) \
	(_BO_OPTDATA_ALIGN_PTR((char *)(anOpthdr) + sizeof(struct bo_opthdr)))
# endif

/* ------------------------------------------------------------------------- */

/*
 * Header only init
 */

# define BO_OPT_STATIC_INIT(_TDataType, anOptdom, anOptname) \
	{ \
		/* .hdr = */ { \
			/* .total_len = */ BO_OPT_TOTAL_LEN_OF(_TDataType), \
			/* .domain = */ (anOptdom), \
			/* .name = */ (anOptname), \
			/* .status = */ BoS_NOTHANDLED \
		} \
	}

# define BO_OPT_RTS_INIT(_TDataType, anOption, anOptdom, anOptname) \
	do { \
		struct bo_opthdr *___ohdr = bo_opthdr_p(anOption); \
		___ohdr->total_len = BO_OPT_TOTAL_LEN_OF(_TDataType); \
		___ohdr->domain = (anOptdom); \
		___ohdr->name = (anOptname); \
		___ohdr->status = BoS_NOTHANDLED; \
	} while (0)

# define _BO_OPT_RTS_INIT(_TDataType, anOption, anOptdom, anOptname) \
	BO_OPT_TotalLen(anOption) = BO_OPT_TOTAL_LEN_OF(_TDataType); \
	BO_OPT_Domain(anOption) = (anOptdom); \
	BO_OPT_Name(anOption) = (anOptname); \
	BO_OPT_Status(anOption) = BoS_NOTHANDLED

# define BO_OPT_SET(anOpthdr, anOptdom, anOptname, anOptval, aVallen) \
	do { \
		struct bo_opthdr *___ohdr = bo_opthdr_p(anOpthdr); \
		___ohdr->total_len = BO_OPT_TOTAL_LEN(aVallen); \
		___ohdr->domain = (anOptdom); \
		___ohdr->name = (anOptname); \
		___ohdr->status = BoS_NOTHANDLED; \
		memcpy(_BO_OPT_DATA(___ohdr), (anOptval), (aVallen)); \
	} while (0)

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

# define BO_OPTIONS_TAG(aName) __myc_xconcat(bo_options_,aName)
# define BO_OPTIONS_TYPENAME(aName) struct BO_OPTIONS_TAG(aName)
# define BO_OPTIONS_STRUCT(aName,NB) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[NB]; \
	}


# define _BO_OPTIONS_STRUCT1(aName,aDatalen) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE(aDatalen)]; \
	}
# define _BO_OPTIONS_STRUCT2(aName,aLen1,aLen2) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE2(aLen1,aLen2)]; \
	}
# define _BO_OPTIONS_STRUCT3(aName,aLen1,aLen2,aLen3) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE3(aLen1,aLen2,aLen3)]; \
	}
# define _BO_OPTIONS_STRUCT4(aName,aLen1,aLen2,aLen3,aLen4) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE4(aLen1,aLen2,aLen3,aLen4)]; \
	}
# define _BO_OPTIONS_STRUCT5(aName,aLen1,aLen2,aLen3,aLen4,_tp5) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE5(aLen1,aLen2,aLen3,aLen4,aLen5)]; \
	}


# define BO_OPTIONS_STRUCT1(aName,_Tp) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE_OF(_Tp)]; \
	}
# define BO_OPTIONS_STRUCT2(aName,_Tp1,_Tp2) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE_OF2(_Tp1,_Tp2)]; \
	}
# define BO_OPTIONS_STRUCT3(aName,_Tp1,_Tp2,_Tp3) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE_OF3(_Tp1,_Tp2,_Tp3)]; \
	}
# define BO_OPTIONS_STRUCT4(aName,_Tp1,_Tp2,_Tp3,_Tp4) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE_OF4(_Tp1,_Tp2,_Tp3,_Tp4)]; \
	}
# define BO_OPTIONS_STRUCT5(aName,_Tp1,_Tp2,_Tp3,_Tp4,_tp5) \
	BO_OPTIONS_TYPENAME(aName) { \
		BO_OPTIONS_GUTS; \
		unsigned char  optbuf[BO_OPT_SIZE_OF5(_Tp1,_Tp2,_Tp3,_Tp4,_Tp5)]; \
	}

/* ------------------------------------------------------------------------- */

# define _BO_OPTIONS_ACS_STATIC_INIT(anAction) { anAction }

# define BO_OPTIONS_STATIC_INIT(_TOptionsType, anAction) \
	{ \
		/* .opts = */ { \
			/* .capacity = */ sizeof(_TOptionsType) - sizeof(struct bo_options), \
			/* .size = */ sizeof(_TOptionsType) - sizeof(struct bo_options), \
			/* .base = */ NULL \
		}, \
		/* .action = */ _BO_OPTIONS_ACS_STATIC_INIT(anAction) \
	}

# define BO_OPTIONS_RTS_INIT(_TOptionsType, Options, anAction) \
	do { \
		struct bo_options *___options = bo_options_p(Options); \
		___options->opts.capacity = sizeof(_TOptionsType) - sizeof(struct bo_options); \
		___options->opts.size = sizeof(_TOptionsType) - sizeof(struct bo_options); \
		___options->opts.base = (bo_pointer_t)((bo_byte_t *)___options + sizeof(struct bo_options)); \
		BO_OPTIONS_Action(___options) = (anAction); \
	} while (0)

# define BO_OPTIONS_RTS_INIT2(Options, anAction) \
	do { \
		struct bo_options *___options = bo_options_p(Options); \
		___options->opts.capacity = (sizeof *(Options)) - sizeof(struct bo_options); \
		___options->opts.size = (sizeof *(Options)) - sizeof(struct bo_options); \
		___options->opts.base = (bo_pointer_t)((bo_byte_t *)___options + sizeof(struct bo_options)); \
		BO_OPTIONS_Action(___options) = (anAction); \
	} while (0)

/* ------------------------------------------------------------------------- */

# define BO_OPTIONS_FIRSTHDR(Options) \
	((struct bo_opthdr *) _BO_OPTIONS_FIRSTHDR(Options))

# define BO_OPTIONS_NEXTHDR(Options, anOpthdr) \
	((NULL == (anOpthdr)) \
		? ((struct bo_opthdr *) _BO_OPTIONS_FIRSTHDR(Options)) \
		: ((struct bo_opthdr *) _BO_OPTIONS_NEXTHDR(Options, anOpthdr)))

# define BO_OPTIONS_ADD(Options, anOpthdr, anOptdom, anOptname, anOptval, aVallen) \
	do { \
		struct bo_opthdr *___nextohdr, *___ohdr = bo_opthdr_p(anOpthdr); \
		struct bo_options *___options = (Options); \
		(anOpthdr) = NULL; \
		if (NULL != (___nextohdr = BO_OPTIONS_NEXTHDR(___options, ___ohdr))) { \
			BO_OPT_SET(___nextohdr, anOptdom, anOptname, anOptval, aVallen); \
			*(struct bo_opthdr **)&(anOpthdr) = ___nextohdr; \
		} \
	} while (0)

# define BO_OPTIONS_ADD2(_TDataType, Options, anOpthdr, anOptdom, anOptname, CONSTRUCTOR) \
	do { \
		struct bo_opthdr *_$1, *___ohdr = bo_opthdr_p(anOpthdr); \
		struct bo_options *___options = (Options); \
		(anOpthdr) = NULL; \
		if (NULL != (_$1 = BO_OPTIONS_NEXTHDR(___options, ___ohdr))) { \
			_BO_OPT_RTS_INIT(_TDataType, _$1, anOptdom, anOptname); \
			CONSTRUCTOR; \
			*(struct bo_opthdr **)&(anOpthdr) = _$1; \
		} \
	} while (0)


# define BO_OPTIONS_FINDI(anOpthdr, Options, anIndex) \
	do { \
		bo_index_t ___idx, ___idx2; \
		struct bo_opthdr *___ohdr; \
		struct bo_options *___options = bo_options_p(Options); \
		(anOpthdr) = NULL; \
		___idx = 0; \
		___idx2 = (bo_index_t)(anIndex); \
		for (___ohdr = (struct bo_opthdr *)_BO_OPTIONS_FIRSTHDR(___options); \
				NULL != ___ohdr; \
					___ohdr = (struct bo_opthdr *)_BO_OPTIONS_NEXTHDR(___options, ___ohdr)) \
		{ \
			if (___idx++ == ___idx2) { \
				*(struct bo_opthdr **)&(anOpthdr) = ___ohdr; \
				break; \
			} \
		} \
	} while (0)

# define BO_OPTIONS_FIND_FIRST(anOpthdr, Options, anOptdom, anOptname) \
	do { \
		struct bo_opthdr *___ohdr; \
		struct bo_options *___options = bo_options_p(Options); \
		(anOpthdr) = NULL; \
		for (___ohdr = (struct bo_opthdr *)_BO_OPTIONS_FIRSTHDR(___options); \
				NULL != ___ohdr; \
					___ohdr = (struct bo_opthdr *)_BO_OPTIONS_NEXTHDR(___options, ___ohdr)) \
		{ \
			if ((anOptdom) == ___ohdr->domain && (anOptname) == ___ohdr->name) { \
				*(struct bo_opthdr **)&(anOpthdr) = ___ohdr; \
				break; \
			} \
		} \
	} while (0)

# define BO_OPTIONS_FIND_LAST(anOpthdr, Options, anOptdom, anOptname) \
	do { \
		struct bo_opthdr *___ohdr, *___ohdrFound; \
		struct bo_options *___options = bo_options_p(Options); \
		___ohdrFound = NULL; \
		for (___ohdr = (struct bo_opthdr *)_BO_OPTIONS_FIRSTHDR(___options); \
				NULL != ___ohdr; \
					___ohdr = (struct bo_opthdr *)_BO_OPTIONS_NEXTHDR(___options, ___ohdr)) \
		{ \
			if ((anOptdom) == ___ohdr->domain && (anOptname) == ___ohdr->name) { \
				___ohdrFound = ___ohdr; \
			} \
		} \
		*(struct bo_opthdr **)&(anOpthdr) = ___ohdrFound; \
	} while (0)

# define BO_OPTIONS_ENUM(Options, PREDICATE) \
	do { \
		struct bo_opthdr *_$1; \
		struct bo_options *___options = bo_options_p(Options); \
		for (_$1 = (struct bo_opthdr *)_BO_OPTIONS_FIRSTHDR(___options); \
				NULL != _$1; \
					_$1 = (struct bo_opthdr *)_BO_OPTIONS_NEXTHDR(___options, _$1)) \
		{ \
			PREDICATE; \
		} \
	} while (0)


# define BO_OPTIONS_FIND_FIRST3(anOpthdr, Options, anOptdom, anOptname, aName3) \
	do { \
		struct bo_nmoptdata *___optdata; \
		struct bo_opthdr *___ohdr; \
		struct bo_options *___options = bo_options_p(Options); \
		(anOpthdr) = NULL; \
		for (___ohdr = (struct bo_opthdr *)_BO_OPTIONS_FIRSTHDR(___options); \
				NULL != ___ohdr; \
					___ohdr = (struct bo_opthdr *)_BO_OPTIONS_NEXTHDR(___options, ___ohdr)) \
		{ \
			___optdata = BO_OPT_DATA(struct bo_nmoptdata, ___ohdr); \
			if ((anOptdom) == ___ohdr->domain && \
				(anOptname) == ___ohdr->name && \
				(aName3) == ___optdata->nam) \
			{ \
				*(struct bo_opthdr **)&(anOpthdr) = ___ohdr; \
				break; \
			} \
		} \
	} while (0)

# define BO_OPTIONS_FIND_LAST3(anOpthdr, Options, anOptdom, anOptname, aName3) \
	do { \
		struct bo_nmoptdata *___optdata; \
		struct bo_opthdr *___ohdr, *___ohdrFound; \
		struct bo_options *___options = bo_options_p(Options); \
		___ohdrFound = NULL; \
		for (___ohdr = (struct bo_opthdr *)_BO_OPTIONS_FIRSTHDR(___options); \
				NULL != ___ohdr; \
					___ohdr = (struct bo_opthdr *)_BO_OPTIONS_NEXTHDR(___options, ___ohdr)) \
		{ \
			___optdata = BO_OPT_DATA(struct bo_nmoptdata, ___ohdr); \
			if ((anOptdom) == ___ohdr->domain && \
				(anOptname) == ___ohdr->name && \
				(aName3) == ___optdata->nam) \
			{ \
				___ohdrFound = ___ohdr; \
			} \
		} \
		*(struct bo_opthdr **)&(anOpthdr) = ___ohdrFound; \
	} while (0)


/* NOT FOR YOUR BUSINESS @{ */
# define _BO_OPTIONS_Size(Options) bo_options_p(Options)->opts.size
# define _BO_OPTIONS_Base(Options) bo_options_p(Options)->opts.base

# define _BO_OPTIONS_FIRSTHDR(Options) \
	( ((ssize_t)sizeof(struct bo_opthdr) <= _BO_OPTIONS_Size(Options)) \
		? ((char *)_BO_OPTIONS_Base(Options)) \
		: ((char *)0) )

# define _BO_OPTIONS_NEXTHDR(Options, anOpthdr) \
	( ((_BO_OPTHDR_Nextdp(anOpthdr) - (char *)_BO_OPTIONS_Base(Options)) <= _BO_OPTIONS_Size(Options)) \
			? _BO_OPTHDR_Nextp(anOpthdr) \
			: ((char *)0) )

# define _BO_OPTHDR_Nextp(anOpthdr) \
	((char *)_BO_OPTHDR_ALIGN_PTR((char *)(anOpthdr) + bo_opthdr_p(anOpthdr)->total_len))

# define _BO_OPTHDR_Nextdp(anOpthdr) \
	(_BO_OPTHDR_Nextp(anOpthdr) + sizeof(struct bo_opthdr))

# define _BO_OPTHDR_ALIGN(aLen) __myc_align((aLen), _BoK_OPT_HALIGN)
# define _BO_OPTHDR_ALIGN_PTR(aPtr) __myc_align_ptr((aPtr), _BoK_OPT_HALIGN)
# define _BO_OPTDATA_ALIGN(aLen) __myc_align((aLen), _BoK_OPT_DALIGN)
# define _BO_OPTDATA_ALIGN_PTR(aPtr) __myc_align_ptr((aPtr), _BoK_OPT_DALIGN)

# if 1
#  define _BoK_OPT_HALIGN __myc_alignof(struct bo_opthdr)
# else
#  define _BoK_OPT_HALIGN sizeof(bo_scalar_t)
# endif
# define _BoK_OPT_DALIGN sizeof(int32_t)
/* @} */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

# define BO_OPTIONS_AT(Options,anIdx) \
	(bo_optb_p(bo_options_p(Options)->opts.base)[anIdx])

#endif /* !__BO_OPTDEFS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
