/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Statedefs.h	(V.Sokur)
 * include/Bo/Statedefs.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_STATEDEFS__
# define __BO_STATEDEFS__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SYS_TYPES__
#  include <Bo/sys/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# ifdef HAS_INT64_TYPE
typedef int64_t bo_state_t;
typedef int32_t bo_state1_t; /* low 16 bits are only significant */
typedef int64_t bo_state_x;
# else
typedef int32_t bo_state_t;
typedef int32_t bo_state1_t; /* low 16 bits are only significant */
typedef int32_t bo_state_x;
# endif

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_bo_state_t,
	sizeof(bo_state_t) <= sizeof(bo_status_t));
# endif
/* @} */


# ifdef HAS_INT64_TYPE

#  define BoK_STATE_DOMAIN_SHIFT 32
#  define BoK_STATE_DOMAIN_MASK 0xffffffffLL
#  define BoK_STATE_DOMAIN_MASK_S 0xffffffff00000000LL
#  define BoK_STATE_NSTATE1_SHIFT 16
#  define BoK_STATE_NSTATE1_MASK 0xffff0000LL
#  define BoK_STATE_STATE1_MASK 0xffffLL

#  define BO_MAKE_STATE3(aDomain,aNewState,anOldState) \
	BO_MAKE_STATE64(aDomain,aNewState,anOldState)

# else /* !HAS_INT64_TYPE */

#  define BoK_STATE_DOMAIN_SHIFT 0
#  define BoK_STATE_DOMAIN_MASK 0
#  define BoK_STATE_DOMAIN_MASK_S 0
#  define BoK_STATE_NSTATE1_SHIFT 16
#  define BoK_STATE_NSTATE1_MASK 0xffff0000L
#  define BoK_STATE_STATE1_MASK 0xffffL

#  define BO_MAKE_STATE3(aDomain,aNewState,anOldState) \
	BO_MAKE_STATE32(aDomain,aNewState,anOldState)

# endif /* HAS_INT64_TYPE */

#  define BO_MAKE_STATE2(aNewState,anOldState) \
	((((aNewState) & BoK_STATE_STATE1_MASK) << BoK_STATE_NSTATE1_SHIFT) | \
		((anOldState) & BoK_STATE_STATE1_MASK))

#  define BO_MAKE_STATE64(aDomain,aNewState,anOldState) \
	(((u_int64_t)((aDomain) & BoK_STATE_DOMAIN_MASK) << BoK_STATE_DOMAIN_SHIFT) | \
		(((aNewState) & BoK_STATE_STATE1_MASK) << BoK_STATE_NSTATE1_SHIFT) | \
		((anOldState) & BoK_STATE_STATE1_MASK))

#  define BO_MAKE_STATE32(aDomain,aNewState,anOldState) \
	BO_MAKE_STATE2(aNewState,anOldState)

/* make established state */
# define _BoESTABLISHEDSTATE(aDomain,aState1) \
	BO_MAKE_STATE3(aDomain,BoK_ESTABLISHED_STATE,aState1)

/* make transient state */
# define _BoTRANSIENTSTATE(aDomain,aNewState,anOldState) \
	BO_MAKE_STATE3(aDomain,aNewState,anOldState)

/*
 * domain functionality of state
 */
# define BoSTATE_DOMAIN(aState) \
	((u_int32_t) (((aState) >> BoK_STATE_DOMAIN_SHIFT) & BoK_STATE_DOMAIN_MASK))

# define BoSTATE_CLEAR_DOMAIN(aState) \
	((void) ((aState) &= ~BoK_STATE_DOMAIN_MASK_S))

# define BoSTATE_SET_DOMAIN(aState,aDomain) \
	do { \
		BoSTATE_CLEAR_DOMAIN(aState); \
		(aState) |= (bo_state_t)((u_int32_t)(aDomain) & BoK_STATE_DOMAIN_MASK) << BoK_STATE_DOMAIN_SHIFT; \
	} while (0)

/*
 * new-state functionality of state
 */
# define BoSTATE_NSTATE1(aState) \
	((bo_state1_t) (((aState) >> BoK_STATE_NSTATE1_SHIFT) & BoK_STATE_STATE1_MASK))

# define BoSTATE_CLEAR_NSTATE1(aState) \
	((void) ((aState) &= ~BoK_STATE_NSTATE1_MASK))

# define BoSTATE_SET_NSTATE1(aState,aState1) \
	do { \
		BoSTATE_CLEAR_NSTATE1(aState); \
		(aState) |= (bo_state_t)((aState1) & BoK_STATE_STATE1_MASK) << BoK_STATE_NSTATE1_SHIFT; \
	} while (0)

/*
 * old-state functionality of state
 */
# define BoSTATE_OSTATE1(aState) \
	((bo_state1_t) ((aState) & BoK_STATE_STATE1_MASK))

# define BoSTATE_CLEAR_OSTATE1(aState) \
	((void) ((aState) &= ~BoK_STATE_STATE1_MASK))

# define BoSTATE_SET_OSTATE1(aState,aState1) \
	do { \
		BoSTATE_CLEAR_OSTATE1(aState); \
		(aState) |= (bo_state_t)((aState1) & BoK_STATE_STATE1_MASK); \
	} while (0)

# define BoSTATE_STATE1(aState) BoSTATE_OSTATE1(aState)

/*
 * "real"-state functionality of state
 */
# define BoSTATE_RSTATE1(aState) \
	((bo_state1_t) ((aState) & (BoK_STATE_STATE1_MASK | BoK_STATE_NSTATE1_MASK)))

# define BoSTATE_CLEAR_RSTATE1(aState) \
	((void) ((aState) &= ~(BoK_STATE_STATE1_MASK | BoK_STATE_NSTATE1_MASK)))

# define BoSTATE_SET_RSTATE1(aState,aNewState,anOldState) \
	do { \
		BoSTATE_CLEAR_RSTATE1(aState); \
		(aState) |= (bo_state_t)((aNewState) & BoK_STATE_STATE1_MASK) << BoK_STATE_NSTATE1_SHIFT; \
		(aState) |= (bo_state_t)((anOldState) & BoK_STATE_STATE1_MASK); \
	} while (0)

/*
 * mark state as transient (rstate: {new-state, old-state})
 */
# define BoSTATE_SET_TRANSIENT(aState,aNewState) \
	BoSTATE_SET_NSTATE1(aState,aNewState)

# define BoSTATE_REVERT_TRANSIENT(aState) \
	BoSTATE_SET_NSTATE1(aState,BoK_ESTABLISHED_STATE)

/*
 * mark state as established (rstate: {established-state, state1})
 */
# define BoSTATE_SET_ESTABLISHED(aState) \
	do { \
		bo_state1_t ___ostate = BoSTATE_NSTATE1(aState); \
		if (BoK_ESTABLISHED_STATE != ___ostate) { \
			u_int32_t __domain = BoSTATE_DOMAIN(aState); \
			(aState) = _BoESTABLISHEDSTATE(__domain,___ostate); \
		} \
	} while (0)

/*
 * mark state as established (rstate: {established-state, new-state})
 */
# define BoSTATE_SET_ESTABLISHED2(aState,aNewState) \
	BoSTATE_SET_RSTATE1(aState, BoK_ESTABLISHED_STATE, aNewState)


# define BoSTATE_ISINVAL(aState) (BoK_INVAL_STATE == (aState))
# define BoSTATE_ISTRANSIENT(aState) (!BoSTATE_ISESTABLISHED(aState))
# define BoSTATE_ISESTABLISHED(aState) \
	(BoK_ESTABLISHED_STATE == BoSTATE_NSTATE1(aState))


/* INVALID state */
# ifdef HAS_INT64_TYPE
#  define BoK_INVAL_STATE (-1LL)
# else
#  define BoK_INVAL_STATE (-1L)
# endif

/* "NULL" state */
# define BoK_NULL_STATE (0) /* fini, init */

/* "ESTABLISHED" state */
# define BoK_ESTABLISHED_STATE (1)

/* UNKNOWN state */
# define BoK_UNK_STATE (2)

# define BoK_INITED_STATE (3)
# define BoK_CONFIGURED_STATE (4)

/*
 * MUI
 */

/* TEARDOWN: {inited,idle,playing,paused}->inited */

/* SETUP: {inited}->idle; STOP: {playing,paused}->idle */
# define BoK_MUI_IDLE_STATE (501)

/* START: {inited,idle,paused}->playing; RESUME: {paused}->playing */
# define BoK_MUI_PLAYING_STATE (502)

/* PAUSE: {playing,paused}->paused */
# define BoK_MUI_PAUSED_STATE (503)

#endif /* !__BO_STATEDEFS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
