/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Sync.h	(V.Sokur)
 * include/Bo/Sync.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYNC__
# define __BO_SYNC__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>
# include <Bo/Thrtypes.h>

# include <Bo/sys/Atomic.h>
	/* BoAtomicBitTestAndSet, BoAtomicBitTestAndClear */
# include <Bo/sys/Sched.h>
	/* BoSchedYield */

/******************************************************************************
 * %% BeginSection: CPU-LOCK definitions
 */
# pragma mark -
# pragma mark [ CPU-LOCK definitions ]
# pragma mark -

# ifdef __arm__
#  define __BoCpuLockRMB() BoAtomicReadMemoryBarrier()
#  define __BoCpuLockWMB() BoAtomicWriteMemoryBarrier()
# else
#  define __BoCpuLockRMB() ((void)1)
#  define __BoCpuLockWMB() ((void)1)
# endif
/*
void BoCpulock32(bo_cpulock_t &x, u_int32_t aBit);
 */
# define BoCpulock32(x,aBit) \
	while (0 != BoAtomicBitTestAndSet32(&(x),(aBit))) { \
		BoSchedYield(); \
	} \
	__BoCpuLockRMB()

/*
void BoCpuunlock32(bo_cpulock_t &x, u_int32_t aBit);
 */
# define BoCpuunlock32(x,aBit) \
	__BoCpuLockWMB(); \
	BoAtomicBitTestAndClear32(&(x),(aBit))

/*
_Bool BoCputrylock32(bo_cpulock_t &x, u_int32_t aBit);
 */
# define BoCputrylock32(x,aBit) __inl_BoCputrylock32(&(x), (aBit))
__myc_extern_inline__ int __inl_BoCputrylock32(bo_cpulock_t *x, u_int32_t aBit)
{
	int retval = 0;
	if (0 == BoAtomicBitTestAndSet32(x,aBit)) {
		__BoCpuLockRMB();
		retval = 1;
	}
	return (retval);
}


/* ------------------------------------------------------------------------- */

/*
void BoCpulock64(bo_cpulock_t &x, u_int64_t aBit);
 */
# define BoCpulock64(x,aBit) \
	while (0 != BoAtomicBitTestAndSet64(&(x),(aBit))) { \
		BoSchedYield(); \
	} \
	__BoCpuLockRMB()

/*
void BoCpuunlock64(bo_cpulock_t &x, u_int64_t aBit);
 */
# define BoCpuunlock64(x,aBit) \
	__BoCpuLockWMB(); \
	BoAtomicBitTestAndClear64(&(x),(aBit))

/*
bool BoCputrylock64(bo_cpulock_t &x, u_int64_t aBit);
 */
# define BoCputrylock64(x,aBit) __inl_BoCputrylock64(&(x), (aBit))
# if (BO_M_WORD == BO_M_WORD64)
__myc_extern_inline__ int __inl_BoCputrylock64(bo_cpulock_t *x, u_int64_t aBit)
{
	int retval = 0;
	if (0 == BoAtomicBitTestAndSet64(x,aBit)) {
		__BoCpuLockRMB();
		retval = 1;
	}
	return (retval);
}
# endif

/* ------------------------------------------------------------------------- */

# if (BO_M_WORD == BO_M_WORD32)

#  define BoCpulock BoCpulock32
#  define BoCpuunlock BoCpuunlock32
#  define BoCputrylock BoCputrylock32

# else
# if (BO_M_WORD == BO_M_WORD64)

#  define BoCpulock BoCpulock64
#  define BoCpuunlock BoCpuunlock64
#  define BoCputrylock BoCputrylock64

# else
#  error Unsupported machine architecture!!!
# endif /* (BO_M_WORD == BO_M_WORD64) */
# endif /* (BO_M_WORD == BO_M_WORD32) */

/* ========================================================================= */

/*
void BoCpulock32_1(bo_cpulock_t &x);
 */
# define BoCpulock32_1(x) \
	while (0 == BoAtomicCompareStore32(&(x),0,1)) { \
		BoSchedYield(); \
	} \
	__BoCpuLockRMB()

/*
void BoCpuunlock32_1(bo_cpulock_t &x);
 */
# define BoCpuunlock32_1(x) \
	__BoCpuLockWMB(); \
	BoAtomicFetchClear32(&(x))

/*
_Bool BoCputrylock32_1(bo_cpulock_t &x);
 */
# define BoCputrylock32_1(x) __inl_BoCputrylock32_1(&(x))
__myc_extern_inline__ int __inl_BoCputrylock32_1(bo_cpulock_t *x)
{
	int retval = 0;
	if (0 != BoAtomicCompareStore32(x,0,1)) {
		__BoCpuLockRMB();
		retval = 1;
	}
	return (retval);
}

/*
_Bool BoCpuislocked32_1(bo_cpulock_t &x);
 */
# define BoCpuislocked32_1(x) \
	(0 != BoAtomicLoad32_acq(&(x)))

/* ------------------------------------------------------------------------- */

/*
void BoCpulock64_1(bo_cpulock_t &x);
 */
# define BoCpulock64_1(x) \
	while (0 == BoAtomicCompareStore64(&(x),0,1)) { \
		BoSchedYield(); \
	} \
	__BoCpuLockRMB()

/*
void BoCpuunlock64_1(bo_cpulock_t &x);
 */
# define BoCpuunlock64_1(x) \
	__BoCpuLockWMB(); \
	BoAtomicFetchClear64(&(x))

/*
_Bool BoCputrylock64_1(bo_cpulock_t &x);
 */
# define BoCputrylock64_1(x) __inl_BoCputrylock64_1(&(x))
# if (BO_M_WORD == BO_M_WORD64)
__myc_extern_inline__ int __inl_BoCputrylock64_1(bo_cpulock_t *x)
{
	int retval = 0;
	if (0 != BoAtomicCompareStore64(x,0,1)) {
		__BoCpuLockRMB();
		retval = 1;
	}
	return (retval);
}
# endif

/*
_Bool BoCpuislocked64_1(bo_cpulock_t &x);
 */
# define BoCpuislocked64_1(x) \
	(0 != BoAtomicLoad64_acq(&(x)))

/* ------------------------------------------------------------------------- */

# if (BO_M_WORD == BO_M_WORD32)

#  define BoCpulock_1 BoCpulock32_1
#  define BoCpuunlock_1 BoCpuunlock32_1
#  define BoCputrylock_1 BoCputrylock32_1
#  define BoCpuislocked_1 BoCpuislocked32_1

# else
# if (BO_M_WORD == BO_M_WORD64)

#  define BoCpulock_1 BoCpulock64_1
#  define BoCpuunlock_1 BoCpuunlock64_1
#  define BoCputrylock_1 BoCputrylock64_1
#  define BoCpuislocked_1 BoCpuislocked64_1

# else
#  error Unsupported machine architecture!!!
# endif /* (BO_M_WORD == BO_M_WORD64) */
# endif /* (BO_M_WORD == BO_M_WORD32) */

/* ========================================================================= */

/* compat @{ */
# define bo_llock2 BoCpulock
# define bo_lunlock2 BoCpuunlock
# define bo_ltrylock2 BoCputrylock

# define bo_llock BoCpulock_1
# define bo_lunlock BoCpuunlock_1
# define bo_ltrylock BoCputrylock_1
/* @} */

/******************************************************************************
 * %% BeginSection: CPU-RWLOCK definitions
 */
# pragma mark -
# pragma mark [ CPU-RWLOCK definitions ]
# pragma mark -

/*
void BoCpurwlockInit(bo_cpurwlock_t &aLockRef);
 */
# define BoCpurwlockInit(aLockRef) \
	do { \
		(aLockRef).lrw_mutex = 0; \
		(aLockRef).lrw_rcond = 0; \
		(aLockRef).lrw_wcond = 0; \
		(aLockRef).lrw_nwriters = 0; \
		(aLockRef).lrw_nworkers = 0; \
	} while (0)

/*
void BoCpurwlockFini(bo_cpurwlock_t &aLockRef);
 */
# define BoCpurwlockFini(aLockRef) \
	do { \
		(aLockRef).lrw_mutex = 0; \
		(aLockRef).lrw_rcond = 0; \
		(aLockRef).lrw_wcond = 0; \
		(aLockRef).lrw_nwriters = 0; \
		(aLockRef).lrw_nworkers = 0; \
	} while (0)

/*
void BoCpurlock(bo_cpurwlock_t &aLockRef);
 */
# define BoCpurlock(aLockRef) \
	do { \
		BoCpulock_1((aLockRef).lrw_mutex); \
		while ((aLockRef).lrw_nwriters) { \
			/* cond-wait @{ */ \
			BoCpuunlock_1((aLockRef).lrw_mutex); \
			BoCpulock_1((aLockRef).lrw_rcond); \
			BoCpuunlock_1((aLockRef).lrw_rcond); \
			BoCpulock_1((aLockRef).lrw_mutex); \
			/* @} cond-wait */ \
		} \
		if (1 == ++(aLockRef).lrw_nworkers) \
			BoCpulock_1((aLockRef).lrw_wcond); \
		BoCpuunlock_1((aLockRef).lrw_mutex); \
	} while (0)

/*
_Bool BoCputryrlock(bo_cpurwlock_t &aLockRef);
 */
# if __myc_support(macro_retval)
#  define BoCputryrlock(aLockRef) \
	__extension__ ({ \
		_Bool ltryrdlockRetval; \
		ltryrdlockRetval = false; \
		BoCpulock_1((aLockRef).lrw_mutex); \
		if (!(aLockRef).lrw_nwriters) { \
			if (1 == ++(aLockRef).lrw_nworkers) \
				BoCpulock_1((aLockRef).lrw_wcond); \
			ltryrdlockRetval = true; \
		} \
		BoCpuunlock_1((aLockRef).lrw_mutex); \
		/* return */ (ltryrdlockRetval); \
	})
# else /* !__myc_support(macro_retval) */
#  define BoCputryrlock(aLockRef) _im_BoCputryrlock(&(aLockRef))
__myc_extern_inline__ _Bool
_im_BoCputryrlock(bo_cpurwlock_t *aLock)
{
	_Bool retval;

	retval = false;
	BoCpulock_1(aLock->lrw_mutex);
	if (!aLock->lrw_nwriters) {
		if (1 == ++aLock->lrw_nworkers)
			BoCpulock_1(aLock->lrw_wcond);
		retval = true;
	}
	BoCpuunlock_1(aLock->lrw_mutex);

	return (retval);
}
# endif /* !__myc_support(macro_retval) */

/*
void BoCpuwlock(bo_cpurwlock_t &aLockRef);
 */
# define BoCpuwlock(aLockRef) \
	do { \
		BoCpulock_1((aLockRef).lrw_mutex); \
		if (1 == ++(aLockRef).lrw_nwriters) \
			BoCpulock_1((aLockRef).lrw_rcond); \
		while ((aLockRef).lrw_nworkers) { \
			/* cond-wait @{ */ \
			BoCpuunlock_1((aLockRef).lrw_mutex); \
			BoCpulock_1((aLockRef).lrw_wcond); \
			BoCpuunlock_1((aLockRef).lrw_wcond); \
			BoCpulock_1((aLockRef).lrw_mutex); \
			/* @} cond-wait */ \
		} \
		BoCpulock_1((aLockRef).lrw_wcond); \
		(aLockRef).lrw_nworkers = -1; \
		BoCpuunlock_1((aLockRef).lrw_mutex); \
	} while (0)

/*
_Bool BoCputrywlock(bo_cpurwlock_t &aLockRef);
 */
# if __myc_support(macro_retval)
#  define BoCputrywlock(aLockRef) \
	__extension__ ({ \
		_Bool ltrywrlockRetval; \
		ltrywrlockRetval = false; \
		BoCpulock_1((aLockRef).lrw_mutex); \
		if (!(aLockRef).lrw_nworkers) { \
			if (1 == ++(aLockRef).lrw_nwriters) \
				BoCpulock_1((aLockRef).lrw_rcond); \
			BoCpulock_1((aLockRef).lrw_wcond); \
			(aLockRef).lrw_nworkers = -1; \
		} \
		BoCpuunlock_1((aLockRef).lrw_mutex); \
		/* return */ (ltrywrlockRetval); \
	})
# else /* !__myc_support(macro_retval) */
#  define BoCputrywlock(aLockRef) _im_BoCputrywlock(&(aLockRef))
__myc_extern_inline__ _Bool
_im_BoCputrywlock(bo_cpurwlock_t *aLock)
{
	_Bool retval;

	retval = false;
	BoCpulock_1(aLock->lrw_mutex);
	if (!aLock->lrw_nworkers) {
		if (1 == ++aLock->lrw_nwriters)
			BoCpulock_1(aLock->lrw_rcond);
		BoCpulock_1(aLock->lrw_wcond);
		aLock->lrw_nworkers = -1;
		retval = true;
	}
	BoCpuunlock_1(aLock->lrw_mutex);

	return (retval);
}
# endif /* !__myc_support(macro_retval) */

/*
void BoCpurwunlock(bo_cpurwlock_t &aLockRef);
 */
# define BoCpurwunlock(aLockRef) \
	do { \
		BoCpulock_1((aLockRef).lrw_mutex); \
		if (0 < (aLockRef).lrw_nworkers) { \
			if (0 == --(aLockRef).lrw_nworkers) { \
				BoCpuunlock_1((aLockRef).lrw_wcond); \
			} \
		} \
		else { \
			(aLockRef).lrw_nworkers = 0; \
			BoCpuunlock_1((aLockRef).lrw_wcond); \
			if (0 == --(aLockRef).lrw_nwriters) \
				BoCpuunlock_1((aLockRef).lrw_rcond); \
		} \
		BoCpuunlock_1((aLockRef).lrw_mutex); \
	} while (0)

/* ========================================================================= */

/* compat @{ */
# define bo_lrwlock_init BoCpurwlockInit
# define bo_lrwlock_fini BoCpurwlockFini
# define bo_lrdlock BoCpurlock
# define bo_ltryrdlock BoCputryrlock
# define bo_lwrlock BoCpuwlock
# define bo_ltrywrlock BoCputrywlock
# define bo_lrwunlock BoCpurwunlock
/* @} */

/******************************************************************************
* %% BeginSection: CPU-SEM definitions
*/
# pragma mark -
# pragma mark [ CPU-SEM definitions ]
# pragma mark -

/*
void BoCpusemOpx32(bo_cpusem32_t &x, int32_t v1, int32_t v2);
 * (0 <= v)
 */
# define BoCpusemOpx32(x,v1,v2) \
	while (0 == BoAtomicCompareStore32(&(x),v1,v2)) { \
		BoSchedYield(); \
	}

/*
_Bool BoCpusemTryopx32(bo_cpusem32_t &x, int32_t v1, int32_t v2);
 * (0 <= v)
 */
# define BoCpusemTryopx32(x,v1,v2) \
	(0 != BoAtomicCompareStore32(&(x),v1,v2))

/*
void BoCpusemOpz32(bo_cpusem32_t &x, int32_t v);
 * (0 <= v)
 */
# define BoCpusemOpz32(x,v) BoCpusemOpx32(x,0,v)

/*
_Bool BoCpusemTryopz32(bo_cpusem32_t &x, int32_t v);
 * (0 <= v)
 */
# define BoCpusemTryopz32(x,v) BoCpusemTryopx32(x,0,v)

/*
void BoCpusemOpa32(bo_cpusem32_t &x, int32_t v);
 * (0 <= v)
 */
# define BoCpusemOpa32(x,v) \
	do { \
		volatile int32_t *___dst = (volatile int32_t *)(&(x)); \
		int32_t ___oval, ___nval, ___v = (int32_t)(v); \
		for (;;) { \
			___nval = ((___oval = *___dst) + ___v); \
			if (0 != BoAtomicCompareStore32(___dst,___oval,___nval)) { \
				break; \
			} \
			BoSchedYield(); \
		} \
	} while (0)

/*
_Bool BoCpusemTryopa32(bo_cpusem32_t &x, int32_t v);
 * (0 <= v)
 */
# if __myc_support(macro_retval)

#  define BoCpusemTryopa32(x,v) \
	__extension__ ({ \
		volatile int32_t *___dst = (volatile int32_t *)(&(x)); \
		int32_t ___oval, ___nval, ___v = (int32_t)(v); \
		___nval = ((___oval = *___dst) + ___v); \
		/* return */ (0 != BoAtomicCompareStore32(___dst,___oval,___nval)); \
	})

# else /* !__myc_support(macro_retval) */

#  define BoCpusemTryopa32(x,v) \
	_im_BoCpusemTryopa32((volatile int32_t *)(&(x)),(int32_t)(v))

__myc_extern_inline__ _Bool
_im_BoCpusemTryopa32(volatile u_int8_t *aDst, int32_t aValue)
{
	int32_t oval, nval;
	nval = ((oval = *aDst) + aValue);
	return (0 != BoAtomicCompareStore32(aDst,oval,nval));
}

# endif /* __myc_support(macro_retval) */

/*
void BoCpusemOpm32(bo_cpusem32_t &x, int32_t v);
 * (v < 0)
 */
# define BoCpusemOpm32(x,v) \
	do { \
		volatile int32_t *___dst = (volatile int32_t *)(&(x)); \
		int32_t ___oval, ___nval, ___v = (int32_t)(v); \
		for (;;) { \
			___nval = ((___oval = *___dst) + ___v); \
			if (0 <= ___nval && 0 != BoAtomicCompareStore32(___dst,___oval,___nval)) { \
				break; \
			} \
			BoSchedYield(); \
		} \
	} while (0)

/*
_Bool BoCpusemTryopm32(bo_cpusem32_t &x, int32_t v);
 * (v < 0)
 */
# if __myc_support(macro_retval)

#  define BoCpusemTryopm32(x,v) \
	__extension__ ({ \
		volatile int32_t *___dst = (volatile int32_t *)(&(x)); \
		int32_t ___oval, ___nval, ___v = (int32_t)(v); \
		___nval = ((___oval = *___dst) + ___v); \
		/* return */ (0 <= ___nval && 0 != BoAtomicCompareStore32(___dst,___oval,___nval)); \
	})

# else /* !__myc_support(macro_retval) */

#  define BoCpusemTryopm32(x,v) \
	_im_BoCpusemTryopm32((volatile int32_t *)(&(x)),(int32_t)(v))

__myc_extern_inline__ _Bool
_im_BoCpusemTryopm32(volatile u_int8_t *aDst, int32_t aValue)
{
	int32_t oval, nval;
	nval = ((oval = *aDst) + aValue);
	return (0 <= nval && 0 != BoAtomicCompareStore32(aDst,oval,nval));
}

# endif /* __myc_support(macro_retval) */

/*
u_int32_t BoCpusemValue32(bo_cpusem32_t &x);
 */
# define BoCpusemValue32(x) BoAtomicLoad32_acq(&(x))

/*
u_int32_t BoCpusemSetval32(bo_cpusem32_t &x, u_int32_t v);
 */
# define BoCpusemSetval32(x,v) BoAtomicFetchStore32(&(x),v)

/*
void BoCpusemSet32(bo_cpusem32_t &x, u_int32_t v);
 */
# define BoCpusemSet32(x,v) BoAtomicStore32_rel(&(x),v)

/* ------------------------------------------------------------------------- */

# if (BO_M_WORD == BO_M_WORD64)

/*
void BoCpusemOpx64(bo_cpusem64_t &x, int64_t v1, int64_t v2);
 * (0 <= v)
 */
#  define BoCpusemOpx64(x,v1,v2) \
	while (0 == BoAtomicCompareStore64(&(x),v1,v2)) { \
		BoSchedYield(); \
	}

/*
_Bool BoCpusemTryopx64(bo_cpusem64_t &x, int64_t v1, int64_t v2);
 * (0 <= v)
 */
#  define BoCpusemTryopx64(x,v1,v2) \
	(0 != BoAtomicCompareStore64(&(x),v1,v2))

/*
void BoCpusemOpz64(bo_cpusem64_t &x, int64_t v);
 * (0 <= v)
 */
#  define BoCpusemOpz64(x,v) BoCpusemOpx64(x,0,v)

/*
_Bool BoCpusemTryopz64(bo_cpusem64_t &x, int64_t v);
 * (0 <= v)
 */
#  define BoCpusemTryopz64(x,v) BoCpusemTryopx64(x,0,v)

/*
void BoCpusemOpa64(bo_cpusem64_t &x, int64_t v);
 * (0 <= v)
 */
#  define BoCpusemOpa64(x,v) \
	do { \
		volatile int64_t *___dst = (volatile int64_t *)(&(x)); \
		int64_t ___oval, ___nval, ___v = (int64_t)(v); \
		for (;;) { \
			___nval = ((___oval = *___dst) + ___v); \
			if (0 != BoAtomicCompareStore64(___dst,___oval,___nval)) { \
				break; \
			} \
			BoSchedYield(); \
		} \
	} while (0)

/*
_Bool BoCpusemTryopa64(bo_cpusem64_t &x, int64_t v);
 * (0 <= v)
 */
# if __myc_support(macro_retval)

#  define BoCpusemTryopa64(x,v) \
	__extension__ ({ \
		volatile int64_t *___dst = (volatile int64_t *)(&(x)); \
		int64_t ___oval, ___nval, ___v = (int64_t)(v); \
		___nval = ((___oval = *___dst) + ___v); \
		/* return */ (0 != BoAtomicCompareStore64(___dst,___oval,___nval)); \
	})

# else /* !__myc_support(macro_retval) */

#  define BoCpusemTryopa64(x,v) \
	_im_BoCpusemTryopa64((volatile int64_t *)(&(x)),(int64_t)(v))

__myc_extern_inline__ _Bool
_im_BoCpusemTryopa64(volatile u_int8_t *aDst, int64_t aValue)
{
	int64_t oval, nval;
	nval = ((oval = *aDst) + aValue);
	return (0 != BoAtomicCompareStore64(aDst,oval,nval));
}

# endif /* __myc_support(macro_retval) */

/*
void BoCpusemOpm64(bo_cpusem64_t &x, int64_t v);
 * (v < 0)
 */
#  define BoCpusemOpm64(x,v) \
	do { \
		volatile int64_t *___dst = (volatile int64_t *)(&(x)); \
		int64_t ___oval, ___nval, ___v = (int64_t)(v); \
		for (;;) { \
			___nval = ((___oval = *___dst) + ___v); \
			if (0 <= ___nval && 0 != BoAtomicCompareStore64(___dst,___oval,___nval)) { \
				break; \
			} \
			BoSchedYield(); \
		} \
	} while (0)

/*
_Bool BoCpusemTryopm64(bo_cpusem64_t &x, int64_t v);
 * (v < 0)
 */
# if __myc_support(macro_retval)

#  define BoCpusemTryopm64(x,v) \
	__extension__ ({ \
		volatile int64_t *___dst = (volatile int64_t *)(&(x)); \
		int64_t ___oval, ___nval, ___v = (int64_t)(v); \
		___nval = ((___oval = *___dst) + ___v); \
		/* return */ (0 <= ___nval && 0 != BoAtomicCompareStore64(___dst,___oval,___nval)); \
	})

# else /* !__myc_support(macro_retval) */

#  define BoCpusemTryopm64(x,v) \
	_im_BoCpusemTryopm64((volatile int64_t *)(&(x)),(int64_t)(v))

__myc_extern_inline__ _Bool
_im_BoCpusemTryopm64(volatile u_int8_t *aDst, int64_t aValue)
{
	int64_t oval, nval;
	nval = ((oval = *aDst) + aValue);
	return (0 <= nval && 0 != BoAtomicCompareStore64(aDst,oval,nval));
}

# endif /* __myc_support(macro_retval) */

/*
u_int64_t BoCpusemValue64(bo_cpusem64_t &x);
 */
#  define BoCpusemValue64(x) BoAtomicLoad64_acq(&(x))

/*
u_int64_t BoCpusemSetval64(bo_cpusem64_t &x, u_int64_t v);
 */
#  define BoCpusemSetval64(x,v) BoAtomicFetchStore64(&(x),v)

/*
void BoCpusemSet64(bo_cpusem64_t &x, u_int64_t v);
 */
#  define BoCpusemSet64(x,v) BoAtomicStore64_rel(&(x),v)

# endif /* (BO_M_WORD == BO_M_WORD64) */

/* ------------------------------------------------------------------------- */

# if (BO_M_WORD == BO_M_WORD32)

#  define BoCpusemOpx BoCpusemOpx32
#  define BoCpusemTryopx BoCpusemTryopx32
#  define BoCpusemOpz BoCpusemOpz32
#  define BoCpusemTryopz BoCpusemTryopz32
#  define BoCpusemOpa BoCpusemOpa32
#  define BoCpusemTryopa BoCpusemTryopa32
#  define BoCpusemOpm BoCpusemOpm32
#  define BoCpusemTryopm BoCpusemTryopm32
#  define BoCpusemValue BoCpusemValue32
#  define BoCpusemSetval BoCpusemSetval32
#  define BoCpusemSet BoCpusemSet32

# else
# if (BO_M_WORD == BO_M_WORD64)

#  define BoCpusemOpx BoCpusemOpx64
#  define BoCpusemTryopx BoCpusemTryopx64
#  define BoCpusemOpz BoCpusemOpz64
#  define BoCpusemTryopz BoCpusemTryopz64
#  define BoCpusemOpa BoCpusemOpa64
#  define BoCpusemTryopa BoCpusemTryopa64
#  define BoCpusemOpm BoCpusemOpm64
#  define BoCpusemTryopm BoCpusemTryopm64
#  define BoCpusemValue BoCpusemValue64
#  define BoCpusemSetval BoCpusemSetval64
#  define BoCpusemSet BoCpusemSet64

# else
#  error Unsupported machine architecture!!!
# endif /* (BO_M_WORD == BO_M_WORD64) */
# endif /* (BO_M_WORD == BO_M_WORD32) */

/******************************************************************************
 * %% BeginSection: L-MUTEX definitions
 */
# pragma mark -
# pragma mark [ L-MUTEX definitions ]
# pragma mark -

typedef bo_cpulock_t  bo_lmutex_t, *bo_lmutex_ref;
typedef int           bo_lmutexattr_t, bo_lmutexattr_ref;

# define BO_LMUTEX_REF(x) (&(x))
# define BO_LMUTEXATTR_REF(x) (x)

# define BoLMutexInit(aMutexPtr) (*(aMutexPtr) = 0, BoS_NORMAL)
# define BoLMutexInit2(aMutexPtr,aMxattr) (*(aMutexPtr) = 0, BoS_NORMAL)
# define BoLMutexDestroy(aMutexPtr) (*(aMutexPtr) = 0, BoS_NORMAL)

__myc_extern_inline__ bo_status_t
BoLMutexLock(bo_lmutex_ref aMutex) {
	BoCpulock_1(*aMutex);
	return (BoS_NORMAL);
}

# define BoLMutexTrylock(aMutex) (BoCputrylock_1(*(aMutex)) ? BoS_TRUE : BoS_FALSE)

__myc_extern_inline__ bo_status_t
BoLMutexUnlock(bo_lmutex_ref aMutex) {
	BoCpuunlock_1(*aMutex);
	return (BoS_NORMAL);
}

/******************************************************************************
 * %% BeginSection: MUTEX function prototypes
 */
# pragma mark -
# pragma mark [ MUTEX function prototypes ]
# pragma mark -

# ifdef HAS_PTHREAD_MUTEX
#  define BoMutexInit  BoPMutexInit
#  define BoMutexInit2 BoPMutexInit2
#  define BoMutexDestroy BoPMutexDestroy
#  define BoMutexLock BoPMutexLock
#  define BoMutexTrylock BoPMutexTrylock
#  define BoMutexUnlock BoPMutexUnlock
# else
# ifdef BO_SYSTYPE_WIN32
#  if (0x0600 <= _BO_BUILD_WIN32_WINNT)
#   define BoMutexInit  BoSMutexInit
#   define BoMutexInit2 BoSMutexInit2
#   define BoMutexDestroy BoSMutexDestroy
#   define BoMutexLock BoSMutexLock
#   define BoMutexTrylock BoSMutexTrylock
#   define BoMutexUnlock BoSMutexUnlock
#  else
#   define BoMutexInit  BoPMutexInit
#   define BoMutexInit2 BoPMutexInit2
#   define BoMutexDestroy BoPMutexDestroy
#   define BoMutexLock BoPMutexLock
#   define BoMutexTrylock BoPMutexTrylock
#   define BoMutexUnlock BoPMutexUnlock
#  endif /* (0x0600 <= _BO_BUILD_WIN32_WINNT) */
# endif /* BO_SYSTYPE_WIN32 */
# endif /* HAS_PTHREAD_MUTEX */

/******************************************************************************
 * %% BeginSection: CONDITION function prototypes
 */
# pragma mark -
# pragma mark [ CONDITION function prototypes ]
# pragma mark -

# ifdef HAS_PTHREAD_COND
#  define BoCondInit BoPCondInit
#  define BoCondInit2 BoPCondInit2
#  define BoCondDestroy BoPCondDestroy
#  define BoCondSignal BoPCondSignal
#  define BoCondBroadcast BoPCondBroadcast
#  define BoCondWait BoPCondWait
#  define BoCondTimedwait BoPCondTimedwait
#  define BoCondTimedwait2 BoPCondTimedwait2
#  define BoCondTimedwait_rel BoPCondTimedwait_rel
#  define BoCondTimedwait_ms BoPCondTimedwait_ms
# else
# ifdef BO_SYSTYPE_WIN32
#  if (0x0600 <= _BO_BUILD_WIN32_WINNT)
#   define BoCondInit BoSCondInit
#   define BoCondInit2 BoSCondInit2
#   define BoCondDestroy BoSCondDestroy
#   define BoCondSignal BoSCondSignal
#   define BoCondBroadcast BoSCondBroadcast
#   define BoCondWait BoSCondWait
#   define BoCondTimedwait BoSCondTimedwait
#   define BoCondTimedwait2 BoSCondTimedwait2
#   define BoCondTimedwait_rel BoSCondTimedwait_rel
#   define BoCondTimedwait_ms BoSCondTimedwait_ms
#  else
#   define BoCondInit BoPCondInit
#   define BoCondInit2 BoPCondInit2
#   define BoCondDestroy BoPCondDestroy
#   define BoCondSignal BoPCondSignal
#   define BoCondBroadcast BoPCondBroadcast
#   define BoCondWait BoPCondWait
#   define BoCondTimedwait BoPCondTimedwait
#   define BoCondTimedwait2 BoPCondTimedwait2
#   define BoCondTimedwait_rel BoPCondTimedwait_rel
#   define BoCondTimedwait_ms BoPCondTimedwait_ms
#  endif /* (0x0600 <= _BO_BUILD_WIN32_WINNT) */
# endif /* BO_SYSTYPE_WIN32 */
# endif /* HAS_PTHREAD_COND */

/******************************************************************************
 * %% BeginSection: S-MUTEX function prototypes
 */
# pragma mark -
# pragma mark [ S-MUTEX function prototypes ]
# pragma mark -

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoMemory, BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoSMutexInit(
# if __myc_require(prototypes)
	bo_smutex_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoSMutexInit2(
# if __myc_require(prototypes)
	bo_smutex_t *,
	bo_smutexattr_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_Busy);
	];
))
__attribute__((nonnull(1)))
BoSMutexDestroy(
# if __myc_require(prototypes)
	bo_smutex_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_DeadLock);
	];
))
/*__attribute__((nonnull(1)))*/
BoSMutexLock(
# if __myc_require(prototypes)
	bo_smutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_stsbool_t
__status__((
	BoS_SUCCESS = [
		BoS_TRUE = ();
		BoS_FALSE = (BoK_RacError_Busy);
	];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoSMutexTrylock(
# if __myc_require(prototypes)
	bo_smutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_OperationNotPermitted);
	];
))
/*__attribute__((nonnull(1)))*/
BoSMutexUnlock(
# if __myc_require(prototypes)
	bo_smutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

# ifdef BO_SMUTEX_IS_PMUTEX
#  define BoSMutexInit  BoPMutexInit
#  define BoSMutexInit2 BoPMutexInit2
#  define BoSMutexDestroy BoPMutexDestroy
#  define BoSMutexLock BoPMutexLock
#  define BoSMutexTrylock BoPMutexTrylock
#  define BoSMutexUnlock BoPMutexUnlock
# endif /* BO_SMUTEX_IS_PMUTEX */

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: S-COND function prototypes
 */
# pragma mark -
# pragma mark [ S-COND function prototypes ]
# pragma mark -

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoMemory, BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoSCondInit(
# if __myc_require(prototypes)
	bo_scond_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoSCondInit2(
# if __myc_require(prototypes)
	bo_scond_t *,
	bo_scondattr_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_Busy);
	];
))
__attribute__((nonnull(1)))
BoSCondDestroy(
# if __myc_require(prototypes)
	bo_scond_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoSCondSignal(
# if __myc_require(prototypes)
	bo_scond_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoSCondBroadcast(
# if __myc_require(prototypes)
	bo_scond_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1,2)))*/
BoSCondWait(
# if __myc_require(prototypes)
	bo_scond_ref ,
	bo_smutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoSCondTimedwait(
# if __myc_require(prototypes)
	bo_scond_ref ,
	bo_smutex_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoSCondTimedwait2(
# if __myc_require(prototypes)
	bo_scond_ref ,
	bo_smutex_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct timespec const *aCurAbstime __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoSCondTimedwait_rel(
# if __myc_require(prototypes)
	bo_scond_ref ,
	bo_smutex_ref ,
	struct timespec const *aDuration __myc_defval__(NULL), /* NULL: nowait */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoSCondTimedwait_ms(
# if __myc_require(prototypes)
	bo_scond_ref ,
	bo_smutex_ref ,
	bo_msec_t aDuration, /* 0: nowait, -1: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

# ifdef BO_SCOND_IS_PCOND
#  define BoSCondInit  BoPCondInit
#  define BoSCondInit2 BoPCondInit2
#  define BoSCondDestroy BoPCondDestroy
#  define BoSCondSignal BoPCondSignal
#  define BoSCondBroadcast BoPCondBroadcast
#  define BoSCondWait BoPCondWait
#  define BoSCondTimedwait BoPCondTimedwait
#  define BoSCondTimedwait2 BoPCondTimedwait2
#  define BoSCondTimedwait_rel BoPCondTimedwait_rel
#  define BoSCondTimedwait_ms BoPCondTimedwait_ms
# endif /* BO_SCOND_IS_PCOND */

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: P-MUTEX ATTRIBUTES function prototypes
 */
# pragma mark -
# pragma mark [ P-MUTEX ATTRIBUTES function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_SysError_NoMemory); ];
))
__attribute__((nonnull(1)))
BoPMutexattrInit(
# if __myc_require(prototypes)
	bo_pmutexattr_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
__attribute__((nonnull(1)))
BoPMutexattrDestroy(
# if __myc_require(prototypes)
	bo_pmutexattr_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
__attribute__((nonnull(/*1,*/2)))
BoPMutexattrGettype(
# if __myc_require(prototypes)
	bo_pmutexattr_ref ,
	int *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPMutexattrSettype(
# if __myc_require(prototypes)
	bo_pmutexattr_ref ,
	int ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
__attribute__((nonnull(/*1,*/2)))
BoPMutexattrGetprioceiling(
# if __myc_require(prototypes)
	bo_pmutexattr_ref ,
	int *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPMutexattrSetprioceiling(
# if __myc_require(prototypes)
	bo_pmutexattr_ref ,
	int ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
__attribute__((nonnull(/*1,*/2)))
BoPMutexattrGetprotocol(
# if __myc_require(prototypes)
	bo_pmutexattr_ref ,
	int *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPMutexattrSetprotocol(
# if __myc_require(prototypes)
	bo_pmutexattr_ref ,
	int ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: P-MUTEX function prototypes
 */
# pragma mark -
# pragma mark [ P-MUTEX function prototypes ]
# pragma mark -

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoMemory, BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoPMutexInit(
# if __myc_require(prototypes)
	bo_pmutex_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoPMutexInit2(
# if __myc_require(prototypes)
	bo_pmutex_t *,
	bo_pmutexattr_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_Busy);
	];
))
__attribute__((nonnull(1)))
BoPMutexDestroy(
# if __myc_require(prototypes)
	bo_pmutex_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_DeadLock);
	];
))
/*__attribute__((nonnull(1)))*/
BoPMutexLock(
# if __myc_require(prototypes)
	bo_pmutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_stsbool_t
__status__((
	BoS_SUCCESS = [
		BoS_TRUE = ();
		BoS_FALSE = (BoK_RacError_Busy);
	];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPMutexTrylock(
# if __myc_require(prototypes)
	bo_pmutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_OperationNotPermitted);
	];
))
/*__attribute__((nonnull(1)))*/
BoPMutexUnlock(
# if __myc_require(prototypes)
	bo_pmutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: P-CONDITION ATTRIBUTES function prototypes
 */
# pragma mark -
# pragma mark [ P-CONDITION ATTRIBUTES function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_SysError_NoMemory); ];
))
__attribute__((nonnull(1)))
BoPCondattrInit(
# if __myc_require(prototypes)
	bo_pcondattr_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
__attribute__((nonnull(1)))
BoPCondattrDestroy(
# if __myc_require(prototypes)
	bo_pcondattr_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: P-COND function prototypes
 */
# pragma mark -
# pragma mark [ P-COND function prototypes ]
# pragma mark -

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoMemory, BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoPCondInit(
# if __myc_require(prototypes)
	bo_pcond_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoPCondInit2(
# if __myc_require(prototypes)
	bo_pcond_t *,
	bo_pcondattr_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_Busy);
	];
))
__attribute__((nonnull(1)))
BoPCondDestroy(
# if __myc_require(prototypes)
	bo_pcond_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPCondSignal(
# if __myc_require(prototypes)
	bo_pcond_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPCondBroadcast(
# if __myc_require(prototypes)
	bo_pcond_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1,2)))*/
BoPCondWait(
# if __myc_require(prototypes)
	bo_pcond_ref ,
	bo_pmutex_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoPCondTimedwait(
# if __myc_require(prototypes)
	bo_pcond_ref ,
	bo_pmutex_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoPCondTimedwait2(
# if __myc_require(prototypes)
	bo_pcond_ref ,
	bo_pmutex_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct timespec const *aCurAbstime __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoPCondTimedwait_rel(
# if __myc_require(prototypes)
	bo_pcond_ref ,
	bo_pmutex_ref ,
	struct timespec const *aDuration __myc_defval__(NULL), /* NULL: nowait */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1,2)))*/
BoPCondTimedwait_ms(
# if __myc_require(prototypes)
	bo_pcond_ref ,
	bo_pmutex_ref ,
	bo_msec_t aDuration, /* 0: nowait, -1: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: VAXEVENT function prototypes
 */
# pragma mark -
# pragma mark [ VAXEVENT function prototypes ]
# pragma mark -
/*
 * POSIX-MUTEX-COND semantics
 */

# ifdef HAS_PTHREAD_COND
#  define BoVaxeventInit BoPVaxeventInit
#  define BoVaxeventInit2 BoPVaxeventInit2
#  define BoVaxeventDestroy BoPVaxeventDestroy
#  define BoVaxeventLock BoPVaxeventLock
#  define BoVaxeventTrylock BoPVaxeventTrylock
#  define BoVaxeventUnlock BoPVaxeventUnlock
#  define BoVaxeventSignal BoPVaxeventSignal
#  define BoVaxeventBroadcast BoPVaxeventBroadcast
#  define BoVaxeventWait BoPVaxeventWait
#  define BoVaxeventTimedwait BoPVaxeventTimedwait
#  define BoVaxeventTimedwait2 BoPVaxeventTimedwait2
#  define BoVaxeventTimedwait_rel BoPVaxeventTimedwait_rel
#  define BoVaxeventTimedwait_ms BoPVaxeventTimedwait_ms
# else
# ifdef BO_SYSTYPE_WIN32
#  if (0x0600 <= _BO_BUILD_WIN32_WINNT)
#   define BoVaxeventInit BoSVaxeventInit
#   define BoVaxeventInit2 BoSVaxeventInit2
#   define BoVaxeventDestroy BoSVaxeventDestroy
#   define BoVaxeventLock BoSVaxeventLock
#   define BoVaxeventTrylock BoSVaxeventTrylock
#   define BoVaxeventUnlock BoSVaxeventUnlock
#   define BoVaxeventSignal BoSVaxeventSignal
#   define BoVaxeventBroadcast BoSVaxeventBroadcast
#   define BoVaxeventWait BoSVaxeventWait
#   define BoVaxeventTimedwait BoSVaxeventTimedwait
#   define BoVaxeventTimedwait2 BoSVaxeventTimedwait2
#   define BoVaxeventTimedwait_rel BoSVaxeventTimedwait_rel
#   define BoVaxeventTimedwait_ms BoSVaxeventTimedwait_ms
#  else
#   define BoVaxeventInit BoPVaxeventInit
#   define BoVaxeventInit2 BoPVaxeventInit2
#   define BoVaxeventDestroy BoPVaxeventDestroy
#   define BoVaxeventLock BoPVaxeventLock
#   define BoVaxeventTrylock BoPVaxeventTrylock
#   define BoVaxeventUnlock BoPVaxeventUnlock
#   define BoVaxeventSignal BoPVaxeventSignal
#   define BoVaxeventBroadcast BoPVaxeventBroadcast
#   define BoVaxeventWait BoPVaxeventWait
#   define BoVaxeventTimedwait BoPVaxeventTimedwait
#   define BoVaxeventTimedwait2 BoPVaxeventTimedwait2
#   define BoVaxeventTimedwait_rel BoPVaxeventTimedwait_rel
#   define BoVaxeventTimedwait_ms BoPVaxeventTimedwait_ms
#  endif /* (0x0600 <= _BO_BUILD_WIN32_WINNT) */
# endif /* BO_SYSTYPE_WIN32 */
# endif /* HAS_PTHREAD_COND */

/******************************************************************************
 * %% BeginSection: S-VAXEVENT function prototypes
 */
# pragma mark -
# pragma mark [ S-VAXEVENT function prototypes ]
# pragma mark -
/*
 * POSIX-MUTEX-COND semantics
 */

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoMemory, BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoSVaxeventInit(
# if __myc_require(prototypes)
	bo_svaxevent_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoSVaxeventInit2(
# if __myc_require(prototypes)
	bo_svaxevent_t *,
	bo_smutexattr_ref ,
	bo_scondattr_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_Busy);
	];
))
__attribute__((nonnull(1)))
BoSVaxeventDestroy(
# if __myc_require(prototypes)
	bo_svaxevent_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_DeadLock);
	];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventLock(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventLock(anEvent,anErrPtr) \
	BoSMutexLock(BO_SVAXEVENT_REF_Mutexref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_stsbool_t
__status__((
	BoS_SUCCESS = [
		BoS_TRUE = ();
		BoS_FALSE = (BoK_RacError_Busy);
	];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventTrylock(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventTrylock(anEvent,anErrPtr) \
	BoSMutexTrylock(BO_SVAXEVENT_REF_Mutexref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_OperationNotPermitted);
	];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventUnlock(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventUnlock(anEvent,anErrPtr) \
	BoSMutexUnlock(BO_SVAXEVENT_REF_Mutexref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventSignal(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventSignal(anEvent,anErrPtr) \
	BoSCondSignal(BO_SVAXEVENT_REF_Condref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventBroadcast(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventBroadcast(anEvent,anErrPtr) \
	BoSCondBroadcast(BO_SVAXEVENT_REF_Condref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventWait(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventWait(anEvent,anErrPtr) \
	BoSCondWait( \
		BO_SVAXEVENT_REF_Condref(anEvent), \
		BO_SVAXEVENT_REF_Mutexref(anEvent), \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventTimedwait(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventTimedwait(anEvent,anAbstime,anErrPtr) \
	BoSCondTimedwait( \
		BO_SVAXEVENT_REF_Condref(anEvent), \
		BO_SVAXEVENT_REF_Mutexref(anEvent), \
		anAbstime, \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventTimedwait2(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct timespec const *aCurAbstime __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventTimedwait2(anEvent,anAbstime,aCurAbstime,anErrPtr) \
	BoSCondTimedwait2( \
		BO_SVAXEVENT_REF_Condref(anEvent), \
		BO_SVAXEVENT_REF_Mutexref(anEvent), \
		anAbstime,aCurAbstime, \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventTimedwait_rel(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	struct timespec const *aDuration __myc_defval__(NULL), /* NULL: nowait */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventTimedwait_rel(anEvent,aDuration,anErrPtr) \
	BoSCondTimedwait_rel( \
		BO_SVAXEVENT_REF_Condref(anEvent), \
		BO_SVAXEVENT_REF_Mutexref(anEvent), \
		aDuration, \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoSVaxeventTimedwait_ms(
# if __myc_require(prototypes)
	bo_svaxevent_ref ,
	bo_msec_t aDuration, /* 0: nowait, -1: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoSVaxeventTimedwait_ms(anEvent,aDuration,anErrPtr) \
	BoSCondTimedwait_ms( \
		BO_SVAXEVENT_REF_Condref(anEvent), \
		BO_SVAXEVENT_REF_Mutexref(anEvent), \
		aDuration, \
		anErrPtr \
	)

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: P-VAXEVENT function prototypes
 */
# pragma mark -
# pragma mark [ P-VAXEVENT function prototypes ]
# pragma mark -
/*
 * POSIX-MUTEX-COND semantics
 */

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoMemory, BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoPVaxeventInit(
# if __myc_require(prototypes)
	bo_pvaxevent_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1)))
BoPVaxeventInit2(
# if __myc_require(prototypes)
	bo_pvaxevent_t *,
	bo_pmutexattr_ref ,
	bo_pcondattr_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_Busy);
	];
))
__attribute__((nonnull(1)))
BoPVaxeventDestroy(
# if __myc_require(prototypes)
	bo_pvaxevent_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_DeadLock);
	];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventLock(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventLock(anEvent,anErrPtr) \
	BoPMutexLock(BO_PVAXEVENT_REF_Mutexref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_stsbool_t
__status__((
	BoS_SUCCESS = [
		BoS_TRUE = ();
		BoS_FALSE = (BoK_RacError_Busy);
	];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventTrylock(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventTrylock(anEvent,anErrPtr) \
	BoPMutexTrylock(BO_PVAXEVENT_REF_Mutexref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_OperationNotPermitted);
	];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventUnlock(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventUnlock(anEvent,anErrPtr) \
	BoPMutexUnlock(BO_PVAXEVENT_REF_Mutexref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventSignal(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventSignal(anEvent,anErrPtr) \
	BoPCondSignal(BO_PVAXEVENT_REF_Condref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventBroadcast(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventBroadcast(anEvent,anErrPtr) \
	BoPCondBroadcast(BO_PVAXEVENT_REF_Condref(anEvent),anErrPtr)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventWait(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventWait(anEvent,anErrPtr) \
	BoPCondWait( \
		BO_PVAXEVENT_REF_Condref(anEvent), \
		BO_PVAXEVENT_REF_Mutexref(anEvent), \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventTimedwait(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventTimedwait(anEvent,anAbstime,anErrPtr) \
	BoPCondTimedwait( \
		BO_PVAXEVENT_REF_Condref(anEvent), \
		BO_PVAXEVENT_REF_Mutexref(anEvent), \
		anAbstime, \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventTimedwait2(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct timespec const *anAbstime __myc_defval__(NULL), /* NULL: infinite */
	struct timespec const *aCurAbstime __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventTimedwait2(anEvent,anAbstime,aCurAbstime,anErrPtr) \
	BoPCondTimedwait2( \
		BO_PVAXEVENT_REF_Condref(anEvent), \
		BO_PVAXEVENT_REF_Mutexref(anEvent), \
		anAbstime,aCurAbstime, \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventTimedwait_rel(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	struct timespec const *aDuration __myc_defval__(NULL), /* NULL: nowait */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventTimedwait_rel(anEvent,aDuration,anErrPtr) \
	BoPCondTimedwait_rel( \
		BO_PVAXEVENT_REF_Condref(anEvent), \
		BO_PVAXEVENT_REF_Mutexref(anEvent), \
		aDuration, \
		anErrPtr \
	)

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
/*__attribute__((nonnull(1)))*/
BoPVaxeventTimedwait_ms(
# if __myc_require(prototypes)
	bo_pvaxevent_ref ,
	bo_msec_t aDuration, /* 0: nowait, -1: infinite */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoPVaxeventTimedwait_ms(anEvent,aDuration,anErrPtr) \
	BoPCondTimedwait_ms( \
		BO_PVAXEVENT_REF_Condref(anEvent), \
		BO_PVAXEVENT_REF_Mutexref(anEvent), \
		aDuration, \
		anErrPtr \
	)

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: L-SEMAPHORE ATTRIBUTES function prototypes
 */
# pragma mark -
# pragma mark [ L-SEMAPHORE ATTRIBUTES function prototypes ]
# pragma mark -

/* @todo: implement @{ */
#ifdef BO_SYSTYPE_WIN32

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemattrInit(
# if __myc_require(prototypes)
	bo_lsemattr_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemattrDestroy(
# if __myc_require(prototypes)
	bo_lsemattr_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemattrGetshared(
# if __myc_require(prototypes)
	bo_lsemattr_ref ,
	LPCTSTR *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemattrSetshared(
# if __myc_require(prototypes)
	bo_lsemattr_ref ,
	LPCTSTR ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemattrGetinitvalue(
# if __myc_require(prototypes)
	bo_lsemattr_ref ,
	unsigned long *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemattrSetinitvalue(
# if __myc_require(prototypes)
	bo_lsemattr_ref ,
	unsigned long ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

# endif /* BO_SYSTYPE_WIN32 */
/* @} */

/******************************************************************************
 * %% BeginSection: LIGHT-SEMAPHORE function prototypes
 */
# pragma mark -
# pragma mark [ LIGHT-SEMAPHORE function prototypes ]
# pragma mark -

/* @todo: implement @{ */
#ifdef BO_SYSTYPE_WIN32

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoLSemOpen2(
# if __myc_require(prototypes)
	bo_lsem_t *,
	bo_lsemattr_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemClose(
# if __myc_require(prototypes)
	bo_lsem_t *,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemOp(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	unsigned anOpc,
	int anOpv[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemTimedop(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	unsigned anOpc,
	int anOpv[],
	struct timespec const *anAbstime __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemTimedop_ms(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	unsigned anOpc,
	int anOpv[],
	bo_msec_t aDuration,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemTryop(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	unsigned anOpc,
	int anOpv[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) ssize_t
__attribute__((nonnull(1)))
BoLSemValue(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemSet(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	u_int32_t aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemTimedset(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	u_int32_t aValue,
	struct timespec const *anAbstime __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public, libBo) bo_status_t
__attribute__((nonnull(1)))
BoLSemTimedset_ms(
# if __myc_require(prototypes)
	bo_lsem_ref ,
	u_int32_t aValue,
	bo_msec_t aDuration,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

# endif /* BO_SYSTYPE_WIN32 */
/* @} */

#endif /* !__BO_SYNC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
