/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Thread.h	(V.Sokur)
 * include/Bo/Thread.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_THREAD__
# define __BO_THREAD__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>
# include <Bo/Thrtypes.h>

/******************************************************************************
 * %% BeginSection: P-THREAD ATTRIBUTES function prototypes
 */
# pragma mark -
# pragma mark [ P-THREAD ATTRIBUTES function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_SysError_NoMemory); ];
))
__attribute__((nonnull(1)))
BoPThrattrInit(
# if __myc_require(prototypes)
	bo_pthrattr_t *anAttribPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
))
__attribute__((nonnull(1)))
BoPThrattrDestroy(
# if __myc_require(prototypes)
	bo_pthrattr_t *anAttribPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
__attribute__((nonnull(/*1,*/2)))
BoPThrattrGetstacksize(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	size_t *aStackSizePtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrSetstacksize(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	size_t aStackSize,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
__attribute__((nonnull(/*1,*/2)))
BoPThrattrGetstackaddr(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	bo_pointer_t *aStackAddrPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrSetstackaddr(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	bo_pointer_t aStackAddr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_TRUE = ();
	BoS_FALSE = ();
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrIsjoinable(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrEnablejoin(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrDisablejoin(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_TRUE = ();
	BoS_FALSE = ();
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrIscancelable(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrEnablecancel(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrDisablecancel(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
__attribute__((nonnull(/*1,*/2)))
BoPThrattrGetschedpolicy(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	int *aPolicyPtr, /* { fifo, rr, other } */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp, BoK_OpError_FeatureNotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrSetschedpolicy(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	int aPolicy, /* { fifo, rr, other } */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
__attribute__((nonnull(/*1,*/2)))
BoPThrattrGetpriority(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	int *aPriorityPtr, /* [min, max] */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp, BoK_OpError_FeatureNotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrSetpriority(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	int aPriority, /* [min, max] */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_TRUE = ();
	BoS_FALSE = ();
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrIsinheritsched(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrInheritsched(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThrattrExplicitsched(
# if __myc_require(prototypes)
	bo_pthrattr_ref anAttrib,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: P-THREAD function prototypes
 */
# pragma mark -
# pragma mark [ P-THREAD function prototypes ]
# pragma mark -

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoMemory, BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1,2)))
BoPThreadCreate(
# if __myc_require(prototypes)
	bo_pthread_t *aThreadPtr,
	bo_pthread_proc aStartRoutine,
	bo_pointer_t Args,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
	];
))
__attribute__((nonnull(1,3)))
BoPThreadCreate2(
# if __myc_require(prototypes)
	bo_pthread_t *aThreadPtr,
	bo_pthrattr_ref anAttrib,
	bo_pthread_proc aStartRoutine,
	bo_pointer_t Args,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_SysError_NoMemory,
		           BoK_RacError_WouldBlock);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
__attribute__((nonnull(1,3)))
BoPThreadCreate2_suspended(
# if __myc_require(prototypes)
	bo_pthread_t *aThreadPtr,
	bo_pthrattr_ref anAttrib,
	bo_pthread_proc aStartRoutine,
	bo_pointer_t Args,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThreadSuspend(
# if __myc_require(prototypes)
	bo_pthread_ref aThread,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThreadResume(
# if __myc_require(prototypes)
	bo_pthread_ref aThread,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

/*
void
BoPThreadExit(bo_pthread_retval_t );
 */
# define BoPThreadExit _BoPThreadExit

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess, BoK_OpError_UnwillingToPerform,
		           BoK_RacError_DeadLock);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThreadJoin(
# if __myc_require(prototypes)
	bo_pthread_ref aThread,
	bo_pthread_retval_t *aRetvalPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
__attribute__((nonnull(/*1,*/2)))
BoPThreadGetschedpolicy(
#if __myc_require(prototypes)
	bo_pthread_ref aThread,
	int *aPolicyPtr, /* { fifo, rr, other } */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp, BoK_OpError_FeatureNotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThreadSetschedpolicy(
#if __myc_require(prototypes)
	bo_pthread_ref aThread,
	int aPolicy, /* { fifo, rr, other } */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
__attribute__((nonnull(/*1,*/2)))
BoPThreadGetpriority(
#if __myc_require(prototypes)
	bo_pthread_ref aThread,
	int *aPriorityPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp, BoK_OpError_FeatureNotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThreadSetpriority(
#if __myc_require(prototypes)
	bo_pthread_ref aThread,
	int aPriority,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_NOTIMPL = (BoK_OpError_NotImpl); ];
))
__attribute__((nonnull(/*1,*/2)))
BoPThreadGetcanceltype(
#if __myc_require(prototypes)
	int *aTypePtr, /* { deferred, asynchronous } */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThreadSetcanceltype(
#if __myc_require(prototypes)
	int aType, /* { deferred, asynchronous } */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_TRUE = ();
	BoS_FALSE = ();
	BoS_FAILURE = [ BoS_NOTIMPL = (BoK_OpError_NotImpl); ];
))
/*__attribute__((nonnull(1)))*/
BoPThreadIscancelable(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_NOTIMPL = (BoK_OpError_NotImpl); ];
))
/*__attribute__((nonnull(1)))*/
BoPThreadEnablecancel(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_NOTIMPL = (BoK_OpError_NotImpl); ];
))
/*__attribute__((nonnull(1)))*/
BoPThreadDisablecancel(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_SysError_NoSuchProcess, BoK_OpError_UnwillingToPerform);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
/*__attribute__((nonnull(1)))*/
BoPThreadDetach(
# if __myc_require(prototypes)
	bo_pthread_ref aThread,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_OpError_UnwillingToPerform);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
BoPThreadDetachself(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_THREAD__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
