/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Thrtypes.h	(V.Sokur)
 * include/Bo/Thrtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_THRTYPES__
# define __BO_THRTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Cdefs.h>

# include _BO_PLATFORM_HEADER(_Thrtypes.h)

/******************************************************************************
 * %% BeginSection: constant declarations
 */

/*
 * P-Thread attributes
 */

# define BoK_PTHREAD_STACK_MIN             0x100000

/* priority */
# define BoK_PTHREAD_PRIORITY_MIN         -4
# define BoK_PTHREAD_PRIORITY_LOW         -2
# define BoK_PTHREAD_PRIORITY_NORMAL       0
# define BoK_PTHREAD_PRIORITY_HIGH         2
# define BoK_PTHREAD_PRIORITY_MAX          4

/* joinable state */
# define BoK_PTHREAD_JOIN_ENABLE           1 /* joinable thread */
# define BoK_PTHREAD_JOIN_DISABLE          0 /* detached thread */

/* cancel type */
# ifdef PTHREAD_CANCEL_ASYNCHRONOUS
#  define BoK_PTHREAD_CANCEL_DEFERRED      PTHREAD_CANCEL_DEFERRED
#  define BoK_PTHREAD_CANCEL_ASYNCHRONOUS  PTHREAD_CANCEL_ASYNCHRONOUS
# else
#  define BoK_PTHREAD_CANCEL_DEFERRED      1
#  define BoK_PTHREAD_CANCEL_ASYNCHRONOUS  0
# endif

/* cancel state */
# ifdef PTHREAD_CANCEL_ENABLE
#  define BoK_PTHREAD_CANCEL_ENABLE        PTHREAD_CANCEL_ENABLE
#  define BoK_PTHREAD_CANCEL_DISABLE       PTHREAD_CANCEL_DISABLE
# else
#  define BoK_PTHREAD_CANCEL_ENABLE        1
#  define BoK_PTHREAD_CANCEL_DISABLE       0
# endif

/* sched policy */
# ifdef SCHED_FIFO
#  define BoK_PTHREAD_POLICY_FIFO          SCHED_FIFO
#  define BoK_PTHREAD_POLICY_RR            SCHED_RR
#  define BoK_PTHREAD_POLICY_OTHER         SCHED_OTHER
# else
#  define BoK_PTHREAD_POLICY_FIFO          1
#  define BoK_PTHREAD_POLICY_RR            2
#  define BoK_PTHREAD_POLICY_OTHER         0
# endif

# ifdef PTHREAD_INHERIT_SCHED
#  define BoK_PTHREAD_INHERIT_ENABLE       PTHREAD_INHERIT_SCHED
#  define BoK_PTHREAD_INHERIT_DISABLE      PTHREAD_EXPLICIT_SCHED
# else
#  define BoK_PTHREAD_INHERIT_ENABLE       1
#  define BoK_PTHREAD_INHERIT_DISABLE      0
# endif

/*
 * P-Mutex attributes
 */

/* mutex protocol */
# ifdef PTHREAD_PRIO_INHERIT
#  define BoK_PMUTEXATTR_PROTOCOL_NONE     PTHREAD_PRIO_NONE
#  define BoK_PMUTEXATTR_PROTOCOL_INHERIT  PTHREAD_PRIO_INHERIT
#  define BoK_PMUTEXATTR_PROTOCOL_PROTECT  PTHREAD_PRIO_PROTECT
# else
#  define BoK_PMUTEXATTR_PROTOCOL_NONE     0
#  define BoK_PMUTEXATTR_PROTOCOL_INHERIT  1
#  define BoK_PMUTEXATTR_PROTOCOL_PROTECT  2
# endif

/* mutex type */
# ifdef PTHREAD_MUTEX_ERROR_CHECK
#  define BoK_PMUTEXATTR_TYPE_NORMAL       PTHREAD_MUTEX_NORMAL
#  define BoK_PMUTEXATTR_TYPE_RECURSIVE    PTHREAD_MUTEX_RECURSIVE
#  define BoK_PMUTEXATTR_TYPE_ERROR_CHECK  PTHREAD_MUTEX_ERROR_CHECK
#  define BoK_PMUTEXATTR_TYPE_DEFAULT      PTHREAD_MUTEX_DEFAULT
# else
#  define BoK_PMUTEXATTR_TYPE_NORMAL       3
#  define BoK_PMUTEXATTR_TYPE_RECURSIVE    2
#  define BoK_PMUTEXATTR_TYPE_ERROR_CHECK  1
#  define BoK_PMUTEXATTR_TYPE_DEFAULT      BoK_PMUTEXATTR_TYPE_ERROR_CHECK /* posix */
# endif

/******************************************************************************
 * %% BeginSection: VAXEVENT definitions
 */
# pragma mark -
# pragma mark [ VAXEVENT definitions ]
# pragma mark -

struct bo_svaxevent {
	bo_smutex_t       vxe_mx;
	bo_scond_t        vxe_cond;
	volatile int32_t  vxe_event;
	union {
		volatile int32_t    vxe_val32;
		volatile int32_t    vxe_val32v[2];
		volatile u_int64_t  vxe_val64;
	} u;
	bo_pointer_t      vxe_sender;
	bo_pointer_t      vxe_uctx;
};
typedef struct bo_svaxevent  bo_svaxevent_t, *bo_svaxevent_ref;

# define BO_SVAXEVENT_REF(x) (&(x))
# define BO_SVAXEVENT_REF_INVALID NULL

# define BO_SVAXEVENT_REF_Mutex(x) ((x)->vxe_mx)
# define BO_SVAXEVENT_REF_Mutexref(x) BO_SMUTEX_REF(BO_SVAXEVENT_REF_Mutex(x))
# define BO_SVAXEVENT_REF_Cond(x) ((x)->vxe_cond)
# define BO_SVAXEVENT_REF_Condref(x) BO_SCOND_REF(BO_SVAXEVENT_REF_Cond(x))
# define BO_SVAXEVENT_REF_Event(x) ((x)->vxe_event)
# define BO_SVAXEVENT_REF_Val32(x) ((x)->u.vxe_val32)
# define BO_SVAXEVENT_REF_Val64(x) ((x)->u.vxe_val64)
# define BO_SVAXEVENT_REF_Val32v0(x) ((x)->u.vxe_val32v[0])
# define BO_SVAXEVENT_REF_Val32v1(x) ((x)->u.vxe_val32v[1])
# define BO_SVAXEVENT_REF_Val32v(x,i) ((x)->u.vxe_val32v[i])
# define BO_SVAXEVENT_REF_Sender(x) ((x)->vxe_sender)
# define BO_SVAXEVENT_REF_Context(x) ((x)->vxe_uctx)

# define BO_SVAXEVENT_REF_ReadEvent(x) ((int32_t) BoAtomicLoad32_acq(&(x)->vxe_event))
# define BO_SVAXEVENT_REF_WriteEvent(x,ev) BoAtomicStore32_rel(&(x)->vxe_event,ev)
# define BO_SVAXEVENT_REF_ReadVal32(x) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32))
# define BO_SVAXEVENT_REF_WriteVal32(x,v) BoAtomicStore32_rel(&(x)->vxe_val32,v)
# define BO_SVAXEVENT_REF_ReadVal64(x) BoAtomicLoad64_acq(&(x)->u.vxe_val64)
# define BO_SVAXEVENT_REF_WriteVal64(x,v) BoAtomicStore64_rel(&(x)->u.vxe_val64,v)
# define BO_SVAXEVENT_REF_ReadVal32v0(x) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32v[0]))
# define BO_SVAXEVENT_REF_WriteVal32v0(x,v) BoAtomicStore32_rel(&(x)->u.vxe_val32v[0],v)
# define BO_SVAXEVENT_REF_ReadVal32v1(x) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32v[1]))
# define BO_SVAXEVENT_REF_WriteVal32v1(x,v) BoAtomicStore32_rel(&(x)->u.vxe_val32v[1],v)
# define BO_SVAXEVENT_REF_ReadVal32v(x,i) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32v[i]))
# define BO_SVAXEVENT_REF_WriteVal32v(x,i,v) BoAtomicStore32_rel(&(x)->u.vxe_val32v[i],v)

# pragma mark -

struct bo_pvaxevent {
	bo_pmutex_t       vxe_mx;
	bo_pcond_t        vxe_cond;
	volatile int32_t  vxe_event;
	union {
		volatile int32_t    vxe_val32;
		volatile int32_t    vxe_val32v[2];
		volatile u_int64_t  vxe_val64;
	} u;
	bo_pointer_t      vxe_sender;
	bo_pointer_t      vxe_uctx;
};
typedef struct bo_pvaxevent  bo_pvaxevent_t, *bo_pvaxevent_ref;

# define BO_PVAXEVENT_REF(x) (&(x))
# define BO_PVAXEVENT_REF_INVALID NULL

# define BO_PVAXEVENT_REF_Mutex(x) ((x)->vxe_mx)
# define BO_PVAXEVENT_REF_Mutexref(x) BO_PMUTEX_REF(BO_PVAXEVENT_REF_Mutex(x))
# define BO_PVAXEVENT_REF_Cond(x) ((x)->vxe_cond)
# define BO_PVAXEVENT_REF_Condref(x) BO_PCOND_REF(BO_PVAXEVENT_REF_Cond(x))
# define BO_PVAXEVENT_REF_Event(x) ((x)->vxe_event)
# define BO_PVAXEVENT_REF_Val32(x) ((x)->u.vxe_val32)
# define BO_PVAXEVENT_REF_Val64(x) ((x)->u.vxe_val64)
# define BO_PVAXEVENT_REF_Val32v0(x) ((x)->u.vxe_val32v[0])
# define BO_PVAXEVENT_REF_Val32v1(x) ((x)->u.vxe_val32v[1])
# define BO_PVAXEVENT_REF_Val32v(x,i) ((x)->u.vxe_val32v[i])
# define BO_PVAXEVENT_REF_Sender(x) ((x)->vxe_sender)
# define BO_PVAXEVENT_REF_Context(x) ((x)->vxe_uctx)

# define BO_PVAXEVENT_REF_ReadEvent(x) ((int32_t) BoAtomicLoad32_acq(&(x)->vxe_event))
# define BO_PVAXEVENT_REF_WriteEvent(x,ev) BoAtomicStore32_rel(&(x)->vxe_event,ev)
# define BO_PVAXEVENT_REF_ReadVal32(x) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32))
# define BO_PVAXEVENT_REF_WriteVal32(x,v) BoAtomicStore32_rel(&(x)->vxe_val32,v)
# define BO_PVAXEVENT_REF_ReadVal64(x) BoAtomicLoad64_acq(&(x)->u.vxe_val64)
# define BO_PVAXEVENT_REF_WriteVal64(x,v) BoAtomicStore64_rel(&(x)->u.vxe_val64,v)
# define BO_PVAXEVENT_REF_ReadVal32v0(x) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32v[0]))
# define BO_PVAXEVENT_REF_WriteVal32v0(x,v) BoAtomicStore32_rel(&(x)->u.vxe_val32v[0],v)
# define BO_PVAXEVENT_REF_ReadVal32v1(x) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32v[1]))
# define BO_PVAXEVENT_REF_WriteVal32v1(x,v) BoAtomicStore32_rel(&(x)->u.vxe_val32v[1],v)
# define BO_PVAXEVENT_REF_ReadVal32v(x,i) ((int32_t) BoAtomicLoad32_acq(&(x)->u.vxe_val32v[i]))
# define BO_PVAXEVENT_REF_WriteVal32v(x,i,v) BoAtomicStore32_rel(&(x)->u.vxe_val32v[i],v)

# pragma mark -

# ifdef HAS_PTHREAD_COND
#  define bo_vaxevent_t bo_pvaxevent_t
#  define bo_vaxevent_ref bo_pvaxevent_ref
#  define BO_VAXEVENT_REF BO_PVAXEVENT_REF
#  define BO_VAXEVENT_REF_INVALID BO_PVAXEVENT_REF_INVALID
#  define BO_VAXEVENT_REF_Mutex BO_PVAXEVENT_REF_Mutex
#  define BO_VAXEVENT_REF_Mutexref BO_PVAXEVENT_REF_Mutexref
#  define BO_VAXEVENT_REF_Cond BO_PVAXEVENT_REF_Cond
#  define BO_VAXEVENT_REF_Condref BO_PVAXEVENT_REF_Condref
#  define BO_VAXEVENT_REF_Event BO_PVAXEVENT_REF_Event
#  define BO_VAXEVENT_REF_Val32 BO_PVAXEVENT_REF_Val32
#  define BO_VAXEVENT_REF_Val64 BO_PVAXEVENT_REF_Val64
#  define BO_VAXEVENT_REF_Val32v0 BO_PVAXEVENT_REF_Val32v0
#  define BO_VAXEVENT_REF_Val32v1 BO_PVAXEVENT_REF_Val32v1
#  define BO_VAXEVENT_REF_Val32v BO_PVAXEVENT_REF_Val32v
#  define BO_VAXEVENT_REF_Sender BO_PVAXEVENT_REF_Sender
#  define BO_VAXEVENT_REF_Context BO_PVAXEVENT_REF_Context

#  define BO_VAXEVENT_REF_ReadEvent BO_PVAXEVENT_REF_ReadEvent
#  define BO_VAXEVENT_REF_WriteEvent BO_PVAXEVENT_REF_WriteEvent
#  define BO_VAXEVENT_REF_ReadVal32 BO_PVAXEVENT_REF_ReadVal32
#  define BO_VAXEVENT_REF_WriteVal32 BO_PVAXEVENT_REF_WriteVal32
#  define BO_VAXEVENT_REF_ReadVal64 BO_PVAXEVENT_REF_ReadVal64
#  define BO_VAXEVENT_REF_WriteVal64 BO_PVAXEVENT_REF_WriteVal64
#  define BO_VAXEVENT_REF_ReadVal32v0 BO_PVAXEVENT_REF_ReadVal32v0
#  define BO_VAXEVENT_REF_WriteVal32v0 BO_PVAXEVENT_REF_WriteVal32v0
#  define BO_VAXEVENT_REF_ReadVal32v1 BO_PVAXEVENT_REF_ReadVal32v1
#  define BO_VAXEVENT_REF_WriteVal32v1 BO_PVAXEVENT_REF_WriteVal32v1
#  define BO_VAXEVENT_REF_ReadVal32v BO_PVAXEVENT_REF_ReadVal32v
#  define BO_VAXEVENT_REF_WriteVal32v BO_PVAXEVENT_REF_WriteVal32v
# else
# ifdef BO_SYSTYPE_WIN32
#  if (0x0600 <= _BO_BUILD_WIN32_WINNT)
#   define bo_vaxevent_t bo_svaxevent_t
#   define bo_vaxevent_ref bo_svaxevent_ref
#   define BO_VAXEVENT_REF BO_SVAXEVENT_REF
#   define BO_VAXEVENT_REF_INVALID BO_SVAXEVENT_REF_INVALID
#   define BO_VAXEVENT_REF_Mutex BO_SVAXEVENT_REF_Mutex
#   define BO_VAXEVENT_REF_Mutexref BO_SVAXEVENT_REF_Mutexref
#   define BO_VAXEVENT_REF_Cond BO_SVAXEVENT_REF_Cond
#   define BO_VAXEVENT_REF_Condref BO_SVAXEVENT_REF_Condref
#   define BO_VAXEVENT_REF_Event BO_SVAXEVENT_REF_Event
#   define BO_VAXEVENT_REF_Val32 BO_SVAXEVENT_REF_Val32
#   define BO_VAXEVENT_REF_Val64 BO_SVAXEVENT_REF_Val64
#   define BO_VAXEVENT_REF_Val32v0 BO_SVAXEVENT_REF_Val32v0
#   define BO_VAXEVENT_REF_Val32v1 BO_SVAXEVENT_REF_Val32v1
#   define BO_VAXEVENT_REF_Val32v BO_SVAXEVENT_REF_Val32v
#   define BO_VAXEVENT_REF_Sender BO_SVAXEVENT_REF_Sender
#   define BO_VAXEVENT_REF_Context BO_SVAXEVENT_REF_Context

#   define BO_VAXEVENT_REF_ReadEvent BO_SVAXEVENT_REF_ReadEvent
#   define BO_VAXEVENT_REF_WriteEvent BO_SVAXEVENT_REF_WriteEvent
#   define BO_VAXEVENT_REF_ReadVal32 BO_SVAXEVENT_REF_ReadVal32
#   define BO_VAXEVENT_REF_WriteVal32 BO_SVAXEVENT_REF_WriteVal32
#   define BO_VAXEVENT_REF_ReadVal64 BO_SVAXEVENT_REF_ReadVal64
#   define BO_VAXEVENT_REF_WriteVal64 BO_SVAXEVENT_REF_WriteVal64
#   define BO_VAXEVENT_REF_ReadVal32v0 BO_SVAXEVENT_REF_ReadVal32v0
#   define BO_VAXEVENT_REF_WriteVal32v0 BO_SVAXEVENT_REF_WriteVal32v0
#   define BO_VAXEVENT_REF_ReadVal32v1 BO_SVAXEVENT_REF_ReadVal32v1
#   define BO_VAXEVENT_REF_WriteVal32v1 BO_SVAXEVENT_REF_WriteVal32v1
#   define BO_VAXEVENT_REF_ReadVal32v BO_SVAXEVENT_REF_ReadVal32v
#   define BO_VAXEVENT_REF_WriteVal32v BO_SVAXEVENT_REF_WriteVal32v
#  else
#   define bo_vaxevent_t bo_pvaxevent_t
#   define bo_vaxevent_ref bo_pvaxevent_ref
#   define BO_VAXEVENT_REF BO_PVAXEVENT_REF
#   define BO_VAXEVENT_REF_INVALID BO_PVAXEVENT_REF_INVALID
#   define BO_VAXEVENT_REF_Mutex BO_PVAXEVENT_REF_Mutex
#   define BO_VAXEVENT_REF_Mutexref BO_PVAXEVENT_REF_Mutexref
#   define BO_VAXEVENT_REF_Cond BO_PVAXEVENT_REF_Cond
#   define BO_VAXEVENT_REF_Condref BO_PVAXEVENT_REF_Condref
#   define BO_VAXEVENT_REF_Event BO_PVAXEVENT_REF_Event
#   define BO_VAXEVENT_REF_Val32 BO_PVAXEVENT_REF_Val32
#   define BO_VAXEVENT_REF_Val64 BO_PVAXEVENT_REF_Val64
#   define BO_VAXEVENT_REF_Val32v0 BO_PVAXEVENT_REF_Val32v0
#   define BO_VAXEVENT_REF_Val32v1 BO_PVAXEVENT_REF_Val32v1
#   define BO_VAXEVENT_REF_Val32v BO_PVAXEVENT_REF_Val32v
#   define BO_VAXEVENT_REF_Sender BO_PVAXEVENT_REF_Sender
#   define BO_VAXEVENT_REF_Context BO_PVAXEVENT_REF_Context

#   define BO_VAXEVENT_REF_ReadEvent BO_PVAXEVENT_REF_ReadEvent
#   define BO_VAXEVENT_REF_WriteEvent BO_PVAXEVENT_REF_WriteEvent
#   define BO_VAXEVENT_REF_ReadVal32 BO_PVAXEVENT_REF_ReadVal32
#   define BO_VAXEVENT_REF_WriteVal32 BO_PVAXEVENT_REF_WriteVal32
#   define BO_VAXEVENT_REF_ReadVal64 BO_PVAXEVENT_REF_ReadVal64
#   define BO_VAXEVENT_REF_WriteVal64 BO_PVAXEVENT_REF_WriteVal64
#   define BO_VAXEVENT_REF_ReadVal32v0 BO_PVAXEVENT_REF_ReadVal32v0
#   define BO_VAXEVENT_REF_WriteVal32v0 BO_PVAXEVENT_REF_WriteVal32v0
#   define BO_VAXEVENT_REF_ReadVal32v1 BO_PVAXEVENT_REF_ReadVal32v1
#   define BO_VAXEVENT_REF_WriteVal32v1 BO_PVAXEVENT_REF_WriteVal32v1
#   define BO_VAXEVENT_REF_ReadVal32v BO_PVAXEVENT_REF_ReadVal32v
#   define BO_VAXEVENT_REF_WriteVal32v BO_PVAXEVENT_REF_WriteVal32v
#  endif /* (0x0600 <= _BO_BUILD_WIN32_WINNT) */
# endif /* BO_SYSTYPE_WIN32 */
# endif /* HAS_PTHREAD_COND */

#endif /* !__BO_THRTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
