/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Trpdefs.h	(V.Sokur)
 * include/Bo/Trpdefs.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_TRPDEFS__
# define __BO_TRPDEFS__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_TYPES__
#  include <Bo/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define BoK_T_FEATURENOTSUPP (0) /* idea not supported */
# define BoK_T_LIMITNOTSUPP (-1)
# define BoK_T_NOTSUPP (-2)
# define BoK_T_NOTALLOWED (-2)

/* ============================================================================
 * transport info declarations
 */
# pragma mark -

struct bo_tinfohdr {
	u_int8_t     ti_magic;
	u_int8_t     ti_pad;
	u_int16_t    ti_dslen; /* including this header */
};
# define BO_TINFOHDR_GUTS \
	u_int8_t     ti_magic; \
	u_int8_t     ti_pad; \
	u_int16_t    ti_dslen

struct bo_tinfo_ds {
	BO_TINFOHDR_GUTS;

	/* mandatory fields */
	bo_scalar_t  ti_addr_max;  /* max size of [protocol] address, in bytes */
};
# define bo_tinfo_p(x) ((struct bo_tinfo_ds *)(x))

/*
 * [well known] values for transport-info magic (.ti_magic)
 */
# define BoK_INVAL_TINFO (-1)
# define BoK_UNK_TINFO 2
# define BoK_FILE_TINFO 3
# define BoK_FIFO_TINFO 4
# define BoK_SOCK_TINFO 5
# define BoK_XTI_TINFO 6

/* XTI based */
struct bo_tinfo_oti {
	BO_TINFOHDR_GUTS;

	bo_scalar_t  ti_addr_max;     /* max size of [protocol] address, in bytes */
	bo_scalar_t  ti_opts_max;     /* max size of [protocol] options */
	bo_scalar_t  ti_du_max;       /* max size of [max transport service] data unit */
	bo_scalar_t  ti_edu_max;      /* max size of max expedited tsdu */
	bo_scalar_t  ti_conda_max;    /* max data for connection primitives */
	bo_scalar_t  ti_disconda_max; /* max data for disconnect primitives */
	bo_scalar_t  ti_servtype;	  /* provider service type */
	bo_scalar_t  ti_flags;        /* other info about transport providers */
};

/*
 * values for transport provider service type (.ti_servtype)
 * XTI
 */
# define BoK_T_COTS 1       /* connection oriented transport service */
# define BoK_T_COTS_ORD 2   /* connection oriented with orderly release */
# define BoK_T_CLTS 3       /* connectionless transport service */
# define BoK_T_RDMA 4       /* rdma transport service */

/*
 * values for transport-info flags (.ti_flags)
 */
# define BoF_T_SNDZERO 0x01    /* supports 0-length TSDUs */
# define BoF_T_ORDRELDATA 0x02 /* supports orderly release data */

/*
 * values for transport semantics (TPI - transport independent)
 */
# define BoK_TPI_CLTS 1     /* SOCK_DGRAM */
# define BoK_TPI_COTS 2     /* SOCK_STREAM (???) */
# define BoK_TPI_COTS_ORD 3 /* SOCK_STREAM (???) */
# define BoK_TPI_RAW 4      /* SOCK_RAW */
# define BoK_TPI_RDM 5      /* reliably-delivered message, SOCK_RDM */
# define BoK_TPI_SEQPKT 6   /* sequenced packet stream, SOCK_SEQPACKET */
# define BoK_TPI_RDMA 7     /* Remote Direct Memory Access provider interfaces */

/* ============================================================================
 * uconn declarations
 */
# pragma mark -

struct bo_uconn_ds {
	u_int8_t   uc_magic;
	u_int8_t   uc_form;
	u_int16_t  uc_dslen; /* including this header */
};
# define bo_uconn_p(x) ((struct bo_uconn_ds *)(x))

# define BO_UCONN_MAGIC(x) (bo_uconn_p(x)->uc_magic)
# define BO_UCONN_FORM(x) (bo_uconn_p(x)->uc_form)
# define BO_UCONN_DSLEN(x) (bo_uconn_p(x)->uc_dslen)

# define BO_UCONN_TAG(aName) __myc_xconcat(bo_uconn_,aName)
# define BO_UCONN_TYPENAME(aName) struct BO_UCONN_TAG(aName)

/*
 * [well known] values for bo_uconn_ds.uc_magic
 */
# define BoK_INVAL_UCONN (-1)
# define BoK_UNK_UCONN 2
# define BoK_MEM_UCONN 3
# define BoK_FD_UCONN 4 /* file-descriptor(regular, socket, xti, ...) */
# define BoK_TPO_UCONN 5 /* transport endpoint object */
# define BoK_URI_UCONN 6
# define BoK_FSPATH_UCONN BoK_URI_UCONN
# define BoK_SOCK_UCONN 7
# define BoK_SLUZBA_UCONN 8

/*
 * bo_uconn_ds.uc_magic = BoK_MEM_UCONN
 * bo_uconn_ds.uc_form not used
 */
struct bo_uconn_mem {
	struct bo_uconn_ds  hdr;
	struct bo_nabsbuf   uc_buf;
	bo_ownership_t      uc_owns;
};
# define bo_uconn_mem_p(x) ((struct bo_uconn_mem *)(x))

/*
 * bo_uconn_ds.uc_magic = BoK_FD_UCONN
 */

/* values for bo_uconn_ds.uc_form */
# define BoK_UC_FDFILE 0 /* regular or link */
# define BoK_UC_FDFIFO 1
# define BoK_UC_FDSOCK 2
# define BoK_UC_FDXTI 3

struct bo_uconn_fd {
	struct bo_uconn_ds  hdr;
	intptr_t            uc_fd;
	bo_ownership_t      uc_owns;
	int                 uc_family; /* used for oti */
};
# define bo_uconn_fd_p(x) ((struct bo_uconn_fd *)(x))

/*
 * bo_uconn_ds.uc_magic = BoK_TPO_UCONN
 */

/* values for bo_uconn_ds.uc_form */
# define BoK_UC_TPOAOBJ 0
# define BoK_UC_TPOFILE 1 /* FILE */

struct bo_uconn_tpo {
	struct bo_uconn_ds  hdr;
	union {
		aorp_object_t   uc_aobj;
		bo_pointer_t    uc_file;
	} u1;
	bo_ownership_t      uc_owns; /* used for .uc_file */
};
# define bo_uconn_tpo_p(x) ((struct bo_uconn_tpo *)(x))

/*
 * bo_uconn_ds.uc_magic = BoK_URI_UCONN
 * bo_uconn_ds.uc_magic = BoK_FSPATH_UCONN
 */

/* values for bo_uconn_ds.uc_form */
# define BoK_UC_URI 0
# define BoK_UC_URISTR 1
# define BoK_UC_URIWCS 2
# define BoK_UC_URIPATH 3 /* /var/tmp/test1.jpg */
# define BoK_UC_URIWPATH 4

# define BoK_UC_FSPATH_URI BoK_UC_URI
# define BoK_UC_FSPATH_URISTR BoK_UC_URISTR
# define BoK_UC_FSPATH_URIWCS BoK_UC_URIWCS
# define BoK_UC_FSPATH_STR BoK_UC_URIPATH
# define BoK_UC_FSPATH_WCS BoK_UC_URIWPATH

struct bo_uconn_uri {
	struct bo_uconn_ds  hdr;
	union {
		bo_pointer_t    uc_any;
		aorp_object_t   uc_uri;
		char const     *uc_uristr;
		wchar_t const  *uc_uriwcs;
		char const     *uc_path;
		wchar_t const  *uc_wpath;
	} u1;
};
# define bo_uconn_uri_p(x) ((struct bo_uconn_uri *)(x))

# define bo_uconn_fspath bo_uconn_uri
# define bo_uconn_fspath_p(x) ((struct bo_uconn_fspath *)(x))

/*
 * bo_uconn_ds.uc_magic = BoK_SLUZBA_UCONN
 */

/* values for bo_uconn_ds.uc_form */
# define BoK_UC_SLUZBA_SI 0
# define BoK_UC_SLUZBA_WSI 1

struct bo_uconn_srvcinfo {
	char const  *host;
	char const  *service;
	int          port;
	char const  *protocol;
	char const  *protofmly;
};

struct bo_uconn_wsrvcinfo {
	wchar_t const  *host;
	wchar_t const  *service;
	int             port;
	wchar_t const  *protocol;
	wchar_t const  *protofmly;
};

struct bo_uconn_sluzba {
	struct bo_uconn_ds  hdr;
	union {
		struct bo_uconn_srvcinfo  uc_info;
		struct bo_uconn_wsrvcinfo uc_winfo;
	} u1;
};
# define bo_uconn_sluzba_p(x) ((struct bo_uconn_sluzba *)(x))

/*
 * bo_uconn_ds.uc_magic = BoK_SOCK_UCONN
 */

/* values for bo_uconn_ds.uc_form */
# define BoK_UC_SOCK_FILE 0
# define BoK_UC_SOCK_IPV4 1
# define BoK_UC_SOCK_IPV6 2

struct bo_uconn_sock_file {
	struct bo_uconn_ds  hdr;
	u_int16_t           uc_semantics;
	u_int16_t           uc_protofmly; /* AF_LOCAL */
	char                uc_addr[104];
};

struct bo_uconn_sock_ipv4 {
	struct bo_uconn_ds  hdr;
	int16_t             uc_byteorder; /* byte order; 0: host, otherwise: net */
	u_int16_t           uc_proto;
	u_int16_t           uc_protofmly;
	u_int16_t           uc_port;
	u_int32_t           uc_addr;
};

struct bo_uconn_sock_ipv6 {
	struct bo_uconn_ds  hdr;
	int16_t             uc_byteorder; /* byte order; 0: host, otherwise: net */
	u_int16_t           uc_proto;
	u_int16_t           uc_protofmly;
	u_int16_t           uc_port;
	u_int32_t           uc_flowinfo;
	u_int8_t            uc_addr[16];
};

union bo_uconn_sock {
	struct bo_uconn_sock_file file;
	struct bo_uconn_sock_ipv4 ipv4;
	struct bo_uconn_sock_ipv6 ipv6;
};

#endif /* !__BO_TRPDEFS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
