/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Types.h	(V.Sokur)
 * include/Bo/Types.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_TYPES__
# define __BO_TYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SYS_TYPES__
#  include <Bo/sys/Types.h>
# endif
# ifndef __BO_SYS_CSTYPE__
#  include <Bo/sys/csType.h>
# endif
# ifndef __BO_AORP_TYPES__
#  include <Bo/aorp/Types.h>
# endif
# ifndef __BO_LIBC_STDBOOL__
#  include <Bo/libc/Stdbool.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_bo_status_t,
	sizeof(intptr_t) <= sizeof(bo_status_t) &&
	sizeof(size_t) <= sizeof(bo_status_t) &&
	sizeof(ssize_t) <= sizeof(bo_status_t) &&
	sizeof(bo_index_t) <= sizeof(bo_status_t) &&
	sizeof(bo_hash_t) <= sizeof(bo_status_t) &&
	sizeof(bo_mismatch_t) <= sizeof(bo_status_t) &&
	sizeof(bo_refc_t) <= sizeof(bo_status_t));
_BoDbg_StaticAssert(
	sizeof_bo_index_t,
	sizeof(int32_t) <= sizeof(bo_index_t));
_BoDbg_StaticAssert(
	sizeof_bo_stsptr_t,
	sizeof(intptr_t) <= sizeof(bo_stsptr_t));
# endif
/* @} */

/* ------------------------------------------------------------------------- */

/* @todo: fixme */
# ifndef __BO_SCALAR_TYPE_DEFINED__
#  if (BO_M_WORD == BO_M_WORD32)
typedef int32_t   bo_scalar_t;
typedef u_int32_t bo_uscalar_t;
#  else
#  if (BO_M_WORD == BO_M_WORD64)
typedef int64_t   bo_scalar_t;
typedef u_int64_t bo_uscalar_t;
#  endif
#  endif
#  define __BO_SCALAR_TYPE_DEFINED__
# endif

/* ------------------------------------------------------------------------- */

struct bo_charseq {
	size_t   len;
	char    *base;
};

struct bo_wcharseq {
	size_t   len;
	wchar_t *base;
};

/* ------------------------------------------------------------------------- */

struct bo_absbuf {
	size_t        size;
	bo_pointer_t  base;
};

struct bo_nabsbuf {
	size_t        capacity;
	size_t        size;
	bo_pointer_t  base;
};

# ifndef __BO_ABSBUFX_STRUC_DEFINED__
#  define __BO_ABSBUFX_STRUC_DEFINED__
struct bo_malloc;

struct bo_absbufx {
	struct bo_malloc const *alloc; /* used on base */
	size_t                  size;
	bo_pointer_t            base;
};

struct bo_nabsbufx {
	struct bo_malloc const *alloc; /* used on base */
	size_t                  capacity;
	size_t                  size;
	bo_pointer_t            base;
};
# endif /* !__BO_ABSBUFX_STRUC_DEFINED__ */

struct bo_qabsbuf {
	size_t              size;
	bo_pointer_t        base;
	struct bo_qabsbuf  *next;
};

struct bo_qnabsbuf {
	size_t              capacity;
	size_t              size;
	bo_pointer_t        base;
	struct bo_qnabsbuf *next;
};

struct bo_dqabsbuf {
	size_t              size;
	bo_pointer_t        base;
	struct bo_dqabsbuf *next, *prev;
};

struct bo_dqnabsbuf {
	size_t               capacity;
	size_t               size;
	bo_pointer_t         base;
	struct bo_dqnabsbuf *next, *prev;
};

/* ------------------------------------------------------------------------- */

struct bo_sabsbuf {
	ssize_t       size;
	bo_pointer_t  base;
};

struct bo_snabsbuf {
	ssize_t       capacity;
	ssize_t       size;
	bo_pointer_t  base;
};

struct bo_qsabsbuf {
	ssize_t              size;
	bo_pointer_t         base;
	struct bo_qsabsbuf  *next;
};

struct bo_qsnabsbuf {
	ssize_t              capacity;
	ssize_t              size;
	bo_pointer_t         base;
	struct bo_qsnabsbuf *next;
};

struct bo_dqsabsbuf {
	ssize_t              size;
	bo_pointer_t         base;
	struct bo_dqsabsbuf *next, *prev;
};

struct bo_dqsnabsbuf {
	ssize_t               capacity;
	ssize_t               size;
	bo_pointer_t          base;
	struct bo_dqsnabsbuf *next, *prev;
};

struct bo_databuf {
	size_t  size;
	char    base[FAM_LENGTH];
};

struct bo_qdatabuf {
	struct bo_qdatabuf  *next;
	size_t               size;
	char                 base[FAM_LENGTH];
};

struct bo_dqdatabuf {
	struct bo_dqdatabuf *next, *prev;
	size_t               size;
	char                 base[FAM_LENGTH];
};

#endif /* !__BO_TYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
