/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Aorplib.h	(V.Sokur)
 * include/Bo/aorp/Aorplib.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_AORP_LIB__
# define __BO_AORP_LIB__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_AORP_TYPES__
#  include <Bo/aorp/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# ifndef __BOLINKAGE__
#  include <Bo/BoLinkage.h>
# endif

__MYC_BEGIN_CDECLS

/*
void
AORP_OID_Assign(aorp_oid_t aDest[], aorp_oid_t const aSource[]);
 */
# define AORP_OID_Assign(aDest, aSource) \
	do { \
		register unsigned aorpOidCopyCounter; \
		register aorp_oid_t *aorpOidCopyD; \
		register aorp_oid_t const *aorpOidCopyS; \
		aorpOidCopyD = (aDest); \
		aorpOidCopyS = (aSource); \
		aorpOidCopyCounter = AORP_OID_Count(aorpOidCopyS) + 2; \
		while (aorpOidCopyCounter--) \
			*aorpOidCopyD++ = *aorpOidCopyS++; \
	} while (0)

# if 0 && __myc_support(macro_retval)
/*
bo_mismatch_t
AORP_OID_Compare(aorp_oid_t const aLhs[], aorp_oid_t const aRhs[]); */
#  define AORP_OID_Compare(aLhs, aRhs) \
	__extension__ ({ \
		bo_mismatch_t aorpOidCmpMismatch; \
		register unsigned aorpOidCmpCounter, aorpOidCmpLhsLen, aorpOidCmpRhsLen; \
		register aorp_oid1_t aorpOidCmpLhs1, aorpOidCmpRhs1; \
		register aorp_oid_t const *aorpOidCmpLhs, *aorpOidCmpRhs; \
		aorpOidCmpLhs = (aLhs); \
		aorpOidCmpLhsLen = AORP_OID_Count(aorpOidCmpLhs); \
		aorpOidCmpRhs = (aRhs); \
		aorpOidCmpRhsLen = AORP_OID_Count(aorpOidCmpRhs); \
		if (aorpOidCmpRhsLen < (aorpOidCmpCounter = aorpOidCmpLhsLen)) \
			aorpOidCmpCounter = aorpOidCmpRhsLen; \
		aorpOidCmpLhs += 2; \
		aorpOidCmpRhs += 2; \
		assert(0 != aorpOidCmpCounter); \
		for (;;) { \
			aorpOidCmpLhs1 = *aorpOidCmpLhs++; \
			aorpOidCmpRhs1 = *aorpOidCmpRhs++; \
			if (0 != (aorpOidCmpMismatch = aorpOidCmpLhs1 - aorpOidCmpRhs1)) { \
				if (AorpK_ANY_OID != aorpOidCmpLhs1 && AorpK_ANY_OID != aorpOidCmpRhs1) \
					break; \
			} \
			if (0 == --aorpOidCmpCounter) { \
				aorpOidCmpMismatch = 0; \
				break; \
			} \
		} \
		/* return */ (aorpOidCmpMismatch); \
	})

/*
bo_mismatch_t
AORP_OID_LCompare(aorp_oid_t const aLhs[], aorp_oid_t const aRhs[]); */
#  define AORP_OID_LCompare(aLhs, aRhs) \
	__extension__ ({ \
		bo_mismatch_t aorpOidCmpMismatch; \
		register unsigned aorpOidCmpCounter, aorpOidCmpLhsLen, aorpOidCmpRhsLen; \
		register aorp_oid1_t aorpOidCmpLhs1, aorpOidCmpRhs1; \
		register aorp_oid_t const *aorpOidCmpLhs, *aorpOidCmpRhs; \
		aorpOidCmpLhs = (aLhs); \
		aorpOidCmpLhsLen = AORP_OID_Count(aorpOidCmpLhs); \
		aorpOidCmpRhs = (aRhs); \
		aorpOidCmpRhsLen = AORP_OID_Count(aorpOidCmpRhs); \
		if (aorpOidCmpRhsLen < (aorpOidCmpCounter = aorpOidCmpLhsLen)) \
			aorpOidCmpCounter = aorpOidCmpRhsLen; \
		aorpOidCmpLhs += 2; \
		aorpOidCmpRhs += 2; \
		assert(0 != aorpOidCmpCounter); \
		for (;;) { \
			aorpOidCmpLhs1 = *aorpOidCmpLhs++; \
			aorpOidCmpRhs1 = *aorpOidCmpRhs++; \
			if (0 != (aorpOidCmpMismatch = aorpOidCmpLhs1 - aorpOidCmpRhs1)) { \
				if (AorpK_ANY_OID != aorpOidCmpLhs1 && AorpK_ANY_OID != aorpOidCmpRhs1) \
					break; \
			} \
			if (0 == --aorpOidCmpCounter) { \
				aorpOidCmpMismatch = 0; \
				if (aorpOidCmpRhsLen < aorpOidCmpLhsLen) \
					aorpOidCmpMismatch = aorpOidCmpLhs1 - 0; \
				else if (aorpOidCmpLhsLen < aorpOidCmpRhsLen) \
					aorpOidCmpMismatch = 0 - aorpOidCmpRhs1; \
				break; \
			} \
		} \
		/* return */ (aorpOidCmpMismatch); \
	})

# else /* !__myc_support(macro_retval) */

#  define AORP_OID_Compare(aLhs, aRhs) AorpOidcmp(aLhs, aRhs)
#  define AORP_OID_LCompare(aLhs, aRhs) AorpOidlcmp(aLhs, aRhs)

# endif /* !__myc_support(macro_retval) */

__myc_flinkage__(public,libBo) bo_mismatch_t
__attribute__((nonnull(1,2)))
AorpOidcmp(
# if __myc_require(prototypes)
	aorp_oid_t const aLhs[],
	aorp_oid_t const aRhs[]
# endif
	);

__myc_flinkage__(public,libBo) bo_mismatch_t
__attribute__((nonnull(1,2)))
AorpOidlcmp(
# if __myc_require(prototypes)
	aorp_oid_t const aLhs[],
	aorp_oid_t const aRhs[]
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_AORP_LIB__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
