/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * IntrinsicP.h	(V.Sokur)
 * include/Bo/aorp/IntrinsicP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_AORP_TYPES_P__
# include <Bo/aorp/TypesP.h>
#endif
#ifndef __BO_AORP_INTRINSIC__
# include <Bo/aorp/Intrinsic.h>
#endif

#ifndef __BO_AORP_INTRINSIC_P__
# define __BO_AORP_INTRINSIC_P__

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# ifndef __BOLINKAGE__
#  include <Bo/BoLinkage.h>
# endif

__MYC_BEGIN_CDECLS

/*
 *  X-OBJECT < C-OBJECT < OPS < M-OBJECT
 */

__myc_flinkage__(public,libBo) bo_mismatch_t
__attribute__((nonnull(1,2)))
AorpObjectCompare(
# if __myc_require(prototypes)
	aorp_object_t aLhs,
	aorp_object_t aRhs
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) size_t
__attribute__((nonnull(1)))
AorpInstcount(
# if __myc_require(prototypes)
	aorp_object_t aThis
# endif
	);

__myc_flinkage__(public,libBo) size_t
__attribute__((nonnull(1)))
AorpInstsize(
# if __myc_require(prototypes)
	aorp_object_t aThis
# endif
	);

__myc_flinkage__(public,libBo) size_t
__attribute__((nonnull(1)))
AorpExtrainstsz(
# if __myc_require(prototypes)
	aorp_object_t aThis
# endif
	);

__myc_flinkage__(public,libBo) size_t
__attribute__((nonnull(1)))
AorpExtrainstsz(
# if __myc_require(prototypes)
	aorp_object_t aThis
# endif
	);

# define AorpInstextrasz(aThis) AorpExtrainstsz(aThis)

/* ----------------------------------------------------------------------------
 * lookup-POD
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_pointer_t
__attribute__((nonnull(1,2)))
AorpLookupPOD(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid_t const anOid[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpLookupPOD1(aThis, aServiceId, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId); \
		/* return */ AorpLookupPOD(aThis, ___oid, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_pointer_t
__attribute__((nonnull(1,3)))
AorpLookupPOD1(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId);
	return AorpLookupPOD(aThis, oid, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpLookupPOD2(aThis, aServiceId, aSecondId, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId); \
		/* return */ AorpLookupPOD(aThis, ___oid, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_pointer_t
__attribute__((nonnull(1,4)))
AorpLookupPOD2(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	aorp_oid1_t aSecondId,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, aSecondId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId, aSecondId;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId);
	return AorpLookupPOD(aThis, oid, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_pointer_t
__attribute__((nonnull(1,2)))
_AorpLookupPOD(
# if __myc_require(prototypes)
	aorp_object_t aThis, /* x-object, c-object, m-object */
	aorp_oid_t const anOid[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define _AorpLookupPOD1(aThis, aServiceId, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId); \
		/* return */ _AorpLookupPOD(aThis, ___oid, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_pointer_t
__attribute__((nonnull(1,3)))
_AorpLookupPOD1(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId);
	return _AorpLookupPOD(aThis, oid, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define _AorpLookupPOD2(aThis, aServiceId, aSecondId, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId); \
		/* return */ _AorpLookupPOD(aThis, ___oid, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_pointer_t
__attribute__((nonnull(1,4)))
_AorpLookupPOD2(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	aorp_oid1_t aSecondId,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, aSecondId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId, aSecondId;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId);
	return _AorpLookupPOD(aThis, oid, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ----------------------------------------------------------------------------
 * lookup-SE
 */
# pragma mark -

__myc_flinkage__(public,libBo) struct aorp_se *
__attribute__((nonnull(1,2,3)))
AorpLookupSE(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid_t const anOid[],
	aorp_object_t *aProviderPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_se *
__attribute__((nonnull(1,2,3)))
_AorpLookupSE(
# if __myc_require(prototypes)
	aorp_object_t aThis, /* x-object, c-object, m-object */
	aorp_oid_t const anOid[],
	aorp_object_t *aProviderPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * is-service-supported
 */
# pragma mark -

__myc_extern_inline__ bo_stsbool_t
__attribute__((nonnull(1,2)))
AorpSuppservice(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid_t const anOid[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, anOid, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid_t const anOid[];
	struct aorp_error *anErrPtr;
# endif
{
	bo_pointer_t p;
	aorp_object_t provider;
	p = AorpLookupSE(aThis, anOid, &provider, anErrPtr);
	return (!BO_ILLPTR(p)
		? BoS_TRUE
		: ((NULL == p) ? BoS_FALSE : BoS_ERR));
}

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpSuppservice1(aThis, aServiceId, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId); \
		/* return */ AorpSuppservice(aThis, ___oid, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_stsbool_t
__attribute__((nonnull(1,3)))
AorpSuppservice1(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId);
	return AorpSuppservice(aThis, oid, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpSuppservice2(aThis, aServiceId, aSecondId, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId); \
		/* return */ AorpSuppservice(aThis, ___oid, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_stsbool_t
__attribute__((nonnull(1,4)))
AorpSuppservice2(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	aorp_oid1_t aSecondId,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, aSecondId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId, aSecondId;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId);
	return AorpSuppservice(aThis, oid, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ----------------------------------------------------------------------------
 * lookup-op
 */
# pragma mark -

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2,3)))
AorpLookupop(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid_t const anOid[],
	struct aorp_closure *aClosure,
	struct aorp_error *anErrPtr /* __myc_defval__(NULL) */
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2,3)))
_AorpLookupop(
# if __myc_require(prototypes)
	aorp_object_t aThis, /* x-object, c-object, m-object */
	aorp_oid_t const anOid[],
	struct aorp_closure *aClosure,
	struct aorp_error *anErrPtr /* __myc_defval__(NULL) */
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2,3)))
AorpLookupoporstub(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid_t const anOid[],
	struct aorp_closure *aClosure
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2,3)))
_AorpLookupoporstub(
# if __myc_require(prototypes)
	aorp_object_t aThis, /* x-object, c-object, m-object */
	aorp_oid_t const anOid[],
	struct aorp_closure *aClosure
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2)))
AorpVlookupop(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	struct aorp_closure *aClosure,
	struct aorp_error *anErrPtr,
	aorp_oid1_t anOpcode,
	aorp_oid1_t aPrimId,
	...
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2)))
_AorpVlookupop(
# if __myc_require(prototypes)
	aorp_object_t aThis, /* x-object, c-object, m-object */
	struct aorp_closure *aClosure,
	struct aorp_error *anErrPtr,
	aorp_oid1_t anOpcode,
	aorp_oid1_t aPrimId,
	...
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2)))
AorpVlookupoporstub(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	struct aorp_closure *aClosure,
	aorp_oid1_t anOpcode,
	aorp_oid1_t aPrimId,
	...
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_operation *
__attribute__((nonnull(1,2)))
_AorpVlookupoporstub(
# if __myc_require(prototypes)
	aorp_object_t aThis, /* x-object, c-object, m-object */
	struct aorp_closure *aClosure,
	aorp_oid1_t anOpcode,
	aorp_oid1_t aPrimId,
	...
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_AORP_INTRINSIC_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
