/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Pool.h	(V.Sokur)
 * include/Bo/aorp/Pool.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_AORP_POOL__
# define __BO_AORP_POOL__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/memman/Zmalloc.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# define aorp_pool bo_zmalloc

# ifndef __BOLINKAGE__
#  include <Bo/BoLinkage.h>
# endif

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) struct aorp_pool *
__attribute__((nonnull(1)))
AorpPoolOpen(
# if __myc_require(prototypes)
	aorp_oid_t const anOID[], /* a pool for object with the @oid */
	size_t aZoneGranularity __myc_defval__(0), /* = 0 -> default-value: 64 (?) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# if __myc_support(macro_retval)
#  define AorpPoolOpen1(anObjectId, aZoneGranularity, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_classoid1_i(anObjectId); \
		/* return */ AorpPoolOpen(___oid, aZoneGranularity, anErrPtr); \
	})
# else
__myc_extern_inline__ struct aorp_pool *
AorpPoolOpen1(
# if __myc_require(prototypes)
	aorp_oid1_t anObjectId,
	size_t aZoneGranularity __myc_defval__(0), /* = 0 -> default-value: 64 (?) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	anObjectId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_oid1_t anObjectId;
	size_t aZoneGranularity;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_classoid1_i(anObjectId);
	return AorpPoolOpen(oid, aZoneGranularity, anErrPtr);
}
# endif

# pragma mark -

/* Allocated memory will be inited (and fini-ed) only once. */
__myc_flinkage__(public,libBo) struct aorp_pool *
__attribute__((nonnull(1)))
AorpPoolOpen_ooi(
# if __myc_require(prototypes)
	aorp_oid_t const anOID[], /* a pool for object with the @oid */
	size_t aZoneGranularity __myc_defval__(0), /* = 0 -> default-value: 64 (?) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# if __myc_support(macro_retval)
#  define AorpPoolOpen1_ooi(anObjectId, aZoneGranularity, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_classoid1_i(anObjectId); \
		/* return */ AorpPoolOpen_ooi(___oid, aZoneGranularity, anErrPtr); \
	})
# else
__myc_extern_inline__ struct aorp_pool *
AorpPoolOpen1_ooi(
# if __myc_require(prototypes)
	aorp_oid1_t anObjectId,
	size_t aZoneGranularity __myc_defval__(0), /* = 0 -> default-value: 64 (?) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	anObjectId, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_oid1_t anObjectId;
	size_t aZoneGranularity;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_classoid1_i(anObjectId);
	return AorpPoolOpen_ooi(oid, aZoneGranularity, anErrPtr);
}
# endif

# pragma mark -

__myc_flinkage__(public,libBo) struct aorp_pool *
_AorpPoolOpen(
# if __myc_require(prototypes)
	aorp_object_t aClass,
	size_t aZoneGranularity __myc_defval__(0), /* = 0 -> default-value: 64 (?) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# pragma mark -

/*
ssize_t
AorpPoolClose(
	struct aorp_pool *aPool,
	aorp_opflags_t Flags __myc_defval__(0), // AorpF_OP_FORCE
	struct aorp_error *anErrPtr __myc_defval__(NULL)
	);
 */
# define AorpPoolClose(aPool, Flags, anErrPtr) \
	_BoZmallocClose(aPool, Flags, anErrPtr)

# pragma mark -

/* NOT FOR YOUR BUSINESS */
/* @{ */

# define _AorpPoolGet(x, z) _BoZmallocGet(x, z)
# define _AorpPoolElemRelease(x) _BoZmPtrRelease(x)

/* @} */

__MYC_END_CDECLS

#endif /* !__BO_AORP_POOL__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
