/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Runtime.h	(V.Sokur)
 * include/Bo/aorp/Runtime.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_AORP_TYPES__
# include <Bo/aorp/Types.h>
#endif

#ifndef __BO_AORP_RUNTIME__
# define __BO_AORP_RUNTIME__

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# ifndef __BOLINKAGE__
#  include <Bo/BoLinkage.h>
# endif

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1,2,3)))
AorpSeobject(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid_t const anOid[],
	aorp_sobject_t *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpSeobject1(aThis, aServiceId, aBuffer, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId); \
		/* return */ AorpSeobject(aThis, ___oid, aBuffer, anErrPtr); \
	})
# else
__myc_extern_inline__ aorp_object_t
__attribute__((nonnull(1,3)))
AorpSeobject1(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	aorp_sobject_t *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, aBuffer, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId;
	aorp_sobject_t *aBuffer;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId);
	return AorpSeobject(aThis, oid, aBuffer, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpSeobject2(aThis, aServiceId, aSecondId, aBuffer, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId); \
		/* return */ AorpSeobject(aThis, ___oid, aBuffer, anErrPtr); \
	})
# else
__myc_extern_inline__ aorp_object_t
__attribute__((nonnull(1,4)))
AorpSeobject2(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	aorp_oid1_t aSecondId,
	aorp_sobject_t *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, aSecondId, aBuffer, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId, aSecondId;
	aorp_sobject_t *aBuffer;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId);
	return AorpSeobject(aThis, oid, aBuffer, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_closure *
__attribute__((nonnull(1,2,3)))
_AorpClosure(
# if __myc_require(prototypes)
	aorp_object_t aThis, /* x-object, c-object, m-object */
	aorp_oid_t const anOid[],
	struct aorp_closure *aClosure,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define _AorpClosure1(aThis, aServiceId, aBuffer, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId); \
		/* return */ _AorpClosure(aThis, ___oid, aBuffer, anErrPtr); \
	})
# else
__myc_extern_inline__ struct aorp_closure *
__attribute__((nonnull(1,3)))
_AorpClosure1(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	struct aorp_closure *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, aBuffer, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId;
	struct aorp_closure *aBuffer;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid1_i(aServiceId);
	return _AorpClosure(aThis, oid, aBuffer, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define _AorpClosure2(aThis, aServiceId, aSecondId, aBuffer, anErrPtr) \
	__extension__ ({ \
		aorp_oid_t ___oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId); \
		/* return */ _AorpClosure(aThis, ___oid, aBuffer, anErrPtr); \
	})
# else
__myc_extern_inline__ struct aorp_closure *
__attribute__((nonnull(1,4)))
_AorpClosure2(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	aorp_oid1_t aServiceId,
	aorp_oid1_t aSecondId,
	struct aorp_closure *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aThis, aServiceId, aSecondId, aBuffer, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	aorp_oid1_t aServiceId, aSecondId;
	struct aorp_closure *aBuffer;
	struct aorp_error *anErrPtr;
# endif
{
	aorp_oid_t oid[AorpK_OID_LEN] = _Art_seoid2_i(aServiceId, aSecondId);
	return _AorpClosure(aThis, oid, aBuffer, anErrPtr);
}
# endif /* __myc_support(macro_retval) */

__MYC_END_CDECLS

#endif /* !__BO_AORP_RUNTIME__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
