/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * RuntimeP.h	(V.Sokur)
 * include/Bo/aorp/RuntimeP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_AORP_TYPES_P__
# include <Bo/aorp/TypesP.h>
#endif
#ifndef __BO_AORP_RUNTIME__
# include <Bo/aorp/Runtime.h>
#endif

#ifndef __BO_AORP_RUNTIME_P__
# define __BO_AORP_RUNTIME_P__

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# ifndef __BOLINKAGE__
#  include <Bo/BoLinkage.h>
# endif

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
_AorpObjRdlock(
# if __myc_require(prototypes)
	struct aorp_dobject *aThis,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
_AorpObjWrlock(
# if __myc_require(prototypes)
	struct aorp_dobject *aThis,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* BoS_TRUE, BoS_FALSE */
__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
_AorpObjTryrdlock(
# if __myc_require(prototypes)
	struct aorp_dobject *aThis,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
_AorpObjTrywrlock(
# if __myc_require(prototypes)
	struct aorp_dobject *aThis,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
_AorpObjUnlock(
# if __myc_require(prototypes)
	struct aorp_dobject *aThis,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
AorpEndops(
# if __myc_require(prototypes)
	aorp_object_t aSelf,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
 * These functions require 'AorpEndops()' call,
 * when OPS is not longer needed.
 * @{
 */
__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
AorpGetops(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
AorpGetopsorstub(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[]
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
AorpVgetopsorstub(
# if __myc_require(prototypes)
	aorp_oid1_t aPrimId,
	...
# endif
	);
/* @} */

/* ARE YOU SURE YOU WANT TO USE THESE FUNCTIONS? */
/*
 * These functions do not require 'AorpEndops()' call,
 * when OPS is not longer needed.
 * @{
 */
__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
_AorpGetops(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
_AorpGetopsorstub(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[]
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
_AorpVgetopsorstub(
# if __myc_require(prototypes)
	aorp_oid1_t aPrimId,
	...
# endif
	);
/* @} */

# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
AorpEndcobject(
# if __myc_require(prototypes)
	aorp_object_t aSelf,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Very dangerous function! */
__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
AorpEndcobject_nlk(
# if __myc_require(prototypes)
	aorp_object_t ,
	struct aorp_error * __myc_defval__(NULL)
# endif
	);

/*
 * These functions require 'AorpEndcobject()' call,
 * when C-OBJECT is not longer needed.
 * @{
 */

/* Get the cobject (or stub aorp-object) specified by the @oid. */

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
AorpGetcobject(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Very dangerous function! */
__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
AorpGetcobject_nlk(
# if __myc_require(prototypes)
	aorp_oid_t const [],
	struct aorp_error * __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
AorpGetclsorstub(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[]
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
AorpVgetclsorstub(
# if __myc_require(prototypes)
	aorp_oid1_t aPrimId,
	...
# endif
	);

# define  AorpVgetmodule _AorpVgetmodule
/* @} */

/* ARE YOU SURE YOU WANT TO USE THESE FUNCTIONS? */
/*
 * These functions do not require 'AorpEndcobject()' call,
 * when C-OBJECT is not longer needed.
 * @{
 */

/* Get the cobject (or stub aorp-object) specified by the @oid. */

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
_AorpGetcobject(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Very dangerous function! */
__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
_AorpGetcobject_nlk(
# if __myc_require(prototypes)
	aorp_oid_t const [],
	struct aorp_error * __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(1)))
_AorpGetclsorstub(
# if __myc_require(prototypes)
	aorp_oid_t const anOid[]
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
_AorpVgetclsorstub(
# if __myc_require(prototypes)
	aorp_oid1_t aPrimId,
	...
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
_AorpVgetmodule(
# if __myc_require(prototypes)
	aorp_oid1_t aPrimId,
	...
# endif
	);
/* @} */

# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
AorpForeach(
# if __myc_require(prototypes)
	aorp_oid_t const aDirOid[],
	struct aorp_foreach_pred *aPred,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
AorpForeach_nlk(
# if __myc_require(prototypes)
	aorp_oid_t const aDirOid[],
	struct aorp_foreach_pred *aPred,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_AORP_RUNTIME_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
