/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Types.h	(V.Sokur)
 * include/Bo/aorp/Types.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_TYPES__
# include <Bo/sys/Types.h>
#endif

#ifndef __BO_AORP_TYPES__
# define __BO_AORP_TYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/aorp/AorpVersion.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/*
 * Forward declarations
 */
struct aorp_closure;
struct aorp_error;

/* ============================================================================
 * Flags
 */
# pragma mark -

typedef bo_opflags_t aorp_opflags_t;
typedef bo_opflags_x aorp_opflags_x;

/*
 * Common well-known flags
 */

# define AorpF_OP_FORCE            0x01 /* @seealso: BoF_OP_FORCE */
# define AorpF_OP_DELAY            0x02
# define AorpF_OP_WAIT             0x04
# define AorpF_ASYNC_MODE          0
# define AorpF_SYNC_MODE           0x08

/* constructor */
# define AorpF_USE_POOL            0x010000 /* a pool should be used to get memory for an object */
# define _AorpF_FINI_BY_POOL       0x020000 /* not for your business */
# define AorpF_NOBZERO_ON_ALLOC    0x040000
# define AorpF_FREE_ON_DEALLOC     0x080000

/* duplicator */
# define AorpF_DUP_LAZYCOPY        0x100000
# define AorpF_DUP_INIT            0x200000 /* pod-init; you should not set the flag directly */

/* ============================================================================
 * Common callback types
 */

typedef bo_pointer_t
(__myc_tcalltype__(mcall) *aorp_dup_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anObj,
	aorp_opflags_t Flags, /* = 0 */
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_pointer_t
(__myc_tcalltype__(mcall) *aorp_dup2_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anObj,
	aorp_opflags_t Flags, /* = 0 */
	bo_pointer_t aParm, /* = NULL */
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_retain_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anObj,
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_release_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anObj,
	aorp_opflags_t Flags, /* = 0 */
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_mismatch_t
(__myc_tcalltype__(mcall) *aorp_compare_cb)(
# if __myc_require(prototypes)
	bo_pointer_t aLhs,
	bo_pointer_t aRhs /* comparable-object */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

/* ============================================================================
 * OID
 */
# pragma mark -

typedef int       aorp_oid1_t;
typedef u_int16_t aorp_oid_t;
typedef u_int16_t aorp_oid_x;

# define AorpK_OID_LEN (8)
# define AorpL_OID_LEN AorpK_OID_LEN

/* anOid: aorp_oid_t anOid[] */
/* @{ */

/* standard */
# define AORP_OID_Count(anOid) ((anOid)[_AorpK_OID_COUNT])
	/* 0 <= retval <= AorpK_OID_LEN - 2 */

/* standard */
# define AORP_OID_Opcode(anOid) ((anOid)[_AorpK_OID_OPCODE])

/*
	CATEGORY  PRIMARY-ID SECONDARY-ID
	   0          1          2           [3]     [4]       [5]

	'ops'      service     domain     subclass   impl      ...
	'object'   object       impl        ...
	'module'   module       impl        ...
 */
# define AORP_OID_Field(N,anOid) ((anOid)[(N) + 2])
	/* 0 <= N <= AorpK_OID_LEN - 2 */

# define AORP_OID_Category(anOid) AORP_OID_Field(0,anOid)
# define AORP_OID_PrimID(anOid) AORP_OID_Field(1,anOid)
# define AORP_OID_SeconID(anOid) AORP_OID_Field(2,anOid)

# define _AorpK_OID_COUNT (0)
# define _AorpK_OID_OPCODE (1)
# define _AORP_OID_NONOP (0xffff)
/* @} */

/* ------------------------------------------------------------------------- */

# define AorpK_INVAL_OID (-1)
# define AorpK_NULL_OID (0)
# define AORP_NULL_OID "null"
# define AorpK_ANY_OID AorpK_NULL_OID
# define AorpK_UNK_OID (2)

/*
 * OID categories
 */

/* get-ops(\internal-database\) */
# define AorpK_OPS_CATEGORY (3)
# define AORP_OPS_CATEGORY "operations"

/* lookup-service-element(object) */
# define AorpK_SE_CATEGORY AorpK_OPS_CATEGORY /* pseudo-category */

/* lookup-pod(object) */
# define AorpK_POD_CATEGORY AorpK_SE_CATEGORY /* pseudo-category */

/* get-cobject(\internal-database\) */
# define AorpK_OBJECT_CATEGORY (4)
# define AORP_OBJECT_CATEGORY "objects"

/* get-mobject(\internal-database\) */
# define AorpK_MODULE_CATEGORY (5)
# define AORP_MODULE_CATEGORY "modules"

/*
 * Aorp System DB categories
 */
# define AorpK_SERVICEDB_CATEGORY (6)
# define AorpK_OPDOMDB_CATEGORY (7)
# define AorpK_OBJECTDB_CATEGORY (8)
# define AorpK_MODULEDB_CATEGORY (9)

/* misc.: generic parameter */
# define AORP_ONE_OF(x,parm) x
# define AORP_ONEP_OF(parm) AORP_ONE_OF(bo_pointer_t, parm)
# define AORP_ONEPP_OF(parm) AORP_ONE_OF(bo_pointer_t *, parm)

/* ============================================================================
 * AORP-OBJECT
 */
# pragma mark -

/*
 * Hint
 */

typedef int       aorp_hint_t;
typedef u_int16_t aorp_hint_x;

/* AORP well-known object hints. */
/* DO NOT MODIFY ORDER!!! */
# define AorpK_HINT_INVAL (-1)
# define AorpK_HINT_NULL (0)
# define AorpK_HINT_UNK (2)

/* special (dummy) hint */
# define _AorpK_HINT_ANYOB (1)
	/* any aorp-object: AorpK_HINT_SOBJECT, AorpK_HINT_XOBJECT, AorpK_HINT_COBJECT, ... */

# define AorpK_HINT_SOBJECT (3) /* aorp-service-object */
# define AorpK_HINT_XOBJECT (4) /* aorp-creatable-object */
/* compat @{ */
# define AorpK_HINT_ANY _AorpK_HINT_ANYOB
# define AorpK_HINT_AOBJECT AorpK_HINT_XOBJECT
/* @} */
# define AorpK_HINT_COBJECT (5) /* aorp-class-object */
# define AorpK_HINT_OPS (6) /* aorp-operation-set */
# define AorpK_HINT_MOBJECT (7) /* aorp-module-object */

# define AorpK_HINT_SE (8) /* aorp-service-element */
# define AorpK_HINT_SES (9) /* aorp-service-element-set */

/* AORP reserved object hints: [19-127] */
# define AorpK_OBJECT_HINT_RESERVED_BEG (19)
# define AorpK_OBJECT_HINT_RESERVED_END (127)

/* Other well-known object hints. @{ */
# define AorpK_HINT_POINTER (20)
# define AorpK_HINT_CURSOR (21)
# define AorpK_HINT_STR (30)
# define AorpK_HINT_WCS (31)
# define AorpK_HINT_POINTER_MAX (63) /* dummy hint */

# define AorpK_HINT_INDEX (70)
# define AorpK_HINT_INT32 (71)
# define AorpK_HINT_INT64 (72)
# define AorpK_HINT_FLOAT32 (80)
# define AorpK_HINT_FLOAT64 (81)
# define AorpK_HINT_FLOAT128 (82)
# define AorpK_HINT_COMPLEX (83)

# define AorpK_HINT_HASH (87)
# define AorpK_HINT_SHASH (88)
# define AorpK_HINT_BHASH (89)
# define AorpK_HINT_WCSHASH (90)
# define AorpK_HINT_STRIX (91)
# define AorpK_HINT_BSEQIX (92)
# define AorpK_HINT_WCSIX (93)
# define AorpK_HINT_INT32IX (94)
# define AorpK_HINT_INT64IX (95)
# define AorpK_HINT_FLOAT32IX (96)
# define AorpK_HINT_FLOAT64IX (97)
# define AorpK_HINT_FLOAT128IX (98)
# define AorpK_HINT_OBJIX (99)

# define AorpK_HINT_CMPPRED (100)

# define AorpK_HINT_FOREACHPRED (101)
/* @} */

/* ----------------------------------------------------------------------------
 * any object
 */

struct aorp_object_s {
	aorp_hint_x  hint_;
	u_int16_t    reserved_;
};
typedef struct aorp_object_s *aorp_object_t;

# define aorp_object_p(x) ((struct aorp_object_s *)(x))

# define AORP_OBJECT_Hint(x) (aorp_object_p(x)->hint_)

/* ----------------------------------------------------------------------------
 * "Real" aorp object of some type
 */

# if BO_M_WORD == BO_M_WORD32
#  define AorpK_AOBJECT_OPAQUE_SIZE 4
#  define AorpK_AOBJECT_SIZE 8
# else
#  define AorpK_AOBJECT_OPAQUE_SIZE 12
#  define AorpK_AOBJECT_SIZE 16
# endif

struct aorp_aobject_s {
	aorp_hint_x  ao_hint;
	u_int16_t    ao_reserved;
	char         ao_opaque[AorpK_AOBJECT_OPAQUE_SIZE];
	/* data */
};

/* ----------------------------------------------------------------------------
 * Service object.
 * Designed to be a stack object.
 */

# if BO_M_WORD == BO_M_WORD32
#  define AorpK_SOBJECT_OPAQUE_SIZE 20
#  define AorpK_SOBJECT_SIZE 24
# else
#  define AorpK_SOBJECT_OPAQUE_SIZE 44
#  define AorpK_SOBJECT_SIZE 48
# endif

struct aorp_sobject_s {
	aorp_hint_x  aso_hint;
	u_int16_t    aso_reserved;
	char         aso_opaque[AorpK_SOBJECT_OPAQUE_SIZE];
};
typedef struct aorp_sobject_s aorp_sobject_t;

/* ----------------------------------------------------------------------------
 * Custom object
 */

/*
 * Known magic for custom object
 * @{
 */
# define AorpK_KUM_OBJECT __myc_mk4cc('a','k','u','m')
# define AorpK_KUMAS_OBJECT __myc_mk4cc('k','u','m','a')
# define AorpK_KUMFAM4_OBJECT __myc_mk4cc('k','u','m','4')
# define AorpK_KUMFAM8_OBJECT __myc_mk4cc('k','u','m','8')
# define AorpK_KUMFAM12_OBJECT __myc_mk4cc('k','u','1','2')
# define AorpK_KUMFAM16_OBJECT __myc_mk4cc('k','u','1','6')
# define AorpK_KUMFAM24_OBJECT __myc_mk4cc('k','u','2','4')
# define AorpK_KUMFAM32_OBJECT __myc_mk4cc('k','u','3','2')
# define AorpK_KUMFAM36_OBJECT __myc_mk4cc('k','u','3','6')
# define AorpK_KUMFAM48_OBJECT __myc_mk4cc('k','u','4','8')
# define AorpK_KUMFAM64_OBJECT __myc_mk4cc('k','u','6','4')
# define AorpK_KUMFAM128_OBJECT __myc_mk4cc('k','1','2','8')
# define AorpK_KUMFAM256_OBJECT __myc_mk4cc('k','2','5','6')
# define AorpK_KUMI32_OBJECT __myc_mk4cc('k','i','3','2')
# define AorpK_KUMI64_OBJECT __myc_mk4cc('k','i','6','4')
# define AorpK_KUMF32_OBJECT __myc_mk4cc('k','f','3','2')
# define AorpK_KUMF64_OBJECT __myc_mk4cc('k','f','6','4')
# define AorpK_KUMSTR_OBJECT __myc_mk4cc('k','s','t','r')
# define AorpK_KUMWCS_OBJECT __myc_mk4cc('k','w','c','s')
# define AorpK_KUMBUF_OBJECT __myc_mk4cc('k','b','u','f')
# define AorpK_KUMAGGR_OBJECT __myc_mk4cc('a','g','g','r')

// @todo: remove
# define AorpK_KUMBUFX_OBJECT __myc_mk4cc('k','b','f','x')
/* @} */

typedef bo_pointer_t
(__myc_tcalltype__(mcall) *aorp_custom_dup_cb)(
# if __myc_require(prototypes)
	bo_pointer_t aData,
	aorp_opflags_t Flags, /* = 0 */
	bo_pointer_t aParm, /* = NULL */
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef void
(__myc_tcalltype__(mcall) *aorp_custom_free_cb)(
# if __myc_require(prototypes)
	bo_pointer_t aData,
	aorp_opflags_t Flags, /* = 0 */
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_hash_t
(__myc_tcalltype__(mcall) *aorp_custom_hash_cb)(
# if __myc_require(prototypes)
	bo_pointer_t aData,
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

struct aorp_custom_opv {
	aorp_custom_dup_cb   co_dup_;
	aorp_custom_free_cb  co_free_; /* .co_destroy_ */
	aorp_custom_hash_cb  co_hash_;
};

struct aorp_custom_s {
	struct aorp_aobject_s  h_;
	bo_pointer_t aco_data;
};
typedef struct aorp_custom_s *aorp_custom_t;

# define aorp_custom_p(x) ((struct aorp_custom_s *)(x))

# ifndef _AorpF_RTXI_CUSTOM
#  define _AorpF_RTXI_CUSTOM 0x8000
# endif

/* ------------------------------------------------------------------------- */

/* reserved: 0-3 */
# define _AorpK_CUSTOM_RESERVED0_TAG 0
# define _AorpK_CUSTOM_CONST_SIZEOF_TAG 1
# define _AorpK_CUSTOM_FAMCB_TAG 2
# define _AorpK_CUSTOM_RESERVED3_TAG 3

# define AorpK_CUSTOM_4_TAG 4
# define AorpK_CUSTOM_8_TAG 8
# define AorpK_CUSTOM_12_TAG 12
# define AorpK_CUSTOM_16_TAG 16
# define AorpK_CUSTOM_24_TAG 24
# define AorpK_CUSTOM_32_TAG 32
# define AorpK_CUSTOM_36_TAG 36
# define AorpK_CUSTOM_48_TAG 48
# define AorpK_CUSTOM_64_TAG 64
# define AorpK_CUSTOM_128_TAG 128
# define AorpK_CUSTOM_256_TAG 256
# define AorpK_CUSTOM_I32_TAG 0xd32
# define AorpK_CUSTOM_I64_TAG 0xd64
# define AorpK_CUSTOM_F32_TAG 0xf32
# define AorpK_CUSTOM_F64_TAG 0xf64
# define AorpK_CUSTOM_STR_TAG 0xffc
# define AorpK_CUSTOM_WCS_TAG 0xfcc
# define AorpK_CUSTOM_BUF_TAG 0xfab
# define AorpK_CUSTOM_BUFCB_TAG 0xfbc
# define AorpK_CUSTOM_AGGR_TAG 0xaaa

// @todo: remove
# define AorpK_CUSTOM_BUFX_TAG 0xfbb

struct aorp_customb_s {
	struct aorp_aobject_s  h_;
	char aco_data[FAM_LENGTH];
};
typedef struct aorp_customb_s *aorp_customb_t;

# define aorp_customb_p(x) ((struct aorp_customb_s *)(x))

# define _STRUCT_AORP_CUSTOMB(N) \
	struct __myc_xconcat(aorp_customb,N) { \
		struct aorp_aobject_s  h_; \
		char aco_data[N]; \
	}

_STRUCT_AORP_CUSTOMB(4);
_STRUCT_AORP_CUSTOMB(8);
_STRUCT_AORP_CUSTOMB(12);
_STRUCT_AORP_CUSTOMB(16);
_STRUCT_AORP_CUSTOMB(24);
_STRUCT_AORP_CUSTOMB(32);
_STRUCT_AORP_CUSTOMB(36);
_STRUCT_AORP_CUSTOMB(48);
_STRUCT_AORP_CUSTOMB(64);
_STRUCT_AORP_CUSTOMB(128);
_STRUCT_AORP_CUSTOMB(256);

struct aorp_custom_i32 {
	struct aorp_aobject_s  h_;
	int32_t aco_data;
};

struct aorp_custom_i64 {
	struct aorp_aobject_s  h_;
	int64_t aco_data;
};

struct aorp_custom_f32 {
	struct aorp_aobject_s  h_;
	float aco_data;
};

struct aorp_custom_f64 {
	struct aorp_aobject_s  h_;
	double aco_data;
};

struct aorp_custom_str {
	struct aorp_aobject_s  h_;
	char *aco_data;
};

struct aorp_custom_wcs {
	struct aorp_aobject_s  h_;
	wchar_t *aco_data;
};


struct aorp_kumfam_buf {
	size_t        aco_capacity;
	size_t        aco_size;
	bo_pointer_t  aco_base;
};

struct aorp_custom_buf {
	struct aorp_aobject_s  h_;
	struct aorp_kumfam_buf b;
};

// @todo: remove
// @{
struct aorp_kumfam_bufx {
	size_t          aco_capacity;
	size_t          aco_size;
	bo_pointer_t    aco_base;
	struct aorp_custom_opv  aco_copv;
};

struct aorp_custom_bufx {
	struct aorp_aobject_s   h_;
	struct aorp_kumfam_bufx b;
};
// @}


struct aorp_kumfam_aggr {
	aorp_object_t  aco_lhs;
	aorp_object_t  aco_rhs;
};

struct aorp_custom_aggr {
	struct aorp_aobject_s   h_;
	struct aorp_kumfam_aggr a;
};


/* ----------------------------------------------------------------------------
 * MACROS to test object TYPE
 */

/* object-category */
# define AORP_IS_SXCOB(x) \
	(AorpK_HINT_SOBJECT == AORP_OBJECT_Hint(x) || \
		AorpK_HINT_XOBJECT == AORP_OBJECT_Hint(x) || \
		AorpK_HINT_COBJECT == AORP_OBJECT_Hint(x))
# define AORP_IS_SOBJECT(x) (AorpK_HINT_SOBJECT == AORP_OBJECT_Hint(x))
# define AORP_IS_XOBJECT(x) (AorpK_HINT_XOBJECT == AORP_OBJECT_Hint(x))
# define AORP_IS_COBJECT(x) (AorpK_HINT_COBJECT == AORP_OBJECT_Hint(x))
# define AORP_IS_CUSTOM(x) (_AorpF_RTXI_CUSTOM & aorp_object_p(x)->reserved_)
/* compat @{ */
# define AORP_IS_AOBJECT(x) AORP_IS_XOBJECT(x)
# define AORP_IS_ANY(x) AORP_IS_SXCOB(x)
/* @} */

/* module-category */
# define AORP_IS_MOBJECT(x) (AorpK_HINT_MOBJECT == AORP_OBJECT_Hint(x))

/* ops-category */
# define AORP_IS_OPS(x) (AorpK_HINT_OPS == AORP_OBJECT_Hint(x))

# define AORP_IS_SE(x) (AorpK_HINT_SE == AORP_OBJECT_Hint(x))
# define AORP_IS_SES(x) (AorpK_HINT_SES == AORP_OBJECT_Hint(x))

/* ----------------------------------------------------------------------------
 * Closure
 */

# if BO_M_WORD == BO_M_WORD32
#  define AorpK_CLOSURE_SIZE 20
# else
#  define AorpK_CLOSURE_SIZE 40
# endif

struct aorp_closure_s {
	char aclo_opaque[AorpK_CLOSURE_SIZE];
};
typedef struct aorp_closure_s aorp_closure_t;

# define AORP_CLOSURE_ZERO_INITIALIZER {{0}}

/* ============================================================================
 * Comparable object
 */
# pragma mark -

struct aorp_comparable_s {
	aorp_hint_x  cmp_hint;
	u_int16_t    cmp_reserved;
	/* data */
};
typedef struct aorp_comparable_s *aorp_comparable_t;

# define aorp_comparable_p(x) ((struct aorp_comparable_s *)(x))

struct aorp_comparable_idx {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_INDEX */
	u_int16_t    cmp_reserved; /* should be 0 */
	bo_index_t   cmp_data;
};

struct aorp_comparable_i32 {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_INT32 */
	u_int16_t    cmp_reserved; /* should be 0 */
	int32_t      cmp_data;
};

struct aorp_comparable_i64 {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_INT64 */
	u_int16_t    cmp_reserved; /* should be 0 */
	int64_t      cmp_data;
};

struct aorp_comparable_f32 {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_FLOAT32 */
	u_int16_t    cmp_reserved; /* should be 0 */
	float        cmp_data;
};

struct aorp_comparable_f64 {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_FLOAT64 */
	u_int16_t    cmp_reserved; /* should be 0 */
	double       cmp_data;
};

struct aorp_comparable_f128 {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_FLOAT128 */
	u_int16_t    cmp_reserved; /* should be 0 */
	long double  cmp_data;
};

struct aorp_comparable_str {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_STR */
	u_int16_t    cmp_reserved; /* should be 0 */
	char const  *cmp_data;
};

struct aorp_comparable_wcs {
	aorp_hint_x     cmp_hint;     /* AorpK_HINT_WCS */
	u_int16_t       cmp_reserved; /* should be 0 */
	wchar_t const  *cmp_data;
};

struct aorp_comparable_hash {
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_HASH */
	u_int16_t    cmp_reserved; /* should be 0 */
	bo_hash_t    cmp_data;
};

struct aorp_comparable_shash {
	/* struct aorp_comparable_str { */
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_SHASH */
	u_int16_t    cmp_reserved; /* should be 0 */
	char const  *cmp_data;
	/* } */
	bo_hash_t    cmp_hash;
};

struct aorp_comparable_bhash {
	aorp_hint_x       cmp_hint;     /* AorpK_HINT_BHASH */
	u_int16_t         cmp_reserved; /* should be 0 */
	size_t            cmp_bytec;
	bo_byte_t const  *cmp_data;
	bo_hash_t         cmp_hash;
};

struct aorp_comparable_wcshash {
	/* struct aorp_comparable_wcs { */
	aorp_hint_x     cmp_hint;     /* AorpK_HINT_WCSHASH */
	u_int16_t       cmp_reserved; /* should be 0 */
	wchar_t const  *cmp_data;
	/* } */
	bo_hash_t       cmp_hash;
};

struct aorp_comparable_strix {
	/* struct aorp_comparable_shash { */
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_STRIX */
	u_int16_t    cmp_reserved; /* should be 0 */
	char const  *cmp_data;
	bo_hash_t    cmp_hash;
	/* } */
	bo_index_t   cmp_ino;
};

struct aorp_comparable_bseqix {
	/* struct aorp_comparable_bhash { */
	aorp_hint_x       cmp_hint;     /* AorpK_HINT_BSEQIX */
	u_int16_t         cmp_reserved; /* should be 0 */
	size_t            cmp_bytec;
	bo_byte_t const  *cmp_data;
	bo_hash_t         cmp_hash;
	/* } */
	bo_index_t        cmp_ino;
};

struct aorp_comparable_wcsix {
	/* struct aorp_comparable_wcshash { */
	aorp_hint_x     cmp_hint;     /* AorpK_HINT_WCSIX */
	u_int16_t       cmp_reserved; /* should be 0 */
	wchar_t const  *cmp_data;
	bo_hash_t       cmp_hash;
	/* } */
	bo_index_t      cmp_ino;
};

struct aorp_comparable_i32ix {
	/* struct aorp_comparable_i32 { */
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_INT32IX */
	u_int16_t    cmp_reserved; /* should be 0 */
	int32_t      cmp_data;
	/* } */
	bo_index_t   cmp_ino;
};

struct aorp_comparable_i64ix {
	/* struct aorp_comparable_i64 { */
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_INT64IX */
	u_int16_t    cmp_reserved; /* should be 0 */
	int64_t      cmp_data;
	/* } */
	bo_index_t   cmp_ino;
};

struct aorp_comparable_f32ix {
	/* struct aorp_comparable_f32 { */
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_FLOAT32IX */
	u_int16_t    cmp_reserved; /* should be 0 */
	float        cmp_data;
	/* } */
	bo_index_t   cmp_ino;
};

struct aorp_comparable_f64ix {
	/* struct aorp_comparable_f64 { */
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_FLOAT64IX */
	u_int16_t    cmp_reserved; /* should be 0 */
	double       cmp_data;
	/* } */
	bo_index_t   cmp_ino;
};

struct aorp_comparable_f128ix {
	/* struct aorp_comparable_f128 { */
	aorp_hint_x  cmp_hint;     /* AorpK_HINT_FLOAT128IX */
	u_int16_t    cmp_reserved; /* should be 0 */
	long double  cmp_data;
	/* } */
	bo_index_t   cmp_ino;
};

struct aorp_comparable_objix {
	aorp_hint_x            cmp_hint;     /* AorpK_HINT_OBJIX */
	u_int16_t              cmp_reserved; /* should be 0 */
	struct aorp_object_s  *cmp_data;
	bo_index_t             cmp_ino;
};

/* forward declaration */
struct aorp_comparable_pred;

/* ============================================================================
 * POD init declarations
 */
# pragma mark -

struct aorp_podinit_s {
	aorp_oid_t oid[AorpK_OID_LEN];
	/* data */
};
typedef struct aorp_podinit_s *aorp_podinit_t;

# define aorp_podinit_p(x) ((struct aorp_podinit_s *)(x))


struct aorp_dup_predicate;

typedef bo_stsbool_t
(__myc_tcalltype__(mcall) *aorp_duplicator_predicate_fn)(
# if __myc_require(prototypes)
	struct aorp_dup_predicate *,
	aorp_oid_t const [/* AorpK_OID_LEN */],
	bo_pointer_t ,
	struct aorp_error *
# endif
	) __attribute__((mcalltype, nonnull(1)));

struct aorp_dup_predicate {
	aorp_duplicator_predicate_fn exec;
};

struct aorp_duppodinit {
	aorp_closure_t             dpi_source;
	aorp_object_t              dpi_duplicator;
	bo_pointer_t               dpi_params;
	struct aorp_dup_predicate *dpi_predicate;
};

# define AORP_DUPPODINIT_ZERO_INITIALIZER \
	{ AORP_CLOSURE_ZERO_INITIALIZER, NULL, NULL, NULL }

/* ============================================================================
 * User-data variant
 */
# pragma mark -

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define aorp_udvariant aorp_udvariant128
#  define aorp_udvariant_p aorp_udvariant128_p
# else
#  define aorp_udvariant aorp_udvariant64
#  define aorp_udvariant_p aorp_udvariant64_p
# endif

struct aorp_udvariant64 {
	/* struct aorp_object_s @{ */
	aorp_hint_x        udc_hint;
	u_int16_t          udc_reserved; /* should be 0 */
	/* @} */

	union {
		bo_pointer_t   udc_udata; /* .udc_hint: AorpK_HINT_POINTER */
		aorp_object_t  udc_obj;   /* .udc_hint: _AorpK_HINT_ANYOB */
		char          *udc_str;   /* .udc_hint: AorpK_HINT_STR */
		wchar_t       *udc_wcs;   /* .udc_hint: AorpK_HINT_WCS */
		int32_t        udc_i32;   /* .udc_hint: AorpK_HINT_INT32 */
		int64_t        udc_i64;   /* .udc_hint: AorpK_HINT_INT64 */
		float          udc_f32;   /* .udc_hint: AorpK_HINT_FLOAT32 */
		double         udc_f64;   /* .udc_hint: AorpK_HINT_FLOAT64 */

# if (BO_M_WORD == BO_M_WORD32 || BO_M_WORD == BO_M_WORD64)
		char           udc_opaque[8];
		int16_t        udc_hv[4];
		int32_t        udc_lv[2];
		int64_t        udc_llv[1];
		float          udc_fv[2];
		double         udc_dv[1];
# else
		char           udc_opaque[...];
		int16_t        udc_hv[4];
		int32_t        udc_lv[2];
		int64_t        udc_llv[1];
		float          udc_fv[2];
		double         udc_dv[1];
# endif
	} u;
};
# define aorp_udvariant64_p(x) ((struct aorp_udvariant64 *)(x))

/* ------------------------------------------------------------------------- */

struct aorp_udvariant128 {
	/* struct aorp_object_s @{ */
	aorp_hint_x        udc_hint;
	u_int16_t          udc_reserved; /* should be 0 */
	/* @} */

	union {
		bo_pointer_t   udc_udata; /* .udc_hint: AorpK_HINT_POINTER */
		aorp_object_t  udc_obj;   /* .udc_hint: _AorpK_HINT_ANYOB */
		char          *udc_str;   /* .udc_hint: AorpK_HINT_STR */
		wchar_t       *udc_wcs;   /* .udc_hint: AorpK_HINT_WCS */
		int32_t        udc_i32;   /* .udc_hint: AorpK_HINT_INT32 */
		int64_t        udc_i64;   /* .udc_hint: AorpK_HINT_INT64 */
		float          udc_f32;   /* .udc_hint: AorpK_HINT_FLOAT32 */
		double         udc_f64;   /* .udc_hint: AorpK_HINT_FLOAT64 */
		long double    udc_f128;  /* .udc_hint: AorpK_HINT_FLOAT128 */

# if (BO_M_WORD == BO_M_WORD32 || BO_M_WORD == BO_M_WORD64)
		char           udc_opaque[16];
		int16_t        udc_hv[8];
		int32_t        udc_lv[4];
		int64_t        udc_llv[2];
		float          udc_fv[4];
		double         udc_dv[2];
# else
		char           udc_opaque[...];
		int16_t        udc_hv[8];
		int32_t        udc_lv[4];
		int64_t        udc_llv[2];
		float          udc_fv[4];
		double         udc_dv[2];
# endif
	} u;
};
# define aorp_udvariant128_p(x) ((struct aorp_udvariant128 *)(x))

/* ----------------------------------------------------------------------------
 * Common udvariant callbacks
 */

struct aorp_udopv {
	aorp_dup2_cb      ud_dup2;
	aorp_retain_cb    ud_retain;
	aorp_release_cb   ud_release;
	aorp_compare_cb   ud_compare;
};

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDVARIANT_Copy AORP_UDVARIANT128_Copy
# else
#  define AORP_UDVARIANT_Copy AORP_UDVARIANT64_Copy
# endif

# define AORP_UDVARIANT64_Copy(aStatusRef, anOpv, aSource, aDest, Flags, aParm, anErrPtr) \
	do { \
		bo_pointer_t udvariantCopyTmp = aorp_udvariant64_p(aSource)->u.udc_udata; \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_dup2 && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aSource) && \
				AORP_OBJECT_Hint(aSource) <= AorpK_HINT_POINTER_MAX && \
				NULL != udvariantCopyTmp) \
		{ \
			udvariantCopyTmp = (*(anOpv)->ud_dup2)(udvariantCopyTmp, Flags, aParm, anErrPtr); \
			if (NULL != udvariantCopyTmp) { \
				aorp_udvariant64_p(aDest)->udc_hint = aorp_udvariant64_p(aSource)->udc_hint; \
				aorp_udvariant64_p(aDest)->udc_reserved = aorp_udvariant64_p(aSource)->udc_reserved; \
				aorp_udvariant64_p(aDest)->u.udc_udata = udvariantCopyTmp; \
				break; \
			} \
			(aStatusRef) = BoS_ERR; \
			break; \
		} \
		*aorp_udvariant64_p(aDest) = *aorp_udvariant64_p(aSource); \
	} while (0)

# define AORP_UDVARIANT128_Copy(aStatusRef, anOpv, aSource, aDest, Flags, aParm, anErrPtr) \
	do { \
		bo_pointer_t udvariantCopyTmp = aorp_udvariant128_p(aSource)->u.udc_udata; \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_dup2 && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aSource) && \
				AORP_OBJECT_Hint(aSource) <= AorpK_HINT_POINTER_MAX && \
				NULL != udvariantCopyTmp) \
		{ \
			udvariantCopyTmp = (*(anOpv)->ud_dup2)(udvariantCopyTmp, Flags, aParm, anErrPtr); \
			if (NULL != udvariantCopyTmp) { \
				aorp_udvariant128_p(aDest)->udc_hint = aorp_udvariant128_p(aSource)->udc_hint; \
				aorp_udvariant128_p(aDest)->udc_reserved = aorp_udvariant128_p(aSource)->udc_reserved; \
				aorp_udvariant128_p(aDest)->u.udc_udata = udvariantCopyTmp; \
				break; \
			} \
			(aStatusRef) = BoS_ERR; \
			break; \
		} \
		*aorp_udvariant128_p(aDest) = *aorp_udvariant128_p(aSource); \
	} while (0)

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDVARIANT_RetainData AORP_UDVARIANT128_RetainData
# else
#  define AORP_UDVARIANT_RetainData AORP_UDVARIANT64_RetainData
# endif

# define AORP_UDVARIANT64_RetainData(aStatusRef, anOpv, aVar, anErrPtr) \
	do { \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_retain && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aVar) && \
				AORP_OBJECT_Hint(aVar) <= AorpK_HINT_POINTER_MAX && \
				NULL != aorp_udvariant64_p(aVar)->u.udc_udata) \
		{ \
			(aStatusRef) = (*(anOpv)->ud_retain)(aorp_udvariant64_p(aVar)->u.udc_udata, anErrPtr); \
		} \
	} while (0)

# define AORP_UDVARIANT128_RetainData(aStatusRef, anOpv, aVar, anErrPtr) \
	do { \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_retain && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aVar) && \
				AORP_OBJECT_Hint(aVar) <= AorpK_HINT_POINTER_MAX && \
				NULL != aorp_udvariant128_p(aVar)->u.udc_udata) \
		{ \
			(aStatusRef) = (*(anOpv)->ud_retain)(aorp_udvariant128_p(aVar)->u.udc_udata, anErrPtr); \
		} \
	} while (0)

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDVARIANT_ReleaseData AORP_UDVARIANT128_ReleaseData
#  define _AORP_UDVARIANT_ReleaseData _AORP_UDVARIANT128_ReleaseData
# else
#  define AORP_UDVARIANT_ReleaseData AORP_UDVARIANT64_ReleaseData
#  define _AORP_UDVARIANT_ReleaseData _AORP_UDVARIANT64_ReleaseData
# endif

# define AORP_UDVARIANT64_ReleaseData(aStatusRef, anOpv, aVar, Flags, anErrPtr) \
	do { \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_release && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aVar) && \
				AORP_OBJECT_Hint(aVar) <= AorpK_HINT_POINTER_MAX && \
				NULL != aorp_udvariant64_p(aVar)->u.udc_udata) \
		{ \
			(aStatusRef) = (*(anOpv)->ud_release)(aorp_udvariant64_p(aVar)->u.udc_udata, Flags, anErrPtr); \
			aorp_udvariant64_p(aVar)->u.udc_udata = NULL; \
		} \
	} while (0)

# define _AORP_UDVARIANT64_ReleaseData(aStatusRef, anOpv, aVar, Flags, anErrPtr) \
	do { \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_release && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aVar) && \
				AORP_OBJECT_Hint(aVar) <= AorpK_HINT_POINTER_MAX && \
				NULL != aorp_udvariant64_p(aVar)->u.udc_udata) \
		{ \
			(aStatusRef) = (*(anOpv)->ud_release)(aorp_udvariant64_p(aVar)->u.udc_udata, Flags, anErrPtr); \
		} \
	} while (0)

# define AORP_UDVARIANT128_ReleaseData(aStatusRef, anOpv, aVar, Flags, anErrPtr) \
	do { \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_release && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aVar) && \
				AORP_OBJECT_Hint(aVar) <= AorpK_HINT_POINTER_MAX && \
				NULL != aorp_udvariant128_p(aVar)->u.udc_udata) \
		{ \
			(aStatusRef) = (*(anOpv)->ud_release)(aorp_udvariant128_p(aVar)->u.udc_udata, Flags, anErrPtr); \
			aorp_udvariant128_p(aVar)->u.udc_udata = NULL; \
		} \
	} while (0)

# define _AORP_UDVARIANT128_ReleaseData(aStatusRef, anOpv, aVar, Flags, anErrPtr) \
	do { \
		(aStatusRef) = BoS_NORMAL; \
		if (NULLF != (anOpv)->ud_release && \
				AorpK_HINT_NULL < AORP_OBJECT_Hint(aVar) && \
				AORP_OBJECT_Hint(aVar) <= AorpK_HINT_POINTER_MAX && \
				NULL != aorp_udvariant128_p(aVar)->u.udc_udata) \
		{ \
			(aStatusRef) = (*(anOpv)->ud_release)(aorp_udvariant128_p(aVar)->u.udc_udata, Flags, anErrPtr); \
		} \
	} while (0)

/* ============================================================================
 * User-data context
 */
# pragma mark -

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define aorp_udcontext aorp_udcontext128
#  define aorp_udcontext_p aorp_udcontext128_p
# else
#  define aorp_udcontext aorp_udcontext64
#  define aorp_udcontext_p aorp_udcontext64_p
# endif

struct aorp_udcontext64 {
	/* struct aorp_object_s @{ */
	aorp_hint_x        udc_hint;
	u_int16_t          udc_reserved; /* should be 0 */
	/* @} */

	union {
		bo_pointer_t   udc_udata; /* .udc_hint: AorpK_HINT_POINTER */
		aorp_object_t  udc_obj;   /* .udc_hint: _AorpK_HINT_ANYOB */
		char          *udc_str;   /* .udc_hint: AorpK_HINT_STR */
		wchar_t       *udc_wcs;   /* .udc_hint: AorpK_HINT_WCS */
		int32_t        udc_i32;   /* .udc_hint: AorpK_HINT_INT32 */
		int64_t        udc_i64;   /* .udc_hint: AorpK_HINT_INT64 */
		float          udc_f32;   /* .udc_hint: AorpK_HINT_FLOAT32 */
		double         udc_f64;   /* .udc_hint: AorpK_HINT_FLOAT64 */
# if (BO_M_WORD == BO_M_WORD32 || BO_M_WORD == BO_M_WORD64)
		char           udc_opaque[8];
# else
		char           udc_opaque[...];
# endif
	} u;

	struct aorp_udopv  udc_dopv;
};
# define aorp_udcontext64_p(x) ((struct aorp_udcontext64 *)(x))

/* ------------------------------------------------------------------------- */

struct aorp_udcontext128 {
	/* struct aorp_object_s @{ */
	aorp_hint_x        udc_hint;
	u_int16_t          udc_reserved; /* should be 0 */
	/* @} */

	union {
		bo_pointer_t   udc_udata; /* .udc_hint: AorpK_HINT_POINTER */
		aorp_object_t  udc_obj;   /* .udc_hint: _AorpK_HINT_ANYOB */
		char          *udc_str;   /* .udc_hint: AorpK_HINT_STR */
		wchar_t       *udc_wcs;   /* .udc_hint: AorpK_HINT_WCS */
		int32_t        udc_i32;   /* .udc_hint: AorpK_HINT_INT32 */
		int64_t        udc_i64;   /* .udc_hint: AorpK_HINT_INT64 */
		float          udc_f32;   /* .udc_hint: AorpK_HINT_FLOAT32 */
		double         udc_f64;   /* .udc_hint: AorpK_HINT_FLOAT64 */
		long double    udc_f128;  /* .udc_hint: AorpK_HINT_FLOAT128 */
# if (BO_M_WORD == BO_M_WORD32 || BO_M_WORD == BO_M_WORD64)
		char           udc_opaque[16];
# else
		char           udc_opaque[...];
# endif
	} u;

	struct aorp_udopv  udc_dopv;
};
# define aorp_udcontext128_p(x) ((struct aorp_udcontext128 *)(x))

/* Required headers <Bo/services/Ucntl.h>, <Bo/services/Dup.h>
   are included. */

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_Destroy AORP_UDCONTEXT128_Destroy
# else
#  define AORP_UDCONTEXT_Destroy AORP_UDCONTEXT64_Destroy
# endif

# define AORP_UDCONTEXT64_Destroy(aStatusRef, aThis, Flags, anErrPtr) \
	do { \
		AORP_UDCONTEXT64_ReleaseData(aStatusRef, aThis, Flags, anErrPtr); \
		(aThis)->udc_hint = 0; \
		(aThis)->udc_reserved = 0; \
	} while (0)

# define AORP_UDCONTEXT128_Destroy(aStatusRef, aThis, Flags, anErrPtr) \
	do { \
		AORP_UDCONTEXT128_ReleaseData(aStatusRef, aThis, Flags, anErrPtr); \
		(aThis)->udc_hint = 0; \
		(aThis)->udc_reserved = 0; \
	} while (0)

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_Init AORP_UDCONTEXT128_Init
# else
#  define AORP_UDCONTEXT_Init AORP_UDCONTEXT64_Init
# endif

# define AORP_UDCONTEXT64_Init(aThis, aHint, _TField, aData, Dup, Retain, Release, Compare) \
	_im_AORP_UDCONTEXT64_Init(aThis, aHint, _TField, aData, Dup, Retain, Release, Compare)

# define AORP_UDCONTEXT128_Init(aThis, aHint, _TField, aData, Dup, Retain, Release, Compare) \
	_im_AORP_UDCONTEXT128_Init(aThis, aHint, _TField, aData, Dup, Retain, Release, Compare)

/*
 * Special cases
 * @{
 */

/*
 * AORP_UDCONTEXT_InitObj
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitObj AORP_UDCONTEXT128_InitObj
# else
#  define AORP_UDCONTEXT_InitObj AORP_UDCONTEXT64_InitObj
# endif

# define AORP_UDCONTEXT64_InitObj(aThis, anObj) \
	_im_AORP_UDCONTEXT64_Init( \
		aThis, _AorpK_HINT_ANYOB, udc_obj, anObj, \
		AorpLazydup /* AorpDups2 */, AorpRetain, AorpRelease, AorpCompare)

# define AORP_UDCONTEXT128_InitObj(aThis, anObj) \
	_im_AORP_UDCONTEXT128_Init( \
		aThis, _AorpK_HINT_ANYOB, udc_obj, anObj, \
		AorpLazydup /* AorpDups2 */, AorpRetain, AorpRelease, AorpCompare)

/*
 * AORP_UDCONTEXT_InitPtr
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitPtr AORP_UDCONTEXT128_InitPtr
# else
#  define AORP_UDCONTEXT_InitPtr AORP_UDCONTEXT64_InitPtr
# endif

# define AORP_UDCONTEXT64_InitPtr(aThis, aData, Dup, Retain, Release, Compare) \
	_im_AORP_UDCONTEXT64_Init( \
		aThis, AorpK_HINT_POINTER, udc_udata, aData, \
		Dup, Retain, Release, Compare)

# define AORP_UDCONTEXT128_InitPtr(aThis, aData, Dup, Retain, Release, Compare) \
	_im_AORP_UDCONTEXT128_Init( \
		aThis, AorpK_HINT_POINTER, udc_udata, aData, \
		Dup, Retain, Release, Compare)

/*
 * AORP_UDCONTEXT_InitPtr0
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitPtr0 AORP_UDCONTEXT128_InitPtr0
# else
#  define AORP_UDCONTEXT_InitPtr0 AORP_UDCONTEXT64_InitPtr0
# endif

# define AORP_UDCONTEXT64_InitPtr0(aThis, aData) \
	AORP_UDCONTEXT64_InitPtr(aThis, aData, NULLF, NULLF, NULLF, NULLF)

# define AORP_UDCONTEXT128_InitPtr0(aThis, aData) \
	AORP_UDCONTEXT128_InitPtr(aThis, aData, NULLF, NULLF, NULLF, NULLF)

/*
 * AORP_UDCONTEXT_InitInt32
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitInt32 AORP_UDCONTEXT128_InitInt32
# else
#  define AORP_UDCONTEXT_InitInt32 AORP_UDCONTEXT64_InitInt32
# endif

# define AORP_UDCONTEXT64_InitInt32(aThis, aData) \
	_im_AORP_UDCONTEXT64_Init( \
		aThis, AorpK_HINT_INT32, udc_i32, aData, NULLF, NULLF, NULLF, NULLF)

# define AORP_UDCONTEXT128_InitInt32(aThis, aData) \
	_im_AORP_UDCONTEXT128_Init( \
		aThis, AorpK_HINT_INT32, udc_i32, aData, NULLF, NULLF, NULLF, NULLF)

/*
 * AORP_UDCONTEXT_InitInt64
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitInt64 AORP_UDCONTEXT128_InitInt64
# else
#  define AORP_UDCONTEXT_InitInt64 AORP_UDCONTEXT64_InitInt64
# endif

# define AORP_UDCONTEXT64_InitInt64(aThis, aData) \
	_im_AORP_UDCONTEXT64_Init( \
		aThis, AorpK_HINT_INT64, udc_i64, aData, NULLF, NULLF, NULLF, NULLF)

# define AORP_UDCONTEXT128_InitInt64(aThis, aData) \
	_im_AORP_UDCONTEXT128_Init( \
		aThis, AorpK_HINT_INT64, udc_i64, aData, NULLF, NULLF, NULLF, NULLF)

/*
 * AORP_UDCONTEXT_InitFloat32
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitFloat32 AORP_UDCONTEXT128_InitFloat32
# else
#  define AORP_UDCONTEXT_InitFloat32 AORP_UDCONTEXT64_InitFloat32
# endif

# define AORP_UDCONTEXT64_InitFloat32(aThis, aData) \
	_im_AORP_UDCONTEXT64_Init( \
		aThis, AorpK_HINT_FLOAT32, udc_f32, aData, NULLF, NULLF, NULLF, NULLF)

# define AORP_UDCONTEXT128_InitFloat32(aThis, aData) \
	_im_AORP_UDCONTEXT128_Init( \
		aThis, AorpK_HINT_FLOAT32, udc_f32, aData, NULLF, NULLF, NULLF, NULLF)

/*
 * AORP_UDCONTEXT_InitFloat64
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitFloat64 AORP_UDCONTEXT128_InitFloat64
# else
#  define AORP_UDCONTEXT_InitFloat64 AORP_UDCONTEXT64_InitFloat64
# endif

# define AORP_UDCONTEXT64_InitFloat64(aThis, aData) \
	_im_AORP_UDCONTEXT64_Init( \
		aThis, AorpK_HINT_FLOAT64, udc_f64, aData, NULLF, NULLF, NULLF, NULLF)

# define AORP_UDCONTEXT128_InitFloat128(aThis, aData) \
	_im_AORP_UDCONTEXT128_Init( \
		aThis, AorpK_HINT_FLOAT128, udc_f128, aData, NULLF, NULLF, NULLF, NULLF)

/*
 * AORP_UDCONTEXT_InitStr
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitStr AORP_UDCONTEXT128_InitStr
# else
#  define AORP_UDCONTEXT_InitStr AORP_UDCONTEXT64_InitStr
# endif

# define AORP_UDCONTEXT64_InitStr(aThis, aData, Dup, Retain, Release, Compare) \
	AORP_UDCONTEXT64_InitPtr(aThis, aData, Dup, Retain, Release, Compare)

# define AORP_UDCONTEXT128_InitStr(aThis, aData, Dup, Retain, Release, Compare) \
	AORP_UDCONTEXT128_InitPtr(aThis, aData, Dup, Retain, Release, Compare)

/*
 * AORP_UDCONTEXT_InitWcs
 */
# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_InitWcs AORP_UDCONTEXT128_InitWcs
# else
#  define AORP_UDCONTEXT_InitWcs AORP_UDCONTEXT64_InitWcs
# endif

# define AORP_UDCONTEXT64_InitWcs(aThis, aData, Dup, Retain, Release, Compare) \
	AORP_UDCONTEXT64_InitPtr(aThis, aData, Dup, Retain, Release, Compare)

# define AORP_UDCONTEXT128_InitWcs(aThis, aData, Dup, Retain, Release, Compare) \
	AORP_UDCONTEXT128_InitPtr(aThis, aData, Dup, Retain, Release, Compare)

/* NOT FOR YOUR BUSINESS @{ */
# define _im_AORP_UDCONTEXT64_Init(aThis, aHint, _TField, aData, Dup, Retain, Release, Compare) \
	do { \
		(aThis)->udc_hint = aHint; \
		(aThis)->udc_reserved = 0; \
		(aThis)->u._TField = (aData); \
		(aThis)->udc_dopv.ud_dup2 = (aorp_dup2_cb)(Dup); \
		(aThis)->udc_dopv.ud_retain = (aorp_retain_cb)(Retain); \
		(aThis)->udc_dopv.ud_release = (aorp_release_cb)(Release); \
		(aThis)->udc_dopv.ud_compare = (aorp_compare_cb)(Compare); \
	} while (0)

# define _im_AORP_UDCONTEXT128_Init(aThis, aHint, _TField, aData, Dup, Retain, Release, Compare) \
	do { \
		(aThis)->udc_hint = aHint; \
		(aThis)->udc_reserved = 0; \
		(aThis)->u._TField = (aData); \
		(aThis)->udc_dopv.ud_dup2 = (aorp_dup2_cb)(Dup); \
		(aThis)->udc_dopv.ud_retain = (aorp_retain_cb)(Retain); \
		(aThis)->udc_dopv.ud_release = (aorp_release_cb)(Release); \
		(aThis)->udc_dopv.ud_compare = (aorp_compare_cb)(Compare); \
	} while (0)
/* @} */
/* @} */

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_Copy AORP_UDCONTEXT128_Copy
#  define AORP_UDCONTEXT_Copy2 AORP_UDCONTEXT128_Copy2
# else
#  define AORP_UDCONTEXT_Copy AORP_UDCONTEXT64_Copy
#  define AORP_UDCONTEXT_Copy2 AORP_UDCONTEXT64_Copy2
# endif

# define AORP_UDCONTEXT64_Copy(aStatusRef, aSource, aDest, Flags, anErrPtr) \
	AORP_UDCONTEXT64_Copy2(aStatusRef, aSource, aDest, Flags, NULL, anErrPtr)

# define AORP_UDCONTEXT128_Copy(aStatusRef, aSource, aDest, Flags, anErrPtr) \
	AORP_UDCONTEXT128_Copy2(aStatusRef, aSource, aDest, Flags, NULL, anErrPtr)

# define AORP_UDCONTEXT64_Copy2(aStatusRef, aSource, aDest, Flags, aParm, anErrPtr) \
	do { \
		AORP_UDVARIANT64_Copy(aStatusRef, &aorp_udcontext64_p(aSource)->udc_dopv, aSource, aDest, Flags, aParm, anErrPtr); \
		if (BoS_SUCCESS(aStatusRef)) \
			aorp_udcontext64_p(aDest)->udc_dopv = aorp_udcontext64_p(aSource)->udc_dopv; \
	} while (0)

# define AORP_UDCONTEXT128_Copy2(aStatusRef, aSource, aDest, Flags, aParm, anErrPtr) \
	do { \
		AORP_UDVARIANT128_Copy(aStatusRef, &aorp_udcontext128_p(aSource)->udc_dopv, aSource, aDest, Flags, aParm, anErrPtr); \
		if (BoS_SUCCESS(aStatusRef)) \
			aorp_udcontext128_p(aDest)->udc_dopv = aorp_udcontext128_p(aSource)->udc_dopv; \
	} while (0)

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_RetainData AORP_UDCONTEXT128_RetainData
# else
#  define AORP_UDCONTEXT_RetainData AORP_UDCONTEXT64_RetainData
# endif

# define AORP_UDCONTEXT64_RetainData(aStatusRef, aThis, anErrPtr) \
	AORP_UDVARIANT64_RetainData(aStatusRef, &aorp_udcontext64_p(aThis)->udc_dopv, aThis, anErrPtr)

# define AORP_UDCONTEXT128_RetainData(aStatusRef, aThis, anErrPtr) \
	AORP_UDVARIANT128_RetainData(aStatusRef, &aorp_udcontext128_p(aThis)->udc_dopv, aThis, anErrPtr)

/* ------------------------------------------------------------------------- */

# if (BO_USE_UDVARIANT_VERSION == 128)
#  define AORP_UDCONTEXT_ReleaseData AORP_UDCONTEXT128_ReleaseData
#  define _AORP_UDCONTEXT_ReleaseData _AORP_UDCONTEXT128_ReleaseData
# else
#  define AORP_UDCONTEXT_ReleaseData AORP_UDCONTEXT64_ReleaseData
#  define _AORP_UDCONTEXT_ReleaseData _AORP_UDCONTEXT64_ReleaseData
# endif

# define AORP_UDCONTEXT64_ReleaseData(aStatusRef, aThis, Flags, anErrPtr) \
	AORP_UDVARIANT64_ReleaseData(aStatusRef, &aorp_udcontext64_p(aThis)->udc_dopv, aThis, Flags, anErrPtr)

# define _AORP_UDCONTEXT64_ReleaseData(aStatusRef, aThis, Flags, anErrPtr) \
	_AORP_UDVARIANT64_ReleaseData(aStatusRef, &aorp_udcontext64_p(aThis)->udc_dopv, aThis, Flags, anErrPtr)

# define AORP_UDCONTEXT128_ReleaseData(aStatusRef, aThis, Flags, anErrPtr) \
	AORP_UDVARIANT128_ReleaseData(aStatusRef, &aorp_udcontext128_p(aThis)->udc_dopv, aThis, Flags, anErrPtr)

# define _AORP_UDCONTEXT128_ReleaseData(aStatusRef, aThis, Flags, anErrPtr) \
	_AORP_UDVARIANT128_ReleaseData(aStatusRef, &aorp_udcontext128_p(aThis)->udc_dopv, aThis, Flags, anErrPtr)

/* ========================================================================= */
# pragma mark -
/* ========================================================================= */

/*
 * Comparable predicate
 */

typedef aorp_compare_cb aorp_cmppred_cb;

struct aorp_comparable_pred {
	aorp_hint_x       cmp_hint;     /* AorpK_HINT_CMPPRED */
	u_int16_t         cmp_reserved; /* should be 0 */
	aorp_cmppred_cb   cmp_pred;
};

/*
 * Foreach predicate
 */

struct aorp_foreach_pred;

/*
 * BoS_OK: success, continue iteration
 * BoS_NORMAL: success, stop iteration
 * BoS_ERR: error, stop iteration
 */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_feachpred_cb)(
# if __myc_require(prototypes)
	bo_pointer_t , /* aorp_udvariant *, aorp_object_t, ... */
	struct aorp_foreach_pred *
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

struct aorp_foreach_pred {
	aorp_hint_x        fe_hint;     /* AorpK_HINT_FOREACHPRED */
	u_int16_t          fe_reserved; /* should be 0 */
	aorp_feachpred_cb  fe_pred;
};

/* ============================================================================
 * ERROR
 */
# pragma mark -

struct aorp_error {
	char const   *func;
	char const   *file;
	long          line;
	bo_status_t   status; /* Bo error code + domain */
	syserrcode_t  syserr; /* @todo: implement; POSIX, windows, MacOS, ... */
	unsigned      msgidx;
	unsigned      bufsz;
	char         *msg;
};
# define aorp_error_p(x) ((struct aorp_error *)(x))

# define AORP_ERROR_ZERO_INITIALIZER { NULL, NULL, 0, 0, 0, 0, 0, NULL}

# define _AORP_ERROR_Init(anErrPtr,aBufsz,aBuffer) \
	do { \
		aorp_error_p(anErrPtr)->func = NULL; \
		aorp_error_p(anErrPtr)->file = NULL; \
		aorp_error_p(anErrPtr)->line = 0; \
		aorp_error_p(anErrPtr)->status = 0; \
		aorp_error_p(anErrPtr)->syserr = 0; \
		aorp_error_p(anErrPtr)->msgidx = 0; \
		aorp_error_p(anErrPtr)->bufsz = aBufsz; \
		aorp_error_p(anErrPtr)->msg = aBuffer; \
	} while (0)


# ifndef AorpL_ERROR_MSGBUF_MIN
#  define AorpL_ERROR_MSGBUF_MIN 256
# endif

struct aorp_error256 {
	char const   *func;
	char const   *file;
	long          line;
	bo_status_t   status;
	syserrcode_t  syserr;
	unsigned      msgidx;
	unsigned      bufsz;
	char         *msg;
	char          msgbuf[AorpL_ERROR_MSGBUF_MIN];
};
# define aorp_error256_p(x) ((struct aorp_error256 *)(x))

# define AORP_ERROR256_ZERO_INITIALIZER { NULL, NULL, 0, 0, 0, 0, NULL, {0}}

# define _AORP_ERROR256_Init(anErrPtr) \
	do { \
		aorp_error256_p(anErrPtr)->func = NULL; \
		aorp_error256_p(anErrPtr)->file = NULL; \
		aorp_error256_p(anErrPtr)->line = 0; \
		aorp_error256_p(anErrPtr)->status = 0; \
		aorp_error256_p(anErrPtr)->syserr = 0; \
		aorp_error256_p(anErrPtr)->msgidx = 0; \
		aorp_error256_p(anErrPtr)->bufsz = AorpL_ERROR_MSGBUF_MIN; \
		aorp_error256_p(anErrPtr)->msg = aorp_error256_p(anErrPtr)->msgbuf; \
	} while (0)


# define _AORP_ERROR_Domain(anErrPtr) \
	BoSTATUS_DOMAIN(aorp_error_p(anErrPtr)->status)

# define AORP_ERROR_SetDomain(anErrPtr,aDomain) \
	do { \
		struct aorp_error *errorSetDomainTmp; \
		if (NULL != (errorSetDomainTmp = aorp_error_p(anErrPtr))) { \
			BoSTATUS_SET_DOMAIN(errorSetDomainTmp->status,aDomain); \
		} \
	} while (0)

# define _AORP_ERROR_SetDomain(anErrPtr,aDomain) \
	do { \
		struct aorp_error *errorSetDomainTmp; \
		errorSetDomainTmp = aorp_error_p(anErrPtr); \
		BoSTATUS_SET_DOMAIN(errorSetDomainTmp->status,aDomain); \
	} while (0)

# define _AORP_ERROR_Code(anErrPtr) \
	BoSTATUS_CODE(aorp_error_p(anErrPtr)->status)

# define AORP_ERROR_SetCode(anErrPtr,aCode) \
	do { \
		struct aorp_error *errorSetCodeTmp; \
		if (NULL != (errorSetCodeTmp = aorp_error_p(anErrPtr))) { \
			BoSTATUS_SET_CODE(errorSetCodeTmp->status,aCode); \
		} \
	} while (0)

# define _AORP_ERROR_SetCode(anErrPtr,aCode) \
	do { \
		struct aorp_error *errorSetCodeTmp; \
		errorSetCodeTmp = aorp_error_p(anErrPtr); \
		BoSTATUS_SET_CODE(errorSetCodeTmp->status,aCode); \
	} while (0)

/* ============================================================================
 * DESCRIPTORS
 */
# pragma mark -

/* "generic" descriptor */
struct aorp_descriptor_s {
	int         dsc_dtyp;
};
typedef bo_pointer_t /* struct aorp_descriptor_s* */ aorp_descriptor_t;

/* descriptor of list of descriptors */
struct aorp_dslist_descriptor_s {
	int         dsl_dtyp; /* AorpK_DSLIST_DESCRIPTOR */    /* in */

	size_t      dsl_rq_count;                              /* in */
	size_t      dsl_actual_count;                          /* out */
	size_t      dsl_total_count;                           /* out */
	int         dsl_items[FAM_LENGTH];                     /* out */
};

struct aorp_version_descriptor_s {
	int         vrd_dtyp; /* AorpK_VERSION_DESCRIPTOR */   /* in */

	u_int8_t    vrd_major_version;                         /* out */
	u_int8_t    vrd_minor_version;                         /* out */
	u_int16_t   vrd_teeny_version;                         /* out */

# ifndef AorpL_VERSION_BUILD_SIZE_MAX
#  define AorpL_VERSION_BUILD_SIZE_MAX (64) /* Including the terminating NUL character */
# endif
	char        vrd_build[AorpL_VERSION_BUILD_SIZE_MAX];   /* out */
};

/* @seealso: <Bo/libc/Limits.h> */
# ifndef BoL_LOCALE_LEN_MAX
#  define BoL_LOCALE_LEN_MAX (32) /* Including the terminating NUL character */
# endif

struct aorp_locales_descriptor_s {
	int         lcd_dtyp; /* AorpK_LOCALES_DESCRIPTOR */   /* in */

	size_t      lcd_rq_count;                              /* in */
	size_t      lcd_actual_count;                          /* out */
	size_t      lcd_total_count;                           /* out */
	char        lcd_items[FAM_LENGTH][BoL_LOCALE_LEN_MAX]; /* out */
};

struct aorp_text_descriptor_s {
	int         txd_dtyp; /* AorpK_TEXT_DESCRIPTOR */      /* in */

	size_t      txd_rq_bufsz;                              /* in */
	char       *txd_rq_locale;                             /* in */
	char        txd_actual_locale[BoL_LOCALE_LEN_MAX];     /* out */
	size_t      txd_actual_datasz;                         /* out */
	size_t      txd_total_datasz;                          /* out */
	char        txd_text[FAM_LENGTH];                      /* out */
};

#endif /* !__BO_AORP_TYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
