/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * TypesP.h	(V.Sokur)
 * include/Bo/aorp/TypesP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_AORP_TYPES__
# include <Bo/aorp/Types.h>
#endif
#ifndef __BO_MEMMAN_TYPES__
# include <Bo/memman/Types.h>
#endif

#ifndef __BO_AORP_TYPES_P__
# define __BO_AORP_TYPES_P__

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/*
 * Forward declarations
 */
struct aorp_aobject;
struct aorp_cobject;
struct aorp_mobject;
struct aorp_sobject;
struct aorp_closure;
struct aorp_operation;
struct aorp_ops;      /* operation set */
struct aorp_se;       /* service-element */
struct aorp_ses;      /* service-element set */

/*
 * AORP module
 */

# ifndef AORP_MODULE_PRINCIPAL_NAME
#  define AORP_MODULE_PRINCIPAL_NAME "_G_aorp_module"
# endif

# ifndef AORP_MODULE_SEARCH_PATH_ENV_NAME
#  define AORP_MODULE_SEARCH_PATH_ENV_NAME "AORP_MODULE_PATH"
# endif


/* Element of a service-set list */
typedef bo_cdllelem_t aorp_selist_elem_t;

# define AORP_SELIST_ELEM_ZERO_INITIALIZER BO_CDLLELEM_ZERO_INITIALIZER

/* Service-set list */
struct aorp_selist_s {
	_Bool           inited_;
	bo_cpurwlock_t  rwlock_;
	bo_cdllist_t    list_;
};
typedef struct aorp_selist_s aorp_selist_t;

# define AORP_SELIST_ZERO_INITIALIZER \
	{ false, BO_CPURWLOCK_ZERO_INITIALIZER, BO_CDLLIST_ZERO_INITIALIZER }

# define AORP_SELIST1_ZERO_INITIALIZER BO_CDLLIST_ZERO_INITIALIZER


/* Module object list */
struct aorp_modlist_s {
	_Bool           inited_;
	bo_cpurwlock_t  rwlock_;
	bo_seqlist_t    list_;
};
typedef struct aorp_modlist_s aorp_modlist_t;

# define AORP_MODLIST_ZERO_INITIALIZER \
	{ false, BO_CPURWLOCK_ZERO_INITIALIZER, BO_SEQLIST_ZERO_INITIALIZER }

# define AORP_MODLIST1_ZERO_INITIALIZER BO_SEQLIST_ZERO_INITIALIZER

/*
 * ODB
 */

struct aorp_odb_s {
	_Bool           inited_;
	bo_cpurwlock_t  rwlock_;
	bo_rtbtree_t    tree_;
};
typedef struct aorp_odb_s aorp_odb_t;

# define AORP_ODB_ZERO_INITIALIZER \
	{ false, BO_CPURWLOCK_ZERO_INITIALIZER, BO_RTBTREE_ZERO_INITIALIZER }

# define AORP_ODB1_ZERO_INITIALIZER BO_RTBTREE_ZERO_INITIALIZER


struct aorp_odbelem_s {
	aorp_oid_t          oid_[AorpK_OID_LEN];
	bo_rtbtelem_t       btelem_;
	struct aorp_odb_s  *db_;
};
typedef struct aorp_odbelem_s aorp_odbelem_t;

# define AORP_ODBELEM_ZERO_INITIALIZER { {0}, BO_RTBTELEM_ZERO_INITIALIZER, NULL }
# define AORP_ODBELEM1_ZERO_INITIALIZER BO_RTBTELEM_ZERO_INITIALIZER


struct aorp_ndbelem_s {
	char const         *name_;
	bo_rtbtelem_t       btelem_;
	struct aorp_odb_s  *db_;
};
typedef struct aorp_ndbelem_s aorp_ndbelem_t;

# define AORP_NDBELEM_ZERO_INITIALIZER { NULL, BO_RTBTELEM_ZERO_INITIALIZER, NULL }
# define AORP_NDBELEM1_ZERO_INITIALIZER BO_RTBTELEM_ZERO_INITIALIZER


typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_operation_fn)(
# if __myc_require(prototypes)
	struct aorp_closure *aClosure,
	struct aorp_error *anErrPtr,
	...
# endif
	) __attribute__((mcalltype, nonnull(1)));

# define AORP_STD_SIGNATURE _AORP_SIGN_POINTER "," _AORP_SIGN_POINTER

/*
 * OPERATION SIGNATURE
 * @{
 */

/* modifiers */
# define _AORP_SMOD_POINTER "^"
# define _AORP_SMOD_VECTOR "["
# define _AORP_SMOD_UNSIGNED "+"

#if 0
# define _AORP_SIGN_ANY "a"        /* memory block */
#endif

# define _AORP_SIGN_VOID "v"
# define _AORP_SIGN_CHAR "c"       /* int8_t */
# define _AORP_SIGN_WCHAR "w"
# define _AORP_SIGN_BYTE "b"       /* u_int8_t */
# define _AORP_SIGN_SHORT "h"      /* int16_t */
# define _AORP_SIGN_INT "i"        /* int32_t */
# define _AORP_SIGN_LONG "l"       /* int64_t */
# define _AORP_SIGN_INTPTR "j"     /* intptr_t */
# define _AORP_SIGN_FLOAT "f"
# define _AORP_SIGN_DOUBLE "d"

# define _AORP_SIGN_POINTER "P"
# define _AORP_SIGN_CSTRING "C"
# define _AORP_SIGN_WCSTRING "W"

# define _AORP_SIGN_SIZE_T "u"     /* u_int32_t/u_int64_t */
# define _AORP_SIGN_SSIZE_T "s"    /* int32_t/int64_t */
# define _AORP_SIGN_OFF_T "o"      /* int32_t/int64_t */
# define _AORP_SIGN_TIME_T "t"     /* int32_t/int64_t */
# define _AORP_SIGN_INDEX_T "n"    /* int32_t/int64_t */
# define _AORP_SIGN_FLAGS_T "F"    /* u_int32_t */

# define _AORP_SIGN_VA_LIST "V"
# define _AORP_SIGN_VA_ELLIPSIS "."

# define _AORP_SIGN_FUNCTION "*"

/*
a[ ~ P
*/

/* @} */


/*
 * CLOSURE
 */

struct aorp_closure {
	aorp_object_t    acceptor_;
	aorp_object_t    provider_;
	bo_pointer_t     pod_;
	struct aorp_ops *ops_;
	int32_t          opcode_;
};

# define aorp_closure_p(x) ((struct aorp_closure *)(x))

# define AORP_CLOSURE_Acceptor(x) (aorp_closure_p(x)->acceptor_)
# define AORP_CLOSURE_Provider(x) (aorp_closure_p(x)->provider_)
# define AORP_CLOSURE_POD(x) (aorp_closure_p(x)->pod_)
# define AORP_CLOSURE_POD_(x) ((_A_self_pod *)AORP_CLOSURE_POD(x))

# define AORP_CLOSURE_Ops(x) (aorp_closure_p(x)->ops_)
# define AORP_CLOSURE_ServiceID(x) AORP_OPS_ServiceID(AORP_CLOSURE_Ops(x))
# define AORP_CLOSURE_ServiceKey(x) AORP_OPS_ServiceKey(AORP_CLOSURE_Ops(x))
# define AORP_CLOSURE_ServiceName(x) AORP_OPS_ServiceName(AORP_CLOSURE_Ops(x))

# define AORP_CLOSURE_Opcode(x) (aorp_closure_p(x)->opcode_)

# define AORP_CLOSURE_Op(x,anOpcode) AORP_OPS_Op(AORP_CLOSURE_Ops(x),anOpcode)
# define AORP_CLOSURE_Oprname(x,anOpcode) AORP_OP_Rname(AORP_CLOSURE_Op(x,anOpcode))
# define AORP_CLOSURE_Ophname(x,anOpcode) AORP_OP_Hname(AORP_CLOSURE_Op(x,anOpcode))
# define AORP_CLOSURE_Opimpl(x,anOpcode) AORP_OP_Impl(AORP_CLOSURE_Op(x,anOpcode))
# define AORP_CLOSURE_Op_(x) AORP_CLOSURE_Op(x,AORP_CLOSURE_Opcode(x))
# define AORP_CLOSURE_Oprname_(x) AORP_CLOSURE_Oprname(x,AORP_CLOSURE_Opcode(x))
# define AORP_CLOSURE_Ophname_(x) AORP_CLOSURE_Ophname(x,AORP_CLOSURE_Opcode(x))
# define AORP_CLOSURE_Opimpl_(x) AORP_CLOSURE_Opimpl(x,AORP_CLOSURE_Opcode(x))

/*
 * OBJECT
 */

/*
 * RTXI (Run Time Extra Info) (.rtxi_)
 */
# define _AorpF_RTXI_EXTRA_SIZE 0x01
# define _AorpF_RTXI_FOREIGN_MEMORY 0x02
# define _AorpF_RTXI_POOL_MEMORY 0x04
# define _AorpF_RTXI_CTOR_INITED 0x08

# define _AorpF_RTXI_HAS_SELIST 0x80
# define _AorpF_RTXI_AOBJECT 0x0100
# define _AorpF_RTXI_DOBJECT 0x0200
# define _AorpF_RTXI_COBJECT 0x0400
# define _AorpF_RTXI_OPS 0x0800
# define _AorpF_RTXI_MOBJECT 0x1000
# ifndef _AorpF_RTXI_CUSTOM
#  define _AorpF_RTXI_CUSTOM 0x8000
# endif

# define _AORP_RTXI_AOBJECT_FLAGS (_AorpF_RTXI_AOBJECT)
# define _AORP_RTXI_DOBJECT_FLAGS (_AORP_RTXI_AOBJECT_FLAGS | _AorpF_RTXI_HAS_SELIST | _AorpF_RTXI_DOBJECT)
# define _AORP_RTXI_COBJECT_FLAGS (_AORP_RTXI_DOBJECT_FLAGS | _AorpF_RTXI_COBJECT)
# define _AORP_RTXI_OPS_FLAGS (_AORP_RTXI_DOBJECT_FLAGS | _AorpF_RTXI_OPS)
# define _AORP_RTXI_MOBJECT_FLAGS (_AORP_RTXI_COBJECT_FLAGS | _AorpF_RTXI_MOBJECT)

# define _AORP_HAS_SELIST(x) \
	(_AorpF_RTXI_HAS_SELIST & aorp_object_p(x)->reserved_)

# define _AORP_IS_AOBJECT(x) \
	(_AorpF_RTXI_AOBJECT & aorp_object_p(x)->reserved_)

# define _AORP_IS_DOBJECT(x) \
	(_AorpF_RTXI_DOBJECT & aorp_object_p(x)->reserved_)

# define _AORP_IS_COBJECT(x) \
	(_AorpF_RTXI_COBJECT & aorp_object_p(x)->reserved_)

# define _AORP_IS_OPS(x) \
	(_AorpF_RTXI_OPS & aorp_object_p(x)->reserved_)

# define _AORP_IS_MOBJECT(x) \
	(_AorpF_RTXI_MOBJECT & aorp_object_p(x)->reserved_)

# define _AORP_IS_CUSTOM(x) \
	(_AorpF_RTXI_CUSTOM & aorp_object_p(x)->reserved_)

/*
 * Object Extra Header
 * The bit _AorpF_RTXI_EXTRA_SIZE & .rtxi_
 */

struct aorp_extra_header {
	size_t extrasz;
};

/*
 * S-OBJECT
 * Designed to be a stack object.
 */

struct aorp_sobject {
	aorp_hint_x          hint_;
	u_int16_t            rtxi_;
	struct aorp_closure  closure_;
};

# define aorp_sobject_p(x) ((struct aorp_sobject *)(x))

# define AORP_SOBJECT_Acceptor(x) AORP_CLOSURE_Acceptor(AORP_SOBJECT_Getclosure(x))
# define AORP_SOBJECT_Provider(x) AORP_CLOSURE_Provider(AORP_SOBJECT_Getclosure(x))
# define AORP_SOBJECT_Ops(x) AORP_CLOSURE_Ops(AORP_SOBJECT_Getclosure(x))
# define AORP_SOBJECT_Closure(x) (aorp_sobject_p(x)->closure_)
# define AORP_SOBJECT_Getclosure(x) (&AORP_SOBJECT_Closure(x))

/*
 * A-OBJECT
 */

struct aorp_aobject {
	/*
	 * Inited by aorp-runtime when [object] creating.
	 */

	aorp_hint_x    hint_;
	u_int16_t      rtxi_;
	aorp_object_t  isa_;  /* a-object, c-object, m-object */

	/* data */
};

# define aorp_aobject_p(x) ((struct aorp_aobject *)(x))

/*
 * D-OBJECT {\dummy}
 */

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_object_bootstrap_fn)(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_object_halt_fn)(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_object_configure_fn)(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1)));

/* RTSI (Run Time State Info) (.rtsi_) */
# define _AorpF_RTSI_PRE_CONFIGURED 0x01
# define _AorpF_RTSI_POST_CONFIGURED 0x02
# define _AorpF_RTSI_ONCE_INITED 0x04 /* c-object */
# define _AorpF_RTSI_EXPLICITLY_LOADED 0x08 /* m-object */

struct aorp_dobject {
	/*
	 * Inited by aorp-runtime when [object] bootstraping.
	 */

	/* aorp_aobject @{ */
	aorp_hint_x                    hint_;
	u_int16_t                      rtxi_;
	aorp_object_t                  isa_;
	/* @} */

	unsigned                       rtsi_;
	bo_refc_t                      refc_;
	bo_cpurwlock_t                 rwlock_;

	/*
	 * Static inited [by the developer].
	 */
	bo_version_t                   aorp_version_;
	char const                    *long_key_,
	                              *short_key_,
	                              *name_;
	aorp_odbelem_t                 dbelem_;

	/*
	 * Static inited [by the developer].
	 * user-common-block
	 */
	struct {
		aorp_object_bootstrap_fn       bootstrap_;
		aorp_object_halt_fn            halt_;
		aorp_object_configure_fn       configure_;
	} ucb;

	/* data */
};

# define aorp_dobject_p(x) ((struct aorp_dobject *)(x))

# define AORP_DOBJECT_Version(x) (aorp_dobject_p(x)->aorp_version_)

# define AORP_DOBJECT_Longkey(x) (aorp_dobject_p(x)->long_key_)
# define AORP_DOBJECT_Shortkey(x) (aorp_dobject_p(x)->short_key_)
# define AORP_DOBJECT_Name(x) (aorp_dobject_p(x)->name_)
# define AORP_DOBJECT_Oid(x) (aorp_dobject_p(x)->dbelem_.oid_)
# define AORP_DOBJECT_Category(x) AORP_OID_Category(AORP_DOBJECT_Oid(x))
# define AORP_DOBJECT_PrimID(x) AORP_OID_PrimID(AORP_DOBJECT_Oid(x))

# define AORP_DOBJECT_DBElem(x) (&(aorp_dobject_p(x)->dbelem_.btelem_))
# define AORP_DOBJECT_DBElemUserinfo(x) (AORP_DOBJECT_DBElem(x)->rtbte_user)
# define AORP_DOBJECT_DB(x) (aorp_dobject_p(x)->dbelem_.db_)

# define AORP_DOBJECT_Refc(x) (aorp_dobject_p(x)->refc_)

/*
 * C-OBJECT
 */

struct bo_zmalloc;

struct aorp_cobject {
	struct aorp_dobject  basic_;

	/*
	 * Inited by aorp-runtime when [object] bootstraping.
	 *
	 * Developer is responsible to construct the list of service-set.
	 * It may be done when self-bootstraping
	 * (.bootstrap_ procedure, set by the developer, called by aorp-runtime).
	 */
	aorp_selist_t        selist_;

	/*
	 * Static inited [by developer].
	 */
	size_t               instsz_;

	/* data */
};

# define aorp_cobject_p(x) ((struct aorp_cobject *)(x))

# define AORP_COBJECT_Version(x) AORP_DOBJECT_Version(x)

# define AORP_COBJECT_Oid(x) AORP_DOBJECT_Oid(x)
# define AORP_COBJECT_PrimID(x) AORP_DOBJECT_PrimID(x)
# define AORP_COBJECT_Instc(x) AORP_DOBJECT_Refc(x)
# define AORP_COBJECT_Instsz(x) (aorp_cobject_p(x)->instsz_)

/*
 * M-OBJECT
 */

/* values for .flags_ */
# define AorpF_WEAK_MODRQ 0x01

struct aorp_modrq_ds {
	char const  *modname_;
	u_int32_t    flags_;
	size_t       nalternates_;
	char const **alternates_;
		/* NULL terminated list of alternate module names */
};

struct aorp_modrq_list {
	size_t                modc_;
	struct aorp_modrq_ds  modv_[1];
		/* NULL(.modname_) terminated list of required modules */
};

# define _TAG_AORP_MODRQ_LIST(aModule) __myc_xconcat(aorp_modrq_list_,aModule)
# define _TYPENAME_AORP_MODRQ_LIST(aModule) struct _TAG_AORP_MODRQ_LIST(aModule)
# define _STRUCT_AORP_MODRQ_LIST(aModule,N) \
	_TAG_AORP_MODRQ_LIST(aModule) { \
		size_t                modc_; \
		struct aorp_modrq_ds  modv_[N + 1]; \
	}

typedef struct aorp_modrq_list *
(__myc_tcalltype__(mcall) *aorp_mobject_required_fn)(
# if __myc_require(prototypes)
	struct aorp_mobject *aThis
# endif
	) __attribute__((mcalltype, nonnull(1)));

struct aorp_mobject {
	struct aorp_cobject       basic_;

	/*
	 * Inited by aorp-runtime when [object] loading.
	 */
	bo_refc_t                 openc_;
	bo_dlsysobj_t             dl_self_;

	/*
	 * Inited by aorp-runtime when [object] bootstraping.
	 */
	aorp_modlist_t            prerequisites_;
	aorp_modlist_t            dependencies_;

	/*
	 * Static inited [by the developer].
	 */
	aorp_ndbelem_t            mld_dbelem_;

	/*
	 * Static inited [by the developer].
	 */
	aorp_mobject_required_fn  required_;

	/*
	 * Static inited or when self-bootstraping
	 * (.bootstrap_ procedure, set by the developer, called by aorp-runtime).
	 */
	size_t                    ops_count_;
	struct aorp_ops         **ops_list_;    /* NULL terminated */
	size_t                    class_count_;
	struct aorp_cobject     **class_list_;  /* NULL terminated */

	/* data */
};

# define aorp_mobject_p(x) ((struct aorp_mobject *)(x))

# define AORP_MOBJECT_Version(x) AORP_DOBJECT_Version(x)

# define AORP_MOBJECT_Oid(x) AORP_DOBJECT_Oid(x)
# define AORP_MOBJECT_PrimID(x) AORP_DOBJECT_PrimID(x)
# define AORP_MOBJECT_Refc(x) AORP_DOBJECT_Refc(x)

# define AORP_MOBJECT_MLDDBElem(x) (&(aorp_mobject_p(x)->mld_dbelem_.btelem_))
# define AORP_MOBJECT_MLDDBElemUserinfo(x) (AORP_MOBJECT_MLDDBElem(x)->rtbte_user)
# define AORP_MOBJECT_MLDDB(x) (aorp_mobject_p(x)->mld_dbelem_.db_)
# define AORP_MOBJECT_MLDName(x) (aorp_mobject_p(x)->mld_dbelem_.name_)

# define AORP_MOBJECT_Openc(x) (aorp_mobject_p(x)->openc_)

# define AORP_MOBJECT_Rqmodules(x) \
	((NULLF == aorp_mobject_p(x)->required_) ? NULL : (*aorp_mobject_p(x)->required_)(x))

# define _AORP_MODULE_GUTS \
	struct aorp_cobject       basic_; \
	bo_refc_t                 openc_; \
	bo_dlsysobj_t             dl_self_; \
	aorp_modlist_t            prerequisites_; \
	aorp_modlist_t            dependencies_; \
	aorp_ndbelem_t            mld_dbelem_; \
	aorp_mobject_required_fn  required_; \
	size_t                    ops_count_; \
	struct aorp_ops         **ops_list_; \
	size_t                    class_count_; \
	struct aorp_cobject     **class_list_

/*
 * OPERATION
 */

# define AORP_FOREIGN_OP ((aorp_operation_fn)1)
# define AORP_REDIRECT_OP ((aorp_operation_fn)2)

struct aorp_operation {
	char const          *art_name_;
	char const          *human_name_;
	char const          *signature_;
	aorp_operation_fn    impl_;

	/* used if .impl_ == AORP_FOREIGN_OP or AORP_REDIRECT_OP */
	aorp_oid_t           frgn_opsid_[AorpK_OID_LEN];
	struct aorp_ops     *frgn_ops_;
};

# define aorp_operation_p(x) ((struct aorp_operation *)(x))

# define AORP_OP_Rname(x) (aorp_operation_p(x)->art_name_)
# define AORP_OP_Hname(x) (aorp_operation_p(x)->human_name_)
# define AORP_OP_Impl(x) (aorp_operation_p(x)->impl_)

/*
 * OPS
 */

struct aorp_ops {
	struct aorp_dobject    basic_;

	/*
	 * Static inited or during self-bootstraping via .bootstrap_ procedure
	 * (set by the client), called by aorp-runtime.
	 */

	ssize_t                podsize_;
	size_t                 opc_;
	struct aorp_operation  opv_[4 + 1];
		/* variable length array, 4 - mandatory, last operation - dummy */

# define AorpK_any_init_once 0
# define AorpK_any_fini_once 1
# define AorpK_any_pod_init 2
# define AorpK_any_pod_fini 3
};

enum {
	_AorpK_any_init_once = AorpK_any_init_once,
	_AorpK_any_fini_once = AorpK_any_fini_once,
	_AorpK_any_pod_init = AorpK_any_pod_init,
	_AorpK_any_pod_fini = AorpK_any_pod_fini
};

# define aorp_ops_p(x) ((struct aorp_ops *)(x))

# define AORP_OPS_Version(x) AORP_DOBJECT_Version(x)

# define AORP_OPS_ServiceKey(x) AORP_DOBJECT_Shortkey(x)
# define AORP_OPS_ServiceName(x) AORP_DOBJECT_Name(x)

# define AORP_OPS_Oid(x) AORP_DOBJECT_Oid(x)
# define AORP_OPS_ServiceID(x) AORP_DOBJECT_PrimID(x)
# define AORP_OPS_Attachc(x) AORP_DOBJECT_Refc(x)

# define AORP_OPS_Op(x,anOpcode) (&aorp_ops_p(x)->opv_[(anOpcode)])
# define AORP_OPS_Oprname(x,anOpcode) (AORP_OPS_Op(x,anOpcode)->art_name_)
# define AORP_OPS_Ophname(x,anOpcode) (AORP_OPS_Op(x,anOpcode)->human_name_)
# define AORP_OPS_Opimpl(x,anOpcode) (AORP_OPS_Op(x,anOpcode)->impl_)

# define _TAG_AORP_OPS(aService) __myc_xconcat(aorp_ops_,aService)
# define _TYPENAME_AORP_OPS(aService) struct _TAG_AORP_OPS(aService)
# define _STRUCT_AORP_OPS(aService,N) \
	_TYPENAME_AORP_OPS(aService) { \
		struct aorp_dobject    basic_; \
		ssize_t                podsize_; \
		size_t                 opc_; \
		struct aorp_operation  opv_[N + 1]; \
	}

/*
 * SE
 */

# define AorpK_DATA_PROVIDER_RELDIST (16)
# define AORP_FOREIGN_OPS ((struct aorp_ops *)1)

struct aorp_se {
	aorp_hint_x          hint_;
	u_int16_t            unused_;

	aorp_oid_t           opsid_[AorpK_OID_LEN];

	/* distance from the ops to a protocol data provider */
	unsigned             reldist_;
	ssize_t              podoffs_;

	_Bool                once_inited_;

	struct aorp_ops     *ops_;
	/* used if .ops_ == AORP_FOREIGN_OPS */
	aorp_oid_t           frgn_opsid_[AorpK_OID_LEN];
	struct aorp_ops     *frgn_ops_;
};

# define aorp_se_p(x) ((struct aorp_se *)(x))

# define AORP_SE_Opsid(x) (aorp_se_p(x)->opsid_)
# define AORP_SE_Oid(x) AORP_SE_Opsid(x)
# define AORP_SE_Name(x) AORP_OPS_ServiceName(aorp_se_p(x)->ops_)
# define AORP_SE_Id(x) AORP_OID_Field(1,AORP_SE_Opsid(x))

/*
 * SES
 */

struct aorp_ses {
	aorp_hint_x            hint_;
	u_int16_t              unused_;

	struct bo_malloc       self_alloc_;

	aorp_selist_elem_t     lelem_;

	size_t                 sec_;
	struct aorp_se         sev_[1]; /* variable length array */
};

# define aorp_ses_p(x) ((struct aorp_ses *)(x))

# define _TAG_AORP_SES(aName) __myc_xconcat(aorp_ses_,aName)
# define _TYPENAME_AORP_SES(aName) struct _TAG_AORP_SES(aName)
# define _STRUCT_AORP_SES(aName,N) \
	_TYPENAME_AORP_SES(aName) { \
		aorp_hint_x            hint_; \
		u_int16_t              unused_; \
		struct bo_malloc       self_alloc_; \
		aorp_selist_elem_t     lelem_; \
		size_t                 sec_; \
		struct aorp_se         sev_[N + 1]; \
	}

#endif /* !__BO_AORP_TYPES_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
