/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*-
 * Copyright (c) 1989, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Paul Vixie.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD: src/sys/sys/bitstring.h,v 1.5 2005/01/07 02:29:23 imp Exp $
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
* Bitseq.h	(S.Makhmali)
* include/Bo/bitis/Bitseq.h
*
* $Id$
*/

/* ------------------------------------------------------------------------- */

#ifndef __BO_BITIS_BITSEQ__
# define __BO_BITIS_BITSEQ__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Cdefs.h>

/******************************************************************************
 * %% BeginSection: definitions
 */

# if 0/*__myc_exists(bitstring,h)*/
#  include <bitstring.h>
# else
# if 0/*__myc_exists(sys7bitstring,h)*/
#  include <sys/bitstring.h>
# else

typedef	unsigned char bo_bitseq_t;

/* bytes in a bitseq of @nbits bits */
#  define BO_NBBITSEQ(nbits) ((unsigned)((unsigned)(nbits) + 7) >> 3)


/* allocate a bitseq on the stack */
#  define BO_BITSEQ_DECLARE(aName, nbits) ((aName)[BO_NBBITSEQ(nbits)])

/* allocate a bitseq */
#  define BO_BITSEQ_Alloc(nbits) \
	((bo_bitseq_t *)_BoMcalloc((size_t)BO_NBBITSEQ(nbits), sizeof(bo_bitseq_t), NULL))

/* deallocate a bitseq */
#  define BO_BITSEQ_Free(p) _BoMfree(p)

/* NOT FOR YOUR BUSINESS @{ */
/* byte of the bitseq bit is in */
#  define __BITSEQ_Byte(bit) ((unsigned)(bit) >> 3)

/* mask for the bit within its byte */
#  define __BITSEQ_Mask(bit) (1 << ((unsigned)(bit) & 0x7))
/* @} */

/* is bit N of bitseq @aName set? */
#  define BO_BITSEQ_Test(aName, bit) \
	((aName)[__BITSEQ_Byte(bit)] & __BITSEQ_Mask(bit))

# if __myc_support(macro_retval)
#  define BO_BITSEQ_Test_n(aName, aStart, aStop) \
	__extension__ ({ \
		int ___result; \
		register int ___mask1, ___mask2; \
		register bo_bitseq_t const *___name = (aName); \
		register int ___start = (aStart), ___stop = (aStop); \
		register int ___startbyte = __BITSEQ_Byte(___start); \
		register int ___stopbyte = __BITSEQ_Byte(___stop); \
		___mask1 = 0xff << (___start & 0x7); \
		___mask2 = 0xff >> (7 - (___stop & 0x7)); \
		if (___startbyte == ___stopbyte) { \
			___result = (___name[___startbyte] & (___mask1 | ___mask2)); \
		} \
		else { \
			___result = (___name[___startbyte] & ___mask1) | (___name[___stopbyte] & ___mask2); \
			if (!___result) { \
				while (++___startbyte < ___stopbyte) \
					if (___result = (___name[___startbyte] & 0xff)) \
						break; \
			} \
		} \
		/* return */ (___result); \
	})
# else
__myc_extern_inline__ int
BO_BITSEQ_Test_n(register bo_bitseq_t const *aName, int aStart, int aStop)
{
	int result;
	register int mask1, mask2;
	register int startbyte = __BITSEQ_Byte(aStart);
	register int stopbyte = __BITSEQ_Byte(aStop);
	mask1 = 0xff << (aStart & 0x7);
	mask2 = 0xff >> (7 - (aStop & 0x7));
	if (startbyte == stopbyte) {
		result = (aName[startbyte] & (mask1 & mask2));
	}
	else {
		result = (aName[startbyte] & mask1) | (aName[stopbyte] & mask2);
		if (!result) {
			while (++startbyte < stopbyte)
				if (result = (aName[startbyte] & 0xff))
					break;
		}
	}
	return (result);
}
# endif

/* set bit N of bitseq @aName */
#  define BO_BITSEQ_Set(aName, bit) \
	((aName)[__BITSEQ_Byte(bit)] |= __BITSEQ_Mask(bit))

/* set bits [@aStart, @aStop] in bitseq */
#  define BO_BITSEQ_Set_n(aName, aStart, aStop) \
	do { \
		register int ___mask1, ___mask2; \
		register bo_bitseq_t *___name = (aName); \
		register int ___start = (aStart), ___stop = (aStop); \
		register int ___startbyte = __BITSEQ_Byte(___start); \
		register int ___stopbyte = __BITSEQ_Byte(___stop); \
		___mask1 = 0xff << (___start & 0x7); \
		___mask2 = 0xff >> (7 - (___stop & 0x7)); \
		if (___startbyte == ___stopbyte) { \
			___name[___startbyte] |= (___mask1 & ___mask2); \
			break; \
		} \
		___name[___startbyte] |= ___mask1; \
		while (++___startbyte < ___stopbyte) \
			___name[___startbyte] = 0xff; \
		___name[___stopbyte] |= ___mask2; \
	} while (0)

/* clear bit N of bitseq @aName */
#  define BO_BITSEQ_Clear(aName, bit) \
	((aName)[__BITSEQ_Byte(bit)] &= ~__BITSEQ_Mask(bit))

/* clear bits [@aStart, @aStop] in bitseq */
#  define BO_BITSEQ_Clear_n(aName, aStart, aStop) \
	do { \
		register int ___mask1, ___mask2; \
		register bo_bitseq_t *___name = (aName); \
		register int ___start = (aStart), ___stop = (aStop); \
		register int ___startbyte = __BITSEQ_Byte(___start); \
		register int ___stopbyte = __BITSEQ_Byte(___stop); \
		___mask1 = 0xff >> (8 - (___start & 0x7)); \
		___mask2 = 0xff << ((___stop & 0x7) + 1); \
		if (___startbyte == ___stopbyte) { \
			___name[___startbyte] &= (___mask1 | ___mask2); \
			break; \
		} \
		___name[___startbyte] &= ___mask1; \
		while (++___startbyte < ___stopbyte) \
			___name[___startbyte] = 0; \
		___name[___stopbyte] &= ___mask2; \
	} while (0)


/* find first bit set in @aName */
# if __myc_support(macro_retval)
#  define BO_BITSEQ_Ffs(aName, nbits) \
	__extension__ ({ \
		register bo_bitseq_t const *___name = (aName); \
		register int ___nbits = (nbits), ___result = -1; \
		register int ___stopbyte = __BITSEQ_Byte(___nbits - 1); \
		if (0 < ___nbits) { \
			register int ___byte; \
			for (___byte = 0; ___byte <= ___stopbyte; ++___byte) { \
				if (___name[___byte]) { \
					bo_bitseq_t ___lb; \
					___result = ___byte << 3; \
					for (___lb = ___name[___byte]; !(___lb & 0x1); ___lb >>= 1) \
						++___result; \
					break; \
				} \
			} \
		} \
		if (___nbits <= ___result) \
			___result = -1; \
		/* return */ ___result; \
	})
# else
__myc_extern_inline__ int
BO_BITSEQ_Ffs(register bo_bitseq_t const *aName, int nbits)
{
	register int result = -1;
	register int stopbyte = __BITSEQ_Byte(nbits - 1);
	if (0 < nbits) {
		register int ibyte;
		for (ibyte = 0; ibyte <= stopbyte; ++ibyte) {
			if (aName[ibyte]) {
				bo_bitseq_t lb;
				result = ibyte << 3;
				for (lb = aName[ibyte]; !(lb & 0x1); lb >>= 1)
					++result;
				break;
			}
		}
	}
	if (nbits <= result)
		result = -1;
	return result;
}
# endif

/* find first bit clear in @aName */
# if __myc_support(macro_retval)
#  define BO_BITSEQ_Ffc(aName, nbits) \
	__extension__ ({ \
		register bo_bitseq_t const *___name = (aName); \
		register int ___nbits = (nbits), ___result = -1; \
		register int ___stopbyte = __BITSEQ_Byte(___nbits - 1); \
		if (0 < ___nbits) { \
			register int ___byte; \
			for (___byte = 0; ___byte <= ___stopbyte; ++___byte) { \
				if (___name[___byte] != 0xff) { \
					bo_bitseq_t ___lb; \
					___result = ___byte << 3; \
					for (___lb = ___name[___byte]; (___lb & 0x1); ___lb >>= 1) \
						++___result; \
					break; \
				} \
			} \
		} \
		if (___nbits <= ___result) \
			___result = -1; \
		/* return */ (___result); \
	})
# else
__myc_extern_inline__ int
BO_BITSEQ_Ffc(register bo_bitseq_t const *aName, int nbits)
{
	register int result = -1;
	register int stopbyte = __BITSEQ_Byte(nbits - 1);
	if (0 < nbits) {
		register int ibyte;
		for (ibyte = 0; ibyte <= stopbyte; ++ibyte) {
			if (aName[ibyte] != 0xff) {
				bo_bitseq_t lb;
				result = ibyte << 3;
				for (lb = aName[ibyte]; (lb & 0x1); lb >>= 1)
					++result;
				break;
			}
		}
	}
	if (nbits <= result)
		result = -1;
	return (result);
}
# endif

# endif /* __myc_exists(sys7bitstring,h) */
# endif /* __myc_exists(bitstring,h) */

#endif /* !__BO_BITIS_BITSEQ__ */

/******************************************************************************
* Revision History:
*		$Log$
*
*****************************************************************************/

