/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Funcjmp.h	(V.Sokur)
 * include/Bo/compiler/gcc/_Funcjmp.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_GCC__FUNCJMP__
# define __BO_GCC__FUNCJMP__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# include <Bo/libc/Limits.h>
	/* ULONG_MAX */

/******************************************************************************
 * %% BeginSection: definitions
 */

# define BO_USES_FUNCJMP \
	struct bo_funcjmpbuf *funcjmpbufHead = NULL

# define BO_FUNCJMPBUF_(aVar) \
	struct bo_funcjmpbuf aVar = { funcjmpbufHead, 0 }


struct bo_funcjmpbuf {
	struct bo_funcjmpbuf *next;
	bo_pointer_t          label1,
	                      label2;
	u_int32_t             next_chain;
	bo_pointer_t          retaddr;
};

# define BO_INITFUNCJMP(aVarRef,aTEXT) \
	{ \
		__label__ __myc_xconcat3(_L_funcjmpbuf_,aTEXT,1), \
		          __myc_xconcat3(_L_funcjmpbuf_,aTEXT,2); \
		do { \
			(aVarRef).label1 = &&__myc_xconcat3(_L_funcjmpbuf_,aTEXT,1); \
			(aVarRef).label2 = &&__myc_xconcat3(_L_funcjmpbuf_,aTEXT,2); \
			funcjmpbufHead = &(aVarRef); \
		} while (0)

# define BO_SETFUNCJMP(aTEXT) \
		{ \
			bo_pointer_t funcjmpAddr; \
			u_int32_t funcjmpNextChain; \
			funcjmpAddr = funcjmpbufHead->label2; \
		__myc_xconcat3(_L_funcjmpbuf_,aTEXT,1):

# define BO_ENDFUNCJMP(aTEXT) \
			if ((funcjmpNextChain = funcjmpbufHead->next_chain)) { \
				/* funcjmpbufHead->next_chain = */ --funcjmpNextChain; \
				funcjmpAddr = funcjmpbufHead->retaddr; \
				funcjmpbufHead = funcjmpbufHead->next; \
				if (funcjmpNextChain && funcjmpbufHead) { \
					funcjmpbufHead->next_chain = funcjmpNextChain; \
					funcjmpbufHead->retaddr = funcjmpAddr; \
					funcjmpAddr = funcjmpbufHead->label1; \
				} \
			} \
			goto *funcjmpAddr; \
		__myc_xconcat3(_L_funcjmpbuf_,aTEXT,2): \
			funcjmpbufHead = funcjmpbufHead->next; \
		} \
	}

# define BO_FUNCJMP_EXIT(/* {...} */ aPredicate) \
	BO_FUNCJMP_BREAK(ULONG_MAX,aPredicate)

# define BO_FUNCJMP_BREAK(aNBlocks,/* {...} */ aPredicate) \
	{ \
		__label__ _L_funcjmp_break_addr; \
		funcjmpbufHead->next_chain = (aNBlocks); \
		funcjmpbufHead->retaddr = &&_L_funcjmp_break_addr; \
		goto *funcjmpbufHead->label1; \
	_L_funcjmp_break_addr: \
		aPredicate; \
	}; ((void)0)

#endif /* !__BO_GCC__FUNCJMP__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
