/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Jmptab.h	(V.Sokur)
 * include/Bo/compiler/gcc/_Jmptab.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_GCC__JMPTAB__
# define __BO_GCC__JMPTAB__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/libc/Stddef.h>

/******************************************************************************
 * %% BeginSection: definitions
 */

# ifndef BO_JMPTAB_DEFINED
#  define BO_JMPTAB_DEFINED

#  define __bo_janame(aTabName,anActionName) \
	__myc_xconcat4(_L_jmptab_,aTabName,_,anActionName)

#  define __bo_jatab(aTabName) \
	__myc_xconcat(_s_jmptab_,aTabName)


# if 1

#  define BO_BEGIN_JMPTAB(aTabName,N) \
	static int const __bo_jatab(aTabName)[] = { \
		BO_JALINK_DEFAULT(aTabName)

#  define BO_JALINK_DEFAULT(T) \
		&&__bo_janame(T,default) - &&__bo_janame(T,start)

#  define BO_JALINK(T,A) \
		, &&__bo_janame(T,A) - &&__bo_janame(T,start)

#  define BO_END_JMPTAB(T) };

# else

#  define BO_BEGIN_JMPTAB(aTabName,N) \
	static int __myc_xconcat3(_s_jmptab_,aTabName,_inited) = 0; \
	static int __bo_jatab(aTabName)[((0 == N) ? BoL_JMPTAB_SIZE : N) + 1]; \
	if (!__myc_xconcat3(_s_jmptab_,aTabName,_inited)) { \
		int jmptabActionIdx = 0; \
		BO_JALINK_DEFAULT(aTabName)

#  define BO_JALINK_DEFAULT(T) BO_JALINK(T,default)
#  define BO_JALINK(T,A) \
	__bo_jatab(T)[jmptabActionIdx++] = &&__bo_janame(T,A) - &&__bo_janame(T,start);

#  define BO_END_JMPTAB(T) \
		__myc_xconcat3(_s_jmptab_,T,_inited) = 1; \
	}

# endif /* if 1 */


/* x = [0,N], default: 0 */
#  define BO_JMPSW(T,x) { \
__bo_janame(T,start):; \
	if ((x) < 0 || __myc_namembs(__bo_jatab(T)) <= (size_t)(x)) \
		goto *(&&__bo_janame(T,start) + __bo_jatab(T)[0]); \
	assert(0 <= (x) && (size_t)(x) < __myc_namembs(__bo_jatab(T))); \
	goto *(&&__bo_janame(T,start) + __bo_jatab(T)[(x)]);

#  define BO_JMPCASE_DEFAULT(T) __bo_janame(T,default)
#  define BO_JMPCASE(T,A) __bo_janame(T,A)
#  define BO_JMPBREAKSW(T) goto __bo_janame(T,finish);

#  define BO_JMPENDSW(T) \
__bo_janame(T,finish):; \
	}


/* @{ */
#if 0
#  define BO_JMPTAB_LINK1(T,A) \
	&&__bo_janame(T,A) - &&_L_jmptab_start

#  define BO_JMPTAB_LINK2(T,A2,A1) \
	BO_JMPTAB_LINK1(T,A1) \
	, &&__bo_janame(T,A2) - &&__bo_janame(T,A1)

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK3(T,A3,A2,A1...) \
		BO_JMPTAB_LINK2(T,A2,A1) \
		, &&__bo_janame(T,A3) - &&__bo_janame(T,A2)
#  else
#   define BO_JMPTAB_LINK3(T,A3,A2,...) \
		BO_JMPTAB_LINK2(T,A2,__VA_ARGS__) \
		, &&__bo_janame(T,A3) - &&__bo_janame(T,A2)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK4(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK3(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK4(T,An,An_1,...) \
		BO_JMPTAB_LINK3(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK5(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK4(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK5(T,An,An_1,...) \
		BO_JMPTAB_LINK4(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK6(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK5(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK6(T,An,An_1,...) \
		BO_JMPTAB_LINK5(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK7(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK6(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK7(T,An,An_1,...) \
		BO_JMPTAB_LINK6(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK8(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK7(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK8(T,An,An_1,...) \
		BO_JMPTAB_LINK7(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK9(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK8(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK9(T,An,An_1,...) \
		BO_JMPTAB_LINK8(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK10(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK9(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK10(T,An,An_1,...) \
		BO_JMPTAB_LINK9(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK11(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK10(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK11(T,An,An_1,...) \
		BO_JMPTAB_LINK10(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK12(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK11(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK12(T,An,An_1,...) \
		BO_JMPTAB_LINK11(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK13(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK12(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK13(T,An,An_1,...) \
		BO_JMPTAB_LINK12(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK14(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK13(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK14(T,An,An_1,...) \
		BO_JMPTAB_LINK13(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK15(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK14(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK15(T,An,An_1,...) \
		BO_JMPTAB_LINK14(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK16(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK15(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK16(T,An,An_1,...) \
		BO_JMPTAB_LINK15(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK17(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK16(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK17(T,An,An_1,...) \
		BO_JMPTAB_LINK16(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK18(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK17(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK18(T,An,An_1,...) \
		BO_JMPTAB_LINK17(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK19(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK18(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK19(T,An,An_1,...) \
		BO_JMPTAB_LINK18(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK20(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK19(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK20(T,An,An_1,...) \
		BO_JMPTAB_LINK19(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK21(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK20(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK21(T,An,An_1,...) \
		BO_JMPTAB_LINK20(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK22(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK21(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK22(T,An,An_1,...) \
		BO_JMPTAB_LINK21(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK23(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK22(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK23(T,An,An_1,...) \
		BO_JMPTAB_LINK22(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK24(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK23(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK24(T,An,An_1,...) \
		BO_JMPTAB_LINK23(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK25(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK24(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK25(T,An,An_1,...) \
		BO_JMPTAB_LINK24(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK26(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK25(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK26(T,An,An_1,...) \
		BO_JMPTAB_LINK25(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK27(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK26(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK27(T,An,An_1,...) \
		BO_JMPTAB_LINK26(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK28(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK27(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK28(T,An,An_1,...) \
		BO_JMPTAB_LINK27(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK29(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK28(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK29(T,An,An_1,...) \
		BO_JMPTAB_LINK28(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK30(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK29(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK30(T,An,An_1,...) \
		BO_JMPTAB_LINK29(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK31(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK30(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK31(T,An,An_1,...) \
		BO_JMPTAB_LINK30(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK32(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK31(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK32(T,An,An_1,...) \
		BO_JMPTAB_LINK31(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK33(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK32(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK33(T,An,An_1,...) \
		BO_JMPTAB_LINK32(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK34(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK33(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK34(T,An,An_1,...) \
		BO_JMPTAB_LINK33(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK35(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK34(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK35(T,An,An_1,...) \
		BO_JMPTAB_LINK34(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK36(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK35(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK36(T,An,An_1,...) \
		BO_JMPTAB_LINK35(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK37(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK36(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK37(T,An,An_1,...) \
		BO_JMPTAB_LINK36(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK38(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK37(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK38(T,An,An_1,...) \
		BO_JMPTAB_LINK37(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK39(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK38(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK39(T,An,An_1,...) \
		BO_JMPTAB_LINK38(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK40(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK39(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK40(T,An,An_1,...) \
		BO_JMPTAB_LINK39(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK41(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK40(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK41(T,An,An_1,...) \
		BO_JMPTAB_LINK40(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK42(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK41(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK42(T,An,An_1,...) \
		BO_JMPTAB_LINK41(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK43(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK42(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK43(T,An,An_1,...) \
		BO_JMPTAB_LINK42(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK44(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK43(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK44(T,An,An_1,...) \
		BO_JMPTAB_LINK43(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK45(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK44(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK45(T,An,An_1,...) \
		BO_JMPTAB_LINK44(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK46(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK45(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK46(T,An,An_1,...) \
		BO_JMPTAB_LINK45(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK47(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK46(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK47(T,An,An_1,...) \
		BO_JMPTAB_LINK46(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK48(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK47(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK48(T,An,An_1,...) \
		BO_JMPTAB_LINK47(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK49(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK48(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK49(T,An,An_1,...) \
		BO_JMPTAB_LINK48(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif

#  if __myc_support(NVM)
#   define BO_JMPTAB_LINK50(T,An,An_1,An_2...) \
		BO_JMPTAB_LINK49(T,An_1,An_2) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  else
#   define BO_JMPTAB_LINK50(T,An,An_1,...) \
		BO_JMPTAB_LINK49(T,An_1,__VA_ARGS__) \
		, &&__bo_janame(T,An) - &&__bo_janame(T,An_1)
#  endif
#endif
/* @} */

# endif /* BO_JMPTAB_DEFINED */

#endif /* !__BO_GCC__JMPTAB__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
