/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Absbtree.h	(V.Sokur)
 * include/Bo/fundis/Absbtree.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FUNDIS_ABSBTREE__
# define __BO_FUNDIS_ABSBTREE__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/aorp/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# ifndef BoL_BTREE_HIGH_MAX
#  define BoL_BTREE_HIGH_MAX 48
# endif

/* forward declatations */
struct bo_btree;
struct bo_btree_node;

/* b-tree direction */
# define BoK_BTREE_LEFT_DIR 0
# define BoK_BTREE_RIGHT_DIR 1

typedef int bo_btree_dir_t;

# define BO_BTREE_UserinfoDir(aCompareExpr) \
	(((aCompareExpr) <= 0) ? BoK_BTREE_RIGHT_DIR : BoK_BTREE_LEFT_DIR)

/* component of a b-tree path */
struct bo_btpath_component {
	struct bo_btree_node *node;
	bo_btree_dir_t        dir;
};

/* ============================================================================
 * B-TREE proper
 */
# pragma mark -
# pragma mark [ b-tree proper ]
# pragma mark -

/* NOT FOR YOUR BUSINESS @{ */
struct __bo_btree_s {
	struct {
		bo_pointer_t  btn_link[2];
		u_int32_t     btn_internal;
	} bt_head;
};

# define __bo_btree_p(aTree) ((struct __bo_btree_s *)(aTree))
/* @} */

struct bo_btree {
	struct {
		struct bo_btree_node  *btn_link[2];
		unsigned               btn_lbackref :1;
		unsigned               btn_rbackref :1;
		unsigned               btn_first :1;
		unsigned               btn_last :1;
		unsigned               btn_last_defo :1;
		signed                 btn_balance :3;
		unsigned               btn_pad :24;
	} bt_head;
};

# define bo_btree_p(aTree) ((struct bo_btree *)(aTree))

/* ----------------------------------------------------------------------------
 * PROPERTIES: ACCESS
 */

/* bo_btree::unnamed_node *
   _BO_BTREE_Head(bo_btree *aTree); */
# define _BO_BTREE_Head(aTree) (&bo_btree_p(aTree)->bt_head)

/* bo_btree_node *
   BO_BTREE_Head_(bo_btree *aTree); */
# ifndef __cplusplus
#  define BO_BTREE_Head_(aTree) ((struct bo_btree_node *)_BO_BTREE_Head(aTree))
# else
#  define BO_BTREE_Head_(aTree) \
	reinterpret_cast< bo_btree_node * >(_BO_BTREE_Head(aTree))
# endif

/* bo_btree::unnamed_node *
   _BO_BTREE_Tail(bo_btree *aTree); */
# define _BO_BTREE_Tail(aTree) _BO_BTREE_Head(aTree)

/* bo_btree_node *
   BO_BTREE_Tail_(bo_btree *aTree); */
# define BO_BTREE_Tail_(aTree) BO_BTREE_Head_(aTree)

/* ============================================================================
 * NODE
 */
# pragma mark -
# pragma mark [ node ]
# pragma mark -

/* NOT FOR YOUR BUSINESS @{ */
struct __bo_btree_node_s {
	bo_pointer_t  btn_link[2];
	u_int16_t     btn_internal;
	u_int16_t     btn_internal2;
};

# define __bo_btree_node_p(aNode) ((struct __bo_btree_node_s *)(aNode))
/* @} */

struct __MYC_QALIGN(SIZEOF_PTR) bo_btree_node {
	struct bo_btree_node      *btn_link[2];
# ifdef _MSC_VER
	unsigned short             btn_lbackref :1;
	unsigned short             btn_rbackref :1;
	unsigned short             btn_first :1;
	unsigned short             btn_last :1;
	unsigned short             btn_last_defo :1;
	signed short               btn_balance :3;
	unsigned short             btn_pad :8;
# else
	unsigned                   btn_lbackref :1;
	unsigned                   btn_rbackref :1;
	unsigned                   btn_first :1;
	unsigned                   btn_last :1;
	unsigned                   btn_last_defo :1;
	signed                     btn_balance :3;
	unsigned                   btn_pad :8;
# endif
	u_int16_t                  btn_ubits;
	bo_pointer_t               btn_info;
};

# define bo_btree_node_p(aNode) ((struct bo_btree_node *)(aNode))

# define bo_btree_node_of(_Tfield,anObject) \
	bo_btree_node_p(__myc_membaddrof(_Tfield,anObject)); \

/* ----------------------------------------------------------------------------
 * NODE PROPERTIES
 */

# define BO_BTREE_NODE_UserBits(aNode) (bo_btree_node_p(aNode)->btn_ubits)

/* bo_pointer_t
   BO_RTBTREE_NODE_UserInfo(bo_btree_node *aNode); */
# define BO_BTREE_NODE_UserInfo(aNode) (bo_btree_node_p(aNode)->btn_info)

/* bo_btree_node *
   BO_BTREE_NODE_Link(bo_btree_node *aNode, bo_btree_dir aDirection); */
# define BO_BTREE_NODE_Link(aNode,aDir) (bo_btree_node_p(aNode)->btn_link[(aDir)])

/* bo_btree_node *
   BO_BTREE_NODE_BLink(bo_btree_node *aNode, int aBalance); */
# define BO_BTREE_NODE_BLink(aNode,aBalance) \
	BO_BTREE_NODE_Link(aNode,((-1 == (aBalance)) ? BoK_BTREE_LEFT_DIR : BoK_BTREE_RIGHT_DIR))

/* bo_btree_node *
   BO_BTREE_NODE_LLink(bo_btree_node *aNode); */
# define BO_BTREE_NODE_LLink(aNode) BO_BTREE_NODE_Link(aNode,BoK_BTREE_LEFT_DIR)

/* bo_btree_node *
   BO_BTREE_NODE_RLink(bo_btree_node *aNode); */
# define BO_BTREE_NODE_RLink(aNode) BO_BTREE_NODE_Link(aNode,BoK_BTREE_RIGHT_DIR)

/* bool
   BO_BTREE_NODE_IsLast(bo_btree_node *aNode); */
# define BO_BTREE_NODE_IsLast(aNode) (!!bo_btree_node_p(aNode)->btn_last)

/* bool
   BO_BTREE_NODE_IsLast_defo(bo_btree_node *aNode); */
# define BO_BTREE_NODE_IsLast_defo(aNode) (!!bo_btree_node_p(aNode)->btn_last_defo)

/* bool
   BO_BTREE_NODE_IsLBackref(bo_btree_node *aNode); */
# define BO_BTREE_NODE_IsLBackref(aNode) (!!bo_btree_node_p(aNode)->btn_lbackref)

/* bool
   BO_BTREE_NODE_IsRBackref(bo_btree_node *aNode); */
# define BO_BTREE_NODE_IsRBackref(aNode) (!!bo_btree_node_p(aNode)->btn_rbackref)

/* bool
   BO_BTREE_NODE_IsUnbalanced(bo_btree_node *aNode); */
# define BO_BTREE_NODE_IsUnbalanced(aNode) (!!bo_btree_node_p(aNode)->btn_balance)

/* ----------------------------------------------------------------------------
 * NODE METHODS
 */

# define BO_BTREE_NODE_Nextbydir(aNode,aDir) \
	((aNode) = BO_BTREE_NODE_Link(aNode,aDir))

/* ============================================================================
 * (LPR) ITERATOR
 */
# pragma mark -
# pragma mark [ (lpr) iterator ]
# pragma mark -

# ifndef __cplusplus
typedef struct bo_btree_node       *bo_btree_iterator;
typedef struct bo_btree_node const *bo_btree_const_iterator;
# else
typedef bo_btree_node              *bo_btree_iterator;
typedef bo_btree_node const        *bo_btree_const_iterator;
# endif
typedef bo_btree_iterator        bo_btree_lpr_iterator;
typedef bo_btree_const_iterator  bo_btree_const_lpr_iterator;

/* ------------------------------------------------------------------------- */

/* bo_pointer_t
   _BO_BTREE_ITER_Deref(bo_btree_iterator anIter); */
# define _BO_BTREE_ITER_Deref(anIter) (bo_btree_node_p(anIter)->btn_info)

/* _Tp
   BO_BTREE_ITER_Deref< _Tp >(bo_btree_iterator anIter); */
# ifndef __cplusplus
#  define BO_BTREE_ITER_Deref(_Tp,anIter) ((_Tp)_BO_BTREE_ITER_Deref(anIter))
# else
#  define BO_BTREE_ITER_Deref(_Tp,anIter) \
	reinterpret_cast< _Tp >(_BO_BTREE_ITER_Deref(anIter))
# endif

/* ----------------------------------------------------------------------------
 * _BO_BTREE_ITER_Inc_lpr, __BO_BTREE_ITER_Inc_lpr
 */
# pragma mark -

/* void
   _BO_BTREE_ITER_Inc_lpr(bo_btree_iterator &anIter); */
# define _BO_BTREE_ITER_Inc_lpr(anIter) \
	do { \
		if (BO_BTREE_NODE_IsLast(anIter)) { \
			(anIter) = NULL; \
			break; \
		} \
		__BO_BTREE_ITER_Inc_lpr(anIter); \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   __BO_BTREE_ITER_Inc_lpr(bo_btree_iterator &anIter); */
# ifndef __cplusplus
#  define __BO_BTREE_ITER_Inc_lpr(anIter) \
	do { \
		register bo_btree_iterator ___bt_iter_inc_lpr_it; \
		___bt_iter_inc_lpr_it = (bo_btree_iterator)(anIter); \
		(anIter) = (bo_pointer_t)BO_BTREE_NODE_RLink(___bt_iter_inc_lpr_it); \
		if (!BO_BTREE_NODE_IsRBackref(___bt_iter_inc_lpr_it)) { \
			while (NULL != (___bt_iter_inc_lpr_it = BO_BTREE_NODE_LLink(anIter))) \
				(anIter) = (bo_pointer_t)___bt_iter_inc_lpr_it; \
		} \
	} while (0)
# else /* __cplusplus */
#  define __BO_BTREE_ITER_Inc_lpr(anIter) \
	do { \
		register bo_btree_iterator ___bt_iter_inc_lpr_it; \
		___bt_iter_inc_lpr_it = (bo_btree_iterator)(anIter); \
		*(bo_pointer_t*)&(anIter) = BO_BTREE_NODE_RLink(___bt_iter_inc_lpr_it); \
		if (!BO_BTREE_NODE_IsRBackref(___bt_iter_inc_lpr_it)) { \
			while (NULL != (___bt_iter_inc_lpr_it = BO_BTREE_NODE_LLink(anIter))) \
				*(bo_pointer_t*)&(anIter) = ___bt_iter_inc_lpr_it; \
		} \
	} while (0)
# endif /* !__cplusplus */

/* ----------------------------------------------------------------------------
 * _BO_BTREE_ITER_Next_lpr
 */
# pragma mark -

/* bo_btree_iterator (=NULL)
   _BO_BTREE_ITER_Next_lpr(bo_btree_iterator anIter); */
# if __myc_support(macro_retval)

#  define _BO_BTREE_ITER_Next_lpr(anIter) \
	__extension__ ({ \
		register bo_btree_iterator ___bt_iter_next_lpr_it; \
		___bt_iter_next_lpr_it = (bo_btree_iterator)(anIter); \
		_BO_BTREE_ITER_Inc_lpr(___bt_iter_next_lpr_it); \
		/* return */ (___bt_iter_next_lpr_it); \
	})

# else /* !__myc_support(macro_retval) */

__myc_extern_inline__ bo_btree_iterator
_BO_BTREE_ITER_Next_lpr(
#if __myc_require(prototypes)
	register bo_btree_iterator anIter
#else
	anIter
#endif
	)
#if !__myc_require(prototypes)
	register bo_btree_iterator anIter;
#endif
{
	_BO_BTREE_ITER_Inc_lpr(anIter);
	return (anIter);
}

# endif /* __myc_support(macro_retval) */

/* ----------------------------------------------------------------------------
 * __BO_BTREE_ITER_Next_lpr
 */
# pragma mark -

/* bo_btree_iterator
   __BO_BTREE_ITER_Next_lpr(bo_btree_iterator anIter); */
# if __myc_support(macro_retval)

#  define __BO_BTREE_ITER_Next_lpr(anIter) \
	__extension__ ({ \
		register bo_btree_iterator ___bt_iter_next_lpr__it; \
		___bt_iter_next_lpr__it = (bo_btree_iterator)(anIter); \
		__BO_BTREE_ITER_Inc_lpr(___bt_iter_next_lpr__it); \
		/* return */ (___bt_iter_next_lpr__it); \
	})

# else /* !__myc_support(macro_retval) */

__myc_extern_inline__ bo_btree_iterator
__BO_BTREE_ITER_Next_lpr(
#if __myc_require(prototypes)
	register bo_btree_iterator anIter
#else
	anIter
#endif
	)
#if !__myc_require(prototypes)
	register bo_btree_iterator anIter;
#endif
{
	__BO_BTREE_ITER_Inc_lpr(anIter);
	return (anIter);
}

# endif /* __myc_support(macro_retval) */

/* ============================================================================
 * (PLR) ITERATOR
 */
# pragma mark -
# pragma mark [ (plr) iterator ]
# pragma mark -

struct bo_btree_plr_iterator_s {
	struct bo_btree_node *cur_;
	unsigned              np_;
	struct bo_btree_node *next_[BoL_BTREE_HIGH_MAX];
};

typedef struct bo_btree_plr_iterator_s        *bo_btree_plr_iterator;
typedef struct bo_btree_plr_iterator_s const  *bo_btree_const_plr_iterator;

/* ------------------------------------------------------------------------- */

/* bo_pointer_t
   _BO_BTREE_PLR_ITER_Deref(bo_btree_plr_iterator anIter); */
# define _BO_BTREE_PLR_ITER_Deref(anIter) _BO_BTREE_ITER_Deref((anIter)->cur_)

/* _Tp
   BO_BTREE_PLR_ITER_Deref< _Tp >(bo_btree_plr_iterator anIter); */
# define BO_BTREE_PLR_ITER_Deref(_Tp,anIter) \
	BO_BTREE_ITER_Deref(_Tp,(anIter)->cur_)

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

struct bo_btninfo_opv;

# define BoK_BTREE_INORDER 0 /* LPR, symmetric */
# define BoK_BTREE_PREORDER 1 /* PLR */
# define BoK_BTREE_POSTORDER 2 /* LRP */

/*
 * BoS_OK: success, continue iteration
 * BoS_NORMAL: success, stop iteration
 * BoS_ERR: error, stop iteration
 */
typedef bo_status_t
(__myc_tcalltype__(mcall) *bo_btninfo_walk_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anInfo,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aParm, /* = NULL */
	struct aorp_error * /* = NULL */
# endif
	) __attribute__((mcalltype));

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_TRUE = (/* full tree is traversed */);
	BoS_FALSE = (/* part of tree is only traversed */);
))
__attribute__((nonnull(1,2)))
BoBTWalk(
# if __myc_require(prototypes)
	struct bo_btree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	int aWalkOrder,
	bo_btninfo_walk_cb aWalkCb,
	bo_pointer_t aParm __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: btree-node-info operations
 */
# pragma mark -
# pragma mark [ btree-node-info operations ]
# pragma mark -

typedef void
(__myc_tcalltype__(mcall) *bo_btninfo_dump_fn)(
# if __myc_require(prototypes)
	bo_pointer_t aStream,
	bo_pointer_t anInfo,
	unsigned aLevel
# endif
	) __attribute__((mcalltype));

struct bo_btninfo_opv {
	aorp_dup2_cb        btni_dup;
	aorp_retain_cb      btni_retain;
	aorp_release_cb     btni_release;
	aorp_compare_cb     btni_compare;
	bo_btninfo_dump_fn  btni_dump;
};

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS
__myc_vlinkage__(public,libBo) struct bo_btninfo_opv const _G_default_btninfo_opv;
__MYC_END_CDECLS

#endif /* !__BO_FUNDIS_ABSBTREE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
