/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Cdllist.h	(V.Sokur)
 * include/Bo/fundis/Cdllist.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FUNDIS_CDLLIST__
# define __BO_FUNDIS_CDLLIST__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: circular-double-linked-list declarations
 */
# pragma mark -
# pragma mark [ circular-double-linked-list declarations ]
# pragma mark -

/* forward declatations */
struct bo_cdllelem;
struct bo_cdllist;

/*
 * circular-double-linked-list proper
 */

struct __MYC_QALIGN(SIZEOF_PTR) bo_cdllist {
	struct bo_cdllelem  *next,
	                   **prevn;
	size_t               elemc;
};

# define bo_cdllist_p(aList) ((struct bo_cdllist *)(aList))

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_cdllist,
	sizeof(struct bo_cdllist_s) == sizeof(struct bo_cdllist));
# endif /* _DEBUG */
/* @} */

/*
 * circular-double-linked-list element
 */

struct __MYC_QALIGN(SIZEOF_PTR) bo_cdllelem {
	struct bo_cdllelem  *next,
	                   **prevn;
};

# define bo_cdllelem_p(anElem) ((struct bo_cdllelem *)(anElem))

# define bo_cdllelem_of(_Tfield,anObject) \
	bo_cdllelem_p(__myc_membaddrof(_Tfield,anObject))

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_cdllelem,
	sizeof(struct bo_cdllelem_s) == sizeof(struct bo_cdllelem));
# endif /* _DEBUG */
/* @} */

/* ============================================================================
 * ITERATOR
 */
# pragma mark -
# pragma mark [ iterator ]
# pragma mark -

typedef struct bo_cdllelem       *bo_cdllist_iterator;
typedef struct bo_cdllelem const *bo_cdllist_const_iterator;

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_ITER_Inc(bo_cdllist_iterator &anIter); */
# ifndef __cplusplus
#  define _BO_CDLLIST_ITER_Inc(anIter) \
	((anIter) = (bo_pointer_t)bo_cdllelem_p(anIter)->next)
# else
#  define _BO_CDLLIST_ITER_Inc(anIter) \
	(*(bo_pointer_t*)&(anIter) = bo_cdllelem_p(anIter)->next)
# endif

/* void
   BO_CDLLIST_ITER_Inc< _Tp, _Tfield >(_Tp *anIter); */
# define BO_CDLLIST_ITER_Inc(_Tp,_Tfield,anIter) \
	((anIter) = __myc_objectof(_Tp,_Tfield,bo_cdllelem_of(_Tfield,anIter)->next))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_ITER_Dec(bo_cdllist_iterator &anIter); */
# ifndef __cplusplus
#  define _BO_CDLLIST_ITER_Dec(anIter) \
	((anIter) = (bo_pointer_t)bo_cdllelem_p(anIter)->prevn)
# else
#  define _BO_CDLLIST_ITER_Dec(anIter) \
	(*(bo_pointer_t*)&(anIter) = bo_cdllelem_p(anIter)->prevn)
# endif

/* void
   BO_CDLLIST_ITER_Dec< _Tp, _Tfield >(_Tp *anIter); */
# define BO_CDLLIST_ITER_Dec(_Tp,_Tfield,anIter) \
	((anIter) = __myc_objectof(_Tp,_Tfield,bo_cdllelem_of(_Tfield,anIter)->prevn))

/* ------------------------------------------------------------------------- */

/* _Tp *
   BO_CDLLIST_ITER_Next< _Tp, _Tfield >(_Tp *anIter); */
# define BO_CDLLIST_ITER_Next(_Tp,_Tfield,anIter) \
	(__myc_objectof(_Tp,_Tfield,bo_cdllelem_of(_Tfield,anIter)->next))

/* _Tp *
   BO_CDLLIST_ITER_Prev< _Tp, _Tfield >(_Tp *anIter); */
# define BO_CDLLIST_ITER_Prev(_Tp,_Tfield,anIter) \
	(__myc_objectof(_Tp,_Tfield,bo_cdllelem_of(_Tfield,anIter)->prevn))

/* ============================================================================
 * CONSTRUCT/DESTROY
 */
# pragma mark -
# pragma mark [ construct/destroy ]
# pragma mark -

# define _BO_CDLLIST_INITIALIZER(aListRef) \
	{ \
		/* .next = */   bo_cdllelem_p(&(aListRef)), \
		/* .prevn = */ (struct bo_cdllelem **)(&(aListRef)), \
		/* .elemc = */  0 \
	}

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_Init(bo_cdllist *aList); */
# define _BO_CDLLIST_Init(aList) \
	do { \
		struct bo_cdllist *___cdll_init_list = bo_cdllist_p(aList); \
		___cdll_init_list->next = bo_cdllelem_p(___cdll_init_list); \
		___cdll_init_list->prevn = /* &(aList)->next */ (struct bo_cdllelem **)(___cdll_init_list); \
		___cdll_init_list->elemc = 0; \
	} while (0)

/* void
   BO_CDLLIST_Destroy< _Tp, _Tfield >(
		bo_cdllist *aList, void (*_DELETER)(_Tp *)
	); */
# define BO_CDLLIST_Destroy(_Tp,_Tfield,aList,_DELETER) \
	do { \
		_Tp *___cdll_dstroy_obj; \
		register struct bo_cdllelem *___cdll_dstroy_it; \
		register size_t ___cdll_dstroy_count; \
		struct bo_cdllist *___cdll_dstroy_list = bo_cdllist_p(aList); \
		for (___cdll_dstroy_count = _BO_CDLLIST_Size(___cdll_dstroy_list); \
				___cdll_dstroy_count--; \
					/* empty */ ) \
		{ \
			___cdll_dstroy_it = _BO_CDLLIST_Begin(___cdll_dstroy_list); \
			_BO_CDLLIST_Remove(___cdll_dstroy_list,___cdll_dstroy_it); \
			___cdll_dstroy_obj = __myc_objectof(_Tp,_Tfield,___cdll_dstroy_it); \
			_DELETER(___cdll_dstroy_obj); \
		} \
	} while (0)

/* void
   BO_CDLLIST_Destroy_< _Tp, _Tfield >(
		bo_cdllist *aList, {(_DELETER: _Tp *_$1)}
	); */
# define BO_CDLLIST_Destroy_(_Tp,_Tfield,aList,_DELETER) \
	do { \
		_Tp *_$1; \
		register struct bo_cdllelem *___cdll_dstroy__it; \
		register size_t ___cdll_dstroy__count; \
		struct bo_cdllist *___cdll_dstroy__list = bo_cdllist_p(aList); \
		for (___cdll_dstroy__count = _BO_CDLLIST_Size(___cdll_dstroy__list); \
				___cdll_dstroy__count--; \
					/* empty */ ) \
		{ \
			___cdll_dstroy__it = _BO_CDLLIST_Begin(___cdll_dstroy__list); \
			_BO_CDLLIST_Remove(___cdll_dstroy__list,___cdll_dstroy__it); \
			_$1 = __myc_objectof(_Tp,_Tfield,___cdll_dstroy__it); \
			_DELETER; \
		} \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_CDLLIST_Clear< _Tp, _Tfield >(
		bo_cdllist *aList, void (*_DELETER)(_Tp *)
	); */
# define BO_CDLLIST_Clear BO_CDLLIST_Destroy

/* void
   BO_CDLLIST_Clear_< _Tp, _Tfield >(
		bo_cdllist *aList, {(_DELETER: _Tp *_$1)}
	); */
# define BO_CDLLIST_Clear_ BO_CDLLIST_Destroy_

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_Swap(bo_cdllist *aList1, bo_cdllist *aList2); */
# define _BO_CDLLIST_Swap(aList1, aList2) \
	do { \
		if (_BO_CDLLIST_Empty(aList1)) { \
			if (_BO_CDLLIST_Empty(aList2)) \
				break; \
			___BO_CDLLIST_Relink12(aList1, aList2); \
			_BO_CDLLIST_Init(aList2); \
			break; \
		} \
		if (_BO_CDLLIST_Empty(aList2)) { \
			___BO_CDLLIST_Relink12(aList2, aList1); \
			_BO_CDLLIST_Init(aList1); \
			break; \
		} \
		{ \
			struct bo_cdllist ___cdll_swap_tlist = *bo_cdllist_p(aList1); \
			___BO_CDLLIST_Relink12(aList1, aList2); \
			___BO_CDLLIST_Relink12(aList2, &___cdll_swap_tlist); \
		} \
	} while (0)

# define ___BO_CDLLIST_Relink12(aList1, aList2) \
	do { \
		bo_cdllelem_p(aList1)->next = bo_cdllelem_p(aList2)->next; \
		bo_cdllelem_p(aList1)->next->prevn = /* &(aList1)->next */ (struct bo_cdllelem **)(aList1); \
		bo_cdllelem_p(aList1)->prevn = bo_cdllelem_p(aList2)->prevn; \
		*bo_cdllelem_p(aList1)->prevn = bo_cdllelem_p(aList1); \
		bo_cdllist_p(aList1)->elemc = bo_cdllist_p(aList2)->elemc; \
	} while (0)

/* ------------------------------------------------------------------------- */

# define _BO_CDLLIST_Link(aDest, aSource) \
	do { \
		struct bo_cdllelem *___cdll_link_sfirst = _BO_CDLLIST_Front(aSource); \
		if (NULL != ___cdll_link_sfirst) { \
			struct bo_cdllelem *___cdll_link_dend = _BO_CDLLIST_End(aDest), \
			                   *___cdll_link_slast = _BO_CDLLIST_Back(aSource); \
			___cdll_link_slast->next = ___cdll_link_dend; \
			___cdll_link_sfirst->prevn = ___cdll_link_dend->prevn; \
			*___cdll_link_sfirst->prevn = ___cdll_link_sfirst; \
			___cdll_link_dend->prevn = /* &(elem)->cdl_next */ (struct bo_cdllelem **)(___cdll_link_slast); \
			bo_cdllist_p(aDest)->elemc += bo_cdllist_p(aSource)->elemc; \
		} \
	} while (0)

/* ============================================================================
 * PROPERTIES: ACCESS
 */
# pragma mark -
# pragma mark [ properties: access ]
# pragma mark -

/* bo_cdllelem *
   _BO_CDLLIST_Head(bo_cdllist *aList); */
# define _BO_CDLLIST_Head(aList) bo_cdllelem_p(aList)

/* bo_cdllelem *
   _BO_CDLLIST_Tail(bo_cdllist *aList); */
# define _BO_CDLLIST_Tail(aList) _BO_CDLLIST_Head(aList)

/* ------------------------------------------------------------------------- */

/* size_t
   _BO_CDLLIST_Size(bo_cdllist *aList); */
# define _BO_CDLLIST_Size(aList) (bo_cdllist_p(aList)->elemc)

/* bool
   _BO_CDLLIST_Empty(bo_cdllist *aList); */
# define _BO_CDLLIST_Empty(aList) (0 == _BO_CDLLIST_Size(aList))

/* ============================================================================
 * METHODS: ACCESS
 */
# pragma mark -
# pragma mark [ methods: access ]
# pragma mark -

/* bo_cdllist_iterator
   _BO_CDLLIST_Begin(bo_cdllist *aList); */
# define _BO_CDLLIST_Begin(aList) (_BO_CDLLIST_Head(aList)->next)

/* _Tp *
   BO_CDLLIST_Begin< _Tp, _Tfield >(bo_cdllist *aList); */
# define BO_CDLLIST_Begin(_Tp,_Tfield,aList) \
	__myc_objectof(_Tp,_Tfield,_BO_CDLLIST_Begin(aList))

/* bo_cdllist_iterator
   _BO_CDLLIST_End(bo_cdllist *aList); */
# define _BO_CDLLIST_End(aList) _BO_CDLLIST_Tail(aList)

/* _Tp *
   BO_CDLLIST_End< _Tp, _Tfield >(bo_cdllist *aList); */
# define BO_CDLLIST_End(_Tp,_Tfield,aList) \
	__myc_objectof(_Tp,_Tfield,_BO_CDLLIST_End(aList))

/* ------------------------------------------------------------------------- */

/* bo_cdllelem * (=NULL)
   _BO_CDLLIST_Front(bo_cdllist *aList); */
# define _BO_CDLLIST_Front(aList) \
	(_BO_CDLLIST_Size(aList) ? _BO_CDLLIST_Begin(aList) : NULL)

/* _Tp * (=NULL)
   BO_CDLLIST_Front< _Tp, _Tfield >(bo_cdllist *aList); */
# define BO_CDLLIST_Front(_Tp,_Tfield,aList) \
	(_BO_CDLLIST_Size(aList) ? BO_CDLLIST_Begin(_Tp,_Tfield,aList) : NULL)

/* bo_cdllelem * (=NULL)
   _BO_CDLLIST_Back(bo_cdllist *aList); */
# define _BO_CDLLIST_Back(aList) \
	(_BO_CDLLIST_Size(aList) \
		? bo_cdllelem_p(_BO_CDLLIST_End(aList)->prevn) \
		: NULL)

/* _Tp * (=NULL)
   BO_CDLLIST_Back< _Tp, _Tfield >(bo_cdllist *aList); */
# define BO_CDLLIST_Back(_Tp,_Tfield,aList) \
	(_BO_CDLLIST_Size(aList) \
		? __myc_objectof(_Tp,_Tfield,_BO_CDLLIST_End(aList)->prevn) \
		: NULL)

/* ------------------------------------------------------------------------- */

/* bo_cdllelem * (=NULL)
   _BO_CDLLIST_At(bo_cdllist *aList, bo_index_t anIndex); */
# if __myc_support(macro_retval)
#  define _BO_CDLLIST_At(aList,anIndex) \
	__extension__ ({ \
		bo_cdllist_iterator ___cdll_at_it; \
		bo_index_t ___cdll_at_idx, ___cdll_at_oidx = (anIndex); \
		struct bo_cdllist *___cdll_at__list = bo_cdllist_p(aList); \
		___cdll_at_it = NULL; \
		if ((___cdll_at_idx = (___cdll_at_oidx)) < 0) \
			___cdll_at_idx = -(___cdll_at_oidx) - 1; \
		if ((size_t)___cdll_at_idx < _BO_CDLLIST_Size(___cdll_at__list)) { \
			if (0 <= (___cdll_at_oidx)) { \
				_BO_CDLLIST_FOREACH_INF(___cdll_at_it,___cdll_at__list) \
					if (0 == ___cdll_at_idx--) { \
						break; \
					} \
				FOREACH_END \
			} \
			else { \
				_BO_CDLLIST_REVERSE_FOREACH_INF(___cdll_at_it,___cdll_at__list) \
					if (0 == ___cdll_at_idx--) { \
						break; \
					} \
				FOREACH_END \
			} \
		} \
		/* return */ (___cdll_at_it); \
	})
# else
__myc_extern_inline__ struct bo_cdllelem *
_BO_CDLLIST_At(
	register struct bo_cdllist *aList,
	register bo_index_t anIndex
	);
# endif /* __myc_support(macro_retval) */

/* ------------------------------------------------------------------------- */

/* _Tp * (=NULL)
   BO_CDLLIST_At< _Tp, _Tfield >(bo_cdllist *aList, bo_index_t anIndex); */
# define BO_CDLLIST_At(_Tp,_Tfield,aList,anIndex) \
	( assert((0 <= (anIndex) && (size_t)(anIndex) < _BO_CDLLIST_Size(aList)) || \
				((anIndex) < 0 && (size_t)(-(anIndex)) <= _BO_CDLLIST_Size(aList))), \
		__myc_objectof(_Tp,_Tfield,_BO_CDLLIST_At(bo_cdllist_p(aList),anIndex)) )

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

/* void
   BO_CDLLIST_Enumerate< _Tp, _Tfield >(
		_Tp *anIter,
		bo_cdllist *aList,
		{(_PREDICATE: _Tp *anIter)}
	); */
# define BO_CDLLIST_Enumerate(_Tp,_Tfield,anIter,aList,_PREDICATE) \
	do { \
		bo_cdllist_iterator ___cdll_enum_it; \
		_BO_CDLLIST_FOREACH(___cdll_enum_it,aList) \
			(anIter) = __myc_objectof(_Tp,_Tfield,___cdll_enum_it); \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* void
   BO_CDLLIST_Enumerate_< _Tp, _Tfield >(
		bo_cdllist *aList,
		{(_PREDICATE: (_Tp *_$1))}
	); */
# define BO_CDLLIST_Enumerate_(_Tp,_Tfield,aList,_PREDICATE) \
	do { \
		_Tp *_$1; \
		bo_cdllist_iterator ___cdll_enum__it; \
		_BO_CDLLIST_FOREACH(___cdll_enum__it,aList) \
			_$1 = __myc_objectof(_Tp,_Tfield,___cdll_enum__it); \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* void
   BO_CDLLIST_ReverseEnumerate_< _Tp, _Tfield >(
		bo_cdllist *aList,
		{(_PREDICATE: (_Tp *_$1))}
	); */
# define BO_CDLLIST_ReverseEnumerate_(_Tp,_Tfield,aList,_PREDICATE) \
	do { \
		_Tp *_$1; \
		bo_cdllist_iterator ___cdll_renum__it; \
		_BO_CDLLIST_REVERSE_FOREACH(___cdll_renum__it,aList) \
			_$1 = __myc_objectof(_Tp,_Tfield,___cdll_renum__it); \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_Insert(
		bo_cdllist *aList,
		bo_cdllelem *anElem,
		bo_cdllist_iterator aPos
	); */
# define _BO_CDLLIST_Insert _BO_CDLLIST_InsertBefore

/* void
   _BO_CDLLIST_InsertBefore(
		bo_cdllist *aList,
		bo_cdllelem *anElem,
		bo_cdllist_iterator aPos
	); */
# define _BO_CDLLIST_InsertBefore(aList,anElem,aPos) \
	do { \
		struct bo_cdllelem *___cdll_insb_elem = bo_cdllelem_p(anElem); \
		struct bo_cdllelem *___cdll_insb_pos = bo_cdllelem_p(aPos); \
		___cdll_insb_elem->next = ___cdll_insb_pos; \
		___cdll_insb_elem->prevn = ___cdll_insb_pos->prevn; \
		*___cdll_insb_pos->prevn = ___cdll_insb_elem; \
		___cdll_insb_pos->prevn = /* &(elem)->next */ (struct bo_cdllelem **)(___cdll_insb_elem); \
		bo_cdllist_p(aList)->elemc++; \
	} while (0)

/* void
   BO_CDLLIST_InsertBefore< _Tp, _Tfield >(
		bo_cdllist *aList,
		_Tp *anObject,
		_Tp *aPos
	); */
# define BO_CDLLIST_InsertBefore(_Tp,_Tfield,aList,anObject,aPos) \
	_BO_CDLLIST_InsertBefore(aList,bo_cdllelem_of(_Tfield,anObject),bo_cdllelem_of(_Tfield,aPos))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_InsertAfter(
		bo_cdllist *aList,
		bo_cdllelem *anElem,
		bo_cdllist_iterator aPos
	); */
# define _BO_CDLLIST_InsertAfter(aList,anElem,aPos) \
	do { \
		struct bo_cdllelem *___cdll_insa_elem = bo_cdllelem_p(anElem); \
		struct bo_cdllelem *___cdll_insa_pos = bo_cdllelem_p(aPos); \
		___cdll_insa_elem->next = ___cdll_insa_pos->next; \
		___cdll_insa_elem->prevn = /* &(pos)->next */ (struct bo_cdllelem **)(___cdll_insa_pos); \
		___cdll_insa_pos->next->prevn = /* &(elem)->next */ (struct bo_cdllelem **)(___cdll_insa_elem); \
		___cdll_insa_pos->next = ___cdll_insa_elem; \
		bo_cdllist_p(aList)->elemc++; \
	} while (0)

/* void
   BO_CDLLIST_InsertAfter< _Tp, _Tfield >(
		bo_cdllist *aList,
		_Tp *anObject,
		_Tp *aPos
	); */
# define BO_CDLLIST_InsertAfter(_Tp,_Tfield,aList,anObject,aPos) \
	_BO_CDLLIST_InsertAfter(aList,bo_cdllelem_of(_Tfield,anObject),bo_cdllelem_of(_Tfield,aPos))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_Remove(
		bo_cdllist *aList,
		bo_cdllist_iterator aPos
	); */
# define _BO_CDLLIST_Remove(aList,aPos) \
	do { \
		struct bo_cdllelem *___cdll_rm_pos = bo_cdllelem_p(aPos); \
		___cdll_rm_pos->next->prevn = ___cdll_rm_pos->prevn; \
		*___cdll_rm_pos->prevn = ___cdll_rm_pos->next; \
		___cdll_rm_pos->next = NULL; \
		___cdll_rm_pos->prevn = NULL; \
		bo_cdllist_p(aList)->elemc--; \
	} while (0)

/* void
   BO_CDLLIST_Remove< _Tp, _Tfield >(
		bo_cdllist *aList,
		_Tp *anObject
	); */
# define BO_CDLLIST_Remove(_Tp,_Tfield,aList,anObject) \
	_BO_CDLLIST_Remove(aList,bo_cdllelem_of(_Tfield,anObject))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_PushFront(
		bo_cdllist *aList,
		bo_cdllelem *anElem
	); */
# define _BO_CDLLIST_PushFront(aList,anElem) \
	_BO_CDLLIST_InsertAfter((aList),bo_cdllelem_p(anElem),_BO_CDLLIST_End(aList))

/* void
   BO_CDLLIST_PushFront< _Tp, _Tfield >(
		bo_cdllist *aList,
		_Tp *anObject
	); */
# define BO_CDLLIST_PushFront(_Tp,_Tfield,aList,anObject) \
	_BO_CDLLIST_InsertAfter((aList),bo_cdllelem_of(_Tfield,anObject),_BO_CDLLIST_End(aList))

/* void
   _BO_CDLLIST_PopFront(bo_cdllist *aList); */
# define _BO_CDLLIST_PopFront(aList) \
	_BO_CDLLIST_Remove((aList),_BO_CDLLIST_Begin(aList))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_PushBack(
		bo_cdllist *aList,
		bo_cdllelem *anElem
	); */
# define _BO_CDLLIST_PushBack(aList,anElem) \
	_BO_CDLLIST_InsertBefore((aList),bo_cdllelem_p(anElem),_BO_CDLLIST_End(aList))

/* void
   BO_CDLLIST_PushBack< _Tp, _Tfield >(
		bo_cdllist *aList,
		_Tp *anObject
	); */
# define BO_CDLLIST_PushBack(_Tp,_Tfield,aList,anObject) \
	_BO_CDLLIST_InsertBefore((aList),bo_cdllelem_of(_Tfield,anObject),_BO_CDLLIST_End(aList))

/* void
   _BO_CDLLIST_PopBack(bo_cdllist *aList); */
# define _BO_CDLLIST_PopBack(aList) \
	_BO_CDLLIST_Remove((aList),bo_cdllelem_p(_BO_CDLLIST_End(aList)->prevn))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CDLLIST_MvFront(
		bo_cdllist *aList,
		bo_cdllist_iterator aPos
	); */
# define _BO_CDLLIST_MvFront(aList,aPos) \
	do { \
		struct bo_cdllelem *___cdll_mvfront_pos = bo_cdllelem_p(aPos); \
		struct bo_cdllist *___cdll_mvfront_list = bo_cdllist_p(aList); \
		_BO_CDLLIST_Remove(___cdll_mvfront_list,___cdll_mvfront_pos); \
		_BO_CDLLIST_PushFront(___cdll_mvfront_list,___cdll_mvfront_pos); \
	} while (0)

/* void
   BO_CDLLIST_MvFront< _Tp, _Tfield >(
		bo_cdllist *aList,
		_Tp *anObject
	); */
# define BO_CDLLIST_MvFront(_Tp,_Tfield,aList,anObject) \
	_BO_CDLLIST_MvFront(aList,bo_cdllelem_of(_Tfield,anObject))

/* ============================================================================
 * LOOP HELPERS
 * @{
 */
# pragma mark -
# pragma mark [ loop helpers ]
# pragma mark -

/*
	bo_cdllist *list;
	bo_cdllist_iterator iter;
	...
	_BO_CDLLIST_FOREACH(iter, list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_FOREACH(anIter,aList) { \
	struct bo_cdllist *___cdll_foreach_list = bo_cdllist_p(aList); \
	for ((anIter) = _BO_CDLLIST_Begin(___cdll_foreach_list); \
			_BO_CDLLIST_End(___cdll_foreach_list) != (anIter); \
				(anIter) = bo_cdllelem_p(anIter)->next) {

# define _BO_CDLLIST_SAFE_FOREACH(anIter,aList) { \
	bo_cdllist_iterator ___cdll_safe_foreach_next; \
	struct bo_cdllist *___cdll_safe_foreach_list = bo_cdllist_p(aList); \
	for ((anIter) = _BO_CDLLIST_Begin(___cdll_safe_foreach_list); \
			(___cdll_safe_foreach_next = bo_cdllelem_p(anIter)->next, \
			_BO_CDLLIST_End(___cdll_safe_foreach_list) != (anIter)); \
					(anIter) = ___cdll_safe_foreach_next) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	bo_cdllist_iterator iter;
	...
	_BO_CDLLIST_RFOREACH(iter, list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_RFOREACH(anIter,aList) { \
	struct bo_cdllist *___cdll_rforeach_list = bo_cdllist_p(aList); \
	for ((anIter) = bo_cdllelem_p(_BO_CDLLIST_End(___cdll_rforeach_list)->prevn); \
			_BO_CDLLIST_End(___cdll_rforeach_list) != (anIter); \
				(anIter) = bo_cdllelem_p(bo_cdllelem_p(anIter)->prevn)) {

# define _BO_CDLLIST_SAFE_RFOREACH(anIter,aList) { \
	bo_cdllist_iterator ___cdll_safe_rforeach_prev; \
	struct bo_cdllist *___cdll_safe_rforeach_list = bo_cdllist_p(aList); \
	for ((anIter) = bo_cdllelem_p(_BO_CDLLIST_End(___cdll_safe_rforeach_list)->prevn); \
			(___cdll_safe_rforeach_prev = bo_cdllelem_p(bo_cdllelem_p(anIter)->prevn), \
			_BO_CDLLIST_End(___cdll_safe_rforeach_list) != (anIter)); \
					(anIter) = ___cdll_safe_rforeach_prev) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	...
	// iterator *_$1;
	_BO_CDLLIST_FOREACH_(list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_FOREACH_(aList) { \
	bo_cdllist_iterator _$1; \
	struct bo_cdllist *___cdll_foreach__list = bo_cdllist_p(aList); \
	for (_$1 = _BO_CDLLIST_Begin(___cdll_foreach__list); \
			_BO_CDLLIST_End(___cdll_foreach__list) != _$1; \
				_$1 = _$1->next) {

# define _BO_CDLLIST_SAFE_FOREACH_(aList) { \
	bo_cdllist_iterator _$1; \
	bo_cdllist_iterator ___cdll_safe_foreach__next; \
	struct bo_cdllist *___cdll_safe_foreach__list = bo_cdllist_p(aList); \
	for (_$1 = _BO_CDLLIST_Begin(___cdll_safe_foreach__list); \
			(___cdll_safe_foreach__next = _$1->next, \
			_BO_CDLLIST_End(___cdll_safe_foreach__list) != _$1); \
					_$1 = ___cdll_safe_foreach__next) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	...
	// iterator *_$1;
	_BO_CDLLIST_RFOREACH_(list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_RFOREACH_(aList) { \
	bo_cdllist_iterator _$1; \
	struct bo_cdllist *___cdll_rforeach__list = bo_cdllist_p(aList); \
	for (_$1 = bo_cdllelem_p(_BO_CDLLIST_End(___cdll_rforeach__list)->prevn); \
			_BO_CDLLIST_End(___cdll_rforeach__list) != _$1; \
				_$1 = bo_cdllelem_p(_$1->prevn)) {

# define _BO_CDLLIST_SAFE_RFOREACH_(aList) { \
	bo_cdllist_iterator _$1; \
	bo_cdllist_iterator ___cdll_safe_rforeach__prev; \
	struct bo_cdllist *___cdll_safe_rforeach__list = bo_cdllist_p(aList); \
	for (_$1 = bo_cdllelem_p(_BO_CDLLIST_End(___cdll_safe_rforeach__list)->prevn); \
			(___cdll_safe_rforeach__prev = bo_cdllelem_p(_$1->prevn), \
			_BO_CDLLIST_End(___cdll_safe_rforeach__list) != _$1); \
					_$1 = ___cdll_safe_rforeach__prev) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	bo_cdllist_iterator iter;
	...
	_BO_CDLLIST_FOREACH_INF(iter, list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_FOREACH_INF(anIter,aList) { \
	for ((anIter) = _BO_CDLLIST_Begin(aList); \
			/* true */ ; \
				(anIter) = bo_cdllelem_p(anIter)->next) {

/*
	bo_cdllist *list;
	bo_cdllist_iterator iter;
	...
	_BO_CDLLIST_RFOREACH_INF(iter, list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_RFOREACH_INF(anIter,aList) { \
	for ((anIter) = bo_cdllelem_p(_BO_CDLLIST_End(aList)->prevn); \
			/* true */ ; \
				(anIter) = bo_cdllelem_p(bo_cdllelem_p(anIter)->prevn)) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	...
	// iterator *_$1;
	_BO_CDLLIST_FOREACH_INF_(list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_FOREACH_INF_(aList) { \
	bo_cdllist_iterator _$1; \
	for (_$1 = _BO_CDLLIST_Begin(aList); \
			/* true */ ; \
				_$1 = _$1->next) {

/*
	bo_cdllist *list;
	...
	// iterator *_$1;
	_BO_CDLLIST_RFOREACH_INF_(list)
		...
	FOREACH_END
 */
# define _BO_CDLLIST_RFOREACH_INF_(aList) { \
	bo_cdllist_iterator _$1; \
	for (_$1 = bo_cdllelem_p(_BO_CDLLIST_End(aList)->prevn); \
			/* true */ ; \
				_$1 = bo_cdllelem_p(_$1->prevn)) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	_Tp *iter;
	...
	BO_CDLLIST_FOREACH< _Tp, _Tfield >(iter, list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_FOREACH(_Tp,_Tfield,anIter,aList) { \
	bo_cdllist_iterator ___cdll_foreach_it; \
	struct bo_cdllist *___cdll_foreach_list = bo_cdllist_p(aList); \
	for (___cdll_foreach_it = _BO_CDLLIST_Begin(___cdll_foreach_list); \
			((anIter) = ((_BO_CDLLIST_End(___cdll_foreach_list) != ___cdll_foreach_it) \
					? __myc_objectof(_Tp,_Tfield,___cdll_foreach_it) \
					: NULL)); \
				___cdll_foreach_it = ___cdll_foreach_it->next) {

# define BO_CDLLIST_SAFE_FOREACH(_Tp,_Tfield,anIter,aList) { \
	bo_cdllist_iterator ___cdll_safe_foreach_it, ___cdll_safe_foreach_next; \
	struct bo_cdllist *___cdll_safe_foreach_list = bo_cdllist_p(aList); \
	for (___cdll_safe_foreach_it = _BO_CDLLIST_Begin(___cdll_safe_foreach_list); \
			(___cdll_safe_foreach_next = ___cdll_safe_foreach_it->next, \
			((anIter) = ((_BO_CDLLIST_End(___cdll_safe_foreach_list) != ___cdll_safe_foreach_it) \
					? __myc_objectof(_Tp,_Tfield,___cdll_safe_foreach_it) \
					: NULL))); \
				___cdll_safe_foreach_it = ___cdll_safe_foreach_next) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	_Tp *iter;
	...
	BO_CDLLIST_RFOREACH< _Tp, _Tfield >(iter, list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_RFOREACH(_Tp,_Tfield,anIter,aList) { \
	bo_cdllist_iterator ___cdll_rforeach_it; \
	struct bo_cdllist *___cdll_rforeach_list = bo_cdllist_p(aList); \
	for (___cdll_rforeach_it = bo_cdllelem_p(_BO_CDLLIST_End(___cdll_rforeach_list)->prevn); \
			(anIter = ((_BO_CDLLIST_End(___cdll_rforeach_list) != ___cdll_rforeach_it) \
					? __myc_objectof(_Tp,_Tfield,___cdll_rforeach_it) \
					: NULL)); \
				___cdll_rforeach_it = bo_cdllelem_p(___cdll_rforeach_it->prevn)) {

# define BO_CDLLIST_SAFE_RFOREACH(_Tp,_Tfield,anIter,aList) { \
	bo_cdllist_iterator ___cdll_safe_rforeach_it, ___cdll_safe_rforeach_prev; \
	struct bo_cdllist *___cdll_safe_rforeach_list = bo_cdllist_p(aList); \
	for (___cdll_safe_rforeach_it = bo_cdllelem_p(_BO_CDLLIST_End(___cdll_safe_rforeach_list)->prevn); \
			(___cdll_safe_rforeach_prev = bo_cdllelem_p(___cdll_safe_rforeach_it->prevn), \
			(anIter = ((_BO_CDLLIST_End(___cdll_safe_rforeach_list) != ___cdll_safe_rforeach_it) \
					? __myc_objectof(_Tp,_Tfield,___cdll_safe_rforeach_it) \
					: NULL))); \
				___cdll_safe_rforeach_it = ___cdll_safe_rforeach_prev) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	...
	// _Tp *_$1;
	BO_CDLLIST_FOREACH_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_FOREACH_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_cdllist_iterator ___cdll_foreach__it; \
	struct bo_cdllist *___cdll_foreach__list = bo_cdllist_p(aList); \
	for (___cdll_foreach__it = _BO_CDLLIST_Begin(___cdll_foreach__list); \
			(_$1 = ((_BO_CDLLIST_End(___cdll_foreach__list) != ___cdll_foreach__it) \
					? __myc_objectof(_Tp,_Tfield,___cdll_foreach__it) \
					: NULL)); \
				___cdll_foreach__it = ___cdll_foreach__it->next) {

/*
	bo_cdllist *list;
	...
	// _Tp *_$1;
	BO_CDLLIST_SAFE_FOREACH_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_SAFE_FOREACH_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_cdllist_iterator ___cdll_safe_foreach__it, ___cdll_safe_foreach__next; \
	struct bo_cdllist *___cdll_safe_foreach__list = bo_cdllist_p(aList); \
	for (___cdll_safe_foreach__it = _BO_CDLLIST_Begin(___cdll_safe_foreach__list); \
			(___cdll_safe_foreach__next = ___cdll_safe_foreach__it->next, \
			(_$1 = ((_BO_CDLLIST_End(___cdll_safe_foreach__list) != ___cdll_safe_foreach__it) \
					? __myc_objectof(_Tp,_Tfield,___cdll_safe_foreach__it) \
					: NULL))); \
				___cdll_safe_foreach__it = ___cdll_safe_foreach__next) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	...
	// _Tp *_$1;
	BO_CDLLIST_RFOREACH_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_RFOREACH_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_cdllist_iterator ___cdll_rforeach__it; \
	struct bo_cdllist *___cdll_rforeach__list = bo_cdllist_p(aList); \
	for (___cdll_rforeach__it = bo_cdllelem_p(_BO_CDLLIST_End(___cdll_rforeach__list)->prevn); \
			(_$1 = ((_BO_CDLLIST_End(___cdll_rforeach__list) != ___cdll_rforeach__it) \
					? __myc_objectof(_Tp,_Tfield,___cdll_rforeach__it) \
					: NULL)); \
				___cdll_rforeach__it = bo_cdllelem_p(___cdll_rforeach__it->prevn)) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	_Tp *iter;
	...
	BO_CDLLIST_FOREACH_INF< _Tp, _Tfield >(iter, list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_FOREACH_INF(_Tp,_Tfield,anIter,aList) { \
	bo_cdllist_iterator ___cdll_foreach_inf_it; \
	for (___cdll_foreach_inf_it = _BO_CDLLIST_Begin(aList), \
		(anIter) = __myc_objectof(_Tp,_Tfield,___cdll_foreach_inf_it); \
			/* true */ ; \
				___cdll_foreach_inf_it = ___cdll_foreach_inf_it->next, \
				(anIter) = __myc_objectof(_Tp,_Tfield,___cdll_foreach_inf_it)) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	_Tp *iter;
	...
	BO_CDLLIST_RFOREACH_INF< _Tp, _Tfield >(iter, list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_RFOREACH_INF(_Tp,_Tfield,anIter,aList) { \
	bo_cdllist_iterator ___cdll_rforeach_inf_it; \
	for (___cdll_rforeach_inf_it = bo_cdllelem_p(_BO_CDLLIST_End(aList)->prevn), \
		(anIter) = __myc_objectof(_Tp,_Tfield,___cdll_rforeach_inf_it); \
			/* true */ ; \
				___cdll_rforeach_inf_it = bo_cdllelem_p(___cdll_rforeach_inf_it->prevn), \
				(anIter) = __myc_objectof(_Tp,_Tfield,___cdll_rforeach_inf_it)) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	...
	// _Tp *_$1;
	BO_CDLLIST_FOREACH_INF_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_FOREACH_INF_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_cdllist_iterator ___cdll_foreach_inf__it; \
	for (___cdll_foreach_inf__it = _BO_CDLLIST_Begin(aList), \
		_$1 = __myc_objectof(_Tp,_Tfield,___cdll_foreach_inf__it); \
			/* true */ ; \
				___cdll_foreach_inf__it = ___cdll_foreach_inf__it->next, \
				_$1 = __myc_objectof(_Tp,_Tfield,___cdll_foreach_inf__it)) {

/* ------------------------------------------------------------------------- */

/*
	bo_cdllist *list;
	...
	// _Tp *_$1;
	BO_CDLLIST_RFOREACH_INF_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CDLLIST_RFOREACH_INF_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_cdllist_iterator ___cdll_rforeach_inf__it; \
	for (___cdll_rforeach_inf__it = bo_cdllelem_p(_BO_CDLLIST_End(aList)->prevn), \
		_$1 = __myc_objectof(_Tp,_Tfield,___cdll_rforeach_inf__it); \
			/* true */ ; \
				___cdll_rforeach_inf__it = bo_cdllelem_p(___cdll_rforeach_inf__it->prevn), \
				_$1 = __myc_objectof(_Tp,_Tfield,___cdll_rforeach_inf__it)) {


# ifndef FOREACH_END
#  define FOREACH_END }}
# endif
/* @} */

/* compat @{ */
# define _BO_CDLLIST_REVERSE_FOREACH _BO_CDLLIST_RFOREACH
# define _BO_CDLLIST_SAFE_REVERSE_FOREACH _BO_CDLLIST_SAFE_RFOREACH
# define _BO_CDLLIST_REVERSE_FOREACH_ _BO_CDLLIST_RFOREACH_
# define _BO_CDLLIST_SAFE_REVERSE_FOREACH_ _BO_CDLLIST_SAFE_RFOREACH_
# define _BO_CDLLIST_REVERSE_FOREACH_INF _BO_CDLLIST_RFOREACH_INF
# define _BO_CDLLIST_REVERSE_FOREACH_INF_ _BO_CDLLIST_RFOREACH_INF_

# define BO_CDLLIST_REVERSE_FOREACH BO_CDLLIST_RFOREACH
# define BO_CDLLIST_SAFE_REVERSE_FOREACH BO_CDLLIST_SAFE_RFOREACH
# define BO_CDLLIST_REVERSE_FOREACH_ BO_CDLLIST_RFOREACH_
# define BO_CDLLIST_SAFE_REVERSE_FOREACH_ BO_CDLLIST_SAFE_RFOREACH_ /* no such impl */
# define BO_CDLLIST_REVERSE_FOREACH_INF BO_CDLLIST_RFOREACH_INF
# define BO_CDLLIST_REVERSE_FOREACH_INF_ BO_CDLLIST_RFOREACH_INF_
/* @} */


# if !__myc_support(macro_retval)
__myc_extern_inline__ struct bo_cdllelem *
_BO_CDLLIST_At(
	register struct bo_cdllist *aList,
	register bo_index_t anIndex
	)
{
	bo_cdllist_iterator it;
	bo_index_t idx;

	if ((idx = anIndex) < 0)
		idx = -anIndex - 1;

	if ((size_t)idx < _BO_CDLLIST_Size(aList)) {
		if (0 <= anIndex) {
			_BO_CDLLIST_FOREACH_INF(it,aList)
				if (0 == idx--) {
					return (it);
					/* NOTREACHED */
				}
			FOREACH_END
		}
		else {
			_BO_CDLLIST_RFOREACH_INF(it,aList)
				if (0 == idx--) {
					return (it);
					/* NOTREACHED */
				}
			FOREACH_END
		}
	}

	return (NULL);
}
# endif /* !__myc_support(macro_retval) */

#endif /* !__BO_FUNDIS_CDLLIST__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
