/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Csllist.h	(V.Sokur)
 * include/Bo/fundis/Csllist.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FUNDIS_CSLLIST__
# define __BO_FUNDIS_CSLLIST__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: circular-single-linked-list declarations
 */
# pragma mark -
# pragma mark [ circular-single-linked-list declarations ]
# pragma mark -

/* forward declatations */
struct bo_csllelem;
struct bo_csllist;

/*
 * circular-single-linked-list proper
 */

struct __MYC_QALIGN(SIZEOF_PTR) bo_csllist {
	struct bo_csllelem  *next,
	                    *last;
	size_t               elemc;
};

# define bo_csllist_p(aList) ((struct bo_csllist *)(aList))

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_csllist,
	sizeof(struct bo_csllist_s) == sizeof(struct bo_csllist));
# endif /* _DEBUG */
/* @} */

/*
 * circular-double-linked-list element
 */

struct __MYC_QALIGN(SIZEOF_PTR) bo_csllelem {
	struct bo_csllelem  *next;
};

# define bo_csllelem_p(anElem) ((struct bo_csllelem *)(anElem))

# define bo_csllelem_of(_Tfield,anObject) \
	bo_csllelem_p(__myc_membaddrof(_Tfield,anObject))

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_csllelem,
	sizeof(struct bo_csllelem_s) == sizeof(struct bo_csllelem));
# endif /* _DEBUG */
/* @} */

/* ============================================================================
 * ITERATOR
 */
# pragma mark -
# pragma mark [ iterator ]
# pragma mark -

typedef struct bo_csllelem       *bo_csllist_iterator;
typedef struct bo_csllelem const *bo_csllist_const_iterator;

/* ------------------------------------------------------------------------- */

/* void
   _BO_CSLLIST_ITER_Inc(bo_csllist_iterator &anIter); */
# ifndef __cplusplus
#  define _BO_CSLLIST_ITER_Inc(anIter) \
	((anIter) = (bo_pointer_t)bo_csllelem_p(anIter)->next)
# else
#  define _BO_CSLLIST_ITER_Inc(anIter) \
	(*(bo_pointer_t*)&(anIter) = bo_csllelem_p(anIter)->next)
# endif

/* void
   BO_CSLLIST_ITER_Inc< _Tp, _Tfield >(_Tp *anIter); */
# define BO_CSLLIST_ITER_Inc(_Tp,_Tfield,anIter) \
	((anIter) = __myc_objectof(_Tp,_Tfield,bo_csllelem_of(_Tfield,anIter)->next))

/* ------------------------------------------------------------------------- */

/* _Tp *
   BO_CSLLIST_ITER_Next< _Tp, _Tfield >(_Tp *anIter); */
# define BO_CSLLIST_ITER_Next(_Tp,_Tfield,anIter) \
	(__myc_objectof(_Tp,_Tfield,bo_csllelem_of(_Tfield,anIter)->next))

/* ============================================================================
 * CONSTRUCT/DESTROY
 */
# pragma mark -
# pragma mark [ construct/destroy ]
# pragma mark -

# define _BO_CSLLIST_INITIALIZER(aListRef) \
	{ \
		/* .next = */   bo_csllelem_p(&(aListRef)), \
		/* .last = */   bo_csllelem_p(&(aListRef)), \
		/* .elemc = */  0 \
	}

/* ------------------------------------------------------------------------- */

/* void
   _BO_CSLLIST_Init(bo_csllist *aList); */
# define _BO_CSLLIST_Init(aList) \
	do { \
		struct bo_csllist *___csll_init_list = bo_csllist_p(aList); \
		___csll_init_list->next = bo_csllelem_p(___csll_init_list); \
		___csll_init_list->last = bo_csllelem_p(___csll_init_list); \
		___csll_init_list->elemc = 0; \
	} while (0)

/* void
   BO_CSLLIST_Destroy< _Tp, _Tfield >(
		bo_csllist *aList, void (*_DELETER)(_Tp *)
	); */
# define BO_CSLLIST_Destroy(_Tp,_Tfield,aList,_DELETER) \
	do { \
		_Tp *___csll_dstroy_obj; \
		struct bo_csllist *___csll_dstroy_list = bo_csllist_p(aList); \
		while (NULL != (___csll_dstroy_obj = BO_CSLLIST_Front(_Tp,_Tfield,___csll_dstroy_list))) { \
			_BO_CSLLIST_PopFront(___csll_dstroy_list); \
			_DELETER(___csll_dstroy_obj); \
		} \
	} while (0)

/* void
   BO_CSLLIST_Destroy_< _Tp, _Tfield >(
		bo_csllist *aList, {(_DELETER: _Tp *_$1)}
	); */
# define BO_CSLLIST_Destroy_(_Tp,_Tfield,aList,_DELETER) \
	do { \
		_Tp *_$1; \
		struct bo_csllist *___csll_dstroy__list = bo_csllist_p(aList); \
		while (NULL != (_$1 = BO_CSLLIST_Front(_Tp,_Tfield,___csll_dstroy__list))) { \
			_BO_CSLLIST_PopFront(___csll_dstroy__list); \
			_DELETER; \
		} \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_CSLLIST_Clear< _Tp, _Tfield >(
		bo_csllist *aList, void (*_DELETER)(_Tp *)
	); */
# define BO_CSLLIST_Clear BO_CSLLIST_Destroy

/* void
   BO_CSLLIST_Clear_< _Tp, _Tfield >(
		bo_csllist *aList, {(_DELETER: _Tp *_$1)}
	); */
# define BO_CSLLIST_Clear_ BO_CSLLIST_Destroy_

/* ------------------------------------------------------------------------- */

/* void
   _BO_CSLLIST_Swap(bo_csllist *aList1, bo_csllist *aList2); */
# define _BO_CSLLIST_Swap(aList1, aList2) \
	do { \
		if (_BO_CSLLIST_Empty(aList1)) { \
			if (_BO_CSLLIST_Empty(aList2)) \
				break; \
			___BO_CSLLIST_Relink12(aList1, aList2); \
			_BO_CSLLIST_Init(aList2); \
			break; \
		} \
		if (_BO_CSLLIST_Empty(aList2)) { \
			___BO_CSLLIST_Relink12(aList2, aList1); \
			_BO_CSLLIST_Init(aList1); \
			break; \
		} \
		{ \
			struct bo_csllist *___csll_swap_tlist = bo_csllist_p(aList1); \
			___BO_CSLLIST_Relink12(aList1, aList2); \
			___BO_CSLLIST_Relink12(aList2, &___csll_swap_tlist); \
		} \
	} while (0)

# define ___BO_CSLLIST_Relink12(aList1, aList2) \
	do { \
		bo_csllelem_p(aList1)->next = bo_csllelem_p(aList2)->next; \
		bo_csllelem_p(aList1)->last = bo_csllelem_p(aList2)->last; \
		bo_csllelem_p(aList1)->last->next = bo_csllelem_p(aList1); \
		bo_csllist_p(aList1)->elemc = bo_csllist_p(aList2)->elemc; \
	} while (0)

/* ============================================================================
 * PROPERTIES: ACCESS
 */
# pragma mark -
# pragma mark [ properties: access ]
# pragma mark -

/* bo_csllelem *
   _BO_CSLLIST_Head(bo_csllist *aList); */
# define _BO_CSLLIST_Head(aList) bo_csllelem_p(aList)

/* bo_csllelem *
   _BO_CSLLIST_Tail(bo_csllist *aList); */
# define _BO_CSLLIST_Tail(aList) _BO_CSLLIST_Head(aList)

/* ------------------------------------------------------------------------- */

/* size_t
   _BO_CSLLIST_Size(bo_csllist *aList); */
# define _BO_CSLLIST_Size(aList) (bo_csllist_p(aList)->elemc)

/* bool
   _BO_CSLLIST_Empty(bo_csllist *aList); */
# define _BO_CSLLIST_Empty(aList) (0 == _BO_CSLLIST_Size(aList))

/* ============================================================================
 * METHODS: ACCESS
 */
# pragma mark -
# pragma mark [ methods: access ]
# pragma mark -

/* bo_csllist_iterator
   _BO_CSLLIST_Begin(bo_csllist *aList); */
# define _BO_CSLLIST_Begin(aList) (_BO_CSLLIST_Head(aList)->next)

/* _Tp *
   BO_CSLLIST_Begin< _Tp, _Tfield >(bo_csllist *aList); */
# define BO_CSLLIST_Begin(_Tp,_Tfield,aList) \
	__myc_objectof(_Tp,_Tfield,_BO_CSLLIST_Begin(aList))

/* bo_csllist_iterator
   _BO_CSLLIST_End(bo_csllist *aList); */
# define _BO_CSLLIST_End(aList) _BO_CSLLIST_Tail(aList)

/* _Tp *
   BO_CSLLIST_End< _Tp, _Tfield >(bo_csllist *aList); */
# define BO_CSLLIST_End(_Tp,_Tfield,aList) \
	__myc_objectof(_Tp,_Tfield,_BO_CSLLIST_End(aList))

/* ------------------------------------------------------------------------- */

/* bo_csllelem * (=NULL)
   _BO_CSLLIST_Front(bo_csllist *aList); */
# define _BO_CSLLIST_Front(aList) \
	(_BO_CSLLIST_Size(aList) ? _BO_CSLLIST_Begin(aList) : NULL)

/* _Tp * (=NULL)
   BO_CSLLIST_Front< _Tp, _Tfield >(bo_csllist *aList); */
# define BO_CSLLIST_Front(_Tp,_Tfield,aList) \
	(_BO_CSLLIST_Size(aList) ? BO_CSLLIST_Begin(_Tp,_Tfield,aList) : NULL)

/* bo_csllelem * (=NULL)
   _BO_CSLLIST_Back(bo_csllist *aList); */
# define _BO_CSLLIST_Back(aList) \
	(_BO_CSLLIST_Size(aList) ? bo_csllist_p(aList)->last : NULL)

/* _Tp * (=NULL)
   BO_CSLLIST_Back< _Tp, _Tfield >(bo_csllist *aList); */
# define BO_CSLLIST_Back(_Tp,_Tfield,aList) \
	(_BO_CSLLIST_Size(aList) \
		? __myc_objectof(_Tp,_Tfield,bo_csllist_p(aList)->last) \
		: NULL)

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

/* void
   BO_CSLLIST_Enumerate< _Tp, _Tfield >(
		_Tp *anIter,
		bo_csllist *aList,
		{(_PREDICATE: _Tp *anIter)}
	); */
# define BO_CSLLIST_Enumerate(_Tp,_Tfield,anIter,aList,_PREDICATE) \
	do { \
		bo_csllist_iterator ___csll_enum_it; \
		_BO_CSLLIST_FOREACH(___csll_enum_it,aList) \
			(anIter) = __myc_objectof(_Tp,_Tfield,___csll_enum_it); \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* void
   BO_CSLLIST_Enumerate_< _Tp, _Tfield >(
		bo_csllist *aList,
		{(_PREDICATE: (_Tp *_$1))}
	); */
# define BO_CSLLIST_Enumerate_(_Tp,_Tfield,aList,_PREDICATE) \
	do { \
		_Tp *_$1; \
		bo_csllist_iterator ___csll_enum__it; \
		_BO_CSLLIST_FOREACH(___csll_enum__it,aList) \
			_$1 = __myc_objectof(_Tp,_Tfield,___csll_enum__it); \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* void
   BO_CSLLIST_ReverseEnumerate_< _Tp, _Tfield >(
		bo_csllist *aList,
		{(_PREDICATE: (_Tp *_$1))}
	); */
# define BO_CSLLIST_ReverseEnumerate_(_Tp,_Tfield,aList,_PREDICATE) \
	do { \
		_Tp *_$1; \
		bo_csllist_iterator ___csll_renum__it; \
		_BO_CSLLIST_RFOREACH(___csll_renum__it,aList) \
			_$1 = __myc_objectof(_Tp,_Tfield,___csll_renum__it); \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   _BO_CSLLIST_Insert(
		bo_csllist *aList,
		bo_csllelem *anElem,
		bo_csllist_iterator aPos
	); */
# define _BO_CSLLIST_Insert _BO_CSLLIST_InsertAfter

/* void
   _BO_CSLLIST_InsertAfter(
		bo_csllist *aList,
		bo_csllelem *anElem,
		bo_csllist_iterator aPos
	); */
# define _BO_CSLLIST_InsertAfter(aList,anElem,aPos) \
	do { \
		struct bo_csllelem *___csll_insa_elem = bo_csllelem_p(anElem); \
		struct bo_csllelem *___csll_insa_pos = bo_csllelem_p(aPos); \
		struct bo_csllist *___csll_insa_list = bo_csllist_p(aList); \
		___csll_insa_elem->next = ___csll_insa_pos->next; \
		___csll_insa_pos->next = ___csll_insa_elem; \
		if (___csll_insa_pos == ___csll_insa_list->last) \
			___csll_insa_list->last = ___csll_insa_elem; \
		___csll_insa_list->elemc++; \
	} while (0)

/* void
   BO_CSLLIST_InsertAfter< _Tp, _Tfield >(
		bo_csllist *aList,
		_Tp *anObject,
		_Tp *aPos
	); */
# define BO_CSLLIST_InsertAfter(_Tp,_Tfield,aList,anObject,aPos) \
	_BO_CSLLIST_InsertAfter(aList,bo_csllelem_of(_Tfield,anObject),bo_csllelem_of(_Tfield,aPos))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CSLLIST_Remove(
		bo_csllist *aList,
		bo_csllist_iterator aPos,
		bo_csllist_iterator aPrevPos
	); */
# define _BO_CSLLIST_Remove(aList,aPos,aPrevPos) \
	do { \
		struct bo_csllelem *___csll_rm_pos = bo_csllelem_p(aPos); \
		struct bo_csllelem *___csll_rm_prev = bo_csllelem_p(aPrevPos); \
		struct bo_csllist *___csll_rm_list = bo_csllist_p(aList); \
		___csll_rm_prev->next = ___csll_rm_pos->next; \
		___csll_rm_pos->next = NULL; \
		if (___csll_rm_pos == ___csll_rm_list->last) \
			___csll_rm_list->last = ___csll_rm_prev; \
		___csll_rm_list->elemc--; \
	} while (0)

/* void
   BO_CSLLIST_Remove< _Tp, _Tfield >(
		bo_csllist *aList,
		_Tp *anObject,
		_Tp *aPrev
	); */
# define BO_CSLLIST_Remove(_Tp,_Tfield,aList,anObject,aPrev) \
	_BO_CSLLIST_Remove(aList,bo_csllelem_of(_Tfield,anObject),bo_csllelem_of(_Tfield,aPrev))

/* ------------------------------------------------------------------------- */

/* void
   _BO_CSLLIST_PushFront(
		bo_csllist *aList,
		bo_csllelem *anElem
	); */
# define _BO_CSLLIST_PushFront(aList,anElem) \
	_BO_CSLLIST_InsertAfter((aList),bo_csllelem_p(anElem),_BO_CSLLIST_End(aList))

/* void
   BO_CSLLIST_PushFront< _Tp, _Tfield >(
		bo_csllist *aList,
		_Tp *anObject
	); */
# define BO_CSLLIST_PushFront(_Tp,_Tfield,aList,anObject) \
	_BO_CSLLIST_InsertAfter((aList),bo_csllelem_of(_Tfield,anObject),_BO_CSLLIST_End(aList))

/* void
   _BO_CSLLIST_PopFront(bo_csllist *aList); */
# define _BO_CSLLIST_PopFront(aList) \
	_BO_CSLLIST_Remove((aList),_BO_CSLLIST_Begin(aList),_BO_CSLLIST_End(aList))

/* ------------------------------------------------------------------------- */

# define ___BO_CSLLIST_Append(aList,anElem) \
	do { \
		struct bo_csllist *___csll_insa_list = bo_csllist_p(aList); \
		struct bo_csllelem *___csll_insa_pos = ___csll_insa_list->last; \
		struct bo_csllelem *___csll_insa_elem = bo_csllelem_p(anElem); \
		___csll_insa_elem->next = ___csll_insa_pos->next; \
		___csll_insa_pos->next = ___csll_insa_elem; \
		___csll_insa_list->last = ___csll_insa_elem; \
		___csll_insa_list->elemc++; \
	} while (0)

/* void
   _BO_CSLLIST_PushBack(
		bo_csllist *aList,
		bo_csllelem *anElem
	); */
# define _BO_CSLLIST_PushBack(aList,anElem) \
	___BO_CSLLIST_Append((aList),bo_csllelem_p(anElem))

/* void
   BO_CSLLIST_PushBack< _Tp, _Tfield >(
		bo_csllist *aList,
		_Tp *anObject
	); */
# define BO_CSLLIST_PushBack(_Tp,_Tfield,aList,anObject) \
	___BO_CSLLIST_Append((aList),bo_csllelem_of(_Tfield,anObject))

/* ============================================================================
 * LOOP HELPERS
 * @{
 */
# pragma mark -
# pragma mark [ loop helpers ]
# pragma mark -

/*
	bo_csllist *list;
	bo_csllist_iterator iter;
	...
	_BO_CSLLIST_FOREACH(iter, list)
		...
	FOREACH_END
 */
# define _BO_CSLLIST_FOREACH(anIter,aList) { \
	struct bo_csllist *___csll_foreach_list = bo_csllist_p(aList); \
	for ((anIter) = _BO_CSLLIST_Begin(___csll_foreach_list); \
			_BO_CSLLIST_End(___csll_foreach_list) != (anIter); \
				(anIter) = bo_csllelem_p(anIter)->next) {

# define _BO_CSLLIST_SAFE_FOREACH(anIter,aList) { \
	bo_csllist_iterator ___csll_safe_foreach_next; \
	struct bo_csllist *___csll_safe_foreach_list = bo_csllist_p(aList); \
	for ((anIter) = _BO_CSLLIST_Begin(___csll_safe_foreach_list); \
			(___csll_safe_foreach_next = bo_csllelem_p(anIter)->next, \
			_BO_CSLLIST_End(___csll_safe_foreach_list) != (anIter)); \
					(anIter) = ___csll_safe_foreach_next) {

/* ------------------------------------------------------------------------- */

/*
	bo_csllist *list;
	...
	// iterator *_$1;
	_BO_CSLLIST_FOREACH_(list)
		...
	FOREACH_END
 */
# define _BO_CSLLIST_FOREACH_(aList) { \
	bo_csllist_iterator _$1; \
	struct bo_csllist *___csll_foreach__list = bo_csllist_p(aList); \
	for (_$1 = _BO_CSLLIST_Begin(___csll_foreach__list); \
			_BO_CSLLIST_End(___csll_foreach__list) != _$1; \
				_$1 = _$1->next) {

# define _BO_CSLLIST_SAFE_FOREACH_(aList) { \
	bo_csllist_iterator _$1; \
	bo_csllist_iterator ___csll_safe_foreach__next; \
	struct bo_csllist *___csll_safe_foreach__list = bo_csllist_p(aList); \
	for (_$1 = _BO_CSLLIST_Begin(___csll_safe_foreach__list); \
			(___csll_safe_foreach__next = _$1->next, \
			_BO_CSLLIST_End(___csll_safe_foreach__list) != _$1); \
					_$1 = ___csll_safe_foreach__next) {

/* ------------------------------------------------------------------------- */

/*
	bo_csllist *list;
	bo_csllist_iterator iter;
	...
	_BO_CSLLIST_FOREACH_INF(iter, list)
		...
	FOREACH_END
 */
# define _BO_CSLLIST_FOREACH_INF(anIter,aList) { \
	for ((anIter) = _BO_CSLLIST_Begin(aList); \
			/* true */ ; \
				(anIter) = bo_csllelem_p(anIter)->next) {

/* ------------------------------------------------------------------------- */

/*
	bo_csllist *list;
	...
	// iterator *_$1;
	_BO_CSLLIST_FOREACH_INF_(list)
		...
	FOREACH_END
 */
# define _BO_CSLLIST_FOREACH_INF_(aList) { \
	bo_csllist_iterator _$1; \
	for (_$1 = _BO_CSLLIST_Begin(aList); \
			/* true */ ; \
				_$1 = _$1->next) {

/* ------------------------------------------------------------------------- */

/*
	bo_csllist *list;
	_Tp *iter;
	...
	BO_CSLLIST_FOREACH< _Tp, _Tfield >(iter, list)
		...
	FOREACH_END
 */
# define BO_CSLLIST_FOREACH(_Tp,_Tfield,anIter,aList) { \
	bo_csllist_iterator ___csll_foreach_it; \
	struct bo_csllist *___csll_foreach_list = bo_csllist_p(aList); \
	for (___csll_foreach_it = _BO_CSLLIST_Begin(___csll_foreach_list); \
			((anIter) = ((_BO_CSLLIST_End(___csll_foreach_list) != ___csll_foreach_it) \
					? __myc_objectof(_Tp,_Tfield,___csll_foreach_it) \
					: NULL)); \
				___csll_foreach_it = ___csll_foreach_it->next) {

# define BO_CSLLIST_SAFE_FOREACH(_Tp,_Tfield,anIter,aList) { \
	bo_csllist_iterator ___csll_safe_foreach_it, ___csll_safe_foreach_next; \
	struct bo_csllist *___csll_safe_foreach_list = bo_csllist_p(aList); \
	for (___csll_safe_foreach_it = _BO_CSLLIST_Begin(___csll_safe_foreach_list); \
			(___csll_safe_foreach_next = ___csll_safe_foreach_it->next, \
			((anIter) = ((_BO_CSLLIST_End(___csll_safe_foreach_list) != ___csll_safe_foreach_it) \
					? __myc_objectof(_Tp,_Tfield,___csll_safe_foreach_it) \
					: NULL))); \
				___csll_safe_foreach_it = ___csll_safe_foreach_next) {

/* ------------------------------------------------------------------------- */

/*
	bo_csllist *list;
	...
	// _Tp *_$1;
	BO_CSLLIST_FOREACH_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CSLLIST_FOREACH_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_csllist_iterator ___csll_foreach__it; \
	struct bo_csllist *___csll_foreach__list = bo_csllist_p(aList); \
	for (___csll_foreach__it = _BO_CSLLIST_Begin(___csll_foreach__list); \
			(_$1 = ((_BO_CSLLIST_End(___csll_foreach__list) != ___csll_foreach__it) \
					? __myc_objectof(_Tp,_Tfield,___csll_foreach__it) \
					: NULL)); \
				___csll_foreach__it = ___csll_foreach__it->next) {

/*
	bo_csllist *list;
	...
	// _Tp *_$1;
	BO_CSLLIST_SAFE_FOREACH_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CSLLIST_SAFE_FOREACH_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_csllist_iterator ___csll_safe_foreach__it, ___csll_safe_foreach__next; \
	struct bo_csllist *___csll_safe_foreach__list = bo_csllist_p(aList); \
	for (___csll_safe_foreach__it = _BO_CSLLIST_Begin(___csll_safe_foreach__list); \
			(___csll_safe_foreach__next = ___csll_safe_foreach__it->next, \
			(_$1 = ((_BO_CSLLIST_End(___csll_safe_foreach__list) != ___csll_safe_foreach__it) \
					? __myc_objectof(_Tp,_Tfield,___csll_safe_foreach__it) \
					: NULL))); \
				___csll_safe_foreach__it = ___csll_safe_foreach__next) {

/* ------------------------------------------------------------------------- */

/*
	bo_csllist *list;
	_Tp *iter;
	...
	BO_CSLLIST_FOREACH_INF< _Tp, _Tfield >(iter, list)
		...
	FOREACH_END
 */
# define BO_CSLLIST_FOREACH_INF(_Tp,_Tfield,anIter,aList) { \
	bo_csllist_iterator ___csll_foreach_inf_it; \
	for (___csll_foreach_inf_it = _BO_CSLLIST_Begin(aList), \
		(anIter) = __myc_objectof(_Tp,_Tfield,___csll_foreach_inf_it); \
			/* true */ ; \
				___csll_foreach_inf_it = ___csll_foreach_inf_it->next, \
				(anIter) = __myc_objectof(_Tp,_Tfield,___csll_foreach_inf_it)) {

/* ------------------------------------------------------------------------- */

/*
	bo_csllist *list;
	...
	// _Tp *_$1;
	BO_CSLLIST_FOREACH_INF_< _Tp, _Tfield >(list)
		...
	FOREACH_END
 */
# define BO_CSLLIST_FOREACH_INF_(_Tp,_Tfield,aList) { \
	_Tp *_$1; \
	bo_csllist_iterator ___csll_foreach_inf__it; \
	for (___csll_foreach_inf__it = _BO_CSLLIST_Begin(aList), \
		_$1 = __myc_objectof(_Tp,_Tfield,___csll_foreach_inf__it); \
			/* true */ ; \
				___csll_foreach_inf__it = ___csll_foreach_inf__it->next, \
				_$1 = __myc_objectof(_Tp,_Tfield,___csll_foreach_inf__it)) {


# ifndef FOREACH_END
#  define FOREACH_END }}
# endif
/* @} */

#endif /* !__BO_FUNDIS_CSLLIST__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
