/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Lstrdic.h	(V.Sokur)
 * include/Bo/fundis/Lstrdic.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIST_LIGHTSTRINGDICTIONARY__
# define __BO_LIST_LIGHTSTRINGDICTIONARY__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/fundis/Seqlist.h>

/*
 * It is required the next files are optionally included first:
 *
# include <Bo/memman/BoMalloc.h>
# include <Bo/libc/String.h>
	// memmove, memcpy, strlen
 */

/******************************************************************************
 * %% BeginSection: light string-dictionary declarations
 */
# pragma mark -
# pragma mark [ light string-dictionary declarations ]
# pragma mark -

/* light string-dictionary proper */
typedef bo_seqlist_t bo_lstrdic_t;

# define bo_lstrdic_p(aDic) ((bo_lstrdic_t *)(aDic))

/* ----------------------------------------------------------------------------
 * ENTRY
 */

struct bo_lstrdic_entry {
	char const *sdic_key,
			   *sdic_value;
};

typedef struct bo_lstrdic_entry        bo_lstrdic_entry_t;

# define bo_lstrdic_entry_p(anEntry) ((bo_lstrdic_entry_t *)(anEntry))

# ifndef __cplusplus
typedef struct bo_lstrdic_entry       *bo_lstrdic_pointer;
typedef struct bo_lstrdic_entry const *bo_lstrdic_const_pointer;
# else
typedef bo_lstrdic_entry              *bo_lstrdic_pointer;
typedef bo_lstrdic_entry const        *bo_lstrdic_const_pointer;
# endif

/* ----------------------------------------------------------------------------
 * ITERATOR
 */

# ifndef __cplusplus
typedef struct bo_lstrdic_entry       *bo_lstrdic_iterator;
typedef struct bo_lstrdic_entry const *bo_lstrdic_const_iterator;
# else
typedef bo_lstrdic_entry              *bo_lstrdic_iterator;
typedef bo_lstrdic_entry const        *bo_lstrdic_const_iterator;
# endif

/* bo_lstrdic_entry_t&
   BO_LSTRDIC_ITER_Deref(bo_lstrdic_iterator anIter); */
# define BO_LSTRDIC_ITER_Deref(anIter) (*(anIter))

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ============================================================================
 * CONSTRUCT/DESTROY
 */

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLstrdic_Init_n(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	size_t anElemc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLstrdic_Init_nx(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	bo_lstrdic_entry_t const anElemv[],
	size_t anElemc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoLstrdic_Init_cp(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDest,
	bo_lstrdic_t const *aSource,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) void
__attribute__((nonnull(1)))
BoLstrdic_Destroy(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic
# endif
	);

__myc_flinkage__(public,libBo) void
__attribute__((nonnull(1)))
BoLstrdic_Clear(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic
# endif
	);

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoLstrdic_Add(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	bo_lstrdic_entry_t const *anElem,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoLstrdic_ForceAdd(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	bo_lstrdic_entry_t const *anElem,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLstrdic_Add2(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	char const *aName,
	char const *aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLstrdic_ForceAdd2(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	char const *aName,
	char const *aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# if __myc_support(TUNO)

#  define BoLstrdic_Add2(aDic,aName,aValue,anErrPtr) \
	BoLstrdic_Add(aDic, &(bo_lstrdic_entry_t){ aName, aValue }, anErrPtr)

#  define BoLstrdic_ForceAdd2(aDic,aName,aValue,anErrPtr) \
	BoLstrdic_ForceAdd(aDic, &(bo_lstrdic_entry_t){ aName, aValue }, anErrPtr)

# else
# if __myc_support(macro_retval)

#  define BoLstrdic_Add2(aDic,aName,aValue,anErrPtr) \
	__extension__ ({ \
		bo_lstrdic_entry_t t = { aName, aValue }; \
		/* return */ BoLstrdic_Add(aDic, &t, anErrPtr); \
	})

#  define BoLstrdic_ForceAdd2(aDic,aName,aValue,anErrPtr) \
	__extension__ ({ \
		bo_lstrdic_entry_t t = { aName, aValue }; \
		/* return */ BoLstrdic_ForceAdd(aDic, &t, anErrPtr); \
	})

# endif /* !__myc_support(macro_retval) */
# endif /* !__myc_support(TUNO) */

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoLstrdic_Replace(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	bo_lstrdic_entry_t const *anElem,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
BoLstrdic_Replace2(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	char const *aName,
	char const *aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# if __myc_support(TUNO)
#  define BoLstrdic_Replace2(aDic,aName,aValue,anErrPtr) \
	BoLstrdic_Replace(aDic, &(bo_lstrdic_entry_t){ aName, aValue }, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define BoLstrdic_Replace2(aDic,aName,aValue,anErrPtr) \
	__extension__ ({ \
		bo_lstrdic_entry_t t = { aName, aValue }; \
		/* return */ BoLstrdic_Replace(aDic, &t, anErrPtr); \
	})
# endif /* !__myc_support(macro_retval) */
# endif /* !__myc_support(TUNO) */

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoLstrdic_Delete(
# if __myc_require(prototypes)
	bo_lstrdic_t *aDic,
	char const *aName,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_lstrdic_iterator /* = NULL */
__attribute__((nonnull(1,2)))
BoLstrdic_Findbykey(
# if __myc_require(prototypes)
	bo_lstrdic_t const *aDic,
	char const *aName,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_lstrdic_iterator /* = NULL */
__attribute__((nonnull(1,2)))
BoLstrdic_Findbyval(
# if __myc_require(prototypes)
	bo_lstrdic_t const *aDic,
	char const *aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_lstrdic_iterator /* = NULL */
__attribute__((nonnull(1,2,3)))
BoLstrdic_Findbykeyval(
# if __myc_require(prototypes)
	bo_lstrdic_t const *aDic,
	char const *aName,
	char const *aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/* ============================================================================
 * LSTRDIC::ENTRY CONSTRUCT/DESTROY
 */
# pragma mark -
# pragma mark [ lstrdic::entry construct/destroy ]
# pragma mark -

# define BO_LSTRDIC_ENTRY_Construct(aDest,aKey,aValue,_NOMEMHANDLER) \
	do { \
		char *___lsdic_ent_ctor_s; \
		size_t ___lsdic_ent_ctor_sz, ___lsdic_ent_ctor_klen; \
		struct bo_lstrdic_entry *___lsdic_ent_ctor_dest = bo_lstrdic_entry_p(aDest); \
		char *___lsdic_ent_ctor_key = (char *)(aKey), \
			*___lsdic_ent_ctor_value = (char *)(aValue); \
		assert(NULL != (___lsdic_ent_ctor_dest)); \
		assert(NULL != (___lsdic_ent_ctor_key)); \
		___lsdic_ent_ctor_klen = strlen(___lsdic_ent_ctor_key); \
		___lsdic_ent_ctor_sz = ___lsdic_ent_ctor_klen + 1; \
		if (NULL != ___lsdic_ent_ctor_value) \
			___lsdic_ent_ctor_sz += strlen(___lsdic_ent_ctor_value) + 1; \
		if ((___lsdic_ent_ctor_s = _BoMalloc(___lsdic_ent_ctor_sz,NULL))) { \
			memcpy(___lsdic_ent_ctor_s, ___lsdic_ent_ctor_key, ___lsdic_ent_ctor_klen + 1); \
			___lsdic_ent_ctor_dest->sdic_key = ___lsdic_ent_ctor_s; \
			___lsdic_ent_ctor_dest->sdic_value = NULL; \
			if (NULL != ___lsdic_ent_ctor_value) { \
				___lsdic_ent_ctor_s += ___lsdic_ent_ctor_klen + 1; \
				strcpy(___lsdic_ent_ctor_s, ___lsdic_ent_ctor_value); \
				___lsdic_ent_ctor_dest->sdic_value = ___lsdic_ent_ctor_s; \
			} \
			break; \
		} \
		_NOMEMHANDLER; \
	} while (0)

# define BO_LSTRDIC_ENTRY_CopyConstruct(aDest,aSource,_NOMEMHANDLER) \
	BO_LSTRDIC_ENTRY_Construct( \
		aDest,bo_lstrdic_entry_p(aSource)->sdic_key, \
		bo_lstrdic_entry_p(aSource)->sdic_value, \
		_NOMEMHANDLER \
	)

# define BO_LSTRDIC_ENTRY_Destroy(aDest) \
	do { \
		struct bo_lstrdic_entry *___lsdic_ent_dtor_dest = bo_lstrdic_entry_p(aDest); \
		_BoMfree((bo_pointer_t)(___lsdic_ent_dtor_dest->sdic_key)); \
		___lsdic_ent_dtor_dest->sdic_key = NULL; \
	} while (0)

/* ============================================================================
 * CONSTRUCT/DESTROY
 */
# pragma mark -
# pragma mark [ construct/destroy ]
# pragma mark -

/* void
   BO_LSTRDIC_Init(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Init(aDic) _BO_SEQLIST_Init(aDic)

/* void
   BO_LSTRDIC_Init_n(
		bo_lstrdic *aDic, size_t anElemc, {(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_Init_n(aDic,anElemc,_NOMEMHANDLER) \
	BO_SEQLIST_Init_n(bo_lstrdic_entry_t,aDic,anElemc,_NOMEMHANDLER)

/* void
   BO_LSTRDIC_Init_nx(
		bo_lstrdic *aDic,
		bo_lstrdic::const_pointer anAttrv,
		size_t anAttrc,
		{(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_Init_nx(aDic,anAttrv,anAttrc,_NOMEMHANDLER) \
	BO_SEQLIST_Init_nx_(bo_lstrdic_entry_t,aDic,anAttrv,anAttrc, \
		{ BO_LSTRDIC_ENTRY_CopyConstruct(_$1,_$2,_NOMEMHANDLER); }, \
		_NOMEMHANDLER \
	)

/* void
   BO_LSTRDIC_CopyInit_s(
		bo_lstrdic *aDic,
		bo_lstrdic const *aSource,
		{(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_CopyInit_s(aDic,aSource,_NOMEMHANDLER) \
	BO_SEQLIST_CopyInit_s(bo_lstrdic_entry_t,aDic,aSource,_NOMEMHANDLER)

/* void
   BO_LSTRDIC_CopyInit_x(
		bo_lstrdic *aDic,
		bo_lstrdic const *aSource,
		{(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_CopyInit_x(aDic,aSource,_NOMEMHANDLER) \
	BO_SEQLIST_CopyInit_x_(bo_lstrdic_entry_t,aDic,aSource, \
		{ BO_LSTRDIC_ENTRY_CopyConstruct(_$1,_$2,_NOMEMHANDLER); }, \
		_NOMEMHANDLER \
	)

/* ------------------------------------------------------------------------- */

/* void
   BO_LSTRDIC_Destroy_s(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Destroy_s(aDic) \
	_BO_SEQLIST_Destroy_s(aDic)

/* void
   BO_LSTRDIC_Destroy_x(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Destroy_x(aDic) \
	BO_SEQLIST_Destroy_x(bo_lstrdic_entry_t,aDic,BO_LSTRDIC_ENTRY_Destroy)

/* ------------------------------------------------------------------------- */

/* void
   BO_LSTRDIC_Clear_s(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Clear_s(aDic) _BO_SEQLIST_Clear_s(aDic)

/* void
   BO_LSTRDIC_Clear_x(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Clear_x(aDic) \
	BO_SEQLIST_Clear_x(bo_lstrdic_entry_t,aDic,BO_LSTRDIC_ENTRY_Destroy)

/* ============================================================================
 * PROPERTIES: ACCESS
 */
# pragma mark -
# pragma mark [ properties: access ]
# pragma mark -

/* size_t
   BO_LSTRDIC_Capacity(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Capacity(aDic) _BO_SEQLIST_Capacity(aDic)

/* size_t
   BO_LSTRDIC_Size(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Size(aDic) _BO_SEQLIST_Size(aDic)

/* bool
   BO_LSTRDIC_Empty(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Empty(aDic) _BO_SEQLIST_Empty(aDic)

/* ============================================================================
 * METHODS: ACCESS
 */
# pragma mark -
# pragma mark [ methods: access ]
# pragma mark -

/* bo_lstrdic_iterator
   BO_LSTRDIC_Begin(bo_lstrdic *aDic); */
# define BO_LSTRDIC_Begin(aDic) BO_SEQLIST_Begin(struct bo_lstrdic_entry,aDic)

/* bo_lstrdic_iterator
   BO_LSTRDIC_End(bo_lstrdic *aDic); */
# define BO_LSTRDIC_End(aDic) BO_SEQLIST_End(struct bo_lstrdic_entry,aDic)

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

/* void
   BO_LSTRDIC_ForceAdd_s(
		bo_lstrdic *aDic,
		bo_lstrdic::const_reference aEntryRef,
		{(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_ForceAdd_s(aDic,aEntryRef,_NOMEMHANDLER) \
	BO_SEQLIST_Push(bo_lstrdic_entry_t,aDic,aEntryRef,_NOMEMHANDLER)

/* void
   BO_LSTRDIC_ForceAdd_x(
		bo_lstrdic *aDic,
		bo_lstrdic::const_reference aEntryRef,
		{(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_ForceAdd_x(aDic,aEntryRef,_NOMEMHANDLER) \
	do { \
		struct bo_lstrdic_entry ___lsdic_faddx_tmp; \
		BO_LSTRDIC_ENTRY_CopyConstruct(&___lsdic_faddx_tmp,&(aEntryRef),_NOMEMHANDLER); \
		BO_SEQLIST_Push(bo_lstrdic_entry_t,aDic,___lsdic_faddx_tmp,_NOMEMHANDLER); \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_LSTRDIC_Add_s(
		bo_lstrdic *aDic,
		bo_lstrdic::const_reference aEntryRef,
		{(_EXISTHANDLER)},
		{(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_Add_s(aDic,aEntryRef,_EXISTHANDLER,_NOMEMHANDLER) \
	do { \
		struct bo_lstrdic_entry *___lsdic_adds_tmp; \
		bo_lstrdic_t *___lsdic_adds_dic = bo_lstrdic_p(aDic); \
		char const *___lsdic_adds_key = bo_lstrdic_entry_p(&(aEntryRef))->sdic_key; \
		BO_LSTRDIC_FindByKey(___lsdic_adds_dic,___lsdic_adds_key,___lsdic_adds_tmp); \
		if (NULL == ___lsdic_adds_tmp) { \
			BO_LSTRDIC_ForceAdd_s(___lsdic_adds_dic,(aEntryRef),_NOMEMHANDLER); \
			break; \
		} \
		_EXISTHANDLER; \
	} while (0)

/* void
   BO_LSTRDIC_Add_x(
		bo_lstrdic *aDic,
		bo_lstrdic::const_reference aEntryRef,
		{(_EXISTHANDLER)},
		{(_NOMEMHANDLER)}
	); */
# define BO_LSTRDIC_Add_x(aDic,aEntryRef,_EXISTHANDLER,_NOMEMHANDLER) \
	do { \
		struct bo_lstrdic_entry *___lsdic_addx_tmp; \
		bo_lstrdic_t *___lsdic_addx_dic = bo_lstrdic_p(aDic); \
		char const *___lsdic_addx_key = bo_lstrdic_entry_p(&(aEntryRef))->sdic_key; \
		BO_LSTRDIC_FindByKey(___lsdic_addx_dic,___lsdic_addx_key,___lsdic_addx_tmp); \
		if (NULL == ___lsdic_addx_tmp) { \
			BO_LSTRDIC_ForceAdd_x(___lsdic_addx_dic,(aEntryRef),_NOMEMHANDLER); \
			break; \
		} \
		_EXISTHANDLER; \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_LSTRDIC_RemoveByKey(
		bo_lstrdic *aDic,
		char const *aKey,
		bo_lstrdic::reference aEntryRef,
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_LSTRDIC_RemoveByKey(aDic,aKey,aEntryRef,_NOTFOUNDHANDLER) \
	do { \
		struct bo_lstrdic_entry *___lsdic_rmbk_tmp; \
		bo_lstrdic_t *___lsdic_rmbk_dic = bo_lstrdic_p(aDic); \
		BO_LSTRDIC_FindByKey(___lsdic_rmbk_dic,aKey,___lsdic_rmbk_tmp); \
		if (NULL != ___lsdic_rmbk_tmp) { \
			BO_LSTRDIC_Remove(___lsdic_rmbk_dic,___lsdic_rmbk_tmp,(aEntryRef)); \
			break; \
		} \
		_NOTFOUNDHANDLER; \
	} while (0)

/* void
   BO_LSTRDIC_Remove(
		bo_lstrdic *aDic,
		bo_lstrdic_iterator aPos,
		bo_lstrdic::reference aEntryRef,
	); */
# define BO_LSTRDIC_Remove(aDic,aPos,aEntryRef) \
	BO_SEQLIST_Remove(struct bo_lstrdic_entry,aDic,aPos,aEntryRef)

/* ------------------------------------------------------------------------- */

/* void
   BO_LSTRDIC_DeleteByKey_s(
		bo_lstrdic *aDic,
		char const *aKey,
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_LSTRDIC_DeleteByKey_s(aDic,aKey,_NOTFOUNDHANDLER) \
	do { \
		struct bo_lstrdic_entry *___lsdic_delbk_pos; \
		bo_lstrdic_t *___lsdic_delbk_dic = bo_lstrdic_p(aDic); \
		BO_LSTRDIC_FindByKey(___lsdic_delbk_dic,aKey,___lsdic_delbk_pos); \
		if (NULL != ___lsdic_delbk_pos) { \
			struct bo_lstrdic_entry ___lsdic_delbk_tmp; \
			BO_LSTRDIC_Remove(___lsdic_delbk_dic,___lsdic_delbk_pos,___lsdic_delbk_tmp); \
			break; \
		} \
		_NOTFOUNDHANDLER; \
	} while (0)

/* void
   BO_LSTRDIC_DeleteByKey_x(
		bo_lstrdic *aDic,
		char const *aKey,
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_LSTRDIC_DeleteByKey_x(aDic,aKey,_NOTFOUNDHANDLER) \
	do { \
		struct bo_lstrdic_entry *___lsdic_delbkx_pos; \
		bo_lstrdic_t *___lsdic_delbkx_dic = bo_lstrdic_p(aDic); \
		BO_LSTRDIC_FindByKey(___lsdic_delbkx_dic,aKey,___lsdic_delbkx_pos); \
		if (NULL != ___lsdic_delbkx_pos) { \
			struct bo_lstrdic_entry ___lsdic_delbkx_tmp; \
			BO_LSTRDIC_Remove(___lsdic_delbkx_dic,___lsdic_delbkx_pos,___lsdic_delbkx_tmp); \
			BO_LSTRDIC_ENTRY_Destroy(&___lsdic_delbkx_tmp); \
			break; \
		} \
		_NOTFOUNDHANDLER; \
	} while (0)

/* ------------------------------------------------------------------------- */

# ifndef __cplusplus

/* void
   BO_LSTRDIC_FindByKey(
		bo_lstrdic *aDic,
		char const *aKey,
		bo_lstrdic_iterator &anEntryPtrRef
	); */
# define BO_LSTRDIC_FindByKey(aDic,aKey,anEntryPtrRef) \
	do { \
		char const *___lsdic_findbk_key = (char const *)(aKey); \
		(anEntryPtrRef) = NULL; \
		BO_SEQLIST_Enumerate_(bo_lstrdic_entry_t,aDic, \
			{ \
				if (0 == strcmp(_$1->sdic_key,___lsdic_findbk_key)) { \
					(anEntryPtrRef) = (bo_pointer_t)_$1; \
					break; \
				} \
			} \
		); \
	} while (0)

/* void
   BO_LSTRDIC_FindByValue(
		bo_lstrdic *aDic,
		char const *aMappedValue,
		bo_lstrdic_iterator &anEntryPtrRef
	); */
# define BO_LSTRDIC_FindByValue(aDic,aMappedValue,anEntryPtrRef) \
	do { \
		char const *___lsdic_findbv_mapval = (char const *)(aMappedValue); \
		(anEntryPtrRef) = NULL; \
		BO_SEQLIST_Enumerate_(bo_lstrdic_entry_t,aDic, \
			{ \
				if (NULL != _$1->sdic_value && \
					0 == strcmp(_$1->sdic_value,___lsdic_findbv_mapval)) \
				{ \
					(anEntryPtrRef) = (bo_pointer_t)_$1; \
					break; \
				} \
			} \
		); \
	} while (0)

/* void
   BO_LSTRDIC_FindByKeyValue(
		bo_lstrdic *aDic,
		char const *aKey,
		char const *aMappedValue,
		bo_lstrdic_iterator &anEntryPtrRef
	); */
# define BO_LSTRDIC_FindByKeyValue(aDic,aKey,aMappedValue,anEntryPtrRef) \
	do { \
		char const *___lsdic_findbkv_key = (char const *)(aKey); \
		char const *___lsdic_findbkv_mapval = (char const *)(aMappedValue); \
		(anEntryPtrRef) = NULL; \
		BO_SEQLIST_Enumerate_(bo_lstrdic_entry_t,aDic, \
			{ \
				if (NULL != _$1->sdic_value && \
					0 == strcmp(_$1->sdic_key,___lsdic_findbkv_key) && \
					0 == strcmp(_$1->sdic_value,___lsdic_findbkv_mapval)) \
				{ \
					(anEntryPtrRef) = (bo_pointer_t)_$1; \
					break; \
				} \
			} \
		); \
	} while (0)

# else /* __cplusplus */

# define BO_LSTRDIC_FindByKey(aDic,aKey,anEntry) \
	do { \
		char const *___lsdic_findbk_key = (char const *)(aKey); \
		(anEntry) = NULL; \
		BO_SEQLIST_Enumerate_(bo_lstrdic_entry_t,aDic, \
			{ \
				if (0 == strcmp(_$1->sdic_key,___lsdic_findbk_key)) { \
					*(void**)&(anEntry) = (void*)_$1; \
					break; \
				} \
			} \
		); \
	} while (0)

# define BO_LSTRDIC_FindByValue(aDic,aMappedValue,anEntry) \
	do { \
		char const *___lsdic_findbv_mapval = (char const *)(aMappedValue); \
		(anEntry) = NULL; \
		BO_SEQLIST_Enumerate_(bo_lstrdic_entry_t,aDic, \
			{ \
				if (_$1->sdic_value && \
					0 == strcmp(_$1->sdic_value,___lsdic_findbv_mapval)) \
				{ \
					*(void**)&(anEntry) = (void*)_$1; \
					break; \
				} \
			} \
		); \
	} while (0)

# define BO_LSTRDIC_FindByKeyValue(aDic,aKey,aMappedValue,anEntry) \
	do { \
		char const *___lsdic_findbkv_key = (char const *)(aKey); \
		char const *___lsdic_findbkv_mapval = (char const *)(aMappedValue); \
		(anEntry) = NULL; \
		BO_SEQLIST_Enumerate_(bo_lstrdic_entry_t,aDic, \
			{ \
				if (_$1->sdic_value && \
					0 == strcmp(_$1->sdic_key,___lsdic_findbkv_key) && \
					0 == strcmp(_$1->sdic_value,___lsdic_findbkv_mapval)) \
				{ \
					*(void**)&(anEntry) = (void*)_$1; \
					break; \
				} \
			} \
		); \
	} while (0)

# endif /* __cplusplus */

/* ============================================================================
 * LOOP HELPERS
 * @{
 */
# pragma mark -
# pragma mark [ loop helpers ]
# pragma mark -

/*
	bo_lstrdic dic;
	bo_lstrdic_iterator iter;
	...
	BO_LSTRDIC_FOREACH(iter, &dic)
		...
	FOREACH_END
 */
# define BO_LSTRDIC_FOREACH(anIter,aDic) \
	BO_SEQLIST_FOREACH(bo_lstrdic_entry_t,anIter,aDic)

/*
	bo_lstrdic dic;
	...
	// iterator _$1;
	BO_LSTRDIC_FOREACH_(&dic)
		...
	FOREACH_END
 */
# define BO_LSTRDIC_FOREACH_(aDic) \
	BO_SEQLIST_FOREACH_(bo_lstrdic_entry_t,aDic)

# ifndef FOREACH_END
#  define FOREACH_END }}
# endif
/* @} */

#endif /* !__BO_LIST_LIGHTSTRINGDICTIONARY__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
