/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Rtbtfcns.h	(V.Sokur)
 * include/Bo/fundis/Rtbtfcns.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FUNDIS_RTBTFCNS__
# define __BO_FUNDIS_RTBTFCNS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/fundis/Rtbtree.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * CONSTRUCT/DESTROY
 */
# pragma mark -

__myc_flinkage__(public,libBo) struct bo_rtbtree *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ NULL = (BoK_SysError_NoMemory); ];
))
BoRTBTCreate(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
))
__attribute__((nonnull(1)))
BoRTBTInit(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_AIBUpdateError_AlreadyExists, BoK_SysError_NoMemory, ...);
	];
))
__attribute__((nonnull(1)))
BoRTBTInit_cp(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_rtbtree const *aSource __myc_defval__(NULL),
	struct bo_btninfo_opv const *anInfoOpv __myc_defval__(NULL),
			/* (NULL != @source) --> (NULL != @info-opv) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
))
__attribute__((nonnull(1,2)))
BoRTBTInit_mv(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_rtbtree *aSource,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) void
__attribute__((nonnull(1)))
BoRTBTDestroy(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) void
BoRTBTFree(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv
# endif
	);

/* ----------------------------------------------------------------------------
 * METHODS: ALGORITHMS
 */
# pragma mark -

__myc_flinkage__(public,libBo) void
__attribute__((nonnull(1,2)))
BoRTBTDump(
# if __myc_require(prototypes)
	struct bo_rtbtree const *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	aorp_object_t aStream
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_rtbtree_iterator
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ NULL = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTFind(
# if __myc_require(prototypes)
	struct bo_rtbtree const *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aRqInfo,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_rtbtree_iterator
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ NULL = (BoK_AIBUpdateError_AlreadyExists); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTInsnode(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	struct bo_btree_node *aNode,
	int /* _Bool */ Force __myc_defval__(0),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_rtbtree_iterator
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_AIBUpdateError_AlreadyExists,
		        BoK_AIBServiceError_NestLevelLimit);
	];
))
__attribute__((nonnull(1,2,3)))
BoRTBTInsnode_b(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	struct bo_btree_node *aNode,
	int /* _Bool */ Force __myc_defval__(0),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct bo_btree_node *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ NULL = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTRmnode(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aRqInfo,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct bo_btree_node *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ NULL = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTRmnode_b(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aRqInfo,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_rtbtree_iterator
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_AIBUpdateError_AlreadyExists, BoK_SysError_NoMemory, ...);
	];
))
__attribute__((nonnull(1,2)))
BoRTBTInsert(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t anInfo,
	int /* _Bool */ Force __myc_defval__(0),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_rtbtree_iterator
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_AIBUpdateError_AlreadyExists,
		        BoK_AIBServiceError_NestLevelLimit,
		        BoK_SysError_NoMemory, ...);
	];
))
__attribute__((nonnull(1,2)))
BoRTBTInsert_b(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t anInfo,
	int /* _Bool */ Force __myc_defval__(0),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_pointer_t
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ BoS_EPTR = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTRemove(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aRqInfo,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_pointer_t
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ BoS_EPTR = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTRemove_b(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aRqInfo,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTDelete(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aRqInfo,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
BoRTBTDelete_b(
# if __myc_require(prototypes)
	struct bo_rtbtree *aTree,
	struct bo_btninfo_opv const *anInfoOpv,
	bo_pointer_t aRqInfo,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

# pragma mark -

/* NOT FOR YOUR BUSINESS @{ */
__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [ BoS_ERR = (BoK_AIBUpdateError_AlreadyExists); ];
))
__attribute__((nonnull(1,2,3)))
_BoRTBTLinknode(
# if __myc_require(prototypes)
	struct bo_rtbtree *,
	struct bo_btninfo_opv const *,
	struct bo_btree_node *,
	int /* _Bool */ ,
	struct bo_btree_node ** __myc_defval__(NULL),
	struct bo_btree_node ** __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_AIBUpdateError_AlreadyExists,
		           BoK_AIBServiceError_NestLevelLimit); ];
))
__attribute__((nonnull(1,2,3)))
_BoRTBTLinknode_b(
# if __myc_require(prototypes)
	struct bo_rtbtree *,
	struct bo_btninfo_opv const *,
	struct bo_btree_node *,
	int /* _Bool */ ,
	struct bo_btree_node ** __myc_defval__(NULL),
	struct bo_btree_node ** __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct bo_btree_node *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ NULL = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
_BoRTBTUnlinknode(
# if __myc_require(prototypes)
	struct bo_rtbtree *,
	struct bo_btninfo_opv const *,
	bo_pointer_t ,
	struct bo_btree_node ** __myc_defval__(NULL),
	struct bo_btree_node ** __myc_defval__(NULL)
# endif
	);

/* -------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct bo_btree_node *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [ NULL = (BoK_AIBNameError_NoSuchEntry); ];
))
__attribute__((nonnull(1,2,3)))
_BoRTBTUnlinknode_b(
# if __myc_require(prototypes)
	struct bo_rtbtree *,
	struct bo_btninfo_opv const *,
	bo_pointer_t ,
	struct bo_btree_node ** __myc_defval__(NULL),
	struct bo_btree_node ** __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS
/* @} */

#endif /* !__BO_FUNDIS_RTBTFCNS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
