/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Rtbtree.h	(V.Sokur)
 * include/Bo/fundis/Rtbtree.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FUNDIS_RTBTREE__
# define __BO_FUNDIS_RTBTREE__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/fundis/Absbtree.h>

# if !__myc_support(macro_retval)
#  include <Bo/Debug.h>
# endif

/******************************************************************************
 * %% BeginSection: rtbtree declarations
 */
# pragma mark -
# pragma mark [ rtbtree declarations ]
# pragma mark -

/* right-threaded b-tree proper */
struct __MYC_QALIGN(SIZEOF_PTR) bo_rtbtree {
	struct {
		struct bo_btree_node  *btn_link[2];
		unsigned               btn_lbackref :1;
		unsigned               btn_rbackref :1;
		unsigned               btn_first :1;
		unsigned               btn_last :1;
		unsigned               btn_last_defo :1;
		signed                 btn_balance :3;
		unsigned               btn_pad :24;
	}       bt_head;
	size_t  bt_elemc;
};

# define bo_rtbtree_p(aTree) ((struct bo_rtbtree *)(aTree))

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_rtbtree,
	sizeof(struct bo_rtbtree_s) == sizeof(struct bo_rtbtree));
# endif /* _DEBUG */
/* @} */

/* ============================================================================
 * NODE PROPERTIES
 */

# define BO_RTBTREE_NODE_UserBits(aNode) BO_BTREE_NODE_UserBits(aNode)

/* bo_pointer_t
   BO_RTBTREE_NODE_UserInfo(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_UserInfo(aNode) BO_BTREE_NODE_UserInfo(aNode)

/* bo_btree_node *
   BO_RTBTREE_NODE_Link(bo_btree_node *aNode, bo_btree_dir_t aDirection); */
# define BO_RTBTREE_NODE_Link(aNode,aDir) BO_BTREE_NODE_Link(aNode,aDir)

/* bo_btree_node * (=NULL)
   BO_RTBTREE_NODE_XLink(bo_btree_node *aNode, bo_btree_dir_t aDirection); */
# define BO_RTBTREE_NODE_XLink(aNode,aDir) \
	((BoK_BTREE_RIGHT_DIR == (aDir) && BO_BTREE_NODE_IsRBackref(aNode)) \
		? NULL \
		: BO_BTREE_NODE_Link(aNode,aDir))

/* bo_btree_node *
   BO_RTBTREE_NODE_BLink(bo_btree_node *aNode, int aBalance); */
# define BO_RTBTREE_NODE_BLink(aNode,aBalance) BO_BTREE_NODE_BLink(aNode,aBalance)

/* bo_btree_node *
   BO_RTBTREE_NODE_LLink(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_LLink(aNode) BO_BTREE_NODE_LLink(aNode)

/* bo_btree_node *
   BO_RTBTREE_NODE_RLink(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_RLink(aNode) BO_BTREE_NODE_RLink(aNode)

/* bool
   BO_RTBTREE_NODE_IsLast(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_IsLast(aNode) BO_BTREE_NODE_IsLast(aNode)

/* bool
   BO_RTBTREE_NODE_IsLast_defo(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_IsLast_defo(aNode) BO_BTREE_NODE_IsLast_defo(aNode)

/* bool
   BO_RTBTREE_NODE_IsBackref(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_IsBackref(aNode) BO_BTREE_NODE_IsRBackref(aNode)

/* bool
   BO_RTBTREE_NODE_IsUnbalanced(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_IsUnbalanced(aNode) BO_BTREE_NODE_IsUnbalanced(aNode)

/* bool
   BO_RTBTREE_NODE_IsLeaf(bo_btree_node *aNode); */
# define BO_RTBTREE_NODE_IsLeaf(aNode) \
	(NULL == BO_BTREE_NODE_LLink(aNode) && BO_BTREE_NODE_IsRBackref(aNode))

/* ============================================================================
 * LPR ITERATOR
 */
# pragma mark -
# pragma mark [ lpr iterator ]
# pragma mark -

typedef bo_btree_iterator          bo_rtbtree_iterator;
typedef bo_btree_const_iterator    bo_rtbtree_const_iterator;
typedef bo_rtbtree_iterator        bo_rtbtree_lpr_iterator;
typedef bo_rtbtree_const_iterator  bo_rtbtree_const_lpr_iterator;

/* ------------------------------------------------------------------------- */

/* bo_pointer_t
   _BO_RTBTREE_ITER_Deref(bo_rtbtree_iterator anIter); */
# define _BO_RTBTREE_ITER_Deref(anIter) _BO_BTREE_ITER_Deref(anIter)

/* _Tp
   BO_RTBTREE_ITER_Deref< _Tp >(bo_rtbtree_iterator anIter); */
# define BO_RTBTREE_ITER_Deref(_Tp,anIter) BO_BTREE_ITER_Deref(_Tp,anIter)

/* ------------------------------------------------------------------------- */

/* void
   _BO_RTBTREE_ITER_Inc_lpr(bo_rtbtree_iterator &anIter); */
# define _BO_RTBTREE_ITER_Inc_lpr(anIter) _BO_BTREE_ITER_Inc_lpr(anIter)

/* void
   __BO_RTBTREE_ITER_Inc_lpr(bo_rtbtree_iterator &anIter); */
# define __BO_RTBTREE_ITER_Inc_lpr(anIter) __BO_BTREE_ITER_Inc_lpr(anIter)

/* ------------------------------------------------------------------------- */

/* bo_rtbtree_iterator (=NULL)
   _BO_RTBTREE_ITER_Next_lpr(bo_rtbtree_iterator anIter); */
# define _BO_RTBTREE_ITER_Next_lpr(anIter) _BO_BTREE_ITER_Next_lpr(anIter)

/* bo_rtbtree_iterator
   __BO_RTBTREE_ITER_Next_lpr(bo_rtbtree_iterator anIter); */
# define __BO_RTBTREE_ITER_Next_lpr(anIter) __BO_BTREE_ITER_Next_lpr(anIter)

/* ============================================================================
 * PLR ITERATOR
 */
# pragma mark -
# pragma mark [ plr iterator ]
# pragma mark -

typedef bo_btree_plr_iterator          bo_rtbtree_plr_iterator;
typedef bo_btree_const_plr_iterator    bo_rtbtree_const_plr_iterator;

/* ------------------------------------------------------------------------- */

/* bo_pointer_t
   _BO_RTBTREE_PLR_ITER_Deref(bo_rtbtree_plr_iterator anIter); */
# define _BO_RTBTREE_PLR_ITER_Deref(anIter) _BO_BTREE_PLR_ITER_Deref(anIter)

/* _Tp
   BO_RTBTREE_PLR_ITER_Deref< _Tp >(bo_rtbtree_plr_iterator anIter); */
# define BO_RTBTREE_PLR_ITER_Deref(_Tp,anIter) BO_BTREE_PLR_ITER_Deref(_Tp,anIter)

/* ----------------------------------------------------------------------------
 * _BO_RTBTREE_ITER_Inc_plr
 */
# pragma mark -

/* void
   _BO_RTBTREE_ITER_Inc_plr(bo_rtbtree_plr_iterator &anIter); */
# define _BO_RTBTREE_ITER_Inc_plr(anIter) \
	do { \
		register bo_rtbtree_iterator ___rtbt_iter_inc_plr_it; \
		if (NULL != (___rtbt_iter_inc_plr_it = BO_RTBTREE_NODE_LLink((anIter)->cur_))) { \
			if (!BO_BTREE_NODE_IsRBackref((anIter)->cur_)) { \
				assert((anIter)->np_ < BoL_BTREE_HIGH_MAX); \
				(anIter)->next_[(anIter)->np_++] = BO_RTBTREE_NODE_RLink((anIter)->cur_); \
			} \
			(anIter)->cur_ = ___rtbt_iter_inc_plr_it; \
			break; \
		} \
		if (!BO_BTREE_NODE_IsRBackref((anIter)->cur_)) { \
			(anIter)->cur_ = BO_RTBTREE_NODE_RLink((anIter)->cur_); \
			break; \
		} \
		if (0 < (anIter)->np_) { \
			(anIter)->cur_ = (anIter)->next_[--(anIter)->np_]; \
			break; \
		} \
		(anIter)->cur_ = NULL; \
		(anIter) = NULL; \
	} while (0)

/* ----------------------------------------------------------------------------
 * __BO_RTBTREE_ITER_Inc_plr
 */
# pragma mark -

/* void
   __BO_RTBTREE_ITER_Inc_plr(bo_rtbtree_plr_iterator &anIter); */
# define __BO_RTBTREE_ITER_Inc_plr(anIter) \
	do { \
		register bo_rtbtree_iterator ___rtbt_iter_inc_plr__it; \
		if (NULL != (___rtbt_iter_inc_plr__it = BO_RTBTREE_NODE_LLink((anIter)->cur_))) { \
			if (!BO_BTREE_NODE_IsRBackref((anIter)->cur_)) { \
				assert((anIter)->np_ < BoL_BTREE_HIGH_MAX); \
				(anIter)->next_[(anIter)->np_++] = BO_RTBTREE_NODE_RLink((anIter)->cur_); \
			} \
			(anIter)->cur_ = ___rtbt_iter_inc_plr__it; \
			break; \
		} \
		if (!BO_BTREE_NODE_IsRBackref((anIter)->cur_)) { \
			(anIter)->cur_ = BO_RTBTREE_NODE_RLink((anIter)->cur_); \
			break; \
		} \
		assert(0 < (anIter)->np_); \
		(anIter)->cur_ = (anIter)->next_[--(anIter)->np_]; \
	} while (0)

/* ----------------------------------------------------------------------------
 * _BO_RTBTREE_ITER_Next_plr
 */
# pragma mark -

/* bo_rtbtree_plr_iterator (=NULL)
   _BO_RTBTREE_ITER_Next_plr(bo_rtbtree_plr_iterator anIter); */
# if __myc_support(macro_retval)

#  define _BO_RTBTREE_ITER_Next_plr(anIter) \
	__extension__ ({ \
		register bo_rtbtree_plr_iterator ___rtbt_iter_next_plr_it = (bo_btree_plr_iterator)(anIter); \
		_BO_RTBTREE_ITER_Inc_plr(___rtbt_iter_next_plr_it); \
		/* return */ (___rtbt_iter_next_plr_it); \
	})

# else /* !__myc_support(macro_retval) */

__myc_extern_inline__ bo_rtbtree_plr_iterator
_BO_RTBTREE_ITER_Next_plr(
#if __myc_require(prototypes)
	register bo_rtbtree_plr_iterator anIter
#else
	anIter
#endif
	)
#if !__myc_require(prototypes)
	register bo_rtbtree_plr_iterator anIter;
#endif
{
	_BO_RTBTREE_ITER_Inc_plr(anIter);
	return (anIter);
}

# endif /* __myc_support(macro_retval) */

/* ----------------------------------------------------------------------------
 * __BO_RTBTREE_ITER_Next_plr
 */
# pragma mark -

/* bo_rtbtree_plr_iterator (=NULL)
   __BO_RTBTREE_ITER_Next_plr(bo_rtbtree_plr_iterator anIter); */
# if __myc_support(macro_retval)

#  define __BO_RTBTREE_ITER_Next_plr(anIter) \
	__extension__ ({ \
		register bo_rtbtree_plr_iterator ___rtbt_iter_next_plr__it = (bo_btree_plr_iterator)(anIter); \
		__BO_RTBTREE_ITER_Inc_plr(___rtbt_iter_next_plr__it); \
		/* return */ (___rtbt_iter_next_plr__it); \
	})

# else /* !__myc_support(macro_retval) */

__myc_extern_inline__ bo_rtbtree_plr_iterator
__BO_RTBTREE_ITER_Next_plr(
#if __myc_require(prototypes)
	register bo_rtbtree_plr_iterator anIter
#else
	anIter
#endif
	)
#if !__myc_require(prototypes)
	register bo_rtbtree_plr_iterator anIter;
#endif
{
	__BO_RTBTREE_ITER_Inc_plr(anIter);
	return (anIter);
}

# endif /* __myc_support(macro_retval) */

/* ============================================================================
 * CONSTRUCT/DESTROY
 */
# pragma mark -
# pragma mark [ construct/destroy ]
# pragma mark -

/* void
   _BO_RTBTREE_Init(bo_rtbtree *aTree); */
# define _BO_RTBTREE_Init(aTree) \
	do { \
		struct bo_rtbtree *___rtbt_init_list = bo_rtbtree_p(aTree); \
		___rtbt_init_list->bt_head.btn_link[0] = NULL; \
		___rtbt_init_list->bt_head.btn_link[1] = NULL; \
		__bo_btree_p(___rtbt_init_list)->bt_head.btn_internal = 0; \
		___rtbt_init_list->bt_elemc = 0; \
		/* @todo: do we need this? */ \
		BO_RTBTREE_NODE_RLink(BO_RTBTREE_Head_(___rtbt_init_list)) = \
			BO_RTBTREE_Head_(___rtbt_init_list); \
	} while (0)

/* ============================================================================
 * PROPERTIES: ACCESS
 */
# pragma mark -
# pragma mark [ properties: access ]
# pragma mark -

/* bo_btree::unnamed_node *
   _BO_RTBTREE_Head(bo_rtbtree *aTree); */
# define _BO_RTBTREE_Head(aTree) _BO_BTREE_Head(aTree)

/* bo_btree_node *
   BO_RTBTREE_Head_(bo_rtbtree *aTree); */
# define BO_RTBTREE_Head_(aTree) BO_BTREE_Head_(aTree)

/* bo_btree::unnamed_node *
   _BO_RTBTREE_Tail(bo_rtbtree *aTree); */
# define _BO_RTBTREE_Tail(aTree) _BO_BTREE_Tail(aTree)

/* bo_btree_node *
   BO_RTBTREE_Tail_(bo_rtbtree *aTree); */
# define BO_RTBTREE_Tail_(aTree) BO_BTREE_Tail_(aTree)

/* ------------------------------------------------------------------------- */

/* size_t
   _BO_RTBTREE_Size(bo_rtbtree *aTree); */
# define _BO_RTBTREE_Size(aTree) (bo_rtbtree_p(aTree)->bt_elemc)

/* bool
   _BO_RTBTREE_Empty(bo_rtbtree *aTree); */
# define _BO_RTBTREE_Empty(aTree) (0 == _BO_RTBTREE_Size(aTree))

/* ============================================================================
 * METHODS: ACCESS
 */
# pragma mark -
# pragma mark [ methods: access ]
# pragma mark -

/* bo_rtbtree_iterator (=NULL)
   _BO_RTBTREE_Begin(bo_rtbtree *aTree); */
# define _BO_RTBTREE_Begin(aTree) _BO_RTBTREE_Begin_lpr(aTree)

/* bo_rtbtree_iterator (=NULL)
   _BO_RTBTREE_Begin_lpr(bo_rtbtree *aTree); */
# define _BO_RTBTREE_Begin_lpr(aTree) _BO_RTBTREE_First_lpr(aTree)

/* bo_rtbtree_plr_iterator (=NULL)
   _BO_RTBTREE_Begin_plr(bo_rtbtree *aTree, struct bo_btree_plr_iterator_s anIterBuffer); */
# define _BO_RTBTREE_Begin_plr(aTree, anIterBuffer) \
	((anIterBuffer).np_ = 0, \
		((anIterBuffer).cur_ = BO_RTBTREE_NODE_LLink(BO_RTBTREE_Head_(aTree))) \
			? &(anIterBuffer) \
			: NULL)

/* ----------------------------------------------------------------------------
 * _BO_RTBTREE_First_lpr
 */
# pragma mark -

/* bo_btree_node * (=NULL)
   _BO_RTBTREE_First_lpr(bo_rtbtree const *aTree); */
# define _BO_RTBTREE_First_lpr(aTree) _im_BO_RTBTREE_First_lpr(bo_rtbtree_p(aTree))

# if __myc_support(macro_retval)

#  define _im_BO_RTBTREE_First_lpr(aTree) \
	__extension__ ({ \
		register struct bo_btree_node *___rtbt_first_lpr_it, *___rtbt_first_lpr_retval; \
		___rtbt_first_lpr_it = BO_RTBTREE_Head_(aTree); \
		if (NULL != (___rtbt_first_lpr_retval = BO_RTBTREE_NODE_LLink(___rtbt_first_lpr_it))) { \
			while (NULL != (___rtbt_first_lpr_it = BO_RTBTREE_NODE_LLink(___rtbt_first_lpr_retval))) \
				___rtbt_first_lpr_retval = ___rtbt_first_lpr_it; \
		} \
		/* return */ (___rtbt_first_lpr_retval); \
	})

# else /* !__myc_support(macro_retval) */

__myc_extern_inline__ struct bo_btree_node *
_im_BO_RTBTREE_First_lpr(
#if __myc_require(prototypes)
	struct bo_rtbtree const *aTree
#else
	aTree
#endif
	)
#if !__myc_require(prototypes)
	struct bo_rtbtree const *aTree;
#endif
{
	struct bo_btree_node *retval, *t;
	t = BO_RTBTREE_Head_(aTree);
	if (NULL != (retval = BO_RTBTREE_NODE_LLink(t))) {
		while (NULL != (t = BO_RTBTREE_NODE_LLink(retval)))
			retval = t;
	}
	return (retval);
}

# endif /* __myc_support(macro_retval) */

/* ----------------------------------------------------------------------------
 * _BO_RTBTREE_Last_lpr
 */
# pragma mark -

/* bo_btree_node * (=NULL)
   _BO_RTBTREE_Last_lpr(bo_rtbtree const *aTree); */
# define _BO_RTBTREE_Last_lpr(aTree) _im_BO_RTBTREE_Last_lpr(bo_rtbtree_p(aTree))

# if __myc_support(macro_retval)

#  define _im_BO_RTBTREE_Last_lpr(aTree) \
	__extension__ ({ \
		register struct bo_btree_node *___rtbt_last_lpr_it, *___rtbt_last_lpr_retval; \
		___rtbt_last_lpr_it = BO_RTBTREE_Head_(aTree); \
		if (NULL != (___rtbt_last_lpr_retval = BO_RTBTREE_NODE_LLink(___rtbt_last_lpr_it))) { \
			while (!BO_BTREE_NODE_IsRBackref(___rtbt_last_lpr_retval)) \
				___rtbt_last_lpr_retval = BO_RTBTREE_NODE_RLink(___rtbt_last_lpr_retval); \
		} \
		/* return */ (___rtbt_last_lpr_retval); \
	})

# else /* !__myc_support(macro_retval) */

__myc_extern_inline__ struct bo_btree_node *
_im_BO_RTBTREE_Last_lpr(
#if __myc_require(prototypes)
	struct bo_rtbtree const *aTree
#else
	aTree
#endif
	)
#if !__myc_require(prototypes)
	struct bo_rtbtree const *aTree;
#endif
{
	struct bo_btree_node *retval, *t;
	t = BO_RTBTREE_Head_(aTree);
	if (NULL != (retval = BO_RTBTREE_NODE_LLink(t))) {
		while (!BO_BTREE_NODE_IsRBackref(retval))
			retval = BO_RTBTREE_NODE_RLink(retval);
	}
	return (retval);
}

# endif /* __myc_support(macro_retval) */

/* ============================================================================
 * METHODS: HELP-ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: help-algorithms ]
# pragma mark -

# define _BO_RTBTREE_RotateLeft(aParent,aRight) \
	do { \
		register struct bo_btree_node *___rtbt_rotl_node; \
		___rtbt_rotl_node = BO_RTBTREE_NODE_LLink(aRight); \
		if (NULL == ___rtbt_rotl_node) { \
			___rtbt_rotl_node = (aRight); \
			(aParent)->btn_rbackref = 1; \
		} \
		BO_RTBTREE_NODE_RLink(aParent) = ___rtbt_rotl_node; \
		BO_RTBTREE_NODE_LLink(aRight) = (aParent); \
	} while (0)

/* ------------------------------------------------------------------------- */

# define _BO_RTBTREE_RotateRight(aParent,aLeft) \
	do { \
		register struct bo_btree_node *___rtbt_rotr_node; \
		___rtbt_rotr_node = BO_RTBTREE_NODE_RLink(aLeft); \
		if (BO_BTREE_NODE_IsRBackref(aLeft)) { \
			___rtbt_rotr_node = NULL; \
			(aLeft)->btn_rbackref = 0; \
		}  \
		BO_RTBTREE_NODE_LLink(aParent) = ___rtbt_rotr_node; \
		BO_RTBTREE_NODE_RLink(aLeft) = (aParent); \
	} while (0)

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_Find
 */
# pragma mark -

/* ()
   BO_RTBTREE_Find< _Tp, _Tfield >(
		_Tp *anIter,
		bo_rtbtree const *aTree,
		{(_COMPARATOR+_SUCCESSHANDLER: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_Find(_Tp,_Tfield,anIter,aTree,_COMPARATOR,_NOTFOUNDHANDLER) \
	do { \
		bo_mismatch_t _$0; \
		struct bo_btree_node const *___rtbt_find_node; \
 \
		assert(NULL != (aTree)); \
 \
		___rtbt_find_node = BO_RTBTREE_Head_(aTree); \
 \
		if (NULL == (___rtbt_find_node = BO_RTBTREE_NODE_LLink(___rtbt_find_node))) { \
			/* return (NULL); */ _NOTFOUNDHANDLER; \
			assert(0); \
		} \
 \
		for (;;) { \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_find_node); \
			_COMPARATOR/*(_$0,anIter)*/; \
 \
			if (0 < _$0) { \
				if (NULL == (___rtbt_find_node = BO_RTBTREE_NODE_LLink(___rtbt_find_node))) { \
					/* return (NULL); */ _NOTFOUNDHANDLER; \
					assert(0); \
				} \
				continue; \
			} \
 \
			if (BO_BTREE_NODE_IsRBackref(___rtbt_find_node)) { \
				/* return (NULL); */ _NOTFOUNDHANDLER; \
				assert(0); \
			} \
			___rtbt_find_node = BO_RTBTREE_NODE_RLink(___rtbt_find_node); \
		} \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_Find_
 */
# pragma mark -

/* ()
   BO_RTBTREE_Find_< _Tp, _Tfield >(
		bo_rtbtree const *aTree,
		{(_COMPARATOR+_SUCCESSHANDLER: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_Find_(_Tp,_Tfield,aTree,_COMPARATOR,_NOTFOUNDHANDLER) \
	do { \
		_Tp *_$1; \
		bo_mismatch_t _$0; \
		struct bo_btree_node const *___rtbt_find__node; \
 \
		assert(NULL != (aTree)); \
 \
		___rtbt_find__node = BO_RTBTREE_Head_(aTree); \
 \
		if (NULL == (___rtbt_find__node = BO_RTBTREE_NODE_LLink(___rtbt_find__node))) { \
			/* return (NULL); */ _NOTFOUNDHANDLER; \
			assert(0); \
		} \
 \
		for (;;) { \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_find__node); \
			_COMPARATOR/*(_$0,_$1)*/; \
 \
			if (0 < _$0) { \
				if (NULL == (___rtbt_find__node = BO_RTBTREE_NODE_LLink(___rtbt_find__node))) { \
					/* return (NULL); */ _NOTFOUNDHANDLER; \
					assert(0); \
				} \
				continue; \
			} \
 \
			if (BO_BTREE_NODE_IsRBackref(___rtbt_find__node)) { \
				/* return (NULL); */ _NOTFOUNDHANDLER; \
				assert(0); \
			} \
			___rtbt_find__node = BO_RTBTREE_NODE_RLink(___rtbt_find__node); \
		} \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_DoLinkNode
 */
# pragma mark -

/* ()
   BO_RTBTREE_DoLinkNode(
		bo_rtbtree *aTree,
		bo_btree_node *aParent,
		bo_btree_node *anObject,
		bo_btree_dir_t aDir,
		{(_SUCCESSHANDLER)}
	); */
# define BO_RTBTREE_DoLinkNode(aTree,aParent,anObject,aDir,_SUCCESSHANDLER) \
	do { \
		struct bo_btree_node *___rtbt_dolinkn_head, \
			*___rtbt_dolinkn_parent, \
			*___rtbt_dolinkn_onode; \
 \
		assert(NULL != (aTree)); \
		assert(NULL != (aParent)); \
		assert(NULL != (anObject)); \
 \
		___rtbt_dolinkn_head = BO_RTBTREE_Head_(aTree); \
		___rtbt_dolinkn_parent = (aParent); \
		___rtbt_dolinkn_onode = (anObject); \
 \
		BO_RTBTREE_NODE_LLink(___rtbt_dolinkn_onode) = NULL; \
		BO_RTBTREE_NODE_RLink(___rtbt_dolinkn_onode) = ___rtbt_dolinkn_parent; \
 \
		if (___rtbt_dolinkn_parent == ___rtbt_dolinkn_head) { \
			BO_RTBTREE_NODE_LLink(___rtbt_dolinkn_parent) = ___rtbt_dolinkn_onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (BoK_BTREE_LEFT_DIR == (aDir)) { \
			BO_RTBTREE_NODE_LLink(___rtbt_dolinkn_parent) = ___rtbt_dolinkn_onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		/* (BoK_BTREE_RIGHT_DIR == (aDir)) */ \
		{ \
			BO_RTBTREE_NODE_RLink(___rtbt_dolinkn_onode) = BO_RTBTREE_NODE_RLink(___rtbt_dolinkn_parent); \
			BO_RTBTREE_NODE_RLink(___rtbt_dolinkn_parent) = ___rtbt_dolinkn_onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_LinkNode
 */
# pragma mark -

/* ()
   BO_RTBTREE_LinkNode< _Tp, _Tfield >(
		_Tp *anIter,
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)}
	); */
# define BO_RTBTREE_LinkNode(_Tp,_Tfield, \
		anIter,anObject,aTree,Force,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER) \
	do { \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_linkn_head, \
			*___rtbt_linkn_t, \
			*___rtbt_linkn_parent, \
			*___rtbt_linkn_onode; \
		bo_btree_dir_t ___rtbt_linkn_nodeDir; \
		bo_pointer_t ___rtbt_linkn_userInfo; \
 \
		assert(NULL != (aTree)); \
		assert(NULL != (anObject)); \
 \
		___rtbt_linkn_head = BO_RTBTREE_Head_(aTree); \
		___rtbt_linkn_onode = bo_btree_node_of(_Tfield,anObject); \
		___rtbt_linkn_userInfo = ___rtbt_linkn_onode->btn_info; \
		(void)___rtbt_linkn_userInfo; \
 \
		___rtbt_linkn_parent = ___rtbt_linkn_head; \
		___rtbt_linkn_nodeDir = BoK_BTREE_LEFT_DIR; \
		for (;;) { \
			if (NULL == (___rtbt_linkn_t = BO_RTBTREE_NODE_XLink(___rtbt_linkn_parent,___rtbt_linkn_nodeDir))) { \
				break; \
			} \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_linkn_t); \
			_COMPARATOR/*(_$0,anIter)*/; \
			if (0 == _$0 && !(Force)) { \
				/* return (BoS_ERR); */ _EXISTHANDLER; \
				assert(0); \
			} \
			___rtbt_linkn_nodeDir = BO_BTREE_UserinfoDir(_$0); \
			___rtbt_linkn_parent = ___rtbt_linkn_t; \
		} \
 \
		BO_RTBTREE_NODE_LLink(___rtbt_linkn_onode) = NULL; \
		BO_RTBTREE_NODE_RLink(___rtbt_linkn_onode) = ___rtbt_linkn_parent; \
		__bo_btree_node_p(___rtbt_linkn_onode)->btn_internal = 0; \
		___rtbt_linkn_onode->btn_rbackref = 1; \
 \
		if (___rtbt_linkn_parent == ___rtbt_linkn_head) { \
			___rtbt_linkn_onode->btn_last = ___rtbt_linkn_onode->btn_first = 1; \
			BO_RTBTREE_NODE_LLink(___rtbt_linkn_parent) = ___rtbt_linkn_onode; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linkn_onode; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linkn_onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (BoK_BTREE_LEFT_DIR == ___rtbt_linkn_nodeDir) { \
			if ((___rtbt_linkn_onode->btn_first = ___rtbt_linkn_parent->btn_first)) { \
				___rtbt_linkn_parent->btn_first = 0; \
				if (NULL != (aNewFirstPtr)) \
					*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linkn_onode; \
			} \
			BO_RTBTREE_NODE_LLink(___rtbt_linkn_parent) = ___rtbt_linkn_onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		/* (BoK_BTREE_RIGHT_DIR == ___rtbt_linkn_nodeDir) */ \
		{ \
			BO_RTBTREE_NODE_RLink(___rtbt_linkn_onode) = BO_RTBTREE_NODE_RLink(___rtbt_linkn_parent); \
			if ((___rtbt_linkn_onode->btn_last = ___rtbt_linkn_parent->btn_last)) { \
				___rtbt_linkn_parent->btn_last = 0; \
				if (NULL != (aNewLastPtr)) \
					*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linkn_onode; \
			} \
			BO_RTBTREE_NODE_RLink(___rtbt_linkn_parent) = ___rtbt_linkn_onode; \
			___rtbt_linkn_parent->btn_rbackref = 0; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_LinkNode_
 */
# pragma mark -

/* ()
   BO_RTBTREE_LinkNode_< _Tp, _Tfield >(
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)}
	); */
# define BO_RTBTREE_LinkNode_(_Tp,_Tfield, \
		anObject,aTree,Force,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER) \
	do { \
		_Tp *_$1; \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_linkn__head, \
			*___rtbt_linkn__t, \
			*___rtbt_linkn__parent, \
			*___rtbt_linkn__onode; \
		bo_btree_dir_t ___rtbt_linkn__nodeDir; \
		bo_pointer_t ___rtbt_linkn__userInfo; \
 \
		assert(NULL != (aTree)); \
		assert(NULL != (anObject)); \
 \
		___rtbt_linkn__head = BO_RTBTREE_Head_(aTree); \
		___rtbt_linkn__onode = bo_btree_node_of(_Tfield,anObject); \
		___rtbt_linkn__userInfo = ___rtbt_linkn__onode->btn_info; \
 \
		___rtbt_linkn__parent = ___rtbt_linkn__head; \
		___rtbt_linkn__nodeDir = BoK_BTREE_LEFT_DIR; \
		for (;;) { \
			if (NULL == (___rtbt_linkn__t = BO_RTBTREE_NODE_XLink(___rtbt_linkn__parent,___rtbt_linkn__nodeDir))) { \
				break; \
			} \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_linkn__t); \
			_COMPARATOR/*(_$0,_$1)*/; \
			if (0 == _$0 && !(Force)) { \
				/* return (BoS_ERR); */ _EXISTHANDLER; \
				assert(0); \
			} \
			___rtbt_linkn__nodeDir = BO_BTREE_UserinfoDir(_$0); \
			___rtbt_linkn__parent = ___rtbt_linkn__t; \
		} \
 \
		BO_RTBTREE_NODE_LLink(___rtbt_linkn__onode) = NULL; \
		BO_RTBTREE_NODE_RLink(___rtbt_linkn__onode) = ___rtbt_linkn__parent; \
		__bo_btree_node_p(___rtbt_linkn__onode)->btn_internal = 0; \
		___rtbt_linkn__onode->btn_rbackref = 1; \
 \
		if (___rtbt_linkn__parent == ___rtbt_linkn__head) { \
			___rtbt_linkn__onode->btn_last = ___rtbt_linkn__onode->btn_first = 1; \
			BO_RTBTREE_NODE_LLink(___rtbt_linkn__parent) = ___rtbt_linkn__onode; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linkn__onode; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linkn__onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (BoK_BTREE_LEFT_DIR == ___rtbt_linkn__nodeDir) { \
			if ((___rtbt_linkn__onode->btn_first = ___rtbt_linkn__parent->btn_first)) { \
				___rtbt_linkn__parent->btn_first = 0; \
				if (NULL != (aNewFirstPtr)) \
					*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linkn__onode; \
			} \
			BO_RTBTREE_NODE_LLink(___rtbt_linkn__parent) = ___rtbt_linkn__onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		/* (BoK_BTREE_RIGHT_DIR == ___rtbt_linkn__nodeDir) */ \
		{ \
			BO_RTBTREE_NODE_RLink(___rtbt_linkn__onode) = BO_RTBTREE_NODE_RLink(___rtbt_linkn__parent); \
			if ((___rtbt_linkn__onode->btn_last = ___rtbt_linkn__parent->btn_last)) { \
				___rtbt_linkn__parent->btn_last = 0; \
				if (NULL != (aNewLastPtr)) \
					*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linkn__onode; \
			} \
			BO_RTBTREE_NODE_RLink(___rtbt_linkn__parent) = ___rtbt_linkn__onode; \
			___rtbt_linkn__parent->btn_rbackref = 0; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_LinkNode_b
 */
# pragma mark -

/* ()
   BO_RTBTREE_LinkNode_b< _Tp, _Tfield >(
		_Tp *anIter,
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)},
		{(_HIMAXHANDLER)}
	); */

# define BO_RTBTREE_LinkNode_b(_Tp,_Tfield, \
		anIter,anObject,aTree,Force,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER,_HIMAXHANDLER) \
	do { \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_linknb_head, \
			*___rtbt_linknb_t, \
			*___rtbt_linknb_parent, \
			*___rtbt_linknb_onode, \
			*___rtbt_linknb_rebalance, \
			*___rtbt_linknb_rebalParent, \
			*___rtbt_linknb_rebalChild, \
			*___rtbt_linknb_rebalChild2; \
		bo_btree_dir_t ___rtbt_linknb_nodeDir, \
			___rtbt_linknb_rebalDir, \
			___rtbt_linknb_direction, \
			___rtbt_linknb_tdir; \
		bo_pointer_t ___rtbt_linknb_userInfo; \
		int ___rtbt_linknb_factor; \
		unsigned ___rtbt_linknb_high; \
 \
		assert(NULL != (aTree)); \
		assert(NULL != (anObject)); \
 \
		___rtbt_linknb_head = BO_BTREE_Head_(aTree); \
		___rtbt_linknb_onode = bo_btree_node_of(_Tfield,anObject); \
		___rtbt_linknb_userInfo = ___rtbt_linknb_onode->btn_info; \
		(void)___rtbt_linknb_userInfo; \
 \
		___rtbt_linknb_high = 0; \
		___rtbt_linknb_parent = ___rtbt_linknb_head; \
		___rtbt_linknb_direction = ___rtbt_linknb_nodeDir = BoK_BTREE_LEFT_DIR; \
		___rtbt_linknb_rebalParent = ___rtbt_linknb_parent; \
		___rtbt_linknb_rebalDir = ___rtbt_linknb_nodeDir; \
		___rtbt_linknb_rebalance = ___rtbt_linknb_t = \
			BO_RTBTREE_NODE_Link(___rtbt_linknb_parent,___rtbt_linknb_nodeDir); \
		if (NULL != ___rtbt_linknb_t) { \
			/* _$0 = @COMPARE(___rtbt_linknb_t,___rtbt_linknb_t->btn_info,___rtbt_linknb_onode,___rtbt_linknb_userInfo); */ \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_linknb_t); \
			_COMPARATOR/*(_$0,anIter)*/; \
			if (0 == _$0 && !(Force)) { \
				/* return (BoS_ERR); */ _EXISTHANDLER; \
				assert(0); \
			} \
			___rtbt_linknb_direction = ___rtbt_linknb_nodeDir = BO_BTREE_UserinfoDir(_$0); \
 \
			for (;;) { \
				___rtbt_linknb_parent = ___rtbt_linknb_t; \
				if (NULL == (___rtbt_linknb_t = BO_RTBTREE_NODE_XLink(___rtbt_linknb_parent,___rtbt_linknb_nodeDir))) { \
					break; \
				} \
				if (BoL_BTREE_HIGH_MAX == ++___rtbt_linknb_high) { \
					/* return (BoS_ERR); */ _HIMAXHANDLER; \
					assert(0); \
				} \
 \
				/* _$0 = @COMPARE(___rtbt_linknb_t,___rtbt_linknb_t->btn_info,___rtbt_linknb_onode,___rtbt_linknb_userInfo); */ \
				(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_linknb_t); \
				_COMPARATOR/*(_$0,anIter)*/; \
				if (0 == _$0 && !(Force)) { \
					/* return (BoS_ERR); */ _EXISTHANDLER; \
					assert(0); \
				} \
				___rtbt_linknb_tdir = BO_BTREE_UserinfoDir(_$0); \
 \
				if (BO_RTBTREE_NODE_IsUnbalanced(___rtbt_linknb_t)) { \
					___rtbt_linknb_rebalParent = ___rtbt_linknb_parent; \
					___rtbt_linknb_rebalDir = ___rtbt_linknb_nodeDir; \
					___rtbt_linknb_rebalance = ___rtbt_linknb_t; \
					___rtbt_linknb_direction = ___rtbt_linknb_tdir; \
				} \
				___rtbt_linknb_nodeDir = ___rtbt_linknb_tdir; \
			} \
		} \
 \
		BO_RTBTREE_NODE_LLink(___rtbt_linknb_onode) = NULL; \
		BO_RTBTREE_NODE_RLink(___rtbt_linknb_onode) = ___rtbt_linknb_parent; \
		__bo_btree_node_p(___rtbt_linknb_onode)->btn_internal = 0; \
		___rtbt_linknb_onode->btn_rbackref = 1; \
 \
		if (___rtbt_linknb_parent == ___rtbt_linknb_head) { \
			___rtbt_linknb_onode->btn_last = ___rtbt_linknb_onode->btn_first = 1; \
			BO_RTBTREE_NODE_LLink(___rtbt_linknb_parent) = ___rtbt_linknb_onode; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linknb_onode; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linknb_onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (BoK_BTREE_LEFT_DIR == ___rtbt_linknb_nodeDir) { \
			if ((___rtbt_linknb_onode->btn_first = ___rtbt_linknb_parent->btn_first)) { \
				___rtbt_linknb_parent->btn_first = 0; \
				if (NULL != (aNewFirstPtr)) \
					*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linknb_onode; \
			} \
			BO_RTBTREE_NODE_LLink(___rtbt_linknb_parent) = ___rtbt_linknb_onode; \
		} \
		else { \
			BO_RTBTREE_NODE_RLink(___rtbt_linknb_onode) = BO_RTBTREE_NODE_RLink(___rtbt_linknb_parent); \
			if ((___rtbt_linknb_onode->btn_last = ___rtbt_linknb_parent->btn_last)) { \
				___rtbt_linknb_parent->btn_last = 0; \
				if (NULL != (aNewLastPtr)) \
					*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linknb_onode; \
			} \
			BO_RTBTREE_NODE_RLink(___rtbt_linknb_parent) = ___rtbt_linknb_onode; \
			___rtbt_linknb_parent->btn_rbackref = 0; \
		} \
 \
		___rtbt_linknb_factor = ((BoK_BTREE_LEFT_DIR == ___rtbt_linknb_direction) ? -1 : +1); \
		___rtbt_linknb_rebalChild = BO_RTBTREE_NODE_Link(___rtbt_linknb_rebalance,___rtbt_linknb_direction); \
 \
		assert(((BoK_BTREE_LEFT_DIR == ___rtbt_linknb_direction) && \
					(NULL != ___rtbt_linknb_rebalChild)) || \
				!BO_BTREE_NODE_IsRBackref(___rtbt_linknb_rebalance)); \
 \
		for (___rtbt_linknb_t = ___rtbt_linknb_rebalChild; \
				___rtbt_linknb_onode != ___rtbt_linknb_t; \
					___rtbt_linknb_t = BO_RTBTREE_NODE_Link(___rtbt_linknb_t,___rtbt_linknb_tdir)) \
		{ \
			/* _$0 = @COMPARE(___rtbt_linknb_t,___rtbt_linknb_t->btn_info,___rtbt_linknb_onode,___rtbt_linknb_userInfo); */ \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_linknb_t); \
			_COMPARATOR/*(_$0,anIter)*/; \
			___rtbt_linknb_tdir = BO_BTREE_UserinfoDir(_$0); \
			___rtbt_linknb_t->btn_balance = ((BoK_BTREE_LEFT_DIR == ___rtbt_linknb_tdir) ? -1 : 1); \
		} \
 \
		if (0 == ___rtbt_linknb_rebalance->btn_balance) { \
			___rtbt_linknb_rebalance->btn_balance = ___rtbt_linknb_factor; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (-___rtbt_linknb_factor == ___rtbt_linknb_rebalance->btn_balance) { \
			___rtbt_linknb_rebalance->btn_balance = 0; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (___rtbt_linknb_factor == ___rtbt_linknb_rebalChild->btn_balance) { \
			if (BoK_BTREE_LEFT_DIR == ___rtbt_linknb_direction) \
				_BO_RTBTREE_RotateRight(___rtbt_linknb_rebalance,___rtbt_linknb_rebalChild); \
			else \
				_BO_RTBTREE_RotateLeft(___rtbt_linknb_rebalance,___rtbt_linknb_rebalChild); \
			___rtbt_linknb_rebalance->btn_balance = 0; \
		} \
		else { \
			___rtbt_linknb_rebalChild2 = BO_RTBTREE_NODE_Link(___rtbt_linknb_rebalChild,!___rtbt_linknb_direction); \
 \
			if (BoK_BTREE_LEFT_DIR == ___rtbt_linknb_direction) { \
				_BO_RTBTREE_RotateLeft(___rtbt_linknb_rebalChild,___rtbt_linknb_rebalChild2); \
				_BO_RTBTREE_RotateRight(___rtbt_linknb_rebalance,___rtbt_linknb_rebalChild2); \
			} \
			else { \
				_BO_RTBTREE_RotateRight(___rtbt_linknb_rebalChild,___rtbt_linknb_rebalChild2); \
				_BO_RTBTREE_RotateLeft(___rtbt_linknb_rebalance,___rtbt_linknb_rebalChild2); \
			} \
 \
			___rtbt_linknb_rebalance->btn_balance = ___rtbt_linknb_rebalChild->btn_balance = 0; \
			if (___rtbt_linknb_factor == ___rtbt_linknb_rebalChild2->btn_balance) \
				___rtbt_linknb_rebalance->btn_balance = -___rtbt_linknb_factor; \
			else if (-___rtbt_linknb_factor == ___rtbt_linknb_rebalChild2->btn_balance) \
				___rtbt_linknb_rebalChild->btn_balance = ___rtbt_linknb_factor; \
 \
			___rtbt_linknb_rebalChild = ___rtbt_linknb_rebalChild2; \
		} /* (___rtbt_linknb_factor != ___rtbt_linknb_rebalChild->btn_balance) */ \
		___rtbt_linknb_rebalChild->btn_balance = 0; \
 \
		BO_RTBTREE_NODE_Link(___rtbt_linknb_rebalParent,___rtbt_linknb_rebalDir) = ___rtbt_linknb_rebalChild; \
		/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
		assert(0); \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_LinkNode_b_
 */
# pragma mark -

/* ()
   BO_RTBTREE_LinkNode_b_< _Tp, _Tfield >(
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)},
		{(_HIMAXHANDLER)}
	); */

# define BO_RTBTREE_LinkNode_b_(_Tp,_Tfield, \
		anObject,aTree,Force,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER,_HIMAXHANDLER) \
	do { \
		_Tp *_$1; \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_linknb__head, \
			*___rtbt_linknb__t, \
			*___rtbt_linknb__parent, \
			*___rtbt_linknb__onode, \
			*___rtbt_linknb__rebalance, \
			*___rtbt_linknb__rebalParent, \
			*___rtbt_linknb__rebalChild, \
			*___rtbt_linknb__rebalChild2; \
		bo_btree_dir_t ___rtbt_linknb__nodeDir, \
			___rtbt_linknb__rebalDir, \
			___rtbt_linknb__direction, \
			___rtbt_linknb__tdir; \
		bo_pointer_t ___rtbt_linknb__userInfo; \
		int ___rtbt_linknb__factor; \
		unsigned ___rtbt_linknb__high; \
 \
		assert(NULL != (aTree)); \
		assert(NULL != (anObject)); \
 \
		___rtbt_linknb__head = BO_BTREE_Head_(aTree); \
		___rtbt_linknb__onode = bo_btree_node_of(_Tfield,anObject); \
		___rtbt_linknb__userInfo = ___rtbt_linknb__onode->btn_info; \
		(void)___rtbt_linknb__userInfo; \
 \
		___rtbt_linknb__high = 0; \
		___rtbt_linknb__parent = ___rtbt_linknb__head; \
		___rtbt_linknb__direction = ___rtbt_linknb__nodeDir = BoK_BTREE_LEFT_DIR; \
		___rtbt_linknb__rebalParent = ___rtbt_linknb__parent; \
		___rtbt_linknb__rebalDir = ___rtbt_linknb__nodeDir; \
		___rtbt_linknb__rebalance = ___rtbt_linknb__t = \
			BO_RTBTREE_NODE_Link(___rtbt_linknb__parent,___rtbt_linknb__nodeDir); \
		if (NULL != ___rtbt_linknb__t) { \
			/* _$0 = @COMPARE(___rtbt_linknb__t,___rtbt_linknb__t->btn_info,___rtbt_linknb__onode,___rtbt_linknb__userInfo); */ \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_linknb__t); \
			_COMPARATOR/*(_$0,_$1)*/; \
			if (0 == _$0 && !(Force)) { \
				/* return (BoS_ERR); */ _EXISTHANDLER; \
				assert(0); \
			} \
			___rtbt_linknb__direction = ___rtbt_linknb__nodeDir = BO_BTREE_UserinfoDir(_$0); \
 \
			for (;;) { \
				___rtbt_linknb__parent = ___rtbt_linknb__t; \
				if (NULL == (___rtbt_linknb__t = BO_RTBTREE_NODE_XLink(___rtbt_linknb__parent,___rtbt_linknb__nodeDir))) { \
					break; \
				} \
				if (BoL_BTREE_HIGH_MAX == ++___rtbt_linknb__high) { \
					/* return (BoS_ERR); */ _HIMAXHANDLER; \
					assert(0); \
				} \
 \
				/* _$0 = @COMPARE(___rtbt_linknb__t,___rtbt_linknb__t->btn_info,___rtbt_linknb__onode,___rtbt_linknb__userInfo); */ \
				_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_linknb__t); \
				_COMPARATOR/*(_$0,_$1)*/; \
				if (0 == _$0 && !(Force)) { \
					/* return (BoS_ERR); */ _EXISTHANDLER; \
					assert(0); \
				} \
				___rtbt_linknb__tdir = BO_BTREE_UserinfoDir(_$0); \
 \
				if (BO_RTBTREE_NODE_IsUnbalanced(___rtbt_linknb__t)) { \
					___rtbt_linknb__rebalParent = ___rtbt_linknb__parent; \
					___rtbt_linknb__rebalDir = ___rtbt_linknb__nodeDir; \
					___rtbt_linknb__rebalance = ___rtbt_linknb__t; \
					___rtbt_linknb__direction = ___rtbt_linknb__tdir; \
				} \
				___rtbt_linknb__nodeDir = ___rtbt_linknb__tdir; \
			} \
		} \
 \
		BO_RTBTREE_NODE_LLink(___rtbt_linknb__onode) = NULL; \
		BO_RTBTREE_NODE_RLink(___rtbt_linknb__onode) = ___rtbt_linknb__parent; \
		__bo_btree_node_p(___rtbt_linknb__onode)->btn_internal = 0; \
		___rtbt_linknb__onode->btn_rbackref = 1; \
 \
		if (___rtbt_linknb__parent == ___rtbt_linknb__head) { \
			___rtbt_linknb__onode->btn_last = ___rtbt_linknb__onode->btn_first = 1; \
			BO_RTBTREE_NODE_LLink(___rtbt_linknb__parent) = ___rtbt_linknb__onode; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linknb__onode; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linknb__onode; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (BoK_BTREE_LEFT_DIR == ___rtbt_linknb__nodeDir) { \
			if ((___rtbt_linknb__onode->btn_first = ___rtbt_linknb__parent->btn_first)) { \
				___rtbt_linknb__parent->btn_first = 0; \
				if (NULL != (aNewFirstPtr)) \
					*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_linknb__onode; \
			} \
			BO_RTBTREE_NODE_LLink(___rtbt_linknb__parent) = ___rtbt_linknb__onode; \
		} \
		else { \
			BO_RTBTREE_NODE_RLink(___rtbt_linknb__onode) = BO_RTBTREE_NODE_RLink(___rtbt_linknb__parent); \
			if ((___rtbt_linknb__onode->btn_last = ___rtbt_linknb__parent->btn_last)) { \
				___rtbt_linknb__parent->btn_last = 0; \
				if (NULL != (aNewLastPtr)) \
					*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_linknb__onode; \
			} \
			BO_RTBTREE_NODE_RLink(___rtbt_linknb__parent) = ___rtbt_linknb__onode; \
			___rtbt_linknb__parent->btn_rbackref = 0; \
		} \
 \
		___rtbt_linknb__factor = ((BoK_BTREE_LEFT_DIR == ___rtbt_linknb__direction) ? -1 : +1); \
		___rtbt_linknb__rebalChild = BO_RTBTREE_NODE_Link(___rtbt_linknb__rebalance,___rtbt_linknb__direction); \
 \
		assert(((BoK_BTREE_LEFT_DIR == ___rtbt_linknb__direction) && \
					(NULL != ___rtbt_linknb__rebalChild)) || \
				!BO_BTREE_NODE_IsRBackref(___rtbt_linknb__rebalance)); \
 \
		for (___rtbt_linknb__t = ___rtbt_linknb__rebalChild; \
				___rtbt_linknb__onode != ___rtbt_linknb__t; \
					___rtbt_linknb__t = BO_RTBTREE_NODE_Link(___rtbt_linknb__t,___rtbt_linknb__tdir)) \
		{ \
			/* _$0 = @COMPARE(___rtbt_linknb__t,___rtbt_linknb__t->btn_info,___rtbt_linknb__onode,___rtbt_linknb__userInfo); */ \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_linknb__t); \
			_COMPARATOR/*(_$0,_$1)*/; \
			___rtbt_linknb__tdir = BO_BTREE_UserinfoDir(_$0); \
			___rtbt_linknb__t->btn_balance = ((BoK_BTREE_LEFT_DIR == ___rtbt_linknb__tdir) ? -1 : 1); \
		} \
 \
		if (0 == ___rtbt_linknb__rebalance->btn_balance) { \
			___rtbt_linknb__rebalance->btn_balance = ___rtbt_linknb__factor; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (-___rtbt_linknb__factor == ___rtbt_linknb__rebalance->btn_balance) { \
			___rtbt_linknb__rebalance->btn_balance = 0; \
			/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		if (___rtbt_linknb__factor == ___rtbt_linknb__rebalChild->btn_balance) { \
			if (BoK_BTREE_LEFT_DIR == ___rtbt_linknb__direction) \
				_BO_RTBTREE_RotateRight(___rtbt_linknb__rebalance,___rtbt_linknb__rebalChild); \
			else \
				_BO_RTBTREE_RotateLeft(___rtbt_linknb__rebalance,___rtbt_linknb__rebalChild); \
			___rtbt_linknb__rebalance->btn_balance = 0; \
		} \
		else { \
			___rtbt_linknb__rebalChild2 = BO_RTBTREE_NODE_Link(___rtbt_linknb__rebalChild,!___rtbt_linknb__direction); \
 \
			if (BoK_BTREE_LEFT_DIR == ___rtbt_linknb__direction) { \
				_BO_RTBTREE_RotateLeft(___rtbt_linknb__rebalChild,___rtbt_linknb__rebalChild2); \
				_BO_RTBTREE_RotateRight(___rtbt_linknb__rebalance,___rtbt_linknb__rebalChild2); \
			} \
			else { \
				_BO_RTBTREE_RotateRight(___rtbt_linknb__rebalChild,___rtbt_linknb__rebalChild2); \
				_BO_RTBTREE_RotateLeft(___rtbt_linknb__rebalance,___rtbt_linknb__rebalChild2); \
			} \
 \
			___rtbt_linknb__rebalance->btn_balance = ___rtbt_linknb__rebalChild->btn_balance = 0; \
			if (___rtbt_linknb__factor == ___rtbt_linknb__rebalChild2->btn_balance) \
				___rtbt_linknb__rebalance->btn_balance = -___rtbt_linknb__factor; \
			else if (-___rtbt_linknb__factor == ___rtbt_linknb__rebalChild2->btn_balance) \
				___rtbt_linknb__rebalChild->btn_balance = ___rtbt_linknb__factor; \
 \
			___rtbt_linknb__rebalChild = ___rtbt_linknb__rebalChild2; \
		} /* (___rtbt_linknb__factor != ___rtbt_linknb__rebalChild->btn_balance) */ \
		___rtbt_linknb__rebalChild->btn_balance = 0; \
 \
		BO_RTBTREE_NODE_Link(___rtbt_linknb__rebalParent,___rtbt_linknb__rebalDir) = ___rtbt_linknb__rebalChild; \
		/* return (BoS_NORMAL); */ _SUCCESSHANDLER; \
		assert(0); \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_UnlinkNode
 */
# pragma mark -

/* ()
   BO_RTBTREE_UnlinkNode< _Tp, _Tfield >(
		_Tp *anIter,
		bo_rtbtree *aTree,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER: _Tp *anIter)},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_UnlinkNode(_Tp,_Tfield, \
		anIter,aTree,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_ulinkn_head, \
			*___rtbt_ulinkn_t, \
			*___rtbt_ulinkn_node, \
			*___rtbt_ulinkn_parent, \
			*___rtbt_ulinkn_right, \
			*___rtbt_ulinkn_left, \
			*___rtbt_ulinkn_r, \
			*___rtbt_ulinkn_q; \
		bo_btree_dir_t ___rtbt_ulinkn_direction; \
 \
		assert(NULL != (aTree)); \
 \
		___rtbt_ulinkn_head = BO_BTREE_Head_(aTree); \
 \
		___rtbt_ulinkn_parent = ___rtbt_ulinkn_head; \
		___rtbt_ulinkn_direction = BoK_BTREE_LEFT_DIR; \
		for (;;) { \
			if (NULL == (___rtbt_ulinkn_node = BO_RTBTREE_NODE_XLink(___rtbt_ulinkn_parent,___rtbt_ulinkn_direction))) { \
				/* return (___rtbt_ulinkn_node); */ _NOTFOUNDHANDLER; \
				assert(0); \
			} \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinkn_node); \
			_COMPARATOR/*(_$0,anIter)*/; \
			if (0 == _$0) { \
				break; \
			} \
			___rtbt_ulinkn_direction = ((_$0 <= 0) ? BoK_BTREE_RIGHT_DIR : BoK_BTREE_LEFT_DIR); \
			___rtbt_ulinkn_parent = ___rtbt_ulinkn_node; \
		} \
 \
		if (___rtbt_ulinkn_parent == ___rtbt_ulinkn_head && \
			BO_RTBTREE_NODE_IsLeaf(___rtbt_ulinkn_node)) \
		{ \
			BO_RTBTREE_NODE_LLink(___rtbt_ulinkn_head) = NULL; \
			BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_head) = ___rtbt_ulinkn_head; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = NULL; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = NULL; \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinkn_node); \
			/* return (___rtbt_ulinkn_node); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		___rtbt_ulinkn_left = BO_RTBTREE_NODE_LLink(___rtbt_ulinkn_node); \
		___rtbt_ulinkn_right = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_node); \
 \
		if (BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn_node)) { \
			___rtbt_ulinkn_r = ___rtbt_ulinkn_parent; \
			if (BoK_BTREE_RIGHT_DIR == ___rtbt_ulinkn_direction && NULL == ___rtbt_ulinkn_left) { \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_parent) = ___rtbt_ulinkn_right; \
				___rtbt_ulinkn_parent->btn_rbackref = 1; \
			} \
			else { \
				if (NULL != (BO_RTBTREE_NODE_Link(___rtbt_ulinkn_parent,___rtbt_ulinkn_direction) = ___rtbt_ulinkn_left)) { \
					___rtbt_ulinkn_r = ___rtbt_ulinkn_left; \
					do { \
						___rtbt_ulinkn_t = ___rtbt_ulinkn_left; \
						___rtbt_ulinkn_left = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_t); \
					} \
					while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn_t)); \
					BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_t) = ___rtbt_ulinkn_right; \
				} \
			} \
		} \
		else { \
			___rtbt_ulinkn_r = ___rtbt_ulinkn_right; \
			if (NULL != (___rtbt_ulinkn_t = BO_RTBTREE_NODE_LLink(___rtbt_ulinkn_r))) { \
				do { \
					___rtbt_ulinkn_q = ___rtbt_ulinkn_r; \
					___rtbt_ulinkn_r = ___rtbt_ulinkn_t; \
				} \
				while (NULL != (___rtbt_ulinkn_t = BO_RTBTREE_NODE_LLink(___rtbt_ulinkn_r))); \
				if (___rtbt_ulinkn_q == (___rtbt_ulinkn_t = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_r))) { \
					assert(BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn_r)); \
					___rtbt_ulinkn_t = NULL; \
				} \
				BO_RTBTREE_NODE_LLink(___rtbt_ulinkn_q) = ___rtbt_ulinkn_t; \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_r) = ___rtbt_ulinkn_right; \
				___rtbt_ulinkn_r->btn_rbackref = 0; \
			} \
			BO_RTBTREE_NODE_Link(___rtbt_ulinkn_parent,___rtbt_ulinkn_direction) = ___rtbt_ulinkn_r; \
			if (NULL != (BO_RTBTREE_NODE_LLink(___rtbt_ulinkn_r) = ___rtbt_ulinkn_left)) { \
				do { \
					___rtbt_ulinkn_t = ___rtbt_ulinkn_left; \
					___rtbt_ulinkn_left = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_t); \
				} \
				while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn_t)); \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinkn_t) = ___rtbt_ulinkn_r; \
			} \
		} \
 \
		if (___rtbt_ulinkn_node->btn_first) { \
			___rtbt_ulinkn_r->btn_first = ___rtbt_ulinkn_node->btn_first; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_ulinkn_r; \
		} \
		if (___rtbt_ulinkn_node->btn_last) { \
			___rtbt_ulinkn_r->btn_last = ___rtbt_ulinkn_node->btn_last; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_ulinkn_r; \
		} \
 \
		(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinkn_node); \
		/* return (___rtbt_ulinkn_node); */ _SUCCESSHANDLER; \
		assert(0); \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_UnlinkNode_
 */
# pragma mark -

/* ()
   BO_RTBTREE_UnlinkNode_< _Tp, _Tfield >(
		bo_rtbtree *aTree,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER: (_Tp *_$1))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_UnlinkNode_(_Tp,_Tfield, \
		aTree,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		_Tp *_$1; \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_ulinkn__head, \
			*___rtbt_ulinkn__t, \
			*___rtbt_ulinkn__node, \
			*___rtbt_ulinkn__parent, \
			*___rtbt_ulinkn__right, \
			*___rtbt_ulinkn__left, \
			*___rtbt_ulinkn__r, \
			*___rtbt_ulinkn__q; \
		bo_btree_dir_t ___rtbt_ulinkn__direction; \
 \
		assert(NULL != (aTree)); \
 \
		___rtbt_ulinkn__head = BO_BTREE_Head_(aTree); \
 \
		___rtbt_ulinkn__parent = ___rtbt_ulinkn__head; \
		___rtbt_ulinkn__direction = BoK_BTREE_LEFT_DIR; \
		for (;;) { \
			if (NULL == (___rtbt_ulinkn__node = BO_RTBTREE_NODE_XLink(___rtbt_ulinkn__parent,___rtbt_ulinkn__direction))) { \
				/* return (___rtbt_ulinkn__node); */ _NOTFOUNDHANDLER; \
				assert(0); \
			} \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinkn__node); \
			_COMPARATOR/*(_$0,_$1)*/; \
			if (0 == _$0) { \
				break; \
			} \
			___rtbt_ulinkn__direction = ((_$0 <= 0) ? BoK_BTREE_RIGHT_DIR : BoK_BTREE_LEFT_DIR); \
			___rtbt_ulinkn__parent = ___rtbt_ulinkn__node; \
		} \
 \
		if (___rtbt_ulinkn__parent == ___rtbt_ulinkn__head && \
			BO_RTBTREE_NODE_IsLeaf(___rtbt_ulinkn__node)) \
		{ \
			BO_RTBTREE_NODE_LLink(___rtbt_ulinkn__head) = NULL; \
			BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__head) = ___rtbt_ulinkn__head; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = NULL; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = NULL; \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinkn__node); \
			/* return (___rtbt_ulinkn__node); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		___rtbt_ulinkn__left = BO_RTBTREE_NODE_LLink(___rtbt_ulinkn__node); \
		___rtbt_ulinkn__right = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__node); \
 \
		if (BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn__node)) { \
			___rtbt_ulinkn__r = ___rtbt_ulinkn__parent; \
			if (BoK_BTREE_RIGHT_DIR == ___rtbt_ulinkn__direction && NULL == ___rtbt_ulinkn__left) { \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__parent) = ___rtbt_ulinkn__right; \
				___rtbt_ulinkn__parent->btn_rbackref = 1; \
			} \
			else { \
				if (NULL != (BO_RTBTREE_NODE_Link(___rtbt_ulinkn__parent,___rtbt_ulinkn__direction) = ___rtbt_ulinkn__left)) { \
					___rtbt_ulinkn__r = ___rtbt_ulinkn__left; \
					do { \
						___rtbt_ulinkn__t = ___rtbt_ulinkn__left; \
						___rtbt_ulinkn__left = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__t); \
					} \
					while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn__t)); \
					BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__t) = ___rtbt_ulinkn__right; \
				} \
			} \
		} \
		else { \
			___rtbt_ulinkn__r = ___rtbt_ulinkn__right; \
			if (NULL != (___rtbt_ulinkn__t = BO_RTBTREE_NODE_LLink(___rtbt_ulinkn__r))) { \
				do { \
					___rtbt_ulinkn__q = ___rtbt_ulinkn__r; \
					___rtbt_ulinkn__r = ___rtbt_ulinkn__t; \
				} \
				while (NULL != (___rtbt_ulinkn__t = BO_RTBTREE_NODE_LLink(___rtbt_ulinkn__r))); \
				if (___rtbt_ulinkn__q == (___rtbt_ulinkn__t = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__r))) { \
					assert(BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn__r)); \
					___rtbt_ulinkn__t = NULL; \
				} \
				BO_RTBTREE_NODE_LLink(___rtbt_ulinkn__q) = ___rtbt_ulinkn__t; \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__r) = ___rtbt_ulinkn__right; \
				___rtbt_ulinkn__r->btn_rbackref = 0; \
			} \
			BO_RTBTREE_NODE_Link(___rtbt_ulinkn__parent,___rtbt_ulinkn__direction) = ___rtbt_ulinkn__r; \
			if (NULL != (BO_RTBTREE_NODE_LLink(___rtbt_ulinkn__r) = ___rtbt_ulinkn__left)) { \
				do { \
					___rtbt_ulinkn__t = ___rtbt_ulinkn__left; \
					___rtbt_ulinkn__left = BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__t); \
				} \
				while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinkn__t)); \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinkn__t) = ___rtbt_ulinkn__r; \
			} \
		} \
 \
		if (___rtbt_ulinkn__node->btn_first) { \
			___rtbt_ulinkn__r->btn_first = ___rtbt_ulinkn__node->btn_first; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_ulinkn__r; \
		} \
		if (___rtbt_ulinkn__node->btn_last) { \
			___rtbt_ulinkn__r->btn_last = ___rtbt_ulinkn__node->btn_last; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_ulinkn__r; \
		} \
 \
		_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinkn__node); \
		/* return (___rtbt_ulinkn__node); */ _SUCCESSHANDLER; \
		assert(0); \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_UnlinkNode_b
 */
# pragma mark -

/* ()
   BO_RTBTREE_UnlinkNode_b< _Tp, _Tfield >(
		_Tp *anIter,
		bo_rtbtree *aTree,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER: _Tp *anIter)},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_UnlinkNode_b(_Tp,_Tfield, \
		anIter,aTree,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_ulinknb_head, \
			*___rtbt_ulinknb_t, \
			*___rtbt_ulinknb_node, \
			*___rtbt_ulinknb_parent, \
			*___rtbt_ulinknb_right, \
			*___rtbt_ulinknb_left, \
			*___rtbt_ulinknb_r, \
			*___rtbt_ulinknb_q; \
		bo_btree_dir_t ___rtbt_ulinknb_direction; \
		unsigned ___rtbt_ulinknb_pathsz; \
		struct bo_btpath_component ___rtbt_ulinknb_path[BoL_BTREE_HIGH_MAX]; \
 \
		assert(NULL != (aTree)); \
 \
		___rtbt_ulinknb_head = BO_BTREE_Head_(aTree); \
 \
		___rtbt_ulinknb_pathsz = 0; \
		___rtbt_ulinknb_parent = ___rtbt_ulinknb_head; \
		___rtbt_ulinknb_direction = BoK_BTREE_LEFT_DIR; \
		for (;;) { \
			if (NULL == (___rtbt_ulinknb_node = BO_RTBTREE_NODE_XLink(___rtbt_ulinknb_parent,___rtbt_ulinknb_direction))) { \
				/* return (___rtbt_ulinknb_node); */ _NOTFOUNDHANDLER; \
				assert(0); \
			} \
			___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz].node = ___rtbt_ulinknb_parent; \
			___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz++].dir = ___rtbt_ulinknb_direction; \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb_node); \
			_COMPARATOR/*(_$0,anIter)*/; \
			if (0 == _$0) { \
				break; \
			} \
			___rtbt_ulinknb_direction = ((_$0 <= 0) ? BoK_BTREE_RIGHT_DIR : BoK_BTREE_LEFT_DIR); \
			___rtbt_ulinknb_parent = ___rtbt_ulinknb_node; \
		} \
 \
		if (___rtbt_ulinknb_parent == ___rtbt_ulinknb_head && \
			BO_RTBTREE_NODE_IsLeaf(___rtbt_ulinknb_node)) \
		{ \
			BO_RTBTREE_NODE_LLink(___rtbt_ulinknb_head) = NULL; \
			BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_head) = ___rtbt_ulinknb_head; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = NULL; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = NULL; \
			(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb_node); \
			/* return (___rtbt_ulinknb_node); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		___rtbt_ulinknb_left = BO_RTBTREE_NODE_LLink(___rtbt_ulinknb_node); \
		___rtbt_ulinknb_right = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_node); \
 \
		if (BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb_node)) { \
			___rtbt_ulinknb_r = ___rtbt_ulinknb_parent; \
			if (BoK_BTREE_RIGHT_DIR == ___rtbt_ulinknb_direction && NULL == ___rtbt_ulinknb_left) { \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_parent) = ___rtbt_ulinknb_right; \
				___rtbt_ulinknb_parent->btn_rbackref = 1; \
			} \
			else { \
				if (NULL != (BO_RTBTREE_NODE_Link(___rtbt_ulinknb_parent,___rtbt_ulinknb_direction) = ___rtbt_ulinknb_left)) { \
					___rtbt_ulinknb_r = ___rtbt_ulinknb_left; \
					do { \
						___rtbt_ulinknb_t = ___rtbt_ulinknb_left; \
						___rtbt_ulinknb_left = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_t); \
					} \
					while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb_t)); \
					BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_t) = ___rtbt_ulinknb_right; \
				} \
			} \
		} \
		else { \
			unsigned nodepos = ___rtbt_ulinknb_pathsz++; /* removed-node-position */ \
 \
			___rtbt_ulinknb_r = ___rtbt_ulinknb_right; \
			if (NULL != (___rtbt_ulinknb_t = BO_RTBTREE_NODE_LLink(___rtbt_ulinknb_r))) { \
				do { \
					___rtbt_ulinknb_q = ___rtbt_ulinknb_r; \
					___rtbt_ulinknb_r = ___rtbt_ulinknb_t; \
					___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz].node = ___rtbt_ulinknb_q; \
					___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz++].dir = BoK_BTREE_LEFT_DIR; \
				} \
				while (NULL != (___rtbt_ulinknb_t = BO_RTBTREE_NODE_LLink(___rtbt_ulinknb_r))); \
				if (___rtbt_ulinknb_q == (___rtbt_ulinknb_t = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_r))) { \
					assert(BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb_r)); \
					___rtbt_ulinknb_t = NULL; \
				} \
				BO_RTBTREE_NODE_LLink(___rtbt_ulinknb_q) = ___rtbt_ulinknb_t; \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_r) = ___rtbt_ulinknb_right; \
				___rtbt_ulinknb_r->btn_rbackref = 0; \
			} \
			BO_RTBTREE_NODE_Link(___rtbt_ulinknb_parent,___rtbt_ulinknb_direction) = ___rtbt_ulinknb_r; \
			if (NULL != (BO_RTBTREE_NODE_LLink(___rtbt_ulinknb_r) = ___rtbt_ulinknb_left)) { \
				do { \
					___rtbt_ulinknb_t = ___rtbt_ulinknb_left; \
					___rtbt_ulinknb_left = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_t); \
				} \
				while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb_t)); \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinknb_t) = ___rtbt_ulinknb_r; \
			} \
 \
			___rtbt_ulinknb_path[nodepos].node = ___rtbt_ulinknb_r; \
			___rtbt_ulinknb_path[nodepos].dir = BoK_BTREE_RIGHT_DIR; \
			___rtbt_ulinknb_r->btn_balance = ___rtbt_ulinknb_node->btn_balance; \
		} \
 \
		if (___rtbt_ulinknb_node->btn_first) { \
			___rtbt_ulinknb_r->btn_first = ___rtbt_ulinknb_node->btn_first; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_ulinknb_r; \
		} \
		if (___rtbt_ulinknb_node->btn_last) { \
			___rtbt_ulinknb_r->btn_last = ___rtbt_ulinknb_node->btn_last; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_ulinknb_r; \
		} \
 \
		for (;;) { \
			struct bo_btree_node *___rtbt_ulinknb_rebalance, \
				*___rtbt_ulinknb_rebalParent, \
				*___rtbt_ulinknb_rebalChild, \
				*___rtbt_ulinknb_rebalChild2; \
			bo_btree_dir_t ___rtbt_ulinknb_rebalDownDir, ___rtbt_ulinknb_rebalDir; \
			int ___rtbt_ulinknb_factor; \
 \
			if (0 == --___rtbt_ulinknb_pathsz) { \
				(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb_node); \
				/* return (___rtbt_ulinknb_node); */ _SUCCESSHANDLER; \
				assert(0); \
			} \
 \
			___rtbt_ulinknb_rebalance = ___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz].node; \
			___rtbt_ulinknb_rebalDownDir = ___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz].dir; \
			___rtbt_ulinknb_factor = ((BoK_BTREE_LEFT_DIR == ___rtbt_ulinknb_rebalDownDir) ? -1 : +1); \
 \
			if (0 == ___rtbt_ulinknb_rebalance->btn_balance) { \
				___rtbt_ulinknb_rebalance->btn_balance = -___rtbt_ulinknb_factor; \
				(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb_node); \
				/* return (___rtbt_ulinknb_node); */ _SUCCESSHANDLER; \
				assert(0); \
			} \
 \
			if (___rtbt_ulinknb_factor == ___rtbt_ulinknb_rebalance->btn_balance) { \
				___rtbt_ulinknb_rebalance->btn_balance = 0; \
				continue; \
				/* NOTREACHED */ \
			} \
 \
			___rtbt_ulinknb_rebalParent = ___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz - 1].node; \
			___rtbt_ulinknb_rebalDir = ___rtbt_ulinknb_path[___rtbt_ulinknb_pathsz - 1].dir; \
			___rtbt_ulinknb_rebalChild = BO_RTBTREE_NODE_Link(___rtbt_ulinknb_rebalance,!___rtbt_ulinknb_rebalDownDir); \
 \
			if (___rtbt_ulinknb_factor != ___rtbt_ulinknb_rebalChild->btn_balance) { \
				if (BoK_BTREE_LEFT_DIR == ___rtbt_ulinknb_rebalDownDir) \
					_BO_RTBTREE_RotateLeft(___rtbt_ulinknb_rebalance,___rtbt_ulinknb_rebalChild); \
				else \
					_BO_RTBTREE_RotateRight(___rtbt_ulinknb_rebalance,___rtbt_ulinknb_rebalChild); \
 \
				BO_RTBTREE_NODE_Link(___rtbt_ulinknb_rebalParent,___rtbt_ulinknb_rebalDir) = ___rtbt_ulinknb_rebalChild; \
 \
				if (0 == ___rtbt_ulinknb_rebalChild->btn_balance) { \
					___rtbt_ulinknb_rebalChild->btn_balance = ___rtbt_ulinknb_factor; \
					(anIter) = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb_node); \
					/* return (___rtbt_ulinknb_node); */ _SUCCESSHANDLER; \
					assert(0); \
				} \
 \
				___rtbt_ulinknb_rebalance->btn_balance = ___rtbt_ulinknb_rebalChild->btn_balance = 0; \
				continue; \
			} \
 \
			___rtbt_ulinknb_rebalChild2 = BO_RTBTREE_NODE_Link(___rtbt_ulinknb_rebalChild,___rtbt_ulinknb_rebalDownDir); \
 \
			if (BoK_BTREE_LEFT_DIR == ___rtbt_ulinknb_rebalDownDir) { \
				_BO_RTBTREE_RotateRight(___rtbt_ulinknb_rebalChild,___rtbt_ulinknb_rebalChild2); \
				_BO_RTBTREE_RotateLeft(___rtbt_ulinknb_rebalance,___rtbt_ulinknb_rebalChild2); \
			} \
			else { \
				_BO_RTBTREE_RotateLeft(___rtbt_ulinknb_rebalChild,___rtbt_ulinknb_rebalChild2); \
				_BO_RTBTREE_RotateRight(___rtbt_ulinknb_rebalance,___rtbt_ulinknb_rebalChild2); \
			} \
 \
			___rtbt_ulinknb_rebalance->btn_balance = ___rtbt_ulinknb_rebalChild->btn_balance = 0; \
			if (___rtbt_ulinknb_factor == ___rtbt_ulinknb_rebalChild2->btn_balance) \
				___rtbt_ulinknb_rebalChild->btn_balance = -___rtbt_ulinknb_factor; \
			else if (-___rtbt_ulinknb_factor == ___rtbt_ulinknb_rebalChild2->btn_balance) \
				___rtbt_ulinknb_rebalance->btn_balance = ___rtbt_ulinknb_factor; \
			___rtbt_ulinknb_rebalChild2->btn_balance = 0; \
 \
			BO_RTBTREE_NODE_Link(___rtbt_ulinknb_rebalParent,___rtbt_ulinknb_rebalDir) = ___rtbt_ulinknb_rebalChild2; \
		} \
		assert(0); \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_UnlinkNode_b_
 */
# pragma mark -

/* ()
   BO_RTBTREE_UnlinkNode_b_< _Tp, _Tfield >(
		bo_rtbtree *aTree,
		struct bo_btree_node **aNewFirstPtr,
		struct bo_btree_node **aNewLastPtr,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER: (_Tp *_$1))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_UnlinkNode_b_(_Tp,_Tfield, \
		aTree,aNewFirstPtr,aNewLastPtr, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		_Tp *_$1; \
		bo_mismatch_t _$0; \
		struct bo_btree_node *___rtbt_ulinknb__head, \
			*___rtbt_ulinknb__t, \
			*___rtbt_ulinknb__node, \
			*___rtbt_ulinknb__parent, \
			*___rtbt_ulinknb__right, \
			*___rtbt_ulinknb__left, \
			*___rtbt_ulinknb__r, \
			*___rtbt_ulinknb__q; \
		bo_btree_dir_t ___rtbt_ulinknb__direction; \
		unsigned ___rtbt_ulinknb__pathsz; \
		struct bo_btpath_component ___rtbt_ulinknb__path[BoL_BTREE_HIGH_MAX]; \
 \
		assert(NULL != (aTree)); \
 \
		___rtbt_ulinknb__head = BO_BTREE_Head_(aTree); \
 \
		___rtbt_ulinknb__pathsz = 0; \
		___rtbt_ulinknb__parent = ___rtbt_ulinknb__head; \
		___rtbt_ulinknb__direction = BoK_BTREE_LEFT_DIR; \
		for (;;) { \
			if (NULL == (___rtbt_ulinknb__node = BO_RTBTREE_NODE_XLink(___rtbt_ulinknb__parent,___rtbt_ulinknb__direction))) { \
				/* return (___rtbt_ulinknb__node); */ _NOTFOUNDHANDLER; \
				assert(0); \
			} \
			___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz].node = ___rtbt_ulinknb__parent; \
			___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz++].dir = ___rtbt_ulinknb__direction; \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb__node); \
			_COMPARATOR/*(_$0,_$1)*/; \
			if (0 == _$0) { \
				break; \
			} \
			___rtbt_ulinknb__direction = ((_$0 <= 0) ? BoK_BTREE_RIGHT_DIR : BoK_BTREE_LEFT_DIR); \
			___rtbt_ulinknb__parent = ___rtbt_ulinknb__node; \
		} \
 \
		if (___rtbt_ulinknb__parent == ___rtbt_ulinknb__head && \
			BO_RTBTREE_NODE_IsLeaf(___rtbt_ulinknb__node)) \
		{ \
			BO_RTBTREE_NODE_LLink(___rtbt_ulinknb__head) = NULL; \
			BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__head) = ___rtbt_ulinknb__head; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = NULL; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = NULL; \
			_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb__node); \
			/* return (___rtbt_ulinknb__node); */ _SUCCESSHANDLER; \
			assert(0); \
		} \
 \
		___rtbt_ulinknb__left = BO_RTBTREE_NODE_LLink(___rtbt_ulinknb__node); \
		___rtbt_ulinknb__right = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__node); \
 \
		if (BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb__node)) { \
			___rtbt_ulinknb__r = ___rtbt_ulinknb__parent; \
			if (BoK_BTREE_RIGHT_DIR == ___rtbt_ulinknb__direction && NULL == ___rtbt_ulinknb__left) { \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__parent) = ___rtbt_ulinknb__right; \
				___rtbt_ulinknb__parent->btn_rbackref = 1; \
			} \
			else { \
				if (NULL != (BO_RTBTREE_NODE_Link(___rtbt_ulinknb__parent,___rtbt_ulinknb__direction) = ___rtbt_ulinknb__left)) { \
					___rtbt_ulinknb__r = ___rtbt_ulinknb__left; \
					do { \
						___rtbt_ulinknb__t = ___rtbt_ulinknb__left; \
						___rtbt_ulinknb__left = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__t); \
					} \
					while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb__t)); \
					BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__t) = ___rtbt_ulinknb__right; \
				} \
			} \
		} \
		else { \
			unsigned nodepos = ___rtbt_ulinknb__pathsz++; /* removed-node-position */ \
 \
			___rtbt_ulinknb__r = ___rtbt_ulinknb__right; \
			if (NULL != (___rtbt_ulinknb__t = BO_RTBTREE_NODE_LLink(___rtbt_ulinknb__r))) { \
				do { \
					___rtbt_ulinknb__q = ___rtbt_ulinknb__r; \
					___rtbt_ulinknb__r = ___rtbt_ulinknb__t; \
					___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz].node = ___rtbt_ulinknb__q; \
					___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz++].dir = BoK_BTREE_LEFT_DIR; \
				} \
				while (NULL != (___rtbt_ulinknb__t = BO_RTBTREE_NODE_LLink(___rtbt_ulinknb__r))); \
				if (___rtbt_ulinknb__q == (___rtbt_ulinknb__t = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__r))) { \
					assert(BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb__r)); \
					___rtbt_ulinknb__t = NULL; \
				} \
				BO_RTBTREE_NODE_LLink(___rtbt_ulinknb__q) = ___rtbt_ulinknb__t; \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__r) = ___rtbt_ulinknb__right; \
				___rtbt_ulinknb__r->btn_rbackref = 0; \
			} \
			BO_RTBTREE_NODE_Link(___rtbt_ulinknb__parent,___rtbt_ulinknb__direction) = ___rtbt_ulinknb__r; \
			if (NULL != (BO_RTBTREE_NODE_LLink(___rtbt_ulinknb__r) = ___rtbt_ulinknb__left)) { \
				do { \
					___rtbt_ulinknb__t = ___rtbt_ulinknb__left; \
					___rtbt_ulinknb__left = BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__t); \
				} \
				while (!BO_BTREE_NODE_IsRBackref(___rtbt_ulinknb__t)); \
				BO_RTBTREE_NODE_RLink(___rtbt_ulinknb__t) = ___rtbt_ulinknb__r; \
			} \
 \
			___rtbt_ulinknb__path[nodepos].node = ___rtbt_ulinknb__r; \
			___rtbt_ulinknb__path[nodepos].dir = BoK_BTREE_RIGHT_DIR; \
			___rtbt_ulinknb__r->btn_balance = ___rtbt_ulinknb__node->btn_balance; \
		} \
 \
		if (___rtbt_ulinknb__node->btn_first) { \
			___rtbt_ulinknb__r->btn_first = ___rtbt_ulinknb__node->btn_first; \
			if (NULL != (aNewFirstPtr)) \
				*((struct bo_btree_node **)(aNewFirstPtr)) = ___rtbt_ulinknb__r; \
		} \
		if (___rtbt_ulinknb__node->btn_last) { \
			___rtbt_ulinknb__r->btn_last = ___rtbt_ulinknb__node->btn_last; \
			if (NULL != (aNewLastPtr)) \
				*((struct bo_btree_node **)(aNewLastPtr)) = ___rtbt_ulinknb__r; \
		} \
 \
		for (;;) { \
			struct bo_btree_node *___rtbt_ulinknb__rebalance, \
				*___rtbt_ulinknb__rebalParent, \
				*___rtbt_ulinknb__rebalChild, \
				*___rtbt_ulinknb__rebalChild2; \
			bo_btree_dir_t ___rtbt_ulinknb__rebalDownDir, ___rtbt_ulinknb__rebalDir; \
			int ___rtbt_ulinknb__factor; \
 \
			if (0 == --___rtbt_ulinknb__pathsz) { \
				_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb__node); \
				/* return (___rtbt_ulinknb__node); */ _SUCCESSHANDLER; \
				assert(0); \
			} \
 \
			___rtbt_ulinknb__rebalance = ___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz].node; \
			___rtbt_ulinknb__rebalDownDir = ___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz].dir; \
			___rtbt_ulinknb__factor = ((BoK_BTREE_LEFT_DIR == ___rtbt_ulinknb__rebalDownDir) ? -1 : +1); \
 \
			if (0 == ___rtbt_ulinknb__rebalance->btn_balance) { \
				___rtbt_ulinknb__rebalance->btn_balance = -___rtbt_ulinknb__factor; \
				_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb__node); \
				/* return (___rtbt_ulinknb__node); */ _SUCCESSHANDLER; \
				assert(0); \
			} \
 \
			if (___rtbt_ulinknb__factor == ___rtbt_ulinknb__rebalance->btn_balance) { \
				___rtbt_ulinknb__rebalance->btn_balance = 0; \
				continue; \
				/* NOTREACHED */ \
			} \
 \
			___rtbt_ulinknb__rebalParent = ___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz - 1].node; \
			___rtbt_ulinknb__rebalDir = ___rtbt_ulinknb__path[___rtbt_ulinknb__pathsz - 1].dir; \
			___rtbt_ulinknb__rebalChild = BO_RTBTREE_NODE_Link(___rtbt_ulinknb__rebalance,!___rtbt_ulinknb__rebalDownDir); \
 \
			if (___rtbt_ulinknb__factor != ___rtbt_ulinknb__rebalChild->btn_balance) { \
				if (BoK_BTREE_LEFT_DIR == ___rtbt_ulinknb__rebalDownDir) \
					_BO_RTBTREE_RotateLeft(___rtbt_ulinknb__rebalance,___rtbt_ulinknb__rebalChild); \
				else \
					_BO_RTBTREE_RotateRight(___rtbt_ulinknb__rebalance,___rtbt_ulinknb__rebalChild); \
 \
				BO_RTBTREE_NODE_Link(___rtbt_ulinknb__rebalParent,___rtbt_ulinknb__rebalDir) = ___rtbt_ulinknb__rebalChild; \
 \
				if (0 == ___rtbt_ulinknb__rebalChild->btn_balance) { \
					___rtbt_ulinknb__rebalChild->btn_balance = ___rtbt_ulinknb__factor; \
					_$1 = __myc_objectof(_Tp,_Tfield,___rtbt_ulinknb__node); \
					/* return (___rtbt_ulinknb__node); */ _SUCCESSHANDLER; \
					assert(0); \
				} \
 \
				___rtbt_ulinknb__rebalance->btn_balance = ___rtbt_ulinknb__rebalChild->btn_balance = 0; \
				continue; \
			} \
 \
			___rtbt_ulinknb__rebalChild2 = BO_RTBTREE_NODE_Link(___rtbt_ulinknb__rebalChild,___rtbt_ulinknb__rebalDownDir); \
 \
			if (BoK_BTREE_LEFT_DIR == ___rtbt_ulinknb__rebalDownDir) { \
				_BO_RTBTREE_RotateRight(___rtbt_ulinknb__rebalChild,___rtbt_ulinknb__rebalChild2); \
				_BO_RTBTREE_RotateLeft(___rtbt_ulinknb__rebalance,___rtbt_ulinknb__rebalChild2); \
			} \
			else { \
				_BO_RTBTREE_RotateLeft(___rtbt_ulinknb__rebalChild,___rtbt_ulinknb__rebalChild2); \
				_BO_RTBTREE_RotateRight(___rtbt_ulinknb__rebalance,___rtbt_ulinknb__rebalChild2); \
			} \
 \
			___rtbt_ulinknb__rebalance->btn_balance = ___rtbt_ulinknb__rebalChild->btn_balance = 0; \
			if (___rtbt_ulinknb__factor == ___rtbt_ulinknb__rebalChild2->btn_balance) \
				___rtbt_ulinknb__rebalChild->btn_balance = -___rtbt_ulinknb__factor; \
			else if (-___rtbt_ulinknb__factor == ___rtbt_ulinknb__rebalChild2->btn_balance) \
				___rtbt_ulinknb__rebalance->btn_balance = ___rtbt_ulinknb__factor; \
			___rtbt_ulinknb__rebalChild2->btn_balance = 0; \
 \
			BO_RTBTREE_NODE_Link(___rtbt_ulinknb__rebalParent,___rtbt_ulinknb__rebalDir) = ___rtbt_ulinknb__rebalChild2; \
		} \
		assert(0); \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_InsertNode
 */
# pragma mark -

/* ()
   BO_RTBTREE_InsertNode< _Tp, _Tfield >(
		_Tp *anIter,
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)}
	); */
# define BO_RTBTREE_InsertNode(_Tp,_Tfield, \
		anIter,anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER) \
	do { \
		BO_RTBTREE_LinkNode( \
			_Tp,_Tfield, \
			anIter,anObject,bo_rtbtree_p(aTree),Force, \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_EXIST,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_InsertNode_SUCCESS,__LINE__): \
		bo_rtbtree_p(aTree)->bt_elemc++; \
		_SUCCESSHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_InsertNode_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_InsertNode_
 */
# pragma mark -

/* ()
   BO_RTBTREE_InsertNode_< _Tp, _Tfield >(
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)}
	); */
# define BO_RTBTREE_InsertNode_(_Tp,_Tfield, \
		anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER) \
	do { \
		BO_RTBTREE_LinkNode_( \
			_Tp,_Tfield, \
			anObject,bo_rtbtree_p(aTree),Force, \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_EXIST,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_InsertNode_SUCCESS,__LINE__): \
		bo_rtbtree_p(aTree)->bt_elemc++; \
		_SUCCESSHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_InsertNode_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_InsertNode_b
 */
# pragma mark -

/* ()
   BO_RTBTREE_InsertNode_b< _Tp, _Tfield >(
		_Tp *anIter,
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)},
		{(_HIMAXHANDLER)}
	); */
# define BO_RTBTREE_InsertNode_b(_Tp,_Tfield, \
		anIter,anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER,_HIMAXHANDLER) \
	do { \
		BO_RTBTREE_LinkNode_b( \
			_Tp,_Tfield, \
			anIter,anObject,bo_rtbtree_p(aTree),Force, \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_b_EXIST,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_b_HIMAX,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_InsertNode_b_SUCCESS,__LINE__): \
		bo_rtbtree_p(aTree)->bt_elemc++; \
		_SUCCESSHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_InsertNode_b_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_InsertNode_b_HIMAX,__LINE__): \
		_HIMAXHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_InsertNode_b_
 */
# pragma mark -

/* ()
   BO_RTBTREE_InsertNode_b_< _Tp, _Tfield >(
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)},
		{(_HIMAXHANDLER)}
	); */
# define BO_RTBTREE_InsertNode_b_(_Tp,_Tfield, \
		anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER,_HIMAXHANDLER) \
	do { \
		BO_RTBTREE_LinkNode_b_( \
			_Tp,_Tfield, \
			anObject,bo_rtbtree_p(aTree),Force, \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_b_EXIST,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_InsertNode_b_HIMAX,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_InsertNode_b_SUCCESS,__LINE__): \
		bo_rtbtree_p(aTree)->bt_elemc++; \
		_SUCCESSHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_InsertNode_b_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_InsertNode_b_HIMAX,__LINE__): \
		_HIMAXHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_RemoveNode
 */
# pragma mark -

/* ()
   BO_RTBTREE_RemoveNode< _Tp, _Tfield >(
		_Tp *anIter,
		bo_rtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER: _Tp *anIter)},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_RemoveNode(_Tp,_Tfield, \
		anIter,aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		BO_RTBTREE_UnlinkNode( \
			_Tp,_Tfield, \
			anIter,bo_rtbtree_p(aTree), \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_SUCCESS,__LINE__): \
		bo_rtbtree_p(aTree)->bt_elemc--; \
		_SUCCESSHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_RemoveNode_
 */
# pragma mark -

/* ()
   BO_RTBTREE_RemoveNode_< _Tp, _Tfield >(
		bo_rtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER: (_Tp *_$1))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_RemoveNode_(_Tp,_Tfield, \
		aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		_Tp *___rtbt_rmn__it; \
		BO_RTBTREE_UnlinkNode_( \
			_Tp,_Tfield, \
			bo_rtbtree_p(aTree), \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ ___rtbt_rmn__it = _$1; goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_SUCCESS,__LINE__): \
		{ \
			_Tp *_$1; \
			bo_rtbtree_p(aTree)->bt_elemc--; \
			_$1 = ___rtbt_rmn__it; \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_RemoveNode_b
 */
# pragma mark -

/* ()
   BO_RTBTREE_RemoveNode_b< _Tp, _Tfield >(
		_Tp *anIter,
		bo_rtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER: _Tp *anIter)},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_RemoveNode_b(_Tp,_Tfield, \
		anIter,aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		BO_RTBTREE_UnlinkNode_b( \
			_Tp,_Tfield, \
			anIter,bo_rtbtree_p(aTree), \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_b_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_b_SUCCESS,__LINE__): \
		bo_rtbtree_p(aTree)->bt_elemc--; \
		_SUCCESSHANDLER; \
		break; \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_b_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_RTBTREE_RemoveNode_b_
 */
# pragma mark -

/* ()
   BO_RTBTREE_RemoveNode_b_< _Tp, _Tfield >(
		bo_rtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER: (_Tp *_$1))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_RTBTREE_RemoveNode_b_(_Tp,_Tfield, \
		aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		_Tp *___rtbt_rmnb__it; \
		BO_RTBTREE_UnlinkNode_b_( \
			_Tp,_Tfield, \
			bo_rtbtree_p(aTree), \
			NULL, \
			NULL, \
			_COMPARATOR, \
			{ ___rtbt_rmnb__it = _$1; goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_RTBTREE_RemoveNode_b_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_b_SUCCESS,__LINE__): \
		{ \
			_Tp *_$1; \
			bo_rtbtree_p(aTree)->bt_elemc--; \
			_$1 = ___rtbt_rmnb__it; \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_RTBTREE_RemoveNode_b_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ============================================================================
 * LOOP HELPERS
 * @{
 */
# pragma mark -
# pragma mark [ loop helpers ]
# pragma mark -

/*
	bo_rtbtree tree;
	bo_rtbtree_iterator iter;
	...
	_BO_RTBTREE_FOREACH_LPR(iter, &tree)
		...
	FOREACH_END
 */
# define _BO_RTBTREE_FOREACH_LPR(anIter,aTree) { \
	size_t ___rtbt_foreach_lpr_count; \
	(anIter) = _BO_RTBTREE_Begin_lpr(aTree); \
	for (___rtbt_foreach_lpr_count = _BO_RTBTREE_Size(aTree); \
			___rtbt_foreach_lpr_count--; \
				(anIter) = __BO_RTBTREE_ITER_Next_lpr(anIter)) {

/*
	bo_rtbtree tree;
	bo_rtbtree_iterator iter;
	...
	_BO_RTBTREE_SAFE_FOREACH_LPR(iter, &tree)
		...
	FOREACH_END
 */
# define _BO_RTBTREE_SAFE_FOREACH_LPR(anIter,aTree) { \
	bo_rtbtree_iterator ___rtbt_safe_foreach_lpr_it; \
	size_t ___rtbt_safe_foreach_lpr_count; \
	(anIter) = _BO_RTBTREE_Begin_lpr(aTree); \
	for (___rtbt_safe_foreach_lpr_count = _BO_RTBTREE_Size(aTree); \
			(___rtbt_safe_foreach_lpr_count-- \
					? (___rtbt_safe_foreach_lpr_it = __BO_RTBTREE_ITER_Next_lpr(anIter)) \
					: NULL); \
				(anIter) = ___rtbt_safe_foreach_lpr_it) {

/* ------------------------------------------------------------------------- */

/*
	bo_rtbtree tree;
	...
	// iterator _$1;
	_BO_RTBTREE_FOREACH_LPR_(&tree)
		...
	FOREACH_END
 */
# define _BO_RTBTREE_FOREACH_LPR_(aTree) { \
	bo_rtbtree_iterator _$1; \
	size_t ___rtbt_foreach_lpr__count; \
	_$1 = _BO_RTBTREE_Begin_lpr(aTree); \
	for (___rtbt_foreach_lpr__count = _BO_RTBTREE_Size(aTree); \
			___rtbt_foreach_lpr__count--; \
				_$1 = __BO_RTBTREE_ITER_Next_lpr(_$1)) {

/*
	bo_rtbtree tree;
	...
	// iterator _$1;
	_BO_RTBTREE_SAFE_FOREACH_LPR_(&tree)
		...
	FOREACH_END
 */
# define _BO_RTBTREE_SAFE_FOREACH_LPR_(aTree) { \
	bo_rtbtree_iterator _$1, ___rtbt_safe_foreach_lpr__it; \
	size_t ___rtbt_safe_foreach_lpr__count; \
	_$1 = _BO_RTBTREE_Begin_lpr(aTree); \
	for (___rtbt_safe_foreach_lpr__count = _BO_RTBTREE_Size(aTree); \
			(___rtbt_safe_foreach_lpr__count-- ? \
					(___rtbt_safe_foreach_lpr__it = __BO_RTBTREE_ITER_Next_lpr(_$1)) \
					: NULL); \
				_$1 = ___rtbt_safe_foreach_lpr__it) {

/* ------------------------------------------------------------------------- */

/*
	bo_rtbtree tree;
	bo_rtbtree_plr_iterator iter;
	...
	_BO_RTBTREE_FOREACH_PLR(iter, &tree)
		...
	FOREACH_END
 */
# define _BO_RTBTREE_FOREACH_PLR(anIter,aTree) { \
	struct bo_btree_plr_iterator_s ___rtbt_foreach_plr_itbuf; \
	(anIter) = _BO_RTBTREE_Begin_plr(aTree, ___rtbt_foreach_plr_itbuf); \
	for ( ; (anIter); (anIter) = _BO_RTBTREE_ITER_Next_plr(anIter)) {

/* ------------------------------------------------------------------------- */

/*
	bo_rtbtree tree;
	...
	// iterator _$1;
	_BO_RTBTREE_FOREACH_PLR_(&tree)
		...
	FOREACH_END
 */
# define _BO_RTBTREE_FOREACH_PLR_(aTree) { \
	bo_rtbtree_plr_iterator _$1; \
	struct bo_btree_plr_iterator_s ___rtbt_foreach_plr__itbuf; \
	_$1 = _BO_RTBTREE_Begin_plr(aTree, ___rtbt_foreach_plr__itbuf); \
	for ( ; _$1; _$1 = _BO_RTBTREE_ITER_Next_plr(_$1)) {

/* ------------------------------------------------------------------------- */

# ifndef FOREACH_END
#  define FOREACH_END }}
# endif
/* @} */

#endif /* !__BO_FUNDIS_RTBTREE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
