/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Seqlist.h	(V.Sokur)
 * include/Bo/fundis/Seqlist.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FUNDIS_SEQLIST__
# define __BO_FUNDIS_SEQLIST__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/*
 * It is required the next files are optionally included first:
 *
# include <Bo/memman/BoMalloc.h>
# include <Bo/libc/String.h>
	// memmove, memcpy
 */

/******************************************************************************
 * %% BeginSection: sequential-list declarations
 */
# pragma mark -
# pragma mark [ sequential-list declarations ]
# pragma mark -

# define SEQLIST_MALLOC_DEFAULT(aSize,anErrPtr) _BoMalloc(aSize,anErrPtr)
# define SEQLIST_CALLOC_DEFAULT(anElemc,anElemsz,anErrPtr) \
	_BoMcalloc(anElemc,anElemsz,anErrPtr)
# define SEQLIST_REALLOC_DEFAULT(aPtr,aSize,anErrPtr) \
	_BoMrealloc(aPtr,aSize,anErrPtr)
# define SEQLIST_FREE_DEFAULT(aPtr) _BoMfree(aPtr)


# ifndef SEQLIST_MALLOC
#  define SEQLIST_MALLOC_DEFINED
#  define SEQLIST_MALLOC SEQLIST_MALLOC_DEFAULT
# endif

# ifndef SEQLIST_CALLOC
#  define SEQLIST_CALLOC_DEFINED
#  define SEQLIST_CALLOC SEQLIST_CALLOC_DEFAULT
# endif

# ifndef SEQLIST_REALLOC
#  define SEQLIST_REALLOC_DEFINED
#  define SEQLIST_REALLOC SEQLIST_REALLOC_DEFAULT
# endif

# ifndef SEQLIST_FREE
#  define SEQLIST_FREE_DEFINED
#  define SEQLIST_FREE SEQLIST_FREE_DEFAULT
# endif

/* ============================================================================
 * SEQLIST proper
 */
# pragma mark -
# pragma mark [ seqlist proper ]
# pragma mark -

struct __MYC_QALIGN(SIZEOF_PTR) bo_seqlist {
	size_t        capacity, elemc;
	bo_pointer_t  memory;
};

# define bo_seqlist_p(aList) ((struct bo_seqlist *)(aList))

# ifndef __cplusplus
#  define bo_tseqlist_p(_TListValueType,aList) \
	((struct __MYC_QALIGN(SIZEOF_PTR) { size_t capacity, elemc; _TListValueType *elemv; } *)(aList))
# else
#  define bo_tseqlist_p(_TListValueType,aList) \
	((struct __MYC_QALIGN(SIZEOF_PTR) __myc_xconcat(bo_seqlist,__LINE__) { \
			size_t capacity, elemc; _TListValueType *elemv; \
		} *)(aList))
# endif

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_seqlist,
	sizeof(struct bo_seqlist_s) == sizeof(struct bo_seqlist));
# endif /* _DEBUG */
/* @} */

/* ============================================================================
 * CONSTRUCT/DESTROY
 */
# pragma mark -
# pragma mark [ construct/destroy ]
# pragma mark -

/* void
   _BO_SEQLIST_Init(bo_seqlist *aList); */
# define _BO_SEQLIST_Init(aList) \
	do { \
		struct bo_seqlist *___seql_init_list = bo_seqlist_p(aList); \
		___seql_init_list->capacity = 0; \
		___seql_init_list->elemc = 0; \
		___seql_init_list->memory = NULL; \
	} while (0)

/* void
   BO_SEQLIST_Init_n< _TListValueType >(
		bo_seqlist *aList, size_t anElemc, {(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Init_n(_TListValueType,aList,anElemc,_NOMEMHANDLER) \
	do { \
		struct bo_seqlist *___seql_initn_list = bo_seqlist_p(aList); \
		_BO_SEQLIST_Init(___seql_initn_list); \
		if (NULL != (___seql_initn_list->memory = SEQLIST_CALLOC(anElemc,sizeof(_TListValueType),NULL))) { \
			bo_seqlist_p(___seql_initn_list)->capacity = (anElemc); \
			break; \
		} \
		_NOMEMHANDLER; \
	} while (0)

/* void
   BO_SEQLIST_Init_nx_< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist::const_pointer anElemv,
		size_t anElemc,
		{(_COPYPROC: (bo_seqlist::iterator _$1/dest/), (bo_seqlist::const_pointer _$2/source/))},
		{(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Init_nx_(_TListValueType,aList,anElemv,anElemc,_COPYPROC,_NOMEMHANDLER) \
	do { \
		struct bo_seqlist *___seql_initnx_list = bo_seqlist_p(aList); \
		struct bo_seqlist ___seql_initnx_source; \
		bo_seqlist_p(&___seql_initnx_source)->elemc = \
			bo_seqlist_p(&___seql_initnx_source)->capacity = (anElemc); \
		bo_seqlist_p(&___seql_initnx_source)->memory = (anElemv); \
		_BO_SEQLIST_Init(___seql_initnx_list); \
		/* BO_SEQLIST_Assign_x_(_TListValueType,___seql_initnx_list,&___seql_initnx_source,_COPYPROC,_NOMEMHANDLER); */ \
		do { \
			size_t ___seql_initnx_capacity, ___seql_initnx_countSrc; \
			_TListValueType *_$1; \
			_TListValueType const *_$2; \
			if (0 == (___seql_initnx_countSrc = _BO_SEQLIST_Size(&___seql_initnx_source))) { \
				_BO_SEQLIST_Clear_s(___seql_initnx_list); \
				break; \
			} \
			___seql_initnx_capacity = _BO_SEQLIST_Capacity(___seql_initnx_list); \
			if (___seql_initnx_capacity < ___seql_initnx_countSrc) { \
				___seql_initnx_countSrc -= ___seql_initnx_capacity; \
				BO_SEQLIST_Reserve(_TListValueType,___seql_initnx_list,___seql_initnx_countSrc,\
					goto __myc_xconcat(L_seqlist_assign_nomem,__LINE__)); \
			} \
			bo_seqlist_p(___seql_initnx_list)->elemc = 0; \
			_$1 = BO_SEQLIST_Begin(_TListValueType,___seql_initnx_list); \
			BO_SEQLIST_FOREACH(_TListValueType,_$2,&___seql_initnx_source) \
				_COPYPROC; \
				___seql_initnx_list->elemc++; \
				++_$1; \
			FOREACH_END \
			break; \
		__myc_xconcat(L_seqlist_assign_nomem,__LINE__): \
			_NOMEMHANDLER; \
		} while (0); \
	} while (0)

/* void
   BO_SEQLIST_From(bo_seqlist *aList, _TListValueType anElemv[N]); */
# define BO_SEQLIST_From(aList,anElemv) \
	do { \
		struct bo_seqlist *___seql_fromnc_list = bo_seqlist_p(aList); \
		bo_seqlist_p(___seql_fromnc_list)->elemc = \
			bo_seqlist_p(___seql_fromnc_list)->capacity = \
			__myc_namembs(anElemv); \
		bo_seqlist_p(___seql_fromnc_list)->memory = (bo_pointer_t)(anElemv); \
	} while (0)

/* void
   BO_SEQLIST_From_n(
		bo_seqlist *aList,
		bo_seqlist::const_pointer anElemv,
		size_t anElemc
	); */
# define BO_SEQLIST_From_n(aList,anElemv,anElemc) \
	do { \
		struct bo_seqlist *___seql_fromn_list = bo_seqlist_p(aList); \
		bo_seqlist_p(___seql_fromn_list)->elemc = \
			bo_seqlist_p(___seql_fromn_list)->capacity = (anElemc); \
		bo_seqlist_p(___seql_fromn_list)->memory = (bo_pointer_t)(anElemv); \
	} while (0)

/* void
   BO_SEQLIST_CopyInit_s< _TListValueType >(
		bo_seqlist *aList, bo_seqlist const *aSource, {(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_CopyInit_s(_TListValueType,aList,aSource,_NOMEMHANDLER) \
	do { \
		struct bo_seqlist *___seql_cpinis_list = bo_seqlist_p(aList); \
		_BO_SEQLIST_Init(___seql_cpinis_list); \
		/* BO_SEQLIST_Assign_s(_TListValueType,aList,aSource,_NOMEMHANDLER); */ \
		do { \
			size_t ___seql_cpinis_capacity, ___seql_cpinis_countSrc; \
			struct bo_seqlist *___seql_cpinis_source = bo_seqlist_p(aSource); \
			if (0 == (___seql_cpinis_countSrc = _BO_SEQLIST_Size(___seql_cpinis_source))) { \
				_BO_SEQLIST_Clear_s(___seql_cpinis_list); \
				break; \
			} \
			___seql_cpinis_capacity = _BO_SEQLIST_Capacity(___seql_cpinis_list); \
			if (___seql_cpinis_capacity < ___seql_cpinis_countSrc) { \
				___seql_cpinis_countSrc -= ___seql_cpinis_capacity; \
				BO_SEQLIST_Reserve(_TListValueType,___seql_cpinis_list,___seql_cpinis_countSrc,\
					goto __myc_xconcat(L_seqlist_assign_nomem,__LINE__)); \
			} \
			___seql_cpinis_countSrc = _BO_SEQLIST_Size(___seql_cpinis_source); \
			___seql_cpinis_capacity = sizeof(_TListValueType) * ___seql_cpinis_countSrc; \
			memcpy(___seql_cpinis_list->memory, ___seql_cpinis_source->memory, ___seql_cpinis_capacity); \
			___seql_cpinis_list->elemc = ___seql_cpinis_countSrc; \
			break; \
		__myc_xconcat(L_seqlist_assign_nomem,__LINE__): \
			_NOMEMHANDLER; \
		} while (0); \
	} while (0)

/* void
   BO_SEQLIST_CopyInit_x_< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist const *aSource,
		{(_COPYPROC: (bo_seqlist::iterator _$1/dest/), (bo_seqlist::const_iterator _$2/source/))},
		{(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_CopyInit_x_(_TListValueType,aList,aSource,_COPYPROC,_NOMEMHANDLER) \
	do { \
		struct bo_seqlist *___seql_cpinix_list = bo_seqlist_p(aList); \
		_BO_SEQLIST_Init(___seql_cpinix_list); \
		/* BO_SEQLIST_Assign_x_(_TListValueType,aList,aSource,_COPYPROC,_NOMEMHANDLER); */ \
		do { \
			size_t ___seql_cpinix_capacity, ___seql_cpinix_countSrc; \
			_TListValueType *_$1; \
			_TListValueType const *_$2; \
			struct bo_seqlist *___seql_cpinix_source = bo_seqlist_p(aSource); \
			if (0 == (___seql_cpinix_countSrc = _BO_SEQLIST_Size(___seql_cpinix_source))) { \
				_BO_SEQLIST_Clear_s(___seql_cpinix_list); \
				break; \
			} \
			___seql_cpinix_capacity = _BO_SEQLIST_Capacity(___seql_cpinix_list); \
			if (___seql_cpinix_capacity < ___seql_cpinix_countSrc) { \
				___seql_cpinix_countSrc -= ___seql_cpinix_capacity; \
				BO_SEQLIST_Reserve(_TListValueType,___seql_cpinix_list,___seql_cpinix_countSrc,\
					goto __myc_xconcat(L_seqlist_assign_nomem,__LINE__)); \
			} \
			bo_seqlist_p(___seql_cpinix_list)->elemc = 0; \
			_$1 = BO_SEQLIST_Begin(_TListValueType,___seql_cpinix_list); \
			BO_SEQLIST_FOREACH(_TListValueType,_$2,___seql_cpinix_source) \
				_COPYPROC; \
				___seql_cpinix_list->elemc++; \
				++_$1; \
			FOREACH_END \
			break; \
		__myc_xconcat(L_seqlist_assign_nomem,__LINE__): \
			_NOMEMHANDLER; \
		} while (0); \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   _BO_SEQLIST_Destroy_s(bo_seqlist *aList); */
# define _BO_SEQLIST_Destroy_s(aList) \
	do { \
		struct bo_seqlist *___seql_dstroys_list = bo_seqlist_p(aList); \
		SEQLIST_FREE(___seql_dstroys_list->memory); \
		___seql_dstroys_list->memory = NULL; \
		___seql_dstroys_list->elemc = ___seql_dstroys_list->capacity = 0; \
	} while (0)

/* void
   BO_SEQLIST_Destroy_x< _TListValueType >(
		bo_seqlist *aList,
		void (*_DTOR)(bo_seqlist::pointer )
	); */
# define BO_SEQLIST_Destroy_x(_TListValueType,aList,_DTOR) \
	do { \
		_TListValueType *___seql_dstroyx_it; \
		struct bo_seqlist *___seql_dstroyx_list = bo_seqlist_p(aList); \
		BO_SEQLIST_FOREACH(_TListValueType,___seql_dstroyx_it,___seql_dstroyx_list) \
			_DTOR(___seql_dstroyx_it); \
		FOREACH_END \
		SEQLIST_FREE(___seql_dstroyx_list->memory); \
		___seql_dstroyx_list->memory = NULL; \
		___seql_dstroyx_list->elemc = ___seql_dstroyx_list->capacity = 0; \
	} while (0)

/* void
   BO_SEQLIST_Destroy_x_< _TListValueType >(
		bo_seqlist *aList,
		{(_DTOR: (bo_seqlist::pointer _$1))}
	); */
# define BO_SEQLIST_Destroy_x_(_TListValueType,aList,_DTOR) \
	do { \
		struct bo_seqlist *___seql_dstroyx__list = bo_seqlist_p(aList); \
		BO_SEQLIST_FOREACH_(_TListValueType,___seql_dstroyx__list) \
			_DTOR; \
		FOREACH_END \
		SEQLIST_FREE(___seql_dstroyx__list->memory); \
		___seql_dstroyx__list->memory = NULL; \
		___seql_dstroyx__list->elemc = ___seql_dstroyx__list->capacity = 0; \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   _BO_SEQLIST_Clear_s(bo_seqlist *aList); */
# define _BO_SEQLIST_Clear_s(aList) \
	((void) (bo_seqlist_p(aList)->elemc = 0))

/* void
   BO_SEQLIST_Clear_x< _TListValueType >(
		bo_seqlist *aList,
		void (*_DTOR)(bo_seqlist::pointer )
	); */
# define BO_SEQLIST_Clear_x(_TListValueType,aList,_DTOR) \
	do { \
		_TListValueType *___seql_clearx_it; \
		struct bo_seqlist *___seql_clearx_list = bo_seqlist_p(aList); \
		BO_SEQLIST_FOREACH(_TListValueType,___seql_clearx_it,___seql_clearx_list) \
			_DTOR(___seql_clearx_it); \
		FOREACH_END \
		___seql_clearx_list->elemc = 0; \
	} while (0)

/* void
   BO_SEQLIST_Clear_x_< _TListValueType >(
		bo_seqlist *aList,
		{(_DTOR: (bo_seqlist::pointer _$1))}
	); */
# define BO_SEQLIST_Clear_x_(_TListValueType,aList,_DTOR) \
	do { \
		struct bo_seqlist *___seql_clearx__list = bo_seqlist_p(aList); \
		BO_SEQLIST_FOREACH_(_TListValueType,___seql_clearx__list) \
			_DTOR; \
		FOREACH_END \
		___seql_clearx__list->elemc = 0; \
	} while (0)

/* ------------------------------------------------------------------------- */

# define _BO_SEQLIST_Swap(aList1, aList2) \
	do { \
		__myc_swap_i(bo_seqlist_p(aList1)->capacity, bo_seqlist_p(aList2)->capacity); \
		__myc_swap_i(bo_seqlist_p(aList1)->elemc, bo_seqlist_p(aList2)->elemc); \
		__myc_swap_p(bo_seqlist_p(aList1)->memory, bo_seqlist_p(aList2)->memory); \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_Assign_s< _TListValueType >(
		bo_seqlist *aList, bo_seqlist const *aSource, {(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Assign_s(_TListValueType,aList,aSource,_NOMEMHANDLER) \
	do { \
		size_t ___seql_asgns_capacity, ___seql_asgns_countSrc; \
		struct bo_seqlist *___seql_asgns_list = bo_seqlist_p(aList); \
		struct bo_seqlist *___seql_asgns_source = bo_seqlist_p(aSource); \
		if (0 == (___seql_asgns_countSrc = _BO_SEQLIST_Size(___seql_asgns_source))) { \
			_BO_SEQLIST_Clear_s(___seql_asgns_list); \
			break; \
		} \
		___seql_asgns_capacity = _BO_SEQLIST_Capacity(___seql_asgns_list); \
		if (___seql_asgns_capacity < ___seql_asgns_countSrc) { \
			___seql_asgns_countSrc -= ___seql_asgns_capacity; \
			BO_SEQLIST_Reserve(_TListValueType,___seql_asgns_list,___seql_asgns_countSrc,\
				goto __myc_xconcat(L_seqlist_asgns_nomem,__LINE__)); \
		} \
		___seql_asgns_countSrc = _BO_SEQLIST_Size(___seql_asgns_source); \
		___seql_asgns_capacity = sizeof(_TListValueType) * ___seql_asgns_countSrc; \
		memcpy(___seql_asgns_list->memory, ___seql_asgns_source->memory, ___seql_asgns_capacity); \
		___seql_asgns_list->elemc = ___seql_asgns_countSrc; \
		break; \
	__myc_xconcat(L_seqlist_asgns_nomem,__LINE__): \
		_NOMEMHANDLER; \
	} while (0)

/* void
   BO_SEQLIST_Assign_x_< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist const *aSource,
		{(_COPYPROC: (bo_seqlist::iterator _$1/dest/), (bo_seqlist::const_iterator _$2/source/))},
		{(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Assign_x_(_TListValueType,aList,aSource,_COPYPROC,_NOMEMHANDLER) \
	do { \
		_TListValueType *_$1; \
		_TListValueType const *_$2; \
		size_t ___seql_asgnx_capacity, ___seql_asgnx_countSrc; \
		struct bo_seqlist *___seql_asgnx_list = bo_seqlist_p(aList); \
		struct bo_seqlist *___seql_asgnx_source = bo_seqlist_p(aSource); \
		if (0 == (___seql_asgnx_countSrc = _BO_SEQLIST_Size(___seql_asgnx_source))) { \
			_BO_SEQLIST_Clear_s(___seql_asgnx_list); \
			break; \
		} \
		___seql_asgnx_capacity = _BO_SEQLIST_Capacity(___seql_asgnx_list); \
		if (___seql_asgnx_capacity < ___seql_asgnx_countSrc) { \
			___seql_asgnx_countSrc -= ___seql_asgnx_capacity; \
			BO_SEQLIST_Reserve(_TListValueType,___seql_asgnx_list,___seql_asgnx_countSrc,\
				goto __myc_xconcat(L_seqlist_asgnx_nomem,__LINE__)); \
		} \
		___seql_asgnx_list->elemc = 0; \
		_$1 = BO_SEQLIST_Begin(_TListValueType,___seql_asgnx_list); \
		BO_SEQLIST_FOREACH(_TListValueType,_$2,___seql_asgnx_source) \
			_COPYPROC; \
			___seql_asgnx_list->elemc++; \
			++_$1; \
		FOREACH_END \
		break; \
	__myc_xconcat(L_seqlist_asgnx_nomem,__LINE__): \
		_NOMEMHANDLER; \
	} while (0)

/* ============================================================================
 * PROPERTIES: ACCESS
 */
# pragma mark -
# pragma mark [ properties: access ]
# pragma mark -

/* size_t
   _BO_SEQLIST_Capacity(bo_seqlist *aList); */
# define _BO_SEQLIST_Capacity(aList) (bo_seqlist_p(aList)->capacity)

/* size_t
   _BO_SEQLIST_Size(bo_seqlist *aList); */
# define _BO_SEQLIST_Size(aList) (bo_seqlist_p(aList)->elemc)

/* size_t
   _BO_SEQLIST_FreeSpace(bo_seqlist *aList); */
# define _BO_SEQLIST_FreeSpace(aList) \
	(_BO_SEQLIST_Capacity(aList) - _BO_SEQLIST_Size(aList))

/* bool
   _BO_SEQLIST_Empty(bo_seqlist *aList); */
# define _BO_SEQLIST_Empty(aList) (0 == _BO_SEQLIST_Size(aList))

/* bool
   _BO_SEQLIST_Full(bo_seqlist *aList); */
# define _BO_SEQLIST_Full(aList) \
	(_BO_SEQLIST_Capacity(aList) == _BO_SEQLIST_Size(aList))

/* ============================================================================
 * METHODS: ACCESS
 */
# pragma mark -
# pragma mark [ methods: access ]
# pragma mark -

/* bo_seqlist::iterator
   BO_SEQLIST_Begin< _TListValueType >(bo_seqlist *aList); */
# ifndef __cplusplus
#  define BO_SEQLIST_Begin(_TListValueType,aList) \
	(bo_tseqlist_p(_TListValueType,aList)->elemv)
# else
#  define BO_SEQLIST_Begin(_TListValueType,aList) \
	(reinterpret_cast< _TListValueType* >(bo_seqlist_p(aList)->memory))
# endif

/* bo_seqlist::iterator
   BO_SEQLIST_End< _TListValueType >(bo_seqlist *aList); */
# define BO_SEQLIST_End(_TListValueType,aList) \
	(BO_SEQLIST_Begin(_TListValueType,aList) + _BO_SEQLIST_Size(aList))

/* ------------------------------------------------------------------------- */

/* bo_seqlist::reference
   BO_SEQLIST_At< _TListValueType >(bo_seqlist *aList, bo_index_t anIndex); */
# define BO_SEQLIST_At(_TListValueType,aList,anIndex) \
	((0 <= (anIndex)) \
		? BO_SEQLIST_Atb(_TListValueType,aList,anIndex) \
		: BO_SEQLIST_Ate(_TListValueType,aList,anIndex))

/* bo_seqlist::reference
   BO_SEQLIST_Atb< _TListValueType >(bo_seqlist *aList, bo_index_t anIndex); */
# define BO_SEQLIST_Atb(_TListValueType,aList,anIndex) \
	(BO_SEQLIST_Begin(_TListValueType,aList)[(anIndex)])

/* bo_seqlist::reference
   BO_SEQLIST_Ate< _TListValueType >(bo_seqlist *aList, bo_index_t anIndex); */
# define BO_SEQLIST_Ate(_TListValueType,aList,anIndex) \
	(BO_SEQLIST_End(_TListValueType,aList)[(anIndex)])

/* bo_seqlist::reference
   BO_SEQLIST_Front< _TListValueType >(bo_seqlist *aList); */
# define BO_SEQLIST_Front(_TListValueType,aList) \
	(BO_SEQLIST_Begin(_TListValueType,aList)[0])

/* bo_seqlist::reference
   BO_SEQLIST_Back< _TListValueType >(bo_seqlist *aList); */
# define BO_SEQLIST_Back(_TListValueType,aList) \
	(BO_SEQLIST_Begin(_TListValueType,aList)[_BO_SEQLIST_Size(aList) - 1])

/* ------------------------------------------------------------------------- */

/* bo_index_t
   BO_SEQLIST_Index< _TListValueType >(bo_seqlist *aList, bo_seqlist::iterator aPos); */
# ifndef __cplusplus
#  define BO_SEQLIST_Index(_TListValueType,aList,aPos) \
	((bo_index_t)((_TListValueType *)(aPos) - BO_SEQLIST_Begin(_TListValueType,aList)))
# else
#  define BO_SEQLIST_Index(_TListValueType,aList,aPos) \
	((bo_index_t)(reinterpret_cast< _TListValueType* >(aPos) - BO_SEQLIST_Begin(_TListValueType,aList)))
# endif

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

/* void
   BO_SEQLIST_Enumerate< _TListValueType >(
		bo_seqlist::iterator anIter,
		bo_seqlist *aList,
		{(_PREDICATE: bo_seqlist::iterator anIter)}
	); */
# define BO_SEQLIST_Enumerate(_TListValueType,anIter,aList,_PREDICATE) \
	do { \
		BO_SEQLIST_FOREACH(_TListValueType,anIter,aList) \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* void
   BO_SEQLIST_Enumerate_< _TListValueType >(
		bo_seqlist *aList,
		{(_PREDICATE: (bo_seqlist::iterator _$1))}
	); */
# define BO_SEQLIST_Enumerate_(_TListValueType,aList,_PREDICATE) \
	do { \
		BO_SEQLIST_FOREACH_(_TListValueType,aList) \
			_PREDICATE; \
		FOREACH_END \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_Find< _TListValueType >(
		bo_seqlist::iterator anIterRef,
		bo_seqlist *aList,
		{(_PREDICATE: (_Bool _$0), (bo_seqlist::iterator anIterRef))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_SEQLIST_Find(_TListValueType,anIterRef,aList,_PREDICATE,_NOTFOUNDHANDLER) \
	do { \
		_Bool _$0 = false; \
		BO_SEQLIST_Enumerate(_TListValueType,anIterRef,aList,{ \
			_PREDICATE/*(_$0,anIterRef)*/; \
			if (_$0) \
				break; \
		}); \
		if (!_$0) { \
			_NOTFOUNDHANDLER; \
		} \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_Insert< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist::iterator aPos,
		bo_seqlist::const_reference aValueRef,
		{(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Insert(_TListValueType,aList,aPos,aValueRef,_NOMEMHANDLER) \
	do { \
		struct bo_seqlist *___seql_ins_list = bo_seqlist_p(aList); \
		bo_index_t ___seql_ins_idx = BO_SEQLIST_Index(_TListValueType,___seql_ins_list,aPos); \
		BO_SEQLIST_InsertByIndex(_TListValueType,___seql_ins_list,___seql_ins_idx,aValueRef,_NOMEMHANDLER); \
	} while (0)

/* void
   BO_SEQLIST_InsertByIndex< _TListValueType >(
		bo_seqlist *aList,
		bo_index_t anIndex,
		bo_seqlist::const_reference aValueRef,
		{(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_InsertByIndex(_TListValueType,aList,anIndex,aValueRef,_NOMEMHANDLER) \
	do { \
		bo_index_t ___seql_insbi_idx = (anIndex); \
		struct bo_seqlist *___seql_insbi_list = bo_seqlist_p(aList); \
		if (_BO_SEQLIST_Full(___seql_insbi_list)) { \
			BO_SEQLIST_Reserve(_TListValueType,___seql_insbi_list,1,\
				goto __myc_xconcat(L_seqlist_overflow,__LINE__)); \
		} \
		if (___seql_insbi_idx < 0) \
			___seql_insbi_idx += (bo_index_t)_BO_SEQLIST_Size(___seql_insbi_list); \
		assert(0 <= ___seql_insbi_idx && (size_t)___seql_insbi_idx <= _BO_SEQLIST_Size(___seql_insbi_list)); \
		if ((size_t)___seql_insbi_idx < _BO_SEQLIST_Size(___seql_insbi_list)) { \
			memmove(&BO_SEQLIST_Atb(_TListValueType,___seql_insbi_list,___seql_insbi_idx + 1), \
				&BO_SEQLIST_Atb(_TListValueType,___seql_insbi_list,___seql_insbi_idx), \
				((_BO_SEQLIST_Size(___seql_insbi_list) - ___seql_insbi_idx) * sizeof(_TListValueType))); \
		} \
		BO_SEQLIST_Atb(_TListValueType,___seql_insbi_list,___seql_insbi_idx) = (aValueRef); \
		___seql_insbi_list->elemc++; \
		break; \
	__myc_xconcat(L_seqlist_overflow,__LINE__): \
		_NOMEMHANDLER; \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_Remove< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist::iterator aPos,
		bo_seqlist::reference aValueRef
	); */
# define BO_SEQLIST_Remove(_TListValueType,aList,aPos,aValueRef) \
	do {                                                          \
		struct bo_seqlist *___seql_rm_list = bo_seqlist_p(aList); \
		bo_index_t ___seql_rm_idx = BO_SEQLIST_Index(_TListValueType,___seql_rm_list,aPos); \
		BO_SEQLIST_RemoveByIndex(_TListValueType,___seql_rm_list,___seql_rm_idx,aValueRef); \
	} while (0)

/* void
   BO_SEQLIST_RemoveByIndex< _TListValueType >(
		bo_seqlist *aList,
		bo_index_t anIndex,
		bo_seqlist::reference aValueRef
	); */
# define BO_SEQLIST_RemoveByIndex(_TListValueType,aList,anIndex,aValueRef) \
	do { \
		bo_index_t ___seql_rmbi_idx = (anIndex); \
		struct bo_seqlist *___seql_rmbi_list = bo_seqlist_p(aList); \
		if (___seql_rmbi_idx < 0) \
			___seql_rmbi_idx += (bo_index_t)_BO_SEQLIST_Size(___seql_rmbi_list); \
		assert(0 <= ___seql_rmbi_idx && (size_t)___seql_rmbi_idx < _BO_SEQLIST_Size(___seql_rmbi_list)); \
		___seql_rmbi_list->elemc--; \
		(aValueRef) = BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx); \
		if ((size_t)___seql_rmbi_idx < _BO_SEQLIST_Size(___seql_rmbi_list)) { \
			memmove(&BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx), \
			        &BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx + 1), \
			        ((_BO_SEQLIST_Size(___seql_rmbi_list) - ___seql_rmbi_idx) * sizeof(_TListValueType))); \
		} \
	} while (0)

# define BO_SEQLIST_RemoveByIndex_chk(_TListValueType,aList,anIndex,aValueRef,_INVALHANDLER) \
	do { \
		bo_index_t ___seql_rmbi_idx = (anIndex); \
		struct bo_seqlist *___seql_rmbi_list = bo_seqlist_p(aList); \
		if (___seql_rmbi_idx < 0) \
			___seql_rmbi_idx += (bo_index_t)_BO_SEQLIST_Size(___seql_rmbi_list); \
		if (0 <= ___seql_rmbi_idx && (size_t)___seql_rmbi_idx < _BO_SEQLIST_Size(___seql_rmbi_list)) { \
			___seql_rmbi_list->elemc--; \
			(aValueRef) = BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx); \
			if ((size_t)___seql_rmbi_idx < _BO_SEQLIST_Size(___seql_rmbi_list)) { \
				memmove(&BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx), \
				        &BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx + 1), \
				        ((_BO_SEQLIST_Size(___seql_rmbi_list) - ___seql_rmbi_idx) * sizeof(_TListValueType))); \
			} \
			break; \
		} \
		_INVALHANDLER; \
	} while (0)


# define BO_SEQLIST_RemoveByIndex_n(_TListValueType,aList,anIndex,aCountRef,aValuev,_INVALHANDLER) \
	do { \
		bo_index_t ___seql_rmbi_idx = (anIndex); \
		struct bo_seqlist *___seql_rmbi_list = bo_seqlist_p(aList); \
		bo_pointer_t ___seql_rmbi_valuev = (bo_pointer_t)(aValuev); \
		if (___seql_rmbi_idx < 0) \
			___seql_rmbi_idx += (bo_index_t)_BO_SEQLIST_Size(___seql_rmbi_list); \
		if (0 <= ___seql_rmbi_idx && (size_t)___seql_rmbi_idx < _BO_SEQLIST_Size(___seql_rmbi_list)) { \
			if (_BO_SEQLIST_Size(___seql_rmbi_list) - ___seql_rmbi_idx < (aCountRef)) \
				(aCountRef) = _BO_SEQLIST_Size(___seql_rmbi_list) - ___seql_rmbi_idx; \
			___seql_rmbi_list->elemc -= (aCountRef); \
			if (NULL != ___seql_rmbi_valuev) { \
				memmove(___seql_rmbi_valuev, &BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx), \
				        (aCountRef) * sizeof(_TListValueType)); \
			} \
			if ((size_t)___seql_rmbi_idx < _BO_SEQLIST_Size(___seql_rmbi_list)) { \
				memmove(&BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx), \
				        &BO_SEQLIST_Atb(_TListValueType,___seql_rmbi_list,___seql_rmbi_idx + (aCountRef)), \
				        ((_BO_SEQLIST_Size(___seql_rmbi_list) - ___seql_rmbi_idx) * sizeof(_TListValueType))); \
			} \
			break; \
		} \
		_INVALHANDLER; \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_Push< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist::const_reference aValueRef,
		{(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Push(_TListValueType,aList,aValueRef,_NOMEMHANDLER) \
	do { \
		struct bo_seqlist *___seql_push_list = bo_seqlist_p(aList); \
		if (_BO_SEQLIST_Full(___seql_push_list)) { \
			BO_SEQLIST_Reserve(_TListValueType,___seql_push_list,1,\
				goto __myc_xconcat(L_seqlist_overflow,__LINE__)); \
		} \
		*BO_SEQLIST_End(_TListValueType,___seql_push_list) = (aValueRef); \
		___seql_push_list->elemc++; \
		break; \
	__myc_xconcat(L_seqlist_overflow,__LINE__): \
		_NOMEMHANDLER; \
	} while (0)

# define BO_SEQLIST_PushX2(_TListValueType,aList,aValueRef,_NOMEMHANDLER) \
	do { \
		struct bo_seqlist *___seql_push_list = bo_seqlist_p(aList); \
		if (_BO_SEQLIST_Full(___seql_push_list)) { \
			BO_SEQLIST_ReserveX2(_TListValueType,___seql_push_list,\
				goto __myc_xconcat(L_seqlist_overflow,__LINE__)); \
		} \
		*BO_SEQLIST_End(_TListValueType,___seql_push_list) = (aValueRef); \
		___seql_push_list->elemc++; \
		break; \
	__myc_xconcat(L_seqlist_overflow,__LINE__): \
		_NOMEMHANDLER; \
	} while (0)

# define BO_SEQLIST_Push_nck(_TListValueType,aList,aValueRef) \
	do { \
		struct bo_seqlist *___seql_push_list = bo_seqlist_p(aList); \
		*BO_SEQLIST_End(_TListValueType,___seql_push_list) = (aValueRef); \
		___seql_push_list->elemc++; \
	} while (0)

/* void
   BO_SEQLIST_Pop< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist::reference aValueRef,
		{(_UNDERFLOWHANDLER)}
	); */
# define BO_SEQLIST_Pop(_TListValueType,aList,aValueRef,_UNDERFLOWHANDLER) \
	do { \
		struct bo_seqlist *___seql_pop_list = bo_seqlist_p(aList); \
		if (_BO_SEQLIST_Size(___seql_pop_list)) { \
			___seql_pop_list->elemc--; \
			(aValueRef) = *BO_SEQLIST_End(_TListValueType,___seql_pop_list); \
			break; \
		} \
		_UNDERFLOWHANDLER; \
	} while (0)

/* void
   _BO_SEQLIST_Pop_s(bo_seqlist *aList); */
# define _BO_SEQLIST_Pop_s(aList) (bo_seqlist_p(aList)->elemc--)

/* void
   _BO_SEQLIST_Pop_n(bo_seqlist *aList); */
# define _BO_SEQLIST_Pop_n(aList,N) (bo_seqlist_p(aList)->elemc -= (N))

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_Append_s< _TListValueType >(
		bo_seqlist *aList, bo_seqlist const *aSource, {(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Append_s(_TListValueType,aList,aSource,_NOMEMHANDLER) \
	do { \
		size_t ___seql_apnds_space, ___seql_apnds_countSrc; \
		struct bo_seqlist *___seql_apnds_list = bo_seqlist_p(aList); \
		struct bo_seqlist *___seql_apnds_source = bo_seqlist_p(aSource); \
		if (0 == (___seql_apnds_countSrc = _BO_SEQLIST_Size(___seql_apnds_source))) { \
			break; \
		} \
		___seql_apnds_space = _BO_SEQLIST_FreeSpace(___seql_apnds_list); \
		if (___seql_apnds_space < ___seql_apnds_countSrc) { \
			___seql_apnds_countSrc -= ___seql_apnds_space; \
			BO_SEQLIST_Reserve(_TListValueType,___seql_apnds_list,___seql_apnds_countSrc,\
				goto __myc_xconcat(L_seqlist_apnds_nomem,__LINE__)); \
		} \
		___seql_apnds_countSrc = _BO_SEQLIST_Size(___seql_apnds_source); \
		___seql_apnds_space = sizeof(_TListValueType) * ___seql_apnds_countSrc; \
		memcpy(BO_SEQLIST_End(_TListValueType,___seql_apnds_list), \
			___seql_apnds_source->memory, ___seql_apnds_space); \
		___seql_apnds_list->elemc += ___seql_apnds_countSrc; \
		break; \
	__myc_xconcat(L_seqlist_apnds_nomem,__LINE__): \
		_NOMEMHANDLER; \
	} while (0)

/* void
   BO_SEQLIST_Append_x_< _TListValueType >(
		bo_seqlist *aList,
		bo_seqlist const *aSource,
		{(_COPYPROC: (bo_seqlist::iterator _$1/dest/), (bo_seqlist::const_iterator _$2/source/))},
		{(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Append_x_(_TListValueType,aList,aSource,_COPYPROC,_NOMEMHANDLER) \
	do { \
		_TListValueType *_$1; \
		_TListValueType const *_$2; \
		size_t ___seql_apndx_space, ___seql_apndx_countSrc; \
		struct bo_seqlist *___seql_apndx_list = bo_seqlist_p(aList); \
		struct bo_seqlist *___seql_apndx_source = bo_seqlist_p(aSource); \
		if (0 == (___seql_apndx_countSrc = _BO_SEQLIST_Size(___seql_apndx_source))) { \
			break; \
		} \
		___seql_apndx_space = _BO_SEQLIST_FreeSpace(___seql_apndx_list); \
		if (___seql_apndx_space < ___seql_apndx_countSrc) { \
			___seql_apndx_countSrc -= ___seql_apndx_space; \
			BO_SEQLIST_Reserve(_TListValueType,___seql_apndx_list,___seql_apndx_countSrc,\
				goto __myc_xconcat(L_seqlist_apndx_nomem,__LINE__)); \
		} \
		_$1 = BO_SEQLIST_End(_TListValueType,___seql_apndx_list); \
		BO_SEQLIST_FOREACH(_TListValueType,_$2,___seql_apndx_source) \
			_COPYPROC; \
			___seql_apndx_list->elemc++; \
			++_$1; \
		FOREACH_END \
		break; \
	__myc_xconcat(L_seqlist_apndx_nomem,__LINE__): \
		_NOMEMHANDLER; \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_Reserve< _TListValueType >(
		bo_seqlist *aList, size_t anExtraElemc, {(_NOMEMHANDLER)}
	); */
# define BO_SEQLIST_Reserve(_TListValueType,aList,anExtraElemc,_NOMEMHANDLER) \
	do { \
		bo_pointer_t ___seql_rsrv_ptr; \
		size_t ___seql_rsrv_newsz, ___seql_rsrv_extrac = (anExtraElemc); \
		struct bo_seqlist *___seql_rsrv_list = bo_seqlist_p(aList); \
		___seql_rsrv_newsz = (_BO_SEQLIST_Capacity(___seql_rsrv_list) + ___seql_rsrv_extrac) * \
			sizeof(_TListValueType); \
		___seql_rsrv_ptr = SEQLIST_REALLOC(___seql_rsrv_list->memory,___seql_rsrv_newsz,NULL); \
		if (NULL != ___seql_rsrv_ptr) { \
			___seql_rsrv_list->memory = ___seql_rsrv_ptr; \
			___seql_rsrv_list->capacity += ___seql_rsrv_extrac; \
			break; \
		} \
		_NOMEMHANDLER; \
	} while (0)

# define BO_SEQLIST_ReserveX2(_TListValueType,aList,_NOMEMHANDLER) \
	do { \
		bo_pointer_t ___seql_rsrv_ptr; \
		size_t ___seql_rsrv_newsz, ___seql_rsrv_newc; \
		struct bo_seqlist *___seql_rsrv_list = bo_seqlist_p(aList); \
		___seql_rsrv_newc = 2 * _BO_SEQLIST_Capacity(___seql_rsrv_list); \
		if (0 == ___seql_rsrv_newc) \
			___seql_rsrv_newc = 2; \
		___seql_rsrv_newsz = ___seql_rsrv_newc * sizeof(_TListValueType); \
		___seql_rsrv_ptr = SEQLIST_REALLOC(___seql_rsrv_list->memory,___seql_rsrv_newsz,NULL); \
		if (NULL != ___seql_rsrv_ptr) { \
			___seql_rsrv_list->memory = ___seql_rsrv_ptr; \
			___seql_rsrv_list->capacity = ___seql_rsrv_newc; \
			break; \
		} \
		_NOMEMHANDLER; \
	} while (0)

/* ------------------------------------------------------------------------- */

/* void
   BO_SEQLIST_XchgI2< _TListValueType >(
		bo_seqlist *aList, bo_index_t anIndex1, bo_index_t anIndex2
	); */
# define BO_SEQLIST_XchgI2(_TListValueType,aList,anIndex1,anIndex2) \
	do { \
		_TListValueType ___seql_xchgi2_val; \
		bo_index_t ___seql_xchgi2_idx1 = (anIndex1), ___seql_xchgi2_idx2 = (anIndex2); \
		struct bo_seqlist *___seql_xchgi2_list = bo_seqlist_p(aList); \
		assert(2 <= _BO_SEQLIST_Size(___seql_xchgi2_list)); \
		assert(0 <= ___seql_xchgi2_idx1 && 0 <= ___seql_xchgi2_idx2); \
		___seql_xchgi2_val = BO_SEQLIST_Atb(_TListValueType,___seql_xchgi2_list,___seql_xchgi2_idx1); \
		BO_SEQLIST_Atb(_TListValueType,___seql_xchgi2_list,___seql_xchgi2_idx1) = \
			BO_SEQLIST_Atb(_TListValueType,___seql_xchgi2_list,___seql_xchgi2_idx2); \
		BO_SEQLIST_Atb(_TListValueType,___seql_xchgi2_list,___seql_xchgi2_idx2) = ___seql_xchgi2_val; \
	} while (0)

/* void
   BO_SEQLIST_XchgIBack< _TListValueType >(bo_seqlist *aList, bo_index_t anIndex); */
# define BO_SEQLIST_XchgIBack(_TListValueType,aList,anIndex) \
	do { \
		_TListValueType ___seql_xchgib_val; \
		bo_index_t ___seql_xchgib_idx = (anIndex); \
		struct bo_seqlist *___seql_xchgib_list = bo_seqlist_p(aList); \
		assert(2 <= _BO_SEQLIST_Size(___seql_xchgib_list)); \
		assert(0 <= ___seql_xchgib_idx); \
		___seql_xchgib_val = BO_SEQLIST_Atb(_TListValueType,___seql_xchgib_list,___seql_xchgib_idx); \
		BO_SEQLIST_Atb(_TListValueType,___seql_xchgib_list,___seql_xchgib_idx) = \
			BO_SEQLIST_Back(_TListValueType,___seql_xchgib_list); \
		BO_SEQLIST_Back(_TListValueType,___seql_xchgib_list) = ___seql_xchgib_val; \
	} while (0)

/* void
   BO_SEQLIST_XchgBack< _TListValueType >(bo_seqlist *aList); */
# define BO_SEQLIST_XchgBack(_TListValueType,aList) \
	do { \
		_TListValueType ___seql_xchgb_val; \
		bo_index_t ___seql_xchgb_idx = -2; \
		struct bo_seqlist *___seql_xchgb_list = bo_seqlist_p(aList); \
		assert(2 <= _BO_SEQLIST_Size(___seql_xchgb_list)); \
		___seql_xchgb_val = BO_SEQLIST_Ate(_TListValueType,___seql_xchgb_list,___seql_xchgb_idx); \
		BO_SEQLIST_Ate(_TListValueType,___seql_xchgb_list,___seql_xchgb_idx) = \
			BO_SEQLIST_Back(_TListValueType,___seql_xchgb_list); \
		BO_SEQLIST_Back(_TListValueType,___seql_xchgb_list) = ___seql_xchgb_val; \
	} while (0)

/* ============================================================================
 * LOOP HELPERS
 * @{
 */
# pragma mark -
# pragma mark [ loop helpers ]
# pragma mark -

/*
	bo_seqlist list;
	bo_seqlist::iterator iter;
	...
	BO_SEQLIST_FOREACH< _TListValueType >(iter, &list)
		...
	FOREACH_END
 */
# define BO_SEQLIST_FOREACH(_TListValueType,anIter,aList) { \
	size_t ___seql_foreach_count; \
	struct bo_seqlist *___seql_foreach_list = bo_seqlist_p(aList); \
	(anIter) = BO_SEQLIST_Begin(_TListValueType,___seql_foreach_list); \
	for (___seql_foreach_count = _BO_SEQLIST_Size(___seql_foreach_list); \
			___seql_foreach_count--; \
				++(anIter)) {

/* ------------------------------------------------------------------------- */

/*
	bo_seqlist list;
	...
	// iterator *_$1;
	BO_SEQLIST_FOREACH_< _TListValueType >(&list)
		...
	FOREACH_END
 */
# define BO_SEQLIST_FOREACH_(_TListValueType,aList) { \
	_TListValueType *_$1; \
	size_t ___seql_foreach__count; \
	struct bo_seqlist *___seql_foreach__list = bo_seqlist_p(aList); \
	_$1 = BO_SEQLIST_Begin(_TListValueType,___seql_foreach__list); \
	for (___seql_foreach__count = _BO_SEQLIST_Size(___seql_foreach__list); \
			___seql_foreach__count--; \
				++_$1) {

/* ------------------------------------------------------------------------- */

/*
	bo_seqlist list;
	bo_seqlist::iterator iter;
	bo_seqlist::iterator iterNext;
	...
	BO_SEQLIST_FOREACH2< _TListValueType >(iter, iterNext, &list)
		...
	FOREACH_END
 */
# define BO_SEQLIST_FOREACH2(_TListValueType,anIter,anIterNext,aList) { \
	size_t ___seql_foreach2_count; \
	struct bo_seqlist *___seql_foreach2_list = bo_seqlist_p(aList); \
	(anIterNext) = (anIter) = BO_SEQLIST_Begin(_TListValueType,___seql_foreach2_list); \
	(anIterNext)++; \
	for (___seql_foreach2_count = _BO_SEQLIST_Size(___seql_foreach2_list) - 1; \
			___seql_foreach2_count--; \
				(anIter) = (anIterNext)++) {

/* ------------------------------------------------------------------------- */

/*
	bo_seqlist list;
	...
	// iterator *_$1;
	// iterator *_$2; // next
	BO_SEQLIST_FOREACH2_< _TListValueType >(&list)
		...
	FOREACH_END
 */
# define BO_SEQLIST_FOREACH2_(_TListValueType,aList) { \
	_TListValueType *_$1; \
	_TListValueType *_$2; \
	size_t ___seql_foreach2__count; \
	struct bo_seqlist *___seql_foreach2__list = bo_seqlist_p(aList); \
	_$2 = _$1 = BO_SEQLIST_Begin(_TListValueType,___seql_foreach2__list); \
	_$2++; \
	for (___seql_foreach2__count = _BO_SEQLIST_Size(___seql_foreach2__list) - 1; \
			___seql_foreach2__count--; \
				_$1 = _$2++) {

# ifndef FOREACH_END
#  define FOREACH_END }}
# endif
/* @} */

#endif /* !__BO_FUNDIS_SEQLIST__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
