/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Srtbtree.h	(V.Sokur)
 * include/Bo/fundis/Srtbtree.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FUNDIS_SRTBTREE__
# define __BO_FUNDIS_SRTBTREE__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/fundis/Rtbtree.h>
# include <Bo/fundis/Cdllist.h>

/******************************************************************************
 * %% BeginSection: srtbtree declarations
 */
# pragma mark -
# pragma mark [ srtbtree declarations ]
# pragma mark -

/* forward declatations */
struct bo_srtbtree;
struct bo_srtbtree_node;

/* super right-threaded b-tree proper */
struct __MYC_QALIGN(SIZEOF_PTR) bo_srtbtree {
	struct {
		struct bo_srtbtree_node   *btn_link[2];
		unsigned                   btn_lbackref :1;
		unsigned                   btn_rbackref :1;
		unsigned                   btn_first :1;
		unsigned                   btn_last :1;
		unsigned                   btn_last_defo :1;
		signed                     btn_balance :3;
		unsigned                   btn_pad :24;
	}                         bt_head;
	struct bo_srtbtree_node  *bt_first,
	                         *bt_last;
	bo_cdllist_t              bt_nodelist;
};

# define bo_srtbtree_p(aTree) ((struct bo_srtbtree *)(aTree))

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_srtbtree,
	sizeof(struct bo_srtbtree_s) == sizeof(struct bo_srtbtree));
# endif /* _DEBUG */
/* @} */

/* ============================================================================
 * NODE
 */
# pragma mark -
# pragma mark [ node ]
# pragma mark -

struct __MYC_QALIGN(SIZEOF_PTR) bo_srtbtree_node {
	struct bo_srtbtree_node   *btn_link[2];
# ifdef _MSC_VER
	unsigned short             btn_lbackref :1;
	unsigned short             btn_rbackref :1;
	unsigned short             btn_first :1;
	unsigned short             btn_last :1;
	unsigned short             btn_last_defo :1;
	signed short               btn_balance :3;
	unsigned short             btn_pad :8;
# else
	unsigned                   btn_lbackref :1;
	unsigned                   btn_rbackref :1;
	unsigned                   btn_first :1;
	unsigned                   btn_last :1;
	unsigned                   btn_last_defo :1;
	signed                     btn_balance :3;
	unsigned                   btn_pad :8;
# endif
	u_int16_t                  btn_ubits;
	bo_pointer_t               btn_info;

	/* NOT FOR YOUR BUSINESS @{ */
	bo_cdllelem_t              btn_lelem;
	/* @} */
};

# define bo_srtbtree_node_p(aNode) ((struct bo_srtbtree_node *)(aNode))

# define bo_srtbtree_node_of(_Tfield,anObject) \
	bo_srtbtree_node_p(__myc_membaddrof(_Tfield,anObject)); \

/* NOT FOR YOUR BUSINESS @{ */
# ifdef _DEBUG
#  include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_srtbtree_node,
	sizeof(struct bo_srtbtelem_s) == sizeof(struct bo_srtbtree_node));
# endif /* _DEBUG */
/* @} */

/* ----------------------------------------------------------------------------
 * NODE PROPERTIES
 */

# define BO_SRTBTREE_NODE_UserBits(aNode) BO_RTBTREE_NODE_UserBits(aNode)

/* bo_pointer_t
   BO_SRTBTREE_NODE_UserInfo(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_UserInfo(aNode) BO_RTBTREE_NODE_UserInfo(aNode)

/* bo_srtbtree_node *
   BO_SRTBTREE_NODE_Link(bo_srtbtree_node *aNode, bo_btree_dir_t aDirection); */
# define BO_SRTBTREE_NODE_Link(aNode,aDir) BO_RTBTREE_NODE_Link(aNode,aDir)

/* bo_srtbtree_node * (=NULL)
   BO_SRTBTREE_NODE_XLink(bo_srtbtree_node *aNode, bo_btree_dir_t aDirection); */
# define BO_SRTBTREE_NODE_XLink(aNode,aDir) BO_RTBTREE_NODE_XLink(aNode,aDir)

/* bo_srtbtree_node *
   BO_SRTBTREE_NODE_BLink(bo_srtbtree_node *aNode, int aBalance); */
# define BO_SRTBTREE_NODE_BLink(aNode,aBalance) BO_RTBTREE_NODE_BLink(aNode,aBalance)

/* bo_srtbtree_node *
   BO_SRTBTREE_NODE_LLink(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_LLink(aNode) BO_RTBTREE_NODE_LLink(aNode)

/* bo_srtbtree_node *
   BO_SRTBTREE_NODE_RLink(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_RLink(aNode) BO_RTBTREE_NODE_RLink(aNode)

/* bool
   BO_SRTBTREE_NODE_IsLast(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_IsLast(aNode) BO_RTBTREE_NODE_IsLast(aNode)

/* bool
   BO_SRTBTREE_NODE_IsLast_defo(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_IsLast_defo(aNode) BO_RTBTREE_NODE_IsLast_defo(aNode)

/* bool
   BO_SRTBTREE_NODE_IsBackref(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_IsBackref(aNode) BO_RTBTREE_NODE_IsBackref(aNode)

/* bool
   BO_SRTBTREE_NODE_IsUnbalanced(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_IsUnbalanced(aNode) BO_RTBTREE_NODE_IsUnbalanced(aNode)

/* bool
   BO_SRTBTREE_NODE_IsLeaf(bo_srtbtree_node *aNode); */
# define BO_SRTBTREE_NODE_IsLeaf(aNode) BO_RTBTREE_NODE_IsLeaf(aNode)

/* ============================================================================
 * LPR ITERATOR
 */
# pragma mark -
# pragma mark [ lpr iterator ]
# pragma mark -

# ifndef __cplusplus
typedef struct bo_srtbtree_node       *bo_srtbtree_iterator;
typedef struct bo_srtbtree_node const *bo_srtbtree_const_iterator;
# else
typedef bo_srtbtree_node              *bo_srtbtree_iterator;
typedef bo_srtbtree_node const        *bo_srtbtree_const_iterator;
# endif
typedef bo_srtbtree_iterator        bo_srtbtree_lpr_iterator;
typedef bo_srtbtree_const_iterator  bo_srtbtree_const_lpr_iterator;

/* ------------------------------------------------------------------------- */

/* bo_pointer_t
   _BO_SRTBTREE_ITER_Deref(bo_srtbtree_iterator anIter); */
# define _BO_SRTBTREE_ITER_Deref(anIter) _BO_RTBTREE_ITER_Deref(anIter)

/* _Tp
   BO_SRTBTREE_ITER_Deref_< _Tp >(bo_srtbtree_iterator anIter); */
# define BO_SRTBTREE_ITER_Deref(_Tp,anIter) BO_RTBTREE_ITER_Deref(_Tp,anIter)

/* ------------------------------------------------------------------------- */

/* void
   _BO_SRTBTREE_ITER_Inc_lpr(bo_srtbtree_iterator &anIter); */
# define _BO_SRTBTREE_ITER_Inc_lpr(anIter) _BO_RTBTREE_ITER_Inc_lpr(anIter)

/* void
   __BO_SRTBTREE_ITER_Inc_lpr(bo_srtbtree_iterator &anIter); */
# define __BO_SRTBTREE_ITER_Inc_lpr(anIter) __BO_RTBTREE_ITER_Inc_lpr(anIter)

/* void
   _BO_SRTBTREE_ITER_Inc_defo(bo_srtbtree_iterator &anIter); */
# define _BO_SRTBTREE_ITER_Inc_defo(anIter) \
	do { \
		if (BO_RTBTREE_NODE_IsLast_defo(anIter)) { \
			(anIter) = NULL; \
			break; \
		} \
		__BO_SRTBTREE_ITER_Inc_defo(anIter); \
	} while (0)

/* void
   __BO_SRTBTREE_ITER_Inc_defo(bo_srtbtree_iterator &anIter); */
# define __BO_SRTBTREE_ITER_Inc_defo(anIter) \
	BO_CDLLIST_ITER_Inc(struct bo_srtbtree_node,btn_lelem,anIter)

/* ------------------------------------------------------------------------- */

/* bo_srtbtree_iterator (=NULL)
   _BO_SRTBTREE_ITER_Next_lpr(bo_srtbtree_iterator anIter); */
# define _BO_SRTBTREE_ITER_Next_lpr(anIter) \
	((bo_srtbtree_iterator)_BO_RTBTREE_ITER_Next_lpr(anIter))

/* bo_srtbtree_iterator
   __BO_SRTBTREE_ITER_Next_lpr(bo_srtbtree_iterator anIter); */
# define __BO_SRTBTREE_ITER_Next_lpr(anIter) \
	((bo_srtbtree_iterator)__BO_RTBTREE_ITER_Next_lpr(anIter))

/* ============================================================================
 * PLR ITERATOR
 */
# pragma mark -
# pragma mark [ plr iterator ]
# pragma mark -

typedef bo_rtbtree_plr_iterator        bo_srtbtree_plr_iterator;
typedef bo_rtbtree_const_plr_iterator  bo_srtbtree_const_plr_iterator;

/* ------------------------------------------------------------------------- */

/* bo_pointer_t
   _BO_SRTBTREE_PLR_ITER_Deref(bo_srtbtree_plr_iterator anIter); */
# define _BO_SRTBTREE_PLR_ITER_Deref(anIter) _BO_RTBTREE_PLR_ITER_Deref(anIter)

/* _Tp
   BO_SRTBTREE_PLR_ITER_Deref< _Tp >(bo_srtbtree_plr_iterator anIter); */
# define BO_SRTBTREE_PLR_ITER_Deref(_Tp,anIter) BO_RTBTREE_PLR_ITER_Deref(_Tp,anIter)

/* ------------------------------------------------------------------------- */

/* void
   _BO_SRTBTREE_ITER_Inc_plr(bo_srtbtree_plr_iterator &anIter); */
# define _BO_SRTBTREE_ITER_Inc_plr(anIter) _BO_RTBTREE_ITER_Inc_plr(anIter)

/* void
   __BO_SRTBTREE_ITER_Inc_plr(bo_srtbtree_plr_iterator &anIter); */
# define __BO_SRTBTREE_ITER_Inc_plr(anIter) __BO_RTBTREE_ITER_Inc_plr(anIter)

/* ------------------------------------------------------------------------- */

/* bo_srtbtree_plr_iterator (=NULL)
   _BO_SRTBTREE_ITER_Next_plr(bo_srtbtree_plr_iterator anIter); */
# define _BO_SRTBTREE_ITER_Next_plr(anIter) _BO_RTBTREE_ITER_Next_plr(anIter)

/* bo_srtbtree_plr_iterator
   __BO_SRTBTREE_ITER_Next_plr(bo_srtbtree_plr_iterator anIter); */
# define __BO_SRTBTREE_ITER_Next_plr(anIter) __BO_RTBTREE_ITER_Next_plr(anIter)

/* ============================================================================
 * CONSTRUCT/DESTROY
 */
# pragma mark -
# pragma mark [ construct/destroy ]
# pragma mark -

/* void
   _BO_SRTBTREE_Init(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Init(aTree) \
	do { \
		struct bo_srtbtree *___srtbt_init_list = bo_srtbtree_p(aTree); \
		___srtbt_init_list->bt_head.btn_link[0] = NULL; \
		___srtbt_init_list->bt_head.btn_link[1] = NULL; \
		__bo_btree_p(___srtbt_init_list)->bt_head.btn_internal = 0; \
		___srtbt_init_list->bt_first = ___srtbt_init_list->bt_last = NULL; \
		/* @todo: do we need this? */ \
		BO_SRTBTREE_NODE_RLink(BO_SRTBTREE_Head_(___srtbt_init_list)) = \
			BO_SRTBTREE_Head_(___srtbt_init_list); \
		_BO_CDLLIST_Init(&___srtbt_init_list->bt_nodelist); \
	} while (0)

/* ============================================================================
 * PROPERTIES: ACCESS
 */
# pragma mark -
# pragma mark [ properties: access ]
# pragma mark -

/* bo_srtbtree::unnamed_node *
   _BO_SRTBTREE_Head(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Head(aTree) _BO_RTBTREE_Head(aTree)

/* bo_srtbtree_node *
   BO_SRTBTREE_Head_(bo_srtbtree *aTree); */
# define BO_SRTBTREE_Head_(aTree) BO_RTBTREE_Head_(aTree)

/* bo_srtbtree::unnamed_node *
   _BO_SRTBTREE_Tail(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Tail(aTree) _BO_RTBTREE_Tail(aTree)

/* bo_srtbtree_node *
   BO_SRTBTREE_Tail_(bo_srtbtree *aTree); */
# define BO_SRTBTREE_Tail_(aTree) BO_RTBTREE_Tail_(aTree)

/* ------------------------------------------------------------------------- */

/* size_t
   _BO_SRTBTREE_Size(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Size(aTree) _BO_CDLLIST_Size(&bo_srtbtree_p(aTree)->bt_nodelist)

/* bool
   _BO_SRTBTREE_Empty(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Empty(aTree) (0 == _BO_SRTBTREE_Size(aTree))

/* ============================================================================
 * METHODS: ACCESS
 */
# pragma mark -
# pragma mark [ methods: access ]
# pragma mark -

/* bo_srtbtree_iterator (=NULL)
   _BO_SRTBTREE_Begin(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Begin(aTree) _BO_SRTBTREE_Begin_lpr(aTree)

/* bo_srtbtree_iterator (=NULL)
   _BO_SRTBTREE_Begin_lpr(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Begin_lpr(aTree) _BO_SRTBTREE_First_lpr(aTree)

/* bo_srtbtree_plr_iterator (=NULL)
   _BO_SRTBTREE_Begin_plr(bo_srtbtree *aTree, struct bo_btree_plr_iterator_s anIterBuffer); */
#define _BO_SRTBTREE_Begin_plr(aTree, anIterBuffer) _BO_RTBTREE_Begin_plr(aTree, anIterBuffer)

/* bo_srtbtree_iterator (=NULL)
   _BO_SRTBTREE_Begin_defo(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Begin_defo(aTree) _BO_SRTBTREE_First_defo(aTree)

/* ------------------------------------------------------------------------- */

/* bo_srtbtree_node * (=NULL)
   _BO_SRTBTREE_First_lpr(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_First_lpr(aTree) (bo_srtbtree_p(aTree)->bt_first)

/* bo_srtbtree_node * (=NULL)
   _BO_SRTBTREE_Last_lpr(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Last_lpr(aTree) (bo_srtbtree_p(aTree)->bt_last)

/* ------------------------------------------------------------------------- */

/* bo_srtbtree_node * (=NULL)
   _BO_SRTBTREE_First_defo(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_First_defo(aTree) \
	BO_CDLLIST_Front(struct bo_srtbtree_node,btn_lelem,&bo_srtbtree_p(aTree)->bt_nodelist)

/* bo_srtbtree_node * (=NULL)
   _BO_SRTBTREE_Last_defo(bo_srtbtree *aTree); */
# define _BO_SRTBTREE_Last_defo(aTree) \
	BO_CDLLIST_Back(struct bo_srtbtree_node,btn_lelem,&bo_srtbtree_p(aTree)->bt_nodelist)

/* ============================================================================
 * METHODS: ALGORITHMS
 */
# pragma mark -
# pragma mark [ methods: algorithms ]
# pragma mark -

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_InsertNode
 */
# pragma mark -

/* ()
   BO_SRTBTREE_InsertNode< _Tp, _Tfield >(
		_Tp *anIter,
		_Tp *anObject,
		bo_srtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)}
	); */
# define BO_SRTBTREE_InsertNode(_Tp,_Tfield, \
		anIter,anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER) \
	do { \
		BO_RTBTREE_LinkNode( \
			_Tp,_Tfield, \
			anIter,anObject,bo_rtbtree_p(aTree),Force, \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_EXIST,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_SUCCESS,__LINE__): \
		{ \
			struct bo_srtbtree_node *___srtbt_insn_node; \
			bo_cdllist_t *___srtbt_insn_ilist; \
 \
			if (NULL != (___srtbt_insn_node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_insn_node->btn_last_defo = 0; \
 \
			___srtbt_insn_node = bo_srtbtree_node_of(_Tfield,anObject); \
			___srtbt_insn_node->btn_last_defo = 1; \
			___srtbt_insn_ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_PushBack(struct bo_srtbtree_node,btn_lelem,___srtbt_insn_ilist,___srtbt_insn_node); \
 \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_InsertNode_
 */
# pragma mark -

/* ()
   BO_SRTBTREE_InsertNode_< _Tp, _Tfield >(
		_Tp *anObject,
		bo_rtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)}
	); */
# define BO_SRTBTREE_InsertNode_(_Tp,_Tfield, \
		anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER) \
	do { \
		BO_RTBTREE_LinkNode_( \
			_Tp,_Tfield, \
			anObject,bo_rtbtree_p(aTree),Force, \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_EXIST,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_SUCCESS,__LINE__): \
		{ \
			struct bo_srtbtree_node *___srtbt_insn__node; \
			bo_cdllist_t *___srtbt_insn__ilist; \
 \
			if (NULL != (___srtbt_insn__node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_insn__node->btn_last_defo = 0; \
 \
			___srtbt_insn__node = bo_srtbtree_node_of(_Tfield,anObject); \
			___srtbt_insn__node->btn_last_defo = 1; \
			___srtbt_insn__ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_PushBack(struct bo_srtbtree_node,btn_lelem,___srtbt_insn__ilist,___srtbt_insn__node); \
 \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_InsertNode_b
 */
# pragma mark -

/* ()
   BO_SRTBTREE_InsertNode_b< _Tp, _Tfield >(
		_Tp *anIter,
		_Tp *anObject,
		bo_srtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)},
		{(_HIMAXHANDLER)}
	); */
# define BO_SRTBTREE_InsertNode_b(_Tp,_Tfield, \
		anIter,anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER,_HIMAXHANDLER) \
	do { \
		BO_RTBTREE_LinkNode_b( \
			_Tp,_Tfield, \
			anIter,anObject,bo_rtbtree_p(aTree),Force, \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_b_EXIST,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_b_HIMAX,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_b_SUCCESS,__LINE__): \
		{ \
			struct bo_srtbtree_node *___srtbt_insnb_node; \
			bo_cdllist_t *___srtbt_insnb_ilist; \
 \
			if (NULL != (___srtbt_insnb_node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_insnb_node->btn_last_defo = 0; \
 \
			___srtbt_insnb_node = bo_srtbtree_node_of(_Tfield,anObject); \
			___srtbt_insnb_node->btn_last_defo = 1; \
			___srtbt_insnb_ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_PushBack(struct bo_srtbtree_node,btn_lelem,___srtbt_insnb_ilist,___srtbt_insnb_node); \
 \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_b_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_b_HIMAX,__LINE__): \
		_HIMAXHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_InsertNode_b_
 */
# pragma mark -

/* ()
   BO_SRTBTREE_InsertNode_b_< _Tp, _Tfield >(
		_Tp *anObject,
		bo_srtbtree *aTree,
		int{/bool} Force,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER)},
		{(_EXISTHANDLER)},
		{(_HIMAXHANDLER)}
	); */
# define BO_SRTBTREE_InsertNode_b_(_Tp,_Tfield, \
		anObject,aTree,Force, \
		_COMPARATOR,_SUCCESSHANDLER,_EXISTHANDLER,_HIMAXHANDLER) \
	do { \
		BO_RTBTREE_LinkNode_b_( \
			_Tp,_Tfield, \
			anObject,bo_rtbtree_p(aTree),Force, \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_b_EXIST,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_InsertNode_b_HIMAX,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_b_SUCCESS,__LINE__): \
		{ \
			struct bo_srtbtree_node *___srtbt_insnb__node; \
			bo_cdllist_t *___srtbt_insnb__ilist; \
 \
			if (NULL != (___srtbt_insnb__node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_insnb__node->btn_last_defo = 0; \
 \
			___srtbt_insnb__node = bo_srtbtree_node_of(_Tfield,anObject); \
			___srtbt_insnb__node->btn_last_defo = 1; \
			___srtbt_insnb__ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_PushBack(struct bo_srtbtree_node,btn_lelem,___srtbt_insnb__ilist,___srtbt_insnb__node); \
 \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_b_EXIST,__LINE__): \
		_EXISTHANDLER; \
		break; \
__myc_xconcat(L_BO_SRTBTREE_InsertNode_b_HIMAX,__LINE__): \
		_HIMAXHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_RemoveNode
 */
# pragma mark -

/* ()
   BO_SRTBTREE_RemoveNode< _Tp, _Tfield >(
		_Tp *anIter,
		bo_srtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER: _Tp *anIter)},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_SRTBTREE_RemoveNode(_Tp,_Tfield, \
		anIter,aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		BO_RTBTREE_UnlinkNode( \
			_Tp,_Tfield, \
			anIter,bo_rtbtree_p(aTree), \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_SUCCESS,__LINE__): \
		{ \
			struct bo_srtbtree_node *___srtbt_rmn_node; \
			bo_cdllist_t *___srtbt_rmn_ilist; \
 \
			___srtbt_rmn_node = bo_srtbtree_node_of(_Tfield,anIter); \
			___srtbt_rmn_node->btn_last_defo = 0; \
			___srtbt_rmn_ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_Remove(struct bo_srtbtree_node,btn_lelem,___srtbt_rmn_ilist,___srtbt_rmn_node); \
 \
			if (NULL != (___srtbt_rmn_node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_rmn_node->btn_last_defo = 1; \
 \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_RemoveNode_
 */
# pragma mark -

/* ()
   BO_SRTBTREE_RemoveNode_< _Tp, _Tfield >(
		bo_srtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER: (_Tp *_$1))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_SRTBTREE_RemoveNode_(_Tp,_Tfield, \
		aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		_Tp *___srtbt_rmn__it; \
		BO_RTBTREE_UnlinkNode_( \
			_Tp,_Tfield, \
			bo_rtbtree_p(aTree), \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ ___srtbt_rmn__it = _$1; goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_SUCCESS,__LINE__): \
		{ \
			_Tp *_$1; \
			struct bo_srtbtree_node *___srtbt_rmn__node; \
			bo_cdllist_t *___srtbt_rmn__ilist; \
 \
			___srtbt_rmn__node = bo_srtbtree_node_of(_Tfield,___srtbt_rmn__it); \
			___srtbt_rmn__node->btn_last_defo = 0; \
			___srtbt_rmn__ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_Remove(struct bo_srtbtree_node,btn_lelem,___srtbt_rmn__ilist,___srtbt_rmn__node); \
 \
			if (NULL != (___srtbt_rmn__node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_rmn__node->btn_last_defo = 1; \
 \
			_$1 = ___srtbt_rmn__it; \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_RemoveNode_b
 */
# pragma mark -

/* ()
   BO_SRTBTREE_RemoveNode_b< _Tp, _Tfield >(
		_Tp *anIter,
		bo_srtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), _Tp *anIter)},
		{(_SUCCESSHANDLER: _Tp *anIter)},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_SRTBTREE_RemoveNode_b(_Tp,_Tfield, \
		anIter,aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		BO_RTBTREE_UnlinkNode_b( \
			_Tp,_Tfield, \
			anIter,bo_rtbtree_p(aTree), \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_SUCCESS,__LINE__): \
		{ \
			struct bo_srtbtree_node *___srtbt_rmnb_node; \
			bo_cdllist_t *___srtbt_rmnb_ilist; \
 \
			___srtbt_rmnb_node = bo_srtbtree_node_of(_Tfield,anIter); \
			___srtbt_rmnb_node->btn_last_defo = 0; \
			___srtbt_rmnb_ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_Remove(struct bo_srtbtree_node,btn_lelem,___srtbt_rmnb_ilist,___srtbt_rmnb_node); \
 \
			if (NULL != (___srtbt_rmnb_node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_rmnb_node->btn_last_defo = 1; \
 \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ----------------------------------------------------------------------------
 * BO_SRTBTREE_RemoveNode_b_
 */
# pragma mark -

/* ()
   BO_SRTBTREE_RemoveNode_b_< _Tp, _Tfield >(
		bo_srtbtree *aTree,
		{(_COMPARATOR: (bo_mismatch_t _$0), (_Tp *_$1))},
		{(_SUCCESSHANDLER: (_Tp *_$1))},
		{(_NOTFOUNDHANDLER)}
	); */
# define BO_SRTBTREE_RemoveNode_b_(_Tp,_Tfield, \
		aTree, \
		_COMPARATOR,_SUCCESSHANDLER,_NOTFOUNDHANDLER) \
	do { \
		_Tp *___srtbt_rmnb__it; \
		BO_RTBTREE_UnlinkNode_b_( \
			_Tp,_Tfield, \
			bo_rtbtree_p(aTree), \
			(struct bo_btree_node **)&(aTree)->bt_first, \
			(struct bo_btree_node **)&(aTree)->bt_last, \
			_COMPARATOR, \
			{ ___srtbt_rmnb__it = _$1; goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_SUCCESS,__LINE__); }, \
			{ goto __myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_NOTFOUND,__LINE__); } \
		); \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_SUCCESS,__LINE__): \
		{ \
			_Tp *_$1; \
			struct bo_srtbtree_node *___srtbt_rmnb__node; \
			bo_cdllist_t *___srtbt_rmnb__ilist; \
 \
			___srtbt_rmnb__node = bo_srtbtree_node_of(_Tfield,___srtbt_rmnb__it); \
			___srtbt_rmnb__node->btn_last_defo = 0; \
			___srtbt_rmnb__ilist = &(aTree)->bt_nodelist; \
			BO_CDLLIST_Remove(struct bo_srtbtree_node,btn_lelem,___srtbt_rmnb__ilist,___srtbt_rmnb__node); \
 \
			if (NULL != (___srtbt_rmnb__node = _BO_SRTBTREE_Last_defo(aTree))) \
				___srtbt_rmnb__node->btn_last_defo = 1; \
 \
			_$1 = ___srtbt_rmnb__it; \
			_SUCCESSHANDLER; \
			break; \
		} \
__myc_xconcat(L_BO_SRTBTREE_RemoveNode_b_NOTFOUND,__LINE__): \
		_NOTFOUNDHANDLER; \
		break; \
	} while (0)

/* ============================================================================
 * LOOP HELPERS
 * @{
 */
# pragma mark -
# pragma mark [ loop helpers ]
# pragma mark -

/*
	bo_srtbtree tree;
	bo_srtbtree_iterator iter;
	...
	_BO_SRTBTREE_FOREACH_DEFO(iter, &tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_FOREACH_DEFO(anIter,aTree) \
	BO_CDLLIST_FOREACH(struct bo_srtbtree_node,btn_lelem,anIter,&bo_srtbtree_p(aTree)->bt_nodelist)

/* ------------------------------------------------------------------------- */

/*
	bo_srtbtree tree;
	...
	// iterator _$1;
	_BO_SRTBTREE_FOREACH_DEFO_(&tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_FOREACH_DEFO_(aTree) \
	BO_CDLLIST_FOREACH_(struct bo_srtbtree_node,btn_lelem,&bo_srtbtree_p(aTree)->bt_nodelist)

/* ------------------------------------------------------------------------- */

/*
	bo_srtbtree tree;
	bo_srtbtree_iterator iter;
	...
	_BO_SRTBTREE_FOREACH_LPR(iter, &tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_FOREACH_LPR(anIter,aTree) { \
	size_t ___srtbt_foreach_lpr_count; \
	(anIter) = _BO_SRTBTREE_Begin_lpr(aTree); \
	for (___srtbt_foreach_lpr_count = _BO_SRTBTREE_Size(aTree); \
			___srtbt_foreach_lpr_count--; \
				(anIter) = __BO_SRTBTREE_ITER_Next_lpr(anIter)) {

/*
	bo_srtbtree tree;
	bo_srtbtree_iterator iter;
	...
	_BO_SRTBTREE_SAFE_FOREACH_LPR(iter, &tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_SAFE_FOREACH_LPR(anIter,aTree) { \
	bo_srtbtree_iterator ___srtbt_safe_foreach_lpr_it; \
	size_t ___srtbt_safe_foreach_lpr_count; \
	(anIter) = _BO_SRTBTREE_Begin_lpr(aTree); \
	for (___srtbt_safe_foreach_lpr_count = _BO_SRTBTREE_Size(aTree); \
			(___srtbt_safe_foreach_lpr_count-- \
					? (___srtbt_safe_foreach_lpr_it = __BO_SRTBTREE_ITER_Next_lpr(anIter)) \
					: NULL); \
				(anIter) = ___srtbt_safe_foreach_lpr_it) {

/* ------------------------------------------------------------------------- */

/*
	bo_srtbtree tree;
	...
	// iterator _$1;
	_BO_SRTBTREE_FOREACH_LPR_(&tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_FOREACH_LPR_(aTree) { \
	bo_srtbtree_iterator _$1; \
	size_t ___srtbt_foreach_lpr__count; \
	_$1 = _BO_SRTBTREE_Begin_lpr(aTree); \
	for (___srtbt_foreach_lpr__count = _BO_SRTBTREE_Size(aTree); \
			___srtbt_foreach_lpr__count--; \
				_$1 = __BO_SRTBTREE_ITER_Next_lpr(_$1)) {

/*
	bo_srtbtree tree;
	...
	// iterator _$1;
	_BO_SRTBTREE_SAFE_FOREACH_LPR_(&tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_SAFE_FOREACH_LPR_(aTree) { \
	bo_srtbtree_iterator _$1, ___srtbt_safe_foreach_lpr__it; \
	size_t ___srtbt_safe_foreach_lpr__count; \
	_$1 = _BO_SRTBTREE_Begin_lpr(aTree); \
	for (___srtbt_safe_foreach_lpr__count = _BO_SRTBTREE_Size(aTree); \
			(___srtbt_safe_foreach_lpr__count-- ? \
					(___srtbt_safe_foreach_lpr__it = __BO_SRTBTREE_ITER_Next_lpr(_$1)) \
					: NULL); \
				_$1 = ___srtbt_safe_foreach_lpr__it) {

/* ------------------------------------------------------------------------- */

/*
	bo_srtbtree tree;
	bo_srtbtree_plr_iterator iter;
	...
	_BO_SRTBTREE_FOREACH_PLR(iter, &tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_FOREACH_PLR(anIter,aTree) _BO_RTBTREE_FOREACH_PLR(anIter,aTree)

/* ------------------------------------------------------------------------- */

/*
	bo_srtbtree tree;
	...
	// iterator _$1;
	_BO_SRTBTREE_FOREACH_PLR_(&tree)
		...
	FOREACH_END
 */
# define _BO_SRTBTREE_FOREACH_PLR_(aTree) _BO_RTBTREE_FOREACH_PLR_(aTree)

/* ------------------------------------------------------------------------- */

# ifndef FOREACH_END
#  define FOREACH_END }}
# endif
/* @} */

#endif /* !__BO_FUNDIS_SRTBTREE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
