/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Fmtio.h	(V.Sokur)
 * include/Bo/libc/Fmtio.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_FMTIO__
# define __BO_LIBC_FMTIO__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Cdefs.h>

# include _BO_COMPILER_HEADER(_Fmtio.h)

/******************************************************************************
 * %% BeginSection: format specifiers definitions
 */
# pragma mark -

# ifndef PRIc
#  define PRIc "c" /* char */
# endif
# ifndef PRId
#  define PRId "d" /* int */
# endif
# ifndef PRIi
#  define PRIi "i" /* int */
# endif
# ifndef PRIo
#  define PRIo "o" /* unsigned int, octal */
# endif
# ifndef PRIu
#  define PRIu "u" /* unsigned int */
# endif
# ifndef PRIx
#  define PRIx "x" /* unsigned int, hexadecimal */
# endif
# ifndef PRIX
#  define PRIX "X" /* unsigned int, hexadecimal */
# endif
# ifndef PRIe
#  define PRIe "e" /* double of the form: [ñ] d.dddd e [+/-] dd[d] */
# endif
# ifndef PRIE
#  define PRIE "E" /* double of the form: [ñ] d.dddd E [+/-] dd[d] */
# endif
# ifndef PRIf
#  define PRIf "f" /* double of the form: [ñ] dddd.dddd */
# endif
# ifndef PRIg
#  define PRIg "g" /* double */
# endif
# ifndef PRIG
#  define PRIG "G" /* double */
# endif
# ifndef PRIs
#  define PRIs "s" /* char* */
# endif
# ifndef PRIp
#  define PRIp PRIxPTR
# endif

/*
 * fprintf(3) macros for signed integers.
 */

/* decimal integers */
# ifndef PRId8
#  define PRId8 "d" /* int8_t */
# endif
# ifndef PRId16
#  define PRId16 "d" /* int16_t */
# endif
# ifndef PRId32
#  define PRId32 "d" /* int32_t */
# endif
# ifndef PRId64
#  define PRId64 "lld" /* int64_t */
# endif
# ifndef PRIdLEAST8
#  define PRIdLEAST8 "d" /* int_least8_t */
# endif
# ifndef PRIdLEAST16
#  define PRIdLEAST16 "d" /* int_least16_t */
# endif
# ifndef PRIdLEAST32
#  define PRIdLEAST32 "d" /* int_least32_t */
# endif
# ifndef PRIdLEAST64
#  define PRIdLEAST64 "lld" /* int_least64_t */
# endif
# ifndef PRIdFAST8
#  define PRIdFAST8 "d" /* int_fast8_t */
# endif
# ifndef PRIdFAST16
#  define PRIdFAST16 "d" /* int_fast16_t */
# endif
# ifndef PRIdFAST32
#  define PRIdFAST32 "d" /* int_fast32_t */
# endif
# ifndef PRIdFAST64
#  define PRIdFAST64 "lld" /* int_fast64_t */
# endif
# ifndef PRIdMAX
#  define PRIdMAX "jd" /* intmax_t */
# endif
# ifndef PRIdPTR
#  define PRIdPTR "d" /* intptr_t */
# endif

/* decimal integers */
# ifndef PRIi8
#  define PRIi8 "i" /* int8_t */
# endif
# ifndef PRIi16
#  define PRIi16 "i" /* int16_t */
# endif
# ifndef PRIi32
#  define PRIi32 "i" /* int32_t */
# endif
# ifndef PRIi64
#  define PRIi64 "lli" /* int64_t */
# endif
# ifndef PRIiLEAST8
#  define PRIiLEAST8 "i" /* int_least8_t  */
# endif
# ifndef PRIiLEAST16
#  define PRIiLEAST16 "i" /* int_least16_t */
# endif
# ifndef PRIiLEAST32
#  define PRIiLEAST32 "i" /* int_least32_t */
# endif
# ifndef PRIiLEAST64
#  define PRIiLEAST64 "lli" /* int_least64_t */
# endif
# ifndef PRIiFAST8
#  define PRIiFAST8 "i" /* int_fast8_t */
# endif
# ifndef PRIiFAST16
#  define PRIiFAST16 "i" /* int_fast16_t */
# endif
# ifndef PRIiFAST32
#  define PRIiFAST32 "i" /* int_fast32_t */
# endif
# ifndef PRIiFAST64
#  define PRIiFAST64 "lli" /* int_fast64_t */
# endif
# ifndef PRIiMAX
#  define PRIiMAX "ji" /* intmax_t */
# endif
# ifndef PRIiPTR
#  define PRIiPTR "i" /* intptr_t */
# endif

/*
 * fprintf(3) macros for unsigned integers.
 */

/* octal integers */
# ifndef PRIo8
#  define PRIo8 "o" /* uint8_t */
# endif
# ifndef PRIo16
#  define PRIo16 "o" /* uint16_t */
# endif
# ifndef PRIo32
#  define PRIo32 "o" /* uint32_t */
# endif
# ifndef PRIo64
#  define PRIo64 "llo" /* uint64_t */
# endif
# ifndef PRIoLEAST8
#  define PRIoLEAST8 "o" /* uint_least8_t */
# endif
# ifndef PRIoLEAST16
#  define PRIoLEAST16 "o" /* uint_least16_t */
# endif
# ifndef PRIoLEAST32
#  define PRIoLEAST32 "o" /* uint_least32_t */
# endif
# ifndef PRIoLEAST64
#  define PRIoLEAST64 "llo" /* uint_least64_t */
# endif
# ifndef PRIoFAST8
#  define PRIoFAST8 "o" /* uint_fast8_t */
# endif
# ifndef PRIoFAST16
#  define PRIoFAST16 "o" /* uint_fast16_t */
# endif
# ifndef PRIoFAST32
#  define PRIoFAST32 "o" /* uint_fast32_t */
# endif
# ifndef PRIoFAST64
#  define PRIoFAST64 "llo" /* uint_fast64_t */
# endif
# ifndef PRIoMAX
#  define PRIoMAX "jo" /* uintmax_t */
# endif
# ifndef PRIoPTR
#  define PRIoPTR "o" /* uintptr_t */
# endif

/* decimal integers */
# ifndef PRIu8
#  define PRIu8 "u" /* uint8_t */
# endif
# ifndef PRIu16
#  define PRIu16 "u" /* uint16_t */
# endif
# ifndef PRIu32
#  define PRIu32 "u" /* uint32_t */
# endif
# ifndef PRIu64
#  define PRIu64 "llu" /* uint64_t */
# endif
# ifndef PRIuLEAST8
#  define PRIuLEAST8 "u" /* uint_least8_t */
# endif
# ifndef PRIuLEAST16
#  define PRIuLEAST16 "u" /* uint_least16_t */
# endif
# ifndef PRIuLEAST32
#  define PRIuLEAST32 "u" /* uint_least32_t */
# endif
# ifndef PRIuLEAST64
#  define PRIuLEAST64 "llu" /* uint_least64_t */
# endif
# ifndef PRIuFAST8
#  define PRIuFAST8 "u" /* uint_fast8_t */
# endif
# ifndef PRIuFAST16
#  define PRIuFAST16 "u" /* uint_fast16_t */
# endif
# ifndef PRIuFAST32
#  define PRIuFAST32 "u" /* uint_fast32_t */
# endif
# ifndef PRIuFAST64
#  define PRIuFAST64 "llu" /* uint_fast64_t */
# endif
# ifndef PRIuMAX
#  define PRIuMAX "ju" /* uintmax_t */
# endif
# ifndef PRIuPTR
#  define PRIuPTR "u" /* uintptr_t */
# endif

/* hexadecimal integers */
# ifndef PRIx8
#  define PRIx8 "x" /* uint8_t */
# endif
# ifndef PRIx16
#  define PRIx16 "x" /* uint16_t */
# endif
# ifndef PRIx32
#  define PRIx32 "x" /* uint32_t */
# endif
# ifndef PRIx64
#  define PRIx64 "llx" /* uint64_t */
# endif
# ifndef PRIxLEAST8
#  define PRIxLEAST8 "x" /* uint_least8_t */
# endif
# ifndef PRIxLEAST16
#  define PRIxLEAST16 "x" /* uint_least16_t */
# endif
# ifndef PRIxLEAST32
#  define PRIxLEAST32 "x" /* uint_least32_t */
# endif
# ifndef PRIxLEAST64
#  define PRIxLEAST64 "llx" /* uint_least64_t */
# endif
# ifndef PRIxFAST8
#  define PRIxFAST8 "x" /* uint_fast8_t */
# endif
# ifndef PRIxFAST16
#  define PRIxFAST16 "x" /* uint_fast16_t */
# endif
# ifndef PRIxFAST32
#  define PRIxFAST32 "x" /* uint_fast32_t */
# endif
# ifndef PRIxFAST64
#  define PRIxFAST64 "llx" /* uint_fast64_t */
# endif
# ifndef PRIxMAX
#  define PRIxMAX "jx" /* uintmax_t */
# endif
# ifndef PRIxPTR
#  define PRIxPTR "x" /* uintptr_t */
# endif

# ifndef PRIX8
#  define PRIX8 "X" /* uint8_t */
# endif
# ifndef PRIX16
#  define PRIX16 "X" /* uint16_t */
# endif
# ifndef PRIX32
#  define PRIX32 "X" /* uint32_t */
# endif
# ifndef PRIX64
#  define PRIX64 "llX" /* uint64_t */
# endif
# ifndef PRIXLEAST8
#  define PRIXLEAST8 "X" /* uint_least8_t */
# endif
# ifndef PRIXLEAST16
#  define PRIXLEAST16 "X" /* uint_least16_t */
# endif
# ifndef PRIXLEAST32
#  define PRIXLEAST32 "X" /* uint_least32_t */
# endif
# ifndef PRIXLEAST64
#  define PRIXLEAST64 "llX" /* uint_least64_t */
# endif
# ifndef PRIXFAST8
#  define PRIXFAST8 "X" /* uint_fast8_t */
# endif
# ifndef PRIXFAST16
#  define PRIXFAST16 "X" /* uint_fast16_t */
# endif
# ifndef PRIXFAST32
#  define PRIXFAST32 "X" /* uint_fast32_t */
# endif
# ifndef PRIXFAST64
#  define PRIXFAST64 "llX" /* uint_fast64_t */
# endif
# ifndef PRIXMAX
#  define PRIXMAX "jX" /* uintmax_t */
# endif
# ifndef PRIXPTR
#  define PRIXPTR "X" /* uintptr_t */
# endif

# pragma mark -

# ifndef SCNc
#  define SCNc "c" /* char* */
# endif
# ifndef SCNd
#  define SCNd "d" /* int* */
# endif
# ifndef SCNi
#  define SCNi "i" /* int* */
# endif
# ifndef SCNo
#  define SCNo "o" /* unsigned int*, octal */
# endif
# ifndef SCNu
#  define SCNu "u" /* unsigned int* */
# endif
# ifndef SCNx
#  define SCNx "x" /* unsigned int*, hexadecimal */
# endif
# ifndef SCNX
#  define SCNX "X" /* unsigned int*, hexadecimal */
# endif
# ifndef SCNe
#  define SCNe "e" /* float* */
# endif
# ifndef SCNE
#  define SCNE "E" /* float* */
# endif
# ifndef SCNf
#  define SCNf "f" /* float* */
# endif
# ifndef SCNg
#  define SCNg "g" /* float* */
# endif
# ifndef SCNG
#  define SCNG "G" /* float* */
# endif
# ifndef SCNs
#  define SCNs "s" /* char* */
# endif

/*
 * fscanf(3) macros for signed integers.
 */

# ifndef SCNd8
#  define SCNd8 "hhd" /* int8_t* */
# endif
# ifndef SCNd16
#  define SCNd16 "hd" /* int16_t* */
# endif
# ifndef SCNd32
#  define SCNd32 "d" /* int32_t* */
# endif
# ifndef SCNd64
#  define SCNd64 "lld" /* int64_t* */
# endif
# ifndef SCNdLEAST8
#  define SCNdLEAST8 "hhd" /* int_least8_t* */
# endif
# ifndef SCNdLEAST16
#  define SCNdLEAST16 "hd" /* int_least16_t* */
# endif
# ifndef SCNdLEAST32
#  define SCNdLEAST32 "d" /* int_least32_t* */
# endif
# ifndef SCNdLEAST64
#  define SCNdLEAST64 "lld" /* int_least64_t* */
# endif
# ifndef SCNdFAST8
#  define SCNdFAST8 "d" /* int_fast8_t* */
# endif
# ifndef SCNdFAST16
#  define SCNdFAST16 "d" /* int_fast16_t* */
# endif
# ifndef SCNdFAST32
#  define SCNdFAST32 "d" /* int_fast32_t* */
# endif
# ifndef SCNdFAST64
#  define SCNdFAST64 "lld" /* int_fast64_t* */
# endif
# ifndef SCNdMAX
#  define SCNdMAX "jd" /* intmax_t* */
# endif
# ifndef SCNdPTR
#  define SCNdPTR "d" /* intptr_t* */
# endif

# ifndef SCNi8
#  define SCNi8 "hhi" /* int8_t* */
# endif
# ifndef SCNi16
#  define SCNi16 "hi" /* int16_t* */
# endif
# ifndef SCNi32
#  define SCNi32 "i" /* int32_t* */
# endif
# ifndef SCNi64
#  define SCNi64 "lli" /* int64_t* */
# endif
# ifndef SCNiLEAST8
#  define SCNiLEAST8 "hhi" /* int_least8_t* */
# endif
# ifndef SCNiLEAST16
#  define SCNiLEAST16 "hi" /* int_least16_t* */
# endif
# ifndef SCNiLEAST32
#  define SCNiLEAST32 "i" /* int_least32_t* */
# endif
# ifndef SCNiLEAST64
#  define SCNiLEAST64 "lli" /* int_least64_t* */
# endif
# ifndef SCNiFAST8
#  define SCNiFAST8 "i" /* int_fast8_t* */
# endif
# ifndef SCNiFAST16
#  define SCNiFAST16 "i" /* int_fast16_t* */
# endif
# ifndef SCNiFAST32
#  define SCNiFAST32 "i" /* int_fast32_t* */
# endif
# ifndef SCNiFAST64
#  define SCNiFAST64 "lli" /* int_fast64_t* */
# endif
# ifndef SCNiMAX
#  define SCNiMAX "ji" /* intmax_t* */
# endif
# ifndef SCNiPTR
#  define SCNiPTR "i" /* intptr_t* */
# endif

/*
 * fscanf(3) macros for unsigned integers.
 */

# ifndef SCNo8
#  define SCNo8 "hho" /* uint8_t* */
# endif
# ifndef SCNo16
#  define SCNo16 "ho" /* uint16_t* */
# endif
# ifndef SCNo32
#  define SCNo32 "o" /* uint32_t* */
# endif
# ifndef SCNo64
#  define SCNo64 "llo" /* uint64_t* */
# endif
# ifndef SCNoLEAST8
#  define SCNoLEAST8 "hho" /* uint_least8_t* */
# endif
# ifndef SCNoLEAST16
#  define SCNoLEAST16 "ho" /* uint_least16_t* */
# endif
# ifndef SCNoLEAST32
#  define SCNoLEAST32 "o" /* uint_least32_t* */
# endif
# ifndef SCNoLEAST64
#  define SCNoLEAST64 "llo" /* uint_least64_t* */
# endif
# ifndef SCNoFAST8
#  define SCNoFAST8 "o" /* uint_fast8_t* */
# endif
# ifndef SCNoFAST16
#  define SCNoFAST16 "o" /* uint_fast16_t* */
# endif
# ifndef SCNoFAST32
#  define SCNoFAST32 "o" /* uint_fast32_t* */
# endif
# ifndef SCNoFAST64
#  define SCNoFAST64 "llo" /* uint_fast64_t* */
# endif
# ifndef SCNoMAX
#  define SCNoMAX "jo" /* uintmax_t* */
# endif
# ifndef SCNoPTR
#  define SCNoPTR "o" /* uintptr_t* */
# endif

# ifndef SCNu8
#  define SCNu8 "hhu" /* uint8_t* */
# endif
# ifndef SCNu16
#  define SCNu16 "hu" /* uint16_t* */
# endif
# ifndef SCNu32
#  define SCNu32 "u" /* uint32_t* */
# endif
# ifndef SCNu64
#  define SCNu64 "llu" /* uint64_t* */
# endif
# ifndef SCNuLEAST8
#  define SCNuLEAST8 "hhu" /* uint_least8_t* */
# endif
# ifndef SCNuLEAST16
#  define SCNuLEAST16 "hu" /* uint_least16_t* */
# endif
# ifndef SCNuLEAST32
#  define SCNuLEAST32 "u" /* uint_least32_t* */
# endif
# ifndef SCNuLEAST64
#  define SCNuLEAST64 "llu" /* uint_least64_t* */
# endif
# ifndef SCNuFAST8
#  define SCNuFAST8 "u" /* uint_fast8_t* */
# endif
# ifndef SCNuFAST16
#  define SCNuFAST16 "u" /* uint_fast16_t* */
# endif
# ifndef SCNuFAST32
#  define SCNuFAST32 "u" /* uint_fast32_t* */
# endif
# ifndef SCNuFAST64
#  define SCNuFAST64 "llu" /* uint_fast64_t* */
# endif
# ifndef SCNuMAX
#  define SCNuMAX "ju" /* uintmax_t* */
# endif
# ifndef SCNuPTR
#  define SCNuPTR "u" /* uintptr_t* */
# endif

# ifndef SCNx8
#  define SCNx8 "hhx" /* uint8_t* */
# endif
# ifndef SCNx16
#  define SCNx16 "hx" /* uint16_t* */
# endif
# ifndef SCNx32
#  define SCNx32 "x" /* uint32_t* */
# endif
# ifndef SCNx64
#  define SCNx64 "llx" /* uint64_t* */
# endif
# ifndef SCNxLEAST8
#  define SCNxLEAST8 "hhx" /* uint_least8_t* */
# endif
# ifndef SCNxLEAST16
#  define SCNxLEAST16 "hx" /* uint_least16_t* */
# endif
# ifndef SCNxLEAST32
#  define SCNxLEAST32 "x" /* uint_least32_t* */
# endif
# ifndef SCNxLEAST64
#  define SCNxLEAST64 "llx" /* uint_least64_t* */
# endif
# ifndef SCNxFAST8
#  define SCNxFAST8 "x" /* uint_fast8_t* */
# endif
# ifndef SCNxFAST16
#  define SCNxFAST16 "x" /* uint_fast16_t* */
# endif
# ifndef SCNxFAST32
#  define SCNxFAST32 "x" /* uint_fast32_t* */
# endif
# ifndef SCNxFAST64
#  define SCNxFAST64 "llx" /* uint_fast64_t* */
# endif
# ifndef SCNxMAX
#  define SCNxMAX "jx" /* uintmax_t* */
# endif
# ifndef SCNxPTR
#  define SCNxPTR "x" /* uintptr_t* */
# endif

#endif /* !__BO_LIBC_FMTIO__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
