/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Limits.h	(V.Sokur)
 * include/Bo/libc/Limits.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_LIMITS__
# define __BO_LIBC_LIMITS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Cdefs.h>

# include __MYC_LIBC_HEADER(limits.h)
/*
	Required that this file contains machine-dep-limits (<machine/limits.h>)
	and sys-dep-limits (<sys/syslimits.h>).

	machine-dep-limits:
		CHAR_BIT - number of bits in a char
		MB_LEN_MAX
		CLK_TCK - ticks per second
		signed/unsigned char/short/int/long/long long min/max constants
		...

	sys-dep-limits:
		constants for the arg string max. size, max. count of opened files
		per process, max. pathname size (in bytes), max. file name length
		(in bytes) and ...
 */

# if __myc_exists(sys7syslimits,h)
#  include __MYC_LIBC_HEADER(sys/syslimits.h)
# endif

# if __myc_exists(sys7param,h)
#  include __MYC_LIBC_HEADER(sys/param.h)
/*
	machine-indep-constants:
		MAXLOGNAME - max. login name length
		MAXHOSTNAMELEN - max. host name size
		MAXDOMNAMELEN - max. domain name size
		MAXPATHLEN - max. pathname size
		...
 */
# endif

/******************************************************************************
 * %% BeginSection: constants
 */
# pragma mark -

/*
 * Machine dependent constants
 */

# ifndef NBBY
#  ifdef __BO_CHAR_BIT__
#   define NBBY	__BO_CHAR_BIT__
#  else
#  ifdef CHAR_BIT
#   define NBBY	CHAR_BIT
#  else
#   define NBBY	(8)
#  endif
#  endif
# endif

# ifndef SSIZE_MAX
#  define SSIZE_MAX LONG_MAX
# endif
# ifndef SSIZE_MIN
#  define SSIZE_MIN LONG_MIN
# endif

# define BoL_INDEX_MAX SSIZE_MAX
# define BoL_INDEX_MIN SSIZE_MIN

/*
 * Machine independent constants
 */

/* BoL_NAME_MAX defines the longest size of a file name, including
   the terminating NUL character if it is required. */

# ifndef BoL_MAXFNAMELEN
#  ifdef NAME_MAX
#   if (255 <= NAME_MAX)
#    define BoL_MAXFNAMELEN  NAME_MAX
#   endif
#  endif
# endif

# ifndef BoL_MAXFNAMELEN
#  define BoL_MAXFNAMELEN 255
# endif

# ifndef BoL_NAME_MAX
#  define BoL_NAME_MAX (BoL_MAXFNAMELEN + 1)
# endif

# ifndef NAME_MAX
#  define NAME_MAX BoL_MAXFNAMELEN
# endif


/* BoL_PATH_MAX defines the longest size of a pathname, including
   the terminating NUL character if it is required. */

# ifndef BoL_MAXPATHLEN
#  ifdef PATH_MAX
#   if (1024 <= PATH_MAX)
#    define BoL_MAXPATHLEN  PATH_MAX
#   endif
#  endif
# endif

# ifndef BoL_MAXPATHLEN
#  ifdef MAXPATHLEN
#   if (1024 <= MAXPATHLEN)
#    define BoL_MAXPATHLEN  MAXPATHLEN
#   endif
#  endif
# endif

# ifndef BoL_MAXPATHLEN
#  define BoL_MAXPATHLEN 1024
# endif

# ifndef BoL_PATH_MAX
#  define BoL_PATH_MAX (BoL_MAXPATHLEN + 1)
# endif

# ifndef PATH_MAX
#  define PATH_MAX BoL_MAXPATHLEN
# endif


/* BoL_HOST_NAME_MAX defines the longest size of a hostname, including the
   the terminating NUL character if it is required. */

# ifndef BoL_MAXHOSTNAMELEN
#  ifdef HOST_NAME_MAX
#   if (256 <= HOST_NAME_MAX)
#    define BoL_MAXHOSTNAMELEN  HOST_NAME_MAX
#   endif
#  endif
# endif

# ifndef BoL_MAXHOSTNAMELEN
#  ifdef MAXHOSTNAMELEN
#   if (256 <= MAXHOSTNAMELEN)
#    define BoL_MAXHOSTNAMELEN  MAXHOSTNAMELEN
#   endif
#  endif
# endif

# ifndef BoL_MAXHOSTNAMELEN
#  define BoL_MAXHOSTNAMELEN 256
# endif

# ifndef BoL_HOST_NAME_MAX
#  define BoL_HOST_NAME_MAX (BoL_MAXHOSTNAMELEN + 1)
# endif

# ifndef HOST_NAME_MAX
#  define HOST_NAME_MAX BoL_MAXHOSTNAMELEN
# endif


/* Max count of arguments (strings). */
# ifndef BoL_EXEC_ARG_MAX
#  define BoL_EXEC_ARG_MAX (1024) /* Including the trailing NUL string. */
# endif

# ifndef BoL_PIPE_BUFSIZ
#  ifdef PIPE_BUF
#   define BoL_PIPE_BUFSIZ PIPE_BUF
#  else
#   define BoL_PIPE_BUFSIZ (1024)
#  endif
# endif
# ifndef BoL_PIPE_TIMEOUT
#  define BoL_PIPE_TIMEOUT (5000000) /* microseconds */
# endif

/*
 * STDIO
 */

# ifndef BoL_FOMODE_SIZE_MAX
#  define BoL_FOMODE_SIZE_MAX (64) /* Including the terminating NUL character */
# endif

/*
 * CHARACTER-SET
 */

# ifndef MB_LEN_MAX
#  define BoL_MB_LEN_MAX (6)
# else /* MB_LEN_MAX */
# if MB_LEN_MAX < 6
#  define BoL_MB_LEN_MAX (6)
# else
#  define BoL_MB_LEN_MAX MB_LEN_MAX
# endif
# endif /* MB_LEN_MAX */

# ifndef MB_LEN_MAX
#  define MB_LEN_MAX BoL_MB_LEN_MAX
# endif

# ifndef BoL_CSNAME_MAX
#  define BoL_CSNAME_MAX (32) /* Including the terminating NUL character */
# endif

/* @seealso: <Bo/aorp/Types.h> */
# ifndef BoL_LOCALE_LEN_MAX
#  define BoL_LOCALE_LEN_MAX (32) /* Including the terminating NUL character */
# endif

/*
 * AORP
 */

# ifndef BoL_AORP_CALL_NEST_MAX
#  define BoL_AORP_CALL_NEST_MAX (512)
# endif

# ifndef BoL_AORP_MODNAME_MAX
#  define BoL_AORP_MODNAME_MAX (256) /* Including the terminating NUL character */
# endif

/*
 * URI
 */

# ifndef BoL_URI_SCHEMENAME_MAX
#  define BoL_URI_SCHEMENAME_MAX (32) /* Including the terminating NUL character */
# endif

/*
 * PLIST
 */

# ifndef BoL_PLIST_NAME_MAX
#  define BoL_PLIST_NAME_MAX (256)      /* Including the terminating NUL character */
# endif
# ifndef BoL_PLIST_KEY_MAX
#  define BoL_PLIST_KEY_MAX (1024)      /* Including the terminating NUL character */
# endif

# ifndef BoL_PLIST_NATTRIBS_MAX
#  define BoL_PLIST_NATTRIBS_MAX (100)
# endif
# ifndef BoL_PLIST_ATTRNAME_MAX
#  define BoL_PLIST_ATTRNAME_MAX (256)  /* Including the terminating NUL character */
# endif
# ifndef BoL_PLIST_ATTRVAL_MAX
#  define BoL_PLIST_ATTRVAL_MAX (2048)   /* Including the terminating NUL character */
# endif

#endif /* !__BO_LIBC_LIMITS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
