/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Stdarg.h	(V.Sokur)
 * include/Bo/libc/Stdarg.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_STDARG__
# define __BO_LIBC_STDARG__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/libc/Stddef.h>

# if __myc_exists(stdarg,h) && __myc_support(stdarg)
#  include __MYC_LIBC_HEADER(stdarg.h)
# else
# if __myc_exists(machine7varargs,h)
#  include __MYC_LIBC_HEADER(machine/varargs.h)
# else
# if __myc_exists(varargs,h)
#  include __MYC_LIBC_HEADER(varargs.h)
# endif /* varargs.h */
# endif /* machine/varargs.h */
# endif /* stdarg.h */

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -

# if __myc_support(stdarg)

#  define __myc_va_start(__ap,__lastarg) va_start(__ap,__lastarg)
#  define __myc_va_arg(__ap,__tp) va_arg(__ap,__tp)
#  define __myc_va_end(__ap) va_end(__ap)

# else /* !__myc_support(stdarg) */

/* fixme */
#  ifdef __GNUC__
#  if __GNUC__ > 3
#    undef va_dcl
#    undef va_alist
#    undef va_start
#    undef va_arg
#    undef va_end

typedef char	*__bo_va_list;

#   define __bo_va_size(type) \
	(((sizeof(type) + sizeof(int) - 1) / sizeof(int)) * sizeof(int))

#   if (__GNUC__ >= 3)
#    define va_dcl	int va_alist;
#   else
#   if (__GNUC__ > 1)
#    define va_dcl	int va_alist; ...
#   else
#    define va_dcl	int va_alist; 
#   endif /* !(__GNUC__ > 1) */
#   endif /* (__GNUC__ >= 3) */

#   define va_start(ap) ((ap) = (__bo_va_list)&va_alist)
#   define va_arg(ap,type) (*(type *)((ap) += __bo_va_size(type), (ap) - __bo_va_size(type)))
#   define va_end(ap) ((ap) = 0)
#  endif /* __GNUC__ > 3 */
#  endif /* __GNUC__ */

#  define __myc_va_start(__ap,__lastarg) va_start(__ap)
#  define __myc_va_arg(__ap,__tp) va_arg(__ap,__tp)
#  define __myc_va_end(__ap) va_end(__ap)

# endif /* !__myc_support(stdarg) */

# ifdef BO_SYSTYPE_WIN32
#  define __myc_va_equ(aDst,aSrc)        ((aDst) = (aSrc))
#  define __myc_va_eend(__ap)            ((void)(__ap))
# else
#  define __myc_va_equ(aDst,aSrc)        va_copy(aDst,aSrc)
#  define __myc_va_eend(__ap)            __myc_va_end(__ap)
# endif

/* compat */
# define bo_va_start(__ap,__lastarg) __myc_va_start(__ap,__lastarg)
# define bo_va_arg(__ap,__tp) __myc_va_arg(__ap,__tp)
# define bo_va_end(__ap) __myc_va_end(__ap)

#endif /* !__BO_LIBC_STDARG__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
