/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Stddir.h	(V.Sokur)
 * include/Bo/libc/Stddir.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_STDDIR__
# define __BO_LIBC_STDDIR__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/libc/Stddef.h>

/* unix */
# if __myc_exists(dirent,h)
#  include __MYC_LIBC_HEADER(dirent.h)
# endif

/* windows */
# if __myc_exists(io,h)
#  include __MYC_LIBC_HEADER(io.h)
# endif

# include <Bo/sys/Dirent.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -

struct bo_direct_s {
# ifdef BO_SYSTYPE_WIN32
	intptr_t                dd_fd;      /* [directory] handle */
	union {
		struct _finddata_t  dd_finddata;
		struct _wfinddata_t dd_wfinddata;
	} u1;
# else
# ifdef HAS_DIR_STRUC
	DIR                 *dd_impl;
# else
#  error Unsupported C environment!!!
# endif /* HAS_DIR_STRUC */
# endif /* BO_SYSTYPE_WIN32 */

	int                  dd_is_unicode;
	union {
		char const      *dd_tmpl;    /* template for readdir */
		wchar_t const   *dd_wtmpl;   /* template for wreaddir */
	} u2;
};

typedef struct bo_direct_s *bo_direct_t;

# ifndef HAS_DIR_STRUC
typedef struct bo_direct_s DIR;
# endif


# ifndef DIRBLKSIZ
#  define DIRBLKSIZ 1024
# endif

#if 0
/* flags for opendir2 */
# define DTF_HIDEW 0x01     /* hide whiteout entries */
# define DTF_NODUP 0x02     /* do not return duplicate names */
# define DTF_REWIND 0x04    /* rewind after reading union stack */
# define __DTF_READALL 0x08 /* everything has been reading */
#endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoOpendir
 */

__myc_flinkage__(public,libBo) bo_direct_t
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		        BoK_OpError_Interrupted, BoK_RacError_AccessViolation,
		        BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		        BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		        BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		        BoK_IoError_NoInformation, BoK_SysError_NoMemory);
	];
))
BoOpendir(
# if __myc_require(prototypes)
	char const *aPath,
	char const *aTemplate __myc_defval__(NULL), /* used by readdir */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_OPENDIR
#  ifdef HAS_DIR_STRUC
#   error Unsupported C environment!!!
#  endif
#  define NEED_OPENDIR_IMPL
# endif /* !HAS_OPENDIR */

# ifdef NEED_OPENDIR_IMPL

__myc_flinkage__(public,libBo) DIR *
opendir(
# if __myc_require(prototypes)
	char const *aPath
# endif
	);

#  define opendir(aPath) BoOpendir(aPath, NULL, NULL)

# endif /* NEED_OPENDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoWopendir
 */

__myc_flinkage__(public,libBo) bo_direct_t
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		        BoK_OpError_Interrupted, BoK_RacError_AccessViolation,
		        BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		        BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		        BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		        BoK_IoError_NoInformation, BoK_SysError_NoMemory);
	];
))
BoWopendir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	wchar_t const *aTemplate __myc_defval__(NULL), /* used by readdir */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_WOPENDIR
#  define NEED_WOPENDIR_IMPL
# endif /* !HAS_WOPENDIR */

# ifdef NEED_WOPENDIR_IMPL

__myc_flinkage__(public,libBo) DIR *
wopendir(
# if __myc_require(prototypes)
	wchar_t const *aPath
# endif
	);

#  ifndef HAS_DIR_STRUC
#   define wopendir(aPath) BoWopendir(aPath, NULL, NULL)
#  endif

# endif /* NEED_WOPENDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoClosedir
 */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoClosedir(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_CLOSEDIR
#  ifdef HAS_DIR_STRUC
#   error Unsupported C environment!!!
#  endif
#  define NEED_CLOSEDIR_IMPL
# endif /* !HAS_CLOSEDIR */

# ifdef NEED_CLOSEDIR_IMPL

__myc_flinkage__(public,libBo) int
closedir(
# if __myc_require(prototypes)
	DIR *aDir
# endif
	);

#  define closedir(aDir) BoClosedir(aDir, NULL)

# endif /* NEED_CLOSEDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoReaddir
 */

__myc_flinkage__(public,libBo) struct dirent *
__status__((
	BoS_SUCCESS = [
		1++ = ();
		0 /* no more items */ = ();
	];
	BoS_FAILURE = [
		BoS_EPTR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		            BoK_OpError_Interrupted, BoK_FileError_NameSizeLimit,
		            BoK_IoError_NoInformation);
	];
))
BoReaddir(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_READDIR
#  ifdef HAS_DIR_STRUC
#   error Unsupported C environment!!!
#  endif
#  define NEED_READDIR_IMPL
# endif /* !HAS_READDIR */

# ifdef NEED_READDIR_IMPL

__myc_flinkage__(public,libBo) struct dirent *
readdir(
# if __myc_require(prototypes)
	DIR *aDir
# endif
	);

# endif /* NEED_READDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoWreaddir
 */

__myc_flinkage__(public,libBo) struct wdirent *
__status__((
	BoS_SUCCESS = [
		1++ = ();
		0 /* no more items */ = ();
	];
	BoS_FAILURE = [
		BoS_EPTR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		            BoK_OpError_Interrupted, BoK_FileError_NameSizeLimit,
		            BoK_IoError_NoInformation);
	];
))
BoWreaddir(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_WREADDIR
#  define NEED_WREADDIR_IMPL
# endif /* !HAS_WREADDIR */

# ifdef NEED_WREADDIR_IMPL

__myc_flinkage__(public,libBo) struct wdirent *
wreaddir(
# if __myc_require(prototypes)
	DIR *aDir
# endif
	);

# endif /* NEED_WREADDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoReaddir_r
 */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_TRUE = ();
	BoS_FALSE /* no more items */ = ();
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_FileError_NameSizeLimit, BoK_IoError_NoInformation);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoReaddir_r(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	struct dirent *aBuffer,
	struct dirent **aResultPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_READDIR_R
#  define NEED_READDIR_R_IMPL
# endif /* !HAS_READDIR_R */

# ifdef NEED_READDIR_R_IMPL

__myc_flinkage__(public,libBo) int
readdir_r(
# if __myc_require(prototypes)
	DIR *aDir,
	struct dirent *aBuffer,
	struct dirent **aResultPtr
# endif
	);

#  ifndef HAS_DIR_STRUC
#   define readdir_r(aDir, aBuffer, aResultPtr) \
		BoReaddir_r(aDir, aBuffer, aResultPtr, NULL)
#  endif

# endif /* NEED_READDIR_R_IMPL */

/* ----------------------------------------------------------------------------
 * BoWreaddir_r
 */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_TRUE = ();
	BoS_FALSE /* no more items */ = ();
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_FileError_NameSizeLimit, BoK_IoError_NoInformation);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoWreaddir_r(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	struct wdirent *aBuffer,
	struct wdirent **aResultPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_WREADDIR_R
#  define NEED_WREADDIR_R_IMPL
# endif /* !HAS_WREADDIR_R */

# ifdef NEED_WREADDIR_R_IMPL

__myc_flinkage__(public,libBo) int
wreaddir_r(
# if __myc_require(prototypes)
	DIR *aDir,
	struct wdirent *aBuffer,
	struct wdirent **aResultPtr
# endif
	);

#  ifndef HAS_DIR_STRUC
#   define wreaddir_r(aDir, aBuffer, aResultPtr) \
		BoWreaddir_r(aDir, aBuffer, aResultPtr, NULL)
#  endif

# endif /* NEED_WREADDIR_R_IMPL */

/* ----------------------------------------------------------------------------
 * BoRewinddir
 */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoRewinddir(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_REWINDDIR
#  ifdef HAS_DIR_STRUC
#   error Unsupported C environment!!!
#  endif
#  define NEED_REWINDDIR_IMPL
# endif /* !HAS_REWINDDIR */

# ifdef NEED_REWINDDIR_IMPL

__myc_flinkage__(public,libBo) void
rewinddir(
# if __myc_require(prototypes)
	DIR *aDir
# endif
	);

#  define rewinddir(aDir) ((void) BoRewinddir(aDir, NULL))

# endif /* NEED_REWINDDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoSeekdir
 */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoSeekdir(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	long aPos,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_SEEKDIR
#  ifdef HAS_DIR_STRUC
#   error Unsupported C environment!!!
#  endif
#  define NEED_SEEKDIR_IMPL
# endif /* !HAS_SEEKDIR */

# ifdef NEED_SEEKDIR_IMPL

__myc_flinkage__(public,libBo) void
seekdir(
# if __myc_require(prototypes)
	DIR *aDir,
	long aPos
# endif
	);

#  define seekdir(aDir, aPos) ((void) BoSeekdir(aDir, aPos, NULL))

# endif /* !HAS_SEEKDIR */

/* ----------------------------------------------------------------------------
 * BoTelldir
 */

__myc_flinkage__(public,libBo) long
__status__((
	BoS_SUCCESS = [ 0++ = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoTelldir(
# if __myc_require(prototypes)
	bo_direct_t aDir,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_TELLDIR
#  ifdef HAS_DIR_STRUC
#   error Unsupported C environment!!!
#  endif
#  define NEED_TELLDIR_IMPL
# endif /* !HAS_TELLDIR */

# ifdef NEED_TELLDIR_IMPL

__myc_flinkage__(public,libBo) long
telldir(
# if __myc_require(prototypes)
	DIR *aDir
# endif
	);

#  define telldir(aDir) BoTelldir(aDir, NULL)

# endif /* NEED_TELLDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoScandir
 */

/* The select routine returns bool value: if selected - 1, otherwise - 0. */
__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_OpError_Interrupted, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_IoError_NoInformation, BoK_SysError_NoMemory);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoScandir(
# if __myc_require(prototypes)
	char const *aPath,
	struct dirent ***aListPtr,
	int (*aSelect) __myc_proto__((struct dirent *, bo_pointer_t aContext)) __myc_defval__(NULLF),
	bo_pointer_t aContext __myc_defval__(NULL),
	int (*aCompare) __myc_proto__((bo_const_pointer_t , bo_const_pointer_t )) __myc_defval__(NULLF),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_SCANDIR
#  define NEED_SCANDIR_IMPL
# endif /* !HAS_SCANDIR */

# ifdef NEED_SCANDIR_IMPL

__myc_flinkage__(public,libBo) int
scandir(
# if __myc_require(prototypes)
	char const *aPath,
	struct dirent ***aListPtr,
	int (*aSelect) __myc_proto__((struct dirent *, bo_pointer_t )) __myc_defval__(NULLF),
	int (*aCompare) __myc_proto__((bo_const_pointer_t , bo_const_pointer_t )) __myc_defval__(NULLF)
# endif
	);

#  define scandir(aPath, aListPtr, aSelect, aCompare) \
	BoScandir(aPath, aListPtr, aSelect, NULL, aCompare, NULL)

# endif /* NEED_SCANDIR_IMPL */

__myc_flinkage__(public,libBo) void
BoDirelsFree(
# if __myc_require(prototypes)
	size_t aCount,
	struct dirent **aList
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWscandir
 */

/* The select routine returns bool value: if selected - 1, otherwise - 0. */
__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_OpError_Interrupted, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_IoError_NoInformation, BoK_SysError_NoMemory);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoWscandir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct wdirent ***aListPtr,
	int (*aSelect) __myc_proto__((struct wdirent *, bo_pointer_t aContext)) __myc_defval__(NULLF),
	bo_pointer_t aContext __myc_defval__(NULL),
	int (*aCompare) __myc_proto__((bo_const_pointer_t , bo_const_pointer_t )) __myc_defval__(NULLF),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_WSCANDIR
#  define NEED_WSCANDIR_IMPL
# endif /* !HAS_WSCANDIR */

# ifdef NEED_WSCANDIR_IMPL

__myc_flinkage__(public,libBo) int
wscandir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct wdirent ***aListPtr,
	int (*aSelect) __myc_proto__((struct wdirent *, bo_pointer_t )) __myc_defval__(NULLF),
	int (*aCompare) __myc_proto__((bo_const_pointer_t , bo_const_pointer_t )) __myc_defval__(NULLF)
# endif
	);

#  define wscandir(aPath, aListPtr, aSelect, aCompare) \
	BoWscandir(aPath, aListPtr, aSelect, NULL, aCompare, NULL)

# endif /* NEED_WSCANDIR_IMPL */

__myc_flinkage__(public,libBo) void
BoWdirelsFree(
# if __myc_require(prototypes)
	size_t aCount,
	struct wdirent **aList
# endif
	);

/* ----------------------------------------------------------------------------
 * alphasort, walphasort
 */

# ifndef HAS_ALPHASORT
#  define NEED_ALPHASORT_IMPL
# endif /* !HAS_ALPHASORT */

# ifdef NEED_ALPHASORT_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
alphasort(
# if __myc_require(prototypes)
	bo_const_pointer_t aLhs,
	bo_const_pointer_t aRhs
# endif
	);

# endif /* NEED_ALPHASORT_IMPL */

# ifndef HAS_WALPHASORT
#  define NEED_WALPHASORT_IMPL
# endif /* !HAS_WALPHASORT */

# ifdef NEED_WALPHASORT_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
walphasort(
# if __myc_require(prototypes)
	bo_const_pointer_t aLhs,
	bo_const_pointer_t aRhs
# endif
	);

# endif /* NEED_WALPHASORT_IMPL */

__MYC_END_CDECLS

#endif /* !__BO_LIBC_STDDIR__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
