/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Stdio.h	(V.Sokur)
 * include/Bo/libc/Stdio.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_STDIO__
# define __BO_LIBC_STDIO__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# if __myc_exists(stdio,h)
#  include __MYC_LIBC_HEADER(stdio.h)
# endif /* stdio.h */

# include <Bo/libc/Stddef.h>
# include <Bo/libc/Stdarg.h>
# include <Bo/libc/Wchar.h>

/******************************************************************************
 * %% BeginSection: string function prototypes
 */

# include <Bo/BoLinkage.h>

/* ============================================================================
 * Overloaded version of printf functions:
 * snprintf, vsnprintf, swprintf, vswprintf.
 */

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * snprintf
 */
# pragma mark -

# ifndef HAS_SNPRINTF
#  ifdef HAS__SPRINTF_P
#   ifndef OVERLOAD_SNPRINTF__SPRINTF_P
#    define OVERLOAD_SNPRINTF__SPRINTF_P
#   endif
#  else
#  ifdef HAS__SNPRINTF_S
#   ifndef OVERLOAD_SNPRINTF__SNPRINTF_S
#    define OVERLOAD_SNPRINTF__SNPRINTF_S
#   endif
#  endif /* HAS__SNPRINTF_S */
#  endif /* HAS__SPRINTF_P */
# endif

# ifdef OVERLOAD_SNPRINTF__SPRINTF_P

#  ifdef SNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef SNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _SPRINTF_P_OUTPUT_ALWAYS_NULL_TERMINATED
#   define SNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef SNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef SNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _SPRINTF_P_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define SNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef snprintf
#  define snprintf _sprintf_p

# else
# ifdef OVERLOAD_SNPRINTF__SNPRINTF_S

#  ifdef SNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef SNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _SNPRINTF_S_OUTPUT_ALWAYS_NULL_TERMINATED
#   define SNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef SNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef SNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _SNPRINTF_S_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define SNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef snprintf
#  define snprintf(aBuffer,aBufsz,aFormat,...) \
	_snprintf_s(aBuffer,aBufsz,((aBufsz) - 1),aFormat,__VA_ARGS__)

# endif /* OVERLOAD_SNPRINTF__SNPRINTF_S */
# endif /* OVERLOAD_SNPRINTF__SPRINTF_P */

/* ----------------------------------------------------------------------------
 * vsnprintf
 */
# pragma mark -

# ifndef HAS_VSNPRINTF
#  ifdef HAS__VSPRINTF_P
#   ifndef OVERLOAD_VSNPRINTF__VSPRINTF_P
#    define OVERLOAD_VSNPRINTF__VSPRINTF_P
#   endif
#  else
#  ifdef HAS__VSNPRINTF_S
#   ifndef OVERLOAD_VSNPRINTF__VSNPRINTF_S
#    define OVERLOAD_VSNPRINTF__VSNPRINTF_S
#   endif
#  endif
#  endif
# endif

# ifdef OVERLOAD_VSNPRINTF__VSPRINTF_P

#  ifdef VSNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef VSNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _VSPRINTF_P_OUTPUT_ALWAYS_NULL_TERMINATED
#   define VSNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef VSNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef VSNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _VSPRINTF_P_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define VSNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef vsnprintf
#  define vsnprintf _vsprintf_p

# else
# ifdef OVERLOAD_VSNPRINTF__VSNPRINTF_S

#  ifdef VSNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef VSNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _VSNPRINTF_S_OUTPUT_ALWAYS_NULL_TERMINATED
#   define VSNPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef VSNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef VSNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _VSNPRINTF_S_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define VSNPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef vsnprintf
#  define vsnprintf(aBuffer,aBufsz,aFormat,anArgList) \
	_vsnprintf_s(aBuffer,aBufsz,((aBufsz) - 1),aFormat,anArgList)

# endif /* OVERLOAD_VSNPRINTF__VSNPRINTF_S */
# endif /* OVERLOAD_VSNPRINTF__VSPRINTF_P */

/* ----------------------------------------------------------------------------
 * swprintf
 * wsnprintf (alternate)
 */
# pragma mark -

# define wsnprintf swprintf

# ifndef HAS_SWPRINTF
#  ifdef HAS__SWPRINTF_P
#   ifndef OVERLOAD_SWPRINTF__SWPRINTF_P
#    define OVERLOAD_SWPRINTF__SWPRINTF_P
#   endif
#  else
#  ifdef HAS__SNWPRINTF_S
#   ifndef OVERLOAD_SWPRINTF__SNWPRINTF_S
#    define OVERLOAD_SWPRINTF__SNWPRINTF_S
#   endif
#  endif
#  endif
# endif

# ifdef OVERLOAD_SWPRINTF__SWPRINTF_P

#  ifdef SWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef SWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _SWPRINTF_P_OUTPUT_ALWAYS_NULL_TERMINATED
#   define SWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef SWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef SWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _SWPRINTF_P_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define SWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef swprintf
#  define swprintf _swprintf_p

# else
# ifdef OVERLOAD_SWPRINTF__SNWPRINTF_S

#  ifdef SWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef SWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _SNWPRINTF_S_OUTPUT_ALWAYS_NULL_TERMINATED
#   define SWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef SWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef SWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _SNWPRINTF_S_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define SWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef swprintf
#  define swprintf(aBuffer,aWCharc,aFormat,...) \
	_snwprintf_s(aBuffer,aWCharc,((aWCharc) - 1),aFormat,__VA_ARGS__)

# endif /* OVERLOAD_SWPRINTF__SNWPRINTF_S */
# endif /* OVERLOAD_SWPRINTF__SWPRINTF_P */

/* ----------------------------------------------------------------------------
 * vswprintf
 * vwsnprintf (alternate)
 */
# pragma mark -

# define vwsnprintf vswprintf

# ifndef HAS_VSWPRINTF
#  ifdef HAS__VSWPRINTF_P
#   ifndef OVERLOAD_VSWPRINTF__VSWPRINTF_P
#    define OVERLOAD_VSWPRINTF__VSWPRINTF_P
#   endif
#  else
#  ifdef HAS__VSNWPRINTF_S
#   ifndef OVERLOAD_VSWPRINTF__VSNWPRINTF_S
#    define OVERLOAD_VSWPRINTF__VSNWPRINTF_S
#   endif
#  endif
#  endif
# endif

# ifdef OVERLOAD_VSWPRINTF__VSWPRINTF_P

#  ifdef VSWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef VSWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _VSWPRINTF_P_OUTPUT_ALWAYS_NULL_TERMINATED
#   define VSWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef VSWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef VSWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _VSWPRINTF_P_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define VSWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef vswprintf
#  define vswprintf _vswprintf_p

# else
# ifdef OVERLOAD_VSWPRINTF__VSNWPRINTF_S

#  ifdef VSWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#   undef VSWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif
#  ifdef _VSNWPRINTF_S_OUTPUT_ALWAYS_NULL_TERMINATED
#   define VSWPRINTF_OUTPUT_ALWAYS_NULL_TERMINATED
#  endif

#  ifdef VSWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   undef VSWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif
#  ifdef _VSNWPRINTF_S_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#   define VSWPRINTF_RETURNS_NCHARS_WOULD_HAVE_BEEN_PRINTED
#  endif

#  undef vswprintf
#  define vswprintf(aBuffer,aWCharc,aFormat,anArgList) \
	_vsnwprintf_s(aBuffer,aWCharc,((aWCharc) - 1),aFormat,anArgList)

# endif /* OVERLOAD_SWPRINTF__SNWPRINTF_S */
# endif /* OVERLOAD_VSWPRINTF__VSWPRINTF_P */

__MYC_END_CDECLS

/* ============================================================================
 * Missing version of printf functions:
 * asprintf, vasprintf, aswprintf, vaswprintf,
 * _scprintf, _vscprintf, _scwprintf, _vscwprintf,
 */

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * asprintf
 */
# pragma mark -

# ifndef HAS_ASPRINTF
#  define NEED_ASPRINTF_IMPL
#  define asprintf_free bo_free
# else
#  define asprintf_free free
# endif /* !HAS_ASPRINTF */

# ifdef NEED_ASPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
asprintf(
# if __myc_require(prototypes)
	char **aResultPtr,
	char const *aFormat,
	...
# endif
	);

# endif /* NEED_ASPRINTF_IMPL */

/* ----------------------------------------------------------------------------
 * vasprintf
 */
# pragma mark -

# ifndef HAS_VASPRINTF
#  define NEED_VASPRINTF_IMPL
#  define vasprintf_free bo_free
# else
#  define vasprintf_free free
# endif /* !HAS_VASPRINTF */

# ifdef NEED_VASPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
vasprintf(
# if __myc_require(prototypes)
	char **aResultPtr,
	char const *aFormat,
	va_list
# endif
	);

# endif /* NEED_ASPRINTF_IMPL */

/* ----------------------------------------------------------------------------
 * aswprintf
 * awsprintf (alternate)
 */
# pragma mark -

# define awsprintf aswprintf

# ifndef HAS_ASWPRINTF
#  define NEED_ASWPRINTF_IMPL
#  define awsprintf_free bo_free
# else
#  define awsprintf_free free
# endif /* !HAS_ASWPRINTF */

# ifdef NEED_ASWPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
aswprintf(
# if __myc_require(prototypes)
	wchar_t **aResultPtr,
	wchar_t const *aFormat,
	...
# endif
	);

# endif /* NEED_ASWPRINTF_IMPL */

/* ----------------------------------------------------------------------------
 * vaswprintf
 * vawsprintf (alternate)
 */
# pragma mark -

# define vawsprintf vaswprintf

# ifndef HAS_VASWPRINTF
#  define NEED_VASWPRINTF_IMPL
#  define vawsprintf_free bo_free
# else
#  define vawsprintf_free free
# endif /* !HAS_VASWPRINTF */

# ifdef NEED_VASWPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
vaswprintf(
# if __myc_require(prototypes)
	wchar_t **aResultPtr,
	wchar_t const *aFormat,
	va_list
# endif
	);

# endif /* NEED_VASWPRINTF_IMPL */

/* ----------------------------------------------------------------------------
 * _scprintf
 */
# pragma mark -

# ifndef HAS__SCPRINTF
#  define NEED__SCPRINTF_IMPL
# endif /* !HAS__SCPRINTF */

# ifdef NEED__SCPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
_scprintf(
# if __myc_require(prototypes)
	char const *aFormat,
	...
# endif
	);

# endif /* NEED__SCPRINTF_IMPL */

/* ----------------------------------------------------------------------------
 * _vscprintf
 */
# pragma mark -

# ifndef HAS__VSCPRINTF
#  define NEED__VSCPRINTF_IMPL
# endif /* !HAS__VSCPRINTF */

# ifdef NEED__VSCPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
_vscprintf(
# if __myc_require(prototypes)
	char const *aFormat,
	va_list
# endif
	);

# endif /* NEED__VSCPRINTF_IMPL */

/* ----------------------------------------------------------------------------
 * _scwprintf
 * _wscprintf (alternate)
 */
# pragma mark -

# define _wscprintf _scwprintf

# ifndef HAS__SCWPRINTF
#  define NEED__SCWPRINTF_IMPL
# endif /* !HAS__SCWPRINTF */

# ifdef NEED__SCWPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
_scwprintf(
# if __myc_require(prototypes)
	wchar_t const *aFormat,
	...
# endif
	);

# endif /* NEED__SCWPRINTF_IMPL */

/* ----------------------------------------------------------------------------
 * _vscwprintf
 * _vwscprintf (alternate)
 */
# pragma mark -

# define _vwscprintf _vscwprintf

# ifndef HAS__VSCWPRINTF
#  define NEED__VSCWPRINTF_IMPL
# endif /* !HAS__VSCWPRINTF */

# ifdef NEED__VSCWPRINTF_IMPL

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
_vscwprintf(
# if __myc_require(prototypes)
	wchar_t const *aFormat,
	va_list
# endif
	);

# endif /* NEED__VSCWPRINTF_IMPL */

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: file function prototypes
 */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/*
The argument @mode points to a string beginning with one of the following
sequences (Additional characters may follow these sequences.):

    `r'   Open text file for reading.  The stream is positioned at the
             beginning of the file.

    `r+'  Open for reading and writing.  The stream is positioned at the
             beginning of the file.

    `w'   Truncate file to zero length or create text file for writing.
             The stream is positioned at the beginning of the file.

    `w+'  Open for reading and writing.  The file is created if it does not
             exist, otherwise it is truncated.  The stream is positioned at
             the beginning of the file.

    `a'   Open for writing.  The file is created if it does not exist.  The
             stream is positioned at the end of the file.  Subsequent writes
             to the file will always end up at the then current end of file,
             irrespective of any intervening fseek(3) or similar.

    `a+'  Open for reading and writing.  The file is created if it does not
             exist.  The stream is positioned at the end of the file.  Subse-
             quent writes to the file will always end up at the then current
             end of file, irrespective of any intervening fseek(3) or similar.

The mode string can also include the letter ``b'' either as a third character
or as a character between the characters in any of the two-character
strings described above.  This is strictly for compatibility with
ISO/IEC 9899:1990 (``ISO C90'') and has no effect; the ``b'' is ignored.
 */

# define BO_FOMODE_DEFAULT "r+"
# define BO_WFOMODE_DEFAULT ___myc_wtext("r+")
# define BoK_FOMODE_FLAGS_DEFAULT O_RDWR
# define BoK_FOMODE_PERM_DEFAULT (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)

/* ----------------------------------------------------------------------------
 *
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
BoCcmode(
# if __myc_require(prototypes)
	char *aBuffer,
	int Flags,
	bo_mode_t aPerm, /* unused */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
BoWcmode(
# if __myc_require(prototypes)
	wchar_t *aBuffer,
	int Flags,
	bo_mode_t aPerm, /* unused */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoMkcmode(
# if __myc_require(prototypes)
	char const *aStr,
	int *aFlagsPtr,
	bo_mode_t *aPermPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoWmkcmode(
# if __myc_require(prototypes)
	wchar_t const *aStr,
	int *aFlagsPtr,
	bo_mode_t *aPermPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoMkcunmode(
# if __myc_require(prototypes)
	char const *aStr, /* libc or unix style */
	int *aFlagsPtr,
	bo_mode_t *aPermPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoWmkcunmode(
# if __myc_require(prototypes)
	wchar_t const *aStr, /* libc or unix style */
	int *aFlagsPtr,
	bo_mode_t *aPermPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoFopen_p
 */
# pragma mark -

__myc_flinkage__(public,libBo) FILE *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		        BoK_OpError_Interrupted, BoK_OpError_FeatureNotSupp,
		        BoK_RacError_AccessViolation, BoK_RacError_NotConfigured,
		        BoK_FileError_NameSizeLimit, BoK_FileError_NotDir, BoK_FileError_NoSuchEntry,
		        BoK_FileError_AlreadyExists, BoK_FileError_IsDir, BoK_FileError_Loop,
		        BoK_FileError_TextBusy, BoK_FileError_ReadOnlyFS, BoK_FileError_NoFreeSpace,
		        BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		        BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoFopen_p(
# if __myc_require(prototypes)
	char const *aPath,
	char const *aMode __myc_defval__(BO_FOMODE_DEFAULT), /* libc, unix style */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoFopen_wp
 * wfopen
 */
# pragma mark -

__myc_flinkage__(public,libBo) FILE *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		        BoK_OpError_Interrupted, BoK_OpError_FeatureNotSupp,
		        BoK_RacError_AccessViolation, BoK_RacError_NotConfigured,
		        BoK_FileError_NameSizeLimit, BoK_FileError_NotDir, BoK_FileError_NoSuchEntry,
		        BoK_FileError_AlreadyExists, BoK_FileError_IsDir, BoK_FileError_Loop,
		        BoK_FileError_TextBusy, BoK_FileError_ReadOnlyFS, BoK_FileError_NoFreeSpace,
		        BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		        BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoFopen_wp(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	wchar_t const *aMode __myc_defval__(BO_WFOMODE_DEFAULT), /* libc, unix style */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_WFOPEN
#  ifdef HAS__WFOPEN
#   define wfopen(aPath, aMode) _wfopen(aPath, aMode)
#  else
#   define NEED_WFOPEN_IMPL
#  endif /* !HAS__WFOPEN */
# endif /* !HAS_WFOPEN */

# ifdef NEED_WFOPEN_IMPL

__myc_flinkage__(public,libBo) FILE *
wfopen(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	wchar_t const *aMode
# endif
	);

# endif /* NEED_WFOPEN_IMPL */

/* ----------------------------------------------------------------------------
 * BoFopen_uri
 */
# pragma mark -

__myc_flinkage__(public,libBo) FILE *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		        BoK_OpError_NotSupp, BoK_OpError_Interrupted, BoK_OpError_FeatureNotSupp,
		        BoK_RacError_AccessViolation, BoK_RacError_NotConfigured,
		        BoK_FileError_NameSizeLimit, BoK_FileError_NotDir, BoK_FileError_NoSuchEntry,
		        BoK_FileError_AlreadyExists, BoK_FileError_IsDir, BoK_FileError_Loop,
		        BoK_FileError_TextBusy, BoK_FileError_ReadOnlyFS, BoK_FileError_NoFreeSpace,
		        BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		        BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoFopen_uri(
# if __myc_require(prototypes)
	aorp_object_t aLocation,
	char const *aMode __myc_defval__(BO_FOMODE_DEFAULT), /* libc, unix style */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoFopen2_uri
 */
# pragma mark -

__myc_flinkage__(public,libBo) FILE *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		        BoK_OpError_NotSupp, BoK_OpError_Interrupted, BoK_OpError_FeatureNotSupp,
		        BoK_RacError_AccessViolation, BoK_RacError_NotConfigured,
		        BoK_FileError_NameSizeLimit, BoK_FileError_NotDir, BoK_FileError_NoSuchEntry,
		        BoK_FileError_AlreadyExists, BoK_FileError_IsDir, BoK_FileError_Loop,
		        BoK_FileError_TextBusy, BoK_FileError_ReadOnlyFS, BoK_FileError_NoFreeSpace,
		        BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		        BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoFopen2_uri(
# if __myc_require(prototypes)
	aorp_object_t aLocation,
	int Flags,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoFopen_fd
 */
# pragma mark -

#ifndef HAS_FILENO
# ifdef HAS__FILENO
#  define fileno(fp) _fileno(fp)
# endif
#endif

__myc_flinkage__(public,libBo) FILE *
__status__((
	BoS_SUCCESS = [ 1++ = (); ];
	BoS_FAILURE = [
		NULL = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		        BoK_OpError_Interrupted, BoK_OpError_FeatureNotSupp,
		        BoK_RacError_AccessViolation,
		        BoK_FileError_BadDescriptor, BoK_FileError_ProcessDescLimit,
		        BoK_IoError_NoInformation);
	];
))
BoFopen_fd(
# if __myc_require(prototypes)
	int aFD,
	char const *aMode __myc_defval__(BO_FOMODE_DEFAULT), /* only libc style */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_LIBC_STDIO__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
