/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * String.h	(V.Sokur)
 * include/Bo/libc/String.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_STRING__
# define __BO_LIBC_STRING__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/libc/Stddef.h>

# if __myc_exists(string,h)
#  include __MYC_LIBC_HEADER(string.h)
# endif
# if __myc_exists(wchar,h)
#  include __MYC_LIBC_HEADER(wchar.h)
# endif
# if __myc_exists(tchar,h)
#  include __MYC_LIBC_HEADER(tchar.h)
# endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Bo/BoLinkage.h>

/* ----------------------------------------------------------------------------
 * 'strdup()'
 */

# ifndef HAS_STRDUP
#  ifdef HAS__STRDUP
#   define strdup _strdup
#   define strdup_free free
#  else
#   define NEED_STRDUP_IMPL
#   define strdup_free bo_free
#  endif
# else
#  define strdup_free free
# endif /* !HAS_STRDUP */

# ifdef NEED_STRDUP_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) char *
__attribute__((nonnull(1)))
strdup(
# if __myc_require(prototypes)
	char const *aSource
# endif
	);

__MYC_END_CDECLS

# endif /* NEED_STRDUP_IMPL */

/* ----------------------------------------------------------------------------
 * 'wcsdup()'
 */

# ifndef HAS_WCSDUP
#  ifdef HAS__WCSDUP
#   define wcsdup _wcsdup
#   define wcsdup_free free
#  else
#   define NEED_WCSDUP_IMPL
#   define wcsdup_free bo_free
#  endif
# else
#  define wcsdup_free free
# endif /* !HAS_WCSDUP */

# ifdef NEED_WCSDUP_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) wchar_t *
__attribute__((nonnull(1)))
wcsdup(
# if __myc_require(prototypes)
	wchar_t const *aSource
# endif
	);

__MYC_END_CDECLS

# endif /* NEED_WCSDUP_IMPL */

__MYC_BEGIN_CDECLS

/* canonical (without locale) */
__myc_flinkage__(public,libBo) wchar_t *
__attribute__((nonnull(1)))
BoWcslwr_c(
# if __myc_require(prototypes)
	wchar_t *aSrcDst
# endif
	);

/* canonical (without locale) */
__myc_flinkage__(public,libBo) wchar_t *
__attribute__((nonnull(1)))
BoWcsupr_c(
# if __myc_require(prototypes)
	wchar_t *aSrcDst
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_LIBC_STRING__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
