/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Strings.h	(V.Sokur)
 * include/Bo/libc/Strings.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_STRINGS__
# define __BO_LIBC_STRINGS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# include <Bo/libc/String.h>

# if __myc_exists(strings,h)
#  include __MYC_LIBC_HEADER(strings.h)
# endif
# if __myc_exists(extras,h)
#  include __MYC_LIBC_HEADER(extras.h)
# endif
# if __myc_exists(string,h)
#  include __MYC_LIBC_HEADER(string.h)
# endif
# if __myc_exists(wchar,h)
#  include __MYC_LIBC_HEADER(wchar.h)
# endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Bo/BoLinkage.h>

/* ----------------------------------------------------------------------------
 * 'strcasecmp()'
 */
# pragma mark -

# ifndef HAS_STRCASECMP
#  ifdef HAS__STRICMP
#   define strcasecmp _stricmp
#  else
#  ifdef HAS_STRICMP
#   define strcasecmp stricmp
#  else
#  ifdef HAS_STRCMPI
#   define strcasecmp strcmpi
#  else
#   define NEED_STRCASECMP_IMPL
#  endif /* !HAS_STRCMPI */
#  endif /* !HAS_STRICMP */
#  endif /* !HAS__STRICMP */
# endif /* !HAS_STRCASECMP */

# ifdef NEED_STRCASECMP_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
strcasecmp(
# if __myc_require(prototypes)
	char const *aLhs,
	char const *aRhs
# endif
	);

__MYC_END_CDECLS

# endif /* NEED_STRCASECMP_IMPL */

/* ----------------------------------------------------------------------------
 * 'wcscasecmp()'
 */
# pragma mark -

# ifndef HAS_WCSCASECMP
#  ifdef HAS__WCSICMP
#   define wcscasecmp _wcsicmp
#  else
#  ifdef HAS_WCSICMP
#   define wcscasecmp wcsicmp
#  else
#  ifdef HAS_WCSCMPI
#   define wcscasecmp wcscmpi
#  else
#   define NEED_WCSCASECMP_IMPL
#  endif /* !HAS_WCSCMPI */
#  endif /* !HAS_WCSICMP */
#  endif /* !HAS__WCSICMP */
# endif /* !HAS_WCSCASECMP */

# ifdef NEED_WCSCASECMP_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
wcscasecmp(
# if __myc_require(prototypes)
	wchar_t const *aLhs,
	wchar_t const *aRhs
# endif
	);

__MYC_END_CDECLS

# endif /* NEED_WCSCASECMP_IMPL */

/* ----------------------------------------------------------------------------
 * 'strncasecmp()'
 */
# pragma mark -

# ifndef HAS_STRNCASECMP
#  ifdef HAS__STRNICMP
#   define strncasecmp _strnicmp
#  else
#  ifdef HAS_STRNICMP
#   define strncasecmp strnicmp
#  else
#  ifdef HAS_STRNCMPI
#   define strncasecmp strncmpi
#  else
#   define NEED_STRNCASECMP_IMPL
#  endif /* !HAS_STRNCMPI */
#  endif /* !HAS_STRNICMP */
#  endif /* !HAS__STRNICMP */
# endif /* !HAS_STRNCASECMP */

# ifdef NEED_STRNCASECMP_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
strncasecmp(
# if __myc_require(prototypes)
	char const *aLhs,
	char const *aRhs,
	size_t aCharc
# endif
	);

__MYC_END_CDECLS

# endif /* NEED_STRNCASECMP_IMPL */

/* ----------------------------------------------------------------------------
 * 'bzero()'
 */
# pragma mark -

# ifndef HAS_BZERO
#  define NEED_BZERO_IMPL
# endif /* !HAS_BZERO */

# ifdef NEED_BZERO_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) void
__attribute__((nonnull(1)))
bzero(
# if __myc_require(prototypes)
	bo_pointer_t aDest,
	size_t aBytec
# endif
	);

#  define bzero(aDest,aBytec) memset(aDest, 0, aBytec)

__MYC_END_CDECLS

# endif /* NEED_BZERO_IMPL */

/* ----------------------------------------------------------------------------
 * 'bcmp()'
 */
# pragma mark -

# ifndef HAS_BCMP
#  define NEED_BCMP_IMPL
# endif /* !HAS_BCMP */

# ifdef NEED_BCMP_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
bcmp(
#if __myc_require(prototypes)
	bo_const_pointer_t aLhs,
	bo_const_pointer_t aRhs,
	size_t aBytec
#endif
	);

__MYC_END_CDECLS

#  define bcmp(aLhs,aRhs,aBytec) memcmp(aLhs, aRhs, aBytec)

# endif /* NEED_BCMP_IMPL */

/* ----------------------------------------------------------------------------
 * 'bcopy()'
 */
# pragma mark -

# ifndef HAS_BCOPY
#  define NEED_BCOPY_IMPL
# endif /* !HAS_BCOPY */

# ifdef NEED_BCOPY_IMPL

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) void
__attribute__((nonnull(1,2)))
bcopy(
#if __myc_require(prototypes)
	bo_const_pointer_t aSource,
	bo_pointer_t aDest,
	size_t aBytec
#endif
	);

__MYC_END_CDECLS

#  define bcopy(aSource,aDest,aBytec) memmove(aDest, aSource, aBytec)

# endif /* NEED_BCOPY_IMPL */

/* ----------------------------------------------------------------------------
 * 'strshellcmp()', 'wcsshellcmp()'
 */
# pragma mark -

__MYC_BEGIN_CDECLS

/*
 * 1. '*': match 0 or more symbols (except '.')
 * 2. '?': match 0 or 1 symbol (except '.')
 * 3. '{' expr '}': where @expr is comma separated groups (alternates) of symbol, without wildcards
 */

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
strshellcmp(
#if __myc_require(prototypes)
	char const *aLhs,
	char const *aRhs /* template */
#endif
	);

# define strshellcmp(aLhs,aRhs) strshellcmp3(aLhs,aRhs,0)

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
wcsshellcmp(
#if __myc_require(prototypes)
	wchar_t const *aLhs,
	wchar_t const *aRhs /* template */
#endif
	);

# define wcsshellcmp(aLhs,aRhs) wcsshellcmp3(aLhs,aRhs,0)

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
strshellcmp3(
#if __myc_require(prototypes)
	char const *aLhs,
	char const *aRhs, /* template */
	int UnmatchDot __myc_defval__(0)
#endif
	);

__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
wcsshellcmp3(
#if __myc_require(prototypes)
	wchar_t const *aLhs,
	wchar_t const *aRhs, /* template */
	int UnmatchDot __myc_defval__(0)
#endif
	);

__MYC_END_CDECLS

#endif /* !__BO_LIBC_STRINGS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
