/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Time.h	(V.Sokur)
 * include/Bo/libc/Time.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LIBC_TIME__
# define __BO_LIBC_TIME__

/******************************************************************************
 * %% BeginSection: includes
 */
# pragma mark -

# include <Bo/sys/Types.h>

# if __myc_exists(time,h)
#  if defined(_ANSI_SOURCE) || defined(_POSIX_C_SOURCE)
#   ifdef _ANSI_SOURCE
#    undef _ANSI_SOURCE
#   endif
#   ifdef _POSIX_C_SOURCE
#    undef _POSIX_C_SOURCE
#   endif
#   include __MYC_LIBC_HEADER(time.h)
#   ifndef _ANSI_SOURCE
#    define _ANSI_SOURCE 1L
#   endif
#   ifndef _POSIX_C_SOURCE
#    define _POSIX_C_SOURCE 198808L
#   endif
#  else
#   include __MYC_LIBC_HEADER(time.h)
#  endif
# endif

# include <Bo/sys/Time.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -

# ifndef HAS_TIMESPEC_STRUC
struct timespec {
	tv_time_t  tv_sec;   /* seconds */
	long       tv_nsec;  /* nanoseconds */  
};
# endif


# ifndef BO_TIMEVAL_TO_TIMESPEC
#  define BO_TIMEVAL_TO_TIMESPEC(tvp, tsp) \
	do { \
		(tsp)->tv_sec = (tvp)->tv_sec; \
		(tsp)->tv_nsec = (tvp)->tv_usec * 1000; \
	} while (0)
# endif

# ifndef BO_TIMESPEC_TO_TIMEVAL
#  define BO_TIMESPEC_TO_TIMEVAL(tsp, tvp) \
	do { \
		(tvp)->tv_sec = (tsp)->tv_sec; \
		(tvp)->tv_usec = (tsp)->tv_nsec / 1000; \
	} while (0)
# endif


# define BO_TIMESPEC_Add(aSrc1, aSrc2, aDst) \
	do { \
		(aDst)->tv_sec = (aSrc1)->tv_sec + (aSrc2)->tv_sec; \
		(aDst)->tv_nsec = (aSrc1)->tv_nsec + (aSrc2)->tv_nsec; \
		if (1000000000 <= (aDst)->tv_nsec) { \
			(aDst)->tv_nsec -= 1000000000; \
			(aDst)->tv_sec++; \
		} \
	} while (0)

# define BO_TIMESPEC_Sub(aSrc1, aSrc2, aDst) \
	do { \
		(aDst)->tv_sec = (aSrc1)->tv_sec - (aSrc2)->tv_sec; \
		(aDst)->tv_nsec = (aSrc1)->tv_nsec - (aSrc2)->tv_nsec; \
		if ((aDst)->tv_nsec < 0) { \
			(aDst)->tv_nsec += 1000000000; \
			(aDst)->tv_sec--; \
		} \
	} while (0)

# define BO_TIMESPEC_Compare(_CMP, aLhs, aRhs) \
	(((aLhs)->tv_sec == (aRhs)->tv_sec) \
			? ((aLhs)->tv_nsec _CMP (aRhs)->tv_nsec) \
			: ((aLhs)->tv_sec _CMP (aRhs)->tv_sec))


# define BO_TIMESPEC_AsMsec(tsp) \
	(((bo_msec_t)(tsp)->tv_sec) * 1000 + (tsp)->tv_nsec / 1000000)

# define BO_TIMESPEC_AsUsec(tsp) \
	(((bo_usec_t)(tsp)->tv_sec) * 1000000 + (tsp)->tv_nsec / 1000)

# define BO_TIMESPEC_FromMsec(tsp, ms) \
	do { \
		(tsp)->tv_sec = (tv_time_t)((ms) / 1000); \
		(tsp)->tv_nsec = (long)(((ms) - ((bo_msec_t)(tsp)->tv_sec) * 1000) * 1000000); \
	} while (0)

# define BO_TIMESPEC_FromUsec(tsp, us) \
	do { \
		(tsp)->tv_sec = (tv_time_t)((us) / 1000000); \
		(tsp)->tv_nsec = (long)(((us) - ((bo_usec_t)(tsp)->tv_sec) * 1000000) * 1000); \
	} while (0)


# define BO_TIMESPEC_AbsFromDuration(tsp, dur) \
	do { \
		struct timeval dur2abstsTmp; \
		BoGettimeofday(&dur2abstsTmp, NULL); \
		BO_TIMEVAL_TO_TIMESPEC(&dur2abstsTmp, tsp); \
		BO_TIMESPEC_Add(dur, tsp, tsp); \
	} while (0)

# define BO_TIMESPEC_AbsFromDuration_ms(tsp, ms) \
	do { \
		struct timeval ms2abstsTmp; \
		BoGettimeofday(&ms2abstsTmp, NULL); \
		BO_TIMEVAL_TO_TIMESPEC(&ms2abstsTmp, tsp); \
		(tsp)->tv_sec += (tv_time_t)((ms) / 1000); \
		(tsp)->tv_nsec += (long)(((ms) % 1000) * 1000000); \
		if (1000000000 <= (tsp)->tv_nsec) { \
			(tsp)->tv_nsec -= 1000000000; \
			(tsp)->tv_sec++; \
		} \
	} while (0)

# define BO_TIMESPEC_AbsFromDuration_us(tsp, us) \
	do { \
		struct timeval us2abstsTmp; \
		BoGettimeofday(&us2abstsTmp, NULL); \
		BO_TIMEVAL_TO_TIMESPEC(&us2abstsTmp, tsp); \
		(tsp)->tv_sec += (tv_time_t)((us) / 1000000); \
		(tsp)->tv_nsec += (long)(((us) % 1000000) * 1000); \
		if (1000000000 <= (tsp)->tv_nsec) { \
			(tsp)->tv_nsec -= 1000000000; \
			(tsp)->tv_sec++; \
		} \
	} while (0)



/* Bo version of the standard tm structure. */
struct bo_tm {
	long       tm_usec;
	struct tm  c; /* standard C.library */
};

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

# define BoIsLeapYear(aYear) \
	( (0 == (aYear) % 4 && 0 != (aYear) % 100) || (0 == (aYear) % 400) )

__myc_flinkage__(public,libBo) bo_tick_t BoTick();
__myc_flinkage__(public,libBo) bo_tick_t BoTickpersec();
__myc_flinkage__(public,libBo) bo_tick_t BoTickpermsec();
__myc_flinkage__(public,libBo) bo_tick_t BoTickperusec();

/* compat */
# define BoIsleapyear(aYear) BoIsLeapYear(aYear)

__MYC_END_CDECLS

/* ============================================================================
 * Missing thread-safe version of standard time functions:
 * ctime_r, gmtime_r, localtime_r.
 */

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * ctime_r
 */
# pragma mark -

/* Including the trailing LF and NULL characters. */
# define BoK_STD_CTIME_BUFSIZ 26

/*
char *
ctime_r(time_t const *aClock, char aBuffer[BoK_STD_CTIME_BUFSIZ]);
 */
# ifndef HAS_CTIME_R
#  ifdef HAS_CTIME_S
#   define ctime_r(aClock, aBuffer) \
		(ctime_s(aBuffer, BoK_STD_CTIME_BUFSIZ, aClock), aBuffer)
#  else
#   define ctime_r(aClock, aBuffer) \
		(strcpy(aBuffer, ctime(aClock)), aBuffer)
#  endif
# endif /* !HAS_CTIME_R */

/* ----------------------------------------------------------------------------
 * gmtime_r
 */
# pragma mark -

/*
struct tm *
gmtime_r(time_t const *aClock, struct tm *aBuffer);
 */
# ifndef HAS_GMTIME_R
#  ifdef HAS_GMTIME_S
#   define gmtime_r(aClock, aBuffer) (gmtime_s(aBuffer, aClock), aBuffer)
#  else
#   define gmtime_r(aClock, aBuffer) (*(aBuffer) = *gmtime(aClock), aBuffer)
#  endif
# endif /* !HAS_GMTIME_R */

/* ----------------------------------------------------------------------------
 * localtime_r
 */
# pragma mark -

/*
struct tm *
localtime_r(time_t const *aClock, struct tm *aBuffer);
 */
# ifndef HAS_LOCALTIME_R
#  ifdef HAS_LOCALTIME_S
#   define localtime_r(aClock, aBuffer) (localtime_s(aBuffer, aClock), aBuffer)
#  else
#   define localtime_r(aClock, aBuffer) (*(aBuffer) = *localtime(aClock), aBuffer)
#  endif
# endif /* !HAS_LOCALTIME_R */

/* ----------------------------------------------------------------------------
 * timegm, _mkgmtime
 */
# pragma mark -

/* windows has not the timegm() function. */
# ifndef HAS_TIMEGM
#  ifdef HAS__MKGMTIME
#   define timegm(aTime) _mkgmtime(aTime)
#  endif
# endif

/* Unix has not the _mkgmtime() function. */
# ifndef HAS__MKGMTIME
#  ifdef HAS_TIMEGM
#   define _mkgmtime(aTime) timegm(aTime)
#  endif
# endif

/* ============================================================================
 * Bo version of standard time functions: time, ctime, gmtime, mkgmtime,
 * localtime, mktime (mklocaltime).
 */

/* ----------------------------------------------------------------------------
 * BoTime
 */
# pragma mark -

/* Since midnight (0 hour, 0 minutes, 0 seconds), 1 January, 1970, UTC,
   without including leap seconds. */
__myc_flinkage__(public,libBo) bo_utime_t
BoTime(
# if __myc_require(prototypes)
	bo_utime_t *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define utime(aBuffer) BoTime(aBuffer, NULL)

/* ----------------------------------------------------------------------------
 * BoCtime
 */
# pragma mark -

/* Including the trailing LF and NULL characters. */
# define BoK_CTIME_BUFSIZ 33

__myc_flinkage__(public,libBo) char *
BoCtime(
# if __myc_require(prototypes)
	bo_utime_t const *aTime,
	char aBuffer[/* BoK_CTIME_BUFSIZ */],
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define cutime_r(aTime, aBuffer) BoCtime(aTime, aBuffer, NULL)
# define cutime(aTime) BoCtime(aTime, NULL, NULL)

/* ----------------------------------------------------------------------------
 * BoGmtime
 */
# pragma mark -

__myc_flinkage__(public,libBo) struct bo_tm *
BoGmtime(
# if __myc_require(prototypes)
	bo_utime_t const *aTime,
	struct bo_tm *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define gmutime_r(aTime, aBuffer) BoGmtime(aTime, aBuffer, NULL)
# define gmutime(aTime) BoGmtime(aTime, NULL, NULL)

/* ----------------------------------------------------------------------------
 * BoMkgmtime
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_utime_t
BoMkgmtime(
# if __myc_require(prototypes)
	struct bo_tm *aTime,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define mkgmutime(aTime) BoMkgmtime(aTime, NULL)

/* ----------------------------------------------------------------------------
 * BoLocaltime
 */
# pragma mark -

__myc_flinkage__(public,libBo) struct bo_tm *
BoLocaltime(
# if __myc_require(prototypes)
	bo_utime_t const *aTime,
	struct bo_tm *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define localutime_r(aTime, aBuffer) BoLocaltime(aTime, aBuffer, NULL)
# define localutime(aTime) BoLocaltime(aTime, NULL, NULL)

/* ----------------------------------------------------------------------------
 * BoMklocaltime
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_utime_t
BoMklocaltime(
# if __myc_require(prototypes)
	struct bo_tm *aTime,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define mklocalutime(aTime) BoMklocaltime(aTime, NULL)
# define mkutime(aTime) BoMklocaltime(aTime, NULL)

__MYC_END_CDECLS

/* ============================================================================
 * Bo version of standard sleep functions: sleep, msleep, usleep, nanosleep.
 * @seealso: <Bo/sys/Plafstd.h>
 */

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoSleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoSleep(
# if __myc_require(prototypes)
	bo_sec_t aRequest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_SLEEP
#  define NEED_SLEEP_IMPL

#  undef sleep
__myc_flinkage__(public,libBo) int
sleep(
# if __myc_require(prototypes)
	seconds_t
# endif
	);

#  define sleep(aRequest) BoSleep((bo_sec_t)aRequest, NULL)

# endif /* !HAS_SLEEP */

/* ----------------------------------------------------------------------------
 * BoMsleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoMsleep(
# if __myc_require(prototypes)
	bo_msec_t aRequest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_MSLEEP
#  define NEED_MSLEEP_IMPL

#  undef msleep
__myc_flinkage__(public,libBo) int
msleep(
# if __myc_require(prototypes)
	mseconds_t
# endif
	);

#  define msleep(aRequest) BoMsleep((bo_msec_t)aRequest, NULL)

# endif /* !HAS_MSLEEP */

/* ----------------------------------------------------------------------------
 * BoUsleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoUsleep(
# if __myc_require(prototypes)
	bo_usec_t aRequest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_USLEEP
#  define NEED_USLEEP_IMPL

#  undef usleep
__myc_flinkage__(public,libBo) int
usleep(
# if __myc_require(prototypes)
	useconds_t
# endif
	);

#  define usleep(aRequest) BoUsleep((bo_usec_t)aRequest, NULL)

# endif /* !HAS_USLEEP */

__MYC_END_CDECLS

#endif /* !__BO_LIBC_TIME__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
