/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Atomic.h	(V.Sokur)
 * include/Bo/machine/amd64/gcc/_Atomic.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_AMD64_GCC__ATOMIC__
# define __BO_AMD64_GCC__ATOMIC__

/******************************************************************************
 * %% BeginSection: includes
 */

/******************************************************************************
 * %% BeginSection: definitions
 */

/*
 * @todo: fixme (I don't know assembler)
 */

# ifndef MPLOCKED
#  if defined(SMP)
#   define MPLOCKED " lock "
#  else
#   define MPLOCKED
#  endif
# endif /* MPLOCKED */

# ifdef mb
#  define atomic_memory_barrier() mb()
# else
//#  define atomic_memory_barrier() __asm __volatile__(/*MPLOCKED*/ "mfence;" ::: "memory")
#  define atomic_memory_barrier() __asm __volatile__(MPLOCKED "addq $0,(%%rsp) \n\t" ::: "memory")
# endif
# ifdef wmb
#  define atomic_write_memory_barrier() wmb()
# else
//#  define atomic_write_memory_barrier() __asm __volatile__(/*MPLOCKED*/ "sfence;" ::: "memory")
#  define atomic_write_memory_barrier() atomic_memory_barrier()
# endif
# ifdef rmb
#  define atomic_read_memory_barrier() rmb()
# else
//#  define atomic_read_memory_barrier() __asm __volatile__(/*MPLOCKED*/ "lfence;" ::: "memory")
#  define atomic_read_memory_barrier() atomic_memory_barrier()
# endif

/* ----------------------------------------------------------------------------
 * atomic_readandclear_xxx: (oldValue = *(aPtr), *(aPtr) = 0, oldValue)
 */

# define _BO_ATOMIC_READANDCLEAR(_Tp, _Op) \
__myc_extern_inline__ _Tp \
__myc_xconcat(atomic_readandclear_,_Tp)(volatile unsigned _Tp *aPtr) \
{ \
	unsigned _Tp retval; \
 \
	retval = 0; \
	__asm __volatile__( \
		_Op \
		: "+r" (retval), /* 0 (result) */ \
		  "=m" (*aPtr)   /* 1 */ \
		: "m" (*aPtr) \
	); \
	return (retval); \
} \
struct __hack

# ifndef HAS_ATOMIC_READANDCLEAR_CHAR
_BO_ATOMIC_READANDCLEAR(char, "xchgb %b1,%0;");
# endif /* HAS_ATOMIC_READANDCLEAR_CHAR */

# ifndef HAS_ATOMIC_READANDCLEAR_SHORT
_BO_ATOMIC_READANDCLEAR(short, "xchgw %w1,%0;");
# endif /* HAS_ATOMIC_READANDCLEAR_SHORT */

# ifndef HAS_ATOMIC_READANDCLEAR_INT
_BO_ATOMIC_READANDCLEAR(int, "xchgl %1,%0;");
# endif /* HAS_ATOMIC_READANDCLEAR_INT */

# ifndef HAS_ATOMIC_READANDCLEAR_LONG
_BO_ATOMIC_READANDCLEAR(long, "xchgq %1,%0;");
# endif /* HAS_ATOMIC_READANDCLEAR_LONG */

# ifndef atomic_readandclear_8
#  define atomic_readandclear_8 atomic_readandclear_char
#  define atomic_readandclear_acq_8 atomic_readandclear_char
#  define atomic_readandclear_rel_8 atomic_readandclear_char
# endif
# ifndef atomic_readandclear_16
#  define atomic_readandclear_16 atomic_readandclear_short
#  define atomic_readandclear_acq_16 atomic_readandclear_short
#  define atomic_readandclear_rel_16 atomic_readandclear_short
# endif
# ifndef atomic_readandclear_32
#  define atomic_readandclear_32 atomic_readandclear_int
#  define atomic_readandclear_acq_32 atomic_readandclear_int
#  define atomic_readandclear_rel_32 atomic_readandclear_int
# endif
# ifndef atomic_readandclear_64
#  define atomic_readandclear_64 atomic_readandclear_long
#  define atomic_readandclear_acq_64 atomic_readandclear_long
#  define atomic_readandclear_rel_64 atomic_readandclear_long
# endif

/* ----------------------------------------------------------------------------
 * atomic_cmpset_xxx: ((*(aDst) == (aComparand)) ? (*(aDst) = (aValue), !0) : 0)
 */

# define _BO_ATOMIC_CMPSET(_Tp, _Op) \
__myc_extern_inline__ int \
__myc_xconcat(atomic_cmpset_,_Tp)(volatile unsigned _Tp *aDst, unsigned _Tp aComparand, unsigned _Tp aValue) \
{ \
	unsigned char retval; \
 \
	__asm __volatile__( \
		MPLOCKED \
		_Op \
		"	sete %0; " \
		"1:          " \
		: "=a" (retval),    /* 0 */ \
		  "=m" (*aDst)      /* 1 */ \
		: "r" (aValue),     /* 2 */ \
		  "a" (aComparand), /* 3 */ \
		  "m" (*aDst)       /* 4 */ \
		: "memory" \
	); \
 \
	return (retval); \
} \
struct __hack

# ifndef HAS_ATOMIC_CMPSET_CHAR
_BO_ATOMIC_CMPSET(char, "cmpxchgb %b2,%1;");
# endif /* HAS_ATOMIC_CMPSET_CHAR */

# ifndef HAS_ATOMIC_CMPSET_SHORT
_BO_ATOMIC_CMPSET(short, "cmpxchgw %w2,%1;");
# endif /* HAS_ATOMIC_CMPSET_SHORT */

# ifndef HAS_ATOMIC_CMPSET_INT
_BO_ATOMIC_CMPSET(int, "cmpxchgl %2,%1;");
# endif /* HAS_ATOMIC_CMPSET_INT */

# ifndef HAS_ATOMIC_CMPSET_LONG
_BO_ATOMIC_CMPSET(long, "cmpxchgq %2,%1;");
# endif /* HAS_ATOMIC_CMPSET_LONG */

# ifndef atomic_cmpset_8
#  define atomic_cmpset_8 atomic_cmpset_char
#  define atomic_cmpset_acq_8 atomic_cmpset_char
#  define atomic_cmpset_rel_8 atomic_cmpset_char
# endif
# ifndef atomic_cmpset_16
#  define atomic_cmpset_16 atomic_cmpset_short
#  define atomic_cmpset_acq_16 atomic_cmpset_short
#  define atomic_cmpset_rel_16 atomic_cmpset_short
# endif
# ifndef atomic_cmpset_32
#  define atomic_cmpset_32 atomic_cmpset_int
#  define atomic_cmpset_acq_32 atomic_cmpset_int
#  define atomic_cmpset_rel_32 atomic_cmpset_int
# endif
# ifndef atomic_cmpset_64
#  define atomic_cmpset_64 atomic_cmpset_long
#  define atomic_cmpset_acq_64 atomic_cmpset_long
#  define atomic_cmpset_rel_64 atomic_cmpset_long
# endif

/* ----------------------------------------------------------------------------
 * atomic_fetchadd_xxx: (oldValue = *(aPtr), *(aPtr) += (aValue), oldValue)
 */

# define _BO_ATOMIC_FETCHADD(_Tp, _Op) \
__myc_extern_inline__ _Tp \
__myc_xconcat(atomic_fetchadd_,_Tp)(volatile unsigned _Tp *aPtr, unsigned _Tp aValue) \
{ \
	__asm __volatile__( \
		MPLOCKED \
		_Op \
		: "+r" (aValue), /* 0 (result) */ \
		  "=m" (*aPtr)   /* 1 */ \
		: "m" (*aPtr)    /* 2 */ \
	); \
	return (aValue); \
} \
struct __hack

/* @todo: fixme (xaddb) */
# ifndef HAS_ATOMIC_FETCHADD_CHAR
_BO_ATOMIC_FETCHADD(char, "xaddb %b0,%1;");
# endif /* HAS_ATOMIC_FETCHADD_CHAR */

/* @todo: fixme (xaddw) */
# ifndef HAS_ATOMIC_FETCHADD_SHORT
_BO_ATOMIC_FETCHADD(short, "xaddw %w0,%1;");
# endif /* HAS_ATOMIC_FETCHADD_SHORT */

# ifndef HAS_ATOMIC_FETCHADD_INT
_BO_ATOMIC_FETCHADD(int, "xaddl %0,%1;");
# endif /* HAS_ATOMIC_FETCHADD_INT */

# ifndef HAS_ATOMIC_FETCHADD_LONG
_BO_ATOMIC_FETCHADD(long, "xaddq %0,%1;");
# endif /* HAS_ATOMIC_FETCHADD_LONG */

# ifndef atomic_fetchadd_8
#  define atomic_fetchadd_8 atomic_fetchadd_char
#  define atomic_fetchadd_acq_8 atomic_fetchadd_char
#  define atomic_fetchadd_rel_8 atomic_fetchadd_char
# endif
# ifndef atomic_fetchadd_16
#  define atomic_fetchadd_16 atomic_fetchadd_short
#  define atomic_fetchadd_acq_16 atomic_fetchadd_short
#  define atomic_fetchadd_rel_16 atomic_fetchadd_short
# endif
# ifndef atomic_fetchadd_32
#  define atomic_fetchadd_32 atomic_fetchadd_int
#  define atomic_fetchadd_acq_32 atomic_fetchadd_int
#  define atomic_fetchadd_rel_32 atomic_fetchadd_int
# endif
# ifndef atomic_fetchadd_64
#  define atomic_fetchadd_64 atomic_fetchadd_long
#  define atomic_fetchadd_acq_64 atomic_fetchadd_long
#  define atomic_fetchadd_rel_64 atomic_fetchadd_long
# endif

#endif /* !__BO_AMD64_GCC__ATOMIC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
