/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2014 T.Zakharchenko
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Atomic.h	(T.Zakharchenko)
 * include/Bo/machine/arm/gcc/_Atomic.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_ARM_GCC__ATOMIC__
# define __BO_ARM_GCC__ATOMIC__

/******************************************************************************
 * %% BeginSection: includes
 */

/******************************************************************************
 * %% BeginSection: definitions
 */

/*
 * @todo: fixme (I don't know assembler)
 */

# ifndef MPLOCKED
#  if defined(SMP)
#   define MPLOCKED " lock "
#  else
#   define MPLOCKED
#  endif
# endif /* MPLOCKED */

/*
 * Memory barriers
 */

# if defined (_M_ARM_7) || defined(_M_ARM_8)
#  define isb() __asm __volatile("isb" : : : "memory")
#  define dsb() __asm __volatile("dsb" : : : "memory")
#  define dmb() __asm __volatile("dmb" : : : "memory")
# elif defined (_M_ARM_6)
#  define isb() __asm __volatile("mcr p15, 0, %0, c7, c5, 4" : : "r" (0) : "memory")
#  define dsb() __asm __volatile("mcr p15, 0, %0, c7, c10, 4" : : "r" (0) : "memory")
#  define dmb() __asm __volatile("mcr p15, 0, %0, c7, c10, 5" : : "r" (0) : "memory")
# else
#  define isb() __asm __volatile("mcr p15, 0, %0, c7, c5, 4" : : "r" (0) : "memory")
#  define dsb() __asm __volatile("mcr p15, 0, %0, c7, c10, 4" : : "r" (0) : "memory")
#  define dmb() dsb()
# endif

# ifdef mb
#  define atomic_memory_barrier() mb()
# else
#  define atomic_memory_barrier() dmb()
# endif
# ifdef wmb
#  define atomic_write_memory_barrier() wmb()
# else
#  define atomic_write_memory_barrier() dmb()
# endif
# ifdef rmb
#  define atomic_read_memory_barrier() rmb()
# else
#  define atomic_read_memory_barrier() dmb()
# endif

/* ----------------------------------------------------------------------------
 * atomic_readandclear_xxx: (oldValue = *(aPtr), *(aPtr) = 0, oldValue)
 */

# ifndef HAS_ATOMIC_READANDCLEAR_CHAR
__myc_extern_inline__ unsigned char
atomic_readandclear_char(volatile unsigned char *p)
{
	unsigned char ret, tmp = 0, tmp2 = 0;
	__asm __volatile("1: ldrexb %0, [%3]\n"
	"mov %1, #0\n"
	"strexb %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "=r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p) : : "cc", "memory");
	return (ret);
}
# endif /* HAS_ATOMIC_READANDCLEAR_CHAR */

# ifndef HAS_ATOMIC_READANDCLEAR_SHORT
__myc_extern_inline__ unsigned short
atomic_readandclear_short(volatile unsigned short *p)
{
	unsigned short ret, tmp = 0, tmp2 = 0;
	__asm __volatile("1: ldrexh %0, [%3]\n"
	"mov %1, #0\n"
	"strexh %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "=r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p) : : "cc", "memory");
	return (ret);
}
# endif /* HAS_ATOMIC_READANDCLEAR_SHORT */

# ifndef HAS_ATOMIC_READANDCLEAR_INT
__myc_extern_inline__ unsigned int
atomic_readandclear_int(volatile unsigned int *p)
{
	unsigned int ret, tmp = 0, tmp2 = 0;
	__asm __volatile("1: ldrex %0, [%3]\n"
	"mov %1, #0\n"
	"strex %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "=r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p) : : "cc", "memory");
	return (ret);
}
# endif /* HAS_ATOMIC_READANDCLEAR_INT */

# ifndef HAS_ATOMIC_READANDCLEAR_LONG
__myc_extern_inline__ unsigned long
atomic_readandclear_long(volatile unsigned long *p)
{
	unsigned long ret, tmp = 0, tmp2 = 0;
	__asm __volatile("1: ldrex %0, [%3]\n"
	"mov %1, #0\n"
	"strex %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "=r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p) : : "cc", "memory");
	return (ret);
}
# endif /* HAS_ATOMIC_READANDCLEAR_LONG */

# ifndef atomic_readandclear_8
#  define atomic_readandclear_8 atomic_readandclear_char
#  define atomic_readandclear_acq_8 atomic_readandclear_char
#  define atomic_readandclear_rel_8 atomic_readandclear_char
# endif
# ifndef atomic_readandclear_16
#  define atomic_readandclear_16 atomic_readandclear_short
#  define atomic_readandclear_acq_16 atomic_readandclear_short
#  define atomic_readandclear_rel_16 atomic_readandclear_short
# endif
# ifndef atomic_readandclear_32
#  define atomic_readandclear_32 atomic_readandclear_int
#  define atomic_readandclear_acq_32 atomic_readandclear_int
#  define atomic_readandclear_rel_32 atomic_readandclear_int
# endif

/* ----------------------------------------------------------------------------
 * atomic_cmpset_xxx: ((*(aDst) == (aComparand)) ? (*(aDst) = (aValue), !0) : 0)
 */

# ifndef HAS_ATOMIC_CMPSET_CHAR
__myc_extern_inline__ unsigned char
atomic_cmpset_char(volatile unsigned char *p, unsigned char cmpval,
	unsigned char newval)
{
	unsigned char ret;
	__asm __volatile("1: ldrexb %0, [%1]\n"
	"cmp %0, %2\n"
	"itt ne\n"
	"movne %0, #0\n"
	"bne 2f\n"
	"strexb %0, %3, [%1]\n"
	"cmp %0, #0\n"
	"ite eq\n"
	"moveq %0, #1\n"
	"bne 1b\n"
	"2:"
	: "=&r" (ret)
	,"+r" (p), "+r" (cmpval), "+r" (newval) : : "cc",
	"memory");
	return (ret);
}
# endif /* HAS_ATOMIC_CMPSET_CHAR */

# ifndef HAS_ATOMIC_CMPSET_SHORT
__myc_extern_inline__ unsigned short
atomic_cmpset_short(volatile unsigned short *p, unsigned short cmpval,
	unsigned short newval)
{
	unsigned short ret;
	__asm __volatile("1: ldrexh %0, [%1]\n"
	"cmp %0, %2\n"
	"itt ne\n"
	"movne %0, #0\n"
	"bne 2f\n"
	"strexh %0, %3, [%1]\n"
	"cmp %0, #0\n"
	"ite eq\n"
	"moveq %0, #1\n"
	"bne 1b\n"
	"2:"
	: "=&r" (ret)
	,"+r" (p), "+r" (cmpval), "+r" (newval) : : "cc",
	"memory");
	return (ret);
}
# endif /* HAS_ATOMIC_CMPSET_SHORT */

# ifndef HAS_ATOMIC_CMPSET_INT
__myc_extern_inline__ unsigned int
atomic_cmpset_int(volatile unsigned int *p, unsigned int cmpval,
	unsigned int newval)
{
	unsigned int ret;
	//printf("ARMv8 compare-set\n");
	__asm __volatile("1: ldrex %0, [%1]\n"
	"cmp %0, %2\n"
	"itt ne\n"
	"movne %0, #0\n"
	"bne 2f\n"
	"strex %0, %3, [%1]\n"
	"cmp %0, #0\n"
	"ite eq\n"
	"moveq %0, #1\n"
	"bne 1b\n"
	"2:"
	: "=&r" (ret)
	,"+r" (p), "+r" (cmpval), "+r" (newval) : : "cc",
	"memory");
	return (ret);
}
# endif /* HAS_ATOMIC_CMPSET_INT */

# ifndef HAS_ATOMIC_CMPSET_LONG
__myc_extern_inline__ unsigned long
atomic_cmpset_long(volatile unsigned long *p, unsigned long cmpval,
	unsigned long newval)
{
	unsigned long ret;
	__asm __volatile("1: ldrex %0, [%1]\n"
	"cmp %0, %2\n"
	"itt ne\n"
	"movne %0, #0\n"
	"bne 2f\n"
	"strex %0, %3, [%1]\n"
	"cmp %0, #0\n"
	"ite eq\n"
	"moveq %0, #1\n"
	"bne 1b\n"
	"2:"
	: "=&r" (ret)
	,"+r" (p), "+r" (cmpval), "+r" (newval) : : "cc",
	"memory");
	return (ret);
}
# endif /* HAS_ATOMIC_CMPSET_LONG */

# ifndef atomic_cmpset_8
#  define atomic_cmpset_8 atomic_cmpset_char
#  define atomic_cmpset_acq_8 atomic_cmpset_char
#  define atomic_cmpset_rel_8 atomic_cmpset_char
# endif
# ifndef atomic_cmpset_16
#  define atomic_cmpset_16 atomic_cmpset_short
#  define atomic_cmpset_acq_16 atomic_cmpset_short
#  define atomic_cmpset_rel_16 atomic_cmpset_short
# endif
# ifndef atomic_cmpset_32
#  define atomic_cmpset_32 atomic_cmpset_int
#  define atomic_cmpset_acq_32 atomic_cmpset_int
#  define atomic_cmpset_rel_32 atomic_cmpset_int
# endif

/* ----------------------------------------------------------------------------
 * atomic_fetchadd_xxx: (oldValue = *(aPtr), *(aPtr) += (aValue), oldValue)
 */

/* @todo: fixme (xaddb) */
# ifndef HAS_ATOMIC_FETCHADD_CHAR
__myc_extern_inline__ unsigned char
atomic_fetchadd_char(volatile unsigned char *p, unsigned char val)
{
	unsigned char tmp = 0, tmp2 = 0, ret = 0;
	__asm __volatile("1: ldrexb %0, [%3]\n"
	"uadd8 %1, %0, %4\n"
	"strexb %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "+r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p), "+r" (val) : : "cc", "memory");
	return (ret);
}
# endif /* HAS_ATOMIC_FETCHADD_CHAR */

/* @todo: fixme (xaddw) */
# ifndef HAS_ATOMIC_FETCHADD_SHORT
__myc_extern_inline__ unsigned short
atomic_fetchadd_short(volatile unsigned short *p, unsigned short val)
{
	unsigned short tmp = 0, tmp2 = 0, ret = 0;
	__asm __volatile("1: ldrexh %0, [%3]\n"
	"uadd16 %1, %0, %4\n"
	"strexh %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "+r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p), "+r" (val) : : "cc", "memory");
	return (ret);
}
# endif /* HAS_ATOMIC_FETCHADD_SHORT */

# ifndef HAS_ATOMIC_FETCHADD_INT
__myc_extern_inline__ unsigned int
atomic_fetchadd_int(volatile unsigned int *p, unsigned int val)
{
	unsigned int tmp = 0, tmp2 = 0, ret = 0;
	__asm __volatile("1: ldrex %0, [%3]\n"
	"add %1, %0, %4\n"
	"strex %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "+r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p), "+r" (val) : : "cc", "memory");
	return (ret);
}
# endif /* HAS_ATOMIC_FETCHADD_INT */

# ifndef HAS_ATOMIC_FETCHADD_LONG
__myc_extern_inline__ unsigned long
atomic_fetchadd_long(volatile unsigned long *p, unsigned long val)
{
	unsigned long tmp = 0, tmp2 = 0, ret = 0;
	__asm __volatile("1: ldrex %0, [%3]\n"
	"add %1, %0, %4\n"
	"strex %2, %1, [%3]\n"
	"cmp %2, #0\n"
	"it ne\n"
	"bne 1b\n"
	: "+r" (ret), "=&r" (tmp), "+r" (tmp2)
	,"+r" (p), "+r" (val) : : "cc", "memory");
return (ret);
}
# endif /* HAS_ATOMIC_FETCHADD_LONG */

# ifndef atomic_fetchadd_8
#  define atomic_fetchadd_8 atomic_fetchadd_char
#  define atomic_fetchadd_acq_8 atomic_fetchadd_char
#  define atomic_fetchadd_rel_8 atomic_fetchadd_char
# endif
# ifndef atomic_fetchadd_16
#  define atomic_fetchadd_16 atomic_fetchadd_short
#  define atomic_fetchadd_acq_16 atomic_fetchadd_short
#  define atomic_fetchadd_rel_16 atomic_fetchadd_short
# endif
# ifndef atomic_fetchadd_32
#  define atomic_fetchadd_32 atomic_fetchadd_int
#  define atomic_fetchadd_acq_32 atomic_fetchadd_int
#  define atomic_fetchadd_rel_32 atomic_fetchadd_int
# endif

#endif /* !__BO_ARM_GCC__ATOMIC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
