/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Byteorder.h	(N.Panasiuk)
 * include/Bo/machine/i386/msc/_Byteorder.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_I386_MSC__BYTEORDER__
# define __BO_I386_MSC__BYTEORDER__

/******************************************************************************
 * %% BeginSection: includes
 */

/******************************************************************************
 * %% BeginSection: definitions
 */

/*
 * Generic byte/word/dword swapping functions
 */

__myc_extern_inline__ u_int16_t
BoBswapi16(u_int16_t aData)
{
	__asm {
		mov ax, word ptr [aData]
		xchg ah, al
		mov word ptr [aData], ax
	}
	return (aData);
}

__myc_extern_inline__ u_int32_t
BoBswapi32(u_int32_t aData)
{
	__asm {
		mov eax, dword ptr [aData]
		bswap eax
		mov dword ptr [aData], eax
	}
	return (aData);
}

__myc_extern_inline__ u_int32_t
BoWswapi32(u_int32_t aData)
{
	__asm {
		mov eax, dword ptr [aData]
		ror eax, 16
		mov dword ptr [aData], eax
	}
	return (aData);
}

__myc_extern_inline__ u_int64_t
BoBswapi64(u_int64_t aData)
{
	__asm {
		mov eax, dword ptr [aData]
		bswap eax
		xchg eax, dword ptr [aData + 4]
		bswap eax
		mov dword ptr [aData], eax
	}
	return (aData);
}

__myc_extern_inline__ u_int64_t
BoWswapi64(u_int64_t aData)
{
	__asm {
		mov eax, dword ptr [aData]
		ror eax, 16
		xchg eax, dword ptr [aData + 4]
		ror eax, 16
		mov dword ptr [aData], eax
	}
	return (aData);
}

__myc_extern_inline__ u_int64_t
BoDswapi64(u_int64_t aData)
{
	__asm {
		mov eax, dword ptr [aData]
		xchg eax, dword ptr [aData + 4]
		mov dword ptr [aData], eax
	}
	return (aData);
}

/*
 * Functions for byte/word/dword reversed loads
 */

__myc_extern_inline__ u_int16_t
BoLoadi16bsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int16_t result;

	result = *(volatile u_int16_t *)((u_intptr_t)aBase + aByteOffset);
	return BoBswapi16(result);
}

__myc_extern_inline__ u_int32_t
BoLoadi32bsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int32_t result;

	result = *(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset);
	return BoBswapi32(result);
}

__myc_extern_inline__ u_int32_t
BoLoadi32wsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int32_t result;

	result = *(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset);
	return BoWswapi32(result);
}

__myc_extern_inline__ u_int64_t
BoLoadi64bsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int64_t result;

	result = *(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
	return BoBswapi64(result);
}

__myc_extern_inline__ u_int64_t
BoLoadi64wsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int64_t result;

	result = *(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
	return BoWswapi64(result);
}

__myc_extern_inline__ u_int64_t
BoLoadi64dsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int64_t result;

	result = *(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
	return BoDswapi64(result);
}

/*
 * Functions for byte/word/dword reversed stores
 */

__myc_extern_inline__ void
BoStorei16bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int16_t aData)
{
	*(volatile u_int16_t *)((u_intptr_t)aBase + aByteOffset) = BoBswapi16(aData);
}

__myc_extern_inline__ void
BoStorei32bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int32_t aData)
{
	*(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset) = BoBswapi32(aData);
}

__myc_extern_inline__ void
BoStorei32wsw(volatile void *aBase, u_intptr_t aByteOffset, u_int32_t aData)
{
	*(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset) = BoWswapi32(aData);
}

__myc_extern_inline__ void
BoStorei64bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData)
{
	*(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset) = BoBswapi64(aData);
}

__myc_extern_inline__ void
BoStorei64wsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData)
{
	*(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset) = BoWswapi64(aData);
}

__myc_extern_inline__ void
BoStorei64dsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData)
{
	*(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset) = BoDswapi64(aData);
}

#endif /* !__BO_I386_MSC__BYTEORDER__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
