/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Funcjmp.h	(V.Sokur)
 * include/Bo/machine/i386/msc/_Funcjmp.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_I386_MSC__FUNCJMP__
# define __BO_I386_MSC__FUNCJMP__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# include <Bo/libc/Limits.h>
	/* ULONG_MAX */

/******************************************************************************
 * %% BeginSection: definitions
 */

# define BO_USES_FUNCJMP \
	struct bo_funcjmpbuf *funcjmpbufHead = NULL

# define BO_FUNCJMPBUF_(aVar) \
	struct bo_funcjmpbuf aVar = { funcjmpbufHead, 0 }


struct bo_funcjmpbuf {
	struct bo_funcjmpbuf *next;
	DWORD                 label1,
	                      label2;
	DWORD                 next_chain;
	DWORD                 retaddr;
};

# define BO_INITFUNCJMP(aVarRef,aTEXT) \
	do { \
		__asm { \
			__asm push __myc_xconcat3(_L_funcjmpbuf_,aTEXT,1) \
			__asm pop dword ptr [aVarRef.label1] \
			__asm push __myc_xconcat3(_L_funcjmpbuf_,aTEXT,2) \
			__asm pop dword ptr [aVarRef.label2] \
		}; \
		funcjmpbufHead = &aVarRef; \
	} while (0)

# define BO_SETFUNCJMP(aTEXT) \
	{ \
		DWORD funcjmpAddr, funcjmpNextChain; \
		funcjmpAddr = funcjmpbufHead->label2; \
	__myc_xconcat3(_L_funcjmpbuf_,aTEXT,1):

# define BO_ENDFUNCJMP(aTEXT) \
		if ((funcjmpNextChain = funcjmpbufHead->next_chain)) { \
			/* funcjmpbufHead->next_chain = */ --funcjmpNextChain; \
			funcjmpAddr = funcjmpbufHead->retaddr; \
			funcjmpbufHead = funcjmpbufHead->next; \
			if (funcjmpNextChain && funcjmpbufHead) { \
				funcjmpbufHead->next_chain = funcjmpNextChain; \
				funcjmpbufHead->retaddr = funcjmpAddr; \
				funcjmpAddr = funcjmpbufHead->label1; \
			} \
		} \
		__asm { jmp funcjmpAddr }; \
	__myc_xconcat3(_L_funcjmpbuf_,aTEXT,2): \
		funcjmpbufHead = funcjmpbufHead->next; \
	}

# define BO_FUNCJMP_EXIT(/* {...} */ aPredicate) \
	BO_FUNCJMP_BREAK(ULONG_MAX,aPredicate)

# define BO_FUNCJMP_BREAK(aNBlocks,/* {...} */ aPredicate) \
	{ \
		DWORD funcjmpbreakAddr; \
		funcjmpbufHead->next_chain = (aNBlocks); \
		__asm { \
			__asm push __myc_xconcat(_L_funcjmp_,__LINE__) \
			__asm pop dword ptr [funcjmpbreakAddr] \
		}; \
		funcjmpbufHead->retaddr = funcjmpbreakAddr; \
		funcjmpbreakAddr = funcjmpbufHead->label1; \
		__asm { jmp funcjmpbreakAddr }; \
	__myc_xconcat(_L_funcjmp_,__LINE__): \
		aPredicate; \
	}; ((void)0)

#endif /* !__BO_I386_MSC__FUNCJMP__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
