/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Byteorder.h	(N.Panasiuk)
 * include/Bo/machine/i386/gcc/_Byteorder.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_PPC_GCC__BYTEORDER__
# define __BO_PPC_GCC__BYTEORDER__

/******************************************************************************
 * %% BeginSection: includes
 */

/******************************************************************************
 * %% BeginSection: definitions
 */

/*
 * Functions for byte/word/dword reversed loads
 */

__myc_extern_inline__ u_int16_t
BoLoadi16bsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int16_t result;
	volatile u_int16_t *addr;

	addr = (volatile u_int16_t *)((u_intptr_t)aBase + aByteOffset);
	__asm__ ("lhbrx %0, %2, %1"
             : "=r" (result)
             : "r"  (aBase), "bO" (aByteOffset), "m" (*addr));
	return (result);
}

__myc_extern_inline__ u_int32_t
BoLoadi32bsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	u_int32_t result;
	volatile u_int32_t *addr;

	addr = (volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset);
	__asm__ ("lwbrx %0, %2, %1"
             : "=r" (result)
             : "r"  (aBase), "bO" (aByteOffset), "m" (*addr));
	return (result);
}

__myc_extern_inline__ u_int32_t
BoLoadi32wsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	/* @todo: redesign */
	u_int32_t result;

	result = *(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset);
	return BoWswapi32c(result);
}

__myc_extern_inline__ u_int64_t
BoLoadi64bsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	volatile u_int64_t *addr;
	union {
		u_int64_t u64;
		u_int32_t u32[2];
	} u;

	addr = (volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
	__asm__ ("lwbrx %0, %3, %2\n\t"
             "lwbrx %1, %4, %2"
             : "=&r" (u.u32[1]), "=r" (u.u32[0])
             : "r"  (aBase), "bO" (aByteOffset), "b" (aByteOffset + 4), "m" (*addr));
    return (u.u64);
}

__myc_extern_inline__ u_int64_t
BoLoadi64wsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	/* @todo: redesign */
	u_int64_t result;

	result = *(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
	return BoWswapi64c(result);
}

__myc_extern_inline__ u_int64_t
BoLoadi64dsw(const volatile void *aBase, u_intptr_t aByteOffset)
{
	/* @todo: redesign */
	u_int64_t result;

	result = *(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
	return BoDswapi64c(result);
}

/*
 * Functions for byte/word/dword reversed stores
 */

__myc_extern_inline__ void
BoStorei16bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int16_t aData)
{
	volatile u_int16_t *addr;
	
	addr = (volatile u_int16_t *)((u_intptr_t)aBase + aByteOffset);
	__asm__ ("sthbrx %1, %3, %2"
             : "=m" (*addr)
             : "r" (aData), "r" (aBase), "bO" (aByteOffset));
}

__myc_extern_inline__ void
BoStorei32bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int32_t aData)
{
	volatile u_int32_t *addr;
	
	addr = (volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset);
	__asm__ ("stwbrx %1, %3, %2"
             : "=m" (*addr)
             : "r" (aData), "r" (aBase), "bO" (aByteOffset));
}

__myc_extern_inline__ void
BoStorei32wsw(volatile void *aBase, u_intptr_t aByteOffset, u_int32_t aData)
{
	/* @todo: redesign */
	*(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset) = BoWswapi32c(aData);
}

__myc_extern_inline__ void
BoStorei64bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData)
{
	volatile u_int64_t *addr;
	u_int32_t hi, lo;

	addr = (volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
	hi = aData >> 32;
	lo = aData & 0xffffffff;
    __asm__ ("stwbrx %1, %4, %3\n\t"
             "stwbrx %2, %5, %3"
             : "=m" (*addr)
             : "r" (lo), "r" (hi), "r" (aBase), "bO" (aByteOffset), "b" (aByteOffset + 4));
}

__myc_extern_inline__ void
BoStorei64wsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData)
{
	/* @todo: redesign */
	*(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset) = BoWswapi64c(aData);
}

__myc_extern_inline__ void
BoStorei64dsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData)
{
	/* @todo: redesign */
	*(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset) = BoDswapi64c(aData);
}

/*
 * Generic byte/word/dword swapping functions
 */

__myc_extern_inline__ u_int16_t
BoBswapi16(u_int16_t aData)
{
	return BoLoadi16bsw(&aData, 0);
}

__myc_extern_inline__ u_int32_t
BoBswapi32(u_int32_t aData)
{
	return BoLoadi32bsw(&aData, 0);
}

__myc_extern_inline__ u_int32_t
BoWswapi32(u_int32_t aData)
{
	return BoLoadi32wsw(&aData, 0);
}

__myc_extern_inline__ u_int64_t
BoBswapi64(u_int64_t aData)
{
	return BoLoadi64bsw(&aData, 0);
}

__myc_extern_inline__ u_int64_t
BoWswapi64(u_int64_t aData)
{
	return BoLoadi64wsw(&aData, 0);
}

__myc_extern_inline__ u_int64_t
BoDswapi64(u_int64_t aData)
{
	return BoLoadi64dsw(&aData, 0);
}

#endif /* !__BO_PPC_GCC__BYTEORDER__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
