/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Types.h	(V.Sokur)
 * include/Bo/memman/Types.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_TYPES__
# include <Bo/sys/Types.h>
#endif

#ifndef __BO_MEMMAN_TYPES__
# define __BO_MEMMAN_TYPES__

/******************************************************************************
 * %% BeginSection: bo_malloc declarations
 */
# pragma mark -
# pragma mark [ bo_malloc declarations ]
# pragma mark -

# ifndef __BO_MALLOC_STRUC_DEFINED__
#  define __BO_MALLOC_STRUC_DEFINED__

typedef bo_pointer_t
(__myc_tcalltype__(mcall) *bo_malloc_alloc_fn)(
# if __myc_require(prototypes)
	size_t
# endif
	) __attribute__((mcalltype));

typedef bo_pointer_t
(__myc_tcalltype__(mcall) *bo_malloc_calloc_fn)(
# if __myc_require(prototypes)
	size_t anElemc,
	size_t anElemsz
# endif
	) __attribute__((mcalltype));

typedef bo_pointer_t
(__myc_tcalltype__(mcall) *bo_malloc_realloc_fn)(
# if __myc_require(prototypes)
	bo_pointer_t ,
	size_t
# endif
	) __attribute__((mcalltype));

typedef void
(__myc_tcalltype__(mcall) *bo_malloc_free_fn)(
# if __myc_require(prototypes)
	bo_pointer_t
# endif
	) __attribute__((mcalltype));

struct bo_malloc {
	bo_malloc_alloc_fn   mm_alloc;
	bo_malloc_calloc_fn  mm_calloc;
	bo_malloc_realloc_fn mm_realloc;
	bo_malloc_free_fn    mm_free;
};

#  define BO_MALLOC_ZERO_INITIALIZER { NULLF, NULLF, NULLF, NULLF }

# endif /* !__BO_MALLOC_STRUC_DEFINED__ */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

struct bo_zmalloc;
struct bo_zmalloc_uctx;

typedef bo_status_t
(__myc_tcalltype__(mcall) *bo_zmalloc_uinit_fn)(
# if __myc_require(prototypes)
	bo_pointer_t anUserExt,
	bo_pointer_t anUserObj
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *bo_zmalloc_ufini_fn)(
# if __myc_require(prototypes)
	bo_pointer_t anUserExt,
	bo_pointer_t anUserObj
# endif
	) __attribute__((mcalltype));

struct bo_zmalloc_uctx {
	bo_zmalloc_uinit_fn  zmu_uinit_once;
	bo_zmalloc_ufini_fn  zmu_ufini_once;
	bo_zmalloc_uinit_fn  zmu_uinit;
	bo_zmalloc_ufini_fn  zmu_ufini;
	bo_pointer_t         zmu_ext;
};

struct bo_zmalloc_uctx_v2 {
	bo_zmalloc_uinit_fn  zmu_uinit_once;
	bo_zmalloc_ufini_fn  zmu_ufini_once;
	bo_zmalloc_uinit_fn  zmu_uinit;
	bo_zmalloc_ufini_fn  zmu_ufini;
	bo_pointer_t         zmu_ext;
	size_t               zmu_extsz;
};

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

# ifndef _BoK_MEMZONE_SIZE_DEFAULT
#  define _BoK_MEMZONE_SIZE_DEFAULT (1 << 16)
# endif

struct bo_memzone {
	bo_cdllelem_t  lelem_;
	size_t         size;
	/*
	char           data[0];
	 */
};

struct bo_memstg_s {
	struct bo_memstg_s *parent;
	bo_cdllist_t        zlist;
	struct bo_memzone  *zone;
};
# define bo_memstg_p(x)  ((struct bo_memstg_s *)(x))

struct bo_memstg {
	struct bo_memstg_s  _h;
	size_t              freespace;
};
typedef struct bo_memstg  bo_memstg_t;

struct bo_pool {
	struct bo_memstg_s  _h;
	size_t              objsz,
	                    align;
	bo_pointer_t        next,
	                    end;
};
typedef struct bo_pool  bo_pool_t;

struct bo_memstg_state {
	bo_pointer_t a, b, c, d;
};
typedef struct bo_memstg_state  bo_memstg_state_t;

#endif /* !__BO_MEMMAN_TYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
