/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Zmalloc.h	(V.Sokur)
 * include/Bo/memman/Zmalloc.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_MEMMAN_ZMALLOC__
# define __BO_MEMMAN_ZMALLOC__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/memman/Types.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# ifndef BoK_ZMALLOC_GRANULARITY_DEFAULT
#  define BoK_ZMALLOC_GRANULARITY_DEFAULT (64)
# endif

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) struct bo_zmalloc *
_BoZmallocOpen(
# if __myc_require(prototypes)
	bo_pointer_t aMemory, /* = NULL */
	struct bo_malloc const *anAllocator, /* = NULL */
	struct bo_zmalloc_uctx const *anUserCtx, /* = NULL */
	size_t aZoneGranularity, /* = 0 -> default-value */ /* granule-count in zone */
	size_t anUserObjsz, /* != 0 */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) struct bo_zmalloc *
_BoZmallocOpen_v2(
# if __myc_require(prototypes)
	bo_pointer_t aMemory, /* = NULL */
	struct bo_malloc const *anAllocator, /* = NULL */
	struct bo_zmalloc_uctx_v2 const *anUserCtx, /* = NULL */
	size_t aZoneGranularity, /* = 0 -> default-value */ /* granule-count in zone */
	size_t anUserObjsz, /* != 0 */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
_BoZmallocClose(
# if __myc_require(prototypes)
	struct bo_zmalloc *aZmalloc,
	bo_opflags_t Flags __myc_defval__(0), /* BoF_OP_FORCE */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/* NOT FOR DIRECT USING @{ */
/* Default procedure */
__myc_flinkage__(public,libBo) bo_status_t
__myc_calltype__(mcall)
__attribute__((nonnull(1)))
_BoZmallocDestroyDefault(
# if __myc_require(prototypes)
	struct bo_zmalloc *aZmalloc,
	bo_opflags_t Flags __myc_defval__(0), /* BoF_OP_FORCE */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);
/* @} */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/* BoS_FALSE, BoS_TRUE */
__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
_BoZmallocHasfreeg(
# if __myc_require(prototypes)
	struct bo_zmalloc *aZmalloc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_pointer_t
__attribute__((nonnull(1)))
_BoZmallocGet(
# if __myc_require(prototypes)
	struct bo_zmalloc *aZmalloc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* return NULL, if there are not free granules */
__myc_flinkage__(public,libBo) bo_pointer_t
__attribute__((nonnull(1)))
_BoZmallocGet0(
# if __myc_require(prototypes)
	struct bo_zmalloc *aZmalloc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_refc_t
__attribute__((nonnull(1)))
_BoZmPtrRetain(
# if __myc_require(prototypes)
	bo_pointer_t aPtr
# endif
	);

__myc_flinkage__(public,libBo) bo_refc_t
__attribute__((nonnull(1)))
_BoZmPtrRelease(
# if __myc_require(prototypes)
	bo_pointer_t aPtr
# endif
	);

__myc_flinkage__(public,libBo) struct bo_zmalloc *
__attribute__((nonnull(1)))
_BoZmPtrOwner(
# if __myc_require(prototypes)
	bo_pointer_t aPtr
# endif
	);

__myc_flinkage__(public,libBo) bo_refc_t
__attribute__((nonnull(1)))
_BoZmPtrRetainValue(
# if __myc_require(prototypes)
	bo_pointer_t aPtr
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct bo_zmalloc_uctx *
__attribute__((nonnull(1)))
_BoZmallocUctx(
# if __myc_require(prototypes)
	struct bo_zmalloc *aZmalloc
# endif
	);

__myc_flinkage__(public,libBo) size_t
__attribute__((nonnull(1)))
_BoZmallocGranularity(
# if __myc_require(prototypes)
	struct bo_zmalloc *aZmalloc
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_MEMMAN_ZMALLOC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
