/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Bouri.h	(N.Panasiuk)
 * include/Bo/objects/Bouri.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_OBJECTS_BOURI__
# define __BO_OBJECTS_BOURI__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

/*
 * Bouri
 * oid: <<category>>/<<object>>/<<impl>>
 *
 * constructor, descriptor, ucntl, uri
 */
/*
# define AorpK_Bouri_object (101)
# define AorpK_Bouri_object_creatable 1
 */
# include <Bo/services/MIB/Objects.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

_A_xobject(Bouri);

# include <Bo/services/alg/hilis/_Uri.h>
# include <Bo/services/alg/_Xucntl.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * Canonical prototype
 *
aorp_object_t
BouriOpen_c(
	aorp_opflags_t Flags,            // 0
	bo_pointer_t aMemory,            // NULL
	aorp_podinit_t aParmv[],
	size_t aParmc,
	struct aorp_error *anErrPtr      // NULL
	);
 */
# define BouriOpen_c(Flags, aMemory, aParmv, aParmc, anErrPtr) \
	_A_open1(Bouri, Flags, aMemory, aParmv, (size_t)(aParmc), anErrPtr)

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/*
void
BouriOpenWithString(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,                    // 0
	bo_pointer_t aMemory,                    // NULL
	aorp_object_t anUrilib,                  // NULL
	char const *anUriStr,
	bo_pointer_t aSpecBuf,                   // NULL
	size_t aSpecBufsz,                       // 0
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriOpenWithString(aResultRef,Flags,aMemory,anUrilib,anUriStr,aSpecBuf,aSpecBufsz,Options,anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Bouri), Flags, aMemory, \
		(2,(uri,Uri), AorpK_URI_INIT_WITH_STRING, \
			anUrilib, (intptr_t)(anUriStr), (size_t)(-1), \
			aSpecBuf, aSpecBufsz, Options \
		), \
		anErrPtr \
	)

/*
void
BouriAOpenWithString(
	aorp_object_t &aResultRef,
	aorp_object_t anUrilib,                  // NULL
	char const *anUriStr,
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriAOpenWithString(aResultRef,anUrilib,anUriStr,Options,anErrPtr) \
	BouriOpenWithString(aResultRef,0,NULL, \
			anUrilib,anUriStr,NULL,0,Options,anErrPtr)

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/*
void
BouriOpenWithWstring(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,                    // 0
	bo_pointer_t aMemory,                    // NULL
	aorp_object_t anUrilib,                  // NULL
	wchar_t const *anUriStr,
	bo_pointer_t aSpecBuf,                   // NULL
	size_t aSpecBufsz,                       // 0
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriOpenWithWstring(aResultRef,Flags,aMemory,anUrilib,anUriStr,aSpecBuf,aSpecBufsz,Options,anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Bouri), Flags, aMemory, \
		(2,(uri,Uri), AorpK_URI_INIT_WITH_WSTRING, \
			anUrilib, (intptr_t)(anUriStr), (size_t)(-1), \
			aSpecBuf, aSpecBufsz, Options \
		), \
		anErrPtr \
	)

/*
void
BouriAOpenWithWstring(
	aorp_object_t &aResultRef,
	aorp_object_t anUrilib,                  // NULL
	wchar_t const *anUriStr,
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriAOpenWithWstring(aResultRef,anUrilib,anUriStr,Options,anErrPtr) \
	BouriOpenWithWstring(aResultRef,0,NULL, \
			anUrilib,anUriStr,NULL,0,Options,anErrPtr)

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/*
void
BouriOpenWithFileSystemPath(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,                    // 0
	bo_pointer_t aMemory,                    // NULL
	aorp_object_t anUrilib,                  // NULL
	char const *aPath,
	bo_pointer_t aSpecBuf,                   // NULL
	size_t aSpecBufsz,                       // 0
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriOpenWithFileSystemPath(aResultRef,Flags,aMemory,anUrilib,aPath,aSpecBuf,aSpecBufsz,Options,anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Bouri), Flags, aMemory, \
		(2,(uri,Uri), AorpK_URI_INIT_WITH_FSPATH, \
			anUrilib, (intptr_t)(aPath), (size_t)(-1), \
			aSpecBuf, aSpecBufsz, Options \
		), \
		anErrPtr \
	)

/*
void
BouriAOpenWithFileSystemPath(
	aorp_object_t &aResultRef,
	aorp_object_t anUrilib,                  // NULL
	char const *aPath,
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriAOpenWithFileSystemPath(aResultRef,anUrilib,aPath,Options,anErrPtr) \
	BouriOpenWithFileSystemPath(aResultRef,0,NULL, \
			anUrilib,aPath,NULL,0,Options,anErrPtr)

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

/*
void
BouriOpenWithFileSystemWpath(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,                    // 0
	bo_pointer_t aMemory,                    // NULL
	aorp_object_t anUrilib,                  // NULL
	wchar_t const *aPath,
	bo_pointer_t aSpecBuf,                   // NULL
	size_t aSpecBufsz,                       // 0
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriOpenWithFileSystemWpath(aResultRef,Flags,aMemory,anUrilib,aPath,aSpecBuf,aSpecBufsz,Options,anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Bouri), Flags, aMemory, \
		(2,(uri,Uri), AorpK_URI_INIT_WITH_FSWPATH, \
			anUrilib, (intptr_t)(aPath), (size_t)(-1), \
			aSpecBuf, aSpecBufsz, Options \
		), \
		anErrPtr \
	)

/*
void
BouriAOpenWithFileSystemWpath(
	aorp_object_t &aResultRef,
	aorp_object_t anUrilib,                  // NULL
	wchar_t const *aPath,
	struct aorp_uri_options const *Options,  // NULL
	struct aorp_error *anErrPtr              // NULL
	);
 */
# define BouriAOpenWithFileSystemWpath(aResultRef,anUrilib,aPath,Options,anErrPtr) \
	BouriOpenWithFileSystemWpath(aResultRef,0,NULL, \
			anUrilib,aPath,NULL,0,Options,anErrPtr)

#endif /* !__BO_OBJECTS_BOURI__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
