/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Fstream.h	(V.Sokur)
 * include/Bo/objects/Fstream.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_OBJECTS_FSTREAM__
# define __BO_OBJECTS_FSTREAM__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

/*
 * Fstream
 * oid: <<category>>/<<object>>/<<impl>>
 *
 * constructor, descriptor, ucntl, ios
 */
/*
# define AorpK_Fstream_object (51)
# define AorpK_Fstream_object_creatable 1
 */
# include <Bo/services/MIB/Objects.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

_A_xobject(Fstream);

# include <Bo/services/alg/io/_Fiios.h>
# include <Bo/services/alg/_Xucntl.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * Canonical prototype
 *
aorp_object_t
FstreamOpen_c(
	aorp_opflags_t Flags,            // 0
	bo_pointer_t aMemory,            // NULL
	aorp_podinit_t aParmv[],
	size_t aParmc,
	struct aorp_error *anErrPtr      // NULL
	);
 */
# define FstreamOpen_c(Flags, aMemory, aParmv, aParmc, anErrPtr) \
	_A_open1(Fstream, Flags, aMemory, aParmv, (size_t)(aParmc), anErrPtr)

/* ------------------------------------------------------------------------- */

/*
void
FstreamOpen_p(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	char const *aPath,
	char const *aMode,          // NULL -> "r+",
	struct aorp_error *anErrPtr // NULL
	);
 */
# define FstreamOpen_p(aResultRef, Flags, aMemory, aPath, aMode, anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Fstream), Flags, aMemory, \
		(2,(ios,Fiios), AorpK_FIIOS_INIT_PATH, (intptr_t)(aPath), (bo_pointer_t)(aMode), BoK_OWNS_YOUR_DATA), \
		anErrPtr \
	)

/*
void
FstreamOpen_wp(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	wchar_t const *aPath,
	wchar_t const *aMode,       // NULL -> "r+",
	struct aorp_error *anErrPtr // NULL
	);
 */
# define FstreamOpen_wp(aResultRef, Flags, aMemory, aPath, aMode, anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Fstream), Flags, aMemory, \
		(2,(ios,Fiios), AorpK_FIIOS_INIT_WPATH, (intptr_t)(aPath), (bo_pointer_t)(aMode), BoK_OWNS_YOUR_DATA), \
		anErrPtr \
	)

/*
void
FstreamOpen_uri(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	aorp_object_t aLocation,
	char const *aMode,          // NULL -> "r+",
	struct aorp_error *anErrPtr // NULL
	);
 */
# define FstreamOpen_uri(aResultRef, Flags, aMemory, aLocation, aMode, anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Fstream), Flags, aMemory, \
		(2,(ios,Fiios), AorpK_FIIOS_INIT_URI, (intptr_t)(aLocation), (bo_pointer_t)(aMode), BoK_OWNS_YOUR_DATA), \
		anErrPtr \
	)

/*
void
FstreamOpen_fd(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	int aFD,
	char const *aMode,          // NULL -> "r+",
	bo_ownership_t aPolicy,
	struct aorp_error *anErrPtr // NULL
	);
 */
# define FstreamOpen_fd(aResultRef, Flags, aMemory, \
	                    aFD, aMode, aPolicy, anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Fstream), Flags, aMemory, \
		(2,(ios,Fiios), AorpK_FIIOS_INIT_FD, (intptr_t)(aFD), (bo_pointer_t)(aMode), aPolicy), \
		anErrPtr \
	)

/* ------------------------------------------------------------------------- */

/*
# include "Bo/libc/Stdio.h"
*/

/*
void
FstreamOpen_f(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	FILE *aFile,
	bo_ownership_t aPolicy,
	struct aorp_error *anErrPtr // NULL
	);
 */
# define FstreamOpen_f(aResultRef, Flags, aMemory, \
	                   aFile, aPolicy, anErrPtr) \
	_A_openbyname_v1( \
		aResultRef, (Fstream), Flags, aMemory, \
		(2,(ios,Fiios), AorpK_FIIOS_INIT_FILE, (intptr_t)(aFile), (bo_pointer_t)(0), aPolicy), \
		anErrPtr \
	)

/*
void
BoStdinOpen(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	struct aorp_error *anErrPtr // NULL
	);
 */
# define BoStdinOpen(aResultRef, Flags, aMemory, anErrPtr) \
	FstreamOpen_f(aResultRef, Flags, aMemory, 0, BoK_OWNS_MY_DATA, anErrPtr)

/*
void
BoStdoutOpen(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	struct aorp_error *anErrPtr // NULL
	);
 */
# define BoStdoutOpen(aResultRef, Flags, aMemory, anErrPtr) \
	FstreamOpen_f(aResultRef, Flags, aMemory, 1, BoK_OWNS_MY_DATA, anErrPtr)

/*
void
BoStderrOpen(
	aorp_object_t &aResultRef,
	aorp_opflags_t Flags,       // 0
	bo_pointer_t aMemory,       // NULL
	struct aorp_error *anErrPtr // NULL
	);
 */
# define BoStderrOpen(aResultRef, Flags, aMemory, anErrPtr) \
	FstreamOpen_f(aResultRef, Flags, aMemory, 2, BoK_OWNS_MY_DATA, anErrPtr)

#endif /* !__BO_OBJECTS_FSTREAM__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
