/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Ccsconv.h	(V.Sokur)
 * include/Bo/services/Ccsconv.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CCSCONV__
# define __BO_SERVICES_CCSCONV__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/CcsconvP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

__myc_vlinkage__(public,libBo) aorp_object_t _G_ccsconv_service_default;

# pragma mark -

# define AorpOpenCcscvtor_i_(aSourceCharset,aDestCharset,aConv,anErrPtr) \
	AorpOpenCcscvtor_i(NULL,aSourceCharset,aDestCharset,aConv,anErrPtr)

# define AorpOpenCcscvtor_s_(aSourceCharset,aDestCharset,aConv,anErrPtr) \
	AorpOpenCcscvtor_s(NULL,aSourceCharset,aDestCharset,aConv,anErrPtr)

# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(4)))
AorpOpenCcscvtor_i(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_cset_t aSourceCharset,
	bo_cset_t aDestCharset,
	struct aorp_ccscvtor *aConv,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(2,3,4)))
AorpOpenCcscvtor_s(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aSourceCharset,
	char const *aDestCharset,
	struct aorp_ccscvtor *aConv,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
AorpCcscvtorClose(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# pragma mark -

/* Upon a successful completion the function returns a byte count of a dest
   multibyte character. This is useful if NULL pointer is passed for
   a formal parameter aDestPtr or aDestBytesLeftPtr. Otherwise the value
   of (-1) returned. */
__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,2,3)))
AorpCcsconv(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	char const **aSourcePtr,     /* in/out */
	size_t *aSourceBytesLeftPtr, /* in/out */
	char **aDestPtr,             /* in/out */
	size_t *aDestBytesLeftPtr,   /* in/out */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Upon a successful completion the function returns a character count of
   a dest wide-character. This is always 1. This is useful if NULL pointer
   is passed for a formal parameter aDestPtr or aDestBytesLeftPtr. Otherwise
   the value of (-1) returned. */
__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,2,3)))
AorpCcsconv_2wc(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	char const **aSourcePtr,     /* in/out */
	size_t *aSourceBytesLeftPtr, /* in/out */
	wchar_t **aDestPtr,          /* in/out */
	size_t *aDestCharsLeftPtr,   /* in/out */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Upon a successful completion the function returns a byte count of a dest
   multibyte character. This is useful if NULL pointer is passed for
   a formal parameter aDestPtr or aDestBytesLeftPtr. Otherwise the value
   of (-1) returned. */
__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,2,3)))
AorpCcsconv_2mb(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	wchar_t const **aSourcePtr,  /* in/out */
	size_t *aSourceCharsLeftPtr, /* in/out */
	char **aDestPtr,             /* in/out */
	size_t *aDestBytesLeftPtr,   /* in/out */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# pragma mark -

/* Required that a size of a destination buffer is large enough
   or a NULL pointer is passed for a destination buffer pointed
   to by the formal parameter aDest.
   Upon a successful completion the function returns a byte count
   of a dest multibyte string, not including a terminating NUL character.
   Otherwise the value of (-1) returned. */
__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,2)))
AorpCcsconv_s(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	char const *aSource,
	char *aDest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,2)))
AorpCcsconv_b(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	char const *aSource,
	size_t aSrcBytec, /* if (-1 == aSrcBytec) aSource is NUL terminated. */
	char *aDest,
	size_t aDstBytec,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Required that a size of a destination buffer is large enough
   or a NULL pointer is passed for a destination buffer pointed
   to by the formal parameter aDest.
   Upon a successful completion the function returns a character count
   of a dest wide-char string, not including a terminating NUL character.
   Otherwise the value of (-1) returned. */
__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,2)))
AorpCcsconv_s_2wc(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	char const *aSource,
	wchar_t *aDest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1)))
AorpCcsconv_b_2wc(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	char const *aSource,
	size_t aSrcBytec, /* if (-1 == aSrcBytec) aSource is NUL terminated. */
	wchar_t *aDest,
	size_t aDstCharc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Required that a size of a destination buffer is large enough
   or a NULL pointer is passed for a destination buffer pointed
   to by the formal parameter aDest.
   Upon a successful completion the function returns a byte count
   of a dest multibyte string, not including a terminating NUL character.
   Otherwise the value of (-1) returned. */
__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,2)))
AorpCcsconv_s_2mb(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	wchar_t const *aSource,
	char *aDest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1)))
AorpCcsconv_b_2mb(
# if __myc_require(prototypes)
	struct aorp_ccscvtor *aConv,
	wchar_t const *aSource,
	size_t aSrcCharc, /* if (-1 == aSrcCharc) aSource is NUL terminated. */
	char *aDest,
	size_t aDstBytec,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_CCSCONV__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
