/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * CcsconvP.h	(V.Sokur)
 * include/Bo/services/CcsconvP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CCSCONV_P__
# define __BO_SERVICES_CCSCONV_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/Ccsconvtypes.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define __aorpc_need_service_ccsconv 1
# define __aorpc_need_service_ccscvtor 1

#endif /* !__BO_SERVICES_CCSCONV_P__ */

#pragma mark -

#if __aorpc_uses_service(ccsconv)
# undef __aorpc_need_service_ccsconv

/*
 * ccsconv service
 * @{
 */

/*
 * ccsconv {\object(class)} [Constructor]
 * Coded Character Set Conversion
 */
/*
# define AorpK_ccsconv_service (1010)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name ccsconv
# include <Bo/aorp/AorpCsrvc.h>

_A_service(ccsconv)
_A_end_ctb

	_A_return((
		BoS_SUCCESS = [ BoS_NORMAL = (); ];
		BoS_FAILURE = [ BoS_ERR = (BoK_RacError_NotConfigured); ];
	))
	_A_attribute((nonnull(1,5)))
	_A_op(opencvtor, "ccsconv::open-convertor", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_cset_t aSourceCharset,
			bo_cset_t aDestCharset,
			struct aorp_ccscvtor *aConv
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NORMAL = (); ];
		BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
	))
	_A_attribute((nonnull(1,3)))
	_A_op(closecvtor, "ccsconv::close-convertor", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_ccscvtor *aConv
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(ccsconv) */

/* ------------------------------------------------------------------------- */
#pragma mark -
/* ------------------------------------------------------------------------- */

#if __aorpc_uses_service(ccscvtor)
# undef __aorpc_need_service_ccscvtor

/*
 * ccscvtor service
 * @{
 */

/*
 * ccscvtor {\ops}
 * Coded Character Set Convertor
 */
/*
# define AorpK_ccscvtor_service (1011)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name ccscvtor
# include <Bo/aorp/AorpCsrvc.h>

_A_service(ccscvtor)

# if __aorpc_interface

/* Returned from mb2wc */
# define BoS_CCSCVTOR_EILSEQ (-2)
# define BoS_CCSCVTOR_EMBTOOFEW (-3)
/* Returned from wc2mb */
# define BoS_CCSCVTOR_EBADWC (-2)
# define BoS_CCSCVTOR_EMBOVERFLOW (-3)

# endif /* __aorpc_interface */

_A_end_ctb

	_A_return((
		BoS_SUCCESS = [ 0++ = (/* number of bytes of a source character */); ];
		BoS_FAILURE = [
			BoS_CCSCVTOR_EILSEQ = (BoK_TextError_IllegalMBSequence);
			BoS_CCSCVTOR_EMBTOOFEW = (BoK_TextError_IncompleteMBSequence);
			BoS_ERR = (...);
		];
	))
	_A_attribute((nonnull(1,3,4,5)))
	_A_op(mb2wc, "ccscvtor::convert-mb-to-wc", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_ccscvtor *aConv,
			u_char_t const *aSource,
			wchar_t *aDest,
			size_t aBytec /* max inspected bytes */
		)),

	_A_return((
		BoS_SUCCESS = [ 0++ = (/* number of bytes of a destination character */); ];
		BoS_FAILURE = [
			BoS_CCSCVTOR_EBADWC = (BoK_TextError_BadUnicodeCharacter);
			BoS_CCSCVTOR_EMBOVERFLOW = (BoK_TextError_MBOverflow);
			BoS_ERR = (...);
		];
	))
	_A_attribute((nonnull(1,3,4,5)))
	_A_op(wc2mb, "ccscvtor::convert-wc-to-mb", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_ccscvtor *aConv,
			wchar_t const *aSource,
			u_char_t *aDest,
			size_t aBytec /* max required bytes */
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(ccscvtor) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
