/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * CcsiP.h	(V.Sokur)
 * include/Bo/services/CcsiP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CCSI_P__
# define __BO_SERVICES_CCSI_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/csType.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define __aorpc_need_service_ccsi 1

#endif /* !__BO_SERVICES_CCSI_P__ */

#pragma mark -

#if __aorpc_uses_service(ccsi)
# undef __aorpc_need_service_ccsi

/*
 * ccsi service
 * @{
 */

/*
 * ccsi {\object(class)}
 * Coded Character Set Infrastructure
 */
/*
# define AorpK_ccsi_service (1009)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name ccsi
# include <Bo/aorp/AorpCsrvc.h>

_A_service(ccsi)

# if __aorpc_interface

	struct aorp_ccsstat {
		bo_cset_t   iana_id;
		char const *iana_name;
		char const *mime_name;
		long        wincp;
		unsigned    mb_max; /* 0: unknown */
	};

	struct aorp_ccsastat {
		bo_cset_t   iana_id;
		char const *iana_name;
		char const *mime_name;
		long        wincp;
		unsigned    mb_max; /* 0: unknown */

		unsigned    aliasc;
		char const *aliasv[1];
	};

# define _TAG_AORP_CCSASTAT(N) __myc_xconcat(aorp_ccsastat,N)
# define _TYPENAME_AORP_CCSASTAT(N) struct _TAG_AORP_CCSASTAT(N)
# define _STRUCT_AORP_CCSASTAT(N) \
	_TYPENAME_AORP_CCSASTAT(N) { \
		bo_cset_t   iana_id; \
		char const *iana_name; \
		char const *mime_name; \
		long        wincp; \
		unsigned    mb_max; \
		unsigned    aliasc; \
		char const *aliasv[N + 1]; \
	}

/* ------------------------------------------------------------------------- */

	struct Ccsi_dbelem {
		bo_cset_t             iana_id;
		char const           *iana_name_or_alias;
		struct aorp_ccsastat *sb;

		/* NOT FOR YOUR BUSINESS @{ */
		bo_rtbtelem_t         btelem1; /* enum-to-string */
		bo_rtbtelem_t         btelem2; /* string-to-enum */
		/* @} */
	};

# define _AORP_CCSASTAT_NAME ((char const *)1)
# define _AORP_CCSASTAT_ALIAS ((char const *)2)

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(stat, "ccsi::get-charset-stat", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_cset_t anId
		)),

	_A_attribute((nonnull(1)))
	_A_op(iananame, "ccsi::get-charset-iana-name", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_cset_t anId
		)),

	_A_attribute((nonnull(1)))
	_A_op(mimename, "ccsi::get-charset-mime-name", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_cset_t anId
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(getidbyname, "ccsi::get-charset-id-by-name", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aNameOrAlias
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(additab, "ccsi::add-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Ccsi_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(additab_nlk, "ccsi::add-table-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Ccsi_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmitab, "ccsi::remove-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Ccsi_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmitab_nlk, "ccsi::remove-table-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Ccsi_dbelem aTable[]
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(ccsi) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
