/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Err.h	(V.Sokur)
 * include/Bo/services/Err.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ERR__
# define __BO_SERVICES_ERR__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/ErrP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# ifndef _AORP_MAKE_ERR_WITH_DEBUG
#  ifdef _DEBUG
#   define _AORP_MAKE_ERR_WITH_DEBUG
#  endif
# endif
# ifdef _AORP_MAKE_ERR_WITHOUT_DEBUG
#  ifdef _AORP_MAKE_ERR_WITH_DEBUG
#   undef _AORP_MAKE_ERR_WITH_DEBUG
#  endif
# endif

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

# if __myc_support(AVM)

#  ifdef _AORP_MAKE_ERR_WITH_DEBUG

#   define AorpMakeErr(anErrPtr,aDomain,aCode,aSyserr,/* aMsgidx, */...) \
		AorpMkerr(NULL, anErrPtr, __func__, __FILE__, __LINE__, \
			aDomain, aCode, aSyserr, __VA_ARGS__)

#   define AorpMakeLocalizedErr(anErrPtr,aLocale,aDomain,aCode,aSyserr,/* aMsgidx, */...) \
		AorpMkerr_l(NULL, anErrPtr, aLocale, __func__, __FILE__, __LINE__, \
			aDomain, aCode, aSyserr, __VA_ARGS__)

#   define AorpMakeCustomErr(anErrPtr,aDomain,aCode,aSyserr,/* aMsg, */...) \
		AorpMkerr_u(anErrPtr, __func__, __FILE__, __LINE__, \
			aDomain, aCode, aSyserr, __VA_ARGS__)

#  else /* !_AORP_MAKE_ERR_WITH_DEBUG */

#   define AorpMakeErr(anErrPtr,aDomain,aCode,aSyserr,/* aMsgidx, */...) \
		AorpMkerr(NULL, anErrPtr, NULL, NULL, 0, aDomain, aCode, aSyserr, __VA_ARGS__)

#   define AorpMakeLocalizedErr(anErrPtr,aLocale,aDomain,aCode,aSyserr,/* aMsgidx, */...) \
		AorpMkerr_l(NULL, anErrPtr, aLocale, NULL, NULL, 0, \
			aDomain, aCode, aSyserr, __VA_ARGS__)

#   define AorpMakeCustomErr(anErrPtr,aDomain,aCode,aSyserr,/* aMsg, */...) \
		AorpMkerr_u(anErrPtr, NULL, NULL, 0, aDomain, aCode, aSyserr, __VA_ARGS__)

#  endif /* !_AORP_MAKE_ERR_WITH_DEBUG */

# else
# if __myc_support(NVM)

#  ifdef _AORP_MAKE_ERR_WITH_DEBUG

#   define AorpMakeErr(anErrPtr,aDomain,aCode,aSyserr,aMsgidx...) \
		AorpMkerr(NULL, anErrPtr, __func__, __FILE__, __LINE__, \
			aDomain, aCode, aSyserr, aMsgidx)

#   define AorpMakeLocalizedErr(anErrPtr,aLocale,aDomain,aCode,aSyserr,aMsgidx...) \
		AorpMkerr_l(NULL, anErrPtr, aLocale, __func__, __FILE__, __LINE__, \
			aDomain, aCode, aSyserr, aMsgidx)

#   define AorpMakeCustomErr(anErrPtr,aDomain,aCode,aSyserr,aMsg...) \
		AorpMkerr_u(anErrPtr, __func__, __FILE__, __LINE__, \
			aDomain, aCode, aSyserr, aMsg)

#  else /* !_AORP_MAKE_ERR_WITH_DEBUG */

#   define AorpMakeErr(anErrPtr,aDomain,aCode,aSyserr,aMsgidx...) \
		AorpMkerr(NULL, anErrPtr, NULL, NULL, 0, aDomain, aCode, aSyserr, aMsgidx)

#   define AorpMakeLocalizedErr(anErrPtr,aLocale,aDomain,aCode,aSyserr,aMsgidx...) \
		AorpMkerr_l(NULL, anErrPtr, aLocale, NULL, NULL, 0, aDomain, aCode, aSyserr, aMsgidx)

#   define AorpMakeCustomErr(anErrPtr,aDomain,aCode,aSyserr,aMsg...) \
		AorpMkerr_u(anErrPtr, NULL, NULL, 0, aDomain, aCode, aSyserr, aMsg)

#  endif /* !_AORP_MAKE_ERR_WITH_DEBUG */

# endif /* !__myc_support(NVM) */
# endif /* !__myc_support(AVM) */

/* ------------------------------------------------------------------------- */

/* make-error */
#  ifdef _AORP_MAKE_ERR_WITH_DEBUG

#   define AorpMakeErr0(anErrPtr,aDomain,aCode,aMsgidx) \
		AorpMkerr(NULL,anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsgidx)

#   define AorpMakeErr1(anErrPtr,aDomain,aCode,aMsgidx,p1) \
		AorpMkerr(NULL,anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsgidx,p1)

#   define AorpMakeErr2(anErrPtr,aDomain,aCode,aMsgidx,p1,p2) \
		AorpMkerr(NULL,anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsgidx,p1,p2)

#   define AorpMakeErr3(anErrPtr,aDomain,aCode,aMsgidx,p1,p2,p3) \
		AorpMkerr(NULL,anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsgidx,p1,p2,p3)

#   define AorpMakeErr4(anErrPtr,aDomain,aCode,aMsgidx,p1,p2,p3,p4) \
		AorpMkerr(NULL,anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsgidx,p1,p2,p3,p4)

#   define AorpMakeErr5(anErrPtr,aDomain,aCode,aMsgidx,p1,p2,p3,p4,p5) \
		AorpMkerr(NULL,anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsgidx,p1,p2,p3,p4,p5)

#  else /* !_AORP_MAKE_ERR_WITH_DEBUG */

#   define AorpMakeErr0(anErrPtr,aDomain,aCode,aMsgidx) \
		AorpMkerr(NULL,anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsgidx)

#   define AorpMakeErr1(anErrPtr,aDomain,aCode,aMsgidx,p1) \
		AorpMkerr(NULL,anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1)

#   define AorpMakeErr2(anErrPtr,aDomain,aCode,aMsgidx,p1,p2) \
		AorpMkerr(NULL,anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2)

#   define AorpMakeErr3(anErrPtr,aDomain,aCode,aMsgidx,p1,p2,p3) \
		AorpMkerr(NULL,anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2,p3)

#   define AorpMakeErr4(anErrPtr,aDomain,aCode,aMsgidx,p1,p2,p3,p4) \
		AorpMkerr(NULL,anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2,p3,p4)

#   define AorpMakeErr5(anErrPtr,aDomain,aCode,aMsgidx,p1,p2,p3,p4,p5) \
		AorpMkerr(NULL,anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2,p3,p4,p5)

#  endif /* !_AORP_MAKE_ERR_WITH_DEBUG */

/* ------------------------------------------------------------------------- */

/* make-localized-error */
#  ifdef _AORP_MAKE_ERR_WITH_DEBUG

#   define AorpMakeLocalizedErr0(anErrPtr,aLocale,aDomain,aCode,aMsgidx) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,__func__,__FILE__,__LINE__,\
			aDomain,aCode,0,aMsgidx)

#   define AorpMakeLocalizedErr1(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,__func__,__FILE__,__LINE__,\
			aDomain,aCode,0,aMsgidx,p1)

#   define AorpMakeLocalizedErr2(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,__func__,__FILE__,__LINE__,\
			aDomain,aCode,0,aMsgidx,p1,p2)

#   define AorpMakeLocalizedErr3(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2,p3) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,__func__,__FILE__,__LINE__,\
			aDomain,aCode,0,aMsgidx,p1,p2,p3)

#   define AorpMakeLocalizedErr4(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2,p3,p4) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,__func__,__FILE__,__LINE__,\
			aDomain,aCode,0,aMsgidx,p1,p2,p3,p4)

#   define AorpMakeLocalizedErr5(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2,p3,p4,p5) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,__func__,__FILE__,__LINE__,\
			aDomain,aCode,0,aMsgidx,p1,p2,p3,p4,p5)

#  else /* !_AORP_MAKE_ERR_WITH_DEBUG */

#   define AorpMakeLocalizedErr0(anErrPtr,aLocale,aDomain,aCode,aMsgidx) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,NULL,NULL,0,aDomain,aCode,0,aMsgidx)

#   define AorpMakeLocalizedErr1(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1)

#   define AorpMakeLocalizedErr2(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2)

#   define AorpMakeLocalizedErr3(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2,p3) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2,p3)

#   define AorpMakeLocalizedErr4(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2,p3,p4) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2,p3,p4)

#   define AorpMakeLocalizedErr5(anErrPtr,aLocale,aDomain,aCode,aMsgidx,p1,p2,p3,p4,p5) \
		AorpMkerr_l(NULL,anErrPtr,aLocale,NULL,NULL,0,aDomain,aCode,0,aMsgidx,p1,p2,p3,p4,p5)

#  endif /* !_AORP_MAKE_ERR_WITH_DEBUG */

/* ------------------------------------------------------------------------- */

/* make-custom-error */
#  ifdef _AORP_MAKE_ERR_WITH_DEBUG

#   define AorpMakeCustomErr0(anErrPtr,aDomain,aCode,aMsg) \
		AorpMkerr_u(anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsg)

#   define AorpMakeCustomErr1(anErrPtr,aDomain,aCode,aMsg,p1) \
		AorpMkerr_u(anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsg,p1)

#   define AorpMakeCustomErr2(anErrPtr,aDomain,aCode,aMsg,p1,p2) \
		AorpMkerr_u(anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsg,p1,p2)

#   define AorpMakeCustomErr3(anErrPtr,aDomain,aCode,aMsg,p1,p2,p3) \
		AorpMkerr_u(anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsg,p1,p2,p3)

#   define AorpMakeCustomErr4(anErrPtr,aDomain,aCode,aMsg,p1,p2,p3,p4) \
		AorpMkerr_u(anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsg,p1,p2,p3,p4)

#   define AorpMakeCustomErr5(anErrPtr,aDomain,aCode,aMsg,p1,p2,p3,p4,p5) \
		AorpMkerr_u(anErrPtr,__func__,__FILE__,__LINE__,aDomain,aCode,0,aMsg,p1,p2,p3,p4,p5)

#  else /* !_AORP_MAKE_ERR_WITH_DEBUG */

#   define AorpMakeCustomErr0(anErrPtr,aDomain,aCode,aMsg) \
		AorpMkerr_u(anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsg)

#   define AorpMakeCustomErr1(anErrPtr,aDomain,aCode,aMsg,p1) \
		AorpMkerr_u(anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsg,p1)

#   define AorpMakeCustomErr2(anErrPtr,aDomain,aCode,aMsg,p1,p2) \
		AorpMkerr_u(anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsg,p1,p2)

#   define AorpMakeCustomErr3(anErrPtr,aDomain,aCode,aMsg,p1,p2,p3) \
		AorpMkerr_u(anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsg,p1,p2,p3)

#   define AorpMakeCustomErr4(anErrPtr,aDomain,aCode,aMsg,p1,p2,p3,p4) \
		AorpMkerr_u(anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsg,p1,p2,p3,p4)

#   define AorpMakeCustomErr5(anErrPtr,aDomain,aCode,aMsg,p1,p2,p3,p4,p5) \
		AorpMkerr_u(anErrPtr,NULL,NULL,0,aDomain,aCode,0,aMsg,p1,p2,p3,p4,p5)

#  endif /* !_AORP_MAKE_ERR_WITH_DEBUG */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_vlinkage__(public,libBo) aorp_object_t _G_error_service_default;

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
AorpMkerr(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	struct aorp_error *anErrPtr,
	char const *aFuncName,
	char const *aFileName,
	long aLine,
	int aDomain,
	bo_errcode_t aCode,
	syserrcode_t aSyserr,
	unsigned aMsgidx /* = 0 */,
	...
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
AorpMkerr_l(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	struct aorp_error *anErrPtr,
	char const *aLocale,
	char const *aFuncName,
	char const *aFileName,
	long aLine,
	int aDomain,
	bo_errcode_t aCode,
	syserrcode_t aSyserr,
	unsigned aMsgidx /* = 0 */,
	...
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(8), format(printf,8,9)))
AorpMkerr_u(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr,
	char const *aFuncName,
	char const *aFileName,
	long aLine,
	int aDomain,
	bo_errcode_t aCode,
	syserrcode_t aSyserr,
	char const *aMsg,
	...
# endif
	);

__myc_flinkage__(public,libBo) int
AorpStatus2unix(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_status_t aStatus
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
_AorpAddetab(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	size_t aCount,
	struct Err_dbelem aTable[]
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
_AorpAddetab_nlk(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	size_t aCount,
	struct Err_dbelem aTable[]
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
_AorpRmetab(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	size_t aCount,
	struct Err_dbelem aTable[]
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
_AorpRmetab_nlk(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	size_t aCount,
	struct Err_dbelem aTable[]
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_ERR__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
