/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * ErrP.h	(V.Sokur)
 * include/Bo/services/ErrP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ERR_P__
# define __BO_SERVICES_ERR_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

/******************************************************************************
 * %% BeginSection: error service declarations
 */
# pragma mark -
# pragma mark [ error service declarations ]
# pragma mark -

# define __aorpc_need_service_error 1

#endif /* !__BO_SERVICES_ERR_P__ */

#pragma mark -

#if __aorpc_uses_service(error)
# undef __aorpc_need_service_error

/*
 * error service
 * @{
 */

# include <Bo/libc/Stdarg.h>
	/* va_list */

/*
 * error {\object(class)}
 */
/*
# define AorpK_error_service (1008)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name error
# include <Bo/aorp/AorpCsrvc.h>

_A_service(error)

# if __aorpc_interface

	struct Err_dbelem {
		bo_errcode_t   bo_ecode; /* Bo error code */
		int            un_ecode; /* unix error code */
		char const    *emsg;
		unsigned       count;

		/* NOT FOR YOUR BUSINESS */
		bo_rtbtelem_t  btelem;
	};

# define _ErrK_UNK_BO_ECODE (-1)
# define _ErrK_UNK_UNIX_ECODE (-1)

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1,2)))
	_A_op(make, "make-error", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aFuncName,
			char const *aFileName,
			long aLine,
			int aDomain,
			bo_errcode_t aCode,
			syserrcode_t aSyserr,
			unsigned aMsgidx,
			va_list
		)),

	_A_attribute((nonnull(1,2,3)))
	_A_op(make_l, "make-localized-error", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aLocale,
			char const *aFuncName,
			char const *aFileName,
			long aLine,
			int aDomain,
			bo_errcode_t aCode,
			syserrcode_t aSyserr,
			unsigned aMsgidx,
			va_list
		)),

	_A_attribute((nonnull(1)))
	_A_op(s2unix, "convert-status-to-unix-error", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_status_t aStatus
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(addetab, "add-error-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Err_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(addetab_nlk, "add-error-table-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Err_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmetab, "remove-error-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Err_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmetab_nlk, "remove-error-table-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Err_dbelem aTable[]
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(error) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
