/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Errblttab.h	(V.Sokur)
 * include/Bo/services/alg/_Errblttab.h
 *
 * $Id$
 */

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -
# pragma mark [ definitions ]
# pragma mark -

static struct Err_dbelem _S_Err_builtin_table[] = {
	/*
	 * ARGUMENT ERRORS
	 */
	{ BoK_ArgError_ArgListLimit,               E2BIG,          SMSG_ArgError_ArgListLimit, 4 },
	{ BoK_ArgError_ArgListLimit,               E2BIG,          SMSG_ArgError_ArgListLimit_s, 0 },
	{ BoK_ArgError_ArgListLimit,               E2BIG,          SMSG_ArgError_ArgListLimit_s2, 0 },
	{ BoK_ArgError_ArgListLimit,               E2BIG,          SMSG_ArgError_ArgListLimit_s3, 0 },
	{ BoK_ArgError_BadAddress,                 EFAULT,         SMSG_ArgError_BadAddress, 4 },
	{ BoK_ArgError_BadAddress,                 EFAULT,         SMSG_ArgError_BadAddress_s, 0 },
	{ BoK_ArgError_BadAddress,                 EFAULT,         SMSG_ArgError_BadAddress_s2, 0 },
	{ BoK_ArgError_BadAddress,                 EFAULT,         SMSG_ArgError_BadAddress_s3, 0 },
	{ BoK_ArgError_InvalidArgument,            EINVAL,         SMSG_ArgError_InvalidArgument, 4 },
	{ BoK_ArgError_InvalidArgument,            EINVAL,         SMSG_ArgError_InvalidArgument_s, 0 },
	{ BoK_ArgError_InvalidArgument,            EINVAL,         SMSG_ArgError_InvalidArgument_s2, 0 },
	{ BoK_ArgError_InvalidArgument,            EINVAL,         SMSG_ArgError_InvalidArgument_s3, 0 },
	{ BoK_ArgError_InvalidString,              EINVAL,         SMSG_ArgError_InvalidString, 4 },
	{ BoK_ArgError_InvalidString,              EINVAL,         SMSG_ArgError_InvalidString_s, 0 },
	{ BoK_ArgError_InvalidString,              EINVAL,         SMSG_ArgError_InvalidString_s2, 0 },
	{ BoK_ArgError_InvalidString,              EINVAL,         SMSG_ArgError_InvalidString_s3, 0 },
	{ BoK_ArgError_Overflow,                   EOVERFLOW,      SMSG_ArgError_Overflow, 4 },
	{ BoK_ArgError_Overflow,                   EOVERFLOW,      SMSG_ArgError_Overflow_s, 0 },
	{ BoK_ArgError_Overflow,                   EOVERFLOW,      SMSG_ArgError_Overflow_s2, 0 },
	{ BoK_ArgError_Overflow,                   EOVERFLOW,      SMSG_ArgError_Overflow_s3, 0 },

	/*
	 * RESOURCE ACCESS ERRORS
	 */
	{ BoK_RacError_OperationNotPermitted,      EPERM,          SMSG_RacError_OperationNotPermitted, 3 },
	{ BoK_RacError_OperationNotPermitted,      EPERM,          SMSG_RacError_OperationNotPermitted_s, 0 },
	{ BoK_RacError_OperationNotPermitted,      EPERM,          SMSG_RacError_OperationNotPermitted_s3, 0 },
	{ BoK_RacError_AccessViolation,            EACCES,         SMSG_RacError_AccessViolation, 3 },
	{ BoK_RacError_AccessViolation,            EACCES,         SMSG_RacError_AccessViolation_s, 0 },
	{ BoK_RacError_AccessViolation,            EACCES,         SMSG_RacError_AccessViolation_s2, 0 },
	{ BoK_RacError_DeadLock,                   EDEADLK,        SMSG_RacError_DeadLock, 4 },
	{ BoK_RacError_DeadLock,                   EDEADLK,        SMSG_RacError_DeadLock_s, 0 },
	{ BoK_RacError_DeadLock,                   EDEADLK,        SMSG_RacError_DeadLock_s2, 0 },
	{ BoK_RacError_DeadLock,                   EDEADLK,        SMSG_RacError_DeadLock_s3, 0 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_Again, 8 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_Again_s, 0 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_Again_s2, 0 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_Again_s3, 0 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_WouldBlock, 0 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_WouldBlock_s, 0 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_WouldBlock_s2, 0 },
	{ BoK_RacError_WouldBlock,                 EWOULDBLOCK,    SMSG_RacError_WouldBlock_s3, 0 },
	{ BoK_RacError_Busy,                       EBUSY,          SMSG_RacError_Busy, 4 },
	{ BoK_RacError_Busy,                       EBUSY,          SMSG_RacError_Busy_s, 0 },
	{ BoK_RacError_Busy,                       EBUSY,          SMSG_RacError_Busy_s2, 0 },
	{ BoK_RacError_Busy,                       EBUSY,          SMSG_RacError_Busy_s3, 0 },
	{ BoK_RacError_NotConfigured,              ENXIO,          SMSG_RacError_NotConfigured, 7 },
	{ BoK_RacError_NotConfigured,              ENXIO,          SMSG_RacError_NotConfigured_s, 0 },
	{ BoK_RacError_NotConfigured,              ENXIO,          SMSG_RacError_NotConfigured_s2, 0 },
	{ BoK_RacError_NotConfigured,              ENXIO,          SMSG_RacError_NotConfigured_s3, 0 },
	{ BoK_RacError_NotConfigured,              ENXIO,          SMSG_RacError_NotConfigured_i, 0 },
	{ BoK_RacError_NotConfigured,              ENXIO,          SMSG_RacError_NotConfigured_si, 0 },
	{ BoK_RacError_NotConfigured,              ENXIO,          SMSG_RacError_NotConfigured_s2i, 0 },
	{ BoK_RacError_NoRsrcAvail,                EAGAIN,         SMSG_RacError_NoRsrcAvail, 7 },
	{ BoK_RacError_NoRsrcAvail,                EAGAIN,         SMSG_RacError_NoRsrcAvail_s, 0 },
	{ BoK_RacError_NoRsrcAvail,                EAGAIN,         SMSG_RacError_NoRsrcAvail_s2, 0 },
	{ BoK_RacError_NoRsrcAvail,                EAGAIN,         SMSG_RacError_NoRsrcAvail_s3, 0 },
	{ BoK_RacError_NoRsrcAvail,                EAGAIN,         SMSG_RacError_NoRsrcAvail_i, 0 },
	{ BoK_RacError_NoRsrcAvail,                EAGAIN,         SMSG_RacError_NoRsrcAvail_si, 0 },
	{ BoK_RacError_NoRsrcAvail,                EAGAIN,         SMSG_RacError_NoRsrcAvail_s2i, 0 },

	/*
	 * STATE ERRORS
	 */
	{ BoK_StateError_Invalid,                  EINVAL,         SMSG_StateError_Invalid, 2 },
	{ BoK_StateError_Invalid,                  EINVAL,         SMSG_StateError_Invalid_s, 0 },
	{ BoK_StateError_NotConfigured,            ENXIO,          SMSG_StateError_NotConfigured, 7 },
	{ BoK_StateError_NotConfigured,            ENXIO,          SMSG_StateError_NotConfigured_s, 0 },
	{ BoK_StateError_NotConfigured,            ENXIO,          SMSG_StateError_NotConfigured_s2, 0 },
	{ BoK_StateError_NotConfigured,            ENXIO,          SMSG_StateError_NotConfigured_s3, 0 },
	{ BoK_StateError_NotConfigured,            ENXIO,          SMSG_StateError_NotConfigured_i, 0 },
	{ BoK_StateError_NotConfigured,            ENXIO,          SMSG_StateError_NotConfigured_si, 0 },
	{ BoK_StateError_NotConfigured,            ENXIO,          SMSG_StateError_NotConfigured_s2i, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState, 9 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_s, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_s2, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_s3, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_s4, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_i, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_i2, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_si2, 0 },
	{ BoK_StateError_WrongState,               EINVAL,         SMSG_StateError_WrongState_s2i2, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already, 9 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_s, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_s2, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_s3, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_s4, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_i, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_i2, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_si2, 0 },
	{ BoK_StateError_Already,                  EALREADY,       SMSG_StateError_Already_s2i2, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible, 9 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_s, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_s2, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_s3, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_s4, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_i, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_i2, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_si2, 0 },
	{ BoK_StateError_Destructible,             EINVAL,         SMSG_StateError_Destructible_s2i2, 0 },

	/*
	 * OPERATIONAL ERRORS
	 */
	{ BoK_OpError_Inappropriate,               ENODEV,         SMSG_OpError_Inappropriate, 7 },
	{ BoK_OpError_Inappropriate,               ENODEV,         SMSG_OpError_Inappropriate_i, 0 },
	{ BoK_OpError_Inappropriate,               ENODEV,         SMSG_OpError_Inappropriate_i2, 0 },
	{ BoK_OpError_Inappropriate,               ENODEV,         SMSG_OpError_Inappropriate_s, 0 },
	{ BoK_OpError_Inappropriate,               ENODEV,         SMSG_OpError_Inappropriate_s2, 0 },
	{ BoK_OpError_Inappropriate,               ENODEV,         SMSG_OpError_Inappropriate_is, 0 },
	{ BoK_OpError_Inappropriate,               ENODEV,         SMSG_OpError_Inappropriate_si, 0 },
	{ BoK_OpError_NotImpl,                     ENOSYS,         SMSG_OpError_NotImpl, 2 },
	{ BoK_OpError_NotImpl,                     ENOSYS,         SMSG_OpError_NotImpl_s, 0 },
	{ BoK_OpError_NotAvail,                    ENODEV,         SMSG_OpError_NotAvail, 7 },
	{ BoK_OpError_NotAvail,                    ENODEV,         SMSG_OpError_NotAvail_i, 0 },
	{ BoK_OpError_NotAvail,                    ENODEV,         SMSG_OpError_NotAvail_i2, 0 },
	{ BoK_OpError_NotAvail,                    ENODEV,         SMSG_OpError_NotAvail_s, 0 },
	{ BoK_OpError_NotAvail,                    ENODEV,         SMSG_OpError_NotAvail_s2, 0 },
	{ BoK_OpError_NotAvail,                    ENODEV,         SMSG_OpError_NotAvail_is, 0 },
	{ BoK_OpError_NotAvail,                    ENODEV,         SMSG_OpError_NotAvail_si, 0 },
	{ BoK_OpError_NotSupp,                     EOPNOTSUPP,     SMSG_OpError_NotSupp, 7 },
	{ BoK_OpError_NotSupp,                     EOPNOTSUPP,     SMSG_OpError_NotSupp_i, 0 },
	{ BoK_OpError_NotSupp,                     EOPNOTSUPP,     SMSG_OpError_NotSupp_i2, 0 },
	{ BoK_OpError_NotSupp,                     EOPNOTSUPP,     SMSG_OpError_NotSupp_s, 0 },
	{ BoK_OpError_NotSupp,                     EOPNOTSUPP,     SMSG_OpError_NotSupp_s2, 0 },
	{ BoK_OpError_NotSupp,                     EOPNOTSUPP,     SMSG_OpError_NotSupp_si, 0 },
	{ BoK_OpError_NotSupp,                     EOPNOTSUPP,     SMSG_OpError_NotSupp_is, 0 },
	{ BoK_OpError_FeatureNotSupp,              ENOTSUP,        SMSG_OpError_FeatureNotSupp, 7 },
	{ BoK_OpError_FeatureNotSupp,              ENOTSUP,        SMSG_OpError_FeatureNotSupp_i, 0 },
	{ BoK_OpError_FeatureNotSupp,              ENOTSUP,        SMSG_OpError_FeatureNotSupp_i2, 0 },
	{ BoK_OpError_FeatureNotSupp,              ENOTSUP,        SMSG_OpError_FeatureNotSupp_i3, 0 },
	{ BoK_OpError_FeatureNotSupp,              ENOTSUP,        SMSG_OpError_FeatureNotSupp_s, 0 },
	{ BoK_OpError_FeatureNotSupp,              ENOTSUP,        SMSG_OpError_FeatureNotSupp_s2, 0 },
	{ BoK_OpError_FeatureNotSupp,              ENOTSUP,        SMSG_OpError_FeatureNotSupp_s3, 0 },
	{ BoK_OpError_InProgress,                  EINPROGRESS,    SMSG_OpError_InProgress, 3 },
	{ BoK_OpError_InProgress,                  EINPROGRESS,    SMSG_OpError_InProgress_s, 0 },
	{ BoK_OpError_InProgress,                  EINPROGRESS,    SMSG_OpError_InProgress_s2, 0 },
	{ BoK_OpError_Already,                     EALREADY,       SMSG_OpError_Already, 3 },
	{ BoK_OpError_Already,                     EALREADY,       SMSG_OpError_Already_s, 0 },
	{ BoK_OpError_Already,                     EALREADY,       SMSG_OpError_Already_s2, 0 },
	{ BoK_OpError_Canceled,                    ECANCELED,      SMSG_OpError_Canceled, 3 },
	{ BoK_OpError_Canceled,                    ECANCELED,      SMSG_OpError_Canceled_s, 0 },
	{ BoK_OpError_Canceled,                    ECANCELED,      SMSG_OpError_Canceled_s2, 0 },
	{ BoK_OpError_Interrupted,                 EINTR,          SMSG_OpError_Interrupted, 3 },
	{ BoK_OpError_Interrupted,                 EINTR,          SMSG_OpError_Interrupted_s, 0 },
	{ BoK_OpError_Interrupted,                 EINTR,          SMSG_OpError_Interrupted_s2, 0 },
	{ BoK_OpError_TimedOut,                    ETIMEDOUT,      SMSG_OpError_TimedOut, 3 },
	{ BoK_OpError_TimedOut,                    ETIMEDOUT,      SMSG_OpError_TimedOut_s, 0 },
	{ BoK_OpError_TimedOut,                    ETIMEDOUT,      SMSG_OpError_TimedOut_s2, 0 },
	{ BoK_OpError_UnwillingToPerform,          EINVAL,         SMSG_OpError_UnwillingToPerform, 3 },
	{ BoK_OpError_UnwillingToPerform,          EINVAL,         SMSG_OpError_UnwillingToPerform_s, 0 },
	{ BoK_OpError_UnwillingToPerform,          EINVAL,         SMSG_OpError_UnwillingToPerform_s2, 0 },
	{ BoK_OpError_NotAvailIf,                  ENODEV,         SMSG_OpError_NotAvailIf_s, 7 },
	{ BoK_OpError_NotAvailIf,                  ENODEV,         SMSG_OpError_NotAvailIf_is, 0 },
	{ BoK_OpError_NotAvailIf,                  ENODEV,         SMSG_OpError_NotAvailIf_i2s, 0 },
	{ BoK_OpError_NotAvailIf,                  ENODEV,         SMSG_OpError_NotAvailIf_s2, 0 },
	{ BoK_OpError_NotAvailIf,                  ENODEV,         SMSG_OpError_NotAvailIf_s3, 0 },
	{ BoK_OpError_NotAvailIf,                  ENODEV,         SMSG_OpError_NotAvailIf_is2, 0 },
	{ BoK_OpError_NotAvailIf,                  ENODEV,         SMSG_OpError_NotAvailIf_sis, 0 },
	{ BoK_OpError_NotSuppIf,                   EOPNOTSUPP,     SMSG_OpError_NotSuppIf_s, 7 },
	{ BoK_OpError_NotSuppIf,                   EOPNOTSUPP,     SMSG_OpError_NotSuppIf_is, 0 },
	{ BoK_OpError_NotSuppIf,                   EOPNOTSUPP,     SMSG_OpError_NotSuppIf_i2s, 0 },
	{ BoK_OpError_NotSuppIf,                   EOPNOTSUPP,     SMSG_OpError_NotSuppIf_s2, 0 },
	{ BoK_OpError_NotSuppIf,                   EOPNOTSUPP,     SMSG_OpError_NotSuppIf_s3, 0 },
	{ BoK_OpError_NotSuppIf,                   EOPNOTSUPP,     SMSG_OpError_NotSuppIf_is2, 0 },
	{ BoK_OpError_NotSuppIf,                   EOPNOTSUPP,     SMSG_OpError_NotSuppIf_sis, 0 },

	/*
	 * TEXT ERRORS
	 */
	{ BoK_TextError_IllegalMBSequence,         EILSEQ,         SMSG_TextError_IllegalMBSequence, 1 },
	{ BoK_TextError_IncompleteMBSequence,      EILSEQ,         SMSG_TextError_IncompleteMBSequence, 1 },
	{ BoK_TextError_BadUnicodeCharacter,       EILSEQ,         SMSG_TextError_BadUnicodeCharacter, 1 },
	{ BoK_TextError_MBOverflow,                EOVERFLOW,      SMSG_TextError_MBOverflow, 1 },
	{ BoK_TextError_TokenMaxLen,               EINVAL,         SMSG_TextError_TokenMaxLen, 2 },
	{ BoK_TextError_TokenMaxLen,               EINVAL,         SMSG_TextError_TokenMaxLen_s, 0 },
	{ BoK_TextError_InvalidSyntax,             EINVAL,         SMSG_TextError_InvalidSyntax, 2 },
	{ BoK_TextError_InvalidSyntax,             EINVAL,         SMSG_TextError_InvalidSyntax_s2, 0 },
	{ BoK_TextError_EndOfText,                 EINVAL,         SMSG_TextError_EndOfText, 2 },
	{ BoK_TextError_EndOfText,                 EINVAL,         SMSG_TextError_EndOfText_s, 0 },
	{ BoK_TextError_UnmatchedCharacter,        EINVAL,         SMSG_TextError_UnmatchedCharacter, 2 },
	{ BoK_TextError_UnmatchedCharacter,        EINVAL,         SMSG_TextError_UnmatchedCharacter_s2c, 0 },
	{ BoK_TextError_EmptyToken,                EINVAL,         SMSG_TextError_EmptyToken, 2 },
	{ BoK_TextError_EmptyToken,                EINVAL,         SMSG_TextError_EmptyToken_s2, 0 },
	{ BoK_TextError_UnexpectedToken,           EINVAL,         SMSG_TextError_UnexpectedToken, 3 },
	{ BoK_TextError_UnexpectedToken,           EINVAL,         SMSG_TextError_UnexpectedToken_s2, 0 },
	{ BoK_TextError_UnexpectedToken,           EINVAL,         SMSG_TextError_UnexpectedToken_s3, 0 },
	{ BoK_TextError_MissingToken,              EINVAL,         SMSG_TextError_MissingToken, 3 },
	{ BoK_TextError_MissingToken,              EINVAL,         SMSG_TextError_MissingToken_s2, 0 },
	{ BoK_TextError_MissingToken,              EINVAL,         SMSG_TextError_MissingToken_s3, 0 },
	{ BoK_TextError_BadToken,                  EINVAL,         SMSG_TextError_BadToken, 3 },
	{ BoK_TextError_BadToken,                  EINVAL,         SMSG_TextError_BadToken_s2, 0 },
	{ BoK_TextError_BadToken,                  EINVAL,         SMSG_TextError_BadToken_s3, 0 },

	/*
	 * AUTHENTICATION ERRORS
	 */
	{ BoK_AuthError_NoInformation,             EAUTH,          SMSG_AuthError_NoInformation, 3 },
	{ BoK_AuthError_NoInformation,             EAUTH,          SMSG_AuthError_NoInformation_s, 0 },
	{ BoK_AuthError_NoInformation,             EAUTH,          SMSG_AuthError_NoInformation_s2, 0 },
	{ BoK_AuthError_NeedAuth,                  ENEEDAUTH,      SMSG_AuthError_NeedAuth, 3 },
	{ BoK_AuthError_NeedAuth,                  ENEEDAUTH,      SMSG_AuthError_NeedAuth_s, 0 },
	{ BoK_AuthError_NeedAuth,                  ENEEDAUTH,      SMSG_AuthError_NeedAuth_s2, 0 },

	/*
	 * MATH ERRORS
	 */
	{ BoK_MathError_OutOfDomain,               EDOM,           SMSG_MathError_OutOfDomain, 3 },
	{ BoK_MathError_OutOfDomain,               EDOM,           SMSG_MathError_OutOfDomain_s, 0 },
	{ BoK_MathError_OutOfDomain,               EDOM,           SMSG_MathError_OutOfDomain_s2, 0 },
	{ BoK_MathError_ResultRange,               ERANGE,         SMSG_MathError_ResultRange, 3 },
	{ BoK_MathError_ResultRange,               ERANGE,         SMSG_MathError_ResultRange_s, 0 },
	{ BoK_MathError_ResultRange,               ERANGE,         SMSG_MathError_ResultRange_s2, 0 },

	/*
	 * COMMON SYSTEM ERRORS
	 */
	{ BoK_SysError_NoInformation,              EINVAL,         SMSG_SysError_NoInformation, 3 },
	{ BoK_SysError_NoInformation,              EINVAL,         SMSG_SysError_NoInformation_i, 0 },
	{ BoK_SysError_NoInformation,              EINVAL,         SMSG_SysError_NoInformation_s, 0 },
	{ BoK_SysError_NoSuchProcess,              ESRCH,          SMSG_SysError_NoSuchProcess, 3 },
	{ BoK_SysError_NoSuchProcess,              ESRCH,          SMSG_SysError_NoSuchProcess_i, 0 },
	{ BoK_SysError_NoSuchProcess,              ESRCH,          SMSG_SysError_NoSuchProcess_s, 0 },
	{ BoK_SysError_InvalidExecFormat,          ENOEXEC,        SMSG_SysError_InvalidExecFormat, 2 },
	{ BoK_SysError_InvalidExecFormat,          ENOEXEC,        SMSG_SysError_InvalidExecFormat_s, 0 },
	{ BoK_SysError_NoChildToWait,              ECHILD,         SMSG_SysError_NoChildToWait, 1 },
	{ BoK_SysError_NoMemory,                   ENOMEM,         SMSG_SysError_NoMemory, 4 },
	{ BoK_SysError_NoMemory,                   ENOMEM,         SMSG_SysError_NoMemory_s, 0 },
	{ BoK_SysError_NoMemory,                   ENOMEM,         SMSG_SysError_ObjectCreate_s, 0 },
	{ BoK_SysError_NoMemory,                   ENOMEM,         SMSG_SysError_Strdup_s, 0 },
	{ BoK_SysError_ProcsLimit,                 EPROCLIM,       SMSG_SysError_ProcsLimit, 1 },
	{ BoK_SysError_UsersLimit,                 EUSERS,         SMSG_SysError_UsersLimit, 1 },
	{ BoK_SysError_DiskQuota,                  EDQUOT,         SMSG_SysError_DiskQuota, 1 },

	/*
	 * I/O ERRORS
	 */
	{ BoK_IoError_NoInformation,               EIO,            SMSG_IoError_NoInformation, 2 },
	{ BoK_IoError_NoInformation,               EIO,            SMSG_IoError_NoInformation_i, 0 },
	{ BoK_IoError_NotConfigured,               ENXIO,          SMSG_IoError_NotConfigured, 2 },
	{ BoK_IoError_NotConfigured,               ENXIO,          SMSG_IoError_NotConfigured_s, 0 },
	{ BoK_IoError_NotBlockDevice,              ENOTBLK,        SMSG_IoError_NotBlockDevice, 2 },
	{ BoK_IoError_NotBlockDevice,              ENOTBLK,        SMSG_IoError_NotBlockDevice_s, 0 },
	{ BoK_IoError_NotSeekable,                 ESPIPE,         SMSG_IoError_NotSeekable, 3 },
	{ BoK_IoError_NotSeekable,                 ESPIPE,         SMSG_IoError_NotSeekable_s, 0 },
	{ BoK_IoError_NotSeekable,                 ESPIPE,         SMSG_IoError_NotSeekable_s2, 0 },
	{ BoK_IoError_NotSuppDevice,               ENOTBLK,        SMSG_IoError_NotSuppDevice, 3 },
	{ BoK_IoError_NotSuppDevice,               ENOTBLK,        SMSG_IoError_NotSuppDevice_s, 0 },
	{ BoK_IoError_NotSuppDevice,               ENOTBLK,        SMSG_IoError_NotSuppDevice_s2, 0 },
	{ BoK_IoError_NoPortAvail,                 ENOSPC,         SMSG_IoError_NoPortAvail, 2 },
	{ BoK_IoError_NoPortAvail,                 ENOSPC,         SMSG_IoError_NoPortAvail_s, 0 },

	/*
	 * FILE ERRORS
	 */
	{ BoK_FileError_NoSuchEntry,               ENOENT,         SMSG_FileError_NoSuchEntry, 4 },
	{ BoK_FileError_NoSuchEntry,               ENOENT,         SMSG_FileError_NoSuchEntry_s, 0 },
	{ BoK_FileError_NoSuchEntry,               ENOENT,         SMSG_FileError_NotFound_s, 0 },
	{ BoK_FileError_NoSuchEntry,               ENOENT,         SMSG_FileError_NotFound_s2, 0 },
	{ BoK_FileError_BadDescriptor,             EBADF,          SMSG_FileError_BadDescriptor, 2 },
	{ BoK_FileError_BadDescriptor,             EBADF,          SMSG_FileError_BadDescriptor_i, 0 },
	{ BoK_FileError_Busy,                      EBUSY,          SMSG_FileError_Busy, 2 },
	{ BoK_FileError_Busy,                      EBUSY,          SMSG_FileError_Busy_s, 0 },
	{ BoK_FileError_AlreadyExists,             EEXIST,         SMSG_FileError_AlreadyExists, 2 },
	{ BoK_FileError_AlreadyExists,             EEXIST,         SMSG_FileError_AlreadyExists_s, 0 },
	{ BoK_FileError_CrossDeviceLink,           EXDEV,          SMSG_FileError_CrossDeviceLink, 1 },
	{ BoK_FileError_OperationNotAvail,         ENODEV,         SMSG_FileError_OperationNotAvail, 3 },
	{ BoK_FileError_OperationNotAvail,         ENODEV,         SMSG_FileError_OperationNotAvail_s, 0 },
	{ BoK_FileError_OperationNotAvail,         ENODEV,         SMSG_FileError_OperationNotAvail_s2, 0 },
	{ BoK_FileError_NotDir,                    ENOTDIR,        SMSG_FileError_NotDir, 2 },
	{ BoK_FileError_NotDir,                    ENOTDIR,        SMSG_FileError_NotDir_s, 0 },
	{ BoK_FileError_IsDir,                     EISDIR,         SMSG_FileError_IsDir, 2 },
	{ BoK_FileError_IsDir,                     EISDIR,         SMSG_FileError_IsDir_s, 0 },
	{ BoK_FileError_SysDescLimit,              ENFILE,         SMSG_FileError_SysDescLimit, 1 },
	{ BoK_FileError_ProcessDescLimit,          EMFILE,         SMSG_FileError_ProcessDescLimit, 1 },
	{ BoK_FileError_IoctlOperationNotAvail,    ENOTTY,         SMSG_FileError_IoctlOperationNotAvail, 2 },
	{ BoK_FileError_IoctlOperationNotAvail,    ENOTTY,         SMSG_FileError_IoctlOperationNotAvail_i, 0 },
	{ BoK_FileError_TextBusy,                  ETXTBSY,        SMSG_FileError_TextBusy, 2 },
	{ BoK_FileError_TextBusy,                  ETXTBSY,        SMSG_FileError_TextBusy_s, 0 },
	{ BoK_FileError_FileSizeLimit,             EFBIG,          SMSG_FileError_FileSizeLimit, 2 },
	{ BoK_FileError_FileSizeLimit,             EFBIG,          SMSG_FileError_FileSizeLimit_s, 0 },
	{ BoK_FileError_NoFreeSpace,               ENOSPC,         SMSG_FileError_NoFreeSpace, 1 },
	{ BoK_FileError_SeekOnPipe,                ESPIPE,         SMSG_FileError_SeekOnPipe, 1 },
	{ BoK_FileError_ReadOnlyFS,                EROFS,          SMSG_FileError_ReadOnlyFS, 2 },
	{ BoK_FileError_ReadOnlyFS,                EROFS,          SMSG_FileError_ReadOnlyFS_s, 0 },
	{ BoK_FileError_LinksLimit,                EMLINK,         SMSG_FileError_LinksLimit, 1 },
	{ BoK_FileError_Loop,                      ELOOP,          SMSG_FileError_Loop, 2 },
	{ BoK_FileError_Loop,                      ELOOP,          SMSG_FileError_Loop_s, 0 },
	{ BoK_FileError_NameSizeLimit,             ENAMETOOLONG,   SMSG_FileError_NameSizeLimit, 4 },
	{ BoK_FileError_NameSizeLimit,             ENAMETOOLONG,   SMSG_FileError_NameSizeLimit_s, 0 },
	{ BoK_FileError_NameSizeLimit,             ENAMETOOLONG,   SMSG_FileError_PathSizeLimit, 0 },
	{ BoK_FileError_NameSizeLimit,             ENAMETOOLONG,   SMSG_FileError_PathSizeLimit_s, 0 },
	{ BoK_FileError_DirNotEmpty,               ENOTEMPTY,      SMSG_FileError_DirNotEmpty, 2 },
	{ BoK_FileError_DirNotEmpty,               ENOTEMPTY,      SMSG_FileError_DirNotEmpty_s, 0 },
	{ BoK_FileError_NoLockAvail,               ENOLCK,         SMSG_FileError_NoLockAvail, 1 },

	/*
	 * IPC ERRORS
	 */
	{ BoK_IpcError_NotSocket,                  ENOTSOCK,        SMSG_IpcError_NotSocket, 4 },
	{ BoK_IpcError_NotSocket,                  ENOTSOCK,        SMSG_IpcError_NotSocket_s, 0 },
	{ BoK_IpcError_NotSocket,                  ENOTSOCK,        SMSG_IpcError_NotSocket_is, 0 },
	{ BoK_IpcError_NotSocket,                  ENOTSOCK,        SMSG_IpcError_NotSocket_s2, 0 },
	{ BoK_IpcError_DestAddrRequired,           EDESTADDRREQ,    SMSG_IpcError_DestAddrRequired, 1 },
	{ BoK_IpcError_MsgSizeLimit,               EMSGSIZE,        SMSG_IpcError_MsgSizeLimit, 1 },
	{ BoK_IpcError_Prototype,                  EPROTOTYPE,      SMSG_IpcError_Prototype, 1 },
	{ BoK_IpcError_ProtoNotAvail,              ENOPROTOOPT,     SMSG_IpcError_ProtoNotAvail, 1 },
	{ BoK_IpcError_ProtoNotSupp,               EPROTONOSUPPORT, SMSG_IpcError_ProtoNotSupp, 2 },
	{ BoK_IpcError_ProtoNotSupp,               EPROTONOSUPPORT, SMSG_IpcError_ProtoNotSupp_s, 0 },
	{ BoK_IpcError_SockTypeNotSupp,            ESOCKTNOSUPPORT, SMSG_IpcError_SockTypeNotSupp, 2 },
	{ BoK_IpcError_SockTypeNotSupp,            ESOCKTNOSUPPORT, SMSG_IpcError_SockTypeNotSupp_s, 0 },
	{ BoK_IpcError_OperationNotAvailOnSock,    EOPNOTSUPP,      SMSG_IpcError_OperationNotAvailOnSock, 2 },
	{ BoK_IpcError_OperationNotAvailOnSock,    EOPNOTSUPP,      SMSG_IpcError_OperationNotAvailOnSock_s, 0 },
	{ BoK_IpcError_PFNotSupp,                  EPFNOSUPPORT,    SMSG_IpcError_PFNotSupp, 2 },
	{ BoK_IpcError_PFNotSupp,                  EPFNOSUPPORT,    SMSG_IpcError_PFNotSupp_i, 0 },
	{ BoK_IpcError_AFNotSupp,                  EAFNOSUPPORT,    SMSG_IpcError_AFNotSupp, 2 },
	{ BoK_IpcError_AFNotSupp,                  EAFNOSUPPORT,    SMSG_IpcError_AFNotSupp_i2, 0 },
	{ BoK_IpcError_AddrInUse,                  EADDRINUSE,      SMSG_IpcError_AddrInUse, 1 },
	{ BoK_IpcError_AddrNotAvail,               EADDRNOTAVAIL,   SMSG_IpcError_AddrNotAvail, 1 },
	{ BoK_IpcError_NetDown,                    ENETDOWN,        SMSG_IpcError_NetDown, 1 },
	{ BoK_IpcError_NetUnreach,                 ENETUNREACH,     SMSG_IpcError_NetUnreach, 1 },
	{ BoK_IpcError_NetReset,                   ENETRESET,       SMSG_IpcError_NetReset, 1 },
	{ BoK_IpcError_ConnectionAborted,          ECONNABORTED,    SMSG_IpcError_ConnectionAborted, 1 },
	{ BoK_IpcError_ConnectionReset,            ECONNRESET,      SMSG_IpcError_ConnectionReset, 1 },
	{ BoK_IpcError_NoBufferSpace,              ENOBUFS,         SMSG_IpcError_NoBufferSpace, 1 },
	{ BoK_IpcError_IsConnected,                EISCONN,         SMSG_IpcError_IsConnected, 1 },
	{ BoK_IpcError_NotConnected,               ENOTCONN,        SMSG_IpcError_NotConnected, 1 },
	{ BoK_IpcError_Shutdown,                   ESHUTDOWN,       SMSG_IpcError_Shutdown, 1 },
	{ BoK_IpcError_ConnectionRefused,          ECONNREFUSED,    SMSG_IpcError_ConnectionRefused, 1 },
	{ BoK_IpcError_HostDown,                   EHOSTDOWN,       SMSG_IpcError_HostDown, 1 },
	{ BoK_IpcError_HostUnreach,                EHOSTUNREACH,    SMSG_IpcError_HostUnreach, 1 },
	{ BoK_IpcError_BrokenPipe,                 EPIPE,           SMSG_IpcError_BrokenPipe, 1 },
	{ BoK_IpcError_BadMsg,                     EBADMSG,         SMSG_IpcError_BadMsg, 1 },

	/*
	 * AIB ERRORS
	 */
	{ BoK_AIBNameError_InvalidSyntax,          EINVAL,         SMSG_AIBNameError_InvalidSyntax, 2 },
	{ BoK_AIBNameError_InvalidSyntax,          EINVAL,         SMSG_AIBNameError_InvalidSyntax_s, 0 },
	{ BoK_AIBNameError_NoSuchEntry,            ENOENT,         SMSG_AIBNameError_NoSuchEntry, 3 },
	{ BoK_AIBNameError_NoSuchEntry,            ENOENT,         SMSG_AIBNameError_NoSuchEntry_i, 0 },
	{ BoK_AIBNameError_NoSuchEntry,            ENOENT,         SMSG_AIBNameError_NoSuchEntry_s, 0 },
	{ BoK_AIBNameError_SizeLimit,              ENAMETOOLONG,   SMSG_AIBNameError_SizeLimit, 2 },
	{ BoK_AIBNameError_SizeLimit,              ENAMETOOLONG,   SMSG_AIBNameError_SizeLimit_s, 0 },

	{ BoK_AIBAttrError_InvalidSyntax,          EINVAL,         SMSG_AIBAttrError_InvalidSyntax, 2 },
	{ BoK_AIBAttrError_InvalidSyntax,          EINVAL,         SMSG_AIBAttrError_InvalidSyntax_s, 0 },
	{ BoK_AIBAttrError_NoSuchAttribute,        ENOENT,         SMSG_AIBAttrError_NoSuchEntry, 3 },
	{ BoK_AIBAttrError_NoSuchAttribute,        ENOENT,         SMSG_AIBAttrError_NoSuchEntry_s, 0 },
	{ BoK_AIBAttrError_NoSuchAttribute,        ENOENT,         SMSG_AIBAttrError_NoSuchEntry_s2, 0 },
	{ BoK_AIBAttrError_AlreadyExists,          EEXIST,         SMSG_AIBAttrError_AlreadyExists, 2 },
	{ BoK_AIBAttrError_AlreadyExists,          EEXIST,         SMSG_AIBAttrError_AlreadyExists_s, 0 },

	{ BoK_AIBUpdateError_AlreadyExists,        EEXIST,         SMSG_AIBUpdateError_AlreadyExists, 3 },
	{ BoK_AIBUpdateError_AlreadyExists,        EEXIST,         SMSG_AIBUpdateError_AlreadyExists_s, 0 },
	{ BoK_AIBUpdateError_AlreadyExists,        EEXIST,         SMSG_AIBUpdateError_AlreadyExists_p_s, 0 },
	{ BoK_AIBUpdateError_NoSuchParent,         ENOENT,         SMSG_AIBUpdateError_NoSuchParent, 2 },
	{ BoK_AIBUpdateError_NoSuchParent,         ENOENT,         SMSG_AIBUpdateError_NoSuchParent_s, 0 },
	{ BoK_AIBUpdateError_NotAllowedOnTheEntry, ENODEV,         SMSG_AIBUpdateError_NotAllowedOnTheEntry, 5 },
	{ BoK_AIBUpdateError_NotAllowedOnTheEntry, ENODEV,         SMSG_AIBUpdateError_NotAllowedOnTheEntry_i, 0 },
	{ BoK_AIBUpdateError_NotAllowedOnTheEntry, ENODEV,         SMSG_AIBUpdateError_NotAllowedOnTheEntry_i2, 3 },
	{ BoK_AIBUpdateError_NotAllowedOnTheEntry, ENODEV,         SMSG_AIBUpdateError_NotAllowedOnTheEntry_s, 0 },
	{ BoK_AIBUpdateError_NotAllowedOnTheEntry, ENODEV,         SMSG_AIBUpdateError_NotAllowedOnTheEntry_s2, 0 },
	{ BoK_AIBUpdateError_IsCompound,           EISDIR,         SMSG_AIBUpdateError_IsCompound, 2 },
	{ BoK_AIBUpdateError_IsCompound,           EISDIR,         SMSG_AIBUpdateError_IsCompound_s, 0 },
	{ BoK_AIBUpdateError_NotCompound,          ENOTDIR,        SMSG_AIBUpdateError_NotCompound, 2 },
	{ BoK_AIBUpdateError_NotCompound,          ENOTDIR,        SMSG_AIBUpdateError_NotCompound_s, 0 },
	{ BoK_AIBUpdateError_IsEmptyCompound,      EINVAL,         SMSG_AIBUpdateError_IsEmptyCompound, 2 },
	{ BoK_AIBUpdateError_IsEmptyCompound,      EINVAL,         SMSG_AIBUpdateError_IsEmptyCompound_s, 0 },
	{ BoK_AIBUpdateError_NotEmptyCompound,     ENOTEMPTY,      SMSG_AIBUpdateError_NotEmptyCompound, 2 },
	{ BoK_AIBUpdateError_NotEmptyCompound,     ENOTEMPTY,      SMSG_AIBUpdateError_NotEmptyCompound_s, 0 },
	{ BoK_AIBUpdateError_CompoundOverflow,     EINVAL,         SMSG_AIBUpdateError_CompoundOverflow, 2 },
	{ BoK_AIBUpdateError_CompoundOverflow,     EINVAL,         SMSG_AIBUpdateError_CompoundOverflow_s, 0 },
	{ BoK_AIBUpdateError_IsInserted,           EINVAL,         SMSG_AIBUpdateError_IsInserted, 3 },
	{ BoK_AIBUpdateError_IsInserted,           EINVAL,         SMSG_AIBUpdateError_IsInserted_s, 0 },
	{ BoK_AIBUpdateError_IsInserted,           EINVAL,         SMSG_AIBUpdateError_IsInserted_s2, 0 },
	{ BoK_AIBUpdateError_NotInserted,          EINVAL,         SMSG_AIBUpdateError_NotInserted, 2 },
	{ BoK_AIBUpdateError_NotInserted,          EINVAL,         SMSG_AIBUpdateError_NotInserted_s, 0 },
	{ BoK_AIBUpdateError_IsMounted,            EINVAL,         SMSG_AIBUpdateError_IsMounted, 2 },
	{ BoK_AIBUpdateError_IsMounted,            EINVAL,         SMSG_AIBUpdateError_IsMounted_s2, 0 },
	{ BoK_AIBUpdateError_NotMounted,           EINVAL,         SMSG_AIBUpdateError_NotMounted, 2 },
	{ BoK_AIBUpdateError_NotMounted,           EINVAL,         SMSG_AIBUpdateError_NotMounted_s, 0 },
	{ BoK_AIBUpdateError_MultipleMountNotSupp, ENOTSUP,        SMSG_AIBUpdateError_MultipleMountNotSupp, 2 },
	{ BoK_AIBUpdateError_MultipleMountNotSupp, ENOTSUP,        SMSG_AIBUpdateError_MultipleMountNotSupp_s, 0 },
	{ BoK_AIBUpdateError_NotMountpoint,        ENODEV,         SMSG_AIBUpdateError_NotMountpoint, 2 },
	{ BoK_AIBUpdateError_NotMountpoint,        ENODEV,         SMSG_AIBUpdateError_NotMountpoint_s, 0 },
	{ BoK_AIBUpdateError_NotMountable,         ENODEV,         SMSG_AIBUpdateError_NotMountable, 2 },
	{ BoK_AIBUpdateError_NotMountable,         ENODEV,         SMSG_AIBUpdateError_NotMountable_s, 0 },

	{ BoK_AIBServiceError_NoInformation,       EINVAL,         SMSG_AIBServiceError_NoInformation, 3 },
	{ BoK_AIBServiceError_NoInformation,       EINVAL,         SMSG_AIBServiceError_NoInformation_i, 0 },
	{ BoK_AIBServiceError_NoInformation,       EINVAL,         SMSG_AIBServiceError_NoInformation_s, 0 },
	{ BoK_AIBServiceError_Busy,                EBUSY,          SMSG_AIBServiceError_Busy, 4 },
	{ BoK_AIBServiceError_Busy,                EBUSY,          SMSG_AIBServiceError_Busy_s, 0 },
	{ BoK_AIBServiceError_Busy,                EBUSY,          SMSG_AIBServiceError_Busy_si, 0 },
	{ BoK_AIBServiceError_Busy,                EBUSY,          SMSG_AIBServiceError_Busy_s2, 0 },
	{ BoK_AIBServiceError_NestLevelLimit,      ENAMETOOLONG,   SMSG_AIBServiceError_NestLevelLimit, 1 },

	{ _ErrK_UNK_BO_ECODE, _ErrK_UNK_UNIX_ECODE, NULL, 0 }
};

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
