/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _KumfamP.h	(V.Sokur)
 * include/Bo/services/alg/_KumfamP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG__KUMFAM_P__
# define __BO_SERVICES_ALG__KUMFAM_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/_Kumfam.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# if 0
#  define _KumfamK_FAM_LEN FAM_LENGTH
# else
#  define _KumfamK_FAM_LEN 4
# endif

_A_opdom(Kumfam) {
	char  cfam_data[_KumfamK_FAM_LEN];
}
_A_end;

/* ----------------------------------------------------------------------------
 * {ucntl,Kumfam} protocol
 * @{
 */

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (ucntl,Kumfam)
# define __aorpc_realize_service_file <Bo/services/UcntlP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol(ucntl,Kumfam)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop(pod_init),
_A_imop(pod_fini),

_A_imop_nf(retain_value,2,(ucntl,PoolUcntl)),
_A_imop_nf(retain,2,(ucntl,PoolUcntl)),
_A_imop_nf(release,2,(ucntl,PoolUcntl)),
_A_imop(hash),
_A_imop(compare),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

/* ----------------------------------------------------------------------------
 * {ucntl,Kumfam,1} protocol
 * The protocol designed to be used for custom object with const data size,
 * known at compile time.
 * Basic for (ucntl,Kumfam,TypeT)
 * @{
 */

# define __aorpc_ns_protocol_c 3
# define __aorpc_ns_protocol (ucntl,Kumfam,1)
# define __aorpc_realize_service_file <Bo/services/UcntlP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol_(3,(ucntl,Kumfam,1))
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop(pod_init),
_A_imop(pod_fini),

_A_imop_nf(retain_value,2,(ucntl,Kumfam)),
_A_imop_nf(retain,2,(ucntl,Kumfam)),
_A_imop_nf(release,2,(ucntl,Kumfam)),
_A_imop_nf(hash,2,(ucntl,Kumfam)),
_A_imop_nf(compare,2,(ucntl,Kumfam)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

/* ----------------------------------------------------------------------------
 * {ucntl,Kumfam,2} protocol
 * The protocol designed to be used for custom object with callbacks.
 * @{
 */

# define __aorpc_ns_protocol_c 3
# define __aorpc_ns_protocol (ucntl,Kumfam,2)
# define __aorpc_realize_service_file <Bo/services/UcntlP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol_(3,(ucntl,Kumfam,2))
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop(pod_init),
_A_imop(pod_fini),

_A_imop_nf(retain_value,2,(ucntl,Kumfam)),
_A_imop_nf(retain,2,(ucntl,Kumfam)),
_A_imop_nf(release,2,(ucntl,Kumfam)),
_A_imop(hash),
_A_imop_nf(compare,2,(ucntl,Kumfam)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

/* ----------------------------------------------------------------------------
 * {ucntl,Kumfam,TypeT} protocol
 * Generic declaration to use with (i32, i64, f32, f64, ...)
 * Dummy protocol
 * @{
 */

# define __aorpc_ns_protocol_c 3
# define __aorpc_ns_protocol (ucntl,Kumfam,TypeT)
# define __aorpc_realize_service_file <Bo/services/UcntlP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol_(3,(ucntl,Kumfam,TypeT))
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop_nf(pod_init,3,(ucntl,Kumfam,1)),
_A_imop_nf(pod_fini,3,(ucntl,Kumfam,1)),

_A_imop_nf(retain_value,3,(ucntl,Kumfam,1)),
_A_imop_nf(retain,3,(ucntl,Kumfam,1)),
_A_imop_nf(release,3,(ucntl,Kumfam,1)),
_A_imop(hash),
_A_imop(compare),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

#endif /* !__BO_SERVICES_ALG__KUMFAM_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
