/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Ctnrevents.h	(N.Panasiuk)
 * include/Bo/services/alg/basis/Ctnrevents.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_BASIS_CTNREVENTS__
# define __BO_SERVICES_ALG_BASIS_CTNREVENTS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Noticedefs.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define _AORP_MAKE_CTNR_NEVENT(x) BO_MAKE_NEVENT(AorpK_container_service,x)

# define _AorpK_CtnrEvent_Overflow 3
# define _AorpK_CtnrEvent_Empty 4
# define _AorpK_CtnrEvent_Notempty 5
# define _AorpK_CtnrEvent_Insert 6
# define _AorpK_CtnrEvent_Remove 7
# define _AorpK_CtnrEvent_Update 8

/* OVERFLOW-EVENT-SPECIFICATION @{ */
# define AorpK_CtnrEvent_Overflow _AORP_MAKE_CTNR_NEVENT(_AorpK_CtnrEvent_Overflow)
/*
	overflow-notice = [
		.n_event = AorpK_CtnrEvent_Overflow;
		.u.n_val32 = @container.size;
		.n_sender = @acceptor;
	];

	overflow-vaxevent = [
		.vxe_event = AorpK_CtnrEvent_Overflow;
		.u.vxe_val32 = @container.size;
		.vxe_sender = @acceptor;
	];
 */
/* @} */

/* EMPTY-EVENT-SPECIFICATION @{ */
# define AorpK_CtnrEvent_Empty _AORP_MAKE_CTNR_NEVENT(_AorpK_CtnrEvent_Empty)
/*
	empty-notice = [
		.n_event = AorpK_CtnrEvent_Empty;
		.n_sender = @acceptor;
	];

	empty-vaxevent = [
		.vxe_event = AorpK_CtnrEvent_Empty;
		.vxe_sender = @acceptor;
	];
 */
/* @} */

/* NOTEMPTY-EVENT-SPECIFICATION @{ */
# define AorpK_CtnrEvent_Notempty _AORP_MAKE_CTNR_NEVENT(_AorpK_CtnrEvent_Notempty)
/*
	notempty-notice = [
		.n_event = AorpK_CtnrEvent_Notempty;
		.u.n_val32 = @container.size;
		.n_sender = @acceptor;
	];

	notempty-vaxevent = [
		.vxe_event = AorpK_CtnrEvent_Notempty;
		.u.vxe_val32 = 0;
		.vxe_sender = @container.size;
	];
 */
/* @} */

/* INSERT-EVENT-SPECIFICATION @{ */
# define AorpK_CtnrEvent_Insert _AORP_MAKE_CTNR_NEVENT(_AorpK_CtnrEvent_Insert)
/*
	insert-notice = [
		.n_event = AorpK_CtnrEvent_Insert;
		.u.n_val32 = @container.size;
		.n_sender = @acceptor;
	];

	insert-vaxevent = [
		.vxe_event = AorpK_CtnrEvent_Insert;
		.u.vxe_val32 = @container.size;
		.vxe_sender = @acceptor;
	];
 */
/* @} */

/* REMOVE-EVENT-SPECIFICATION @{ */
# define AorpK_CtnrEvent_Remove _AORP_MAKE_CTNR_NEVENT(_AorpK_CtnrEvent_Remove)
/*
	remove-notice = [
		.n_event = AorpK_CtnrEvent_Remove;
		.u.n_val32 = @container.size;
		.n_sender = @acceptor;
	];

	remove-vaxevent = [
		.vxe_event = AorpK_CtnrEvent_Remove;
		.u.vxe_val32 = @container.size;
		.vxe_sender = @acceptor;
	];
 */
/* @} */

/* UPDATE-EVENT-SPECIFICATION @{ */
# define AorpK_CtnrEvent_Update _AORP_MAKE_CTNR_NEVENT(_AorpK_CtnrEvent_Update)
/*
	update-notice = [
		.n_event = AorpK_CtnrEvent_Update;
		.n_sender = @acceptor;
	];

	update-vaxevent = [
		.vxe_event = AorpK_CtnrEvent_Update;
		.vxe_sender = @acceptor;
	];
 */
/* @} */

#endif /* !__BO_SERVICES_ALG_BASIS_CTNREVENTS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
