/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _CHashP.h	(V.Sokur)
 * include/Bo/services/alg/crypt/_CHashP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_CRYPT__CHASH_P__
# define __BO_SERVICES_ALG_CRYPT__CHASH_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/crypt/_CHash.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/*
 * {CHash,chash} protocol
 * @{
 */

# include <Bo/services/crypt/CHashP.h>

# ifdef __need_CHash_chash_protocol_impl
#  define __aorpc_impl 1
# endif
# define __aorpc_decl __aorpc_decl_protocol
# define __aorpc_service_name chash
# define __aorp_service_opdom CHash
# include <Bo/aorp/AorpC.h>

_A_protocol(CHash,chash)
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop_nil(pod_init),
_A_imop_nil(pod_fini),

_A_imop(openalg),
_A_imop(closealg),

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

/*
 * {CHashal,chashal} protocol
 * @{
 */

/* Common */
# ifdef AORP_NEED_COMMON_CHASHAL_PROTOCOL
#  undef AORP_NEED_COMMON_CHASHAL_PROTOCOL

#  include <Bo/services/crypt/CHashP.h>

#  ifdef __need_CHashal_chashal_protocol_impl
#   define __aorpc_impl 1
#  endif
#  define __aorpc_decl __aorpc_decl_protocol
#  define __aorpc_service_name chashal
#  define __aorp_service_opdom CHashal
#  include <Bo/aorp/AorpC.h>

_A_protocol(CHashal,chashal)
_A_end_pod

_A_imop(init_once),
_A_imop(fini_once),
_A_imop_nil(pod_init),
_A_imop_nil(pod_fini),

_A_imop(block_size),
_A_imop(getlen),
_A_imop(init),
_A_imop(update),
_A_imop(final),
_A_imop(perform),

_A_end;

#  include <Bo/aorp/AorpCend.h>

# endif /* AORP_NEED_COMMON_CHASHAL_PROTOCOL */
/* @} */

/*
 * AHA protocol
 * @{
 */
# ifdef AORP_NEED_AHA_CHASHAL_PROTOCOL
#  undef AORP_NEED_AHA_CHASHAL_PROTOCOL

#  include <Bo/services/crypt/CHashP.h>

#  ifdef __need_Aha_chashal_protocol_impl
#   define __aorpc_impl 1
#  endif
#  define __aorpc_decl __aorpc_decl_protocol
#  define __aorpc_service_name chashal
#  define __aorp_service_opdom CHashal
#  include <Bo/aorp/AorpC.h>

_A_protocol(CHashal,chashal)
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop_nil(pod_init),
_A_imop_nil(pod_fini),

_A_imop(block_size),
_A_imop(getlen),
_A_imop(init),
_A_imop(update),
_A_imop(final),
_A_imop(perform),

_A_end;

#  include <Bo/aorp/AorpCend.h>

# endif /* AORP_NEED_AHA_CHASHAL_PROTOCOL */
/* @} */

/*
 * AHAC protocol
 * @{
 */
# ifdef AORP_NEED_AHAC_CHASHAL_PROTOCOL
#  undef AORP_NEED_AHAC_CHASHAL_PROTOCOL

#  include <Bo/services/crypt/CHashP.h>

#  ifdef __need_Ahac_chashal_protocol_impl
#   define __aorpc_impl 1
#  endif
#  define __aorpc_decl __aorpc_decl_protocol
#  define __aorpc_service_name chashal
#  define __aorp_service_opdom CHashal
#  include <Bo/aorp/AorpC.h>

_A_protocol(CHashal,chashal)
_A_end_pod

_A_imop_ff(init_once,CHashal,_BoK_AHA_CHASHAL,chashal),
_A_imop_ff(fini_once,CHashal,_BoK_AHA_CHASHAL,chashal),
_A_imop_ff(pod_init,CHashal,_BoK_AHA_CHASHAL,chashal),
_A_imop_ff(pod_fini,CHashal,_BoK_AHA_CHASHAL,chashal),

_A_imop(block_size),
_A_imop(getlen),
_A_imop(init),
_A_imop_ff(update,CHashal,_BoK_AHA_CHASHAL,chashal),
_A_imop_ff(final,CHashal,_BoK_AHA_CHASHAL,chashal),
_A_imop_ff(perform,CHashal,_BoK_AHA_CHASHAL,chashal),

_A_end;

#  include <Bo/aorp/AorpCend.h>

# endif /* AORP_NEED_AHAC_CHASHAL_PROTOCOL */
/* @} */

#endif /* !__BO_SERVICES_ALG_CRYPT__CHASH_P__ */

/******************************************************************************
 * Revision History:
 *		$Ics$
 *
 *****************************************************************************/
