/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _GuritypesI.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_GuritypesI.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__GURITYPES_I__
# define __BO_SERVICES_ALG_HILIS__GURITYPES_I__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/hilis/_UritypesI.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# define _AorpL_URI_SCANER_TOKFIXTAB_LEN_MIN (258)

# define _AorpK_URI_UNK_TOKEN ((aorp_uri_toktype_t)(-1))

/* State types (for generic parser open) */
# define _AorpK_ABSURI_STATE (0)  /* parse absolute uri */
# define _AorpK_RELURI_STATE (1)  /* parse relative uri */

/* Print modes (for generic printer open) */
# define _AorpK_PRINT_FULL      (0)  /* print full uri representation */
# define _AorpK_PRINT_COMPONENT (1)  /* print uri component (without delimiters) */

/* Token type */
typedef u_int32_t aorp_uri_toktype_t;

/* Parser state type */
typedef u_int32_t aorp_uri_state_t;

struct aorp_uri_token {
	/* MUST be the first field of the structure */
	aorp_uri_toktype_t       urt_type;

	char                    *urt_value;
	size_t                   urt_size;  /* not including the terminating NUL character */
};

/* ============================================================================
 * PARSER TABLE ELEMENT
 * @{
 */
# pragma mark -
# pragma mark +++ parser table element +++
# pragma mark -

# define _AorpK_URI_PARSER_END_STATE ((-1) & 0xFF)

# define _AorpK_URI_PARSER_CMDCODE_MASK    (0x000F)
# define _AorpK_URI_PARSER_CMDCODE_MASK_S  (0)

# define _AorpK_URI_PARSER_CMDARG1_MASK    (0x03F0)
# define _AorpK_URI_PARSER_CMDARG1_MASK_S  (4)

# define _AorpK_URI_PARSER_CMDARG2_MASK    (0xFC00)
# define _AorpK_URI_PARSER_CMDARG2_MASK_S  (8)

/* Parser commands */
# define _AorpK_URI_PARSER_ERRHANDLE  (0)

# define _AorpK_URI_PARSER_SKIP       (1)  /* skip token */
# define _AorpK_URI_PARSER_PUSH       (2)  /* push token */
# define _AorpK_URI_PARSER_ATTACH     (3)  /* attach token */

# define _AorpK_URI_PARSER_SREDUC     (4)  /* skip token and call reduction handler */
# define _AorpK_URI_PARSER_PREDUC     (5)  /* push token and call reduction handler */
# define _AorpK_URI_PARSER_AREDUC     (6)  /* attach token and call reduction handler */
# define _AorpK_URI_PARSER_SREDUC2    (7)  /* skip token and two call reduction handlers */

# define _AorpK_URI_PARSER_REDUCP     (8)  /* call reduction handler and push token */
# define _AorpK_URI_PARSER_REDUC2P    (9)  /* two call reduction handlers and push token */

struct aorp_uri_parser_tabelem {
	int16_t  state_to;
	int16_t  command;
};

typedef struct aorp_uri_parser_tabelem aorp_uri_parser_tabelem_t;

/* ----------------------------------------------------------------------------
 * macro-helper declarations
 * @{
 */

# define _AORP_URI_PARSER_ExtractCmdcode(aCmd) \
	(((aCmd) & _AorpK_URI_PARSER_CMDCODE_MASK) >> _AorpK_URI_PARSER_CMDCODE_MASK_S)

# define _AORP_URI_PARSER_PackCmdcode(aCode) \
	(((int16_t)(aCode) << _AorpK_URI_PARSER_CMDCODE_MASK_S) \
			& _AorpK_URI_PARSER_CMDCODE_MASK)

# define _AORP_URI_PARSER_ExtractCmdarg1(aCmd) \
	(((aCmd) & _AorpK_URI_PARSER_CMDARG1_MASK) >> _AorpK_URI_PARSER_CMDARG1_MASK_S)

# define _AORP_URI_PARSER_PackCmdarg1(anArg) \
	(((int16_t)(anArg) << _AorpK_URI_PARSER_CMDARG1_MASK_S) \
			& _AorpK_URI_PARSER_CMDARG1_MASK)

# define _AORP_URI_PARSER_ExtractCmdarg2(aCmd) \
	(((aCmd) & _AorpK_URI_PARSER_CMDARG2_MASK) >> _AorpK_URI_PARSER_CMDARG2_MASK_S)

# define _AORP_URI_PARSER_PackCmdarg2(anArg) \
	(((int16_t)(anArg) << _AorpK_URI_PARSER_CMDARG2_MASK_S) \
			& _AorpK_URI_PARSER_CMDARG2_MASK)


# define _AORP_MAKE_URI_PARSER_TRANS(aCmdcode, aState) \
	{ aState, _AORP_URI_PARSER_PackCmdcode(aCmdcode) }

# define _AORP_MAKE_URI_PARSER_TRANS_a1(aCmdcode, aCmdarg1, aState) \
	{ aState, _AORP_URI_PARSER_PackCmdcode(aCmdcode) \
			| _AORP_URI_PARSER_PackCmdarg1(aCmdarg1) }

# define _AORP_MAKE_URI_PARSER_TRANS_a2(aCmdcode, aCmdarg1, aCmdarg2, aState) \
	{ aState, _AORP_URI_PARSER_PackCmdcode(aCmdcode) \
			| _AORP_URI_PARSER_PackCmdarg1(aCmdarg1) \
			| _AORP_URI_PARSER_PackCmdarg2(aCmdarg2) }

# define _AORP_URI_PARSER_ERRTRANS \
	_AORP_MAKE_URI_PARSER_TRANS(_AorpK_URI_PARSER_ERRHANDLE,_AorpK_URI_PARSER_END_STATE)

/* @} */

/* ============================================================================
 * PRINTER TABLE ELEMENT
 * @{
 */
# pragma mark -
# pragma mark +++ printer table element +++
# pragma mark -

# define _AorpK_URI_PRINTER_END_STATE (-1)
# define _AorpK_URI_PRINTER_ERR_STATE (-2)

struct aorp_uri_printer_tabelem {
	int          state_to;
	int          pad_;
	char const  *begin_sep;
	char const  *end_sep;
};
typedef struct aorp_uri_printer_tabelem aorp_uri_printer_tabelem_t;

# define _AORP_MAKE_URI_PRINTER_TRANS(aBeginSep,anEndSep,aState) \
	{ aState, 0, aBeginSep, anEndSep }

# define _AORP_URI_PRINTER_ERRTRANS \
	_AORP_MAKE_URI_PRINTER_TRANS(NULL,NULL,_AorpK_URI_PRINTER_ERR_STATE)

/* @} */

/* ============================================================================
 * SPECIFICATION INFO
 * @{
 */
# pragma mark -
# pragma mark +++ specification info +++
# pragma mark -

# define _AorpK_URI_INVAL_FIELD_INDEX (-1)

/* Field types */
# define _AorpK_URI_FIELD_INVAL_TYPE    (-1)
# define _AorpK_URI_FIELD_UNK_TYPE      (0)
# define _AorpK_URI_FIELD_INT_TYPE      (1) /* int */
# define _AorpK_URI_FIELD_CSTRPTR_TYPE  (2) /* char * */
# define _AorpK_URI_FIELD_CSTRBUF_TYPE  (3) /* char [N] */
# define _AorpK_URI_FIELD_CHARSEQ_TYPE  (4) /* struct bo_charseq */

/* Field specoficators */
# define _AorpK_URI_FIELD_INVAL_SPECTOR (-1)
# define _AorpK_URI_FIELD_UNK_SPECTOR   (0)
# define _AorpK_URI_FIELD_PATH_SPECTOR  (4)
# define _AorpK_URI_FIELD_CHAR_SPECTOR  (5)

struct aorp_uri_fieldinfo {
	int          urfi_type;
	int          urfi_spector;
	size_t       urfi_offset; /* offset of specification begin */
	size_t       urfi_size;   /* 0: determine by type */
	char const  *urfi_defval; /* NULL: no default value */
};

struct aorp_uri_specinfo {
	struct aorp_uri_schemeinfo const  *urspi_scheme;
	size_t                             urspi_size;
	size_t                             urspi_fieldc;
	struct aorp_uri_fieldinfo const   *urspi_fieldv;
	size_t                             urspi_ptrfixc;
	struct aorp_uri_fieldinfo        **urspi_ptrfixv;
	bo_index_t                         urspi_stdfieldv[_AorpL_URI_GENERIC_COMPONENTS_MAX];
};

# define aorp_uri_specinfo_p(x) ((struct aorp_uri_specinfo *)(x))

/* ----------------------------------------------------------------------------
 * attributes
 * @{
 */

/*
struct aorp_uri_schemeinfo const *
_AORP_URI_SPECINFO_Scheme(struct aorp_uri_specinfo *aSpecInfo)
 */
# define _AORP_URI_SPECINFO_Scheme(aSpecInfo) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_scheme)

/* size_t _AORP_URI_SPECINFO_Size(struct aorp_uri_specinfo *aSpecInfo) */
# define _AORP_URI_SPECINFO_Size(aSpecInfo) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_size)

/* size_t _AORP_URI_SPECINFO_Fieldc(struct aorp_uri_specinfo *aSpecInfo) */
# define _AORP_URI_SPECINFO_Fieldc(aSpecInfo) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_fieldc)

/*
struct aorp_uri_fieldinfo const *
_AORP_URI_SPECINFO_Fieldv(struct aorp_uri_specinfo *aSpecInfo)
 */
# define _AORP_URI_SPECINFO_Fieldv(aSpecInfo) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_fieldv)

/* size_t _AORP_URI_SPECINFO_Ptrfixc(struct aorp_uri_specinfo *aSpecInfo) */
# define _AORP_URI_SPECINFO_Ptrfixc(aSpecInfo) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_ptrfixc)

/*
struct aorp_uri_fieldinfo **
_AORP_URI_SPECINFO_Ptrfixv(struct aorp_uri_specinfo *aSpecInfo)
 */
# define _AORP_URI_SPECINFO_Ptrfixv(aSpecInfo) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_ptrfixv)

/*
bo_index_t *
_AORP_URI_SPECINFO_Stdfieldv(struct aorp_uri_specinfo *aSpecInfo)
 */
# define _AORP_URI_SPECINFO_Stdfieldv(aSpecInfo) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_stdfieldv)

/*
bo_index_t
_AORP_URI_SPECINFO_Stdfield_idx(struct aorp_uri_specinfo *aSpecInfo, int aComponentID)
 */
# define _AORP_URI_SPECINFO_Stdfield_idx(aSpecInfo, aComponentID) \
	(aorp_uri_specinfo_p(aSpecInfo)->urspi_stdfieldv[(aComponentID)])

/*
 * Specification access
 */

# define __aorp_uri_spec_field_p(aSpec,aFieldInfo) \
	((bo_pointer_t)((char *)(aSpec) + (aFieldInfo)->urfi_offset))

# define __aorp_uri_spec_int_p(aSpec,aFieldInfo) \
	((int *)__aorp_uri_spec_field_p(aSpec,aFieldInfo))

# define __aorp_uri_spec_cstrptr_p(aSpec,aFieldInfo) \
	((char **)__aorp_uri_spec_field_p(aSpec,aFieldInfo))

# define __aorp_uri_spec_cstrbuf_p(aSpec,aFieldInfo) \
	((char *)__aorp_uri_spec_field_p(aSpec,aFieldInfo))

# define __aorp_uri_spec_charseq_p(aSpec,aFieldInfo) \
	((struct bo_charseq *)__aorp_uri_spec_field_p(aSpec,aFieldInfo))


# define __aorp_uri_spec_access_int_cond(aSpecOffset,aFieldInfo,aTotalLen) \
	((aSpecOffset) + (aFieldInfo)->urfi_offset + sizeof(int) <= (aTotalLen))

# define __aorp_uri_spec_access_cstrptr_cond(aSpecOffset,aFieldInfo,aTotalLen) \
	((aSpecOffset) + (aFieldInfo)->urfi_offset + sizeof(char *) <= (aTotalLen))

# define __aorp_uri_spec_access_cstrbuf_cond(aSpecOffset,aFieldInfo,aTotalLen) \
	((aSpecOffset) + (aFieldInfo)->urfi_offset + (aFieldInfo)->urfi_size <= (aTotalLen))

# define __aorp_uri_spec_access_charseq_cond(aSpecOffset,aFieldInfo,aTotalLen) \
	((aSpecOffset) + (aFieldInfo)->urfi_offset + sizeof(struct bo_charseq) <= (aTotalLen))

/* @} */

#endif /* !__BO_SERVICES_ALG_HILIS__GURITYPES_I__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
