/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UribenddefsI.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_UribenddefsI.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URIBENDDEFS_I__
# define __BO_SERVICES_ALG_HILIS__URIBENDDEFS_I__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/hilis/_GuritypesI.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define _GuribendF_TOKFIX_INITED 0x01
# define _GuribendF_PARSER_STATETAB_INITED 0x02
# define _GuribendF_PRINTER_STATETAB_INITED 0x04
# define _GuribendF_SPECINFO_INITED 0x08

/* Forward declarations */
struct aorp_uri_scaner;
struct aorp_uri_parser;
struct aorp_uri_parser_callbacks;
struct aorp_uri_builder;
struct aorp_uri_serializer;
struct aorp_uri_serializer_callbacks;
struct aorp_uri_printer;
struct aorp_uri_info;

/*
 * Operation types
 */

typedef struct aorp_uri_scaner *
(__myc_tcalltype__(mcall) *guribend_scaner_open_fn)(
# if __myc_require(prototypes)
	struct aorp_closure *aClosure,
	bo_pointer_t aMemory /* = NULL */,
	bo_cset_t anEncoding,
	char const *anUriStr,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,4)));

typedef struct aorp_uri_parser *
(__myc_tcalltype__(mcall) *guribend_parser_open_fn)(
# if __myc_require(prototypes)
	struct aorp_closure *aClosure,
	bo_pointer_t aMemory /* = NULL */,
	struct aorp_uri_parser_callbacks const *aCallbacks /* = NULL */,
	struct aorp_uri_scaner *aScaner,
	int aStateBegin /* = _AorpK_ABSURI_STATE */,
	bo_pointer_t anUserCtx /* = NULL */,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,4)));

typedef struct aorp_uri_serializer *
(__myc_tcalltype__(mcall) *guribend_serializer_open_fn)(
# if __myc_require(prototypes)
	struct aorp_closure *aClosure,
	bo_pointer_t aMemory /* = NULL */,
	struct aorp_uri_serializer_callbacks const *aCallbacks /* = NULL */,
	struct aorp_uri const *anUri,
	bo_pointer_t anUserCtx /* = NULL */,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,4)));

typedef struct aorp_uri_builder *
(__myc_tcalltype__(mcall) *guribend_builder_open_fn)(
# if __myc_require(prototypes)
	struct aorp_closure *aClosure,
	bo_pointer_t aMemory /* = NULL */,
	struct aorp_uri *aHeader /* = NULL */,
	bo_pointer_t aSpecBuf /* = NULL */,
	size_t aSpecBufsz /* = 0 */,
	struct aorp_uri_options const *Options,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,6)));

typedef struct aorp_uri_printer *
(__myc_tcalltype__(mcall) *guribend_printer_open_fn)(
# if __myc_require(prototypes)
	struct aorp_closure *aClosure,
	bo_pointer_t aMemory /* = NULL */,
	int aMode /* = _AorpK_PRINT_FULL */,
	size_t aBufferSize /* = 0 */,
	bo_pointer_t aBuffer /* = NULL */,
	struct aorp_uri_options const *Options,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,6)));

/*
 * Guribend guts declaration
 */

# define _GURIBEND_GUTS \
	guribend_scaner_open_fn                  urbe_scaner_open; \
	guribend_parser_open_fn                  urbe_parser_open; \
	guribend_serializer_open_fn              urbe_serializer_open; \
	guribend_builder_open_fn                 urbe_builder_open; \
	guribend_printer_open_fn                 urbe_printer_open; \
	char const                              *urbe_delim; \
	size_t                                   urbe_tokfixelemc; \
	aorp_uri_toktype_t                      *urbe_tokfixtab; \
	int                                      urbe_parser_astate; \
	int                                      urbe_parser_rstate; \
	size_t                                   urbe_parser_tokenc; \
	size_t                                   urbe_parser_statec; \
	aorp_uri_parser_tabelem_t              **urbe_parser_statetab; \
	int                                      urbe_printer_astate; \
	int                                      urbe_printer_rstate; \
	size_t                                   urbe_printer_printc; \
	size_t                                   urbe_printer_statec; \
	aorp_uri_printer_tabelem_t             **urbe_printer_statetab; \
	struct aorp_uri_specinfo                *urbe_specinfo; \
	unsigned                                 urbe_initflags

# define guribend_opdom_p(x) ((_A_opdom(Guribend) *)(x))

/*
 * METHODS
 */

# define __GURIBEND_OPDOM(aClosure) (guribend_opdom_p(AORP_CLOSURE_POD(aClosure)))

# define GURIBEND_SCANER_Open(aClosure,aMemory,anEncoding,anUriStr,anErrPtr) \
	((*__GURIBEND_OPDOM(aClosure)->urbe_scaner_open)( \
		aClosure,aMemory,anEncoding,anUriStr,anErrPtr \
	))

# define GURIBEND_PARSER_Open(aClosure,aMemory,aCallbacks,aScaner,aStateBegin,anUserCtx,anErrPtr) \
	((*__GURIBEND_OPDOM(aClosure)->urbe_parser_open)( \
		aClosure,aMemory,aCallbacks,aScaner,aStateBegin,anUserCtx,anErrPtr \
	))

# define GURIBEND_SERIALIZER_Open(aClosure,aMemory,aCallbacks,anUri,anUserCtx,anErrPtr) \
	((*__GURIBEND_OPDOM(aClosure)->urbe_serializer_open)( \
		aClosure,aMemory,aCallbacks,anUri,anUserCtx,anErrPtr \
	))

# define GURIBEND_BUILDER_Open(aClosure,aMemory,aHeader,aSpecBuf,aSpecBufsz,Options,anErrPtr) \
	((*__GURIBEND_OPDOM(aClosure)->urbe_builder_open)( \
		aClosure,aMemory,aHeader,aSpecBuf,aSpecBufsz,Options,anErrPtr \
	))

# define GURIBEND_PRINTER_Open(aClosure,aMemory,aMode,aBufferSize,aBuffer,Options,anErrPtr) \
	((*__GURIBEND_OPDOM(aClosure)->urbe_printer_open)( \
		aClosure,aMemory,aMode,aBufferSize,aBuffer,Options,anErrPtr \
	))

#endif /* !__BO_SERVICES_ALG_HILIS__URIBENDDEFS_I__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
