/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UribuilderP.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_UribuilderP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URIBUILDER_P__
# define __BO_SERVICES_ALG_HILIS__URIBUILDER_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Forward declarations */
struct aorp_uri;
struct aorp_uri_builder;

/* Builder status */
# define AorpS_URI_BUILDER_INITED   (0)
# define AorpS_URI_BUILDER_BUILDING (1)
# define AorpS_URI_BUILDER_FINISHED (2)

/*
 * Operation types
 */

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urbd_close_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_builder *aBuilder,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urbd_build_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_builder *aBuilder,
	bo_index_t aFieldIdx,
	bo_cset_t anEnc,
	size_t aStrLen,
	char const *aStr,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,5)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urbd_build_v2_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_builder *aBuilder,
	bo_index_t aFieldIdx,
	struct aorp_uri const *anUri,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,3)));

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_urbd_result_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_builder *aBuilder,
	bo_pointer_t *aResultPtr,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1)));

/*
 * Builder operations
 */

struct aorp_uri_builder_opv {
	aorp_urbd_build_fn     urbd_build;
	aorp_urbd_build_v2_fn  urbd_build_v2;
	aorp_urbd_result_fn    urbd_result;
};

/******************************************************************************
 * %% BeginSection: aorp_uri_builder declarations
 */
# pragma mark -
# pragma mark [ aorp_uri_builder declarations ]
# pragma mark -

# ifndef AorpL_URI_BUILDER_SIZE
#  if BO_M_WORD == BO_M_WORD32
#   define AorpL_URI_BUILDER_SIZE (256)
#  else
#   define AorpL_URI_BUILDER_SIZE (512)
#  endif
# endif

struct aorp_uri_builder {
	aorp_urbd_close_fn                   urbd_close;
	struct aorp_uri_builder_opv          urbd_opv;

	bo_status_t                          urbd_status;
};

/* NOT FOR YOUR BUSINESS @{ */
# include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_aorp_uri_builder,
	sizeof(struct aorp_uri_builder) <= AorpL_URI_BUILDER_SIZE);
/* @} */

# define aorp_uri_builder_p(aBuilder) ((struct aorp_uri_builder *)(aBuilder))

# define __AORP_URI_BUILDER_Opv(aBuilder) (&aorp_uri_builder_p(aBuilder)->urbd_opv)

/*
 * ATTRIBUTES
 */

# define AORP_URI_BUILDER_Status(aBuilder) \
	(aorp_uri_builder_p(aBuilder)->urbd_status)

/*
 * METHODS
 */

# define AORP_URI_BUILDER_Close(aBuilder,anErrPtr) \
	((NULL != aBuilder) \
		? (*aorp_uri_builder_p(aBuilder)->urbd_close)( \
				aorp_uri_builder_p(aBuilder),anErrPtr \
		) \
		: BoS_NORMAL \
	)

# define AORP_URI_BUILDER_Build(aBuilder,aFieldIdx,anEnc,aStrLen,aStr,anErrPtr) \
	((*__AORP_URI_BUILDER_Opv(aBuilder)->urbd_build)( \
		aorp_uri_builder_p(aBuilder),aFieldIdx,anEnc,aStrLen,aStr,anErrPtr \
	))

# define AORP_URI_BUILDER_Build_v2(aBuilder,aFieldIdx,anUri,anErrPtr) \
	((*__AORP_URI_BUILDER_Opv(aBuilder)->urbd_build_v2)( \
		aorp_uri_builder_p(aBuilder),aFieldIdx,anUri,anErrPtr \
	))

# define AORP_URI_BUILDER_Result(aBuilder,aResultPtr,anErrPtr) \
	((*__AORP_URI_BUILDER_Opv(aBuilder)->urbd_result)( \
		aorp_uri_builder_p(aBuilder),aResultPtr,anErrPtr \
	))

#endif /* !__BO_SERVICES_ALG_HILIS__URIBUILDER_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
