/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2008 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UricsetP.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_UricsetP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URICSET_P__
# define __BO_SERVICES_ALG_HILIS__URICSET_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/bitis/Bitseq.h>

/******************************************************************************
 * %% BeginSection: definitions
 */

struct aorp_uricset {
	unsigned     ucs_nchars;
	bo_bitseq_t  ucs_buf[FAM_LENGTH];
};

typedef struct aorp_uricset *aorp_uricset_t;
typedef struct aorp_uricset const *aorp_const_uricset_t;

# define AORP_URICSET_STRUCT(NCHARS) \
	struct __myc_xconcat(aorp_uricset_,NCHARS) { \
		unsigned     ucs_nchars; \
		bo_bitseq_t  BO_BITSEQ_DECLARE(ucs_buf,NCHARS); \
	}

/* allocate a uricset on the stack */
# define AORP_URICSET_DECLARE(aName, NCHARS) AORP_URICSET_STRUCT(NCHARS)(aName)

/* aorp_uricset_t AORP_URICSET_Alloc(unsigned aNchars) */
# define AORP_URICSET_Alloc(aNchars) \
	((aorp_uricset_t)_BoMalloc(sizeof(unsigned) + (size_t)BO_NBBITSEQ(aNchars), NULL))

/* void AORP_URICSET_Free(aorp_uricset_t p) */
# define AORP_URICSET_Free(p) _BoMfree(p)

/*
 * INITIALIZE
 */

/* void AORP_URICSET_Init_b0(aorp_uricset_t anUriCset, unsigned aNchars) */
# define AORP_URICSET_Init_b0(anUriCset, aNchars) \
	do { \
		(anUriCset)->ucs_nchars = (aNchars); \
		bzero((char *)(anUriCset) + sizeof(unsigned), BO_NBBITSEQ(aNchars)); \
	} while (0)

/* void AORP_URICSET_Init_b1(aorp_uricset_t anUriCset, unsigned aNchars) */
# define AORP_URICSET_Init_b1(anUriCset, aNchars) \
	do { \
		(anUriCset)->ucs_nchars = (aNchars); \
		memset((char *)(anUriCset) + sizeof(unsigned), 0xff, BO_NBBITSEQ(aNchars)); \
	} while (0)

/* void
   AORP_URICSET_Init_u(
		aorp_uricset_t anUriCset, unsigned aNchars,
		aorp_uricset_t aLhs, aorp_uricset_t aRhs
	) */
# define AORP_URICSET_Init_u(anUriCset, aNchars, aLhs, aRhs) \
	do { \
		register unsigned count; \
		register unsigned char const *src; \
		register unsigned char *dst; \
		aorp_const_uricset_t lcset, gcset; \
		lcset = (aLhs); \
		gcset = (aRhs); \
		if (gcset->ucs_nchars < lcset->ucs_nchars) { \
			aorp_const_uricset_t tcset = lcset; \
			lcset = gcset; \
			gcset = tcset; \
		} \
		assert(gcset->ucs_nchars < = (aNchars)); \
		AORP_URICSET_Init_b0(anUriCset, aNchars); \
		memcpy((anUriCset)->ucs_buf, (gcset)->ucs_buf,	BO_NBBITSEQ(gcset->ucs_nchars)); \
		count = BO_NBBITSEQ(lcset->ucs_nchars); \
		src = (lcset)->ucs_buf; \
		dst = (anUriCset)->ucs_buf; \
		for ( ; count--; ++dst, ++src) \
			*dst |= *src; \
	} while (0)

/* void
   AORP_URICSET_Init_str(
		aorp_uricset_t anUriCset, unsigned aNchars, char const *aStr
	) */
# define AORP_URICSET_Init_str(anUriCset, aNchars, aStr) \
	do { \
		register unsigned ch; \
		register unsigned char const *s; \
		AORP_URICSET_Init_b0(anUriCset, aNchars); \
		if ((s = (unsigned char const *)(aStr))) { \
			while ('\0' != (ch = *s++)) \
				AORP_URICSET_Set(anUriCset, ch); \
		} \
	} while (0)

/* void
   AORP_URICSET_Init_bseq(
		aorp_uricset_t anUriCset, unsigned aNchars,
		unsigned aLen, unsigned char const *aBseq
	) */
# define AORP_URICSET_Init_bseq(anUriCset, aNchars, aLen, aBseq) \
	do { \
		register unsigned count = (unsigned)(aLen); \
		register unsigned char const *b = (unsigned char const *)(aBseq); \
		AORP_URICSET_Init_b0(anUriCset, aNchars); \
		for ( ; count--; ++b) \
			AORP_URICSET_Set(anUriCset, *b); \
	} while (0)

/*
 * OPERATIONS
 */

/* _Bool AORP_URICSET_Test(aorp_uricset_t anUriCset, unsigned int c) */
# define AORP_URICSET_Test(anUriCset, c) \
	( ((unsigned)(c) < (anUriCset)->ucs_nchars) \
		&& BO_BITSEQ_Test((anUriCset)->ucs_buf, (unsigned)(c)) )

/* void AORP_URICSET_Set(aorp_uricset_t anUriCset, unsigned int c) */
# define AORP_URICSET_Set(anUriCset, c) \
	( assert((unsigned)(c) < (anUriCset)->ucs_nchars), \
		BO_BITSEQ_Set((anUriCset)->ucs_buf, (unsigned)(c)) )

/* void AORP_URICSET_Clear(aorp_uricset_t anUriCset, unsigned int c) */
# define AORP_URICSET_Clear(anUriCset, c) \
	( assert((unsigned)(c) < (anUriCset)->ucs_nchars), \
		BO_BITSEQ_Clear((anUriCset)->ucs_buf, (unsigned)(c)) )

#endif /* !__BO_SERVICES_ALG_HILIS__URICSET_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
