/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UriparserP.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_UriparserP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URIPARSER_P__
# define __BO_SERVICES_ALG_HILIS__URIPARSER_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Forward declarations */
struct aorp_urirulestack;
struct aorp_uri_parser;

/* Parser status */
# define AorpS_URI_PARSER_WRONG    (-2)
# define AorpS_URI_PARSER_INITED   (0)
# define AorpS_URI_PARSER_RUN      (1)
# define AorpS_URI_PARSER_STOPPED  (2)
# define AorpS_URI_PARSER_FINISHED (3)

/*
 * Operation types
 */

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urpr_close_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_parser *aParser,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urpr_parse_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_parser *aParser,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urpr_error_handler)(
# if __myc_require(prototypes)
	struct aorp_uri_parser *aParser,
	struct aorp_urirulestack *aRulestk,
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urpr_reduction_handler)(
# if __myc_require(prototypes)
	struct aorp_uri_parser *aParser,
	struct aorp_urirulestack *aRulestk,
	int aParam,
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

/*
 * Parser operations
 */

struct aorp_uri_parser_opv {
	aorp_urpr_parse_fn   urpr_parse;
};

/*
 * Parser callbacks
 */

struct aorp_uri_parser_callbacks {
	aorp_urpr_error_handler       urpr_error_handler;
	aorp_urpr_reduction_handler   urpr_reduction_handler;
};

/******************************************************************************
 * %% BeginSection: aorp_uri_parser declarations
 */
# pragma mark -
# pragma mark [ aorp_uri_parser declarations ]
# pragma mark -

# ifndef AorpL_URI_PARSER_SIZE
#  define AorpL_URI_PARSER_SIZE (128)
# endif

struct aorp_uri_parser {
	aorp_urpr_close_fn                  urpr_close;
	struct aorp_uri_parser_opv          urpr_opv;
	struct aorp_uri_parser_callbacks    urpr_cbs;

	bo_status_t                         urpr_status;
};

/* NOT FOR YOUR BUSINESS @{ */
# include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_aorp_uri_parser,
	(sizeof(struct aorp_uri_parser) <= AorpL_URI_PARSER_SIZE));
/* @} */

# define aorp_uri_parser_p(aParser) ((struct aorp_uri_parser *)(aParser))

# define __AORP_URI_PARSER_Opv(aParser) (&aorp_uri_parser_p(aParser)->urpr_opv)
# define __AORP_URI_PARSER_Cbs(aParser) (&aorp_uri_parser_p(aParser)->urpr_cbs)

/*
 * ATTRIBUTES
 */

# define AORP_URI_PARSER_Status(aParser) (aorp_uri_parser_p(aParser)->urpr_status)

/*
 * METHODS
 */

# define AORP_URI_PARSER_Close(aParser,anErrPtr) \
	((NULL != aParser) \
		? (*aorp_uri_parser_p(aParser)->urpr_close)( \
				aorp_uri_parser_p(aParser),anErrPtr \
			) \
		: BoS_NORMAL \
	)

# define AORP_URI_PARSER_Parse(aParser,anErrPtr) \
	((*__AORP_URI_PARSER_Opv(aParser)->urpr_parse)( \
		aorp_uri_parser_p(aParser),anErrPtr \
	))

/*
 * HANDLERS
 */

# define AORP_URI_PARSER_ErrorHandler(aParser,aRulestk,anUserCtx,anErrPtr) \
	((*__AORP_URI_PARSER_Cbs(aParser)->urpr_error_handler)( \
		aorp_uri_parser_p(aParser),aRulestk,anUserCtx,anErrPtr \
	))

# define AORP_URI_PARSER_ReductionHandler(aParser,aRulestk,aParam,anUserCtx,anErrPtr) \
	((*__AORP_URI_PARSER_Cbs(aParser)->urpr_reduction_handler)( \
		aorp_uri_parser_p(aParser),aRulestk,aParam,anUserCtx,anErrPtr \
	))

#endif /* !__BO_SERVICES_ALG_HILIS__URIPARSER_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
