/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UriprinterP.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_UriprinterP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URIPRINTER_P__
# define __BO_SERVICES_ALG_HILIS__URIPRINTER_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Forward declarations */
struct aorp_uri;
struct aorp_uri_printer;

/* Printer status */
# define AorpS_URI_PRINTER_INITED   (0)
# define AorpS_URI_PRINTER_PRINT    (1)
# define AorpS_URI_PRINTER_FINISHED (2)

/*
 * Operation types
 */

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urpt_close_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_printer *aPrinter,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urpt_print_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_printer *aPrinter,
	bo_index_t aFieldIdx,
	bo_cset_t anEnc,
	size_t aStrLen,
	char const *aStr,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,5)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_urpt_print_v2_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_printer *aPrinter,
	bo_index_t aFieldIdx,
	struct aorp_uri const *anUri,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,3)));

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_urpt_result_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_printer *aPrinter,
	bo_pointer_t *aResultPtr,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1)));

/*
 * Printer operations
 */

struct aorp_uri_printer_opv {
	aorp_urpt_print_fn     urpt_print;
	aorp_urpt_print_v2_fn  urpt_print_v2;
	aorp_urpt_result_fn    urpt_result;
};

/******************************************************************************
 * %% BeginSection: aorp_uri_printer declarations
 */
# pragma mark -
# pragma mark [ aorp_uri_printer declarations ]
# pragma mark -

# ifndef AorpL_URI_PRINTER_SIZE
#  if BO_M_WORD == BO_M_WORD32
#   define AorpL_URI_PRINTER_SIZE (256)
#  else
#   define AorpL_URI_PRINTER_SIZE (512)
#  endif
# endif

struct aorp_uri_printer {
	aorp_urpt_close_fn           urpt_close;
	struct aorp_uri_printer_opv  urpt_opv;

	bo_status_t                  urpt_status;
};

/* NOT FOR YOUR BUSINESS @{ */
# include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_aorp_uri_printer,
	(sizeof(struct aorp_uri_printer) <= AorpL_URI_PRINTER_SIZE));
/* @} */

# define aorp_uri_printer_p(aPrinter) ((struct aorp_uri_printer *)(aPrinter))

# define __AORP_URI_PRINTER_Opv(aPrinter) (&aorp_uri_printer_p(aPrinter)->urpt_opv)

/*
 * ATTRIBUTES
 */

# define AORP_URI_PRINTER_Status(aPrinter) \
	(aorp_uri_printer_p(aPrinter)->urpt_status)

/*
 * METHODS
 */

# define AORP_URI_PRINTER_Close(aPrinter,anErrPtr) \
	((NULL != aPrinter) \
		? (*aorp_uri_printer_p(aPrinter)->urpt_close)( \
			aorp_uri_printer_p(aPrinter),anErrPtr \
			) \
		: BoS_NORMAL \
	)

# define AORP_URI_PRINTER_Print(aPrinter,aFieldIdx,anEnc,aStrLen,aStr,anErrPtr) \
	((*__AORP_URI_PRINTER_Opv(aPrinter)->urpt_print)( \
		aorp_uri_printer_p(aPrinter),aFieldIdx,anEnc,aStrLen,aStr,anErrPtr \
	))

# define AORP_URI_PRINTER_Print_v2(aPrinter,aFieldIdx,anUri,anErrPtr) \
	((*__AORP_URI_PRINTER_Opv(aPrinter)->urpt_print_v2)( \
		aorp_uri_printer_p(aPrinter),aFieldIdx,anUri,anErrPtr \
	))

# define AORP_URI_PRINTER_Result(aPrinter,aResultPtr,anErrPtr) \
	((*__AORP_URI_PRINTER_Opv(aPrinter)->urpt_result)( \
		aorp_uri_printer_p(aPrinter),aResultPtr,anErrPtr \
	))

#endif /* !__BO_SERVICES_ALG_HILIS__URIPRINTER_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
