/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UrirulestkP.h	(A.Kuzina)
 * include/Bo/services/alg/hilis/_UrirulestkP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URIRULESTK_P__
# define __BO_SERVICES_ALG_HILIS__URIRULESTK_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/hilis/_GuritypesI.h>

# include <Bo/fundis/Seqlist.h> 

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# ifndef AORP_URIRULESTACK_INIT_SIZE
#  define AORP_URIRULESTACK_INIT_SIZE 20
# endif

# ifndef AORP_URIRULESTACK_STATE_MASK
#  define AORP_URIRULESTACK_STATE_MASK 0x00FF0000
# endif

# ifndef AORP_URIRULESTACK_STATE_MASK_S
#  define AORP_URIRULESTACK_STATE_MASK_S 16
# endif

# ifndef AORP_URIRULESTACK_TOKEN_MASK
#  define AORP_URIRULESTACK_TOKEN_MASK 0x0FFFF
# endif

# ifndef AORP_URIRULESTACK_TOKEN_MASK_S
#  define AORP_URIRULESTACK_TOKEN_MASK_S 0
# endif

struct aorp_urirulestack {
	size_t       urstk_tslmemb1, urstk_tslmemb2;
	bo_pointer_t urstk_tslmemb3;
};

union aorp_urirulestack_value {
	u_int32_t              isa;
	aorp_uri_state_t       state;
	struct aorp_uri_token  token;
};
typedef union aorp_urirulestack_value aorp_urirulestack_value_t;

# define AORP_URIRULESTACK_VALUE_IsState(aValueRef) \
	((aValueRef).isa & AORP_URIRULESTACK_STATE_MASK)

# define AORP_URIRULESTACK_VALUE_IsToken(aValueRef) \
	((aValueRef).isa & AORP_URIRULESTACK_TOKEN_MASK)

# define AORP_URIRULESTACK_VALUE_GetState(aValueRef) \
	(((aValueRef).isa & AORP_URIRULESTACK_STATE_MASK) >> AORP_URIRULESTACK_STATE_MASK_S)

# define AORP_URIRULESTACK_VALUE_SetState(aValueRef, aState) \
	do { \
		(aValueRef).isa &= ~AORP_URIRULESTACK_STATE_MASK; \
		(aValueRef).isa |= ( \
			((u_int32_t)(aState) << AORP_URIRULESTACK_STATE_MASK_S) \
			& AORP_URIRULESTACK_STATE_MASK \
		); \
	} while (0)

# define AORP_URIRULESTACK_VALUE_GetToken(aValueRef) \
	(((aValueRef).isa & AORP_URIRULESTACK_TOKEN_MASK) >> AORP_URIRULESTACK_TOKEN_MASK_S)

# define AORP_URIRULESTACK_VALUE_SetToken(aValueRef, aToktype) \
	do { \
		(aValueRef).isa &= ~AORP_URIRULESTACK_TOKEN_MASK; \
		(aValueRef).isa |= ( \
			((u_int32_t)(aToktype) << AORP_URIRULESTACK_TOKEN_MASK_S) \
			& AORP_URIRULESTACK_TOKEN_MASK \
		); \
	} while (0)

/******************************************************************************
 * %% BeginSection: macro declarations
 */
# pragma mark -
# pragma mark [ macro declarations ]
# pragma mark -

# define AORP_URIRULESTACK_Init(aStack,anElemc,NOMEMHANDLER) \
	BO_SEQLIST_Init_n(aorp_urirulestack_value_t,aStack,anElemc,NOMEMHANDLER)

# define AORP_URIRULESTACK_Clear(aStack) \
	_BO_SEQLIST_Clear_s(aStack)

# define AORP_URIRULESTACK_Destroy(aStack) \
	_BO_SEQLIST_Destroy_s(aStack)

# define AORP_URIRULESTACK_Empty(aStack) \
	_BO_SEQLIST_Empty(aStack)

# define AORP_URIRULESTACK_Size(aStack) \
	_BO_SEQLIST_Size(aStack)

# define AORP_URIRULESTACK_Begin(aStack) \
	BO_SEQLIST_Begin(aorp_urirulestack_value_t,aStack)

# define AORP_URIRULESTACK_End(aStack) \
	BO_SEQLIST_End(aorp_urirulestack_value_t,aStack)

# define AORP_URIRULESTACK_Front(aStack) \
	BO_SEQLIST_Front(aorp_urirulestack_value_t,aStack)

# define AORP_URIRULESTACK_Back(aStack) \
	BO_SEQLIST_Back(aorp_urirulestack_value_t,aStack)

# define AORP_URIRULESTACK_Ate(aStack,anIndex) \
	BO_SEQLIST_Ate(aorp_urirulestack_value_t,aStack,anIndex)

# define AORP_URIRULESTACK_Atb(aStack,anIndex) \
	BO_SEQLIST_Atb(aorp_urirulestack_value_t,aStack,anIndex)

# define AORP_URIRULESTACK_Push(aStack,aValueRef,NOMEMHANDLER) \
	BO_SEQLIST_Push(aorp_urirulestack_value_t,aStack,aValueRef,NOMEMHANDLER)

# define AORP_URIRULESTACK_PushState(aStack,aState,NOMEMHANDLER) \
	do { \
		aorp_urirulestack_value_t rulestkPushStateValue = {0}; \
		AORP_URIRULESTACK_VALUE_SetState(rulestkPushStateValue, aState); \
		AORP_URIRULESTACK_Push(aStack,rulestkPushStateValue,NOMEMHANDLER); \
	} while (0)

# define AORP_URIRULESTACK_Pop(aStack,aValueRef,anUnderflowHandler) \
	BO_SEQLIST_Pop(aorp_urirulestack_value_t,aStack,aValueRef,anUnderflowHandler)

# define AORP_URIRULESTACK_Pop_s(aStack) \
	_BO_SEQLIST_Pop_s(aStack)

# define AORP_URIRULESTACK_Pop_n(aStack,N) \
	_BO_SEQLIST_Pop_n(aStack,N)

# define AORP_URIRULESTACK_Xchg(aStack) \
	BO_SEQLIST_XchgBack(aorp_urirulestack_value_t,aStack)

#endif /* !__BO_SERVICES_ALG_HILIS__URIRULESTK_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
