/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UriscanerP.h	(A.Kuzina)
 * include/Bo/services/alg/hilis/_UriscanerP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URISCANER_P__
# define __BO_SERVICES_ALG_HILIS__URISCANER_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Forward declarations */
struct aorp_uri_token;
struct aorp_uri_scaner;

/*
 * Operation types
 */

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ursc_close_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_scaner *aScaner,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ursc_token_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_scaner *aScaner,
	struct aorp_uri_token *aToken,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

typedef bo_cset_t
(__myc_tcalltype__(mcall) *aorp_ursc_encoding_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_scaner *aScaner,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype));

/*
 * Scaner operations
 */

struct aorp_uri_scaner_opv {
	aorp_ursc_token_fn      ursc_token;
	aorp_ursc_encoding_fn   ursc_encoding;
};

/******************************************************************************
 * %% BeginSection: aorp_uri_scaner declarations
 */
# pragma mark -
# pragma mark [ aorp_uri_scaner declarations ]
# pragma mark -

# ifndef AorpL_URI_SCANER_SIZE
#  if BO_M_WORD == BO_M_WORD32
#   define AorpL_URI_SCANER_SIZE (128)
#  else
#   define AorpL_URI_SCANER_SIZE (256)
#  endif
# endif

struct aorp_uri_scaner {
	aorp_ursc_close_fn          ursc_close;
	struct aorp_uri_scaner_opv  ursc_opv;
};

/* NOT FOR YOUR BUSINESS @{ */
# include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_aorp_uri_scaner,
	(sizeof(struct aorp_uri_scaner) <= AorpL_URI_SCANER_SIZE));
/* @} */

# define aorp_uri_scaner_p(aScaner) ((struct aorp_uri_scaner *)(aScaner))

# define __AORP_URI_SCANER_Opv(aScaner) (&aorp_uri_scaner_p(aScaner)->ursc_opv)

/*
 * METHODS
 */

# define AORP_URI_SCANER_Close(aScaner,anErrPtr) \
	((NULL != aScaner) \
		? (*aorp_uri_scaner_p(aScaner)->ursc_close)( \
				aorp_uri_scaner_p(aScaner),anErrPtr \
			) \
		: BoS_NORMAL \
	)

# define AORP_URI_SCANER_Token(aScaner,aToken,anErrPtr) \
	((*__AORP_URI_SCANER_Opv(aScaner)->ursc_token)( \
		aorp_uri_scaner_p(aScaner),aToken,anErrPtr \
	))

# define AORP_URI_SCANER_Encoding(aScaner,anErrPtr) \
	((*__AORP_URI_SCANER_Opv(aScaner)->ursc_encoding)( \
		aorp_uri_scaner_p(aScaner),anErrPtr \
	))

#endif /* !__BO_SERVICES_ALG_HILIS__URISCANER_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
