/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UriserializerP.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_UriserializerP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URISERIALIZER_P__
# define __BO_SERVICES_ALG_HILIS__URISERIALIZER_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Forward declarations */
struct aorp_uri;
struct aorp_uri_serializer;

/* Serializer status */
# define AorpS_URI_SERIALIZER_INITED   (0)
# define AorpS_URI_SERIALIZER_RUN      (1)
# define AorpS_URI_SERIALIZER_STOPPED  (2)
# define AorpS_URI_SERIALIZER_FINISHED (3)

/*
 * Operation types
 */

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ursl_close_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_serializer *aSerializer,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ursl_serilize_fn)(
# if __myc_require(prototypes)
	struct aorp_uri_serializer *aSerializer,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ursl_field_handler)(
# if __myc_require(prototypes)
	struct aorp_uri_serializer *aSerializer,
	bo_index_t aFieldIdx,
	struct aorp_uri const *anUri,
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,3)));

/*
 * Serializer operations
 */

struct aorp_uri_serializer_opv {
	aorp_ursl_serilize_fn  ursl_serilize;
};

/*
 * Serializer callbacks
 */

struct aorp_uri_serializer_callbacks {
	aorp_ursl_field_handler  ursl_field_handler;
};

/******************************************************************************
 * %% BeginSection: aorp_uri_serializer declarations
 */
# pragma mark -
# pragma mark [ aorp_uri_serializer declarations ]
# pragma mark -

# ifndef AorpL_URI_SERIALIZER_SIZE
#  define AorpL_URI_SERIALIZER_SIZE (128)
# endif

struct aorp_uri_serializer {
	aorp_ursl_close_fn                    ursl_close;
	struct aorp_uri_serializer_opv        ursl_opv;
	struct aorp_uri_serializer_callbacks  ursl_cbs;

	bo_status_t                           ursl_status;
};

/* NOT FOR YOUR BUSINESS @{ */
# include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_aorp_uri_serializer,
	(sizeof(struct aorp_uri_serializer) <= AorpL_URI_SERIALIZER_SIZE));
/* @} */

# define aorp_uri_serializer_p(aSerializer) ((struct aorp_uri_serializer *)(aSerializer))

# define __AORP_URI_SERIALIZER_Opv(aSerializer) (&aorp_uri_serializer_p(aSerializer)->ursl_opv)
# define __AORP_URI_SERIALIZER_Cbs(aSerializer) (&aorp_uri_serializer_p(aSerializer)->ursl_cbs)

/*
 * ATTRIBUTES
 */

# define AORP_URI_SERIALIZER_Status(aSerializer) \
	(aorp_uri_serializer_p(aSerializer)->ursl_status)

/*
 * METHODS
 */

# define AORP_URI_SERIALIZER_Close(aSerializer,anErrPtr) \
	((NULL != aSerializer) \
		? (*aorp_uri_serializer_p(aSerializer)->ursl_close)( \
				aorp_uri_serializer_p(aSerializer),anErrPtr \
			) \
		: BoS_NORMAL \
	)

# define AORP_URI_SERIALIZER_Serialize(aSerializer,anErrPtr) \
	((*__AORP_URI_SERIALIZER_Opv(aSerializer)->ursl_serilize)( \
		aorp_uri_serializer_p(aSerializer),anErrPtr \
	))

/*
 * HANDLERS
 */

# define AORP_URI_SERIALIZER_FieldHandler(aSerializer,aFieldIdx,anUri,anUserCtx,anErrPtr) \
	((*__AORP_URI_SERIALIZER_Cbs(aSerializer)->ursl_field_handler)( \
		aorp_uri_serializer_p(aSerializer),aFieldIdx,anUri,anUserCtx,anErrPtr \
	))

#endif /* !__BO_SERVICES_ALG_HILIS__URISERIALIZER_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
