/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _UritypesI.h	(N.Panasiuk)
 * include/Bo/services/alg/hilis/_UritypesI.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_HILIS__URITYPES_I__
# define __BO_SERVICES_ALG_HILIS__URITYPES_I__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/hilis/Uritypes.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define _AorpK_URI_INTERNAL_FLAGS_MASK  (0xffff00)
# define _AorpK_URI_PSTYLE_MASK          (0x000f00)
# define _AorpK_URI_PSTYLE_MASK_S        (8)

/* Generic uri components */
# define _AorpK_URI_HOST_COMPONENT          (0)
# define _AorpK_URI_PORT_COMPONENT          (1)
# define _AorpK_URI_USER_COMPONENT          (2)
# define _AorpK_URI_PASSWD_COMPONENT        (3)
# define _AorpK_URI_PATH_COMPONENT          (4)
# define _AorpK_URI_PARM_COMPONENT          (5)
# define _AorpK_URI_QUERY_COMPONENT         (6)
# define _AorpK_URI_FRAGMENT_COMPONENT      (7)
# define _AorpL_URI_GENERIC_COMPONENTS_MAX  (8)

/* Operation flags */
# define _AorpF_URI_GET_FSPATHPTR   (0x10000)  /* if possible returns pointer to internal buffer */
# define _AorpF_URI_CREATE_FSPATH   (0x20000)

/* URI flags */
# define _AorpF_URI_MALLOCED        (0x01000)
# define _AorpF_URI_IS_DIRECTORY    (0x04000)

/* ============================================================================
 * OPAQUE-PART declarations
 * @{
 */

struct aorp_uri_opaque_part {
	aorp_object_t  uri_bend;
};

/* NOT FOR YOUR BUSINESS @{ */
# include <Bo/Debug.h>
_BoDbg_StaticAssert(
	sizeof_aorp_uri_opaque_part,
	sizeof(struct aorp_uri_opaque_part) <= AorpK_URI_OPAQUE_SIZE
);
/* @} */

# define aorp_uri_opaque_p(x) ((struct aorp_uri_opaque_part *)aorp_uri_p(x)->_h.uri_opaque)

/* ----------------------------------------------------------------------------
 * attributes
 * @{
 */

/* _Bool _AORP_URI_Directory(struct aorp_uri *anUri) */
# define _AORP_URI_Directory(anUri) \
	(_AorpF_URI_IS_DIRECTORY & _AORP_URI_Flags(anUri))

/* aorp_object_t _AORP_URI_Bend(struct aorp_uri *anUri) */
# define _AORP_URI_Bend(anUri) (aorp_uri_opaque_p(anUri)->uri_bend)

/* int _AORP_URI_Getpstyle(struct aorp_uri *anUri) */
# define _AORP_URI_Getpstyle(anUri) \
	(_AORP_URI_PSTYLE_Extract(_AORP_URI_Flags(anUri)))

/* void _AORP_URI_Setpstyle(struct aorp_uri *anUri, int aPStyle) */
# define _AORP_URI_Setpstyle(anUri,aPStyle) \
	do { \
		_AORP_URI_Flags(anUri) &= ~_AorpK_URI_PSTYLE_MASK; \
		_AORP_URI_Flags(anUri) |= _AORP_URI_PSTYLE_Make(aPStyle); \
	} while (0)

/* @} */

/* ----------------------------------------------------------------------------
 * path-style operations
 * @{
 */

/* u_int32_t _AORP_URI_PSTYLE_Make(int aPStyle) */
# define _AORP_URI_PSTYLE_Make(aPStyle) \
	(((u_int32_t)(aPStyle) << _AorpK_URI_PSTYLE_MASK_S) & _AorpK_URI_PSTYLE_MASK)

/* int _AORP_URI_PSTYLE_Extract(u_int32_t aPStyle) */
# define _AORP_URI_PSTYLE_Extract(aPStyle) \
	(((u_int32_t)(aPStyle) & _AorpK_URI_PSTYLE_MASK) >> _AorpK_URI_PSTYLE_MASK_S)

/* @} */

#endif /* !__BO_SERVICES_ALG_HILIS__URITYPES_I__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
