/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Slogcli2P.h	(S.Makhmali)
 * include/Bo/services/alg/infos/_Slogcli2P.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_INFOS__SLOGCLI2_P__
# define __BO_SERVICES_ALG_INFOS__SLOGCLI2_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/infos/_Slogcli2.h>

# include <Bo/Thrtypes.h>
	/* S-mutex */

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

struct bo_zmalloc;


# ifndef _Slogcli2_K_MSGRACKALLOC_GRANULARITY
#  define _Slogcli2_K_MSGRACKALLOC_GRANULARITY  32
# endif


# define _Slogcli2_F_ASYNC_MODE 0x01
# define _Slogcli2_F_LATENCY    0x02
# define _Slogcli2_F_MQFROZEN   0x04


struct Slogcli2_msgrack {
	bo_csllelem_t  slmr_lelm_;
	aorp_logmsg_t  slmr_msg;
};


_A_opdom(Slogcli2) {
	bo_smutex_t                    slog2_state_guard;
	short                          slog2_state;
	short                          slog2_iflags;

	aorp_logattr_t                 slog2_attr;

	union {
		struct bo_uconn_sock_ipv4  slog2_addr4;
		struct bo_uconn_sock_ipv6  slog2_addr6;
		struct bo_uconn_uri        slog2_uri;
		struct bo_uconn_sluzba     slog2_sluzba;
	} usrv;
	struct timespec                slog2_handshake_timeout;
	u_int32_t                      slog2_connid;

#ifdef BO_SYSTYPE_WIN32
	HANDLE                         slog2_trim_event;
#endif

	aorp_object_t                  slog2_main_strm, /* or console */
	                               slog2_pout_strm,
	                               slog2_perr_strm;

	bo_msec_t                      slog2_latency;

	struct bo_zmalloc             *slog2_msgrack_alloc;
	bo_cpulock_t                   slog2_mqguard;
	bo_csllist_t                   slog2_msgque;
	size_t                         slog2_mqsup,     /* upper waterline */
	                               slog2_mqslo;     /* lower waterline */

	aorp_object_t                  slog2_runloop;
	unsigned                       slog2_jobid;
}
_A_end;

/* ----------------------------------------------------------------------------
 * {log,Slogcli2} protocol
 * @{
 */

# include <Bo/services/infos/LogP.h>

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (log,Slogcli2)
# define __aorpc_realize_service_file <Bo/services/infos/LogP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol(log,Slogcli2)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop(init_once),
_A_imop(fini_once),
_A_imop(pod_init),
_A_imop(pod_fini),

_A_imop(openconn),
_A_imop(reopenconn),
_A_imop(closeconn),
_A_imop(setprifilter),
_A_imop(getprifilter),
_A_imop(setattrfilter),
_A_imop(getattrfilter),
_A_imop(printf),
_A_imop(vprintf),
#if 1
_A_imop_nil(wprintf),
_A_imop_nil(vwprintf),
#else
_A_imop(wprintf),
_A_imop(vwprintf),
#endif
_A_imop(write),
_A_imop(_send),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

/* ----------------------------------------------------------------------------
 * {job,Slogcli2} protocol
 * @{
 */

# include <Bo/services/jobcntl/JobP.h>

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (job,Slogcli2)
# define __aorpc_realize_service_file <Bo/services/jobcntl/JobP.h>
# include <Bo/aorp/AorpCpro.h>

/*
 * The impl of the 'job' service acts on Slogcli2 domain.
 */
_A_protocol(job,Slogcli2)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop_nil(pod_init),
_A_imop_nil(pod_fini),

_A_imop(schedule),
_A_imop(unschedule),
_A_imop(exec),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

#endif /* !__BO_SERVICES_ALG_INFOS__SLOGCLI2_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
