/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Slogsrv2P.h	(S.Makhmali)
 * include/Bo/services/alg/infos/_Slogsrv2P.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_INFO__SLOGSRV2_P__
# define __BO_SERVICES_ALG_INFO__SLOGSRV2_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/infos/_Slogsrv2.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

struct Slogsrv2_cliinfo {
	u_int32_t             cli_connid_;
	bo_pointer_t          cli_logfile_;
	bo_utime_t            cli_last_activetm_;
};

struct Slogsrv2_lelem {
	bo_cdllelem_t            cle_listel_;
	struct Slogsrv2_cliinfo  cle_value_;
};

_A_opdom(Slogsrv2) {
	int                            sls2_state;
	union {
		struct bo_uconn_sock_ipv4  sls2_addr4;
		struct bo_uconn_sock_ipv6  sls2_addr6;
		struct bo_uconn_uri        sls2_uri;
		struct bo_uconn_sluzba     sls2_sluzba;
	} usrv;
	bo_cdllist_t                   sls2_clilist;
	aorp_object_t                  sls2_sock;
	bo_usec_t                      sls2_timeout;
	struct slsrv2_fdriver         *sls2_driver;
}
_A_end;

/*
 * {log,Slogsrv2} protocol
 * @{
 */

# include <Bo/services/infos/LogP.h>

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (log,Slogsrv2)
# define __aorpc_realize_service_file <Bo/services/infos/LogP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol(log,Slogsrv2)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop(pod_init),
_A_imop(pod_fini),

_A_imop(openconn),
_A_imop(reopenconn),
_A_imop(closeconn),
_A_imop_nil(setprifilter),
_A_imop_nil(getprifilter),
_A_imop_nil(setattrfilter),
_A_imop_nil(getattrfilter),
_A_imop_nil(printf),
_A_imop_nil(vprintf),
#if 1
_A_imop_nil(wprintf),
_A_imop_nil(vwprintf),
#else
_A_imop(wprintf),
_A_imop(vwprintf),
#endif
_A_imop_nil(write),
_A_imop_nil(_send),

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

#endif /* !__BO_SERVICES_ALG_INFO__SLOGSRV2_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
