/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _SlogsrvP.h	(S.Makhmali)
 * include/Bo/services/alg/infos/_SlogsrvP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_INFOS__SLOGSRV_P__
# define __BO_SERVICES_ALG_INFOS__SLOGSRV_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/infos/_Slogsrv.h>

# if __myc_exists(netinet7in,h)
#  include __MYC_LIBC_HEADER(netinet/in.h)
	/* address structures */
# endif
# if __myc_exists(winsock2,h)
#  include __MYC_LIBC_HEADER(winsock2.h)
#  ifndef HAS_TIMEVAL_STRUC
#   define HAS_TIMEVAL_STRUC
#  endif
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Forward declarations */
struct slsrv_driver;

# define _SlogsrvK_INVAL_STATE     (-1)
# define _SlogsrvK_NULL_STATE      (0)
# define _SlogsrvK_UNK_STATE       (2)
# define _SlogsrvK_INITED_STATE    (3) /* ->: open */
# define _SlogsrvK_OPENING_STATE   (4)
# define _SlogsrvK_OPENED_STATE    (5)
# define _SlogsrvK_CLOSING_STATE   (6)
# define _SlogsrvK_CLOSED_STATE    _SlogsrvK_INITED_STATE
# define _SlogsrvK_FINISHING_STATE (7)

_A_opdom(Slogsrv) {
	int                     sls_state_;
	struct sockaddr_in      sls_srvaddr_;
	bo_cdllist_t            sls_clilist_;
	aorp_object_t           sls_sock_;
	bo_usec_t               sls_timeout_;
	struct slsrv_driver    *sls_driver_;
}
_A_end;

/*
 * {log,Slogsrv} protocol
 * @{
 */

# include <Bo/services/infos/LogP.h>

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (log,Slogsrv)
# define __aorpc_realize_service_file <Bo/services/infos/LogP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol(log,Slogsrv)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop(pod_init),
_A_imop(pod_fini),

_A_imop(openconn),
_A_imop(reopenconn),
_A_imop(closeconn),
_A_imop_nil(setprifilter),
_A_imop_nil(getprifilter),
_A_imop_nil(setattrfilter),
_A_imop_nil(getattrfilter),
_A_imop_nil(printf),
_A_imop_nil(vprintf),
#if 1
_A_imop_nil(wprintf),
_A_imop_nil(vwprintf),
#else
_A_imop(wprintf),
_A_imop(vwprintf),
#endif
_A_imop_nil(write),
_A_imop_nil(_send),

_A_end;

# include <Bo/aorp/AorpCend.h>

struct Slogsrv_cliinfo {
	u_int32_t             cli_connid_;
	bo_pointer_t          cli_logfile_;
	bo_utime_t            cli_last_activetm_;
};

struct Slogsrv_lelem {
	bo_cdllelem_t           cle_listel_;
	struct Slogsrv_cliinfo  cle_value_;
};

/* @} */

#endif /* !__BO_SERVICES_ALG_INFOS__SLOGSRV_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
