/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _SlogsrvdrvP.h	(N.Panasiuk)
 * include/Bo/services/alg/_SlogsrvdrvP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG__SLOGSRVDRV_P__
# define __BO_SERVICES_ALG__SLOGSRVDRV_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/infos/Logtypes.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Forward declarations */
struct slsrv_driver;

/*
 * Operation types
 */

typedef bo_status_t
(__myc_tcalltype__(mcall) *slsrv_drv_close_fn)(
# if __myc_require(prototypes)
	struct slsrv_driver *aDriver,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *slsrv_drv_logopen_fn)(
# if __myc_require(prototypes)
	struct slsrv_driver *aDriver,
	struct aorp_logattr const *aLogattr,
	bo_pointer_t *aLogPtr,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,2,3)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *slsrv_drv_logclose_fn)(
# if __myc_require(prototypes)
	struct slsrv_driver *aDriver,
	bo_pointer_t aLog,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *slsrv_drv_logadd_fn)(
# if __myc_require(prototypes)
	struct slsrv_driver *aDriver,
	bo_pointer_t aLog,
	size_t aDatasz,
	char const *aData,
	aorp_logid_t *aResultLogId,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,2,4)));

/*
 * Driver operations
 */

struct slsrv_driver_opv {
	slsrv_drv_logopen_fn   drv_logopen;
	slsrv_drv_logclose_fn  drv_logclose;
	slsrv_drv_logadd_fn    drv_logadd;
};

/******************************************************************************
 * %% BeginSection: slsrv_driver declarations
 */
# pragma mark -
# pragma mark [ slsrv_driver declarations ]
# pragma mark -

struct slsrv_driver {
	slsrv_drv_close_fn       drv_close;
	struct slsrv_driver_opv  drv_opv;
};

# define slsrv_driver_p(aDriver) ((struct slsrv_driver *)(aDriver))

# define __SLSRV_DRIVER_Opv(aDriver) (&slsrv_driver_p(aDriver)->drv_opv)

/*
 * METHODS
 */

# define SLSRV_DRIVER_Close(aDriver,anErrPtr) \
	((*slsrv_driver_p(aDriver)->drv_close)( \
		slsrv_driver_p(aDriver),anErrPtr \
	))

# define SLSRV_DRIVER_Logopen(aDriver,aLogattr,aLogPtr,anErrPtr) \
	((*__SLSRV_DRIVER_Opv(aDriver)->drv_logopen)( \
		slsrv_driver_p(aDriver),aLogattr,aLogPtr,anErrPtr \
	))

# define SLSRV_DRIVER_Logclose(aDriver,aLog,anErrPtr) \
	((*__SLSRV_DRIVER_Opv(aDriver)->drv_logclose)( \
		slsrv_driver_p(aDriver),aLog,anErrPtr \
	))

# define SLSRV_DRIVER_Logadd(aDriver,aLog,aDatasz,aData,aResultLogId,anErrPtr) \
	((*__SLSRV_DRIVER_Opv(aDriver)->drv_logadd)( \
		slsrv_driver_p(aDriver),aLog,aDatasz,aData,aResultLogId,anErrPtr \
	))

#endif /* !__BO_SERVICES_ALG__SLOGSRVDRV_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
