/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Protoblttab.h	(S.Makhmali)
 * include/Bo/services/alg/net/_Protoblttab.h
 *
 * $Id$
 */

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -
# pragma mark [ definitions ]
# pragma mark -

static int _S_ticots_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_ticots_socktypes[] = { TrifK_STREAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_ticotsord_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_ticotsord_socktypes[] = { TrifK_STREAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_ticlts_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_ticlts_socktypes[] = { TrifK_DGRAM_SOCKET, TrifK_RAW_SOCKET };


static int _S_ip_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_ip_socktypes[] = { TrifK_STREAM_SOCKET, TrifK_DGRAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_hopopts_families[] = { TrifK_INET6_FAMILY };
static int _S_hopopts_socktypes[] = {0};

static int _S_icmp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_icmp_socktypes[] = { TrifK_RAW_SOCKET, TrifK_STREAM_SOCKET, TrifK_DGRAM_SOCKET };

static int _S_igmp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_igmp_socktypes[] = { TrifK_RAW_SOCKET, TrifK_STREAM_SOCKET, TrifK_DGRAM_SOCKET };

static int _S_ggp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_ggp_socktypes[] = {0};

static int _S_ipv4_families[] = { TrifK_INET_FAMILY };
static int _S_ipv4_socktypes[] = { TrifK_STREAM_SOCKET, TrifK_DGRAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_st_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_st_socktypes[] = { TrifK_STREAM_SOCKET };

static int _S_tcp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_tcp_socktypes[] = { TrifK_STREAM_SOCKET };

static int _S_cbt_families[] = {0};
static int _S_cbt_socktypes[] = {0};

static int _S_egp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_egp_socktypes[] = {0};

static int _S_igp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_igp_socktypes[] = {0};

static int _S_pup_families[] = {0};
static int _S_pup_socktypes[] = {0};

static int _S_udp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_udp_socktypes[] = { TrifK_DGRAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_idp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY, TrifK_IPX_FAMILY };
static int _S_idp_socktypes[] = { TrifK_DGRAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_rdp_families[] = {0};
static int _S_rdp_socktypes[] = {0};

static int _S_ipv6_families[] = { TrifK_INET6_FAMILY };
static int _S_ipv6_socktypes[] = { TrifK_STREAM_SOCKET, TrifK_DGRAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_routing_families[] = { TrifK_INET6_FAMILY, TrifK_INET_FAMILY, TrifK_ROUTE_FAMILY };
static int _S_routing_socktypes[] = { TrifK_STREAM_SOCKET, TrifK_DGRAM_SOCKET, TrifK_RAW_SOCKET };

static int _S_fragment_families[] = { TrifK_INET_FAMILY };
static int _S_fragment_socktypes[] = {0};

static int _S_esp_families[] = { TrifK_INET_FAMILY };
static int _S_esp_socktypes[] = {0};

static int _S_ah_families[] = { TrifK_INET_FAMILY };
static int _S_ah_socktypes[] = {0};

static int _S_icmpv6_families[] = { TrifK_INET6_FAMILY };
static int _S_icmpv6_socktypes[] = { TrifK_RAW_SOCKET, TrifK_STREAM_SOCKET, TrifK_DGRAM_SOCKET };

static int _S_none_families[] = {0};
static int _S_none_socktypes[] = {0};

static int _S_dstopts_families[] = { TrifK_INET6_FAMILY };
static int _S_dstopts_socktypes[] = {0};

static int _S_nd_families[] = {0};
static int _S_nd_socktypes[] = {0};

static int _S_iclfxbm_families[] = {0};
static int _S_iclfxbm_socktypes[] = {0};

static int _S_pim_families[] = {0};
static int _S_pim_socktypes[] = {0};

static int _S_pgm_families[] = {0};
static int _S_pgm_socktypes[] = {0};

static int _S_l2tp_families[] = {0};
static int _S_l2tp_socktypes[] = {0};

static int _S_sctp_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_sctp_socktypes[] = { TrifK_STREAM_SOCKET };

static int _S_raw_families[] = { TrifK_INET_FAMILY, TrifK_INET6_FAMILY };
static int _S_raw_socktypes[] = { TrifK_RAW_SOCKET };

static int _S_max_families[] = {0};
static int _S_max_socktypes[] = {0};


static struct Tris_proto_dbelem _S_Tris_proto_builtin_table[] = {
	/*
	 * LOOPBACK
	 */
	{ TrifK_TICOTS_PROTOCOL, TRIF_TICOTS_PROTOCOL, _S_ticots_families, _S_ticots_socktypes },
	{ TrifK_TICOTSORD_PROTOCOL, TRIF_TICOTSORD_PROTOCOL, _S_ticotsord_families, _S_ticotsord_socktypes },
	{ TrifK_TICLTS_PROTOCOL, TRIF_TICLTS_PROTOCOL, _S_ticlts_families, _S_ticlts_socktypes },

	{ TrifK_IP_PROTOCOL	, TRIF_IP_PROTOCOL, _S_ip_families, _S_ip_socktypes },
	{ TrifK_HOPOPTS_PROTOCOL	, TRIF_HOPOPTS_PROTOCOL, _S_hopopts_families, NULL },
	{ TrifK_ICMP_PROTOCOL, TRIF_ICMP_PROTOCOL, _S_icmp_families, _S_icmp_socktypes },
	{ TrifK_IGMP_PROTOCOL, TRIF_IGMP_PROTOCOL, _S_igmp_families, _S_igmp_socktypes },
	{ TrifK_GGP_PROTOCOL, TRIF_GGP_PROTOCOL, _S_ggp_families, NULL },
	{ TrifK_IPV4_PROTOCOL, TRIF_IPV4_PROTOCOL, _S_ipv4_families, _S_ipv4_socktypes },
	{ TrifK_ST_PROTOCOL, TRIF_ST_PROTOCOL, _S_st_families, _S_st_socktypes },
	{ TrifK_TCP_PROTOCOL, TRIF_TCP_PROTOCOL,  _S_tcp_families, _S_tcp_socktypes },
	{ TrifK_EGP_PROTOCOL, TRIF_EGP_PROTOCOL, _S_egp_families, NULL },
	{ TrifK_PUP_PROTOCOL, TRIF_PUP_PROTOCOL, NULL, NULL },
	{ TrifK_UDP_PROTOCOL, TRIF_UDP_PROTOCOL, _S_udp_families, _S_udp_socktypes },
	{ TrifK_IDP_PROTOCOL, TRIF_IDP_PROTOCOL, _S_idp_families, _S_idp_socktypes },
	{ TrifK_RDP_PROTOCOL, TRIF_RDP_PROTOCOL, NULL, NULL },
	{ TrifK_IPV6_PROTOCOL, TRIF_IPV6_PROTOCOL, _S_ipv6_families, _S_ipv6_socktypes },
	{ TrifK_ROUTING_PROTOCOL, TRIF_ROUTING_PROTOCOL, _S_routing_families, _S_routing_socktypes },
	{ TrifK_FRAGMENT_PROTOCOL, TRIF_FRAGMENT_PROTOCOL, _S_fragment_families, NULL },
	{ TrifK_ESP_PROTOCOL, TRIF_ESP_PROTOCOL, _S_esp_families, NULL },
	{ TrifK_AH_PROTOCOL, TRIF_AH_PROTOCOL, _S_ah_families, NULL },
	{ TrifK_ICMPV6_PROTOCOL, TRIF_ICMPV6_PROTOCOL, _S_icmpv6_families, _S_icmpv6_socktypes },
	{ TrifK_NONE_PROTOCOL, TRIF_NONE_PROTOCOL, NULL, NULL },
	{ TrifK_DSTOPTS_PROTOCOL, TRIF_DSTOPTS_PROTOCOL, _S_dstopts_families, NULL },
	{ TrifK_ND_PROTOCOL, TRIF_ND_PROTOCOL, NULL, NULL },
	{ TrifK_PIM_PROTOCOL, TRIF_PIM_PROTOCOL, NULL, NULL },
	{ TrifK_PGM_PROTOCOL, TRIF_PGM_PROTOCOL, NULL, NULL },
	{ TrifK_SCTP_PROTOCOL, TRIF_SCTP_PROTOCOL, _S_sctp_families, _S_sctp_socktypes },
	{ TrifK_RAW_PROTOCOL, TRIF_RAW_PROTOCOL, _S_raw_families, _S_raw_socktypes },

#ifdef BO_SYSTYPE_WIN32
	/*
	 * WIN32
	 */
	{ TrifK_CBT_PROTOCOL, TRIF_CBT_PROTOCOL, NULL, NULL },
	{ TrifK_IGP_PROTOCOL, TRIF_IGP_PROTOCOL, _S_igp_families, NULL },
	{ TrifK_ICLFXBM_PROTOCOL, TRIF_ICLFXBM_PROTOCOL, NULL, NULL },
	{ TrifK_L2TP_PROTOCOL, TRIF_L2TP_PROTOCOL, NULL, NULL },
#else
	/*
	 * UNIX
	 */
	{ TrifK_IPIP_PROTOCOL, TRIF_IPIP_PROTOCOL, _S_ipv4_families, _S_ipv4_socktypes },
	{ TrifK_PIGP_PROTOCOL, TRIF_PIGP_PROTOCOL, _S_igp_families, _S_igp_socktypes },
	{ TrifK_RCCMON_PROTOCOL, TRIF_RCCMON_PROTOCOL, NULL, NULL },
	{ TrifK_NVPII_PROTOCOL, TRIF_NVPII_PROTOCOL, NULL, NULL },
	{ TrifK_ARGUS_PROTOCOL, TRIF_ARGUS_PROTOCOL, NULL, NULL },
	{ TrifK_EMCON_PROTOCOL, TRIF_EMCON_PROTOCOL, NULL, NULL },
	{ TrifK_XNET_PROTOCOL, TRIF_XNET_PROTOCOL, NULL, NULL },
	{ TrifK_CHAOS_PROTOCOL, TRIF_CHAOS_PROTOCOL, NULL, NULL },
	{ TrifK_MUX_PROTOCOL, TRIF_MUX_PROTOCOL, NULL, NULL },
	{ TrifK_MEAS_PROTOCOL, TRIF_MEAS_PROTOCOL, NULL, NULL },
	{ TrifK_HMP_PROTOCOL, TRIF_HMP_PROTOCOL, NULL, NULL },
	{ TrifK_PRM_PROTOCOL, TRIF_PRM_PROTOCOL, NULL, NULL },
	{ TrifK_TRUNK1_PROTOCOL, TRIF_TRUNK1_PROTOCOL, NULL, NULL },
	{ TrifK_TRUNK2_PROTOCOL, TRIF_TRUNK2_PROTOCOL, NULL, NULL },
	{ TrifK_LEAF1_PROTOCOL, TRIF_LEAF1_PROTOCOL, NULL, NULL },
	{ TrifK_LEAF2_PROTOCOL, TRIF_LEAF2_PROTOCOL, NULL, NULL },
	{ TrifK_IRTP_PROTOCOL, TRIF_IRTP_PROTOCOL, NULL, NULL },
	{ TrifK_TP_PROTOCOL, TRIF_TP_PROTOCOL, NULL, NULL },
	{ TrifK_BLT_PROTOCOL, TRIF_BLT_PROTOCOL, NULL, NULL },
	{ TrifK_NSP_PROTOCOL, TRIF_NSP_PROTOCOL, NULL, NULL },
	{ TrifK_INP_PROTOCOL, TRIF_INP_PROTOCOL, NULL, NULL },
	{ TrifK_SEP_PROTOCOL, TRIF_SEP_PROTOCOL, NULL, NULL },
	{ TrifK_3PC_PROTOCOL, TRIF_3PC_PROTOCOL, NULL, NULL },
	{ TrifK_IDPR_PROTOCOL, TRIF_IDPR_PROTOCOL, NULL, NULL },
	{ TrifK_XTP_PROTOCOL, TRIF_XTP_PROTOCOL, NULL, NULL },
	{ TrifK_DDP_PROTOCOL, TRIF_DDP_PROTOCOL, NULL, NULL },
	{ TrifK_CMTP_PROTOCOL, TRIF_CMTP_PROTOCOL, NULL, NULL },
	{ TrifK_TPXX_PROTOCOL, TRIF_TPXX_PROTOCOL, NULL, NULL },
	{ TrifK_IL_PROTOCOL, TRIF_IL_PROTOCOL, NULL, NULL },
	{ TrifK_SDRP_PROTOCOL, TRIF_SDRP_PROTOCOL, NULL, NULL },
	{ TrifK_IDRP_PROTOCOL, TRIF_IDRP_PROTOCOL, NULL, NULL },
	{ TrifK_RSVP_PROTOCOL, TRIF_RSVP_PROTOCOL, NULL, NULL },
	{ TrifK_GRE_PROTOCOL, TRIF_GRE_PROTOCOL, NULL, NULL },
	{ TrifK_MHRP_PROTOCOL, TRIF_MHRP_PROTOCOL, NULL, NULL },
	{ TrifK_BHA_PROTOCOL, TRIF_BHA_PROTOCOL, NULL, NULL },
	{ TrifK_INLSP_PROTOCOL, TRIF_INLSP_PROTOCOL, NULL, NULL },
	{ TrifK_SWIPE_PROTOCOL, TRIF_SWIPE_PROTOCOL, NULL, NULL },
	{ TrifK_NHRP_PROTOCOL, TRIF_NHRP_PROTOCOL, NULL, NULL },
	{ TrifK_MOBILE_PROTOCOL, TRIF_MOBILE_PROTOCOL, NULL, NULL },
	{ TrifK_TLSP_PROTOCOL, TRIF_TLSP_PROTOCOL, NULL, NULL },
	{ TrifK_SKIP_PROTOCOL, TRIF_SKIP_PROTOCOL, NULL, NULL },
	{ TrifK_AHIP_PROTOCOL, TRIF_AHIP_PROTOCOL, NULL, NULL },
	{ TrifK_CFTP_PROTOCOL, TRIF_CFTP_PROTOCOL, NULL, NULL },
	{ TrifK_HELLO_PROTOCOL, TRIF_HELLO_PROTOCOL, NULL, NULL },
	{ TrifK_SATEXPAK_PROTOCOL, TRIF_SATEXPAK_PROTOCOL, NULL, NULL },
	{ TrifK_KRYPTOLAN_PROTOCOL, TRIF_KRYPTOLAN_PROTOCOL, NULL, NULL },
	{ TrifK_RVD_PROTOCOL, TRIF_RVD_PROTOCOL, NULL, NULL },
	{ TrifK_IPPC_PROTOCOL, TRIF_IPPC_PROTOCOL, NULL, NULL },
	{ TrifK_ADFS_PROTOCOL, TRIF_ADFS_PROTOCOL, NULL, NULL },
	{ TrifK_SATMON_PROTOCOL, TRIF_SATMON_PROTOCOL, NULL, NULL },
	{ TrifK_VISA_PROTOCOL, TRIF_VISA_PROTOCOL, NULL, NULL },
	{ TrifK_IPCV_PROTOCOL, TRIF_IPCV_PROTOCOL, NULL, NULL },
	{ TrifK_CPNX_PROTOCOL, TRIF_CPNX_PROTOCOL, NULL, NULL },
	{ TrifK_CPHB_PROTOCOL, TRIF_CPHB_PROTOCOL, NULL, NULL },
	{ TrifK_WSN_PROTOCOL, TRIF_WSN_PROTOCOL, NULL, NULL },
	{ TrifK_PVP_PROTOCOL, TRIF_PVP_PROTOCOL, NULL, NULL },
	{ TrifK_BRSATMON_PROTOCOL, TRIF_BRSATMON_PROTOCOL, NULL, NULL },
	{ TrifK_WBMON_PROTOCOL, TRIF_WBMON_PROTOCOL, NULL, NULL },
	{ TrifK_WBEXPAK_PROTOCOL, TRIF_WBEXPAK_PROTOCOL, NULL, NULL },
	{ TrifK_EON_PROTOCOL, TRIF_EON_PROTOCOL, NULL, NULL },
	{ TrifK_VMTP_PROTOCOL, TRIF_VMTP_PROTOCOL, NULL, NULL },
	{ TrifK_SVMTP_PROTOCOL, TRIF_SVMTP_PROTOCOL, NULL, NULL },
	{ TrifK_VINES_PROTOCOL, TRIF_VINES_PROTOCOL, NULL, NULL },
	{ TrifK_TTP_PROTOCOL, TRIF_TTP_PROTOCOL, NULL, NULL },
	{ TrifK_IGP_PROTOCOL, TRIF_IGP_PROTOCOL, _S_igp_families, _S_igp_socktypes },
	{ TrifK_DGP_PROTOCOL, TRIF_DGP_PROTOCOL, NULL, NULL },
	{ TrifK_TCF_PROTOCOL, TRIF_TCF_PROTOCOL, NULL, NULL },
	{ TrifK_IGRP_PROTOCOL, TRIF_IGRP_PROTOCOL, NULL, NULL },
	{ TrifK_OSPFIGP_PROTOCOL, TRIF_OSPFIGP_PROTOCOL, NULL, NULL },
	{ TrifK_SRPC_PROTOCOL, TRIF_SRPC_PROTOCOL, NULL, NULL },
	{ TrifK_LARP_PROTOCOL, TRIF_LARP_PROTOCOL, NULL, NULL },
	{ TrifK_MTP_PROTOCOL, TRIF_MTP_PROTOCOL, NULL, NULL },
	{ TrifK_AX25_PROTOCOL, TRIF_AX25_PROTOCOL, NULL, NULL },
	{ TrifK_IPEIP_PROTOCOL, TRIF_IPEIP_PROTOCOL, NULL, NULL },
	{ TrifK_MICP_PROTOCOL, TRIF_MICP_PROTOCOL, NULL, NULL },
	{ TrifK_SCCSP_PROTOCOL, TRIF_SCCSP_PROTOCOL, NULL, NULL },
	{ TrifK_ETHERIP_PROTOCOL, TRIF_ETHERIP_PROTOCOL, NULL, NULL },
	{ TrifK_ENCAP_PROTOCOL, TRIF_ENCAP_PROTOCOL, NULL, NULL },
	{ TrifK_APES_PROTOCOL, TRIF_APES_PROTOCOL, NULL, NULL },
	{ TrifK_GMTP_PROTOCOL, TRIF_GMTP_PROTOCOL, NULL, NULL },
	{ TrifK_IPCOMP_PROTOCOL, TRIF_IPCOMP_PROTOCOL, NULL, NULL },
	{ TrifK_CARP_PROTOCOL, TRIF_CARP_PROTOCOL, NULL, NULL },
	{ TrifK_PFSYNC_PROTOCOL, TRIF_PFSYNC_PROTOCOL, NULL, NULL },
	{ TrifK_OLD_DIVERT_PROTOCOL, TRIF_OLD_DIVERT_PROTOCOL, NULL, NULL },
	{ TrifK_DONE_PROTOCOL, TRIF_DONE_PROTOCOL, NULL, NULL },
	{ TrifK_SPACER_PROTOCOL, TRIF_SPACER_PROTOCOL, NULL, NULL },
#endif

	{ TrifK_MAX_PROTOCOL, TRIF_MAX_PROTOCOL, NULL, NULL },

	{ TrifK_INVAL_PROTOCOL, NULL, NULL, NULL },
};

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
