/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2009 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _SocketP.h	(S.Makhmali)
 * include/Bo/services/alg/net/_SocketP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_NET__SOCKET_P__
# define __BO_SERVICES_ALG_NET__SOCKET_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/alg/net/_Socket.h>

# include <Bo/Types.h>

# ifdef BO_SYSTYPE_WIN32
#  if __myc_exists(winsock2,h)
#   include __MYC_LIBC_HEADER(winsock2.h)
#   ifndef HAS_TIMEVAL_STRUC
#    define HAS_TIMEVAL_STRUC
#   endif
#  endif
	/* SOCKET */
# endif /* BO_SYSTYPE_WIN32 */

/******************************************************************************
 * %% BeginSection: type declarations
 */

# ifdef __BO_SOCKSYSOBJ_TYPE__
typedef __BO_SOCKSYSOBJ_TYPE__ _socksysobj_t;
#  define _SockK_INVAL_SOCKSYSOBJ __BO_INVAL_SOCKSYSOBJ
# else
# ifdef BO_SYSTYPE_WIN32
typedef SOCKET _socksysobj_t;
#  define _SockK_INVAL_SOCKSYSOBJ (INVALID_SOCKET)
# else
typedef int _socksysobj_t;
#  define _SockK_INVAL_SOCKSYSOBJ ((_socksysobj_t)-1)
# endif
# endif /* __BO_SOCKSYSOBJ_TYPE__ */


_A_opdom(Socket) {
	_socksysobj_t  sck_sysobj;
	int            sck_qlen; /* backlog */
	int            sck_family;
	int            sck_type;
	int            sck_protocol;
	int            sck_semantics;
	_Bool          sck_loopback;
	_Bool          sck_listening;
};

/* ----------------------------------------------------------------------------
 * {oti,Socket} protocol
 * @{
 */

#if 0
# define _SockK_NONE_SYSCALL (-1)
# define _SockK_SYSCALL_SOCKET (0)
# define _SockK_SYSCALL_CLOSE (1)
# define _SockK_SYSCALL_SHUTDOWN (2)
# define _SockK_SYSCALL_BIND (3)
# define _SockK_SYSCALL_LISTEN (4)
# define _SockK_SYSCALL_CONNECT (5)
# define _SockK_SYSCALL_ACCEPT (6)
# define _SockK_SYSCALL_RECV (7)
# define _SockK_SYSCALL_SEND (8)
# define _SockK_SYSCALL_RCVUDATA (9)
# define _SockK_SYSCALL_SNDUDATA (10)
# define _SockK_SYSCALL_RCVVUDATA (11)
# define _SockK_SYSCALL_SNDVUDATA (12)
# define _SockK_SYSCALL_GETOPT (13)
# define _SockK_SYSCALL_SETOPT (14)
# define _SockK_SYSCALL_GETSOCKNAME (15)
# define _SockK_SYSCALL_GETPEERNAME (16)
# define _SockK_SYSCALL_IOCTLSOCKET (17)
# define _SockK_SYSCALL_GETADDRINFO (18)
# define _SockK_SYSCALL_IOCTL (19)
# define _SockK_SYSCALL_SENDMSG (20)
# define _SockK_SYSCALL_RECVMSG (21)
# define _SockK_SYSCALL_SENDTO (22)
#endif

# include <Bo/services/net/OtiP.h>

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (oti,Socket)
# define __aorpc_realize_service_file <Bo/services/net/OtiP.h>
# include <Bo/aorp/AorpCpro.h>

_A_protocol(oti,Socket)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop(init_once),
_A_imop(fini_once),
_A_imop(pod_init),
_A_imop(pod_fini),

_A_imop(mklisten),
_A_imop(listen),
_A_imop(accept),

_A_imop(connect),

_A_imop(receive),
_A_imop(send),

_A_imop_nil(rcvconn),
_A_imop_nil(reserved1),

_A_imop(rcvudata),
_A_imop(rcvuderr),
_A_imop(sndudata),

_A_imop(rcvvudata),
_A_imop(sndvudata),

_A_imop(recvmsg),
_A_imop(sendmsg),

_A_imop(getprotaddr),

_A_imop(disconnect),
_A_imop(shutdown),

_A_imop(rcvrel),
_A_imop(rcvdis),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

/* ----------------------------------------------------------------------------
 * {ios,Socket} protocol
 * @{
 */

# include <Bo/services/io/IosP.h>

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (ios,Socket)
# define __aorpc_realize_service_file <Bo/services/io/IosP.h>
# include <Bo/aorp/AorpCpro.h>

/*
 * The impl of the 'ios' service acts on Socket domain.
 */
_A_protocol(ios,Socket)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop_nil(pod_init),
_A_imop_nil(pod_fini),

_A_imop(gettinfo),
_A_imop(getstate),
_A_imop(sync),
_A_imop(optctl),
_A_imop(bind),
_A_imop(rebind),
_A_imop(unbind),
_A_imop(control),

_A_imop(poll),
_A_imop(poll_abs),
_A_imop(getpollbox),
_A_imop_rr(getselbox,getpollbox),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

/* ----------------------------------------------------------------------------
 * {iostr,Socket} protocol
 * @{
 */

# include <Bo/services/io/IostrP.h>

# define __aorpc_ns_protocol_c 2
# define __aorpc_ns_protocol (iostr,Socket)
# define __aorpc_realize_service_file <Bo/services/io/IostrP.h>
# include <Bo/aorp/AorpCpro.h>

/*
 * The impl of the 'iostr' service acts on Socket domain.
 */
_A_protocol(iostr,Socket)
{
	_A_domdecl_;
}
_A_end_pod

_A_imop_nil(init_once),
_A_imop_nil(fini_once),
_A_imop_nil(pod_init),
_A_imop_nil(pod_fini),

_A_imop(setiobuf),
_A_imop(getiobuf),

_A_imop(read),
_A_imop(readall),
_A_imop(getrdbuf),
_A_imop(endrdbuf),

_A_imop(write),
_A_imop(printf),
_A_imop(vprintf),
_A_imop(wprintf),
_A_imop(vwprintf),
_A_imop(flush),
_A_imop(seek),

_A_imop(readv),
_A_imop(writev),
_A_imop_nil(pread),
_A_imop_nil(pwrite),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */

#endif /* !__BO_SERVICES_ALG_NET__SOCKET_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
