/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _TypesI.h	(N.Panasiuk)
 * include/Bo/services/alg/textan/_TypesI.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_ALG_TEXTAN__TYPES_I__
# define __BO_SERVICES_ALG_TEXTAN__TYPES_I__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# define _AorpK_LRSYXAN_TABELEM_VALUE_MASK     (0x0fffffff)
# define _AorpK_LRSYXAN_TABELEM_COMMAND_MASK   (0xf0000000)
# define _AorpK_LRSYXAN_TABELEM_COMMAND_MASK_S (28)

# define _AorpK_LRSYXAN_TABELEM_ERROR_COMMAND  (0x00000000)
# define _AorpK_LRSYXAN_TABELEM_SHIFT_COMMAND  (0x10000000)
# define _AorpK_LRSYXAN_TABELEM_REDUC_COMMAND  (0x20000000)
# define _AorpK_LRSYXAN_TABELEM_ACCEPT_COMMAND (0x30000000)
# define _AorpK_LRSYXAN_TABELEM_GOTO_COMMAND   (0x40000000)

typedef u_int32_t aorp_lrsyxan_tabelem_t;
typedef u_int32_t aorp_lrsyxan_command_t;
typedef u_int32_t aorp_lrsyxan_value_t;

/* ----------------------------------------------------------------------------
 * macro-helper declarations
 * @{
 */

/*
aorp_lrsyxan_tabelem_t
_AORP_MAKE_LRSYXAN_ShiftCommand(aorp_lrsyxan_value_t aState);
 */
# define _AORP_MAKE_LRSYXAN_ShiftCommand(aState) \
	((aorp_lrsyxan_tabelem_t)(_AorpK_LRSYXAN_TABELEM_SHIFT_COMMAND \
			| ((aState) & _AorpK_LRSYXAN_TABELEM_VALUE_MASK) \
		) \
	)

/*
aorp_lrsyxan_tabelem_t
_AORP_MAKE_LRSYXAN_ReducCommand(aorp_lrsyxan_value_t aProductNumber);
 */
# define _AORP_MAKE_LRSYXAN_ReducCommand(aProductNumber) \
	((aorp_lrsyxan_tabelem_t)(_AorpK_LRSYXAN_TABELEM_REDUC_COMMAND \
			| ((aProductNumber) & _AorpK_LRSYXAN_TABELEM_VALUE_MASK) \
		) \
	)

/*
aorp_lrsyxan_tabelem_t
_AORP_MAKE_LRSYXAN_AcceptCommand();
 */
# define _AORP_MAKE_LRSYXAN_AcceptCommand() \
	((aorp_lrsyxan_tabelem_t)_AorpK_LRSYXAN_TABELEM_ACCEPT_COMMAND)

/*
aorp_lrsyxan_tabelem_t
_AORP_MAKE_LRSYXAN_GotoCommand(aorp_lrsyxan_value_t aState);
 */
# define _AORP_MAKE_LRSYXAN_GotoCommand(aState) \
	((aorp_lrsyxan_tabelem_t)(_AorpK_LRSYXAN_TABELEM_GOTO_COMMAND \
			| ((aState) & _AorpK_LRSYXAN_TABELEM_VALUE_MASK) \
		) \
	)

# pragma mark -

/*
aorp_lrsyxan_command_t
_AORP_LRSYXAN_ExtractCommand(aorp_lrsyxan_tabelem_t aTabelem);
 */
# define _AORP_LRSYXAN_ExtractCommand(aTabelem) \
	((aorp_lrsyxan_command_t)((aTabelem) & _AorpK_LRSYXAN_TABELEM_COMMAND_MASK))

/*
aorp_lrsyxan_value_t
_AORP_LRSYXAN_ExtractValue(aorp_lrsyxan_tabelem_t aTabelem);
 */
# define _AORP_LRSYXAN_ExtractValue(aTabelem) \
	((aorp_lrsyxan_value_t)((aTabelem) & _AorpK_LRSYXAN_TABELEM_VALUE_MASK))

/* @} */

#endif /* !__BO_SERVICES_ALG_TEXTAN__TYPES_I__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
