/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Ctnr.h	(V.Sokur)
 * include/Bo/services/basis/Ctnr.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_BASIS_CTNR__
# define __BO_SERVICES_BASIS_CTNR__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/basis/CtnrP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * Key interpretation in the operations below depends
 * on the implementation.
 * For example, the impl. may treat name as index: idx = strtol(name).
 */

__MYC_BEGIN_CDECLS

/*
 * COMMON
 */

/*
bo_index_t
__attribute__((nonnull(1)))
AorpCtnrCount(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrCount(aThis, anErrPtr) \
	((bo_index_t) _A_call(count, container, aThis, anErrPtr))

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpCtnrMaxcount(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrMaxcount(aThis, anErrPtr) \
	((bo_index_t) _A_call(maxcount, container, aThis, anErrPtr))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpCtnrClear(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 *
 * All objects in the container are released.
 */
# define AorpCtnrClear(aThis, anErrPtr) \
	_A_call(clear, container, aThis, anErrPtr)

/*
 * CURSOR
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCtnrOpencrsr(
	aorp_object_t aThis,
	struct aorp_qcursor *aCursor,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrOpencrsr(aThis, aCursor, anErrPtr) \
	_A_call(opencrsr, container, aThis, anErrPtr, aCursor)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpQcursorClose(
	struct aorp_qcursor *aCursor,
	struct aorp_error *anErrPtr
	);
 */
# define AorpQcursorClose(aCursor, anErrPtr) \
	_A_closcall(closecrsr, container, AORP_QCURSOR_Closure(aCursor), anErrPtr, aCursor)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpQcursorAdvance(
	struct aorp_qcursor *aCursor,
	bo_index_t aDistance,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * BoS_ERR: error;
 * BoS_FALSE: partial success: end of container;
 * BoS_TRUE: success;
 */
# define AorpQcursorAdvance(aCursor, aDistance, anErrPtr) \
	_A_closcall(advancecrsr, container, AORP_QCURSOR_Closure(aCursor), anErrPtr, \
		aCursor, (bo_index_t)(aDistance))

# define AorpQcursorNext(aCursor, anErrPtr) AorpQcursorAdvance(aCursor, 1, anErrPtr)
# define AorpQcursorPrev(aCursor, anErrPtr) AorpQcursorAdvance(aCursor, -1, anErrPtr)

/*
bo_index_t
__attribute__((nonnull(1)))
AorpQcursorGet(
	struct aorp_qcursor *aCursor,
	struct aorp_udvariant aValuev[],
	size_t aValuec,
	_Bool WithRetain, // = false
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The function returns an actual count of values.
 */
# define AorpQcursorGet(aCursor, aValuev, aValuec, WithRetain, anErrPtr) \
	((bo_index_t) _A_closcall(getbycrsr, container, AORP_QCURSOR_Closure(aCursor), anErrPtr, \
		aCursor, aValuev, (size_t)(aValuec), (int)((WithRetain) & 0x0f)))

# pragma mark -

/* NOT FOR YOUR BUSINESS
 * Theses operations are intended to be called by a cursor.
 * @{
 */

/*
bo_status_t
__attribute__((nonnull(1,2)))
_AorpCtnrClosecrsr(
	aorp_object_t ,
	struct aorp_qcursor *,
	struct aorp_error *
	);
 */
# define _AorpCtnrClosecrsr(x, y, z) \
	_A_call(closecrsr, container, x, z, y)

/*
bo_status_t
__attribute__((nonnull(1,2)))
_AorpCtnrAdvancecrsr(
	aorp_object_t ,
	struct aorp_qcursor *,
	bo_index_t ,
	struct aorp_error *
	);
 *
 * NOTE!!!
 * BoS_ERR: error;
 * BoS_FALSE: partial success: end of container;
 * BoS_TRUE: success;
 */
# define _AorpCtnrAdvancecrsr(x, y1, y2, z) \
	_A_call(advancecrsr, container, x, z, y1, (bo_index_t)(y2))

/*
bo_index_t
__attribute__((nonnull(1,2)))
_AorpCtnrGetbycrsr(
	aorp_object_t ,
	struct aorp_qcursor *,
	struct aorp_udvariant [],
	size_t ,
	_Bool ,
	struct aorp_error *
	);
 *
 * NOTE!!!
 * The function returns an actual count of values.
 */
# define _AorpCtnrGetbycrsr(x, y1, y2, y3, y4, z) \
	((bo_index_t) _A_call(getbycrsr, container, x, z, y1, y2, (size_t)(y3), (int)((y3) & 0x0f)))

/* @} */

/*
 * ELEMENT ACCESS
 *
 * The returned value in all these algorithms is not retained.
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrAt(
	aorp_object_t aThis,
	AORP_ONEP_OF((aorp_comparable_t, aorp_object_t)) aKey,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrAt(aThis, aKey, aValue, anErrPtr) \
	_A_call(at, container, aThis, anErrPtr, aKey, aValue)

/*
 * Special variants
 * @{
 */
# define AorpCtnrAt_tmpl(_TValueType, _TField, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	do { \
		bo_status_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrAt(aThis, aKey, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrAt_ptr(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrAt_obj(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrAt_str(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(char*, udc_str, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrAt_wcs(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrAt_i32(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrAt_i64(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrAt_f32(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(float, udc_f32, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrAt_f64(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrAt_tmpl(double, udc_f64, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)
/* @} */

/*
 * ELEMENT ACCESS <BY INDEX>
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCtnrFront(
	aorp_object_t aThis,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrFront(aThis, aValue, anErrPtr) \
	AorpCtnrAtindex(aThis, CtnrK_FRONT_INDEX, aValue, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrFront_tmpl(_TValueType, _TField, aStatusEqu, aThis, aValuePtr, anErrPtr) \
	do { \
		bo_status_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrFront(aThis, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrFront_ptr(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrFront_obj(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrFront_str(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(char*, udc_str, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrFront_wcs(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrFront_i32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrFront_i64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrFront_f32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(float, udc_f32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrFront_f64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrFront_tmpl(double, udc_f64, aStatusEqu, aThis, aValuePtr, anErrPtr)
/* @} */

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCtnrBack(
	aorp_object_t aThis,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrBack(aThis, aValue, anErrPtr) \
	AorpCtnrAtindex(aThis, CtnrK_BACK_INDEX, aValue, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrBack_tmpl(_TValueType, _TField, aStatusEqu, aThis, aValuePtr, anErrPtr) \
	do { \
		bo_status_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrBack(aThis, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrBack_ptr(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrBack_obj(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrBack_str(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(char*, udc_str, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrBack_wcs(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrBack_i32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrBack_i64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrBack_f32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(float, udc_f32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrBack_f64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrBack_tmpl(double, udc_f64, aStatusEqu, aThis, aValuePtr, anErrPtr)
/* @} */

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpCtnrAtindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrAtindex(aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrAt(aThis, \
		(&(struct aorp_comparable_idx){ AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) }), \
		aValue, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrAtindex(aThis, anIndex, aValue, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_idx ___key = { \
			AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) \
		}; \
		/* return */ AorpCtnrAt(aThis, &___key, aValue, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_status_t
__attribute__((nonnull(1,3)))
AorpCtnrAtindex(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
# else
	aThis, anIndex, aValue, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	bo_index_t anIndex;
	struct aorp_udvariant *aValue;
	struct aorp_error *anErrPtr;
# endif
{
	bo_status_t status;
	struct aorp_comparable_idx ___key = {
		AorpK_HINT_INDEX, 0, (anIndex)
	};
	status = AorpCtnrAt(aThis, &___key, aValue, anErrPtr);
	return (status);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrAtindex_tmpl(_TValueType, _TField, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	do { \
		bo_status_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrAtindex(aThis, anIndex, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrAtindex_ptr(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrAtindex_obj(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrAtindex_str(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(char*, udc_str, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrAtindex_wcs(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrAtindex_i32(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(int32_t, udc_i32, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrAtindex_i64(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(int64_t, udc_i64, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrAtindex_f32(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(float, udc_f32, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrAtindex_f64(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrAtindex_tmpl(double, udc_f64, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)
/* @} */

/*
 * ELEMENT ACCESS <BY NAME>
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrAtname(
	aorp_object_t aThis,
	char const *aName,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrAtname(aThis, aName, aValue, anErrPtr) \
	AorpCtnrAt(aThis, \
		(&(struct aorp_comparable_str){ AorpK_HINT_STR, 0, (aName) }), \
		aValue, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrAtname(aThis, aName, aValue, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_str ___key = { \
			AorpK_HINT_STR, 0, (aName) \
		}; \
		/* return */ AorpCtnrAt(aThis, &___key, aValue, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrAtname(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	char const *aName,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
# else
	aThis, aName, aValue, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	char const *aName;
	struct aorp_udvariant *aValue;
	struct aorp_error *anErrPtr;
# endif
{
	bo_status_t status;
	struct aorp_comparable_str ___key = {
		AorpK_HINT_STR, 0, (aName)
	};
	status = AorpCtnrAt(aThis, &___key, aValue, anErrPtr);
	return (status);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrAtname_tmpl(_TValueType, _TField, aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	do { \
		bo_status_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrAtname(aThis, aName, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrAtname_ptr(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrAtname_obj(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrAtname_str(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(char*, udc_str, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrAtname_wcs(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrAtname_i32(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrAtname_i64(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrAtname_f32(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(float, udc_f32, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrAtname_f64(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrAtname_tmpl(double, udc_f64, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)
/* @} */

/*
 * CURSOR ACCESS
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrFind(
	aorp_object_t aThis,
	AORP_ONEP_OF((aorp_comparable_t, aorp_object_t)) aKey,
	struct aorp_qcursor *aCursor, // cursor should be opened
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrFind(aThis, aKey, aCursor, anErrPtr) \
	_A_call(find, container, aThis, anErrPtr, aKey, aCursor)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCtnrBegin(
	aorp_object_t aThis,
	struct aorp_qcursor *aCursor, // cursor should be opened
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrBegin(aThis, aCursor, anErrPtr) \
	AorpCtnrFindbyindex(aThis, CtnrK_BEGIN_INDEX, aCursor, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCtnrEnd(
	aorp_object_t aThis,
	struct aorp_qcursor *aCursor, // cursor should be opened
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrEnd(aThis, aCursor, anErrPtr) \
	AorpCtnrFindbyindex(aThis, CtnrK_END_INDEX, aCursor, anErrPtr)

/*
 * CURSOR ACCESS <BY INDEX>
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpCtnrFindbyindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_qcursor *aCursor, // cursor should be opened
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrFindbyindex(aThis, anIndex, aCursor, anErrPtr) \
	AorpCtnrFind(aThis, \
		(&(struct aorp_comparable_idx){ AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) }), \
		aCursor, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrFindbyindex(aThis, anIndex, aCursor, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_idx ___key = { \
			AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) \
		}; \
		/* return */ AorpCtnrFind(aThis, &___key, aCursor, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_status_t
__attribute__((nonnull(1,3)))
AorpCtnrFindbyindex(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_qcursor *aCursor, /* cursor should be opened */
	struct aorp_error *anErrPtr
# else
	aThis, anIndex, aCursor, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	bo_index_t anIndex;
	struct aorp_qcursor *aCursor; /* cursor should be opened */
	struct aorp_error *anErrPtr;
# endif
{
	bo_status_t status;
	struct aorp_comparable_idx ___key = {
		AorpK_HINT_INDEX, 0, (anIndex)
	};
	status = AorpCtnrFind(aThis, &___key, aCursor, anErrPtr);
	return (status);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * CURSOR ACCESS <BY NAME>
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrFindbyname(
	aorp_object_t aThis,
	char const *aName,
	struct aorp_qcursor *aCursor, // cursor should be opened
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrFindbyname(aThis, aName, aCursor, anErrPtr) \
	AorpCtnrFind(aThis, \
		(&(struct aorp_comparable_str){ AorpK_HINT_STR, 0, (aName) }), \
		aCursor, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrFindbyname(aThis, aName, aCursor, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_str ___key = { \
			AorpK_HINT_STR, 0, (aName) \
		}; \
		/* return */ AorpCtnrFind(aThis, &___key, aCursor, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrFindbyname(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	char const *aName,
	struct aorp_qcursor *aCursor, /* cursor should be opened */
	struct aorp_error *anErrPtr
# else
	aThis, aName, aCursor, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	char const *aName;
	struct aorp_qcursor *aCursor; /* cursor should be opened */
	struct aorp_error *anErrPtr;
# endif
{
	bo_status_t status;
	struct aorp_comparable_str ___key = {
		AorpK_HINT_STR, 0, (aName)
	};
	status = AorpCtnrFind(aThis, &___key, aCursor, anErrPtr);
	return (status);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * ALGORITHMS: INSERT
 *
 * The inserted value in all these algorithms is retained, if possible.
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2,3)))
AorpCtnrInsert(
	aorp_object_t aThis,
	AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrInsert(aThis, aKey, aValue, anErrPtr) \
	((bo_index_t) _A_call(insert, container, aThis, anErrPtr, aKey, aValue))

/*
 * Special variants
 * @{
 */
# define AorpCtnrInsert_tmpl(aHint, _TField, aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrInsert(aThis, aKey, &___value, anErrPtr); \
	} while (0)

# define AorpCtnrInsert_ptr(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	AorpCtnrInsert_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, aKey, aValue, anErrPtr)

# define AorpCtnrInsert_obj(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	(aStatusEqu AorpCtnrInsert(aThis, aKey, aValue, anErrPtr))

# define AorpCtnrInsert_str(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	AorpCtnrInsert_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, aKey, aValue, anErrPtr)

# define AorpCtnrInsert_wcs(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	AorpCtnrInsert_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, aKey, aValue, anErrPtr)

# define AorpCtnrInsert_i32(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	AorpCtnrInsert_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, aKey, aValue, anErrPtr)

# define AorpCtnrInsert_i64(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	AorpCtnrInsert_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, aKey, aValue, anErrPtr)

# define AorpCtnrInsert_f32(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	AorpCtnrInsert_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, aKey, aValue, anErrPtr)

# define AorpCtnrInsert_f64(aStatusEqu, aThis, aKey, aValue, anErrPtr) \
	AorpCtnrInsert_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, aKey, aValue, anErrPtr)
/* @} */

/*
 * ALGORITHMS: INSERT <BY INDEX>
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrPush(
	aorp_object_t aThis,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrPush(aThis, aValue, anErrPtr) \
	AorpCtnrInsbyindex(aThis, CtnrK_PUSH_DEFAULT_INDEX, aValue, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrPush_tmpl(aHint, _TField, aStatusEqu, aThis, aValue, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrPush(aThis, &___value, anErrPtr); \
	} while (0)

# define AorpCtnrPush_ptr(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPush_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPush_obj(aStatusEqu, aThis, aValue, anErrPtr) \
	(aStatusEqu AorpCtnrPush(aThis, aValue, anErrPtr))

# define AorpCtnrPush_str(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPush_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPush_wcs(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPush_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPush_i32(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPush_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPush_i64(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPush_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPush_f32(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPush_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPush_f64(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPush_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, aValue, anErrPtr)
/* @} */

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrPushfront(
	aorp_object_t aThis,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrPushfront(aThis, aValue, anErrPtr) \
	AorpCtnrInsbyindex(aThis, CtnrK_BEGIN_INDEX, aValue, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrPushfront_tmpl(aHint, _TField, aStatusEqu, aThis, aValue, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrPushfront(aThis, &___value, anErrPtr); \
	} while (0)

# define AorpCtnrPushfront_ptr(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushfront_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushfront_obj(aStatusEqu, aThis, aValue, anErrPtr) \
	(aStatusEqu AorpCtnrPushfront(aThis, aValue, anErrPtr))

# define AorpCtnrPushfront_str(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushfront_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushfront_wcs(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushfront_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushfront_i32(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushfront_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushfront_i64(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushfront_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushfront_f32(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushfront_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushfront_f64(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushfront_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, aValue, anErrPtr)
/* @} */

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrPushback(
	aorp_object_t aThis,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrPushback(aThis, aValue, anErrPtr) \
	AorpCtnrInsbyindex(aThis, CtnrK_END_INDEX, aValue, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrPushback_tmpl(aHint, _TField, aStatusEqu, aThis, aValue, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrPushback(aThis, &___value, anErrPtr); \
	} while (0)

# define AorpCtnrPushback_ptr(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushback_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushback_obj(aStatusEqu, aThis, aValue, anErrPtr) \
	(aStatusEqu AorpCtnrPushback(aThis, aValue, anErrPtr))

# define AorpCtnrPushback_str(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushback_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushback_wcs(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushback_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushback_i32(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushback_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushback_i64(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushback_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushback_f32(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushback_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, aValue, anErrPtr)

# define AorpCtnrPushback_f64(aStatusEqu, aThis, aValue, anErrPtr) \
	AorpCtnrPushback_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, aValue, anErrPtr)
/* @} */

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,3)))
AorpCtnrInsbyindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrInsbyindex(aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsert(aThis, \
		(&(struct aorp_comparable_idx){ AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) }), \
		aValue, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrInsbyindex(aThis, anIndex, aValue, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_idx ___key = { \
			AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) \
		}; \
		/* return */ AorpCtnrInsert(aThis, &___key, aValue, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_index_t
__attribute__((nonnull(1,3)))
AorpCtnrInsbyindex(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	bo_index_t anIndex,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
	struct aorp_error *anErrPtr
# else
	aThis, anIndex, aValue, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	bo_index_t anIndex;
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue;
	struct aorp_error *anErrPtr;
# endif
{
	bo_index_t retval;
	struct aorp_comparable_idx ___key = {
		AorpK_HINT_INDEX, 0, (anIndex)
	};
	retval = AorpCtnrInsert(aThis, &___key, aValue, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrInsbyindex_tmpl(aHint, _TField, aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrInsbyindex(aThis, anIndex, &___value, anErrPtr); \
	} while (0)

# define AorpCtnrInsbyindex_ptr(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsbyindex_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, anIndex, aValue, anErrPtr)

# define AorpCtnrInsbyindex_obj(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	(aStatusEqu AorpCtnrInsbyindex(aThis, anIndex, aValue, anErrPtr))

# define AorpCtnrInsbyindex_str(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsbyindex_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, anIndex, aValue, anErrPtr)

# define AorpCtnrInsbyindex_wcs(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsbyindex_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, anIndex, aValue, anErrPtr)

# define AorpCtnrInsbyindex_i32(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsbyindex_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, anIndex, aValue, anErrPtr)

# define AorpCtnrInsbyindex_i64(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsbyindex_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, anIndex, aValue, anErrPtr)

# define AorpCtnrInsbyindex_f32(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsbyindex_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, anIndex, aValue, anErrPtr)

# define AorpCtnrInsbyindex_f64(aStatusEqu, aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrInsbyindex_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, anIndex, aValue, anErrPtr)
/* @} */

/*
 * ALGORITHMS: INSERT <BY NAME>
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2,3)))
AorpCtnrInsbyname(
	aorp_object_t aThis,
	char const *aName,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrInsbyname(aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsert(aThis, \
		(&(struct aorp_comparable_str){ AorpK_HINT_INDEX, 0, (aName) }), \
		aValue, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrInsbyname(aThis, aName, aValue, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_str ___key = { \
			AorpK_HINT_STR, 0, (aName) \
		}; \
		/* return */ AorpCtnrInsert(aThis, &___key, aValue, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_index_t
__attribute__((nonnull(1,2,3)))
AorpCtnrInsbyname(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	char const *aName,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
	struct aorp_error *anErrPtr
# else
	aThis, aName, aValue, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	char const *aName;
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue;
	struct aorp_error *anErrPtr;
# endif
{
	bo_index_t retval;
	struct aorp_comparable_str ___key = {
		AorpK_HINT_STR, 0, (aName)
	};
	retval = AorpCtnrInsert(aThis, &___key, aValue, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrInsbyname_tmpl(aHint, _TField, aStatusEqu, aThis, aName, aValue, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrInsbyname(aThis, aName, &___value, anErrPtr); \
	} while (0)

# define AorpCtnrInsbyname_ptr(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsbyname_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, aName, aValue, anErrPtr)

# define AorpCtnrInsbyname_obj(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	(aStatusEqu AorpCtnrInsbyname(aThis, aName, aValue, anErrPtr))

# define AorpCtnrInsbyname_str(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsbyname_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, aName, aValue, anErrPtr)

# define AorpCtnrInsbyname_wcs(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsbyname_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, aName, aValue, anErrPtr)

# define AorpCtnrInsbyname_i32(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsbyname_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, aName, aValue, anErrPtr)

# define AorpCtnrInsbyname_i64(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsbyname_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, aName, aValue, anErrPtr)

# define AorpCtnrInsbyname_f32(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsbyname_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, aName, aValue, anErrPtr)

# define AorpCtnrInsbyname_f64(aStatusEqu, aThis, aName, aValue, anErrPtr) \
	AorpCtnrInsbyname_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, aName, aValue, anErrPtr)
/* @} */

/*
 * ALGORITHMS: SET
 *
 * The inserted value in all these algorithms is retained, if possible.
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrSet(
	aorp_object_t aThis,
	AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
	int aCond,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrSet(aThis, aKey, aValue, aCond, anErrPtr) \
	_A_call(set, container, aThis, anErrPtr, aKey, aValue, aCond)

/*
 * Special variants
 * @{
 */
# define AorpCtnrSet_tmpl(aHint, _TField, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrSet(aThis, aKey, &___value, aCond, anErrPtr); \
	} while (0)

# define AorpCtnrSet_ptr(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	AorpCtnrSet_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr)

# define AorpCtnrSet_obj(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	(aStatusEqu AorpCtnrSet(aThis, aKey, aValue, aCond, anErrPtr))

# define AorpCtnrSet_str(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	AorpCtnrSet_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr)

# define AorpCtnrSet_wcs(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	AorpCtnrSet_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr)

# define AorpCtnrSet_i32(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	AorpCtnrSet_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr)

# define AorpCtnrSet_i64(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	AorpCtnrSet_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr)

# define AorpCtnrSet_f32(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	AorpCtnrSet_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr)

# define AorpCtnrSet_f64(aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr) \
	AorpCtnrSet_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, aKey, aValue, aCond, anErrPtr)
/* @} */

/*
 * ALGORITHMS: SET <BY INDEX>
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpCtnrSetbyindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
	int aCond,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrSetbyindex(aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSet(aThis, \
		(&(struct aorp_comparable_idx){ AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) }), \
		aValue, aCond, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrSetbyindex(aThis, anIndex, aValue, aCond, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_idx ___key = { \
			AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) \
		}; \
		/* return */ AorpCtnrSet(aThis, &___key, aValue, aCond, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_status_t
__attribute__((nonnull(1,3)))
AorpCtnrSetbyindex(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	bo_index_t anIndex,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
	int aCond,
	struct aorp_error *anErrPtr
# else
	aThis, anIndex, aValue, aCond, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	bo_index_t anIndex;
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue;
	int aCond;
	struct aorp_error *anErrPtr;
# endif
{
	bo_status_t retval;
	struct aorp_comparable_idx ___key = {
		AorpK_HINT_INDEX, 0, (anIndex)
	};
	retval = AorpCtnrSet(aThis, &___key, aValue, aCond, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrSetbyindex_tmpl(aHint, _TField, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrSetbyindex(aThis, anIndex, &___value, aCond, anErrPtr); \
	} while (0)

# define AorpCtnrSetbyindex_ptr(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyindex_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyindex_obj(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	(aStatusEqu AorpCtnrSetbyindex(aThis, anIndex, aValue, aCond, anErrPtr))

# define AorpCtnrSetbyindex_str(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyindex_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyindex_wcs(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyindex_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyindex_i32(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyindex_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyindex_i64(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyindex_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyindex_f32(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyindex_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyindex_f64(aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyindex_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, anIndex, aValue, aCond, anErrPtr)
/* @} */

/*
 * ALGORITHMS: SET <BY NAME>
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrSetbyname(
	aorp_object_t aThis,
	char const *aName,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
	int aCond,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrSetbyname(aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSet(aThis, \
		(&(struct aorp_comparable_str){ AorpK_HINT_INDEX, 0, (aName) }), \
		aValue, aCond, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrSetbyname(aThis, aName, aValue, aCond, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_str ___key = { \
			AorpK_HINT_STR, 0, (aName) \
		}; \
		/* return */ AorpCtnrSet(aThis, &___key, aValue, aCond, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_status_t
__attribute__((nonnull(1,2,3)))
AorpCtnrSetbyname(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	char const *aName,
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
	int aCond,
	struct aorp_error *anErrPtr
# else
	aThis, aName, aValue, aCond, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	char const *aName;
	AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue;
	int aCond;
	struct aorp_error *anErrPtr;
# endif
{
	bo_status_t retval;
	struct aorp_comparable_str ___key = {
		AorpK_HINT_STR, 0, (aName)
	};
	retval = AorpCtnrSet(aThis, &___key, aValue, aCond, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrSetbyname_tmpl(aHint, _TField, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	do { \
		struct aorp_udvariant ___value = { \
			/* .udc_hint = */ aHint, \
			/* .udc_reserved = */ 0 \
		}; \
		___value.u._TField = (aValue); \
		aStatusEqu AorpCtnrSetbyname(aThis, aName, &___value, aCond, anErrPtr); \
	} while (0)

# define AorpCtnrSetbyname_ptr(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyname_tmpl(AorpK_HINT_POINTER, udc_udata, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyname_obj(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	(aStatusEqu AorpCtnrSetbyname(aThis, aName, aValue, aCond, anErrPtr))

# define AorpCtnrSetbyname_str(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyname_tmpl(AorpK_HINT_STR, udc_str, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyname_wcs(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyname_tmpl(AorpK_HINT_WCS, udc_wcs, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyname_i32(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyname_tmpl(AorpK_HINT_INT32, udc_i32, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyname_i64(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyname_tmpl(AorpK_HINT_INT64, udc_i64, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyname_f32(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyname_tmpl(AorpK_HINT_FLOAT32, udc_f32, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr)

# define AorpCtnrSetbyname_f64(aStatusEqu, aThis, aName, aValue, aCond, anErrPtr) \
	AorpCtnrSetbyname_tmpl(AorpK_HINT_FLOAT64, udc_f64, aStatusEqu, aThis, aName, aValue, aCond, anErrPtr)
/* @} */

/*
 * ALGORITHMS: REMOVE
 *
 * The removed value is not released.
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2,3)))
AorpCtnrRemove(
	aorp_object_t aThis,
	AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrRemove(aThis, aKey, aValue, anErrPtr) \
	((bo_index_t) _A_call(remove, container, aThis, anErrPtr, aKey, aValue))

/*
 * Special variants
 * @{
 */
# define AorpCtnrRemove_tmpl(_TValueType, _TField, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	do { \
		bo_index_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrRemove(aThis, aKey, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrRemove_ptr(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrRemove_obj(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrRemove_str(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(char*, udc_str, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrRemove_wcs(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrRemove_i32(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrRemove_i64(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrRemove_f32(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(float, udc_f32, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)

# define AorpCtnrRemove_f64(aStatusEqu, aThis, aKey, aValuePtr, anErrPtr) \
	AorpCtnrRemove_tmpl(double, udc_f64, aStatusEqu, aThis, aKey, aValuePtr, anErrPtr)
/* @} */

/*
 * ALGORITHMS: REMOVE <BY INDEX>
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrPop(
	aorp_object_t aThis,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrPop(aThis, aValue, anErrPtr) \
	AorpCtnrRmbyindex(aThis, CtnrK_POP_DEFAULT_INDEX, aValue, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrPop_tmpl(_TValueType, _TField, aStatusEqu, aThis, aValuePtr, anErrPtr) \
	do { \
		bo_index_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrPop(aThis, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrPop_ptr(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPop_obj(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPop_str(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(char*, udc_str, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPop_wcs(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPop_i32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPop_i64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPop_f32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(float, udc_f32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPop_f64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPop_tmpl(double, udc_f64, aStatusEqu, aThis, aValuePtr, anErrPtr)
/* @} */

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrPopfront(
	aorp_object_t aThis,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrPopfront(aThis, aValue, anErrPtr) \
	AorpCtnrRmbyindex(aThis, CtnrK_FRONT_INDEX, aValue, anErrPtr)

# define AorpCtnrPopdelfront(aThis, anErrPtr) \
	AorpCtnrDelbyindex(aThis, CtnrK_FRONT_INDEX, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrPopfront_tmpl(_TValueType, _TField, aStatusEqu, aThis, aValuePtr, anErrPtr) \
	do { \
		bo_index_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrPopfront(aThis, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrPopfront_ptr(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopfront_obj(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopfront_str(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(char*, udc_str, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopfront_wcs(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopfront_i32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopfront_i64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopfront_f32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(float, udc_f32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopfront_f64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopfront_tmpl(double, udc_f64, aStatusEqu, aThis, aValuePtr, anErrPtr)
/* @} */

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrPopback(
	aorp_object_t aThis,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrPopback(aThis, aValue, anErrPtr) \
	AorpCtnrRmbyindex(aThis, CtnrK_BACK_INDEX, aValue, anErrPtr)

# define AorpCtnrPopdelback(aThis, anErrPtr) \
	AorpCtnrDelbyindex(aThis, CtnrK_BACK_INDEX, anErrPtr)

/*
 * Special variants
 * @{
 */
# define AorpCtnrPopback_tmpl(_TValueType, _TField, aStatusEqu, aThis, aValuePtr, anErrPtr) \
	do { \
		bo_index_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrPopback(aThis, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrPopback_ptr(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopback_obj(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopback_str(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(char*, udc_str, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopback_wcs(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopback_i32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopback_i64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopback_f32(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(float, udc_f32, aStatusEqu, aThis, aValuePtr, anErrPtr)

# define AorpCtnrPopback_f64(aStatusEqu, aThis, aValuePtr, anErrPtr) \
	AorpCtnrPopback_tmpl(double, udc_f64, aStatusEqu, aThis, aValuePtr, anErrPtr)
/* @} */

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,3)))
AorpCtnrRmbyindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrRmbyindex(aThis, anIndex, aValue, anErrPtr) \
	AorpCtnrRemove(aThis, \
		(&(struct aorp_comparable_idx){ AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) }), \
		aValue, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrRmbyindex(aThis, anIndex, aValue, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_idx ___key = { \
			AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) \
		}; \
		/* return */ AorpCtnrRemove(aThis, &___key, aValue, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_index_t
__attribute__((nonnull(1,3)))
AorpCtnrRmbyindex(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
# else
	aThis, anIndex, aValue, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	bo_index_t anIndex;
	struct aorp_udvariant *aValue;
	struct aorp_error *anErrPtr;
# endif
{
	bo_index_t retval;
	struct aorp_comparable_idx ___key = {
		AorpK_HINT_INDEX, 0, (anIndex)
	};
	retval = AorpCtnrRemove(aThis, &___key, aValue, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrRmbyindex_tmpl(_TValueType, _TField, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	do { \
		bo_index_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrRmbyindex(aThis, anIndex, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrRmbyindex_ptr(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrRmbyindex_obj(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrRmbyindex_str(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(char*, udc_str, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrRmbyindex_wcs(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrRmbyindex_i32(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(int32_t, udc_i32, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrRmbyindex_i64(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(int64_t, udc_i64, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrRmbyindex_f32(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(float, udc_f32, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)

# define AorpCtnrRmbyindex_f64(aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr) \
	AorpCtnrRmbyindex_tmpl(double, udc_f64, aStatusEqu, aThis, anIndex, aValuePtr, anErrPtr)
/* @} */

/*
 * ALGORITHMS: REMOVE <BY NAME>
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2,3)))
AorpCtnrRmbyname(
	aorp_object_t aThis,
	char const *aName,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrRmbyname(aThis, aName, aValue, anErrPtr) \
	AorpCtnrRemove(aThis, \
		(&(struct aorp_comparable_str){ AorpK_HINT_STR, 0, (aName) }), \
		aValue, anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrRmbyname(aThis, aName, aValue, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_str ___key = { \
			AorpK_HINT_STR, 0, (aName) \
		}; \
		/* return */ AorpCtnrRemove(aThis, &___key, aValue, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_index_t
__attribute__((nonnull(1,2,3)))
AorpCtnrRmbyname(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	char const *aName,
	struct aorp_udvariant *aValue,
	struct aorp_error *anErrPtr
# else
	aThis, aName, aValue, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	char const *aName;
	struct aorp_udvariant *aValue;
	struct aorp_error *anErrPtr;
# endif
{
	bo_index_t retval;
	struct aorp_comparable_str ___key = {
		AorpK_HINT_STR, 0, (aName)
	};
	retval = AorpCtnrRemove(aThis, &___key, aValue, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * Special variants
 * @{
 */
# define AorpCtnrRmbyname_tmpl(_TValueType, _TField, aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	do { \
		bo_index_t ___status; \
		struct aorp_udvariant ___value; \
		___status = AorpCtnrRmbyname(aThis, aName, &___value, anErrPtr); \
		if (BoS_SUCCESS(___status)) \
			*((_TValueType*)(aValuePtr)) = ___value.u._TField; \
		aStatusEqu ___status; \
	} while (0)

# define AorpCtnrRmbyname_ptr(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(bo_pointer_t, udc_udata, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrRmbyname_obj(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(aorp_object_t, udc_obj, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrRmbyname_str(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(char*, udc_str, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrRmbyname_wcs(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(wchar_t*, udc_wcs, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrRmbyname_i32(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(int32_t, udc_i32, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrRmbyname_i64(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(int64_t, udc_i64, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrRmbyname_f32(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(float, udc_f32, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)

# define AorpCtnrRmbyname_f64(aStatusEqu, aThis, aName, aValuePtr, anErrPtr) \
	AorpCtnrRmbyname_tmpl(double, udc_f64, aStatusEqu, aThis, aName, aValuePtr, anErrPtr)
/* @} */

/*
 * ALGORITHMS: DELETE
 *
 * The removed value is released, if possible.
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrDelete(
	aorp_object_t aThis,
	AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrDelete(aThis, aKey, anErrPtr) \
	((bo_index_t) _A_call(delete, container, aThis, anErrPtr, aKey))

/*
 * ALGORITHMS: DELETE <BY INDEX>
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpCtnrDelbyindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrDelbyindex(aThis, anIndex, anErrPtr) \
	AorpCtnrDelete(aThis, \
		(&(struct aorp_comparable_idx){ AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) }), \
		anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrDelbyindex(aThis, anIndex, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_idx ___key = { \
			AorpK_HINT_INDEX, 0, (bo_index_t)(anIndex) \
		}; \
		/* return */ AorpCtnrDelete(aThis, &___key, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_index_t
__attribute__((nonnull(1)))
AorpCtnrDelbyindex(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_error *anErrPtr
# else
	aThis, anIndex, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	bo_index_t anIndex;
	struct aorp_error *anErrPtr;
# endif
{
	bo_index_t retval;
	struct aorp_comparable_idx ___key = {
		AorpK_HINT_INDEX, 0, (anIndex)
	};
	retval = AorpCtnrDelete(aThis, &___key, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * ALGORITHMS: DELETE <BY NAME>
 */
# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrDelbyname(
	aorp_object_t aThis,
	char const *aName,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(TUNO)
#  define AorpCtnrDelbyname(aThis, aName, anErrPtr) \
	AorpCtnrDelete(aThis, \
		(&(struct aorp_comparable_str){ AorpK_HINT_STR, 0, (aName) }), \
		anErrPtr)
# else
# if __myc_support(macro_retval)
#  define AorpCtnrDelbyname(aThis, aName, anErrPtr) \
	__extension__ ({ \
		struct aorp_comparable_str ___key = { \
			AorpK_HINT_STR, 0, (aName) \
		}; \
		/* return */ AorpCtnrDelete(aThis, &___key, anErrPtr); \
	})
# else
__myc_extern_inline__ bo_index_t
__attribute__((nonnull(1,2)))
AorpCtnrDelbyname(
# if __myc_require(prototypes)
	aorp_object_t aThis,
	char const *aName,
	struct aorp_error *anErrPtr
# else
	aThis, aName, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aThis;
	char const *aName;
	struct aorp_error *anErrPtr;
# endif
{
	bo_index_t retval;
	struct aorp_comparable_str ___key = {
		AorpK_HINT_STR, 0, (aName)
	};
	retval = AorpCtnrDelete(aThis, &___key, anErrPtr);
	return (retval);
}
# endif /* __myc_support(macro_retval) */
# endif /* __myc_support(TUNO) */

/*
 * ALGORITHMS: FOREACH
 */
# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCtnrForeach(
	aorp_object_t aThis,
	struct aorp_foreach_pred *aPred,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrForeach(aThis, aPred, anErrPtr) \
	_A_call(foreach, container, aThis, anErrPtr, aPred)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCtnrForeach_nlk(
	aorp_object_t aThis,
	struct aorp_foreach_pred *aPred,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrForeach_nlk(aThis, aPred, anErrPtr) \
	_A_call(foreach_nlk, container, aThis, anErrPtr, aPred)

# pragma mark -

/*															//	@note: MaxL
bo_index_t
__attribute__((nonnull(1)))
AorpCtnrCount_nlk(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCtnrCount_nlk(aThis, anErrPtr) \
	((bo_index_t) _A_call(count_nlk, container, aThis, anErrPtr))

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_BASIS_CTNR__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
