/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * CtnrP.h	(V.Sokur)
 * include/Bo/services/basis/CtnrP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_BASIS_CTNR_P__
# define __BO_SERVICES_BASIS_CTNR_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/basis/Ctnrtypes.h>

/******************************************************************************
 * %% BeginSection: container service declarations
 */
# pragma mark -
# pragma mark [ container service declarations ]
# pragma mark -

# define __aorpc_need_service_container 1

#endif /* !__BO_SERVICES_BASIS_CTNR_P__ */

#pragma mark -

#if __aorpc_uses_service(container)
# undef __aorpc_need_service_container

/*
 * container service
 * @{
 */

/*
 * container {\object(instance)}
 */
/*
# define AorpK_container_service (1050)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name container
# include <Bo/aorp/AorpCsrvc.h>

_A_service(container)

# if __aorpc_interface

/*
 * cursor
 */

# ifndef AorpK_QCURSOR_OPAQUE_SIZE
#  if BO_M_WORD == BO_M_WORD32
#   define AorpK_QCURSOR_OPAQUE_SIZE (40)
#  else
#   define AorpK_QCURSOR_OPAQUE_SIZE (90)
#  endif
# endif

	struct aorp_qcursor {
		/* struct aorp_object_s @{ */
		aorp_hint_x            qc_hint;     /* should be AorpK_HINT_CURSOR */
		u_int16_t              qc_reserved; /* should be 0 */
		/* @} */
		char                   qc_closure[AorpK_CLOSURE_SIZE];
		char                   qc_opaque[AorpK_QCURSOR_OPAQUE_SIZE];
	};
# define aorp_qcursor_p(x) ((struct aorp_qcursor *)(x))

/* ------------------------------------------------------------------------- */

# define AorpK_YQCURSOR_OPAQUE_SIZE \
	(AorpK_QCURSOR_OPAQUE_SIZE - sizeof(struct aorp_yqcursor_assoc_private))

	struct aorp_yqcursor_assoc_private {
		aorp_hint_t            yc_assoc_hint;
		intptr_t               yc_key;
		bo_hash_t              yc_hash;
		bo_index_t             yc_idx;
	};

	struct aorp_yqcursor {
		/* struct aorp_object_s @{ */
		aorp_hint_x            qc_hint;     /* should be AorpK_HINT_CURSOR */
		u_int16_t              qc_reserved; /* should be 0 */
		/* @} */
		char                   qc_closure[AorpK_CLOSURE_SIZE];
		char                   qc_opaque[AorpK_YQCURSOR_OPAQUE_SIZE];

		/* struct aorp_yqcursor_assoc_private @{ */
		aorp_hint_t            yc_assoc_hint;
		intptr_t               yc_key;
		bo_hash_t              yc_hash;
		bo_index_t             yc_idx;
		/* @} */
	};
# define aorp_yqcursor_p(x) ((struct aorp_yqcursor *)(x))

/* ------------------------------------------------------------------------- */

# define AorpK_OQCURSOR_OPAQUE_SIZE \
	(AorpK_QCURSOR_OPAQUE_SIZE - sizeof(struct aorp_oqcursor_assoc_private))

	struct aorp_oqcursor_assoc_private {
		aorp_hint_t            oc_assoc_hint;
		union {
			intptr_t           oc_key;
			int64_t            oc_key64;
		} u;
	};

	struct aorp_oqcursor {
		/* struct aorp_object_s @{ */
		aorp_hint_x            qc_hint;     /* should be AorpK_HINT_CURSOR */
		u_int16_t              qc_reserved; /* should be 0 */
		/* @} */
		char                   qc_closure[AorpK_CLOSURE_SIZE];
		char                   qc_opaque[AorpK_OQCURSOR_OPAQUE_SIZE];

		/* struct aorp_oqcursor_assoc_private @{ */
		aorp_hint_t            oc_assoc_hint;
		union {
			intptr_t           oc_key;
			int64_t            oc_key64;
		} u;
		/* @} */
	};
# define aorp_oqcursor_p(x) ((struct aorp_oqcursor *)(x))

/* ------------------------------------------------------------------------- */

# define AORP_QCURSOR_Closure(aCursor) \
	((struct aorp_closure *)aorp_qcursor_p(aCursor)->qc_closure)

/*
 * Reserved indices
 */

/* Used for 'insert' operations */
# define CtnrK_BEGIN_INDEX (BoL_INDEX_MAX - 3)
# define CtnrK_END_INDEX (BoL_INDEX_MAX - 2)
# define CtnrK_PUSH_DEFAULT_INDEX (BoL_INDEX_MAX - 1)

/* Used for 'remove' operations */
# define CtnrK_POP_DEFAULT_INDEX BoL_INDEX_MAX

/* Used for all operations */
# define CtnrK_FRONT_INDEX ((bo_index_t)0)
# define CtnrK_BACK_INDEX ((bo_index_t)-1)

# define CtnrK_INVAL_INDEX BoL_INDEX_MIN

/*
 * Constants for condition (operation - set)
 */

# define CtnrK_UPDATE_COND (0) /* default */
# define CtnrK_EXCL_COND (1)
# define CtnrK_EXIST_COND (2)
# define CtnrK_INSERT_COND (3)

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(count, "container::get-size", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(maxcount, "container::get-max-size", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(clear, "container::clear", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(opencrsr, "container::open-cursor", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_qcursor *aCursor
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(closecrsr, "container::close-cursor", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_qcursor *aCursor
		)),

/* BoS_ERR: error;
   BoS_NORMAL: success, end of container;
   BoS_OK: success */
	_A_attribute((nonnull(1,3)))
	_A_op(advancecrsr, "container::advance-cursor", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_qcursor *aCursor,
			bo_index_t aDistance
		)),

/* The function returns an actual count of values.
   The returned values are retained, if possible. */
	_A_attribute((nonnull(1,3)))
	_A_op(getbycrsr, "container::get-values-by-cursor", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_qcursor *aCursor,
			struct aorp_udvariant aValuev[],
			size_t aValuec,
			int WithRetain
		)),

	_A_attribute((nonnull(1)))
	_A_op(rdlock, "container::read-lock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(wrlock, "container::write-lock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(unlock, "container::unlock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

/* The returned value is not retained. */
	_A_attribute((nonnull(1,3,4)))
	_A_op(at, "container::get-value-by-key", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t)) aKey,
			struct aorp_udvariant *aValue
		)),

/* The returned value is not retained. */
	_A_attribute((nonnull(1,3,4)))
	_A_op(at_nlk, "container::get-value-by-key-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t)) aKey,
			struct aorp_udvariant *aValue
		)),

	_A_attribute((nonnull(1,3,4)))
	_A_op(find, "container::find-position-by-key", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t)) aKey,
			struct aorp_qcursor *aCursor /* cursor should be opened */
		)),

	_A_attribute((nonnull(1,3,4)))
	_A_op(find_nlk, "container::find-position-by-key-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t)) aKey,
			struct aorp_qcursor *aCursor /* cursor should be opened */
		)),

/* The inserted value is retained, if possible. */
	_A_attribute((nonnull(1,3,4)))
	_A_op(insert, "container::insert-value-by-key", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
			AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
		)),

	_A_attribute((nonnull(1,3,4)))
	_A_op(insert_nlk, "container::insert-value-by-key-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
			AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue
		)),

/* The removed value is not released. */
	_A_attribute((nonnull(1,3,4)))
	_A_op(remove, "container::remove-value-by-key", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
			struct aorp_udvariant *aValue
		)),

	_A_attribute((nonnull(1,3,4)))
	_A_op(remove_nlk, "container::remove-value-by-key-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
			struct aorp_udvariant *aValue
		)),

/* The removed value is released, if possible. */
	_A_attribute((nonnull(1,3)))
	_A_op(delete, "container::delete-value-by-key", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(delete_nlk, "container::delete-value-by-key-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(foreach, "container::foreach-element-apply-predicate", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_foreach_pred *aPred
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(foreach_nlk, "container::foreach-element-apply-predicate-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_foreach_pred *aPred
		)),

/* The inserted value is retained, if possible. */
	_A_attribute((nonnull(1,3,4)))
	_A_op(set, "container::set-value-by-key", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
			AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
			int aCond
		)),

	_A_attribute((nonnull(1,3,4)))
	_A_op(set_nlk, "container::set-value-by-key-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((aorp_comparable_t, aorp_object_t, struct aorp_qcursor *)) aKey,
			AORP_ONEP_OF((struct aorp_udvariant *, aorp_object_t)) aValue,
			int aCond
		)),

	_A_attribute((nonnull(1)))									//	@note: MaxL
	_A_op(count_nlk, "container::get-size-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(container) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
