/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * CHash.h	(V.Sokur)
 * include/Bo/services/crypt/CHash.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CRYPT_CHASH__
# define __BO_SERVICES_CRYPT_CHASH__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/crypt/Crypttypes.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# include <Bo/services/crypt/CHashop.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpOpenCHashal(
	aorp_object_t aThis,
	aorp_calgid_t anAlgid,
	struct aorp_chashal *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpOpenCHashal(aThis, anAlgid, anAlg, anErrPtr) \
	_A_call(openalg, chash, aThis, anErrPtr, (aorp_calgid_t)(anAlgid), anAlg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpCHashClose(
	struct aorp_chashal *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCHashClose(anAlg, anErrPtr) \
	_A_closcall(closealg, chash, (anAlg)->ca_opaque1, anErrPtr, anAlg)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpCHashBlocksize(
	struct aorp_chashal const *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCHashBlocksize(anAlg, anErrPtr) \
	_A_closcall(block_size, chashal, (anAlg)->ca_opaque2, anErrPtr, anAlg)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpCHashLen(
	struct aorp_chashal const *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCHashLen(anAlg, anErrPtr) \
	_A_closcall(getlen, chashal, (anAlg)->ca_opaque2, anErrPtr, anAlg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpCHashInit(
	struct aorp_chashal *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCHashInit(anAlg, anErrPtr) \
	_A_closcall(init, chashal, (anAlg)->ca_opaque2, anErrPtr, anAlg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpCHashUpdate(
	struct aorp_chashal *anAlg,
	unsigned aBufc,                  // = -1, ==> NULL terminated list @bufq
	struct bo_qabsbuf const *aBufq,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCHashUpdate(anAlg, aBufc, aBufq, anErrPtr) \
	_A_closcall(update, chashal, (anAlg)->ca_opaque2, anErrPtr, anAlg, aBufc, aBufq)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCHashFinal(
	struct aorp_chashal *anAlg,
	struct bo_databuf *aHashPtr,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCHashFinal(anAlg, aHashPtr, anErrPtr) \
	_A_closcall(final, chashal, (anAlg)->ca_opaque2, anErrPtr, anAlg, aHashPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,4)))
AorpCHash(
	struct aorp_chashal *anAlg,
	struct bo_databuf *aHashPtr,
	unsigned aBufc,                  // = -1, ==> NULL terminated array @bufq
	struct bo_qabsbuf const *aBufq,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCHash(anAlg, aHashPtr, aBufc, aBufq, anErrPtr) \
	_A_closcall(perform, chashal, (anAlg)->ca_opaque2, anErrPtr, anAlg, aHashPtr, aBufc, aBufq)

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_CRYPT_CHASH__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
