/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * CHashP.h	(V.Sokur)
 * include/Bo/services/crypt/CHashP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CRYPT_CHASH_P__
# define __BO_SERVICES_CRYPT_CHASH_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/crypt/Crypttypes.h>

/******************************************************************************
 * %% BeginSection: chash service declarations
 */
# pragma mark -
# pragma mark [ chash service declarations ]
# pragma mark -

/*
 * chash service
 * @{
 */

# include <Bo/services/crypt/CHashop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name chash
# include <Bo/aorp/AorpC.h>

_A_service(chash)

# define AORP_chash_init_once AORP_any_init_once
# define AORP_chash_init_once_signature AORP_any_init_once_signature
# define AORP_chash_fini_once AORP_any_fini_once
# define AORP_chash_fini_once_signature AORP_any_fini_once_signature
# define AORP_chash_pod_init AORP_any_pod_init
# define AORP_chash_pod_init_signature AORP_any_pod_init_signature
# define AORP_chash_pod_fini AORP_any_pod_fini
# define AORP_chash_pod_fini_signature AORP_any_pod_fini_signature

_A_op(openalg, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		aorp_calgid_t anAlgid,
		struct aorp_chashal *anAlg
	)) __attribute__((nonnull(1,4)));

# define AORP_chash_openalg "open-algorithm"
# define AORP_chash_openalg_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_POINTER

_A_op(closealg, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_chashal *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_chash_closealg "close-algorithm"
# define AORP_chash_closealg_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

# pragma mark -

/*
 * chashal service
 * @{
 */

# include <Bo/services/crypt/CHashop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name chashal
# include <Bo/aorp/AorpC.h>

_A_service(chashal)

# define AORP_chashal_init_once AORP_any_init_once
# define AORP_chashal_init_once_signature AORP_any_init_once_signature
# define AORP_chashal_fini_once AORP_any_fini_once
# define AORP_chashal_fini_once_signature AORP_any_fini_once_signature
# define AORP_chashal_pod_init AORP_any_pod_init
# define AORP_chashal_pod_init_signature AORP_any_pod_init_signature
# define AORP_chashal_pod_fini AORP_any_pod_fini
# define AORP_chashal_pod_fini_signature AORP_any_pod_fini_signature

_A_op(block_size, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_chashal const *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_chashal_block_size "block_size"
# define AORP_chashal_block_size_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(getlen, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_chashal const *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_chashal_getlen "getlen"
# define AORP_chashal_getlen_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(init, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_chashal *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_chashal_init "init"
# define AORP_chashal_init_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(update, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_chashal *anAlg,
		unsigned aBufc,
		struct bo_qabsbuf const *aBufq
	)) __attribute__((nonnull(1,3,5)));

# define AORP_chashal_update "update"
# define AORP_chashal_update_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_INT _AORP_SMOD_UNSIGNED \
	" " _AORP_SIGN_POINTER

_A_op(final, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_chashal *anAlg,
		struct bo_databuf *aHashPtr
	)) __attribute__((nonnull(1,3,4)));

# define AORP_chashal_final "final"
# define AORP_chashal_final_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

_A_op(perform, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_chashal *anAlg,
		struct bo_databuf *aHashPtr,
		unsigned aBufc,
		struct bo_qabsbuf const *aBufq
	)) __attribute__((nonnull(1,3,4,6)));

# define AORP_chashal_perform "perform"
# define AORP_chashal_perform_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_INT _AORP_SMOD_UNSIGNED \
	" " _AORP_SIGN_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

#endif /* !__BO_SERVICES_CRYPT_CHASH_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
