/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Crypt.h	(V.Sokur)
 * include/Bo/services/crypt/Crypt.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CRYPT_CRYPT__
# define __BO_SERVICES_CRYPT_CRYPT__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/crypt/Crypttypes.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# include <Bo/services/crypt/Cryptop.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpOpenCryptal(
	aorp_object_t aThis,
	aorp_calgid_t anAlgid,
	struct aorp_cryptal *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpOpenCryptal(aThis, anAlgid, anAlg, anErrPtr) \
	_A_call(openalg, crypt, aThis, anErrPtr, (aorp_calgid_t)(anAlgid), anAlg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpCryptClose(
	struct aorp_cryptal *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCryptClose(anAlg, anErrPtr) \
	_A_closcall(closealg, crypt, (anAlg)->ca_opaque1, anErrPtr, anAlg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpCryptInitctx(
	struct aorp_cryptal *anAlg,
	struct bo_databuf const *aKey,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCryptInitctx(anAlg, aKey, anErrPtr) \
	_A_closcall(initctx, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg, aKey)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpCryptDestroyctx(
	struct aorp_cryptal *anAlg,
	struct aorp_error *anErrPtr
	);
 */
# define AorpCryptDestroyctx(anAlg, anErrPtr) \
	_A_closcall(destroyctx, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpEncrypt(
	struct aorp_cryptal *anAlg,
	aorp_object_t anInStream,
	aorp_object_t anOutStream,
	struct aorp_error *anErrPtr
	);
 */
# define AorpEncrypt(anAlg, anInStream, anOutStream, anErrPtr) \
	_A_closcall(encrypt, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg, anInStream, anOutStream)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpDecrypt(
	struct aorp_cryptal *anAlg,
	aorp_object_t anInStream,
	aorp_object_t anOutStream,
	struct aorp_error *anErrPtr
	);
*/
# define AorpDecrypt(anAlg, anInStream, anOutStream, anErrPtr) \
	_A_closcall(decrypt, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg, anInStream, anOutStream)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpEncrypt_inplace(
	struct aorp_cryptal *anAlg,
	size_t aBufsz,
	bo_pointer_t aBuf,
	struct aorp_error *anErrPtr
	);
 */
# define AorpEncrypt_inplace(anAlg, aBufsz, aBuf, anErrPtr) \
	_A_closcall(encrypt_inplace, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg, \
		(size_t)(aBufsz), aBuf)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpDecrypt_inplace(
	struct aorp_cryptal *anAlg,
	size_t aBufsz,
	bo_pointer_t aBuf,
	struct aorp_error *anErrPtr
	);
*/
# define AorpDecrypt_inplace(anAlg, aBufsz, aBuf, anErrPtr) \
	_A_closcall(decrypt_inplace, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg, \
		(size_t)(aBufsz), aBuf)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3,5)))
AorpEncrypt_mem(
	struct aorp_cryptal *anAlg,
	size_t anInBufsz,
	bo_pointer_t anInBuf,
	size_t anOutBufsz,
	bo_pointer_t anOutBuf,
	struct aorp_error *anErrPtr
	);
 */
# define AorpEncrypt_mem(anAlg, anInBufsz, anInBuf, anOutBufsz, anOutBuf, anErrPtr) \
	_A_closcall(encrypt_mem, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg, \
		(size_t)(anInBufsz), anInBuf, (size_t)(anOutBufsz), anOutBuf)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3,5)))
AorpDecrypt_mem(
	struct aorp_cryptal *anAlg,
	size_t anInBufsz,
	bo_pointer_t anInBuf,
	size_t anOutBufsz,
	bo_pointer_t anOutBuf,
	struct aorp_error *anErrPtr
	);
*/
# define AorpDecrypt_mem(anAlg, anInBufsz, anInBuf, anOutBufsz, anOutBuf, anErrPtr) \
	_A_closcall(decrypt_mem, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg, \
		(size_t)(anInBufsz), anInBuf, (size_t)(anOutBufsz), anOutBuf)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpCryptBlocksize(
	struct aorp_cryptal *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCryptBlocksize(anAlg, anErrPtr) \
	_A_closcall(block_size, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpCryptKeylen(
	struct aorp_cryptal *anAlg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpCryptKeylen(anAlg, anErrPtr) \
	_A_closcall(keylen, cryptal, (anAlg)->ca_opaque2, anErrPtr, anAlg)

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_CRYPT_CRYPT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
