/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * CryptP.h	(V.Sokur)
 * include/Bo/services/crypt/CryptP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CRYPT_CRYPT_P__
# define __BO_SERVICES_CRYPT_CRYPT_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/crypt/Crypttypes.h>

/******************************************************************************
 * %% BeginSection: crypt service declarations
 */
# pragma mark -
# pragma mark [ crypta service declarations ]
# pragma mark -

/*
 * crypt service
 * @{
 */

# include <Bo/services/crypt/Cryptop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name crypt
# include <Bo/aorp/AorpC.h>

_A_service(crypt)

# define AORP_crypt_init_once AORP_any_init_once
# define AORP_crypt_init_once_signature AORP_any_init_once_signature
# define AORP_crypt_fini_once AORP_any_fini_once
# define AORP_crypt_fini_once_signature AORP_any_fini_once_signature
# define AORP_crypt_pod_init AORP_any_pod_init
# define AORP_crypt_pod_init_signature AORP_any_pod_init_signature
# define AORP_crypt_pod_fini AORP_any_pod_fini
# define AORP_crypt_pod_fini_signature AORP_any_pod_fini_signature

_A_op(openalg, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		aorp_calgid_t anAlgid,
		struct aorp_cryptal *anAlg
	)) __attribute__((nonnull(1,4)));

# define AORP_crypt_openalg "open-algorithm"
# define AORP_crypt_openalg_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_POINTER

_A_op(closealg, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_crypt_closealg "close-algorithm"
# define AORP_crypt_closealg_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

# pragma mark -

/*
 * cryptal service
 * @{
 */

# include <Bo/services/crypt/Cryptop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name cryptal
# include <Bo/aorp/AorpC.h>

_A_service(cryptal)

# define AORP_cryptal_init_once AORP_any_init_once
# define AORP_cryptal_init_once_signature AORP_any_init_once_signature
# define AORP_cryptal_fini_once AORP_any_fini_once
# define AORP_cryptal_fini_once_signature AORP_any_fini_once_signature
# define AORP_cryptal_pod_init AORP_any_pod_init
# define AORP_cryptal_pod_init_signature AORP_any_pod_init_signature
# define AORP_cryptal_pod_fini AORP_any_pod_fini
# define AORP_cryptal_pod_fini_signature AORP_any_pod_fini_signature

_A_op(initctx, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg,
		struct bo_databuf const *aKey
	)) __attribute__((nonnull(1,3,4)));

# define AORP_cryptal_initctx "init-context"
# define AORP_cryptal_initctx_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

_A_op(destroyctx, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_cryptal_destroyctx "destroy-context"
# define AORP_cryptal_destroyctx_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(encrypt, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg,
		aorp_object_t anInStream,
		aorp_object_t anOutStream
	)) __attribute__((nonnull(1,3,4,5)));

# define AORP_cryptal_encrypt "encrypt"
# define AORP_cryptal_encrypt_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

_A_op(decrypt, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg,
		aorp_object_t anInStream,
		aorp_object_t anOutStream
	)) __attribute__((nonnull(1,3,4,5)));

# define AORP_cryptal_decrypt "decrypt"
# define AORP_cryptal_decrypt_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

_A_op(encrypt_inplace, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg,
		size_t aBufsz,
		bo_pointer_t aBuf
	)) __attribute__((nonnull(1,3,5)));

# define AORP_cryptal_encrypt_inplace "encrypt-inplace"
# define AORP_cryptal_encrypt_inplace_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_POINTER

_A_op(decrypt_inplace, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg,
		size_t aBufsz,
		bo_pointer_t aBuf
	)) __attribute__((nonnull(1,3,5)));

# define AORP_cryptal_decrypt_inplace "decrypt-inplace"
# define AORP_cryptal_decrypt_inplace_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_POINTER

_A_op(encrypt_mem, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg,
		size_t anInBufsz,
		bo_pointer_t anInBuf,
		size_t anOutBufsz,
		bo_pointer_t anOutBuf
	)) __attribute__((nonnull(1,3,5,7)));

# define AORP_cryptal_encrypt_mem "encrypt-mem"
# define AORP_cryptal_encrypt_mem_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_POINTER

_A_op(decrypt_mem, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg,
		size_t anInBufsz,
		bo_pointer_t anInBuf,
		size_t anOutBufsz,
		bo_pointer_t anOutBuf
	)) __attribute__((nonnull(1,3,5,7)));

# define AORP_cryptal_decrypt_mem "decrypt-mem"
# define AORP_cryptal_decrypt_mem_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_POINTER

_A_op(block_size, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_cryptal_block_size "block-size"
# define AORP_cryptal_block_size_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(keylen, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_cryptal *anAlg
	)) __attribute__((nonnull(1,3)));

# define AORP_cryptal_keylen "keylen"
# define AORP_cryptal_keylen_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

#endif /* !__BO_SERVICES_CRYPT_CRYPT_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
