/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Cryptatypes.h	(V.Sokur)
 * include/Bo/services/crypt/Crypttypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_CRYPT_CRYPTTYPES__
# define __BO_SERVICES_CRYPT_CRYPTTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: constants declarations
 */

/*
 * HASH constants declarations
 */

/* Common limits */
# define BoL_CHASHAL_HASH_MAX (80)
# define BoL_CHASHAL_HASHBLOCK_MAX (80)

# define BoK_INVAL_CHASHAL (-1)
# define BoK_UNK_CHASHAL (2)
# define BoK_NULL_CHASHAL (0)

# define BoK_AHA_CHASHAL (3)
/* NOT FOR YOUR BUSINESS @{ */
# define _BoK_AHA_CHASHAL 3
# define _BO_AHA_CHASHAL_NAME "aha"
/* @} */

# define BoK_MD5_CHASHAL (10)
/* NOT FOR YOUR BUSINESS @{ */
# define _BoK_MD5_CHASHAL 10
# define _BO_MD5_CHASHAL_NAME "md5"
/* @} */
# define BoK_MD5_CHASHAL_HASH_SIZE 16
# define BoK_MD5_CHASHAL_BLOCK_SIZE 64

/*
 * CRYPT constants declarations
 */

# define BoK_INVAL_CRYPTAL (-1)
# define BoK_NULL_CRYPTAL (0)
# define BoK_UNK_CRYPTAL (2)

# define BoK_RC4_CRYPTAL (20)
/* NOT FOR YOUR BUSINESS @{ */
# define _BoK_RC4_CRYPTAL 20
# define _BO_RC4_CRYPTAL_NAME "rc4"
/* @} */
# define BoK_RC4_CRYPTAL_KEY_MAX 256
# define BoK_RC4_CRYPTAL_REPTABLE_SIZE 256

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

typedef int32_t aorp_calgid_t;

/******************************************************************************
 * %% BeginSection: cryptal type declarations
 */

# define AorpK_CRYPTAL_OPAQUE3_SIZE 256

struct aorp_cryptal {
	char     ca_opaque1[AorpK_CLOSURE_SIZE];
	char     ca_opaque2[AorpK_CLOSURE_SIZE];
	char     ca_opaque3[AorpK_CRYPTAL_OPAQUE3_SIZE];
};

/*
 * RC4 type declarations
 */

struct cryptal_rc4key {
	size_t   size;
	char     base[BoK_RC4_CRYPTAL_KEY_MAX];
};

/******************************************************************************
 * %% BeginSection: chashal type declarations
 */

# define AorpK_CHASHAL_OPAQUE3_SIZE 256

struct aorp_chashal {
	char     ca_opaque1[AorpK_CLOSURE_SIZE];
	char     ca_opaque2[AorpK_CLOSURE_SIZE];
	char     ca_opaque3[AorpK_CHASHAL_OPAQUE3_SIZE];
};

/*
 * AHA type declarations
 */

typedef void
(__myc_tcalltype__(mcall) *crypt_hashx_trans_fn)(
# if __myc_require(prototypes)
	u_int32_t *aHash,
	u_int32_t *aBlock
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

struct crypt_hashx {
	crypt_hashx_trans_fn  trans;
	u_int16_t             hashsz;
	u_int16_t             blocksz;

	u_int8_t              hash[BoL_CHASHAL_HASH_MAX];
    u_int8_t              block[BoL_CHASHAL_HASHBLOCK_MAX];
    u_int64_t             counter;
};
typedef struct crypt_hashx crypt_hashx_t;

/*
 * MD5 type declarations
 */

struct chashal_md5buf {
	size_t   size;
	char     base[BoK_MD5_CHASHAL_HASH_SIZE];
};

#endif /* !__BO_SERVICES_CRYPT_CRYPTTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
