/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Plist.h	(N.Panasiuk)
 * include/Bo/services/plist/Plist.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_PLIST_PLIST__
# define __BO_SERVICES_PLIST_PLIST__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/plist/Pltypes.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# include <Bo/services/plist/Plistop.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Bo/aorp/Runtime.h>

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(4)))
AorpPlistInsert(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aToParentPath,
	aorp_object_t aNode,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlistInsert(aRoot, aStartNode, aToParentPath, aNode, anErrPtr) \
	_A_call(insert, plist, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, aStartNode, aToParentPath, aNode)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(5)))
AorpPlistRemove(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aPath,
	aorp_opflags_t Flags,
	aorp_object_t *aResultPtr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlistRemove(aRoot, aStartNode, aPath, Flags, aResultPtr, anErrPtr) \
	_A_call(remove, plist, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, \
		aStartNode, aPath, (aorp_opflags_t)(Flags), aResultPtr)

# pragma mark -

/*
bo_status_t
AorpPlistDelete(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aPath,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlistDelete(aRoot, aStartNode, aPath, Flags, anErrPtr) \
	_A_call(delete, plist, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, \
		aStartNode, aPath, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
AorpPlistMove(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aFromPath,
	char const *aToParentPath,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlistMove(aRoot, aStartNode, aFromPath, aToParentPath, anErrPtr) \
	_A_call(move, plist, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, \
		aStartNode, aFromPath, aToParentPath)

# pragma mark -

/*
bo_status_t
AorpPlistCopy(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aFromPath,
	char const *aToParentPath,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlistCopy(aRoot, aStartNode, aFromPath, aToParentPath, anErrPtr) \
	_A_call(copy, plist, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, aStartNode, aFromPath, aToParentPath)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,4)))
AorpPlistGet(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aPath,
	aorp_opflags_t Flags,
	aorp_object_t *aResultPtr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlistGet(aRoot, aStartNode, aPath, Flags, aResultPtr, anErrPtr) \
	_A_call(get, plist, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, \
		aStartNode, aPath, (aorp_opflags_t)(Flags), aResultPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,4,5)))
AorpPlistLink(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aPath,
	char const *aTarget,
	aorp_object_t *aResultPtr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlistLink(aRoot, aStartNode, aPath, aLinkName, aReference, aResultPtr, anErrPtr) \
	_A_call(link, plist, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, \
		aStartNode, aPath, aLinkName, aReference, aResultPtr)

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_PLIST_PLIST__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
