/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Plnode.h	(N.Panasiuk)
 * include/Bo/services/plist/Plnode.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_PLIST_PLNODE__
# define __BO_SERVICES_PLIST_PLNODE__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/plist/Pltypes.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# include <Bo/services/plist/Plnodeop.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Bo/aorp/Runtime.h>

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeStat(
	aorp_object_t aThis,
	struct aorp_plist_stat *aStatPtr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeStat(aThis, aStatPtr, anErrPtr) \
	_A_call(stat, plnode, aThis, anErrPtr, aStatPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpPlnodeCntl(
	aorp_object_t aThis,
	int aCmd,
	intptr_t anArg,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeCntl(aThis, aCmd, anArg, anErrPtr) \
	_A_call(cntl, plnode, aThis, anErrPtr, aCmd, (intptr_t)(anArg))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeRead(
	aorp_object_t aThis,
	struct aorp_plist_rwarg *aBuf,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeRead(aThis, aBuf, Flags, anErrPtr) \
	_A_call(read, plnode, aThis, anErrPtr, aBuf, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpPlnodeReadbyindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_plist_rwarg *aBuf,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * Useful for list-node. In other cases the index is ignored.
 */
# define AorpPlnodeReadbyindex(aThis, anIndex, aBuf, Flags, anErrPtr) \
	_A_call(readbyindex, plnode, aThis, anErrPtr, \
		(bo_index_t)(anIndex), aBuf, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeWrite(
	aorp_object_t aThis,
	struct aorp_plist_rwarg const *aBuf,
	aorp_opflags_t Flags, // not used currently
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeWrite(aThis, aBuf, Flags, anErrPtr) \
	_A_call(write, plnode, aThis, anErrPtr, aBuf, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeWritebyindex(
	aorp_object_t aThis,
	bo_index_t anIndex,
	struct aorp_plist_rwarg const *aBuf,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * Useful for list-node. In other cases the index is ignored.
 */
# define AorpPlnodeWritebyindex(aThis, anIndex, aBuf, Flags, anErrPtr) \
	_A_call(writebyindex, plnode, aThis, anErrPtr, \
		(bo_index_t)(anIndex), aBuf, (aorp_opflags_t)(Flags))

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpPlnodeCount(
	aorp_object_t aCntr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeCount(aCntr, anErrPtr) \
	_A_call(count, plnode, aCntr, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeInsert(
	aorp_object_t aCntr,
	aorp_object_t aNode,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeInsert(aCntr, aNode, Flags, anErrPtr) \
	_A_call(insert, plnode, aCntr, anErrPtr, aNode, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,5)))
AorpPlnodeRemove(
	aorp_object_t aCntr,
	char const *aNodeName,
	aorp_plist_ino_t aNodeIno,
	aorp_opflags_t Flags,
	aorp_object_t *aResultNodePtr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeRemove(aCntr, aNodeName, aNodeIno, Flags, aResultNodePtr, anErrPtr) \
	_A_call(remove, plnode, aCntr, anErrPtr, \
		aNodeName, (aorp_plist_ino_t)(aNodeIno), (aorp_opflags_t)(Flags), aResultNodePtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeDelete(
	aorp_object_t aCntr,
	char const *aNodeName,
	aorp_plist_ino_t aNodeIno,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeDelete(aCntr, aNodeName, aNodeIno, Flags, anErrPtr) \
	_A_call(delete, plnode, aCntr, anErrPtr, \
		aNodeName, (aorp_plist_ino_t)(aNodeIno), (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,5)))
AorpPlnodeFind(
	aorp_object_t aCntr,
	char const *aNodeName,
	aorp_plist_ino_t aNodeIno,
	aorp_opflags_t Flags,
	aorp_object_t *aResultNodePtr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeFind(aCntr, aNodeName, aNodeIno, Flags, aResultNodePtr, anErrPtr) \
	_A_call(find, plnode, aCntr, anErrPtr, \
		aNodeName, (aorp_plist_ino_t)(aNodeIno), (aorp_opflags_t)(Flags), aResultNodePtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpPlnodeForeach(
	aorp_object_t aCntr,
	int aPresentationMode,
	bo_status_t (__myc_calltype__(mcall) *aPred)(
# if __myc_require(nested_prototypes)
		aorp_object_t *aNode,
		bo_pointer_t aParm,
		struct aorp_error *anErrPtr
# endif
	),
	bo_pointer_t aParm,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeForeach(aCntr, aPresentationMode, aPred, aParm, anErrPtr) \
	_A_call(foreach, plnode, aCntr, anErrPtr, aPresentationMode, aPred, aParm)

# pragma mark -

/*
 * NOTE!!!
 * The operations below require opened cursor!
 */

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpPLnodeInsert_crsr(
	aorp_object_t aCntr,
	struct aorp_plist_cursor *aPosition,
	aorp_object_t *aNode,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPLnodeInsert_crsr(aCntr, aPosition, aNode, Flags, anErrPtr) \
	_A_call(insert_crsr, plnode, aCntr, anErrPtr, \
		aPosition, aNode, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,4)))
AorpPlnodeRemove_crsr(
	aorp_object_t aCntr,
	struct aorp_plist_cursor *aPosition,
	aorp_opflags_t Flags,
	aorp_object_t *aResultNodePtr,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeRemove_crsr(aCntr, aPosition, Flags, aResultNodePtr, anErrPtr) \
	_A_call(remove_crsr, plnode, aCntr, anErrPtr, \
		aPosition, (aorp_opflags_t)(Flags), aResultNodePtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeDelete_crsr(
	aorp_object_t aCntr,
	struct aorp_plist_cursor *aPosition,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeDelete_crsr(aCntr, aPosition, Flags, anErrPtr) \
	_A_call(delete_crsr, plnode, aCntr, anErrPtr, \
		aPosition, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpPlnodeFind_crsr(
	aorp_object_t aCntr,
	struct aorp_plist_cursor *aResultPosition,
	char const *aNodeName,
	aorp_plist_ino_t aNodeIno,
	aorp_opflags_t Flags,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeFind_crsr(aCntr, aResultPosition, aNodeName, aNodeIno, Flags, anErrPtr) \
	_A_call(find_crsr, plnode, aCntr, anErrPtr, \
		aResultPosition, aNodeName, (aorp_plist_ino_t)(aNodeIno), (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpPlnodeForeach_crsr(
	aorp_object_t aCntr,
	int aPresentationMode,
	bo_status_t (__myc_calltype__(mcall) *aPred)(
# if __myc_require(nested_prototypes)
		struct aorp_plist_cursor *aPosition,
		bo_pointer_t aParm,
		struct bo_error *anErrPtr
# endif
	),
	bo_pointer_t aParm,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeForeach_crsr(aCntr, aPresentationMode, aPred, aParm, anErrPtr) \
	_A_call(foreach_crsr, plnode, aCntr, anErrPtr, \
		aPresentationMode, aPred, aParm)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpPlnodeGetpos(
	aorp_object_t aCntr,
	struct aorp_plist_cursor *aResultPosition,
	aorp_object_t aNode,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeGetpos(aCntr, aResultPosition, aNode, anErrPtr) \
	_A_call(getpos, plnode, aCntr, anErrPtr, aResultPosition, aNode)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpPlnodeOpencrsr(
	aorp_object_t aCntr,
	struct aorp_plist_cursor *aCursor,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeOpencrsr(aCntr, aCursor, anErrPtr) \
	_A_call(opencrsr, plnode, aCntr, anErrPtr, aCursor)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpPLcursorClose(
	struct aorp_plist_cursor *aCursor,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPLcursorClose(aCursor, anErrPtr) \
	_A_closcall(closecrsr, plnode, AORP_QCURSOR_Closure(aCursor), anErrPtr, aCursor)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpPLcursorAdvance(
	struct aorp_plist_cursor *aCursor,
	ssize_t aDistance,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * BoS_ERR: error;
 * BoS_NOERR: success, end of container;
 * BoS_OK: success;
 */
# define AorpPLcursorAdvance(aCursor, aDistance, anErrPtr) \
	_A_closcall(advancecrsr, plnode, AORP_QCURSOR_Closure(aCursor), anErrPtr, \
		aCursor, (ssize_t)(aDistance))

/*
bo_status_t
__attribute__((nonnull(1)))
AorpPLcursorNext(
	struct aorp_plist_cursor *aCursor,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPLcursorNext(aCursor, anErrPtr) AorpPLcursorAdvance(aCursor, 1, anErrPtr)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpPLcursorPrev(
	struct aorp_plist_cursor *aCursor,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPLcursorPrev(aCursor, anErrPtr) AorpPLcursorAdvance(aCursor, -1, anErrPtr)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpPLcursorGet(
	struct aorp_plist_cursor *aCursor,
	size_t aValuec,
	struct aorp_udvariant aValuev[],
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The function returns an actual count of values.
 * The returned values, stored to the parameter @valuev, are retained,
 * if possible.
 */
# define AorpPLcursorGet(aCursor, aValuec, aValuev, anErrPtr) \
	_A_closcall(getbycrsr, plnode, AORP_QCURSOR_Closure(aCursor), anErrPtr, \
		aCursor, (size_t)(aValuec), aValuev)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpPlnodeWalk(
	aorp_object_t aThis,
	struct aorp_plist_options const *Options,
	struct aorp_error *anErrPtr
	);
 */
# define AorpPlnodeWalk(aThis, Options, anErrPtr) \
	_A_call(walk, plnode, aThis, anErrPtr, Options)

# pragma mark -

/* NOT FOR YOUR BUSINESS */
/* @{ */

/*
bo_status_t
__attribute__((nonnull(4)))
_AorpPlnodeLookup(
	aorp_object_t aRoot, // = NULL
	aorp_object_t aStartNode, // = NULL
	char const *aPath,
	aorp_object_t *aResultPtr,
	struct aorp_error *anErrPtr
	);
 */
# define _AorpPlnodeLookup(aRoot, aStartNode, aPath, aResultPtr, anErrPtr) \
	_A_call(lookup, plnode, ((aRoot) ? (aRoot) : (aStartNode)), anErrPtr, \
		aStartNode, aPath, aResultPtr)

/* @} */

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_PLIST_PLNODE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
