/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * PlnodeP.h	(N.Panasiuk)
 * include/Bo/services/plist/PlnodeP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_PLIST_PLNODE_P__
# define __BO_SERVICES_PLIST_PLNODE_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/plist/Pltypes.h>

/******************************************************************************
 * %% BeginSection: plnode service declarations
 */
# pragma mark -
# pragma mark [ plnode service declarations ]
# pragma mark -

/*
 * plnode service
 * @{
 */

# include <Bo/services/plist/Plnodeop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name plnode
# include <Bo/aorp/AorpC.h>

_A_service(plnode)

# define AORP_plnode_init_once AORP_any_init_once
# define AORP_plnode_init_once_signature AORP_any_init_once_signature
# define AORP_plnode_fini_once AORP_any_fini_once
# define AORP_plnode_fini_once_signature AORP_any_fini_once_signature
# define AORP_plnode_pod_init AORP_any_pod_init
# define AORP_plnode_pod_init_signature AORP_any_pod_init_signature
# define AORP_plnode_pod_fini AORP_any_pod_fini
# define AORP_plnode_pod_fini_signature AORP_any_pod_fini_signature

_A_op(stat, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_stat *aStatPtr
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_stat "statistic"
# define AORP_plnode_stat_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(cntl, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		int aCmd,
		intptr_t anArg
	)) __attribute__((nonnull(1)));

# define AORP_plnode_cntl "control"
# define AORP_plnode_cntl_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_INTPTR

_A_op(read, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_rwarg *aBuf,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_read "read"
# define AORP_plnode_read_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T

_A_op(readbyindex, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		bo_index_t anIndex,
		struct aorp_plist_rwarg *aBuf,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,4)));

# define AORP_plnode_readbyindex "read-by-index"
# define AORP_plnode_readbyindex_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INDEX_T \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T

_A_op(write, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_rwarg const *aBuf,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_write "write"
# define AORP_plnode_write_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T

_A_op(writebyindex, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		bo_index_t anIndex,
		struct aorp_plist_rwarg const *aBuf,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,4)));

# define AORP_plnode_writebyindex "write-by-index"
# define AORP_plnode_writebyindex_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INDEX_T \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T

_A_op(count, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr
	)) __attribute__((nonnull(1)));

# define AORP_plnode_count "count"
# define AORP_plnode_count_signature AORP_STD_SIGNATURE

_A_op(insert, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		aorp_object_t aNode,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_insert "insert"
# define AORP_plnode_insert_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T

_A_op(remove, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aNodeName,
		aorp_plist_ino_t aNodeIno,
		aorp_opflags_t Flags,
		aorp_object_t *aResultNodePtr
	)) __attribute__((nonnull(1,3,6)));

# define AORP_plnode_remove "remove"
# define AORP_plnode_remove_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_INDEX_T \
	" " _AORP_SIGN_FLAGS_T \
	" " _AORP_SMOD_POINTER _AORP_SIGN_POINTER

_A_op(delete, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aNodeName,
		aorp_plist_ino_t aNodeIno,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_delete "delete"
# define AORP_plnode_delete_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_INDEX_T \
	" " _AORP_SIGN_FLAGS_T

_A_op(find, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aNodeName,
		aorp_plist_ino_t aNodeIno,
		aorp_opflags_t Flags,
		aorp_object_t *aResultNodePtr
	)) __attribute__((nonnull(1,3,6)));

# define AORP_plnode_find "find"
# define AORP_plnode_find_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_INDEX_T \
	" " _AORP_SIGN_FLAGS_T \
	" " _AORP_SMOD_POINTER _AORP_SIGN_POINTER

_A_op(foreach, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		int aPresentationMode,
		bo_status_t (__myc_calltype__(mcall) *aPred)(
#  if __myc_require(nested_prototypes)
			aorp_object_t *aNode,
			bo_pointer_t aParm,
			struct aorp_error *anErrPtr
#  endif
		),
		bo_pointer_t aParm
	)) __attribute__((nonnull(1,4)));

# define AORP_plnode_foreach "foreach"
# define AORP_plnode_foreach_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_FUNCTION \
	" " _AORP_SIGN_POINTER

_A_op(insert_crsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aPosition,
		aorp_object_t *aNode,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,3,4)));

# define AORP_plnode_insert_crsr "insert-crsr"
# define AORP_plnode_insert_crsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T

_A_op(remove_crsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aPosition,
		aorp_opflags_t Flags,
		aorp_object_t *aResultNodePtr
	)) __attribute__((nonnull(1,3,5)));

# define AORP_plnode_remove_crsr "remove-crsr"
# define AORP_plnode_remove_crsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T \
	" " _AORP_SMOD_POINTER _AORP_SIGN_POINTER

_A_op(delete_crsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aPosition,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_delete_crsr "delete-crsr"
# define AORP_plnode_delete_crsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_FLAGS_T

_A_op(find_crsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aResultPosition,
		char const *aNodeName,
		aorp_plist_ino_t aNodeIno,
		aorp_opflags_t Flags
	)) __attribute__((nonnull(1,3,4)));

# define AORP_plnode_find_crsr "find-crsr"
# define AORP_plnode_find_crsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_INDEX_T \
	" " _AORP_SIGN_FLAGS_T

_A_op(foreach_crsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		int aPresentationMode,
		bo_status_t (__myc_calltype__(mcall) *aPred)(
#  if __myc_require(nested_prototypes)
			struct aorp_plist_cursor *aPosition,
			bo_pointer_t aParm,
			struct bo_error *anErrPtr
#  endif
		),
		bo_pointer_t aParm,
	)) __attribute__((nonnull(1,4)));

# define AORP_plnode_foreach_crsr "foreach-crsr"
# define AORP_plnode_foreach_crsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_FUNCTION \
	" " _AORP_SIGN_POINTER

_A_op(getpos, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aResultPosition,
		aorp_object_t aNode
	)) __attribute__((nonnull(1,3,4)));

# define AORP_plnode_getpos "get-position"
# define AORP_plnode_getpos_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

_A_op(opencrsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aCursor
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_opencrsr "open-cursor"
# define AORP_plnode_opencrsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(closecrsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aCursor
	)) __attribute__((nonnull(1,3)));

# define AORP_plnode_closecrsr "close-cursor"
# define AORP_plnode_closecrsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(advancecrsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aCursor,
		ssize_t aDistance
	)) __attribute__((nonnull(1,3,4)));

# define AORP_plnode_advancecrsr "advance-cursor"
# define AORP_plnode_advancecrsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SSIZE_T

_A_op(getbycrsr, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_cursor *aCursor,
		size_t aValuec,
		struct aorp_udvariant aValuev[]
	)) __attribute__((nonnull(1)));

# define AORP_plnode_getbycrsr "get-by-cursor"
# define AORP_plnode_getbycrsr_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_POINTER

_A_op(walk, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct aorp_plist_options const *Options
	)) __attribute__((nonnull(1)));

# define AORP_plnode_walk "walk"
# define AORP_plnode_walk_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(lookup, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		aorp_object_t aStartNode,
		char const *aPath,
		aorp_object_t *aResultPtr
	)) __attribute__((nonnull(1,5)));

# define AORP_plnode_lookup "lookup"
# define AORP_plnode_lookup_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SMOD_POINTER _AORP_SIGN_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

#endif /* !__BO_SERVICES_PLIST_PLNODE_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
