/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Pltypes.h	(N.Panasiuk)
 * include/Bo/services/hilis/Pltypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_HILIS_PLTYPES__
# define __BO_SERVICES_HILIS_PLTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>
# include <Bo/services/basis/Ctnrtypes.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/*
 * Forward declarations
 */

//struct aorp_plist_cconv;
//struct aorp_plist_parser;
//struct aorp_plist_scaner;
//struct aorp_plist_printer;

/* External */
struct aorp_uri;

/*
 * PLIST-NODE-INDEX-NUMBER
 */

typedef u_int32_t aorp_plist_ino_t;
typedef u_int32_t aorp_plist_ino_x;

/* ============================================================================
 * PLIST-STAT
 * @{
 */
# pragma mark -
# pragma mark +++ plist-stat +++
# pragma mark -

/*
 * PLIST-NODE-TYPE
 */

# define _AorpK_PL_NODETYPE_MASK (0x0f)

# define AorpK_PL_INVAL_NODE   (-1)
# define AorpK_PL_NULL_NODE    (0)
# define AorpK_PL_UNK_NODE     (2)

# define AorpK_PL_CDATA_NODE   (3)
# define AorpK_PL_COMM_NODE    (4)
# define AorpK_PL_VAR_NODE     (5)

# define AorpK_PL_LIST_NODE    (6)
# define AorpK_PL_TUPLE_NODE   (7)
# define AorpK_PL_DICT_NODE    (8)
# define AorpK_PL_DOC_NODE     (9)

# define AorpK_PL_REF_NODE     (10) /* ${identifier} */
# define AorpK_PL_EXPR_NODE    (11) /* $(func[param1[:param2:...]](argument-list)) */

/*
 * PLIST-NODE-SPECIFICATOR
 */

# define _AorpK_PL_SPECTOR_MASK (0xf0)
# define _AorpK_PL_SPECTOR_SHIFT (8)

# define AorpK_PL_INVAL_SPECTOR   (-1)
# define AorpK_PL_UNK_SPECTOR     (2)

/* CDATA, LIST, VAR */
# define AorpK_PL_RAW_VALUE       (0)  /* identifier */
# define AorpK_PL_SQUO_VALUE      (4)  /* 'identifier' */
# define AorpK_PL_DQUO_VALUE      (5)  /* "identifier" */

# define AorpK_PL_TINFO_SPECTOR   (6)  /* ex.: $(date(identifier)) */
/* COMM */
# define AorpK_PL_CCOMM_STYLE     (7)  /* C comment style */
# define AorpK_PL_CXXCOMM_STYLE   (8)  /* C++ comment style */

/*
 * PLIST-FLAGS-ATTRIBUTES
 */

# define _AorpK_PL_FATTRIBS_MASK (0xffffff00)
# define _AorpK_PL_FATTRIBS_SHIFT (16)

/*
 * PLIST-STAT declaration
 */

struct aorp_plist_stat {
	int                       plst_isa;
	char const               *plst_strisa;
	char const               *plst_clsname;
	aorp_plist_ino_t          plst_ino;

	/* @todo: redesign, what this? */
	bo_cset_t                 plst_enc,
	                          plst_docenc;
	aorp_object_t             plst_flocation; /* file://[<host>]/... */
};

# define AORP_PL_ISA_Gettype(isa) \
	((isa) & _AorpK_PL_NODETYPE_MASK)

# define AORP_PL_ISA_Settype(isa,type) \
	do { \
		(isa) &= ~_AorpK_PL_NODETYPE_MASK; \
		(isa) |= (u_int32_t)(type) & _AorpK_PL_NODETYPE_MASK; \
	} while (0)

# define AORP_PL_ISA_Getspec(isa) \
	(((isa) & _AorpK_PL_SPECTOR_MASK) >> _AorpK_PL_SPECTOR_SHIFT)

# define AORP_PL_ISA_Setspec(isa,type) \
	do { \
		(isa) &= ~_AorpK_PL_SPECTOR_MASK; \
		(isa) |= (((u_int32_t)(type) << _AorpK_PL_SPECTOR_SHIFT) \
			& _AorpK_PL_SPECTOR_MASK); \
	} while (0)

# define AORP_PL_ISA_Getflags(isa) \
	(((isa) & _AorpK_PL_FATTRIBS_MASK)) >> _AorpK_PL_FATTRIBS_SHIFT)

# define AORP_PL_ISA_Setflags(isa,type) \
	do { \
		(isa) &= ~_AorpK_PL_FATTRIBS_MASK; \
		(isa) |= (((u_int32_t)(type) << _AorpK_PL_FATTRIBS_SHIFT) \
			& _AorpK_PL_FATTRIBS_MASK); \
	} while (0)

# define AORP_PLSTAT_Isa(aStat) ((aStat)->plst_isa)

# define AORP_PLSTAT_Gettype(aStat) (AORP_PL_ISA_Gettype(AORP_PLSTAT_Isa(aStat)))
# define AORP_PLSTAT_Settype(aStat,aType) (AORP_PL_ISA_Settype(AORP_PLSTAT_Isa(aStat),aType))
# define AORP_PLSTAT_Getspec(aStat) (AORP_PL_ISA_Getspec(AORP_PLSTAT_Isa(aStat)))
# define AORP_PLSTAT_Setspec(aStat,aSpec) (AORP_PL_ISA_Setspec(AORP_PLSTAT_Isa(aStat),aSpec))
# define AORP_PLSTAT_Getflags(aStat) (AORP_PL_ISA_Getflags(AORP_PLSTAT_Isa(aStat)))
# define AORP_PLSTAT_Setflags(aStat,Flags) (AORP_PL_ISA_Setflags(AORP_PLSTAT_Isa(aStat),Flags))

# define AORP_PLSTAT_Strisa(aStat) ((aStat)->plst_strisa)
# define AORP_PLSTAT_Classname(aStat) ((aStat)->plst_clsname)
# define AORP_PLSTAT_Ino(aStat) ((aStat)->plst_ino)
# define AORP_PLSTAT_Enc(aStat) ((aStat)->plst_enc)
# define AORP_PLSTAT_Docenc(aStat) ((aStat)->plst_docenc)
# define AORP_PLSTAT_Floc(aStat) ((aStat)->plst_flocation)

/* @} */

/* ============================================================================
 * PLIST-NODE-ATTRIBUTE
 * @{
 */
# pragma mark -
# pragma mark +++ plist-node-attribute +++
# pragma mark -

struct aorp_plist_attrib {
	char const *pla_name, *pla_value;
};
typedef struct aorp_plist_attrib  aorp_plist_attrib_t;

// @todo: remove @{
#if 0
struct aorp_plist_attrlist {
	/* NOT FOR YOUR BUSINESS @{ */
	size_t                    _plal_scho_ty_tut_zabuv_1,
	                          _plal_scho_ty_tut_zabuv_2;
	struct aorp_plist_attrib *_plal_scho_ty_tut_zabuv_3;
	/* @} */
};
typedef struct aorp_plist_attrlist  aorp_plist_attrlist_t;
#endif
// @}
typedef bo_seqlist_t aorp_plist_attrlist_t;

/* @} */

#if 0

/*
 * ARGUMENT FOR CONTAINER-NODE READ OPERATIONS
 */

/* packed */
struct aorp_plist_node_ds {
	int          isa;
	char const  *name;
};

struct aorp_plist_rwarg {
	union {
		char                      *plrw_data;
		struct aorp_plist_node_ds *plrw_list;
	} u;
	size_t  plrw_dsize;
	/* @todo: redesign, what this? */
	int     plrw_seqform;
};

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) void
AorpPLRWArgDestroy(
# if __myc_require(prototypes)
	struct aorp_plist_rwarg *
# endif
	);

__MYC_END_CDECLS

/*
 * PLIST-CURSOR
 */

# define aorp_plist_cursor aorp_qcursor







//typedef size_t    aorp_plist_dsize_t;
//typedef u_int16_t aorp_plist_dsize_x;


/*
 * PLIST-FLAGS
 *
 * Flags used in various operations.
 */

#define AorpF_PLIST_UMASK 0xffff

/* Node operations:
   free, destroy, insert, remove, remove2, delete, delete2 */
# define AorpF_PLIST_FORCE 0x0001

/* Node operations: create, dup, dup2 */
/* NOT FOR YOUR BUSINESS @{ */
# define _AorpF_PLIST_C_DUP 0x00010000
# define _AorpF_PLIST_C_NOCLD 0x00020000
# define _AorpF_PLIST_C_NOMNT 0x00040000
/* @} */
# define AorpF_PLIST_C_MOUNT 0x0002      /* the node should be mounted not inserted to the parent */
# define AorpF_PLIST_C_PARENT 0x0004     /* dup, dup2: use new node parent from the request-node */
# define AorpF_PLIST_C_NAME 0x0008       /* dup, dup2: use new node name from the request-node */
# define AorpF_PLIST_C_ATTRLIST 0x0010   /* dup, dup2: use new node attributes from the request-node */
/* Used by _AorpPLNCreate */
# define AorpF_PLIST_C_INO 0x0020        /* use new node ino from the request-node */

/* Node operations: read, cntl.get, cntl.set */
# define AorpF_PLIST_DDA 0               /* direct data access, can be used with only leaf nodes */
# define AorpF_PLIST_COPY 0x0100
# define AorpF_PLIST_ALLOC 0x0200
# define AorpF_PLIST_SIZE_ONLY 0x0400
/* Node operations: find, find2
   Functions: AorpPLNReadByIndex, AorpPLNWriteByIndex, AorpPLNFind, AorpPLNFind2 */
# define AorpF_PLIST_IGNORE_COMM 0x1000  /* ignore comments while caclulating index position. Useful for a list-node. */
/* NOT FOR YOUR BUSINESS (???) @{ */
# define _AorpF_PLIST_NAMEISINDEX 0x2000 /* plist-coded-decimal, see aorp_plcdecimal_t */
/* @} */

/* Functions: is-ancestor */
# define AorpF_PLIST_COMMON 0
# define AorpF_PLIST_IMMED 0x8000


/* FOREACH MODE */
/* u_int8_t */

/* Enumerate own direct children */
# define AorpF_PLIST_LINEAR_FOREACH 0x01

/* If there is a mountpoint plist, then enumerate children
   of the mounted plist. Otherwise, enumerate own direct children. */
# define AorpF_PLIST_CLEVER_FOREACH 0x02

/*
 * PLIST MISC.
 */

/*
 * CALLBACK TYPES used while node duplicating or serializing
 */

typedef _Bool
(__myc_tcalltype__(mcall) *aorp_plist_node_filter_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	int aPresentationMode,
	struct aorp_plist_node const *aNode
# endif
	);

typedef _Bool
(__myc_tcalltype__(mcall) *aorp_plist_attr_filter_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	int aPresentationMode,
	struct aorp_plist_node const *aNode,
	struct aorp_plist_attrib const *anAttrib
# endif
	);

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plist_attr_transform_cb)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	int aPresentationMode,
	struct aorp_plist_node const *aNode,
	bo_pointer_t /* aorp_plist_attrlist_t* */ aNodeAttrList
# endif
	);

struct aorp_plist_dup_callbacks
{
	aorp_plist_node_filter_cb         pldup_node_filter;
	aorp_plist_attr_filter_cb         pldup_attr_filter;
	aorp_plist_attr_transform_cb      pldup_attr_transform;
};

struct aorp_plist_serializer_callbacks
{
	aorp_plist_node_filter_cb         plsr_node_filter;
	aorp_plist_attr_filter_cb         plsr_attr_filter;
	aorp_plist_attr_transform_cb      plsr_attr_transform;
};

/* ============================================================================
 * PLIST-PARSER HANDLER TYPES
 * @{
 */
# pragma mark -
# pragma mark +++ plist-parser handler types +++
# pragma mark -

/*
 * Handlers should return only values:
 * AorpS_ERR
 * AorpS_PLPR_CONT
 * AorpS_PLPR_STOP
 * AorpS_PLPR_STOP_HANDLED
 */

# define AorpS_PLPR_CONT (0)
# define AorpS_PLPR_STOP (1)
# define AorpS_PLPR_STOP_HANDLED (2)

/* This is called for a text document encoding. */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plpr_encoding_handler)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr,
	bo_cset_t aDocEncoding
# endif
	);

/* These are called for a property-list's start and finish respectively. */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plpr_begin_plist_handler)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr,
	char const *aName,
	struct aorp_plist_attrib anAttrv[],
	size_t anAttrc,
	bo_cset_t aRequiredEncoding,
	struct aorp_uri const *aFileLoc
# endif
	)__attribute__((mcalltype, nonnull(3,4)));

/* These are called for a plist-dict's, plist's-list, plist's-tuple
   start and finish respectively. */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plpr_begin_obj_handler)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr,
	char const *aName,
	struct aorp_plist_attrib anAttrv[],
	size_t anAttrc
# endif
	);

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plpr_end_obj_handler)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr
# endif
	);

/* This is called for a plist-list's element. */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plpr_lstelem_handler)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr,
	char const *anElement,
	aorp_plist_seqform_t aForm    /* raw, quo, ref */
# endif
	);

/* This is called for a plist-var. */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plpr_var_handler)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr,
	char const *aName,
	struct aorp_plist_attrib anAttrv[],
	size_t anAttrc,
	char const *aValue,
	aorp_plist_seqform_t aValtype
# endif
	);

/* This is called for a plist-comment. */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_plpr_comm_handler)(
# if __myc_require(prototypes)
	bo_pointer_t anUserCtx,
	struct aorp_error *anErrPtr,
	char const *aComment,
	aorp_plist_seqform_t aCommtype
# endif
	);

/*
 * PLIST-PARSER CALLBACKS
 */

struct aorp_plist_parser_callbacks {
	aorp_plpr_encoding_handler        plpr_encoding_handler;
	aorp_plpr_begin_plist_handler     plpr_begin_root_handler;
	aorp_plpr_end_obj_handler         plpr_end_root_handler;
	aorp_plpr_begin_plist_handler     plpr_begin_plist_handler;
	aorp_plpr_end_obj_handler         plpr_end_plist_handler;
	aorp_plpr_begin_obj_handler       plpr_begin_dict_handler;
	aorp_plpr_end_obj_handler         plpr_end_dict_handler;
	aorp_plpr_begin_obj_handler       plpr_begin_list_handler;
	aorp_plpr_end_obj_handler         plpr_end_list_handler;
	aorp_plpr_lstelem_handler         plpr_lstelem_handler;
	aorp_plpr_begin_obj_handler       plpr_begin_tuple_handler;
	aorp_plpr_end_obj_handler         plpr_end_tuple_handler;
	aorp_plpr_var_handler             plpr_var_handler;
	aorp_plpr_comm_handler            plpr_comm_handler;
};

/* @} */

/* Common user-context for SCAN functions. */
struct aorp_plist_scan_uctx
{
	struct aorp_plist_node *scuc_result;
};

/*
 * PLIST-PRINTER CALLBACK TYPES
 */

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_bwrite_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	bo_const_pointer_t aData,                 /* byte-data */
	size_t aBytec                             /* byte-count */
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_swrite_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	char const *aString                       /* string-data */
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_iwrite_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_write_enc_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	struct aorp_plist_cconv *aConv,           /* converter */
	char const *anEncoding                    /* encoding */
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_write_begin_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	struct aorp_plist_cconv *aConv,           /* converter */
	char const *aName,                        /* object-name */
	struct aorp_plist_attrib anAttrv[],       /* attribute-list */
	size_t anAttrc,                           /* attribute-count */
	_Bool WithLineFeed
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_write_end_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	struct aorp_plist_cconv *aConv,           /* converter */
	_Bool WithLineFeed
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_write_lstelem_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	struct aorp_plist_cconv *aConv,           /* converter */
	char const *anElement,                    /* list-element */
	size_t aBytec,
	aorp_plist_seqform_t aForm,               /* list-element-form: raw, quo, ref */
	_Bool IsLast,                             /* is-last-element */
	_Bool WithLineFeed
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_write_var_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	struct aorp_plist_cconv *aConv,           /* converter */
	char const *aName,                        /* var-name */
	struct aorp_plist_attrib anAttrv[],       /* attribute-list */
	size_t anAttrc,                           /* attribute-count */
	char const *aValue,                       /* var-value */
	aorp_plist_seqform_t aValtype,            /* var-value-type */
	_Bool WithLineFeed
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_write_comm_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	struct aorp_plist_cconv *aConv,           /* converter */
	char const *aComment,                     /* comment-string */
	size_t aBytec, /* = 0 */
	aorp_plist_seqform_t aCommtype            /* comment-type */
# endif
	);

typedef ssize_t
(__myc_tcalltype__(mcall) *aorp_plprn_write_cdata_fn)(
# if __myc_require(prototypes)
	struct aorp_plist_printer *aPrinter,      /* printer */
	struct aorp_error *anErrPtr,
	struct aorp_plist_cconv *aConv,           /* converter */
	bo_const_pointer_t aData,                 /* cdata-value */
	size_t aBytec,                            /* cdata-size */
	aorp_plist_seqform_t aForm                /* cdata-form: raw, quo, ref */
# endif
	);

struct aorp_plist_printer_callbacks
{
	aorp_plprn_write_enc_fn       plprn_write_encoding;
	aorp_plprn_write_begin_fn     plprn_write_begin_root;
	aorp_plprn_write_end_fn       plprn_write_end_root;
	aorp_plprn_write_begin_fn     plprn_write_begin_plist;
	aorp_plprn_write_end_fn       plprn_write_end_plist;
	aorp_plprn_write_begin_fn     plprn_write_begin_dict;
	aorp_plprn_write_end_fn       plprn_write_end_dict;
	aorp_plprn_write_begin_fn     plprn_write_begin_list;
	aorp_plprn_write_end_fn       plprn_write_end_list;
	aorp_plprn_write_lstelem_fn   plprn_write_lstelem;
	aorp_plprn_write_begin_fn     plprn_write_begin_tuple;
	aorp_plprn_write_end_fn       plprn_write_end_tuple;
	aorp_plprn_write_var_fn       plprn_write_var;
	aorp_plprn_write_comm_fn      plprn_write_comm;

	aorp_plprn_write_cdata_fn     plprn_write_cdata;

	aorp_plprn_bwrite_fn          plprn_write_mem;
	aorp_plprn_swrite_fn          plprn_write_str;
	aorp_plprn_iwrite_fn          plprn_write_indent;
	aorp_plprn_iwrite_fn          plprn_write_linefeed;
};

/*
 * PLIST-OPTIONS
 */

struct aorp_plist_options
{
	union
	{
		struct
		{
			aorp_plist_pformat_x   reserved;
			aorp_plist_pmode_x     pmode;    /* presentation mode */
			aorp_plist_pdepth_x    pdepth;   /* presentation depth */
			bo_pointer_t           uctx;     /* user context */
			aorp_plist_walk_cb     ucb;      /* user callback */
		} plo_walk;

		struct
		{
			aorp_plist_pformat_x   reserved;
			aorp_plist_pmode_x     pmode;    /* presentation mode */
			aorp_plist_pdepth_x    pdepth;   /* presentation depth */
			bo_pointer_t           uctx;     /* user context */
			struct aorp_plist_dup_callbacks const *ucbs;   /* user callbacks */
		} plo_dup;

		struct
		{
			aorp_plist_pformat_x   pfmt;    /* presentation format */ /* unsupported */
			aorp_plist_pmode_x     pmode;   /* presentation mode */ /* unsupported */
			char const            *clsname;
			bo_pointer_t           uctx;    /* user context */
			struct aorp_plist_parser_callbacks const *ucbs;
			bo_cset_t              src_enc,
							       dst_enc;
			struct aorp_uri const *floc;
		} plo_scan;

		struct
		{
			aorp_plist_pformat_x   pfmt;    /* presentation format */ /* unsupported */
			aorp_plist_pmode_x     pmode;   /* presentation mode */
			aorp_plist_pdepth_x    pdepth;  /* presentation depth */
			bo_pointer_t           uctx;    /* user context */
			struct aorp_plist_serializer_callbacks const *ucbs;   /* user callbacks */
			bo_cset_t              src_enc,
							       dst_enc;

			u_int16_t              level;
			u_int16_t              noindent :1;
			u_int16_t              pad :15;
			struct aorp_plist_printer_callbacks const *prncbs;
		} plo_prin;
	} u;
};

/*
 * PLIST-CURSOR
 */

//struct aorp_plist_cursor;
//
//typedef void
//(__myc_tcalltype__(mcall) *aorp_plcursor_close_fn)(
//# if __myc_require(prototypes)
//	struct aorp_plist_cursor *aThis
//# endif
//	);
//
//typedef struct aorp_plist_cursor *
//(__myc_tcalltype__(mcall) *aorp_plcursor_advance_fn)(
//# if __myc_require(prototypes)
//	struct aorp_plist_cursor *aThis,
//	size_t aDistance
//# endif
//	);
//
//typedef struct aorp_plist_node *
//(__myc_tcalltype__(mcall) *aorp_plcursor_deref_fn)(
//# if __myc_require(prototypes)
//	struct aorp_plist_cursor *aThis
//# endif
//	);
//
//struct aorp_plist_cursor
//{
//	char                      plcu_reserved[16];
//
//	aorp_plcursor_close_fn    plcu_close;
//	aorp_plcursor_advance_fn  plcu_advance;
//	aorp_plcursor_deref_fn    plcu_deref;
//};
//# define aorp_plist_cursor_p(aThis) ((struct aorp_plist_cursor *)(aThis))
//
//# define AORP_PLIST_CURSOR_Close(aThis)                                            \
//	((aorp_plist_cursor_p(aThis)->plcu_close)                                      \
//		? ((*aorp_plist_cursor_p(aThis)->plcu_close)(aorp_plist_cursor_p(aThis)), 0) \
//		: 0                                                                      \
//	)
//
//# define AORP_PLIST_CURSOR_Advance(aThis,aDistance) \
//	((*aorp_plist_cursor_p(aThis)->plcu_advance)(aorp_plist_cursor_p(aThis),aDistance))
//
//# define AORP_PLIST_CURSOR_Next(aThis) AORP_PLIST_CURSOR_Advance(aThis,1)
//
//# define AORP_PLIST_CURSOR_Deref(aThis) \
//	((*aorp_plist_cursor_p(aThis)->plcu_deref)(aorp_plist_cursor_p(aThis)))


/*
 * PL CODED DECIMAL
 */

//typedef char aorp_plcdecimal_t[10];
//typedef char const aorp_const_plcdecimal_t[10];
//
//# include <Bo/BoLinkage.h>
//
//__MYC_BEGIN_CDECLS
//
//AORP_LINKAGE_(char *)
//BoBin2plcd(
//# if __myc_require(prototypes)
//	u_int32_t ,
//	aorp_plcdecimal_t
//# endif
//	);
//
//# define BoInt2plcd(aNumber,aStr) BoBin2plcd((u_int32_t)aNumber, aStr)
//# if SIZEOF_PTR == 4
//#  define BoPtr2plcd(aNumber,aStr) BoBin2plcd((u_int32_t)aNumber, aStr)
//# else
//#  define BoPtr2plcd(aNumber,aStr) ...
//# endif
//
//AORP_LINKAGE_(u_int32_t)
//BoPlcd2bin(
//# if __myc_require(prototypes)
//	aorp_const_plcdecimal_t
//# endif
//	);
//
//# define BoPlcd2int(aStr) ((int32_t)BoPlcd2bin(aStr))
//# if SIZEOF_PTR == 4
//#  define BoPlcd2ptr(aStr) ((bo_pointer_t)BoPlcd2bin(aStr))
//# else
//#  define BoPlcd2ptr(aStr) ...
//# endif
//
//__MYC_END_CDECLS

# endif

#endif /* !__BO_SERVICES_HILIS_PLTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
