/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Urilib.h	(N.Panasiuk)
 * include/Bo/services/hilis/Urilib.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_HILIS_URILIB__
# define __BO_SERVICES_HILIS_URILIB__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/hilis/UrilibP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

__myc_vlinkage__(public,libBo) aorp_object_t _G_urilib_service_default;

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) aorp_object_t
__status__((
	BoS_SUCCESS = [
		!BoSP_ILLPTR = (/* created uri object */);
		NULL = (/* calculated actual specification buffer size */);
	];
	BoS_FAILURE = [
		BoS_EPTR = (
			BoK_RacError_NotConfigured,
			BoK_AIBNameError_NoSuchEntry,
			BoK_ArgError_Overflow,
			BoK_SysError_NoMemory,
			BoK_TextError_BadToken
		);
	];
))
__attribute__((nonnull(3)))
AorpCreateUriWithString(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_pointer_t anObjectMemory /* = NULL */,
	char const *anUriStr,
	bo_pointer_t aSpecBuf __myc_defval__(NULL),
	size_t aSpecBufsz __myc_defval__(0),
	size_t *anActualSpecszPtr __myc_defval__(NULL),
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__status__((
	BoS_SUCCESS = [
		!BoSP_ILLPTR = (/* created uri object */);
	];
	BoS_FAILURE = [
		NULL = (
			BoK_RacError_NotConfigured,
			BoK_AIBNameError_NoSuchEntry,
			BoK_SysError_NoMemory,
			BoK_TextError_BadToken
		);
	];
))
__attribute__((nonnull(2)))
AorpACreateUriWithString(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *anUriStr,
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(3)))
AorpCreateUriWithWstring(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_pointer_t anObjectMemory /* = NULL */,
	wchar_t const *anUriStr,
	bo_pointer_t aSpecBuf __myc_defval__(NULL),
	size_t aSpecBufsz __myc_defval__(0),
	size_t *anActualSpecszPtr __myc_defval__(NULL),
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(2)))
AorpACreateUriWithWstring(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	wchar_t const *anUriStr,
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(3)))
AorpCreateUriWithFileSystemPath(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_pointer_t anObjectMemory /* = NULL */,
	char const *aPath,
	bo_pointer_t aSpecBuf __myc_defval__(NULL),
	size_t aSpecBufsz __myc_defval__(0),
	size_t *anActualSpecszPtr __myc_defval__(NULL),
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(2)))
AorpACreateUriWithFileSystemPath(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aPath,
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(3)))
AorpCreateUriWithFileSystemWpath(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_pointer_t anObjectMemory /* = NULL */,
	wchar_t const *aPath,
	bo_pointer_t aSpecBuf __myc_defval__(NULL),
	size_t aSpecBufsz __myc_defval__(0),
	size_t *anActualSpecszPtr __myc_defval__(NULL),
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_object_t
__attribute__((nonnull(2)))
AorpACreateUriWithFileSystemWpath(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	wchar_t const *aPath,
	struct aorp_uri_options const *Options __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) _Bool
__attribute__((nonnull(2)))
AorpSupportUriScheme(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aSchemeNamev[],
	size_t aCount
# endif
	);

/*
_Bool
AorpSupportUriScheme1(
	aorp_object_t aSelf,
	char const *aSchemeName
	);
*/
# if __myc_support(macro_retval)
#  define AorpSupportUriScheme1(aSelf, aSchemeName) \
	__extension__ ({ \
		char const *___namev[1] = { aSchemeName }; \
		/* return */ AorpSupportUriScheme(aSelf, ___namev, __myc_namembs(___namev)); \
	})
# else
__myc_extern_inline__ _Bool
__attribute__((nonnull(2)))
AorpSupportUriScheme1(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aSchemeName
# else
	aSelf, aSchemeName
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aSelf;
	char const *aSchemeName;
# endif
{
	char const *namev[1] = { aSchemeName };
	return (AorpSupportUriScheme(aSelf, namev, __myc_namembs(namev)));
}
# endif

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_uri_scheme_t
__status__((
	BoS_SUCCESS = [];
	BoS_FAILURE = [
		BoK_URI_INVAL_SCHEME = (
			BoK_RacError_NotConfigured,
			BoK_AIBNameError_NoSuchEntry
		);
	];
))
__attribute__((nonnull(2)))
AorpGetUriSchemeByName(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aSchemeNamev[],
	size_t aCount,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
bo_uri_scheme_t
AorpGetUriSchemeByName1(
	aorp_object_t aSelf,
	char const *aSchemeName,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(macro_retval)
#  define AorpGetUriSchemeByName1(aSelf, aSchemeName, anErrPtr) \
	__extension__ ({ \
		bo_uri_scheme_t ___retval; \
		char const *___namev[1] = { aSchemeName }; \
		___retval = AorpGetUriSchemeByName(aSelf, ___namev, __myc_namembs(___namev), anErrPtr); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ bo_uri_scheme_t
__attribute__((nonnull(2)))
AorpGetUriSchemeByName1(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aSchemeName,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aSelf, aSchemeName, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aSelf;
	char const *aSchemename;
	struct aorp_error *anErrPtr;
# endif
{
	bo_uri_scheme_t retval;
	char const *namev[1] = { aSchemeName };
	retval = AorpGetUriSchemeByName(aSelf, namev, __myc_namembs(namev), anErrPtr);
	return (retval);
}
# endif

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) char const **
__status__((
	BoS_SUCCESS = [];
	BoS_FAILURE = [
		NULL = (BoK_RacError_NotConfigured, BoK_AIBNameError_NoSuchEntry);
	];
))
AorpGetUriSchemeName(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_uri_scheme_t aScheme,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
char const *
AorpGetUriSchemeName1(
	aorp_object_t aSelf,
	bo_uri_scheme_t aScheme,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(macro_retval)
#  define AorpGetUriSchemeName1(aSelf, aScheme, anErrPtr) \
	__extension__ ({ \
		char const *___retval; \
		char const **___namev; \
		___namev = AorpGetUriSchemeName(aSelf, aScheme, anErrPtr); \
		___retval = NULL; \
		if (NULL != ___namev) \
			___retval = *___namev; \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ char const *
AorpGetUriSchemeName1(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_uri_scheme_t aScheme,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# else
	aSelf, aScheme, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aSelf;
	bo_uri_scheme_t aScheme;
	struct aorp_error *anErrPtr;
# endif
{
	char const *retval;
	char const **namev;
	namev = AorpGetUriSchemeName(aSelf, aScheme, anErrPtr);
	retval = NULL;
	if (NULL != namev)
		retval = *namev;
	return (retval);
}
# endif

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_uri_schemeinfo const *
__status__((
	BoS_SUCCESS = [];
	BoS_FAILURE = [
		NULL = (BoK_RacError_NotConfigured, BoK_AIBNameError_NoSuchEntry);
	];
))
__attribute__((nonnull(2)))
AorpGetUriSchemeInfoByName(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aSchemeNamev[],
	size_t aCount,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
struct aorp_uri_schemeinfo const *
AorpGetUriSchemeInfoByName1(
	aorp_object_t aSelf,
	char const *aSchemeName,
	struct aorp_error *anErrPtr
	);
 */
# if __myc_support(macro_retval)
#  define AorpGetUriSchemeInfoByName1(aSelf, aSchemeName, anErrPtr) \
	__extension__ ({ \
		char const *___namev[1] = { aSchemeName }; \
		/* return */ AorpGetUriSchemeInfoByName(aSelf, ___namev, __myc_namembs(___namev), anErrPtr); \
	})
# else
__myc_extern_inline__ struct aorp_uri_schemeinfo const *
__attribute__((nonnull(2)))
AorpGetUriSchemeInfoByName1(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *aSchemeName,
	struct aorp_error *anErrPtr
# else
	aSelf, aSchemeName, anErrPtr
# endif
	)
# if !__myc_require(prototypes)
	aorp_object_t aSelf;
	char const *aSchemeName;
	struct aorp_error *anErrPtr;
# endif
{
	char const *namev[1] = { aSchemeName };
	return (AorpGetUriSchemeInfoByName(aSelf, namev, __myc_namembs(namev), anErrPtr));
}
# endif

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) struct aorp_uri_schemeinfo const *
__status__((
	BoS_SUCCESS = [];
	BoS_FAILURE = [
		NULL = (BoK_RacError_NotConfigured, BoK_AIBNameError_NoSuchEntry);
	];
))
AorpGetUriSchemeInfoByID(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	bo_uri_scheme_t aScheme,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(2)))
AorpAddUriSchemeInfo(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	struct aorp_uri_schemeinfo *anInfov[],
	size_t aCount
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(2)))
AorpAddUriSchemeInfo_nlk(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	struct aorp_uri_schemeinfo *anInfov[],
	size_t aCount
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(2)))
AorpRemUriSchemeInfo(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	struct aorp_uri_schemeinfo *anInfov[],
	size_t aCount
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(2)))
AorpRemUriSchemeInfo_nlk(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	struct aorp_uri_schemeinfo *anInfov[],
	size_t aCount
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(2)))
_AorpParseUriScheme(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	char const *anUriStr,
	char *aBuffer __myc_defval__(NULL),
	size_t aBytec __myc_defval__(0),
	char const **aNextPartPtr __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(2)))
_AorpWparseUriScheme(
# if __myc_require(prototypes)
	aorp_object_t aSelf /* = NULL */,
	wchar_t const *anUriStr,
	wchar_t *aBuffer __myc_defval__(NULL),
	size_t aCharc __myc_defval__(0),
	wchar_t const **aNextPartPtr __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_HILIS_URILIB__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
