/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Uris.h	(A.Kuzina)
 * include/Bo/services/hilis/Uris.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_HILIS_URISPECIFICATIONS__
# define __BO_SERVICES_HILIS_URISPECIFICATIONS__

/******************************************************************************
 * %% BeginSection: includes
 */

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

/*
 * HTTP URI declarations
 *
 * http://[<user>[:<passwd>]@]<host>[:<port>][/[<path>[?<query>][#<fragment>]]]
 */

aorp_uri_spec(http) {
	struct bo_charseq  user;
	struct bo_charseq  passwd;
	struct bo_charseq  host;
	struct bo_charseq  port;
# define AORP_URI_HTTP_PORT_DEFAULT "80"
# define AorpK_URI_HTTP_PORT_DEFAULT 80

	struct bo_charseq  path;
	struct bo_charseq  query;
	struct bo_charseq  fragment;
};

/*
 * FTP URI declarations
 *
 * ftp://[<user>[:<password>]@]<host>[:<port>][/[<path>[;type=<typecode>]]]
 */

aorp_uri_spec(ftp) {
	char *user;
	char *passwd;

	char *host;
	char *port;
# define AORP_URI_FTP_PORT_DEFAULT "21"
# define AorpK_URI_FTP_PORT_DEFAULT 21

	char *path;
	int   type;
# define AorpK_URI_FTP_TYPE_UNSPEC  0
# define AorpK_URI_FTP_TYPE_A  'a'
# define AorpK_URI_FTP_TYPE_I  'i'
# define AorpK_URI_FTP_TYPE_D  'd'
};

/*
 * MAILTO URI declarations
 *
 * mailto:<username>[:<passwd>]@<host>[:<port>]
 */

aorp_uri_spec(mailto) {
	struct bo_charseq  user;
	struct bo_charseq  passwd;
	struct bo_charseq  host;
	struct bo_charseq  port;
};

/*
 * FILE URI declarations
 *
 * file://[<user>[:<passwd>]@][<host>[:<port>]]/<path>
 */

aorp_uri_spec(file) {
	struct bo_charseq  user;
	struct bo_charseq  passwd;
	struct bo_charseq  host;
	struct bo_charseq  port;
# define AORP_URI_FILE_HOST_DEFAULT "localhost"

	struct bo_charseq  path;
};

/*
 * SLUZBA URI declarations
 *
 * sluzba://[<host>]/<service>[:<port>][/<protocol>[/<family>]]
 */

aorp_uri_spec(sluzba) {
	char *host;
# define AORP_URI_SLUZBA_HOST_DEFAULT "localhost"

	char *service;
	char *port;
	char *protocol;
	char *family;
};

/* ----------------------------------------------------------------------------
 * ADBC-SCHEME declarations
 * @{
 */

/*
 * ADBC URI declarations
 *
 * adbc:<protocol>:<subprotocol>
 */

aorp_uri_spec(adbc) {
	struct bo_charseq protocol;
	struct bo_charseq subprotocol;
};

/*
 * ADBC:POSTGRESQL URI declarations
 *
 * adbc:postgresql://[<username>[:<password>]@][<host>[:<port>]]/<databasename>
 */

aorp_uri_spec2(adbc,postgresql) {
	struct bo_charseq  user;
# define AORP_URI_ADBC_POSTGRESQL_USER_DEFAULT "postgres"

	struct bo_charseq  passwd;
# define AORP_URI_ADBC_POSTGRESQL_PASSWD_DEFAULT ""

	struct bo_charseq  host;
# define AORP_URI_ADBC_POSTGRESQL_HOST_DEFAULT "localhost"

	struct bo_charseq  port;
# define AORP_URI_ADBC_POSTGRESQL_PORT_DEFAULT "5432"

	struct bo_charseq  database;
};

/*
 * ADBC:SQLITE URI declarations
 *
 * adbc:sqlite://[<username>[:<password>]@][<host>]/<path>
 */

aorp_uri_spec2(adbc,sqlite) {
	struct bo_charseq  user;
# define AORP_URI_ADBC_SQLITE_USER_DEFAULT ""

	struct bo_charseq  passwd;
# define AORP_URI_ADBC_SQLITE_PASSWD_DEFAULT ""

	struct bo_charseq  host;
# define AORP_URI_ADBC_SQLITE_HOST_DEFAULT "localhost"

	struct bo_charseq  reserved;

	struct bo_charseq  path;
};

/*
 * ADBC:ODBC URI declarations
 *
 * adbc:odbc:<data-source-name>[;param=value]*
 */

aorp_uri_spec2(adbc,odbc) {
	struct bo_charseq  dsn;
	struct bo_charseq  parm;
};

/* @} */

#endif /* !__BO_SERVICES_HILIS_URISPECIFICATIONS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
