/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Uritypes.h	(N.Panasiuk)
 * include/Bo/services/Uritypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_HILIS_URITYPES__
# define __BO_SERVICES_HILIS_URITYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>
# include <Bo/uriType.h>

# include <Bo/services/MIB/Services.h>
	/* AorpK_uri_service */

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Default uri encoding */
# define AorpK_URI_DEF_ENC BoK_UTF8_CSET

/* Path styles */
# define _AorpL_URI_PSTYLE_MAX   (5)
# define AorpK_URI_INVAL_PSTYLE  (-1)
# define AorpK_URI_UNK_PSTYLE    (0)
# define AorpK_URI_POSIX_PSTYLE  (1)
# define AorpK_URI_WIN_PSTYLE    (2)
# define AorpK_URI_HFS_PSTYLE    (3)
# define AorpK_URI_RFC_PSTYLE    (4)

/* Escape characters sets */
# define AorpK_URI_EMPTY_ESCCSET  (NULL)
# define AorpK_URI_DEF_ESCCSET    ((void *)1)
# define AorpK_URI_UNI_ESCCSET    ((void *)2)

/* Uri memory */
# define AorpK_URI_NULL_MEM (NULL)      /* create zero object */
# define AorpK_URI_SYS_MEM  ((void *)1) /* create object with system memory alloc */

/* ============================================================================
 * COMMON URI declarations
 * @{
 */

# ifndef AorpK_URI_OPAQUE_SIZE
#  define AorpK_URI_OPAQUE_SIZE (16)
# endif

struct aorp_uri {
# ifdef __cplusplus
	struct aorp_uri_header;
# endif

	struct aorp_uri_header {
		bo_cset_t          uri_enc;
		bo_uri_scheme_t    uri_scheme;
		bo_pointer_t       uri_spec;
		u_int16_t          uri_speclen;
		u_int16_t          uri_flags;
		char               uri_opaque[AorpK_URI_OPAQUE_SIZE];
	} _h;
};

# ifdef __cplusplus
#  define aorp_uri_header aorp_uri::aorp_uri_header
# endif

# define AORP_URI_HEADER_GUTS \
		bo_cset_t          uri_enc; \
		bo_uri_scheme_t    uri_scheme; \
		bo_pointer_t       uri_spec; \
		u_int16_t          uri_speclen; \
		u_int16_t          uri_flags; \
		char               uri_opaque[AorpK_URI_OPAQUE_SIZE] \

# define aorp_uri_p(x) ((struct aorp_uri *)(x))

/* ----------------------------------------------------------------------------
 * attributes
 * @{
 */

/*
bo_uri_scheme_t AORP_URI_Scheme(struct aorp_uri *anUri);
 */
# define AORP_URI_Scheme(anUri) (aorp_uri_p(anUri)->_h.uri_scheme)

/*
bo_cset_t AORP_URI_Enc(struct aorp_uri *anUri);
 */
# define AORP_URI_Enc(anUri) (aorp_uri_p(anUri)->_h.uri_enc)

/*
size_t AORP_URI_Speclen(struct aorp_uri *anUri);
 */
# define AORP_URI_Speclen(anUri) (aorp_uri_p(anUri)->_h.uri_speclen)

/*
bo_pointer_t _AORP_URI_Specdata(struct aorp_uri *anUri);
 */
# define _AORP_URI_Specdata(anUri) (aorp_uri_p(anUri)->_h.uri_spec)

/*
u_int16_t _AORP_URI_Flags(struct aorp_uri *anUri);
 */
# define _AORP_URI_Flags(anUri) (aorp_uri_p(anUri)->_h.uri_flags)

/* ------------------------------------------------------------------------- */

/*
bo_uri_scheme_t
AORP_URIOBJECT_Scheme(aorp_object_t anUri);
 */
# if __myc_support(macro_retval)
#  define AORP_URIOBJECT_Scheme(anUri) \
	__extension__ ({ \
		bo_pointer_t ___pod; \
		bo_uri_scheme_t ___scheme = BoK_URI_INVAL_SCHEME; \
		if (NULL != (___pod = _Art_podof(anUri,AorpK_uri_service,NULL))) \
			___scheme = AORP_URI_Scheme(___pod); \
		/* return */ (___scheme); \
	})
# else
__myc_extern_inline__ bo_uri_scheme_t
AORP_URIOBJECT_Scheme(aorp_object_t anUri)
{
	bo_pointer_t pod;
	bo_uri_scheme_t scheme = BoK_URI_INVAL_SCHEME;
	if (NULL != (pod = _Art_podof(anUri,AorpK_uri_service,NULL)))
		scheme = AORP_URI_Scheme(pod);
	return (scheme);
}
# endif

/*
bo_cset_t
AORP_URIOBJECT_Enc(aorp_object_t anUri);
 */
# if __myc_support(macro_retval)
#  define AORP_URIOBJECT_Enc(anUri) \
	__extension__ ({ \
		bo_pointer_t ___pod; \
		bo_cset_t ___cset = BoK_INVAL_CSET; \
		if (NULL != (___pod = _Art_podof(anUri,AorpK_uri_service,NULL))) \
			___cset = AORP_URI_Enc(___pod); \
		/* return */ (___cset); \
	})
# else
__myc_extern_inline__ bo_cset_t
AORP_URIOBJECT_Enc(aorp_object_t anUri)
{
	bo_pointer_t pod;
	bo_cset_t cset = BoK_INVAL_CSET;
	if (NULL != (pod = _Art_podof(anUri,AorpK_uri_service,NULL)))
		cset = AORP_URI_Enc(pod);
	return (cset);
}
# endif

/*
ssize_t
AORP_URIOBJECT_Speclen(aorp_object_t anUri);
 */
# if __myc_support(macro_retval)
#  define AORP_URIOBJECT_Speclen(anUri) \
	__extension__ ({ \
		bo_pointer_t ___pod; \
		ssize_t len = BoS_ERR; \
		if (NULL != (___pod = _Art_podof(anUri,AorpK_uri_service,NULL))) \
			len = (ssize_t)AORP_URI_Speclen(___pod); \
		/* return */ (len); \
	})
# else
__myc_extern_inline__ ssize_t
AORP_URIOBJECT_Speclen(aorp_object_t anUri)
{
	bo_pointer_t pod;
	ssize_t len = BoS_ERR;
	if (NULL != (pod = _Art_podof(anUri,AorpK_uri_service,NULL)))
		len = (ssize_t)AORP_URI_Speclen(pod);
	return (len);
}
# endif

/*
bo_pointer_t
_AORP_URIOBJECT_Specdata(aorp_object_t anUri);
 */
# if __myc_support(macro_retval)
#  define _AORP_URIOBJECT_Specdata(anUri) \
	__extension__ ({ \
		bo_pointer_t ___pod; \
		bo_pointer_t ___retval = NULL; \
		if (NULL != (___pod = _Art_podof(anUri,AorpK_uri_service,NULL))) \
			___retval = _AORP_URI_Specdata(___pod); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ bo_pointer_t
_AORP_URIOBJECT_Specdata(aorp_object_t anUri)
{
	bo_pointer_t pod;
	bo_pointer_t retval = NULL;
	if (NULL != (pod = _Art_podof(anUri,AorpK_uri_service,NULL)))
		retval = _AORP_URI_Specdata(pod);
	return (retval);
}
# endif

/* @} */

/* ----------------------------------------------------------------------------
 * specification
 * @{
 */

/*
 * 1. declare specification
 * 2. specification reference
 */

# define aorp_uri_spec(scheme) __aorp_uri_spec_typename(scheme)
# define aorp_uri_spec2(scheme, subscheme) \
	__aorp_uri_spec_typename2(scheme,subscheme)

/*
aorp_uri_spec(scheme) *
AORP_URI_Spec< scheme >(struct aorp_uri *anUri);
 */
# define AORP_URI_Spec(scheme, anUri) \
	((aorp_uri_spec(scheme) *) _AORP_URI_Specdata(anUri))

/*
aorp_uri_spec2(scheme, subscheme) *
AORP_URI_Spec2< scheme, subscheme >(struct aorp_uri *anUri);
 */
# define AORP_URI_Spec2(scheme, subscheme, anUri) \
	((aorp_uri_spec2(scheme,subscheme) *) _AORP_URI_Specdata(anUri))

/*
aorp_uri_spec(scheme) *
AORP_URIOBJECT_Spec< scheme >(aorp_object_t anUri);
 */
# define AORP_URIOBJECT_Spec(scheme, anUri) \
	((aorp_uri_spec(scheme) *) _AORP_URIOBJECT_Specdata(anUri))

/*
aorp_uri_spec2(scheme,subscheme) *
AORP_URIOBJECT_Spec2< scheme, subscheme >(aorp_object_t anUri);
 */
# define AORP_URIOBJECT_Spec2(scheme,subscheme, anUri) \
	((aorp_uri_spec2(scheme,subscheme) *) _AORP_URIOBJECT_Specdata(anUri))


# define __aorp_uri_spec_tag(scheme) __myc_xconcat3(aorp_uri_,scheme,_spec)
# define __aorp_uri_spec_tag2(scheme, subscheme) \
	__myc_xconcat5(aorp_uri_,scheme,_,subscheme,_spec)

# define __aorp_uri_spec_typename(scheme) struct __aorp_uri_spec_tag(scheme)
# define __aorp_uri_spec_typename2(scheme, subscheme) \
	struct __aorp_uri_spec_tag2(scheme,subscheme)

/* @} */

/* ============================================================================
 * URI-OPTIONS declarations
 * @{
 */

# define AorpF_URI_NCOPY_ON_CREATE (0x01)

/* AorpCreateUriWithString, AorpCreateUriWithWstring,
   AorpCreateUriWithFileSystemPath, AorpCreateUriWithFileSystemWpath,
   AorpUriGetstring, AorpUriGetstring_w */
# define AorpF_URI_DEL_BASENAME    (0x02)  /* return path without basename */
# define AorpF_URI_DEL_PATHEXT     (0x04)  /* return path without path extension */

/* AorpUriGetstring, AorpUriGetstring_w */
# define AorpF_URI_GET_FSPATH      (0x20)  /* returns file system path */
# define AorpF_URI_DEL_DIRNAME     (0x40)  /* returns path without dirname */

# define AorpF_URI_ALLOW_PDEVOLVE  (0x80)  /* allow the path to devolve */

struct aorp_uri_options {
	struct aorp_uri const  *base_uri;

	bo_cset_t               src_enc,
				            dst_enc;

	int                     src_pstyle,
	                        dst_pstyle;

	char const             *cset2esc,
				           *cset2unesc;

	int                     flags;
};

/* @} */

/* ============================================================================
 * URI-SCHEME-INFO declaration
 * @{
 */

struct aorp_uri_schemeinfo {
	size_t               ursi_levelc;
	bo_uri_scheme_t      ursi_id;
	char const          *ursi_rule;

	/* NOT FOR YOUR BUSINESS @{ */
	bo_rtbtelem_t        ursi_btelem1; /* enum-to-string */
	bo_rtbtelem_t        ursi_btelem2; /* string-to-enum */
	/* @} */

	char const          *ursi_subdelim; /* NULL if (1 == .ursi_levelc) */
	char const          *ursi_namev[1];
};

# define aorp_uri_schemeinfo_p(x) ((struct aorp_uri_schemeinfo *)(x))

# define AORP_URI_SCHEMEINFO_STRUCT(N) \
	struct __myc_xconcat(aorp_uri_schemeinfo,N) { \
		size_t               ursi_levelc; \
		bo_uri_scheme_t      ursi_id; \
		char const          *ursi_rule; \
		bo_rtbtelem_t        ursi_btelem1; \
		bo_rtbtelem_t        ursi_btelem2; \
		char const          *ursi_subdelims; \
		char const          *ursi_namev[N]; \
	}

typedef AORP_URI_SCHEMEINFO_STRUCT(1) aorp_uri_schemeinfo_t;
typedef AORP_URI_SCHEMEINFO_STRUCT(2) aorp_uri_schemeinfo2_t;

/*
 * make schemeinfo
 */

# define AORP_MAKE_URI_SCHEMEINFO(anID,aName,aRule) \
	{ \
		1, (anID), (aRule), \
		BO_RTBTELEM_ZERO_INITIALIZER, BO_RTBTELEM_ZERO_INITIALIZER, \
		NULL, {aName} \
	}

# define AORP_MAKE_URI_SCHEMEINFO2(anID,aName,aSubname,aSubdelim,aRule) \
	{ \
		2, (anID), (aRule), \
		BO_RTBTELEM_ZERO_INITIALIZER, BO_RTBTELEM_ZERO_INITIALIZER, \
		(aSubdelim), {(aName), (aSubname)} \
	}

/* ----------------------------------------------------------------------------
 * attributes
 * @{
 */

/*
size_t
AORP_URI_SCHEMEINFO_Levelc(struct aorp_uri_schemeinfo *aSchemeInfo);
 */
# define AORP_URI_SCHEMEINFO_Levelc(aSchemeInfo) \
	(aorp_uri_schemeinfo_p(aSchemeInfo)->ursi_levelc)

/*
bo_uri_scheme_t
AORP_URI_SCHEMEINFO_ID(struct aorp_uri_schemeinfo *aSchemeInfo);
 */
# define AORP_URI_SCHEMEINFO_ID(aSchemeInfo) \
	(aorp_uri_schemeinfo_p(aSchemeInfo)->ursi_id)

/*
char const **
AORP_URI_SCHEMEINFO_Namev(struct aorp_uri_schemeinfo *aSchemeInfo);
 */
# define AORP_URI_SCHEMEINFO_Namev(aSchemeInfo) \
	(aorp_uri_schemeinfo_p(aSchemeInfo)->ursi_namev)

/*
char const *
AORP_URI_SCHEMEINFO_Subdelim(struct aorp_uri_schemeinfo *aSchemeInfo);
 */
# define AORP_URI_SCHEMEINFO_Subdelim(aSchemeInfo) \
	(aorp_uri_schemeinfo_p(aSchemeInfo)->ursi_subdelim)

/*
char const *
AORP_URI_SCHEMEINFO_Rule(struct aorp_uri_schemeinfo *aSchemeInfo);
 */
# define AORP_URI_SCHEMEINFO_Rule(aSchemeInfo) \
	(aorp_uri_schemeinfo_p(aSchemeInfo)->ursi_rule)

/* @} */

#endif /* !__BO_SERVICES_HILIS_URITYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
