/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Log.h	(V.Sokur)
 * include/Bo/services/infos/Log.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_INFOS_LOG__
# define __BO_SERVICES_INFOS_LOG__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/infos/LogP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * The implementation may not do log-message duplicate when printing (retain(msg)).
 * Thus, you should not modify log-message after print/write/send-calls.
 */

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogOpenconn(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogOpenconn(aThis, anErrPtr) _A_call(openconn, log, aThis, anErrPtr, 1)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogOpenconn_nlk(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogOpenconn_nlk(aThis, anErrPtr) _A_call(openconn, log, aThis, anErrPtr, 0)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogReopenconn(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogReopenconn(aThis, anErrPtr) _A_call(reopenconn, log, aThis, anErrPtr, 1)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogReopenconn_nlk(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogReopenconn_nlk(aThis, anErrPtr) _A_call(reopenconn, log, aThis, anErrPtr, 0)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogCloseconn(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogCloseconn(aThis, anErrPtr) _A_call(closeconn, log, aThis, anErrPtr, 1)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogCloseconn_nlk(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogCloseconn_nlk(aThis, anErrPtr) _A_call(closeconn, log, aThis, anErrPtr, 0)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogSetprifilter(
	aorp_object_t aThis,
	unsigned aFilter,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogSetprifilter(aThis, aFilter, anErrPtr) \
	_A_call(setprifilter, log, aThis, anErrPtr, aFilter)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpLogGetprifilter(
	aorp_object_t aThis,
	unsigned *aFilterPtr,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogGetprifilter(aThis, aFilterPtr, anErrPtr) \
	_A_call(getprifilter, log, aThis, anErrPtr, aFilterPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogSetattrfilter(
	aorp_object_t aThis,
	unsigned aFilter,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogSetattrfilter(aThis, aFilter, anErrPtr) \
	_A_call(setattrfilter, log, aThis, anErrPtr, aFilter)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpLogGetattrfilter(
	aorp_object_t aThis,
	unsigned *aFilterPtr,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogGetattrfilter(aThis, aFilterPtr, anErrPtr) \
	_A_call(getattrfilter, log, aThis, anErrPtr, aFilterPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogPrintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	int aLevel,
	aorp_logmsg_t aMsg, // = NULL
	char const *aFormat,
	...
	);
*/
# if __myc_support(NVM)
#  define AorpLogPrintf(aThis, anErrPtr, aLevel, aMsg, aFormat...) \
		_A_call(printf, log, aThis, anErrPtr, aLevel, aMsg, aFormat)
# else
# if __myc_support(AVM)
#  define AorpLogPrintf(aThis, anErrPtr, aLevel, aMsg, /* aFormat, */...) \
		_A_call(printf, log, aThis, anErrPtr, aLevel, aMsg, /* aFormat, */__VA_ARGS__)
# endif /* __myc_support(AVM) */
# endif /* __myc_support(NVM) */

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogVprintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr,
	int aLevel,
	aorp_logmsg_t aMsg, // = NULL
	char const *aFormat,
	va_list ArgPtr
	);
*/
# define AorpLogVprintf(aThis, anErrPtr, aLevel, aMsg, aFormat, ArgPtr) \
	_A_call(vprintf, log, aThis, anErrPtr, aLevel, aMsg, aFormat, ArgPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogWprintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr,
	int aLevel,
	aorp_logmsg_t aMsg, // = NULL
	wchar_t const *aFormat,
	...
	);
*/
# if __myc_support(NVM)
#  define AorpLogWprintf(aThis, anErrPtr, aLevel, aMsg, aFormat...) \
		_A_call(wprintf, log, aThis, anErrPtr, aLevel, aMsg, aFormat)
# else
# if __myc_support(AVM)
#  define AorpLogWprintf(aThis, anErrPtr, aLevel, aMsg, /* aFormat, */...) \
		_A_call(wprintf, log, aThis, anErrPtr, aLevel, aMsg, /* aFormat, */__VA_ARGS__)
# endif /* __myc_support(AVM) */
# endif /* __myc_support(NVM) */

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpLogVwprintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	int aLevel,
	aorp_logmsg_t aMsg, // = NULL
	wchar_t const *aFormat,
	va_list ArgPtr
	);
*/
# define AorpLogVwprintf(aThis, anErrPtr, aLevel, aMsg, aFormat, ArgPtr) \
	_A_call(vwprintf, log, aThis, anErrPtr, aLevel, aMsg, aFormat, ArgPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpLogWrite(
	aorp_object_t aThis,
	aorp_logmsg_t aMsg,
	struct aorp_error *anErrPtr
	);
*/
# define AorpLogWrite(aThis, aMsg, anErrPtr) \
	_A_call(write, log, aThis, anErrPtr, aMsg)

/* ----------------------------------------------------------------------------
 * log message functions
 */
# pragma mark -

__myc_flinkage__(public,libBo) __aorp_logmsg_t *
AorpLogmsgOpen(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
AorpLogmsgClose(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1)))
AorpLogmsgClear(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1)))
AorpLogmsgCount(
# if __myc_require(prototypes)
	__aorp_logmsg_t const *aMsg,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
AorpLogmsgAdd(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	char const *aKey,
	char const *aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
AorpLogmsgReplace(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	char const *aKey,
	char const *aValue,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
AorpLogmsgRemove(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	char const *aKey,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) char const *
__attribute__((nonnull(1,2)))
AorpLogmsgValue(
# if __myc_require(prototypes)
	__aorp_logmsg_t const *aMsg,
	char const *aKey,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) char const *
__attribute__((nonnull(1)))
AorpLogmsgKey(
# if __myc_require(prototypes)
	__aorp_logmsg_t const *aMsg,
	bo_index_t anIndex,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,3,4)))
AorpLogmsgGet(
# if __myc_require(prototypes)
	__aorp_logmsg_t const *aMsg,
	bo_index_t anIndex,
	char const **aKeyPtr,
	char const **aValuePtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__myc_calltype__(mcall)
__attribute__((nonnull(1)))
AorpLogmsgRetain(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__myc_calltype__(mcall)
__attribute__((nonnull(1)))
AorpLogmsgRelease(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	aorp_opflags_t Flags __myc_defval__(0),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) aorp_logmsg_t
__attribute__((nonnull(1)))
AorpLogmsgDup(
# if __myc_require(prototypes)
	__aorp_logmsg_t *aMsg,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* Uninitialized copy */
__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
AorpLogmsgCopy(
# if __myc_require(prototypes)
	__aorp_logmsg_t const *aSource,
	__aorp_logmsg_t *aDest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * misc. log functions
 */
# pragma mark -

__myc_flinkage__(public,libBo) char const *
AorpLogLevelname(
# if __myc_require(prototypes)
	int
# endif
	);

__myc_flinkage__(public,libBo) char const *
AorpLogStdattrname(
# if __myc_require(prototypes)
	unsigned
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_INFOS_LOG__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
