/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Log2types.h	(N.Panasiuk)
 * include/Bo/services/infos/Log2types.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_INFOS_LOG2TYPES__
# define __BO_SERVICES_INFOS_LOG2TYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SERVICES_INFOS_LOGTYPES__
#  include <Bo/services/infos/Logtypes.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define AorpK_LOG2_INVAL_ID    (0)

/* Command constants @{ */

# define AorpK_LOG2_INVAL_CMD   (0)
# define AorpK_LOG2_END_CMD     (6)

# define AorpK_LOG2_RCVATTR_CMD (7)
# define AorpK_LOG2_RCVID_CMD   (8)
# define AorpK_LOG2_RCVLOG_CMD  (9)

# define AorpK_LOG2_INSLOG_CMD  (10)
# define AorpK_LOG2_SELLOG_CMD  (11)
# define AorpK_LOG2_UPDLOG_CMD  (12)
# define AorpK_LOG2_DELLOG_CMD  (13)

/* @} */

/* Key declarations @{ */

/* Key to declaration attribute list */
# define AORP_LOG2_ATTRLIST_BEGIN     ".attrlist"

/* Key to declaration attribute/value list */
# define AORP_LOG2_ATTRVALLIST_BEGIN  ".attrvallist"

/* Key to declaration condition */
# define AORP_LOG2_CONDLIST_BEGIN     ".condlist"

/* Condition predicates */
# define AORP_LOG2_AND_PRED           ".and"  /* && */
# define AORP_LOG2_OR_PRED            ".or"   /* || */
# define AORP_LOG2_NOT_PRED           ".not"  /* ! */
# define AORP_LOG2_LT_PRED            ".lt"   /* < */
# define AORP_LOG2_GT_PRED            ".gt"   /* > */
# define AORP_LOG2_LE_PRED            ".le"   /* <= */
# define AORP_LOG2_GE_PRED            ".ge"   /* >= */
# define AORP_LOG2_EQ_PRED            ".eq"   /* == */
# define AORP_LOG2_NE_PRED            ".ne"   /* != */
# define AORP_LOG2_LIKE_PRED          ".like"

# define AORP_LOG2_KEY_END            "."
/* @} */

/*
 * PACKET declaration
 */

struct aorp_log2pkt {
# ifdef __cplusplus
	struct aorp_log2pkt_header;
# endif

	struct aorp_log2pkt_header {
		u_int32_t  lpk_connid;
		u_int32_t  lpk_cmd;
	} __h;

/*
	specific for command
 */
};
# define aorp_log2pkt_p(x)      ((struct aorp_log2pkt *)(x))

# ifdef __cplusplus
#  define aorp_log2pkt_header   aorp_log2pkt::aorp_log2pkt_header
# endif


# define AORP_LOG2PKT_CONNID(x) (aorp_log2pkt_p(x)->__h.lpk_connid)
# define AORP_LOG2PKT_CMD(x)    (aorp_log2pkt_p(x)->__h.lpk_cmd)
# define AORP_LOG2PKT_SPEC(x)   ((char *)(x) + sizeof(struct aorp_log2pkt_header))

/* ============================================================================
 * ATTRIBUTES-SPECIFIC DEFINITIONS
 * @{
 */
# pragma mark -
# pragma mark +++ attributes-specific definitions +++
# pragma mark -

struct aorp_log2attr_spec {
	u_int16_t  lat_ident;
	u_int16_t  lat_session;
	u_int16_t  lat_facility;
	u_int16_t  lat_fname;
	unsigned   lat_opts;
	unsigned   lat_primask;
	unsigned   lat_attrmask;
	size_t     lat_reclen;
/*
	char       lat_data[FAM_LENGTH];
 */
};
# define aorp_log2attr_spec_p(x)      ((struct aorp_log2attr_spec *)(x))

/* Offset of fields */
# define AORP_LOG2ATTR_IDENT_OF(x)    (aorp_log2attr_spec_p(x)->lat_ident)
# define AORP_LOG2ATTR_SESSION_OF(x)  (aorp_log2attr_spec_p(x)->lat_session)
# define AORP_LOG2ATTR_FACILITY_OF(x) (aorp_log2attr_spec_p(x)->lat_facility)
# define AORP_LOG2ATTR_FNAME_OF(x)    (aorp_log2attr_spec_p(x)->lat_fname)

# define AORP_LOG2ATTR_IDENT(x)       ((char *)(x) + AORP_LOG2ATTR_IDENT_OF(x))
# define AORP_LOG2ATTR_SESSION(x)     ((char *)(x) + AORP_LOG2ATTR_SESSION_OF(x))
# define AORP_LOG2ATTR_FACILITY(x)    ((char *)(x) + AORP_LOG2ATTR_FACILITY_OF(x))
# define AORP_LOG2ATTR_FNAME(x)       ((char *)(x) + AORP_LOG2ATTR_FNAME_OF(x))
# define AORP_LOG2ATTR_OPTS(x)        (aorp_log2attr_spec_p(x)->lat_opts)
# define AORP_LOG2ATTR_PRIMASK(x)     (aorp_log2attr_spec_p(x)->lat_primask)
# define AORP_LOG2ATTR_ATTRMASK(x)    (aorp_log2attr_spec_p(x)->lat_attrmask)
# define AORP_LOG2ATTR_RECLEN(x)      (aorp_log2attr_spec_p(x)->lat_reclen)

/* @} */

/* ============================================================================
 * COMMON-SPECIFIC DEFINITIONS
 * @{
 */
# pragma mark -
# pragma mark +++ common-specific definitions +++
# pragma mark -

struct aorp_log2common_spec {
	u_int32_t  lcm_speclen;
	u_int16_t  lcm_pad;
	u_int16_t  lcm_nrecords;
/*
	char       lcm_data[FAM_LENGTH];
 */
};

# define AORP_LOG2COMMON_SPEC_STRUCT(N)             \
	struct __myc_xconcat(aorp_log2common_spec_,N) { \
		u_int32_t  lcm_speclen;                     \
		u_int16_t  lcm_pad;                         \
		u_int16_t  lcm_nrecords;                    \
		char       lcm_data[N];                     \
	}

# define aorp_log2common_spec_p(x)   ((struct aorp_log2common_spec *)(x))

# define AORP_LOG2COMMON_SPECLEN(x)  (aorp_log2common_spec_p(x)->lcm_speclen)
# define AORP_LOG2COMMON_NRECORDS(x) (aorp_log2common_spec_p(x)->lcm_nrecords)
# define AORP_LOG2COMMON_DATA(x)     ((char *)(x) + sizeof(struct aorp_log2common_spec))
# define AORP_LOG2COMMON_RECBUF(x)   AORP_LOG2COMMON_DATA(x)

/*
 * RECORD-PART declaration
 */

struct aorp_log2record_part {
	u_int32_t  lrc_partlen;
/*
	char       lrc_data[FAM_LENGTH];
 */	
};

# define AORP_LOG2RECORD_PART_STRUCT(N)             \
	struct __myc_xconcat(aorp_log2record_part_,N) { \
		u_int32_t  lrc_partlen;                     \
		char       lrc_data[N];                     \
	}

# define aorp_log2record_part_p(x)   ((struct aorp_log2record_part *)(x))

# define AORP_LOG2RECORD_PARTLEN(x)  (aorp_log2record_part_p(x)->lrc_partlen)
# define AORP_LOG2RECORD_DATA(x)     ((char *)(x) + sizeof(struct aorp_log2record_part))
# define AORP_LOG2RECORD_Next(x) \
	(aorp_log2record_part_p((char *)(x) + AORP_LOG2RECORD_PARTLEN(x)))

/*
 * KEY-PART declaration
 */

struct aorp_log2key_part {
	u_int16_t  lke_partlen;
	u_int16_t  lke_dlen;
/*
	char       lke_data[FAM_LENGTH];
 */
};

# define AORP_LOG2KEY_PART_STRUCT(N)             \
	struct __myc_xconcat(aorp_log2key_part_,N) { \
		u_int16_t  lke_partlen;                  \
		u_int16_t  lke_dlen;                     \
		char       lke_data[N];                  \
	}

# define aorp_log2key_part_p(x)      ((struct aorp_log2key_part *)(x))

# define AORP_LOG2KEY_PARTLEN(x)     (aorp_log2key_part_p(x)->lke_partlen)
# define AORP_LOG2KEY_DLEN(x)        (aorp_log2key_part_p(x)->lke_dlen)
# define AORP_LOG2KEY_DATA(x)        ((char *)(x) + sizeof(struct aorp_log2key_part))

/*
 * VALUE-PART declaration
 */

struct aorp_log2value_part {
	u_int16_t  lva_partlen;
	u_int16_t  lva_valds;
/*
	char       lva_data[FAM_LENGTH];
 */
};

# define AORP_LOG2VALUE_PART_STRUCT(N)             \
	struct __myc_xconcat(aorp_log2value_part_,N) { \
		u_int16_t  lva_partlen;                    \
		u_int16_t  lva_valds;                      \
		char       lva_data[N];                    \
	}

# define aorp_log2value_part_p(x)    ((struct aorp_log2value_part *)(x))

/* NOT FOR YOUR BUSINESS @{ */
# define _AorpK_LOG2VALUE_EOF         (0)

# define _AorpK_LOG2VALUE_LEN_MASK    (0x3FFF)
# define _AorpK_LOG2VALUE_FORM_MASK   (0xC000)

/*
 * Value data form: { text, binary-seq }
 */
# define _AorpF_LOG2VALUE_TEXT        (0x0000)
# define _AorpF_LOG2VALUE_BINSEQ      (0x8000)

	/* binary-seq-form: { binary-raw, binary-type }*/
# define _AorpF_LOG2VALUE_BINARY_RAW  (0x0000)
# define _AorpF_LOG2VALUE_BINARY_TYPE (0x4000)

# define _AorpK_LOG2VALUE_BINARY_RAW  _AorpF_LOG2VALUE_BINSEQ
# define _AorpK_LOG2VALUE_BINARY_TYPE (_AorpF_LOG2VALUE_BINSEQ | _AorpF_LOG2VALUE_BINARY_TYPE)
/* @} */

# define AORP_LOG2VALUE_PARTLEN(x) (aorp_log2value_part_p(x)->lva_partlen)


# define AORP_LOG2VALUE_IsText(x)  (!AORP_LOG2VALUE_IsBinseq(x))

# define AORP_LOG2VALUE_IsBinseq(x) \
	(_AorpF_LOG2VALUE_BINSEQ & aorp_log2value_part_p(x)->lva_valds)

# define AORP_LOG2VALUE_IsBinaryRaw(x) \
	(_AorpK_LOG2VALUE_BINARY_RAW == (_AorpK_LOG2VALUE_FORM_MASK & aorp_log2value_part_p(x)->lva_valds))

# define AORP_LOG2VALUE_IsBinaryType(x) \
	(_AorpK_LOG2VALUE_BINARY_TYPE == (_AorpK_LOG2VALUE_FORM_MASK & aorp_log2value_part_p(x)->lva_valds))


# define AORP_LOG2VALUE_ClearForm(x) _AORP_LOG2VALUE_ClearForm(x)
# define _AORP_LOG2VALUE_ClearForm(x) \
	(aorp_log2value_part_p(x)->lva_valds &= ~_AorpK_LOG2VALUE_FORM_MASK)

# define AORP_LOG2VALUE_MarkText(x) _AORP_LOG2VALUE_MarkText(x)
# define _AORP_LOG2VALUE_MarkText(x) _AORP_LOG2VALUE_ClearForm(x)

# define AORP_LOG2VALUE_MarkBinseq(x) \
	(_AORP_LOG2VALUE_ClearForm(x), _AORP_LOG2VALUE_MarkBinseq(x))
# define _AORP_LOG2VALUE_MarkBinseq(x) \
	(aorp_log2value_part_p(x)->lva_valds |= _AorpF_LOG2VALUE_BINSEQ)

# define AORP_LOG2VALUE_MarkBinaryRaw(x)  AORP_LOG2VALUE_MarkBinseq(x)
# define _AORP_LOG2VALUE_MarkBinaryRaw(x) _AORP_LOG2VALUE_MarkBinseq(x)

# define AORP_LOG2VALUE_MarkBinaryType(x) \
	(_AORP_LOG2VALUE_ClearForm(x), _AORP_LOG2VALUE_MarkBinaryType(x))
# define _AORP_LOG2VALUE_MarkBinaryType(x) \
	(aorp_log2value_part_p(x)->lva_valds |= _AorpF_LOG2VALUE_BINSEQ | _AorpF_LOG2VALUE_BINARY_TYPE)


/* Returns the exact value of len's bits. */
# define AORP_LOG2VALUE_GetLenRaw(x) \
	(aorp_log2value_part_p(x)->lva_valds & _AorpK_LOG2VALUE_LEN_MASK)

/* Returns value's len, so if len is set as a type constant, the size of
   this type will be returned. */
# define AORP_LOG2VALUE_GetLen(x)                                   \
	(!AORP_LOG2VALUE_IsBinaryType(x) ? AORP_LOG2VALUE_GetLenRaw(x)  \
	                                 : AORP_LOG2TYPE_Size(AORP_LOG2VALUE_GetLenRaw(x)))

/* Sets y to len's bits */
# define AORP_LOG2VALUE_SetLenRaw(x,y)                                  \
	(aorp_log2value_part_p(x)->lva_valds &= ~_AorpK_LOG2VALUE_LEN_MASK, \
		aorp_log2value_part_p(x)->lva_valds |= (u_int16_t)(y) & _AorpK_LOG2VALUE_LEN_MASK)

/* Sets type to len's bits */
# define AORP_LOG2VALUE_SetLenType(x,aTypeId) \
	(aorp_log2value_part_p(x)->lva_valds = ((u_int16_t)(aTypeId) & _AorpK_LOG2VALUE_LEN_MASK), \
		_AORP_LOG2VALUE_MarkBinaryType(x))
#if 0
# define AORP_LOG2VALUE_SetLenType(x,aTypeId) AORP_LOG2VALUE_SetLenRaw(x,aTypeId)
#endif

/*
 * Each of these macros sets a constant of a specific type
 */

# define AORP_LOG2VALUE_SetLenInt8(x)     AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_INT8)
# define AORP_LOG2VALUE_SetLenUInt8(x)    AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_UINT8)
# define AORP_LOG2VALUE_SetLenInt16(x)    AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_INT16)
# define AORP_LOG2VALUE_SetLenUInt16(x)   AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_UINT16)
# define AORP_LOG2VALUE_SetLenInt32(x)    AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_INT32)
# define AORP_LOG2VALUE_SetLenUInt32(x)   AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_UINT32)
# define AORP_LOG2VALUE_SetLenInt64(x)    AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_INT64)
# define AORP_LOG2VALUE_SetLenUInt64(x)   AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_UINT64)
/*
# define AORP_LOG2VALUE_SetLenInt128(x)   AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_INT128)
# define AORP_LOG2VALUE_SetLenUInt128(x)  AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_UINT128)
 */
# define AORP_LOG2VALUE_SetLenFloat32(x)  AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_FLOAT32)
# define AORP_LOG2VALUE_SetLenFloat64(x)  AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_FLOAT64)
# define AORP_LOG2VALUE_SetLenFloat128(x) AORP_LOG2VALUE_SetLenType(x,AorpK_LOG2TYPE_FLOAT128)


# define AORP_LOG2VALUE_DS_NetToHost(x)                              \
	do {                                                             \
		struct aorp_log2value_part *___x = aorp_log2value_part_p(x); \
		___x->lva_valds = BoNtohi16(___x->lva_valds);                \
	} while (0)

# define AORP_LOG2VALUE_DS_HostToNet(x)                              \
	do {                                                             \
		struct aorp_log2value_part *___x = aorp_log2value_part_p(x); \
		___x->lva_valds = BoHtoni16(___x->lva_valds);                \
	} while (0)

# define AORP_LOG2VALUE_DATA(x)   ((char *)(x) + sizeof(struct aorp_log2value_part))

# define AORP_LOG2VALUE_IsEof(x)  (_AorpK_LOG2VALUE_EOF == AORP_LOG2VALUE_PARTLEN(x))

# define AORP_LOG2VALUE_Next(x) \
	(aorp_log2value_part_p((char *)(x) + AORP_LOG2VALUE_PARTLEN(x)))

# define AORP_LOG2VALUE_Next_(x)                           \
	(AORP_LOG2VALUE_IsEof(AORP_LOG2VALUE_Next(x)) ? (NULL) \
	                                              : (AORP_LOG2VALUE_Next(x)))

/*
 * BINARY VALUES supported types
 */

# define AorpK_LOG2TYPE_UNK      (0)

# define AorpK_LOG2TYPE_INT8     (3)
# define AorpK_LOG2TYPE_UINT8    (4)
# define AorpK_LOG2TYPE_INT16    (5)
# define AorpK_LOG2TYPE_UINT16   (6)
# define AorpK_LOG2TYPE_INT32    (7)
# define AorpK_LOG2TYPE_UINT32   (8)
# define AorpK_LOG2TYPE_INT64    (9)
# define AorpK_LOG2TYPE_UINT64   (10)
/*
# define AorpK_LOG2TYPE_INT128   (11)
# define AorpK_LOG2TYPE_UINT128  (12)
 */
# define AorpK_LOG2TYPE_FLOAT32  (13)
# define AorpK_LOG2TYPE_FLOAT64  (14)
# define AorpK_LOG2TYPE_FLOAT128 (15)

# define AorpK_LOG2TYPE_COUNT    (16)


/*
 * u_int16_t AORP_LOG2TYPE_Size(u_int8_t &aType);
 */

# define AORP_LOG2TYPE_Size(aType) _im_AORP_LOG2TYPE_Size(aType)
__myc_extern_inline__ u_int16_t
_im_AORP_LOG2TYPE_Size(u_int8_t aType)
{
	static u_int16_t const sizeoftypev[AorpK_LOG2TYPE_COUNT] = {
		0,
		0,
		0,
		sizeof(int8_t),
		sizeof(u_int8_t),
		sizeof(int16_t),
		sizeof(u_int16_t),
		sizeof(int32_t),
		sizeof(u_int32_t),
		sizeof(int64_t),
		sizeof(u_int64_t),
		16/*sizeof(int128_t)*/,
		16/*sizeof(u_int128_t)*/,
		sizeof(float),
		sizeof(double),
		sizeof(long double),
	};
	return (sizeoftypev[aType]);
}


/*
 * BINARY VALUES definitions for byte sequence values.
 */
/* NOTE: If the binary value is a byte sequence, it will be written in log in hex 
   form of the format $(hex:<len>([string containing the hex value of each byte
   (2 symbols for each byte)])). <len> is the number of characters of the
   text-byte-sequence, <len> should occupy AorpK_LOG2VALUE_BSEQLEN_SIZE bytes.
   EXAMPLE: $(hex:00004(0095))*/
# define AorpK_LOG2VALUE_BSEQLEN_SIZE (5)

/* @} */

#endif /* !__BO_SERVICES_INFOS_LOG2TYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
