/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * LogP.h	(V.Sokur)
 * include/Bo/services/infos/LogP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_INFOS_LOG_P__
# define __BO_SERVICES_INFOS_LOG_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/infos/Logtypes.h>

# include <Bo/libc/Stdarg.h>

/******************************************************************************
 * %% BeginSection: log service declarations
 */
# pragma mark -
# pragma mark [ log service declarations ]
# pragma mark -

# define __aorpc_need_service_log 1


/* NOT FOR YOUR BUSINESS @{ */

# define _SlogcliK_INVAL_STATE     (-1)
# define _SlogcliK_NULL_STATE      (0)
# define _SlogcliK_UNK_STATE       (2)
# define _SlogcliK_INITED_STATE    (3) /* ->: open */
# define _SlogcliK_OPENING_STATE   (4)
# define _SlogcliK_OPENED_STATE    (5)
# define _SlogcliK_CLOSING_STATE   (6)
# define _SlogcliK_CLOSED_STATE    _SlogcliK_INITED_STATE
# define _SlogcliK_FINISHING_STATE (7)

# define _SlogsrvK_INVAL_STATE     (-1)
# define _SlogsrvK_NULL_STATE      (0)
# define _SlogsrvK_UNK_STATE       (2)
# define _SlogsrvK_INITED_STATE    (3) /* ->: open */
# define _SlogsrvK_OPENING_STATE   (4)
# define _SlogsrvK_OPENED_STATE    (5)
# define _SlogsrvK_CLOSING_STATE   (6)
# define _SlogsrvK_CLOSED_STATE    _SlogsrvK_INITED_STATE
# define _SlogsrvK_FINISHING_STATE (7)

/* @} */

#endif /* !__BO_SERVICES_INFOS_LOG_P__ */

#pragma mark -

#if __aorpc_uses_service(log)
# undef __aorpc_need_service_log

/*
 * log service
 * @{
 */

/*
 * log {\object(instance)}
 */
/*
# define AorpK_log_service (1190)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name log
# include <Bo/aorp/AorpCsrvc.h>

_A_service(log)
_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(openconn, "open-connection-to-storage", (
			struct aorp_closure *aClosure,
			int WithLock,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(reopenconn, "reopen-connection-to-storage", (
			struct aorp_closure *aClosure,
			int WithLock,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(closeconn, "close-connection-to-storage", (
			struct aorp_closure *aClosure,
			int WithLock,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(setprifilter, "set-priority-filter", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aFilter
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(getprifilter, "get-priority-filter", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned *aFilterPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(setattrfilter, "set-attribute-filter", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aFilter
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(getattrfilter, "get-attribute-filter", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned *aFilterPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(printf, "printf-log-message", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aLevel,
			aorp_logmsg_t aMsg /* = NULL */,
			char const *aFormat,
			...
		)),

	_A_attribute((nonnull(1)))
	_A_op(vprintf, "vprintf-log-message", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aLevel,
			aorp_logmsg_t aMsg /* = NULL */,
			char const *aFormat,
			va_list ArgPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(wprintf, "wprintf-log-message", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aLevel,
			aorp_logmsg_t aMsg /* = NULL */,
			wchar_t const *aFormat,
			...
		)),

	_A_attribute((nonnull(1)))
	_A_op(vwprintf, "vwprintf-log-message", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aLevel,
			aorp_logmsg_t aMsg /* = NULL */,
			wchar_t const *aFormat,
			va_list ArgPtr
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(write, "write-log-message", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_logmsg_t aMsg
		)),

	_A_attribute((nonnull(1)))
	_A_op(_send, "send-log-msg", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_logmsg_t aMsg
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(log) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
