/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Logtypes.h	(V.Sokur)
 * include/Bo/services/infos/Logtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_INFOS_LOGTYPES__
# define __BO_SERVICES_INFOS_LOGTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/sys/Paths.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/*
is-server(output):
	(!fname):
		(ident and session):
			fname = ident-session.log
		else:
			fname = NULL <# server-log-file-default #>
is-file(output):
	(!fname):
		(ident and session):
			fname = ident-session.log
		else:
			fname = local-log-file-default
 */

struct aorp_logattr {
	char const *ident;     /* [default: NUL] */
	char const *session;   /* [default: NUL] */

	/* describes the part of the system generating the message. [default: "User"] */
	char const *facility;

	char const *fname;
	unsigned    opts;
	unsigned    primask;
	unsigned    attrmask;
};

typedef struct aorp_logattr aorp_logattr_t;

struct aorp_logattr_x {
	u_int16_t   ident;
	u_int16_t   session;
	u_int16_t   facility;
	u_int16_t   fname;
	unsigned    opts;
	unsigned    primask;
	unsigned    attrmask;
	size_t      reclen;
};


struct aorp_logmsg_s;
typedef struct aorp_logmsg_s *aorp_logmsg_t, __aorp_logmsg_t;


typedef int64_t aorp_logid_t;


# define AORP_LOG_FACILITY_DEFAULT     "User"
# define AORP_LOG_SERVER_FNAME_DEFAULT "bosl.log"
# define AORP_LOG_LOCAL_FNAME_DEFAULT  "local.log" /* <process-name>.log, <pid>.log */


/*
 * LOG Open options
 */

# define AorpF_LOG_CONS        0x0001
	/* Attempt to write message to the system console if the message
	   cannot be sent to the server. */

# define AorpF_LOG_NDELAY      0x0002
	/* Open the connection to the log server immediately. */

# define AorpF_LOG_POUT        0x0004
	/* Send messages to the standard output as well as to the server (or file). */

# define AorpF_LOG_PERR        0x0008
	/* Send messages to the standard error as well as to the server (or file). */

# define AorpF_LOG_SERVER      0
# define AorpF_LOG_FILE        0x0100
# define AorpF_LOG_STDOUT      0x0200
# define AorpF_LOG_STDERR      0x0300
# define AorpK_LOG_OUT_MASK    0x0300


/*
 * LOG level of the message
 */

# define AorpK_LOG_EMERG       0 /* A panic condition. */
# define AorpK_LOG_ALERT       1 /* A condition that should be corrected immediately. */
# define AorpK_LOG_CRIT        2 /* Critical errors: hardware, ... */
# define AorpK_LOG_ERR         3
# define AorpK_LOG_WARN        4
# define AorpK_LOG_NOTICE      5 /* Not error, but special handling is required. */
# define AorpK_LOG_INFO        6
# define AorpK_LOG_DEBUG       7

/* NOT FOR YOUR BUSINESS @{ */
# define _AorpK_LOG_PRIVATE1   255
/* @} */

/* Corresponding level strings */
# define AORP_LOG_EMERG        "Emergency"
# define AORP_LOG_ALERT        "Alert"
# define AORP_LOG_CRIT         "Critical"
# define AORP_LOG_ERR          "Error"
# define AORP_LOG_WARN         "Warning"
# define AORP_LOG_NOTICE       "Notice"
# define AORP_LOG_INFO         "Info"
# define AORP_LOG_DEBUG        "Debug"


/*
 * Macros to handle priority bitmask
 */
# define AORP_LOG_MAKE_PRIMASK(aLevel)        (1 << ((aLevel) & 0x7))
# define AORP_LOG_PRIMASK_SET(aMask,aLevel)   ((aMask) |= AORP_LOG_MAKE_PRIMASK(aLevel))
# define AORP_LOG_PRIMASK_CLEAR(aMask,aLevel) ((aMask) &= ~AORP_LOG_MAKE_PRIMASK(aLevel))

/*
 * Individual priorities of the message
 */
# define AorpF_LOG_EMERG_PRI   0x01
# define AorpF_LOG_ALERT_PRI   0x02
# define AorpF_LOG_CRIT_PRI    0x04
# define AorpF_LOG_ERR_PRI     0x08
# define AorpF_LOG_WARN_PRI    0x10
# define AorpF_LOG_NOTICE_PRI  0x20
# define AorpF_LOG_INFO_PRI    0x40
# define AorpF_LOG_DEBUG_PRI   0x80

# define AorpK_LOG_PRIMASK_DEFAULT \
	( AorpF_LOG_EMERG_PRI | AorpF_LOG_ALERT_PRI | AorpF_LOG_CRIT_PRI | \
		AorpF_LOG_ERR_PRI | AorpF_LOG_WARN_PRI | AorpF_LOG_NOTICE_PRI )

# define AorpK_LOG_PRIMASK_ALL 0xFF


/*
 * Attributes, attached to the log message.
 */

# define AorpK_LOG_IDENT_ATTR     0  /* Set automatically. */
# define AorpK_LOG_FACILITY_ATTR  1  /* Set automatically. */
# define AorpK_LOG_TIME_ATTR      2  /* Timestamp (as ctime). Set automatically. */
# define AorpK_LOG_HOST_ATTR      3  /* Sender's address (set by the server). */
# define AorpK_LOG_SENDER_ATTR    4  /* Sender's identification string. Default is a process name. */
# define AorpK_LOG_PID_ATTR       5  /* Process ID encoded as a string.  Set automatically. */
# define AorpK_LOG_UID_ATTR       6  /* User ID encoded as a string. Set automatically (set by the server?). */
# define AorpK_LOG_GID_ATTR       7  /* Group ID encoded as a string. Set automatically (set by the server?). */
# define AorpK_LOG_LEVEL_ATTR     8  /* Log level encoded as a string. */
# define AorpK_LOG_MSG_ATTR       9  /* Actual message string that will be logged. */
# define AorpK_LOG_STDATTR_COUNT  10

# define AorpF_LOG_IDENT_ATTR     0x0001
# define AorpF_LOG_FACILITY_ATTR  0x0002
# define AorpF_LOG_TIME_ATTR      0x0010
# define AorpF_LOG_HOST_ATTR      0x0020
# define AorpF_LOG_SENDER_ATTR    0x0040
# define AorpF_LOG_PID_ATTR       0x0080
# define AorpF_LOG_UID_ATTR       0x0100
# define AorpF_LOG_GID_ATTR       0x0200
# define AorpF_LOG_LEVEL_ATTR     0x0400
# define AorpF_LOG_MSG_ATTR       0x0800

# define AorpK_LOG_ATTRMASK_DEFAULT \
	( AorpF_LOG_TIME_ATTR | AorpF_LOG_HOST_ATTR | AorpF_LOG_SENDER_ATTR | \
		AorpF_LOG_PID_ATTR | AorpF_LOG_LEVEL_ATTR | AorpF_LOG_MSG_ATTR )

# define AorpK_LOG_ATTRMASK_ALL   0xFFFF

# define AORP_LOG_IDENT_ATTR      "Ident"
# define AORP_LOG_FACILITY_ATTR   "Facility"
# define AORP_LOG_TIME_ATTR       "Time"
# define AORP_LOG_HOST_ATTR       "Host"
# define AORP_LOG_SENDER_ATTR     "Sender"
# define AORP_LOG_PID_ATTR        "PID"
# define AORP_LOG_UID_ATTR        "UID"
# define AORP_LOG_GID_ATTR        "GID"
# define AORP_LOG_LEVEL_ATTR      "Level"
# define AORP_LOG_MSG_ATTR        "Message"


#define AorpK_LOG_SERVER_PORT_DEFAULT (27016)

/******************************************************************************
 * %% BeginSection: communication type declarations
 */
# pragma mark -
# pragma mark [ communication type declarations ]
# pragma mark -

struct aorp_logheader {
	u_int32_t  connid;
	u_int32_t  cmd;
};

typedef struct aorp_logheader aorp_logheader_t;

/* command constants @{ */

# define AorpK_LOG_RCVATTR_CMD (3)
# define AorpK_LOG_RCVID_CMD   (4)
# define AorpK_LOG_RCVLOG_CMD  (5)
# define AorpK_LOG_END_CMD     (6)

# define AorpK_LOG_INVAL_CMD   (0)
# define AorpK_LOG_INVAL_ID    (0)

/* @} */

# define AorpK_LOG_TIMEOUT_DEFAULT (3600 * 1000000LL) /* usec */

#endif /* !__BO_SERVICES_INFOS_LOGTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
