/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Cmniosopt.h	(V.Sokur)
 * include/Bo/services/io/Cmniosopt.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_CMNIOSOPT__
# define __BO_SERVICES_IO_CMNIOSOPT__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SERVICES_IO_IOSTYPES__
#  include <Bo/services/io/Iostypes.h>
# endif

/******************************************************************************
 * %% BeginSection: constant/type declarations
 */
# pragma mark -
# pragma mark [ constant/type declarations ]
# pragma mark -

/*
 * option domain (bo_opthdr.domain)
 */
# ifndef BoK_CMNIOS_OPT
#  define BoK_CMNIOS_OPT __myc_mk4ccs("gios")
#  define _BoK_CMNIOS_OPT __myc_mk4cc('g','i','o','s')
# endif

/*
 * values for option name (bo_opthdr.name)
 */

/* [0, 127] */
# define AorpK_IOS_OPTCTL 0   /* args for ios::optctl operation */
# define AorpK_IOS_OPTBIND 1  /* args for ios::bind operation */

/* [128, 255] */
# define AorpK_GOPTPROP 128   /* any-operation */
# define AorpK_GOPTLLAPI 129  /* any-operation */
# define AorpK_GOPTPARM 130   /* any-operation */

/* ----------------------------------------------------------------------------
 * GENERIC-OPTCTL OPTION
 * ios::optctl
 */
# pragma mark -

/*
 * hdr.domain = {BoK_CMNIOS_OPT,...}
 * hdr.name = {AorpK_IOS_OPTCTL}
 */

struct aorp_goptctl_data {
	unsigned  argnam;
};
# define aorp_goptctl_data_p(x) ((struct aorp_goptctl_data *)(x))

struct aorp_goptctl_idata {
	unsigned  argnam;
	union {
		intptr_t      ip;
		u_int32_t     u32;
		u_int16_t     u16[2];
		u_int8_t      u8[4];
	}  argval;
};

struct aorp_goptctl_bdata {
	unsigned           argnam;
	struct bo_nabsbuf  argval;
};

BO_OPT_STRUCT(aorp_goptctli_s, struct aorp_goptctl_idata);
BO_OPT_STRUCT(aorp_goptctlb_s, struct aorp_goptctl_bdata);

# define AORP_GOPTCTLI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptctl_idata, anOption, _BoK_CMNIOS_OPT, AorpK_IOS_OPTCTL)

# define AORP_GOPTCTLB_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptctl_bdata, anOption, _BoK_CMNIOS_OPT, AorpK_IOS_OPTCTL)

/*
 * values for aorp_goptctl.argnam
 */

# define AorpK_GOPTC_CLIENT_MAX 127
	/* aorp_goptctl_idata: {bino:u_int16_t, client-max:u_int16_t} */

# define AorpK_GOPTC_CLIENT_MAX_LIST 126
	/* aorp_goptctl_bdata: elemc * {bino:u_int16_t, client-max:u_int16_t} */

# define AorpK_GOPTC_SYNCIO 125
	/* aorp_goptctl_idata: {u_int8_t};
	   arg=0: async i/o;
	   arg=1: sync i/o. */

/* ----------------------------------------------------------------------------
 * GENERIC-[GET/SET]PROP OPTION
 * ios::control(getprop,setprop)
 * any-operation
 */
# pragma mark -

/*
 * hdr.domain = {BoK_CMNIOS_OPT,...}
 * hdr.name = {AorpK_GOPTPROP, AorpK_IosCntl_Getprop, AorpK_IosCntl_Setprop}
 */

struct aorp_goptprop_data {
	unsigned  propnam;
};
# define aorp_goptprop_data_p(x) ((struct aorp_goptprop_data *)(x))

struct aorp_goptprop_idata {
	unsigned  propnam;
	union {
		intptr_t      ip;
		u_int32_t     u32;
		u_int16_t     u16[2];
		u_int8_t      u8[4];
	}  propval;
};

struct aorp_goptprop_bdata {
	unsigned           propnam;
	struct bo_nabsbuf  propval;
};

BO_OPT_STRUCT(aorp_goptpropi_s, struct aorp_goptprop_idata);
BO_OPT_STRUCT(aorp_goptpropb_s, struct aorp_goptprop_bdata);

# define AORP_GOPTPROPI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptprop_idata, anOption, _BoK_CMNIOS_OPT, AorpK_GOPTPROP)

# define AORP_GOPTPROPB_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptprop_bdata, anOption, _BoK_CMNIOS_OPT, AorpK_GOPTPROP)

# define AORP_GOPT_GETPROPI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptprop_idata, anOption, _BoK_CMNIOS_OPT, AorpK_IosCntl_Getprop)

# define AORP_GOPT_SETPROPI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptprop_idata, anOption, _BoK_CMNIOS_OPT, AorpK_IosCntl_Setprop)

# define AORP_GOPT_GETPROPB_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptprop_bdata, anOption, _BoK_CMNIOS_OPT, AorpK_IosCntl_Getprop)

# define AORP_GOPT_SETPROPB_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptprop_bdata, anOption, _BoK_CMNIOS_OPT, AorpK_IosCntl_Setprop)

/*
 * values for aorp_goptprop.propnam
 */

# define AorpK_GPROP_MAXBINOCOUNT 127
	/* aorp_goptprop_idata: {u_int16_t};
	   readonly */

# define AorpK_GPROP_BINOCOUNT 126
	/* aorp_goptprop_idata: {u_int16_t};
	   readonly */

# define AorpK_GPROP_BINOCLIENTMAX 125
	/* aorp_goptprop_idata: {u_int16_t};
	   operation: ios::bind */

# define AorpK_GPROP_BINOCLIENTCOUNT 124
	/* aorp_goptprop_idata: {bino:u_int16_t, client-count:u_int16_t};
	   (0 == bino) => sum of all clients of all binds;
	   readonly */

# define AorpK_GPROP_BINOCLIENTCOUNT_LIST 123
	/* aorp_goptprop_bdata: elemc * {bino:u_int16_t, client-count:u_int16_t};
	   readonly */

# define AorpK_GPROP_GETBINDS 122
	/* aorp_goptprop_bdata: elemc * {bino:u_int16_t};
	   readonly */

/* ----------------------------------------------------------------------------
 * LLAPI OPTION
 * ios::control(QueryLLAPI)
 * any-operation
 */
# pragma mark -

/*
 * hdr.domain = {BoK_CMNIOS_OPT,...}
 * hdr.name = {AorpK_GOPTLLAPI, AorpK_IosCntl_QueryLLAPI}
 */

struct aorp_goptllapi_data {
	bo_uuid_t         api_id;
	bo_pointer_t      api_obj; /* result; IUnknown* */
};

BO_OPT_STRUCT(aorp_goptllapi_s, struct aorp_goptllapi_data);

# define AORP_GOPTLLAPI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptllapi_data, anOption, _BoK_CMNIOS_OPT, AorpK_GOPTLLAPI)

# define AORP_GOPT_QUERYLLAPI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptllapi_data, anOption, _BoK_CMNIOS_OPT, AorpK_IosCntl_QueryLLAPI)

/* ----------------------------------------------------------------------------
 * GENERIC-PARM OPTION
 * any-operation
 */
# pragma mark -

/*
 * hdr.domain = {BoK_CMNIOS_OPT,...}
 * hdr.name = {AorpK_GOPTPARM}
 */

struct aorp_goptparm_data {
	unsigned  parmnam;
};
# define aorp_goptparm_data_p(x) ((struct aorp_goptparm_data *)(x))

struct aorp_goptparm_idata {
	unsigned  parmnam;
	union {
		intptr_t      ip;
		u_int32_t     u32;
		u_int16_t     u16[2];
		u_int8_t      u8[4];
	}  parmval;
};

struct aorp_goptparm_bdata {
	unsigned           parmnam;
	struct bo_nabsbuf  parmval;
};

BO_OPT_STRUCT(aorp_goptparmi_s, struct aorp_goptparm_idata);
BO_OPT_STRUCT(aorp_goptparmb_s, struct aorp_goptparm_bdata);

# define AORP_GOPTPARMI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptparm_idata, anOption, _BoK_CMNIOS_OPT, AorpK_GOPTPARM)

# define AORP_GOPTPARMB_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_goptparm_bdata, anOption, _BoK_CMNIOS_OPT, AorpK_GOPTPARM)

/*
 * values for aorp_goptparm.parmnam
 */

# define AorpK_GPARM_BINO 127
	/* aorp_goptparm_idata: {u_int16_t} */

#endif /* !__BO_SERVICES_IO_CMNIOSOPT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
