/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Fiosopt.h	(V.Sokur)
 * include/Bo/services/io/Fiosopt.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_FIOSOPT__
# define __BO_SERVICES_IO_FIOSOPT__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SERVICES_IO_CMNIOSOPT__
#  include <Bo/services/io/Cmniosopt.h>
# endif

/******************************************************************************
 * %% BeginSection: constant/type declarations
 */
# pragma mark -
# pragma mark [ constant/type declarations ]
# pragma mark -

/*
 * option domain (bo_opthdr.domain)
 */
# ifndef BoK_FIOS_OPT
#  define BoK_FIOS_OPT __myc_mk4ccs("fios")
#  define _BoK_FIOS_OPT __myc_mk4cc('f','i','o','s')
# endif

/* ----------------------------------------------------------------------------
 * FILE-OPTCTL OPTION
 * ios::optctl
 */
# pragma mark -

/*
 * hdr.domain = BoK_FIOS_OPT
 * hdr.name = {AorpK_IOS_OPTCTL}
 */

struct aorp_foptctl_data {
	int       argnam;
	intptr_t  argval; /* result */
};

BO_OPT_STRUCT(aorp_foptctl_s, struct aorp_foptctl_data);

# define AORP_FOPTCTL_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_foptctl_data, anOption, _BoK_FIOS_OPT, AorpK_IOS_OPTCTL)

/*
 * values for aorp_foptctl.argnam
 */

# define AorpK_FOPTC_CLEX 1
	/* int;
	   arg=0: clear close-on-exec flag;
	   arg=1: set close-on-exec flag;
	   Corresponds to the O_NOINHERIT flag of the open. */

# define AorpK_FOPTC_DFLAGS 127
	/* int;
	   get/set status flags of the underlying [system] descriptor.
	   The [possible] flags are as follows: O_NONBLOCK, O_APPEND [, O_ASYNC]. */

# define AorpK_FOPTC_APPEND 126
	/* int;
	   Corresponds to the O_APPEND flag of the open. */

# define AorpK_FOPTC_NBIO 125
	/* int;
	   arg=0: clear non-blocking i/o;
	   arg=1: set non-blocking i/o. */

/* ----------------------------------------------------------------------------
 * FILE-BIND OPTION
 * ios::bind
 */
# pragma mark -

/*
 * hdr.domain = BoK_FIOS_OPT
 * hdr.name = {AorpK_IOS_OPTBIND}
 */

struct aorp_foptbind_data {
	int        oflags;
	bo_mode_t  perm;
};

BO_OPT_STRUCT(aorp_foptbind_s, struct aorp_foptbind_data);

# define AORP_FOPTBIND_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_foptbind_data, anOption, _BoK_FIOS_OPT, AorpK_IOS_OPTBIND)

/* ----------------------------------------------------------------------------
 * FILE-SEEK OPTION
 * ios::control(Seek)
 */
# pragma mark -

/*
 * hdr.domain = BoK_FIOS_OPT
 * hdr.name = {AorpK_IosCntl_Seek}
 */

struct aorp_foptseek_data {
	aorp_seekorigin_t  whence;
	bo_off_t           offset;
};

BO_OPT_STRUCT(aorp_foptseek_s, struct aorp_foptseek_data);

# define AORP_FOPTSEEK_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_foptseek_data, anOption, _BoK_FIOS_OPT, AorpK_IosCntl_Seek)

/* ----------------------------------------------------------------------------
 * FILE-STAT OPTION
 * ios::control(Stat)
 */
# pragma mark -

/*
 * hdr.domain = BoK_FIOS_OPT
 * hdr.name = {AorpK_IosCntl_Stat}
 */

#if 0

/*
 * SHOULD BE DECLARED BY A CLIENT ANYWHERE
 * #include <Bo/sys/Stat.h>
 */
struct aorp_foptstat_data {
	struct bo_stat  sb;
};

BO_OPT_STRUCT(aorp_foptstat_s, struct aorp_foptstat_data);

#endif

# define AORP_FOPTSTAT_STRUCT \
	struct aorp_foptstat_data { \
		struct bo_stat  sb; \
	}; \
	BO_OPT_STRUCT(aorp_foptstat_s, struct aorp_foptstat_data)

# define AORP_FOPTSTAT_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_foptstat_data, anOption, _BoK_FIOS_OPT, AorpK_IosCntl_Stat)

#endif /* !__BO_SERVICES_IO_FIOSOPT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
