/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Iopump.h	(V.Sokur)
 * include/Bo/services/io/Iopump.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_IOPUMP__
# define __BO_SERVICES_IO_IOPUMP__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/io/IopumpP.h>

# include <Bo/services/io/Ios.h>
# include <Bo/services/io/MUI.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
aorp_object_t
__attribute__((nonnull(1)))
AorpIopumpGetport(
	aorp_object_t aThis,
	int aPortType,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned object is not retained.
 */

# if __myc_support(macro_retval)
#  define AorpIopumpGetport(aThis, aPortType, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(getport, iopump, aThis, anErrPtr, aPortType); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_AOB_FROM_STATUS(___retval) : BoS_EAOB); \
	})
# else
__myc_extern_inline__ aorp_object_t
AorpIopumpGetport(aorp_object_t aThis, int aPortType, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(getport, iopump, aThis, anErrPtr, aPortType);
	return (BoSP_NORMAL(retval) ? BO_AOB_FROM_STATUS(retval) : BoS_EAOB);
}
# endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIopumpAddport(
	aorp_object_t aThis,
	aorp_object_t aPort,
	int aPortType,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The object referenced by the parameter @port is retained.
 */
# define AorpIopumpAddport(aThis, aPort, aPortType, anErrPtr) \
	_A_call(addport, iopump, aThis, anErrPtr, aPort, aPortType)

# pragma mark -

/*
aorp_object_t
__attribute__((nonnull(1)))
AorpIopumpRmport(
	aorp_object_t aThis,
	int aPortType,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned object is not released.
 */

# if __myc_support(macro_retval)
#  define AorpIopumpRmport(aThis, aPortType, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(rmport, iopump, aThis, anErrPtr, aPortType); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_AOB_FROM_STATUS(___retval) : BoS_EAOB); \
	})
# else
__myc_extern_inline__ aorp_object_t
AorpIopumpRmport(aorp_object_t aThis, int aPortType, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(rmport, iopump, aThis, anErrPtr, aPortType);
	return (BoSP_NORMAL(retval) ? BO_AOB_FROM_STATUS(retval) : BoS_EAOB);
}
# endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIopumpDelport(
	aorp_object_t aThis,
	int aPortType,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The object referenced by the parameter @port-type is released.
 */
# define AorpIopumpDelport(aThis, aPortType, anErrPtr) \
	_A_call(delport, iopump, aThis, anErrPtr, aPortType)

# pragma mark -

/*
aorp_object_t
__attribute__((nonnull(1)))
AorpIopumpGetiod(
	aorp_object_t aThis,
	u_int32_t aDevId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned object is not retained.
 */

# if __myc_support(macro_retval)
#  define AorpIopumpGetiod(aThis, aDevId, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(getiod, iopump, aThis, anErrPtr, (u_int32_t)(aDevId)); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_AOB_FROM_STATUS(___retval) : BoS_EAOB); \
	})
# else
__myc_extern_inline__ aorp_object_t
AorpIopumpGetiod(aorp_object_t aThis, u_int32_t aDevId, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(getiod, iopump, aThis, anErrPtr, aDevId);
	return (BoSP_NORMAL(retval) ? BO_AOB_FROM_STATUS(retval) : BoS_EAOB);
}
# endif

# pragma mark -

/*
int32_t
__attribute__((nonnull(1)))
AorpIopumpAttiod(
	aorp_object_t aThis,
	int aPortType,
	unsigned aBino, // = 0; 1-based, 0 means index chosen by implementation
	aorp_object_t aDev,
	struct bo_options *Options,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The object referenced by the parameter @dev is retained.
 */
# define AorpIopumpAttiod(aThis, aPortType, aBino, aDev, Options, anErrPtr) \
	((int32_t) _A_call(attiod, iopump, aThis, anErrPtr, aPortType, aBino, aDev, Options))

# pragma mark -

/*
int32_t
__attribute__((nonnull(1)))
AorpIopumpReattiod(
	aorp_object_t aThis,
	u_int32_t aDevId,
	aorp_object_t aDev,
	struct bo_options *Options,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The object referenced by the parameter @dev is retained.
 */
# define AorpIopumpReattiod(aThis, aDevId, aDev, Options, anErrPtr) \
	((int32_t) _A_call(reattiod, iopump, aThis, anErrPtr, (u_int32_t)(aDevId), aDev, Options))

# pragma mark -

/*
aorp_object_t
__attribute__((nonnull(1)))
AorpIopumpDetiod(
	aorp_object_t aThis,
	u_int32_t aDevId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned object is not released.
 */

# if __myc_support(macro_retval)
#  define AorpIopumpDetiod(aThis, aDevId, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(detiod, iopump, aThis, anErrPtr, (u_int32_t)(aDevId)); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_AOB_FROM_STATUS(___retval) : BoS_EAOB); \
	})
# else
__myc_extern_inline__ aorp_object_t
AorpIopumpDetiod(aorp_object_t aThis, int32_t aDevId, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(detiod, iopump, aThis, anErrPtr, aDevId);
	return (BoSP_NORMAL(retval) ? BO_AOB_FROM_STATUS(retval) : BoS_EAOB);
}
# endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIopumpDeliod(
	aorp_object_t aThis,
	u_int32_t aDevId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The object referenced by the parameter @portid is released.
 */
# define AorpIopumpDeliod(aThis, aDevId, anErrPtr) \
	_A_call(deliod, iopump, aThis, anErrPtr, (u_int32_t)(aDevId))

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_IO_IOPUMP__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
