/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * IopumpP.h	(V.Sokur)
 * include/Bo/services/io/IopumpP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_IOPUMP_P__
# define __BO_SERVICES_IO_IOPUMP_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/io/Iostypes.h>

/******************************************************************************
 * %% BeginSection: iopump service declarations
 */
# pragma mark -
# pragma mark [ iopump service declarations ]
# pragma mark -

# define __aorpc_need_service_iopump 1

#endif /* !__BO_SERVICES_IO_IOPUMP_P__ */

#pragma mark -

#if __aorpc_uses_service(iopump)
# undef __aorpc_need_service_iopump

/*
 * iopump service
 * @{
 */

/*
 * iopump {\object(instance)}
 */
/*
# define AorpK_iopump_service (1073)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name iopump
# include <Bo/aorp/AorpCsrvc.h>

_A_service(iopump)

# if __aorpc_interface

/*
 * state domain
 */
# ifndef BoK_STATE_IOPUMP
#  define BoK_STATE_IOPUMP __myc_mk4ccs("iopu")
#  define _BoK_STATE_IOPUMP __myc_mk4cc('i','o','p','u')
# endif

/*
 * values for port-type
 */
# define AorpK_IOPUMP_INP 0
# define AorpK_IOPUMP_OUTP 1

/*
 * pump open flags
 */

// @togo: remove
//# define AorpF_IOPUMP_ASYNC_MODE 0x000000
//# define AorpF_IOPUMP_SYNC_MODE 0x010000
// @todo: redesign
//# define AorpF_IOPUMP_ASYNC_MODE AorpF_ASYNC_MODE
//# define AorpF_IOPUMP_SYNC_MODE AorpF_SYNC_MODE
// @togo: remove
//# define AorpF_IOPUMP_MODE_DEFAULT AorpF_IOPUMP_ASYNC_MODE
# define AorpF_IOPUMP_MODE_DEFAULT AorpF_ASYNC_MODE

/*
 * various flags
 */

/* NOTE!!!
   You should use these flags if and only if you know what you do. */
# define AorpF_IOPUMP_MUI_INPORT 0x010000
# define AorpF_IOPUMP_MUI_OUTPORT 0x020000
# define AorpF_IOPUMP_MUI_PMASK 0x030000

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(getport, "iopump::get-port", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aPortType
		)),

	_A_attribute((nonnull(1)))
	_A_op(addport, "iopump::add-port", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_object_t aPort,
			int aPortType
		)),

	_A_attribute((nonnull(1)))
	_A_op(rmport, "iopump::remove-port", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aPortType
		)),

	_A_attribute((nonnull(1)))
	_A_op(delport, "iopump::delete-port", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aPortType
		)),

	_A_attribute((nonnull(1)))
	_A_op(getiod, "iopump::get-device", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			u_int32_t aDevId
		)),

	_A_attribute((nonnull(1)))
	_A_op(attiod, "iopump::attach-device", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aPortType,
			unsigned aBino, /* = 0; 1-based, 0 means index chosen by implementation */
			aorp_object_t aDev,
			struct bo_options *Options
		)),

	_A_attribute((nonnull(1)))
	_A_op(reattiod, "iopump::reattach-device", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			u_int32_t aDevId,
			aorp_object_t aDev,
			struct bo_options *Options
		)),

	_A_attribute((nonnull(1)))
	_A_op(detiod, "iopump::detach-device", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			u_int32_t aDevId
		)),

	_A_attribute((nonnull(1)))
	_A_op(deliod, "iopump::delete-device", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			u_int32_t aDevId
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(iopump) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
