/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Ios.h	(V.Sokur)
 * include/Bo/services/io/Ios.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_IOS__
# define __BO_SERVICES_IO_IOS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/io/IosP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIosGettinfo(
	aorp_object_t aThis,
	bo_pointer_t anInfo,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIosGettinfo(aThis, anInfo, anErrPtr) \
	_A_call(gettinfo, ios, aThis, anErrPtr, anInfo)

# pragma mark -

/*
bo_state_t
__attribute__((nonnull(1)))
AorpIosState(
	aorp_object_t aThis,
	u_int32_t aDomain, // = 0
	struct aorp_error *anErrPtr
	);
*/
# define AorpIosState(aThis, aDomain, anErrPtr) \
	((bo_state_t) _A_call(getstate, ios, aThis, anErrPtr, (u_int32_t)(aDomain)))

# define AorpIosState0(aThis, anErrPtr) \
	AorpIosState(aThis, 0, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIosSync(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIosSync(aThis, anErrPtr) \
	_A_call(sync, ios, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIosOptctl(
	aorp_object_t aThis,
	struct bo_options *Options,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIosOptctl(aThis, Options, anErrPtr) \
	_A_call(optctl, ios, aThis, anErrPtr, Options)

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpIosBind(
	aorp_object_t aThis,
	unsigned aBino, // = 0; 1-based, 0 means index chosen by implementation
	struct bo_uconn_ds *anUconn,
	struct bo_options *Options,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIosBind(aThis, aBino, anUconn, Options, anErrPtr) \
	((bo_index_t) _A_call(bind, ios, aThis, anErrPtr, aBino, anUconn, Options))

# define AorpIosBind0(aThis, anUconn, Options, anErrPtr) \
	AorpIosBind(aThis, 0, anUconn, Options, anErrPtr)

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpIosRebind(
	aorp_object_t aThis,
	unsigned aBino, // = 0; 1-based, 0 means index chosen by implementation
	struct bo_uconn_ds *anUconn,
	struct bo_options *Options,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIosRebind(aThis, aBino, anUconn, Options, anErrPtr) \
	((bo_index_t) _A_call(rebind, ios, aThis, anErrPtr, aBino, anUconn, Options))

# define AorpIosRebind0(aThis, anUconn, Options, anErrPtr) \
	AorpIosRebind(aThis, 0, anUconn, Options, anErrPtr)

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpIosUnbind(
	aorp_object_t aThis,
	unsigned aBino, // = 0; 1-based, 0 means all indices
	struct aorp_error *anErrPtr
	);
*/
# define AorpIosUnbind(aThis, aBino, anErrPtr) \
	((bo_index_t) _A_call(unbind, ios, aThis, anErrPtr, aBino))

# define AorpIosUnbind0(aThis, anErrPtr) \
	AorpIosUnbind(aThis, 0, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIosControl(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr,
	int aRq,
	...
	);
*/
# if __myc_support(NVM)
#  define AorpIosControl(aThis, anErrPtr, aRq...) \
		_A_call(control, ios, aThis, anErrPtr, aRq)
# else
# if __myc_support(AVM)
#  define AorpIosControl(aThis, anErrPtr, ...) \
		_A_call(control, ios, aThis, anErrPtr, __VA_ARGS__)
# endif /* __myc_support(AVM) */
# endif /* __myc_support(NVM) */

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpIosPoll(
	aorp_object_t aThis,
	struct aorp_pollevents *Events,
	struct timespec *aTimeout,
	struct aorp_error *anErrPtr
	);
*/
#  define AorpIosPoll(aThis, Events, aTimeout, anErrPtr) \
	_A_call(poll, ios, aThis, anErrPtr, Events, aTimeout)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpIosPoll_abs(
	aorp_object_t aThis,
	struct aorp_pollevents *Events,
	struct timespec *anAbstime,
	struct aorp_error *anErrPtr
	);
*/
#  define AorpIosPoll_abs(aThis, Events, anAbstime, anErrPtr) \
	_A_call(poll_abs, ios, aThis, anErrPtr, Events, anAbstime)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIosGetpollbox(
	aorp_object_t aThis,
	bo_pointer_t aDscrPtr,
	struct aorp_error *anErrPtr
	);
*/
#  define AorpIosGetpollbox(aThis, aDscrPtr, anErrPtr) \
	_A_call(getpollbox, ios, aThis, anErrPtr, aDscrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIosGetselbox(
	aorp_object_t aThis,
	bo_pointer_t aDscrPtr,
	struct aorp_error *anErrPtr
	);
*/
#  define AorpIosGetselbox(aThis, aDscrPtr, anErrPtr) \
	_A_call(getselbox, ios, aThis, anErrPtr, aDscrPtr)

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_IO_IOS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
