/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * IosP.h	(V.Sokur)
 * include/Bo/services/io/IosP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_IOS_P__
# define __BO_SERVICES_IO_IOS_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/io/Iostypes.h>

# include <Bo/libc/Stdarg.h>

/******************************************************************************
 * %% BeginSection: ios service declarations
 */
# pragma mark -
# pragma mark [ ios service declarations ]
# pragma mark -

# define __aorpc_need_service_ios 1

#endif /* !__BO_SERVICES_IO_IOS_P__ */

#pragma mark -

#if __aorpc_uses_service(ios)
# undef __aorpc_need_service_ios

/*
 * ios service
 * @{
 */

/*
 * ios {\object(instance)}
 */
/*
# define AorpK_ios_service (1070)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name ios
# include <Bo/aorp/AorpCsrvc.h>

_A_service(ios)

	/* forward declaration */
	struct timespec;

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(gettinfo, "ios::get-transport-info", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t anInfo
		)),

	_A_attribute((nonnull(1)))
	_A_op(getstate, "ios::get-state", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			u_int32_t aDomain /* = 0 */
		)),

	_A_attribute((nonnull(1)))
	_A_op(sync, "ios::sync", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(optctl, "ios::optctl", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_options *Options
		)),

	_A_attribute((nonnull(1)))
	_A_op(bind, "ios::bind", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aBino, /* = 0; 1-based, 0 means index chosen by implementation */
			struct bo_uconn_ds *anUconn,
			struct bo_options *Options
		)),

	_A_attribute((nonnull(1)))
	_A_op(rebind, "ios::rebind", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aBino, /* = 0; 1-based, 0 means index chosen by implementation */
			struct bo_uconn_ds *anUconn,
			struct bo_options *Options
		)),

	_A_attribute((nonnull(1)))
	_A_op(unbind, "ios::unbind", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aBino /* = 0; 1-based, 0 means all indices */
		)),

	_A_attribute((nonnull(1)))
	_A_op(control, "ios::control", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aRq,
			...
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(poll, "ios::poll-with-relative-timeout", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_pollevents *Events,
			struct timespec *aTimeout
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(poll_abs, "ios::poll-with-absolute-timeout", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_pollevents *Events,
			struct timespec *anAbstime
		)),

	/* for system interface poll */
	_A_attribute((nonnull(1)))
	_A_op(getpollbox, "ios::get-poll-box", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t aDscrPtr
		)),

	/* for system interface select */
	_A_attribute((nonnull(1)))
	_A_op(getselbox, "ios::get-select-box", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t aDscrPtr
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(ios) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
