/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Ioscntlcom.h	(V.Sokur)
 * include/Bo/services/io/Ioscntlcom.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_IOSCNTLCOM__
# define __BO_SERVICES_IO_IOSCNTLCOM__

/******************************************************************************
 * %% BeginSection: constant/type declarations
 */
# pragma mark -
# pragma mark [ constant/type declarations ]
# pragma mark -

# define AORP_MAKE_IOSC(aCategory,aCmd) \
	((((aCategory) & 0xffff) << 16) | ((aCmd) & 0xffff))

# define _AorpIOSC(aCategory,aCmd) AORP_MAKE_IOSC(aCategory,aCmd)

# define AorpIOSC_CATEGORY(aCom) ((unsigned)(((aCom) >> 16) & 0xffff))
# define AorpIOSC_CMD(aCom) ((unsigned)((aCom) & 0xffff))

/*
 * Well known categories
 */

# define AorpK_IOSC_MISC_CATEGORY (3)
# define AorpK_IOSC_FILE_CATEGORY (4)
# define AorpK_IOSC_DEVICE_CATEGORY (5)
# define AorpK_IOSC_MMEDIA_CATEGORY (10)
# define AorpK_IOSC_MUI_CATEGORY (20)

/*
 * MISC
 */

# define _AorpK_IosCntl_Getprop (3)
# define _AorpK_IosCntl_Setprop (4)
# define _AorpK_IosCntl_QueryLLAPI (5)

/* [arguments required (struct bo_options *Arguments)] */
# define AorpK_IosCntl_Getprop _AorpIOSC(AorpK_IOSC_MISC_CATEGORY,_AorpK_IosCntl_Getprop)
# define AorpK_IosCntl_Setprop _AorpIOSC(AorpK_IOSC_MISC_CATEGORY,_AorpK_IosCntl_Setprop)

/* [arguments required (struct bo_options *Arguments)] */
# define AorpK_IosCntl_QueryLLAPI _AorpIOSC(AorpK_IOSC_MISC_CATEGORY,_AorpK_IosCntl_QueryLLAPI)

/*
 * FILE
 */

# define _AorpK_IosCntl_Seek (3)
# define _AorpK_IosCntl_Stat (4)

/* [arguments required (struct bo_options *Arguments)] */
# define AorpK_IosCntl_Seek _AorpIOSC(AorpK_IOSC_FILE_CATEGORY,_AorpK_IosCntl_Seek)

/* [arguments required (struct bo_options *Arguments)] */
# define AorpK_IosCntl_Stat _AorpIOSC(AorpK_IOSC_FILE_CATEGORY,_AorpK_IosCntl_Stat)

/*
 * DEVICE
 */

# define _AorpK_IosCntl_Mkiod (3)

/* [arguments required (struct bo_options *Arguments)] */
# define AorpK_IosCntl_Mkiod _AorpIOSC(AorpK_IOSC_DEVICE_CATEGORY,_AorpK_IosCntl_Mkiod)

/*
 * MMEDIA
 */

/*
 * MUI
 */

# define _AorpK_IosCntl_Setup (3)
# define _AorpK_IosCntl_Teardown (4)
# define _AorpK_IosCntl_Start (5)
# define _AorpK_IosCntl_Play AorpK_IosCntl_Start
# define _AorpK_IosCntl_Stop (6)
# define _AorpK_IosCntl_Pause (7)
# define _AorpK_IosCntl_Resume (8)

/* [arguments required (struct bo_options *Arguments = NULL)] */

# define AorpK_IosCntl_Setup _AorpIOSC(AorpK_IOSC_MUI_CATEGORY,_AorpK_IosCntl_Setup)
# define AorpK_IosCntl_Teardown _AorpIOSC(AorpK_IOSC_MUI_CATEGORY,_AorpK_IosCntl_Teardown)
# define AorpK_IosCntl_Start _AorpIOSC(AorpK_IOSC_MUI_CATEGORY,_AorpK_IosCntl_Start)
# define AorpK_IosCntl_Play AorpK_IosCntl_Start
# define AorpK_IosCntl_Stop _AorpIOSC(AorpK_IOSC_MUI_CATEGORY,_AorpK_IosCntl_Stop)
# define AorpK_IosCntl_Pause _AorpIOSC(AorpK_IOSC_MUI_CATEGORY,_AorpK_IosCntl_Pause)
# define AorpK_IosCntl_Resume _AorpIOSC(AorpK_IOSC_MUI_CATEGORY,_AorpK_IosCntl_Resume)


/* speed values used with start operation */
# ifndef AorpK_IOS_MAX_SPEED
#  define AorpK_IOS_MAX_SPEED 10.f
#  define AorpK_IOS_MIN_SPEED -10.f
#  define AorpK_IOS_NORMAL_SPEED 1.f
# endif

#endif /* !__BO_SERVICES_IO_IOSCNTLCOM__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
