/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Iostr.h	(V.Sokur)
 * include/Bo/services/io/Iostr.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_IOSTR__
# define __BO_SERVICES_IO_IOSTR__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/io/IostrP.h>
# include <Bo/services/io/IosP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIostrSetiobuf(
	aorp_object_t aThis,
	bo_pointer_t aBuffer,        // NULL
	size_t aBufsz,               // 0
	aorp_iobufmode_t aBufmode,   // AorpK_IOSTR_NONEBF
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrSetiobuf(aThis, aBuffer, aBufsz, aBufmode, anErrPtr) \
	_A_call(setiobuf, iostr, aThis, anErrPtr, \
		aBuffer, (size_t)(aBufsz), (aorp_iobufmode_t)(aBufmode))

# pragma mark -

/*
bo_pointer_t
__attribute__((nonnull(1)))
AorpIostrGetiobuf(
	aorp_object_t aThis,
	size_t *aBufszPtr,           // NULL
	struct aorp_error *anErrPtr
	);
*/

# if __myc_support(macro_retval)
#  define AorpIostrGetiobuf(aThis, aBufszPtr, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(getiobuf, iostr, aThis, anErrPtr, aBufszPtr); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_PTR_FROM_STATUS(___retval) : BoS_EPTR); \
	})
# else
__myc_extern_inline__ bo_pointer_t
AorpIostrGetiobuf(aorp_object_t aThis, size_t *aBufszPtr, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(getiobuf, iostr, aThis, anErrPtr, aBufszPtr);
	return (BoSP_NORMAL(retval) ? BO_PTR_FROM_STATUS(retval) : BoS_EPTR);
}
# endif

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpIostrRead(
	aorp_object_t aThis,
	bo_pointer_t aBuffer,        // NULL
	size_t aBufsz,               // 0
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrRead(aThis, aBuffer, aBufsz, anErrPtr) \
	((ssize_t) _A_call(read, iostr, aThis, anErrPtr, aBuffer, (size_t)(aBufsz)))

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpIostrReadall(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrReadall(aThis, anErrPtr) \
	((ssize_t) _A_call(readall, iostr, aThis, anErrPtr))

# pragma mark -

/*
bo_pointer_t
__attribute__((nonnull(1)))
AorpIostrGetrdbuf(
	aorp_object_t aThis,
	size_t *aDataszPtr,          // NULL
	size_t *aBufszPtr,           // NULL
	struct aorp_error *anErrPtr
	);
*/

# if __myc_support(macro_retval)
#  define AorpIostrGetrdbuf(aThis, aDataszPtr, aBufszPtr, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(getrdbuf, iostr, aThis, anErrPtr, aDataszPtr, aBufszPtr); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_PTR_FROM_STATUS(___retval) : BoS_EPTR); \
	})
# else
__myc_extern_inline__ bo_pointer_t
AorpIostrGetrdbuf(
	aorp_object_t aThis,
	size_t *aDataszPtr,
	size_t *aBufszPtr,
	struct aorp_error *anErrPtr
	)
{
	bo_stsptr_t retval = _A_call(getrdbuf, iostr, aThis, anErrPtr, aDataszPtr, aBufszPtr);
	return (BoSP_NORMAL(retval) ? BO_PTR_FROM_STATUS(retval) : BoS_EPTR);
}
# endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIostrEndrdbuf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrEndrdbuf(aThis, anErrPtr) \
	_A_call(endrdbuf, iostr, aThis, anErrPtr)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpIostrWrite(
	aorp_object_t aThis,
	bo_const_pointer_t aBuffer,  // NULL
	size_t aBufsz,               // 0
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrWrite(aThis, aBuffer, aBufsz, anErrPtr) \
	((ssize_t) _A_call(write, iostr, aThis, anErrPtr, aBuffer, (size_t)(aBufsz)))

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1,3)))
AorpIostrPrintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr,
	char const *aFormat,
	...
	);
*/
# if __myc_support(NVM)
#  define AorpIostrPrintf(aThis, anErrPtr, aFormat...) \
		((ssize_t) _A_call(printf, iostr, aThis, anErrPtr, aFormat))
# else
# if __myc_support(AVM)
#  define AorpIostrPrintf(aThis, anErrPtr, ...) \
		((ssize_t) _A_call(printf, iostr, aThis, anErrPtr, __VA_ARGS__))
# endif /* __myc_support(AVM) */
# endif /* __myc_support(NVM) */

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1,3)))
AorpIostrVprintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr,
	char const *aFormat,
	va_list ArgPtr
	);
*/
# define AorpIostrVprintf(aThis, anErrPtr, aFormat, ArgPtr) \
	((ssize_t) _A_call(vprintf, iostr, aThis, anErrPtr, aFormat, ArgPtr))

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1,3)))
AorpIostrWprintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr,
	wchar_t const *aFormat,
	...
	);
*/
# if __myc_support(NVM)
#  define AorpIostrWprintf(aThis, anErrPtr, aFormat...) \
		((ssize_t) _A_call(wprintf, iostr, aThis, anErrPtr, aFormat))
# else
# if __myc_support(AVM)
#  define AorpIostrWprintf(aThis, anErrPtr, ...) \
		((ssize_t) _A_call(wprintf, iostr, aThis, anErrPtr, __VA_ARGS__))
# endif /* __myc_support(AVM) */
# endif /* __myc_support(NVM) */

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1,3)))
AorpIostrVwprintf(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr,
	wchar_t const *aFormat,
	va_list ArgPtr
	);
*/
# define AorpIostrVwprintf(aThis, anErrPtr, aFormat, ArgPtr) \
	((ssize_t) _A_call(vwprintf, iostr, aThis, anErrPtr, aFormat, ArgPtr))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpIostrFlush(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrFlush(aThis, anErrPtr) \
	_A_call(flush, iostr, aThis, anErrPtr)

# pragma mark -

/*
bo_off_t
__attribute__((nonnull(1)))
AorpIostrSeek(
	aorp_object_t aThis,
	bo_off_t anOffset,
	aorp_seekorigin_t Whence,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrSeek(aThis, anOffset, Whence, anErrPtr) \
	((bo_off_t) _A_call(seek, iostr, aThis, anErrPtr, \
		(bo_off_t)(anOffset), (aorp_seekorigin_t)(Whence)))

# pragma mark -

# define AorpIostrRdoffset(aThis, anErrPtr) \
	AorpIostrSeek(aThis, 0, (AorpK_IOS_SEEK_CUR | AorpK_IOS_SEEK_RDPOS), anErrPtr)

# define AorpIostrWroffset(aThis, anErrPtr) \
	AorpIostrSeek(aThis, 0, (AorpK_IOS_SEEK_CUR | AorpK_IOS_SEEK_WRPOS), anErrPtr)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpIostrReadv(
	aorp_object_t aThis,
	aorp_opflags_t Flags, // = 0
	struct bo_qnabsbuf *aBufferv,
	ssize_t aBufc, // <= BoK_IOV_MAX
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrReadv(aThis, Flags, aBufferv, aBufc, anErrPtr) \
	((ssize_t) _A_call(readv, iostr, aThis, anErrPtr, \
		Flags, aBufferv, (ssize_t)(aBufc)))

# if __myc_support(macro_retval)
#  define AorpIostrPeek(aThis, aBuffer, aBufsz, anErrPtr) \
	__extension__ ({ \
		ssize_t ___retval; \
		struct bo_qnabsbuf ___buf; \
		___buf.size = ___buf.capacity = (aBufsz); \
		___buf.base = (aBuffer); \
		___retval = AorpIostrReadv(aThis, AorpF_IOR_PEEK, &___buf, 1, anErrPtr); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ ssize_t
AorpIostrPeek(
	aorp_object_t aThis,
	bo_pointer_t aBuffer,
	size_t aBufsz,
	struct aorp_error *anErrPtr
	)
{
	ssize_t retval;
	struct bo_qnabsbuf buf;
	buf.size = buf.capacity = (aBufsz);
	buf.base = (aBuffer);
	retval = AorpIostrReadv(aThis, AorpF_IOR_PEEK, &buf, 1, anErrPtr);
	return (retval);
}
# endif

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpIostrWritev(
	aorp_object_t aThis,
	aorp_opflags_t Flags, // = 0
	struct bo_qnabsbuf const *aBufferv,
	ssize_t aBufc, // <= BoK_IOV_MAX
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrWritev(aThis, Flags, aBufferv, aBufc, anErrPtr) \
	((ssize_t) _A_call(writev, iostr, aThis, anErrPtr, \
		Flags, aBufferv, (ssize_t)(aBufc)))

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpIostrPread(
	aorp_object_t aThis,
	bo_pointer_t aBuffer,        // NULL
	size_t aBufsz,               // 0
	bo_off_t anOffset,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrPread(aThis, aBuffer, aBufsz, anOffset, anErrPtr) \
	((ssize_t) _A_call(pread, iostr, aThis, anErrPtr, \
		aBuffer, (size_t)(aBufsz), (bo_off_t)(anOffset)))

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpIostrPwrite(
	aorp_object_t aThis,
	bo_const_pointer_t aBuffer,  // NULL
	size_t aBufsz,               // 0
	bo_off_t anOffset,
	struct aorp_error *anErrPtr
	);
*/
# define AorpIostrPwrite(aThis, aBuffer, aBufsz, anOffset, anErrPtr) \
	((ssize_t) _A_call(pwrite, iostr, aThis, anErrPtr, \
		aBuffer, (size_t)(aBufsz), (bo_off_t)(anOffset)))

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_IO_IOSTR__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
