/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * IostrP.h	(V.Sokur)
 * include/Bo/services/io/IostrP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_IOSTR_P__
# define __BO_SERVICES_IO_IOSTR_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/io/Iostypes.h>

# include <Bo/libc/Stdarg.h>

/******************************************************************************
 * %% BeginSection: iostr service declarations
 */
# pragma mark -
# pragma mark [ iostr service declarations ]
# pragma mark -

# define __aorpc_need_service_iostr 1

#endif /* !__BO_SERVICES_IO_IOSTR_P__ */

#pragma mark -

#if __aorpc_uses_service(iostr)
# undef __aorpc_need_service_iostr

/*
 * iostr service
 * @{
 */

/*
 * iostr {\object(instance)}
 */
/*
# define AorpK_iostr_service (1071)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name iostr
# include <Bo/aorp/AorpCsrvc.h>

_A_service(iostr)

# if __aorpc_interface

/*
 * state domain
 */
# ifndef BoK_STATE_IOSTR
#  define BoK_STATE_IOSTR __myc_mk4ccs("iosr")
#  define _BoK_STATE_IOSTR __myc_mk4cc('i','o','s','r')
# endif

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(setiobuf, "iostr::set-io-buffer", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t aBuffer /* = NULL */,
			size_t aBufsz /* = 0 */,
			aorp_iobufmode_t aBufmode /* = AorpK_IOS_NONEBF */
		)),

	_A_attribute((nonnull(1)))
	_A_op(getiobuf, "iostr::get-io-buffer", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t *aBufszPtr /* = NULL */
		)),

	_A_attribute((nonnull(1)))
	_A_op(read, "iostr::read-data", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t aBuffer /* = NULL */,
			size_t aBufsz /* = 0 */
		)),

	_A_attribute((nonnull(1)))
	_A_op(readall, "iostr::read-all-to-internal-buffer", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(getrdbuf, "iostr::get-internal-read-buffer", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t *aDataszPtr /* = NULL */,
			size_t *aBufszPtr /* = NULL */
		)),

	_A_attribute((nonnull(1)))
	_A_op(endrdbuf, "iostr::end-internal-read-buffer", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(write, "iostr::write-data", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_const_pointer_t aBuffer /* = NULL */,
			size_t aBytec /* = 0 */
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(printf, "iostr::print-data-with-format", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aFormat,
			...
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(vprintf, "iostr::vprint-data-with-format", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aFormat,
			va_list ArgPtr
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(wprintf, "iostr::wprint-data-with-format", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			wchar_t const *aFormat,
			...
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(vwprintf, "iostr::vwprint-data-with-format", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			wchar_t const *aFormat,
			va_list ArgPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(flush, "iostr::flush", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(seek, "iostr::seek", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_off_t anOffset,
			aorp_seekorigin_t Whence
		)),

	_A_attribute((nonnull(1)))
	_A_op(readv, "iostr::scatter-read-data", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_opflags_t Flags,
			struct bo_qnabsbuf *aBufferv,
			ssize_t aBufc
				/* (0 < aBufc) => aBufferv[aBufc],
				   (0 == aBufc) => aBufferv is a linked-list of buffers (NULL terminated),
				   (aBufc < 0) => aBufferv is a linked-list of buffers (count = -aBufc) */
		)),

	_A_attribute((nonnull(1)))
	_A_op(writev, "iostr::gather-write-data", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_opflags_t Flags,
			struct bo_qnabsbuf const *aBufferv,
			ssize_t aBufc
				/* (0 < aBufc) => aBufferv[aBufc],
				   (0 == aBufc) => aBufferv is a linked-list of buffers (NULL terminated),
				   (aBufc < 0) => aBufferv is a linked-list of buffers (count = -aBufc) */
		)),

	_A_attribute((nonnull(1)))
	_A_op(pread, "iostr::read-data-from-position", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t aBuffer,
			size_t aBufsz,
			bo_off_t anOffset
		)),

	_A_attribute((nonnull(1)))
	_A_op(pwrite, "iostr::write-data-to-position", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_const_pointer_t aBuffer,
			size_t aBufsz,
			bo_off_t anOffset
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(iostr) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
