/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Miosopt.h	(V.Sokur)
 * include/Bo/services/io/Miosopt.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_MIOSOPT__
# define __BO_SERVICES_IO_MIOSOPT__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SERVICES_IO_FIOSOPT__
#  include <Bo/services/io/Fiosopt.h>
# endif

/******************************************************************************
 * %% BeginSection: constant/type declarations
 */
# pragma mark -
# pragma mark [ constant/type declarations ]
# pragma mark -

/*
 * option domain (bo_opthdr.domain)
 */
# ifndef BoK_MIOS_OPT
#  define BoK_MIOS_OPT __myc_mk4ccs("mios")
#  define _BoK_MIOS_OPT __myc_mk4cc('m','i','o','s')
# endif

/* ----------------------------------------------------------------------------
 * MEDIA-OPTCTL OPTION
 * ios::optctl
 */
# pragma mark -

/*
 * hdr.domain = BoK_MIOS_OPT
 * hdr.name = {AorpK_IOS_OPTCTL}
 */

# define aorp_moptctl_data aorp_goptctl_data
# define aorp_moptctl_data_p(x) aorp_goptctl_data_p(x)

# define aorp_moptctl_idata aorp_goptctl_idata
# define aorp_moptctl_bdata aorp_goptctl_bdata

BO_OPT_STRUCT(aorp_moptctli_s, struct aorp_goptctl_idata);
BO_OPT_STRUCT(aorp_moptctlb_s, struct aorp_goptctl_bdata);

# define AORP_MOPTCTLI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptctl_idata, anOption, _BoK_MIOS_OPT, AorpK_IOS_OPTCTL)

# define AORP_MOPTCTLB_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptctl_bdata, anOption, _BoK_MIOS_OPT, AorpK_IOS_OPTCTL)

/*
 * values for aorp_moptctl.argnam
 */

# define AorpK_MOPTC_COMPRESSED_SAMPLE 127
	/* aorp_goptctl_idata: {u_int8_t} */

# define AorpK_MOPTC_LIVE_SOURCE 126
	/* aorp_goptctl_idata: {u_int8_t} */

# define AorpK_MOPTC_STRNUMSEL 125
	/* aorp_goptctl_bdata: elemc * {stream-desc:struct bo_strnum_ds} */

/* ----------------------------------------------------------------------------
 * MEDIA-SEEK OPTION
 * ios::control(Seek)
 */
# pragma mark -

/*
 * hdr.domain = BoK_MIOS_OPT
 * hdr.name = {AorpK_IosCntl_Seek}
 */

struct aorp_moptseek_data {
	aorp_seekformat_t  format;
	aorp_seekorigin_t  whence;
	int64_t            offset;
};

BO_OPT_STRUCT(aorp_moptseek_s, struct aorp_moptseek_data);

# define AORP_MOPTSEEK_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptseek_data, anOption, _BoK_MIOS_OPT, AorpK_IosCntl_Seek)

/* ----------------------------------------------------------------------------
 * MEDIA-START OPTION
 * ios::control(Start)
 */
# pragma mark -

/*
 * hdr.domain = BoK_MIOS_OPT
 * hdr.name = {AorpK_IosCntl_Start}
 */

struct aorp_moptstart_data {
	/* seek
	 * @(
	 * not used if NULL-format
	 */
	aorp_seekformat_t  format;
	aorp_seekorigin_t  whence;
	int64_t            offset;
	/* @} */

	int64_t            duration;
	float              speed;
		/* [-10, 10], 1: normal speed;
		   negative value indicates reverse rate (rewinding) */

	u_int32_t          strno;
		/* stream number, used when the seek format
		   is AorpK_SEEK_VIDEOFRAME_FORMAT;
		   1-based, 0 means that stream number is not used
		   or set automatically. */
};

BO_OPT_STRUCT(aorp_moptstart_s, struct aorp_moptstart_data);

# define AORP_MOPTSTART_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptstart_data, anOption, _BoK_MIOS_OPT, AorpK_IosCntl_Start)

/* ----------------------------------------------------------------------------
 * MEDIA-PARM OPTION
 * any-operation
 * ios::control(Pause,Resume)
 */
# pragma mark -

/*
 * hdr.domain = BoK_MIOS_OPT
 * hdr.name = {AorpK_GOPTPARM, AorpK_IosCntl_Pause, AorpK_IosCntl_Resume, ...}
 */

# define aorp_moptparm_data aorp_goptparm_data
# define aorp_moptparm_data_p(x) aorp_goptparm_data_p(x)

# define aorp_moptparm_idata aorp_goptparm_idata
# define aorp_moptparm_bdata aorp_goptparm_bdata

BO_OPT_STRUCT(aorp_moptparmi_s, struct aorp_goptparm_idata);
BO_OPT_STRUCT(aorp_moptparmb_s, struct aorp_goptparm_bdata);

# define AORP_MOPTPARMI_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptparm_idata, anOption, _BoK_MIOS_OPT, AorpK_GOPTPARM)

# define AORP_MOPTPARMB_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptparm_bdata, anOption, _BoK_MIOS_OPT, AorpK_GOPTPARM)

# define AORP_MOPTPAUSE_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptparm_bdata, anOption, _BoK_MIOS_OPT, AorpK_IosCntl_Pause)

# define AORP_MOPTRESUME_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_moptparm_bdata, anOption, _BoK_MIOS_OPT, AorpK_IosCntl_Resume)

/*
 * values for aorp_moptparm.parmnam
 */

# define AorpK_MPARM_STRNUMLIST 127
	/* aorp_moptparm_bdata: elemc * {stream-desc:struct bo_strnum_ds} */

#endif /* !__BO_SERVICES_IO_MIOSOPT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
