/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Idea is originated with Ritchie D.M., "A Stream Input-Output System"
 */

/*
 * StrjetP.h	(V.Sokur)
 * include/Bo/services/io/StrjetP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_STRJET_P__
# define __BO_SERVICES_IO_STRJET_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/aorp/Types.h>

/******************************************************************************
 * %% BeginSection: strjet service declarations
 */
# pragma mark -
# pragma mark [ strjet service declarations ]
# pragma mark -

/* ----------------------------------------------------------------------------
 * strjet service
 * @{
 */

# include <Bo/services/Strmop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name strjet
# include <Bo/aorp/AorpC.h>

_A_service(strjet)

# define AORP_strjet_init_once AORP_any_init_once
# define AORP_strjet_init_once_signature AORP_any_init_once_signature
# define AORP_strjet_fini_once AORP_any_fini_once
# define AORP_strjet_fini_once_signature AORP_any_fini_once_signature
# define AORP_strjet_pod_init AORP_any_pod_init
# define AORP_strjet_pod_init_signature AORP_any_pod_init_signature
# define AORP_strjet_pod_fini AORP_any_pod_fini
# define AORP_strjet_pod_fini_signature AORP_any_pod_fini_signature

_A_op(openstrum, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aName,
			/* ex.: "/dev/strum/<device>";
			   where device:
			   -) zero device [any free], zero0, zero1, ...
			   -) rfx - remote functional execution device [any free], rfx0, rfx1, ...
			   -) ... */
		aorp_object_t *aStrumPtr
	)) __attribute__((nonnull(1,3,4)));

# define AORP_strjet_openstrum "open-strum"
# define AORP_strjet_openstrum_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_op(closestrum, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		aorp_object_t aStrum
	)) __attribute__((nonnull(1,3)));

# define AORP_strjet_closestrum "close-strum"
# define AORP_strjet_closestrum_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(closeall, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr
	)) __attribute__((nonnull(1)));

# define AORP_strjet_closeall "close-all-strums"
# define AORP_strjet_closeall_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(register, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char *aMajorName,
		unsigned aMinorNum,
		aorp_object_t aStrum
	)) __attribute__((nonnull(1,3,5)));

# define AORP_strjet_register "register-strum"
# define AORP_strjet_register_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_INT _AORP_SMOD_UNSIGNED \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_op(unregister, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char *aMajorName,
		unsigned aMinorNum
	)) __attribute__((nonnull(1,3)));

# define AORP_strjet_unregister "unregister-strum"
# define AORP_strjet_unregister_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_INT _AORP_SMOD_UNSIGNED

_A_op(getalloc, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_alloctx const **anAllocPtr
	)) __attribute__((nonnull(1,3)));

# define AORP_strjet_getalloc "get-data-allocator"
# define AORP_strjet_getalloc_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

#endif /* !__BO_SERVICES_IO_STRJET_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
