/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Idea is originated with Ritchie D.M., "A Stream Input-Output System"
 */

/*
 * StrmodP.h	(V.Sokur)
 * include/Bo/services/io/StrmodP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_STRMOD_P__
# define __BO_SERVICES_IO_STRMOD_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/aorp/Types.h>

/******************************************************************************
 * %% BeginSection: strmod service declarations
 */
# pragma mark -
# pragma mark [ strmod service declarations ]
# pragma mark -

/* ----------------------------------------------------------------------------
 * strmod service
 * @{
 */

# include <Bo/services/Strmop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name strmod
# include <Bo/aorp/AorpC.h>

_A_service(strmod)

# define AORP_strmod_init_once AORP_any_init_once
# define AORP_strmod_init_once_signature AORP_any_init_once_signature
# define AORP_strmod_fini_once AORP_any_fini_once
# define AORP_strmod_fini_once_signature AORP_any_fini_once_signature
# define AORP_strmod_pod_init AORP_any_pod_init
# define AORP_strmod_pod_init_signature AORP_any_pod_init_signature
# define AORP_strmod_pod_fini AORP_any_pod_fini
# define AORP_strmod_pod_fini_signature AORP_any_pod_fini_signature

_A_op(open, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		aorp_object_t aStrum,
		int aQuetype
	)) __attribute__((nonnull(1,3)));

# define AORP_strmod_open "open"
# define AORP_strmod_open_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_INT

_A_op(close, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		aorp_object_t aStrum,
		int aQuetype
	)) __attribute__((nonnull(1,3)));

# define AORP_strmod_close "close"
# define AORP_strmod_close_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_INT

_A_op(connect, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue,
		int aQuetype,
		int aConntype
	)) __attribute__((nonnull(1,3)));

# define AORP_strmod_connect "connect"
# define AORP_strmod_connect_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_INT

/*
 * BoS_OK: success, yes
 * BoS_NORMAL: success, no
 */
_A_op(canconnect, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue,
		int aQuetype,
		int aConntype,
		struct strm_conninfo const *aRqInfo,
		struct strm_conninfo *aReInfo
	)) __attribute__((nonnull(1,3)));

# define AORP_strmod_canconnect "can-connect"
# define AORP_strmod_canconnect_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_INT \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

/*
 * BoS_OK: success, yes
 * BoS_NORMAL: success, no
 * BoS_STRMOD_AFUNC: afunctional
 */
_A_op(poll, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		int aQuetype
	)) __attribute__((nonnull(1)));

# define AORP_strmod_poll "poll"
# define AORP_strmod_poll_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_INT

/*
 * BoS_OK: if OKAY(strum.putnext)
 * BoS_NORMAL: if SUCCESS(strum.putnext)
 */
_A_op(putmsg, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue,
		struct strm_msg_ds *aMsg
	)) __attribute__((nonnull(1,3,4)));

# define AORP_strmod_putmsg "put-message"
# define AORP_strmod_putmsg_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

/*
 * BoS_OK: success, yes
 * BoS_NORMAL: success, no
 * BoS_STRMOD_AFUNC: afunctional
 */
_A_op(service, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue
	)) __attribute__((nonnull(1,3)));

# define AORP_strmod_service "service"
# define AORP_strmod_service_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(getinfo, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_modinfo const **anInfoPtr
	)) __attribute__((nonnull(1,3)));

# define AORP_strmod_getinfo "getinfo"
# define AORP_strmod_getinfo_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_op(getstat, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_modstat **aStatPtr
	)) __attribute__((nonnull(1,3)));

# define AORP_strmod_getstat "getstat"
# define AORP_strmod_getstat_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

#endif /* !__BO_SERVICES_IO_STRMOD_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
