/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Idea is originated with Ritchie D.M., "A Stream Input-Output System"
 */

/*
 * Strmtypes.h	(V.Sokur)
 * include/Bo/services/io/Strmtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_STRMTYPES__
# define __BO_SERVICES_IO_STRMTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: constant/type declarations
 */
# pragma mark -
# pragma mark [ constant/type declarations ]
# pragma mark -

/* Specific status */
# define BoS_STRMOD_AFUNC (-2)

/* ----------------------------------------------------------------------------
 * Limits
 */

# define AorpL_STRMOD_NAME_MAX (16) /* including the terminating NUL character */

/* ----------------------------------------------------------------------------
 * Forward declarations
 */

struct aorp_ustrmod;

struct strm_msg_ds;
struct strm_data_ds;
struct strm_queue;
struct strm_conninfo;

struct strm_modinfo;
struct strm_modstat;

/* ----------------------------------------------------------------------------
 * Message queue declarations
 */

struct strm_msg_ds {
	bo_cdllelem_t        smb_lelem; /* struct strm_msg_ds *smb_next, *smb_prev; */
	struct strm_data_ds *smb_datap;
	size_t               smb_beg,
	                     smb_end;
	struct strm_msg_ds  *smb_cont;
};

struct strm_data_ds {
	long          sdb_type;
	bo_refc_t     sdb_refc;
	ssize_t       sdb_bufsz;
	bo_pointer_t  sdb_base;
	size_t        sdb_beg,
	              sdb_end;
};

/* Queue type */
# define StrmK_UNK_QUE (0)
# define StrmK_READ_QUE (1)
# define StrmK_WRITE_QUE (2)

struct strm_queue {
	int                                  sq_type;
	bo_cdllist_t                         sq_msgs;

	/* que of next module */
	struct strm_queue                   *sq_nextq;

	/* used for multiplexing */
	struct strm_queue                   *sq_upmuxq,
	                                    *sq_downmuxq;

	unsigned                             sq_hiwat, /* high limit of amount of que messages */
	                                     sq_lowat; /* low limit of amount of que messages */
	u_int32_t                            sq_state;
		/* .ex: module is ready to do something, ... */

	struct strm_modnode                 *sq_mnode;
	AORP_ONEP_OF((struct aorp_ustrmod *, aorp_object_t)) sq_mod;
};

/* ----------------------------------------------------------------------------
 * strmod declarations
 */

/* @todo: move to protocol (???) */
struct strm_modnode {
	char                sn_name[AorpL_STRMOD_NAME_MAX]; /* registered name at the strum */
	AORP_ONEP_OF((struct aorp_ustrmod *, aorp_object_t)) sn_mod;

	u_int32_t           sn_mhints;
		/* module hints: afunctional poll, service, ... */

	struct strm_queue  *sn_wrque,
	                   *sn_rdque;
};

/* Connect type */
# define StrmK_UPCONN (0)
# define StrmK_DOWNCONN (1)

struct strm_modinfo {
	u_int32_t  smi_id;
	char       smi_name[AorpL_STRMOD_NAME_MAX]; /* module name */

	unsigned   smi_wrq_hiwat,  /* high limit of amount of write-que messages */
	           smi_wrq_lowat,  /* low limit of amount of que messages */
	           smi_rdq_hiwat,
	           smi_rdq_lowat;
};

/*
 * ustrmod callback types
 */

typedef aorp_retain_cb  aorp_ustrmod_retain_fn;
typedef aorp_release_cb aorp_ustrmod_release_fn;

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ustrmod_open_fn)(
# if __myc_require(prototypes)
	struct aorp_ustrmod *aMod,
	aorp_object_t aStrum,
	int aQuetype,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ustrmod_close_fn)(
# if __myc_require(prototypes)
	struct aorp_ustrmod *aMod,
	aorp_object_t aStrum,
	int aQuetype,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ustrmod_connect_fn)(
# if __myc_require(prototypes)
	struct aorp_ustrmod *aMod,
	struct strm_queue *aQue,
	int aQuetype,
	int aConntype,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

/*
 * BoS_OK: yes
 * BoS_NORMAL: no
 */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ustrmod_canconnect_fn)(
# if __myc_require(prototypes)
	struct aorp_ustrmod *aMod,
	struct strm_queue *aQue,
	int aQuetype,
	int aConntype,
	struct strm_conninfo const *aRqInfo,
	struct strm_conninfo *aReInfo,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

/*
 * BoS_OK: yes
 * BoS_NORMAL: no
 * BoS_STRMOD_AFUNC: afunctional
 */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ustrmod_poll_fn)(
# if __myc_require(prototypes)
	struct aorp_ustrmod *aMod,
	int aQuetype,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

/*
 * BoS_OK: if OKAY(strum.putnext)
 * BoS_NORMAL: if SUCCESS(strum.putnext)
 */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ustrmod_putmsg_fn)(
# if __myc_require(prototypes)
	struct aorp_ustrmod *aMod,
	struct strm_queue *aQue,
	struct strm_msg_ds *aMsg,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2,3)));

/*
 * BoS_OK: yes
 * BoS_NORMAL: no
 * BoS_STRMOD_AFUNC: afunctional
 */
typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ustrmod_service_fn)(
# if __myc_require(prototypes)
	struct aorp_ustrmod *aMod,
	struct strm_queue *aQue,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

struct aorp_ustrmod_opv {
	aorp_ustrmod_open_fn       usm_open;    /* called while adding to a strum */
	aorp_ustrmod_close_fn      usm_close;   /* called before removing from a strum */
	aorp_ustrmod_connect_fn    usm_connect; /* called while module connecting [down,up] */
	aorp_ustrmod_canconnect_fn usm_cannconnect; /* called while module connecting [down,up] */
	aorp_ustrmod_poll_fn       usm_poll;
	aorp_ustrmod_putmsg_fn     usm_putmsg;
	aorp_ustrmod_service_fn    usm_service;
};

struct aorp_ustrmod {
	/* struct aorp_object_s @{ */
	aorp_hint_x                usm_hint;     /* should be 0 */
	u_int16_t                  usm_reserved; /* should be 0 */
	/* @} */

	aorp_ustrmod_retain_fn     usm_retain;
	aorp_ustrmod_release_fn    usm_release;

	struct aorp_ustrmod_opv   *usm_wrq_opv,
	                          *usm_rdq_opv;

	struct strm_modinfo       *usm_info;
	struct strm_modstat       *usm_stat;

	aorp_object_t              usm_strum;
	struct strm_queue         *usm_wrq,
	                          *usm_rdq;

	/* internal data */
};
# define aorp_ustrmod_p(x) ((struct aorp_ustrmod *)(x))


# define AORP_STRMOD_Retain(aMod, anErrPtr) \
	( _AORP_IS_AOBJECT(aMod) \
		? AorpRetain(aorp_object_p(aMod), anErrPtr) \
		: ((NULLF != aorp_ustrmod_p(aMod)->usm_retain) \
			? (*aorp_ustrmod_p(aMod)->usm_retain)(aMod, anErrPtr) \
			: BoS_NORMAL))

# define AORP_STRMOD_Release(aMod, Flags, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? AorpRelease(aorp_object_p(aMod), Flags, anErrPtr) \
		: ((NULLF != aorp_ustrmod_p(aMod)->usm_release) \
			? (*aorp_ustrmod_p(aMod)->usm_release)(aMod, Flags, anErrPtr) \
			: BoS_NORMAL))


# define AORP_STRMOD_Open(aMod, aStrum, aQuetype, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodOpen(aMod, aStrum, aQuetype, anErrPtr) \
		: ((StrmK_WRITE_QUE == aQuetype) \
			? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_open)(aMod, aStrum, aQuetype, anErrPtr) \
			: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_open)(aMod, aStrum, aQuetype, anErrPtr)))

# define AORP_STRMOD_Open_wrq(aMod, aStrum, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodOpen(aMod, aStrum, StrmK_WRITE_QUE, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_open)(aMod, aStrum, StrmK_WRITE_QUE, anErrPtr))

# define AORP_STRMOD_Open_rdq(aMod, aStrum, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodOpen(aMod, aStrum, StrmK_READ_QUE, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_open)(aMod, aStrum, StrmK_READ_QUE, anErrPtr))


# define AORP_STRMOD_Close(aMod, aStrum, aQuetype, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodClose(aMod, aStrum, aQuetype, anErrPtr) \
		: ((StrmK_WRITE_QUE == aQuetype) \
			? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_close)(aMod, aStrum, aQuetype, anErrPtr) \
			: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_close)(aMod, aStrum, aQuetype, anErrPtr)))

# define AORP_STRMOD_Close_wrq(aMod, aStrum, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodClose(aMod, aStrum, StrmK_WRITE_QUE, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_close)(aMod, aStrum, StrmK_WRITE_QUE, anErrPtr))

# define AORP_STRMOD_Close_rdq(aMod, aStrum, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodClose(aMod, aStrum, StrmK_READ_QUE, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_close)(aMod, aStrum, StrmK_READ_QUE, anErrPtr))


# define AORP_STRMOD_Connect(aMod, aQue, aQuetype, aConntype, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodConnect(aMod, aQue, aQuetype, aConntype, anErrPtr) \
		: ((StrmK_WRITE_QUE == aQuetype) \
			? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_connect)(aMod, aQue, aQuetype, aConntype, anErrPtr) \
			: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_connect)(aMod, aQue, aQuetype, aConntype, anErrPtr)))

# define AORP_STRMOD_Connect_wrq(aMod, aQue, aConntype, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodConnect(aMod, aQue, StrmK_WRITE_QUE, aConntype, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_connect)(aMod, aQue, StrmK_WRITE_QUE, aConntype, anErrPtr))

# define AORP_STRMOD_Connect_rdq(aMod, aQue, aConntype, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodConnect(aMod, aQue, StrmK_READ_QUE, aConntype, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_connect)(aMod, aQue, StrmK_READ_QUE, aConntype, anErrPtr))


# define AORP_STRMOD_Canconnect(aMod, aQue, aQuetype, aConntype, aRqInfo, aReInfo, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodCanconnect(aMod, aQue, aQuetype, aConntype, aRqInfo, aReInfo, anErrPtr) \
		: ((StrmK_WRITE_QUE == aQuetype) \
			? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_canconnect)(aMod, aQue, aQuetype, aConntype, aRqInfo, aReInfo, anErrPtr) \
			: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_canconnect)(aMod, aQue, aQuetype, aConntype, aRqInfo, aReInfo, anErrPtr)))

# define AORP_STRMOD_Canconnect_wrq(aMod, aQue, aConntype, aRqInfo, aReInfo, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodConnect(aMod, aQue, StrmK_WRITE_QUE, aConntype, aRqInfo, aReInfo, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_canconnect)(aMod, aQue, StrmK_WRITE_QUE, aConntype, aRqInfo, aReInfo, anErrPtr))

# define AORP_STRMOD_Canconnect_rdq(aMod, aQue, aConntype, aRqInfo, aReInfo, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodConnect(aMod, aQue, StrmK_READ_QUE, aConntype, aRqInfo, aReInfo, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_canconnect)(aMod, aQue, StrmK_READ_QUE, aConntype, aRqInfo, aReInfo, anErrPtr))


# define AORP_STRMOD_Poll(aMod, aQuetype, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodPoll(aMod, aQuetype, anErrPtr) \
		: ((StrmK_WRITE_QUE == aQuetype) \
			? ((NULLF != aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_poll) \
				? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_poll)(aMod, aQuetype, anErrPtr) \
				: BoS_STRMOD_AFUNC) \
			: ((NULLF != aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_poll) \
				? (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_poll)(aMod, aQuetype, anErrPtr) \
				: BoS_STRMOD_AFUNC)))

# define AORP_STRMOD_Poll_wrq(aMod, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodPoll(aMod, StrmK_WRITE_QUE, anErrPtr) \
		: ((NULLF != aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_poll) \
			? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_poll)(aMod, StrmK_WRITE_QUE, anErrPtr) \
			: BoS_STRMOD_AFUNC))

# define AORP_STRMOD_Poll_rdq(aMod, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodPoll(aMod, StrmK_READ_QUE, anErrPtr) \
		: ((NULLF != aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_poll) \
			? (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_poll)(aMod, StrmK_READ_QUE, anErrPtr) \
			: BoS_STRMOD_AFUNC))


# define AORP_STRMOD_Putmsg(aMod, aQue, aMsg, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodPutmsg(aMod, aQue, aMsg, anErrPtr) \
		: ((StrmK_WRITE_QUE == (aQue)->sq_type) \
			? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_putmsg)(aMod, aQue, aMsg, anErrPtr) \
			: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_putmsg)(aMod, aQue, aMsg, anErrPtr)))

# define AORP_STRMOD_Putmsg_wrq(aMod, aQue, aMsg, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodPutmsg(aMod, aQue, aMsg, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_putmsg)(aMod, aQue, aMsg, anErrPtr))

# define AORP_STRMOD_Putmsg_rdq(aMod, aQue, aMsg, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodPutmsg(aMod, aQue, aMsg, anErrPtr) \
		: (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_putmsg)(aMod, aQue, aMsg, anErrPtr))


# define AORP_STRMOD_Service(aMod, aQue, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodService(aMod, aQue, anErrPtr) \
		: ((StrmK_WRITE_QUE == (aQue)->sq_type) \
			? ((NULLF != aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_service) \
				? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_service)(aMod, aQue, anErrPtr) \
				: BoS_NORMAL) \
			: ((NULLF != aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_service) \
				? (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_service)(aMod, aQue, anErrPtr) \
				: BoS_NORMAL)))

# define AORP_STRMOD_Service_wrq(aMod, aQue, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodService(aMod, aQue, anErrPtr) \
		: ((NULLF != aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_service) \
			? (*aorp_ustrmod_p(aMod)->usm_wrq_opv->usm_service)(aMod, aQue, anErrPtr) \
			: BoS_STRMOD_AFUNC))

# define AORP_STRMOD_Service_rdq(aMod, aQue, anErrPtr) \
	(_AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodService(aMod, aQue, anErrPtr) \
		: ((NULLF != aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_service) \
			? (*aorp_ustrmod_p(aMod)->usm_rdq_opv->usm_service)(aMod, aQue, anErrPtr) \
			: BoS_STRMOD_AFUNC))


# define AORP_STRMOD_Getinfo(aMod, anInfoPtr, anErrPtr) \
	( _AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodGetinfo(aorp_object_p(aMod), anInfoPtr, anErrPtr) \
		: (*anInfoPtr = aorp_ustrmod_p(aMod)->usm_info, BoS_NORMAL))

# define AORP_STRMOD_Getstat(aMod, aStatPtr, anErrPtr) \
	( _AORP_IS_AOBJECT(aMod) \
		? _AorpStrmodGetstat(aorp_object_p(aMod), aStatPtr, anErrPtr) \
		: (*aStatPtr = aorp_ustrmod_p(aMod)->usm_stat, BoS_NORMAL))

/* ----------------------------------------------------------------------------
 * alloc declarations
 */

# define StrmK_ALLOC_DATA (0)
# define StrmK_ALLOC_MSG (1)
# define StrmK_ALLOC_QUE (2)
# define StrmK_ALLOC_MODNODE (3)

struct strm_alloctx;

typedef bo_pointer_t
(__myc_tcalltype__(mcall) *strm_alloctx_get_fn)(
# if __myc_require(prototypes)
	struct strm_alloctx *anAlloc,
	int anObjtype,
	size_t anExtraSize, /* for StrmK_ALLOC_DATA, ignored for other */
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

struct strm_alloctx {
	strm_alloctx_get_fn sma_get;

	/* for object of type StrmK_ALLOC_DATA */
	aorp_retain_cb      sma_dretain;
	aorp_release_cb     sma_drelease;

	/* for object of type StrmK_ALLOC_MSG or StrmK_ALLOC_QUE
	   or StrmK_ALLOC_MODNODE */
	aorp_retain_cb      sma_retain;
	aorp_release_cb     sma_release;

	/* internal data */
};

#endif /* !__BO_SERVICES_IO_STRMTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
