/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Idea is originated with Ritchie D.M., "A Stream Input-Output System"
 */

/*
 * Strum.h	(V.Sokur)
 * include/Bo/services/io/Strum.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_STRUM__
# define __BO_SERVICES_IO_STRUM__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/Strmtypes.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# include <Bo/services/Strmop.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1)))
AorpStrumStart(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpStrumStart(aThis, anErrPtr) \
	_A_call(start, strum, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpStrumStop(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpStrumStop(aThis, anErrPtr) \
	_A_call(stop, strum, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
_AorpStrumLock(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define _AorpStrumLock(aThis, anErrPtr) \
	_A_call(lock, strum, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
_AorpStrumUnlock(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define _AorpStrumUnlock(aThis, anErrPtr) \
	_A_call(unlock, strum, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpStrumFindmod(
	aorp_object_t aThis,
	char const *aModname,
	AORP_ONEPP_OF((struct aorp_ustrmod **, aorp_object_t *)) aModPtr,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned object, stored to the (*aModPtr), is not retained.
 */
#  define AorpStrumFindmod(aThis, aModname, aModPtr, anErrPtr) \
	_A_call(findmod, strum, aThis, anErrPtr, aModname, aModPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpStrumAddmod(
	aorp_object_t aThis,
	char const *aModname,
	AORP_ONEP_OF((struct aorp_ustrmod *, aorp_object_t)) aMod,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The module referenced by the parameter @mod is retained.
 */
# define AorpStrumAddmod(aThis, aModname, aMod, anErrPtr) \
	_A_call(addmod, strum, aThis, anErrPtr, aModname, aMod)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
AorpStrumRmmod(
	aorp_object_t aThis,
	char const *aModname,
	AORP_ONEPP_OF((struct aorp_ustrmod **, aorp_object_t *)) aModPtr,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned module, stored to the (*aModPtr), is not released.
 */
# define AorpStrumRmmod(aThis, aModname, aModPtr, anErrPtr) \
	_A_call(rmmod, strum, aThis, anErrPtr, aModname, aModPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpStrumDelmod(
	aorp_object_t aThis,
	char const *aModname,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The module referenced by the parameter @modname is released.
 */
# define AorpStrumDelmod(aThis, aModname, anErrPtr) \
	_A_call(delmod, strum, aThis, anErrPtr, aModname)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpStrumConnect(
	aorp_object_t aThis,
	size_t aNamec,
	char const *aModnamev[],
	struct aorp_error *anErrPtr
	);
 */
# define AorpStrumConnect(aThis, aNamec, aModnamev, anErrPtr) \
	_A_call(connect, strum, aThis, anErrPtr, (size_t)(aNamec), aModnamev)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
_AorpStrumPutnext(
	aorp_object_t aThis,
	struct strm_queue *aQue,
	struct strm_msg_ds *aMsg,
	struct aorp_error *anErrPtr
	);
 *
 * BoS_OK: success, yes
 * BoS_NORMAL: success, no
 */
# define _AorpStrumPutnext(aThis, aQue, aMsg, anErrPtr) \
	_A_call(putnext, strum, aThis, anErrPtr, aQue, aMsg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
_AorpStrumCanputnext(
	aorp_object_t aThis,
	struct strm_queue *aQue,
	struct aorp_error *anErrPtr
	);
 *
 * BoS_OK: success, yes
 * BoS_NORMAL: success, no
 */
# define _AorpStrumCanputnext(aThis, aQue, anErrPtr) \
	_A_call(canputnext, strum, aThis, anErrPtr, aQue)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
_AorpStrumPushwq(
	aorp_object_t aThis,
	struct strm_queue *aQue,
	struct strm_msg_ds *aMsg,
	struct aorp_error *anErrPtr
	);
 */
# define _AorpStrumPushwq(aThis, aQue, aMsg, anErrPtr) \
	_A_call(pushwq, strum, aThis, anErrPtr, aQue, aMsg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
_AorpStrumPushfwq(
	aorp_object_t aThis,
	struct strm_queue *aQue,
	struct strm_msg_ds *aMsg,
	struct aorp_error *anErrPtr
	);
 */
# define _AorpStrumPushfwq(aThis, aQue, aMsg, anErrPtr) \
	_A_call(pushfwq, strum, aThis, anErrPtr, aQue, aMsg)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2,3)))
_AorpStrumPopwq(
	aorp_object_t aThis,
	struct strm_queue *aQue,
	struct strm_msg_ds **aMsgPtr,
	struct aorp_error *anErrPtr
	);
 */
# define _AorpStrumPopwq(aThis, aQue, aMsgPtr, anErrPtr) \
	_A_call(popwq, strum, aThis, anErrPtr, aQue, aMsgPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
_AorpStrumGetalloc(
	aorp_object_t aThis,
	struct strm_alloctx const **anAllocPtr,
	struct aorp_error *anErrPtr
	);
 */
# define _AorpStrumGetalloc(aThis, anAllocPtr, anErrPtr) \
	_A_call(getalloc, strum, aThis, anErrPtr, anAllocPtr)

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_IO_STRUM__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
