/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Idea is originated with Ritchie D.M., "A Stream Input-Output System"
 */

/*
 * StrumP.h	(V.Sokur)
 * include/Bo/services/io/StrumP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_IO_STRUM_P__
# define __BO_SERVICES_IO_STRUM_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/aorp/Types.h>

/******************************************************************************
 * %% BeginSection: strum service declarations
 */
# pragma mark -
# pragma mark [ strum service declarations ]
# pragma mark -

/* ----------------------------------------------------------------------------
 * strum service
 * @{
 */

# include <Bo/services/Strmop.h>
# include <Bo/aorp/StdopP.h>

# define __aorpc_decl __aorpc_decl_service
# define __aorpc_service_name strum
# include <Bo/aorp/AorpC.h>

_A_service(strum)

# define AORP_strum_init_once AORP_any_init_once
# define AORP_strum_init_once_signature AORP_any_init_once_signature
# define AORP_strum_fini_once AORP_any_fini_once
# define AORP_strum_fini_once_signature AORP_any_fini_once_signature
# define AORP_strum_pod_init AORP_any_pod_init
# define AORP_strum_pod_init_signature AORP_any_pod_init_signature
# define AORP_strum_pod_fini AORP_any_pod_fini
# define AORP_strum_pod_fini_signature AORP_any_pod_fini_signature

_A_op(start, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr
	)) __attribute__((nonnull(1)));

# define AORP_strum_start "start-strum"
# define AORP_strum_start_signature AORP_STD_SIGNATURE

_A_op(stop, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr
	)) __attribute__((nonnull(1)));

# define AORP_strum_stop "stop-strum"
# define AORP_strum_stop_signature AORP_STD_SIGNATURE

_A_op(lock, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr
	)) __attribute__((nonnull(1)));

# define AORP_strum_lock "strum-global-lock"
# define AORP_strum_lock_signature AORP_STD_SIGNATURE

_A_op(unlock, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr
	)) __attribute__((nonnull(1)));

# define AORP_strum_unlock "strum-global-unlock"
# define AORP_strum_unlock_signature AORP_STD_SIGNATURE

_A_op(findmod, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aModname,
		AORP_ONEPP_OF((struct aorp_ustrmod **, aorp_object_t *)) aModPtr
	)) __attribute__((nonnull(1,3)));

# define AORP_strum_findmod "find-module"
# define AORP_strum_findmod_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_op(addmod, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aModname,
		AORP_ONEP_OF((struct aorp_ustrmod *, aorp_object_t)) aMod
	)) __attribute__((nonnull(1,4)));

# define AORP_strum_addmod "add-module"
# define AORP_strum_addmod_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_POINTER

_A_op(rmmod, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aModname,
		AORP_ONEPP_OF((struct aorp_ustrmod **, aorp_object_t *)) aModPtr
	)) __attribute__((nonnull(1,3,4)));

# define AORP_strum_rmmod "remove-module"
# define AORP_strum_rmmod_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_op(delmod, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		char const *aModname
	)) __attribute__((nonnull(1,3)));

# define AORP_strum_delmod "delete-module"
# define AORP_strum_delmod_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_CSTRING

_A_op(connect, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		size_t aNamec,
		char const *aModnamev[]
	)) __attribute__((nonnull(1)));

# define AORP_strum_connect "connect"
# define AORP_strum_connect_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_SIZE_T \
	" " _AORP_SIGN_CSTRING _AORP_SMOD_VECTOR

_A_op(reserved1, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr
	)) __attribute__((nonnull(1)));

# define AORP_strum_reserved1 "reserved1"
# define AORP_strum_reserved1_signature AORP_STD_SIGNATURE

/*
 * BoS_OK: success, yes
 * BoS_NORMAL: success, no
 */
_A_op(putnext, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue,
		struct strm_msg_ds *aMsg
	)) __attribute__((nonnull(1,3,4)));

# define AORP_strum_putnext "put-message-to-next-queue"
# define AORP_strum_putnext_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

/*
 * BoS_OK: success, yes
 * BoS_NORMAL: success, no
 */
_A_op(canputnext, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue
	)) __attribute__((nonnull(1,3)));

# define AORP_strum_canputnext "can-put-message-to-next-queue"
# define AORP_strum_canputnext_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER

_A_op(pushwq, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue,
		struct strm_msg_ds *aMsg
	)) __attribute__((nonnull(1,3,4)));

# define AORP_strum_pushwq "pushback-message-to-wait-queue"
# define AORP_strum_pushwq_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

_A_op(pushfwq, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue,
		struct strm_msg_ds *aMsg
	)) __attribute__((nonnull(1,3,4)));

# define AORP_strum_pushfwq "pushfront-message-to-wait-queue"
# define AORP_strum_pushfwq_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER

_A_op(popwq, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_queue *aQue,
		struct strm_msg_ds **aMsgPtr
	)) __attribute__((nonnull(1,3,4)));

# define AORP_strum_popwq "popback-message-from-wait-queue"
# define AORP_strum_popwq_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_op(getalloc, (
		struct aorp_closure *aClosure,
		struct aorp_error *anErrPtr,
		struct strm_alloctx const **anAllocPtr
	)) __attribute__((nonnull(1,3)));

# define AORP_strum_getalloc "get-data-allocator"
# define AORP_strum_getalloc_signature AORP_STD_SIGNATURE \
	" " _AORP_SIGN_POINTER _AORP_SMOD_POINTER

_A_end;

# include <Bo/aorp/AorpCend.h>

/* @} */

#endif /* !__BO_SERVICES_IO_STRUM_P__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
