/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Jobtypes.h	(V.Sokur)
 * include/Bo/services/jobcntl/Jobtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_JOBCNTL_JOBTYPES__
# define __BO_SERVICES_JOBCNTL_JOBTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/*
 * EXEC:
 * -1: error, continue exec;
 *  0: success, finished, not exec longer, removed;
 *  1: success, finished, not exec longer;
 *  2: success, continue exec;
 */
# define BoS_JOB_ERR BoS_ERR
# define BoS_JOB_FINC 0
# define BoS_JOB_FINW 1
# define BoS_JOB_CONT 2
# define BoS_JOB_RLPAUSE 3

# define BoS_JOB_ISFIN(aStatus) \
	(BoS_JOB_FINC == (aStatus) || BoS_JOB_FINW == (aStatus))


struct aorp_ujob;

typedef aorp_retain_cb  aorp_ujob_retain_fn;
typedef aorp_release_cb aorp_ujob_release_fn;

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ujob_schedule_fn)(
# if __myc_require(prototypes)
	struct aorp_ujob *aJob,
	aorp_object_t anExecutor,
	unsigned aJobId,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ujob_unschedule_fn)(
# if __myc_require(prototypes)
	struct aorp_ujob *aJob,
	aorp_object_t anExecutor,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_ujob_exec_fn)(
# if __myc_require(prototypes)
	struct aorp_ujob *aJob,
	bo_pointer_t aParam,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1)));

struct aorp_ujob {
	/* struct aorp_object_s @{ */
	aorp_hint_x               uj_hint;     /* should be 0 */
	u_int16_t                 uj_reserved; /* should be 0 */
	/* @} */

	aorp_ujob_retain_fn       uj_retain;
	aorp_ujob_release_fn      uj_release;
	aorp_ujob_schedule_fn     uj_schedule;
	aorp_ujob_unschedule_fn   uj_unschedule;
	aorp_ujob_exec_fn         uj_exec;

	/* internal data */
};
# define aorp_ujob_p(x) ((struct aorp_ujob *)(x))

# define AORP_JOB_Retain(aJob, anErrPtr) \
	( (0 != AORP_OBJECT_Hint(aJob)) \
		? AorpRetain(aorp_object_p(aJob), anErrPtr) \
		: ((NULLF != aorp_ujob_p(aJob)->uj_retain) \
				? (*aorp_ujob_p(aJob)->uj_retain)(aJob, anErrPtr) \
				: BoS_NORMAL))

# define AORP_JOB_Release(aJob, Flags, anErrPtr) \
	( (0 != AORP_OBJECT_Hint(aJob)) \
		? AorpRelease(aorp_object_p(aJob), Flags, anErrPtr) \
		: ((NULLF != aorp_ujob_p(aJob)->uj_release) \
				? (*aorp_ujob_p(aJob)->uj_release)(aJob, Flags, anErrPtr) \
				: BoS_NORMAL))

# define AORP_JOB_Schedule(aJob, anExecutor, aJobId, anErrPtr) \
	( AORP_IS_SOBJECT(aJob) /* assumed, that service-object is of job service */ \
		? _A_socall(schedule, job, aJob, anErrPtr, anExecutor, aJobId) \
		: (_AORP_IS_AOBJECT(aJob) \
			? _AorpJobSchedule(aJob, anExecutor, aJobId, anErrPtr) \
			: ((NULLF != aorp_ujob_p(aJob)->uj_schedule) \
					? (*aorp_ujob_p(aJob)->uj_schedule)(aorp_ujob_p(aJob), anExecutor, aJobId, anErrPtr) \
					: BoS_NORMAL)))

# define AORP_JOB_Unschedule(aJob, anExecutor, anErrPtr) \
	( AORP_IS_SOBJECT(aJob) /* assumed, that service-object is of job service */ \
		? _A_socall(unschedule, job, aJob, anErrPtr, anExecutor) \
		: (_AORP_IS_AOBJECT(aJob) \
			? _AorpJobUnschedule(aJob, anExecutor, anErrPtr) \
			: ((NULLF != aorp_ujob_p(aJob)->uj_unschedule) \
					? (*aorp_ujob_p(aJob)->uj_unschedule)(aorp_ujob_p(aJob), anExecutor, anErrPtr) \
					: BoS_NORMAL)))

# define AORP_JOB_Exec(aJob, aParam, anErrPtr) \
	( AORP_IS_SOBJECT(aJob) /* assumed, that service-object is of job service */ \
		? _A_socall(exec, job, aJob, anErrPtr, aParam) \
		: (_AORP_IS_AOBJECT(aJob) \
			? _AorpJobExec(aJob, aParam, anErrPtr) \
			: ((NULLF != aorp_ujob_p(aJob)->uj_exec) \
					? (*aorp_ujob_p(aJob)->uj_exec)(aorp_ujob_p(aJob), aParam, anErrPtr) \
					: BoS_NORMAL)))

#endif /* !__BO_SERVICES_JOBCNTL_JOBTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
