/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Runloop.h	(V.Sokur)
 * include/Bo/services/jobcntl/Runloop.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_JOBCNTL_RUNLOOP__
# define __BO_SERVICES_JOBCNTL_RUNLOOP__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/jobcntl/RunloopP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopRun(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopRun(aThis, anErrPtr) \
	_A_call(run, runloop, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopCancel(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
#if 0
# define AorpRunloopCancel(aThis, anErrPtr) \
	_A_call(cancel, runloop, aThis, anErrPtr)
#else
# define AorpRunloopCancel(aThis, anErrPtr) \
	AorpRunloopCancelf(aThis, 0, anErrPtr)
#endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopSuspend(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
#if 0
# define AorpRunloopSuspend(aThis, anErrPtr) \
	_A_call(suspend, runloop, aThis, anErrPtr)
#else
# define AorpRunloopSuspend(aThis, anErrPtr) \
	AorpRunloopSuspendf(aThis, 0, anErrPtr)
#endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopResume(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
#if 0
# define AorpRunloopResume(aThis, anErrPtr) \
	_A_call(resume, runloop, aThis, anErrPtr)
#else
# define AorpRunloopResume(aThis, anErrPtr) \
	AorpRunloopResumef(aThis, 0, anErrPtr)
#endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopLockJL(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopLockJL(aThis, anErrPtr) \
	_A_call(lockjl, runloop, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopUnlockJL(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopUnlockJL(aThis, anErrPtr) \
	_A_call(unlockjl, runloop, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopExecjob(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopExecjob(aThis, aJobId, anErrPtr) \
	_A_call(execjob, runloop, aThis, anErrPtr, aJobId)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopExecall(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopExecall(aThis, anErrPtr) \
	AorpRunloopExecjob(aThis, 0, anErrPtr)

# pragma mark -

/*
AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
__attribute__((nonnull(1)))
AorpRunloopFindjob(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned job is not retained.
 */

# if __myc_support(macro_retval)
#  define AorpRunloopFindjob(aThis, aJobId, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(findjob, runloop, aThis, anErrPtr, aJobId); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_PTR_FROM_STATUS(___retval) : BoS_EPTR); \
	})
# else
__myc_extern_inline__ AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
AorpRunloopFindjob(aorp_object_t aThis, unsigned aJobId, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(findjob, runloop, aThis, anErrPtr, aJobId);
	return (BoSP_NORMAL(retval) ? BO_PTR_FROM_STATUS(retval) : BoS_EPTR);
}
# endif

# pragma mark -

/*
AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
__attribute__((nonnull(1)))
AorpRunloopFindjob_nlk(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned job is not retained.
 */

# if __myc_support(macro_retval)
#  define AorpRunloopFindjob_nlk(aThis, aJobId, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(findjob_nlk, runloop, aThis, anErrPtr, aJobId); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_PTR_FROM_STATUS(___retval) : BoS_EPTR); \
	})
# else
__myc_extern_inline__ AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
AorpRunloopFindjob_nlk(aorp_object_t aThis, unsigned aJobId, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(findjob_nlk, runloop, aThis, anErrPtr, aJobId);
	return (BoSP_NORMAL(retval) ? BO_PTR_FROM_STATUS(retval) : BoS_EPTR);
}
# endif

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpRunloopAddjob(
	aorp_object_t aThis,
	AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The job referenced by the parameter @job is retained.
 */
#if 0
# define AorpRunloopAddjob(aThis, aJob, anErrPtr) \
	((bo_index_t) _A_call(addjob, runloop, aThis, anErrPtr, aJob))
#else
# define AorpRunloopAddjob(aThis, aJob, anErrPtr) \
	AorpRunloopAddjob3(aThis, aJob, NULL, NULL, anErrPtr)
#endif

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpRunloopAddjob_nlk(
	aorp_object_t aThis,
	AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The job referenced by the parameter @job is retained.
 */
#if 0
# define AorpRunloopAddjob_nlk(aThis, aJob, anErrPtr) \
	((bo_index_t) _A_call(addjob_nlk, runloop, aThis, anErrPtr, aJob))
#else
# define AorpRunloopAddjob_nlk(aThis, aJob, anErrPtr) \
	AorpRunloopAddjob3_nlk(aThis, aJob, NULL, NULL, anErrPtr)
#endif

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpRunloopAddjob3(
	aorp_object_t aThis,
	AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob,
	bo_pointer_t aJobParm,
	struct aorp_error *aJobErrReport,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The job referenced by the parameter @job is retained.
 * The lifetime of the parameters @job-parm and @job-err-report
 * may not be controlled by the runloop (impl dependent).
 */
# define AorpRunloopAddjob3(aThis, aJob, aJobParm, aJobErrReport, anErrPtr) \
	((bo_index_t) _A_call(addjob3, runloop, aThis, anErrPtr, aJob, aJobParm, aJobErrReport))

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpRunloopAddjob3_nlk(
	aorp_object_t aThis,
	AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob,
	bo_pointer_t aJobParm,
	struct aorp_error *aJobErrReport,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The job referenced by the parameter @job is retained.
 * The lifetime of the parameters @job-parm and @job-err-report
 * may not be controlled by the runloop (impl dependent).
 */
# define AorpRunloopAddjob3_nlk(aThis, aJob, aJobParm, aJobErrReport, anErrPtr) \
	((bo_index_t) _A_call(addjob3_nlk, runloop, aThis, anErrPtr, aJob, aJobParm, aJobErrReport))

# pragma mark -

/*
AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
__attribute__((nonnull(1,3)))
AorpRunloopRmjob(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned job is not released.
 */

# if __myc_support(macro_retval)
#  define AorpRunloopRmjob(aThis, aJobId, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(rmjob, runloop, aThis, anErrPtr, aJobId); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_PTR_FROM_STATUS(___retval) : BoS_EPTR); \
	})
# else
__myc_extern_inline__ AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
AorpRunloopRmjob(aorp_object_t aThis, unsigned aJobId, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(rmjob, runloop, aThis, anErrPtr, aJobId);
	return (BoSP_NORMAL(retval) ? BO_PTR_FROM_STATUS(retval) : BoS_EPTR);
}
# endif

# pragma mark -

/*
AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
__attribute__((nonnull(1,3)))
AorpRunloopRmjob_nlk(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned job is not released.
 */

# if __myc_support(macro_retval)
#  define AorpRunloopRmjob_nlk(aThis, aJobId, anErrPtr) \
	__extension__ ({ \
		bo_stsptr_t ___retval = _A_call(rmjob_nlk, runloop, aThis, anErrPtr, aJobId); \
		/* return */ (BoSP_NORMAL(___retval) ? BO_PTR_FROM_STATUS(___retval) : BoS_EPTR); \
	})
# else
__myc_extern_inline__ AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))
AorpRunloopRmjob_nlk(aorp_object_t aThis, unsigned aJobId, struct aorp_error *anErrPtr)
{
	bo_stsptr_t retval = _A_call(rmjob_nlk, runloop, aThis, anErrPtr, aJobId);
	return (BoSP_NORMAL(retval) ? BO_PTR_FROM_STATUS(retval) : BoS_EPTR);
}
# endif

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopDeljob(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The job referenced by the parameter @jobid is released.
 */
# define AorpRunloopDeljob(aThis, aJobId, anErrPtr) \
	_A_call(deljob, runloop, aThis, anErrPtr, aJobId)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopDeljob_nlk(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The job referenced by the parameter @jobid is released.
 */
# define AorpRunloopDeljob_nlk(aThis, aJobId, anErrPtr) \
	_A_call(deljob_nlk, runloop, aThis, anErrPtr, aJobId)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopJobstatus(
	aorp_object_t aThis,
	unsigned aJobId,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopJobstatus(aThis, aJobId, anErrPtr) \
	_A_call(getjobstatus, runloop, aThis, anErrPtr, aJobId)

# pragma mark -

/*
bo_state_t
__attribute__((nonnull(1)))
AorpRunloopState(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopState(aThis, anErrPtr) \
	((bo_state_t) _A_call(getstate, runloop, aThis, anErrPtr))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpRunloopGetperiod(
	aorp_object_t aThis,
	struct timespec *anInterval,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopGetperiod(aThis, anInterval, anErrPtr) \
	_A_call(getperiod, runloop, aThis, anErrPtr, anInterval)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,2)))
AorpRunloopSetperiod(
	aorp_object_t aThis,
	struct timespec const *aRq,
	struct timespec *aResult,      // = NULL
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopSetperiod(aThis, aRq, aResult, anErrPtr) \
	_A_call(setperiod, runloop, aThis, anErrPtr, aRq, aResult)

# pragma mark -

/*
int
__attribute__((nonnull(1)))
AorpRunloopMode(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopMode(aThis, anErrPtr) \
	((int) _A_call(getmode, runloop, aThis, anErrPtr))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopCancelf(
	aorp_object_t aThis,
	aorp_opflags_t Flags, // = 0, AorpF_OP_WAIT
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopCancelf(aThis, Flags, anErrPtr) \
	_A_call(cancelf, runloop, aThis, anErrPtr, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopSuspendf(
	aorp_object_t aThis,
	aorp_opflags_t Flags, // = 0, AorpF_OP_WAIT
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopSuspendf(aThis, Flags, anErrPtr) \
	_A_call(suspendf, runloop, aThis, anErrPtr, (aorp_opflags_t)(Flags))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpRunloopResumef(
	aorp_object_t aThis,
	aorp_opflags_t Flags, // = 0, AorpF_OP_WAIT
	struct aorp_error *anErrPtr
	);
 */
# define AorpRunloopResumef(aThis, Flags, anErrPtr) \
	_A_call(resumef, runloop, aThis, anErrPtr, (aorp_opflags_t)(Flags))

__MYC_END_CDECLS

# pragma mark -

# include <Bo/libc/Time.h>

__MYC_BEGIN_CDECLS

/*
bo_msec_t
__attribute__((nonnull(1)))
AorpRunloopGetperiod_ms(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */

# if __myc_support(macro_retval)
#  define AorpRunloopGetperiod_ms(aThis, anErrPtr) \
	__extension__ ({ \
		bo_msec_t ___period; \
		bo_status_t ___status; \
		struct timespec ___ts; \
		___period = BoK_INVAL_DURATION; \
		___status = AorpRunloopGetperiod(aThis, &___ts, anErrPtr); \
		if (BoS_SUCCESS(___status)) { \
			if (BoK_INFINITE_DURATION != (___period = (bo_msec_t)___ts.tv_sec)) \
				___period = BO_TIMESPEC_AsMsec(&___ts); \
		} \
		/* return */ (___period); \
	})
# else
__myc_extern_inline__ bo_msec_t
AorpRunloopGetperiod_ms(aorp_object_t aThis, struct aorp_error *anErrPtr)
{
	bo_msec_t ___period;
	bo_status_t ___status;
	struct timespec ___ts;

	___period = BoK_INVAL_DURATION;
	___status = AorpRunloopGetperiod(aThis, &___ts, anErrPtr);
	if (BoS_SUCCESS(___status)) {
		if (BoK_INFINITE_DURATION != (___period = (bo_msec_t)___ts.tv_sec))
			___period = BO_TIMESPEC_AsMsec(&___ts);
	}
	return (___period);
}
# endif

# pragma mark -

/*
bo_usec_t
__attribute__((nonnull(1)))
AorpRunloopGetperiod_us(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */

# if __myc_support(macro_retval)
#  define AorpRunloopGetperiod_us(aThis, anErrPtr) \
	__extension__ ({ \
		bo_usec_t ___period; \
		bo_status_t ___status; \
		struct timespec ___ts; \
		___period = BoK_INVAL_DURATION; \
		___status = AorpRunloopGetperiod(aThis, &___ts, anErrPtr); \
		if (BoS_SUCCESS(___status)) { \
			if (BoK_INFINITE_DURATION != (___period = (bo_usec_t)___ts.tv_sec)) \
				___period = BO_TIMESPEC_AsUsec(&___ts); \
		} \
		/* return */ (___period); \
	})
# else
__myc_extern_inline__ bo_usec_t
AorpRunloopGetperiod_us(aorp_object_t aThis, struct aorp_error *anErrPtr)
{
	bo_usec_t ___period;
	bo_status_t ___status;
	struct timespec ___ts;

	___period = BoK_INVAL_DURATION;
	___status = AorpRunloopGetperiod(aThis, &___ts, anErrPtr);
	if (BoS_SUCCESS(___status)) {
		if (BoK_INFINITE_DURATION != (___period = (bo_usec_t)___ts.tv_sec))
			___period = BO_TIMESPEC_AsUsec(&___ts);
	}
	return (___period);
}
# endif

# pragma mark -

/*
bo_msec_t
__attribute__((nonnull(1)))
AorpRunloopSetperiod_ms(
	aorp_object_t aThis,
	bo_msec_t aRq,
	struct aorp_error *anErrPtr
	);
 */

# if __myc_support(macro_retval)
#  define AorpRunloopSetperiod_ms(aThis, aRq, anErrPtr) \
	__extension__ ({ \
		bo_status_t ___status; \
		bo_msec_t ___period; \
		struct timespec ___rqts, ___rests; \
		___period = (aRq); \
		___rqts.tv_sec = (tv_time_t)___period; \
		___rqts.tv_nsec = 0; \
		if (BoK_INFINITE_DURATION != ___period) \
			BO_TIMESPEC_FromMsec(&___rqts, ___period); \
		___status = AorpRunloopSetperiod(aThis, &___rqts, &___rests, anErrPtr); \
		___period = BoS_ERR; \
		if (BoS_SUCCESS(status)) \
			___period = BO_TIMESPEC_AsMsec(&___rests); \
		/* return */ (___period); \
	})
# else
__myc_extern_inline__ bo_msec_t
AorpRunloopSetperiod_ms(aorp_object_t aThis,
	bo_msec_t aRq, struct aorp_error *anErrPtr)
{
	bo_status_t status;
	struct timespec rqts, rests;

	rqts.tv_sec = (tv_time_t)aRq;
	rqts.tv_nsec = 0;
	if (BoK_INFINITE_DURATION != aRq)
		BO_TIMESPEC_FromMsec(&rqts, aRq);
	status = AorpRunloopSetperiod(aThis, &rqts, &rests, anErrPtr);
	aRq = BoS_ERR;
	if (BoS_SUCCESS(status))
		aRq = BO_TIMESPEC_AsMsec(&rests);

	return (aRq);
}
# endif

/*
bo_usec_t
__attribute__((nonnull(1)))
AorpRunloopSetperiod_us(
	aorp_object_t aThis,
	bo_usec_t aRq,
	struct aorp_error *anErrPtr
	);
 */

# if __myc_support(macro_retval)
#  define AorpRunloopSetperiod_us(aThis, aRq, anErrPtr) \
	__extension__ ({ \
		bo_status_t ___status; \
		bo_usec_t ___period; \
		struct timespec ___rqts, ___rests; \
		___period = (aRq); \
		___rqts.tv_sec = (tv_time_t)___period; \
		___rqts.tv_nsec = 0; \
		if (BoK_INFINITE_DURATION != ___period) \
			BO_TIMESPEC_FromUsec(&___rqts, ___period); \
		___status = AorpRunloopSetperiod(aThis, &___rqts, &___rests, anErrPtr); \
		___period = BoS_ERR; \
		if (BoS_SUCCESS(status)) \
			___period = BO_TIMESPEC_AsUsec(&___rests); \
		/* return */ (___period); \
	})
# else
__myc_extern_inline__ bo_usec_t
AorpRunloopSetperiod_us(aorp_object_t aThis,
	bo_usec_t aRq, struct aorp_error *anErrPtr)
{
	bo_status_t status;
	struct timespec rqts, rests;

	rqts.tv_sec = (tv_time_t)aRq;
	rqts.tv_nsec = 0;
	if (BoK_INFINITE_DURATION != aRq)
		BO_TIMESPEC_FromUsec(&rqts, aRq);
	status = AorpRunloopSetperiod(aThis, &rqts, &rests, anErrPtr);
	aRq = BoS_ERR;
	if (BoS_SUCCESS(status))
		aRq = BO_TIMESPEC_AsUsec(&rests);

	return (aRq);
}
# endif

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_JOBCNTL_RUNLOOP__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
