/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * RunloopP.h	(V.Sokur)
 * include/Bo/services/jobcntl/RunloopP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_JOBCNTL_RUNLOOP_P__
# define __BO_SERVICES_JOBCNTL_RUNLOOP_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/jobcntl/Runlooptypes.h>

/******************************************************************************
 * %% BeginSection: runloop service declarations
 */
# pragma mark -
# pragma mark [ runloop service declarations ]
# pragma mark -

# define __aorpc_need_service_runloop 1

#endif /* !__BO_SERVICES_JOBCNTL_RUNLOOP_P__ */

#pragma mark -

#if __aorpc_uses_service(runloop)
# undef __aorpc_need_service_runloop

/*
 * runloop service
 * @{
 */

/*
 * runloop {\object(instance)}
 */
/*
# define AorpK_runloop_service (1111)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name runloop
# include <Bo/aorp/AorpCsrvc.h>

_A_service(runloop)

# if __aorpc_interface

/* external */
struct timespec;

/*
 * state domain
 */
# ifndef BoK_STATE_RL
#  define BoK_STATE_RL __myc_mk4ccs("runl")
#  define _BoK_STATE_RL __myc_mk4cc('r','u','n','l')
# endif

/*
 * values for runloop state [advisory]
 */
# define AorpK_RL_INVAL_STATE BoK_INVAL_STATE
# define AorpK_RL_NULL_STATE BoK_NULL_STATE
# define AorpK_RL_UNK_STATE BoK_UNK_STATE

/* CANCEL: {idle,working,suspended}->init */
# define AorpK_RL_INITED_STATE BoK_INITED_STATE /* ->: run(idle->working) */

/* RUN: {init}->{idle+working}; RESUME: {suspended}->{idle+working}; EXECJOB */
# define AorpK_RL_IDLE_STATE BoK_MUI_IDLE_STATE

/* RESUME: {suspended}->{idle+working}; EXECJOB */
# define AorpK_RL_WORKING_STATE BoK_MUI_PLAYING_STATE

/* SUSPEND: {idle,working}->{suspended} */
# define AorpK_RL_SUSPENDED_STATE BoK_MUI_PAUSED_STATE

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(run, "runloop::run", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(cancel, "runloop::cancel", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(suspend, "runloop::suspend", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(resume, "runloop::resume", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(lockjl, "runloop::lock-job-list", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(unlockjl, "runloop::unlock-job-list", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(execjob, "runloop::execute-job", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1)))
	_A_op(findjob, "runloop::find-job", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1)))
	_A_op(findjob_nlk, "runloop::find-job-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1)))
	_A_op(addjob, "runloop::add-job", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob
		)),

	_A_attribute((nonnull(1)))
	_A_op(addjob_nlk, "runloop::add-job-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmjob, "runloop::remove-job", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmjob_nlk, "runloop::remove-job-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1)))
	_A_op(deljob, "runloop::delete-job", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1)))
	_A_op(deljob_nlk, "runloop::delete-job-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1)))
	_A_op(getjobstatus, "runloop::get-job-status", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned aJobId
		)),

	_A_attribute((nonnull(1)))
	_A_op(getstate, "runloop::get-state", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(getperiod, "runloop::get-timer-period", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct timespec *anInterval
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(setperiod, "runloop::set-timer-period", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct timespec const *aRq,
			struct timespec *aResult
		)),

	_A_attribute((nonnull(1)))
	_A_op(addjob3, "runloop::add-job3", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob,
			bo_pointer_t aJobParm,
			struct aorp_error *aJobErrReport
		)),

	_A_attribute((nonnull(1)))
	_A_op(addjob3_nlk, "runloop::add-job3-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) aJob,
			bo_pointer_t aJobParm,
			struct aorp_error *aJobErrReport
		)),

	_A_attribute((nonnull(1)))
	_A_op(getmode, "runloop::get-mode", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(cancelf, "runloop::cancel-with-flags", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_opflags_t Flags
		)),

	_A_attribute((nonnull(1)))
	_A_op(suspendf, "runloop::suspend-with-flags", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_opflags_t Flags
		)),

	_A_attribute((nonnull(1)))
	_A_op(resumef, "runloop::resume-with-flags", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_opflags_t Flags
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(runloop) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
