/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Sched.h	(S.Makhmali)
 * include/Bo/services/jobcntl/Sched.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_JOBCNTL_SCHED__
# define __BO_SERVICES_JOBCNTL_SCHED__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/jobcntl/SchedP.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
ssize_t
__attribute__((nonnull(1)))
AorpSchedParseent(
	aorp_object_t aThis,
	char const *aStr,
	struct aorp_sched_entry *anEntry,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedParseent(aThis, aStr, anEntry, anErrPtr) \
	_A_call(parseent, sched, aThis, anErrPtr, aStr, anEntry)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1)))
AorpSchedParseent2(
	struct aorp_closure *aClosure,
	char *aStr,
	char aDelim[],
	struct bo_strtok_ctx *aTokctx,
	struct aorp_sched_entry *anEntry,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedParseent2(aThis, aStr, aDelim, aTokctx, anEntry, anErrPtr) \
	_A_call(parseent2, sched, aThis, anErrPtr, aStr, aDelim, aTokctx, anEntry)

# pragma mark -

/*
ssize_t
__attribute__((nonnull(1,2)))
AorpSchedPrintent(
	aorp_object_t aThis,
	struct aorp_sched_tmval aTable[],
	size_t aCount,
	int anActsw,
	char const *anActstr, // <process-name | <id>
	char const *anActargs,
	char *aBuffer,
	size_t aBufsz,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedPrintent(aThis, aTable, aCount, anActsw, anActstr, anActargs, aBuffer, aBufsz, anErrPtr) \
	_A_call(printent, sched, aThis, anErrPtr, aTable, (size_t)(aCount), \
		anActsw, anActstr, anActargs, aBuffer, (size_t)(aBufsz))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedLockEL(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedLockEL(aThis, anErrPtr) \
	_A_call(lockel, sched, aThis, anErrPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedUnlockEL(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedUnlockEL(aThis, anErrPtr) \
	_A_call(unlockel, sched, aThis, anErrPtr)

# pragma mark -

/*
bo_index_t
__attribute__((nonnull(1)))
AorpSchedAddent(
	aorp_object_t aThis,
	struct aorp_sched_entry *anEntry,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedAddent(aThis, anEntry, anErrPtr) \
	((bo_index_t) _A_call(addent, sched, aThis, anErrPtr, anEntry))

# pragma mark -

 /*
 bo_index_t
 __attribute__((nonnull(1)))
 AorpSchedAddent_nlk(
	aorp_object_t aThis,
	struct aorp_sched_entry *anEntry,
	struct aorp_error *anErrPtr
 );
 */
# define AorpSchedAddent_nlk(aThis, anEntry, anErrPtr) \
	((bo_index_t) _A_call(addent_nlk, sched, aThis, anErrPtr, anEntry))

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1,3)))
AorpSchedRment(
	aorp_object_t aThis,
	unsigned anEntId,
	struct aorp_sched_entry **anEntPtr,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The returned entry, stored to the (*anEntPtr), is not released.
 */
# define AorpSchedRment(aThis, anEntId, anEntPtr, anErrPtr) \
	_A_call(rment, sched, aThis, anErrPtr, anEntId, anEntPtr)

# pragma mark -

 /*
 bo_status_t
 __attribute__((nonnull(1,3)))
 AorpSchedRment_nlk(
	aorp_object_t aThis,
	unsigned anEntId,
	struct aorp_sched_entry **anEntPtr,
	struct aorp_error *anErrPtr
 );
 *
 * NOTE!!!
 * The returned entry, stored to the (*anEntPtr), is not released.
 */
# define AorpSchedRment_nlk(aThis, anEntId, anEntPtr, anErrPtr) \
	_A_call(rment_nlk, sched, aThis, anErrPtr, anEntId, anEntPtr)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedDelent(
	aorp_object_t aThis,
	unsigned anEntId,
	struct aorp_error *anErrPtr
	);
 *
 * NOTE!!!
 * The entry referenced by the parameter @entid is released.
 */
# define AorpSchedDelent(aThis, anEntId, anErrPtr) \
	_A_call(delent, sched, aThis, anErrPtr, anEntId)

# pragma mark -

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedDelent_nlk(
	aorp_object_t aThis,
	unsigned anEntId,
	struct aorp_error *anErrPtr
);
*
* NOTE!!!
* The entry referenced by the parameter @entid is released.
*/
# define AorpSchedDelent_nlk(aThis, anEntId, anErrPtr) \
	_A_call(delent_nlk, sched, aThis, anErrPtr, anEntId)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedSetaction(
	aorp_object_t aThis,
	unsigned anEntId,
	int anActsw,
	intptr_t aPtr,
	struct aorp_error *anErrPtr
);
*/
# define AorpSchedSetaction(aThis, anEntId, anActsw, aPtr, anErrPtr) \
	_A_call(setaction, sched, aThis, anErrPtr, anEntId, anActsw, (intptr_t)(aPtr))

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedUnsetaction(
	aorp_object_t aThis,
	unsigned anEntId,
	struct aorp_error *anErrPtr
);
*/
# define AorpSchedUnsetaction(aThis, anEntId, anErrPtr) \
	_A_call(unsetaction, sched, aThis, anErrPtr, anEntId)

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedSetjtable(
	aorp_object_t aThis,
	aorp_sched_jtelem *aTable,
	size_t anElemc,
	struct aorp_error *anErrPtr
);
*/
# define AorpSchedSetjtable(aThis, aTable, anElemc, anErrPtr) \
	_A_call(setjtable, sched, aThis, anErrPtr, aTable, (size_t)(anElemc))

/*
bo_status_t
__attribute__((nonnull(1)))
AorpSchedUnsetjtable(
	aorp_object_t aThis,
	struct aorp_error *anErrPtr
);
*/
# define AorpSchedUnsetjtable(aThis, anErrPtr) \
	_A_call(unsetjtable, sched, aThis, anErrPtr)

/*
bo_stsbool_t
__attribute__((nonnull(1)))
AorpSchedChecktime(
	aorp_object_t aThis,
	bo_utime_t aTime,
	struct aorp_sched_entry *anEntry,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedChecktime(aThis, aTime, anEntry, anErrPtr) \
	_A_call(checktime, sched, aThis, anErrPtr, (bo_utime_t)(aTime), anEntry)

# pragma mark -

/*
bo_stsbool_t
__attribute__((nonnull(1)))
AorpSchedChecktime_tm(
	aorp_object_t aThis,
	struct bo_tm const *aTime,
	struct aorp_sched_entry *anEntry,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedChecktime_tm(aThis, aTime, anEntry, anErrPtr) \
	_A_call(checktime_tm, sched, aThis, anErrPtr, aTime, anEntry)

# pragma mark -

/*
int
__attribute__((nonnull(1)))
AorpSchedExecmode(
	aorp_object_t aThis,
	int aMode,
	struct aorp_error *anErrPtr
	);
 */
# define AorpSchedExecmode(aThis, aMode, anErrPtr) \
	((int) _A_call(execmode, sched, aThis, anErrPtr, aMode))

__MYC_END_CDECLS

#endif /* !__BO_SERVICES_JOBCNTL_SCHED__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
