/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * SchedP.h	(S.Makhmali)
 * include/Bo/services/jobcntl/SchedP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_JOBCNTL_SCHED_P__
# define __BO_SERVICES_JOBCNTL_SCHED_P__

/******************************************************************************
* %% BeginSection: includes
*/

# include <Bo/services/jobcntl/Schedtypes.h>

/******************************************************************************
* %% BeginSection: sched service declarations
*/
# pragma mark -
# pragma mark [ sched service declarations ]
# pragma mark -

# define __aorpc_need_service_sched 1

#endif /* !__BO_SERVICES_JOBCNTL_SCHED_P__ */

#pragma mark -

#if __aorpc_uses_service(sched)
# undef __aorpc_need_service_sched

/*
 * sched service
 * @{
 */

/*
 * sched {\object(instance)}
 */
/*
# define AorpK_sched_service (1112)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name sched
# include <Bo/aorp/AorpCsrvc.h>

_A_service(sched)

/* external */
struct bo_strtok_ctx;
struct bo_tm;

_A_end_ctb

	_A_return((
		BoS_SUCCESS = [ 0++ = (/* number of parsed symbols. */); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_BadAddress, BoK_TextError_UnexpectedToken,
			           BoK_TextError_BadToken, BoK_SysError_NoMemory, ...);
	        BoS_ERR_SHED_EOT = (BoK_TextError_MissingToken);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(parseent, "sched::parse-entry", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aStr,
			struct aorp_sched_entry *anEntry
		)),

	_A_return((
		BoS_SUCCESS = [ 0++ = (/* number of parsed symbols. */); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_BadAddress, BoK_TextError_UnexpectedToken,
			           BoK_TextError_BadToken, BoK_SysError_NoMemory, ...);
			BoS_ERR_SHED_EOT = (BoK_TextError_MissingToken);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(parseent2, "sched::parse-entry2", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char *aStr,
			char aDelim[],
			struct bo_strtok_ctx *aTokctx,
			struct aorp_sched_entry *anEntry
		)),

	_A_return((
		BoS_SUCCESS = [ 0++ = (/* number of printed symbols. */); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
			           BoK_ArgError_Overflow, ...);
		];
	))
	_A_attribute((nonnull(1,3)))
	_A_op(printent, "sched::print-entry", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_sched_tmval aTable[],
			size_t aCount,
			int anActsw,
			char const *anActstr,
			char const *anActargs,
			char *aBuffer,
			size_t aBufsz
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NORMAL = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_DeadLock);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(lockel, "sched::lock-entry-list", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NORMAL = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_RacError_OperationNotPermitted);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(unlockel, "sched::unlock-entry-list", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_return((
		BoS_SUCCESS = [ 0++ = (/* id number of added entry to schedule. */); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_BadAddress, BoK_SysError_NoMemory,
			           BoK_ArgError_InvalidArgument, BoK_RacError_DeadLock,
		               BoK_RacError_OperationNotPermitted);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(addent, "sched::add-entry-to-list", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_sched_entry *anEntry
		)),

	_A_return((
		BoS_SUCCESS = [ 0++ = (/* id number of added entry to schedule. */); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_BadAddress, BoK_SysError_NoMemory);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(addent_nlk, "sched::add-entry-to-list-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_sched_entry *anEntry
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_AIBNameError_NoSuchEntry,
			           BoK_RacError_DeadLock, BoK_RacError_OperationNotPermitted);

		];
	))
	_A_attribute((nonnull(1,4)))
	_A_op(rment, "sched::remove-entry-from-list", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned anEntId,
			struct aorp_sched_entry **anEntPtr
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_AIBNameError_NoSuchEntry);
		];
	))
	_A_attribute((nonnull(1,4)))
	_A_op(rment_nlk, "sched::remove-entry-from-list-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned anEntId,
			struct aorp_sched_entry **anEntPtr
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_AIBNameError_NoSuchEntry,
			           BoK_RacError_DeadLock, BoK_RacError_OperationNotPermitted);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(delent, "sched::delete-entry", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned anEntId
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_AIBNameError_NoSuchEntry);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(delent_nlk, "sched::delete-entry-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned anEntId
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_AIBNameError_NoSuchEntry,
			           BoK_SysError_NoMemory, ...);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(setaction, "sched::set-user-action-for-entry", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned anEntId,
			int anActsw,
			intptr_t aPtr
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
		BoS_FAILURE = [
			BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_AIBNameError_NoSuchEntry, ...);
		];
	))
	_A_attribute((nonnull(1)))
	_A_op(unsetaction, "sched::unset-user-action-for-entry", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			unsigned anEntId
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
		BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_BadAddress); ];
	))
	_A_attribute((nonnull(1)))
	_A_op(setjtable, "sched::set-user-job-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_sched_jtelem *aTable,
			size_t anElemc
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NOERR = (); ];
	))
	_A_attribute((nonnull(1)))
	_A_op(unsetjtable, "sched::unset-user-job-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_FALSE = (/* if aTime does not match with anEntry */); ];
		BoS_SUCCESS = [ BoS_TRUE = (/* if aTime matchs with anEntry */); ];
		BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_BadAddress, ...); ];
	))
	_A_attribute((nonnull(1)))
	_A_op(checktime, "sched::check-time-of-entry", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_utime_t aTime,
			struct aorp_sched_entry *anEntry
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_FALSE = (/* if aTime does not match with anEntry */); ];
		BoS_SUCCESS = [ BoS_TRUE = (/* if aTime matchs with anEntry */); ];
		BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_BadAddress, ...); ];
	))
	_A_attribute((nonnull(1)))
	_A_op(checktime_tm, "sched::check-timetm-of-entry", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_tm const *aTime,
			struct aorp_sched_entry *anEntry
		)),

	_A_return((
		BoS_SUCCESS = [ AorpK_RL_STD_MODE = (/* previous mode */); ];
		BoS_SUCCESS = [ AorpK_RL_HIR_MODE = (/* previous mode */); ];
		BoS_FAILURE = [ BoS_ERR = (...); ];
	))
	_A_attribute((nonnull(1)))
	_A_op(execmode, "sched::execmode", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int aMode
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(sched) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
