/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Schedtypes.h	(S.Makhmali)
 * include/Bo/services/jobcntl/Schedtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_JOBCNTL_SCHEDTYPES__
# define __BO_SERVICES_JOBCNTL_SCHEDTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/services/jobcntl/Jobtypes.h>
# include <Bo/services/jobcntl/Runlooptypes.h>

# include <Bo/bitis/Bitseq.h>
	/* bo_bitseq_t, BO_BITSEQ_DECLARE */
# include <Bo/memman/Types.h>
	/* bo_malloc_free_fn */

# include <Bo/libc/Time.h>
	/* struct bo_tm */

/******************************************************************************
* %% BeginSection: constant declarations
*/

/*
 * Format of an entry action
 *
 * 1. process: "$(process:<name>: args...)"
 * 2. job: "$(job: args...)"
 * 3. jid: "$(id:<id>: args...)"
 */

/*
 * Limits
 */

# define AorpL_SCHED_ENTSTR_MAX    1024
# define AorpL_SCHED_TEMPSTR_MAX   256


/*
 * @{
 * DOMAINS
 */

# define AorpK_SCHED_UNK_DOMAIN    0
# define AorpK_SCHED_DOMAIN_BASE   3
# define AorpK_SCHED_DOMAIN_LAST   11

# define AorpK_SCHED_MONTH         3
# define AorpK_SCHED_DOM           4 /* day of month */
# define AorpK_SCHED_DOW           5 /* day of week */
# define AorpK_SCHED_HOUR          6
# define AorpK_SCHED_MINUTE        7
# define AorpK_SCHED_SECOND        8
# define AorpK_SCHED_MSECOND       9
# define AorpK_SCHED_USECOND       10
# define AorpK_SCHED_COMMAND       11

# define _AorpK_SCHED_MONTH        (AorpK_SCHED_MONTH - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_DOM          (AorpK_SCHED_DOM - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_DOW          (AorpK_SCHED_DOW - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_HOUR         (AorpK_SCHED_HOUR - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_MINUTE       (AorpK_SCHED_MINUTE - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_SECOND       (AorpK_SCHED_SECOND - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_MSECOND      (AorpK_SCHED_MSECOND - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_USECOND      (AorpK_SCHED_USECOND - AorpK_SCHED_DOMAIN_BASE)
# define _AorpK_SCHED_COMMAND      (AorpK_SCHED_COMMAND - AorpK_SCHED_DOMAIN_BASE)

/* @} */


# define AorpK_SCHED_FIRST_MONTH       1
# define AorpK_SCHED_LAST_MONTH        12
# define AorpK_SCHED_MONTH_COUNT       12

# define AorpK_SCHED_FIRST_DOM         1
# define AorpK_SCHED_LAST_DOM          31
# define AorpK_SCHED_DOM_COUNT         31

/* note on DOW: 0 and 7 are both Sunday, for compatibility reasons. */
# define AorpK_SCHED_FIRST_DOW         0
# define _AorpK_SCHED_LAST_DOW         6
# define AorpK_SCHED_LAST_DOW          7
# define AorpK_SCHED_DOW_COUNT         8

# define AorpK_SCHED_FIRST_HOUR        0
# define AorpK_SCHED_LAST_HOUR         23
# define AorpK_SCHED_HOUR_COUNT        24

# define AorpK_SCHED_FIRST_MINUTE      0
# define AorpK_SCHED_LAST_MINUTE       59
# define AorpK_SCHED_MINUTE_COUNT      60

# define AorpK_SCHED_FIRST_SECOND      0
# define AorpK_SCHED_LAST_SECOND       59
# define AorpK_SCHED_SECOND_COUNT      60

# define AorpK_SCHED_FIRST_MSECOND     0
# define AorpK_SCHED_LAST_MSECOND      999
# define AorpK_SCHED_MSECOND_COUNT     1000

# define AorpK_SCHED_FIRST_USECOND     0
# define AorpK_SCHED_LAST_USECOND      999
# define AorpK_SCHED_USECOND_COUNT     1000


/*
 * @{
 * Entry actions
 */

# define AorpK_SCHED_INVAL_ENTACTION  (-1)
# define AorpK_SCHED_NULL_ENTACTION    0
# define AorpK_SCHED_UNK_ENTACTION     2

# define AorpK_SCHED_PROG_ENTACTION    3
# define AorpK_SCHED_JOB_ENTACTION     4
# define AorpK_SCHED_JID_ENTACTION     5

/* @} */


/* Buffer, used by the operation parseent2 */
# define AorpK_SCHED_DELIM_BUFSIZE     64

/* Status, returned by the operation parseent, parseent2 */
# define BoS_ERR_SHED_EOT             (-20)

/******************************************************************************
* %% BeginSection: type declarations
*/

struct aorp_sched_range {
	unsigned  from,
	          to,
	          step;
};

/* Specific values for aorp_sched_tmval.sctv_value field */
# define AorpK_SCHED_TMV_ALWAYS ((int32_t)0xFFFFFFFF)
# define AorpK_SCHED_TMV_ONCE   ((int32_t)0xFFFFFFFE)
# define AorpK_SCHED_TMV_RANGE  ((int32_t)0xFFFFFFFD)

struct aorp_sched_tmval {
	unsigned                 sctv_domain;
	int32_t                  sctv_value;
	struct aorp_sched_range  sctv_range;
};


struct aorp_sched_prog {
	/*
	 * memory for these fields should be released by _BoMfree().
	 */
	char         *cmdstr;   /* program's name and args as C-strings. */
	char        **procargv; /* @todo: ? */
};

struct aorp_sched_job {
	AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t))   job;
	char                                              **procargv; /* @todo: ? */
};

struct aorp_sched_jid {
	bo_index_t    jid;
	char        **procargv; /* @todo: ? */
};

union aorp_sched_action {
	struct aorp_sched_prog sca_prog;
	struct aorp_sched_job  sca_job;
	struct aorp_sched_jid  sca_jid;
};


/*
 * Values for aorp_sched_entry.accuracy
 */

# define AorpK_SCHED_UNK_ACCURACY      0
# define AorpK_SCHED_ACCURACY_BASE     3
# define AorpK_SCHED_ACCURACY_LAST     9

# define AorpK_SCHED_MONTH_ACCURACY    3
# define AorpK_SCHED_DAY_ACCURACY      4
# define AorpK_SCHED_HOUR_ACCURACY     5
# define AorpK_SCHED_MINUTE_ACCURACY   6
# define AorpK_SCHED_SECOND_ACCURACY   7
# define AorpK_SCHED_MSECOND_ACCURACY  8
# define AorpK_SCHED_USECOND_ACCURACY  9

# define AorpK_SCHED_DEFAULT_ACCURACY  AorpK_SCHED_MSECOND_ACCURACY


/*
 * Values for aorp_sched_entry.flags
 */

# define AorpF_SCHED_DOM_STAR      0x01
# define AorpF_SCHED_DOW_STAR      0x02


struct aorp_sched_entry;

struct aorp_sched_entry_header {
	bo_malloc_free_fn        sce_free_self;

	unsigned                 sce_accuracy;
	unsigned                 sce_flags;
	int                      sce_actsw;    /* =3: prog; =4: job; =5: job-id */
	union aorp_sched_action  sce_action;

	unsigned                 sce_once[8];  /* month, dom, dow, hour, minute, sec, (msec|usec) */
};

# define aorp_sched_entry_header_p(x) ((struct aorp_sched_entry_header *)(x))

# define AORP_SCHED_ENTRY_Free(anEntry) \
	((void)( \
		(NULL != (anEntry) && NULLF != aorp_sched_entry_header_p(anEntry)->sce_free_self) \
			? ((*aorp_sched_entry_header_p(anEntry)->sce_free_self)(anEntry), 0) \
			: 0))


/* .sce_accuracy = AorpK_SCHED_MONTH_ACCURACY */
struct aorp_sched_entry_mon {
	struct aorp_sched_entry_header __h;

	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_mon,  AorpK_SCHED_MONTH_COUNT);
};

/* .sce_accuracy = AorpK_SCHED_DAY_ACCURACY */
struct aorp_sched_entry_day {
	struct aorp_sched_entry_header __h;

	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_mon,  AorpK_SCHED_MONTH_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dom,  AorpK_SCHED_DOM_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dow,  AorpK_SCHED_DOW_COUNT);
};

/* .sce_accuracy = AorpK_SCHED_HOUR_ACCURACY */
struct aorp_sched_entry_hour {
	struct aorp_sched_entry_header __h;

	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_mon,  AorpK_SCHED_MONTH_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dom,  AorpK_SCHED_DOM_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dow,  AorpK_SCHED_DOW_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_hour, AorpK_SCHED_HOUR_COUNT);
};

/* .sce_accuracy = AorpK_SCHED_MINUTE_ACCURACY */
struct aorp_sched_entry_min {
	struct aorp_sched_entry_header __h;

	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_mon,  AorpK_SCHED_MONTH_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dom,  AorpK_SCHED_DOM_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dow,  AorpK_SCHED_DOW_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_hour, AorpK_SCHED_HOUR_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_min,  AorpK_SCHED_MINUTE_COUNT);
};

/* .sce_accuracy = AorpK_SCHED_SECOND_ACCURACY */
struct aorp_sched_entry_sec {
	struct aorp_sched_entry_header __h;

	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_mon,  AorpK_SCHED_MONTH_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dom,  AorpK_SCHED_DOM_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dow,  AorpK_SCHED_DOW_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_hour, AorpK_SCHED_HOUR_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_min,  AorpK_SCHED_MINUTE_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_sec,  AorpK_SCHED_SECOND_COUNT);
};

/* .sce_accuracy = AorpK_SCHED_MSECOND_ACCURACY */
struct aorp_sched_entry_msec {
	struct aorp_sched_entry_header __h;

	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_mon,  AorpK_SCHED_MONTH_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dom,  AorpK_SCHED_DOM_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dow,  AorpK_SCHED_DOW_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_hour, AorpK_SCHED_HOUR_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_min,  AorpK_SCHED_MINUTE_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_sec,  AorpK_SCHED_SECOND_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_msec, AorpK_SCHED_MSECOND_COUNT);
};

/* .sce_accuracy = AorpK_SCHED_USECOND_ACCURACY */
struct aorp_sched_entry_usec {
	struct aorp_sched_entry_header __h;

	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_mon,  AorpK_SCHED_MONTH_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dom,  AorpK_SCHED_DOM_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_dow,  AorpK_SCHED_DOW_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_hour, AorpK_SCHED_HOUR_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_min,  AorpK_SCHED_MINUTE_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_sec,  AorpK_SCHED_SECOND_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_msec, AorpK_SCHED_MSECOND_COUNT);
	bo_bitseq_t  BO_BITSEQ_DECLARE(sce_usec, AorpK_SCHED_USECOND_COUNT);
};


struct aorp_sched_jtelem {
	int                                               jte_active;
	bo_index_t                                        jte_id;
	AORP_ONEP_OF((struct aorp_ujob *, aorp_object_t)) jte_job;
};

struct aorp_sched_entargv {
	struct aorp_sched_entry_header  *entry;
	char                           **procargv;
};

#endif /* !defined(__BO_SERVICES_JOBCNTL_SCHEDTYPES__) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
