/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2009 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * OtiP.h	(S.Makhmali)
 * include/Bo/services/net/OtiP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_NET_OTI_P__
# define __BO_SERVICES_NET_OTI_P__

/******************************************************************************
* %% BeginSection: includes
*/

# include <Bo/services/net/Otitypes.h>

# include <Bo/sys/Fcntl.h>
	/* open flags */

/******************************************************************************
* %% BeginSection: oti service declarations
*/
# pragma mark -
# pragma mark [ oti service declarations ]
# pragma mark -

# define __aorpc_need_service_oti 1

#endif /* !__BO_SERVICES_NET_OTI_P__ */

#pragma mark -

#if __aorpc_uses_service(oti)
# undef __aorpc_need_service_oti

/*
 * oti service
 * @{
 */

/*
 * oti {\object(instance)}
 */
/*
# define AorpK_oti_service (1041)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name oti
# include <Bo/aorp/AorpCsrvc.h>

_A_service(oti)

# if __aorpc_interface

#  define BO_DEVICE "dev"
#  define BO_WDEVICE L"dev"
#  define BO_OTI "oti"
#  define BO_WOTI L"oti"

/* how arguments for shutdown op */
#  define BoK_OTI_SHUT_RD 0
#  define BoK_OTI_SHUT_WR 1
#  define BoK_OTI_SHUT_RDWR 2

/*
 * Send, receive flags
 */

#  ifdef T_EXPEDITED
#   define BoF_OTI_EXPEDITED_DATA T_EXPEDITED
#  else
#   define BoF_OTI_EXPEDITED_DATA 0x01
#  endif

#  ifdef T_MORE
#   define BoF_OTI_MORE_DATA T_MORE
#  else
#   define BoF_OTI_MORE_DATA 0x02
#  endif

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(mklisten, "oti::make-listen-endpoint", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(listen, "oti::listen-for-connection", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t anUconn,
			size_t aBufsz,
			struct aorp_oti_tcall **aCall
		)),

	_A_attribute((nonnull(1,3,4)))
	_A_op(accept, "oti::accept-connection", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_object_t aConnobj,
			struct aorp_oti_tcall *aCall
		)),

	_A_attribute((nonnull(1)))
	_A_op(connect, "oti::connect", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_uconn_ds *anUconn,
			struct bo_options *Options,
			struct bo_nabsbuf *aUdata,
			struct aorp_oti_tcall **aRecvcall
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(receive, "oti::receive-data", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_pointer_t aBuffer,
			size_t aBufsz,
			int *FlagsPtr
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(send, "oti::send-data", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_const_pointer_t aBuffer,
			size_t aLen,
			int Flags
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(rcvconn, "oti::receive-connect-confirmation", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_oti_tcall *aCall
		)),

	_A_attribute((nonnull(1)))
	_A_op(reserved1, "oti::reserved1", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1,5)))
	_A_op(rcvudata, "oti::receive-unitdata", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_uconn_ds *anUconn,
			struct bo_options *Options,
			struct bo_nabsbuf *aUdata,
			int *FlagsPtr
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(rcvuderr, "oti::receive-unitdata-error", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_oti_tuderr *aUderr
		)),

	_A_attribute((nonnull(1,5)))
	_A_op(sndudata, "oti::send-unitdata", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_uconn_ds *anUconn,
			struct bo_options *Options,
			struct bo_nabsbuf *aUdata,
			int Flags
		)),

	_A_attribute((nonnull(1,5)))
	_A_op(rcvvudata, "oti::receive-unitdata-vector", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_uconn_ds *anUconn,
			struct bo_options *Options,
			struct bo_qnabsbuf *aUdatav,
			ssize_t aUdatac,
			int *FlagsPtr
		)),
	
	_A_attribute((nonnull(1,5)))
	_A_op(sndvudata, "oti::send-unitdata-vector", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_uconn_ds *anUconn,
			struct bo_options *Options,
			struct bo_qnabsbuf *aUdatav,
			ssize_t aUdatac,
			int Flags
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(recvmsg, "oti::receive-message", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_oti_msghdr *aMsg,
			int Flags
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(sendmsg, "oti::send-message", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_oti_msghdr *aMsg,
			int Flags
		)),

	_A_attribute((nonnull(1)))
	_A_op(getprotaddr, "oti::get-protocol-addresses", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_oti_tbind **aLocaladdr,
			struct aorp_oti_tbind **aPeeraddr
		)),

	_A_attribute((nonnull(1)))
	_A_op(disconnect, "oti::send-disconnect-request", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct bo_nabsbuf *aUdata,
			int aSequence

		)),

	_A_attribute((nonnull(1)))
	_A_op(shutdown, "oti::shutdown", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int How
		)),

	_A_attribute((nonnull(1)))
	_A_op(rcvrel, "oti::receive-final-information", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr
		)),

	_A_attribute((nonnull(1)))
	_A_op(rcvdis, "oti::receive-disconnect-information", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_oti_tdiscon *aDiscon
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(oti) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
