/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2008 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Otitypes.h	(S.Makhmali)
 * include/Bo/services/net/Otitypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_NET_OTITYPES__
# define __BO_SERVICES_NET_OTITYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>
# include <Bo/Optdefs.h>
# include <Bo/Trpdefs.h>
# include <Bo/trpType.h>

/******************************************************************************
 * %% BeginSection: definitions
 */

# define BO_OTI_DEVICE "dev/oti"
# define BO_OTI_WDEVICE L"dev/oti"
# define BO_OTI_DEVICE_PATH "/dev/oti"
# define BO_OTI_DEVICE_WPATH L"/dev/oti"
# define BO_OTI_DEVICE_PATHDIR "/dev/oti/"
# define BO_OTI_DEVICE_WPATHDIR "/dev/oti/"

# define BO_OTIDEV_TCP "tcp"
# define BO_OTIDEV_WTCP L"tcp"
# define BO_OTIDEV_TCP_PATH "/tcp"
# define BO_OTIDEV_TCP_WPATH L"/tcp"

# define BO_OTIDEV_TCP6 "tcp6"
# define BO_OTIDEV_WTCP6 L"tcp6"
# define BO_OTIDEV_TCP6_PATH "/tcp6"
# define BO_OTIDEV_TCP6_WPATH L"/tcp6"

# define BO_OTIDEV_UDP "udp"
# define BO_OTIDEV_WUDP L"udp"
# define BO_OTIDEV_UDP_PATH "/udp"
# define BO_OTIDEV_UDP_WPATH L"/udp"

# define BO_OTIDEV_UDP6 "udp6"
# define BO_OTIDEV_WUDP6 L"udp6"
# define BO_OTIDEV_UDP6_PATH "/udp6"
# define BO_OTIDEV_UDP6_WPATH L"/udp6"

# define BO_OTIDEV_ICMP "icmp"
# define BO_OTIDEV_WICMP L"icmp"
# define BO_OTIDEV_ICMP_PATH "/icmp"
# define BO_OTIDEV_ICMP_WPATH L"/icmp"

# define BO_OTIDEV_ICMP6 "icmp6"
# define BO_OTIDEV_WICMP6 L"icmp6"
# define BO_OTIDEV_ICMP6_PATH "/icmp6"
# define BO_OTIDEV_ICMP6_WPATH L"/icmp6"

/* loopback devices */
# define BO_OTIDEV_TICLTS "ticlts"
# define BO_OTIDEV_TICOTS "ticots"
# define BO_OTIDEV_TICOTSORD "ticotsord"

# define BO_OTIDEV_TICLTS6 "ticlts6"
# define BO_OTIDEV_TICOTS6 "ticots6"
# define BO_OTIDEV_TICOTSORD6 "ticotsord6"

# define BO_OTIDEV_INVAL NULL


# define BO_OTIDOM_SOCKET "socket"
# define BO_OTIDOM_WSOCKET L"socket"
# define BO_OTIDOM_SOCKET_PATH "/socket"
# define BO_OTIDOM_SOCKET_WPATH L"/socket"

# define BO_OTIDOM_XTI "xti"
# define BO_OTIDOM_WXTI L"xti"
# define BO_OTIDOM_XTI_PATH "/xti"
# define BO_OTIDOM_XTI_WPATH L"/xti"


/* common oti device names */

# define BO_OTIDEVNM_TCPSOCK "/dev/oti/socket/tcp"
# define BO_OTIDEVNM_WTCPSOCK L"/dev/oti/socket/tcp"
# define BO_OTIDEVNM_TCP6SOCK "/dev/oti/socket/tcp6"
# define BO_OTIDEVNM_WTCP6SOCK L"/dev/oti/socket/tcp6"

# define BO_OTIDEVNM_UDPSOCK "/dev/oti/socket/udp"
# define BO_OTIDEVNM_WUDPSOCK L"/dev/oti/socket/udp"
# define BO_OTIDEVNM_UDP6SOCK "/dev/oti/socket/udp6"
# define BO_OTIDEVNM_WUDP6SOCK L"/dev/oti/socket/udp6"

# define BO_OTIDEVNM_TCPXTI "/dev/oti/xti/tcp"
# define BO_OTIDEVNM_WTCPXTI L"/dev/oti/xti/tcp"
# define BO_OTIDEVNM_TCP6XTI "/dev/oti/xti/tcp6"
# define BO_OTIDEVNM_WTCP6XTI L"/dev/oti/xti/tcp6"

# define BO_OTIDEVNM_UDPXTI "/dev/oti/xti/udp"
# define BO_OTIDEVNM_WUDPXTI L"/dev/oti/xti/udp"
# define BO_OTIDEVNM_UDP6XTI "/dev/oti/xti/udp6"
# define BO_OTIDEVNM_WUDP6XTI L"/dev/oti/xti/udp6"


/******************************************************************************
 * %% BeginSection: constants
 */

# define BoK_OTIDEV_NAME_MAX (64)
# define BoK_OTIDEV_NAME_PARTS_MIN (3)

# define BoK_OTI_QLEN_DEFAULT (5)
# define BoK_OTI_DEFAULT_PORT (27015)

/*
 * Send, receive flags
 */

# define BoK_MSG_EXPEDITED 0x1 /* process out-of-band data */
# define BoK_MSG_PEEK 0x2 /* peek at incoming message */
# define BoK_MSG_DONTROUTE 0x4 /* send without using routing tables */
# define BoK_MSG_EOR 0x8 /* data completes record */
# define BoK_MSG_TRUNC 0x10	/* data discarded before delivery */
# define BoK_MSG_CTRUNC 0x20 /* control data lost before delivery */
# define BoK_MSG_WAITALL 0x40 /* wait for full request or error */
# define BoK_MSG_DONTWAIT 0x80 /* this message should be nonblocking */
# define BoK_MSG_EOF 0x100 /* data completes connection */
# define BoK_MSG_NBIO 0x4000 /* FIONBIO mode, used by fifofs */
# define BoK_MSG_SOCALLBCK 0x8000 /* for use by socket callbacks - soreceive (TCP) */
# define BoK_MSG_NOSIGNAL 0x10000 /* do not generate SIGPIPE on EOF */

# define BoK_MSG_BCAST 0x20000
# define BoK_MSG_MCAST 0x40000

# define BoK_MSG_MORE 0x80000 /* more data to send or receive */
# define BoK_MSG_PUSH 0x100000 /* send data immediately */

# define BoK_MSG_PARTIAL 0x200000 /* partial send or recv for message xport */
# define BoK_MSG_INTERRUPT 0x400000  /* send/recv in the interrupt context */

/*
 * Status
 */

/*
 * These status definitions would not always indicate success or failure.
 */
# define BoS_T_DISCONNECT (-10)
# define BoS_T_UDERR (-11)
# define BoS_T_ORDREL (-12)

/******************************************************************************
 * %% BeginSection: type declarations
 */

struct aorp_oti_tcall {
	void              (*free_self) __myc_proto__((struct aorp_oti_tcall *));

	struct bo_uconn_ds *addr;     /* address */
	struct bo_options   opt;      /* options */
	struct bo_nabsbuf   udata;    /* user data */
	int                 sequence; /* sequence number (for listen, accept) */ 
};

# define AORP_OTI_TCALL_Free(x) \
	do { \
		if (NULLF != (x)->free_self) \
			(*(x)->free_self)(x); \
	} while (0)


struct aorp_oti_tbind {
	void              (*free_self) __myc_proto__((struct aorp_oti_tbind *));
	struct bo_uconn_ds *addr;
	unsigned int        qlen; /* max amount of connections (@seealso: backlog for sockets) */
};

# define AORP_OTI_TBIND_Free(x) \
	do { \
		if (NULLF != (x)->free_self) \
			(*(x)->free_self)(x); \
	} while (0)

struct aorp_oti_tdiscon {
	struct bo_nabsbuf udata;
	int               reason;
	int               sequence;
};

struct aorp_oti_tuderr {
	struct bo_uconn_ds *addr;  /* address */
	struct bo_options   opt;   /* options */
	bo_scalar_t         error; /* error code */
};


struct aorp_oti_msghdr {
	struct bo_uconn_ds *name;    /* address */
	ssize_t             bufc;
	struct bo_qnabsbuf *bufv;    /* buffer array */
	struct bo_absbuf    control; /* ancillary data */
	int                 flags;
};
# define aorp_oti_msghdr_p(x) ((struct aorp_oti_msghdr *)(x))

typedef u_int32_t oti_msg_datalen_t;

struct aorp_oti_cmsghdr {
	oti_msg_datalen_t len; /* len, including this header */
	int               level;
	int               type;
	/* data[] */
};
# define aorp_oti_cmsghdr_p(x) ((struct aorp_oti_cmsghdr *)(x))

/* ------------------------------------------------------------------------- */

# define BO_CMSG_DATA(_Tp, aCmsghdr) \
	((_Tp *) _BO_CMSG_DATA(aCmsghdr))

# if __myc_support(macro_retval)
#  define _BO_CMSG_DATA(aCmsghdr) \
	__extension__ ({ \
		bo_pointer_t ___ohp = (bo_pointer_t)((bo_byte_t *)(aCmsghdr) + sizeof(struct aorp_oti_cmsghdr)); \
		___ohp = _BO_CMSGDATA_ALIGN_PTR(___ohp); \
		/* return */ (___ohp); \
	})
# else
#  define _BO_CMSG_DATA(aCmsghdr) \
	(_BO_CMSGDATA_ALIGN_PTR((char *)(aCmsghdr) + sizeof(struct aorp_oti_cmsghdr)))
# endif


# define BO_CMSG_FIRSTHDR(Cmsgs) \
	((struct aorp_oti_cmsghdr *) _BO_CMSG_FIRSTHDR(Options))

# define BO_CMSG_NEXTHDR(Cmsgs, aCmsghdr) \
	((NULL == (aCmsghdr)) \
		? ((struct aorp_oti_cmsghdr *) _BO_CMSG_FIRSTHDR(Cmsgs)) \
		: ((struct aorp_oti_cmsghdr *) _BO_CMSG_NEXTHDR(Cmsgs, aCmsghdr)))

#define BO_CMSG_SPACE(aLength)  \
        (_BO_CMSGDATA_ALIGN(sizeof(struct aorp_oti_cmsghdr) + _BO_CMSGHDR_ALIGN(aLength)))

#define BO_CMSG_LEN(aLength)    \
         (_BO_CMSGDATA_ALIGN(sizeof(struct aorp_oti_cmsghdr)) + aLength)


/* NOT FOR YOUR BUSINESS @{ */
# define _BO_CMSG_FIRSTHDR(Cmsgs) \
	((sizeof(struct aorp_oti_cmsghdr) <= aorp_oti_msghdr_p(Cmsgs)->control.size) \
		? ((char *)aorp_oti_msghdr_p(Cmsgs)->control.base) \
		: ((char *)0))

# define _BO_CMSG_NEXTHDR(Cmsgs, aCmsghdr) \
	( ((((char *)_BO_CMSGHDR_ALIGN_PTR((char *)(aCmsghdr) + aorp_oti_cmsghdr_p(aCmsghdr)->len) + \
		sizeof(struct aorp_oti_cmsghdr)) - (char *)aorp_oti_msghdr_p(Cmsgs)->control.base) <= \
		aorp_oti_msghdr_p(Cmsgs)->control.size) \
			? ((char *)_BO_CMSGHDR_ALIGN_PTR((char *)(aCmsghdr) + aorp_oti_cmsghdr_p(aCmsghdr)->len)) \
			: ((char *)0) )

# define _BO_CMSGHDR_ALIGN(aLen) __myc_align((aLen), _BoK_CMSG_HALIGN)
# define _BO_CMSGHDR_ALIGN_PTR(aPtr) __myc_align_ptr((aPtr), _BoK_CMSG_HALIGN)
# define _BO_CMSGDATA_ALIGN(aLen) __myc_align((aLen), _BoK_OPT_DALIGN)
# define _BO_CMSGDATA_ALIGN_PTR(aPtr) __myc_align_ptr((aPtr), _BoK_OPT_DALIGN)

# if 1
#  define _BoK_CMSG_HALIGN __myc_alignof(struct aorp_oti_cmsghdr)
# else
#  define _BoK_CMSG_HALIGN sizeof(bo_scalar_t) /* @todo: ??? */
# endif
# define _BoK_OPT_DALIGN sizeof(int32_t) /* @todo: ??? */
/* @} */

#endif /* !__BO_SERVICES_NET_OTITYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
