/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * PafiP.h	(V.Sokur)
 * include/Bo/services/net/PafiP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_NET_PAFI_P__
# define __BO_SERVICES_NET_PAFI_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/services/net/pafType.h>

/******************************************************************************
 * %% BeginSection: tris service declarations
 */
# pragma mark -
# pragma mark [ tris service declarations ]
# pragma mark -

# define __aorpc_need_service_tris 1

#endif /* !__BO_SERVICES_NET_PAFI_P__ */

#pragma mark -

#if __aorpc_uses_service(tris)
# undef __aorpc_need_service_tris

/*
 * pafi service
 * @{
 */

/*
 * pafi {\object(class)}
 * protocol-address-family Info Service
 */
/*
# define AorpK_pafi_service (1150)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name pafi
# include <Bo/aorp/AorpCsrvc.h>

_A_service(pafi)

# if __aorpc_interface

	struct aorp_pafi_protoinfo {
		int    family;
		int    type;
		int    protocol;
		_Bool  loopback;
	};

	struct Pafi_af_dbelem {
		int             af_id;
		char const     *af_name;

		/* NOT FOR YOUR BUSINESS @{ */
		bo_rtbtelem_t   btelem1; /* enum-to-string */
		bo_rtbtelem_t   btelem2; /* string-to-enum */
		/* @} */
	};

	struct Pafi_proto_dbelem {
		int             proto_id;
		char const     *proto_name;
		int            *families;
		int            *types;

		/* NOT FOR YOUR BUSINESS @{ */
		bo_rtbtelem_t   btelem1; /* enum-to-string */
		bo_rtbtelem_t   btelem2; /* string-to-enum */
		/* @} */
	};

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(afnamebyid, "get-address-family-name-by-id", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int anId
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(afidbyname, "get-address-family-id-by-name", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aName
		)),

	_A_attribute((nonnull(1)))
	_A_op(protonamebyid, "get-transport-protocol-name-by-id", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int anId
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(protoidbyname, "get-transport-protocol-id-by-name", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aName
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(protoinfobyid, "get-protoinfo-by-id", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			int anId,
			struct aorp_pafi_protoinfo *aResult
		)),

	_A_attribute((nonnull(1,3,4)))
	_A_op(protoinfobyname, "get-protoinfo-by-name", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aName,
			struct aorp_pafi_protoinfo *aResult
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(pafi) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
