/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2009 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Tiosopt.h	(S.Makhmali)
 * src/include/Bo/services/net/Tiosopt.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_NET_TIOSOPT__
# define __BO_SERVICES_NET_TIOSOPT__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SERVICES_IO_IOSTYPES__
#  include <Bo/services/io/Iostypes.h>
# endif

# ifndef __BO_SERVICES_IO_CMNIOSOPT__
#  include <Bo/services/io/Cmniosopt.h>
# endif

# include <Bo/trpType.h>

/******************************************************************************
 * %% BeginSection: constant/type declarations
 */
# pragma mark -
# pragma mark [ constant/type declarations ]
# pragma mark -

/*
 * option domain (bo_opthdr.domain)
 */
# ifndef BoK_TIOS_OPT
#  define BoK_TIOS_OPT __myc_mk4ccs("tios")
#  define _BoK_TIOS_OPT __myc_mk4cc('t','i','o','s')
# endif

/*
 * values for option level (bo_opthdr.name)
 */

/* [256, ...] */
# define AorpK_OTI_OPTLVL 256

# define AorpK_IRLMP_OPTLVL 257

# define _AorpK_OPTLVL_PROTOCOL_BASE 512
# define AorpK_IP_OPTLVL (_AorpK_OPTLVL_PROTOCOL_BASE + BoK_OTI_IP_PROTOCOL)
# define AorpK_IPV6_OPTLVL (_AorpK_OPTLVL_PROTOCOL_BASE + BoK_OTI_IPV6_PROTOCOL)
# define AorpK_TCP_OPTLVL (_AorpK_OPTLVL_PROTOCOL_BASE + BoK_OTI_TCP_PROTOCOL)
# define AorpK_UDP_OPTLVL (_AorpK_OPTLVL_PROTOCOL_BASE + BoK_OTI_UDP_PROTOCOL)
# define AorpK_ICMPV6_OPTLVL (_AorpK_OPTLVL_PROTOCOL_BASE + BoK_OTI_ICMPV6_PROTOCOL)

/*
 * values for aorp_toptctl.optnam
 */

/* level: AorpK_OTI_OPTLVL */
# define AorpK_TOPTC_DEBUG 1 /* int */
# define AorpK_TOPTC_ACCEPTCONN 2 /* int */
# define AorpK_TOPTC_REUSEADDR 3 /* int */
# define AorpK_TOPTC_KEEPALIVE 4 /* int */
# define AorpK_TOPTC_DONTROUTE 5 /* int */
# define AorpK_TOPTC_BROADCAST 6 /* int */
# define AorpK_TOPTC_USELOOPBACK 7 /* int */
# define AorpK_TOPTC_LINGER 8 /* linger{} */
# define AorpK_TOPTC_OOBINLINE 9 /* int */

# define AorpK_TOPTC_REUSEPORT 10 /* int */

# define AorpK_TOPTC_TIMESTAMP 11
# define AorpK_TOPTC_NOSIGPIPE 12
# define AorpK_TOPTC_ACCEPTFILTER 13
# define AorpK_TOPTC_BINTIME 14

# define AorpK_TOPTC_DONTTRUNC 15
# define AorpK_TOPTC_WANTMORE 16
# define AorpK_TOPTC_WANTOOBFLAG 17

# define AorpK_TOPTC_SNDBUF 1001 /* int */
# define AorpK_TOPTC_RCVBUF 1002 /* int */
# define AorpK_TOPTC_SNDLOWAT 1003 /* int */
# define AorpK_TOPTC_RCVLOWAT 1004 /* int */
# define AorpK_TOPTC_SNDTIMEO 1005 /* timeval{} */
# define AorpK_TOPTC_RCVTIMEO 1006 /* timeval{} */
# define AorpK_TOPTC_ERROR 1007 /* int */
# define AorpK_TOPTC_TYPE 1008 /* int */
# define AorpK_TOPTC_PRIVSTATE 1009

# define AorpK_TOPTC_LABEL	1010
# define AorpK_TOPTC_PEERLABEL	1011
# define AorpK_TOPTC_LISTENQLIMIT	1012
# define AorpK_TOPTC_LISTENQLEN	1013
# define AorpK_TOPTC_LISTENINCQLEN	1014

# define AorpK_TOPTC_NREAD 1020
# define AorpK_TOPTC_NKE 1021
# define AorpK_TOPTC_NOADDRERR 1023
# define AorpK_TOPTC_NWRITE 1024


/* level: AorpK_IP_OPTLVL */
# define AorpK_TOPTC_OPTIONS 1 /* buf/ip_opts; set/get IP options */
# define AorpK_TOPTC_HDRINCL 2 /* int; header is included with data */
# define AorpK_TOPTC_TOS 3 /* int; IP type of service and preced. */
# define AorpK_TOPTC_TTL 4 /* int; IP time to live */

# define AorpK_TOPTC_RECVOPTS 5 /* bool; receive all IP opts w/dgram */
# define AorpK_TOPTC_RECVRETOPTS 6 /* bool; receive IP opts for response */
# define AorpK_TOPTC_RECVDSTADDR 7 /* bool; receive IP dst addr w/dgram */
# define AorpK_TOPTC_SENDSRCADDR 8 /* cmsg_type to set src addr */
# define AorpK_TOPTC_RETOPTS 9    /* ip_opts; set/get IP options */

# define AorpK_TOPTC_MULTICAST_IF 10 /* in_addr{} */
# define AorpK_TOPTC_MULTICAST_TTL 11 /* u_char */
# define AorpK_TOPTC_MULTICAST_LOOP 12 /* u_char */
# define AorpK_TOPTC_ADD_MEMBERSHIP 13 /* ip_mreq{} */
# define AorpK_TOPTC_DROP_MEMBERSHIP 14 /* ip_mreq{} */
# define AorpK_TOPTC_DONTFRAGMENT 15 /* don't fragment packet */
# define AorpK_TOPTC_ADD_SOURCE_MEMBERSHIP 16
# define AorpK_TOPTC_DROP_SOURCE_MEMBERSHIP 17
# define AorpK_TOPTC_BLOCK_SOURCE 18
# define AorpK_TOPTC_UNBLOCK_SOURCE 19
# define AorpK_TOPTC_PKTINFO 20
# define AorpK_TOPTC_RECEIVE_BROADCAST 23

# define AorpK_TOPTC_MULTICAST_VIF 30   /* set/get IP mcast virt. iface */
# define AorpK_TOPTC_RSVP_ON 31   /* enable RSVP in kernel */
# define AorpK_TOPTC_RSVP_OFF 32   /* disable RSVP in kernel */
# define AorpK_TOPTC_RSVP_VIF_ON 33   /* set RSVP per-vif socket */
# define AorpK_TOPTC_RSVP_VIF_OFF 34   /* unset RSVP per-vif socket */
# define AorpK_TOPTC_PORTRANGE 35 /* int; range to choose for unspec port */
# define AorpK_TOPTC_RECVIF 36   /* bool; receive reception if w/dgram */
/* for IPSEC */
# define AorpK_TOPTC_IPSEC_POLICY 37   /* int; set/get security policy */
# define AorpK_TOPTC_FAITH 38   /* bool; accept FAITH'ed connections */

# define AorpK_TOPTC_ONESBCAST 39   /* bool: send all-ones broadcast */

# define AorpK_TOPTC_FW_TABLE_ADD 50   /* add entry */
# define AorpK_TOPTC_FW_TABLE_DEL 51   /* delete entry */
# define AorpK_TOPTC_FW_TABLE_FLUSH	52   /* flush table */
# define AorpK_TOPTC_FW_TABLE_GETSIZE 53   /* get table size */
# define AorpK_TOPTC_FW_TABLE_LIST	54   /* list table contents */

# define AorpK_TOPTC_FW_ADD	55   /* add a firewall rule to chain */
# define AorpK_TOPTC_FW_DEL	56   /* delete a firewall rule from chain */
# define AorpK_TOPTC_FW_FLUSH 57   /* flush firewall rule chain */
# define AorpK_TOPTC_FW_ZERO 58   /* clear single/all firewall counter(s) */
# define AorpK_TOPTC_FW_GET	59   /* get entire firewall rule chain */
# define AorpK_TOPTC_FW_RESETLOG 60   /* reset logging counters */

# define AorpK_TOPTC_DUMMYNET_CONFIGURE	65   /* add/configure a dummynet pipe */
# define AorpK_TOPTC_DUMMYNET_DEL 66   /* delete a dummynet pipe from chain */
# define AorpK_TOPTC_DUMMYNET_FLUSH	67   /* flush dummynet */
# define AorpK_TOPTC_DUMMYNET_GET 68   /* get entire dummynet pipes */

# define AorpK_TOPTC_RECVTTL 69   /* bool; receive IP TTL w/dgram */
# define AorpK_TOPTC_MINTTL 70   /* minimum TTL for packet or drop */


/* level: AorpK_IPV6_OPTLVL */
# define AorpK_TOPTC_HDRINCL 2  /* Header is included with data */
# define AorpK_TOPTC_ADDRFORM 3  /* Header is included with data */
# define AorpK_TOPTC_UNICAST_HOPS 4  /* int; IP6 hops */
/*
 * Use previouse definitions of IPV4 level
 */
#if 0
# define AorpK_TOPTC_MULTICAST_IF 10  /* u_int; set/get IP6 multicast i/f  */
# define AorpK_TOPTC_MULTICAST_HOPS 11 /* int; set/get IP6 multicast hops */
# define AorpK_TOPTC_MULTICAST_LOOP 12 /* u_int; set/get IP6 multicast loopback */
# define AorpK_TOPTC_ADD_MEMBERSHIP 13 /* ip_mreq{} */
# define AorpK_TOPTC_DROP_MEMBERSHIP 14 /* ip_mreq{} */
# define AorpK_TOPTC_DONTFRAGMENT 15 /* don't fragment packet */
#endif /* 0 */
# define AorpK_TOPTC_JOIN_GROUP	16 /* ip6_mreq; join a group membership */
# define AorpK_TOPTC_LEAVE_GROUP 17 /* ip6_mreq; leave a group membership */

#if 0
# define AorpK_TOPTC_PKTINFO 20 /* previous definition is used. */
#endif

# define AorpK_TOPTC_CHECKSUM 26 /* int; checksum offset for raw socket */
# define AorpK_TOPTC_V6ONLY 27 /* bool; make AF_INET6 sockets v6 only */

#if 0
# define AorpK_TOPTC_PORTRANGE 35 /* int; range to choose for unspec port */
#endif

/* IPSEC */
#if 0
# define AorpK_TOPTC_IPSEC_POLICY 37 /* struct; get/set security policy */
# define AorpK_TOPTC_FAITH 38 /* bool; accept FAITH'ed connections */
#endif

#if 0 /* IPV6FIREWALL */
# define AorpK_TOPTC_FW_ADD 55 /* add a firewall rule to chain */
# define AorpK_TOPTC_FW_DEL 56 /* delete a firewall rule from chain */
# define AorpK_TOPTC_FW_FLUSH 57 /* flush firewall rule chain */
# define AorpK_TOPTC_FW_ZERO 58 /* clear single/all firewall counter(s) */
# define AorpK_TOPTC_FW_GET 59 /* get entire firewall rule chain */
#endif

/* new socket options introduced in RFC3542 */
# define AorpK_TOPTC_RTHDRDSTOPTS 65 /* ip6_dest; send dst option before rthdr */

# define AorpK_TOPTC_RECVPKTINFO 66 /* bool; recv if, dst addr */
# define AorpK_TOPTC_RECVHOPLIMIT 67 /* bool; recv hop limit */
# define AorpK_TOPTC_RECVRTHDR 68 /* bool; recv routing header */
# define AorpK_TOPTC_RECVHOPOPTS 69 /* bool; recv hop-by-hop option */
# define AorpK_TOPTC_RECVDSTOPTS 70 /* bool; recv dst option after rthdr */

# define AorpK_TOPTC_USE_MIN_MTU 72 /* bool; send packets at the minimum MTU */
# define AorpK_TOPTC_RECVPATHMTU 73 /* bool; notify an according MTU */
# define AorpK_TOPTC_PATHMTU 74 /* mtuinfo; get the current path MTU (sopt),
				      4 bytes int; MTU notification (cmsg) */

/* more new socket options introduced in RFC3542 */
# define AorpK_TOPTC_HOPLIMIT 77 /* int; send hop limit */
# define AorpK_TOPTC_NEXTHOP 78 /* sockaddr; next hop addr */
# define AorpK_TOPTC_HOPOPTS 79 /* int; send hop-by-hop option */
# define AorpK_TOPTC_DSTOPTS 80 /* int; send dst option befor rthdr */
# define AorpK_TOPTC_RTHDR 81 /* int; send routing header */

# define AorpK_TOPTC_RECVTCLASS 87 /* bool; recv traffic class values */
# define AorpK_TOPTC_AUTOFLOWLABEL 89 /* bool; attach flowlabel automagically */
# define AorpK_TOPTC_TCLASS	91 /* int; send traffic class value */
# define AorpK_TOPTC_PREFER_TEMPADDR 93 /* int; prefer temporary addresses as the source address. */

# define AorpK_TOPTC_PKTOPTIONS 95

/* winsock protection level */
# define AorpK_TOPTC_PROTECTION_LEVEL 100 /* Set/get IPv6 protection level */


/* Values of IPV6_PROTECTION_LEVEL */
#define AorpK_TOPTC_PROTECTION_LEVEL_UNRESTRICTED  10  /* For peer-to-peer apps */
#define AorpK_TOPTC_PROTECTION_LEVEL_DEFAULT       20  /* Default level         */
#define AorpK_TOPTC_PROTECTION_LEVEL_RESTRICTED    30  /* For Intranet apps     */


/*
 * Defaults and limits for options
 */
# define AorpK_TOPTC_DEFAULT_MULTICAST_TTL  1	/* normally limit m'casts to 1 hop  */
# define AorpK_TOPTC_DEFAULT_MULTICAST_LOOP 1	/* normally hear sends if a member  */
# define AorpK_TOPTC_MAX_MEMBERSHIPS 20	/* per socket */
# define AorpK_TOPTC_DEFAULT_MULTICAST_LOOP 1	/* normally hear sends if a member */


/* level: AorpK_ICMPV6_OPTLVL */
# define AorpK_TOPTC_FILTER 1 /* icmp6_filter */

/* level: AorpK_TCP_OPTLVL */
# define AorpK_TOPTC_NODELAY 1 /* int */
# define AorpK_TOPTC_MAXSEG 2 /* int */
# define AorpK_TOPTC_NOPUSH	3	/* don't push last block of write */
#if 0
# define AorpK_TOPTC_KEEPALIVE 4 /* previous definition is used. */
#endif
# define AorpK_TOPTC_NOOPT	5	/* don't use TCP options */
# define AorpK_TOPTC_MD5SIG	6	/* use MD5 digests (RFC2385) */
# define AorpK_TOPTC_INFO	7	/* retrieve tcp_info structure */

# define AorpK_TOPTC_MAXRT 8 /* int */
# define AorpK_TOPTC_STDURG 9 /* int */

/* level: AorpK_UDP_OPTLVL */
# define AorpK_TOPTC_NOCHECKSUM 1
# define AorpK_TOPTC_CHECKSUM_COVERAGE 2

# define AorpK_TOPTC_INVAL (-1)

/* ----------------------------------------------------------------------------
 * TRANSPORT-OPTCTL OPTION
 * ios::optctl
 */
# pragma mark -

/*
 * hdr.domain = BoK_TIOS_OPT
 * hdr.name = {AorpK_OTI_OPTLVL, AorpK_AorpK_TOPTC_OPTLVL, AorpK_IPV6_OPTLVL,
 *             AorpK_TCP_OPTLVL, AorpK_UDP_OPTLVL, AorpK_ICMPV6_OPTLVL,
 *             AorpK_IRLMP_OPTLVL}
 */

struct aorp_toptctl_data {
	unsigned  optnam;
};
# define aorp_toptctl_data_p(x) ((struct aorp_toptctl_data *)(x))

struct aorp_toptctl_idata {
	unsigned  optnam;
	union {
		intptr_t      ip;
		u_int32_t     u32;
		u_int16_t     u16[2];
		u_int8_t      u8[4];
	} optval; /* result */
};

struct aorp_toptctl_bdata {
	unsigned           optnam;
	struct bo_nabsbuf  optval;
};

BO_OPT_STRUCT(aorp_toptctli_s, struct aorp_toptctl_idata);
BO_OPT_STRUCT(aorp_toptctlb_s, struct aorp_toptctl_bdata);

# define AORP_TOPTCTLI_RTS_INIT(anOption, aLevel) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_idata, anOption, _BoK_TIOS_OPT, aLevel)

# define AORP_TOPTCTLB_RTS_INIT(anOption, aLevel) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_bdata, anOption, _BoK_TIOS_OPT, aLevel)

/* ------------------------------------------------------------------------- */

#if 0

/*
 * SHOULD BE DECLARED BY A CLIENT ANYWHERE
 * #include <Bo/sys/Time.h>
 */

struct aorp_toptctl_timeo {
	unsigned           optnam;
	struct timeval     tv;
};

BO_OPT_STRUCT(aorp_toptctltmo_s, struct aorp_toptctl_timeo);

#endif

# define AORP_TOPTCTLTMO_STRUCT \
	struct aorp_toptctl_timeo { \
		unsigned           optnam; \
		struct timeval     tv; \
	}; \
	BO_OPT_STRUCT(aorp_toptctltmo_s, struct aorp_toptctl_timeo)

# define AORP_TOPTCTLTMO_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_timeo, anOption, _BoK_TIOS_OPT, AorpK_OTI_OPTLVL)


/* ------------------------------------------------------------------------- */

#if 0

/*
 * SHOULD BE DECLARED BY A CLIENT ANYWHERE
 * # if __myc_exists(netinet7in,h)
 * #  include __MYC_LIBC_HEADER(netinet/in.h)
 * # endif
 * # if __myc_exists(WinSock2,h)
 * #  include __MYC_LIBC_HEADER(ws2tcpip.h)
 * # endif
 */

struct aorp_toptctl_addr4 {
	unsigned           optnam;
	struct in_addr     addr;
};

BO_OPT_STRUCT(aorp_toptctladdr4_s, struct aorp_toptctl_addr4);

#endif

# define AORP_TOPTCTLADDR4_STRUCT \
	struct aorp_toptctl_addr4 { \
		unsigned          optnam; \
		struct in_addr    addr; \
	}; \
	BO_OPT_STRUCT(aorp_toptctladdr4_s, struct aorp_toptctl_addr4)

# define AORP_TOPTCTLADDR4_RTS_INIT(anOption, aLevel) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_addr4, anOption, _BoK_TIOS_OPT, aLevel)

/* ------------------------------------------------------------------------- */

#if 0

/*
 * SHOULD BE DECLARED BY A CLIENT ANYWHERE
 * # if __myc_exists(netinet7in,h)
 * #  include __MYC_LIBC_HEADER(netinet/in.h)
 * # endif
 * # if __myc_exists(ws2tcpip,h)
 * #  include __MYC_LIBC_HEADER(ws2tcpip.h)
 * # endif
 */

struct aorp_toptctl_addr6 {
	unsigned           optnam;
	struct in6_addr    addr;
};

BO_OPT_STRUCT(aorp_toptctladdr6_s, struct aorp_toptctl_addr6);

#endif

# define AORP_TOPTCTLADDR6_STRUCT \
	struct aorp_toptctl_addr6 { \
		unsigned           optnam; \
		struct in6_addr    addr; \
	}; \
	BO_OPT_STRUCT(aorp_toptctladdr6_s, struct aorp_toptctl_addr6)

# define AORP_TOPTCTLADDR6_RTS_INIT(anOption, aLevel) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_addr6, anOption, _BoK_TIOS_OPT, aLevel)

/* ------------------------------------------------------------------------- */

#if 0

/*
 * SHOULD BE DECLARED BY A CLIENT ANYWHERE
 * # if __myc_exists(netinet7in,h)
 * #  include __MYC_LIBC_HEADER(netinet/in.h)
 * # endif
 * # if __myc_exists(ws2tcpip,h)
 * #  include __MYC_LIBC_HEADER(ws2tcpip.h)
 * # endif
 */

struct aorp_toptctl_ipmreq {
	unsigned           optnam;
	struct ip_mreq     mrq;
};

BO_OPT_STRUCT(aorp_toptctlipmreq_s, struct aorp_toptctl_ipmreq);

#endif

# define AORP_TOPTCTLIPMREQ_STRUCT \
	struct aorp_toptctl_ipmreq { \
		unsigned           optnam; \
		struct ip_mreq     mrq; \
	}; \
	BO_OPT_STRUCT(aorp_toptctlipmreq_s, struct aorp_toptctl_ipmreq)

# define AORP_TOPTCTLIPMREQ_RTS_INIT(anOption, aLevel) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_ipmreq, anOption, _BoK_TIOS_OPT, aLevel)

/* ------------------------------------------------------------------------- */

#if 0

/*
 * SHOULD BE DECLARED BY A CLIENT ANYWHERE
 * # if __myc_exists(netinet7in,h) // @todo: ??
 * #  include __MYC_LIBC_HEADER(netinet/in.h)
 * # endif
 * # if __myc_exists(ws2tcpip,h)
 * #  include __MYC_LIBC_HEADER(ws2tcpip.h)
 * # endif
 */

struct aorp_toptctl_ipv6mreq {
	unsigned           optnam;
	struct ipv6_mreq   mrq;
};

BO_OPT_STRUCT(aorp_toptctlipmreq6_s, struct aorp_toptctl_ipv6mreq);

#endif

# define AORP_TOPTCTLIPMREQ6_STRUCT \
	struct aorp_toptctl_ipv6mreq { \
		unsigned           optnam; \
		struct ipv6_mreq   mrq; \
	}; \
	BO_OPT_STRUCT(aorp_toptctlipmreq6_s, struct aorp_toptctl_ipv6mreq)

# define AORP_TOPTCTLIPMREQ6_RTS_INIT(anOption, aLevel) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_ipv6mreq, anOption, _BoK_TIOS_OPT, aLevel)

/* ------------------------------------------------------------------------- */

#if 0

/*
 * SHOULD BE DECLARED BY A CLIENT ANYWHERE
 * # if __myc_exists(sys7socket,h)
 * #  include __MYC_LIBC_HEADER(sys/socket.h)
 * # endif
 * # if __myc_exists(WinSock2,h)
 * #  include __MYC_LIBC_HEADER(WinSock2.h)
 * # endif
 */

struct aorp_toptctl_sockaddr {
	unsigned           optnam;
	struct sockaddr    addr;
};

BO_OPT_STRUCT(aorp_toptctlsockaddr_s, struct aorp_toptctl_sockaddr);

#endif

# define AORP_TOPTCTLSOCKADDR_STRUCT \
	struct aorp_toptctl_sockaddr { \
		unsigned           optnam; \
		struct sockaddr    addr; \
	}; \
	BO_OPT_STRUCT(aorp_toptctlsockaddr_s, struct aorp_toptctl_sockaddr)

# define AORP_TOPTCTLSOCKADDR_RTS_INIT(anOption, aLevel) \
	BO_OPT_RTS_INIT(struct aorp_toptctl_sockaddr, anOption, _BoK_TIOS_OPT, aLevel)

/* ------------------------------------------------------------------------- */

// @todo: ???
//
//struct aorp_toptctl_icmp6filter {
//	unsigned             optnam;
//	struct bo_nabsbuf    optval;
//	struct icmp6_filter  opticmp6;
//};

/* ----------------------------------------------------------------------------
 * TRANSPORT-BIND OPTION
 * ios::bind
 */
# pragma mark -

/*
 * hdr.domain = BoK_TIOS_OPT
 * hdr.name = {AorpK_IOS_OPTBIND}
 */

struct aorp_toptbind_data {
	unsigned  qlen;
};

BO_OPT_STRUCT(aorp_toptbind_s, struct aorp_toptbind_data);

# define AORP_TOPTBIND_RTS_INIT(anOption) \
	BO_OPT_RTS_INIT(struct aorp_toptbind_data, anOption, _BoK_TIOS_OPT, AorpK_IOS_OPTBIND)

#endif /* !__BO_SERVICES_NET_TIOSOPT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
