/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2008 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Trifdefs.h	(S.Makhmali)
 * include/Bo/services/net/Trifdefs.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_NET_TRIFDEFS__
# define __BO_SERVICES_NET_TRIFDEFS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

# if __myc_exists(xti,h)
#  include __MYC_LIBC_HEADER(xti.h)
# endif

# if __myc_exists(sys7socket,h)
#  include __MYC_LIBC_HEADER(sys/socket.h)
# endif
# if __myc_exists(netinet7in,h)
#  include __MYC_LIBC_HEADER(netinet/in.h)
	/* protocol constants, address structures, ... */
# endif

# if __myc_exists(winsock2,h)
#  include __MYC_LIBC_HEADER(winsock2.h)
#  ifndef HAS_TIMEVAL_STRUC
#   define HAS_TIMEVAL_STRUC
#  endif
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */

# ifdef __BO_SOCKSYSOBJ_TYPE__
typedef __BO_SOCKSYSOBJ_TYPE__ oti_socksysobj_t;
#  define TrifK_INVAL_SOCKSYSOBJ __BO_INVAL_SOCKSYSOBJ
# else
# ifdef BO_SYSTYPE_WIN32
typedef SOCKET oti_socksysobj_t;
#  define TrifK_INVAL_SOCKSYSOBJ (INVALID_SOCKET)
# else
typedef int oti_socksysobj_t;
#  define TrifK_INVAL_SOCKSYSOBJ ((oti_socksysobj_t)-1)
# endif
# endif /* __BO_SOCKSYSOBJ_TYPE__ */

/* ------------------------------------------------------------------------- */

# ifndef HAS_T_SCALAR_TYPE
typedef int32_t t_scalar_t;
typedef u_int32_t t_uscalar_t;
# endif
# define trif_scalar_t t_scalar_t
# define trif_uscalar_t t_uscalar_t

/* ------------------------------------------------------------------------- */

/*
 * Transport-independent address storage
 */

# ifndef HAS_NETBUF_STRUC
#  ifndef NETBUF_STRUC_DEFINED
#   define NETBUF_STRUC_DEFINED

struct netbuf {
	unsigned int  maxlen;
	unsigned int  len;
	bo_pointer_t  buf;
};

#  endif /* !NETBUF_STRUC_DEFINED */
# endif /* !HAS_NETBUF_STRUC */

# define trif_netbuf netbuf

/* ------------------------------------------------------------------------- */

/*
 * Struc (below) types
 */

# ifdef T_BIND
#  define TrifK_BIND T_BIND
# else
#  define TrifK_BIND 1
# endif

# ifdef T_OPTMGMT
#  define TrifK_OPTMGMT T_OPTMGMT
# else
#  define TrifK_OPTMGMT 2
# endif

# ifdef T_CALL
#  define TrifK_CALL T_CALL
# else
#  define TrifK_CALL 3
# endif

# ifdef T_DIS
#  define TrifK_DISCON T_DIS
# else
#  define TrifK_DISCON 4
# endif

# ifdef T_UNITDATA
#  define TrifK_UNITDATA T_UNITDATA
# else
#  define TrifK_UNITDATA 5
# endif

# ifdef T_UDERROR
#  define TrifK_UDERROR T_UDERROR
# else
#  define TrifK_UDERROR 6
# endif

# ifdef T_INFO
#  define TrifK_INFO T_INFO
# else
#  define TrifK_INFO 7
# endif

/* ------------------------------------------------------------------------- */

/*
 * Format of the address arguments of bind
 */

# ifndef HAS_T_BIND_STRUC
struct t_bind {
	struct netbuf addr;
	unsigned int  qlen; /* max amount of connections (@seealso: backlog for sockets) */
};
# endif
# define trif_bind t_bind

# ifndef HAS_T_CALL_STRUC
struct t_call {
	struct netbuf addr;     /* address */
	struct netbuf opt;      /* options */
	struct netbuf udata;    /* user data */
	int           sequence; /* sequence number (??? for listen, accept) */ 
};
# endif
# define trif_call t_call

/*
 * Disconnection
 */

# ifndef HAS_T_DISCON_STRUC
struct t_discon {
	struct netbuf udata;
	int           reason;
	int           sequence;
};
# endif
# define trif_discon t_discon

/*
 * Datagram structure
 */

# ifndef HAS_T_UNITDATA_STRUC
struct t_unitdata {
	struct netbuf addr;  /* address */
	struct netbuf opt;   /* options */
	struct netbuf udata; /* user data */
};
# endif
# define trif_unitdata t_unitdata

/*
 * User datagram error
 */
# ifndef HAS_T_UDERR_STRUC
struct t_uderr {
	struct netbuf addr;  /* address */
	struct netbuf opt;   /* options */
	t_scalar_t    error; /* error code */
};
# endif
# define trif_uderr t_uderr

/* ----------------------------------------------------------------------------
 * Protocol specific service limits
 */

# ifndef HAS_T_INFO_STRUC
struct t_info {
	t_scalar_t addr;     /* max size of the transport protocol address */
	t_scalar_t options;  /* max number of bytes of protocol-specific options */
	t_scalar_t tsdu;     /* max size of a transport service data unit */
	t_scalar_t etsdu;    /* max size of expedited transport service data unit (out-of-band-data) */
	t_scalar_t connect;  /* max amount of data allowed on connection
						    establishment functions */
	t_scalar_t discon;   /* max data allowed on t_snddis and t_rcvdis functions */
	t_scalar_t servtype; /* service type supported by transport provider */
	t_scalar_t flags;    /* other info about the transport provider */
};
# endif
# define trif_info t_info

/*
 * Predefined limit values for info fields
 */

/* [idea] limits not supported */
# define TrifK_LIMNOTSUPP (0)

# ifdef T_INFINITE
#  define TrifK_NOLIMIT T_INFINITE
# else
#  define TrifK_NOLIMIT (-1)
# endif

/* feature not supported */
# ifdef T_INVALID
#  define TrifK_FEATURENOTSUPP T_INVALID
# else
#  define TrifK_FEATURENOTSUPP (-2)
# endif

/*
 * connection mode of transport service (.servtype)
 */

# define TrifK_UNKNOWN_MODE 0xffff

/* connection-mode without orderly release: for example, OSI */ 
# ifndef T_COTS
#  define TrifK_CONNECTION_MODE T_COTS
# else
#  define TrifK_CONNECTION_MODE 01
# endif

/* connection-mode with orderly release: for example, TCP */ 
# ifndef T_COTS_ORD
#  define TrifK_ORD_CONNECTION_MODE T_COTS_ORD
# else
#  define TrifK_ORD_CONNECTION_MODE 02
# endif

/* connectionless-mode: for example, UDP */  
# ifndef T_CLTS
#  define TrifK_CONNECTIONLESS_MODE T_CLTS
# else
#  define TrifK_CONNECTIONLESS_MODE 03
# endif

/*
 * Flags (other info about transport provider)
 */

/* supports 0-length TSDUs */
# ifdef T_SENDZERO
#  define TrifF_SENDZERO T_SENDZERO
# else
#  define TrifF_SENDZERO 0x01
# endif

/* supports orderly release data */
# ifdef T_ORDRELDATA
#  define TrifF_ORDRELDATA T_ORDRELDATA
# else
#  define TrifF_ORDRELDATA 0x02
# endif

/* ------------------------------------------------------------------------- */

/******************************************************************************
 * %% BeginSection: misc. constant declarations
 */

# ifdef INADDR_ANY
#  define TrifK_ANY_INADDR INADDR_ANY
# else
#  define TrifK_ANY_INADDR ((u_int32_t)0)
# endif

# ifdef INADDR_LOOPBACK
#  define TrifK_LOOPBACK_INADDR INADDR_LOOPBACK
# else
#  define TrifK_LOOPBACK_INADDR ((u_int32_t)0x7f000001)
# endif

# ifdef INADDR_BROADCAST
#  define TrifK_BROADCAST_INADDR INADDR_BROADCAST
# else
#  define TrifK_BROADCAST_INADDR ((u_int32_t)0xffffffff)
# endif

# ifdef INADDR_NONE
#  define TrifK_NONE_INADDR INADDR_NONE
# else
#  define TrifK_NONE_INADDR (0xffffffff)
# endif

/* ------------------------------------------------------------------------- */

# define TrifK_LOOPBACK_DEFAULT_PORT (27015)

# define TrifL_HOSTNAME_MAX (256)
# define TrifL_SERVICENAME_MAX (128)

/******************************************************************************
 * %% BeginSection: endpoint declarations
 */
# pragma mark -
# pragma mark [ endpoint declarations ]
# pragma mark -

/*
 * Transport provider
 *
 * oti -> streams/xtiso
 * oti -> socket
 */
# define TRIF_DEVICE "dev/oti"
# define TRIF_DEVICE_PATH "/dev/oti"
# define TRIF_DEVICE_PATHDIR "/dev/oti/"

/* ------------------------------------------------------------------------- */

/*
 * loopback
 * @{
 */
# define TrifK_TICOTS_PROTOCOL (0x0fff + 0) /* @todo: fixme */
# define TRIF_TICOTS_PROTOCOL "ticots"
/* endpoint: /dev/ticots */
# define TRIF_DEV_COTS TRIF_DEVICE_PATHDIR TRIF_TICOTS_PROTOCOL
	/* connection transport service */

# define TrifK_TICOTSORD_PROTOCOL (0x0fff + 1)
# define TRIF_TICOTSORD_PROTOCOL "ticotsord"
/* endpoint: /dev/ticotsord */
# define TRIF_DEV_COTSORD TRIF_DEVICE_PATHDIR TRIF_TICOTSORD_PROTOCOL
	/* connection transport service with orderly release. [tcp] */

# define TrifK_TICLTS_PROTOCOL (0x0fff + 2)
# define TRIF_TICLTS_PROTOCOL "ticlts"
/* endpoint: /dev/ticlts */
# define TRIF_DEV_CLTS TRIF_DEVICE_PATHDIR TRIF_TICLTS_PROTOCOL
	/* connectionless transport service. [udp] */
/* @} */

/* ------------------------------------------------------------------------- */

/*
 * other well known endpoints
 * @{
 */

/* raw-socket endpoint: /dev/rawsocket */
# define TRIF_DEV_RAWSOCKET TRIF_DEVICE_PATHDIR "rawsocket"

/* icmp endpoint: /dev/icmp */
# define TRIF_DEV_ICMP TRIF_DEVICE_PATHDIR TRIF_ICMP_PROTOCOL
/* tcp endpoint: /dev/tcp */
# define TRIF_DEV_TCP TRIF_DEVICE_PATHDIR TRIF_TCP_PROTOCOL
/* udp endpoint: /dev/udp */
# define TRIF_DEV_UDP TRIF_DEVICE_PATHDIR TRIF_UDP_PROTOCOL

/******************************************************************************
 * %% BeginSection: socket type constant declarations
 */

# ifdef SOCK_STREAM
#  define TrifK_STREAM_SOCKET SOCK_STREAM
# else
#  define TrifK_STREAM_SOCKET (1)
# endif
# define TRIF_STREAM_SOCKET "stream"

# ifdef SOCK_DGRAM
#  define TrifK_DGRAM_SOCKET SOCK_DGRAM
# else
#  define TrifK_DGRAM_SOCKET (2)
# endif
# define TRIF_DGRAM_SOCKET "dgram"

# ifdef SOCK_RAW
#  define TrifK_RAW_SOCKET SOCK_RAW
# else
#  define TrifK_RAW_SOCKET (3)
# endif
# define TRIF_RAW_SOCKET "raw"

# ifdef SOCK_RDM
#  define TrifK_RDM_SOCKET SOCK_RDM
# else
#  define TrifK_RDM_SOCKET (4)
# endif
# define TRIF_RDM_SOCKET "rdm"

# ifdef SOCK_SEQPACKET
#  define TrifK_SEQPACKET_SOCKET SOCK_SEQPACKET
# else
#  define TrifK_SEQPACKET_SOCKET (5)
# endif
# define TRIF_SEQPACKET_SOCKET "seqpacket"

# define TrifK_INVAL_SOCKTYPE (-1)

/******************************************************************************
 * %% BeginSection: protocol constant declarations
 */
# pragma mark -
# pragma mark [ protocol constant declarations ]
# pragma mark -

# ifdef BO_SYSTYPE_WIN32

/*
 * WIN32
 * @{
 */
#  define TrifK_IP_PROTOCOL (0)
#  define TRIF_IP_PROTOCOL "ip"

#  define TrifK_HOPOPTS_PROTOCOL (0) /* IPv6 Hop-by-Hop options */
#  define TRIF_HOPOPTS_PROTOCOL "hopopts"

#  define TrifK_ICMP_PROTOCOL (1)
#  define TRIF_ICMP_PROTOCOL "icmp"

#  define TrifK_IGMP_PROTOCOL (2)
#  define TRIF_IGMP_PROTOCOL "igmp"

#  define TrifK_GGP_PROTOCOL (3)
#  define TRIF_GGP_PROTOCOL "ggp"

#  define TrifK_IPV4_PROTOCOL (4)
#  define TRIF_IPV4_PROTOCOL "ipv4"

#  define TrifK_ST_PROTOCOL (5)
#  define TRIF_ST_PROTOCOL "st"

#  define TrifK_TCP_PROTOCOL (6)
#  define TRIF_TCP_PROTOCOL "tcp"

#  define TrifK_CBT_PROTOCOL (7)
#  define TRIF_CBT_PROTOCOL "cbt"

#  define TrifK_EGP_PROTOCOL (8)
#  define TRIF_EGP_PROTOCOL "egp"

#  define TrifK_IGP_PROTOCOL (9)
#  define TRIF_IGP_PROTOCOL "igp"

#  define TrifK_PUP_PROTOCOL (12)
#  define TRIF_PUP_PROTOCOL "pup"

#  define TrifK_UDP_PROTOCOL (17)
#  define TRIF_UDP_PROTOCOL "udp"

#  define TrifK_IDP_PROTOCOL (22)
#  define TRIF_IDP_PROTOCOL "idp"

#  define TrifK_RDP_PROTOCOL (27)
#  define TRIF_RDP_PROTOCOL "rdp"


#  define TrifK_IPV6_PROTOCOL (41) /* IPv6 header */
#  define TRIF_IPV6_PROTOCOL "ipv6"

#  define TrifK_ROUTING_PROTOCOL (43) /* IPv6 Routing header */
#  define TRIF_ROUTING_PROTOCOL "routing"

#  define TrifK_FRAGMENT_PROTOCOL (44) /* IPv6 fragmentation header */
#  define TRIF_FRAGMENT_PROTOCOL "fragment"

#  define TrifK_ESP_PROTOCOL (50) /* encapsulating security payload */
#  define TRIF_ESP_PROTOCOL "esp"

#  define TrifK_AH_PROTOCOL (51) /* authentication header*/
#  define TRIF_AH_PROTOCOL "ah"

#  define TrifK_ICMPV6_PROTOCOL (58) /* ICMPv6 */
#  define TRIF_ICMPV6_PROTOCOL "icmpv6"

#  define TrifK_NONE_PROTOCOL (59) /* IPv6 no next header */
#  define TRIF_NONE_PROTOCOL "none"

#  define TrifK_DSTOPTS_PROTOCOL (60) /* IPv6 Destination options */
#  define TRIF_DSTOPTS_PROTOCOL "dstopts"

#  define TrifK_ND_PROTOCOL (77)
#  define TRIF_ND_PROTOCOL "nd"

#  define TrifK_ICLFXBM_PROTOCOL (78)
#  define TRIF_ICLFXBM_PROTOCOL "iclfxbm"

#  define TrifK_PIM_PROTOCOL (103)
#  define TRIF_PIM_PROTOCOL "pim"

#  define TrifK_PGM_PROTOCOL (113)
#  define TRIF_PGM_PROTOCOL "pgm"

#  define TrifK_L2TP_PROTOCOL (115)
#  define TRIF_L2TP_PROTOCOL "l2tp"

#  define TrifK_SCTP_PROTOCOL (132)
#  define TRIF_SCTP_PROTOCOL "sctp"

#  define TrifK_RAW_PROTOCOL (255)
#  define TRIF_RAW_PROTOCOL "raw"

#  define TrifK_MAX_PROTOCOL (256)
#  define TRIF_MAX_PROTOCOL "max"

/* @} */

# else

/*
 * UNIX
 * @{
 */

/* dummy for IP */
#  ifdef IPPROTO_IP
#   define TrifK_IP_PROTOCOL IPPROTO_IP
#  else
#   define TrifK_IP_PROTOCOL (0)
#  endif
#  define TRIF_IP_PROTOCOL "ip"

/* IP6 hop-by-hop options */
#  ifdef IPPROTO_HOPOPTS
#   define TrifK_HOPOPTS_PROTOCOL IPPROTO_HOPOPTS
#  else
#   define TrifK_HOPOPTS_PROTOCOL (0)
#  endif
#  define TRIF_HOPOPTS_PROTOCOL "hopopts"

 /* control message protocol */
#  ifdef IPPROTO_ICMP
#   define TrifK_ICMP_PROTOCOL IPPROTO_ICMP
#  else
#   define TrifK_ICMP_PROTOCOL (1)
#  endif
#  define TRIF_ICMP_PROTOCOL "icmp"

/* group mgmt protocol */
#  ifdef IPPROTO_IGMP
#   define TrifK_IGMP_PROTOCOL IPPROTO_IGMP
#  else
#   define TrifK_IGMP_PROTOCOL (2)
#  endif
#  define TRIF_IGMP_PROTOCOL "igmp"

/* gateway^2 (deprecated) */
#  ifdef IPPROTO_GGP
#   define TrifK_GGP_PROTOCOL IPPROTO_GGP
#  else
#   define TrifK_GGP_PROTOCOL (3)
#  endif
#  define TRIF_GGP_PROTOCOL "ggp"

/* IPv4 encapsulation */
#  ifdef IPPROTO_IPV4
#   define TrifK_IPV4_PROTOCOL IPPROTO_IPV4
#  else
#   define TrifK_IPV4_PROTOCOL (4)
#  endif
#  define TRIF_IPV4_PROTOCOL "ipv4"

/* for compatibility */
#  ifdef IPPROTO_IPIP
#   define TrifK_IPIP_PROTOCOL IPPROTO_IPIP
#  else
#   define TrifK_IPIP_PROTOCOL TrifK_IPV4_PROTOCOL
#  endif
#  define TRIF_IPIP_PROTOCOL "ipip"

/* tcp */
#  ifdef IPPROTO_TCP
#   define TrifK_TCP_PROTOCOL IPPROTO_TCP
#  else
#   define TrifK_TCP_PROTOCOL (6)
#  endif
#  define TRIF_TCP_PROTOCOL "tcp"

/* Stream protocol II */
#  ifdef IPPROTO_ST
#   define TrifK_ST_PROTOCOL IPPROTO_ST
#  else
#   define TrifK_ST_PROTOCOL (7)
#  endif
#  define TRIF_ST_PROTOCOL "st"

/* exterior gateway protocol */
#  ifdef IPPROTO_EGP
#   define TrifK_EGP_PROTOCOL IPPROTO_EGP
#  else
#   define TrifK_EGP_PROTOCOL (8)
#  endif
#  define TRIF_EGP_PROTOCOL "egp"

/* private interior gateway */
#  ifdef IPPROTO_PIGP
#   define TrifK_PIGP_PROTOCOL IPPROTO_PIGP
#  else
#   define TrifK_PIGP_PROTOCOL (9)
#  endif
#  define TRIF_PIGP_PROTOCOL "pigp"

/* BBN RCC Monitoring */
#  ifdef IPPROTO_RCCMON
#   define TrifK_RCCMON_PROTOCOL IPPROTO_RCCMON
#  else
#   define TrifK_RCCMON_PROTOCOL (10)
#  endif
#  define TRIF_RCCMON_PROTOCOL "rccmon"

/* network voice protocol*/
#  ifdef IPPROTO_NVPII
#   define TrifK_NVPII_PROTOCOL IPPROTO_NVPII
#  else
#   define TrifK_NVPII_PROTOCOL (11)
#  endif
#  define TRIF_NVPII_PROTOCOL "nvpii"

/* pup */
#  ifdef IPPROTO_PUP
#   define TrifK_PUP_PROTOCOL IPPROTO_PUP
#  else
#   define TrifK_PUP_PROTOCOL (12)
#  endif
#  define TRIF_PUP_PROTOCOL "pup"

/* Argus */
#  ifdef IPPROTO_ARGUS
#   define TrifK_ARGUS_PROTOCOL IPPROTO_ARGUS
#  else
#   define TrifK_ARGUS_PROTOCOL (13)
#  endif
#  define TRIF_ARGUS_PROTOCOL "argus"

/* EMCON */
#  ifdef IPPROTO_EMCON
#   define TrifK_EMCON_PROTOCOL IPPROTO_EMCON
#  else
#   define TrifK_EMCON_PROTOCOL (14)
#  endif
#  define TRIF_EMCON_PROTOCOL "emcon"

/* Cross Net Debugger */
#  ifdef IPPROTO_XNET
#   define TrifK_XNET_PROTOCOL IPPROTO_XNET
#  else
#   define TrifK_XNET_PROTOCOL (15)
#  endif
#  define TRIF_XNET_PROTOCOL "xnet"

/* Chaos*/
#  ifdef IPPROTO_CHAOS
#   define TrifK_CHAOS_PROTOCOL IPPROTO_CHAOS
#  else
#   define TrifK_CHAOS_PROTOCOL (16)
#  endif
#  define TRIF_CHAOS_PROTOCOL "chaos"

/* user datagram protocol */
#  ifdef IPPROTO_UDP
#   define TrifK_UDP_PROTOCOL IPPROTO_UDP
#  else
#   define TrifK_UDP_PROTOCOL (17)
#  endif
#  define TRIF_UDP_PROTOCOL "udp"

/* Multiplexing */
#  ifdef IPPROTO_MUX
#   define TrifK_MUX_PROTOCOL IPPROTO_MUX
#  else
#   define TrifK_MUX_PROTOCOL (18)
#  endif
#  define TRIF_MUX_PROTOCOL "mux"

/* DCN Measurement Subsystems */
#  ifdef IPPROTO_MEAS
#   define TrifK_MEAS_PROTOCOL IPPROTO_MEAS
#  else
#   define TrifK_MEAS_PROTOCOL (19)
#  endif
#  define TRIF_MEAS_PROTOCOL "meas"

/* Host Monitoring */
#  ifdef IPPROTO_HMP
#   define TrifK_HMP_PROTOCOL IPPROTO_HMP
#  else
#   define TrifK_HMP_PROTOCOL (20)
#  endif
#  define TRIF_HMP_PROTOCOL "hmp"

/* Packet Radio Measurement */
#  ifdef IPPROTO_PRM
#   define TrifK_PRM_PROTOCOL IPPROTO_PRM
#  else
#   define TrifK_PRM_PROTOCOL (21)
#  endif
#  define TRIF_PRM_PROTOCOL "prm"

/* xns idp */
#  ifdef IPPROTO_IDP
#   define TrifK_IDP_PROTOCOL IPPROTO_IDP
#  else
#   define TrifK_IDP_PROTOCOL (22)
#  endif
#  define TRIF_IDP_PROTOCOL "idp"

/* Trunk-1 */
#  ifdef IPPROTO_TRUNK1
#   define TrifK_TRUNK1_PROTOCOL IPPROTO_TRUNK1
#  else
#   define TrifK_TRUNK1_PROTOCOL (23)
#  endif
#  define TRIF_TRUNK1_PROTOCOL "trunk1"

/* Trunk-2 */
#  ifdef IPPROTO_TRUNK2
#   define TrifK_TRUNK2_PROTOCOL IPPROTO_TRUNK2
#  else
#   define TrifK_TRUNK2_PROTOCOL (24)
#  endif
#  define TRIF_TRUNK2_PROTOCOL "trunk2"

/* Leaf-1 */
#  ifdef IPPROTO_LEAF1
#   define TrifK_LEAF1_PROTOCOL IPPROTO_LEAF1
#  else
#   define TrifK_LEAF1_PROTOCOL (25)
#  endif
#  define TRIF_LEAF1_PROTOCOL "leaf1"

/* Leaf-2 */
#  ifdef IPPROTO_LEAF2
#   define TrifK_LEAF2_PROTOCOL IPPROTO_LEAF2
#  else
#   define TrifK_LEAF2_PROTOCOL (26)
#  endif
#  define TRIF_LEAF2_PROTOCOL "leaf2"

/* Reliable Data */
#  ifdef IPPROTO_RDP
#   define TrifK_RDP_PROTOCOL IPPROTO_RDP
#  else
#   define TrifK_RDP_PROTOCOL (27)
#  endif
#  define TRIF_RDP_PROTOCOL "rdp"

/* Reliable Transaction */
#  ifdef IPPROTO_IRTP
#   define TrifK_IRTP_PROTOCOL IPPROTO_IRTP
#  else
#   define TrifK_IRTP_PROTOCOL (28)
#  endif
#  define TRIF_IRTP_PROTOCOL "irtp"

/* tp-4 w/ class negotiation */
#  ifdef IPPROTO_TP
#   define TrifK_TP_PROTOCOL IPPROTO_TP
#  else
#   define TrifK_TP_PROTOCOL (29)
#  endif
#  define TRIF_TP_PROTOCOL "tp"

/* Bulk Data Transfer */
#  ifdef IPPROTO_BLT
#   define TrifK_BLT_PROTOCOL IPPROTO_BLT
#  else
#   define TrifK_BLT_PROTOCOL (30)
#  endif
#  define TRIF_BLT_PROTOCOL "blt"

/* Network Services */
#  ifdef IPPROTO_NSP
#   define TrifK_NSP_PROTOCOL IPPROTO_NSP
#  else
#   define TrifK_NSP_PROTOCOL (31)
#  endif
#  define TRIF_NSP_PROTOCOL "nsp"

/* Merit Internodal */
#  ifdef IPPROTO_INP
#   define TrifK_INP_PROTOCOL IPPROTO_INP
#  else
#   define TrifK_INP_PROTOCOL (32)
#  endif
#  define TRIF_INP_PROTOCOL "inp"

/* Sequential Exchange */
#  ifdef IPPROTO_SEP
#   define TrifK_SEP_PROTOCOL IPPROTO_SEP
#  else
#   define TrifK_SEP_PROTOCOL (33)
#  endif
#  define TRIF_SEP_PROTOCOL "sep"

/* Third Party Connect */
#  ifdef IPPROTO_3PC
#   define TrifK_3PC_PROTOCOL IPPROTO_3PC
#  else
#   define TrifK_3PC_PROTOCOL (34)
#  endif
#  define TRIF_3PC_PROTOCOL "3pc"

/* InterDomain Policy Routing */
#  ifdef IPPROTO_IDPR
#   define TrifK_IDPR_PROTOCOL IPPROTO_IDPR
#  else
#   define TrifK_IDPR_PROTOCOL (35)
#  endif
#  define TRIF_IDPR_PROTOCOL "idpr"

/* XTP */
#  ifdef IPPROTO_XTP
#   define TrifK_XTP_PROTOCOL IPPROTO_XTP
#  else
#   define TrifK_XTP_PROTOCOL (36)
#  endif
#  define TRIF_XTP_PROTOCOL "xtp"

/* Datagram Delivery */
#  ifdef IPPROTO_DDP
#   define TrifK_DDP_PROTOCOL IPPROTO_DDP
#  else
#   define TrifK_DDP_PROTOCOL (37)
#  endif
#  define TRIF_DDP_PROTOCOL "ddp"

/* Control Message Transport */
#  ifdef IPPROTO_CMTP
#   define TrifK_CMTP_PROTOCOL IPPROTO_CMTP
#  else
#   define TrifK_CMTP_PROTOCOL (38)
#  endif
#  define TRIF_CMTP_PROTOCOL "cmtp"

/* TP++ Transport */
#  ifdef IPPROTO_TPXX
#   define TrifK_TPXX_PROTOCOL IPPROTO_TPXX
#  else
#   define TrifK_TPXX_PROTOCOL (39)
#  endif
#  define TRIF_TPXX_PROTOCOL "tpxx"

/* IL transport protocol */
#  ifdef IPPROTO_IL
#   define TrifK_IL_PROTOCOL IPPROTO_IL
#  else
#   define TrifK_IL_PROTOCOL (40)
#  endif
#  define TRIF_IL_PROTOCOL "il"

/* IP6 header */
#  ifdef IPPROTO_IPV6
#   define TrifK_IPV6_PROTOCOL IPPROTO_IPV6
#  else
#   define TrifK_IPV6_PROTOCOL (41)
#  endif
#  define TRIF_IPV6_PROTOCOL "ipv6"

/* Source Demand Routing */
#  ifdef IPPROTO_SDRP
#   define TrifK_SDRP_PROTOCOL IPPROTO_IL
#  else
#   define TrifK_SDRP_PROTOCOL (42)
#  endif
#  define TRIF_SDRP_PROTOCOL "sdrp"

/* IP6 routing header */
#  ifdef IPPROTO_ROUTING
#   define TrifK_ROUTING_PROTOCOL IPPROTO_ROUTING
#  else
#   define TrifK_ROUTING_PROTOCOL (43)
#  endif
#  define TRIF_ROUTING_PROTOCOL "routing"

/* IP6 fragmentation header */
#  ifdef IPPROTO_FRAGMENT
#   define TrifK_FRAGMENT_PROTOCOL IPPROTO_FRAGMENT
#  else
#   define TrifK_FRAGMENT_PROTOCOL (44)
#  endif
#  define TRIF_FRAGMENT_PROTOCOL "fragment"

/* InterDomain Routing */
#  ifdef IPPROTO_IDRP
#   define TrifK_IDRP_PROTOCOL IPPROTO_IDRP
#  else
#   define TrifK_IDRP_PROTOCOL (45)
#  endif
#  define TRIF_IDRP_PROTOCOL "idrp"

/* resource reservation */
#  ifdef IPPROTO_RSVP
#   define TrifK_RSVP_PROTOCOL IPPROTO_RSVP
#  else
#   define TrifK_RSVP_PROTOCOL (46)
#  endif
#  define TRIF_RSVP_PROTOCOL "rsvp"

/* General Routing Encap. */
#  ifdef IPPROTO_GRE
#   define TrifK_GRE_PROTOCOL IPPROTO_GRE
#  else
#   define TrifK_GRE_PROTOCOL (47)
#  endif
#  define TRIF_GRE_PROTOCOL "gre"

/* Mobile Host Routing */
#  ifdef IPPROTO_MHRP
#   define TrifK_MHRP_PROTOCOL IPPROTO_MHRP
#  else
#   define TrifK_MHRP_PROTOCOL (48)
#  endif
#  define TRIF_MHRP_PROTOCOL "mhrp"

/* BHA */
#  ifdef IPPROTO_BHA
#   define TrifK_BHA_PROTOCOL IPPROTO_BHA
#  else
#   define TrifK_BHA_PROTOCOL (49)
#  endif
#  define TRIF_BHA_PROTOCOL "bha"

/* IP6 Encap Sec. Payload */
#  ifdef IPPROTO_ESP
#   define TrifK_ESP_PROTOCOL IPPROTO_ESP
#  else
#   define TrifK_ESP_PROTOCOL (50)
#  endif
#  define TRIF_ESP_PROTOCOL "esp"

/* IP6 Auth Header */
#  ifdef IPPROTO_AH
#   define TrifK_AH_PROTOCOL IPPROTO_AH
#  else
#   define TrifK_AH_PROTOCOL (51)
#  endif
#  define TRIF_AH_PROTOCOL "ah"

/* Integ. Net Layer Security */
#  ifdef IPPROTO_INLSP
#   define TrifK_INLSP_PROTOCOL IPPROTO_INLSP
#  else
#   define TrifK_INLSP_PROTOCOL (52)
#  endif
#  define TRIF_INLSP_PROTOCOL "inlsp"

/* IP with encryption */
#  ifdef IPPROTO_SWIPE
#   define TrifK_SWIPE_PROTOCOL IPPROTO_SWIPE
#  else
#   define TrifK_SWIPE_PROTOCOL (53)
#  endif
#  define TRIF_SWIPE_PROTOCOL "swipe"

/* Next Hop Resolution */
#  ifdef IPPROTO_NHRP
#   define TrifK_NHRP_PROTOCOL IPPROTO_NHRP
#  else
#   define TrifK_NHRP_PROTOCOL (54)
#  endif
#  define TRIF_NHRP_PROTOCOL "nhrp"

/* IP Mobility */
#  ifdef IPPROTO_MOBILE
#   define TrifK_MOBILE_PROTOCOL IPPROTO_MOBILE
#  else
#   define TrifK_MOBILE_PROTOCOL (55)
#  endif
#  define TRIF_MOBILE_PROTOCOL "mobile"

/* Transport Layer Security */
#  ifdef IPPROTO_TLSP
#   define TrifK_TLSP_PROTOCOL IPPROTO_TLSP
#  else
#   define TrifK_TLSP_PROTOCOL (56)
#  endif
#  define TRIF_TLSP_PROTOCOL "tlsp"

/* SKIP */
#  ifdef IPPROTO_SKIP
#   define TrifK_SKIP_PROTOCOL IPPROTO_SKIP
#  else
#   define TrifK_SKIP_PROTOCOL (57)
#  endif
#  define TRIF_SKIP_PROTOCOL "skip"

/* ICMP6 */
#  ifdef IPPROTO_ICMPV6
#   define TrifK_ICMPV6_PROTOCOL IPPROTO_ICMPV6
#  else
#   define TrifK_ICMPV6_PROTOCOL (58)
#  endif
#  define TRIF_ICMPV6_PROTOCOL "icmpv6"

/* IP6 no next header */
#  ifdef IPPROTO_NONE
#   define TrifK_NONE_PROTOCOL IPPROTO_NONE
#  else
#   define TrifK_NONE_PROTOCOL (59)
#  endif
#  define TRIF_NONE_PROTOCOL "none"

/* IP6 destination option */
#  ifdef IPPROTO_DSTOPTS
#   define TrifK_DSTOPTS_PROTOCOL IPPROTO_DSTOPTS
#  else
#   define TrifK_DSTOPTS_PROTOCOL (60)
#  endif
#  define TRIF_DSTOPTS_PROTOCOL "dstopts"

/* any host internal protocol */
#  ifdef IPPROTO_AHIP
#   define TrifK_AHIP_PROTOCOL IPPROTO_AHIP
#  else
#   define TrifK_AHIP_PROTOCOL (61)
#  endif
#  define TRIF_AHIP_PROTOCOL "ahip"

/* CFTP */
#  ifdef IPPROTO_CFTP
#   define TrifK_CFTP_PROTOCOL IPPROTO_CFTP
#  else
#   define TrifK_CFTP_PROTOCOL (62)
#  endif
#  define TRIF_CFTP_PROTOCOL "cftp"

/* "hello" routing protocol */
#  ifdef IPPROTO_HELLO
#   define TrifK_HELLO_PROTOCOL IPPROTO_HELLO
#  else
#   define TrifK_HELLO_PROTOCOL (63)
#  endif
#  define TRIF_HELLO_PROTOCOL "hello"

/* SATNET/Backroom EXPAK */
#  ifdef IPPROTO_SATEXPAK
#   define TrifK_SATEXPAK_PROTOCOL IPPROTO_SATEXPAK
#  else
#   define TrifK_SATEXPAK_PROTOCOL (64)
#  endif
#  define TRIF_SATEXPAK_PROTOCOL "satexpak"

/* Kryptolan */
#  ifdef IPPROTO_KRYPTOLAN
#   define TrifK_KRYPTOLAN_PROTOCOL IPPROTO_KRYPTOLAN
#  else
#   define TrifK_KRYPTOLAN_PROTOCOL (65)
#  endif
#  define TRIF_KRYPTOLAN_PROTOCOL "kryptolan"

/* Remote Virtual Disk */
#  ifdef IPPROTO_RVD
#   define TrifK_RVD_PROTOCOL IPPROTO_RVD
#  else
#   define TrifK_RVD_PROTOCOL (66)
#  endif
#  define TRIF_RVD_PROTOCOL "rvd"

/* Pluribus Packet Core */
#  ifdef IPPROTO_IPPC
#   define TrifK_IPPC_PROTOCOL IPPROTO_IPPC
#  else
#   define TrifK_IPPC_PROTOCOL (67)
#  endif
#  define TRIF_IPPC_PROTOCOL "ippc"

/* Any distributed FS */
#  ifdef IPPROTO_ADFS
#   define TrifK_ADFS_PROTOCOL IPPROTO_ADFS
#  else
#   define TrifK_ADFS_PROTOCOL (68)
#  endif
#  define TRIF_ADFS_PROTOCOL "adfs"

/* Satnet Monitoring */
#  ifdef IPPROTO_SATMON
#   define TrifK_SATMON_PROTOCOL IPPROTO_SATMON
#  else
#   define TrifK_SATMON_PROTOCOL (69)
#  endif
#  define TRIF_SATMON_PROTOCOL "satmon"

/* VISA Protocol */
#  ifdef IPPROTO_VISA
#   define TrifK_VISA_PROTOCOL IPPROTO_VISA
#  else
#   define TrifK_VISA_PROTOCOL (70)
#  endif
#  define TRIF_VISA_PROTOCOL "visa"

/* Packet Core Utility */
#  ifdef IPPROTO_IPCV
#   define TrifK_IPCV_PROTOCOL IPPROTO_IPCV
#  else
#   define TrifK_IPCV_PROTOCOL (71)
#  endif
#  define TRIF_IPCV_PROTOCOL "ipcv"

/* Comp. Prot. Net. Executive */
#  ifdef IPPROTO_CPNX
#   define TrifK_CPNX_PROTOCOL IPPROTO_CPNX
#  else
#   define TrifK_CPNX_PROTOCOL (72)
#  endif
#  define TRIF_CPNX_PROTOCOL "cpnx"

/* Comp. Prot. HeartBeat */
#  ifdef IPPROTO_CPHB
#   define TrifK_CPHB_PROTOCOL IPPROTO_CPHB
#  else
#   define TrifK_CPHB_PROTOCOL (73)
#  endif
#  define TRIF_CPHB_PROTOCOL "cphb"

/* Wang Span Network */
#  ifdef IPPROTO_WSN
#   define TrifK_WSN_PROTOCOL IPPROTO_WSN
#  else
#   define TrifK_WSN_PROTOCOL (74)
#  endif
#  define TRIF_WSN_PROTOCOL "wsn"

/* Packet Video Protocol */
#  ifdef IPPROTO_PVP
#   define TrifK_PVP_PROTOCOL IPPROTO_PVP
#  else
#   define TrifK_PVP_PROTOCOL (75)
#  endif
#  define TRIF_PVP_PROTOCOL "pvp"

/* BackRoom SATNET Monitoring */
#  ifdef IPPROTO_BRSATMON
#   define TrifK_BRSATMON_PROTOCOL IPPROTO_BRSATMON
#  else
#   define TrifK_BRSATMON_PROTOCOL (76)
#  endif
#  define TRIF_BRSATMON_PROTOCOL "brsatmon"

/* Sun net disk proto (temp.) */
#  ifdef IPPROTO_ND
#   define TrifK_ND_PROTOCOL IPPROTO_ND
#  else
#   define TrifK_ND_PROTOCOL (77)
#  endif
#  define TRIF_ND_PROTOCOL "nd"

/* WIDEBAND Monitoring */
#  ifdef IPPROTO_WBMON
#   define TrifK_WBMON_PROTOCOL IPPROTO_WBMON
#  else
#   define TrifK_WBMON_PROTOCOL (78)
#  endif
#  define TRIF_WBMON_PROTOCOL "wbmon"

/* WIDEBAND EXPAK */
#  ifdef IPPROTO_WBEXPAK
#   define TrifK_WBEXPAK_PROTOCOL IPPROTO_WBEXPAK
#  else
#   define TrifK_WBEXPAK_PROTOCOL (79)
#  endif
#  define TRIF_WBEXPAK_PROTOCOL "wbexpak"

/* ISO cnlp */
#  ifdef IPPROTO_EON
#   define TrifK_EON_PROTOCOL IPPROTO_EON
#  else
#   define TrifK_EON_PROTOCOL (80)
#  endif
#  define TRIF_EON_PROTOCOL "eon"

/* VMTP */
#  ifdef IPPROTO_VMTP
#   define TrifK_VMTP_PROTOCOL IPPROTO_VMTP
#  else
#   define TrifK_VMTP_PROTOCOL (81)
#  endif
#  define TRIF_VMTP_PROTOCOL "vmtp"

/* Secure VMTP */
#  ifdef IPPROTO_SVMTP
#   define TrifK_SVMTP_PROTOCOL IPPROTO_SVMTP
#  else
#   define TrifK_SVMTP_PROTOCOL (82)
#  endif
#  define TRIF_SVMTP_PROTOCOL "svmtp"

/* Banyon VINES */
#  ifdef IPPROTO_VINES
#   define TrifK_VINES_PROTOCOL IPPROTO_VINES
#  else
#   define TrifK_VINES_PROTOCOL (83)
#  endif
#  define TRIF_VINES_PROTOCOL "vines"

/* TTP */
#  ifdef IPPROTO_TTP
#   define TrifK_TTP_PROTOCOL IPPROTO_TTP
#  else
#   define TrifK_TTP_PROTOCOL (84)
#  endif
#  define TRIF_TTP_PROTOCOL "ttp"

/* NSFNET-IGP */
#  ifdef IPPROTO_IGP
#   define TrifK_IGP_PROTOCOL IPPROTO_IGP
#  else
#   define TrifK_IGP_PROTOCOL (85)
#  endif
#  define TRIF_IGP_PROTOCOL "igp"

/* dissimilar gateway prot. */
#  ifdef IPPROTO_DGP
#   define TrifK_DGP_PROTOCOL IPPROTO_DGP
#  else
#   define TrifK_DGP_PROTOCOL (86)
#  endif
#  define TRIF_DGP_PROTOCOL "dgp"

/* TCF */
#  ifdef IPPROTO_TCF
#   define TrifK_TCF_PROTOCOL IPPROTO_TCF
#  else
#   define TrifK_TCF_PROTOCOL (87)
#  endif
#  define TRIF_TCF_PROTOCOL "tcf"

/* Cisco/GXS IGRP */
#  ifdef IPPROTO_IGRP
#   define TrifK_IGRP_PROTOCOL IPPROTO_IGRP
#  else
#   define TrifK_IGRP_PROTOCOL (88)
#  endif
#  define TRIF_IGRP_PROTOCOL "igrp"

/* OSPFIGP */
#  ifdef IPPROTO_OSPFIGP
#   define TrifK_OSPFIGP_PROTOCOL IPPROTO_OSPFIGP
#  else
#   define TrifK_OSPFIGP_PROTOCOL (89)
#  endif
#  define TRIF_OSPFIGP_PROTOCOL "ospfigp"

/* Strite RPC protocol */
#  ifdef IPPROTO_SRPC
#   define TrifK_SRPC_PROTOCOL IPPROTO_SRPC
#  else
#   define TrifK_SRPC_PROTOCOL (90)
#  endif
#  define TRIF_SRPC_PROTOCOL "srpc"

/* Locus Address Resoloution */
#  ifdef IPPROTO_LARP
#   define TrifK_LARP_PROTOCOL IPPROTO_LARP
#  else
#   define TrifK_LARP_PROTOCOL (91)
#  endif
#  define TRIF_LARP_PROTOCOL "larp"

/* Multicast Transport */
#  ifdef IPPROTO_MTP
#   define TrifK_MTP_PROTOCOL IPPROTO_MTP
#  else
#   define TrifK_MTP_PROTOCOL (92)
#  endif
#  define TRIF_MTP_PROTOCOL "mtp"

/* AX.25 Frames */
#  ifdef IPPROTO_AX25
#   define TrifK_AX25_PROTOCOL IPPROTO_AX25
#  else
#   define TrifK_AX25_PROTOCOL (93)
#  endif
#  define TRIF_AX25_PROTOCOL "ax25"

/* IP encapsulated in IP */
#  ifdef IPPROTO_IPEIP
#   define TrifK_IPEIP_PROTOCOL IPPROTO_IPEIP
#  else
#   define TrifK_IPEIP_PROTOCOL (94)
#  endif
#  define TRIF_IPEIP_PROTOCOL "ipeip"

/* Mobile Int.ing control */
#  ifdef IPPROTO_MICP
#   define TrifK_MICP_PROTOCOL IPPROTO_MICP
#  else
#   define TrifK_MICP_PROTOCOL (95)
#  endif
#  define TRIF_MICP_PROTOCOL "micp"

/* Semaphore Comm. security */
#  ifdef IPPROTO_SCCSP
#   define TrifK_SCCSP_PROTOCOL IPPROTO_SCCSP
#  else
#   define TrifK_SCCSP_PROTOCOL (96)
#  endif
#  define TRIF_SCCSP_PROTOCOL "sccsp"

/* Ethernet IP encapsulation */
#  ifdef IPPROTO_ETHERIP
#   define TrifK_ETHERIP_PROTOCOL IPPROTO_ETHERIP
#  else
#   define TrifK_ETHERIP_PROTOCOL (97)
#  endif
#  define TRIF_ETHERIP_PROTOCOL "etherip"

/* encapsulation header */
#  ifdef IPPROTO_ENCAP
#   define TrifK_ENCAP_PROTOCOL IPPROTO_ENCAP
#  else
#   define TrifK_ENCAP_PROTOCOL (98)
#  endif
#  define TRIF_ENCAP_PROTOCOL "encap"

/* any private encr. scheme */
#  ifdef IPPROTO_APES
#   define TrifK_APES_PROTOCOL IPPROTO_APES
#  else
#   define TrifK_APES_PROTOCOL (99)
#  endif
#  define TRIF_APES_PROTOCOL "apes"

/* GMTP*/
#  ifdef IPPROTO_GMTP
#   define TrifK_GMTP_PROTOCOL IPPROTO_GMTP
#  else
#   define TrifK_GMTP_PROTOCOL (100)
#  endif
#  define TRIF_GMTP_PROTOCOL "gmtp"

/* payload compression (IPComp) */
#  ifdef IPPROTO_IPCOMP
#   define TrifK_IPCOMP_PROTOCOL IPPROTO_IPCOMP
#  else
#   define TrifK_IPCOMP_PROTOCOL (108)
#  endif
#  define TRIF_IPCOMP_PROTOCOL "ipcomp"

/* SCTP */
#  ifdef IPPROTO_SCTP
#   define TrifK_SCTP_PROTOCOL IPPROTO_SCTP
#  else
#   define TrifK_SCTP_PROTOCOL (132)
#  endif
#  define TRIF_SCTP_PROTOCOL "sctp"


/* 101-254: Partly Unassigned */

/* Protocol Independent Mcast */
#  ifdef IPPROTO_PIM
#   define TrifK_PIM_PROTOCOL IPPROTO_PIM
#  else
#   define TrifK_PIM_PROTOCOL (103)
#  endif
#  define TRIF_PIM_PROTOCOL "pim"

/* CARP */
#  ifdef IPPROTO_CARP
#   define TrifK_CARP_PROTOCOL IPPROTO_CARP
#  else
#   define TrifK_CARP_PROTOCOL (112)
#  endif
#  define TRIF_CARP_PROTOCOL "carp"

/* PGM */
#  ifdef IPPROTO_PGM
#   define TrifK_PGM_PROTOCOL IPPROTO_PGM
#  else
#   define TrifK_PGM_PROTOCOL (113)
#  endif
#  define TRIF_PGM_PROTOCOL "pgm"

/* PFSYNC */
#  ifdef IPPROTO_PFSYNC
#   define TrifK_PFSYNC_PROTOCOL IPPROTO_PFSYNC
#  else
#   define TrifK_PFSYNC_PROTOCOL (240)
#  endif
#  define TRIF_PFSYNC_PROTOCOL "pfsync"

/* OLD divert pseudo-protocol */
#  ifdef IPPROTO_OLD_DIVERT
#   define TrifK_OLD_DIVERT_PROTOCOL IPPROTO_OLD_DIVERT
#  else
#   define TrifK_OLD_DIVERT_PROTOCOL (254)
#  endif
#  define TRIF_OLD_DIVERT_PROTOCOL "old_divert"

#  ifdef IPPROTO_RAW
#   define TrifK_RAW_PROTOCOL IPPROTO_RAW
#  else
#   define TrifK_RAW_PROTOCOL (255)
#  endif
#  define TRIF_RAW_PROTOCOL "raw"

#  ifdef IPPROTO_MAX
#   define TrifK_MAX_PROTOCOL IPPROTO_MAX
#  else
#   define TrifK_MAX_PROTOCOL (256)
#  endif
#  define TRIF_MAX_PROTOCOL "max"


/* last return value of *_input(), meaning "all job for this pkt is done".  */
#  ifdef IPPROTO_DONE
#   define TrifK_DONE_PROTOCOL IPPROTO_DONE
#  else
#   define TrifK_DONE_PROTOCOL (257)
#  endif
#  define TRIF_DONE_PROTOCOL "done"


/* spacer for loadable protos */
#  ifdef IPPROTO_SPACER
#   define TrifK_SPACER_PROTOCOL IPPROTO_SPACER
#  else
#   define TrifK_SPACER_PROTOCOL (32767)
#  endif
#  define TRIF_SPACER_PROTOCOL "spacer"

/* @} */

# endif

# define TrifK_INVAL_PROTOCOL (-1)

/******************************************************************************
 * %% BeginSection: address family constant declarations
 */
# pragma mark -
# pragma mark [ address family constant declarations ]
# pragma mark -

/* unspecified */
# ifdef AF_UNSPEC
#  define TrifK_UNSPEC_FAMILY AF_UNSPEC
# else
#  define TrifK_UNSPEC_FAMILY (0)
# endif
# define TRIF_UNSPEC_FAMILY "unspec"

/* local to host_FAMILY (pipes, portals) */
#  ifdef AF_LOCAL
#   define TrifK_LOCAL_FAMILY AF_LOCAL
#  else
#   define TrifK_LOCAL_FAMILY TrifK_UNIX_FAMILY
#  endif
#  define TRIF_LOCAL_FAMILY "local"

/* standardized name for AF_LOCAL */
# ifdef AF_UNIX
#  define TrifK_UNIX_FAMILY AF_UNIX
# else
#  define TrifK_UNIX_FAMILY (1)
# endif
# define TRIF_UNIX_FAMILY "unix"

/* internetwork: UDP, TCP, etc. */
# ifdef AF_INET
#  define TrifK_INET_FAMILY AF_INET
# else
#  define TrifK_INET_FAMILY (2)
# endif
# define TRIF_INET_FAMILY "inet"

/* arpanet imp addresses */
# ifdef AF_IMPLINK
#  define TrifK_IMPLINK_FAMILY AF_IMPLINK
# else
#  define TrifK_IMPLINK_FAMILY (3)
# endif
# define TRIF_IMPLINK_FAMILY "implink"

/* pup protocols: e.g. BSP */
# ifdef AF_PUP
#  define TrifK_PUP_FAMILY AF_PUP
# else
#  define TrifK_PUP_FAMILY (4)
# endif
# define TRIF_PUP_FAMILY "pup"

/* mit CHAOS protocols */
# ifdef AF_CHAOS
#  define TrifK_CHAOS_FAMILY AF_CHAOS
# else
#  define TrifK_CHAOS_FAMILY (5)
# endif
# define TRIF_CHAOS_FAMILY "chaos"

/* SMB protocols */
#  ifdef AF_NETBIOS
#   define TrifK_NETBIOS_FAMILY AF_NETBIOS
#  else
#   define TrifK_NETBIOS_FAMILY (6)
#  endif
#  define TRIF_NETBIOS_FAMILY "netbios"

/* XEROX NS protocols */
#  ifdef AF_NS
#   define TrifK_NS_FAMILY AF_NS
#  else
#   define TrifK_NS_FAMILY (6)
#  endif
#  define TRIF_NS_FAMILY "ns"

/* ISO protocols */
# ifdef AF_ISO
#  define TrifK_ISO_FAMILY AF_ISO
# else
#  define TrifK_ISO_FAMILY (7)
# endif
# define TRIF_ISO_FAMILY "iso"

# ifdef AF_OSI
#  define TrifK_OSI_FAMILY AF_OSI
# else
#  define TrifK_OSI_FAMILY (TrifK_ISO_FAMILY)
# endif
# define TRIF_OSI_FAMILY "osi"

/* European computer manufacturers */
# ifdef AF_ECMA
#  define TrifK_ECMA_FAMILY AF_ECMA
# else
#  define TrifK_ECMA_FAMILY (8)
# endif
# define TRIF_ECMA_FAMILY "ecma"

/* datakit protocols */
# ifdef AF_DATAKIT
#  define TrifK_DATAKIT_FAMILY AF_DATAKIT
# else
#  define TrifK_DATAKIT_FAMILY (9)
# endif
# define TRIF_DATAKIT_FAMILY "datakit"

/* CCITT protocols, X.25 etc */
# ifdef AF_CCITT
#  define TrifK_CCITT_FAMILY AF_CCITT
# else
#  define TrifK_CCITT_FAMILY (10)
# endif
# define TRIF_CCITT_FAMILY "ccitt"

/* IBM SNA */
# ifdef AF_SNA
#  define TrifK_SNA_FAMILY AF_SNA
# else
#  define TrifK_SNA_FAMILY (11)
# endif
# define TRIF_SNA_FAMILY "sna"

/* DECnet */
# ifdef AF_DECnet
#  define TrifK_DECnet_FAMILY AF_DECnet
# else
#  define TrifK_DECnet_FAMILY (12)
# endif
# define TRIF_DECnet_FAMILY "decnet"

/* DEC Direct data link interface */
# ifdef AF_DLI
#  define TrifK_DLI_FAMILY AF_DLI
# else
#  define TrifK_DLI_FAMILY (13)
# endif
# define TRIF_DLI_FAMILY "dli"

/* LAT */
# ifdef AF_LAT
#  define TrifK_LAT_FAMILY AF_LAT
# else
#  define TrifK_LAT_FAMILY (14)
# endif
# define TRIF_LAT_FAMILY "lat"

/* NSC Hyperchannel */
# ifdef AF_HYLINK
#  define TrifK_HYLINK_FAMILY AF_HYLINK
# else
#  define TrifK_HYLINK_FAMILY (15)
# endif
# define TRIF_HYLINK_FAMILY "hylink"

/* Apple Talk */
# ifdef AF_APPLETALK
#  define TrifK_APPLETALK_FAMILY AF_APPLETALK
# else
#  define TrifK_APPLETALK_FAMILY (16)
# endif
# define TRIF_APPLETALK_FAMILY "appletalk"

/* Internal Routing Protocol */
# ifdef AF_ROUTE
#  define TrifK_ROUTE_FAMILY AF_ROUTE
# else
#  define TrifK_ROUTE_FAMILY (17)
# endif
# define TRIF_ROUTE_FAMILY "route"

/* Link layer interface */
# ifdef AF_LINK
#  define TrifK_LINK_FAMILY AF_LINK
# else
#  define TrifK_LINK_FAMILY (18)
# endif
# define TRIF_LINK_FAMILY "link"

/* eXpress Transfer Protocol_FAMILY (no AF) */
# ifdef pseudo_AF_XTP
#  define TrifK_pseudo_XTP_FAMILY pseudo_AF_XTP
# else
#  define TrifK_pseudo_XTP_FAMILY (19)
# endif
# define TRIF_pseudo_XTP_FAMILY "pseudo_xtp"

/* connection-oriented IP, aka ST II */
# ifdef AF_COIP
#  define TrifK_COIP_FAMILY AF_COIP
# else
#  define TrifK_COIP_FAMILY (20)
# endif
# define TRIF_COIP_FAMILY "coip"

/* Computer Network Technology */
# ifdef AF_CNT
#  define TrifK_CNT_FAMILY AF_CNT
# else
#  define TrifK_CNT_FAMILY (21)
# endif
# define TRIF_CNT_FAMILY "cnt"

/* Help Identify RTIP packets */
# ifdef pseudo_AF_RTIP
#  define TrifK_pseudo_RTIP_FAMILY pseudo_AF_RTIP
# else
#  define TrifK_pseudo_RTIP_FAMILY (22)
# endif
# define TRIF_pseudo_RTIP_FAMILY "pseudo_rtip"

/* Novell Internet Protocol */
# ifdef AF_IPX
#  define TrifK_IPX_FAMILY AF_IPX
# else
#  define TrifK_IPX_FAMILY (23)
# endif
# define TRIF_IPX_FAMILY "ipx"

/* Simple Internet Protocol */
# ifdef AF_SIP
#  define TrifK_SIP_FAMILY AF_SIP
# else
#  define TrifK_SIP_FAMILY (24)
# endif
# define TRIF_SIP_FAMILY "sip"

/* Help Identify PIP packets */
# ifdef pseudo_AF_PIP
#  define TrifK_pseudo_PIP_FAMILY pseudo_AF_PIP
# else
#  define TrifK_pseudo_PIP_FAMILY (25)
# endif
# define TRIF_pseudo_PIP_FAMILY "pseudo_pip"

/* Integrated Services Digital Network*/
# ifdef AF_ISDN
#  define TrifK_ISDN_FAMILY AF_ISDN
# else
#  define TrifK_ISDN_FAMILY (26)
# endif
# define TRIF_ISDN_FAMILY "isdn"

/* CCITT E.164 recommendation */
# ifdef AF_E164
#  define TrifK_E164_FAMILY AF_E164
# else
#  define TrifK_E164_FAMILY (TrifK_ISDN_FAMILY)
# endif
# define TRIF_E164_FAMILY "e164"

/* Internal key-management function */
# ifdef pseudo_AF_KEY
#  define TrifK_pseudo_KEY_FAMILY pseudo_AF_KEY
# else
#  define TrifK_pseudo_KEY_FAMILY (27)
# endif
# define TRIF_pseudo_KEY_FAMILY "pseudo_key"

/* IPv6 */
# ifdef AF_INET6
#  define TrifK_INET6_FAMILY AF_INET6
# else
#  define TrifK_INET6_FAMILY (28)
# endif
# define TRIF_INET6_FAMILY "inet6"

/* native ATM access */
# ifdef AF_NATM
#  define TrifK_NATM_FAMILY AF_NATM
# else
#  define TrifK_NATM_FAMILY (29)
# endif
# define TRIF_NATM_FAMILY "natm"

/* ATM */
# ifdef AF_ATM
#  define TrifK_ATM_FAMILY AF_ATM
# else
#  define TrifK_ATM_FAMILY (30)
# endif
# define TRIF_ATM_FAMILY "atm"

/* Used by BPF to not rewrite headers
* in interface output routine
*/
# ifdef pseudo_AF_HDRCMPLT
#  define TrifK_pseudo_HDRCMPLT_FAMILY pseudo_AF_HDRCMPLT
# else
#  define TrifK_pseudo_HDRCMPLT_FAMILY (31)
# endif
# define TRIF_pseudo_HDRCMPLT_FAMILY "pseudo_hdrcmplt"

/* Netgraph sockets */
# ifdef AF_NETGRAPH
#  define TrifK_NETGRAPH_FAMILY AF_NETGRAPH
# else
#  define TrifK_NETGRAPH_FAMILY (32)
# endif
# define TRIF_NETGRAPH_FAMILY "netgraph"

/* 802.3ad slow protocol */
# ifdef AF_SLOW
#  define TrifK_SLOW_FAMILY AF_SLOW
# else
#  define TrifK_SLOW_FAMILY (33)
# endif
# define TRIF_SLOW_FAMILY "slow"

/* Sitara cluster protocol */
# ifdef AF_SCLUSTER
#  define TrifK_SCLUSTER_FAMILY AF_SCLUSTER
# else
#  define TrifK_SCLUSTER_FAMILY (34)
# endif
# define TRIF_SCLUSTER_FAMILY "scluster"

# ifdef AF_ARP
#  define TrifK_ARP_FAMILY AF_ARP
# else
#  define TrifK_ARP_FAMILY (35)
# endif
# define TRIF_ARP_FAMILY "arp"

/* Bluetooth sockets */
# ifdef AF_BLUETOOTH
#  define TrifK_BLUETOOTH_FAMILY AF_BLUETOOTH
# else
#  define TrifK_BLUETOOTH_FAMILY (36)
# endif
# define TRIF_BLUETOOTH_FAMILY "bluetooth"

/* VoiceView */
# ifdef AF_VOICEVIEW
#  define TrifK_VOICEVIEW_FAMILY AF_VOICEVIEW
# else
#  define TrifK_VOICEVIEW_FAMILY (18)
# endif
# define TRIF_VOICEVIEW_FAMILY "voiceview"

/* Protocols from Firefox */
# ifdef AF_FIREFOX
#  define TrifK_FIREFOX_FAMILY AF_FIREFOX
# else
#  define TrifK_FIREFOX_FAMILY (19)
# endif
# define TRIF_FIREFOX_FAMILY "firefox"

/* Somebody is using this! */
# ifdef AF_UNKNOWN1
#  define TrifK_UNKNOWN1_FAMILY AF_UNKNOWN1
# else
#  define TrifK_UNKNOWN1_FAMILY (20)
# endif
# define TRIF_UNKNOWN1_FAMILY "unknown1"

/* Banyan */
# ifdef AF_BAN
#  define TrifK_BAN_FAMILY AF_BAN
# else
#  define TrifK_BAN_FAMILY (21)
# endif
# define TRIF_BAN_FAMILY "ban"

/* Microsoft Wolfpack */
# ifdef AF_CLUSTER
#  define TrifK_CLUSTER_FAMILY AF_CLUSTER
# else
#  define TrifK_CLUSTER_FAMILY (24)
# endif
# define TRIF_CLUSTER_FAMILY "cluster"

/* IEEE 1284.4 WG AF */
# ifdef AF_12844
#  define TrifK_12844_FAMILY AF_12844
# else
#  define TrifK_12844_FAMILY (25)
# endif
# define TRIF_12844_FAMILY "12844"

/* IrDA */
# ifdef AF_IRDA
#  define TrifK_IRDA_FAMILY AF_IRDA
# else
#  define TrifK_IRDA_FAMILY (26)
# endif
# define TRIF_IRDA_FAMILY "irda"

/* Network Designers OSI & gateway */
# ifdef AF_NETDES
#  define TrifK_NETDES_FAMILY AF_NETDES
# else
#  define TrifK_NETDES_FAMILY (28)
# endif
# define TRIF_NETDES_FAMILY "atm"

# ifdef AF_TCNPROCESS
#  define TrifK_TCNPROCESS_FAMILY AF_TCNPROCESS
# else
#  define TrifK_TCNPROCESS_FAMILY (29)
# endif
# define TRIF_TCNPROCESS_FAMILY "tcnprocess"

# ifdef AF_TCNMESSAGE
#  define TrifK_TCNMESSAGE_FAMILY AF_TCNMESSAGE
# else
#  define TrifK_TCNMESSAGE_FAMILY (30)
# endif
# define TRIF_TCNMESSAGE_FAMILY "tcnmessage"

# ifdef AF_ICLFXBM
#  define TrifK_ICLFXBM_FAMILY AF_ICLFXBM
# else
#  define TrifK_ICLFXBM_FAMILY (31)
# endif
# define TRIF_ICLFXBM_FAMILY "iclfxbm"

/* Bluetooth RFCOMM/L2CAP protocols */
# ifdef AF_BTH
#  define TrifK_BTH_FAMILY AF_BTH
# else
#  define TrifK_BTH_FAMILY (32)
# endif
# define TRIF_BTH_FAMILY "bth"

# ifdef AF_MAX
#  define TrifK_MAX_FAMILY AF_MAX
# else
#  define TrifK_MAX_FAMILY (37)
# endif
# define TRIF_MAX_FAMILY "max"

# define TrifK_INVAL_FAMILY (-1)

#endif /* !__BO_SERVICES_NET_TRIFDEFS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
