/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Gramtypes.h	(N.Panasiuk)
 * include/Bo/services/textan/Gramtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_TEXTAN_GRAMTYPES__
# define __BO_SERVICES_TEXTAN_GRAMTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* AorpL_CFGRAM_SYMBOL_ID_MAX defines the longest value of a symbol. */
# define AorpL_CFGRAM_SYMBOL_VALUE_MAX (32766)

# define _AorpK_CFGRAM_SYMBOL_VALUE_MASK    (0x3fff)
# define _AorpF_CFGRAM_NONTERMINAL_SYMBOL   (0x8000)
# define _AorpK_CFGRAM_INVALID_SYMBOL       (0xffff)

typedef u_int16_t aorp_cfgram_symbol_t;

struct aorp_cfgram_product {
	aorp_cfgram_symbol_t  gmpd_head; /* is nonterminal symbol */
	size_t                gmpd_symc; /* if (0 == .gmpd_symc) then is empty product */
/*
	aorp_cfgram_symbol_t  gmpd_symseq[FAM_LENGTH];
 */
};

# define aorp_cfgram_product_p(x) ((struct aorp_cfgram_product *)(x))

# define AORP_CFGRAM_PRODUCT_STRUCT(N) \
	struct __myc_xconcat(aorp_cfgram_product,N) { \
		aorp_cfgram_symbol_t  gmpd_head; \
		size_t                gmpd_symc; \
		aorp_cfgram_symbol_t  gmpd_symseq[N]; \
	}

/*
 * Context-free grammar declaration.
 */

struct aorp_cfgram {
	aorp_cfgram_symbol_t         gm_startsym; /* is nonterminal symbol */
	size_t                       gm_terminalc;
	size_t                       gm_nonterminalc;
	size_t                       gm_productc;
/*
	struct aorp_cfgram_product  *gm_productv[FAM_LENGTH];
 */
};

# define aorp_cfgram_p(x) ((struct aorp_cfgram *)(x))

# define AORP_CFGRAM_STRUCT(N) \
	struct __myc_xconcat(aorp_cfgram,N) { \
		aorp_cfgram_symbol_t         gm_startsym; \
		size_t                       gm_terminalc; \
		size_t                       gm_nonterminalc; \
		size_t                       gm_productc; \
		struct aorp_cfgram_product  *gm_productv[N]; \
	}

/* ============================================================================
 * MACRO-HELPER declarations
 * @{
 */

/*
aorp_cfgram_symbol_t
AORP_MAKE_CFGRAM_TERMINAL_SYMBOL(unsigned anValue);
 */
# define AORP_MAKE_CFGRAM_TERMINAL_SYMBOL(anValue) \
	((aorp_cfgram_symbol_t)(anValue) & _AorpK_CFGRAM_SYMBOL_VALUE_MASK)

/*
aorp_cfgram_symbol_t
AORP_MAKE_CFGRAM_NONTERMINAL_SYMBOL(unsigned anValue);
 */
# define AORP_MAKE_CFGRAM_NONTERMINAL_SYMBOL(anValue) \
	( ((aorp_cfgram_symbol_t)(anValue) & _AorpK_CFGRAM_SYMBOL_VALUE_MASK) \
		| _AorpF_CFGRAM_NONTERMINAL_SYMBOL \
	)

/*
_Bool
AORP_CFGRAM_SYMBOL_IsValid(aorp_cfgram_symbol_t aSymbol);
 */
# define AORP_CFGRAM_SYMBOL_IsValid(aSymbol) \
	(_AorpK_CFGRAM_INVALID_SYMBOL != (aSymbol))

/*
_Bool
AORP_CFGRAM_SYMBOL_IsTerminal(aorp_cfgram_symbol_t aSymbol);
 */
# define AORP_CFGRAM_SYMBOL_IsTerminal(aSymbol) \
	(!(_AorpF_CFGRAM_NONTERMINAL_SYMBOL & (aSymbol)))

/*
_Bool
AORP_CFGRAM_SYMBOL_IsNonterminal(aorp_cfgram_symbol_t aSymbol);
 */
# define AORP_CFGRAM_SYMBOL_IsNonterminal(aSymbol) \
	(_AorpF_CFGRAM_NONTERMINAL_SYMBOL & (aSymbol))

/*
unsigned
AORP_CFGRAM_SYMBOL_GetValue(aorp_cfgram_symbol_t aSymbol);
 */
# define AORP_CFGRAM_SYMBOL_GetValue(aSymbol) \
	((aSymbol) & _AorpK_CFGRAM_SYMBOL_VALUE_MASK)

/* ------------------------------------------------------------------------- */

/*
aorp_cfgram_symbol_t
AORP_CFGRAM_PRODUCT_Head(struct aorp_cfgram_product *aProduct);
 */
# define AORP_CFGRAM_PRODUCT_Head(aProduct) \
	(aorp_cfgram_product_p(aProduct)->gmpd_head)

/*
size_t
AORP_CFGRAM_PRODUCT_SymbolCount(struct aorp_cfgram_product *aProduct);
 */
# define AORP_CFGRAM_PRODUCT_SymbolCount(aProduct) \
	(aorp_cfgram_product_p(aProduct)->gmpd_symc)

/*
aorp_cfgram_symbol_t *
AORP_CFGRAM_PRODUCT_Symseq(struct aorp_cfgram_product *aProduct);
 */
# define AORP_CFGRAM_PRODUCT_Symseq(aProduct) \
	((aorp_cfgram_symbol_t *)((char *)(aProduct) + sizeof(struct aorp_cfgram_product)))

/*
_Bool
AORP_CFGRAM_PRODUCT_Empty(struct aorp_cfgram_product *aProduct);
 */
# define AORP_CFGRAM_PRODUCT_Empty(aProduct) \
	(0 == AORP_CFGRAM_PRODUCT_SymbolCount(aProduct))

/* ------------------------------------------------------------------------- */

/*
aorp_cfgram_symbol_t
AORP_CFGRAM_StartSymbol(struct aorp_cfgrm *aGrammar);
 */
# define AORP_CFGRAM_StartSymbol(aGrammar) \
	(aorp_cfgram_p(aGrammar)->gm_startsym)

/*
size_t
AORP_CFGRAM_TerminalCount(struct aorp_cfgrm *aGrammar);
 */
# define AORP_CFGRAM_TerminalCount(aGrammar) \
	(aorp_cfgram_p(aGrammar)->gm_terminalc)

/*
size_t
AORP_CFGRAM_NonterminalCount(struct aorp_cfgrm *aGrammar);
 */
# define AORP_CFGRAM_NonterminalCount(aGrammar) \
	(aorp_cfgram_p(aGrammar)->gm_nonterminalc)

/*
size_t
AORP_CFGRAM_ProductCount(struct aorp_cfgrm *aGrammar);
 */
# define AORP_CFGRAM_ProductCount(aGrammar) \
	(aorp_cfgram_p(aGrammar)->gm_productc)

/*
struct aorp_cfgram_product **
AORP_CFGRAM_ProductTable(struct aorp_cfgrm *aGrammar);
 */
# define AORP_CFGRAM_ProductTable(aGrammar) \
	((struct aorp_cfgram_product **)((char *)(aGrammar) + sizeof(struct aorp_cfgrm)))

/* @} */

#endif /* !__BO_SERVICES_TEXTAN_GRAMTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
