/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * TcpsiP.h	(N.Panasiuk)
 * include/Bo/services/textan/TcpsiP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_TCPSI_P__
# define __BO_SERVICES_TCPSI_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/tproType.h>
# include <Bo/uriType.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# define __aorpc_need_service_tcpsi 1

#endif /* !__BO_SERVICES_TCPSI_P__ */

#pragma mark -

#if __aorpc_uses_service(tcpsi)
# undef __aorpc_need_service_tcpsi

/*
 * tcpsi service
 * @{
 */

/*
 * tcpsi {\object(class)}
 * Text Command Protocols Infrastructure
 */
/*
# define AorpK_tcpsi_service (xxxx)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name tcpsi
# include <Bo/aorp/AorpCsrvc.h>

_A_service(tcpsi)

# if __aorpc_interface

	struct aorp_tcpstat {
		bo_textproto_t   id;
		bo_uri_scheme_t  uri;
		char const      *name;
		char const      *description;
	};

/* ------------------------------------------------------------------------- */

	struct Tcpsi_dbelem {
		bo_textproto_t        id;
		char const           *name;
		struct aorp_tcpstat  *sb;

		/* NOT FOR YOUR BUSINESS @{ */
		bo_rtbtelem_t         btelem1; /* enum-to-string */
		bo_rtbtelem_t         btelem2; /* string-to-enum */
		/* @} */
	};

# endif /* __aorpc_interface */

_A_end_ctb

	_A_attribute((nonnull(1)))
	_A_op(stat, "tcpsi::get-protocol-stat", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_textproto_t anId
		)),

	_A_attribute((nonnull(1)))
	_A_op(getnamebyid, "tcpsi::get-protocol-name-by-id", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_textproto_t anId
		)),

	_A_attribute((nonnull(1,3)))
	_A_op(getidbyname, "tcpsi::get-protocol-id-by-name", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aName
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(additab, "tcpsi::add-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Tcpsi_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(additab_nlk, "tcpsi::add-table-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Tcpsi_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmitab, "tcpsi::remove-table", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Tcpsi_dbelem aTable[]
		)),

	_A_attribute((nonnull(1,4)))
	_A_op(rmitab_nlk, "tcpsi::remove-table-nolock", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			size_t aCount,
			struct Tcpsi_dbelem aTable[]
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(tcpsi) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
