/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * TextcompP.h	(N.Panasiuk)
 * include/Bo/services/textan/TextcompP.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_TEXTAN_TEXTCOMP_P__
# define __BO_SERVICES_TEXTAN_TEXTCOMP_P__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: textcomp service declarations
 */
# pragma mark -
# pragma mark [ textcomp service declarations ]
# pragma mark -

# define __aorpc_need_service_textcomp 1

#endif /* !__BO_SERVICES_TEXTAN_TEXTCOMP_P__ */

#pragma mark -

#if __aorpc_uses_service(textcomp)
# undef __aorpc_need_service_textcomp

/*
 * textcomp {\object(class)} [Constructor]
 * Text Command Protocol
 */
/*
# define AorpK_textcomp_service (xxxx)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name textcomp
# include <Bo/aorp/AorpCsrvc.h>

_A_service(textcomp)

# if __aorpc_interface

	struct aorp_textcomp {
		char                     tcp_closure[AorpK_CLOSURE_SIZE];
		aorp_object_t            tcp_bend;

# ifndef AorpK_TEXTCOMP_OPAQUE_SIZE
#  define AorpK_TEXTCOMP_OPAQUE_SIZE (36)
# endif
		char                     tcp_opaque[AorpK_TEXTCOMP_OPAQUE_SIZE];
	};

# endif /* __aorpc_interface */

_A_end_ctb

	_A_return((
		BoS_SUCCESS = [ BoS_NORMAL = (); ];
		BoS_FAILURE = [ BoS_ERR = (BoK_RacError_NotConfigured); ];
	))
	_A_attribute((nonnull(1,5)))
	_A_op(open, "textcomp::open-protocol", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			bo_textproto_t aProtocol,
			struct aorp_textcomp *aTextcomp
		)),

	_A_return((
		BoS_SUCCESS = [ BoS_NORMAL = (); ];
		BoS_FAILURE = [ BoS_ERR = (BoK_ArgError_InvalidArgument); ];
	))
	_A_attribute((nonnull(1,3)))
	_A_op(close, "textcomp::close-protocol", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			struct aorp_textcomp *aTextcomp
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(textcomp) */

/* ------------------------------------------------------------------------- */
#pragma mark -
/* ------------------------------------------------------------------------- */

#if __aorpc_uses_service(textcompbend)
# undef __aorpc_need_service_textcompbend

/*
 * textcompbend service
 * @{
 */

/*
 * textcompbend {\ops}
 * Text Command Protocol Backend
 */
/*
# define AorpK_textcompbend_service (xxxx)
 */
# include <Bo/services/MIB/Services.h>

# define __aorpc_service_name textcompbend
# include <Bo/aorp/AorpCsrvc.h>

_A_service(textcompbend)

# if __aorpc_interface

# endif /* __aorpc_interface */

_A_end_ctb

	_A_return((
		BoS_SUCCESS = [ ];
		BoS_FAILURE = [ BoS_ERR = (/* @todo */...); ];
	))
	_A_attribute((nonnull(1,3)))
	_A_op(scanmsg, "textcompbend::scan-message-from-stream", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_object_t aStream
		)),

	_A_return((
		BoS_SUCCESS = [ ];
		BoS_FAILURE = [ BoS_ERR = (/* @todo */...); ];
	))
	_A_attribute((nonnull(1,3)))
	_A_op(scanmsg_b, "textcompbend::scan-message-from-buffer", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char const *aBuffer,
			size_t aBufsz
		)),

	_A_return((
		BoS_SUCCESS = [ ];
		BoS_FAILURE = [ BoS_ERR = (/* @todo */...); ];
	))
	_A_attribute((nonnull(1,3,4)))
	_A_op(printmsg, "textcompbend::print-message-to-stream", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			aorp_object_t aStream,
			aorp_object_t aMessage
		)),

	_A_return((
		BoS_SUCCESS = [ ];
		BoS_FAILURE = [ BoS_ERR = (/* @todo */...); ];
	))
	_A_attribute((nonnull(1,3,5)))
	_A_op(printmsg_b, "textcompbend::print-message-to-buffer", (
			struct aorp_closure *aClosure,
			struct aorp_error *anErrPtr,
			char *aBuffer,
			size_t aBufsz,
			aorp_object_t aMessage
		)),

_A_end;

# include <Bo/aorp/AorpCend.h>
/* @} */
#endif /* __aorpc_uses_service(textcompbend) */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
