/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Types.h	(N.Panasiuk)
 * include/Bo/services/textan/Types.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SERVICES_TEXTAN_TYPES__
# define __BO_SERVICES_TEXTAN_TYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/*
 * Forward declarations
 */
struct aorp_error;

/* ============================================================================
 * Table-element types
 */
# pragma mark -

/* NOTE!!! DO NOT MODIFY */
# define AorpK_TEXTAN_UCHAR_TABELEM   0 /* unsigned char */
# define AorpK_TEXTAN_USHORT_TABELEM  1 /* unsigned short */
# define AorpK_TEXTAN_UINT_TABELEM    2 /* unsigned int */
# define AorpK_TEXTAN_ULONG_TABELEM   3 /* unsigned long */

/* ============================================================================
 * Token types
 */
# pragma mark -

typedef u_int32_t aorp_tokenid_t;

/*
 * Common well-known tokens
 */

/* "INVALID" token */
# define AorpK_INVAL_TOKEN ((aorp_tokenid_t)-1)

/* "NULL" token */
# define AorpK_NULL_TOKEN (0)
# define AorpK_EOF_TOKEN AorpK_NULL_TOKEN

/* Unknown token */
# define AorpK_UNK_TOKEN (2)

# define AorpK_USER_AVAILABLE_TOKEN (3)

/* ------------------------------------------------------------------------- */

struct aorp_token {
	aorp_tokenid_t  tok_id;
	char const     *tok_lexeme;
	size_t          tok_lexlen;
};

/* ----------------------------------------------------------------------------
 * Token predicate
 */

struct aorp_token_predicate;

typedef bo_status_t
(__myc_tcalltype__(mcall) *aorp_token_predicate_fn)(
# if __myc_require(prototypes)
	struct aorp_token_predicate *aPredicate,
	unsigned long aRuleID,
	struct aorp_token *aToken,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((mcalltype, nonnull(1,3)));

struct aorp_token_predicate {
	aorp_token_predicate_fn exec;
};

/* ============================================================================
 * POSIX character classes
 */
# pragma mark -

# define AORP_TEXTAN_ALNUM_CCLS  "alnum"  /* [A-Za-z0-9] - alphanumeric characters */
# define AORP_TEXTAN_WORD_CCLS   "word"   /* [A-Za-z0-9_] - alphanumeric characters plus "_" */
# define AORP_TEXTAN_ALPHA_CCLS  "alpha"  /* [A-Za-z] - alphabetic characters */
# define AORP_TEXTAN_BLANK_CCLS  "blank"  /* [ \t] - space and tab */
# define AORP_TEXTAN_CNTRL_CCLS  "cntrl"  /* [\x00-\x1F\x7F] - control characters */
# define AORP_TEXTAN_DIGIT_CCLS  "digit"  /* [0-9] - digits */
# define AORP_TEXTAN_GRAPH_CCLS  "graph"  /* [\x21-\x7E] - visible characters */
# define AORP_TEXTAN_LOWER_CCLS  "lower"  /* [a-z] - lowercase letters */
# define AORP_TEXTAN_PRINT_CCLS  "print"  /* [\x20-\x7E] - visible characters and spaces */
# define AORP_TEXTAN_PUNCT_CCLS  "punct"  /* [-!"#$%&'()*+,./:;<=>?@[\\\]^_`{|}~] - punctuation characters */
# define AORP_TEXTAN_SPACE_CCLS  "space"  /* [ \t\r\n\v\f] - whitespace characters */
# define AORP_TEXTAN_UPPER_CCLS  "upper"  /* [A-Z] - uppercase letters */
# define AORP_TEXTAN_XDIGIT_CCLS "xdigit" /* [A-Fa-f0-9] - hexadecimal digits */

#endif /* !__BO_SERVICES_TEXTAN_TYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
