/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * BoStrtok, originated from FreeBSD-5.4 C Library strtok_r
 */

/*
 * Strtok.h	(V.Sokur)
 * include/Bo/stralg/Strtok.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_STRALG_STRTOK__
# define __BO_STRALG_STRTOK__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/Types.h>

/******************************************************************************
 * %% BeginSection: bo_strtok_ctx declarations
 */
# pragma mark -
# pragma mark [ bo_strtok_ctx declarations ]
# pragma mark -

struct bo_strtok_ctx {
	/* READONLY @{ */
	char                *stx_token;
	size_t               stx_toklen; /* not including a terminating NUL character */

	int                  stx_tokisa;
		/* 1. [0 - 0xff] - a token is a delimiter and a lexem-character at the same time;
		   2. 0x0101 - a token is a value;
		   3. small negative integer number specifies the index in the delim
		      substring and a token is an atom-expression. */

	int                  stx_delimc;
	/* @} */

	/* NOT FOR YOUR BUSINESS @{ */
	u_char_t            *stx_str,
	                    *stx_lookahead;

	u_char_t            *stx_aexprv,
	                    *stx_lexdelim,
	                    *stx_invdelim,
	                    *stx_delim;

	u_char_t const      *stx_aexprb, *stx_aexpre;
	int                  stx_aexprisa;

	u_char_t             stx_quotbuf[2];
	u_char_t             stx_lexemc;        /* token was delimited by a lexem-character */
	u_char_t             stx_atomc;         /* token was delimited by an atom expression */
	int                  stx_aexprsize;     /* size of an atom expression */
	/* @} */
};

struct bo_wcstok_ctx {
	/* READONLY @{ */
	wchar_t             *stx_token;
	size_t               stx_toklen; /* not including a terminating NUL character */

	int                  stx_tokisa;
		/* 1. [0 - 0xff] - a token is a delimiter and a lexem-character at the same time;
		   2. 0x0101 - a token is a value;
		   3. small negative integer number specifies the index in the delim
		      substring and a token is an atom-expression. */

	int                  stx_delimc;
	/* @} */

	/* NOT FOR YOUR BUSINESS @{ */
	wchar_t             *stx_str,
	                    *stx_lookahead;

	wchar_t             *stx_aexprv,
	                    *stx_lexdelim,
	                    *stx_invdelim,
	                    *stx_delim;

	wchar_t const       *stx_aexprb, *stx_aexpre;
	int                  stx_aexprisa;

	wchar_t              stx_quotbuf[2];
	wchar_t              stx_lexemc;        /* token was delimited by a lexem-character */
	wchar_t              stx_atomc;         /* token was delimited by an atom expression */
	int                  stx_aexprsize;     /* size of an atom expression */
	/* @} */
};

/*
 * FLAGS
 */

/* No write NUL ('\0') character at the end of the token. */
# define BoF_STOK_NOSCHG	((u_int32_t)0x01)

/* Preserve NL ('\n') character. */
# define BoF_STOK_NL	((u_int32_t)0x02)

/* Preserve CR ('\r') character. */
# define BoF_STOK_CR	((u_int32_t)0x04)

/* Atom-expression enclosed within single quotes - '...'.
   The atom is not delimiter itself. */
# define BoF_STOK_SQUOA	((u_int32_t)0x10)

/* Atom-expression enclosed within double quotes - "...".
   The atom is not delimiter itself. */
# define BoF_STOK_DQUOA	((u_int32_t)0x20)

# define BoF_STOK_QUOA	(BoF_STOK_SQUOA | BoF_STOK_DQUOA)

/* Ignore quoted atom-expressions.
   The flag is useful in functions of the families
   'BoStrword()' and 'BoStrsplit()', because these
   functions set flag BoF_STOK_QUOA by default. */
# define BoF_STOK_IQUOA	((u_int32_t)0x40)

/* Ignore Escape character '\'.
   By default, all functions handle Escape character. */
# define BoF_STOK_IESC	((u_int32_t)0x80)

/* Ignore Escape character '\' in an atom expressions. */
# define BoF_STOK_IAESC	((u_int32_t)0x0100)

/* Remove Escape character '\'.
   Useless, if the flag BoF_STOK_IESC (BoF_STOK_IAESC)
   is specified. */
# define BoF_STOK_RMESC	((u_int32_t)0x0200)

/* Only skip leading delimiters. */
# define BoF_STOK_SKIPLEAD	((u_int32_t)0x0400)

/* Only set new delimiters. */
# define BoF_STOK_SET_DELIM	((u_int32_t)0x0800)

/* The delimiter string is size_t in the bin form. */
# define BoF_STOK_BINARY	((u_int32_t)0x1000)

# if 0
enum /* bo_strtok_ctx.stx_tokisa */ {
	BoK_STOK_TOKEN_IS_SQUOT = -256,
	BoK_STOK_TOKEN_IS_DQUOT = -257,

	BoK_STOK_TOKEN_WATERLINE = 256,

	BoK_STOK_TOKEN_IS_VALUE
};
# else
#  define BoK_STOK_TOKEN_IS_SQUOT (-256)
#  define BoK_STOK_TOKEN_IS_DQUOT (-257)
#  define BoK_STOK_TOKEN_WATERLINE (256)
#  define BoK_STOK_TOKEN_IS_VALUE (257)
# endif


# define BO_STOK_AEXPR_BEGIN "[@"
# define BO_STOK_AEXPR_END "]"
# define BO_STOK_AEXPR_SEP "*"
# define BO_STOK_AEXPR_ALT "||"
# define BO_STOK_LEXDELIM_BEGIN "[!"
# define BO_STOK_LEXDELIM_END "]"
# define BO_STOK_INVDELIM_BEGIN "[^"
# define BO_STOK_INVDELIM_END "]"

# define BO_WCSTOK_AEXPR_BEGIN __myc_wtext("[@")
# define BO_WCSTOK_AEXPR_END __myc_wtext("]")
# define BO_WCSTOK_AEXPR_SEP __myc_wtext("*")
# define BO_WCSTOK_AEXPR_ALT __myc_wtext("||")
# define BO_WCSTOK_LEXDELIM_BEGIN __myc_wtext("[!")
# define BO_WCSTOK_LEXDELIM_END __myc_wtext("]")
# define BO_WCSTOK_INVDELIM_BEGIN __myc_wtext("[^")
# define BO_WCSTOK_INVDELIM_END __myc_wtext("]")

/*
 * Returned status
 */

/* Success */
# define BoS_STRTOK_EOS BoS_NORMAL /* no more tokens, no words in string, etc. */
# define BoS_STRTOK_OK BoS_OK

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(4)))
BoStrtok(
# if __myc_require(prototypes)
	char *aStr,                               /* input-string */
	char *aDelim,                             /* delimit-string */
		/* The delim string is (size_t *aCount) in the bin form,
		   if the flag BoF_STOK_BINARY is set. */
	u_int32_t Flags,                          /* tokenize-flags */
	struct bo_strtok_ctx *aCtx,               /* tokenize-context */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(4)))
BoWcstok(
# if __myc_require(prototypes)
	wchar_t *aStr,                            /* input-string */
	wchar_t *aDelim,                          /* delimit-string */
	u_int32_t Flags,                          /* tokenize-flags */
	struct bo_wcstok_ctx *aCtx,               /* tokenize-context */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);

# pragma mark -

/* error: -1; error-with-partial-split: -(0xff + elemc) */
__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,4)))
BoStrsplit(
# if __myc_require(prototypes)
	char *aStr,                               /* input-string */
	char *aDelim,                             /* delimit-string */
	u_int32_t Flags,                          /* tokenize-flags */
	char *aStrv[],                            /* output-strings-vector */
	size_t aSize,                             /* output-strings-vector-size */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,4)))
BoAStrsplit(
# if __myc_require(prototypes)
	char *aStr,                               /* input-string */
	char *aDelim,                             /* delimit-string */
	u_int32_t Flags,                          /* tokenize-flags */
	char ***aStrvPtr,                         /* output-strings-vector-ptr */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);

__myc_flinkage__(public,libBo) char **
__attribute__((nonnull(1)))
BoAStrsplit_s(
# if __myc_require(prototypes)
	char *aStr,
	struct aorp_error *anErrPtr
# endif
	);

# define BoAStrsplit_s(aStr)	((BoAStrsplit_s)(aStr, NULL))

# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1)))
BoStrwordc(
# if __myc_require(prototypes)
	char const *aStr,                         /* input-string */
	char *aDelim,                             /* delimit-string */
	u_int32_t Flags,                          /* tokenize-flags */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1)))
BoStrword(
# if __myc_require(prototypes)
	char const *aStr,                         /* input-string */
	char *aDelim,                             /* delimit-string */
	u_int32_t Flags,                          /* tokenize-flags */
	bo_index_t anIndex,                       /* word-index */
	char aWord[] __myc_defval__(NULL),        /* output-word */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1)))
BoStrlword(
# if __myc_require(prototypes)
	char const *aStr,                         /* input-string */
	char *aDelim,                             /* delimit-string */
	u_int32_t Flags,                          /* tokenize-flags */
	bo_index_t anIndex,                       /* word-index */
	char aWord[],                             /* output-word */
	size_t aSize,                             /* output-word-size */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);


# define BO_STRWORD_S_MAX	1024

__myc_flinkage__(public,libBo) char *
__attribute__((nonnull(1)))
BoStrword_nr(
# if __myc_require(prototypes)
	char const *aStr,
	bo_index_t anIndex,
	struct aorp_error *anErrPtr
# endif
	);

# define BoStrword_nr(aStr, anIndex)	((BoStrword_nr)(aStr, anIndex, NULL))


__myc_flinkage__(public,libBo) ssize_t
__attribute__((nonnull(1,5)))
BoAStrword(
# if __myc_require(prototypes)
	char const *aStr,                         /* input-string */
	char *aDelim,                             /* delimit-string */
	u_int32_t Flags,                          /* tokenize-flags */
	bo_index_t anIndex,                       /* word-index */
	char **aWordPtr,                          /* output-word-ptr */
	struct aorp_error *anErrPtr __myc_defval__(NULL) /* error */
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_STRALG_STRTOK__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
