/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Atomic.h	(V.Sokur)
 * include/Bo/sys/Atomic.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_ATOMIC__
# define __BO_SYS_ATOMIC__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
 * Various simple operations on memory, each of which is atomic in the
 * presence of interrupts and multiple processors.
 *
 * //
 * // BASIC PRIMITIVES
 * //
 *
 * // read
 * _Tp BoAtomicLoad_acq< _Tp >(volatile _Tp *aPtr): (*(aPtr))
 * // write
 * void BoAtomicStore_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue): (void)(*(aPtr) = (aValue))
 *
 * _Tp BoAtomicFetchClear< _Tp >(volatile _Tp *aPtr): (oldValue = *(aPtr), *(aPtr) = 0, oldValue)
 *
 * int BoAtomicCompareStore(volatile *aDst,aComparand,aSrc): ((*(aDst) == (aComparand)) ? (*(aDst) = (aSrc), !0) : 0)
 * int BoAtomicCompareStore_acq(volatile *aDst,aComparand,aSrc)
 * int BoAtomicCompareStore_rel(volatile *aDst,aComparand,aSrc)
 *
 * void BoAtomicSetmask(volatile *aPtr,aMask): (void)(*(aPtr) |= (aMask))
 * void BoAtomicSetmask_acq(volatile *aPtr,aMask)
 * void BoAtomicSetmask_rel(volatile *aPtr,aMask)
 * void BoAtomicClearmask(volatile *aPtr,aMask) (void)(*(aPtr) &= ~(aMask))
 * void BoAtomicClearmask_acq(volatile *aPtr,aMask)
 * void BoAtomicClearmask_rel(volatile *aPtr,aMask)
 *
 * _Tp BoAtomicFetchAdd< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) += (aValue), oldValue)
 * _Tp BoAtomicFetchAdd_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicFetchAdd_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 *
 * //
 * // OTHER PRIMITIVES
 * //
 *
 * _Tp BoAtomicFetchStore< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) = (aValue), oldValue)
 * _Tp BoAtomicFetchStore_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) = (aValue), oldValue)
 * _Tp BoAtomicFetchStore_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) = (aValue), oldValue)
 *
 * _Tp BoAtomicCompareFetchStore< _Tp >(volatile _Tp *aDst, _Tp aComparand, _Tp aSrc): (oldValue = *(aDst), WAIT(*(aDst) == (aComparand)) *(aDst) = (aSrc), oldValue)
 * _Tp BoAtomicCompareFetchStore_acq< _Tp >(volatile _Tp *aDst, _Tp aComparand, _Tp aSrc)
 * _Tp BoAtomicCompareFetchStore_rel< _Tp >(volatile _Tp *aDst, _Tp aComparand, _Tp aSrc)
 *
 * void BoAtomicAdd(volatile *aPtr,aValue): (void)(*(aPtr) += (aValue))
 * void BoAtomicAdd_acq(volatile *aPtr,aValue)
 * void BoAtomicAdd_rel(volatile *aPtr,aValue)
 * _Tp BoAtomicRetAdd< _Tp >(volatile _Tp *aPtr, _Tp aValue): (*(aPtr) += (aValue), *(aPtr))
 * _Tp BoAtomicRetAdd_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicRetAdd_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 * void BoAtomicSub(volatile *aPtr,aValue): (void)(*(aPtr) -= (aValue))
 * void BoAtomicSub_acq(volatile *aPtr,aValue)
 * void BoAtomicSub_rel(volatile *aPtr,aValue)
 * _Tp BoAtomicFetchSub< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) -= (aValue), oldValue)
 * _Tp BoAtomicFetchSub_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicFetchSub_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicRetSub< _Tp >(volatile _Tp *aPtr, _Tp aValue): (*(aPtr) -= (aValue), *(aPtr))
 * _Tp BoAtomicRetSub_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicRetSub_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 * _Tp BoAtomicInc< _Tp >(volatile _Tp *aPtr): (++(*(aPtr)))
 * _Tp BoAtomicDec< _Tp >(volatile _Tp *aPtr): (--(*(aPtr)))
 *
 * _Tp BoAtomicFetchAnd< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) &= (aValue), oldValue)
 * _Tp BoAtomicFetchAnd_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicFetchAnd_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 * _Tp BoAtomicRetAnd< _Tp >(volatile _Tp *aPtr, _Tp aValue): (*(aPtr) &= (aValue), *(aPtr))
 * _Tp BoAtomicRetAnd_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicRetAnd_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 * _Tp BoAtomicFetchOr< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) |= (aValue), oldValue)
 * _Tp BoAtomicFetchOr_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicFetchOr_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 * _Tp BoAtomicRetOr< _Tp >(volatile _Tp *aPtr, _Tp aValue): (*(aPtr) |= (aValue), *(aPtr))
 * _Tp BoAtomicRetOr_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicRetOr_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 * _Tp BoAtomicFetchXor< _Tp >(volatile _Tp *aPtr, _Tp aValue): (oldValue = *(aPtr), *(aPtr) ^= (aValue), oldValue)
 * _Tp BoAtomicFetchXor_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicFetchXor_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 * _Tp BoAtomicRetXor< _Tp >(volatile _Tp *aPtr, _Tp aValue): (*(aPtr) ^= (aValue), *(aPtr))
 * _Tp BoAtomicRetXor_acq< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 * _Tp BoAtomicRetXor_rel< _Tp >(volatile _Tp *aPtr, _Tp aValue)
 *
 */

/*
 * Bit test and set. They return the original value of the bit, and operate
 * on the specified bit of the specified value.
 *
 *
 * int BoAtomicBitTestAndSet32(volatile u_int32_t *aValuePtr, unsigned aBit);
 * int BoAtomicBitTestAndClear32(volatile u_int32_t *aValuePtr, unsigned aBit);
 *
 * int BoAtomicBitTestAndSet64(volatile u_int64_t *aValuePtr, unsigned aBit);
 * int BoAtomicBitTestAndClear64(volatile u_int64_t *aValuePtr, unsigned aBit);
 *
 */

# include <Bo/sys/Cdefs.h>

# include _BO_PLATFORM_HEADER(_Atomic.h)

/******************************************************************************
 * %% BeginSection: definitions
 */

# if __myc_support(macro_retval)

#  define _BO_ATOMIC_CMPFETCHSTORE_CALL(_Suff, _Tp, aDst, aComparand, aSrc) \
	__extension__ ({ \
		volatile _Tp *___dst = (volatile _Tp *)(aDst); \
		_Tp ___comparand = (_Tp)(aComparand), ___src = (_Tp)(aSrc); \
		do { \
		} while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(___dst,___comparand,___src)); \
		/* return */ (___comparand); \
	})

#  define _BO_ATOMIC_CMPFETCHSTORE_IMP(_Suff, _Tp, aDst, aComparand, aSrc) struct __hack

# else /* !__myc_support(macro_retval) */

#  define _BO_ATOMIC_CMPFETCHSTORE_CALL(_Suff, _Tp, aDst, aComparand, aSrc) \
	__myc_xconcat(_im_BoAtomicCompareFetchStore,_Suff)((volatile _Tp *)(aDst), (_Tp)(aComparand), (_Tp)(aSrc))

#  define _BO_ATOMIC_CMPFETCHSTORE_IMP(_Suff, _Tp, aDst, aComparand, aSrc) \
__myc_extern_inline__ _Tp \
__myc_xconcat(_im_BoAtomicCompareFetchStore,_Suff)(volatile _Tp *aDst, _Tp aComparand, _Tp aSrc) \
{ \
	do { \
	} \
	while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(aDst,aComparand,aSrc)); \
	return (aComparand); \
} \
struct __hack

# endif /* __myc_support(macro_retval) */

# if __myc_support(macro_retval)

#  define _BO_ATOMIC_FETCHSTORE_CALL(_Suff, _Tp, aDst, aSrc) \
	__extension__ ({ \
		volatile _Tp *___dst = (volatile _Tp *)(aDst); \
		_Tp ___oval, ___nval = (_Tp)(aSrc); \
		do { \
			___oval = *___dst; \
		} while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(___dst,___oval,___nval)); \
		/* return */ (___oval); \
	})

#  define _BO_ATOMIC_FETCHSTORE_IMP(_Suff, _Tp, aDst, aSrc) struct __hack

# else /* !__myc_support(macro_retval) */

#  define _BO_ATOMIC_FETCHSTORE_CALL(_Suff, _Tp, aDst, aSrc) \
	__myc_xconcat(_im_BoAtomicFetchStore,_Suff)((volatile _Tp *)(aDst), (_Tp)(aSrc))

#  define _BO_ATOMIC_FETCHSTORE_IMP(_Suff, _Tp, aDst, aSrc) \
__myc_extern_inline__ _Tp \
__myc_xconcat(_im_BoAtomicFetchStore,_Suff)(volatile _Tp *aDst, _Tp aSrc) \
{ \
	_Tp old; \
	do { \
		old = *aDst; \
	} \
	while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(aDst,old,aSrc)); \
	return (old); \
} \
struct __hack

# endif /* __myc_support(macro_retval) */

# if __myc_support(macro_retval)

#  define _BO_ATOMIC_FETCHBINOP_CALL(_Name, _Suff, _Tp, _Op, aDst, aSrc) \
	__extension__ ({ \
		volatile _Tp *___fetchbinop_dst = (volatile _Tp *)(aDst); \
		_Tp ___fetchbinop_oval, ___fetchbinop_nval, ___v = (_Tp)(aSrc); \
		do { \
			___fetchbinop_nval = ((___fetchbinop_oval = *___fetchbinop_dst) _Op ___v); \
		} while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(___fetchbinop_dst,___fetchbinop_oval,___fetchbinop_nval)); \
		/* return */ (___fetchbinop_oval); \
	})

#  define _BO_ATOMIC_FETCHBINOP_IMP(_Name, _Suff, _Tp, _Op, aDst, aSrc) struct __hack

# else /* !__myc_support(macro_retval) */

#  define _BO_ATOMIC_FETCHBINOP_CALL(_Name, _Suff, _Tp, _Op, aDst, aSrc) \
	__myc_xconcat3(_im_BoAtomicFetch,_Name,_Suff)(aDst, aSrc)

#  define _BO_ATOMIC_FETCHBINOP_IMP(_Name, _Suff, _Tp, _Op, aDst, aSrc) \
__myc_extern_inline__ _Tp \
__myc_xconcat3(_im_BoAtomicFetch,_Name,_Suff)(volatile _Tp *aDst, _Tp aSrc) \
{ \
	_Tp oval, nval; \
	do { \
		nval = ((oval = *aDst) _Op aSrc); \
	} \
	while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(aDst,oval,nval)); \
	return (oval); \
} \
struct __hack

# endif /* __myc_support(macro_retval) */

# if __myc_support(macro_retval)

#  define _BO_ATOMIC_RETBINOP_CALL(_Name, _Suff, _Tp, _Op, aDst, aSrc) \
	__extension__ ({ \
		volatile _Tp *___dst = (volatile _Tp *)(aDst); \
		_Tp ___oval, ___nval, ___v = (_Tp)(aSrc); \
		do { \
			___nval = ((___oval = *___dst) _Op ___v); \
		} while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(___dst,___oval,___nval)); \
		/* return */ (___nval); \
	})

#  define _BO_ATOMIC_RETBINOP_IMP(_Name, _Suff, _Tp, _Op, aDst, aSrc) struct __hack

# else /* !__myc_support(macro_retval) */

#  define _BO_ATOMIC_RETBINOP_CALL(_Name, _Suff, _Tp, _Op, aDst, aSrc) \
	__myc_xconcat3(_im_BoAtomicRet,_Name,_Suff)(aDst, aSrc)

#  define _BO_ATOMIC_RETBINOP_IMP(_Name, _Suff, _Tp, _Op, aDst, aSrc) \
__myc_extern_inline__ _Tp \
__myc_xconcat3(_im_BoAtomicRet,_Name,_Suff)(volatile _Tp *aDst, _Tp aSrc) \
{ \
	_Tp oval, nval; \
	do { \
		nval = ((oval = *aDst) _Op aSrc); \
	} \
	while (0 == __myc_xconcat(BoAtomicCompareStore,_Suff)(aDst,oval,nval)); \
	return (nval); \
} \
struct __hack

# endif /* __myc_support(macro_retval) */

/* ----------------------------------------------------------------------------
 * 8-bit operations
 */

/*
 * u_int8_t BoAtomicLoad8_acq(volatile u_int8_t *aPtr);
 * void BoAtomicStore8_rel(volatile u_int8_t *aPtr);
 */

# ifndef BoAtomicLoad8_acq
#  define BoAtomicLoad8_acq(aPtr) \
	_im_BoAtomicLoad8_acq((volatile u_int8_t *)(aPtr))
__myc_extern_inline__ u_int8_t
_im_BoAtomicLoad8_acq(volatile u_int8_t *aPtr)
{
	u_int8_t retval = *aPtr;
	BoAtomicReadMemoryBarrier();
	return (retval);
}
# endif /* !BoAtomicLoad8_acq */

# ifndef BoAtomicStore8_rel
#  define BoAtomicStore8_rel(aPtr,aValue) \
	_im_BoAtomicStore8_rel((volatile u_int8_t *)(aPtr),(u_int8_t)(aValue))
__myc_extern_inline__ void
_im_BoAtomicStore8_rel(volatile u_int8_t *aPtr, u_int8_t aValue) {
	BoAtomicWriteMemoryBarrier();
	*aPtr = aValue;
	return;
}
# endif /* !BoAtomicStore8_rel */

# ifdef BoAtomicCompareStore8

/*
 * u_int8_t BoAtomicFetchStore8(volatile u_int8_t *aDst, u_int8_t aValue);
 * u_int8_t BoAtomicFetchStore8_acq(volatile u_int8_t *aDst, u_int8_t aValue);
 * u_int8_t BoAtomicFetchStore8_rel(volatile u_int8_t *aDst, u_int8_t aValue);
 */

# ifndef BoAtomicFetchStore8

#  define BoAtomicFetchStore8(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(8,u_int8_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(8,u_int8_t,aDst,aValue);

#  define BoAtomicFetchStore8_acq(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(8_acq,u_int8_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(8_acq,u_int8_t,aDst,aValue);

#  define BoAtomicFetchStore8_rel(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(8_rel,u_int8_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(8_rel,u_int8_t,aDst,aValue);

# endif /* !BoAtomicFetchStore8 */

/*
 * u_int8_t BoAtomicCompareFetchStore8(volatile u_int8_t *aDst, u_int8_t aComparand, u_int8_t aValue);
 * u_int8_t BoAtomicCompareFetchStore8_acq(volatile u_int8_t *aDst, u_int8_t aComparand, u_int8_t aValue);
 * u_int8_t BoAtomicCompareFetchStore8_rel(volatile u_int8_t *aDst, u_int8_t aComparand, u_int8_t aValue);
 */

# ifndef BoAtomicCompareFetchStore8

#  define BoAtomicCompareFetchStore8(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(8,u_int8_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(8,u_int8_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore8_acq(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(8_acq,u_int8_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(8_acq,u_int8_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore8_rel(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(8_rel,u_int8_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(8_rel,u_int8_t,aDst,aComparand,aValue);

# endif /* !BoAtomicCompareFetchStore8 */

/*
 * u_int8_t BoAtomicFetchClear8(volatile u_int8_t *aPtr);
 */
# ifndef BoAtomicFetchClear8
#  define BoAtomicFetchClear8(aPtr) \
	BoAtomicFetchStore8((volatile u_int8_t *)(aPtr),(u_int8_t)(0))
# endif

/*
 * void BoAtomicAdd8(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicAdd8_acq(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicAdd8_rel(volatile u_int8_t *aPtr, u_int8_t aValue);
 */
# ifndef BoAtomicAdd8
#  define BoAtomicAdd8(aPtr,aValue) ((void) BoAtomicFetchAdd8(aPtr,aValue))
#  define BoAtomicAdd8_acq(aPtr,aValue) ((void) BoAtomicFetchAdd8_acq(aPtr,aValue))
#  define BoAtomicAdd8_rel(aPtr,aValue) ((void) BoAtomicFetchAdd8_rel(aPtr,aValue))
# endif

/*
 * u_int8_t BoAtomicFetchAdd8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchAdd8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchAdd8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicFetchAdd8

#  define BoAtomicFetchAdd8(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 8, u_int8_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 8, u_int8_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd8_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 8_acq, u_int8_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 8_acq, u_int8_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd8_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 8_rel, u_int8_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 8_rel, u_int8_t, +, aDst, aSrc);

# endif /* !BoAtomicFetchAdd8 */

/*
 * u_int8_t BoAtomicRetAdd8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetAdd8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetAdd8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicRetAdd8

#  define BoAtomicRetAdd8(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 8, u_int8_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 8, u_int8_t, +, aDst, aSrc);

#  define BoAtomicRetAdd8_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 8_acq, u_int8_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 8_acq, u_int8_t, +, aDst, aSrc);

#  define BoAtomicRetAdd8_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 8_rel, u_int8_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 8_rel, u_int8_t, +, aDst, aSrc);

# endif /* !BoAtomicRetAdd8 */

/*
 * void BoAtomicSub8(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicSub8_acq(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicSub8_rel(volatile u_int8_t *aPtr, u_int8_t aValue);
 */

# ifndef BoAtomicSub8

# define BoAtomicSub8(aPtr,aValue) \
	((void) BoAtomicAdd8(aPtr, ((u_int8_t)(-(int8_t)(aValue)))))

# define BoAtomicSub8_acq(aPtr,aValue) \
	((void) BoAtomicAdd8_acq(aPtr, ((u_int8_t)(-(int8_t)(aValue)))))

# define BoAtomicSub8_rel(aPtr,aValue) \
	((void) BoAtomicAdd8_rel(aPtr, ((u_int8_t)(-(int8_t)(aValue)))))

# endif /* !BoAtomicSub8 */

/*
 * void BoAtomicFetchSub8(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicFetchSub8_acq(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicFetchSub8_rel(volatile u_int8_t *aPtr, u_int8_t aValue);
 */

# ifndef BoAtomicFetchSub8

# define BoAtomicFetchSub8(aPtr,aValue) \
	BoAtomicFetchAdd8(aPtr, ((u_int8_t)(-(int8_t)(aValue))))

# define BoAtomicFetchSub8_acq(aPtr,aValue) \
	BoAtomicFetchAdd8_acq(aPtr, ((u_int8_t)(-(int8_t)(aValue))))

# define BoAtomicFetchSub8_rel(aPtr,aValue) \
	BoAtomicFetchAdd8_rel(aPtr, ((u_int8_t)(-(int8_t)(aValue))))

# endif /* !BoAtomicFetchSub8 */

/*
 * void BoAtomicRetSub8(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicRetSub8_acq(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicRetSub8_rel(volatile u_int8_t *aPtr, u_int8_t aValue);
 */

# ifndef BoAtomicRetSub8

# define BoAtomicRetSub8(aPtr,aValue) \
	BoAtomicRetAdd8(aPtr, ((u_int8_t)(-(int8_t)(aValue))))

# define BoAtomicRetSub8_acq(aPtr,aValue) \
	BoAtomicRetAdd8_acq(aPtr, ((u_int8_t)(-(int8_t)(aValue))))

# define BoAtomicRetSub8_rel(aPtr,aValue) \
	BoAtomicRetAdd8_rel(aPtr, ((u_int8_t)(-(int8_t)(aValue))))

# endif /* !BoAtomicRetSub8 */

/*
 * u_int8_t BoAtomicInc8(volatile u_int8_t *aPtr);
 * u_int8_t BoAtomicDec8(volatile u_int8_t *aPtr);
 */
# ifndef BoAtomicInc8
#  define BoAtomicInc8(aPtr) BoAtomicRetAdd8(aPtr,1)
# endif
# ifndef BoAtomicDec8
#  define BoAtomicDec8(aPtr) BoAtomicRetSub8(aPtr,1)
# endif

/*
 * u_int8_t BoAtomicFetchAnd8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchAnd8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchAnd8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicFetchAnd8

#  define BoAtomicFetchAnd8(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 8, u_int8_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 8, u_int8_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd8_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 8_acq, u_int8_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 8_acq, u_int8_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd8_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 8_rel, u_int8_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 8_rel, u_int8_t, &, aDst, aSrc);

# endif /* !BoAtomicFetchAnd8 */

/*
 * u_int8_t BoAtomicRetAnd8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetAnd8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetAnd8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicRetAnd8

#  define BoAtomicRetAnd8(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 8, u_int8_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 8, u_int8_t, &, aDst, aSrc);

#  define BoAtomicRetAnd8_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 8_acq, u_int8_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 8_acq, u_int8_t, &, aDst, aSrc);

#  define BoAtomicRetAnd8_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 8_rel, u_int8_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 8_rel, u_int8_t, &, aDst, aSrc);

# endif /* !BoAtomicRetAnd8 */

/*
 * u_int8_t BoAtomicFetchOr8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchOr8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchOr8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicFetchOr8

#  define BoAtomicFetchOr8(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 8, u_int8_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 8, u_int8_t, |, aDst, aSrc);

#  define BoAtomicFetchOr8_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 8_acq, u_int8_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 8_acq, u_int8_t, |, aDst, aSrc);

#  define BoAtomicFetchOr8_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 8_rel, u_int8_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 8_rel, u_int8_t, |, aDst, aSrc);

# endif /* !BoAtomicFetchOr8 */

/*
 * u_int8_t BoAtomicRetOr8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetOr8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetOr8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicRetOr8

#  define BoAtomicRetOr8(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 8, u_int8_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 8, u_int8_t, |, aDst, aSrc);

#  define BoAtomicRetOr8_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 8_acq, u_int8_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 8_acq, u_int8_t, |, aDst, aSrc);

#  define BoAtomicRetOr8_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 8_rel, u_int8_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 8_rel, u_int8_t, |, aDst, aSrc);

# endif /* !BoAtomicRetOr8 */

/*
 * u_int8_t BoAtomicFetchXor8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchXor8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicFetchXor8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicFetchXor8

#  define BoAtomicFetchXor8(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 8, u_int8_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 8, u_int8_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor8_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 8_acq, u_int8_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 8_acq, u_int8_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor8_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 8_rel, u_int8_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 8_rel, u_int8_t, ^, aDst, aSrc);

# endif /* !BoAtomicFetchXor8 */

/*
 * u_int8_t BoAtomicRetXor8(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetXor8_acq(volatile u_int8_t *aDst, u_int8_t aSrc);
 * u_int8_t BoAtomicRetXor8_rel(volatile u_int8_t *aDst, u_int8_t aSrc);
 */

# ifndef BoAtomicRetXor8

#  define BoAtomicRetXor8(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 8, u_int8_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 8, u_int8_t, ^, aDst, aSrc);

#  define BoAtomicRetXor8_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 8_acq, u_int8_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 8_acq, u_int8_t, ^, aDst, aSrc);

#  define BoAtomicRetXor8_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 8_rel, u_int8_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 8_rel, u_int8_t, ^, aDst, aSrc);

# endif /* !BoAtomicRetXor8 */

# endif /* BoAtomicCompareStore8 */

/* ----------------------------------------------------------------------------
 * 16-bit operations
 */

/*
 * u_int16_t BoAtomicLoad16_acq(volatile u_int16_t *aPtr);
 * void BoAtomicStore16_rel(volatile u_int16_t *aPtr);
 */

# ifndef BoAtomicLoad16_acq
#  define BoAtomicLoad16_acq(aPtr) \
	_im_BoAtomicLoad16_acq((volatile u_int16_t *)(aPtr))
__myc_extern_inline__ u_int16_t
_im_BoAtomicLoad16_acq(volatile u_int16_t *aPtr)
{
	u_int16_t retval = *aPtr;
	BoAtomicReadMemoryBarrier();
	return (retval);
}
# endif /* !BoAtomicLoad16_acq */

# ifndef BoAtomicStore16_rel
#  define BoAtomicStore16_rel(aPtr,aValue) \
	_im_BoAtomicStore16_rel((volatile u_int16_t *)(aPtr),(u_int16_t)(aValue))
__myc_extern_inline__ void
_im_BoAtomicStore16_rel(volatile u_int16_t *aPtr, u_int16_t aValue) {
	BoAtomicWriteMemoryBarrier();
	*aPtr = aValue;
	return;
}
# endif /* !BoAtomicStore16_rel */

# ifdef BoAtomicCompareStore16

/*
 * u_int16_t BoAtomicFetchStore16(volatile u_int16_t *aDst, u_int16_t aValue);
 * u_int16_t BoAtomicFetchStore16_acq(volatile u_int16_t *aDst, u_int16_t aValue);
 * u_int16_t BoAtomicFetchStore16_rel(volatile u_int16_t *aDst, u_int16_t aValue);
 */

# ifndef BoAtomicFetchStore16

#  define BoAtomicFetchStore16(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(16,u_int16_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(16,u_int16_t,aDst,aValue);

#  define BoAtomicFetchStore16_acq(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(16_acq,u_int16_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(16_acq,u_int16_t,aDst,aValue);

#  define BoAtomicFetchStore16_rel(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(16_rel,u_int16_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(16_rel,u_int16_t,aDst,aValue);

# endif /* !BoAtomicFetchStore16 */

/*
 * u_int16_t BoAtomicCompareFetchStore16(volatile u_int16_t *aDst, u_int16_t aComparand, u_int16_t aValue);
 * u_int16_t BoAtomicCompareFetchStore16_acq(volatile u_int16_t *aDst, u_int16_t aComparand, u_int16_t aValue);
 * u_int16_t BoAtomicCompareFetchStore16_rel(volatile u_int16_t *aDst, u_int16_t aComparand, u_int16_t aValue);
 */

# ifndef BoAtomicCompareFetchStore16

#  define BoAtomicCompareFetchStore16(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(16,u_int16_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(16,u_int16_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore16_acq(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(16_acq,u_int16_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(16_acq,u_int16_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore16_rel(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(16_rel,u_int16_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(16_rel,u_int16_t,aDst,aComparand,aValue);

# endif /* !BoAtomicCompareFetchStore16 */

/*
 * u_int16_t BoAtomicFetchClear16(volatile u_int16_t *aPtr);
 */
# ifndef BoAtomicFetchClear16
#  define BoAtomicFetchClear16(aPtr) \
	BoAtomicFetchStore16((volatile u_int16_t *)(aPtr),(u_int16_t)(0))
# endif

/*
 * void BoAtomicAdd16(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicAdd16_acq(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicAdd16_rel(volatile u_int16_t *aPtr, u_int16_t aValue);
 */
# ifndef BoAtomicAdd16
#  define BoAtomicAdd16(aPtr,aValue) ((void) BoAtomicFetchAdd16(aPtr,aValue))
#  define BoAtomicAdd16_acq(aPtr,aValue) ((void) BoAtomicFetchAdd16_acq(aPtr,aValue))
#  define BoAtomicAdd16_rel(aPtr,aValue) ((void) BoAtomicFetchAdd16_rel(aPtr,aValue))
# endif

/*
 * u_int16_t BoAtomicFetchAdd16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchAdd16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchAdd16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicFetchAdd16

#  define BoAtomicFetchAdd16(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 16, u_int16_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 16, u_int16_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd16_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 16_acq, u_int16_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 16_acq, u_int16_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd16_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 16_rel, u_int16_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 16_rel, u_int16_t, +, aDst, aSrc);

# endif /* !BoAtomicFetchAdd16 */

/*
 * u_int16_t BoAtomicRetAdd16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetAdd16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetAdd16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicRetAdd16

#  define BoAtomicRetAdd16(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 16, u_int16_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 16, u_int16_t, +, aDst, aSrc);

#  define BoAtomicRetAdd16_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 16_acq, u_int16_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 16_acq, u_int16_t, +, aDst, aSrc);

#  define BoAtomicRetAdd16_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 16_rel, u_int16_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 16_rel, u_int16_t, +, aDst, aSrc);

# endif /* !BoAtomicRetAdd16 */

/*
 * void BoAtomicSub16(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicSub16_acq(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicSub16_rel(volatile u_int16_t *aPtr, u_int16_t aValue);
 */

# ifndef BoAtomicSub16

# define BoAtomicSub16(aPtr,aValue) \
	((void) BoAtomicAdd16(aPtr, ((u_int16_t)(-(int16_t)(aValue)))))

# define BoAtomicSub16_acq(aPtr,aValue) \
	((void) BoAtomicAdd16_acq(aPtr, ((u_int16_t)(-(int16_t)(aValue)))))

# define BoAtomicSub16_rel(aPtr,aValue) \
	((void) BoAtomicAdd16_rel(aPtr, ((u_int16_t)(-(int16_t)(aValue)))))

# endif /* !BoAtomicSub16 */

/*
 * void BoAtomicFetchSub16(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicFetchSub16_acq(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicFetchSub16_rel(volatile u_int16_t *aPtr, u_int16_t aValue);
 */

# ifndef BoAtomicFetchSub16

# define BoAtomicFetchSub16(aPtr,aValue) \
	BoAtomicFetchAdd16(aPtr, ((u_int16_t)(-(int16_t)(aValue))))

# define BoAtomicFetchSub16_acq(aPtr,aValue) \
	BoAtomicFetchAdd16_acq(aPtr, ((u_int16_t)(-(int16_t)(aValue))))

# define BoAtomicFetchSub16_rel(aPtr,aValue) \
	BoAtomicFetchAdd16_rel(aPtr, ((u_int16_t)(-(int16_t)(aValue))))

# endif /* !BoAtomicFetchSub16 */

/*
 * void BoAtomicRetSub16(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicRetSub16_acq(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicRetSub16_rel(volatile u_int16_t *aPtr, u_int16_t aValue);
 */

# ifndef BoAtomicRetSub16

# define BoAtomicRetSub16(aPtr,aValue) \
	BoAtomicRetAdd16(aPtr, ((u_int16_t)(-(int16_t)(aValue))))

# define BoAtomicRetSub16_acq(aPtr,aValue) \
	BoAtomicRetAdd16_acq(aPtr, ((u_int16_t)(-(int16_t)(aValue))))

# define BoAtomicRetSub16_rel(aPtr,aValue) \
	BoAtomicRetAdd16_rel(aPtr, ((u_int16_t)(-(int16_t)(aValue))))

# endif /* !BoAtomicRetSub16 */

/*
 * u_int16_t BoAtomicInc16(volatile u_int16_t *aPtr);
 * u_int16_t BoAtomicDec16(volatile u_int16_t *aPtr);
 */
# ifndef BoAtomicInc16
#  define BoAtomicInc16(aPtr) BoAtomicRetAdd16(aPtr,1)
# endif
# ifndef BoAtomicDec16
#  define BoAtomicDec16(aPtr) BoAtomicRetSub16(aPtr,1)
# endif

/*
 * u_int16_t BoAtomicFetchAnd16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchAnd16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchAnd16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicFetchAnd16

#  define BoAtomicFetchAnd16(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 16, u_int16_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 16, u_int16_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd16_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 16_acq, u_int16_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 16_acq, u_int16_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd16_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 16_rel, u_int16_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 16_rel, u_int16_t, &, aDst, aSrc);

# endif /* !BoAtomicFetchAnd16 */

/*
 * u_int16_t BoAtomicRetAnd16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetAnd16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetAnd16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicRetAnd16

#  define BoAtomicRetAnd16(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 16, u_int16_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 16, u_int16_t, &, aDst, aSrc);

#  define BoAtomicRetAnd16_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 16_acq, u_int16_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 16_acq, u_int16_t, &, aDst, aSrc);

#  define BoAtomicRetAnd16_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 16_rel, u_int16_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 16_rel, u_int16_t, &, aDst, aSrc);

# endif /* !BoAtomicRetAnd16 */

/*
 * u_int16_t BoAtomicFetchOr16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchOr16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchOr16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicFetchOr16

#  define BoAtomicFetchOr16(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 16, u_int16_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 16, u_int16_t, |, aDst, aSrc);

#  define BoAtomicFetchOr16_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 16_acq, u_int16_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 16_acq, u_int16_t, |, aDst, aSrc);

#  define BoAtomicFetchOr16_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 16_rel, u_int16_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 16_rel, u_int16_t, |, aDst, aSrc);

# endif /* !BoAtomicFetchOr16 */

/*
 * u_int16_t BoAtomicRetOr16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetOr16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetOr16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicRetOr16

#  define BoAtomicRetOr16(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 16, u_int16_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 16, u_int16_t, |, aDst, aSrc);

#  define BoAtomicRetOr16_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 16_acq, u_int16_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 16_acq, u_int16_t, |, aDst, aSrc);

#  define BoAtomicRetOr16_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 16_rel, u_int16_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 16_rel, u_int16_t, |, aDst, aSrc);

# endif /* !BoAtomicRetOr16 */

/*
 * u_int16_t BoAtomicFetchXor16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchXor16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicFetchXor16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicFetchXor16

#  define BoAtomicFetchXor16(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 16, u_int16_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 16, u_int16_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor16_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 16_acq, u_int16_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 16_acq, u_int16_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor16_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 16_rel, u_int16_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 16_rel, u_int16_t, ^, aDst, aSrc);

# endif /* !BoAtomicFetchXor16 */

/*
 * u_int16_t BoAtomicRetXor16(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetXor16_acq(volatile u_int16_t *aDst, u_int16_t aSrc);
 * u_int16_t BoAtomicRetXor16_rel(volatile u_int16_t *aDst, u_int16_t aSrc);
 */

# ifndef BoAtomicRetXor16

#  define BoAtomicRetXor16(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 16, u_int16_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 16, u_int16_t, ^, aDst, aSrc);

#  define BoAtomicRetXor16_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 16_acq, u_int16_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 16_acq, u_int16_t, ^, aDst, aSrc);

#  define BoAtomicRetXor16_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 16_rel, u_int16_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 16_rel, u_int16_t, ^, aDst, aSrc);

# endif /* !BoAtomicRetXor16 */

# endif /* BoAtomicCompareStore16 */

/* ----------------------------------------------------------------------------
 * 32-bit operations
 */

/*
 * u_int32_t BoAtomicLoad32_acq(volatile u_int32_t *aPtr);
 * void BoAtomicStore32_rel(volatile u_int32_t *aPtr);
 */

# ifndef BoAtomicLoad32_acq
#  define BoAtomicLoad32_acq(aPtr) \
	_im_BoAtomicLoad32_acq((volatile u_int32_t *)(aPtr))
__myc_extern_inline__ u_int32_t
_im_BoAtomicLoad32_acq(volatile u_int32_t *aPtr)
{
	u_int32_t retval = *aPtr;
	BoAtomicReadMemoryBarrier();
	return (retval);
}
# endif /* !BoAtomicLoad32_acq */

# ifndef BoAtomicStore32_rel
#  define BoAtomicStore32_rel(aPtr,aValue) \
	_im_BoAtomicStore32_rel((volatile u_int32_t *)(aPtr),(u_int32_t)(aValue))
__myc_extern_inline__ void
_im_BoAtomicStore32_rel(volatile u_int32_t *aPtr, u_int32_t aValue) {
	BoAtomicWriteMemoryBarrier();
	*aPtr = aValue;
	return;
}
# endif /* !BoAtomicStore32_rel */

# ifdef BoAtomicCompareStore32

/*
 * u_int32_t BoAtomicFetchStore32(volatile u_int32_t *aDst, u_int32_t aValue);
 * u_int32_t BoAtomicFetchStore32_acq(volatile u_int32_t *aDst, u_int32_t aValue);
 * u_int32_t BoAtomicFetchStore32_rel(volatile u_int32_t *aDst, u_int32_t aValue);
 */

# ifndef BoAtomicFetchStore32

#  define BoAtomicFetchStore32(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(32,u_int32_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(32,u_int32_t,aDst,aValue);

#  define BoAtomicFetchStore32_acq(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(32_acq,u_int32_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(32_acq,u_int32_t,aDst,aValue);

#  define BoAtomicFetchStore32_rel(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(32_rel,u_int32_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(32_rel,u_int32_t,aDst,aValue);

# endif /* !BoAtomicFetchStore32 */

/*
 * u_int32_t BoAtomicCompareFetchStore32(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 * u_int32_t BoAtomicCompareFetchStore32_acq(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 * u_int32_t BoAtomicCompareFetchStore32_rel(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 */

# ifndef BoAtomicCompareFetchStore32

#  define BoAtomicCompareFetchStore32(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(32,u_int32_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(32,u_int32_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore32_acq(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(32_acq,u_int32_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(32_acq,u_int32_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore32_rel(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(32_rel,u_int32_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(32_rel,u_int32_t,aDst,aComparand,aValue);

# endif /* !BoAtomicCompareFetchStore32 */

/*
 * u_int32_t BoAtomicFetchClear32(volatile u_int32_t *aPtr);
 */
# ifndef BoAtomicFetchClear32
#  define BoAtomicFetchClear32(aPtr) \
	BoAtomicFetchStore32((volatile u_int32_t *)(aPtr),(u_int32_t)(0))
# endif

/*
 * void BoAtomicAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */
# ifndef BoAtomicAdd32
#  define BoAtomicAdd32(aPtr,aValue) ((void) BoAtomicFetchAdd32(aPtr,aValue))
#  define BoAtomicAdd32_acq(aPtr,aValue) ((void) BoAtomicFetchAdd32_acq(aPtr,aValue))
#  define BoAtomicAdd32_rel(aPtr,aValue) ((void) BoAtomicFetchAdd32_rel(aPtr,aValue))
# endif

/*
 * u_int32_t BoAtomicFetchAdd32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchAdd32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchAdd32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicFetchAdd32

#  define BoAtomicFetchAdd32(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 32, u_int32_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 32, u_int32_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd32_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 32_acq, u_int32_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 32_acq, u_int32_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd32_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 32_rel, u_int32_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 32_rel, u_int32_t, +, aDst, aSrc);

# endif /* !BoAtomicFetchAdd32 */

/*
 * u_int32_t BoAtomicRetAdd32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetAdd32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetAdd32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicRetAdd32

#  define BoAtomicRetAdd32(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 32, u_int32_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 32, u_int32_t, +, aDst, aSrc);

#  define BoAtomicRetAdd32_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 32_acq, u_int32_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 32_acq, u_int32_t, +, aDst, aSrc);

#  define BoAtomicRetAdd32_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 32_rel, u_int32_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 32_rel, u_int32_t, +, aDst, aSrc);

# endif /* !BoAtomicRetAdd32 */

/*
 * void BoAtomicSub32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicSub32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicSub32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# ifndef BoAtomicSub32

# define BoAtomicSub32(aPtr,aValue) \
	((void) BoAtomicAdd32(aPtr, ((u_int32_t)(-(int32_t)(aValue)))))

# define BoAtomicSub32_acq(aPtr,aValue) \
	((void) BoAtomicAdd32_acq(aPtr, ((u_int32_t)(-(int32_t)(aValue)))))

# define BoAtomicSub32_rel(aPtr,aValue) \
	((void) BoAtomicAdd32_rel(aPtr, ((u_int32_t)(-(int32_t)(aValue)))))

# endif /* !BoAtomicSub32 */

/*
 * void BoAtomicFetchSub32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicFetchSub32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicFetchSub32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# ifndef BoAtomicFetchSub32

# define BoAtomicFetchSub32(aPtr,aValue) \
	BoAtomicFetchAdd32(aPtr, ((u_int32_t)(-(int32_t)(aValue))))

# define BoAtomicFetchSub32_acq(aPtr,aValue) \
	BoAtomicFetchAdd32_acq(aPtr, ((u_int32_t)(-(int32_t)(aValue))))

# define BoAtomicFetchSub32_rel(aPtr,aValue) \
	BoAtomicFetchAdd32_rel(aPtr, ((u_int32_t)(-(int32_t)(aValue))))

# endif /* !BoAtomicFetchSub32 */

/*
 * void BoAtomicRetSub32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicRetSub32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicRetSub32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# ifndef BoAtomicRetSub32

# define BoAtomicRetSub32(aPtr,aValue) \
	BoAtomicRetAdd32(aPtr, ((u_int32_t)(-(int32_t)(aValue))))

# define BoAtomicRetSub32_acq(aPtr,aValue) \
	BoAtomicRetAdd32_acq(aPtr, ((u_int32_t)(-(int32_t)(aValue))))

# define BoAtomicRetSub32_rel(aPtr,aValue) \
	BoAtomicRetAdd32_rel(aPtr, ((u_int32_t)(-(int32_t)(aValue))))

# endif /* !BoAtomicRetSub32 */

/*
 * u_int32_t BoAtomicInc32(volatile u_int32_t *aPtr);
 * u_int32_t BoAtomicDec32(volatile u_int32_t *aPtr);
 */
# ifndef BoAtomicInc32
#  define BoAtomicInc32(aPtr) BoAtomicRetAdd32(aPtr,1)
# endif
# ifndef BoAtomicDec32
#  define BoAtomicDec32(aPtr) BoAtomicRetSub32(aPtr,1)
# endif

/*
 * u_int32_t BoAtomicFetchAnd32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchAnd32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchAnd32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicFetchAnd32

#  define BoAtomicFetchAnd32(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 32, u_int32_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 32, u_int32_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd32_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 32_acq, u_int32_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 32_acq, u_int32_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd32_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 32_rel, u_int32_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 32_rel, u_int32_t, &, aDst, aSrc);

# endif /* !BoAtomicFetchAnd32 */

/*
 * u_int32_t BoAtomicRetAnd32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetAnd32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetAnd32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicRetAnd32

#  define BoAtomicRetAnd32(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 32, u_int32_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 32, u_int32_t, &, aDst, aSrc);

#  define BoAtomicRetAnd32_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 32_acq, u_int32_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 32_acq, u_int32_t, &, aDst, aSrc);

#  define BoAtomicRetAnd32_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 32_rel, u_int32_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 32_rel, u_int32_t, &, aDst, aSrc);

# endif /* !BoAtomicRetAnd32 */

/*
 * u_int32_t BoAtomicFetchOr32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchOr32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchOr32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicFetchOr32

#  define BoAtomicFetchOr32(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 32, u_int32_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 32, u_int32_t, |, aDst, aSrc);

#  define BoAtomicFetchOr32_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 32_acq, u_int32_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 32_acq, u_int32_t, |, aDst, aSrc);

#  define BoAtomicFetchOr32_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 32_rel, u_int32_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 32_rel, u_int32_t, |, aDst, aSrc);

# endif /* !BoAtomicFetchOr32 */

/*
 * u_int32_t BoAtomicRetOr32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetOr32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetOr32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicRetOr32

#  define BoAtomicRetOr32(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 32, u_int32_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 32, u_int32_t, |, aDst, aSrc);

#  define BoAtomicRetOr32_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 32_acq, u_int32_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 32_acq, u_int32_t, |, aDst, aSrc);

#  define BoAtomicRetOr32_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 32_rel, u_int32_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 32_rel, u_int32_t, |, aDst, aSrc);

# endif /* !BoAtomicRetOr32 */

/*
 * u_int32_t BoAtomicFetchXor32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchXor32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicFetchXor32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicFetchXor32

#  define BoAtomicFetchXor32(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 32, u_int32_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 32, u_int32_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor32_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 32_acq, u_int32_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 32_acq, u_int32_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor32_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 32_rel, u_int32_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 32_rel, u_int32_t, ^, aDst, aSrc);

# endif /* !BoAtomicFetchXor32 */

/*
 * u_int32_t BoAtomicRetXor32(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetXor32_acq(volatile u_int32_t *aDst, u_int32_t aSrc);
 * u_int32_t BoAtomicRetXor32_rel(volatile u_int32_t *aDst, u_int32_t aSrc);
 */

# ifndef BoAtomicRetXor32

#  define BoAtomicRetXor32(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 32, u_int32_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 32, u_int32_t, ^, aDst, aSrc);

#  define BoAtomicRetXor32_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 32_acq, u_int32_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 32_acq, u_int32_t, ^, aDst, aSrc);

#  define BoAtomicRetXor32_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 32_rel, u_int32_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 32_rel, u_int32_t, ^, aDst, aSrc);

# endif /* !BoAtomicRetXor32 */

# endif /* BoAtomicCompareStore32 */

/* ----------------------------------------------------------------------------
 * 64-bit operations
 */

/*
 * u_int64_t BoAtomicLoad64_acq(volatile u_int64_t *aPtr);
 * void BoAtomicStore64_rel(volatile u_int64_t *aPtr);
 */

# ifndef BoAtomicLoad64_acq
#  define BoAtomicLoad64_acq(aPtr) \
	_im_BoAtomicLoad64_acq((volatile u_int64_t *)(aPtr))
__myc_extern_inline__ u_int64_t
_im_BoAtomicLoad64_acq(volatile u_int64_t *aPtr)
{
	u_int64_t retval = *aPtr;
	BoAtomicReadMemoryBarrier();
	return (retval);
}
# endif /* !BoAtomicLoad64_acq */

# ifndef BoAtomicStore64_rel
#  define BoAtomicStore64_rel(aPtr,aValue) \
	_im_BoAtomicStore64_rel((volatile u_int64_t *)(aPtr),(u_int64_t)(aValue))
__myc_extern_inline__ void
_im_BoAtomicStore64_rel(volatile u_int64_t *aPtr, u_int64_t aValue) {
	BoAtomicWriteMemoryBarrier();
	*aPtr = aValue;
	return;
}
# endif /* !BoAtomicStore64_rel */


# if (BO_M_WORD == BO_M_WORD64)

# ifdef BoAtomicCompareStore64

/*
 * u_int64_t BoAtomicFetchStore64(volatile u_int64_t *aDst, u_int64_t aValue);
 * u_int64_t BoAtomicFetchStore64_acq(volatile u_int64_t *aDst, u_int64_t aValue);
 * u_int64_t BoAtomicFetchStore64_rel(volatile u_int64_t *aDst, u_int64_t aValue);
 */

# ifndef BoAtomicFetchStore64

#  define BoAtomicFetchStore64(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(64,u_int64_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(64,u_int64_t,aDst,aValue);

#  define BoAtomicFetchStore64_acq(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(64_acq,u_int64_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(64_acq,u_int64_t,aDst,aValue);

#  define BoAtomicFetchStore64_rel(aDst,aValue) \
	_BO_ATOMIC_FETCHSTORE_CALL(64_rel,u_int64_t,aDst,aValue)
_BO_ATOMIC_FETCHSTORE_IMP(64_rel,u_int64_t,aDst,aValue);

# endif /* !BoAtomicFetchStore64 */

/*
 * u_int64_t BoAtomicCompareFetchStore64(volatile u_int64_t *aDst, u_int64_t aComparand, u_int64_t aValue);
 * u_int64_t BoAtomicCompareFetchStore64_acq(volatile u_int64_t *aDst, u_int64_t aComparand, u_int64_t aValue);
 * u_int64_t BoAtomicCompareFetchStore64_rel(volatile u_int64_t *aDst, u_int64_t aComparand, u_int64_t aValue);
 */

# ifndef BoAtomicCompareFetchStore64

#  define BoAtomicCompareFetchStore64(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(64,u_int64_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(64,u_int64_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore64_acq(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(64_acq,u_int64_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(64_acq,u_int64_t,aDst,aComparand,aValue);

#  define BoAtomicCompareFetchStore64_rel(aDst,aComparand,aValue) \
	_BO_ATOMIC_CMPFETCHSTORE_CALL(64_rel,u_int64_t,aDst,aComparand,aValue)
_BO_ATOMIC_CMPFETCHSTORE_IMP(64_rel,u_int64_t,aDst,aComparand,aValue);

# endif /* !BoAtomicCompareFetchStore64 */

/*
 * u_int64_t BoAtomicFetchClear64(volatile u_int64_t *aPtr);
 */
# ifndef BoAtomicFetchClear64
#  define BoAtomicFetchClear64(aPtr) \
	BoAtomicFetchStore64((volatile u_int64_t *)(aPtr),(u_int64_t)(0))
# endif

/*
 * void BoAtomicAdd64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicAdd64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicAdd64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */
# ifndef BoAtomicAdd64
#  define BoAtomicAdd64(aPtr,aValue) ((void) BoAtomicFetchAdd64(aPtr,aValue))
#  define BoAtomicAdd64_acq(aPtr,aValue) ((void) BoAtomicFetchAdd64_acq(aPtr,aValue))
#  define BoAtomicAdd64_rel(aPtr,aValue) ((void) BoAtomicFetchAdd64_rel(aPtr,aValue))
# endif

/*
 * u_int64_t BoAtomicFetchAdd64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchAdd64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchAdd64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicFetchAdd64

#  define BoAtomicFetchAdd64(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 64, u_int64_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 64, u_int64_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd64_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 64_acq, u_int64_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 64_acq, u_int64_t, +, aDst, aSrc);

#  define BoAtomicFetchAdd64_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Add, 64_rel, u_int64_t, +, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Add, 64_rel, u_int64_t, +, aDst, aSrc);

# endif /* !BoAtomicFetchAdd64 */

/*
 * u_int64_t BoAtomicRetAdd64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetAdd64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetAdd64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicRetAdd64

#  define BoAtomicRetAdd64(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 64, u_int64_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 64, u_int64_t, +, aDst, aSrc);

#  define BoAtomicRetAdd64_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 64_acq, u_int64_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 64_acq, u_int64_t, +, aDst, aSrc);

#  define BoAtomicRetAdd64_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Add, 64_rel, u_int64_t, +, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Add, 64_rel, u_int64_t, +, aDst, aSrc);

# endif /* !BoAtomicRetAdd64 */

/*
 * void BoAtomicSub64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicSub64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicSub64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# ifndef BoAtomicSub64

# define BoAtomicSub64(aPtr,aValue) \
	((void) BoAtomicAdd64(aPtr, ((u_int64_t)(-(int64_t)(aValue)))))

# define BoAtomicSub64_acq(aPtr,aValue) \
	((void) BoAtomicAdd64_acq(aPtr, ((u_int64_t)(-(int64_t)(aValue)))))

# define BoAtomicSub64_rel(aPtr,aValue) \
	((void) BoAtomicAdd64_rel(aPtr, ((u_int64_t)(-(int64_t)(aValue)))))

# endif /* !BoAtomicSub64 */

/*
 * void BoAtomicFetchSub64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicFetchSub64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicFetchSub64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# ifndef BoAtomicFetchSub64

# define BoAtomicFetchSub64(aPtr,aValue) \
	BoAtomicFetchAdd64(aPtr, ((u_int64_t)(-(int64_t)(aValue))))

# define BoAtomicFetchSub64_acq(aPtr,aValue) \
	BoAtomicFetchAdd64_acq(aPtr, ((u_int64_t)(-(int64_t)(aValue))))

# define BoAtomicFetchSub64_rel(aPtr,aValue) \
	BoAtomicFetchAdd64_rel(aPtr, ((u_int64_t)(-(int64_t)(aValue))))

# endif /* !BoAtomicFetchSub64 */

/*
 * void BoAtomicRetSub64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicRetSub64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicRetSub64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# ifndef BoAtomicRetSub64

# define BoAtomicRetSub64(aPtr,aValue) \
	BoAtomicRetAdd64(aPtr, ((u_int64_t)(-(int64_t)(aValue))))

# define BoAtomicRetSub64_acq(aPtr,aValue) \
	BoAtomicRetAdd64_acq(aPtr, ((u_int64_t)(-(int64_t)(aValue))))

# define BoAtomicRetSub64_rel(aPtr,aValue) \
	BoAtomicRetAdd64_rel(aPtr, ((u_int64_t)(-(int64_t)(aValue))))

# endif /* !BoAtomicRetSub64 */

/*
 * u_int64_t BoAtomicInc64(volatile u_int64_t *aPtr);
 * u_int64_t BoAtomicDec64(volatile u_int64_t *aPtr);
 */
# ifndef BoAtomicInc64
#  define BoAtomicInc64(aPtr) BoAtomicRetAdd64(aPtr,1)
# endif
# ifndef BoAtomicDec64
#  define BoAtomicDec64(aPtr) BoAtomicRetSub64(aPtr,1)
# endif

/*
 * u_int64_t BoAtomicFetchAnd64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchAnd64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchAnd64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicFetchAnd64

#  define BoAtomicFetchAnd64(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 64, u_int64_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 64, u_int64_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd64_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 64_acq, u_int64_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 64_acq, u_int64_t, &, aDst, aSrc);

#  define BoAtomicFetchAnd64_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(And, 64_rel, u_int64_t, &, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(And, 64_rel, u_int64_t, &, aDst, aSrc);

# endif /* !BoAtomicFetchAnd64 */

/*
 * u_int64_t BoAtomicRetAnd64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetAnd64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetAnd64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicRetAnd64

#  define BoAtomicRetAnd64(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 64, u_int64_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 64, u_int64_t, &, aDst, aSrc);

#  define BoAtomicRetAnd64_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 64_acq, u_int64_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 64_acq, u_int64_t, &, aDst, aSrc);

#  define BoAtomicRetAnd64_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(And, 64_rel, u_int64_t, &, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(And, 64_rel, u_int64_t, &, aDst, aSrc);

# endif /* !BoAtomicRetAnd64 */

/*
 * u_int64_t BoAtomicFetchOr64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchOr64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchOr64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicFetchOr64

#  define BoAtomicFetchOr64(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 64, u_int64_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 64, u_int64_t, |, aDst, aSrc);

#  define BoAtomicFetchOr64_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 64_acq, u_int64_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 64_acq, u_int64_t, |, aDst, aSrc);

#  define BoAtomicFetchOr64_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Or, 64_rel, u_int64_t, |, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Or, 64_rel, u_int64_t, |, aDst, aSrc);

# endif /* !BoAtomicFetchOr64 */

/*
 * u_int64_t BoAtomicRetOr64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetOr64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetOr64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicRetOr64

#  define BoAtomicRetOr64(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 64, u_int64_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 64, u_int64_t, |, aDst, aSrc);

#  define BoAtomicRetOr64_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 64_acq, u_int64_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 64_acq, u_int64_t, |, aDst, aSrc);

#  define BoAtomicRetOr64_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Or, 64_rel, u_int64_t, |, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Or, 64_rel, u_int64_t, |, aDst, aSrc);

# endif /* !BoAtomicRetOr64 */

/*
 * u_int64_t BoAtomicFetchXor64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchXor64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicFetchXor64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicFetchXor64

#  define BoAtomicFetchXor64(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 64, u_int64_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 64, u_int64_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor64_acq(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 64_acq, u_int64_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 64_acq, u_int64_t, ^, aDst, aSrc);

#  define BoAtomicFetchXor64_rel(aDst,aSrc) \
	_BO_ATOMIC_FETCHBINOP_CALL(Xor, 64_rel, u_int64_t, ^, aDst, aSrc)
_BO_ATOMIC_FETCHBINOP_IMP(Xor, 64_rel, u_int64_t, ^, aDst, aSrc);

# endif /* !BoAtomicFetchXor64 */

/*
 * u_int64_t BoAtomicRetXor64(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetXor64_acq(volatile u_int64_t *aDst, u_int64_t aSrc);
 * u_int64_t BoAtomicRetXor64_rel(volatile u_int64_t *aDst, u_int64_t aSrc);
 */

# ifndef BoAtomicRetXor64

#  define BoAtomicRetXor64(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 64, u_int64_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 64, u_int64_t, ^, aDst, aSrc);

#  define BoAtomicRetXor64_acq(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 64_acq, u_int64_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 64_acq, u_int64_t, ^, aDst, aSrc);

#  define BoAtomicRetXor64_rel(aDst,aSrc) \
	_BO_ATOMIC_RETBINOP_CALL(Xor, 64_rel, u_int64_t, ^, aDst, aSrc)
_BO_ATOMIC_RETBINOP_IMP(Xor, 64_rel, u_int64_t, ^, aDst, aSrc);

# endif /* !BoAtomicRetXor64 */

# endif /* BoAtomicCompareStore64 */
# endif /* (BO_M_WORD == BO_M_WORD64) */

/* ----------------------------------------------------------------------------
 * Bit operations
 */

# if __myc_support(macro_retval)

#  define _BO_ATOMIC_BITTESTANDSET_CALL(_Suff, _Tp, aDst, aBit) \
	__extension__ ({ \
		volatile _Tp *___bittestandset_dst = (volatile _Tp *)(aDst); \
		unsigned ___bittestandset_bit = (unsigned)(aBit); \
		u_int32_t ___bittestandset_bitValue = 1 << (___bittestandset_bit & ((sizeof *___bittestandset_dst) * 8 - 1)); \
		/* return */ (0 != (__myc_xconcat(BoAtomicFetchOr,_Suff)(&___bittestandset_dst[___bittestandset_bit / ((sizeof *___bittestandset_dst) * 8)], ___bittestandset_bitValue) & ___bittestandset_bitValue)); \
	})

#  define _BO_ATOMIC_BITTESTANDSET_IMP(_Suff, _Tp, aDst, aBit) struct __hack

# else /* !__myc_support(macro_retval) */

#  define _BO_ATOMIC_BITTESTANDSET_CALL(_Suff, _Tp, aDst, aBit) \
	__myc_xconcat(_im_BoAtomicBitTestAndSet,_Suff)((volatile _Tp *)(aDst), (unsigned)(aBit))

#  define _BO_ATOMIC_BITTESTANDSET_IMP(_Suff, _Tp, aDst, aBit) \
__myc_extern_inline__ int \
__myc_xconcat(_im_BoAtomicBitTestAndSet,_Suff)(volatile _Tp *aDst, unsigned aBit) \
{ \
    u_int32_t bitValue = 1 << (aBit & ((sizeof *aDst) * 8 - 1)); \
    return (0 != (__myc_xconcat(BoAtomicFetchOr,_Suff)(&aDst[aBit / ((sizeof *aDst) * 8)], bitValue) & bitValue)); \
} \
struct __hack

# endif /* __myc_support(macro_retval) */

# if __myc_support(macro_retval)

#  define _BO_ATOMIC_BITTESTANDCLEAR_CALL(_Suff, _Tp, aDst, aBit) \
	__extension__ ({ \
		volatile _Tp *___bittestandclr_dst = (volatile _Tp *)(aDst); \
		unsigned ___bittestandclr_bit = (unsigned)(aBit); \
		u_int32_t ___bittestandclr_bitValue = 1 << (___bittestandclr_bit & ((sizeof *___bittestandclr_dst) * 8 - 1)); \
		/* return */ (0 != (__myc_xconcat(BoAtomicFetchAnd,_Suff)(&___bittestandclr_dst[___bittestandclr_bit / ((sizeof *___bittestandclr_dst) * 8)], ~___bittestandclr_bitValue) & ___bittestandclr_bitValue)); \
	})

#  define _BO_ATOMIC_BITTESTANDCLEAR_IMP(_Suff, _Tp, aDst, aBit) struct __hack

# else /* !__myc_support(macro_retval) */

#  define _BO_ATOMIC_BITTESTANDCLEAR_CALL(_Suff, _Tp, aDst, aBit) \
	__myc_xconcat(_im_BoAtomicBitTestAndClear,_Suff)((volatile _Tp *)(aDst), (unsigned)(aBit))

#  define _BO_ATOMIC_BITTESTANDCLEAR_IMP(_Suff, _Tp, aDst, aBit) \
__myc_extern_inline__ int \
__myc_xconcat(_im_BoAtomicBitTestAndClear,_Suff)(volatile _Tp *aDst, unsigned aBit) \
{ \
    u_int32_t bitValue = 1 << (aBit & ((sizeof *aDst) * 8 - 1)); \
    return (0 != (__myc_xconcat(BoAtomicFetchAnd,_Suff)(&aDst[aBit / ((sizeof *aDst) * 8)], ~bitValue) & bitValue)); \
} \
struct __hack

# endif /* __myc_support(macro_retval) */

/*
 * int BoAtomicBitTestAndSet32(volatile u_int32_t *aDst, unsigned aBit);
 * int BoAtomicBitTestAndClear32(volatile u_int32_t *aDst, unsigned aBit);
 */

# ifndef BoAtomicBitTestAndSet32

#  define BoAtomicBitTestAndSet32(aDst,aBit) \
	_BO_ATOMIC_BITTESTANDSET_CALL(32, u_int32_t, aDst, aBit)
_BO_ATOMIC_BITTESTANDSET_IMP(32, u_int32_t, aDst, aBit);

# endif /* !BoAtomicBitTestAndSet32 */

# ifndef BoAtomicBitTestAndClear32

#  define BoAtomicBitTestAndClear32(aDst,aBit) \
	_BO_ATOMIC_BITTESTANDCLEAR_CALL(32, u_int32_t, aDst, aBit)
_BO_ATOMIC_BITTESTANDCLEAR_IMP(32, u_int32_t, aDst, aBit);

# endif /* !BoAtomicBitTestAndClear32 */

# if (BO_M_WORD == BO_M_WORD64)

/*
 * int BoAtomicBitTestAndSet64(volatile u_int64_t *aDst, unsigned aBit);
 * int BoAtomicBitTestAndClear64(volatile u_int64_t *aDst, unsigned aBit);
 */

# ifndef BoAtomicBitTestAndSet64

#  define BoAtomicBitTestAndSet64(aDst,aBit) \
	_BO_ATOMIC_BITTESTANDSET_CALL(64, u_int64_t, aDst, aBit)
_BO_ATOMIC_BITTESTANDSET_IMP(64, u_int64_t, aDst, aBit);

# endif /* !BoAtomicBitTestAndSet64 */

# ifndef BoAtomicBitTestAndClear64

#  define BoAtomicBitTestAndClear64(aDst,aBit) \
	_BO_ATOMIC_BITTESTANDCLEAR_CALL(64, u_int64_t, aDst, aBit)
_BO_ATOMIC_BITTESTANDCLEAR_IMP(64, u_int64_t, aDst, aBit);

# endif /* !BoAtomicBitTestAndClear64 */

# endif /* (BO_M_WORD == BO_M_WORD64) */

/* ----------------------------------------------------------------------------
 * compat @{
 */
# ifndef BoAtomicIncrement32
#  ifdef BoAtomicInc32
#   define BoAtomicIncrement32 BoAtomicInc32
#  endif
# endif
# ifndef BoAtomicDecrement32
#  ifdef BoAtomicDec32
#   define BoAtomicDecrement32 BoAtomicDec32
#  endif
# endif
/* @} */

/******************************************************************************
 * %% BeginSection: definitions
 */

# if (BO_M_WORD == BO_M_WORD32)
#  define BoAtomicInc BoAtomicInc32
#  define BoAtomicDec BoAtomicDec32
# else
# if (BO_M_WORD == BO_M_WORD64)
#  define BoAtomicInc BoAtomicInc64
#  define BoAtomicDec BoAtomicDec64
# else
#  error Unsupported machine architecture!!!
# endif /* (BO_M_WORD == BO_M_WORD64) */
# endif /* (BO_M_WORD == BO_M_WORD32) */

#endif /* !__BO_SYS_ATOMIC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
