/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Byteorder.h	(N.Panasiuk)
 * include/Bo/sys/Byteorder.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_BYTEORDER__
# define __BO_SYS_BYTEORDER__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -

/*
Swapping examples:
- byte swapping  (Bswap): 12345678 -> 87654321
- word swapping  (Wswap): 12345678 -> 78563412
- dword swapping (Dswap): 12345678 -> 56781234
 */

/*
 * Macros for swapping constant values in the preprocessing stage
 */

# define BoBswapi16c(x) \
	((u_int16_t)((((u_int16_t)(x) & 0xff00) >> 8) | \
	            (((u_int16_t)(x) & 0x00ff) << 8)))

# define BoBswapi32c(x) \
	((u_int32_t)((((u_int32_t)(x) & 0xff000000) >> 24) | \
	            (((u_int32_t)(x) & 0x00ff0000) >>  8) |  \
	            (((u_int32_t)(x) & 0x0000ff00) <<  8) |  \
	            (((u_int32_t)(x) & 0x000000ff) << 24)))

# define BoWswapi32c(x) \
	((u_int32_t)((((u_int32_t)(x) & 0xffff0000) >> 16) | \
	            (((u_int32_t)(x) & 0x0000ffff) << 16)))

# define BoBswapi64c(x) \
	((u_int64_t)((((u_int64_t)(x) & 0xff00000000000000ULL) >> 56) | \
	            (((u_int64_t)(x) & 0x00ff000000000000ULL) >> 40) |  \
	            (((u_int64_t)(x) & 0x0000ff0000000000ULL) >> 24) |  \
	            (((u_int64_t)(x) & 0x000000ff00000000ULL) >>  8) |  \
	            (((u_int64_t)(x) & 0x00000000ff000000ULL) <<  8) |  \
	            (((u_int64_t)(x) & 0x0000000000ff0000ULL) << 24) |  \
	            (((u_int64_t)(x) & 0x000000000000ff00ULL) << 40) |  \
	            (((u_int64_t)(x) & 0x00000000000000ffULL) << 56)))

# define BoWswapi64c(x) \
	((u_int64_t)((((u_int64_t)(x) & 0xffff000000000000ULL) >> 48) | \
	            (((u_int64_t)(x) & 0x0000ffff00000000ULL) >> 16) |  \
	            (((u_int64_t)(x) & 0x00000000ffff0000ULL) << 16) |  \
	            (((u_int64_t)(x) & 0x000000000000ffffULL) << 48)))

# define BoDswapi64c(x) \
	((u_int64_t)((((u_int64_t)(x) & 0xffffffff00000000ULL) >> 32) | \
	            (((u_int64_t)(x) & 0x000000000fffffffULL) << 32)))

/*
 * Functions for swapping values in the runtime
 */

/*
// Generic byte/word/dword swapping functions
u_int16_t BoBswapi16(u_int16_t aData);
u_int32_t BoBswapi32(u_int32_t aData);
u_int32_t BoWswapi32(u_int32_t aData);
u_int64_t BoBswapi64(u_int64_t aData);
u_int64_t BoWswapi64(u_int64_t aData);
u_int64_t BoDswapi64(u_int64_t aData);

// Functions for byte/word/dword reversed loads (read)
u_int16_t BoLoadi16bsw(const volatile void *aBase, u_intptr_t aByteOffset);
u_int32_t BoLoadi32bsw(const volatile void *aBase, u_intptr_t aByteOffset);
u_int32_t BoLoadi32wsw(const volatile void *aBase, u_intptr_t aByteOffset);
u_int64_t BoLoadi64bsw(const volatile void *aBase, u_intptr_t aByteOffset);
u_int64_t BoLoadi64wsw(const volatile void *aBase, u_intptr_t aByteOffset);
u_int64_t BoLoadi64dsw(const volatile void *aBase, u_intptr_t aByteOffset);

// Functions for byte/word/dword reversed stores (write)
void BoStorei16bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int16_t aData);
void BoStorei32bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int32_t aData);
void BoStorei32wsw(volatile void *aBase, u_intptr_t aByteOffset, u_int32_t aData);
void BoStorei64bsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData);
void BoStorei64wsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData);
void BoStorei64dsw(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData);
 */
# include _BO_COMPILER_HEADER(_Byteorder.h)

# pragma mark -

# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)

/* 
 * Functions for loading big endian to host endianess
 */

#  define BoLoadi16be(aBase, aByteOffset) _BoLoadi16(aBase, aByteOffset)
#  define BoLoadi32be(aBase, aByteOffset) _BoLoadi32(aBase, aByteOffset)
#  define BoLoadi64be(aBase, aByteOffset) _BoLoadi64(aBase, aByteOffset)

/*
 * Functions for storing host endianess to big endian
 */

#  define BoStorei16be(aBase, aByteOffset, aData) _BoStorei16(aBase, aByteOffset, aData)
#  define BoStorei32be(aBase, aByteOffset, aData) _BoStorei32(aBase, aByteOffset, aData)
#  define BoStorei64be(aBase, aByteOffset, aData) _BoStorei64(aBase, aByteOffset, aData)

/*
 * Functions for loading little endian to host endianess
 */

#  define BoLoadi16le(aBase, aByteOffset) BoLoadi16bsw(aBase, aByteOffset)
#  define BoLoadi32le(aBase, aByteOffset) BoLoadi32bsw(aBase, aByteOffset)
#  define BoLoadi64le(aBase, aByteOffset) BoLoadi64bsw(aBase, aByteOffset)

/*
 * Functions for storing host endianess to little endian
 */

#  define BoStorei16le(aBase, aByteOffset, aData) BoStorei16bsw(aBase, aByteOffset, aData)
#  define BoStorei32le(aBase, aByteOffset, aData) BoStorei32bsw(aBase, aByteOffset, aData)
#  define BoStorei64le(aBase, aByteOffset, aData) BoStorei64bsw(aBase, aByteOffset, aData)

/*
 * Host endianess to big endian byte swapping macros for constants
 */

#  define BoHtobi16c(x) (x)
#  define BoHtobi32c(x) (x)
#  define BoHtobi64c(x) (x)

/*
 * Generic host endianess to big endian byte swapping functions
 */

#  define BoHtobi16(x) ((u_int16_t)(x))
#  define BoHtobi32(x) ((u_int32_t)(x))
#  define BoHtobi64(x) ((u_int64_t)(x))

/*
 * Big endian to host endianess byte swapping macros for constants
 */

#  define BoBtohi16c(x) (x)
#  define BoBtohi32c(x) (x)
#  define BoBtohi64c(x) (x)

/*
 * Generic big endian to host endianess byte swapping functions
 */

#  define BoBtohi16(x) ((u_int16_t)(x))
#  define BoBtohi32(x) ((u_int32_t)(x))
#  define BoBtohi64(x) ((u_int64_t)(x))

/*
 * Host endianess to little endian byte swapping macros for constants
 */

#  define BoHtoli16c(x) BoBswapi16c(x)
#  define BoHtoli32c(x) BoBswapi32c(x) 
#  define BoHtoli64c(x) BoBswapi64c(x) 

/*
 * Generic host endianess to little endian byte swapping functions
 */

#  define BoHtoli16(x) BoBswapi16(x)
#  define BoHtoli32(x) BoBswapi32(x)
#  define BoHtoli64(x) BoBswapi64(x)

/*
 * Little endian to host endianess byte swapping macros for constants
 */
   
#  define BoLtohi16c(x) BoBswapi16c(x)
#  define BoLtohi32c(x) BoBswapi32c(x)
#  define BoLtohi64c(x) BoBswapi64c(x)

/*
 * Generic little endian to host endianess byte swapping functions
 */

#  define BoLtohi16(x) BoBswapi16(x)
#  define BoLtohi32(x) BoBswapi32(x)
#  define BoLtohi64(x) BoBswapi64(x)

# else
# if (BO_BYTE_ORDER == LITTLE_BYTE_ORDER)

/*
 * Functions for loading big endian to host endianess
 */

#  define BoLoadi16be(aBase, aByteOffset) BoLoadi16bsw(aBase, aByteOffset)
#  define BoLoadi32be(aBase, aByteOffset) BoLoadi32bsw(aBase, aByteOffset)
#  define BoLoadi64be(aBase, aByteOffset) BoLoadi64bsw(aBase, aByteOffset)

/*
 * Functions for storing host endianess to big endian
 */

#  define BoStorei16be(aBase, aByteOffset, aData) BoStorei16bsw(aBase, aByteOffset, aData)
#  define BoStorei32be(aBase, aByteOffset, aData) BoStorei32bsw(aBase, aByteOffset, aData)
#  define BoStorei64be(aBase, aByteOffset, aData) BoStorei64bsw(aBase, aByteOffset, aData)

/*
 * Functions for loading little endian to host endianess
 */

#  define BoLoadi16le(aBase, aByteOffset) _BoLoadi16(aBase, aByteOffset)
#  define BoLoadi32le(aBase, aByteOffset) _BoLoadi32(aBase, aByteOffset)
#  define BoLoadi64le(aBase, aByteOffset) _BoLoadi64(aBase, aByteOffset)

/*
 * Functions for storing host endianess to little endian
 */

#  define BoStorei16le(aBase, aByteOffset, aData) _BoStorei16(aBase, aByteOffset, aData)
#  define BoStorei32le(aBase, aByteOffset, aData) _BoStorei32(aBase, aByteOffset, aData)
#  define BoStorei64le(aBase, aByteOffset, aData) _BoStorei64(aBase, aByteOffset, aData)

/*
 * Host endianess to big endian byte swapping macros for constants
 */

#  define BoHtobi16c(x) BoBswapi16c(x)
#  define BoHtobi32c(x) BoBswapi32c(x)
#  define BoHtobi64c(x) BoBswapi64c(x)

/*
 * Generic host endianess to big endian byte swapping functions
 */

#  define BoHtobi16(x) BoBswapi16(x)
#  define BoHtobi32(x) BoBswapi32(x)
#  define BoHtobi64(x) BoBswapi64(x)

/*
 * Big endian to host endianess byte swapping macros for constants
 */

#  define BoBtohi16c(x) BoBswapi16c(x)
#  define BoBtohi32c(x) BoBswapi32c(x)
#  define BoBtohi64c(x) BoBswapi64c(x)

/*
 * Generic big endian to host endianess byte swapping functions
 */

#  define BoBtohi16(x) BoBswapi16(x)
#  define BoBtohi32(x) BoBswapi32(x)
#  define BoBtohi64(x) BoBswapi64(x)

/*
 * Host endianess to little endian byte swapping macros for constants
 */

#  define BoHtoli16c(x) (x)
#  define BoHtoli32c(x) (x)
#  define BoHtoli64c(x) (x) 

/*
 * Generic host endianess to little endian byte swapping functions
 */

#  define BoHtoli16(x) ((u_int16_t)(x))
#  define BoHtoli32(x) ((u_int32_t)(x))
#  define BoHtoli64(x) ((u_int64_t)(x))

/*
 * Little endian to host endianess byte swapping macros for constants
 */

#  define BoLtohi16c(x) (x)
#  define BoLtohi32c(x) (x)
#  define BoLtohi64c(x) (x)

/*
 * Generic little endian to host endianess byte swapping functions
 */

#  define BoLtohi16(x) ((u_int16_t)(x))
#  define BoLtohi32(x) ((u_int32_t)(x))
#  define BoLtohi64(x) ((u_int64_t)(x))

# else /* (BO_BYTE_ORDER != LITTLE_BYTE_ORDER) */
#  error Unknown endianess!!!
# endif /* (BO_BYTE_ORDER == LITTLE_BYTE_ORDER) */
# endif /* (BO_BYTE_ORDER == BIG_BYTE_ORDER) */

/*
 * Functions for loading net endian to host endianess
 */

# define BoLoadi16ne(aBase, aByteOffset) BoLoadi16be(aBase, aByteOffset)
# define BoLoadi32ne(aBase, aByteOffset) BoLoadi32be(aBase, aByteOffset)
# define BoLoadi64ne(aBase, aByteOffset) BoLoadi64be(aBase, aByteOffset)

/*
 * Functions for storing host endianess to net endian
 */

# define BoStorei16ne(aBase, aByteOffset, aData) BoStorei16be(aBase, aByteOffset, aData)
# define BoStorei32ne(aBase, aByteOffset, aData) BoStorei32be(aBase, aByteOffset, aData)
# define BoStorei64ne(aBase, aByteOffset, aData) BoStorei64be(aBase, aByteOffset, aData)

/*
 * Host endianess to net endian byte swapping macros for constants
 */

# define BoHtoni16c(x) BoHtobi16c(x)
# define BoHtoni32c(x) BoHtobi32c(x)
# define BoHtoni64c(x) BoHtobi64c(x)

/*
 * Generic host endianess to net endian byte swapping functions
 */

# define BoHtoni16(x) BoHtobi16(x)
# define BoHtoni32(x) BoHtobi32(x)
# define BoHtoni64(x) BoHtobi64(x)

/*
 * Net endian to host endianess byte swapping macros for constants
 */

# define BoNtohi16c(x) BoBtohi16c(x)
# define BoNtohi32c(x) BoBtohi32c(x)
# define BoNtohi64c(x) BoBtohi64c(x)

/*
 * Generic net endian to host endianess byte swapping functions
 */

# define BoNtohi16(x) BoBtohi16(x)
# define BoNtohi32(x) BoBtohi32(x)
# define BoNtohi64(x) BoBtohi64(x)

# pragma mark -

/* NOT FOR YOUR BUSINESS */
/* @{ */

/*
 * Functions for loading native endian values
 */

# if 0
__myc_extern_inline__ u_int16_t
_BoLoadi16(volatile void const *aBase, u_intptr_t aByteOffset)
{
	return *(volatile u_int16_t *)((u_intptr_t)aBase + aByteOffset);
}
# else
#  define _BoLoadi16(aBase, aByteOffset) \
	(*(volatile u_int16_t *)((u_intptr_t)(volatile void const *)(aBase) + (u_intptr_t)(aByteOffset)))
# endif

# if 0
__myc_extern_inline__ u_int32_t
_BoLoadi32(volatile void const *aBase, u_intptr_t aByteOffset)
{
	return *(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset);
}
# else
#  define _BoLoadi32(aBase, aByteOffset) \
	(*(volatile u_int32_t *)((u_intptr_t)(volatile void const *)(aBase) + (u_intptr_t)(aByteOffset)))
# endif

# if 0
__myc_extern_inline__ u_int64_t
_BoLoadi64(volatile void const *aBase, u_intptr_t aByteOffset)
{
	return *(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset);
}
# else
#  define _BoLoadi64(aBase, aByteOffset) \
	(*(volatile u_int64_t *)((u_intptr_t)(volatile void const *)(aBase) + (u_intptr_t)(aByteOffset)))
# endif

/*
 * Functions for storing native endian values
 */

# if 0
__myc_extern_inline__ void
_BoStorei16(volatile void *aBase, u_intptr_t aByteOffset, u_int16_t aData)
{
	*(volatile u_int16_t *)((u_intptr_t)aBase + aByteOffset) = aData;
}
# else
#  define _BoStorei16(aBase, aByteOffset, aData) \
	((void) (*(volatile u_int16_t *)((u_intptr_t)(volatile void *)(aBase) + (aByteOffset)) = (u_int16_t)((int)(aData) & 0xFFFF)))
# endif

# if 0
__myc_extern_inline__ void
_BoStorei32(volatile void *aBase, u_intptr_t aByteOffset, u_int32_t aData)
{
	*(volatile u_int32_t *)((u_intptr_t)aBase + aByteOffset) = aData;
}
# else
#  define _BoStorei32(aBase, aByteOffset, aData) \
	((void) (*(volatile u_int32_t *)((u_intptr_t)(volatile void *)(aBase) + (aByteOffset)) = (u_int32_t)(aData)))
# endif

# if 0
__myc_extern_inline__ void
_BoStorei64(volatile void *aBase, u_intptr_t aByteOffset, u_int64_t aData)
{
	*(volatile u_int64_t *)((u_intptr_t)aBase + aByteOffset) = aData;
}
# else
#  define _BoStorei64(aBase, aByteOffset, aData) \
	((void) (*(volatile u_int64_t *)((u_intptr_t)(volatile void *)(aBase) + (aByteOffset)) = (u_int64_t)(aData)))
# endif

/* @} */

/*
 * compat @{
 */

# define BoBswapConstInt16(x) BoBswapi16c(x)
# define BoBswapConstInt32(x) BoBswapi32c(x)
# define BoWswapConstInt32(x) BoWswapi32c(x)
# define BoBswapConstInt64(x) BoBswapi64c(x)
# define BoWswapConstInt64(x) BoWswapi64c(x)
# define BoDswapConstInt64(x) BoDswapi64c(x)

# define BoBswapInt16(x) BoBswapi16(x)
# define BoBswapInt32(x) BoBswapi32(x)
# define BoWswapInt32(x) BoWswapi32(x)
# define BoBswapInt64(x) BoBswapi64(x)
# define BoWswapInt64(x) BoWswapi64(x)
# define BoDswapInt64(x) BoDswapi64(x)


# define BoReadBswapInt16(aBase, aByteOffset) BoLoadi16bsw(aBase, aByteOffset)
# define BoReadBswapInt32(aBase, aByteOffset) BoLoadi32bsw(aBase, aByteOffset)
# define BoReadWswapInt32(aBase, aByteOffset) BoLoadi32wsw(aBase, aByteOffset)
# define BoReadBswapInt64(aBase, aByteOffset) BoLoadi64bsw(aBase, aByteOffset)
# define BoReadWswapInt64(aBase, aByteOffset) BoLoadi64wsw(aBase, aByteOffset)
# define BoReadDswapInt64(aBase, aByteOffset) BoLoadi64dsw(aBase, aByteOffset)

# define BoWriteBswapInt16(aBase, aByteOffset, aData) BoStorei16bsw(aBase, aByteOffset, aData)
# define BoWriteBswapInt32(aBase, aByteOffset, aData) BoStorei32bsw(aBase, aByteOffset, aData)
# define BoWriteWswapInt32(aBase, aByteOffset, aData) BoStorei32wsw(aBase, aByteOffset, aData)
# define BoWriteBswapInt64(aBase, aByteOffset, aData) BoStorei64bsw(aBase, aByteOffset, aData)
# define BoWriteWswapInt64(aBase, aByteOffset, aData) BoStorei64wsw(aBase, aByteOffset, aData)
# define BoWriteDswapInt64(aBase, aByteOffset, aData) BoStorei64dsw(aBase, aByteOffset, aData)


#  define BoReadBigInt16(aBase, aByteOffset) BoLoadi16be(aBase, aByteOffset)
#  define BoReadBigInt32(aBase, aByteOffset) BoLoadi32be(aBase, aByteOffset)
#  define BoReadBigInt64(aBase, aByteOffset) BoLoadi64be(aBase, aByteOffset)

#  define BoWriteBigInt16(aBase, aByteOffset, aData) BoStorei16be(aBase, aByteOffset, aData)
#  define BoWriteBigInt32(aBase, aByteOffset, aData) BoStorei32be(aBase, aByteOffset, aData)
#  define BoWriteBigInt64(aBase, aByteOffset, aData) BoStorei64be(aBase, aByteOffset, aData)


#  define BoReadLittleInt16(aBase, aByteOffset) BoLoadi16le(aBase, aByteOffset)
#  define BoReadLittleInt32(aBase, aByteOffset) BoLoadi32le(aBase, aByteOffset)
#  define BoReadLittleInt64(aBase, aByteOffset) BoLoadi64le(aBase, aByteOffset)

#  define BoWriteLittleInt16(aBase, aByteOffset, aData) BoStorei16le(aBase, aByteOffset, aData)
#  define BoWriteLittleInt32(aBase, aByteOffset, aData) BoStorei32le(aBase, aByteOffset, aData)
#  define BoWriteLittleInt64(aBase, aByteOffset, aData) BoStorei64le(aBase, aByteOffset, aData)


#  define BoHostToBigConstInt16(x) BoHtobi16c(x)
#  define BoHostToBigConstInt32(x) BoHtobi32c(x)
#  define BoHostToBigConstInt64(x) BoHtobi64c(x)

#  define BoHostToBigInt16(x) BoHtobi16(x)
#  define BoHostToBigInt32(x) BoHtobi32(x)
#  define BoHostToBigInt64(x) BoHtobi64(x)

#  define BoBigToHostConstInt16(x) BoBtohi16c(x)
#  define BoBigToHostConstInt32(x) BoBtohi32c(x)
#  define BoBigToHostConstInt64(x) BoBtohi64c(x)

#  define BoBigToHostInt16(x) BoBtohi16(x)
#  define BoBigToHostInt32(x) BoBtohi32(x)
#  define BoBigToHostInt64(x) BoBtohi64(x)


#  define BoHostToLittleConstInt16(x) BoHtoli16c(x)
#  define BoHostToLittleConstInt32(x) BoHtoli32c(x) 
#  define BoHostToLittleConstInt64(x) BoHtoli64c(x) 

#  define BoHostToLittleInt16(x) BoHtoli16(x)
#  define BoHostToLittleInt32(x) BoHtoli32(x)
#  define BoHostToLittleInt64(x) BoHtoli64(x)
   
#  define BoLittleToHostConstInt16(x) BoLtohi16c(x)
#  define BoLittleToHostConstInt32(x) BoLtohi32c(x)
#  define BoLittleToHostConstInt64(x) BoLtohi64c(x)

#  define BoLittleToHostInt16(x) BoLtohi16(x)
#  define BoLittleToHostInt32(x) BoLtohi32(x)
#  define BoLittleToHostInt64(x) BoLtohi64(x)


# define BoReadNetInt16(aBase, aByteOffset) BoLoadi16ne(aBase, aByteOffset)
# define BoReadNetInt32(aBase, aByteOffset) BoLoadi32ne(aBase, aByteOffset)
# define BoReadNetInt64(aBase, aByteOffset) BoLoadi64ne(aBase, aByteOffset)

# define BoWriteNetInt16(aBase, aByteOffset, aData) BoStorei16ne(aBase, aByteOffset, aData)
# define BoWriteNetInt32(aBase, aByteOffset, aData) BoStorei32ne(aBase, aByteOffset, aData)
# define BoWriteNetInt64(aBase, aByteOffset, aData) BoStorei64ne(aBase, aByteOffset, aData)

# define BoHostToNetConstInt16(x) BoHtoni16c(x)
# define BoHostToNetConstInt32(x) BoHtoni32c(x)
# define BoHostToNetConstInt64(x) BoHtoni64c(x)

# define BoHostToNetInt16(x) BoHtoni16(x)
# define BoHostToNetInt32(x) BoHtoni32(x)
# define BoHostToNetInt64(x) BoHtoni64(x)

# define BoNetToHostConstInt16(x) BoNtohi16c(x)
# define BoNetToHostConstInt32(x) BoNtohi32c(x)
# define BoNetToHostConstInt64(x) BoNtohi64c(x)

# define BoNetToHostInt16(x) BoNtohi16(x)
# define BoNetToHostInt32(x) BoNtohi32(x)
# define BoNetToHostInt64(x) BoNtohi64(x)
/* @} */

#endif /* !__BO_SYS_BYTEORDER__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
