/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Dir.h	(V.Sokur)
 * include/Bo/sys/Dir.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_DIR__
# define __BO_SYS_DIR__

/******************************************************************************
 * %% BeginSection: includes
 */
# pragma mark -

# include <Bo/sys/Types.h>

/* unix */
# if __myc_exists(unistd,h)
#  include __MYC_LIBC_HEADER(unistd.h)
# endif
	/* unlink, rmdir, chdir, getcwd, getwd, ... */
# if __myc_exists(sys7stat,h)
#  include __MYC_LIBC_HEADER(sys/stat.h)
# endif
# include __MYC_LIBC_HEADER(stdio.h)
	/* rename, remove, ... */

/* windows */
# if __myc_exists(direct,h)
#  include __MYC_LIBC_HEADER(direct.h)
# endif
	/* rmdir, chdir, getcwd, ... */
# if __myc_exists(io,h)
#  include __MYC_LIBC_HEADER(io.h)
# endif
	/* unlink, rename, remove, ... */

/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoRemove
 * remove
 * ISO/IEC 9899:1990 (ISO C90)
 * X/Open Portability Guide Issue 4 (XPG4.2)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_DirNotEmpty, BoK_FileError_Busy,
		           BoK_FileError_ReadOnlyFS, BoK_IoError_NoInformation);
	];
))
BoRemove(
# if __myc_require(prototypes)
	char const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int remove(char const *aPath);
 */
# ifndef HAS_REMOVE
#  ifdef HAS__REMOVE
#   define remove(aPath) _remove(aPath)
#  endif /* HAS__REMOVE*/
# endif /* !HAS_REMOVE */

/* ----------------------------------------------------------------------------
 * BoWremove
 * wremove
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_DirNotEmpty, BoK_FileError_Busy,
		           BoK_FileError_ReadOnlyFS, BoK_IoError_NoInformation);
	];
))
BoWremove(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int wremove(wchar_t const *aPath);
 */
# ifndef HAS_WREMOVE
#  ifdef HAS__WREMOVE
#   define wremove(aPath) _wremove(aPath)
#  else
#   define NEED_WREMOVE_IMPL
#  endif /* !HAS__WREMOVE */
# endif /* !HAS_WREMOVE */

# ifdef NEED_WREMOVE_IMPL
__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
wremove(
# if __myc_require(prototypes)
	wchar_t const *aPath
# endif
	);
# endif /* NEED_WREMOVE_IMPL */

/* ----------------------------------------------------------------------------
 * BoRename
 * rename
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_IsDir,
		           BoK_FileError_DirNotEmpty, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_CrossDeviceLink, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace, BoK_IoError_NoInformation,
		           BoK_SysError_DiskQuota);
	];
))
BoRename(
# if __myc_require(prototypes)
	char const *anOldFilename,
	char const *aNewFilename,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int rename(char const *anOldFilename, char const *aNewFilename);
 */
# ifndef HAS_RENAME
#  ifdef HAS__RENAME
#   define rename(anOldFilename,aNewFilename) _rename(anOldFilename,aNewFilename)
#  endif /* HAS__RENAME */
# endif /* !HAS_RENAME */

/* ----------------------------------------------------------------------------
 * BoWrename
 * wrename
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_IsDir,
		           BoK_FileError_DirNotEmpty, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_CrossDeviceLink, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace, BoK_IoError_NoInformation,
		           BoK_SysError_DiskQuota);
	];
))
BoWrename(
# if __myc_require(prototypes)
	wchar_t const *anOldFilename,
	wchar_t const *aNewFilename,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int wrename(wchar_t const *anOldFilename, wchar_t const *aNewFilename);
 */
# ifndef HAS_WRENAME
#  ifdef HAS__WRENAME
#   define wrename(anOldFilename,aNewFilename) _wrename(anOldFilename,aNewFilename)
#  else
#   define NEED_WRENAME_IMPL
#  endif /* !HAS__WRENAME */
# endif /* !HAS_WRENAME */

# ifdef NEED_WRENAME_IMPL
__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1,2)))
wrename(
# if __myc_require(prototypes)
	wchar_t const *anOldFilename,
	wchar_t const *aNewFilename
# endif
	);
# endif /* NEED_WRENAME_IMPL */

/* ----------------------------------------------------------------------------
 * BoUnlink
 * unlink
 * Version 6 AT&T UNIX
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_Busy, BoK_FileError_ReadOnlyFS,
		           BoK_IoError_NoInformation);
	];
))
BoUnlink(
# if __myc_require(prototypes)
	char const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int unlink(char const *aPath);
 */
# ifndef HAS_UNLINK
#  ifdef HAS__UNLINK
#   define unlink(aPath) _unlink(aPath)
#  endif /* HAS__UNLINK */
# endif /* !HAS_UNLINK */

/* ----------------------------------------------------------------------------
 * BoWunlink
 * wunlink
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_Busy, BoK_FileError_ReadOnlyFS,
		           BoK_IoError_NoInformation);
	];
))
BoWunlink(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int wunlink(wchar_t const *aPath);
 */
# ifndef HAS_WUNLINK
#  ifdef HAS__WUNLINK
#   define wunlink(aPath) _wunlink(aPath)
#  else
#   define NEED_WUNLINK_IMPL
#  endif /* !HAS__WUNLINK */
# endif /* !HAS_WUNLINK */

# ifdef NEED_WUNLINK_IMPL
__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
wunlink(
# if __myc_require(prototypes)
	wchar_t const *aPath
# endif
	);
# endif /* NEED_WUNLINK_IMPL */

/* ----------------------------------------------------------------------------
 * BoRmdir
 * rmdir
 * BSD4.2
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_DirNotEmpty, BoK_FileError_Busy,
		           BoK_FileError_ReadOnlyFS, BoK_IoError_NoInformation);
	];
))
BoRmdir(
# if __myc_require(prototypes)
	char const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int rmdir(char const *aPath);
 */
# ifndef HAS_RMDIR
#  ifdef HAS__RMDIR
#   define rmdir(aPath) _rmdir(aPath)
#  endif /* HAS__RMDIR */
# endif /* !HAS_RMDIR */

/* ----------------------------------------------------------------------------
 * BoWrmdir
 * wrmdir
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_DirNotEmpty, BoK_FileError_Busy,
		           BoK_FileError_ReadOnlyFS, BoK_IoError_NoInformation);
	];
))
BoWrmdir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int wrmdir(wchar_t const *aPath);
 */
# ifndef HAS_WRMDIR
#  ifdef HAS__WRMDIR
#   define wrmdir(aPath) _wrmdir(aPath)
#  else
#   define NEED_WRMDIR_IMPL
#  endif /* !HAS__WRMDIR */
# endif /* !HAS_WRMDIR */

# ifdef NEED_WRMDIR_IMPL
__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
wrmdir(
# if __myc_require(prototypes)
	wchar_t const *aPath
# endif
	);
# endif /* NEED_WRMDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoChdir
 * chdir
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_FileError_NotDir,
		           BoK_FileError_NameSizeLimit, BoK_FileError_NoSuchEntry,
		           BoK_FileError_Loop, BoK_IoError_NoInformation);
	];
))
BoChdir(
# if __myc_require(prototypes)
	char const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int chdir(char const *aPath);
 */
# ifndef HAS_CHDIR
#  ifdef HAS__CHDIR
#   define chdir(aPath) _chdir(aPath)
#  endif /* HAS__CHDIR */
# endif /* !HAS_CHDIR */

/* ----------------------------------------------------------------------------
 * BoWchdir
 * wchdir
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_FileError_NotDir,
		           BoK_FileError_NameSizeLimit, BoK_FileError_NoSuchEntry,
		           BoK_FileError_Loop, BoK_IoError_NoInformation);
	];
))
BoWchdir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int wchdir(wchar_t const *aPath);
 */
# ifndef HAS_WCHDIR
#  ifdef HAS__WCHDIR
#   define wchdir(aPath) _wchdir(aPath)
#  else
#   define NEED_WCHDIR_IMPL
#  endif /* !HAS__WCHDIR */
# endif /* !HAS_WCHDIR */

# ifdef NEED_WCHDIR_IMPL
__myc_flinkage__(public,libBo) int
__attribute__((nonnull(1)))
wchdir(
# if __myc_require(prototypes)
	wchar_t const *aPath
# endif
	);
# endif /* NEED_WCHDIR_IMPL */

/* ----------------------------------------------------------------------------
 * BoGetcwd
 * getcwd
 * IEEE Std 9945.1-1990 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) char *
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_Overflow, BoK_RacError_AccessViolation,
		           BoK_FileError_NoSuchEntry, BoK_SysError_NoMemory);
	];
))
BoGetcwd(
# if __myc_require(prototypes)
	char *aBuffer,
	size_t aBufsz, /* in characters(bytes) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
char *getcwd(char *aBuffer, size_t aBufsz);
 */
# ifndef HAS_GETCWD
#  ifdef HAS__GETCWD
#   define getcwd(aBuffer,aBufsz) _getcwd(aBuffer,aBufsz)
#  endif /* HAS__GETCWD */
# endif /* !HAS_GETCWD */

/* ----------------------------------------------------------------------------
 * BoWgetcwd
 * wgetcwd
 */
# pragma mark -

__myc_flinkage__(public,libBo) wchar_t *
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_Overflow, BoK_RacError_AccessViolation,
		           BoK_FileError_NoSuchEntry, BoK_SysError_NoMemory);
	];
))
BoWgetcwd(
# if __myc_require(prototypes)
	wchar_t *aBuffer,
	size_t aBufsz, /* in characters(wchar) */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
wchar_t *wgetcwd(wchar_t *aBuffer, size_t aCharc);
 */
# ifndef HAS_WGETCWD
#  ifdef HAS__WGETCWD
#   define wgetcwd(aBuffer,aCharc) _wgetcwd(aBuffer,aCharc)
#  else
#   define NEED_WGETCWD_IMPL
#  endif /* !HAS__WGETCWD */
# endif /* !HAS_WGETCWD */

# ifdef NEED_WGETCWD_IMPL
__myc_flinkage__(public,libBo) wchar_t *
__attribute__((nonnull(1)))
wgetcwd(
# if __myc_require(prototypes)
	wchar_t *aBuffer,
	size_t aCharc
# endif
	);
# endif /* NEED_WGETCWD_IMPL */

__MYC_END_CDECLS

#endif /* !__BO_SYS_DIR__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
