/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Dirent.h	(V.Sokur)
 * include/Bo/sys/Dirent.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_DIRENT__
# define __BO_SYS_DIRENT__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

/* linux */
# ifdef __linux__
#  if __myc_exists(dirent,h)
#   include __MYC_LIBC_HEADER(dirent.h)
#  endif
# endif

# if __myc_exists(sys7dirent,h)
#  include __MYC_LIBC_HEADER(sys/dirent.h)
# endif

/*
# include <Bo/sys/_FTypedefs.h>
 */

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -

# ifndef HAS_DIRENT_STRUC
#  ifdef MAX_PATH
#   if 260 == MAX_PATH
#    define BoL_DIRENT_NAME_MAX 256
#   else
#    define BoL_DIRENT_NAME_MAX (MAX_PATH + 1)
#   endif
#  endif
# endif /* !HAS_DIRENT_STRUC */

# ifdef __DARWIN_MAXNAMLEN
#  define BoL_DIRENT_NAME_MAX (__DARWIN_MAXNAMLEN + 1)
# else
# ifdef MAXNAMLEN
#  define BoL_DIRENT_NAME_MAX (MAXNAMLEN + 1)
# endif /* MAXNAMLEN */
# endif /* __DARWIN_MAXNAMLEN */

/* @seealso Bo/libc/Limits.h::BoL_NAME_MAX */
# ifndef BoL_DIRENT_NAME_MAX
#  define BoL_DIRENT_NAME_MAX 256 /* including the terminating NUL character */
# endif

# ifndef HAS_DIRENT_STRUC

/* @todo: fixme */
#  ifdef _MSC_VER
#   pragma pack(push,4)
#  else
#   pragma pack(4)
#  endif

struct dirent {
	ino_t     d_ino;    /* file number of entry */
	u_int16_t d_reclen; /* length of this record */
	u_int8_t  d_type;   /* file type */
	u_int8_t  d_namlen; /* length of string in .d_name, in bytes */
	char      d_name[BoL_DIRENT_NAME_MAX];
};

#  ifdef _MSC_VER
#   pragma pack(pop)
#  else
#   pragma pack()
#  endif

# endif /* !HAS_DIRENT_STRUC */

# ifndef HAS_WDIRENT

/* @todo: fixme */
#  ifdef _MSC_VER
#   pragma pack(push,4)
#  else
#   pragma pack(4)
#  endif

struct wdirent {
	ino_t     d_ino;    /* file number of entry */
	u_int16_t d_reclen; /* length of this record */
	u_int8_t  d_type;   /* file type */
	u_int8_t  d_namlen; /* length of string in .d_name, in wide-characters */
	wchar_t   d_name[BoL_DIRENT_NAME_MAX];
};

#  ifdef _MSC_VER
#   pragma pack(pop)
#  else
#   pragma pack()
#  endif

# endif /* !HAS_WDIRENT */


/* File types */
# ifndef DT_UNKNOWN
#  define DT_UNKNOWN (0)
# endif
# ifndef DT_FIFO
#  define DT_FIFO (1)
# endif
# ifndef DT_CHR
#  define DT_CHR (2)
# endif
# ifndef DT_DIR
#  define DT_DIR (4)
# endif
# ifndef DT_BLK
#  define DT_BLK (6)
# endif
# ifndef DT_REG
#  define DT_REG (8)
# endif
# ifndef DT_LNK
#  define DT_LNK (10)
# endif
# ifndef DT_SOCK
#  define DT_SOCK (12)
# endif
/*
# ifndef DT_WHT
#  define DT_WHT (14)
# endif
 */

# ifndef D_ISUNKNOWN
#  define D_ISUNKNOWN(aType) (DT_UNKNOWN == (aType))
# endif
# ifndef D_ISFIFO
#  define D_ISFIFO(aType) (DT_FIFO == (aType))
# endif
# ifndef D_ISCHR
#  define D_ISCHR(aType) (DT_CHR == (aType))
# endif
# ifndef D_ISDIR
#  define D_ISDIR(aType) (DT_DIR == (aType))
# endif
# ifndef D_ISBLK
#  define D_ISBLK(aType) (DT_BLK == (aType))
# endif
# ifndef D_ISREG
#  define D_ISREG(aType) (DT_REG == (aType))
# endif
# ifndef D_ISLNK
#  define D_ISLNK(aType) (DT_LNK == (aType))
# endif
# ifndef D_ISSOCK
#  define D_ISSOCK(aType) (DT_SOCK == (aType))
# endif


/* @todo: fixme, windows */
# ifndef IFTODT
#  define IFTODT(aMode) (((aMode) & S_IFMT) >> 12)
# endif
# ifndef DTTOIF
#  define DTTOIF(aDirentype) ((aDirentype) << 12)
# endif


# define DIRENTSIZE(aDirent) \
	(((sizeof *(aDirent)) - (sizeof (aDirent)->d_name)) + \
		((aDirent)->d_namlen + 1) * (sizeof (aDirent)->d_name[0]))

# define DIRENTSIZE_ALIGN4(aDirent) \
	((DIRENTSIZE(aDirent) + 3) & ~3)

#endif /* !__BO_SYS_DIRENT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
