/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Fcntl.h	(V.Sokur)
 * include/Bo/sys/Fcntl.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_FCNTL__
# define __BO_SYS_FCNTL__

/******************************************************************************
 * %% BeginSection: includes
 */
# pragma mark -

# include <Bo/sys/Types.h>

# if __myc_exists(fcntl,h)
#  include __MYC_LIBC_HEADER(fcntl.h)
# endif
# if __myc_exists(sys7fcntl,h)
#  include __MYC_LIBC_HEADER(sys/fcntl.h)
# endif
	/* open, creat, fcntl, flock */
# if __myc_exists(sys7file,h)
#  include __MYC_LIBC_HEADER(sys/file.h)
# endif
	/* flock */
# if __myc_exists(unistd,h)
#  include __MYC_LIBC_HEADER(unistd.h)
# endif
	/* dup, dup2 */

/* windows */
# if __myc_exists(io,h)
#  include __MYC_LIBC_HEADER(io.h)
# endif
	/* open, creat, ... */
# if __myc_exists(share,h)
#  include __MYC_LIBC_HEADER(share.h)
# endif
	/* share-flags */
# if __myc_exists(sys7locking,h)
#  include __MYC_LIBC_HEADER(sys/locking.h)
# endif
	/* lock-flags */

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

/* @todo: do we really need this? */
# ifndef S_IFMMASK
#  include <Bo/sys/Stat.h>
	/* _FModedefs.h, _FTypedefs.h */
# endif
# include <Bo/sys/_FSeekdefs.h>

/*
 * Open flags
 */
# include <Bo/sys/_FOpenflags.h>

/*
 * Commands
 */

# ifndef F_DUPFD
#  define F_DUPFD 0     /* duplicate file descriptor */
# endif
# ifndef F_GETFD
#  define F_GETFD 1     /* get file descriptor flags: CLOSE-ON-EXEC */
# endif
# ifndef F_SETFD
#  define F_SETFD 2     /* set file descriptor flags: CLOSE-ON-EXEC */
# endif
# ifndef F_GETFL
#  define F_GETFL 3     /* get file status flags */
# endif
# ifndef F_SETFL
#  define F_SETFL 4     /* set file status flags */
# endif
# ifndef F_GETOWN
#  define F_GETOWN 5    /* get SIGIO/SIGURG proc/pgrp */
# endif
# ifndef F_SETOWN
#  define F_SETOWN 6    /* set SIGIO/SIGURG proc/pgrp */
# endif
# ifndef F_GETLK
#  define F_GETLK 7     /* get record locking information */
# endif
# ifndef F_SETLK
#  define F_SETLK 8     /* set record locking information */
# endif
# ifndef F_SETLKW
#  define F_SETLKW 9    /* F_SETLK; wait if blocked */
# endif

# ifndef HAS_FLOCK_STRUC
struct flock {
	bo_off_t  l_start;     /* starting offset */
	bo_off_t  l_len;       /* len = 0 means until end of file */
	pid_t     l_pid;       /* lock owner */
	short     l_type;      /* lock type: read/write, etc. */
	short     l_whence;    /* type of l_start */
};
# endif

/*
 The fcntl-lock interface follows the completely stupid semantics of System V and
	IEEE Std 1003.1-1988 (POSIX.1) that require that all locks associated
	with a file for a given process are removed when any file descriptor for
	that file is closed by that process.

 @todo: what about imlementation for windows?
 */

/* Record locking arguments (fcntl(): F_GETLK, F_SETLK, F_SETLKW) */
/* compat @{ */
# ifndef HAS_FLOCK_STRUC
#  define F_RDLCK 1
#  define F_WRLCK 2
#  define F_UNLCK 3
# endif
/* @} */
# define BoK_FCNTL_SHLCK F_RDLCK  /* shared or read lock */
# define BoK_FCNTL_EXLCK F_WRLCK  /* exclusive or write lock */
# define BoK_FCNTL_UNLCK F_UNLCK  /* unlock */

/* File locking operations(flags) (flock()) */
/* compat @{ */
# ifndef HAS_FLOCK
#  define LOCK_SH 0x01
#  define LOCK_EX 0x02
#  define LOCK_NB 0x04
#  define LOCK_UN 0x08
# endif
/* @} */
# define BoF_FLOCK_SHLCK LOCK_SH     /* shared or read file lock */
# define BoF_FLOCK_EXLCK LOCK_EX     /* exclusive or write file lock */
# define BoF_FLOCK_NONBLOCK LOCK_NB  /* don't block when locking */
# define BoF_FLOCK_UNLCK LOCK_UN     /* unlock file */

/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * unix-mode => unix-mode-str
 * unix-mode-str => unix-mode
 * "unix: rwazcxlLndtb, u=rwx g=rwx o=rwx"
 * r: read-only
 * w: write-only
 * a: append
 * z: truncate
 * c: create
 * x: exclusive
 * l: shared lock
 * L: exclusive lock
 * n: nonblock
 * d: close-on-exec
 *
 * ???
 * t: text
 * b: binary
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
BoCunmode(
# if __myc_require(prototypes)
	char *aBuffer,
	int Flags,
	bo_mode_t aPerm,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) ssize_t
BoWunmode(
# if __myc_require(prototypes)
	wchar_t *aBuffer,
	int Flags,
	bo_mode_t aPerm,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoMkunmode(
# if __myc_require(prototypes)
	char const *aStr,
	int *aFlagsPtr,
	bo_mode_t *aPermPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoWmkunmode(
# if __myc_require(prototypes)
	wchar_t const *aStr,
	int *aFlagsPtr,
	bo_mode_t *aPermPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoOpen
 * open
 * Version 6 AT&T UNIX
 */
# pragma mark -

/*
 * NOTE!!! For windows. Fixme.
 * FIFO should be named as: '\\.\pipe\<pipename>'
 */

__myc_flinkage__(public,libBo) int
__status__((
	BoS_SUCCESS = [ 0++ = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_NotConfigured,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_IsDir, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		           BoK_FileError_NoFreeSpace, BoK_FileError_TextBusy,
		           BoK_IoError_NoInformation, BoK_SysError_DiskQuota);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_NOTSUPP = (BoK_OpError_FeatureNotSupp);
	];
))
BoOpen(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr,
	char const *aPath,
	int Flags,
	...
/*
	[bo_mode_t aPermMode]
 */
# endif
	);

/*
int open(const char *aPath, int Flags, ...);
 */
# ifndef HAS_OPEN
#  ifdef HAS__SOPEN
#   define NEED__BO_OPEN_IMPL
#   if __myc_support(NVM)
#    define open(aPath,Flags...) _bo_open(aPath, Flags)
#   else
#   if __myc_support(AVM)
#    define open(aPath, ...) _bo_open(aPath, __VA_ARGS__)
#   else
#    define open _bo_open
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  else
#  ifdef HAS__OPEN
#   if __myc_support(NVM)
#    define open(aPath,Flags...) _open(aPath, Flags)
#   else
#   if __myc_support(AVM)
#    define open(aPath, ...) _open(aPath, __VA_ARGS__)
#   else
#    define open _open
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  endif /* HAS__OPEN */
#  endif /* HAS__SOPEN */
# endif /* !HAS_OPEN */

# ifdef NEED__BO_OPEN_IMPL
__myc_flinkage__(public,libBo) int
_bo_open(
# if __myc_require(prototypes)
	char const *aPath,
	int Flags,
	...
# endif
	);
# endif /* NEED__BO_OPEN_IMPL */

/* ----------------------------------------------------------------------------
 * BoWopen
 * wopen
 */
# pragma mark -

__myc_flinkage__(public,libBo) int
__status__((
	BoS_SUCCESS = [ 0++ = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_NotConfigured,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_IsDir, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_ProcessDescLimit, BoK_FileError_SysDescLimit,
		           BoK_FileError_NoFreeSpace, BoK_FileError_TextBusy,
		           BoK_IoError_NoInformation, BoK_SysError_DiskQuota);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_NOTSUPP = (BoK_OpError_FeatureNotSupp);
	];
))
BoWopen(
# if __myc_require(prototypes)
	struct aorp_error *anErrPtr,
	wchar_t const *aPath,
	int Flags,
	...
/*
	[bo_mode_t aPermMode]
 */
# endif
	);

/*
int wopen(wchar_t const *aPath, int Flags, [mode_t aPermMode]...);
 */
# ifndef HAS_WOPEN
#  ifdef HAS__WSOPEN
#   define NEED__BO_WOPEN_IMPL
#   if __myc_support(NVM)
#    define wopen(aPath,Flags...) _bo_wopen(aPath, Flags)
#   else
#   if __myc_support(AVM)
#    define wopen(aPath, ...) _bo_wopen(aPath, __VA_ARGS__)
#   else
#    define wopen _bo_wopen
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  else
#  ifdef HAS__WOPEN
#   if __myc_support(NVM)
#    define wopen(aPath,Flags...) _wopen(aPath, Flags)
#   else
#   if __myc_support(AVM)
#    define wopen(aPath, ...) _wopen(aPath, __VA_ARGS__)
#   else
#    define wopen _wopen
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  else
#   define NEED_WOPEN_IMPL
#  endif /* HAS__WOPEN */
#  endif /* HAS__WSOPEN */
# endif /* !HAS_WOPEN */

# ifdef NEED__BO_WOPEN_IMPL
__myc_flinkage__(public,libBo) int
_bo_wopen(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	int Flags,
	...
# endif
	);
# endif /* NEED__BO_WOPEN_IMPL */

# ifdef NEED_WOPEN_IMPL
__myc_flinkage__(public,libBo) int
wopen(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	int Flags,
	...
# endif
	);
# endif /* NEED_WOPEN_IMPL */

/* ----------------------------------------------------------------------------
 * BoDup, BoDup2
 * dup, dup2
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) int
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_FileError_BadDescriptor, BoK_FileError_ProcessDescLimit);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoDup(
# if __myc_require(prototypes)
	int aFD,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int dup(int aFD);
 */
# ifndef HAS_DUP
#  ifdef HAS__DUP
#   define dup(aFD) _dup(aFD)
#  endif /* HAS__DUP */
# endif /* !HAS_DUP */

/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) int
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_FileError_BadDescriptor, BoK_FileError_ProcessDescLimit);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoDup2(
# if __myc_require(prototypes)
	int anOldFD,
	int aNewFD,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int dup2(int anOldFD, int aNewFD);
 */
# ifndef HAS_DUP2
#  ifdef HAS__DUP2
#   define dup2(anOldFD, aNewFD) _dup2(anOldFD, aNewFD)
#  endif /* HAS__DUP2 */
# endif /* !HAS_DUP2 */

/* ----------------------------------------------------------------------------
 * BoFcntl
 * fcntl
 * BSD4.2
 */
# pragma mark -

__myc_flinkage__(public,libBo) int
__status__((
	BoS_SUCCESS = [ 0++ = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_DeadLock,
		           BoK_FileError_BadDescriptor, BoK_FileError_ProcessDescLimit,
		           BoK_FileError_NoLockAvail, BoK_SysError_NoSuchProcess);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_NOTSUPP = (BoK_OpError_FeatureNotSupp);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoFcntl(
# if __myc_require(prototypes)
	int aFD,
	int aCmd,
	...
/*
	{struct flock *aFlkb | int anArg},
	struct aorp_error *anErrPtr __myc_defval__(NULL)
 */
# endif
	);

/*
int
fcntl(int aFD, int aCmd, ...);
 */
# ifndef HAS_FCNTL
#  ifdef HAS__FCNTL
#   if __myc_support(NVM)
#    define fcntl(aFD, aCmd...) _fcntl(aFD, aCmd)
#   else
#   if __myc_support(AVM)
#    define fcntl(aFD, ...) _fcntl(aFD, __VA_ARGS__)
#   else
#    define fcntl _fcntl
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  else
#   define NEED_FCNTL_IMPL
#  endif /* HAS__FCNTL */
# endif /* !HAS_FCNTL */

# ifdef NEED_FCNTL_IMPL
__myc_flinkage__(public,libBo) int
fcntl(
# if __myc_require(prototypes)
	int aFD,
	int aCmd,
	...
# endif
	);

#  if __myc_support(NVM)
#   define fcntl(aFD, aCmd...) BoFcntl(aFD, aCmd, NULL)
#  else
#  if __myc_support(AVM)
#   define fcntl(aFD, ...) BoFcntl(aFD, __VA_ARGS__, NULL)
#  endif /* __myc_support(AVM) */
#  endif /* __myc_support(NVM) */

# endif /* NEED_FCNTL_IMPL */

/* ----------------------------------------------------------------------------
 * BoFlock
 * flock
 * BSD4.2
 */
# pragma mark -

/* BSD semantics */
/* Windows implements mandatory lock */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_RacError_WouldBlock,
		           BoK_FileError_BadDescriptor, BoK_FileError_NoLockAvail);
		BoS_NOTSUPP = (BoK_OpError_Inappropriate, BoK_OpError_NotAvail,
		               BoK_OpError_NotSupp);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoFlock(
# if __myc_require(prototypes)
	int aFD,
	int Operations,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int
flock(int aFD, int Flags);
 */
# ifndef HAS_FLOCK
#  ifdef HAS__FLOCK
#   define flock(aFD, Operations) _flock(aFD, Operations)
#  else
#   define NEED_FLOCK_IMPL
#  endif /* HAS__FLOCK */
# endif /* !HAS_FLOCK */

# ifdef NEED_FLOCK_IMPL
__myc_flinkage__(public,libBo) int
flock(
# if __myc_require(prototypes)
	int aFD,
	int Operations
# endif
	);
#  define flock(aFD, Operations) BoFlock(aFD, Operations, NULL)
# endif /* NEED_FLOCK_IMPL */

__MYC_END_CDECLS

#endif /* !__BO_SYS_FCNTL__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
