/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * FilePath.h	(N.Panasiuk)
 * include/Bo/sys/FilePath.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_FILEPATH__
# define __BO_SYS_FILEPATH__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Cdefs.h>

# include _BO_PLATFORM_HEADER(_FilePath.h)


/******************************************************************************
 * %% BeginSection: definitions
 */

# include <Bo/BoLinkage.h>

/* ============================================================================
 * Utils
 */
# pragma mark -
# pragma mark [ Utils ]
# pragma mark -

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libBo) bo_seqlist_t *
BoDestroyPathList(
# if __myc_require(prototypes)
	bo_seqlist_t *aList
# endif
	);

__myc_flinkage__(public,libBo) bo_seqlist_t *
BoClearPathList(
# if __myc_require(prototypes)
	bo_seqlist_t *aList
# endif
	);

__MYC_END_CDECLS

/* ============================================================================
 * Separator predicates
 */
# pragma mark -
# pragma mark [ Separator predicates ]
# pragma mark -

# define BoK_PATH_SEP   __myc_xconcat3(BoK_,__BO_FILEPATH_PREFIX__,_PATH_SEP)
# define BO_PATH_SEPS   __myc_xconcat3(BO_,__BO_FILEPATH_PREFIX__,_PATH_SEPS)
# define BoIsPathSep(x) __myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,IsPathSep)(x)

# define BoK_SEARCH_PATH_SEP \
	__myc_xconcat3(BoK_,__BO_FILEPATH_PREFIX__,_SEARCH_PATH_SEP)
# define BoIsSearchPathSep(x) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,IsSearchPathSep)(x)

# define BoK_EXT_SEP   __myc_xconcat3(BoK_,__BO_FILEPATH_PREFIX__,_EXT_SEP)
# define BoIsExtSep(x) __myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,IsExtSep)(x)

/* ============================================================================
 * $PATH methods
 */
# pragma mark -
# pragma mark [ $PATH methods ]
# pragma mark -

/*
 * __attribute__((nonnull(1)))
 * bo_seqlist_(char *) *BoSplitSearchPath(bo_seqlist_(char *) *, char const *);
 *
 * Take a string, split it on the BoK_SEARCH_PATH_SEP character.
 */
# define BoSplitSearchPath(aList,aSearchPath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,SplitSearchPath)(aList,aSearchPath)

/*
 * __attribute__((nonnull(1)))
 * bo_seqlist_(char *) *BoGetSearchPath(bo_seqlist_(char *) *);
 *
 * Get a list of FilePaths in the $PATH variable.
 */
# define BoGetSearchPath(aList) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,GetSearchPath)(aList)

/* ============================================================================
 * Extension functions
 */
# pragma mark -
# pragma mark [ Extension functions ]
# pragma mark -

/*
 * void BoSplitExtension(char **, char **, char const *);
 *
 * Split on the extension. BoAddExtension is the inverse.
 */
# define BoSplitExtension(aRestPtr,anExtPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,SplitExtension)(aRestPtr,anExtPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoTakeExtension(char **, char const *);
 *
 * Get the extension of a file, returns "" for no extension, .ext otherwise.
 */
# define BoTakeExtension(anExtPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,TakeExtension)(anExtPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoReplaceExtension(char **, char const *);
 *
 * Set the extension of a file, overwriting one if already present.
 */
# define BoReplaceExtension(aFilePathPtr,aFilePath,aNewExtension) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,ReplaceExtension)(aFilePathPtr,aFilePath,aNewExtension)

/*
 * __attribute__((nonnull(1)))
 * char **BoDropExtension(char **, char const *);
 *
 * Remove last extension, and the "." preceding it.
 */
# define BoDropExtension(aFilePathPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,DropExtension)(aFilePathPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoAddExtension(char **, char const *, char const *);
 *
 * Add an extension, even if there is already one there.
 */
# define BoAddExtension(aFilePathPtr,aFilePath,anExtension) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,AddExtension)(aFilePathPtr,aFilePath,anExtension)

/*
 * _Bool BoHasExtension(char const *);
 *
 * Does the given filename have an extension?
 */
# define BoHasExtension(aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,HasExtension)(aFilePath)

/*
 * void BoSplitExtensions(char **, char **, char const *);
 *
 * Split on all extensions.
 */
# define BoSplitExtensions(aRestPtr,anExtPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,SplitExtensions)(aRestPtr,anExtPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoTakeExtensions(char **, char const *);
 *
 * Get all extensions.
 */
# define BoTakeExtensions(anExtPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,TakeExtensions)(anExtPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoDropExtensions(char **, char const *);
 *
 * Drop all extensions.
 */
# define BoDropExtensions(aFilePathPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,DropExtensions)(aFilePathPtr,aFilePath)

/* ============================================================================
 * Filename/directory functions
 */
# pragma mark -
# pragma mark [ Filename/directory functions ]
# pragma mark -

/*
 * void BoSplitFileName(char **, char **, char const *);
 *
 * Split a filename into directory and file. BoCombinePaths is the inverse.
 * The first component will often end with a trailing slash.
 */
# define BoSplitFileName(aRestPtr,aFileNamePtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,SplitFileName)(aRestPtr,aFileNamePtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoTakeFileName(char **, char const *);
 *
 * Get the file name.
 */
# define BoTakeFileName(aFileNamePtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,TakeFileName)(aFileNamePtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoReplaceFileName(char **, char const *, char const *);
 *
 * Set the filename.
 */
# define BoReplaceFileName(aFilePathPtr,aFilePath,aNewFileName) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,ReplaceFileName)(aFilePathPtr,aFilePath,aNewFileName)

/*
 * __attribute__((nonnull(1)))
 * char **BoDropFileName(char **, char const *);
 *
 * Drop the filename. Unlike BoTakeDirectory, this function will leave
 * a trailing path separator on the directory.
 */
# define BoDropFileName(aFileNamePtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,DropFileName)(aFileNamePtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoTakeBaseName(char **, char const *);
 *
 * Get the base name, without an extension or path.
 */
# define BoTakeBaseName(aBaseNamePtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,TakeBaseName)(aBaseNamePtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoReplaceBaseName(char **, char const *, char const *);
 *
 * Set the base name.
 */
# define BoReplaceBaseName(aFilePathPtr,aFilePath,aNewBaseName) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,ReplaceBaseName)(aFilePathPtr,aFilePath,aNewBaseName)

/*
 * __attribute__((nonnull(1)))
 * char **BoTakeDirectory(char **, char const *);
 *
 * Get the directory name, move up one level.
 */
# define BoTakeDirectory(aDirectoryPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,TakeDirectory)(aDirectoryPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoReplaceDirectory(char **, char const *, char const *);
 *
 * Set the directory, keeping the filename the same.
 */
# define BoReplaceDirectory(aFilePathPtr,aFilePath,aNewDirectory) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,ReplaceDirectory)(aFilePathPtr,aFilePath,aNewDirectory)

/*
 * __attribute__((nonnull(1)))
 * char **BoCombinePaths(char **, char const *, char const *);
 *
 * Combine two paths, if the second path starts with a path separator
 * or a drive letter, then it returns the second.
 */
# define BoCombinePaths(aFilePathPtr,aLhs,aRhs) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,CombinePaths)(aFilePathPtr,aLhs,aRhs)

/*
 * __attribute__((nonnull(1)))
 * bo_seqlist_(char *) *BoSplitPath(bo_seqlist_(char *) *, char const *);
 *
 * Split a path by the directory separator.
 */
# define BoSplitPath(aList,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,SplitPath)(aList,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoJoinPath(char **, char const **, size_t);
 *
 * Join path elements back together.
 */
# define BoJoinPath(aFilePathPtr,anElements,anElementc) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,JoinPath)(aFilePathPtr,anElements,anElementc)

/*
 * __attribute__((nonnull(1)))
 * bo_seqlist_(char *) *BoSplitDirectories(bo_seqlist_(char *) *, char const *);
 *
 * Just as BoSplitPath, but don't add the trailing slashes to each element.
 */
# define BoSplitDirectories(aList,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,SplitDirectories)(aList,aFilePath)

/* ============================================================================
 * Drive functions
 */
# pragma mark -
# pragma mark [ Drive functions ]
# pragma mark -

/*
 * void BoSplitDrive(char **, char **, char const *);
 *
 * Split a path into a drive and a path. On Posix, / is a Drive.
 */
# define BoSplitDrive(aDrivePtr,aRestPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,SplitDrive)(aDrivePtr,aRestPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoJoinDrive(char **, char const *, char const *);
 *
 * Join a drive and the rest of the path.
 */
# define BoJoinDrive(aFilePathPtr,aDrive,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,JoinDrive)(aFilePathPtr,aDrive,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoTakeDrive(char **, char const *);
 *
 * Get the drive from a filepath.
 */
# define BoTakeDrive(aDrivePtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,TakeDrive)(aDrivePtr,aFilePath)

/*
 * _Bool BoHasDrive(char const *);
 *
 * Does a path have a drive?
 */
# define BoHasDrive(aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,HasDrive)(aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoDropDrive(char **, char const *);
 *
 * Delete the drive, if it exists.
 */
# define BoDropDrive(aRestPathPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,DropDrive)(aRestPathPtr,aFilePath)

/*
 * _Bool BoIsDrive(char const *);
 *
 * Is an element a drive?
 */
# define BoIsDrive(aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,IsDrive)(aFilePath)

/* ============================================================================
 * Trailing slash functions
 */
# pragma mark -
# pragma mark [ Trailing slash functions ]
# pragma mark -

/*
 * _Bool BoHasTrailingPathSep(char const *);
 *
 * Is an item either a directory or the last character a path separator?
 */
# define BoHasTrailingPathSep(aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,HasTrailingPathSep)(aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoAddTrailingPathSep(char **, char const *);
 *
 * Add a trailing file path separator if one is not already present.
 */
# define BoAddTrailingPathSep(aFilePathPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,AddTrailingPathSep)(aFilePathPtr,aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoDropTrailingPathSep(char **, char const *);
 *
 * Remove any trailing path separators.
 */
# define BoDropTrailingPathSep(aFilePathPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,DropTrailingPathSep)(aFilePathPtr,aFilePath)

/* ============================================================================
 * File name manipulations
 */
# pragma mark -
# pragma mark [ File name manipulations ]
# pragma mark -

/*
 * __attribute__((nonnull(1)))
 * char **BoNormalisePath(char **, char const *);
 *
 * Normalise a file
 *     - // outside of the drive can be made blank
 *     - /  -> BoK_PATH_SEP
 *     - ./ -> ""
 */
# define BoNormalisePath(aFilePathPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,NormalisePath)(aFilePathPtr,aFilePath)

/*
 * _Bool BoEqualFilePath(char const *, char const *);
 *
 * Equality of two FilePaths.
 * Note that this doesn't follow symlinks or DOSNAM~1s.
 */
# define BoEqualFilePath(aLhs,aRhs) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,EqualFilePath)(aLhs,aRhs)

/*
 * __attribute__((nonnull(1)))
 * char **BoMakeRelativePath(char **, char const *, char const *);
 *
 * Contract a filename, based on a relative path.
 */
# define BoMakeRelativePath(aFilePathPtr,aLhs,aRhs) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,MakeRelativePath)(aFilePathPtr,aLhs,aRhs)

/*
 * _Bool BoIsRelativePath(char const *);
 *
 * Is a path relative, or is it fixed to the root?
 *
 * According to
 * http://msdn.microsoft.com/en-us/library/windows/desktop/aa365247.aspx:
 *    - "A UNC name of any format [is never relative]."
 *    - "You cannot use the "\?" prefix with a relative path."
 */
# define BoIsRelativePath(aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,IsRelativePath)(aFilePath)

/*
 * _Bool BoIsAbsolutePath(char const *);
 *
 * BoIsAbsolutePath(x) { return !BoIsRelativePath(x); }
 */
# define BoIsAbsolutePath(aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,IsAbsolutePath)(aFilePath)

/*
 * _Bool BoIsValidPath(char const *);
 *
 * Is a FilePath valid, i.e. could you create a file like it?
 */
# define BoIsValidPath(aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,IsValidPath)(aFilePath)

/*
 * __attribute__((nonnull(1)))
 * char **BoMakeValidPath(char **, char const *);
 *
 * Take a FilePath and make it valid; does not change already valid FilePaths.
 */
# define BoMakeValidPath(aFilePathPtr,aFilePath) \
	__myc_xconcat3(Bo,__BO_FILEPATH_Prefix__,MakeValidPath)(aFilePathPtr,aFilePath)

#endif /* !__BO_SYS_FILEPATH__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
