/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Io.h	(V.Sokur)
 * include/Bo/sys/Io.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_IO__
# define __BO_SYS_IO__

/******************************************************************************
 * %% BeginSection: includes
 */
# pragma mark -

# include <Bo/sys/Types.h>

# if __myc_exists(unistd,h)
#  include __MYC_LIBC_HEADER(unistd.h)
# endif
# if __myc_exists(sys7uio,h)
#  include __MYC_LIBC_HEADER(sys/uio.h)
# endif
	/* readv, writev */

/* windows */
# if __myc_exists(io,h)
#  include __MYC_LIBC_HEADER(io.h)
# endif

#if 0
# include <Bo/sys/Fcntl.h>
	/* seek flags */
#endif

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

# ifndef UIO_MAXIOV
#  define UIO_MAXIOV 16
# endif
# define BoK_IOV_MAX UIO_MAXIOV

# ifndef HAS_IOVEC_STRUC
#  ifndef IOVEC_STRUC_DEFINED
#   define IOVEC_STRUC_DEFINED
/* Es tut mir leid, this is contrary to the Bo style */
struct iovec {
	bo_pointer_t iov_base;
	size_t       iov_len;
};
#  endif
# endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoClose
 * close
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_FileError_BadDescriptor);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoClose(
# if __myc_require(prototypes)
	int aFD,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int close(int aFD);
 */
# ifndef HAS_CLOSE
#  ifdef HAS__CLOSE
#   define close(aFD) _close(aFD)
#  endif /* HAS__CLOSE */
# endif /* !HAS_CLOSE */

/* ----------------------------------------------------------------------------
 * BoRead
 * read
 * IEEE Std 9945.1-1990 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* the number of bytes actually read */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_WouldBlock, BoK_FileError_BadDescriptor,
		           BoK_IoError_NoInformation);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoRead(
# if __myc_require(prototypes)
	int aFD,
	bo_pointer_t aBuffer,
	size_t aBytec,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
ssize_t read(int aFD, bo_pointer_t aBuffer, size_t aBytec);
 */
# ifndef HAS_READ
#  ifdef HAS__READ
#   define read(aFD,aBuffer,aBytec) _read(aFD,aBuffer,(unsigned)(aBytec))
#  endif /* HAS__READ */
# endif /* !HAS_READ */

/* ----------------------------------------------------------------------------
 * BoWrite
 * write
 * IEEE Std 9945.1-1990 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* the number of bytes which were written */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_WouldBlock, BoK_FileError_BadDescriptor,
		           BoK_FileError_FileSizeLimit, BoK_FileError_NoFreeSpace,
		           BoK_IpcError_BrokenPipe, BoK_IoError_NoInformation,
		           BoK_SysError_DiskQuota);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoWrite(
# if __myc_require(prototypes)
	int aFD,
	bo_const_pointer_t aBuffer,
	size_t aBytec,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
ssize_t write(int aFD, bo_const_pointer_t aBuffer, size_t aBytec);
 */
# ifndef HAS_WRITE
#  ifdef HAS__WRITE
#   define write(aFD,aBuffer,aBytec) _write(aFD,aBuffer,(unsigned)(aBytec))
#  endif /* HAS__WRITE */
# endif /* !HAS_WRITE */

/* ----------------------------------------------------------------------------
 * BoLseek
 * lseek
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_off_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* resulting offset location from the beginning of the file */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument,
		           BoK_FileError_BadDescriptor, BoK_FileError_SeekOnPipe);
	];
))
BoLseek(
# if __myc_require(prototypes)
	int aFD,
	bo_off_t anOffset,
	int Whence,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
off_t lseek(int aFD, off_t anOffset, int Whence);
 */
# ifdef OVERLOAD_LSEEK__LSEEKi64
#  define lseek(aFD,anOffset,Whence) _lseeki64(aFD,anOffset,Whence)
# else
# ifndef HAS_LSEEK
#  ifdef HAS__LSEEK
#   define lseek(aFD,anOffset,Whence) _lseek(aFD,anOffset,Whence)
#  endif /* HAS__LSEEK */
# endif /* !HAS_LSEEK */
# endif /* OVERLOAD_LSEEK__LSEEKi64 */

/* ----------------------------------------------------------------------------
 * BoTell
 * tell
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_off_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* resulting offset location from the beginning of the file */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_FileError_BadDescriptor, BoK_FileError_SeekOnPipe);
	];
))
BoTell(
# if __myc_require(prototypes)
	int aFD,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
off_t tell(int aFD);
 */
# ifdef OVERLOAD_TELL__TELLi64
#  define tell(aFD) _telli64(aFD)
# else
# ifndef HAS_TELL
#  ifdef HAS__TELL
#   define tell(aFD) _tell(aFD)
#  else
#   define tell(aFD) lseek(aFD, 0, SEEK_CUR)
#  endif /* HAS__TELL */
# endif /* !HAS_TELL */
# endif /* OVERLOAD_TELL__TELLi64 */

/* ----------------------------------------------------------------------------
 * BoPread
 * pread
 * X/Open Portability Guide Issue 4 (XPG4.2)
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* the number of bytes actually read */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_WouldBlock, BoK_FileError_BadDescriptor,
		           BoK_FileError_SeekOnPipe, BoK_IoError_NoInformation);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoPread(
# if __myc_require(prototypes)
	int aFD,
	bo_pointer_t aBuffer,
	size_t aBytec,
	bo_off_t anOffset,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
ssize_t pread(int aFD, bo_pointer_t aBuffer, size_t aBytec, off_t anOffset);
 */
# ifndef HAS_PREAD
#  ifdef HAS__PREAD
#   define pread(aFD,aBuffer,aBytec,anOffset) \
		_pread(aFD,aBuffer,(unsigned)(aBytec),anOffset)
#  endif /* HAS__PREAD */
# endif /* !HAS_PREAD */

/* ----------------------------------------------------------------------------
 * BoPwrite
 * pwrite
 * X/Open Portability Guide Issue 4 (XPG4.2)
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* the number of bytes which were written */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_WouldBlock, BoK_FileError_BadDescriptor,
		           BoK_FileError_NoFreeSpace, BoK_FileError_SeekOnPipe,
		           BoK_IpcError_BrokenPipe, BoK_IoError_NoInformation,
		           BoK_SysError_DiskQuota);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoPwrite(
# if __myc_require(prototypes)
	int aFD,
	bo_const_pointer_t aBuffer,
	size_t aBytec,
	bo_off_t anOffset,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
ssize_t pwrite(int aFD, bo_const_pointer_t aBuffer, size_t aBytec, off_t anOffset);
 */
# ifndef HAS_PWRITE
#  ifdef HAS__PWRITE
#   define pwrite(aFD,aBuffer,aBytec,anOffset) \
		_pwrite(aFD,aBuffer,(unsigned)(aBytec),anOffset)
#  endif /* HAS__PWRITE */
# endif /* !HAS_PWRITE */

/* ----------------------------------------------------------------------------
 * BoReadv
 * readv
 * X/Open Portability Guide Issue 4 (XPG4.2)
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* the number of bytes actually read */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_WouldBlock, BoK_FileError_BadDescriptor,
		           BoK_IoError_NoInformation);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoReadv(
# if __myc_require(prototypes)
	int aFD,
	struct iovec aBufv[/* BoK_IOV_MAX */],
	unsigned aBufc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
ssize_t readv(int aFD, struct iovec *aBufv, int aBufc);
 */
# ifndef HAS_READV
#  ifdef HAS__READV
#   define readv(aFD,aBufv,aBufc) _readv(aFD,aBufv,aBufc)
#  endif /* HAS__READV */
# endif /* !HAS_READV */

/* ----------------------------------------------------------------------------
 * BoWritev
 * writev
 * X/Open Portability Guide Issue 4 (XPG4.2)
 */
# pragma mark -

__myc_flinkage__(public,libBo) ssize_t
__status__((
	BoS_SUCCESS = [ 0++ = (/* the number of bytes which were written */); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_WouldBlock, BoK_FileError_BadDescriptor,
		           BoK_FileError_NoFreeSpace, BoK_IpcError_BrokenPipe,
		           BoK_IpcError_DestAddrRequired, BoK_IpcError_NoBufferSpace,
		           BoK_IoError_NoInformation, BoK_SysError_DiskQuota);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoWritev(
# if __myc_require(prototypes)
	int aFD,
	struct iovec const aBufv[/* BoK_IOV_MAX */],
	unsigned aBufc,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
ssize_t writev(int aFD, struct iovec const *aBufv, int aBufc);
 */
# ifndef HAS_WRITEV
#  ifdef HAS__WRITEV
#   define writev(aFD,aBufv,aBufc) _writev(aFD,aBufv,aBufc)
#  endif /* HAS__WRITE */
# endif /* !HAS_WRITE */

__MYC_END_CDECLS

#endif /* !__BO_SYS_IO__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
