/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Linkage.h	(V.Sokur)
 * include/Bo/sys/Linkage.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_LINKAGE__
# define __BO_SYS_LINKAGE__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SYS_CDEFS__
#  include <Bo/sys/Cdefs.h>
# endif

# include _BO_PLATFORM_CONFIG_HEADER(Linkage.h)
# include _BO_COMPILER_CONFIG_HEADER(Linkage.h)

/******************************************************************************
 * BeginSection: definitions
 */
# pragma mark -

# ifndef __MYC_BEGIN_CDECLS
#  ifdef __BEGIN_DECLS
#   define __MYC_BEGIN_CDECLS __BEGIN_DECLS
#   define __MYC_END_CDECLS __END_DECLS
#  else
#   ifdef __cplusplus
#    define __MYC_BEGIN_CDECLS extern "C" {
#    define __MYC_END_CDECLS }
#   else
#    define __MYC_BEGIN_CDECLS
#    define __MYC_END_CDECLS
#   endif
#  endif
# endif /* !__MYC_BEGIN_CDECLS */

# ifndef __MYC_BEGIN_CXXDECLS
#  define __MYC_BEGIN_CXXDECLS extern "C++" {
#  define __MYC_END_CXXDECLS }
# endif /* !__MYC_BEGIN_CXXDECLS */

# pragma mark -

# ifndef __myc_extern_C
#  ifdef __cplusplus
#   define __myc_extern_C extern "C"
#  else
#   define __myc_extern_C extern
#  endif
# endif

# ifndef __myc_extern_CXX
#  ifdef __cplusplus
#   define __myc_extern_CXX extern "C++"
#  else
#   define __myc_extern_CXX extern
#  endif
# endif

# pragma mark -

# ifndef __myc_extern_inline__
#  ifdef __cplusplus
#   define __myc_extern_inline__ inline
#  else
#   define __myc_extern_inline__ static inline
#  endif /* !__cplusplus */
# endif /* !__myc_extern_inline__ */

# ifndef __myc_nested_inline__
#  define __myc_nested_inline__ auto inline
# endif

# ifndef __myc_nested_linkage
#  define __myc_nested_linkage auto
# endif

# ifndef __myc_asm_linkage
#  define __myc_asm_linkage
# endif

/* @todo: move to ... */
#ifdef _MSC_VER
# ifndef __myc_linker_export
#  ifdef _A_AMD64
#   define __myc_linker_export(_NAME) \
	comment(linker, __myc_xstringify(__myc_xconcat(/export:,_NAME)))
#  else
#   define __myc_linker_export(_NAME) \
	comment(linker, __myc_xstringify(__myc_xconcat3(/export:,_,_NAME)))
#  endif
# endif
#else
# define __myc_linker_export(_NAME) /* what is this? */
#endif

/*
 * LINKAGE DECL
 */
# pragma mark -

# ifndef myc_flinkage_shlib_are_defined
#  define myc_flinkage_shlib_export extern
#  define myc_flinkage_shlib_private /* extern */
#  define myc_flinkage_shlib_internal /* extern */
#  define myc_flinkage_shlib_import extern
# endif

# ifndef myc_vlinkage_shlib_are_defined
#  define myc_vlinkage_shlib_export extern
#  define myc_vlinkage_shlib_private extern
#  define myc_vlinkage_shlib_internal extern
#  define myc_vlinkage_shlib_import extern
# endif

# ifndef myc_flinkage_archive_are_defined
#  define myc_flinkage_archive_export extern
#  define myc_flinkage_archive_private /* extern */
#  define myc_flinkage_archive_internal /* extern */
#  define myc_flinkage_archive_import extern
# endif

# ifndef myc_vlinkage_archive_are_defined
#  define myc_vlinkage_archive_export extern
#  define myc_vlinkage_archive_private extern
#  define myc_vlinkage_archive_internal extern
#  define myc_vlinkage_archive_import extern
# endif

/* BUILDING SHLIB */
# define myc_vlinkage_shlib_public myc_vlinkage_shlib_export

# define __myc_flinkage_shlib(type) __myc_xconcat(myc_flinkage_shlib_,type)
# define __myc_vlinkage_shlib(type) __myc_xconcat(myc_vlinkage_shlib_,type)

/* BUILDING AORP-MODULE */
# define __myc_flinkage_amod(type) __myc_flinkage_shlib(type)
# define __myc_vlinkage_amod(type) __myc_vlinkage_shlib(type)

/*
 * BUILDING or USING STAGE
 *
 * The each product (shlib, archive) should define the next macros:
 * myc_flinkage_public_<<product>>
 * myc_flinkage_private_<<product>>
 * myc_flinkage_internal_<<product>>
 */
# define __myc_flinkage__(type,product) __myc_xconcat4(myc_flinkage_,type,_,product)
# define __myc_vlinkage__(type,product) __myc_xconcat4(myc_vlinkage_,type,_,product)
/* type:
public
private
internal
*/

/*
 * The each product (shlib, archive) should define the next macros:
 *

For example, the product target has name libBo:

// BUILDING
#define build_shlib_libBo 1
or
#define build_archive_libBo 1

// USING
#define use_shlib_libBo 1
or
#define use_archive_libBo 1
*/
# define ___myc_build_shlib(x) (x + 0)
# define __myc_build_shlib(Name) ___myc_build_shlib(__myc_xconcat(build_shlib_,Name))

# define ___myc_use_shlib(x) (x + 0)
# define __myc_use_shlib(Name) ___myc_use_shlib(__myc_xconcat(use_shlib_,Name))

# define ___myc_build_archive(x) (x + 0)
# define __myc_build_archive(Name) ___myc_build_archive(__myc_xconcat(build_archive_,Name))

# define ___myc_use_archive(x) (x + 0)
# define __myc_use_archive(Name) ___myc_use_archive(__myc_xconcat(use_archive_,Name))

/*
 * CALLTYPE
 */
# pragma mark -

# define __myc_calltype__(x) __myc_xconcat(myc_calltype_,x)
# define __myc_tcalltype__(x) __myc_xconcat(myc_tcalltype_,x)
/*
cdecl
stdcall
mcall
*/
/* @todo: fixme */
# ifdef cdecl
#  undef cdecl
# endif
# ifdef stdcall
#  undef stdcall
# endif
# ifdef mcall
#  undef mcall
# endif

# ifndef myc_calltype_are_defined
#  if __myc_support_keyword(__attribute__)
#   define myc_calltype_cdecl __attribute__((cdecl))
#   define myc_calltype_stdcall __attribute__((stdcall))
#   define myc_calltype_mcall __attribute__((cdecl))
#   define mcalltype cdecl
#  else /* !__myc_support_keyword(__attribute__) */
#   define myc_calltype_cdecl
#   define myc_calltype_stdcall
#   define myc_calltype_mcall
#   define mcalltype
#  endif /* !__myc_support_keyword(__attribute__) */
# endif

# ifndef myc_tcalltype_are_defined
#  if __myc_support_keyword(__attribute__)
#   define myc_tcalltype_cdecl __attribute__((cdecl))
#   define myc_tcalltype_stdcall __attribute__((stdcall))
#   define myc_tcalltype_mcall __attribute__((cdecl))
#  else /* !__myc_support_keyword(__attribute__) */
#   define myc_tcalltype_cdecl
#   define myc_tcalltype_stdcall
#   define myc_tcalltype_mcall
#  endif /* !__myc_support_keyword(__attribute__) */
# endif

#endif /* !__BO_SYS_LINKAGE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
