/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Paths.h	(V.Sokur)
 * include/Bo/sys/Paths.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_PATHS__
# define __BO_SYS_PATHS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Stdconfig.h>

# if __myc_exists(paths,h)
#  include __MYC_LIBC_HEADER(paths.h)
# endif

/******************************************************************************
 * %% BeginSection: constant declarations
 */

/* Default search path. */
# ifndef _PATH_DEFPATH
#  define _PATH_DEFPATH "/usr/bin:/bin"
# endif
/* All standard utilities path. */
# ifndef _PATH_STDPATH
#  define _PATH_STDPATH "/usr/bin:/bin:/usr/sbin:/sbin"
# endif

# ifndef _PATH_UNIX
#  define _PATH_UNIX "/unix"
# endif

/*
 * Provide trailing slash, since mostly used for building pathnames.
 * @{
 */
# ifndef _PATH_DEV
#  define _PATH_DEV "/dev/"
# endif

# ifdef BO_SYSTYPE_WIN32

#  ifndef _PATH_BOWORLD
#   define _PATH_BOWORLD "C:\\ProgramData\\Bo.world\\"
#  endif

#  ifndef _PATH_TMP
#   define _PATH_TMP _PATH_BOWORLD "tmp\\"
#  endif

#  ifndef _PATH_CONF
#   define _PATH_CONF _PATH_BOWORLD "etc\\"
#  endif

#  ifndef _PATH_VAR
#   define _PATH_VAR _PATH_BOWORLD "var\\"
#  endif
#  ifndef _PATH_VARDB
#   define _PATH_VARDB _PATH_VAR "db\\"
#  endif
#  ifndef _PATH_VARLOG
#   define _PATH_VARLOG _PATH_VAR "log\\"
#  endif
#  ifndef _PATH_VARRUN
#   define _PATH_VARRUN _PATH_VAR "run\\"
#  endif
#  ifndef _PATH_VARTMP
#   define _PATH_VARTMP _PATH_VAR "tmp\\"
#  endif

#  ifndef _PATH_BOSLDIR
#   define _PATH_BOSLDIR  _PATH_VARLOG "bosl"
#   define _PATH_BOSLLOG  _PATH_BOSLDIR "\\"
#  endif

# else /* !BO_SYSTYPE_WIN32 */

#  ifndef _PATH_BOWORLD
#   define _PATH_BOWORLD "/opt/Bo.world/"
#  endif

#  ifndef _PATH_TMP
#   define _PATH_TMP "/tmp/"
#  endif

#  ifndef _PATH_CONF
#   define _PATH_CONF "/etc/"
#  endif
#  ifndef _PATH_VAR
#   define _PATH_VAR "/var/"
#  endif
#  ifndef _PATH_VARDB
#   define _PATH_VARDB _PATH_VAR "db/"
#  endif
#  ifndef _PATH_VARLOG
#   define _PATH_VARLOG _PATH_VAR "log/"
#  endif
#  ifndef _PATH_VARRUN
#   define _PATH_VARRUN _PATH_VAR "run/"
#  endif
#  ifndef _PATH_VARTMP
#   define _PATH_VARTMP _PATH_VAR "tmp/"
#  endif

#  ifndef _PATH_UUCPLOCK
#   define _PATH_UUCPLOCK _PATH_VAR "spool/lock/"
#  endif

#  ifndef _PATH_BOSLDIR
#   define _PATH_BOSLDIR  _PATH_VARLOG "bosl"
#   define _PATH_BOSLLOG  _PATH_BOSLDIR "/"
#  endif

# endif /* BO_SYSTYPE_WIN32 */
/* @} */

/* shell */
# ifndef _PATH_SHELLS
#  define _PATH_SHELLS "/etc/shells"
# endif
# ifndef _PATH_BSHELL
#  define _PATH_BSHELL "/bin/sh"
# endif
# ifndef _PATH_CSHELL
#  define _PATH_CSHELL "/bin/csh"
# endif
# ifndef _PATH_RSH
#  define _PATH_RSH "/usr/bin/rsh"
# endif

/* utilities */
# ifndef _PATH_NOLOGIN
#  define _PATH_NOLOGIN "/sbin/nologin"
# endif
# ifndef _PATH_NOLOGIN_MSG
#  define _PATH_NOLOGIN_MSG "/etc/nologin.txt"
# endif
# ifndef _PATH_FSIRAND
#  define _PATH_FSIRAND "/sbin/fsirand"
# endif
# ifndef _PATH_SENDMAIL
#  define _PATH_SENDMAIL "/usr/sbin/sendmail"
# endif
# ifndef _PATH_VI
#  define _PATH_VI "/usr/bin/vi"
# endif

/* devices */
# ifndef _PATH_TTY
#  define _PATH_TTY _PATH_DEV "tty"
# endif
# ifndef _PATH_CONSOLE
#  define _PATH_CONSOLE _PATH_DEV "console"
# endif
# ifndef _PATH_DEVNULL
#  define _PATH_DEVNULL _PATH_DEV "null"
# endif
# ifndef _PATH_KMEM
#  define _PATH_KMEM _PATH_DEV "kmem"
# endif
# ifndef _PATH_MEM
#  define _PATH_MEM _PATH_DEV "mem"
# endif

# ifndef _PATH_DEFTAPE
#  define _PATH_DEFTAPE _PATH_DEV "nrst0"
# endif
# ifndef _PATH_DRUM
#  define _PATH_DRUM _PATH_DEV "drum"
# endif

/* state info */
# ifndef _PATH_DEVDB
#  define _PATH_DEVDB _PATH_VARRUN "dev.db"
# endif
# ifndef _PATH_KVMDB
#  define _PATH_KVMDB _PATH_VARDB "kvm.db"
# endif

# ifndef _PATH_MAILDIR
#  define _PATH_MAILDIR _PATH_VAR "mail"
# endif

/* other directories */
# ifndef _PATH_LOCALE
#  define _PATH_LOCALE "/usr/share/locale"
# endif
# ifndef _PATH_MAN
#  define _PATH_MAN "/usr/share/man"
# endif

#endif /* !__BO_SYS_PATHS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
