/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Plafstd.h	(V.Sokur)
 * include/Bo/sys/Plafstd.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_PLAFSTD__
# define __BO_SYS_PLAFSTD__

/******************************************************************************
 * %% BeginSection: includes
 */
# pragma mark -

# include <Bo/sys/Types.h>

# if __myc_exists(unistd,h)
#  include __MYC_LIBC_HEADER(unistd.h)
# endif

# if __myc_exists(windows,h)
#  include __MYC_LIBC_HEADER(windows.h)
# endif
# ifdef BO_SYSTYPE_WIN32
#  include __MYC_LIBC_HEADER(winsock2.h)
#  ifndef HAS_TIMEVAL_STRUC
#   define HAS_TIMEVAL_STRUC
#  endif
# endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -

/* ============================================================================
 * Common functions
 */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/*
int gethostname(char *, size_t );
 */

__MYC_END_CDECLS

/* ============================================================================
 * Bo version of standard sleep functions: sleep, msleep, usleep, nanosleep.
 * @seealso: <Bo/libc/Time.h>,  <Bo/sys/Time.h>
 */

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoSleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoSleep(
# if __myc_require(prototypes)
	bo_sec_t aRequest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_SLEEP
#  define NEED_SLEEP_IMPL

#  undef sleep
__myc_flinkage__(public,libBo) int
sleep(
# if __myc_require(prototypes)
	seconds_t
# endif
	);

#  define sleep(aRequest) BoSleep((bo_sec_t)aRequest, NULL)

# endif /* !HAS_SLEEP */

/* ----------------------------------------------------------------------------
 * BoMsleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoMsleep(
# if __myc_require(prototypes)
	bo_msec_t aRequest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_MSLEEP
#  define NEED_MSLEEP_IMPL

#  undef msleep
__myc_flinkage__(public,libBo) int
msleep(
# if __myc_require(prototypes)
	mseconds_t
# endif
	);

#  define msleep(aRequest) BoMsleep((bo_msec_t)aRequest, NULL)

# endif /* !HAS_MSLEEP */

/* ----------------------------------------------------------------------------
 * BoUsleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoUsleep(
# if __myc_require(prototypes)
	bo_usec_t aRequest,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_USLEEP
#  define NEED_USLEEP_IMPL

#  undef usleep
__myc_flinkage__(public,libBo) int
usleep(
# if __myc_require(prototypes)
	useconds_t
# endif
	);

#  define usleep(aRequest) BoUsleep((bo_usec_t)aRequest, NULL)

# endif /* !HAS_USLEEP */

/* ----------------------------------------------------------------------------
 * BoNanosleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoNanosleep(
# if __myc_require(prototypes)
	struct timespec const *aRequest,
	struct timespec *aRemaind __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_NANOSLEEP
#  define NEED_NANOSLEEP_IMPL

#  undef nanosleep
__myc_flinkage__(public,libBo) int
nanosleep(
# if __myc_require(prototypes)
	const struct timespec *,
	struct timespec *
# endif
	);

#  define nanosleep(aRequest, aRemaind) BoNanosleep(aRequest, aRemaind, NULL)

# endif /* !HAS_NANOSLEEP */

__MYC_END_CDECLS

/* ============================================================================
 * I/O functions
 */
# include <Bo/sys/Io.h>

/* ============================================================================
 * Directory control functions
 */
# include <Bo/sys/Dir.h>

/* ============================================================================
 * Process [execution] functions
 */
# include <Bo/sys/Process.h>

#endif /* !__BO_SYS_PLAFSTD__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
