/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Process.h	(V.Sokur)
 * include/Bo/sys/Process.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_PROCESS__
# define __BO_SYS_PROCESS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# if __myc_exists(unistd,h)
#  include __MYC_LIBC_HEADER(unistd.h)
# endif
# include <Bo/libc/Stdlib.h>
	/* exit */

# if __myc_exists(process,h)
#  include __MYC_LIBC_HEADER(process.h)
# endif

/******************************************************************************
 * %% BeginSection: definitions
 */

/* spawn modes @{ */
# define BoK_X_OVERLAY (0)
# define BoK_X_WAIT (1)
# define BoK_X_NONBLOCK (2)
# define BoK_X_NOWAIT (3)
# define BoK_X_DAEMON (4)
/* @} */

/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* @todo: implement */
# ifdef HAS_ENVIRON
#  define _G_proc_environ environ
# else
# ifdef HAS__ENVIRON
#  define _G_proc_environ _environ
# else
#  define _G_proc_environ NULL
# endif
# endif

/* ----------------------------------------------------------------------------
 * BoExit, _BoExit
 */
# pragma mark -

/* @seealso: Sysexits.h */
__myc_flinkage__(public,libBo) void
BoExit(
# if __myc_require(prototypes)
	int aStatus
# endif
	);

# define BoExit(aStatus) exit(aStatus)

__myc_flinkage__(public,libBo) void
_BoExit(
# if __myc_require(prototypes)
	int aStatus
# endif
	);

# define _BoExit(aStatus) _exit(aStatus)

/* ----------------------------------------------------------------------------
 * BoFork
 */
# pragma mark -

/* @todo: implement */
# ifdef HAS_FORK
#  define BoFork(anErrPtr) fork()
# else
# ifdef HAS__FORK
#  define BoFork(anErrPtr) _fork()
# else
#  define BoFork(anErrPtr) (BoS_NOTIMPL)
# endif /* HAS__FORK */
# endif /* HAS_FORK */

# ifndef HAS_FORK
#  ifdef HAS__FORK
#   define fork() _fork()
#  endif /* HAS__FORK */
# endif /* !HAS_FORK */

/* ----------------------------------------------------------------------------
 * BoExecl
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoExecl(
# if __myc_require(prototypes)
	char /* const */ *aPath,
	char /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

# ifndef HAS_EXECL
#  ifdef HAS__EXECL
#   if __myc_support(NVM)
#    define execl(aPath, anArg...) _execl(aPath, anArg)
#   else
#   if __myc_support(AVM)
#    define execl(aPath, ...) _execl(aPath, __VA_ARGS__)
#   else
#    define execl _execl
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  endif /* HAS__EXECL */
# endif /* !HAS_EXECL */

/* ----------------------------------------------------------------------------
 * BoWexecl
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWexecl(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

# ifndef HAS_WEXECL
#  ifdef HAS__WEXECL
#   if __myc_support(NVM)
#    define wexecl(aPath, anArg...) _wexecl(aPath, anArg)
#   else
#   if __myc_support(AVM)
#    define wexecl(aPath, ...) _wexecl(aPath, __VA_ARGS__)
#   else
#    define wexecl _wexecl
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  else
#   define NEED_WEXECL_IMPL
#  endif /* HAS__WEXECL */
# endif /* !HAS_WEXECL */

# ifdef NEED_WEXECL_IMPL
__myc_flinkage__(public,libBo) int
wexecl(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *anArg,
	...
/*	, NULL */
# endif
	);
# endif /* NEED_WEXECL_IMPL */

/* ----------------------------------------------------------------------------
 * BoExeclp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoExeclp(
# if __myc_require(prototypes)
	char /* const */ *aName,
	char /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

# ifndef HAS_EXECLP
#  ifdef HAS__EXECLP
#   if __myc_support(NVM)
#    define execlp(aPath, anArg...) _execlp(aPath, anArg)
#   else
#   if __myc_support(AVM)
#    define execlp(aPath, ...) _execlp(aPath, __VA_ARGS__)
#   else
#    define execlp _execlp
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  endif /* HAS__EXECLP */
# endif /* !HAS_EXECLP */

/* ----------------------------------------------------------------------------
 * BoWexeclp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWexeclp(
# if __myc_require(prototypes)
	wchar_t /* const */ *aName,
	wchar_t /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

# ifndef HAS_WEXECLP
#  ifdef HAS__WEXECLP
#   if __myc_support(NVM)
#    define wexeclp(aPath, anArg...) _wexeclp(aPath, anArg)
#   else
#   if __myc_support(AVM)
#    define wexeclp(aPath, ...) _wexeclp(aPath, __VA_ARGS__)
#   else
#    define wexeclp _wexeclp
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  else
#   define NEED_WEXECLP_IMPL
#  endif /* HAS__WEXECLP */
# endif /* !HAS_WEXECLP */

# ifdef NEED_WEXECLP_IMPL
__myc_flinkage__(public,libBo) int
wexeclp(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *anArg,
	...
/*	, NULL */
# endif
	);
# endif /* NEED_WEXECLP_IMPL */

/* ----------------------------------------------------------------------------
 * BoExecle
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoExecle(
# if __myc_require(prototypes)
	char /* const */ *aPath,
	char /* const */ *anArg,
	...
/* 
	, NULL
	, char const *const *anEnvp
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

# ifndef HAS_EXECLE
#  ifdef HAS__EXECLE
#   if __myc_support(NVM)
#    define execle(aPath, anArg...) _execle(aPath, anArg)
#   else
#   if __myc_support(AVM)
#    define execle(aPath, ...) _execle(aPath, __VA_ARGS__)
#   else
#    define execle _execle
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  endif /* HAS__EXECLE */
# endif /* !HAS_EXECLE */

/* ----------------------------------------------------------------------------
 * BoWexecle
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWexecle(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *anArg,
	...
/* 
	, NULL
	, wchar_t const *const *anEnvp
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

# ifndef HAS_WEXECLE
#  ifdef HAS__WEXECLE
#   if __myc_support(NVM)
#    define wexecle(aPath, anArg...) _wexecle(aPath, anArg)
#   else
#   if __myc_support(AVM)
#    define wexecle(aPath, ...) _wexecle(aPath, __VA_ARGS__)
#   else
#    define wexecle _wexecle
#   endif /* __myc_support(AVM) */
#   endif /* __myc_support(NVM) */
#  else
#   define NEED_WEXECLE_IMPL
#  endif /* HAS__WEXECLE */
# endif /* !HAS_WEXECLE */

# ifdef NEED_WEXECLE_IMPL
__myc_flinkage__(public,libBo) int
wexecle(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *anArg,
	...
/* 
	, NULL
	, wchar_t const *const *anEnvp
*/
# endif
	);
# endif /* NEED_WEXECLE_IMPL */

/* ----------------------------------------------------------------------------
 * BoExecv
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoExecv(
# if __myc_require(prototypes)
	char /* const */ *aPath,
	char /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

# ifndef HAS_EXECV
#  ifdef HAS__EXECV
#   define execv(aPath, anArgv) _execv(aPath, anArgv)
#  endif /* HAS__EXECV */
# endif /* !HAS_EXECV */

/* ----------------------------------------------------------------------------
 * BoWexecv
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWexecv(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

# ifndef HAS_WEXECV
#  ifdef HAS__WEXECV
#   define wexecv(aPath, anArgv) _wexecv(aPath, anArgv)
#  else
#   define NEED_WEXECV_IMPL
#  endif /* HAS__WEXECV */
# endif /* !HAS_WEXECV */

# ifdef NEED_WEXECV_IMPL
__myc_flinkage__(public,libBo) int
wexecv(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *const anArgv[]
# endif
	);
# endif /* NEED_WEXECV_IMPL */

/* ----------------------------------------------------------------------------
 * BoExecvp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoExecvp(
# if __myc_require(prototypes)
	char /* const */ *aName,
	char /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

# ifndef HAS_EXECVP
#  ifdef HAS__EXECVP
#   define execvp(aPath, anArgv) _execvp(aPath, anArgv)
#  endif /* HAS__EXECVP */
# endif /* !HAS_EXECVP */

/* ----------------------------------------------------------------------------
 * BoWexecvp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWexecvp(
# if __myc_require(prototypes)
	wchar_t /* const */ *aName,
	wchar_t /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

# ifndef HAS_WEXECVP
#  ifdef HAS__WEXECVP
#   define wexecvp(aPath, anArgv) _wexecvp(aPath, anArgv)
#  else
#   define NEED_WEXECVP_IMPL
#  endif /* HAS__WEXECVP */
# endif /* !HAS_WEXECVP */

# ifdef NEED_WEXECVP_IMPL
__myc_flinkage__(public,libBo) int
wexecvp(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *const anArgv[]
# endif
	);
# endif /* NEED_WEXECVP_IMPL */

/* ----------------------------------------------------------------------------
 * BoExecve
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoExecve(
# if __myc_require(prototypes)
	char /* const */ *aPath,
	char /* const */ *const anArgv[],
	char /* const */ *const *anEnvp,
	struct aorp_error *anErrPtr
# endif
	);

# ifndef HAS_EXECVE
#  ifdef HAS__EXECVE
#   define execve(aPath, anArgv, anEnviron) _execve(aPath, anArgv, anEnviron)
#  endif /* HAS__EXECVE */
# endif /* !HAS_EXECVE */

/* ----------------------------------------------------------------------------
 * BoWexecve
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWexecve(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *const anArgv[],
	wchar_t /* const */ *const *anEnvp,
	struct aorp_error *anErrPtr
# endif
	);

# ifndef HAS_WEXECVE
#  ifdef HAS__WEXECVE
#   define wexecve(aPath, anArgv, anEnviron) _wexecve(aPath, anArgv, anEnviron)
#  else
#   define NEED_WEXECVE_IMPL
#  endif /* HAS__WEXECVE */
# endif /* !HAS_WEXECVE */

# ifdef NEED_WEXECVE_IMPL
__myc_flinkage__(public,libBo) int
wexecve(
# if __myc_require(prototypes)
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *const anArgv[],
	wchar_t /* const */ *const *anEnvp
# endif
	);
# endif /* NEED_WEXECVE_IMPL */

/* ----------------------------------------------------------------------------
 * BoSpawnl
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoSpawnl(
# if __myc_require(prototypes)
	int aMode,
	char /* const */ *aPath,
	char /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWspawnl
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWspawnl(
# if __myc_require(prototypes)
	int aMode,
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

/* ----------------------------------------------------------------------------
 * BoSpawnlp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoSpawnlp(
# if __myc_require(prototypes)
	int aMode,
	char /* const */ *aName,
	char /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWspawnlp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWspawnlp(
# if __myc_require(prototypes)
	int aMode,
	wchar_t /* const */ *aName,
	wchar_t /* const */ *anArg,
	...
/* 
	, NULL
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

/* ----------------------------------------------------------------------------
 * BoSpawnle
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoSpawnle(
# if __myc_require(prototypes)
	int aMode,
	char /* const */ *aPath,
	char /* const */ *anArg,
	...
/* 
	, NULL
	, char const *const *anEnvp
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWspawnle
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWspawnle(
# if __myc_require(prototypes)
	int aMode,
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *anArg,
	...
/* 
	, NULL
	, wchar_t const *const *anEnvp
	, struct aorp_error *anErrPtr __myc_defval__(NULL)
*/
# endif
	);

/* ----------------------------------------------------------------------------
 * BoSpawnv
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoSpawnv(
# if __myc_require(prototypes)
	int aMode,
	char /* const */ *aPath,
	char /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWspawnv
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWspawnv(
# if __myc_require(prototypes)
	int aMode,
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

/* ----------------------------------------------------------------------------
 * BoSpawnvp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoSpawnvp(
# if __myc_require(prototypes)
	int aMode,
	char /* const */ *aName,
	char /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWspawnvp
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWspawnvp(
# if __myc_require(prototypes)
	int aMode,
	wchar_t /* const */ *aName,
	wchar_t /* const */ *const anArgv[],
	struct aorp_error *anErrPtr
# endif
	);

/* ----------------------------------------------------------------------------
 * BoSpawnve
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoSpawnve(
# if __myc_require(prototypes)
	int aMode,
	char /* const */ *aPath,
	char /* const */ *const anArgv[],
	char /* const */ *const *anEnvp,
	struct aorp_error *anErrPtr
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWspawnve
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [
		0++{ .cond="@mode = BoK_X_WAIT" } = (/* executed process status */);
		1++{ .cond="@mode = BoK_X_NONBLOCK" } = (/* child process identifier */);
		1++{ .cond="@mode = BoK_X_NOWAIT" } = (/* child process identifier */);
		0  { .cond="@mode = BoK_X_DAEMON" } = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_ArgError_ArgListLimit, BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_TextBusy, BoK_SysError_InvalidExecFormat,
		           BoK_SysError_NoMemory, BoK_SysError_ProcsLimit,
		           BoK_IoError_NoInformation);
	];
))
BoWspawnve(
# if __myc_require(prototypes)
	int aMode,
	wchar_t /* const */ *aPath,
	wchar_t /* const */ *const anArgv[],
	wchar_t /* const */ *const *anEnvp,
	struct aorp_error *anErrPtr
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
BoDaemon(
# if __myc_require(prototypes)
	int aNochdir,
	int aNoclose,
	struct aorp_error *anErrPtr
# endif
	);

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

# define BoGetpid() getpid()

# ifndef HAS_GETPID
#  ifdef HAS__GETPID
#   define getpid() _getpid()
#  else
#   define getpid() ((__BO_PID_TYPE__)30900)
#  endif
# endif /* !HAS_GETPID */

# define BoGetgid() getgid()

# ifndef HAS_GETGID
#  ifdef HAS__GETGID
#   define getgid() _getgid()
#  else
#   define getgid() ((__BO_GID_TYPE__)30900)
#  endif
# endif /* !HAS_GETGID */

# define BoGetuid() getpid()

# ifndef HAS_GETUID
#  ifdef HAS__GETUID
#   define getuid() _getuid()
#  else
#   define getuid() ((__BO_UID_TYPE__)30900)
#  endif
# endif /* !HAS_GETUID */

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libBo) bo_status_t
BoCreatPidfile(
# if __myc_require(prototypes)
	char const *aPidfileBase,
	bo_mode_t Perms, /* 0644 */
	struct aorp_error *anErrPtr
# endif
	) __attribute__((nonnull(1)));

__myc_flinkage__(public,libBo) bo_status_t
BoUnlinkPidfile(
# if __myc_require(prototypes)
	char const *aPidfileBase,
	struct aorp_error *anErrPtr
# endif
	) __attribute__((nonnull(1)));

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Wait.h>

#endif /* !__BO_SYS_PROCESS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
