/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Stat.h	(V.Sokur)
 * include/Bo/sys/Stat.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_STAT__
# define __BO_SYS_STAT__

/******************************************************************************
 * %% BeginSection: includes
 */
# pragma mark -

# include <Bo/sys/Types.h>

/* @todo: fixme */
# if (64 <= (_FILE_OFFSET_BITS - 0))
#  ifndef __USE_FILE_OFFSET64
#   define __USE_FILE_OFFSET64
#  endif
# endif

# if __myc_exists(sys7stat,h)
#  include __MYC_LIBC_HEADER(sys/stat.h)
# endif
	/* stat, umask, chmod, mkdir, mkfifo [, mknod ], ... */

/* windows */
# if __myc_exists(direct,h)
#  include __MYC_LIBC_HEADER(direct.h)
# endif
	/* mkdir, [ _getdrive ], [ _chdrive ], [ _getdcwd ], ... */
# if __myc_exists(io,h)
#  include __MYC_LIBC_HEADER(io.h)
# endif
	/* chmod, ... */

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

# include <Bo/sys/_FTypedefs.h>
# include <Bo/sys/_FModedefs.h>

/*
 * Structure declaration.
 * Problem is that the structure name and function name
 * should be the same.
 */

# if (64 <= (_FILE_OFFSET_BITS + 0))
#  ifdef HAS_STAT64_STRUC
#   define bo_stat stat64
#  else
#  ifdef HAS__STAT64_STRUC
#   define bo_stat _stat64
#  else
#  ifdef HAS_STAT_STRUC
#   define bo_stat stat
#  else
#  ifdef HAS__STAT_STRUC
#   define bo_stat _stat
#  else
#   define bo_stat stat
#  endif /* HAS__STAT_STRUC */
#  endif /* HAS_STAT_STRUC */
#  endif /* HAS__STAT64_STRUC */
#  endif /* HAS_STAT64_STRUC */
# else
#  ifdef HAS_STAT_STRUC
#   define bo_stat stat
#  else
#  ifdef HAS__STAT_STRUC
#   define bo_stat _stat
#  else
#   define bo_stat stat
#  endif /* HAS__STAT_STRUC */
#  endif /* HAS_STAT_STRUC */
# endif /* (64 <= (_FILE_OFFSET_BITS + 0)) */

/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoStat
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_IoError_NoInformation);
	];
))
BoStat(
# if __myc_require(prototypes)
	char const *aPath,
	struct bo_stat *aResult,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWstat
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_IoError_NoInformation);
	];
))
BoWstat(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct bo_stat *aResult,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoLstat
 * BSD4.2
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_IoError_NoInformation);
	];
))
BoLstat(
# if __myc_require(prototypes)
	char const *aPath,
	struct bo_stat *aResult,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWlstat
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_IoError_NoInformation);
	];
))
BoWlstat(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	struct bo_stat *aResult,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoFstat
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_FileError_BadDescriptor,
		           BoK_IoError_NoInformation);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoFstat(
# if __myc_require(prototypes)
	int aFD,
	struct bo_stat *aResult,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoUmask
 * umask
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_mode_t
__status__((
	BoS_SUCCESS = [ 0++ = (); ];
	BoS_FAILURE = [ BoS_NOTIMPL = (BoK_OpError_NotImpl); ];
))
BoUmask(
# if __myc_require(prototypes)
	bo_mode_t aPermModeMask,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# define BoUmask(aPermModeMask,anErrPtr) \
	((bo_mode_t)(unsigned long)umask((mode_t)(aPermModeMask)))

/*
mode_t umask(mode_t aPermModeMask);
 */
#  ifndef HAS_UMASK
#   ifdef HAS__UMASK
#    define umask(aPermModeMask) _umask(aPermModeMask)
#   endif /* HAS__UMASK */
#  endif /* !HAS_UMASK */

/* ----------------------------------------------------------------------------
 * BoChmod
 * chmod
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_ReadOnlyFS, BoK_IoError_NoInformation);
	];
))
BoChmod(
# if __myc_require(prototypes)
	char const *aPath,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int chmod(char const *aPath, mode_t aPermMode);
 */
# ifndef HAS_CHMOD
#  ifdef HAS__CHMOD
#   define chmod(aPath, aPermMode) _chmod(aPath, aPermMode)
#  endif /* HAS__CHMOD */
# endif /* !HAS_CHMOD */

/* ----------------------------------------------------------------------------
 * BoWchmod
 * wchmod
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation, BoK_RacError_OperationNotPermitted,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_Loop,
		           BoK_FileError_ReadOnlyFS, BoK_IoError_NoInformation);
	];
))
BoWchmod(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

#  ifndef HAS_WCHMOD
#   ifdef HAS__WCHMOD
#    define wchmod(aPath, aPermMode) _wchmod(aPath, aPermMode)
#   else
#    define NEED_WCHMOD_IMPL
#   endif /* HAS__WCHMOD */
#  endif /* !HAS_WCHMOD */

# ifdef NEED_WCHMOD_IMPL
__myc_flinkage__(public,libBo) int
wchmod(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	mode_t aPermMode
# endif
	);
# endif /* NEED_WCHMOD_IMPL */

/* ----------------------------------------------------------------------------
 * BoFchmod
 * fchmod
 * BSD4.2
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_InvalidArgument, BoK_FileError_BadDescriptor,
		           BoK_FileError_ReadOnlyFS, BoK_IoError_NoInformation);
		BoS_NOTSUPP = (BoK_OpError_Inappropriate, BoK_OpError_NotSupp);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoFchmod(
# if __myc_require(prototypes)
	int aFD,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_FCHMOD
#  ifdef HAS__FCHMOD
#   define fchmod(aFD, aPermMode) _fchmod(aFD, aPermMode)
#  else
#   define NEED_FCHMOD_IMPL
#  endif /* HAS__FCHMOD */
# endif /* !HAS_FCHMOD */

# ifdef NEED_FCHMOD_IMPL
__myc_flinkage__(public,libBo) int
fchmod(
# if __myc_require(prototypes)
	int aFD,
	mode_t aPermMode
# endif
	);
#  define fchmod(aFD, aPermMode) BoFchmod(aFD, (bo_mode_t)(aPermMode), NULL)
# endif /* NEED_FCHMOD_IMPL */

/* ----------------------------------------------------------------------------
 * BoMkfifo
 * mkfifo
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

/*
 * NOTE!!! For windows. Fixme.
 * FIFO should be named as: '\\.\pipe\<pipename>'
 */

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace,
		           BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
	];
))
BoMkfifo(
# if __myc_require(prototypes)
	char const *aPath,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_MKFIFO
#  ifdef HAS__MKFIFO
#   define mkfifo(aPath, aPermMode) _mkfifo(aPath, aPermMode)
#  else
#   define NEED_MKFIFO_IMPL
#  endif /* HAS__MKFIFO */
# endif /* !HAS_MKFIFO */

# ifdef NEED_MKFIFO_IMPL
__myc_flinkage__(public,libBo) int
mkfifo(
# if __myc_require(prototypes)
	char const *aPath,
	mode_t aPermMode
# endif
	);
#  define mkfifo(aPath, aPermMode) BoMkfifo(aPath, (bo_mode_t)(aPermMode), NULL)
# endif /* NEED_MKFIFO_IMPL */

/* ----------------------------------------------------------------------------
 * BoWmkfifo
 * wmkfifo
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NameSizeLimit, BoK_FileError_NotDir,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace,
		           BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
		BoS_NOTIMPL = (BoK_OpError_NotImpl);
		BoS_NOTSUPP = (BoK_OpError_NotSupp);
	];
))
BoWmkfifo(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_WMKFIFO
#  ifdef HAS__WMKFIFO
#   define wmkfifo(aPath, aPermMode) _wmkfifo(aPath, aPermMode)
#  else
#   define NEED_WMKFIFO_IMPL
#  endif /* HAS__WMKFIFO */
# endif /* !HAS_WMKFIFO */

# ifdef NEED_WMKFIFO_IMPL
__myc_flinkage__(public,libBo) int
wmkfifo(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	mode_t aPermMode
# endif
	);
# endif /* NEED_WMKFIFO_IMPL */

/* ----------------------------------------------------------------------------
 * BoMkdir
 * mkdir
 * IEEE Std 1003.1-1988 (POSIX.1)
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace,
		           BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoMkdir(
# if __myc_require(prototypes)
	char const *aPath,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int mkdir(char const *aPath, mode_t aPermMode);
 */
# ifndef HAS_MKDIR
#  ifdef HAS__MKDIR
#   define mkdir(aPath, aPermMode) _mkdir(aPath /*, aPermMode */)
#  endif /* HAS__MKDIR */
# endif /* !HAS_MKDIR */

/* ----------------------------------------------------------------------------
 * BoWmkdir
 * wmkdir
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace,
		           BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoWmkdir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	bo_mode_t aPermMode,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/*
int wmkdir(wchar_t const *aPath, mode_t aPermMode);
 */
# ifndef HAS_WMKDIR
#  ifdef HAS__WMKDIR
#   define wmkdir(aPath, aPermMode) _wmkdir(aPath /*, aPermMode */)
#  else
#   define NEED_WMKDIR_IMPL
#  endif /* HAS__WMKDIR */
# endif /* !HAS_WMKDIR */

# ifdef NEED_WMKDIR_IMPL
__myc_flinkage__(public,libBo) int
wmkdir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	mode_t aPermMode
# endif
	);
# endif /* NEED_WMKDIR_IMPL */

/* ----------------------------------------------------------------------------
 * Flags for 'BoXmkdir()'.
 */
# pragma mark -

/* Create intermediate dir as required.*/
# define BoF_XMKD_P    BoF_OP_FORCE

/* ----------------------------------------------------------------------------
 * BoXmkdir
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace,
		           BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoXmkdir(
# if __myc_require(prototypes)
	char const *aPath,
	bo_mode_t aPermMode,
	unsigned Flags,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWxmkdir
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
__status__((
	BoS_SUCCESS = [ BoS_NORMAL = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_RacError_AccessViolation,
		           BoK_FileError_NotDir, BoK_FileError_NameSizeLimit,
		           BoK_FileError_NoSuchEntry, BoK_FileError_AlreadyExists,
		           BoK_FileError_Loop, BoK_FileError_ReadOnlyFS,
		           BoK_FileError_NoFreeSpace,
		           BoK_SysError_DiskQuota, BoK_IoError_NoInformation);
	];
))
BoWxmkdir(
# if __myc_require(prototypes)
	wchar_t const *aPath,
	bo_mode_t aPermMode,
	unsigned Flags,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_SYS_STAT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
