/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Sysexits.h	(V.Sokur)
 * include/Bo/sys/Sysexits.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_SYSEXITS__
# define __BO_SYS_SYSEXITS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Stdconfig.h>

# if __myc_exists(sysexits,h)
#  include __MYC_LIBC_HEADER(sysexits.h)
# endif

/******************************************************************************
 * %% BeginSection: constant declarations
 */

/* ISO/IEC 9899:1999 (ISO C99) */
# ifndef EXIT_SUCCESS
#  define EXIT_SUCCESS EX_NORMAL
# endif
# ifndef EXIT_FAILURE
#  define EXIT_FAILURE EX_FAILURE
# endif

/* ------------------------------------------------------------------------- */

# ifndef EX_NORMAL
#  define EX_NORMAL 0
# endif
# ifndef EX_FAILURE
#  define EX_FAILURE 1
# endif

# ifndef EX_OK
#  define EX_OK EX_NORMAL
# endif


/* base value for error messages */
# ifndef EX__BASE
#  define EX__BASE 64
# endif

/* Command line usage error. The command was used incorrectly, e.g., with
   the wrong number of arguments, a bad flag, a bad syntax in a parameter,
   or whatever. */
# ifndef EX_USAGE
#  define EX_USAGE 64
# endif

/* Data format error. The input data was incorrect in some way.
   This should only be used for user's data & not system files. */
# ifndef EX_DATAERR
#  define EX_DATAERR 65
# endif

/* Cannot open input. An input file (not a system file) did not
   exist or was not readable.  This could also include
   errors like "No message" to a mailer (if it cared to catch it). */
# ifndef EX_NOINPUT
#  define EX_NOINPUT 66
# endif

/* Addressee unknown. The user specified did not exist.  This might
   be used for mail addresses or remote logins. */
# ifndef EX_NOUSER
#  define EX_NOUSER 67
# endif

/* Host name unknown. The host specified did not exist.  This is used
   in mail addresses or network requests. */
# ifndef EX_NOHOST
#  define EX_NOHOST 68
# endif

/* Service unavailable. This can occur if a support program or file does
   not exist. This can also be used as a catchall message when something
   you wanted to do doesn't work, but you don't know why. */
# ifndef EX_UNAVAILABLE
#  define EX_UNAVAILABLE 69
# endif

/* Internal software error. This should be limited to non-operating system
   related errors as possible. */
# ifndef EX_SOFTWARE
#  define EX_SOFTWARE 70
# endif

/* System error (e.g., can't fork). */
# ifndef EX_OSERR
#  define EX_OSERR 71
# endif

/* Critical OS file missing. Some system file (e.g., /etc/passwd, /etc/utmp,
   etc.) does not exist, cannot be opened, or has some sort of
   error (e.g., syntax error). */
# ifndef EX_OSFILE
#  define EX_OSFILE 72
# endif

/* Can't create (user) output file. */
# ifndef EX_CANTCREAT
#  define EX_CANTCREAT 73
# endif

/* Input/output error. */
# ifndef EX_IOERR
#  define EX_IOERR 74
# endif

/* Temporary failure; user is invited to retry. In sendmail, this means
   that a mailer (e.g.) could not create a connection,
   and the request should be reattempted later. */
# ifndef EX_TEMPFAIL
#  define EX_TEMPFAIL 75
# endif

/* Remote error in protocol. The remote system returned something that
   was "not possible" during a protocol exchange. */
# ifndef EX_PROTOCOL
#  define EX_PROTOCOL 76
# endif

/* Permission denied. You did not have sufficient permission to
   perform the operation. This is not intended for file system problems,
   which should use NOINPUT or CANTCREAT, but rather for higher
   level permissions. */
# ifndef EX_NOPERM
#  define EX_NOPERM 77
# endif

/* Configuration error. */
# ifndef EX_CONFIG
#  define EX_CONFIG 78
# endif

/* Maximum listed value */
# ifndef EX__MAX
#  define EX__MAX 78
# endif

#endif /* !__BO_SYS_SYSEXITS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
