/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Time.h	(V.Sokur)
 * include/Bo/sys/Time.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_TIME__
# define __BO_SYS_TIME__

/******************************************************************************
 * %% BeginSection: includes
 */
# pragma mark -

# include <Bo/sys/Types.h>

# if __myc_exists(sys7time,h)
#  if defined(_ANSI_SOURCE) || defined(_POSIX_C_SOURCE)
#   ifdef _ANSI_SOURCE
#    undef _ANSI_SOURCE
#   endif
#   ifdef _POSIX_C_SOURCE
#    undef _POSIX_C_SOURCE
#   endif
#   include __MYC_LIBC_HEADER(sys/time.h)
#   ifndef _ANSI_SOURCE
#    define _ANSI_SOURCE 1L
#   endif
#   ifndef _POSIX_C_SOURCE
#    define _POSIX_C_SOURCE 198808L
#   endif
#  else
#   include __MYC_LIBC_HEADER(sys/time.h)
#  endif
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -

# ifndef HAS_TIMEVAL_STRUC
struct timeval {
	tv_time_t    tv_sec;   /* seconds */
	suseconds_t	 tv_usec;  /* microseconds */
};
# endif


# ifndef BO_TIMEVAL_TO_TIMESPEC
#  define BO_TIMEVAL_TO_TIMESPEC(tvp, tsp) \
	do { \
		(tsp)->tv_sec = (tvp)->tv_sec; \
		(tsp)->tv_nsec = (tvp)->tv_usec * 1000; \
	} while (0)
# endif

# ifndef BO_TIMESPEC_TO_TIMEVAL
#  define BO_TIMESPEC_TO_TIMEVAL(tsp, tvp) \
	do { \
		(tvp)->tv_sec = (tsp)->tv_sec; \
		(tvp)->tv_usec = (tsp)->tv_nsec / 1000; \
	} while (0)
# endif


# define BO_TIMEVAL_Add(aSrc1, aSrc2, aDst) \
	do { \
		(aDst)->tv_sec = (aSrc1)->tv_sec + (aSrc2)->tv_sec; \
		(aDst)->tv_usec = (aSrc1)->tv_usec + (aSrc2)->tv_usec; \
		if (1000000 <= (aDst)->tv_usec) { \
			(aDst)->tv_usec -= 1000000; \
			(aDst)->tv_sec++; \
		} \
	} while (0)

# define BO_TIMEVAL_Sub(aSrc1, aSrc2, aDst) \
	do { \
		(aDst)->tv_sec = (aSrc1)->tv_sec - (aSrc2)->tv_sec; \
		(aDst)->tv_usec = (aSrc1)->tv_usec - (aSrc2)->tv_usec; \
		if ((aDst)->tv_usec < 0) { \
			(aDst)->tv_usec += 1000000; \
			(aDst)->tv_sec--; \
		} \
	} while (0)

# define BO_TIMEVAL_Compare(_CMP, aLhs, aRhs) \
	(((aLhs)->tv_sec == (aRhs)->tv_sec) \
			? ((aLhs)->tv_usec _CMP (aRhs)->tv_usec) \
			: ((aLhs)->tv_sec _CMP (aRhs)->tv_sec))


# define BO_TIMEVAL_AsMsec(tvp) \
	(((bo_msec_t)(tvp)->tv_sec) * 1000 + (tvp)->tv_usec / 1000)

# define BO_TIMEVAL_AsUsec(tvp) \
	(((bo_usec_t)(tvp)->tv_sec) * 1000000 + (tvp)->tv_usec)

# define BO_TIMEVAL_FromMsec(tvp, ms) \
	do { \
		(tvp)->tv_sec = (tv_time_t)((ms) / 1000); \
		(tvp)->tv_usec = (suseconds_t)(((ms) - ((bo_msec_t)(tvp)->tv_sec) * 1000) * 1000); \
	} while (0)

# define BO_TIMEVAL_FromUsec(tvp, us) \
	do { \
		(tvp)->tv_sec = (tv_time_t)((us) / 1000000); \
		(tvp)->tv_usec = (suseconds_t)(((us) - ((bo_usec_t)(tvp)->tv_sec) * 1000000)); \
	} while (0)


# ifndef BO_SYSTYPE_WIN32
struct w32_system_time_s {
	u_int16_t wYear,
	          wMonth,
	          wDayOfWeek,
	          wDay,
	          wHour,
	          wMinute,
	          wSecond,
	          wMilliseconds;
};
typedef struct w32_system_time_s W32_SYSTEMTIME;

struct w32_file_time_s {
	u_int32_t dwLowDateTime,
	          dwHighDateTime;
};
typedef struct w32_file_time_s W32_FILETIME;
# else
typedef SYSTEMTIME W32_SYSTEMTIME;
typedef FILETIME W32_FILETIME;
# endif /* !BO_SYSTYPE_WIN32 */

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoGettimeofday
 */
# pragma mark -

/* Since midnight (0 hour), January 1, 1970. */
__myc_flinkage__(public,libBo) bo_status_t
BoGettimeofday(
# if __myc_require(prototypes)
	struct timeval *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_GETTIMEOFDAY
#  define NEED_GETTIMEOFDAY_IMPL

#  undef gettimeofday
__myc_flinkage__(public,libBo) int
gettimeofday(
# if __myc_require(prototypes)
	struct timeval *,
	struct timezone *
# endif
	);

#  define gettimeofday(aBuffer, aTmzone) BoGettimeofday(aBuffer, NULL)

# endif /* !HAS_GETTIMEOFDAY */

/* ----------------------------------------------------------------------------
 * BoNanosleep
 */
# pragma mark -

__myc_flinkage__(public,libBo) bo_status_t
BoNanosleep(
# if __myc_require(prototypes)
	struct timespec const *aRequestDuration,
	struct timespec *aRemaind __myc_defval__(NULL),
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

# ifndef HAS_NANOSLEEP
#  define NEED_NANOSLEEP_IMPL

#  undef nanosleep
__myc_flinkage__(public,libBo) int
nanosleep(
# if __myc_require(prototypes)
	const struct timespec *,
	struct timespec *
# endif
	);

#  define nanosleep(aRequest, aRemaind) BoNanosleep(aRequest, aRemaind, NULL)

# endif /* !HAS_NANOSLEEP */

/* ----------------------------------------------------------------------------
 * BoTimevalToW32SytemTime, BoTimevalFromW32SytemTime,
 * BoTimevalToW32FileTime, BoTimevalFromW32FileTime
 */
# pragma mark -

/* For win32 only!!! */

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoTimevalToW32SytemTime(
# if __myc_require(prototypes)
	struct timeval const *aTm,
	W32_SYSTEMTIME *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoTimevalFromW32SytemTime(
# if __myc_require(prototypes)
	struct timeval *aBuffer,
	W32_SYSTEMTIME const *aTm,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoTimevalToW32FileTime(
# if __myc_require(prototypes)
	struct timeval const *aTm,
	W32_FILETIME *aBuffer,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__myc_flinkage__(public,libBo) bo_status_t
__attribute__((nonnull(1,2)))
BoTimevalFromW32FileTime(
# if __myc_require(prototypes)
	struct timeval *aBuffer,
	W32_FILETIME const *aTm,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/libc/Time.h>

#endif /* !__BO_SYS_TIME__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
