/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Types.h	(V.Sokur)
 * include/Bo/sys/Types.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_CDEFS__
# include <Bo/sys/Cdefs.h>
#endif

#ifndef __BO_SYS_TYPES__
# define __BO_SYS_TYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# if __myc_exists(sys7types,h)
#  include __MYC_LIBC_HEADER(sys/types.h)
# endif

# ifdef _AFXDLL
#  include __MYC_LIBC_HEADER(afx.h)
# else
# if __myc_exists(windows,h)
#  include __MYC_LIBC_HEADER(windows.h)
# endif
# endif /* _AFXDLL */

/* @todo: fixme */
# ifdef cdecl
#  undef cdecl
# endif
# ifdef stdcall
#  undef stdcall
# endif
# ifdef mcall
#  undef mcall
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# include <Bo/sys/_Stddef.h>
# include <Bo/sys/_Stdint.h>
# include <Bo/sys/_Stdbool.h>

/* ------------------------------------------------------------------------- */

# ifndef HAS_INO_TYPE
#  ifdef HAS__INO_TYPE
typedef _ino_t ino_t;
#   define PRIino PRI_ino
#  else
typedef u_int32_t ino_t;
#   define PRIino PRIu32
#  endif /* HAS__INO_TYPE */
# endif /* !HAS_INO_TYPE */

/* ----------------------------------------------------------------------------
 * FILE OFFSET type
 * @{
 */

# ifndef HAS_OFF_TYPE
#  if (64 <= (_FILE_OFFSET_BITS + 0))
typedef int64_t off_t;
#   define PRIoff PRIi64
#  else
#  ifdef HAS__OFF_TYPE
typedef _off_t off_t;
#   define PRIoff PRI_off
#  else
typedef long off_t;
#   define PRIoff PRIi
#  endif /* HAS__OFF_TYPE */
#  endif /* (64 <= (_FILE_OFFSET_BITS + 0)) */
# endif /* !HAS_OFF_TYPE */

# if (64 <= (_FILE_OFFSET_BITS + 0))
typedef int64_t bo_off_t;
#  define PRIboff PRIi64
# else
typedef off_t bo_off_t;
#  define PRIboff PRIoff
# endif /* (64 <= (_FILE_OFFSET_BITS + 0)) */
/* @} */

/* ----------------------------------------------------------------------------
 * FILE MODE type
 * @{
 */

# ifndef HAS_MODE_TYPE
typedef int mode_t;
# endif
# ifndef PRImode
#  define PRImode PRIi
# endif

/* @seealso: Bo/sys/Stat.h */
typedef int    bo_mode_t; /* BSD mode_t is short, problem with the open() function. */
typedef mode_t bo_mode_x;
# define PRIbmode PRIi
/* @} */

/* ----------------------------------------------------------------------------
 * Process/Group/User ID
 * @{
 */

# ifndef HAS_PID_TYPE
#  ifdef __BO_PID_TYPE__
typedef __BO_PID_TYPE__ pid_t;
#  else
typedef int32_t pid_t;
#  endif
# endif
# ifndef PRIpid
#  define PRIpid PRIi32
# endif

# ifndef HAS_GID_TYPE
#  ifdef __BO_GID_TYPE__
typedef __BO_GID_TYPE__ gid_t;
#  else
typedef u_int32_t gid_t;
#  endif
# endif
# ifndef PRIgid
#  define PRIgid PRIu32
# endif

# ifndef HAS_UID_TYPE
#  ifdef __BO_UID_TYPE__
typedef __BO_UID_TYPE__ uid_t;
#  else
typedef u_int32_t uid_t;
#  endif
# endif
# ifndef PRIuid
#  define PRIuid PRIu32
# endif
/* @} */

/* ----------------------------------------------------------------------------
 * TIME types
 */

# ifndef HAS_TIME_TYPE
typedef long time_t;
# endif
# ifndef PRItime
#  define PRItime PRIi32
# endif

# ifndef BO_SYSTYPE_WIN32
typedef time_t tv_time_t;
# else
typedef long tv_time_t;
# endif

# ifndef HAS_SECONDS_TYPE
typedef u_int32_t seconds_t;
# endif
# ifndef PRIsec
#  define PRIsec PRIu32
# endif
# ifndef HAS_SSECONDS_TYPE
typedef int32_t sseconds_t;
# endif
# ifndef PRIssec
#  define PRIssec PRIi32
# endif

# ifndef HAS_MSECONDS_TYPE
typedef u_int32_t mseconds_t;
# endif
# ifndef PRImsec
#  define PRImsec PRIu32
# endif
# ifndef HAS_SMSECONDS_TYPE
typedef int32_t smseconds_t;
# endif
# ifndef PRIsmsec
#  define PRIsmsec PRIi32
# endif

# ifndef HAS_USECONDS_TYPE
typedef u_int32_t useconds_t;
# endif
# ifndef PRIusec
#  define PRIusec PRIu32
# endif
# ifndef HAS_SUSECONDS_TYPE
typedef int32_t suseconds_t;
# endif
# ifndef PRIsusec
#  define PRIsusec PRIi32
# endif

/******************************************************************************
 * Bo C type declarations
 */
# pragma mark -
# pragma mark [ Bo C type declarations ]
# pragma mark -

typedef unsigned char u_char_t;

typedef u_int16_t bo_ucs2_t;
typedef u_int32_t bo_ucs4_t;

# ifndef __BO_UTF8_CHAR__
#  define __BO_UTF8_CHAR__ unsigned char
# endif
# ifndef __BO_UTF8_CHAR_DEFINED__
#  define __BO_UTF8_CHAR_DEFINED__
typedef __BO_UTF8_CHAR__ bo_u8char_t;
# endif
# define _UTF8_T(s) ((bo_u8char_t const *)s)

# ifndef __BO_BYTE_TYPE_DEFINED__
#  define __BO_BYTE_TYPE_DEFINED__
typedef unsigned char bo_byte_t;
# endif

# ifndef __BO_POINTER_TYPE_DEFINED__
#  define __BO_POINTER_TYPE_DEFINED__
typedef __BO_VOID_PTR__ bo_pointer_t;
typedef const __BO_VOID_PTR__ bo_const_pointer_t;
# endif

/* ----------------------------------------------------------------------------
 * ref-counter
 */

# ifndef __BO_REFC_TYPE__
#  if (BO_M_WORD == BO_M_WORD32)
#   define __BO_REFC_TYPE__ /* volatile */ int32_t
#   define __BO_REFC_xTYPE__ int32_t
#   define __REFC_TYPE_MAX__ 0x3fffffff
#   define __REFC_TYPE_MAX2__ 0x7fffffff
#   define PRIrefc PRIi32
#  else
#  if (BO_M_WORD == BO_M_WORD64)
#   define __BO_REFC_TYPE__ /* volatile */ int64_t
#   define __BO_REFC_xTYPE__ int64_t
#   define __REFC_TYPE_MAX__ 0x3fffffffffffffffLL
#   define __REFC_TYPE_MAX2__ 0x7fffffffffffffffLL
#   define PRIrefc PRIi64
#  endif
#  endif
# endif
typedef __BO_REFC_TYPE__  bo_refc_t __attribute__((aligned(BoK_NATURAL_ALIGN)));
typedef __BO_REFC_xTYPE__ bo_refc_x;

# define BoL_REFC_MAX __REFC_TYPE_MAX__
# define BoL_REFC_MAX2 __REFC_TYPE_MAX2__

/* ----------------------------------------------------------------------------
 * md-lock
 */

typedef /* volatile */ u_int32_t  bo_cpulock32_t __attribute__((aligned(BoK_NATURAL_ALIGN)));
typedef int32_t             bo_cpulock32_x;
# ifdef HAS_INT64_TYPE
typedef /* volatile */ u_int64_t  bo_cpulock64_t __attribute__((aligned(BoK_NATURAL_ALIGN)));
typedef int64_t             bo_cpulock64_x;
# endif

# if (BO_M_WORD == BO_M_WORD32)
typedef bo_cpulock32_t bo_cpulock_t;
typedef bo_cpulock32_x bo_cpulock_x;
# else
# if (BO_M_WORD == BO_M_WORD64)
typedef bo_cpulock64_t bo_cpulock_t;
typedef bo_cpulock64_x bo_cpulock_x;
# endif
# endif


struct bo_cpurwlock32_s {
	bo_cpulock32_t lrw_mutex;
	bo_cpulock32_t lrw_rcond,
	               lrw_wcond;
	int16_t        lrw_nworkers,     /* -1: work-writer, 0 < : work-readers */
	               lrw_nwriters;
};
typedef struct bo_cpurwlock32_s  bo_cpurwlock32_t;

struct bo_cpurwlock64_s {
	bo_cpulock64_t lrw_mutex;
	bo_cpulock64_t lrw_rcond,
	               lrw_wcond;
	short          lrw_nworkers,     /* -1: work-writer, 0 < : work-readers */
	               lrw_nwriters;
};
typedef struct bo_cpurwlock64_s  bo_cpurwlock64_t;

# if (BO_M_WORD == BO_M_WORD32)
typedef bo_cpurwlock32_t bo_cpurwlock_t;
# else
# if (BO_M_WORD == BO_M_WORD64)
typedef bo_cpurwlock64_t bo_cpurwlock_t;
# endif
# endif

# define BO_CPURWLOCK_ZERO_INITIALIZER {0,0,0,0,0}


typedef bo_cpulock32_t bo_cpusem32_t;
typedef bo_cpulock32_x bo_cpusem32_x;
typedef bo_cpulock64_t bo_cpusem64_t;
typedef bo_cpulock64_x bo_cpusem64_x;

# if (BO_M_WORD == BO_M_WORD32)
typedef bo_cpusem32_t bo_cpusem_t;
typedef bo_cpusem32_x bo_cpusem_x;
# else
# if (BO_M_WORD == BO_M_WORD64)
typedef bo_cpusem64_t bo_cpusem_t;
typedef bo_cpusem64_x bo_cpusem_x;
# endif
# endif


/* compat @{ */
typedef bo_cpulock_t    bo_llock_t;
typedef bo_cpulock_x    bo_llock_x;
typedef bo_cpurwlock_t  bo_lrwlock_t;

# define BO_LRWLOCK_ZERO_INITIALIZER BO_CPURWLOCK_ZERO_INITIALIZER
/* @} */

/* ------------------------------------------------------------------------- */

typedef ssize_t bo_index_t;
typedef int32_t bo_index_x;

typedef u_int32_t bo_opflags_t;
typedef u_int32_t bo_opflags_x;
/*
 * Common well known flags
 */
# define BoF_OP_FORCE 0x01


# ifndef __BO_HASH_TYPE__
#  define __BO_HASH_TYPE__ u_int32_t
# endif
typedef __BO_HASH_TYPE__ bo_hash_t;
typedef __BO_HASH_TYPE__ bo_hash_x;

/*
 * comparison
 */

# ifndef __BO_MISMATCH_TYPE__
#  define __BO_MISMATCH_TYPE__ int32_t
# endif
typedef __BO_MISMATCH_TYPE__ bo_mismatch_t;
typedef __BO_MISMATCH_TYPE__ bo_mismatch_x;

# define BoS_ECOMPARE ((bo_mismatch_t)0x8fffffff)

# define BoK_LESS_CMPOP 3
# define BoK_LESSEQ_CMPOP 4
# define BoK_EQU_CMPOP 5
# define BoK_GREATER_CMPOP 6
# define BoK_GREATEQ_CMPOP 7

/*
 * Ownership policy
 *
 * @{
 */

# define BoK_OWNS_MY_DATA 0
# define BoK_OWNS_YOUR_DATA 1
# define BoK_OWNS_COPY_DATA 2

# define _BoK_OWNS_YOUR_DATA BoK_OWNS_MY_DATA
# define _BoK_OWNS_MY_DATA BoK_OWNS_YOUR_DATA
# define _BoK_OWNS_COPY_DATA BoK_OWNS_COPY_DATA

typedef unsigned  bo_ownership_t;
typedef u_int8_t  bo_ownership_x;
/* @} */

/* ----------------------------------------------------------------------------
 * TIME
 *
 * @{
 */

/* @todo: fixme */
# ifdef HAS_INT64_TYPE

typedef u_int64_t  bo_tick_t;
typedef int64_t    bo_time_t;
typedef int64_t    bo_time_x;
#  define PRIbtick PRIu64
#  define PRIbtime PRIi64

#  define BoS_ETIME (~0LL)

#  define BoK_INVAL_TIME_INTERVAL (-2LL)
#  define BoK_INFINITE_TIME_INTERVAL (-1LL)
#  define BoK_NULL_TIME_INTERVAL (0LL)
#  define BoK_MAX_TIME_INTERVAL (0x7fffffffffffffffLL)
#  define BoK_INVAL_DURATION (-2LL)
#  define BoK_INFINITE_DURATION (-1LL)
#  define BoK_NULL_DURATION (0LL)
#  define BoK_MAX_DURATION (0x7fffffffffffffffLL)

# else /* !HAS_INT64_TYPE */

typedef u_int32_t  bo_tick_t;
typedef int32_t    bo_time_t;
typedef int32_t    bo_time_x;
#  define PRIbtick PRIu32
#  define PRIbtime PRIi32

#  define BoS_ETIME (~0L)

#  define BoK_INVAL_TIME_INTERVAL (-2)
#  define BoK_INFINITE_TIME_INTERVAL (-1)
#  define BoK_NULL_TIME_INTERVAL (0)
#  define BoK_MAX_TIME_INTERVAL (0x7fffffff)
#  define BoK_INVAL_DURATION (-2)
#  define BoK_INFINITE_DURATION (-1)
#  define BoK_NULL_DURATION (0)
#  define BoK_MAX_DURATION (0x7fffffff)

# endif /* HAS_INT64_TYPE */

# define BoK_INVAL_TS_INTERVAL ((struct timespec const *)(-2))
# define BoK_INFINITE_TS_INTERVAL ((struct timespec const *)(-1))
# define BoK_NULL_TS_INTERVAL ((struct timespec const *)0)
# define BoK_INVAL_TS_DURATION ((struct timespec const *)(-2))
# define BoK_INFINITE_TS_DURATION ((struct timespec const *)(-1))
# define BoK_NULL_TS_DURATION ((struct timespec const *)0)

/*
 * 1. poll strategy (relative duration) // extension: select-rel
 *		wait infinite: BoK_INFINITE_TS_DURATION
 *		no wait: NULL (BoK_NULL_TS_DURATION)
 * 2. select strategy (absolute timer) // extension: poll-abs
 *		wait infinite: NULL (BoK_NULL_TS_INTERVAL) or BoK_INFINITE_TS_INTERVAL
 *		no wait: TS = {0}
 */

/* The number of milliseconds since 0 hour January 1, 1970
   Coordinated Universal Time (UTC). */
typedef bo_time_t  bo_mtime_t;
typedef bo_time_x  bo_mtime_x;

/* The number of microseconds since 0 hour January 1, 1970
   Coordinated Universal Time (UTC). */
typedef bo_time_t  bo_utime_t;
typedef bo_time_x  bo_utime_x;

/* The number of nanoseconds since 0 hour January 1, 1970
   Coordinated Universal Time (UTC). */
typedef bo_time_t  bo_ntime_t;
typedef bo_time_x  bo_ntime_x;

typedef bo_time_t  bo_sec_t;  /* duration in seconds */
typedef bo_time_x  bo_sec_x;

typedef bo_time_t  bo_msec_t; /* duration in milliseconds */
typedef bo_time_x  bo_msec_x;

typedef bo_time_t  bo_usec_t; /* duration in microseconds */
typedef bo_time_x  bo_usec_x;

typedef bo_time_t  bo_nsec_t; /* duration in nanoseconds */
typedef bo_time_x  bo_nsec_x;
/* @} */

/* ----------------------------------------------------------------------------
 * UUID
 * @{
 */

struct bo_uuid {
	u_int8_t  data[16];
};

struct bo_uuid_s {
	u_int32_t  time_low;
	u_int16_t  time_mid,
	           time_hi_and_version;
	u_int8_t   clock_seq_hi_and_version,
	           clock_seq_low;
	u_int8_t   node[6];
};
# define bo_uuid_p(x) ((struct bo_uuid_s *)(x))

typedef struct bo_uuid_s  bo_uuid_t;

# define BO_UUID_Version(x) ((bo_uuid_p(x)->time_hi_and_version >> 12) & 0xf)

/* @} */

/* ----------------------------------------------------------------------------
 * result/error/status code
 *
 * @{
 */

/* @todo: fixme */
# ifdef HAS_INT64_TYPE
typedef int64_t  bo_status_t;
typedef int64_t  bo_status_x;
#  define PRIbsts PRIi64
# else
typedef int32_t  bo_status_t;
typedef int32_t  bo_status_x;
#  define PRIbsts PRIi32
# endif

# ifndef __BO_SYSERRCODE_TYPE__
#  define __BO_SYSERRCODE_TYPE__ int
# endif
typedef __BO_SYSERRCODE_TYPE__ syserrcode_t;


# ifndef __BO_ERRCODE_TYPE__
#  define __BO_ERRCODE_TYPE__ int32_t
# endif
typedef __BO_ERRCODE_TYPE__ bo_errcode_t;
typedef u_int16_t           bo_errcode_x;


typedef bo_status_t  bo_stsptr_t;
typedef bo_status_t  bo_stsbool_t;
typedef bo_status_t  bo_stsint_t;

/* ------------------------------------------------------------------------- */

/* pointer @{ */
# define BO_STATUS_FROM_PTR(aPtr)         ((bo_stsptr_t)(intptr_t)(aPtr))
# define BO_PTR_FROM_STATUS(aStatus)      BO_PTR_FROM_STATUS_(bo_pointer_t,aStatus)
# define BO_AOB_FROM_STATUS(aStatus)      BO_PTR_FROM_STATUS_(aorp_object_t,aStatus)
# define BO_STR_FROM_STATUS(aStatus)      BO_PTR_FROM_STATUS_(char *,aStatus)
# define BO_WCS_FROM_STATUS(aStatus)      BO_PTR_FROM_STATUS_(wchar_t *,aStatus)
# define BO_PTR_FROM_STATUS_(_Ty,aStatus) ((_Ty)(u_intptr_t)(aStatus))

# define BO_BADPTR(aPtr)                  (BoS_EPTR == (bo_pointer_t)(aPtr))
# define BO_ILLPTR(aPtr)                  (NULL == (bo_pointer_t)(aPtr) || BO_BADPTR(aPtr))
/* @} */

/* bool @{ */
# define BoS_ISTRUE(aStatus)              (BoS_TRUE <= (aStatus))
# define BoS_ISFALSE(aStatus)             (BoS_FALSE == (aStatus))
/* @} */

# define BoS_TEST(aStatus,aVal)           ((aVal) <= (aStatus))

# define BoS_DOBRE(aStatus)               BoS_TEST(aStatus,BoS_OK)
# define BoS_OKAY(aStatus)                BoS_DOBRE(aStatus)
# define BoS_SUCCESS(aStatus)             BoS_TEST(aStatus,BoS_NORMAL)
# define BoS_FAILURE(aStatus)             ((aStatus) <= BoS_ERR) /* (-1 == (aStatus)) */

/*
 * test status-pointer
 * non-portable
 * @{
 */
# define BoSP_NORMAL(aPtr) (!BoSP_BADPTR(aPtr))

# define BoSP_BADPTR(aPtr) \
	(BoS_LAST <= BO_STATUS_FROM_PTR(aPtr) && BO_STATUS_FROM_PTR(aPtr) < (bo_stsptr_t)0)

# define BoSP_ILLPTR(aPtr) \
	(BoS_LAST <= BO_STATUS_FROM_PTR(aPtr) && BO_STATUS_FROM_PTR(aPtr) <= (bo_stsptr_t)0)
/* @} */

/* no more data */
# define BoS_ISEOF(aStatus)     (BoS_EOF == (aStatus))

/* status test used when foreach, ... */
# define BoS_DALI(aStatus)      (0 < (aStatus))
# define BoS_HODI(aStatus)      (0 == (aStatus))
# define BoS_DOSYT(aStatus)     BoS_HODI(aStatus)
# define BoS_CONT(aStatus)      BoS_DALI(aStatus)
# define BoS_STOP(aStatus)      BoS_DOSYT(aStatus)


# define BoS_OK                 (1)
# define BoS_NORMAL             (0)
# define BoS_NOERR              BoS_NORMAL
# define BoS_TRUE               (1)
# define BoS_FALSE              (0)
# define BoS_EOF                (0)

# define BoS_ERR                (-1) /* generic error signal */
# define BoS_EPTR               ((bo_pointer_t)-1)
# define BoS_EAOB               ((aorp_object_t)-1)
# define BoS_ESTR               ((char *)-1)
# define BoS_EWCS               ((wchar_t *)-1)

# define BoS_NOTHANDLED         (-2) /* for option status: opt.status (see Optdefs.h) */

# define BoS_NOTIMPL            (-3) /* feature, functionality, ... */
# define BoS_NOTSUPP            (-4) /* feature, functionality, ... */
# define BoS_INTERRUPTED        (-5)
# define BoS_TIMEDOUT           (-6)
# define BoS_READONLY           (-7)
# define BoS_ALREADY            (-8)
# define BoS_LAST               (-10)

/* ------------------------------------------------------------------------- */

/* [error] status */
# define BoK_ERROR_DOMAIN_SHIFT 16
# define BoK_ERROR_DOMAIN_MASK 0xffff
# define BoK_ERROR_DOMAIN_MASK_S 0xffff0000
# define BoK_ERROR_CODE_MASK 0xffff

# define _BoSTATUS(aDomain,aCode) BO_MAKE_STATUS(aDomain,aCode)
# define BO_MAKE_STATUS(aDomain,aCode) \
	((((aDomain) & BoK_ERROR_DOMAIN_MASK) << BoK_ERROR_DOMAIN_SHIFT) | \
		((aCode) & BoK_ERROR_CODE_MASK))

# define BoSTATUS_DOMAIN(aStatus) \
	(((aStatus) >> BoK_ERROR_DOMAIN_SHIFT) & BoK_ERROR_DOMAIN_MASK)

# define BoSTATUS_SET_DOMAIN(aStatus,aDomain) \
	do { \
		(aStatus) &= ~BoK_ERROR_DOMAIN_MASK_S; \
		(aStatus) |= ((aDomain) & BoK_ERROR_DOMAIN_MASK) << BoK_ERROR_DOMAIN_SHIFT; \
	} while (0)

# define BoSTATUS_CODE(aStatus) \
	((bo_errcode_t)((aStatus) & BoK_ERROR_CODE_MASK))

# define BoSTATUS_SET_CODE(aStatus,aCode) \
	do { \
		(aStatus) &= ~BoK_ERROR_CODE_MASK; \
		(aStatus) |= ((aCode) & BoK_ERROR_CODE_MASK); \
	} while (0)

/* compat @{ */
# define BoSTATUS_ERRCODE(aStatus) BoSTATUS_CODE(aStatus)
# define BoSTATUS_SET_ERRCODE(aStatus,aCode) BoSTATUS_SET_CODE(aStatus,aCode)
/* @} */

/* @} */

/* ----------------------------------------------------------------------------
 * Sequential list
 * @{
 */

# if BO_M_WORD == BO_M_WORD32
#  define BoK_SEQLIST_SIZE 12
# else
#  define BoK_SEQLIST_SIZE 24
# endif

struct __MYC_QALIGN(SIZEOF_PTR) bo_seqlist_s {
	char tsl_opaque[BoK_SEQLIST_SIZE];
};
typedef struct bo_seqlist_s bo_seqlist_t;

# define bo_seqlist_(_Tp) bo_seqlist_t

# define BO_SEQLIST_ZERO_INITIALIZER {{0}}

/* @} */

/* ----------------------------------------------------------------------------
 * Circular double/single linked list
 * @{
 */

# if BO_M_WORD == BO_M_WORD32
#  define BoK_CDLLELEM_SIZE 8
#  define BoK_CDLLIST_SIZE 12
# else
#  define BoK_CDLLELEM_SIZE 16
#  define BoK_CDLLIST_SIZE 24
# endif

struct __MYC_QALIGN(SIZEOF_PTR) bo_cdllelem_s {
	char cdlle_opaque[BoK_CDLLELEM_SIZE];
};
typedef struct bo_cdllelem_s bo_cdllelem_t;

# define BO_CDLLELEM_ZERO_INITIALIZER {{0}}

struct __MYC_QALIGN(SIZEOF_PTR) bo_cdllist_s {
	char cdll_opaque[BoK_CDLLIST_SIZE];
};
typedef struct bo_cdllist_s bo_cdllist_t;

# define bo_cdllist_(_Tp) bo_cdllist_t

# define BO_CDLLIST_ZERO_INITIALIZER {{0}}

/* ------------------------------------------------------------------------- */

# if BO_M_WORD == BO_M_WORD32
#  define BoK_CSLLELEM_SIZE 4
#  define BoK_CSLLIST_SIZE 12
# else
#  define BoK_CSLLELEM_SIZE 8
#  define BoK_CSLLIST_SIZE 24
# endif

struct __MYC_QALIGN(SIZEOF_PTR) bo_csllelem_s {
	char cslle_opaque[BoK_CSLLELEM_SIZE];
};
typedef struct bo_csllelem_s bo_csllelem_t;

# define BO_CSLLELEM_ZERO_INITIALIZER {{0}}

struct __MYC_QALIGN(SIZEOF_PTR) bo_csllist_s {
	char csll_opaque[BoK_CSLLIST_SIZE];
};
typedef struct bo_csllist_s bo_csllist_t;

# define bo_csllist_(_Tp) bo_csllist_t

# define BO_CSLLIST_ZERO_INITIALIZER {{0}}

/* @} */

/* ----------------------------------------------------------------------------
 * B-tree
 * @{
 */

/*
 * Abstract b-tree
 */

# if BO_M_WORD == BO_M_WORD32
#  define BoK_BTELEM_OPAQUE_SIZE 10
# else
#  define BoK_BTELEM_OPAQUE_SIZE 18
# endif

struct __MYC_QALIGN(SIZEOF_PTR) bo_btelem_s {
	char          bte_opaque[BoK_BTELEM_OPAQUE_SIZE];
	u_int16_t     bte_ubits;
	bo_pointer_t  bte_user;
};
typedef struct bo_btelem_s bo_btelem_t;

# define BO_BTELEM_ZERO_INITIALIZER {{0}, 0, NULL}

/* ------------------------------------------------------------------------- */

/*
 * Right-threaded b-tree
 */

# if BO_M_WORD == BO_M_WORD32
#  define BoK_RTBTELEM_OPAQUE_SIZE 10
#  define BoK_RTBTREE_SIZE 16
# else
#  define BoK_RTBTELEM_OPAQUE_SIZE 18
#  define BoK_RTBTREE_SIZE 32
# endif

struct __MYC_QALIGN(SIZEOF_PTR) bo_rtbtelem_s {
	char          rtbte_opaque[BoK_RTBTELEM_OPAQUE_SIZE];
	u_int16_t     rtbte_ubits;
	bo_pointer_t  rtbte_user;
};
typedef struct bo_rtbtelem_s bo_rtbtelem_t;

# define BO_RTBTELEM_ZERO_INITIALIZER {{0}, 0, NULL}

struct __MYC_QALIGN(SIZEOF_PTR) bo_rtbtree_s {
	char rtbt_opaque[BoK_RTBTREE_SIZE];
};
typedef struct bo_rtbtree_s bo_rtbtree_t;

# define bo_rtbtree_(_Tp) bo_rtbtree_t

# define BO_RTBTREE_ZERO_INITIALIZER {{0}}

/* ------------------------------------------------------------------------- */

/*
 * Super right-threaded b-tree
 */

# if BO_M_WORD == BO_M_WORD32
#  define BoK_SRTBTELEM_OPAQUE1_SIZE 10
#  define BoK_SRTBTELEM_OPAQUE2_SIZE 8
#  define BoK_SRTBTREE_SIZE 32
# else
#  define BoK_SRTBTELEM_OPAQUE1_SIZE 18
#  define BoK_SRTBTELEM_OPAQUE2_SIZE 16
#  define BoK_SRTBTREE_SIZE 64
# endif

struct __MYC_QALIGN(SIZEOF_PTR) bo_srtbtelem_s {
	char          srtbte_opaque1[BoK_SRTBTELEM_OPAQUE1_SIZE];
	u_int16_t     srtbte_ubits;
	bo_pointer_t  srtbte_user;
	char          srtbte_opaque2[BoK_SRTBTELEM_OPAQUE2_SIZE];
};
typedef struct bo_srtbtelem_s bo_srtbtelem_t;

# define BO_SRTBTELEM_ZERO_INITIALIZER {{0}, 0, NULL, {0}}

struct __MYC_QALIGN(SIZEOF_PTR) bo_srtbtree_s {
	char srtbt_opaque[BoK_SRTBTREE_SIZE];
};
typedef struct bo_srtbtree_s bo_srtbtree_t;

# define bo_srtbtree_(_Tp) bo_srtbtree_t

# define BO_SRTBTREE_ZERO_INITIALIZER {{0}}

/* @} */

/* @todo: move */
/* ----------------------------------------------------------------------------
 * Dynamic library support
 * @{
 */

# ifdef __BO_DLSYSOBJ_TYPE__
typedef __BO_DLSYSOBJ_TYPE__ bo_dlsysobj_t;
typedef __BO_DLFUNC_TYPE__ bo_dlfunc_t;
typedef __BO_DLVAR_TYPE__ bo_dlvar_t;
# else
typedef __BO_VOID_PTR__ bo_dlsysobj_t;
typedef __BO_VOID_PTR__ bo_dlfunc_t;
typedef __BO_VOID_PTR__ bo_dlvar_t;
#  define __BO_INVAL_DLSYSOBJ NULL
# endif

# define BO_INVAL_DLSYSOBJ __BO_INVAL_DLSYSOBJ

/* @} */

/* @todo: move */
# ifndef HAS_SOCKLEN_TYPE
#  ifdef __BO_SOCKLEN_TYPE__
typedef __BO_SOCKLEN_TYPE__ socklen_t;
#  else
typedef u_int32_t socklen_t;
#  endif
# endif

#endif /* !__BO_SYS_TYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
