/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Wait.h	(S.Makhmali)
 * include/Bo/sys/Wait.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_WAIT__
# define __BO_SYS_WAIT__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# if __myc_exists(sys7wait,h)
#  include __MYC_LIBC_HEADER(sys/wait.h)
# endif

/******************************************************************************
 * %% BeginSection: definitions
 */

/*
 * Special cases of pid
 */

# ifdef WAIT_ANY
#  define BoK_WAIT_ANY_CHILD ((pid_t)WAIT_ANY)
# else
#  define BoK_WAIT_ANY_CHILD ((pid_t)-1)
# endif
# ifdef WAIT_MYPGRP
#  define BoK_WAIT_ANY_CLDMYPGRP ((pid_t)WAIT_MYPGRP)
# else
#  define BoK_WAIT_ANY_CLDMYPGRP ((pid_t)0)
# endif

/* compat */
# ifndef WAIT_ANY
#  define WAIT_ANY BoK_WAIT_ANY_CHILD
# endif
# ifndef WAIT_MYPGRP
#  define WAIT_MYPGRP BoK_WAIT_ANY_CLDMYPGRP
# endif

/*
 * Options
 */

# ifndef WHANG
#  define WHANG (0)
# endif
# ifndef WNOHANG
#  define WNOHANG (0x01)
# endif
/*
# ifndef WUNTRACED
#  define WUNTRACED (0x02)
# endif
 */


# if !__myc_exists(sys7wait,h)
/* windows?! */

#  define WEXITSTATUS(aStatus) ((aStatus) & 0xff)

#  define WIFEXITED(aStatus) (1)
#  define WIFSIGNALED(aStatus) (0)
#  define WIFSTOPPED(aStatus) (0)
#  define WIFCONTINUED(aStatus) (0)

#  define WSTOPSIG(aStatus) (0)
#  define WTERMSIG(aStatus) (0)
# endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */

struct timespec;

# include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ----------------------------------------------------------------------------
 * BoWait
 */
# pragma mark -

__myc_flinkage__(public,libBo) pid_t
__status__((
	BoS_SUCCESS = [ 0++ /* child process identifier */ = (); ];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_SysError_NoChildToWait);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
	];
))
BoWait(
# if __myc_require(prototypes)
	int *aProcStatusPtr,
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWaitpid
 */
# pragma mark -

__myc_flinkage__(public,libBo) pid_t
__status__((
	BoS_SUCCESS = [
		0 = (/* no process that wish to report status */);
		1++ = (/* child process identifier */);
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_SysError_NoChildToWait);
		BoS_NOTSUPP = (BoK_OpError_FeatureNotSupp);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
BoWaitpid(
# if __myc_require(prototypes)
	pid_t aWPid,
	int *aProcStatusPtr,
	int Options,
	struct timespec const *anAbstime __myc_defval__(NULL), /* unused if @options = WNOHANG */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWaitpid_rel
 */

__myc_flinkage__(public,libBo) pid_t
__status__((
	BoS_SUCCESS = [
		0 /* no process that wish to report status */ = ();
		1++ /* child process identifier */ = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_SysError_NoChildToWait);
		BoS_NOTSUPP = (BoK_OpError_FeatureNotSupp);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
BoWaitpid_rel(
# if __myc_require(prototypes)
	pid_t aWPid,
	int *aProcStatusPtr,
	int Options,
	struct timespec const *aDuration __myc_defval__(NULL), /* unused if @options = WNOHANG */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

/* ----------------------------------------------------------------------------
 * BoWaitpid_ms
 */

__myc_flinkage__(public,libBo) pid_t
__status__((
	BoS_SUCCESS = [
		0 /* no process that wish to report status */ = ();
		1++ /* child process identifier */ = ();
	];
	BoS_FAILURE = [
		BoS_ERR = (BoK_ArgError_BadAddress, BoK_ArgError_InvalidArgument,
		           BoK_SysError_NoChildToWait);
		BoS_NOTSUPP = (BoK_OpError_FeatureNotSupp);
		BoS_INTERRUPTED = (BoK_OpError_Interrupted);
		BoS_TIMEDOUT = (BoK_OpError_TimedOut);
	];
))
BoWaitpid_ms(
# if __myc_require(prototypes)
	pid_t aWPid,
	int *aProcStatusPtr,
	int Options,
	bo_msec_t aDuration, /* unused if @options = WNOHANG */
	struct aorp_error *anErrPtr __myc_defval__(NULL)
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_SYS_WAIT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
