/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _FOpenflags.h	(V.Sokur)
 * include/Bo/sys/_FOpenflags.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_FOPENFLAGS__
# define __BO_SYS_FOPENFLAGS__

/******************************************************************************
 * %% BeginSection: constant declarations / macro definitions
 */

/* fcntl interface */

/*
 * access flags
 */

/* mask for the modes below */
# ifndef O_ACCMODE
#  define O_ACCMODE (O_RDONLY | O_WRONLY | O_RDWR)
# endif

# ifndef O_RDONLY
#  ifdef _O_RDONLY
#   define O_RDONLY _O_RDONLY /* open for reading only */
#  endif
# endif
# ifndef O_WRONLY
#  ifdef _O_WRONLY
#   define O_WRONLY _O_WRONLY /* open for writing only */
#  endif
# endif
# ifndef O_RDWR
#  ifdef _O_RDWR
#   define O_RDWR _O_RDWR /* open for reading and writing */
#  endif
# endif

/* @todo: fixme */
# ifndef O_SHLOCK

#  ifdef HAS__SOPEN
#   ifdef _SH_DENYNO
#    define O_SHLOCK 0x01000000 /* open with shared or read file lock */
#    define O_EXLOCK 0x02000000 /* open with exclusive or write  file lock */
#   else
#    define O_SHLOCK 0
#    define O_EXLOCK 0
#   endif
#  else
#   define O_SHLOCK 0
#   define O_EXLOCK 0
#  endif

#  define O_SHLOCK_MYIMP
#  define O_EXLOCK_MYIMP
# endif /* !O_SHLOCK */

/*
 * other [open] flags
 */

/* no delay */
# ifndef O_NONBLOCK
#  ifdef O_NDELAY
#   define O_NONBLOCK O_NDELAY
#  else
#  ifdef _O_NONBLOCK
#   define O_NONBLOCK _O_NONBLOCK
#  endif
#  endif
# endif

# ifndef O_APPEND
#  ifdef _O_APPEND
#   define O_APPEND _O_APPEND /* set append mode; writes done at eof */
#  endif
# endif

/* compat for w32 */
# ifndef O_BINARY
#  ifdef _O_BINARY
#   define O_BINARY _O_BINARY
#  else
#   define O_BINARY 0
#  endif
# endif

/*
 * create flags
 */

# ifndef O_CREAT
#  ifdef _O_CREAT
#   define O_CREAT _O_CREAT /* create if nonexistant */
#  endif
# endif
# ifndef O_TRUNC
#  ifdef _O_TRUNC
#   define O_TRUNC _O_TRUNC /* truncate to zero length */
#  endif
# endif
# ifndef O_EXCL
#  ifdef _O_EXCL
#   define O_EXCL _O_EXCL /* error if already exists (when creating) */
#  endif
# endif

/* ------------------------------------------------------------------------- */

# ifndef O_NONBLOCK
#  define O_NONBLOCK 0x10000000
#  define O_NONBLOCK_MYIMP
# endif
/* compat */
# ifndef O_NDELAY
#  ifdef O_NONBLOCK
#   define O_NDELAY O_NONBLOCK
#  endif
# endif

/* CLOSE-ON-EXEC */
# ifndef O_NOINHERIT
#  ifdef _O_NOINHERIT
#   define O_NOINHERIT _O_NOINHERIT
#  else
#   define O_NOINHERIT 0x20000000
#   define O_NOINHERIT_MYIMP
#  endif
# endif

#endif /* !__BO_SYS_FOPENFLAGS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
