/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _FilePath.h	(N.Panasiuk)
 * include/Bo/sys/_FilePath.h
 *
 * $Id$
 */

/*
 * Parameters (examples):
 * _TCHAR                        char
 * _Prefix                       Win
 *
 * BoK_<PREFIX>_PATH_SEP         '\\'
 * BO_<PREFIX>_PATH_SEPS         "/\\"
 * BoK_<PREFIX>_SEARCH_PATH_SEP  ';'
 * BoK_<PREFIX>_EXT_SEP          '.'
 */

#define _F(_Name) __myc_xconcat3(Bo,_Prefix,_Name)

/******************************************************************************
 * %% BeginSection: includes
 */

#include <Bo/sys/Types.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */

#include <Bo/BoLinkage.h>

__MYC_BEGIN_CDECLS

/* ============================================================================
 * Utils
 */
#pragma mark -
#pragma mark [ Utils ]
#pragma mark -

__myc_flinkage__(public,libBo) bo_seqlist_(_TCHAR *) *
BoDestroyPathList(
#if __myc_require(prototypes)
	bo_seqlist_(_TCHAR *) *aList
#endif
	);

__myc_flinkage__(public,libBo) bo_seqlist_(_TCHAR *) *
BoClearPathList(
#if __myc_require(prototypes)
	bo_seqlist_(_TCHAR *) *aList
#endif
	);

/* ============================================================================
 * Separator predicates
 */
#pragma mark -
#pragma mark [ Separator predicates ]
#pragma mark -

__myc_flinkage__(public,libBo) _Bool
_F(IsPathSep)(
#if __myc_require(prototypes)
	int aChar
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(IsSearchPathSep)(
#if __myc_require(prototypes)
	int aChar
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(IsExtSep)(
#if __myc_require(prototypes)
	int aChar
#endif
	);

/* ============================================================================
 * $PATH methods
 */
#pragma mark -
#pragma mark [ $PATH methods ]
#pragma mark -

__myc_flinkage__(public,libBo) bo_seqlist_(_TCHAR *) *
__attribute__((nonnull(1)))
_F(SplitSearchPath)(
#if __myc_require(prototypes)
	bo_seqlist_(_TCHAR *) *aList,
	_TCHAR const *aSearchPath
#endif
	);

__myc_flinkage__(public,libBo) bo_seqlist_(_TCHAR *) *
__attribute__((nonnull(1)))
_F(GetSearchPath)(
#if __myc_require(prototypes)
	bo_seqlist_(_TCHAR *) *aList
#endif
	);

/* ============================================================================
 * Extension functions
 */
#pragma mark -
#pragma mark [ Extension functions ]
#pragma mark -

__myc_flinkage__(public,libBo) void
_F(SplitExtension)(
#if __myc_require(prototypes)
	_TCHAR **aRestPtr,
	_TCHAR **anExtPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(TakeExtension)(
#if __myc_require(prototypes)
	_TCHAR **anExtPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(ReplaceExtension)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath,
	_TCHAR const *aNewExtension
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(DropExtension)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(AddExtension)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath,
	_TCHAR const *anExtension
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(HasExtension)(
#if __myc_require(prototypes)
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) void
_F(SplitExtensions)(
#if __myc_require(prototypes)
	_TCHAR **aRestPtr,
	_TCHAR **anExtPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(TakeExtensions)(
#if __myc_require(prototypes)
	_TCHAR **anExtPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(DropExtensions)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath
#endif
	);

/* ============================================================================
 * Filename/directory functions
 */
#pragma mark -
#pragma mark [ Filename/directory functions ]
#pragma mark -

__myc_flinkage__(public,libBo) void
_F(SplitFileName)(
#if __myc_require(prototypes)
	_TCHAR **aRestPtr,
	_TCHAR **aFileNamePtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(TakeFileName)(
#if __myc_require(prototypes)
	_TCHAR **aFileNamePtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(ReplaceFileName)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath,
	_TCHAR const *aNewFileName
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(DropFileName)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(TakeBaseName)(
#if __myc_require(prototypes)
	_TCHAR **aBaseNamePtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(ReplaceBaseName)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath,
	_TCHAR const *aNewBaseName
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(TakeDirectory)(
#if __myc_require(prototypes)
	_TCHAR **aDirectoryPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(ReplaceDirectory)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath,
	_TCHAR const *aNewDirectory
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(CombinePaths)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aLhs,
	_TCHAR const *aRhs
#endif
	);

__myc_flinkage__(public,libBo) bo_seqlist_(_TCHAR *) *
__attribute__((nonnull(1)))
_F(SplitPath)(
#if __myc_require(prototypes)
	bo_seqlist_(_TCHAR *) *aList,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(JoinPath)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *anElements[],
	size_t anElementc
#endif
	);

__myc_flinkage__(public,libBo) bo_seqlist_(_TCHAR *) *
__attribute__((nonnull(1)))
_F(SplitDirectories)(
#if __myc_require(prototypes)
	bo_seqlist_(_TCHAR *) *aList,
	_TCHAR const *aFilePath
#endif
	);

/* ============================================================================
 * Drive functions
 */
#pragma mark -
#pragma mark [ Drive functions ]
#pragma mark -

__myc_flinkage__(public,libBo) void
_F(SplitDrive)(
#if __myc_require(prototypes)
	_TCHAR **aDrivePtr,
	_TCHAR **aRestPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(JoinDrive)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aDrive,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(TakeDrive)(
#if __myc_require(prototypes)
	_TCHAR **aDrivePtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(HasDrive)(
#if __myc_require(prototypes)
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(DropDrive)(
#if __myc_require(prototypes)
	_TCHAR **aRestPathPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(IsDrive)(
#if __myc_require(prototypes)
	_TCHAR const *aFilePath
#endif
	);

/* ============================================================================
 * Trailing slash functions
 */
#pragma mark -
#pragma mark [ Trailing slash functions ]
#pragma mark -

__myc_flinkage__(public,libBo) _Bool
_F(HasTrailingPathSep)(
#if __myc_require(prototypes)
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(AddTrailingPathSep)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(DropTrailingPathSep)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath
#endif
	);

/* ============================================================================
 * File name manipulations
 */
#pragma mark -
#pragma mark [ File name manipulations ]
#pragma mark -

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(NormalisePath)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(EqualFilePath)(
#if __myc_require(prototypes)
	_TCHAR const *aLhs,
	_TCHAR const *aRhs
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(MakeRelativePath)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aLhs,
	_TCHAR const *aRhs
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(IsRelativePath)(
#if __myc_require(prototypes)
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(IsAbsolutePath)(
#if __myc_require(prototypes)
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _Bool
_F(IsValidPath)(
#if __myc_require(prototypes)
	_TCHAR const *aFilePath
#endif
	);

__myc_flinkage__(public,libBo) _TCHAR **
__attribute__((nonnull(1)))
_F(MakeValidPath)(
#if __myc_require(prototypes)
	_TCHAR **aFilePathPtr,
	_TCHAR const *aFilePath
#endif
	);

__MYC_END_CDECLS

#undef _F
#undef _TCHAR
#undef _Prefix

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
