/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Stddef.h	(V.Sokur)
 * include/Bo/sys/_Stddef.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS__STDDEF__
# define __BO_SYS__STDDEF__

/******************************************************************************
 * %% BeginSection: includes
 */

# if __myc_exists(stddef,h)
#  include __MYC_LIBC_HEADER(stddef.h)
# endif

/******************************************************************************
 * %% BeginSection: types
 */

/*
 * Assume ptrdiff_t and size_t are defined in <stddef.h>
 */

# ifndef HAS_PTRDIFF_TYPE
#  ifndef __PTRDIFF_TYPE_DEFINED__
#   define __PTRDIFF_TYPE_DEFINED__
typedef long ptrdiff_t;
#  endif
# endif

# ifndef HAS_SIZE_TYPE
#  ifndef __SIZE_TYPE_DEFINED__
#   define __SIZE_TYPE_DEFINED__
typedef unsigned long size_t;
#  endif
# endif

# ifndef HAS_SSIZE_TYPE
#  ifndef __SSIZE_TYPE_DEFINED__
#   define __SSIZE_TYPE_DEFINED__
typedef long ssize_t;
#  endif
# endif


# if __myc_support_type(void_ptr)
#  ifndef __BO_VOID_PTR__
#   define __BO_VOID_PTR__ void *
#  endif
# endif
# ifdef __STDC__
#  ifndef __BO_VOID_PTR__
#   define __BO_VOID_PTR__ void *
#  endif
# endif
# ifdef __cplusplus
#  ifndef __BO_VOID_PTR__
#   define __BO_VOID_PTR__ void *
#  endif
# endif
# ifndef __BO_VOID_PTR__
#  define __BO_VOID_PTR__ char *
# endif

/******************************************************************************
 * %% BeginSection: constants
 */

# ifndef YES
#  define YES 1
# endif
# ifndef NO
#  define NO 0
# endif


# ifndef NULL
#  ifndef __cplusplus
#   define NULL	((__BO_VOID_PTR__)0)
#  else
#   define NULL	(0)
#  endif
# endif

# ifndef NULLF
#  ifndef __cplusplus
#   ifdef _WIN32
#    define NULLF	(0)
#   else
#    define NULLF	NULL /* ((void (*)())0) */
#   endif
#  else
#   define NULLF	(0)
#  endif
# endif


# ifndef FAM_LENGTH
#  ifdef __cplusplus
#   define FAM_LENGTH	1
#  else
#  if __myc_support(FAM)
#   define FAM_LENGTH	/* 0 */
#  else
#   define FAM_LENGTH	1
#  endif /* __myc_support(FAM) */
#  endif /* __cplusplus */
# endif /* FAM_LENGTH */

/******************************************************************************
 * %% BeginSection: std. definitions
 */

typedef u_int32_t bo_version_t;

# define _BO_MKVERSION(aMajor,aMinor,aTeeny) \
	((((u_int32_t)(aMajor) & 0xff) << 24) | \
		(((u_int32_t)(aMinor) & 0xff) << 16) | \
		((u_int32_t)(aTeeny) & 0xffff))

# define _BO_MKSVERSION(aMajor,aMinor,aTeeny) \
	(__myc_xstringify(aMajor) "." __myc_xstringify(aMinor) "." __myc_xstringify(aTeeny))

/* compat @{ */
# define _BoMAKEVERSION(aMajor,aMinor,aTeeny) _BO_MKVERSION(aMajor,aMinor,aTeeny)
# define _BoMAKESTRVERSION(aMajor,aMinor,aTeeny) _BO_MKSVERSION(aMajor,aMinor,aTeeny)
/* @} */

# ifdef offsetof
#  define __myc_offsetof(_Type,_Field)	offsetof(_Type,_Field)
# else
#  define __myc_offsetof(_Type,_Field) \
	((size_t) ((char *)(&((_Type *)0)->_Field) - ((char *)0)))
# endif
# ifndef offsetof
#  define offsetof(_Type,_Field)	__myc_offsetof(_Type,_Field)
# endif

# define __myc_membaddrof(_Field,anObj) (&(anObj)->_Field)
# define __myc_membaddrof3(_Type,_Field,anObj) (&((_Type *)(anObj))->_Field)

# define __myc_membsizeof(_Type,_Field) (sizeof(((_Type *)0)->_Field))

# define __myc_objectof(_Type,_Field,aPtr) \
	((_Type *)((char*)(aPtr) - __myc_offsetof(_Type,_Field)))


# define ___myc_namembs(a) ((sizeof a) / (sizeof a[0]))
# define __myc_namembs(a) ___myc_namembs(a)

# define __myc_roundup(x, y) ((((x) + ((y) - 1)) / (y)) * (y))


/* @todo: fixme */
# define __myc_align(aValue,anAlign) \
	(((aValue) + ((anAlign) - 1L)) & ~((anAlign) - 1L))

# define __myc_align_ptr(aValue,anAlign) \
	((bo_pointer_t) (((u_intptr_t)(aValue) + ((anAlign) - 1L)) & ~((anAlign) - 1L)))

# define __myc_align_equ(aValue,anAlign) \
	((aValue) = __myc_align(aValue, anAlign))

# ifndef __cplusplus
#  define __myc_align_ptr_equ(aValue,anAlign) \
	((aValue) = __myc_align_ptr(aValue, anAlign))
# else
#  define __myc_align_ptr_equ(aValue,anAlign) \
	(*(bo_pointer_t *)&(aValue) = __myc_align_ptr(aValue, anAlign))
# endif


// @todo: move to Cdefs.h???
// @todo: fixme
# if __myc_support(alignof)
#  define __myc_alignof(_Type) alignof(_Type)
# else
# ifdef TYPE_ALIGNMENT
#  define __myc_alignof(_Type) TYPE_ALIGNMENT(_Type)
# else
#  define __myc_alignof(_Type) \
	__myc_offsetof(struct { char x_; _Type t_; }, t_)
# endif /* TYPE_ALIGNMENT */
# endif /* __myc_support(alignof) */


# ifndef assert
#  define assert(x) ((void)0)
# endif

# define __myc_ct_assert(aName,anExpr) \
	typedef char __myc_xconcat(MYC_CT_ASSERT,aName)[(anExpr) ? 1 : -1]

# ifdef _NO_MYC_CT_ASSERT
#  undef __MYC_CT_ASSERT
#  define __MYC_CT_ASSERT(aName,anExpr)
# else
#  ifndef __MYC_CT_ASSERT
#   ifdef _DEBUG
#    define __MYC_CT_ASSERT(aName,anExpr) __myc_ct_assert(aName,anExpr)
#   else
#    define __MYC_CT_ASSERT(aName,anExpr) struct __myc_ct_assert_hack_s
#   endif
#  endif
# endif


/* compat @{ */
# define bo_offsetof(aType,aField) __myc_offsetof(aType,aField)
# define bo_objectof(aPtr,aType,aField) __myc_objectof(aType,aField,aPtr)
# define bo_membaddrof(anObj,aField) __myc_membaddrof(aField,anObj)
# define bo_namembs(a) __myc_namembs(a)

# define __myc_objectof_compat_0_1(aPtr,aType,aField) __myc_objectof(aType,aField,aPtr)
# define __myc_membaddrof_compat_0_1(anObj,aField) __myc_membaddrof(aField,anObj)
/* @} */

/******************************************************************************
 * %% BeginSection: misc. definitions
 */

# define __myc_defvar_VLA(_Type,aName,anElemc,anAltElemc) \
	_Type __myc_var_VLA(aName,anElemc,anAltElemc)

# if __myc_support(VLA)
#  define __myc_var_VLA(aName,anElemc,anAltElemc) \
	aName[(anElemc)]; \
	enum { __myc_xconcat3(k_,aName,_is_static) = 0 }
# else
# ifdef HAS_ALLOCA
#  define __myc_var_VLA(aName,anElemc,anAltElemc) \
	*aName = alloca((anElemc) * (sizeof *aName)); \
	enum { __myc_xconcat3(k_,aName,_is_static) = 0 }
# else
#  define __myc_var_VLA(aName,anElemc,anAltElemc) \
	aName[(anAltElemc)]; \
	int __myc_xconcat2(aName,_fixup) = (assert(((anElemc) <= (anAltElemc))), 1); \
	enum { __myc_xconcat3(k_,aName,_is_static) = 1 }
# endif /* !HAS_ALLOCA */
# endif /* !__myc_support(VLA) */

/******************************************************************************
 * %% BeginSection: macro-functions
 */

# define __myc_mk4cc(c1,c2,c3,c4) __myc_make4cc(c1,c2,c3,c4)
# define __myc_mk4ccs(S) __myc_mk4cc(S[0],S[1],S[2],S[3])

# if (BO_BYTE_ORDER == LITTLE_BYTE_ORDER)
#  define __myc_make4cc(c1,c2,c3,c4) \
	( (u_int32_t)(unsigned char)(c1) \
		| ((u_int32_t)(unsigned char)(c2) << 8) \
		| ((u_int32_t)(unsigned char)(c3) << 16) \
		| ((u_int32_t)(unsigned char)(c4) << 24) )
# else
# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
#  define __myc_make4cc(c1,c2,c3,c4) \
	( (u_int32_t)(unsigned char)(c4) \
		| ((u_int32_t)(unsigned char)(c3) << 8) \
		| ((u_int32_t)(unsigned char)(c2) << 16) \
		| ((u_int32_t)(unsigned char)(c1) << 24) )
# else
#  error Unsupported byte order!!!
# endif
# endif


# define __myc_bcd2dec(x) (((x) >> 4) * 10 + ((x) & 0xf))
# define __myc_dec2bcd(x) (((x) / 10) * 16 + ((x) % 10))


# ifndef __myc_max
#  define __myc_max(x,y) (((x) < (y)) ? (y) : (x))
# endif
# ifndef __myc_min
#  define __myc_min(x,y) (((x) < (y)) ? (x) : (y))
# endif
# ifndef __myc_middle
#  define __myc_middle(x,y) (((x) + (y)) / 2)
# endif

# ifndef __myc_abs
#  define __myc_abs(x) (((x) < 0) ? -(x) : (x))
# endif


# ifndef __myc_swap3
#  define __myc_swap3(_Type,x,y) \
    do { \
        _Type boswapTmp = x; x = y; y = boswapTmp; \
    } while (0)
# endif

# ifndef __myc_swap
#  define __myc_swap(x,y) __myc_swap3(typeof(x),x,y)
# endif

# ifndef __myc_swap_i
#  define __myc_swap_i(x,y) (x ^= y, y ^= x, x ^= y)
# endif

# ifndef __myc_swap_p
#  if __myc_support_keyword(typeof)
#   define __myc_swap_p(x,y) __myc_swap(x,y)
#  else
#   define __myc_swap_p(x,y) \
    do { \
        __BO_VOID_PTR__ bopswapTmp = (__BO_VOID_PTR__)x; \
        x = (__BO_VOID_PTR__)y; \
        y = (__BO_VOID_PTR__)bopswapTmp; \
    } while (0)
#  endif
# endif


# define __myc_make_i32_4i8(b4,b3,b2,b1) \
	((u_int32_t) ((((b4) & 0xff) << 24) | (((b3) & 0xff) << 16) | (((b2) & 0xff) << 8) | ((b1) & 0xff)))

# define __myc_make_i32_2i8(b2,b1) \
	((u_int32_t) ((((b2) & 0xff) << 8) | ((b1) & 0xff)))

# define __myc_make_i16_2i8(b2,b1) \
	((u_int16_t) ((((b2) & 0xff) << 8) | ((b1) & 0xff)))


/* compat @{ */
# define _BoMake4CC __myc_make4cc
# define _BoMax __myc_max
# define _BoMin __myc_min
# define _BoMiddle __myc_middle
# define _BoAbs __myc_abs
# define _BoSwap3 __myc_swap3
# define _BoSwap __myc_swap
# define _BoSwap_i __myc_swap_i
# define _BoSwap_p __myc_swap_p
/* @} */

#endif /* !__BO_SYS__STDDEF__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
