/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Stdint.h	(V.Sokur)
 * include/Bo/sys/_Stdint.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS__STDINT__
# define __BO_SYS__STDINT__

/******************************************************************************
 * %% BeginSection: includes
 */

# if __myc_exists(stdint,h)
#  include __MYC_LIBC_HEADER(stdint.h)
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# ifndef HAS_INTPTR_TYPE
#  if (SIZEOF_INT < SIZEOF_PTR)
#   if (SIZEOF_LONG < SIZEOF_PTR)
typedef __BO_VOID_PTR__ intptr_t;
#   else
typedef long intptr_t;
#   endif
#  else
typedef int intptr_t;
#  endif
#  define HAS_INTPTR_TYPE
# endif

# ifndef HAS_UINTPTR_TYPE
#  if (SIZEOF_INT < SIZEOF_PTR)
#   if (SIZEOF_LONG < SIZEOF_PTR)
typedef __BO_VOID_PTR__ uintptr_t;
#   else
typedef unsigned long uintptr_t;
#   endif
#  else
typedef unsigned int uintptr_t;
#  endif
# endif

# ifndef HAS_U_INTPTR_TYPE
typedef uintptr_t u_intptr_t;
#  define HAS_U_INTPTR_TYPE
# endif


# ifndef HAS_INT8_TYPE
typedef char int8_t;
#  define HAS_INT8_TYPE
# endif

# ifndef HAS_UINT8_TYPE
#  ifdef HAS_U_INT8_TYPE
typedef u_int8_t uint8_t;
#  else
typedef unsigned int8_t uint8_t;
#  endif
# endif

# ifndef HAS_U_INT8_TYPE
#  ifdef HAS_UINT8_TYPE
typedef uint8_t u_int8_t;
#  else
typedef unsigned int8_t u_int8_t;
#  endif
#  define HAS_U_INT8_TYPE
# endif

# ifndef HAS_INT16_TYPE
typedef short int16_t;
#  define HAS_INT16_TYPE
# endif

# ifndef HAS_UINT16_TYPE
#  ifdef HAS_U_INT16_TYPE
typedef u_int16_t uint16_t;
#  else
typedef unsigned int16_t uint16_t;
#  endif
# endif

# ifndef HAS_U_INT16_TYPE
#  ifdef HAS_UINT16_TYPE
typedef uint16_t u_int16_t;
#  else
typedef unsigned int16_t u_int16_t;
#  endif
#  define HAS_U_INT16_TYPE
# endif

# ifndef HAS_INT32_TYPE
typedef long int32_t;
#  define HAS_INT32_TYPE
# endif

# ifndef HAS_UINT32_TYPE
#  ifdef HAS_U_INT32_TYPE
typedef u_int32_t uint32_t;
#  else
typedef unsigned int32_t uint32_t;
#  endif
# endif

# ifndef HAS_U_INT32_TYPE
#  ifdef HAS_UINT32_TYPE
typedef uint32_t u_int32_t;
#  else
typedef unsigned int32_t u_int32_t;
#  endif
#  define HAS_U_INT32_TYPE
# endif

# ifndef HAS_INT64_TYPE
#  if __myc_support_type(long_long)
typedef long long int64_t;
#   define HAS_INT64_TYPE
#  endif
# endif

# ifndef HAS_UINT64_TYPE
#  ifdef HAS_U_INT64_TYPE
typedef u_int64_t uint64_t;
#  else
#  ifdef HAS_INT64_TYPE
typedef unsigned int64_t uint64_t;
#  else
#  if __myc_support_type(long_long)
typedef unsigned long long uint64_t;
#  endif /* __myc_support_type(long_long) */
#  endif /* HAS_INT64_TYPE */
#  endif /* HAS_U_INT64_TYPE */
# endif /* !HAS_UINT64_TYPE */

# ifndef HAS_U_INT64_TYPE
#  ifdef HAS_UINT64_TYPE
typedef uint64_t u_int64_t;
#   define HAS_U_INT64_TYPE
#  else
#  ifdef HAS_INT64_TYPE
typedef unsigned int64_t u_int64_t;
#   define HAS_U_INT64_TYPE
#  else
#  if __myc_support_type(long_long)
typedef unsigned int64_t u_int64_t;
#   define HAS_U_INT64_TYPE
#  endif /* __myc_support_type(long_long) */
#  endif /* HAS_INT64_TYPE */
#  endif /* HAS_UINT64_TYPE */
# endif /* !HAS_U_INT64_TYPE */

#endif /* !__BO_SYS__STDINT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
