/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _crt.h	(V.Sokur)
 * include/Bo/sys/crt/_crt.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_CRT__CRT__
# define __BO_SYS_CRT__CRT__

/******************************************************************************
 * %% BeginSection: includes
 */

/*

FUNCTIONS:

static bo_status_t __bo_crt_init()
static void __bo_crt_fini()

 */

# include "Bo/sys/crt/crtaction.h"

# include "Bo/Crtdbg.h"

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

static bo_status_t __bo_crt_init(
# if __myc_require(prototypes)
	void
# endif
	);

static void __bo_crt_fini(
# if __myc_require(prototypes)
	void
# endif
	);

# define _S_bo_crtaction_atenter (_t_bo_crtaction_atenter())
# define _S_bo_crtaction_atexit (_t_bo_crtaction_atexit())

/******************************************************************************
 * %% BeginSection: function definitions
 */
# pragma mark -
# pragma mark [ function definitions ]
# pragma mark -

# include "Bo/fundis/Rtbtree.h"
# include "Bo/fundis/Cdllist.h"

static volatile struct bo_rtbtree *
_t_bo_crtaction_atenter()
{
	static volatile struct bo_rtbtree _s_bo_crtaction_atenter;
	static volatile _Bool _s_inited = false;

	if (!_s_inited) {
		_BO_RTBTREE_Init(&_s_bo_crtaction_atenter);
		_s_inited = true;
	}
	return &_s_bo_crtaction_atenter;
}

static volatile struct bo_cdllist *
_t_bo_crtaction_atexit()
{
	static volatile struct bo_cdllist _s_bo_crtaction_atexit;
	static volatile _Bool _s_inited = false;

	if (!_s_inited) {
		_BO_CDLLIST_Init(&_s_bo_crtaction_atexit);
		_s_inited = true;
	}
	return &_s_bo_crtaction_atexit;
}

static volatile _Bool _S_bo_crtaction_processed = false;

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

# define MY_BTREE_Cmpuserinfo(aNodeLhs,aNodeInfoLhs,aNodeRhs,anUserInfoRhs) \
	( ((__bo_crtaction_t const *)(aNodeInfoLhs))->crta_priority - \
			((__bo_crtaction_t const *)(anUserInfoRhs))->crta_priority )

# include <assert.h>
# define NEED_MY_RTBTREE_INSERTNODE_B_DECL1
# include "Bo/fundis/btreetmpl/Rtbtins_b.h"

# undef MY_BTREE_Cmpuserinfo

__MYC_BEGIN_CDECLS

__myc_flinkage_shlib(private) int
__bo_crtaction(
#if __myc_require(prototypes)
	struct __bo_crtaction *anAction
#else
	anAction
#endif
	)
#if !__myc_require(prototypes)
	struct __bo_crtaction *anAction;
#endif
{
	struct bo_btree_node *node;

	node = bo_btree_node_p(&anAction->crta_btelem);
	node->btn_info = anAction;
	MY_RTBTREE_InsertNode_b(bo_rtbtree_p(_S_bo_crtaction_atenter), node, true);

	_BoCrtDbg_Printf(
		"%s: %s:%d: { %s(init), %s(fini) }\n", __func__,
		anAction->crta_file, anAction->crta_line,
		anAction->crta_initfunc_name, anAction->crta_finifunc_name
	);

	return (BoS_NORMAL);
}

__MYC_END_CDECLS

/* ------------------------------------------------------------------------- */
# pragma mark -
/* ------------------------------------------------------------------------- */

static bo_status_t __bo_crt_init()
{
	bo_status_t status;

	_BoCrtDbg_EnterFunc();

	status = BoS_NORMAL;
	if (!_S_bo_crtaction_processed) {
		__bo_crtaction_t *act;

		_S_bo_crtaction_processed = true;

		_BO_RTBTREE_FOREACH_LPR_(_S_bo_crtaction_atenter)
			act = _BO_RTBTREE_ITER_Deref(_$1);
			assert(NULL != act);

			_BoCrtDbg_Printf(
				"%s: call %s:%d:%s\n", __func__,
				act->crta_file, act->crta_line,
				act->crta_initfunc_name
			);

			if (NULLF != act->crta_initfunc) {
				status = (*act->crta_initfunc)(act->crta_initparm);
				if (BoS_FAILURE(status)) {
					break;
					/* NOTREACHED */
				}
				BO_CDLLIST_PushFront(__bo_crtaction_t, crta_lelem, _S_bo_crtaction_atexit, act);
			}
		FOREACH_END
	}

	_BoCrtDbg_LeaveFunc(status);

	return (status);
}

static void __bo_crt_fini()
{
	_BoCrtDbg_EnterFunc();

	if (_S_bo_crtaction_processed) {
		__bo_crtaction_t *act;

		_S_bo_crtaction_processed = false;

		BO_CDLLIST_FOREACH(__bo_crtaction_t, crta_lelem, act, _S_bo_crtaction_atexit)
			_BoCrtDbg_Printf(
				"%s: call %s:%d:%s\n", __func__,
				act->crta_file, act->crta_line,
				act->crta_finifunc_name
			);
			if (NULLF != act->crta_finifunc) {
				(void)(*act->crta_finifunc)(act->crta_finiparm);
			}
		FOREACH_END
	}

	_BoCrtDbg_LeaveFunc0();

	return;
}

#endif /* !__BO_SYS_CRT__CRT__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
