/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _crt_shlib.h	(V.Sokur)
 * include/Bo/sys/crt/_crt_shlib.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_CRT__CRT_SHLIB__
# define __BO_SYS_CRT__CRT_SHLIB__

/******************************************************************************
 * %% BeginSection: includes
 */

/*

REQUIREMENTS:

bo_status_t
ShlibInit(OS-SPECIFIC-PARAMETER-LIST)

bo_status_t
ShlibFini(OS-SPECIFIC-PARAMETER-LIST)

WIN32:
	__myc_flinkage_shlib(private) bo_status_t __cdecl ShlibInit(HMODULE aModuleHandle);
	__myc_flinkage_shlib(private) bo_status_t __cdecl ShlibFini(HMODULE aModuleHandle);

UNIX-GCC:
	__myc_flinkage_shlib(private) bo_status_t ShlibInit();
	__myc_flinkage_shlib(private) bo_status_t ShlibFini();

 */

# include "Bo/sys/crt/_crt.h"

# include <Bo/libc/Stdlib.h>
	/* abort, _onexit */

/******************************************************************************
 * %% BeginSection: user function prototypes
 */
# pragma mark -
# pragma mark [ user function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

__myc_flinkage_shlib(private) bo_status_t
__myc_calltype__(cdecl)
ShlibInit();

__myc_flinkage_shlib(private) bo_status_t
__myc_calltype__(cdecl)
ShlibFini();

__MYC_END_CDECLS

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

static int
__MYC_CRT_INIT_FUN_(__MYC_CDTOR_KPRIO)
_t_ShlibInit(
# if __myc_require(prototypes)
	void
# endif
	);

__MYC_CRT_LIBINIT_SECTION_IFNOACTOR(_t_ShlibInit)

# ifdef BO_SYSTYPE_WIN32
# ifdef _DLL
#  define _t_ShlibFini_RETTYPE int
# endif
# endif
# ifndef _t_ShlibFini_RETTYPE
#  define _t_ShlibFini_RETTYPE void
# endif

static _t_ShlibFini_RETTYPE
__MYC_CRT_FINI_FUN_(__MYC_CDTOR_KPRIO)
_t_ShlibFini(
# if __myc_require(prototypes)
	void
# endif
	);

# ifdef BO_SYSTYPE_WIN32
# ifndef _DLL
__MYC_CRT_LIBFINI_SECTION_IFNOACTOR(_t_ShlibFini)
# endif
# endif

/******************************************************************************
 * %% BeginSection: function definitions
 */
# pragma mark -
# pragma mark [ function definitions ]
# pragma mark -

#ifdef BO_SYSTYPE_WIN32
static HMODULE _S_module_handle = NULL;
static HMODULE _t_get_module_handle()
{
	if (NULL == _S_module_handle)
		_S_module_handle = GetModuleHandle(NULL);
	return (_S_module_handle);
}
# define _S_module_handle _t_get_module_handle()
#else
# define _S_module_handle
#endif

static int
__MYC_CRT_INIT_FUN_(__MYC_CDTOR_KPRIO)
_t_ShlibInit()
{
	bo_status_t status;

	status = __bo_crt_init();
	if (BoS_FAILURE(status)) {
		abort();
		/* NOTREACHED */
	}
	status = ShlibInit(_S_module_handle);
	if (BoS_FAILURE(status)) {
		abort();
		/* NOTREACHED */
	}

# ifdef BO_SYSTYPE_WIN32
# ifdef _DLL
	_onexit(_t_ShlibFini);
# endif
# endif

	return (BoS_NORMAL);
}

static _t_ShlibFini_RETTYPE
__MYC_CRT_FINI_FUN_(__MYC_CDTOR_KPRIO)
_t_ShlibFini()
{
	bo_status_t status;

	status = ShlibFini(_S_module_handle);
	__bo_crt_fini();

# ifdef BO_SYSTYPE_WIN32
# ifdef _DLL
	return (0);
# endif
# endif
}

#undef _S_module_handle

#endif /* !__BO_SYS_CRT__CRT_SHLIB__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
