/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * crtaction.h	(V.Sokur)
 * include/Bo/sys/crt/crtaction.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_CRT_CRTACTION__
# define __BO_SYS_CRT_CRTACTION__

/******************************************************************************
 * %% BeginSection: includes
 */

/*

USER FUNCTIONS:

INIT-FUNC:
static int
__myc_calltype__(mcall)
some-init-func(
# if __myc_require(prototypes)
	bo_pointer_t aParm
# endif
	);

FINI-FUNC:
static void
__myc_calltype__(mcall)
some-fini-func(
# if __myc_require(prototypes)
	bo_pointer_t aParm
# endif
	);

MACRODEFS:

BO_CRTACTION(aPriority,anInitFunc,anInitParm,anFiniFunc,anFiniParm)

 */

# include <Bo/sys/crt/crtdefs.h>

/******************************************************************************
 * %% BeginSection: definitions
 */
# pragma mark -
# pragma mark [ definitions ]
# pragma mark -

# define BO_CRTACTION(aPriority,anInitFunc,anInitParm,aFiniFunc,aFiniParm) \
	_im_BO_CRTACTION(__myc_xconcat(_S_bo_crtaction,__LINE__),aPriority,anInitFunc,anInitParm,aFiniFunc,aFiniParm)


# ifdef _BO_CRT_DEBUG0
#  define _im_BO_CRTACTION_DEBUG0_SPEC1_INITIALIZER(aFile,aLine,anInitFuncName,aFiniFuncName) \
	,aFile,aLine,anInitFuncName,aFiniFuncName
# else
#  define _im_BO_CRTACTION_DEBUG0_SPEC1_INITIALIZER(aFile,aLine,anInitFuncName,aFiniFuncName) \
	,"",0,"(null)","(null)"
# endif

# ifndef _im_BO_CRTACTION
#  define _im_BO_CRTACTION(aVar,aPriority,anInitFunc,anInitParm,aFiniFunc,aFiniParm) \
		static volatile \
		struct __bo_crtaction aVar = { \
			__BO_CRTACTION_MAGIC, \
			(aPriority), \
			(__bo_crta_init_fn)(anInitFunc), \
			(anInitParm), \
			(__bo_crta_fini_fn)(aFiniFunc), \
			(aFiniParm), \
			BO_RTBTELEM_ZERO_INITIALIZER, \
			BO_CDLLELEM_ZERO_INITIALIZER \
			_im_BO_CRTACTION_DEBUG0_SPEC1_INITIALIZER(__FILE__,__LINE__, \
				__myc_xstringify(anInitFunc),__myc_xstringify(aFiniFunc)) \
		}; \
		static int __MYC_CRT_INIT_FUN_(__MYC_CDTOR_UPRIO) __myc_xconcat(_t_crtaction_,aVar)(); \
		__MYC_CRT_INIT_SECTION_IFNOACTOR(__myc_xconcat(_t_crtaction_,aVar)) \
		static int __MYC_CRT_INIT_FUN_(__MYC_CDTOR_UPRIO) \
		__myc_xconcat(_t_crtaction_,aVar)() { \
			return __bo_crtaction((struct __bo_crtaction *)&aVar); \
		}
# endif /* !_im_BO_CRTACTION */

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

__myc_flinkage_shlib(private) int
__bo_crtaction(
# if __myc_require(prototypes)
	struct __bo_crtaction *
# endif
	);

__MYC_END_CDECLS

#endif /* !__BO_SYS_CRT_CRTACTION__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
