/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * crtdefs.h	(V.Sokur)
 * include/Bo/sys/crt/crtdefs.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_CRT_CRTDEFS__
# define __BO_SYS_CRT_CRTDEFS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>
# include <Bo/sys/Linkage.h>

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

typedef int
(__myc_tcalltype__(mcall) *__bo_crta_init_fn)(
# if __myc_require(prototypes)
	bo_pointer_t
# endif
	) __attribute__((mcalltype));

typedef void
(__myc_tcalltype__(mcall) *__bo_crta_fini_fn)(
# if __myc_require(prototypes)
	bo_pointer_t
# endif
	) __attribute__((mcalltype));


struct __bo_crtaction {
	u_int32_t           crta_magic;
# ifndef __BO_CRTACTION_MAGIC
#  define __BO_CRTACTION_MAGIC 0x74696F62 /* BOIT */
# endif

	int                 crta_priority;
	__bo_crta_init_fn   crta_initfunc;
	bo_pointer_t        crta_initparm;
	__bo_crta_fini_fn   crta_finifunc;
	bo_pointer_t        crta_finiparm;

	bo_rtbtelem_t       crta_btelem;
	bo_cdllelem_t       crta_lelem;

	char const         *crta_file;
	int                 crta_line;
	char const         *crta_initfunc_name,
	                   *crta_finifunc_name;
};

typedef struct __bo_crtaction __bo_crtaction_t;

/*
 * Other definitions
 */
# pragma mark -

# ifndef SUPPORT_CONSTRUCTOR_ATTRIBUTE
#  include _BO_PLATFORM_HEADER(crt/crtdefs.h)
# endif


# ifndef __MYC_CDTOR_UPRIO
#  define __MYC_CDTOR_UPRIO   4094
# endif
# ifndef __MYC_CDTOR_KPRIO
#  define __MYC_CDTOR_KPRIO   4095
# endif

# ifdef SUPPORT_CONSTRUCTOR_ATTRIBUTE
#  ifdef SUPPORT_CONSTRUCTOR_WITH_PARM_ATTRIBUTE
#   define __MYC_CRT_INIT_FUN_(N) __attribute__((constructor(N)))
#   define __MYC_CRT_FINI_FUN_(N) __attribute__((destructor(N)))
#  else
#   define __MYC_CRT_INIT_FUN_(N) __attribute__((constructor))
#   define __MYC_CRT_FINI_FUN_(N) __attribute__((destructor))
#  endif
#  define __MYC_CRT_INIT_FUN __attribute__((constructor))
#  define __MYC_CRT_FINI_FUN __attribute__((destructor))
# else
#  define __MYC_CRT_INIT_FUN_(N)
#  define __MYC_CRT_FINI_FUN_(N)
#  define __MYC_CRT_INIT_FUN
#  define __MYC_CRT_FINI_FUN
# endif


# ifdef SUPPORT_CONSTRUCTOR_ATTRIBUTE
#  define __MYC_CRT_INIT_SECTION_IFNOACTOR(anEntry)
#  define __MYC_CRT_FINI_SECTION_IFNOACTOR(anEntry)
#  define __MYC_CRT_LIBINIT_SECTION_IFNOACTOR(anEntry)
#  define __MYC_CRT_LIBFINI_SECTION_IFNOACTOR(anEntry)
#  define __MYC_CRT_APPINIT_SECTION_IFNOACTOR(anEntry)
#  define __MYC_CRT_APPFINI_SECTION_IFNOACTOR(anEntry)
# else
#  define __MYC_CRT_INIT_SECTION_IFNOACTOR(anEntry) __myc_crt_init_section(anEntry)
#  define __MYC_CRT_FINI_SECTION_IFNOACTOR(anEntry) __myc_crt_fini_section(anEntry)
#  define __MYC_CRT_LIBINIT_SECTION_IFNOACTOR(anEntry) __myc_crt_libinit_section(anEntry)
#  define __MYC_CRT_LIBFINI_SECTION_IFNOACTOR(anEntry) __myc_crt_libfini_section(anEntry)
#  define __MYC_CRT_APPINIT_SECTION_IFNOACTOR(anEntry) __myc_crt_appinit_section(anEntry)
#  define __MYC_CRT_APPFINI_SECTION_IFNOACTOR(anEntry) __myc_crt_appfini_section(anEntry)
# endif


# ifndef __bo_crt_int_void_fn_are_defined

typedef int
(__myc_tcalltype__(mcall) *__bo_crt_int_fn)(
# if __myc_require(prototypes)
	void
# endif
	) __attribute__((mcalltype));

typedef void
(__myc_tcalltype__(mcall) *__bo_crt_void_fn)(
# if __myc_require(prototypes)
	void
# endif
	) __attribute__((mcalltype));

#  define __bo_crt_int_void_fn_are_defined
# endif

#endif /* !__BO_SYS_CRT_CRTDEFS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
