/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * csType.h	(V.Sokur)
 * include/Bo/sys/csType.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_SYS_CSTYPE__
# define __BO_SYS_CSTYPE__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SYS_TYPES__
#  include <Bo/sys/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

# ifndef __BO_CSET_TYPE__
#  define __BO_CSET_TYPE__ int32_t
# endif
# ifndef BO_CSET_TYPE_DEFINED
#  define BO_CSET_TYPE_DEFINED
typedef __BO_CSET_TYPE__ bo_cset_t;
# endif

typedef int16_t bo_cset_x;


/* "INVALID" character set */
# ifndef BoK_INVAL_CSET
#  define BoK_INVAL_CSET (-1)
# endif

/* "BINARY" character set */
# ifndef BoK_NULL_CSET
#  define BoK_NULL_CSET (0)
# endif

/*
 * IANA registered 'charset' - coded character set (CCS)
 */

/* Unknown character set.
   Program dependent, default value. */
# ifndef BoK_UNK_CSET
#  define BoK_UNK_CSET (2)
# endif

# ifndef BoK_UNICODE_CSET
/* @todo: redesign */
#  ifdef BO_UNICODE_UCS4
#   define BoK_UNICODE_CSET BoK_UCS4_CSET

#   define BO_UNICODE_CSET_NAME BO_UCS4_CSET_NAME
#   define _BO_UNICODE_CSET_NAME _BO_UCS4_CSET_NAME

#   define BO_UNICODE_CSET_MIME BO_UCS4_CSET_MIME
#   define _BO_UNICODE_CSET_MIME _BO_UCS4_CSET_MIME
#  else /* !BO_UNICODE_UCS4 */
#   define BoK_UNICODE_CSET BoK_UCS2_CSET

#   define BO_UNICODE_CSET_NAME BO_UCS2_CSET_NAME
#   define _BO_UNICODE_CSET_NAME _BO_UCS2_CSET_NAME

#   define BO_UNICODE_CSET_MIME BO_UCS2_CSET_MIME
#   define _BO_UNICODE_CSET_MIME _BO_UCS2_CSET_MIME
#  endif /* BO_UNICODE_UCS4 */
# endif /* BoK_UNICODE_CSET */


/* ASCII */
# ifndef BoK_ASCII_CSET
#  define BoK_ASCII_CSET (3)

#  define BO_ASCII_CSET_NAME ANSI_X3.4-1968
#  define _BO_ASCII_CSET_NAME "ANSI_X3.4-1968"

#  define BO_ASCII_CSET_MIME US-ASCII
#  define _BO_ASCII_CSET_MIME "US-ASCII"
/*
Name: ANSI_X3.4-1968                                   [RFC1345,KXS2]
MIBenum: 3
Source: ECMA registry
Alias: iso-ir-6
Alias: ANSI_X3.4-1986
Alias: ISO_646.irv:1991
Alias: ASCII
Alias: ISO646-US
Alias: US-ASCII (preferred MIME name)
Alias: us
Alias: IBM367
Alias: CP367
Alias: csASCII
 */
# endif

/* LATIN1 */
# ifndef BoK_LATIN1_CSET
#  define BoK_LATIN1_CSET (4)

#  define BO_LATIN1_CSET_NAME ISO_8859-1:1987
#  define _BO_LATIN1_CSET_NAME "ISO_8859-1:1987"

#  define BO_LATIN1_CSET_MIME ISO-8859-1
#  define _BO_LATIN1_CSET_MIME "ISO-8859-1"
/*
Name: ISO_8859-1:1987                                    [RFC1345,KXS2]
MIBenum: 4
Source: ECMA registry
Alias: iso-ir-100
Alias: ISO_8859-1
Alias: ISO-8859-1 (preferred MIME name)
Alias: latin1
Alias: l1
Alias: csISOLatin1
Alias: IBM819
Alias: CP819
 */
# endif

/* LATIN2 */
# ifndef BoK_LATIN2_CSET
#  define BoK_LATIN2_CSET (5)

#  define BO_LATIN2_CSET_NAME ISO_8859-2:1987
#  define _BO_LATIN2_CSET_NAME "ISO_8859-2:1987"

#  define BO_LATIN2_CSET_MIME ISO-8859-2
#  define _BO_LATIN2_CSET_MIME "ISO-8859-2"
/*
Name: ISO_8859-2:1987                                    [RFC1345,KXS2]
MIBenum: 5
Source: ECMA registry
Alias: iso-ir-101
Alias: ISO_8859-2
Alias: ISO-8859-2 (preferred MIME name)
Alias: latin2
Alias: l2
Alias: csISOLatin2
 */
# endif

/* LATIN3 */
# ifndef BoK_LATIN3_CSET
#  define BoK_LATIN3_CSET (6)

#  define BO_LATIN3_CSET_NAME ISO_8859-3:1988
#  define _BO_LATIN3_CSET_NAME "ISO_8859-3:1988"

#  define BO_LATIN3_CSET_MIME ISO-8859-3
#  define _BO_LATIN3_CSET_MIME "ISO-8859-3"
/*
Name: ISO_8859-3:1988                                    [RFC1345,KXS2]
MIBenum: 6
Source: ECMA registry
Alias: iso-ir-109
Alias: ISO_8859-3
Alias: ISO-8859-3 (preferred MIME name)
Alias: latin3
Alias: l3
Alias: csISOLatin3
 */
# endif

/* LATIN4 */
# ifndef BoK_LATIN4_CSET
#  define BoK_LATIN4_CSET (7)

#  define BO_LATIN4_CSET_NAME ISO_8859-4:1988
#  define _BO_LATIN4_CSET_NAME "ISO_8859-4:1988"

#  define BO_LATIN4_CSET_MIME ISO-8859-4
#  define _BO_LATIN4_CSET_MIME "ISO-8859-4"
/*
Name: ISO_8859-4:1988                                    [RFC1345,KXS2]
MIBenum: 7
Source: ECMA registry
Alias: iso-ir-110
Alias: ISO_8859-4
Alias: ISO-8859-4 (preferred MIME name)
Alias: latin4
Alias: l4
Alias: csISOLatin4
 */
# endif

/* LATIN-CYRILLIC */
# ifndef BoK_LATIN_CYR_CSET
#  define BoK_LATIN_CYR_CSET (8)

#  define BO_LATIN_CYR_CSET_NAME ISO_8859-5:1988
#  define _BO_LATIN_CYR_CSET_NAME "ISO_8859-5:1988"

#  define BO_LATIN_CYR_CSET_MIME ISO-8859-5
#  define _BO_LATIN_CYR_CSET_MIME "ISO-8859-5"
/*
Name: ISO_8859-5:1988                                     [RFC1345,KXS2]
MIBenum: 8
Source: ECMA registry
Alias: iso-ir-144
Alias: ISO_8859-5
Alias: ISO-8859-5 (preferred MIME name)
Alias: cyrillic
Alias: csISOLatinCyrillic

My-alias: ISO8859-5 // X11R6.4, FreeBSD
My-alias: CP28595
My-alias: 28595
 */
# endif

/* LATIN-ARABIC */
# ifndef BoK_LATIN_ARABIC_CSET
#  define BoK_LATIN_ARABIC_CSET (9)

#  define BO_LATIN_ARABIC_CSET_NAME ISO_8859-6:1987
#  define _BO_LATIN_ARABIC_CSET_NAME "ISO_8859-6:1987"

#  define BO_LATIN_ARABIC_CSET_MIME ISO-8859-6
#  define _BO_LATIN_ARABIC_CSET_MIME "ISO-8859-6"
/*
Name: ISO_8859-6:1987                                    [RFC1345,KXS2]
MIBenum: 9
Source: ECMA registry
Alias: iso-ir-127
Alias: ISO_8859-6
Alias: ISO-8859-6 (preferred MIME name)
Alias: ECMA-114
Alias: ASMO-708
Alias: arabic
Alias: csISOLatinArabic
 */
# endif

/* LATIN-GREEK */
# ifndef BoK_LATIN_GREEK_CSET
#  define BoK_LATIN_GREEK_CSET (10)

#  define BO_LATIN_GREEK_CSET_NAME ISO_8859-7:1987
#  define _BO_LATIN_GREEK_CSET_NAME "ISO_8859-7:1987"

#  define BO_LATIN_GREEK_CSET_MIME ISO-8859-7
#  define _BO_LATIN_GREEK_CSET_MIME "ISO-8859-7"
/*
Name: ISO_8859-7:1987                            [RFC1947,RFC1345,KXS2]
MIBenum: 10
Source: ECMA registry
Alias: iso-ir-126
Alias: ISO_8859-7
Alias: ISO-8859-7 (preferred MIME name)
Alias: ELOT_928
Alias: ECMA-118
Alias: greek
Alias: greek8
Alias: csISOLatinGreek
 */
# endif

/* LATIN-HEBREW */
# ifndef BoK_LATIN_HEBREW_CSET
#  define BoK_LATIN_HEBREW_CSET (11)

#  define BO_LATIN_HEBREW_CSET_NAME ISO_8859-8:1987
#  define _BO_LATIN_HEBREW_CSET_NAME "ISO_8859-8:1987"

#  define BO_LATIN_HEBREW_CSET_MIME ISO-8859-8
#  define _BO_LATIN_HEBREW_CSET_MIME "ISO-8859-8"
/*
Name: ISO_8859-8:1987                            [RFC1345,KXS2]
MIBenum: 11
Source: ECMA registry
Alias: iso-ir-138
Alias: ISO_8859-8
Alias: ISO-8859-8 (preferred MIME name)
Alias: hebrew
Alias: csISOLatinHebrew
 */
# endif

/* LATIN5 */
# ifndef BoK_LATIN5_CSET
#  define BoK_LATIN5_CSET (12)

#  define BO_LATIN5_CSET_NAME ISO_8859-9:1989
#  define _BO_LATIN5_CSET_NAME "ISO_8859-9:1989"

#  define BO_LATIN5_CSET_MIME ISO-8859-9
#  define _BO_LATIN5_CSET_MIME "ISO-8859-9"
/*
Name: ISO_8859-9:1989                                     [RFC1345,KXS2]
MIBenum: 12
Source: ECMA registry
Alias: iso-ir-148
Alias: ISO_8859-9
Alias: ISO-8859-9 (preferred MIME name)
Alias: latin5
Alias: l5
Alias: csISOLatin5
 */
# endif

/* LATIN6 */
# ifndef BoK_LATIN6_CSET
#  define BoK_LATIN6_CSET (13)

#  define BO_LATIN6_CSET_NAME ISO-8859-10:1992
#  define _BO_LATIN6_CSET_NAME "ISO-8859-10:1992"

#  define BO_LATIN6_CSET_MIME ISO-8859-10
#  define _BO_LATIN6_CSET_MIME "ISO-8859-10"
/*
Name: ISO-8859-10 (preferred MIME name)			  [RFC1345,KXS2]
MIBenum: 13
Source: ECMA registry
Alias: iso-ir-157
Alias: l6
Alias: ISO_8859-10:1992
Alias: csISOLatin6
Alias: latin6
 */
# endif

/* TEXT-COMM */
# ifndef BoK_TEXT_COMM_CSET
#  define BoK_TEXT_COMM_CSET (14)

#  define BO_TEXT_COMM_CSET_NAME ISO_6937-2-add
#  define _BO_TEXT_COMM_CSET_NAME "ISO_6937-2-add"

#  define BO_TEXT_COMM_CSET_MIME ISO_6937-2-add
#  define _BO_TEXT_COMM_CSET_MIME "ISO_6937-2-add"
/*
Name: ISO_6937-2-add (preferred MIME name)			  [RFC1345,KXS2]
MIBenum: 14
Source: ECMA registry and ISO 6937-2:1983
Alias: iso-ir-142
Alias: csISOTextComm
 */
# endif

/* HalfWidthKatakana */
# ifndef BoK_HalfWidthKatakana_CSET
#  define BoK_HalfWidthKatakana_CSET (15)

#  define BO_HalfWidthKatakana_CSET_NAME JIS_X0201
#  define _BO_HalfWidthKatakana_CSET_NAME "JIS_X0201"

#  define BO_HalfWidthKatakana_CSET_MIME JIS_X0201
#  define _BO_HalfWidthKatakana_CSET_MIME "JIS_X0201"
/*
Name: JIS_X0201 (preferred MIME name)			  [RFC1345,KXS2]
MIBenum: 15
Source: JIS X 0201-1976. One byte only, this is equivalent to
        JIS/Roman (similar to ASCII) plus eight-bit half-width.
Alias: X0201
Alias: csHalfWidthKatakana
 */
# endif

/* JIS_ENCODING */
# ifndef BoK_JIS_ENCODING_CSET
#  define BoK_JIS_ENCODING_CSET (16)

#  define BO_JIS_ENCODING_CSET_NAME JIS_Encoding
#  define _BO_JIS_ENCODING_CSET_NAME "JIS_Encoding"

#  define BO_JIS_ENCODING_CSET_MIME JIS_Encoding
#  define _BO_JIS_ENCODING_CSET_MIME "JIS_Encoding"
/*
Name: JIS_Encoding
MIBenum: 16
Source: JIS X 0202-1991.  Uses ISO 2022 escape sequences to
        shift code sets as documented in JIS X 0202-1991.
Alias: csJISEncoding
 */
# endif

/* MS_KANJI */
# ifndef BoK_MS_KANJI_CSET
#  define BoK_MS_KANJI_CSET (17)

#  define BO_MS_KANJI_CSET_NAME Shift_JIS
#  define _BO_MS_KANJI_CSET_NAME "Shift_JIS"

#  define BO_MS_KANJI_CSET_MIME Shift_JIS
#  define _BO_MS_KANJI_CSET_MIME "Shift_JIS"
/*
Name: Shift_JIS  (preferred MIME name)
MIBenum: 17
Source: This charset is an extension of csHalfWidthKatakana by
        adding graphic characters in JIS X 0208.  The CCS's are
        JIS X0201:1997 and JIS X0208:1997.  The
        complete definition is shown in Appendix 1 of JIS
        X0208:1997.
        This charset can be used for the top-level media type "text".
Alias: MS_Kanji 
Alias: csShiftJIS
*/
# endif

/* EUCPkdFmtJapanese */
# ifndef BoK_EUCPkdFmtJapanese_CSET
#  define BoK_EUCPkdFmtJapanese_CSET (18)

#  define BO_EUCPkdFmtJapanese_CSET_NAME Extended_UNIX_Code_Packed_Format_for_Japanese
#  define _BO_EUCPkdFmtJapanese_CSET_NAME "Extended_UNIX_Code_Packed_Format_for_Japanese"

#  define BO_EUCPkdFmtJapanese_CSET_MIME EUC-JP
#  define _BO_EUCPkdFmtJapanese_CSET_MIME "EUC-JP"
/*
Name: Extended_UNIX_Code_Packed_Format_for_Japanese
MIBenum: 18
Source: Standardized by OSF, UNIX International, and UNIX Systems
        Laboratories Pacific.  Uses ISO 2022 rules to select
               code set 0: US-ASCII (a single 7-bit byte set)
               code set 1: JIS X0208-1990 (a double 8-bit byte set)
                           restricted to A0-FF in both bytes
               code set 2: Half Width Katakana (a single 7-bit byte set)
                           requiring SS2 as the character prefix
               code set 3: JIS X0212-1990 (a double 7-bit byte set)
                           restricted to A0-FF in both bytes
                           requiring SS3 as the character prefix
Alias: csEUCPkdFmtJapanese
Alias: EUC-JP  (preferred MIME name)
*/
# endif

/* EUCFixWidJapanese */
# ifndef BoK_EUCFixWidJapanese_CSET
#  define BoK_EUCFixWidJapanese_CSET (19)

#  define BO_EUCFixWidJapanese_CSET_NAME Extended_UNIX_Code_Fixed_Width_for_Japanese
#  define _BO_EUCFixWidJapanese_CSET_NAME "Extended_UNIX_Code_Fixed_Width_for_Japanese"

#  define BO_EUCFixWidJapanese_CSET_MIME
#  define _BO_EUCFixWidJapanese_CSET_MIME
/*
Name: Extended_UNIX_Code_Fixed_Width_for_Japanese
MIBenum: 19
Source: Used in Japan.  Each character is 2 octets.
                code set 0: US-ASCII (a single 7-bit byte set)
                              1st byte = 00
                              2nd byte = 20-7E
                code set 1: JIS X0208-1990 (a double 7-bit byte set)
                            restricted  to A0-FF in both bytes 
                code set 2: Half Width Katakana (a single 7-bit byte set)
                              1st byte = 00
                              2nd byte = A0-FF
                code set 3: JIS X0212-1990 (a double 7-bit byte set)
                            restricted to A0-FF in 
                            the first byte
                and 21-7E in the second byte
Alias: csEUCFixWidJapanese
*/
# endif

/* UnitedKingdom */
# ifndef BoK_UnitedKingdom_CSET
#  define BoK_UnitedKingdom_CSET (20)

#  define BO_UnitedKingdom_CSET_NAME BS_4730
#  define _BO_UnitedKingdom_CSET_NAME "BS_4730"

#  define BO_UnitedKingdom_CSET_MIME ISO646-GB
#  define _BO_UnitedKingdom_CSET_MIME "ISO646-GB"
/*
Name: BS_4730                                           [RFC1345,KXS2]
MIBenum: 20
Source: ECMA registry
Alias: iso-ir-4
Alias: ISO646-GB
Alias: gb
Alias: uk
Alias: csISO4UnitedKingdom
*/
# endif

/* SwedishForNames */
# ifndef BoK_SwedishForNames_CSET
#  define BoK_SwedishForNames_CSET (21)

#  define BO_SwedishForNames_CSET_NAME SEN_850200_C
#  define _BO_SwedishForNames_CSET_NAME "SEN_850200_C"

#  define BO_SwedishForNames_CSET_MIME ISO646-SE2
#  define _BO_SwedishForNames_CSET_MIME "ISO646-SE2"
/*
Name: SEN_850200_C                                      [RFC1345,KXS2]
MIBenum: 21
Source: ECMA registry
Alias: iso-ir-11
Alias: ISO646-SE2
Alias: se2
Alias: csISO11SwedishForNames
*/
# endif

/* ITALIAN */
# ifndef BoK_ITALIAN_CSET
#  define BoK_ITALIAN_CSET (22)

#  define BO_ITALIAN_CSET_NAME IT
#  define _BO_ITALIAN_CSET_NAME "IT"

#  define BO_ITALIAN_CSET_MIME ISO646-IT
#  define _BO_ITALIAN_CSET_MIME "ISO646-IT"
/*
Name: IT                                                [RFC1345,KXS2]
MIBenum: 22
Source: ECMA registry
Alias: iso-ir-15
Alias: ISO646-IT
Alias: csISO15Italian
*/
# endif

/* SPANISH */
# ifndef BoK_SPANISH_CSET
#  define BoK_SPANISH_CSET (23)

#  define BO_SPANISH_CSET_NAME ES
#  define _BO_SPANISH_CSET_NAME "ES"

#  define BO_SPANISH_CSET_MIME ISO646-ES
#  define _BO_SPANISH_CSET_MIME "ISO646-ES"
/*
Name: ES                                                [RFC1345,KXS2]
MIBenum: 23
Source: ECMA registry
Alias: iso-ir-17
Alias: ISO646-ES
Alias: csISO17Spanish
*/
# endif

/* GERMAN */
# ifndef BoK_GERMAN_CSET
#  define BoK_GERMAN_CSET (24)

#  define BO_GERMAN_CSET_NAME DIN_66003
#  define _BO_GERMAN_CSET_NAME "DIN_66003"

#  define BO_GERMAN_CSET_MIME ISO646-DE
#  define _BO_GERMAN_CSET_MIME "ISO646-DE"
/*
Name: DIN_66003                                         [RFC1345,KXS2]
MIBenum: 24
Source: ECMA registry
Alias: iso-ir-21
Alias: de
Alias: ISO646-DE
Alias: csISO21German
*/
# endif

/* DAN_NORWEGIAN */
# ifndef BoK_DAN_NORWEGIAN_CSET
#  define BoK_DAN_NORWEGIAN_CSET (25)

#  define BO_DAN_NORWEGIAN_CSET_NAME NS_4551-1
#  define _BO_DAN_NORWEGIAN_CSET_NAME "NS_4551-1"

#  define BO_DAN_NORWEGIAN_CSET_MIME ISO646-NO
#  define _BO_DAN_NORWEGIAN_CSET_MIME "ISO646-NO"
/*
Name: NS_4551-1                                         [RFC1345,KXS2]
MIBenum: 25
Source: ECMA registry
Alias: iso-ir-60
Alias: ISO646-NO
Alias: no
Alias: csISO60DanishNorwegian
Alias: csISO60Norwegian1
*/
# endif

/* FRENCH */
# ifndef BoK_FRENCH_CSET
#  define BoK_FRENCH_CSET (27)

#  define BO_FRENCH_CSET_NAME NF_Z_62-010
#  define _BO_FRENCH_CSET_NAME "NF_Z_62-010"

#  define BO_FRENCH_CSET_MIME ISO646-FR
#  define _BO_FRENCH_CSET_MIME "ISO646-FR"
/*
Name: NF_Z_62-010                                        [RFC1345,KXS2]
MIBenum: 26
Source: ECMA registry
Alias: iso-ir-69
Alias: ISO646-FR
Alias: fr
Alias: csISO69French
*/
# endif

/* UNICODE UTF1 */
# ifndef BoK_UTF1_CSET
#  define BoK_UTF1_CSET (27)

#  define BO_UTF1_CSET_NAME ISO-10646-UTF-1
#  define _BO_UTF1_CSET_NAME "ISO-10646-UTF-1"

#  define BO_UTF1_CSET_MIME ISO-10646-UTF-1
#  define _BO_UTF1_CSET_MIME "ISO-10646-UTF-1"
/*
Name: ISO-10646-UTF-1
MIBenum: 27
Source: Universal Transfer Format (1), this is the multibyte
        encoding, that subsets ASCII-7. It does not have byte
        ordering issues.
Alias: csISO10646UTF1
 */
# endif

/* ISO5427-CYRILLIC */
# ifndef BoK_ISO5427_CYR_CSET
#  define BoK_ISO5427_CYR_CSET (48)

#  define BO_ISO5427_CYR_CSET_NAME ISO_5427
#  define _BO_ISO5427_CYR_CSET_NAME "ISO_5427"

#  define BO_ISO5427_CYR_CSET_MIME ISO_5427
#  define _BO_ISO5427_CYR_CSET_MIME "ISO_5427"
/*
Name: ISO_5427                                          [RFC1345,KXS2]
MIBenum: 48
Source: ECMA registry
Alias: iso-ir-37
Alias: csISO5427Cyrillic
 */
# endif

/* INIS-CYRILLIC */
# ifndef BoK_INIS_CYR_CSET
#  define BoK_INIS_CYR_CSET (53)

#  define BO_INIS_CYR_CSET_NAME INIS-cyrillic
#  define _BO_INIS_CYR_CSET_NAME "INIS-cyrillic"

#  define BO_INIS_CYR_CSET_MIME INIS-cyrillic
#  define _BO_INIS_CYR_CSET_MIME "INIS-cyrillic"
/*
Name: INIS-cyrillic                                     [RFC1345,KXS2]
MIBenum: 53
Source: ECMA registry
Alias: iso-ir-51
Alias: csISO51INISCyrillic
 */
# endif

/* ISO5427-1981-CYRILLIC */
# ifndef BoK_ISO5427_1981_CYR_CSET
#  define BoK_ISO5427_1981_CYR_CSET (54)

#  define BO_ISO5427_1981_CYR_CSET_NAME ISO_5427:1981
#  define _BO_ISO5427_1981_CYR_CSET_NAME "ISO_5427:1981"

#  define BO_ISO5427_1981_CYR_CSET_MIME ISO_5427:1981
#  define _BO_ISO5427_1981_CYR_CSET_MIME "ISO_5427:1981"
/*
Name: ISO_5427:1981                                     [RFC1345,KXS2]
MIBenum: 54
Source: ECMA registry
Alias: iso-ir-54
Alias: ISO5427Cyrillic1981
 */
# endif

/* ECMA-CYRILLIC */
# ifndef BoK_KOI8E_CSET
#  define BoK_KOI8E_CSET (77)

#  define BO_KOI8E_CSET_NAME ECMA-cyrillic
#  define _BO_KOI8E_CSET_NAME "ECMA-cyrillic"

#  define BO_KOI8E_CSET_MIME KOI8-E
#  define _BO_KOI8E_CSET_MIME "KOI8-E"
/*
Name: ECMA-cyrillic                                     
MIBenum: 77
Source: ISO registry (formerly ECMA registry)
         http://www.itscj.ipsj.jp/ISO-IR/111.pdf
Alias: iso-ir-111
Alias: KOI8-E
Alias: csISO111ECMACyrillic

My-alias: KOI8_E
My-alias: KOI8E
 */
# endif

/* UNICODE UTF7 */
# ifndef BoK_UNICODE_UTF7_CSET
#  define BoK_UNICODE_UTF7_CSET (103)

#  define BO_UNICODE_UTF7_CSET_NAME UNICODE-1-1-UTF-7
#  define _BO_UNICODE_UTF7_CSET_NAME "UNICODE-1-1-UTF-7"

#  define BO_UNICODE_UTF7_CSET_MIME UNICODE-1-1-UTF-7
#  define _BO_UNICODE_UTF7_CSET_MIME "UNICODE-1-1-UTF-7"
/*
Name: UNICODE-1-1-UTF-7                                        [RFC1642]
MIBenum: 103
Source: RFC 1642
Alias: csUnicode11UTF7
 */
# endif

/* UTF8 */
# ifndef BoK_UTF8_CSET
#  define BoK_UTF8_CSET (106)

#  define BO_UTF8_CSET_NAME UTF-8
#  define _BO_UTF8_CSET_NAME "UTF-8"

#  define BO_UTF8_CSET_MIME UTF-8
#  define _BO_UTF8_CSET_MIME "UTF-8"
/*
Name: UTF-8                                                    [RFC3629]
MIBenum: 106
Source: RFC 3629
Alias: None 

My-alias: UTF8
My-alias: UTF_8
My-alias: CP65001
My-alias: 65001
 */
# endif

/* LATIN8 */
# ifndef BoK_LATIN8_CSET
#  define BoK_LATIN8_CSET (110)

#  define BO_LATIN8_CSET_NAME ISO-8859-14
#  define _BO_LATIN8_CSET_NAME "ISO-8859-14"

#  define BO_LATIN8_CSET_MIME ISO-8859-14
#  define _BO_LATIN8_CSET_MIME "ISO-8859-14"
/*
Name: ISO-8859-14
MIBenum: 110
Source: ISO See (http://www.iana.org/assignments/charset-reg/ISO-8859-14) [Simonsen]
Alias: iso-ir-199
Alias: ISO_8859-14:1998
Alias: ISO_8859-14
Alias: latin8
Alias: iso-celtic
Alias: l8
 */
# endif

/* LATIN9 */
# ifndef BoK_LATIN9_CSET
#  define BoK_LATIN9_CSET (111)

#  define BO_LATIN9_CSET_NAME ISO-8859-15
#  define _BO_LATIN9_CSET_NAME "ISO-8859-15"

#  define BO_LATIN9_CSET_MIME ISO-8859-15
#  define _BO_LATIN9_CSET_MIME "ISO-8859-15"
/*
Name: ISO-8859-15
MIBenum: 111
Source: ISO 
        Please see: <http://www.iana.org/assignments/charset-reg/ISO-8859-15>
Alias: ISO_8859-15
Alias: Latin-9
 */
# endif

/* LATIN10 */
# ifndef BoK_LATIN10_CSET
#  define BoK_LATIN10_CSET (112)

#  define BO_LATIN10_CSET_NAME ISO-8859-16
#  define _BO_LATIN10_CSET_NAME "ISO-8859-16"

#  define BO_LATIN10_CSET_MIME ISO-8859-16
#  define _BO_LATIN10_CSET_MIME "ISO-8859-16"
/*
Name: ISO-8859-16
MIBenum: 112
Source: ISO
Alias: iso-ir-226
Alias: ISO_8859-16:2001
Alias: ISO_8859-16
Alias: latin10
Alias: l10 
 */
# endif

/* UNICODE UCS-2 */
# ifndef BoK_UCS2_CSET
#  define BoK_UCS2_CSET (1000)

#  define BO_UCS2_CSET_NAME ISO-10646-UCS-2
#  define _BO_UCS2_CSET_NAME "ISO-10646-UCS-2"

#  define BO_UCS2_CSET_MIME UCS-2
#  define _BO_UCS2_CSET_MIME "UCS-2"
/*
Name: ISO-10646-UCS-2
MIBenum: 1000
Source: the 2-octet Basic Multilingual Plane, aka Unicode
        this needs to specify network byte order: the standard
        does not specify (it is a 16-bit integer space)
Alias: csUnicode
 */
# endif

/* UNICODE UCS-4 */
# ifndef BoK_UCS4_CSET
#  define BoK_UCS4_CSET (1001)

#  define BO_UCS4_CSET_NAME ISO-10646-UCS-4
#  define _BO_UCS4_CSET_NAME "ISO-10646-UCS-4"

#  define BO_UCS4_CSET_MIME UCS-4
#  define _BO_UCS4_CSET_MIME "UCS-4"
/*
Name: ISO-10646-UCS-4
MIBenum: 1001
Source: the full code space. (same comment about byte order,
        these are 31-bit numbers.
Alias: csUCS4
 */
# endif

/* UNICODE ASCII */
# ifndef BoK_UNICODE_ASCII_CSET
#  define BoK_UNICODE_ASCII_CSET (1002)

#  define BO_UNICODE_ASCII_CSET_NAME ISO-10646-UCS-Basic
#  define _BO_UNICODE_ASCII_CSET_NAME "ISO-10646-UCS-Basic"

#  define BO_UNICODE_ASCII_CSET_MIME UCS-Basic
#  define _BO_UNICODE_ASCII_CSET_MIME "UCS-Basic"
/*
Name: ISO-10646-UCS-Basic
MIBenum: 1002
Source: ASCII subset of Unicode.  Basic Latin = collection 1
        See ISO 10646, Appendix A
Alias: csUnicodeASCII
 */
# endif

/* UNICODE LATIN1 */
# ifndef BoK_UNICODE_LATIN1_CSET
#  define BoK_UNICODE_LATIN1_CSET (1003)

#  define BO_UNICODE_LATIN1_CSET_NAME ISO-10646-Unicode-Latin1
#  define _BO_UNICODE_LATIN1_CSET_NAME "ISO-10646-Unicode-Latin1"

#  define BO_UNICODE_LATIN1_CSET_MIME Unicode-Latin1
#  define _BO_UNICODE_LATIN1_CSET_MIME "Unicode-Latin1"
/*
Name: ISO-10646-Unicode-Latin1
MIBenum: 1003
Source: ISO Latin-1 subset of Unicode. Basic Latin and Latin-1 
         Supplement  = collections 1 and 2.  See ISO 10646, 
         Appendix A.  See RFC 1815.
Alias: csUnicodeLatin1
Alias: ISO-10646
 */
# endif

/* UNICODE 1.1, ISO/IEC 10646-1:1993(E) */
# ifndef BoK_UNICODE11_CSET
#  define BoK_UNICODE11_CSET (1010)

#  define BO_UNICODE11_CSET_NAME UNICODE-1-1
#  define _BO_UNICODE11_CSET_NAME "UNICODE-1-1"

#  define BO_UNICODE11_CSET_MIME UNICODE11
#  define _BO_UNICODE11_CSET_MIME "UNICODE11"
/*
Name: UNICODE-1-1                                              [RFC1641]
MIBenum: 1010
Source: RFC 1641
Alias: csUnicode11
 */
# endif

/* UTF7 */
# ifndef BoK_UTF7_CSET
#  define BoK_UTF7_CSET (1012)

#  define BO_UTF7_CSET_NAME UTF-7
#  define _BO_UTF7_CSET_NAME "UTF-7"

#  define BO_UTF7_CSET_MIME UTF-7
#  define _BO_UTF7_CSET_MIME "UTF-7"
/*
Name: UTF-7                                                    [RFC2152]
MIBenum: 1012
Source: RFC 2152
Alias: None
 */
# endif

/* EBCDIC-CYRILLIC */
# ifndef BoK_EBCDIC_CYR_CSET
#  define BoK_EBCDIC_CYR_CSET (2057)

#  define BO_EBCDIC_CYR_CSET_NAME IBM880
#  define _BO_EBCDIC_CYR_CSET_NAME "IBM880"

#  define BO_EBCDIC_CYR_CSET_MIME IBM880
#  define _BO_EBCDIC_CYR_CSET_MIME "IBM880"
/*
Name: IBM880                                              [RFC1345,KXS2]
MIBenum: 2057
Source: IBM NLS RM Vol2 SE09-8002-01, March 1990
Alias: cp880
Alias: EBCDIC-Cyrillic
Alias: csIBM880
 */
# endif

/* KOI8-R */
# ifndef BoK_KOI8R_CSET
#  define BoK_KOI8R_CSET (2084)

#  define BO_KOI8R_CSET_NAME KOI8-R
#  define _BO_KOI8R_CSET_NAME "KOI8-R"

#  define BO_KOI8R_CSET_MIME KOI8-R
#  define _BO_KOI8R_CSET_MIME "KOI8-R"
/*
Name: KOI8-R  (preferred MIME name)                            [RFC1489]
MIBenum: 2084
Source: RFC 1489, based on GOST-19768-74, ISO-6937/8, 
        INIS-Cyrillic, ISO-5427.
Alias: csKOI8R

My-alias: KOI8R
My-alias: KOI8-R
My-alias: KOI8_R
My-alias: CP20866
My-alias: 20866
 */
# endif

/* IBM-CYRILLIC */
# ifndef BoK_IBM_CYR_CSET
#  define BoK_IBM_CYR_CSET (2086)

#  define BO_IBM_CYR_CSET_NAME IBM866
#  define _BO_IBM_CYR_CSET_NAME "IBM866"

#  define BO_IBM_CYR_CSET_MIME IBM866
#  define _BO_IBM_CYR_CSET_MIME "IBM866"
/*
Name: IBM866                                                     [Pond]
MIBenum: 2086
Source: IBM NLDG Volume 2 (SE09-8002-03) August 1994
Alias: csIBM866
Alias: CP866
Alias: 866
 */
# endif

/* KOI8-U */
# ifndef BoK_KOI8U_CSET
#  define BoK_KOI8U_CSET (2088)

#  define BO_KOI8U_CSET_NAME KOI8-U
#  define _BO_KOI8U_CSET_NAME "KOI8-U"

#  define BO_KOI8U_CSET_MIME KOI8-U
#  define _BO_KOI8U_CSET_MIME "KOI8-U"
/*
Name: KOI8-U  (preferred MIME name)                            [RFC2319]
MIBenum: 2088
Source: RFC 2319

My-alias: csKOI8U
My-alias: KOI8U
My-alias: KOI8-U
My-alias: KOI8_U
My-alias: CP21866
My-alias: 21866
 */
# endif

/* WIN-CYRILLIC */
# ifndef BoK_WIN_CYR_CSET
#  define BoK_WIN_CYR_CSET (2251)

#  define BO_WIN_CYR_CSET_NAME windows-1251
#  define _BO_WIN_CYR_CSET_NAME "windows-1251"

#  define BO_WIN_CYR_CSET_MIME windows-1251
#  define _BO_WIN_CYR_CSET_MIME "windows-1251"
/*
Name: windows-1251
MIBenum: 2251
Source: Microsoft  (http://www.iana.org/assignments/charset-reg/windows-1251) [Lazhintseva]
Alias: None

My-alias: MS-CYRL
My-alias: CP1251
My-alias: 1251
 */
# endif

#endif /* !__BO_SYS_CSTYPE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
