/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Atomic.h	(V.Sokur)
 * include/Bo/sys/freebsd/_Atomic.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FREEBSD__ATOMIC__
# define __BO_FREEBSD__ATOMIC__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# if __myc_exists(machine7atomic,h)
#  include __MYC_LIBC_HEADER(machine/atomic.h)

/*
 * @todo: fixme
 */

#  define HAS_ATOMIC_CMPSET_INT
#  define HAS_ATOMIC_CMPSET_LONG

#  define HAS_ATOMIC_FETCHADD_INT
#  define HAS_ATOMIC_FETCHADD_LONG

#  define HAS_ATOMIC_SET_CHAR
#  define HAS_ATOMIC_SET_SHORT
#  define HAS_ATOMIC_SET_INT
#  define HAS_ATOMIC_SET_LONG

#  define HAS_ATOMIC_CLEAR_CHAR
#  define HAS_ATOMIC_CLEAR_SHORT
#  define HAS_ATOMIC_CLEAR_INT
#  define HAS_ATOMIC_CLEAR_LONG

#  define HAS_ATOMIC_ADD_CHAR
#  define HAS_ATOMIC_ADD_SHORT
#  define HAS_ATOMIC_ADD_INT
#  define HAS_ATOMIC_ADD_LONG

#  define HAS_ATOMIC_SUB_CHAR
#  define HAS_ATOMIC_SUB_SHORT
#  define HAS_ATOMIC_SUB_INT
#  define HAS_ATOMIC_SUB_LONG

#  define HAS_ATOMIC_READANDCLEAR_INT
#  define HAS_ATOMIC_READANDCLEAR_LONG

# endif /* __myc_exists(machine7atomic,h) */

# include _BO_COMPILER_HEADER(_Atomic.h)

/******************************************************************************
 * %% BeginSection: definitions
 */

# ifdef atomic_memory_barrier
# define BoAtomicMemoryBarrier() atomic_memory_barrier()
# endif
# ifdef atomic_write_memory_barrier
# define BoAtomicWriteMemoryBarrier() atomic_write_memory_barrier()
# endif
# ifdef atomic_read_memory_barrier
# define BoAtomicReadMemoryBarrier() atomic_read_memory_barrier()
# endif

/* ----------------------------------------------------------------------------
 * 8-bit operations
 */

/*
 * u_int8_t BoAtomicLoad8_acq(volatile u_int8_t *aPtr);
 * void BoAtomicStore8_rel(volatile u_int8_t *aPtr, u_int8_t aValue);
 */

# ifdef atomic_load_acq_8
# define BoAtomicLoad8_acq(aPtr) \
	atomic_load_acq_8((volatile u_int8_t *)(aPtr))
# endif

# ifdef atomic_store_rel_8
# define BoAtomicStore8_rel(aPtr,aValue) \
	atomic_store_rel_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aValue))
# endif

/*
 * u_int8_t BoAtomicFetchClear8(volatile u_int8_t *aPtr);
 */
# ifdef atomic_readandclear_8
# define BoAtomicFetchClear8(aPtr) \
	atomic_readandclear_8((volatile u_int8_t *)(aPtr))
# endif

/*
 * int BoAtomicCompareStore8(volatile u_int8_t *aDst, u_int8_t aComparand, u_int8_t aValue);
 * int BoAtomicCompareStore8_acq(volatile u_int8_t *aDst, u_int8_t aComparand, u_int8_t aValue);
 * int BoAtomicCompareStore8_rel(volatile u_int8_t *aDst, u_int8_t aComparand, u_int8_t aValue);
 */

# ifdef atomic_cmpset_8
# define BoAtomicCompareStore8(aDst,aComparand,aValue) \
	atomic_cmpset_8((volatile u_int8_t *)(aDst), (u_int8_t)(aComparand), (u_int8_t)(aValue))
# endif

# ifdef atomic_cmpset_acq_8
# define BoAtomicCompareStore8_acq(aDst,aComparand,aValue) \
	atomic_cmpset_acq_8((volatile u_int8_t *)(aDst), (u_int8_t)(aComparand), (u_int8_t)(aValue))
# endif

# ifdef atomic_cmpset_rel_8
# define BoAtomicCompareStore8_rel(aDst,aComparand,aValue) \
	atomic_cmpset_rel_8((volatile u_int8_t *)(aDst), (u_int8_t)(aComparand), (u_int8_t)(aValue))
# endif

/*
 * void BoAtomicSetmask8(volatile u_int8_t *aPtr, u_int8_t aMask);
 * void BoAtomicSetmask8_acq(volatile u_int8_t *aPtr, u_int8_t aMask);
 * void BoAtomicSetmask8_rel(volatile u_int8_t *aPtr, u_int8_t aMask);
 */

# ifdef atomic_set_8
# define BoAtomicSetmask8(aPtr,aMask) \
	atomic_set_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aMask))
# endif

# ifdef atomic_set_acq_8
# define BoAtomicSetmask8_acq(aPtr,aMask) \
	atomic_set_acq_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aMask))
# endif

# ifdef atomic_set_rel_8
# define BoAtomicSetmask8_rel(aPtr,aMask) \
	atomic_set_rel_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aMask))
# endif

/*
 * void BoAtomicClearmask8(volatile u_int8_t *aPtr, u_int8_t aMask);
 * void BoAtomicClearmask8_acq(volatile u_int8_t *aPtr, u_int8_t aMask);
 * void BoAtomicClearmask8_rel(volatile u_int8_t *aPtr, u_int8_t aMask);
 */

# ifdef atomic_clear_8
# define BoAtomicClearmask8(aPtr,aMask) \
	atomic_clear_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aMask))
# endif

# ifdef atomic_clear_acq_8
# define BoAtomicClearmask8_acq(aPtr,aMask) \
	atomic_clear_acq_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aMask))
# endif

# ifdef atomic_clear_rel_8
# define BoAtomicClearmask8_rel(aPtr,aMask) \
	atomic_clear_rel_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aMask))
# endif

/*
 * void BoAtomicAdd8(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicAdd8_acq(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicAdd8_rel(volatile u_int8_t *aPtr, u_int8_t aValue);
 */

# ifdef atomic_add_8
# define BoAtomicAdd8(aPtr,aValue) \
	atomic_add_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aValue))
# endif

# ifdef atomic_add_acq_8
# define BoAtomicAdd8_acq(aPtr,aValue) \
	atomic_add_acq_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aValue))
# endif

# ifdef atomic_add_rel_8
# define BoAtomicAdd8_rel(aPtr,aValue) \
	atomic_add_rel_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aValue))
# endif

/*
 * u_int8_t BoAtomicFetchAdd8(volatile u_int8_t *aDst, u_int8_t aValue);
 * u_int8_t BoAtomicFetchAdd8_acq(volatile u_int8_t *aDst, u_int8_t aValue);
 * u_int8_t BoAtomicFetchAdd8_rel(volatile u_int8_t *aDst, u_int8_t aValue);
 */

# ifdef atomic_fetchadd_8
# define BoAtomicFetchAdd8(aDst,aValue) \
	atomic_fetchadd_8((volatile u_int8_t *)(aDst), (u_int8_t)(aValue))
# endif

# ifdef atomic_fetchadd_acq_8
# define BoAtomicFetchAdd8_acq(aDst,aValue) \
	atomic_fetchadd_acq_8((volatile u_int8_t *)(aDst), (u_int8_t)(aValue))
# endif

# ifdef atomic_fetchadd_rel_8
# define BoAtomicFetchAdd8_rel(aDst,aValue) \
	atomic_fetchadd_rel_8((volatile u_int8_t *)(aDst), (u_int8_t)(aValue))
# endif

/*
 * void BoAtomicSub8(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicSub8_acq(volatile u_int8_t *aPtr, u_int8_t aValue);
 * void BoAtomicSub8_rel(volatile u_int8_t *aPtr, u_int8_t aValue);
 */

# ifdef atomic_subtract_8
# define BoAtomicSub8(aPtr,aValue) \
	atomic_subtract_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aValue))
# endif

# ifdef atomic_subtract_acq_8
# define BoAtomicSub8_acq(aPtr,aValue) \
	atomic_subtract_acq_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aValue))
# endif

# ifdef atomic_subtract_rel_8
# define BoAtomicSub8_rel(aPtr,aValue) \
	atomic_subtract_rel_8((volatile u_int8_t *)(aPtr), (u_int8_t)(aValue))
# endif

/* ----------------------------------------------------------------------------
 * 16-bit operations
 */

/*
 * u_int16_t BoAtomicLoad16_acq(volatile u_int16_t *aPtr);
 * void BoAtomicStore16_rel(volatile u_int16_t *aPtr, u_int16_t aValue);
 */

# ifdef atomic_load_acq_16
# define BoAtomicLoad16_acq(aPtr) \
	atomic_load_acq_16((volatile u_int16_t *)(aPtr))
# endif

# ifdef atomic_store_rel_16
# define BoAtomicStore16_rel(aPtr,aValue) \
	atomic_store_rel_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aValue))
# endif

/*
 * u_int16_t BoAtomicFetchClear16(volatile u_int16_t *aPtr);
 */
# ifdef atomic_readandclear_16
# define BoAtomicFetchClear16(aPtr) \
	atomic_readandclear_16((volatile u_int16_t *)(aPtr))
# endif

/*
 * int BoAtomicCompareStore16(volatile u_int16_t *aDst, u_int16_t aComparand, u_int16_t aValue);
 * int BoAtomicCompareStore16_acq(volatile u_int16_t *aDst, u_int16_t aComparand, u_int16_t aValue);
 * int BoAtomicCompareStore16_rel(volatile u_int16_t *aDst, u_int16_t aComparand, u_int16_t aValue);
 */

# ifdef atomic_cmpset_16
# define BoAtomicCompareStore16(aDst,aComparand,aValue) \
	atomic_cmpset_16((volatile u_int16_t *)(aDst), (u_int16_t)(aComparand), (u_int16_t)(aValue))
# endif

# ifdef atomic_cmpset_acq_16
# define BoAtomicCompareStore16_acq(aDst,aComparand,aValue) \
	atomic_cmpset_acq_16((volatile u_int16_t *)(aDst), (u_int16_t)(aComparand), (u_int16_t)(aValue))
# endif

# ifdef atomic_cmpset_rel_16
# define BoAtomicCompareStore16_rel(aDst,aComparand,aValue) \
	atomic_cmpset_rel_16((volatile u_int16_t *)(aDst), (u_int16_t)(aComparand), (u_int16_t)(aValue))
# endif

/*
 * void BoAtomicSetmask16(volatile u_int16_t *aPtr, u_int16_t aMask);
 * void BoAtomicSetmask16_acq(volatile u_int16_t *aPtr, u_int16_t aMask);
 * void BoAtomicSetmask16_rel(volatile u_int16_t *aPtr, u_int16_t aMask);
 */

# ifdef atomic_set_16
# define BoAtomicSetmask16(aPtr,aMask) \
	atomic_set_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aMask))
# endif

# ifdef atomic_set_acq_16
# define BoAtomicSetmask16_acq(aPtr,aMask) \
	atomic_set_acq_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aMask))
# endif

# ifdef atomic_set_rel_16
# define BoAtomicSetmask16_rel(aPtr,aMask) \
	atomic_set_rel_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aMask))
# endif

/*
 * void BoAtomicClearmask16(volatile u_int16_t *aPtr, u_int16_t aMask);
 * void BoAtomicClearmask16_acq(volatile u_int16_t *aPtr, u_int16_t aMask);
 * void BoAtomicClearmask16_rel(volatile u_int16_t *aPtr, u_int16_t aMask);
 */

# ifdef atomic_clear_16
# define BoAtomicClearmask16(aPtr,aMask) \
	atomic_clear_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aMask))
# endif

# ifdef atomic_clear_acq_16
# define BoAtomicClearmask16_acq(aPtr,aMask) \
	atomic_clear_acq_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aMask))
# endif

# ifdef atomic_clear_rel_16
# define BoAtomicClearmask16_rel(aPtr,aMask) \
	atomic_clear_rel_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aMask))
# endif

/*
 * void BoAtomicAdd16(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicAdd16_acq(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicAdd16_rel(volatile u_int16_t *aPtr, u_int16_t aValue);
 */

# ifdef atomic_add_16
# define BoAtomicAdd16(aPtr,aValue) \
	atomic_add_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aValue))
# endif

# ifdef atomic_add_acq_16
# define BoAtomicAdd16_acq(aPtr,aValue) \
	atomic_add_acq_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aValue))
# endif

# ifdef atomic_add_rel_16
# define BoAtomicAdd16_rel(aPtr,aValue) \
	atomic_add_rel_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aValue))
# endif

/*
 * u_int16_t BoAtomicFetchAdd16(volatile u_int16_t *aDst, u_int16_t aValue);
 * u_int16_t BoAtomicFetchAdd16_acq(volatile u_int16_t *aDst, u_int16_t aValue);
 * u_int16_t BoAtomicFetchAdd16_rel(volatile u_int16_t *aDst, u_int16_t aValue);
 */

# ifdef atomic_fetchadd_16
# define BoAtomicFetchAdd16(aDst,aValue) \
	atomic_fetchadd_16((volatile u_int16_t *)(aDst), (u_int16_t)(aValue))
# endif

# ifdef atomic_fetchadd_acq_16
# define BoAtomicFetchAdd16_acq(aDst,aValue) \
	atomic_fetchadd_acq_16((volatile u_int16_t *)(aDst), (u_int16_t)(aValue))
# endif

# ifdef atomic_fetchadd_rel_16
# define BoAtomicFetchAdd16_rel(aDst,aValue) \
	atomic_fetchadd_rel_16((volatile u_int16_t *)(aDst), (u_int16_t)(aValue))
# endif

/*
 * void BoAtomicSub16(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicSub16_acq(volatile u_int16_t *aPtr, u_int16_t aValue);
 * void BoAtomicSub16_rel(volatile u_int16_t *aPtr, u_int16_t aValue);
 */

# ifdef atomic_subtract_16
# define BoAtomicSub16(aPtr,aValue) \
	atomic_subtract_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aValue))
# endif

# ifdef atomic_subtract_acq_16
# define BoAtomicSub16_acq(aPtr,aValue) \
	atomic_subtract_acq_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aValue))
# endif

# ifdef atomic_subtract_rel_16
# define BoAtomicSub16_rel(aPtr,aValue) \
	atomic_subtract_rel_16((volatile u_int16_t *)(aPtr), (u_int16_t)(aValue))
# endif

/* ----------------------------------------------------------------------------
 * 32-bit operations
 */

/*
 * u_int32_t BoAtomicLoad32_acq(volatile u_int32_t *aPtr);
 * void BoAtomicStore32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# ifdef atomic_load_acq_32
# define BoAtomicLoad32_acq(aPtr) \
	atomic_load_acq_32((volatile u_int32_t *)(aPtr))
# endif

# ifdef atomic_store_rel_32
# define BoAtomicStore32_rel(aPtr,aValue) \
	atomic_store_rel_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aValue))
# endif

/*
 * u_int32_t BoAtomicFetchClear32(volatile u_int32_t *aPtr);
 */
# ifdef atomic_readandclear_32
# define BoAtomicFetchClear32(aPtr) \
	atomic_readandclear_32((volatile u_int32_t *)(aPtr))
# endif

/*
 * int BoAtomicCompareStore32(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 * int BoAtomicCompareStore32_acq(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 * int BoAtomicCompareStore32_rel(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 */

# ifdef atomic_cmpset_32
# define BoAtomicCompareStore32(aDst,aComparand,aValue) \
	atomic_cmpset_32((volatile u_int32_t *)(aDst), (u_int32_t)(aComparand), (u_int32_t)(aValue))
# endif

# ifdef atomic_cmpset_acq_32
# define BoAtomicCompareStore32_acq(aDst,aComparand,aValue) \
	atomic_cmpset_acq_32((volatile u_int32_t *)(aDst), (u_int32_t)(aComparand), (u_int32_t)(aValue))
# endif

# ifdef atomic_cmpset_rel_32
# define BoAtomicCompareStore32_rel(aDst,aComparand,aValue) \
	atomic_cmpset_rel_32((volatile u_int32_t *)(aDst), (u_int32_t)(aComparand), (u_int32_t)(aValue))
# endif

/*
 * void BoAtomicSetmask32(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicSetmask32_acq(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicSetmask32_rel(volatile u_int32_t *aPtr, u_int32_t aMask);
 */

# ifdef atomic_set_32
# define BoAtomicSetmask32(aPtr,aMask) \
	atomic_set_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aMask))
# endif

# ifdef atomic_set_acq_32
# define BoAtomicSetmask32_acq(aPtr,aMask) \
	atomic_set_acq_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aMask))
# endif

# ifdef atomic_set_rel_32
# define BoAtomicSetmask32_rel(aPtr,aMask) \
	atomic_set_rel_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aMask))
# endif

/*
 * void BoAtomicClearmask32(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicClearmask32_acq(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicClearmask32_rel(volatile u_int32_t *aPtr, u_int32_t aMask);
 */

# ifdef atomic_clear_32
# define BoAtomicClearmask32(aPtr,aMask) \
	atomic_clear_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aMask))
# endif

# ifdef atomic_clear_acq_32
# define BoAtomicClearmask32_acq(aPtr,aMask) \
	atomic_clear_acq_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aMask))
# endif

# ifdef atomic_clear_rel_32
# define BoAtomicClearmask32_rel(aPtr,aMask) \
	atomic_clear_rel_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aMask))
# endif

/*
 * void BoAtomicAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# ifdef atomic_add_32
# define BoAtomicAdd32(aPtr,aValue) \
	atomic_add_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aValue))
# endif

# ifdef atomic_add_acq_32
# define BoAtomicAdd32_acq(aPtr,aValue) \
	atomic_add_acq_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aValue))
# endif

# ifdef atomic_add_rel_32
# define BoAtomicAdd32_rel(aPtr,aValue) \
	atomic_add_rel_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aValue))
# endif

/*
 * u_int32_t BoAtomicFetchAdd32(volatile u_int32_t *aDst, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAdd32_acq(volatile u_int32_t *aDst, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAdd32_rel(volatile u_int32_t *aDst, u_int32_t aValue);
 */

# ifdef atomic_fetchadd_32
# define BoAtomicFetchAdd32(aDst,aValue) \
	atomic_fetchadd_32((volatile u_int32_t *)(aDst), (u_int32_t)(aValue))
# endif

# ifdef atomic_fetchadd_acq_32
# define BoAtomicFetchAdd32_acq(aDst,aValue) \
	atomic_fetchadd_acq_32((volatile u_int32_t *)(aDst), (u_int32_t)(aValue))
# endif

# ifdef atomic_fetchadd_rel_32
# define BoAtomicFetchAdd32_rel(aDst,aValue) \
	atomic_fetchadd_rel_32((volatile u_int32_t *)(aDst), (u_int32_t)(aValue))
# endif

/*
 * void BoAtomicSub32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicSub32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicSub32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# ifdef atomic_subtract_32
# define BoAtomicSub32(aPtr,aValue) \
	atomic_subtract_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aValue))
# endif

# ifdef atomic_subtract_acq_32
# define BoAtomicSub32_acq(aPtr,aValue) \
	atomic_subtract_acq_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aValue))
# endif

# ifdef atomic_subtract_rel_32
# define BoAtomicSub32_rel(aPtr,aValue) \
	atomic_subtract_rel_32((volatile u_int32_t *)(aPtr), (u_int32_t)(aValue))
# endif

/* ----------------------------------------------------------------------------
 * 64-bit operations
 */

# if (BO_M_WORD == BO_M_WORD64)

/*
 * u_int64_t BoAtomicLoad64_acq(volatile u_int64_t *aPtr);
 * void BoAtomicStore64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# ifdef atomic_load_acq_64
# define BoAtomicLoad64_acq(aPtr) \
	atomic_load_acq_64((volatile u_int64_t *)(aPtr))
# endif

# ifdef atomic_store_rel_64
# define BoAtomicStore64_rel(aPtr,aValue) \
	atomic_store_rel_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aValue))
# endif

/*
 * u_int64_t BoAtomicFetchClear64(volatile u_int64_t *aPtr);
 */
# ifdef atomic_readandclear_64
# define BoAtomicFetchClear64(aPtr) \
	atomic_readandclear_64((volatile u_int64_t *)(aPtr))
# endif

/*
 * int BoAtomicCompareStore64(volatile u_int64_t *aDst, u_int64_t aComparand, u_int64_t aValue);
 * int BoAtomicCompareStore64_acq(volatile u_int64_t *aDst, u_int64_t aComparand, u_int64_t aValue);
 * int BoAtomicCompareStore64_rel(volatile u_int64_t *aDst, u_int64_t aComparand, u_int64_t aValue);
 */

# ifdef atomic_cmpset_64
# define BoAtomicCompareStore64(aDst,aComparand,aValue) \
	atomic_cmpset_64((volatile u_int64_t *)(aDst), (u_int64_t)(aComparand), (u_int64_t)(aValue))
# endif

# ifdef atomic_cmpset_acq_64
# define BoAtomicCompareStore64_acq(aDst,aComparand,aValue) \
	atomic_cmpset_acq_64((volatile u_int64_t *)(aDst), (u_int64_t)(aComparand), (u_int64_t)(aValue))
# endif

# ifdef atomic_cmpset_rel_64
# define BoAtomicCompareStore64_rel(aDst,aComparand,aValue) \
	atomic_cmpset_rel_64((volatile u_int64_t *)(aDst), (u_int64_t)(aComparand), (u_int64_t)(aValue))
# endif

/*
 * void BoAtomicSetmask64(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicSetmask64_acq(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicSetmask64_rel(volatile u_int64_t *aPtr, u_int64_t aMask);
 */

# ifdef atomic_set_64
# define BoAtomicSetmask64(aPtr,aMask) \
	atomic_set_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aMask))
# endif

# ifdef atomic_set_acq_64
# define BoAtomicSetmask64_acq(aPtr,aMask) \
	atomic_set_acq_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aMask))
# endif

# ifdef atomic_set_rel_64
# define BoAtomicSetmask64_rel(aPtr,aMask) \
	atomic_set_rel_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aMask))
# endif

/*
 * void BoAtomicClearmask64(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicClearmask64_acq(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicClearmask64_rel(volatile u_int64_t *aPtr, u_int64_t aMask);
 */

# ifdef atomic_clear_64
# define BoAtomicClearmask64(aPtr,aMask) \
	atomic_clear_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aMask))
# endif

# ifdef atomic_clear_acq_64
# define BoAtomicClearmask64_acq(aPtr,aMask) \
	atomic_clear_acq_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aMask))
# endif

# ifdef atomic_clear_rel_64
# define BoAtomicClearmask64_rel(aPtr,aMask) \
	atomic_clear_rel_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aMask))
# endif

/*
 * void BoAtomicAdd64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicAdd64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicAdd64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# ifdef atomic_add_64
# define BoAtomicAdd64(aPtr,aValue) \
	atomic_add_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aValue))
# endif

# ifdef atomic_add_acq_64
# define BoAtomicAdd64_acq(aPtr,aValue) \
	atomic_add_acq_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aValue))
# endif

# ifdef atomic_add_rel_64
# define BoAtomicAdd64_rel(aPtr,aValue) \
	atomic_add_rel_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aValue))
# endif

/*
 * u_int64_t BoAtomicFetchAdd64(volatile u_int64_t *aDst, u_int64_t aValue);
 * u_int64_t BoAtomicFetchAdd64_acq(volatile u_int64_t *aDst, u_int64_t aValue);
 * u_int64_t BoAtomicFetchAdd64_rel(volatile u_int64_t *aDst, u_int64_t aValue);
 */

# ifdef atomic_fetchadd_64
# define BoAtomicFetchAdd64(aDst,aValue) \
	atomic_fetchadd_64((volatile u_int64_t *)(aDst), (u_int64_t)(aValue))
# endif

# ifdef atomic_fetchadd_acq_64
# define BoAtomicFetchAdd64_acq(aDst,aValue) \
	atomic_fetchadd_acq_64((volatile u_int64_t *)(aDst), (u_int64_t)(aValue))
# endif

# ifdef atomic_fetchadd_rel_64
# define BoAtomicFetchAdd64_rel(aDst,aValue) \
	atomic_fetchadd_rel_64((volatile u_int64_t *)(aDst), (u_int64_t)(aValue))
# endif

/*
 * void BoAtomicSub64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicSub64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicSub64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# ifdef atomic_subtract_64
# define BoAtomicSub64(aPtr,aValue) \
	atomic_subtract_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aValue))
# endif

# ifdef atomic_subtract_acq_64
# define BoAtomicSub64_acq(aPtr,aValue) \
	atomic_subtract_acq_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aValue))
# endif

# ifdef atomic_subtract_rel_64
# define BoAtomicSub64_rel(aPtr,aValue) \
	atomic_subtract_rel_64((volatile u_int64_t *)(aPtr), (u_int64_t)(aValue))
# endif

# endif /* (BO_M_WORD == BO_M_WORD64) */

#endif /* !__BO_FREEBSD__ATOMIC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
