/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Errors.h	(V.Sokur)
 * include/Bo/sys/freebsd/_Errors.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_FREEBSD__ERRORS__
# define __BO_FREEBSD__ERRORS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <errno.h>

/******************************************************************************
 * %% BeginSection: definitions
 */

# ifndef EPERM
#  define EPERM 1               /* Operation not permitted */
# endif
# ifndef ENOENT
#  define ENOENT 2              /* No such file or directory */
# endif
# ifndef ESRCH
#  define ESRCH 3               /* No such process */
# endif
# ifndef EINTR
#  define EINTR 4               /* Interrupted system call */
# endif
# ifndef EIO
#  define EIO 5                 /* Input/output error */
# endif
# ifndef ENXIO
#  define ENXIO 6               /* Device not configured */
# endif
# ifndef E2BIG
#  define E2BIG 7               /* Argument list too long */
# endif
# ifndef ENOEXEC
#  define ENOEXEC 8             /* Exec format error */
# endif
# ifndef EBADF
#  define EBADF 9               /* Bad file descriptor */
# endif
# ifndef ECHILD
#  define ECHILD 10             /* No child processes */
# endif
# ifndef EDEADLK
#  define EDEADLK 11            /* Resource deadlock avoided */
                                /* 11 was EAGAIN */
# endif
# ifndef ENOMEM
#  define ENOMEM 12             /* Cannot allocate memory */
# endif
# ifndef EACCES
#  define EACCES 13             /* Permission denied */
# endif
# ifndef EFAULT
#  define EFAULT 14             /* Bad address */
# endif

# ifndef _POSIX_SOURCE
#  ifndef ENOTBLK
#   define ENOTBLK 15           /* Block device required */
#  endif
# endif

# ifndef EBUSY
#  define EBUSY 16              /* Device busy */
# endif
# ifndef EEXIST
#  define EEXIST 17             /* File exists */
# endif
# ifndef EXDEV
#  define EXDEV 18              /* Cross-device link */
# endif
# ifndef ENODEV
#  define ENODEV 19             /* Operation not supported by device */
# endif
# ifndef ENOTDIR
#  define ENOTDIR 20            /* Not a directory */
# endif
# ifndef EISDIR
#  define EISDIR 21             /* Is a directory */
# endif
# ifndef EINVAL
#  define EINVAL 22             /* Invalid argument */
# endif
# ifndef ENFILE
#  define ENFILE 23             /* Too many open files in system */
# endif
# ifndef EMFILE
#  define EMFILE 24             /* Too many open files */
# endif
# ifndef ENOTTY
#  define ENOTTY 25             /* Inappropriate ioctl for device */
# endif

# ifndef _POSIX_SOURCE
#  ifndef ETXTBSY
#   define ETXTBSY 26           /* Text file busy */
#  endif
# endif

# ifndef EFBIG
#  define EFBIG 27              /* File too large */
# endif
# ifndef ENOSPC
#  define ENOSPC 28             /* No space left on device */
# endif
# ifndef ESPIPE
#  define ESPIPE 29             /* Illegal seek */
# endif
# ifndef EROFS
#  define EROFS 30              /* Read-only filesystem */
# endif
# ifndef EMLINK
#  define EMLINK 31             /* Too many links */
# endif
# ifndef EPIPE
#  define EPIPE 32              /* Broken pipe */
# endif

/* math software */
# ifndef EDOM
#  define EDOM 33               /* Numerical argument out of domain */
# endif
# ifndef ERANGE
#  define ERANGE 34             /* Result too large */
# endif

/* non-blocking and interrupt i/o */
# ifndef EAGAIN
#  define EAGAIN 35             /* Resource temporarily unavailable */
# endif

# ifndef _POSIX_SOURCE
#  ifndef EWOULDBLOCK
#   define EWOULDBLOCK /* EAGAIN */ 35   /* Operation would block */
#  endif
#  ifndef EINPROGRESS
#   define EINPROGRESS 36       /* Operation now in progress */
#  endif
#  ifndef EALREADY
#   define EALREADY 37          /* Operation already in progress */
#  endif

/* ipc/network software -- argument errors */
#  ifndef ENOTSOCK
#   define ENOTSOCK 38          /* Socket operation on non-socket */
#  endif
#  ifndef EDESTADDRREQ
#   define EDESTADDRREQ 39      /* Destination address required */
#  endif
#  ifndef EMSGSIZE
#   define EMSGSIZE 40          /* Message too long */
#  endif
#  ifndef EPROTOTYPE
#   define EPROTOTYPE 41        /* Protocol wrong type for socket */
#  endif
#  ifndef ENOPROTOOPT
#   define ENOPROTOOPT 42       /* Protocol not available */
#  endif
#  ifndef EPROTONOSUPPORT
#   define EPROTONOSUPPORT 43   /* Protocol not supported */
#  endif
#  ifndef ESOCKTNOSUPPORT
#   define ESOCKTNOSUPPORT 44   /* Socket type not supported */
#  endif
#  ifndef EOPNOTSUPP
#   define EOPNOTSUPP 45        /* Operation not supported */
#  endif
#  ifndef ENOTSUP
#   define ENOTSUP EOPNOTSUPP   /* Operation not supported */
#  endif
#  ifndef EPFNOSUPPORT
#   define EPFNOSUPPORT 46      /* Protocol family not supported */
#  endif
#  ifndef EAFNOSUPPORT
#   define EAFNOSUPPORT 47      /* Address family not supported by protocol family */
#  endif
#  ifndef EADDRINUSE
#   define EADDRINUSE 48        /* Address already in use */
#  endif
#  ifndef EADDRNOTAVAIL
#   define EADDRNOTAVAIL 49     /* Can't assign requested address */
#  endif

/* ipc/network software -- operational errors */
#  ifndef ENETDOWN
#   define ENETDOWN 50          /* Network is down */
#  endif
#  ifndef ENETUNREACH
#   define ENETUNREACH 51       /* Network is unreachable */
#  endif
#  ifndef ENETRESET
#   define ENETRESET 52         /* Network dropped connection on reset */
#  endif
#  ifndef ECONNABORTED
#   define ECONNABORTED 53      /* Software caused connection abort */
#  endif
#  ifndef ECONNRESET
#   define ECONNRESET 54        /* Connection reset by peer */
#  endif
#  ifndef ENOBUFS
#   define ENOBUFS 55           /* No buffer space available */
#  endif
#  ifndef EISCONN
#   define EISCONN 56           /* Socket is already connected */
#  endif
#  ifndef ENOTCONN
#   define ENOTCONN 57          /* Socket is not connected */
#  endif
#  ifndef ESHUTDOWN
#   define ESHUTDOWN 58         /* Can't send after socket shutdown */
#  endif
#  ifndef ETOOMANYREFS
#   define ETOOMANYREFS 59      /* Too many references: can't splice */
#  endif
#  ifndef ETIMEDOUT
#   define ETIMEDOUT 60         /* Operation timed out */
#  endif
#  ifndef ECONNREFUSED
#   define ECONNREFUSED 61      /* Connection refused */
#  endif
#  ifndef ELOOP
#   define ELOOP 62             /* Too many levels of symbolic links */
#  endif
# endif /* !_POSIX_SOURCE */

# ifndef ENAMETOOLONG
#  define ENAMETOOLONG	63		/* File name too long */
# endif

# ifndef _POSIX_SOURCE
#  ifndef EHOSTDOWN
#   define EHOSTDOWN 64         /* Host is down */
#  endif
#  ifndef EHOSTUNREACH
#   define EHOSTUNREACH 65      /* No route to host */
#  endif
# endif /* !_POSIX_SOURCE */

# ifndef ENOTEMPTY
#  define ENOTEMPTY 66          /* Directory not empty */
# endif

/* quotas & mush */
# ifndef _POSIX_SOURCE
#  ifndef EPROCLIM
#   define EPROCLIM 67          /* Too many processes */
#  endif
#  ifndef EUSERS
#   define EUSERS 68            /* Too many users */
#  endif
#  ifndef EDQUOT
#   define EDQUOT 69            /* Disc quota exceeded */
#  endif

/* Network File System */
#  ifndef ESTALE
#   define ESTALE 70            /* Stale NFS file handle */
#  endif
#  ifndef EREMOTE
#   define EREMOTE 71           /* Too many levels of remote in path */
#  endif
#  ifndef EBADRPC
#   define EBADRPC 72           /* RPC struct is bad */
#  endif
#  ifndef ERPCMISMATCH
#   define ERPCMISMATCH 73      /* RPC version wrong */
#  endif
#  ifndef EPROGUNAVAIL
#   define EPROGUNAVAIL 74      /* RPC prog. not avail */
#  endif
#  ifndef EPROGMISMATCH
#   define EPROGMISMATCH 75     /* Program version wrong */
#  endif
#  ifndef EPROCUNAVAIL
#   define EPROCUNAVAIL 76      /* Bad procedure for program */
#  endif
# endif /* !_POSIX_SOURCE */

# ifndef ENOLCK
#  define ENOLCK 77             /* No locks available */
# endif
# ifndef ENOSYS
#  define ENOSYS 78             /* Function not implemented */
# endif

# ifndef _POSIX_SOURCE
#  ifndef EFTYPE
#   define EFTYPE 79            /* Inappropriate file type or format */
#  endif
#  ifndef EAUTH
#   define EAUTH 80             /* Authentication error */
#  endif
#  ifndef ENEEDAUTH
#   define ENEEDAUTH 81         /* Need authenticator */
#  endif
#  ifndef EIDRM
#   define EIDRM 82             /* Identifier removed */
#  endif
#  ifndef ENOMSG
#   define ENOMSG 83            /* No message of desired type */
#  endif
#  ifndef EOVERFLOW
#   define EOVERFLOW 84         /* Value too large to be stored in data type */
#  endif
#  ifndef ECANCELED
#   define ECANCELED 85         /* Operation canceled */
#  endif
#  ifndef EILSEQ
#   define EILSEQ 86            /* Illegal byte sequence */
#  endif
#  ifndef ENOATTR
#   define ENOATTR 87           /* Attribute not found */
#  endif
#  ifndef EDOOFUS
#   define EDOOFUS 88           /* Programming error */
#  endif
# endif /* !_POSIX_SOURCE */

# ifndef EBADMSG
#  define EBADMSG 89            /* Bad message */
# endif
# ifndef EMULTIHOP
#  define EMULTIHOP 90          /* Multihop attempted */
# endif
# ifndef ENOLINK
#  define ENOLINK 91            /* Link has been severed */
# endif
# ifndef EPROTO
#  define EPROTO 92             /* Protocol error */
# endif

#endif /* !__BO_FREEBSD__ERRORS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
