/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Thrtypes.h	(V.Sokur)
 * include/Bo/sys/linux/_Thrtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_LINUX__THRTYPES__
# define __BO_LINUX__THRTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# include __MYC_LIBC_HEADER(pthread.h)

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

/*
 * P-MUTEX
 */

typedef pthread_mutex_t     bo_pmutex_t, *bo_pmutex_ref;
typedef pthread_mutexattr_t bo_pmutexattr_t, *bo_pmutexattr_ref;

# define BO_PMUTEX_ZERO_INITIALIZER PTHREAD_MUTEX_INITIALIZER
# define BO_PMUTEXATTR_ZERO_INITIALIZER {0}

# define BO_PMUTEX_REF(x) (&(x))
# define BO_PMUTEXATTR_REF(x) (&(x))

/*
 * P-CONDITION
 */

typedef pthread_cond_t      bo_pcond_t, *bo_pcond_ref;
typedef pthread_condattr_t  bo_pcondattr_t, *bo_pcondattr_ref;

# define BO_PCOND_ZERO_INITIALIZER PTHREAD_COND_INITIALIZER
# define BO_PCONDATTR_ZERO_INITIALIZER {0}

# define BO_PCOND_REF(x) (&(x))
# define BO_PCONDATTR_REF(x) (&(x))

/*
 * P-THREAD
 */

typedef pthread_t           bo_pthread_t, bo_pthread_ref;
typedef pthread_attr_t      bo_pthrattr_t, *bo_pthrattr_ref;

# define BO_PTHREAD_NULL 0
# define BO_PTHREAD_ZERO_INITIALIZER 0
# define BO_PTHRATTR_ZERO_INITIALIZER {0}

# define BO_PTHREAD_REF(x) (x)
# define BO_PTHRATTR_REF(x) (&(x))

typedef bo_pointer_t bo_pthread_retval_t;
typedef bo_pthread_retval_t (*bo_pthread_proc)(bo_pointer_t );

# define BO_PTHREAD_PROC_CALLTYPE __myc_calltype__(cdecl)

# define BO_PTHREAD_EXIT_NORMAL ((bo_pthread_retval_t)0)
# define BO_PTHREAD_EXIT_ERROR ((bo_pthread_retval_t)-1)

# ifdef PTHREAD_CANCELED
#  define BO_PTHREAD_CANCELED PTHREAD_CANCELED
# else
#  define BO_PTHREAD_CANCELED ((bo_pthread_retval_t)1)
# endif

# define _BoPThreadExit pthread_exit

/*
 * S-MUTEX
 */

# define BO_SMUTEX_IS_PMUTEX

typedef bo_pmutex_t         bo_smutex_t;
typedef bo_pmutex_ref       bo_smutex_ref;
typedef bo_pmutexattr_t     bo_smutexattr_t;
typedef bo_pmutexattr_ref   bo_smutexattr_ref;

# define BO_SMUTEX_ZERO_INITIALIZER BO_PMUTEX_ZERO_INITIALIZER
# define BO_SMUTEXATTR_ZERO_INITIALIZER BO_PMUTEXATTR_ZERO_INITIALIZER

# define BO_SMUTEX_REF(x) BO_PMUTEX_REF(x)
# define BO_SMUTEXATTR_REF(x) BO_PMUTEXATTR_REF(x)

/*
 * S-CONDITION
 */

# define BO_SCOND_IS_PCOND

typedef bo_pcond_t          bo_scond_t;
typedef bo_pcond_ref        bo_scond_ref;
typedef bo_pcondattr_t      bo_scondattr_t;
typedef bo_pcondattr_ref    bo_scondattr_ref;

# define BO_SCOND_ZERO_INITIALIZER BO_PCOND_ZERO_INITIALIZER
# define BO_SCONDATTR_ZERO_INITIALIZER BO_PCONDATTR_ZERO_INITIALIZER

# define BO_SCOND_REF(x) BO_PCOND_REF(x)
# define BO_SCONDATTR_REF(x) BO_PCONDATTR_REF(x)

/*
 * MUTEX
 */

typedef bo_pmutex_t         bo_mutex_t;
typedef bo_pmutex_ref       bo_mutex_ref;
typedef bo_pmutexattr_t     bo_mutexattr_t;
typedef bo_pmutexattr_ref   bo_mutexattr_ref;

# define BO_MUTEX_ZERO_INITIALIZER BO_PMUTEX_ZERO_INITIALIZER
# define BO_MUTEXATTR_ZERO_INITIALIZER BO_PMUTEXATTR_ZERO_INITIALIZER

# define BO_MUTEX_REF(x) BO_PMUTEX_REF(x)
# define BO_MUTEXATTR_REF(x) BO_PMUTEXATTR_REF(x)

/*
 * CONDITION
 */

typedef bo_pcond_t          bo_cond_t;
typedef bo_pcond_ref        bo_cond_ref;
typedef bo_pcondattr_t      bo_condattr_t;
typedef bo_pcondattr_ref    bo_condattr_ref;

# define BO_COND_ZERO_INITIALIZER BO_PCOND_ZERO_INITIALIZER
# define BO_CONDATTR_ZERO_INITIALIZER BO_PCONDATTR_ZERO_INITIALIZER

# define BO_COND_REF(x) BO_PCOND_REF(x)
# define BO_CONDATTR_REF(x) BO_PCONDATTR_REF(x)

#endif /* !__BO_LINUX__THRTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
