/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Atomic.h	(V.Sokur)
 * include/Bo/sys/macosx/_Atomic.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_MACOSX__ATOMIC__
# define __BO_MACOSX__ATOMIC__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# include __MYC_LIBC_HEADER(libkern/OSAtomic.h)

/******************************************************************************
 * %% BeginSection: definitions
 */

# define BoAtomicMemoryBarrier() OSMemoryBarrier()
# define BoAtomicWriteMemoryBarrier() BoAtomicMemoryBarrier()
# define BoAtomicReadMemoryBarrier() BoAtomicMemoryBarrier()

/* ----------------------------------------------------------------------------
 * 32-bit operations
 */

/*
 * int BoAtomicCompareStore32(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 * int BoAtomicCompareStore32_acq(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 * int BoAtomicCompareStore32_rel(volatile u_int32_t *aDst, u_int32_t aComparand, u_int32_t aValue);
 */

# define BoAtomicCompareStore32(aDst,aComparand,aValue) \
	((int) OSAtomicCompareAndSwap32((int32_t)(aComparand), (int32_t)(aValue), (volatile int32_t *)(aDst)))

# define BoAtomicCompareStore32_acq(aDst,aComparand,aValue) \
	((int) OSAtomicCompareAndSwap32Barrier((int32_t)(aComparand), (int32_t)(aValue), (volatile int32_t *)(aDst)))

# define BoAtomicCompareStore32_rel(aDst,aComparand,aValue) \
	((int) OSAtomicCompareAndSwap32Barrier((int32_t)(aComparand), (int32_t)(aValue), (volatile int32_t *)(aDst)))

/*
 * void BoAtomicSetmask32(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicSetmask32_acq(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicSetmask32_rel(volatile u_int32_t *aPtr, u_int32_t aMask);
 */
# define BoAtomicSetmask32(aPtr,aMask) ((void) BoAtomicRetOr32(aPtr,aMask))
# define BoAtomicSetmask32_acq(aPtr,aMask) ((void) BoAtomicRetOr32_acq(aPtr,aMask))
# define BoAtomicSetmask32_rel(aPtr,aMask) ((void) BoAtomicRetOr32_rel(aPtr,aMask))

/*
 * void BoAtomicClearmask32(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicClearmask32_acq(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicClearmask32_rel(volatile u_int32_t *aPtr, u_int32_t aMask);
 */
# define BoAtomicClearmask32(aPtr,aMask) ((void) BoAtomicRetAnd32(aPtr,(~(u_int32_t)(aMask))))
# define BoAtomicClearmask32_acq(aPtr,aMask) ((void) BoAtomicRetAnd32_acq(aPtr,(~(u_int32_t)(aMask))))
# define BoAtomicClearmask32_rel(aPtr,aMask) ((void) BoAtomicRetAnd32_rel(aPtr,(~(u_int32_t)(aMask))))

/*
 * void BoAtomicAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */
# define BoAtomicAdd32(aPtr,aValue) ((void) BoAtomicRetAdd32(aPtr,aValue))
# define BoAtomicAdd32_acq(aPtr,aValue) ((void) BoAtomicRetAdd32_acq(aPtr,aValue))
# define BoAtomicAdd32_rel(aPtr,aValue) ((void) BoAtomicRetAdd32_rel(aPtr,aValue))

/*
 * u_int32_t BoAtomicFetchAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicFetchAdd32(aPtr,aValue) \
	(BoAtomicRetAdd32(aPtr, aValue) - (u_int32_t)(aValue))

# define BoAtomicFetchAdd32_acq(aPtr,aValue) \
	(BoAtomicRetAdd32_acq(aPtr, aValue) - (u_int32_t)(aValue))

# define BoAtomicFetchAdd32_rel(aPtr,aValue) \
	(BoAtomicRetAdd32_rel(aPtr, aValue) - (u_int32_t)(aValue))

/*
 * u_int32_t BoAtomicRetAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicRetAdd32(aPtr,aValue) \
	OSAtomicAdd32((int32_t)(aValue), (volatile int32_t *)(aPtr))

# define BoAtomicRetAdd32_acq(aPtr,aValue) \
	OSAtomicAdd32Barrier((int32_t)(aValue), (volatile int32_t *)(aPtr))

# define BoAtomicRetAdd32_rel(aPtr,aValue) \
	OSAtomicAdd32Barrier((int32_t)(aValue), (volatile int32_t *)(aPtr))

/*
 * u_int32_t BoAtomicInc32(volatile u_int32_t *aPtr);
 * u_int32_t BoAtomicDec32(volatile u_int32_t *aPtr);
 */
# define BoAtomicInc32(aPtr) OSAtomicIncrement32((int32_t *)(aPtr))
# define BoAtomicDec32(aPtr) OSAtomicDecrement32((int32_t *)(aPtr))

/*
 * u_int32_t BoAtomicRetAnd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetAnd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetAnd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicRetAnd32(aPtr,aValue) \
	OSAtomicAnd32((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

# define BoAtomicRetAnd32_acq(aPtr,aValue) \
	OSAtomicAnd32Barrier((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

# define BoAtomicRetAnd32_rel(aPtr,aValue) \
	OSAtomicAnd32Barrier((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

/*
 * u_int32_t BoAtomicRetOr32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetOr32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetOr32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicRetOr32(aPtr,aValue) \
	OSAtomicOr32((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

# define BoAtomicRetOr32_acq(aPtr,aValue) \
	OSAtomicOr32Barrier((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

# define BoAtomicRetOr32_rel(aPtr,aValue) \
	OSAtomicOr32Barrier((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

/*
 * u_int32_t BoAtomicRetXor32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetXor32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetXor32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicRetXor32(aPtr,aValue) \
	OSAtomicXor32((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

# define BoAtomicRetXor32_acq(aPtr,aValue) \
	OSAtomicXor32Barrier((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

# define BoAtomicRetXor32_rel(aPtr,aValue) \
	OSAtomicXor32Barrier((u_int32_t)(aValue), (volatile u_int32_t *)(aPtr))

/* ----------------------------------------------------------------------------
 * Bit operations
 */

/*
 * int BoAtomicBitTestAndSet32(volatile u_int32_t *aDst, unsigned aBit);
 * int BoAtomicBitTestAndClear32(volatile u_int32_t *aDst, unsigned aBit);
 */

# define BoAtomicBitTestAndSet32(aDst,aBit) \
	((int) OSAtomicTestAndSet((u_int32_t)(aBit),((bo_pointer_t)(aDst))))

# define BoAtomicTestAndClear32(aDst,aBit) \
	((int) OSAtomicTestAndClear((u_int32_t)(aBit),((bo_pointer_t)(aDst))))

#endif /* !__BO_MACOSX__ATOMIC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
