/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Errors.h	(V.Sokur)
 * include/Bo/sys/posix/_Errors.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_POSIX__ERRORS__
# define __BO_POSIX__ERRORS__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <errno.h>

/******************************************************************************
 * %% BeginSection: definitions
 */

/* @todo: fixme */

# ifndef EPERM
#  define EPERM 1               /* Operation not permitted */
# endif
# ifndef ENOENT
#  define ENOENT 2              /* No such file or directory */
# endif
# ifndef ESRCH
#  define ESRCH 3               /* No such process */
# endif
# ifndef EINTR
#  define EINTR 4               /* Interrupted system call */
# endif
# ifndef EIO
#  define EIO 5                 /* Input/output error */
# endif
# ifndef ENXIO
#  define ENXIO 6               /* Device not configured */
# endif
# ifndef E2BIG
#  define E2BIG 7               /* Argument list too long */
# endif
# ifndef ENOEXEC
#  define ENOEXEC 8             /* Exec format error */
# endif
# ifndef EBADF
#  define EBADF 9               /* Bad file descriptor */
# endif
# ifndef ECHILD
#  define ECHILD 10             /* No child processes */
# endif
# ifndef EDEADLK
#  define EDEADLK 11            /* Resource deadlock avoided */
                                /* 11 was EAGAIN */
# endif
# ifndef ENOMEM
#  define ENOMEM 12             /* Cannot allocate memory */
# endif
# ifndef EACCES
#  define EACCES 13             /* Permission denied */
# endif
# ifndef EFAULT
#  define EFAULT 14             /* Bad address */
# endif

# ifndef _POSIX_SOURCE
#  ifndef ENOTBLK
#   define ENOTBLK 15           /* Block device required */
#  endif
# endif

# ifndef EBUSY
#  define EBUSY 16              /* Device busy */
# endif
# ifndef EEXIST
#  define EEXIST 17             /* File exists */
# endif
# ifndef EXDEV
#  define EXDEV 18              /* Cross-device link */
# endif
# ifndef ENODEV
#  define ENODEV 19             /* Operation not supported by device */
# endif
# ifndef ENOTDIR
#  define ENOTDIR 20            /* Not a directory */
# endif
# ifndef EISDIR
#  define EISDIR 21             /* Is a directory */
# endif
# ifndef EINVAL
#  define EINVAL 22             /* Invalid argument */
# endif
# ifndef ENFILE
#  define ENFILE 23             /* Too many open files in system */
# endif
# ifndef EMFILE
#  define EMFILE 24             /* Too many open files */
# endif
# ifndef ENOTTY
#  define ENOTTY 25             /* Inappropriate ioctl for device */
# endif

# ifndef _POSIX_SOURCE
#  ifndef ETXTBSY
#   define ETXTBSY 26           /* Text file busy */
#  endif
# endif

# ifndef EFBIG
#  define EFBIG 27              /* File too large */
# endif
# ifndef ENOSPC
#  define ENOSPC 28             /* No space left on device */
# endif
# ifndef ESPIPE
#  define ESPIPE 29             /* Illegal seek */
# endif
# ifndef EROFS
#  define EROFS 30              /* Read-only filesystem */
# endif
# ifndef EMLINK
#  define EMLINK 31             /* Too many links */
# endif
# ifndef EPIPE
#  define EPIPE 32              /* Broken pipe */
# endif

/* math software */
# ifndef EDOM
#  define EDOM 33               /* Numerical argument out of domain */
# endif
# ifndef ERANGE
#  define ERANGE 34             /* Result too large */
# endif

/* non-blocking and interrupt i/o */
# ifndef EAGAIN
#  define EAGAIN 35             /* Resource temporarily unavailable */
# endif

#endif /* !__BO_POSIX__ERRORS__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
