/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Atomic.h	(V.Sokur)
 * include/Bo/sys/w32/_Atomic.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_W32__ATOMIC__
# define __BO_W32__ATOMIC__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# include __MYC_LIBC_HEADER(windows.h)

/******************************************************************************
 * %% BeginSection: definitions
 */

# define BoAtomicMemoryBarrier() MemoryBarrier()
# define BoAtomicWriteMemoryBarrier() BoAtomicMemoryBarrier()
# define BoAtomicReadMemoryBarrier() BoAtomicMemoryBarrier()

/* @todo: I don't know why the <winbase.h> does not contain these functions. */
# if defined(_A_I386) && (_MSC_VER < 1800)
FORCEINLINE
LONG
InterlockedAnd (
    __inout LONG volatile *Destination,
    __in    LONG Value
    )
{
    LONG Old;

    do {
        Old = *Destination;
    } while (InterlockedCompareExchange(Destination,
                                          Old & Value,
                                          Old) != Old);

    return Old;
}

FORCEINLINE
LONG
InterlockedOr (
    __inout LONG volatile *Destination,
    __in    LONG Value
    )
{
    LONG Old;

    do {
        Old = *Destination;
    } while (InterlockedCompareExchange(Destination,
                                          Old | Value,
                                          Old) != Old);

    return Old;
}

FORCEINLINE
LONG
InterlockedXor (
    __inout LONG volatile *Destination,
    __in    LONG Value
    )
{
    LONG Old;

    do {
        Old = *Destination;
    } while (InterlockedCompareExchange(Destination,
                                          Old ^ Value,
                                          Old) != Old);

    return Old;
}
# endif /* _A_I386 */

/* ----------------------------------------------------------------------------
 * 32-bit operations
 */

/*
 * void BoAtomicSetmask32(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicSetmask32_acq(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicSetmask32_rel(volatile u_int32_t *aPtr, u_int32_t aMask);
 */
# define BoAtomicSetmask32(aPtr,aMask) ((void) BoAtomicFetchOr32(aPtr,aMask))
# define BoAtomicSetmask32_acq(aPtr,aMask) ((void) BoAtomicFetchOr32_acq(aPtr,aMask))
# define BoAtomicSetmask32_rel(aPtr,aMask) ((void) BoAtomicFetchOr32_rel(aPtr,aMask))

/*
 * void BoAtomicClearmask32(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicClearmask32_acq(volatile u_int32_t *aPtr, u_int32_t aMask);
 * void BoAtomicClearmask32_rel(volatile u_int32_t *aPtr, u_int32_t aMask);
 */
# define BoAtomicClearmask32(aPtr,aMask) ((void) BoAtomicFetchAnd32(aPtr,(~(u_int32_t)(aMask))))
# define BoAtomicClearmask32_acq(aPtr,aMask) ((void) BoAtomicFetchAnd32_acq(aPtr,(~(u_int32_t)(aMask))))
# define BoAtomicClearmask32_rel(aPtr,aMask) ((void) BoAtomicFetchAnd32_rel(aPtr,(~(u_int32_t)(aMask))))

/*
 * void BoAtomicAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * void BoAtomicAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */
# define BoAtomicAdd32(aPtr,aValue) ((void) BoAtomicFetchAdd32(aPtr,aValue))
# define BoAtomicAdd32_acq(aPtr,aValue) ((void) BoAtomicFetchAdd32_acq(aPtr,aValue))
# define BoAtomicAdd32_rel(aPtr,aValue) ((void) BoAtomicFetchAdd32_rel(aPtr,aValue))

/*
 * u_int32_t BoAtomicFetchAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicFetchAdd32(aPtr,aValue) \
	InterlockedExchangeAdd((volatile LONG *)(aPtr),(LONG)(aValue))

# ifdef InterlockedExchangeAddAcquire
#  define BoAtomicFetchAdd32_acq(aPtr,aValue) \
	InterlockedExchangeAddAcquire((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchAdd32_acq(aPtr,aValue) \
	InterlockedExchangeAdd((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

# ifdef InterlockedExchangeAddRelease
#  define BoAtomicFetchAdd32_rel(aPtr,aValue) \
	InterlockedExchangeAddRelease((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchAdd32_rel(aPtr,aValue) \
	InterlockedExchangeAdd((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

/*
 * u_int32_t BoAtomicRetAdd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetAdd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicRetAdd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicRetAdd32(aPtr,aValue) \
	(BoAtomicFetchAdd32(aPtr, aValue) + (u_int32_t)(aValue))

# define BoAtomicRetAdd32_acq(aPtr,aValue) \
	(BoAtomicFetchAdd32_acq(aPtr, aValue) + (u_int32_t)(aValue))

# define BoAtomicRetAdd32_rel(aPtr,aValue) \
	(BoAtomicFetchAdd32_rel(aPtr, aValue) + (u_int32_t)(aValue))

/*
 * u_int32_t BoAtomicInc32(volatile u_int32_t *aPtr);
 * u_int32_t BoAtomicDec32(volatile u_int32_t *aPtr);
 */
# define BoAtomicInc32(aPtr) InterlockedIncrement((LONG *)(aPtr))
# define BoAtomicDec32(aPtr) InterlockedDecrement((LONG *)(aPtr))

/*
 * u_int32_t BoAtomicFetchStore32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchStore32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchStore32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicFetchStore32(aPtr,aValue) \
	InterlockedExchange((volatile LONG *)(aPtr),(LONG)(aValue))

# ifdef InterlockedExchangeAcquire
#  define BoAtomicFetchStore32_acq(aPtr,aValue) \
	InterlockedExchangeAcquire((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchStore32_acq(aPtr,aValue) \
	InterlockedExchange((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

# ifdef InterlockedExchangeRelease
#  define BoAtomicFetchStore32_rel(aPtr,aValue) \
	InterlockedExchangeRelease((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchStore32_rel(aPtr,aValue) \
	InterlockedExchange((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

/*
 * int BoAtomicCompareStore32(volatile u_int32_t *aPtr, u_int32_t aComparand, u_int32_t aValue);
 * int BoAtomicCompareStore32_acq(volatile u_int32_t *aPtr, u_int32_t aComparand, u_int32_t aValue);
 * int BoAtomicCompareStore32_rel(volatile u_int32_t *aPtr, u_int32_t aComparand, u_int32_t aValue);
 */

# define BoAtomicCompareStore32(aPtr,aComparand,aValue) \
	_BoAtomicCompareStore32((volatile u_int32_t *)(aPtr),(u_int32_t)(aComparand),(u_int32_t)(aValue))

__myc_extern_inline__ int
_BoAtomicCompareStore32(volatile u_int32_t *aPtr, u_int32_t aComparand, u_int32_t aValue)
{
	return ((int) ((LONG)(aComparand) == InterlockedCompareExchange(
		(volatile LONG *)(aPtr),(LONG)(aValue),(LONG)(aComparand)
	)));
}

# define BoAtomicCompareStore32_acq(aPtr,aComparand,aValue) \
	_BoAtomicCompareStore32_acq((volatile u_int32_t *)(aPtr),(u_int32_t)(aComparand),(u_int32_t)(aValue))

__myc_extern_inline__ int
_BoAtomicCompareStore32_acq(volatile u_int32_t *aPtr, u_int32_t aComparand, u_int32_t aValue)
{
# ifdef InterlockedCompareExchangeAcquire
	return ((int) ((LONG)(aComparand) == InterlockedCompareExchangeAcquire(
		(volatile LONG *)(aPtr),(LONG)(aValue),(LONG)(aComparand)
	)));
# else
	return ((int) ((LONG)(aComparand) == InterlockedCompareExchange(
		(volatile LONG *)(aPtr),(LONG)(aValue),(LONG)(aComparand)
	)));
# endif
}

# define BoAtomicCompareStore32_rel(aPtr,aComparand,aValue) \
	_BoAtomicCompareStore32_rel((volatile u_int32_t *)(aPtr),(u_int32_t)(aComparand),(u_int32_t)(aValue))

__myc_extern_inline__ int
_BoAtomicCompareStore32_rel(volatile u_int32_t *aPtr, u_int32_t aComparand, u_int32_t aValue)
{
# ifdef InterlockedCompareExchangeRelease
	return ((int) ((LONG)(aComparand) == InterlockedCompareExchangeRelease(
		(volatile LONG *)(aPtr),(LONG)(aValue),(LONG)(aComparand)
	)));
# else
	return ((int) ((LONG)(aComparand) == InterlockedCompareExchange(
		(volatile LONG *)(aPtr),(LONG)(aValue),(LONG)(aComparand)
	)));
# endif
}

/*
 * u_int32_t BoAtomicFetchAnd32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAnd32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchAnd32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicFetchAnd32(aPtr,aValue) \
	InterlockedAnd((volatile LONG *)(aPtr),(LONG)(aValue))

# ifdef InterlockedAndAcquire
#  define BoAtomicFetchAnd32_acq(aPtr,aValue) \
	InterlockedAndAcquire((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchAnd32_acq(aPtr,aValue) \
	InterlockedAnd((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

# ifdef InterlockedAndRelease
#  define BoAtomicFetchAnd32_rel(aPtr,aValue) \
	InterlockedAndRelease((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchAnd32_rel(aPtr,aValue) \
	InterlockedAnd((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

/*
 * u_int32_t BoAtomicFetchOr32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchOr32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchOr32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicFetchOr32(aPtr,aValue) \
	InterlockedOr((volatile LONG *)(aPtr),(LONG)(aValue))

# ifdef InterlockedOrAcquire
#  define BoAtomicFetchOr32_acq(aPtr,aValue) \
	InterlockedOrAcquire((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchOr32_acq(aPtr,aValue) \
	InterlockedOr((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

# ifdef InterlockedOrRelease
#  define BoAtomicFetchOr32_rel(aPtr,aValue) \
	InterlockedOrRelease((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchOr32_rel(aPtr,aValue) \
	InterlockedOr((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

/*
 * u_int32_t BoAtomicFetchXor32(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchXor32_acq(volatile u_int32_t *aPtr, u_int32_t aValue);
 * u_int32_t BoAtomicFetchXor32_rel(volatile u_int32_t *aPtr, u_int32_t aValue);
 */

# define BoAtomicFetchXor32(aPtr,aValue) \
	InterlockedXor((volatile LONG *)(aPtr),(LONG)(aValue))

# ifdef InterlockedXorAcquire
#  define BoAtomicFetchXor32_acq(aPtr,aValue) \
	InterlockedXorAcquire((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchXor32_acq(aPtr,aValue) \
	InterlockedXor((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

# ifdef InterlockedXorRelease
#  define BoAtomicFetchXor32_rel(aPtr,aValue) \
	InterlockedXorRelease((volatile LONG *)(aPtr),(LONG)(aValue))
# else
#  define BoAtomicFetchXor32_rel(aPtr,aValue) \
	InterlockedXor((volatile LONG *)(aPtr),(LONG)(aValue))
# endif

/* ----------------------------------------------------------------------------
 * 64-bit operations
 */

/*
 * void BoAtomicSetmask64(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicSetmask64_acq(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicSetmask64_rel(volatile u_int64_t *aPtr, u_int64_t aMask);
 */
# define BoAtomicSetmask64(aPtr,aMask) ((void) BoAtomicFetchOr64(aPtr,aMask))
# define BoAtomicSetmask64_acq(aPtr,aMask) ((void) BoAtomicFetchOr64_acq(aPtr,aMask))
# define BoAtomicSetmask64_rel(aPtr,aMask) ((void) BoAtomicFetchOr64_rel(aPtr,aMask))

/*
 * void BoAtomicClearmask64(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicClearmask64_acq(volatile u_int64_t *aPtr, u_int64_t aMask);
 * void BoAtomicClearmask64_rel(volatile u_int64_t *aPtr, u_int64_t aMask);
 */
# define BoAtomicClearmask64(aPtr,aMask) ((void) BoAtomicFetchAnd64(aPtr,(~(u_int64_t)(aMask))))
# define BoAtomicClearmask64_acq(aPtr,aMask) ((void) BoAtomicFetchAnd64_acq(aPtr,(~(u_int64_t)(aMask))))
# define BoAtomicClearmask64_rel(aPtr,aMask) ((void) BoAtomicFetchAnd64_rel(aPtr,(~(u_int64_t)(aMask))))

/*
 * void BoAtomicAdd64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicAdd64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * void BoAtomicAdd64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */
# define BoAtomicAdd64(aPtr,aValue) ((void) BoAtomicFetchAdd64(aPtr,aValue))
# define BoAtomicAdd64_acq(aPtr,aValue) ((void) BoAtomicFetchAdd64_acq(aPtr,aValue))
# define BoAtomicAdd64_rel(aPtr,aValue) ((void) BoAtomicFetchAdd64_rel(aPtr,aValue))

/*
 * u_int64_t BoAtomicFetchAdd64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchAdd64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchAdd64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# define BoAtomicFetchAdd64(aPtr,aValue) \
	InterlockedExchangeAdd64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))

# ifdef InterlockedExchangeAddAcquire64
#  define BoAtomicFetchAdd64_acq(aPtr,aValue) \
	InterlockedExchangeAddAcquire64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchAdd64_acq(aPtr,aValue) \
	InterlockedExchangeAdd64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

# ifdef InterlockedExchangeAddRelease64
#  define BoAtomicFetchAdd64_rel(aPtr,aValue) \
	InterlockedExchangeAddRelease64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchAdd64_rel(aPtr,aValue) \
	InterlockedExchangeAdd64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

/*
 * u_int64_t BoAtomicRetAdd64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicRetAdd64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicRetAdd64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# define BoAtomicRetAdd64(aPtr,aValue) \
	(BoAtomicFetchAdd64(aPtr, aValue) + (u_int64_t)(aValue))

# define BoAtomicRetAdd64_acq(aPtr,aValue) \
	(BoAtomicFetchAdd64_acq(aPtr, aValue) + (u_int64_t)(aValue))

# define BoAtomicRetAdd64_rel(aPtr,aValue) \
	(BoAtomicFetchAdd64_rel(aPtr, aValue) + (u_int64_t)(aValue))

/*
 * u_int64_t BoAtomicInc64(volatile u_int64_t *aPtr);
 * u_int64_t BoAtomicDec64(volatile u_int64_t *aPtr);
 */
# define BoAtomicInc64(aPtr) InterlockedIncrement64((LONGLONG *)(aPtr))
# define BoAtomicDec64(aPtr) InterlockedDecrement64((LONGLONG *)(aPtr))

/*
 * u_int64_t BoAtomicFetchStore64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchStore64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchStore64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# define BoAtomicFetchStore64(aPtr,aValue) \
	InterlockedExchange64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))

# ifdef InterlockedExchangeAcquire64
#  define BoAtomicFetchStore64_acq(aPtr,aValue) \
	InterlockedExchangeAcquire64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchStore64_acq(aPtr,aValue) \
	InterlockedExchange64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

# ifdef InterlockedExchangeRelease64
#  define BoAtomicFetchStore64_rel(aPtr,aValue) \
	InterlockedExchangeRelease64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchStore64_rel(aPtr,aValue) \
	InterlockedExchange64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

/*
 * int BoAtomicCompareStore64(volatile u_int64_t *aPtr, u_int64_t aComparand, u_int64_t aValue);
 * int BoAtomicCompareStore64_acq(volatile u_int64_t *aPtr, u_int64_t aComparand, u_int64_t aValue);
 * int BoAtomicCompareStore64_rel(volatile u_int64_t *aPtr, u_int64_t aComparand, u_int64_t aValue);
 */

# ifdef InterlockedCompareExchange64
#  define BoAtomicCompareStore64(aPtr,aComparand,aValue) \
	_BoAtomicCompareStore64((volatile u_int64_t *)(aPtr),(u_int64_t)(aComparand),(u_int64_t)(aValue))

__myc_extern_inline__ int
_BoAtomicCompareStore64(volatile u_int64_t *aPtr, u_int64_t aComparand, u_int64_t aValue)
{
	return ((int) ((LONGLONG)(aComparand) == InterlockedCompareExchange64(
		(volatile LONGLONG *)(aPtr),(LONGLONG)(aValue),(LONGLONG)(aComparand)
	)));
}

# define BoAtomicCompareStore64_acq(aPtr,aComparand,aValue) \
	_BoAtomicCompareStore64_acq((volatile u_int64_t *)(aPtr),(u_int64_t)(aComparand),(u_int64_t)(aValue))

__myc_extern_inline__ int
_BoAtomicCompareStore64_acq(volatile u_int64_t *aPtr, u_int64_t aComparand, u_int64_t aValue)
{
# ifdef InterlockedCompareExchangeAcquire64
	return ((int) ((LONGLONG)(aComparand) == InterlockedCompareExchangeAcquire64(
		(volatile LONGLONG *)(aPtr),(LONGLONG)(aValue),(LONGLONG)(aComparand)
	)));
# else
	return ((int) ((LONGLONG)(aComparand) == InterlockedCompareExchange64(
		(volatile LONGLONG *)(aPtr),(LONGLONG)(aValue),(LONGLONG)(aComparand)
	)));
# endif
}

# define BoAtomicCompareStore64_rel(aPtr,aComparand,aValue) \
	_BoAtomicCompareStore64_rel((volatile u_int64_t *)(aPtr),(u_int64_t)(aComparand),(u_int64_t)(aValue))

__myc_extern_inline__ int
_BoAtomicCompareStore64_rel(volatile u_int64_t *aPtr, u_int64_t aComparand, u_int64_t aValue)
{
# ifdef InterlockedCompareExchangeRelease64
	return ((int) ((LONGLONG)(aComparand) == InterlockedCompareExchangeRelease64(
		(volatile LONGLONG *)(aPtr),(LONGLONG)(aValue),(LONGLONG)(aComparand)
	)));
# else
	return ((int) ((LONGLONG)(aComparand) == InterlockedCompareExchange64(
		(volatile LONGLONG *)(aPtr),(LONGLONG)(aValue),(LONGLONG)(aComparand)
	)));
# endif
}
# endif /* defined(InterlockedCompareExchange64) */

/*
 * u_int64_t BoAtomicFetchAnd64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchAnd64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchAnd64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# define BoAtomicFetchAnd64(aPtr,aValue) \
	InterlockedAnd64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))

# ifdef InterlockedAndAcquire64
#  define BoAtomicFetchAnd64_acq(aPtr,aValue) \
	InterlockedAndAcquire64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchAnd64_acq(aPtr,aValue) \
	InterlockedAnd64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

# ifdef InterlockedAndRelease64
#  define BoAtomicFetchAnd64_rel(aPtr,aValue) \
	InterlockedAndRelease64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchAnd64_rel(aPtr,aValue) \
	InterlockedAnd64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

/*
 * u_int64_t BoAtomicFetchOr64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchOr64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchOr64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# define BoAtomicFetchOr64(aPtr,aValue) \
	InterlockedOr64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))

# ifdef InterlockedOrAcquire64
#  define BoAtomicFetchOr64_acq(aPtr,aValue) \
	InterlockedOrAcquire64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchOr64_acq(aPtr,aValue) \
	InterlockedOr64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

# ifdef InterlockedOrRelease64
#  define BoAtomicFetchOr64_rel(aPtr,aValue) \
	InterlockedOrRelease64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchOr64_rel(aPtr,aValue) \
	InterlockedOr64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

/*
 * u_int64_t BoAtomicFetchXor64(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchXor64_acq(volatile u_int64_t *aPtr, u_int64_t aValue);
 * u_int64_t BoAtomicFetchXor64_rel(volatile u_int64_t *aPtr, u_int64_t aValue);
 */

# define BoAtomicFetchXor64(aPtr,aValue) \
	InterlockedXor64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))

# ifdef InterlockedXorAcquire64
#  define BoAtomicFetchXor64_acq(aPtr,aValue) \
	InterlockedXorAcquire64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchXor64_acq(aPtr,aValue) \
	InterlockedXor64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

# ifdef InterlockedXorRelease64
#  define BoAtomicFetchXor64_rel(aPtr,aValue) \
	InterlockedXorRelease64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# else
#  define BoAtomicFetchXor64_rel(aPtr,aValue) \
	InterlockedXor64((volatile LONGLONG *)(aPtr),(LONGLONG)(aValue))
# endif

#endif /* !__BO_W32__ATOMIC__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
