/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * _Thrtypes.h	(R.Falchuk)
 * include/Bo/sys/w32/_Thrtypes.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_W32__THRTYPES__
# define __BO_W32__THRTYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Bo/sys/Types.h>

# include __MYC_LIBC_HEADER(windows.h)
# include __MYC_LIBC_HEADER(process.h)

/******************************************************************************
 * %% BeginSection: type declarations
 */

/*
 * P-MUTEX
 */

typedef HANDLE                  bo_pmutex_t, bo_pmutex_ref;

struct bo_pmutexattr_s {
	SECURITY_ATTRIBUTES pma_sa;
	LPCTSTR             pma_name;
	DWORD               pma_flags;
	DWORD               pma_access;
};
typedef struct bo_pmutexattr_s  bo_pmutexattr_t, *bo_pmutexattr_ref;

# define BO_PMUTEX_ZERO_INITIALIZER NULL
# define BO_PMUTEXATTR_ZERO_INITIALIZER \
	{ { sizeof(SECURITY_ATTRIBUTES), NULL, FALSE }, NULL, 0, STANDARD_RIGHTS_REQUIRED }

# define BO_PMUTEX_REF(x) (x)
# define BO_PMUTEXATTR_REF(x) (&(x))

/*
 * P-CONDITION
 */

typedef HANDLE                  bo_pcond_t, bo_pcond_ref;

struct bo_pcondattr_s {
	SECURITY_ATTRIBUTES pca_sa;
	LPCTSTR             pca_name;
	DWORD               pca_flags;
	DWORD               pca_access;
};
typedef struct bo_pcondattr_s   bo_pcondattr_t, *bo_pcondattr_ref;

# define BO_PCOND_ZERO_INITIALIZER NULL
# define BO_PCONDATTR_ZERO_INITIALIZER \
	{ { sizeof(SECURITY_ATTRIBUTES), NULL, FALSE }, NULL, 0, STANDARD_RIGHTS_REQUIRED }

# define BO_PCOND_REF(x) (x)
# define BO_PCONDATTR_REF(x) (&(x))

/*
 * P-THREAD
 */

typedef HANDLE                  bo_pthread_t, bo_pthread_ref;

struct bo_pthrattr_s {
	SECURITY_ATTRIBUTES pta_sa;
	LPCTSTR             pta_name;  /* unused? */
	unsigned            pta_flags;
	unsigned            pta_stack_size;
	void               *pta_stack_addr;
	int					pta_priority;
	int					pta_shedpolicy;
};
typedef struct bo_pthrattr_s    bo_pthrattr_t, *bo_pthrattr_ref;

# define BO_PTHREAD_NULL NULL
# define BO_PTHREAD_ZERO_INITIALIZER NULL
# define BO_PTHRATTR_ZERO_INITIALIZER \
	{ { sizeof(SECURITY_ATTRIBUTES), NULL, FALSE }, NULL, 0, 0 }

# define BO_PTHREAD_REF(x) (x)
# define BO_PTHRATTR_REF(x) (&(x))

typedef unsigned bo_pthread_retval_t;
typedef bo_pthread_retval_t (__stdcall *bo_pthread_proc)(void *);

# define BO_PTHREAD_PROC_CALLTYPE __myc_calltype__(stdcall)

# define BO_PTHREAD_EXIT_NORMAL ((bo_pthread_retval_t)0)
# define BO_PTHREAD_EXIT_ERROR ((bo_pthread_retval_t)-1)

# ifdef PTHREAD_CANCELED
#  define BO_PTHREAD_CANCELED PTHREAD_CANCELED
# else
#  define BO_PTHREAD_CANCELED ((bo_pthread_retval_t)1)
# endif

# define _BoPThreadExit _endthreadex

/*
 * S-MUTEX
 */

typedef CRITICAL_SECTION    bo_smutex_t, *bo_smutex_ref;
typedef int                 bo_smutexattr_t, bo_smutexattr_ref;

# define BO_SMUTEX_ZERO_INITIALIZER {0}
# define BO_SMUTEXATTR_ZERO_INITIALIZER 0

# define BO_SMUTEX_REF(x) (&(x))
# define BO_SMUTEXATTR_REF(x) (x)

/*
 * S-CONDITION
 */

# if (0x0600 <= _BO_BUILD_WIN32_WINNT)

typedef CONDITION_VARIABLE  bo_scond_t, *bo_scond_ref;
typedef int                 bo_scondattr_t, bo_scondattr_ref;

# define BO_SCOND_ZERO_INITIALIZER {NULL}
# define BO_SCONDATTR_ZERO_INITIALIZER 0

# define BO_SCOND_REF(x) (&(x))
# define BO_SCONDATTR_REF(x) (x)

# else

/*
 * Dummy
 */
typedef int                 bo_scond_t, bo_scond_ref;
typedef int                 bo_scondattr_t, bo_scondattr_ref;

# define BO_SCOND_ZERO_INITIALIZER 0
# define BO_SCONDATTR_ZERO_INITIALIZER 0

# define BO_SCOND_REF(x) (x)
# define BO_SCONDATTR_REF(x) (x)

# endif /* (0x0600 <= _BO_BUILD_WIN32_WINNT) */

/*
 * MUTEX
 */

# if (0x0600 <= _BO_BUILD_WIN32_WINNT)

typedef bo_smutex_t         bo_mutex_t;
typedef bo_smutex_ref       bo_mutex_ref;
typedef bo_smutexattr_t     bo_mutexattr_t;
typedef bo_smutexattr_ref   bo_mutexattr_ref;

# define BO_MUTEX_ZERO_INITIALIZER BO_SMUTEX_ZERO_INITIALIZER
# define BO_MUTEXATTR_ZERO_INITIALIZER BO_SMUTEXATTR_ZERO_INITIALIZER

# define BO_MUTEX_REF(x) BO_SMUTEX_REF(x)
# define BO_MUTEXATTR_REF(x) BO_SMUTEXATTR_REF(x)

# else

typedef bo_pmutex_t         bo_mutex_t;
typedef bo_pmutex_ref       bo_mutex_ref;
typedef bo_pmutexattr_t     bo_mutexattr_t;
typedef bo_pmutexattr_ref   bo_mutexattr_ref;

# define BO_MUTEX_ZERO_INITIALIZER BO_PMUTEX_ZERO_INITIALIZER
# define BO_MUTEXATTR_ZERO_INITIALIZER BO_PMUTEXATTR_ZERO_INITIALIZER

# define BO_MUTEX_REF(x) BO_PMUTEX_REF(x)
# define BO_MUTEXATTR_REF(x) BO_PMUTEXATTR_REF(x)

# endif /* (0x0600 <= _BO_BUILD_WIN32_WINNT) */

/*
 * CONDITION
 */

# if (0x0600 <= _BO_BUILD_WIN32_WINNT)

typedef bo_scond_t         bo_cond_t;
typedef bo_scond_ref       bo_cond_ref;
typedef bo_scondattr_t     bo_condattr_t;
typedef bo_scondattr_ref   bo_condattr_ref;

# define BO_COND_ZERO_INITIALIZER BO_SCOND_ZERO_INITIALIZER
# define BO_CONDATTR_ZERO_INITIALIZER BO_SCONDATTR_ZERO_INITIALIZER

# define BO_COND_REF(x) BO_SCOND_REF(x)
# define BO_CONDATTR_REF(x) BO_SCONDATTR_REF(x)

# else

typedef bo_pcond_t          bo_cond_t;
typedef bo_pcond_ref        bo_cond_ref;
typedef bo_pcondattr_t      bo_condattr_t;
typedef bo_pcondattr_ref    bo_condattr_ref;

# define BO_COND_ZERO_INITIALIZER BO_PCOND_ZERO_INITIALIZER
# define BO_CONDATTR_ZERO_INITIALIZER BO_PCONDATTR_ZERO_INITIALIZER

# define BO_COND_REF(x) BO_PCOND_REF(x)
# define BO_CONDATTR_REF(x) BO_PCONDATTR_REF(x)

# endif /* (0x0600 <= _BO_BUILD_WIN32_WINNT) */

/*
 * L-SEMAPHORE
 */

# define BoL_LSEM_MAX 0x7fffffff

# define BoL_LSEM_OP_MAX 128
# define BoL_LSEM_NAME_MAX 245

struct bo_lsem {
	bo_pointer_t p_;
	HANDLE       handles_[3];
};

struct bo_lsemattr_s {
	SECURITY_ATTRIBUTES lsa_sa;
	LPCTSTR             lsa_name;
	DWORD               lsa_flags;
	DWORD               lsa_access;
	unsigned long       lsa_value;
};

typedef struct bo_lsem        bo_lsem_t, *bo_lsem_ref;
typedef struct bo_lsemattr_s  bo_lsemattr_t, *bo_lsemattr_ref;

# define BO_LSEM_ZERO_INITIALIZER { NULL, { NULL, NULL, NULL } }
# define BO_LSEMATTR_ZERO_INITIALIZER \
	{ { sizeof(SECURITY_ATTRIBUTES), NULL, FALSE }, NULL, 0, STANDARD_RIGHTS_REQUIRED, 0 }

# define BO_LSEM_REF(x) (&(x))
# define BO_LSEMATTR_REF(x) (&(x))

#endif /* !__BO_W32__THRTYPES__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
