/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2008 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * trpType.h	(V.Sokur)
 * include/Bo/trpType.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_TRPTYPE__
# define __BO_TRPTYPE__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SYS_TYPES__
#  include <Bo/sys/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: protocol constant declarations
 */
# pragma mark -
# pragma mark [ protocol constant declarations ]
# pragma mark -

# define BoK_OTI_INVAL_PROTOCOL (-1)
# define BoK_OTI_UNK_PROTOCOL (-2)

# define BO_OTI_NULL_PROTOCOL "-"
# define BO_OTI_NOPROTO BO_OTI_NULL_PROTOCOL

/*
 * loopback pseudo protocols
 * @{
 */
# define BoK_OTI_TICLTS_PROTOCOL (512) /* IPPROTO_UDP (???) */
# define BO_OTI_TICLTS_PROTOCOL "ticlts"

# define BoK_OTI_TICOTS_PROTOCOL (513) /* IPPROTO_TCP (???) */
# define BO_OTI_TICOTS_PROTOCOL "ticots"

# define BoK_OTI_TICOTSORD_PROTOCOL (514) /* IPPROTO_TCP (???) */
# define BO_OTI_TICOTSORD_PROTOCOL "ticotsord"
/* @} */

/* dummy for IP */
# define BoK_OTI_IP_PROTOCOL (0) /* IPPROTO_IP */
# define BO_OTI_IP_PROTOCOL "ip"

/* IPv6 Hop-by-Hop options */
# define BoK_OTI_HOPOPTS_PROTOCOL (0) /* IPPROTO_HOPOPTS */
# define BO_OTI_HOPOPTS_PROTOCOL "hopopts"

/* control message protocol */
# define BoK_OTI_ICMP_PROTOCOL (1) /* IPPROTO_ICMP */
# define BO_OTI_ICMP_PROTOCOL "icmp"

/* group management protocol */
# define BoK_OTI_IGMP_PROTOCOL (2) /* IPPROTO_IGMP */
# define BO_OTI_IGMP_PROTOCOL "igmp"

/* gateway^2 (deprecated) */
# define BoK_OTI_GGP_PROTOCOL (3) /* IPPROTO_GGP */
# define BO_OTI_GGP_PROTOCOL "ggp"

/* IPv4 encapsulation */
# define BoK_OTI_IPV4_PROTOCOL (4) /* IPPROTO_IPV4 */
# define BO_OTI_IPV4_PROTOCOL "ipv4"

/* transmission control protocol */
# define BoK_OTI_TCP_PROTOCOL (6) /* IPPROTO_TCP */
# define BO_OTI_TCP_PROTOCOL "tcp"

# define BoK_OTI_CBT_PROTOCOL (5) /* IPPROTO_CBT */
# define BO_OTI_CBT_PROTOCOL "cbt"

/* stream protocol II */
# define BoK_OTI_ST_PROTOCOL (7) /* IPPROTO_ST */
# define BO_OTI_ST_PROTOCOL "st"

/* exterior gateway protocol */
# define BoK_OTI_EGP_PROTOCOL (8) /* IPPROTO_EGP */
# define BO_OTI_EGP_PROTOCOL "egp"

/* private interior gateway protocol */
# define BoK_OTI_PIGP_PROTOCOL (9) /* IPPROTO_PIGP */
# define BO_OTI_PIGP_PROTOCOL "pigp"

/* BBN RCC Monitoring */
# define BoK_OTI_RCCMON_PROTOCOL (10) /* IPPROTO_RCCMON */
# define BO_OTI_RCCMON_PROTOCOL "rccmon"

/* network voice protocol*/
# define BoK_OTI_NVPII_PROTOCOL (11) /* IPPROTO_NVPII */
# define BO_OTI_NVPII_PROTOCOL "nvpii"

# define BoK_OTI_PUP_PROTOCOL (12) /* IPPROTO_PUP */
# define BO_OTI_PUP_PROTOCOL "pup"

/* Argus */
# define BoK_OTI_ARGUS_PROTOCOL (13) /* IPPROTO_ARGUS */
# define BO_OTI_ARGUS_PROTOCOL "argus"

/* EMCON */
# define BoK_OTI_EMCON_PROTOCOL (14) /* IPPROTO_EMCON */
# define BO_OTI_EMCON_PROTOCOL "emcon"

/* Cross Net Debugger */
# define BoK_OTI_XNET_PROTOCOL (15) /* IPPROTO_XNET */
# define BO_OTI_XNET_PROTOCOL "xnet"

/* Chaos*/
# define BoK_OTI_CHAOS_PROTOCOL (16) /* IPPROTO_CHAOS */
# define BO_OTI_CHAOS_PROTOCOL "chaos"

/* user datagram protocol */
# define BoK_OTI_UDP_PROTOCOL (17) /* IPPROTO_UDP */
# define BO_OTI_UDP_PROTOCOL "udp"

/* Multiplexing */
# define BoK_OTI_MUX_PROTOCOL (18) /* IPPROTO_MUX */
# define BO_OTI_MUX_PROTOCOL "mux"

/* DCN Measurement Subsystems */
# define BoK_OTI_MEAS_PROTOCOL (19) /* IPPROTO_MEAS */
# define BO_OTI_MEAS_PROTOCOL "meas"

/* Host Monitoring */
# define BoK_OTI_HMP_PROTOCOL (20) /* IPPROTO_HMP */
# define BO_OTI_HMP_PROTOCOL "hmp"

/* Packet Radio Measurement */
# define BoK_OTI_PRM_PROTOCOL (21) /* IPPROTO_PRM */
# define BO_OTI_PRM_PROTOCOL "prm"

/* xns idp */
# define BoK_OTI_IDP_PROTOCOL (22) /* IPPROTO_IDP */
# define BO_OTI_IDP_PROTOCOL "idp"

/* Trunk-1 */
# define BoK_OTI_TRUNK1_PROTOCOL (23) /* IPPROTO_TRUNK1 */
# define BO_OTI_TRUNK1_PROTOCOL "trunk1"

/* Trunk-2 */
# define BoK_OTI_TRUNK2_PROTOCOL (24) /* IPPROTO_TRUNK2 */
# define BO_OTI_TRUNK2_PROTOCOL "trunk2"

/* Leaf-1 */
# define BoK_OTI_LEAF1_PROTOCOL (25) /* IPPROTO_LEAF1*/
# define BO_OTI_LEAF1_PROTOCOL "leaf1"

/* Leaf-2 */
# define BoK_OTI_LEAF2_PROTOCOL (26) /* IPPROTO_LEAF2 */
# define BO_OTI_LEAF2_PROTOCOL "leaf2"

/* Reliable Data */
# define BoK_OTI_RDP_PROTOCOL (27) /* IPPROTO_RDP */
# define BO_OTI_RDP_PROTOCOL "rdp"

/* Reliable Transaction */
# define BoK_OTI_IRTP_PROTOCOL (28) /* IPPROTO_IRTP */
# define BO_OTI_IRTP_PROTOCOL "irtp"

/* tp-4 with class negotiation */
# define BoK_OTI_TP_PROTOCOL (29) /* IPPROTO_TP */
# define BO_OTI_TP_PROTOCOL "tp"

/* Bulk Data Transfer */
# define BoK_OTI_BLT_PROTOCOL (30) /* IPPROTO_BLT */
# define BO_OTI_BLT_PROTOCOL "blt"

/* Network Services */
# define BoK_OTI_NSP_PROTOCOL (31) /* IPPROTO_NSP */
# define BO_OTI_NSP_PROTOCOL "nsp"

/* Merit Internodal */
# define BoK_OTI_INP_PROTOCOL (32) /* IPPROTO_INP */
# define BO_OTI_INP_PROTOCOL "inp"

/* Sequential Exchange */
# define BoK_OTI_SEP_PROTOCOL (33) /* IPPROTO_SEP */
# define BO_OTI_SEP_PROTOCOL "sep"

/* Third Party Connect */
# define BoK_OTI_3PC_PROTOCOL (34) /* IPPROTO_3PC */
# define BO_OTI_3PC_PROTOCOL "3pc"

/* InterDomain Policy Routing */
# define BoK_OTI_IDPR_PROTOCOL (35) /* IPPROTO_IDPR */
# define BO_OTI_IDPR_PROTOCOL "idpr"

/* XTP */
# define BoK_OTI_XTP_PROTOCOL (36) /* IPPROTO_XTP */
# define BO_OTI_XTP_PROTOCOL "xtp"

/* Datagram Delivery */
# define BoK_OTI_DDP_PROTOCOL (37) /* IPPROTO_DDP */
# define BO_OTI_DDP_PROTOCOL "ddp"

/* Control Message Transport */
# define BoK_OTI_CMTP_PROTOCOL (38) /* IPPROTO_CMTP */
# define BO_OTI_CMTP_PROTOCOL "cmtp"

/* TP++ Transport */
# define BoK_OTI_TPXX_PROTOCOL (39) /* IPPROTO_TPXX */
# define BO_OTI_TPXX_PROTOCOL "tpxx"

/* IL transport protocol */
# define BoK_OTI_IL_PROTOCOL (40) /* IPPROTO_IL */
# define BO_OTI_IL_PROTOCOL "il"

/* IPv6 header */
# define BoK_OTI_IPV6_PROTOCOL (41) /* IPPROTO_IPV6 */
# define BO_OTI_IPV6_PROTOCOL "ipv6"

/* Source Demand Routing */
# define BoK_OTI_SDRP_PROTOCOL (42) /* IPPROTO_SDRP */
# define BO_OTI_SDRP_PROTOCOL "sdrp"

/* IPv6 Routing header */
# define BoK_OTI_ROUTING_PROTOCOL (43) /* IPPROTO_ROUTING */
# define BO_OTI_ROUTING_PROTOCOL "routing"

/* IPv6 fragmentation header */
# define BoK_OTI_FRAGMENT_PROTOCOL (44) /* IPPROTO_FRAGMENT */
# define BO_OTI_FRAGMENT_PROTOCOL "fragment"

/* InterDomain Routing */
# define BoK_OTI_IDRP_PROTOCOL (45) /* IPPROTO_IDRP */
# define BO_OTI_IDRP_PROTOCOL "idrp"

/* resource reservation */
# define BoK_OTI_RSVP_PROTOCOL (46) /* IPPROTO_RSVP */
# define BO_OTI_RSVP_PROTOCOL "rsvp"

/* General Routing Encap. */
# define BoK_OTI_GRE_PROTOCOL (47) /* IPPROTO_GRE */
# define BO_OTI_GRE_PROTOCOL "gre"

/* Mobile Host Routing */
# define BoK_OTI_MHRP_PROTOCOL (48) /* IPPROTO_MHRP */
# define BO_OTI_MHRP_PROTOCOL "mhrp"

/* BHA */
# define BoK_OTI_BHA_PROTOCOL (49) /* IPPROTO_BHA */
# define BO_OTI_BHA_PROTOCOL "bha"

/* IP6 Encap Sec. Payload */
# define BoK_OTI_ESP_PROTOCOL (50) /* IPPROTO_ESP */
# define BO_OTI_ESP_PROTOCOL "esp"

/* IP6 Auth Header */
# define BoK_OTI_AH_PROTOCOL (51) /* IPPROTO_AH */
# define BO_OTI_AH_PROTOCOL "ah"

/* Integ. Net Layer Security */
# define BoK_OTI_INLSP_PROTOCOL (52) /* IPPROTO_INLSP */
# define BO_OTI_INLSP_PROTOCOL "inlsp"

/* IP with encryption */
# define BoK_OTI_SWIPE_PROTOCOL (53) /* IPPROTO_SWIPE */
# define BO_OTI_SWIPE_PROTOCOL "swipe"

/* Next Hop Resolution */
# define BoK_OTI_NHRP_PROTOCOL (54) /* IPPROTO_NHRP */
# define BO_OTI_NHRP_PROTOCOL "nhrp"

/* IP Mobility */
# define BoK_OTI_MOBILE_PROTOCOL (55) /* IPPROTO_MOBILE */
# define BO_OTI_MOBILE_PROTOCOL "mobile"

/* Transport Layer Security */
# define BoK_OTI_TLSP_PROTOCOL (56) /* IPPROTO_TLSP */
# define BO_OTI_TLSP_PROTOCOL "tlsp"

/* SKIP */
# define BoK_OTI_SKIP_PROTOCOL (57) /* IPPROTO_SKIP */
# define BO_OTI_SKIP_PROTOCOL "skip"

/* ICMPv6 */
# define BoK_OTI_ICMPV6_PROTOCOL (58) /* IPPROTO_ICMPV6 */
# define BO_OTI_ICMPV6_PROTOCOL "icmpv6"

/* IPv6 no next header */
# define BoK_OTI_NONE_PROTOCOL (59) /* IPPROTO_NONE */
# define BO_OTI_NONE_PROTOCOL "none"

/* IPv6 Destination options */
# define BoK_OTI_DSTOPTS_PROTOCOL (60) /* IPPROTO_DSTOPTS */
# define BO_OTI_DSTOPTS_PROTOCOL "dstopts"

/* any host internal protocol */
# define BoK_OTI_AHIP_PROTOCOL (61) /* IPPROTO_AHIP */
# define BO_OTI_AHIP_PROTOCOL "ahip"

/* CFTP */
# define BoK_OTI_CFTP_PROTOCOL (62) /* IPPROTO_CFTP */
# define BO_OTI_CFTP_PROTOCOL "cftp"

/* "hello" routing protocol */
# define BoK_OTI_HELLO_PROTOCOL (63) /* IPPROTO_HELLO */
# define BO_OTI_HELLO_PROTOCOL "hello"

/* SATNET/Backroom EXPAK */
# define BoK_OTI_SATEXPAK_PROTOCOL (64) /* IPPROTO_SATEXPAK */
# define BO_OTI_SATEXPAK_PROTOCOL "satexpak"

/* Kryptolan */
# define BoK_OTI_KRYPTOLAN_PROTOCOL (65) /* IPPROTO_KRYPTOLAN */
# define BO_OTI_KRYPTOLAN_PROTOCOL "kryptolan"

/* Remote Virtual Disk */
# define BoK_OTI_RVD_PROTOCOL (66) /* IPPROTO_RVD */
# define BO_OTI_RVD_PROTOCOL "rvd"

/* Pluribus Packet Core */
# define BoK_OTI_IPPC_PROTOCOL (67) /* IPPROTO_IPPC */
# define BO_OTI_IPPC_PROTOCOL "ippc"

/* Any distributed FS */
# define BoK_OTI_ADFS_PROTOCOL (68) /* IPPROTO_ADFS */
# define BO_OTI_ADFS_PROTOCOL "adfs"

/* Satnet Monitoring */
# define BoK_OTI_SATMON_PROTOCOL (69) /* IPPROTO_SATMON */
# define BO_OTI_SATMON_PROTOCOL "satmon"

/* VISA Protocol */
# define BoK_OTI_VISA_PROTOCOL (70) /* IPPROTO_VISA */
# define BO_OTI_VISA_PROTOCOL "visa"

/* Packet Core Utility */
# define BoK_OTI_IPCV_PROTOCOL (71) /* IPPROTO_IPCV */
# define BO_OTI_IPCV_PROTOCOL "ipcv"

/* Comp. Prot. Net. Executive */
# define BoK_OTI_CPNX_PROTOCOL (72) /* IPPROTO_CPNX */
# define BO_OTI_CPNX_PROTOCOL "cpnx"

/* Comp. Prot. HeartBeat */
# define BoK_OTI_CPHB_PROTOCOL (73) /* IPPROTO_CPHB */
# define BO_OTI_CPHB_PROTOCOL "cphb"

/* Wang Span Network */
# define BoK_OTI_WSN_PROTOCOL (74) /* IPPROTO_WSN */
# define BO_OTI_WSN_PROTOCOL "wsn"

/* Packet Video Protocol */
# define BoK_OTI_PVP_PROTOCOL (75) /* IPPROTO_PVP */
# define BO_OTI_PVP_PROTOCOL "pvp"

/* BackRoom SATNET Monitoring */
# define BoK_OTI_BRSATMON_PROTOCOL (76) /* IPPROTO_BRSATMON */
# define BO_OTI_BRSATMON_PROTOCOL "brsatmon"

/* UNOFFICIAL Sun net disk proto */
# define BoK_OTI_ND_PROTOCOL (77) /* IPPROTO_ND */
# define BO_OTI_ND_PROTOCOL "nd"

#if 0
// @todo: what about protocol "iclfxbm"???
//# define BoK_OTI_ICLFXBM_PROTOCOL (78) /* IPPROTO_ICLFXBM */
//# define BO_OTI_ICLFXBM_PROTOCOL "iclfxbm"
#endif

/* WIDEBAND Monitoring */
# define BoK_OTI_WBMON_PROTOCOL (78) /* IPPROTO_WBMON */
# define BO_OTI_WBMON_PROTOCOL "wbmon"

/* WIDEBAND EXPAK */
# define BoK_OTI_WBEXPAK_PROTOCOL (79) /* IPPROTO_WBEXPAK */
# define BO_OTI_WBEXPAK_PROTOCOL "wbexpak"

/* ISO clnp */
# define BoK_OTI_EON_PROTOCOL (80) /* IPPROTO_EON */
# define BO_OTI_EON_PROTOCOL "eon"

/* VMTP */
# define BoK_OTI_VMTP_PROTOCOL (81) /* IPPROTO_VMTP */
# define BO_OTI_VMTP_PROTOCOL "vmtp"

/* Secure VMTP */
# define BoK_OTI_SVMTP_PROTOCOL (82) /* IPPROTO_SVMTP */
# define BO_OTI_SVMTP_PROTOCOL "svmtp"

/* Banyon VINES */
# define BoK_OTI_VINES_PROTOCOL (83) /* IPPROTO_VINES */
# define BO_OTI_VINES_PROTOCOL "vines"

/* TTP */
# define BoK_OTI_TTP_PROTOCOL (84) /* IPPROTO_TTP */
# define BO_OTI_TTP_PROTOCOL "ttp"

/* NSFNET-IGP */
# define BoK_OTI_IGP_PROTOCOL (85) /* IPPROTO_IGP */
# define BO_OTI_IGP_PROTOCOL "igp"

/* dissimilar gateway prot. */
# define BoK_OTI_DGP_PROTOCOL (86) /* IPPROTO_DGP */
# define BO_OTI_DGP_PROTOCOL "dgp"

/* TCF */
# define BoK_OTI_TCF_PROTOCOL (87) /* IPPROTO_TCF */
# define BO_OTI_TCF_PROTOCOL "tcf"

/* Cisco/GXS IGRP */
# define BoK_OTI_IGRP_PROTOCOL (88) /* IPPROTO_IGRP */
# define BO_OTI_IGRP_PROTOCOL "igrp"

/* OSPF */
# define BoK_OTI_OSPF_PROTOCOL (89) /* IPPROTO_OSPF */
# define BO_OTI_OSPF_PROTOCOL "ospf"

/* OSPFIGP */
# define BoK_OTI_OSPFIGP_PROTOCOL (89) /* IPPROTO_OSPFIGP */
# define BO_OTI_OSPFIGP_PROTOCOL "ospfigp"

/* Strite RPC protocol */
# define BoK_OTI_SRPC_PROTOCOL (90) /* IPPROTO_SRPC */
# define BO_OTI_SRPC_PROTOCOL "srpc"

/* Locus Address Resoloution */
# define BoK_OTI_LARP_PROTOCOL (91) /* IPPROTO_LARP */
# define BO_OTI_LARP_PROTOCOL "larp"

/* Multicast Transport */
# define BoK_OTI_MTP_PROTOCOL (92) /* IPPROTO_MTP */
# define BO_OTI_MTP_PROTOCOL "mtp"

/* AX.25 Frames */
# define BoK_OTI_AX25_PROTOCOL (93) /* IPPROTO_AX25 */
# define BO_OTI_AX25_PROTOCOL "ax25"

/* IP encapsulated in IP */
# define BoK_OTI_IPEIP_PROTOCOL (94) /* IPPROTO_IPEIP */
# define BO_OTI_IPEIP_PROTOCOL "ipeip"

/* Mobile Int.ing control */
# define BoK_OTI_MICP_PROTOCOL (95) /* IPPROTO_MICP */
# define BO_OTI_MICP_PROTOCOL "micp"

/* Semaphore Comm. security */
# define BoK_OTI_SCCSP_PROTOCOL (96) /* IPPROTO_SCCSP */
# define BO_OTI_SCCSP_PROTOCOL "sccsp"

/* Ethernet IP encapsulation */
# define BoK_OTI_ETHERIP_PROTOCOL (97) /* IPPROTO_ETHERIP */
# define BO_OTI_ETHERIP_PROTOCOL "etherip"

/* IP in IP encapsulation header */
# define BoK_OTI_ENCAP_PROTOCOL (98) /* IPPROTO_ENCAP */
# define BO_OTI_ENCAP_PROTOCOL "encap"

/* any private encr. scheme */
# define BoK_OTI_APES_PROTOCOL (99) /* IPPROTO_APES */
# define BO_OTI_APES_PROTOCOL "apes"

/* GMTP */
# define BoK_OTI_GMTP_PROTOCOL (100) /* IPPROTO_GMTP */
# define BO_OTI_GMTP_PROTOCOL "gmtp"

/* Protocol Independent Mcast (PIM) routing protocol */
# define BoK_OTI_PIM_PROTOCOL (103) /* IPPROTO_PIM */
# define BO_OTI_PIM_PROTOCOL "pim"

/* payload compression (IPComp) */
# define BoK_OTI_IPCOMP_PROTOCOL (108) /* IPPROTO_IPCOMP */
# define BO_OTI_IPCOMP_PROTOCOL "ipcomp"

/* CARP */
# define BoK_OTI_CARP_PROTOCOL (112) /* IPPROTO_CARP */
# define BO_OTI_CARP_PROTOCOL "carp"

/* PGM */
# define BoK_OTI_PGM_PROTOCOL (113) /* IPPROTO_PGM */
# define BO_OTI_PGM_PROTOCOL "pgm"

# define BoK_OTI_L2TP_PROTOCOL (115) /* IPPROTO_L2TP */
# define BO_OTI_L2TP_PROTOCOL "l2tp"

/* Stream Control Transmission Protocol */
# define BoK_OTI_SCTP_PROTOCOL (132) /* IPPROTO_SCTP */
# define BO_OTI_SCTP_PROTOCOL "sctp"

/* PFSYNC */
# define BoK_OTI_PFSYNC_PROTOCOL (240) /* IPPROTO_PFSYNC */
# define BO_OTI_PFSYNC_PROTOCOL "pfsync"

/* OLD divert pseudo-protocol */
# define BoK_OTI_OLD_DIVERT_PROTOCOL (254) /* IPPROTO_OLD_DIVERT */
# define BO_OTI_OLD_DIVERT_PROTOCOL "old_divert"

/* raw IP packet */
# define BoK_OTI_RAW_PROTOCOL (255) /* IPPROTO_RAW */
# define BO_OTI_RAW_PROTOCOL "rawip"

# define BoK_OTI_MAX_PROTOCOL (256) /* IPPROTO_MAX */
# define BO_OTI_MAX_PROTOCOL "max"

/* last return value of *_input(), meaning "all job for this pkt is done".  */
# define BoK_OTI_DONE_PROTOCOL (257) /* IPPROTO_DONE */
# define BO_OTI_DONE_PROTOCOL "done"

/* spacer for loadable protos */
# define BoK_OTI_SPACER_PROTOCOL (32767) /* IPPROTO_SPACER */
# define BO_OTI_SPACER_PROTOCOL "spacer"

/******************************************************************************
 * %% BeginSection: protocol family constant declarations
 */
# pragma mark -
# pragma mark [ protocol family constant declarations ]
# pragma mark -

# define BoK_OTI_INVAL_PROTOFMLY (-1)
# define BoK_OTI_UNK_PROTOFMLY (-2)

# define BO_OTI_NULL_PROTOFMLY "-"
# define BO_OTI_NOPROTOFMLY BO_OTI_NULL_PROTOFMLY

/*
 * loopback
 */
# define BoK_OTI_LOOPBACK_PROTOFMLY (256)
# define BO_OTI_LOOPBACK_PROTOFMLY "loopback"

/* unspecified */
# define BoK_OTI_UNSPEC_PROTOFMLY BoK_OTI_UNSPEC_ADDRFMLY
# define BO_OTI_UNSPEC_PROTOFMLY "unspec"

/* standardized name for AF_LOCAL */
# define BoK_OTI_UNIX_PROTOFMLY BoK_OTI_UNIX_ADDRFMLY
# define BO_OTI_UNIX_PROTOFMLY "unix"

/* local to host_ADDRFMLY (pipes, portals) */
# define BoK_OTI_LOCAL_PROTOFMLY BoK_OTI_LOCAL_ADDRFMLY
# define BO_OTI_LOCAL_PROTOFMLY "local"

# define BoK_OTI_FILE_PROTOFMLY BoK_OTI_FILE_ADDRFMLY
# define BO_OTI_FILE_PROTOFMLY "file"

/* internetwork: UDP, TCP, etc. */
# define BoK_OTI_INET_PROTOFMLY BoK_OTI_INET_ADDRFMLY
# define BO_OTI_INET_PROTOFMLY "inet"

/* arpanet imp addresses */
# define BoK_OTI_IMPLINK_PROTOFMLY BoK_OTI_IMPLINK_ADDRFMLY
# define BO_OTI_IMPLINK_PROTOFMLY "implink"

/* pup protocols: e.g. BSP */
# define BoK_OTI_PUP_PROTOFMLY BoK_OTI_PUP_ADDRFMLY
# define BO_OTI_PUP_PROTOFMLY "pup"

/* mit CHAOS protocols */
# define BoK_OTI_CHAOS_PROTOFMLY BoK_OTI_CHAOS_ADDRFMLY
# define BO_OTI_CHAOS_PROTOFMLY "chaos"

/* SMB protocols */
#  define BoK_OTI_NETBIOS_PROTOFMLY BoK_OTI_NETBIOS_ADDRFMLY
# define BO_OTI_NETBIOS_PROTOFMLY "netbios"

/* ISO protocols */
# define BoK_OTI_ISO_PROTOFMLY BoK_OTI_ISO_ADDRFMLY
# define BO_OTI_ISO_PROTOFMLY "iso"

# define BoK_OTI_OSI_PROTOFMLY BoK_OTI_OSI_ADDRFMLY
# define BO_OTI_OSI_PROTOFMLY "osi"

/* European computer manufacturers */
# define BoK_OTI_ECMA_PROTOFMLY BoK_OTI_ECMA_ADDRFMLY
# define BO_OTI_ECMA_PROTOFMLY "ecma"

/* datakit protocols */
# define BoK_OTI_DATAKIT_PROTOFMLY BoK_OTI_DATAKIT_ADDRFMLY
# define BO_OTI_DATAKIT_PROTOFMLY "datakit"

/* CCITT protocols, X.25 etc */
# define BoK_OTI_CCITT_PROTOFMLY BoK_OTI_CCITT_ADDRFMLY
# define BO_OTI_CCITT_PROTOFMLY "ccitt"

/* IBM SNA */
# define BoK_OTI_SNA_APAMILY BoK_OTI_SNA_ADDRFMLY
# define BO_OTI_SNA_PROTOFMLY "sna"

/* DECnet */
# define BoK_OTI_DECnet_PROTOFMLY BoK_OTI_DECnet_ADDRFMLY
# define BO_OTI_DECnet_PROTOFMLY "decnet"

/* DEC Direct data link interface */
# define BoK_OTI_DLI_PROTOFMLY BoK_OTI_DLI_ADDRFMLY
# define BO_OTI_DLI_PROTOFMLY "dli"

/* LAT */
# define BoK_OTI_LAT_PROTOFMLY BoK_OTI_LAT_ADDRFMLY
# define BO_OTI_LAT_PROTOFMLY "lat"

/* NSC Hyperchannel */
# define BoK_OTI_HYLINK_PROTOFMLY BoK_OTI_HYLINK_ADDRFMLY
# define BO_OTI_HYLINK_PROTOFMLY "hylink"

/* Apple Talk */
# define BoK_OTI_APPLETALK_PROTOFMLY BoK_OTI_APPLETALK_ADDRFMLY
# define BO_OTI_APPLETALK_PROTOFMLY "appletalk"

/* Internal Routing Protocol */
# define BoK_OTI_ROUTE_PROTOFMLY BoK_OTI_ROUTE_ADDRFMLY
# define BO_OTI_ROUTE_PROTOFMLY "route"

/* Link layer interface */
# define BoK_OTI_LINK_PROTOFMLY BoK_OTI_LINK_ADDRFMLY
# define BO_OTI_LINK_PROTOFMLY "link"

/* eXpress Transfer Protocol (no AF) */
# define BoK_OTI_XTP_PROTOFMLY BoK_OTI_XTP_pseudo_ADDRFMLY
# define BO_OTI_XTP_PROTOFMLY "xtp"

/* connection-oriented IP, aka ST II */
# define BoK_OTI_COIP_PROTOFMLY BoK_OTI_COIP_ADDRFMLY
# define BO_OTI_COIP_PROTOFMLY "coip"

/* Computer Network Technology */
# define BoK_OTI_CNT_PROTOFMLY BoK_OTI_CNT_ADDRFMLY
# define BO_OTI_CNT_PROTOFMLY "cnt"

/* Help Identify RTIP packets */
# define BoK_OTI_RTIP_PROTOFMLY BoK_OTI_RTIP_pseudo_ADDRFMLY
# define BO_OTI_RTIP_PROTOFMLY "rtip"

/* Novell Internet Protocol */
# define BoK_OTI_IPX_PROTOFMLY BoK_OTI_IPX_ADDRFMLY
# define BO_OTI_IPX_PROTOFMLY "ipx"

/* Simple Internet Protocol */
# define BoK_OTI_SIP_PROTOFMLY BoK_OTI_SIP_ADDRFMLY
# define BO_OTI_SIP_PROTOFMLY "sip"

/* Help Identify PIP packets */
# define BoK_OTI_PIP_PROTOFMLY BoK_OTI_PIP_pseudo_ADDRFMLY
# define BO_OTI_PIP_PROTOFMLY "pip"

/* IPv6 */
# define BoK_OTI_INET6_PROTOFMLY BoK_OTI_INET6_ADDRFMLY
# define BO_OTI_INET6_PROTOFMLY "inet6"

/* Internal key-management function */
# define BoK_OTI_KEY_PROTOFMLY BoK_OTI_KEY_pseudo_ADDRFMLY
# define BO_OTI_KEY_PROTOFMLY "key"

/* Integrated Services Digital Network */
# define BoK_OTI_ISDN_PROTOFMLY BoK_OTI_ISDN_ADDRFMLY
# define BO_OTI_ISDN_PROTOFMLY "isdn"

/* Network Driver 'raw' access */
# define BoK_OTI_NDRV_PROTOFMLY BoK_OTI_NDRV_ADDRFMLY
# define BO_OTI_NDRV_PROTOFMLY "ndrv"

/* ATM */
# define BoK_OTI_ATM_PROTOFMLY BoK_OTI_ATM_ADDRFMLY
# define BO_OTI_ATM_PROTOFMLY "atm"

/* native ATM access */
# define BoK_OTI_NATM_PROTOFMLY BoK_OTI_NATM_ADDRFMLY
# define BO_OTI_NATM_PROTOFMLY "natm"

/* Netgraph sockets */
# define BoK_OTI_NETGRAPH_PROTOFMLY BoK_OTI_NETGRAPH_ADDRFMLY
# define BO_OTI_NETGRAPH_PROTOFMLY "netgraph"

/* 802.3ad slow protocol */
# define BoK_OTI_SLOW_PROTOFMLY BoK_OTI_SLOW_ADDRFMLY
# define BO_OTI_SLOW_PROTOFMLY "slow"

/* Sitara cluster protocol */
# define BoK_OTI_SCLUSTER_PROTOFMLY BoK_OTI_SCLUSTER_ADDRFMLY
# define BO_OTI_SCLUSTER_PROTOFMLY "scluster"

/* Used by BPF to not rewrite headers
 * in interface output routine
 */
# define BoK_OTI_HDRCMPLT_PROTOFMLY BoK_OTI_HDRCMPLT_pseudo_ADDRFMLY
# define BO_OTI_HDRCMPLT_PROTOFMLY "hdrcmplt"

# define BoK_OTI_ARP_PROTOFMLY BoK_OTI_ARP_ADDRFMLY
# define BO_OTI_ARP_PROTOFMLY "arp"

/* Bluetooth sockets */
# define BoK_OTI_BLUETOOTH_PROTOFMLY BoK_OTI_BLUETOOTH_ADDRFMLY
# define BO_OTI_BLUETOOTH_PROTOFMLY "bluetooth"

/* VoiceView */
# define BoK_OTI_VOICEVIEW_PROTOFMLY BoK_OTI_VOICEVIEW_ADDRFMLY
# define BO_OTI_VOICEVIEW_PROTOFMLY "voiceview"

/* Protocols from Firefox */
# define BoK_OTI_FIREFOX_PROTOFMLY BoK_OTI_FIREFOX_ADDRFMLY
# define BO_OTI_FIREFOX_PROTOFMLY "firefox"

/* Banyan */
# define BoK_OTI_BAN_PROTOFMLY BoK_OTI_BAN_ADDRFMLY
# define BO_OTI_BAN_PROTOFMLY "ban"

/* Microsoft Wolfpack */
# define BoK_OTI_CLUSTER_PROTOFMLY BoK_OTI_CLUSTER_ADDRFMLY
# define BO_OTI_CLUSTER_PROTOFMLY "cluster"

/* IEEE 1284.4 WG AF */
# define BoK_OTI_12844_PROTOFMLY BoK_OTI_12844_ADDRFMLY
# define BO_OTI_12844_PROTOFMLY "ieee12844"

/* IrDA */
# define BoK_OTI_IRDA_PROTOFMLY BoK_OTI_IRDA_ADDRFMLY
# define BO_OTI_IRDA_PROTOFMLY "irda"

/* Network Designers OSI & gateway */
# define BoK_OTI_NETDES_PROTOFMLY BoK_OTI_NETDES_ADDRFMLY
# define BO_OTI_NETDES_PROTOFMLY "netdes"

# define BoK_OTI_TCNPROCESS_PROTOFMLY BoK_OTI_TCNPROCESS_ADDRFMLY
# define BO_OTI_TCNPROCESS_PROTOFMLY "tcnprocess"

# define BoK_OTI_TCNMESSAGE_PROTOFMLY BoK_OTI_TCNMESSAGE_ADDRFMLY
# define BO_OTI_TCNMESSAGE_PROTOFMLY "tcnmessage"

# define BoK_OTI_ICLFXBM_PROTOFMLY BoK_OTI_ICLFXBM_ADDRFMLY
# define BO_OTI_ICLFXBM_PROTOFMLY "iclfxbm"

/* Bluetooth RFCOMM/L2CAP protocols */
# define BoK_OTI_BTH_PROTOFMLY BoK_OTI_BTH_ADDRFMLY
# define BO_OTI_BTH_PROTOFMLY "bth"


/* XEROX NS protocols */
# define BoK_OTI_NS_PROTOFMLY BoK_OTI_NS_ADDRFMLY
# define BO_OTI_NS_PROTOFMLY "ns"

# define BoK_OTI_NBS_PROTOFMLY BoK_OTI_NBS_ADDRFMLY
# define BO_OTI_NBS_PROTOFMLY "nbs"

/* Network Interface Tap */
#  define BoK_OTI_NIT_PROTOFMLY BoK_OTI_NIT_ADDRFMLY
#  define BO_OTI_NIT_PROTOFMLY "nit"

/* IEEE 802.2, also ISO 8802 */
#  define BoK_OTI_802_PROTOFMLY BoK_OTI_802_ADDRFMLY
#  define BO_OTI_802_PROTOFMLY "ieee802"

#  define BoK_OTI_ISO8802_PROTOFMLY BoK_OTI_802_PROTOFMLY
#  define BO_OTI_ISO8802_PROTOFMLY "iso8802"

/* CCITT X.25 in particular */
#  define BoK_OTI_X25_PROTOFMLY BoK_OTI_X25_ADDRFMLY
#  define BO_OTI_X25_PROTOFMLY "x25"

/* AFI = 47, IDI = 4 */
#  define BoK_OTI_OSINET_PROTOFMLY BoK_OTI_OSINET_ADDRFMLY
#  define BO_OTI_OSINET_PROTOFMLY "osinet"

/* U.S. Government OSI */
#  define BoK_OTI_GOSIP_PROTOFMLY BoK_OTI_GOSIP_ADDRFMLY
#  define BO_OTI_GOSIP_PROTOFMLY "gosip"

/* NCA socket */
#  define BoK_OTI_NCA_PROTOFMLY BoK_OTI_NCA_ADDRFMLY
#  define BO_OTI_NCA_PROTOFMLY "nca"

/* Security Policy DB socket */
#  define BoK_OTI_POLICY_PROTOFMLY BoK_OTI_POLICY_ADDRFMLY
#  define BO_OTI_POLICY_PROTOFMLY "policy"

/* CCITT E.164 recommendation */
# define BoK_OTI_E164_PROTOFMLY BoK_OTI_E164_ADDRFMLY
# define BO_OTI_E164_PROTOFMLY "e164"


# define BoK_OTI_MAX_PROTOFMLY BoK_OTI_MAX_ADDRFMLY
# define BO_OTI_MAX_PROTOFMLY "max"

/******************************************************************************
 * %% BeginSection: address family constant declarations
 */
# pragma mark -
# pragma mark [ address family constant declarations ]
# pragma mark -

# define BoK_OTI_INVAL_ADDRFMLY (-1)

/*
 * loopback pseudo address family
 */
# define BoK_OTI_LOOPBACK_ADDRFMLY (256)
# define BO_OTI_LOOPBACK_ADDRFMLY "loopback"

/* unspecified */
# define BoK_OTI_UNSPEC_ADDRFMLY (0) /* AF_UNSPEC */
# define BO_OTI_UNSPEC_ADDRFMLY "unspec"

/* standardized name for AF_LOCAL */
# define BoK_OTI_UNIX_ADDRFMLY (1) /* AF_UNIX */
# define BO_OTI_UNIX_ADDRFMLY "unix"

/* local to host_ADDRFMLY (pipes, portals) */
# define BoK_OTI_LOCAL_ADDRFMLY BoK_OTI_UNIX_ADDRFMLY
# define BO_OTI_LOCAL_ADDRFMLY "local"

# define BoK_OTI_FILE_ADDRFMLY BoK_OTI_UNIX_ADDRFMLY
# define BO_OTI_FILE_ADDRFMLY "file"

/* internetwork: UDP, TCP, etc. */
# define BoK_OTI_INET_ADDRFMLY (2) /* AF_INET */
# define BO_OTI_INET_ADDRFMLY "inet"

/* arpanet imp addresses */
# define BoK_OTI_IMPLINK_ADDRFMLY (3) /* AF_IMPLINK */
# define BO_OTI_IMPLINK_ADDRFMLY "implink"

/* pup protocols: e.g. BSP */
# define BoK_OTI_PUP_ADDRFMLY (4) /* AF_PUP */
# define BO_OTI_PUP_ADDRFMLY "pup"

/* mit CHAOS protocols */
# define BoK_OTI_CHAOS_ADDRFMLY (5) /* AF_CHAOS */
# define BO_OTI_CHAOS_ADDRFMLY "chaos"

/* SMB protocols */
# define BoK_OTI_NETBIOS_ADDRFMLY (6) /* AF_NETBIOS */
# define BO_OTI_NETBIOS_ADDRFMLY "netbios"

/* ISO protocols */
# define BoK_OTI_ISO_ADDRFMLY (7) /* AF_ISO */
# define BO_OTI_ISO_ADDRFMLY "iso"

# define BoK_OTI_OSI_ADDRFMLY BoK_OTI_ISO_ADDRFMLY /* AF_OSI */
# define BO_OTI_OSI_ADDRFMLY "osi"

/* European computer manufacturers */
# define BoK_OTI_ECMA_ADDRFMLY (8) /* AF_ECMA */
# define BO_OTI_ECMA_ADDRFMLY "ecma"

/* datakit protocols */
# define BoK_OTI_DATAKIT_ADDRFMLY (9) /* AF_DATAKIT */
# define BO_OTI_DATAKIT_ADDRFMLY "datakit"

/* CCITT protocols, X.25 etc */
# define BoK_OTI_CCITT_ADDRFMLY (10) /* AF_CCITT */
# define BO_OTI_CCITT_ADDRFMLY "ccitt"

/* IBM SNA */
# define BoK_OTI_SNA_ADDRFMLY (11) /* AF_SNA */
# define BO_OTI_SNA_ADDRFMLY "sna"

/* DECnet */
# define BoK_OTI_DECnet_ADDRFMLY (12) /* AF_DECnet */
# define BO_OTI_DECnet_ADDRFMLY "decnet"

/* DEC Direct data link interface */
# define BoK_OTI_DLI_ADDRFMLY (13) /* AF_DLI */
# define BO_OTI_DLI_ADDRFMLY "dli"

/* LAT */
# define BoK_OTI_LAT_ADDRFMLY (14) /* AF_LAT */
# define BO_OTI_LAT_ADDRFMLY "lat"

/* NSC Hyperchannel */
# define BoK_OTI_HYLINK_ADDRFMLY (15) /* AF_HYLINK */
# define BO_OTI_HYLINK_ADDRFMLY "hylink"

/* Apple Talk */
# define BoK_OTI_APPLETALK_ADDRFMLY (16) /* AF_APPLETALK */
# define BO_OTI_APPLETALK_ADDRFMLY "appletalk"

/* Internal Routing Protocol */
# define BoK_OTI_ROUTE_ADDRFMLY (17) /* AF_ROUTE */
# define BO_OTI_ROUTE_ADDRFMLY "route"

/* Link layer interface */
# define BoK_OTI_LINK_ADDRFMLY (18) /* AF_LINK */
# define BO_OTI_LINK_ADDRFMLY "link"

/* eXpress Transfer Protocol (no AF) */
# define BoK_OTI_XTP_pseudo_ADDRFMLY (19) /* pseudo_AF_XTP */
# define BO_OTI_XTP_pseudo_ADDRFMLY "pseudo_xtp"

# define BoK_OTI_XTP_ADDRFMLY BoK_OTI_XTP_pseudo_ADDRFMLY /* AF_XTP */
# define BO_OTI_XTP_ADDRFMLY "xtp"

/* connection-oriented IP, aka ST II */
# define BoK_OTI_COIP_ADDRFMLY (20) /* AF_COIP */
# define BO_OTI_COIP_ADDRFMLY "coip"

/* Computer Network Technology */
# define BoK_OTI_CNT_ADDRFMLY (21) /* AF_CNT */
# define BO_OTI_CNT_ADDRFMLY "cnt"

/* Help Identify RTIP packets */
# define BoK_OTI_RTIP_pseudo_ADDRFMLY (22) /* pseudo_AF_RTIP */
# define BO_OTI_RTIP_pseudo_ADDRFMLY "pseudo_rtip"

# define BoK_OTI_RTIP_ADDRFMLY BoK_OTI_RTIP_pseudo_ADDRFMLY /* AF_RTIP */
# define BO_OTI_RTIP_ADDRFMLY "rtip"

/* Novell Internet Protocol */
# define BoK_OTI_IPX_ADDRFMLY (23) /* AF_IPX */
# define BO_OTI_IPX_ADDRFMLY "ipx"

/* Simple Internet Protocol */
# define BoK_OTI_SIP_ADDRFMLY (24) /* AF_SIP */
# define BO_OTI_SIP_ADDRFMLY "sip"

/* Help Identify PIP packets */
# define BoK_OTI_PIP_pseudo_ADDRFMLY (25) /* pseudo_AF_PIP */
# define BO_OTI_PIP_pseudo_ADDRFMLY "pseudo_pip"

# define BoK_OTI_PIP_ADDRFMLY BoK_OTI_PIP_pseudo_ADDRFMLY /* AF_PIP */
# define BO_OTI_PIP_ADDRFMLY "pip"

/* IPv6 */
# define BoK_OTI_INET6_ADDRFMLY (26) /* AF_INET6 */
# define BO_OTI_INET6_ADDRFMLY "inet6"

/* Internal key-management function */
# define BoK_OTI_KEY_pseudo_ADDRFMLY (27) /* pseudo_AF_KEY */
# define BO_OTI_KEY_pseudo_ADDRFMLY "pseudo_key"

/* Security Association DB socket */
# define BoK_OTI_KEY_ADDRFMLY BoK_OTI_KEY_pseudo_ADDRFMLY /* AF_KEY */
# define BO_OTI_KEY_ADDRFMLY "key"

/* Integrated Services Digital Network */
# define BoK_OTI_ISDN_ADDRFMLY (28) /* AF_ISDN */
# define BO_OTI_ISDN_ADDRFMLY "isdn"

/* Network Driver 'raw' access */
# define BoK_OTI_NDRV_ADDRFMLY (29) /* AF_NDRV */
# define BO_OTI_NDRV_ADDRFMLY "ndrv"

/* ATM */
# define BoK_OTI_ATM_ADDRFMLY (30) /* AF_ATM */
# define BO_OTI_ATM_ADDRFMLY "atm"

/* native ATM access */
# define BoK_OTI_NATM_ADDRFMLY (31) /* AF_NATM */
# define BO_OTI_NATM_ADDRFMLY "natm"

/* Netgraph sockets */
# define BoK_OTI_NETGRAPH_ADDRFMLY (32) /* AF_NETGRAPH */
# define BO_OTI_NETGRAPH_ADDRFMLY "netgraph"

/* 802.3ad slow protocol */
# define BoK_OTI_SLOW_ADDRFMLY (33) /* AF_SLOW */
# define BO_OTI_SLOW_ADDRFMLY "slow"

/* Sitara cluster protocol */
# define BoK_OTI_SCLUSTER_ADDRFMLY (34) /* AF_SCLUSTER */
# define BO_OTI_SCLUSTER_ADDRFMLY "scluster"

/* Used by BPF to not rewrite headers
 * in interface output routine
 */
# define BoK_OTI_HDRCMPLT_pseudo_ADDRFMLY (35) /* pseudo_AF_HDRCMPLT */
# define BO_OTI_HDRCMPLT_pseudo_ADDRFMLY "pseudo_hdrcmplt"

# define BoK_OTI_HDRCMPLT_ADDRFMLY BoK_OTI_HDRCMPLT_pseudo_ADDRFMLY /* AF_HDRCMPLT */
# define BO_OTI_HDRCMPLT_ADDRFMLY "hdrcmplt"

# define BoK_OTI_ARP_ADDRFMLY (36) /* AF_ARP */
# define BO_OTI_ARP_ADDRFMLY "arp"

/* Bluetooth sockets */
# define BoK_OTI_BLUETOOTH_ADDRFMLY (37) /* AF_BLUETOOTH */
# define BO_OTI_BLUETOOTH_ADDRFMLY "bluetooth"

/* VoiceView */
# define BoK_OTI_VOICEVIEW_ADDRFMLY (38) /* AF_VOICEVIEW */
# define BO_OTI_VOICEVIEW_ADDRFMLY "voiceview"

/* Protocols from Firefox */
# define BoK_OTI_FIREFOX_ADDRFMLY (39) /* AF_FIREFOX */
# define BO_OTI_FIREFOX_ADDRFMLY "firefox"

/* Banyan */
# define BoK_OTI_BAN_ADDRFMLY (41) /* AF_BAN */
# define BO_OTI_BAN_ADDRFMLY "ban"

/* Microsoft Wolfpack */
# define BoK_OTI_CLUSTER_ADDRFMLY (42) /* AF_CLUSTER */
# define BO_OTI_CLUSTER_ADDRFMLY "cluster"

/* IEEE 1284.4 WG AF */
# define BoK_OTI_12844_ADDRFMLY (43) /* AF_12844 */
# define BO_OTI_12844_ADDRFMLY "ieee12844"

/* IrDA */
# define BoK_OTI_IRDA_ADDRFMLY (44) /* AF_IRDA */
# define BO_OTI_IRDA_ADDRFMLY "irda"

/* Network Designers OSI & gateway */
# define BoK_OTI_NETDES_ADDRFMLY (45) /* AF_NETDES */
# define BO_OTI_NETDES_ADDRFMLY "netdes"

# define BoK_OTI_TCNPROCESS_ADDRFMLY (46) /* AF_TCNPROCESS */
# define BO_OTI_TCNPROCESS_ADDRFMLY "tcnprocess"

# define BoK_OTI_TCNMESSAGE_ADDRFMLY (47) /* AF_TCNMESSAGE */
# define BO_OTI_TCNMESSAGE_ADDRFMLY "tcnmessage"

# define BoK_OTI_ICLFXBM_ADDRFMLY (48) /* AF_ICLFXBM */
# define BO_OTI_ICLFXBM_ADDRFMLY "iclfxbm"

/* Bluetooth RFCOMM/L2CAP protocols */
# define BoK_OTI_BTH_ADDRFMLY (49) /* AF_BTH */
# define BO_OTI_BTH_ADDRFMLY "bth"


/* XEROX NS protocols */
# define BoK_OTI_NS_ADDRFMLY (50) /* AF_NS */
# define BO_OTI_NS_ADDRFMLY "ns"

# define BoK_OTI_NBS_ADDRFMLY (51) /* AF_NBS */
# define BO_OTI_NBS_ADDRFMLY "nbs"

/* Network Interface Tap */
#  define BoK_OTI_NIT_ADDRFMLY (52) /* AF_NIT */
#  define BO_OTI_NIT_ADDRFMLY "nit"

/* IEEE 802.2, also ISO 8802 */
#  define BoK_OTI_802_ADDRFMLY (53) /* AF_802 */
#  define BO_OTI_802_ADDRFMLY "ieee802"

/* CCITT X.25 in particular */
#  define BoK_OTI_X25_ADDRFMLY (54) /* AF_X25 */
#  define BO_OTI_X25_ADDRFMLY "x25"

/* AFI = 47, IDI = 4 */
#  define BoK_OTI_OSINET_ADDRFMLY (55) /* AF_OSINET */
#  define BO_OTI_OSINET_ADDRFMLY "osinet"

/* U.S. Government OSI */
#  define BoK_OTI_GOSIP_ADDRFMLY (56) /* AF_GOSIP */
#  define BO_OTI_GOSIP_ADDRFMLY "gosip"

/* NCA socket */
#  define BoK_OTI_NCA_ADDRFMLY (57) /* AF_NCA */
#  define BO_OTI_NCA_ADDRFMLY "nca"

/* Security Policy DB socket */
#  define BoK_OTI_POLICY_ADDRFMLY (58) /* AF_POLICY */
#  define BO_OTI_POLICY_ADDRFMLY "policy"

/* CCITT E.164 recommendation */
# define BoK_OTI_E164_ADDRFMLY (59) /* AF_E164 */
# define BO_OTI_E164_ADDRFMLY "e164"


# define BoK_OTI_MAX_ADDRFMLY (60) /* AF_MAX */
# define BO_OTI_MAX_ADDRFMLY "max"

#endif /* !__BO_TRPTYPE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
