/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright (c) 2005-2007 V.Sokur
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * uriType.h	(N.Panasiuk)
 * include/Bo/uriType.h
 *
 * $Id$
 */

/* ------------------------------------------------------------------------- */

#ifndef __BO_URITYPE__
# define __BO_URITYPE__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_SYS_TYPES__
#  include <Bo/sys/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: type declarations
 */
# pragma mark -
# pragma mark [ type declarations ]
# pragma mark -

/* Max count levels of scheme. */
# define BoL_URI_SCHEME_LEVELS_MAX 2

# ifndef __BO_URI_SCHEME_TYPE__
#  define __BO_URI_SCHEME_TYPE__ u_int32_t
# endif
# ifndef BO_URI_SCHEME_TYPE_DEFINED
#  define BO_URI_SCHEME_TYPE_DEFINED
typedef __BO_URI_SCHEME_TYPE__ bo_uri_scheme_t;
# endif

typedef u_int32_t bo_uri_scheme_x;

/* ============================================================================
 * MACRO-HELPER declarations
 * @{
 */

# define _BoK_URI_SCHEME_PRIM_MASK     (0x0000FFFF)
# define _BoK_URI_SCHEME_SECON_MASK    (0x00FF0000)
# define _BoK_URI_SCHEME_SECON_MASK_S  (16)

/*
 * make scheme
 */

/* bo_uri_scheme_t BO_MAKE_URI_SCHEME(int anID) */
# define BO_MAKE_URI_SCHEME(anID) ((anID) & _BoK_URI_SCHEME_PRIM_MASK)

/* bo_uri_scheme_t BO_MAKE_URI_SCHEME2(int anID, int anID2) */
# define BO_MAKE_URI_SCHEME2(anID,anID2) \
	( ((anID) & _BoK_URI_SCHEME_PRIM_MASK) \
		| (((anID2) << _BoK_URI_SCHEME_SECON_MASK_S) & _BoK_URI_SCHEME_SECON_MASK) )

/* ------------------------------------------------------------------------- */

/* 0 <= retval <= BoL_URI_SCHEME_LEVELS_MAX */
/* size_t BO_URI_SCHEME_IDCount(bo_uri_scheme_t aScheme) */
# define BO_URI_SCHEME_IDCount(aScheme) \
	( (0 != ((aScheme) & _BoK_URI_SCHEME_PRIM_MASK)) \
		+ (0 != ((aScheme) & _BoK_URI_SCHEME_SECON_MASK)) )

/* 0 <= N < BoL_URI_SCHEME_LEVELS_MAX */
/* void BO_URI_SCHEME_SetID(unsigned N, bo_uri_scheme_t &aDest, int aSrc) */
# define BO_URI_SCHEME_SetID(N,aDest,aSrc) \
	do { \
		assert(0 <= (N) && (N) <= 1); \
		if (0 == (N)) { \
			(aDest) &= ~_BoK_URI_SCHEME_PRIM_MASK; \
			(aDest) |= ((bo_uri_scheme_t)(aSrc) & _BoK_URI_SCHEME_PRIM_MASK); \
			break; \
		} \
		if (1 == (N)) { \
			(aDest) &= ~_BoK_URI_SCHEME_SECON_MASK; \
			(aDest) |= ( \
				((bo_uri_scheme_t)(aSrc) << _BoK_URI_SCHEME_SECON_MASK_S) \
				& _BoK_URI_SCHEME_SECON_MASK \
			); \
			break; \
		} \
	} while (0)

/* void BO_URI_SCHEME_GetID(unsigned N, int &aDest, bo_uri_scheme_t aSrc) */
# define BO_URI_SCHEME_GetID(N,aDest,aSrc) \
	do { \
		assert(0 <= (N) && (N) <= 1); \
		if (0 == (N)) { \
			(aDest) = ((bo_uri_scheme_t)(aSrc) & _BoK_URI_SCHEME_PRIM_MASK); \
			break; \
		} \
		if (1 == (N)) { \
			(aDest) = ( \
				((bo_uri_scheme_t)(aSrc) & _BoK_URI_SCHEME_SECON_MASK) \
				>> _BoK_URI_SCHEME_SECON_MASK_S \
			); \
			break; \
		} \
	} while (0)

/* void BO_URI_SCHEME_SetprimID(bo_uri_scheme_t &aDest, int aSrc) */
# define BO_URI_SCHEME_SetprimID(aDest,aSrc) \
	do { \
		(aDest) &= ~_BoK_URI_SCHEME_PRIM_MASK; \
		(aDest) |= ((bo_uri_scheme_t)(aSrc) & _BoK_URI_SCHEME_PRIM_MASK); \
	} while (0)

/* int BO_URI_SCHEME_GetprimID(bo_uri_scheme_t aSrc) */
# define BO_URI_SCHEME_GetprimID(aSrc) \
	((bo_uri_scheme_t)(aScheme) & _BoK_URI_SCHEME_PRIM_MASK)

/* void BO_URI_SCHEME_SetseconID(bo_uri_scheme_t &aDest, int aSrc) */
# define BO_URI_SCHEME_SetseconID(aDest,aSrc) \
	do { \
		(aDest) &= ~_BoK_URI_SCHEME_SECON_MASK; \
		(aDest) |= ( \
			((bo_uri_scheme_t)(aSrc) << _BoK_URI_SCHEME_SECON_MASK_S) \
			& _BoK_URI_SCHEME_SECON_MASK \
		); \
	} while (0)

/* int BO_URI_SCHEME_GetseconID(bo_uri_scheme_t aSrc) */
# define BO_URI_SCHEME_GetseconID(aSrc) \
	( ((bo_uri_scheme_t)(aScheme) & _BoK_URI_SCHEME_PRIM_MASK) \
		>> _BoK_URI_SCHEME_SECON_MASK_S )

/* @} */

/* ============================================================================
 * URI-SCHEME declarations
 * @{
 */

/* "INVALID" scheme */
# ifndef BoK_URI_INVAL_SCHEME
#  define BoK_URI_INVAL_SCHEME ((bo_uri_scheme_t)_BoK_URI_INVAL_SCHEME_ID)
#  define _BoK_URI_INVAL_SCHEME_ID (-1)
# endif

/* "NULL" scheme */
# ifndef BoK_URI_NULL_SCHEME
#  define BoK_URI_NULL_SCHEME ((bo_uri_scheme_t)_BoK_URI_NULL_SCHEME_ID)
#  define _BoK_URI_NULL_SCHEME_ID (0)
# endif

/* Unknown scheme. Program dependent, default value. */
# ifndef BoK_URI_UNK_SCHEME
#  define BoK_URI_UNK_SCHEME ((bo_uri_scheme_t)_BoK_URI_UNK_SCHEME_ID)
#  define _BoK_URI_UNK_SCHEME_ID (2)
# endif

/* ------------------------------------------------------------------------- */

/* GENERIC */
# ifndef BoK_URI_GEN_SCHEME
#  define BoK_URI_GEN_SCHEME BO_MAKE_URI_SCHEME(_BoK_URI_GEN_SCHEME_ID)
#  define _BoK_URI_GEN_SCHEME_ID (3)

#  define BO_URI_GEN_SCHEME_NAME generic
#  define _BO_URI_GEN_SCHEME_NAME "generic"

#  define BO_URI_GEN_SCHEME_RULE ""
# endif /* BoK_URI_GEN_SCHEME */

/* HTTP */
# ifndef BoK_URI_HTTP_SCHEME
#  define BoK_URI_HTTP_SCHEME BO_MAKE_URI_SCHEME(_BoK_URI_HTTP_SCHEME_ID)
#  define _BoK_URI_HTTP_SCHEME_ID (4)

#  define BO_URI_HTTP_SCHEME_NAME http
#  define _BO_URI_HTTP_SCHEME_NAME "http"

#  define BO_URI_HTTP_SCHEME_RULE "http://<host>[:<port>][/[<path>[?<query>][#<fragment>]]]"
# endif /* BoK_URI_HTTP_SCHEME */

/* FTP */
# ifndef BoK_URI_FTP_SCHEME
#  define BoK_URI_FTP_SCHEME BO_MAKE_URI_SCHEME(_BoK_URI_FTP_SCHEME_ID)
#  define _BoK_URI_FTP_SCHEME_ID (5)

#  define BO_URI_FTP_SCHEME_NAME ftp
#  define _BO_URI_FTP_SCHEME_NAME "ftp"

#  define BO_URI_FTP_SCHEME_RULE "ftp://[<user>[:<password>]@]<host>[:<port>][/[<path>[;type=<typecode>]]]"
# endif /* BoK_URI_FTP_SCHEME */

/* MAILTO */
# ifndef BoK_URI_MAILTO_SCHEME
#  define BoK_URI_MAILTO_SCHEME BO_MAKE_URI_SCHEME(_BoK_URI_MAILTO_SCHEME_ID)
#  define _BoK_URI_MAILTO_SCHEME_ID (6)

#  define BO_URI_MAILTO_SCHEME_NAME mailto
#  define _BO_URI_MAILTO_SCHEME_NAME "mailto"

#  define BO_URI_MAILTO_SCHEME_RULE "mailto:<username>@<address>"
# endif /* BoK_URI_MAILTO_SCHEME */

/* FILE */
# ifndef BoK_URI_FILE_SCHEME
#  define BoK_URI_FILE_SCHEME BO_MAKE_URI_SCHEME(_BoK_URI_FILE_SCHEME_ID)
#  define _BoK_URI_FILE_SCHEME_ID (7)

#  define BO_URI_FILE_SCHEME_NAME file
#  define _BO_URI_FILE_SCHEME_NAME "file"

#  define BO_URI_FILE_SCHEME_RULE "file://[<host>]/<path>"
# endif /* BoK_URI_FILE_SCHEME */

/* SLUZBA */
# ifndef BoK_URI_SLUZBA_SCHEME
#  define BoK_URI_SLUZBA_SCHEME BO_MAKE_URI_SCHEME(_BoK_URI_SLUZBA_SCHEME_ID)
#  define _BoK_URI_SLUZBA_SCHEME_ID (10)

#  define BO_URI_SLUZBA_SCHEME_NAME sluzba
#  define _BO_URI_SLUZBA_SCHEME_NAME "sluzba"

#  define BO_URI_SLUZBA_SCHEME_RULE "sluzba://[<host>]/<service>[:<port>][/<protocol>[/<family>]]"
# endif /* BoK_URI_SLUZBA_SCHEME */

/* ----------------------------------------------------------------------------
 * ADBC-SCHEME declarations
 * @{
 */

/* ADBC (AORP DATABASE CONNECTIVITY) */
# ifndef BoK_URI_ADBC_SCHEME
#  define BoK_URI_ADBC_SCHEME BO_MAKE_URI_SCHEME(_BoK_URI_ADBC_SCHEME_ID)
#  define _BoK_URI_ADBC_SCHEME_ID (30)

#  define BO_URI_ADBC_SCHEME_NAME adbc
#  define _BO_URI_ADBC_SCHEME_NAME "adbc"

#  define BO_URI_ADBC_SCHEME_RULE "adbc:<protocol>:<subprotocol>"
# endif /* BoK_URI_ADBC_SCHEME */

/* ADBC:POSTGRESQL */
# ifndef BoK_URI_ADBC_POSTGRESQL_SCHEME
#  define BoK_URI_ADBC_POSTGRESQL_SCHEME \
	BO_MAKE_URI_SCHEME2(_BoK_URI_ADBC_SCHEME_ID,_BoK_URI_ADBC_POSTGRESQL_SUBSCHEME_ID)

#  define _BoK_URI_ADBC_POSTGRESQL_SUBSCHEME_ID (3)

#  define BO_URI_ADBC_POSTGRESQL_SUBSCHEME_NAME postgresql
#  define _BO_URI_ADBC_POSTGRESQL_SUBSCHEME_NAME "postgresql"

#  define BO_URI_ADBC_POSTGRESQL_SCHEME_RULE "adbc:postgresql://[<username>[:<password>]@][<host>[:<port>]]/<databasename>"
# endif /* BoK_URI_ADBC_POSTGRESQL_SCHEME */

/* ADBC:SQLITE */
# ifndef BoK_URI_ADBC_SQLITE_SCHEME
#  define BoK_URI_ADBC_SQLITE_SCHEME \
	BO_MAKE_URI_SCHEME2(_BoK_URI_ADBC_SCHEME_ID,_BoK_URI_ADBC_SQLITE_SUBSCHEME_ID)

#  define _BoK_URI_ADBC_SQLITE_SUBSCHEME_ID (4)

#  define BO_URI_ADBC_SQLITE_SUBSCHEME_NAME sqlite
#  define _BO_URI_ADBC_SQLITE_SUBSCHEME_NAME "sqlite"

#  define BO_URI_ADBC_SQLITE_SCHEME_RULE "adbc:sqlite://[<username>[:<password>]@][<host>]/<path>"
# endif /* BoK_URI_ADBC_SQLITE_SCHEME */

/* ADBC:ODBC */
# ifndef BoK_URI_ADBC_ODBC_SCHEME
#  define BoK_URI_ADBC_ODBC_SCHEME \
	BO_MAKE_URI_SCHEME2(_BoK_URI_ADBC_SCHEME_ID,_BoK_URI_ADBC_ODBC_SUBSCHEME_ID)

#  define _BoK_URI_ADBC_ODBC_SUBSCHEME_ID (5)

#  define BO_URI_ADBC_ODBC_SUBSCHEME_NAME odbc
#  define _BO_URI_ADBC_ODBC_SUBSCHEME_NAME "odbc"

#  define BO_URI_ADBC_ODBC_SCHEME_RULE "adbc:odbc:<data-source-name>[;param=value]*"
# endif /* BoK_URI_ADBC_ODBC_SCHEME */

/* @} */

#endif /* !__BO_URITYPE__ */

/******************************************************************************
 * Revision History:
 *		$Log$
 *
 *****************************************************************************/
