/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(V.Sokur)
 * Vipm/Analysis/Watershed/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ANALYSIS_WATERSHED_ALGORITHM_H__
# define __VIPM_ANALYSIS_WATERSHED_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_ANALYSIS_WATERSHED_TYPES_H__
#  include <Vipm/Analysis/Watershed/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

__PUB bo_status_t
VipmTGAwatershed(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aWtsctx, /* (struct _Vipm_wtscontext *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

__PUB bo_status_t
VipmTGAwatershedEx(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aWtsctx, /* (struct _Vipm_wtscontext *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_uint                       anElemTypeMsk, /* unused, assumed U8 */
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_watershed(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	struct _Vipm_wtscontext        *aWtsctx
#endif
	);

__PUB bo_status_t
vipma_watershed_xm(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[],
	struct _Vipm_wtscontext        *aWtsctx
#endif
	);

/* ------------------------------------------------------------------------- */

__PUB bo_status_t
vipma_fwatershed(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	struct _Vipm_wtscontext        *aWtsctx
#endif
	);

__PUB bo_status_t
vipma_fwatershed_xm(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[],
	struct _Vipm_wtscontext        *aWtsctx
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * watershed
 *
 * Data type: U8
 */

# define vipma_watershed_u8(x,f,rank,vol,S,offsS,strideS,con) \
    vipma_watershed(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,con)

/* ------------------------------------------------------------------------- */

/*
 * watershed_xm (with "pixel" mask)
 *
 * Data type: U8
 */

# define vipma_watershed_xm_u8(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,con) \
    vipma_watershed_xm(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,Sm,offsSm,strideSm,con)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_ANALYSIS_WATERSHED_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
