/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * MomentFeature.h	(V.Sokur)
 * Vipm/Analysis/Watershed/Features/MomentFeature.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ANALYSIS_WATERSHED_FEATURES_MOMENT_FEATURE_H__
# define __VIPM_ANALYSIS_WATERSHED_FEATURES_MOMENT_FEATURE_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif
# ifndef __VIPM_ANALYSIS_WATERSHED_TYPES_H__
#  include <Vipm/Analysis/Watershed/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

struct _Vipm_wtsmomentfeat {
# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
	bo_int32_t              wsmf_reserved1;
	bo_int32_t              wsmf_area;
# else
	bo_int32_t              wsmf_area;
	bo_int32_t              wsmf_reserved1;
# endif
	union {
		struct {
			double          wsmf_sy, wsmf_sx, wsmf_syy, wsmf_sxy, wsmf_sxx;
		};
		double              wsmf_moments[5];
	};
};
typedef struct _Vipm_wtsmomentfeat      vipm_wtsmomentfeat_t;
# define vipm_wtsmomentfeat_p(x)        vipm_wtsmomentfeat_cp(x)
# define vipm_wtsmomentfeat_cp(x)      ((struct _Vipm_wtsmomentfeat const *)(x))
# define vipm_wtsmomentfeat_mp(x)      ((struct _Vipm_wtsmomentfeat *)(x))


# define vipm_mf_area(f)            (vipm_wtsmomentfeat_mp(f)->wsmf_area)
# define vipm_mf_isgathered(f)      (0 < vipm_wtsmomentfeat_p(f)->wsmf_area)

# define vipm_mf_mx(f)              (vipm_mf_mx)(vipm_wtsmomentfeat_p(f))
# define vipm_mf_my(f)              (vipm_mf_my)(vipm_wtsmomentfeat_p(f))
# define vipm_mf_Mxx(f)             (vipm_mf_Mxx)(vipm_wtsmomentfeat_p(f))
# define vipm_mf_Myy(f)             (vipm_mf_Myy)(vipm_wtsmomentfeat_p(f))
# define vipm_mf_Mxy(f)             (vipm_mf_Mxy)(vipm_wtsmomentfeat_p(f))
# define vipm_mf_M2(r,f)            (vipm_mf_M2)(r,vipm_wtsmomentfeat_p(f))

# define vipm_mf_m10(f)             (vipm_mf_mx)(vipm_wtsmomentfeat_p(f))
# define vipm_mf_m01(f)             (vipm_mf_my)(vipm_wtsmomentfeat_p(f))

# define vipm_mf_theta(f)           (vipm_mf_theta)(vipm_wtsmomentfeat_p(f))


struct _Vipm_wtsmfregion {
	struct _Vipm_wtsregion          base;
	struct _Vipm_wtsmomentfeat      wsr_feat;
};
typedef struct _Vipm_wtsmfregion        vipm_wtsmfregion_t;
# define vipm_wtsmfregion_p(x)          vipm_wtsmfregion_cp(x)
# define vipm_wtsmfregion_cp(x)        ((struct _Vipm_wtsmfregion const *)(x))
# define vipm_wtsmfregion_mp(x)        ((struct _Vipm_wtsmfregion *)(x))

typedef char __STATIC_ASSERT_WTSMFREGION_ALIGN[(sizeof(vipm_wtsmfregion_t) % VipmK_WTSFR_ALIGN) == 0 ? 1 : -1];
typedef char __STATIC_ASSERT_WTSMFR_FEAT_OFFSET[(__myc_offsetof(vipm_wtsmfregion_t, wsr_feat) % VipmK_WTSFR_ALIGN) == 0 ? 1 : -1];


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PROT __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

__PUB struct _Vipm_wtsfeature_ds *
VipmWTSMFinitFeatureDs(
#if __myc_require(prototypes)
	struct _Vipm_wtsfeature_ds     *aFeature,
	void                           *aShared
#endif
	);

/* ------------------------------------------------------------------------- */

__PUB void
VipmWTSMFupdate(
#if __myc_require(prototypes)
	struct _Vipm_wtsmomentfeat     *aFeature,           /* != NULL */
	vipm_Xpoint_t const            *aPixPointv,
	vipm_uint                       aPixPointc
#endif
	);

__PUB void
VipmWTSMFupdate2D(
#if __myc_require(prototypes)
	struct _Vipm_wtsmomentfeat     *aFeature,           /* != NULL */
	vipm_2d_point_t const          *aPixPointv,
	vipm_uint                       aPixPointc
#endif
	);

__PUB void
VipmWTSMFmerge(
#if __myc_require(prototypes)
	struct _Vipm_wtsmomentfeat     *aFeatureLhs,        /* != NULL */
	struct _Vipm_wtsmomentfeat     *aFeatureRhs         /* != NULL */
#endif
	);

__PUB void
VipmWTSMFlazymerge(
#if __myc_require(prototypes)
	struct _Vipm_wtsmomentfeat     *aFeatureLhs,        /* != NULL */
	struct _Vipm_wtsmomentfeat     *aFeatureRhs         /* != NULL */
#endif
	);

__PUB void
VipmWTSMFisgathered(
#if __myc_require(prototypes)
	struct _Vipm_wtsmomentfeat     *aFeature            /* != NULL */
#endif
	);


/* ------------------------------------------------------------------------- */

__PUB double   (vipm_mf_mx)             __P((struct _Vipm_wtsmomentfeat const *));
__PUB double   (vipm_mf_my)             __P((struct _Vipm_wtsmomentfeat const *));
__PUB double   (vipm_mf_Mxx)            __P((struct _Vipm_wtsmomentfeat const *));
__PUB double   (vipm_mf_Myy)            __P((struct _Vipm_wtsmomentfeat const *));
__PUB double   (vipm_mf_Mxy)            __P((struct _Vipm_wtsmomentfeat const *));
__PUB double  *(vipm_mf_M2)             __P((double [/*3*/], struct _Vipm_wtsmomentfeat const *));

__PUB double   (vipm_mf_theta)          __P((struct _Vipm_wtsmomentfeat const *));


/* ========================================================================= */

__PROT void   VipmWTSMF_update2D        __P((struct _Vipm_wtsmomentfeat *, vipm_int, vipm_int));
__PROT void   VipmWTSMF_update2D_n      __P((struct _Vipm_wtsmomentfeat *, vipm_2d_point_t const *, vipm_uint));
__PROT void   VipmWTSMF_update_n        __P((struct _Vipm_wtsmomentfeat *, vipm_Xpoint_t const *, vipm_uint));


/* ------------------------------------------------------------------------- */

__PROT void   __VipmWTSMF__update__     __P((vipm_feature_t *, vipm_Xpoint_t const *, vipm_uint));
__PROT void   __VipmWTSMF__update2D__   __P((vipm_feature_t *, vipm_2d_point_t const *, vipm_uint));
__PROT void   __VipmWTSMF__merge__      __P((vipm_feature_t *, vipm_feature_t *));
__PROT void   __VipmWTSMF__lazymerge__  __P((vipm_feature_t *, vipm_feature_t *));
__PROT int    __VipmWTSMF__isgathered__ __P((vipm_feature_t *));
__PROT void   __VipmWTSMF__ongather__   __P((vipm_feature_t *, vipm_wtsregion_t *));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

# define VipmWTSMFupdate                __VipmWTSMF__update__
# define VipmWTSMFupdate2D              __VipmWTSMF__update2D__
# define VipmWTSMFmerge                 __VipmWTSMF__merge__
# define VipmWTSMFlazymerge             __VipmWTSMF__lazymerge__
# define VipmWTSMFisgathered            __VipmWTSMF__isgathered__

# define __VipmWTSMF__isgathered__(f)   vipm_mf_isgathered(f)


_VIPM_INLINE double (vipm_mf_mx)(vipm_wtsmomentfeat_t const *f)
{
	return (f->wsmf_sx / f->wsmf_area);
}

_VIPM_INLINE double (vipm_mf_my)(vipm_wtsmomentfeat_t const *f)
{
	return (f->wsmf_sy / f->wsmf_area);
}

_VIPM_INLINE double (vipm_mf_Mxx)(vipm_wtsmomentfeat_t const *f)
{
	/* Sxx / n - mx * mx */
	double sx2 = f->wsmf_sx;
	double n_1 = 1.0 / f->wsmf_area;
	sx2 *= sx2;
	return (vipmi_mls_f64(f->wsmf_sxx, sx2, n_1) * n_1);
}

_VIPM_INLINE double (vipm_mf_Myy)(vipm_wtsmomentfeat_t const *f)
{
	/* Syy / n - my * my */
	double sy2 = f->wsmf_sy;
	double n_1 = 1.0 / f->wsmf_area;
	sy2 *= sy2;
	return (vipmi_mls_f64(f->wsmf_syy, sy2, n_1) * n_1);
}

_VIPM_INLINE double (vipm_mf_Mxy)(vipm_wtsmomentfeat_t const *f)
{
	/* Sxy / n - mx * mx */
	double n_1 = 1.0 / f->wsmf_area;
	double sy_sx = f->wsmf_sy * f->wsmf_sx;
	return (vipmi_mls_f64(f->wsmf_sxy, sy_sx, n_1) * n_1);
}


__MYC_END_CDECLS
# undef __PUB
# undef __PROT
# undef __PRIV


#endif /* !defined(__VIPM_ANALYSIS_WATERSHED_FEATURES_MOMENT_FEATURE_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
