/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * PixelFeature.h	(V.Sokur)
 * Vipm/Analysis/Watershed/Features/PixelFeature.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ANALYSIS_WATERSHED_FEATURES_PIXEL_FEATURE_H__
# define __VIPM_ANALYSIS_WATERSHED_FEATURES_PIXEL_FEATURE_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif
# ifndef __VIPM_ANALYSIS_WATERSHED_TYPES_H__
#  include <Vipm/Analysis/Watershed/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

struct _Vipm_wtspixelfeat {
	bo_size_t               wspf_capacity_;
	bo_size_t               wspf_elemc;
	bo_pointer_t            wspf_memory;
};
typedef struct _Vipm_wtspixelfeat       vipm_wtspixelfeat_t;
# define vipm_wtspixelfeat_p(x)         vipm_wtspixelfeat_cp(x)
# define vipm_wtspixelfeat_cp(x)       ((struct _Vipm_wtspixelfeat const *)(x))
# define vipm_wtspixelfeat_mp(x)       ((struct _Vipm_wtspixelfeat *)(x))


/* @{ */
# define __vipm_pf_capacity(f)      (vipm_wtspixelfeat_p(f)->wspf_capacity_ & ~(bo_size_t)BO_MWORD_SIGNBIT)
/* @} */
# define vipm_pf_pixcount(f)        (vipm_wtspixelfeat_p(f)->wspf_elemc)
# define vipm_pf_isgathered(f)      (!!(vipm_wtspixelfeat_p(f)->wspf_capacity_ & BO_MWORD_SIGNBIT))
# define vipm_pf_gathered(f)        (vipm_wtspixelfeat_mp(f)->wspf_capacity_ |= BO_MWORD_SIGNBIT)


/* Forward declarations */
struct _Vipm_wtsmomentfeat;


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PROT __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

__PUB struct _Vipm_wtsfeature_ds *
VipmWTSPFinitFeatureDs(
#if __myc_require(prototypes)
	struct _Vipm_wtsfeature_ds     *aFeature,
	void                           *aShared
#endif
	);

/* ------------------------------------------------------------------------- */

__PUB void
VipmWTSPFinit(
#if __myc_require(prototypes)
	struct _Vipm_wtspixelfeat      *aFeature,           /* != NULL */
	void                           *aShared
#endif
	);

__PUB void
VipmWTSPFdestroy(
#if __myc_require(prototypes)
	struct _Vipm_wtspixelfeat      *aFeature            /* != NULL */
#endif
	);

__PUB void
VipmWTSPFupdate(
#if __myc_require(prototypes)
	struct _Vipm_wtspixelfeat      *aFeature,           /* != NULL */
	vipm_Xpoint_t const            *aPixPointv,
	vipm_uint                       aPixPointc
#endif
	);

__PUB void
VipmWTSPFupdate2D(
#if __myc_require(prototypes)
	struct _Vipm_wtspixelfeat      *aFeature,           /* != NULL */
	vipm_2d_point_t const          *aPixPointv,
	vipm_uint                       aPixPointc
#endif
	);

__PUB void
VipmWTSPFmerge(
#if __myc_require(prototypes)
	struct _Vipm_wtspixelfeat      *aFeatureLhs,        /* != NULL */
	struct _Vipm_wtspixelfeat      *aFeatureRhs         /* != NULL */
#endif
	);

__PUB void
VipmWTSPFlazymerge(
#if __myc_require(prototypes)
	struct _Vipm_wtspixelfeat      *aFeatureLhs,        /* != NULL */
	struct _Vipm_wtspixelfeat      *aFeatureRhs         /* != NULL */
#endif
	);

__PUB void
VipmWTSPFisgathered(
#if __myc_require(prototypes)
	struct _Vipm_wtspixelfeat      *aFeature            /* != NULL */
#endif
	);

/* ------------------------------------------------------------------------- */

/* @todo: fixme - doc */
/*
 * Input pixel feature should belong wts-region[-structure] @{
 */

__PUB struct _Vipm_wtsmomentfeat *
VipmWTSPFmakemf(
#if __myc_require(prototypes)
	struct _Vipm_wtsmomentfeat      *aMomentFeat,
	struct _Vipm_wtspixelfeat const *aPixelFeat,
	vipm_int                         anOffset
#endif
	);

/* @} */


/* ========================================================================= */

__PROT void   VipmWTSPF_update2D        __P((struct _Vipm_wtspixelfeat *, vipm_int, vipm_int));
__PROT void   VipmWTSPF_update2D_n      __P((struct _Vipm_wtspixelfeat *, vipm_2d_point_t const *, vipm_uint));
__PROT void   VipmWTSPF_update_n        __P((struct _Vipm_wtspixelfeat *, vipm_Xpoint_t const *, vipm_uint));


/* ------------------------------------------------------------------------- */

__PROT void   __VipmWTSPF__init__       __P((vipm_feature_t *, void *));
__PROT void   __VipmWTSPF__destroy__    __P((vipm_feature_t *));
__PROT void   __VipmWTSPF__update__     __P((vipm_feature_t *, vipm_Xpoint_t const *, vipm_uint));
__PROT void   __VipmWTSPF__update2D__   __P((vipm_feature_t *, vipm_2d_point_t const *, vipm_uint));
__PROT void   __VipmWTSPF__merge__      __P((vipm_feature_t *, vipm_feature_t *));
__PROT void   __VipmWTSPF__lazymerge__  __P((vipm_feature_t *, vipm_feature_t *));
__PROT int    __VipmWTSPF__isgathered__ __P((vipm_feature_t *));
__PROT void   __VipmWTSPF__ongather__   __P((vipm_feature_t *, vipm_wtsregion_t *));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

# define VipmWTSPFinit                  __VipmWTSMF__init__
# define VipmWTSPFdestroy               __VipmWTSMF__destroy__
# define VipmWTSPFupdate                __VipmWTSMF__update__
# define VipmWTSPFupdate2D              __VipmWTSMF__update2D__
# define VipmWTSPFmerge                 __VipmWTSMF__merge__
# define VipmWTSPFlazymerge             __VipmWTSMF__lazymerge__
# define VipmWTSPFisgathered            __VipmWTSMF__isgathered__

# define __VipmWTSPF__isgathered__(f)   vipm_pf_isgathered(f)


__MYC_END_CDECLS
# undef __PUB
# undef __PROT
# undef __PRIV


#endif /* !defined(__VIPM_ANALYSIS_WATERSHED_FEATURES_PIXEL_FEATURE_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
