/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Fwtsregion.h	(V.Sokur)
 * Vipm/Analysis/Watershed/Fwtsregion.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ANALYSIS_WATERSHED_FWTSREGION_H__
# define __VIPM_ANALYSIS_WATERSHED_FWTSREGION_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_ANALYSIS_WATERSHED_TYPES_H__
#  include <Vipm/Analysis/Watershed/Types.h>
# endif
#ifndef __VIPM_ANALYSIS_WATERSHED_FEATURES_MOMENT_FEATURE_H__
# include "Vipm/Analysis/Watershed/Features/MomentFeature.h"
#endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

struct _Vipm_wtsmoment1 {
	union {
		struct {
			double          wsm1_sy, wsm1_sx;
		};
		double              wsm1_moments[2];
	};
};
typedef struct _Vipm_wtsmoment1         vipm_wtsmoment1_t;

struct _Vipm_wtsmoment2 {
	union {
		struct {
			double          wsm2_syy, wsm2_sxy, wsm2_sxx;
		};
		double              wsm2_moments[3];
	};
};
typedef struct _Vipm_wtsmoment2         vipm_wtsmoment2_t;


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PROT __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * void vipm_fwtsr_(walk)   (aRegion, anOrder, aVisitor, aContext);
 *
 * Visitor:
 *
 *    0 : stop walk
 *   >0 : continue walk
 *
 * additionally for preorder:
 *   -1 : continue walk, skip children
 * <(-1): continue walk, skip children, skip following sibling
 */
__PUB void  vipm_fwtsr_(walk)           __P((struct _Vipm_fwtsregion *, vipm_int, vipm_tt_visitor_fn, void *));

__PUB int   vipm_fwtsr_(walk,preorder)  __P((struct _Vipm_fwtsregion *, vipm_tt_visitor_fn, void *));
__PUB int   vipm_fwtsr_(walk,inorder)   __P((struct _Vipm_fwtsregion *, vipm_tt_visitor_fn, void *));
__PUB int   vipm_fwtsr_(walk,postorder) __P((struct _Vipm_fwtsregion *, vipm_tt_visitor_fn, void *));

/*
 * vipm_int vipm_fwtsr_(pixenum,2d)     (aRootRegion, aPxlStg, aHeight, aWidth, aRegVisitor, aPixVisitor, aCtx);
 *
 * calls for every pixel
 * int  vipm_fwtsr_pixvisitor_fn        (Y, X, aContext);
 *
 * calls for every region before children enum(anIdx = -1) and after each child(anIdx = child num)
 * return value has same behavior as walk visitors
 * int  vipm_fwtsr_regvisitor_fn        (aRegion, aContext, anIdx, aFlags);
 * flags: VipmF_FWTSR_LAST_CHILD
 */
__PUB vipm_int  vipm_fwtsr_(pixenum,2d) __P((struct _Vipm_fwtsregion *, bo_int32_t const *,
                                             vipm_int, vipm_int, vipm_fwtsr_regvisitor_fn, vipm_fwtsr_pixvisitor_fn, void *));

/* ------------------------------------------------------------------------- */

/*
 * bo_seqlist_(vipm_2d_point_t)    *vipm_fwtsr_(fetchpix)       (aPixList, aRegion, aPxlStg, aProperPixOnly, aRank, aVolume);
 * bo_seqlist_(vipm_2d_point_t)    *vipm_fwtsr_(fetchpix,2d)    (aPixList, aRegion, aPxlStg, aProperPixOnly, aHeight, aWidth);
 */
__PUB bo_seqlist_(vipm_2d_point_t) *vipm_fwtsr_(fetchpix)       __P((bo_seqlist_(vipm_2d_point_t) *,
                                    struct _Vipm_fwtsregion const *, bo_int32_t const *, int, vipm_rank_t, vipm_int const []));
__PUB bo_seqlist_(vipm_2d_point_t) *vipm_fwtsr_(fetchpix,2d)    __P((bo_seqlist_(vipm_2d_point_t) *,
                                    struct _Vipm_fwtsregion const *, bo_int32_t const *, int, vipm_int, vipm_int));

/*
 * below functions return add-number-of-regions
 * vipm_int vipm_fwtsr_(fetchpixregs,2d)        (aPixRegionList, aPixRegionListSize, aPixList, aRegion, aPxlStg, aHeight, aWidth);
 * vipm_int vipm_fwtsr_(efetchpixregs,2d)       (aPixRegionList, aPixList, aRegion, aPxlStg, aHeight, aWidth);
 * vipm_int vipm_fwtsr_(fetchpixregs,mser,2d)   (aPixRegionList, aPixRegionListSize, aPixList, aRegion, aPxlStg, aHeight, aWidth);
 * vipm_int vipm_fwtsr_(efetchpixregs,mser,2d)  (aPixRegionList, aPixList, aRegion, aPxlStg, aHeight, aWidth);
 */
__PUB vipm_int  vipm_fwtsr_(fetchpixregs,2d)        __P((vipm_fwtspixregion_t [], vipm_int, bo_seqlist_(vipm_2d_point_t) *,
                                                         struct _Vipm_fwtsregion const *, bo_int32_t const *, vipm_int, vipm_int));
__PUB vipm_int  vipm_fwtsr_(efetchpixregs,2d)       __P((bo_seqlist_(vipm_fwtspixregion_t) *, bo_seqlist_(vipm_2d_point_t) *,
                                                         struct _Vipm_fwtsregion const *, bo_int32_t const *, vipm_int, vipm_int));
__PUB vipm_int  vipm_fwtsr_(fetchpixregs,mser,2d)   __P((vipm_fwtspixregion_t [], vipm_int, bo_seqlist_(vipm_2d_point_t) *,
                                                         struct _Vipm_fwtsregion const *, bo_int32_t const *, vipm_int, vipm_int));
__PUB vipm_int  vipm_fwtsr_(efetchpixregs,mser,2d)  __P((bo_seqlist_(vipm_fwtspixregion_t) *, bo_seqlist_(vipm_2d_point_t) *,
                                                         struct _Vipm_fwtsregion const *, bo_int32_t const *, vipm_int, vipm_int));

/* ------------------------------------------------------------------------- */

/*
 * void vipm_fwtsr_(_fetchpixregs,2d)       (aPixRegionList, aPixList, aRegion, aPxlStg, aHeight, aWidth);
 * void vipm_fwtsr_(_efetchpixregs,2d)      (aPixRegionList, aPixList, aRegion, aNumCldRegions, aPxlStg, aHeight, aWidth);
 * void vipm_fwtsr_(_fetchpixregs,mser,2d)  (aPixRegionList, aPixList, aRegion, aNumMserCldRegions, aPxlStg, aHeight, aWidth);
 * void vipm_fwtsr_(_efetchpixregs,mser,2d) (aPixRegionList, aPixList, aRegion, aNumMserCldRegions, aPxlStg, aHeight, aWidth);
 */

__PUB void  vipm_fwtsr_(_fetchpixregs,2d)       __P((vipm_fwtspixregion_t [], bo_seqlist_(vipm_2d_point_t) *,
                                                     struct _Vipm_fwtsregion const *, bo_int32_t const *, vipm_int, vipm_int));
__PUB void  vipm_fwtsr_(_efetchpixregs,2d)      __P((bo_seqlist_(vipm_fwtspixregion_t) *, bo_seqlist_(vipm_2d_point_t) *,
                                                     struct _Vipm_fwtsregion const *, vipm_int, bo_int32_t const *, vipm_int, vipm_int));
__PUB void  vipm_fwtsr_(_fetchpixregs,mser,2d)  __P((vipm_fwtspixregion_t [], bo_seqlist_(vipm_2d_point_t) *,
                                                     struct _Vipm_fwtsregion const *, vipm_int, bo_int32_t const *, vipm_int, vipm_int));
__PUB void  vipm_fwtsr_(_efetchpixregs,mser,2d) __P((bo_seqlist_(vipm_fwtspixregion_t) *, bo_seqlist_(vipm_2d_point_t) *,
                                                     struct _Vipm_fwtsregion const *, vipm_int, bo_int32_t const *, vipm_int, vipm_int));

/* ------------------------------------------------------------------------- */

/*
 * bo_seqlist_(...) vipm_fwtsr_(...)    (...List, aPixRegionList);
 */

__PUB vipm_wtsmomentfeat_t *vipm_fwtsr_(mf1,2d) __P((vipm_wtsmomentfeat_t *, vipm_fwtspixregion_t *));

__PUB bo_seqlist_(vipm_wtsmomentfeat_t)    *vipm_fwtsr_(mf,2d)  __P((bo_seqlist_(vipm_wtsmomentfeat_t) *,
                                                                     bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_wtsmoment1_t)       *vipm_fwtsr_(m1,2d)  __P((bo_seqlist_(vipm_wtsmoment1_t) *,
                                                                     bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_wtsmoment2_t)       *vipm_fwtsr_(m2,2d)  __P((bo_seqlist_(vipm_wtsmoment2_t) *,
                                                                     bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_wtsmoment2_t)       *vipm_fwtsr_(m2c,2d) __P((bo_seqlist_(vipm_wtsmoment2_t) *,
                                                                     bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_2d_rect_t)          *vipm_fwtsr_(brect,2d)   __P((bo_seqlist_(vipm_2d_rect_t) *,
                                                                         bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_Quad_t(f2d))        *vipm_fwtsr_(minbquad,f2d)   __P((bo_seqlist_(vipm_Quad_t(f2d)) *,
                                                                             bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_Quad_t(r2d))        *vipm_fwtsr_(minbquad,r2d)   __P((bo_seqlist_(vipm_Quad_t(r2d)) *,
                                                                             bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_Ell_t(f2d))         *vipm_fwtsr_(ellipse,f2d)    __P((bo_seqlist_(vipm_Ell_t(f2d)) *,
                                                                             bo_seqlist_(vipm_fwtspixregion_t) *));
__PUB bo_seqlist_(vipm_Ell_t(r2d))         *vipm_fwtsr_(ellipse,r2d)    __P((bo_seqlist_(vipm_Ell_t(r2d)) *,
                                                                             bo_seqlist_(vipm_fwtspixregion_t) *));

/* ------------------------------------------------------------------------- */

/*
 * bo_int32_t   vipm_fwtsr_(countpix{,proper})  (aRegion);
 */
__PUB bo_int32_t    vipm_fwtsr_(countpix)           __P((struct _Vipm_fwtsregion const *));
__PUB bo_int32_t    vipm_fwtsr_(countpix,proper)    __P((struct _Vipm_fwtsregion const *));

/* ------------------------------------------------------------------------- */

/*
 * make tail lind refering to the end of proper pixels of the region
 * vipm_fwtsregion_t *vipm_fwtsr_(fixup)    (aRegion, aPxlStg);
 */
__PUB struct _Vipm_fwtsregion  *vipm_fwtsr_(fixup)  __P((struct _Vipm_fwtsregion *, bo_int32_t const *));

/* ------------------------------------------------------------------------- */

/*
 * vipm_fwtsregion_t   *vipm_fwtsr_(addcld)             (aRegionPar, aRegionCld);
 * vipm_int             vipm_fwtsr_(countcld{,mser})    (aRegion, aRecursively);
 * vipm_int             vipm_fwtsr_(countcld{,mser},R)  (aRegion);
 */
__PUB struct _Vipm_fwtsregion  *vipm_fwtsr_(addcld)             __P((struct _Vipm_fwtsregion *, struct _Vipm_fwtsregion *));
__PUB vipm_int                  vipm_fwtsr_(countcld)           __P((struct _Vipm_fwtsregion const *, int));
__PUB vipm_int                  vipm_fwtsr_(countcld,mser)      __P((struct _Vipm_fwtsregion const *, int));
__PUB vipm_int                  vipm_fwtsr_(countcld,R)         __P((struct _Vipm_fwtsregion const *));
__PUB vipm_int                  vipm_fwtsr_(countcld,mser,R)    __P((struct _Vipm_fwtsregion const *));


/* ========================================================================= */

//__PUB double             *VipmFWTSRmoment1           __P((double [/*2*/], vipm_fwtsregion_t const *, bo_int32_t const *));

/* ------------------------------------------------------------------------- */

//__PUB double             *vipm_2d_ptv_moment1           __P((double [/*2*/], vipm_2d_point_t const [], vipm_int));


/* ========================================================================= */

/*
 * void vipm_fwtsr_(__addcld)   (aRegionPar, aRegionCld);
 */
__PROT void vipm_fwtsr_(__addcld)   __P((struct _Vipm_fwtsregion *, struct _Vipm_fwtsregion *));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

# define vipm_fwtsr_countpix(r)    ((r)->wsr_pixelc)

_VIPM_INLINE void
vipm_fwtsr_(__addcld)(
#if __myc_require(prototypes)
	struct _Vipm_fwtsregion        *aRegionPar,
	struct _Vipm_fwtsregion        *aRegionCld
#else
	aRegionPar, aRegionCld
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_fwtsregion        *aRegionPar, *aRegionCld;
#endif

{
	aRegionCld->wsr_parent = aRegionPar;
	aRegionCld->wsr_next = aRegionPar->wsr_child;
	aRegionPar->wsr_child = aRegionCld;
}


__MYC_END_CDECLS
# undef __PUB
# undef __PROT
# undef __PRIV


#endif /* !defined(__VIPM_ANALYSIS_WATERSHED_FWTSREGION_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
