/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Types.h	(V.Sokur)
 * Vipm/Analysis/Watershed/Types.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ANALYSIS_WATERSHED_TYPES_H__
# define __VIPM_ANALYSIS_WATERSHED_TYPES_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

# define VipmK_UPWARD_WATERSHED         0
# define VipmK_DOWNWARD_WATERSHED       1
# define VipmK_UP_DOWN_WATERSHED        2

# define VipmK_WTSFR_ALIGN              8

/* ------------------------------------------------------------------------- */

struct _Vipm_wtspixel;
struct _Vipm_wtsregion;
struct _Vipm_wtscontext;
struct _Vipm_wtsparm;

struct _Vipm_wtslistener;
struct _Vipm_wtslsropv;
struct _Vipm_wtslsrctx;

/* ------------------------------------------------------------------------- */

struct _Vipm_wtspixel {
	bo_int32_t              lind;
	bo_int32_t              value;
};
typedef struct _Vipm_wtspixel           vipm_wtspixel_t;
# define vipm_wtspixel_p(x)             vipm_wtspixel_cp(x)
# define vipm_wtspixel_cp(x)           ((struct _Vipm_wtspixel const *)(x))
# define vipm_wtspixel_mp(x)           ((struct _Vipm_wtspixel *)(x))

/* ------------------------------------------------------------------------- */

/*
 * Watershed region
 */

# ifndef __vipm_wtsregion_t
#  define __vipm_wtsregion_t            struct _Vipm_wtsregion
# endif

struct _Vipm_wtsregion {
	/* struct _Vipm_dgtdata { */
	void                   *wsr_node_;                  /* private */
	/* } */

	struct _Vipm_wtspixel   wsr_pivot;
	bo_int32_t              wsr_pixelc;
	bo_int32_t              wsr_ismser;

# if (BO_WIDTHOF_MWORD < 64)
	bo_int32_t              wsr_pad_;
# endif

# if 0
	char                    wsr_features[];
# endif
};
typedef __vipm_wtsregion_t              vipm_wtsregion_t;
# define vipm_wtsregion_p(x)            vipm_wtsregion_cp(x)
# define vipm_wtsregion_cp(x)          ((__vipm_wtsregion_t const *)(x))
# define vipm_wtsregion_mp(x)          ((__vipm_wtsregion_t *)(x))

typedef char __STATIC_ASSERT_WTSREGION_ALIGN[(sizeof(vipm_wtsregion_t) % VipmK_WTSFR_ALIGN) == 0 ? 1 : -1];


# define vipm_wtsregion_feature(_Ty,r,i,featList) ((_Ty *)vipm_wtsregion__feature(r,i,featList))
# define vipm_wtsregion__feature(r,i,featList)    ((char *)(r) + (featList)[(i)].wsf_offset)
/* short version */
# define vipm_wtsr_f(_Ty,r,i,featList)            ((_Ty *)vipm_wtsregion__feature(r,i,featList))
# define vipm_wtsr__f(r,i,featList)               vipm_wtsregion__feature(r,i,featList)


/*
 * "Fast" watershed region
 */

struct _Vipm_fwtsregion {
	union {
		struct {
			bo_int32_t                      wsr_head;
			union {
				/* [ .0 = lind, .1 = value ] */
				struct _Vipm_wtspixel       wsr_tail_;
				struct _Vipm_wtspixel       wsr_pivot;
			};
			bo_int32_t                      wsr_reserve1;
			bo_int32_t                      wsr_pixelc;
		};
		struct { /* [2D] pix-region */
			void/* ex.: vipm_2d_point_t */ *wsr_pixelv;
# if (BO_WIDTHOF_MWORD <= 36)
			bo_int32_t                      wsr_reserve2;
# endif
			bo_int32_t                      wsr_level;
			bo_int32_t                      wsr_nproper;
			bo_int32_t                      wsr_ntotal;
		};
	};

	bo_int32_t                      wsr_ismser;

	/* internal @{ */
	struct _Vipm_fwtsregion        *wsr_next,
	                               *wsr_child,
	                               *wsr_parent;
	/* @} */
};
typedef struct _Vipm_fwtsregion         vipm_fwtsregion_t;
# define vipm_fwtsregion_p(x)           vipm_fwtsregion_cp(x)
# define vipm_fwtsregion_cp(x)         ((struct _Vipm_fwtsregion const *)(x))
# define vipm_fwtsregion_mp(x)         ((struct _Vipm_fwtsregion *)(x))

struct _Vipm_fwtspixregion;
# define _Vipm_fwtspixregion            _Vipm_fwtsregion
typedef struct _Vipm_fwtspixregion      vipm_fwtspixregion_t;
# define vipm_fwtspixregion_p(x)        vipm_fwtspixregion_cp(x)
# define vipm_fwtspixregion_cp(x)      ((struct _Vipm_fwtspixregion const *)(x))
# define vipm_fwtspixregion_mp(x)      ((struct _Vipm_fwtspixregion *)(x))

# define vipm_fwtsregion(...)           VIPM_CTOR_FCNAME(vipm_fwtsregion, __VA_ARGS__)
# define vipm_fwtsregion_(...)          VIPM_PFX_FCNAME(vipm_fwtsregion, __VA_ARGS__)

/* short versions */
# define vipm_fwtsr_(...)               VIPM_PFX_FCNAME(vipm_fwtsr, __VA_ARGS__)

/* callbacks for traverse */
typedef int (*vipm_fwtsr_pixvisitor_fn)     __P((vipm_int, vipm_int, void *));
typedef int (*vipm_fwtsr_regvisitor_fn)     __P((struct _Vipm_fwtspixregion *, void *, vipm_int, bo_uint32_t));

# define VipmF_FWTSR_LAST_CHILD         0x00000001

# define VipmK_FWTS_PIXVAL_MASK         0xFF
# define VipmK_FWTS_PIXLIND_MASK        0x7FFFFFFF
# define VipmK_FWTS_PIXLIND_INVAL       0x7FFFFFFF
# define VipmK_FWTS_PIXNIL              VipmK_FWTS_PIXLIND_INVAL
# define VipmK_FWTS_PIXMAX              255
# define VipmK_FWTS_PIXSUP              256

/* internal @{ */
# define __VIPM_FWTSREGION_Init(r,l,v)                  \
	(r)->wsr_head = VipmK_FWTS_PIXNIL;                  \
	(r)->wsr_tail_.lind = (l);                          \
	(r)->wsr_tail_.value = (v);                         \
	(r)->wsr_pixelc = 0;                                \
	(r)->wsr_ismser = 0;                                \
	(r)->wsr_next = NULL;                               \
	(r)->wsr_child = NULL;                              \
	(r)->wsr_parent = NULL
/* @} */


struct _Vipm_fwtsresult {
	vipm_fwtsregion_t              *wts_rregion;
	bo_int32_t                     *wts_pxlstg;
	bo_int32_t                     *wts_opxlstg;
	vipm_rref_t                     wts_mmstack;

	/* internal @{ */
	bo_memstg_t                     wts_mmstackb_;
	vipm_mmstate_t                  wts_mmstate_;
	/* @} */
};
typedef struct _Vipm_fwtsresult         vipm_fwtsresult_t;
# define vipm_fwtsresult_p(x)           vipm_fwtsresult_cp(x)
# define vipm_fwtsresult_cp(x)         ((struct _Vipm_fwtsresult const *)(x))
# define vipm_fwtsresult_mp(x)         ((struct _Vipm_fwtsresult *)(x))

/*
 * Watershed "fast" pixel iterator
 */

typedef bo_int32_t                              vipm_fwtspixiter_t;
# define vipm_fwtspixiter_begin(aRegion)       (aRegion)->wsr_head
# define vipm_fwtspixiter_next(anIter,aPxlStg) (aPxlStg)[(anIter)]
# define vipm_fwtspixiter_end(anIter)          ((anIter) == VipmK_FWTS_PIXNIL)


/*
 * Watershed feature
 */

typedef void                            vipm_feature_t;

/* Feature callbacks */
typedef void (*vipm_wsf_init_fn)        __P((vipm_feature_t *, void *));
typedef void (*vipm_wsf_destroy_fn)     __P((vipm_feature_t *));
typedef void (*vipm_wsf_copy_fn)        __P((vipm_feature_t *, vipm_feature_t const *));
typedef void (*vipm_wsf_update_fn)      __P((vipm_feature_t *, vipm_Xpoint_t const *, vipm_uint));
typedef void (*vipm_wsf_update2D_fn)    __P((vipm_feature_t *, vipm_2d_point_t const *, vipm_uint));
typedef void (*vipm_wsf_merge_fn)       __P((vipm_feature_t *, vipm_feature_t *));
typedef void (*vipm_wsf_lazymerge_fn)   __P((vipm_feature_t *, vipm_feature_t *));    /* called by gather procedure */
typedef int  (*vipm_wsf_isgathered_fn)  __P((vipm_feature_t *));
typedef void (*vipm_wsf_ongather_fn)    __P((vipm_feature_t *, vipm_wtsregion_t *));  /* called on start of gather procedure */

struct _Vipm_wtsfeature_ds {
	bo_uint32_t                     wsf_magic;
	vipm_int                        wsf_size, wsf_offset;

	void                           *wsf_shared;

	/* optional */
	vipm_wsf_init_fn                wsf_init;
	vipm_wsf_destroy_fn             wsf_destroy;
	vipm_wsf_copy_fn                wsf_copy;
	/* mandatory */
	vipm_wsf_update_fn              wsf_update;
	vipm_wsf_update2D_fn            wsf_update2D;
	vipm_wsf_merge_fn               wsf_merge;
	/* optional */
	vipm_wsf_lazymerge_fn           wsf_lazymerge;
	vipm_wsf_isgathered_fn          wsf_isgathered;
	vipm_wsf_ongather_fn            wsf_ongather;
};

/* ------------------------------------------------------------------------- */

/*
 * Watershed parameters
 */

struct _Vipm_wtsparm {
	short                           wsp_direction;      /* = 0 ; up-ward watershed */
	short                           wsp_dovalxor;       /* = 0 ; useful when down-ward watershed */
	short                           wsp_use8nn;         /* = 0 */
	short                           wsp_scalefactor;    /* = 0 */
	vipm_Npoint_t                   wsp_start_pix;      /* = {0} ; local coordinate system */
};
typedef struct _Vipm_wtsparm            vipm_wtsparm_t;
# define vipm_wtsparm_p(x)              vipm_wtsparm_cp(x)
# define vipm_wtsparm_cp(x)            ((struct _Vipm_wtsparm const *)(x))
# define vipm_wtsparm_mp(x)            ((struct _Vipm_wtsparm *)(x))

/* ------------------------------------------------------------------------- */

/*
 * Watershed listener and listener-context
 */

struct _Vipm_wtslsrctx {
	struct _Vipm_wtsfeature_ds     *wlc_featdes;
	vipm_int                        wlc_featurec;
	vipm_uint                       wlc_sizeof_region;  /* initial: 0 or valid-value */
	vipm_Xvector_t                  wlc_image_;         /* internal (?) */
};
typedef struct _Vipm_wtslsrctx          vipm_wtslsrctx_t;
# define vipm_wtslsrctx_p(x)            vipm_wtslsrctx_cp(x)
# define vipm_wtslsrctx_cp(x)          ((struct _Vipm_wtslsrctx const *)(x))
# define vipm_wtslsrctx_mp(x)          ((struct _Vipm_wtslsrctx *)(x))


# ifndef __vipm_wtslistener_t
#  define __vipm_wtslistener_t          struct _Vipm_wtslistener
# endif

struct _Vipm_wtslistener {
	volatile struct _Vipm_xinterface   *wls_opv;
};

typedef __vipm_wtslistener_t            vipm_wtslistener_t;
# define vipm_wtslistener_p(x)          vipm_wtslistener_cp(x)
# define vipm_wtslistener_cp(x)        ((__vipm_wtslistener_t const *)(x))
# define vipm_wtslistener_mp(x)        ((__vipm_wtslistener_t *)(x))


/*
 * Watershed listener operations
 */

typedef vipm_mutptr_t     (*vipm_wslsr_alloca_fn)       __P((vipm_wtslistener_t *, bo_size_t, bo_size_t));
typedef void              (*vipm_wslsr_start_fn)        __P((vipm_wtslistener_t *, vipm_wtslsrctx_t *, vipm_ulong, vipm_wtsparm_t const *));

typedef void              (*vipm_wslsr_finish_fn)       __P((vipm_wtslistener_t *, vipm_wtslsrctx_t *, vipm_wtsparm_t const *));
typedef vipm_wtsregion_t *(*vipm_wslsr_newregion_fn)    __P((vipm_wtsregion_t *, vipm_wtslistener_t *, vipm_wtslsrctx_t *));
typedef void              (*vipm_wslsr_destroyrgn_fn)   __P((vipm_wtsregion_t *, vipm_wtslistener_t *, vipm_wtslsrctx_t *));
typedef void              (*vipm_wslsr_disposergn_fn)   __P((vipm_wtsregion_t *, vipm_wtslistener_t *, vipm_wtslsrctx_t *));
typedef void              (*vipm_wslsr_accumulate_fn)   __P((vipm_wtslistener_t *, vipm_wtslsrctx_t *,
                                                             vipm_wtsregion_t *, vipm_Xpoint_t const *, vipm_uint));
typedef void              (*vipm_wslsr_accumulate2D_fn) __P((vipm_wtslistener_t *, vipm_wtslsrctx_t *,
                                                             vipm_wtsregion_t *, vipm_2d_point_t const *, vipm_uint));
typedef void              (*vipm_wslsr_merge_fn)        __P((vipm_wtslistener_t *, vipm_wtslsrctx_t *,
                                                             vipm_wtsregion_t *, vipm_wtsregion_t *));
typedef void             *(*vipm_wslsr_gatherfeat_fn)   __P((vipm_wtslistener_t *, vipm_wtslsrctx_t *,
                                                             vipm_feature_t *, vipm_feature_t *, vipm_int, bo_uint32_t));


# ifndef __vipm_wtslsropv_t
#  define __vipm_wtslsropv_t            struct _Vipm_wtslsropv
# endif

struct _Vipm_wtslsropv {
	/* struct _Vipm_xinterface { */
	bo_uint32_t                     magic; /* 'wtsl' */
	bo_uint32_t                     size;
	/* } */
	vipm_wslsr_alloca_fn            alloca;

	vipm_wslsr_start_fn             start;
	vipm_wslsr_finish_fn            finish;
	vipm_wslsr_newregion_fn         newregion;
	vipm_wslsr_destroyrgn_fn        destroyrgn;
	vipm_wslsr_disposergn_fn        disposergn;
	vipm_wslsr_accumulate_fn        accumulate;
	vipm_wslsr_accumulate2D_fn      accumulate2D;
	vipm_wslsr_merge_fn             merge;
	vipm_wslsr_gatherfeat_fn        gatherfeat;
};

typedef __vipm_wtslsropv_t              vipm_wtslsropv_t;
# define vipm_wtslsropv_p(x)            vipm_wtslsropv_cp(x)
# define vipm_wtslsropv_cp(x)          ((__vipm_wtslsropv_t const *)(x))
# define vipm_wtslsropv_mp(x)          ((__vipm_wtslsropv_t *)(x))


/* ------------------------------------------------------------------------- */

/*
 * Watershed context
 */

struct _Vipm_wtscontext {
	union {
		struct {
			vipm_wtslistener_t     *wsc_lsr;
			vipm_wtslsrctx_t       *wsc_lsrctx;
		};
		vipm_fwtsresult_t          *wsc_res;            /* "fast" watershed */
	};
	vipm_wtsparm_t                  wsc_parm;
};
typedef struct _Vipm_wtscontext         vipm_wtscontext_t;
# define vipm_wtscontext_p(x)           vipm_wtscontext_cp(x)
# define vipm_wtscontext_cp(x)         ((struct _Vipm_wtscontext const *)(x))
# define vipm_wtscontext_mp(x)         ((struct _Vipm_wtscontext *)(x))


/* ========================================================================= */

typedef bo_status_t (*vipma_watershed_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	struct _Vipm_wtscontext        *aWtsctx
# endif
	);

typedef bo_status_t (*vipma_watershedex_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[],
	struct _Vipm_wtscontext        *aWtsctx
# endif
	);


#endif /* !defined(__VIPM_ANALYSIS_WATERSHED_TYPES_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
