/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Arithmetic.h	(V.Sokur)
 * Vipm/ArithLogic/Arithmetic.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ARITHLOGIC_ARITHMETIC_H__
# define __VIPM_ARITHLOGIC_ARITHMETIC_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_ARITHLOGIC_ARITHMETIC_ALGORITHM_H__
#  include "Vipm/ArithLogic/Arithmetic/Algorithm.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
__MYC_BEGIN_CDECLS

/*
 * ROI samples:
 * 1) point : (nplanes:0,y,x,nchannels:0)
 *    volume: (nplanes:1,h,w,nchannels)
 */

/*
 * ADD
 *
 * add: r[i] = (a[i] + b[i]) * (2**f)
 * addq{int}: saturate(round(scale(add)))
 * addsq: addq with signed result
 */

__PUB bo_status_t
VipmAdd(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmAddC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmAddq(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmAddqC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmAddsq(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmAddsqC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * SUB
 *
 * sub: r[i] = (a[i] + b[i]) * (2**f)
 * subq{int}: saturate(round(scale(sub)))
 * subsq: subq with signed result
 */

__PUB bo_status_t
VipmSub(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmSubC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmSubq(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmSubqC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmSubsq(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmSubsqC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * RSUB
 *
 * rsub: r[i] = (b[i] - a[i]) * (2**f)
 * rsubq{int}: saturate(round(scale(rsub)))
 * rsubsq: rsubq with signed result
 */

__PUB bo_status_t
VipmRSub(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmRSubC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmRSubq(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmRSubqC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmRSubsq(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    Addend,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartAddend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmRSubsqC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * MUL
 *
 * mul: r[i] = (a[i] * b[i]) * (2**f)
 * mul{int}: saturate(round(scale(mul)))
 */

__PUB bo_status_t
VipmMul(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_array_t                    aMultiplier,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartMultiplicand,
	vipm_Npoint_t const            *aStartMultiplier,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_scalar_t const             aMultiplier[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartMultiplicand,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * DIV(R)
 *
 * div: r[i] = (a[i] / b[i]) * (2**f)
 * div{int} : saturate(round(scale(int-div)))
 * divr{int}: saturate(round(scale(float-div)))
 */

__PUB bo_status_t
VipmDiv(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_array_t                    aDivisor,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	vipm_Npoint_t const            *aStartDivisor,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);

__PUB bo_status_t
VipmDivC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_scalar_t const             aDivisor[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);

__PUB bo_status_t
VipmDivr(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_array_t                    aDivisor,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	vipm_Npoint_t const            *aStartDivisor,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);

__PUB bo_status_t
VipmDivrC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_scalar_t const             aDivisor[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);


/*
 * RDIV(R)
 *
 * rdiv: r[i] = (b[i] / a[i]) * (2**f)
 * rdiv{int} : saturate(round(scale(int-div)))
 * rdivr{int}: saturate(round(scale(float-div)))
 */

__PUB bo_status_t
VipmRDiv(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_array_t                    aDivisor,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	vipm_Npoint_t const            *aStartDivisor,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);

__PUB bo_status_t
VipmRDivC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_scalar_t const             aDivisor[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);

__PUB bo_status_t
VipmRDivr(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_array_t                    aDivisor,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	vipm_Npoint_t const            *aStartDivisor,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);

__PUB bo_status_t
VipmRDivrC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aDividend,
	vipm_scalar_t const             aDivisor[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartDividend,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT
# endif
	);


/*
 * MULADD(MLA)
 *
 * mla  : r[i] = (a[i] + b[i] * c[i]) * (2**f)
 * mlaC : r[i] = (a[i] + b[i] * c) * (2**f)
 * mlaCC: r[i] = (a[i] * b + c) * (2**f)
 * mla{int}: saturate(round(scale(mla)))
 */

__PUB bo_status_t
VipmMulAdd(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    aMultiplicand,
	vipm_array_t                    aMultiplier,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartMultiplicand,
	vipm_Npoint_t const            *aStartMultiplier,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulAddC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    aMultiplicand,
	vipm_scalar_t const             aMultiplier[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartMultiplicand,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulAddCC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_scalar_t const             aMultiplier[/* nchannels */],
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartMultiplicand,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * MULSUB(MLS)
 *
 * mls  : r[i] = (a[i] - b[i] * c[i]) * (2**f)
 * mlsC : r[i] = (a[i] - b[i] * c) * (2**f)
 * mlsCC: r[i] = (a[i] * b - c) * (2**f)
 * mls{int}: saturate(round(scale(mls)))
 */

__PUB bo_status_t
VipmMulSub(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    aMultiplicand,
	vipm_array_t                    aMultiplier,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartMultiplicand,
	vipm_Npoint_t const            *aStartMultiplier,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulSubC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    aMultiplicand,
	vipm_scalar_t const             aMultiplier[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartMultiplicand,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulSubCC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_scalar_t const             aMultiplier[/* nchannels */],
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartMultiplicand,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * MULRSUB(MLRS)
 *
 * mlrs  : r[i] = (b[i] * c[i] - a[i]) * (2**f)
 * mlrsC : r[i] = (b[i] * c - a[i]) * (2**f)
 * mlrsCC: r[i] = (c - a[i] * b) * (2**f)
 * mlrs{int}: saturate(round(scale(mlrs)))
 */

__PUB bo_status_t
VipmMulRSub(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    aMultiplicand,
	vipm_array_t                    aMultiplier,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartMultiplicand,
	vipm_Npoint_t const            *aStartMultiplier,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulRSubC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    Augend,
	vipm_array_t                    aMultiplicand,
	vipm_scalar_t const             aMultiplier[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartAugend,
	vipm_Npoint_t const            *aStartMultiplicand,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulRSubCC(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_scalar_t const             aMultiplier[/* nchannels */],
	vipm_scalar_t const             Addend[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartMultiplicand,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * MATMUL
 *
 * mulm*: r[i] = (a[i] * b[i]) * (2**f)
 * mulm*{int}: saturate(round(scale(mulmm)))
 *
 * mulmm : matrix-matrix multiply
 * mulmv : matrix-vector multiply
 * mulmvn: matrix-vectorList multiply
 */

__PUB bo_status_t
VipmMulMM(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_array_t                    aMultiplier,
	vipm_site_t const              *aRoiDst,
	vipm_site_t const              *aRoiMultiplicand,
	vipm_Npoint_t const            *aStartMultiplier,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulMV(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_array_t                    aMultiplier,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartMultiplicand,
	vipm_Npoint_t const            *aStartMultiplier,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmMulMV_n(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aMultiplicand,
	vipm_array_t                    aMultiplier[/* aMultiplierNum */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartMultiplicand,
	vipm_Npoint_t const            *aStartMultiplier,
	vipm_uint                       aMultiplierNum,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * NEG
 */

__PUB bo_status_t
VipmNegate(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aSource,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartSrc,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);

__PUB bo_status_t
VipmNegateq(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aSource,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartSrc,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * EXP
 */

__PUB bo_status_t
VipmExp(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aSource,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartSrc,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * Sigmoid
 *
 * sigmoid:      1/(1 + e^(-a[i]))
 * sigmoidalpha: 1/(1 + e^(alpha * a[i]))
 */

__PUB bo_status_t
VipmSigmoidAlpha(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aSource,
	vipm_scalar_t const             anAlpha[/* nchannels */],
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartSrc,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


/*
 * LN
 */

__PUB bo_status_t
VipmLn(
# ifdef __cplusplus
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_mut_array_t                aDest,
	vipm_array_t                    aSource,
	vipm_site_t const              *aRoiDst,
	vipm_Npoint_t const            *aStartSrc,
	/* Useful for integer types */
	vipm_int                        aScaleFactor = 0,
	vipm_int                        aRoundMode   = VipmK_RND_DEFAULT /* used when scale-factor<0 */
# endif
	);


# ifndef __cplusplus
#  define VipmAdd(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmAdd)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmAddC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmAddC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmAddq(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmAddq)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmAddqC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmAddqC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmAddsq(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmAddsq)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmAddsqC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmAddsqC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmSub(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmSub)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmSubC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmSubC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmSubq(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmSubq)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmSubqC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmSubqC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmSubsq(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmSubsq)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmSubsqC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmSubsqC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRSub(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmRSub)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRSubC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmRSubC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRSubq(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmRSubq)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRSubqC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmRSubqC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRSubsq(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmRSubsq)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRSubsqC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmRSubsqC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMul(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmMul)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmMulC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmDiv(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmDiv)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmDivC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmDivC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmDivr(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmDivr)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmDivrC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmDivrC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRDiv(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmRDiv)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRDivC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmRDivC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRDivr(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmRDivr)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmRDivrC(x,f,d,s1,s2,rcd,os1,scf,rnd) \
	(VipmRDivrC)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulAdd(x,f,d,s1,s2,s3,rcd,os1,os2,os3,scf,rnd) \
	(VipmMulAdd)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,os2,os3,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulAddC(x,f,d,s1,s2,s3,rcd,os1,os2,scf,rnd) \
	(VipmMulAddC)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulAddCC(x,f,d,s1,s2,s3,rcd,os1,scf,rnd) \
	(VipmMulAddCC)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulSub(x,f,d,s1,s2,s3,rcd,os1,os2,os3,scf,rnd) \
	(VipmMulSub)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,os2,os3,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulSubC(x,f,d,s1,s2,s3,rcd,os1,os2,scf,rnd) \
	(VipmMulSubC)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulSubCC(x,f,d,s1,s2,s3,rcd,os1,scf,rnd) \
	(VipmMulSubCC)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulRSub(x,f,d,s1,s2,s3,rcd,os1,os2,os3,scf,rnd) \
	(VipmMulRSub)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,os2,os3,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulRSubC(x,f,d,s1,s2,s3,rcd,os1,os2,scf,rnd) \
	(VipmMulRSubC)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulRSubCC(x,f,d,s1,s2,s3,rcd,os1,scf,rnd) \
	(VipmMulRSubCC)(x,(vipm_ulong)(f),d,s1,s2,s3,rcd,os1,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulMM(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmMulMM)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulMV(x,f,d,s1,s2,rcd,os1,os2,scf,rnd) \
	(VipmMulMV)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmMulMV_n(x,f,d,s1,s2,rcd,os1,os2,sn,scf,rnd) \
	(VipmMulMV_n)(x,(vipm_ulong)(f),d,s1,s2,rcd,os1,os2,sn,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmNegate(x,f,d,s,rcd,os,scf,rnd) \
	(VipmNegate)(x,(vipm_ulong)(f),d,s,rcd,os,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmNegateq(x,f,d,s,rcd,os,scf,rnd) \
	(VipmNegateq)(x,(vipm_ulong)(f),d,s,rcd,os,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmExp(x,f,d,s,rcd,os,scf,rnd) \
	(VipmExp)(x,(vipm_ulong)(f),d,s,rcd,os,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmSigmoidAlpha(x,f,d,s,a,rcd,os,scf,rnd) \
	(VipmSigmoidAlpha)(x,(vipm_ulong)(f),d,s,a,rcd,os,(vipm_int)(scf),(vipm_int)(rnd))
#  define VipmLn(x,f,d,s,rcd,os,scf,rnd) \
	(VipmLn)(x,(vipm_ulong)(f),d,s,rcd,os,(vipm_int)(scf),(vipm_int)(rnd))
# endif /* !defined(__cplusplus) */


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_ARITHLOGIC_ARITHMETIC_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
