/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2018 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * MatMul.h	(D.Vinnyk)
 * Vipm/ArithLogic/Arithmetic/MatMul.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ARITHLOGIC_ARITHMETIC_MATMUL_H__
# define __VIPM_ARITHLOGIC_ARITHMETIC_MATMUL_H__

/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * mulmm
 */

/*
 * Source Data types: U8U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8u8u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16u8u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32u8u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32u8u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8u8s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16u8s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32u8s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32u8s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8u8s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16u8s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32u8s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32u8s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8u8f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s16u8f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s32u8f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_f32u8f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s16u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16s16u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32s16u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32s16u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s16s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16s16s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32s16s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32s16s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s16s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16s16s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32s16s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32s16s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s16f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s16s16f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s32s16f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_f32s16f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16s32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32s32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32s32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16s32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32s32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32s32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s16s32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_s32s32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmm_f32s32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8s32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s16s32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s32s32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_f32s32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8f32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s16f32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s32f32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_f32f32u8(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8f32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s16f32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s32f32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_f32f32s16(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8f32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s16f32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s32f32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_f32f32s32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmm_u8f32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s16f32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_s32f32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmm_f32f32f32(x,f,rank,vol1,vol2,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmm(x,f,rank,vol1,vol2,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * mulmv
 */

/*
 * Source Data types: U8U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s16u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s32u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_f32u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s16s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s32s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_f32s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s16s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_s32s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_mulmv_f32s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s16s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s32s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_f32s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s16f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s32f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_f32f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s16f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s32f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_f32f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s16f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s32f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_f32f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmv_u8f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s16f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_s32f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_mulmv_f32f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_mulmv(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * mulmvn
 */

/*
 * Source Data types: U8U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: U8S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: U8S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: U8F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s16u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s32u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_f32u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S16U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S16S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S16S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S16F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s16s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s32s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_f32s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s16s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_s32s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2,sf,rnd) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,sf,rnd)

# define vipma_mulmvn_f32s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: S32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s16s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s32s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_f32s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: F32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s16f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s32f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_f32f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: F32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s16f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s32f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_f32f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: F32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s16f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s32f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_f32f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,numS2,0,0)

/*
 * Source Data types: F32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_mulmvn_u8f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s16f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_s32f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

# define vipma_mulmvn_f32f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,numS2) \
    vipma_mulmvn(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,numS2,0,0)

/* ------------------------------------------------------------------------- */

__MYC_BEGIN_CDECLS

/*
 * Special cases
 * Source Data types: F32F32
 * Dest Data type   : F32
 */
/*
 * Special mulmm: Cik = Aij x Bjk
 * function name template:
 * vipma_mulmm_{elemtype(C)}{elemtype(A)}{elemtype(B)}_{i}x{j}x{j}x{k}
 */
__PUB bo_status_t   vipma_mulmm_f32f32f32_1x1x1x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_1x1x1x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_1x1x1x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x1x1x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x1x1x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x1x1x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x1x1x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x1x1x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x1x1x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

__PUB bo_status_t   vipma_mulmm_f32f32f32_1x2x2x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_1x2x2x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_1x2x2x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x2x2x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x2x2x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x2x2x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x2x2x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x2x2x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x2x2x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

__PUB bo_status_t   vipma_mulmm_f32f32f32_1x3x3x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_1x3x3x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_1x3x3x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x3x3x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x3x3x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_2x3x3x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x3x3x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x3x3x2   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_3x3x3x3   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

__PUB bo_status_t   vipma_mulmm_f32f32f32_4x4x4x1   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmm_f32f32f32_4x4x4x4   __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

/*
 * Special mulmmt: Cik = Aij x Bkj
 * function name template:
 * vipma_mulmmt_{elemtype(C)}{elemtype(A)}{elemtype(B)}_{i}x{j}x{k}x{j}
 */
__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x1x1x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x1x2x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x1x3x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x1x1x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x1x2x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x1x3x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x1x1x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x1x2x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x1x3x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x2x1x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x2x2x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x2x3x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x2x1x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x2x2x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x2x3x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x2x1x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x2x2x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x2x3x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x3x1x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x3x2x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_1x3x3x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x3x1x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x3x2x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_2x3x3x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x3x1x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x3x2x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_3x3x3x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

__PUB bo_status_t   vipma_mulmmt_f32f32f32_4x4x1x4  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));
__PUB bo_status_t   vipma_mulmmt_f32f32f32_4x4x4x4  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int));

/*
 * Special mulmvn: Cik = Aij x B[k]j
 * function name template:
 * vipma_mulmvn_{elemtype(C)}{elemtype(A)}{elemtype(B)}_{i}x{j}x{k}x{j}
 */
__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x1x1x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x1x2x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x1x3x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x1x1x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x1x2x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x1x3x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x1x1x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x1x2x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x1x3x1  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));

__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x2x1x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x2x2x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x2x3x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x2x1x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x2x2x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x2x3x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x2x1x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x2x2x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x2x3x2  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));

__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x3x1x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x3x2x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_1x3x3x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x3x1x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x3x2x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_2x3x3x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x3x1x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x3x2x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_3x3x3x3  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));

__PUB bo_status_t   vipma_mulmvn_f32f32f32_4x4x1x4  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));
__PUB bo_status_t   vipma_mulmvn_f32f32f32_4x4x4x4  __P((vipm_mutptr_t, vipm_int,
                                                         vipm_imuptr_t, vipm_int, vipm_imuptr_t *));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

# include <Vipm/ArithLogic/Arithmetic/MatMul_inl.h>


__MYC_END_CDECLS

#endif /* !defined(__VIPM_ARITHLOGIC_ARITHMETIC_MATMUL_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
