/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2018 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * RDivR.h	(D.Vinnyk)
 * Vipm/ArithLogic/Arithmetic/RDivR.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ARITHLOGIC_ARITHMETIC_RDIVR_H__
# define __VIPM_ARITHLOGIC_ARITHMETIC_RDIVR_H__

/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * rdivr
 */

/*
 * Source Data types: U8U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: U8F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s16u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s32u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_f32u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S16F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s16s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s32s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_f32s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s16s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_s32s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2,sf,rnd) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,sf,rnd)

# define vipma_rdivr_f32s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: S32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s16s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s32s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_f32s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32U8
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s16f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s32f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_f32f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_U8,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32S16
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s16f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s32f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_f32f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S16,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32S32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s16f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s32f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_f32f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_S32,S2,offsS2,strideS2,0,0)

/*
 * Source Data types: F32F32
 * Dest Data type   : U8,S16,S32,F32
 */
# define vipma_rdivr_u8f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s16f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_s32f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

# define vipma_rdivr_f32f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,offsS2,strideS2) \
    vipma_rdivr(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S1,offsS1,strideS1,VipmK_ELEMTYPE_F32,S2,offsS2,strideS2,0,0)

/*
 * rdivrC
 */

/*
 * Source Data type: U8
 * Dest Data type  : U8,S16,S32,F32
 */
# define vipma_rdivrC_u8u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_s16u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_s32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_f32u8(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,0,0)

/*
 * Source Data type: S16
 * Dest Data type  : U8,S16,S32,F32
 */
# define vipma_rdivrC_u8s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_s16s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_s32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_f32s16(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,0,0)

/*
 * Source Data type: S32
 * Dest Data type  : U8,S16,S32,F32
 */
# define vipma_rdivrC_u8s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_s16s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_s32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2,sf,rnd) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,sf,rnd)

# define vipma_rdivrC_f32s32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,0,0)

/*
 * Source Data type: F32
 * Dest Data type  : U8,S16,S32,F32
 */
# define vipma_rdivrC_u8f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,0,0)

# define vipma_rdivrC_s16f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,0,0)

# define vipma_rdivrC_s32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,0,0)

# define vipma_rdivrC_f32f32(x,f,rank,vol,D,offsD,strideD,S1,offsS1,strideS1,S2) \
    vipma_rdivrC(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S1,offsS1,strideS1,S2,0,0)


#endif /* !defined(__VIPM_ARITHLOGIC_ARITHMETIC_RDIVR_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
