/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Cdefs.h	(V.Sokur)
 * Vipm/Cdefs.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_CDEFS_H__
# define __VIPM_CDEFS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __BO_CDEFS__
#  include <Bo/Cdefs.h>
# endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

/*
 * Compiler specific
 * @todo: implement
 */

// @todo: fixme; version
# ifndef _ICC_VER
#  if defined(__ICL)
#   define _ICC_VER         __ICL
#  elif defined(__ICC)
#   define _ICC_VER         __ICC
#  elif defined(__ECL)
#   define _ICC_VER         __ECL
#  elif defined(__ECC)
#   define _ICC_VER         __ECC
#  elif defined(__INTEL_COMPILER)
#   define _ICC_VER         __INTEL_COMPILER
#  endif
# endif

# ifdef __GNUC__
#  define _GCC_VER          (__GNUC__ * 10000 + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
# endif

/* ------------------------------------------------------------------------- */

/* suppress some MSC warnings */
# if defined(_MSC_VER) && (1300 <= _MSC_VER)
#  ifndef _CRT_SECURE_NO_DEPRECATE
#   define _CRT_SECURE_NO_DEPRECATE
#  endif
# endif

/* ========================================================================= */

/*
 * Machine/DataModel specific
 * @todo: fixme
 */

	/*
	 * ARM
	 */

# ifdef __aarch64__
#  ifndef __arm64__
#   define __arm64__
#  endif
# endif
# ifdef __ARM_64BIT_STATE
#  ifndef __arm64__
#   define __arm64__
#  endif
# endif
# if (defined(WIN64) || defined(_WIN64)) && defined(_M_ARM)
#  ifndef __arm64__
#   define __arm64__
#  endif
# endif

# ifdef arm
#  ifndef __arm__
#   define __arm__
#  endif
# endif
# ifdef __arm
#  ifndef __arm__
#   define __arm__
#  endif
# endif
# ifdef __ARM_32BIT_STATE
#  ifndef __arm__
#   define __arm__
#  endif
# endif
# if (defined(WIN32) || defined(_WIN32)) && defined(_M_ARM)
#  ifndef __arm__
#   define __arm__
#  endif
# endif

# ifdef __arm64__
#  ifndef _A_ARM64
#   define _A_ARM64
#  endif
#  ifndef _A_ARM
#   ifdef __ARM_ARCH
#    define _A_ARM                  __ARM_ARCH
#   else
#    define _A_ARM                  8
#   endif
#  endif
# endif
# if defined(__arm__) && !defined(__arm64__)
#  ifndef _A_ARM32
#   define _A_ARM32
#  endif
#  ifndef _A_ARM
#   ifdef __ARM_ARCH
#    define _A_ARM                  __ARM_ARCH
#   else
#    define _A_ARM                  7
#   endif
#  endif
# endif


	/*
	 * X86-64
	 */

# if defined(_A_AMD64) || (defined(_MSC_VER) && defined(_M_X64)) || defined(__x86_64__)
#  ifndef _A_X86_64
#   define _A_X86_64
#  endif
#  ifndef _A_X64
#   define _A_X64
#  endif
# endif

	/*
	 * X86
	 */

# if defined(_A_I386) || (defined(_MSC_VER) && defined(_M_IX86)) || defined(__i386__) || defined(__x86__)
#  ifndef _A_X86
#   define _A_X86
#  endif
# endif


	/*
	 * DataModel
	 */

# if defined(_WIN64) || defined(WIN64)
#  ifndef __LLP64__
#   define __LLP64__
#  endif
# endif

# ifndef __LLP64__
#  if defined(_A_ARM64) || defined(_A_X86_64)
#   ifndef __LP64__
#    define __LP64__
#   endif
#  endif
# endif

/* ------------------------------------------------------------------------- */

/* @todo: fixme */

# ifndef BO_M_WORD
#  if defined(__LP64__) || defined(__LLP64__)
#   define BO_M_WORD                BO_M_WORD64
#  else
#   define BO_M_WORD                BO_M_WORD32
#  endif
# endif

# ifndef BO_SIZEOF_MWORD
#  if (BO_M_WORD <= BO_M_WORD32)
#   define BO_SIZEOF_MWORD          4
#  else
#  if (BO_M_WORD <= BO_M_WORD64)
#   define BO_SIZEOF_MWORD          8
#  else
#  endif
#  endif
# endif /* !defined(BO_SIZEOF_MWORD) */

# ifndef BO_WIDTHOF_MWORD
#  if (BO_M_WORD <= BO_M_WORD32)
#   define BO_WIDTHOF_MWORD         32
#  else
#  if (BO_M_WORD <= BO_M_WORD64)
#   define BO_WIDTHOF_MWORD         64
#  else
#  endif
#  endif
# endif /* !defined(BO_WIDTHOF_MWORD) */

# ifndef BO_MWORD_SIGNBIT
#  if (BO_M_WORD <= BO_M_WORD32)
#   define BO_MWORD_SIGNBIT         0x80000000
#  else
#  if (BO_M_WORD <= BO_M_WORD64)
#   define BO_MWORD_SIGNBIT         0x8000000000000000
#  else
#  endif
#  endif
# endif /* !defined(BO_MWORD_SIGNBIT) */

/* ------------------------------------------------------------------------- */

	/*
	 * Integer Division
	 */

# define VipmF_IDIV_FI8                 0x1
# define VipmF_IDIV_FI16                0x2
# define VipmF_IDIV_FI32                0x4


# ifdef _A_ARM
#  if (_A_ARM == 7)
#   define __VIPM_TM_IDIV               (VipmF_IDIV_FI8 | VipmF_IDIV_FI16)
#  endif
# endif

# ifndef __VIPM_TM_IDIV
#  define __VIPM_TM_IDIV                0
# endif

/* ------------------------------------------------------------------------- */

	/*
	 * Arm specific
	 */

# ifndef __VIPM_ARM_CACHE_LINE_SIZE
#  define __VIPM_ARM_CACHE_LINE_SIZE        64
# endif


	/*
	 * X86 specific
	 */

# ifndef __VIPM_X86_CACHE_LINE_SIZE
#  define __VIPM_X86_CACHE_LINE_SIZE        64
# endif


	/*
	 * X86_64 specific
	 */

# ifndef __VIPM_X86_64_CACHE_LINE_SIZE
#  define __VIPM_X86_64_CACHE_LINE_SIZE     64
# endif


	/*
	 * Generic
	 */

# ifndef __VIPM_CACHE_LINE_SIZE
#  ifdef _A_ARM
#   define __VIPM_CACHE_LINE_SIZE           __VIPM_ARM_CACHE_LINE_SIZE
#  else
#  ifdef _A_X86
#   define __VIPM_CACHE_LINE_SIZE           __VIPM_X86_CACHE_LINE_SIZE
#  else
#  ifdef _A_X86_64
#   define __VIPM_CACHE_LINE_SIZE           __VIPM_X86_64_CACHE_LINE_SIZE
#  else
#   define __VIPM_CACHE_LINE_SIZE           64
#  endif
#  endif
#  endif
# endif

/* ------------------------------------------------------------------------- */

	/*
	 * SIMD
	 */

# define VipmK_SIMD_NONE                0
# define VipmK_SIMD_MMX                 4
# define VipmK_SIMD_SSE2                5
# define VipmK_SIMD_SSE3                6
# define VipmK_SIMD_SSSE3               7
# define VipmK_SIMD_SSE4_1              8
# define VipmK_SIMD_SSE4_2              9
# define VipmK_SIMD_AVX                 10
# define VipmK_SIMD_AVX2                11
# define VipmK_SIMD_FMA3                12
# define VipmK_SIMD_AVX_512F            13
# define VipmK_SIMD_AVX_512BW           14
# define VipmK_SIMD_AVX_512CD           15
# define VipmK_SIMD_AVX_512DQ           16
# define VipmK_SIMD_AVX_512ER           17
# define VipmK_SIMD_AVX_512IFMA512      18
# define VipmK_SIMD_AVX_512PF           19
# define VipmK_SIMD_AVX_512VBMI         20
# define VipmK_SIMD_AVX_512VL           21
# define VipmK_SIMD_ARM_NEON            100


# ifdef __CUDACC__
#  ifndef __VIPM_NO_MMINTRIN_HEADER
#   define __VIPM_NO_MMINTRIN_HEADER
#  endif
# endif


	/*
	 * Arm Neon specific
	 */

/*
__ARM_FEATURE_UNALIGNED : -munaligned-access
__ARM_FEATURE_SIMD32
__ARM_NEON__            : -mfpu=neon // -marm
 */

# if defined(_A_ARM) && (defined(WIN32) || defined(_WIN32))

#  define __VIPM_ARM_NEON
#  define __VIPM_NEON_MMINTRIN_HEADER       <arm_neon.h>

#  ifndef __VIPM_SIMD
#   define __VIPM_SIMD                      VipmK_SIMD_ARM_NEON
#  endif
#  ifndef __VIPM_MD_MMINTRIN_HEADER
#   define __VIPM_MD_MMINTRIN_HEADER        __VIPM_NEON_MMINTRIN_HEADER
#  endif

# elif defined(__ARM_NEON__) || (defined(__ARM_NEON) && defined(_A_ARM64))

#  define __VIPM_ARM_NEON
#  define __VIPM_NEON_MMINTRIN_HEADER       <arm_neon.h>

#  ifndef __VIPM_SIMD
#   define __VIPM_SIMD                      VipmK_SIMD_ARM_NEON
#  endif
#  ifndef __VIPM_MD_MMINTRIN_HEADER
#   define __VIPM_MD_MMINTRIN_HEADER        __VIPM_NEON_MMINTRIN_HEADER
#  endif

# endif /* defined(_A_ARM) */

# if defined(_A_ARM) && \
        defined(_GCC_VER) && \
        !defined(__SOFTFP__) && \
        (defined(__ARM_PCS_VFP) || defined(__ARM_VFPV3__) || defined(__VFP_FP__) || \
         defined(__ARM_NEON__) || defined(__ARM_NEON))

#  ifndef __VIPM_ARM_VFP
#   define __VIPM_ARM_VFP
#  endif
#  ifndef __VIPM_VFP
#   define __VIPM_VFP
#  endif
# endif


	/*
	 * SSE(2,3,...) specific
	 */

# if defined(__SSE2__) || defined(_A_X86_64) || (defined(_M_IX86_FP) && 2 <= _M_IX86_FP)

#  define __VIPM_MMX
#  define __VIPM_SSE
#  define __VIPM_SSE2

#  define __VIPM_SSE_MMINTRIN_HEADER        <xmmintrin.h>
#  define __VIPM_SSE2_MMINTRIN_HEADER       <emmintrin.h>


/* @todo: fixme */
#  if defined(__AVX2__) || (defined(_MSC_VER) && 1800 <= _MSC_VER && 0)
#   ifndef __VIPM_AVX
#    define __VIPM_AVX /* @todo: fixme - ? */
#   endif
#   ifndef __VIPM_AVX2
#    define __VIPM_AVX2
#   endif
#   if defined(__FMA__) && !defined(__VIPM_FMA3)
#    define __VIPM_FMA3
#   endif
#   define __VIPM_AVX2_MMINTRIN_HEADER      <immintrin.h>

#   ifndef __VIPM_SIMD
#    define __VIPM_SIMD                     VipmK_SIMD_AVX2
#   endif
#   ifndef __VIPM_MD_MMINTRIN_HEADER
#    define __VIPM_MD_MMINTRIN_HEADER       __VIPM_AVX2_MMINTRIN_HEADER
#   endif
#  endif /* avx2 */


/* @todo: fixme */
#  if defined(__AVX__) || (defined(_MSC_VER) && 1600 <= _MSC_VER && 0)
#   ifndef __VIPM_AVX
#    define __VIPM_AVX
#   endif
#   define __VIPM_AVX_MMINTRIN_HEADER       <immintrin.h>

#   ifndef __VIPM_SIMD
#    define __VIPM_SIMD                     VipmK_SIMD_AVX
#   endif
#   ifndef __VIPM_MD_MMINTRIN_HEADER
#    define __VIPM_MD_MMINTRIN_HEADER       __VIPM_AVX_MMINTRIN_HEADER
#   endif

#   if defined(_XCR_XFEATURE_ENABLED_MASK)
#    define __xgetbv()                      _xgetbv(_XCR_XFEATURE_ENABLED_MASK)
#   else
#    define __xgetbv()                      0
#   endif
#  endif /* avx */


#  if defined(__SSE4_2__) || (defined(_MSC_VER) && 1500 <= _MSC_VER)
#   ifndef __VIPM_SSE4
#    define __VIPM_SSE4
#   endif
#   define __VIPM_SSE4_2
#   define __VIPM_SSE4_2_MMINTRIN_HEADER    <nmmintrin.h>

#   ifndef __VIPM_SIMD
#    define __VIPM_SIMD                     VipmK_SIMD_SSE4_2
#   endif
#   ifndef __VIPM_MD_MMINTRIN_HEADER
#    define __VIPM_MD_MMINTRIN_HEADER       __VIPM_SSE4_2_MMINTRIN_HEADER
#   endif
#  endif /* sse4.2 */


#  if defined(__SSE4_1__) || (defined(_MSC_VER) && 1500 <= _MSC_VER)
#   ifndef __VIPM_SSE4
#    define __VIPM_SSE4
#   endif
#   define __VIPM_SSE4_1
#   define __VIPM_SSE4_1_MMINTRIN_HEADER    <smmintrin.h>

#   ifndef __VIPM_SIMD
#    define __VIPM_SIMD                     VipmK_SIMD_SSE4_1
#   endif
#   ifndef __VIPM_MD_MMINTRIN_HEADER
#    define __VIPM_MD_MMINTRIN_HEADER       __VIPM_SSE4_1_MMINTRIN_HEADER
#   endif
#  endif /* sse4.1 */


#  if defined(__SSSE3__)  || (defined(_MSC_VER) && 1500 <= _MSC_VER)
#   define __VIPM_SSSE3
#   define __VIPM_SSSE3_MMINTRIN_HEADER     <tmmintrin.h>

#   ifndef __VIPM_SIMD
#    define __VIPM_SIMD                     VipmK_SIMD_SSSE3
#   endif
#   ifndef __VIPM_MD_MMINTRIN_HEADER
#    define __VIPM_MD_MMINTRIN_HEADER       __VIPM_SSSE3_MMINTRIN_HEADER
#   endif
#  endif /* ssse3 */


#  if defined(__SSE3__) || (defined(_MSC_VER) && 1500 <= _MSC_VER)
#   define __VIPM_SSE3
#   define __VIPM_SSE3_MMINTRIN_HEADER      <pmmintrin.h>

#   ifndef __VIPM_SIMD
#    define __VIPM_SIMD                     VipmK_SIMD_SSE3
#   endif
#   ifndef __VIPM_MD_MMINTRIN_HEADER
#    define __VIPM_MD_MMINTRIN_HEADER       __VIPM_SSE3_MMINTRIN_HEADER
#   endif
#  endif /* sse3 */


/*
 * defaults
 */
#  ifndef __VIPM_SIMD
#   define __VIPM_SIMD                      VipmK_SIMD_SSE2
#  endif
#  ifndef __VIPM_MD_MMINTRIN_HEADER
#   define __VIPM_MD_MMINTRIN_HEADER        __VIPM_SSE2_MMINTRIN_HEADER
#  endif

# endif /* defined(__SSE2__) || defined(_A_X86_64) || (defined(_M_IX86_FP) && 2 <= _M_IX86_FP) */


# ifndef __VIPM_SIMD
#  define __VIPM_SIMD                       0
# endif


/* ========================================================================= */

/*
 * Conditional macros
 * @todo: fixme
 */

# ifndef __VIPM_USE_SIMD
#  ifdef _DEBUG
#   define __VIPM_USE_SIMD                  0
#  else
#   define __VIPM_USE_SIMD                  __VIPM_SIMD
#  endif
# endif

// @todo: fixme - implement simd sse
# ifndef __VIPM_ARM_NEON
#  undef __VIPM_USE_SIMD
# endif

# ifndef __VIPM_USE_SIMD
#  define __VIPM_USE_SIMD                   0
# endif

/* ------------------------------------------------------------------------- */

# ifndef __VIPM_UNALIGNED_ACCESS
#  if defined(__ARM_FEATURE_UNALIGNED)
#   define __VIPM_UNALIGNED_ACCESS
#  else
#  ifdef _A_X86
#   define __VIPM_UNALIGNED_ACCESS
#  else
#  ifdef _A_X86_64
#   define __VIPM_UNALIGNED_ACCESS
#  else
#  endif
#  endif
#  endif
# endif


// @todo: fixme - was _VIPM_SIMD_UNALIGNED_ACCESS
# ifndef __VIPM_SIMD_UNALIGNED_LOAD_STORE
#  if defined(__VIPM_ARM_NEON)
#   define __VIPM_SIMD_UNALIGNED_LOAD_STORE
#  else
#  if defined(__VIPM_AVX) || defined(__VIPM_AVX2)
#   define __VIPM_SIMD_UNALIGNED_LOAD_STORE
#  else
#  if defined(__VIPM_SSE2)
#   define __VIPM_SIMD_UNALIGNED_LOAD_STORE
#  else
/* #   define __VIPM_SIMD_UNALIGNED_LOAD_STORE */
#  endif
#  endif
#  endif
# endif /* !defined(__VIPM_SIMD_UNALIGNED_LOAD_STORE) */

# ifndef __VIPM_SIMD_FAVOR_ALIGNED_LOAD_STORE
#  if defined(__VIPM_ARM_NEON)
/* #   define __VIPM_SIMD_FAVOR_ALIGNED_LOAD_STORE */
#  else
#  if defined(__VIPM_AVX) || defined(__VIPM_AVX2)
#   define __VIPM_SIMD_FAVOR_ALIGNED_LOAD_STORE
#  else
#  if defined(__VIPM_SSE2)
#   define __VIPM_SIMD_FAVOR_ALIGNED_LOAD_STORE
#  else
#   define __VIPM_SIMD_FAVOR_ALIGNED_LOAD_STORE
#  endif
#  endif
#  endif
# endif /* !defined(__VIPM_SIMD_FAVOR_ALIGNED_LOAD_STORE) */


/* ========================================================================= */

	/*
	 * Arm Neon specific
	 */

/* @todo: fixme */
# ifndef __VIPM_ARM_NEON_OPT_LINESIZ_MIN
#  define __VIPM_ARM_NEON_OPT_LINESIZ_MIN   8
# endif
# ifndef __VIPM_ARM_NEON_PINT_PADDING
#  define __VIPM_ARM_NEON_PINT_PADDING      8
# endif


	/*
	 * SSE2 specific
	 */

/* @todo: fixme */
# ifndef __VIPM_SSE2_OPT_LINESIZ_MIN
#  define __VIPM_SSE2_OPT_LINESIZ_MIN       16
# endif
# ifndef __VIPM_SSE2_PINT_PADDING
#  define __VIPM_SSE2_PINT_PADDING          16
# endif


	/*
	 * Simd
	 */

# ifdef __VIPM_ARM_NEON
#  define __VIPM_SIMD_OPT_LINESIZ_MIN       __VIPM_ARM_NEON_OPT_LINESIZ_MIN
#  define __VIPM_SIMD_PINT_PADDING          __VIPM_ARM_NEON_PINT_PADDING
# else
# ifdef __VIPM_SSE2
#  define __VIPM_SIMD_OPT_LINESIZ_MIN       __VIPM_SSE2_OPT_LINESIZ_MIN
#  define __VIPM_SIMD_PINT_PADDING          __VIPM_SSE2_PINT_PADDING
# else
# endif
# endif

# ifndef __VIPM_SIMD_OPT_LINESIZ_MIN
#  define __VIPM_SIMD_OPT_LINESIZ_MIN       0
# endif
# ifndef __VIPM_SIMD_PINT_PADDING
#  define __VIPM_SIMD_PINT_PADDING          0
# endif


	/*
	 * Generic
	 */

/* @todo: fixme */
# ifndef __VIPM_OPT_LINESIZ_MIN
#  ifdef __VIPM_UNALIGNED_ACCESS
#   define __VIPM_OPT_LINESIZ_MIN           8
#  else
#  if (8 <= BO_SIZEOF_MWORD)
#   define __VIPM_OPT_LINESIZ_MIN           40
#  else
#   define __VIPM_OPT_LINESIZ_MIN           36
#  endif
#  endif
# endif

/*
# define __VIPM_FAVOR_DEREF_ADV
 */

# ifndef __VIPM_BASE_ALIGN
#  define __VIPM_BASE_ALIGN                 16 /* 1 */
# endif


/* ========================================================================= */

# if 0 && defined(__myc_xconcat)
#  define VIPM__GLUE                    __myc_concat
#  define VIPM__GLUE3                   __myc_concat3
#  define VIPM__GLUE4                   __myc_concat4
#  define VIPM__GLUE5                   __myc_concat5
#  define VIPM_GLUE                     __myc_xconcat
#  define VIPM_GLUE3                    __myc_xconcat3
#  define VIPM_GLUE4                    __myc_xconcat4
#  define VIPM_GLUE5                    __myc_xconcat5
# else /* !defined(__myc_xconcat) */
#  define VIPM__GLUE(a,b)               a ## b
#  define VIPM__GLUE3(a,b,c)            a ## b ## c
#  define VIPM__GLUE4(a,b,c,d)          a ## b ## c ## d
#  define VIPM__GLUE5(a,b,c,d,e)        a ## b ## c ## d ## e
#  define VIPM_GLUE(a,b)                VIPM__GLUE(a,b)
#  define VIPM_GLUE3(a,b,c)             VIPM__GLUE3(a,b,c)
#  define VIPM_GLUE4(a,b,c,d)           VIPM__GLUE4(a,b,c,d)
#  define VIPM_GLUE5(a,b,c,d,e)         VIPM__GLUE5(a,b,c,d,e)
# endif /* defined(__myc_xconcat) */


# if 0 && defined(__myc_xstringify)
#  define VIPM__STR_C                   __myc_stringify
#  define VIPM_STR_C                    __myc_xstringify
# else /* !defined(__myc_xstringify) */
#  define VIPM__STR_C(a)                #a
#  define VIPM_STR_C(a)                 VIPM__STR_C(a)
# endif /* defined(__myc_xstringify) */


# define VIPM_IDENTITY(x)               x
# define VIPM_ID(x)                     x
# define VIPM_EVAL(x)                   x
# ifdef _MSC_VER
#  define VIPM_UNPAREN(P)               VIPM_EVAL(__VIPM__UNPAREN ## P)
# else
#  define VIPM_UNPAREN(P)               VIPM_EVAL(__VIPM__UNPAREN P)
# endif
/* @{ */
# define __VIPM__UNPAREN(...)           __VA_ARGS__
/* @} */


# define VIPM_PP_GENERIC2(_1,_2, _Name, ...)            _Name
# define VIPM_PP_GENERIC3(_1,_2,_3, _Name, ...)         _Name
# define VIPM_PP_GENERIC4(_1,_2,_3,_4, _Name, ...)      _Name
# define VIPM_PP_GENERIC5(_1,_2,_3,_4,_5, _Name, ...)   _Name


/* @todo: fixme - move to ? */
/* @sa: "Vipm/_Types/Imel_Types.h" */
# ifndef VIPM_PP_W2SZ
#  define VIPM_PP_W2SZ(W)               VIPM_GLUE(VIPM_PP__W2SZ_,W)()
/* @{ */
#  define VIPM_PP__W2SZ_1()             0
#  define VIPM_PP__W2SZ_2()             0
#  define VIPM_PP__W2SZ_4()             0
#  define VIPM_PP__W2SZ_8()             1
#  define VIPM_PP__W2SZ_16()            2
#  define VIPM_PP__W2SZ_32()            4
#  define VIPM_PP__W2SZ_64()            8
#  define VIPM_PP__W2SZ_128()           16

#  define VIPM_PP__W2SZ_I8()            1
#  define VIPM_PP__W2SZ_I16()           2
#  define VIPM_PP__W2SZ_I32()           4
#  define VIPM_PP__W2SZ_I64()           8
#  define VIPM_PP__W2SZ_I128()          16
#  define VIPM_PP__W2SZ_S8()            1
#  define VIPM_PP__W2SZ_S16()           2
#  define VIPM_PP__W2SZ_S32()           4
#  define VIPM_PP__W2SZ_S64()           8
#  define VIPM_PP__W2SZ_S128()          16
#  define VIPM_PP__W2SZ_U8()            1
#  define VIPM_PP__W2SZ_U16()           2
#  define VIPM_PP__W2SZ_U32()           4
#  define VIPM_PP__W2SZ_U64()           8
#  define VIPM_PP__W2SZ_U128()          16
#  define VIPM_PP__W2SZ_F16()           2
#  define VIPM_PP__W2SZ_F32()           4
#  define VIPM_PP__W2SZ_F64()           8
#  define VIPM_PP__W2SZ_F128()          16
/* @} */


#  define VIPM_PP_WDIV2(W)              VIPM_GLUE(VIPM_PP__WDIV2_,W)()
/* @{ */
#  define VIPM_PP__WDIV2_0()            0
#  define VIPM_PP__WDIV2_1()            0
#  define VIPM_PP__WDIV2_2()            1
#  define VIPM_PP__WDIV2_4()            2
#  define VIPM_PP__WDIV2_8()            4
#  define VIPM_PP__WDIV2_16()           8
#  define VIPM_PP__WDIV2_32()           16
#  define VIPM_PP__WDIV2_64()           32
#  define VIPM_PP__WDIV2_128()          64
#  define VIPM_PP__WDIV2_256()          128

//#  define VIPM_PP__WDIV2_I8()
#  define VIPM_PP__WDIV2_I16()          I8
#  define VIPM_PP__WDIV2_I32()          I16
#  define VIPM_PP__WDIV2_I64()          I32
#  define VIPM_PP__WDIV2_I128()         I64
//#  define VIPM_PP__WDIV2_S8()
#  define VIPM_PP__WDIV2_S16()          S8
#  define VIPM_PP__WDIV2_S32()          S16
#  define VIPM_PP__WDIV2_S64()          S32
#  define VIPM_PP__WDIV2_S128()         S64
//#  define VIPM_PP__WDIV2_U8()
#  define VIPM_PP__WDIV2_U16()          U8
#  define VIPM_PP__WDIV2_U32()          U16
#  define VIPM_PP__WDIV2_U64()          U32
#  define VIPM_PP__WDIV2_U128()         U64
//#  define VIPM_PP__WDIV2_F16()
#  define VIPM_PP__WDIV2_F32()          F16
#  define VIPM_PP__WDIV2_F64()          F32
#  define VIPM_PP__WDIV2_F128()         F64
/* @} */


#  define VIPM_PP_WMUL2(W)              VIPM_GLUE(VIPM_PP__WMUL2_,W)()
/* @{ */
#  define VIPM_PP__WMUL2_0()            0
#  define VIPM_PP__WMUL2_1()            2
#  define VIPM_PP__WMUL2_2()            4
#  define VIPM_PP__WMUL2_4()            8
#  define VIPM_PP__WMUL2_8()            16
#  define VIPM_PP__WMUL2_16()           32
#  define VIPM_PP__WMUL2_32()           64
#  define VIPM_PP__WMUL2_64()           128
#  define VIPM_PP__WMUL2_128()          256
#  define VIPM_PP__WMUL2_256()          512

#  define VIPM_PP__WMUL2_I8()           I16
#  define VIPM_PP__WMUL2_I16()          I32
#  define VIPM_PP__WMUL2_I32()          I64
#  define VIPM_PP__WMUL2_I64()          I128
#  define VIPM_PP__WMUL2_I128()         I256
#  define VIPM_PP__WMUL2_S8()           S16
#  define VIPM_PP__WMUL2_S16()          S32
#  define VIPM_PP__WMUL2_S32()          S64
#  define VIPM_PP__WMUL2_S64()          S128
#  define VIPM_PP__WMUL2_S128()         S256
#  define VIPM_PP__WMUL2_U8()           U16
#  define VIPM_PP__WMUL2_U16()          U32
#  define VIPM_PP__WMUL2_U32()          U64
#  define VIPM_PP__WMUL2_U64()          U128
#  define VIPM_PP__WMUL2_U128()         U256
//#  define VIPM_PP__WMUL2_F8()           F16
#  define VIPM_PP__WMUL2_F16()          F32
#  define VIPM_PP__WMUL2_F32()          F64
#  define VIPM_PP__WMUL2_F64()          F128
#  define VIPM_PP__WMUL2_F128()         F256
/* @} */


#  define VIPM_PP_WMUL4(W)              VIPM_GLUE(VIPM_PP__WMUL4_,W)()
/* @{ */
#  define VIPM_PP__WMUL4_0()            0
#  define VIPM_PP__WMUL4_1()            4
#  define VIPM_PP__WMUL4_2()            8
#  define VIPM_PP__WMUL4_4()            16
#  define VIPM_PP__WMUL4_8()            32
#  define VIPM_PP__WMUL4_16()           64
#  define VIPM_PP__WMUL4_32()           128
#  define VIPM_PP__WMUL4_64()           256
#  define VIPM_PP__WMUL4_128()          512
#  define VIPM_PP__WMUL4_256()          1024

#  define VIPM_PP__WMUL4_I8()           I32
#  define VIPM_PP__WMUL4_I16()          I64
#  define VIPM_PP__WMUL4_I32()          I128
#  define VIPM_PP__WMUL4_I64()          I256
#  define VIPM_PP__WMUL4_I128()         I512
#  define VIPM_PP__WMUL4_S8()           S32
#  define VIPM_PP__WMUL4_S16()          S64
#  define VIPM_PP__WMUL4_S32()          S128
#  define VIPM_PP__WMUL4_S64()          S256
#  define VIPM_PP__WMUL4_S128()         S512
#  define VIPM_PP__WMUL4_U8()           U32
#  define VIPM_PP__WMUL4_U16()          U64
#  define VIPM_PP__WMUL4_U32()          U128
#  define VIPM_PP__WMUL4_U64()          U256
#  define VIPM_PP__WMUL4_U128()         U512
//#  define VIPM_PP__WMUL4_F8()           F32
#  define VIPM_PP__WMUL4_F16()          F64
#  define VIPM_PP__WMUL4_F32()          F128
#  define VIPM_PP__WMUL4_F64()          F256
#  define VIPM_PP__WMUL4_F128()         F512
/* @} */


#  define VIPM_PP_WMUL8(W)              VIPM_GLUE(VIPM_PP__WMUL8_,W)()
/* @{ */
#  define VIPM_PP__WMUL8_0()            0
#  define VIPM_PP__WMUL8_1()            8
#  define VIPM_PP__WMUL8_2()            16
#  define VIPM_PP__WMUL8_4()            32
#  define VIPM_PP__WMUL8_8()            64
#  define VIPM_PP__WMUL8_16()           128
#  define VIPM_PP__WMUL8_32()           256
#  define VIPM_PP__WMUL8_64()           512
#  define VIPM_PP__WMUL8_128()          1024
#  define VIPM_PP__WMUL8_256()          2048

#  define VIPM_PP__WMUL8_I8()           I64
#  define VIPM_PP__WMUL8_I16()          I128
#  define VIPM_PP__WMUL8_I32()          I256
#  define VIPM_PP__WMUL8_I64()          I512
#  define VIPM_PP__WMUL8_I128()         I1024
#  define VIPM_PP__WMUL8_S8()           S64
#  define VIPM_PP__WMUL8_S16()          S128
#  define VIPM_PP__WMUL8_S32()          S256
#  define VIPM_PP__WMUL8_S64()          S512
#  define VIPM_PP__WMUL8_S128()         S1024
#  define VIPM_PP__WMUL8_U8()           U64
#  define VIPM_PP__WMUL8_U16()          U128
#  define VIPM_PP__WMUL8_U32()          U256
#  define VIPM_PP__WMUL8_U64()          U512
#  define VIPM_PP__WMUL8_U128()         U1024
//#  define VIPM_PP__WMUL8_F8()           F64
#  define VIPM_PP__WMUL8_F16()          F128
#  define VIPM_PP__WMUL8_F32()          F256
#  define VIPM_PP__WMUL8_F64()          F512
#  define VIPM_PP__WMUL8_F128()         F1024
/* @} */
# endif /* !defined(VIPM_PP_W2SZ) */


/* ========================================================================= */

/*
 * Pseudo-keywords, attributes, ...
 */


# ifndef __P
#  if __myc_require(prototypes)
#   define __P(args) args
#  else
#   define __P(args) ()
#  endif
# endif



# ifdef _VIPM_BUILD
#  ifndef _VIPM_EXTERN
#   if defined(_MSC_VER) || (defined WIN32 || defined _WIN32)
#    define _VIPM_EXTERN                extern __declspec(dllexport)
#   else
#   ifdef _GCC_VER
#    define _VIPM_EXTERN                extern __attribute__((__visibility__("default")))
#   else
#    define _VIPM_EXTERN                extern
#   endif
#   endif
#  endif
# endif
# ifndef _VIPM_EXTERN
#  if defined(_MSC_VER) || (defined WIN32 || defined _WIN32)
#   define _VIPM_EXTERN                 extern __declspec(dllimport)
#  else
#  ifdef _GCC_VER
#   define _VIPM_EXTERN                 extern __attribute__((__visibility__("default")))
#  else
#   define _VIPM_EXTERN                 extern
#  endif
#  endif
# endif

/* @todo: fixme */
# ifndef _VIPM_PRIVATE_EXTERN
#  ifdef _MSC_VER
#   define _VIPM_PRIVATE_EXTERN
#  else
#  ifdef _GCC_VER
#   define _VIPM_PRIVATE_EXTERN         __attribute__((__visibility__("hidden")))
#  else
#   define _VIPM_PRIVATE_EXTERN
#  endif
#  endif
# endif


/* @todo: fixme */
# ifndef _VIPM_INLINE
#  ifdef __cplusplus
#   define _VIPM_INLINE                 inline
#  else
#  ifdef _MSC_VER
#   define _VIPM_INLINE                 __forceinline
#  else
#  ifdef _GCC_VER
#   define _VIPM_INLINE                 extern __inline__ __attribute__((__gnu_inline__, __always_inline__, __artificial__))
#  else
#  if defined(__STDC_ISO_10646__) && (201103L <= __STDC_ISO_10646__)
#   define _VIPM_INLINE                 inline __attribute__((__always_inline__))
#  else
#   define _VIPM_INLINE                 static /* inline */
#  endif
#  endif
#  endif
#  endif
# endif


/* @todo: fixme */
# ifndef _VIPM_TRANSPARENT_UNION
#  ifdef _MSC_VER
#   define _VIPM_TRANSPARENT_UNION
#  else
#  ifdef _GCC_VER
#   define _VIPM_TRANSPARENT_UNION      __attribute__((__transparent_union__))
#  else
#   define _VIPM_TRANSPARENT_UNION
#  endif
#  endif
# endif

# ifndef _VIPM_ALIGNAS
#  ifdef _MSC_VER
//#   define _VIPM_ALIGNAS(N)             __declspec((__align(N)))
#   define _VIPM_ALIGNAS(N)             __declspec(align(N))
#  else
#  ifdef _GCC_VER
#   define _VIPM_ALIGNAS(N)             __attribute__((__aligned__(N)))
#  else
#   define _VIPM_ALIGNAS(N)
#  endif
#  endif
# endif

# ifndef _VIPM_UNALIGNED
#  if defined(_MSC_VER) || (defined WIN32 || defined _WIN32)
#   define _VIPM_UNALIGNED              /* @todo: fixme */
#  else
#  ifdef _GCC_VER
/* @todo: fixme */
#   if defined(_A_ARM)
#    define _VIPM_UNALIGNED             __volatile__ __attribute__((__aligned__(1)))
#   else
#    define _VIPM_UNALIGNED             /* __attribute__((__aligned__(1))) */
#   endif
#  else
#   define _VIPM_UNALIGNED              /* @todo: fixme */
#  endif
#  endif
# endif


# ifndef _VIPM_C89_BLOCK_BACK
#  if defined(_MSC_VER) && (_MSC_VER <= 1700)
#   define _VIPM_C89_BLOCK_BACK         {
#  else
#   define _VIPM_C89_BLOCK_BACK
#  endif
# endif

# ifndef _VIPM_C89_BLOCK_END
#  if defined(_MSC_VER) && (_MSC_VER <= 1700)
#   define _VIPM_C89_BLOCK_END          }
#  else
#   define _VIPM_C89_BLOCK_END
#  endif
# endif


/* ========================================================================= */

/*
 * "Devel" macros
 */

	/*
	 * Name making macros
	 *
	 * (constructors, ... )
	 */

# define VIPM_CTOR_FCNAME1(_Name)               VIPM_GLUE3(__, _Name, __$)
# define VIPM_CTOR_FCNAME2(_Name,_Sign)         VIPM_GLUE3(VIPM_GLUE3(__, _Name, __), $,_Sign)
# define VIPM_CTOR_FCNAME3(_Name,_S,_S2)        VIPM_GLUE(VIPM_GLUE3(__, _Name, __), VIPM_GLUE4($,_S,$,_S2))
# define VIPM_CTOR_FCNAME4(_Name,_S,_S2,_S3)    VIPM_GLUE(VIPM_GLUE3(__, _Name, __), VIPM_GLUE3(VIPM_GLUE4($,_S,$,_S2),$,_S3))
# ifdef _MSC_VER
//#  define VIPM_CTOR_FCNAME(...)                 VIPM_EVAL(VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_CTOR_FCNAME4, VIPM_CTOR_FCNAME3, VIPM_CTOR_FCNAME2, VIPM_CTOR_FCNAME1, @)(__VA_ARGS__))
#  define VIPM_CTOR_FCNAME(...)                 VIPM_EVAL(VIPM_EVAL(VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_CTOR_FCNAME4, VIPM_CTOR_FCNAME3, VIPM_CTOR_FCNAME2, VIPM_CTOR_FCNAME1, @))(__VA_ARGS__))
# else
#  define VIPM_CTOR_FCNAME(...)                 VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_CTOR_FCNAME4, VIPM_CTOR_FCNAME3, VIPM_CTOR_FCNAME2, VIPM_CTOR_FCNAME1, @)(__VA_ARGS__)
# endif

/*
 * VIPM_RAW_FCNAME(Foo)
 * VIPM_RAW_FCNAME(Foo,LS)
 */
# define VIPM_RAW_FCNAME1(_Name)                _Name
# define VIPM_RAW_FCNAME2(_Name,_Sign)          VIPM_GLUE3(_Name, _$, _Sign)
# define VIPM_RAW_FCNAME3(_Nm,_S,_S2)           VIPM_GLUE(_Nm, VIPM_GLUE4(_$,_S,$,_S2))
# define VIPM_RAW_FCNAME4(_Nm,_S,_S2,_S3)       VIPM_GLUE(_Nm, VIPM_GLUE3(VIPM_GLUE4(_$,_S,$,_S2),$,_S3))
# ifdef _MSC_VER
//#  define VIPM_RAW_FCNAME(...)                  VIPM_EVAL(VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_RAW_FCNAME4, VIPM_RAW_FCNAME3, VIPM_RAW_FCNAME2, VIPM_RAW_FCNAME1, @)(__VA_ARGS__))
#  define VIPM_RAW_FCNAME(...)                  VIPM_EVAL(VIPM_EVAL(VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_RAW_FCNAME4, VIPM_RAW_FCNAME3, VIPM_RAW_FCNAME2, VIPM_RAW_FCNAME1, @))(__VA_ARGS__))
# else
#  define VIPM_RAW_FCNAME(...)                  VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_RAW_FCNAME4, VIPM_RAW_FCNAME3, VIPM_RAW_FCNAME2, VIPM_RAW_FCNAME1, @)(__VA_ARGS__)
# endif

/*
 * VIPM_PFX_FCNAME(thr,Foo)
 * VIPM_PFX_FCNAME(thr,Foo,LS)
 */
# define VIPM_PFX_FCNAME1(_Pfx,_Nm)             VIPM_GLUE3(_Pfx, _, _Nm)
# define VIPM_PFX_FCNAME2(_Pfx,_Nm,_Sign)       VIPM_GLUE3(VIPM_PFX_FCNAME1(_Pfx,_Nm), _$,_Sign)
# define VIPM_PFX_FCNAME3(_Pfx,_Nm,_S,_S2)      VIPM_GLUE(VIPM_PFX_FCNAME1(_Pfx,_Nm), VIPM_GLUE4(_$,_S,$,_S2))
# define VIPM_PFX_FCNAME4(_Pfx,_Nm,_S,_S2,_S3)  VIPM_GLUE(VIPM_PFX_FCNAME1(_Pfx,_Nm), VIPM_GLUE3(VIPM_GLUE4(_$,_S,$,_S2),$,_S3))
# ifdef _MSC_VER
//#  define VIPM_PFX_FCNAME(_Pfx,...)             VIPM_EVAL(VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_PFX_FCNAME4, VIPM_PFX_FCNAME3, VIPM_PFX_FCNAME2, VIPM_PFX_FCNAME1, VIPM_RAW_FCNAME1, @)(__VA_ARGS__))
#  define VIPM_PFX_FCNAME(_Pfx,...)             VIPM_EVAL(VIPM_EVAL(VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_PFX_FCNAME4, VIPM_PFX_FCNAME3, VIPM_PFX_FCNAME2, VIPM_PFX_FCNAME1, VIPM_RAW_FCNAME1, @))(_Pfx, __VA_ARGS__))
# else
#  define VIPM_PFX_FCNAME(_Pfx,...)             VIPM_PP_GENERIC4(__VA_ARGS__, VIPM_PFX_FCNAME4, VIPM_PFX_FCNAME3, VIPM_PFX_FCNAME2, VIPM_PFX_FCNAME1, VIPM_RAW_FCNAME1, @)(_Pfx, __VA_ARGS__)
# endif


/*
 * Well known names
 */

# define vipm_NS(...)                   VIPM_PFX_FCNAME(vipm, __VA_ARGS__)
# define vipm_2d_NS(...)                VIPM_PFX_FCNAME(vipm_2d, __VA_ARGS__)
# define vipm_f2d_NS(...)               VIPM_PFX_FCNAME(vipm_f2d, __VA_ARGS__)
# define vipm_r2d_NS(...)               VIPM_PFX_FCNAME(vipm_r2d, __VA_ARGS__)
# define vipm_XNS(_Space)               VIPM_GLUE3(vipm_,_Space,_NS)

# define vipmi_NS(...)                  VIPM_PFX_FCNAME(vipmi, __VA_ARGS__)
# define vipmi_2d_NS(...)               VIPM_PFX_FCNAME(vipmi_2d, __VA_ARGS__)
# define vipmi_f2d_NS(...)              VIPM_PFX_FCNAME(vipmi_f2d, __VA_ARGS__)
# define vipmi_r2d_NS(...)              VIPM_PFX_FCNAME(vipmi_r2d, __VA_ARGS__)
# define vipmi_XNS(_Space)              VIPM_GLUE3(vipmi_,_Space,_NS)


# define vipm_math_(...)                VIPM_PFX_FCNAME(vipm_math, __VA_ARGS__)
# define vipm_geom_(...)                VIPM_PFX_FCNAME(vipm_geom, __VA_ARGS__)
# define vipm_vec_(...)                 VIPM_PFX_FCNAME(vipm_vec, __VA_ARGS__)
# define vipm_volume_(...)              VIPM_PFX_FCNAME(vipm_volume, __VA_ARGS__)
# define vipm_base_(...)                VIPM_PFX_FCNAME(vipm_base, __VA_ARGS__)


# define vipm_poly_(...)                VIPM_PFX_FCNAME(vipm_poly, __VA_ARGS__)
# define vipm_2d_poly_(...)             VIPM_PFX_FCNAME(vipm_2d_poly, __VA_ARGS__)
# define vipm_f2d_poly_(...)            VIPM_PFX_FCNAME(vipm_f2d_poly, __VA_ARGS__)
# define vipm_r2d_poly_(...)            VIPM_PFX_FCNAME(vipm_r2d_poly, __VA_ARGS__)
# define vipm_Poly_(_Space)             VIPM_GLUE3(vipm_,_Space,_poly_)

# define vipm_ptv_(...)                 VIPM_PFX_FCNAME(vipm_ptv, __VA_ARGS__)
# define vipm_2d_ptv_(...)              VIPM_PFX_FCNAME(vipm_2d_ptv, __VA_ARGS__)
# define vipm_f2d_ptv_(...)             VIPM_PFX_FCNAME(vipm_f2d_ptv, __VA_ARGS__)
# define vipm_r2d_ptv_(...)             VIPM_PFX_FCNAME(vipm_r2d_ptv, __VA_ARGS__)
# define vipm_Ptv_(_Space)              VIPM_GLUE3(vipm_,_Space,_ptv_)

/* ------------------------------------------------------------------------- */

/*
 * "Impl" macros
 */

# define VIPM_FCNAME_TY(_BaseName,_Ty) \
	VIPM_GLUE3(_BaseName,_,VIPM_ELTY_LTW(_Ty))

# define VIPM_FCNAME_TY2(_BaseName,_Ty1,_Ty2) \
	VIPM_GLUE3(_BaseName,_,VIPM_GLUE(VIPM_ELTY_LTW(_Ty1),VIPM_ELTY_LTW(_Ty2)))

# define VIPM_FCNAME_TY3(_BaseName,_Ty1,_Ty2,_Ty3)  \
	VIPM_GLUE3(_BaseName,_,VIPM_GLUE3(VIPM_ELTY_LTW(_Ty1),VIPM_ELTY_LTW(_Ty2),VIPM_ELTY_LTW(_Ty3)))

# define VIPM_FCNAME_TY4(_BaseName,_Ty1,_Ty2,_Ty3,_Ty4)  \
	VIPM_GLUE3(_BaseName,_,VIPM_GLUE4(VIPM_ELTY_LTW(_Ty1),VIPM_ELTY_LTW(_Ty2),VIPM_ELTY_LTW(_Ty3),VIPM_ELTY_LTW(_Ty4)))


# define VIPM_FCNAME_CNTY(_BaseName,_CnNA,_Alpha,_Ty) \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_,VIPM_ELTY_LTW(_Ty))

# define VIPM_FCNAME_CNTY2(_BaseName,_CnNA,_Alpha,_Ty1,_Ty2) \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_,VIPM_GLUE(VIPM_ELTY_LTW(_Ty1),VIPM_ELTY_LTW(_Ty2)))

# define VIPM_FCNAME_CNTY3(_BaseName,_CnNA,_Alpha,_Ty1,_Ty2,_Ty3)  \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_, \
		VIPM_GLUE3(VIPM_ELTY_LTW(_Ty1),VIPM_ELTY_LTW(_Ty2),VIPM_ELTY_LTW(_Ty3)))

# define VIPM_FCNAME_CNTY4(_BaseName,_CnNA,_Alpha,_Ty1,_Ty2,_Ty3,_Ty4)  \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_,      \
		VIPM_GLUE4(VIPM_ELTY_LTW(_Ty1),VIPM_ELTY_LTW(_Ty2),VIPM_ELTY_LTW(_Ty3),VIPM_ELTY_LTW(_Ty4)))


# define VIPM_FCNAME_WI(_BaseName,_W)               VIPM_GLUE3(_BaseName,_,_W)
# define VIPM_FCNAME_WI2(_BaseName,_W1,_W2)         VIPM_GLUE3(_BaseName,_,VIPM_GLUE(_W1,_W2))
# define VIPM_FCNAME_WI3(_BaseName,_W1,_W2,_W3)     VIPM_GLUE3(_BaseName,_,VIPM_GLUE3(_W1,_W2,_W3))
# define VIPM_FCNAME_WI4(_BaseName,_W1,_W2,_W3,_W4) VIPM_GLUE3(_BaseName,_,VIPM_GLUE4(_W1,_W2,_W3,_W4))


# define VIPM_FCNAME_CN_WI(_BaseName,_CnNA,_Alpha,_W) \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_,_W)

# define VIPM_FCNAME_CN_WI2(_BaseName,_CnNA,_Alpha,_W1,_W2) \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_,VIPM_GLUE(_W1,_W2))

# define VIPM_FCNAME_CN_WI3(_BaseName,_CnNA,_Alpha,_W1,_W2,_W3)  \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_,VIPM_GLUE3(_W1,_W2,_W3))

# define VIPM_FCNAME_CN_WI4(_BaseName,_CnNA,_Alpha,_W1,_W2,_W3,_W4)  \
	VIPM_GLUE5(_BaseName,_,VIPM_CHANNELC_LTW(_CnNA,_Alpha),_,VIPM_GLUE4(_W1,_W2,_W3,_W4))


#endif /* !defined(__VIPM_CDEFS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
