/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(V.Sokur)
 * Vipm/DXchgInit/Convert/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_DXCHGINIT_CONVERT_ALGORITHM_H__
# define __VIPM_DXCHGINIT_CONVERT_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_DXCHGINIT_CONVERT_TYPES_H__
#  include "Vipm/DXchgInit/Convert/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB  __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

__PUB bo_status_t
VipmTGAconvert(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aScaleFactorAndValueOrigin, /* range-scalar-pair */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

__PUB bo_status_t
VipmTGAvconvert(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aValueRangePair,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_econvert(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_scalar_t const             aScaleFactor[],
	vipm_scalar_t const             aValueOrigin[],
	vipm_range_t                    aValueRangevSrc[]   /* = NULL */
#endif
	);

__PUB bo_status_t
vipma_escale(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_scalar_t const             aScaleFactor[],
	vipm_scalar_t const             aValueOrigin[],
	vipm_range_t                    aValueRangevSrc[]   /* = NULL */
#endif
	);

# define vipma_escale(x,f,rank,vol,e,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_escale_inl(x,f,rank,vol,e,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS)

/* ------------------------------------------------------------------------- */

__PUB bo_status_t
vipma_vconvert(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_range_t                    aValueRangevDst[],
	vipm_range_t                    aValueRangevSrc[]
#endif
	);

__PUB bo_status_t
vipma_vscale(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_range_t                    aValueRangevDst[],
	vipm_range_t                    aValueRangevSrc[]
#endif
	);

# define vipma_vscale(x,f,rank,vol,e,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vscale_inl(x,f,rank,vol,e,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS)


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * econvert
 */

/*
 * Dest Data type   : U8
 * Source Data type : S16,S32, F32
 */
# define vipma_econvert_u8s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_u8s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_u8f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S,offsS,strideS,scf,valo,vrngS)


/*
 * Dest Data type   : S16
 * Source Data type : U8,S32, F32
 */
# define vipma_econvert_s16u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_s16s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_s16f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S,offsS,strideS,scf,valo,vrngS)


/*
 * Dest Data type   : S32
 * Source Data type : U8,S16, F32
 */
# define vipma_econvert_s32u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_s32s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_s32f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S,offsS,strideS,scf,valo,vrngS)


/*
 * Dest Data type   : F32
 * Source Data type : U8,S16,S32
 */
# define vipma_econvert_f32u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_f32s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S,offsS,strideS,scf,valo,vrngS)

# define vipma_econvert_f32s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_econvert(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S,offsS,strideS,scf,valo,vrngS)

/* ------------------------------------------------------------------------- */

/*
 * escale
 *
 * Data type: U8,S16,S32, F32
 */

# define vipma_escale_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_escale(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS)

# define vipma_escale_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_escale(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS)

# define vipma_escale_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_escale(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS)

# define vipma_escale_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS) \
	vipma_escale(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,scf,valo,vrngS)

/* ------------------------------------------------------------------------- */

/*
 * vconvert
 */

/*
 * Dest Data type   : U8
 * Source Data type : S16,S32, F32
 */
# define vipma_vconvert_u8s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S16,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_u8s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_S32,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_u8f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,VipmK_ELEMTYPE_F32,S,offsS,strideS,vrngD,vrngS)


/*
 * Dest Data type   : S16
 * Source Data type : U8,S32, F32
 */
# define vipma_vconvert_s16u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_U8,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_s16s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_S32,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_s16f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,VipmK_ELEMTYPE_F32,S,offsS,strideS,vrngD,vrngS)


/*
 * Dest Data type   : S32
 * Source Data type : U8,S16, F32
 */
# define vipma_vconvert_s32u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_s32s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_s32f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,VipmK_ELEMTYPE_F32,S,offsS,strideS,vrngD,vrngS)


/*
 * Dest Data type   : F32
 * Source Data type : U8,S16,S32
 */
# define vipma_vconvert_f32u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_U8,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_f32s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S16,S,offsS,strideS,vrngD,vrngS)

# define vipma_vconvert_f32s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vconvert(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,VipmK_ELEMTYPE_S32,S,offsS,strideS,vrngD,vrngS)

/* ------------------------------------------------------------------------- */

/*
 * vscale
 *
 * Data type: U8,S16,S32, F32
 */

# define vipma_vscale_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vscale(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS)

# define vipma_vscale_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vscale(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS)

# define vipma_vscale_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vscale(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS)

# define vipma_vscale_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS) \
	vipma_vscale(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,vrngD,vrngS)


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

bo_status_t
vipma_escale_inl(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst,
	vipm_int const                  aStartDst[],
	vipm_int const                  aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,
	vipm_int const                  aStartSrc[],
	vipm_int const                  aStrideSrc[],
	vipm_scalar_t const             aScaleFactor[],
	vipm_scalar_t const             aValueOrigin[],
	vipm_range_t                    aValueRangevSrc[]
#endif
	);

bo_status_t
vipma_vscale_inl(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst,
	vipm_int const                  aStartDst[],
	vipm_int const                  aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,
	vipm_int const                  aStartSrc[],
	vipm_int const                  aStrideSrc[],
	vipm_range_t                    aValueRangevDst[],
	vipm_range_t                    aValueRangevSrc[]
#endif
	);

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_escale_inl(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst,
	vipm_int const                  aStartDst[],
	vipm_int const                  aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,
	vipm_int const                  aStartSrc[],
	vipm_int const                  aStrideSrc[],
	vipm_scalar_t const             aScaleFactor[],
	vipm_scalar_t const             aValueOrigin[],
	vipm_range_t                    aValueRangevSrc[]
#else
	aXenv, Options, aRank, aVolume, anElemType,
	aBaseDst, aStartDst, aStrideDst,
	aBaseSrc, aStartSrc, aStrideSrc,
	aScaleFactor, aValueOrigin, aValueRangevSrc
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_xenv              *aXenv;
	vipm_ulong                      Options;
	vipm_rank_t                     aRank;
	vipm_int const                  aVolume, aStartDst[], aStrideDst[], aStartSrc[], aStrideSrc[];
	vipm_uint                       anElemType;
	vipm_mutptr_t                   aBaseDst;
	vipm_imuptr_t                   aBaseSrc;
	vipm_scalar_t                  *aScaleFactor, *aValueOrigin;
	vipm_range_t                   *aValueRangevSrc;
#endif
{
	return vipma_econvert(
		aXenv, Options, aRank, aVolume,
		anElemType, aBaseDst, aStartDst, aStrideDst,
		anElemType, aBaseSrc, aStartSrc, aStrideSrc,
		aScaleFactor, aValueOrigin, aValueRangevSrc
	);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_vscale_inl(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst,
	vipm_int const                  aStartDst[],
	vipm_int const                  aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,
	vipm_int const                  aStartSrc[],
	vipm_int const                  aStrideSrc[],
	vipm_range_t                    aValueRangevDst[],
	vipm_range_t                    aValueRangevSrc[]
#else
	aXenv, Options, aRank, aVolume, anElemType,
	aBaseDst, aStartDst, aStrideDst,
	aBaseSrc, aStartSrc, aStrideSrc,
	aValueRangevDst, aValueRangevSrc
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_xenv              *aXenv;
	vipm_ulong                      Options;
	vipm_rank_t                     aRank;
	vipm_int const                  aVolume, aStartDst[], aStrideDst[], aStartSrc[], aStrideSrc[];
	vipm_uint                       anElemType;
	vipm_mutptr_t                   aBaseDst;
	vipm_imuptr_t                   aBaseSrc;
	vipm_range_t                   *aValueRangevDst, *aValueRangevSrc;
#endif
{
	return vipma_vconvert(
		aXenv, Options, aRank, aVolume,
		anElemType, aBaseDst, aStartDst, aStrideDst,
		anElemType, aBaseSrc, aStartSrc, aStrideSrc,
		aValueRangevDst, aValueRangevSrc
	);
}


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_DXCHGINIT_CONVERT_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
