/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Types.h	(V.Sokur)
 * Vipm/FeatDetect/MSER/Types.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_FEATDETECT_MSER_TYPES_H__
# define __VIPM_FEATDETECT_MSER_TYPES_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif
# ifndef __VIPM_ANALYSIS_WATERSHED_TYPES_H__
#  include <Vipm/Analysis/Watershed/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

struct _Vipm_mserparm {
	vipm_int                        erp_delta;
	float                           erp_rel_area_min,  /* Minimum area of any stable region relative to the image domain area */
	                                erp_rel_area_max;  /* Maximum area of any stable region relative to the image domain area */
	float                           erp_variation_max,
	                                erp_diversity_min;

	/* optional */
	vipm_int                        erp_pix_area_min;  /* 0 --> 9 */
	int                             erp_gatherfeat;    /* 0 */
};
typedef struct _Vipm_mserparm           vipm_mserparm_t;
/*
 * Defauts:
 *
 * ?. { 10, 0.0001, 0.5,  1,    0    }
 * 1. { 10, 0.0001, 0.75, 0.33, 0.2  }
 * 2. {  2, 0.0001, 0.5,  0.5,  0.33 }  ;; idiap
 * 3. {  5, 0.0001, 0.75, 0.25, 0.2  }  ;; vlfeat
 */


/*
 * @{
 * @note: NOT FOR YOUR BUSINESS
 */
struct _Vipm_mseriparm {
	vipm_int                        erp_delta;
	vipm_int                        erp_area_min,
	                                erp_area_max;
	float                           erp_variation_max,
	                                erp_diversity_min;
};
typedef struct _Vipm_mseriparm          vipm_mseriparm_t;
/*
 * Defauts:
 *
 * 1. { 10, 1, int-max, 1, 0 }
 *
 * @}
 */

/* ------------------------------------------------------------------------- */

struct _Vipm_mserfilt {
	/* params @{ */
	struct _Vipm_mserparm               erf_detparm;
	struct _Vipm_wtsparm                erf_wtsparm;
	struct _Vipm_wtslsrctx              erf_lsrctx_;
# define erf_featdes                    erf_lsrctx_.wlc_featdes
# define erf_featurec                   erf_lsrctx_.wlc_featurec
	/* @} */

	/* readonly @{ */
	bo_seqlist_(vipm_wtsregion_t *)     erf_msers;
	vipm_int                            erf_mserc_up,
	                                    erf_mserc_down;
	struct _Vipm_wtsregion            **erf_msers_up,
	                                  **erf_msers_down;
	/* @} */

	/* internal @{ */
	void                               *erf_opaque1_;
	void                               *erf_opaque2_;
	struct _Vipm_wtsfeature_ds          erf_featdes_dfl_;
	/* @} */
};
typedef struct _Vipm_mserfilt           vipm_mserfilt_t;

# define vipm_mserfilt(...)             VIPM_CTOR_FCNAME(vipm_mserfilt, __VA_ARGS__)
# define vipm_mserfilt_(...)            VIPM_PFX_FCNAME(vipm_mserfilt, __VA_ARGS__)

/* short versions */
# define vipm_mserf_(...)               VIPM_PFX_FCNAME(vipm_mserf, __VA_ARGS__)


/*
 * "Fast" mser filter
 */

struct _Vipm_fmserfilt {
	/* params @{ */
	struct _Vipm_mserparm               erf_detparm;
	struct _Vipm_wtsparm                erf_wtsparm;
	/* @} */

	/* readonly @{ */
	bo_seqlist_(vipm_fwtsregion_t *)    erf_msers;
	vipm_int                            erf_mserc_up,
	                                    erf_mserc_down;
	struct _Vipm_fwtsregion           **erf_msers_up,
	                                  **erf_msers_down;
	/* @} */

	/* internal @{ */
	vipm_Xvector_t                      erf_image_;         /* internal (?) */
	vipm_fwtsresult_t                  *erf_fwtsres_up_;
	vipm_fwtsresult_t                  *erf_fwtsres_down_;
	vipm_fwtsresult_t                   erf_fwtsres_upb_;
	vipm_fwtsresult_t                   erf_fwtsres_downb_;
	/* @} */
};
typedef struct _Vipm_fmserfilt          vipm_fmserfilt_t;

# define vipm_fmserfilt(...)            VIPM_CTOR_FCNAME(vipm_fmserfilt, __VA_ARGS__)
# define vipm_fmserfilt_(...)           VIPM_PFX_FCNAME(vipm_fmserfilt, __VA_ARGS__)

/* short versions */
# define vipm_fmserf_(...)              VIPM_PFX_FCNAME(vipm_fmserf, __VA_ARGS__)


/* ========================================================================= */

typedef bo_status_t (*vipma_mser_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	struct _Vipm_mserfilt          *aMserflt
# endif
	);

typedef bo_status_t (*vipma_mserex_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[],
	struct _Vipm_mserfilt          *aMserflt
# endif
	);


#endif /* !defined(__VIPM_FEATDETECT_MSER_TYPES_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/

