/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Utils.h	(V.Sokur)
 * Vipm/FeatDetect/MSER/Utils.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_FEATDETECT_MSER_UTILS_H__
# define __VIPM_FEATDETECT_MSER_UTILS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_FEATDETECT_MSER_TYPES_H__
#  include <Vipm/FeatDetect/MSER/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PROT __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * MSER-filter defaults:
 *
 * mser-detector-params = {
 *     delta         = 10;
 *     rel-area-min  = 0.0001;
 *     rel-area-max  = 0.75;
 *     variation-max = 0.33;
 *     diversity-min = 0.2;
 * };
 *
 * watershed-params = {
 *     direction   = VipmK_UP_DOWN_WATERSHED;
 *     dovalxor    = 0;
 *     use8nn      = 0;
 *     scalefactor = 0;
 *     start-pix   = NULL;
 * };
 */

/* ------------------------------------------------------------------------- */

/*
 * Construct
 */

/*
 * vipm_mserfilt_t *vipm_mserfilt()        (aThis);
 * vipm_mserfilt_t *vipm_mserfilt(feats)   (aThis, aFeatdesList, aNumFeatures);
 * vipm_mserfilt_t *vipm_mserfilt(mf)      (aThis, aDelta, aRelAreaMin, aRelAreaMax, aVariationMax, aDiversityMin,
 *                                          aDirection, aWithValxor, Use8nn, aScaleFactor, aStartPix);
 * vipm_mserfilt_t *vipm_mserfilt(ex)      (aThis, aFeatdesList, aNumFeatures,
 *                                          aDelta, aRelAreaMin, aRelAreaMax, aVariationMax, aDiversityMin,
 *                                          aDirection, aWithValxor, Use8nn, aScaleFactor, aStartPix);
 *
 * vipm_mserfilt_t *vipm_mserf_(destroy)   (aThis);
 */

__PUB struct _Vipm_mserfilt    *vipm_mserfilt()         __P((struct _Vipm_mserfilt *));
__PUB struct _Vipm_mserfilt    *vipm_mserfilt(feats)    __P((struct _Vipm_mserfilt *, struct _Vipm_wtsfeature_ds *, vipm_int));
__PUB struct _Vipm_mserfilt    *vipm_mserfilt(mf)       __P((struct _Vipm_mserfilt *, vipm_int, double, double, double, double,
                                                             vipm_int, int, int, vipm_int, vipm_Npoint_t const *));
__PUB struct _Vipm_mserfilt    *vipm_mserfilt(ex)       __P((struct _Vipm_mserfilt *, struct _Vipm_wtsfeature_ds *, vipm_int,
                                                             vipm_int, double, double, double, double,
                                                             vipm_int, int, int, vipm_int, vipm_Npoint_t const *));

__PUB struct _Vipm_mserfilt    *vipm_mserf_(destroy)    __P((struct _Vipm_mserfilt *));


/* ========================================================================= */

/*
 * vipm_fmserfilt_t    *vipm_fmserfilt()       (aThis);
 * vipm_fmserfilt_t    *vipm_fmserfilt(fwtsres)(aThis, aWtsresUp, aWtsresDown);
 * vipm_fmserfilt_t    *vipm_fmserfilt(ex)     (aThis, aWtsresUp, aWtsresDown,
 *                                              aDelta, aRelAreaMin, aRelAreaMax, aVariationMax, aDiversityMin,
 *                                              aDirection, aWithValxor, Use8nn, aScaleFactor, aStartPix);
 *
 * vipm_fmserfilt_t    *vipm_fmserf_(destroy)  (aThis);
 */

__PUB struct _Vipm_fmserfilt   *vipm_fmserfilt()        __P((struct _Vipm_fmserfilt *));
__PUB struct _Vipm_fmserfilt   *vipm_fmserfilt(fwtsres) __P((struct _Vipm_fmserfilt *, struct _Vipm_fwtsresult *, struct _Vipm_fwtsresult *));
__PUB struct _Vipm_fmserfilt   *vipm_fmserfilt(ex)      __P((struct _Vipm_fmserfilt *, struct _Vipm_fwtsresult *, struct _Vipm_fwtsresult *,
                                                             vipm_int, double, double, double, double,
                                                             vipm_int, int, int, vipm_int, vipm_Npoint_t const *));

__PUB struct _Vipm_fmserfilt   *vipm_fmserf_(destroy)   __P((struct _Vipm_fmserfilt *));

/* ------------------------------------------------------------------------- */

/*
 * Functional
 */

/*
 * vipm_fmserfilt_t    *vipm_fmserf_(setimvol)     (aMserflt, aRank, aVolume, aStrideSrc);
 * vipm_fmserfilt_t    *vipm_fmserf_(setimvol,2d)  (aMserflt, H, W, aYStride);
 */

__PUB struct _Vipm_fmserfilt   *vipm_fmserf_(setimvol)      __P((struct _Vipm_fmserfilt *, vipm_rank_t, vipm_int const [], vipm_int const []));
__PUB struct _Vipm_fmserfilt   *vipm_fmserf_(setimvol,2d)   __P((struct _Vipm_fmserfilt *, vipm_int, vipm_int, vipm_int));

/* ------------------------------------------------------------------------- */

/*
 * below functions return add-number-of-regions
 * vipm_int vipm_fmserf_(pixreg,2d)    (aMserflt, aPixRegionList, aPixList, aDirection);
 * vipm_int vipm_fmserf_(...,2d)       (aMserflt, ...List, aDirection);
 */

__PUB vipm_int  vipm_fmserf_(pixreg,2d) __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_fwtspixregion_t) *, bo_seqlist_(vipm_2d_point_t) *, vipm_int));

__PUB vipm_int  vipm_fmserf_(mf,2d)     __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_wtsmomentfeat_t) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(m1,2d)     __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_wtsmoment1_t) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(m2,2d)     __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_wtsmoment2_t) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(m2c,2d)    __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_wtsmoment2_t) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(brect,2d)  __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_2d_rect_t) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(minbquad,f2d)  __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_Quad_t(f2d)) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(minbquad,r2d)  __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_Quad_t(r2d)) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(ellipse,f2d)   __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_Ell_t(f2d)) *, vipm_int));
__PUB vipm_int  vipm_fmserf_(ellipse,r2d)   __P((struct _Vipm_fmserfilt *, bo_seqlist_(vipm_Ell_t(r2d)) *, vipm_int));


/* ========================================================================= */

__PROT float
__VipmMSERregionVariation(
#if __myc_require(prototypes)
	struct _Vipm_wtsregion const   *aCurrent,
	struct _Vipm_wtsregion const   *aParent
#endif
	);

__PROT float
__VipmMSERregionVariationAbs(
#if __myc_require(prototypes)
	struct _Vipm_wtsregion const   *aCurrent,
	struct _Vipm_wtsregion const   *aParent
#endif
	);

/* ------------------------------------------------------------------------- */

__PROT float
__VipmMSERfregionVariation(
#if __myc_require(prototypes)
	struct _Vipm_fwtsregion const  *aCurrent,
	struct _Vipm_fwtsregion const  *aParent
#endif
	);

__PROT float
__VipmMSERfregionVariationAbs(
#if __myc_require(prototypes)
	struct _Vipm_fwtsregion const  *aCurrent,
	struct _Vipm_fwtsregion const  *aParent
#endif
	);

/* ------------------------------------------------------------------------- */

__PROT float
__VipmMSERareaVariation(
#if __myc_require(prototypes)
	vipm_int                        anAreaCur,
	vipm_int                        anAreaPar
#endif
	);

__PROT float
__VipmMSERareaVariationAbs(
#if __myc_require(prototypes)
	vipm_int                        anAreaCur,
	vipm_int                        anAreaPar
#endif
	);


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

_VIPM_INLINE float
__VipmMSERareaVariation(
#if __myc_require(prototypes)
	vipm_int                        anAreaCur,
	vipm_int                        anAreaPar
#else
	anAreaCur, anAreaPar
#endif
	)
#if !__myc_require(prototypes)
	vipm_int                        anAreaCur,
	vipm_int                        anAreaPar
#endif
{
	return ((float)(anAreaPar - anAreaCur) / (float)anAreaPar);
}

_VIPM_INLINE float
__VipmMSERareaVariationAbs(
#if __myc_require(prototypes)
	vipm_int                        anAreaCur,
	vipm_int                        anAreaPar
#else
	anAreaCur, anAreaPar
#endif
	)
#if !__myc_require(prototypes)
	vipm_int                        anAreaCur,
	vipm_int                        anAreaPar
#endif
{
	return vipmi_absf((float)(anAreaPar - anAreaCur) / (float)anAreaPar);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE float
__VipmMSERregionVariation(
#if __myc_require(prototypes)
	struct _Vipm_wtsregion const   *aRegionCur,
	struct _Vipm_wtsregion const   *aRegionPar
#else
	aRegionCur, aRegionPar
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_wtsregion const   *aRegionCur,
	struct _Vipm_wtsregion const   *aRegionPar
#endif
{
	return __VipmMSERareaVariation(aRegionCur->wsr_pixelc, aRegionPar->wsr_pixelc);
}

_VIPM_INLINE float
__VipmMSERregionVariationAbs(
#if __myc_require(prototypes)
	struct _Vipm_wtsregion const   *aRegionCur,
	struct _Vipm_wtsregion const   *aRegionPar
#else
	aRegionCur, aRegionPar
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_wtsregion const   *aRegionCur,
	struct _Vipm_wtsregion const   *aRegionPar
#endif
{
	return __VipmMSERareaVariationAbs(aRegionCur->wsr_pixelc, aRegionPar->wsr_pixelc);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE float
__VipmMSERfregionVariation(
#if __myc_require(prototypes)
	struct _Vipm_fwtsregion const  *aRegionCur,
	struct _Vipm_fwtsregion const  *aRegionPar
#else
	aRegionCur, aRegionPar
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_fwtsregion const  *aRegionCur,
	struct _Vipm_fwtsregion const  *aRegionPar
#endif
{
	return __VipmMSERareaVariation(aRegionCur->wsr_pixelc, aRegionPar->wsr_pixelc);
}

_VIPM_INLINE float
__VipmMSERfregionVariationAbs(
#if __myc_require(prototypes)
	struct _Vipm_fwtsregion const  *aRegionCur,
	struct _Vipm_fwtsregion const  *aRegionPar
#else
	aRegionCur, aRegionPar
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_fwtsregion const  *aRegionCur,
	struct _Vipm_fwtsregion const  *aRegionPar
#endif
{
	return __VipmMSERareaVariationAbs(aRegionCur->wsr_pixelc, aRegionPar->wsr_pixelc);
}


__MYC_END_CDECLS
# undef __PUB
# undef __PROT
# undef __PRIV


#endif /* !defined(__VIPM_FEATDETECT_MSER_UTILS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
