/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2018 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(D.Vinnyk)
 * Vipm/Filtering/GlobalFilters/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_FILTERING_GLOBALFILTERS_ALGORITHM_H__
# define __VIPM_FILTERING_GLOBALFILTERS_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_FILTERING_GLOBALFILTERS_TYPES_H__
#  include "Vipm/Filtering/GlobalFilters/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

/*
 * firefly:
 */
__PUB bo_status_t
VipmTGAfirefly(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_firflparm_t const *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

/*
 * fireflyex:
 */
__PUB bo_status_t
VipmTGAfireflyex(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_fflexparm_t const *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_firefly(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	bo_intptr_t                     anAlgType,
	vipm_seshape_t const           *aStrel
#endif
	);

__PUB bo_status_t
vipma_efirefly(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	bo_intptr_t                     anAlgType,
	vipm_seshape_t const           *aStrel,
	vipm_firflopts_t const         *aFirflOpts,
	vipm_arrshape_t const          *aWholeArrShapeSrc
#endif
	);

__PUB bo_status_t
vipma_fireflyex(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	bo_intptr_t                     anAlgType,
	vipm_seshape_t const            aStrel[],
	vipm_int const                  aWeights[],
	vipm_int                        aFuncCnt
#endif
	);

__PUB bo_status_t
vipma_efireflyex(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	bo_intptr_t                     anAlgType,
	vipm_seshape_t const            aStrel[],
	vipm_int const                  aWeights[],
	vipm_int                        aFuncCnt,
	vipm_fflexopts_t const         *aFirflOpts,
	vipm_arrshape_t const          *aWholeArrShapeSrc
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * firefly
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_firefly_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se) \
    vipma_firefly(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,alg,se)

# define vipma_firefly_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se) \
    vipma_firefly(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,alg,se)

# define vipma_firefly_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se) \
    vipma_firefly(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,alg,se)

# define vipma_firefly_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se) \
    vipma_firefly(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,alg,se)

/* ------------------------------------------------------------------------- */

/*
 * efirefly
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_efirefly_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS) \
    vipma_efirefly(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS)

# define vipma_efirefly_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS) \
    vipma_efirefly(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS)

# define vipma_efirefly_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS) \
    vipma_efirefly(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS)

# define vipma_efirefly_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS) \
    vipma_efirefly(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,alg,se,opts,shapeS)

/* ------------------------------------------------------------------------- */

/*
 * fireflyex
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_fireflyex_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt) \
    vipma_fireflyex(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt)

# define vipma_fireflyex_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt) \
    vipma_fireflyex(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt)

# define vipma_fireflyex_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt) \
    vipma_fireflyex(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt)

# define vipma_fireflyex_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt) \
    vipma_fireflyex(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt)

/* ------------------------------------------------------------------------- */

/*
 * efireflyex
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_efireflyex_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS) \
    vipma_efireflyex(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS)

# define vipma_efireflyex_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS) \
    vipma_efireflyex(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS)

# define vipma_efireflyex_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS) \
    vipma_efireflyex(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS)

# define vipma_efireflyex_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS) \
    vipma_efireflyex(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,alg,se,w,fcnt,opts,shapeS)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_FILTERING_GLOBALFILTERS_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/

