/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2017 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(D.Vinnyk)
 * Vipm/Filtering/LocalFilters/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_FILTERING_LOCALFILTERS_ALGORITHM_H__
# define __VIPM_FILTERING_LOCALFILTERS_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_FILTERING_LOCALFILTERS_TYPES_H__
#  include "Vipm/Filtering/LocalFilters/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

/*
 * filter: { box,min,max,median;close,open,bothat,tophat,gradient }
 */
__PUB bo_status_t
VipmTGAfilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_filtparm_t const *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

/*
 * gaussianfilter
 */
__PUB bo_status_t
VipmTGAgaussianfilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_gfltparm_t const *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

/*
 * separable filter
 */
__PUB bo_status_t
VipmTGAsepfilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_sfltparm_t const *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_filter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_int                        aFilterMethod,
	vipm_seshape_t const           *aStrel
#endif
	);

__PUB bo_status_t
vipma_efilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_int                        aFilterMethod,
	vipm_seshape_t const           *aStrel,
	vipm_filtopts_t const          *aFiltOpts,
	vipm_arrshape_t const          *aWholeArrShapeSrc
#endif
	);

__PUB bo_status_t
vipma_gaussianfilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_seshape_t const           *aStrel,
	vipm_scalar_t const             aSigmas[]           /* = NULL */
#endif
	);

__PUB bo_status_t
vipma_egaussianfilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_seshape_t const           *aStrel,
	vipm_scalar_t const             aSigmas[],          /* = NULL */
	vipm_filtopts_t const          *aFiltOpts,
	vipm_arrshape_t const          *aWholeArrShapeSrc
#endif
	);

__PUB bo_status_t
vipma_sepfilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_sepkern_t const           *aSepKern
#endif
	);

__PUB bo_status_t
vipma_esepfilter(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_sepkern_t const           *aSepKern,
	vipm_filtopts_t const          *aFiltOpts,
	vipm_arrshape_t const          *aWholeArrShapeSrc
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * filter
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_filter_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se) \
    vipma_filter(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,filt,se)

# define vipma_filter_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se) \
    vipma_filter(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,filt,se)

# define vipma_filter_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se) \
    vipma_filter(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,filt,se)

# define vipma_filter_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se) \
    vipma_filter(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,filt,se)

/* ------------------------------------------------------------------------- */

/*
 * efilter
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_efilter_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS) \
    vipma_efilter(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS)

# define vipma_efilter_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS) \
    vipma_efilter(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS)

# define vipma_efilter_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS) \
    vipma_efilter(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS)

# define vipma_efilter_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS) \
    vipma_efilter(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,filt,se,opts,shapeS)

/* ------------------------------------------------------------------------- */

/*
 * gaussianfilter
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_gaussianfilter_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm) \
    vipma_gaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,se,sigm)

# define vipma_gaussianfilter_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm) \
    vipma_gaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,se,sigm)

# define vipma_gaussianfilter_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm) \
    vipma_gaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,se,sigm)

# define vipma_gaussianfilter_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm) \
    vipma_gaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,se,sigm)

/* ------------------------------------------------------------------------- */

/*
 * egaussianfilter
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_egaussianfilter_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS) \
    vipma_egaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS)

# define vipma_egaussianfilter_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS) \
    vipma_egaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS)

# define vipma_egaussianfilter_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS) \
    vipma_egaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS)

# define vipma_egaussianfilter_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS) \
    vipma_egaussianfilter(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,se,sigm,opts,shapeS)

/* ------------------------------------------------------------------------- */

/*
 * sepfilter
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_sepfilter_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern) \
    vipma_sepfilter(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,sepkern)

# define vipma_sepfilter_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern) \
    vipma_sepfilter(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,sepkern)

# define vipma_sepfilter_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern) \
    vipma_sepfilter(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,sepkern)

# define vipma_sepfilter_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern) \
    vipma_sepfilter(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,sepkern)

/* ------------------------------------------------------------------------- */

/*
 * esepfilter
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_esepfilter_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS) \
    vipma_esepfilter(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS)

# define vipma_esepfilter_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS) \
    vipma_esepfilter(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS)

# define vipma_esepfilter_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS) \
    vipma_esepfilter(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS)

# define vipma_esepfilter_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS) \
    vipma_esepfilter(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,sepkern,opts,shapeS)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_FILTERING_LOCALFILTERS_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/

