/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2007 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Generic.h	(V.Sokur)
 * Vipm/Generic.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_GENERIC_H__
# define __VIPM_GENERIC_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif
# ifndef __VIPM_UTILS_H__
#  include <Vipm/Utils.h>
# endif

# ifndef __BO_SYS_BYTEORDER__
#  include <Bo/sys/Byteorder.h>
# endif


/******************************************************************************
 * %% BeginSection: types/constants
 */

/*
 * Available Options: VipmF_MKBRONLY; VipmF_MUTABLE_PADDATA, ...
 */

typedef bo_status_t (*vipma_anyop_fn)   __P((struct _Vipm_xenv *, ...));

/* ------------------------------------------------------------------------- */

typedef bo_status_t (*vipma_tgfold_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,     /* for example: 3 */
	vipm_int const                  aVolume[], /* for example: height, width, nchannels */
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStartSrc[],  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*vipma_fold_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStartSrc[],  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*__vipma_fold_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int aStartSrc[],  vipm_int aStrideSrc[]
# endif
	);


typedef bo_status_t (*vipma_tgbfold_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*vipma_bfold_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*__vipma_bfold_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int aStrideSrc[]
# endif
	);

/* ------------------------------------------------------------------------- */

typedef bo_status_t (*vipma_tggen_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,     /* for example: 3 */
	vipm_int const                  aVolume[], /* for example: height, width, nchannels */
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[]
# endif
	);
typedef bo_status_t (*vipma_gen_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[]
# endif
	);
typedef bo_status_t (*__vipma_gen_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStartDst[],  vipm_int aStrideDst[]
# endif
	);


typedef bo_status_t (*vipma_tgbgen_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[]
# endif
	);
typedef bo_status_t (*vipma_bgen_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[]
# endif
	);
typedef bo_status_t (*__vipma_bgen_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStrideDst[]
# endif
	);

/* ------------------------------------------------------------------------- */

typedef bo_status_t (*vipma_tgmap_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStartSrc[],  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*vipma_map_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStartSrc[],  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*__vipma_map_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStartDst[],  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int aStartSrc[],  vipm_int aStrideSrc[]
# endif
	);


typedef bo_status_t (*vipma_tgbmap_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*vipma_bmap_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStrideSrc[]
# endif
	);
typedef bo_status_t (*__vipma_bmap_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc,  vipm_int aStrideSrc[]
# endif
	);

/* ------------------------------------------------------------------------- */

typedef bo_status_t (*vipma_tgzip_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[]
# endif
	);
typedef bo_status_t (*vipma_zip_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[]
# endif
	);
typedef bo_status_t (*__vipma_zip_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStartDst[],  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int aStartSrc1[], vipm_int aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int aStartSrc2[], vipm_int aStrideSrc2[]
# endif
	);


typedef bo_status_t (*vipma_tgbzip_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStrideSrc2[]
# endif
	);
typedef bo_status_t (*vipma_bzip_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStrideSrc2[]
# endif
	);
typedef bo_status_t (*__vipma_bzip_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int aStrideSrc2[]
# endif
	);

/* ------------------------------------------------------------------------- */

typedef bo_status_t (*vipma_tgzip3_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[],
	vipm_uint                       anElemTypeSrc3,
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStartSrc3[], vipm_int const aStrideSrc3[]
# endif
	);
typedef bo_status_t (*vipma_zip3_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStartSrc3[], vipm_int const aStrideSrc3[]
# endif
	);
typedef bo_status_t (*__vipma_zip3_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStartDst[],  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int aStartSrc1[], vipm_int aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int aStartSrc2[], vipm_int aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int aStartSrc3[], vipm_int aStrideSrc3[]
# endif
	);


typedef bo_status_t (*vipma_tgbzip3_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStrideSrc2[],
	vipm_uint                       anElemTypeSrc3,
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStrideSrc3[]
# endif
	);
typedef bo_status_t (*vipma_bzip3_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStrideSrc3[]
# endif
	);
typedef bo_status_t (*__vipma_bzip3_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int aStrideSrc3[]
# endif
	);

/* ------------------------------------------------------------------------- */

typedef bo_status_t (*vipma_tgzip4_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[],
	vipm_uint                       anElemTypeSrc3,
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStartSrc3[], vipm_int const aStrideSrc3[],
	vipm_uint                       anElemTypeSrc4,
	vipm_imuptr_t                   aBaseSrc4, vipm_int const aStartSrc4[], vipm_int const aStrideSrc4[]
# endif
	);
typedef bo_status_t (*vipma_zip4_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStartSrc3[], vipm_int const aStrideSrc3[],
	vipm_imuptr_t                   aBaseSrc4, vipm_int const aStartSrc4[], vipm_int const aStrideSrc4[]
# endif
	);
typedef bo_status_t (*__vipma_zip4_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStartDst[],  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int aStartSrc1[], vipm_int aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int aStartSrc2[], vipm_int aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int aStartSrc3[], vipm_int aStrideSrc3[],
	vipm_imuptr_t                   aBaseSrc4, vipm_int aStartSrc4[], vipm_int aStrideSrc4[]
# endif
	);


typedef bo_status_t (*vipma_tgbzip4_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStrideSrc2[],
	vipm_uint                       anElemTypeSrc3,
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStrideSrc3[],
	vipm_uint                       anElemTypeSrc4,
	vipm_imuptr_t                   aBaseSrc4, vipm_int const aStrideSrc4[]
# endif
	);
typedef bo_status_t (*vipma_bzip4_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStrideSrc3[],
	vipm_imuptr_t                   aBaseSrc4, vipm_int const aStrideSrc4[]
# endif
	);
typedef bo_status_t (*__vipma_bzip4_fn)(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aVolume[],
	vipm_mutptr_t                   aBaseDst,  vipm_int aStrideDst[],
	vipm_imuptr_t                   aBaseSrc1, vipm_int aStrideSrc1[],
	vipm_imuptr_t                   aBaseSrc2, vipm_int aStrideSrc2[],
	vipm_imuptr_t                   aBaseSrc3, vipm_int aStrideSrc3[],
	vipm_imuptr_t                   aBaseSrc4, vipm_int aStrideSrc4[]
# endif
	);

/* ------------------------------------------------------------------------- */

_VIPM_LOCAL_ALGCB_STRUCT(_Vipm_local_genalgcb,1);
_VIPM_ALGCB_STRUCT(_Vipm_genalgcb,1);

_VIPM_LOCAL_ALGCB_STRUCT(_Vipm_local_mapalgcb,2);
_VIPM_ALGCB_STRUCT(_Vipm_mapalgcb,2);

_VIPM_LOCAL_ALGCB_STRUCT(_Vipm_local_zipalgcb,3);
_VIPM_ALGCB_STRUCT(_Vipm_zipalgcb,3);
_VIPM_LOCAL_ALGCB_STRUCT(_Vipm_local_zip3algcb,4);
_VIPM_ALGCB_STRUCT(_Vipm_zip3algcb,4);
_VIPM_LOCAL_ALGCB_STRUCT(_Vipm_local_zip4algcb,5);
_VIPM_ALGCB_STRUCT(_Vipm_zip4algcb,5);


/******************************************************************************
 * %% BeginSection: function prototypes
 */

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
__MYC_BEGIN_CDECLS

__PUB bo_status_t
vipma_proc_enotimp(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	...
# endif
	);

__PUB bo_status_t
vipma_proc_enotsup(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	...
# endif
	);

__PUB bo_status_t
vipma_proc_einval(
# if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	...
# endif
	);

/* ------------------------------------------------------------------------- */

__PUB bo_status_t
vipma_call(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp
#endif
	);

/* ------------------------------------------------------------------------- */

__PUB struct _Vipm_algcb *
vipma_foldpackL(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp,
	bo_uint32_t                     anOptype,
	bo_status_t                   (*anAlgRoutine) __P((void)),
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aRankMin,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStartSrc[],  vipm_int const aStrideSrc[]
#endif
	);


__PUB struct _Vipm_algcb *
vipma_genpackL(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp,
	bo_uint32_t                     anOptype,
	bo_status_t                   (*anAlgRoutine) __P((void)),
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aRankMin,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[]
#endif
	);

__PUB struct _Vipm_algcb *
vipma_mappackL(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp,
	bo_uint32_t                     anOptype,
	bo_status_t                   (*anAlgRoutine) __P((void)),
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aRankMin,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStartSrc[],  vipm_int const aStrideSrc[]
#endif
	);

__PUB struct _Vipm_algcb *
vipma_zippackL(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp,
	bo_uint32_t                     anOptype,
	bo_status_t                   (*anAlgRoutine) __P((void)),
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aRankMin,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[]
#endif
	);

__PUB struct _Vipm_algcb *
vipma_zip3packL(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp,
	bo_uint32_t                     anOptype,
	bo_status_t                   (*anAlgRoutine) __P((void)),
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aRankMin,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[],
	vipm_uint                       anElemTypeSrc3,
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStartSrc3[], vipm_int const aStrideSrc3[]
#endif
	);

__PUB struct _Vipm_algcb *
vipma_zip4packL(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp,
	bo_uint32_t                     anOptype,
	bo_status_t                   (*anAlgRoutine) __P((void)),
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aRankMin,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst,  vipm_int const aStartDst[],  vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc1,
	vipm_imuptr_t                   aBaseSrc1, vipm_int const aStartSrc1[], vipm_int const aStrideSrc1[],
	vipm_uint                       anElemTypeSrc2,
	vipm_imuptr_t                   aBaseSrc2, vipm_int const aStartSrc2[], vipm_int const aStrideSrc2[],
	vipm_uint                       anElemTypeSrc3,
	vipm_imuptr_t                   aBaseSrc3, vipm_int const aStartSrc3[], vipm_int const aStrideSrc3[],
	vipm_uint                       anElemTypeSrc4,
	vipm_imuptr_t                   aBaseSrc4, vipm_int const aStartSrc4[], vipm_int const aStrideSrc4[]
#endif
	);


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

_VIPM_INLINE struct _Vipm_algcb *
vipma_foldpackL(
#if __myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp,
	bo_uint32_t                     anOptype,
	bo_status_t                   (*anAlgRoutine) __P((void)),
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm,
	vipm_rank_t                     aRank,
	vipm_int                        aRankMin,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc,  vipm_int const aStartSrc[],  vipm_int const aStrideSrc[]
#else
	anAlgcbp, anOptype, anAlgRoutine,
	aXenv, Options, aParm, aRank, aRankMin, aVolume,
	anElemTypeSrc, aBaseSrc, aStartSrc, aStrideSrc
#endif
	)
#if !__myc_require(prototypes)
	struct _Vipm_algcb             *anAlgcbp;
	bo_uint32_t                     anOptype;
	bo_status_t                   (*anAlgRoutine) __P((void));
	struct _Vipm_xenv              *aXenv;
	vipm_ulong                      Options;
	vipm_rank_t                     aRank;
	vipm_int                        aRankMin;
	vipm_int const                  aVolume, aStartSrc[], aStrideSrc[];
	vipm_uint                       anElemTypeSrc;
	vipm_imuptr_t                   aBaseSrc;
#endif
{
	return vipma_genpackL(
		anAlgcbp, anOptype, anAlgRoutine,
		aXenv, Options, aParm, aRank, aRankMin, aVolume,
		anElemTypeSrc, (vipm_mutptr_t)aBaseSrc, aStartSrc, aStrideSrc
	);
}


__MYC_END_CDECLS
# undef __PUB


#endif /* !defined(__VIPM_GENERIC_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
