/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * PointList.h	(V.Sokur)
 * Vipm/Math/Geometry/2D/PointList.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_MATH_GEOMETRY_2D_POINTLIST_H__
# define __VIPM_MATH_GEOMETRY_2D_POINTLIST_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * @note: top-left origin
 *
 * math origin is bottom-left
 * math counter-clockwise rotation ~ our clockwise rotation
 */

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PROT __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Point-list{2D}
 */

__PUB int                 vipm_Ptv_(2d)(isclockwise,M)  __P((vipm_Point_t(2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(2d)(iscclockwise,M) __P((vipm_Point_t(2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(2d)(isclockwise)    __P((vipm_Point_t(2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(2d)(iscclockwise)   __P((vipm_Point_t(2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(2d)(isconvex)       __P((vipm_Point_t(2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(2d)(all_colinear)   __P((vipm_Point_t(2d) const [], vipm_int));

/* ------------------------------------------------------------------------- */

__PUB vipm_Point_t(2d)   *vipm_Ptv_(2d)(lowest)         __P((vipm_Point_t(2d) const [], vipm_int));

__PUB double             *vipm_Ptv_(2d)(moment1)        __P((double [/*2*/], vipm_Point_t(2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(2d)(moment2)        __P((double [/*3*/], vipm_Point_t(2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(2d)(moment2,c)      __P((double [/*3*/], vipm_Point_t(2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(2d)(covariance)     __P((double [/*4*/], vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Point_t(2d)   *vipm_Ptv_(2d)(centroid)       __P((vipm_Point_t(2d)  *, vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Point_t(f2d)  *vipm_Ptv_(2d)(centroid,f)     __P((vipm_Point_t(f2d) *, vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Point_t(r2d)  *vipm_Ptv_(2d)(centroid,r)     __P((vipm_Point_t(r2d) *, vipm_Point_t(2d) const [], vipm_int));

/* ------------------------------------------------------------------------- */

__PUB vipm_Rect_t(2d)    *vipm_Ptv_(2d)(brect)          __P((vipm_Rect_t(2d)  *, vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Quad_t(f2d)   *vipm_Ptv_(2d)(bquad,f)        __P((vipm_Quad_t(f2d) *, vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(2d)(bquad,r)        __P((vipm_Quad_t(r2d) *, vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Quad_t(f2d)   *vipm_Ptv_(2d)(minbquad,f)     __P((vipm_Quad_t(f2d) *, vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(2d)(minbquad,r)     __P((vipm_Quad_t(r2d) *, vipm_Point_t(2d) const [], vipm_int));

/* for convex, clockwise(math:counter-clockwise) oriented shape */
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(2d)(_minbquad,r)  __P((vipm_Quad_t(r2d) *, vipm_Point_t(2d) const [], vipm_int/* >=3 */));

/* ------------------------------------------------------------------------- */

__PUB vipm_int            vipm_Ptv_(2d)(convexhull,I)   __P((vipm_Point_t(2d) [], vipm_int));
__PUB vipm_int            vipm_Ptv_(2d)(convexhull)     __P((vipm_Point_t(2d) [], vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_int            vipm_Ptv_(2d)(aconvexhull)    __P((vipm_Point_t(2d) **, vipm_Point_t(2d) const [], vipm_int));
__PUB vipm_Point_t(2d)   *vipm_Ptv_(2d)(sort,I)         __P((vipm_Point_t(2d) [], vipm_int));


/* ========================================================================= */

/*
 * Point-list{F2D}
 */

__PUB int                 vipm_Ptv_(f2d)(isclockwise,M) __P((vipm_Point_t(f2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(f2d)(iscclockwise,M)__P((vipm_Point_t(f2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(f2d)(isclockwise)   __P((vipm_Point_t(f2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(f2d)(iscclockwise)  __P((vipm_Point_t(f2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(f2d)(isconvex)      __P((vipm_Point_t(f2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(f2d)(all_colinear)  __P((vipm_Point_t(f2d) const [], vipm_int));

/* ------------------------------------------------------------------------- */

__PUB vipm_Point_t(f2d)  *vipm_Ptv_(f2d)(lowest)        __P((vipm_Point_t(f2d) const [], vipm_int));

__PUB double             *vipm_Ptv_(f2d)(moment1)       __P((double [/*2*/], vipm_Point_t(f2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(f2d)(moment2)       __P((double [/*3*/], vipm_Point_t(f2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(f2d)(moment2,c)     __P((double [/*3*/], vipm_Point_t(f2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(f2d)(covariance)    __P((double [/*4*/], vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_Point_t(f2d)  *vipm_Ptv_(f2d)(centroid)      __P((vipm_Point_t(f2d) *, vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_Point_t(r2d)  *vipm_Ptv_(f2d)(centroid,r)    __P((vipm_Point_t(r2d) *, vipm_Point_t(f2d) const [], vipm_int));
# define vipm_f2d_ptv_centroid_$f           vipm_f2d_ptv_centroid /* vipm_Ptv_(f2d)(centroid) */

/* ------------------------------------------------------------------------- */

__PUB vipm_Rect_t(f2d)   *vipm_Ptv_(f2d)(brect)         __P((vipm_Rect_t(f2d) *, vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_Quad_t(f2d)   *vipm_Ptv_(f2d)(bquad)         __P((vipm_Quad_t(f2d) *, vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(f2d)(bquad,r)       __P((vipm_Quad_t(r2d) *, vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_Quad_t(f2d)   *vipm_Ptv_(f2d)(minbquad)      __P((vipm_Quad_t(f2d) *, vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(f2d)(minbquad,r)    __P((vipm_Quad_t(r2d) *, vipm_Point_t(f2d) const [], vipm_int));
# define vipm_f2d_ptv_bquad_$f              vipm_f2d_ptv_bquad
# define vipm_f2d_ptv_minbquad_$f           vipm_f2d_ptv_minbquad

/* for convex, clockwise(math:counter-clockwise) oriented shape */
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(f2d)(_minbquad,r) __P((vipm_Quad_t(r2d) *, vipm_Point_t(f2d) const [], vipm_int/* >=3 */));

/* ------------------------------------------------------------------------- */

__PUB vipm_int            vipm_Ptv_(f2d)(convexhull,I)  __P((vipm_Point_t(f2d) [], vipm_int));
__PUB vipm_int            vipm_Ptv_(f2d)(convexhull)    __P((vipm_Point_t(f2d) [], vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_int            vipm_Ptv_(f2d)(aconvexhull)   __P((vipm_Point_t(f2d) **, vipm_Point_t(f2d) const [], vipm_int));
__PUB vipm_Point_t(f2d)  *vipm_Ptv_(f2d)(sort,I)        __P((vipm_Point_t(f2d) [], vipm_int));


/* ========================================================================= */

/*
 * Point-list{R2D}
 */

__PUB int                 vipm_Ptv_(r2d)(isclockwise,M) __P((vipm_Point_t(r2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(r2d)(iscclockwise,M)__P((vipm_Point_t(r2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(r2d)(isclockwise)   __P((vipm_Point_t(r2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(r2d)(iscclockwise)  __P((vipm_Point_t(r2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(r2d)(isconvex)      __P((vipm_Point_t(r2d) const [], vipm_int));
__PUB int                 vipm_Ptv_(r2d)(all_colinear)  __P((vipm_Point_t(r2d) const [], vipm_int));

/* ------------------------------------------------------------------------- */

__PUB vipm_Point_t(r2d)  *vipm_Ptv_(r2d)(lowest)        __P((vipm_Point_t(r2d) const [], vipm_int));

__PUB double             *vipm_Ptv_(r2d)(moment1)       __P((double [/*2*/], vipm_Point_t(r2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(r2d)(moment2)       __P((double [/*3*/], vipm_Point_t(r2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(r2d)(moment2,c)     __P((double [/*3*/], vipm_Point_t(r2d) const [], vipm_int));
__PUB double             *vipm_Ptv_(r2d)(covariance)    __P((double [/*4*/], vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_Point_t(r2d)  *vipm_Ptv_(r2d)(centroid)      __P((vipm_Point_t(r2d) *, vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_Point_t(f2d)  *vipm_Ptv_(r2d)(centroid,f)    __P((vipm_Point_t(f2d) *, vipm_Point_t(r2d) const [], vipm_int));
# define vipm_r2d_ptv_centroid_$r           vipm_r2d_ptv_centroid

/* ------------------------------------------------------------------------- */

__PUB vipm_Rect_t(r2d)   *vipm_Ptv_(r2d)(brect)         __P((vipm_Rect_t(r2d) *, vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(r2d)(bquad)         __P((vipm_Quad_t(r2d) *, vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_Quad_t(f2d)   *vipm_Ptv_(r2d)(bquad,f)       __P((vipm_Quad_t(f2d) *, vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(r2d)(minbquad)      __P((vipm_Quad_t(r2d) *, vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_Quad_t(f2d)   *vipm_Ptv_(r2d)(minbquad,f)    __P((vipm_Quad_t(f2d) *, vipm_Point_t(r2d) const [], vipm_int));
# define vipm_r2d_ptv_bquad_$r              vipm_r2d_ptv_bquad
# define vipm_r2d_ptv_minbquad_$r           vipm_r2d_ptv_minbquad

/* for convex, clockwise(math:counter-clockwise) oriented shape */
__PUB vipm_Quad_t(r2d)   *vipm_Ptv_(r2d)(_minbquad)     __P((vipm_Quad_t(r2d) *, vipm_Point_t(r2d) const [], vipm_int/* >=3 */));
# define vipm_r2d_ptv__minbquad_$r          vipm_r2d_ptv__minbquad

/* ------------------------------------------------------------------------- */

__PUB vipm_int            vipm_Ptv_(r2d)(convexhull,I)  __P((vipm_Point_t(r2d) [], vipm_int));
__PUB vipm_int            vipm_Ptv_(r2d)(convexhull)    __P((vipm_Point_t(r2d) [], vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_int            vipm_Ptv_(r2d)(aconvexhull)   __P((vipm_Point_t(r2d) **, vipm_Point_t(r2d) const [], vipm_int));
__PUB vipm_Point_t(r2d)  *vipm_Ptv_(r2d)(sort,I)        __P((vipm_Point_t(r2d) [], vipm_int));


/* ========================================================================= */

__PROT double             vipm_Ptv_(2d)(__sarea)        __P((vipm_Point_t(2d)  const [], vipm_int));
__PROT double             vipm_Ptv_(f2d)(__sarea)       __P((vipm_Point_t(f2d) const [], vipm_int));
__PROT double             vipm_Ptv_(r2d)(__sarea)       __P((vipm_Point_t(r2d) const [], vipm_int));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/* @{ */

_VIPM_INLINE int
vipm_Ptv_(2d)(isclockwise)(vipm_Point_t(2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__gequ0d(vipm_Ptv_(2d)(__sarea)(aList, aNumPoints));
}

_VIPM_INLINE int
vipm_Ptv_(2d)(iscclockwise)(vipm_Point_t(2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__less0d(vipm_Ptv_(2d)(__sarea)(aList, aNumPoints));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_Ptv_(2d)(isclockwise,M)(vipm_Point_t(2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__less0d(vipm_Ptv_(2d)(__sarea)(aList, aNumPoints));
}

_VIPM_INLINE int
vipm_Ptv_(2d)(iscclockwise,M)(vipm_Point_t(2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__gequ0d(vipm_Ptv_(2d)(__sarea)(aList, aNumPoints));
}

/* ------------------------------------------------------------------------- */

# define vipm_2d_ptv_centroid_$r(r,sh,n)    vipm_r2d_point_mp(vipm_2d_ptv_moment1((double *)(r),sh,n))

/* @} */


/* ========================================================================= */

/* @{ */

_VIPM_INLINE int
vipm_Ptv_(f2d)(isclockwise)(vipm_Point_t(f2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__gequ0d(vipm_Ptv_(f2d)(__sarea)(aList, aNumPoints));
}

_VIPM_INLINE int
vipm_Ptv_(f2d)(iscclockwise)(vipm_Point_t(f2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__less0d(vipm_Ptv_(f2d)(__sarea)(aList, aNumPoints));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_Ptv_(f2d)(isclockwise,M)(vipm_Point_t(f2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__less0d(vipm_Ptv_(f2d)(__sarea)(aList, aNumPoints));
}

_VIPM_INLINE int
vipm_Ptv_(f2d)(iscclockwise,M)(vipm_Point_t(f2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__gequ0d(vipm_Ptv_(f2d)(__sarea)(aList, aNumPoints));
}

/* ------------------------------------------------------------------------- */

# define vipm_f2d_ptv_centroid_$r(r,sh,n)   vipm_r2d_point_mp(vipm_f2d_ptv_moment1((double *)(r),sh,n))

/* @} */


/* ========================================================================= */

/* @{ */

_VIPM_INLINE int
vipm_Ptv_(r2d)(isclockwise)(vipm_Point_t(r2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__gequ0d(vipm_Ptv_(r2d)(__sarea)(aList, aNumPoints));
}

_VIPM_INLINE int
vipm_Ptv_(r2d)(iscclockwise)(vipm_Point_t(r2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__less0d(vipm_Ptv_(r2d)(__sarea)(aList, aNumPoints));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_Ptv_(r2d)(isclockwise,M)(vipm_Point_t(r2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__less0d(vipm_Ptv_(r2d)(__sarea)(aList, aNumPoints));
}

_VIPM_INLINE int
vipm_Ptv_(r2d)(iscclockwise,M)(vipm_Point_t(r2d) const aList[], vipm_int aNumPoints)
{
	return vipmi__gequ0d(vipm_Ptv_(r2d)(__sarea)(aList, aNumPoints));
}

/* ------------------------------------------------------------------------- */

# define vipm_r2d_ptv_centroid(r,sh,n)  vipm_r2d_point_mp(vipm_r2d_ptv_moment1((double *)(r),sh,n))

/* @} */


__MYC_END_CDECLS
# undef __PUB
# undef __PROT
# undef __PRIV


#endif /* !defined(__VIPM_MATH_GEOMETRY_2D_POINTLIST_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
