/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * BorderUtils.h	(V.Sokur)
 * Vipm/Math/Geometry/BorderUtils.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_MATH_GEOMETRY_BORDERUTILS_H__
# define __VIPM_MATH_GEOMETRY_BORDERUTILS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif

# ifndef __VIPM_MATH_UTILS_VECSPACEUTILS_H__
#  include <Vipm/Math/Utils/VecSpaceUtils.h>
# endif
# ifndef __VIPM_MATH_GEOMETRY_CORE_UTILS_H__
#  include <Vipm/Math/Geometry/Core_Utils.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * @note: min.unit/space-point is a pixel, not element
 *
 * A `rank` formal parameter [of type 'vipm_int'] does not comprise channel dimension. (?)
 */

/* ------------------------------------------------------------------------- */

# include <Vipm/Linkage.h>
# define __PUB  __myc_flinkage__(public,libVipm)
# define __VPUB __myc_vlinkage__(public,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Construct
 */

/*
 * brshape constructors take [geometric] values of border, not a vector values
 *
 * i.e. vipm_brshape(g2)(aThis, th, lw, bh, rw)
 * ------------------------------------------
 * |       ↑                                |
 * |       | th                             |
 * |       ↓                                |
 * | lw  ------------------------------     |
 * |<--->|        inner               | rw  |
 * |     |                            |<--->|
 * |     ------------------------------     |
 * |                               ↑        |
 * |                               | bh     |
 * |              outer            ↓        |
 * ------------------------------------------
 *
 */

__PUB vipm_brshape_t   *vipm_brshape()      __P((vipm_brshape_t *));
__PUB vipm_brshape_t   *vipm_brshape(g1)    __P((vipm_brshape_t *, vipm_int, vipm_int));
__PUB vipm_brshape_t   *vipm_brshape(g2)    __P((vipm_brshape_t *, vipm_int, vipm_int, vipm_int, vipm_int));
__PUB vipm_brshape_t   *vipm_brshape(g3)    __P((vipm_brshape_t *, vipm_int, vipm_int, vipm_int,
                                                                   vipm_int, vipm_int, vipm_int));
__PUB vipm_brshape_t   *vipm_brshape(gv)    __P((vipm_brshape_t *, vipm_int, vipm_int const [], vipm_int const []));
__PUB vipm_brshape_t   *vipm_brshape(gv,L)  __P((vipm_brshape_t *, vipm_rank_t, vipm_int const [], vipm_int const []));

__PUB vipm_brshape_t   *vipm_brshape(ois)   __P((vipm_brshape_t *, vipm_site_t const *, vipm_site_t const *));
__PUB vipm_brshape_t   *vipm_brshape(se)    __P((vipm_brshape_t *, vipm_seshape_t const *));

/* ------------------------------------------------------------------------- */

/*
 * Functional
 */

__PUB vipm_int          vipm_brshape_(empty)    __P((vipm_brshape_t const *));

__PUB vipm_brshape_t   *vipm_brshape_(zero)     __P((vipm_brshape_t *));

/* ------------------------------------------------------------------------- */

/*
 * compute brshape from inner and outer volumes
 * int vipm_brshape_(_compute,gv)(aDst, aRank, aStartOuter, aVolumeOuter, aStartInner, aVolumeInner)
 */
__PUB vipm_brshape_t   *vipm_brshape_(_compute,gv)  __P((vipm_brshape_t *, vipm_int,
                                                         vipm_int const [], vipm_int const [],
                                                         vipm_int const [], vipm_int const []));

/* ------------------------------------------------------------------------- */

/*
 * adjust: fixup brshape vectors
 * retval:bool = 1, if succes, 0 - otherwise
 */

__PUB int   vipm_brshape_(adjust)       __P((vipm_brshape_t *, vipm_brshape_t const *,
                                                 vipm_site_t const *, vipm_site_t const *));
__PUB int   vipm_brshape_(adjustI)      __P((vipm_brshape_t *, vipm_site_t const *, vipm_site_t const *));

/* ------------------------------------------------------------------------- */

__PUB int   vipm_brshape_(_adjust,gv)   __P((vipm_brshape_t *, vipm_brshape_t const *, vipm_int,
                                                 vipm_int const [], vipm_int const [],
                                                 vipm_int const [], vipm_int const []));
__PUB int   vipm_brshape_(_adjust,gv,I) __P((vipm_brshape_t *, vipm_int, vipm_int const [], vipm_int const [],
                                                 vipm_int const [], vipm_int const []));


/* ========================================================================= */

/*
 * Construct
 */

__PUB vipm_brparm_t    *vipm_brparm(se)     __P((vipm_brparm_t *, vipm_seshape_t const *));

# define VIPM_SE2BRP(sep,brp)       vipm_brparm(se)(brp,vipm_seshape_cp(sep))

/* ------------------------------------------------------------------------- */

/*
 * Functional
 */

__PUB vipm_int          vipm_brparm_(empty) __P((vipm_brparm_t const *));

__PUB vipm_brparm_t    *vipm_brparm_(zero)  __P((vipm_brparm_t *));

/* ------------------------------------------------------------------------- */

__PUB vipm_brparm_t    *vipm_brparm_(_compute,gv)   __P((vipm_brparm_t *, vipm_int,
                                                         vipm_int const [], vipm_int const [],
                                                         vipm_int const [], vipm_int const []));


/* ========================================================================= */

/*
 * vipm_int *vipm_Nvolume_({widen,narrow},br)   (aDest != nil, aSource, aBorder != nil)
 * vipm_int *vipm_Nvolume_({widen,narrow},I,br) (aDstSrc != nil, aBorder != nil)
 */

__PUB vipm_Nvolume_t   *vipm_Nvolume_(widen,br)     __P((vipm_Nvolume_t *, vipm_Nvolume_t const *,
                                                             vipm_brshape_t const *));
__PUB vipm_Nvolume_t   *vipm_Nvolume_(widen,I,br)   __P((vipm_Nvolume_t *, vipm_brshape_t const *));

__PUB vipm_Nvolume_t   *vipm_Nvolume_(narrow,br)    __P((vipm_Nvolume_t *, vipm_Nvolume_t const *,
                                                             vipm_brshape_t const *));
__PUB vipm_Nvolume_t   *vipm_Nvolume_(narrow,I,br)  __P((vipm_Nvolume_t *, vipm_brshape_t const *));

/* ------------------------------------------------------------------------- */

/*
 * vipm_int *vipm_volume_({_widen,_narrow},br)   (aRank, aDest != nil, aSource, aBorder != nil)
 * vipm_int *vipm_volume_({_widen,_narrow},I,br) (aRank, aDstSrc != nil, aBorder != nil)
 */

__PUB vipm_int *vipm_volume_(_widen,br)     __P((vipm_int, vipm_int [], vipm_int const [], vipm_brshape_t const *));
__PUB vipm_int *vipm_volume_(_widen,I,br)   __P((vipm_int, vipm_int [], vipm_brshape_t const *));
__PUB vipm_int *vipm_volume_(_narrow,br)    __P((vipm_int, vipm_int [], vipm_int const [], vipm_brshape_t const *));
__PUB vipm_int *vipm_volume_(_narrow,I,br)  __P((vipm_int, vipm_int [], vipm_brshape_t const *));


/* ========================================================================= */

/*
 * bo_size_t  vipm_vmemsize{L}_br (aRank, aVector, aStride{mutable}, aBorder, anExtraSize <- {0,1,2});
 * bo_size_t  vipm_vmemsizeI{L}_br(aRank, aVector{mutable}, aStride{mutable}, aBorder, anExtraSize <- {0,1,2});
 */

__PUB bo_size_t vipm_vmemsizeL_br   __P((vipm_rank_t, vipm_int const [], vipm_int [], vipm_brshape_t const *, vipm_int));
__PUB bo_size_t vipm_vmemsize_br    __P((vipm_rank_t, vipm_int const [], vipm_int [], vipm_brshape_t const *, vipm_int));

__PUB bo_size_t vipm_vmemsizeIL_br  __P((vipm_rank_t, vipm_int [], vipm_int [], vipm_brshape_t const *, vipm_int));
__PUB bo_size_t vipm_vmemsizeI_br   __P((vipm_rank_t, vipm_int [], vipm_int [], vipm_brshape_t const *, vipm_int));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

_VIPM_INLINE vipm_brshape_t *vipm_brshape()(vipm_brshape_t *aThis)
{
	(void)vipm_Nvector()(&aThis->br_X0);
	(void)vipm_Nvector()(&aThis->br_X1);
	return (aThis);
}

_VIPM_INLINE vipm_brshape_t *
vipm_brshape(g1)(vipm_brshape_t *aThis, vipm_int lw, vipm_int rw)
{
	(void)vipm_Nvector(g1)(&aThis->br_X0, -lw);
	(void)vipm_Nvector(g1)(&aThis->br_X1, rw);
	return (aThis);
}

_VIPM_INLINE vipm_brshape_t *
vipm_brshape(g2)(vipm_brshape_t *aThis, vipm_int th, vipm_int lw, vipm_int bh, vipm_int rw)
{
	(void)vipm_Nvector(g2)(&aThis->br_X0, -th, -lw);
	(void)vipm_Nvector(g2)(&aThis->br_X1, bh, rw);
	return (aThis);
}

_VIPM_INLINE vipm_brshape_t *
vipm_brshape(g3)(vipm_brshape_t *aThis, vipm_int z0, vipm_int th, vipm_int lw,
		vipm_int z1, vipm_int bh, vipm_int rw)
{
	(void)vipm_Nvector(g3)(&aThis->br_X0, -z0, -th, -lw);
	(void)vipm_Nvector(g3)(&aThis->br_X1, z1, bh, rw);
	return (aThis);
}


_VIPM_INLINE vipm_brshape_t *
vipm_brshape(gv)(vipm_brshape_t *aThis, vipm_int aRank, vipm_int const ltb[], vipm_int const rbb[])
{
	(void)vipm_vec_(neg,I)(VipmK_MAXGDIMS, vipm_Nvector(gv)(&aThis->br_X0, aRank, ltb)->v);
	(void)vipm_Nvector(gv)(&aThis->br_X1, aRank, rbb);
	return (aThis);
}

_VIPM_INLINE vipm_brshape_t *
vipm_brshape(gv,L)(vipm_brshape_t *aThis, vipm_rank_t aRank, vipm_int const ltb[], vipm_int const rbb[])
{
	(void)vipm_vec_(neg,I)(VipmK_MAXGDIMS, vipm_Nvector(gv,L)(&aThis->br_X0, aRank, ltb)->v);
	(void)vipm_Nvector(gv,L)(&aThis->br_X1, aRank, rbb);
	return (aThis);
}


_VIPM_INLINE vipm_brshape_t *
vipm_brshape(ois)(vipm_brshape_t *aThis, vipm_site_t const *anOuter, vipm_site_t const *anInner)
{
	return vipm_brshape_(_compute,gv)(aThis, VipmK_MAXGDIMS,
		anOuter->si_point.v, anOuter->si_volume.v, anInner->si_point.v, anInner->si_volume.v);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_brshape_(empty)(vipm_brshape_t const *aThis)
{
	return (vipm_vec_(_equal0)(VipmK_MAXGDIMS, aThis->br_X0.v) &&
		vipm_vec_(_equal0)(VipmK_MAXGDIMS, aThis->br_X1.v));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_brshape_t *
vipm_brshape_(zero)(vipm_brshape_t *aThis)
{
	(void)vipm_Nvector_(zero,g)(&aThis->br_X0);
	(void)vipm_Nvector_(zero,g)(&aThis->br_X1);
	return (aThis);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_brshape_(adjust)(vipm_brshape_t *aDst, vipm_brshape_t const *aSrc,
		vipm_site_t const *anOuter, vipm_site_t const *anInner)
{
	return vipm_brshape_(_adjust,gv)(aDst, aSrc, VipmK_MAXGDIMS,
		anOuter->si_point.v, anOuter->si_volume.v, anInner->si_point.v, anInner->si_volume.v);
}

_VIPM_INLINE int
vipm_brshape_(adjust,I)(vipm_brshape_t *aDstSrc, vipm_site_t const *anOuter, vipm_site_t const *anInner)
{
	return vipm_brshape_(_adjust,gv,I)(aDstSrc, VipmK_MAXGDIMS,
		anOuter->si_point.v, anOuter->si_volume.v, anInner->si_point.v, anInner->si_volume.v);
}


/* ========================================================================= */

_VIPM_INLINE vipm_brparm_t *
vipm_brparm(se)(vipm_brparm_t *aThis, vipm_seshape_t const *aStrel)
{
	return vipm_brparm_mp((vipm_brshape(se))(vipm_brshape_mp(aThis), aStrel));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_brparm_(empty)(vipm_brparm_t const *aThis)
{
	return vipm_brshape_(empty)(vipm_brshape_p(aThis));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_brparm_t *
vipm_brparm_(zero)(vipm_brparm_t *aThis)
{
	return vipm_brparm_mp(vipm_brshape_(zero)(vipm_brshape_mp(aThis)));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_brparm_t *
vipm_brparm_(_compute,gv)(vipm_brparm_t *aThis, vipm_int aRank,
		vipm_int const aStartOuter[], vipm_int const aVolumeOuter[],
		vipm_int const aStartInner[], vipm_int const aVolumeInner[])
{
	return vipm_brparm_mp((vipm_brshape_(_compute,gv))(vipm_brshape_mp(aThis), aRank,
		aStartOuter, aVolumeOuter, aStartInner, aVolumeInner));
}


/* ========================================================================= */

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(widen,I,br)(vipm_Nvolume_t *aResult, vipm_brshape_t const *aBorder)
{
	(void)vipm_volume_(_widen,I,br)(VipmK_MAXGDIMS, aResult->v, aBorder);
	return (aResult);
}

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(widen,br)(vipm_Nvolume_t *aResult, vipm_Nvolume_t const *aVolume, vipm_brshape_t const *aBorder)
{
	(void)vipm_volume_(_widen,br)(VipmK_MAXGDIMS, aResult->v, (NULL != aVolume ? aVolume->v : NULL), aBorder);
	return (aResult);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(narrow,I,br)(vipm_Nvolume_t *aResult, vipm_brshape_t const *aBorder)
{
	(void)vipm_volume_(_narrow,I,br)(VipmK_MAXGDIMS, aResult->v, aBorder);
	return (aResult);
}

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(narrow,br)(vipm_Nvolume_t *aResult, vipm_Nvolume_t const *aVolume, vipm_brshape_t const *aBorder)
{
	(void)vipm_volume_(_narrow,br)(VipmK_MAXGDIMS, aResult->v, (NULL != aVolume ? aVolume->v : NULL), aBorder);
	return (aResult);
}


/* ========================================================================= */

_VIPM_INLINE bo_size_t
vipm_vmemsizeL_br(vipm_rank_t aRank, vipm_int const aVolume[], vipm_int aStride[],
		vipm_brshape_t const *aBorder, vipm_int anExtraSz)
{
	vipm_int vol[2 * VipmK_MAXDIMS];
    return vipm_vmemsizeL(aRank,
		vipm_volume_(_widen,I,br)(VIPM_ERANK(aRank) - 1,
				vipm_vec_(assign,L)(aRank, vol + VipmK_MAXDIMS, aVolume), aBorder),
		aStride, anExtraSz);
}

_VIPM_INLINE bo_size_t
vipm_vmemsize_br(vipm_rank_t aRank, vipm_int const aVolume[], vipm_int aStride[],
		vipm_brshape_t const *aBorder, vipm_int anExtraSz)
{
	vipm_int rank = VIPM_ERANK(aRank);
	vipm_int vol[VipmK_MAXDIMS];
    return vipm_vmemsize(aRank,
		vipm_volume_(_widen,I,br)(rank - 1, vipm_vec_(_assign)(rank, vol, aVolume), aBorder),
		aStride, anExtraSz);
}


_VIPM_INLINE bo_size_t
vipm_vmemsizeIL_br(vipm_rank_t aRank, vipm_int aVolume[], vipm_int aStride[],
		vipm_brshape_t const *aBorder, vipm_int anExtraSz)
{
    return vipm_vmemsizeL(aRank,
		vipm_volume_(_widen,I,br)(VIPM_ERANK(aRank) - 1, aVolume, aBorder),
		aStride, anExtraSz
	);
}

_VIPM_INLINE bo_size_t
vipm_vmemsizeI_br(vipm_rank_t aRank, vipm_int aVolume[], vipm_int aStride[],
		vipm_brshape_t const *aBorder, vipm_int anExtraSz)
{
    return vipm_vmemsize(aRank,
		vipm_volume_(_widen,I,br)(VIPM_ERANK(aRank) - 1, aVolume, aBorder),
		aStride, anExtraSz
	);
}


__MYC_END_CDECLS
# undef __PUB
# undef __VPUB


#endif /* !defined(__VIPM_MATH_GEOMETRY_BORDERUTILS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
