/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * StrelUtils.h	(V.Sokur)
 * Vipm/Math/Geometry/StrelUtils.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_MATH_GEOMETRY_STRELUTILS_H__
# define __VIPM_MATH_GEOMETRY_STRELUTILS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif

# ifndef __VIPM_MATH_UTILS_VECSPACEUTILS_H__
#  include <Vipm/Math/Utils/VecSpaceUtils.h>
# endif
# ifndef __VIPM_MATH_GEOMETRY_CORE_UTILS_H__
#  include <Vipm/Math/Geometry/Core_Utils.h>
# endif

# ifndef __VIPM_UTILS_ARRAY_H__
#  include <Vipm/Utils/Array.h>
# endif
# ifndef __VIPM_UTILS_EXTRAS_H__
#  include <Vipm/Utils/Extras.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * @note: min.unit/space-point is a pixel, not element
 *
 * A `rank` formal parameter [of type 'vipm_int'] does not comprise channel dimension. (?)
 */

/* ------------------------------------------------------------------------- */

# include <Vipm/Linkage.h>
# define __PUB  __myc_flinkage__(public,libVipm)
# define __VPUB __myc_vlinkage__(public,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Construct
 */

__PUB vipm_seshape_t   *vipm_seshape()      __P((vipm_seshape_t *));
__PUB vipm_seshape_t   *vipm_seshape(g1)    __P((vipm_seshape_t *, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape(g11)   __P((vipm_seshape_t *, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape(g2)    __P((vipm_seshape_t *, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape(g22)   __P((vipm_seshape_t *, vipm_int, vipm_int, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape(g3)    __P((vipm_seshape_t *, vipm_int, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape(g33)   __P((vipm_seshape_t *, vipm_int, vipm_int, vipm_int,
                                                                   vipm_int, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape(gv)    __P((vipm_seshape_t *, vipm_int, vipm_int const []));
__PUB vipm_seshape_t   *vipm_seshape(gvv)   __P((vipm_seshape_t *, vipm_int, vipm_int const [], vipm_int const []));
__PUB vipm_seshape_t   *vipm_seshape(gv,L)  __P((vipm_seshape_t *, vipm_rank_t, vipm_int const []));
__PUB vipm_seshape_t   *vipm_seshape(gvv,L) __P((vipm_seshape_t *, vipm_rank_t, vipm_int const [], vipm_int const []));

/* ------------------------------------------------------------------------- */

/*
 * Functional
 */

__PUB int               vipm_seshape_(empty)        __P((vipm_seshape_t const *));
__PUB vipm_int          vipm_seshape_(capacity)     __P((vipm_seshape_t const *));
__PUB vipm_int          vipm_seshape_(capacity,Q)   __P((vipm_seshape_t const *));


__PUB vipm_seshape_t   *vipm_seshape_(zero)         __P((vipm_seshape_t *));
__PUB vipm_seshape_t   *vipm_seshape_(azero)        __P((vipm_seshape_t *));
__PUB vipm_seshape_t   *vipm_seshape_(vzero)        __P((vipm_seshape_t *));

__PUB vipm_seshape_t   *vipm_seshape_(setvol,g1)    __P((vipm_seshape_t *, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape_(setvol,g2)    __P((vipm_seshape_t *, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape_(setvol,g3)    __P((vipm_seshape_t *, vipm_int, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape_(setvol,gv)    __P((vipm_seshape_t *, vipm_int, vipm_int const []));
__PUB vipm_seshape_t   *vipm_seshape_(setvol,gv,L)  __P((vipm_seshape_t *, vipm_rank_t, vipm_int const []));

/*
 * rstanchor: reset anchor as half of volume
 */
__PUB vipm_seshape_t   *vipm_seshape_(rstanchor)    __P((vipm_seshape_t *));
__PUB vipm_seshape_t   *vipm_seshape_(rstanchor,g1) __P((vipm_seshape_t *));
__PUB vipm_seshape_t   *vipm_seshape_(rstanchor,g2) __P((vipm_seshape_t *));
__PUB vipm_seshape_t   *vipm_seshape_(rstanchor,g3) __P((vipm_seshape_t *));
__PUB vipm_seshape_t   *vipm_seshape_(rstanchor,gn) __P((vipm_seshape_t *, vipm_int));

/*
 * xsetvol: rstanchor(setvol)
 */
__PUB vipm_seshape_t   *vipm_seshape_(xsetvol,g1)   __P((vipm_seshape_t *, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape_(xsetvol,g2)   __P((vipm_seshape_t *, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape_(xsetvol,g3)   __P((vipm_seshape_t *, vipm_int, vipm_int, vipm_int));
__PUB vipm_seshape_t   *vipm_seshape_(xsetvol,gv)   __P((vipm_seshape_t *, vipm_int, vipm_int const []));
__PUB vipm_seshape_t   *vipm_seshape_(xsetvol,gv,L) __P((vipm_seshape_t *, vipm_rank_t, vipm_int const []));


/* ------------------------------------------------------------------------- */

/*
 * adjust: fixup seshape anchor and volume
 * retval:bool = 1, if succes, 0 - otherwise
 */

__PUB int   vipm_seshape_(adjust)       __P((vipm_seshape_t *, vipm_seshape_t const *, vipm_Nvolume_t const *));
__PUB int   vipm_seshape_(adjust,I)     __P((vipm_seshape_t *, vipm_Nvolume_t const *));

__PUB int   vipm_seshape_(adjust,g2)    __P((vipm_seshape_t *, vipm_seshape_t const *, vipm_int, vipm_int));
__PUB int   vipm_seshape_(adjust,g2,I)  __P((vipm_seshape_t *, vipm_int, vipm_int));

/* ------------------------------------------------------------------------- */

__PUB int   vipm_seshape_(_adjust,gv)   __P((vipm_seshape_t *, vipm_seshape_t const *, vipm_int, vipm_int const []));
__PUB int   vipm_seshape_(_adjust,gv,I) __P((vipm_seshape_t *, vipm_int, vipm_int const []));


/* ========================================================================= */

/*
 * Functional
 */

	/*
	 * retval:bool
	 */
__PUB int               vipm_strel_(empty)          __P((vipm_strel_t const *));
__PUB int               vipm_strel_(issolid,v255)   __P((vipm_strel_t const *));
__PUB int               vipm_sea_issolid_v255       __P((struct _Vipm_array const *));

__PUB vipm_int          vipm_strel_(capacity)       __P((vipm_strel_t const *));
__PUB vipm_int          vipm_strel_(capacity,Q)     __P((vipm_strel_t const *));


__PUB vipm_strel_t     *vipm_strel_(azero)          __P((vipm_strel_t *));

__PUB vipm_strel_t     *vipm_strel_(rstanchor)      __P((vipm_strel_t *));
__PUB vipm_strel_t     *vipm_strel_(rstanchor,g1)   __P((vipm_strel_t *));
__PUB vipm_strel_t     *vipm_strel_(rstanchor,g2)   __P((vipm_strel_t *));
__PUB vipm_strel_t     *vipm_strel_(rstanchor,g3)   __P((vipm_strel_t *));
__PUB vipm_strel_t     *vipm_strel_(rstanchor,gn)   __P((vipm_strel_t *));

/* ------------------------------------------------------------------------- */

/*
 * retval:bool = 1, if succes, 0 - otherwise
 */

__PUB int   vipm_strel_(adjust)         __P((vipm_strel_t *, vipm_strel_t const *, vipm_Nvolume_t const *));
__PUB int   vipm_strel_(adjust,I)       __P((vipm_strel_t *, vipm_Nvolume_t const *));

__PUB int   vipm_strel_(adjust,g2)      __P((vipm_strel_t *, vipm_strel_t const *, vipm_int, vipm_int));
__PUB int   vipm_strel_(adjust,g2,I)    __P((vipm_strel_t *, vipm_int, vipm_int));

/* ------------------------------------------------------------------------- */

__PUB int   vipm_strel_(_adjust,gv)     __P((vipm_strel_t *, vipm_strel_t const *, vipm_int, vipm_int const []));
__PUB int   vipm_strel_(_adjust,gv,I)   __P((vipm_strel_t *, vipm_int, vipm_int const []));


/* ========================================================================= */

/*
 * vipm_int *vipm_Nvolume_({widen,narrow},se)   (aDest != nil, aSource, aBorder != nil)
 * vipm_int *vipm_Nvolume_({widen,narrow},I,se) (aDstSrc != nil, aBorder != nil)
 */

__PUB vipm_Nvolume_t   *vipm_Nvolume_(widen,se)     __P((vipm_Nvolume_t *, vipm_Nvolume_t const *,
                                                             vipm_seshape_t const *));
__PUB vipm_Nvolume_t   *vipm_Nvolume_(widen,I,se)   __P((vipm_Nvolume_t *, vipm_seshape_t const *));

__PUB vipm_Nvolume_t   *vipm_Nvolume_(narrow,se)    __P((vipm_Nvolume_t *, vipm_Nvolume_t const *,
                                                             vipm_seshape_t const *));
__PUB vipm_Nvolume_t   *vipm_Nvolume_(narrow,I,se)  __P((vipm_Nvolume_t *, vipm_seshape_t const *));

/* ------------------------------------------------------------------------- */

/*
 * vipm_int *vipm_volume_({_widen,_narrow},se)   (aRank, aDest != nil, aSource, aBorder != nil)
 * vipm_int *vipm_volume_({_widen,_narrow},I,se) (aRank, aDstSrc != nil, aBorder != nil)
 */

__PUB vipm_int *vipm_volume_(_widen,se)     __P((vipm_int, vipm_int [], vipm_int const [], vipm_seshape_t const *));
__PUB vipm_int *vipm_volume_(_widen,I,se)   __P((vipm_int, vipm_int [], vipm_seshape_t const *));
__PUB vipm_int *vipm_volume_(_narrow,se)    __P((vipm_int, vipm_int [], vipm_int const [], vipm_seshape_t const *));
__PUB vipm_int *vipm_volume_(_narrow,I,se)  __P((vipm_int, vipm_int [], vipm_seshape_t const *));


/* ========================================================================= */

/*
 * bo_size_t  vipm_vmemsize{[L,]se} (aRank, aVector, aStride{mutable}, aStrel, anExtraSize <- {0,1,2});
 * bo_size_t  vipm_vmemsize{I[L],se}(aRank, aVector{mutable}, aStride{mutable}, aStrel, anExtraSize <- {0,1,2});
 */

__PUB bo_size_t vipm_NS(vmemsize,L,se)  __P((vipm_rank_t, vipm_int const [], vipm_int [], vipm_seshape_t const *, vipm_int));
__PUB bo_size_t vipm_NS(vmemsize,se)    __P((vipm_rank_t, vipm_int const [], vipm_int [], vipm_seshape_t const *, vipm_int));

__PUB bo_size_t vipm_NS(vmemsize,IL,se) __P((vipm_rank_t, vipm_int [], vipm_int [], vipm_seshape_t const *, vipm_int));
__PUB bo_size_t vipm_NS(vmemsize,I,se)  __P((vipm_rank_t, vipm_int [], vipm_int [], vipm_seshape_t const *, vipm_int));


/* ========================================================================= */

/* Options:
 * VipmF_ISOLATED_BORDER    --  the function always returns false
 * VipmF_ISOLATED_PARENT
 *
 * retval: bool
 */

/*
 * int vipm_NS(compute_roi,se)  (aRoiAvail, aRoiReq, Options, aRoiInput, aStrel, anActualArrSite, aParentArrSite);
 * int vipm_NS(testfiltop,se)   (aRoiAvail, aRoiReq, Options, aRoiInput, aStrel, aWholeArrShape);
 *
 * int vipm_NS(_compute_roi,se) (Options, aRank, aStartAvail[], aVolumeAvail[], aStartReq[], aVolumeReq[],
 *                               aStartInput[], aVolumeInput[], aStrel, anActualArrSite, aParentArrSite);
 * int vipm_NS(_testfiltop,se)  (Options, aRank, aStartAvail[], aVolumeAvail[], aStartReq[], aVolumeReq[],
 *                               aStartInput[], aVolumeInput[], aStrel, aWholeArrShape);
 *
 *
 * @note: the 'compute-roi' functions do not check a validity of an actual-arr-site
 *        or a parent-arr-site (i.e.: nil pointer, empty site).
 * @note: the 'testfiltop' functions use 'compute-roi' functions internally.
 */

__PUB int   vipm_NS(compute_roi,se) __P((vipm_site_t *, vipm_site_t *, vipm_ulong, vipm_site_t const *,
                                             vipm_seshape_t const *, vipm_site_t const *, vipm_site_t const *));
__PUB int   vipm_NS(testfiltop,se)  __P((vipm_site_t *, vipm_site_t *, vipm_ulong, vipm_site_t const *,
                                             vipm_seshape_t const *, vipm_arrshape_t const *));

/* ------------------------------------------------------------------------- */

__PUB int   vipm_NS(_compute_roi,se)__P((vipm_ulong, vipm_int, vipm_int [], vipm_int [], vipm_int [], vipm_int [],
                                             vipm_int const [], vipm_int const [], vipm_seshape_t const *,
                                             vipm_site_t const *, vipm_site_t const *));
__PUB int   vipm_NS(_testfiltop,se) __P((vipm_ulong, vipm_int, vipm_int [], vipm_int [], vipm_int [], vipm_int [],
                                             vipm_int const [], vipm_int const [], vipm_seshape_t const *,
                                             vipm_arrshape_t const *));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(zero)(vipm_seshape_t *aThis)
{
	(void)vipm_Npoint_(zero,g)(&aThis->se_anchor);
	(void)vipm_Nvolume_(zero,g)(&aThis->se_volume);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(azero)(vipm_seshape_t *aThis)
{
	(void)vipm_Npoint_(zero,g)(&aThis->se_anchor);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(vzero)(vipm_seshape_t *aThis)
{
	(void)vipm_Nvolume_(zero,g)(&aThis->se_volume);
	return (aThis);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(setvol,g1)(vipm_seshape_t *aThis, vipm_int X)
{
	(void)vipm_Nvolume(g1)(&aThis->se_volume, X);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(setvol,g2)(vipm_seshape_t *aThis, vipm_int Y, vipm_int X)
{
	(void)vipm_Nvolume(g2)(&aThis->se_volume, Y, X);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(setvol,g3)(vipm_seshape_t *aThis, vipm_int Z, vipm_int Y, vipm_int X)
{
	(void)vipm_Nvolume(g3)(&aThis->se_volume, Z, Y, X);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(setvol,gv)(vipm_seshape_t *aThis, vipm_rank_t aRank, vipm_int const aVolume[])
{
	(void)vipm_Nvolume(gv)(&aThis->se_volume, aRank, aVolume);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(setvol,gv,L)(vipm_seshape_t *aThis, vipm_rank_t aRank, vipm_int const aVolume[])
{
	(void)vipm_Nvolume(gv,L)(&aThis->se_volume, aRank, aVolume);
	return (aThis);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(rstanchor)(vipm_seshape_t *aThis)
{
	return (vipm_seshape_(rstanchor,gn)(aThis, VipmK_MAXGDIMS));
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(rstanchor,g1)(vipm_seshape_t *aThis)
{
	vipm_pt_x(aThis->se_anchor) = vipm_pt_x(aThis->se_volume) / 2;
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(rstanchor,g2)(vipm_seshape_t *aThis)
{
	vipm_pt_y(aThis->se_anchor) = vipm_pt_y(aThis->se_volume) / 2;
	vipm_pt_x(aThis->se_anchor) = vipm_pt_x(aThis->se_volume) / 2;
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(rstanchor,g3)(vipm_seshape_t *aThis)
{
	vipm_pt_z(aThis->se_anchor) = vipm_pt_z(aThis->se_volume) / 2;
	vipm_pt_y(aThis->se_anchor) = vipm_pt_y(aThis->se_volume) / 2;
	vipm_pt_x(aThis->se_anchor) = vipm_pt_x(aThis->se_volume) / 2;
	return (aThis);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_seshape_t *vipm_seshape()(vipm_seshape_t *aThis)
{
	(void)vipm_Npoint(g2)(&aThis->se_anchor, VipmK_SE_DFL_HEIGHT / 2, VipmK_SE_DFL_WIDTH / 2);
	(void)vipm_Nvolume(g2)(&aThis->se_volume, VipmK_SE_DFL_HEIGHT, VipmK_SE_DFL_WIDTH);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(g1)(vipm_seshape_t *aThis, vipm_int dx)
{
	(void)vipm_Npoint(g1)(&aThis->se_anchor, dx / 2);
	(void)vipm_Nvolume(g1)(&aThis->se_volume, dx);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(g11)(vipm_seshape_t *aThis, vipm_int x, vipm_int dx)
{
	(void)vipm_Npoint(g1)(&aThis->se_anchor, x);
	(void)vipm_Nvolume(g1)(&aThis->se_volume, dx);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(g2)(vipm_seshape_t *aThis, vipm_int dy, vipm_int dx)
{
	(void)vipm_Npoint(g2)(&aThis->se_anchor, dy / 2, dx / 2);
	(void)vipm_Nvolume(g2)(&aThis->se_volume, dy, dx);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(g22)(vipm_seshape_t *aThis, vipm_int y, vipm_int x, vipm_int dy, vipm_int dx)
{
	(void)vipm_Npoint(g2)(&aThis->se_anchor, y, x);
	(void)vipm_Nvolume(g2)(&aThis->se_volume, dy, dx);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(g3)(vipm_seshape_t *aThis, vipm_int dz, vipm_int dy, vipm_int dx)
{
	(void)vipm_Npoint(g3)(&aThis->se_anchor, dz / 2, dy / 2, dx / 2);
	(void)vipm_Nvolume(g3)(&aThis->se_volume, dz, dy, dx);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(g33)(vipm_seshape_t *aThis, vipm_int z, vipm_int y, vipm_int x,
                                        vipm_int dz, vipm_int dy, vipm_int dx)
{
	(void)vipm_Npoint(g3)(&aThis->se_anchor, z, y, x);
	(void)vipm_Nvolume(g3)(&aThis->se_volume, dz, dy, dx);
	return (aThis);
}


_VIPM_INLINE vipm_seshape_t *
vipm_seshape(gv)(vipm_seshape_t *aThis, vipm_int rank, vipm_int const vol[])
{
	return (vipm_seshape_(rstanchor)(vipm_seshape_(setvol,gv)(aThis, rank, vol)));
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(gvv)(vipm_seshape_t *aThis, vipm_int rank, vipm_int const x0[], vipm_int const vol[])
{
	(void)vipm_Npoint(gv)(&aThis->se_anchor, rank, x0);
	(void)vipm_Nvolume(gv)(&aThis->se_volume, rank, vol);
	return (aThis);
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(gv,L)(vipm_seshape_t *aThis, vipm_int rank, vipm_int const vol[])
{
	return (vipm_seshape_(rstanchor)(vipm_seshape_(setvol,gv,L)(aThis, rank, vol)));
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape(gvv,L)(vipm_seshape_t *aThis, vipm_int rank, vipm_int const x0[], vipm_int const vol[])
{
	(void)vipm_Npoint(gv,L)(&aThis->se_anchor, rank, x0);
	(void)vipm_Nvolume(gv,L)(&aThis->se_volume, rank, vol);
	return (aThis);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_seshape_(empty)(vipm_seshape_t const *aThis)
{
	return vipm_Nvolume_(empty,g)(&aThis->se_volume);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_int
vipm_seshape_(capacity)(vipm_seshape_t const *aThis)
{
	return (vipm_volume_(_capacity)(VipmK_MAXGDIMS, aThis->se_volume.v));
}

_VIPM_INLINE vipm_int
vipm_seshape_(capacity,Q)(vipm_seshape_t const *aThis)
{
	return (vipm_volume_(_capacity,Q)(VipmK_MAXGDIMS, aThis->se_volume.v));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(xsetvol,g1)(vipm_seshape_t *aThis, vipm_int X)
{
	return (vipm_seshape(g1)(aThis, X));
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(xsetvol,g2)(vipm_seshape_t *aThis, vipm_int Y, vipm_int X)
{
	return (vipm_seshape(g2)(aThis, Y, X));
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(xsetvol,g3)(vipm_seshape_t *aThis, vipm_int Z, vipm_int Y, vipm_int X)
{
	return (vipm_seshape(g3)(aThis, Z, Y, X));
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(xsetvol,gv)(vipm_seshape_t *aThis, vipm_rank_t aRank, vipm_int const aVolume[])
{
	return (vipm_seshape(gv)(aThis, aRank, aVolume));
}

_VIPM_INLINE vipm_seshape_t *
vipm_seshape_(xsetvol,gv,L)(vipm_seshape_t *aThis, vipm_rank_t aRank, vipm_int const aVolume[])
{
	return (vipm_seshape(gv,L)(aThis, aRank, aVolume));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_seshape_(adjust)(vipm_seshape_t *aDst, vipm_seshape_t const *aSrc, vipm_Nvolume_t const *aVolume)
{
	return vipm_seshape_(_adjust,gv)(aDst, aSrc, VipmK_MAXGDIMS, aVolume->v);
}

_VIPM_INLINE int
vipm_seshape_(adjust,I)(vipm_seshape_t *aDstSrc, vipm_Nvolume_t const *aVolume)
{
	return vipm_seshape_(_adjust,gv,I)(aDstSrc, VipmK_MAXGDIMS, aVolume->v);
}


_VIPM_INLINE int
vipm_seshape_(adjust,g2)(vipm_seshape_t *aDst, vipm_seshape_t const *aSrc, vipm_int aHeight, vipm_int aWidth)
{
	vipm_int volume[2] = { aHeight, aWidth };
	return vipm_seshape_(_adjust,gv)(aDst, aSrc, 2, volume);
}

_VIPM_INLINE int
vipm_seshape_(adjust,g2,I)(vipm_seshape_t *aDstSrc, vipm_int aHeight, vipm_int aWidth)
{
	vipm_int volume[2] = { aHeight, aWidth };
	return vipm_seshape_(_adjust,gv,I)(aDstSrc, 2, volume);
}


/* ========================================================================= */

_VIPM_INLINE int
vipm_strel_(empty)(vipm_strel_t const *aThis)
{
	return (vipm_seshape_(empty)(vipm_seshape_p(aThis)));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_int
vipm_strel_(capacity)(vipm_strel_t const *aThis)
{
	return (vipm_seshape_(capacity)(vipm_seshape_p(aThis)));
}

_VIPM_INLINE vipm_int
vipm_strel_(capacity,Q)(vipm_strel_t const *aThis)
{
	return (vipm_seshape_(capacity,Q)(vipm_seshape_p(aThis)));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_strel_(issolid,v255)(vipm_strel_t const *aThis)
{
	return (vipm_sea_issolid_v255(&aThis->se_arr));
}

_VIPM_INLINE int
vipm_sea_issolid_v255(struct _Vipm_array const *aStrelArr)
{
	return (vipm_memcelem2dS(aStrelArr->ipar_elemtype,
		vipm_arr_(base)(aStrelArr), aStrelArr->ipar_rowstride,
		aStrelArr->ipar_height, aStrelArr->ipar_width * aStrelArr->ipar_nchannels, 255
	) < 0);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_strel_t *
vipm_strel_(azero)(vipm_strel_t *aThis)
{
	return (vipm_strel_mp(vipm_seshape_(azero)(vipm_seshape_mp(aThis))));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_strel_t *
vipm_strel_(rstanchor)(vipm_strel_t *aThis)
{
	return (vipm_strel_mp(vipm_seshape_(rstanchor)(vipm_seshape_mp(aThis))));
}

_VIPM_INLINE vipm_strel_t *
vipm_strel_(rstanchor,g1)(vipm_strel_t *aThis)
{
	return (vipm_strel_mp(vipm_seshape_(rstanchor,g1)(vipm_seshape_mp(aThis))));
}

_VIPM_INLINE vipm_strel_t *
vipm_strel_(rstanchor,g2)(vipm_strel_t *aThis)
{
	return (vipm_strel_mp(vipm_seshape_(rstanchor,g2)(vipm_seshape_mp(aThis))));
}

_VIPM_INLINE vipm_strel_t *
vipm_strel_(rstanchor,g3)(vipm_strel_t *aThis)
{
	return (vipm_strel_mp(vipm_seshape_(rstanchor,g3)(vipm_seshape_mp(aThis))));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int
vipm_strel_(adjust)(vipm_strel_t *aDst, vipm_strel_t const *aSrc, vipm_Nvolume_t const *aVolume)
{
	return vipm_seshape_(adjust)(vipm_seshape_mp(aDst), vipm_seshape_p(aSrc), aVolume);
}

_VIPM_INLINE int
vipm_strel_(adjust,I)(vipm_strel_t *aDstSrc, vipm_Nvolume_t const *aVolume)
{
	return vipm_seshape_(adjust,I)(vipm_seshape_mp(aDstSrc), aVolume);
}


_VIPM_INLINE int
vipm_strel_(adjust,g2)(vipm_strel_t *aDst, vipm_strel_t const *aSrc, vipm_int aHeight, vipm_int aWidth)
{
	return vipm_seshape_(adjust,g2)(vipm_seshape_mp(aDst), vipm_seshape_p(aSrc), aHeight, aWidth);
}

_VIPM_INLINE int
vipm_strel_(adjust,g2,I)(vipm_strel_t *aDstSrc, vipm_int aHeight, vipm_int aWidth)
{
	return vipm_seshape_(adjust,g2,I)(vipm_seshape_mp(aDstSrc), aHeight, aWidth);
}


_VIPM_INLINE int
vipm_strel_(_adjust,gv)(vipm_strel_t *aDst, vipm_strel_t const *aSrc, vipm_int aRank, vipm_int const aVolume[])
{
	return vipm_seshape_(_adjust,gv)(vipm_seshape_mp(aDst), vipm_seshape_p(aSrc), aRank, aVolume);
}

_VIPM_INLINE int
vipm_strel_(_adjust,gv,I)(vipm_strel_t *aDstSrc, vipm_int aRank, vipm_int const aVolume[])
{
	return vipm_seshape_(_adjust,gv,I)(vipm_seshape_mp(aDstSrc), aRank, aVolume);
}


/* ========================================================================= */

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(widen,se)(vipm_Nvolume_t *aResult, vipm_Nvolume_t const *aVolume, vipm_seshape_t const *aStrel)
{
	(void)vipm_volume_(_widen,se)(VipmK_MAXGDIMS, aResult->v, (NULL != aVolume ? aVolume->v : NULL), aStrel);
	return (aResult);
}

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(widen,I,se)(vipm_Nvolume_t *aResult, vipm_seshape_t const *aStrel)
{
	(void)vipm_volume_(_widen,I,se)(VipmK_MAXGDIMS, aResult->v, aStrel);
	return (aResult);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(narrow,se)(vipm_Nvolume_t *aResult, vipm_Nvolume_t const *aVolume, vipm_seshape_t const *aStrel)
{
	(void)vipm_volume_(_narrow,se)(VipmK_MAXGDIMS, aResult->v, (NULL != aVolume ? aVolume->v : NULL), aStrel);
	return (aResult);
}

_VIPM_INLINE vipm_Nvolume_t *
vipm_Nvolume_(narrow,I,se)(vipm_Nvolume_t *aResult, vipm_seshape_t const *aStrel)
{
	(void)vipm_volume_(_narrow,I,se)(VipmK_MAXGDIMS, aResult->v, aStrel);
	return (aResult);
}


/* ========================================================================= */

_VIPM_INLINE bo_size_t
vipm_NS(vmemsize,L,se)(vipm_rank_t aRank, vipm_int const aVolume[], vipm_int aStride[],
		vipm_seshape_t const *aStrel, vipm_int anExtraSz)
{
	vipm_int vol[2 * VipmK_MAXDIMS];
    return vipm_vmemsizeL(aRank,
		vipm_volume_(_widen,I,se)(VIPM_ERANK(aRank) - 1,
				vipm_vec_(assign,L)(aRank, vol + VipmK_MAXDIMS, aVolume), aStrel),
		aStride, anExtraSz
	);
}

_VIPM_INLINE bo_size_t
vipm_NS(vmemsize,se)(vipm_rank_t aRank, vipm_int const aVolume[], vipm_int aStride[],
		vipm_seshape_t const *aStrel, vipm_int anExtraSz)
{
	vipm_int rank = VIPM_ERANK(aRank);
	vipm_int vol[VipmK_MAXDIMS];
    return vipm_vmemsize(aRank,
		vipm_volume_(_widen,I,se)(rank - 1, vipm_vec_(_assign)(rank, vol, aVolume), aStrel),
		aStride, anExtraSz
	);
}


_VIPM_INLINE bo_size_t
vipm_NS(vmemsize,IL,se)(vipm_rank_t aRank, vipm_int aVolume[], vipm_int aStride[],
		vipm_seshape_t const *aStrel, vipm_int anExtraSz)
{
    return vipm_vmemsizeL(aRank,
		vipm_volume_(_widen,I,se)(VIPM_ERANK(aRank) - 1, aVolume, aStrel),
		aStride, anExtraSz
	);
}

_VIPM_INLINE bo_size_t
vipm_NS(vmemsize,I,se)(vipm_rank_t aRank, vipm_int aVolume[], vipm_int aStride[],
		vipm_seshape_t const *aStrel, vipm_int anExtraSz)
{
    return vipm_vmemsize(aRank,
		vipm_volume_(_widen,I,se)(VIPM_ERANK(aRank) - 1, aVolume, aStrel),
		aStride, anExtraSz
	);
}


/* ========================================================================= */

_VIPM_INLINE int
vipm_NS(testfiltop,se)(vipm_site_t *aRoiAvail, vipm_site_t *aRoiReq, vipm_ulong Options,
		vipm_site_t const *aRoiInput, vipm_seshape_t const *aStrel, vipm_arrshape_t const *aWholeArrShape)
{
	return vipm_NS(compute_roi,se)(
		aRoiAvail, aRoiReq, Options,
		aRoiInput, aStrel, (NULL != aWholeArrShape ? &aWholeArrShape->ars_aasite : NULL),
		(NULL != aWholeArrShape ? &aWholeArrShape->ars_pasite : NULL)
	);
}

_VIPM_INLINE int
vipm_NS(_testfiltop,se)(vipm_ulong Options, vipm_int aRank,
		vipm_int aRoiStartAvail[], vipm_int aRoiVolumeAvail[],
		vipm_int aRoiStartReq[], vipm_int aRoiVolumeReq[],
		vipm_int const aRoiStartInput[], vipm_int const aRoiVolumeInput[],
		vipm_seshape_t const *aStrel, vipm_arrshape_t const *aWholeArrShape)
{
	return vipm_NS(_compute_roi,se)(Options, aRank,
		aRoiStartAvail, aRoiVolumeAvail, aRoiStartReq, aRoiVolumeReq,
		aRoiStartInput, aRoiVolumeInput, aStrel,
		(NULL != aWholeArrShape ? &aWholeArrShape->ars_aasite : NULL),
		(NULL != aWholeArrShape ? &aWholeArrShape->ars_pasite : NULL)
	);
}


__MYC_END_CDECLS
# undef __PUB
# undef __VPUB


#endif /* !defined(__VIPM_MATH_GEOMETRY_STRELUTILS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
