/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * MomentUtils.h	(V.Sokur)
 * Vipm/Math/Utils/2D/MomentUtils.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_MATH_UTILS_2D_MOMENTUTILS_H__
# define __VIPM_MATH_UTILS_2D_MOMENTUTILS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/* 1st moment: ( my, mx )
   2nd moment: ( Myy, Myx, Mxx )
   2nd moment centralized: ( u02, u11, u20) (@todo: fixme - ?) */

# define vipm_moment1_(...)             VIPM_PFX_FCNAME(vipm_moment1, __VA_ARGS__)
# define vipm_2d_moment1_(...)          VIPM_PFX_FCNAME(vipm_2d_moment1, __VA_ARGS__)
# define vipm_f2d_moment1_(...)         VIPM_PFX_FCNAME(vipm_f2d_moment1, __VA_ARGS__)
# define vipm_r2d_moment1_(...)         VIPM_PFX_FCNAME(vipm_r2d_moment1, __VA_ARGS__)
# define vipm_Moment1_(_Space)          VIPM_GLUE3(vipm_,_Space,_moment1_)

# define vipm_moment2_(...)             VIPM_PFX_FCNAME(vipm_moment2, __VA_ARGS__)
# define vipm_2d_moment2_(...)          VIPM_PFX_FCNAME(vipm_2d_moment2, __VA_ARGS__)
# define vipm_f2d_moment2_(...)         VIPM_PFX_FCNAME(vipm_f2d_moment2, __VA_ARGS__)
# define vipm_r2d_moment2_(...)         VIPM_PFX_FCNAME(vipm_r2d_moment2, __VA_ARGS__)
# define vipm_Moment2_(_Space)          VIPM_GLUE3(vipm_,_Space,_moment2_)

/* central moment2 */
# define vipm_moment2c_(...)            VIPM_PFX_FCNAME(vipm_moment2c, __VA_ARGS__)
# define vipm_2d_moment2c_(...)         VIPM_PFX_FCNAME(vipm_2d_moment2c, __VA_ARGS__)
# define vipm_f2d_moment2c_(...)        VIPM_PFX_FCNAME(vipm_f2d_moment2c, __VA_ARGS__)
# define vipm_r2d_moment2c_(...)        VIPM_PFX_FCNAME(vipm_r2d_moment2c, __VA_ARGS__)
# define vipm_Moment2c_(_Space)         VIPM_GLUE3(vipm_,_Space,_moment2c_)

# define vipm_central_moment2_(...)     VIPM_PFX_FCNAME(vipm_moment2c, __VA_ARGS__)
# define vipm_2d_central_moment2_(...)  VIPM_PFX_FCNAME(vipm_2d_moment2c, __VA_ARGS__)
# define vipm_f2d_central_moment2_(...) VIPM_PFX_FCNAME(vipm_f2d_moment2c, __VA_ARGS__)
# define vipm_r2d_central_moment2_(...) VIPM_PFX_FCNAME(vipm_r2d_moment2c, __VA_ARGS__)
# define vipm_CentralMoment2_(_Space)   VIPM_GLUE3(vipm_,_Space,_central_moment2_)


# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PROT __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Central 2nd moments
 */

__PUB double   *vipm_Moment2_(r2d)(centralize)  __P((double [/*3*/], double const [/*3*/], double const [/*2*/]));
__PUB double   *vipm_Moment2_(r2d)(centralize,I)__P((double [/*3*/], double const [/*2*/]));


__PUB double    vipm_Moment2c_(r2d)(rotation)   __P((double const [/*3*/]));
__PUB double   *vipm_Moment2c_(r2d)(covariance) __P((double [/*4*/], double const [/*3*/]));
__PUB double   *vipm_Moment2c_(r2d)(eigvals)    __P((double [/*2*/], double const [/*3*/]));
__PUB double    vipm_Moment2c_(r2d)(xeigvals)   __P((double [/*2*/], double const [/*3*/]));

/* ------------------------------------------------------------------------- */

__PUB vipm_Point_t(2d)   *vipm_Moment1_(2d)(centroid)   __P((vipm_Point_t(2d) *, double const [/*3*/]));
__PUB vipm_Point_t(f2d)  *vipm_Moment1_(f2d)(centroid)  __P((vipm_Point_t(f2d) *, double const [/*3*/]));
__PUB vipm_Point_t(r2d)  *vipm_Moment1_(r2d)(centroid)  __P((vipm_Point_t(r2d) *, double const [/*3*/]));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/* @{ */

# define vipm_r2d_moment1_centroid(r,m)     (*(r) = *vipm_r2d_point_p(m), (r))
# define vipm_f2d_moment1_centroid(r,m)     vipm_f2d_point_cast_$r(r, vipm_r2d_point_p(m))
# define vipm_2d_moment1_centroid(r,m)      vipm_2d_point_cast_$r(r, vipm_r2d_point_p(m))

/* @} */


__MYC_END_CDECLS
# undef __PUB
# undef __PROT
# undef __PRIV


#endif /* !defined(__VIPM_MATH_UTILS_2D_MOMENTUTILS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
