/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Mmi_inl.unk.h	(V.Sokur)
 * Vipm/Mmi_inl.unk.h
 */

/******************************************************************************
 * %% BeginSection: function definitions
 */
# pragma mark -
# pragma mark [ function definitions ]
# pragma mark -

/*
 * Packing/Unpacking operations
 */

# ifndef vipmi_tm_swiden88B
_VIPM_INLINE bo_uint32_t vipmi_swiden88B(/* register */ bo_uint32_t x)
{
	return ((((bo_int32_t)(bo_int8_t)(x & 0xFF)) & 0xFFFF) | ((bo_int32_t)(bo_int8_t)((x >> 8) & 0xFF) << 16));
}
# endif

# ifndef vipmi_tm_swiden88T
_VIPM_INLINE bo_uint32_t vipmi_swiden88T(/* register */ bo_uint32_t x)
{
	return vipmi_swiden88B(x >> 16);
}
# endif

# ifndef vipmi_tm_swiden88TX
_VIPM_INLINE bo_uint32_t vipmi_swiden88TX(/* register */ bo_uint32_t x)
{
	return ((bo_int32_t)(bo_int8_t)((x >> 24) & 0xFF) | ((bo_int32_t)(bo_int8_t)((x >> 16) & 0xFF) << 16));
}
# endif

/* @todo: fixme - implement */
# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
#  define vipmi_swiden8_32(d1,d2, x)    ((d1) = vipmi_swiden88T(x), (d2) = vipmi_swiden88B(x))
# else
# if (BO_BYTE_ORDER == LITTLE_BYTE_ORDER)
#  define vipmi_swiden8_32(d1,d2, x)    ((d1) = vipmi_swiden88B(x), (d2) = vipmi_swiden88T(x))
# else
# endif
# endif


# ifndef vipmi_tm_uwiden88B
_VIPM_INLINE bo_uint32_t vipmi_uwiden88B(/* register */ bo_uint32_t x)
{
	return ((x & 0x00FF) | ((x & 0xFF00) << 8));
}
# endif

# ifndef vipmi_tm_uwiden88T
_VIPM_INLINE bo_uint32_t vipmi_uwiden88T(/* register */ bo_uint32_t x)
{
	return vipmi_uwiden88B(x >> 16);
}
# endif

# ifndef vipmi_tm_uwiden88TX
_VIPM_INLINE bo_uint32_t vipmi_uwiden88TX(/* register */ bo_uint32_t x)
{
	return (((x & 0xFF000000) >> 24) | (x & 0x00FF0000));
}
# endif

/* @todo: fixme - implement */
# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
#  define vipmi_uwiden8_32(d1,d2, x)    ((d1) = vipmi_uwiden88T(x), (d2) = vipmi_uwiden88B(x))
# else
# if (BO_BYTE_ORDER == LITTLE_BYTE_ORDER)
#  define vipmi_uwiden8_32(d1,d2, x)    ((d1) = vipmi_uwiden88B(x), (d2) = vipmi_uwiden88T(x))
# else
# endif
# endif


/* ========================================================================= */

/*
 * Shift operations
 */

	/*
	 * Rotate left/right
	 */

# ifndef vipmi_tm_rotl_16
_VIPM_INLINE bo_uint16_t vipmi_rotl_16(bo_uint16_t aValue, bo_uint16_t aShift)
{
	return ((aValue << aShift) | (aValue >> (16 - aShift)));
}
# endif

# ifndef vipmi_tm_rotr_16
_VIPM_INLINE bo_uint16_t vipmi_rotr_16(bo_uint16_t aValue, bo_uint16_t aShift)
{
	return ((aValue >> aShift) | (aValue << (16 - aShift)));
}
# endif


# ifndef vipmi_tm_rotl_32
_VIPM_INLINE bo_uint32_t vipmi_rotl_32(bo_uint32_t aValue, bo_uint32_t aShift)
{
	return ((aValue << aShift) | (aValue >> (32 - aShift)));
}
# endif

# ifndef vipmi_tm_rotr_32
_VIPM_INLINE bo_uint32_t vipmi_rotr_32(bo_uint32_t aValue, bo_uint32_t aShift)
{
	return ((aValue >> aShift) | (aValue << (32 - aShift)));
}
# endif

/* ------------------------------------------------------------------------- */

	/*
	 * Unsigned rounding shift right
	 * @note: rounding half toward infinity
	 */

# ifndef vipmi_tm_urshr_16
_VIPM_INLINE bo_uint16_t vipmi_urshr_16(bo_uint16_t aValue, bo_uint16_t aShift)
{
	return ((aValue + (1 << (aShift - 1))) >> aShift);
}
# endif

# ifndef vipmi_tm_urshr_32
_VIPM_INLINE bo_uint32_t vipmi_urshr_32(bo_uint32_t aValue, bo_uint32_t aShift)
{
	return ((aValue + (1 << (aShift - 1))) >> aShift);
}
# endif

# ifndef vipmi_tm_urshr_64
_VIPM_INLINE bo_uint64_t vipmi_urshr_64(bo_uint64_t aValue, bo_uint64_t aShift)
{
	return ((aValue + ((bo_uint64_t)1 << (aShift - 1))) >> aShift);
}
# endif


/* ========================================================================= */

/*
 * Logical and Compare operations
 */

	/*
	 * Parallel Signed/Unsigned Min/Max select
	 */

# ifndef vipmi_tm_smin8_32
_VIPM_INLINE bo_uint32_t vipmi_smin8_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_int8_t b10 = (bo_int8_t)VIPM_BYAT32_0(aSrc1),
	          b11 = (bo_int8_t)VIPM_BYAT32_1(aSrc1),
	          b12 = (bo_int8_t)VIPM_BYAT32_2(aSrc1),
	          b13 = (bo_int8_t)VIPM_BYAT32_3(aSrc1);
	bo_int8_t b20 = (bo_int8_t)VIPM_BYAT32_0(aSrc2),
	          b21 = (bo_int8_t)VIPM_BYAT32_1(aSrc2),
	          b22 = (bo_int8_t)VIPM_BYAT32_2(aSrc2),
	          b23 = (bo_int8_t)VIPM_BYAT32_3(aSrc2);
	return VIPM_MKI32BY(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21), _VIPM_MIN(b12,b22), _VIPM_MIN(b13,b23));
}
# endif

# ifndef vipmi_tm_smin16_32
_VIPM_INLINE bo_uint32_t vipmi_smin16_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_int16_t b10 = (bo_int16_t)VIPM_HRAT32_0(aSrc1),
	           b11 = (bo_int16_t)VIPM_HRAT32_1(aSrc1);
	bo_int16_t b20 = (bo_int16_t)VIPM_HRAT32_0(aSrc2),
	           b21 = (bo_int16_t)VIPM_HRAT32_1(aSrc2);
	return VIPM_MKI32HR(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21));
}
# endif

# ifndef vipmi_tm_smax8_32
_VIPM_INLINE bo_uint32_t vipmi_smax8_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_int8_t b10 = (bo_int8_t)VIPM_BYAT32_0(aSrc1),
	          b11 = (bo_int8_t)VIPM_BYAT32_1(aSrc1),
	          b12 = (bo_int8_t)VIPM_BYAT32_2(aSrc1),
	          b13 = (bo_int8_t)VIPM_BYAT32_3(aSrc1);
	bo_int8_t b20 = (bo_int8_t)VIPM_BYAT32_0(aSrc2),
	          b21 = (bo_int8_t)VIPM_BYAT32_1(aSrc2),
	          b22 = (bo_int8_t)VIPM_BYAT32_2(aSrc2),
	          b23 = (bo_int8_t)VIPM_BYAT32_3(aSrc2);
	return VIPM_MKI32BY(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21), _VIPM_MIN(b12,b22), _VIPM_MIN(b13,b23));
}
# endif

# ifndef vipmi_tm_smax16_32
_VIPM_INLINE bo_uint32_t vipmi_smax16_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_int16_t b10 = (bo_int16_t)VIPM_HRAT32_0(aSrc1),
	           b11 = (bo_int16_t)VIPM_HRAT32_1(aSrc1);
	bo_int16_t b20 = (bo_int16_t)VIPM_HRAT32_0(aSrc2),
	           b21 = (bo_int16_t)VIPM_HRAT32_1(aSrc2);
	return VIPM_MKI32HR(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21));
}
# endif


# ifndef vipmi_tm_umin8_32
_VIPM_INLINE bo_uint32_t vipmi_umin8_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_uint8_t b10 = (bo_uint8_t)VIPM_BYAT32_0(aSrc1),
	           b11 = (bo_uint8_t)VIPM_BYAT32_1(aSrc1),
	           b12 = (bo_uint8_t)VIPM_BYAT32_2(aSrc1),
	           b13 = (bo_uint8_t)VIPM_BYAT32_3(aSrc1);
	bo_uint8_t b20 = (bo_uint8_t)VIPM_BYAT32_0(aSrc2),
	           b21 = (bo_uint8_t)VIPM_BYAT32_1(aSrc2),
	           b22 = (bo_uint8_t)VIPM_BYAT32_2(aSrc2),
	           b23 = (bo_uint8_t)VIPM_BYAT32_3(aSrc2);
	return VIPM_MKI32BY(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21), _VIPM_MIN(b12,b22), _VIPM_MIN(b13,b23));
}
# endif

# ifndef vipmi_tm_umin16_32
_VIPM_INLINE bo_uint32_t vipmi_umin16_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_uint16_t b10 = (bo_uint16_t)VIPM_HRAT32_0(aSrc1),
	            b11 = (bo_uint16_t)VIPM_HRAT32_1(aSrc1);
	bo_uint16_t b20 = (bo_uint16_t)VIPM_HRAT32_0(aSrc2),
	            b21 = (bo_uint16_t)VIPM_HRAT32_1(aSrc2);
	return VIPM_MKI32HR(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21));
}
# endif

# ifndef vipmi_tm_umax8_32
_VIPM_INLINE bo_uint32_t vipmi_umax8_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_uint8_t b10 = (bo_uint8_t)VIPM_BYAT32_0(aSrc1),
	           b11 = (bo_uint8_t)VIPM_BYAT32_1(aSrc1),
	           b12 = (bo_uint8_t)VIPM_BYAT32_2(aSrc1),
	           b13 = (bo_uint8_t)VIPM_BYAT32_3(aSrc1);
	bo_uint8_t b20 = (bo_uint8_t)VIPM_BYAT32_0(aSrc2),
	           b21 = (bo_uint8_t)VIPM_BYAT32_1(aSrc2),
	           b22 = (bo_uint8_t)VIPM_BYAT32_2(aSrc2),
	           b23 = (bo_uint8_t)VIPM_BYAT32_3(aSrc2);
	return VIPM_MKI32BY(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21), _VIPM_MIN(b12,b22), _VIPM_MIN(b13,b23));
}
# endif

# ifndef vipmi_tm_umax16_32
_VIPM_INLINE bo_uint32_t vipmi_umax16_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2)
{
	bo_uint16_t b10 = (bo_uint16_t)VIPM_HRAT32_0(aSrc1),
	            b11 = (bo_uint16_t)VIPM_HRAT32_1(aSrc1);
	bo_uint16_t b20 = (bo_uint16_t)VIPM_HRAT32_0(aSrc2),
	            b21 = (bo_uint16_t)VIPM_HRAT32_1(aSrc2);
	return VIPM_MKI32HR(_VIPM_MIN(b10,b20), _VIPM_MIN(b11,b21));
}
# endif

/* ------------------------------------------------------------------------- */

	/*
	 * Bitwise select
	 */

# ifndef vipmi_tm_bitift_8
_VIPM_INLINE bo_uint8_t
vipmi_bitift_8(bo_uint8_t aSrc1, bo_uint8_t aSrc2, bo_uint8_t aMask)
{
	return ((aSrc1 & ~aMask) | (aSrc2 & aMask));
}
# endif

# ifndef vipmi_tm_bitift_16
_VIPM_INLINE bo_uint16_t
vipmi_bitift_16(bo_uint16_t aSrc1, bo_uint16_t aSrc2, bo_uint16_t aMask)
{
	return ((aSrc1 & ~aMask) | (aSrc2 & aMask));
}
# endif

# ifndef vipmi_tm_bitift_32
_VIPM_INLINE bo_uint32_t
vipmi_bitift_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2, bo_uint32_t aMask)
{
	return ((aSrc1 & ~aMask) | (aSrc2 & aMask));
}
# endif


# ifndef vipmi_tm_bitiff_8
_VIPM_INLINE bo_uint8_t
vipmi_bitiff_8(bo_uint8_t aSrc1, bo_uint8_t aSrc2, bo_uint8_t aMask)
{
#if 0
	return vipmi_bitift_8(aSrc2, aSrc1, aMask);
#endif
	return ((aSrc1 & aMask) | (aSrc2 & ~aMask));
}
# endif

# ifndef vipmi_tm_bitiff_16
_VIPM_INLINE bo_uint16_t
vipmi_bitiff_16(bo_uint16_t aSrc1, bo_uint16_t aSrc2, bo_uint16_t aMask)
{
#if 0
	return vipmi_bitift_16(aSrc2, aSrc1, aMask);
#endif
	return ((aSrc1 & aMask) | (aSrc2 & ~aMask));
}
# endif

# ifndef vipmi_tm_bitiff_32
_VIPM_INLINE bo_uint32_t
vipmi_bitiff_32(bo_uint32_t aSrc1, bo_uint32_t aSrc2, bo_uint32_t aMask)
{
#if 0
	return vipmi_bitift_32(aSrc2, aSrc1, aMask);
#endif
	return ((aSrc1 & aMask) | (aSrc2 & ~aMask));
}
# endif


/* ========================================================================= */

/*
 * [Parallel] Add and Subtract operations
 */

# ifndef vipmi_tm_saddw88B
_VIPM_INLINE bo_uint32_t vipmi_saddw88B(bo_uint32_t a, bo_uint32_t x)
{
	bo_int32_t a0, a1;
	a0 = (bo_int32_t)(bo_int16_t)(a & 0xFFFF);
	a1 = (bo_int32_t)(bo_int16_t)((a >> 16) & 0xFFFF);
	return ( (a0 + (bo_int32_t)(bo_int8_t)(x & 0xFF)) | ((a1 + (bo_int32_t)(bo_int8_t)((x >> 8) & 0xFF)) << 16) );
}
# endif

# ifndef vipmi_tm_saddw88T
_VIPM_INLINE bo_uint32_t vipmi_saddw88T(bo_uint32_t a, bo_uint32_t x)
{
	return vipmi_saddw88B(a, x >> 16);
}
# endif

# ifndef vipmi_tm_saddw88TX
_VIPM_INLINE bo_uint32_t vipmi_saddw88TX(bo_uint32_t a, bo_uint32_t x)
{
	bo_int32_t a0, a1;
	a0 = (bo_int32_t)(bo_int16_t)(a & 0xFFFF);
	a1 = (bo_int32_t)(bo_int16_t)((a >> 16) & 0xFFFF);
	return ( (a0 + (bo_int32_t)(bo_int8_t)((x >> 24) & 0xFF)) | ((a1 + (bo_int32_t)(bo_int8_t)((x >> 16) & 0xFF)) << 16) );
}
# endif


# ifndef vipmi_tm_uaddw88B
_VIPM_INLINE bo_uint32_t vipmi_uaddw88B(bo_uint32_t a, bo_uint32_t x)
{
	bo_uint32_t a0, a1;
	a0 = a & 0xFFFF;
	a1 = (a >> 16) & 0xFFFF;
	return ( (a0 + (x & 0xFF)) | ((a1 + ((x >> 8) & 0xFF)) << 16) );
}
# endif

# ifndef vipmi_tm_uaddw88T
_VIPM_INLINE bo_uint32_t vipmi_uaddw88T(bo_uint32_t a, bo_uint32_t x)
{
	return vipmi_uaddw88B(a, x >> 16);
}
# endif

# ifndef vipmi_tm_uaddw88TX
_VIPM_INLINE bo_uint32_t vipmi_uaddw88TX(bo_uint32_t a, bo_uint32_t x)
{
	bo_uint32_t a0, a1;
	a0 = a & 0xFFFF;
	a1 = (a >> 16) & 0xFFFF;
	return ( (a0 + ((x >> 24) & 0xFF)) | ((a1 + ((x >> 16) & 0xFF)) << 16) );
}
# endif

/* ------------------------------------------------------------------------- */

	/*
	 * Unsigned sum [and accumulate] of bytes
	 */

# ifndef vipmi_tm_usum8_32
_VIPM_INLINE bo_uint32_t vipmi_usum8_32(bo_uint32_t x)
{
	return ((x & 0xFF) + ((x >> 8) & 0xFF) + ((x >> 16) & 0xFF) + ((x >> 24) & 0xFF));
}
# endif

# ifndef vipmi_tm_usuma8_32
_VIPM_INLINE bo_uint32_t vipmi_usuma8_32(bo_uint32_t a, bo_uint32_t x)
{
	return (a + vipmi_usum8_32(x));
}
# endif


# ifndef vipmi_tm_usum16_32
_VIPM_INLINE bo_uint32_t vipmi_usum16_32(bo_uint32_t x)
{
	return ((x & 0xFFFF) + ((x >> 16) & 0xFFFF));
}
# endif

# ifndef vipmi_tm_usuma16_32
_VIPM_INLINE bo_uint32_t vipmi_usuma16_32(bo_uint32_t a, bo_uint32_t x)
{
	return (a + vipmi_usum16_32(x));
}
# endif


/* ========================================================================= */

/*
 * [Parallel] Multiply operations
 */

	/*
	 * Signed[/Unsigned] Multiply, Multiply-Accumulate, Multiply-Subtract
	 */

		/*
		 * 32-bit mla
		 */

# ifndef vipmi_tm_smla_32
_VIPM_INLINE bo_int32_t vipmi_smla_32(bo_int32_t a, bo_int32_t x, bo_int32_t y)
{
	return (a + x * y);
}
# endif
# define vipmi_smla_32(a,x,y)       (vipmi_smla_32)((bo_int32_t)(a), (bo_int32_t)(x), (bo_int32_t)(y))

# ifndef vipmi_tm_smlal_32
_VIPM_INLINE bo_int64_t vipmi_smlal_32(bo_int64_t a, bo_int32_t x, bo_int32_t y)
{
	return (a + (bo_int64_t)x * y);
}
# endif
# define vipmi_smlal_32(a,x,y)      (vipmi_smlal_32)((bo_int64_t)(a), (bo_int32_t)(x), (bo_int32_t)(y))

# ifndef vipmi_tm_smlaw_32
_VIPM_INLINE bo_int64_t vipmi_smlaw_32(bo_int64_t a, bo_int64_t x, bo_int32_t y)
{
	return (a + x * y);
}
# endif
# define vipmi_smlaw_32(a,x,y)      (vipmi_smlaw_32)((bo_int64_t)(a), (bo_int64_t)(x), (bo_int32_t)(y))


		/*
		 * 32-bit mls
		 */

# ifndef vipmi_tm_smls_32
_VIPM_INLINE bo_int32_t vipmi_smls_32(bo_int32_t a, bo_int32_t x, bo_int32_t y)
{
	return (a - x * y);
}
# endif
# define vipmi_smls_32(a,x,y)       (vipmi_smls_32)((bo_int32_t)(a), (bo_int32_t)(x), (bo_int32_t)(y))

# ifndef vipmi_tm_smlsl_32
_VIPM_INLINE bo_int64_t vipmi_smlsl_32(bo_int64_t a, bo_int32_t x, bo_int32_t y)
{
	return (a - (bo_int64_t)x * y);
}
# endif
# define vipmi_smlsl_32(a,x,y)      (vipmi_smlsl_32)((bo_int64_t)(a), (bo_int32_t)(x), (bo_int32_t)(y))

# ifndef vipmi_tm_smlsw_32
_VIPM_INLINE bo_int64_t vipmi_smlsw_32(bo_int64_t a, bo_int64_t x, bo_int32_t y)
{
	return (a - x * y);
}
# endif
# define vipmi_smlsw_32(a,x,y)      (vipmi_smlsw_32)((bo_int64_t)(a), (bo_int64_t)(x), (bo_int32_t)(y))

/* ------------------------------------------------------------------------- */

		/*
		 * 64-bit mla
		 */

# ifndef vipmi_tm_smla_64
_VIPM_INLINE bo_int64_t vipmi_smla_64(bo_int64_t a, bo_int64_t x, bo_int64_t y)
{
	return (a + x * y);
}
# endif
# define vipmi_smla_64(a,x,y)       (vipmi_smla_64)((bo_int64_t)(a), (bo_int64_t)(x), (bo_int64_t)(y))

		/*
		 * 64-bit mls
		 */

# ifndef vipmi_tm_smls_64
_VIPM_INLINE bo_int64_t vipmi_smls_64(bo_int64_t a, bo_int64_t x, bo_int64_t y)
{
	return (a - x * y);
}
# endif
# define vipmi_smls_64(a,x,y)       (vipmi_smls_64)((bo_int64_t)(a), (bo_int64_t)(x), (bo_int64_t)(y))

/* ------------------------------------------------------------------------- */

		/*
		 * 32-bit float mla
		 */

# ifndef vipmi_tm_mla_f32
_VIPM_INLINE bo_float32_t vipmi_mla_f32(bo_float32_t a, bo_float32_t x, bo_float32_t y)
{
	return (float)(a + x * y);
}
# endif
# define vipmi_mla_f32(a,x,y)       (vipmi_mla_f32)((bo_float32_t)(a), (bo_float32_t)(x), (bo_float32_t)(y))

# ifndef vipmi_tm_mlal_f32
_VIPM_INLINE bo_float64_t vipmi_mlal_f32(bo_float64_t a, bo_float32_t x, bo_float32_t y)
{
	return (a + (double)x * y);
}
# endif
# define vipmi_mlal_f32(a,x,y)      (vipmi_mlal_f32)((bo_float64_t)(a), (bo_float32_t)(x), (bo_float32_t)(y))

# ifndef vipmi_tm_mlaw_f32
_VIPM_INLINE bo_float64_t vipmi_mlaw_f32(bo_float64_t a, bo_float64_t x, bo_float32_t y)
{
	return (a + x * y);
}
# endif
# define vipmi_mlaw_f32(a,x,y)      (vipmi_mlaw_f32)((bo_float64_t)(a), (bo_float64_t)(x), (bo_float32_t)(y))


		/*
		 * 32-bit float mls
		 */

# ifndef vipmi_tm_mls_f32
_VIPM_INLINE bo_float32_t vipmi_mls_f32(bo_float32_t a, bo_float32_t x, bo_float32_t y)
{
	return (float)(a - x * y);
}
# endif
# define vipmi_mls_f32(a,x,y)       (vipmi_mls_f32)((bo_float32_t)(a), (bo_float32_t)(x), (bo_float32_t)(y))

# ifndef vipmi_tm_mlsl_f32
_VIPM_INLINE bo_float64_t vipmi_mlsl_f32(bo_float64_t a, bo_float32_t x, bo_float32_t y)
{
	return (a - (double)x * y);
}
# endif
# define vipmi_mlsl_f32(a,x,y)      (vipmi_mlsl_f32)((bo_float64_t)(a), (bo_float32_t)(x), (bo_float32_t)(y))

# ifndef vipmi_tm_mlsw_f32
_VIPM_INLINE bo_float64_t vipmi_mlsw_f32(bo_float64_t a, bo_float64_t x, bo_float32_t y)
{
	return (a - x * y);
}
# endif
# define vipmi_mlsw_f32(a,x,y)      (vipmi_mlsw_f32)((bo_float64_t)(a), (bo_float64_t)(x), (bo_float32_t)(y))

/* ------------------------------------------------------------------------- */

		/*
		 * 64-bit float mla
		 */

# ifndef vipmi_tm_mla_f64
_VIPM_INLINE bo_float64_t vipmi_mla_f64(bo_float64_t a, bo_float64_t x, bo_float64_t y)
{
	return (a + x * y);
}
# endif
# define vipmi_mla_f64(a,x,y)       (vipmi_mla_f64)((bo_float64_t)(a), (bo_float64_t)(x), (bo_float64_t)(y))

		/*
		 * 64-bit float mls
		 */

# ifndef vipmi_tm_mls_f64
_VIPM_INLINE bo_float64_t vipmi_mls_f64(bo_float64_t a, bo_float64_t x, bo_float64_t y)
{
	return (a - x * y);
}
# endif
# define vipmi_mls_f64(a,x,y)       (vipmi_mls_f64)((bo_float64_t)(a), (bo_float64_t)(x), (bo_float64_t)(y))


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
