/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Mmintrin.h	(V.Sokur)
 * Vipm/Mmintrin.h
 *
 * [ mmi - my machine intrinsic(s)/instruction/... ]
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_TYPES_H__
# include <Vipm/Types.h>
#endif

#ifndef __VIPM_MMINTRIN_H__
# define __VIPM_MMINTRIN_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <math.h>
# include <float.h>


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

# ifndef __VIPM_NO_MMINTRIN_HEADER

#  ifdef __VIPM_SSE2_MMINTRIN_HEADER
#   include __VIPM_SSE2_MMINTRIN_HEADER
#  endif
#  ifdef __VIPM_SSE_MMINTRIN_HEADER
#   include __VIPM_SSE_MMINTRIN_HEADER
#  endif

# endif /* !defined(__VIPM_NO_MMINTRIN_HEADER) */


/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_MACHINE_ASM__
#  include <Vipm/machine/Asm.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

// @todo: fixme - move to (?)

/*
 * load32:  bo_uint32_t const *aBase;
 * store32: bo_uint32_t *aBase;
 *
 * load64:  bo_uint64_t const *aBase;
 * store64: bo_uint64_t *aBase;
 */
# ifdef _A_ARM

#  define _VipmLoadi32c(aBase, _Index)              VIPM_GLUE(_Vipm_Loadi32c_, _Index)(aBase)
#  define _VipmStorei32c(aBase, _Index, aData)      VIPM_GLUE(_Vipm_Storei32c_, _Index)(aBase, aData)
#  define _VipmLoadu32c(aBase, _Index)              ((bo_uint32_t) _VipmLoadi32c(aBase, _Index))
#  define _VipmStoreu32c(aBase, _Index, aData)      _VipmStorei32c(aBase, _Index, (bo_int32_t)(aData))

#  define _VipmLoadi64c(aBase, _Index)              VIPM_GLUE(_Vipm_Loadi64c_, _Index)(aBase)
#  define _VipmStorei64c(aBase, _Index, aData)      VIPM_GLUE(_Vipm_Storei64c_, _Index)(aBase, aData)
#  define _VipmLoadu64c(aBase, _Index)              ((bo_uint64_t) _VipmLoadi64c(aBase, _Index))
#  define _VipmStoreu64c(aBase, _Index, aData)      _VipmStorei64c(aBase, _Index, (bo_int64_t)(aData))

/* @sa: "Bo/sys/Byteorder.h" */
/* @{ */
#  define _Vipm_Loadi32c_0(aBase)                   _BoLoadi32(aBase, 0)
#  define _Vipm_Loadi32c_1(aBase)                   _BoLoadi32(aBase, 4)
#  define _Vipm_Loadi32c_2(aBase)                   _BoLoadi32(aBase, 8)
#  define _Vipm_Loadi32c_3(aBase)                   _BoLoadi32(aBase, 12)
#  define _Vipm_Loadi32c_4(aBase)                   _BoLoadi32(aBase, 16)
#  define _Vipm_Loadi32c_5(aBase)                   _BoLoadi32(aBase, 20)
#  define _Vipm_Loadi32c_6(aBase)                   _BoLoadi32(aBase, 24)
#  define _Vipm_Loadi32c_7(aBase)                   _BoLoadi32(aBase, 28)
#  define _Vipm_Loadi32c_8(aBase)                   _BoLoadi32(aBase, 32)
#  define _Vipm_Loadi32c_9(aBase)                   _BoLoadi32(aBase, 36)
#  define _Vipm_Loadi32c_10(aBase)                  _BoLoadi32(aBase, 40)
#  define _Vipm_Loadi32c_11(aBase)                  _BoLoadi32(aBase, 44)
#  define _Vipm_Loadi32c_12(aBase)                  _BoLoadi32(aBase, 48)
#  define _Vipm_Loadi32c_13(aBase)                  _BoLoadi32(aBase, 52)
#  define _Vipm_Loadi32c_14(aBase)                  _BoLoadi32(aBase, 56)
#  define _Vipm_Loadi32c_15(aBase)                  _BoLoadi32(aBase, 60)
#  define _Vipm_Loadi64c_0(aBase)                   _BoLoadi64(aBase, 0)
#  define _Vipm_Loadi64c_1(aBase)                   _BoLoadi64(aBase, 8)
#  define _Vipm_Loadi64c_2(aBase)                   _BoLoadi64(aBase, 16)
#  define _Vipm_Loadi64c_3(aBase)                   _BoLoadi64(aBase, 24)
#  define _Vipm_Loadi64c_4(aBase)                   _BoLoadi64(aBase, 32)
#  define _Vipm_Loadi64c_5(aBase)                   _BoLoadi64(aBase, 40)
#  define _Vipm_Loadi64c_6(aBase)                   _BoLoadi64(aBase, 48)
#  define _Vipm_Loadi64c_7(aBase)                   _BoLoadi64(aBase, 56)
#  define _Vipm_Loadi64c_8(aBase)                   _BoLoadi64(aBase, 64)
#  define _Vipm_Loadi64c_9(aBase)                   _BoLoadi64(aBase, 72)
#  define _Vipm_Loadi64c_10(aBase)                  _BoLoadi64(aBase, 80)
#  define _Vipm_Loadi64c_11(aBase)                  _BoLoadi64(aBase, 88)
#  define _Vipm_Loadi64c_12(aBase)                  _BoLoadi64(aBase, 96)
#  define _Vipm_Loadi64c_13(aBase)                  _BoLoadi64(aBase, 104)
#  define _Vipm_Loadi64c_14(aBase)                  _BoLoadi64(aBase, 112)
#  define _Vipm_Loadi64c_15(aBase)                  _BoLoadi64(aBase, 120)

#  define _Vipm_Storei32c_0(aBase,aData)            _BoStorei32(aBase, 0, aData)
#  define _Vipm_Storei32c_1(aBase,aData)            _BoStorei32(aBase, 4, aData)
#  define _Vipm_Storei32c_2(aBase,aData)            _BoStorei32(aBase, 8, aData)
#  define _Vipm_Storei32c_3(aBase,aData)            _BoStorei32(aBase, 12, aData)
#  define _Vipm_Storei32c_4(aBase,aData)            _BoStorei32(aBase, 16, aData)
#  define _Vipm_Storei32c_5(aBase,aData)            _BoStorei32(aBase, 20, aData)
#  define _Vipm_Storei32c_6(aBase,aData)            _BoStorei32(aBase, 24, aData)
#  define _Vipm_Storei32c_7(aBase,aData)            _BoStorei32(aBase, 28, aData)
#  define _Vipm_Storei32c_8(aBase,aData)            _BoStorei32(aBase, 32, aData)
#  define _Vipm_Storei32c_9(aBase,aData)            _BoStorei32(aBase, 36, aData)
#  define _Vipm_Storei32c_10(aBase,aData)           _BoStorei32(aBase, 40, aData)
#  define _Vipm_Storei32c_11(aBase,aData)           _BoStorei32(aBase, 44, aData)
#  define _Vipm_Storei32c_12(aBase,aData)           _BoStorei32(aBase, 48, aData)
#  define _Vipm_Storei32c_13(aBase,aData)           _BoStorei32(aBase, 52, aData)
#  define _Vipm_Storei32c_14(aBase,aData)           _BoStorei32(aBase, 56, aData)
#  define _Vipm_Storei32c_15(aBase,aData)           _BoStorei32(aBase, 60, aData)
#  define _Vipm_Storei64c_0(aBase,aData)            _BoStorei64(aBase, 0, aData)
#  define _Vipm_Storei64c_1(aBase,aData)            _BoStorei64(aBase, 8, aData)
#  define _Vipm_Storei64c_2(aBase,aData)            _BoStorei64(aBase, 16, aData)
#  define _Vipm_Storei64c_3(aBase,aData)            _BoStorei64(aBase, 24, aData)
#  define _Vipm_Storei64c_4(aBase,aData)            _BoStorei64(aBase, 32, aData)
#  define _Vipm_Storei64c_5(aBase,aData)            _BoStorei64(aBase, 40, aData)
#  define _Vipm_Storei64c_6(aBase,aData)            _BoStorei64(aBase, 48, aData)
#  define _Vipm_Storei64c_7(aBase,aData)            _BoStorei64(aBase, 56, aData)
#  define _Vipm_Storei64c_8(aBase,aData)            _BoStorei64(aBase, 64, aData)
#  define _Vipm_Storei64c_9(aBase,aData)            _BoStorei64(aBase, 72, aData)
#  define _Vipm_Storei64c_10(aBase,aData)           _BoStorei64(aBase, 80, aData)
#  define _Vipm_Storei64c_11(aBase,aData)           _BoStorei64(aBase, 88, aData)
#  define _Vipm_Storei64c_12(aBase,aData)           _BoStorei64(aBase, 96, aData)
#  define _Vipm_Storei64c_13(aBase,aData)           _BoStorei64(aBase, 104, aData)
#  define _Vipm_Storei64c_14(aBase,aData)           _BoStorei64(aBase, 112, aData)
#  define _Vipm_Storei64c_15(aBase,aData)           _BoStorei64(aBase, 120, aData)
/* @} */

# else /* !defined(_A_ARM) */

#  define _VipmLoadi32c(aBase, _Index)             ((bo_int32_t) (aBase)[(_Index)])
#  define _VipmStorei32c(aBase, _Index, aData)     ((aBase)[(_Index)] = (bo_int32_t)(aData))
#  define _VipmLoadu32c(aBase, _Index)             ((bo_uint32_t) (aBase)[(_Index)])
#  define _VipmStoreu32c(aBase, _Index, aData)     ((aBase)[(_Index)] = (bo_uint32_t)(aData))

#  define _VipmLoadi64c(aBase, _Index)             ((bo_int64_t) (aBase)[(_Index)])
#  define _VipmStorei64c(aBase, _Index, aData)     ((aBase)[(_Index)] = (bo_int64_t)(aData))
#  define _VipmLoadu64c(aBase, _Index)             ((bo_uint64_t) (aBase)[(_Index)])
#  define _VipmStoreu64c(aBase, _Index, aData)     ((aBase)[(_Index)] = (bo_uint64_t)(aData))

# endif /* defined(_A_ARM) */


/* ========================================================================= */

# ifndef vipmi_compiler_barrier
#  if defined(_GCC_VER)
#   define vipmi_compiler_barrier()     __asm__ __volatile__ ("" ::: "memory")
#  else
#  if defined(_MSC_VER) && (1300 <= _MSC_VER)
#   if (_MSC_VER <= 1400) && defined(_DEBUG)
__MYC_BEGIN_CDECLS
extern void _ReadWriteBarrier();
__MYC_END_CDECLS
#   endif
#	pragma intrinsic(_ReadWriteBarrier)
#   define vipmi_compiler_barrier()     _ReadWriteBarrier()
#  else
#   define vipmi_compiler_barrier()
#  endif /* _MSC_VER */
#  endif /* _GCC_VER */
# endif /* vipmi_compiler_barrier */


/*
 * _LocalityDegree <- { 0,1,2,3 }, [default: 3]
 */
# if defined(_GCC_VER)

/* @todo: fixme */
#  if defined(_A_ARM) || defined(_A_X86) || defined(_A_X86_64)
#   define __VIPMI_PREFETCH                         2   /* useful prefetch next row */
#  else
#   define __VIPMI_PREFETCH                         1   /* useless prefetch next row */
#  endif

#  define vipmi_prfm_rd(Addr)                       __builtin_prefetch(Addr, 0)
#  define vipmi_prfm2_rd(Addr, _LocalityDegree)     __builtin_prefetch(Addr, 0, _LocalityDegree)
#  define vipmi_prfm_wr(Addr)                       __builtin_prefetch(Addr, 1)
#  define vipmi_prfm2_wr(Addr, _LocalityDegree)     __builtin_prefetch(Addr, 1, _LocalityDegree)

# else

// @todo: fixme - implement (?)
#  define __VIPMI_PREFETCH                          0

#  define vipmi_prfm_rd(Addr)                       (void)(Addr)
#  define vipmi_prfm2_rd(Addr, _LocalityDegree)     (void)(Addr)
#  define vipmi_prfm_wr(Addr)                       (void)(Addr)
#  define vipmi_prfm2_wr(Addr, _LocalityDegree)     (void)(Addr)

# endif /* _GCC_VER */

# define vipmi_xprfm_rd(Addr,_Off)                  vipmi_prfm_rd((char *)(Addr) + _Off)
# define vipmi_xxprfm_rd(Addr,_Off,_LocalityDegree) vipmi_prfm2_rd((char *)(Addr) + _Off, _LocalityDegree)
# define vipmi_xprfm_wr(Addr,_Off)                  vipmi_prfm_wr((char *)(Addr) + _Off)
# define vipmi_xxprfm_wr(Addr,_Off,_LocalityDegree) vipmi_prfm2_wr((char *)(Addr) + _Off, _LocalityDegree)


/* ========================================================================= */

int         vipmi_isinff            __P((float));
int         vipmi_isnanf            __P((float));

int         vipmi_isinfd            __P((double));
int         vipmi_isnand            __P((double));

/* ------------------------------------------------------------------------- */

/* required:
#include <stdlib.h>
 */

vipm_int    vipmi_abs               __P((vipm_int));
vipm_long   vipmi_absl              __P((vipm_long));
long long   vipmi_absll             __P((long long));
float       vipmi_absf              __P((float));
double      vipmi_absd              __P((double));


# define vipmi_absf                 fabsf
# define vipmi_absd                 fabs
# define vipmi_fabs                 fabs

/* ------------------------------------------------------------------------- */

float       vipmi_logf              __P((float));
double      vipmi_logd              __P((double));

float       vipmi_log2f             __P((float));
double      vipmi_log2d             __P((double));


# define vipmi_logf                 logf
# define vipmi_logd                 log

/* ------------------------------------------------------------------------- */

vipm_int    vipmi_rintf             __P((float));
vipm_int    vipmi_rfloorf           __P((float));
vipm_int    vipmi_rceilf            __P((float));
vipm_int    vipmi_rtruncf           __P((float));

bo_int64_t  vipmi_llrintf           __P((float));

vipm_int    vipmi_rintd             __P((double));
vipm_int    vipmi_rfloord           __P((double));
vipm_int    vipmi_rceild            __P((double));
vipm_int    vipmi_rtruncd           __P((double));

bo_int64_t  vipmi_llrintd           __P((double));

/* ------------------------------------------------------------------------- */

int         vipmi_eequalfS          __P((float, float, float));    /* essentially(strong) equal */
int         vipmi_aequalfS          __P((float, float, float));    /* approximately equal */
# define vipmi_equalfS              vipmi_aequalfS
int         vipmi_lessfS            __P((float, float, float));
int         vipmi_greaterfS         __P((float, float, float));
int         vipmi_lequfS            __P((float, float, float));
int         vipmi_gequfS            __P((float, float, float));

int         vipmi_eequaldS          __P((double, double, double)); /* essentially(strong) equal */
int         vipmi_aequaldS          __P((double, double, double)); /* approximately equal */
# define vipmi_equaldS              vipmi_aequaldS
int         vipmi_lessdS            __P((double, double, double));
int         vipmi_greaterdS         __P((double, double, double));
int         vipmi_lequdS            __P((double, double, double));
int         vipmi_gequdS            __P((double, double, double));

# define vipmi_eequalS(x,y,eps)    (sizeof(x) == sizeof(float) ? vipmi_eequalfS(x,y,eps)  : vipmi_eequaldS(x,y,eps))
# define vipmi_aequalS(x,y,eps)    (sizeof(x) == sizeof(float) ? vipmi_aequalfS(x,y,eps)  : vipmi_aequaldS(x,y,eps))
# define vipmi_equalS(x,y,eps)     (sizeof(x) == sizeof(float) ? vipmi_equalfS(x,y,eps)   : vipmi_equaldS(x,y,eps))
# define vipmi_lessS(x,y,eps)      (sizeof(x) == sizeof(float) ? vipmi_lessfS(x,y,eps)    : vipmi_lessdS(x,y,eps))
# define vipmi_greaterS(x,y,eps)   (sizeof(x) == sizeof(float) ? vipmi_greaterfS(x,y,eps) : vipmi_greaterdS(x,y,eps))
# define vipmi_lequS(x,y,eps)      (sizeof(x) == sizeof(float) ? vipmi_lequfS(x,y,eps)    : vipmi_lequdS(x,y,eps))
# define vipmi_gequS(x,y,eps)      (sizeof(x) == sizeof(float) ? vipmi_gequfS(x,y,eps)    : vipmi_gequdS(x,y,eps))


int         vipmi_eequalf           __P((float, float, float));    /* essentially(strong) equal */
int         vipmi_aequalf           __P((float, float, float));    /* approximately equal */
# define vipmi_equalf               vipmi_aequalf
int         vipmi_lessf             __P((float, float, float));
int         vipmi_greaterf          __P((float, float, float));
int         vipmi_lequf             __P((float, float, float));
int         vipmi_gequf             __P((float, float, float));

int         vipmi_eequald           __P((double, double, double)); /* essentially(strong) equal */
int         vipmi_aequald           __P((double, double, double)); /* approximately equal */
# define vipmi_equald               vipmi_aequald
int         vipmi_lessd             __P((double, double, double));
int         vipmi_greaterd          __P((double, double, double));
int         vipmi_lequd             __P((double, double, double));
int         vipmi_gequd             __P((double, double, double));

# define vipmi_eequal(x,y,eps)     (sizeof(x) == sizeof(float) ? vipmi_eequalf(x,y,eps)  : vipmi_eequald(x,y,eps))
# define vipmi_aequal(x,y,eps)     (sizeof(x) == sizeof(float) ? vipmi_aequalf(x,y,eps)  : vipmi_aequald(x,y,eps))
# define vipmi_equal(x,y,eps)      (sizeof(x) == sizeof(float) ? vipmi_equalf(x,y,eps)   : vipmi_equald(x,y,eps))
# define vipmi_less(x,y,eps)       (sizeof(x) == sizeof(float) ? vipmi_lessf(x,y,eps)    : vipmi_lessd(x,y,eps))
# define vipmi_greater(x,y,eps)    (sizeof(x) == sizeof(float) ? vipmi_greaterf(x,y,eps) : vipmi_greaterd(x,y,eps))
# define vipmi_lequ(x,y,eps)       (sizeof(x) == sizeof(float) ? vipmi_lequf(x,y,eps)    : vipmi_lequd(x,y,eps))
# define vipmi_gequ(x,y,eps)       (sizeof(x) == sizeof(float) ? vipmi_gequf(x,y,eps)    : vipmi_gequd(x,y,eps))


int         vipmi__equalf           __P((float, float));
int         vipmi__lessf            __P((float, float));
int         vipmi__greaterf         __P((float, float));
int         vipmi__lequf            __P((float, float));
int         vipmi__gequf            __P((float, float));

int         vipmi__equald           __P((double, double));
int         vipmi__lessd            __P((double, double));
int         vipmi__greaterd         __P((double, double));
int         vipmi__lequd            __P((double, double));
int         vipmi__gequd            __P((double, double));

# define vipmi__equal(x,y)         (sizeof(x) == sizeof(float) ? vipmi__equalf(x,y)   : vipmi__equald(x,y))
# define vipmi__less(x,y)          (sizeof(x) == sizeof(float) ? vipmi__lessf(x,y)    : vipmi__lessd(x,y))
# define vipmi__greater(x,y)       (sizeof(x) == sizeof(float) ? vipmi__greaterf(x,y) : vipmi__greaterd(x,y))
# define vipmi__lequ(x,y)          (sizeof(x) == sizeof(float) ? vipmi__lequf(x,y)    : vipmi__lequd(x,y))
# define vipmi__gequ(x,y)          (sizeof(x) == sizeof(float) ? vipmi__gequf(x,y)    : vipmi__gequd(x,y))


int         vipmi__equal0f          __P((float));
int         vipmi__less0f           __P((float));
int         vipmi__greater0f        __P((float));
int         vipmi__lequ0f           __P((float));
int         vipmi__gequ0f           __P((float));

int         vipmi__equal0d          __P((double));
int         vipmi__less0d           __P((double));
int         vipmi__greater0d        __P((double));
int         vipmi__lequ0d           __P((double));
int         vipmi__gequ0d           __P((double));

# define vipmi__equal0(x)          (sizeof(x) == sizeof(float) ? vipmi__equal0f(x)   : vipmi__equal0d(x))
# define vipmi__less0(x)           (sizeof(x) == sizeof(float) ? vipmi__less0f(x)    : vipmi__less0d(x))
# define vipmi__greater0(x)        (sizeof(x) == sizeof(float) ? vipmi__greater0f(x) : vipmi__greater0d(x))
# define vipmi__lequ0(x)           (sizeof(x) == sizeof(float) ? vipmi__lequ0f(x)    : vipmi__lequ0d(x))
# define vipmi__gequ0(x)           (sizeof(x) == sizeof(float) ? vipmi__gequ0f(x)    : vipmi__gequ0d(x))

/* ------------------------------------------------------------------------- */

/*
 * Packing/Unpacking operations
 */

	/*
	 * 0088  : 0xUUAB --> 0x0A0B
	 * 8800  : 0xABUU --> 0x0A0B
	 * 8800X : 0xBAUU --> 0x0A0B
	 */

bo_uint32_t vipmi_swiden88B         __P((bo_uint32_t));
bo_uint32_t vipmi_swiden88T         __P((bo_uint32_t));
bo_uint32_t vipmi_swiden88TX        __P((bo_uint32_t));
/*
       void vipmi_swiden8_32        __P((bo_uint32_t&, bo_uint32_t&, bo_uint32_t));
 */

bo_uint32_t vipmi_uwiden88B         __P((bo_uint32_t));
bo_uint32_t vipmi_uwiden88T         __P((bo_uint32_t));
bo_uint32_t vipmi_uwiden88TX        __P((bo_uint32_t));
/*
       void vipmi_uwiden8_32        __P((bo_uint32_t&, bo_uint32_t&, bo_uint32_t));
 */

/* ------------------------------------------------------------------------- */

/*
 * Shift operations
 */

	/*
	 * Rotate left/right
	 */

bo_uint16_t vipmi_rotl_16           __P((bo_uint16_t, bo_uint16_t));
bo_uint16_t vipmi_rotr_16           __P((bo_uint16_t, bo_uint16_t));
bo_uint32_t vipmi_rotl_32           __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_rotr_32           __P((bo_uint32_t, bo_uint32_t));


	/*
	 * Unsigned rounding shift right
	 * @note: rounding half toward infinity
	 */

bo_uint16_t vipmi_urshr_16          __P((bo_uint16_t, bo_uint16_t));
bo_uint32_t vipmi_urshr_32          __P((bo_uint32_t, bo_uint32_t));
bo_uint64_t vipmi_urshr_64          __P((bo_uint64_t, bo_uint64_t));

/* ------------------------------------------------------------------------- */

/*
 * Logical and Compare operations
 */

	/*
	 * Parallel Signed/Unsigned Min/Max select
	 *
	 * Min(s1,s2) : result[0:7]   = min(s1[0:7],   s2[0:7])
	 *              result[8:15]  = min(s1[8:15],  s2[8:15])
	 *              result[16:23] = min(s1[16:23], s2[16:23])
	 *              result[24:31] = min(s1[24:31], s2[24:31])
	 *
	 * Max(s1,s2) : result[0:7]   = max(s1[0:7],   s2[0:7])
	 *              result[8:15]  = max(s1[8:15],  s2[8:15])
	 *              result[16:23] = max(s1[16:23], s2[16:23])
	 *              result[24:31] = max(s1[24:31], s2[24:31])
	 */

bo_uint32_t vipmi_smin8_32          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_smin16_32         __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_smax8_32          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_smax16_32         __P((bo_uint32_t, bo_uint32_t));

bo_uint32_t vipmi_umin8_32          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_umin16_32         __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_umax8_32          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_umax16_32         __P((bo_uint32_t, bo_uint32_t));


	/*
	 * Bitwise select
	 *
	 * bitift[rue]  (src1,src2,msk) : b(result) =  b(msk) ? b(src2) : b(src1)
	 * bitiff[alse] (src1,src2,msk) : b(result) = !b(msk) ? b(src2) : b(src1)
	 */

bo_uint8_t  vipmi_bitift_8          __P((bo_uint8_t,  bo_uint8_t,  bo_uint8_t));
bo_uint16_t vipmi_bitift_16         __P((bo_uint16_t, bo_uint16_t, bo_uint16_t));
bo_uint32_t vipmi_bitift_32         __P((bo_uint32_t, bo_uint32_t, bo_uint32_t));

bo_uint8_t  vipmi_bitiff_8          __P((bo_uint8_t,  bo_uint8_t,  bo_uint8_t));
bo_uint16_t vipmi_bitiff_16         __P((bo_uint16_t, bo_uint16_t, bo_uint16_t));
bo_uint32_t vipmi_bitiff_32         __P((bo_uint32_t, bo_uint32_t, bo_uint32_t));

/* ------------------------------------------------------------------------- */

/*
 * Integer Division operations
 */

# if (VipmF_IDIV_FI8 & __VIPM_TM_IDIV)
#  define vipmi_idiv_s8(a,b)     ((bo_int8_t)(((float)a) / ((float)b)))
#  define vipmi_idiv_u8(a,b)     ((bo_uint8_t)(((float)a) / ((float)b)))
# else
#  define vipmi_idiv_s8(a,b)     ((a) / (b))
#  define vipmi_idiv_u8(a,b)     vipmi_idiv_s8(a,b)
# endif

# if (VipmF_IDIV_FI16 & __VIPM_TM_IDIV)
#  define vipmi_idiv_s16(a,b)    ((bo_int16_t)(((float)a) / ((float)b)))
#  define vipmi_idiv_u16(a,b)    ((bo_uint16_t)(((float)a) / ((float)b)))
# else
#  define vipmi_idiv_s16(a,b)    ((a) / (b))
#  define vipmi_idiv_u16(a,b)    vipmi_idiv_s16(a,b)
# endif

# if (VipmF_IDIV_FI32 & __VIPM_TM_IDIV)
#  define vipmi_idiv_s32(a,b)    ((bo_int32_t)(((double)a) / ((double)b)))
#  define vipmi_idiv_u32(a,b)    ((bo_uint32_t)(((double)a) / ((double)b)))
# else
#  define vipmi_idiv_s32(a,b)    ((a) / (b))
#  define vipmi_idiv_u32(a,b)    vipmi_idiv_s32(a,b)
# endif

/* ------------------------------------------------------------------------- */

/*
 * [Parallel] Add and Subtract operations
 */

	/*
	 * addw88B  (a,x) : result[0:15]  = a[0:15]  + x[0:7]
	 *                  result[16:31] = a[16:31] + x[8:15]
	 *
	 * addw88T  (a,x) : result[0:15]  = a[0:15]  + x[15:23]
	 *                  result[16:31] = a[16:31] + x[24:31]
	 *
	 * addw88TX (a,x) : result[0:15]  = a[0:15]  + x[24:31]
	 *                  result[16:31] = a[16:31] + x[15:23]
	 */

bo_uint32_t vipmi_saddw88B          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_saddw88T          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_saddw88TX         __P((bo_uint32_t, bo_uint32_t));

bo_uint32_t vipmi_uaddw88B          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_uaddw88T          __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_uaddw88TX         __P((bo_uint32_t, bo_uint32_t));


	/*
	 * Unsigned sum [and accumulate] of bytes:
	 *
	 * usum8 (a,x) : result =     (x[0:7] + x[8:15] + x[16:23] + x[24:31])
	 * usuma8(a,x) : result = a + (x[0:7] + x[8:15] + x[16:23] + x[24:31])
	 */

bo_uint32_t vipmi_usum8_32          __P((bo_uint32_t));
bo_uint32_t vipmi_usuma8_32         __P((bo_uint32_t, bo_uint32_t));
bo_uint32_t vipmi_usum16_32         __P((bo_uint32_t));
bo_uint32_t vipmi_usuma16_32        __P((bo_uint32_t, bo_uint32_t));

/* ------------------------------------------------------------------------- */

/*
 * [Parallel] Multiply operations
 */

	/*
	 * Signed[/Unsigned] Multiply, Multiply-Accumulate, Multiply-Subtract
	 *
	 * mla (accum, multiplicand, multiplier) : result = accum + multiplicand * multiplier
	 * mls (accum, multiplicand, multiplier) : result = accum - multiplicand * multiplier
	 */

bo_int32_t    vipmi_smla_32         __P((bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t    vipmi_smlal_32        __P((bo_int64_t, bo_int32_t, bo_int32_t));
bo_int64_t    vipmi_smlaw_32        __P((bo_int64_t, bo_int64_t, bo_int32_t));
bo_int32_t    vipmi_smls_32         __P((bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t    vipmi_smlsl_32        __P((bo_int64_t, bo_int32_t, bo_int32_t));
bo_int64_t    vipmi_smlsw_32        __P((bo_int64_t, bo_int64_t, bo_int32_t));

bo_int64_t    vipmi_smla_64         __P((bo_int64_t, bo_int64_t, bo_int64_t));
bo_int64_t    vipmi_smls_64         __P((bo_int64_t, bo_int64_t, bo_int64_t));


bo_float32_t  vipmi_mla_f32         __P((bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_mlal_f32        __P((bo_float64_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_mlaw_f32        __P((bo_float64_t, bo_float64_t, bo_float32_t));
bo_float32_t  vipmi_mls_f32         __P((bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_mlsl_f32        __P((bo_float64_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_mlsw_f32        __P((bo_float64_t, bo_float64_t, bo_float32_t));

bo_float64_t  vipmi_mla_f64         __P((bo_float64_t, bo_float64_t, bo_float64_t));
bo_float64_t  vipmi_mls_f64         __P((bo_float64_t, bo_float64_t, bo_float64_t));


/* ------------------------------------------------------------------------- */

/*
 * dmua(x1,y1,x2,y2)       : x1 * x2 + y1 * y2             // double-mul-add
 * tmua(x1,y1,z1,x2,y2,z2) : x1 * x2 + y1 * y2 + z1 * z2   // tripple-mul-add
 * dcms(x1,y1,x2,y2)       : x1 * y2 - x2 * y1             // double-cross-mul-sub
 * dcmrs(x1,y1,x2,y2)      : -dcms(x2,y2,x1,y1)            // double-cross-mul-reverse-sub
 */

bo_int32_t  vipmi_dmua32_I16xI16    __P((bo_int32_t, bo_int32_t));
bo_int32_t  vipmi_dmua32_I16xU8B    __P((bo_int32_t, bo_int32_t));
bo_int32_t  vipmi_dmua32_I16xU8T    __P((bo_int32_t, bo_int32_t));
bo_int32_t  vipmi_tmua32_I16xU8B    __P((bo_int32_t, bo_int32_t, bo_int32_t));
bo_int32_t  vipmi_tmua32_I16xU8T    __P((bo_int32_t, bo_int32_t, bo_int32_t));
bo_int32_t  vipmi_tmuaw32_I16BxU8B  __P((bo_int32_t, bo_int32_t, bo_int32_t));
bo_int32_t  vipmi_tmuaw32_I16TxU8T  __P((bo_int32_t, bo_int32_t, bo_int32_t));


bo_int32_t  vipmi_dmua32            __P((bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t  vipmi_dmual32           __P((bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t  vipmi_dmuaw32           __P((bo_int64_t, bo_int64_t, bo_int32_t, bo_int32_t));
bo_int32_t  vipmi_dcms32            __P((bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t  vipmi_dcmsl32           __P((bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t  vipmi_dcmsw32           __P((bo_int64_t, bo_int64_t, bo_int32_t, bo_int32_t));
bo_int64_t  vipmi_dcmrsw32          __P((bo_int64_t, bo_int64_t, bo_int32_t, bo_int32_t));


bo_int64_t  vipmi_dmua64            __P((bo_int64_t, bo_int64_t, bo_int64_t, bo_int64_t));
bo_int64_t  vipmi_dcms64            __P((bo_int64_t, bo_int64_t, bo_int64_t, bo_int64_t));

/* ------------------------------------------------------------------------- */

bo_float32_t  vipmi_dmua_f32        __P((bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_dmual_f32       __P((bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_dmuaw_f32       __P((bo_float64_t, bo_float64_t, bo_float32_t, bo_float32_t));
bo_float32_t  vipmi_dcms_f32        __P((bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_dcmsl_f32       __P((bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_dcmsw_f32       __P((bo_float64_t, bo_float64_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_dcmrsw_f32      __P((bo_float64_t, bo_float64_t, bo_float32_t, bo_float32_t));


bo_float64_t  vipmi_dmua_f64        __P((bo_float64_t, bo_float64_t, bo_float64_t, bo_float64_t));
bo_float64_t  vipmi_dcms_f64        __P((bo_float64_t, bo_float64_t, bo_float64_t, bo_float64_t));

/* ========================================================================= */

/*
 * vectors: A, B
 * result : (A, B) = AxBx + AyBy
 */

# define vipmi_2d_dotprod(y1,x1,y2,x2)      vipmi_dmua32(x1,y1,x2,y2)
# define vipmi_2d_dotprodl(y1,x1,y2,x2)     vipmi_dmual32(x1,y1,x2,y2)
# define vipmi_2d_dotprodw(y1,x1,y2,x2)     vipmi_dmuaw32(x1,y1,x2,y2)
# define vipmi_2d_dotprod64(y1,x1,y2,x2)    vipmi_dmua64(x1,y1,x2,y2)

# define vipmi_f2d_dotprod(y1,x1,y2,x2)     vipmi_dmua_f32(x1,y1,x2,y2)
# define vipmi_f2d_dotprodl(y1,x1,y2,x2)    vipmi_dmual_f32(x1,y1,x2,y2)
# define vipmi_f2d_dotprodw(y1,x1,y2,x2)    vipmi_dmuaw_f32(x1,y1,x2,y2)
# define vipmi_f2d_dotprod64                vipmi_r2d_dotprod

# define vipmi_r2d_dotprod(y1,x1,y2,x2)     vipmi_dmua_f64(x1,y1,x2,y2)
# define vipmi_r2d_dotprodl                 vipmi_r2d_dotprod
# define vipmi_r2d_dotprodw                 vipmi_r2d_dotprod
# define vipmi_r2d_dotprod64                vipmi_r2d_dotprod

/* ------------------------------------------------------------------------- */

# define vipmi_2d_dotprod2(y1,x1)           vipmi_2d_dotprod(y1,x1,y1,x1)
# define vipmi_2d_dotprod2l(y1,x1)          vipmi_2d_dotprodl(y1,x1,y1,x1)
# define vipmi_2d_dotprod2w(y1,x1)          vipmi_dmua64(x1,y1,x1,y1)
# define vipmi_2d_dotprod2_64               vipmi_2d_dotprod2w

# define vipmi_f2d_dotprod2(y1,x1)          vipmi_f2d_dotprod(y1,x1,y1,x1)
# define vipmi_f2d_dotprod2l(y1,x1)         vipmi_f2d_dotprodl(y1,x1,y1,x1)
# define vipmi_f2d_dotprod2w                vipmi_r2d_dotprod2
# define vipmi_f2d_dotprod2_64              vipmi_f2d_dotprod2w

# define vipmi_r2d_dotprod2(y1,x1)          vipmi_r2d_dotprod(y1,x1,y1,x1)
# define vipmi_r2d_dotprod2l                vipmi_r2d_dotprod2
# define vipmi_r2d_dotprod2w                vipmi_r2d_dotprod2
# define vipmi_r2d_dotprod2_64              vipmi_r2d_dotprod2

/* ------------------------------------------------------------------------- */

/*
 * vectors: A, B
 * result : [A x B] = AxBy - AyBx
 */

# define vipmi_2d_vecprod(y1,x1,y2,x2)      vipmi_dcms32(x1,y1,x2,y2)
# define vipmi_2d_vecprodl(y1,x1,y2,x2)     vipmi_dcmsl32(x1,y1,x2,y2)
# define vipmi_2d_vecprodw(y1,x1,y2,x2)     vipmi_dcmsw32(x1,y1,x2,y2)
# define vipmi_2d_vecprod64(y1,x1,y2,x2)    vipmi_dcms64(x1,y1,x2,y2)

# define vipmi_f2d_vecprod(y1,x1,y2,x2)     vipmi_dcms_f32(x1,y1,x2,y2)
# define vipmi_f2d_vecprodl(y1,x1,y2,x2)    vipmi_dcmsl_f32(x1,y1,x2,y2)
# define vipmi_f2d_vecprodw(y1,x1,y2,x2)    vipmi_dcmsw_f32(x1,y1,x2,y2)
# define vipmi_f2d_vecprod64                vipmi_r2d_vecprod

# define vipmi_r2d_vecprod(y1,x1,y2,x2)     vipmi_dcms_f64(x1,y1,x2,y2)
# define vipmi_r2d_vecprodl                 vipmi_r2d_vecprod
# define vipmi_r2d_vecprodw                 vipmi_r2d_vecprod
# define vipmi_r2d_vecprod64                vipmi_r2d_vecprod

/* ------------------------------------------------------------------------- */

/*
 * points: a,b,c
 * result: [(b - a) x (c - a)]
 */

bo_int32_t    vipmi_XNS(2d)(crossprod)  __P((bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t    vipmi_XNS(2d)(crossprodl) __P((bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t, bo_int32_t));
bo_int64_t    vipmi_XNS(2d)(crossprodw) __P((bo_int64_t, bo_int64_t, bo_int64_t, bo_int64_t, bo_int32_t, bo_int32_t));
bo_int64_t    vipmi_XNS(2d)(crossprod64)__P((bo_int64_t, bo_int64_t, bo_int64_t, bo_int64_t, bo_int64_t, bo_int64_t));

bo_float32_t  vipmi_XNS(f2d)(crossprod) __P((bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_XNS(f2d)(crossprodl)__P((bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t, bo_float32_t));
bo_float64_t  vipmi_XNS(f2d)(crossprodw)__P((bo_float64_t, bo_float64_t, bo_float64_t, bo_float64_t, bo_float32_t, bo_float32_t));
# define vipmi_f2d_crossprod64          vipmi_r2d_crossprod

bo_float64_t  vipmi_XNS(r2d)(crossprod) __P((bo_float64_t, bo_float64_t, bo_float64_t, bo_float64_t, bo_float64_t, bo_float64_t));
# define vipmi_r2d_crossprodl           vipmi_r2d_crossprod
# define vipmi_r2d_crossprodw           vipmi_r2d_crossprod
# define vipmi_r2d_crossprod64          vipmi_r2d_crossprod

/* ------------------------------------------------------------------------- */

bo_int32_t    vipmi_XNS(2d)(crossprod,mm)   __P((bo_int32_t const [/*6*/]));
bo_int64_t    vipmi_XNS(2d)(crossprodl,mm)  __P((bo_int32_t const [/*6*/]));
bo_int64_t    vipmi_XNS(2d)(crossprod64,mm) __P((bo_int64_t const [/*6*/]));

bo_float32_t  vipmi_XNS(f2d)(crossprod,mm)  __P((bo_float32_t const [/*6*/]));
bo_float64_t  vipmi_XNS(f2d)(crossprodl,mm) __P((bo_float32_t const [/*6*/]));
# define vipmi_f2d_crossprod64_$mm          vipmi_r2d_crossprod_$mm

bo_float64_t  vipmi_XNS(r2d)(crossprod,mm)  __P((bo_float64_t const [/*6*/]));
# define vipmi_r2d_crossprodl_$mm           vipmi_r2d_crossprod_$mm
# define vipmi_r2d_crossprod64_$mm          vipmi_r2d_crossprod_$mm


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

# include <Vipm/Mmintrin_inl.h>


__MYC_END_CDECLS

#endif /* !defined(__VIPM_MMINTRIN_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
