/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Mmintrin_inl.h	(V.Sokur)
 * Vipm/Mmintrin_inl.h
 *
 * [ mmi - my machine intrinsic(s)/instruction/... ]
 */

/******************************************************************************
 * %% BeginSection: function definitions
 */
# pragma mark -
# pragma mark [ function definitions ]
# pragma mark -

/*
 * float32, IEEE 754
 *
 * huge_val (inf)
 * nan
 */

# define VipmK_F32_MAX              3.40282347e+38F
# define VipmK_F32_HUGE             3.40282347e+38F
/*
# define VipmK_F32_INF              ...
# define VipmK_F32_NAN              ...
 */

/* @todo: fixme
# define VipmK_F32_HUGE_VAL        (__extension__ 0x1.0p255F)
 */
# ifdef _MSC_VER
#  define VipmK_F32_HUGE_VAL       (1e25F * 1e25F)
# else
#  define VipmK_F32_HUGE_VAL             \
    (__extension__ ((union {             \
            bo_fint32_t     __i;         \
            bo_float32_t    __f;         \
         }) { .__i = 0x7F800000UL }).__f \
    )
# endif
# define VipmK_F32_INF              VipmK_F32_HUGE_VAL

# ifdef _MSC_VER
union __msc_vipm_nanf32_u {
	bo_uint8_t    __b[4];
	bo_float32_t  __f;
};
static union __msc_vipm_nanf32_u _S_msc_vipm_nanf32_bytes = {
#  if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
	{ 0x7F,0xC0,0,0 }
#  else
	{ 0,0,0xC0,0x7F }
#  endif
};
#  define VipmK_F32_NAN            (_S_msc_vipm_nanf32_bytes.__f)
# else
#  define VipmK_F32_NAN                  \
    (__extension__ ((union {             \
            bo_fint32_t     __i;         \
            bo_float32_t    __f;         \
         }) { .__i = 0x7FC00000UL }).__f \
    )
# endif

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi_isinff(float aValue)
{
	union {
		bo_uint32_t   __i;
		bo_float32_t  __f;
	} x;
	x.__f = aValue;
	return (0x7F800000 == (x.__i & 0x7FFFFFFF));
}

_VIPM_INLINE int vipmi_isnanf(float aValue)
{
	union {
		bo_uint32_t   __i;
		bo_float32_t  __f;
	} x;
	x.__f = aValue;
	return (0x7F800000 < (x.__i & 0x7FFFFFFF));
}

/* ========================================================================= */

/*
 * float64, IEEE 754
 *
 * huge_val (inf)
 * nan
 */

/*
# define VipmK_F64_INF              ...
# define VipmK_F64_NAN              ...
 */

/* @todo: fixme
# define VipmK_F64_HUGE_VAL        (__extension__ 0x1.0p2047)
 */
# ifdef _MSC_VER
#if 0
union __msc_vipm_hugef64_u {
	bo_uint8_t  __b[8];
	double      __f;
};
static union __msc_vipm_hugef64_u _S_msc_vipm_hugef64_bytes = {
#  if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
	{ 0x7F,0xF0,0,0,0,0,0,0 }
#  elif 0 && (BO_BYTE_ORDER == MIDDLE_BYTE_ORDER)
	{ 0,0,0xF0,0x7F,0,0,0,0 }
#  else
	{ 0,0,0,0,0,0,0xF0,0x7F }
#  endif
};
#  define VipmK_F64_HUGE_VAL       (_S_msc_vipm_hugef64_bytes.__f)
#endif /* if 0 */
#  define VipmK_F64_HUGE_VAL       (1e250 * 1e250)
# else
#  define VipmK_F64_HUGE_VAL                      \
    (__extension__ ((union {                      \
            bo_uint64_t    __i;                   \
            bo_float64_t   __f;                   \
         }) { .__i = 0x7FF0000000000000ULL }).__f \
    )
# endif
# define VipmK_F64_INF              VipmK_F64_HUGE_VAL

# ifdef _MSC_VER
union __msc_vipm_nanf64_u {
	bo_uint8_t    __b[8];
	bo_float64_t  __f;
};
static union __msc_vipm_nanf64_u _S_msc_vipm_nanf64_bytes = {
#  if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
	{ 0x7F,0xF8,0,0,0,0,0,0 }
#  elif 0 && (BO_BYTE_ORDER == MIDDLE_BYTE_ORDER)
	{ 0,0,0xF8,0x7F,0,0,0,0 }
#  else
	{ 0,0,0,0,0,0,0xF8,0x7F }
#  endif
};
#  define VipmK_F64_NAN            (_S_msc_vipm_nanf64_bytes.__f)
# else
#  define VipmK_F64_NAN                           \
    (__extension__ ((union {                      \
            bo_uint64_t    __i;                   \
            bo_float64_t   __f;                   \
         }) { .__i = 0x7FF8000000000000ULL }).__f \
    )
# endif

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi_isinfd(double aValue)
{
	union {
		bo_uint64_t   __i;
		bo_float64_t  __f;
	} x;
	x.__f = aValue;
	return ((0x7FF00000 == ((bo_uint32_t)(x.__i >> 32) & 0x7FFFFFFF)) && ((bo_uint32_t)(x.__i  & 0xFFFFFFFF) == 0));
}

_VIPM_INLINE int vipmi_isnand(double aValue)
{
	union {
		bo_uint64_t   __i;
		bo_float64_t  __f;
	} x;
	x.__f = aValue;
	return (0x7FF00000 < ((bo_uint32_t)(x.__i >> 32) & 0x7FFFFFFF) + ((bo_uint32_t)(x.__i  & 0xFFFFFFFF) != 0));
}


/* ========================================================================= */

/* @todo: fixme */
# if 0
# define vipmi_abs                  abs
# ifdef __LLP64__
#  define vipmi_absl                llabs
#  define vipmi_absll               llabs
# else
#  define vipmi_absl                labs
#  define vipmi_absll               llabs
# endif
# else /* if (1) */

#  ifdef _MSC_VER
_VIPM_INLINE vipm_int __inl_vipmi_abs(vipm_int x)       { return (x >= 0 ? x : -x); }
#   define vipmi_abs(x) __inl_vipmi_abs(x)
#  else
#   define vipmi_abs(x)                                 \
    __extension__ ({                                    \
        vipm_int ___x = (x);                            \
        /* return */ (___x >= 0 ? ___x : -___x);        \
    })
#  endif

#  ifdef _MSC_VER
_VIPM_INLINE long long __inl_vipmi_absll(long long x)   { return (x >= 0 ? x : -x); }
#   define vipmi_absll(x) __inl_vipmi_absll(x)
#  else
#   define vipmi_absll(x)                               \
    __extension__ ({                                    \
        long long ___x = (x);                           \
        /* return */ (___x >= 0 ? ___x : -___x);        \
    })
#  endif

#  if (64 <= BO_WIDTHOF_MWORD)
#   ifdef _MSC_VER
_VIPM_INLINE vipm_long __inl_vipmi_absl(vipm_long x)    { return (x >= 0 ? x : -x); }
#    define vipmi_absl(x) __inl_vipmi_absl(x)
#   else
#    define vipmi_absl(x)                               \
    __extension__ ({                                    \
        vipm_long ___x = (x);                           \
        /* return */ (___x >= 0 ? ___x : -___x);        \
    })
#   endif
#  else
#   define vipmi_absl(x)            vipmi_abs(x)
#  endif
# endif /* if (0) */


/* ========================================================================= */

# if (defined(_MSC_VER) && _MSC_VER < 1800)
_VIPM_INLINE float vipmi_log2f(float x)       { return (vipmi_logf(x) / VipmK_F64_M_LN2); }
# else
#  define vipmi_log2f               log2f
# endif

# if (defined(_MSC_VER) && _MSC_VER < 1800)
_VIPM_INLINE double vipmi_log2d(double x)       { return (vipmi_logd(x) / VipmK_F64_M_LN2); }
# else
#  define vipmi_log2d               log2
# endif


/* ========================================================================= */

_VIPM_INLINE int vipmi_eequalfS(float x, float y, float e)
{
	return ((vipmi_absf(x) <= FLT_EPSILON) ? (vipmi_absf(x - y) <= FLT_EPSILON)
	      : (vipmi_absf(y) <= FLT_EPSILON) ? (vipmi_absf(x - y) <= FLT_EPSILON)
	              : ((vipmi_absf(x - y) / (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(x) : vipmi_absf(y))) <= e));
}

_VIPM_INLINE int vipmi_aequalfS(float x, float y, float e)
{
#if 0
	return ((vipmi_absf(x) <= FLT_EPSILON) ? (vipmi_absf(y) <= FLT_EPSILON)
	      : (vipmi_absf(y) <= FLT_EPSILON) ? (vipmi_absf(x) <= FLT_EPSILON)
	              : ((vipmi_absf(x - y) / (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x))) <= e));
#else
	return ((vipmi_absf(x) <= FLT_EPSILON) ? (vipmi_absf(x - y) <= 2 * FLT_EPSILON)
	      : (vipmi_absf(y) <= FLT_EPSILON) ? (vipmi_absf(x - y) <= 2 * FLT_EPSILON)
	              : ((vipmi_absf(x - y) / (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x))) <= e));
#endif
}

_VIPM_INLINE int vipmi_lessfS(float x, float y, float e)
{
	return ((vipmi_absf(x) <= FLT_EPSILON) ? ((y - x) > FLT_EPSILON)
	      : (vipmi_absf(y) <= FLT_EPSILON) ? ((y - x) > FLT_EPSILON)
	              : (((y - x) / (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x))) > e));
}

_VIPM_INLINE int vipmi_greaterfS(float x, float y, float e)
{
	return ((vipmi_absf(x) <= FLT_EPSILON) ? ((x - y) > FLT_EPSILON)
	      : (vipmi_absf(y) <= FLT_EPSILON) ? ((x - y) > FLT_EPSILON)
	              : (((x - y) / (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x))) > e));
}

_VIPM_INLINE int vipmi_lequfS(float x, float y, float e)
{
	return ((vipmi_absf(x) <= FLT_EPSILON) ? ((x - y) <= FLT_EPSILON)
	      : (vipmi_absf(y) <= FLT_EPSILON) ? ((x - y) <= FLT_EPSILON)
	              : (((x - y) / (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x))) <= e));
}

_VIPM_INLINE int vipmi_gequfS(float x, float y, float e)
{
	return ((vipmi_absf(x) <= FLT_EPSILON) ? ((y - x) <= FLT_EPSILON)
	      : (vipmi_absf(y) <= FLT_EPSILON) ? ((y - x) <= FLT_EPSILON)
	              : (((y - x) / (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x))) <= e));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi_eequaldS(double x, double y, double e)
{
	return ((vipmi_absd(x) <= DBL_EPSILON) ? (vipmi_absd(x - y) <= DBL_EPSILON)
	      : (vipmi_absd(y) <= DBL_EPSILON) ? (vipmi_absd(x - y) <= DBL_EPSILON)
	              : ((vipmi_absd(x - y) / (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(x) : vipmi_absd(y))) <= e));
}

_VIPM_INLINE int vipmi_aequaldS(double x, double y, double e)
{
#if 0
	return ((vipmi_absd(x) <= DBL_EPSILON) ? (vipmi_absd(y) <= DBL_EPSILON)
	      : (vipmi_absd(y) <= DBL_EPSILON) ? (vipmi_absd(x) <= DBL_EPSILON)
	              : ((vipmi_absd(x - y) / (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x))) <= e));
#else
	return ((vipmi_absd(x) <= DBL_EPSILON) ? (vipmi_absd(x - y) <= 2 * DBL_EPSILON)
	      : (vipmi_absd(y) <= DBL_EPSILON) ? (vipmi_absd(x - y) <= 2 * DBL_EPSILON)
	              : ((vipmi_absd(x - y) / (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x))) <= e));
#endif
}

_VIPM_INLINE int vipmi_lessdS(double x, double y, double e)
{
	return ((vipmi_absd(x) <= DBL_EPSILON) ? ((y - x) > DBL_EPSILON)
	      : (vipmi_absd(y) <= DBL_EPSILON) ? ((y - x) > DBL_EPSILON)
	              : (((y - x) / (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x))) > e));
}

_VIPM_INLINE int vipmi_greaterdS(double x, double y, double e)
{
	return ((vipmi_absd(x) <= DBL_EPSILON) ? ((x - y) > DBL_EPSILON)
	      : (vipmi_absd(y) <= DBL_EPSILON) ? ((x - y) > DBL_EPSILON)
	              : (((x - y) / (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x))) > e));
}

_VIPM_INLINE int vipmi_lequdS(double x, double y, double e)
{
	return ((vipmi_absd(x) <= DBL_EPSILON) ? ((x - y) <= DBL_EPSILON)
	      : (vipmi_absd(y) <= DBL_EPSILON) ? ((x - y) <= DBL_EPSILON)
	              : (((x - y) / (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x))) <= e));
}

_VIPM_INLINE int vipmi_gequdS(double x, double y, double e)
{
	return ((vipmi_absd(x) <= DBL_EPSILON) ? ((y - x) <= DBL_EPSILON)
	      : (vipmi_absd(y) <= DBL_EPSILON) ? ((y - x) <= DBL_EPSILON)
	              : (((y - x) / (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x))) <= e));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi_eequalf(float x, float y, float e)
{
	return ((vipmi_absf(x - y) <= FLT_EPSILON) ||
			(vipmi_absf(x - y) <= (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(x) : vipmi_absf(y)) * e));
}

_VIPM_INLINE int vipmi_aequalf(float x, float y, float e)
{
	return ((vipmi_absf(x - y) <= FLT_EPSILON) ||
			(vipmi_absf(x - y) <= (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x)) * e));
}

_VIPM_INLINE int vipmi_lessf(float x, float y, float e)
{
	return (((y - x) > FLT_EPSILON) ||
			((y - x) > (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x)) * e));
}

_VIPM_INLINE int vipmi_greaterf(float x, float y, float e)
{
	return (((x - y) > FLT_EPSILON) ||
			((x - y) > (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x)) * e));
}

_VIPM_INLINE int vipmi_lequf(float x, float y, float e)
{
	return (((x - y) <= FLT_EPSILON) ||
			((x - y) <= (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x)) * e));
}

_VIPM_INLINE int vipmi_gequf(float x, float y, float e)
{
	return (((y - x) <= FLT_EPSILON) ||
			((y - x) <= (vipmi_absf(x) < vipmi_absf(y) ? vipmi_absf(y) : vipmi_absf(x)) * e));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi_eequald(double x, double y, double e)
{
	return ((vipmi_absd(x - y) <= DBL_EPSILON) ||
			(vipmi_absd(x - y) <= (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(x) : vipmi_absd(y)) * e));
}

_VIPM_INLINE int vipmi_aequald(double x, double y, double e)
{
	return ((vipmi_absd(x - y) <= DBL_EPSILON) ||
			(vipmi_absd(x - y) <= (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x)) * e));
}

_VIPM_INLINE int vipmi_lessd(double x, double y, double e)
{
	return (((y - x) > DBL_EPSILON) ||
			((y - x) > (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x)) * e));
}

_VIPM_INLINE int vipmi_greaterd(double x, double y, double e)
{
	return (((x - y) > DBL_EPSILON) ||
			((x - y) > (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x)) * e));
}

_VIPM_INLINE int vipmi_lequd(double x, double y, double e)
{
	return (((x - y) <= DBL_EPSILON) ||
			((x - y) <= (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x)) * e));
}

_VIPM_INLINE int vipmi_gequd(double x, double y, double e)
{
	return (((y - x) <= DBL_EPSILON) ||
			((y - x) <= (vipmi_absd(x) < vipmi_absd(y) ? vipmi_absd(y) : vipmi_absd(x)) * e));
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi__equalf  (float x, float y) { return (vipmi_absf(x - y) <= FLT_EPSILON); }
_VIPM_INLINE int vipmi__lessf   (float x, float y) { return ((y - x) > FLT_EPSILON); }
_VIPM_INLINE int vipmi__greaterf(float x, float y) { return ((x - y) > FLT_EPSILON); }
_VIPM_INLINE int vipmi__lequf   (float x, float y) { return ((x - y) <= FLT_EPSILON); }
_VIPM_INLINE int vipmi__gequf   (float x, float y) { return ((y - x) <= FLT_EPSILON); }

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi__equald  (double x, double y) { return (vipmi_absd(x - y) <= /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__lessd   (double x, double y) { return ((y - x) > /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__greaterd(double x, double y) { return ((x - y) > /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__lequd   (double x, double y) { return ((x - y) <= /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__gequd   (double x, double y) { return ((y - x) <= /* DBL_EPSILON */FLT_EPSILON); }

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi__equal0f  (float x) { return (vipmi_absf(x) <= FLT_EPSILON); }
_VIPM_INLINE int vipmi__less0f   (float x) { return (-x > FLT_EPSILON); }
_VIPM_INLINE int vipmi__greater0f(float x) { return (x > FLT_EPSILON); }
_VIPM_INLINE int vipmi__lequ0f   (float x) { return (x <= FLT_EPSILON); }
_VIPM_INLINE int vipmi__gequ0f   (float x) { return (-x <= FLT_EPSILON); }

/* ------------------------------------------------------------------------- */

_VIPM_INLINE int vipmi__equal0d  (double x) { return (vipmi_absd(x) <= /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__less0d   (double x) { return (-x > /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__greater0d(double x) { return (x > /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__lequ0d   (double x) { return (x <= /* DBL_EPSILON */FLT_EPSILON); }
_VIPM_INLINE int vipmi__gequ0d   (double x) { return (-x <= /* DBL_EPSILON */FLT_EPSILON); }


/* ========================================================================= */

/*
 * llrint -- round{ bo_int64_t, float{32,64,128} }
 */

/*
 * Double variant
 */

_VIPM_INLINE bo_int64_t vipmi_llrintd(double aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(_A_X86_64) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128d val = _mm_set_sd(aValue);
	return _mm_cvtsd_si64(val);

# elif defined(_MSC_VER) && defined(_A_X86)

	/* @todo: fixme - implement */
	vipm_int rv;
	__asm {
		fld aValue;
		fistp rv;
	}
	return ((bo_int64_t)rv);

# elif defined(_ICC_VER) || defined(_GCC_VER)
#  ifdef __VIPM_ARM_VFP
#   ifdef _A_ARM64

	/* @note: round half-to-even */
	bo_int64_t rv;
	__asm__ __volatile__ (
		_AAS_RND_F64TOSI64([rv], [val])
		: [rv]  "=r"(rv)
		: [val] "w" (aValue)
	);
	return (rv);

#   else /* _A_ARM32 */

/* @todo: fixme - implement
	bo_int64_t rv;
	float tmp;
	__asm__ __volatile__ (
		_AAS_RND_VF64TOSI([tmp], [val])
		_AAS_VMOV([rv], [tmp])
		: [rv]  "=r"(rv),
		  [tmp] "=w"(tmp)
		: [val] "w" (aValue)
	);
	return (rv);
 */
	return (bo_int64_t)llrint(aValue);

#   endif /* _A_ARM64 */
#  else
/*
	__asm__ __volatile__ (
		"fistpl %0"
		: "=m"(rv)
		: "t" (aValue)
		: "st"
	);
 */
	return (bo_int64_t)llrint(aValue);

#  endif
# else /* any compiler/machine */

	bo_int64_t rv;
#if 1
	int rmode = fegetround();
	return (
		   (FE_TONEAREST == rmode) ? (rv = (bo_int64_t)(aValue += (0 <= aValue ? 0.5 : -0.5)),
		                                   (vipmi__equald(aValue,rv) ? (2 * (rv / 2)) : rv))
		:     (FE_UPWARD == rmode) ? (rv = (bo_int64_t)aValue,
		                                   ((aValue <= 0 || vipmi__equald(aValue, rv)) ? rv : (rv + 1)))
		:   (FE_DOWNWARD == rmode) ? (rv = (bo_int64_t)aValue,
		                                   ((0 <= aValue || vipmi__equald(aValue, rv)) ? rv : (rv - 1)))
		: (FE_TOWARDZERO == rmode) ? (rv = (bo_int64_t)aValue)
		                           : (rv = (bo_int64_t)aValue)
	);
#else
	switch (fegetround()) {
		case FE_TONEAREST:
			rv = (bo_int64_t)(aValue += (0 <= aValue ? 0.5 : -0.5));
			return (vipmi__equald(aValue, rv) ? (2 * (rv / 2)) : rv);
			/* NOTREACHED */
		case FE_UPWARD:
			rv = (bo_int64_t)aValue;
			return ((aValue <= 0 || vipmi__equald(aValue, rv)) ? rv : (rv + 1));
			/* NOTREACHED */
		case FE_DOWNWARD:
			rv = (bo_int64_t)aValue;
			return ((0 <= aValue || vipmi__equald(aValue, rv)) ? rv : (rv - 1));
			/* NOTREACHED */
		case FE_TOWARDZERO:
		default:
			return ((bo_int64_t)aValue);
			/* NOTREACHED */
	}
#endif /* if 1 */

# endif
}

/* ========================================================================= */

/*
 * Float variant
 */

_VIPM_INLINE bo_int64_t vipmi_llrintf(float aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(_A_X86_64) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128 val = _mm_set_ss(aValue);
	return _mm_cvtss_si64(val);

# elif defined(_MSC_VER) && defined(_A_X86)

	/* @todo: fixme - implement */
	vipm_int rv;
	__asm {
		fld aValue;
		fistp rv;
	}
	return ((bo_int64_t)rv);

# elif defined(_ICC_VER) || defined(_GCC_VER)
#  ifdef __VIPM_ARM_VFP
#   ifdef _A_ARM64

	/* @note: round half-to-even */
	bo_int64_t rv;
	__asm__ __volatile__ (
		_AAS_RND_FTOSI64([rv], [val])
		: [rv]  "=r"(rv)
		: [val] "w" (aValue)
	);
	return (rv);

#   else /* _A_ARM32 */

/* @todo: fixme - implement
	bo_int64_t rv;
	float tmp;
	__asm__ __volatile__ (
        _AAS_RND_VFTOSI([tmp], [val])
        _AAS_VMOV([rv], [tmp])
		: [rv]  "=r"(rv),
		  [tmp] "=t"(tmp)
		: [val] "t" (aValue)
	);
	return (rv);
 */
	return (bo_int64_t)llrintf(aValue);

#   endif /* _A_ARM64 */
#  else
/*
	__asm__ __volatile__ (
		"fistpl %0"
		: "=m"(rv)
		: "t" (aValue)
		: "st"
	);
 */
    return (bo_int64_t)llrintf(aValue);

#  endif
# else /* any compiler/machine */

	bo_int64_t rv;
#if 1
	int rmode = fegetround();
	return (
		   (FE_TONEAREST == rmode) ? (rv = (bo_int64_t)(aValue += (0 <= aValue ? 0.5F : -0.5F)),
		                             (vipmi__equalf(aValue, rv) ? (2 * (rv / 2)) : rv))
		:     (FE_UPWARD == rmode) ? (rv = (bo_int64_t)aValue,
		                             ((aValue <= 0 || vipmi__equalf(aValue, rv)) ? rv : (rv + 1)))
		:   (FE_DOWNWARD == rmode) ? (rv = (bo_int64_t)aValue,
		                             ((0 <= aValue || vipmi__equalf(aValue, rv)) ? rv : (rv - 1)))
		: (FE_TOWARDZERO == rmode) ? (rv = (bo_int64_t)aValue)
		                           : (rv = (bo_int64_t)aValue)
	);
#else
	switch (fegetround()) {
		case FE_TONEAREST:
			rv = (bo_int64_t)(aValue += (0 <= aValue ? 0.5F : -0.5F));
			return (vipmi__equalf(aValue, rv) ? (2 * (rv / 2)) : rv);
			/* NOTREACHED */
		case FE_UPWARD:
			rv = (bo_int64_t)aValue;
			return ((aValue <= 0 || vipmi__equalf(aValue, rv)) ? rv : (rv + 1));
			/* NOTREACHED */
		case FE_DOWNWARD:
			rv = (bo_int64_t)aValue;
			return ((0 <= aValue || vipmi__equalf(aValue, rv)) ? rv : (rv - 1));
			/* NOTREACHED */
		case FE_TOWARDZERO:
		default:
			return ((bo_int64_t)aValue);
			/* NOTREACHED */
	}
#endif /* if 1 */

# endif
}

/* ========================================================================= */

/*
 * rint -- round{ int, float{32,64,128} }
 */

/*
 * Double variant
 */

_VIPM_INLINE vipm_int vipmi_rintd(double aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(_A_X86_64) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128d val = _mm_set_sd(aValue);
	return _mm_cvtsd_si32(val);

# elif defined(_MSC_VER) && defined(_A_X86)

	vipm_int rv;
	__asm {
		fld aValue;
		fistp rv;
	}
	return (rv);

# elif defined(_ICC_VER) || defined(_GCC_VER)
#  ifdef __VIPM_ARM_VFP
#   ifdef _A_ARM64

	/* @note: round half-to-even */
	vipm_int rv;
	__asm__ __volatile__ (
		_AAS_RND_F64TOSI([rv], [val])
		: [rv]  "=r"(rv)
		: [val] "w" (aValue)
	);
	return (rv);

#   else /* _A_ARM32 */

	/* @todo: fixme; round half-to-inf (?), should be round half-to-even */
	vipm_int rv;
	float tmp;
	__asm__ __volatile__ (
		_AAS_RND_VF64TOSI([tmp], [val])
		_AAS_VMOV([rv], [tmp])
		: [rv]  "=r"(rv),
		  [tmp] "=w"(tmp)
		: [val] "w" (aValue)
	);
	return (rv);

#   endif /* _A_ARM64 */
#  else
/*
	__asm__ __volatile__ (
		"fistpl %0"
		: "=m"(rv)
		: "t" (aValue)
		: "st"
	);
 */
	return (vipm_int)lrint(aValue);

#  endif
# else /* any compiler/machine */

	vipm_int rv;
#if 1
	int rmode = fegetround();
	return (
		   (FE_TONEAREST == rmode) ? (rv = (vipm_int)(aValue += (0 <= aValue ? 0.5 : -0.5)),
		                                   (vipmi__equald(aValue,rv) ? (2 * (rv / 2)) : rv))
		:     (FE_UPWARD == rmode) ? (rv = (vipm_int)aValue,
		                                   ((aValue <= 0 || vipmi__equald(aValue, rv)) ? rv : (rv + 1)))
		:   (FE_DOWNWARD == rmode) ? (rv = (vipm_int)aValue,
		                                   ((0 <= aValue || vipmi__equald(aValue, rv)) ? rv : (rv - 1)))
		: (FE_TOWARDZERO == rmode) ? (rv = (vipm_int)aValue)
		                           : (rv = (vipm_int)aValue)
	);
#else
	switch (fegetround()) {
		case FE_TONEAREST:
			rv = (vipm_int)(aValue += (0 <= aValue ? 0.5 : -0.5));
			return (vipmi__equald(aValue, rv) ? (2 * (rv / 2)) : rv);
			/* NOTREACHED */
		case FE_UPWARD:
			rv = (vipm_int)aValue;
			return ((aValue <= 0 || vipmi__equald(aValue, rv)) ? rv : (rv + 1));
			/* NOTREACHED */
		case FE_DOWNWARD:
			rv = (vipm_int)aValue;
			return ((0 <= aValue || vipmi__equald(aValue, rv)) ? rv : (rv - 1));
			/* NOTREACHED */
		case FE_TOWARDZERO:
		default:
			return ((vipm_int)aValue);
			/* NOTREACHED */
	}
#endif /* if 1 */

# endif
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_int vipmi_rfloord(double aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128d val = _mm_set_sd(aValue);
	vipm_int itmp = _mm_cvtsd_si32(val);
	return (itmp - _mm_movemask_pd(_mm_cmplt_sd(val, _mm_cvtsi32_sd(val, itmp))));

# elif defined(_GCC_VER)

	vipm_int itmp = (vipm_int)aValue;
	return (itmp - (aValue < itmp));

# else

	vipm_int itmp = vipmi_rintd(aValue);
	float diff = (float)(aValue - itmp);
	return (itmp - (diff < 0));

# endif
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_int vipmi_rceild(double aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128d val = _mm_set_sd(aValue);
	vipm_int itmp = _mm_cvtsd_si32(val);
	return (itmp + _mm_movemask_pd(_mm_cmplt_sd(_mm_cvtsi32_sd(val, itmp), val)));

# elif defined(_GCC_VER)

	vipm_int itmp = (vipm_int)aValue;
	return (itmp + (itmp < aValue));

# else

	vipm_int itmp = vipmi_rintd(aValue);
	float diff = (float)(itmp - aValue);
	return (itmp + (diff < 0));

# endif
}

/* ------------------------------------------------------------------------- */

/* @todo: fixme */
# define vipmi_rtozero(x)           vipmi_rtruncd(x)

_VIPM_INLINE vipm_int vipmi_rtruncd(double aValue) { return (vipm_int)aValue; }

/* ========================================================================= */

/*
 * Float variant
 */

_VIPM_INLINE vipm_int vipmi_rintf(float aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(_A_X86_64) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128 val = _mm_set_ss(aValue);
	return _mm_cvtss_si32(val);

# elif defined(_MSC_VER) && defined(_A_X86)

	vipm_int rv;
	__asm {
		fld aValue;
		fistp rv;
	}
	return (rv);

# elif defined(_ICC_VER) || defined(_GCC_VER)
#  ifdef __VIPM_ARM_VFP
#   ifdef _A_ARM64

	/* @note: round half-to-even */
	vipm_int rv;
	__asm__ __volatile__ (
		_AAS_RND_FTOSI([rv], [val])
		: [rv]  "=r"(rv)
		: [val] "w" (aValue)
	);
	return (rv);

#   else /* _A_ARM32 */

	/* @todo: fixme; round half-to-inf (?), should be round half-to-even */
	vipm_int rv;
	float tmp;
	__asm__ __volatile__ (
        _AAS_RND_VFTOSI([tmp], [val])
        _AAS_VMOV([rv], [tmp])
		: [rv]  "=r"(rv),
		  [tmp] "=t"(tmp)
		: [val] "t" (aValue)
	);
	return (rv);

#   endif /* _A_ARM64 */
#  else
/*
	__asm__ __volatile__ (
		"fistpl %0"
		: "=m"(rv)
		: "t" (aValue)
		: "st"
	);
 */
    return (vipm_int)lrintf(aValue);

#  endif
# else /* any compiler/machine */

	vipm_int rv;
#if 1
	int rmode = fegetround();
	return (
		   (FE_TONEAREST == rmode) ? (rv = (vipm_int)(aValue += (0 <= aValue ? 0.5F : -0.5F)),
		                             (vipmi__equalf(aValue, rv) ? (2 * (rv / 2)) : rv))
		:     (FE_UPWARD == rmode) ? (rv = (vipm_int)aValue,
		                             ((aValue <= 0 || vipmi__equalf(aValue, rv)) ? rv : (rv + 1)))
		:   (FE_DOWNWARD == rmode) ? (rv = (vipm_int)aValue,
		                             ((0 <= aValue || vipmi__equalf(aValue, rv)) ? rv : (rv - 1)))
		: (FE_TOWARDZERO == rmode) ? (rv = (vipm_int)aValue)
		                           : (rv = (vipm_int)aValue)
	);
#else
	switch (fegetround()) {
		case FE_TONEAREST:
			rv = (vipm_int)(aValue += (0 <= aValue ? 0.5F : -0.5F));
			return (vipmi__equalf(aValue, rv) ? (2 * (rv / 2)) : rv);
			/* NOTREACHED */
		case FE_UPWARD:
			rv = (vipm_int)aValue;
			return ((aValue <= 0 || vipmi__equalf(aValue, rv)) ? rv : (rv + 1));
			/* NOTREACHED */
		case FE_DOWNWARD:
			rv = (vipm_int)aValue;
			return ((0 <= aValue || vipmi__equalf(aValue, rv)) ? rv : (rv - 1));
			/* NOTREACHED */
		case FE_TOWARDZERO:
		default:
			return ((vipm_int)aValue);
			/* NOTREACHED */
	}
#endif /* if 1 */

# endif
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_int vipmi_rfloorf(float aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128 val = _mm_set_ss(aValue);
	vipm_int itmp = _mm_cvtss_si32(val);
	return (itmp - _mm_movemask_ps(_mm_cmplt_ss(val, _mm_cvtsi32_ss(val, itmp))));

# elif defined(_GCC_VER)

	vipm_int itmp = (vipm_int)aValue;
	return (itmp - (aValue < itmp));

# else

	vipm_int itmp = vipmi_rintf(aValue);
	float diff = (float)(aValue - itmp);
	return (itmp - (diff < 0));

# endif
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_int vipmi_rceilf(float aValue)
{
# if ((defined(_MSC_VER) && defined(_A_X86_64)) || \
          (defined(_GCC_VER) && defined(__VIPM_SSE2) && !defined(__APPLE__))) && \
      !defined(__VIPM_NO_MMINTRIN_HEADER)

	__m128 val = _mm_set_ss(aValue);
	vipm_int itmp = _mm_cvtss_si32(val);
	return (itmp + _mm_movemask_ps(_mm_cmplt_ss(_mm_cvtsi32_ss(val, itmp), val)));

# elif defined(_GCC_VER)

	vipm_int itmp = (vipm_int)aValue;
	return (itmp + (itmp < aValue));

# else

	vipm_int itmp = vipmi_rintf(aValue);
	float diff = (float)(itmp - aValue);
	return (itmp + (diff < 0));

# endif
}

/* ------------------------------------------------------------------------- */

/* @todo: fixme */
# define vipmi_rtozerof(x)          vipmi_rtruncf(x)

_VIPM_INLINE vipm_int vipmi_rtruncf(float aValue) { return (vipm_int)aValue; }


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

# if defined(_A_ARM64) && (defined(_ICC_VER) || defined(_GCC_VER))
#  include <Vipm/Mmi_inl.arm64.h>
# elif defined(_A_ARM) && (defined(_ICC_VER) || defined(_GCC_VER))
#  include <Vipm/Mmi_inl.arm.h>
# elif defined(_A_X86_64)
#  include <Vipm/Mmi_inl.x86_64.h>
# elif defined(_A_X86)
#  include <Vipm/Mmi_inl.x86.h>
# endif

# include <Vipm/Mmi_inl.unk.h>


/* ========================================================================= */

/* Dual 16-bit Signed Multiply with Addition of products:
   r = a[31:16] * b[31:16] + a[15:0] * b[15:0] */
_VIPM_INLINE bo_int32_t
vipmi_dmua32_I16xI16(/* register */ bo_int32_t aa, /* register */ bo_int32_t bb)
{
# if defined(_A_ARM32) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
	register bo_int32_t rv;
	__asm__ __volatile__ ("smuad %[rv], %[aa], %[bb] \n\t" : [rv]"=r"(rv) : [aa]"r"(aa), [bb]"r"(bb));
	return (rv);
#  define vipmi_tm_dmua32_I16xI16 vipmi_dmua32_I16xI16
# else
	return ((aa & 0xFFFF) * (bb & 0xFFFF) + (aa >> 16) * (bb >> 16));
# endif
}
# define vipmi_dmua32_I16xI16(aa,bb) (vipmi_dmua32_I16xI16)((bo_int32_t)(aa), (bo_int32_t)(bb))

/* ------------------------------------------------------------------------- */

/* Dual 16-bit [Signed] Multiply [Bottom] with Addition of products:
   r = a[31:16] * b[15:8] + a[15:0] * b[7:0] */
_VIPM_INLINE bo_int32_t
vipmi_dmua32_I16xU8B(/* register */ bo_int32_t aa, /* register */ bo_int32_t bb)
{
# if defined(_A_ARM32) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
	register bo_int32_t rv;
	__asm__ __volatile__ (
		"pkhbt  %[bb], %[bb], %[bb],lsl #8  \n\t"
		"uxtb16 %[bb], %[bb]                \n\t"
		"smuad  %[rv], %[aa], %[bb]         \n\t"
		: [rv] "=r"(rv), [bb] "+r"(bb)
		: [aa] "r"(aa)
	);
	return (rv);
#  define vipmi_tm_dmua32_I16xU8B vipmi_dmua32_I16xU8B
# else
	return vipmi_dmua32_I16xI16(aa, vipmi_uwiden88B(bb));
# endif
}

/* ------------------------------------------------------------------------- */

/* Dual 16-bit [Signed] Multiply [Top] with Addition of products:
   r = a[31:16] * b[31:24] + a[15:0] * b[23:16] */
_VIPM_INLINE bo_int32_t
vipmi_dmua32_I16xU8T(/* register */ bo_int32_t aa, /* register */ bo_int32_t bb)
{
# if defined(_A_ARM32) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
	register bo_int32_t rv;
	__asm__ __volatile__ (
		"lsr    %[bb], %[bb], #16           \n\t"
		"pkhbt  %[bb], %[bb], %[bb],lsl #8  \n\t"
		"uxtb16 %[bb], %[bb]                \n\t"
		"smuad  %[rv], %[aa], %[bb]         \n\t"
		: [rv] "=r"(rv), [bb] "+r"(bb)
		: [aa] "r"(aa)
	);
	return (rv);
#  define vipmi_tm_dmua32_I16xU8T vipmi_dmua32_I16xU8T
# else
	return vipmi_dmua32_I16xI16(aa, vipmi_uwiden88TX(bb));
# endif
}

# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
#  define vipmi_dmua32_I16xU8(aa,bb) vipmi_dmua32_I16xU8T((bo_int32_t)(aa), (bo_int32_t)(bb))
#  ifdef vipmi_tm_dmua32_I16xU8T
#   define vipmi_tm_dmua32_I16xU8 vipmi_tm_dmua32_I16xU8T
#  endif
# else
#  define vipmi_dmua32_I16xU8(aa,bb) vipmi_dmua32_I16xU8B((bo_int32_t)(aa), (bo_int32_t)(bb))
#  ifdef vipmi_tm_dmua32_I16xU8B
#   define vipmi_tm_dmua32_I16xU8 vipmi_tm_dmua32_I16xU8B
#  endif
# endif

/* ------------------------------------------------------------------------- */

/* Triple 16-bit [Signed] Multiply [Bottom] with Addition of products:
   r = a01[31:16] * b02[15:8] + a01[15:0] * b02[7:0] + a2{[15:0]} * b02[23:16] */
_VIPM_INLINE bo_int32_t
vipmi_tmua32_I16xU8B(/* register */ bo_int32_t a01, /* register */ bo_int32_t a2, /* register */ bo_int32_t b02)
{
# if defined(_A_ARM32) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
	register bo_int32_t rv;
	__asm__ __volatile__ (
		"uxtb   %[rv],  %[b02],ror #16         \n\t"
		"pkhbt  %[b02], %[b02], %[b02],lsl #8  \n\t"
		"smulBB %[rv],  %[a2], %[rv]           \n\t"
		"uxtb16 %[b02], %[b02]                 \n\t"
		"smlad  %[rv], %[a01], %[b02], %[rv]   \n\t"
		: [rv]  "=r"(rv), [b02] "+r"(b02)
		: [a01] "r"(a01), [a2] "r"(a2)
	);
	return (rv);
#  define vipmi_tm_tmua32_I16xU8B vipmi_tmua32_I16xU8B
# else
	return vipmi_smla_32(vipmi_dmua32_I16xU8B(a01, b02), (a2 & 0xFFFF), VIPM_BYAT32_2(b02));
# endif
}

/* ------------------------------------------------------------------------- */

/* Triple 16-bit [Signed] Multiply [Top] with Addition of products:
   r = a01[31:16] * b02[31:24] + a01[15:0] * b02[23:16] + a2{[15:0]} * b02[15:8] */
_VIPM_INLINE bo_int32_t
vipmi_tmua32_I16xU8T(/* register */ bo_int32_t a01, /* register */ bo_int32_t a2, /* register */ bo_int32_t b02)
{
# if defined(_A_ARM32) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
	register bo_int32_t rv;
	__asm__ __volatile__ (
		"uxtb   %[rv],  %[b02],ror #8          \n\t"
		"lsr    %[b02], %[b02], #16            \n\t"
		"pkhbt  %[b02], %[b02], %[b02],lsl #8  \n\t"
		"smulBB %[rv],  %[a2], %[rv]           \n\t"
		"uxtb16 %[b02], %[b02]                 \n\t"
		"smlad  %[rv], %[a01], %[b02], %[rv]   \n\t"
		: [rv]  "=r"(rv), [b02] "+r"(b02)
		: [a01] "r"(a01), [a2] "r"(a2)
	);
	return (rv);
#  define vipmi_tm_tmua32_I16xU8T vipmi_tmua32_I16xU8T
# else
	return vipmi_smla_32(vipmi_dmua32_I16xU8T(a01, b02), (a2 & 0xFFFF), VIPM_BYAT32_1(b02));
# endif
}

# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
#  define vipmi_tmua32_I16xU8(a01,a2,b02) \
		vipmi_tmua32_I16xU8T((bo_int32_t)(a01), (bo_int32_t)(a2), (bo_int32_t)(b02))
#  ifdef vipmi_tm_tmua32_I16xU8T
#   define vipmi_tm_tmua32_I16xU8 vipmi_tm_tmua32_I16xU8T
#  endif
# else
#  define vipmi_tmua32_I16xU8(a01,a2,b02) \
		vipmi_tmua32_I16xU8B((bo_int32_t)(a01), (bo_int32_t)(a2), (bo_int32_t)(b02))
#  ifdef vipmi_tm_tmua32_I16xU8B
#   define vipmi_tm_tmua32_I16xU8 vipmi_tm_tmua32_I16xU8B
#  endif
# endif

/* ------------------------------------------------------------------------- */

/* @note: vipmi_tmuaw32_I16BxU8B ~ vipmi_tmua32_I16xU8B
 *
 * Triple 16-bit [Signed] Multiply [Bottom] with Addition Wide of products:
 * r = a01[31:16] * b02[15:8] + a01[15:0] * b02[7:0] + a2{[15:0]} * b02[23:16]
 */
_VIPM_INLINE bo_int32_t
vipmi_tmuaw32_I16BxU8B(/* register */ bo_int32_t a01, /* register */ bo_int32_t a2, /* register */ bo_int32_t b02)
{
# if defined(_A_ARM32) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
	register bo_int32_t rv;
	__asm__ __volatile__ (
		"uxtb   %[rv],  %[b02],ror #16         \n\t"
		"pkhbt  %[b02], %[b02], %[b02],lsl #8  \n\t"
		"smulBB %[rv],  %[a2], %[rv]           \n\t"
		"uxtb16 %[b02], %[b02]                 \n\t"
		"smlad  %[rv], %[a01], %[b02], %[rv]   \n\t"
		: [rv]  "=r"(rv), [b02] "+r"(b02)
		: [a01] "r"(a01), [a2] "r"(a2)
	);
	return (rv);
#  define vipmi_tm_tmuaw32_I16BxU8B vipmi_tmuaw32_I16BxU8B
# else
	return vipmi_smla_32(vipmi_dmua32_I16xU8B(a01, b02), (a2 & 0xFFFF), VIPM_BYAT32_2(b02));
# endif
}

/* ------------------------------------------------------------------------- */

/* Triple 16-bit [Signed] Multiply [Top] with Addition Wide of products:
   r = a01[31:16] * b02[31:24] + a01[15:0] * b02[23:16] + a2[31:16] * b02[15:8] */
_VIPM_INLINE bo_int32_t
vipmi_tmuaw32_I16TxU8T(/* register */ bo_int32_t a01, /* register */ bo_int32_t a2, /* register */ bo_int32_t b02)
{
# if defined(_A_ARM32) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
	register bo_int32_t rv;
	__asm__ __volatile__ (
		"uxtb   %[rv],  %[b02],ror #8          \n\t"
		"lsr    %[b02], %[b02], #16            \n\t"
		"pkhbt  %[b02], %[b02], %[b02],lsl #8  \n\t"
		"smulTB %[rv],  %[a2], %[rv]           \n\t"
		"uxtb16 %[b02], %[b02]                 \n\t"
		"smlad  %[rv], %[a01], %[b02], %[rv]   \n\t"
		: [rv]  "=r"(rv), [b02] "+r"(b02)
		: [a01] "r"(a01), [a2] "r"(a2)
	);
	return (rv);
#  define vipmi_tm_tmuaw32_I16TxU8T vipmi_tmuaw32_I16TxU8T
# else
	return vipmi_smla_32(vipmi_dmua32_I16xU8T(a01, b02), (a2 >> 16), VIPM_BYAT32_1(b02));
# endif
}

# if (BO_BYTE_ORDER == BIG_BYTE_ORDER)
#  define vipmi_tmuaw32_I16xU8(a01,a2,b02) \
		vipmi_tmuaw32_I16TxU8T((bo_int32_t)(a01), (bo_int32_t)(a2), (bo_int32_t)(b02))
#  ifdef vipmi_tm_tmuaw32_I16TxU8T
#   define vipmi_tm_tmuaw32_I16xU8 vipmi_tm_tmuaw32_I16TxU8T
#  endif
# else
#  define vipmi_tmuaw32_I16xU8(a01,a2,b02) \
		vipmi_tmuaw32_I16BxU8B((bo_int32_t)(a01), (bo_int32_t)(a2), (bo_int32_t)(b02))
#  ifdef vipmi_tm_tmuaw32_I16BxU8B
#   define vipmi_tm_tmuaw32_I16xU8 vipmi_tm_tmuaw32_I16BxU8B
#  endif
# endif

/* ------------------------------------------------------------------------- */

/* r = (x1 * x2 + y1 * y2) */
_VIPM_INLINE bo_int64_t
vipmi_dmuaw32(bo_int64_t x1, bo_int64_t y1, bo_int32_t x2, bo_int32_t y2)
{
	return (x1 * x2 + y1 * y2);
}
# define vipmi_dmuaw32(x1,y1,x2,y2) (vipmi_dmuaw32)((bo_int64_t)(x1), (bo_int64_t)(y1), (bo_int32_t)(x2), (bo_int32_t)(y2))

/* ------------------------------------------------------------------------- */

/* r = (x1 * x2 + y1 * y2) */
_VIPM_INLINE bo_int64_t
vipmi_dmual32(bo_int32_t x1, bo_int32_t y1, bo_int32_t x2, bo_int32_t y2)
{
# if defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	bo_int64_t rv;
	__asm__ __volatile__ (
		"smull  %[rv], %w[x1], %w[x2]        \n\t"
		"smaddl %[rv], %w[y1], %w[y2], %[rv] \n\t"
		: [rv] "=&r"(rv)
		: [x1] "r"(x1), [y1] "r"(y1), [x2] "r"(x2), [y2] "r"(y2)
	);
	return (rv);
#  else /* _A_ARM32 */
	bo_int64_t rv;
	__asm__ __volatile__ (
		"smull %Q[rv], %R[rv], %[x1], %[x2]  \n\t"
		"smlal %Q[rv], %R[rv], %[y1], %[y2]  \n\t"
		: [rv] "=&r"(rv)
		: [x1] "r"(x1), [y1] "r"(y1), [x2] "r"(x2), [y2] "r"(y2)
	);
	return (rv);
#  endif /* _A_ARM64 */
#  define vipmi_tm_dmual32 vipmi_dmual32
# else
	return ((bo_int64_t)x1 * x2 + (bo_int64_t)y1 * y2);
# endif
}
# define vipmi_dmual32(x1,y1,x2,y2) (vipmi_dmual32)((bo_int32_t)(x1), (bo_int32_t)(y1), (bo_int32_t)(x2), (bo_int32_t)(y2))

/* ------------------------------------------------------------------------- */

# define vipmi_dmua32(x1,y1,x2,y2) ((bo_int32_t)vipmi_dmual32(x1,y1,x2,y2))
# ifdef vipmi_tm_dmual32
#  define vipmi_tm_dmua32 vipmi_dmua32
# endif

/* ------------------------------------------------------------------------- */

/* r = (x1 * y2 - x2 * y1) */
_VIPM_INLINE bo_int64_t
vipmi_dcmsw32(bo_int64_t x1, bo_int64_t y1, bo_int32_t x2, bo_int32_t y2)
{
	return (x1 * y2 - x2 * y1);
}
# define vipmi_dcmsw32(x1,y1,x2,y2) (vipmi_dcmsw32)((bo_int64_t)(x1), (bo_int64_t)(y1), (bo_int32_t)(x2), (bo_int32_t)(y2))

/* r = (y1 * x2 - x1 * y2) */
_VIPM_INLINE bo_int64_t
vipmi_dcmrsw32(bo_int64_t x1, bo_int64_t y1, bo_int32_t x2, bo_int32_t y2)
{
	return (y1 * x2 - x1 * y2);
}
# define vipmi_dcmrsw32(x1,y1,x2,y2) (vipmi_dcmrsw32)((bo_int64_t)(x1), (bo_int64_t)(y1), (bo_int32_t)(x2), (bo_int32_t)(y2))

/* ------------------------------------------------------------------------- */

/* r = (x1 * y2 - x2 * y1) */
_VIPM_INLINE bo_int64_t
vipmi_dcmsl32(bo_int32_t x1, bo_int32_t y1, bo_int32_t x2, bo_int32_t y2)
{
# if defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	bo_int64_t rv;
	__asm__ __volatile__ (
		"smnegl %[rv], %w[x2], %w[y1]        \n\t"
		"smaddl %[rv], %w[x1], %w[y2], %[rv] \n\t"
		: [rv] "=&r"(rv)
		: [x1] "r"(x1), [y1] "r"(y1), [x2] "r"(x2), [y2] "r"(y2)
	);
	return (rv);
#  else /* _A_ARM32 */
	bo_int64_t rv;
	__asm__ __volatile__ (
		"rsb   %[x2], %[x2], #0              \n\t"
		"smull %Q[rv], %R[rv], %[x1], %[y2]  \n\t"
		"smlal %Q[rv], %R[rv], %[x2], %[y1]  \n\t"
		: [rv] "=&r"(rv), [x2] "+r"(x2)
		: [x1] "r"(x1), [y1] "r"(y1), [y2] "r"(y2)
	);
	return (rv);
#  endif /* _A_ARM64 */
#  define vipmi_tm_dcmsl32 vipmi_dcmsl32
# else
	return ((bo_int64_t)x1 * y2 - (bo_int64_t)x2 * y1);
# endif
}
# define vipmi_dcmsl32(x1,y1,x2,y2) (vipmi_dcmsl32)((bo_int32_t)(x1), (bo_int32_t)(y1), (bo_int32_t)(x2), (bo_int32_t)(y2))

/* ------------------------------------------------------------------------- */

# define vipmi_dcms32(x1,y1,x2,y2) ((bo_int32_t)vipmi_dcmsl32(x1,y1,x2,y2))
# ifdef vipmi_tm_dcmsl32
#  define vipmi_tm_dcms32 vipmi_dcms32
# endif

/* ========================================================================= */

/* r = (x1 * x2 + y1 * y2) */
_VIPM_INLINE bo_int64_t
vipmi_dmua64(bo_int64_t x1, bo_int64_t y1, bo_int64_t x2, bo_int64_t y2)
{
	return (x1 * x2 + y1 * y2);
}
# define vipmi_dmua64(x1,y1,x2,y2)  (vipmi_dmua64)((bo_int64_t)(x1), (bo_int64_t)(y1), (bo_int64_t)(x2), (bo_int64_t)(y2))

/* ------------------------------------------------------------------------- */

/* r = (x1 * y2 - x2 * y1) */
_VIPM_INLINE bo_int64_t
vipmi_dcms64(bo_int64_t x1, bo_int64_t y1, bo_int64_t x2, bo_int64_t y2)
{
	return (x1 * y2 - x2 * y1);
}
# define vipmi_dcms64(x1,y1,x2,y2)  (vipmi_dcms64)((bo_int64_t)(x1), (bo_int64_t)(y1), (bo_int64_t)(x2), (bo_int64_t)(y2))

/* ========================================================================= */


/* r = (x1 * x2 + y1 * y2) */
_VIPM_INLINE bo_float64_t
vipmi_dmuaw_f32(bo_float64_t x1, bo_float64_t y1, bo_float32_t x2, bo_float32_t y2)
{
	return (x1 * x2 + y1 * y2);
}
# define vipmi_dmuaw_f32(x1,y1,x2,y2)   (vipmi_dmuaw_f32)((bo_float64_t)(x1), (bo_float64_t)(y1), (bo_float32_t)(x2), (bo_float32_t)(y2))

/* ------------------------------------------------------------------------- */

/* r = (x1 * x2 + y1 * y2) */
_VIPM_INLINE bo_float64_t
vipmi_dmual_f32(bo_float32_t x1, bo_float32_t y1, bo_float32_t x2, bo_float32_t y2)
{
	return ((double)x1 * x2 + (double)y1 * y2);
}
# define vipmi_dmual_f32(x1,y1,x2,y2)   (vipmi_dmual_f32)((bo_float32_t)(x1), (bo_float32_t)(y1), (bo_float32_t)(x2), (bo_float32_t)(y2))

/* ------------------------------------------------------------------------- */

# define vipmi_dmua_f32(x1,y1,x2,y2)    ((bo_float32_t)vipmi_dmual_f32(x1,y1,x2,y2))
# ifdef vipmi_tm_dmual_f32
#  define vipmi_tm_dmua_f32 vipmi_dmua_f32
# endif

/* ------------------------------------------------------------------------- */

/* r = (x1 * y2 - x2 * y1) */
_VIPM_INLINE bo_float64_t
vipmi_dcmsw_f32(bo_float64_t x1, bo_float64_t y1, bo_float32_t x2, bo_float32_t y2)
{
	return (x1 * y2 - x2 * y1);
}
# define vipmi_dcmsw_f32(x1,y1,x2,y2)   (vipmi_dcmsw_f32)((bo_float64_t)(x1), (bo_float64_t)(y1), (bo_float32_t)(x2), (bo_float32_t)(y2))

/* r = (y1 * x2 - x1 * y2) */
_VIPM_INLINE bo_float64_t
vipmi_dcmrsw_f32(bo_float64_t x1, bo_float64_t y1, bo_float32_t x2, bo_float32_t y2)
{
	return (y1 * x2 - x1 * y2);
}
# define vipmi_dcmrsw_f32(x1,y1,x2,y2)  (vipmi_dcmrsw_f32)((bo_float64_t)(x1), (bo_float64_t)(y1), (bo_float32_t)(x2), (bo_float32_t)(y2))

/* ------------------------------------------------------------------------- */

/* r = (x1 * y2 - x2 * y1) */
_VIPM_INLINE bo_float64_t
vipmi_dcmsl_f32(bo_float32_t x1, bo_float32_t y1, bo_float32_t x2, bo_float32_t y2)
{
	return ((double)x1 * y2 - (double)x2 * y1);
}
# define vipmi_dcmsl_f32(x1,y1,x2,y2)   (vipmi_dcmsl_f32)((bo_float32_t)(x1), (bo_float32_t)(y1), (bo_float32_t)(x2), (bo_float32_t)(y2))

/* ------------------------------------------------------------------------- */

# define vipmi_dcms_f32(x1,y1,x2,y2)    ((bo_float32_t)vipmi_dcmsl_f32(x1,y1,x2,y2))
# ifdef vipmi_tm_dcmsl_f32
#  define vipmi_tm_dcms_f32 vipmi_dcms_f32
# endif

/* ========================================================================= */


/* r = (x1 * x2 + y1 * y2) */
_VIPM_INLINE bo_float64_t
vipmi_dmua_f64(bo_float64_t x1, bo_float64_t y1, bo_float64_t x2, bo_float64_t y2)
{
	return (x1 * x2 + y1 * y2);
}

/* ------------------------------------------------------------------------- */

/* r = (x1 * y2 - x2 * y1) */
_VIPM_INLINE bo_float64_t
vipmi_dcms_f64(bo_float64_t x1, bo_float64_t y1, bo_float64_t x2, bo_float64_t y2)
{
	return (x1 * y2 - x2 * y1);
}


/* ========================================================================= */

_VIPM_INLINE bo_int64_t
vipmi_2d_crossprod64(bo_int64_t y, bo_int64_t x, bo_int64_t y1, bo_int64_t x1, bo_int64_t y2, bo_int64_t x2)
{
//	y1 -= y;
//	y2 -= y;
//	x1 -= x;
//	x2 -= x;
//	return vipmi_2d_vecprod64(y1, x1, y2, x2);
	return vipmi_2d_vecprod64(y1 - y, x1 - x, y2 - y, x2 - x);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_int32_t
vipmi_2d_crossprod(bo_int32_t y, bo_int32_t x, bo_int32_t y1, bo_int32_t x1, bo_int32_t y2, bo_int32_t x2)
{
//	y1 -= y;
//	y2 -= y;
//	x1 -= x;
//	x2 -= x;
//	return vipmi_2d_vecprod(y1, x1, y2, x2);
	return vipmi_2d_vecprod(y1 - y, x1 - x, y2 - y, x2 - x);
}

_VIPM_INLINE bo_int64_t
vipmi_2d_crossprodl(bo_int32_t y, bo_int32_t x, bo_int32_t y1, bo_int32_t x1, bo_int32_t y2, bo_int32_t x2)
{
	return vipmi_2d_crossprod64(y, x, y1, x1, y2, x2);
}

_VIPM_INLINE bo_int64_t
vipmi_2d_crossprodw(bo_int64_t y, bo_int64_t x, bo_int64_t y1, bo_int64_t x1, bo_int32_t y2, bo_int32_t x2)
{
	return vipmi_2d_crossprod64(y, x, y1, x1, y2, x2);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE double
vipmi_r2d_crossprod(double y, double x, double y1, double x1, double y2, double x2)
{
//	y1 -= y;
//	y2 -= y;
//	x1 -= x;
//	x2 -= x;
//	return vipmi_r2d_vecprod(y1, x1, y2, x2);
	return vipmi_r2d_vecprod(y1 - y, x1 - x, y2 - y, x2 - x);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE float
vipmi_f2d_crossprod(float y, float x, float y1, float x1, float y2, float x2)
{
//	y1 -= y;
//	y2 -= y;
//	x1 -= x;
//	x2 -= x;
//	return vipmi_f2d_vecprod(y1, x1, y2, x2);
	return vipmi_f2d_vecprod(y1 - y, x1 - x, y2 - y, x2 - x);
}

_VIPM_INLINE double
vipmi_f2d_crossprodl(float y, float x, float y1, float x1, float y2, float x2)
{
	return vipmi_f2d_crossprod64(y, x, y1, x1, y2, x2);
}

_VIPM_INLINE double
vipmi_f2d_crossprodw(double y, double x, double y1, double x1, float y2, float x2)
{
	return vipmi_f2d_crossprod64(y, x, y1, x1, y2, x2);
}

/* ========================================================================= */

_VIPM_INLINE bo_int32_t
vipmi_XNS(2d)(crossprod,mm)(bo_int32_t const v[/*6*/])
{
	return vipmi_XNS(2d)(crossprod)(v[0], v[1], v[2], v[3], v[4], v[5]);
}

_VIPM_INLINE bo_int64_t
vipmi_XNS(2d)(crossprodl,mm)(bo_int32_t const v[/*6*/])
{
	return vipmi_XNS(2d)(crossprodl)(v[0], v[1], v[2], v[3], v[4], v[5]);
}

_VIPM_INLINE bo_int64_t
vipmi_XNS(2d)(crossprod64,mm)(bo_int64_t const v[/*6*/])
{
	return vipmi_XNS(2d)(crossprod64)(v[0], v[1], v[2], v[3], v[4], v[5]);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE float
vipmi_XNS(f2d)(crossprod,mm)(float const v[/*6*/])
{
	return vipmi_XNS(f2d)(crossprod)(v[0], v[1], v[2], v[3], v[4], v[5]);
}

_VIPM_INLINE double
vipmi_XNS(f2d)(crossprodl,mm)(float const v[/*6*/])
{
	return vipmi_XNS(f2d)(crossprodl)(v[0], v[1], v[2], v[3], v[4], v[5]);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE double
vipmi_XNS(r2d)(crossprod,mm)(double const v[/*6*/])
{
	return vipmi_XNS(r2d)(crossprod)(v[0], v[1], v[2], v[3], v[4], v[5]);
}


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
