/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2017 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(D.Vinnyk)
 * Vipm/Morphology/Morphop/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_MORPHOLOGY_MORPHOP_ALGORITHM_H__
# define __VIPM_MORPHOLOGY_MORPHOP_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_MORPHOLOGY_MORPHOP_TYPES_H__
#  include "Vipm/Morphology/Morphop/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

/*
 * morphop: { erode,dilate;close,open,bothat,tophat }
 */
__PUB bo_status_t
VipmTGAmorphop(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_morphparm_t const *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_morphop(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_int                        aMorphOpMethod,
	vipm_strel_t const             *aStrel
#endif
	);

__PUB bo_status_t
vipma_emorphop(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_int                        aMorphOpMethod,
	vipm_strel_t const             *aStrel,
	vipm_morphopts_t const         *aMorphOpts,
	vipm_arrshape_t const          *aWholeArrShapeSrc
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * morphop
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_morphop_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se) \
    vipma_morphop(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,morph,se)

# define vipma_morphop_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se) \
    vipma_morphop(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,morph,se)

# define vipma_morphop_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se) \
    vipma_morphop(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,morph,se)

# define vipma_morphop_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se) \
    vipma_morphop(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,morph,se)

/* ------------------------------------------------------------------------- */

/*
 * emorphop
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_emorphop_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS) \
    vipma_emorphop(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS)

# define vipma_emorphop_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS) \
    vipma_emorphop(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS)

# define vipma_emorphop_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS) \
    vipma_emorphop(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS)

# define vipma_emorphop_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS) \
    vipma_emorphop(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,morph,se,opts,shapeS)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_MORPHOLOGY_MORPHOP_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
