/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2018 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Simd.arm_neon.h	(V.Sokur)
 * Vipm/Simd.arm_neon.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_SIMD_ARM_NEON_H__
# define __VIPM_SIMD_ARM_NEON_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifdef __VIPM_NEON_MMINTRIN_HEADER
#  include __VIPM_NEON_MMINTRIN_HEADER
# endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

/*
 * 1 element
 */

typedef int8x8_t                    bo_m64_s8_t;
typedef int16x4_t                   bo_m64_s16_t;
typedef int32x2_t                   bo_m64_s32_t;
typedef int64x1_t                   bo_m64_s64_t;
typedef uint8x8_t                   bo_m64_u8_t;
typedef uint16x4_t                  bo_m64_u16_t;
typedef uint32x2_t                  bo_m64_u32_t;
typedef uint64x1_t                  bo_m64_u64_t;
typedef float32x2_t                 bo_m64_f32_t;

typedef int8x16_t                   bo_m128_s8_t;
typedef int16x8_t                   bo_m128_s16_t;
typedef int32x4_t                   bo_m128_s32_t;
typedef int64x2_t                   bo_m128_s64_t;
typedef uint8x16_t                  bo_m128_u8_t;
typedef uint16x8_t                  bo_m128_u16_t;
typedef uint32x4_t                  bo_m128_u32_t;
typedef uint64x2_t                  bo_m128_u64_t;
typedef float32x4_t                 bo_m128_f32_t;

# if defined(__clang__) || \
        (defined(__ARM_FP16_FORMAT_IEEE) || defined(__ARM_FP16_FORMAT_ALTERNATIVE))
typedef float16x4_t                 bo_m64_f16_t;
typedef float16x8_t                 bo_m128_f16_t;
# endif
# ifdef _A_ARM64
typedef float64x1_t                 bo_m64_f64_t;
typedef float64x2_t                 bo_m128_f64_t;
# endif

/*
 * 2 elements
 */

typedef int8x8x2_t                  bo_m64x2_s8_t;
typedef int16x4x2_t                 bo_m64x2_s16_t;
typedef int32x2x2_t                 bo_m64x2_s32_t;
typedef int64x1x2_t                 bo_m64x2_s64_t;
typedef uint8x8x2_t                 bo_m64x2_u8_t;
typedef uint16x4x2_t                bo_m64x2_u16_t;
typedef uint32x2x2_t                bo_m64x2_u32_t;
typedef uint64x1x2_t                bo_m64x2_u64_t;
typedef float32x2x2_t               bo_m64x2_f32_t;

typedef int8x16x2_t                 bo_m128x2_s8_t;
typedef int16x8x2_t                 bo_m128x2_s16_t;
typedef int32x4x2_t                 bo_m128x2_s32_t;
typedef int64x2x2_t                 bo_m128x2_s64_t;
typedef uint8x16x2_t                bo_m128x2_u8_t;
typedef uint16x8x2_t                bo_m128x2_u16_t;
typedef uint32x4x2_t                bo_m128x2_u32_t;
typedef uint64x2x2_t                bo_m128x2_u64_t;
typedef float32x4x2_t               bo_m128x2_f32_t;

# if defined(__clang__) || \
        (defined(__ARM_FP16_FORMAT_IEEE) || defined(__ARM_FP16_FORMAT_ALTERNATIVE))
typedef float16x4x2_t               bo_m64x2_f16_t;
typedef float16x8x2_t               bo_m128x2_f16_t;
# endif
# ifdef _A_ARM64
typedef float64x1x2_t               bo_m64x2_f64_t;
typedef float64x2x2_t               bo_m128x2_f64_t;
# endif

/*
 * 3 elements
 */

typedef int8x8x3_t                  bo_m64x3_s8_t;
typedef int16x4x3_t                 bo_m64x3_s16_t;
typedef int32x2x3_t                 bo_m64x3_s32_t;
typedef int64x1x3_t                 bo_m64x3_s64_t;
typedef uint8x8x3_t                 bo_m64x3_u8_t;
typedef uint16x4x3_t                bo_m64x3_u16_t;
typedef uint32x2x3_t                bo_m64x3_u32_t;
typedef uint64x1x3_t                bo_m64x3_u64_t;
typedef float32x2x3_t               bo_m64x3_f32_t;

typedef int8x16x3_t                 bo_m128x3_s8_t;
typedef int16x8x3_t                 bo_m128x3_s16_t;
typedef int32x4x3_t                 bo_m128x3_s32_t;
typedef int64x2x3_t                 bo_m128x3_s64_t;
typedef uint8x16x3_t                bo_m128x3_u8_t;
typedef uint16x8x3_t                bo_m128x3_u16_t;
typedef uint32x4x3_t                bo_m128x3_u32_t;
typedef uint64x2x3_t                bo_m128x3_u64_t;
typedef float32x4x3_t               bo_m128x3_f32_t;

# if defined(__clang__) || \
        (defined(__ARM_FP16_FORMAT_IEEE) || defined(__ARM_FP16_FORMAT_ALTERNATIVE))
typedef float16x4x3_t               bo_m64x3_f16_t;
typedef float16x8x3_t               bo_m128x3_f16_t;
# endif
# ifdef _A_ARM64
typedef float64x1x3_t               bo_m64x3_f64_t;
typedef float64x2x3_t               bo_m128x3_f64_t;
# endif

/*
 * 4 elements
 */

typedef int8x8x4_t                  bo_m64x4_s8_t;
typedef int16x4x4_t                 bo_m64x4_s16_t;
typedef int32x2x4_t                 bo_m64x4_s32_t;
typedef int64x1x4_t                 bo_m64x4_s64_t;
typedef uint8x8x4_t                 bo_m64x4_u8_t;
typedef uint16x4x4_t                bo_m64x4_u16_t;
typedef uint32x2x4_t                bo_m64x4_u32_t;
typedef uint64x1x4_t                bo_m64x4_u64_t;
typedef float32x2x4_t               bo_m64x4_f32_t;

typedef int8x16x4_t                 bo_m128x4_s8_t;
typedef int16x8x4_t                 bo_m128x4_s16_t;
typedef int32x4x4_t                 bo_m128x4_s32_t;
typedef int64x2x4_t                 bo_m128x4_s64_t;
typedef uint8x16x4_t                bo_m128x4_u8_t;
typedef uint16x8x4_t                bo_m128x4_u16_t;
typedef uint32x4x4_t                bo_m128x4_u32_t;
typedef uint64x2x4_t                bo_m128x4_u64_t;
typedef float32x4x4_t               bo_m128x4_f32_t;

# if defined(__clang__) || \
        (defined(__ARM_FP16_FORMAT_IEEE) || defined(__ARM_FP16_FORMAT_ALTERNATIVE))
typedef float16x4x4_t               bo_m64x4_f16_t;
typedef float16x8x4_t               bo_m128x4_f16_t;
# endif
# ifdef _A_ARM64
typedef float64x1x4_t               bo_m64x4_f64_t;
typedef float64x2x4_t               bo_m128x4_f64_t;
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
 * Load/Store
 * @{
 *
 * cmda : aligned access
 * cmdu : unaligned access
 * cmd  : aligned ? unaligned access
 */

	/*
	 * Load/Store 1-element structures
	 * @{
	 */

		/*
		 * Transfer 8x1=8 bytes
		 * 1 D register
		 */

# define /* bo_m64_s8_t  */    vipmvi_load1x8_s8(p)             vld1_s8( (void const *)(p))
# define /* bo_m64_s16_t */    vipmvi_load1x8_s16(p)            vld1_s16((void const *)(p))
# define /* bo_m64_s32_t */    vipmvi_load1x8_s32(p)            vld1_s32((void const *)(p))
# define /* bo_m64_u8_t  */    vipmvi_load1x8_u8(p)             vld1_u8( (void const *)(p))
# define /* bo_m64_u16_t */    vipmvi_load1x8_u16(p)            vld1_u16((void const *)(p))
# define /* bo_m64_u32_t */    vipmvi_load1x8_u32(p)            vld1_u32((void const *)(p))
# define /* bo_m64_f32_t */    vipmvi_load1x8_f32(p)            vld1_f32((void const *)(p))

# define /* bo_m64_s8_t  */    vipmvi_loada1x8_s8(p)            vipmvi_load1x8_s8(p)
# define /* bo_m64_s16_t */    vipmvi_loada1x8_s16(p)           vipmvi_load1x8_s16(p)
# define /* bo_m64_s32_t */    vipmvi_loada1x8_s32(p)           vipmvi_load1x8_s32(p)
# define /* bo_m64_u8_t  */    vipmvi_loada1x8_u8(p)            vipmvi_load1x8_u8(p)
# define /* bo_m64_u16_t */    vipmvi_loada1x8_u16(p)           vipmvi_load1x8_u16(p)
# define /* bo_m64_u32_t */    vipmvi_loada1x8_u32(p)           vipmvi_load1x8_u32(p)
# define /* bo_m64_f32_t */    vipmvi_loada1x8_f32(p)           vipmvi_load1x8_f32(p)

# define /* bo_m64_s8_t  */    vipmvi_loadu1x8_s8(p)            vipmvi_load1x8_s8(p)
# define /* bo_m64_s16_t */    vipmvi_loadu1x8_s16(p)           vipmvi_load1x8_s16(p)
# define /* bo_m64_s32_t */    vipmvi_loadu1x8_s32(p)           vipmvi_load1x8_s32(p)
# define /* bo_m64_u8_t  */    vipmvi_loadu1x8_u8(p)            vipmvi_load1x8_u8(p)
# define /* bo_m64_u16_t */    vipmvi_loadu1x8_u16(p)           vipmvi_load1x8_u16(p)
# define /* bo_m64_u32_t */    vipmvi_loadu1x8_u32(p)           vipmvi_load1x8_u32(p)
# define /* bo_m64_f32_t */    vipmvi_loadu1x8_f32(p)           vipmvi_load1x8_f32(p)

# define /* void */            vipmvi_store1x8_s8(p,x)          vst1_s8( (void *)(p),x)
# define /* void */            vipmvi_store1x8_s16(p,x)         vst1_s16((void *)(p),x)
# define /* void */            vipmvi_store1x8_s32(p,x)         vst1_s32((void *)(p),x)
# define /* void */            vipmvi_store1x8_u8(p,x)          vst1_u8( (void *)(p),x)
# define /* void */            vipmvi_store1x8_u16(p,x)         vst1_u16((void *)(p),x)
# define /* void */            vipmvi_store1x8_u32(p,x)         vst1_u32((void *)(p),x)
# define /* void */            vipmvi_store1x8_f32(p,x)         vst1_f32((void *)(p),x)

# define /* void */            vipmvi_storea1x8_s8(p,x)         vipmvi_store1x8_s8( p,x)
# define /* void */            vipmvi_storea1x8_s16(p,x)        vipmvi_store1x8_s16(p,x)
# define /* void */            vipmvi_storea1x8_s32(p,x)        vipmvi_store1x8_s32(p,x)
# define /* void */            vipmvi_storea1x8_u8(p,x)         vipmvi_store1x8_u8( p,x)
# define /* void */            vipmvi_storea1x8_u16(p,x)        vipmvi_store1x8_u16(p,x)
# define /* void */            vipmvi_storea1x8_u32(p,x)        vipmvi_store1x8_u32(p,x)
# define /* void */            vipmvi_storea1x8_f32(p,x)        vipmvi_store1x8_f32(p,x)

# define /* void */            vipmvi_storeu1x8_s8(p,x)         vipmvi_store1x8_s8( p,x)
# define /* void */            vipmvi_storeu1x8_s16(p,x)        vipmvi_store1x8_s16(p,x)
# define /* void */            vipmvi_storeu1x8_s32(p,x)        vipmvi_store1x8_s32(p,x)
# define /* void */            vipmvi_storeu1x8_u8(p,x)         vipmvi_store1x8_u8( p,x)
# define /* void */            vipmvi_storeu1x8_u16(p,x)        vipmvi_store1x8_u16(p,x)
# define /* void */            vipmvi_storeu1x8_u32(p,x)        vipmvi_store1x8_u32(p,x)
# define /* void */            vipmvi_storeu1x8_f32(p,x)        vipmvi_store1x8_f32(p,x)


		/*
		 * Transfer 16x1=16 bytes
		 * 1 Q register
		 */

# define /* bo_m128_s8_t  */   vipmvi_load1x16_s8(p)            vld1q_s8( (void const *)(p))
# define /* bo_m128_s16_t */   vipmvi_load1x16_s16(p)           vld1q_s16((void const *)(p))
# define /* bo_m128_s32_t */   vipmvi_load1x16_s32(p)           vld1q_s32((void const *)(p))
# define /* bo_m128_u8_t  */   vipmvi_load1x16_u8(p)            vld1q_u8( (void const *)(p))
# define /* bo_m128_u16_t */   vipmvi_load1x16_u16(p)           vld1q_u16((void const *)(p))
# define /* bo_m128_u32_t */   vipmvi_load1x16_u32(p)           vld1q_u32((void const *)(p))
# define /* bo_m128_f32_t */   vipmvi_load1x16_f32(p)           vld1q_f32((void const *)(p))

# define /* bo_m128_s8_t  */   vipmvi_loada1x16_s8(p)           vipmvi_load1x16_s8(p)
# define /* bo_m128_s16_t */   vipmvi_loada1x16_s16(p)          vipmvi_load1x16_s16(p)
# define /* bo_m128_s32_t */   vipmvi_loada1x16_s32(p)          vipmvi_load1x16_s32(p)
# define /* bo_m128_u8_t  */   vipmvi_loada1x16_u8(p)           vipmvi_load1x16_u8(p)
# define /* bo_m128_u16_t */   vipmvi_loada1x16_u16(p)          vipmvi_load1x16_u16(p)
# define /* bo_m128_u32_t */   vipmvi_loada1x16_u32(p)          vipmvi_load1x16_u32(p)
# define /* bo_m128_f32_t */   vipmvi_loada1x16_f32(p)          vipmvi_load1x16_f32(p)

# define /* bo_m128_s8_t  */   vipmvi_loadu1x16_s8(p)           vipmvi_load1x16_s8(p)
# define /* bo_m128_s16_t */   vipmvi_loadu1x16_s16(p)          vipmvi_load1x16_s16(p)
# define /* bo_m128_s32_t */   vipmvi_loadu1x16_s32(p)          vipmvi_load1x16_s32(p)
# define /* bo_m128_u8_t  */   vipmvi_loadu1x16_u8(p)           vipmvi_load1x16_u8(p)
# define /* bo_m128_u16_t */   vipmvi_loadu1x16_u16(p)          vipmvi_load1x16_u16(p)
# define /* bo_m128_u32_t */   vipmvi_loadu1x16_u32(p)          vipmvi_load1x16_u32(p)
# define /* bo_m128_f32_t */   vipmvi_loadu1x16_f32(p)          vipmvi_load1x16_f32(p)

# define /* void */            vipmvi_store1x16_s8(p,x)         vst1q_s8( (void *)(p),x)
# define /* void */            vipmvi_store1x16_s16(p,x)        vst1q_s16((void *)(p),x)
# define /* void */            vipmvi_store1x16_s32(p,x)        vst1q_s32((void *)(p),x)
# define /* void */            vipmvi_store1x16_u8(p,x)         vst1q_u8( (void *)(p),x)
# define /* void */            vipmvi_store1x16_u16(p,x)        vst1q_u16((void *)(p),x)
# define /* void */            vipmvi_store1x16_u32(p,x)        vst1q_u32((void *)(p),x)
# define /* void */            vipmvi_store1x16_f32(p,x)        vst1q_f32((void *)(p),x)

# define /* void */            vipmvi_storea1x16_s8(p,x)        vipmvi_store1x16_s8( p,x)
# define /* void */            vipmvi_storea1x16_s16(p,x)       vipmvi_store1x16_s16(p,x)
# define /* void */            vipmvi_storea1x16_s32(p,x)       vipmvi_store1x16_s32(p,x)
# define /* void */            vipmvi_storea1x16_u8(p,x)        vipmvi_store1x16_u8( p,x)
# define /* void */            vipmvi_storea1x16_u16(p,x)       vipmvi_store1x16_u16(p,x)
# define /* void */            vipmvi_storea1x16_u32(p,x)       vipmvi_store1x16_u32(p,x)
# define /* void */            vipmvi_storea1x16_f32(p,x)       vipmvi_store1x16_f32(p,x)

# define /* void */            vipmvi_storeu1x16_s8(p,x)        vipmvi_store1x16_s8( p,x)
# define /* void */            vipmvi_storeu1x16_s16(p,x)       vipmvi_store1x16_s16(p,x)
# define /* void */            vipmvi_storeu1x16_s32(p,x)       vipmvi_store1x16_s32(p,x)
# define /* void */            vipmvi_storeu1x16_u8(p,x)        vipmvi_store1x16_u8( p,x)
# define /* void */            vipmvi_storeu1x16_u16(p,x)       vipmvi_store1x16_u16(p,x)
# define /* void */            vipmvi_storeu1x16_u32(p,x)       vipmvi_store1x16_u32(p,x)
# define /* void */            vipmvi_storeu1x16_f32(p,x)       vipmvi_store1x16_f32(p,x)

/* @todo: fixme */
# if defined(__clang__)

		/*
		 * Transfer 8x2=16 bytes
		 * 2 D registers
		 */

#  define /* bo_m64x2_s8_t  */ vipmvi_load1x8x2_s8(p)           vld1_s8_x2( (void const *)(p))
#  define /* bo_m64x2_s16_t */ vipmvi_load1x8x2_s16(p)          vld1_s16_x2((void const *)(p))
#  define /* bo_m64x2_s32_t */ vipmvi_load1x8x2_s32(p)          vld1_s32_x2((void const *)(p))
#  define /* bo_m64x2_u8_t  */ vipmvi_load1x8x2_u8(p)           vld1_u8_x2( (void const *)(p))
#  define /* bo_m64x2_u16_t */ vipmvi_load1x8x2_u16(p)          vld1_u16_x2((void const *)(p))
#  define /* bo_m64x2_u32_t */ vipmvi_load1x8x2_u32(p)          vld1_u32_x2((void const *)(p))
#  define /* bo_m64x2_f32_t */ vipmvi_load1x8x2_f32(p)          vld1_f32_x2((void const *)(p))

#  define /* bo_m64x2_s8_t  */ vipmvi_loada1x8x2_s8(p)          vipmvi_load1x8x2_s8(p)
#  define /* bo_m64x2_s16_t */ vipmvi_loada1x8x2_s16(p)         vipmvi_load1x8x2_s16(p)
#  define /* bo_m64x2_s32_t */ vipmvi_loada1x8x2_s32(p)         vipmvi_load1x8x2_s32(p)
#  define /* bo_m64x2_u8_t  */ vipmvi_loada1x8x2_u8(p)          vipmvi_load1x8x2_u8(p)
#  define /* bo_m64x2_u16_t */ vipmvi_loada1x8x2_u16(p)         vipmvi_load1x8x2_u16(p)
#  define /* bo_m64x2_u32_t */ vipmvi_loada1x8x2_u32(p)         vipmvi_load1x8x2_u32(p)
#  define /* bo_m64x2_f32_t */ vipmvi_loada1x8x2_f32(p)         vipmvi_load1x8x2_f32(p)

#  define /* bo_m64x2_s8_t  */ vipmvi_loadu1x8x2_s8(p)          vipmvi_load1x8x2_s8(p)
#  define /* bo_m64x2_s16_t */ vipmvi_loadu1x8x2_s16(p)         vipmvi_load1x8x2_s16(p)
#  define /* bo_m64x2_s32_t */ vipmvi_loadu1x8x2_s32(p)         vipmvi_load1x8x2_s32(p)
#  define /* bo_m64x2_u8_t  */ vipmvi_loadu1x8x2_u8(p)          vipmvi_load1x8x2_u8(p)
#  define /* bo_m64x2_u16_t */ vipmvi_loadu1x8x2_u16(p)         vipmvi_load1x8x2_u16(p)
#  define /* bo_m64x2_u32_t */ vipmvi_loadu1x8x2_u32(p)         vipmvi_load1x8x2_u32(p)
#  define /* bo_m64x2_f32_t */ vipmvi_loadu1x8x2_f32(p)         vipmvi_load1x8x2_f32(p)

#  define /* void */           vipmvi_store1x8x2_s8(p,x)        vst1_s8_x2( (void *)(p),x)
#  define /* void */           vipmvi_store1x8x2_s16(p,x)       vst1_s16_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x8x2_s32(p,x)       vst1_s32_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x8x2_u8(p,x)        vst1_u8_x2( (void *)(p),x)
#  define /* void */           vipmvi_store1x8x2_u16(p,x)       vst1_u16_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x8x2_u32(p,x)       vst1_u32_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x8x2_f32(p,x)       vst1_f32_x2((void *)(p),x)

#  define /* void */           vipmvi_storea1x8x2_s8(p,x)       vipmvi_store1x8x2_s8( p,x)
#  define /* void */           vipmvi_storea1x8x2_s16(p,x)      vipmvi_store1x8x2_s16(p,x)
#  define /* void */           vipmvi_storea1x8x2_s32(p,x)      vipmvi_store1x8x2_s32(p,x)
#  define /* void */           vipmvi_storea1x8x2_u8(p,x)       vipmvi_store1x8x2_u8( p,x)
#  define /* void */           vipmvi_storea1x8x2_u16(p,x)      vipmvi_store1x8x2_u16(p,x)
#  define /* void */           vipmvi_storea1x8x2_u32(p,x)      vipmvi_store1x8x2_u32(p,x)
#  define /* void */           vipmvi_storea1x8x2_f32(p,x)      vipmvi_store1x8x2_f32(p,x)

#  define /* void */           vipmvi_storeu1x8x2_s8(p,x)       vipmvi_store1x8x2_s8( p,x)
#  define /* void */           vipmvi_storeu1x8x2_s16(p,x)      vipmvi_store1x8x2_s16(p,x)
#  define /* void */           vipmvi_storeu1x8x2_s32(p,x)      vipmvi_store1x8x2_s32(p,x)
#  define /* void */           vipmvi_storeu1x8x2_u8(p,x)       vipmvi_store1x8x2_u8( p,x)
#  define /* void */           vipmvi_storeu1x8x2_u16(p,x)      vipmvi_store1x8x2_u16(p,x)
#  define /* void */           vipmvi_storeu1x8x2_u32(p,x)      vipmvi_store1x8x2_u32(p,x)
#  define /* void */           vipmvi_storeu1x8x2_f32(p,x)      vipmvi_store1x8x2_f32(p,x)


		/*
		 * Transfer 8x3=24 bytes
		 * 3 D registers
		 */

#  define /* bo_m64x3_s8_t  */ vipmvi_load1x8x3_s8(p)           vld1_s8_x3( (void const *)(p))
#  define /* bo_m64x3_s16_t */ vipmvi_load1x8x3_s16(p)          vld1_s16_x3((void const *)(p))
#  define /* bo_m64x3_s32_t */ vipmvi_load1x8x3_s32(p)          vld1_s32_x3((void const *)(p))
#  define /* bo_m64x3_u8_t  */ vipmvi_load1x8x3_u8(p)           vld1_u8_x3( (void const *)(p))
#  define /* bo_m64x3_u16_t */ vipmvi_load1x8x3_u16(p)          vld1_u16_x3((void const *)(p))
#  define /* bo_m64x3_u32_t */ vipmvi_load1x8x3_u32(p)          vld1_u32_x3((void const *)(p))
#  define /* bo_m64x3_f32_t */ vipmvi_load1x8x3_f32(p)          vld1_f32_x3((void const *)(p))

#  define /* bo_m64x3_s8_t  */ vipmvi_loada1x8x3_s8(p)          vipmvi_load1x8x3_s8(p)
#  define /* bo_m64x3_s16_t */ vipmvi_loada1x8x3_s16(p)         vipmvi_load1x8x3_s16(p)
#  define /* bo_m64x3_s32_t */ vipmvi_loada1x8x3_s32(p)         vipmvi_load1x8x3_s32(p)
#  define /* bo_m64x3_u8_t  */ vipmvi_loada1x8x3_u8(p)          vipmvi_load1x8x3_u8(p)
#  define /* bo_m64x3_u16_t */ vipmvi_loada1x8x3_u16(p)         vipmvi_load1x8x3_u16(p)
#  define /* bo_m64x3_u32_t */ vipmvi_loada1x8x3_u32(p)         vipmvi_load1x8x3_u32(p)
#  define /* bo_m64x3_f32_t */ vipmvi_loada1x8x3_f32(p)         vipmvi_load1x8x3_f32(p)

#  define /* bo_m64x3_s8_t  */ vipmvi_loadu1x8x3_s8(p)          vipmvi_load1x8x3_s8(p)
#  define /* bo_m64x3_s16_t */ vipmvi_loadu1x8x3_s16(p)         vipmvi_load1x8x3_s16(p)
#  define /* bo_m64x3_s32_t */ vipmvi_loadu1x8x3_s32(p)         vipmvi_load1x8x3_s32(p)
#  define /* bo_m64x3_u8_t  */ vipmvi_loadu1x8x3_u8(p)          vipmvi_load1x8x3_u8(p)
#  define /* bo_m64x3_u16_t */ vipmvi_loadu1x8x3_u16(p)         vipmvi_load1x8x3_u16(p)
#  define /* bo_m64x3_u32_t */ vipmvi_loadu1x8x3_u32(p)         vipmvi_load1x8x3_u32(p)
#  define /* bo_m64x3_f32_t */ vipmvi_loadu1x8x3_f32(p)         vipmvi_load1x8x3_f32(p)

#  define /* void */           vipmvi_store1x8x3_s8(p,x)        vst1_s8_x3( (void *)(p),x)
#  define /* void */           vipmvi_store1x8x3_s16(p,x)       vst1_s16_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x8x3_s32(p,x)       vst1_s32_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x8x3_u8(p,x)        vst1_u8_x3( (void *)(p),x)
#  define /* void */           vipmvi_store1x8x3_u16(p,x)       vst1_u16_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x8x3_u32(p,x)       vst1_u32_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x8x3_f32(p,x)       vst1_f32_x3((void *)(p),x)

#  define /* void */           vipmvi_storea1x8x3_s8(p,x)       vipmvi_store1x8x3_s8( p,x)
#  define /* void */           vipmvi_storea1x8x3_s16(p,x)      vipmvi_store1x8x3_s16(p,x)
#  define /* void */           vipmvi_storea1x8x3_s32(p,x)      vipmvi_store1x8x3_s32(p,x)
#  define /* void */           vipmvi_storea1x8x3_u8(p,x)       vipmvi_store1x8x3_u8( p,x)
#  define /* void */           vipmvi_storea1x8x3_u16(p,x)      vipmvi_store1x8x3_u16(p,x)
#  define /* void */           vipmvi_storea1x8x3_u32(p,x)      vipmvi_store1x8x3_u32(p,x)
#  define /* void */           vipmvi_storea1x8x3_f32(p,x)      vipmvi_store1x8x3_f32(p,x)

#  define /* void */           vipmvi_storeu1x8x3_s8(p,x)       vipmvi_store1x8x3_s8( p,x)
#  define /* void */           vipmvi_storeu1x8x3_s16(p,x)      vipmvi_store1x8x3_s16(p,x)
#  define /* void */           vipmvi_storeu1x8x3_s32(p,x)      vipmvi_store1x8x3_s32(p,x)
#  define /* void */           vipmvi_storeu1x8x3_u8(p,x)       vipmvi_store1x8x3_u8( p,x)
#  define /* void */           vipmvi_storeu1x8x3_u16(p,x)      vipmvi_store1x8x3_u16(p,x)
#  define /* void */           vipmvi_storeu1x8x3_u32(p,x)      vipmvi_store1x8x3_u32(p,x)
#  define /* void */           vipmvi_storeu1x8x3_f32(p,x)      vipmvi_store1x8x3_f32(p,x)


		/*
		 * Transfer 8x4=32 bytes
		 * 4 D registers
		 */

#  define /* bo_m64x4_s8_t  */ vipmvi_load1x8x4_s8(p)           vld1_s8_x4( (void const *)(p))
#  define /* bo_m64x4_s16_t */ vipmvi_load1x8x4_s16(p)          vld1_s16_x4((void const *)(p))
#  define /* bo_m64x4_s32_t */ vipmvi_load1x8x4_s32(p)          vld1_s32_x4((void const *)(p))
#  define /* bo_m64x4_u8_t  */ vipmvi_load1x8x4_u8(p)           vld1_u8_x4( (void const *)(p))
#  define /* bo_m64x4_u16_t */ vipmvi_load1x8x4_u16(p)          vld1_u16_x4((void const *)(p))
#  define /* bo_m64x4_u32_t */ vipmvi_load1x8x4_u32(p)          vld1_u32_x4((void const *)(p))
#  define /* bo_m64x4_f32_t */ vipmvi_load1x8x4_f32(p)          vld1_f32_x4((void const *)(p))

#  define /* bo_m64x4_s8_t  */ vipmvi_loada1x8x4_s8(p)          vipmvi_load1x8x4_s8(p)
#  define /* bo_m64x4_s16_t */ vipmvi_loada1x8x4_s16(p)         vipmvi_load1x8x4_s16(p)
#  define /* bo_m64x4_s32_t */ vipmvi_loada1x8x4_s32(p)         vipmvi_load1x8x4_s32(p)
#  define /* bo_m64x4_u8_t  */ vipmvi_loada1x8x4_u8(p)          vipmvi_load1x8x4_u8(p)
#  define /* bo_m64x4_u16_t */ vipmvi_loada1x8x4_u16(p)         vipmvi_load1x8x4_u16(p)
#  define /* bo_m64x4_u32_t */ vipmvi_loada1x8x4_u32(p)         vipmvi_load1x8x4_u32(p)
#  define /* bo_m64x4_f32_t */ vipmvi_loada1x8x4_f32(p)         vipmvi_load1x8x4_f32(p)

#  define /* bo_m64x4_s8_t  */ vipmvi_loadu1x8x4_s8(p)          vipmvi_load1x8x4_s8(p)
#  define /* bo_m64x4_s16_t */ vipmvi_loadu1x8x4_s16(p)         vipmvi_load1x8x4_s16(p)
#  define /* bo_m64x4_s32_t */ vipmvi_loadu1x8x4_s32(p)         vipmvi_load1x8x4_s32(p)
#  define /* bo_m64x4_u8_t  */ vipmvi_loadu1x8x4_u8(p)          vipmvi_load1x8x4_u8(p)
#  define /* bo_m64x4_u16_t */ vipmvi_loadu1x8x4_u16(p)         vipmvi_load1x8x4_u16(p)
#  define /* bo_m64x4_u32_t */ vipmvi_loadu1x8x4_u32(p)         vipmvi_load1x8x4_u32(p)
#  define /* bo_m64x4_f32_t */ vipmvi_loadu1x8x4_f32(p)         vipmvi_load1x8x4_f32(p)

#  define /* void */           vipmvi_store1x8x4_s8(p,x)        vst1_s8_x4( (void *)(p),x)
#  define /* void */           vipmvi_store1x8x4_s16(p,x)       vst1_s16_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x8x4_s32(p,x)       vst1_s32_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x8x4_u8(p,x)        vst1_u8_x4( (void *)(p),x)
#  define /* void */           vipmvi_store1x8x4_u16(p,x)       vst1_u16_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x8x4_u32(p,x)       vst1_u32_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x8x4_f32(p,x)       vst1_f32_x4((void *)(p),x)

#  define /* void */           vipmvi_storea1x8x4_s8(p,x)       vipmvi_store1x8x4_s8( p,x)
#  define /* void */           vipmvi_storea1x8x4_s16(p,x)      vipmvi_store1x8x4_s16(p,x)
#  define /* void */           vipmvi_storea1x8x4_s32(p,x)      vipmvi_store1x8x4_s32(p,x)
#  define /* void */           vipmvi_storea1x8x4_u8(p,x)       vipmvi_store1x8x4_u8( p,x)
#  define /* void */           vipmvi_storea1x8x4_u16(p,x)      vipmvi_store1x8x4_u16(p,x)
#  define /* void */           vipmvi_storea1x8x4_u32(p,x)      vipmvi_store1x8x4_u32(p,x)
#  define /* void */           vipmvi_storea1x8x4_f32(p,x)      vipmvi_store1x8x4_f32(p,x)

#  define /* void */           vipmvi_storeu1x8x4_s8(p,x)       vipmvi_store1x8x4_s8( p,x)
#  define /* void */           vipmvi_storeu1x8x4_s16(p,x)      vipmvi_store1x8x4_s16(p,x)
#  define /* void */           vipmvi_storeu1x8x4_s32(p,x)      vipmvi_store1x8x4_s32(p,x)
#  define /* void */           vipmvi_storeu1x8x4_u8(p,x)       vipmvi_store1x8x4_u8( p,x)
#  define /* void */           vipmvi_storeu1x8x4_u16(p,x)      vipmvi_store1x8x4_u16(p,x)
#  define /* void */           vipmvi_storeu1x8x4_u32(p,x)      vipmvi_store1x8x4_u32(p,x)
#  define /* void */           vipmvi_storeu1x8x4_f32(p,x)      vipmvi_store1x8x4_f32(p,x)


		/*
		 * Transfer 16x2=32 bytes
		 * 2 Q registers
		 */

#  define /* bo_m128x2_s8_t  */vipmvi_load1x16x2_s8(p)          vld1q_s8_x2( (void const *)(p))
#  define /* bo_m128x2_s16_t */vipmvi_load1x16x2_s16(p)         vld1q_s16_x2((void const *)(p))
#  define /* bo_m128x2_s32_t */vipmvi_load1x16x2_s32(p)         vld1q_s32_x2((void const *)(p))
#  define /* bo_m128x2_u8_t  */vipmvi_load1x16x2_u8(p)          vld1q_u8_x2( (void const *)(p))
#  define /* bo_m128x2_u16_t */vipmvi_load1x16x2_u16(p)         vld1q_u16_x2((void const *)(p))
#  define /* bo_m128x2_u32_t */vipmvi_load1x16x2_u32(p)         vld1q_u32_x2((void const *)(p))
#  define /* bo_m128x2_f32_t */vipmvi_load1x16x2_f32(p)         vld1q_f32_x2((void const *)(p))

#  define /* bo_m128x2_s8_t  */vipmvi_loada1x16x2_s8(p)         vipmvi_load1x16x2_s8(p)
#  define /* bo_m128x2_s16_t */vipmvi_loada1x16x2_s16(p)        vipmvi_load1x16x2_s16(p)
#  define /* bo_m128x2_s32_t */vipmvi_loada1x16x2_s32(p)        vipmvi_load1x16x2_s32(p)
#  define /* bo_m128x2_u8_t  */vipmvi_loada1x16x2_u8(p)         vipmvi_load1x16x2_u8(p)
#  define /* bo_m128x2_u16_t */vipmvi_loada1x16x2_u16(p)        vipmvi_load1x16x2_u16(p)
#  define /* bo_m128x2_u32_t */vipmvi_loada1x16x2_u32(p)        vipmvi_load1x16x2_u32(p)
#  define /* bo_m128x2_f32_t */vipmvi_loada1x16x2_f32(p)        vipmvi_load1x16x2_f32(p)

#  define /* bo_m128x2_s8_t  */vipmvi_loadu1x16x2_s8(p)         vipmvi_load1x16x2_s8(p)
#  define /* bo_m128x2_s16_t */vipmvi_loadu1x16x2_s16(p)        vipmvi_load1x16x2_s16(p)
#  define /* bo_m128x2_s32_t */vipmvi_loadu1x16x2_s32(p)        vipmvi_load1x16x2_s32(p)
#  define /* bo_m128x2_u8_t  */vipmvi_loadu1x16x2_u8(p)         vipmvi_load1x16x2_u8(p)
#  define /* bo_m128x2_u16_t */vipmvi_loadu1x16x2_u16(p)        vipmvi_load1x16x2_u16(p)
#  define /* bo_m128x2_u32_t */vipmvi_loadu1x16x2_u32(p)        vipmvi_load1x16x2_u32(p)
#  define /* bo_m128x2_f32_t */vipmvi_loadu1x16x2_f32(p)        vipmvi_load1x16x2_f32(p)

#  define /* void */           vipmvi_store1x16x2_s8(p,x)       vst1q_s8_x2( (void *)(p),x)
#  define /* void */           vipmvi_store1x16x2_s16(p,x)      vst1q_s16_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x16x2_s32(p,x)      vst1q_s32_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x16x2_u8(p,x)       vst1q_u8_x2( (void *)(p),x)
#  define /* void */           vipmvi_store1x16x2_u16(p,x)      vst1q_u16_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x16x2_u32(p,x)      vst1q_u32_x2((void *)(p),x)
#  define /* void */           vipmvi_store1x16x2_f32(p,x)      vst1q_f32_x2((void *)(p),x)

#  define /* void */           vipmvi_storea1x16x2_s8(p,x)      vipmvi_store1x16x2_s8( p,x)
#  define /* void */           vipmvi_storea1x16x2_s16(p,x)     vipmvi_store1x16x2_s16(p,x)
#  define /* void */           vipmvi_storea1x16x2_s32(p,x)     vipmvi_store1x16x2_s32(p,x)
#  define /* void */           vipmvi_storea1x16x2_u8(p,x)      vipmvi_store1x16x2_u8( p,x)
#  define /* void */           vipmvi_storea1x16x2_u16(p,x)     vipmvi_store1x16x2_u16(p,x)
#  define /* void */           vipmvi_storea1x16x2_u32(p,x)     vipmvi_store1x16x2_u32(p,x)
#  define /* void */           vipmvi_storea1x16x2_f32(p,x)     vipmvi_store1x16x2_f32(p,x)

#  define /* void */           vipmvi_storeu1x16x2_s8(p,x)      vipmvi_store1x16x2_s8( p,x)
#  define /* void */           vipmvi_storeu1x16x2_s16(p,x)     vipmvi_store1x16x2_s16(p,x)
#  define /* void */           vipmvi_storeu1x16x2_s32(p,x)     vipmvi_store1x16x2_s32(p,x)
#  define /* void */           vipmvi_storeu1x16x2_u8(p,x)      vipmvi_store1x16x2_u8( p,x)
#  define /* void */           vipmvi_storeu1x16x2_u16(p,x)     vipmvi_store1x16x2_u16(p,x)
#  define /* void */           vipmvi_storeu1x16x2_u32(p,x)     vipmvi_store1x16x2_u32(p,x)
#  define /* void */           vipmvi_storeu1x16x2_f32(p,x)     vipmvi_store1x16x2_f32(p,x)


		/*
		 * Transfer 16x3=48 bytes
		 * 3 Q registers
		 */

#  define /* bo_m128x3_s8_t  */vipmvi_load1x16x3_s8(p)          vld1q_s8_x3( (void const *)(p))
#  define /* bo_m128x3_s16_t */vipmvi_load1x16x3_s16(p)         vld1q_s16_x3((void const *)(p))
#  define /* bo_m128x3_s32_t */vipmvi_load1x16x3_s32(p)         vld1q_s32_x3((void const *)(p))
#  define /* bo_m128x3_u8_t  */vipmvi_load1x16x3_u8(p)          vld1q_u8_x3( (void const *)(p))
#  define /* bo_m128x3_u16_t */vipmvi_load1x16x3_u16(p)         vld1q_u16_x3((void const *)(p))
#  define /* bo_m128x3_u32_t */vipmvi_load1x16x3_u32(p)         vld1q_u32_x3((void const *)(p))
#  define /* bo_m128x3_f32_t */vipmvi_load1x16x3_f32(p)         vld1q_f32_x3((void const *)(p))

#  define /* bo_m128x3_s8_t  */vipmvi_loada1x16x3_s8(p)         vipmvi_load1x16x3_s8(p)
#  define /* bo_m128x3_s16_t */vipmvi_loada1x16x3_s16(p)        vipmvi_load1x16x3_s16(p)
#  define /* bo_m128x3_s32_t */vipmvi_loada1x16x3_s32(p)        vipmvi_load1x16x3_s32(p)
#  define /* bo_m128x3_u8_t  */vipmvi_loada1x16x3_u8(p)         vipmvi_load1x16x3_u8(p)
#  define /* bo_m128x3_u16_t */vipmvi_loada1x16x3_u16(p)        vipmvi_load1x16x3_u16(p)
#  define /* bo_m128x3_u32_t */vipmvi_loada1x16x3_u32(p)        vipmvi_load1x16x3_u32(p)
#  define /* bo_m128x3_f32_t */vipmvi_loada1x16x3_f32(p)        vipmvi_load1x16x3_f32(p)

#  define /* bo_m128x3_s8_t  */vipmvi_loadu1x16x3_s8(p)         vipmvi_load1x16x3_s8(p)
#  define /* bo_m128x3_s16_t */vipmvi_loadu1x16x3_s16(p)        vipmvi_load1x16x3_s16(p)
#  define /* bo_m128x3_s32_t */vipmvi_loadu1x16x3_s32(p)        vipmvi_load1x16x3_s32(p)
#  define /* bo_m128x3_u8_t  */vipmvi_loadu1x16x3_u8(p)         vipmvi_load1x16x3_u8(p)
#  define /* bo_m128x3_u16_t */vipmvi_loadu1x16x3_u16(p)        vipmvi_load1x16x3_u16(p)
#  define /* bo_m128x3_u32_t */vipmvi_loadu1x16x3_u32(p)        vipmvi_load1x16x3_u32(p)
#  define /* bo_m128x3_f32_t */vipmvi_loadu1x16x3_f32(p)        vipmvi_load1x16x3_f32(p)

#  define /* void */           vipmvi_store1x16x3_s8(p,x)       vst1q_s8_x3( (void *)(p),x)
#  define /* void */           vipmvi_store1x16x3_s16(p,x)      vst1q_s16_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x16x3_s32(p,x)      vst1q_s32_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x16x3_u8(p,x)       vst1q_u8_x3( (void *)(p),x)
#  define /* void */           vipmvi_store1x16x3_u16(p,x)      vst1q_u16_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x16x3_u32(p,x)      vst1q_u32_x3((void *)(p),x)
#  define /* void */           vipmvi_store1x16x3_f32(p,x)      vst1q_f32_x3((void *)(p),x)

#  define /* void */           vipmvi_storea1x16x3_s8(p,x)      vipmvi_store1x16x3_s8( p,x)
#  define /* void */           vipmvi_storea1x16x3_s16(p,x)     vipmvi_store1x16x3_s16(p,x)
#  define /* void */           vipmvi_storea1x16x3_s32(p,x)     vipmvi_store1x16x3_s32(p,x)
#  define /* void */           vipmvi_storea1x16x3_u8(p,x)      vipmvi_store1x16x3_u8( p,x)
#  define /* void */           vipmvi_storea1x16x3_u16(p,x)     vipmvi_store1x16x3_u16(p,x)
#  define /* void */           vipmvi_storea1x16x3_u32(p,x)     vipmvi_store1x16x3_u32(p,x)
#  define /* void */           vipmvi_storea1x16x3_f32(p,x)     vipmvi_store1x16x3_f32(p,x)

#  define /* void */           vipmvi_storeu1x16x3_s8(p,x)      vipmvi_store1x16x3_s8( p,x)
#  define /* void */           vipmvi_storeu1x16x3_s16(p,x)     vipmvi_store1x16x3_s16(p,x)
#  define /* void */           vipmvi_storeu1x16x3_s32(p,x)     vipmvi_store1x16x3_s32(p,x)
#  define /* void */           vipmvi_storeu1x16x3_u8(p,x)      vipmvi_store1x16x3_u8( p,x)
#  define /* void */           vipmvi_storeu1x16x3_u16(p,x)     vipmvi_store1x16x3_u16(p,x)
#  define /* void */           vipmvi_storeu1x16x3_u32(p,x)     vipmvi_store1x16x3_u32(p,x)
#  define /* void */           vipmvi_storeu1x16x3_f32(p,x)     vipmvi_store1x16x3_f32(p,x)


		/*
		 * Transfer 16x4=64 bytes
		 * 4 Q registers
		 */

#  define /* bo_m128x4_s8_t  */vipmvi_load1x16x4_s8(p)          vld1q_s8_x4( (void const *)(p))
#  define /* bo_m128x4_s16_t */vipmvi_load1x16x4_s16(p)         vld1q_s16_x4((void const *)(p))
#  define /* bo_m128x4_s32_t */vipmvi_load1x16x4_s32(p)         vld1q_s32_x4((void const *)(p))
#  define /* bo_m128x4_u8_t  */vipmvi_load1x16x4_u8(p)          vld1q_u8_x4( (void const *)(p))
#  define /* bo_m128x4_u16_t */vipmvi_load1x16x4_u16(p)         vld1q_u16_x4((void const *)(p))
#  define /* bo_m128x4_u32_t */vipmvi_load1x16x4_u32(p)         vld1q_u32_x4((void const *)(p))
#  define /* bo_m128x4_f32_t */vipmvi_load1x16x4_f32(p)         vld1q_f32_x4((void const *)(p))

#  define /* bo_m128x4_s8_t  */vipmvi_loada1x16x4_s8(p)         vipmvi_load1x16x4_s8(p)
#  define /* bo_m128x4_s16_t */vipmvi_loada1x16x4_s16(p)        vipmvi_load1x16x4_s16(p)
#  define /* bo_m128x4_s32_t */vipmvi_loada1x16x4_s32(p)        vipmvi_load1x16x4_s32(p)
#  define /* bo_m128x4_u8_t  */vipmvi_loada1x16x4_u8(p)         vipmvi_load1x16x4_u8(p)
#  define /* bo_m128x4_u16_t */vipmvi_loada1x16x4_u16(p)        vipmvi_load1x16x4_u16(p)
#  define /* bo_m128x4_u32_t */vipmvi_loada1x16x4_u32(p)        vipmvi_load1x16x4_u32(p)
#  define /* bo_m128x4_f32_t */vipmvi_loada1x16x4_f32(p)        vipmvi_load1x16x4_f32(p)

#  define /* bo_m128x4_s8_t  */vipmvi_loadu1x16x4_s8(p)         vipmvi_load1x16x4_s8(p)
#  define /* bo_m128x4_s16_t */vipmvi_loadu1x16x4_s16(p)        vipmvi_load1x16x4_s16(p)
#  define /* bo_m128x4_s32_t */vipmvi_loadu1x16x4_s32(p)        vipmvi_load1x16x4_s32(p)
#  define /* bo_m128x4_u8_t  */vipmvi_loadu1x16x4_u8(p)         vipmvi_load1x16x4_u8(p)
#  define /* bo_m128x4_u16_t */vipmvi_loadu1x16x4_u16(p)        vipmvi_load1x16x4_u16(p)
#  define /* bo_m128x4_u32_t */vipmvi_loadu1x16x4_u32(p)        vipmvi_load1x16x4_u32(p)
#  define /* bo_m128x4_f32_t */vipmvi_loadu1x16x4_f32(p)        vipmvi_load1x16x4_f32(p)

#  define /* void */           vipmvi_store1x16x4_s8(p,x)       vst1q_s8_x4( (void *)(p),x)
#  define /* void */           vipmvi_store1x16x4_s16(p,x)      vst1q_s16_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x16x4_s32(p,x)      vst1q_s32_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x16x4_u8(p,x)       vst1q_u8_x4( (void *)(p),x)
#  define /* void */           vipmvi_store1x16x4_u16(p,x)      vst1q_u16_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x16x4_u32(p,x)      vst1q_u32_x4((void *)(p),x)
#  define /* void */           vipmvi_store1x16x4_f32(p,x)      vst1q_f32_x4((void *)(p),x)

#  define /* void */           vipmvi_storea1x16x4_s8(p,x)      vipmvi_store1x16x4_s8( p,x)
#  define /* void */           vipmvi_storea1x16x4_s16(p,x)     vipmvi_store1x16x4_s16(p,x)
#  define /* void */           vipmvi_storea1x16x4_s32(p,x)     vipmvi_store1x16x4_s32(p,x)
#  define /* void */           vipmvi_storea1x16x4_u8(p,x)      vipmvi_store1x16x4_u8( p,x)
#  define /* void */           vipmvi_storea1x16x4_u16(p,x)     vipmvi_store1x16x4_u16(p,x)
#  define /* void */           vipmvi_storea1x16x4_u32(p,x)     vipmvi_store1x16x4_u32(p,x)
#  define /* void */           vipmvi_storea1x16x4_f32(p,x)     vipmvi_store1x16x4_f32(p,x)

#  define /* void */           vipmvi_storeu1x16x4_s8(p,x)      vipmvi_store1x16x4_s8( p,x)
#  define /* void */           vipmvi_storeu1x16x4_s16(p,x)     vipmvi_store1x16x4_s16(p,x)
#  define /* void */           vipmvi_storeu1x16x4_s32(p,x)     vipmvi_store1x16x4_s32(p,x)
#  define /* void */           vipmvi_storeu1x16x4_u8(p,x)      vipmvi_store1x16x4_u8( p,x)
#  define /* void */           vipmvi_storeu1x16x4_u16(p,x)     vipmvi_store1x16x4_u16(p,x)
#  define /* void */           vipmvi_storeu1x16x4_u32(p,x)     vipmvi_store1x16x4_u32(p,x)
#  define /* void */           vipmvi_storeu1x16x4_f32(p,x)     vipmvi_store1x16x4_f32(p,x)

# else /* not supported _xn versions */

		/*
		 * Transfer 8x2=16 bytes
		 * 2 D registers
		 */

#  define /* bo_m64x2_s8_t  */ vipmvi_loada1x8x2_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s8_t) {{ vipmvi_loada1x8_s8( ___p), vipmvi_loada1x8_s8( ___p + 8) }}; })
#  define /* bo_m64x2_s16_t */ vipmvi_loada1x8x2_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s16_t){{ vipmvi_loada1x8_s16(___p), vipmvi_loada1x8_s16(___p + 8) }}; })
#  define /* bo_m64x2_s32_t */ vipmvi_loada1x8x2_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s32_t){{ vipmvi_loada1x8_s32(___p), vipmvi_loada1x8_s32(___p + 8) }}; })
#  define /* bo_m64x2_u8_t  */ vipmvi_loada1x8x2_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u8_t) {{ vipmvi_loada1x8_u8( ___p), vipmvi_loada1x8_u8( ___p + 8) }}; })
#  define /* bo_m64x2_u16_t */ vipmvi_loada1x8x2_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u16_t){{ vipmvi_loada1x8_u16(___p), vipmvi_loada1x8_u16(___p + 8) }}; })
#  define /* bo_m64x2_u32_t */ vipmvi_loada1x8x2_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u32_t){{ vipmvi_loada1x8_u32(___p), vipmvi_loada1x8_u32(___p + 8) }}; })
#  define /* bo_m64x2_f32_t */ vipmvi_loada1x8x2_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_f32_t){{ vipmvi_loada1x8_f32(___p), vipmvi_loada1x8_f32(___p + 8) }}; })

#  define /* bo_m64x2_s8_t  */ vipmvi_loadu1x8x2_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s8_t) {{ vipmvi_loadu1x8_s8( ___p), vipmvi_loadu1x8_s8( ___p + 8) }}; })
#  define /* bo_m64x2_s16_t */ vipmvi_loadu1x8x2_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s16_t){{ vipmvi_loadu1x8_s16(___p), vipmvi_loadu1x8_s16(___p + 8) }}; })
#  define /* bo_m64x2_s32_t */ vipmvi_loadu1x8x2_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s32_t){{ vipmvi_loadu1x8_s32(___p), vipmvi_loadu1x8_s32(___p + 8) }}; })
#  define /* bo_m64x2_u8_t  */ vipmvi_loadu1x8x2_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u8_t) {{ vipmvi_loadu1x8_u8( ___p), vipmvi_loadu1x8_u8( ___p + 8) }}; })
#  define /* bo_m64x2_u16_t */ vipmvi_loadu1x8x2_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u16_t){{ vipmvi_loadu1x8_u16(___p), vipmvi_loadu1x8_u16(___p + 8) }}; })
#  define /* bo_m64x2_u32_t */ vipmvi_loadu1x8x2_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u32_t){{ vipmvi_loadu1x8_u32(___p), vipmvi_loadu1x8_u32(___p + 8) }}; })
#  define /* bo_m64x2_f32_t */ vipmvi_loadu1x8x2_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_f32_t){{ vipmvi_loadu1x8_f32(___p), vipmvi_loadu1x8_f32(___p + 8) }}; })

#  define /* bo_m64x2_s8_t  */ vipmvi_load1x8x2_s8(p)           __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s8_t) {{ vipmvi_load1x8_s8( ___p), vipmvi_load1x8_s8( ___p + 8) }}; })
#  define /* bo_m64x2_s16_t */ vipmvi_load1x8x2_s16(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s16_t){{ vipmvi_load1x8_s16(___p), vipmvi_load1x8_s16(___p + 8) }}; })
#  define /* bo_m64x2_s32_t */ vipmvi_load1x8x2_s32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_s32_t){{ vipmvi_load1x8_s32(___p), vipmvi_load1x8_s32(___p + 8) }}; })
#  define /* bo_m64x2_u8_t  */ vipmvi_load1x8x2_u8(p)           __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u8_t) {{ vipmvi_load1x8_u8( ___p), vipmvi_load1x8_u8( ___p + 8) }}; })
#  define /* bo_m64x2_u16_t */ vipmvi_load1x8x2_u16(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u16_t){{ vipmvi_load1x8_u16(___p), vipmvi_load1x8_u16(___p + 8) }}; })
#  define /* bo_m64x2_u32_t */ vipmvi_load1x8x2_u32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_u32_t){{ vipmvi_load1x8_u32(___p), vipmvi_load1x8_u32(___p + 8) }}; })
#  define /* bo_m64x2_f32_t */ vipmvi_load1x8x2_f32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x2_f32_t){{ vipmvi_load1x8_f32(___p), vipmvi_load1x8_f32(___p + 8) }}; })

#  define /* void */           vipmvi_storea1x8x2_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storea1x8_s8( ___p,(x).val[0]), vipmvi_storea1x8_s8( ___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x2_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_s16(___p,(x).val[0]), vipmvi_storea1x8_s16(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x2_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_s32(___p,(x).val[0]), vipmvi_storea1x8_s32(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x2_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storea1x8_u8( ___p,(x).val[0]), vipmvi_storea1x8_u8( ___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x2_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_u16(___p,(x).val[0]), vipmvi_storea1x8_u16(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x2_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_u32(___p,(x).val[0]), vipmvi_storea1x8_u32(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x2_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_f32(___p,(x).val[0]), vipmvi_storea1x8_f32(___p + 8,(x).val[1]) ); } while (0)

#  define /* void */           vipmvi_storeu1x8x2_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s8( ___p,(x).val[0]), vipmvi_storeu1x8_s8( ___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s16(___p,(x).val[0]), vipmvi_storeu1x8_s16(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s32(___p,(x).val[0]), vipmvi_storeu1x8_s32(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u8( ___p,(x).val[0]), vipmvi_storeu1x8_u8( ___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u16(___p,(x).val[0]), vipmvi_storeu1x8_u16(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u32(___p,(x).val[0]), vipmvi_storeu1x8_u32(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_f32(___p,(x).val[0]), vipmvi_storeu1x8_f32(___p + 8,(x).val[1]) ); } while (0)

#  define /* void */           vipmvi_store1x8x2_s8(p,x)        do { char *___p = (char *)(p); ( vipmvi_store1x8_s8( ___p,(x).val[0]), vipmvi_store1x8_s8( ___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x8x2_s16(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_s16(___p,(x).val[0]), vipmvi_store1x8_s16(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x8x2_s32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_s32(___p,(x).val[0]), vipmvi_store1x8_s32(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x8x2_u8(p,x)        do { char *___p = (char *)(p); ( vipmvi_store1x8_u8( ___p,(x).val[0]), vipmvi_store1x8_u8( ___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x8x2_u16(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_u16(___p,(x).val[0]), vipmvi_store1x8_u16(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x8x2_u32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_u32(___p,(x).val[0]), vipmvi_store1x8_u32(___p + 8,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x8x2_f32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_f32(___p,(x).val[0]), vipmvi_store1x8_f32(___p + 8,(x).val[1]) ); } while (0)


		/*
		 * Transfer 8x3=24 bytes
		 * 3 D registers
		 */

#  define /* bo_m64x3_s8_t  */ vipmvi_loada1x8x3_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s8_t) {{ vipmvi_loada1x8_s8( ___p), vipmvi_loada1x8_s8( ___p + 8), vipmvi_loada1x8_s8( ___p + 16) }}; })
#  define /* bo_m64x3_s16_t */ vipmvi_loada1x8x3_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s16_t){{ vipmvi_loada1x8_s16(___p), vipmvi_loada1x8_s16(___p + 8), vipmvi_loada1x8_s16(___p + 16) }}; })
#  define /* bo_m64x3_s32_t */ vipmvi_loada1x8x3_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s32_t){{ vipmvi_loada1x8_s32(___p), vipmvi_loada1x8_s32(___p + 8), vipmvi_loada1x8_s32(___p + 16) }}; })
#  define /* bo_m64x3_u8_t  */ vipmvi_loada1x8x3_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u8_t) {{ vipmvi_loada1x8_u8( ___p), vipmvi_loada1x8_u8( ___p + 8), vipmvi_loada1x8_u8( ___p + 16) }}; })
#  define /* bo_m64x3_u16_t */ vipmvi_loada1x8x3_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u16_t){{ vipmvi_loada1x8_u16(___p), vipmvi_loada1x8_u16(___p + 8), vipmvi_loada1x8_u16(___p + 16) }}; })
#  define /* bo_m64x3_u32_t */ vipmvi_loada1x8x3_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u32_t){{ vipmvi_loada1x8_u32(___p), vipmvi_loada1x8_u32(___p + 8), vipmvi_loada1x8_u32(___p + 16) }}; })
#  define /* bo_m64x3_f32_t */ vipmvi_loada1x8x3_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_f32_t){{ vipmvi_loada1x8_f32(___p), vipmvi_loada1x8_f32(___p + 8), vipmvi_loada1x8_f32(___p + 16) }}; })

#  define /* bo_m64x3_s8_t  */ vipmvi_loadu1x8x3_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s8_t) {{ vipmvi_loadu1x8_s8( ___p), vipmvi_loadu1x8_s8( ___p + 8), vipmvi_loadu1x8_s8( ___p + 16) }}; })
#  define /* bo_m64x3_s16_t */ vipmvi_loadu1x8x3_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s16_t){{ vipmvi_loadu1x8_s16(___p), vipmvi_loadu1x8_s16(___p + 8), vipmvi_loadu1x8_s16(___p + 16) }}; })
#  define /* bo_m64x3_s32_t */ vipmvi_loadu1x8x3_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s32_t){{ vipmvi_loadu1x8_s32(___p), vipmvi_loadu1x8_s32(___p + 8), vipmvi_loadu1x8_s32(___p + 16) }}; })
#  define /* bo_m64x3_u8_t  */ vipmvi_loadu1x8x3_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u8_t) {{ vipmvi_loadu1x8_u8( ___p), vipmvi_loadu1x8_u8( ___p + 8), vipmvi_loadu1x8_u8( ___p + 16) }}; })
#  define /* bo_m64x3_u16_t */ vipmvi_loadu1x8x3_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u16_t){{ vipmvi_loadu1x8_u16(___p), vipmvi_loadu1x8_u16(___p + 8), vipmvi_loadu1x8_u16(___p + 16) }}; })
#  define /* bo_m64x3_u32_t */ vipmvi_loadu1x8x3_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u32_t){{ vipmvi_loadu1x8_u32(___p), vipmvi_loadu1x8_u32(___p + 8), vipmvi_loadu1x8_u32(___p + 16) }}; })
#  define /* bo_m64x3_f32_t */ vipmvi_loadu1x8x3_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_f32_t){{ vipmvi_loadu1x8_f32(___p), vipmvi_loadu1x8_f32(___p + 8), vipmvi_loadu1x8_f32(___p + 16) }}; })

#  define /* bo_m64x3_s8_t  */ vipmvi_load1x8x3_s8(p)           __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s8_t) {{ vipmvi_load1x8_s8( ___p), vipmvi_load1x8_s8( ___p + 8), vipmvi_load1x8_s8( ___p + 16) }}; })
#  define /* bo_m64x3_s16_t */ vipmvi_load1x8x3_s16(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s16_t){{ vipmvi_load1x8_s16(___p), vipmvi_load1x8_s16(___p + 8), vipmvi_load1x8_s16(___p + 16) }}; })
#  define /* bo_m64x3_s32_t */ vipmvi_load1x8x3_s32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_s32_t){{ vipmvi_load1x8_s32(___p), vipmvi_load1x8_s32(___p + 8), vipmvi_load1x8_s32(___p + 16) }}; })
#  define /* bo_m64x3_u8_t  */ vipmvi_load1x8x3_u8(p)           __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u8_t) {{ vipmvi_load1x8_u8( ___p), vipmvi_load1x8_u8( ___p + 8), vipmvi_load1x8_u8( ___p + 16) }}; })
#  define /* bo_m64x3_u16_t */ vipmvi_load1x8x3_u16(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u16_t){{ vipmvi_load1x8_u16(___p), vipmvi_load1x8_u16(___p + 8), vipmvi_load1x8_u16(___p + 16) }}; })
#  define /* bo_m64x3_u32_t */ vipmvi_load1x8x3_u32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_u32_t){{ vipmvi_load1x8_u32(___p), vipmvi_load1x8_u32(___p + 8), vipmvi_load1x8_u32(___p + 16) }}; })
#  define /* bo_m64x3_f32_t */ vipmvi_load1x8x3_f32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x3_f32_t){{ vipmvi_load1x8_f32(___p), vipmvi_load1x8_f32(___p + 8), vipmvi_load1x8_f32(___p + 16) }}; })

#  define /* void */           vipmvi_storea1x8x3_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storea1x8_s8( ___p,(x).val[0]), vipmvi_storea1x8_s8( ___p + 8,(x).val[1]), vipmvi_storea1x8_s8( ___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x3_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_s16(___p,(x).val[0]), vipmvi_storea1x8_s16(___p + 8,(x).val[1]), vipmvi_storea1x8_s16(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x3_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_s32(___p,(x).val[0]), vipmvi_storea1x8_s32(___p + 8,(x).val[1]), vipmvi_storea1x8_s32(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x3_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storea1x8_u8( ___p,(x).val[0]), vipmvi_storea1x8_u8( ___p + 8,(x).val[1]), vipmvi_storea1x8_u8( ___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x3_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_u16(___p,(x).val[0]), vipmvi_storea1x8_u16(___p + 8,(x).val[1]), vipmvi_storea1x8_u16(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x3_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_u32(___p,(x).val[0]), vipmvi_storea1x8_u32(___p + 8,(x).val[1]), vipmvi_storea1x8_u32(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x3_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_f32(___p,(x).val[0]), vipmvi_storea1x8_f32(___p + 8,(x).val[1]), vipmvi_storea1x8_f32(___p + 16,(x).val[2]) ); } while (0)

#  define /* void */           vipmvi_storeu1x8x3_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s8( ___p,(x).val[0]), vipmvi_storeu1x8_s8( ___p + 8,(x).val[1]), vipmvi_storeu1x8_s8( ___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s16(___p,(x).val[0]), vipmvi_storeu1x8_s16(___p + 8,(x).val[1]), vipmvi_storeu1x8_s16(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s32(___p,(x).val[0]), vipmvi_storeu1x8_s32(___p + 8,(x).val[1]), vipmvi_storeu1x8_s32(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u8( ___p,(x).val[0]), vipmvi_storeu1x8_u8( ___p + 8,(x).val[1]), vipmvi_storeu1x8_u8( ___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u16(___p,(x).val[0]), vipmvi_storeu1x8_u16(___p + 8,(x).val[1]), vipmvi_storeu1x8_u16(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u32(___p,(x).val[0]), vipmvi_storeu1x8_u32(___p + 8,(x).val[1]), vipmvi_storeu1x8_u32(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_f32(___p,(x).val[0]), vipmvi_storeu1x8_f32(___p + 8,(x).val[1]), vipmvi_storeu1x8_f32(___p + 16,(x).val[2]) ); } while (0)

#  define /* void */           vipmvi_store1x8x3_s8(p,x)        do { char *___p = (char *)(p); ( vipmvi_store1x8_s8( ___p,(x).val[0]), vipmvi_store1x8_s8( ___p + 8,(x).val[1]), vipmvi_store1x8_s8( ___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_store1x8x3_s16(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_s16(___p,(x).val[0]), vipmvi_store1x8_s16(___p + 8,(x).val[1]), vipmvi_store1x8_s16(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_store1x8x3_s32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_s32(___p,(x).val[0]), vipmvi_store1x8_s32(___p + 8,(x).val[1]), vipmvi_store1x8_s32(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_store1x8x3_u8(p,x)        do { char *___p = (char *)(p); ( vipmvi_store1x8_u8( ___p,(x).val[0]), vipmvi_store1x8_u8( ___p + 8,(x).val[1]), vipmvi_store1x8_u8( ___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_store1x8x3_u16(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_u16(___p,(x).val[0]), vipmvi_store1x8_u16(___p + 8,(x).val[1]), vipmvi_store1x8_u16(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_store1x8x3_u32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_u32(___p,(x).val[0]), vipmvi_store1x8_u32(___p + 8,(x).val[1]), vipmvi_store1x8_u32(___p + 16,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_store1x8x3_f32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_f32(___p,(x).val[0]), vipmvi_store1x8_f32(___p + 8,(x).val[1]), vipmvi_store1x8_f32(___p + 16,(x).val[2]) ); } while (0)


		/*
		 * Transfer 8x4=32 bytes
		 * 4 D registers
		 */

#  define /* bo_m64x4_s8_t  */ vipmvi_loada1x8x4_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s8_t) {{ vipmvi_loada1x8_s8( ___p), vipmvi_loada1x8_s8( ___p + 8), vipmvi_loada1x8_s8( ___p + 16), vipmvi_loada1x8_s8( ___p + 24) }}; })
#  define /* bo_m64x4_s16_t */ vipmvi_loada1x8x4_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s16_t){{ vipmvi_loada1x8_s16(___p), vipmvi_loada1x8_s16(___p + 8), vipmvi_loada1x8_s16(___p + 16), vipmvi_loada1x8_s16(___p + 24) }}; })
#  define /* bo_m64x4_s32_t */ vipmvi_loada1x8x4_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s32_t){{ vipmvi_loada1x8_s32(___p), vipmvi_loada1x8_s32(___p + 8), vipmvi_loada1x8_s32(___p + 16), vipmvi_loada1x8_s32(___p + 24) }}; })
#  define /* bo_m64x4_u8_t  */ vipmvi_loada1x8x4_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u8_t) {{ vipmvi_loada1x8_u8( ___p), vipmvi_loada1x8_u8( ___p + 8), vipmvi_loada1x8_u8( ___p + 16), vipmvi_loada1x8_u8( ___p + 24) }}; })
#  define /* bo_m64x4_u16_t */ vipmvi_loada1x8x4_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u16_t){{ vipmvi_loada1x8_u16(___p), vipmvi_loada1x8_u16(___p + 8), vipmvi_loada1x8_u16(___p + 16), vipmvi_loada1x8_u16(___p + 24) }}; })
#  define /* bo_m64x4_u32_t */ vipmvi_loada1x8x4_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u32_t){{ vipmvi_loada1x8_u32(___p), vipmvi_loada1x8_u32(___p + 8), vipmvi_loada1x8_u32(___p + 16), vipmvi_loada1x8_u32(___p + 24) }}; })
#  define /* bo_m64x4_f32_t */ vipmvi_loada1x8x4_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_f32_t){{ vipmvi_loada1x8_f32(___p), vipmvi_loada1x8_f32(___p + 8), vipmvi_loada1x8_f32(___p + 16), vipmvi_loada1x8_f32(___p + 24) }}; })

#  define /* bo_m64x4_s8_t  */ vipmvi_loadu1x8x4_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s8_t) {{ vipmvi_loadu1x8_s8( ___p), vipmvi_loadu1x8_s8( ___p + 8), vipmvi_loadu1x8_s8( ___p + 16), vipmvi_loadu1x8_s8( ___p + 24) }}; })
#  define /* bo_m64x4_s16_t */ vipmvi_loadu1x8x4_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s16_t){{ vipmvi_loadu1x8_s16(___p), vipmvi_loadu1x8_s16(___p + 8), vipmvi_loadu1x8_s16(___p + 16), vipmvi_loadu1x8_s16(___p + 24) }}; })
#  define /* bo_m64x4_s32_t */ vipmvi_loadu1x8x4_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s32_t){{ vipmvi_loadu1x8_s32(___p), vipmvi_loadu1x8_s32(___p + 8), vipmvi_loadu1x8_s32(___p + 16), vipmvi_loadu1x8_s32(___p + 24) }}; })
#  define /* bo_m64x4_u8_t  */ vipmvi_loadu1x8x4_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u8_t) {{ vipmvi_loadu1x8_u8( ___p), vipmvi_loadu1x8_u8( ___p + 8), vipmvi_loadu1x8_u8( ___p + 16), vipmvi_loadu1x8_u8( ___p + 24) }}; })
#  define /* bo_m64x4_u16_t */ vipmvi_loadu1x8x4_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u16_t){{ vipmvi_loadu1x8_u16(___p), vipmvi_loadu1x8_u16(___p + 8), vipmvi_loadu1x8_u16(___p + 16), vipmvi_loadu1x8_u16(___p + 24) }}; })
#  define /* bo_m64x4_u32_t */ vipmvi_loadu1x8x4_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u32_t){{ vipmvi_loadu1x8_u32(___p), vipmvi_loadu1x8_u32(___p + 8), vipmvi_loadu1x8_u32(___p + 16), vipmvi_loadu1x8_u32(___p + 24) }}; })
#  define /* bo_m64x4_f32_t */ vipmvi_loadu1x8x4_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_f32_t){{ vipmvi_loadu1x8_f32(___p), vipmvi_loadu1x8_f32(___p + 8), vipmvi_loadu1x8_f32(___p + 16), vipmvi_loadu1x8_f32(___p + 24) }}; })

#  define /* bo_m64x4_s8_t  */ vipmvi_load1x8x4_s8(p)           __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s8_t) {{ vipmvi_load1x8_s8( ___p), vipmvi_load1x8_s8( ___p + 8), vipmvi_load1x8_s8( ___p + 16), vipmvi_load1x8_s8( ___p + 24) }}; })
#  define /* bo_m64x4_s16_t */ vipmvi_load1x8x4_s16(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s16_t){{ vipmvi_load1x8_s16(___p), vipmvi_load1x8_s16(___p + 8), vipmvi_load1x8_s16(___p + 16), vipmvi_load1x8_s16(___p + 24) }}; })
#  define /* bo_m64x4_s32_t */ vipmvi_load1x8x4_s32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_s32_t){{ vipmvi_load1x8_s32(___p), vipmvi_load1x8_s32(___p + 8), vipmvi_load1x8_s32(___p + 16), vipmvi_load1x8_s32(___p + 24) }}; })
#  define /* bo_m64x4_u8_t  */ vipmvi_load1x8x4_u8(p)           __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u8_t) {{ vipmvi_load1x8_u8( ___p), vipmvi_load1x8_u8( ___p + 8), vipmvi_load1x8_u8( ___p + 16), vipmvi_load1x8_u8( ___p + 24) }}; })
#  define /* bo_m64x4_u16_t */ vipmvi_load1x8x4_u16(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u16_t){{ vipmvi_load1x8_u16(___p), vipmvi_load1x8_u16(___p + 8), vipmvi_load1x8_u16(___p + 16), vipmvi_load1x8_u16(___p + 24) }}; })
#  define /* bo_m64x4_u32_t */ vipmvi_load1x8x4_u32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_u32_t){{ vipmvi_load1x8_u32(___p), vipmvi_load1x8_u32(___p + 8), vipmvi_load1x8_u32(___p + 16), vipmvi_load1x8_u32(___p + 24) }}; })
#  define /* bo_m64x4_f32_t */ vipmvi_load1x8x4_f32(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m64x4_f32_t){{ vipmvi_load1x8_f32(___p), vipmvi_load1x8_f32(___p + 8), vipmvi_load1x8_f32(___p + 16), vipmvi_load1x8_f32(___p + 24) }}; })

#  define /* void */           vipmvi_storea1x8x4_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storea1x8_s8( ___p,(x).val[0]), vipmvi_storea1x8_s8( ___p + 8,(x).val[1]), vipmvi_storea1x8_s8( ___p + 16,(x).val[2]), vipmvi_storea1x8_s8( ___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x4_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_s16(___p,(x).val[0]), vipmvi_storea1x8_s16(___p + 8,(x).val[1]), vipmvi_storea1x8_s16(___p + 16,(x).val[2]), vipmvi_storea1x8_s16(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x4_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_s32(___p,(x).val[0]), vipmvi_storea1x8_s32(___p + 8,(x).val[1]), vipmvi_storea1x8_s32(___p + 16,(x).val[2]), vipmvi_storea1x8_s32(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x4_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storea1x8_u8( ___p,(x).val[0]), vipmvi_storea1x8_u8( ___p + 8,(x).val[1]), vipmvi_storea1x8_u8( ___p + 16,(x).val[2]), vipmvi_storea1x8_u8( ___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x4_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_u16(___p,(x).val[0]), vipmvi_storea1x8_u16(___p + 8,(x).val[1]), vipmvi_storea1x8_u16(___p + 16,(x).val[2]), vipmvi_storea1x8_u16(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x4_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_u32(___p,(x).val[0]), vipmvi_storea1x8_u32(___p + 8,(x).val[1]), vipmvi_storea1x8_u32(___p + 16,(x).val[2]), vipmvi_storea1x8_u32(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x8x4_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x8_f32(___p,(x).val[0]), vipmvi_storea1x8_f32(___p + 8,(x).val[1]), vipmvi_storea1x8_f32(___p + 16,(x).val[2]), vipmvi_storea1x8_f32(___p + 24,(x).val[3]) ); } while (0)

#  define /* void */           vipmvi_storeu1x8x4_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s8( ___p,(x).val[0]), vipmvi_storeu1x8_s8( ___p + 8,(x).val[1]), vipmvi_storeu1x8_s8( ___p + 16,(x).val[2]), vipmvi_storeu1x8_s8( ___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s16(___p,(x).val[0]), vipmvi_storeu1x8_s16(___p + 8,(x).val[1]), vipmvi_storeu1x8_s16(___p + 16,(x).val[2]), vipmvi_storeu1x8_s16(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_s32(___p,(x).val[0]), vipmvi_storeu1x8_s32(___p + 8,(x).val[1]), vipmvi_storeu1x8_s32(___p + 16,(x).val[2]), vipmvi_storeu1x8_s32(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u8( ___p,(x).val[0]), vipmvi_storeu1x8_u8( ___p + 8,(x).val[1]), vipmvi_storeu1x8_u8( ___p + 16,(x).val[2]), vipmvi_storeu1x8_u8( ___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u16(___p,(x).val[0]), vipmvi_storeu1x8_u16(___p + 8,(x).val[1]), vipmvi_storeu1x8_u16(___p + 16,(x).val[2]), vipmvi_storeu1x8_u16(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_u32(___p,(x).val[0]), vipmvi_storeu1x8_u32(___p + 8,(x).val[1]), vipmvi_storeu1x8_u32(___p + 16,(x).val[2]), vipmvi_storeu1x8_u32(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x8_f32(___p,(x).val[0]), vipmvi_storeu1x8_f32(___p + 8,(x).val[1]), vipmvi_storeu1x8_f32(___p + 16,(x).val[2]), vipmvi_storeu1x8_f32(___p + 24,(x).val[3]) ); } while (0)

#  define /* void */           vipmvi_store1x8x4_s8(p,x)        do { char *___p = (char *)(p); ( vipmvi_store1x8_s8( ___p,(x).val[0]), vipmvi_store1x8_s8( ___p + 8,(x).val[1]), vipmvi_store1x8_s8( ___p + 16,(x).val[2]), vipmvi_store1x8_s8( ___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x8x4_s16(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_s16(___p,(x).val[0]), vipmvi_store1x8_s16(___p + 8,(x).val[1]), vipmvi_store1x8_s16(___p + 16,(x).val[2]), vipmvi_store1x8_s16(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x8x4_s32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_s32(___p,(x).val[0]), vipmvi_store1x8_s32(___p + 8,(x).val[1]), vipmvi_store1x8_s32(___p + 16,(x).val[2]), vipmvi_store1x8_s32(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x8x4_u8(p,x)        do { char *___p = (char *)(p); ( vipmvi_store1x8_u8( ___p,(x).val[0]), vipmvi_store1x8_u8( ___p + 8,(x).val[1]), vipmvi_store1x8_u8( ___p + 16,(x).val[2]), vipmvi_store1x8_u8( ___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x8x4_u16(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_u16(___p,(x).val[0]), vipmvi_store1x8_u16(___p + 8,(x).val[1]), vipmvi_store1x8_u16(___p + 16,(x).val[2]), vipmvi_store1x8_u16(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x8x4_u32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_u32(___p,(x).val[0]), vipmvi_store1x8_u32(___p + 8,(x).val[1]), vipmvi_store1x8_u32(___p + 16,(x).val[2]), vipmvi_store1x8_u32(___p + 24,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x8x4_f32(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x8_f32(___p,(x).val[0]), vipmvi_store1x8_f32(___p + 8,(x).val[1]), vipmvi_store1x8_f32(___p + 16,(x).val[2]), vipmvi_store1x8_f32(___p + 24,(x).val[3]) ); } while (0)


		/*
		 * Transfer 16x2=32 bytes
		 * 2 Q registers
		 */

#  define /* bo_m128x2_s8_t  */vipmvi_loada1x16x2_s8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s8_t) {{ vipmvi_loada1x16_s8( ___p), vipmvi_loada1x16_s8( ___p + 16) }}; })
#  define /* bo_m128x2_s16_t */vipmvi_loada1x16x2_s16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s16_t){{ vipmvi_loada1x16_s16(___p), vipmvi_loada1x16_s16(___p + 16) }}; })
#  define /* bo_m128x2_s32_t */vipmvi_loada1x16x2_s32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s32_t){{ vipmvi_loada1x16_s32(___p), vipmvi_loada1x16_s32(___p + 16) }}; })
#  define /* bo_m128x2_u8_t  */vipmvi_loada1x16x2_u8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u8_t) {{ vipmvi_loada1x16_u8( ___p), vipmvi_loada1x16_u8( ___p + 16) }}; })
#  define /* bo_m128x2_u16_t */vipmvi_loada1x16x2_u16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u16_t){{ vipmvi_loada1x16_u16(___p), vipmvi_loada1x16_u16(___p + 16) }}; })
#  define /* bo_m128x2_u32_t */vipmvi_loada1x16x2_u32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u32_t){{ vipmvi_loada1x16_u32(___p), vipmvi_loada1x16_u32(___p + 16) }}; })
#  define /* bo_m128x2_f32_t */vipmvi_loada1x16x2_f32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_f32_t){{ vipmvi_loada1x16_f32(___p), vipmvi_loada1x16_f32(___p + 16) }}; })

#  define /* bo_m128x2_s8_t  */vipmvi_loadu1x16x2_s8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s8_t) {{ vipmvi_loadu1x16_s8( ___p), vipmvi_loadu1x16_s8( ___p + 16) }}; })
#  define /* bo_m128x2_s16_t */vipmvi_loadu1x16x2_s16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s16_t){{ vipmvi_loadu1x16_s16(___p), vipmvi_loadu1x16_s16(___p + 16) }}; })
#  define /* bo_m128x2_s32_t */vipmvi_loadu1x16x2_s32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s32_t){{ vipmvi_loadu1x16_s32(___p), vipmvi_loadu1x16_s32(___p + 16) }}; })
#  define /* bo_m128x2_u8_t  */vipmvi_loadu1x16x2_u8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u8_t) {{ vipmvi_loadu1x16_u8( ___p), vipmvi_loadu1x16_u8( ___p + 16) }}; })
#  define /* bo_m128x2_u16_t */vipmvi_loadu1x16x2_u16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u16_t){{ vipmvi_loadu1x16_u16(___p), vipmvi_loadu1x16_u16(___p + 16) }}; })
#  define /* bo_m128x2_u32_t */vipmvi_loadu1x16x2_u32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u32_t){{ vipmvi_loadu1x16_u32(___p), vipmvi_loadu1x16_u32(___p + 16) }}; })
#  define /* bo_m128x2_f32_t */vipmvi_loadu1x16x2_f32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_f32_t){{ vipmvi_loadu1x16_f32(___p), vipmvi_loadu1x16_f32(___p + 16) }}; })

#  define /* bo_m128x2_s8_t  */vipmvi_load1x16x2_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s8_t) {{ vipmvi_load1x16_s8( ___p), vipmvi_load1x16_s8( ___p + 16)}; })
#  define /* bo_m128x2_s16_t */vipmvi_load1x16x2_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s16_t){{ vipmvi_load1x16_s16(___p), vipmvi_load1x16_s16(___p + 16)}; })
#  define /* bo_m128x2_s32_t */vipmvi_load1x16x2_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_s32_t){{ vipmvi_load1x16_s32(___p), vipmvi_load1x16_s32(___p + 16)}; })
#  define /* bo_m128x2_u8_t  */vipmvi_load1x16x2_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u8_t) {{ vipmvi_load1x16_u8( ___p), vipmvi_load1x16_u8( ___p + 16)}; })
#  define /* bo_m128x2_u16_t */vipmvi_load1x16x2_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u16_t){{ vipmvi_load1x16_u16(___p), vipmvi_load1x16_u16(___p + 16)}; })
#  define /* bo_m128x2_u32_t */vipmvi_load1x16x2_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_u32_t){{ vipmvi_load1x16_u32(___p), vipmvi_load1x16_u32(___p + 16)}; })
#  define /* bo_m128x2_f32_t */vipmvi_load1x16x2_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x2_f32_t){{ vipmvi_load1x16_f32(___p), vipmvi_load1x16_f32(___p + 16)}; })

#  define /* void */           vipmvi_storea1x16x2_s8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x16_s8( ___p,(x).val[0]), vipmvi_storea1x16_s8( ___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x2_s16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_s16(___p,(x).val[0]), vipmvi_storea1x16_s16(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x2_s32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_s32(___p,(x).val[0]), vipmvi_storea1x16_s32(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x2_u8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x16_u8( ___p,(x).val[0]), vipmvi_storea1x16_u8( ___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x2_u16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_u16(___p,(x).val[0]), vipmvi_storea1x16_u16(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x2_u32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_u32(___p,(x).val[0]), vipmvi_storea1x16_u32(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x2_f32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_f32(___p,(x).val[0]), vipmvi_storea1x16_f32(___p + 16,(x).val[1]) ); } while (0)

#  define /* void */           vipmvi_storeu1x16x2_s8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s8( ___p,(x).val[0]), vipmvi_storeu1x16_s8( ___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_s16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s16(___p,(x).val[0]), vipmvi_storeu1x16_s16(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_s32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s32(___p,(x).val[0]), vipmvi_storeu1x16_s32(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_u8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u8( ___p,(x).val[0]), vipmvi_storeu1x16_u8( ___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_u16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u16(___p,(x).val[0]), vipmvi_storeu1x16_u16(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_u32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u32(___p,(x).val[0]), vipmvi_storeu1x16_u32(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_f32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_f32(___p,(x).val[0]), vipmvi_storeu1x16_f32(___p + 16,(x).val[1]) ); } while (0)

#  define /* void */           vipmvi_store1x16x2_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x16_s8( ___p,(x).val[0]), vipmvi_store1x16_s8( ___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x16x2_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_s16(___p,(x).val[0]), vipmvi_store1x16_s16(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x16x2_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_s32(___p,(x).val[0]), vipmvi_store1x16_s32(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x16x2_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x16_u8( ___p,(x).val[0]), vipmvi_store1x16_u8( ___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x16x2_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_u16(___p,(x).val[0]), vipmvi_store1x16_u16(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x16x2_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_u32(___p,(x).val[0]), vipmvi_store1x16_u32(___p + 16,(x).val[1]) ); } while (0)
#  define /* void */           vipmvi_store1x16x2_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_f32(___p,(x).val[0]), vipmvi_store1x16_f32(___p + 16,(x).val[1]) ); } while (0)


		/*
		 * Transfer 16x3=48 bytes
		 * 3 Q registers
		 */

#  define /* bo_m128x3_s8_t  */vipmvi_loada1x16x3_s8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s8_t) {{ vipmvi_loada1x16_s8( ___p), vipmvi_loada1x16_s8( ___p + 16), vipmvi_loada1x16_s8( ___p + 32) }}; })
#  define /* bo_m128x3_s16_t */vipmvi_loada1x16x3_s16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s16_t){{ vipmvi_loada1x16_s16(___p), vipmvi_loada1x16_s16(___p + 16), vipmvi_loada1x16_s16(___p + 32) }}; })
#  define /* bo_m128x3_s32_t */vipmvi_loada1x16x3_s32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s32_t){{ vipmvi_loada1x16_s32(___p), vipmvi_loada1x16_s32(___p + 16), vipmvi_loada1x16_s32(___p + 32) }}; })
#  define /* bo_m128x3_u8_t  */vipmvi_loada1x16x3_u8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u8_t) {{ vipmvi_loada1x16_u8( ___p), vipmvi_loada1x16_u8( ___p + 16), vipmvi_loada1x16_u8( ___p + 32) }}; })
#  define /* bo_m128x3_u16_t */vipmvi_loada1x16x3_u16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u16_t){{ vipmvi_loada1x16_u16(___p), vipmvi_loada1x16_u16(___p + 16), vipmvi_loada1x16_u16(___p + 32) }}; })
#  define /* bo_m128x3_u32_t */vipmvi_loada1x16x3_u32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u32_t){{ vipmvi_loada1x16_u32(___p), vipmvi_loada1x16_u32(___p + 16), vipmvi_loada1x16_u32(___p + 32) }}; })
#  define /* bo_m128x3_f32_t */vipmvi_loada1x16x3_f32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_f32_t){{ vipmvi_loada1x16_f32(___p), vipmvi_loada1x16_f32(___p + 16), vipmvi_loada1x16_f32(___p + 32) }}; })

#  define /* bo_m128x3_s8_t  */vipmvi_loadu1x16x3_s8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s8_t) {{ vipmvi_loadu1x16_s8( ___p), vipmvi_loadu1x16_s8( ___p + 16), vipmvi_loadu1x16_s8( ___p + 32) }}; })
#  define /* bo_m128x3_s16_t */vipmvi_loadu1x16x3_s16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s16_t){{ vipmvi_loadu1x16_s16(___p), vipmvi_loadu1x16_s16(___p + 16), vipmvi_loadu1x16_s16(___p + 32) }}; })
#  define /* bo_m128x3_s32_t */vipmvi_loadu1x16x3_s32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s32_t){{ vipmvi_loadu1x16_s32(___p), vipmvi_loadu1x16_s32(___p + 16), vipmvi_loadu1x16_s32(___p + 32) }}; })
#  define /* bo_m128x3_u8_t  */vipmvi_loadu1x16x3_u8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u8_t) {{ vipmvi_loadu1x16_u8( ___p), vipmvi_loadu1x16_u8( ___p + 16), vipmvi_loadu1x16_u8( ___p + 32) }}; })
#  define /* bo_m128x3_u16_t */vipmvi_loadu1x16x3_u16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u16_t){{ vipmvi_loadu1x16_u16(___p), vipmvi_loadu1x16_u16(___p + 16), vipmvi_loadu1x16_u16(___p + 32) }}; })
#  define /* bo_m128x3_u32_t */vipmvi_loadu1x16x3_u32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u32_t){{ vipmvi_loadu1x16_u32(___p), vipmvi_loadu1x16_u32(___p + 16), vipmvi_loadu1x16_u32(___p + 32) }}; })
#  define /* bo_m128x3_f32_t */vipmvi_loadu1x16x3_f32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_f32_t){{ vipmvi_loadu1x16_f32(___p), vipmvi_loadu1x16_f32(___p + 16), vipmvi_loadu1x16_f32(___p + 32) }}; })

#  define /* bo_m128x3_s8_t  */vipmvi_load1x16x3_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s8_t) {{ vipmvi_load1x16_s8( ___p), vipmvi_load1x16_s8( ___p + 16), vipmvi_load1x16_s8( ___p + 32)}}; })
#  define /* bo_m128x3_s16_t */vipmvi_load1x16x3_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s16_t){{ vipmvi_load1x16_s16(___p), vipmvi_load1x16_s16(___p + 16), vipmvi_load1x16_s16(___p + 32)}}; })
#  define /* bo_m128x3_s32_t */vipmvi_load1x16x3_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_s32_t){{ vipmvi_load1x16_s32(___p), vipmvi_load1x16_s32(___p + 16), vipmvi_load1x16_s32(___p + 32)}}; })
#  define /* bo_m128x3_u8_t  */vipmvi_load1x16x3_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u8_t) {{ vipmvi_load1x16_u8( ___p), vipmvi_load1x16_u8( ___p + 16), vipmvi_load1x16_u8( ___p + 32)}}; })
#  define /* bo_m128x3_u16_t */vipmvi_load1x16x3_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u16_t){{ vipmvi_load1x16_u16(___p), vipmvi_load1x16_u16(___p + 16), vipmvi_load1x16_u16(___p + 32)}}; })
#  define /* bo_m128x3_u32_t */vipmvi_load1x16x3_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_u32_t){{ vipmvi_load1x16_u32(___p), vipmvi_load1x16_u32(___p + 16), vipmvi_load1x16_u32(___p + 32)}}; })
#  define /* bo_m128x3_f32_t */vipmvi_load1x16x3_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x3_f32_t){{ vipmvi_load1x16_f32(___p), vipmvi_load1x16_f32(___p + 16), vipmvi_load1x16_f32(___p + 32)}}; })

#  define /* void */           vipmvi_storea1x16x3_s8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x16_s8( ___p,(x).val[0]), vipmvi_storea1x16_s8( ___p + 16,(x).val[1]), vipmvi_storea1x16_s8( ___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x3_s16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_s16(___p,(x).val[0]), vipmvi_storea1x16_s16(___p + 16,(x).val[1]), vipmvi_storea1x16_s16(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x3_s32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_s32(___p,(x).val[0]), vipmvi_storea1x16_s32(___p + 16,(x).val[1]), vipmvi_storea1x16_s32(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x3_u8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x16_u8( ___p,(x).val[0]), vipmvi_storea1x16_u8( ___p + 16,(x).val[1]), vipmvi_storea1x16_u8( ___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x3_u16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_u16(___p,(x).val[0]), vipmvi_storea1x16_u16(___p + 16,(x).val[1]), vipmvi_storea1x16_u16(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x3_u32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_u32(___p,(x).val[0]), vipmvi_storea1x16_u32(___p + 16,(x).val[1]), vipmvi_storea1x16_u32(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x3_f32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_f32(___p,(x).val[0]), vipmvi_storea1x16_f32(___p + 16,(x).val[1]), vipmvi_storea1x16_f32(___p + 32,(x).val[2]) ); } while (0)

#  define /* void */           vipmvi_storeu1x16x3_s8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s8( ___p,(x).val[0]), vipmvi_storeu1x16_s8( ___p + 16,(x).val[1]), vipmvi_storeu1x16_s8( ___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_s16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s16(___p,(x).val[0]), vipmvi_storeu1x16_s16(___p + 16,(x).val[1]), vipmvi_storeu1x16_s16(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_s32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s32(___p,(x).val[0]), vipmvi_storeu1x16_s32(___p + 16,(x).val[1]), vipmvi_storeu1x16_s32(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_u8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u8( ___p,(x).val[0]), vipmvi_storeu1x16_u8( ___p + 16,(x).val[1]), vipmvi_storeu1x16_u8( ___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_u16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u16(___p,(x).val[0]), vipmvi_storeu1x16_u16(___p + 16,(x).val[1]), vipmvi_storeu1x16_u16(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_u32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u32(___p,(x).val[0]), vipmvi_storeu1x16_u32(___p + 16,(x).val[1]), vipmvi_storeu1x16_u32(___p + 32,(x).val[2]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_f32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_f32(___p,(x).val[0]), vipmvi_storeu1x16_f32(___p + 16,(x).val[1]), vipmvi_storeu1x16_f32(___p + 32,(x).val[2]) ); } while (0)

#  define /* void */           vipmvi_store1x16x3_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x16_s8( ___p,(x).val[0]), vipmvi_store1x16_s8( ___p + 16,(x).val[1]), vipmvi_store1x16_s8( ___p + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_s16(___p,(x).val[0]), vipmvi_store1x16_s16(___p + 16,(x).val[1]), vipmvi_store1x16_s16(___p + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_s32(___p,(x).val[0]), vipmvi_store1x16_s32(___p + 16,(x).val[1]), vipmvi_store1x16_s32(___p + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x16_u8( ___p,(x).val[0]), vipmvi_store1x16_u8( ___p + 16,(x).val[1]), vipmvi_store1x16_u8( ___p + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_u16(___p,(x).val[0]), vipmvi_store1x16_u16(___p + 16,(x).val[1]), vipmvi_store1x16_u16(___p + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_u32(___p,(x).val[0]), vipmvi_store1x16_u32(___p + 16,(x).val[1]), vipmvi_store1x16_u32(___p + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_f32(___p,(x).val[0]), vipmvi_store1x16_f32(___p + 16,(x).val[1]), vipmvi_store1x16_f32(___p + 32,(x).val[2])); } while (0)


		/*
		 * Transfer 16x4=64 bytes
		 * 4 Q registers
		 */

#  define /* bo_m128x4_s8_t  */vipmvi_loada1x16x4_s8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s8_t) {{ vipmvi_loada1x16_s8( ___p), vipmvi_loada1x16_s8( ___p + 16), vipmvi_loada1x16_s8( ___p + 32), vipmvi_loada1x16_s8( ___p + 48) }}; })
#  define /* bo_m128x4_s16_t */vipmvi_loada1x16x4_s16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s16_t){{ vipmvi_loada1x16_s16(___p), vipmvi_loada1x16_s16(___p + 16), vipmvi_loada1x16_s16(___p + 32), vipmvi_loada1x16_s16(___p + 48) }}; })
#  define /* bo_m128x4_s32_t */vipmvi_loada1x16x4_s32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s32_t){{ vipmvi_loada1x16_s32(___p), vipmvi_loada1x16_s32(___p + 16), vipmvi_loada1x16_s32(___p + 32), vipmvi_loada1x16_s32(___p + 48) }}; })
#  define /* bo_m128x4_u8_t  */vipmvi_loada1x16x4_u8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u8_t) {{ vipmvi_loada1x16_u8( ___p), vipmvi_loada1x16_u8( ___p + 16), vipmvi_loada1x16_u8( ___p + 32), vipmvi_loada1x16_u8( ___p + 48) }}; })
#  define /* bo_m128x4_u16_t */vipmvi_loada1x16x4_u16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u16_t){{ vipmvi_loada1x16_u16(___p), vipmvi_loada1x16_u16(___p + 16), vipmvi_loada1x16_u16(___p + 32), vipmvi_loada1x16_u16(___p + 48) }}; })
#  define /* bo_m128x4_u32_t */vipmvi_loada1x16x4_u32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u32_t){{ vipmvi_loada1x16_u32(___p), vipmvi_loada1x16_u32(___p + 16), vipmvi_loada1x16_u32(___p + 32), vipmvi_loada1x16_u32(___p + 48) }}; })
#  define /* bo_m128x4_f32_t */vipmvi_loada1x16x4_f32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_f32_t){{ vipmvi_loada1x16_f32(___p), vipmvi_loada1x16_f32(___p + 16), vipmvi_loada1x16_f32(___p + 32), vipmvi_loada1x16_f32(___p + 48) }}; })

#  define /* bo_m128x4_s8_t  */vipmvi_loadu1x16x4_s8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s8_t) {{ vipmvi_loadu1x16_s8( ___p), vipmvi_loadu1x16_s8( ___p + 16), vipmvi_loadu1x16_s8( ___p + 32), vipmvi_loadu1x16_s8( ___p + 48) }}; })
#  define /* bo_m128x4_s16_t */vipmvi_loadu1x16x4_s16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s16_t){{ vipmvi_loadu1x16_s16(___p), vipmvi_loadu1x16_s16(___p + 16), vipmvi_loadu1x16_s16(___p + 32), vipmvi_loadu1x16_s16(___p + 48) }}; })
#  define /* bo_m128x4_s32_t */vipmvi_loadu1x16x4_s32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s32_t){{ vipmvi_loadu1x16_s32(___p), vipmvi_loadu1x16_s32(___p + 16), vipmvi_loadu1x16_s32(___p + 32), vipmvi_loadu1x16_s32(___p + 48) }}; })
#  define /* bo_m128x4_u8_t  */vipmvi_loadu1x16x4_u8(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u8_t) {{ vipmvi_loadu1x16_u8( ___p), vipmvi_loadu1x16_u8( ___p + 16), vipmvi_loadu1x16_u8( ___p + 32), vipmvi_loadu1x16_u8( ___p + 48) }}; })
#  define /* bo_m128x4_u16_t */vipmvi_loadu1x16x4_u16(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u16_t){{ vipmvi_loadu1x16_u16(___p), vipmvi_loadu1x16_u16(___p + 16), vipmvi_loadu1x16_u16(___p + 32), vipmvi_loadu1x16_u16(___p + 48) }}; })
#  define /* bo_m128x4_u32_t */vipmvi_loadu1x16x4_u32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u32_t){{ vipmvi_loadu1x16_u32(___p), vipmvi_loadu1x16_u32(___p + 16), vipmvi_loadu1x16_u32(___p + 32), vipmvi_loadu1x16_u32(___p + 48) }}; })
#  define /* bo_m128x4_f32_t */vipmvi_loadu1x16x4_f32(p)        __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_f32_t){{ vipmvi_loadu1x16_f32(___p), vipmvi_loadu1x16_f32(___p + 16), vipmvi_loadu1x16_f32(___p + 32), vipmvi_loadu1x16_f32(___p + 48) }}; })

#  define /* bo_m128x4_s8_t  */vipmvi_load1x16x4_s8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s8_t) {{ vipmvi_load1x16_s8( ___p), vipmvi_load1x16_s8( ___p + 16), vipmvi_load1x16_s8( ___p + 32), vipmvi_load1x16_s8( ___p + 48) }}; })
#  define /* bo_m128x4_s16_t */vipmvi_load1x16x4_s16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s16_t){{ vipmvi_load1x16_s16(___p), vipmvi_load1x16_s16(___p + 16), vipmvi_load1x16_s16(___p + 32), vipmvi_load1x16_s16(___p + 48) }}; })
#  define /* bo_m128x4_s32_t */vipmvi_load1x16x4_s32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_s32_t){{ vipmvi_load1x16_s32(___p), vipmvi_load1x16_s32(___p + 16), vipmvi_load1x16_s32(___p + 32), vipmvi_load1x16_s32(___p + 48) }}; })
#  define /* bo_m128x4_u8_t  */vipmvi_load1x16x4_u8(p)          __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u8_t) {{ vipmvi_load1x16_u8( ___p), vipmvi_load1x16_u8( ___p + 16), vipmvi_load1x16_u8( ___p + 32), vipmvi_load1x16_u8( ___p + 48) }}; })
#  define /* bo_m128x4_u16_t */vipmvi_load1x16x4_u16(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u16_t){{ vipmvi_load1x16_u16(___p), vipmvi_load1x16_u16(___p + 16), vipmvi_load1x16_u16(___p + 32), vipmvi_load1x16_u16(___p + 48) }}; })
#  define /* bo_m128x4_u32_t */vipmvi_load1x16x4_u32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_u32_t){{ vipmvi_load1x16_u32(___p), vipmvi_load1x16_u32(___p + 16), vipmvi_load1x16_u32(___p + 32), vipmvi_load1x16_u32(___p + 48) }}; })
#  define /* bo_m128x4_f32_t */vipmvi_load1x16x4_f32(p)         __extension__ ({ char const *___p = (char const *)(p); (bo_m128x4_f32_t){{ vipmvi_load1x16_f32(___p), vipmvi_load1x16_f32(___p + 16), vipmvi_load1x16_f32(___p + 32), vipmvi_load1x16_f32(___p + 48) }}; })

#  define /* void */           vipmvi_storea1x16x4_s8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x16_s8( ___p,(x).val[0]), vipmvi_storea1x16_s8( ___p + 16,(x).val[1]), vipmvi_storea1x16_s8( ___p + 32,(x).val[2]), vipmvi_storea1x16_s8( ___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x4_s16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_s16(___p,(x).val[0]), vipmvi_storea1x16_s16(___p + 16,(x).val[1]), vipmvi_storea1x16_s16(___p + 32,(x).val[2]), vipmvi_storea1x16_s16(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x4_s32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_s32(___p,(x).val[0]), vipmvi_storea1x16_s32(___p + 16,(x).val[1]), vipmvi_storea1x16_s32(___p + 32,(x).val[2]), vipmvi_storea1x16_s32(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x4_u8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storea1x16_u8( ___p,(x).val[0]), vipmvi_storea1x16_u8( ___p + 16,(x).val[1]), vipmvi_storea1x16_u8( ___p + 32,(x).val[2]), vipmvi_storea1x16_u8( ___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x4_u16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_u16(___p,(x).val[0]), vipmvi_storea1x16_u16(___p + 16,(x).val[1]), vipmvi_storea1x16_u16(___p + 32,(x).val[2]), vipmvi_storea1x16_u16(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x4_u32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_u32(___p,(x).val[0]), vipmvi_storea1x16_u32(___p + 16,(x).val[1]), vipmvi_storea1x16_u32(___p + 32,(x).val[2]), vipmvi_storea1x16_u32(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storea1x16x4_f32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storea1x16_f32(___p,(x).val[0]), vipmvi_storea1x16_f32(___p + 16,(x).val[1]), vipmvi_storea1x16_f32(___p + 32,(x).val[2]), vipmvi_storea1x16_f32(___p + 48,(x).val[3]) ); } while (0)

#  define /* void */           vipmvi_storeu1x16x4_s8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s8( ___p,(x).val[0]), vipmvi_storeu1x16_s8( ___p + 16,(x).val[1]), vipmvi_storeu1x16_s8( ___p + 32,(x).val[2]), vipmvi_storeu1x16_s8( ___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_s16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s16(___p,(x).val[0]), vipmvi_storeu1x16_s16(___p + 16,(x).val[1]), vipmvi_storeu1x16_s16(___p + 32,(x).val[2]), vipmvi_storeu1x16_s16(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_s32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_s32(___p,(x).val[0]), vipmvi_storeu1x16_s32(___p + 16,(x).val[1]), vipmvi_storeu1x16_s32(___p + 32,(x).val[2]), vipmvi_storeu1x16_s32(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_u8(p,x)      do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u8( ___p,(x).val[0]), vipmvi_storeu1x16_u8( ___p + 16,(x).val[1]), vipmvi_storeu1x16_u8( ___p + 32,(x).val[2]), vipmvi_storeu1x16_u8( ___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_u16(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u16(___p,(x).val[0]), vipmvi_storeu1x16_u16(___p + 16,(x).val[1]), vipmvi_storeu1x16_u16(___p + 32,(x).val[2]), vipmvi_storeu1x16_u16(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_u32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_u32(___p,(x).val[0]), vipmvi_storeu1x16_u32(___p + 16,(x).val[1]), vipmvi_storeu1x16_u32(___p + 32,(x).val[2]), vipmvi_storeu1x16_u32(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_f32(p,x)     do { char *___p = (char *)(p); ( vipmvi_storeu1x16_f32(___p,(x).val[0]), vipmvi_storeu1x16_f32(___p + 16,(x).val[1]), vipmvi_storeu1x16_f32(___p + 32,(x).val[2]), vipmvi_storeu1x16_f32(___p + 48,(x).val[3]) ); } while (0)

#  define /* void */           vipmvi_store1x16x4_s8(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x16_s8( ___p,(x).val[0]), vipmvi_store1x16_s8( ___p + 16,(x).val[1]), vipmvi_store1x16_s8( ___p + 32,(x).val[2]), vipmvi_store1x16_s8( ___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x16x4_s16(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_s16(___p,(x).val[0]), vipmvi_store1x16_s16(___p + 16,(x).val[1]), vipmvi_store1x16_s16(___p + 32,(x).val[2]), vipmvi_store1x16_s16(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x16x4_s32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_s32(___p,(x).val[0]), vipmvi_store1x16_s32(___p + 16,(x).val[1]), vipmvi_store1x16_s32(___p + 32,(x).val[2]), vipmvi_store1x16_s32(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x16x4_u8(p,x)       do { char *___p = (char *)(p); ( vipmvi_store1x16_u8( ___p,(x).val[0]), vipmvi_store1x16_u8( ___p + 16,(x).val[1]), vipmvi_store1x16_u8( ___p + 32,(x).val[2]), vipmvi_store1x16_u8( ___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x16x4_u16(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_u16(___p,(x).val[0]), vipmvi_store1x16_u16(___p + 16,(x).val[1]), vipmvi_store1x16_u16(___p + 32,(x).val[2]), vipmvi_store1x16_u16(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x16x4_u32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_u32(___p,(x).val[0]), vipmvi_store1x16_u32(___p + 16,(x).val[1]), vipmvi_store1x16_u32(___p + 32,(x).val[2]), vipmvi_store1x16_u32(___p + 48,(x).val[3]) ); } while (0)
#  define /* void */           vipmvi_store1x16x4_f32(p,x)      do { char *___p = (char *)(p); ( vipmvi_store1x16_f32(___p,(x).val[0]), vipmvi_store1x16_f32(___p + 16,(x).val[1]), vipmvi_store1x16_f32(___p + 32,(x).val[2]), vipmvi_store1x16_f32(___p + 48,(x).val[3]) ); } while (0)

# endif /* supported _xn versions */

	/*
	 * Load/Store 1-element structures
	 * @}
	 */


	/*
	 * Load/Store 2-element structures
	 * @{
	 */

		/*
		 * Transfer 2x8=16 bytes
		 * 2 D registers
		 */

# define /* bo_m64x2_s8_t  */  vipmvi_load2x8_s8(p)             vld2_s8( (void const *)(p))
# define /* bo_m64x2_s16_t */  vipmvi_load2x8_s16(p)            vld2_s16((void const *)(p))
# define /* bo_m64x2_s32_t */  vipmvi_load2x8_s32(p)            vld2_s32((void const *)(p))
# define /* bo_m64x2_u8_t  */  vipmvi_load2x8_u8(p)             vld2_u8( (void const *)(p))
# define /* bo_m64x2_u16_t */  vipmvi_load2x8_u16(p)            vld2_u16((void const *)(p))
# define /* bo_m64x2_u32_t */  vipmvi_load2x8_u32(p)            vld2_u32((void const *)(p))
# define /* bo_m64x2_f32_t */  vipmvi_load2x8_f32(p)            vld2_f32((void const *)(p))

# define /* bo_m64x2_s8_t  */  vipmvi_loada2x8_s8(p)            vipmvi_load2x8_s8(p)
# define /* bo_m64x2_s16_t */  vipmvi_loada2x8_s16(p)           vipmvi_load2x8_s16(p)
# define /* bo_m64x2_s32_t */  vipmvi_loada2x8_s32(p)           vipmvi_load2x8_s32(p)
# define /* bo_m64x2_u8_t  */  vipmvi_loada2x8_u8(p)            vipmvi_load2x8_u8(p)
# define /* bo_m64x2_u16_t */  vipmvi_loada2x8_u16(p)           vipmvi_load2x8_u16(p)
# define /* bo_m64x2_u32_t */  vipmvi_loada2x8_u32(p)           vipmvi_load2x8_u32(p)
# define /* bo_m64x2_f32_t */  vipmvi_loada2x8_f32(p)           vipmvi_load2x8_f32(p)

# define /* bo_m64x2_s8_t  */  vipmvi_loadu2x8_s8(p)            vipmvi_load2x8_s8(p)
# define /* bo_m64x2_s16_t */  vipmvi_loadu2x8_s16(p)           vipmvi_load2x8_s16(p)
# define /* bo_m64x2_s32_t */  vipmvi_loadu2x8_s32(p)           vipmvi_load2x8_s32(p)
# define /* bo_m64x2_u8_t  */  vipmvi_loadu2x8_u8(p)            vipmvi_load2x8_u8(p)
# define /* bo_m64x2_u16_t */  vipmvi_loadu2x8_u16(p)           vipmvi_load2x8_u16(p)
# define /* bo_m64x2_u32_t */  vipmvi_loadu2x8_u32(p)           vipmvi_load2x8_u32(p)
# define /* bo_m64x2_f32_t */  vipmvi_loadu2x8_f32(p)           vipmvi_load2x8_f32(p)

# define /* void */            vipmvi_store2x8_s8(p,x)          vst2_s8( (void *)(p),x)
# define /* void */            vipmvi_store2x8_s16(p,x)         vst2_s16((void *)(p),x)
# define /* void */            vipmvi_store2x8_s32(p,x)         vst2_s32((void *)(p),x)
# define /* void */            vipmvi_store2x8_u8(p,x)          vst2_u8( (void *)(p),x)
# define /* void */            vipmvi_store2x8_u16(p,x)         vst2_u16((void *)(p),x)
# define /* void */            vipmvi_store2x8_u32(p,x)         vst2_u32((void *)(p),x)
# define /* void */            vipmvi_store2x8_f32(p,x)         vst2_f32((void *)(p),x)

# define /* void */            vipmvi_storea2x8_s8(p,x)         vipmvi_store2x8_s8( p,x)
# define /* void */            vipmvi_storea2x8_s16(p,x)        vipmvi_store2x8_s16(p,x)
# define /* void */            vipmvi_storea2x8_s32(p,x)        vipmvi_store2x8_s32(p,x)
# define /* void */            vipmvi_storea2x8_u8(p,x)         vipmvi_store2x8_u8( p,x)
# define /* void */            vipmvi_storea2x8_u16(p,x)        vipmvi_store2x8_u16(p,x)
# define /* void */            vipmvi_storea2x8_u32(p,x)        vipmvi_store2x8_u32(p,x)
# define /* void */            vipmvi_storea2x8_f32(p,x)        vipmvi_store2x8_f32(p,x)

# define /* void */            vipmvi_storeu2x8_s8(p,x)         vipmvi_store2x8_s8( p,x)
# define /* void */            vipmvi_storeu2x8_s16(p,x)        vipmvi_store2x8_s16(p,x)
# define /* void */            vipmvi_storeu2x8_s32(p,x)        vipmvi_store2x8_s32(p,x)
# define /* void */            vipmvi_storeu2x8_u8(p,x)         vipmvi_store2x8_u8( p,x)
# define /* void */            vipmvi_storeu2x8_u16(p,x)        vipmvi_store2x8_u16(p,x)
# define /* void */            vipmvi_storeu2x8_u32(p,x)        vipmvi_store2x8_u32(p,x)
# define /* void */            vipmvi_storeu2x8_f32(p,x)        vipmvi_store2x8_f32(p,x)


		/*
		 * Transfer 2x16=32 bytes
		 * 2 Q registers
		 */

# define /* bo_m128x2_s8_t  */ vipmvi_load2x16_s8(p)            vld2q_s8( (void const *)(p))
# define /* bo_m128x2_s16_t */ vipmvi_load2x16_s16(p)           vld2q_s16((void const *)(p))
# define /* bo_m128x2_s32_t */ vipmvi_load2x16_s32(p)           vld2q_s32((void const *)(p))
# define /* bo_m128x2_u8_t  */ vipmvi_load2x16_u8(p)            vld2q_u8( (void const *)(p))
# define /* bo_m128x2_u16_t */ vipmvi_load2x16_u16(p)           vld2q_u16((void const *)(p))
# define /* bo_m128x2_u32_t */ vipmvi_load2x16_u32(p)           vld2q_u32((void const *)(p))
# define /* bo_m128x2_f32_t */ vipmvi_load2x16_f32(p)           vld2q_f32((void const *)(p))

# define /* bo_m128x2_s8_t  */ vipmvi_loada2x16_s8(p)           vipmvi_load2x16_s8(p)
# define /* bo_m128x2_s16_t */ vipmvi_loada2x16_s16(p)          vipmvi_load2x16_s16(p)
# define /* bo_m128x2_s32_t */ vipmvi_loada2x16_s32(p)          vipmvi_load2x16_s32(p)
# define /* bo_m128x2_u8_t  */ vipmvi_loada2x16_u8(p)           vipmvi_load2x16_u8(p)
# define /* bo_m128x2_u16_t */ vipmvi_loada2x16_u16(p)          vipmvi_load2x16_u16(p)
# define /* bo_m128x2_u32_t */ vipmvi_loada2x16_u32(p)          vipmvi_load2x16_u32(p)
# define /* bo_m128x2_f32_t */ vipmvi_loada2x16_f32(p)          vipmvi_load2x16_f32(p)

# define /* bo_m128x2_s8_t  */ vipmvi_loadu2x16_s8(p)           vipmvi_load2x16_s8(p)
# define /* bo_m128x2_s16_t */ vipmvi_loadu2x16_s16(p)          vipmvi_load2x16_s16(p)
# define /* bo_m128x2_s32_t */ vipmvi_loadu2x16_s32(p)          vipmvi_load2x16_s32(p)
# define /* bo_m128x2_u8_t  */ vipmvi_loadu2x16_u8(p)           vipmvi_load2x16_u8(p)
# define /* bo_m128x2_u16_t */ vipmvi_loadu2x16_u16(p)          vipmvi_load2x16_u16(p)
# define /* bo_m128x2_u32_t */ vipmvi_loadu2x16_u32(p)          vipmvi_load2x16_u32(p)
# define /* bo_m128x2_f32_t */ vipmvi_loadu2x16_f32(p)          vipmvi_load2x16_f32(p)

# define /* void */            vipmvi_store2x16_s8(p,x)         vst2q_s8( (void *)(p),x)
# define /* void */            vipmvi_store2x16_s16(p,x)        vst2q_s16((void *)(p),x)
# define /* void */            vipmvi_store2x16_s32(p,x)        vst2q_s32((void *)(p),x)
# define /* void */            vipmvi_store2x16_u8(p,x)         vst2q_u8( (void *)(p),x)
# define /* void */            vipmvi_store2x16_u16(p,x)        vst2q_u16((void *)(p),x)
# define /* void */            vipmvi_store2x16_u32(p,x)        vst2q_u32((void *)(p),x)
# define /* void */            vipmvi_store2x16_f32(p,x)        vst2q_f32((void *)(p),x)

# define /* void */            vipmvi_storea2x16_s8(p,x)        vipmvi_store2x16_s8( p,x)
# define /* void */            vipmvi_storea2x16_s16(p,x)       vipmvi_store2x16_s16(p,x)
# define /* void */            vipmvi_storea2x16_s32(p,x)       vipmvi_store2x16_s32(p,x)
# define /* void */            vipmvi_storea2x16_u8(p,x)        vipmvi_store2x16_u8( p,x)
# define /* void */            vipmvi_storea2x16_u16(p,x)       vipmvi_store2x16_u16(p,x)
# define /* void */            vipmvi_storea2x16_u32(p,x)       vipmvi_store2x16_u32(p,x)
# define /* void */            vipmvi_storea2x16_f32(p,x)       vipmvi_store2x16_f32(p,x)

# define /* void */            vipmvi_storeu2x16_s8(p,x)        vipmvi_store2x16_s8( p,x)
# define /* void */            vipmvi_storeu2x16_s16(p,x)       vipmvi_store2x16_s16(p,x)
# define /* void */            vipmvi_storeu2x16_s32(p,x)       vipmvi_store2x16_s32(p,x)
# define /* void */            vipmvi_storeu2x16_u8(p,x)        vipmvi_store2x16_u8( p,x)
# define /* void */            vipmvi_storeu2x16_u16(p,x)       vipmvi_store2x16_u16(p,x)
# define /* void */            vipmvi_storeu2x16_u32(p,x)       vipmvi_store2x16_u32(p,x)
# define /* void */            vipmvi_storeu2x16_f32(p,x)       vipmvi_store2x16_f32(p,x)

	/*
	 * Load/Store 2-element structures
	 * @}
	 */


	/*
	 * Load/Store 3-element structures
	 * @{
	 */

		/*
		 * Transfer 3x8=24 bytes
		 * 3 D registers
		 */

# define /* bo_m64x3_s8_t  */  vipmvi_load3x8_s8(p)             vld3_s8( (void const *)(p))
# define /* bo_m64x3_s16_t */  vipmvi_load3x8_s16(p)            vld3_s16((void const *)(p))
# define /* bo_m64x3_s32_t */  vipmvi_load3x8_s32(p)            vld3_s32((void const *)(p))
# define /* bo_m64x3_u8_t  */  vipmvi_load3x8_u8(p)             vld3_u8( (void const *)(p))
# define /* bo_m64x3_u16_t */  vipmvi_load3x8_u16(p)            vld3_u16((void const *)(p))
# define /* bo_m64x3_u32_t */  vipmvi_load3x8_u32(p)            vld3_u32((void const *)(p))
# define /* bo_m64x3_f32_t */  vipmvi_load3x8_f32(p)            vld3_f32((void const *)(p))

# define /* bo_m64x3_s8_t  */  vipmvi_loada3x8_s8(p)            vipmvi_load3x8_s8(p)
# define /* bo_m64x3_s16_t */  vipmvi_loada3x8_s16(p)           vipmvi_load3x8_s16(p)
# define /* bo_m64x3_s32_t */  vipmvi_loada3x8_s32(p)           vipmvi_load3x8_s32(p)
# define /* bo_m64x3_u8_t  */  vipmvi_loada3x8_u8(p)            vipmvi_load3x8_u8(p)
# define /* bo_m64x3_u16_t */  vipmvi_loada3x8_u16(p)           vipmvi_load3x8_u16(p)
# define /* bo_m64x3_u32_t */  vipmvi_loada3x8_u32(p)           vipmvi_load3x8_u32(p)
# define /* bo_m64x3_f32_t */  vipmvi_loada3x8_f32(p)           vipmvi_load3x8_f32(p)

# define /* bo_m64x3_s8_t  */  vipmvi_loadu3x8_s8(p)            vipmvi_load3x8_s8(p)
# define /* bo_m64x3_s16_t */  vipmvi_loadu3x8_s16(p)           vipmvi_load3x8_s16(p)
# define /* bo_m64x3_s32_t */  vipmvi_loadu3x8_s32(p)           vipmvi_load3x8_s32(p)
# define /* bo_m64x3_u8_t  */  vipmvi_loadu3x8_u8(p)            vipmvi_load3x8_u8(p)
# define /* bo_m64x3_u16_t */  vipmvi_loadu3x8_u16(p)           vipmvi_load3x8_u16(p)
# define /* bo_m64x3_u32_t */  vipmvi_loadu3x8_u32(p)           vipmvi_load3x8_u32(p)
# define /* bo_m64x3_f32_t */  vipmvi_loadu3x8_f32(p)           vipmvi_load3x8_f32(p)

# define /* void */            vipmvi_store3x8_s8(p,x)          vst3_s8( (void *)(p),x)
# define /* void */            vipmvi_store3x8_s16(p,x)         vst3_s16((void *)(p),x)
# define /* void */            vipmvi_store3x8_s32(p,x)         vst3_s32((void *)(p),x)
# define /* void */            vipmvi_store3x8_u8(p,x)          vst3_u8( (void *)(p),x)
# define /* void */            vipmvi_store3x8_u16(p,x)         vst3_u16((void *)(p),x)
# define /* void */            vipmvi_store3x8_u32(p,x)         vst3_u32((void *)(p),x)
# define /* void */            vipmvi_store3x8_f32(p,x)         vst3_f32((void *)(p),x)

# define /* void */            vipmvi_storea3x8_s8(p,x)         vipmvi_store3x8_s8( p,x)
# define /* void */            vipmvi_storea3x8_s16(p,x)        vipmvi_store3x8_s16(p,x)
# define /* void */            vipmvi_storea3x8_s32(p,x)        vipmvi_store3x8_s32(p,x)
# define /* void */            vipmvi_storea3x8_u8(p,x)         vipmvi_store3x8_u8( p,x)
# define /* void */            vipmvi_storea3x8_u16(p,x)        vipmvi_store3x8_u16(p,x)
# define /* void */            vipmvi_storea3x8_u32(p,x)        vipmvi_store3x8_u32(p,x)
# define /* void */            vipmvi_storea3x8_f32(p,x)        vipmvi_store3x8_f32(p,x)

# define /* void */            vipmvi_storeu3x8_s8(p,x)         vipmvi_store3x8_s8( p,x)
# define /* void */            vipmvi_storeu3x8_s16(p,x)        vipmvi_store3x8_s16(p,x)
# define /* void */            vipmvi_storeu3x8_s32(p,x)        vipmvi_store3x8_s32(p,x)
# define /* void */            vipmvi_storeu3x8_u8(p,x)         vipmvi_store3x8_u8( p,x)
# define /* void */            vipmvi_storeu3x8_u16(p,x)        vipmvi_store3x8_u16(p,x)
# define /* void */            vipmvi_storeu3x8_u32(p,x)        vipmvi_store3x8_u32(p,x)
# define /* void */            vipmvi_storeu3x8_f32(p,x)        vipmvi_store3x8_f32(p,x)


		/*
		 * Transfer 3x16=48 bytes
		 * 3 Q registers
		 */

# define /* bo_m128x3_s8_t  */ vipmvi_load3x16_s8(p)            vld3q_s8( (void const *)(p))
# define /* bo_m128x3_s16_t */ vipmvi_load3x16_s16(p)           vld3q_s16((void const *)(p))
# define /* bo_m128x3_s32_t */ vipmvi_load3x16_s32(p)           vld3q_s32((void const *)(p))
# define /* bo_m128x3_u8_t  */ vipmvi_load3x16_u8(p)            vld3q_u8( (void const *)(p))
# define /* bo_m128x3_u16_t */ vipmvi_load3x16_u16(p)           vld3q_u16((void const *)(p))
# define /* bo_m128x3_u32_t */ vipmvi_load3x16_u32(p)           vld3q_u32((void const *)(p))
# define /* bo_m128x3_f32_t */ vipmvi_load3x16_f32(p)           vld3q_f32((void const *)(p))

# define /* bo_m128x3_s8_t  */ vipmvi_loada3x16_s8(p)           vipmvi_load3x16_s8(p)
# define /* bo_m128x3_s16_t */ vipmvi_loada3x16_s16(p)          vipmvi_load3x16_s16(p)
# define /* bo_m128x3_s32_t */ vipmvi_loada3x16_s32(p)          vipmvi_load3x16_s32(p)
# define /* bo_m128x3_u8_t  */ vipmvi_loada3x16_u8(p)           vipmvi_load3x16_u8(p)
# define /* bo_m128x3_u16_t */ vipmvi_loada3x16_u16(p)          vipmvi_load3x16_u16(p)
# define /* bo_m128x3_u32_t */ vipmvi_loada3x16_u32(p)          vipmvi_load3x16_u32(p)
# define /* bo_m128x3_f32_t */ vipmvi_loada3x16_f32(p)          vipmvi_load3x16_f32(p)

# define /* bo_m128x3_s8_t  */ vipmvi_loadu3x16_s8(p)           vipmvi_load3x16_s8(p)
# define /* bo_m128x3_s16_t */ vipmvi_loadu3x16_s16(p)          vipmvi_load3x16_s16(p)
# define /* bo_m128x3_s32_t */ vipmvi_loadu3x16_s32(p)          vipmvi_load3x16_s32(p)
# define /* bo_m128x3_u8_t  */ vipmvi_loadu3x16_u8(p)           vipmvi_load3x16_u8(p)
# define /* bo_m128x3_u16_t */ vipmvi_loadu3x16_u16(p)          vipmvi_load3x16_u16(p)
# define /* bo_m128x3_u32_t */ vipmvi_loadu3x16_u32(p)          vipmvi_load3x16_u32(p)
# define /* bo_m128x3_f32_t */ vipmvi_loadu3x16_f32(p)          vipmvi_load3x16_f32(p)

# define /* void */            vipmvi_store3x16_s8(p,x)         vst3q_s8( (void *)(p),x)
# define /* void */            vipmvi_store3x16_s16(p,x)        vst3q_s16((void *)(p),x)
# define /* void */            vipmvi_store3x16_s32(p,x)        vst3q_s32((void *)(p),x)
# define /* void */            vipmvi_store3x16_u8(p,x)         vst3q_u8( (void *)(p),x)
# define /* void */            vipmvi_store3x16_u16(p,x)        vst3q_u16((void *)(p),x)
# define /* void */            vipmvi_store3x16_u32(p,x)        vst3q_u32((void *)(p),x)
# define /* void */            vipmvi_store3x16_f32(p,x)        vst3q_f32((void *)(p),x)

# define /* void */            vipmvi_storea3x16_s8(p,x)        vipmvi_store3x16_s8( p,x)
# define /* void */            vipmvi_storea3x16_s16(p,x)       vipmvi_store3x16_s16(p,x)
# define /* void */            vipmvi_storea3x16_s32(p,x)       vipmvi_store3x16_s32(p,x)
# define /* void */            vipmvi_storea3x16_u8(p,x)        vipmvi_store3x16_u8( p,x)
# define /* void */            vipmvi_storea3x16_u16(p,x)       vipmvi_store3x16_u16(p,x)
# define /* void */            vipmvi_storea3x16_u32(p,x)       vipmvi_store3x16_u32(p,x)
# define /* void */            vipmvi_storea3x16_f32(p,x)       vipmvi_store3x16_f32(p,x)

# define /* void */            vipmvi_storeu3x16_s8(p,x)        vipmvi_store3x16_s8( p,x)
# define /* void */            vipmvi_storeu3x16_s16(p,x)       vipmvi_store3x16_s16(p,x)
# define /* void */            vipmvi_storeu3x16_s32(p,x)       vipmvi_store3x16_s32(p,x)
# define /* void */            vipmvi_storeu3x16_u8(p,x)        vipmvi_store3x16_u8( p,x)
# define /* void */            vipmvi_storeu3x16_u16(p,x)       vipmvi_store3x16_u16(p,x)
# define /* void */            vipmvi_storeu3x16_u32(p,x)       vipmvi_store3x16_u32(p,x)
# define /* void */            vipmvi_storeu3x16_f32(p,x)       vipmvi_store3x16_f32(p,x)

	/*
	 * Load/Store 3-element structures
	 * @}
	 */


	/*
	 * Load/Store 4-element structures
	 * @{
	 */

		/*
		 * Transfer 4x8=32 bytes
		 * 4 D registers
		 */

# define /* bo_m64x4_s8_t  */  vipmvi_load4x8_s8(p)             vld4_s8( (void const *)(p))
# define /* bo_m64x4_s16_t */  vipmvi_load4x8_s16(p)            vld4_s16((void const *)(p))
# define /* bo_m64x4_s32_t */  vipmvi_load4x8_s32(p)            vld4_s32((void const *)(p))
# define /* bo_m64x4_u8_t  */  vipmvi_load4x8_u8(p)             vld4_u8( (void const *)(p))
# define /* bo_m64x4_u16_t */  vipmvi_load4x8_u16(p)            vld4_u16((void const *)(p))
# define /* bo_m64x4_u32_t */  vipmvi_load4x8_u32(p)            vld4_u32((void const *)(p))
# define /* bo_m64x4_f32_t */  vipmvi_load4x8_f32(p)            vld4_f32((void const *)(p))

# define /* bo_m64x4_s8_t  */  vipmvi_loada4x8_s8(p)            vipmvi_load4x8_s8(p)
# define /* bo_m64x4_s16_t */  vipmvi_loada4x8_s16(p)           vipmvi_load4x8_s16(p)
# define /* bo_m64x4_s32_t */  vipmvi_loada4x8_s32(p)           vipmvi_load4x8_s32(p)
# define /* bo_m64x4_u8_t  */  vipmvi_loada4x8_u8(p)            vipmvi_load4x8_u8(p)
# define /* bo_m64x4_u16_t */  vipmvi_loada4x8_u16(p)           vipmvi_load4x8_u16(p)
# define /* bo_m64x4_u32_t */  vipmvi_loada4x8_u32(p)           vipmvi_load4x8_u32(p)
# define /* bo_m64x4_f32_t */  vipmvi_loada4x8_f32(p)           vipmvi_load4x8_f32(p)

# define /* bo_m64x4_s8_t  */  vipmvi_loadu4x8_s8(p)            vipmvi_load4x8_s8(p)
# define /* bo_m64x4_s16_t */  vipmvi_loadu4x8_s16(p)           vipmvi_load4x8_s16(p)
# define /* bo_m64x4_s32_t */  vipmvi_loadu4x8_s32(p)           vipmvi_load4x8_s32(p)
# define /* bo_m64x4_u8_t  */  vipmvi_loadu4x8_u8(p)            vipmvi_load4x8_u8(p)
# define /* bo_m64x4_u16_t */  vipmvi_loadu4x8_u16(p)           vipmvi_load4x8_u16(p)
# define /* bo_m64x4_u32_t */  vipmvi_loadu4x8_u32(p)           vipmvi_load4x8_u32(p)
# define /* bo_m64x4_f32_t */  vipmvi_loadu4x8_f32(p)           vipmvi_load4x8_f32(p)

# define /* void */            vipmvi_store4x8_s8(p,x)          vst4_s8( (void *)(p),x)
# define /* void */            vipmvi_store4x8_s16(p,x)         vst4_s16((void *)(p),x)
# define /* void */            vipmvi_store4x8_s32(p,x)         vst4_s32((void *)(p),x)
# define /* void */            vipmvi_store4x8_u8(p,x)          vst4_u8( (void *)(p),x)
# define /* void */            vipmvi_store4x8_u16(p,x)         vst4_u16((void *)(p),x)
# define /* void */            vipmvi_store4x8_u32(p,x)         vst4_u32((void *)(p),x)
# define /* void */            vipmvi_store4x8_f32(p,x)         vst4_f32((void *)(p),x)

# define /* void */            vipmvi_storea4x8_s8(p,x)         vipmvi_store4x8_s8( p,x)
# define /* void */            vipmvi_storea4x8_s16(p,x)        vipmvi_store4x8_s16(p,x)
# define /* void */            vipmvi_storea4x8_s32(p,x)        vipmvi_store4x8_s32(p,x)
# define /* void */            vipmvi_storea4x8_u8(p,x)         vipmvi_store4x8_u8( p,x)
# define /* void */            vipmvi_storea4x8_u16(p,x)        vipmvi_store4x8_u16(p,x)
# define /* void */            vipmvi_storea4x8_u32(p,x)        vipmvi_store4x8_u32(p,x)
# define /* void */            vipmvi_storea4x8_f32(p,x)        vipmvi_store4x8_f32(p,x)

# define /* void */            vipmvi_storeu4x8_s8(p,x)         vipmvi_store4x8_s8( p,x)
# define /* void */            vipmvi_storeu4x8_s16(p,x)        vipmvi_store4x8_s16(p,x)
# define /* void */            vipmvi_storeu4x8_s32(p,x)        vipmvi_store4x8_s32(p,x)
# define /* void */            vipmvi_storeu4x8_u8(p,x)         vipmvi_store4x8_u8( p,x)
# define /* void */            vipmvi_storeu4x8_u16(p,x)        vipmvi_store4x8_u16(p,x)
# define /* void */            vipmvi_storeu4x8_u32(p,x)        vipmvi_store4x8_u32(p,x)
# define /* void */            vipmvi_storeu4x8_f32(p,x)        vipmvi_store4x8_f32(p,x)


		/*
		 * Transfer 4x16=64 bytes
		 * 4 Q registers
		 */

# define /* bo_m128x4_s8_t  */ vipmvi_load4x16_s8(p)            vld4q_s8( (void const *)(p))
# define /* bo_m128x4_s16_t */ vipmvi_load4x16_s16(p)           vld4q_s16((void const *)(p))
# define /* bo_m128x4_s32_t */ vipmvi_load4x16_s32(p)           vld4q_s32((void const *)(p))
# define /* bo_m128x4_u8_t  */ vipmvi_load4x16_u8(p)            vld4q_u8( (void const *)(p))
# define /* bo_m128x4_u16_t */ vipmvi_load4x16_u16(p)           vld4q_u16((void const *)(p))
# define /* bo_m128x4_u32_t */ vipmvi_load4x16_u32(p)           vld4q_u32((void const *)(p))
# define /* bo_m128x4_f32_t */ vipmvi_load4x16_f32(p)           vld4q_f32((void const *)(p))

# define /* bo_m128x4_s8_t  */ vipmvi_loada4x16_s8(p)           vipmvi_load4x16_s8(p)
# define /* bo_m128x4_s16_t */ vipmvi_loada4x16_s16(p)          vipmvi_load4x16_s16(p)
# define /* bo_m128x4_s32_t */ vipmvi_loada4x16_s32(p)          vipmvi_load4x16_s32(p)
# define /* bo_m128x4_u8_t  */ vipmvi_loada4x16_u8(p)           vipmvi_load4x16_u8(p)
# define /* bo_m128x4_u16_t */ vipmvi_loada4x16_u16(p)          vipmvi_load4x16_u16(p)
# define /* bo_m128x4_u32_t */ vipmvi_loada4x16_u32(p)          vipmvi_load4x16_u32(p)
# define /* bo_m128x4_f32_t */ vipmvi_loada4x16_f32(p)          vipmvi_load4x16_f32(p)

# define /* bo_m128x4_s8_t  */ vipmvi_loadu4x16_s8(p)           vipmvi_load4x16_s8(p)
# define /* bo_m128x4_s16_t */ vipmvi_loadu4x16_s16(p)          vipmvi_load4x16_s16(p)
# define /* bo_m128x4_s32_t */ vipmvi_loadu4x16_s32(p)          vipmvi_load4x16_s32(p)
# define /* bo_m128x4_u8_t  */ vipmvi_loadu4x16_u8(p)           vipmvi_load4x16_u8(p)
# define /* bo_m128x4_u16_t */ vipmvi_loadu4x16_u16(p)          vipmvi_load4x16_u16(p)
# define /* bo_m128x4_u32_t */ vipmvi_loadu4x16_u32(p)          vipmvi_load4x16_u32(p)
# define /* bo_m128x4_f32_t */ vipmvi_loadu4x16_f32(p)          vipmvi_load4x16_f32(p)

# define /* void */            vipmvi_store4x16_s8(p,x)         vst4q_s8( (void *)(p),x)
# define /* void */            vipmvi_store4x16_s16(p,x)        vst4q_s16((void *)(p),x)
# define /* void */            vipmvi_store4x16_s32(p,x)        vst4q_s32((void *)(p),x)
# define /* void */            vipmvi_store4x16_u8(p,x)         vst4q_u8( (void *)(p),x)
# define /* void */            vipmvi_store4x16_u16(p,x)        vst4q_u16((void *)(p),x)
# define /* void */            vipmvi_store4x16_u32(p,x)        vst4q_u32((void *)(p),x)
# define /* void */            vipmvi_store4x16_f32(p,x)        vst4q_f32((void *)(p),x)

# define /* void */            vipmvi_storea4x16_s8(p,x)        vipmvi_store4x16_s8( p,x)
# define /* void */            vipmvi_storea4x16_s16(p,x)       vipmvi_store4x16_s16(p,x)
# define /* void */            vipmvi_storea4x16_s32(p,x)       vipmvi_store4x16_s32(p,x)
# define /* void */            vipmvi_storea4x16_u8(p,x)        vipmvi_store4x16_u8( p,x)
# define /* void */            vipmvi_storea4x16_u16(p,x)       vipmvi_store4x16_u16(p,x)
# define /* void */            vipmvi_storea4x16_u32(p,x)       vipmvi_store4x16_u32(p,x)
# define /* void */            vipmvi_storea4x16_f32(p,x)       vipmvi_store4x16_f32(p,x)

# define /* void */            vipmvi_storeu4x16_s8(p,x)        vipmvi_store4x16_s8( p,x)
# define /* void */            vipmvi_storeu4x16_s16(p,x)       vipmvi_store4x16_s16(p,x)
# define /* void */            vipmvi_storeu4x16_s32(p,x)       vipmvi_store4x16_s32(p,x)
# define /* void */            vipmvi_storeu4x16_u8(p,x)        vipmvi_store4x16_u8( p,x)
# define /* void */            vipmvi_storeu4x16_u16(p,x)       vipmvi_store4x16_u16(p,x)
# define /* void */            vipmvi_storeu4x16_u32(p,x)       vipmvi_store4x16_u32(p,x)
# define /* void */            vipmvi_storeu4x16_f32(p,x)       vipmvi_store4x16_f32(p,x)

	/*
	 * Load/Store 4-element structures
	 * @}
	 */

/* @} */

/* ------------------------------------------------------------------------- */

/*
 * Extract/Put lanes from/to a vector
 * @{
 */

	/*
	 * Get lane from D register
	 */

# define /* int8_t */          vipmvi_lane8_s8( /* bo_m64_s8_t  */x,imm)    vget_lane_s8( x,imm)
# define /* int16_t */         vipmvi_lane8_s16(/* bo_m64_s16_t */x,imm)    vget_lane_s16(x,imm)
# define /* int32_t */         vipmvi_lane8_s32(/* bo_m64_s32_t */x,imm)    vget_lane_s32(x,imm)
# define /* int64_t */         vipmvi_lane8_s64(/* bo_m64_s64_t */x,imm)    vget_lane_s64(x,imm)
# define /* uint8_t */         vipmvi_lane8_u8( /* bo_m64_u8_t  */x,imm)    vget_lane_u8( x,imm)
# define /* uint16_t */        vipmvi_lane8_u16(/* bo_m64_u16_t */x,imm)    vget_lane_u16(x,imm)
# define /* uint32_t */        vipmvi_lane8_u32(/* bo_m64_u32_t */x,imm)    vget_lane_u32(x,imm)
# define /* uint64_t */        vipmvi_lane8_u64(/* bo_m64_u64_t */x,imm)    vget_lane_u64(x,imm)
# define /* float32_t */       vipmvi_lane8_f32(/* bo_m64_f32_t */x,imm)    vget_lane_f32(x,imm)
# define /* float64_t */       vipmvi_lane8_f64(/* bo_m64_f64_t */x,imm)    vget_lane_f64(x,imm)


	/*
	 * Get lane from Q register
	 */

# define /* int8_t */          vipmvi_lane16_s8( /* bo_m128_s8_t  */x,imm)  vgetq_lane_s8( x,imm)
# define /* int16_t */         vipmvi_lane16_s16(/* bo_m128_s16_t */x,imm)  vgetq_lane_s16(x,imm)
# define /* int32_t */         vipmvi_lane16_s32(/* bo_m128_s32_t */x,imm)  vgetq_lane_s32(x,imm)
# define /* int64_t */         vipmvi_lane16_s64(/* bo_m128_s64_t */x,imm)  vgetq_lane_s64(x,imm)
# define /* uint8_t */         vipmvi_lane16_u8( /* bo_m128_u8_t  */x,imm)  vgetq_lane_u8( x,imm)
# define /* uint16_t */        vipmvi_lane16_u16(/* bo_m128_u16_t */x,imm)  vgetq_lane_u16(x,imm)
# define /* uint32_t */        vipmvi_lane16_u32(/* bo_m128_u32_t */x,imm)  vgetq_lane_u32(x,imm)
# define /* uint64_t */        vipmvi_lane16_u64(/* bo_m128_u64_t */x,imm)  vgetq_lane_u64(x,imm)
# define /* float32_t */       vipmvi_lane16_f32(/* bo_m128_f32_t */x,imm)  vgetq_lane_f32(x,imm)
# define /* float64_t */       vipmvi_lane16_f64(/* bo_m128_f64_t */x,imm)  vgetq_lane_f64(x,imm)

/* @} */


__MYC_END_CDECLS

#endif /* !defined(__VIPM_SIMD_ARM_NEON_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
