/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2018 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(D.Vinnyk)
 * Vipm/Statistics/CountRelBox/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_STATISTICS_COUNTRELBOX_ALGORITHM_H__
# define __VIPM_STATISTICS_COUNTRELBOX_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_STATISTICS_COUNTRELBOX_TYPES_H__
#  include "Vipm/Statistics/CountRelBox/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

__PUB bo_status_t
VipmTGAcountrelbox(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_cntrelboxparm_t const *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_countrelbox(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_int                        aCmpOp,
	vipm_seshape_t const           *aStrel,
	vipm_range_t                    aValueRangevSrc[],
	/* Useful for large strel */
	vipm_int                        aScaleFactor, vipm_int aRoundMode
#endif
	);

__PUB bo_status_t
vipma_ecountrelbox(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_int                        aCmpOp,
	vipm_seshape_t const           *aStrel,
	vipm_range_t                    aValueRangevSrc[],
	vipm_cntrelboxopts_t const     *anOpts,
	vipm_arrshape_t const          *aWholeArrShapeSrc,
	/* Useful for large strel */
	vipm_int                        aScaleFactor, vipm_int aRoundMode
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * countrelbox
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_countrelbox_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd) \
    vipma_countrelbox(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd)

# define vipma_countrelbox_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd) \
    vipma_countrelbox(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd)

# define vipma_countrelbox_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd) \
    vipma_countrelbox(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd)

# define vipma_countrelbox_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd) \
    vipma_countrelbox(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,sf,rnd)

/* ------------------------------------------------------------------------- */

/*
 * ecountrelbox
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_ecountrelbox_u8(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd) \
    vipma_ecountrelbox(x,f,rank,vol,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd)

# define vipma_ecountrelbox_s16(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd) \
    vipma_ecountrelbox(x,f,rank,vol,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd)

# define vipma_ecountrelbox_s32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd) \
    vipma_ecountrelbox(x,f,rank,vol,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd)

# define vipma_ecountrelbox_f32(x,f,rank,vol,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd) \
    vipma_ecountrelbox(x,f,rank,vol,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,cmpop,se,vrngS,opts,shapeS,sf,rnd)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_STATISTICS_COUNTRELBOX_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
