/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(V.Sokur)
 * Vipm/Statistics/Histogram/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_STATISTICS_HISTOGRAM_ALGORITHM_H__
# define __VIPM_STATISTICS_HISTOGRAM_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_STATISTICS_HISTOGRAM_TYPES_H__
#  include "Vipm/Statistics/Histogram/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

__PUB bo_status_t
VipmTGAhistogram(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aHistv, /* (struct _Vipm_histogram *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

__PUB bo_status_t
VipmTGAhistogramEx(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aHistv, /* (struct _Vipm_histogram *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_uint                       anElemTypeMsk, /* unused, assumed U8 */
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_histogram(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	struct _Vipm_histogram         *aHistv
#endif
	);

__PUB bo_status_t
vipma_histogram_xm(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[],
	struct _Vipm_histogram         *aHistv
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * histogram
 *
 * Data type: U8, S16, S32, F32
 */

# define vipma_histogram_s8(x,f,rank,vol,S,offsS,strideS,hgv) \
    vipma_histogram(x,f,rank,vol,VipmK_ELEMTYPE_S8,S,offsS,strideS,hgv)

# define vipma_histogram_u8(x,f,rank,vol,S,offsS,strideS,hgv) \
    vipma_histogram(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,hgv)

# define vipma_histogram_s16(x,f,rank,vol,S,offsS,strideS,hgv) \
    vipma_histogram(x,f,rank,vol,VipmK_ELEMTYPE_S16,S,offsS,strideS,hgv)

# define vipma_histogram_s32(x,f,rank,vol,S,offsS,strideS,hgv) \
    vipma_histogram(x,f,rank,vol,VipmK_ELEMTYPE_S32,S,offsS,strideS,hgv)

# define vipma_histogram_f32(x,f,rank,vol,S,offsS,strideS,hgv) \
    vipma_histogram(x,f,rank,vol,VipmK_ELEMTYPE_F32,S,offsS,strideS,hgv)

/* ------------------------------------------------------------------------- */

/*
 * histogram_xm (with "pixel" mask)
 *
 * Data type: U8, S16, S32, F32
 */

# define vipma_histogram_xm_u8(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,hgv) \
    vipma_histogram_xm(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,Sm,offsSm,strideSm,hgv)

# define vipma_histogram_xm_s16(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,hgv) \
    vipma_histogram_xm(x,f,rank,vol,VipmK_ELEMTYPE_S16,S,offsS,strideS,Sm,offsSm,strideSm,hgv)

# define vipma_histogram_xm_s32(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,hgv) \
    vipma_histogram_xm(x,f,rank,vol,VipmK_ELEMTYPE_S32,S,offsS,strideS,Sm,offsSm,strideSm,hgv)

# define vipma_histogram_xm_f32(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,hgv) \
    vipma_histogram_xm(x,f,rank,vol,VipmK_ELEMTYPE_F32,S,offsS,strideS,Sm,offsSm,strideSm,hgv)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_STATISTICS_HISTOGRAM_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
