/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2018 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(D.Vinnyk)
 * Vipm/Statistics/Mean/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_STATISTICS_MEAN_ALGORITHM_H__
# define __VIPM_STATISTICS_MEAN_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_STATISTICS_MEAN_TYPES_H__
#  include "Vipm/Statistics/Mean/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

__PUB bo_status_t
VipmTGAmean(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aResult, /* (vipm_scalar_t *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

__PUB bo_status_t
VipmTGAmeanEx(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aResult, /* (vipm_scalar_t *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_uint                       anElemTypeMsk, /* unused, asmeaned U8 */
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_mean(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_scalar_t                   aResult[/* nchannels */]
#endif
	);

__PUB bo_status_t
vipma_mean_xm(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[],
	vipm_scalar_t                   aResult[/* nchannels */]
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * mean
 *
 * Data type: U8, S16, S32, F32
 */

# define vipma_mean_u8(x,f,rank,vol,S,offsS,strideS,res) \
    vipma_mean(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,res)

# define vipma_mean_s16(x,f,rank,vol,S,offsS,strideS,res) \
    vipma_mean(x,f,rank,vol,VipmK_ELEMTYPE_S16,S,offsS,strideS,res)

# define vipma_mean_s32(x,f,rank,vol,S,offsS,strideS,res) \
    vipma_mean(x,f,rank,vol,VipmK_ELEMTYPE_S32,S,offsS,strideS,res)

# define vipma_mean_f32(x,f,rank,vol,S,offsS,strideS,res) \
    vipma_mean(x,f,rank,vol,VipmK_ELEMTYPE_F32,S,offsS,strideS,res)

/* ------------------------------------------------------------------------- */

/*
 * mean_xm (with "pixel" mask)
 *
 * Data type: U8, S16, S32, F32
 */

# define vipma_mean_xm_u8(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,res) \
    vipma_mean_xm(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,Sm,offsSm,strideSm,res)

# define vipma_mean_xm_s16(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,res) \
    vipma_mean_xm(x,f,rank,vol,VipmK_ELEMTYPE_S16,S,offsS,strideS,Sm,offsSm,strideSm,res)

# define vipma_mean_xm_s32(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,res) \
    vipma_mean_xm(x,f,rank,vol,VipmK_ELEMTYPE_S32,S,offsS,strideS,Sm,offsSm,strideSm,res)

# define vipma_mean_xm_f32(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,res) \
    vipma_mean_xm(x,f,rank,vol,VipmK_ELEMTYPE_F32,S,offsS,strideS,Sm,offsSm,strideSm,res)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_STATISTICS_MEAN_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
