/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(V.Sokur)
 * Vipm/Statistics/Minmax/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_STATISTICS_MINMAX_ALGORITHM_H__
# define __VIPM_STATISTICS_MINMAX_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_STATISTICS_MINMAX_TYPES_H__
#  include "Vipm/Statistics/Minmax/Types.h"
# endif
# ifndef __VIPM_STATISTICS_MINMAX_EXTRA_H__
#  include "Vipm/Statistics/Minmax/Extra.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

__PUB bo_status_t
VipmTGAminmax(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aMinMaxInfo, /* (struct _Vipm_minmaxinfo *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);

__PUB bo_status_t
VipmTGAminmaxEx(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aMinMaxInfo, /* (struct _Vipm_minmaxinfo *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_uint                       anElemTypeMsk, /* unused, assumed U8 */
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_minmax(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	struct _Vipm_minmaxinfo        *aMinMaxInfo
#endif
	);

__PUB bo_status_t
vipma_minmax_xm(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolume[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_imuptr_t                   aBaseMsk, vipm_int const aStartMsk[], vipm_int const aStrideMsk[],
	struct _Vipm_minmaxinfo        *aMinMaxInfo
#endif
	);


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * minmax
 *
 * Data type: U8, S16, S32, F32
 */

# define vipma_minmax_u8(x,f,rank,vol,S,offsS,strideS,xinfo) \
    vipma_minmax(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,xinfo)

# define vipma_minmax_s16(x,f,rank,vol,S,offsS,strideS,xinfo) \
    vipma_minmax(x,f,rank,vol,VipmK_ELEMTYPE_S16,S,offsS,strideS,xinfo)

# define vipma_minmax_s32(x,f,rank,vol,S,offsS,strideS,xinfo) \
    vipma_minmax(x,f,rank,vol,VipmK_ELEMTYPE_S32,S,offsS,strideS,xinfo)

# define vipma_minmax_f32(x,f,rank,vol,S,offsS,strideS,xinfo) \
    vipma_minmax(x,f,rank,vol,VipmK_ELEMTYPE_F32,S,offsS,strideS,xinfo)

/* ------------------------------------------------------------------------- */

/*
 * minmax_xm (with "pixel" mask)
 *
 * Data type: U8, S16, S32, F32
 */

# define vipma_minmax_xm_u8(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,xinfo) \
    vipma_minmax_xm(x,f,rank,vol,VipmK_ELEMTYPE_U8,S,offsS,strideS,Sm,offsSm,strideSm,xinfo)

# define vipma_minmax_xm_s16(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,xinfo) \
    vipma_minmax_xm(x,f,rank,vol,VipmK_ELEMTYPE_S16,S,offsS,strideS,Sm,offsSm,strideSm,xinfo)

# define vipma_minmax_xm_s32(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,xinfo) \
    vipma_minmax_xm(x,f,rank,vol,VipmK_ELEMTYPE_S32,S,offsS,strideS,Sm,offsSm,strideSm,xinfo)

# define vipma_minmax_xm_f32(x,f,rank,vol,S,offsS,strideS,Sm,offsSm,strideSm,xinfo) \
    vipma_minmax_xm(x,f,rank,vol,VipmK_ELEMTYPE_F32,S,offsS,strideS,Sm,offsSm,strideSm,xinfo)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_STATISTICS_MINMAX_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
