/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Array.h	(V.Sokur)
 * Vipm/Utils/Array.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_UTILS_ARRAY_H__
# define __VIPM_UTILS_ARRAY_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_TYPES_H__
#  include <Vipm/Types.h>
# endif

# ifndef __VIPM_MATH_UTILS_VECSPACEUTILS_H__
#  include <Vipm/Math/Utils/VecSpaceUtils.h>
# endif
# ifndef __VIPM_MATH_GEOMETRY_CORE_UTILS_H__
#  include <Vipm/Math/Geometry/Core_Utils.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * @note: min.unit/space-point is a pixel, not element
 *
 * A `rank` formal parameter [of type 'vipm_int'] does not comprise channel dimension. (?)
 */

/* ------------------------------------------------------------------------- */

# include <Vipm/Linkage.h>
# define __PUB  __myc_flinkage__(public,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Construct
 */

/* @note: the destination array treats the input data ('aBase') as external
 *
 * vipm_mut_array_t vipm_array(cp,xtrn)(aThis, aSource);
 * vipm_mut_array_t vipm_array(xtrn)   (aThis, aRank, aVolume, ElemType, aBase, aStart, aStride);
 * vipm_mut_array_t vipm_array(xtrn,3) (aThis, H,W,Cn, ElemType, aBase, aYStart, aXStart, aYStride);
 */

__PUB vipm_mut_array_t  vipm_array(cp,xtrn) __P((vipm_mut_array_t, vipm_array_t));

__PUB vipm_mut_array_t  vipm_array(xtrn)    __P((vipm_mut_array_t, vipm_rank_t, vipm_int const [],
                                                     vipm_uint, vipm_mutptr_t, vipm_int const [], vipm_int const []));
__PUB vipm_mut_array_t  vipm_array(xtrn,3)  __P((vipm_mut_array_t, vipm_int, vipm_int, vipm_int,
                                                     vipm_uint, vipm_mutptr_t, vipm_int, vipm_int, vipm_int));


/*
 * Make sub-array
 */

__PUB vipm_mut_array_t  vipm_array(sar)     __P((vipm_mut_array_t, vipm_array_t, vipm_site_t const *));
__PUB vipm_mut_array_t  vipm_array(sar,I)   __P((vipm_mut_array_t, vipm_site_t const *));

__PUB vipm_mut_array_t  vipm_array(_sar)    __P((vipm_mut_array_t, vipm_array_t, vipm_grank_t,
                                                     vipm_int const [], vipm_int const []));
__PUB vipm_mut_array_t  vipm_array(_sar,I)  __P((vipm_mut_array_t, vipm_grank_t, vipm_int const [], vipm_int const []));


/* ------------------------------------------------------------------------- */

/*
 * Functional
 */

__PUB vipm_int          vipm_arr_(capacity)         __P((vipm_array_t));
__PUB vipm_int          vipm_arr_(capacity,Q)       __P((vipm_array_t));


__PUB vipm_mutptr_t     vipm_arr_(base)             __P((vipm_array_t));
__PUB vipm_mutptr_t     vipm_arr_(at)               __P((vipm_array_t, vipm_Npoint_t const *));
__PUB vipm_mutptr_t     vipm_arr_(at,gv)            __P((vipm_array_t, vipm_grank_t, vipm_int const []));

__PUB vipm_site_t      *vipm_arr_(parsiteof)        __P((vipm_array_t, vipm_site_t *));
__PUB vipm_site_t      *vipm_arr_(siteof)           __P((vipm_array_t, vipm_site_t *));
__PUB vipm_site_t      *vipm_arr_(siteofAt)         __P((vipm_array_t, vipm_site_t *, vipm_Npoint_t const *));
__PUB vipm_site_t      *vipm_arr_(siteofAt,gv)      __P((vipm_array_t, vipm_site_t *, vipm_grank_t, vipm_int const []));

__PUB vipm_mutptr_t     vipm_arr_(at,g2)            __P((vipm_array_t, vipm_int, vipm_int));
__PUB vipm_site_t      *vipm_arr_(siteofAt,g2)      __P((vipm_array_t, vipm_site_t *, vipm_int, vipm_int));

/* ------------------------------------------------------------------------- */

/*
 * retval:bool - non-empty
 *
 * int vipm_arr_(_siteof)   (Arr, aRank, aSiteStartPoint, aSiteVolume);
 * int vipm_arr_(_siteofAt) (Arr, aRank, aSiteStartPoint, aSiteVolume, aPoint);
 */

__PUB int               vipm_arr_(_parsiteof)       __P((vipm_array_t, vipm_grank_t, vipm_int [], vipm_int []));
__PUB int               vipm_arr_(_siteof)          __P((vipm_array_t, vipm_grank_t, vipm_int [], vipm_int []));
__PUB int               vipm_arr_(_siteofAt)        __P((vipm_array_t, vipm_grank_t, vipm_int [], vipm_int [], vipm_int const []));

__PUB int               vipm_arr_(_siteofAt,g2)     __P((vipm_array_t, vipm_int[/*2*/], vipm_int[/*2*/], vipm_int, vipm_int));


/* ------------------------------------------------------------------------- */

__PUB vipm_site_t      *vipm_arr_(adjustRoi)        __P((vipm_array_t, vipm_site_t *, vipm_site_t const *));
__PUB vipm_site_t      *vipm_arr_(adjustRoi,I)      __P((vipm_array_t, vipm_site_t *));
__PUB vipm_site_t      *vipm_arr_(adjorgetRoi)      __P((vipm_array_t, vipm_site_t *, vipm_site_t const *));

/*
 * retval:bool - non-empty (success or failure)
 *
 * int vipm_arr_(negotiateRoi)  (aDest, aSource, aRoiDstRes, aRoiSrcRes,
 *                                               aRoiDst, aRoiSrc, aNumDimsToNegotiate);
 * int vipm_arr_(negotiateRoiEx)(aDest, aSource, aRoiDstRes, aRoiSrcRes, aScaleFactorvInOut[? VipmK_MAXDIMS],
 *                                               aRoiDst, aRoiSrc, aNumDimsToNegotiate);
 *
 * int vipm_arr_(overlapped)    (aLhs, aRhs, aRoiLhs, aRoiRhs);
 */
__PUB int   vipm_arr_(negotiateRoi)     __P((vipm_array_t, vipm_array_t, vipm_site_t *, vipm_site_t *,
                                                 vipm_site_t const *, vipm_Npoint_t const *, vipm_int));
__PUB int   vipm_arr_(negotiateRoiEx)   __P((vipm_array_t, vipm_array_t,
                                                 vipm_site_t *, vipm_site_t *, vipm_scalar_t [],
                                                 vipm_site_t const *, vipm_site_t const *, vipm_int));

__PUB int   vipm_arr_(overlapped)       __P((vipm_array_t, vipm_array_t, vipm_site_t const *, vipm_site_t const *));

/* ------------------------------------------------------------------------- */

/*
 * retval:bool - non-empty (success or failure)
 *
 * int vipm_arr_(_adjustRoi)     (anArr, aRank, aRoiStartRes, aRoiVolumeRes,
 *                                              aRoiStartInput, aRoiVolumeInput);
 * int vipm_arr_(_adjustRoi,I)   (anArr, aRank, aRoiStart, aRoiVolume);
 *
 * int vipm_arr_(_negotiateRoi)  (aDest, aSource, aRank, aNumDimsToNegotiate,
 *                                       aRoiStartDstRes, aRoiVolumeDstRes,
 *                                       aRoiStartSrcRes, aRoiVolumeSrcRes,
 *                                       aRoiStartDstInput, aRoiVolumeDstInput, aRoiStartSrcInput);
 * int vipm_arr_(_negotiateRoiEx)(aDest, aSource, aRank, aNumDimsToNegotiate,
 *                                       aRoiStartDstRes, aRoiVolumeDstRes,
 *                                       aRoiStartSrcRes, aRoiVolumeSrcRes, aScaleFactorvInOut[? VipmK_MAXDIMS],
 *                                       aRoiStartDstInput, aRoiVolumeDstInput,
 *                                       aRoiStartSrcInput, aRoiVolumeSrcInput);
 *
 * int vipm_arr_(_overlapped)    (aLhs, aRhs, aRank, aRoiStartLhs, aRoiVolumeLhs, aRoiStartRhs, aRoiVolumeRhs);
 */

__PUB int   vipm_arr_(_adjustRoi)       __P((vipm_array_t, vipm_grank_t, vipm_int [], vipm_int [],
                                                 vipm_int const [], vipm_int const []));
__PUB int   vipm_arr_(_adjustRoi,I)     __P((vipm_array_t, vipm_grank_t, vipm_int [], vipm_int []));
__PUB int   vipm_arr_(_adjorgetRoi)     __P((vipm_array_t, vipm_grank_t, vipm_int [], vipm_int [],
                                                 vipm_int const [], vipm_int const []));

__PUB int   vipm_arr_(_negotiateRoi)    __P((vipm_array_t, vipm_array_t, vipm_grank_t, vipm_grank_t,
                                                 vipm_int [], vipm_int [], vipm_int [], vipm_int [],
                                                 vipm_int const [], vipm_int const [], vipm_int const []));
__PUB int   vipm_arr_(_negotiateRoiEx)  __P((vipm_array_t, vipm_array_t, vipm_grank_t, vipm_grank_t,
                                                 vipm_int [], vipm_int [], vipm_int [], vipm_int [], vipm_scalar_t [],
                                                 vipm_int const [], vipm_int const [], vipm_int const [], vipm_int const []));

__PUB int   vipm_arr_(_overlapped)      __P((vipm_array_t, vipm_array_t, vipm_grank_t,
                                                 vipm_int const [], vipm_int const [],
                                                 vipm_int const [], vipm_int const []));


/* ------------------------------------------------------------------------- */

/*
 * The functions return a base of array in a parent-coordinate-system
 */

__PUB vipm_mutptr_t     vipm_arr_(pt2pcs)   __P((vipm_array_t, vipm_Npoint_t *, vipm_Npoint_t const *));
__PUB vipm_mutptr_t     vipm_arr_(pt2pcs,I) __P((vipm_array_t, vipm_Npoint_t *));
__PUB vipm_mutptr_t     vipm_arr_(topcs)    __P((vipm_array_t, vipm_grank_t, vipm_int [], vipm_int const []));
__PUB vipm_mutptr_t     vipm_arr_(topcs,I)  __P((vipm_array_t, vipm_grank_t, vipm_int []));


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

_VIPM_INLINE int
vipm_arr_(_adjustRoi)(vipm_array_t anArr, vipm_grank_t aRank,
		vipm_int aRoiStartR[], vipm_int aRoiVolumeR[],
		vipm_int const aRoiStart[], vipm_int const aRoiVolume[])
{
	return (NULL != aRoiVolume
		? vipm_arr_(_adjorgetRoi)(anArr, aRank, aRoiStartR, aRoiVolumeR, aRoiStart, aRoiVolume)
		: vipm_arr_(_adjustRoi,I)(anArr, aRank, aRoiStartR, aRoiVolumeR)
	);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_site_t *
vipm_arr_(adjustRoi)(vipm_array_t anArr, vipm_site_t *aRoiR, vipm_site_t const *aRoi)
{
	return ((NULL != aRoi && vipm_site_cp(aRoiR) != aRoi)
		? vipm_arr_(adjorgetRoi)(anArr, aRoiR, aRoi)
		: vipm_arr_(adjustRoi,I)(anArr, aRoiR)
	);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_mut_array_t
vipm_array(_sar,I)(vipm_mut_array_t aDst, vipm_grank_t aRank,
		vipm_int const aRoiStart[], vipm_int const aRoiVolume[])
{
	return vipm_array(_sar)(aDst, NULL, aRank, aRoiStart, aRoiVolume);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_mut_array_t
vipm_array(sar,I)(vipm_mut_array_t aDst, vipm_site_t const *aRoi)
{
// @todo: fixme - remove (?)
//	return vipm_array(_sar,I)(aDst, VipmK_MAXDIMS, aRoi->si_point.v, aRoi->si_volume.v);
	return vipm_array(_sar,I)(aDst, VipmK_MAXGDIMS, aRoi->si_point.v, aRoi->si_volume.v);
}

_VIPM_INLINE vipm_mut_array_t
vipm_array(sar)(vipm_mut_array_t aDst, vipm_array_t aSrc, vipm_site_t const *aRoi)
{
// @todo: fixme - remove (?)
//	return vipm_array(_sar)(aDst, aSrc, VipmK_MAXDIMS, aRoi->si_point.v, aRoi->si_volume.v);
	return vipm_array(_sar)(aDst, aSrc, VipmK_MAXGDIMS, aRoi->si_point.v, aRoi->si_volume.v);
}


/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_int vipm_arr_(capacity)(vipm_array_t anArr)
{
	return vipm_volume_(_capacity)(VipmK_MAXGDIMS, vipm_array_p(anArr)->ipar_volume_.v);
}

_VIPM_INLINE vipm_int vipm_arr_(capacity,Q)(vipm_array_t anArr)
{
	return vipm_volume_(_capacity,Q)(VipmK_MAXGDIMS, vipm_array_p(anArr)->ipar_volume_.v);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_mutptr_t vipm_arr_(base)(vipm_array_t anArr)
{
	struct _Vipm_array const *arr = vipm_array_p(anArr);
	return vipm_advance_ptr(arr->ipar_base_, (vipm_int)arr->ipar_offset_);
}


_VIPM_INLINE vipm_mutptr_t
vipm_arr_(at)(vipm_array_t anArr, vipm_Npoint_t const *aPoint)
{
	struct _Vipm_array const *arr = vipm_array_p(anArr);
	return vipm_base_(advance)(
// @todo: fixme - remove (?)
//		arr->ipar_base_, (vipm_int)arr->ipar_offset_, VipmK_MAXDIMS, aPoint->v, arr->ipar_strides_.v
		arr->ipar_base_, (vipm_int)arr->ipar_offset_, VipmK_MAXGDIMS, aPoint->v, arr->ipar_strides_.v
	);
}

_VIPM_INLINE vipm_mutptr_t
vipm_arr_(at,gv)(vipm_array_t anArr, vipm_grank_t aRank, vipm_int const aPoint[])
{
	struct _Vipm_array const *arr = vipm_array_p(anArr);
	return vipm_base_(advance)(
		arr->ipar_base_, (vipm_int)arr->ipar_offset_, aRank, aPoint, VIPM_ARR__GStridesQ(arr, aRank)
	);
}

_VIPM_INLINE vipm_mutptr_t
vipm_arr_(at,g2)(vipm_array_t anArr, vipm_int Y, vipm_int X)
{
	struct _Vipm_array const *arr = vipm_array_p(anArr);
	return vipm_base_(at,2d)(
		arr->ipar_base_, (vipm_int)arr->ipar_offset_, Y, X, VIPM_ARR__GStrides(arr, 2)
	);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_site_t *
vipm_arr_(siteofAt)(vipm_array_t anArr, vipm_site_t *aSite, vipm_Npoint_t const *aPoint)
{
	(void)vipm_arr_(_siteofAt)(anArr, VipmK_MAXDIMS, aSite->si_point.v, aSite->si_volume.v,
			(NULL == aPoint ? NULL : aPoint->v));
	return (aSite);
}


/* ------------------------------------------------------------------------- */

_VIPM_INLINE vipm_mutptr_t
vipm_arr_(topcs,I)(vipm_array_t anArr, vipm_grank_t aRank, vipm_int aPoint[])
{
	struct _Vipm_array const *arr = vipm_array_p(anArr);
	(void)vipm_vec_(update,loff)(aRank, aPoint, VIPM_ARR__GStridesQ(arr, aRank), arr->ipar_offset_);
	return (arr->ipar_base_);
}


_VIPM_INLINE vipm_mutptr_t
vipm_arr_(pt2pcs,I)(vipm_array_t anArr, vipm_Npoint_t *aPoint)
{
	struct _Vipm_array const *arr = vipm_array_p(anArr);
	(void)vipm_vec_(update,loff)(VipmK_MAXDIMS, aPoint->v, arr->ipar_strides_.v, arr->ipar_offset_);
	return (arr->ipar_base_);
}


__MYC_END_CDECLS
# undef __PUB


#endif /* !defined(__VIPM_UTILS_ARRAY_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
