/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * STreeBuilder.h	(V.Sokur)
 * Vipm/Utils/STreeBuilder.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_UTILS_STREE_BUILDER_H__
# define __VIPM_UTILS_STREE_BUILDER_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_UTILS_STREE_H__
#  include <Vipm/Utils/STree.h>
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB __myc_flinkage__(public,libVipm)
# define __PROT __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

__PUB struct _Vipm_streebuilder *
VipmSTBopen(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,           /* = NULL */
	void                           *aXInterface,        /* = NULL */
	vipm_uint                       aNumBytesExtra,     /* = 0; usefull, if (aBuilder == nil) */
	struct _Vipm_stree             *aXtrnTree,          /* = NULL */
	vipm_rref_t                     aXtrnMmstack,       /* = NULL */
	vipm_rref_t                     aParentMmstack      /* = NULL */
#endif
	);

__PUB void
VipmSTBclose(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	int                             NoCloseTree     /* [default: 0] */
#endif
	);

/*
 * The next fields should be set [by default] before calling of the routine:
 *
 * 1. aTree->stb_xdata = NULLF (for instance)
 */
__PUB struct _Vipm_streebuilder *
VipmSTBinit(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,           /* != NULL */
	void                           *aXInterface,        /* = NULL */
	vipm_uint                       aNumBytesExtra,     /* = 0; usefull, if (aBuilder == nil) */
	struct _Vipm_stree             *aXtrnTree,          /* = NULL */
	vipm_rref_t                     aXtrnMmstack,       /* = NULL */
	vipm_rref_t                     aParentMmstack      /* = NULL */
#endif
	);

__PUB struct _Vipm_streebuilder *
VipmSTBfini(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	int                             NoCloseTree     /* [default: 0] */
#endif
	);

__PUB struct _Vipm_streebuilder *
VipmSTBclear(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	int                             NoDetachTree    /* [default: 0]*/
#endif
	);

__PUB struct _Vipm_streebuilder *
VipmSTBxtrntree(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	struct _Vipm_stree             *aXtrnTree
#endif
	);

/* ------------------------------------------------------------------------- */

__PUB struct _Vipm_streenode *
VipmSTBnewnode(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder
#endif
	);


/* ========================================================================= */

__PROT struct _Vipm_stree *
__VipmSTBtree(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder
#endif
	);

__PROT void
__VipmSTBrootnode(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	struct _Vipm_streenode         *aNode
#endif
	);


/* ========================================================================= */

__PRIV void
__VipmSTBattach(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	struct _Vipm_stree             *aXtrnTree
#endif
	);

__PRIV void
__VipmSTBdetach(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder
#endif
	);

__PRIV struct _Vipm_streebuilder *
__VipmSTB_xtrntree(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	struct _Vipm_stree             *aXtrnTree
#endif
	);

__PRIV struct _Vipm_streebuilder *
__VipmSTB_sxtrntree(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	struct _Vipm_stree             *aXtrnTree
#endif
	);

/* ------------------------------------------------------------------------- */

__PRIV struct _Vipm_streebuilder *
__VipmSTBinitMmstack(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	vipm_rref_t                     aParentMmstack      /* = NULL */
#endif
	);

__PRIV struct _Vipm_streebuilder *
__VipmSTBdestroyMmstack(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder
#endif
	);

__PRIV struct _Vipm_streebuilder *
__VipmSTBresetMmstack(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	vipm_rref_t                     aXtrnMmstack,       /* = NULL */
	vipm_rref_t                     aParentMmstack      /* = NULL */
#endif
	);

__PRIV vipm_rref_t
__VipmSTBmmstack(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder
#endif
	);

/* ------------------------------------------------------------------------- */

__PRIV void
__VipmSTBgetmmsp(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	vipm_mmstate_t                 *aState
#endif
	);

__PRIV void
__VipmSTBsetmmsp(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	vipm_mmstate_t const           *aState
#endif
	);


__PRIV vipm_mutptr_t
__VipmSTBalloca(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	bo_size_t                       aBytec,
	bo_size_t                       Align
#endif
	);

__PRIV void
__VipmSTBfreea(
#if __myc_require(prototypes)
	struct _Vipm_streebuilder      *aBuilder,
	vipm_mutptr_t                   aPtr
#endif
	);


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

# define __VipmSTBrootnode(b,n)    (__VipmSTBtree(b)->gt_root = (n))
# define __VipmSTBdestroyMmstack(b) __VipmSTBresetMmstack(b, (vipm_rref_t)NULL, (vipm_rref_t)NULL)


__MYC_END_CDECLS
# undef __PUB
# undef __PROT
# undef __PRIV


#endif /* !defined(__VIPM_UTILS_STREE_BUILDER_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
