/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Algorithm.h	(V.Sokur)
 * Vipm/Warp/Resize/Algorithm.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_WARP_RESIZE_ALGORITHM_H__
# define __VIPM_WARP_RESIZE_ALGORITHM_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VIPM_WARP_RESIZE_TYPES_H__
#  include "Vipm/Warp/Resize/Types.h"
# endif


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vipm/Linkage.h>
# define __PUB  __myc_flinkage__(public,libVipm)
# define __PRIV __myc_flinkage__(private,libVipm)
__MYC_BEGIN_CDECLS

/*
 * Generic
 */

__PUB bo_status_t
VipmTGAresize(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rref_t                     aParm, /* (vipm_tga_reszparm_t *) */
	vipm_rank_t                     aRank,
	vipm_int const                  aVolumeDst[],
	vipm_uint                       anElemTypeDst,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_uint                       anElemTypeSrc,
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[]
#endif
	);


/* ========================================================================= */

__PUB bo_status_t
vipma_resize(
#if __myc_require(prototypes)
	struct _Vipm_xenv              *aXenv,
	vipm_ulong                      Options,
	vipm_rank_t                     aRank,
	vipm_int const                  aVolumeDst[],
	vipm_int const                  aVolumeSrc[],       /* = NULL */
	vipm_uint                       anElemType,
	vipm_mutptr_t                   aBaseDst, vipm_int const aStartDst[], vipm_int const aStrideDst[],
	vipm_imuptr_t                   aBaseSrc, vipm_int const aStartSrc[], vipm_int const aStrideSrc[],
	vipm_int                        anInterMethod,
	vipm_scalar_t const             aReszFactor[]       /* = NULL */
#endif
	);


# define vipma_resize_nn(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,VipmK_NN_INTERP,rszf)

# define vipma_resize_linear(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,VipmK_LINEAR_INTERP,rszf)

# define vipma_resize_cubic(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,VipmK_CUBIC_INTERP,rszf)

# define vipma_resize_lanczos(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,VipmK_LANCZOS_INTERP,rszf)

# define vipma_resize_fast(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,VipmK_FAST_INTERP,rszf)

# define vipma_resize_best(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,VipmK_BEST_INTERP,rszf)


/* ============================================================================
 * Specialized versions
 */
# pragma mark -

/*
 * resize
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_resize_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,im,rszf)


# define vipma_resize_nvd_u8(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,NULL,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_nvd_s16(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_nvd_s32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_nvd_f32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,NULL,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,im,rszf)


# define vipma_resize_nvs_u8(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,NULL,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_nvs_s16(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_nvs_s32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,im,rszf)

# define vipma_resize_nvs_f32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,im,rszf) \
    vipma_resize(x,f,rank,volD,NULL,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,im,rszf)


# define vipma_resize_nf_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,im,NULL)

# define vipma_resize_nf_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,im,NULL)

# define vipma_resize_nf_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,im,NULL)

# define vipma_resize_nf_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,im) \
    vipma_resize(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,im,NULL)

/* ------------------------------------------------------------------------- */

/*
 * resize_nn
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_resize_nn_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_nn_nvd(x,f,rank,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,NULL,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_nvd_u8(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,NULL,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_nvd_s16(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_nvd_s32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_nvd_f32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,NULL,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_nn_nvs(x,f,rank,volD,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,NULL,ET,D,offsD,strideD,S,offsS,strideS,VipmK_NN_INTERP,rszf)

# define vipma_resize_nn_nvs_u8(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,NULL,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_nvs_s16(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_nvs_s32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_nn_nvs_f32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_nn(x,f,rank,volD,NULL,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_nn_nf(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_nn(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,VipmK_NN_INTERP,NULL)

# define vipma_resize_nn_nf_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_nn_nf_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_nn_nf_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_nn_nf_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_nn(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,NULL)

/* ------------------------------------------------------------------------- */

/*
 * resize_linear
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_resize_linear_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_linear_nvd(x,f,rank,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,NULL,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvd_u8(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,NULL,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvd_s16(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvd_s32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvd_f32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,NULL,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_linear_nvs(x,f,rank,volD,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,NULL,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvs_u8(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,NULL,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvs_s16(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvs_s32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_linear_nvs_f32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_linear(x,f,rank,volD,NULL,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_linear_nf(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_linear(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_linear_nf_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_linear_nf_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_linear_nf_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_linear_nf_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_linear(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,NULL)

/* ------------------------------------------------------------------------- */

/*
 * resize_cubic
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_resize_cubic_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_cubic_nvd(x,f,rank,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,NULL,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvd_u8(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,NULL,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvd_s16(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvd_s32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvd_f32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,NULL,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_cubic_nvs(x,f,rank,volD,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,NULL,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvs_u8(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,NULL,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvs_s16(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvs_s32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_cubic_nvs_f32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_cubic(x,f,rank,volD,NULL,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_cubic_nf(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_cubic(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_cubic_nf_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_cubic_nf_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_cubic_nf_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_cubic_nf_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_cubic(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,NULL)

/* ------------------------------------------------------------------------- */

/*
 * resize_lanczos
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_resize_lanczos_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_lanczos_nvd(x,f,rank,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,NULL,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvd_u8(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,NULL,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvd_s16(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvd_s32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvd_f32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,NULL,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_lanczos_nvs(x,f,rank,volD,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,NULL,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvs_u8(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,NULL,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvs_s16(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvs_s32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_lanczos_nvs_f32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_lanczos(x,f,rank,volD,NULL,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_lanczos_nf(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_lanczos(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_lanczos_nf_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_lanczos_nf_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_lanczos_nf_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_lanczos_nf_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_lanczos(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,NULL)

/* ------------------------------------------------------------------------- */

/*
 * resize_fast
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_resize_fast_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_fast_nvd(x,f,rank,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,NULL,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvd_u8(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,NULL,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvd_s16(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvd_s32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvd_f32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,NULL,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_fast_nvs(x,f,rank,volD,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,NULL,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvs_u8(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,NULL,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvs_s16(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvs_s32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_fast_nvs_f32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_fast(x,f,rank,volD,NULL,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_fast_nf(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_fast(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_fast_nf_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_fast_nf_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_fast_nf_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_fast_nf_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_fast(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,NULL)

/* ------------------------------------------------------------------------- */

/*
 * resize_best
 *
 * Result/Data type: U8,S16,S32, F32
 */

# define vipma_resize_best_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_best_nvd(x,f,rank,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,NULL,volS,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvd_u8(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,NULL,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvd_s16(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvd_s32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,NULL,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvd_f32(x,f,rank,volS,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,NULL,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_best_nvs(x,f,rank,volD,ET,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,NULL,ET,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvs_u8(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,NULL,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvs_s16(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvs_s32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,NULL,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,rszf)

# define vipma_resize_best_nvs_f32(x,f,rank,volD,D,offsD,strideD,S,offsS,strideS,rszf) \
    vipma_resize_best(x,f,rank,volD,NULL,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,rszf)


# define vipma_resize_best_nf(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_best(x,f,rank,volD,volS,ET,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_best_nf_u8(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_U8,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_best_nf_s16(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_S16,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_best_nf_s32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_S32,D,offsD,strideD,S,offsS,strideS,NULL)

# define vipma_resize_best_nf_f32(x,f,rank,volD,volS,D,offsD,strideD,S,offsS,strideS) \
    vipma_resize_best(x,f,rank,volD,volS,VipmK_ELEMTYPE_F32,D,offsD,strideD,S,offsS,strideS,NULL)


__MYC_END_CDECLS
# undef __PUB
# undef __PRIV


#endif /* !defined(__VIPM_WARP_RESIZE_ALGORITHM_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
