/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Base_Macros.h	(V.Sokur)
 * Vipm/_Types/Base_Macros.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES_BASE__MACROS_H__
# define __VIPM__TYPES_BASE__MACROS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif

# ifndef __VIPM__TYPES_BASE__TYPES_H__
#  include "Vipm/_Types/Base_Types.h"
# endif
 */


/******************************************************************************
 * %% BeginSection: definitions
 */

/*
 * @{
 * Base [rt] macros
 */

/*
 * Byte Manipulation macros
 */
# if (BO_BYTE_ORDER == LITTLE_BYTE_ORDER)

/* is-unsigned(typeof(v)) */
#  define VIPM_BYAT16_0(v)          ((bo_uint8_t)(v))
#  define VIPM_BYAT16_1(v)          ((bo_uint8_t)((v) >> 8))
#  define VIPM_BYAT32_0(v)          ((bo_uint8_t)(v))
#  define VIPM_BYAT32_1(v)          ((bo_uint8_t)((v) >> 8))
#  define VIPM_BYAT32_2(v)          ((bo_uint8_t)((v) >> 16))
#  define VIPM_BYAT32_3(v)          ((bo_uint8_t)((v) >> 24))

#  define VIPM_HRAT32_0(v)          ((bo_uint16_t)(v))
#  define VIPM_HRAT32_1(v)          ((bo_uint16_t)((v) >> 16))


#  define VIPM_BYAT16(i,v)          ((bo_uint8_t)((v) >> (i * 8)))
#  define VIPM_BYAT32(i,v)          ((bo_uint8_t)((v) >> (i * 8)))
#  define VIPM_BYAT64(i,v)          ((bo_uint8_t)((v) >> (i * 8)))

#  define VIPM_MKI32BY(b0,b1,b2,b3)                                                     \
    (((bo_uint32_t)(bo_uint8_t)(b0) << 0) | ((bo_uint32_t)(bo_uint8_t)(b1) << 8) |      \
        ((bo_uint32_t)(bo_uint8_t)(b2) << 16) | ((bo_uint32_t)(bo_uint8_t)(b3) << 24))

#  define VIPM_MKI64BY(b0,b1,b2,b3,b4,b5,b6,b7)                                         \
    (((bo_uint64_t)(bo_uint8_t)(b0) << 0) | ((bo_uint64_t)(bo_uint8_t)(b1) << 8) |      \
        ((bo_uint64_t)(bo_uint8_t)(b2) << 16) | ((bo_uint64_t)(bo_uint8_t)(b3) << 24) | \
        ((bo_uint64_t)(bo_uint8_t)(b4) << 32) | ((bo_uint64_t)(bo_uint8_t)(b5) << 40) | \
        ((bo_uint64_t)(bo_uint8_t)(b6) << 48) | ((bo_uint64_t)(bo_uint8_t)(b7) << 56))


#  define VIPM_HRAT32(i,v)          ((bo_uint16_t)((v) >> (i * 16)))
#  define VIPM_HRAT64(i,v)          ((bo_uint16_t)((v) >> (i * 16)))

#  define VIPM_MKI32HR(h0,h1) \
    (((bo_uint32_t)(bo_uint16_t)(h0) << 0) | ((bo_uint32_t)(bo_uint16_t)(h1) << 16))

#  define VIPM_MKI64HR(h0,h1,h2,h3)                                                  \
    (((u_int64_t)(bo_uint16_t)(h0) << 0) | ((u_int64_t)(bo_uint16_t)(h1) << 16) |    \
        ((u_int64_t)(bo_uint16_t)(h2) << 32) | ((u_int64_t)(bo_uint16_t)(h3) << 48))


# define VIPM_BYMASK32(i,m)         ((m) << (i * 8))
# define VIPM_HRMASK32(i,m)         ((m) << (i * 16))

# else /* if (BO_BYTE_ORDER == BIG_BYTE_ORDER) */

/* is-unsigned(typeof(v)) */
#  define VIPM_BYAT16_0(v)          ((bo_uint8_t)((v) >> 8))
#  define VIPM_BYAT16_1(v)          ((bo_uint8_t)(v))
#  define VIPM_BYAT32_0(v)          ((bo_uint8_t)((v) >> 24))
#  define VIPM_BYAT32_1(v)          ((bo_uint8_t)((v) >> 16))
#  define VIPM_BYAT32_2(v)          ((bo_uint8_t)((v) >> 8))
#  define VIPM_BYAT32_3(v)          ((bo_uint8_t)(v))

#  define VIPM_HRAT32_0(v)          ((bo_uint16_t)((v) >> 16))
#  define VIPM_HRAT32_1(v)          ((bo_uint16_t)(v))


#  define VIPM_BYAT16(i,v)          ((bo_uint8_t)((v) >> ((1 - i) * 8)))
#  define VIPM_BYAT32(i,v)          ((bo_uint8_t)((v) >> ((3 - i) * 8)))
#  define VIPM_BYAT64(i,v)          ((bo_uint8_t)((v) >> ((7 - i) * 8)))

#  define VIPM_MKI32BY(b0,b1,b2,b3)                                                     \
    (((bo_uint32_t)(bo_uint8_t)(b3) << 0) | ((bo_uint32_t)(bo_uint8_t)(b2) << 8) |      \
        ((bo_uint32_t)(bo_uint8_t)(b1) << 16) | ((bo_uint32_t)(bo_uint8_t)(b0) << 24))

#  define VIPM_MKI64BY(b0,b1,b2,b3,b4,b5,b6,b7)                                         \
    (((bo_uint64_t)(bo_uint8_t)(b7) << 0) | ((bo_uint64_t)(bo_uint8_t)(b6) << 8) |      \
        ((bo_uint64_t)(bo_uint8_t)(b5) << 16) | ((bo_uint64_t)(bo_uint8_t)(b4) << 24) | \
        ((bo_uint64_t)(bo_uint8_t)(b3) << 32) | ((bo_uint64_t)(bo_uint8_t)(b2) << 40) | \
        ((bo_uint64_t)(bo_uint8_t)(b1) << 48) | ((bo_uint64_t)(bo_uint8_t)(b0) << 56))


#  define VIPM_HRAT32(i,v)          ((bo_uint16_t)((v) >> ((1 - i) * 16)))
#  define VIPM_HRAT64(i,v)          ((bo_uint16_t)((v) >> ((3 - i) * 16)))

#  define VIPM_MKI32HR(h0,h1) \
    (((bo_uint32_t)(bo_uint16_t)(h1) << 0) | ((bo_uint32_t)(bo_uint16_t)(h0) << 16))

#  define VIPM_MKI64HR(h0,h1,h2,h3)                                                  \
    (((u_int64_t)(bo_uint16_t)(h3) << 0) | ((u_int64_t)(bo_uint16_t)(h2) << 16) |    \
        ((u_int64_t)(bo_uint16_t)(h1) << 32) | ((u_int64_t)(bo_uint16_t)(h0) << 48))


# define VIPM_BYMASK32(i,m)         ((m) << ((3 - i) * 8))
# define VIPM_HRMASK32(i,m)         ((m) << ((1 - i) * 16))

# endif /* (BO_BYTE_ORDER == LITTLE_BYTE_ORDER) */

/* ------------------------------------------------------------------------- */

# define __VIPM_COUNTOF(a)          (sizeof(a) / sizeof((a)[0]))


# define VIPM_SWAP(_Ty, x, y)  \
    do {                       \
        _Ty ___t = y;          \
        y = x;                 \
        x = ___t;              \
    } while (0)

# define VIPM_SWAP_IF(_Ty, x, _Cmp, y)  \
    do {                                \
        if (x _Cmp y) {                 \
            _Ty ___t = y;               \
            y = x;                      \
            x = ___t;                   \
        }                               \
    } while (0)

# ifdef _MSC_VER
#  define VIPM_SWAP_PTR(x, y)       VIPM_SWAP(void *, x, y)
#  define VIPM_SWAP_CPTR(x, y)      VIPM_SWAP(void const *, x, y)
# else
#  define VIPM_SWAP_PTR(x, y)       VIPM_SWAP(__typeof__(x), x, y)
#  define VIPM_SWAP_CPTR(x, y)      VIPM_SWAP(__typeof__(x), x, y)
# endif

# define VIPM_PSWAP3(_Ty, xp, yp)         \
    do {                                  \
        _Ty *___xp = (xp), *___yp = (yp); \
        _Ty ___t = *___xp;                \
        *___xp = *___yp;                  \
        *___yp = ___t;                    \
    } while (0)


# define _VIPM_MIN(x, y)            ((x) <= (y) ? (x) : (y))
# define _VIPM_MAX(x, y)            ((x) <= (y) ? (y) : (x))

/* [int] min/max without jumps */
# define _VIPM_IMIN(x, y)           ((x) ^ (((x)^(y)) & (((x) < (y)) - 1)))
# define _VIPM_IMAX(x, y)           ((x) ^ (((x)^(y)) & (((x) > (y)) - 1)))

/* absolute [int] value without jumps */
# ifndef __cplusplus
//#  define _VIPM_IABS(x)             ( ((x) ^ ((x) < 0 ? -1 : 0)) - ((x) < 0 ? -1 : 0) )
#  define _VIPM_IABS(x)             ( ((x) ^ (-((x) < 0))) - (-((x) < 0)) )
# else
#  define _VIPM_IABS(x)             abs(x)
# endif


/* safe pointer advance */
# define _VIPM_ADDPS(p,a)           ((NULL != (p)) ? ((p) + (a)) : NULL)
# define _VIPM_SUBPS(p,a)           ((NULL != (p)) ? ((p) - (a)) : NULL)

/*
 * Base [rt] macros
 * @}
 */

#endif /* !defined(__VIPM__TYPES_BASE__MACROS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
