/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Base_Types.h	(V.Sokur)
 * Vipm/_Types/Base_Types.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES_BASE__TYPES_H__
# define __VIPM__TYPES_BASE__TYPES_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif

# ifndef __BO_TYPES__
#  include <Bo/Types.h>
# endif
 */


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

# if defined(_WIN64) || defined(WIN64)
#  ifndef __LLP64__
#   define __LLP64__                1
#  endif
# endif


/*
 * @{
 * Base Integer/Float Types
 */

/* @todo: fixme */
# ifndef __BO_INT_TYPES_DEFINED__
#  define __BO_INT_TYPES_DEFINED__

#  if defined(_MSC_VER)

typedef signed char                 bo_int8_t;
typedef signed short                bo_int16_t;
typedef __int32                     bo_int32_t;
typedef __int64                     bo_int64_t;

typedef unsigned char               bo_uint8_t;
typedef unsigned short              bo_uint16_t;
typedef unsigned __int32            bo_uint32_t;
typedef unsigned __int64            bo_uint64_t;
#   if defined(__LLP64__) || (64 <= BO_WIDTHOF_MWORD)
typedef unsigned long long          bo_size_t;
typedef long long                   bo_ptrdiff_t;
typedef unsigned long long          bo_uintptr_t;
typedef long long                   bo_intptr_t;
#   else
typedef unsigned long               bo_size_t;
typedef long                        bo_ptrdiff_t;
typedef unsigned long               bo_uintptr_t;
typedef long                        bo_intptr_t;
#   endif

#  else /* if !defined(_MSC_VER) */
/* human compiler */

typedef int8_t                      bo_int8_t;
typedef int16_t                     bo_int16_t;
typedef int32_t                     bo_int32_t;
typedef int64_t                     bo_int64_t;

typedef uint8_t                     bo_uint8_t;
typedef uint16_t                    bo_uint16_t;
typedef uint32_t                    bo_uint32_t;
typedef uint64_t                    bo_uint64_t;

/*
 * typedef size_t                      bo_size_t;
 * typedef ptrdiff_t                   bo_ptrdiff_t;
 */
typedef __typeof__(sizeof(0))               bo_size_t;
typedef __typeof__((char *)0 - (char *)0)   bo_ptrdiff_t;

typedef intptr_t                    bo_intptr_t;
typedef uintptr_t                   bo_uintptr_t;

#  endif /* defined(_MSC_VER) */

typedef bo_size_t                   bo_rsize_t;
typedef bo_ptrdiff_t                bo_ssize_t;

# endif /* !defined(__BO_INT_TYPES_DEFINED__) */


# ifndef __BO_FLOAT_TYPES_DEFINED__
#  define __BO_FLOAT_TYPES_DEFINED__
typedef bo_int16_t                  bo_float16_t;
typedef float                       bo_float32_t;
typedef double                      bo_float64_t;
# endif /* !defined(__BO_FLOAT_TYPES_DEFINED__) */


# ifndef __BO_FINT32_TYPE_DEFINED__
#  define __BO_FINT32_TYPE_DEFINED__
typedef bo_int32_t                  bo_fint32_t;
# endif

/* ------------------------------------------------------------------------- */

/* @todo: fixme */
# ifndef __BO_XINT_TYPES_DEFINED__
#  define __BO_XINT_TYPES_DEFINED__

	/*
	 * 8 bit [u]int
	 */

struct bo_int8x1 {
	union {
		bo_int8_t   a[1];
		bo_int8_t   x;
	};
};
struct bo_int8x2 {
	union {
		bo_int8_t       a[2];
		struct {
			bo_int8_t   x, y;
		};
	};
};
struct bo_int8x3 {
	union {
		bo_int8_t       a[3];
		struct {
			bo_int8_t   x, y, z;
		};
	};
};
struct bo_int8x4 {
	union {
		bo_int8_t       a[4];
		struct {
			bo_int8_t   x, y, z, t;
		};
	};
};
typedef struct bo_int8x1            bo_int8x1_t;
typedef struct bo_int8x2            bo_int8x2_t;
typedef struct bo_int8x3            bo_int8x3_t;
typedef struct bo_int8x4            bo_int8x4_t;

struct bo_uint8x1 {
	union {
		bo_uint8_t  a[1];
		bo_uint8_t  x;
	};
};
struct bo_uint8x2 {
	union {
		bo_uint8_t      a[2];
		struct {
			bo_uint8_t  x, y;
		};
	};
};
struct bo_uint8x3 {
	union {
		bo_uint8_t      a[3];
		struct {
			bo_uint8_t  x, y, z;
		};
	};
};
struct bo_uint8x4 {
	union {
		bo_uint8_t      a[4];
		struct {
			bo_uint8_t  x, y, z, t;
		};
	};
};
typedef struct bo_uint8x1           bo_uint8x1_t;
typedef struct bo_uint8x2           bo_uint8x2_t;
typedef struct bo_uint8x3           bo_uint8x3_t;
typedef struct bo_uint8x4           bo_uint8x4_t;


	/*
	 * 16 bit [u]int
	 */

struct bo_int16x1 {
	union {
		bo_int16_t  a[1];
		bo_int16_t  x;
	};
};
struct bo_int16x2 {
	union {
		bo_int16_t      a[2];
		struct {
			bo_int16_t  x, y;
		};
	};
};
struct bo_int16x3 {
	union {
		bo_int16_t      a[3];
		struct {
			bo_int16_t  x, y, z;
		};
	};
};
struct bo_int16x4 {
	union {
		bo_int16_t      a[4];
		struct {
			bo_int16_t  x, y, z, t;
		};
	};
};
typedef struct bo_int16x1           bo_int16x1_t;
typedef struct bo_int16x2           bo_int16x2_t;
typedef struct bo_int16x3           bo_int16x3_t;
typedef struct bo_int16x4           bo_int16x4_t;

struct bo_uint16x1 {
	union {
		bo_uint16_t     a[1];
		bo_uint16_t     x;
	};
};
struct bo_uint16x2 {
	union {
		bo_uint16_t     a[2];
		struct {
			bo_uint16_t x, y;
		};
	};
};
struct bo_uint16x3 {
	union {
		bo_uint16_t     a[3];
		struct {
			bo_uint16_t x, y, z;
		};
	};
};
struct bo_uint16x4 {
	union {
		bo_uint16_t     a[4];
		struct {
			bo_uint16_t x, y, z, t;
		};
	};
};
typedef struct bo_uint16x1          bo_uint16x1_t;
typedef struct bo_uint16x2          bo_uint16x2_t;
typedef struct bo_uint16x3          bo_uint16x3_t;
typedef struct bo_uint16x4          bo_uint16x4_t;


	/*
	 * 32 bit [u]int
	 */

struct bo_int32x1 {
	union {
		bo_int32_t  a[1];
		bo_int32_t  x;
	};
};
struct bo_int32x2 {
	union {
		bo_int32_t      a[2];
		struct {
			bo_int32_t  x, y;
		};
	};
};
struct bo_int32x3 {
	union {
		bo_int32_t      a[3];
		struct {
			bo_int32_t  x, y, z;
		};
	};
};
struct bo_int32x4 {
	union {
		bo_int32_t      a[4];
		struct {
			bo_int32_t  x, y, z, t;
		};
	};
};
typedef struct bo_int32x1           bo_int32x1_t;
typedef struct bo_int32x2           bo_int32x2_t;
typedef struct bo_int32x3           bo_int32x3_t;
typedef struct bo_int32x4           bo_int32x4_t;

struct bo_uint32x1 {
	union {
		bo_uint32_t     a[1];
		bo_uint32_t     x;
	};
};
struct bo_uint32x2 {
	union {
		bo_uint32_t     a[2];
		struct {
			bo_uint32_t x, y;
		};
	};
};
struct bo_uint32x3 {
	union {
		bo_uint32_t     a[3];
		struct {
			bo_uint32_t x, y, z;
		};
	};
};
struct bo_uint32x4 {
	union {
		bo_uint32_t     a[4];
		struct {
			bo_uint32_t x, y, z, t;
		};
	};
};
typedef struct bo_uint32x1          bo_uint32x1_t;
typedef struct bo_uint32x2          bo_uint32x2_t;
typedef struct bo_uint32x3          bo_uint32x3_t;
typedef struct bo_uint32x4          bo_uint32x4_t;


	/*
	 * 64 bit [u]int
	 */

struct bo_int64x1 {
	union {
		bo_int64_t  a[1];
		bo_int64_t  x;
	};
};
struct bo_int64x2 {
	union {
		bo_int64_t      a[2];
		struct {
			bo_int64_t  x, y;
		};
	};
};
struct bo_int64x3 {
	union {
		bo_int64_t      a[3];
		struct {
			bo_int64_t  x, y, z;
		};
	};
};
struct bo_int64x4 {
	union {
		bo_int64_t      a[4];
		struct {
			bo_int64_t  x, y, z, t;
		};
	};
};
typedef struct bo_int64x1           bo_int64x1_t;
typedef struct bo_int64x2           bo_int64x2_t;
typedef struct bo_int64x3           bo_int64x3_t;
typedef struct bo_int64x4           bo_int64x4_t;

struct bo_uint64x1 {
	union {
		bo_uint64_t     a[1];
		bo_uint64_t     x;
	};
};
struct bo_uint64x2 {
	union {
		bo_uint64_t     a[2];
		struct {
			bo_uint64_t x, y;
		};
	};
};
struct bo_uint64x3 {
	union {
		bo_uint64_t     a[3];
		struct {
			bo_uint64_t x, y, z;
		};
	};
};
struct bo_uint64x4 {
	union {
		bo_uint64_t     a[4];
		struct {
			bo_uint64_t x, y, z, t;
		};
	};
};
typedef struct bo_uint64x1          bo_uint64x1_t;
typedef struct bo_uint64x2          bo_uint64x2_t;
typedef struct bo_uint64x3          bo_uint64x3_t;
typedef struct bo_uint64x4          bo_uint64x4_t;

# endif /* !defined(__BO_XINT_TYPES_DEFINED__) */


# ifndef __BO_XFLOAT_TYPES_DEFINED__
#  define __BO_XFLOAT_TYPES_DEFINED__

	/*
	 * 16 bit float
	 */

struct bo_float16x1 {
	union {
		bo_float16_t    a[1];
		bo_float16_t    x;
	};
};
struct bo_float16x2 {
	union {
		bo_float16_t        a[2];
		struct {
			bo_float16_t    x, y;
		};
	};
};
struct bo_float16x3 {
	union {
		bo_float16_t        a[3];
		struct {
			bo_float16_t    x, y, z;
		};
	};
};
struct bo_float16x4 {
	union {
		bo_float16_t        a[4];
		struct {
			bo_float16_t    x, y, z, t;
		};
	};
};
typedef struct bo_float16x1         bo_float16x1_t;
typedef struct bo_float16x2         bo_float16x2_t;
typedef struct bo_float16x3         bo_float16x3_t;
typedef struct bo_float16x4         bo_float16x4_t;

	/*
	 * 32 bit float
	 */

struct bo_float32x1 {
	union {
		bo_float32_t    a[1];
		bo_float32_t    x;
	};
};
struct bo_float32x2 {
	union {
		bo_float32_t        a[2];
		struct {
			bo_float32_t    x, y;
		};
	};
};
struct bo_float32x3 {
	union {
		bo_float32_t        a[3];
		struct {
			bo_float32_t    x, y, z;
		};
	};
};
struct bo_float32x4 {
	union {
		bo_float32_t        a[4];
		struct {
			bo_float32_t    x, y, z, t;
		};
	};
};
typedef struct bo_float32x1         bo_float32x1_t;
typedef struct bo_float32x2         bo_float32x2_t;
typedef struct bo_float32x3         bo_float32x3_t;
typedef struct bo_float32x4         bo_float32x4_t;

	/*
	 * 64 bit float
	 */

struct bo_float64x1 {
	union {
		bo_float64_t    a[1];
		bo_float64_t    x;
	};
};
struct bo_float64x2 {
	union {
		bo_float64_t        a[2];
		struct {
			bo_float64_t    x, y;
		};
	};
};
struct bo_float64x3 {
	union {
		bo_float64_t        a[3];
		struct {
			bo_float64_t    x, y, z;
		};
	};
};
struct bo_float64x4 {
	union {
		bo_float64_t        a[4];
		struct {
			bo_float64_t    x, y, z, t;
		};
	};
};
typedef struct bo_float64x1         bo_float64x1_t;
typedef struct bo_float64x2         bo_float64x2_t;
typedef struct bo_float64x3         bo_float64x3_t;
typedef struct bo_float64x4         bo_float64x4_t;

# endif /* !defined(__BO_XFLOAT_TYPES_DEFINED__) */

/* ------------------------------------------------------------------------- */

typedef bo_intptr_t                 vipm_rref_t;


# ifndef __VIPM_POINTERS_DEFINED__
#  define __VIPM_POINTERS_DEFINED__
typedef void                       *vipm_mutptr_t;
typedef void const                 *vipm_imuptr_t;
# endif


# ifndef __VIPM_INT_TYPES_DEFINED__
#  define __VIPM_INT_TYPES_DEFINED__
typedef char signed                 vipm_schar;
typedef char unsigned               vipm_uchar;
typedef vipm_schar                  vipm_char;
typedef bo_int16_t                  vipm_short;
typedef bo_uint16_t                 vipm_ushort;
typedef bo_int32_t                  vipm_int;
typedef bo_uint32_t                 vipm_uint;
#  if (64 <= BO_WIDTHOF_MWORD)
typedef bo_int64_t                  vipm_long;
typedef bo_uint64_t                 vipm_ulong;
#  else
typedef bo_int32_t                  vipm_long;
typedef bo_uint32_t                 vipm_ulong;
#  endif
# endif /* !defined(__VIPM_INT_TYPES_DEFINED__) */


/*
 * Assume, sizeof(short) == sizeof(int16_t)
 * Assume, sizeof(int) == sizeof(int32_t)
 */
struct __shrt_size_test { char x[2 == sizeof(short) ? (int)sizeof(short) : -1]; };
struct __int_size_test  { char x[4 == sizeof(int)   ? (int)sizeof(int)   : -1]; };

/* ------------------------------------------------------------------------- */

struct vipm_complexf32 {
	bo_float32_t c_re, c_im;
};
typedef struct vipm_complexf32      vipm_complexf32_t;

struct vipm_complexf64 {
	bo_float64_t c_re, c_im;
};
typedef struct vipm_complexf64      vipm_complexf64_t;

/*
 * Base Integer/Float Types
 * @}
 */


/* ========================================================================= */

/*
 * @{
 * Missing Bo Types
 */

# if (64 <= BO_WIDTHOF_MWORD)
typedef bo_int64_t                  bo_xword_t;
typedef bo_uint64_t                 bo_uxword_t;
# else
typedef bo_int32_t                  bo_xword_t;
typedef bo_uint32_t                 bo_uxword_t;
# endif

/* ------------------------------------------------------------------------- */

# ifndef __BO_ARC4DEV_TYPE_DECLARED
#  define __BO_ARC4DEV_TYPE_DECLARED

struct bo_arc4dev {
	bo_uint8_t x[264];
};

#  ifndef __bo_arc4dev
#   define __bo_arc4dev bo_arc4dev
#  endif
struct __bo_arc4dev;
typedef struct __bo_arc4dev                 bo_arc4dev_t;

# endif /* !defined(__BO_ARC4DEV_TYPE_DECLARED) */

/* ------------------------------------------------------------------------- */

typedef bo_xword_t    (*bo_retain_fn)   __P((bo_intptr_t));
typedef bo_xword_t    (*bo_release_fn)  __P((bo_intptr_t, bo_opflags_t));
typedef bo_status_t   (*bo_dup_fn)      __P((bo_intptr_t, bo_intptr_t *, bo_opflags_t));
typedef bo_status_t   (*bo_dup3_fn)     __P((bo_intptr_t, bo_intptr_t *, bo_opflags_t, void *));


# define BoK_CNTL_NUL_CMD           0
# define BoK_CNTL_ANY_CMD           0
# define BoK_CNTL_UNK_CMD           2
# define BoK_CNTL_RETAIN            3  /* (*)(thisp, BoK_CNTL_RETAIN) */
# define BoK_CNTL_RELEASE           4  /* (*)(thisp, BoK_CNTL_RELEASE, (bo_opflags_t)? = 0) */
# define BoK_CNTL_URETAIN           5  /* (*)(thisp, BoK_CNTL_URETAIN) */
# define BoK_CNTL_URELEASE          6  /* (*)(thisp, BoK_CNTL_URELEASE, (bo_opflags_t)? = 0) */
# define BoK_CNTL_GETIF             7
# define BoK_CNTL_SETIF             8
# define BoK_CNTL_IFNAME            9

typedef bo_intptr_t   (*bo_control_fn)  __P((bo_intptr_t, int aCmd, ...));


typedef bo_uint32_t   (*bo_hash32_fn)   __P((bo_intptr_t));

typedef bo_mismatch_t (*bo_compare_fn)  __P((bo_intptr_t, bo_intptr_t, bo_opflags_t));
typedef bo_mismatch_t (*bo_compare2_fn) __P((bo_intptr_t, bo_intptr_t));
typedef bo_mismatch_t (*bo_compare3_fn) __P((bo_intptr_t, bo_intptr_t, void *));
typedef bo_mismatch_t (*bo_compare4_fn) __P((bo_intptr_t, bo_intptr_t, bo_opflags_t, void *));

/*
 * Missing Bo Types
 * @}
 */

#endif /* !defined(__VIPM__TYPES_BASE__TYPES_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
