/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Core_Consts.h	(V.Sokur)
 * Vipm/_Types/Core_Consts.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES_CORE__CONSTS_H__
# define __VIPM__TYPES_CORE__CONSTS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif
 */


/******************************************************************************
 * %% BeginSection: constant declarations
 */

/*
 * @{
 * Core constants/macros
 */

/* ----------------------------------------------------------------------------
 * @{
 * Border Type constants
 */
# pragma mark -
# pragma mark [ Border type constants ]
# pragma mark -

/*
 * reflect101: ... babcdefghgfedcb | abcdefgh | gfedcbabcdefghg ...
 * reflect   : ... bcdefghhgfedcba | abcdefgh | hgfedcbaabcdefg ...
 * replicate : ... aaaaaaaaaaaaaaa | abcdefgh | hhhhhhhhhhhhhhh ...
 * wrap      : ... bcdefghabcdefgh | abcdefgh | abcdefghabcdefg ...
 * const     : ... iiiiiiiiiiiiiii | abcdefgh | iiiiiiiiiiiiiii ...
 */

# define VipmF_BORDER_MASK              0x0F

# define VipmK_INVAL_BORDER           (-1)
# define VipmK_NULL_BORDER              0
# define VipmK_UNK_BORDER               2
# define VipmK_CONST_BORDER             3
# define VipmK_REPLICATE_BORDER         4
# define VipmK_WRAP_BORDER              5
# define VipmK_REFLECT_BORDER           6
# define VipmK_REFLECT101_BORDER        7

# ifndef VipmK_DEFAULT_BORDER
#  define VipmK_DEFAULT_BORDER          VipmK_REFLECT101_BORDER
# endif

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Interpolation-Method constants
 *
 * @note: DO NOT MODIFY ORDER!!!
 */
# pragma mark -
# pragma mark [ Interpolation-Method constants ]
# pragma mark -

# define VipmF_INTERP_MTDM              0x0F

# define VipmK_INVAL_INTERP           (-1)
# define VipmK_NULL_INTERP              0
# define VipmK_UNK_INTERP               2
# define VipmK_NN_INTERP                3 /* nearest neighbourhood */
# define VipmK_LINEAR_INTERP            4
# define VipmK_CUBIC_INTERP             5
# define VipmK_LANCZOS_INTERP           6
# define VipmK_FAST_INTERP              14
# define VipmK_BEST_INTERP              15

# ifndef VipmK_DEFAULT_INTERP
#  define VipmK_DEFAULT_INTERP          VipmK_CUBIC_INTERP
# endif

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Filter-Type constants
 */
# pragma mark -
# pragma mark [ Filter-Type constants ]
# pragma mark -

# define VipmF_FILTER_TYPM              0xFF

	/*
	 * Standard [base] filters
	 */
# define VipmK_INVAL_FILTER           (-1)
# define VipmK_NULL_FILTER              0
# define VipmK_UNK_FILTER               2
# define VipmK_MIN_FILTER               3 /* erode-morphop */
# define VipmK_MAX_FILTER               4 /* dilate-morphop */
# define VipmK_BOX_FILTER               5
# define VipmK_MEDIAN_FILTER            6

	/*
	 * Non-Standard filters
	 */
# define VipmK_CLOSE_FILTER             105 /* d = close(s,k) = erode(dilate(s,k)) */
# define VipmK_OPEN_FILTER              106 /* d = open(s,k) = dilate(erode(s,k)) */
# define VipmK_BOTHAT_FILTER            107 /* d = close(s,k) - s */
# define VipmK_TOPHAT_FILTER            108 /* d = s - open(s,k) */
# define VipmK_GRADIENT_FILTER          109 /* d = morph_gradient(s,k) = dilate(s,k) - erode(s,k) */

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Morphology-Op constants
 */
# pragma mark -
# pragma mark [ Morphology-Op constants ]
# pragma mark -

# define VipmF_MORPH_OPM                0x0F

	/*
	 * operation category
	 */
# define VipmK_INVAL_MORPH            (-1)
# define VipmK_NULL_MORPH               0
# define VipmK_UNK_MORPH                2

	/*
	 * basic morphological operation
	 */
# define VipmK_ERODE_MORPH              3 /* min-filter */
# define VipmK_DILATE_MORPH             4 /* max-filter */

	/*
	 * advanced morphological operation
	 */
# define VipmK_CLOSE_MORPH              5 /* d = close(s,k) = erode(dilate(s,k)) */
# define VipmK_OPEN_MORPH               6 /* d = open(s,k) = dilate(erode(s,k)) */
# define VipmK_BOTHAT_MORPH             7 /* d = close(s,k) - s */
# define VipmK_TOPHAT_MORPH             8 /* d = s - open(s,k) */
# define VipmK_GRADIENT_MORPH           9 /* d = morph_gradient(s,k) = dilate(s,k) - erode(s,k) */

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Firefly-Type constants
 */
# pragma mark -
# pragma mark [ Firefly-Type constants ]
# pragma mark -

# define VipmF_FIREFLY_TYPM             0xFF

# define VipmK_INVAL_FIREFLY          (-1)
# define VipmK_NULL_FIREFLY             0
# define VipmK_UNK_FIREFLY              2
# define VipmK_CRB_FIREFLY              3 /* calculating using countrelbox and integral histogram */
/* d = norm(w[0](s) + w[1](s*histscl) + w[2](cntrelbox(s)))
 * se[0] ignores(considered to be (1,1))
 * se[1] uses as coefficients for thresh (see below)
 * se[2] uses in countrelbox
 * histscl = max(et(s)) / idx, where ihist(s)[last] * thresh < ihist(s)[idx]
 * thresh = anchor(se[1])[0] / volume(se[1])[0]
 */
# define VipmK_CRBLN_FIREFLY            4 /* CRB: use w[i](ln(s)-ln(f[i](s))) instead w[i](f[i](s)) */
# define VipmK_GAUSS_FIREFLY            5 /* calculating using gauss filter with different kernels */
/* d = norm(sum{i=0..n}(w[i](ln(s)-ln(gaussf[i](s))))) */

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Options:
 * 1. Common options (mask: 0x000000FF)
 * 2. Specific options (mask: 0xFFFFFF00)
 */
# pragma mark -
# pragma mark [ Option constants ]
# pragma mark -

/*
 * Common options (mask: 0x000000FF)
 */

# define VipmF_CMN_OPTSM                0x000000FF
/* Reserved for channel bitmap @{ */
/* @sa: VIPM_ADD_CNBM */
# define VipmF_CMNOPTS_RESERVED1        0x10
# define VipmF_CMNOPTS_RESERVED2        0x20
# define VipmF_CMNOPTS_RESERVED3        0x40
# define VipmF_CMNOPTS_RESERVED4        0x80
/* @} */

/* @note: NOT FOR YOUR BUSINESS
   internal option */
# define __VipmF_TEST_MODE              0x01

# define VipmF_NOSIMD                   0x02
# define VipmF_FORCE_SIMD               0x04 /* 'vipm_use_simd()' doesn't test this flag;
                                                The flag intended to be used by alg. impl. directly */


/*
 * Specific options (mask: 0xFFFFFF00)
 */

# define VipmF_SPEC_OPTSM               0xFFFFFF00

/* dest */
# define VipmF_PAD_OPTSM                0x00000300
# define VipmF_MUTABLE_PADDATA          0x00000100 /* array values in range [width, stride:width+padding] can be modified */
# define VipmF_PINTPADDING              0x00000200 /* pint-padding permitted */


# define VipmF_MISC_OPTSM               0x0000F000

	/*
	 * type-conversion, ...
	 */
# define VipmF_WITH_ROUND               0x00001000

	/*
	 * copy-make-border
	 */
# define VipmF_MKBRONLY                 0x00002000


	/*
	 * operations with mask
	 */
# define VipmF_MASK_OPTSM               0x000F0000
# define _VipmF_PER_ELEMENT_MASK        0x00010000
# define VipmF_COMPLEMENTARY_MASK       0x00020000
# define VipmF_FORCE_CONT_MASK          0x00040000                  /* force-continuous mask */
# define VipmF_NOWHERE_CONT_MASK        0x00080000                  /* nowhere-continuous mask */
# define VipmF_NO_MASK_ANALYSIS         VipmF_NOWHERE_CONT_MASK

	/*
	 * operations accuracy
	 */
# define VipmF_ACCURACY_OPTSM           0x00F00000
# define VipmF_FAST_ACCURATE_ALG        0x00000000
# define VipmF_ACCURATE_ALG             0x00100000
# define VipmF_FAST_ALG                 0x00200000
# define VipmF_SUPERFAST_ALG            0x00400000

	/*
	 * filter
	 */
# define VipmF_ISOLATED_BORDER          0x01000000 /* do not look outside of the input ROI */
# define VipmF_ISOLATED_PARENT          0x02000000 /* do not look outside of the given array data */

	/*
	 * [adaptive] threshold (by blur)
	 */
# define VipmF_TRANSPARENT_BORDER       0x04000000 /* if possible, look outside of the input ROI */
# define VipmF_TRANSPARENT_PARENT       0x08000000 /* if possible, look outside of the given array data */

	/*
	 * sum with mask
	 */
# define VipmF_COUNT_TRUE_PIXELS        0x10000000

	/* @todo: fixme - remove */
	/*
	 * matrix multiply
	 */
# define VipmF_TRANSPOSED_MULTIPLICAND  0x20000000
# define _VipmF_SRC2_LOCAL_ALLOCATED    0x40000000

# define _VipmK_MATMUL_ITRN_KBLOCK_MIN  32
# define _VipmK_MATMUL_ITRN_MBLOCK_MIN  16

# define _VipmK_MATMUL_BASE_IMPALG      0
# define _VipmK_MATMUL_ITRN_IMPALG      1

/* @} */

/*
 * Core constants/macros
 * @}
 */

#endif /* !defined(__VIPM__TYPES_CORE__CONSTS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
