/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Core_Types.h	(V.Sokur)
 * Vipm/_Types/Core_Types.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES_CORE__TYPES_H__
# define __VIPM__TYPES_CORE__TYPES_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif

# ifndef __VIPM__TYPES_BASE__TYPES_H__
#  include "Vipm/_Types/Base_Types.h"
# endif
# ifndef __VIPM__TYPES_IMEL__TYPES_H__
#  include "Vipm/_Types/Imel_Types.h"
# endif

# ifndef __VIPM__TYPES_SCALAR_H__
#  include "Vipm/_Types/Scalar.h"
# endif
 */


/******************************************************************************
 * %% BeginSection: type/constant declarations / definitions
 */

/*
 * @{
 * Core type definitions
 */


/* ============================================================================
 * @{
 * 2D-geometry
 */
# pragma mark -
# pragma mark [ geometry_2D ]
# pragma mark -

/*
 * Point
 */

struct _Vipm_2d_point {
	union {
		vipm_int                __v[2];
		struct {
			vipm_int            pi_y, pi_x;
		};
	};
};
typedef struct _Vipm_2d_point           vipm_2d_point_t;
# define vipm_2d_point_p(x)             vipm_2d_point_cp(x)
# define vipm_2d_point_cp(x)           ((struct _Vipm_2d_point const *)(x))
# define vipm_2d_point_mp(x)           ((struct _Vipm_2d_point *)(x))
# define vipm_2d_point(...)             VIPM_CTOR_FCNAME(vipm_2d_point, __VA_ARGS__)
# define vipm_2d_point_(...)            VIPM_PFX_FCNAME(vipm_2d_point, __VA_ARGS__)
# define vipm_2d_pt_(...)               VIPM_PFX_FCNAME(vipm_2d_pt, __VA_ARGS__)

struct _Vipm_f2d_point {
	union {
		bo_float32_t            __v[2];
		struct {
			bo_float32_t        pi_y, pi_x;
		};
	};
};
typedef struct _Vipm_f2d_point          vipm_f2d_point_t;
# define vipm_f2d_point_p(x)            vipm_f2d_point_cp(x)
# define vipm_f2d_point_cp(x)          ((struct _Vipm_f2d_point const *)(x))
# define vipm_f2d_point_mp(x)          ((struct _Vipm_f2d_point *)(x))
# define vipm_f2d_point(...)            VIPM_CTOR_FCNAME(vipm_f2d_point, __VA_ARGS__)
# define vipm_f2d_point_(...)           VIPM_PFX_FCNAME(vipm_f2d_point, __VA_ARGS__)
# define vipm_f2d_pt_(...)              VIPM_PFX_FCNAME(vipm_f2d_pt, __VA_ARGS__)

struct _Vipm_r2d_point {
	union {
		bo_float64_t            __v[2];
		struct {
			bo_float64_t        pi_y, pi_x;
		};
	};
};
typedef struct _Vipm_r2d_point          vipm_r2d_point_t;
# define vipm_r2d_point_p(x)            vipm_r2d_point_cp(x)
# define vipm_r2d_point_cp(x)          ((struct _Vipm_r2d_point const *)(x))
# define vipm_r2d_point_mp(x)          ((struct _Vipm_r2d_point *)(x))
# define vipm_r2d_point(...)            VIPM_CTOR_FCNAME(vipm_r2d_point, __VA_ARGS__)
# define vipm_r2d_point_(...)           VIPM_PFX_FCNAME(vipm_r2d_point, __VA_ARGS__)
# define vipm_r2d_pt_(...)              VIPM_PFX_FCNAME(vipm_r2d_pt, __VA_ARGS__)


# define vipm_Point_t(_Space)           VIPM_GLUE3(vipm_,_Space,_point_t)
# define vipm_Point_p(_Space)           VIPM_GLUE3(vipm_,_Space,_point_p)
# define vipm_Point_cp(_Space)          VIPM_GLUE3(vipm_,_Space,_point_cp)
# define vipm_Point_mp(_Space)          VIPM_GLUE3(vipm_,_Space,_point_mp)
# define vipm_Point(_Space)             VIPM_GLUE3(vipm_,_Space,_point)
# define vipm_Point_(_Space)            VIPM_GLUE3(vipm_,_Space,_point_)

/* short versions */
# define vipm_pt_(...)                  VIPM_PFX_FCNAME(vipm_pt, __VA_ARGS__)
# define vipm_2d_pt_(...)               VIPM_PFX_FCNAME(vipm_2d_pt, __VA_ARGS__)
# define vipm_f2d_pt_(...)              VIPM_PFX_FCNAME(vipm_f2d_pt, __VA_ARGS__)
# define vipm_r2d_pt_(...)              VIPM_PFX_FCNAME(vipm_r2d_pt, __VA_ARGS__)
# define vipm_Pt_(_Space)               VIPM_GLUE3(vipm_,_Space,_pt_)


/* ------------------------------------------------------------------------- */

/*
 * Size
 */

struct _Vipm_2d_size {
	union {
		vipm_int                __v[2];
		struct {
			vipm_int            sz_h, sz_w;
		};
	};
};
typedef struct _Vipm_2d_size            vipm_2d_size_t;
# define vipm_2d_size_p(x)              vipm_2d_size_cp(x)
# define vipm_2d_size_cp(x)            ((struct _Vipm_2d_size const *)(x))
# define vipm_2d_size_mp(x)            ((struct _Vipm_2d_size *)(x))
# define vipm_2d_size(...)              VIPM_CTOR_FCNAME(vipm_2d_size, __VA_ARGS__)
# define vipm_2d_size_(...)             VIPM_PFX_FCNAME(vipm_2d_size, __VA_ARGS__)

struct _Vipm_f2d_size {
	union {
		bo_float32_t            __v[2];
		struct {
			bo_float32_t        sz_h, sz_w;
		};
	};
};
typedef struct _Vipm_f2d_size           vipm_f2d_size_t;
# define vipm_f2d_size_p(x)             vipm_f2d_size_cp(x)
# define vipm_f2d_size_cp(x)           ((struct _Vipm_f2d_size const *)(x))
# define vipm_f2d_size_mp(x)           ((struct _Vipm_f2d_size *)(x))
# define vipm_f2d_size(...)             VIPM_CTOR_FCNAME(vipm_f2d_size, __VA_ARGS__)
# define vipm_f2d_size_(...)            VIPM_PFX_FCNAME(vipm_f2d_size, __VA_ARGS__)

struct _Vipm_r2d_size {
	union {
		bo_float64_t            __v[2];
		struct {
			bo_float64_t        sz_h, sz_w;
		};
	};
};
typedef struct _Vipm_r2d_size           vipm_r2d_size_t;
# define vipm_r2d_size_p(x)             vipm_r2d_size_cp(x)
# define vipm_r2d_size_cp(x)           ((struct _Vipm_r2d_size const *)(x))
# define vipm_r2d_size_mp(x)           ((struct _Vipm_r2d_size *)(x))
# define vipm_r2d_size(...)             VIPM_CTOR_FCNAME(vipm_r2d_size, __VA_ARGS__)
# define vipm_r2d_size_(...)            VIPM_PFX_FCNAME(vipm_r2d_size, __VA_ARGS__)


# define vipm_Size_t(_Space)            VIPM_GLUE3(vipm_,_Space,_size_t)
# define vipm_Size_p(_Space)            VIPM_GLUE3(vipm_,_Space,_size_p)
# define vipm_Size_cp(_Space)           VIPM_GLUE3(vipm_,_Space,_size_cp)
# define vipm_Size_mp(_Space)           VIPM_GLUE3(vipm_,_Space,_size_mp)
# define vipm_Size(_Space)              VIPM_GLUE3(vipm_,_Space,_size)
# define vipm_Size_(_Space)             VIPM_GLUE3(vipm_,_Space,_size_)

/* short versions */
# define vipm_sz_(...)                  VIPM_PFX_FCNAME(vipm_sz, __VA_ARGS__)
# define vipm_2d_sz_(...)               VIPM_PFX_FCNAME(vipm_2d_sz, __VA_ARGS__)
# define vipm_f2d_sz_(...)              VIPM_PFX_FCNAME(vipm_f2d_sz, __VA_ARGS__)
# define vipm_r2d_sz_(...)              VIPM_PFX_FCNAME(vipm_r2d_sz, __VA_ARGS__)
# define vipm_Sz_(_Space)               VIPM_GLUE3(vipm_,_Space,_sz_)


/* ------------------------------------------------------------------------- */

/*
 * Rect
 */

struct _Vipm_2d_rect {
	union {
		vipm_int                __v[4];

		/* rcpt: {top-left-point, bottom-right-point} @{ */
		struct {
			vipm_int            rc_top,    rc_left;
			vipm_int            rc_bottom, rc_right;
		};
		struct {
			vipm_2d_point_t     rc_lt;
			vipm_2d_point_t     rc_rb;
		};
		vipm_2d_point_t         rc_ptv[2];
		/* @} */

		/* rect: {top-left-point, size{height,width}} @{ */
		struct {
			vipm_int            rc_y, rc_x;
			vipm_int            rc_h, rc_w;
		};
		struct {
			vipm_2d_point_t     rc_xy;
			vipm_2d_size_t      rc_ext;
		};
		/* @} */
	};
};
typedef struct _Vipm_2d_rect            vipm_2d_rect_t;
# define vipm_2d_rect_p(x)              vipm_2d_rect_cp(x)
# define vipm_2d_rect_cp(x)            ((struct _Vipm_2d_rect const *)(x))
# define vipm_2d_rect_mp(x)            ((struct _Vipm_2d_rect *)(x))
# define vipm_2d_rect(...)              VIPM_CTOR_FCNAME(vipm_2d_rect, __VA_ARGS__)
# define vipm_2d_rect_(...)             VIPM_PFX_FCNAME(vipm_2d_rect, __VA_ARGS__)

# define _Vipm_2d_rcpt                  _Vipm_2d_rect
# define vipm_2d_rcpt_t                 vipm_2d_rect_t
# define vipm_2d_rcpt_p(x)              vipm_2d_rect_p(x)
# define vipm_2d_rcpt_cp(x)             vipm_2d_rect_cp(x)
# define vipm_2d_rcpt_mp(x)             vipm_2d_rect_mp(x)
# define vipm_2d_rcpt(...)              VIPM_CTOR_FCNAME(vipm_2d_rcpt, __VA_ARGS__)
# define vipm_2d_rcpt_(...)             VIPM_PFX_FCNAME(vipm_2d_rcpt, __VA_ARGS__)


struct _Vipm_f2d_rect {
	union {
		bo_float32_t            __v[4];

		/* rcpt: {top-left-point, bottom-right-point} @{ */
		struct {
			bo_float32_t        rc_top,    rc_left;
			bo_float32_t        rc_bottom, rc_right;
		};
		struct {
			vipm_f2d_point_t    rc_lt;
			vipm_f2d_point_t    rc_rb;
		};
		vipm_f2d_point_t        rc_ptv[2];
		/* @} */

		/* rect: {top-left-point, size{height,width}} @{ */
		struct {
			bo_float32_t        rc_y, rc_x;
			bo_float32_t        rc_h, rc_w;
		};
		struct {
			vipm_f2d_point_t    rc_xy;
			vipm_f2d_size_t     rc_ext;
		};
		/* @} */
	};
};
typedef struct _Vipm_f2d_rect           vipm_f2d_rect_t;
# define vipm_f2d_rect_p(x)             vipm_f2d_rect_cp(x)
# define vipm_f2d_rect_cp(x)           ((struct _Vipm_f2d_rect const *)(x))
# define vipm_f2d_rect_mp(x)           ((struct _Vipm_f2d_rect *)(x))
# define vipm_f2d_rect(...)             VIPM_CTOR_FCNAME(vipm_f2d_rect, __VA_ARGS__)
# define vipm_f2d_rect_(...)            VIPM_PFX_FCNAME(vipm_f2d_rect, __VA_ARGS__)

# define _Vipm_f2d_rcpt                 _Vipm_f2d_rect
# define vipm_f2d_rcpt_t                vipm_f2d_rect_t
# define vipm_f2d_rcpt_p(x)             vipm_f2d_rect_p(x)
# define vipm_f2d_rcpt_cp(x)            vipm_f2d_rect_cp(x)
# define vipm_f2d_rcpt_mp(x)            vipm_f2d_rect_mp(x)
# define vipm_f2d_rcpt(...)             VIPM_CTOR_FCNAME(vipm_f2d_rcpt, __VA_ARGS__)
# define vipm_f2d_rcpt_(...)            VIPM_PFX_FCNAME(vipm_f2d_rcpt, __VA_ARGS__)


struct _Vipm_r2d_rect {
	union {
		bo_float64_t            __v[4];

		/* rcpt: {top-left-point, bottom-right-point} @{ */
		struct {
			bo_float64_t        rc_top,    rc_left;
			bo_float64_t        rc_bottom, rc_right;
		};
		struct {
			vipm_r2d_point_t    rc_lt;
			vipm_r2d_point_t    rc_rb;
		};
		vipm_r2d_point_t        rc_ptv[2];
		/* @} */

		/* rect: {top-left-point, size{height,width}} @{ */
		struct {
			bo_float64_t        rc_y, rc_x;
			bo_float64_t        rc_h, rc_w;
		};
		struct {
			vipm_r2d_point_t    rc_xy;
			vipm_r2d_size_t     rc_ext;
		};
		/* @} */
	};
};
typedef struct _Vipm_r2d_rect           vipm_r2d_rect_t;
# define vipm_r2d_rect_p(x)             vipm_r2d_rect_cp(x)
# define vipm_r2d_rect_cp(x)           ((struct _Vipm_r2d_rect const *)(x))
# define vipm_r2d_rect_mp(x)           ((struct _Vipm_r2d_rect *)(x))
# define vipm_r2d_rect(...)             VIPM_CTOR_FCNAME(vipm_r2d_rect, __VA_ARGS__)
# define vipm_r2d_rect_(...)            VIPM_PFX_FCNAME(vipm_r2d_rect, __VA_ARGS__)

# define _Vipm_r2d_rcpt                 _Vipm_r2d_rect
# define vipm_r2d_rcpt_t                vipm_r2d_rect_t
# define vipm_r2d_rcpt_p(x)             vipm_r2d_rect_p(x)
# define vipm_r2d_rcpt_cp(x)            vipm_r2d_rect_cp(x)
# define vipm_r2d_rcpt_mp(x)            vipm_r2d_rect_mp(x)
# define vipm_r2d_rcpt(...)             VIPM_CTOR_FCNAME(vipm_r2d_rcpt, __VA_ARGS__)
# define vipm_r2d_rcpt_(...)            VIPM_PFX_FCNAME(vipm_r2d_rcpt, __VA_ARGS__)


# define vipm_Rect_t(_Space)            VIPM_GLUE3(vipm_,_Space,_rect_t)
# define vipm_Rect_p(_Space)            VIPM_GLUE3(vipm_,_Space,_rect_p)
# define vipm_Rect_cp(_Space)           VIPM_GLUE3(vipm_,_Space,_rect_cp)
# define vipm_Rect_mp(_Space)           VIPM_GLUE3(vipm_,_Space,_rect_mp)
# define vipm_Rect(_Space)              VIPM_GLUE3(vipm_,_Space,_rect)
# define vipm_Rect_(_Space)             VIPM_GLUE3(vipm_,_Space,_rect_)

# define vipm_Rcpt_t(_Space)            VIPM_GLUE3(vipm_,_Space,_rcpt_t)
# define vipm_Rcpt_p(_Space)            VIPM_GLUE3(vipm_,_Space,_rcpt_p)
# define vipm_Rcpt_cp(_Space)           VIPM_GLUE3(vipm_,_Space,_rcpt_cp)
# define vipm_Rcpt_mp(_Space)           VIPM_GLUE3(vipm_,_Space,_rcpt_mp)
# define vipm_Rcpt(_Space)              VIPM_GLUE3(vipm_,_Space,_rcpt)
# define vipm_Rcpt_(_Space)             VIPM_GLUE3(vipm_,_Space,_rcpt_)


/* ------------------------------------------------------------------------- */

/*
 * Quad
 */

struct _Vipm_f2d_quad {
	union {
		bo_float32_t            __v[8];
		vipm_f2d_point_t        q_points[4];
	};
};
typedef struct _Vipm_f2d_quad           vipm_f2d_quad_t;
# define vipm_f2d_quad_p(x)             vipm_f2d_quad_cp(x)
# define vipm_f2d_quad_cp(x)           ((bo_float32_t const (*)[2])(x))
# define vipm_f2d_quad_mp(x)           ((bo_float32_t (*)[2])(x))
# define vipm_f2d_quad(...)             VIPM_CTOR_FCNAME(vipm_f2d_quad, __VA_ARGS__)
# define vipm_f2d_quad_(...)            VIPM_PFX_FCNAME(vipm_f2d_quad, __VA_ARGS__)

# define VIPM_F2D_QUAD_INITIALIZER(y0,x0, y1,x1, y2,x2, y3,x3) \
	{{{ (y0),(x0), (y1),(x1), (y2),(x2), (y3),(x3) }}}


struct _Vipm_r2d_quad {
	union {
		bo_float64_t            __v[8];
		vipm_r2d_point_t        q_points[4];
	};
};
typedef struct _Vipm_r2d_quad           vipm_r2d_quad_t;
# define vipm_r2d_quad_p(x)             vipm_r2d_quad_cp(x)
# define vipm_r2d_quad_cp(x)           ((bo_float64_t const (*)[2])(x))
# define vipm_r2d_quad_mp(x)           ((bo_float64_t (*)[2])(x))
# define vipm_r2d_quad(...)             VIPM_CTOR_FCNAME(vipm_r2d_quad, __VA_ARGS__)
# define vipm_r2d_quad_(...)            VIPM_PFX_FCNAME(vipm_r2d_quad, __VA_ARGS__)

# define VIPM_R2D_QUAD_INITIALIZER(y0,x0, y1,x1, y2,x2, y3,x3) \
	{{{ (y0),(x0), (y1),(x1), (y2),(x2), (y3),(x3) }}}


# define vipm_Quad_t(_Space)            VIPM_GLUE3(vipm_,_Space,_quad_t)
# define vipm_Quad_p(_Space)            VIPM_GLUE3(vipm_,_Space,_quad_p)
# define vipm_Quad_cp(_Space)           VIPM_GLUE3(vipm_,_Space,_quad_cp)
# define vipm_Quad_mp(_Space)           VIPM_GLUE3(vipm_,_Space,_quad_mp)
# define vipm_Quad(_Space)              VIPM_GLUE3(vipm_,_Space,_quad)
# define vipm_Quad_(_Space)             VIPM_GLUE3(vipm_,_Space,_quad_)


/* ------------------------------------------------------------------------- */

/*
 * Ellipse
 */

struct _Vipm_f2d_ellipse {
	union {
		bo_float32_t            __v[5];
		struct {
			vipm_f2d_point_t    el_c;
			bo_float32_t        el_minor, el_major;
			bo_float32_t        el_theta;
		};
	};
};
typedef struct _Vipm_f2d_ellipse        vipm_f2d_ellipse_t;
# define vipm_f2d_ellipse_p(x)          vipm_f2d_ellipse_cp(x)
# define vipm_f2d_ellipse_cp(x)        ((struct _Vipm_f2d_ellipse const *)(x))
# define vipm_f2d_ellipse_mp(x)        ((struct _Vipm_f2d_ellipse *)(x))
# define vipm_f2d_ellipse(...)          VIPM_CTOR_FCNAME(vipm_f2d_ellipse, __VA_ARGS__)
# define vipm_f2d_ellipse_(...)         VIPM_PFX_FCNAME(vipm_f2d_ellipse, __VA_ARGS__)

# define VIPM_F2D_ELLIPSE_INITIALIZER(x0,x1,x2,x3,x4) \
	{{{ (x0),(x1),(x2),(x3),(x4) }}}


struct _Vipm_r2d_ellipse {
	union {
		bo_float64_t            __v[5];
		struct {
			vipm_r2d_point_t    el_c;
			bo_float64_t        el_minor, el_major;
			bo_float64_t        el_theta;
		};
	};
};
typedef struct _Vipm_r2d_ellipse        vipm_r2d_ellipse_t;
# define vipm_r2d_ellipse_p(x)          vipm_r2d_ellipse_cp(x)
# define vipm_r2d_ellipse_cp(x)        ((struct _Vipm_r2d_ellipse const *)(x))
# define vipm_r2d_ellipse_mp(x)        ((struct _Vipm_r2d_ellipse *)(x))
# define vipm_r2d_ellipse(...)          VIPM_CTOR_FCNAME(vipm_r2d_ellipse, __VA_ARGS__)
# define vipm_r2d_ellipse_(...)         VIPM_PFX_FCNAME(vipm_r2d_ellipse, __VA_ARGS__)

# define VIPM_R2D_ELLIPSE_INITIALIZER(x0,x1,x2,x3,x4) \
	{{{ (x0),(x1),(x2),(x3),(x4) }}}


# define vipm_Ellipse_t(_Space)         VIPM_GLUE3(vipm_,_Space,_ellipse_t)
# define vipm_Ellipse_p(_Space)         VIPM_GLUE3(vipm_,_Space,_ellipse_p)
# define vipm_Ellipse_cp(_Space)        VIPM_GLUE3(vipm_,_Space,_ellipse_cp)
# define vipm_Ellipse_mp(_Space)        VIPM_GLUE3(vipm_,_Space,_ellipse_mp)
# define vipm_Ellipse(_Space)           VIPM_GLUE3(vipm_,_Space,_ellipse)
# define vipm_Ellipse_(_Space)          VIPM_GLUE3(vipm_,_Space,_ellipse_)

/* short aliases */
# define vipm_Ell_t(_Space)             VIPM_GLUE3(vipm_,_Space,_ellipse_t)
# define vipm_Ell_p(_Space)             VIPM_GLUE3(vipm_,_Space,_ellipse_p)
# define vipm_Ell_cp(_Space)            VIPM_GLUE3(vipm_,_Space,_ellipse_cp)
# define vipm_Ell_mp(_Space)            VIPM_GLUE3(vipm_,_Space,_ellipse_mp)
# define vipm_Ell(_Space)               VIPM_GLUE3(vipm_,_Space,_ellipse)
//# define vipm_Ell_(_Space)              VIPM_GLUE3(vipm_,_Space,_ellipse_)

/* short versions */
# define vipm_ell_(...)                 VIPM_PFX_FCNAME(vipm_ell, __VA_ARGS__)
# define vipm_2d_ell_(...)              VIPM_PFX_FCNAME(vipm_2d_ell, __VA_ARGS__)
# define vipm_f2d_ell_(...)             VIPM_PFX_FCNAME(vipm_f2d_ell, __VA_ARGS__)
# define vipm_r2d_ell_(...)             VIPM_PFX_FCNAME(vipm_r2d_ell, __VA_ARGS__)
# define vipm_Ell_(_Space)              VIPM_GLUE3(vipm_,_Space,_ell_)


/*
 * 2D-geometry
 * @}
 */


/* ========================================================================= */

struct _Vipm_xinterface {
	bo_uint32_t                 magic;
	bo_uint32_t                 size;
};
typedef struct _Vipm_xinterface         vipm_xinterface_t;
# define vipm_xinterface_p(x)           vipm_xinterface_cp(x)
# define vipm_xinterface_cp(x)         ((struct _Vipm_xinterface const *)(x))
# define vipm_xinterface_mp(x)         ((struct _Vipm_xinterface *)(x))


/* ------------------------------------------------------------------------- */

/*
 * Rank
 */

/* @note:
 *
 * 1) rank explanation in case of non-'.g' function:
 *              { nchannels,                rank = 1;
 * dimensions = { w, nchannels,             rank = 2;
 *              { h, w, nchannels,          rank = 3;
 *              { nplanes, h, w, nchannels, rank = 4;
 *
 * 2) rank explanation in case of '.g' function: @sa grank explanation
 *
 * 3) grank explanation:
 *              { w,                        rank = 1;
 * dimensions = { h, w,                     rank = 2;
 *              { nplanes, h, w,            rank = 3;
 *              { nplanes, h, w, nchannels, rank = 4;
 *
 */
typedef vipm_int                        vipm_rank_t;
typedef vipm_int                        vipm_grank_t;        /* should only contain 'effective' part */

# define VIPM_PRANK(x)                  (((x) >> 8) & 0x1F)  /* primary(original) rank */
# define VIPM_RRANK(x)                  (((x) >> 4) & 0xF)   /* reduction */
# define VIPM_ERANK(x)                  ((x) & 0xF)          /* effective(actual) rank */

# define VIPM_MKRANK_E0(EffectiveRank)             ((((EffectiveRank) & 0xF) << 8) | ((EffectiveRank) & 0xF))
# define VIPM_MKRANK_ER(EffectiveRank,aReduction)   VIPM__MKRANK((EffectiveRank) + (aReduction),aReduction,EffectiveRank)
# define VIPM_MKRANK_RE(aReduction,EffectiveRank)   VIPM__MKRANK((EffectiveRank) + (aReduction),aReduction,EffectiveRank)
# define VIPM_MKRANK_PR(aPrimRank,aReduction)       VIPM__MKRANK(aPrimRank,aReduction,(aPrimRank) - (aReduction))

# define VIPM__MKRANK(aPrimRank,aReduction,EffectiveRank) \
	((((aPrimRank) & 0x1F) << 8) | (((aReduction) & 0xF) << 4) | ((EffectiveRank) & 0xF))


/* ------------------------------------------------------------------------- */

/*
 * Dims
 * XDims
 */

/*
 * 0: Z | nplanes ;; -4
 * 1: Y           ;; -3
 * 2: X           ;; -2
 * 3: nchannels   ;; -1
 */

/*
 *                0(-4)        1(-3)       2(-2)         3(-1)
 * volume:    (  nplanes,      height,     width,      nchannels)
 * stride:    (plane-stride, row-stride, pixel-stride, el-stride)
 *
 * stride(0) = volume(1) * stride(1) + padding
 * padding   = stride(0) - volume(1) * stride(1)
 */


# define VipmK_MAXDIMS                  4
# define VipmK_MAXDIMS_1                3
# define VipmK_MAXDIMS_2                2
# define VipmK_MAXGDIMS                 3
# define VipmK_MAXGDIMS_1               2


struct _Vipm_dims {
	union {
		vipm_int                v[VipmK_MAXDIMS];
		bo_float32_t            vf[VipmK_MAXDIMS];
	};
};
typedef struct _Vipm_dims               vipm_dims_t;
# define vipm_dims_p(x)                 vipm_dims_cp(x)
# define vipm_dims_cp(x)               ((struct _Vipm_dims const *)(x))
# define vipm_dims_mp(x)               ((struct _Vipm_dims *)(x))
# define vipm_dims(...)                 VIPM_CTOR_FCNAME(vipm_dims, __VA_ARGS__)
# define vipm_dims_(...)                VIPM_PFX_FCNAME(vipm_dims, __VA_ARGS__)

# define VIPM_DIMS_At(x,i)             ((x).v[i])
# define VIPM_DIMS_Ate(x,i)            (((x).v + __VIPM_COUNTOF((x).v))[i])

# define vipm__pt_cn(a)                 (a)[3] /* VIPM_DIMS_Ate(a,-1) */
# define vipm__pt_x(a)                  (a)[2] /* VIPM_DIMS_Ate(a,-2) */
# define vipm__pt_y(a)                  (a)[1] /* VIPM_DIMS_Ate(a,-3) */
# define vipm__pt_z(a)                  (a)[0] /* VIPM_DIMS_Ate(a,-4) */
# define vipm__pt_plane(a)              vipm__pt_z(a)
# define vipm__pt_nplanes(a)            vipm__pt_z(a)

# define vipm_pt_cn(a)                  (a).v[3] /* VIPM_DIMS_Ate(a,-1) */
# define vipm_pt_x(a)                   (a).v[2] /* VIPM_DIMS_Ate(a,-2) */
# define vipm_pt_y(a)                   (a).v[1] /* VIPM_DIMS_Ate(a,-3) */
# define vipm_pt_z(a)                   (a).v[0] /* VIPM_DIMS_Ate(a,-4) */
# define vipm_pt_plane(a)               vipm_pt_z(a)
# define vipm_pt_nplanes(a)             vipm_pt_z(a)

# define vipm_ptf_cn(a)                 (a).vf[3]
# define vipm_ptf_x(a)                  (a).vf[2]
# define vipm_ptf_y(a)                  (a).vf[1]
# define vipm_ptf_z(a)                  (a).vf[0]
# define vipm_ptf_plane(a)              vipm_ptf_z(a)
# define vipm_ptf_nplanes(a)            vipm_ptf_z(a)

/* strides */
# define vipm_pts_el(a)                 (a).v[3] /* VIPM_DIMS_Ate(a,-1) */
# define vipm_pts_x(a)                  (a).v[2] /* VIPM_DIMS_Ate(a,-2) */
# define vipm_pts_y(a)                  (a).v[1] /* VIPM_DIMS_Ate(a,-3) */
# define vipm_pts_z(a)                  (a).v[0] /* VIPM_DIMS_Ate(a,-4) */
# define vipm_pts_pix(a)                vipm_pts_x(a)
# define vipm_pts_plane(a)              vipm_pts_z(a)

typedef char __STATIC_ASSERT_MAXDIMS[VipmK_MAXDIMS == 4 ? 1 : -1];


struct _Vipm_xdims {
	union {
		vipm_int                v[VipmK_MAXDIMS];
		bo_float32_t            vf[VipmK_MAXDIMS];
	};
	vipm_dims_t                 xd_strides;
};
typedef struct _Vipm_xdims              vipm_Xdims_t;
# define vipm_xdims_p(x)                vipm_xdims_cp(x)
# define vipm_xdims_cp(x)              ((struct _Vipm_xdims const *)(x))
# define vipm_xdims_mp(x)              ((struct _Vipm_xdims *)(x))
# define vipm_xdims(...)                VIPM_CTOR_FCNAME(vipm_xdims, __VA_ARGS__)
# define vipm_xdims_(...)               VIPM_PFX_FCNAME(vipm_xdims, __VA_ARGS__)

/* strides */
# define vipm_xpts_el(a)                vipm_pts_el((a).xd_strides)
# define vipm_xpts_x(a)                 vipm_pts_x((a).xd_strides)
# define vipm_xpts_y(a)                 vipm_pts_y((a).xd_strides)
# define vipm_xpts_z(a)                 vipm_pts_z((a).xd_strides)
# define vipm_xpts_pix(a)               vipm_pts_pix((a).xd_strides)   /* ~ vipm_xpts_x */
# define vipm_xpts_plane(a)             vipm_pts_plane((a).xd_strides) /* ~ vipm_xpts_z */


	/*
	 * Npoint
	 * Nvector
	 * Nvolume
	 */

typedef struct _Vipm_dims               vipm_Npoint_t;
typedef struct _Vipm_dims               vipm_Nvector_t, vipm_Nvolume_t;

# define vipm_Npoint_p(x)               vipm_Npoint_cp(x)
# define vipm_Npoint_cp(x)             ((struct _Vipm_Npoint const *)(x))
# define vipm_Npoint_mp(x)             ((struct _Vipm_Npoint *)(x))
# define vipm_Npoint(...)               VIPM_CTOR_FCNAME(vipm_Npoint, __VA_ARGS__)
# define vipm_Npoint_(...)              VIPM_PFX_FCNAME(vipm_Npoint, __VA_ARGS__)

# define vipm_Nvector_p(x)              vipm_Nvector_cp(x)
# define vipm_Nvector_cp(x)            ((struct _Vipm_Nvector const *)(x))
# define vipm_Nvector_mp(x)            ((struct _Vipm_Nvector *)(x))
# define vipm_Nvector(...)              VIPM_CTOR_FCNAME(vipm_Nvector, __VA_ARGS__)
# define vipm_Nvector_(...)             VIPM_PFX_FCNAME(vipm_Nvector, __VA_ARGS__)

# define vipm_Nvolume_p(x)              vipm_Nvolume_cp(x)
# define vipm_Nvolume_cp(x)            ((struct _Vipm_Nvolume const *)(x))
# define vipm_Nvolume_mp(x)            ((struct _Vipm_Nvolume *)(x))
# define vipm_Nvolume(...)              VIPM_CTOR_FCNAME(vipm_Nvolume, __VA_ARGS__)
# define vipm_Nvolume_(...)             VIPM_PFX_FCNAME(vipm_Nvolume, __VA_ARGS__)


	/*
	 * Xpoint
	 * Xvector
	 * Xvolume
	 */

typedef struct _Vipm_xdims              vipm_Xpoint_t;
typedef struct _Vipm_xdims              vipm_Xvector_t, vipm_Xvolume_t;

# define vipm_Xpoint_p(x)               vipm_Xpoint_cp(x)
# define vipm_Xpoint_cp(x)             ((struct _Vipm_Npoint const *)(x))
# define vipm_Xpoint_mp(x)             ((struct _Vipm_Npoint *)(x))
# define vipm_Xpoint(...)               VIPM_CTOR_FCNAME(vipm_Xpoint, __VA_ARGS__)
# define vipm_Xpoint_(...)              VIPM_PFX_FCNAME(vipm_Xpoint, __VA_ARGS__)

# define vipm_Xvector_p(x)              vipm_Xvector_cp(x)
# define vipm_Xvector_cp(x)            ((struct _Vipm_Nvector const *)(x))
# define vipm_Xvector_mp(x)            ((struct _Vipm_Nvector *)(x))
# define vipm_Xvector(...)              VIPM_CTOR_FCNAME(vipm_Xvector, __VA_ARGS__)
# define vipm_Xvector_(...)             VIPM_PFX_FCNAME(vipm_Xvector, __VA_ARGS__)

# define vipm_Xvolume_p(x)              vipm_Xvolume_cp(x)
# define vipm_Xvolume_cp(x)            ((struct _Vipm_Nvolume const *)(x))
# define vipm_Xvolume_mp(x)            ((struct _Vipm_Nvolume *)(x))
# define vipm_Xvolume(...)              VIPM_CTOR_FCNAME(vipm_Xvolume, __VA_ARGS__)
# define vipm_Xvolume_(...)             VIPM_PFX_FCNAME(vipm_Xvolume, __VA_ARGS__)


/* ------------------------------------------------------------------------- */

/*
 * Site
 * Xsite
 */

struct _Vipm_site {
	vipm_Npoint_t               si_point;
	vipm_Nvolume_t              si_volume;
};
typedef struct _Vipm_site               vipm_site_t;
# define vipm_site_p(x)                 vipm_site_cp(x)
# define vipm_site_cp(x)               ((struct _Vipm_site const *)(x))
# define vipm_site_mp(x)               ((struct _Vipm_site *)(x))
# define vipm_site(...)                 VIPM_CTOR_FCNAME(vipm_site, __VA_ARGS__)
# define vipm_site_(...)                VIPM_PFX_FCNAME(vipm_site, __VA_ARGS__)

struct _Vipm_xsite {
	vipm_Npoint_t               si_point;
	vipm_Nvolume_t              si_volume;
	vipm_dims_t                 si_strides;
};
typedef struct _Vipm_xsite              vipm_xsite_t;
# define vipm_xsite_p(x)                vipm_xsite_cp(x)
# define vipm_xsite_cp(x)              ((struct _Vipm_xsite const *)(x))
# define vipm_xsite_mp(x)              ((struct _Vipm_xsite *)(x))
# define vipm_xsite(...)                VIPM_CTOR_FCNAME(vipm_xsite, __VA_ARGS__)
# define vipm_xsite_(...)               VIPM_PFX_FCNAME(vipm_xsite, __VA_ARGS__)


/* ------------------------------------------------------------------------- */

/*
 * Range
 */

struct _Vipm_range {
	vipm_scalar_t   val[2];
};
typedef struct _Vipm_range              vipm_range_t;
# define vipm_range_p(x)                vipm_range_cp(x)
# define vipm_range_cp(x)              ((struct _Vipm_range const *)(x))
# define vipm_range_mp(x)              ((struct _Vipm_range *)(x))
# define vipm_range(...)                VIPM_CTOR_FCNAME(vipm_range, __VA_ARGS__)
# define vipm_vrng(...)                 VIPM_CTOR_FCNAME(vipm_vrng, __VA_ARGS__)
# define vipm_vrng_(...)                VIPM_PFX_FCNAME(vipm_vrng, __VA_ARGS__)
# define vipm_vrngv(...)                VIPM_CTOR_FCNAME(vipm_vrngv, __VA_ARGS__)
# define vipm_vrngv_(...)               VIPM_PFX_FCNAME(vipm_vrngv, __VA_ARGS__)


	/*
	 * Rngpair: range pair
	 * @sa: _Vipm_rngscl2
	 */

struct _Vipm_rngpair {
	union {
		vipm_range_t const     *__imuv[2];
		vipm_range_t           *__mutv[2];
		struct {
			vipm_range_t const     *first,
	                               *second;
		} imu;
		struct {
			vipm_range_t           *first,
	                               *second;
		} mut;
	};
};
typedef struct _Vipm_rngpair            vipm_rngpair_t;
# define vipm_rngpair_p(x)              vipm_rngpair_cp(x)
# define vipm_rngpair_cp(x)            ((struct _Vipm_rngpair const *)(x))
# define vipm_rngpair_mp(x)            ((struct _Vipm_rngpair *)(x))
# define vipm_rngpair(...)              VIPM_CTOR_FCNAME(vipm_rngpair, __VA_ARGS__)
# define vipm_rngpair_(...)             VIPM_PFX_FCNAME(vipm_rngpair, __VA_ARGS__)


typedef struct { bo_float32_t val[2]; } vipm_rngf32_t;
# define vipm_rngf32_p(x)               vipm_rngf32_cp(x)
# define vipm_rngf32_cp(x)             ((vipm_rngf32_t const *)(x))
# define vipm_rngf32_mp(x)             ((vipm_rngf32_t *)(x))
# define vipm_rngf32(...)               VIPM_CTOR_FCNAME(vipm_rngf32, __VA_ARGS__)
# define vipm_rngf32_(...)              VIPM_PFX_FCNAME(vipm_rngf32, __VA_ARGS__)

# define VIPM_RNGF32_INITIALIZER(x0,x1) {{ (x0),(x1) }}


typedef struct { bo_float64_t val[2]; } vipm_rngf64_t;
# define vipm_rngf64_p(x)               vipm_rngf64_cp(x)
# define vipm_rngf64_cp(x)             ((vipm_rngf64_t const *)(x))
# define vipm_rngf64_mp(x)             ((vipm_rngf64_t *)(x))
# define vipm_rngf64(...)               VIPM_CTOR_FCNAME(vipm_rngf64, __VA_ARGS__)
# define vipm_rngf64_(...)              VIPM_PFX_FCNAME(vipm_rngf64, __VA_ARGS__)

# define VIPM_RNGF64_INITIALIZER(x0,x1) {{ (x0),(x1) }}


	/*
	 * Rngscl2: range + 2 scalars
	 * @sa: _Vipm_rngpair
	 */

struct _Vipm_rngscl2 {
	union {
		struct {
			vipm_range_t const     *r;
			vipm_scalar_t const    *k, *b;
		} imu;
		struct {
			vipm_range_t           *r;
			vipm_scalar_t          *k, *b;
		} mut;
	};
};
typedef struct _Vipm_rngscl2            vipm_rngscl2_t;
# define vipm_rngscl2_p(x)              vipm_rngscl2_cp(x)
# define vipm_rngscl2_cp(x)            ((struct _Vipm_rngscl2 const *)(x))
# define vipm_rngscl2_mp(x)            ((struct _Vipm_rngscl2 *)(x))
# define vipm_rngscl2(...)              VIPM_CTOR_FCNAME(vipm_rngscl2, __VA_ARGS__)
# define vipm_rngscl2_(...)             VIPM_PFX_FCNAME(vipm_rngscl2, __VA_ARGS__)


/* ----------------------------------------------------------------------------
 * @{
 * Array/Structuring element
 */
# pragma mark -
# pragma mark [ Array/Structuring element ]
# pragma mark -

/*
 * [Image] Origin types
 */
# define VipmF_ORIGIN_MASK              0x0F
# define VipmK_INVAL_ORIGIN           (-1)
# define VipmK_NULL_ORIGIN              0
# define VipmK_UNK_ORIGIN               2 /* unknown origin */
# define VipmK_TL_ORIGIN                3 /* top-left origin */
# define VipmK_BL_ORIGIN                4 /* bottom-left origin */

/*
 * Array ordering (arranging) method
 */
# define VipmF_AOM_MASK                 0x0F
# define VipmK_INVAL_AOM              (-1)
# define VipmK_NULL_AOM                 0
# define VipmK_UNK_AOM                  2
# define VipmK_ROW_MAJOR_AOM            3  /* 1: from left to right; 2: from top to bottom */
# define VipmK_COL_MAJOR_AOM            4  /* 1: from top to bottom; 2: from left to right */


/*
 * Default values for width, height of Structuring Element (SE)
 */
# define VipmK_SE_DFL_HEIGHT            5
# define VipmK_SE_DFL_WIDTH             5

# if defined(_MSC_VER)
#  define VipmK_SE_DFL_VOLUME \
	((vipm_Nvolume_t) { 1, VipmK_SE_DFL_HEIGHT, VipmK_SE_DFL_WIDTH, /*1*/0 })
# else
#  define VipmK_SE_DFL_VOLUME             \
	((vipm_Nvolume_t) { [0] = 1, [1] = VipmK_SE_DFL_HEIGHT, [2] = VipmK_SE_DFL_WIDTH, [3] = /*1*/0 })
# endif


struct _Vipm_seshape;
struct _Vipm_sepkernel;
struct _Vipm_strel;
struct _Vipm_array;
struct _Vipm_array_opv;


struct _Vipm_seshape {
	vipm_Npoint_t               se_anchor;  /* relative position of filtered point within the strel */
	vipm_Nvolume_t              se_volume;
};
typedef struct _Vipm_seshape            vipm_seshape_t;
# define vipm_seshape_p(x)              vipm_seshape_cp(x)
# define vipm_seshape_cp(x)            ((struct _Vipm_seshape const *)(x))
# define vipm_seshape_mp(x)            ((struct _Vipm_seshape *)(x))
# define vipm_seshape(...)              VIPM_CTOR_FCNAME(vipm_seshape, __VA_ARGS__)
# define vipm_seshape_(...)             VIPM_PFX_FCNAME(vipm_seshape, __VA_ARGS__)


/* @sa: _Vipm_seshape */
struct _Vipm_sepkernel {
	vipm_seshape_t              ske_shape;
# define ske_anchor             ske_shape.se_anchor
# define ske_volume             ske_shape.se_volume

	vipm_scalar_t              *ske_kervals[VipmK_MAXGDIMS];
};
typedef struct _Vipm_sepkernel          vipm_sepkern_t;
# define vipm_sepkern_p(x)              vipm_sepkern_cp(x)
# define vipm_sepkern_cp(x)            ((struct _Vipm_sepkernel const *)(x))
# define vipm_sepkern_mp(x)            ((struct _Vipm_sepkernel *)(x))
# define vipm_sepkern(...)              VIPM_CTOR_FCNAME(vipm_sepkern, __VA_ARGS__)
# define vipm_sepkern_(...)             VIPM_PFX_FCNAME(vipm_sepkern, __VA_ARGS__)


struct _Vipm_arrshape {
	vipm_site_t                 ars_aasite; /* actual [source] array */
	vipm_site_t                 ars_pasite; /* parent [source] array */
};
typedef struct _Vipm_arrshape           vipm_arrshape_t;
# define vipm_arrshape_p(x)             vipm_arrshape_cp(x)
# define vipm_arrshape_cp(x)           ((struct _Vipm_arrshape const *)(x))
# define vipm_arrshape_mp(x)           ((struct _Vipm_arrshape *)(x))
# define vipm_arrshape(...)             VIPM_CTOR_FCNAME(vipm_arrshape, __VA_ARGS__)
# define vipm_arrshape_(...)            VIPM_PFX_FCNAME(vipm_arrshape, __VA_ARGS__)


/* @sa: _Vipm_strel and _Vipm_seshape */
struct _Vipm_array {

	/*
	 * Actual volume
	 * @{
	 */
	vipm_Nvolume_t              ipar_volume_; /* nplanes, height, width, nchannels */
# define ipar_height        ipar_volume_.v[1]
# define ipar_width         ipar_volume_.v[2]
# define ipar_rowc          ipar_volume_.v[1]
# define ipar_colc          ipar_volume_.v[2]
# define ipar_nchannels     ipar_volume_.v[3]
# define ipar_nplanes       ipar_volume_.v[0]
	/* @} */

	/*
	 * Parent volume
	 * @{
	 */
	vipm_Nvolume_t              ipar_parvol_; /* nplanes, height, width, nchannels */
# define ipar_pheight       ipar_parvol_.v[1]
# define ipar_pwidth        ipar_parvol_.v[2]
# define ipar_prowc         ipar_parvol_.v[1]
# define ipar_pcolc         ipar_parvol_.v[2]
# define ipar_pnchannels    ipar_parvol_.v[3]
# define ipar_pnplanes      ipar_parvol_.v[0]
	/* @} */

	/*
	 * [Space] Strides
	 * @{
	 */
	union {
		vipm_dims_t             ipar_strides_;
		struct {
			vipm_int            ipar_planestride;
			vipm_int            ipar_rowstride;
			vipm_int            ipar_pixstride;
			vipm_int            ipar_elstride;
					/* plane-stride : location(plane1)       = location(plane0)       + plane-stride,
					   row-stride   : location(row1)         = location(row0)         + row-stride,
					   pix-stride   : location(pixel1)       = location(pixel0)       + pix-stride,
					   el-stride    : location(pix-element1) = location(pix-element0) + el-stride */
		};
	};
	/* @} */

	vipm_ushort                 ipar_erank_;
	vipm_ushort                 ipar_perank_; /* rank of a parent array */
	vipm_uint                   ipar_elemtype;

	/*
	 * color bitmap (4 bytes): 1. unpacked: [maybe] 0
	 *                         2. packed: bits-per-channel
	 */
	union {
		bo_uint32_t n;
		char        v[4];
	}                           ipar_clrbm;

	/*
	 * flags (4 bytes): number | struc
	 */
	union {
		bo_uint32_t             ipar_flags;
		struct {
			bo_uint32_t         ipar_origin    :4;
			bo_uint32_t         ipar_aom       :4;
			bo_uint32_t         ipar_reserved  :24;
		};
	};

	/*
	 * Private
	 * @{
	 */
	vipm_long                   ipar_offset_;
	vipm_mutptr_t               ipar_base_;
	struct _Vipm_array_opv     *ipar_opv_;
	/* @} */
};
typedef vipm_mutptr_t                   vipm_mut_array_t;
typedef vipm_imuptr_t                   vipm_array_t;

# define vipm_array_p(x)                vipm_array_cp(x)
# define vipm_mut_array_p(x)            vipm_array_mp(x)
# define vipm_array_cp(x)              ((struct _Vipm_array const *)(x))
# define vipm_array_mp(x)              ((struct _Vipm_array *)(x))
# define vipm_array(...)                VIPM_CTOR_FCNAME(vipm_array, __VA_ARGS__)
# define vipm_array_(...)               VIPM_PFX_FCNAME(vipm_array, __VA_ARGS__)
# define vipm_arr_(...)                 VIPM_PFX_FCNAME(vipm_arr, __VA_ARGS__)


# define VIPM_ARR_COUNT_PIXSTRIDE(a) \
	VIPM_CNT_PIXSTRIDE_TY(vipm_array_p(a)->ipar_nchannels, vipm_array_p(a)->ipar_elemtype)


# define VIPM_ARR_ElemType(a)       (vipm_array_p(a)->ipar_elemtype)
# define VIPM_ARR_ElemAlign(a)      _VIPM_ELTY_ALIGNOF(VIPM_ARR_ElemType(a))


# define VIPM_ARR__Base(a)          ((vipm_imuptr_t) vipm_array_p(a)->ipar_base_)
# define VIPM_MUT_ARR__Base(a)      ((vipm_mutptr_t) vipm_array_mp(a)->ipar_base_)


/*
 * r  : rank
 * gr : rank without nchannels
 * er : "effective" rank
 * ger: "effective" rank without nchannels
 */

# define VIPM_ARR__Strides(a,er)    (vipm_array_mp(a)->ipar_strides_.v + (VipmK_MAXDIMS - (er)))
# define VIPM_ARR__GStrides(a,ger)  (vipm_array_mp(a)->ipar_strides_.v + (VipmK_MAXGDIMS - (ger)))
# define VIPM_ARR__GStridesQ(a,ger) (vipm_array_mp(a)->ipar_strides_.v + ((VipmK_MAXGDIMS <= (ger)) ? 0 : VipmK_MAXGDIMS - (ger)))
# define VIPM_ARR__Volume(a,er)     (vipm_array_mp(a)->ipar_volume_.v + (VipmK_MAXDIMS - (er)))
# define VIPM_ARR__GVolume(a,ger)   (vipm_array_mp(a)->ipar_volume_.v + (VipmK_MAXGDIMS - (ger)))
# define VIPM_ARR__GVolumeQ(a,ger)  (vipm_array_mp(a)->ipar_volume_.v + ((VipmK_MAXGDIMS <= (ger)) ? 0 : VipmK_MAXGDIMS - (ger)))
# define VIPM_PARR__Volume(a,er)    (vipm_array_mp(a)->ipar_parvol_.v + (VipmK_MAXDIMS - (er)))
# define VIPM_PARR__GVolume(a,ger)  (vipm_array_mp(a)->ipar_parvol_.v + (VipmK_MAXGDIMS - (ger)))
# define VIPM_PARR__GVolumeQ(a,ger) (vipm_array_mp(a)->ipar_parvol_.v + ((VipmK_MAXGDIMS <= (ger)) ? 0 : VipmK_MAXGDIMS - (ger)))


# define VIPM_PARR_RRank(a)         (VipmK_MAXDIMS - VIPM_PARR_ERank(a))
# define VIPM_PARR_ERank(a)         ((vipm_int)vipm_array_p(a)->ipar_perank_)
# define VIPM_PARR_GERank(a)        ((vipm_int)vipm_array_p(a)->ipar_perank_ - 1) /* rank without nchannels */
# define VIPM_PARR_MkRank(a)        VIPM_MKRANK_E0(VIPM_PARR_ERank(a))
# define VIPM_PARR_MkGRank(a)       VIPM_MKRANK_E0(VIPM_PARR_GERank(a))           /* rank without nchannels */
# define VIPM_PARR_Volume(a)        VIPM_PARR__Volume(a,VIPM_PARR_ERank(a))
# define VIPM_PARR_GVolume(a)       VIPM_PARR__GVolume(a,VIPM_PARR_GERank(a))
# define VIPM_PARR_Strides(a)       VIPM_ARR__Strides(a,VIPM_PARR_ERank(a))

# define VIPM_PARR_NumPlanes(a)     ((vipm_int) vipm_array_p(a)->ipar_pnplanes)


# define VIPM_ARR_RRank(a)          (VipmK_MAXDIMS - VIPM_ARR_ERank(a))
# define VIPM_ARR_ERank(a)          ((vipm_int)vipm_array_p(a)->ipar_erank_)
# define VIPM_ARR_GERank(a)         ((vipm_int)vipm_array_p(a)->ipar_erank_ - 1)  /* rank without nchannels */
# define VIPM_ARR_MkRank(a)         VIPM_MKRANK_E0(VIPM_ARR_ERank(a))
# define VIPM_ARR_MkGRank(a)        VIPM_MKRANK_E0(VIPM_ARR_GERank(a))            /* rank without nchannels */
# define VIPM_ARR_Volume(a)         VIPM_ARR__Volume(a,VIPM_ARR_ERank(a))
# define VIPM_ARR_GVolume(a)        VIPM_ARR__GVolume(a,VIPM_ARR_GERank(a))
# define VIPM_ARR_Strides(a)        VIPM_ARR__Strides(a,VIPM_ARR_ERank(a))

# define VIPM_ARR_NumPlanes(a)      ((vipm_int) vipm_array_p(a)->ipar_nplanes)

# define VIPM_ARR_NumChannels(a)    ((vipm_int) VIPM_CHANNELC(vipm_array_p(a)->ipar_nchannels))
# define VIPM_ARR_NumSChannels(a)   ((vipm_int) VIPM_SCHANNELC(vipm_array_p(a)->ipar_nchannels))
# define VIPM_ARR_DecodeChannels(a) ((vipm_int) VIPM_DECODE_CHANNELC(vipm_array_p(a)->ipar_nchannels))


struct _Vipm_array_opv {
	char const               *(*description)(struct _Vipm_array const *);
	void                      (*dealloc)    (struct _Vipm_array *);
	void                      (*destroy)    (struct _Vipm_array *);
	vipm_int                  (*retain)     (struct _Vipm_array *);
	vipm_int                  (*release)    (struct _Vipm_array *);
	vipm_int                  (*dretain)    (struct _Vipm_array *);
	vipm_int                  (*drelease)   (struct _Vipm_array *);
};


/* @sa: _Vipm_seshape and _Vipm_array */
struct _Vipm_strel {
	vipm_Npoint_t               se_anchor;  /* relative position of filtered point within the strel */
	union {
		vipm_Nvolume_t          se_volume;
		struct _Vipm_array      se_arr;
	};
};
typedef struct _Vipm_strel              vipm_strel_t;
# define vipm_strel_p(x)                vipm_strel_cp(x)
# define vipm_mut_strel_p(x)            vipm_strel_mp(x)
# define vipm_strel_cp(x)              ((struct _Vipm_strel const *)(x))
# define vipm_strel_mp(x)              ((struct _Vipm_strel *)(x))
# define vipm_strel(...)                VIPM_CTOR_FCNAME(vipm_strel, __VA_ARGS__)
# define vipm_strel_(...)               VIPM_PFX_FCNAME(vipm_strel, __VA_ARGS__)

/* @} */


/* ----------------------------------------------------------------------------
 * @{
 * Border
 */
# pragma mark -
# pragma mark [ Border ]
# pragma mark -

struct _Vipm_brshape {
	vipm_Nvector_t              br_X0, br_X1; /* (outer - inner), vectors in local coordinate system */
};
typedef struct _Vipm_brshape            vipm_brshape_t;
# define vipm_brshape_p(x)              vipm_brshape_cp(x)
# define vipm_brshape_cp(x)            ((struct _Vipm_brshape const *)(x))
# define vipm_brshape_mp(x)            ((struct _Vipm_brshape *)(x))
# define vipm_brshape(...)              VIPM_CTOR_FCNAME(vipm_brshape, __VA_ARGS__)
# define vipm_brshape_(...)             VIPM_PFX_FCNAME(vipm_brshape, __VA_ARGS__)


struct _Vipm_brparm {
	/* _Vipm_brshape @{ */
	vipm_Nvector_t              br_X0, br_X1; /* (outer - inner), vectors in local coordinate system */
	/* @} */
	vipm_scalar_t               br_val[VipmK_MAXCHANNELS];
};
typedef struct _Vipm_brparm             vipm_brparm_t;
# define vipm_brparm_p(x)               vipm_brparm_cp(x)
# define vipm_brparm_cp(x)             ((struct _Vipm_brparm const *)(x))
# define vipm_brparm_mp(x)             ((struct _Vipm_brparm *)(x))
# define vipm_brparm(...)               VIPM_CTOR_FCNAME(vipm_brparm, __VA_ARGS__)
# define vipm_brparm_(...)              VIPM_PFX_FCNAME(vipm_brparm, __VA_ARGS__)


# define VIPM_BR_TH(x)                 (-vipm_pt_y(vipm_brshape_p(x)->br_X0))
# define VIPM_BR_LW(x)                 (-vipm_pt_x(vipm_brshape_p(x)->br_X0))
# define VIPM_BR_BH(x)                  vipm_pt_y(vipm_brshape_p(x)->br_X1)
# define VIPM_BR_RW(x)                  vipm_pt_x(vipm_brshape_p(x)->br_X1)
# define VIPM_BR_H(x)                  (VIPM_BR_TH(x) + VIPM_BR_BH(x))
# define VIPM_BR_W(x)                  (VIPM_BR_LW(x) + VIPM_BR_RW(x))


/*
 * "Border" TGA declarations
 */

struct _Vipm_tga_cpmkbparm {
	vipm_int                    tga_brtype;
	vipm_brparm_t const        *tga_brparm;
	vipm_arrshape_t const      *tga_whole_sshape;
};
typedef struct _Vipm_tga_cpmkbparm      vipm_tga_cpmkbparm_t;
# define vipm_tga_cpmkbparm_p(x)        vipm_tga_cpmkbparm_cp(x)
# define vipm_tga_cpmkbparm_cp(x)      ((struct _Vipm_tga_cpmkbparm const *)(x))
# define vipm_tga_cpmkbparm_mp(x)      ((struct _Vipm_tga_cpmkbparm *)(x))

/* @} */


/* ----------------------------------------------------------------------------
 * @{
 * Fill
 */

struct _Vipm_tga_flxparm {
	int                         tga_pem;     /* "per element" or "per pixel" mask */
	unsigned                    tga_fvcnt;   /* { 1,2 } */
	vipm_scalar_t const        *tga_fval[2];
};
typedef struct _Vipm_tga_flxparm        vipm_tga_flxparm_t;
# define vipm_tga_flxparm_p(x)          vipm_tga_flxparm_cp(x)
# define vipm_tga_flxparm_cp(x)        ((struct _Vipm_tga_flxparm const *)(x))
# define vipm_tga_flxparm_mp(x)        ((struct _Vipm_tga_flxparm *)(x))

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * CountRelBox
 */

struct _Vipm_cntrelboxopts {
	vipm_int                    crbo_brtype;
	vipm_scalar_t               crbo_brval[VipmK_MAXCHANNELS];
};
typedef struct _Vipm_cntrelboxopts      vipm_cntrelboxopts_t;
# define vipm_cntrelboxopts_p(x)        vipm_cntrelboxopts_cp(x)
# define vipm_cntrelboxopts_cp(x)      ((struct _Vipm_cntrelboxopts const *)(x))
# define vipm_cntrelboxopts_mp(x)      ((struct _Vipm_cntrelboxopts *)(x))


struct _Vipm_tga_cntrelboxparm {
	vipm_int                    tga_cmpop;
	vipm_seshape_t const       *tga_strel;
	vipm_range_t               *tga_rng;
	vipm_cntrelboxopts_t const *tga_opts;           /* = NULL */
	vipm_arrshape_t const      *tga_whole_sshape;   /* = NULL */
	vipm_int                    tga_scalefactor;
	vipm_int                    tga_rndmode;
};
typedef struct _Vipm_tga_cntrelboxparm  vipm_tga_cntrelboxparm_t;
# define vipm_tga_cntrelboxparm_p(x)    vipm_tga_cntrelboxparm_cp(x)
# define vipm_tga_cntrelboxparm_cp(x)  ((struct _Vipm_tga_cntrelboxparm const *)(x))
# define vipm_tga_cntrelboxparm_mp(x)  ((struct _Vipm_tga_cntrelboxparm *)(x))

/* @} */

/*
 * Core type definitions
 * @}
 */

#endif /* !defined(__VIPM__TYPES_CORE__TYPES_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
