/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Imel_Types.h	(V.Sokur)
 * Vipm/_Types/Imel_Types.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES_IMEL__TYPES_H__
# define __VIPM__TYPES_IMEL__TYPES_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif

# ifndef __VIPM__TYPES_BASE__TYPES_H__
#  include "Vipm/_Types/Base_Types.h"
# endif
# ifndef __VIPM__TYPES_BASE__LIMITS_H__
#  include "Vipm/_Types/Base_Limits.h"
# endif
 */


/******************************************************************************
 * %% BeginSection: type/constant declarations / definitions
 */

/*
 * @{
 * Channel/Elem-Type constants/macros
 */

/* ----------------------------------------------------------------------------
 * @{
 * Channel macros
 *
 * CN  : [dirty] channel-count
 * XCn : [decoded] channel-count: 1,2,3,4, 11 (0xB = c3 + alpha-channel)
 * Cn  : [properly] channel-count
 * CnNA: [significant] channel-count (without alpha-channel)
 * cn  : channel
 */
# pragma mark -
# pragma mark [ Channel macros ]
# pragma mark -

# define VipmK_MAXCHANNELS              4

# define VipmF_CHANNEL_MASK             0x0F
# define VipmF_CHANNELC_MASK            0x07
# define VipmF_ALPHA_CHANNEL            0x08
# define VipmF_CNBM_MASK                0xF0


# define VIPM_SCHANNELC(CN)                     ((CN) & VipmF_CHANNELC_MASK)
# define VIPM_CHANNELC(CN)                      (VIPM_SCHANNELC(CN) + !!(VipmF_ALPHA_CHANNEL & (CN)))
# define VIPM_DECODE_CHANNELC(CN)               ((CN) & (VipmF_CHANNELC_MASK | VipmF_ALPHA_CHANNEL))
# define VIPM_ENCODE_CHANNELC(CnNA,Alpha)       ((CnNA) | ((Alpha) ? VipmF_ALPHA_CHANNEL : 0))
# define VIPM_ENCODE_XCHANNELC(CnNA,Alpha,cnbm) _VIPM_ADD_CNBM(VIPM_ENCODE_CHANNELC(CnNA,Alpha), cnbm)

# define VIPM_DECODE_CNBM(Cn)                   VIPM_GET_CNBM(Cn)


# define VIPM_GET_CNBM(r)                      ((vipm_uint) (((r) >> 4) & 0xF))
# define VIPM_XGET_CNBM(r,n)                    __VIPM_XGET_CNBM(r,n)
# ifdef _MSC_VER
#  define VIPM_CLR_CNBM(r)                     ((r) &= ~(vipm_uint)VipmF_CNBM_MASK)
#  define VIPM_ADD_CNBM(r,cnbm)                ((r) |= (((cnbm) & 0xF) << 4))
#  define VIPM_SET_CNBM(r,cnbm)                ((r)  = _VIPM_SET_CNBM(r,cnbm))
# else
#  define VIPM_CLR_CNBM(r)                     ((r) &= ~(__typeof__(r))VipmF_CNBM_MASK)
#  define VIPM_ADD_CNBM(r,cnbm)                ((r) |= (__typeof__(r))(((cnbm) & 0xF) << 4))
#  define VIPM_SET_CNBM(r,cnbm)                ((r)  = (__typeof__(r))_VIPM_SET_CNBM(r,cnbm))
# endif

# define VIPM_ASGN_CNBM(d,s)                    VIPM_SET_CNBM((d),VIPM_GET_CNBM(s))
# define VIPM_ASGN_CNBM_IF(d,s) \
	((void) (((0 == ((d) & VipmF_CNBM_MASK)) && ((s) & VipmF_CNBM_MASK)) ? VIPM_ADD_CNBM((d),VIPM_GET_CNBM(s)) : (d)))


# ifdef _MSC_VER
#  define _VIPM_CLR_CNBM(r)                    ((r) & ~(vipm_uint)VipmF_CNBM_MASK)
#  define _VIPM_ADD_CNBM(r,cnbm)               ((r) | (((cnbm) & 0xF) << 4))
# else
#  define _VIPM_CLR_CNBM(r)                    ((r) & ~(__typeof__(r))VipmF_CNBM_MASK)
#  define _VIPM_ADD_CNBM(r,cnbm)               ((r) | (__typeof__(r))(((cnbm) & 0xF) << 4))
# endif
# define _VIPM_SET_CNBM(r,cnbm)                 _VIPM_ADD_CNBM(_VIPM_CLR_CNBM(r), cnbm)

# define _VIPM_ASGN_CNBM(d,s)                   _VIPM_SET_CNBM((d),VIPM_GET_CNBM(s))
# define _VIPM_ASGN_CNBM_IF(d,s) \
	(((0 == ((d) & VipmF_CNBM_MASK)) && ((s) & VipmF_CNBM_MASK)) ? _VIPM_ADD_CNBM((d),VIPM_GET_CNBM(s)) : (d))


# define _VipmK_DFL_CNBM(Cn) \
	(  1 == (Cn) ? 0x1U      \
     : 2 == (Cn) ? 0x3U      \
     : 3 == (Cn) ? 0x7U      \
     : 4 == (Cn) ? 0xFU      \
                 : 0U )

# if __myc_support(macro_retval)
#  define __VIPM_XGET_CNBM(o,Cn)                          \
	__extension__ ({                                      \
		vipm_uint ___cnbm = (vipm_uint)VIPM_GET_CNBM(o);  \
		vipm_int ___nchannels;                            \
		/* return */ (0 != ___cnbm ? ___cnbm : (___nchannels = (Cn), _VipmK_DFL_CNBM(___nchannels))); \
	})
# else
_VIPM_INLINE vipm_uint __VIPM_XGET_CNBM(vipm_ulong Options, vipm_int Cn)
{
	vipm_uint cnbm = (vipm_uint)VIPM_GET_CNBM(Options);
	return (0 != cnbm ? cnbm : _VipmK_DFL_CNBM(Cn));
}
# endif


# define VIPM_TEST_CHANNEL(cn,cnbm)            ((vipm_uint)(1 << (cn)) & (cnbm))

# define VIPM_CHANNELC_UTW(_CnNA,_Alpha)        VIPM_GLUE3(VIPM__CHANNELC_UTW_,_CnNA,_Alpha)()
# define VIPM_CHANNELC_LTW(_CnNA,_Alpha)        VIPM_GLUE3(VIPM__CHANNELC_LTW_,_CnNA,_Alpha)()
# define VIPM_MKSCHANNELC_C(_Cn,_Alpha)         VIPM_GLUE3(VIPM__MKSCHANNELC_,_Cn,_Alpha)()


# define VIPM_PXIDX(idx,cn,ncpnt)              ((idx) + ((cn) - 1) * (ncpnt))


/* internal @{ */
# define VIPM__CHANNELC_UTW_10()        C1
# define VIPM__CHANNELC_UTW_20()        C2
# define VIPM__CHANNELC_UTW_30()        C3
# define VIPM__CHANNELC_UTW_40()        C4
# define VIPM__CHANNELC_UTW_11()        AC1
# define VIPM__CHANNELC_UTW_21()        AC2
# define VIPM__CHANNELC_UTW_31()        AC3
# define VIPM__CHANNELC_UTW_41()        AC4
# define VIPM__CHANNELC_LTW_10()        c1
# define VIPM__CHANNELC_LTW_20()        c2
# define VIPM__CHANNELC_LTW_30()        c3
# define VIPM__CHANNELC_LTW_40()        c4
# define VIPM__CHANNELC_LTW_11()        ac1
# define VIPM__CHANNELC_LTW_21()        ac2
# define VIPM__CHANNELC_LTW_31()        ac3
# define VIPM__CHANNELC_LTW_41()        ac4

# define VIPM__MKSCHANNELC_10()         1
# define VIPM__MKSCHANNELC_20()         2
# define VIPM__MKSCHANNELC_30()         3
# define VIPM__MKSCHANNELC_40()         4
# define VIPM__MKSCHANNELC_11()         0
# define VIPM__MKSCHANNELC_21()         1
# define VIPM__MKSCHANNELC_31()         2
# define VIPM__MKSCHANNELC_41()         3
/* @} */

/*
 * Channel macros
 * @}
 */

/* ----------------------------------------------------------------------------
 * @{
 * Elem-Type constants/macros
 */
# pragma mark -
# pragma mark [ Elem-Type constants/macros ]
# pragma mark -

# define VipmF_ELEMTYPE_MASK            0x0F
# define VipmF_ELEMTYPE_CATMASK         0x07
# define VipmF_ELEMTYPE_SIGN_BIT        0x08

	/* @{ */
# define VipmK_ELEMTYPE_INTEGRAL0       1
# define VipmK_ELEMTYPE_INTEGRAL1       4
# define VipmK_ELEMTYPE_FLOAT0          5
# define VipmK_ELEMTYPE_FLOAT1          7
	/* @} */

	/*
	 * @{
	 * Category
	 */
# define _VipmK_ELEMTYPE_UNK            0
# define VipmK_ELEMTYPE_UNK             0

# define _VipmK_ELEMTYPE_I8             1
# define _VipmK_ELEMTYPE_I16            2
# define _VipmK_ELEMTYPE_I32            3
# define _VipmK_ELEMTYPE_I64            4
# define _VipmK_ELEMTYPE_F32            5
# define _VipmK_ELEMTYPE_F64            6
# define _VipmK_ELEMTYPE_F128           7
# define _VipmK_ELEMTYPE_SCALAR         _VipmK_ELEMTYPE_F64
	/* @} */

# define VipmK_ELEMTYPE_U8              1
# define VipmK_ELEMTYPE_U16             2
# define VipmK_ELEMTYPE_U32             3
# define VipmK_ELEMTYPE_U64             4
# define VipmK_ELEMTYPE_S8              9
# define VipmK_ELEMTYPE_S16             10
# define VipmK_ELEMTYPE_S32             11
# define VipmK_ELEMTYPE_S64             12
# define VipmK_ELEMTYPE_F32             13
# define VipmK_ELEMTYPE_F64             14
# define VipmK_ELEMTYPE_F128            15
# define VipmK_ELEMTYPE_SCALAR          VipmK_ELEMTYPE_F64

# define VipmK_WORK_ELEMTYPE(_ElemType) VIPM_GLUE(VipmK__WORK_ELEMTYPE_,VIPM_ELTY_UTW(_ElemType))()
# define VipmK_HWORK_ELEMTYPE(_ElemType) VIPM_GLUE(VipmK__HWORK_ELEMTYPE_,VIPM_ELTY_UTW(_ElemType))()



# define VIPM_ELTY_CATEGORY(ElemType)  ((ElemType) & VipmF_ELEMTYPE_CATMASK)
# define VIPM_ELTY_IS_INT(ElemType)    (VIPM_ELTY_CATEGORY(ElemType) <= VipmK_ELEMTYPE_INTEGRAL1)
# define VIPM_ELTY_IS_FLOAT(ElemType)  (VipmK_ELEMTYPE_FLOAT0 <= VIPM_ELTY_CATEGORY(ElemType))
# define VIPM_ELTY_SIGNED(ElemType)    ((VipmF_ELEMTYPE_SIGN_BIT & (ElemType)) != 0)


# define _VIPM_ELTY_ALIGNOF(ElemType)   __VIPM_ELTY_ALIGNOF(VIPM_ELTY_CATEGORY(ElemType))
# define __VIPM_ELTY_ALIGNOF(_ElemType)        \
    (  _VipmK_ELEMTYPE_I8   == _ElemType ? 1   \
     : _VipmK_ELEMTYPE_I16  == _ElemType ? 2   \
     : _VipmK_ELEMTYPE_I32  == _ElemType ? 4   \
     : _VipmK_ELEMTYPE_I64  == _ElemType ? 8   \
     : _VipmK_ELEMTYPE_F32  == _ElemType ? 4   \
     : _VipmK_ELEMTYPE_F64  == _ElemType ? 8   \
                                         : 0 )

# define _VIPM_ELTY_SIZEOF(ElemType)    __VIPM_ELTY_SIZEOF(VIPM_ELTY_CATEGORY(ElemType))
# define __VIPM_ELTY_SIZEOF(_ElemType)         \
    (  _VipmK_ELEMTYPE_I8   == _ElemType ? 1   \
     : _VipmK_ELEMTYPE_I16  == _ElemType ? 2   \
     : _VipmK_ELEMTYPE_I32  == _ElemType ? 4   \
     : _VipmK_ELEMTYPE_I64  == _ElemType ? 8   \
     : _VipmK_ELEMTYPE_F32  == _ElemType ? 4   \
     : _VipmK_ELEMTYPE_F64  == _ElemType ? 8   \
                                         : 0 )

# define _VIPM_ELTY_WIDTHOF(ElemType)   __VIPM_ELTY_WIDTHOF(VIPM_ELTY_CATEGORY(ElemType))
# define __VIPM_ELTY_WIDTHOF(_ElemType)        \
    (  _VipmK_ELEMTYPE_I8   == _ElemType ? 8   \
     : _VipmK_ELEMTYPE_I16  == _ElemType ? 16  \
     : _VipmK_ELEMTYPE_I32  == _ElemType ? 32  \
     : _VipmK_ELEMTYPE_I64  == _ElemType ? 64  \
     : _VipmK_ELEMTYPE_F32  == _ElemType ? 32  \
     : _VipmK_ELEMTYPE_F64  == _ElemType ? 64  \
                                         : 0 )

# define _VIPM_ELTY_MIN(ElemType)                                                            \
    (  VipmK_ELEMTYPE_U8  == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MIN_C(VipmK_ELEMTYPE_U8)  \
     : VipmK_ELEMTYPE_S8  == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MIN_C(VipmK_ELEMTYPE_S8)  \
     : VipmK_ELEMTYPE_U16 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MIN_C(VipmK_ELEMTYPE_U16) \
     : VipmK_ELEMTYPE_S16 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MIN_C(VipmK_ELEMTYPE_S16) \
     : VipmK_ELEMTYPE_U32 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MIN_C(VipmK_ELEMTYPE_U32) \
     : VipmK_ELEMTYPE_S32 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MIN_C(VipmK_ELEMTYPE_S32) \
     : VipmK_ELEMTYPE_F32 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MIN_C(VipmK_ELEMTYPE_F32) \
                                        : (vipm_scalar_t)(-VipmK_SCALAR_INF))
# define _VIPM_ELTY_MAX(ElemType)                                                            \
    (  VipmK_ELEMTYPE_U8  == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MAX_C(VipmK_ELEMTYPE_U8)  \
     : VipmK_ELEMTYPE_S8  == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MAX_C(VipmK_ELEMTYPE_S8)  \
     : VipmK_ELEMTYPE_U16 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MAX_C(VipmK_ELEMTYPE_U16) \
     : VipmK_ELEMTYPE_S16 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MAX_C(VipmK_ELEMTYPE_S16) \
     : VipmK_ELEMTYPE_U32 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MAX_C(VipmK_ELEMTYPE_U32) \
     : VipmK_ELEMTYPE_S32 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MAX_C(VipmK_ELEMTYPE_S32) \
     : VipmK_ELEMTYPE_F32 == (ElemType) ? (vipm_scalar_t)VIPM_ELTY_MAX_C(VipmK_ELEMTYPE_F32) \
                                        : (vipm_scalar_t)(VipmK_SCALAR_INF))



# define VIPM_ELTY_UTW(_ElemType)       VIPM_GLUE(VIPM__ELEMTYPE_UTW_,_ElemType)()
# define VIPM_ELTY_LTW(_ElemType)       VIPM_GLUE(VIPM__ELEMTYPE_LTW_,_ElemType)()


# define VIPM__ELTY_ALIGNOF_C(_ETw)     VIPM_GLUE(VIPM__ELEMTYPE_ALIGNOF_,_ETw)()
# define VIPM__ELTY_SIZEOF_C(_ETw)      VIPM_GLUE(VIPM__ELEMTYPE_SIZEOF_,_ETw)()
# define VIPM__ELTY_WIDTHOF_C(_ETw)     VIPM_GLUE(VIPM__ELEMTYPE_WIDTHOF_,_ETw)()
# define VIPM__ELTY_MIN_C(_ETw)         VIPM_GLUE(VIPM__ELEMTYPE_MIN_,_ETw)()
# define VIPM__ELTY_MAX_C(_ETw)         VIPM_GLUE(VIPM__ELEMTYPE_MAX_,_ETw)()

# define VIPM_ELTY_ALIGNOF_C(_ElemType) VIPM__ELTY_ALIGNOF_C(VIPM_ELTY_UTW(_ElemType))
# define VIPM_ELTY_SIZEOF_C(_ElemType)  VIPM__ELTY_SIZEOF_C(VIPM_ELTY_UTW(_ElemType))
# define VIPM_ELTY_WIDTHOF_C(_ElemType) VIPM__ELTY_WIDTHOF_C(VIPM_ELTY_UTW(_ElemType))
# define VIPM_ELTY_MIN_C(_ElemType)     VIPM__ELTY_MIN_C(VIPM_ELTY_UTW(_ElemType))
# define VIPM_ELTY_MAX_C(_ElemType)     VIPM__ELTY_MAX_C(VIPM_ELTY_UTW(_ElemType))


# define VIPM__ELTY_T(_ETw)             VIPM_GLUE(VIPM__ELEMTYPE_T_,_ETw)()
# define VIPM__EINT_T(_ETw)             VIPM_GLUE(VIPM__EINT_T_,_ETw)()
# define VIPM__EINT8_T(_ETw)            VIPM_GLUE(VIPM__EINT8_T_,_ETw)()
# define VIPM__EINT16_T(_ETw)           VIPM_GLUE(VIPM__EINT16_T_,_ETw)()
# define VIPM__EINT32_T(_ETw)           VIPM_GLUE(VIPM__EINT32_T_,_ETw)()
# define VIPM__EINT64_T(_ETw)           VIPM_GLUE(VIPM__EINT64_T_,_ETw)()
# define VIPM__EUINT_T(_ETw)            VIPM_GLUE(VIPM__EUINT_T_,_ETw)()
# define VIPM__ESINT_T(_ETw)            VIPM_GLUE(VIPM__ESINT_T_,_ETw)()
# define VIPM__EFINT_T(_ETw)            VIPM_GLUE(VIPM__EFINT_T_,_ETw)()
# define VIPM__EFINT32_T(_ETw)          VIPM_GLUE(VIPM__EFINT32_T_,_ETw)()
# define VIPM__EWORK_T(_ETw)            VIPM_GLUE(VIPM__EWORK_T_,_ETw)()    /* element work type */
# define VIPM__EHWORK_T(_ETw)           VIPM_GLUE(VIPM__EHWORK_T_,_ETw)()   /* element "half"-work type */
# define VIPM__EMPAR_T(_ETw)            VIPM_GLUE(VIPM__EMPAR_T_,_ETw)()    /* element "machine" param type */

# define VIPM_ELTY_T(_ElemType)         VIPM__ELTY_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EINT_T(_ElemType)         VIPM__EINT_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EINT8_T(_ElemType)        VIPM__EINT8_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EINT16_T(_ElemType)       VIPM__EINT16_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EINT32_T(_ElemType)       VIPM__EINT32_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EINT64_T(_ElemType)       VIPM__EINT64_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EUINT_T(_ElemType)        VIPM__EUINT_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_ESINT_T(_ElemType)        VIPM__ESINT_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EFINT_T(_ElemType)        VIPM__EFINT_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EFINT32_T(_ElemType)      VIPM__EFINT32_T(VIPM_ELTY_UTW(_ElemType))
# define VIPM_EWORK_T(_ElemType)        VIPM__EWORK_T(VIPM_ELTY_UTW(_ElemType))     /* element work type */
# define VIPM_EHWORK_T(_ElemType)       VIPM__EHWORK_T(VIPM_ELTY_UTW(_ElemType))    /* element "half"-work type */
# define VIPM_EMPAR_T(_ElemType)        VIPM__EMPAR_T(VIPM_ELTY_UTW(_ElemType))     /* element "machine" param type */


# define VIPM__ELTYCAST_C(_EDTw,_ESTw, aValue)  VIPM_GLUE3(VIPM__ELTYCAST_,_EDTw,_ESTw)(aValue)

# define VIPM_ELTYCAST_C(_EDT,_EST, aValue)     VIPM__ELTYCAST_C(VIPM_ELTY_UTW(_EDT),VIPM_ELTY_UTW(_EST), aValue)


# define VipmK__ARITH_ETRUE(_ETw)       VIPM_GLUE(VipmK__ARITH_ETRUE_,_ETw)()
# define VipmK__ARITH_EFALSE(_ETw)      VIPM_GLUE(VipmK__ARITH_EFALSE_,_ETw)()

# define VipmK_ARITH_ETRUE(_ElemType)   VipmK__ARITH_ETRUE(VIPM_ELTY_UTW(_ElemType))
# define VipmK_ARITH_EFALSE(_ElemType)  VipmK__ARITH_EFALSE(VIPM_ELTY_UTW(_ElemType))
# define VipmK_ARITH_TRUE               0xFF
# define VipmK_ARITH_FALSE              0
# define VipmK_ARITH_TRUE_8             0xFF
# define VipmK_ARITH_FALSE_8            0
# define VipmK_ARITH_TRUE_16            0xFFFF
# define VipmK_ARITH_FALSE_16           0
# define VipmK_ARITH_TRUE_32            0xFFFFFFFF
# define VipmK_ARITH_FALSE_32           0
# define VipmK_ARITH_TRUE_64            0xFFFFFFFFFFFFFFFF
# define VipmK_ARITH_FALSE_64           0



/* @{ */
# define VIPM_WIDTHOF_ELEM_C(_ElemType)         VIPM_ELTY_WIDTHOF_C(_ElemType)
# define VIPM_SIZEOF_ELEM_C(_ElemType)          VIPM_ELTY_SIZEOF_C(_ElemType)
/* @} */
# define VIPM_WIDTHOF_PIXEL_C(_Cn,_ElemType)    VIPM_GLUE3(VIPM__WIDTHOF_PIXEL_,_Cn,VIPM_ELTY_UTW(_ElemType))()
# define VIPM_SIZEOF_PIXEL_C(_Cn,_ElemType)     VIPM_GLUE3(VIPM__SIZEOF_PIXEL_,_Cn,VIPM_ELTY_UTW(_ElemType))()
/* @{ */
# define VIPM_PIXWIDTH_C(_Cn,_ElemWidth)        VIPM_GLUE3(VIPM__PIXWIDTH_,_Cn,_ElemWidth)()
# define VIPM_PIXSTRIDE_C(_Cn,_Elemsz)          VIPM_GLUE3(VIPM__PIXSTRIDE_,_Cn,_Elemsz)()
/* @} */



# define VIPM_CNT_PIXSTRIDE(CN,Elemsz)          __VIPM_CNT_PIXSTRIDE(VIPM_CHANNELC(CN), Elemsz)
# define VIPM_CNT_PIXSTRIDE_TY(CN,ElemType)     __VIPM_CNT_PIXSTRIDE_TY(VIPM_CHANNELC(CN), ElemType)
# define __VIPM_CNT_PIXSTRIDE(Cn,Elemsz)        ((vipm_int) ((Cn) * (Elemsz)))
# define __VIPM_CNT_PIXSTRIDE_TY(Cn,ElemType)   ((vipm_int) ((Cn) * _VIPM_ELTY_SIZEOF(ElemType)))



	/* internal @{ */
# define VIPM__ELEMTYPE_UTW_1()         U8
# define VIPM__ELEMTYPE_UTW_2()         U16
# define VIPM__ELEMTYPE_UTW_3()         U32
# define VIPM__ELEMTYPE_UTW_4()         U64
# define VIPM__ELEMTYPE_UTW_9()         S8
# define VIPM__ELEMTYPE_UTW_10()        S16
# define VIPM__ELEMTYPE_UTW_11()        S32
# define VIPM__ELEMTYPE_UTW_12()        S64
# define VIPM__ELEMTYPE_UTW_5()         F32
# define VIPM__ELEMTYPE_UTW_6()         F64
# define VIPM__ELEMTYPE_UTW_7()         F128
# define VIPM__ELEMTYPE_UTW_13()        F32
# define VIPM__ELEMTYPE_UTW_14()        F64
# define VIPM__ELEMTYPE_UTW_15()        F128

# define VIPM__ELEMTYPE_LTW_1()         u8
# define VIPM__ELEMTYPE_LTW_2()         u16
# define VIPM__ELEMTYPE_LTW_3()         u32
# define VIPM__ELEMTYPE_LTW_4()         u64
# define VIPM__ELEMTYPE_LTW_9()         s8
# define VIPM__ELEMTYPE_LTW_10()        s16
# define VIPM__ELEMTYPE_LTW_11()        s32
# define VIPM__ELEMTYPE_LTW_12()        s64
# define VIPM__ELEMTYPE_LTW_5()         f32
# define VIPM__ELEMTYPE_LTW_6()         f64
# define VIPM__ELEMTYPE_LTW_7()         f128
# define VIPM__ELEMTYPE_LTW_13()        f32
# define VIPM__ELEMTYPE_LTW_14()        f64
# define VIPM__ELEMTYPE_LTW_15()        f128


# define VipmK__WORK_ELEMTYPE_U8()      VipmK_ELEMTYPE_U32
# define VipmK__WORK_ELEMTYPE_U16()     VipmK_ELEMTYPE_U32
# define VipmK__WORK_ELEMTYPE_U32()     VipmK_ELEMTYPE_U32
# define VipmK__WORK_ELEMTYPE_U64()     VipmK_ELEMTYPE_U64
# define VipmK__WORK_ELEMTYPE_S8()      VipmK_ELEMTYPE_S32
# define VipmK__WORK_ELEMTYPE_S16()     VipmK_ELEMTYPE_S32
# define VipmK__WORK_ELEMTYPE_S32()     VipmK_ELEMTYPE_S32
# define VipmK__WORK_ELEMTYPE_S64()     VipmK_ELEMTYPE_S64
# define VipmK__WORK_ELEMTYPE_F32()     VipmK_ELEMTYPE_F32
# define VipmK__WORK_ELEMTYPE_F64()     VipmK_ELEMTYPE_F64
# define VipmK__WORK_ELEMTYPE_F128()    VipmK_ELEMTYPE_F128

# define VipmK__HWORK_ELEMTYPE_U8()     VipmK_ELEMTYPE_U16
# define VipmK__HWORK_ELEMTYPE_U16()    VipmK_ELEMTYPE_U32
# define VipmK__HWORK_ELEMTYPE_U32()    VipmK_ELEMTYPE_U32
# define VipmK__HWORK_ELEMTYPE_U64()    VipmK_ELEMTYPE_U64
# define VipmK__HWORK_ELEMTYPE_S8()     VipmK_ELEMTYPE_S16
# define VipmK__HWORK_ELEMTYPE_S16()    VipmK_ELEMTYPE_S32
# define VipmK__HWORK_ELEMTYPE_S32()    VipmK_ELEMTYPE_S32
# define VipmK__HWORK_ELEMTYPE_S64()    VipmK_ELEMTYPE_S64
# define VipmK__HWORK_ELEMTYPE_F32()    VipmK_ELEMTYPE_F32
# define VipmK__HWORK_ELEMTYPE_F64()    VipmK_ELEMTYPE_F64
# define VipmK__HWORK_ELEMTYPE_F128()   VipmK_ELEMTYPE_F128


# define VIPM__ELEMTYPE_ALIGNOF_U8()    1
# define VIPM__ELEMTYPE_ALIGNOF_U16()   2
# define VIPM__ELEMTYPE_ALIGNOF_U32()   4
# define VIPM__ELEMTYPE_ALIGNOF_U64()   8
# define VIPM__ELEMTYPE_ALIGNOF_S8()    1
# define VIPM__ELEMTYPE_ALIGNOF_S16()   2
# define VIPM__ELEMTYPE_ALIGNOF_S32()   4
# define VIPM__ELEMTYPE_ALIGNOF_S64()   8
# define VIPM__ELEMTYPE_ALIGNOF_F32()   4
# define VIPM__ELEMTYPE_ALIGNOF_F64()   8
# define VIPM__ELEMTYPE_ALIGNOF_F128()  16

# define VIPM__ELEMTYPE_SIZEOF_U8()     1
# define VIPM__ELEMTYPE_SIZEOF_U16()    2
# define VIPM__ELEMTYPE_SIZEOF_U32()    4
# define VIPM__ELEMTYPE_SIZEOF_U64()    8
# define VIPM__ELEMTYPE_SIZEOF_S8()     1
# define VIPM__ELEMTYPE_SIZEOF_S16()    2
# define VIPM__ELEMTYPE_SIZEOF_S32()    4
# define VIPM__ELEMTYPE_SIZEOF_S64()    8
# define VIPM__ELEMTYPE_SIZEOF_F32()    4
# define VIPM__ELEMTYPE_SIZEOF_F64()    8
# define VIPM__ELEMTYPE_SIZEOF_F128()   16

# define VIPM__ELEMTYPE_WIDTHOF_U8()    8
# define VIPM__ELEMTYPE_WIDTHOF_U16()   16
# define VIPM__ELEMTYPE_WIDTHOF_U32()   32
# define VIPM__ELEMTYPE_WIDTHOF_U64()   64
# define VIPM__ELEMTYPE_WIDTHOF_S8()    8
# define VIPM__ELEMTYPE_WIDTHOF_S16()   16
# define VIPM__ELEMTYPE_WIDTHOF_S32()   32
# define VIPM__ELEMTYPE_WIDTHOF_S64()   64
# define VIPM__ELEMTYPE_WIDTHOF_F32()   32
# define VIPM__ELEMTYPE_WIDTHOF_F64()   64
# define VIPM__ELEMTYPE_WIDTHOF_F128()  128


# define VIPM__ELEMTYPE_MIN_U8()        0
# define VIPM__ELEMTYPE_MIN_U16()       0
# define VIPM__ELEMTYPE_MIN_U32()       0
# define VIPM__ELEMTYPE_MIN_U64()       0
# define VIPM__ELEMTYPE_MIN_S8()        (-127-1)
# define VIPM__ELEMTYPE_MIN_S16()       (-32767-1)
# define VIPM__ELEMTYPE_MIN_S32()       (-2147483647L-1)
# define VIPM__ELEMTYPE_MIN_S64()       (-0x7FFFFFFFFFFFFFFFLL-1)
# define VIPM__ELEMTYPE_MIN_F32()       (-FLT_MAX)
# define VIPM__ELEMTYPE_MIN_F64()       (-DBL_MAX)

# define VIPM__ELEMTYPE_MAX_U8()        255
# define VIPM__ELEMTYPE_MAX_U16()       0xFFFFU
# define VIPM__ELEMTYPE_MAX_U32()       0xFFFFFFFFLU
# define VIPM__ELEMTYPE_MAX_U64()       0xFFFFFFFFFFFFFFFFLLU
# define VIPM__ELEMTYPE_MAX_S8()        127
# define VIPM__ELEMTYPE_MAX_S16()       32767
# define VIPM__ELEMTYPE_MAX_S32()       2147483647L
# define VIPM__ELEMTYPE_MAX_S64()       0x7FFFFFFFFFFFFFFFLL
# define VIPM__ELEMTYPE_MAX_F32()       FLT_MAX
# define VIPM__ELEMTYPE_MAX_F64()       DBL_MAX


# define VIPM__ELEMTYPE_T_U8()          bo_uint8_t
# define VIPM__ELEMTYPE_T_U16()         bo_uint16_t
# define VIPM__ELEMTYPE_T_U32()         bo_uint32_t
# define VIPM__ELEMTYPE_T_U64()         bo_uint64_t
# define VIPM__ELEMTYPE_T_S8()          bo_int8_t
# define VIPM__ELEMTYPE_T_S16()         bo_int16_t
# define VIPM__ELEMTYPE_T_S32()         bo_int32_t
# define VIPM__ELEMTYPE_T_S64()         bo_int64_t
# define VIPM__ELEMTYPE_T_F32()         float
# define VIPM__ELEMTYPE_T_F64()         double


# define VIPM__EINT_T_U8()              bo_uint8_t
# define VIPM__EINT_T_U16()             bo_uint16_t
# define VIPM__EINT_T_U32()             bo_uint32_t
# define VIPM__EINT_T_U64()             bo_uint64_t
# define VIPM__EINT_T_S8()              bo_int8_t
# define VIPM__EINT_T_S16()             bo_int16_t
# define VIPM__EINT_T_S32()             bo_int32_t
# define VIPM__EINT_T_S64()             bo_int64_t
# define VIPM__EINT_T_F32()             bo_int32_t
# define VIPM__EINT_T_F64()             bo_int64_t

# define VIPM__EINT8_T_U8()             bo_uint8_t
# define VIPM__EINT8_T_U16()            bo_uint8_t
# define VIPM__EINT8_T_U32()            bo_uint8_t
# define VIPM__EINT8_T_U64()            bo_uint8_t
# define VIPM__EINT8_T_S8()             bo_int8_t
# define VIPM__EINT8_T_S16()            bo_int8_t
# define VIPM__EINT8_T_S32()            bo_int8_t
# define VIPM__EINT8_T_S64()            bo_int8_t
# define VIPM__EINT8_T_F32()            bo_int8_t
# define VIPM__EINT8_T_F64()            bo_int8_t

# define VIPM__EINT16_T_U8()            bo_uint16_t
# define VIPM__EINT16_T_U16()           bo_uint16_t
# define VIPM__EINT16_T_U32()           bo_uint16_t
# define VIPM__EINT16_T_U64()           bo_uint16_t
# define VIPM__EINT16_T_S8()            bo_int16_t
# define VIPM__EINT16_T_S16()           bo_int16_t
# define VIPM__EINT16_T_S32()           bo_int16_t
# define VIPM__EINT16_T_S64()           bo_int16_t
# define VIPM__EINT16_T_F32()           bo_int16_t
# define VIPM__EINT16_T_F64()           bo_int16_t

# define VIPM__EINT32_T_U8()            bo_uint32_t
# define VIPM__EINT32_T_U16()           bo_uint32_t
# define VIPM__EINT32_T_U32()           bo_uint32_t
# define VIPM__EINT32_T_U64()           bo_uint32_t
# define VIPM__EINT32_T_S8()            bo_int32_t
# define VIPM__EINT32_T_S16()           bo_int32_t
# define VIPM__EINT32_T_S32()           bo_int32_t
# define VIPM__EINT32_T_S64()           bo_int32_t
# define VIPM__EINT32_T_F32()           bo_int32_t
# define VIPM__EINT32_T_F64()           bo_int32_t

# define VIPM__EINT64_T_U8()            bo_uint64_t
# define VIPM__EINT64_T_U16()           bo_uint64_t
# define VIPM__EINT64_T_U32()           bo_uint64_t
# define VIPM__EINT64_T_U64()           bo_uint64_t
# define VIPM__EINT64_T_S8()            bo_int64_t
# define VIPM__EINT64_T_S16()           bo_int64_t
# define VIPM__EINT64_T_S32()           bo_int64_t
# define VIPM__EINT64_T_S64()           bo_int64_t
# define VIPM__EINT64_T_F32()           bo_int64_t
# define VIPM__EINT64_T_F64()           bo_int64_t

# define VIPM__EUINT_T_U8()             bo_uint8_t
# define VIPM__EUINT_T_U16()            bo_uint16_t
# define VIPM__EUINT_T_U32()            bo_uint32_t
# define VIPM__EUINT_T_U64()            bo_uint64_t
# define VIPM__EUINT_T_S8()             bo_uint8_t
# define VIPM__EUINT_T_S16()            bo_uint16_t
# define VIPM__EUINT_T_S32()            bo_uint32_t
# define VIPM__EUINT_T_S64()            bo_uint64_t
# define VIPM__EUINT_T_F32()            bo_uint32_t
# define VIPM__EUINT_T_F64()            bo_uint64_t

# define VIPM__ESINT_T_U8()             bo_int8_t
# define VIPM__ESINT_T_U16()            bo_int16_t
# define VIPM__ESINT_T_U32()            bo_int32_t
# define VIPM__ESINT_T_U64()            bo_int64_t
# define VIPM__ESINT_T_S8()             bo_int8_t
# define VIPM__ESINT_T_S16()            bo_int16_t
# define VIPM__ESINT_T_S32()            bo_int32_t
# define VIPM__ESINT_T_S64()            bo_int64_t
# define VIPM__ESINT_T_F32()            bo_int32_t
# define VIPM__ESINT_T_F64()            bo_int64_t

# define VIPM__EFINT_T_U8()             bo_int32_t
# define VIPM__EFINT_T_U16()            bo_int32_t
# define VIPM__EFINT_T_U32()            bo_int32_t
# define VIPM__EFINT_T_U64()            bo_int64_t
# define VIPM__EFINT_T_S8()             bo_int32_t
# define VIPM__EFINT_T_S16()            bo_int32_t
# define VIPM__EFINT_T_S32()            bo_int32_t
# define VIPM__EFINT_T_S64()            bo_int64_t
# define VIPM__EFINT_T_F32()            bo_int32_t /* @todo: fixme - unsigned(?) */
# define VIPM__EFINT_T_F64()            bo_int64_t /* @todo: fixme - unsigned(?) */

# define VIPM__EFINT32_T_U8()           bo_int32_t
# define VIPM__EFINT32_T_U16()          bo_int32_t
# define VIPM__EFINT32_T_U32()          bo_int32_t
# define VIPM__EFINT32_T_S8()           bo_int32_t
# define VIPM__EFINT32_T_S16()          bo_int32_t
# define VIPM__EFINT32_T_S32()          bo_int32_t
# define VIPM__EFINT32_T_F32()          bo_int32_t /* @todo: fixme - unsigned(?) */

# define VIPM__EWORK_T_U8()             int
# define VIPM__EWORK_T_U16()            int
# define VIPM__EWORK_T_U32()            bo_uint32_t
# define VIPM__EWORK_T_U64()            bo_uint64_t
# define VIPM__EWORK_T_S8()             int
# define VIPM__EWORK_T_S16()            int
# define VIPM__EWORK_T_S32()            bo_int32_t
# define VIPM__EWORK_T_S64()            bo_int64_t
# define VIPM__EWORK_T_F32()            float
# define VIPM__EWORK_T_F64()            double

# define VIPM__EHWORK_T_U8()            short
# define VIPM__EHWORK_T_U16()           int
# define VIPM__EHWORK_T_U32()           bo_uint32_t
# define VIPM__EHWORK_T_U64()           bo_uint64_t
# define VIPM__EHWORK_T_S8()            short
# define VIPM__EHWORK_T_S16()           int
# define VIPM__EHWORK_T_S32()           bo_int32_t
# define VIPM__EHWORK_T_S64()           bo_int64_t
# define VIPM__EHWORK_T_F32()           float
# define VIPM__EHWORK_T_F64()           double

# define VIPM__EMPAR_T_U8()             unsigned
# define VIPM__EMPAR_T_U16()            unsigned
# define VIPM__EMPAR_T_U32()            bo_uint32_t
# define VIPM__EMPAR_T_U64()            bo_uint64_t
# define VIPM__EMPAR_T_S8()             int
# define VIPM__EMPAR_T_S16()            int
# define VIPM__EMPAR_T_S32()            bo_int32_t
# define VIPM__EMPAR_T_S64()            bo_int64_t
# define VIPM__EMPAR_T_F32()            double
# define VIPM__EMPAR_T_F64()            double

# define VipmK__ARITH_ETRUE_U8()        0xFF
# define VipmK__ARITH_ETRUE_U16()       0xFFFF
# define VipmK__ARITH_ETRUE_U32()       0xFFFFFFFF
# define VipmK__ARITH_ETRUE_U64()       0xFFFFFFFFFFFFFFFF
# define VipmK__ARITH_ETRUE_S8()        0xFF
# define VipmK__ARITH_ETRUE_S16()       0xFFFF
# define VipmK__ARITH_ETRUE_S32()       0xFFFFFFFF
# define VipmK__ARITH_ETRUE_S64()       0xFFFFFFFFFFFFFFFF
/* F32/F64 constants based on IEEE-754 */
# define VipmK__ARITH_ETRUE_F32()       1.9999999F /* 0xFFFFFFp-23F; hex:0x3fffffff; old: 1.F */
# define VipmK__ARITH_ETRUE_F64()       1.9999999999999998 /* 0x1FFFFFFFFFFFFFp-52; hex:0x3fffffffffffffff; old: 1. */

# define VipmK__ARITH_EFALSE_U8()       0
# define VipmK__ARITH_EFALSE_U16()      0
# define VipmK__ARITH_EFALSE_U32()      0
# define VipmK__ARITH_EFALSE_U64()      0
# define VipmK__ARITH_EFALSE_S8()       0
# define VipmK__ARITH_EFALSE_S16()      0
# define VipmK__ARITH_EFALSE_S32()      0
# define VipmK__ARITH_EFALSE_S64()      0
# define VipmK__ARITH_EFALSE_F32()      0.F
# define VipmK__ARITH_EFALSE_F64()      0.



# define VIPM__PIXWIDTH_18()            8
# define VIPM__PIXWIDTH_28()            16
# define VIPM__PIXWIDTH_38()            24
# define VIPM__PIXWIDTH_48()            32
# define VIPM__PIXWIDTH_116()           16
# define VIPM__PIXWIDTH_216()           32
# define VIPM__PIXWIDTH_316()           48
# define VIPM__PIXWIDTH_416()           64
# define VIPM__PIXWIDTH_132()           32
# define VIPM__PIXWIDTH_232()           64
# define VIPM__PIXWIDTH_332()           96
# define VIPM__PIXWIDTH_432()           128
# define VIPM__PIXWIDTH_164()           64
# define VIPM__PIXWIDTH_264()           128
# define VIPM__PIXWIDTH_364()           192
# define VIPM__PIXWIDTH_464()           256

# define VIPM__PIXSTRIDE_11()           1
# define VIPM__PIXSTRIDE_21()           2
# define VIPM__PIXSTRIDE_31()           3
# define VIPM__PIXSTRIDE_41()           4
# define VIPM__PIXSTRIDE_12()           2
# define VIPM__PIXSTRIDE_22()           4
# define VIPM__PIXSTRIDE_32()           6
# define VIPM__PIXSTRIDE_42()           8
# define VIPM__PIXSTRIDE_14()           4
# define VIPM__PIXSTRIDE_24()           8
# define VIPM__PIXSTRIDE_34()           12
# define VIPM__PIXSTRIDE_44()           16
# define VIPM__PIXSTRIDE_18()           8
# define VIPM__PIXSTRIDE_28()           16
# define VIPM__PIXSTRIDE_38()           24
# define VIPM__PIXSTRIDE_48()           32


# define VIPM__WIDTHOF_PIXEL_1U8()      VIPM__PIXWIDTH_18()
# define VIPM__WIDTHOF_PIXEL_2U8()      VIPM__PIXWIDTH_28()
# define VIPM__WIDTHOF_PIXEL_3U8()      VIPM__PIXWIDTH_38()
# define VIPM__WIDTHOF_PIXEL_4U8()      VIPM__PIXWIDTH_48()
# define VIPM__WIDTHOF_PIXEL_1S8()      VIPM__PIXWIDTH_18()
# define VIPM__WIDTHOF_PIXEL_2S8()      VIPM__PIXWIDTH_28()
# define VIPM__WIDTHOF_PIXEL_3S8()      VIPM__PIXWIDTH_38()
# define VIPM__WIDTHOF_PIXEL_4S8()      VIPM__PIXWIDTH_48()
# define VIPM__WIDTHOF_PIXEL_1U16()     VIPM__PIXWIDTH_116()
# define VIPM__WIDTHOF_PIXEL_2U16()     VIPM__PIXWIDTH_216()
# define VIPM__WIDTHOF_PIXEL_3U16()     VIPM__PIXWIDTH_316()
# define VIPM__WIDTHOF_PIXEL_4U16()     VIPM__PIXWIDTH_416()
# define VIPM__WIDTHOF_PIXEL_1S16()     VIPM__PIXWIDTH_116()
# define VIPM__WIDTHOF_PIXEL_2S16()     VIPM__PIXWIDTH_216()
# define VIPM__WIDTHOF_PIXEL_3S16()     VIPM__PIXWIDTH_316()
# define VIPM__WIDTHOF_PIXEL_4S16()     VIPM__PIXWIDTH_416()
# define VIPM__WIDTHOF_PIXEL_1U32()     VIPM__PIXWIDTH_132()
# define VIPM__WIDTHOF_PIXEL_2U32()     VIPM__PIXWIDTH_232()
# define VIPM__WIDTHOF_PIXEL_3U32()     VIPM__PIXWIDTH_332()
# define VIPM__WIDTHOF_PIXEL_4U32()     VIPM__PIXWIDTH_432()
# define VIPM__WIDTHOF_PIXEL_1S32()     VIPM__PIXWIDTH_132()
# define VIPM__WIDTHOF_PIXEL_2S32()     VIPM__PIXWIDTH_232()
# define VIPM__WIDTHOF_PIXEL_3S32()     VIPM__PIXWIDTH_332()
# define VIPM__WIDTHOF_PIXEL_4S32()     VIPM__PIXWIDTH_432()
# define VIPM__WIDTHOF_PIXEL_1U64()     VIPM__PIXWIDTH_164()
# define VIPM__WIDTHOF_PIXEL_2U64()     VIPM__PIXWIDTH_264()
# define VIPM__WIDTHOF_PIXEL_3U64()     VIPM__PIXWIDTH_364()
# define VIPM__WIDTHOF_PIXEL_4U64()     VIPM__PIXWIDTH_464()
# define VIPM__WIDTHOF_PIXEL_1S64()     VIPM__PIXWIDTH_164()
# define VIPM__WIDTHOF_PIXEL_2S64()     VIPM__PIXWIDTH_264()
# define VIPM__WIDTHOF_PIXEL_3S64()     VIPM__PIXWIDTH_364()
# define VIPM__WIDTHOF_PIXEL_4S64()     VIPM__PIXWIDTH_464()
# define VIPM__WIDTHOF_PIXEL_1F32()     VIPM__PIXWIDTH_132()
# define VIPM__WIDTHOF_PIXEL_2F32()     VIPM__PIXWIDTH_232()
# define VIPM__WIDTHOF_PIXEL_3F32()     VIPM__PIXWIDTH_332()
# define VIPM__WIDTHOF_PIXEL_4F32()     VIPM__PIXWIDTH_432()
# define VIPM__WIDTHOF_PIXEL_1F64()     VIPM__PIXWIDTH_164()
# define VIPM__WIDTHOF_PIXEL_2F64()     VIPM__PIXWIDTH_264()
# define VIPM__WIDTHOF_PIXEL_3F64()     VIPM__PIXWIDTH_364()
# define VIPM__WIDTHOF_PIXEL_4F64()     VIPM__PIXWIDTH_464()

# define VIPM__SIZEOF_PIXEL_1U8()       VIPM__PIXSTRIDE_11()
# define VIPM__SIZEOF_PIXEL_2U8()       VIPM__PIXSTRIDE_21()
# define VIPM__SIZEOF_PIXEL_3U8()       VIPM__PIXSTRIDE_31()
# define VIPM__SIZEOF_PIXEL_4U8()       VIPM__PIXSTRIDE_41()
# define VIPM__SIZEOF_PIXEL_1S8()       VIPM__PIXSTRIDE_11()
# define VIPM__SIZEOF_PIXEL_2S8()       VIPM__PIXSTRIDE_21()
# define VIPM__SIZEOF_PIXEL_3S8()       VIPM__PIXSTRIDE_31()
# define VIPM__SIZEOF_PIXEL_4S8()       VIPM__PIXSTRIDE_41()
# define VIPM__SIZEOF_PIXEL_1U16()      VIPM__PIXSTRIDE_12()
# define VIPM__SIZEOF_PIXEL_2U16()      VIPM__PIXSTRIDE_22()
# define VIPM__SIZEOF_PIXEL_3U16()      VIPM__PIXSTRIDE_32()
# define VIPM__SIZEOF_PIXEL_4U16()      VIPM__PIXSTRIDE_42()
# define VIPM__SIZEOF_PIXEL_1S16()      VIPM__PIXSTRIDE_12()
# define VIPM__SIZEOF_PIXEL_2S16()      VIPM__PIXSTRIDE_22()
# define VIPM__SIZEOF_PIXEL_3S16()      VIPM__PIXSTRIDE_32()
# define VIPM__SIZEOF_PIXEL_4S16()      VIPM__PIXSTRIDE_42()
# define VIPM__SIZEOF_PIXEL_1U32()      VIPM__PIXSTRIDE_14()
# define VIPM__SIZEOF_PIXEL_2U32()      VIPM__PIXSTRIDE_24()
# define VIPM__SIZEOF_PIXEL_3U32()      VIPM__PIXSTRIDE_34()
# define VIPM__SIZEOF_PIXEL_4U32()      VIPM__PIXSTRIDE_44()
# define VIPM__SIZEOF_PIXEL_1S32()      VIPM__PIXSTRIDE_14()
# define VIPM__SIZEOF_PIXEL_2S32()      VIPM__PIXSTRIDE_24()
# define VIPM__SIZEOF_PIXEL_3S32()      VIPM__PIXSTRIDE_34()
# define VIPM__SIZEOF_PIXEL_4S32()      VIPM__PIXSTRIDE_44()
# define VIPM__SIZEOF_PIXEL_1U64()      VIPM__PIXSTRIDE_18()
# define VIPM__SIZEOF_PIXEL_2U64()      VIPM__PIXSTRIDE_28()
# define VIPM__SIZEOF_PIXEL_3U64()      VIPM__PIXSTRIDE_38()
# define VIPM__SIZEOF_PIXEL_4U64()      VIPM__PIXSTRIDE_48()
# define VIPM__SIZEOF_PIXEL_1S64()      VIPM__PIXSTRIDE_18()
# define VIPM__SIZEOF_PIXEL_2S64()      VIPM__PIXSTRIDE_28()
# define VIPM__SIZEOF_PIXEL_3S64()      VIPM__PIXSTRIDE_38()
# define VIPM__SIZEOF_PIXEL_4S64()      VIPM__PIXSTRIDE_48()
# define VIPM__SIZEOF_PIXEL_1F32()      VIPM__PIXSTRIDE_14()
# define VIPM__SIZEOF_PIXEL_2F32()      VIPM__PIXSTRIDE_24()
# define VIPM__SIZEOF_PIXEL_3F32()      VIPM__PIXSTRIDE_34()
# define VIPM__SIZEOF_PIXEL_4F32()      VIPM__PIXSTRIDE_44()
# define VIPM__SIZEOF_PIXEL_1F64()      VIPM__PIXSTRIDE_18()
# define VIPM__SIZEOF_PIXEL_2F64()      VIPM__PIXSTRIDE_28()
# define VIPM__SIZEOF_PIXEL_3F64()      VIPM__PIXSTRIDE_38()
# define VIPM__SIZEOF_PIXEL_4F64()      VIPM__PIXSTRIDE_48()


# define VIPM__ELTYCAST_U8U8(v)        (v)
# define VIPM__ELTYCAST_U8S8(v)         vipm_satcast_u8s8(v)
# define VIPM__ELTYCAST_U8U16(v)        vipm_satcast_u8uint(v)
# define VIPM__ELTYCAST_U8S16(v)        vipm_satcast_u8sint(v)
# define VIPM__ELTYCAST_U8U32(v)        vipm_satcast_u8uint(v)
# define VIPM__ELTYCAST_U8S32(v)        vipm_satcast_u8sint(v)
# define VIPM__ELTYCAST_U8S64(v)        vipm_satcast_u8s64(v)
# define VIPM__ELTYCAST_U8F32(v)        vipm_satcast_u8f(v)
# define VIPM__ELTYCAST_U8F64(v)        vipm_satcast_u8d(v)

# define VIPM__ELTYCAST_S8U8(v)         vipm_satcast_s8u8(v)
# define VIPM__ELTYCAST_S8S8(v)        (v)
# define VIPM__ELTYCAST_S8U16(v)        vipm_satcast_s8uint(v)
# define VIPM__ELTYCAST_S8S16(v)        vipm_satcast_s8sint(v)
# define VIPM__ELTYCAST_S8U32(v)        vipm_satcast_s8uint(v)
# define VIPM__ELTYCAST_S8S32(v)        vipm_satcast_s8sint(v)
# define VIPM__ELTYCAST_S8S64(v)        vipm_satcast_s8s64(v)
# define VIPM__ELTYCAST_S8F32(v)        vipm_satcast_s8f(v)
# define VIPM__ELTYCAST_S8F64(v)        vipm_satcast_s8d(v)

# define VIPM__ELTYCAST_U16U8(v)       (v)
# define VIPM__ELTYCAST_U16S8(v)        vipm_satcast_u16s16(v)
# define VIPM__ELTYCAST_U16U16(v)      (v)
# define VIPM__ELTYCAST_U16S16(v)       vipm_satcast_u16s16(v)
# define VIPM__ELTYCAST_U16U32(v)       vipm_satcast_u16uint(v)
# define VIPM__ELTYCAST_U16S32(v)       vipm_satcast_u16sint(v)
# define VIPM__ELTYCAST_U16S64(v)       vipm_satcast_u16s64(v)
# define VIPM__ELTYCAST_U16F32(v)       vipm_satcast_u16f(v)
# define VIPM__ELTYCAST_U16F64(v)       vipm_satcast_u16d(v)

# define VIPM__ELTYCAST_S16U8(v)       (v)
# define VIPM__ELTYCAST_S16S8(v)       (v)
# define VIPM__ELTYCAST_S16U16(v)       vipm_satcast_s16u16(v)
# define VIPM__ELTYCAST_S16S16(v)      (v)
# define VIPM__ELTYCAST_S16U32(v)       vipm_satcast_s16uint(v)
# define VIPM__ELTYCAST_S16S32(v)       vipm_satcast_s16sint(v)
# define VIPM__ELTYCAST_S16S64(v)       vipm_satcast_s16s64(v)
# define VIPM__ELTYCAST_S16F32(v)       vipm_satcast_s16f(v)
# define VIPM__ELTYCAST_S16F64(v)       vipm_satcast_s16d(v)

# define VIPM__ELTYCAST_U32U8(v)       (v)
# define VIPM__ELTYCAST_U32S8(v)        vipm_satcast_u32s32(v)
# define VIPM__ELTYCAST_U32U16(v)      (v)
# define VIPM__ELTYCAST_U32S16(v)       vipm_satcast_u32s32(v)
# define VIPM__ELTYCAST_U32U32(v)      (v)
# define VIPM__ELTYCAST_U32S32(v)       vipm_satcast_u32s32(v)
# define VIPM__ELTYCAST_U32S64(v)       vipm_satcast_u32s64(v)
# define VIPM__ELTYCAST_U32F32(v)       vipm_satcast_u32f(v)
# define VIPM__ELTYCAST_U32F64(v)       vipm_satcast_u32d(v)

# define VIPM__ELTYCAST_S32U8(v)       (v)
# define VIPM__ELTYCAST_S32S8(v)       (v)
# define VIPM__ELTYCAST_S32U16(v)      (v)
# define VIPM__ELTYCAST_S32S16(v)      (v)
# define VIPM__ELTYCAST_S32U32(v)       vipm_satcast_s32u32(v)
# define VIPM__ELTYCAST_S32S32(v)      (v)
# define VIPM__ELTYCAST_S32S64(v)       vipm_satcast_s32s64(v)
# define VIPM__ELTYCAST_S32F32(v)       vipm_satcast_s32f(v)
# define VIPM__ELTYCAST_S32F64(v)       vipm_satcast_s32d(v)

# define VIPM__ELTYCAST_F32U8(v)       ((float)(v))
# define VIPM__ELTYCAST_F32S8(v)       ((float)(v))
# define VIPM__ELTYCAST_F32U16(v)      ((float)(v))
# define VIPM__ELTYCAST_F32S16(v)      ((float)(v))
# define VIPM__ELTYCAST_F32U32(v)      ((float)(v))
# define VIPM__ELTYCAST_F32S32(v)      ((float)(v))
# define VIPM__ELTYCAST_F32S64(v)      ((float)(v))
# define VIPM__ELTYCAST_F32F32(v)      (v)
# define VIPM__ELTYCAST_F32F64(v)       vipm_satcast_f32d(v)
	/* @} */


/* @todo: fixme - move to ? */
/* @sa: "Vipm/Cdefs.h" */
# ifndef VIPM_PP_W2SZ
#  define VIPM_PP_W2SZ(W)               VIPM_GLUE(VIPM_PP__W2SZ_,W)()
/* @{ */
#  define VIPM_PP__W2SZ_1()             0
#  define VIPM_PP__W2SZ_2()             0
#  define VIPM_PP__W2SZ_4()             0
#  define VIPM_PP__W2SZ_8()             1
#  define VIPM_PP__W2SZ_16()            2
#  define VIPM_PP__W2SZ_32()            4
#  define VIPM_PP__W2SZ_64()            8
#  define VIPM_PP__W2SZ_128()           16

#  define VIPM_PP__W2SZ_I8()            1
#  define VIPM_PP__W2SZ_I16()           2
#  define VIPM_PP__W2SZ_I32()           4
#  define VIPM_PP__W2SZ_I64()           8
#  define VIPM_PP__W2SZ_I128()          16
#  define VIPM_PP__W2SZ_S8()            1
#  define VIPM_PP__W2SZ_S16()           2
#  define VIPM_PP__W2SZ_S32()           4
#  define VIPM_PP__W2SZ_S64()           8
#  define VIPM_PP__W2SZ_S128()          16
#  define VIPM_PP__W2SZ_U8()            1
#  define VIPM_PP__W2SZ_U16()           2
#  define VIPM_PP__W2SZ_U32()           4
#  define VIPM_PP__W2SZ_U64()           8
#  define VIPM_PP__W2SZ_U128()          16
#  define VIPM_PP__W2SZ_F16()           2
#  define VIPM_PP__W2SZ_F32()           4
#  define VIPM_PP__W2SZ_F64()           8
#  define VIPM_PP__W2SZ_F128()          16
/* @} */


#  define VIPM_PP_WDIV2(W)              VIPM_GLUE(VIPM_PP__WDIV2_,W)()
/* @{ */
#  define VIPM_PP__WDIV2_0()            0
#  define VIPM_PP__WDIV2_1()            0
#  define VIPM_PP__WDIV2_2()            1
#  define VIPM_PP__WDIV2_4()            2
#  define VIPM_PP__WDIV2_8()            4
#  define VIPM_PP__WDIV2_16()           8
#  define VIPM_PP__WDIV2_32()           16
#  define VIPM_PP__WDIV2_64()           32
#  define VIPM_PP__WDIV2_128()          64
#  define VIPM_PP__WDIV2_256()          128

//#  define VIPM_PP__WDIV2_I8()
#  define VIPM_PP__WDIV2_I16()          I8
#  define VIPM_PP__WDIV2_I32()          I16
#  define VIPM_PP__WDIV2_I64()          I32
#  define VIPM_PP__WDIV2_I128()         I64
//#  define VIPM_PP__WDIV2_S8()
#  define VIPM_PP__WDIV2_S16()          S8
#  define VIPM_PP__WDIV2_S32()          S16
#  define VIPM_PP__WDIV2_S64()          S32
#  define VIPM_PP__WDIV2_S128()         S64
//#  define VIPM_PP__WDIV2_U8()
#  define VIPM_PP__WDIV2_U16()          U8
#  define VIPM_PP__WDIV2_U32()          U16
#  define VIPM_PP__WDIV2_U64()          U32
#  define VIPM_PP__WDIV2_U128()         U64
//#  define VIPM_PP__WDIV2_F16()
#  define VIPM_PP__WDIV2_F32()          F16
#  define VIPM_PP__WDIV2_F64()          F32
#  define VIPM_PP__WDIV2_F128()         F64
/* @} */


#  define VIPM_PP_WMUL2(W)              VIPM_GLUE(VIPM_PP__WMUL2_,W)()
/* @{ */
#  define VIPM_PP__WMUL2_0()            0
#  define VIPM_PP__WMUL2_1()            2
#  define VIPM_PP__WMUL2_2()            4
#  define VIPM_PP__WMUL2_4()            8
#  define VIPM_PP__WMUL2_8()            16
#  define VIPM_PP__WMUL2_16()           32
#  define VIPM_PP__WMUL2_32()           64
#  define VIPM_PP__WMUL2_64()           128
#  define VIPM_PP__WMUL2_128()          256
#  define VIPM_PP__WMUL2_256()          512

#  define VIPM_PP__WMUL2_I8()           I16
#  define VIPM_PP__WMUL2_I16()          I32
#  define VIPM_PP__WMUL2_I32()          I64
#  define VIPM_PP__WMUL2_I64()          I128
#  define VIPM_PP__WMUL2_I128()         I256
#  define VIPM_PP__WMUL2_S8()           S16
#  define VIPM_PP__WMUL2_S16()          S32
#  define VIPM_PP__WMUL2_S32()          S64
#  define VIPM_PP__WMUL2_S64()          S128
#  define VIPM_PP__WMUL2_S128()         S256
#  define VIPM_PP__WMUL2_U8()           U16
#  define VIPM_PP__WMUL2_U16()          U32
#  define VIPM_PP__WMUL2_U32()          U64
#  define VIPM_PP__WMUL2_U64()          U128
#  define VIPM_PP__WMUL2_U128()         U256
//#  define VIPM_PP__WMUL2_F8()           F16
#  define VIPM_PP__WMUL2_F16()          F32
#  define VIPM_PP__WMUL2_F32()          F64
#  define VIPM_PP__WMUL2_F64()          F128
#  define VIPM_PP__WMUL2_F128()         F256
/* @} */


#  define VIPM_PP_WMUL4(W)              VIPM_GLUE(VIPM_PP__WMUL4_,W)()
/* @{ */
#  define VIPM_PP__WMUL4_0()            0
#  define VIPM_PP__WMUL4_1()            4
#  define VIPM_PP__WMUL4_2()            8
#  define VIPM_PP__WMUL4_4()            16
#  define VIPM_PP__WMUL4_8()            32
#  define VIPM_PP__WMUL4_16()           64
#  define VIPM_PP__WMUL4_32()           128
#  define VIPM_PP__WMUL4_64()           256
#  define VIPM_PP__WMUL4_128()          512
#  define VIPM_PP__WMUL4_256()          1024

#  define VIPM_PP__WMUL4_I8()           I32
#  define VIPM_PP__WMUL4_I16()          I64
#  define VIPM_PP__WMUL4_I32()          I128
#  define VIPM_PP__WMUL4_I64()          I256
#  define VIPM_PP__WMUL4_I128()         I512
#  define VIPM_PP__WMUL4_S8()           S32
#  define VIPM_PP__WMUL4_S16()          S64
#  define VIPM_PP__WMUL4_S32()          S128
#  define VIPM_PP__WMUL4_S64()          S256
#  define VIPM_PP__WMUL4_S128()         S512
#  define VIPM_PP__WMUL4_U8()           U32
#  define VIPM_PP__WMUL4_U16()          U64
#  define VIPM_PP__WMUL4_U32()          U128
#  define VIPM_PP__WMUL4_U64()          U256
#  define VIPM_PP__WMUL4_U128()         U512
//#  define VIPM_PP__WMUL4_F8()           F32
#  define VIPM_PP__WMUL4_F16()          F64
#  define VIPM_PP__WMUL4_F32()          F128
#  define VIPM_PP__WMUL4_F64()          F256
#  define VIPM_PP__WMUL4_F128()         F512
/* @} */


#  define VIPM_PP_WMUL8(W)              VIPM_GLUE(VIPM_PP__WMUL8_,W)()
/* @{ */
#  define VIPM_PP__WMUL8_0()            0
#  define VIPM_PP__WMUL8_1()            8
#  define VIPM_PP__WMUL8_2()            16
#  define VIPM_PP__WMUL8_4()            32
#  define VIPM_PP__WMUL8_8()            64
#  define VIPM_PP__WMUL8_16()           128
#  define VIPM_PP__WMUL8_32()           256
#  define VIPM_PP__WMUL8_64()           512
#  define VIPM_PP__WMUL8_128()          1024
#  define VIPM_PP__WMUL8_256()          2048

#  define VIPM_PP__WMUL8_I8()           I64
#  define VIPM_PP__WMUL8_I16()          I128
#  define VIPM_PP__WMUL8_I32()          I256
#  define VIPM_PP__WMUL8_I64()          I512
#  define VIPM_PP__WMUL8_I128()         I1024
#  define VIPM_PP__WMUL8_S8()           S64
#  define VIPM_PP__WMUL8_S16()          S128
#  define VIPM_PP__WMUL8_S32()          S256
#  define VIPM_PP__WMUL8_S64()          S512
#  define VIPM_PP__WMUL8_S128()         S1024
#  define VIPM_PP__WMUL8_U8()           U64
#  define VIPM_PP__WMUL8_U16()          U128
#  define VIPM_PP__WMUL8_U32()          U256
#  define VIPM_PP__WMUL8_U64()          U512
#  define VIPM_PP__WMUL8_U128()         U1024
//#  define VIPM_PP__WMUL8_F8()           F64
#  define VIPM_PP__WMUL8_F16()          F128
#  define VIPM_PP__WMUL8_F32()          F256
#  define VIPM_PP__WMUL8_F64()          F512
#  define VIPM_PP__WMUL8_F128()         F1024
/* @} */
# endif /* !defined(VIPM_PP_W2SZ) */

/* @} */

/*
 * Channel/Elem-Type constants/macros
 * @}
 */

#endif /* !defined(__VIPM__TYPES_IMEL__TYPES_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
