/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Scalar.h	(V.Sokur)
 * Vipm/_Types/Scalar.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES_SCALAR_H__
# define __VIPM__TYPES_SCALAR_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif

# ifndef __VIPM__TYPES_BASE__TYPES_H__
#  include "Vipm/_Types/Base_Types.h"
# endif
# ifndef __VIPM__TYPES_BASE__LIMITS_H__
#  include "Vipm/_Types/Base_Limits.h"
# endif
# ifndef __VIPM__TYPES_BASE__CAST_H__
#  include "Vipm/_Types/Base_Cast.h"
# endif

# ifndef __VIPM__TYPES_IMEL__TYPES_H__
#  include "Vipm/_Types/Imel_Types.h"
# endif
 */


/******************************************************************************
 * %% BeginSection: type/constant declarations / definitions
 */

/*
 * @{
 * Scalar definitions
 */

/*
 * @sa: VipmK_ELEMTYPE_F64
 */
typedef double                          vipm_scalar_t;
typedef vipm_scalar_t                  *vipm_scalar_par_t; /* @note: not for your business */

struct _Vipm_sclpair {
	union {
		vipm_scalar_t const    *__imuv[2];
		vipm_scalar_t          *__mutv[2];
		struct {
			vipm_scalar_t const    *first,
	                               *second;
		} imu;
		struct {
			vipm_scalar_t          *first,
	                               *second;
		} mut;
	};
};
typedef struct _Vipm_sclpair            vipm_sclpair_t;


# define VipmK_SCALAR_MIN           DBL_MIN
# define VipmK_SCALAR_MAX           DBL_MAX
# define VipmK_SCALAR_INF           VipmK_F64_INF
# define VipmK_SCALAR_NAN           VipmK_F64_NAN
# define VipmK_SCALAR_CMT           VipmK_SCALAR_INF   /* scalar value should be computed */
# define VipmK_SCALAR_DFL           VipmK_SCALAR_NAN   /* "default" scalar value should be used */

# define VIPM_SCALAR_ISNAN(x)       vipmi_isnand(x)
# define VIPM_SCALAR_ISINF(x)       vipmi_isinfd(x)
# define VIPM_SCALAR_ISINVAL(x)     VIPM_SCALAR_ISNAN(x)

# define VIPM_SCALAR_COMPUTE(x)     VIPM_SCALAR_ISINF(x)
# define VIPM_SCALAR_DEFAULT(x)     VIPM_SCALAR_ISNAN(x)

# define vipm_satcast_u8scl(v)      vipm_satcast_u8d(v)
# define vipm_satcast_s8scl(v)      vipm_satcast_s8d(v)
# define vipm_satcast_u16scl(v)     vipm_satcast_u16d(v)
# define vipm_satcast_s16scl(v)     vipm_satcast_s16d(v)
# define vipm_satcast_u32scl(v)     vipm_satcast_u32d(v)
# define vipm_satcast_s32scl(v)     vipm_satcast_s32d(v)
/* # define vipm_satcast_u64scl(v)     vipm_satcast_u64d(v) */
/* # define vipm_satcast_s64scl(v)     vipm_satcast_s64d(v) */
# define vipm_satcast_f32scl(v)     vipm_satcast_f32d(v)
# define vipm_satcast_f64scl(v)     vipm_satcast_f64d(v)

# define __vipm_satcast_scl(ElemType,x)                                          \
    (  VipmK_ELEMTYPE_U8   == ElemType ? (vipm_scalar_t)vipm_satcast_u8scl(x)    \
     : VipmK_ELEMTYPE_S8   == ElemType ? (vipm_scalar_t)vipm_satcast_s8scl(x)    \
     : VipmK_ELEMTYPE_U16  == ElemType ? (vipm_scalar_t)vipm_satcast_u16scl(x)   \
     : VipmK_ELEMTYPE_S16  == ElemType ? (vipm_scalar_t)vipm_satcast_s16scl(x)   \
     : VipmK_ELEMTYPE_U32  == ElemType ? (vipm_scalar_t)vipm_satcast_u32scl(x)   \
     : VipmK_ELEMTYPE_S32  == ElemType ? (vipm_scalar_t)vipm_satcast_s32scl(x)   \
     /* : VipmK_ELEMTYPE_U64  == ElemType ? (vipm_scalar_t)vipm_satcast_u64scl(x) */   \
     /* : VipmK_ELEMTYPE_S64  == ElemType ? (vipm_scalar_t)vipm_satcast_s64scl(x) */   \
     : VipmK_ELEMTYPE_F32  == ElemType ? (vipm_scalar_t)vipm_satcast_f32scl(x)   \
     : VipmK_ELEMTYPE_F64  == ElemType ? (vipm_scalar_t)vipm_satcast_f64scl(x)   \
                                       : (x) )

#if 0
# define vipm_satcast_scl(ElemType,x) __vipm_satcast_scl(ElemType,x)
#else
__MYC_BEGIN_CDECLS
             vipm_scalar_t vipm_satcast_scl __P((vipm_uint, vipm_scalar_t));
_VIPM_INLINE vipm_scalar_t vipm_satcast_scl(vipm_uint e, vipm_scalar_t x) { return __vipm_satcast_scl(e,x); }
__MYC_END_CDECLS
#endif

/* @} */

#endif /* !defined(__VIPM__TYPES_SCALAR_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
