/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Tree.h	(V.Sokur)
 * Vipm/_Types/Tree.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES_TREE_H__
# define __VIPM__TYPES_TREE_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include "Vipm/Cdefs.h"
# endif

# ifndef __VIPM__TYPES_BASE__TYPES_H__
#  include "Vipm/_Types/Base_Types.h"
# endif

# ifndef __VIPM__TYPES_XENV_H__
#  include "Vipm/_Types/Xenv.h"
# endif
	// vipm_mmstate_t

# ifndef __BO_MEMMAN_TYPES__
#  include <Bo/memman/Types.h>
# endif
	// bo_memstg_t
 */


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

/* tree traversal */
# define VipmK_TT_PREORDER              0x0001
# define VipmK_TT_INORDER               0x0002
# define VipmK_TT_POSTORDER             0x0003

/*
 *    0 : stop walk
 *   >0 : continue walk
 *
 * additionally for preorder:
 *   -1 : continue walk, skip childern
 * <(-1): continue walk, skip children, skip following sibling
 */
typedef int  (*vipm_tt_visitor_fn) __P((void *aNode, void *, vipm_int));



struct _Vipm_agtreenode;
struct _Vipm_agtree;

struct _Vipm_dgtreenode;
struct _Vipm_dgtree;

struct _Vipm_stree;
struct _Vipm_streebuilder;


typedef void (*vipm_agt_construct_fn) __P((struct _Vipm_agtree *, void *));
typedef void (*vipm_agt_destroy_fn)   __P((struct _Vipm_agtree *, void *));
typedef void (*vipm_agt_dispose_fn)   __P((/* struct _Vipm_agtree */void *));

typedef void (*vipm_agt_ddestroy_fn)  __P((struct _Vipm_agtreenode *, void *));

/*
 *    0 : stop walk
 *   >0 : continue walk
 *
 * additionally for preorder:
 *   -1 : continue walk, skip childern
 * <(-1): continue walk, skip children, skip following sibling
 */
typedef int  (*vipm_agt_visitor_fn)   __P((struct _Vipm_agtreenode *, void *, vipm_int));



struct _Vipm_agtreenode {
	struct _Vipm_agtreenode        *gtn_next;
	struct _Vipm_agtreenode        *gtn_child;      /* gt_root; */
	struct _Vipm_agtreenode        *gtn_parent;
};
typedef struct _Vipm_agtreenode         vipm_agtreenode_t;
# define vipm_agtreenode_p(x)           vipm_agtreenode_cp(x)
# define vipm_agtreenode_cp(x)         ((struct _Vipm_agtreenode const *)(x))
# define vipm_agtreenode_mp(x)         ((struct _Vipm_agtreenode *)(x))

struct _Vipm_agtree {
	union {
		struct _Vipm_agtreenode    *gt_root;
		struct _Vipm_agtreenode    *gtn_next;
	};
};
typedef struct _Vipm_agtree             vipm_agtree_t;
# define vipm_agtree_p(x)               vipm_agtree_cp(x)
# define vipm_agtree_cp(x)             ((struct _Vipm_agtree const *)(x))
# define vipm_agtree_mp(x)             ((struct _Vipm_agtree *)(x))


struct _Vipm_dgtreenode {
	struct _Vipm_dgtreenode        *gtn_next;
	struct _Vipm_dgtreenode        *gtn_child;
	struct _Vipm_dgtreenode        *gtn_parent;
	void                           *gtn_data;
};
typedef struct _Vipm_dgtreenode         vipm_dgtreenode_t;
# define vipm_dgtreenode_p(x)           vipm_dgtreenode_cp(x)
# define vipm_dgtreenode_cp(x)         ((struct _Vipm_dgtreenode const *)(x))
# define vipm_dgtreenode_mp(x)         ((struct _Vipm_dgtreenode *)(x))

struct _Vipm_dgtdata {
	void                           *dgt_node;
};
typedef struct _Vipm_dgtdata            vipm_dgtdata_t;
# define vipm_dgtdata_p(x)              vipm_dgtdata_cp(x)
# define vipm_dgtdata_cp(x)            ((struct _Vipm_dgtdata const *)(x))
# define vipm_dgtdata_mp(x)            ((struct _Vipm_dgtdata *)(x))

struct _Vipm_dgtree {
	/* struct _Vipm_agtree { */
	union {
		struct _Vipm_dgtreenode    *gt_root;
		struct _Vipm_dgtreenode    *gtn_next;
	};
	/* } */
	vipm_agt_construct_fn           gt_custom_construct;
	vipm_agt_destroy_fn             gt_custom_destroy;
	vipm_agt_dispose_fn             gt_dispose;
};
typedef struct _Vipm_dgtree             vipm_dgtree_t;
# define vipm_dgtree_p(x)               vipm_dgtree_cp(x)
# define vipm_dgtree_cp(x)             ((struct _Vipm_dgtree const *)(x))
# define vipm_dgtree_mp(x)             ((struct _Vipm_dgtree *)(x))


# define _Vipm_streenode                _Vipm_dgtreenode
typedef struct _Vipm_streenode          vipm_streenode_t;
# define vipm_streenode_p(x)            vipm_streenode_cp(x)
# define vipm_streenode_cp(x)          ((struct _Vipm_streenode const *)(x))
# define vipm_streenode_mp(x)          ((struct _Vipm_streenode *)(x))

struct _Vipm_stree {
	/* struct _Vipm_dgtree { */
	union {
		struct _Vipm_dgtreenode    *gt_root;
		struct _Vipm_dgtreenode    *gtn_next;
	};
	vipm_agt_construct_fn           gt_custom_construct;
	vipm_agt_destroy_fn             gt_custom_destroy;
	vipm_agt_dispose_fn             gt_dispose;
	/* } */
	struct _Vipm_streebuilder      *st_builder;
};
typedef struct _Vipm_stree              vipm_stree_t;
# define vipm_stree_p(x)                vipm_stree_cp(x)
# define vipm_stree_cp(x)              ((struct _Vipm_stree const *)(x))
# define vipm_stree_mp(x)              ((struct _Vipm_stree *)(x))

struct _Vipm_streebuilder {
	volatile struct _Vipm_xinterface   *stb_xitf;
	void                               *stb_xdata;

	struct _Vipm_stree                 *stb_tree;

	/* internal @{ */
	vipm_ushort                         stb_flags_;
	vipm_ushort                         stb_xbytec_;
	vipm_rref_t                         stb_mmstack_;
	vipm_rref_t                         stb_pmmstack_;      /* parent mmstack */
	bo_memstg_t                         stb_mmstackb_;
	vipm_mmstate_t                      stb_mmstate_;
	struct _Vipm_stree                  stb_treeb_;
	/* @} */
};
typedef struct _Vipm_streebuilder       vipm_streebuilder_t;
# define vipm_streebuilder_p(x)         vipm_streebuilder_cp(x)
# define vipm_streebuilder_cp(x)       ((struct _Vipm_streebuilder const *)(x))
# define vipm_streebuilder_mp(x)       ((struct _Vipm_streebuilder *)(x))

# define __VipmF_STB_INITED             0x01
# define __VipmF_STB_SELFALLOC          0x10
# define __VipmF_STB_XTRN_MMST_OPENED   0x20


struct _Vipm_stbxitf {
	/* struct _Vipm_xinterface { */
	bo_uint32_t                     magic; /* 'stbl' */
	bo_uint32_t                     size;
	/* } */
	vipm_agt_ddestroy_fn          (*getddtor)        __P((struct _Vipm_streebuilder *));
	void                          (*attached)        __P((struct _Vipm_streebuilder *));
	void                          (*become_detached) __P((struct _Vipm_streebuilder *));
};
typedef struct _Vipm_stbxitf            vipm_stbxitf_t;
# define vipm_stbxitf_p(x)              vipm_stbxitf_cp(x)
# define vipm_stbxitf_cp(x)            ((struct _Vipm_stbxitf const *)(x))
# define vipm_stbxitf_mp(x)            ((struct _Vipm_stbxitf *)(x))


#endif /* !defined(__VIPM__TYPES_TREE_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
