/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * _Comparing.h	(V.Sokur)
 * Vipm/_Types/_Comparing.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES__COMPARING_H__
# define __VIPM__TYPES__COMPARING_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif
# ifndef __VIPM__TYPES_CORE__TYPES_H__
#  include "Vipm/_Types/Core_Types.h"
# endif
 */


/******************************************************************************
 * %% BeginSection: constant declarations / definitions
 */

/* ----------------------------------------------------------------------------
 * @{
 * Compare types
 */

/*
 * Special [pointer] values
 * meaning the result [pixel] value should be set
 * same as destination or source(lhs) value.
 *
 * Useful for the fields '_Vipm_tga_ecmp{c}parm.tga_[t,f]fval'.
 */

# define VipmK_ECMP_FVEQDST             NULL
# define VipmK_ECMP_FVEQLHS            ((void const *)1)
# define VIPM_ECMP_SFVAL(x)            (((bo_uintptr_t)(x) & ~(bo_uintptr_t)0xF) == 0)


/*
 * Compare TGA declarations
 */

struct _Vipm_tga_cmpcparm {
	vipm_int                tga_cmpop;
	vipm_scalar_t const    *tga_cmprhs;
};
typedef struct _Vipm_tga_cmpcparm       vipm_tga_cmpcparm_t;
# define vipm_tga_cmpcparm_p(x)         vipm_tga_cmpcparm_cp(x)
# define vipm_tga_cmpcparm_cp(x)       ((struct _Vipm_tga_cmpcparm const *)(x))
# define vipm_tga_cmpcparm_mp(x)       ((struct _Vipm_tga_cmpcparm *)(x))


struct _Vipm_tga_ecmpparm {
	vipm_int                tga_cmpop;
	vipm_scalar_t const    *tga_tfval;
	vipm_scalar_t const    *tga_ffval;
};
typedef struct _Vipm_tga_ecmpparm       vipm_tga_ecmpparm_t;
# define vipm_tga_ecmpparm_p(x)         vipm_tga_ecmpparm_cp(x)
# define vipm_tga_ecmpparm_cp(x)       ((struct _Vipm_tga_ecmpparm const *)(x))
# define vipm_tga_ecmpparm_mp(x)       ((struct _Vipm_tga_ecmpparm *)(x))

struct _Vipm_tga_ecmpcparm {
	vipm_int                tga_cmpop;
	vipm_scalar_t const    *tga_cmprhs;
	vipm_scalar_t const    *tga_tfval;
	vipm_scalar_t const    *tga_ffval;
};
typedef struct _Vipm_tga_ecmpcparm      vipm_tga_ecmpcparm_t;
# define vipm_tga_ecmpcparm_p(x)        vipm_tga_ecmpcparm_cp(x)
# define vipm_tga_ecmpcparm_cp(x)      ((struct _Vipm_tga_ecmpcparm const *)(x))
# define vipm_tga_ecmpcparm_mp(x)      ((struct _Vipm_tga_ecmpcparm *)(x))

/*
 * Compare types
 * @}
 */

/* ----------------------------------------------------------------------------
 * @{
 * Threshold types
 */

/*
 * Threshold method mask
 *
 * To get the threshold method the field '_Vipm_threshopts.mtho_method'
 * should be masked with VipmF_THRESH_MTDM.
 */
# define VipmF_THRESH_MTDM              0x0F
# define VipmF_THRESH_DFL_OPTS          0x10

/*
 * Value for the field '_Vipm_threshopts.mtho_method'
 */
# define VipmK_THRESH_NULL_METHOD       0
# define VipmK_THRESH_IMM_METHOD        VipmK_THRESH_NULL_METHOD
# define VipmK_THRESH_UNK_METHOD        2
# define VipmK_THRESH_OTSU_METHOD       3
# define VipmK_THRESH_USTO_METHOD       4
# define VipmK_THRESH_BLUR_METHOD       5


/*
 * Value for the field '_Vipm_threshopts.[otsu|usto].mtho_vrange[0]'
 */
# define VipmK_THRESH_DFL_VRANGE        VipmK_DFL_VRANGE
# define VipmK_THRESH_CMT_VRANGE        VipmK_CMT_VRANGE
# define VIPM_THRESH_DFL_VRANGE(x)      VIPM_DFL_VRANGE(x)
# define VIPM_THRESH_CMT_VRANGE(x)      VIPM_CMT_VRANGE(x)

/*
 * Value for the field '_Vipm_threshopts.usto.mtho_global_thresh'
 */
# define VipmK_INVAL_THRESH             VipmK_SCALAR_NAN
# define VipmK_CMT_THRESH               VipmK_SCALAR_CMT        /* threshold value should be computed */
# define VIPM_INVAL_THRESH(x)           VIPM_SCALAR_ISNAN(x)
# define VIPM_CMT_THRESH(x)             VIPM_SCALAR_COMPUTE(x)


struct __Vipm_threshopts_otsu {
	vipm_range_t            mtho_vrangev[VipmK_MAXCHANNELS];
	vipm_int                mtho_nscaleptv[VipmK_MAXCHANNELS];
# define mtho_nbinsv        mtho_nscaleptv
};

struct __Vipm_threshopts_usto {
	vipm_range_t            mtho_vrangev[VipmK_MAXCHANNELS];
	vipm_int                mtho_nscaleptv[VipmK_MAXCHANNELS];
# define mtho_nbinsv        mtho_nscaleptv
	vipm_Nvolume_t          mtho_kvol0, mtho_kvol1;
};

struct __Vipm_threshopts_blur {
	vipm_seshape_t          mtho_seshape;
};

struct _Vipm_threshopts {
	bo_int32_t                          mtho_method;
	union {
		struct __Vipm_threshopts_otsu   otsu;
		struct __Vipm_threshopts_usto   usto;
		struct __Vipm_threshopts_blur   blur;
	};
};
typedef struct _Vipm_threshopts         vipm_threshopts_t;
# define vipm_threshopts_p(x)           vipm_threshopts_cp(x)
# define vipm_threshopts_cp(x)         ((struct _Vipm_threshopts const *)(x))
# define vipm_threshopts_mp(x)         ((struct _Vipm_threshopts *)(x))


/*
 * @note: explanation of 'usto' threshold options
 *
 * 1. (.mtho_global_factor = 0 & .mtho_local_factor = 0)   -->  defaults
 * 2. (.mtho_global_factor = 0 & .mtho_local_factor != 0)  -->  (.mtho_global_thresh := 0)
 * 3. (.mtho_global_factor != 0 & .mtho_local_factor = 0)  -->  value of .mtho_global_factor ignored
 * 4. (.mtho_global_factor != 0 & .mtho_local_factor = 0)  -->  value of .mtho_global_thresh ignored
 * 5. (.mtho_local_factor = 0)                             -->  value of .mtho_kvol ignored
 * 6. (.mtho_global_factor != 0 &
 *      .mtho_local_factor != 0 &
 *     .mtho_global_thresh = VipmK_CMT_THRESH)       -->  (.mtho_global_thresh := compute-threshold-by-otsu{global})
 *
 * 7.
 *     7.1 (.mtho_vrange[0] = VipmK_THRESH_CMT_VRANGE)     -->  (.mtho_vrange := calculate-value-range{global})
 *     7.2 (.mtho_vrange[1] < .mtho_vrange[0])             -->  (.mtho_vrange := calculate-value-range{global})
 *     7.3 (.mtho_vrange[0] = VipmK_THRESH_DFL_VRANGE)     -->  (.mtho_vrange := default-value-range{global},
 *                                                               .mtho_nbins := default-num-scalepoints)
 *     7.4 (.mtho_nbins = 0)                               -->  (.mtho_nbins := default-num-scalepoints)
 *
 *
 * defaults:
 *         thopts.usto.mtho_global_factor = 1;
 *         thopts.usto.mtho_local_factor = 0;
 *         thopts.usto.mtho_global_thresh = VipmK_CMT_THRESH;       // ignored
 *         thopts.usto.mtho_vrange.val[0] = VipmK_THRESH_DFL_VRANGE;  // use default
 *         thopts.usto.mtho_nbins = 0;                              // use default
 *
 *    default values of .mtho_vrange and .mtho_nbins:
 *         1. Float elem-type:
 *                 (.mtho_nbins := VipmK_STD_VRANGEF_NSCALEPT,
 *                  .mtho_vrange[0] := VipmK_STD_VRANGEF_MIN,
 *                  .mtho_vrange[1] := VipmK_STD_VRANGEF_MAX)
 *         2. Integer elem-type:
 *                 (.mtho_nbins := VipmK_STD_VRANGEI_NSCALEPT,
 *                  .mtho_vrange[0] := VipmK_STD_VRANGEI_MIN,
 *                  .mtho_vrange[1] := VipmK_STD_VRANGEI_MAX)
 */



/*
 * @sa: VipmF_CMPOP_MASK
 * To get the compare-op the field '_Vipm_threshparm.mthp_cmpop'
 * should be masked with VipmF_CMPOP_MASK.
 */

/*
 * Special value for the field '_Vipm_threshparm.mthp_thvar' (.mthp_threshv[0]),
 * meaning the "adaptive" thresholding.
 *
 * @note: when "adaptive" thresholding:
 *     thparmv[1,2,...].mthp_thvar   -  should be set to VipmK_THPARM_ADAPT too
 */
# define VipmK_THPARM_ADAPT             VipmK_SCALAR_NAN
# define VIPM_THPARM_ADAPT(x)           VIPM_SCALAR_ISNAN(x)

/*
 * Special [pixel] value for the field '_Vipm_threshparm.mthp_valv',
 * meaning the result [pixel] value should be set same as threshold value.
 *
 * @note: cannot be used with adaptive threshold (blur-method, ....)
 */
# define VipmK_THPARM_VEQTH             VipmK_SCALAR_NAN
# define VIPM_THPARM_VEQTH(x)           VIPM_SCALAR_ISNAN(x)

/*
 * Max. number of threshold parameters in a specified list of params
 */
# ifndef VipmK_THPARM_MAX
#  define VipmK_THPARM_MAX              2
# endif


struct _Vipm_threshparm {
	bo_int32_t              mthp_cmpop;
	union {
		/* simple threshold */
		vipm_scalar_t       mthp_threshv[VipmK_MAXCHANNELS];

		/* adaptive threshold */
		struct {
			vipm_scalar_t   mthp_thvar;
			vipm_array_t    mthp_tharr;
			char            mthp_pad[(VipmK_MAXCHANNELS - 1) * sizeof(vipm_scalar_t) - sizeof(vipm_array_t)];
		};
	};
	vipm_scalar_t           mthp_valv[VipmK_MAXCHANNELS];
};
typedef struct _Vipm_threshparm         vipm_threshparm_t;
# define vipm_threshparm_p(x)           vipm_threshparm_cp(x)
# define vipm_threshparm_cp(x)         ((struct _Vipm_threshparm const *)(x))
# define vipm_threshparm_mp(x)         ((struct _Vipm_threshparm *)(x))


/*
 * "Immediate" threshold:
 * Sample 1:
 *    ADDR(thopts) = nil or
 *    thopts.mtho_method = VipmK_THRESH_NULL_METHOD or VipmK_THRESH_IMM_METHOD;
 *    thparmv[0].mthp_cmpop = VipmK_LESS_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[0].mthp_threshv[cn] = T(cn);
 * 	      thparmv[0].mthp_valv[cn] = 0.0F;
 *    }
 *    thparmv[1].mthp_cmpop = VipmK_GEQU_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[1].mthp_threshv[cn] = T(cn);
 * 	      thparmv[1].mthp_valv[cn] = 255.0F;
 *    }
 *    thparmc = 2;
 *
 * Sample 2:
 *    thopts.mtho_method = VipmK_THRESH_NULL_METHOD or VipmK_THRESH_IMM_METHOD;
 *    thparmv[0].mthp_cmpop = VipmK_LESS_CMP;
 *    thparmv[0].mthp_thvar = VipmK_THPARM_ADAPT;
 *    thparmv[0].mthp_tharr = some-arr-used-as-threashold-surface;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[0].mthp_valv[cn] = 0.0F;
 *    }
 *    thparmv[1].mthp_cmpop = VipmK_GEQU_CMP;
 *    thparmv[1].mthp_thvar = VipmK_THPARM_ADAPT;
 *    thparmv[1].mthp_tharr = some-arr-used-as-threashold-surface;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[1].mthp_valv[cn] = 255.0F;
 *    }
 *    thparmc = 2;
 *
 *
 * Threshold by Otsu:
 * Sample 1:
 *    thopts.mtho_method = VipmK_THRESH_OTSU_METHOD;
 *    thopts.usto.mtho_vrange.val[0] = VipmK_THRESH_DFL_VRANGE; // use default
 *    thopts.usto.mtho_nbins = 0;                             // use default
 *    thparmv[0].mthp_cmpop = VipmK_LESS_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[0].mthp_valv[cn] = 0.0F;
 *    }
 *    thparmv[1].mthp_cmpop = VipmK_GEQU_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[1].mthp_valv[cn] = 255.0F;
 *    }
 *    thparmc = 2;
 *
 * Sample 2:
 *    thopts.mtho_method = VipmK_THRESH_OTSU_METHOD | VipmF_THRESH_DFL_OPTS;
 *    thparmv[0].mthp_cmpop = VipmK_LESS_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[0].mthp_valv[cn] = 0.0F;
 *    }
 *    thparmv[1].mthp_cmpop = VipmK_GEQU_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[1].mthp_valv[cn] = 255.0F;
 *    }
 *    thparmc = 2;
 *
 *
 * Adaptive threshold (by blur):
 * Sample 1:
 *    thopts.mtho_method = VipmK_THRESH_BLUR_METHOD;
 *    thopts.blur.mtho_seshape.sel_vol.sz_width = 7;
 *    thopts.blur.mtho_seshape.sel_vol.sz_height = 7;
 *    thopts.blur.mtho_seshape.sel_anchor.pi_x = thopts.blur.mtho_seshape.sel_vol.sz_width / 2;
 *    thopts.blur.mtho_seshape.sel_anchor.pi_y = thopts.blur.mtho_seshape.sel_vol.sz_height / 2;
 *    thparmv[0].mthp_cmpop = VipmK_LESS_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[0].mthp_valv[cn] = 0.0F;
 *    }
 *    thparmv[1].mthp_cmpop = VipmK_GEQU_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[1].mthp_valv[cn] = 255.0F;
 *    }
 *    thparmc = 2;
 *
 * Sample 2:
 *    thopts.mtho_method = VipmK_THRESH_BLUR_METHOD | VipmF_THRESH_DFL_OPTS;
 *    thparmv[0].mthp_cmpop = VipmK_LESS_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[0].mthp_valv[cn] = 0.0F;
 *    }
 *    thparmv[1].mthp_cmpop = VipmK_GEQU_CMP;
 *    for (cn = 0; cn < nchannels; cn++) {
 * 	      thparmv[1].mthp_valv[cn] = 255.0F;
 *    }
 *    thparmc = 2;
 */



/*
 * Thresholding TGA declarations
 */

struct _Vipm_tga_thhparm {
	struct _Vipm_threshopts const  *tga_thopts;
	struct _Vipm_threshparm const  *tga_thparms;
	vipm_int                        tga_thparmc;
	vipm_arrshape_t const          *tga_whole_sshape;
};
typedef struct _Vipm_tga_thhparm        vipm_tga_thhparm_t;
# define vipm_tga_thhparm_p(x)          vipm_tga_thhparm_cp(x)
# define vipm_tga_thhparm_cp(x)        ((struct _Vipm_tga_thhparm const *)(x))
# define vipm_tga_thhparm_mp(x)        ((struct _Vipm_tga_thhparm *)(x))


struct _Vipm_tga_compthhparm {
	vipm_scalar_t          *tga_threshv;
	vipm_range_t           *tga_vrangev;
	vipm_int               *tga_nbinsv;
};
typedef struct _Vipm_tga_compthhparm    vipm_tga_compthhparm_t;
# define vipm_tga_compthhparm_p(x)      vipm_tga_compthhparm_cp(x)
# define vipm_tga_compthhparm_cp(x)    ((struct _Vipm_tga_compthhparm const *)(x))
# define vipm_tga_compthhparm_mp(x)    ((struct _Vipm_tga_compthhparm *)(x))

/*
 * Threshold types
 * @}
 */

#endif /* !defined(__VIPM__TYPES__COMPARING_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
