/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * _Filtering.h	(V.Sokur)
 * Vipm/_Types/_Filtering.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES__FILTERING_H__
# define __VIPM__TYPES__FILTERING_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif

# ifndef __VIPM__TYPES_CORE__TYPES_H__
#  include "Vipm/_Types/Core_Types.h"
# endif
 */


/******************************************************************************
 * %% BeginSection: constant declarations / definitions
 */

/*
 * @{
 * Filtering/Morphology Types
 */

/* ----------------------------------------------------------------------------
 * @{
 * Filtering:Filter
 */
# pragma mark -
# pragma mark [ Filtering:Filtering ]
# pragma mark -

struct _Vipm_filtopts {
	vipm_int                    flo_brtype;
	vipm_scalar_t               flo_brval[VipmK_MAXCHANNELS];
};
typedef struct _Vipm_filtopts           vipm_filtopts_t;
# define vipm_filtopts_p(x)             vipm_filtopts_cp(x)
# define vipm_filtopts_cp(x)           ((struct _Vipm_filtopts const *)(x))
# define vipm_filtopts_mp(x)           ((struct _Vipm_filtopts *)(x))


struct _Vipm_tga_filtparm {
	int                         tga_method;
	vipm_seshape_t const       *tga_strel;
	vipm_filtopts_t const      *tga_opts;           /* = NULL */
	vipm_arrshape_t const      *tga_whole_sshape;   /* = NULL */
};
typedef struct _Vipm_tga_filtparm       vipm_tga_filtparm_t;
# define vipm_tga_filtparm_p(x)         vipm_tga_filtparm_cp(x)
# define vipm_tga_filtparm_cp(x)       ((struct _Vipm_tga_filtparm const *)(x))
# define vipm_tga_filtparm_mp(x)       ((struct _Vipm_tga_filtparm *)(x))

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Filtering:Morphology
 */
# pragma mark -
# pragma mark [ Filtering:Morphology ]
# pragma mark -

#define _Vipm_morphopts                 _Vipm_filtopts
typedef struct _Vipm_morphopts          vipm_morphopts_t;
# define vipm_morphopts_p(x)            vipm_morphopts_cp(x)
# define vipm_morphopts_cp(x)          ((struct _Vipm_morphopts const *)(x))
# define vipm_morphopts_mp(x)          ((struct _Vipm_morphopts *)(x))


struct _Vipm_tga_morphparm {
	int                         tga_method;
	vipm_strel_t const         *tga_strel;
	vipm_morphopts_t const     *tga_opts;           /* = NULL */
	vipm_arrshape_t const      *tga_whole_sshape;   /* = NULL */
};
typedef struct _Vipm_tga_morphparm      vipm_tga_morphparm_t;
# define vipm_tga_morphparm_p(x)        vipm_tga_morphparm_cp(x)
# define vipm_tga_morphparm_cp(x)      ((struct _Vipm_tga_morphparm const *)(x))
# define vipm_tga_morphparm_mp(x)      ((struct _Vipm_tga_morphparm *)(x))

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Filtering:Firefly
 */
# pragma mark -
# pragma mark [ Filtering:Firefly ]
# pragma mark -

#define _Vipm_firflopts                 _Vipm_filtopts
typedef struct _Vipm_firflopts          vipm_firflopts_t;
# define vipm_firflopts_p(x)            vipm_firflopts_cp(x)
# define vipm_firflopts_cp(x)          ((struct _Vipm_firflopts const *)(x))
# define vipm_firflopts_mp(x)          ((struct _Vipm_firflopts *)(x))


struct _Vipm_tga_firflparm {
	bo_intptr_t                 tga_method;
	vipm_seshape_t const       *tga_strel;
	vipm_firflopts_t const     *tga_opts;           /* = NULL */
	vipm_arrshape_t const      *tga_whole_sshape;   /* = NULL */
};
typedef struct _Vipm_tga_firflparm      vipm_tga_firflparm_t;
# define vipm_tga_firflparm_p(x)        vipm_tga_firflparm_cp(x)
# define vipm_tga_firflparm_cp(x)      ((struct _Vipm_tga_firflparm const *)(x))
# define vipm_tga_firflparm_mp(x)      ((struct _Vipm_tga_firflparm *)(x))

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Filtering:FireflyEx
 */
# pragma mark -
# pragma mark [ Filtering:FireflyEx ]
# pragma mark -

#define _Vipm_fflexopts                 _Vipm_filtopts
typedef struct _Vipm_fflexopts          vipm_fflexopts_t;
# define vipm_fflexopts_p(x)            vipm_fflexopts_cp(x)
# define vipm_fflexopts_cp(x)          ((struct _Vipm_fflexopts const *)(x))
# define vipm_fflexopts_mp(x)          ((struct _Vipm_fflexopts *)(x))


struct _Vipm_tga_fflexparm {
	bo_intptr_t                 tga_method;
	vipm_int const             *tga_weights;
	vipm_int                    tga_funccnt;
	vipm_seshape_t const       *tga_strel;
	vipm_fflexopts_t const     *tga_opts;           /* = NULL */
	vipm_arrshape_t const      *tga_whole_sshape;   /* = NULL */
};
typedef struct _Vipm_tga_fflexparm      vipm_tga_fflexparm_t;
# define vipm_tga_fflexparm_p(x)        vipm_tga_fflexparm_cp(x)
# define vipm_tga_fflexparm_cp(x)      ((struct _Vipm_tga_fflexparm const *)(x))
# define vipm_tga_fflexparm_mp(x)      ((struct _Vipm_tga_fflexparm *)(x))

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Filtering:Gaussian
 */
# pragma mark -
# pragma mark [ Filtering:Gaussian ]
# pragma mark -

struct _Vipm_tga_gfltparm {
	vipm_seshape_t const       *tga_strel;
	vipm_scalar_t const        *tga_sigmas;         /* = NULL */
	vipm_filtopts_t const      *tga_opts;           /* = NULL */
	vipm_arrshape_t const      *tga_whole_sshape;   /* = NULL */
};
typedef struct _Vipm_tga_gfltparm       vipm_tga_gfltparm_t;
# define vipm_tga_gfltparm_p(x)         vipm_tga_gfltparm_cp(x)
# define vipm_tga_gfltparm_cp(x)       ((struct _Vipm_tga_gfltparm const *)(x))
# define vipm_tga_gfltparm_mp(x)       ((struct _Vipm_tga_gfltparm *)(x))

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Filtering:Separable
 */
# pragma mark -
# pragma mark [ Filtering:Gaussian ]
# pragma mark -

struct _Vipm_tga_sfltparm {
	vipm_sepkern_t const       *tga_sepkern;
	vipm_filtopts_t const      *tga_opts;           /* = NULL */
	vipm_arrshape_t const      *tga_whole_sshape;   /* = NULL */
};
typedef struct _Vipm_tga_sfltparm       vipm_tga_sfltparm_t;
# define vipm_tga_sfltparm_p(x)         vipm_tga_sfltparm_cp(x)
# define vipm_tga_sfltparm_cp(x)       ((struct _Vipm_tga_sfltparm const *)(x))
# define vipm_tga_sfltparm_mp(x)       ((struct _Vipm_tga_sfltparm *)(x))

/* @} */

#endif /* !defined(__VIPM__TYPES__FILTERING_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
