/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * _Statistics.h	(V.Sokur)
 * Vipm/_Types/_Statistics.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM__TYPES__STATISTICS_H__
# define __VIPM__TYPES__STATISTICS_H__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifndef __VIPM_CDEFS_H__
#  include <Vipm/Cdefs.h>
# endif

# ifndef __VIPM__TYPES_CORE__TYPES_H__
#  include "Vipm/_Types/Core_Types.h"
# endif
 */


/******************************************************************************
 * %% BeginSection: constant declarations / definitions
 */

/*
 * @{
 * Statistics Types
 */

/* ----------------------------------------------------------------------------
 * @{
 * Statistics:Histogram
 */
# pragma mark -
# pragma mark [ Statistics:Histogram ]
# pragma mark -

struct _Vipm_histogram {
	struct {
		bo_int32_t          mhg_intensity;
		bo_int32_t          mhg_minval, mhg_maxval,     /* min/max value of the image pixels */
		                    mhg_eovpt,                  /* extent of values of the image pixels in points; = (maxv - minv) + 1 */
		                    mhg_delta;                  /* = (eovpt - 1) / (nbins - 1) */
	} isp;                                              /* discrete */
	struct {
		bo_float32_t        mhg_intensity;
		bo_float32_t        mhg_minval, mhg_maxval,     /* min/max value of the image */
		                    mhg_eov,                    /* extent of values of the image pixels; = (maxv - minv) */
		                    mhg_delta;                  /* = eov / (nbins - 1) */
	} fsp;                                              /* continuous */

	bo_int32_t              mhg_pixc;
	union {
		bo_int32_t          mhg_nbins;
		bo_int32_t          mhg_nscalepoints;
	};
	vipm_int               *mhg_binv;                   /* probability distribution */
};
typedef struct _Vipm_histogram          vipm_histogram_t;
# define vipm_histogram_p(x)            vipm_histogram_cp(x)
# define vipm_histogram_cp(x)          ((struct _Vipm_histogram const *)(x))
# define vipm_histogram_mp(x)          ((struct _Vipm_histogram *)(x))
# define vipm_histogram(...)            VIPM_CTOR_FCNAME(vipm_histogram, __VA_ARGS__)
# define vipm_histogram_(...)           VIPM_PFX_FCNAME(vipm_histogram, __VA_ARGS__)
# define vipm_hist_(...)                VIPM_PFX_FCNAME(vipm_hist, __VA_ARGS__)

# define VIPM_HG_SizeofBinv(aHist) ((bo_size_t)((aHist)->mhg_nbins + 1) * sizeof(vipm_uint))

/*
 * Commands for 'histogram' operation
 */
# define VipmK_HG_CALCULATE             0
/* Currently, should not be used
 * @{
 */
# define VipmK_HG_UPDATE_LSHIFT         1
# define VipmK_HG_UPDATE_RSHIFT         2
# define VipmK_HG_UPDATE_DSHIFT         3
/* @} */


struct _Vipm_hginitarg {
	vipm_uint               mhgp_elemtype;
	vipm_int                mhgp_nbins;
	vipm_range_t            mhgp_vrange;                  /* min/max value of the image */
	vipm_int                mhgp_vol[VipmK_MAXGDIMS];
};
typedef struct _Vipm_hginitarg          vipm_hginitarg_t;
# define vipm_hginitarg_p(x)            vipm_hginitarg_cp(x)
# define vipm_hginitarg_cp(x)          ((struct _Vipm_hginitarg const *)(x))
# define vipm_hginitarg_mp(x)          ((struct _Vipm_hginitarg *)(x))

# define VipmK_HGI_HDR                  0
# define VipmK_HGI_ALL                  1
# define VipmK_HGI_REINIT               2

/* @} */

/* ----------------------------------------------------------------------------
 * @{
 * Statistics:MinMax
 */
# pragma mark -
# pragma mark [ Statistics:MinMax ]
# pragma mark -

struct _Vipm_minmaxinfo {
	bo_uint32_t             xi_bm;
	struct {
		bo_int32_t              ival;
		bo_float32_t            fval;
		union {
			vipm_Npoint_t       loc;
			struct {
				vipm_int        plane;
				vipm_int        row, col;
				vipm_int        channel_;
			};
		};
	}                       xi_min[VipmK_MAXCHANNELS];
	struct {
		bo_int32_t              ival;
		bo_float32_t            fval;
		union {
			vipm_Npoint_t       loc;
			struct {
				vipm_int        plane;
				vipm_int        row, col;
				vipm_int        channel_;
			};
		};
	}                       xi_max[VipmK_MAXCHANNELS];
};
typedef struct _Vipm_minmaxinfo         vipm_minmaxinfo_t;
# define vipm_minmaxinfo_p(x)           vipm_minmaxinfo_cp(x)
# define vipm_minmaxinfo_cp(x)         ((struct _Vipm_minmaxinfo const *)(x))
# define vipm_minmaxinfo_mp(x)         ((struct _Vipm_minmaxinfo *)(x))

# define XI_FMIN(cn)    xi_min[cn].fval
# define XI_IMIN(cn)    xi_min[cn].ival
# define XI_ROWMIN(cn)  xi_min[cn].row
# define XI_COLMIN(cn)  xi_min[cn].col

# define XI_FMAX(cn)    xi_max[cn].fval
# define XI_IMAX(cn)    xi_max[cn].ival
# define XI_ROWMAX(cn)  xi_max[cn].row
# define XI_COLMAX(cn)  xi_max[cn].col


/*
 * Value for the field '_Vipm_minmaxinfo.xi_bm'
 */
# define VipmF_MINMAX_ACM               0x7
# define VipmF_MINMAX_MIN               0x1
# define VipmF_MINMAX_MAX               0x2
# define VipmF_MINMAX_IDX               0x4
# define VipmF_MINMAX_ALL               VipmF_MINMAX_ACM
# define VipmF_MINMAX_VALUES           (VipmF_MINMAX_MIN | VipmF_MINMAX_MAX)


/*
 * #include <Vipm/Utils/ValueRange.h>
 */
# define VIPM_MINMAX_TO_VRNG(xi,vrng,cn)    vipm_vrng(cn,minmax)(vrng,cn,xi)

/* @} */

/*
 * Statistics Types
 * @}
 */

#endif /* !defined(__VIPM__TYPES__STATISTICS_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
