/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2019Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Aas.h    (D.Vinnyk)
 * Vipm/machine/arm/Aas.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ARM_AAS__
# define __VIPM_ARM_AAS__


/******************************************************************************
 * %% BeginSection: function definitions
 */

/*
 * @{
 */

/* @{ */
# define _AAS_IMOD2(Imm)            _ARM_IMOD2(Imm)
# define _AAS_IMOD4(Imm)            _ARM_IMOD4(Imm)

# define _AAS_W2SZ(W)               _ARM_W2SZ(W)
# define _AAS_WDIV2(W)              _ARM_WDIV2(W)
# define _AAS_WMUL2(W)              _ARM_WMUL2(W)
# define _AAS_WMUL4(W)              _ARM_WMUL4(W)

# define _AAS_COND_MAP(_Cond)       _ARM_COND_MAP(_Cond)
# define _AAS_CCOND_MAP(_Cond)      _ARM_CCOND_MAP(_Cond)

# define _AAS_WIDTHOF_PTR           _ARM_WIDTHOF_PTR
# define _AAS_PTR_OFFSET(Offset)    _ARM_PTR_OFFSET(Offset)
/* @} */


/* ========================================================================= */

/*
 * Move operations
 * @{
 */

# define _AAS_MOVI                  _ARM_MOVI
# define _AAS_MOVI_UCF              _ARM_MOVI_UCF
# define _AAS_MOVI_IF               _ARM_MOVI_IF
# define _AAS_MOVI_UCF_IF           _ARM_MOVI_UCF_IF

# define _AAS_MOV                   _ARM_MOV
# define _AAS_MOV_UCF               _ARM_MOV_UCF
# define _AAS_MOV_IF                _ARM_MOV_IF
# define _AAS_MOV_UCF_IF            _ARM_MOV_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 *
 * signed  : (x) --> (-2**(sat-1) <= x <= 2**(sat-1) -1)
 * unsigned: (x) --> (0 <= x <= 2**sat -1)
 * @{
 */

# ifdef _ARM_SATURATE_S
#  define _AAS_SATURATE_S           _ARM_SATURATE_S
#  define _AAS_SATURATE_U           _ARM_SATURATE_U
# endif

/* ------------------------------------------------------------------------- */

# define _AAS_RND_FTOSIEX           _ARM_RND_FTOSIEX
# define _AAS_RND_FTOUIEX           _ARM_RND_FTOUIEX

    /*
     * rounding convert float to signed/unsigned integer
     */
# define _AAS_RND_FTOSI32           _ARM_RND_FTOSI32
# define _AAS_RND_FTOUI32           _ARM_RND_FTOUI32
# define _AAS_RND_FTOSI64           _ARM_RND_FTOSI64
# define _AAS_RND_FTOUI64           _ARM_RND_FTOUI64
# define _AAS_RND_FTOSI             _ARM_RND_FTOSI
# define _AAS_RND_FTOUI             _ARM_RND_FTOUI

    /*
     * rounding convert double to signed/unsigned integer
     */
# define _AAS_RND_F64TOSI32         _ARM_RND_F64TOSI32
# define _AAS_RND_F64TOUI32         _ARM_RND_F64TOUI32
# define _AAS_RND_F64TOSI64         _ARM_RND_F64TOSI64
# define _AAS_RND_F64TOUI64         _ARM_RND_F64TOUI64
# define _AAS_RND_F64TOSI           _ARM_RND_F64TOSI
# define _AAS_RND_F64TOUI           _ARM_RND_F64TOUI

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * NEGATE: (a) --> (-a)
 * ADD : (a,b) --> (a + b)
 *		ADD[I]        : add
 *		ADD[I]_UCF    : add updating the condition flags
 *		ADD[I]_IF     : [conditionally] add
 *		ADD[I]_UCF_IF : [conditionally] add updating the condition flags
 * SUB : (a,b) --> (a - b)
 * RSUB: (a,b) --> (b - a)
 * MUL : (a,b) --> (a * b)
 */

/* ------------------------------------------------------------------------- */

    /*
     * NEGATE
	 * @{
     */
# define _AAS_NEGATE                _ARM_NEGATE
# define _AAS_NEGATE_UCF            _ARM_NEGATE_UCF
# define _AAS_NEGATE_IF             _ARM_NEGATE_IF
# define _AAS_NEGATE_UCF_IF         _ARM_NEGATE_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * ADD
	 * @{
     */
# define _AAS_ADDI                  _ARM_ADDI
# define _AAS_ADDI_UCF              _ARM_ADDI_UCF
# define _AAS_ADDI_IF               _ARM_ADDI_IF
# define _AAS_ADDI_UCF_IF           _ARM_ADDI_UCF_IF

# define _AAS_ADD                   _ARM_ADD
# define _AAS_ADD_UCF               _ARM_ADD_UCF
# define _AAS_ADD_IF                _ARM_ADD_IF
# define _AAS_ADD_UCF_IF            _ARM_ADD_UCF_IF
	/* @} */

    /*
     * ADDEX
	 * @{
     */
# define _AAS_ADDEX                 _ARM_ADDEX
# define _AAS_ADDEX_UCF             _ARM_ADDEX_UCF
# define _AAS_ADDEX_IF              _ARM_ADDEX_IF
# define _AAS_ADDEX_UCF_IF          _ARM_ADDEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * SUB
	 * @{
     */
# define _AAS_SUBI                  _ARM_SUBI
# define _AAS_SUBI_UCF              _ARM_SUBI_UCF
# define _AAS_SUBI_IF               _ARM_SUBI_IF
# define _AAS_SUBI_UCF_IF           _ARM_SUBI_UCF_IF

# define _AAS_SUB                   _ARM_SUB
# define _AAS_SUB_UCF               _ARM_SUB_UCF
# define _AAS_SUB_IF                _ARM_SUB_IF
# define _AAS_SUB_UCF_IF            _ARM_SUB_UCF_IF
	/* @} */

    /*
     * SUBEX
	 * @{
     */
# define _AAS_SUBEX                 _ARM_SUBEX
# define _AAS_SUBEX_UCF             _ARM_SUBEX_UCF
# define _AAS_SUBEX_IF              _ARM_SUBEX_IF
# define _AAS_SUBEX_UCF_IF          _ARM_SUBEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * RSUB
	 * @{
     */
# define _AAS_RSUBI                 _ARM_RSUBI
# define _AAS_RSUBI_UCF             _ARM_RSUBI_UCF
# define _AAS_RSUBI_IF              _ARM_RSUBI_IF
# define _AAS_RSUBI_UCF_IF          _ARM_RSUBI_UCF_IF

# define _AAS_RSUB                  _ARM_RSUB
# define _AAS_RSUB_UCF              _ARM_RSUB_UCF
# define _AAS_RSUB_IF               _ARM_RSUB_IF
# define _AAS_RSUB_UCF_IF           _ARM_RSUB_UCF_IF
	/* @} */

    /*
     * RSUBEX
	 * @{
     */
# define _AAS_RSUBEX                _ARM_RSUBEX
# define _AAS_RSUBEX_UCF            _ARM_RSUBEX_UCF
# define _AAS_RSUBEX_IF             _ARM_RSUBEX_IF
# define _AAS_RSUBEX_UCF_IF         _ARM_RSUBEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * MUL
	 * @{
     */
# define _AAS_MUL                   _ARM_MUL
# define _AAS_MUL_UCF               _ARM_MUL_UCF
# define _AAS_MUL_IF                _ARM_MUL_IF
# define _AAS_MUL_UCF_IF            _ARM_MUL_UCF_IF
	/* @} */

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

# define _AAS_CMPI                  _ARM_CMPI
# define _AAS_CMPI_IF               _ARM_CMPI_IF

# define _AAS_CMP                   _ARM_CMP
# define _AAS_CMP_IF                _ARM_CMP_IF

# define _AAS_CMPNEGI               _ARM_CMPNEGI
# define _AAS_CMPNEGI_IF            _ARM_CMPNEGI_IF

# define _AAS_CMPNEG                _ARM_CMPNEG
# define _AAS_CMPNEG_IF             _ARM_CMPNEG_IF

# define _AAS_UCF                   _ARM_UCF

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * not                                                 : NOT[I]     : DRR = ~SRR
 * not updating the condition flags                    : NOT[I]_UCF : DRR = ~SRR
 * and                                                 : AND[I]     : DRR = SRR & N
 * and updating the condition flags                    : AND[I]_UCF : DRR = SRR & N
 * and complement (and * not)                          : ANT[I]     : DRR = SRR & ~N
 * and complement (and * not) updating the condition flags : ANT[I]_UCF : DRR = SRR & ~N
 * or                                                  : OR[I]      : DRR = SRR | N
 * or updating the condition flags                     : OR[I]_UCF  : DRR = SRR | N
 * xor                                                 : XOR[I]     : DRR = SRR ^ N
 * xor updating the condition flags                    : XOR[I]_UCF : DRR = SRR ^ N
 * set all bits                                        : BITSET     :
 * clear all bits                                      : BITCLR     :
 */

/* ------------------------------------------------------------------------- */

    /*
     * NOT
     */

# define _AAS_NOTI                  _ARM_NOTI
# define _AAS_NOTI_UCF              _ARM_NOTI_UCF
# define _AAS_NOTI_IF               _ARM_NOTI_IF
# define _AAS_NOTI_UCF_IF           _ARM_NOTI_UCF_IF

# define _AAS_NOT                   _ARM_NOT
# define _AAS_NOT_UCF               _ARM_NOT_UCF
# define _AAS_NOT_IF                _ARM_NOT_IF
# define _AAS_NOT_UCF_IF            _ARM_NOT_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * AND
     */

# define _AAS_ANDI                  _ARM_ANDI
# define _AAS_ANDI_UCF              _ARM_ANDI_UCF
# define _AAS_ANDI_IF               _ARM_ANDI_IF
# define _AAS_ANDI_UCF_IF           _ARM_ANDI_UCF_IF

# define _AAS_AND                   _ARM_AND
# define _AAS_AND_UCF               _ARM_AND_UCF
# define _AAS_AND_IF                _ARM_AND_IF
# define _AAS_AND_UCF_IF            _ARM_AND_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * BIC: AND * NOT
     */

# define _AAS_BICI                  _ARM_BICI
# define _AAS_BICI_UCF              _ARM_BICI_UCF
# define _AAS_BICI_IF               _ARM_BICI_IF
# define _AAS_BICI_UCF_IF           _ARM_BICI_UCF_IF

# define _AAS_BIC                   _ARM_BIC
# define _AAS_BIC_UCF               _ARM_BIC_UCF
# define _AAS_BIC_IF                _ARM_BIC_IF
# define _AAS_BIC_UCF_IF            _ARM_BIC_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * OR
     */

# define _AAS_ORRI                  _ARM_ORRI
# define _AAS_ORRI_UCF              _ARM_ORRI_UCF
# define _AAS_ORRI_IF               _ARM_ORRI_IF
# define _AAS_ORRI_UCF_IF           _ARM_ORRI_UCF_IF

# define _AAS_ORR                   _ARM_ORR
# define _AAS_ORR_UCF               _ARM_ORR_UCF
# define _AAS_ORR_IF                _ARM_ORR_IF
# define _AAS_ORR_UCF_IF            _ARM_ORR_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * XOR
     */

# define _AAS_XORI                  _ARM_XORI
# define _AAS_XORI_UCF              _ARM_XORI_UCF
# define _AAS_XORI_IF               _ARM_XORI_IF
# define _AAS_XORI_UCF_IF           _ARM_XORI_UCF_IF

# define _AAS_XOR                   _ARM_XOR
# define _AAS_XOR_UCF               _ARM_XOR_UCF
# define _AAS_XOR_IF                _ARM_XOR_IF
# define _AAS_XOR_UCF_IF            _ARM_XOR_UCF_IF

/* ------------------------------------------------------------------------- */

# define _AAS_BITSET                _ARM_BITSET
# define _AAS_BITSET_IF             _ARM_BITSET_IF

# define _AAS_BITCLR                _ARM_BITCLR
# define _AAS_BITCLR_IF             _ARM_BITCLR_IF

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * rshift: DRR = SRR >> N
 * lshift: DRR = SRR << N
 */

# define _AAS_RSHIFTI               _ARM_RSHIFTI
# define _AAS_RSHIFTI_UCF           _ARM_RSHIFTI_UCF
# define _AAS_RSHIFT                _ARM_RSHIFT
# define _AAS_RSHIFT_UCF            _ARM_RSHIFT_UCF

# define _AAS_RSHIFTI_IF            _ARM_RSHIFTI_IF
# define _AAS_RSHIFTI_UCF_IF        _ARM_RSHIFTI_UCF_IF
# define _AAS_RSHIFT_IF             _ARM_RSHIFT_IF
# define _AAS_RSHIFT_UCF_IF         _ARM_RSHIFT_UCF_IF


# define _AAS_LSHIFTI               _ARM_LSHIFTI
# define _AAS_LSHIFTI_UCF           _ARM_LSHIFTI_UCF
# define _AAS_LSHIFT                _ARM_LSHIFT
# define _AAS_LSHIFT_UCF            _ARM_LSHIFT_UCF

# define _AAS_LSHIFTI_IF            _ARM_LSHIFTI_IF
# define _AAS_LSHIFTI_UCF_IF        _ARM_LSHIFTI_UCF_IF
# define _AAS_LSHIFT_IF             _ARM_LSHIFT_IF
# define _AAS_LSHIFT_UCF_IF         _ARM_LSHIFT_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 */

    /*
     * PUSH/POP
     */
# ifdef _ARM_PUSH
#  define _AAS_PUSH                 _ARM_PUSH
# endif
# define _AAS_PUSH1                 _ARM_PUSH1
# define _AAS_PUSH2                 _ARM_PUSH2
# define _AAS_PUSH_ARGRR            _ARM_PUSH_ARGRR
# define _AAS_PUSH_ALLRR            _ARM_PUSH_ALLRR
# define _AAS_PUSH_ALLRR0           _ARM_PUSH_ALLRR0
# define _AAS_PUSH_AVARR            _ARM_PUSH_AVARR
# define _AAS_PUSH_AVARR0           _ARM_PUSH_AVARR0

# ifdef _ARM_POP
#  define _AAS_POP                  _ARM_POP
# endif
# define _AAS_POP1                  _ARM_POP1
# define _AAS_POP2                  _ARM_POP2
# define _AAS_POP_ARGRR             _ARM_POP_ARGRR
# define _AAS_POP_ALLRR             _ARM_POP_ALLRR
# define _AAS_POP_ALLRR0            _ARM_POP_ALLRR0
# define _AAS_POP_AVARR             _ARM_POP_AVARR
# define _AAS_POP_AVARR0            _ARM_POP_AVARR0

/* ------------------------------------------------------------------------- */

    /*
     * Load
     */

# define _AAS_LOAD                  _ARM_LOAD
# define _AAS_LOADO                 _ARM_LOADO
# define _AAS_LOADOI                _ARM_LOADOI

# define _AAS_LOAD_IF               _ARM_LOAD_IF
# define _AAS_LOADO_IF              _ARM_LOADO_IF
# define _AAS_LOADOI_IF             _ARM_LOADOI_IF

# define _AAS_LOAD_PTR              _ARM_LOAD_PTR
# define _AAS_LOADO_PTR             _ARM_LOADO_PTR
# define _AAS_LOADOI_PTR            _ARM_LOADOI_PTR

# define _AAS_LOAD_PTR_IF           _ARM_LOAD_PTR_IF
# define _AAS_LOADO_PTR_IF          _ARM_LOADO_PTR_IF
# define _AAS_LOADOI_PTR_IF         _ARM_LOADOI_PTR_IF


/* slow operation (?) */
# define _AAS_LOAD_IB               _ARM_LOAD_IB
# define _AAS_LOAD_DB               _ARM_LOAD_DB
# define _AAS_LOAD_IA               _ARM_LOAD_IA
# define _AAS_LOAD_DA               _ARM_LOAD_DA
# define _AAS_LOADI_IB              _ARM_LOADI_IB
# define _AAS_LOADI_DB              _ARM_LOADI_DB
# define _AAS_LOADI_IA              _ARM_LOADI_IA
# define _AAS_LOADI_DA              _ARM_LOADI_DA


/* slow operation (?) */
# define _AAS_LOAD_AIA              _ARM_LOAD_AIA
# define _AAS_LOAD_AIB              _ARM_LOAD_AIB
# define _AAS_LOAD_ADA              _ARM_LOAD_ADA
# define _AAS_LOAD_ADB              _ARM_LOAD_ADB

/* ------------------------------------------------------------------------- */

    /*
     * Store
     */

# define _AAS_STORE                 _ARM_STORE
# define _AAS_STOREO                _ARM_STOREO
# define _AAS_STOREOI               _ARM_STOREOI

# define _AAS_STORE_IF              _ARM_STORE_IF
# define _AAS_STOREO_IF             _ARM_STOREO_IF
# define _AAS_STOREOI_IF            _ARM_STOREOI_IF

# define _AAS_STORE_PTR             _ARM_STORE_PTR
# define _AAS_STOREO_PTR            _ARM_STOREO_PTR
# define _AAS_STOREOI_PTR           _ARM_STOREOI_PTR

# define _AAS_STORE_PTR_IF          _ARM_STORE_PTR_IF
# define _AAS_STOREO_PTR_IF         _ARM_STOREO_PTR_IF
# define _AAS_STOREOI_PTR_IF        _ARM_STOREOI_PTR_IF


/* slow operation (?) */
# define _AAS_STORE_IB              _ARM_STORE_IB
# define _AAS_STORE_DB              _ARM_STORE_DB
# define _AAS_STORE_IA              _ARM_STORE_IA
# define _AAS_STORE_DA              _ARM_STORE_DA
# define _AAS_STOREI_IB             _ARM_STOREI_IB
# define _AAS_STOREI_DB             _ARM_STOREI_DB
# define _AAS_STOREI_IA             _ARM_STOREI_IA
# define _AAS_STOREI_DA             _ARM_STOREI_DA


/* slow operation (?) */
# define _AAS_STORE_AIA             _ARM_STORE_AIA
# define _AAS_STORE_AIB             _ARM_STORE_AIB
# define _AAS_STORE_ADA             _ARM_STORE_ADA
# define _AAS_STORE_ADB             _ARM_STORE_ADB

/* ------------------------------------------------------------------------- */

    /*
     * Prefetch Memory
     */

# define _AAS_PRELOADI_READ             _ARM_PRELOADI_READ
# define _AAS_PRELOADI_WRITE            _ARM_PRELOADI_WRITE
# define _AAS_APRELOADI_READ            _ARM_APRELOADI_READ
# define _AAS_APRELOADI_WRITE           _ARM_APRELOADI_WRITE
# define _AAS_PRELOAD_READ              _ARM_PRELOAD_READ
# define _AAS_PRELOAD_WRITE             _ARM_PRELOAD_WRITE
# define _AAS_PRELOADEX_READ            _ARM_PRELOADEX_READ
# define _AAS_PRELOADEX_WRITE           _ARM_PRELOADEX_WRITE

# define _AAS_PRELOADI_READ_CACHELINE   _ARM_PRELOADI_READ_CACHELINE
# define _AAS_PRELOADI_WRITE_CACHELINE  _ARM_PRELOADI_WRITE_CACHELINE
# define _AAS_APRELOADI_READ_CACHELINE  _ARM_APRELOADI_READ_CACHELINE
# define _AAS_APRELOADI_WRITE_CACHELINE _ARM_APRELOADI_WRITE_CACHELINE
# define _AAS_PRELOAD_READ_CACHELINE    _ARM_PRELOAD_READ_CACHELINE
# define _AAS_PRELOAD_WRITE_CACHELINE   _ARM_PRELOAD_WRITE_CACHELINE
# define _AAS_PRELOADEX_READ_CACHELINE  _ARM_PRELOADEX_READ_CACHELINE
# define _AAS_PRELOADEX_WRITE_CACHELINE _ARM_PRELOADEX_WRITE_CACHELINE

# define _AAS_PREFETCHI             _ARM_PREFETCHI
# define _AAS_APREFETCHI            _ARM_APREFETCHI
# define _AAS_PREFETCH              _ARM_PREFETCH
# define _AAS_PREFETCHEX            _ARM_PREFETCHEX

# define _AAS_PREFETCHI_CACHELINE   _ARM_PREFETCHI_CACHELINE
# define _AAS_APREFETCHI_CACHELINE  _ARM_APREFETCHI_CACHELINE
# define _AAS_PREFETCH_CACHELINE    _ARM_PREFETCH_CACHELINE
# define _AAS_PREFETCHEX_CACHELINE  _ARM_PREFETCHEX_CACHELINE

# define _AAS_PLD                   _ARM_APRELOADI_READ_CACHELINE
# define _AAS_PLDW                  _ARM_APRELOADI_WRITE_CACHELINE

# define _SMA_PRELOAD_READ(PRR,Offset)  _ARM_APRELOADI_READ(PRR,Offset,64)
# define _SMA_PRELOAD_WRITE(PRR,Offset) _ARM_APRELOADI_WRITE(PRR,Offset,64)

/* @} */


/* ========================================================================= */

/*
 * Flow control
 * @{
 */

# define _AAS_GOTO_IMM              _ARM_GOTO_IMM
# define _AAS_GOTO_IF               _ARM_GOTO_IF
# define _AAS_XGOTO_IF              _ARM_XGOTO_IF

    /*
	 *  {
     *  } {* block-name: Label *}
     */
# define _AAS_ENDIF                 _ARM_ENDIF

/* ------------------------------------------------------------------------- */

    /*
     * do { ... } while (CRR -= Imm)
     * do { ... } while (CRR -= RR)
     * do { ... } while (0 <= (CRR -= Imm))
     * do { ... } while (0 <= (CRR -= RR))
     * do { ... } while ((CRR ?? Comparand) ? (CRR -= Value, 1) : 0)
     */

# define _AAS_DO                    _ARM_DO

# define _AAS_WHILE                 _ARM_WHILE
# define _AAS_WHILE_TRUE            _ARM_WHILE_TRUE

# define _AAS_WHILEI_GT0_DB         _ARM_WHILEI_GT0_DB
# define _AAS_WHILEI_GE0_DB         _ARM_WHILEI_GE0_DB
# define _AAS_WHILEI_NE0_DB         _ARM_WHILEI_NE0_DB
# define _AAS_WHILE_GT0_DB          _ARM_WHILE_GT0_DB
# define _AAS_WHILE_GE0_DB          _ARM_WHILE_GE0_DB
# define _AAS_WHILE_NE0_DB          _ARM_WHILE_NE0_DB

# define _AAS_WHILEI_DB             _ARM_WHILEI_DB
# define _AAS_WHILE_DB              _ARM_WHILE_DB

# define _AAS_WHILEI_DA             _ARM_WHILEI_DA
# define _AAS_WHILE_DA              _ARM_WHILE_DA

# define _AAS_XWHILEI12_DA          _ARM_XWHILEI12_DA
# define _AAS_XWHILEI10_DA          _ARM_XWHILEI10_DA
# define _AAS_XWHILEI02_DA          _ARM_XWHILEI02_DA
# define _AAS_XWHILE_DA             _ARM_XWHILE_DA

/* ------------------------------------------------------------------------- */

    /*
     * ADD
     */

# define _AAS_IF_ADDI               _ARM_IF_ADDI
# define _AAS_IF_ADD                _ARM_IF_ADD

# define _AAS_IF_ADDI_NE0           _ARM_IF_ADDI_NE0
# define _AAS_IF_ADDI_GE0           _ARM_IF_ADDI_GE0
# define _AAS_IF_ADDI_GT0           _ARM_IF_ADDI_GT0

# define _AAS_IF_ADD_NE0            _ARM_IF_ADD_NE0
# define _AAS_IF_ADD_GE0            _ARM_IF_ADD_GE0
# define _AAS_IF_ADD_GT0            _ARM_IF_ADD_GT0

/* ------------------------------------------------------------------------- */

    /*
     * SUB
     */

# define _AAS_IF_SUBI               _ARM_IF_SUBI
# define _AAS_IF_SUB                _ARM_IF_SUB

# define _AAS_IF_SUBI_NE0           _ARM_IF_SUBI_NE0
# define _AAS_IF_SUBI_GE0           _ARM_IF_SUBI_GE0
# define _AAS_IF_SUBI_GT0           _ARM_IF_SUBI_GT0
# define _AAS_IF_SUB_NE0            _ARM_IF_SUB_NE0
# define _AAS_IF_SUB_GE0            _ARM_IF_SUB_GE0
# define _AAS_IF_SUB_GT0            _ARM_IF_SUB_GT0

/* ------------------------------------------------------------------------- */

    /*
     * RSUB
     */

# define _AAS_IF_RSUBI              _ARM_IF_RSUBI
# define _AAS_IF_RSUB               _ARM_IF_RSUB

# define _AAS_IF_RSUBI_NE0          _ARM_IF_RSUBI_NE0
# define _AAS_IF_RSUBI_GE0          _ARM_IF_RSUBI_GE0
# define _AAS_IF_RSUBI_GT0          _ARM_IF_RSUBI_GT0
# define _AAS_IF_RSUB_NE0           _ARM_IF_RSUB_NE0
# define _AAS_IF_RSUB_GE0           _ARM_IF_RSUB_GE0
# define _AAS_IF_RSUB_GT0           _ARM_IF_RSUB_GT0

/* ------------------------------------------------------------------------- */

    /*
     *  if {* block-name: Label *} ((DRR = (SRR & Imm)) != 0) { ...
	 *  }
     */

# define _AAS_IF_ANDI               _ARM_IF_ANDI
# define _AAS_IF_AND                _ARM_IF_AND

# define _AAS_IF_ANDI_NE0           _ARM_IF_ANDI_NE0
# define _AAS_IF_ANDI_GE0           _ARM_IF_ANDI_GE0
# define _AAS_IF_ANDI_GT0           _ARM_IF_ANDI_GT0

# define _AAS_IF_AND_NE0            _ARM_IF_AND_NE0
# define _AAS_IF_AND_GE0            _ARM_IF_AND_GE0
# define _AAS_IF_AND_GT0            _ARM_IF_AND_GT0

/* ------------------------------------------------------------------------- */

    /*
     *  if {* block-name: Label *} ((DRR = (SRR & Imm)) != 0) { ...
	 *  }
     */

# define _AAS_IF_XORI               _ARM_IF_XORI
# define _AAS_IF_XOR                _ARM_IF_XOR

# define _AAS_IF_XORI_NE0           _ARM_IF_XORI_NE0
# define _AAS_IF_XORI_GE0           _ARM_IF_XORI_GE0
# define _AAS_IF_XORI_GT0           _ARM_IF_XORI_GT0

# define _AAS_IF_XOR_NE0            _ARM_IF_XOR_NE0
# define _AAS_IF_XOR_GE0            _ARM_IF_XOR_GE0
# define _AAS_IF_XOR_GT0            _ARM_IF_XOR_GT0

/* ------------------------------------------------------------------------- */

    /*
     *  if {* block-name: Label *} ((DRR = (SRR >> Imm)) != 0) { ...
	 *  }
     */

# define _AAS_IF_RSHIFTI            _ARM_IF_RSHIFTI
# define _AAS_IF_RSHIFT             _ARM_IF_RSHIFT

# define _AAS_IF_RSHIFTI_NE0        _ARM_IF_RSHIFTI_NE0
# define _AAS_IF_RSHIFTI_GE0        _ARM_IF_RSHIFTI_GE0
# define _AAS_IF_RSHIFTI_GT0        _ARM_IF_RSHIFTI_GT0

# define _AAS_IF_RSHIFT_NE0         _ARM_IF_RSHIFT_NE0
# define _AAS_IF_RSHIFT_GE0         _ARM_IF_RSHIFT_GE0
# define _AAS_IF_RSHIFT_GT0         _ARM_IF_RSHIFT_GT0


# define _AAS_IF_LSHIFTI            _ARM_IF_LSHIFTI
# define _AAS_IF_LSHIFT             _ARM_IF_LSHIFT

/* ------------------------------------------------------------------------- */

    /*
     * Compare
     */

# define _AAS_IF_CMPI               _ARM_IF_CMPI
# define _AAS_IF_CMP                _ARM_IF_CMP
# define _AAS_IF_CMPNEGI            _ARM_IF_CMPNEGI
# define _AAS_IF_CMPNEG             _ARM_IF_CMPNEG

/* Usage sample:

AAS:
	_AAS_IF_EQ0([tmp], L1)
		stmt1
		_AAS_GOTO_IMM(L2)
	_AAS_ENDIF(L1)
		stmt2
	L2:
	stmt3

C:
	if (0 == tmp)
		stmt1;
	else
		stmt2;
	stmt3;
 */
# define _AAS_IF_EQ0                _ARM_IF_EQ0
# define _AAS_IF_NE0                _ARM_IF_NE0

# define _AAS_IF_EQI                _ARM_IF_EQI
# define _AAS_IF_NEI                _ARM_IF_NEI
# define _AAS_IF_LTI                _ARM_IF_LTI
# define _AAS_IF_LEI                _ARM_IF_LEI
# define _AAS_IF_GEI                _ARM_IF_GEI
# define _AAS_IF_GTI                _ARM_IF_GTI

# define _AAS_IF_EQ                 _ARM_IF_EQ
# define _AAS_IF_NE                 _ARM_IF_NE
# define _AAS_IF_LT                 _ARM_IF_LT
# define _AAS_IF_LE                 _ARM_IF_LE
# define _AAS_IF_GE                 _ARM_IF_GE
# define _AAS_IF_GT                 _ARM_IF_GT

# define _AAS_IF_EQNEG0             _ARM_IF_EQNEG0
# define _AAS_IF_NENEG0             _ARM_IF_NENEG0

# define _AAS_IF_EQNEGI             _ARM_IF_EQNEGI
# define _AAS_IF_NENEGI             _ARM_IF_NENEGI
# define _AAS_IF_LTNEGI             _ARM_IF_LTNEGI
# define _AAS_IF_LENEGI             _ARM_IF_LENEGI
# define _AAS_IF_GENEGI             _ARM_IF_GENEGI
# define _AAS_IF_GTNEGI             _ARM_IF_GTNEGI

# define _AAS_IF_EQNEG              _ARM_IF_EQNEG
# define _AAS_IF_NENEG              _ARM_IF_NENEG
# define _AAS_IF_LTNEG              _ARM_IF_LTNEG
# define _AAS_IF_LENEG              _ARM_IF_LENEG
# define _AAS_IF_GENEG              _ARM_IF_GENEG
# define _AAS_IF_GTNEG              _ARM_IF_GTNEG

/* ------------------------------------------------------------------------- */

    /*
     * Move
     */

# define _AAS_IF_MOVI               _ARM_IF_MOVI
# define _AAS_IF_MOV                _ARM_IF_MOV

# define _AAS_IF_MOVI_NE0           _ARM_IF_MOVI_NE0
# define _AAS_IF_MOVI_GE0           _ARM_IF_MOVI_GE0
# define _AAS_IF_MOVI_GT0           _ARM_IF_MOVI_GT0

# define _AAS_IF_MOV_NE0            _ARM_IF_MOV_NE0
# define _AAS_IF_MOV_GE0            _ARM_IF_MOV_GE0
# define _AAS_IF_MOV_GT0            _ARM_IF_MOV_GT0

/* ------------------------------------------------------------------------- */

    /*
	 * @note: Imm = N, case <- {[0, N], default}
     */

# define _AAS_SWITCH                _ARM_SWITCH

/* Usage sample:

AAS:
	_AAS_SWITCH([x], N, Ldefault)

	_AAS_GOTO_IMM(L0)
	_AAS_GOTO_IMM(L1)
	_AAS_GOTO_IMM(L2)
	_AAS_GOTO_IMM(L3)
	...
	_AAS_GOTO_IMM(Ln)

	L0:       stmt0 _AAS_GOTO_IMM(Lend)
	L1:       stmt1 _AAS_GOTO_IMM(Lend)
	L2:       stmt2 _AAS_GOTO_IMM(Lend)
	L3:       stmt3 _AAS_GOTO_IMM(Lend)
	...
	Ln:       stmtN _AAS_GOTO_IMM(Lend)
	Ldefault: stmtD _AAS_GOTO_IMM(Lend)
	Lend:

C:
	switch (x) {
	case 0:  stmt0 break;
	case 1:  stmt1 break;
	case 2:  stmt2 break;
	case 3:  stmt3 break;
	...
	case N:  stmtN break;
	default: stmtD break;
	}
 */

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define _AAS_DPRINTFI              _ARM_DPRINTFI

# define _AAS_PRINTRR1              _ARM_PRINTRR1
# define _AAS_PRINTRR2              _ARM_PRINTRR2
# define _AAS_PRINTRR3              _ARM_PRINTRR3

# define __AAS_PRINTRR1             __ARM_PRINTRR1
# define __AAS_PRINTRR2             __ARM_PRINTRR2
# define __AAS_PRINTRR3             __ARM_PRINTRR3

/* @} */
/* @} */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*
 * NEON
 * @{
 */

# define _AAS_VENDX_D               _ARM_VENDX_D
# define _AAS_VENDX_Q               _ARM_VENDX_Q

# define _AAS_VESHRNUM_D_MAP        _ARM_VESHRNUM_D_MAP
# define _AAS_VESHRNUM_Q_MAP        _ARM_VESHRNUM_Q_MAP

# define _AAS_QDMAP0                _ARM_QDMAP0
# define _AAS_QDMAP1                _ARM_QDMAP1

# ifdef _ARM_DSMAP0
#  define _AAS_DSMAP0               _ARM_DSMAP0
#  define _AAS_DSMAP1               _ARM_DSMAP1
# endif


/* ************************************************************************* */

/*
 * Move operations
 * @{
 */

    /*
     * Duplicate/Set immediate value into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = Imm
     */
# define _AAS_VDUPI_Q               _ARM_VDUPI_Q
# define _AAS_VDUPI_D               _ARM_VDUPI_D
# define _AAS_VDUPFI_Q              _ARM_VDUPFI_Q
# define _AAS_VDUPFI_D              _ARM_VDUPFI_D
# define _AAS_VDUPFI_S              _ARM_VDUPFI_S
# ifdef _ARM_VDUPIEX_Q
#  define _AAS_VDUPIEX_Q            _ARM_VDUPIEX_Q
#  define _AAS_VDUPIEX_D            _ARM_VDUPIEX_D
# endif

# define _AAS_VSETIn_Q              _ARM_VSETIn_Q
# define _AAS_VSETIn_D              _ARM_VSETIn_D
# define _AAS_VSETFIn_Q             _ARM_VSETFIn_Q
# define _AAS_VSETFIn_D             _ARM_VSETFIn_D
# define _AAS_VSETFIn_S             _ARM_VSETFIn_S
# define _AAS_VSETFI_S              _ARM_VSETFI_S
# ifdef _ARM_VSETInEX_Q
#  define _AAS_VSETInEX_Q           _ARM_VSETInEX_Q
#  define _AAS_VSETInEX_D           _ARM_VSETInEX_D
# endif

# ifdef _ARM_VMOV2I_Q
#  define _AAS_VMOV2I_Q             _ARM_VMOV2I_Q
#  define _AAS_VMOV2I_D             _ARM_VMOV2I_D
# endif


    /*
     * Set zero value into one element of the destination vector register
	 * dM[k] = 0, k <- {0,N}
	 * sM = 0
	 */
# ifdef _ARM_VZERO1_Q
#  define _AAS_VZERO1_Q             _ARM_VZERO1_Q
#  define _AAS_VZERO1_D             _ARM_VZERO1_D
#  define _AAS_VZERO1_S             _ARM_VZERO1_S
# endif

/* ------------------------------------------------------------------------- */

    /*
     * Duplicate/Set [regular register] scalar into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = $RR
     */
# define _AAS_VDUPR_Q               _ARM_VDUPR_Q
# define _AAS_VDUPR_D               _ARM_VDUPR_D

# define _AAS_VSETn_Q               _ARM_VSETn_Q
# define _AAS_VSETn_D               _ARM_VSETn_D


    /*
     * Duplicate/Set [vector] scalar into every element of the destination vector register
	 * (dM[0],..., dM[N]) = dK[i], i <- {0,N}
     */
# define _AAS_VDUP_Q                _ARM_VDUP_Q
# define _AAS_VDUP_D                _ARM_VDUP_D


    /*
     * Set [regular register] scalar into one element of the destination vector register
	 * dM[k] = $RR, k <- {0,N}
	 * sM = $RR
	 */
# define _AAS_VSET1_Q               _ARM_VSET1_Q
# define _AAS_VSET1_D               _ARM_VSET1_D
# define _AAS_VSET1_S               _ARM_VSET1_S

/* ------------------------------------------------------------------------- */

    /*
     * Get [regular register] scalar from one element of the source vector register
	 *
	 * $RR = dM[i], i <- {0,N}
	 * $RR1 = dM[0], $RR2 = dM[1]
	 * $RR = sM
     */
# define _AAS_VGET1_D               _ARM_VGET1_D
# define _AAS_VGET1_D_IF            _ARM_VGET1_D_IF
# define _AAS_VGET1_S               _ARM_VGET1_S
# define _AAS_VGET1_S_IF            _ARM_VGET1_S_IF

# ifdef _ARM_VGET2_D
#  define _AAS_VGET2_D              _ARM_VGET2_D
# endif

/* ------------------------------------------------------------------------- */

    /*
     * Move between vector registers
	 */

/* Q --> Q */
/* D --> D */
/* S --> S */
# define _AAS_VMOV_Q                _ARM_VMOV_Q
# define _AAS_VMOV_D                _ARM_VMOV_D
# define _AAS_VMOV_S                _ARM_VMOV_S

# define _AAS_VMOV                  _ARM_VMOV

# define _AAS_VMVN_Q                _ARM_VMVN_Q
# define _AAS_VMVN_D                _ARM_VMVN_D

/* ------------------------------------------------------------------------- */

    /*
     * Move between ARM regular and SIMD and FP system registers
	 *
	 * VSR - vector system register: FPSCR, FPSID, FPEXC, ...
	 */

# ifdef _AAS_VFPSRTORR
#  define _AAS_VFPSRTORR            _ARM_VFPSRTORR
#  define _AAS_VFPGETCF             _ARM_VFPGETCF
#  define _AAS_VFPGETCF_IF          _ARM_VFPGETCF_IF
# endif

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 * @{
 */

/* w - source data width
 *
 * widen:   D    --> (Q)   ;; uint8 --> int16, int16 --> int32, int32 --> int64
 *          Q    --> (Q,Q)
 * widenf:  S    --> (D)   ;; float16 --> float32, float32 --> float64
 *          D    --> (Q)
 *          Q    --> (Q,Q)
 * narrow: (Q)   -->  D    ;; int16 --> uint8, int32 --> int16, int64 --> int32
 *         (Q,Q) -->  Q
 * narrowf:(D)   -->  S    ;; float32 --> float16, float64 --> float32
 *         (Q)   -->  D
 *         (Q,Q) -->  Q
 */

# define _AAS_VWIDEN_Q1D1           _ARM_VWIDEN_Q1D1
# define _AAS_VWIDEN_Q2Q1           _ARM_VWIDEN_Q2Q1
# define _AAS_VWIDENF_D1S1          _ARM_VWIDENF_D1S1
# define _AAS_VWIDENF_Q1D1          _ARM_VWIDENF_Q1D1
# define _AAS_VWIDENF_Q2Q1          _ARM_VWIDENF_Q2Q1

# define _AAS_VNARROW_D1Q1          _ARM_VNARROW_D1Q1
# define _AAS_VNARROW_Q1Q2          _ARM_VNARROW_Q1Q2
# define _AAS_VNARROWF_S1D1         _ARM_VNARROWF_S1D1
# define _AAS_VNARROWF_D1Q1         _ARM_VNARROWF_D1Q1
# define _AAS_VNARROWF_Q1Q2         _ARM_VNARROWF_Q1Q2

/* narrow: inplace variant; w - source data width; NOTE!!! result placed to SDVRI1, not SDVRI0 (?) */
# ifdef _ARM_VNARROW_Q2
#  define _AAS_VNARROW_D2           _ARM_VNARROW_D2
#  define _AAS_VNARROW_Q1           _ARM_VNARROW_Q1
#  define _AAS_VNARROW_Q2           _ARM_VNARROW_Q2
# endif

/* narrow with saturate: Q --> D */
# define _AAS_VSATURATE_UU_D1Q1     _ARM_VSATURATE_UU_D1Q1
# define _AAS_VSATURATE_SS_D1Q1     _ARM_VSATURATE_SS_D1Q1
# define _AAS_VSATURATE_SU_D1Q1     _ARM_VSATURATE_SU_D1Q1
/* narrow with saturate: (Q,Q) --> Q */
# define _AAS_VSATURATE_UU_Q1Q2     _ARM_VSATURATE_UU_Q1Q2
# define _AAS_VSATURATE_SS_Q1Q2     _ARM_VSATURATE_SS_Q1Q2
# define _AAS_VSATURATE_SU_Q1Q2     _ARM_VSATURATE_SU_Q1Q2


    /*
     * vector convert signed/unsigned integer to floating point
     */
# define _AAS_VSITOFEX_Q            _ARM_VSITOFEX_Q
# define _AAS_VSITOFEX_D            _ARM_VSITOFEX_D
# define _AAS_VSITOFEX_S            _ARM_VSITOFEX_S
# define _AAS_VUITOFEX_Q            _ARM_VUITOFEX_Q
# define _AAS_VUITOFEX_D            _ARM_VUITOFEX_D
# define _AAS_VUITOFEX_S            _ARM_VUITOFEX_S

    /*
     * vector convert signed/unsigned integer to float
     */
# define _AAS_VSITOF_Q              _ARM_VSITOF_Q
# define _AAS_VSITOF_D              _ARM_VSITOF_D
# define _AAS_VSITOF_S              _ARM_VSITOF_S
# define _AAS_VUITOF_Q              _ARM_VUITOF_Q
# define _AAS_VUITOF_D              _ARM_VUITOF_D
# define _AAS_VUITOF_S              _ARM_VUITOF_S

	/*
     * vector convert signed/unsigned long integer to double
	 */
# ifdef _ARM_VLITOF64_Q
#  define _AAS_VLITOF64_Q           _ARM_VLITOF64_Q
#  define _AAS_VLITOF64_D           _ARM_VLITOF64_D
#  define _AAS_VLITOF64_S           _ARM_VLITOF64_S
#  define _AAS_VLUTOF64_Q           _ARM_VLUTOF64_Q
#  define _AAS_VLUTOF64_D           _ARM_VLUTOF64_D
#  define _AAS_VLUTOF64_S           _ARM_VLUTOF64_S
# endif

	/*
     * vector convert signed/unsigned integer to double
	 */
/* S --> D
   D --> Q
   Q --> (Q,Q) */
# ifdef _ARM_VSITOF64_D1S1
#  define _AAS_VSITOF64_D1S1        _ARM_VSITOF64_D1S1
# endif
# define _AAS_VSITOF64_Q1D1         _ARM_VSITOF64_Q1D1
# define _AAS_VSITOF64_Q2Q1         _ARM_VSITOF64_Q2Q1
# ifdef _ARM_VUITOF64_D1S1
#  define _AAS_VUITOF64_D1S1        _ARM_VUITOF64_D1S1
# endif
# define _AAS_VUITOF64_Q1D1         _ARM_VUITOF64_Q1D1
# define _AAS_VUITOF64_Q2Q1         _ARM_VUITOF64_Q2Q1

    /*
     * vector convert floating point to signed/unsigned integer with roundmode
     */
# define _AAS_RND_VFTOSIEX_S        _ARM_RND_VFTOSIEX_S
# define _AAS_RND_VFTOSIEX_D        _ARM_RND_VFTOSIEX_D
# define _AAS_RND_VFTOSIEX_Q        _ARM_RND_VFTOSIEX_Q
# define _AAS_RND_VFTOUIEX_S        _ARM_RND_VFTOUIEX_S
# define _AAS_RND_VFTOUIEX_D        _ARM_RND_VFTOUIEX_D
# define _AAS_RND_VFTOUIEX_Q        _ARM_RND_VFTOUIEX_Q

    /*
     * vector convert double to signed/unsigned integer with roundmode
     */
/* D --> S
   Q --> D
   (Q,Q) --> Q */
# ifdef _ARM_RND_VF64TOSIEX_S1D1
#  define _AAS_RND_VF64TOSIEX_S1D1  _ARM_RND_VF64TOSIEX_S1D1
# endif
# define _AAS_RND_VF64TOSIEX_D1Q1   _ARM_RND_VF64TOSIEX_D1Q1
# define _AAS_RND_VF64TOSIEX_Q1Q2   _ARM_RND_VF64TOSIEX_Q1Q2
# ifdef _ARM_RND_VF64TOUIEX_S1D1
#  define _AAS_RND_VF64TOUIEX_S1D1  _ARM_RND_VF64TOUIEX_S1D1
# endif
# define _AAS_RND_VF64TOUIEX_D1Q1   _ARM_RND_VF64TOUIEX_D1Q1
# define _AAS_RND_VF64TOUIEX_Q1Q2   _ARM_RND_VF64TOUIEX_Q1Q2

    /*
     * vector convert floating point to signed/unsigned [long] integer with roundmode
     */
# define _AAS_RND_VFTOSIEX          _ARM_RND_VFTOSIEX
# define _AAS_RND_VFTOUIEX          _ARM_RND_VFTOUIEX

    /*
     * vector convert truncate float to signed/unsigned integer
     */
# define _AAS_VFTOSI_S              _ARM_VFTOSI_S
# define _AAS_VFTOSI_D              _ARM_VFTOSI_D
# define _AAS_VFTOSI_Q              _ARM_VFTOSI_Q
# define _AAS_VFTOUI_S              _ARM_VFTOUI_S
# define _AAS_VFTOUI_D              _ARM_VFTOUI_D
# define _AAS_VFTOUI_Q              _ARM_VFTOUI_Q

    /*
     * vector convert truncate double to signed/unsigned long integer
     */
# ifdef _ARM_VF64TOLI_Q
#  define _AAS_VF64TOLI_S           _ARM_VF64TOLI_S
#  define _AAS_VF64TOLI_D           _ARM_VF64TOLI_D
#  define _AAS_VF64TOLI_Q           _ARM_VF64TOLI_Q
#  define _AAS_VF64TOLU_S           _ARM_VF64TOLU_S
#  define _AAS_VF64TOLU_D           _ARM_VF64TOLU_D
#  define _AAS_VF64TOLU_Q           _ARM_VF64TOLU_Q
# endif

    /*
     * vector convert truncate double to signed/unsigned integer
     */
/* D --> S
   Q --> D
   (Q,Q) --> Q */
# ifdef _ARM_VF64TOSI_S1D1
#  define _AAS_VF64TOSI_S1D1        _ARM_VF64TOSI_S1D1
# endif
# define _AAS_VF64TOSI_D1Q1         _ARM_VF64TOSI_D1Q1
# define _AAS_VF64TOSI_Q1Q2         _ARM_VF64TOSI_Q1Q2
# ifdef _ARM_VF64TOUI_S1D1
#  define _AAS_VF64TOUI_S1D1        _ARM_VF64TOUI_S1D1
# endif
# define _AAS_VF64TOUI_D1Q1         _ARM_VF64TOUI_D1Q1
# define _AAS_VF64TOUI_Q1Q2         _ARM_VF64TOUI_Q1Q2

    /*
     * vector convert truncate float to signed/unsigned [long] integer
     */
# define _AAS_VFTOSI32              _ARM_VFTOSI32
# define _AAS_VFTOUI32              _ARM_VFTOUI32
# define _AAS_VFTOSI64              _ARM_VFTOSI64
# define _AAS_VFTOUI64              _ARM_VFTOUI64
# define _AAS_VFTOSI                _ARM_VFTOSI
# define _AAS_VFTOUI                _ARM_VFTOUI
# define _AAS_VFTOLI                _ARM_VFTOLI
# define _AAS_VFTOLU                _ARM_VFTOLU

    /*
     * vector convert truncate double to signed/unsigned [long] integer
     */
# define _AAS_VF64TOSI32            _ARM_VF64TOSI32
# define _AAS_VF64TOUI32            _ARM_VF64TOUI32
# define _AAS_VF64TOSI64            _ARM_VF64TOSI64
# define _AAS_VF64TOUI64            _ARM_VF64TOUI64
# define _AAS_VF64TOSI              _ARM_VF64TOSI
# define _AAS_VF64TOUI              _ARM_VF64TOUI
# define _AAS_VF64TOLI              _ARM_VF64TOLI
# define _AAS_VF64TOLU              _ARM_VF64TOLU

    /*
     * vector convert round float to signed/unsigned integer
     */
# define _AAS_RND_VFTOSI_S          _ARM_RND_VFTOSI_S
# ifdef _ARM_RND_VFTOSI_Q
#  define _AAS_RND_VFTOSI_D         _ARM_RND_VFTOSI_D
#  define _AAS_RND_VFTOSI_Q         _ARM_RND_VFTOSI_Q
# endif
# define _AAS_RND_VFTOUI_S          _ARM_RND_VFTOUI_S
# ifdef _ARM_RND_VFTOUI_Q
#  define _AAS_RND_VFTOUI_D         _ARM_RND_VFTOUI_D
#  define _AAS_RND_VFTOUI_Q         _ARM_RND_VFTOUI_Q
# endif

    /*
     * vector rounding convert double to signed/unsigned long integer
     */
# ifdef _ARM_RND_VF64TOLI_Q
#  define _AAS_RND_VF64TOLI_S       _ARM_RND_VF64TOLI_S
#  define _AAS_RND_VF64TOLI_D       _ARM_RND_VF64TOLI_D
#  define _AAS_RND_VF64TOLI_Q       _ARM_RND_VF64TOLI_Q
#  define _AAS_RND_VF64TOLU_S       _ARM_RND_VF64TOLU_S
#  define _AAS_RND_VF64TOLU_D       _ARM_RND_VF64TOLU_D
#  define _AAS_RND_VF64TOLU_Q       _ARM_RND_VF64TOLU_Q
# endif

    /*
     * vector convert round double to signed/unsigned integer
     */
/* D --> S
   Q --> D
   (Q,Q) --> Q */
# ifdef _ARM_RND_VF64TOSI_S1D1
#  define _AAS_RND_VF64TOSI_S1D1    _ARM_RND_VF64TOSI_S1D1
# endif
# define _AAS_RND_VF64TOSI_D1Q1     _ARM_RND_VF64TOSI_D1Q1
# define _AAS_RND_VF64TOSI_Q1Q2     _ARM_RND_VF64TOSI_Q1Q2
# ifdef _ARM_RND_VF64TOUI_S1D1
#  define _AAS_RND_VF64TOUI_S1D1    _ARM_RND_VF64TOUI_S1D1
# endif
# define _AAS_RND_VF64TOUI_D1Q1     _ARM_RND_VF64TOUI_D1Q1
# define _AAS_RND_VF64TOUI_Q1Q2     _ARM_RND_VF64TOUI_Q1Q2

    /*
     * vector convert round float to signed/unsigned [long] integer
     */
# define _AAS_RND_VFTOSI32          _ARM_RND_VFTOSI32
# define _AAS_RND_VFTOUI32          _ARM_RND_VFTOUI32
# define _AAS_RND_VFTOSI64          _ARM_RND_VFTOSI64
# define _AAS_RND_VFTOUI64          _ARM_RND_VFTOUI64
# define _AAS_RND_VFTOSI            _ARM_RND_VFTOSI
# define _AAS_RND_VFTOUI            _ARM_RND_VFTOUI
# define _AAS_RND_VFTOLI            _ARM_RND_VFTOLI
# define _AAS_RND_VFTOLU            _ARM_RND_VFTOLU

    /*
     * vector convert round double to signed/unsigned [long] integer
     */
# define _AAS_RND_VF64TOSI32        _ARM_RND_VF64TOSI32
# define _AAS_RND_VF64TOUI32        _ARM_RND_VF64TOUI32
# define _AAS_RND_VF64TOSI64        _ARM_RND_VF64TOSI64
# define _AAS_RND_VF64TOUI64        _ARM_RND_VF64TOUI64
# define _AAS_RND_VF64TOSI          _ARM_RND_VF64TOSI
# define _AAS_RND_VF64TOUI          _ARM_RND_VF64TOUI
# define _AAS_RND_VF64TOLI          _ARM_RND_VF64TOLI
# define _AAS_RND_VF64TOLU          _ARM_RND_VF64TOLU

	/*
     * vector convert float to double
	 */
/* S --> D
   D --> Q
   Q --> (Q,Q) */
# define _AAS_VF32TOF64_D1S1        _ARM_VF32TOF64_D1S1
# define _AAS_VF32TOF64_Q1D1        _ARM_VF32TOF64_Q1D1
# define _AAS_VF32TOF64_Q2Q1        _ARM_VF32TOF64_Q2Q1

	/*
     * vector convert double to float
	 */
/* D --> S
   Q --> D
   (Q,Q) --> Q */
# define _AAS_VF64TOF32_S1D1        _ARM_VF64TOF32_S1D1
# define _AAS_VF64TOF32_D1Q1        _ARM_VF64TOF32_D1Q1
# define _AAS_VF64TOF32_Q1Q2        _ARM_VF64TOF32_Q1Q2

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * :w  - data width
 * :ww - double data width
 * for instance: w = 8, ww = 16
 *               long operation: ww = w + w
 *               wide operation: ww = ww + w
 *
 * vector negate               : r[i]:w   = -a[i]:w
 *
 * vector add                           : r[i]:w   = a[i]:w + b[i]:w
 * vector add saturated                 : r[i]:w   = saturate(a[i]:w + b[i]:w)
 * vector add long                      : r[i]:ww  = a[i]:w + b[i]:w
 * vector add wide                      : r[i]:ww  = a[i]:ww + b[i]:w
 * vector halving add                   : r[i]:w   = (a[i]:w + b[i]:w) >> 1
 * vector rounding halving add          : r[i]:w   = (a[i]:w + b[i]:w + 1) >> 1
 *
 * vector sub                           : r[i]:w   = a[i]:w - b[i]:w
 * vector sub saturated                 : r[i]:w   = saturate(a[i]:w - b[i]:w)
 * vector sub long                      : r[i]:ww  = a[i]:w - b[i]:w
 * vector sub wide                      : r[i]:ww  = a[i]:ww - b[i]:w
 * vector halving sub                   : r[i]:w   = (a[i]:w - b[i]:w) >> 1
 *
 * vector mul                           : r[i]:w   = a[i]:w * b[i]:w
 * vector mul long                      : r[i]:ww  = a[i]:w * b[i]:w
 * vector mul long saturated            : r[i]:ww  = saturate(a[i]:w * b[i]:w)
 * vector mul accumulate                : r[i]:w  += a[i]:w * b[i]:w
 * vector mul accumulate long           : r[i]:ww += a[i]:w * b[i]:w
 * vector mul accumulate saturated long : r[i]:ww = saturate(r[i]:ww + (a[i]:w * b[i]:w))
 * vector mul subtract                  : r[i]:w  -= a[i]:w * b[i]:w
 * vector mul subtract long             : r[i]:ww -= a[i]:w * b[i]:w
 * vector mul subtract saturated long   : r[i]:ww = saturate(r[i]:ww - (a[i]:w * b[i]:w))
 *
 * vector doubling mul saturated with right shift 16/32          : r = saturate((a * b * 2) >>{U} 16{32})
 * vector rounding doubling mul saturated with right shift 16/32 : r = saturate(round((a * b * 2) >>{U} 16{32}))
 */


    /*
     * VNEGATE
     */

/* negate: Q --> Q
   negate: D --> D */
# define _AAS_VNEGATE_Q             _ARM_VNEGATE_Q
# define _AAS_VNEGATE_D             _ARM_VNEGATE_D

# define _AAS_VNEGATEF_Q            _ARM_VNEGATEF_Q
# define _AAS_VNEGATEF_D            _ARM_VNEGATEF_D

/* negate saturated: Q --> Q
   negate saturated: D --> D */
# define _AAS_SAT_VNEGATE_Q         _ARM_SAT_VNEGATE_Q
# define _AAS_SAT_VNEGATE_D         _ARM_SAT_VNEGATE_D

/* ------------------------------------------------------------------------- */

    /*
     * VADD
     */

/* add: (D,D) --> D
   add: (Q,Q) --> Q */
# define _AAS_VADD_Q                _ARM_VADD_Q
# define _AAS_VADD_D                _ARM_VADD_D

/* add< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _AAS_VADDF_Q               _ARM_VADDF_Q
# define _AAS_VADDF_D               _ARM_VADDF_D
# define _AAS_VADDF_S               _ARM_VADDF_S

/* add< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _AAS_VADDF64_Q             _ARM_VADDF64_Q
# define _AAS_VADDF64_D             _ARM_VADDF64_D

/* add long: (D,D) --> Q
   add long: (Q,Q) --> (Q,Q) */
# define _AAS_VADD_WIDEN_Q1D1D1     _ARM_VADD_WIDEN_Q1D1D1
# define _AAS_VADD_WIDEN_Q2Q1Q1     _ARM_VADD_WIDEN_Q2Q1Q1

/* add wide: (Q,D) --> Q
   add wide: (Q,Q,Q) --> (Q,Q) */
# define _AAS_VADD_WIDEN_Q1Q1D1     _ARM_VADD_WIDEN_Q1Q1D1
# define _AAS_VADD_WIDEN_Q2Q2Q1     _ARM_VADD_WIDEN_Q2Q2Q1


/* add saturated: (D,D) --> D
   add saturated: (Q,Q) --> Q */
# define _AAS_SAT_VADD_Q            _ARM_SAT_VADD_Q
# define _AAS_SAT_VADD_D            _ARM_SAT_VADD_D

/* halving add */
# define _AAS_VHADD_Q               _ARM_VHADD_Q
# define _AAS_VHADD_D               _ARM_VHADD_D

/* rounding halving add */
# define _AAS_RND_VHADD_Q           _ARM_RND_VHADD_Q
# define _AAS_RND_VHADD_D           _ARM_RND_VHADD_D

/* add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _AAS_VADDH_NARROW_D1Q1Q1       _ARM_VADDH_NARROW_D1Q1Q1
/* narrow: ((Q,Q),(Q,Q)) --> Q */
# define _AAS_VADDH_NARROW_Q1Q2Q2       _ARM_VADDH_NARROW_Q1Q2Q2

/* rounding add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _AAS_RND_VADDH_NARROW_D1Q1Q1   _ARM_RND_VADDH_NARROW_D1Q1Q1
/* narrow: ((Q,Q),(Q,Q)) --> Q */
# define _AAS_RND_VADDH_NARROW_Q1Q2Q2   _ARM_RND_VADDH_NARROW_Q1Q2Q2

/* ------------------------------------------------------------------------- */

    /*
     * VSUB
     */

/* sub: (D,D) --> D
   sub: (Q,Q) --> Q */
# define _AAS_VSUB_Q                _ARM_VSUB_Q
# define _AAS_VSUB_D                _ARM_VSUB_D

/* sub< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _AAS_VSUBF_Q               _ARM_VSUBF_Q
# define _AAS_VSUBF_D               _ARM_VSUBF_D
# define _AAS_VSUBF_S               _ARM_VSUBF_S

/* sub< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _AAS_VSUBF64_Q             _ARM_VSUBF64_Q
# define _AAS_VSUBF64_D             _ARM_VSUBF64_D

/* sub long: (D,D) --> Q
   sub long: (Q,Q) --> (Q,Q) */
# define _AAS_VSUB_WIDEN_Q1D1D1     _ARM_VSUB_WIDEN_Q1D1D1
# define _AAS_VSUB_WIDEN_Q2Q1Q1     _ARM_VSUB_WIDEN_Q2Q1Q1

/* sub wide: (Q,D) --> Q
   sub wide: (Q,Q,Q) --> (Q,Q) */
# define _AAS_VSUB_WIDEN_Q1Q1D1     _ARM_VSUB_WIDEN_Q1Q1D1
# define _AAS_VSUB_WIDEN_Q2Q2Q1     _ARM_VSUB_WIDEN_Q2Q2Q1


/* sub saturated: (D,D) --> D
   sub saturated: (Q,Q) --> Q */
# define _AAS_SAT_VSUB_Q            _ARM_SAT_VSUB_Q
# define _AAS_SAT_VSUB_D            _ARM_SAT_VSUB_D


/* halving sub */
# define _AAS_VHSUB_Q               _ARM_VHSUB_Q
# define _AAS_VHSUB_D               _ARM_VHSUB_D


/* sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _AAS_VSUBH_NARROW_D1Q1Q1       _ARM_VSUBH_NARROW_D1Q1Q1
/* narrow: ((Q,Q),(Q,Q)) --> Q */
# define _AAS_VSUBH_NARROW_Q1Q2Q2       _ARM_VSUBH_NARROW_Q1Q2Q2

/* rounding sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _AAS_RND_VSUBH_NARROW_D1Q1Q1   _ARM_RND_VSUBH_NARROW_D1Q1Q1
/* narrow: ((Q,Q),(Q,Q)) --> Q */
# define _AAS_RND_VSUBH_NARROW_Q1Q2Q2   _ARM_RND_VSUBH_NARROW_Q1Q2Q2

/* ------------------------------------------------------------------------- */

    /*
     * VMUL
     */

/* mul: (Q,Q) --> Q
   mul: (D,D) --> D */
# define _AAS_VMUL_Q                _ARM_VMUL_Q
# define _AAS_VMUL_D                _ARM_VMUL_D

/* mulS: (Q,D[L]) --> Q
   mulS: (D,D[L]) --> D */
# define _AAS_VMULS_Q               _ARM_VMULS_Q
# define _AAS_VMULS_D               _ARM_VMULS_D

/* mul< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _AAS_VMULF_Q               _ARM_VMULF_Q
# define _AAS_VMULF_D               _ARM_VMULF_D
# define _AAS_VMULF_S               _ARM_VMULF_S

/* mulS< float >
 * (q,d[L]) --> q
   (d,d[L]) --> d  */
# define _AAS_VMULSF_Q              _ARM_VMULSF_Q
# define _AAS_VMULSF_D              _ARM_VMULSF_D

/* mul< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _AAS_VMULF64_Q             _ARM_VMULF64_Q
# define _AAS_VMULF64_D             _ARM_VMULF64_D

/* mul long: (D,D) --> Q
   mul long: (Q,Q) --> (Q,Q)
   mulS long: (D,D[L]) --> Q
   mulS long: (Q,D[L]) --> (Q,Q) */
# define _AAS_VMUL_WIDEN_Q1D1D1     _ARM_VMUL_WIDEN_Q1D1D1
# define _AAS_VMUL_WIDEN_Q2Q1Q1     _ARM_VMUL_WIDEN_Q2Q1Q1
# define _AAS_VMULS_WIDEN_Q1D1D1    _ARM_VMULS_WIDEN_Q1D1D1
# define _AAS_VMULS_WIDEN_Q2Q1D1    _ARM_VMULS_WIDEN_Q2Q1D1

/* mul wide: (Q,D) --> Q
   mul wide: (Q,Q,Q) --> (Q,Q)
   mulS wide: (Q,D[L]) --> Q
   mulS wide: (Q,Q,D[L]) --> (Q,Q) */
# define _AAS_VMUL_WIDEN_Q1Q1D1     _ARM_VMUL_WIDEN_Q1Q1D1
# define _AAS_VMUL_WIDEN_Q2Q2Q1     _ARM_VMUL_WIDEN_Q2Q2Q1
# define _AAS_VMULS_WIDEN_Q1Q1D1    _ARM_VMULS_WIDEN_Q1Q1D1
# define _AAS_VMULS_WIDEN_Q2Q2D1    _ARM_VMULS_WIDEN_Q2Q2D1


/* doubling mul saturated long: (D,D) --> Q; domain: {int16, int32}
   doubling mul saturated long: (Q,Q) --> (Q,Q) */
# define _AAS_SAT_VMUL2_WIDEN_Q1D1D1    _ARM_SAT_VMUL2_WIDEN_Q1D1D1
# define _AAS_SAT_VMUL2_WIDEN_Q2Q1Q1    _ARM_SAT_VMUL2_WIDEN_Q2Q1Q1


/* doubling mul saturated, storing only high half
   with right shift 16/32 : r = saturate((a * b * 2) >> 16{32}) */
# define _AAS_SAT_VMUL2H_Q          _ARM_SAT_VMUL2H_Q
# define _AAS_SAT_VMUL2H_D          _ARM_SAT_VMUL2H_D

/* rounding doubling mul saturated, storing only high half
   with right shift 16/32: r = saturate(round((a * b * 2) >> 16{32})) */
# define _AAS_SAT_RND_VMUL2H_Q      _ARM_SAT_RND_VMUL2H_Q
# define _AAS_SAT_RND_VMUL2H_D      _ARM_SAT_RND_VMUL2H_D


/* mul accumulate/subtract: (Q,Q) --> Q
   mul accumulate/subtract: (D,D) --> D
   mulS accumulate/subtract: (Q,D[L]) --> Q
   mulS accumulate/subtract: (D,D[L]) --> D */
# define _AAS_VMUL_ACCUM_Q          _ARM_VMUL_ACCUM_Q
# define _AAS_VMUL_ACCUM_D          _ARM_VMUL_ACCUM_D
# define _AAS_VMULS_ACCUM_Q         _ARM_VMULS_ACCUM_Q
# define _AAS_VMULS_ACCUM_D         _ARM_VMULS_ACCUM_D
# define _AAS_VMUL_SUBTR_Q          _ARM_VMUL_SUBTR_Q
# define _AAS_VMUL_SUBTR_D          _ARM_VMUL_SUBTR_D
# define _AAS_VMULS_SUBTR_Q         _ARM_VMULS_SUBTR_Q
# define _AAS_VMULS_SUBTR_D         _ARM_VMULS_SUBTR_D

/* mul accumulate/subtract< float >
   (q,q) --> q
   (d,d) --> d
   (q,d[L]) --> q
   (d,d[L]) --> d */
# define _AAS_VMULF_ACCUM_Q         _ARM_VMULF_ACCUM_Q
# define _AAS_VMULF_ACCUM_D         _ARM_VMULF_ACCUM_D
# define _AAS_VMULSF_ACCUM_Q        _ARM_VMULSF_ACCUM_Q
# define _AAS_VMULSF_ACCUM_D        _ARM_VMULSF_ACCUM_D
# define _AAS_VMULF_SUBTR_Q         _ARM_VMULF_SUBTR_Q
# define _AAS_VMULF_SUBTR_D         _ARM_VMULF_SUBTR_D
# define _AAS_VMULSF_SUBTR_Q        _ARM_VMULSF_SUBTR_Q
# define _AAS_VMULSF_SUBTR_D        _ARM_VMULSF_SUBTR_D

/* mul accumulate/subtract< double >
   (q,q) --> q
   (d,d) --> d */
# define _AAS_VMULF64_ACCUM_Q       _ARM_VMULF64_ACCUM_Q
# define _AAS_VMULF64_ACCUM_D       _ARM_VMULF64_ACCUM_D
# define _AAS_VMULF64_SUBTR_Q       _ARM_VMULF64_SUBTR_Q
# define _AAS_VMULF64_SUBTR_D       _ARM_VMULF64_SUBTR_D


/* mul accumulate/subtract long: (D,D) --> Q
   mul accumulate/subtract long: (Q,Q) --> (Q,Q)
   mulS accumulate/subtract long: (D,D[L]) --> Q
   mulS accumulate/subtract long: (Q,D[L]) --> (Q,Q) */
# define _AAS_VMUL_ACCUM_WIDEN_Q1D1D1   _ARM_VMUL_ACCUM_WIDEN_Q1D1D1
# define _AAS_VMUL_ACCUM_WIDEN_Q2Q1Q1   _ARM_VMUL_ACCUM_WIDEN_Q2Q1Q1
# define _AAS_VMULS_ACCUM_WIDEN_Q1D1D1  _ARM_VMULS_ACCUM_WIDEN_Q1D1D1
# define _AAS_VMULS_ACCUM_WIDEN_Q2Q1D1  _ARM_VMULS_ACCUM_WIDEN_Q2Q1D1
# define _AAS_VMUL_SUBTR_WIDEN_Q1D1D1   _ARM_VMUL_SUBTR_WIDEN_Q1D1D1
# define _AAS_VMUL_SUBTR_WIDEN_Q2Q1Q1   _ARM_VMUL_SUBTR_WIDEN_Q2Q1Q1
# define _AAS_VMULS_SUBTR_WIDEN_Q1D1D1  _ARM_VMULS_SUBTR_WIDEN_Q1D1D1
# define _AAS_VMULS_SUBTR_WIDEN_Q2Q1D1  _ARM_VMULS_SUBTR_WIDEN_Q2Q1D1


/* doubling mul accumulate/subtract saturated long: (D,D) --> Q; domain: {int16, int32}
   doubling mul accumulate/subtract saturated long: (Q,Q) --> (Q,Q) */
# define _AAS_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1  _ARM_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1
# define _AAS_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1  _ARM_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1
# define _AAS_SAT_VMUL2_ACCUM_WIDEN_Q2Q1Q1  _ARM_SAT_VMUL2_ACCUM_WIDEN_Q2Q1Q1
# define _AAS_SAT_VMUL2_SUBTR_WIDEN_Q2Q1Q1  _ARM_SAT_VMUL2_SUBTR_WIDEN_Q2Q1Q1


# define _AAS_VFMAEX                _ARM_VFMAEX
# define _AAS_VFMSEX                _ARM_VFMSEX

# define _AAS_VFMA                  _ARM_VFMA
# define _AAS_VFMS                  _ARM_VFMS
# define _AAS_VFMA64                _ARM_VFMA64
# define _AAS_VFMS64                _ARM_VFMS64

/* ------------------------------------------------------------------------- */

    /*
     * VDIV
     */

/* note: only for F32 */
# define _AAS_VDIVF_Q               _ARM_VDIVF_Q
# define _AAS_VDIVF_D               _ARM_VDIVF_D
# define _AAS_VDIVF_S               _ARM_VDIVF_S

/* note: only for F64 */
# define _AAS_VDIVF64_Q             _ARM_VDIVF64_Q
# define _AAS_VDIVF64_D             _ARM_VDIVF64_D

/* @} */


/* ========================================================================= */

/*
 * Min/Max operations
 * @{
 *
 * r[i] = min(s1[i],s2[i])
 * r[i] = max(s1[i],s2[i])
 */

    /*
     * VMIN
     */

# define _AAS_VMIN_D                _ARM_VMIN_D
# define _AAS_VMIN_Q                _ARM_VMIN_Q

# define _AAS_VMINF_D               _ARM_VMINF_D
# define _AAS_VMINF_Q               _ARM_VMINF_Q

/* ------------------------------------------------------------------------- */

    /*
     * VMAX
     */

# define _AAS_VMAX_D                _ARM_VMAX_D
# define _AAS_VMAX_Q                _ARM_VMAX_Q

# define _AAS_VMAXF_D               _ARM_VMAXF_D
# define _AAS_VMAXF_Q               _ARM_VMAXF_Q

/* @} */


/* ========================================================================= */

/*
 * Folding operations: add, min, max
 * @{
 *
 * vector pairwise add [long] :
 *     (a,b) --> r : {r[i]} = {(a[2*i] + a[2*i + 1]),... (b[2*j] + b[2*j + 1])}
 *
 * vector pairwise add accumulate long :
 *     {r[i]} = {(r[i] + a[2*i] + a[2*i + 1]),... (r[j] + b[2*j] + b[2*j + 1])}
 *
 * vector add across vector :
 *     (a,b) --> r[0] : r[0] = (a[0] + a[1] + ... + a[n - 1])
 */

    /*
     * PADD, PADD long, PADD accumulate long, VADD, VADD long
     */

/* (q,q) --> q */
/* (d,d) --> d */
# define _AAS_VPADD_Q               _ARM_VPADD_Q
# define _AAS_VPADD_D               _ARM_VPADD_D
/* q --> q(result in low half(d))
 * @note: high half can be modified
 */
# define _AAS_VPADD_Q1              _ARM_VPADD_Q1

/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _AAS_VPADD_WIDEN_Q         _ARM_VPADD_WIDEN_Q
# define _AAS_VPADD_WIDEN_D         _ARM_VPADD_WIDEN_D

/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _AAS_VPADD_ACCUM_WIDEN_Q   _ARM_VPADD_ACCUM_WIDEN_Q
# define _AAS_VPADD_ACCUM_WIDEN_D   _ARM_VPADD_ACCUM_WIDEN_D

/* (q,q) --> q[0] */
/* (d,d) --> d[0] */
# define _AAS_VVADD_Q               _ARM_VVADD_Q
# define _AAS_VVADD_D               _ARM_VVADD_D

/* (q{dn,dn+1}) --> q[0] */
/* (hi-d,lo-d) --> d[0] */
# define _AAS_VVADD_WIDEN_Q         _ARM_VVADD_WIDEN_Q
# define _AAS_VVADD_WIDEN_D         _ARM_VVADD_WIDEN_D

/* ------------------------------------------------------------------------- */

    /*
     * PMIN, VMIN
     */

# define _AAS_VPMIN_Q               _ARM_VPMIN_Q
# define _AAS_VPMIN_D               _ARM_VPMIN_D
# define _AAS_VPMIN_Q1              _ARM_VPMIN_Q1

/* pmin< float > */
# define _AAS_VPMINF_Q              _ARM_VPMINF_Q
# define _AAS_VPMINF_D              _ARM_VPMINF_D
# define _AAS_VPMINF_Q1             _ARM_VPMINF_Q1

# define _AAS_VVMIN_Q               _ARM_VVMIN_Q
# define _AAS_VVMIN_D               _ARM_VVMIN_D

/* ------------------------------------------------------------------------- */

    /*
     * PMAX, VMAX
     */

# define _AAS_VPMAX_Q               _ARM_VPMAX_Q
# define _AAS_VPMAX_D               _ARM_VPMAX_D
# define _AAS_VPMAX_Q1              _ARM_VPMAX_Q1

/* pmax< float > */
# define _AAS_VPMAXF_Q              _ARM_VPMAXF_Q
# define _AAS_VPMAXF_D              _ARM_VPMAXF_D
# define _AAS_VPMAXF_Q1             _ARM_VPMAXF_Q1

# define _AAS_VVMAX_Q               _ARM_VVMAX_Q
# define _AAS_VVMAX_D               _ARM_VVMAX_D

/* ------------------------------------------------------------------------- */

    /*
     * VMINMAX
     */

# define _AAS_VVMINMAX_Q            _ARM_VVMINMAX_Q
# define _AAS_VVMINMAX_D            _ARM_VVMINMAX_D

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

    /*
     * Compare 2 vector registers
     * Op: LT, LE, EQ, GE, GT
     */

# define _AAS_VCMPI_Q               _ARM_VCMPI_Q
# define _AAS_VCMPI_D               _ARM_VCMPI_D

# define _AAS_VCMP_Q                _ARM_VCMP_Q
# define _AAS_VCMP_D                _ARM_VCMP_D


/*
 * float
 */

# define _AAS_VCMPFI_Q              _ARM_VCMPFI_Q
# define _AAS_VCMPFI_D              _ARM_VCMPFI_D

# define _AAS_VCMPF_Q               _ARM_VCMPF_Q
# define _AAS_VCMPF_D               _ARM_VCMPF_D


/*
 * Float-Point Subsystem
 *
 * 32-bit operations  -  S registers
 * 64-bit operations  -  D registers
 */

# define _AAS_VFPCMP32              _ARM_VFPCMP32
# define _AAS_VFPCMP64              _ARM_VFPCMP64
# define _AAS_VFPCMP32_IF           _ARM_VFPCMP32_IF
# define _AAS_VFPCMP64_IF           _ARM_VFPCMP64_IF

# define _AAS_VFPCMP32_UCF          _ARM_VFPCMP32_UCF
# define _AAS_VFPCMP64_UCF          _ARM_VFPCMP64_UCF
# define _AAS_VFPCMP32_UCF_IF       _ARM_VFPCMP32_UCF_IF
# define _AAS_VFPCMP64_UCF_IF       _ARM_VFPCMP64_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * vector not
 * vector and
 * vector and complement: BIC
 * vector or
 * vector or complement: ORN
 * vector xor
 * vector set all bits
 * vector clear all bits
 * vector bit select           : bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
 */

    /*
     * NOT
     */

# define _AAS_VNOT_Q                _ARM_VNOT_Q
# define _AAS_VNOT_D                _ARM_VNOT_D

    /*
     * AND
     */

# define _AAS_VAND_Q                _ARM_VAND_Q
# define _AAS_VAND_D                _ARM_VAND_D

    /*
     * BIC : AND complement
     */

# define _AAS_VBIC_Q                _ARM_VBIC_Q
# define _AAS_VBIC_D                _ARM_VBIC_D

    /*
     * OR
     */

# define _AAS_VORR_Q                _ARM_VORR_Q
# define _AAS_VORR_D                _ARM_VORR_D

    /*
     * ORN : OR complement
     */

# define _AAS_VORN_Q                _ARM_VORN_Q
# define _AAS_VORN_D                _ARM_VORN_D

    /*
     * XOR
     */

# define _AAS_VXOR_Q                _ARM_VXOR_Q
# define _AAS_VXOR_D                _ARM_VXOR_D

    /*
     * Set/Clear all bits
     */

# define _AAS_VBITSET_Q             _ARM_VBITSET_Q
# define _AAS_VBITSET_D             _ARM_VBITSET_D
# define _AAS_VBITCLR_Q             _ARM_VBITCLR_Q
# define _AAS_VBITCLR_D             _ARM_VBITCLR_D


    /*
     * Bit select: bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
     * Bit set if true:
     * Bit set if false:
     */

# define _AAS_VBITSEL_Q             _ARM_VBITSEL_Q
# define _AAS_VBITSEL_D             _ARM_VBITSEL_D

# define _AAS_VBITIFT_Q             _ARM_VBITIFT_Q
# define _AAS_VBITIFT_D             _ARM_VBITIFT_D

# define _AAS_VBITIFF_Q             _ARM_VBITIFF_Q
# define _AAS_VBITIFF_D             _ARM_VBITIFF_D


    /*
     * Bit count leading 0/1:
     */

# define _AAS_VCOUNTLEADZEROBITS_Q  _ARM_VCOUNTLEADZEROBITS_Q
# define _AAS_VCOUNTLEADZEROBITS_D  _ARM_VCOUNTLEADZEROBITS_D

# define _AAS_VCOUNTLEADSETBITS_Q   _ARM_VCOUNTLEADSETBITS_Q
# define _AAS_VCOUNTLEADSETBITS_D   _ARM_VCOUNTLEADSETBITS_D

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * vector shift left by immediate
 * vector shift left saturated by immediate
 * vector shift left long by immediate
 *
 * (negative shift-value shift right)
 * vector shift left by signed var
 * vector rounding shift left by signed
 * vector shift left saturated by signed
 * vector rounding shift left saturated by signed
 *
 * vector [rounding] shift right by immediate
 * vector [rounding] shift right narrow by immediate
 * vector saturating [rounding] shift right narrow by immediate
 */

    /*
     * VSHL by immediate
     */

# define _AAS_VSHLI_Q               _ARM_VSHLI_Q
# define _AAS_VSHLI_D               _ARM_VSHLI_D


/* shift saturated */
# define _AAS_SAT_VSHLI_Q           _ARM_SAT_VSHLI_Q
# define _AAS_SAT_VSHLI_D           _ARM_SAT_VSHLI_D

/* signed shift saturated */
# define _AAS_SAT_VSHLSI_Q          _ARM_SAT_VSHLSI_Q
# define _AAS_SAT_VSHLSI_D          _ARM_SAT_VSHLSI_D

/* unsigned shift saturated */
# define _AAS_SAT_VSHLUI_Q          _ARM_SAT_VSHLUI_Q
# define _AAS_SAT_VSHLUI_D          _ARM_SAT_VSHLUI_D


/*
 * shift long
 */

# define _AAS_VSHLI_WIDEN_Q1D1      _ARM_VSHLI_WIDEN_Q1D1
# define _AAS_VSHLI_WIDEN_Q2Q1      _ARM_VSHLI_WIDEN_Q2Q1

/* signed shift long */
# define _AAS_VSHLSI_WIDEN_Q1D1     _ARM_VSHLSI_WIDEN_Q1D1
# define _AAS_VSHLSI_WIDEN_Q2Q1     _ARM_VSHLSI_WIDEN_Q2Q1

/* unsigned shift long */
# define _AAS_VSHLUI_WIDEN_Q1D1     _ARM_VSHLUI_WIDEN_Q1D1
# define _AAS_VSHLUI_WIDEN_Q2Q1     _ARM_VSHLUI_WIDEN_Q2Q1

/* ------------------------------------------------------------------------- */

    /*
     * VSHL by signed var (negative shift-value shift right)
     */

/* shift left */
# define _AAS_VSHL_Q                _ARM_VSHL_Q
# define _AAS_VSHL_D                _ARM_VSHL_D

/* signed shift left */
# define _AAS_VSHLS_Q               _ARM_VSHLS_Q
# define _AAS_VSHLS_D               _ARM_VSHLS_D

/* unsigned shift left */
# define _AAS_VSHLU_Q               _ARM_VSHLU_Q
# define _AAS_VSHLU_D               _ARM_VSHLU_D


/* shift left saturated */
# define _AAS_SAT_VSHL_Q            _ARM_SAT_VSHL_Q
# define _AAS_SAT_VSHL_D            _ARM_SAT_VSHL_D

/* signed shift left saturated */
# define _AAS_SAT_VSHLS_Q           _ARM_SAT_VSHLS_Q
# define _AAS_SAT_VSHLS_D           _ARM_SAT_VSHLS_D

/* unsigned shift left saturated */
# define _AAS_SAT_VSHLU_Q           _ARM_SAT_VSHLU_Q
# define _AAS_SAT_VSHLU_D           _ARM_SAT_VSHLU_D


/* rounding shift left */
# define _AAS_RND_VSHL_Q            _ARM_RND_VSHL_Q
# define _AAS_RND_VSHL_D            _ARM_RND_VSHL_D

/* signed rounding shift left */
# define _AAS_RND_VSHLS_Q           _ARM_RND_VSHLS_Q
# define _AAS_RND_VSHLS_D           _ARM_RND_VSHLS_D

/* unsigned rounding shift left */
# define _AAS_RND_VSHLU_Q           _ARM_RND_VSHLU_Q
# define _AAS_RND_VSHLU_D           _ARM_RND_VSHLU_D


/* rounding shift left saturated */
# define _AAS_SAT_RND_VSHL_Q        _ARM_SAT_RND_VSHL_Q
# define _AAS_SAT_RND_VSHL_D        _ARM_SAT_RND_VSHL_D

/* signed rounding shift left saturated */
# define _AAS_SAT_RND_VSHLS_Q       _ARM_SAT_RND_VSHLS_Q
# define _AAS_SAT_RND_VSHLS_D       _ARM_SAT_RND_VSHLS_D

/* unsigned rounding shift left saturated */
# define _AAS_SAT_RND_VSHLU_Q       _ARM_SAT_RND_VSHLU_Q
# define _AAS_SAT_RND_VSHLU_D       _ARM_SAT_RND_VSHLU_D

/* ------------------------------------------------------------------------- */

    /*
     * VSHR by immediate
     */

/* shift right */
# define _AAS_VSHRI_Q               _ARM_VSHRI_Q
# define _AAS_VSHRI_D               _ARM_VSHRI_D

/* signed shift right */
# define _AAS_VSHRSI_Q              _ARM_VSHRSI_Q
# define _AAS_VSHRSI_D              _ARM_VSHRSI_D

/* unsigned shift right */
# define _AAS_VSHRUI_Q              _ARM_VSHRUI_Q
# define _AAS_VSHRUI_D              _ARM_VSHRUI_D


/* rounding shift right */
# define _AAS_RND_VSHRI_Q           _ARM_RND_VSHRI_Q
# define _AAS_RND_VSHRI_D           _ARM_RND_VSHRI_D

/* signed rounding shift right */
# define _AAS_RND_VSHRSI_Q          _ARM_RND_VSHRSI_Q
# define _AAS_RND_VSHRSI_D          _ARM_RND_VSHRSI_D

/* unsigned rounding shift right */
# define _AAS_RND_VSHRUI_Q          _ARM_RND_VSHRUI_Q
# define _AAS_RND_VSHRUI_D          _ARM_RND_VSHRUI_D


/* [signed/unsigned] shift right narrow */
/* narrow: Q --> D */
# define _AAS_VSHRI_NARROW_D1Q1     _ARM_VSHRI_NARROW_D1Q1
//# define _AAS_VSHRSI_NARROW_D1Q1    _ARM_VSHRSI_NARROW_D1Q1
//# define _AAS_VSHRUI_NARROW_D1Q1    _ARM_VSHRUI_NARROW_D1Q1

/* [signed/unsigned] shift right narrow */
/* narrow: (Q,Q) --> Q */
# define _AAS_VSHRI_NARROW_Q1Q2     _ARM_VSHRI_NARROW_Q1Q2
//# define _AAS_VSHRSI_NARROW_Q1Q2    _ARM_VSHRSI_NARROW_Q1Q2
//# define _AAS_VSHRUI_NARROW_Q1Q2    _ARM_VSHRUI_NARROW_Q1Q2


/* [signed/unsigned] rounding shift right narrow */
/* narrow: Q --> D */
# define _AAS_RND_VSHRI_NARROW_D1Q1 _ARM_RND_VSHRI_NARROW_D1Q1
//# define _AAS_RND_VSHRSI_NARROW_D1Q1    _ARM_RND_VSHRSI_NARROW_D1Q1
//# define _AAS_RND_VSHRUI_NARROW_D1Q1    _ARM_RND_VSHRUI_NARROW_D1Q1

/* [signed/unsigned] rounding shift right narrow */
/* narrow: (Q,Q) --> Q */
# define _AAS_RND_VSHRI_NARROW_Q1Q2 _ARM_RND_VSHRI_NARROW_Q1Q2
//# define _AAS_RND_VSHRSI_NARROW_Q1Q2    _ARM_RND_VSHRSI_NARROW_Q1Q2
//# define _AAS_RND_VSHRUI_NARROW_Q1Q2    _ARM_RND_VSHRUI_NARROW_Q1Q2


/* [signed/unsigned] shift right narrow saturated */
/* narrow: Q --> D */
# define _AAS_SAT_VSHRI_NARROW_D1Q1     _ARM_SAT_VSHRI_NARROW_D1Q1
# define _AAS_SAT_VSHRSI_NARROW_D1Q1    _ARM_SAT_VSHRSI_NARROW_D1Q1
# define _AAS_SAT_VSHRUI_NARROW_D1Q1    _ARM_SAT_VSHRUI_NARROW_D1Q1

/* [signed/unsigned] shift right narrow saturated */
/* narrow: (Q,Q) --> Q */
# define _AAS_SAT_VSHRI_NARROW_Q1Q2     _ARM_SAT_VSHRI_NARROW_Q1Q2
# define _AAS_SAT_VSHRSI_NARROW_Q1Q2    _ARM_SAT_VSHRSI_NARROW_Q1Q2
# define _AAS_SAT_VSHRUI_NARROW_Q1Q2    _ARM_SAT_VSHRUI_NARROW_Q1Q2


/* [signed/unsigned] rounding shift right narrow saturated */
/* narrow: Q --> D */
# define _AAS_SAT_RND_VSHRI_NARROW_D1Q1     _ARM_SAT_RND_VSHRI_NARROW_D1Q1
# define _AAS_SAT_RND_VSHRSI_NARROW_D1Q1    _ARM_SAT_RND_VSHRSI_NARROW_D1Q1
# define _AAS_SAT_RND_VSHRUI_NARROW_D1Q1    _ARM_SAT_RND_VSHRUI_NARROW_D1Q1

/* [signed/unsigned] rounding shift right narrow saturated */
/* narrow: (Q,Q) --> Q */
# define _AAS_SAT_RND_VSHRI_NARROW_Q1Q2     _ARM_SAT_RND_VSHRI_NARROW_Q1Q2
# define _AAS_SAT_RND_VSHRSI_NARROW_Q1Q2    _ARM_SAT_RND_VSHRSI_NARROW_Q1Q2
# define _AAS_SAT_RND_VSHRUI_NARROW_Q1Q2    _ARM_SAT_RND_VSHRUI_NARROW_Q1Q2

/* ------------------------------------------------------------------------- */

    /*
     * VSHR and Accumulate by immediate
     */

/* shift right */
# define _AAS_VSHRADDI_Q            _ARM_VSHRADDI_Q
# define _AAS_VSHRADDI_D            _ARM_VSHRADDI_D

/* signed shift right */
# define _AAS_VSHRADDSI_Q           _ARM_VSHRADDSI_Q
# define _AAS_VSHRADDSI_D           _ARM_VSHRADDSI_D

/* unsigned shift right */
# define _AAS_VSHRADDUI_Q           _ARM_VSHRADDUI_Q
# define _AAS_VSHRADDUI_D           _ARM_VSHRADDUI_D


/* rounding shift right */
# define _AAS_RND_VSHRADDI_Q        _ARM_RND_VSHRADDI_Q
# define _AAS_RND_VSHRADDI_D        _ARM_RND_VSHRADDI_D

/* signed rounding shift right */
# define _AAS_RND_VSHRADDSI_Q       _ARM_RND_VSHRADDSI_Q
# define _AAS_RND_VSHRADDSI_D       _ARM_RND_VSHRADDSI_D

/* unsigned rounding shift right */
# define _AAS_RND_VSHRADDUI_Q       _ARM_RND_VSHRADDUI_Q
# define _AAS_RND_VSHRADDUI_D       _ARM_RND_VSHRADDUI_D

/* @} */


/* ========================================================================= */

/*
 * Generic data processing operations
 * @{
 *
 * vector element shift left by immediate
 * vector element shift right by immediate
 * vector element rotate left by immediate
 * vector element rotate right by immediate
 *
 * interleaves the elements of two vector (zip)
 * de-interleaves the elements of two vector (unzip)
 */

    /*
     * VEXTRACT
     */

# define _AAS_VEXTRACT_Q            _ARM_VEXTRACT_Q
# define _AAS_VEXTRACT_D            _ARM_VEXTRACT_D


/*
 * vector elements: {0,1,2,3,4,5,6,7}
 *
 * shift-left(r,s1,s2,3):
 *    s1: {0,1,2,3,4,5,6,7};
 *    s2: {8,9,10,11,12,13,14,15}
 *    r : {3,4,5,6,7,8,9,10}
 *
 * shift-right(r,s1,s2,3):
 *    s1: {0,1,2,3,4,5,6,7};
 *    s2: {8,9,10,11,12,13,14,15}
 *    r : {5,6,7,8,9,10,11,12}
 */
# define _AAS_VESHL_Q               _ARM_VESHL_Q
# define _AAS_VESHL_D               _ARM_VESHL_D
# define _AAS_VESHR_Q               _ARM_VESHR_Q
# define _AAS_VESHR_D               _ARM_VESHR_D

# define _AAS_VEROTL_D              _ARM_VEROTL_D
# define _AAS_VEROTL_Q              _ARM_VEROTL_Q
# define _AAS_VEROTR_Q              _ARM_VEROTR_Q
# define _AAS_VEROTR_D              _ARM_VEROTR_D

/* ------------------------------------------------------------------------- */

    /*
     * VZIP/VUZP (zip/unzip)
     */

/*
 * vector elements: {0,1,2,3,4,5,6,7}
 *
 * zip(s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    s1: {a0,b0,a1,b1,a2,b2,a3,b3} - also result of zip1;
 *    s2: {a4,b4,a5,b5,a6,b6,a7,b7} - also result of zip2;
 *
 * unzip(s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    s1: {a0,a2,a4,a6,b0,b2,b4,b6} - also result of unzip1;
 *    s2: {a1,a3,a5,a7,b1,b3,b5,b7} - also result of unzip2;
 */
# ifdef _ARM_VZIP_Q
#  define _AAS_VZIP_Q                   _ARM_VZIP_Q
#  define _AAS_VZIP_D                   _ARM_VZIP_D
#  define _AAS_VUZP_Q                   _ARM_VUZP_Q
#  define _AAS_VUZP_D                   _ARM_VUZP_D
# endif
# ifdef _ARM_VZIP1_Q
#  define _AAS_VZIP1_Q                  _ARM_VZIP1_Q
#  define _AAS_VZIP2_Q                  _ARM_VZIP2_Q
#  define _AAS_VZIP1_D                  _ARM_VZIP1_D
#  define _AAS_VZIP2_D                  _ARM_VZIP2_D
#  define _AAS_VUZP1_Q                  _ARM_VUZP1_Q
#  define _AAS_VUZP2_Q                  _ARM_VUZP2_Q
#  define _AAS_VUZP1_D                  _ARM_VUZP1_D
#  define _AAS_VUZP2_D                  _ARM_VUZP2_D
# endif

/* ------------------------------------------------------------------------- */

    /*
     * VTRANSPOSE2x2
     */
# ifdef _ARM_VTRANSPOSE2x2_Q
#  define _AAS_VTRANSPOSE2x2_Q          _ARM_VTRANSPOSE2x2_Q
#  define _AAS_VTRANSPOSE2x2_D          _ARM_VTRANSPOSE2x2_D
# endif
# ifdef _ARM_VTRANSPOSE1_Q
#  define _AAS_VTRANSPOSE1_Q            _ARM_VTRANSPOSE1_Q
#  define _AAS_VTRANSPOSE2_Q            _ARM_VTRANSPOSE2_Q
#  define _AAS_VTRANSPOSE1_D            _ARM_VTRANSPOSE1_D
#  define _AAS_VTRANSPOSE2_D            _ARM_VTRANSPOSE2_D
# endif

/* ------------------------------------------------------------------------- */

    /*
     * VSWAP
     */
# define _AAS_VSWAP_Q                   _ARM_VSWAP_Q
# define _AAS_VSWAP_D                   _ARM_VSWAP_D

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 *
 * vector load multiple N-element structures into one or more registers (with de-interleaving when N>1)
 * vector store multiple N-element structures from one or more registers (with interleaving when N>1)
 *
 * vector load single N-element structure into one or more registers (with de-interleaving when N>1)
 * (Multiple copies of one n-element structure are loaded into one ore more registers).
 */


/*
 * Push/Pop
 */

	/*
     * Push/Pop fdd registers
     */

# define _AAS_VPUSH_ALLFDD          _ARM_VPUSH_ALLFDD
# define _AAS_VPUSH_AVAFDD          _ARM_VPUSH_AVAFDD
# define _AAS_VPUSH_FDD1            _ARM_VPUSH_FDD1
# define _AAS_VPUSH_FDD2            _ARM_VPUSH_FDD2
# define _AAS_VPUSH_FDD3            _ARM_VPUSH_FDD3
# define _AAS_VPUSH_FDD4            _ARM_VPUSH_FDD4
# define _AAS_VPUSH_FDD5            _ARM_VPUSH_FDD5
# define _AAS_VPUSH_FDD6            _ARM_VPUSH_FDD6
# define _AAS_VPUSH_FDD7            _ARM_VPUSH_FDD7
# define _AAS_VPUSH_FDD8            _ARM_VPUSH_FDD8
# define _AAS_VPUSH_FDD9            _ARM_VPUSH_FDD9
# define _AAS_VPUSH_FDD10           _ARM_VPUSH_FDD10
# define _AAS_VPUSH_FDD11           _ARM_VPUSH_FDD11
# define _AAS_VPUSH_FDD12           _ARM_VPUSH_FDD12
# define _AAS_VPUSH_FDD13           _ARM_VPUSH_FDD13
# define _AAS_VPUSH_FDD14           _ARM_VPUSH_FDD14
# define _AAS_VPUSH_FDD15           _ARM_VPUSH_FDD15
# define _AAS_VPUSH_FDD16           _ARM_VPUSH_FDD16

# define _AAS_VPOP_ALLFDD           _ARM_VPOP_ALLFDD
# define _AAS_VPOP_AVAFDD           _ARM_VPOP_AVAFDD
# define _AAS_VPOP_FDD1             _ARM_VPOP_FDD1
# define _AAS_VPOP_FDD2             _ARM_VPOP_FDD2
# define _AAS_VPOP_FDD3             _ARM_VPOP_FDD3
# define _AAS_VPOP_FDD4             _ARM_VPOP_FDD4
# define _AAS_VPOP_FDD5             _ARM_VPOP_FDD5
# define _AAS_VPOP_FDD6             _ARM_VPOP_FDD6
# define _AAS_VPOP_FDD7             _ARM_VPOP_FDD7
# define _AAS_VPOP_FDD8             _ARM_VPOP_FDD8
# define _AAS_VPOP_FDD9             _ARM_VPOP_FDD9
# define _AAS_VPOP_FDD10            _ARM_VPOP_FDD10
# define _AAS_VPOP_FDD11            _ARM_VPOP_FDD11
# define _AAS_VPOP_FDD12            _ARM_VPOP_FDD12
# define _AAS_VPOP_FDD13            _ARM_VPOP_FDD13
# define _AAS_VPOP_FDD14            _ARM_VPOP_FDD14
# define _AAS_VPOP_FDD15            _ARM_VPOP_FDD15
# define _AAS_VPOP_FDD16            _ARM_VPOP_FDD16

	/*
     * Push/Pop vector registers
     */

# define _AAS_VPUSH_ALL             _ARM_VPUSH_ALL

# define _AAS_VPUSH_D1              _ARM_VPUSH_D1
# define _AAS_VPUSH_D2              _ARM_VPUSH_D2
# define _AAS_VPUSH_D3              _ARM_VPUSH_D3
# define _AAS_VPUSH_D4              _ARM_VPUSH_D4
# define _AAS_VPUSH_D5              _ARM_VPUSH_D5
# define _AAS_VPUSH_D6              _ARM_VPUSH_D6
# define _AAS_VPUSH_D7              _ARM_VPUSH_D7
# define _AAS_VPUSH_D8              _ARM_VPUSH_D8
# define _AAS_VPUSH_D9              _ARM_VPUSH_D9
# define _AAS_VPUSH_D10             _ARM_VPUSH_D10
# define _AAS_VPUSH_D11             _ARM_VPUSH_D11
# define _AAS_VPUSH_D12             _ARM_VPUSH_D12
# define _AAS_VPUSH_D13             _ARM_VPUSH_D13
# define _AAS_VPUSH_D14             _ARM_VPUSH_D14
# define _AAS_VPUSH_D15             _ARM_VPUSH_D15
# define _AAS_VPUSH_D16             _ARM_VPUSH_D16

# define _AAS_VPUSH_Q1              _ARM_VPUSH_Q1
# define _AAS_VPUSH_Q2              _ARM_VPUSH_Q2
# define _AAS_VPUSH_Q3              _ARM_VPUSH_Q3
# define _AAS_VPUSH_Q4              _ARM_VPUSH_Q4
# define _AAS_VPUSH_Q5              _ARM_VPUSH_Q5
# define _AAS_VPUSH_Q6              _ARM_VPUSH_Q6
# define _AAS_VPUSH_Q7              _ARM_VPUSH_Q7
# define _AAS_VPUSH_Q8              _ARM_VPUSH_Q8

# define _AAS_VPOP_ALL              _ARM_VPOP_ALL

# define _AAS_VPOP_D1               _ARM_VPOP_D1
# define _AAS_VPOP_D2               _ARM_VPOP_D2
# define _AAS_VPOP_D3               _ARM_VPOP_D3
# define _AAS_VPOP_D4               _ARM_VPOP_D4
# define _AAS_VPOP_D5               _ARM_VPOP_D5
# define _AAS_VPOP_D6               _ARM_VPOP_D6
# define _AAS_VPOP_D7               _ARM_VPOP_D7
# define _AAS_VPOP_D8               _ARM_VPOP_D8
# define _AAS_VPOP_D9               _ARM_VPOP_D9
# define _AAS_VPOP_D10              _ARM_VPOP_D10
# define _AAS_VPOP_D11              _ARM_VPOP_D11
# define _AAS_VPOP_D12              _ARM_VPOP_D12
# define _AAS_VPOP_D13              _ARM_VPOP_D13
# define _AAS_VPOP_D14              _ARM_VPOP_D14
# define _AAS_VPOP_D15              _ARM_VPOP_D15
# define _AAS_VPOP_D16              _ARM_VPOP_D16

# define _AAS_VPOP_Q1               _ARM_VPOP_Q1
# define _AAS_VPOP_Q2               _ARM_VPOP_Q2
# define _AAS_VPOP_Q3               _ARM_VPOP_Q3
# define _AAS_VPOP_Q4               _ARM_VPOP_Q4
# define _AAS_VPOP_Q5               _ARM_VPOP_Q5
# define _AAS_VPOP_Q6               _ARM_VPOP_Q6
# define _AAS_VPOP_Q7               _ARM_VPOP_Q7
# define _AAS_VPOP_Q8               _ARM_VPOP_Q8

/*
 * Load/Store multiple N-element structures
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn) or (Qn,Qn+1)
     */

# define _AAS_VLD1_D1               _ARM_VLD1_D1
# define _AAS_VLD1_D2               _ARM_VLD1_D2
# define _AAS_VLD1_D3               _ARM_VLD1_D3
# define _AAS_VLD1_D4               _ARM_VLD1_D4

# define _AAS_VLD1_Q1               _ARM_VLD1_Q1
# define _AAS_VLD1_Q2               _ARM_VLD1_Q2
# define _AAS_VLD1_Q3               _ARM_VLD1_Q3
# define _AAS_VLD1_Q4               _ARM_VLD1_Q4


# define _AAS_VST1_D1               _ARM_VST1_D1
# define _AAS_VST1_D2               _ARM_VST1_D2
# define _AAS_VST1_D3               _ARM_VST1_D3
# define _AAS_VST1_D4               _ARM_VST1_D4

# define _AAS_VST1_Q1               _ARM_VST1_Q1
# define _AAS_VST1_Q2               _ARM_VST1_Q2
# define _AAS_VST1_Q3               _ARM_VST1_Q3
# define _AAS_VST1_Q4               _ARM_VST1_Q4


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _AAS_VLD1_D1_AIA           _ARM_VLD1_D1_AIA
# define _AAS_VLD1_D2_AIA           _ARM_VLD1_D2_AIA
# define _AAS_VLD1_D3_AIA           _ARM_VLD1_D3_AIA
# define _AAS_VLD1_D4_AIA           _ARM_VLD1_D4_AIA

# define _AAS_VLD1_Q1_AIA           _ARM_VLD1_Q1_AIA
# define _AAS_VLD1_Q2_AIA           _ARM_VLD1_Q2_AIA
# define _AAS_VLD1_Q3_AIA           _ARM_VLD1_Q3_AIA
# define _AAS_VLD1_Q4_AIA           _ARM_VLD1_Q4_AIA


# define _AAS_VST1_D1_AIA           _ARM_VST1_D1_AIA
# define _AAS_VST1_D2_AIA           _ARM_VST1_D2_AIA
# define _AAS_VST1_D3_AIA           _ARM_VST1_D3_AIA
# define _AAS_VST1_D4_AIA           _ARM_VST1_D4_AIA

# define _AAS_VST1_Q1_AIA           _ARM_VST1_Q1_AIA
# define _AAS_VST1_Q2_AIA           _ARM_VST1_Q2_AIA
# define _AAS_VST1_Q3_AIA           _ARM_VST1_Q3_AIA
# define _AAS_VST1_Q4_AIA           _ARM_VST1_Q4_AIA


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1)
		 */

# define _AAS_VLD1_D1_IA            _ARM_VLD1_D1_IA
# define _AAS_VLD1_D2_IA            _ARM_VLD1_D2_IA
# define _AAS_VLD1_D3_IA            _ARM_VLD1_D3_IA
# define _AAS_VLD1_D4_IA            _ARM_VLD1_D4_IA

# define _AAS_VLD1_Q1_IA            _ARM_VLD1_Q1_IA
# define _AAS_VLD1_Q2_IA            _ARM_VLD1_Q2_IA
# define _AAS_VLD1_Q3_IA            _ARM_VLD1_Q3_IA
# define _AAS_VLD1_Q4_IA            _ARM_VLD1_Q4_IA


# define _AAS_VST1_D1_IA            _ARM_VST1_D1_IA
# define _AAS_VST1_D2_IA            _ARM_VST1_D2_IA
# define _AAS_VST1_D3_IA            _ARM_VST1_D3_IA
# define _AAS_VST1_D4_IA            _ARM_VST1_D4_IA

# define _AAS_VST1_Q1_IA            _ARM_VST1_Q1_IA
# define _AAS_VST1_Q2_IA            _ARM_VST1_Q2_IA
# define _AAS_VST1_Q3_IA            _ARM_VST1_Q3_IA
# define _AAS_VST1_Q4_IA            _ARM_VST1_Q4_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn,Qn+1)
     */

# define _AAS_VLD2_D2               _ARM_VLD2_D2
# define _AAS_VLD2_D4               _ARM_VLD2_D4

# define _AAS_VLD2_Q2               _ARM_VLD2_Q2
# define _AAS_VLD2_Q4               _ARM_VLD2_Q4


# define _AAS_VST2_D2               _ARM_VST2_D2
# define _AAS_VST2_D4               _ARM_VST2_D4

# define _AAS_VST2_Q2               _ARM_VST2_Q2
# define _AAS_VST2_Q4               _ARM_VST2_Q4


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _AAS_VLD2_D2_AIA           _ARM_VLD2_D2_AIA
# define _AAS_VLD2_D4_AIA           _ARM_VLD2_D4_AIA

# define _AAS_VLD2_Q2_AIA           _ARM_VLD2_Q2_AIA
# define _AAS_VLD2_Q4_AIA           _ARM_VLD2_Q4_AIA


# define _AAS_VST2_D2_AIA           _ARM_VST2_D2_AIA
# define _AAS_VST2_D4_AIA           _ARM_VST2_D4_AIA

# define _AAS_VST2_Q2_AIA           _ARM_VST2_Q2_AIA
# define _AAS_VST2_Q4_AIA           _ARM_VST2_Q4_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1)
		 */

# define _AAS_VLD2_D2_IA            _ARM_VLD2_D2_IA
# define _AAS_VLD2_D4_IA            _ARM_VLD2_D4_IA

# define _AAS_VLD2_Q2_IA            _ARM_VLD2_Q2_IA
# define _AAS_VLD2_Q4_IA            _ARM_VLD2_Q4_IA


# define _AAS_VST2_D2_IA            _ARM_VST2_D2_IA
# define _AAS_VST2_D4_IA            _ARM_VST2_D4_IA

# define _AAS_VST2_Q2_IA            _ARM_VST2_Q2_IA
# define _AAS_VST2_Q4_IA            _ARM_VST2_Q4_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _AAS_VLD3_D3               _ARM_VLD3_D3
# define _AAS_VLD3_Q3               _ARM_VLD3_Q3

# define _AAS_VST3_D3               _ARM_VST3_D3
# define _AAS_VST3_Q3               _ARM_VST3_Q3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _AAS_VLD3_D3_AIA           _ARM_VLD3_D3_AIA
# define _AAS_VLD3_Q3_AIA           _ARM_VLD3_Q3_AIA

# define _AAS_VST3_D3_AIA           _ARM_VST3_D3_AIA
# define _AAS_VST3_Q3_AIA           _ARM_VST3_Q3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _AAS_VLD3_D3_IA            _ARM_VLD3_D3_IA
# define _AAS_VLD3_D3_IA            _ARM_VLD3_D3_IA

# define _AAS_VST3_D3_IA            _ARM_VST3_D3_IA
# define _AAS_VST3_Q3_IA            _ARM_VST3_Q3_IA


/* ------------------------------------------------------------------------- */

	/*
	 * Load 4-element structures from memory (PRR: base address) into vector registers
	 * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
	 */

# define _AAS_VLD4_D4               _ARM_VLD4_D4
# define _AAS_VLD4_Q4               _ARM_VLD4_Q4

# define _AAS_VST4_D4               _ARM_VST4_D4
# define _AAS_VST4_Q4               _ARM_VST4_Q4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _AAS_VLD4_D4_AIA           _ARM_VLD4_D4_AIA
# define _AAS_VLD4_Q4_AIA           _ARM_VLD4_Q4_AIA

# define _AAS_VST4_D4_AIA           _ARM_VST4_D4_AIA
# define _AAS_VST4_Q4_AIA           _ARM_VST4_Q4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _AAS_VLD4_D4_IA            _ARM_VLD4_D4_IA
# define _AAS_VLD4_Q4_IA            _ARM_VLD4_Q4_IA

# define _AAS_VST4_D4_IA            _ARM_VST4_D4_IA
# define _AAS_VST4_Q4_IA            _ARM_VST4_Q4_IA

/* ========================================================================= */

/*
 * Load (with duplicate) single N-element structure to all lanes (into all elements of vector registers)
 */

    /*
     * Load 1-element structure from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1)
	 * (VRI0, ...): (Qn)
     */

# define _AAS_VLD1_DUP_D1           _ARM_VLD1_DUP_D1
# define _AAS_VLD1_DUP_D2           _ARM_VLD1_DUP_D2

# define _AAS_VLD1_DUP_Q1           _ARM_VLD1_DUP_Q1


		/*
		 * With auto-increment
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _AAS_VLD1_DUP_D1_AIA       _ARM_VLD1_DUP_D1_AIA
# define _AAS_VLD1_DUP_D2_AIA       _ARM_VLD1_DUP_D2_AIA
# define _AAS_VLD1_DUP_D3_AIA       _ARM_VLD1_DUP_D3_AIA
# define _AAS_VLD1_DUP_D4_AIA       _ARM_VLD1_DUP_D4_AIA

# define _AAS_VLD1_DUP_Q1_AIA       _ARM_VLD1_DUP_Q1_AIA
# define _AAS_VLD1_DUP_Q2_AIA       _ARM_VLD1_DUP_Q2_AIA
# define _AAS_VLD1_DUP_Q3_AIA       _ARM_VLD1_DUP_Q3_AIA
# define _AAS_VLD1_DUP_Q4_AIA       _ARM_VLD1_DUP_Q4_AIA


		/*
		 * With offset update
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1)
		 * (VRI0, ...): (Qn)
		 */

# define _AAS_VLD1_DUP_D1_IA        _ARM_VLD1_DUP_D1_IA
# define _AAS_VLD1_DUP_D2_IA        _ARM_VLD1_DUP_D2_IA

# define _AAS_VLD1_DUP_Q1_IA        _ARM_VLD1_DUP_Q1_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
     */

# define _AAS_VLD2_DUP_D2           _ARM_VLD2_DUP_D2

# define _AAS_VLD2_DUP_Q2           _ARM_VLD2_DUP_Q2


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _AAS_VLD2_DUP_D2_AIA       _ARM_VLD2_DUP_D2_AIA
# define _AAS_VLD2_DUP_D4_AIA       _ARM_VLD2_DUP_D4_AIA

# define _AAS_VLD2_DUP_Q2_AIA       _ARM_VLD2_DUP_Q2_AIA
# define _AAS_VLD2_DUP_Q4_AIA       _ARM_VLD2_DUP_Q4_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
		 */

# define _AAS_VLD2_DUP_D2_IA        _ARM_VLD2_DUP_D2_IA

# define _AAS_VLD2_DUP_Q2_IA        _ARM_VLD2_DUP_Q2_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _AAS_VLD3_DUP_D3           _ARM_VLD3_DUP_D3

# define _AAS_VLD3_DUP_Q3           _ARM_VLD3_DUP_Q3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _AAS_VLD3_DUP_D3_AIA       _ARM_VLD3_DUP_D3_AIA

# define _AAS_VLD3_DUP_Q3_AIA       _ARM_VLD3_DUP_Q3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _AAS_VLD3_DUP_D3_IA        _ARM_VLD3_DUP_D3_IA

# define _AAS_VLD3_DUP_Q3_IA        _ARM_VLD3_DUP_Q3_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
     */

# define _AAS_VLD4_DUP_D4           _ARM_VLD4_DUP_D4

# define _AAS_VLD4_DUP_Q4           _ARM_VLD4_DUP_Q4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _AAS_VLD4_DUP_D4_AIA       _ARM_VLD4_DUP_D4_AIA

# define _AAS_VLD4_DUP_Q4_AIA       _ARM_VLD4_DUP_Q4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _AAS_VLD4_DUP_D4_IA        _ARM_VLD4_DUP_D4_IA

# define _AAS_VLD4_DUP_Q4_IA        _ARM_VLD4_DUP_Q4_IA

/* ========================================================================= */

/*
 * Load/Store single N-element structure to/from one lane
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI): (Dn[L])
     */

# define _AAS_VLD1_LANE_D1          _ARM_VLD1_LANE_D1

# define _AAS_VST1_LANE_D1          _ARM_VST1_LANE_D1


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _AAS_VLD1_LANE_D1_AIA      _ARM_VLD1_LANE_D1_AIA

# define _AAS_VST1_LANE_D1_AIA      _ARM_VST1_LANE_D1_AIA


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _AAS_VLD1_LANE_D1_IA       _ARM_VLD1_LANE_D1_IA

# define _AAS_VST1_LANE_D1_IA       _ARM_VST1_LANE_D1_IA

/* @{ */
# define _AAS_VLD1_XLANE_D1         _ARM_VLD1_XLANE_D1
# define _AAS_VST1_XLANE_D1         _ARM_VST1_XLANE_D1
# define _AAS_VLD1_XLANE_D1_AIA     _ARM_VLD1_XLANE_D1_AIA
# define _AAS_VST1_XLANE_D1_AIA     _ARM_VST1_XLANE_D1_AIA
# define _AAS_VLD1_XLANE_D1_IA      _ARM_VLD1_XLANE_D1_IA
# define _AAS_VST1_XLANE_D1_IA      _ARM_VST1_XLANE_D1_IA
/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
     */

# define _AAS_VLD2_LANE_D2          _ARM_VLD2_LANE_D2

# define _AAS_VST2_LANE_D2          _ARM_VST2_LANE_D2


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _AAS_VLD2_LANE_D2_AIA      _ARM_VLD2_LANE_D2_AIA

# define _AAS_VST2_LANE_D2_AIA      _ARM_VST2_LANE_D2_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _AAS_VLD2_LANE_D2_IA       _ARM_VLD2_LANE_D2_IA

# define _AAS_VST2_LANE_D2_IA       _ARM_VST2_LANE_D2_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
     */

# define _AAS_VLD3_LANE_D3          _ARM_VLD3_LANE_D3

# define _AAS_VST3_LANE_D3          _ARM_VST3_LANE_D3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _AAS_VLD3_LANE_D3_AIA      _ARM_VLD3_LANE_D3_AIA

# define _AAS_VST3_LANE_D3_AIA      _ARM_VST3_LANE_D3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _AAS_VLD3_LANE_D3_IA       _ARM_VLD3_LANE_D3_IA

# define _AAS_VST3_LANE_D3_IA       _ARM_VST3_LANE_D3_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
     * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
     */

# define _AAS_VLD4_LANE_D4          _ARM_VLD4_LANE_D4

# define _AAS_VST4_LANE_D4          _ARM_VST4_LANE_D4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _AAS_VLD4_LANE_D4_AIA      _ARM_VLD4_LANE_D4_AIA

# define _AAS_VST4_LANE_D4_AIA      _ARM_VST4_LANE_D4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _AAS_VLD4_LANE_D4_IA       _ARM_VLD4_LANE_D4_IA

# define _AAS_VST4_LANE_D4_IA       _ARM_VST4_LANE_D4_IA

/* ========================================================================= */

/*
 * Store multiple N-element structures
 *
 * !!!NOTE!!!
 * Count of structures used is less then vector register size.
 * For example: only 5 of 8 1-element structures have to be stored.
 */

/*
 * 1) Store N 1-element structures to memory (PRR: base address) from vector register.
 * 2) Store N 1-element structures to memory (PRR: base address) from vector register, 0 ~ 8(16,32,64).
 *
 * Variant 1)
 * Vector size    Datatype     N
 * (D) 8           int-8     [1-7]
 * (D) 8           int-16    [1-3]
 * (D) 8           int-32    [1-1]
 * (Q) 16          int-8     [1-15]
 * (Q) 16          int-16    [1-7]
 * (Q) 16          int-32    [1-3]
 * (.) 32          int-8     [1-31]
 * (.) 32          int-16    [1-15]
 * (.) 32          int-32    [1-7]
 *
 * Variant 2)
 * Vector size    Datatype     N
 * (D) 8           int-8     [0-7]  --> [ 8,1-7]
 * (D) 8           int-16    [0-3]  --> [ 4,1-3]
 * (D) 8           int-32    [0-1]  --> [ 2,1-1]
 * (Q) 16          int-8     [0-15] --> [16,1-15]
 * (Q) 16          int-16    [0-7]  --> [ 8,1-7]
 * (Q) 16          int-32    [0-3]  --> [ 4,1-3]
 * (.) 32          int-8     [0-31] --> [32,1-31]
 * (.) 32          int-16    [0-15] --> [16,1-15]
 * (.) 32          int-32    [0-7]  --> [ 8,1-7]
 */

	/*
	 * 1-element structures
	 */

/* NRR in elements */
/* NRR changed */
# define _AAS_VST1_N_D1_AIA         _ARM_VST1_N_D1_AIA
# define _AAS_VST1_N_Q1_AIA         _ARM_VST1_N_Q1_AIA
# define _AAS_VST1_N_Q2_AIA         _ARM_VST1_N_Q2_AIA

/* NRR is not changed */
# define _AAS_VST1_NC0_D1_AIA       _ARM_VST1_NC0_D1_AIA
# define _AAS_VST1_NC0_Q1_AIA       _ARM_VST1_NC0_Q1_AIA
/* NRR changed */
# define _AAS_VST1_NC00_D1_AIA      _ARM_VST1_NC00_D1_AIA
# define _AAS_VST1_NC00_Q1_AIA      _ARM_VST1_NC00_Q1_AIA


/* NRR in bytes */
/* NRR is not changed */
# define _AAS_VST1_NC0B_D1_AIA      _ARM_VST1_NC0B_D1_AIA
# define _AAS_VST1_NC0B_Q1_AIA      _ARM_VST1_NC0B_Q1_AIA
# define _AAS_VST1_NC0B_Q2_AIA      _ARM_VST1_NC0B_Q2_AIA
# define _AAS_VST1_NC0B_Q4_AIA      _ARM_VST1_NC0B_Q4_AIA

/* ------------------------------------------------------------------------- */

	/*
	 * 2-element structures
	 */

/* NRR in elements */
/* NRR changed */
# define _AAS_VST2_N_D2_AIA         _ARM_VST2_N_D2_AIA
# define _AAS_VST2_N_Q2_AIA         _ARM_VST2_N_Q2_AIA

/* NRR is not changed */
# define _AAS_VST2_NC0_D2_AIA       _ARM_VST2_NC0_D2_AIA
# define _AAS_VST2_NC0_Q2_AIA       _ARM_VST2_NC0_Q2_AIA
/* NRR changed */
# define _AAS_VST2_NC00_D2_AIA      _ARM_VST2_NC00_D2_AIA
# define _AAS_VST2_NC00_Q2_AIA      _ARM_VST2_NC00_Q2_AIA


/* NRR in bytes */
/* NRR is not changed */
# define _AAS_VST2_NC0B_D2_AIA      _ARM_VST2_NC0B_D2_AIA
# define _AAS_VST2_NC0B_Q2_AIA      _ARM_VST2_NC0B_Q2_AIA

/* ------------------------------------------------------------------------- */

	/*
	 * 3-element structures
	 */

/* NRR in elements */
/* NRR changed */
# define _AAS_VST3_N_D3_AIA         _ARM_VST3_N_D3_AIA
# define _AAS_VST3_N_Q3_AIA         _ARM_VST3_N_Q3_AIA

/* NRR is not changed */
# define _AAS_VST3_NC0_D3_AIA       _ARM_VST3_NC0_D3_AIA
# define _AAS_VST3_NC0_Q3_AIA       _ARM_VST3_NC0_Q3_AIA
/* NRR changed */
# define _AAS_VST3_NC00_D3_AIA      _ARM_VST3_NC00_D3_AIA
# define _AAS_VST3_NC00_Q3_AIA      _ARM_VST3_NC00_Q3_AIA


/* NRR in bytes */
/* NRR is not changed */
# define _AAS_VST3_NC0B_D3_AIA      _ARM_VST3_NC0B_D3_AIA
# define _AAS_VST3_NC0B_Q3_AIA      _ARM_VST3_NC0B_Q3_AIA

/* ------------------------------------------------------------------------- */

	/*
	 * 4-element structures
	 */

/* NRR in elements */
/* NRR changed */
# define _AAS_VST4_N_D4_AIA         _ARM_VST4_N_D4_AIA
# define _AAS_VST4_N_Q4_AIA         _ARM_VST4_N_Q4_AIA

/* NRR is not changed */
# define _AAS_VST4_NC0_D4_AIA       _ARM_VST4_NC0_D4_AIA
# define _AAS_VST4_NC0_Q4_AIA       _ARM_VST4_NC0_Q4_AIA
/* NRR changed */
# define _AAS_VST4_NC00_D4_AIA      _ARM_VST4_NC00_D4_AIA
# define _AAS_VST4_NC00_Q4_AIA      _ARM_VST4_NC00_Q4_AIA


/* NRR in bytes */
/* NRR is not changed */
# define _AAS_VST4_NC0B_D4_AIA      _ARM_VST4_NC0B_D4_AIA
# define _AAS_VST4_NC0B_Q4_AIA      _ARM_VST4_NC0B_Q4_AIA

/* ========================================================================= */

/*
 * Store narrow (truncate) multiple N-element structures
 */

        /*
         * Store only 1st byte from data-type
         */

# define _AAS_VST1U8_Q1             _ARM_VST1U8_Q1
# define _AAS_VST1U8_Q2             _ARM_VST1U8_Q2

# define _AAS_VST1U8_Q1_AIA         _ARM_VST1U8_Q1_AIA
# define _AAS_VST1U8_Q2_AIA         _ARM_VST1U8_Q2_AIA
# define _AAS_VST1U8_Q3_AIA         _ARM_VST1U8_Q3_AIA
# define _AAS_VST1U8_Q4_AIA         _ARM_VST1U8_Q4_AIA

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define _AAS_VPRINT_LANE_D1        _ARM_VPRINT_LANE_D1
# define _AAS_VPRINTF32_LANE_D1     _ARM_VPRINTF32_LANE_D1

# define __AAS_VPRINT_LANE_D1       __ARM_VPRINT_LANE_D1
# define __AAS_VPRINTF32_LANE_D1    __ARM_VPRINTF32_LANE_D1

/* @} */
/* @} */

#endif /* !defined(__VIPM_ARM_AAS__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
