/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2007 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Types.h	(N.Panasiuk)
 * include/Vodi/Types.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VODI_TYPES__
# define __VODI_TYPES__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VODI_PARAM__
#  include <Vodi/Param.h>
# endif

# ifndef __VODI_CLRSTYPE__
#  include <Vodi/clrsType.h>
# endif

# ifndef __VODI_DEVTYPE__
#  include <Vodi/devType.h>
# endif

# ifndef __BO_TYPES__
#  include <Bo/Types.h>
# endif

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

/*
 * Common useful types
 */

typedef int                        vodi_bool_t;

struct vodi_point {
	int pi_x, pi_y;
};
typedef struct vodi_point          vodi_point_t;
# define vodi_point_p(x)          ((struct vodi_point *)(x))

struct vodi_size {
	int sz_width, sz_height;
};
typedef struct vodi_size           vodi_size_t;
# define vodi_size_p(x)           ((struct vodi_size *)(x))

struct vodi_rect {
	int rc_left, rc_top,
	    rc_right, rc_bottom;
};
typedef struct vodi_rect           vodi_rect_t;


typedef struct { float a[8]; }     vodi_quad32f_t;
# define vodi_quad32f_p(x)         ((float (*)[2])(x))
# define VODI_QUAD32F_INITIALIZER(x0, y0, x1, y1, x2, y2, x3, y3) \
	{{ (x0), (y0), (x1), (y1), (x2), (y2), (x3), (y3) }}

__myc_extern_inline__ vodi_quad32f_t *
VodiQUAD32Finit(
	vodi_quad32f_t *dp,
	float x0, float y0,
	float x1, float y1,
	float x2, float y2,
	float x3, float y3
	)
{
	float (*q)[2] = vodi_quad32f_p(dp);
	q[0][0] = (x0); q[0][1] = (y0);
	q[1][0] = (x1); q[1][1] = (y1);
	q[2][0] = (x2); q[2][1] = (y2);
	q[3][0] = (x3); q[3][1] = (y3);
	return (dp);
}

typedef struct { double a[8]; }    vodi_quad64f_t;
# define vodi_quad64f_p(x)         ((double (*)[2])(x))
# define VODI_QUAD64F_INITIALIZER(x0, y0, x1, y1, x2, y2, x3, y3) \
	{{ (x0), (y0), (x1), (y1), (x2), (y2), (x3), (y3) }}

__myc_extern_inline__ vodi_quad64f_t *
VodiQUAD64Finit(
	vodi_quad64f_t *dp,
	double x0, double y0,
	double x1, double y1,
	double x2, double y2,
	double x3, double y3
	)
{
	double (*q)[2] = vodi_quad64f_p(dp);
	q[0][0] = (x0); q[0][1] = (y0);
	q[1][0] = (x1); q[1][1] = (y1);
	q[2][0] = (x2); q[2][1] = (y2);
	q[3][0] = (x3); q[3][1] = (y3);
	return (dp);
}


struct vodi_scale_factor {
	double  sc_width;
	double  sc_height;
};
typedef struct vodi_scale_factor   vodi_scale_factor_t;

struct vodi_scale_parm {
	int scp_kind;
# define VodiK_SCALE_NULL_PARM     0
# define VodiK_SCALE_FACTOR_PARM   1
# define VodiK_SCALE_SIZE_PARM     2

	union {
		struct vodi_scale_factor  __scp_factor;
		struct vodi_size          __scp_size;
	} __u;
# define scp_factor __u.__scp_factor
# define scp_size   __u.__scp_size
};
typedef struct vodi_scale_parm     vodi_scale_parm_t;


struct vodi_box2d {
	vodi_point_t bx_center;
	vodi_size_t  bx_size;
	float        bx_angle;
};
typedef struct vodi_box2d          vodi_box2d_t;

struct vodi_strel_shape {
	vodi_point_t sel_anchor; /* relative position of filtered point within the strel */
	vodi_size_t  sel_size;
};
typedef struct vodi_strel_shape    vodi_strel_shape_t;
/* @todo: remove */
//# define vodi_sebox                vodi_strel_shape
//# define vodi_sebox_t              vodi_strel_shape_t


struct vodi_border_size {
	int  br_lw, br_th;
	int  br_rw, br_bh; /* -1: .br_rw == .br_lw, .br_bh == .br_th */
};
typedef struct vodi_border_size    vodi_border_size_t;
# define vodi_border_size_p(x)    ((struct vodi_border_size *)(x))

struct vodi_border_parm {
	int    br_lw, br_th;
	int    br_rw, br_bh; /* -1: .br_rw == .br_lw, .br_bh == .br_th */
	float  br_cval[4];
};
typedef struct vodi_border_parm    vodi_border_parm_t;
# define vodi_border_parm_p(x)    ((struct vodi_border_parm *)(x))

# define VODI_BORDER_SIZE_LeftWidth(x)    (vodi_border_size_p(x)->br_lw)
# define VODI_BORDER_SIZE_TopHeight(x)    (vodi_border_size_p(x)->br_th)
# define VODI_BORDER_SIZE_RightWidth(x) \
    (vodi_border_size_p(x)->br_rw < 0 ? vodi_border_size_p(x)->br_lw : vodi_border_size_p(x)->br_rw)
# define VODI_BORDER_SIZE_BottomHeight(x) \
    (vodi_border_size_p(x)->br_bh < 0 ? vodi_border_size_p(x)->br_th : vodi_border_size_p(x)->br_bh)

/* ============================================================================
 * COMPARE OPERATION
 */

typedef bo_mismatch_t
(__myc_tcalltype__(mcall) *vodi_compare3_fn)(
# if __myc_require(prototypes)
	bo_const_pointer_t aLhs,
	bo_const_pointer_t aRhs,
	bo_pointer_t       aCtx
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

struct vodi_comparator {
	vodi_compare3_fn  cmp;
};
# define vodi_comparator_p(x) ((struct vodi_comparator *)(x))

# define VODI_Compare(aComparator, aLhs, aRhs) \
	((*vodi_comparator_p(aComparator)->cmp)( \
			aLhs, aRhs, (bo_pointer_t)(aComparator) \
		))

/* ============================================================================
 * MOTION DEFINITIONS
 */

/* Direction constants */
enum vodi_motiondir {
	VodiK_INVAL_MDIR = -1,
	VodiK_NULL_MDIR  =  0,
	VodiK_UNK_MDIR   =  2,

	VodiK_NORTH_MDIR,
	VodiK_NORTH_EAST_MDIR,
	VodiK_EAST_MDIR,
	VodiK_SOUTH_EAST_MDIR,
	VodiK_SOUTH_MDIR,
	VodiK_SOUTH_WEST_MDIR,
	VodiK_WEST_MDIR,
	VodiK_NORTH_WEST_MDIR,

	VodiK_QWERTY_MDIR
};

# if __myc_option(enum_always_int)
typedef enum vodi_motiondir vodi_motiondir_t;
# else
typedef int vodi_motiondir_t;
# endif


# define VodiF_RESULT_DUP           0x01  /* duplicate object */
# define VodiF_RESULT_LOST          0x02  /* lost object */
# define VodiF_RESULT_INVALID       0x04  /* invalid object */
# define VodiF_RESULT_INVALID_LOST  0x08  /* invalid distributed lost object
                                             - result was good when detected
                                             and is bad when lost */

# define VodiF_RESULT_INVERSED      0x10
# define VodiF_RESULT_MOVEIN        0x20

/* NOT FOR YOUR BUSINESS */
# define _VodiF_RESULT_DISTRIBUTED  0x10  /* distributed object */


/* ============================================================================
 * IMAGE DEFINITIONS
 */

/*
 * Origin types
 */
# define _VodiK_UNK_ORIGIN         2 /* unknown origin */
# define _VodiK_TL_ORIGIN          3 /* top-left origin */
# define _VodiK_BL_ORIGIN          4 /* bottom-left origin */


# define _VodiK_ALPHA_CHANNEL      0x8
# define _VodiK_CHANNELC_MASK      0x7

//# define _VODI_DECODE_CHANNELC(Channelc)       ((_VodiK_ALPHA_CHANNEL & (Channelc)) ? 4 : Channelc)
# define _VODI_DECODE_CHANNELC(Channelc)       (((Channelc) & _VodiK_CHANNELC_MASK) + !!(_VodiK_ALPHA_CHANNEL & (Channelc)))
# define _VODI_ENCODE_CHANNELC(Channelc,Alpha) ((Channelc) | ((Alpha) ? _VodiK_ALPHA_CHANNEL : 0))


/*
 * Elem types
 */

# define _VodiK_ELEMTYPE_U8        1
# define _VodiK_ELEMTYPE_I16       2
# define _VodiK_ELEMTYPE_I32       3
/* # define _VodiK_ELEMTYPE_OBSOLETE  4 */
# define _VodiK_ELEMTYPE_F32       5
# define _VodiK_ELEMTYPE_F64       6

# define _VodiK_ELEMTYPE_UCHAR     _VodiK_ELEMTYPE_U8
# define _VodiK_ELEMTYPE_SHORT     _VodiK_ELEMTYPE_I16
# define _VodiK_ELEMTYPE_INT       _VodiK_ELEMTYPE_I32
# define _VodiK_ELEMTYPE_FLOAT     _VodiK_ELEMTYPE_F32
# define _VodiK_ELEMTYPE_DOUBLE    _VodiK_ELEMTYPE_F64

/*
# define _VODI_ELEMTYPE_Alignof(ElemType)  ("0444048"[(ElemType)] - '0')
# define _VODI_ELEMTYPE_Sizeof(ElemType)   ("0124048"[(ElemType)] - '0')
 */

# define _VODI_ELEMTYPE_Alignof(ElemType)    \
    (  _VodiK_ELEMTYPE_U8  == (ElemType) ? 1 \
     : _VodiK_ELEMTYPE_I16 == (ElemType) ? 2 \
     : _VodiK_ELEMTYPE_I32 == (ElemType) ? 4 \
     : _VodiK_ELEMTYPE_F32 == (ElemType) ? 4 \
     : _VodiK_ELEMTYPE_F64 == (ElemType) ? 8 \
                                         : 0 )

# define _VODI_ELEMTYPE_Sizeof(ElemType)     \
    (  _VodiK_ELEMTYPE_U8  == (ElemType) ? 1 \
     : _VodiK_ELEMTYPE_I16 == (ElemType) ? 2 \
     : _VodiK_ELEMTYPE_I32 == (ElemType) ? 4 \
     : _VodiK_ELEMTYPE_F32 == (ElemType) ? 4 \
     : _VodiK_ELEMTYPE_F64 == (ElemType) ? 8 \
                                         : 0 )

# define _VODI_ELEMTYPE_ALIGNOF_C(ElemType) __myc_concat(_VODI_ELEMTYPE_Alignof_,ElemType)()
# define _VODI_ELEMTYPE_SIZEOF_C(ElemType)  __myc_concat(_VODI_ELEMTYPE_Sizeof_,ElemType)()
/* @{ */
# define _VODI_ELEMTYPE_Alignof_U8()       1 // 4
# define _VODI_ELEMTYPE_Alignof_I16()      2 // 4
# define _VODI_ELEMTYPE_Alignof_I32()      4
# define _VODI_ELEMTYPE_Alignof_F32()      4 /* sizeof(float) */
# define _VODI_ELEMTYPE_Alignof_F64()      8 /* sizeof(double) */
# define _VODI_ELEMTYPE_Alignof_1()        _VODI_ELEMTYPE_Alignof_U8
# define _VODI_ELEMTYPE_Alignof_2()        _VODI_ELEMTYPE_Alignof_I16
# define _VODI_ELEMTYPE_Alignof_3()        _VODI_ELEMTYPE_Alignof_I32
# define _VODI_ELEMTYPE_Alignof_4()        sizeof(long)
# define _VODI_ELEMTYPE_Alignof_5()        _VODI_ELEMTYPE_Alignof_F32
# define _VODI_ELEMTYPE_Alignof_6()        _VODI_ELEMTYPE_Alignof_F64

# define _VODI_ELEMTYPE_Sizeof_U8()        1
# define _VODI_ELEMTYPE_Sizeof_I16()       2
# define _VODI_ELEMTYPE_Sizeof_I32()       4
# define _VODI_ELEMTYPE_Sizeof_F32()       4 /* sizeof(float) */
# define _VODI_ELEMTYPE_Sizeof_F64()       8 /* sizeof(double) */
# define _VODI_ELEMTYPE_Sizeof_1()         _VODI_ELEMTYPE_Sizeof_U8
# define _VODI_ELEMTYPE_Sizeof_2()         _VODI_ELEMTYPE_Sizeof_I16
# define _VODI_ELEMTYPE_Sizeof_3()         _VODI_ELEMTYPE_Sizeof_I32
# define _VODI_ELEMTYPE_Sizeof_4()         sizeof(long)
# define _VODI_ELEMTYPE_Sizeof_5()         _VODI_ELEMTYPE_Sizeof_F32
# define _VODI_ELEMTYPE_Sizeof_6()         _VODI_ELEMTYPE_Sizeof_F64
/* @} */

# define _VODI_ELEMSTRIDE(Elemsz,Channelc)     __VODI_ELEMSTRIDE(Elemsz, _VODI_DECODE_CHANNELC(Channelc))
# define _VODI_ELEMSTRIDE2(ElemType,Channelc)  __VODI_ELEMSTRIDE2(ElemType, _VODI_DECODE_CHANNELC(Channelc))

# define __VODI_ELEMSTRIDE(Elemsz,Channelc)    ((Elemsz) * (Channelc))
# define __VODI_ELEMSTRIDE2(ElemType,Channelc) (_VODI_ELEMTYPE_Sizeof(ElemType) * (Channelc))


//
// @todo: remove
//

/* Flags useful for vodi_image.img_flags, vodi_grabimg.gim_flags */
# define VodiF_IMAGE_RAW_DATA       0
# define VodiF_IMAGE_IPL_IMAGE      0x01
# define VodiF_IMAGE_PLAIN          0
# define VodiF_IMAGE_ZALLOC         0x02
# define VodiF_IMAGE_RETAINABLE     0x04

struct vodi_image {
	unsigned      img_flags;

	long          img_width;  /* in pixels */
	long          img_height; /* in pixels */
	unsigned      img_bpp;    /* bits per pixel */
	bo_pointer_t  img_base;
};
typedef struct vodi_image *vodi_image_t;

# define VODI_IMAGE_WStride(anImage) __myc_align((anImage)->img_width, 4)

struct vodi_grabimg {
	unsigned                gim_flags;

	u_int32_t               gim_format;
	bo_utime_t              gim_time;
	int32_t                 gim_clock;

	unsigned                gim_bpp;    /* bits per pixel */
	size_t                  gim_datasz; /* in bytes */
	bo_pointer_t            gim_data;

	vodi_image_t            gim_image;
};
typedef struct vodi_grabimg *vodi_grabimg_t;


/* ============================================================================
 * USER-CONTEXT DEFINITIONS
 */

/*
 * Callback declarations
 */

struct vodi_ucontext;

typedef bo_status_t
(__myc_tcalltype__(mcall) *vodi_uctx_dup_fn)(
# if __myc_require(prototypes)
	struct vodi_ucontext const *aSource,
	struct vodi_ucontext       *aDest,
	struct aorp_error          *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

typedef bo_status_t
(__myc_tcalltype__(mcall) *vodi_uctx_change_fn)(
# if __myc_require(prototypes)
	struct vodi_ucontext *aSource,
	struct vodi_ucontext *aDest,
	struct aorp_error    *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype, nonnull(1,2)));

typedef ssize_t
(__myc_tcalltype__(mcall) *vodi_udata_retain_fn)(
# if __myc_require(prototypes)
	bo_pointer_t       aData,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype));

typedef ssize_t
(__myc_tcalltype__(mcall) *vodi_udata_release_fn)(
# if __myc_require(prototypes)
	bo_pointer_t       aData,
	bo_opflags_t       Flags /* = 0 */,
	struct aorp_error *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype));

/*
 * USER-CONTEXT declaration
 */

struct vodi_ucontext {
	vodi_uctx_dup_fn       uctx_dup;
	vodi_udata_retain_fn   uctx_dretain;
	vodi_udata_release_fn  uctx_drelease;
	vodi_uctx_change_fn    uctx_change;
	bo_pointer_t           uctx_udata;
};
typedef struct vodi_ucontext vodi_ucontext_t;

# define VODI_UCONTEXT_Dup(aSource,aDest,anErrPtr) \
	((NULLF != (aSource)->uctx_dup) \
			? ((*(aSource)->uctx_dup)((aSource),(aDest),anErrPtr)) \
			: BoS_NOERR)

# define VODI_UCONTEXT_DChange(aSource,aDest,anErrPtr) \
	((NULLF != (aSource)->uctx_change) \
		? ((*(aSource)->uctx_change)((aSource),(aDest),anErrPtr)) \
		: BoS_NOERR)

# define VODI_UCONTEXT_DRetain(anUserCtx,anErrPtr) \
	((NULLF != (anUserCtx)->uctx_dretain && NULL != (anUserCtx)->uctx_udata) \
			? ((*(anUserCtx)->uctx_dretain)((anUserCtx)->uctx_udata,anErrPtr)) \
			: 0)

# if 0
#  define VODI_UCONTEXT_DRelease(anUserCtx,Flags,anErrPtr) \
	((NULLF != (anUserCtx)->uctx_drelease && NULL != (anUserCtx)->uctx_udata) \
			? (((*(anUserCtx)->uctx_drelease)((anUserCtx)->uctx_udata,Flags,anErrPtr)), \
					(anUserCtx)->uctx_udata = NULL, 0) \
			: 0)
# else
#  define VODI_UCONTEXT_DRelease(anUserCtx,Flags,anErrPtr) \
	((NULLF != (anUserCtx)->uctx_drelease && NULL != (anUserCtx)->uctx_udata) \
			? ((*(anUserCtx)->uctx_drelease)((anUserCtx)->uctx_udata,Flags,anErrPtr)) \
			: 0)
# endif


/* ============================================================================
 * RESULT-INFO DEFINITIONS
 */

struct vodi_result_info;

/* Result-info constants */
enum {
	VodiK_INVAL_RESULT_INFO = -1,
	VodiK_NULL_RESULT_INFO = 0,
	VodiK_UNK_RESULT_INFO = 2,

	VodiK_KVPW_RESULT_INFO,                /* NOT FOR YOUR BUSINESS */
	VodiK_VPW_RESULT_INFO,

	VodiK_KFCW_RESULT_INFO,                /* NOT FOR YOUR BUSINESS */
	VodiK_FCW_RESULT_INFO,

	VodiK_LAST_RESULT_INFO
};

/*
 * RESULT-INFO declaration
 */

struct vodi_result_info {
# ifdef __cplusplus
	struct vodi_result_info_header;
# endif

	struct vodi_result_info_header {
		int  ri_type;
		int  ri_pad;
	} _h;

/*
	specific for type info
 */
};
# define vodi_result_info_p(p) ((struct vodi_result_info *)(p))

# ifdef __cplusplus
#  define vodi_result_info_header vodi_result_info::vodi_result_info_header
# endif

# define VODI_RESULT_INFO_TYPE(p) (vodi_result_info_p(p)->_h.ri_type)
# define VODI_RESULT_INFO_SPEC(p,_SpecType) \
	((struct __myc_concat3(vodi_,_SpecType,_info_spec) *) ((char *)(p) + sizeof(struct vodi_result_info)))


/* ============================================================================
 * USER-CALLBACKS DEFINITIONS
 */

# define VODI_NULL_CALLBACK    (NULLF)
# define VODI_DEFAULT_CALLBACK ((void (*)())1)

typedef bo_status_t
(__myc_tcalltype__(mcall) *vodi_rinit_fn)(
# if __myc_require(prototypes)
	struct vodi_result_info *aResultInfo,
	bo_pointer_t             anUserCtx /* = NULL */,
	struct aorp_error       *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *vodi_rfini_fn)(
# if __myc_require(prototypes)
	struct vodi_result_info *aResultInfo,
	bo_pointer_t             anUserCtx /* = NULL */,
	struct aorp_error       *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype));

typedef double
(__myc_tcalltype__(mcall) *vodi_rcmp_fn)(
# if __myc_require(prototypes)
	struct vodi_result_info const *aLhs,
	struct vodi_result_info const *aRhs,
	bo_pointer_t                   anUserCtx /* = NULL */
# endif
	) __attribute__((mcalltype));

typedef bo_status_t
(__myc_tcalltype__(mcall) *vodi_rpoza_fn)(
# if __myc_require(prototypes)
	struct vodi_result_info *aSrcDst,
	struct vodi_result_info *aSrc,
	bo_pointer_t             anUserCtx /* = NULL */,
	struct aorp_error       *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype));


/* ============================================================================
 * @todo: doc
 */

struct vodi_clsfycb;

typedef double
(__myc_tcalltype__(mcall) *vodi_clsfy_relop_fn)(
# if __myc_require(prototypes)
	struct vodi_clsfycb   *aCB,
	aorp_object_t          aLhs,
	aorp_object_t          aRhs
# endif
	) __attribute__((mcalltype));

typedef double
(__myc_tcalltype__(mcall) *vodi_clsfy_chaos_fn)(
# if __myc_require(prototypes)
	struct vodi_clsfycb   *aCB,
	aorp_object_t          aRescls
# endif
	) __attribute__((mcalltype));

typedef void
(__myc_tcalltype__(mcall) *vodi_clsfy_adjust_fn)(
# if __myc_require(prototypes)
	struct vodi_clsfycb   *aCB,
	aorp_object_t          aResultClass,
	size_t                 aRescAdded,
#  if 0
	union {
		aorp_object_t *list;   /* aRescAdded > 0 */
		aorp_object_t  ens;    /* aRescAdded = -1 */
	}                      ResultsAdded
#  else
	void                  *ResultsAdded
#  endif
# endif
	) __attribute__((mcalltype));

typedef aorp_object_t
(__myc_tcalltype__(mcall) *vodi_clsfy_objopen_fn)(
# if __myc_require(prototypes)
	struct vodi_clsfycb   *aCB,
	struct aorp_error     *anErrPtr /* = NULL */
# endif
	) __attribute__((mcalltype));

struct vodi_clsfycb {
	vodi_clsfy_relop_fn      vccb_mira;
	vodi_clsfy_adjust_fn     vccb_adjust;
	vodi_clsfy_chaos_fn      vccb_chaos;          /* optional; [default: NULF]; */
	                                              /* calculate chaos of result-class */
	vodi_clsfy_objopen_fn    vccb_ensopen;        /* optional; [default: NULF] */
	vodi_clsfy_objopen_fn    vccb_clsopen;        /* optional; [default: NULF] */

	double                   vccb_milo,
	                         vccb_mihi;
	bo_usec_t                vccb_timradius;      /* us; <0 --> unused; =0 --> default */
	aorp_opflags_t           vccb_addflags;       /* [default: 0] */
};

# define VodiK_CLSFY_DFL_TIMRADIUS  (1000000)     /* 1 sec */

#endif /* !defined(__VODI_TYPES__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
