/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2007 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Kum.h	(N.Panasiuk)
 * include/Vodi/objects/Kum.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VODI_OBJECTS_KUM__
# define __VODI_OBJECTS_KUM__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Vodi/Types.h>
# include <Bo/memman/Types.h>
# include <Bo/objects/Bokumfam.h>
# include <Bo/libc/Limits.h>
# include <Vodi/services/Ucntl.h>

/******************************************************************************
* %% BeginSection: function prototypes
*/
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vodi/VodiLinkage.h>

__MYC_BEGIN_CDECLS

/*
	AorpBool   :: vodi_bool_t -> aorp_object_t
	AorpBoolOf :: aorp_object_t -> vodi_bool_t
	AorpBoolOf_rls :: aorp_object_t -> vodi_bool_t

	AorpInt   :: int -> aorp_object_t
	AorpIntOf :: aorp_object_t -> int
	AorpIntOf_rls :: aorp_object_t -> int

	AorpUInt   :: unsigned -> aorp_object_t
	AorpUIntOf :: aorp_object_t -> unsigned
	AorpUIntOf_rls :: aorp_object_t -> unsigned

	AorpLong   :: long -> aorp_object_t
	AorpLongOf :: aorp_object_t -> long
	AorpLongOf_rls :: aorp_object_t -> long

	AorpULong   :: long -> aorp_object_t
	AorpULongOf :: aorp_object_t -> long
	AorpULongOf_rls :: aorp_object_t -> long

	AorpI32   :: int32_t -> aorp_object_t
	AorpI32Of :: aorp_object_t -> int32_t
	AorpI32Of_rls :: aorp_object_t -> int32_t

	AorpU32   :: u_int32_t -> aorp_object_t
	AorpU32Of :: aorp_object_t -> u_int32_t
	AorpU32Of_rls :: aorp_object_t -> u_int32_t

	AorpI64   :: int64_t -> aorp_object_t
	AorpI64Of :: aorp_object_t -> int64_t
	AorpI64Of_rls :: aorp_object_t -> int64_t

	AorpU64   :: u_int64_t -> aorp_object_t
	AorpU64Of :: aorp_object_t -> u_int64_t
	AorpU64Of_rls :: aorp_object_t -> u_int64_t

	AorpF32, AorpFloat      :: float -> aorp_object_t
	AorpF32Of, AorpFloatOf  :: aorp_object_t -> float
	AorpF32Of_rls, AorpFloatOf_rls :: aorp_object_t -> float

	AorpF64, AorpDouble     :: double -> aorp_object_t
	AorpF64Of, AorpDoubleOf :: aorp_object_t -> double
	AorpF64Of_rls, AorpDoubleOf_rls :: aorp_object_t -> double

	AorpTup   :: size_t -> aorp_object_t -> ... -> (aorp_object_t, ...)
	AorpNTup  :: [aorp_object_t] -> size_t -> (aorp_object_t, ...)
	AorpTupAt :: (aorp_object_t, ...) -> bo_index_t -> aorp_object_t

	AorpPair :: aorp_object_t -> aorp_object_t -> (aorp_object_t, aorp_object_t)
	AorpFst, AorpSnd :: (aorp_object_t, aorp_object_t) -> aorp_object_t

	AorpKum   :: bo_pointer_t -> (bo_pointer_t -> void) -> aorp_object_t
	AorpKumOf :: aorp_object_t -> bo_pointer_t

	AorpImage   :: vodi_image_t -> aorp_object_t
	AorpImageOf :: aorp_object_t -> vodi_image_t

	AorpZlfun   :: struct zlfun * -> aorp_object_t
	AorpZlfunOf :: aorp_object_t -> struct zlfun *

	AorpRect    :: int -> int -> int -> int -> aorp_object_t
	AorpVRect   :: vodi_rect_t const * -> aorp_object_t
	AorpRectOf :: vodi_rect_t * -> aorp_object_t -> vodi_rect_t *
	AorpRectOf_rls :: vodi_rect_t * -> aorp_object_t -> vodi_rect_t *
 */

/* ------------------------------------------------------------------------- */
# pragma mark --- boolean ---
/* ------------------------------------------------------------------------- */

# define AorpBool(x) AorpI32((int32_t)!!(x))
# define AorpBoolOf(x) (!!AorpI32Of(x))
# define AorpBoolOf_rls(x) (!!AorpI32Of_rls(x))

/* ------------------------------------------------------------------------- */
# pragma mark --- int ---
/* ------------------------------------------------------------------------- */

/* @todo: redesign */
# if (UINT_MAX == 0xffffffff)
#  define AorpInt(x) AorpI32(x)
#  define AorpIntOf(x) AorpI32Of(x)
#  define AorpIntOf_rls(x) AorpI32Of_rls(x)
# elif (UINT_MAX == 0xffffffffffffffffUL)
#  define AorpInt(x) AorpI64(x)
#  define AorpIntOf(x) AorpI64Of(x)
#  define AorpIntOf_rls(x) AorpI64Of_rls(x)
# else
#  error Unsupported machine architecture!!!
# endif

/* ------------------------------------------------------------------------- */
# pragma mark --- unsigned int ---
/* ------------------------------------------------------------------------- */

# define AorpUInt(x) AorpInt((int)(x))
# define AorpUIntOf(x) ((unsigned)AorpIntOf(x))
# define AorpUIntOf_rls(x) ((unsigned)AorpIntOf_rls(x))

/* ------------------------------------------------------------------------- */
# pragma mark --- long ---
/* ------------------------------------------------------------------------- */

# if (ULONG_MAX == 0xffffffff)
#  define AorpLong(x) AorpI32(x)
#  define AorpLongOf(x) AorpI32Of(x)
#  define AorpLongOf_rls(x) AorpI32Of_rls(x)
# elif (ULONG_MAX == 0xffffffffffffffffUL)
#  define AorpLong(x) AorpI64(x)
#  define AorpLongOf(x) AorpI64Of(x)
#  define AorpLongOf_rls(x) AorpI64Of_rls(x)
# else
#  error Unsupported machine architecture!!!
# endif

/* ------------------------------------------------------------------------- */
# pragma mark --- unsigned long ---
/* ------------------------------------------------------------------------- */

# define AorpULong(x) AorpLong((long)(x))
# define AorpULongOf(x) ((unsigned long)AorpLongOf(x))
# define AorpULongOf_rls(x) ((unsigned long)AorpLongOf_rls(x))

/* ------------------------------------------------------------------------- */
# pragma mark --- int32_t ---
/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpI32(x) \
	__extension__ ({ \
		aorp_object_t ___retval; \
		int32_t ___x = (x); \
		BokumfamOpen_i32(___retval, ___x, NULL); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ aorp_object_t AorpI32(int32_t x)
{
	aorp_object_t retval;
	BokumfamOpen_i32(retval, x, NULL);
	return (retval);
}
# endif

# if __myc_support(macro_retval)
#  define AorpI32Of(x) \
	__extension__ ({ \
		struct aorp_custom_i32 *___custom = (struct aorp_custom_i32 *)(x); \
		/* return */ (___custom->aco_data); \
	})
# else
__myc_extern_inline__ int32_t AorpI32Of(aorp_object_t x)
{
	struct aorp_custom_i32 *custom = (struct aorp_custom_i32 *)x;
	return (custom->aco_data);
}
# endif

# if __myc_support(macro_retval)
#  define AorpI32Of_rls(x) \
	__extension__ ({ \
		aorp_object_t ___x = (x); \
		struct aorp_custom_i32 *___custom = (struct aorp_custom_i32 *)___x; \
		int32_t ___retval = ___custom->aco_data; \
		AorpRls(___x); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ int32_t AorpI32Of_rls(aorp_object_t x)
{
	struct aorp_custom_i32 *custom = (struct aorp_custom_i32 *)x;
	int32_t retval = custom->aco_data;
	AorpRls(x);
	return (retval);
}
# endif

/* ------------------------------------------------------------------------- */
# pragma mark --- u_int32_t ---
/* ------------------------------------------------------------------------- */

# define AorpU32(x) AorpI32((int32_t)(x))
# define AorpU32Of(x) ((u_int32_t)AorpI32Of(x))
# define AorpU32Of_rls(x) ((u_int32_t)AorpI32Of_rls(x))

/* ------------------------------------------------------------------------- */
# pragma mark --- int64_t ---
/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpI64(x) \
	__extension__ ({ \
		aorp_object_t ___retval; \
		int64_t ___x = (x); \
		BokumfamOpen_i64(___retval, ___x, NULL); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ aorp_object_t AorpI64(int64_t x)
{
	aorp_object_t retval;
	BokumfamOpen_i64(retval, x, NULL);
	return (retval);
}
# endif

# if __myc_support(macro_retval)
#  define AorpI64Of(x) \
	__extension__ ({ \
		struct aorp_custom_i64 *___custom = (struct aorp_custom_i64 *)(x); \
		/* return */ (___custom->aco_data); \
	})
# else
__myc_extern_inline__ int64_t AorpI64Of(aorp_object_t x)
{
	struct aorp_custom_i64 *custom = (struct aorp_custom_i64 *)x;
	return (custom->aco_data);
}
# endif

# if __myc_support(macro_retval)
#  define AorpI64Of_rls(x) \
	__extension__ ({ \
		aorp_object_t ___x = (x); \
		struct aorp_custom_i64 *___custom = (struct aorp_custom_i64 *)___x; \
		int64_t ___retval = ___custom->aco_data; \
		AorpRls(___x); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ int64_t AorpI64Of_rls(aorp_object_t x)
{
	struct aorp_custom_i64 *custom = (struct aorp_custom_i64 *)x;
	int64_t retval = custom->aco_data;
	AorpRls(x);
	return (retval);
}
# endif

/* ------------------------------------------------------------------------- */
# pragma mark --- u_int64_t ---
/* ------------------------------------------------------------------------- */

# define AorpU64(x) AorpI64((int64_t)(x))
# define AorpU64Of(x) ((u_int64_t)AorpI64Of(x))
# define AorpU64Of_rls(x) ((u_int64_t)AorpI64Of_rls(x))

/* ------------------------------------------------------------------------- */
# pragma mark --- float ---
/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpF32(x) \
	__extension__ ({ \
		aorp_object_t ___retval; \
		float __x = (x); \
		BokumfamOpen_f32(___retval, __x, NULL); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ aorp_object_t AorpF32(float x)
{
	aorp_object_t retval;
	BokumfamOpen_f32(retval, x, NULL);
	return (retval);
}
# endif

# if __myc_support(macro_retval)
#  define AorpF32Of(x) \
	__extension__ ({ \
		struct aorp_custom_f32 *___custom = (struct aorp_custom_f32 *)(x); \
		/* return */ (___custom->aco_data); \
	})
# else
__myc_extern_inline__ float AorpF32Of(aorp_object_t x)
{
	struct aorp_custom_f32 *custom = (struct aorp_custom_f32 *)x;
	return (custom->aco_data);
}
# endif

# if __myc_support(macro_retval)
#  define AorpF32Of_rls(x) \
	__extension__ ({ \
		aorp_object_t ___x = (x); \
		struct aorp_custom_f32 *___custom = (struct aorp_custom_f32 *)___x; \
		float ___retval = ___custom->aco_data; \
		AorpRls(___x); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ float AorpF32Of_rls(aorp_object_t x)
{
	struct aorp_custom_f32 *custom = (struct aorp_custom_f32 *)x;
	float retval = custom->aco_data;
	AorpRls(x);
	return (retval);
}
# endif

# define AorpFloat(x)   AorpF32(x)
# define AorpFloatOf(x) AorpF32Of(x)
# define AorpFloatOf_rls(x) AorpF32Of_rls(x)

/* ------------------------------------------------------------------------- */
# pragma mark --- double ---
/* ------------------------------------------------------------------------- */

# if __myc_support(macro_retval)
#  define AorpF64(x) \
	__extension__ ({ \
		aorp_object_t ___retval; \
		double __x = (x); \
		BokumfamOpen_f64(___retval, __x, NULL); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ aorp_object_t AorpF64(double x)
{
	aorp_object_t retval;
	BokumfamOpen_f64(retval, x, NULL);
	return (retval);
}
# endif

# if __myc_support(macro_retval)
#  define AorpF64Of(x) \
	__extension__ ({ \
		struct aorp_custom_f64 *___custom = (struct aorp_custom_f64 *)(x); \
		/* return */ (___custom->aco_data); \
	})
# else
__myc_extern_inline__ double AorpF64Of(aorp_object_t x)
{
	struct aorp_custom_f64 *custom = (struct aorp_custom_f64 *)x;
	return (custom->aco_data);
}
# endif

# if __myc_support(macro_retval)
#  define AorpF64Of_rls(x) \
	__extension__ ({ \
		aorp_object_t ___x = (x); \
		struct aorp_custom_f64 *___custom = (struct aorp_custom_f64 *)___x; \
		double ___retval = ___custom->aco_data; \
		AorpRls(___x); \
		/* return */ (___retval); \
	})
# else
__myc_extern_inline__ double AorpF64Of_rls(aorp_object_t x)
{
	struct aorp_custom_f64 *custom = (struct aorp_custom_f64 *)x;
	double retval = custom->aco_data;
	AorpRls(x);
	return (retval);
}
# endif

# define AorpDouble(x)   AorpF64(x)
# define AorpDoubleOf(x) AorpF64Of(x)
# define AorpDoubleOf_rls(x) AorpF64Of_rls(x)

/* ------------------------------------------------------------------------- */
# pragma mark --- tuple ---
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libVodi) aorp_object_t
AorpTup(
# if __myc_require(prototypes)
	size_t anArity,
	...
# endif
	);

__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
AorpNTup(
# if __myc_require(prototypes)
	aorp_object_t anObjectv[],
	size_t anObjectc
# endif
	);

__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
AorpTupAt(
# if __myc_require(prototypes)
	aorp_object_t aTuple,
	unsigned anIndex
# endif
	);

# define AorpPair(x, y)       AorpTup(2, x, y)
# define AorpTriple(x, y, z)  AorpTup(3, x, y, z)

# define AorpFst(x) AorpTupAt(x, 0)
# define AorpSnd(x) AorpTupAt(x, 1)

/* ------------------------------------------------------------------------- */
# pragma mark --- custom ---
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libVodi) aorp_object_t
AorpKum(
# if __myc_require(prototypes)
	bo_pointer_t,
	bo_malloc_free_fn
# endif
	);

# if __myc_support(macro_retval)
#  define AorpKumOf(x) \
	__extension__ ({ \
		struct aorp_custom_s *___custom = (struct aorp_custom_s *)(x); \
		/* return */ (___custom->aco_data); \
	})
# else
__myc_extern_inline__ bo_pointer_t AorpKumOf(aorp_object_t x)
{
	struct aorp_custom_s *custom = (struct aorp_custom_s *)x;
	return (custom->aco_data);
}
# endif

/* ------------------------------------------------------------------------- */
# pragma mark --- image ---
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
AorpImage(
# if __myc_require(prototypes)
	vodi_image_t
# endif
	);

# define AorpImageOf(x) ((vodi_image_t)AorpKumOf(x))

/* ------------------------------------------------------------------------- */
# pragma mark --- function ---
/* ------------------------------------------------------------------------- */

struct zlfun;

__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
AorpZlfun(
# if __myc_require(prototypes)
	struct zlfun *
# endif
	);

# define AorpZlfunOf(x) ((struct zlfun *)AorpKumOf(x))

# ifdef BO_SYSTYPE_WIN32
#  define AorpZlfunCall(f, ...) AorpZlfunCall_I(f, __VA_ARGS__)
#  define AorpZlfunCall_I(f, ...) ZLFUN_Call(AorpZlfunOf(f), __VA_ARGS__)
# else
# if __myc_support(AVM)
#  define AorpZlfunCall(...) AorpZlfunCall_I(__VA_ARGS__)
#  define AorpZlfunCall_I(f, ...) ZLFUN_Call(AorpZlfunOf(f), ##__VA_ARGS__)
# else
# if __myc_support(NVM)
#  define AorpZlfunCall(f, x...) AorpZlfunCall_I(f, x)
#  define AorpZlfunCall_I(f, x...) ZLFUN_Call(AorpZlfunOf(f), ##x)
# endif /* __myc_support(NVM) */
# endif /* __myc_support(AVM) */
# endif /* !BO_SYSTYPE_WIN32 */

/* ------------------------------------------------------------------------- */
# pragma mark --- rect ---
/* ------------------------------------------------------------------------- */

__myc_flinkage__(public,libVodi) aorp_object_t
AorpRect(
# if __myc_require(prototypes)
	int x, int y, int w, int h
# endif
	);

__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
AorpVRect(
# if __myc_require(prototypes)
	vodi_rect_t const *
# endif
	);

__myc_flinkage__(public,libVodi) vodi_rect_t *
__attribute__((nonnull(1,2)))
AorpRectOf(
# if __myc_require(prototypes)
	vodi_rect_t *,
	aorp_object_t
# endif
	);

__myc_flinkage__(public,libVodi) vodi_rect_t *
__attribute__((nonnull(1,2)))
AorpRectOf_rls(
# if __myc_require(prototypes)
	vodi_rect_t *,
	aorp_object_t
# endif
	);

__MYC_END_CDECLS

#endif /* !__VODI_OBJECTS_KUM__ */

/*
 *
 */
