/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2007 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Zlist.h	(N.Panasiuk)
 * include/Vodi/services/Zlist.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VODI_SERVICES_ZLIST__
# define __VODI_SERVICES_ZLIST__

/******************************************************************************
 * %% BeginSection: includes
 */

# include <Vodi/services/ZlistP.h>

# include <Vodi/objects/Zlist.h>

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

# include <Vodi/VodiLinkage.h>

__MYC_BEGIN_CDECLS

/* ========================================================================= */
# pragma mark * Basic functions
/* ========================================================================= */

/*
 * ZLnull :: [a] -> Bool
 *
 * Test whether a list is empty.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1)))
ZLnull(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLhead :: [a] -> a
 *
 * Extract the first element of a list, which must be non-empty.
 * NOTE! The returned object is not retained.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLhead(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLtail :: [a] -> [a]
 *
 * Extract the elements after the head of a list, which must be non-empty.
 * NOTE! The returned object is not retained.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLtail(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLlast :: [a] -> a
 *
 * Extract the last element of a list, which must be finite and non-empty.
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLlast(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLhdtl :: [a] -> (Bool, (a, [a]))
 *
 * Extract the head and tail of a list.
 * NOTE! These returned objects are not retained.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1)))
ZLhdtl(
# if __myc_require(prototypes)
	aorp_object_t aList,
	aorp_object_t *aHeadPtr,
	aorp_object_t *aTailPtr
# endif
	);

/*
 * ZLnt :: [a] -> [a]
 * ZLnt(*xsp) = (ys = AorpRtn(ZLtail(*xsp)), AorpRls(*xsp), *xsp = ys, xsp)
 */
__myc_flinkage__(public,libVodi) aorp_object_t *
__attribute__((nonnull(1)))
ZLnt(
# if __myc_require(prototypes)
	aorp_object_t *aListPtr
# endif
	);

/*
 * ZLnt2 :: [a] -> [a]
 * ZLnt2(*xsp, xs) = (AorpRtn(xs), AorpRls(*xsp), *xsp = xs, xsp)
 *
 * NOTE! Function will become the owner of @list argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t *
__attribute__((nonnull(1,2)))
ZLnt2(
# if __myc_require(prototypes)
	aorp_object_t *aListPtr,
	aorp_object_t aList
# endif
	);

/*
 * ZLhdnt :: [a] -> a
 * ZLhdnt(*xsp) = (y = AorpRtn(ZLhead(*xsp)), *xsp = ZLnt(*xsp), y)
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLhdnt(
# if __myc_require(prototypes)
	aorp_object_t *aListPtr
# endif
	);

/*
 * ZLlen :: [a] -> size_t
 */
__myc_flinkage__(public,libVodi) size_t
__attribute__((nonnull(1)))
ZLlen(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLcat :: [a] -> [a] -> [a]
 *
 * Append two lists.
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLcat(
# if __myc_require(prototypes)
	aorp_object_t aLhs,
	aorp_object_t aRhs
# endif
	);

/* ========================================================================= */
# pragma mark * List transformations
/* ========================================================================= */

/*
 * ZLmap :: (a -> b) -> [a] -> [b]
 * ZLmap f xs = [f x | x <- xs]
 *
 * ZLmap(f, xs) is the list obtained by applying f to each element of xs.
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLmap(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t aList
# endif
	);

/*
 * ZLrevonto :: [a] -> [a] -> [a]
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLrevonto(
# if __myc_require(prototypes)
	aorp_object_t aLhs,
	aorp_object_t aRhs
# endif
	);

/*
 * ZLrev :: [a] -> [a]
 * ZLrev xs = ZLrevonto xs []
 *
 * ZLrev returns the elements of list in reverse order. List must be finite.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLrev(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLintersperse :: a -> [a] -> [a]
 *
 * The ZLintersperse function takes an element and a list and
 * intersperses that element between the elements of the list.
 * For example,
 *
 * ZLintersperse(',', "abcde") == "a,b,c,d,e"
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLintersperse(
# if __myc_require(prototypes)
	aorp_object_t aSep,
	aorp_object_t aList
# endif
	);

#if 0
/*
 * ZLintercalate :: a -> [[a]] -> [a]
 *
 * ZLintercalate(xs, xss) is equivalent to (concat (intersperse xs xss)).
 * It inserts the list xs in between the lists in xss
 * and concatenates the result.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLintercalate(
# if __myc_require(prototypes)
	aorp_object_t aSep,
	aorp_object_t aList
# endif
	);
#endif

/*
 * ZLtranspose :: [[a]] -> [[a]]
 *
 * The ZLtranspose function transposes the rows and columns of its argument.
 * For example, ZLtranspose([[1,2,3],[4,5,6]]) == [[1,4],[2,5],[3,6]]
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLtranspose(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

#if 0
/*
 * ZLsubsequences :: [a] -> [[a]]
 *
 * The subsequences function returns the list of all subsequences of
 * the argument.
 *
 * ZLsubsequences("abc") == ["","a","b","ab","c","ac","bc","abc"]
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLsubsequences(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLpermutations :: [a] -> [[a]]
 *
 * The permutations function returns the list of all permutations of
 * the argument.
 *
 * ZLpermutations("abc") == ["abc","bac","cba","bca","cab","acb"]
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLpermutations(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);
#endif

/* ========================================================================= */
# pragma mark * Reducing lists (folds)
/* ========================================================================= */

/*
 * ZLfoldl :: (b -> a -> b) -> b -> [a] -> b
 *
 * ZLfoldl, applied to a binary operator, a starting value (typically
 * the left-identity of the operator), and a list, reduces the list
 * using the binary operator, from left to right:
 *
 * ZLfoldl(f, z, [x1, x2, ..., xn]) == (...((z `f` x1) `f` x2) `f`...) `f` xn
 *
 * The list must be finite.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLfoldl(
# if __myc_require(prototypes)
	aorp_object_t aBinOp,
	aorp_object_t aStartValue,
	aorp_object_t aList
# endif
	);

/*
 * ZLfoldl1 :: (a -> a -> a) -> [a] -> a
 *
 * ZLfoldl1, applied to a binary operator and a list, reduces the list
 * using the binary operator, from left to right:
 *
 * ZLfoldl1(f, [x1, x2, ..., xn]) == (...((x1 `f` x2) `f` x3) `f`...) `f` xn
 *
 * The list must be finite and non-empty.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLfoldl1(
# if __myc_require(prototypes)
	aorp_object_t aBinOp,
	aorp_object_t aList
# endif
	);

#if 0
/*
 * ZLfoldr :: (a -> b -> b) -> b -> [a] -> b
 *
 * 'ZLfoldr', applied to a binary operator, a starting value (typically
 * the right-identity of the operator), and a list, reduces the list
 * using the binary operator, from right to left:
 *
 * ZLfoldr f z [x1, x2, ..., xn] == x1 `f` (x2 `f` ... (xn `f` z)...)
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLfoldr(
# if __myc_require(prototypes)
	aorp_object_t aBinOp,
	aorp_object_t aStartValue,
	aorp_object_t aList
# endif
	);

/*
 * ZLfoldr1 :: (a -> a -> a) -> [a] -> a
 *
 * 'ZLfoldr1', applied to a binary operator and a list, reduces the list
 * using the binary operator, from right to left:
 *
 * ZLfoldr1 f [x1, x2, ..., xn] == x1 `f` (x2 `f` ... (xn-1 `f` xn)...)
 *
 * The list must be non-empty.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLfoldr1(
# if __myc_require(prototypes)
	aorp_object_t aBinOp,
	aorp_object_t aList
# endif
	);
#endif

/* ========================================================================= */
# pragma mark ** Special folds
/* ========================================================================= */

#if 0
/*
 * ZLconcat :: [[a]] -> [a]
 *
 * Concatenate a list of lists.
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLconcat(
# if __myc_require(prototypes)
	aorp_object_t aListOfLists
# endif
	);
#endif

#if 0
/*
 * ZLconcatmap :: (a -> [b]) -> [a] -> [b]
 *
 * Map a function over a list and concatenate the results.
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLconcatmap(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t aList
# endif
	);
#endif

/*
 * ZLand :: [Bool] -> Bool
 *
 * ZLand returns the conjunction of a Boolean list. For the result to be
 * True, the list must be finite; False, however, results from a False
 * value at a finite index of a finite or infinite list.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1)))
ZLand(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLor :: [Bool] -> Bool
 *
 * ZLor returns the disjunction of a Boolean list. For the result to be
 * False, the list must be finite; True, however, results from a True
 * value at a finite index of a finite or infinite list.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1)))
ZLor(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLany :: (a -> Bool) -> [a] -> Bool
 *
 * Applied to a predicate and a list, ZLany determines if any element
 * of the list satisfies the predicate. For the result to be
 * False, the list must be finite; True, however, results from a True
 * value for the predicate applied to an element at a finite index of a finite
 * or infinite list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1,2)))
ZLany(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);

/*
 * ZLall :: (a -> Bool) -> [a] -> Bool
 *
 * Applied to a predicate and a list, ZLall determines if all elements
 * of the list satisfy the predicate. For the result to be
 * True, the list must be finite; False, however, results from a False
 * value for the predicate applied to an element at a finite index of a finite
 * or infinite list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1,2)))
ZLall(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);

#if 0
/*
 * ZLsum :: Num a => [a] -> a
 *
 * The ZLsum function computes the sum of a finite list of numbers.
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLsum(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLproduct :: Num a => [a] -> a
 *
 * The ZLproduct function computes the product of a finite list of numbers.
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLproduct(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);
#endif

/*
 * ZLmaximum :: Comparable a => [a] -> a
 *
 * ZLmaximum returns the maximum value from a list,
 * which must be non-empty, finite, and of an ordered type.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLmaximum(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLminimum :: Comparable a => [a] -> a
 *
 * ZLminimum returns the minimum value from a list,
 * which must be non-empty, finite, and of an ordered type.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLminimum(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/* ========================================================================= */
# pragma mark * Building lists
# pragma mark ** Scans
/* ========================================================================= */

#if 0
/*
 * ZLscanl :: (b -> a -> b) -> b -> [a] -> [b]
 *
 * ZLscanl is similar to ZLfoldl, but returns a list of successive reduced
 * values from the left:
 *
 * ZLscanl(f, z, [x1, x2, ...]) == [z, z `f` x1, (z `f` x1) `f` x2, ...]
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLscanl(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t aStartValue,
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLscanl1 :: (a -> a -> a) -> [a] -> [a]
 *
 * ZLscanl1 is a variant of ZLscanl that has no starting value argument:
 *
 * ZLscanl1(f, [x1, x2, ...]) == [x1, x1 `f` x2, ...]
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLscanl1(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLscanr :: (a -> b -> b) -> b -> [a] -> [b]
 *
 * ZLscanr is the right-to-left dual of ZLscanl.
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLscanr(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t aStartValue,
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLscanr1 :: (a -> a -> a) -> [a] -> [a]
 *
 * ZLscanr1 is a variant of ZLscanr that has no starting value argument.
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLscanr1(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t aList
# endif
	);
#endif

/* ========================================================================= */
# pragma mark ** Accumulating maps
/* ========================================================================= */

#if 0
/*
 * ZLmapaccl :: (acc -> x -> (acc, y)) -> acc -> [x] -> (acc, [y])
 *
 * The ZLmapaccl function behaves like a combination of ZLmap and ZLfoldl;
 * it applies a function to each element of a list, passing an accumulating
 * parameter from left to right, and returning a final value of this
 * accumulator together with the new list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLmapaccl(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t anAcc,
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLmapaccr :: (acc -> x -> (acc, y)) -> acc -> [x] -> (acc, [y])
 *
 * The ZLmapaccr function behaves like a combination of map and foldr;
 * it applies a function to each element of a list, passing an accumulating
 * parameter from right to left, and returning a final value of this
 * accumulator together with the new list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLmapaccr(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t anAcc,
	aorp_object_t aList
# endif
	);
#endif

/* ========================================================================= */
# pragma mark ** Infinite lists
/* ========================================================================= */

/*
 * ZLiterate :: (a -> a) -> a -> [a]
 *
 * ZLiterate(f, x) returns an infinite list of repeated applications
 * of f to x:
 *
 * ZLiterate(f, x) == [x, f x, f (f x), ...]
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLiterate(
# if __myc_require(prototypes)
	aorp_object_t aFunction,
	aorp_object_t aStartValue
# endif
	);

/*
 * ZLrepeat :: a -> [a]
 *
 * ZLrepeat(x) is an infinite list, with x the value of every element.
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLrepeat(
# if __myc_require(prototypes)
	aorp_object_t anObject
# endif
	);

/*
 * ZLreplicate :: size_t -> a -> [a]
 *
 * ZLreplicate(n, x) is a list of length n with x the value of every element.
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(2)))
ZLreplicate(
# if __myc_require(prototypes)
	size_t aLength,
	aorp_object_t anObject
# endif
	);

/*
 * ZLcycle :: [a] -> [a]
 *
 * ZLcycle ties a finite list into a circular one, or equivalently,
 * the infinite repetition of the original list. It is the identity
 * on infinite lists.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLcycle(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/* ========================================================================= */
# pragma mark ** Unfolding
/* ========================================================================= */

/*
 * ZLunfoldr :: (b -> Maybe (a, b)) -> b -> [a]
 *
 * The ZLunfoldr function is a dual to ZLfoldr: while ZLfoldr
 * reduces a list to a summary value, ZLunfoldr builds a list from
 * a seed value.  The function takes the element and returns NULL
 * if it is done producing the list or returns (a,b), in which
 * case, `a' is a prepended to the list and `b' is used as the next
 * element in a recursive call.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLunfoldr(
# if __myc_require(prototypes)
	aorp_object_t aGenerator,
	aorp_object_t aStartValue
# endif
	);

/* ========================================================================= */
# pragma mark * Sublists
# pragma mark ** Extracting sublists
/* ========================================================================= */

/*
 * ZLtake :: size_t -> [a] -> [a]
 *
 * ZLtake(n, xs), applied to a list xs, returns the prefix of xs
 * of length n, or xs itself if n > ZLlen(xs).
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(2)))
ZLtake(
# if __myc_require(prototypes)
	size_t aCount,
	aorp_object_t aList
# endif
	);

/*
 * ZLdrop :: size_t -> [a] -> [a]
 *
 * ZLdrop(n, xs) returns the suffix of list
 * after the first n elements, or empty-list if n > ZLlen(xs).
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(2)))
ZLdrop(
# if __myc_require(prototypes)
	size_t aCount,
	aorp_object_t aList
# endif
	);

#if 0
/*
 * ZLsplitat :: size_t -> [a] -> ([a], [a])
 *
 * ZLsplitat(n, xs) returns a pair where first element is xs prefix of
 * length n and second element is the remainder of the list.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(2)))
ZLsplitat(
# if __myc_require(prototypes)
	size_t aCount,
	aorp_object_t aList
# endif
	);
#endif

/*
 * ZLtakewhile :: (a -> Bool) -> [a] -> [a]
 *
 * ZLtakewhile, applied to a predicate and a list, returns the
 * longest prefix (possibly empty) of list of elements that satisfy predicate.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLtakewhile(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);

/*
 * ZLdropwhile :: (a -> Bool) -> [a] -> [a]
 *
 * ZLdropwhile, returns the suffix remaining after ZLtakewhile.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLdropwhile(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);

/*
 * ZLspan :: (a -> Bool) -> [a] -> ([a], [a])
 *
 * ZLspan, applied to a predicate and a list, returns a tuple where
 * first element is longest prefix (possibly empty) of list of elements that
 * satisfy predicate and second element is the remainder of the list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLspan(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);

#if 0
/*
 * ZLbreak :: (a -> Bool) -> [a] -> ([a], [a])
 *
 * ZLbreak, applied to a predicate p and a list xs, returns a tuple
 * where first element is longest prefix (possibly empty) of xs of elements
 * that do not satisfy p and second element is the remainder of the list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLbreak(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLstripprefix :: Comparable a => [a] -> [a] -> Maybe [a]
 *
 * The ZLstripprefix function drops the given prefix from a list.
 * It returns NULL if the list did not start with the prefix given,
 * or Just the list after the prefix, if it does.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLstripprefix(
# if __myc_require(prototypes)
	aorp_object_t aPrefix,
	aorp_object_t aList
# endif
	);
#endif

/*
 * ZLgroup :: Comparable a => [a] -> [[a]]
 *
 * The ZLgroup function takes a list and returns a list of lists such
 * that the concatenation of the result is equal to the argument.  Moreover,
 * each sublist in the result contains only equal elements.  For example,
 *
 * ZLgroup("Mississippi") = ["M","i","ss","i","ss","i","pp","i"]
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLgroup(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

#if 0
/*
 * ZLinits :: [a] -> [[a]]
 *
 * The ZLinits function returns all initial segments of the argument,
 * shortest first. For example,
 *
 * ZLinits("abc") == ["","a","ab","abc"]
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLinits(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);
#endif

#if 0
/*
 * ZLtails :: [a] -> [[a]]
 *
 * The ZLtails function returns all final segments of the argument,
 * longest first. For example,
 *
 * ZLtails("abc") == ["abc", "bc", "c",""]
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLtails(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);
#endif

/* ========================================================================= */
# pragma mark ** Predicates
/* ========================================================================= */

#if 0
isPrefixOf
isSuffixOf
isInfixOf
#endif

/* ========================================================================= */
# pragma mark * Searching lists
# pragma mark ** Searching by equality
/* ========================================================================= */

/*
 * ZLelem :: Comparable a => a -> [a] -> Bool
 *
 * ZLelem is the list membership predicate. For the result to be
 * False, the list must be finite; True, however, results from an element
 * equal to @object found at a finite index of a finite or infinite list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1,2)))
ZLelem(
# if __myc_require(prototypes)
	aorp_object_t anObject,
	aorp_object_t aList
# endif
	);

/*
 * ZLnotelem :: Comparable a => a -> [a] -> Bool
 *
 * ZLnotelem is the negation of ZLelem.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) vodi_bool_t
__attribute__((nonnull(1,2)))
ZLnotelem(
# if __myc_require(prototypes)
	aorp_object_t anObject,
	aorp_object_t aList
# endif
	);

/*
 * ZLlookup :: Comparable a => a -> [(a, b)] -> Maybe b
 *
 * ZLlookup looks up a key in an association list.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLlookup(
# if __myc_require(prototypes)
	aorp_object_t aKey,
	aorp_object_t anAssocList
# endif
	);

/* ========================================================================= */
# pragma mark ** Searching with a predicate
/* ========================================================================= */

/*
 * ZLfind :: (a -> Bool) -> [a] -> Maybe a
 *
 * The ZLfind function takes a predicate and a list and returns the
 * first element in the list matching the predicate, or NULL if
 * there is no such element.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLfind(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);

/*
 * ZLfilter :: (a -> Bool) -> [a] -> [a]
 *
 * ZLfilter, applied to a predicate and a list, returns the list of
 * those elements that satisfy the predicate; i.e.,
 *
 * ZLfilter(p, xs) = [ x | x <- xs, p x ]
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLfilter(
# if __myc_require(prototypes)
	aorp_object_t aPredicate,
	aorp_object_t aList
# endif
	);

#if 0
partition
#endif

/* ========================================================================= */
# pragma mark * Indexing lists
/* ========================================================================= */

#if 0
(!!)
elemIndex
elemIndices
findIndex
findIndices
#endif

/* ========================================================================= */
# pragma mark * Zipping and unzipping lists
/* ========================================================================= */

/*
 * ZLzip, ZLFzip :: [a] -> [b] -> [(a, b)]
 *
 * ZLzip takes two lists and returns a list of corresponding pairs.
 * If one input list is short, excess elements of the longer list are discarded.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFzip();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLzip(
# if __myc_require(prototypes)
	aorp_object_t aLhs,
	aorp_object_t aRhs
# endif
	);

/*
 * ZLzip3, ZLFzip3 :: [a] -> [b] -> [c] -> [(a, b, c)]
 *
 * ZLzip3 takes three lists and returns a list of triples,
 * analogous to ZLzip.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFzip3();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLzip3(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzip4, ZLFzip4 :: [a] -> [b] -> [c] -> [d] -> [(a, b, c, d)]
 *
 * ZLzip4 takes four lists and returns a list of quadruples,
 * analogous to ZLzip.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFzip4();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4)))
ZLzip4(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzip5, ZLFzip5 :: [a] -> [b] -> [c] -> [d] -> [e] -> [(a, b, c, d, e)]
 *
 * ZLzip5 takes five lists and returns a list of five-tuples,
 * analogous to ZLzip.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFzip5();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4,5)))
ZLzip5(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzip6, ZLFzip6 :: [a] -> [b] -> [c] -> [d] -> [e] -> [f]
 * -> [(a, b, c, d, e, f)]
 *
 * ZLzip6 takes six lists and returns a list of six-tuples,
 * analogous to ZLzip.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFzip6();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4,5,6)))
ZLzip6(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzip7, ZLFzip7 :: [a] -> [b] -> [c] -> [d] -> [e] -> [f] -> [g]
 * -> [(a, b, c, d, e, f, g)]
 *
 * ZLzip7 takes seven lists and returns a list of seven-tuples,
 * analogous to ZLzip.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFzip7();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4,5,6,7)))
ZLzip7(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzipwith :: (a -> b -> c) -> [a] -> [b] -> [c]
 *
 * ZLzipwith generalises ZLzip by zipping with the function given as
 * the first argument, instead of a tupling function.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3)))
ZLzipwith(
# if __myc_require(prototypes)
	aorp_object_t aBinOp,
	aorp_object_t aLhs,
	aorp_object_t aRhs
# endif
	);

/*
 * ZLzipwith3 :: (a -> b -> c -> d) -> [a] -> [b] -> [c] -> [d]
 *
 * The ZLzipwith3 function takes a function which combines three elements,
 * as well as three lists and returns a list of their point-wise combination,
 * analogous to ZLzipwith.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4)))
ZLzipwith3(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzipwith4 :: (a -> b -> c -> d -> e) -> [a] -> [b] -> [c] -> [d] -> [e]
 *
 * The ZLzipwith4 function takes a function which combines four elements,
 * as well as four lists and returns a list of their point-wise combination,
 * analogous to ZLzipwith.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4,5)))
ZLzipwith4(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzipwith5 :: (a -> b -> c -> d -> e -> f) -> [a] -> [b] -> [c] -> [d]
 * -> [e] -> [f]
 *
 * The ZLzipwith5 function takes a function which combines five elements,
 * as well as five lists and returns a list of their point-wise combination,
 * analogous to ZLzipwith.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4,5,6)))
ZLzipwith5(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzipwith6 :: (a -> b -> c -> d -> e -> f -> g) -> [a] -> [b] -> [c] -> [d]
 * -> [e] -> [f] -> [g]
 *
 * The ZLzipwith6 function takes a function which combines six elements,
 * as well as six lists and returns a list of their point-wise combination,
 * analogous to ZLzipwith.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4,5,6,7)))
ZLzipwith6(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLzipwith7 :: (a -> b -> c -> d -> e -> f -> g -> h) -> [a] -> [b] -> [c]
 * -> [d] -> [e] -> [f] -> [g] -> [h]
 *
 * The ZLzipwith7 function takes a function which combines seven elements,
 * as well as seven lists and returns a list of their point-wise combination,
 * analogous to ZLzipwith.
 *
 * NOTE! Function will become the owner of its arguments.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2,3,4,5,6,7,8)))
ZLzipwith7(
# if __myc_require(prototypes)
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t,
	aorp_object_t
# endif
	);
#endif

/*
 * ZLunzip, ZLFunzip :: [(a, b)] -> ([a], [b])
 *
 * ZLunzip transforms a list of pairs into a list of first components
 * and a list of second components.
 *
 * NOTE! Function will become the owner of its argument.
 */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFunzip();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLunzip(
# if __myc_require(prototypes)
	aorp_object_t aList
# endif
	);

/*
 * ZLunzip3, ZLFunzip3 :: [(a, b, c)] -> ([a], [b], [c])
 *
 * The ZLunzip3 function takes a list of triples and returns three lists,
 * analogous to ZLunzip.
 *
 * NOTE! Function will become the owner of its argument.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFunzip3();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLunzip3(
# if __myc_require(prototypes)
	aorp_object_t
# endif
	);
#endif

/*
 * ZLunzip4, ZLFunzip4 :: [(a, b, c, d)] -> ([a], [b], [c], [d])
 *
 * The ZLunzip4 function takes a list of quadruples and returns four lists,
 * analogous to ZLunzip.
 *
 * NOTE! Function will become the owner of its argument.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFunzip4();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLunzip4(
# if __myc_require(prototypes)
	aorp_object_t
# endif
	);
#endif

/*
 * ZLunzip5, ZLFunzip5 :: [(a, b, c, d, e)] -> ([a], [b], [c], [d], [e])
 *
 * The ZLunzip5 function takes a list of five-tuples and returns five lists,
 * analogous to ZLunzip.
 *
 * NOTE! Function will become the owner of its argument.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFunzip5();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLunzip5(
# if __myc_require(prototypes)
	aorp_object_t
# endif
	);
#endif

/*
 * ZLunzip6, ZLFunzip6 :: [(a, b, c, d, e, f)] -> ([a], [b], [c], [d], [e], [f])
 *
 * The ZLunzip6 function takes a list of six-tuples and returns six lists,
 * analogous to ZLunzip.
 *
 * NOTE! Function will become the owner of its argument.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFunzip6();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLunzip6(
# if __myc_require(prototypes)
	aorp_object_t
# endif
	);
#endif

/*
 * ZLunzip7, ZLFunzip7 :: [(a, b, c, d, e, f)] -> ([a], [b], [c], [d], [e], [f])
 *
 * The ZLunzip7 function takes a list of seven-tuples and returns seven lists,
 * analogous to ZLunzip.
 *
 * NOTE! Function will become the owner of its argument.
 */
#if 0
__myc_flinkage__(public,libVodi) aorp_object_t ZLFunzip7();
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1)))
ZLunzip7(
# if __myc_require(prototypes)
	aorp_object_t
# endif
	);
#endif

/* ========================================================================= */
# pragma mark * Special lists
# pragma mark ** "Set" operations
/* ========================================================================= */

#if 0
nub
delete
(\\)
union
intersect
#endif

/* ========================================================================= */
# pragma mark ** Ordered lists
/* ========================================================================= */

#if 0
sort
insert
#endif

/* ========================================================================= */
# pragma mark * Generalized functions
# pragma mark ** User-supplied equality
/* ========================================================================= */

#if 0
nubBy
deleteBy
deleteFirstsBy
unionBy
intersectBy
#endif

/*
 * ZLgroupby :: (a -> a -> Bool) -> [a] -> [[a]]
 *
 * The ZLgroupby function is the non-overloaded version of ZLgroup.
 *
 * NOTE! Function will become the owner of its arguments.
 */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(1,2)))
ZLgroupby(
# if __myc_require(prototypes)
	aorp_object_t anEqOp,
	aorp_object_t aList
# endif
	);

/* ========================================================================= */
# pragma mark ** User-supplied comparison
/* ========================================================================= */

#if 0
sortBy
insertBy
maximumBy
minimumBy
#endif

/* ========================================================================= */
# pragma mark * Common functions
/* ========================================================================= */

/* ZLFbind :: n -> (x1->...->xn) -> m -> x1 -> ... -> xm -> (xm+1->...->xn) */
__myc_flinkage__(public,libVodi) aorp_object_t
__attribute__((nonnull(2)))
ZLFbind(
# if __myc_require(prototypes)
	size_t anArity,
	aorp_object_t aFunction,
	size_t aParmc,
	...
# endif
	);

/* ZLFdot :: (b -> c) -> (a -> b) -> a -> c */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFdot();

/* ZLFapp :: (a -> b) -> a -> b */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFapp();

/* ZLFpair :: a -> b -> (a, b) */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFpair();

/* ZLFfst :: (a, b) -> a */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFfst();

/* ZLFsnd :: (a, b) -> b */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFsnd();

/* ZLFcurry :: ((a, b) -> c) -> a -> b -> c */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFcurry();

/* ZLFuncurry :: (a -> b -> c) -> (a, b) -> c */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFuncurry();

/* ZLFflip :: (a -> b -> c) -> b -> a -> c */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFflip();

/* ZLFeq :: Comparable a => a -> a -> Bool */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFeq();

/* ZLFne :: Comparable a => a -> a -> Bool */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFne();

/* ZLFls :: Comparable a => a -> a -> Bool */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFls();

/* ZLFle :: Comparable a => a -> a -> Bool */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFle();

/* ZLFgt :: Comparable a => a -> a -> Bool */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFgt();

/* ZLFge :: Comparable a => a -> a -> Bool */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFge();

/* ZLFmax :: Comparable a => a -> a -> a */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFmax();

/* ZLFmin :: Comparable a => a -> a -> a */
__myc_flinkage__(public,libVodi) aorp_object_t ZLFmin();

__MYC_END_CDECLS

#endif /* !__VODI_SERVICES_ZLIST__ */

/*
 *
 */
