/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2007 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Log.h	(N.Panasiuk)
 * include/bwr/Vodi/utils/Log.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VODI_UTILS_LOG__
# define __VODI_UTILS_LOG__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifndef __VODI_UTILS_TYPES__
#  include <Vodi/utils/Types.h>
# endif

# ifndef __BO_LIBC_STDARG__
#  include <Bo/libc/Stdarg.h>
# endif

/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

/*
 * void VodiLOGprint<_Typ>(...);
 * void VodiLOGenter();
 * void VodiLOGleave();
 * void VodiLOGreturn(anExp);
 */

/* for example,
int foo(int a, int b)
{
	int x, y;

	VodiLOGenter();

	x = foo(a, b);

	VodiLOGprint(int, x);

	y = bar(a + b);

	VodiLOGprint(int, y);

	VodiLOGleave();

	return (x + y);
}
 */

# ifdef BO_SYSTYPE_WIN32
#  define VodiLOGprint(_Typ, ...) VodiLOGprint_I(_Typ,__VA_ARGS__)
#  define VodiLOGprint_I(_Typ, ...) VodiLOG_print(__myc_xstringify(_Typ), \
	"",__FILE__,__func__,__LINE__,__VA_ARGS__)
#  define VodiLOGdraw(_Typ, ...) VodiLOGdraw_I(_Typ,__VA_ARGS__)
#  define VodiLOGdraw_I(_Typ, ...) VodiLOG_draw(__myc_xstringify(_Typ), \
	"",__FILE__,__func__,__LINE__,__VA_ARGS__)
#  define VodiLOGreturn(...) do {VodiLOGleave(); return __VA_ARGS__;} while (0)
# else
# if __myc_support(AVM)
#  define VodiLOGprint(...) VodiLOGprint_I(__VA_ARGS__)
#  define VodiLOGprint_I(_Typ, ...) VodiLOG_print(__myc_xstringify(_Typ), \
	"",__FILE__,__func__,__LINE__,##__VA_ARGS__)
#  define VodiLOGdraw(...) VodiLOGdraw_I(__VA_ARGS__)
#  define VodiLOGdraw_I(_Typ, ...) VodiLOG_draw(__myc_xstringify(_Typ), \
	"",__FILE__,__func__,__LINE__,##__VA_ARGS__)
#  define VodiLOGreturn(...) do {VodiLOGleave(); return __VA_ARGS__;} while (0)
# else
# if __myc_support(NVM)
#  define VodiLOGprint(_Typ, x...) VodiLOGprint_I(_Typ, x)
#  define VodiLOGprint_I(_Typ, x...) VodiLOG_print(__myc_xstringify(_Typ), \
	"",__FILE__,__func__,__LINE__,##x)
#  define VodiLOGdraw(_Typ, x...) VodiLOGdraw_I(_Typ, x)
#  define VodiLOGdraw_I(_Typ, x...) VodiLOG_draw(__myc_xstringify(_Typ), \
	"",__FILE__,__func__,__LINE__,##x)
#  define VodiLOGreturn(x...) do { VodiLOGleave(); return x; } while (0)
# endif /* __myc_support(NVM) */
# endif /* __myc_support(AVM) */
# endif /* !BO_SYSTYPE_WIN32 */

# define VodiLOGenter() VodiLOG_enter(__FILE__,__func__,__LINE__)
# define VodiLOGleave() VodiLOG_leave(__FILE__,__func__,__LINE__)

# if !defined(VODI_LOG_TRACE) || (0 == VODI_LOG_TRACE - 0)
#  undef VodiLOGprint_I
#  define VodiLOGprint_I(...) ((void)0)

#  undef VodiLOGdraw_I
#  define VodiLOGdraw_I(...) ((void)0)

#  undef VodiLOGenter
#  define VodiLOGenter() ((void)0)

#  undef VodiLOGleave
#  define VodiLOGleave() ((void)0)

#  undef VodiLOGreturn
#  if (defined(BO_SYSTYPE_WIN32) || __myc_support(AVM))
#   define VodiLOGreturn(...) return __VA_ARGS__
#  else
#  if __myc_support(NVM)
#   define VodiLOGreturn(x...) return x
#  endif /* __myc_support(NVM) */
#  endif /* !BO_SYSTYPE_WIN32 && !__myc_support(AVM) */
# endif

# ifndef __VODI_UTILS_LIBINTR_LINKAGE__
#  include <Vodi/utils/libintr/Linkage.h>
# endif

__MYC_BEGIN_CDECLS

__myc_flinkage__(public,libVodiUTILS) void
__attribute__((nonnull(1,2,3,4)))
VodiLOG_print(
# if __myc_require(prototypes)
	char const *aTypStr,
	char const *anExp,
	char const *aFile,
	char const *aFunc,
	long aLine,
	...
# endif
	);

__myc_flinkage__(public,libVodiUTILS) void
__attribute__((nonnull(1,2,3,4)))
VodiLOG_vprint(
# if __myc_require(prototypes)
	char const *aTypStr,
	char const *anExp,
	char const *aFile,
	char const *aFunc,
	long aLine,
	va_list ArgPtr
# endif
	);

__myc_flinkage__(public,libVodiUTILS) void
__attribute__((nonnull(1,2,3,4)))
VodiLOG_draw(
# if __myc_require(prototypes)
	char const *aTypStr,
	char const *anExp,
	char const *aFile,
	char const *aFunc,
	long aLine,
	...
# endif
	);

__myc_flinkage__(public,libVodiUTILS) void
__attribute__((nonnull(1,2,3,4)))
VodiLOG_vdraw(
# if __myc_require(prototypes)
	char const *aTypStr,
	char const *anExp,
	char const *aFile,
	char const *aFunc,
	long aLine,
	va_list ArgPtr
# endif
	);

__myc_flinkage__(public,libVodiUTILS) void
__attribute__((nonnull(1,2)))
VodiLOG_enter(
# if __myc_require(prototypes)
	char const *aFile,
	char const *aFunc,
	long aLine
# endif
	);

__myc_flinkage__(public,libVodiUTILS) void
__attribute__((nonnull(1,2)))
VodiLOG_leave(
# if __myc_require(prototypes)
	char const *aFile,
	char const *aFunc,
	long aLine
# endif
	);

__MYC_END_CDECLS

#endif /* !defined(__VODI_UTILS_LOG__) */

/******************************************************************************
 *
 *
 *
 *****************************************************************************/
