/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2007 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Errors.h	(N.Panasiuk)
 * include/bwr/Vodi/utils/sex/Errors.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VODI_UTILS_SEX_ERRORS__
# define __VODI_UTILS_SEX_ERRORS__

/******************************************************************************
 * %% BeginSection: type/constant declarations
 */
# pragma mark -
# pragma mark [ type/constant declarations ]
# pragma mark -

/*
 * Common errors: [1..99]
 */

# define VodiK_SEX_EIO       1     /* (char *) */

/*
 * Lexical errors: [100..999]
 */

# define VodiK_SEX_EEOF      100   /* (char *) */

/* integer constant exceeds range of values */
# define VodiK_SEX_EINT      110   /* (char *, long, long, char *) */

/* floating constant exceeds range of values */
# define VodiK_SEX_EFLT      111   /* (char *, long, long, char *) */

/* invalid character in string literal */
# define VodiK_SEX_ESTRCHR   112   /* (char *, long, long, char *) */

/* invalid escape sequence */
# define VodiK_SEX_EESC      113   /* (char *, long, long, char *) */

/* illegal multibyte sequence */
# define VodiK_SEX_EILSEQ    114   /* (char *, long, long) */

/* nest comment limit exceeded */
# define VodiK_SEX_ENCOMMLIM 115   /* (char *, long, long) */

/* unmatched character */
# define VodiK_SEX_WUNMCHR   500   /* (char *, long, long, char *) */

/*
 * Grammar errors: [1000..1999]
 */

/* unexpected token */
# define VodiK_SEX_EUNEXPETOK   1000  /* (char *, long, long, char *) */

/* missed a closing brace */
# define VodiK_SEX_EMISCBRACE   1001  /* (int, char *, long, long) */

/* syntax error in input (unexpected token) */
# define VodiK_SEX_EINPUT       1002  /* (char *, long, long, char *) */

/* syntax error in expression (unexpected token) */
# define VodiK_SEX_EEXPR        1003  /* (char *, long, long, char *) */

/* syntax error in case expression (unexpected token) */
# define VodiK_SEX_ECASE        1004  /* (char *, long, long, char *) */

/* syntax error in do {...} expression (last generator must be an expression) */
# define VodiK_SEX_EDO          1005  /* (char *, long) */

/* syntax error in type expression (unexpected token) */
# define VodiK_SEX_ETYPE        1006  /* (char *, long, long, char *) */

/* syntax error in class expression */
# define VodiK_SEX_ECTX         1007  /* (char *, long, long, char *) */

/* syntax error in pattern expression (unexpected token) */
# define VodiK_SEX_EPATTERN     1008  /* (char *, long, long, char *) */

/* syntax error in fixity declaration (unexpected token) */
# define VodiK_SEX_EFIXITYDECL  1009  /* (char *, long, long, char *) */

/* precedence value must be an integer in the range [0..10] */
# define VodiK_SEX_EPREC        1010  /* (char *, long, long, long, long) */

/* syntax error in type signature (unexpected token) */
# define VodiK_SEX_ETYPESIGN    1011  /* (char *, long, long, char *) */

/* syntax error in declaration (unexpected token) */
# define VodiK_SEX_EDECL        1012  /* (char *, long, long, char *) */

/* syntax error in class declaration (unexpected token) */
# define VodiK_SEX_ECLSDECL     1013  /* (char *, long, long, char *) */

/* syntax error in instance declaration (unexpected token) */
# define VodiK_SEX_EINSTDECL    1014  /* (char *, long, long, char *) */

/* syntax error in default declaration (unexpected token) */
# define VodiK_SEX_EDEFDECL     1015  /* (char *, long, long, char *) */

/* syntax error in functional dependency (unexpected token) */
# define VodiK_SEX_EFUNDEP      1016  /* (char *, long, long, char *) */

/* syntax error in primitive declaration (unexpected token) */
# define VodiK_SEX_EPRIMDECL    1017  /* (char *, long, long, char *) */

/* syntax error in type declaration (unexpected token) */
# define VodiK_SEX_ETYPEDECL    1018  /* (char *, long, long, char *) */

/* syntax error in data declaration (unexpected token) */
# define VodiK_SEX_EDATADECL    1019  /* (char *, long, long, char *) */

/* syntax error in newtype declaration (unexpected token) */
# define VodiK_SEX_ENEWTYPEDECL 1020  /* (char *, long, long, char *) */

/* syntax error in type declaration lhs (unexpected token) */
# define VodiK_SEX_ETYPEDECLLHS 1021  /* (char *, long, long, char *) */

/* qualified constructor in data type declaration */
# define VodiK_SEX_EQCONSTR     1022  /* (char *, long, long, char *) */

/* syntax error in data type declaration (unexpected token) */
# define VodiK_SEX_ECONSTR      1023  /* (char *, long, long, char *) */

/* syntax error in module definition (unexpected token) */
# define VodiK_SEX_EMODDEF      1024  /* (char *, long, long, char *) */

/* illegal left hand side in data type declaration */
# define VodiK_SEX_EILTYLHS     1025  /* (char *, long, long) */

/* tuple too long */
# define VodiK_SEX_ETUPTOOLONG  1026  /* (char *, long, long) */

#endif /* !defined(__VODI_UTILS_SEX_ERRORS__) */

/******************************************************************************
 *
 *
 *
 *****************************************************************************/