/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2019 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * MatMul_inl.h	(D.Vinnyk)
 * Vipm/ArithLogic/Arithmetic/MatMul_inl.h
 */

/******************************************************************************
 * %% BeginSection: function definitions
 */
# pragma mark -
# pragma mark [ function definitions ]
# pragma mark -

# define __vipm_elmat_f32(base,stride,i,j) (*((float const *)((char const *)(base) + (i) * (stride)) + (j)))
# define __vipm_lst_elmat_f32(baseLst,i,j) (*((float const *)((baseLst)[i]) + (j)))
# define __vipm_mut_elmat_f32(base,stride,i,j) (*((float *)((char *)(base) + (i) * (stride)) + (j)))
# define __TEST_C_IMPL 0

/*
 * mulmm
 */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x1x1x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x1x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.S }[0], [%[src1]] \n\t"
		"ld1 { v8.2S },   [%[src2]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0] \n\t"
		"st1 { v0.2S },   [%[dst]]  \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src2] "r"(aBaseSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8[0] }, [%[src1]] \n\t"
		"vld1.32 { d16 },   [%[src2]] \n\t"
		"vmul.F32 d0, d16, d8[0]      \n\t"
		"vst1.32 { d0 },    [%[dst]]  \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src2] "r"(aBaseSrc2)
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * (*(*(float const **)&aBaseSrc2)++);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x1x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * (*(*(float const **)&aBaseSrc2)++);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * (*(*(float const **)&aBaseSrc2)++);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x1x1x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x1x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.2S },   [%[src2]]  \n\t"
		"ld1 { v4.S }[0], [%[src11]] \n\t"
		"ld1 { v4.S }[1], [%[src12]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"fmul v1.2S, v8.2S, v4.S[1]  \n\t"
		"st1 { v0.2S },   [%[dst1]]  \n\t"
		"st1 { v1.2S },   [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16 },   [%[src2]]  \n\t"
		"vld1.32 { d8[0] }, [%[src11]] \n\t"
		"vld1.32 { d8[1] }, [%[src12]] \n\t"
		"vmul.F32 d0, d16, d8[0]       \n\t"
		"vmul.F32 d1, d16, d8[1]       \n\t"
		"vst1.32 { d0 },    [%[dst1]]  \n\t"
		"vst1.32 { d1 },    [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x1x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x1x1x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x1x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.2S },   [%[src2]]  \n\t"
		"ld1 { v4.S }[0], [%[src11]] \n\t"
		"ld1 { v4.S }[1], [%[src12]] \n\t"
		"ld1 { v4.S }[2], [%[src13]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"fmul v1.2S, v8.2S, v4.S[1]  \n\t"
		"fmul v2.2S, v8.2S, v4.S[2]  \n\t"
		"st1 { v0.2S },   [%[dst1]]  \n\t"
		"st1 { v1.2S },   [%[dst2]]  \n\t"
		"st1 { v2.2S },   [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16 },   [%[src2]]  \n\t"
		"vld1.32 { d8[0] }, [%[src11]] \n\t"
		"vld1.32 { d8[1] }, [%[src12]] \n\t"
		"vld1.32 { d9[0] }, [%[src13]] \n\t"
		"vmul.F32 d0, d16, d8[0]       \n\t"
		"vmul.F32 d1, d16, d8[1]       \n\t"
		"vmul.F32 d2, d16, d9[0]       \n\t"
		"vst1.32 { d0 },    [%[dst1]]  \n\t"
		"vst1.32 { d1 },    [%[dst2]]  \n\t"
		"vst1.32 { d2 },    [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x1x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x2x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S },   [%[src1]]  \n\t"
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"fmul  v1.2S, v8.2S, v4.2S   \n\t"
		"faddp v0.2S, v1.2S, v1.2S   \n\t"
		"st1 { v0.S }[0], [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },     [%[src1]]  \n\t"
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vmul.F32  d1, d16, d8          \n\t"
		"vpadd.F32 d0,  d1, d1          \n\t"
		"vst1.32 { d0[0] },  [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S }, [%[src1]]   \n\t"
		"ld1 { v8.2S }, [%[src21]]  \n\t"
		"ld1 { v9.2S }, [%[src22]]  \n\t"
		"fmul v0.2S, v8.2S, v4.S[0] \n\t"
		"fmla v0.2S, v9.2S, v4.S[1] \n\t"
		"st1 { v0.2S }, [%[dst]]    \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src1]]  \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmul.F32 d0, d16, d8[0]     \n\t"
		"vmla.F32 d0, d17, d8[1]     \n\t"
		"vst1.32 { d0 },  [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x2x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x2x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"ld1 { v4.2S },   [%[src11]] \n\t"
		"ld1 { v5.2S },   [%[src12]] \n\t"
		"fmul  v4.2S, v4.2S, v8.2S   \n\t"
		"fmul  v5.2S, v5.2S, v8.2S   \n\t"
		"faddp v6.2S, v4.2S, v5.2S   \n\t"
		"st1 { v6.S }[0], [%[dst1]]  \n\t"
		"st1 { v6.S }[1], [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vld1.32 { d8 },     [%[src11]] \n\t"
		"vld1.32 { d9 },     [%[src12]] \n\t"
		"vmul.F32   d8, d8, d16         \n\t"
		"vmul.F32   d9, d9, d16         \n\t"
		"vpadd.F32 d10, d8,  d9         \n\t"
		"vst1.32 { d10[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d10[1] }, [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S }, [%[src11]]  \n\t"
		"ld1 { v5.2S }, [%[src12]]  \n\t"
		"ld1 { v8.2S }, [%[src21]]  \n\t"
		"ld1 { v9.2S }, [%[src22]]  \n\t"
		"fmul v0.2S, v8.2S, v4.S[0] \n\t"
		"fmul v1.2S, v8.2S, v5.S[0] \n\t"
		"fmla v0.2S, v9.2S, v4.S[1] \n\t"
		"fmla v1.2S, v9.2S, v5.S[1] \n\t"
		"st1 { v0.2S }, [%[dst1]]   \n\t"
		"st1 { v1.2S }, [%[dst2]]   \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src11]] \n\t"
		"vld1.32 { d9 },  [%[src12]] \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmul.F32 d0, d16, d8[0]     \n\t"
		"vmul.F32 d1, d16, d9[0]     \n\t"
		"vmla.F32 d0, d17, d8[1]     \n\t"
		"vmla.F32 d1, d17, d9[1]     \n\t"
		"vst1.32 { d0 },  [%[dst1]]  \n\t"
		"vst1.32 { d1 },  [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x2x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x2x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"ld1 { v4.2S },   [%[src11]] \n\t"
		"ld1 { v5.2S },   [%[src12]] \n\t"
		"ld1 { v6.2S },   [%[src13]] \n\t"
		"fmul  v4.2S, v4.2S, v8.2S   \n\t"
		"fmul  v5.2S, v5.2S, v8.2S   \n\t"
		"fmul  v6.2S, v6.2S, v8.2S   \n\t"
		"faddp v7.2S, v4.2S, v5.2S   \n\t"
		"faddp v8.2S, v6.2S, v6.2S   \n\t"
		"st1 { v7.S }[0], [%[dst1]]  \n\t"
		"st1 { v7.S }[1], [%[dst2]]  \n\t"
		"st1 { v8.S }[0], [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vld1.32 { d8 },     [%[src11]] \n\t"
		"vld1.32 { d9 },     [%[src12]] \n\t"
		"vld1.32 { d10 },    [%[src13]] \n\t"
		"vmul.F32   d8,  d8, d16        \n\t"
		"vmul.F32   d9,  d9, d16        \n\t"
		"vmul.F32  d10, d10, d16        \n\t"
		"vpadd.F32 d12,  d8,  d9        \n\t"
		"vpadd.F32 d13, d10, d10        \n\t"
		"vst1.32 { d12[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d12[1] }, [%[dst2]]  \n\t"
		"vst1.32 { d13[0] }, [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S }, [%[src11]]  \n\t"
		"ld1 { v5.2S }, [%[src12]]  \n\t"
		"ld1 { v6.2S }, [%[src13]]  \n\t"
		"ld1 { v8.2S }, [%[src21]]  \n\t"
		"ld1 { v9.2S }, [%[src22]]  \n\t"
		"fmul v0.2S, v8.2S, v4.S[0] \n\t"
		"fmul v1.2S, v8.2S, v5.S[0] \n\t"
		"fmul v2.2S, v8.2S, v6.S[0] \n\t"
		"fmla v0.2S, v9.2S, v4.S[1] \n\t"
		"fmla v1.2S, v9.2S, v5.S[1] \n\t"
		"fmla v2.2S, v9.2S, v6.S[1] \n\t"
		"st1 { v0.2S }, [%[dst1]]   \n\t"
		"st1 { v1.2S }, [%[dst2]]   \n\t"
		"st1 { v2.2S }, [%[dst3]]   \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src11]] \n\t"
		"vld1.32 { d9 },  [%[src12]] \n\t"
		"vld1.32 { d10 }, [%[src13]] \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]    \n\t"
		"vmul.F32 d1, d16,  d9[0]    \n\t"
		"vmul.F32 d2, d16, d10[0]    \n\t"
		"vmla.F32 d0, d17,  d8[1]    \n\t"
		"vmla.F32 d1, d17,  d9[1]    \n\t"
		"vmla.F32 d2, d17, d10[1]    \n\t"
		"vst1.32 { d0 },  [%[dst1]]  \n\t"
		"vst1.32 { d1 },  [%[dst2]]  \n\t"
		"vst1.32 { d2 },  [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x2x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x3x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x3x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S,v6.S }[0], [%[src1]]  \n\t"
		"ld1 { v8.2S },             [%[src21]] \n\t"
		"ld1 { v9.2S },             [%[src22]] \n\t"
		"ld1 { v10.2S },            [%[src23]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]           \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]           \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]           \n\t"
		"st1 { v0.2S },             [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23]"r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 { d8[0],d9[0],d10[0] }, [%[src1]]  \n\t"
		"vld1.32 { d16 },                [%[src21]] \n\t"
		"vld1.32 { d17 },                [%[src22]] \n\t"
		"vld1.32 { d18 },                [%[src23]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                   \n\t"
		"vmla.F32 d0, d17,  d9[0]                   \n\t"
		"vmla.F32 d0, d18, d10[0]                   \n\t"
		"vst1.32 { d0 },                 [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23]"r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_1x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x3x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x3x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S,v6.S }[0], [%[src11]] \n\t"
		"ld3 { v4.S,v5.S,v6.S }[1], [%[src12]] \n\t"
		"ld1 { v8.2S },             [%[src21]] \n\t"
		"ld1 { v9.2S },             [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]           \n\t"
		"fmul v1.2S,  v8.2S, v4.S[1]           \n\t"
		"ld1 { v10.2S },            [%[src23]] \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]           \n\t"
		"fmla v1.2S,  v9.2S, v5.S[1]           \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]           \n\t"
		"fmla v1.2S, v10.2S, v6.S[1]           \n\t"
		"st1 { v0.2S },             [%[dst1]]  \n\t"
		"st1 { v1.2S },             [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 { d8[0],d9[0],d10[0] }, [%[src11]] \n\t"
		"vld3.32 { d8[1],d9[1],d10[1] }, [%[src12]] \n\t"
		"vld1.32 { d16 },                [%[src21]] \n\t"
		"vld1.32 { d17 },                [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                   \n\t"
		"vmul.F32 d1, d16,  d8[1]                   \n\t"
		"vld1.32 { d18 },                [%[src23]] \n\t"
		"vmla.F32 d0, d17,  d9[0]                   \n\t"
		"vmla.F32 d1, d17,  d9[1]                   \n\t"
		"vmla.F32 d0, d18, d10[0]                   \n\t"
		"vmla.F32 d1, d18, d10[1]                   \n\t"
		"vst1.32 { d0 },                 [%[dst1]]  \n\t"
		"vst1.32 { d1 },                 [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_2x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x3x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x3x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S,v6.S }[0], [%[src11]] \n\t"
		"ld3 { v4.S,v5.S,v6.S }[1], [%[src12]] \n\t"
		"ld3 { v4.S,v5.S,v6.S }[2], [%[src13]] \n\t"
		"ld1 { v8.2S },             [%[src21]] \n\t"
		"ld1 { v9.2S },             [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]           \n\t"
		"fmul v1.2S,  v8.2S, v4.S[1]           \n\t"
		"fmul v2.2S,  v8.2S, v4.S[2]           \n\t"
		"ld1 { v10.2S },            [%[src23]] \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]           \n\t"
		"fmla v1.2S,  v9.2S, v5.S[1]           \n\t"
		"fmla v2.2S,  v9.2S, v5.S[2]           \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]           \n\t"
		"fmla v1.2S, v10.2S, v6.S[1]           \n\t"
		"fmla v2.2S, v10.2S, v6.S[2]           \n\t"
		"st1 { v0.2S },             [%[dst1]]  \n\t"
		"st1 { v1.2S },             [%[dst2]]  \n\t"
		"st1 { v2.2S },             [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 {  d8[0], d9[0],d10[0] }, [%[src11]] \n\t"
		"vld3.32 {  d8[1], d9[1],d10[1] }, [%[src12]] \n\t"
		"vld3.32 { d11[0],d12[0],d13[0] }, [%[src13]] \n\t"
		"vld1.32 { d16 },                  [%[src21]] \n\t"
		"vld1.32 { d17 },                  [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                     \n\t"
		"vmul.F32 d1, d16,  d8[1]                     \n\t"
		"vmul.F32 d2, d16, d11[0]                     \n\t"
		"vld1.32 { d18 },                  [%[src23]] \n\t"
		"vmla.F32 d0, d17,  d9[0]                     \n\t"
		"vmla.F32 d1, d17,  d9[1]                     \n\t"
		"vmla.F32 d2, d17, d12[0]                     \n\t"
		"vmla.F32 d0, d18, d10[0]                     \n\t"
		"vmla.F32 d1, d18, d10[1]                     \n\t"
		"vmla.F32 d2, d18, d13[0]                     \n\t"
		"vst1.32 { d0 },                   [%[dst1]]  \n\t"
		"vst1.32 { d1 },                   [%[dst2]]  \n\t"
		"vst1.32 { d2 },                   [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_3x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_4x4x4x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"ld1 { v8.S }[2], [%[src23]] \n\t"
		"ld1 { v8.S }[3], [%[src24]] \n\t"
		"ld1 { v4.4S },   [%[src11]] \n\t"
		"ld1 { v5.4S },   [%[src12]] \n\t"
		"ld1 { v6.4S },   [%[src13]] \n\t"
		"ld1 { v7.4S },   [%[src14]] \n\t"
		"fmul  v4.4S, v4.4S, v8.4S   \n\t"
		"fmul  v5.4S, v5.4S, v8.4S   \n\t"
		"fmul  v6.4S, v6.4S, v8.4S   \n\t"
		"fmul  v7.4S, v7.4S, v8.4S   \n\t"
		"faddp v2.4S, v4.4S, v5.4S   \n\t"
		"faddp v3.4S, v6.4S, v7.4S   \n\t"
		"faddp v9.4S, v2.4S, v3.4S   \n\t"
		"st1 { v9.S }[0], [%[dst1]]  \n\t"
		"st1 { v9.S }[1], [%[dst2]]  \n\t"
		"st1 { v9.S }[2], [%[dst3]]  \n\t"
		"st1 { v9.S }[3], [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2),
		  [src24] "r"((char const *)aBaseSrc2 + 3 * aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vld1.32 { d17[0] }, [%[src23]] \n\t"
		"vld1.32 { d17[1] }, [%[src24]] \n\t"
		"vld1.32 { q4 },     [%[src11]] \n\t"
		"vld1.32 { q5 },     [%[src12]] \n\t"
		"vld1.32 { q6 },     [%[src13]] \n\t"
		"vld1.32 { q7 },     [%[src14]] \n\t"
		"vmul.F32   q4,  q4,  q8        \n\t"
		"vmul.F32   q5,  q5,  q8        \n\t"
		"vmul.F32   q6,  q6,  q8        \n\t"
		"vmul.F32   q7,  q7,  q8        \n\t"
		"vpadd.F32  d4,  d8,  d9        \n\t"
		"vpadd.F32  d5, d10, d11        \n\t"
		"vpadd.F32  d6, d12, d13        \n\t"
		"vpadd.F32  d7, d14, d15        \n\t"
		"vpadd.F32 d18,  d4,  d5        \n\t"
		"vpadd.F32 d19,  d6,  d7        \n\t"
		"vst1.32 { d18[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d18[1] }, [%[dst2]]  \n\t"
		"vst1.32 { d19[0] }, [%[dst3]]  \n\t"
		"vst1.32 { d19[1] }, [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2),
		  [src24] "r"((char const *)aBaseSrc2 + 3 * aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmm_f32f32f32_4x4x4x4(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 {  v8.4S }, [%[src21]]  \n\t"
		"ld1 {  v9.4S }, [%[src22]]  \n\t"
		"ld1 { v10.4S }, [%[src23]]  \n\t"
		"ld1 { v11.4S }, [%[src24]]  \n\t"
		"ld1 {  v4.4S }, [%[src11]]  \n\t"
		"ld1 {  v5.4S }, [%[src12]]  \n\t"
		"ld1 {  v6.4S }, [%[src13]]  \n\t"
		"ld1 {  v7.4S }, [%[src14]]  \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0] \n\t"
		"fmul v1.2S,  v8.2S, v5.S[0] \n\t"
		"fmul v2.2S,  v8.2S, v6.S[0] \n\t"
		"fmul v3.2S,  v8.2S, v7.S[0] \n\t"
		"fmla v0.2S,  v9.2S, v4.S[1] \n\t"
		"fmla v1.2S,  v9.2S, v5.S[1] \n\t"
		"fmla v2.2S,  v9.2S, v6.S[1] \n\t"
		"fmla v3.2S,  v9.2S, v7.S[1] \n\t"
		"fmla v0.2S, v10.2S, v4.S[2] \n\t"
		"fmla v1.2S, v10.2S, v5.S[2] \n\t"
		"fmla v2.2S, v10.2S, v6.S[2] \n\t"
		"fmla v3.2S, v10.2S, v7.S[2] \n\t"
		"fmla v0.2S, v11.2S, v4.S[3] \n\t"
		"fmla v1.2S, v11.2S, v5.S[3] \n\t"
		"fmla v2.2S, v11.2S, v6.S[3] \n\t"
		"fmla v3.2S, v11.2S, v7.S[3] \n\t"
		"st1 { v0.4S },  [%[dst1]]   \n\t"
		"st1 { v1.4S },  [%[dst2]]   \n\t"
		"st1 { v2.4S },  [%[dst3]]   \n\t"
		"st1 { v3.4S },  [%[dst4]]   \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2),
		  [src24] "r"((char const *)aBaseSrc2 + 3 * aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { q8 },  [%[src21]] \n\t"
		"vld1.32 { q9 },  [%[src22]] \n\t"
		"vld1.32 { q10 }, [%[src23]] \n\t"
		"vld1.32 { q11 }, [%[src24]] \n\t"
		"vld1.32 { q4 },  [%[src11]] \n\t"
		"vld1.32 { q5 },  [%[src12]] \n\t"
		"vld1.32 { q6 },  [%[src13]] \n\t"
		"vld1.32 { q7 },  [%[src14]] \n\t"
		"vmul.F32 q0,  q8,  d8[0]    \n\t"
		"vmul.F32 q1,  q8, d10[0]    \n\t"
		"vmul.F32 q2,  q8, d12[0]    \n\t"
		"vmul.F32 q3,  q8, d14[0]    \n\t"
		"vmla.F32 q0,  q9,  d8[1]    \n\t"
		"vmla.F32 q1,  q9, d10[1]    \n\t"
		"vmla.F32 q2,  q9, d12[1]    \n\t"
		"vmla.F32 q3,  q9, d14[1]    \n\t"
		"vmla.F32 q0, q10,  d9[0]    \n\t"
		"vmla.F32 q1, q10, d11[0]    \n\t"
		"vmla.F32 q2, q10, d13[0]    \n\t"
		"vmla.F32 q3, q10, d15[0]    \n\t"
		"vmla.F32 q0, q11,  d9[1]    \n\t"
		"vmla.F32 q1, q11, d11[1]    \n\t"
		"vmla.F32 q2, q11, d13[1]    \n\t"
		"vmla.F32 q3, q11, d15[1]    \n\t"
		"vst1.32 { q0 },  [%[dst1]]  \n\t"
		"vst1.32 { q1 },  [%[dst2]]  \n\t"
		"vst1.32 { q2 },  [%[dst3]]  \n\t"
		"vst1.32 { q3 },  [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2),
		  [src24] "r"((char const *)aBaseSrc2 + 3 * aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
# endif
	return (VipmK_SUCCESS);
}

/* ========================================================================= */

/*
 * mulmmt
 */

# define vipma_mulmmt_f32f32f32_1x1x1x1 vipma_mulmm_f32f32f32_1x1x1x1

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x1x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.S }[0], [%[src1]]  \n\t"
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"st1 { v0.2S },   [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8[0] },  [%[src1]]  \n\t"
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16, d8[0]        \n\t"
		"vst1.32 { d0 },     [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x1x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

# define vipma_mulmmt_f32f32f32_2x1x1x1 vipma_mulmm_f32f32f32_2x1x1x1

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x1x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"ld1 { v4.S }[0], [%[src11]] \n\t"
		"ld1 { v4.S }[1], [%[src12]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"fmul v1.2S, v8.2S, v4.S[1]  \n\t"
		"st1 { v0.2S },   [%[dst1]]  \n\t"
		"st1 { v1.2S },   [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vld1.32 { d8[0] },  [%[src11]] \n\t"
		"vld1.32 { d8[1] },  [%[src12]] \n\t"
		"vmul.F32 d0, d16, d8[0]        \n\t"
		"vmul.F32 d1, d16, d8[1]        \n\t"
		"vst1.32 { d0 },     [%[dst1]]  \n\t"
		"vst1.32 { d1 },     [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x1x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

# define vipma_mulmmt_f32f32f32_3x1x1x1 vipma_mulmm_f32f32f32_3x1x1x1

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x1x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"ld1 { v4.S }[0], [%[src11]] \n\t"
		"ld1 { v4.S }[1], [%[src12]] \n\t"
		"ld1 { v4.S }[2], [%[src13]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"fmul v1.2S, v8.2S, v4.S[1]  \n\t"
		"fmul v2.2S, v8.2S, v4.S[2]  \n\t"
		"st1 { v0.2S },   [%[dst1]]  \n\t"
		"st1 { v1.2S },   [%[dst2]]  \n\t"
		"st1 { v2.2S },   [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vld1.32 { d8[0] },  [%[src11]] \n\t"
		"vld1.32 { d8[1] },  [%[src12]] \n\t"
		"vld1.32 { d9[0] },  [%[src13]] \n\t"
		"vmul.F32 d0, d16, d8[0]        \n\t"
		"vmul.F32 d1, d16, d8[1]        \n\t"
		"vmul.F32 d2, d16, d9[0]        \n\t"
		"vst1.32 { d0 },     [%[dst1]]  \n\t"
		"vst1.32 { d1 },     [%[dst2]]  \n\t"
		"vst1.32 { d2 },     [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x1x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x2x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S },   [%[src1]] \n\t"
		"ld1 { v8.2S },   [%[src2]] \n\t"
		"fmul  v1.2S, v8.2S, v4.2S  \n\t"
		"faddp v0.2S, v1.2S, v1.2S  \n\t"
		"st1 { v0.S }[0], [%[dst]]  \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src2] "r"(aBaseSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },    [%[src1]] \n\t"
		"vld1.32 { d16 },   [%[src2]] \n\t"
		"vmul.F32  d1, d16, d8        \n\t"
		"vpadd.F32 d0,  d1, d1        \n\t"
		"vst1.32 { d0[0] }, [%[dst]]  \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src2] "r"(aBaseSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S },   [%[src1]]  \n\t"
		"ld1 { v8.2S },   [%[src21]] \n\t"
		"ld1 { v8.D }[1], [%[src22]] \n\t"
		"mov v4.D[1], v4.D[0]        \n\t"
		"fmul  v0.4S, v8.4S, v4.4S   \n\t"
		"faddp v1.4S, v0.4S, v0.4S   \n\t"
		"st1 { v1.2S },  [%[dst]]    \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src1]]  \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmov      d9, d8            \n\t"
		"vmul.F32  q0, q8, q4        \n\t"
		"vpadd.F32 d2, d0, d1        \n\t"
		"vst1.32 { d2 },  [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x2x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x2x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.2S },   [%[src2]]  \n\t"
		"ld1 { v4.2S },   [%[src11]] \n\t"
		"ld1 { v5.2S },   [%[src12]] \n\t"
		"fmul  v4.2S, v4.2S, v8.2S   \n\t"
		"fmul  v5.2S, v5.2S, v8.2S   \n\t"
		"faddp v6.2S, v4.2S, v5.2S   \n\t"
		"st1 { v6.S }[0], [%[dst1]]  \n\t"
		"st1 { v6.S }[1], [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16 },    [%[src2]]  \n\t"
		"vld1.32 { d8 },     [%[src11]] \n\t"
		"vld1.32 { d9 },     [%[src12]] \n\t"
		"vmul.F32   d8, d8, d16         \n\t"
		"vmul.F32   d9, d9, d16         \n\t"
		"vpadd.F32 d10, d8,  d9         \n\t"
		"vst1.32 { d10[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d10[1] }, [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S }, [%[src11]] \n\t"
		"ld1 { v5.2S }, [%[src12]] \n\t"
		"ld1 { v8.2S }, [%[src21]] \n\t"
		"ld1 { v9.2S }, [%[src22]] \n\t"
		"fmul  v0.2S, v4.2S, v8.2S \n\t"
		"fmul  v1.2S, v4.2S, v9.2S \n\t"
		"fmul  v2.2S, v5.2S, v8.2S \n\t"
		"fmul  v3.2S, v5.2S, v9.2S \n\t"
		"faddp v6.2S, v0.2S, v1.2S \n\t"
		"faddp v7.2S, v2.2S, v3.2S \n\t"
		"st1 { v6.2S }, [%[dst1]]  \n\t"
		"st1 { v7.2S }, [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src11]] \n\t"
		"vld1.32 { d9 },  [%[src12]] \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmul.F32  d0, d8, d16       \n\t"
		"vmul.F32  d1, d8, d17       \n\t"
		"vmul.F32  d2, d9, d16       \n\t"
		"vmul.F32  d3, d9, d17       \n\t"
		"vpadd.F32 d4, d0, d1        \n\t"
		"vpadd.F32 d5, d2, d3        \n\t"
		"vst1.32 { d4 },  [%[dst1]]  \n\t"
		"vst1.32 { d5 },  [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x2x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x2x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.2S },   [%[src2]]  \n\t"
		"ld1 { v4.2S },   [%[src11]] \n\t"
		"ld1 { v5.2S },   [%[src12]] \n\t"
		"ld1 { v6.2S },   [%[src13]] \n\t"
		"fmul  v4.2S, v4.2S, v8.2S   \n\t"
		"fmul  v5.2S, v5.2S, v8.2S   \n\t"
		"fmul  v6.2S, v6.2S, v8.2S   \n\t"
		"faddp v0.2S, v4.2S, v5.2S   \n\t"
		"faddp v1.2S, v6.2S, v6.2S   \n\t"
		"st1 { v0.S }[0], [%[dst1]]  \n\t"
		"st1 { v0.S }[1], [%[dst2]]  \n\t"
		"st1 { v1.S }[0], [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16 },    [%[src2]]  \n\t"
		"vld1.32 { d8 },     [%[src11]] \n\t"
		"vld1.32 { d9 },     [%[src12]] \n\t"
		"vld1.32 { d10 },    [%[src13]] \n\t"
		"vmul.F32   d8,  d8, d16        \n\t"
		"vmul.F32   d9,  d9, d16        \n\t"
		"vmul.F32  d10, d10, d16        \n\t"
		"vpadd.F32 d12,  d8,  d9        \n\t"
		"vpadd.F32 d13, d10, d10        \n\t"
		"vst1.32 { d12[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d12[1] }, [%[dst2]]  \n\t"
		"vst1.32 { d13[0] }, [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S },  [%[src11]]   \n\t"
		"ld1 { v5.2S },  [%[src12]]   \n\t"
		"ld1 { v6.2S },  [%[src13]]   \n\t"
		"ld1 { v8.2S },  [%[src21]]   \n\t"
		"ld1 { v9.2S },  [%[src22]]   \n\t"
		"fmul   v0.2S,  v4.2S,  v8.2S \n\t"
		"fmul   v1.2S,  v4.2S,  v9.2S \n\t"
		"fmul   v2.2S,  v5.2S,  v8.2S \n\t"
		"fmul   v3.2S,  v5.2S,  v9.2S \n\t"
		"fmul  v12.2S,  v6.2S,  v8.2S \n\t"
		"fmul  v13.2S,  v6.2S,  v9.2S \n\t"
		"faddp v14.2S,  v0.2S,  v1.2S \n\t"
		"faddp v15.2S,  v2.2S,  v3.2S \n\t"
		"faddp v16.2S, v12.2S, v13.2S \n\t"
		"st1 { v14.2S }, [%[dst1]]    \n\t"
		"st1 { v15.2S }, [%[dst2]]    \n\t"
		"st1 { v16.2S }, [%[dst3]]    \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src11]] \n\t"
		"vld1.32 { d9 },  [%[src12]] \n\t"
		"vld1.32 { d10 }, [%[src13]] \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmul.F32   d0,  d8, d16     \n\t"
		"vmul.F32   d1,  d8, d17     \n\t"
		"vmul.F32   d2,  d9, d16     \n\t"
		"vmul.F32   d3,  d9, d17     \n\t"
		"vmul.F32   d4, d10, d16     \n\t"
		"vmul.F32   d5, d10, d17     \n\t"
		"vpadd.F32 d11,  d0,  d1     \n\t"
		"vpadd.F32 d12,  d2,  d3     \n\t"
		"vpadd.F32 d13,  d4,  d5     \n\t"
		"vst1.32 { d11 }, [%[dst1]]  \n\t"
		"vst1.32 { d12 }, [%[dst2]]  \n\t"
		"vst1.32 { d13 }, [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x2x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x3x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x3x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S, v6.S }[0], [%[src1]]  \n\t"
		"ld3 { v8.S,v9.S,v10.S }[0], [%[src21]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[1], [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]            \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]            \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]            \n\t"
		"st1 { v0.2S },              [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 { d8[0],d9[0],d10[0] },   [%[src1]]  \n\t"
		"vld3.32 { d16[0],d17[0],d18[0] }, [%[src21]] \n\t"
		"vld3.32 { d16[1],d17[1],d18[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                     \n\t"
		"vmla.F32 d0, d17,  d9[0]                     \n\t"
		"vmla.F32 d0, d18, d10[0]                     \n\t"
		"vst1.32 { d0 },                   [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2),
		  [src22]"r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_1x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x3x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x3x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S, v6.S }[0], [%[src11]] \n\t"
		"ld3 { v4.S,v5.S, v6.S }[1], [%[src12]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[0], [%[src21]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[1], [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]            \n\t"
		"fmul v1.2S,  v8.2S, v4.S[1]            \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]            \n\t"
		"fmla v1.2S,  v9.2S, v5.S[1]            \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]            \n\t"
		"fmla v1.2S, v10.2S, v6.S[1]            \n\t"
		"st1 { v0.2S },              [%[dst1]]  \n\t"
		"st1 { v1.2S },              [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 { d8[0],d9[0],d10[0] },   [%[src11]] \n\t"
		"vld3.32 { d8[1],d9[1],d10[1] },   [%[src12]] \n\t"
		"vld3.32 { d16[0],d17[0],d18[0] }, [%[src21]] \n\t"
		"vld3.32 { d16[1],d17[1],d18[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                     \n\t"
		"vmul.F32 d1, d16,  d8[1]                     \n\t"
		"vmla.F32 d0, d17,  d9[0]                     \n\t"
		"vmla.F32 d1, d17,  d9[1]                     \n\t"
		"vmla.F32 d0, d18, d10[0]                     \n\t"
		"vmla.F32 d1, d18, d10[1]                     \n\t"
		"vst1.32 { d0 },                   [%[dst1]]  \n\t"
		"vst1.32 { d1 },                   [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_2x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x3x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x3x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S, v6.S }[0], [%[src11]] \n\t"
		"ld3 { v4.S,v5.S, v6.S }[1], [%[src12]] \n\t"
		"ld3 { v4.S,v5.S, v6.S }[2], [%[src13]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[0], [%[src21]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[1], [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]            \n\t"
		"fmul v1.2S,  v8.2S, v4.S[1]            \n\t"
		"fmul v2.2S,  v8.2S, v4.S[2]            \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]            \n\t"
		"fmla v1.2S,  v9.2S, v5.S[1]            \n\t"
		"fmla v2.2S,  v9.2S, v5.S[2]            \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]            \n\t"
		"fmla v1.2S, v10.2S, v6.S[1]            \n\t"
		"fmla v2.2S, v10.2S, v6.S[2]            \n\t"
		"st1 { v0.2S },              [%[dst1]]  \n\t"
		"st1 { v1.2S },              [%[dst2]]  \n\t"
		"st1 { v2.2S },              [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 {  d8[0], d9[0],d10[0] }, [%[src11]] \n\t"
		"vld3.32 {  d8[1], d9[1],d10[1] }, [%[src12]] \n\t"
		"vld3.32 { d11[0],d12[0],d13[0] }, [%[src13]] \n\t"
		"vld3.32 { d16[0],d17[0],d18[0] }, [%[src21]] \n\t"
		"vld3.32 { d16[1],d17[1],d18[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                     \n\t"
		"vmul.F32 d1, d16,  d8[1]                     \n\t"
		"vmul.F32 d2, d16, d11[0]                     \n\t"
		"vmla.F32 d0, d17,  d9[0]                     \n\t"
		"vmla.F32 d1, d17,  d9[1]                     \n\t"
		"vmla.F32 d2, d17, d12[0]                     \n\t"
		"vmla.F32 d0, d18, d10[0]                     \n\t"
		"vmla.F32 d1, d18, d10[1]                     \n\t"
		"vmla.F32 d2, d18, d13[0]                     \n\t"
		"vst1.32 { d0 },                   [%[dst1]]  \n\t"
		"vst1.32 { d1 },                   [%[dst2]]  \n\t"
		"vst1.32 { d2 },                   [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_3x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_4x4x1x4(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.4S },   [%[src21]] \n\t"
		"ld1 { v4.4S },   [%[src11]] \n\t"
		"ld1 { v5.4S },   [%[src12]] \n\t"
		"ld1 { v6.4S },   [%[src13]] \n\t"
		"ld1 { v7.4S },   [%[src14]] \n\t"
		"fmul  v4.4S, v4.4S, v8.4S   \n\t"
		"fmul  v5.4S, v5.4S, v8.4S   \n\t"
		"fmul  v6.4S, v6.4S, v8.4S   \n\t"
		"fmul  v7.4S, v7.4S, v8.4S   \n\t"
		"faddp v2.4S, v4.4S, v5.4S   \n\t"
		"faddp v3.4S, v6.4S, v7.4S   \n\t"
		"faddp v9.4S, v2.4S, v3.4S   \n\t"
		"st1 { v9.S }[0], [%[dst1]]  \n\t"
		"st1 { v9.S }[1], [%[dst2]]  \n\t"
		"st1 { v9.S }[2], [%[dst3]]  \n\t"
		"st1 { v9.S }[3], [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { q8 },     [%[src21]] \n\t"
		"vld1.32 { q4 },     [%[src11]] \n\t"
		"vld1.32 { q5 },     [%[src12]] \n\t"
		"vld1.32 { q6 },     [%[src13]] \n\t"
		"vld1.32 { q7 },     [%[src14]] \n\t"
		"vmul.F32   q4,  q4,  q8        \n\t"
		"vmul.F32   q5,  q5,  q8        \n\t"
		"vmul.F32   q6,  q6,  q8        \n\t"
		"vmul.F32   q7,  q7,  q8        \n\t"
		"vpadd.F32  d4,  d8,  d9        \n\t"
		"vpadd.F32  d5, d10, d11        \n\t"
		"vpadd.F32  d6, d12, d13        \n\t"
		"vpadd.F32  d7, d14, d15        \n\t"
		"vpadd.F32 d18,  d4,  d5        \n\t"
		"vpadd.F32 d19,  d6,  d7        \n\t"
		"vst1.32 { d18[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d18[1] }, [%[dst2]]  \n\t"
		"vst1.32 { d19[0] }, [%[dst3]]  \n\t"
		"vst1.32 { d19[1] }, [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmmt_f32f32f32_4x4x4x4(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                   aBaseSrc2,
	vipm_int                        aYStrideSrc2
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v12.4S }, [%[src21]]   \n\t"
		"ld1 { v13.4S }, [%[src22]]   \n\t"
		"ld1 { v14.4S }, [%[src23]]   \n\t"
		"ld1 { v15.4S }, [%[src24]]   \n\t"
		"ld1 {  v8.4S }, [%[src11]]   \n\t"
		"ld1 {  v9.4S }, [%[src12]]   \n\t"
		"fmul   v0.4S,  v8.4S, v12.4S \n\t"
		"fmul   v1.4S,  v8.4S, v12.4S \n\t"
		"fmul   v2.4S,  v8.4S, v14.4S \n\t"
		"fmul   v3.4S,  v8.4S, v15.4S \n\t"
		"ld1 { v10.4S }, [%[src13]]   \n\t"
		"fmul   v4.4S,  v9.4S, v12.4S \n\t"
		"fmul   v5.4S,  v9.4S, v13.4S \n\t"
		"fmul   v6.4S,  v9.4S, v14.4S \n\t"
		"fmul   v7.4S,  v9.4S, v15.4S \n\t"
		"ld1 { v11.4S }, [%[src14]]   \n\t"
		"fmul  v16.4S, v10.4S, v12.4S \n\t"
		"fmul  v17.4S, v10.4S, v13.4S \n\t"
		"fmul  v18.4S, v10.4S, v14.4S \n\t"
		"fmul  v19.4S, v10.4S, v15.4S \n\t"
		"fmul  v20.4S, v11.4S, v12.4S \n\t"
		"fmul  v21.4S, v11.4S, v13.4S \n\t"
		"fmul  v22.4S, v11.4S, v14.4S \n\t"
		"fmul  v23.4S, v11.4S, v15.4S \n\t"
		"faddp  v8.4S,  v0.4S,  v1.4S \n\t"
		"faddp  v9.4S,  v2.4S,  v3.4S \n\t"
		"faddp v10.4S,  v4.4S,  v5.4S \n\t"
		"faddp v11.4S,  v6.4S,  v7.4S \n\t"
		"faddp v12.4S, v16.4S, v17.4S \n\t"
		"faddp v13.4S, v18.4S, v19.4S \n\t"
		"faddp v14.4S, v20.4S, v21.4S \n\t"
		"faddp v15.4S, v22.4S, v23.4S \n\t"
		"faddp  v0.4S,  v8.4S,  v9.4S \n\t"
		"faddp  v1.4S, v10.4S, v11.4S \n\t"
		"faddp  v2.4S, v12.4S, v13.4S \n\t"
		"st1 {  v0.4S }, [%[dst1]]    \n\t"
		"faddp  v3.4S, v14.4S, v15.4S \n\t"
		"st1 {  v1.4S }, [%[dst2]]    \n\t"
		"st1 {  v2.4S }, [%[dst3]]    \n\t"
		"st1 {  v3.4S }, [%[dst4]]    \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2),
		  [src24] "r"((char const *)aBaseSrc2 + 3 * aYStrideSrc2)
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { q12 }, [%[src21]] \n\t"
		"vld1.32 { q13 }, [%[src22]] \n\t"
		"vld1.32 { q14 }, [%[src23]] \n\t"
		"vld1.32 { q15 }, [%[src24]] \n\t"
		"vld1.32 { q8 },  [%[src11]] \n\t"
		"vld1.32 { q9 },  [%[src12]] \n\t"
		"vmul.F32   d0, d16, d24     \n\t"
		"vmul.F32   d1, d16, d26     \n\t"
		"vmul.F32   d2, d16, d28     \n\t"
		"vmul.F32   d3, d16, d30     \n\t"
		"vld1.32 { q10 }, [%[src13]] \n\t"
		"vmul.F32   d4, d18, d24     \n\t"
		"vmul.F32   d5, d18, d26     \n\t"
		"vmul.F32   d6, d18, d28     \n\t"
		"vmul.F32   d7, d18, d30     \n\t"
		"vld1.32 { q11 }, [%[src14]] \n\t"
		"vmul.F32   d8, d20, d24     \n\t"
		"vmul.F32   d9, d20, d26     \n\t"
		"vmul.F32  d10, d20, d28     \n\t"
		"vmul.F32  d11, d20, d30     \n\t"
		"vmul.F32  d12, d22, d24     \n\t"
		"vmul.F32  d13, d22, d26     \n\t"
		"vmul.F32  d14, d22, d28     \n\t"
		"vmul.F32  d15, d22, d30     \n\t"
		"vmla.F32   d0, d17, d25     \n\t"
		"vmla.F32   d1, d17, d27     \n\t"
		"vmla.F32   d2, d17, d29     \n\t"
		"vmla.F32   d3, d17, d31     \n\t"
		"vmla.F32   d4, d19, d25     \n\t"
		"vmla.F32   d5, d19, d27     \n\t"
		"vmla.F32   d6, d19, d29     \n\t"
		"vmla.F32   d7, d19, d31     \n\t"
		"vmla.F32   d8, d21, d25     \n\t"
		"vmla.F32   d9, d21, d27     \n\t"
		"vmla.F32  d10, d21, d29     \n\t"
		"vmla.F32  d11, d21, d31     \n\t"
		"vmla.F32  d12, d23, d25     \n\t"
		"vmla.F32  d13, d23, d27     \n\t"
		"vmla.F32  d14, d23, d29     \n\t"
		"vmla.F32  d15, d23, d31     \n\t"
		"vpadd.F32 d16,  d0,  d1     \n\t"
		"vpadd.F32 d17,  d2,  d3     \n\t"
		"vpadd.F32 d18,  d4,  d5     \n\t"
		"vpadd.F32 d19,  d6,  d7     \n\t"
		"vpadd.F32 d20,  d8,  d9     \n\t"
		"vpadd.F32 d21, d10, d11     \n\t"
		"vst1.32 { q8 },  [%[dst1]]  \n\t"
		"vpadd.F32 d22, d12, d13     \n\t"
		"vpadd.F32 d23, d14, d15     \n\t"
		"vst1.32 { q9 },  [%[dst2]]  \n\t"
		"vst1.32 { q10 }, [%[dst3]]  \n\t"
		"vst1.32 { q11 }, [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2),
		  [src22] "r"((char const *)aBaseSrc2 + aYStrideSrc2),
		  [src23] "r"((char const *)aBaseSrc2 + 2 * aYStrideSrc2),
		  [src24] "r"((char const *)aBaseSrc2 + 3 * aYStrideSrc2)
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_elmat_f32(aBaseSrc2, aYStrideSrc2, 3, 3);
# endif
	return (VipmK_SUCCESS);
}

/* ========================================================================= */

/*
 * mulmvn
 */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x1x1x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x1x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.S }[0], [%[src1]]  \n\t"
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"st1 { v0.2S },   [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2List[0]),
		  [src22]"r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8[0] },  [%[src1]]  \n\t"
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16, d8[0]        \n\t"
		"vst1.32 { d0 },     [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2List[0]),
		  [src22]"r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x1x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x1x1x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x1x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"ld1 { v4.S }[0], [%[src11]] \n\t"
		"ld1 { v4.S }[1], [%[src12]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"fmul v1.2S, v8.2S, v4.S[1]  \n\t"
		"st1 { v0.2S },   [%[dst1]]  \n\t"
		"st1 { v1.2S },   [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vld1.32 { d8[0] },  [%[src11]] \n\t"
		"vld1.32 { d8[1] },  [%[src12]] \n\t"
		"vmul.F32 d0, d16, d8[0]        \n\t"
		"vmul.F32 d1, d16, d8[1]        \n\t"
		"vst1.32 { d0 },     [%[dst1]]  \n\t"
		"vst1.32 { d1 },     [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x1x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x1x1x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x1x2x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.S }[0], [%[src21]] \n\t"
		"ld1 { v8.S }[1], [%[src22]] \n\t"
		"ld1 { v4.S }[0], [%[src11]] \n\t"
		"ld1 { v4.S }[1], [%[src12]] \n\t"
		"ld1 { v4.S }[2], [%[src13]] \n\t"
		"fmul v0.2S, v8.2S, v4.S[0]  \n\t"
		"fmul v1.2S, v8.2S, v4.S[1]  \n\t"
		"fmul v2.2S, v8.2S, v4.S[2]  \n\t"
		"st1 { v0.2S },   [%[dst1]]  \n\t"
		"st1 { v1.2S },   [%[dst2]]  \n\t"
		"st1 { v2.2S },   [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16[0] }, [%[src21]] \n\t"
		"vld1.32 { d16[1] }, [%[src22]] \n\t"
		"vld1.32 { d8[0] },  [%[src11]] \n\t"
		"vld1.32 { d8[1] },  [%[src12]] \n\t"
		"vld1.32 { d9[0] },  [%[src13]] \n\t"
		"vmul.F32 d0, d16, d8[0]        \n\t"
		"vmul.F32 d1, d16, d8[1]        \n\t"
		"vmul.F32 d2, d16, d9[0]        \n\t"
		"vst1.32 { d0 },     [%[dst1]]  \n\t"
		"vst1.32 { d1 },     [%[dst2]]  \n\t"
		"vst1.32 { d2 },     [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x1x3x1(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x2x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S },   [%[src1]] \n\t"
		"ld1 { v8.2S },   [%[src2]] \n\t"
		"fmul  v1.2S, v8.2S, v4.2S  \n\t"
		"faddp v0.2S, v1.2S, v1.2S  \n\t"
		"st1 { v0.S }[0], [%[dst]]  \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src2] "r"(aBaseSrc2List[0])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },    [%[src1]] \n\t"
		"vld1.32 { d16 },   [%[src2]] \n\t"
		"vmul.F32  d1, d16, d8        \n\t"
		"vpadd.F32 d0,  d1, d1        \n\t"
		"vst1.32 { d0[0] }, [%[dst]]  \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src2] "r"(aBaseSrc2List[0])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S },   [%[src1]]  \n\t"
		"ld1 { v8.2S },   [%[src21]] \n\t"
		"ld1 { v8.D }[1], [%[src22]] \n\t"
		"mov v4.D[1], v4.D[0]        \n\t"
		"fmul  v0.4S, v8.4S, v4.4S   \n\t"
		"faddp v1.4S, v0.4S, v0.4S   \n\t"
		"st1 { v1.2S },  [%[dst]]    \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2List[0]),
		  [src22]"r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src1]]  \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmov      d9, d8            \n\t"
		"vmul.F32  q0, q8, q4        \n\t"
		"vpadd.F32 d2, d0, d1        \n\t"
		"vst1.32 { d2 },  [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2List[0]),
		  [src22]"r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x2x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x2x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.2S },   [%[src2]]  \n\t"
		"ld1 { v4.2S },   [%[src11]] \n\t"
		"ld1 { v5.2S },   [%[src12]] \n\t"
		"fmul  v4.2S, v4.2S, v8.2S   \n\t"
		"fmul  v5.2S, v5.2S, v8.2S   \n\t"
		"faddp v6.2S, v4.2S, v5.2S   \n\t"
		"st1 { v6.S }[0], [%[dst1]]  \n\t"
		"st1 { v6.S }[1], [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2List[0])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16 },    [%[src2]]  \n\t"
		"vld1.32 { d8 },     [%[src11]] \n\t"
		"vld1.32 { d9 },     [%[src12]] \n\t"
		"vmul.F32   d8, d8, d16         \n\t"
		"vmul.F32   d9, d9, d16         \n\t"
		"vpadd.F32 d10, d8,  d9         \n\t"
		"vst1.32 { d10[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d10[1] }, [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2List[0])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S }, [%[src11]] \n\t"
		"ld1 { v5.2S }, [%[src12]] \n\t"
		"ld1 { v8.2S }, [%[src21]] \n\t"
		"ld1 { v9.2S }, [%[src22]] \n\t"
		"fmul  v0.2S, v4.2S, v8.2S \n\t"
		"fmul  v1.2S, v4.2S, v9.2S \n\t"
		"fmul  v2.2S, v5.2S, v8.2S \n\t"
		"fmul  v3.2S, v5.2S, v9.2S \n\t"
		"faddp v6.2S, v0.2S, v1.2S \n\t"
		"faddp v7.2S, v2.2S, v3.2S \n\t"
		"st1 { v6.2S }, [%[dst1]]  \n\t"
		"st1 { v7.2S }, [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src11]] \n\t"
		"vld1.32 { d9 },  [%[src12]] \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmul.F32  d0, d8, d16       \n\t"
		"vmul.F32  d1, d8, d17       \n\t"
		"vmul.F32  d2, d9, d16       \n\t"
		"vmul.F32  d3, d9, d17       \n\t"
		"vpadd.F32 d4, d0, d1        \n\t"
		"vpadd.F32 d5, d2, d3        \n\t"
		"vst1.32 { d4 },  [%[dst1]]  \n\t"
		"vst1.32 { d5 },  [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x2x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x2x1x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.2S },   [%[src2]]  \n\t"
		"ld1 { v4.2S },   [%[src11]] \n\t"
		"ld1 { v5.2S },   [%[src12]] \n\t"
		"ld1 { v6.2S },   [%[src13]] \n\t"
		"fmul  v4.2S, v4.2S, v8.2S   \n\t"
		"fmul  v5.2S, v5.2S, v8.2S   \n\t"
		"fmul  v6.2S, v6.2S, v8.2S   \n\t"
		"faddp v0.2S, v4.2S, v5.2S   \n\t"
		"faddp v1.2S, v6.2S, v6.2S   \n\t"
		"st1 { v0.S }[0], [%[dst1]]  \n\t"
		"st1 { v0.S }[1], [%[dst2]]  \n\t"
		"st1 { v1.S }[0], [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2List[0])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d16 },    [%[src2]]  \n\t"
		"vld1.32 { d8 },     [%[src11]] \n\t"
		"vld1.32 { d9 },     [%[src12]] \n\t"
		"vld1.32 { d10 },    [%[src13]] \n\t"
		"vmul.F32   d8,  d8, d16        \n\t"
		"vmul.F32   d9,  d9, d16        \n\t"
		"vmul.F32  d10, d10, d16        \n\t"
		"vpadd.F32 d12,  d8,  d9        \n\t"
		"vpadd.F32 d13, d10, d10        \n\t"
		"vst1.32 { d12[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d12[1] }, [%[dst2]]  \n\t"
		"vst1.32 { d13[0] }, [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src2]  "r"(aBaseSrc2List[0])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x2x2x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v4.2S },  [%[src11]]   \n\t"
		"ld1 { v5.2S },  [%[src12]]   \n\t"
		"ld1 { v6.2S },  [%[src13]]   \n\t"
		"ld1 { v8.2S },  [%[src21]]   \n\t"
		"ld1 { v9.2S },  [%[src22]]   \n\t"
		"fmul   v0.2S,  v4.2S,  v8.2S \n\t"
		"fmul   v1.2S,  v4.2S,  v9.2S \n\t"
		"fmul   v2.2S,  v5.2S,  v8.2S \n\t"
		"fmul   v3.2S,  v5.2S,  v9.2S \n\t"
		"fmul  v12.2S,  v6.2S,  v8.2S \n\t"
		"fmul  v13.2S,  v6.2S,  v9.2S \n\t"
		"faddp v14.2S,  v0.2S,  v1.2S \n\t"
		"faddp v15.2S,  v2.2S,  v3.2S \n\t"
		"faddp v16.2S, v12.2S, v13.2S \n\t"
		"st1 { v14.2S }, [%[dst1]]    \n\t"
		"st1 { v15.2S }, [%[dst2]]    \n\t"
		"st1 { v16.2S }, [%[dst3]]    \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { d8 },  [%[src11]] \n\t"
		"vld1.32 { d9 },  [%[src12]] \n\t"
		"vld1.32 { d10 }, [%[src13]] \n\t"
		"vld1.32 { d16 }, [%[src21]] \n\t"
		"vld1.32 { d17 }, [%[src22]] \n\t"
		"vmul.F32   d0,  d8, d16     \n\t"
		"vmul.F32   d1,  d8, d17     \n\t"
		"vmul.F32   d2,  d9, d16     \n\t"
		"vmul.F32   d3,  d9, d17     \n\t"
		"vmul.F32   d4, d10, d16     \n\t"
		"vmul.F32   d5, d10, d17     \n\t"
		"vpadd.F32 d11,  d0,  d1     \n\t"
		"vpadd.F32 d12,  d2,  d3     \n\t"
		"vpadd.F32 d13,  d4,  d5     \n\t"
		"vst1.32 { d11 }, [%[dst1]]  \n\t"
		"vst1.32 { d12 }, [%[dst2]]  \n\t"
		"vst1.32 { d13 }, [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x2x3x2(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x3x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x3x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S, v6.S }[0], [%[src1]]  \n\t"
		"ld3 { v8.S,v9.S,v10.S }[0], [%[src21]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[1], [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]            \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]            \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]            \n\t"
		"st1 { v0.2S },              [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2List[0]),
		  [src22]"r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 { d8[0],d9[0],d10[0] },   [%[src1]]  \n\t"
		"vld3.32 { d16[0],d17[0],d18[0] }, [%[src21]] \n\t"
		"vld3.32 { d16[1],d17[1],d18[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                     \n\t"
		"vmla.F32 d0, d17,  d9[0]                     \n\t"
		"vmla.F32 d0, d18, d10[0]                     \n\t"
		"vst1.32 { d0 },                   [%[dst]]   \n\t"

		:
		: [dst]  "r"(aBaseDst),
		  [src1] "r"(aBaseSrc1),
		  [src21]"r"(aBaseSrc2List[0]),
		  [src22]"r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_1x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	*(*(float **)&aBaseDst)++ =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x3x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x3x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S, v6.S }[0], [%[src11]] \n\t"
		"ld3 { v4.S,v5.S, v6.S }[1], [%[src12]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[0], [%[src21]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[1], [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]            \n\t"
		"fmul v1.2S,  v8.2S, v4.S[1]            \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]            \n\t"
		"fmla v1.2S,  v9.2S, v5.S[1]            \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]            \n\t"
		"fmla v1.2S, v10.2S, v6.S[1]            \n\t"
		"st1 { v0.2S },              [%[dst1]]  \n\t"
		"st1 { v1.2S },              [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 { d8[0],d9[0],d10[0] },   [%[src11]] \n\t"
		"vld3.32 { d8[1],d9[1],d10[1] },   [%[src12]] \n\t"
		"vld3.32 { d16[0],d17[0],d18[0] }, [%[src21]] \n\t"
		"vld3.32 { d16[1],d17[1],d18[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                     \n\t"
		"vmul.F32 d1, d16,  d8[1]                     \n\t"
		"vmla.F32 d0, d17,  d9[0]                     \n\t"
		"vmla.F32 d1, d17,  d9[1]                     \n\t"
		"vmla.F32 d0, d18, d10[0]                     \n\t"
		"vmla.F32 d1, d18, d10[1]                     \n\t"
		"vst1.32 { d0 },                   [%[dst1]]  \n\t"
		"vst1.32 { d1 },                   [%[dst2]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_2x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x3x1x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x3x2x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld3 { v4.S,v5.S, v6.S }[0], [%[src11]] \n\t"
		"ld3 { v4.S,v5.S, v6.S }[1], [%[src12]] \n\t"
		"ld3 { v4.S,v5.S, v6.S }[2], [%[src13]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[0], [%[src21]] \n\t"
		"ld3 { v8.S,v9.S,v10.S }[1], [%[src22]] \n\t"
		"fmul v0.2S,  v8.2S, v4.S[0]            \n\t"
		"fmul v1.2S,  v8.2S, v4.S[1]            \n\t"
		"fmul v2.2S,  v8.2S, v4.S[2]            \n\t"
		"fmla v0.2S,  v9.2S, v5.S[0]            \n\t"
		"fmla v1.2S,  v9.2S, v5.S[1]            \n\t"
		"fmla v2.2S,  v9.2S, v5.S[2]            \n\t"
		"fmla v0.2S, v10.2S, v6.S[0]            \n\t"
		"fmla v1.2S, v10.2S, v6.S[1]            \n\t"
		"fmla v2.2S, v10.2S, v6.S[2]            \n\t"
		"st1 { v0.2S },              [%[dst1]]  \n\t"
		"st1 { v1.2S },              [%[dst2]]  \n\t"
		"st1 { v2.2S },              [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld3.32 {  d8[0], d9[0],d10[0] }, [%[src11]] \n\t"
		"vld3.32 {  d8[1], d9[1],d10[1] }, [%[src12]] \n\t"
		"vld3.32 { d11[0],d12[0],d13[0] }, [%[src13]] \n\t"
		"vld3.32 { d16[0],d17[0],d18[0] }, [%[src21]] \n\t"
		"vld3.32 { d16[1],d17[1],d18[1] }, [%[src22]] \n\t"
		"vmul.F32 d0, d16,  d8[0]                     \n\t"
		"vmul.F32 d1, d16,  d8[1]                     \n\t"
		"vmul.F32 d2, d16, d11[0]                     \n\t"
		"vmla.F32 d0, d17,  d9[0]                     \n\t"
		"vmla.F32 d1, d17,  d9[1]                     \n\t"
		"vmla.F32 d2, d17, d12[0]                     \n\t"
		"vmla.F32 d0, d18, d10[0]                     \n\t"
		"vmla.F32 d1, d18, d10[1]                     \n\t"
		"vmla.F32 d2, d18, d13[0]                     \n\t"
		"vst1.32 { d0 },                   [%[dst1]]  \n\t"
		"vst1.32 { d1 },                   [%[dst2]]  \n\t"
		"vst1.32 { d2 },                   [%[dst3]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
# endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_3x3x3x3(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2);

	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_4x4x1x4(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v8.4S },   [%[src21]] \n\t"
		"ld1 { v4.4S },   [%[src11]] \n\t"
		"ld1 { v5.4S },   [%[src12]] \n\t"
		"ld1 { v6.4S },   [%[src13]] \n\t"
		"ld1 { v7.4S },   [%[src14]] \n\t"
		"fmul  v4.4S, v4.4S, v8.4S   \n\t"
		"fmul  v5.4S, v5.4S, v8.4S   \n\t"
		"fmul  v6.4S, v6.4S, v8.4S   \n\t"
		"fmul  v7.4S, v7.4S, v8.4S   \n\t"
		"faddp v2.4S, v4.4S, v5.4S   \n\t"
		"faddp v3.4S, v6.4S, v7.4S   \n\t"
		"faddp v9.4S, v2.4S, v3.4S   \n\t"
		"st1 { v9.S }[0], [%[dst1]]  \n\t"
		"st1 { v9.S }[1], [%[dst2]]  \n\t"
		"st1 { v9.S }[2], [%[dst3]]  \n\t"
		"st1 { v9.S }[3], [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { q8 },     [%[src21]] \n\t"
		"vld1.32 { q4 },     [%[src11]] \n\t"
		"vld1.32 { q5 },     [%[src12]] \n\t"
		"vld1.32 { q6 },     [%[src13]] \n\t"
		"vld1.32 { q7 },     [%[src14]] \n\t"
		"vmul.F32   q4,  q4,  q8        \n\t"
		"vmul.F32   q5,  q5,  q8        \n\t"
		"vmul.F32   q6,  q6,  q8        \n\t"
		"vmul.F32   q7,  q7,  q8        \n\t"
		"vpadd.F32  d4,  d8,  d9        \n\t"
		"vpadd.F32  d5, d10, d11        \n\t"
		"vpadd.F32  d6, d12, d13        \n\t"
		"vpadd.F32  d7, d14, d15        \n\t"
		"vpadd.F32 d18,  d4,  d5        \n\t"
		"vpadd.F32 d19,  d6,  d7        \n\t"
		"vst1.32 { d18[0] }, [%[dst1]]  \n\t"
		"vst1.32 { d18[1] }, [%[dst2]]  \n\t"
		"vst1.32 { d19[0] }, [%[dst3]]  \n\t"
		"vst1.32 { d19[1] }, [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
#endif
	return (VipmK_SUCCESS);
}

/* ------------------------------------------------------------------------- */

_VIPM_INLINE bo_status_t
vipma_mulmvn_f32f32f32_4x4x4x4(
	vipm_mutptr_t                   aBaseDst,
	vipm_int                        aYStrideDst,
	vipm_imuptr_t                   aBaseSrc1,
	vipm_int                        aYStrideSrc1,
	vipm_imuptr_t                  *aBaseSrc2List
	)
{
# if !__TEST_C_IMPL && __VIPM_USE_SIMD && defined(_A_ARM) && \
		(defined(_ICC_VER) || defined(_GCC_VER))
#  ifdef _A_ARM64
	__asm__ __volatile__ (
		"ld1 { v12.4S }, [%[src21]]   \n\t"
		"ld1 { v13.4S }, [%[src22]]   \n\t"
		"ld1 { v14.4S }, [%[src23]]   \n\t"
		"ld1 { v15.4S }, [%[src24]]   \n\t"
		"ld1 {  v8.4S }, [%[src11]]   \n\t"
		"ld1 {  v9.4S }, [%[src12]]   \n\t"
		"fmul   v0.4S,  v8.4S, v12.4S \n\t"
		"fmul   v1.4S,  v8.4S, v12.4S \n\t"
		"fmul   v2.4S,  v8.4S, v14.4S \n\t"
		"fmul   v3.4S,  v8.4S, v15.4S \n\t"
		"ld1 { v10.4S }, [%[src13]]   \n\t"
		"fmul   v4.4S,  v9.4S, v12.4S \n\t"
		"fmul   v5.4S,  v9.4S, v13.4S \n\t"
		"fmul   v6.4S,  v9.4S, v14.4S \n\t"
		"fmul   v7.4S,  v9.4S, v15.4S \n\t"
		"ld1 { v11.4S }, [%[src14]]   \n\t"
		"fmul  v16.4S, v10.4S, v12.4S \n\t"
		"fmul  v17.4S, v10.4S, v13.4S \n\t"
		"fmul  v18.4S, v10.4S, v14.4S \n\t"
		"fmul  v19.4S, v10.4S, v15.4S \n\t"
		"fmul  v20.4S, v11.4S, v12.4S \n\t"
		"fmul  v21.4S, v11.4S, v13.4S \n\t"
		"fmul  v22.4S, v11.4S, v14.4S \n\t"
		"fmul  v23.4S, v11.4S, v15.4S \n\t"
		"faddp  v8.4S,  v0.4S,  v1.4S \n\t"
		"faddp  v9.4S,  v2.4S,  v3.4S \n\t"
		"faddp v10.4S,  v4.4S,  v5.4S \n\t"
		"faddp v11.4S,  v6.4S,  v7.4S \n\t"
		"faddp v12.4S, v16.4S, v17.4S \n\t"
		"faddp v13.4S, v18.4S, v19.4S \n\t"
		"faddp v14.4S, v20.4S, v21.4S \n\t"
		"faddp v15.4S, v22.4S, v23.4S \n\t"
		"faddp  v0.4S,  v8.4S,  v9.4S \n\t"
		"faddp  v1.4S, v10.4S, v11.4S \n\t"
		"faddp  v2.4S, v12.4S, v13.4S \n\t"
		"st1 {  v0.4S }, [%[dst1]]    \n\t"
		"faddp  v3.4S, v14.4S, v15.4S \n\t"
		"st1 {  v1.4S }, [%[dst2]]    \n\t"
		"st1 {  v2.4S }, [%[dst3]]    \n\t"
		"st1 {  v3.4S }, [%[dst4]]    \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1]),
		  [src23] "r"(aBaseSrc2List[2]),
		  [src24] "r"(aBaseSrc2List[3])
	);
#  else /* _A_ARM32 */
	__asm__ __volatile__ (
		"vld1.32 { q12 }, [%[src21]] \n\t"
		"vld1.32 { q13 }, [%[src22]] \n\t"
		"vld1.32 { q14 }, [%[src23]] \n\t"
		"vld1.32 { q15 }, [%[src24]] \n\t"
		"vld1.32 { q8 },  [%[src11]] \n\t"
		"vld1.32 { q9 },  [%[src12]] \n\t"
		"vmul.F32   d0, d16, d24     \n\t"
		"vmul.F32   d1, d16, d26     \n\t"
		"vmul.F32   d2, d16, d28     \n\t"
		"vmul.F32   d3, d16, d30     \n\t"
		"vld1.32 { q10 }, [%[src13]] \n\t"
		"vmul.F32   d4, d18, d24     \n\t"
		"vmul.F32   d5, d18, d26     \n\t"
		"vmul.F32   d6, d18, d28     \n\t"
		"vmul.F32   d7, d18, d30     \n\t"
		"vld1.32 { q11 }, [%[src14]] \n\t"
		"vmul.F32   d8, d20, d24     \n\t"
		"vmul.F32   d9, d20, d26     \n\t"
		"vmul.F32  d10, d20, d28     \n\t"
		"vmul.F32  d11, d20, d30     \n\t"
		"vmul.F32  d12, d22, d24     \n\t"
		"vmul.F32  d13, d22, d26     \n\t"
		"vmul.F32  d14, d22, d28     \n\t"
		"vmul.F32  d15, d22, d30     \n\t"
		"vmla.F32   d0, d17, d25     \n\t"
		"vmla.F32   d1, d17, d27     \n\t"
		"vmla.F32   d2, d17, d29     \n\t"
		"vmla.F32   d3, d17, d31     \n\t"
		"vmla.F32   d4, d19, d25     \n\t"
		"vmla.F32   d5, d19, d27     \n\t"
		"vmla.F32   d6, d19, d29     \n\t"
		"vmla.F32   d7, d19, d31     \n\t"
		"vmla.F32   d8, d21, d25     \n\t"
		"vmla.F32   d9, d21, d27     \n\t"
		"vmla.F32  d10, d21, d29     \n\t"
		"vmla.F32  d11, d21, d31     \n\t"
		"vmla.F32  d12, d23, d25     \n\t"
		"vmla.F32  d13, d23, d27     \n\t"
		"vmla.F32  d14, d23, d29     \n\t"
		"vmla.F32  d15, d23, d31     \n\t"
		"vpadd.F32 d16,  d0,  d1     \n\t"
		"vpadd.F32 d17,  d2,  d3     \n\t"
		"vpadd.F32 d18,  d4,  d5     \n\t"
		"vpadd.F32 d19,  d6,  d7     \n\t"
		"vpadd.F32 d20,  d8,  d9     \n\t"
		"vpadd.F32 d21, d10, d11     \n\t"
		"vst1.32 { q8 },  [%[dst1]]  \n\t"
		"vpadd.F32 d22, d12, d13     \n\t"
		"vpadd.F32 d23, d14, d15     \n\t"
		"vst1.32 { q9 },  [%[dst2]]  \n\t"
		"vst1.32 { q10 }, [%[dst3]]  \n\t"
		"vst1.32 { q11 }, [%[dst4]]  \n\t"

		:
		: [dst1]  "r"(aBaseDst),
		  [dst2]  "r"((char *)aBaseDst + aYStrideDst),
		  [dst3]  "r"((char *)aBaseDst + 2 * aYStrideDst),
		  [dst4]  "r"((char *)aBaseDst + 3 * aYStrideDst),
		  [src11] "r"(aBaseSrc1),
		  [src12] "r"((char const *)aBaseSrc1 + aYStrideSrc1),
		  [src13] "r"((char const *)aBaseSrc1 + 2 * aYStrideSrc1),
		  [src14] "r"((char const *)aBaseSrc1 + 3 * aYStrideSrc1),
		  [src21] "r"(aBaseSrc2List[0]),
		  [src22] "r"(aBaseSrc2List[1]),
		  [src23] "r"(aBaseSrc2List[2]),
		  [src24] "r"(aBaseSrc2List[3])
	);
#  endif /* _A_ARM64 */
# else
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 0, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 0, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 1, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 1, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 2, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 2, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 0) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 0, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 1) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 1, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 2) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 2, 3);
	__vipm_mut_elmat_f32(aBaseDst, aYStrideDst, 3, 3) =
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 0) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 0) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 1) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 1) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 2) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 2) +
		__vipm_elmat_f32(aBaseSrc1, aYStrideSrc1, 3, 3) * __vipm_lst_elmat_f32(aBaseSrc2List, 3, 3);
# endif
	return (VipmK_SUCCESS);
}

# undef __vipm_elmat_f32
# undef __vipm_lst_elmat_f32
# undef __vipm_mut_elmat_f32
# undef __TEST_C_IMPL


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
