/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2018 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 * $Id$
 */

/*
 * Simd.sse.h	(V.Sokur)
 * Vipm/Simd.sse.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_SIMD_SSE_H__
# define __VIPM_SIMD_SSE_H__

/******************************************************************************
 * %% BeginSection: includes
 */

# ifdef __VIPM_AVX2_MMINTRIN_HEADER
#  include __VIPM_AVX2_MMINTRIN_HEADER
# else
# ifdef __VIPM_AVX_MMINTRIN_HEADER
#  include __VIPM_AVX_MMINTRIN_HEADER
# else
# endif
# endif

# ifdef __VIPM_SSE4_2_MMINTRIN_HEADER
#  include __VIPM_SSE4_2_MMINTRIN_HEADER
# endif
# ifdef __VIPM_SSE4_1_MMINTRIN_HEADER
#  include __VIPM_SSE4_1_MMINTRIN_HEADER
# endif
# ifdef __VIPM_SSSE3_MMINTRIN_HEADER
#  include __VIPM_SSSE3_MMINTRIN_HEADER
# endif
# ifdef __VIPM_SSE3_MMINTRIN_HEADER
#  include __VIPM_SSE3_MMINTRIN_HEADER
# endif


/******************************************************************************
 * %% BeginSection: type/constant declarations
 */

# define maybe_const /* const */

/*
 * "Vector" Types
 */

/* @{ */

	/*
	 * 8 byte signed/unsigned/float vector
	 */

struct bo_m64_s8 {
	bo_int8_t       v[8];
};
struct bo_m64_s16 {
	bo_int16_t      v[4];
};
struct bo_m64_s32 {
	bo_int32_t      v[2];
};
struct bo_m64_s64 {
	bo_int64_t      v[1];
};
typedef struct bo_m64_s8            __bo_m64_s8_t;
typedef struct bo_m64_s16           __bo_m64_s16_t;
typedef struct bo_m64_s32           __bo_m64_s32_t;
typedef struct bo_m64_s64           __bo_m64_s64_t;

struct bo_m64_u8 {
	bo_uint8_t      v[8];
};
struct bo_m64_u16 {
	bo_uint16_t     v[4];
};
struct bo_m64_u32 {
	bo_uint32_t     v[2];
};
struct bo_m64_u64 {
	bo_uint64_t     v[1];
};
typedef struct bo_m64_u8            __bo_m64_u8_t;
typedef struct bo_m64_u16           __bo_m64_u16_t;
typedef struct bo_m64_u32           __bo_m64_u32_t;
typedef struct bo_m64_u64           __bo_m64_u64_t;

struct bo_m64_f16 {
	bo_float16_t    v[4];
};
struct bo_m64_f32 {
	bo_float32_t    v[2];
};
struct bo_m64_f64 {
	bo_float64_t    v[1];
};
typedef struct bo_m64_f16           __bo_m64_f16_t;
typedef struct bo_m64_f32           __bo_m64_f32_t;
typedef struct bo_m64_f64           __bo_m64_f64_t;


	/*
	 * 16 byte signed/unsigned/float vector
	 */

struct bo_m128_s8 {
	bo_int8_t       v[16];
};
struct bo_m128_s16 {
	bo_int16_t      v[8];
};
struct bo_m128_s32 {
	bo_int32_t      v[4];
};
struct bo_m128_s64 {
	bo_int64_t      v[2];
};
typedef struct bo_m128_s8           __bo_m128_s8_t;
typedef struct bo_m128_s16          __bo_m128_s16_t;
typedef struct bo_m128_s32          __bo_m128_s32_t;
typedef struct bo_m128_s64          __bo_m128_s64_t;

struct bo_m128_u8 {
	bo_uint8_t      v[16];
};
struct bo_m128_u16 {
	bo_uint16_t     v[8];
};
struct bo_m128_u32 {
	bo_uint32_t     v[4];
};
struct bo_m128_u64 {
	bo_uint64_t     v[2];
};
typedef struct bo_m128_u8           __bo_m128_u8_t;
typedef struct bo_m128_u16          __bo_m128_u16_t;
typedef struct bo_m128_u32          __bo_m128_u32_t;
typedef struct bo_m128_u64          __bo_m128_u64_t;

struct bo_m128_f16 {
	bo_float16_t    v[8];
};
struct bo_m128_f32 {
	bo_float32_t    v[4];
};
struct bo_m128_f64 {
	bo_float64_t    v[2];
};
typedef struct bo_m128_f16          __bo_m128_f16_t;
typedef struct bo_m128_f32          __bo_m128_f32_t;
typedef struct bo_m128_f64          __bo_m128_f64_t;



typedef __bo_m64_s8_t               bo_m64_s8_t;
typedef __bo_m64_s16_t              bo_m64_s16_t;
typedef __bo_m64_s32_t              bo_m64_s32_t;
typedef __bo_m64_s64_t              bo_m64_s64_t;
typedef __bo_m64_u8_t               bo_m64_u8_t;
typedef __bo_m64_u16_t              bo_m64_u16_t;
typedef __bo_m64_u32_t              bo_m64_u32_t;
typedef __bo_m64_u64_t              bo_m64_u64_t;
typedef __bo_m64_f16_t              bo_m64_f16_t;
typedef __bo_m64_f32_t              bo_m64_f32_t;
typedef __bo_m64_f64_t              bo_m64_f64_t;

typedef __m128i                     bo_m128_s8_t;
typedef __m128i                     bo_m128_s16_t;
typedef __m128i                     bo_m128_s32_t;
typedef __m128i                     bo_m128_s64_t;
typedef __m128i                     bo_m128_u8_t;
typedef __m128i                     bo_m128_u16_t;
typedef __m128i                     bo_m128_u32_t;
typedef __m128i                     bo_m128_u64_t;
typedef __bo_m128_f16_t             bo_m128_f16_t;
typedef __m128                      bo_m128_f32_t;
typedef __m128d                     bo_m128_f64_t;

/* @} */

/* ------------------------------------------------------------------------- */

/* 2 elements @{ */

	/*
	 * 8 byte signed/unsigned/float vector
	 */

struct bo_m64x2_s8 {
	bo_m64_s8_t     val[2];
};
struct bo_m64x2_s16 {
	bo_m64_s16_t    val[2];
};
struct bo_m64x2_s32 {
	bo_m64_s32_t    val[2];
};
struct bo_m64x2_s64 {
	bo_m64_s64_t    val[2];
};
typedef struct bo_m64x2_s8          bo_m64x2_s8_t;
typedef struct bo_m64x2_s16         bo_m64x2_s16_t;
typedef struct bo_m64x2_s32         bo_m64x2_s32_t;
typedef struct bo_m64x2_s64         bo_m64x2_s64_t;

struct bo_m64x2_u8 {
	bo_m64_u8_t     val[2];
};
struct bo_m64x2_u16 {
	bo_m64_u16_t    val[2];
};
struct bo_m64x2_u32 {
	bo_m64_u32_t    val[2];
};
struct bo_m64x2_u64 {
	bo_m64_u64_t    val[2];
};
typedef struct bo_m64x2_u8          bo_m64x2_u8_t;
typedef struct bo_m64x2_u16         bo_m64x2_u16_t;
typedef struct bo_m64x2_u32         bo_m64x2_u32_t;
typedef struct bo_m64x2_u64         bo_m64x2_u64_t;

struct bo_m64x2_f16 {
	bo_m64_f16_t    val[2];
};
struct bo_m64x2_f32 {
	bo_m64_f32_t    val[2];
};
struct bo_m64x2_f64 {
	bo_m64_f64_t    val[2];
};
typedef struct bo_m64x2_f16         bo_m64x2_f16_t;
typedef struct bo_m64x2_f32         bo_m64x2_f32_t;
typedef struct bo_m64x2_f64         bo_m64x2_f64_t;


	/*
	 * 16 byte signed/unsigned/float vector
	 */

struct bo_m128x2_s8 {
	bo_m128_s8_t    val[2];
};
struct bo_m128x2_s16 {
	bo_m128_s16_t   val[2];
};
struct bo_m128x2_s32 {
	bo_m128_s32_t   val[2];
};
struct bo_m128x2_s64 {
	bo_m128_s64_t   val[2];
};
typedef struct bo_m128x2_s8         bo_m128x2_s8_t;
typedef struct bo_m128x2_s16        bo_m128x2_s16_t;
typedef struct bo_m128x2_s32        bo_m128x2_s32_t;
typedef struct bo_m128x2_s64        bo_m128x2_s64_t;

struct bo_m128x2_u8 {
	bo_m128_u8_t    val[2];
};
struct bo_m128x2_u16 {
	bo_m128_u16_t   val[2];
};
struct bo_m128x2_u32 {
	bo_m128_u32_t   val[2];
};
struct bo_m128x2_u64 {
	bo_m128_u64_t   val[2];
};
typedef struct bo_m128x2_u8         bo_m128x2_u8_t;
typedef struct bo_m128x2_u16        bo_m128x2_u16_t;
typedef struct bo_m128x2_u32        bo_m128x2_u32_t;
typedef struct bo_m128x2_u64        bo_m128x2_u64_t;

struct bo_m128x2_f16 {
	bo_m128_f16_t   val[2];
};
struct bo_m128x2_f32 {
	bo_m128_f32_t   val[2];
};
struct bo_m128x2_f64 {
	bo_m128_f64_t   val[2];
};
typedef struct bo_m128x2_f16        bo_m128x2_f16_t;
typedef struct bo_m128x2_f32        bo_m128x2_f32_t;
typedef struct bo_m128x2_f64        bo_m128x2_f64_t;

/* @} */

/* ------------------------------------------------------------------------- */

/* 3 elements @{ */

	/*
	 * 8 byte signed/unsigned/float vector
	 */

struct bo_m64x3_s8 {
	bo_m64_s8_t     val[3];
};
struct bo_m64x3_s16 {
	bo_m64_s16_t    val[3];
};
struct bo_m64x3_s32 {
	bo_m64_s32_t    val[3];
};
struct bo_m64x3_s64 {
	bo_m64_s64_t    val[3];
};
typedef struct bo_m64x3_s8          bo_m64x3_s8_t;
typedef struct bo_m64x3_s16         bo_m64x3_s16_t;
typedef struct bo_m64x3_s32         bo_m64x3_s32_t;
typedef struct bo_m64x3_s64         bo_m64x3_s64_t;

struct bo_m64x3_u8 {
	bo_m64_u8_t     val[3];
};
struct bo_m64x3_u16 {
	bo_m64_u16_t    val[3];
};
struct bo_m64x3_u32 {
	bo_m64_u32_t    val[3];
};
struct bo_m64x3_u64 {
	bo_m64_u64_t    val[3];
};
typedef struct bo_m64x3_u8          bo_m64x3_u8_t;
typedef struct bo_m64x3_u16         bo_m64x3_u16_t;
typedef struct bo_m64x3_u32         bo_m64x3_u32_t;
typedef struct bo_m64x3_u64         bo_m64x3_u64_t;

struct bo_m64x3_f16 {
	bo_m64_f16_t    val[3];
};
struct bo_m64x3_f32 {
	bo_m64_f32_t    val[3];
};
struct bo_m64x3_f64 {
	bo_m64_f64_t    val[3];
};
typedef struct bo_m64x3_f16         bo_m64x3_f16_t;
typedef struct bo_m64x3_f32         bo_m64x3_f32_t;
typedef struct bo_m64x3_f64         bo_m64x3_f64_t;


	/*
	 * 16 byte signed/unsigned/float vector
	 */

struct bo_m128x3_s8 {
	bo_m128_s8_t    val[3];
};
struct bo_m128x3_s16 {
	bo_m128_s16_t   val[3];
};
struct bo_m128x3_s32 {
	bo_m128_s32_t   val[3];
};
struct bo_m128x3_s64 {
	bo_m128_s64_t   val[3];
};
typedef struct bo_m128x3_s8         bo_m128x3_s8_t;
typedef struct bo_m128x3_s16        bo_m128x3_s16_t;
typedef struct bo_m128x3_s32        bo_m128x3_s32_t;
typedef struct bo_m128x3_s64        bo_m128x3_s64_t;

struct bo_m128x3_u8 {
	bo_m128_u8_t    val[3];
};
struct bo_m128x3_u16 {
	bo_m128_u16_t   val[3];
};
struct bo_m128x3_u32 {
	bo_m128_u32_t   val[3];
};
struct bo_m128x3_u64 {
	bo_m128_u64_t   val[3];
};
typedef struct bo_m128x3_u8         bo_m128x3_u8_t;
typedef struct bo_m128x3_u16        bo_m128x3_u16_t;
typedef struct bo_m128x3_u32        bo_m128x3_u32_t;
typedef struct bo_m128x3_u64        bo_m128x3_u64_t;

struct bo_m128x3_f16 {
	bo_m128_f16_t   val[3];
};
struct bo_m128x3_f32 {
	bo_m128_f32_t   val[3];
};
struct bo_m128x3_f64 {
	bo_m128_f64_t   val[3];
};
typedef struct bo_m128x3_f16        bo_m128x3_f16_t;
typedef struct bo_m128x3_f32        bo_m128x3_f32_t;
typedef struct bo_m128x3_f64        bo_m128x3_f64_t;

/* @} */

/* ------------------------------------------------------------------------- */

/* 4 elements @{ */

	/*
	 * 8 byte signed/unsigned/float vector
	 */

struct bo_m64x4_s8 {
	bo_m64_s8_t     val[4];
};
struct bo_m64x4_s16 {
	bo_m64_s16_t    val[4];
};
struct bo_m64x4_s32 {
	bo_m64_s32_t    val[4];
};
struct bo_m64x4_s64 {
	bo_m64_s64_t    val[4];
};
typedef struct bo_m64x4_s8          bo_m64x4_s8_t;
typedef struct bo_m64x4_s16         bo_m64x4_s16_t;
typedef struct bo_m64x4_s32         bo_m64x4_s32_t;
typedef struct bo_m64x4_s64         bo_m64x4_s64_t;

struct bo_m64x4_u8 {
	bo_m64_u8_t     val[4];
};
struct bo_m64x4_u16 {
	bo_m64_u16_t    val[4];
};
struct bo_m64x4_u32 {
	bo_m64_u32_t    val[4];
};
struct bo_m64x4_u64 {
	bo_m64_u64_t    val[4];
};
typedef struct bo_m64x4_u8          bo_m64x4_u8_t;
typedef struct bo_m64x4_u16         bo_m64x4_u16_t;
typedef struct bo_m64x4_u32         bo_m64x4_u32_t;
typedef struct bo_m64x4_u64         bo_m64x4_u64_t;

struct bo_m64x4_f16 {
	bo_m64_f16_t    val[4];
};
struct bo_m64x4_f32 {
	bo_m64_f32_t    val[4];
};
struct bo_m64x4_f64 {
	bo_m64_f64_t    val[4];
};
typedef struct bo_m64x4_f16         bo_m64x4_f16_t;
typedef struct bo_m64x4_f32         bo_m64x4_f32_t;
typedef struct bo_m64x4_f64         bo_m64x4_f64_t;


	/*
	 * 16 byte signed/unsigned/float vector
	 */

struct bo_m128x4_s8 {
	bo_m128_s8_t    val[4];
};
struct bo_m128x4_s16 {
	bo_m128_s16_t   val[4];
};
struct bo_m128x4_s32 {
	bo_m128_s32_t   val[4];
};
struct bo_m128x4_s64 {
	bo_m128_s64_t   val[4];
};
typedef struct bo_m128x4_s8         bo_m128x4_s8_t;
typedef struct bo_m128x4_s16        bo_m128x4_s16_t;
typedef struct bo_m128x4_s32        bo_m128x4_s32_t;
typedef struct bo_m128x4_s64        bo_m128x4_s64_t;

struct bo_m128x4_u8 {
	bo_m128_u8_t    val[4];
};
struct bo_m128x4_u16 {
	bo_m128_u16_t   val[4];
};
struct bo_m128x4_u32 {
	bo_m128_u32_t   val[4];
};
struct bo_m128x4_u64 {
	bo_m128_u64_t   val[4];
};
typedef struct bo_m128x4_u8         bo_m128x4_u8_t;
typedef struct bo_m128x4_u16        bo_m128x4_u16_t;
typedef struct bo_m128x4_u32        bo_m128x4_u32_t;
typedef struct bo_m128x4_u64        bo_m128x4_u64_t;

struct bo_m128x4_f16 {
	bo_m128_f16_t   val[4];
};
struct bo_m128x4_f32 {
	bo_m128_f32_t   val[4];
};
struct bo_m128x4_f64 {
	bo_m128_f64_t   val[4];
};
typedef struct bo_m128x4_f16        bo_m128x4_f16_t;
typedef struct bo_m128x4_f32        bo_m128x4_f32_t;
typedef struct bo_m128x4_f64        bo_m128x4_f64_t;

/* @} */

/* ------------------------------------------------------------------------- */

# if 1 /* defined(_MSC_VER) */

/*
 * 1 element
 */

typedef bo_m64_s8_t                 int8x8_t;
typedef bo_m64_s16_t                int16x4_t;
typedef bo_m64_s32_t                int32x2_t;
typedef bo_m64_s64_t                int64x1_t;
typedef bo_m64_u8_t                 uint8x8_t;
typedef bo_m64_u16_t                uint16x4_t;
typedef bo_m64_u32_t                uint32x2_t;
typedef bo_m64_u64_t                uint64x1_t;
typedef bo_m64_f16_t                float16x4_t; /* @note: not supported */
typedef bo_m64_f32_t                float32x2_t;
typedef bo_m64_f64_t                float64x1_t;

typedef bo_m128_s8_t                int8x16_t;
typedef bo_m128_s16_t               int16x8_t;
typedef bo_m128_s32_t               int32x4_t;
typedef bo_m128_s64_t               int64x2_t;
typedef bo_m128_u8_t                uint8x16_t;
typedef bo_m128_u16_t               uint16x8_t;
typedef bo_m128_u32_t               uint32x4_t;
typedef bo_m128_u64_t               uint64x2_t;
typedef bo_m128_f16_t               float16x8_t; /* @note: not supported */
typedef bo_m128_f32_t               float32x4_t;
typedef bo_m128_f64_t               float64x2_t;

/*
 * 2 elements
 */

typedef bo_m64x2_s8_t               int8x8x2_t;
typedef bo_m64x2_s16_t              int16x4x2_t;
typedef bo_m64x2_s32_t              int32x2x2_t;
typedef bo_m64x2_s64_t              int64x1x2_t;
typedef bo_m64x2_u8_t               uint8x8x2_t;
typedef bo_m64x2_u16_t              uint16x4x2_t;
typedef bo_m64x2_u32_t              uint32x2x2_t;
typedef bo_m64x2_u64_t              uint64x1x2_t;
typedef bo_m64x2_f16_t              float16x4x2_t; /* @note: not supported */
typedef bo_m64x2_f32_t              float32x2x2_t;
typedef bo_m64x2_f64_t              float64x1x2_t;

typedef bo_m128x2_s8_t              int8x16x2_t;
typedef bo_m128x2_s16_t             int16x8x2_t;
typedef bo_m128x2_s32_t             int32x4x2_t;
typedef bo_m128x2_s64_t             int64x2x2_t;
typedef bo_m128x2_u8_t              uint8x16x2_t;
typedef bo_m128x2_u16_t             uint16x8x2_t;
typedef bo_m128x2_u32_t             uint32x4x2_t;
typedef bo_m128x2_u64_t             uint64x2x2_t;
typedef bo_m128x2_f16_t             float16x8x2_t; /* @note: not supported */
typedef bo_m128x2_f32_t             float32x4x2_t;
typedef bo_m128x2_f64_t             float64x2x2_t;

/*
 * 3 elements
 */

typedef bo_m64x3_s8_t               int8x8x3_t;
typedef bo_m64x3_s16_t              int16x4x3_t;
typedef bo_m64x3_s32_t              int32x2x3_t;
typedef bo_m64x3_s64_t              int64x1x3_t;
typedef bo_m64x3_u8_t               uint8x8x3_t;
typedef bo_m64x3_u16_t              uint16x4x3_t;
typedef bo_m64x3_u32_t              uint32x2x3_t;
typedef bo_m64x3_u64_t              uint64x1x3_t;
typedef bo_m64x3_f16_t              float16x4x3_t; /* @note: not supported */
typedef bo_m64x3_f32_t              float32x2x3_t;
typedef bo_m64x3_f64_t              float64x1x3_t;

typedef bo_m128x3_s8_t              int8x16x3_t;
typedef bo_m128x3_s16_t             int16x8x3_t;
typedef bo_m128x3_s32_t             int32x4x3_t;
typedef bo_m128x3_s64_t             int64x2x3_t;
typedef bo_m128x3_u8_t              uint8x16x3_t;
typedef bo_m128x3_u16_t             uint16x8x3_t;
typedef bo_m128x3_u32_t             uint32x4x3_t;
typedef bo_m128x3_u64_t             uint64x2x3_t;
typedef bo_m128x3_f16_t             float16x8x3_t; /* @note: not supported */
typedef bo_m128x3_f32_t             float32x4x3_t;
typedef bo_m128x3_f64_t             float64x2x3_t;

/*
 * 4 elements
 */

typedef bo_m64x4_s8_t               int8x8x4_t;
typedef bo_m64x4_s16_t              int16x4x4_t;
typedef bo_m64x4_s32_t              int32x2x4_t;
typedef bo_m64x4_s64_t              int64x1x4_t;
typedef bo_m64x4_u8_t               uint8x8x4_t;
typedef bo_m64x4_u16_t              uint16x4x4_t;
typedef bo_m64x4_u32_t              uint32x2x4_t;
typedef bo_m64x4_u64_t              uint64x1x4_t;
typedef bo_m64x4_f16_t              float16x4x4_t; /* @note: not supported */
typedef bo_m64x4_f32_t              float32x2x4_t;
typedef bo_m64x4_f64_t              float64x1x4_t;

typedef bo_m128x4_s8_t              int8x16x4_t;
typedef bo_m128x4_s16_t             int16x8x4_t;
typedef bo_m128x4_s32_t             int32x4x4_t;
typedef bo_m128x4_s64_t             int64x2x4_t;
typedef bo_m128x4_u8_t              uint8x16x4_t;
typedef bo_m128x4_u16_t             uint16x8x4_t;
typedef bo_m128x4_u32_t             uint32x4x4_t;
typedef bo_m128x4_u64_t             uint64x2x4_t;
typedef bo_m128x4_f16_t             float16x8x4_t; /* @note: not supported */
typedef bo_m128x4_f32_t             float32x4x4_t;
typedef bo_m128x4_f64_t             float64x2x4_t;

# endif /* defined(_MSC_VER) */


/******************************************************************************
 * %% BeginSection: function prototypes
 */
# pragma mark -
# pragma mark [ function prototypes ]
# pragma mark -

__MYC_BEGIN_CDECLS

/*
 * Load/Store
 * @{
 *
 * cmda : aligned access
 * cmdu : unaligned access
 * cmd  : aligned ? unaligned access
 */

	/*
	 * Load/Store 1-element structures
	 * @{
	 */

		/*
		 * Transfer 8x1=8 bytes
		 * 1 D register
		 */

# define /* bo_m64_s8_t  */    vipmvi_loada1x8_s8(p)            vipmvi_loadu1x8_s8(p)
# define /* bo_m64_s16_t */    vipmvi_loada1x8_s16(p)           vipmvi_loadu1x8_s16(p)
# define /* bo_m64_s32_t */    vipmvi_loada1x8_s32(p)           vipmvi_loadu1x8_s32(p)
# define /* bo_m64_u8_t  */    vipmvi_loada1x8_u8(p)            vipmvi_loadu1x8_u8(p)
# define /* bo_m64_u16_t */    vipmvi_loada1x8_u16(p)           vipmvi_loadu1x8_u16(p)
# define /* bo_m64_u32_t */    vipmvi_loada1x8_u32(p)           vipmvi_loadu1x8_u32(p)
# define /* bo_m64_f32_t */    vipmvi_loada1x8_f32(p)           vipmvi_loadu1x8_f32(p)

# define /* bo_m64_s8_t  */    vipmvi_loadu1x8_s8(p)            (*(bo_m64_s8_t maybe_const*)(p))
# define /* bo_m64_s16_t */    vipmvi_loadu1x8_s16(p)           (*(bo_m64_s16_t maybe_const*)(p))
# define /* bo_m64_s32_t */    vipmvi_loadu1x8_s32(p)           (*(bo_m64_s32_t maybe_const*)(p))
# define /* bo_m64_u8_t  */    vipmvi_loadu1x8_u8(p)            (*(bo_m64_u8_t maybe_const*)(p))
# define /* bo_m64_u16_t */    vipmvi_loadu1x8_u16(p)           (*(bo_m64_u16_t maybe_const*)(p))
# define /* bo_m64_u32_t */    vipmvi_loadu1x8_u32(p)           (*(bo_m64_u32_t maybe_const*)(p))
# define /* bo_m64_f32_t */    vipmvi_loadu1x8_f32(p)           vipmvi__load1x8_f32((bo_float32_t maybe_const*)(p))

# define /* bo_m64_s8_t  */    vipmvi_load1x8_s8(p)             vipmvi_loadu1x8_s8(p)
# define /* bo_m64_s16_t */    vipmvi_load1x8_s16(p)            vipmvi_loadu1x8_s16(p)
# define /* bo_m64_s32_t */    vipmvi_load1x8_s32(p)            vipmvi_loadu1x8_s32(p)
# define /* bo_m64_u8_t  */    vipmvi_load1x8_u8(p)             vipmvi_loadu1x8_u8(p)
# define /* bo_m64_u16_t */    vipmvi_load1x8_u16(p)            vipmvi_loadu1x8_u16(p)
# define /* bo_m64_u32_t */    vipmvi_load1x8_u32(p)            vipmvi_loadu1x8_u32(p)
# define /* bo_m64_f32_t */    vipmvi_load1x8_f32(p)            vipmvi_loadu1x8_f32(p)

# define /* void */            vipmvi_storea1x8_s8(p,x)         vipmvi_storeu1x8_s8(p,x)
# define /* void */            vipmvi_storea1x8_s16(p,x)        vipmvi_storeu1x8_s16(p,x)
# define /* void */            vipmvi_storea1x8_s32(p,x)        vipmvi_storeu1x8_s32(p,x)
# define /* void */            vipmvi_storea1x8_u8(p,x)         vipmvi_storeu1x8_u8(p,x)
# define /* void */            vipmvi_storea1x8_u16(p,x)        vipmvi_storeu1x8_u16(p,x)
# define /* void */            vipmvi_storea1x8_u32(p,x)        vipmvi_storeu1x8_u32(p,x)
# define /* void */            vipmvi_storea1x8_f32(p,x)        vipmvi_storeu1x8_f32(p,x)

# define /* void */            vipmvi_storeu1x8_s8(p,x)         vipmvi__store1x8_s8( (bo_uint8_t *) (p), x)
# define /* void */            vipmvi_storeu1x8_s16(p,x)        vipmvi__store1x8_s16((bo_uint16_t *)(p), x)
# define /* void */            vipmvi_storeu1x8_s32(p,x)        vipmvi__store1x8_s32((bo_uint32_t *)(p), x)
# define /* void */            vipmvi_storeu1x8_u8(p,x)         vipmvi__store1x8_u8( (bo_uint8_t *) (p), x)
# define /* void */            vipmvi_storeu1x8_u16(p,x)        vipmvi__store1x8_u16((bo_uint16_t *)(p), x)
# define /* void */            vipmvi_storeu1x8_u32(p,x)        vipmvi__store1x8_u32((bo_uint32_t *)(p), x)
# define /* void */            vipmvi_storeu1x8_f32(p,x)        vipmvi__store1x8_f32((bo_float32_t *) (p), x)

# define /* void */            vipmvi_store1x8_s8(p,x)          vipmvi_storeu1x8_s8(p,x)
# define /* void */            vipmvi_store1x8_s16(p,x)         vipmvi_storeu1x8_s16(p,x)
# define /* void */            vipmvi_store1x8_s32(p,x)         vipmvi_storeu1x8_s32(p,x)
# define /* void */            vipmvi_store1x8_u8(p,x)          vipmvi_storeu1x8_u8(p,x)
# define /* void */            vipmvi_store1x8_u16(p,x)         vipmvi_storeu1x8_u16(p,x)
# define /* void */            vipmvi_store1x8_u32(p,x)         vipmvi_storeu1x8_u32(p,x)
# define /* void */            vipmvi_store1x8_f32(p,x)         vipmvi_storeu1x8_f32(p,x)

/* without polluting the caches */
# define /* void */            vipmvi_storea1x8_npc_s8(p,x)     vipmvi_storea1x8_s8(p,x)
# define /* void */            vipmvi_storea1x8_npc_s16(p,x)    vipmvi_storea1x8_s16(p,x)
# define /* void */            vipmvi_storea1x8_npc_s32(p,x)    vipmvi_storea1x8_s32(p,x)
# define /* void */            vipmvi_storea1x8_npc_u8(p,x)     vipmvi_storea1x8_u8(p,x)
# define /* void */            vipmvi_storea1x8_npc_u16(p,x)    vipmvi_storea1x8_u16(p,x)
# define /* void */            vipmvi_storea1x8_npc_u32(p,x)    vipmvi_storea1x8_u32(p,x)
# define /* void */            vipmvi_storea1x8_npc_f32(p,x)    vipmvi_storea1x8_f32(p,x)


		/*
		 * Transfer 16x1=16 bytes
		 * 1 Q register
		 */

# define /* bo_m128_s8_t  */   vipmvi_loada1x16_s8(p)           vipmvi__loada_si128(p)
# define /* bo_m128_s16_t */   vipmvi_loada1x16_s16(p)          vipmvi__loada_si128(p)
# define /* bo_m128_s32_t */   vipmvi_loada1x16_s32(p)          vipmvi__loada_si128(p)
# define /* bo_m128_u8_t  */   vipmvi_loada1x16_u8(p)           vipmvi__loada_si128(p)
# define /* bo_m128_u16_t */   vipmvi_loada1x16_u16(p)          vipmvi__loada_si128(p)
# define /* bo_m128_u32_t */   vipmvi_loada1x16_u32(p)          vipmvi__loada_si128(p)
# define /* bo_m128_f32_t */   vipmvi_loada1x16_f32(p)          vipmvi__loada_sf128(p)

# define /* bo_m128_s8_t  */   vipmvi_loadu1x16_s8(p)           vipmvi__loadu_si128(p)
# define /* bo_m128_s16_t */   vipmvi_loadu1x16_s16(p)          vipmvi__loadu_si128(p)
# define /* bo_m128_s32_t */   vipmvi_loadu1x16_s32(p)          vipmvi__loadu_si128(p)
# define /* bo_m128_u8_t  */   vipmvi_loadu1x16_u8(p)           vipmvi__loadu_si128(p)
# define /* bo_m128_u16_t */   vipmvi_loadu1x16_u16(p)          vipmvi__loadu_si128(p)
# define /* bo_m128_u32_t */   vipmvi_loadu1x16_u32(p)          vipmvi__loadu_si128(p)
# define /* bo_m128_f32_t */   vipmvi_loadu1x16_f32(p)          vipmvi__loadu_sf128(p)

# define /* bo_m128_s8_t  */   vipmvi_load1x16_s8(p)            vipmvi__load_si128(p)
# define /* bo_m128_s16_t */   vipmvi_load1x16_s16(p)           vipmvi__load_si128(p)
# define /* bo_m128_s32_t */   vipmvi_load1x16_s32(p)           vipmvi__load_si128(p)
# define /* bo_m128_u8_t  */   vipmvi_load1x16_u8(p)            vipmvi__load_si128(p)
# define /* bo_m128_u16_t */   vipmvi_load1x16_u16(p)           vipmvi__load_si128(p)
# define /* bo_m128_u32_t */   vipmvi_load1x16_u32(p)           vipmvi__load_si128(p)
# define /* bo_m128_f32_t */   vipmvi_load1x16_f32(p)           vipmvi__load_sf128(p)

# define /* void */            vipmvi_storea1x16_s8(p,x)        vipmvi__storea_si128(p,x)
# define /* void */            vipmvi_storea1x16_s16(p,x)       vipmvi__storea_si128(p,x)
# define /* void */            vipmvi_storea1x16_s32(p,x)       vipmvi__storea_si128(p,x)
# define /* void */            vipmvi_storea1x16_u8(p,x)        vipmvi__storea_si128(p,x)
# define /* void */            vipmvi_storea1x16_u16(p,x)       vipmvi__storea_si128(p,x)
# define /* void */            vipmvi_storea1x16_u32(p,x)       vipmvi__storea_si128(p,x)
# define /* void */            vipmvi_storea1x16_f32(p,x)       vipmvi__storea_sf128(p,x)

# define /* void */            vipmvi_storeu1x16_s8(p,x)        vipmvi__storeu_si128(p,x)
# define /* void */            vipmvi_storeu1x16_s16(p,x)       vipmvi__storeu_si128(p,x)
# define /* void */            vipmvi_storeu1x16_s32(p,x)       vipmvi__storeu_si128(p,x)
# define /* void */            vipmvi_storeu1x16_u8(p,x)        vipmvi__storeu_si128(p,x)
# define /* void */            vipmvi_storeu1x16_u16(p,x)       vipmvi__storeu_si128(p,x)
# define /* void */            vipmvi_storeu1x16_u32(p,x)       vipmvi__storeu_si128(p,x)
# define /* void */            vipmvi_storeu1x16_f32(p,x)       vipmvi__storeu_sf128(p,x)

# define /* void */            vipmvi_store1x16_s8(p,x)         vipmvi__store_si128(p,x)
# define /* void */            vipmvi_store1x16_s16(p,x)        vipmvi__store_si128(p,x)
# define /* void */            vipmvi_store1x16_s32(p,x)        vipmvi__store_si128(p,x)
# define /* void */            vipmvi_store1x16_u8(p,x)         vipmvi__store_si128(p,x)
# define /* void */            vipmvi_store1x16_u16(p,x)        vipmvi__store_si128(p,x)
# define /* void */            vipmvi_store1x16_u32(p,x)        vipmvi__store_si128(p,x)
# define /* void */            vipmvi_store1x16_f32(p,x)        vipmvi__store_sf128(p,x)

/* without polluting the caches */
# define /* void */            vipmvi_storea1x16_npc_s8(p,x)    vipmvi__storea_npc_si128(p,x)
# define /* void */            vipmvi_storea1x16_npc_s16(p,x)   vipmvi__storea_npc_si128(p,x)
# define /* void */            vipmvi_storea1x16_npc_s32(p,x)   vipmvi__storea_npc_si128(p,x)
# define /* void */            vipmvi_storea1x16_npc_u8(p,x)    vipmvi__storea_npc_si128(p,x)
# define /* void */            vipmvi_storea1x16_npc_u16(p,x)   vipmvi__storea_npc_si128(p,x)
# define /* void */            vipmvi_storea1x16_npc_u32(p,x)   vipmvi__storea_npc_si128(p,x)
# define /* void */            vipmvi_storea1x16_npc_f32(p,x)   vipmvi__storea_npc_sf128(p,x)


		/*
		 * Transfer 8x2=16 bytes
		 * 2 D registers
		 */

#  define /* bo_m64x2_s8_t  */ vipmvi_loada1x8x2_s8(p)          vipmvi__loada1x8x2_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x2_s16_t */ vipmvi_loada1x8x2_s16(p)         vipmvi__loada1x8x2_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x2_s32_t */ vipmvi_loada1x8x2_s32(p)         vipmvi__loada1x8x2_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x2_u8_t  */ vipmvi_loada1x8x2_u8(p)          vipmvi__loada1x8x2_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x2_u16_t */ vipmvi_loada1x8x2_u16(p)         vipmvi__loada1x8x2_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x2_u32_t */ vipmvi_loada1x8x2_u32(p)         vipmvi__loada1x8x2_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x2_f32_t */ vipmvi_loada1x8x2_f32(p)         vipmvi__loada1x8x2_f32((bo_float32_t const *)(p))

#  define /* bo_m64x2_s8_t  */ vipmvi_loadu1x8x2_s8(p)          vipmvi__loadu1x8x2_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x2_s16_t */ vipmvi_loadu1x8x2_s16(p)         vipmvi__loadu1x8x2_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x2_s32_t */ vipmvi_loadu1x8x2_s32(p)         vipmvi__loadu1x8x2_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x2_u8_t  */ vipmvi_loadu1x8x2_u8(p)          vipmvi__loadu1x8x2_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x2_u16_t */ vipmvi_loadu1x8x2_u16(p)         vipmvi__loadu1x8x2_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x2_u32_t */ vipmvi_loadu1x8x2_u32(p)         vipmvi__loadu1x8x2_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x2_f32_t */ vipmvi_loadu1x8x2_f32(p)         vipmvi__loadu1x8x2_f32((bo_float32_t const *)(p))

#  define /* bo_m64x2_s8_t  */ vipmvi_load1x8x2_s8(p)           vipmvi__load1x8x2_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x2_s16_t */ vipmvi_load1x8x2_s16(p)          vipmvi__load1x8x2_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x2_s32_t */ vipmvi_load1x8x2_s32(p)          vipmvi__load1x8x2_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x2_u8_t  */ vipmvi_load1x8x2_u8(p)           vipmvi__load1x8x2_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x2_u16_t */ vipmvi_load1x8x2_u16(p)          vipmvi__load1x8x2_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x2_u32_t */ vipmvi_load1x8x2_u32(p)          vipmvi__load1x8x2_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x2_f32_t */ vipmvi_load1x8x2_f32(p)          vipmvi__load1x8x2_f32((bo_float32_t const *)(p))

#  define /* void */           vipmvi_storea1x8x2_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x8_s8( ___p,(x).val[0]), vipmvi_storea1x8_s8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x8_s16(___p,(x).val[0]), vipmvi_storea1x8_s16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x8_s32(___p,(x).val[0]), vipmvi_storea1x8_s32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x8_u8( ___p,(x).val[0]), vipmvi_storea1x8_u8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x8_u16(___p,(x).val[0]), vipmvi_storea1x8_u16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x8_u32(___p,(x).val[0]), vipmvi_storea1x8_u32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x8_f32(___p,(x).val[0]), vipmvi_storea1x8_f32((___p) + 2,(x).val[1])); } while (0)

#  define /* void */           vipmvi_storeu1x8x2_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storeu1x8_s8( ___p,(x).val[0]), vipmvi_storeu1x8_s8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storeu1x8_s16(___p,(x).val[0]), vipmvi_storeu1x8_s16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storeu1x8_s32(___p,(x).val[0]), vipmvi_storeu1x8_s32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storeu1x8_u8( ___p,(x).val[0]), vipmvi_storeu1x8_u8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storeu1x8_u16(___p,(x).val[0]), vipmvi_storeu1x8_u16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storeu1x8_u32(___p,(x).val[0]), vipmvi_storeu1x8_u32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x8x2_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storeu1x8_f32(___p,(x).val[0]), vipmvi_storeu1x8_f32((___p) + 2,(x).val[1])); } while (0)

#  define /* void */           vipmvi_store1x8x2_s8(p,x)        do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_store1x8_s8( ___p,(x).val[0]), vipmvi_store1x8_s8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x8x2_s16(p,x)       do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_store1x8_s16(___p,(x).val[0]), vipmvi_store1x8_s16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x8x2_s32(p,x)       do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_store1x8_s32(___p,(x).val[0]), vipmvi_store1x8_s32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x8x2_u8(p,x)        do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_store1x8_u8( ___p,(x).val[0]), vipmvi_store1x8_u8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x8x2_u16(p,x)       do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_store1x8_u16(___p,(x).val[0]), vipmvi_store1x8_u16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x8x2_u32(p,x)       do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_store1x8_u32(___p,(x).val[0]), vipmvi_store1x8_u32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x8x2_f32(p,x)       do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_store1x8_f32(___p,(x).val[0]), vipmvi_store1x8_f32((___p) + 2,(x).val[1])); } while (0)

/* without polluting the caches */
#  define /* void */           vipmvi_storea1x8x2_npc_s8(p,x)   do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x8_npc_s8( ___p,(x).val[0]), vipmvi_storea1x8_npc_s8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_npc_s16(p,x)  do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x8_npc_s16(___p,(x).val[0]), vipmvi_storea1x8_npc_s16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_npc_s32(p,x)  do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x8_npc_s32(___p,(x).val[0]), vipmvi_storea1x8_npc_s32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_npc_u8(p,x)   do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x8_npc_u8( ___p,(x).val[0]), vipmvi_storea1x8_npc_u8( (___p) + 8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_npc_u16(p,x)  do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x8_npc_u16(___p,(x).val[0]), vipmvi_storea1x8_npc_u16((___p) + 4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_npc_u32(p,x)  do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x8_npc_u32(___p,(x).val[0]), vipmvi_storea1x8_npc_u32((___p) + 2,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x8x2_npc_f32(p,x)  do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x8_npc_f32(___p,(x).val[0]), vipmvi_storea1x8_npc_f32((___p) + 2,(x).val[1])); } while (0)


		/*
		 * Transfer 8x3=24 bytes
		 * 3 D registers
		 */

#  define /* bo_m64x3_s8_t  */ vipmvi_loada1x8x3_s8(p)          vipmvi__loada1x8x3_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x3_s16_t */ vipmvi_loada1x8x3_s16(p)         vipmvi__loada1x8x3_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x3_s32_t */ vipmvi_loada1x8x3_s32(p)         vipmvi__loada1x8x3_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x3_u8_t  */ vipmvi_loada1x8x3_u8(p)          vipmvi__loada1x8x3_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x3_u16_t */ vipmvi_loada1x8x3_u16(p)         vipmvi__loada1x8x3_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x3_u32_t */ vipmvi_loada1x8x3_u32(p)         vipmvi__loada1x8x3_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x3_f32_t */ vipmvi_loada1x8x3_f32(p)         vipmvi__loada1x8x3_f32((bo_float32_t const *)(p))

#  define /* bo_m64x3_s8_t  */ vipmvi_loadu1x8x3_s8(p)          vipmvi__loadu1x8x3_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x3_s16_t */ vipmvi_loadu1x8x3_s16(p)         vipmvi__loadu1x8x3_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x3_s32_t */ vipmvi_loadu1x8x3_s32(p)         vipmvi__loadu1x8x3_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x3_u8_t  */ vipmvi_loadu1x8x3_u8(p)          vipmvi__loadu1x8x3_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x3_u16_t */ vipmvi_loadu1x8x3_u16(p)         vipmvi__loadu1x8x3_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x3_u32_t */ vipmvi_loadu1x8x3_u32(p)         vipmvi__loadu1x8x3_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x3_f32_t */ vipmvi_loadu1x8x3_f32(p)         vipmvi__loadu1x8x3_f32((bo_float32_t const *)(p))

#  define /* bo_m64x3_s8_t  */ vipmvi_load1x8x3_s8(p)           vipmvi__load1x8x3_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x3_s16_t */ vipmvi_load1x8x3_s16(p)          vipmvi__load1x8x3_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x3_s32_t */ vipmvi_load1x8x3_s32(p)          vipmvi__load1x8x3_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x3_u8_t  */ vipmvi_load1x8x3_u8(p)           vipmvi__load1x8x3_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x3_u16_t */ vipmvi_load1x8x3_u16(p)          vipmvi__load1x8x3_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x3_u32_t */ vipmvi_load1x8x3_u32(p)          vipmvi__load1x8x3_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x3_f32_t */ vipmvi_load1x8x3_f32(p)          vipmvi__load1x8x3_f32((bo_float32_t const *)(p))

#  define /* void */           vipmvi_storea1x8x3_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x8_s8( ___p,(x).val[0]), vipmvi_storea1x8_s8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_s8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x8_s16(___p,(x).val[0]), vipmvi_storea1x8_s16((___p) + 4,(x).val[1]), vipmvi_storea1x8_s16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x8_s32(___p,(x).val[0]), vipmvi_storea1x8_s32((___p) + 2,(x).val[1]), vipmvi_storea1x8_s32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x8_u8( ___p,(x).val[0]), vipmvi_storea1x8_u8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_u8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x8_u16(___p,(x).val[0]), vipmvi_storea1x8_u16((___p) + 4,(x).val[1]), vipmvi_storea1x8_u16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x8_u32(___p,(x).val[0]), vipmvi_storea1x8_u32((___p) + 2,(x).val[1]), vipmvi_storea1x8_u32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x8_f32(___p,(x).val[0]), vipmvi_storea1x8_f32((___p) + 2,(x).val[1]), vipmvi_storea1x8_f32((___p) +  4,(x).val[2])); } while (0)

#  define /* void */           vipmvi_storeu1x8x3_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storeu1x8_s8( ___p,(x).val[0]), vipmvi_storeu1x8_s8( (___p) + 8,(x).val[1]), vipmvi_storeu1x8_s8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storeu1x8_s16(___p,(x).val[0]), vipmvi_storeu1x8_s16((___p) + 4,(x).val[1]), vipmvi_storeu1x8_s16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storeu1x8_s32(___p,(x).val[0]), vipmvi_storeu1x8_s32((___p) + 2,(x).val[1]), vipmvi_storeu1x8_s32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storeu1x8_u8( ___p,(x).val[0]), vipmvi_storeu1x8_u8( (___p) + 8,(x).val[1]), vipmvi_storeu1x8_u8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storeu1x8_u16(___p,(x).val[0]), vipmvi_storeu1x8_u16((___p) + 4,(x).val[1]), vipmvi_storeu1x8_u16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storeu1x8_u32(___p,(x).val[0]), vipmvi_storeu1x8_u32((___p) + 2,(x).val[1]), vipmvi_storeu1x8_u32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x8x3_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storeu1x8_f32(___p,(x).val[0]), vipmvi_storeu1x8_f32((___p) + 2,(x).val[1]), vipmvi_storeu1x8_f32((___p) +  4,(x).val[2])); } while (0)

#  define /* void */           vipmvi_store1x8x3_s8(p,x)        do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_store1x8_s8( ___p,(x).val[0]), vipmvi_store1x8_s8( (___p) + 8,(x).val[1]), vipmvi_store1x8_s8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x8x3_s16(p,x)       do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_store1x8_s16(___p,(x).val[0]), vipmvi_store1x8_s16((___p) + 4,(x).val[1]), vipmvi_store1x8_s16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x8x3_s32(p,x)       do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_store1x8_s32(___p,(x).val[0]), vipmvi_store1x8_s32((___p) + 2,(x).val[1]), vipmvi_store1x8_s32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x8x3_u8(p,x)        do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_store1x8_u8( ___p,(x).val[0]), vipmvi_store1x8_u8( (___p) + 8,(x).val[1]), vipmvi_store1x8_u8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x8x3_u16(p,x)       do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_store1x8_u16(___p,(x).val[0]), vipmvi_store1x8_u16((___p) + 4,(x).val[1]), vipmvi_store1x8_u16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x8x3_u32(p,x)       do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_store1x8_u32(___p,(x).val[0]), vipmvi_store1x8_u32((___p) + 2,(x).val[1]), vipmvi_store1x8_u32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x8x3_f32(p,x)       do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_store1x8_f32(___p,(x).val[0]), vipmvi_store1x8_f32((___p) + 2,(x).val[1]), vipmvi_store1x8_f32((___p) +  4,(x).val[2])); } while (0)

/* without polluting the caches */
#  define /* void */           vipmvi_storea1x8x3_npc_s8(p,x)   do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x8_npc_s8( ___p,(x).val[0]), vipmvi_storea1x8_npc_s8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_npc_s8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_npc_s16(p,x)  do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x8_npc_s16(___p,(x).val[0]), vipmvi_storea1x8_npc_s16((___p) + 4,(x).val[1]), vipmvi_storea1x8_npc_s16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_npc_s32(p,x)  do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x8_npc_s32(___p,(x).val[0]), vipmvi_storea1x8_npc_s32((___p) + 2,(x).val[1]), vipmvi_storea1x8_npc_s32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_npc_u8(p,x)   do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x8_npc_u8( ___p,(x).val[0]), vipmvi_storea1x8_npc_u8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_npc_u8( (___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_npc_u16(p,x)  do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x8_npc_u16(___p,(x).val[0]), vipmvi_storea1x8_npc_u16((___p) + 4,(x).val[1]), vipmvi_storea1x8_npc_u16((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_npc_u32(p,x)  do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x8_npc_u32(___p,(x).val[0]), vipmvi_storea1x8_npc_u32((___p) + 2,(x).val[1]), vipmvi_storea1x8_npc_u32((___p) +  4,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x8x3_npc_f32(p,x)  do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x8_npc_f32(___p,(x).val[0]), vipmvi_storea1x8_npc_f32((___p) + 2,(x).val[1]), vipmvi_storea1x8_npc_f32((___p) +  4,(x).val[2])); } while (0)


		/*
		 * Transfer 8x4=32 bytes
		 * 4 D registers
		 */

#  define /* bo_m64x4_s8_t  */ vipmvi_loada1x8x4_s8(p)          vipmvi__loada1x8x4_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x4_s16_t */ vipmvi_loada1x8x4_s16(p)         vipmvi__loada1x8x4_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x4_s32_t */ vipmvi_loada1x8x4_s32(p)         vipmvi__loada1x8x4_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x4_u8_t  */ vipmvi_loada1x8x4_u8(p)          vipmvi__loada1x8x4_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x4_u16_t */ vipmvi_loada1x8x4_u16(p)         vipmvi__loada1x8x4_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x4_u32_t */ vipmvi_loada1x8x4_u32(p)         vipmvi__loada1x8x4_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x4_f32_t */ vipmvi_loada1x8x4_f32(p)         vipmvi__loada1x8x4_f32((bo_float32_t const *)(p))

#  define /* bo_m64x4_s8_t  */ vipmvi_loadu1x8x4_s8(p)          vipmvi__loadu1x8x4_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x4_s16_t */ vipmvi_loadu1x8x4_s16(p)         vipmvi__loadu1x8x4_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x4_s32_t */ vipmvi_loadu1x8x4_s32(p)         vipmvi__loadu1x8x4_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x4_u8_t  */ vipmvi_loadu1x8x4_u8(p)          vipmvi__loadu1x8x4_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x4_u16_t */ vipmvi_loadu1x8x4_u16(p)         vipmvi__loadu1x8x4_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x4_u32_t */ vipmvi_loadu1x8x4_u32(p)         vipmvi__loadu1x8x4_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x4_f32_t */ vipmvi_loadu1x8x4_f32(p)         vipmvi__loadu1x8x4_f32((bo_float32_t const *)(p))

#  define /* bo_m64x4_s8_t  */ vipmvi_load1x8x4_s8(p)           vipmvi__load1x8x4_s8( (bo_int8_t    const *)(p))
#  define /* bo_m64x4_s16_t */ vipmvi_load1x8x4_s16(p)          vipmvi__load1x8x4_s16((bo_int16_t   const *)(p))
#  define /* bo_m64x4_s32_t */ vipmvi_load1x8x4_s32(p)          vipmvi__load1x8x4_s32((bo_int32_t   const *)(p))
#  define /* bo_m64x4_u8_t  */ vipmvi_load1x8x4_u8(p)           vipmvi__load1x8x4_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m64x4_u16_t */ vipmvi_load1x8x4_u16(p)          vipmvi__load1x8x4_u16((bo_uint16_t  const *)(p))
#  define /* bo_m64x4_u32_t */ vipmvi_load1x8x4_u32(p)          vipmvi__load1x8x4_u32((bo_uint32_t  const *)(p))
#  define /* bo_m64x4_f32_t */ vipmvi_load1x8x4_f32(p)          vipmvi__load1x8x4_f32((bo_float32_t const *)(p))

#  define /* void */           vipmvi_storea1x8x4_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x8_s8( ___p,(x).val[0]), vipmvi_storea1x8_s8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_s8( (___p) + 16,(x).val[2]), vipmvi_storea1x8_s8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x8_s16(___p,(x).val[0]), vipmvi_storea1x8_s16((___p) + 4,(x).val[1]), vipmvi_storea1x8_s16((___p) +  8,(x).val[2]), vipmvi_storea1x8_s16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x8_s32(___p,(x).val[0]), vipmvi_storea1x8_s32((___p) + 2,(x).val[1]), vipmvi_storea1x8_s32((___p) +  4,(x).val[2]), vipmvi_storea1x8_s32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x8_u8( ___p,(x).val[0]), vipmvi_storea1x8_u8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_u8( (___p) + 16,(x).val[2]), vipmvi_storea1x8_u8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x8_u16(___p,(x).val[0]), vipmvi_storea1x8_u16((___p) + 4,(x).val[1]), vipmvi_storea1x8_u16((___p) +  8,(x).val[2]), vipmvi_storea1x8_u16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x8_u32(___p,(x).val[0]), vipmvi_storea1x8_u32((___p) + 2,(x).val[1]), vipmvi_storea1x8_u32((___p) +  4,(x).val[2]), vipmvi_storea1x8_u32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x8_f32(___p,(x).val[0]), vipmvi_storea1x8_f32((___p) + 2,(x).val[1]), vipmvi_storea1x8_f32((___p) +  4,(x).val[2]), vipmvi_storea1x8_f32((___p) +  6,(x).val[3])); } while (0)

#  define /* void */           vipmvi_storeu1x8x4_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storeu1x8_s8( ___p,(x).val[0]), vipmvi_storeu1x8_s8( (___p) + 8,(x).val[1]), vipmvi_storeu1x8_s8( (___p) + 16,(x).val[2]), vipmvi_storeu1x8_s8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storeu1x8_s16(___p,(x).val[0]), vipmvi_storeu1x8_s16((___p) + 4,(x).val[1]), vipmvi_storeu1x8_s16((___p) +  8,(x).val[2]), vipmvi_storeu1x8_s16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storeu1x8_s32(___p,(x).val[0]), vipmvi_storeu1x8_s32((___p) + 2,(x).val[1]), vipmvi_storeu1x8_s32((___p) +  4,(x).val[2]), vipmvi_storeu1x8_s32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storeu1x8_u8( ___p,(x).val[0]), vipmvi_storeu1x8_u8( (___p) + 8,(x).val[1]), vipmvi_storeu1x8_u8( (___p) + 16,(x).val[2]), vipmvi_storeu1x8_u8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storeu1x8_u16(___p,(x).val[0]), vipmvi_storeu1x8_u16((___p) + 4,(x).val[1]), vipmvi_storeu1x8_u16((___p) +  8,(x).val[2]), vipmvi_storeu1x8_u16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storeu1x8_u32(___p,(x).val[0]), vipmvi_storeu1x8_u32((___p) + 2,(x).val[1]), vipmvi_storeu1x8_u32((___p) +  4,(x).val[2]), vipmvi_storeu1x8_u32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x8x4_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storeu1x8_f32(___p,(x).val[0]), vipmvi_storeu1x8_f32((___p) + 2,(x).val[1]), vipmvi_storeu1x8_f32((___p) +  4,(x).val[2]), vipmvi_storeu1x8_f32((___p) +  6,(x).val[3])); } while (0)

#  define /* void */           vipmvi_store1x8x4_s8(p,x)        do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_store1x8_s8( ___p,(x).val[0]), vipmvi_store1x8_s8( (___p) + 8,(x).val[1]), vipmvi_store1x8_s8( (___p) + 16,(x).val[2]), vipmvi_store1x8_s8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x8x4_s16(p,x)       do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_store1x8_s16(___p,(x).val[0]), vipmvi_store1x8_s16((___p) + 4,(x).val[1]), vipmvi_store1x8_s16((___p) +  8,(x).val[2]), vipmvi_store1x8_s16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x8x4_s32(p,x)       do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_store1x8_s32(___p,(x).val[0]), vipmvi_store1x8_s32((___p) + 2,(x).val[1]), vipmvi_store1x8_s32((___p) +  4,(x).val[2]), vipmvi_store1x8_s32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x8x4_u8(p,x)        do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_store1x8_u8( ___p,(x).val[0]), vipmvi_store1x8_u8( (___p) + 8,(x).val[1]), vipmvi_store1x8_u8( (___p) + 16,(x).val[2]), vipmvi_store1x8_u8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x8x4_u16(p,x)       do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_store1x8_u16(___p,(x).val[0]), vipmvi_store1x8_u16((___p) + 4,(x).val[1]), vipmvi_store1x8_u16((___p) +  8,(x).val[2]), vipmvi_store1x8_u16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x8x4_u32(p,x)       do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_store1x8_u32(___p,(x).val[0]), vipmvi_store1x8_u32((___p) + 2,(x).val[1]), vipmvi_store1x8_u32((___p) +  4,(x).val[2]), vipmvi_store1x8_u32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x8x4_f32(p,x)       do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_store1x8_f32(___p,(x).val[0]), vipmvi_store1x8_f32((___p) + 2,(x).val[1]), vipmvi_store1x8_f32((___p) +  4,(x).val[2]), vipmvi_store1x8_f32((___p) +  6,(x).val[3])); } while (0)

/* without polluting the caches */
#  define /* void */           vipmvi_storea1x8x4_npc_s8(p,x)   do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x8_npc_s8( ___p,(x).val[0]), vipmvi_storea1x8_npc_s8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_npc_s8( (___p) + 16,(x).val[2]), vipmvi_storea1x8_npc_s8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_npc_s16(p,x)  do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x8_npc_s16(___p,(x).val[0]), vipmvi_storea1x8_npc_s16((___p) + 4,(x).val[1]), vipmvi_storea1x8_npc_s16((___p) +  8,(x).val[2]), vipmvi_storea1x8_npc_s16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_npc_s32(p,x)  do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x8_npc_s32(___p,(x).val[0]), vipmvi_storea1x8_npc_s32((___p) + 2,(x).val[1]), vipmvi_storea1x8_npc_s32((___p) +  4,(x).val[2]), vipmvi_storea1x8_npc_s32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_npc_u8(p,x)   do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x8_npc_u8( ___p,(x).val[0]), vipmvi_storea1x8_npc_u8( (___p) + 8,(x).val[1]), vipmvi_storea1x8_npc_u8( (___p) + 16,(x).val[2]), vipmvi_storea1x8_npc_u8( (___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_npc_u16(p,x)  do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x8_npc_u16(___p,(x).val[0]), vipmvi_storea1x8_npc_u16((___p) + 4,(x).val[1]), vipmvi_storea1x8_npc_u16((___p) +  8,(x).val[2]), vipmvi_storea1x8_npc_u16((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_npc_u32(p,x)  do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x8_npc_u32(___p,(x).val[0]), vipmvi_storea1x8_npc_u32((___p) + 2,(x).val[1]), vipmvi_storea1x8_npc_u32((___p) +  4,(x).val[2]), vipmvi_storea1x8_npc_u32((___p) +  6,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x8x4_npc_f32(p,x)  do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x8_npc_f32(___p,(x).val[0]), vipmvi_storea1x8_npc_f32((___p) + 2,(x).val[1]), vipmvi_storea1x8_npc_f32((___p) +  4,(x).val[2]), vipmvi_storea1x8_npc_f32((___p) +  6,(x).val[3])); } while (0)


		/*
		 * Transfer 16x2=32 bytes
		 * 2 Q registers
		 */

#  define /* bo_m128x2_s8_t  */vipmvi_loada1x16x2_s8(p)         vipmvi__loada1x16x2_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x2_s16_t */vipmvi_loada1x16x2_s16(p)        vipmvi__loada1x16x2_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x2_s32_t */vipmvi_loada1x16x2_s32(p)        vipmvi__loada1x16x2_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x2_u8_t  */vipmvi_loada1x16x2_u8(p)         vipmvi__loada1x16x2_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x2_u16_t */vipmvi_loada1x16x2_u16(p)        vipmvi__loada1x16x2_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x2_u32_t */vipmvi_loada1x16x2_u32(p)        vipmvi__loada1x16x2_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x2_f32_t */vipmvi_loada1x16x2_f32(p)        vipmvi__loada1x16x2_f32((bo_float32_t const *)(p))

#  define /* bo_m128x2_s8_t  */vipmvi_loadu1x16x2_s8(p)         vipmvi__loadu1x16x2_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x2_s16_t */vipmvi_loadu1x16x2_s16(p)        vipmvi__loadu1x16x2_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x2_s32_t */vipmvi_loadu1x16x2_s32(p)        vipmvi__loadu1x16x2_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x2_u8_t  */vipmvi_loadu1x16x2_u8(p)         vipmvi__loadu1x16x2_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x2_u16_t */vipmvi_loadu1x16x2_u16(p)        vipmvi__loadu1x16x2_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x2_u32_t */vipmvi_loadu1x16x2_u32(p)        vipmvi__loadu1x16x2_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x2_f32_t */vipmvi_loadu1x16x2_f32(p)        vipmvi__loadu1x16x2_f32((bo_float32_t const *)(p))

#  define /* bo_m128x2_s8_t  */vipmvi_load1x16x2_s8(p)          vipmvi__load1x16x2_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x2_s16_t */vipmvi_load1x16x2_s16(p)         vipmvi__load1x16x2_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x2_s32_t */vipmvi_load1x16x2_s32(p)         vipmvi__load1x16x2_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x2_u8_t  */vipmvi_load1x16x2_u8(p)          vipmvi__load1x16x2_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x2_u16_t */vipmvi_load1x16x2_u16(p)         vipmvi__load1x16x2_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x2_u32_t */vipmvi_load1x16x2_u32(p)         vipmvi__load1x16x2_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x2_f32_t */vipmvi_load1x16x2_f32(p)         vipmvi__load1x16x2_f32((bo_float32_t const *)(p))

#  define /* void */           vipmvi_storea1x16x2_s8(p,x)      do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x16_s8( ___p,(x).val[0]), vipmvi_storea1x16_s8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_s16(p,x)     do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x16_s16(___p,(x).val[0]), vipmvi_storea1x16_s16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_s32(p,x)     do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x16_s32(___p,(x).val[0]), vipmvi_storea1x16_s32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_u8(p,x)      do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x16_u8( ___p,(x).val[0]), vipmvi_storea1x16_u8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_u16(p,x)     do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x16_u16(___p,(x).val[0]), vipmvi_storea1x16_u16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_u32(p,x)     do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x16_u32(___p,(x).val[0]), vipmvi_storea1x16_u32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_f32(p,x)     do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x16_f32(___p,(x).val[0]), vipmvi_storea1x16_f32((___p) +  4,(x).val[1])); } while (0)

#  define /* void */           vipmvi_storeu1x16x2_s8(p,x)      do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storeu1x16_s8( ___p,(x).val[0]), vipmvi_storeu1x16_s8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_s16(p,x)     do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storeu1x16_s16(___p,(x).val[0]), vipmvi_storeu1x16_s16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_s32(p,x)     do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storeu1x16_s32(___p,(x).val[0]), vipmvi_storeu1x16_s32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_u8(p,x)      do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storeu1x16_u8( ___p,(x).val[0]), vipmvi_storeu1x16_u8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_u16(p,x)     do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storeu1x16_u16(___p,(x).val[0]), vipmvi_storeu1x16_u16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_u32(p,x)     do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storeu1x16_u32(___p,(x).val[0]), vipmvi_storeu1x16_u32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storeu1x16x2_f32(p,x)     do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storeu1x16_f32(___p,(x).val[0]), vipmvi_storeu1x16_f32((___p) +  4,(x).val[1])); } while (0)

#  define /* void */           vipmvi_store1x16x2_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_store1x16_s8( ___p,(x).val[0]), vipmvi_store1x16_s8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x16x2_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_store1x16_s16(___p,(x).val[0]), vipmvi_store1x16_s16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x16x2_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_store1x16_s32(___p,(x).val[0]), vipmvi_store1x16_s32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x16x2_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_store1x16_u8( ___p,(x).val[0]), vipmvi_store1x16_u8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x16x2_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_store1x16_u16(___p,(x).val[0]), vipmvi_store1x16_u16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x16x2_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_store1x16_u32(___p,(x).val[0]), vipmvi_store1x16_u32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_store1x16x2_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_store1x16_f32(___p,(x).val[0]), vipmvi_store1x16_f32((___p) +  4,(x).val[1])); } while (0)

/* without polluting the caches */
#  define /* void */           vipmvi_storea1x16x2_npc_s8(p,x)  do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x16_npc_s8( ___p,(x).val[0]), vipmvi_storea1x16_npc_s8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_npc_s16(p,x) do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x16_npc_s16(___p,(x).val[0]), vipmvi_storea1x16_npc_s16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_npc_s32(p,x) do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x16_npc_s32(___p,(x).val[0]), vipmvi_storea1x16_npc_s32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_npc_u8(p,x)  do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x16_npc_u8( ___p,(x).val[0]), vipmvi_storea1x16_npc_u8( (___p) + 16,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_npc_u16(p,x) do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x16_npc_u16(___p,(x).val[0]), vipmvi_storea1x16_npc_u16((___p) +  8,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_npc_u32(p,x) do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x16_npc_u32(___p,(x).val[0]), vipmvi_storea1x16_npc_u32((___p) +  4,(x).val[1])); } while (0)
#  define /* void */           vipmvi_storea1x16x2_npc_f32(p,x) do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x16_npc_f32(___p,(x).val[0]), vipmvi_storea1x16_npc_f32((___p) +  4,(x).val[1])); } while (0)


		/*
		 * Transfer 16x3=48 bytes
		 * 3 Q registers
		 */

#  define /* bo_m128x3_s8_t  */vipmvi_loada1x16x3_s8(p)         vipmvi__loada1x16x3_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x3_s16_t */vipmvi_loada1x16x3_s16(p)        vipmvi__loada1x16x3_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x3_s32_t */vipmvi_loada1x16x3_s32(p)        vipmvi__loada1x16x3_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x3_u8_t  */vipmvi_loada1x16x3_u8(p)         vipmvi__loada1x16x3_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x3_u16_t */vipmvi_loada1x16x3_u16(p)        vipmvi__loada1x16x3_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x3_u32_t */vipmvi_loada1x16x3_u32(p)        vipmvi__loada1x16x3_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x3_f32_t */vipmvi_loada1x16x3_f32(p)        vipmvi__loada1x16x3_f32((bo_float32_t const *)(p))

#  define /* bo_m128x3_s8_t  */vipmvi_loadu1x16x3_s8(p)         vipmvi__loadu1x16x3_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x3_s16_t */vipmvi_loadu1x16x3_s16(p)        vipmvi__loadu1x16x3_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x3_s32_t */vipmvi_loadu1x16x3_s32(p)        vipmvi__loadu1x16x3_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x3_u8_t  */vipmvi_loadu1x16x3_u8(p)         vipmvi__loadu1x16x3_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x3_u16_t */vipmvi_loadu1x16x3_u16(p)        vipmvi__loadu1x16x3_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x3_u32_t */vipmvi_loadu1x16x3_u32(p)        vipmvi__loadu1x16x3_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x3_f32_t */vipmvi_loadu1x16x3_f32(p)        vipmvi__loadu1x16x3_f32((bo_float32_t const *)(p))

#  define /* bo_m128x3_s8_t  */vipmvi_load1x16x3_s8(p)          vipmvi__load1x16x3_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x3_s16_t */vipmvi_load1x16x3_s16(p)         vipmvi__load1x16x3_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x3_s32_t */vipmvi_load1x16x3_s32(p)         vipmvi__load1x16x3_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x3_u8_t  */vipmvi_load1x16x3_u8(p)          vipmvi__load1x16x3_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x3_u16_t */vipmvi_load1x16x3_u16(p)         vipmvi__load1x16x3_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x3_u32_t */vipmvi_load1x16x3_u32(p)         vipmvi__load1x16x3_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x3_f32_t */vipmvi_load1x16x3_f32(p)         vipmvi__load1x16x3_f32((bo_float32_t const *)(p))

#  define /* void */           vipmvi_storea1x16x3_s8(p,x)      do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x16_s8( ___p,(x).val[0]), vipmvi_storea1x16_s8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_s8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_s16(p,x)     do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x16_s16(___p,(x).val[0]), vipmvi_storea1x16_s16((___p) +  8,(x).val[1]), vipmvi_storea1x16_s16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_s32(p,x)     do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x16_s32(___p,(x).val[0]), vipmvi_storea1x16_s32((___p) +  4,(x).val[1]), vipmvi_storea1x16_s32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_u8(p,x)      do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x16_u8( ___p,(x).val[0]), vipmvi_storea1x16_u8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_u8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_u16(p,x)     do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x16_u16(___p,(x).val[0]), vipmvi_storea1x16_u16((___p) +  8,(x).val[1]), vipmvi_storea1x16_u16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_u32(p,x)     do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x16_u32(___p,(x).val[0]), vipmvi_storea1x16_u32((___p) +  4,(x).val[1]), vipmvi_storea1x16_u32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_f32(p,x)     do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x16_f32(___p,(x).val[0]), vipmvi_storea1x16_f32((___p) +  4,(x).val[1]), vipmvi_storea1x16_f32((___p) +  8,(x).val[2])); } while (0)

#  define /* void */           vipmvi_storeu1x16x3_s8(p,x)      do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storeu1x16_s8( ___p,(x).val[0]), vipmvi_storeu1x16_s8( (___p) + 16,(x).val[1]), vipmvi_storeu1x16_s8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_s16(p,x)     do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storeu1x16_s16(___p,(x).val[0]), vipmvi_storeu1x16_s16((___p) +  8,(x).val[1]), vipmvi_storeu1x16_s16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_s32(p,x)     do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storeu1x16_s32(___p,(x).val[0]), vipmvi_storeu1x16_s32((___p) +  4,(x).val[1]), vipmvi_storeu1x16_s32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_u8(p,x)      do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storeu1x16_u8( ___p,(x).val[0]), vipmvi_storeu1x16_u8( (___p) + 16,(x).val[1]), vipmvi_storeu1x16_u8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_u16(p,x)     do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storeu1x16_u16(___p,(x).val[0]), vipmvi_storeu1x16_u16((___p) +  8,(x).val[1]), vipmvi_storeu1x16_u16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_u32(p,x)     do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storeu1x16_u32(___p,(x).val[0]), vipmvi_storeu1x16_u32((___p) +  4,(x).val[1]), vipmvi_storeu1x16_u32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storeu1x16x3_f32(p,x)     do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storeu1x16_f32(___p,(x).val[0]), vipmvi_storeu1x16_f32((___p) +  4,(x).val[1]), vipmvi_storeu1x16_f32((___p) +  8,(x).val[2])); } while (0)

#  define /* void */           vipmvi_store1x16x3_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_store1x16_s8( ___p,(x).val[0]), vipmvi_store1x16_s8( (___p) + 16,(x).val[1]), vipmvi_store1x16_s8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_store1x16_s16(___p,(x).val[0]), vipmvi_store1x16_s16((___p) +  8,(x).val[1]), vipmvi_store1x16_s16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_store1x16_s32(___p,(x).val[0]), vipmvi_store1x16_s32((___p) +  4,(x).val[1]), vipmvi_store1x16_s32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_store1x16_u8( ___p,(x).val[0]), vipmvi_store1x16_u8( (___p) + 16,(x).val[1]), vipmvi_store1x16_u8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_store1x16_u16(___p,(x).val[0]), vipmvi_store1x16_u16((___p) +  8,(x).val[1]), vipmvi_store1x16_u16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_store1x16_u32(___p,(x).val[0]), vipmvi_store1x16_u32((___p) +  4,(x).val[1]), vipmvi_store1x16_u32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_store1x16x3_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_store1x16_f32(___p,(x).val[0]), vipmvi_store1x16_f32((___p) +  4,(x).val[1]), vipmvi_store1x16_f32((___p) +  8,(x).val[2])); } while (0)

/* without polluting the caches */
#  define /* void */           vipmvi_storea1x16x3_npc_s8(p,x)  do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x16_npc_s8( ___p,(x).val[0]), vipmvi_storea1x16_npc_s8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_npc_s8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_npc_s16(p,x) do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x16_npc_s16(___p,(x).val[0]), vipmvi_storea1x16_npc_s16((___p) +  8,(x).val[1]), vipmvi_storea1x16_npc_s16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_npc_s32(p,x) do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x16_npc_s32(___p,(x).val[0]), vipmvi_storea1x16_npc_s32((___p) +  4,(x).val[1]), vipmvi_storea1x16_npc_s32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_npc_u8(p,x)  do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x16_npc_u8( ___p,(x).val[0]), vipmvi_storea1x16_npc_u8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_npc_u8( (___p) + 32,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_npc_u16(p,x) do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x16_npc_u16(___p,(x).val[0]), vipmvi_storea1x16_npc_u16((___p) +  8,(x).val[1]), vipmvi_storea1x16_npc_u16((___p) + 16,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_npc_u32(p,x) do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x16_npc_u32(___p,(x).val[0]), vipmvi_storea1x16_npc_u32((___p) +  4,(x).val[1]), vipmvi_storea1x16_npc_u32((___p) +  8,(x).val[2])); } while (0)
#  define /* void */           vipmvi_storea1x16x3_npc_f32(p,x) do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x16_npc_f32(___p,(x).val[0]), vipmvi_storea1x16_npc_f32((___p) +  4,(x).val[1]), vipmvi_storea1x16_npc_f32((___p) +  8,(x).val[2])); } while (0)


		/*
		 * Transfer 16x4=64 bytes
		 * 4 Q registers
		 */

#  define /* bo_m128x4_s8_t  */vipmvi_loada1x16x4_s8(p)         vipmvi__loada1x16x4_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x4_s16_t */vipmvi_loada1x16x4_s16(p)        vipmvi__loada1x16x4_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x4_s32_t */vipmvi_loada1x16x4_s32(p)        vipmvi__loada1x16x4_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x4_u8_t  */vipmvi_loada1x16x4_u8(p)         vipmvi__loada1x16x4_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x4_u16_t */vipmvi_loada1x16x4_u16(p)        vipmvi__loada1x16x4_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x4_u32_t */vipmvi_loada1x16x4_u32(p)        vipmvi__loada1x16x4_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x4_f32_t */vipmvi_loada1x16x4_f32(p)        vipmvi__loada1x16x4_f32((bo_float32_t const *)(p))

#  define /* bo_m128x4_s8_t  */vipmvi_loadu1x16x4_s8(p)         vipmvi__loadu1x16x4_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x4_s16_t */vipmvi_loadu1x16x4_s16(p)        vipmvi__loadu1x16x4_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x4_s32_t */vipmvi_loadu1x16x4_s32(p)        vipmvi__loadu1x16x4_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x4_u8_t  */vipmvi_loadu1x16x4_u8(p)         vipmvi__loadu1x16x4_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x4_u16_t */vipmvi_loadu1x16x4_u16(p)        vipmvi__loadu1x16x4_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x4_u32_t */vipmvi_loadu1x16x4_u32(p)        vipmvi__loadu1x16x4_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x4_f32_t */vipmvi_loadu1x16x4_f32(p)        vipmvi__loadu1x16x4_f32((bo_float32_t const *)(p))

#  define /* bo_m128x4_s8_t  */vipmvi_load1x16x4_s8(p)          vipmvi__load1x16x4_s8( (bo_int8_t    const *)(p))
#  define /* bo_m128x4_s16_t */vipmvi_load1x16x4_s16(p)         vipmvi__load1x16x4_s16((bo_int16_t   const *)(p))
#  define /* bo_m128x4_s32_t */vipmvi_load1x16x4_s32(p)         vipmvi__load1x16x4_s32((bo_int32_t   const *)(p))
#  define /* bo_m128x4_u8_t  */vipmvi_load1x16x4_u8(p)          vipmvi__load1x16x4_u8( (bo_uint8_t   const *)(p))
#  define /* bo_m128x4_u16_t */vipmvi_load1x16x4_u16(p)         vipmvi__load1x16x4_u16((bo_uint16_t  const *)(p))
#  define /* bo_m128x4_u32_t */vipmvi_load1x16x4_u32(p)         vipmvi__load1x16x4_u32((bo_uint32_t  const *)(p))
#  define /* bo_m128x4_f32_t */vipmvi_load1x16x4_f32(p)         vipmvi__load1x16x4_f32((bo_float32_t const *)(p))

#  define /* void */           vipmvi_storea1x16x4_s8(p,x)      do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x16_s8( ___p,(x).val[0]), vipmvi_storea1x16_s8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_s8( (___p) + 32,(x).val[2]), vipmvi_storea1x16_s8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_s16(p,x)     do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x16_s16(___p,(x).val[0]), vipmvi_storea1x16_s16((___p) +  8,(x).val[1]), vipmvi_storea1x16_s16((___p) + 16,(x).val[2]), vipmvi_storea1x16_s16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_s32(p,x)     do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x16_s32(___p,(x).val[0]), vipmvi_storea1x16_s32((___p) +  4,(x).val[1]), vipmvi_storea1x16_s32((___p) +  8,(x).val[2]), vipmvi_storea1x16_s32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_u8(p,x)      do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x16_u8( ___p,(x).val[0]), vipmvi_storea1x16_u8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_u8( (___p) + 32,(x).val[2]), vipmvi_storea1x16_u8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_u16(p,x)     do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x16_u16(___p,(x).val[0]), vipmvi_storea1x16_u16((___p) +  8,(x).val[1]), vipmvi_storea1x16_u16((___p) + 16,(x).val[2]), vipmvi_storea1x16_u16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_u32(p,x)     do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x16_u32(___p,(x).val[0]), vipmvi_storea1x16_u32((___p) +  4,(x).val[1]), vipmvi_storea1x16_u32((___p) +  8,(x).val[2]), vipmvi_storea1x16_u32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_f32(p,x)     do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x16_f32(___p,(x).val[0]), vipmvi_storea1x16_f32((___p) +  4,(x).val[1]), vipmvi_storea1x16_f32((___p) +  8,(x).val[2]), vipmvi_storea1x16_f32((___p) + 12,(x).val[3])); } while (0)

#  define /* void */           vipmvi_storeu1x16x4_s8(p,x)      do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storeu1x16_s8( ___p,(x).val[0]), vipmvi_storeu1x16_s8( (___p) + 16,(x).val[1]), vipmvi_storeu1x16_s8( (___p) + 32,(x).val[2]), vipmvi_storeu1x16_s8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_s16(p,x)     do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storeu1x16_s16(___p,(x).val[0]), vipmvi_storeu1x16_s16((___p) +  8,(x).val[1]), vipmvi_storeu1x16_s16((___p) + 16,(x).val[2]), vipmvi_storeu1x16_s16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_s32(p,x)     do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storeu1x16_s32(___p,(x).val[0]), vipmvi_storeu1x16_s32((___p) +  4,(x).val[1]), vipmvi_storeu1x16_s32((___p) +  8,(x).val[2]), vipmvi_storeu1x16_s32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_u8(p,x)      do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storeu1x16_u8( ___p,(x).val[0]), vipmvi_storeu1x16_u8( (___p) + 16,(x).val[1]), vipmvi_storeu1x16_u8( (___p) + 32,(x).val[2]), vipmvi_storeu1x16_u8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_u16(p,x)     do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storeu1x16_u16(___p,(x).val[0]), vipmvi_storeu1x16_u16((___p) +  8,(x).val[1]), vipmvi_storeu1x16_u16((___p) + 16,(x).val[2]), vipmvi_storeu1x16_u16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_u32(p,x)     do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storeu1x16_u32(___p,(x).val[0]), vipmvi_storeu1x16_u32((___p) +  4,(x).val[1]), vipmvi_storeu1x16_u32((___p) +  8,(x).val[2]), vipmvi_storeu1x16_u32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storeu1x16x4_f32(p,x)     do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storeu1x16_f32(___p,(x).val[0]), vipmvi_storeu1x16_f32((___p) +  4,(x).val[1]), vipmvi_storeu1x16_f32((___p) +  8,(x).val[2]), vipmvi_storeu1x16_f32((___p) + 12,(x).val[3])); } while (0)

#  define /* void */           vipmvi_store1x16x4_s8(p,x)       do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_store1x16_s8( ___p,(x).val[0]), vipmvi_store1x16_s8( (___p) + 16,(x).val[1]), vipmvi_store1x16_s8( (___p) + 32,(x).val[2]), vipmvi_store1x16_s8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x16x4_s16(p,x)      do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_store1x16_s16(___p,(x).val[0]), vipmvi_store1x16_s16((___p) +  8,(x).val[1]), vipmvi_store1x16_s16((___p) + 16,(x).val[2]), vipmvi_store1x16_s16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x16x4_s32(p,x)      do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_store1x16_s32(___p,(x).val[0]), vipmvi_store1x16_s32((___p) +  4,(x).val[1]), vipmvi_store1x16_s32((___p) +  8,(x).val[2]), vipmvi_store1x16_s32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x16x4_u8(p,x)       do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_store1x16_u8( ___p,(x).val[0]), vipmvi_store1x16_u8( (___p) + 16,(x).val[1]), vipmvi_store1x16_u8( (___p) + 32,(x).val[2]), vipmvi_store1x16_u8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x16x4_u16(p,x)      do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_store1x16_u16(___p,(x).val[0]), vipmvi_store1x16_u16((___p) +  8,(x).val[1]), vipmvi_store1x16_u16((___p) + 16,(x).val[2]), vipmvi_store1x16_u16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x16x4_u32(p,x)      do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_store1x16_u32(___p,(x).val[0]), vipmvi_store1x16_u32((___p) +  4,(x).val[1]), vipmvi_store1x16_u32((___p) +  8,(x).val[2]), vipmvi_store1x16_u32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_store1x16x4_f32(p,x)      do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_store1x16_f32(___p,(x).val[0]), vipmvi_store1x16_f32((___p) +  4,(x).val[1]), vipmvi_store1x16_f32((___p) +  8,(x).val[2]), vipmvi_store1x16_f32((___p) + 12,(x).val[3])); } while (0)

/* without polluting the caches */
#  define /* void */           vipmvi_storea1x16x4_npc_s8(p,x)  do { bo_int8_t    *___p = (bo_int8_t    *)(p); (vipmvi_storea1x16_npc_s8( ___p,(x).val[0]), vipmvi_storea1x16_npc_s8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_npc_s8( (___p) + 32,(x).val[2]), vipmvi_storea1x16_npc_s8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_npc_s16(p,x) do { bo_int16_t   *___p = (bo_int16_t   *)(p); (vipmvi_storea1x16_npc_s16(___p,(x).val[0]), vipmvi_storea1x16_npc_s16((___p) +  8,(x).val[1]), vipmvi_storea1x16_npc_s16((___p) + 16,(x).val[2]), vipmvi_storea1x16_npc_s16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_npc_s32(p,x) do { bo_int32_t   *___p = (bo_int32_t   *)(p); (vipmvi_storea1x16_npc_s32(___p,(x).val[0]), vipmvi_storea1x16_npc_s32((___p) +  4,(x).val[1]), vipmvi_storea1x16_npc_s32((___p) +  8,(x).val[2]), vipmvi_storea1x16_npc_s32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_npc_u8(p,x)  do { bo_uint8_t   *___p = (bo_uint8_t   *)(p); (vipmvi_storea1x16_npc_u8( ___p,(x).val[0]), vipmvi_storea1x16_npc_u8( (___p) + 16,(x).val[1]), vipmvi_storea1x16_npc_u8( (___p) + 32,(x).val[2]), vipmvi_storea1x16_npc_u8( (___p) + 48,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_npc_u16(p,x) do { bo_uint16_t  *___p = (bo_uint16_t  *)(p); (vipmvi_storea1x16_npc_u16(___p,(x).val[0]), vipmvi_storea1x16_npc_u16((___p) +  8,(x).val[1]), vipmvi_storea1x16_npc_u16((___p) + 16,(x).val[2]), vipmvi_storea1x16_npc_u16((___p) + 24,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_npc_u32(p,x) do { bo_uint32_t  *___p = (bo_uint32_t  *)(p); (vipmvi_storea1x16_npc_u32(___p,(x).val[0]), vipmvi_storea1x16_npc_u32((___p) +  4,(x).val[1]), vipmvi_storea1x16_npc_u32((___p) +  8,(x).val[2]), vipmvi_storea1x16_npc_u32((___p) + 12,(x).val[3])); } while (0)
#  define /* void */           vipmvi_storea1x16x4_npc_f32(p,x) do { bo_float32_t *___p = (bo_float32_t *)(p); (vipmvi_storea1x16_npc_f32(___p,(x).val[0]), vipmvi_storea1x16_npc_f32((___p) +  4,(x).val[1]), vipmvi_storea1x16_npc_f32((___p) +  8,(x).val[2]), vipmvi_storea1x16_npc_f32((___p) + 12,(x).val[3])); } while (0)

	/*
	 * Load/Store 1-element structures
	 * @}
	 */


	/*
	 * Load/Store 2-element structures
	 * @{
	 */

		/*
		 * Transfer 2x8=16 bytes
		 * 2 D registers
		 */

//# define /* bo_m64x2_s8_t  */  vipmvi_load2x8_s8(p)         vld2_s8((void const *)(p))
//# define /* bo_m64x2_s16_t */  vipmvi_load2x8_s16(p)        vld2_s16((void const *)(p))
//# define /* bo_m64x2_s32_t */  vipmvi_load2x8_s32(p)        vld2_s32((void const *)(p))
//# define /* bo_m64x2_u8_t  */  vipmvi_load2x8_u8(p)         vld2_u8((void const *)(p))
//# define /* bo_m64x2_u16_t */  vipmvi_load2x8_u16(p)        vld2_u16((void const *)(p))
//# define /* bo_m64x2_u32_t */  vipmvi_load2x8_u32(p)        vld2_u32((void const *)(p))
//# define /* bo_m64x2_f32_t */  vipmvi_load2x8_f32(p)        vld2_f32((void const *)(p))
//
//# define /* void */            vipmvi_store2x8_s8(p,x)      vst2_s8((void *)(p),x)
//# define /* void */            vipmvi_store2x8_s16(p,x)     vst2_s16((void *)(p),x)
//# define /* void */            vipmvi_store2x8_s32(p,x)     vst2_s32((void *)(p),x)
//# define /* void */            vipmvi_store2x8_u8(p,x)      vst2_u8((void *)(p),x)
//# define /* void */            vipmvi_store2x8_u16(p,x)     vst2_u16((void *)(p),x)
//# define /* void */            vipmvi_store2x8_u32(p,x)     vst2_u32((void *)(p),x)
//# define /* void */            vipmvi_store2x8_f32(p,x)     vst2_f32((void *)(p),x)


		/*
		 * Transfer 2x16=32 bytes
		 * 2 Q registers
		 */

//# define /* bo_m128x2_s8_t  */ vipmvi_load2x16_s8(p)        vld2q_s8((void const *)(p))
//# define /* bo_m128x2_s16_t */ vipmvi_load2x16_s16(p)       vld2q_s16((void const *)(p))
//# define /* bo_m128x2_s32_t */ vipmvi_load2x16_s32(p)       vld2q_s32((void const *)(p))
//# define /* bo_m128x2_u8_t  */ vipmvi_load2x16_u8(p)        vld2q_u8((void const *)(p))
//# define /* bo_m128x2_u16_t */ vipmvi_load2x16_u16(p)       vld2q_u16((void const *)(p))
//# define /* bo_m128x2_u32_t */ vipmvi_load2x16_u32(p)       vld2q_u32((void const *)(p))
//# define /* bo_m128x2_f32_t */ vipmvi_load2x16_f32(p)       vld2q_f32((void const *)(p))
//
//# define /* void */            vipmvi_store2x16_s8(p,x)     vst2q_s8((void *)(p),x)
//# define /* void */            vipmvi_store2x16_s16(p,x)    vst2q_s16((void *)(p),x)
//# define /* void */            vipmvi_store2x16_s32(p,x)    vst2q_s32((void *)(p),x)
//# define /* void */            vipmvi_store2x16_u8(p,x)     vst2q_u8((void *)(p),x)
//# define /* void */            vipmvi_store2x16_u16(p,x)    vst2q_u16((void *)(p),x)
//# define /* void */            vipmvi_store2x16_u32(p,x)    vst2q_u32((void *)(p),x)
//# define /* void */            vipmvi_store2x16_f32(p,x)    vst2q_f32((void *)(p),x)

	/*
	 * Load/Store 2-element structures
	 * @}
	 */


	/*
	 * Load/Store 3-element structures
	 * @{
	 */

		/*
		 * Transfer 3x8=24 bytes
		 * 3 D registers
		 */

//# define /* bo_m64x3_s8_t  */  vipmvi_load3x8_s8(p)         vld3_s8((void const *)(p))
//# define /* bo_m64x3_s16_t */  vipmvi_load3x8_s16(p)        vld3_s16((void const *)(p))
//# define /* bo_m64x3_s32_t */  vipmvi_load3x8_s32(p)        vld3_s32((void const *)(p))
//# define /* bo_m64x3_u8_t  */  vipmvi_load3x8_u8(p)         vld3_u8((void const *)(p))
//# define /* bo_m64x3_u16_t */  vipmvi_load3x8_u16(p)        vld3_u16((void const *)(p))
//# define /* bo_m64x3_u32_t */  vipmvi_load3x8_u32(p)        vld3_u32((void const *)(p))
//# define /* bo_m64x3_f32_t */  vipmvi_load3x8_f32(p)        vld3_f32((void const *)(p))
//
//# define /* void */            vipmvi_store3x8_s8(p,x)      vst3_s8((void *)(p),x)
//# define /* void */            vipmvi_store3x8_s16(p,x)     vst3_s16((void *)(p),x)
//# define /* void */            vipmvi_store3x8_s32(p,x)     vst3_s32((void *)(p),x)
//# define /* void */            vipmvi_store3x8_u8(p,x)      vst3_u8((void *)(p),x)
//# define /* void */            vipmvi_store3x8_u16(p,x)     vst3_u16((void *)(p),x)
//# define /* void */            vipmvi_store3x8_u32(p,x)     vst3_u32((void *)(p),x)
//# define /* void */            vipmvi_store3x8_f32(p,x)     vst3_f32((void *)(p),x)


		/*
		 * Transfer 3x16=48 bytes
		 * 3 Q registers
		 */

//# define /* bo_m128x3_s8_t  */ vipmvi_load3x16_s8(p)        vld3q_s8((void const *)(p))
//# define /* bo_m128x3_s16_t */ vipmvi_load3x16_s16(p)       vld3q_s16((void const *)(p))
//# define /* bo_m128x3_s32_t */ vipmvi_load3x16_s32(p)       vld3q_s32((void const *)(p))
//# define /* bo_m128x3_u8_t  */ vipmvi_load3x16_u8(p)        vld3q_u8((void const *)(p))
//# define /* bo_m128x3_u16_t */ vipmvi_load3x16_u16(p)       vld3q_u16((void const *)(p))
//# define /* bo_m128x3_u32_t */ vipmvi_load3x16_u32(p)       vld3q_u32((void const *)(p))
//# define /* bo_m128x3_f32_t */ vipmvi_load3x16_f32(p)       vld3q_f32((void const *)(p))
//
//# define /* void */            vipmvi_store3x16_s8(p,x)     vst3q_s8((void *)(p),x)
//# define /* void */            vipmvi_store3x16_s16(p,x)    vst3q_s16((void *)(p),x)
//# define /* void */            vipmvi_store3x16_s32(p,x)    vst3q_s32((void *)(p),x)
//# define /* void */            vipmvi_store3x16_u8(p,x)     vst3q_u8((void *)(p),x)
//# define /* void */            vipmvi_store3x16_u16(p,x)    vst3q_u16((void *)(p),x)
//# define /* void */            vipmvi_store3x16_u32(p,x)    vst3q_u32((void *)(p),x)
//# define /* void */            vipmvi_store3x16_f32(p,x)    vst3q_f32((void *)(p),x)

	/*
	 * Load/Store 3-element structures
	 * @}
	 */


	/*
	 * Load/Store 4-element structures
	 * @{
	 */

		/*
		 * Transfer 4x8=32 bytes
		 * 4 D registers
		 */

//# define /* bo_m64x4_s8_t  */  vipmvi_load4x8_s8(p)         vld4_s8((void const *)(p))
//# define /* bo_m64x4_s16_t */  vipmvi_load4x8_s16(p)        vld4_s16((void const *)(p))
//# define /* bo_m64x4_s32_t */  vipmvi_load4x8_s32(p)        vld4_s32((void const *)(p))
//# define /* bo_m64x4_u8_t  */  vipmvi_load4x8_u8(p)         vld4_u8((void const *)(p))
//# define /* bo_m64x4_u16_t */  vipmvi_load4x8_u16(p)        vld4_u16((void const *)(p))
//# define /* bo_m64x4_u32_t */  vipmvi_load4x8_u32(p)        vld4_u32((void const *)(p))
//# define /* bo_m64x4_f32_t */  vipmvi_load4x8_f32(p)        vld4_f32((void const *)(p))
//
//# define /* void */            vipmvi_store4x8_s8(p,x)      vst4_s8((void *)(p),x)
//# define /* void */            vipmvi_store4x8_s16(p,x)     vst4_s16((void *)(p),x)
//# define /* void */            vipmvi_store4x8_s32(p,x)     vst4_s32((void *)(p),x)
//# define /* void */            vipmvi_store4x8_u8(p,x)      vst4_u8((void *)(p),x)
//# define /* void */            vipmvi_store4x8_u16(p,x)     vst4_u16((void *)(p),x)
//# define /* void */            vipmvi_store4x8_u32(p,x)     vst4_u32((void *)(p),x)
//# define /* void */            vipmvi_store4x8_f32(p,x)     vst4_f32((void *)(p),x)


		/*
		 * Transfer 4x16=64 bytes
		 * 4 Q registers
		 */

//# define /* bo_m128x4_s8_t  */ vipmvi_load4x16_s8(p)        vld4q_s8((void const *)(p))
//# define /* bo_m128x4_s16_t */ vipmvi_load4x16_s16(p)       vld4q_s16((void const *)(p))
//# define /* bo_m128x4_s32_t */ vipmvi_load4x16_s32(p)       vld4q_s32((void const *)(p))
//# define /* bo_m128x4_u8_t  */ vipmvi_load4x16_u8(p)        vld4q_u8((void const *)(p))
//# define /* bo_m128x4_u16_t */ vipmvi_load4x16_u16(p)       vld4q_u16((void const *)(p))
//# define /* bo_m128x4_u32_t */ vipmvi_load4x16_u32(p)       vld4q_u32((void const *)(p))
//# define /* bo_m128x4_f32_t */ vipmvi_load4x16_f32(p)       vld4q_f32((void const *)(p))
//
//# define /* void */            vipmvi_store4x16_s8(p,x)     vst4q_s8((void *)(p),x)
//# define /* void */            vipmvi_store4x16_s16(p,x)    vst4q_s16((void *)(p),x)
//# define /* void */            vipmvi_store4x16_s32(p,x)    vst4q_s32((void *)(p),x)
//# define /* void */            vipmvi_store4x16_u8(p,x)     vst4q_u8((void *)(p),x)
//# define /* void */            vipmvi_store4x16_u16(p,x)    vst4q_u16((void *)(p),x)
//# define /* void */            vipmvi_store4x16_u32(p,x)    vst4q_u32((void *)(p),x)
//# define /* void */            vipmvi_store4x16_f32(p,x)    vst4q_f32((void *)(p),x)

	/*
	 * Load/Store 4-element structures
	 * @}
	 */

/* @} */

/* ------------------------------------------------------------------------- */

/*
 * Extract/Put lanes from/to a vector
 * @{
 */

	/*
	 * Get lane from D register
	 */

# define /* int8_t */          vipmvi_lane8_s8( /* bo_m64_s8_t  */x,imm)    ((x).v[imm])
# define /* int16_t */         vipmvi_lane8_s16(/* bo_m64_s16_t */x,imm)    ((x).v[imm])
# define /* int32_t */         vipmvi_lane8_s32(/* bo_m64_s32_t */x,imm)    ((x).v[imm])
# define /* int64_t */         vipmvi_lane8_s64(/* bo_m64_s64_t */x,imm)    ((x).v[imm])
# define /* uint8_t */         vipmvi_lane8_u8( /* bo_m64_u8_t  */x,imm)    ((x).v[imm])
# define /* uint16_t */        vipmvi_lane8_u16(/* bo_m64_u16_t */x,imm)    ((x).v[imm])
# define /* uint32_t */        vipmvi_lane8_u32(/* bo_m64_u32_t */x,imm)    ((x).v[imm])
# define /* uint64_t */        vipmvi_lane8_u64(/* bo_m64_u64_t */x,imm)    ((x).v[imm])
# define /* float32_t */       vipmvi_lane8_f32(/* bo_m64_f32_t */x,imm)    ((x).v[imm])
# define /* float64_t */       vipmvi_lane8_f64(/* bo_m64_f64_t */x,imm)    ((x).v[imm])


	/*
	 * Get lane from Q register
	 */

# define /* int8_t */          vipmvi_lane16_s8( /* bo_m128_s8_t  */x,imm)    ((bo_int8_t)  _mm_extract_epi8(x,imm))
# define /* int16_t */         vipmvi_lane16_s16(/* bo_m128_s16_t */x,imm)    ((bo_int16_t) _mm_extract_epi16(x,imm))
# define /* int32_t */         vipmvi_lane16_s32(/* bo_m128_s32_t */x,imm)    ((bo_int32_t) _mm_extract_epi32(x,imm))
# define /* int64_t */         vipmvi_lane16_s64(/* bo_m128_s64_t */x,imm)    ((bo_int64_t) _mm_extract_epi64(x,imm))
# define /* uint8_t */         vipmvi_lane16_u8( /* bo_m128_u8_t  */x,imm)    ((bo_uint8_t) _mm_extract_epi8(x,imm))
# define /* uint16_t */        vipmvi_lane16_u16(/* bo_m128_u16_t */x,imm)    ((bo_uint16_t)_mm_extract_epi16(x,imm))
# define /* uint32_t */        vipmvi_lane16_u32(/* bo_m128_u32_t */x,imm)    ((bo_uint32_t)_mm_extract_epi32(x,imm))
# define /* uint64_t */        vipmvi_lane16_u64(/* bo_m128_u64_t */x,imm)    ((bo_uint64_t)_mm_extract_epi64(x,imm))
# define /* float32_t */       vipmvi_lane16_f32(/* bo_m128_f32_t */x,imm)    vipmvi__lane16_f32(x,imm)
# define /* float64_t */       vipmvi_lane16_f64(/* bo_m128_f64_t */x,imm)    vipmvi__lane16_f64(x,imm)

/* @} */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*
 * Load/Store
 * @{
 */

/* @note: internal @{ */
# define vipmvi__loada_si128(p)         _mm_load_si128((__m128i const *)(p))
//# define vipmvi__loadu_si128(p)         _mm_lddqu_si128((__m128i const *)(p)) // SSE3
# define vipmvi__loadu_si128(p)         _mm_loadu_si128((__m128i const *)(p))
# define vipmvi__load_si128(p)          (vipmvi__load_si128)((__m128i const *)(p))

# define vipmvi__storea_si128(p,x)      _mm_store_si128((__m128i *)(p), x)
# define vipmvi__storea_npc_si128(p,x)  _mm_stream_si128((__m128i *)(p), x)
# define vipmvi__storeu_si128(p,x)      _mm_storeu_si128((__m128i *)(p), x)
# define vipmvi__store_si128(p,x)       (vipmvi__store_si128)((__m128i *)(p), x)

# define vipmvi__loada_sf128(p)         _mm_load_ps((bo_float32_t const *)(p))
# define vipmvi__loadu_sf128(p)         _mm_loadu_ps((bo_float32_t const *)(p))
# define vipmvi__load_sf128(p)          (vipmvi__load_sf128)((bo_float32_t const *)(p))

# define vipmvi__storea_sf128(p,x)      _mm_store_ps((bo_float32_t *)(p), x)
# define vipmvi__storea_npc_sf128(p,x)  _mm_stream_ps((bo_float32_t *)(p), x)
# define vipmvi__storeu_sf128(p,x)      _mm_storeu_ps((bo_float32_t *)(p), x)
# define vipmvi__store_sf128(p,x)       (vipmvi__store_sf128)((bo_float32_t *)(p), x)

# define vipmvi__loada_df128(p)         _mm_load_pd((bo_float64_t const *)(p))
# define vipmvi__loadu_df128(p)         _mm_loadu_pd((bo_float64_t const *)(p))
# define vipmvi__load_df128(p)          (vipmvi__load_df128)((bo_float64_t const *)(p))

# define vipmvi__storea_df128(p,x)      _mm_store_pd((bo_float64_t *)(p), x)
# define vipmvi__storea_npc_df128(p,x)  _mm_stream_pd((bo_float64_t *)(p), x)
# define vipmvi__storeu_df128(p,x)      _mm_storeu_pd((bo_float64_t *)(p), x)
# define vipmvi__store_df128(p,x)       (vipmvi__store_df128)((bo_float64_t *)(p), x)
/* @} */



_VIPM_INLINE __m128i (vipmvi__load_si128)(__m128i const *p)
{
	return ( (((bo_uintptr_t)(p) & 15) == 0) ? vipmvi__loada_si128(p) : vipmvi__loadu_si128(p) );
}

_VIPM_INLINE void (vipmvi__store_si128)(__m128i *p, __m128i x)
{
	if (((bo_uintptr_t)(p) & 15) == 0) {
		vipmvi__storea_si128(p, x);
		return;
	}
	vipmvi__storeu_si128(p, x);
	return;
}


_VIPM_INLINE __m128 (vipmvi__load_sf128)(bo_float32_t const *p)
{
	return ( (((bo_uintptr_t)(p) & 15) == 0) ? vipmvi__loada_sf128(p) : vipmvi__loadu_sf128(p) );
}

_VIPM_INLINE void (vipmvi__store_sf128)(bo_float32_t *p, __m128 x)
{
	if (((bo_uintptr_t)(p) & 15) == 0) {
		vipmvi__storea_sf128(p, x);
		return;
	}
	vipmvi__storeu_sf128(p, x);
	return;
}

_VIPM_INLINE __m128d (vipmvi__load_df128)(bo_float64_t const *p)
{
	return ( (((bo_uintptr_t)(p) & 15) == 0) ? vipmvi__loada_df128(p) : vipmvi__loadu_df128(p) );
}

_VIPM_INLINE void (vipmvi__store_df128)(bo_float64_t *p, __m128d x)
{
	if (((bo_uintptr_t)(p) & 15) == 0) {
		vipmvi__storea_df128(p, x);
		return;
	}
	vipmvi__storeu_df128(p, x);
	return;
}


/* @todo: fixme */
# if defined(_MSC_VER)
#  pragma intrinsic(__movsb)
#  pragma intrinsic(__movsw)
#  pragma intrinsic(__movsd)
# else
# if defined(_A_X86) || defined(_A_X86_64)
_VIPM_INLINE void __movsb(void *p, const void *x, size_t nelts) {
	__asm__ __volatile__ ("rep movsb" : "+D" (p), "+S" (x), "+c" (nelts) : : "memory");
}
_VIPM_INLINE void __movsw(void *p, const void *x, size_t nelts) {
	__asm__ __volatile__ ("rep movsw" : "+D" (p), "+S" (x), "+c" (nelts) : : "memory");
}
_VIPM_INLINE void __movsd(void *p, const void *x, size_t nelts) {
	__asm__ __volatile__ ("rep movsl" : "+D" (p), "+S" (x), "+c" (nelts) : : "memory");
}
# else
# endif
# endif

/* @todo: fixme - retest __movs[bwd] */
#if 1
_VIPM_INLINE void (__vipmvi__store1x8_u8) (bo_uint8_t  *p, bo_uint8_t const  *x) { __movsb(p, x, 8); }
_VIPM_INLINE void (__vipmvi__store1x8_u16)(bo_uint16_t *p, bo_uint16_t const *x) { __movsw(p, x, 4); }
_VIPM_INLINE void (__vipmvi__store1x8_u32)(bo_uint32_t *p, bo_uint32_t const *x) { __movsd(p, x, 2); }
#else
_VIPM_INLINE void (__vipmvi__store1x8_u8) (bo_uint8_t *p, bo_uint8_t const *x)
{
	int i;
	for (i = 0; i < 8; i++)
		p[i] = x[i];
}
_VIPM_INLINE void (__vipmvi__store1x8_u16) (bo_uint16_t *p, bo_uint16_t const *x)
{
	int i;
	for (i = 0; i < 4; i++)
		p[i] = x[i];
}
_VIPM_INLINE void (__vipmvi__store1x8_u32) (bo_uint32_t *p, bo_uint32_t const *x)
{
	p[0] = x[0];
	p[1] = x[1];
}
#endif

_VIPM_INLINE void (vipmvi__store1x8_s8) (bo_uint8_t  *p, bo_m64_s8_t x)  { __vipmvi__store1x8_u8(p, (bo_uint8_t const *)&x); }
_VIPM_INLINE void (vipmvi__store1x8_s16)(bo_uint16_t *p, bo_m64_s16_t x) { __vipmvi__store1x8_u16(p,(bo_uint16_t const *)&x); }
_VIPM_INLINE void (vipmvi__store1x8_s32)(bo_uint32_t *p, bo_m64_s32_t x) { __vipmvi__store1x8_u32(p,(bo_uint32_t const *)&x); }
_VIPM_INLINE void (vipmvi__store1x8_u8) (bo_uint8_t  *p, bo_m64_u8_t x)  { __vipmvi__store1x8_u8(p, (bo_uint8_t const *)&x); }
_VIPM_INLINE void (vipmvi__store1x8_u16)(bo_uint16_t *p, bo_m64_u16_t x) { __vipmvi__store1x8_u16(p,(bo_uint16_t const *)&x); }
_VIPM_INLINE void (vipmvi__store1x8_u32)(bo_uint32_t *p, bo_m64_u32_t x) { __vipmvi__store1x8_u32(p,(bo_uint32_t const *)&x); }


# if defined(_GCC_VER)
#  define vipmvi__load1x8_f32(p) __extension__ ({ bo_float32_t const *___p = (p); (bo_m64_f32_t){{ ___p[0], ___p[1] }}; })
# else
_VIPM_INLINE bo_m64_f32_t (vipmvi__load1x8_f32)(bo_float32_t const *p)
{
    bo_m64_f32_t rv;
    rv.v[0] = p[0];
    rv.v[1] = p[1];
    return (rv);
}
# endif

_VIPM_INLINE void (vipmvi__store1x8_f32)(bo_float32_t *p, bo_m64_f32_t x) { p[0] = x.v[0]; p[1] = x.v[1]; }

/* ------------------------------------------------------------------------- */

	/*
	 * Load/Store 1-element structures
	 * @{
	 */

		/*
		 * Transfer 8x2=16 bytes
		 * 2 D registers
		 */

// @todo: fixme - compiler
# ifdef _GCC_VER

#  define vipmvi__loada1x8x2_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x2_s8_t) {{ vipmvi_loada1x8_s8( ___p), vipmvi_loada1x8_s8( (___p) + 8) }}; })
#  define vipmvi__loada1x8x2_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x2_s16_t){{ vipmvi_loada1x8_s16(___p), vipmvi_loada1x8_s16((___p) + 4) }}; })
#  define vipmvi__loada1x8x2_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x2_s32_t){{ vipmvi_loada1x8_s32(___p), vipmvi_loada1x8_s32((___p) + 2) }}; })
#  define vipmvi__loada1x8x2_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x2_u8_t) {{ vipmvi_loada1x8_u8( ___p), vipmvi_loada1x8_u8( (___p) + 8) }}; })
#  define vipmvi__loada1x8x2_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x2_u16_t){{ vipmvi_loada1x8_u16(___p), vipmvi_loada1x8_u16((___p) + 4) }}; })
#  define vipmvi__loada1x8x2_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x2_u32_t){{ vipmvi_loada1x8_u32(___p), vipmvi_loada1x8_u32((___p) + 2) }}; })
#  define vipmvi__loada1x8x2_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x2_f32_t){{ vipmvi_loada1x8_f32(___p), vipmvi_loada1x8_f32((___p) + 2) }}; })

#  define vipmvi__loadu1x8x2_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x2_s8_t) {{ vipmvi_loadu1x8_s8( ___p), vipmvi_loadu1x8_s8( (___p) + 8) }}; })
#  define vipmvi__loadu1x8x2_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x2_s16_t){{ vipmvi_loadu1x8_s16(___p), vipmvi_loadu1x8_s16((___p) + 4) }}; })
#  define vipmvi__loadu1x8x2_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x2_s32_t){{ vipmvi_loadu1x8_s32(___p), vipmvi_loadu1x8_s32((___p) + 2) }}; })
#  define vipmvi__loadu1x8x2_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x2_u8_t) {{ vipmvi_loadu1x8_u8( ___p), vipmvi_loadu1x8_u8( (___p) + 8) }}; })
#  define vipmvi__loadu1x8x2_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x2_u16_t){{ vipmvi_loadu1x8_u16(___p), vipmvi_loadu1x8_u16((___p) + 4) }}; })
#  define vipmvi__loadu1x8x2_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x2_u32_t){{ vipmvi_loadu1x8_u32(___p), vipmvi_loadu1x8_u32((___p) + 2) }}; })
#  define vipmvi__loadu1x8x2_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x2_f32_t){{ vipmvi_loadu1x8_f32(___p), vipmvi_loadu1x8_f32((___p) + 2) }}; })

#  define vipmvi__load1x8x2_s8(p)           __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x2_s8_t) {{ vipmvi_load1x8_s8( ___p), vipmvi_load1x8_s8( (___p) + 8) }}; })
#  define vipmvi__load1x8x2_s16(p)          __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x2_s16_t){{ vipmvi_load1x8_s16(___p), vipmvi_load1x8_s16((___p) + 4) }}; })
#  define vipmvi__load1x8x2_s32(p)          __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x2_s32_t){{ vipmvi_load1x8_s32(___p), vipmvi_load1x8_s32((___p) + 2) }}; })
#  define vipmvi__load1x8x2_u8(p)           __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x2_u8_t) {{ vipmvi_load1x8_u8( ___p), vipmvi_load1x8_u8( (___p) + 8) }}; })
#  define vipmvi__load1x8x2_u16(p)          __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x2_u16_t){{ vipmvi_load1x8_u16(___p), vipmvi_load1x8_u16((___p) + 4) }}; })
#  define vipmvi__load1x8x2_u32(p)          __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x2_u32_t){{ vipmvi_load1x8_u32(___p), vipmvi_load1x8_u32((___p) + 2) }}; })
#  define vipmvi__load1x8x2_f32(p)          __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x2_f32_t){{ vipmvi_load1x8_f32(___p), vipmvi_load1x8_f32((___p) + 2) }}; })

# else /* default compiler */

_VIPM_INLINE bo_m64x2_s8_t  (vipmvi__loada1x8x2_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s8_t  rv;
	bo_m64_s8_t ra = (*(bo_m64_s8_t *)(p)), rb = (*(bo_m64_s8_t *)(p + 8));
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s8_t  rv = {{ vipmvi_loada1x8_s8(p),  vipmvi_loada1x8_s8(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_s16_t (vipmvi__loada1x8x2_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s16_t rv;
	bo_m64_s16_t ra = vipmvi_loada1x8_s16(p), rb = vipmvi_loada1x8_s16(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s16_t rv = {{ vipmvi_loada1x8_s16(p), vipmvi_loada1x8_s16(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_s32_t (vipmvi__loada1x8x2_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s32_t rv;
	bo_m64_s32_t ra = vipmvi_loada1x8_s32(p), rb = vipmvi_loada1x8_s32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s32_t rv = {{ vipmvi_loada1x8_s32(p), vipmvi_loada1x8_s32(p + 2) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u8_t  (vipmvi__loada1x8x2_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_loada1x8_u8(p), rb = vipmvi_loada1x8_u8(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u8_t  rv = {{ vipmvi_loada1x8_u8(p),  vipmvi_loada1x8_u8(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u16_t (vipmvi__loada1x8x2_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u16_t rv;
	bo_m64_u16_t ra = vipmvi_loada1x8_u16(p), rb = vipmvi_loada1x8_u16(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u16_t rv = {{ vipmvi_loada1x8_u16(p), vipmvi_loada1x8_u16(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u32_t (vipmvi__loada1x8x2_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u32_t rv;
	bo_m64_u32_t ra = vipmvi_loada1x8_u32(p), rb = vipmvi_loada1x8_u32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u32_t rv = {{ vipmvi_loada1x8_u32(p), vipmvi_loada1x8_u32(p + 2) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_f32_t (vipmvi__loada1x8x2_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_f32_t rv;
	bo_m64_f32_t ra = vipmvi_loada1x8_f32(p), rb = vipmvi_loada1x8_f32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_f32_t rv = {{ vipmvi_loada1x8_f32(p), vipmvi_loada1x8_f32(p + 2) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m64x2_s8_t  (vipmvi__loadu1x8x2_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_loadu1x8_s8(p), rb = vipmvi_loadu1x8_s8(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s8_t  rv = {{ vipmvi_loadu1x8_s8(p),  vipmvi_loadu1x8_s8(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_s16_t (vipmvi__loadu1x8x2_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s16_t rv;
	bo_m64_s16_t ra = vipmvi_loadu1x8_s16(p), rb = vipmvi_loadu1x8_s16(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s16_t rv = {{ vipmvi_loadu1x8_s16(p), vipmvi_loadu1x8_s16(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_s32_t (vipmvi__loadu1x8x2_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s32_t rv;
	bo_m64_s32_t ra = vipmvi_loadu1x8_s32(p), rb = vipmvi_loadu1x8_s32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s32_t rv = {{ vipmvi_loadu1x8_s32(p), vipmvi_loadu1x8_s32(p + 2) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u8_t  (vipmvi__loadu1x8x2_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_loadu1x8_u8(p), rb = vipmvi_loadu1x8_u8(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u8_t  rv = {{ vipmvi_loadu1x8_u8(p),  vipmvi_loadu1x8_u8(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u16_t (vipmvi__loadu1x8x2_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u16_t rv;
	bo_m64_u16_t ra = vipmvi_loadu1x8_u16(p), rb = vipmvi_loadu1x8_u16(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u16_t rv = {{ vipmvi_loadu1x8_u16(p), vipmvi_loadu1x8_u16(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u32_t (vipmvi__loadu1x8x2_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u32_t rv;
	bo_m64_u32_t ra = vipmvi_loadu1x8_u32(p), rb = vipmvi_loadu1x8_u32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u32_t rv = {{ vipmvi_loadu1x8_u32(p), vipmvi_loadu1x8_u32(p + 2) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_f32_t (vipmvi__loadu1x8x2_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_f32_t rv;
	bo_m64_f32_t ra = vipmvi_loadu1x8_f32(p), rb = vipmvi_loadu1x8_f32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_f32_t rv = {{ vipmvi_loadu1x8_f32(p), vipmvi_loadu1x8_f32(p + 2) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m64x2_s8_t  (vipmvi__load1x8x2_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_load1x8_s8(p), rb = vipmvi_load1x8_s8(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s8_t  rv = {{ vipmvi_load1x8_s8(p),  vipmvi_load1x8_s8(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_s16_t (vipmvi__load1x8x2_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s16_t rv;
	bo_m64_s16_t ra = vipmvi_load1x8_s16(p), rb = vipmvi_load1x8_s16(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s16_t rv = {{ vipmvi_load1x8_s16(p), vipmvi_load1x8_s16(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_s32_t (vipmvi__load1x8x2_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_s32_t rv;
	bo_m64_s32_t ra = vipmvi_load1x8_s32(p), rb = vipmvi_load1x8_s32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_s32_t rv = {{ vipmvi_load1x8_s32(p), vipmvi_load1x8_s32(p + 2) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u8_t  (vipmvi__load1x8x2_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_load1x8_u8(p), rb = vipmvi_load1x8_u8(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u8_t  rv = {{ vipmvi_load1x8_u8(p),  vipmvi_load1x8_u8(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u16_t (vipmvi__load1x8x2_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u16_t rv;
	bo_m64_u16_t ra = vipmvi_load1x8_u16(p), rb = vipmvi_load1x8_u16(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u16_t rv = {{ vipmvi_load1x8_u16(p), vipmvi_load1x8_u16(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_u32_t (vipmvi__load1x8x2_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_u32_t rv;
	bo_m64_u32_t ra = vipmvi_load1x8_u32(p), rb = vipmvi_load1x8_u32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_u32_t rv = {{ vipmvi_load1x8_u32(p), vipmvi_load1x8_u32(p + 2) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x2_f32_t (vipmvi__load1x8x2_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x2_f32_t rv;
	bo_m64_f32_t ra = vipmvi_load1x8_f32(p), rb = vipmvi_load1x8_f32(p + 2);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m64x2_f32_t rv = {{ vipmvi_load1x8_f32(p), vipmvi_load1x8_f32(p + 2) }};
#  endif
	return (rv);
}

# endif /* _GCC_VER */


		/*
		 * Transfer 8x3=24 bytes
		 * 3 D registers
		 */

// @todo: fixme - compiler
# ifdef _GCC_VER

#  define vipmvi__loada1x8x3_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x3_s8_t) {{ vipmvi_loada1x8_s8( ___p), vipmvi_loada1x8_s8( (___p) + 8), vipmvi_loada1x8_s8( (___p) + 16) }}; })
#  define vipmvi__loada1x8x3_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x3_s16_t){{ vipmvi_loada1x8_s16(___p), vipmvi_loada1x8_s16((___p) + 4), vipmvi_loada1x8_s16((___p) +  8) }}; })
#  define vipmvi__loada1x8x3_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x3_s32_t){{ vipmvi_loada1x8_s32(___p), vipmvi_loada1x8_s32((___p) + 2), vipmvi_loada1x8_s32((___p) +  4) }}; })
#  define vipmvi__loada1x8x3_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x3_u8_t) {{ vipmvi_loada1x8_u8( ___p), vipmvi_loada1x8_u8( (___p) + 8), vipmvi_loada1x8_u8( (___p) + 16) }}; })
#  define vipmvi__loada1x8x3_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x3_u16_t){{ vipmvi_loada1x8_u16(___p), vipmvi_loada1x8_u16((___p) + 4), vipmvi_loada1x8_u16((___p) +  8) }}; })
#  define vipmvi__loada1x8x3_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x3_u32_t){{ vipmvi_loada1x8_u32(___p), vipmvi_loada1x8_u32((___p) + 2), vipmvi_loada1x8_u32((___p) +  4) }}; })
#  define vipmvi__loada1x8x3_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x3_f32_t){{ vipmvi_loada1x8_f32(___p), vipmvi_loada1x8_f32((___p) + 2), vipmvi_loada1x8_f32((___p) +  4) }}; })

#  define vipmvi__loadu1x8x3_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x3_s8_t) {{ vipmvi_loadu1x8_s8( ___p), vipmvi_loadu1x8_s8( (___p) + 8), vipmvi_loadu1x8_s8( (___p) + 16) }}; })
#  define vipmvi__loadu1x8x3_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x3_s16_t){{ vipmvi_loadu1x8_s16(___p), vipmvi_loadu1x8_s16((___p) + 4), vipmvi_loadu1x8_s16((___p) +  8) }}; })
#  define vipmvi__loadu1x8x3_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x3_s32_t){{ vipmvi_loadu1x8_s32(___p), vipmvi_loadu1x8_s32((___p) + 2), vipmvi_loadu1x8_s32((___p) +  4) }}; })
#  define vipmvi__loadu1x8x3_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x3_u8_t) {{ vipmvi_loadu1x8_u8( ___p), vipmvi_loadu1x8_u8( (___p) + 8), vipmvi_loadu1x8_u8( (___p) + 16) }}; })
#  define vipmvi__loadu1x8x3_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x3_u16_t){{ vipmvi_loadu1x8_u16(___p), vipmvi_loadu1x8_u16((___p) + 4), vipmvi_loadu1x8_u16((___p) +  8) }}; })
#  define vipmvi__loadu1x8x3_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x3_u32_t){{ vipmvi_loadu1x8_u32(___p), vipmvi_loadu1x8_u32((___p) + 2), vipmvi_loadu1x8_u32((___p) +  4) }}; })
#  define vipmvi__loadu1x8x3_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x3_f32_t){{ vipmvi_loadu1x8_f32(___p), vipmvi_loadu1x8_f32((___p) + 2), vipmvi_loadu1x8_f32((___p) +  4) }}; })

#  define vipmvi__load1x8x3_s8(p)           __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x3_s8_t) {{ vipmvi_load1x8_s8( ___p), vipmvi_load1x8_s8( (___p) + 8), vipmvi_load1x8_s8( (___p) + 16) }}; })
#  define vipmvi__load1x8x3_s16(p)          __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x3_s16_t){{ vipmvi_load1x8_s16(___p), vipmvi_load1x8_s16((___p) + 4), vipmvi_load1x8_s16((___p) +  8) }}; })
#  define vipmvi__load1x8x3_s32(p)          __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x3_s32_t){{ vipmvi_load1x8_s32(___p), vipmvi_load1x8_s32((___p) + 2), vipmvi_load1x8_s32((___p) +  4) }}; })
#  define vipmvi__load1x8x3_u8(p)           __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x3_u8_t) {{ vipmvi_load1x8_u8( ___p), vipmvi_load1x8_u8( (___p) + 8), vipmvi_load1x8_u8( (___p) + 16) }}; })
#  define vipmvi__load1x8x3_u16(p)          __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x3_u16_t){{ vipmvi_load1x8_u16(___p), vipmvi_load1x8_u16((___p) + 4), vipmvi_load1x8_u16((___p) +  8) }}; })
#  define vipmvi__load1x8x3_u32(p)          __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x3_u32_t){{ vipmvi_load1x8_u32(___p), vipmvi_load1x8_u32((___p) + 2), vipmvi_load1x8_u32((___p) +  4) }}; })
#  define vipmvi__load1x8x3_f32(p)          __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x3_f32_t){{ vipmvi_load1x8_f32(___p), vipmvi_load1x8_f32((___p) + 2), vipmvi_load1x8_f32((___p) +  4) }}; })

# else /* default compiler */

_VIPM_INLINE bo_m64x3_s8_t  (vipmvi__loada1x8x3_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_loada1x8_s8(p), rb = vipmvi_loada1x8_s8((p) + 8), rc = vipmvi_loada1x8_s8((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s8_t  rv = {{ vipmvi_loada1x8_s8(p),  vipmvi_loada1x8_s8((p) + 8),  vipmvi_loada1x8_s8((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_s16_t (vipmvi__loada1x8x3_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s16_t rv;
	bo_m64_s16_t ra = vipmvi_loada1x8_s16(p), rb = vipmvi_loada1x8_s16((p) + 4), rc = vipmvi_loada1x8_s16((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s16_t rv = {{ vipmvi_loada1x8_s16(p), vipmvi_loada1x8_s16((p) + 4), vipmvi_loada1x8_s16((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_s32_t (vipmvi__loada1x8x3_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s32_t rv;
	bo_m64_s32_t ra = vipmvi_loada1x8_s32(p), rb = vipmvi_loada1x8_s32((p) + 2), rc = vipmvi_loada1x8_s32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s32_t rv = {{ vipmvi_loada1x8_s32(p), vipmvi_loada1x8_s32((p) + 2), vipmvi_loada1x8_s32((p) + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u8_t  (vipmvi__loada1x8x3_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_loada1x8_u8(p), rb = vipmvi_loada1x8_u8((p) + 8), rc = vipmvi_loada1x8_u8((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u8_t  rv = {{ vipmvi_loada1x8_u8(p),  vipmvi_loada1x8_u8((p) + 8),  vipmvi_loada1x8_u8((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u16_t (vipmvi__loada1x8x3_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u16_t rv;
	bo_m64_u16_t ra = vipmvi_loada1x8_u16(p), rb = vipmvi_loada1x8_u16((p) + 4), rc = vipmvi_loada1x8_u16((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u16_t rv = {{ vipmvi_loada1x8_u16(p), vipmvi_loada1x8_u16((p) + 4), vipmvi_loada1x8_u16((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u32_t (vipmvi__loada1x8x3_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u32_t rv;
	bo_m64_u32_t ra = vipmvi_loada1x8_u32(p), rb = vipmvi_loada1x8_u32((p) + 2), rc = vipmvi_loada1x8_u32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u32_t rv = {{ vipmvi_loada1x8_u32(p), vipmvi_loada1x8_u32((p) + 2), vipmvi_loada1x8_u32((p) + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_f32_t (vipmvi__loada1x8x3_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_f32_t rv;
	bo_m64_f32_t ra = vipmvi_loada1x8_f32(p), rb = vipmvi_loada1x8_f32((p) + 2), rc = vipmvi_loada1x8_f32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_f32_t rv = {{ vipmvi_loada1x8_f32(p), vipmvi_loada1x8_f32((p) + 2), vipmvi_loada1x8_f32((p) + 4) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m64x3_s8_t  (vipmvi__loadu1x8x3_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_loadu1x8_s8(p), rb = vipmvi_loadu1x8_s8((p) + 8), rc = vipmvi_loadu1x8_s8((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s8_t  rv = {{ vipmvi_loadu1x8_s8(p),  vipmvi_loadu1x8_s8((p) + 8),  vipmvi_loadu1x8_s8((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_s16_t (vipmvi__loadu1x8x3_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s16_t rv;
	bo_m64_s16_t ra = vipmvi_loadu1x8_s16(p), rb = vipmvi_loadu1x8_s16((p) + 4), rc = vipmvi_loadu1x8_s16((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s16_t rv = {{ vipmvi_loadu1x8_s16(p), vipmvi_loadu1x8_s16((p) + 4), vipmvi_loadu1x8_s16((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_s32_t (vipmvi__loadu1x8x3_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s32_t rv;
	bo_m64_s32_t ra = vipmvi_loadu1x8_s32(p), rb = vipmvi_loadu1x8_s32((p) + 2), rc = vipmvi_loadu1x8_s32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s32_t rv = {{ vipmvi_loadu1x8_s32(p), vipmvi_loadu1x8_s32((p) + 2), vipmvi_loadu1x8_s32((p) + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u8_t  (vipmvi__loadu1x8x3_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_loadu1x8_u8(p), rb = vipmvi_loadu1x8_u8((p) + 8), rc = vipmvi_loadu1x8_u8((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u8_t  rv = {{ vipmvi_loadu1x8_u8(p),  vipmvi_loadu1x8_u8((p) + 8),  vipmvi_loadu1x8_u8((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u16_t (vipmvi__loadu1x8x3_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u16_t rv;
	bo_m64_u16_t ra = vipmvi_loadu1x8_u16(p), rb = vipmvi_loadu1x8_u16((p) + 4), rc = vipmvi_loadu1x8_u16((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u16_t rv = {{ vipmvi_loadu1x8_u16(p), vipmvi_loadu1x8_u16((p) + 4), vipmvi_loadu1x8_u16((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u32_t (vipmvi__loadu1x8x3_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u32_t rv;
	bo_m64_u32_t ra = vipmvi_loadu1x8_u32(p), rb = vipmvi_loadu1x8_u32((p) + 2), rc = vipmvi_loadu1x8_u32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u32_t rv = {{ vipmvi_loadu1x8_u32(p), vipmvi_loadu1x8_u32((p) + 2), vipmvi_loadu1x8_u32((p) + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_f32_t (vipmvi__loadu1x8x3_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_f32_t rv;
	bo_m64_f32_t ra = vipmvi_loadu1x8_f32(p), rb = vipmvi_loadu1x8_f32((p) + 2), rc = vipmvi_loadu1x8_f32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_f32_t rv = {{ vipmvi_loadu1x8_f32(p), vipmvi_loadu1x8_f32((p) + 2), vipmvi_loadu1x8_f32((p) + 4) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m64x3_s8_t  (vipmvi__load1x8x3_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_load1x8_s8(p), rb = vipmvi_load1x8_s8((p) + 8), rc = vipmvi_load1x8_s8((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s8_t  rv = {{ vipmvi_load1x8_s8(p),  vipmvi_load1x8_s8((p) + 8),  vipmvi_load1x8_s8((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_s16_t (vipmvi__load1x8x3_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s16_t rv;
	bo_m64_s16_t ra = vipmvi_load1x8_s16(p), rb = vipmvi_load1x8_s16((p) + 4), rc = vipmvi_load1x8_s16((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s16_t rv = {{ vipmvi_load1x8_s16(p), vipmvi_load1x8_s16((p) + 4), vipmvi_load1x8_s16((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_s32_t (vipmvi__load1x8x3_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_s32_t rv;
	bo_m64_s32_t ra = vipmvi_load1x8_s32(p), rb = vipmvi_load1x8_s32((p) + 2), rc = vipmvi_load1x8_s32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_s32_t rv = {{ vipmvi_load1x8_s32(p), vipmvi_load1x8_s32((p) + 2), vipmvi_load1x8_s32((p) + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u8_t  (vipmvi__load1x8x3_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_load1x8_u8(p), rb = vipmvi_load1x8_u8((p) + 8), rc = vipmvi_load1x8_u8((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u8_t  rv = {{ vipmvi_load1x8_u8(p),  vipmvi_load1x8_u8((p) + 8),  vipmvi_load1x8_u8((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u16_t (vipmvi__load1x8x3_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u16_t rv;
	bo_m64_u16_t ra = vipmvi_load1x8_u16(p), rb = vipmvi_load1x8_u16((p) + 4), rc = vipmvi_load1x8_u16((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u16_t rv = {{ vipmvi_load1x8_u16(p), vipmvi_load1x8_u16((p) + 4), vipmvi_load1x8_u16((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_u32_t (vipmvi__load1x8x3_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_u32_t rv;
	bo_m64_u32_t ra = vipmvi_load1x8_u32(p), rb = vipmvi_load1x8_u32((p) + 2), rc = vipmvi_load1x8_u32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_u32_t rv = {{ vipmvi_load1x8_u32(p), vipmvi_load1x8_u32((p) + 2), vipmvi_load1x8_u32((p) + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x3_f32_t (vipmvi__load1x8x3_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x3_f32_t rv;
	bo_m64_f32_t ra = vipmvi_load1x8_f32(p), rb = vipmvi_load1x8_f32((p) + 2), rc = vipmvi_load1x8_f32((p) + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m64x3_f32_t rv = {{ vipmvi_load1x8_f32(p), vipmvi_load1x8_f32((p) + 2), vipmvi_load1x8_f32((p) + 4) }};
#  endif
	return (rv);
}

# endif /* _GCC_VER */


		/*
		 * Transfer 8x4=32 bytes
		 * 4 D registers
		 */

// @todo: fixme - compiler
# ifdef _GCC_VER

#  define vipmvi__loada1x8x4_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x4_s8_t) {{ vipmvi_loada1x8_s8( ___p), vipmvi_loada1x8_s8( (___p) + 8), vipmvi_loada1x8_s8( (___p) + 16), vipmvi_loada1x8_s8( (___p) + 24) }}; })
#  define vipmvi__loada1x8x4_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x4_s16_t){{ vipmvi_loada1x8_s16(___p), vipmvi_loada1x8_s16((___p) + 4), vipmvi_loada1x8_s16((___p) +  8), vipmvi_loada1x8_s16((___p) + 12) }}; })
#  define vipmvi__loada1x8x4_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x4_s32_t){{ vipmvi_loada1x8_s32(___p), vipmvi_loada1x8_s32((___p) + 2), vipmvi_loada1x8_s32((___p) +  4), vipmvi_loada1x8_s32((___p) +  6) }}; })
#  define vipmvi__loada1x8x4_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x4_u8_t) {{ vipmvi_loada1x8_u8( ___p), vipmvi_loada1x8_u8( (___p) + 8), vipmvi_loada1x8_u8( (___p) + 16), vipmvi_loada1x8_u8( (___p) + 24) }}; })
#  define vipmvi__loada1x8x4_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x4_u16_t){{ vipmvi_loada1x8_u16(___p), vipmvi_loada1x8_u16((___p) + 4), vipmvi_loada1x8_u16((___p) +  8), vipmvi_loada1x8_u16((___p) + 12) }}; })
#  define vipmvi__loada1x8x4_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x4_u32_t){{ vipmvi_loada1x8_u32(___p), vipmvi_loada1x8_u32((___p) + 2), vipmvi_loada1x8_u32((___p) +  4), vipmvi_loada1x8_u32((___p) +  6) }}; })
#  define vipmvi__loada1x8x4_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x4_f32_t){{ vipmvi_loada1x8_f32(___p), vipmvi_loada1x8_f32((___p) + 2), vipmvi_loada1x8_f32((___p) +  4), vipmvi_loada1x8_f32((___p) +  6) }}; })

#  define vipmvi__loadu1x8x4_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x4_s8_t) {{ vipmvi_loadu1x8_s8( ___p), vipmvi_loadu1x8_s8( (___p) + 8), vipmvi_loadu1x8_s8( (___p) + 16), vipmvi_loadu1x8_s8( (___p) + 24) }}; })
#  define vipmvi__loadu1x8x4_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x4_s16_t){{ vipmvi_loadu1x8_s16(___p), vipmvi_loadu1x8_s16((___p) + 4), vipmvi_loadu1x8_s16((___p) +  8), vipmvi_loadu1x8_s16((___p) + 12) }}; })
#  define vipmvi__loadu1x8x4_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x4_s32_t){{ vipmvi_loadu1x8_s32(___p), vipmvi_loadu1x8_s32((___p) + 2), vipmvi_loadu1x8_s32((___p) +  4), vipmvi_loadu1x8_s32((___p) +  6) }}; })
#  define vipmvi__loadu1x8x4_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x4_u8_t) {{ vipmvi_loadu1x8_u8( ___p), vipmvi_loadu1x8_u8( (___p) + 8), vipmvi_loadu1x8_u8( (___p) + 16), vipmvi_loadu1x8_u8( (___p) + 24) }}; })
#  define vipmvi__loadu1x8x4_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x4_u16_t){{ vipmvi_loadu1x8_u16(___p), vipmvi_loadu1x8_u16((___p) + 4), vipmvi_loadu1x8_u16((___p) +  8), vipmvi_loadu1x8_u16((___p) + 12) }}; })
#  define vipmvi__loadu1x8x4_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x4_u32_t){{ vipmvi_loadu1x8_u32(___p), vipmvi_loadu1x8_u32((___p) + 2), vipmvi_loadu1x8_u32((___p) +  4), vipmvi_loadu1x8_u32((___p) +  6) }}; })
#  define vipmvi__loadu1x8x4_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x4_f32_t){{ vipmvi_loadu1x8_f32(___p), vipmvi_loadu1x8_f32((___p) + 2), vipmvi_loadu1x8_f32((___p) +  4), vipmvi_loadu1x8_f32((___p) +  6) }}; })

#  define vipmvi__load1x8x4_s8(p)           __extension__ ({ bo_int8_t    const *___p = (p); (bo_m64x4_s8_t) {{ vipmvi_load1x8_s8( ___p), vipmvi_load1x8_s8( (___p) + 8), vipmvi_load1x8_s8( (___p) + 16), vipmvi_load1x8_s8( (___p) + 24) }}; })
#  define vipmvi__load1x8x4_s16(p)          __extension__ ({ bo_int16_t   const *___p = (p); (bo_m64x4_s16_t){{ vipmvi_load1x8_s16(___p), vipmvi_load1x8_s16((___p) + 4), vipmvi_load1x8_s16((___p) +  8), vipmvi_load1x8_s16((___p) + 12) }}; })
#  define vipmvi__load1x8x4_s32(p)          __extension__ ({ bo_int32_t   const *___p = (p); (bo_m64x4_s32_t){{ vipmvi_load1x8_s32(___p), vipmvi_load1x8_s32((___p) + 2), vipmvi_load1x8_s32((___p) +  4), vipmvi_load1x8_s32((___p) +  6) }}; })
#  define vipmvi__load1x8x4_u8(p)           __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m64x4_u8_t) {{ vipmvi_load1x8_u8( ___p), vipmvi_load1x8_u8( (___p) + 8), vipmvi_load1x8_u8( (___p) + 16), vipmvi_load1x8_u8( (___p) + 24) }}; })
#  define vipmvi__load1x8x4_u16(p)          __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m64x4_u16_t){{ vipmvi_load1x8_u16(___p), vipmvi_load1x8_u16((___p) + 4), vipmvi_load1x8_u16((___p) +  8), vipmvi_load1x8_u16((___p) + 12) }}; })
#  define vipmvi__load1x8x4_u32(p)          __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m64x4_u32_t){{ vipmvi_load1x8_u32(___p), vipmvi_load1x8_u32((___p) + 2), vipmvi_load1x8_u32((___p) +  4), vipmvi_load1x8_u32((___p) +  6) }}; })
#  define vipmvi__load1x8x4_f32(p)          __extension__ ({ bo_float32_t const *___p = (p); (bo_m64x4_f32_t){{ vipmvi_load1x8_f32(___p), vipmvi_load1x8_f32((___p) + 2), vipmvi_load1x8_f32((___p) +  4), vipmvi_load1x8_f32((___p) +  6) }}; })

# else /* default compiler */

_VIPM_INLINE bo_m64x4_s8_t  (vipmvi__loada1x8x4_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_loada1x8_s8(p), rb = vipmvi_loada1x8_s8((p) + 8), rc = vipmvi_loada1x8_s8((p) + 16), rd = vipmvi_loada1x8_s8((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s8_t  rv = {{ vipmvi_loada1x8_s8(p),  vipmvi_loada1x8_s8((p) + 8),  vipmvi_loada1x8_s8((p) + 16), vipmvi_loada1x8_s8((p) + 24)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_s16_t (vipmvi__loada1x8x4_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s16_t rv;
	bo_m64_s16_t ra = vipmvi_loada1x8_s16(p), rb = vipmvi_loada1x8_s16((p) + 4), rc = vipmvi_loada1x8_s16((p) + 8), rd = vipmvi_loada1x8_s16((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s16_t rv = {{ vipmvi_loada1x8_s16(p), vipmvi_loada1x8_s16((p) + 4), vipmvi_loada1x8_s16((p) + 8), vipmvi_loada1x8_s16((p) + 12) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_s32_t (vipmvi__loada1x8x4_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s32_t rv;
	bo_m64_s32_t ra = vipmvi_loada1x8_s32(p), rb = vipmvi_loada1x8_s32((p) + 2), rc = vipmvi_loada1x8_s32((p) + 4), rd = vipmvi_loada1x8_s32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s32_t rv = {{ vipmvi_loada1x8_s32(p), vipmvi_loada1x8_s32((p) + 2), vipmvi_loada1x8_s32((p) + 4), vipmvi_loada1x8_s32((p) + 6)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u8_t  (vipmvi__loada1x8x4_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_loada1x8_u8(p), rb = vipmvi_loada1x8_u8((p) + 8), rc = vipmvi_loada1x8_u8((p) + 16), rd = vipmvi_loada1x8_u8((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u8_t  rv = {{ vipmvi_loada1x8_u8(p),  vipmvi_loada1x8_u8((p) + 8),  vipmvi_loada1x8_u8((p) + 16), vipmvi_loada1x8_u8((p) + 24)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u16_t (vipmvi__loada1x8x4_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u16_t rv;
	bo_m64_u16_t ra = vipmvi_loada1x8_u16(p), rb = vipmvi_loada1x8_u16((p) + 4), rc = vipmvi_loada1x8_u16((p) + 8), rd = vipmvi_loada1x8_u16((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u16_t rv = {{ vipmvi_loada1x8_u16(p), vipmvi_loada1x8_u16((p) + 4), vipmvi_loada1x8_u16((p) + 8), vipmvi_loada1x8_u16((p) + 12) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u32_t (vipmvi__loada1x8x4_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u32_t rv;
	bo_m64_u32_t ra = vipmvi_loada1x8_u32(p), rb = vipmvi_loada1x8_u32((p) + 2), rc = vipmvi_loada1x8_u32((p) + 4), rd = vipmvi_loada1x8_u32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u32_t rv = {{ vipmvi_loada1x8_u32(p), vipmvi_loada1x8_u32((p) + 2), vipmvi_loada1x8_u32((p) + 4), vipmvi_loada1x8_u32((p) + 6)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_f32_t (vipmvi__loada1x8x4_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_f32_t rv;
	bo_m64_f32_t ra = vipmvi_loada1x8_f32(p), rb = vipmvi_loada1x8_f32((p) + 2), rc = vipmvi_loada1x8_f32((p) + 4), rd = vipmvi_loada1x8_f32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_f32_t rv = {{ vipmvi_loada1x8_f32(p), vipmvi_loada1x8_f32((p) + 2), vipmvi_loada1x8_f32((p) + 4), vipmvi_loada1x8_f32((p) + 6)  }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m64x4_s8_t  (vipmvi__loadu1x8x4_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_loadu1x8_s8(p), rb = vipmvi_loadu1x8_s8((p) + 8), rc = vipmvi_loadu1x8_s8((p) + 16), rd = vipmvi_loadu1x8_s8((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s8_t  rv = {{ vipmvi_loadu1x8_s8(p),  vipmvi_loadu1x8_s8((p) + 8),  vipmvi_loadu1x8_s8((p) + 16), vipmvi_loadu1x8_s8((p) + 24)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_s16_t (vipmvi__loadu1x8x4_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s16_t rv;
	bo_m64_s16_t ra = vipmvi_loadu1x8_s16(p), rb = vipmvi_loadu1x8_s16((p) + 4), rc = vipmvi_loadu1x8_s16((p) + 8), rd = vipmvi_loadu1x8_s16((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s16_t rv = {{ vipmvi_loadu1x8_s16(p), vipmvi_loadu1x8_s16((p) + 4), vipmvi_loadu1x8_s16((p) + 8), vipmvi_loadu1x8_s16((p) + 12) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_s32_t (vipmvi__loadu1x8x4_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s32_t rv;
	bo_m64_s32_t ra = vipmvi_loadu1x8_s32(p), rb = vipmvi_loadu1x8_s32((p) + 2), rc = vipmvi_loadu1x8_s32((p) + 4), rd = vipmvi_loadu1x8_s32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s32_t rv = {{ vipmvi_loadu1x8_s32(p), vipmvi_loadu1x8_s32((p) + 2), vipmvi_loadu1x8_s32((p) + 4), vipmvi_loadu1x8_s32((p) + 6)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u8_t  (vipmvi__loadu1x8x4_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_loadu1x8_u8(p), rb = vipmvi_loadu1x8_u8((p) + 8), rc = vipmvi_loadu1x8_u8((p) + 16), rd = vipmvi_loadu1x8_u8((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u8_t  rv = {{ vipmvi_loadu1x8_u8(p),  vipmvi_loadu1x8_u8((p) + 8),  vipmvi_loadu1x8_u8((p) + 16), vipmvi_loadu1x8_u8((p) + 24)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u16_t (vipmvi__loadu1x8x4_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u16_t rv;
	bo_m64_u16_t ra = vipmvi_loadu1x8_u16(p), rb = vipmvi_loadu1x8_u16((p) + 4), rc = vipmvi_loadu1x8_u16((p) + 8), rd = vipmvi_loadu1x8_u16((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u16_t rv = {{ vipmvi_loadu1x8_u16(p), vipmvi_loadu1x8_u16((p) + 4), vipmvi_loadu1x8_u16((p) + 8), vipmvi_loadu1x8_u16((p) + 12) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u32_t (vipmvi__loadu1x8x4_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u32_t rv;
	bo_m64_u32_t ra = vipmvi_loadu1x8_u32(p), rb = vipmvi_loadu1x8_u32((p) + 2), rc = vipmvi_loadu1x8_u32((p) + 4), rd = vipmvi_loadu1x8_u32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u32_t rv = {{ vipmvi_loadu1x8_u32(p), vipmvi_loadu1x8_u32((p) + 2), vipmvi_loadu1x8_u32((p) + 4), vipmvi_loadu1x8_u32((p) + 6)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_f32_t (vipmvi__loadu1x8x4_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_f32_t rv;
	bo_m64_f32_t ra = vipmvi_loadu1x8_f32(p), rb = vipmvi_loadu1x8_f32((p) + 2), rc = vipmvi_loadu1x8_f32((p) + 4), rd = vipmvi_loadu1x8_f32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_f32_t rv = {{ vipmvi_loadu1x8_f32(p), vipmvi_loadu1x8_f32((p) + 2), vipmvi_loadu1x8_f32((p) + 4), vipmvi_loadu1x8_f32((p) + 6)  }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m64x4_s8_t  (vipmvi__load1x8x4_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s8_t  rv;
	bo_m64_s8_t ra = vipmvi_load1x8_s8(p), rb = vipmvi_load1x8_s8((p) + 8), rc = vipmvi_load1x8_s8((p) + 16), rd = vipmvi_load1x8_s8((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s8_t  rv = {{ vipmvi_load1x8_s8(p),  vipmvi_load1x8_s8((p) + 8),  vipmvi_load1x8_s8((p) + 16), vipmvi_load1x8_s8((p) + 24)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_s16_t (vipmvi__load1x8x4_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s16_t rv;
	bo_m64_s16_t ra = vipmvi_load1x8_s16(p), rb = vipmvi_load1x8_s16((p) + 4), rc = vipmvi_load1x8_s16((p) + 8), rd = vipmvi_load1x8_s16((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s16_t rv = {{ vipmvi_load1x8_s16(p), vipmvi_load1x8_s16((p) + 4), vipmvi_load1x8_s16((p) + 8), vipmvi_load1x8_s16((p) + 12) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_s32_t (vipmvi__load1x8x4_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_s32_t rv;
	bo_m64_s32_t ra = vipmvi_load1x8_s32(p), rb = vipmvi_load1x8_s32((p) + 2), rc = vipmvi_load1x8_s32((p) + 4), rd = vipmvi_load1x8_s32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_s32_t rv = {{ vipmvi_load1x8_s32(p), vipmvi_load1x8_s32((p) + 2), vipmvi_load1x8_s32((p) + 4), vipmvi_load1x8_s32((p) + 6)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u8_t  (vipmvi__load1x8x4_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u8_t  rv;
	bo_m64_u8_t ra = vipmvi_load1x8_u8(p), rb = vipmvi_load1x8_u8((p) + 8), rc = vipmvi_load1x8_u8((p) + 16), rd = vipmvi_load1x8_u8((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u8_t  rv = {{ vipmvi_load1x8_u8(p),  vipmvi_load1x8_u8((p) + 8),  vipmvi_load1x8_u8((p) + 16), vipmvi_load1x8_u8((p) + 24)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u16_t (vipmvi__load1x8x4_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u16_t rv;
	bo_m64_u16_t ra = vipmvi_load1x8_u16(p), rb = vipmvi_load1x8_u16((p) + 4), rc = vipmvi_load1x8_u16((p) + 8), rd = vipmvi_load1x8_u16((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u16_t rv = {{ vipmvi_load1x8_u16(p), vipmvi_load1x8_u16((p) + 4), vipmvi_load1x8_u16((p) + 8), vipmvi_load1x8_u16((p) + 12) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_u32_t (vipmvi__load1x8x4_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_u32_t rv;
	bo_m64_u32_t ra = vipmvi_load1x8_u32(p), rb = vipmvi_load1x8_u32((p) + 2), rc = vipmvi_load1x8_u32((p) + 4), rd = vipmvi_load1x8_u32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_u32_t rv = {{ vipmvi_load1x8_u32(p), vipmvi_load1x8_u32((p) + 2), vipmvi_load1x8_u32((p) + 4), vipmvi_load1x8_u32((p) + 6)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m64x4_f32_t (vipmvi__load1x8x4_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m64x4_f32_t rv;
	bo_m64_f32_t ra = vipmvi_load1x8_f32(p), rb = vipmvi_load1x8_f32((p) + 2), rc = vipmvi_load1x8_f32((p) + 4), rd = vipmvi_load1x8_f32((p) + 6);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m64x4_f32_t rv = {{ vipmvi_load1x8_f32(p), vipmvi_load1x8_f32((p) + 2), vipmvi_load1x8_f32((p) + 4), vipmvi_load1x8_f32((p) + 6)  }};
#  endif
	return (rv);
}

# endif /* _GCC_VER */


		/*
		 * Transfer 16x2=32 bytes
		 * 2 Q registers
		 */

// @todo: fixme - compiler
# ifdef _GCC_VER

#  define vipmvi__loada1x16x2_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x2_s8_t) {{ vipmvi_loada1x16_s8( ___p), vipmvi_loada1x16_s8( (___p) + 16) }}; })
#  define vipmvi__loada1x16x2_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x2_s16_t){{ vipmvi_loada1x16_s16(___p), vipmvi_loada1x16_s16((___p) +  8) }}; })
#  define vipmvi__loada1x16x2_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x2_s32_t){{ vipmvi_loada1x16_s32(___p), vipmvi_loada1x16_s32((___p) +  4) }}; })
#  define vipmvi__loada1x16x2_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x2_u8_t) {{ vipmvi_loada1x16_u8( ___p), vipmvi_loada1x16_u8( (___p) + 16) }}; })
#  define vipmvi__loada1x16x2_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x2_u16_t){{ vipmvi_loada1x16_u16(___p), vipmvi_loada1x16_u16((___p) +  8) }}; })
#  define vipmvi__loada1x16x2_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x2_u32_t){{ vipmvi_loada1x16_u32(___p), vipmvi_loada1x16_u32((___p) +  4) }}; })
#  define vipmvi__loada1x16x2_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x2_f32_t){{ vipmvi_loada1x16_f32(___p), vipmvi_loada1x16_f32((___p) +  4) }}; })

#  define vipmvi__loadu1x16x2_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x2_s8_t) {{ vipmvi_loadu1x16_s8( ___p), vipmvi_loadu1x16_s8( (___p) + 16) }}; })
#  define vipmvi__loadu1x16x2_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x2_s16_t){{ vipmvi_loadu1x16_s16(___p), vipmvi_loadu1x16_s16((___p) +  8) }}; })
#  define vipmvi__loadu1x16x2_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x2_s32_t){{ vipmvi_loadu1x16_s32(___p), vipmvi_loadu1x16_s32((___p) +  4) }}; })
#  define vipmvi__loadu1x16x2_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x2_u8_t) {{ vipmvi_loadu1x16_u8( ___p), vipmvi_loadu1x16_u8( (___p) + 16) }}; })
#  define vipmvi__loadu1x16x2_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x2_u16_t){{ vipmvi_loadu1x16_u16(___p), vipmvi_loadu1x16_u16((___p) +  8) }}; })
#  define vipmvi__loadu1x16x2_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x2_u32_t){{ vipmvi_loadu1x16_u32(___p), vipmvi_loadu1x16_u32((___p) +  4) }}; })
#  define vipmvi__loadu1x16x2_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x2_f32_t){{ vipmvi_loadu1x16_f32(___p), vipmvi_loadu1x16_f32((___p) +  4) }}; })

#  define vipmvi__load1x16x2_s8(p)           __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x2_s8_t) {{ vipmvi_load1x16_s8( ___p), vipmvi_load1x16_s8( (___p) + 16) }}; })
#  define vipmvi__load1x16x2_s16(p)          __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x2_s16_t){{ vipmvi_load1x16_s16(___p), vipmvi_load1x16_s16((___p) +  8) }}; })
#  define vipmvi__load1x16x2_s32(p)          __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x2_s32_t){{ vipmvi_load1x16_s32(___p), vipmvi_load1x16_s32((___p) +  4) }}; })
#  define vipmvi__load1x16x2_u8(p)           __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x2_u8_t) {{ vipmvi_load1x16_u8( ___p), vipmvi_load1x16_u8( (___p) + 16) }}; })
#  define vipmvi__load1x16x2_u16(p)          __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x2_u16_t){{ vipmvi_load1x16_u16(___p), vipmvi_load1x16_u16((___p) +  8) }}; })
#  define vipmvi__load1x16x2_u32(p)          __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x2_u32_t){{ vipmvi_load1x16_u32(___p), vipmvi_load1x16_u32((___p) +  4) }}; })
#  define vipmvi__load1x16x2_f32(p)          __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x2_f32_t){{ vipmvi_load1x16_f32(___p), vipmvi_load1x16_f32((___p) +  4) }}; })

# else /* default compiler */

_VIPM_INLINE bo_m128x2_s8_t  (vipmvi__loada1x16x2_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_loada1x16_s8(p), rb = vipmvi_loada1x16_s8(p + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s8_t  rv = {{ vipmvi_loada1x16_s8(p),  vipmvi_loada1x16_s8(p + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_s16_t (vipmvi__loada1x16x2_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s16_t rv;
	bo_m128_s16_t ra = vipmvi_loada1x16_s16(p), rb = vipmvi_loada1x16_s16(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s16_t rv = {{ vipmvi_loada1x16_s16(p), vipmvi_loada1x16_s16(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_s32_t (vipmvi__loada1x16x2_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s32_t rv;
	bo_m128_s32_t ra = vipmvi_loada1x16_s32(p), rb = vipmvi_loada1x16_s32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s32_t rv = {{ vipmvi_loada1x16_s32(p), vipmvi_loada1x16_s32(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u8_t  (vipmvi__loada1x16x2_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_loada1x16_u8(p), rb = vipmvi_loada1x16_u8(p + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u8_t  rv = {{ vipmvi_loada1x16_u8(p),  vipmvi_loada1x16_u8(p + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u16_t (vipmvi__loada1x16x2_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u16_t rv;
	bo_m128_u16_t ra = vipmvi_loada1x16_u16(p), rb = vipmvi_loada1x16_u16(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u16_t rv = {{ vipmvi_loada1x16_u16(p), vipmvi_loada1x16_u16(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u32_t (vipmvi__loada1x16x2_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u32_t rv;
	bo_m128_u32_t ra = vipmvi_loada1x16_u32(p), rb = vipmvi_loada1x16_u32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u32_t rv = {{ vipmvi_loada1x16_u32(p), vipmvi_loada1x16_u32(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_f32_t (vipmvi__loada1x16x2_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_f32_t rv;
	bo_m128_f32_t ra = vipmvi_loada1x16_f32(p), rb = vipmvi_loada1x16_f32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_f32_t rv = {{ vipmvi_loada1x16_f32(p), vipmvi_loada1x16_f32(p + 4) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m128x2_s8_t  (vipmvi__loadu1x16x2_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_loadu1x16_s8(p), rb = vipmvi_loadu1x16_s8(p + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s8_t  rv = {{ vipmvi_loadu1x16_s8(p),  vipmvi_loadu1x16_s8(p + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_s16_t (vipmvi__loadu1x16x2_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s16_t rv;
	bo_m128_s16_t ra = vipmvi_loadu1x16_s16(p), rb = vipmvi_loadu1x16_s16(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s16_t rv = {{ vipmvi_loadu1x16_s16(p), vipmvi_loadu1x16_s16(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_s32_t (vipmvi__loadu1x16x2_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s32_t rv;
	bo_m128_s32_t ra = vipmvi_loadu1x16_s32(p), rb = vipmvi_loadu1x16_s32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s32_t rv = {{ vipmvi_loadu1x16_s32(p), vipmvi_loadu1x16_s32(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u8_t  (vipmvi__loadu1x16x2_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_loadu1x16_u8(p), rb = vipmvi_loadu1x16_u8(p + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u8_t  rv = {{ vipmvi_loadu1x16_u8(p),  vipmvi_loadu1x16_u8(p + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u16_t (vipmvi__loadu1x16x2_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u16_t rv;
	bo_m128_u16_t ra = vipmvi_loadu1x16_u16(p), rb = vipmvi_loadu1x16_u16(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u16_t rv = {{ vipmvi_loadu1x16_u16(p), vipmvi_loadu1x16_u16(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u32_t (vipmvi__loadu1x16x2_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u32_t rv;
	bo_m128_u32_t ra = vipmvi_loadu1x16_u32(p), rb = vipmvi_loadu1x16_u32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u32_t rv = {{ vipmvi_loadu1x16_u32(p), vipmvi_loadu1x16_u32(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_f32_t (vipmvi__loadu1x16x2_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_f32_t rv;
	bo_m128_f32_t ra = vipmvi_loadu1x16_f32(p), rb = vipmvi_loadu1x16_f32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_f32_t rv = {{ vipmvi_loadu1x16_f32(p), vipmvi_loadu1x16_f32(p + 4) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m128x2_s8_t  (vipmvi__load1x16x2_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_load1x16_s8(p), rb = vipmvi_load1x16_s8(p + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s8_t  rv = {{ vipmvi_load1x16_s8(p),  vipmvi_load1x16_s8(p + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_s16_t (vipmvi__load1x16x2_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s16_t rv;
	bo_m128_s16_t ra = vipmvi_load1x16_s16(p), rb = vipmvi_load1x16_s16(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s16_t rv = {{ vipmvi_load1x16_s16(p), vipmvi_load1x16_s16(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_s32_t (vipmvi__load1x16x2_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_s32_t rv;
	bo_m128_s32_t ra = vipmvi_load1x16_s32(p), rb = vipmvi_load1x16_s32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_s32_t rv = {{ vipmvi_load1x16_s32(p), vipmvi_load1x16_s32(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u8_t  (vipmvi__load1x16x2_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_load1x16_u8(p), rb = vipmvi_load1x16_u8(p + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u8_t  rv = {{ vipmvi_load1x16_u8(p),  vipmvi_load1x16_u8(p + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u16_t (vipmvi__load1x16x2_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u16_t rv;
	bo_m128_u16_t ra = vipmvi_load1x16_u16(p), rb = vipmvi_load1x16_u16(p + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u16_t rv = {{ vipmvi_load1x16_u16(p), vipmvi_load1x16_u16(p + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_u32_t (vipmvi__load1x16x2_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_u32_t rv;
	bo_m128_u32_t ra = vipmvi_load1x16_u32(p), rb = vipmvi_load1x16_u32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_u32_t rv = {{ vipmvi_load1x16_u32(p), vipmvi_load1x16_u32(p + 4) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x2_f32_t (vipmvi__load1x16x2_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x2_f32_t rv;
	bo_m128_f32_t ra = vipmvi_load1x16_f32(p), rb = vipmvi_load1x16_f32(p + 4);
	rv.val[0] = ra;
	rv.val[1] = rb;
#  else
	bo_m128x2_f32_t rv = {{ vipmvi_load1x16_f32(p), vipmvi_load1x16_f32(p + 4) }};
#  endif
	return (rv);
}

# endif /* _GCC_VER */


		/*
		 * Transfer 16x3=48 bytes
		 * 3 Q registers
		 */

// @todo: fixme - compiler
# ifdef _GCC_VER

#  define vipmvi__loada1x16x3_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x3_s8_t) {{ vipmvi_loada1x16_s8( ___p), vipmvi_loada1x16_s8( (___p) + 16), vipmvi_loada1x16_s8( (___p) + 32) }}; })
#  define vipmvi__loada1x16x3_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x3_s16_t){{ vipmvi_loada1x16_s16(___p), vipmvi_loada1x16_s16((___p) +  8), vipmvi_loada1x16_s16((___p) + 16) }}; })
#  define vipmvi__loada1x16x3_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x3_s32_t){{ vipmvi_loada1x16_s32(___p), vipmvi_loada1x16_s32((___p) +  4), vipmvi_loada1x16_s32((___p) +  8) }}; })
#  define vipmvi__loada1x16x3_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x3_u8_t) {{ vipmvi_loada1x16_u8( ___p), vipmvi_loada1x16_u8( (___p) + 16), vipmvi_loada1x16_u8( (___p) + 32) }}; })
#  define vipmvi__loada1x16x3_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x3_u16_t){{ vipmvi_loada1x16_u16(___p), vipmvi_loada1x16_u16((___p) +  8), vipmvi_loada1x16_u16((___p) + 16) }}; })
#  define vipmvi__loada1x16x3_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x3_u32_t){{ vipmvi_loada1x16_u32(___p), vipmvi_loada1x16_u32((___p) +  4), vipmvi_loada1x16_u32((___p) +  8) }}; })
#  define vipmvi__loada1x16x3_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x3_f32_t){{ vipmvi_loada1x16_f32(___p), vipmvi_loada1x16_f32((___p) +  4), vipmvi_loada1x16_f32((___p) +  8) }}; })

#  define vipmvi__loadu1x16x3_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x3_s8_t) {{ vipmvi_loadu1x16_s8( ___p), vipmvi_loadu1x16_s8( (___p) + 16), vipmvi_loadu1x16_s8( (___p) + 32) }}; })
#  define vipmvi__loadu1x16x3_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x3_s16_t){{ vipmvi_loadu1x16_s16(___p), vipmvi_loadu1x16_s16((___p) +  8), vipmvi_loadu1x16_s16((___p) + 16) }}; })
#  define vipmvi__loadu1x16x3_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x3_s32_t){{ vipmvi_loadu1x16_s32(___p), vipmvi_loadu1x16_s32((___p) +  4), vipmvi_loadu1x16_s32((___p) +  8) }}; })
#  define vipmvi__loadu1x16x3_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x3_u8_t) {{ vipmvi_loadu1x16_u8( ___p), vipmvi_loadu1x16_u8( (___p) + 16), vipmvi_loadu1x16_u8( (___p) + 32) }}; })
#  define vipmvi__loadu1x16x3_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x3_u16_t){{ vipmvi_loadu1x16_u16(___p), vipmvi_loadu1x16_u16((___p) +  8), vipmvi_loadu1x16_u16((___p) + 16) }}; })
#  define vipmvi__loadu1x16x3_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x3_u32_t){{ vipmvi_loadu1x16_u32(___p), vipmvi_loadu1x16_u32((___p) +  4), vipmvi_loadu1x16_u32((___p) +  8) }}; })
#  define vipmvi__loadu1x16x3_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x3_f32_t){{ vipmvi_loadu1x16_f32(___p), vipmvi_loadu1x16_f32((___p) +  4), vipmvi_loadu1x16_f32((___p) +  8) }}; })

#  define vipmvi__load1x16x3_s8(p)           __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x3_s8_t) {{ vipmvi_load1x16_s8( ___p), vipmvi_load1x16_s8( (___p) + 16), vipmvi_load1x16_s8( (___p) + 32) }}; })
#  define vipmvi__load1x16x3_s16(p)          __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x3_s16_t){{ vipmvi_load1x16_s16(___p), vipmvi_load1x16_s16((___p) +  8), vipmvi_load1x16_s16((___p) + 16) }}; })
#  define vipmvi__load1x16x3_s32(p)          __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x3_s32_t){{ vipmvi_load1x16_s32(___p), vipmvi_load1x16_s32((___p) +  4), vipmvi_load1x16_s32((___p) +  8) }}; })
#  define vipmvi__load1x16x3_u8(p)           __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x3_u8_t) {{ vipmvi_load1x16_u8( ___p), vipmvi_load1x16_u8( (___p) + 16), vipmvi_load1x16_u8( (___p) + 32) }}; })
#  define vipmvi__load1x16x3_u16(p)          __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x3_u16_t){{ vipmvi_load1x16_u16(___p), vipmvi_load1x16_u16((___p) +  8), vipmvi_load1x16_u16((___p) + 16) }}; })
#  define vipmvi__load1x16x3_u32(p)          __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x3_u32_t){{ vipmvi_load1x16_u32(___p), vipmvi_load1x16_u32((___p) +  4), vipmvi_load1x16_u32((___p) +  8) }}; })
#  define vipmvi__load1x16x3_f32(p)          __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x3_f32_t){{ vipmvi_load1x16_f32(___p), vipmvi_load1x16_f32((___p) +  4), vipmvi_load1x16_f32((___p) +  8) }}; })

# else /* default compiler */

_VIPM_INLINE bo_m128x3_s8_t  (vipmvi__loada1x16x3_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_loada1x16_s8(p), rb = vipmvi_loada1x16_s8((p) + 16), rc = vipmvi_loada1x16_s8((p) + 32);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s8_t  rv = {{ vipmvi_loada1x16_s8(p),  vipmvi_loada1x16_s8((p) + 16),  vipmvi_loada1x16_s8((p) + 32) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_s16_t (vipmvi__loada1x16x3_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s16_t rv;
	bo_m128_s16_t ra = vipmvi_loada1x16_s16(p), rb = vipmvi_loada1x16_s16((p) + 8), rc = vipmvi_loada1x16_s16((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s16_t rv = {{ vipmvi_loada1x16_s16(p), vipmvi_loada1x16_s16((p) + 8), vipmvi_loada1x16_s16((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_s32_t (vipmvi__loada1x16x3_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s32_t rv;
	bo_m128_s32_t ra = vipmvi_loada1x16_s32(p), rb = vipmvi_loada1x16_s32((p) + 4), rc = vipmvi_loada1x16_s32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s32_t rv = {{ vipmvi_loada1x16_s32(p), vipmvi_loada1x16_s32((p) + 4), vipmvi_loada1x16_s32((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u8_t  (vipmvi__loada1x16x3_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_loada1x16_u8(p), rb = vipmvi_loada1x16_u8((p) + 16), rc = vipmvi_loada1x16_u8((p) + 32);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u8_t  rv = {{ vipmvi_loada1x16_u8(p),  vipmvi_loada1x16_u8((p) + 16),  vipmvi_loada1x16_u8((p) + 32) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u16_t (vipmvi__loada1x16x3_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u16_t rv;
	bo_m128_u16_t ra = vipmvi_loada1x16_u16(p), rb = vipmvi_loada1x16_u16((p) + 8), rc = vipmvi_loada1x16_u16((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u16_t rv = {{ vipmvi_loada1x16_u16(p), vipmvi_loada1x16_u16((p) + 8), vipmvi_loada1x16_u16((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u32_t (vipmvi__loada1x16x3_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u32_t rv;
	bo_m128_u32_t ra = vipmvi_loada1x16_u32(p), rb = vipmvi_loada1x16_u32((p) + 4), rc = vipmvi_loada1x16_u32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u32_t rv = {{ vipmvi_loada1x16_u32(p), vipmvi_loada1x16_u32((p) + 4), vipmvi_loada1x16_u32((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_f32_t (vipmvi__loada1x16x3_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_f32_t rv;
	bo_m128_f32_t ra = vipmvi_loada1x16_f32(p), rb = vipmvi_loada1x16_f32((p) + 4), rc = vipmvi_loada1x16_f32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_f32_t rv = {{ vipmvi_loada1x16_f32(p), vipmvi_loada1x16_f32((p) + 4), vipmvi_loada1x16_f32((p) + 8) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m128x3_s8_t  (vipmvi__loadu1x16x3_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_loadu1x16_s8(p), rb = vipmvi_loadu1x16_s8((p) + 16), rc = vipmvi_loadu1x16_s8((p) + 32);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s8_t  rv = {{ vipmvi_loadu1x16_s8(p),  vipmvi_loadu1x16_s8((p) + 16),  vipmvi_loadu1x16_s8((p) + 32) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_s16_t (vipmvi__loadu1x16x3_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s16_t rv;
	bo_m128_s16_t ra = vipmvi_loadu1x16_s16(p), rb = vipmvi_loadu1x16_s16((p) + 8), rc = vipmvi_loadu1x16_s16((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s16_t rv = {{ vipmvi_loadu1x16_s16(p), vipmvi_loadu1x16_s16((p) + 8), vipmvi_loadu1x16_s16((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_s32_t (vipmvi__loadu1x16x3_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s32_t rv;
	bo_m128_s32_t ra = vipmvi_loadu1x16_s32(p), rb = vipmvi_loadu1x16_s32((p) + 4), rc = vipmvi_loadu1x16_s32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s32_t rv = {{ vipmvi_loadu1x16_s32(p), vipmvi_loadu1x16_s32((p) + 4), vipmvi_loadu1x16_s32((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u8_t  (vipmvi__loadu1x16x3_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_loadu1x16_u8(p), rb = vipmvi_loadu1x16_u8((p) + 16), rc = vipmvi_loadu1x16_u8((p) + 32);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u8_t  rv = {{ vipmvi_loadu1x16_u8(p),  vipmvi_loadu1x16_u8((p) + 16),  vipmvi_loadu1x16_u8((p) + 32) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u16_t (vipmvi__loadu1x16x3_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u16_t rv;
	bo_m128_u16_t ra = vipmvi_loadu1x16_u16(p), rb = vipmvi_loadu1x16_u16((p) + 8), rc = vipmvi_loadu1x16_u16((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u16_t rv = {{ vipmvi_loadu1x16_u16(p), vipmvi_loadu1x16_u16((p) + 8), vipmvi_loadu1x16_u16((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u32_t (vipmvi__loadu1x16x3_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u32_t rv;
	bo_m128_u32_t ra = vipmvi_loadu1x16_u32(p), rb = vipmvi_loadu1x16_u32((p) + 4), rc = vipmvi_loadu1x16_u32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u32_t rv = {{ vipmvi_loadu1x16_u32(p), vipmvi_loadu1x16_u32((p) + 4), vipmvi_loadu1x16_u32((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_f32_t (vipmvi__loadu1x16x3_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_f32_t rv;
	bo_m128_f32_t ra = vipmvi_loadu1x16_f32(p), rb = vipmvi_loadu1x16_f32((p) + 4), rc = vipmvi_loadu1x16_f32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_f32_t rv = {{ vipmvi_loadu1x16_f32(p), vipmvi_loadu1x16_f32((p) + 4), vipmvi_loadu1x16_f32((p) + 8) }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m128x3_s8_t  (vipmvi__load1x16x3_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_load1x16_s8(p), rb = vipmvi_load1x16_s8((p) + 16), rc = vipmvi_load1x16_s8((p) + 32);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s8_t  rv = {{ vipmvi_load1x16_s8(p),  vipmvi_load1x16_s8((p) + 16),  vipmvi_load1x16_s8((p) + 32) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_s16_t (vipmvi__load1x16x3_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s16_t rv;
	bo_m128_s16_t ra = vipmvi_load1x16_s16(p), rb = vipmvi_load1x16_s16((p) + 8), rc = vipmvi_load1x16_s16((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s16_t rv = {{ vipmvi_load1x16_s16(p), vipmvi_load1x16_s16((p) + 8), vipmvi_load1x16_s16((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_s32_t (vipmvi__load1x16x3_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_s32_t rv;
	bo_m128_s32_t ra = vipmvi_load1x16_s32(p), rb = vipmvi_load1x16_s32((p) + 4), rc = vipmvi_load1x16_s32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_s32_t rv = {{ vipmvi_load1x16_s32(p), vipmvi_load1x16_s32((p) + 4), vipmvi_load1x16_s32((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u8_t  (vipmvi__load1x16x3_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_load1x16_u8(p), rb = vipmvi_load1x16_u8((p) + 16), rc = vipmvi_load1x16_u8((p) + 32);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u8_t  rv = {{ vipmvi_load1x16_u8(p),  vipmvi_load1x16_u8((p) + 16),  vipmvi_load1x16_u8((p) + 32) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u16_t (vipmvi__load1x16x3_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u16_t rv;
	bo_m128_u16_t ra = vipmvi_load1x16_u16(p), rb = vipmvi_load1x16_u16((p) + 8), rc = vipmvi_load1x16_u16((p) + 16);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u16_t rv = {{ vipmvi_load1x16_u16(p), vipmvi_load1x16_u16((p) + 8), vipmvi_load1x16_u16((p) + 16) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_u32_t (vipmvi__load1x16x3_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_u32_t rv;
	bo_m128_u32_t ra = vipmvi_load1x16_u32(p), rb = vipmvi_load1x16_u32((p) + 4), rc = vipmvi_load1x16_u32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_u32_t rv = {{ vipmvi_load1x16_u32(p), vipmvi_load1x16_u32((p) + 4), vipmvi_load1x16_u32((p) + 8) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x3_f32_t (vipmvi__load1x16x3_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x3_f32_t rv;
	bo_m128_f32_t ra = vipmvi_load1x16_f32(p), rb = vipmvi_load1x16_f32((p) + 4), rc = vipmvi_load1x16_f32((p) + 8);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
#  else
	bo_m128x3_f32_t rv = {{ vipmvi_load1x16_f32(p), vipmvi_load1x16_f32((p) + 4), vipmvi_load1x16_f32((p) + 8) }};
#  endif
	return (rv);
}

# endif /* _GCC_VER */


		/*
		 * Transfer 16x4=64 bytes
		 * 4 Q registers
		 */

// @todo: fixme - compiler
# ifdef _GCC_VER

#  define vipmvi__loada1x16x4_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x4_s8_t) {{ vipmvi_loada1x16_s8( ___p), vipmvi_loada1x16_s8( (___p) + 16), vipmvi_loada1x16_s8( (___p) + 32), vipmvi_loada1x16_s8( (___p) + 48) }}; })
#  define vipmvi__loada1x16x4_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x4_s16_t){{ vipmvi_loada1x16_s16(___p), vipmvi_loada1x16_s16((___p) +  8), vipmvi_loada1x16_s16((___p) + 16), vipmvi_loada1x16_s16((___p) + 24) }}; })
#  define vipmvi__loada1x16x4_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x4_s32_t){{ vipmvi_loada1x16_s32(___p), vipmvi_loada1x16_s32((___p) +  4), vipmvi_loada1x16_s32((___p) +  8), vipmvi_loada1x16_s32((___p) + 12) }}; })
#  define vipmvi__loada1x16x4_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x4_u8_t) {{ vipmvi_loada1x16_u8( ___p), vipmvi_loada1x16_u8( (___p) + 16), vipmvi_loada1x16_u8( (___p) + 32), vipmvi_loada1x16_u8( (___p) + 48) }}; })
#  define vipmvi__loada1x16x4_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x4_u16_t){{ vipmvi_loada1x16_u16(___p), vipmvi_loada1x16_u16((___p) +  8), vipmvi_loada1x16_u16((___p) + 16), vipmvi_loada1x16_u16((___p) + 24) }}; })
#  define vipmvi__loada1x16x4_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x4_u32_t){{ vipmvi_loada1x16_u32(___p), vipmvi_loada1x16_u32((___p) +  4), vipmvi_loada1x16_u32((___p) +  8), vipmvi_loada1x16_u32((___p) + 12) }}; })
#  define vipmvi__loada1x16x4_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x4_f32_t){{ vipmvi_loada1x16_f32(___p), vipmvi_loada1x16_f32((___p) +  4), vipmvi_loada1x16_f32((___p) +  8), vipmvi_loada1x16_f32((___p) + 12) }}; })

#  define vipmvi__loadu1x16x4_s8(p)          __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x4_s8_t) {{ vipmvi_loadu1x16_s8( ___p), vipmvi_loadu1x16_s8( (___p) + 16), vipmvi_loadu1x16_s8( (___p) + 32), vipmvi_loadu1x16_s8( (___p) + 48) }}; })
#  define vipmvi__loadu1x16x4_s16(p)         __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x4_s16_t){{ vipmvi_loadu1x16_s16(___p), vipmvi_loadu1x16_s16((___p) +  8), vipmvi_loadu1x16_s16((___p) + 16), vipmvi_loadu1x16_s16((___p) + 24) }}; })
#  define vipmvi__loadu1x16x4_s32(p)         __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x4_s32_t){{ vipmvi_loadu1x16_s32(___p), vipmvi_loadu1x16_s32((___p) +  4), vipmvi_loadu1x16_s32((___p) +  8), vipmvi_loadu1x16_s32((___p) + 12) }}; })
#  define vipmvi__loadu1x16x4_u8(p)          __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x4_u8_t) {{ vipmvi_loadu1x16_u8( ___p), vipmvi_loadu1x16_u8( (___p) + 16), vipmvi_loadu1x16_u8( (___p) + 32), vipmvi_loadu1x16_u8( (___p) + 48) }}; })
#  define vipmvi__loadu1x16x4_u16(p)         __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x4_u16_t){{ vipmvi_loadu1x16_u16(___p), vipmvi_loadu1x16_u16((___p) +  8), vipmvi_loadu1x16_u16((___p) + 16), vipmvi_loadu1x16_u16((___p) + 24) }}; })
#  define vipmvi__loadu1x16x4_u32(p)         __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x4_u32_t){{ vipmvi_loadu1x16_u32(___p), vipmvi_loadu1x16_u32((___p) +  4), vipmvi_loadu1x16_u32((___p) +  8), vipmvi_loadu1x16_u32((___p) + 12) }}; })
#  define vipmvi__loadu1x16x4_f32(p)         __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x4_f32_t){{ vipmvi_loadu1x16_f32(___p), vipmvi_loadu1x16_f32((___p) +  4), vipmvi_loadu1x16_f32((___p) +  8), vipmvi_loadu1x16_f32((___p) + 12) }}; })

#  define vipmvi__load1x16x4_s8(p)           __extension__ ({ bo_int8_t    const *___p = (p); (bo_m128x4_s8_t) {{ vipmvi_load1x16_s8( ___p), vipmvi_load1x16_s8( (___p) + 16), vipmvi_load1x16_s8( (___p) + 32), vipmvi_load1x16_s8( (___p) + 48) }}; })
#  define vipmvi__load1x16x4_s16(p)          __extension__ ({ bo_int16_t   const *___p = (p); (bo_m128x4_s16_t){{ vipmvi_load1x16_s16(___p), vipmvi_load1x16_s16((___p) +  8), vipmvi_load1x16_s16((___p) + 16), vipmvi_load1x16_s16((___p) + 24) }}; })
#  define vipmvi__load1x16x4_s32(p)          __extension__ ({ bo_int32_t   const *___p = (p); (bo_m128x4_s32_t){{ vipmvi_load1x16_s32(___p), vipmvi_load1x16_s32((___p) +  4), vipmvi_load1x16_s32((___p) +  8), vipmvi_load1x16_s32((___p) + 12) }}; })
#  define vipmvi__load1x16x4_u8(p)           __extension__ ({ bo_uint8_t   const *___p = (p); (bo_m128x4_u8_t) {{ vipmvi_load1x16_u8( ___p), vipmvi_load1x16_u8( (___p) + 16), vipmvi_load1x16_u8( (___p) + 32), vipmvi_load1x16_u8( (___p) + 48) }}; })
#  define vipmvi__load1x16x4_u16(p)          __extension__ ({ bo_uint16_t  const *___p = (p); (bo_m128x4_u16_t){{ vipmvi_load1x16_u16(___p), vipmvi_load1x16_u16((___p) +  8), vipmvi_load1x16_u16((___p) + 16), vipmvi_load1x16_u16((___p) + 24) }}; })
#  define vipmvi__load1x16x4_u32(p)          __extension__ ({ bo_uint32_t  const *___p = (p); (bo_m128x4_u32_t){{ vipmvi_load1x16_u32(___p), vipmvi_load1x16_u32((___p) +  4), vipmvi_load1x16_u32((___p) +  8), vipmvi_load1x16_u32((___p) + 12) }}; })
#  define vipmvi__load1x16x4_f32(p)          __extension__ ({ bo_float32_t const *___p = (p); (bo_m128x4_f32_t){{ vipmvi_load1x16_f32(___p), vipmvi_load1x16_f32((___p) +  4), vipmvi_load1x16_f32((___p) +  8), vipmvi_load1x16_f32((___p) + 12) }}; })

# else /* default compiler */

_VIPM_INLINE bo_m128x4_s8_t  (vipmvi__loada1x16x4_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_loada1x16_s8(p), rb = vipmvi_loada1x16_s8((p) + 16), rc = vipmvi_loada1x16_s8((p) + 32), rd = vipmvi_loada1x16_s8((p) + 48);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s8_t  rv = {{ vipmvi_loada1x16_s8(p),  vipmvi_loada1x16_s8((p) + 16),  vipmvi_loada1x16_s8((p) + 32), vipmvi_loada1x16_s8((p) + 48)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_s16_t (vipmvi__loada1x16x4_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s16_t rv;
	bo_m128_s16_t ra = vipmvi_loada1x16_s16(p), rb = vipmvi_loada1x16_s16((p) + 8), rc = vipmvi_loada1x16_s16((p) + 16), rd = vipmvi_loada1x16_s16((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s16_t rv = {{ vipmvi_loada1x16_s16(p), vipmvi_loada1x16_s16((p) + 8), vipmvi_loada1x16_s16((p) + 16), vipmvi_loada1x16_s16((p) + 24) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_s32_t (vipmvi__loada1x16x4_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s32_t rv;
	bo_m128_s32_t ra = vipmvi_loada1x16_s32(p), rb = vipmvi_loada1x16_s32((p) + 4), rc = vipmvi_loada1x16_s32((p) + 8), rd = vipmvi_loada1x16_s32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s32_t rv = {{ vipmvi_loada1x16_s32(p), vipmvi_loada1x16_s32((p) + 4), vipmvi_loada1x16_s32((p) + 8), vipmvi_loada1x16_s32((p) + 12)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u8_t  (vipmvi__loada1x16x4_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_loada1x16_u8(p), rb = vipmvi_loada1x16_u8((p) + 16), rc = vipmvi_loada1x16_u8((p) + 32), rd = vipmvi_loada1x16_u8((p) + 48);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u8_t  rv = {{ vipmvi_loada1x16_u8(p),  vipmvi_loada1x16_u8((p) + 16),  vipmvi_loada1x16_u8((p) + 32), vipmvi_loada1x16_u8((p) + 48)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u16_t (vipmvi__loada1x16x4_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u16_t rv;
	bo_m128_u16_t ra = vipmvi_loada1x16_u16(p), rb = vipmvi_loada1x16_u16((p) + 8), rc = vipmvi_loada1x16_u16((p) + 16), rd = vipmvi_loada1x16_u16((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u16_t rv = {{ vipmvi_loada1x16_u16(p), vipmvi_loada1x16_u16((p) + 8), vipmvi_loada1x16_u16((p) + 16), vipmvi_loada1x16_u16((p) + 24) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u32_t (vipmvi__loada1x16x4_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u32_t rv;
	bo_m128_u32_t ra = vipmvi_loada1x16_u32(p), rb = vipmvi_loada1x16_u32((p) + 4), rc = vipmvi_loada1x16_u32((p) + 8), rd = vipmvi_loada1x16_u32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u32_t rv = {{ vipmvi_loada1x16_u32(p), vipmvi_loada1x16_u32((p) + 4), vipmvi_loada1x16_u32((p) + 8), vipmvi_loada1x16_u32((p) + 12)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_f32_t (vipmvi__loada1x16x4_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_f32_t rv;
	bo_m128_f32_t ra = vipmvi_loada1x16_f32(p), rb = vipmvi_loada1x16_f32((p) + 4), rc = vipmvi_loada1x16_f32((p) + 8), rd = vipmvi_loada1x16_f32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_f32_t rv = {{ vipmvi_loada1x16_f32(p), vipmvi_loada1x16_f32((p) + 4), vipmvi_loada1x16_f32((p) + 8), vipmvi_loada1x16_f32((p) + 12)  }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m128x4_s8_t  (vipmvi__loadu1x16x4_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_loadu1x16_s8(p), rb = vipmvi_loadu1x16_s8((p) + 16), rc = vipmvi_loadu1x16_s8((p) + 32), rd = vipmvi_loadu1x16_s8((p) + 48);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s8_t  rv = {{ vipmvi_loadu1x16_s8(p),  vipmvi_loadu1x16_s8((p) + 16),  vipmvi_loadu1x16_s8((p) + 32), vipmvi_loadu1x16_s8((p) + 48)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_s16_t (vipmvi__loadu1x16x4_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s16_t rv;
	bo_m128_s16_t ra = vipmvi_loadu1x16_s16(p), rb = vipmvi_loadu1x16_s16((p) + 8), rc = vipmvi_loadu1x16_s16((p) + 16), rd = vipmvi_loadu1x16_s16((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s16_t rv = {{ vipmvi_loadu1x16_s16(p), vipmvi_loadu1x16_s16((p) + 8), vipmvi_loadu1x16_s16((p) + 16), vipmvi_loadu1x16_s16((p) + 24) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_s32_t (vipmvi__loadu1x16x4_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s32_t rv;
	bo_m128_s32_t ra = vipmvi_loadu1x16_s32(p), rb = vipmvi_loadu1x16_s32((p) + 4), rc = vipmvi_loadu1x16_s32((p) + 8), rd = vipmvi_loadu1x16_s32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s32_t rv = {{ vipmvi_loadu1x16_s32(p), vipmvi_loadu1x16_s32((p) + 4), vipmvi_loadu1x16_s32((p) + 8), vipmvi_loadu1x16_s32((p) + 12)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u8_t  (vipmvi__loadu1x16x4_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_loadu1x16_u8(p), rb = vipmvi_loadu1x16_u8((p) + 16), rc = vipmvi_loadu1x16_u8((p) + 32), rd = vipmvi_loadu1x16_u8((p) + 48);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u8_t  rv = {{ vipmvi_loadu1x16_u8(p),  vipmvi_loadu1x16_u8((p) + 16),  vipmvi_loadu1x16_u8((p) + 32), vipmvi_loadu1x16_u8((p) + 48)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u16_t (vipmvi__loadu1x16x4_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u16_t rv;
	bo_m128_u16_t ra = vipmvi_loadu1x16_u16(p), rb = vipmvi_loadu1x16_u16((p) + 8), rc = vipmvi_loadu1x16_u16((p) + 16), rd = vipmvi_loadu1x16_u16((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u16_t rv = {{ vipmvi_loadu1x16_u16(p), vipmvi_loadu1x16_u16((p) + 8), vipmvi_loadu1x16_u16((p) + 16), vipmvi_loadu1x16_u16((p) + 24) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u32_t (vipmvi__loadu1x16x4_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u32_t rv;
	bo_m128_u32_t ra = vipmvi_loadu1x16_u32(p), rb = vipmvi_loadu1x16_u32((p) + 4), rc = vipmvi_loadu1x16_u32((p) + 8), rd = vipmvi_loadu1x16_u32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u32_t rv = {{ vipmvi_loadu1x16_u32(p), vipmvi_loadu1x16_u32((p) + 4), vipmvi_loadu1x16_u32((p) + 8), vipmvi_loadu1x16_u32((p) + 12)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_f32_t (vipmvi__loadu1x16x4_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_f32_t rv;
	bo_m128_f32_t ra = vipmvi_loadu1x16_f32(p), rb = vipmvi_loadu1x16_f32((p) + 4), rc = vipmvi_loadu1x16_f32((p) + 8), rd = vipmvi_loadu1x16_f32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_f32_t rv = {{ vipmvi_loadu1x16_f32(p), vipmvi_loadu1x16_f32((p) + 4), vipmvi_loadu1x16_f32((p) + 8), vipmvi_loadu1x16_f32((p) + 12)  }};
#  endif
	return (rv);
}

_VIPM_INLINE bo_m128x4_s8_t  (vipmvi__load1x16x4_s8)(bo_int8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s8_t  rv;
	bo_m128_s8_t ra = vipmvi_load1x16_s8(p), rb = vipmvi_load1x16_s8((p) + 16), rc = vipmvi_load1x16_s8((p) + 32), rd = vipmvi_load1x16_s8((p) + 48);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s8_t  rv = {{ vipmvi_load1x16_s8(p),  vipmvi_load1x16_s8((p) + 16),  vipmvi_load1x16_s8((p) + 32), vipmvi_load1x16_s8((p) + 48)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_s16_t (vipmvi__load1x16x4_s16)(bo_int16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s16_t rv;
	bo_m128_s16_t ra = vipmvi_load1x16_s16(p), rb = vipmvi_load1x16_s16((p) + 8), rc = vipmvi_load1x16_s16((p) + 16), rd = vipmvi_load1x16_s16((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s16_t rv = {{ vipmvi_load1x16_s16(p), vipmvi_load1x16_s16((p) + 8), vipmvi_load1x16_s16((p) + 16), vipmvi_load1x16_s16((p) + 24) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_s32_t (vipmvi__load1x16x4_s32)(bo_int32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_s32_t rv;
	bo_m128_s32_t ra = vipmvi_load1x16_s32(p), rb = vipmvi_load1x16_s32((p) + 4), rc = vipmvi_load1x16_s32((p) + 8), rd = vipmvi_load1x16_s32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_s32_t rv = {{ vipmvi_load1x16_s32(p), vipmvi_load1x16_s32((p) + 4), vipmvi_load1x16_s32((p) + 8), vipmvi_load1x16_s32((p) + 12)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u8_t  (vipmvi__load1x16x4_u8)(bo_uint8_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u8_t  rv;
	bo_m128_u8_t ra = vipmvi_load1x16_u8(p), rb = vipmvi_load1x16_u8((p) + 16), rc = vipmvi_load1x16_u8((p) + 32), rd = vipmvi_load1x16_u8((p) + 48);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u8_t  rv = {{ vipmvi_load1x16_u8(p),  vipmvi_load1x16_u8((p) + 16),  vipmvi_load1x16_u8((p) + 32), vipmvi_load1x16_u8((p) + 48)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u16_t (vipmvi__load1x16x4_u16)(bo_uint16_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u16_t rv;
	bo_m128_u16_t ra = vipmvi_load1x16_u16(p), rb = vipmvi_load1x16_u16((p) + 8), rc = vipmvi_load1x16_u16((p) + 16), rd = vipmvi_load1x16_u16((p) + 24);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u16_t rv = {{ vipmvi_load1x16_u16(p), vipmvi_load1x16_u16((p) + 8), vipmvi_load1x16_u16((p) + 16), vipmvi_load1x16_u16((p) + 24) }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_u32_t (vipmvi__load1x16x4_u32)(bo_uint32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_u32_t rv;
	bo_m128_u32_t ra = vipmvi_load1x16_u32(p), rb = vipmvi_load1x16_u32((p) + 4), rc = vipmvi_load1x16_u32((p) + 8), rd = vipmvi_load1x16_u32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_u32_t rv = {{ vipmvi_load1x16_u32(p), vipmvi_load1x16_u32((p) + 4), vipmvi_load1x16_u32((p) + 8), vipmvi_load1x16_u32((p) + 12)  }};
#  endif
	return (rv);
}
_VIPM_INLINE bo_m128x4_f32_t (vipmvi__load1x16x4_f32)(bo_float32_t const *p) {
#  if (defined(_MSC_VER) && _MSC_VER <= 1800)
	bo_m128x4_f32_t rv;
	bo_m128_f32_t ra = vipmvi_load1x16_f32(p), rb = vipmvi_load1x16_f32((p) + 4), rc = vipmvi_load1x16_f32((p) + 8), rd = vipmvi_load1x16_f32((p) + 12);
	rv.val[0] = ra;
	rv.val[1] = rb;
	rv.val[2] = rc;
	rv.val[3] = rd;
#  else
	bo_m128x4_f32_t rv = {{ vipmvi_load1x16_f32(p), vipmvi_load1x16_f32((p) + 4), vipmvi_load1x16_f32((p) + 8), vipmvi_load1x16_f32((p) + 12)  }};
#  endif
	return (rv);
}

# endif /* _GCC_VER */


	/*
	 * Load/Store 1-element structures
	 * @}
	 */

/* @} */

/* ========================================================================= */

/*
 * Extract/Put lanes from/to a vector
 * @{
 */

# ifndef __VIPM_SSE4

//#  define _mm_extract_epi8(x,imm) ( (((imm) & 0x1) == 0) ? (_mm_extract_epi16((x), (imm) >> 1) & 0xFF) : (_mm_extract_epi16(_mm_srli_epi16((x), 8), (imm) >> 1)) )
#  define _mm_extract_epi8(x,imm)   ( (((imm) & 0x1) == 0) ? (_mm_extract_epi16((x), ((imm) >> 1)) & 0xFF) \
                                                           : (_mm_extract_epi16((x), ((imm) >> 1)) >> 8) )

#  define _mm_extract_epi32(x,imm)  ( (bo_int32_t) _mm_extract_epi16((x), (((imm) << 1) + 0)) | \
                                      (bo_int32_t)(_mm_extract_epi16((x), (((imm) << 1) + 1)) << 16) )

#  define _mm_extract_epi64(x,imm)  ( (bo_int64_t) _mm_extract_epi16((x), (((imm) << 2) + 0)) | \
                                      (bo_int64_t)(_mm_extract_epi16((x), (((imm) << 2) + 1)) << 16) | \
                                      (bo_int64_t)(_mm_extract_epi16((x), (((imm) << 2) + 2)) << 32) | \
                                      (bo_int64_t)(_mm_extract_epi16((x), (((imm) << 2) + 3)) << 48) )

# endif /* !defined(__VIPM_SSE4) */


// @todo: fixme - implement
_VIPM_INLINE bo_float32_t (vipmvi__lane16_f32)(__m128 x, int imm)
{
# ifndef __VIPM_SSE4
	_VIPM_ALIGNAS(16) bo_float32_t rv[4];
	vipmvi__storea_sf128(rv, x);
	return (rv[imm]);
# else
	bo_int32_t iv;
//	iv = _mm_extract_ps(x, imm);
	iv = (0 == imm ? _mm_extract_ps(x, 0)
		: 1 == imm ? _mm_extract_ps(x, 1)
		: 2 == imm ? _mm_extract_ps(x, 2)
		: 3 == imm ? _mm_extract_ps(x, 3) : 0);
	return (*(float *)&iv);
# endif
}

// @todo: fixme - implement
_VIPM_INLINE bo_float64_t (vipmvi__lane16_f64)(__m128d x, int imm)
{
# ifndef __VIPM_SSE4
	_VIPM_ALIGNAS(16) bo_float64_t rv[2];
	vipmvi__storea_df128(rv, x);
	return (rv[imm]);
# else
	_VIPM_ALIGNAS(16) bo_float64_t rv[2];
	vipmvi__storea_df128(rv, x);
	return (rv[imm]);
# endif
}

/* @} */


__MYC_END_CDECLS

#endif /* !defined(__VIPM_SIMD_SSE_H__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
