/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 *
 * $Id$
 */

/*
 * Asm.gcc.h	(V.Sokur)
 * Vipm/machine/arm32/Asm.gcc.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ARM32_ASM_GCC__
# define __VIPM_ARM32_ASM_GCC__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifdef __ARM_NEON__
#  include <arm_neon.h>
# endif
 */

# ifndef __VIPM_ARM_SDEFS__
#  include <Vipm/machine/arm/Sdefs.h>
# endif


/******************************************************************************
 * %% BeginSection: function definitions
 */

/*
 * @{
 */

/*
 * [gcc inline asm]
 *
 *
 * ARM32 registers:
 * Name  Alt.name  Usage
 * r0    a1        First function argument
 *                 Integer function result
 *                 Scratch register
 * r1    a2        Second function argument
 *                 Scratch register
 * r2    a3        Third function argument
 *                 Scratch register
 * r3    a4        Fourth function argument
 *                 Scratch register
 * r4    v1        Register variable
 * r5    v2        Register variable
 * r6    v3        Register variable
 * r7    v4        Register variable
 * r8    v5        Register variable
 * r9    v6        Register variable
 *       rfp       Real frame pointer
 * r10   sl        Stack limit
 * r11   fp        Argument pointer
 * r12   ip        Temporary workspace
 * r13   sp        Stack pointer
 * r14   lr        Link register
 *                 Workspace
 * r15   pc        Program counter
 *
 *
 * ARM32 128-bit vector registers:
 * AASname  ARM32name  Notes
 *   q0        q0
 *   q1        q1
 *   q2        q2
 *   q3        q3
 *   q4        q4      need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   q5        q5      need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   q6        q6      need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   q7        q7      need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   q8        q8
 *   q9        q9
 *   q10       q10
 *   q11       q11
 *   q12       q12
 *   q13       q13
 *   q14       q14
 *   q15       q15
 *
 *
 * ARM32 64-bit vector registers:
 * AASname  ARM32name 128-bit map  Notes
 *   d0        d0        q0[0]
 *   d1        d1        q0[1]
 *   d2        d2        q1[0]
 *   d3        d3        q1[1]
 *   d4        d4        q2[0]
 *   d5        d5        q2[1]
 *   d6        d6        q3[0]
 *   d7        d7        q3[1]
 *   d8        d8        q4[0]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d9        d9        q4[1]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d10       d10       q5[0]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d11       d11       q5[1]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d12       d12       q6[0]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d13       d13       q6[1]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d14       d14       q7[0]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d15       d15       q7[1]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   d16       d16       q8[0]
 *   d17       d17       q8[1]
 *   d18       d18       q9[0]
 *   d19       d19       q9[1]
 *   d20       d20       q10[0]
 *   d21       d21       q10[1]
 *   d22       d22       q11[0]
 *   d23       d23       q11[1]
 *   d24       d24       q12[0]
 *   d25       d25       q12[1]
 *   d26       d26       q13[0]
 *   d27       d27       q13[1]
 *   d28       d28       q14[0]
 *   d29       d29       q14[1]
 *   d30       d30       q15[0]
 *   d31       d31       q15[1]
 *
 *
 * ARM32 32-bit vector registers:
 * AASname  ARM32name  64-bit map  Notes
 *   s0        s0        d0[0]
 *   s1        s1        d0[1]
 *   s2        s2        d1[0]
 *   s3        s3        d1[1]
 *   s4        s4        d2[0]
 *   s5        s5        d2[1]
 *   s6        s6        d3[0]
 *   s7        s7        d3[1]
 *   s8        s8        d4[0]
 *   s9        s9        d4[1]
 *   s10       s10       d5[0]
 *   s11       s11       d5[1]
 *   s12       s12       d6[0]
 *   s13       s13       d6[1]
 *   s14       s14       d7[0]
 *   s15       s15       d7[1]
 *   s16       s16       d8[0]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s17       s17       d8[1]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s18       s18       d9[0]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s19       s19       d9[1]     need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s20       s20       d10[0]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s21       s21       d10[1]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s22       s22       d11[0]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s23       s23       d11[1]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s24       s24       d12[0]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s25       s25       d12[1]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s26       s26       d13[0]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s27       s27       d13[1]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s28       s28       d14[0]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s29       s29       d14[1]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s30       s30       d15[0]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *   s31       s31       d15[1]    need to be saved @sa: _ARM32_VPUSH/VPOP_AVA
 *
 *
 * Suffix [A,O]DB - [auto,base offset] decrement before
 * Suffix [A,O]DA - [auto,base offset] decrement after
 * Suffix [A,O]IA - [auto,base offset] increment after
 * Suffix [A,O]IB - [auto,base offset] increment before
 *
 *
 * Vector registers:
 * H - 2 byte register // ISA64
 * S - 4 byte register // ISA64
 * D - 8 byte register
 * Q - 16 byte register
 *
 *
 * UCF    - [update] conditional flag
 *
 * RR     - regular register: 1 (numerated register), [src] (named register)
 * CRR    - counter (regular register)
 * DRR    - destination [data] (regular register)
 * SRR    - source [data] (regular register)
 * PRR    - data pointer (regular register)
 * LHSRR  - [data] regular register
 * RHSRR  - [data] regular register
 * VRI    - vector register index (Q or D)
 * DVRI   - destination [data] vector register index (Q or D)
 * SVRI   - source [data] vector register index (Q or D)
 * LHSVRI - [data] vector register index (Q or D)
 * RHSVRI - [data] vector register index (Q or D)
 * Imm    - immediate value: #K
 */

/* ------------------------------------------------------------------------- */

/*
 * @{
 */

# define _ARM32_VREG_SIZE_MOD(W)             VIPM_GLUE(_ARM32_VREG_SIZE_MOD_,W)()
/* @{ */
# if defined(__clang__)
#  define _ARM32_VREG_SIZE_MOD_8()           %
#  define _ARM32_VREG_SIZE_MOD_16()          %
#  define _ARM32_VREG_SIZE_MOD_32()          %
#  define _ARM32_VREG_SIZE_MOD_64()          %
#  define _ARM32_VREG_SIZE_MOD_128()         %
# else
#  define _ARM32_VREG_SIZE_MOD_8()           %
#  define _ARM32_VREG_SIZE_MOD_16()          %
#  define _ARM32_VREG_SIZE_MOD_32()          %
#  define _ARM32_VREG_SIZE_MOD_64()          %P
#  define _ARM32_VREG_SIZE_MOD_128()         %
# endif

# define _ARM32_VREG_SIZE_MOD_I8()           _ARM32_VREG_SIZE_MOD_8()
# define _ARM32_VREG_SIZE_MOD_I16()          _ARM32_VREG_SIZE_MOD_16()
# define _ARM32_VREG_SIZE_MOD_I32()          _ARM32_VREG_SIZE_MOD_32()
# define _ARM32_VREG_SIZE_MOD_I64()          _ARM32_VREG_SIZE_MOD_64()
# define _ARM32_VREG_SIZE_MOD_I128()         _ARM32_VREG_SIZE_MOD_128()
# define _ARM32_VREG_SIZE_MOD_S8()           _ARM32_VREG_SIZE_MOD_8()
# define _ARM32_VREG_SIZE_MOD_S16()          _ARM32_VREG_SIZE_MOD_16()
# define _ARM32_VREG_SIZE_MOD_S32()          _ARM32_VREG_SIZE_MOD_32()
# define _ARM32_VREG_SIZE_MOD_S64()          _ARM32_VREG_SIZE_MOD_64()
# define _ARM32_VREG_SIZE_MOD_S128()         _ARM32_VREG_SIZE_MOD_128()
# define _ARM32_VREG_SIZE_MOD_U8()           _ARM32_VREG_SIZE_MOD_8()
# define _ARM32_VREG_SIZE_MOD_U16()          _ARM32_VREG_SIZE_MOD_16()
# define _ARM32_VREG_SIZE_MOD_U32()          _ARM32_VREG_SIZE_MOD_32()
# define _ARM32_VREG_SIZE_MOD_U64()          _ARM32_VREG_SIZE_MOD_64()
# define _ARM32_VREG_SIZE_MOD_U128()         _ARM32_VREG_SIZE_MOD_128()
# define _ARM32_VREG_SIZE_MOD_F16()          _ARM32_VREG_SIZE_MOD_16()
# define _ARM32_VREG_SIZE_MOD_F32()          _ARM32_VREG_SIZE_MOD_32()
# define _ARM32_VREG_SIZE_MOD_F64()          _ARM32_VREG_SIZE_MOD_64()
# define _ARM32_VREG_SIZE_MOD_F128()         _ARM32_VREG_SIZE_MOD_128()
/* @} */


# define _ARM32_WIDTHOF_PTR                 32
# define _ARM32_PTR_OFFSET(Offset)          VIPM_GLUE(_ARM32_PTR_OFFSET_,Offset)()
/* @{ */
# define _ARM32_PTR_OFFSET_0()              0
# define _ARM32_PTR_OFFSET_1()              4
# define _ARM32_PTR_OFFSET_2()              8
# define _ARM32_PTR_OFFSET_3()              12
# define _ARM32_PTR_OFFSET_4()              16
# define _ARM32_PTR_OFFSET_5()              20
# define _ARM32_PTR_OFFSET_6()              24
# define _ARM32_PTR_OFFSET_7()              28
# define _ARM32_PTR_OFFSET_8()              32
# define _ARM32_PTR_OFFSET_9()              36
# define _ARM32_PTR_OFFSET_10()             40
# define _ARM32_PTR_OFFSET_11()             44
# define _ARM32_PTR_OFFSET_12()             48
# define _ARM32_PTR_OFFSET_13()             52
# define _ARM32_PTR_OFFSET_14()             56
# define _ARM32_PTR_OFFSET_15()             60
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Move operations
 * @{
 */

# define _ARM32_MOVI(DRR,Imm)               "mov " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_MOVI_UCF(DRR,Imm)           "movS " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_MOVI_IF(_Cond,DRR,Imm)      "mov" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_MOVI_UCF_IF(_Cond,DRR,Imm)  "movS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM32_MOV(DRR,SRR)                "mov " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM32_MOV_UCF(DRR,SRR)            "movS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM32_MOV_IF(_Cond,DRR,SRR)       "mov" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM32_MOV_UCF_IF(_Cond,DRR,SRR)   "movS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 *
 * signed  : (x) --> (-2**(sat-1) <= x <= 2**(sat-1) -1)
 * unsigned: (x) --> (0 <= x <= 2**sat -1)
 * @{
 */

# define _ARM32_SATURATE_S(DRR,_Sat,SRR)    "ssat " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(_Sat) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM32_SATURATE_U(DRR,_Sat,SRR)    "usat " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(_Sat) ", " VIPM_STR_C(%SRR) " \n\t"

/* ------------------------------------------------------------------------- */

/* S - named variable [gcc] inline assembler */
# define _ARM32_RND_FTOSIEX(Wo,Wi,DRR,S,RndM) \
	"fstmfdd sp!,{" VIPM_STR_C(%S) "}\n\t"    \
	_ARM32_RND_VFTOSIEX(Wo,Wi,S,S,RndM)       \
	_ARM32_VMOV(DRR,S)                        \
	"fldmfdd sp!,{" VIPM_STR_C(%S) "}\n\t"
# define _ARM32_RND_FTOUIEX(Wo,Wi,DRR,S,RndM) \
	"fstmfdd sp!,{" VIPM_STR_C(%S) "}\n\t"    \
	_ARM32_RND_VFTOUIEX(Wo,Wi,S,S,RndM)       \
	_ARM32_VMOV(DRR,S)                        \
	"fldmfdd sp!,{" VIPM_STR_C(%S) "}\n\t" 

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * NEGATE: (a) --> (-a)
 * ADD : (a,b) --> (a + b)
 *		ADD[I]        : add
 *		ADD[I]_UCF    : add updating the condition flags
 *		ADD[I]_IF     : [conditionally] add
 *		ADD[I]_UCF_IF : [conditionally] add updating the condition flags
 * SUB : (a,b) --> (a - b)
 * RSUB: (a,b) --> (b - a)
 * MUL : (a,b) --> (a * b)
 */

/* ------------------------------------------------------------------------- */

    /*
     * NEGATE
	 * @{
     */

# define _ARM32_NEGATE(DRR,SRR)             _ARM32_RSUBI(DRR,SRR,0)
# define _ARM32_NEGATE_UCF(DRR,SRR)         "neg " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"

# define _ARM32_NEGATE_IF(_Cond, DRR,SRR)   _ARM32_RSUBI_IF(DRR,SRR,0)

# define _ARM32_NEGATE_UCF_IF(_Cond, DRR,SRR) \
    "neg" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * ADD
	 * @{
     */

# define _ARM32_ADDI(DRR,SRR,Imm)       "add " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_ADDI_UCF(DRR,SRR,Imm)   "addS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_ADDI_IF(_Cond, DRR,SRR,Imm) \
    "add" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_ADDI_UCF_IF(_Cond, DRR,SRR,Imm) \
    "addS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM32_ADD(DRR,SRR1,SRR2)      "add " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_ADD_UCF(DRR,SRR1,SRR2)  "addS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_ADD_IF(_Cond, DRR,SRR1,SRR2) \
    "add" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_ADD_UCF_IF(_Cond, DRR,SRR1,SRR2) \
    "addS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

	/* @} */


    /*
     * ADDEX
	 * @{
     */

# define _ARM32_ADDEX(DRR,SRR1,SRR2,Ext)        "add " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"
# define _ARM32_ADDEX_UCF(DRR,SRR1,SRR2,Ext)    "addS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM32_ADDEX_IF(_Cond, DRR,SRR1,SRR2,Ext) \
    "add" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM32_ADDEX_UCF_IF(_Cond, DRR,SRR1,SRR2,Ext) \
    "addS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * SUB
	 * @{
     */

# define _ARM32_SUBI(DRR,SRR,Imm)       "sub " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_SUBI_UCF(DRR,SRR,Imm)   "subS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_SUBI_IF(_Cond, DRR,SRR,Imm) \
    "sub" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_SUBI_UCF_IF(_Cond, DRR,SRR,Imm) \
    "subS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM32_SUB(DRR,SRR1,SRR2)      "sub " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_SUB_UCF(DRR,SRR1,SRR2)  "subS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_SUB_IF(_Cond, DRR,SRR1,SRR2) \
    "sub" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_SUB_UCF_IF(_Cond, DRR,SRR1,SRR2) \
    "subS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

	/* @} */


    /*
     * SUBEX
	 * @{
     */

# define _ARM32_SUBEX(DRR,SRR1,SRR2,Ext)        "sub " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"
# define _ARM32_SUBEX_UCF(DRR,SRR1,SRR2,Ext)    "subS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM32_SUBEX_IF(_Cond, DRR,SRR1,SRR2,Ext) \
    "sub" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM32_SUBEX_UCF_IF(_Cond, DRR,SRR1,SRR2,Ext) \
    "subS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * RSUB
	 * @{
     */

# define _ARM32_RSUBI(DRR,SRR,Imm)      "rsb " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_RSUBI_UCF(DRR,SRR,Imm)  "rsbS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RSUBI_IF(_Cond, DRR,SRR,Imm) \
    "rsb" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_RSUBI_UCF_IF(_Cond, DRR,SRR,Imm) \
    "rsbS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM32_RSUB(DRR,SRR1,SRR2)     "rsb " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_RSUB_UCF(DRR,SRR1,SRR2) "rsbS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_RSUB_IF(_Cond, DRR,SRR1,SRR2) \
    "rsb" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_RSUB_UCF_IF(_Cond, DRR,SRR1,SRR2) \
    "rsbS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

	/* @} */


    /*
     * RSUBEX
	 * @{
     */

# define _ARM32_RSUBEX(DRR,SRR1,SRR2,Ext)       "rsb " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"
# define _ARM32_RSUBEX_UCF(DRR,SRR1,SRR2,Ext)   "rsbS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM32_RSUBEX_IF(_Cond, DRR,SRR1,SRR2,Ext) \
    "rsb" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM32_RSUBEX_UCF_IF(_Cond, DRR,SRR1,SRR2,Ext) \
    "rsbS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * MUL
	 * @{
     */

# define _ARM32_MUL(DRR,SRR1,SRR2)        "mul " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_MUL_UCF(DRR,SRR1,SRR2)    "mulS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_MUL_IF(_Cond, DRR,SRR1,SRR2) \
    "mul" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM32_MUL_UCF_IF(_Cond, DRR,SRR1,SRR2) \
    "mulS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

	/* @} */

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

# define _ARM32_CMPI(LHSRR,Imm)             "cmp " VIPM_STR_C(%LHSRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_CMPI_IF(_Cond,LHSRR,Imm)    "cmp" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%LHSRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_CMP(LHSRR,RHSRR)            "cmp " VIPM_STR_C(%LHSRR) ", " VIPM_STR_C(%RHSRR) " \n\t"
# define _ARM32_CMP_IF(_Cond,LHSRR,RHSRR)   "cmp" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%LHSRR) ", " VIPM_STR_C(%RHSRR) " \n\t"


# define _ARM32_CMPNEGI(LHSRR,Imm)          "cmn " VIPM_STR_C(%LHSRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_CMPNEGI_IF(_Cond,LHSRR,Imm) "cmn" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%LHSRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_CMPNEG(LHSRR,RHSRR)         "cmn " VIPM_STR_C(%LHSRR) ", " VIPM_STR_C(%RHSRR) " \n\t"
# define _ARM32_CMPNEG_IF(_Cond,LHSRR,RHSRR)"cmn" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%LHSRR) ", " VIPM_STR_C(%RHSRR) " \n\t"


# define _ARM32_UCF(RR)                     "tst " VIPM_STR_C(%RR) ", " VIPM_STR_C(%RR) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * not                                                       : NOT[I]     : DRR = ~SRR
 * not (updating the condition flags)                        : NOT[I]_UCF : DRR = ~SRR
 * and                                                       : AND[I]     : DRR = SRR & N
 * and (updating the condition flags)                        : AND[I]_UCF : DRR = SRR & N
 * and complement (and * not)                                : BIC[I]     : DRR = SRR & ~N
 * and complement (and * not) (updating the condition flags) : BIC[I]_UCF : DRR = SRR & ~N
 * or                                                        : OR[I]      : DRR = SRR | N
 * or (updating the condition flags)                         : OR[I]_UCF  : DRR = SRR | N
 * xor                                                       : XOR[I]     : DRR = SRR ^ N
 * xor (updating the condition flags)                        : XOR[I]_UCF : DRR = SRR ^ N
 * set all bits                                              : BITSET     :
 * clear all bits                                            : BITCLR     :
 */

/* ------------------------------------------------------------------------- */

    /*
     * NOT
     */

# define _ARM32_NOTI(DRR,Imm)           "mvn " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_NOTI_UCF(DRR,Imm)       "mvnS " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_NOTI_IF(_Cond,DRR,Imm)      "mvn" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_NOTI_UCF_IF(_Cond,DRR,Imm)  "mvnS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_NOT(DRR,SRR)            "mvn " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM32_NOT_UCF(DRR,SRR)        "mvnS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM32_NOT_IF(_Cond,DRR,SRR)       "mvn" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM32_NOT_UCF_IF(_Cond,DRR,SRR)   "mvnS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * AND
     */

# define _ARM32_ANDI(DRR, SRR, Imm)     "and " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_ANDI_UCF(DRR,SRR,Imm)   "andS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_ANDI_IF(_Cond,DRR, SRR, Imm)    "and" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_ANDI_UCF_IF(_Cond,DRR,SRR,Imm)  "andS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_AND(DRR,SRR1,SRR2)      "and " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_AND_UCF(DRR,SRR1,SRR2)  "andS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_AND_IF(_Cond,DRR,SRR1,SRR2)     "and" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_AND_UCF_IF(_Cond,DRR,SRR1,SRR2) "andS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * BIC {* bit clear *}: AND * NOT
     */

# define _ARM32_BICI(DRR, SRR, Imm)     "bic " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_BICI_UCF(DRR,SRR,Imm)   "bicS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_BICI_IF(_Cond,DRR, SRR, Imm)    "bic" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_BICI_UCF_IF(_Cond,DRR,SRR,Imm)  "bicS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_BIC(DRR,SRR1,SRR2)      "bic " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_BIC_UCF(DRR,SRR1,SRR2)  "bicS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_BIC_IF(_Cond,DRR,SRR1,SRR2)     "bic" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_BIC_UCF_IF(_Cond,DRR,SRR1,SRR2) "bicS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * OR
     */

# define _ARM32_ORRI(DRR, SRR, Imm)     "orr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_ORRI_UCF(DRR,SRR,Imm)   "orrS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_ORRI_IF(_Cond,DRR, SRR, Imm)    "orr" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_ORRI_UCF_IF(_Cond,DRR,SRR,Imm)  "orrS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_ORR(DRR,SRR1,SRR2)      "orr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_ORR_UCF(DRR,SRR1,SRR2)  "orrS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_ORR_IF(_Cond,DRR,SRR1,SRR2)     "orr" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_ORR_UCF_IF(_Cond,DRR,SRR1,SRR2) "orrS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * XOR
     */

# define _ARM32_XORI(DRR, SRR, Imm)     "eor " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_XORI_UCF(DRR,SRR,Imm)   "eorS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_XORI_IF(_Cond,DRR, SRR, Imm)    "eor" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_XORI_UCF_IF(_Cond,DRR,SRR,Imm)  "eorS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_XOR(DRR,SRR1,SRR2)      "eor " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_XOR_UCF(DRR,SRR1,SRR2)  "eorS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_XOR_IF(_Cond,DRR,SRR1,SRR2)     "eor" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_XOR_UCF_IF(_Cond,DRR,SRR1,SRR2) "eorS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * rshift: DRR = SRR >> N
 * lshift: DRR = SRR << N
 */

# define _ARM32_RSHIFTI(DRR,SRR,Imm)      "lsr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_RSHIFTI_UCF(DRR,SRR,Imm)  "lsrS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_RSHIFTI_IF(_Cond,DRR,SRR,Imm)     "lsr" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_RSHIFTI_UCF_IF(_Cond,DRR,SRR,Imm) "lsrS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RSHIFT(DRR,SRR1,SRR2)     "lsr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_RSHIFT_UCF(DRR,SRR1,SRR2) "lsrS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_RSHIFT_IF(_Cond,DRR,SRR1,SRR2)     "lsr" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_RSHIFT_UCF_IF(_Cond,DRR,SRR1,SRR2) "lsrS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"


# define _ARM32_LSHIFTI(DRR,SRR,Imm)      "lsl " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_LSHIFTI_UCF(DRR,SRR,Imm)  "lslS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_LSHIFTI_IF(_Cond,DRR,SRR,Imm)     "lsl" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_LSHIFTI_UCF_IF(_Cond,DRR,SRR,Imm) "lslS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_LSHIFT(DRR,SRR1,SRR2)     "lsl " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_LSHIFT_UCF(DRR,SRR1,SRR2) "lslS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_LSHIFT_IF(_Cond,DRR,SRR1,SRR2)     "lsl" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM32_LSHIFT_UCF_IF(_Cond,DRR,SRR1,SRR2) "lslS" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 */

    /*
     * PUSH/POP
     */
# define _ARM32_PUSH(...)               "push {" VIPM_STR_C(__VA_ARGS__) "} \n\t"
# define _ARM32_PUSH1(RR)               "push {" VIPM_STR_C(%RR) "} \n\t"
# define _ARM32_PUSH2(RR1,RR2) \
	_ARM32_PUSH1(RR1)          \
	_ARM32_PUSH1(RR2)
# define _ARM32_PUSH_ARGRR()            "push {r0-r3} \n\t"
# define _ARM32_PUSH_ALLRR()            "push {r0-r10,fp,ip,lr} \n\t"
# define _ARM32_PUSH_ALLRR0()           "push {r1-r10,fp,ip,lr} \n\t"
# define _ARM32_PUSH_AVARR()            "push {r0-r9} \n\t"
# define _ARM32_PUSH_AVARR0()           "push {r1-r9} \n\t"
# define _ARM32_POP(...)                "pop {" VIPM_STR_C(__VA_ARGS__) "} \n\t"
# define _ARM32_POP1(RR)                "pop {" VIPM_STR_C(%RR) "} \n\t"
# define _ARM32_POP2(RR1,RR2) \
	_ARM32_POP1(RR2)          \
	_ARM32_POP1(RR1)
# define _ARM32_POP_ARGRR()             "pop {r0-r3} \n\t"
# define _ARM32_POP_ALLRR()             "pop {r0-r10,fp,ip,lr} \n\t"
# define _ARM32_POP_ALLRR0()            "pop {r1-r10,fp,ip,lr} \n\t"
# define _ARM32_POP_AVARR()             "pop {r0-r9} \n\t"
# define _ARM32_POP_AVARR0()            "pop {r1-r9} \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load
     */

# define _ARM32_LOAD(W, DRR,PRR)        VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "] \n\t"
# define _ARM32_LOADO(W, DRR,PRR,ORR)   VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "] \n\t"
# define _ARM32_LOADOI(W, DRR,PRR,Off)  VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "] \n\t"

# define _ARM32_LOAD_IF(W,_Cond, DRR,PRR)       VIPM_STR_C(VIPM_GLUE(_ARM32_LDR_CMD(W),_ARM_COND_MAP(_Cond))) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "] \n\t"
# define _ARM32_LOADO_IF(W,_Cond, DRR,PRR,ORR)  VIPM_STR_C(VIPM_GLUE(_ARM32_LDR_CMD(W),_ARM_COND_MAP(_Cond))) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "] \n\t"
# define _ARM32_LOADOI_IF(W,_Cond, DRR,PRR,Off) VIPM_STR_C(VIPM_GLUE(_ARM32_LDR_CMD(W),_ARM_COND_MAP(_Cond))) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "] \n\t"


/* slow operation (?) */
# define _ARM32_LOAD_IB(W, DRR,PRR,ORR)     VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "]! \n\t"
# define _ARM32_LOAD_DB(W, DRR,PRR,ORR)     VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", -" VIPM_STR_C(%ORR) "]! \n\t"
# define _ARM32_LOAD_IA(W, DRR,PRR,ORR)     VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], " VIPM_STR_C(%ORR) " \n\t"
# define _ARM32_LOAD_DA(W, DRR,PRR,ORR)     VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], -" VIPM_STR_C(%ORR) " \n\t"
# define _ARM32_LOADI_IB(W, DRR,PRR,Off)    VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "]! \n\t"
# define _ARM32_LOADI_DB(W, DRR,PRR,Off)    VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #-" VIPM_STR_C(Off) "]! \n\t"
# define _ARM32_LOADI_IA(W, DRR,PRR,Off)    VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], #" VIPM_STR_C(Off) " \n\t"
# define _ARM32_LOADI_DA(W, DRR,PRR,Off)    VIPM_STR_C(_ARM32_LDR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], #-" VIPM_STR_C(Off) " \n\t"


/* @{ */
# define _ARM32_LDR_CMD(W)          VIPM_GLUE(_ARM32_LDR_CMD_,W)()
# define _ARM32_LDR_CMD_8()         ldrb
# define _ARM32_LDR_CMD_U8()        ldrb
# define _ARM32_LDR_CMD_S8()        ldrsb
# define _ARM32_LDR_CMD_I8()        ldrsb
# define _ARM32_LDR_CMD_16()        ldrsh
# define _ARM32_LDR_CMD_I16()       ldrsh
# define _ARM32_LDR_CMD_S16()       ldrsh
# define _ARM32_LDR_CMD_U16()       ldrh
# define _ARM32_LDR_CMD_32()        ldr
# define _ARM32_LDR_CMD_I32()       ldr
# define _ARM32_LDR_CMD_S32()       ldr
# define _ARM32_LDR_CMD_U32()       ldr
# define _ARM32_LDR_CMD_64()        ldr
# define _ARM32_LDR_CMD_I64()       ldr
# define _ARM32_LDR_CMD_S64()       ldr
# define _ARM32_LDR_CMD_U64()       ldr
/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * Store
     */

# define _ARM32_STORE(W, DRR,PRR)       VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "] \n\t"
# define _ARM32_STOREO(W, DRR,PRR,ORR)  VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "] \n\t"
# define _ARM32_STOREOI(W, DRR,PRR,Off) VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "] \n\t"

# define _ARM32_STORE_IF(W,_Cond, DRR,PRR)      VIPM_STR_C(VIPM_GLUE(_ARM32_STR_CMD(W),_ARM_COND_MAP(_Cond))) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "] \n\t"
# define _ARM32_STOREO_IF(W,_Cond, DRR,PRR,ORR) VIPM_STR_C(VIPM_GLUE(_ARM32_STR_CMD(W),_ARM_COND_MAP(_Cond))) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "] \n\t"
# define _ARM32_STOREOI_IF(W,_Cond, DRR,PRR,Off)VIPM_STR_C(VIPM_GLUE(_ARM32_STR_CMD(W),_ARM_COND_MAP(_Cond))) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "] \n\t"


/* slow operation (?) */
# define _ARM32_STORE_IB(W, DRR,PRR,ORR)    VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "]! \n\t"
# define _ARM32_STORE_DB(W, DRR,PRR,ORR)    VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", -" VIPM_STR_C(%ORR) "]! \n\t"
# define _ARM32_STORE_IA(W, DRR,PRR,ORR)    VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], " VIPM_STR_C(%ORR) " \n\t"
# define _ARM32_STORE_DA(W, DRR,PRR,ORR)    VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], -" VIPM_STR_C(%ORR) " \n\t"
# define _ARM32_STOREI_IB(W, DRR,PRR,Off)   VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "]! \n\t"
# define _ARM32_STOREI_DB(W, DRR,PRR,Off)   VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) ", #-" VIPM_STR_C(Off) "]! \n\t"
# define _ARM32_STOREI_IA(W, DRR,PRR,Off)   VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], #" VIPM_STR_C(Off) " \n\t"
# define _ARM32_STOREI_DA(W, DRR,PRR,Off)   VIPM_STR_C(_ARM32_STR_CMD(W)) " " VIPM_STR_C(%DRR) ", [" VIPM_STR_C(%PRR) "], #-" VIPM_STR_C(Off) " \n\t"


/* @{ */
# define _ARM32_STR_CMD(W)          VIPM_GLUE(_ARM32_STR_CMD_,W)()
# define _ARM32_STR_CMD_8()         strb
# define _ARM32_STR_CMD_U8()        strb
# define _ARM32_STR_CMD_S8()        strb
# define _ARM32_STR_CMD_I8()        strb
# define _ARM32_STR_CMD_16()        strh
# define _ARM32_STR_CMD_I16()       strh
# define _ARM32_STR_CMD_S16()       strh
# define _ARM32_STR_CMD_U16()       strh
# define _ARM32_STR_CMD_32()        str
# define _ARM32_STR_CMD_I32()       str
# define _ARM32_STR_CMD_S32()       str
# define _ARM32_STR_CMD_U32()       str
# define _ARM32_STR_CMD_64()        str
# define _ARM32_STR_CMD_I64()       str
# define _ARM32_STR_CMD_S64()       str
# define _ARM32_STR_CMD_U64()       str
/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * Prefetch Memory
     */

# define _ARM32_PREFETCHI_CACHELINE(_Type,_Target,_Policy,PRR,Offset)   VIPM_STR_C(_ARM32_PRFM_TYPE(_Type)) " [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Offset) "] \n\t"
# define _ARM32_APREFETCHI_CACHELINE(_Type,_Target,_Policy,PRR,Offset)  _ARM32_PREFETCHI_CACHELINE(_Type,_Target,_Policy,PRR,_ARM_PRFM_OFFSET(Offset))
# define _ARM32_PREFETCH_CACHELINE(_Type,_Target,_Policy,PRR,ORR)       VIPM_STR_C(_ARM32_PRFM_TYPE(_Type)) " [" VIPM_STR_C(%PRR) ", +" VIPM_STR_C(%ORR) "] \n\t"
# define _ARM32_PREFETCHEX_CACHELINE(_Type,_Target,_Policy,PRR,ORR,Ext) VIPM_STR_C(_ARM32_PRFM_TYPE(_Type)) " [" VIPM_STR_C(%PRR) ", +" VIPM_STR_C(%ORR) ", " Ext "] \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_PRFM_TYPE(_Type)    VIPM_GLUE(_ARM32_PRFM_TYPE_,_Type)
# define _ARM32_PRFM_TYPE_RD        pld
# define _ARM32_PRFM_TYPE_RW        pld
# define _ARM32_PRFM_TYPE_WR        pldw
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Flow control
 * @{
 */

# define _ARM32_GOTO_IMM(Label)             "b " VIPM_STR_C(Label) " \n\t"
# define _ARM32_GOTO_IF(_Cond,Label)        "b" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(Label) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Switch branch-table (ARM mode)
	 * @note: Imm = N, case <- {[0, N], default}
     */

# define _ARM32_SWITCH(RR,Imm,Label) \
	_ARM32_CMPI(RR, Imm) \
	"addls pc, pc, " VIPM_STR_C(%RR) ", asl #2\n\t" \
	_ARM32_GOTO_IMM(Label)

/* Usage sample:

ARM32:
	_ARM32_SWITCH([x], N, Ldefault)

	_ARM32_GOTO_IMM(L0)
	_ARM32_GOTO_IMM(L1)
	_ARM32_GOTO_IMM(L2)
	_ARM32_GOTO_IMM(L3)
	...
	_ARM32_GOTO_IMM(Ln)

	L0:       stmt0 _ARM32_GOTO_IMM(Lend)
	L1:       stmt1 _ARM32_GOTO_IMM(Lend)
	L2:       stmt2 _ARM32_GOTO_IMM(Lend)
	L3:       stmt3 _ARM32_GOTO_IMM(Lend)
	...
	Ln:       stmtN _ARM32_GOTO_IMM(Lend)
	Ldefault: stmtD _ARM32_GOTO_IMM(Lend)
	Lend:

C:
	switch (x) {
	case 0:  stmt0 break;
	case 1:  stmt1 break;
	case 2:  stmt2 break;
	case 3:  stmt3 break;
	...
	case N:  stmtN break;
	default: stmtD break;
	}
 */

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define _ARM32_DPRINTFI(_Str)          \
    "ldr r0,=10000f               \n\t" \
    "bl printf                    \n\t" \
    "b 10001f                     \n\t" \
	".ltorg                       \n\t" \
    "10000: .asciz " VIPM_STR_C(_Str) " \n\t" \
    ".align                       \n\t" \
    "10001:                       \n\t"


# define __ARM32_PRINTRR1(_Fmt,RR)      \
    "mov r1," VIPM_STR_C(%RR) "   \n\t" \
    _ARM32_DPRINTFI(_Fmt)

# define __ARM32_PRINTRR2(_Fmt,RR1,RR2) \
	_ARM32_PUSH1(RR1)                   \
	_ARM32_PUSH1(RR2)                   \
	"pop {r2}                     \n\t" \
	"pop {r1}                     \n\t" \
    _ARM32_DPRINTFI(_Fmt)

# define __ARM32_PRINTRR3(_Fmt,RR1,RR2,RR3) \
	_ARM32_PUSH1(RR1)                       \
	_ARM32_PUSH1(RR2)                       \
	_ARM32_PUSH1(RR3)                       \
	"pop {r3}                       \n\t"   \
	"pop {r2}                       \n\t"   \
	"pop {r1}                       \n\t"   \
    _ARM32_DPRINTFI(_Fmt)

/* @} */
/* @} */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*
 * NEON
 * @{
 */

# define _ARM32_QDMAP(L,Q)          VIPM_GLUE4(_ARM_QDMAP,L,_,Q)()

# define _ARM32_QDMAPI(W,L)         VIPM_GLUE3(_ARM32_QDMAPI,W,L)()

/* @{ */
# define _ARM32_QDMAPI_80()         0
# define _ARM32_QDMAPI_81()         0
# define _ARM32_QDMAPI_82()         0
# define _ARM32_QDMAPI_83()         0
# define _ARM32_QDMAPI_84()         0
# define _ARM32_QDMAPI_85()         0
# define _ARM32_QDMAPI_86()         0
# define _ARM32_QDMAPI_87()         0
# define _ARM32_QDMAPI_88()         1
# define _ARM32_QDMAPI_89()         1
# define _ARM32_QDMAPI_810()        1
# define _ARM32_QDMAPI_811()        1
# define _ARM32_QDMAPI_812()        1
# define _ARM32_QDMAPI_813()        1
# define _ARM32_QDMAPI_814()        1
# define _ARM32_QDMAPI_815()        1
# define _ARM32_QDMAPI_160()        0
# define _ARM32_QDMAPI_161()        0
# define _ARM32_QDMAPI_162()        0
# define _ARM32_QDMAPI_163()        0
# define _ARM32_QDMAPI_164()        1
# define _ARM32_QDMAPI_165()        1
# define _ARM32_QDMAPI_166()        1
# define _ARM32_QDMAPI_167()        1
# define _ARM32_QDMAPI_320()        0
# define _ARM32_QDMAPI_321()        0
# define _ARM32_QDMAPI_322()        1
# define _ARM32_QDMAPI_323()        1
# define _ARM32_QDMAPI_640()        0
# define _ARM32_QDMAPI_641()        1
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM32_LQDMAP(W,L)         VIPM_GLUE3(_ARM32_LQDMAP_,W,L)()

/* @{ */
# define _ARM32_LQDMAP_80()         0
# define _ARM32_LQDMAP_81()         1
# define _ARM32_LQDMAP_82()         2
# define _ARM32_LQDMAP_83()         3
# define _ARM32_LQDMAP_84()         4
# define _ARM32_LQDMAP_85()         5
# define _ARM32_LQDMAP_86()         6
# define _ARM32_LQDMAP_87()         7
# define _ARM32_LQDMAP_88()         0
# define _ARM32_LQDMAP_89()         1
# define _ARM32_LQDMAP_810()        2
# define _ARM32_LQDMAP_811()        3
# define _ARM32_LQDMAP_812()        4
# define _ARM32_LQDMAP_813()        5
# define _ARM32_LQDMAP_814()        6
# define _ARM32_LQDMAP_815()        7
# define _ARM32_LQDMAP_160()        0
# define _ARM32_LQDMAP_161()        1
# define _ARM32_LQDMAP_162()        2
# define _ARM32_LQDMAP_163()        3
# define _ARM32_LQDMAP_164()        0
# define _ARM32_LQDMAP_165()        1
# define _ARM32_LQDMAP_166()        2
# define _ARM32_LQDMAP_167()        3
# define _ARM32_LQDMAP_320()        0
# define _ARM32_LQDMAP_321()        1
# define _ARM32_LQDMAP_322()        0
# define _ARM32_LQDMAP_323()        1
# define _ARM32_LQDMAP_640()        0
# define _ARM32_LQDMAP_641()        0
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM32_DSMAP(L,D)          VIPM_GLUE4(_ARM32_DSMAP,L,_,D)()
# define _ARM32_DSMAP0(D)           VIPM_GLUE(_ARM32_DSMAP0_,D)()
# define _ARM32_DSMAP1(D)           VIPM_GLUE(_ARM32_DSMAP1_,D)()

/* @{ */
# define _ARM32_DSMAP0_0()          0
# define _ARM32_DSMAP1_0()          1
# define _ARM32_DSMAP0_1()          2
# define _ARM32_DSMAP1_1()          3
# define _ARM32_DSMAP0_2()          4
# define _ARM32_DSMAP1_2()          5
# define _ARM32_DSMAP0_3()          6
# define _ARM32_DSMAP1_3()          7
# define _ARM32_DSMAP0_4()          8
# define _ARM32_DSMAP1_4()          9
# define _ARM32_DSMAP0_5()          10
# define _ARM32_DSMAP1_5()          11
# define _ARM32_DSMAP0_6()          12
# define _ARM32_DSMAP1_6()          13
# define _ARM32_DSMAP0_7()          14
# define _ARM32_DSMAP1_7()          15
# define _ARM32_DSMAP0_8()          16
# define _ARM32_DSMAP1_8()          17
# define _ARM32_DSMAP0_9()          18
# define _ARM32_DSMAP1_9()          19
# define _ARM32_DSMAP0_10()         20
# define _ARM32_DSMAP1_10()         21
# define _ARM32_DSMAP0_11()         22
# define _ARM32_DSMAP1_11()         23
# define _ARM32_DSMAP0_12()         24
# define _ARM32_DSMAP1_12()         25
# define _ARM32_DSMAP0_13()         26
# define _ARM32_DSMAP1_13()         27
# define _ARM32_DSMAP0_14()         28
# define _ARM32_DSMAP1_14()         29
# define _ARM32_DSMAP0_15()         30
# define _ARM32_DSMAP1_15()         31
/* @} */


/* ************************************************************************* */

/*
 * Move operations
 * @{
 */

    /*
     * Duplicate/Set immediate value into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = Imm
     */
# define _ARM32_VDUPI_Q(W,VRI,Imm)      "vmov." VIPM_STR_C(_ARM32_VSETI_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_VDUPI_D(W,VRI,Imm)      "vmov." VIPM_STR_C(_ARM32_VSETI_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_VDUPFI_Q(W,VRI,Imm)     "vmov." VIPM_STR_C(_ARM32_VSETF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_VDUPFI_D(W,VRI,Imm)     "vmov." VIPM_STR_C(_ARM32_VSETF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_VDUPFI_S(W,VRI,Imm)     "vmov." VIPM_STR_C(_ARM32_VSETF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(s,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VMOV2I_Q(W,VRI,Imm)     "vmov2." VIPM_STR_C(_ARM32_VSET2I_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_VMOV2I_D(W,VRI,Imm)     "vmov2." VIPM_STR_C(_ARM32_VSET2I_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Duplicate/Set [regular register] scalar into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = $RR
     */
# define _ARM32_VDUPR_Q(W,DVRI,RR)      "vdup." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(%RR) " \n\t"
# define _ARM32_VDUPR_D(W,DVRI,RR)      "vdup." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(%RR) " \n\t"


    /*
     * Duplicate/Set [vector] scalar into every element of the destination vector register
	 * (dM[0],..., dM[N]) = dK[i], i <- {0,N}
     */
# define _ARM32_VDUP_Q(W,DVRI,L,SVRI)   "vdup." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)[L]) " \n\t"
# define _ARM32_VDUP_D(W,DVRI,L,SVRI)   "vdup." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)[L]) " \n\t"


    /*
     * Set [regular register] scalar into one element of the destination vector register
	 * dM[k] = $RR, k <- {0,N}
	 * sM = $RR
	 */
# define _ARM32_VSET1_Q(W,L,VRI,RR)     "vmov." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,_ARM32_QDMAP(_ARM32_QDMAPI(W,L),VRI))[_ARM32_LQDMAP(W,L)]) ", " VIPM_STR_C(%RR) " \n\t"
# define _ARM32_VSET1_D(W,L,VRI,RR)     "vmov." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) ", " VIPM_STR_C(%RR) " \n\t"
# define _ARM32_VSET1_S(VRI,RR)         "vmov " VIPM_STR_C(VIPM_GLUE(s,VRI)) ", " VIPM_STR_C(%RR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Get [regular register] scalar from one element of the source vector register
	 *
	 * $RR = dM[i], i <- {0,N}
	 * $RR1 = dM[0], $RR2 = dM[1]
	 * $RR = sM
     */
# define _ARM32_VGET1_D(W,L,VRI,RR)         "vmov." VIPM_STR_C(_ARM32_VGET_DATATYPE(W)) " " VIPM_STR_C(%RR) ", " VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) " \n\t"
# define _ARM32_VGET1_D_IF(W,_Cond,L,VRI,RR)"vmov" VIPM_STR_C(_ARM_COND_MAP(_Cond)) "." VIPM_STR_C(_ARM32_VGET_DATATYPE(W)) " " VIPM_STR_C(%RR) ", " VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) " \n\t"
# define _ARM32_VGET1_S(VRI,RR)             "vmov " VIPM_STR_C(%RR) ", " VIPM_STR_C(VIPM_GLUE(s,VRI)) " \n\t"
# define _ARM32_VGET1_S_IF(_Cond,VRI,RR)    "vmov" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(%RR) ", " VIPM_STR_C(VIPM_GLUE(s,VRI)) " \n\t"

# define _ARM32_VGET2_D(VRI,RR1,RR2)        "vmov " VIPM_STR_C(%RR1) ", " VIPM_STR_C(%RR2) ", " VIPM_STR_C(VIPM_GLUE(d,VRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Move between vector registers
	 */

/* Q --> Q */
/* D --> D */
/* S --> S */
# define _ARM32_VMOV_Q(DVRI,SVRI)       "vmov " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VMOV_D(DVRI,SVRI)       "vmov " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
# define _ARM32_VMOV_S(DVRI,SVRI)       "vmov " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"

/* D,S - named variables [gcc] inline assembler */
# define _ARM32_VMOV(D,S)               "vmov " VIPM_STR_C(%D) ", " VIPM_STR_C(%S) " \n\t"

# define _ARM32_VMVN_Q(DVRI,SVRI)       "vmvn " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VMVN_D(DVRI,SVRI)       "vmvn " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Move between ARM regular and SIMD and FP system registers
	 *
	 * VSR - vector system register: FPSCR, FPSID, FPEXC, ...
	 */

# define _ARM32_VFPSRTORR(VSR,RR)       "vmrs " VIPM_STR_C(%RR) ", " VIPM_STR_C(VSR) " \n\t"
# define _ARM32_VFPGETCF()              "vmrs APSR_nzcv, FPSCR \n\t"
# define _ARM32_VFPGETCF_IF(_Cond)      "vmrs" VIPM_STR_C(_ARM_COND_MAP(_Cond)) " APSR_nzcv, FPSCR \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_VSETI_DATATYPE(W)       VIPM_GLUE(_ARM32_VSETI_DATATYPE_,W)()
# define _ARM32_VSETI_DATATYPE_8()      I8
# define _ARM32_VSETI_DATATYPE_16()     I16
# define _ARM32_VSETI_DATATYPE_32()     I32
# define _ARM32_VSETI_DATATYPE_64()     I64
# define _ARM32_VSETI_DATATYPE_S8()     I8
# define _ARM32_VSETI_DATATYPE_S16()    I16
# define _ARM32_VSETI_DATATYPE_S32()    I32
# define _ARM32_VSETI_DATATYPE_S64()    I64
# define _ARM32_VSETI_DATATYPE_U8()     I8
# define _ARM32_VSETI_DATATYPE_U16()    I16
# define _ARM32_VSETI_DATATYPE_U32()    I32
# define _ARM32_VSETI_DATATYPE_U64()    I64
# define _ARM32_VSETI_DATATYPE_F32()    F32
# define _ARM32_VSETI_DATATYPE_F64()    F64

# define _ARM32_VSET2I_DATATYPE(W)      VIPM_GLUE(_ARM32_VSET2I_DATATYPE_,W)()
# define _ARM32_VSET2I_DATATYPE_8()     U8
# define _ARM32_VSET2I_DATATYPE_16()    S16
# define _ARM32_VSET2I_DATATYPE_32()    S32
# define _ARM32_VSET2I_DATATYPE_64()    S64

# define _ARM32_VSETF_DATATYPE(W)       VIPM_GLUE(_ARM32_VSETF_DATATYPE_,W)()
# define _ARM32_VSETF_DATATYPE_32()     F32
# define _ARM32_VSETF_DATATYPE_64()     F64
# define _ARM32_VSETF_DATATYPE_F32()    F32
# define _ARM32_VSETF_DATATYPE_F64()    F64

# define _ARM32_VGET_DATATYPE(W)        VIPM_GLUE(_ARM32_VGET_DATATYPE_,W)()
# define _ARM32_VGET_DATATYPE_8()       U8
# define _ARM32_VGET_DATATYPE_16()      S16
# define _ARM32_VGET_DATATYPE_32()      S32
# define _ARM32_VGET_DATATYPE_64()      S64
# define _ARM32_VGET_DATATYPE_S8()      S8
# define _ARM32_VGET_DATATYPE_S16()     S16
# define _ARM32_VGET_DATATYPE_S32()     S32
# define _ARM32_VGET_DATATYPE_S64()     S64
# define _ARM32_VGET_DATATYPE_U8()      U8
# define _ARM32_VGET_DATATYPE_U16()     U16
# define _ARM32_VGET_DATATYPE_U32()     U32
# define _ARM32_VGET_DATATYPE_U64()     U64
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 * @{
 */

/* w - source data width
 *
 * widen:   D    --> (Q)   ;; uint8 --> int16, int16 --> int32, int32 --> int64
 * widenf:  S    --> (D)   ;; float16 --> float32, float32 --> float64
 *          D    --> (Q)
 * narrow: (Q)   -->  D    ;; int16 --> uint8, int32 --> int16, int64 --> int32
 * narrowf:(D)   -->  S    ;; float32 --> float16, float64 --> float32
 *         (Q)   -->  D
 */


# define _ARM32_VWIDEN_Q1D1(W,DVRI,SVRI)    "vmovL." VIPM_STR_C(_ARM32_VMOVL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

# define _ARM32_VWIDENF_D1S1(W,DVRI,SVRI)   "vcvt.F" VIPM_STR_C(_ARM_WMUL2(W)) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"
# define _ARM32_VWIDENF_Q1D1(W,DVRI,SVRI)                      \
    _ARM32_VWIDENF_D1S1(W,_ARM_QDMAP0(DVRI),_ARM32_DSMAP0(SVRI)) \
    _ARM32_VWIDENF_D1S1(W,_ARM_QDMAP1(DVRI),_ARM32_DSMAP1(SVRI))

# define _ARM32_VNARROW_D1Q1(W,DVRI,SVRI)   "vmovn." VIPM_STR_C(_ARM32_VMOVN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"

# define _ARM32_VNARROWF_S1D1(W,DVRI,SVRI)  "vcvt.F" VIPM_STR_C(_ARM_WDIV2(W)) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
# define _ARM32_VNARROWF_D1Q1(W,DVRI,SVRI)                      \
    _ARM32_VNARROWF_S1D1(W,_ARM32_DSMAP0(DVRI),_ARM_QDMAP0(SVRI)) \
    _ARM32_VNARROWF_S1D1(W,_ARM32_DSMAP1(DVRI),_ARM_QDMAP1(SVRI))


/* narrow with saturate: Q --> D */
# define _ARM32_VSATURATE_UU_D1Q1(W,DVRI,SVRI) "vqmovn." VIPM_STR_C(_ARM32_VUQMOVN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VSATURATE_SS_D1Q1(W,DVRI,SVRI) "vqmovn." VIPM_STR_C(_ARM32_VSQMOVN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VSATURATE_SU_D1Q1(W,DVRI,SVRI) "vqmovun." VIPM_STR_C(_ARM32_VQMOVUN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * vector convert signed/unsigned integer to floating point
     */
# define _ARM32_VSITOFEX_Q(W,DVRI,SVRI) "vcvt.F" VIPM_STR_C(W) ".S" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VSITOFEX_D(W,DVRI,SVRI) "vcvt.F" VIPM_STR_C(W) ".S" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
# define _ARM32_VSITOFEX_S(W,DVRI,SVRI) "vcvt.F" VIPM_STR_C(W) ".S" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"
# define _ARM32_VUITOFEX_Q(W,DVRI,SVRI) "vcvt.F" VIPM_STR_C(W) ".U" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VUITOFEX_D(W,DVRI,SVRI) "vcvt.F" VIPM_STR_C(W) ".U" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
# define _ARM32_VUITOFEX_S(W,DVRI,SVRI) "vcvt.F" VIPM_STR_C(W) ".U" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"

	/*
     * vector convert signed/unsigned integer to double
	 * @note: SVRI must be 0-7 for Q, 0-15 for D, 0-31 for S (cause of S registers limit)
	 */
/* S --> D */
# define _ARM32_VSITOF64_D1S1(DVRI,SVRI)    "vcvt.F64.S32 " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"
/* D --> Q */
# define _ARM32_VSITOF64_Q1D1(DVRI,SVRI)                      \
    _ARM32_VSITOF64_D1S1(_ARM_QDMAP0(DVRI),_ARM32_DSMAP0(SVRI)) \
    _ARM32_VSITOF64_D1S1(_ARM_QDMAP1(DVRI),_ARM32_DSMAP1(SVRI))
/* Q --> (Q,Q) */
# define _ARM32_VSITOF64_Q2Q1(DVRI0,DVRI1,SVRI)   \
    _ARM32_VSITOF64_Q1D1(DVRI0,_ARM_QDMAP0(SVRI)) \
    _ARM32_VSITOF64_Q1D1(DVRI1,_ARM_QDMAP1(SVRI))
/* S --> D */
# define _ARM32_VUITOF64_D1S1(DVRI,SVRI)    "vcvt.F64.U32 " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"
/* D --> Q */
# define _ARM32_VUITOF64_Q1D1(DVRI,SVRI)                      \
    _ARM32_VUITOF64_D1S1(_ARM_QDMAP0(DVRI),_ARM32_DSMAP0(SVRI)) \
    _ARM32_VUITOF64_D1S1(_ARM_QDMAP1(DVRI),_ARM32_DSMAP1(SVRI))
/* Q --> (Q,Q) */
# define _ARM32_VUITOF64_Q2Q1(DVRI0,DVRI1,SVRI)   \
    _ARM32_VUITOF64_Q1D1(DVRI0,_ARM_QDMAP0(SVRI)) \
    _ARM32_VUITOF64_Q1D1(DVRI1,_ARM_QDMAP1(SVRI))

/* ------------------------------------------------------------------------- */

    /*
     * vector convert floating point to signed/unsigned integer with roundmode
     */
# define _ARM32_RND_VFTOSIEX_S(W,DVRI,SVRI,RndM)    "vcvt" VIPM_STR_C(_ARM32_VFTOI_RndM(RndM)) ".S" VIPM_STR_C(W) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"
# define _ARM32_RND_VFTOSIEX_D(W,DVRI,SVRI,RndM)    VIPM_GLUE(_ARM32_RND_VFTOSIEX_D_,_ARM32_VFTOI_RndM(RndM))(W,DVRI,SVRI)
# define _ARM32_RND_VFTOSIEX_Q(W,DVRI,SVRI,RndM)    VIPM_GLUE(_ARM32_RND_VFTOSIEX_Q_,_ARM32_VFTOI_RndM(RndM))(W,DVRI,SVRI)
# define _ARM32_RND_VFTOUIEX_S(W,DVRI,SVRI,RndM)    "vcvt" VIPM_STR_C(_ARM32_VFTOI_RndM(RndM)) ".U" VIPM_STR_C(W) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI)) " \n\t"
# define _ARM32_RND_VFTOUIEX_D(W,DVRI,SVRI,RndM)    VIPM_GLUE(_ARM32_RND_VFTOUIEX_D_,_ARM32_VFTOI_RndM(RndM))(W,DVRI,SVRI) 
# define _ARM32_RND_VFTOUIEX_Q(W,DVRI,SVRI,RndM)    VIPM_GLUE(_ARM32_RND_VFTOUIEX_Q_,_ARM32_VFTOI_RndM(RndM))(W,DVRI,SVRI)

    /*
     * vector convert double to signed/unsigned integer with roundmode
	 * @note: DVRI must be 0-7 for Q, 0-15 for D, 0-31 for S (cause of S registers limit)
     */
/* D --> S */
# define _ARM32_RND_VF64TOSIEX_S1D1(DVRI,SVRI,RndM) "vcvt" VIPM_STR_C(_ARM32_VFTOI_RndM(RndM)) ".S32.F64 " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
/* Q --> D */
# define _ARM32_RND_VF64TOSIEX_D1Q1(DVRI,SVRI,RndM)                      \
    _ARM32_RND_VF64TOSIEX_S1D1(_ARM32_DSMAP0(DVRI),_ARM_QDMAP0(SVRI),RndM) \
    _ARM32_RND_VF64TOSIEX_S1D1(_ARM32_DSMAP1(DVRI),_ARM_QDMAP1(SVRI),RndM)
/* (Q,Q) --> Q */
# define _ARM32_RND_VF64TOSIEX_Q1Q2(DVRI,SVRI0,SVRI1,RndM)   \
    _ARM32_RND_VF64TOSIEX_D1Q1(_ARM_QDMAP0(DVRI),SVRI0,RndM) \
    _ARM32_RND_VF64TOSIEX_D1Q1(_ARM_QDMAP1(DVRI),SVRI1,RndM)
/* D --> S */
# define _ARM32_RND_VF64TOUIEX_S1D1(DVRI,SVRI,RndM) "vcvt" VIPM_STR_C(_ARM32_VFTOI_RndM(RndM)) ".U32.F64 " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
/* Q --> D */
# define _ARM32_RND_VF64TOUIEX_D1Q1(DVRI,SVRI,RndM)                      \
    _ARM32_RND_VF64TOUIEX_S1D1(_ARM32_DSMAP0(DVRI),_ARM_QDMAP0(SVRI),RndM) \
    _ARM32_RND_VF64TOUIEX_S1D1(_ARM32_DSMAP1(DVRI),_ARM_QDMAP1(SVRI),RndM)
/* (Q,Q) --> Q */
# define _ARM32_RND_VF64TOUIEX_Q1Q2(DVRI,SVRI0,SVRI1,RndM)   \
    _ARM32_RND_VF64TOUIEX_D1Q1(_ARM_QDMAP0(DVRI),SVRI0,RndM) \
    _ARM32_RND_VF64TOUIEX_D1Q1(_ARM_QDMAP1(DVRI),SVRI1,RndM)


    /*
     * vector convert floating point to signed/unsigned [long] integer with roundmode
     */
/* D,S - named variables [gcc] inline assembler */
# define _ARM32_RND_VFTOSIEX(Wo,Wi,D,S,RndM) \
	"vcvt" VIPM_STR_C(_ARM32_VFTOI_RndM(RndM)) ".S" VIPM_STR_C(Wo) ".F" VIPM_STR_C(Wi) " " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(Wo)D) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(Wi)S) " \n\t"
# define _ARM32_RND_VFTOUIEX(Wo,Wi,D,S,RndM) \
	"vcvt" VIPM_STR_C(_ARM32_VFTOI_RndM(RndM)) ".U" VIPM_STR_C(Wo) ".F" VIPM_STR_C(Wi) " " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(Wo)D) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(Wi)S) " \n\t"

/* ------------------------------------------------------------------------- */

/* impl.
 * @{
 */

    /*
     * vector rounding convert floating point to signed/unsigned integer
     */
# define _ARM32_RND_VFTOSIEX_D_R(W,DVRI,SVRI)                      \
    _ARM32_RND_VFTOSIEX_S(W,_ARM32_DSMAP0(DVRI),_ARM32_DSMAP0(SVRI),Z) \
    _ARM32_RND_VFTOSIEX_S(W,_ARM32_DSMAP1(DVRI),_ARM32_DSMAP1(SVRI),Z)
# define _ARM32_RND_VFTOSIEX_Q_R(W,DVRI,SVRI)                      \
    _ARM32_RND_VFTOSIEX_D(W,_ARM_QDMAP0(DVRI),_ARM_QDMAP1(SVRI),Z) \
    _ARM32_RND_VFTOSIEX_D(W,_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI),Z)
# define _ARM32_RND_VFTOUIEX_D_R(W,DVRI,SVRI)                      \
    _ARM32_RND_VFTOUIEX_S(W,_ARM32_DSMAP0(DVRI),_ARM32_DSMAP0(SVRI),Z) \
    _ARM32_RND_VFTOUIEX_S(W,_ARM32_DSMAP1(DVRI),_ARM32_DSMAP1(SVRI),Z)
# define _ARM32_RND_VFTOUIEX_Q_R(W,DVRI,SVRI)                      \
    _ARM32_RND_VFTOUIEX_D(W,_ARM_QDMAP0(DVRI),_ARM_QDMAP1(SVRI),Z) \
    _ARM32_RND_VFTOUIEX_D(W,_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI),Z)

    /*
     * vector convert (truncate) floating point to signed/unsigned integer
     */
# define _ARM32_RND_VFTOSIEX_D_(W,DVRI,SVRI)    "vcvt.S" VIPM_STR_C(W) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
# define _ARM32_RND_VFTOSIEX_Q_(W,DVRI,SVRI)    "vcvt.S" VIPM_STR_C(W) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_RND_VFTOUIEX_D_(W,DVRI,SVRI)    "vcvt.U" VIPM_STR_C(W) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
# define _ARM32_RND_VFTOUIEX_Q_(W,DVRI,SVRI)    "vcvt.U" VIPM_STR_C(W) ".F" VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"

/* @} */

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_VMOVL_DATATYPE(W)       VIPM_GLUE(_ARM32_VMOVL_DATATYPE_,W)()
# define _ARM32_VMOVL_DATATYPE_8()      U8
# define _ARM32_VMOVL_DATATYPE_16()     S16
# define _ARM32_VMOVL_DATATYPE_32()     S32
# define _ARM32_VMOVL_DATATYPE_64()     S64
# define _ARM32_VMOVL_DATATYPE_S8()     S8
# define _ARM32_VMOVL_DATATYPE_S16()    S16
# define _ARM32_VMOVL_DATATYPE_S32()    S32
# define _ARM32_VMOVL_DATATYPE_S64()    S64
# define _ARM32_VMOVL_DATATYPE_U8()     U8
# define _ARM32_VMOVL_DATATYPE_U16()    U16
# define _ARM32_VMOVL_DATATYPE_U32()    U32
# define _ARM32_VMOVL_DATATYPE_U64()    U64

# define _ARM32_VMOVN_DATATYPE(W)       VIPM_GLUE(_ARM32_VMOVN_DATATYPE_,W)()
# define _ARM32_VMOVN_DATATYPE_16()     I16
# define _ARM32_VMOVN_DATATYPE_32()     I32
# define _ARM32_VMOVN_DATATYPE_64()     I64
# define _ARM32_VMOVN_DATATYPE_S16()    I16
# define _ARM32_VMOVN_DATATYPE_S32()    I32
# define _ARM32_VMOVN_DATATYPE_S64()    I64
# define _ARM32_VMOVN_DATATYPE_U16()    I16
# define _ARM32_VMOVN_DATATYPE_U32()    I32
# define _ARM32_VMOVN_DATATYPE_U64()    I64

# define _ARM32_VUQMOVN_DATATYPE(W)     VIPM_GLUE(_ARM32_VUQMOVN_DATATYPE_,W)()
# define _ARM32_VUQMOVN_DATATYPE_16()   U16
# define _ARM32_VUQMOVN_DATATYPE_32()   U32
# define _ARM32_VUQMOVN_DATATYPE_64()   U64
# define _ARM32_VUQMOVN_DATATYPE_S16()  U16
# define _ARM32_VUQMOVN_DATATYPE_S32()  U32
# define _ARM32_VUQMOVN_DATATYPE_S64()  U64
# define _ARM32_VUQMOVN_DATATYPE_U16()  U16
# define _ARM32_VUQMOVN_DATATYPE_U32()  U32
# define _ARM32_VUQMOVN_DATATYPE_U64()  U64

# define _ARM32_VSQMOVN_DATATYPE(W)     VIPM_GLUE(_ARM32_VSQMOVN_DATATYPE_,W)()
# define _ARM32_VSQMOVN_DATATYPE_16()   S16
# define _ARM32_VSQMOVN_DATATYPE_32()   S32
# define _ARM32_VSQMOVN_DATATYPE_64()   S64
# define _ARM32_VSQMOVN_DATATYPE_S16()  S16
# define _ARM32_VSQMOVN_DATATYPE_S32()  S32
# define _ARM32_VSQMOVN_DATATYPE_S64()  S64
# define _ARM32_VSQMOVN_DATATYPE_U16()  S16
# define _ARM32_VSQMOVN_DATATYPE_U32()  S32
# define _ARM32_VSQMOVN_DATATYPE_U64()  S64

# define _ARM32_VQMOVUN_DATATYPE(W)     VIPM_GLUE(_ARM32_VQMOVUN_DATATYPE_,W)()
# define _ARM32_VQMOVUN_DATATYPE_16()   S16
# define _ARM32_VQMOVUN_DATATYPE_32()   S32
# define _ARM32_VQMOVUN_DATATYPE_64()   S64
# define _ARM32_VQMOVUN_DATATYPE_S16()  S16
# define _ARM32_VQMOVUN_DATATYPE_S32()  S32
# define _ARM32_VQMOVUN_DATATYPE_S64()  S64
# define _ARM32_VQMOVUN_DATATYPE_U16()  S16
# define _ARM32_VQMOVUN_DATATYPE_U32()  S32
# define _ARM32_VQMOVUN_DATATYPE_U64()  S64

# define _ARM32_VFTOI_RndM(RndM)        VIPM_GLUE(_ARM32_VFTOI_RndM_,RndM)()
# define _ARM32_VFTOI_RndM_A()          R /* toward nearest, ties to away */
# define _ARM32_VFTOI_RndM_M()          R /* toward minus infinity */
# define _ARM32_VFTOI_RndM_N()          R /* toward nearest, ties to even */
# define _ARM32_VFTOI_RndM_P()          R /* toward plus infinity */
# define _ARM32_VFTOI_RndM_Z()            /* toward zero, truncate */
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * :w  - data width
 * :ww - double data width
 * for instance: w = 8, ww = 16
 *               long operation: ww = w + w
 *               wide operation: ww = ww + w
 *
 * vector negate               : r[i]:w   = -a[i]:w
 *
 * vector add                           : r[i]:w   = a[i]:w + b[i]:w
 * vector add saturated                 : r[i]:w   = saturate(a[i]:w + b[i]:w)
 * vector add long                      : r[i]:ww  = a[i]:w + b[i]:w
 * vector add wide                      : r[i]:ww  = a[i]:ww + b[i]:w
 * vector halving add                   : r[i]:w   = (a[i]:w + b[i]:w) >> 1
 * vector rounding halving add          : r[i]:w   = (a[i]:w + b[i]:w + 1) >> 1
 *
 * vector sub                           : r[i]:w   = a[i]:w - b[i]:w
 * vector sub saturated                 : r[i]:w   = saturate(a[i]:w - b[i]:w)
 * vector sub long                      : r[i]:ww  = a[i]:w - b[i]:w
 * vector sub wide                      : r[i]:ww  = a[i]:ww - b[i]:w
 * vector halving sub                   : r[i]:w   = (a[i]:w - b[i]:w) >> 1
 *
 * vector mul                           : r[i]:w   = a[i]:w * b[i]:w
 * vector mul long                      : r[i]:ww  = a[i]:w * b[i]:w
 * vector mul long saturated            : r[i]:ww  = saturate(a[i]:w * b[i]:w)
 * vector mul accumulate                : r[i]:w  += a[i]:w * b[i]:w
 * vector mul accumulate long           : r[i]:ww += a[i]:w * b[i]:w
 * vector mul accumulate saturated long : r[i]:ww = saturate(r[i]:ww + (a[i]:w * b[i]:w))
 * vector mul subtract                  : r[i]:w  -= a[i]:w * b[i]:w
 * vector mul subtract long             : r[i]:ww -= a[i]:w * b[i]:w
 * vector mul subtract saturated long   : r[i]:ww = saturate(r[i]:ww - (a[i]:w * b[i]:w))
 *
 * vector doubling mul saturated with right shift 16/32          : r = saturate((a * b * 2) >>{U} 16{32})
 * vector rounding doubling mul saturated with right shift 16/32 : r = saturate(round((a * b * 2) >>{U} 16{32}))
 */


	/*
     * VNEGATE
	 */

/* negate: Q --> Q */
/* negate: D --> D */
# define _ARM32_VNEGATE_Q(W, DVRI,SVRI) \
    "vneg." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VNEGATE_D(W, DVRI,SVRI) \
    "vneg." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

# define _ARM32_VNEGATEF_Q(W, DVRI,SVRI) \
    "vneg." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VNEGATEF_D(W, DVRI,SVRI) \
    "vneg." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

/* negate saturated: Q --> Q */
/* negate saturated: D --> D */
# define _ARM32_SAT_VNEGATE_Q(W, DVRI,SVRI) \
    "vqneg." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_SAT_VNEGATE_D(W, DVRI,SVRI) \
    "vqneg." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VADD
     */

/* add: (Q,Q) --> Q
   add: (D,D) --> D */
# define _ARM32_VADD_Q(W, DVRI,SVRI1,SVRI2) \
    "vadd." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VADD_D(W, DVRI,SVRI1,SVRI2) \
    "vadd." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* add{float}
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM32_VADDF_Q(W, DVRI,SVRI1,SVRI2) \
    "vadd." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VADDF_D(W, DVRI,SVRI1,SVRI2) \
    "vadd." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VADDF_S(DVRI,SVRI1,SVRI2) \
    "vadd.F32 " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI2)) " \n\t"

/* add{double}
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM32_VADDF64_Q(DVRI,SVRI1,SVRI2)                                   \
	_ARM32_VADDF64_D(_ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
	_ARM32_VADDF64_D(_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))
# define _ARM32_VADDF64_D(DVRI,SVRI1,SVRI2) _ARM32_VADDF_D(64,DVRI,SVRI1,SVRI2)

/* add long: (D,D) --> Q */
# define _ARM32_VADD_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vaddL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* add wide: (Q,D) --> Q */
# define _ARM32_VADD_WIDEN_Q1Q1D1(W, DVRI,SVRI1,SVRI2) \
    "vaddW." VIPM_STR_C(_ARM32_VADDW_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* add saturated: (Q,Q) --> Q
   add saturated: (D,D) --> D */
# define _ARM32_SAT_VADD_Q(W, DVRI,SVRI1,SVRI2) \
    "vqadd." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_SAT_VADD_D(W, DVRI,SVRI1,SVRI2) \
    "vqadd." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* halving add */
# define _ARM32_VHADD_Q(W, DVRI,SVRI1,SVRI2) \
    "vhadd." VIPM_STR_C(_ARM32_VHADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VHADD_D(W, DVRI,SVRI1,SVRI2) \
    "vhadd." VIPM_STR_C(_ARM32_VHADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* rounding halving add */
# define _ARM32_RND_VHADD_Q(W, DVRI,SVRI1,SVRI2) \
    "vrhadd." VIPM_STR_C(_ARM32_VHADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_RND_VHADD_D(W, DVRI,SVRI1,SVRI2) \
    "vrhadd." VIPM_STR_C(_ARM32_VHADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM32_VADDH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "vaddhn." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

/* rounding add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM32_RND_VADDH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "vraddhn." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSUB
     */

/* sub: (Q,Q) --> Q
   sub: (D,D) --> D */
# define _ARM32_VSUB_Q(W, DVRI,SVRI1,SVRI2) \
    "vsub." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VSUB_D(W, DVRI,SVRI1,SVRI2) \
    "vsub." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* sub{float}
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM32_VSUBF_Q(W, DVRI,SVRI1,SVRI2) \
    "vsub." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VSUBF_D(W, DVRI,SVRI1,SVRI2) \
    "vsub." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VSUBF_S(DVRI,SVRI1,SVRI2) \
    "vsub.F32 " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI2)) " \n\t"

/* sub{double}
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM32_VSUBF64_Q(DVRI,SVRI1,SVRI2)                                   \
	_ARM32_VSUBF64_D(_ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
	_ARM32_VSUBF64_D(_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))
# define _ARM32_VSUBF64_D(DVRI,SVRI1,SVRI2) _ARM32_VSUBF_D(64,DVRI,SVRI1,SVRI2)

/* sub long: (D,D) --> Q */
# define _ARM32_VSUB_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vsubL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* sub wide: (Q,D) --> Q */
# define _ARM32_VSUB_WIDEN_Q1Q1D1(W, DVRI,SVRI1,SVRI2) \
    "vsubW." VIPM_STR_C(_ARM32_VADDW_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* sub saturated: (Q,Q) --> Q
   sub saturated: (D,D) --> D */
# define _ARM32_SAT_VSUB_Q(W, DVRI,SVRI1,SVRI2) \
    "vqsub." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_SAT_VSUB_D(W, DVRI,SVRI1,SVRI2) \
    "vqsub." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* halving sub */
# define _ARM32_VHSUB_Q(W, DVRI,SVRI1,SVRI2) \
    "vhsub." VIPM_STR_C(_ARM32_VHADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VHSUB_D(W, DVRI,SVRI1,SVRI2) \
    "vhsub." VIPM_STR_C(_ARM32_VHADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM32_VSUBH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "vsubhn." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

/* rounding sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM32_RND_VSUBH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "vrsubhn." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VMUL
     */

/* mul: (Q,Q) --> Q
   mul: (D,D) --> D */
# define _ARM32_VMUL_Q(W, DVRI,SVRI1,SVRI2) \
    "vmul." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VMUL_D(W, DVRI,SVRI1,SVRI2) \
    "vmul." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* mulS: (Q,D[L]) --> Q
   mulS: (D,D[L]) --> D */
# define _ARM32_VMULS_Q(W, DVRI,SVRI1,L,SVRI2) \
    "vmul." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"
# define _ARM32_VMULS_D(W, DVRI,SVRI1,L,SVRI2) \
    "vmul." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

/* mul{float}
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM32_VMULF_Q(W, DVRI,SVRI1,SVRI2) \
    "vmul." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VMULF_D(W, DVRI,SVRI1,SVRI2) \
    "vmul." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULF_S(DVRI,SVRI1,SVRI2) \
    "vmul.F32 " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI2)) " \n\t"

/* mulS{float}
 * (q,d[L]) --> q
   (d,d[L]) --> d  */
# define _ARM32_VMULSF_Q(W, DVRI,SVRI1,L,SVRI2) \
	"vmul." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"
# define _ARM32_VMULSF_D(W, DVRI,SVRI1,L,SVRI2) \
	"vmul." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

/* mul{double}
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM32_VMULF64_Q(DVRI,SVRI1,SVRI2)                                   \
	_ARM32_VMULF64_D(_ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
	_ARM32_VMULF64_D(_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))
# define _ARM32_VMULF64_D(DVRI,SVRI1,SVRI2) _ARM32_VMULF_D(64,DVRI,SVRI1,SVRI2)

/* mul long: (D,D) --> Q
   mulS long: (D,D[L]) --> Q */
# define _ARM32_VMUL_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vmulL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULS_WIDEN_Q1D1D1(W, DVRI,SVRI1,L,SVRI2) \
    "vmulL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

/* mul wide: (Q,D) --> Q
   mulS wide: (Q,D[L]) --> Q */
# define _ARM32_VMUL_WIDEN_Q1Q1D1(W, DVRI,SVRI1,SVRI2) \
    "vmulW." VIPM_STR_C(_ARM32_VADDW_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULS_WIDEN_Q1Q1D1(W, DVRI,SVRI1,L,SVRI2) \
    "vmulW." VIPM_STR_C(_ARM32_VADDW_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"


/* doubling mul saturated long: (D,D) --> Q; domain: {int16, int32} */
# define _ARM32_SAT_VMUL2_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vqdmulL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* doubling mul saturated, storing only high half
   with right shift 16/32 : r = saturate((a * b * 2) >> 16{32}) */
# define _ARM32_SAT_VMUL2H_Q(W, DVRI,SVRI1,SVRI2) \
    "vqdmulh." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_SAT_VMUL2H_D(W, DVRI,SVRI1,SVRI2) \
    "vqdmulh." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* rounding doubling mul saturated, storing only high half
   with right shift 16/32: r = saturate(round((a * b * 2) >> 16{32})) */
# define _ARM32_SAT_RND_VMUL2H_Q(W, DVRI,SVRI1,SVRI2) \
    "vqrdmulh." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_SAT_RND_VMUL2H_D(W, DVRI,SVRI1,SVRI2) \
    "vqrdmulh." VIPM_STR_C(_ARM32_VQADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* mul accumulate/subtract: (Q,Q) --> Q
   mul accumulate/subtract: (D,D) --> D
   mulS accumulate/subtract: (Q,D[L]) --> Q
   mulS accumulate/subtract: (D,D[L]) --> D */
# define _ARM32_VMUL_ACCUM_Q(W, DVRI,SVRI1,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VMUL_ACCUM_D(W, DVRI,SVRI1,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULS_ACCUM_Q(W, DVRI,SVRI1,L,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"
# define _ARM32_VMULS_ACCUM_D(W, DVRI,SVRI1,L,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

# define _ARM32_VMUL_SUBTR_Q(W, DVRI,SVRI1,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VMUL_SUBTR_D(W, DVRI,SVRI1,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULS_SUBTR_Q(W, DVRI,SVRI1,L,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"
# define _ARM32_VMULS_SUBTR_D(W, DVRI,SVRI1,L,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

/* mul accumulate/subtract{float}
   (q,q) --> q
   (d,d) --> d
   (q,d[L]) --> q
   (d,d[L]) --> d */
# define _ARM32_VMULF_ACCUM_Q(W, DVRI,SVRI1,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VMULF_ACCUM_D(W, DVRI,SVRI1,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULSF_ACCUM_Q(W, DVRI,SVRI1,L,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"
# define _ARM32_VMULSF_ACCUM_D(W, DVRI,SVRI1,L,SVRI2) \
    "vmla." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

# define _ARM32_VMULF_SUBTR_Q(W, DVRI,SVRI1,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"
# define _ARM32_VMULF_SUBTR_D(W, DVRI,SVRI1,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULSF_SUBTR_Q(W, DVRI,SVRI1,L,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"
# define _ARM32_VMULSF_SUBTR_D(W, DVRI,SVRI1,L,SVRI2) \
    "vmls." VIPM_STR_C(_ARM32_VADDF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

/* mul accumulate/subtract{double}
   (q,q) --> q
   (d,d) --> d */
# define _ARM32_VMULF64_ACCUM_Q(DVRI,SVRI1,SVRI2)                                   \
	_ARM32_VMULF64_ACCUM_D(_ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
	_ARM32_VMULF64_ACCUM_D(_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))
# define _ARM32_VMULF64_ACCUM_D(DVRI,SVRI1,SVRI2)   _ARM32_VMULF_ACCUM_D(64, DVRI,SVRI1,SVRI2)

# define _ARM32_VMULF64_SUBTR_Q(DVRI,SVRI1,SVRI2)                                   \
	_ARM32_VMULF64_SUBTR_D(_ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
	_ARM32_VMULF64_SUBTR_D(_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))
# define _ARM32_VMULF64_SUBTR_D(DVRI,SVRI1,SVRI2)   _ARM32_VMULF_SUBTR_D(64, DVRI,SVRI1,SVRI2)


/* mul accumulate/subtract long: (D,D) --> Q
   mulS accumulate/subtract long: (D,D[L]) --> Q */

# define _ARM32_VMUL_ACCUM_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vmlaL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULS_ACCUM_WIDEN_Q1D1D1(W, DVRI,SVRI1,L,SVRI2) \
    "vmlaL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"

# define _ARM32_VMUL_SUBTR_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vmlsL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMULS_SUBTR_WIDEN_Q1D1D1(W, DVRI,SVRI1,L,SVRI2) \
    "vmlsL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)[L]) " \n\t"


/* doubling mul accumulate/subtract saturated long: (D,D) --> Q; domain: {int16, int32} */

# define _ARM32_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vqdmlaL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

# define _ARM32_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    "vqdmlsL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* A,X,Y - named variables [gcc] inline assembler */
/* @todo: fixme - arm version ? */
#if 0
# define _ARM32_VFMAEX(W,A,X,Y) \
	"vfma.F" VIPM_STR_C(W) " " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)A) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)X) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)Y) " \n\t"
# define _ARM32_VFMSEX(W,A,X,Y) \
	"vfms.F" VIPM_STR_C(W) " " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)A) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)X) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)Y) " \n\t"
#else
# define _ARM32_VFMAEX(W,A,X,Y) \
	"vmla.F" VIPM_STR_C(W) " " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)A) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)X) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)Y) " \n\t"
# define _ARM32_VFMSEX(W,A,X,Y) \
	"vmls.F" VIPM_STR_C(W) " " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)A) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)X) ", " VIPM_STR_C(_ARM32_VREG_SIZE_MOD(W)Y) " \n\t"
#endif


/* ------------------------------------------------------------------------- */

    /*
     * VDIV
     */

/* note: only for F32 */
# define _ARM32_VDIVF_Q(DVRI,SVRI1,SVRI2)                                   \
	_ARM32_VDIVF_D(_ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
	_ARM32_VDIVF_D(_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))
# define _ARM32_VDIVF_D(DVRI,SVRI1,SVRI2)                                         \
	_ARM32_VDIVF_S(_ARM32_DSMAP0(DVRI),_ARM32_DSMAP0(SVRI1),_ARM32_DSMAP0(SVRI2)) \
	_ARM32_VDIVF_S(_ARM32_DSMAP1(DVRI),_ARM32_DSMAP1(SVRI1),_ARM32_DSMAP1(SVRI2))
# define _ARM32_VDIVF_S(DVRI,SVRI1,SVRI2) \
    "vdiv." VIPM_STR_C(F32) " " VIPM_STR_C(VIPM_GLUE(s,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(s,SVRI2)) " \n\t"

/* note: only for F64 */
# define _ARM32_VDIVF64_Q(DVRI,SVRI1,SVRI2)                                   \
	_ARM32_VDIVF64_D(_ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
	_ARM32_VDIVF64_D(_ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))
# define _ARM32_VDIVF64_D(DVRI,SVRI1,SVRI2) \
    "vdiv." VIPM_STR_C(F64) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_VADD_DATATYPE(W)        VIPM_GLUE(_ARM32_VADD_DATATYPE_,W)()
# define _ARM32_VADD_DATATYPE_8()       I8
# define _ARM32_VADD_DATATYPE_16()      I16
# define _ARM32_VADD_DATATYPE_32()      I32
# define _ARM32_VADD_DATATYPE_64()      I64
# define _ARM32_VADD_DATATYPE_S8()      I8
# define _ARM32_VADD_DATATYPE_S16()     I16
# define _ARM32_VADD_DATATYPE_S32()     I32
# define _ARM32_VADD_DATATYPE_S64()     I64
# define _ARM32_VADD_DATATYPE_U8()      I8
# define _ARM32_VADD_DATATYPE_U16()     I16
# define _ARM32_VADD_DATATYPE_U32()     I32
# define _ARM32_VADD_DATATYPE_U64()     I64
# define _ARM32_VADD_DATATYPE_F32()     F32
# define _ARM32_VADD_DATATYPE_F64()     F64

# define _ARM32_VQADD_DATATYPE(W)       VIPM_GLUE(_ARM32_VQADD_DATATYPE_,W)()
# define _ARM32_VQADD_DATATYPE_8()      U8
# define _ARM32_VQADD_DATATYPE_16()     S16
# define _ARM32_VQADD_DATATYPE_32()     S32
# define _ARM32_VQADD_DATATYPE_64()     S64
# define _ARM32_VQADD_DATATYPE_S8()     S8
# define _ARM32_VQADD_DATATYPE_S16()    S16
# define _ARM32_VQADD_DATATYPE_S32()    S32
# define _ARM32_VQADD_DATATYPE_S64()    S64
# define _ARM32_VQADD_DATATYPE_U8()     U8
# define _ARM32_VQADD_DATATYPE_U16()    U16
# define _ARM32_VQADD_DATATYPE_U32()    U32
# define _ARM32_VQADD_DATATYPE_U64()    U64

# define _ARM32_VADDL_DATATYPE(W)       VIPM_GLUE(_ARM32_VADDL_DATATYPE_,W)()
# define _ARM32_VADDL_DATATYPE_8()      U8
# define _ARM32_VADDL_DATATYPE_16()     S16
# define _ARM32_VADDL_DATATYPE_32()     S32
# define _ARM32_VADDL_DATATYPE_64()     S64
# define _ARM32_VADDL_DATATYPE_S8()     S8
# define _ARM32_VADDL_DATATYPE_S16()    S16
# define _ARM32_VADDL_DATATYPE_S32()    S32
# define _ARM32_VADDL_DATATYPE_S64()    S64
# define _ARM32_VADDL_DATATYPE_U8()     U8
# define _ARM32_VADDL_DATATYPE_U16()    U16
# define _ARM32_VADDL_DATATYPE_U32()    U32
# define _ARM32_VADDL_DATATYPE_U64()    U64

# define _ARM32_VADDW_DATATYPE(W)       VIPM_GLUE(_ARM32_VADDW_DATATYPE_,W)()
# define _ARM32_VADDW_DATATYPE_8()      U8
# define _ARM32_VADDW_DATATYPE_16()     S16
# define _ARM32_VADDW_DATATYPE_32()     S32
# define _ARM32_VADDW_DATATYPE_64()     S64
# define _ARM32_VADDW_DATATYPE_S8()     S8
# define _ARM32_VADDW_DATATYPE_S16()    S16
# define _ARM32_VADDW_DATATYPE_S32()    S32
# define _ARM32_VADDW_DATATYPE_S64()    S64
# define _ARM32_VADDW_DATATYPE_U8()     U8
# define _ARM32_VADDW_DATATYPE_U16()    U16
# define _ARM32_VADDW_DATATYPE_U32()    U32
# define _ARM32_VADDW_DATATYPE_U64()    U64

# define _ARM32_VHADD_DATATYPE(W)       VIPM_GLUE(_ARM32_VHADD_DATATYPE_,W)()
# define _ARM32_VHADD_DATATYPE_8()      U8
# define _ARM32_VHADD_DATATYPE_16()     S16
# define _ARM32_VHADD_DATATYPE_32()     S32
# define _ARM32_VHADD_DATATYPE_64()     S64
# define _ARM32_VHADD_DATATYPE_S8()     S8
# define _ARM32_VHADD_DATATYPE_S16()    S16
# define _ARM32_VHADD_DATATYPE_S32()    S32
# define _ARM32_VHADD_DATATYPE_S64()    S64
# define _ARM32_VHADD_DATATYPE_U8()     U8
# define _ARM32_VHADD_DATATYPE_U16()    U16
# define _ARM32_VHADD_DATATYPE_U32()    U32
# define _ARM32_VHADD_DATATYPE_U64()    U64

# define _ARM32_VADDF_DATATYPE(W)       VIPM_GLUE(_ARM32_VADDF_DATATYPE_,W)()
# define _ARM32_VADDF_DATATYPE_32()     F32
# define _ARM32_VADDF_DATATYPE_64()     F64
# define _ARM32_VADDF_DATATYPE_F32()    F32
# define _ARM32_VADDF_DATATYPE_F64()    F64
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Min/Max operations
 * @{
 *
 * r[i] = min(s1[i],s2[i])
 * r[i] = max(s1[i],s2[i])
 */

    /*
     * VMIN
     */

# define _ARM32_VMIN_D(W, DVRI,SVRI1,SVRI2) \
    "vmin." VIPM_STR_C(_ARM32_VMAXMIN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMIN_Q(W, DVRI,SVRI1,SVRI2) \
    "vmin." VIPM_STR_C(_ARM32_VMAXMIN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VMINF_D(W, DVRI,SVRI1,SVRI2) \
    "vmin." VIPM_STR_C(_ARM32_VMAXMINF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMINF_Q(W, DVRI,SVRI1,SVRI2) \
    "vmin." VIPM_STR_C(_ARM32_VMAXMINF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VMAX
     */

# define _ARM32_VMAX_D(W, DVRI,SVRI1,SVRI2) \
    "vmax." VIPM_STR_C(_ARM32_VMAXMIN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMAX_Q(W, DVRI,SVRI1,SVRI2) \
    "vmax." VIPM_STR_C(_ARM32_VMAXMIN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VMAXF_D(W, DVRI,SVRI1,SVRI2) \
    "vmax." VIPM_STR_C(_ARM32_VMAXMINF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"
# define _ARM32_VMAXF_Q(W, DVRI,SVRI1,SVRI2) \
    "vmax." VIPM_STR_C(_ARM32_VMAXMINF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_VMAXMIN_DATATYPE(W)     VIPM_GLUE(_ARM32_VMAXMIN_DATATYPE_,W)()
# define _ARM32_VMAXMIN_DATATYPE_8()    U8
# define _ARM32_VMAXMIN_DATATYPE_16()   S16
# define _ARM32_VMAXMIN_DATATYPE_32()   S32
# define _ARM32_VMAXMIN_DATATYPE_64()   S64
# define _ARM32_VMAXMIN_DATATYPE_S8()   S8
# define _ARM32_VMAXMIN_DATATYPE_S16()  S16
# define _ARM32_VMAXMIN_DATATYPE_S32()  S32
# define _ARM32_VMAXMIN_DATATYPE_S64()  S64
# define _ARM32_VMAXMIN_DATATYPE_U8()   U8
# define _ARM32_VMAXMIN_DATATYPE_U16()  U16
# define _ARM32_VMAXMIN_DATATYPE_U32()  U32
# define _ARM32_VMAXMIN_DATATYPE_U64()  U64
# define _ARM32_VMAXMIN_DATATYPE_F32()  F32

# define _ARM32_VMAXMINF_DATATYPE(W)    VIPM_GLUE(_ARM32_VMAXMINF_DATATYPE_,W)()
# define _ARM32_VMAXMINF_DATATYPE_32()  F32
# define _ARM32_VMAXMINF_DATATYPE_64()  F64
/* @} */

/* @} */

/* ========================================================================= */

/*
 * Folding operations: add, min, max
 * @{
 *
 * vector pairwise add [long] :
 *     (a,b) --> r : {r[i]} = {(a[2*i] + a[2*i + 1]),... (b[2*j] + b[2*j + 1])}
 *
 * vector pairwise add accumulate long :
 *     {r[i]} = {(r[i] + a[2*i] + a[2*i + 1]),... (r[j] + b[2*j] + b[2*j + 1])}
 *
 * vector add across vector :
 *     (a,b) --> r[0] : r[0] = (a[0] + a[1] + ... + a[n - 1])
 */

    /*
     * PADD, PADD long, PADD accumulate long, VADD, VADD long
     */

/* (q,q) --> q */
/* (d,d) --> d */
# define _ARM32_OLD_VPADD_Q(W, DVRI,SVRI1,SVRI2) /* formerly used as VPADD in boxfilter_c1_u8_NxM */ \
    _ARM32_VPADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1), _ARM_QDMAP0(SVRI2)) \
    _ARM32_VPADD_D(W, _ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1), _ARM_QDMAP1(SVRI2))
# define _ARM32_VPADD_Q(W, DVRI,SVRI1,SVRI2) \
    _ARM32_VPADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1), _ARM_QDMAP1(SVRI1)) \
    _ARM32_VPADD_D(W, _ARM_QDMAP1(DVRI),_ARM_QDMAP0(SVRI2), _ARM_QDMAP1(SVRI2))

# define _ARM32_VPADD_D(W, DVRI,SVRI1,SVRI2) \
    "vpadd." VIPM_STR_C(_ARM32_VADD_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* q --> q(result in low half(d))
 * @note: high half can be modified
 */
# define _ARM32_VPADD_Q1(W, DVRI,SVRI)  VIPM_GLUE(_ARM32_VPADD_Q1_,_ARM_WIDTHOF(W))(W, DVRI,SVRI)


/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM32_VPADD_WIDEN_Q(W, DVRI,SVRI) \
    "vpaddL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"

# define _ARM32_VPADD_WIDEN_D(W, DVRI,SVRI) \
    "vpaddL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"


/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM32_VPADD_ACCUM_WIDEN_Q(W, DVRI,SVRI) \
    "vpadaL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"

# define _ARM32_VPADD_ACCUM_WIDEN_D(W, DVRI,SVRI) \
    "vpadaL." VIPM_STR_C(_ARM32_VADDL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"


/* (q,q) --> q[0] */
/* (d,d) --> d[0] */
# define _ARM32_VVADD_Q(W, DVRI,SVRI)                                       \
	_ARM32_VADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI)) \
	_ARM32_VVADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(DVRI))

# define _ARM32_VVADD_D(W, DVRI,SVRI)   VIPM_GLUE(_ARM32_VVADD_D_,_ARM_WIDTHOF(W))(W, DVRI,SVRI)


/* (q{dn,dn+1}) --> q[0] */
/* (hi-d,lo-d) --> d[0] */
# define _ARM32_VVADD_WIDEN_Q(W, DVRI,SVRI)                               \
	_ARM32_VADD_WIDEN_Q1D1D1(W, DVRI,_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI)) \
	_ARM32_VVADD_Q(_ARM_WMUL2(W), DVRI,DVRI)

# define _ARM32_VVADD_WIDEN_D(W, DVRI,SVRI) \
	_ARM32_VPADD_WIDEN_D(W, DVRI,SVRI)      \
	_ARM32_VVADD_D(_ARM_WMUL2(W), DVRI,DVRI)

/* ------------------------------------------------------------------------- */

    /*
     * PMIN, VMIN
     */

# define _ARM32_VPMIN_Q(W, DVRI,SVRI1,SVRI2) \
    _ARM32_VPMIN_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
    _ARM32_VPMIN_D(W, _ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))

# define _ARM32_VPMIN_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(vpmin._ARM32_VMAXMIN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

# define _ARM32_VPMIN_Q1(W, DVRI,SVRI) \
	_ARM32_VPMIN_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))

/* pmin< float > */
# define _ARM32_VPMINF_Q(W, DVRI,SVRI1,SVRI2) \
    _ARM32_VPMINF_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
    _ARM32_VPMINF_D(W, _ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))

# define _ARM32_VPMINF_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(vpmin._ARM32_VMAXMINF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

# define _ARM32_VPMINF_Q1(W, DVRI,SVRI) \
	_ARM32_VPMINF_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))


# define _ARM32_VVMIN_Q(W, DVRI,SVRI)                                       \
	_ARM32_VMIN_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI)) \
	_ARM32_VVMIN_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(DVRI))

# define _ARM32_VVMIN_D(W, DVRI,SVRI)   VIPM_GLUE(_ARM32_VVMIN_D_,_ARM_WIDTHOF(W))(W, DVRI,SVRI)

/* ------------------------------------------------------------------------- */

    /*
     * PMAX, VMAX
     */

# define _ARM32_VPMAX_Q(W, DVRI,SVRI1,SVRI2) \
    _ARM32_VPMAX_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
    _ARM32_VPMAX_D(W, _ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))

# define _ARM32_VPMAX_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(vpmax._ARM32_VMAXMIN_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

# define _ARM32_VPMAX_Q1(W, DVRI,SVRI) \
	_ARM32_VPMAX_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))

/* pmax< float > */
# define _ARM32_VPMAXF_Q(W, DVRI,SVRI1,SVRI2) \
    _ARM32_VPMAXF_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI1),_ARM_QDMAP0(SVRI2)) \
    _ARM32_VPMAXF_D(W, _ARM_QDMAP1(DVRI),_ARM_QDMAP1(SVRI1),_ARM_QDMAP1(SVRI2))

# define _ARM32_VPMAXF_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(vpmax._ARM32_VMAXMINF_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

# define _ARM32_VPMAXF_Q1(W, DVRI,SVRI) \
	_ARM32_VPMAXF_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))


# define _ARM32_VVMAX_Q(W, DVRI,SVRI)                                       \
	_ARM32_VMAX_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI)) \
	_ARM32_VVMAX_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(DVRI))

# define _ARM32_VVMAX_D(W, DVRI,SVRI)   VIPM_GLUE(_ARM32_VVMAX_D_,_ARM_WIDTHOF(W))(W, DVRI,SVRI)

/* ------------------------------------------------------------------------- */

    /*
     * VMINMAX
	 * (Qs1,Qs2) -> (Qd1,Qd2)
	 * Qd1 = vmin(Qs1)
	 * Qd2 = vmax(Qs2)
     */

# define _ARM32_VVMINMAX_Q(W, DVRI1,DVRI2,SVRI1,SVRI2)                         \
	_ARM32_VMIN_D(W, _ARM_QDMAP0(DVRI1),_ARM_QDMAP0(SVRI1),_ARM_QDMAP1(SVRI1)) \
	_ARM32_VMAX_D(W, _ARM_QDMAP0(DVRI2),_ARM_QDMAP0(SVRI2),_ARM_QDMAP1(SVRI2)) \
	_ARM32_VVMINMAX_D(W, _ARM_QDMAP0(DVRI1),_ARM_QDMAP0(DVRI2),_ARM_QDMAP0(DVRI1),_ARM_QDMAP0(DVRI2))

# define _ARM32_VVMINMAX_D(W, DVRI1,DVRI2,SVRI1,SVRI2)  VIPM_GLUE(_ARM32_VVMINMAX_D_,_ARM_WIDTHOF(W))(W, DVRI1,DVRI2,SVRI1,SVRI2)

/* ------------------------------------------------------------------------- */

/* impl.
 * @{
 */

    /*
     * PADD
     */

# define _ARM32_VPADD_Q1_8(W, DVRI,SVRI) \
	_ARM32_VPADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))

# define _ARM32_VPADD_Q1_16(W, DVRI,SVRI) \
	_ARM32_VPADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))

# define _ARM32_VPADD_Q1_32(W, DVRI,SVRI) \
	_ARM32_VPADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))

# define _ARM32_VPADD_Q1_64(W, DVRI,SVRI) \
	_ARM32_VADD_D(W, _ARM_QDMAP0(DVRI),_ARM_QDMAP0(SVRI),_ARM_QDMAP1(SVRI))

    /*
     * VADD
     */

# define _ARM32_VVADD_D_8(W, DVRI,SVRI) \
	_ARM32_VPADD_D(W, DVRI,SVRI,SVRI)   \
	_ARM32_VPADD_D(W, DVRI,DVRI,DVRI)   \
	_ARM32_VPADD_D(W, DVRI,DVRI,DVRI)

# define _ARM32_VVADD_D_16(W, DVRI,SVRI) \
	_ARM32_VPADD_D(W, DVRI,SVRI,SVRI)    \
	_ARM32_VPADD_D(W, DVRI,DVRI,DVRI)

# define _ARM32_VVADD_D_32(W, DVRI,SVRI) \
	_ARM32_VPADD_D(W, DVRI,SVRI,SVRI)

# define _ARM32_VVADD_D_64(W, DVRI,SVRI)

    /*
     * VMIN
     */

# define _ARM32_VVMIN_D_8(W, DVRI,SVRI) \
	_ARM32_VPMIN_D(W, DVRI,SVRI,SVRI)   \
	_ARM32_VPMIN_D(W, DVRI,DVRI,DVRI)   \
	_ARM32_VPMIN_D(W, DVRI,DVRI,DVRI)

# define _ARM32_VVMIN_D_16(W, DVRI,SVRI) \
	_ARM32_VPMIN_D(W, DVRI,SVRI,SVRI)    \
	_ARM32_VPMIN_D(W, DVRI,DVRI,DVRI)

# define _ARM32_VVMIN_D_32(W, DVRI,SVRI) \
	_ARM32_VPMIN_D(W, DVRI,SVRI,SVRI)

# define _ARM32_VVMIN_D_64(W, DVRI,SVRI)

    /*
     * VMAX
     */

# define _ARM32_VVMAX_D_8(W, DVRI,SVRI) \
	_ARM32_VPMAX_D(W, DVRI,SVRI,SVRI)   \
	_ARM32_VPMAX_D(W, DVRI,DVRI,DVRI)   \
	_ARM32_VPMAX_D(W, DVRI,DVRI,DVRI)

# define _ARM32_VVMAX_D_16(W, DVRI,SVRI) \
	_ARM32_VPMAX_D(W, DVRI,SVRI,SVRI)    \
	_ARM32_VPMAX_D(W, DVRI,DVRI,DVRI)

# define _ARM32_VVMAX_D_32(W, DVRI,SVRI) \
	_ARM32_VPMAX_D(W, DVRI,SVRI,SVRI)

# define _ARM32_VVMAX_D_64(W, DVRI,SVRI)

    /*
     * VMINMAX
     */

# define _ARM32_VVMINMAX_D_8(W, DVRI1,DVRI2,SVRI1,SVRI2) \
	_ARM32_VPMIN_D(W, DVRI1,SVRI1,SVRI1)                 \
	_ARM32_VPMAX_D(W, DVRI2,SVRI2,SVRI2)                 \
	_ARM32_VPMIN_D(W, DVRI1,DVRI1,DVRI1)                 \
	_ARM32_VPMAX_D(W, DVRI2,DVRI2,DVRI2)                 \
	_ARM32_VPMIN_D(W, DVRI1,DVRI1,DVRI1)                 \
	_ARM32_VPMAX_D(W, DVRI2,DVRI2,DVRI2)

# define _ARM32_VVMINMAX_D_16(W, DVRI1,DVRI2,SVRI1,SVRI2) \
	_ARM32_VPMIN_D(W, DVRI1,SVRI1,SVRI1)                  \
	_ARM32_VPMAX_D(W, DVRI2,SVRI2,SVRI2)                  \
	_ARM32_VPMIN_D(W, DVRI1,DVRI1,DVRI1)                  \
	_ARM32_VPMAX_D(W, DVRI2,DVRI2,DVRI2)

# define _ARM32_VVMINMAX_D_32(W, DVRI1,DVRI2,SVRI1,SVRI2) \
	_ARM32_VPMIN_D(W, DVRI1,SVRI1,SVRI1)                  \
	_ARM32_VPMAX_D(W, DVRI2,SVRI2,SVRI2)

# define _ARM32_VVMINMAX_D_64(W, DVRI1,DVRI2,SVRI1,SVRI2)

/* @} */

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

    /*
     * Compare 2 vector registers
     * Op: LT, LE, EQ, GE, GT
     */

# define _ARM32_VCMPI_Q(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM32_VCMPOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VCMPI_D(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM32_VCMPOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM32_VCMP_Q(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM32_VCMPOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,RHSVRI)) " \n\t"

# define _ARM32_VCMP_D(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM32_VCMPOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,RHSVRI)) " \n\t"


	/*
	 * float
	 */

# define _ARM32_VCMPFI_Q(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM32_VCMPFOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VCMPFI_D(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM32_VCMPFOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM32_VCMPF_Q(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM32_VCMPFOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,RHSVRI)) " \n\t"

# define _ARM32_VCMPF_D(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM32_VCMPFOP_MAP(_Op,W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,RHSVRI)) " \n\t"


/*
 * Float-Point Subsystem
 *
 * 32-bit operations  -  S registers
 * 64-bit operations  -  D registers
 */

# define _ARM32_VFPCMP32(LHSVRI,RHSVRI)             "vcmp.F32 " VIPM_STR_C(VIPM_GLUE(s,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,RHSVRI)) " \n\t"
# define _ARM32_VFPCMP64(LHSVRI,RHSVRI)             "vcmp.F64 " VIPM_STR_C(VIPM_GLUE(d,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,RHSVRI)) " \n\t"
# define _ARM32_VFPCMP32_IF(_Cond,LHSVRI,RHSVRI)    "vcmp" VIPM_STR_C(_ARM_COND_MAP(_Cond)) ".F32 " VIPM_STR_C(VIPM_GLUE(s,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(s,RHSVRI)) " \n\t"
# define _ARM32_VFPCMP64_IF(_Cond,LHSVRI,RHSVRI)    "vcmp" VIPM_STR_C(_ARM_COND_MAP(_Cond)) ".F64 " VIPM_STR_C(VIPM_GLUE(d,LHSVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,RHSVRI)) " \n\t"

# define _ARM32_VFPCMP32_UCF(LHSVRI,RHSVRI)         _ARM32_VFPCMP32(LHSVRI,RHSVRI) _ARM32_VFPGETCF()
# define _ARM32_VFPCMP64_UCF(LHSVRI,RHSVRI)         _ARM32_VFPCMP64(LHSVRI,RHSVRI) _ARM32_VFPGETCF()
# define _ARM32_VFPCMP32_UCF_IF(_Cond,LHSVRI,RHSVRI)_ARM32_VFPCMP32_IF(_Cond,LHSVRI,RHSVRI) _ARM32_VFPGETCF_IF(_Cond)
# define _ARM32_VFPCMP64_UCF_IF(_Cond,LHSVRI,RHSVRI)_ARM32_VFPCMP64_IF(_Cond,LHSVRI,RHSVRI) _ARM32_VFPGETCF_IF(_Cond)

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_VCMP_DATATYPE(W)    VIPM_GLUE(_ARM32_VCMP_DATATYPE_,W)()
# define _ARM32_VCMP_DATATYPE_8()   U8
# define _ARM32_VCMP_DATATYPE_16()  S16
# define _ARM32_VCMP_DATATYPE_32()  S32
# define _ARM32_VCMP_DATATYPE_64()  S64
# define _ARM32_VCMP_DATATYPE_S8()  S8
# define _ARM32_VCMP_DATATYPE_S16() S16
# define _ARM32_VCMP_DATATYPE_S32() S32
# define _ARM32_VCMP_DATATYPE_S64() S64
# define _ARM32_VCMP_DATATYPE_U8()  U8
# define _ARM32_VCMP_DATATYPE_U16() U16
# define _ARM32_VCMP_DATATYPE_U32() U32
# define _ARM32_VCMP_DATATYPE_U64() U64
# define _ARM32_VCMP_DATATYPE_F32() F32
# define _ARM32_VCMP_DATATYPE_F64() F64

# define _ARM32_VCMPF_DATATYPE(W)   VIPM_GLUE(_ARM32_VCMPF_DATATYPE_,W)()
# define _ARM32_VCMPF_DATATYPE_32() F32
# define _ARM32_VCMPF_DATATYPE_64() F64
# define _ARM32_VCMPF_DATATYPE_F32()F32
# define _ARM32_VCMPF_DATATYPE_F64()F64

# define _ARM32_VCEQ_DATATYPE(W)    VIPM_GLUE(_ARM32_VCEQ_DATATYPE_,W)()
# define _ARM32_VCEQ_DATATYPE_8()   I8
# define _ARM32_VCEQ_DATATYPE_16()  I16
# define _ARM32_VCEQ_DATATYPE_32()  I32
# define _ARM32_VCEQ_DATATYPE_64()  I64
# define _ARM32_VCEQ_DATATYPE_S8()  I8
# define _ARM32_VCEQ_DATATYPE_S16() I16
# define _ARM32_VCEQ_DATATYPE_S32() I32
# define _ARM32_VCEQ_DATATYPE_S64() I64
# define _ARM32_VCEQ_DATATYPE_U8()  I8
# define _ARM32_VCEQ_DATATYPE_U16() I16
# define _ARM32_VCEQ_DATATYPE_U32() I32
# define _ARM32_VCEQ_DATATYPE_U64() I64
# define _ARM32_VCEQ_DATATYPE_F32() F32
# define _ARM32_VCEQ_DATATYPE_F64() F64

# define _ARM32_VCEQF_DATATYPE(W)   VIPM_GLUE(_ARM32_VCEQF_DATATYPE_,W)()
# define _ARM32_VCEQF_DATATYPE_32() F32
# define _ARM32_VCEQF_DATATYPE_64() F64

# define _ARM32_VCMPOP_MAP(Op,W)    VIPM_GLUE(_ARM32_VCMPOP_MAP_,Op)(W)
# define _ARM32_VCMPOP_MAP_LT(W)    vclt._ARM32_VCMP_DATATYPE(W)
# define _ARM32_VCMPOP_MAP_LE(W)    vcle._ARM32_VCMP_DATATYPE(W)
# define _ARM32_VCMPOP_MAP_EQ(W)    vceq._ARM32_VCEQ_DATATYPE(W)
# define _ARM32_VCMPOP_MAP_GE(W)    vcge._ARM32_VCMP_DATATYPE(W)
# define _ARM32_VCMPOP_MAP_GT(W)    vcgt._ARM32_VCMP_DATATYPE(W)
# define _ARM32_VCMPOP_MAP_lt(W)    _ARM32_VCMPOP_MAP_LT(W)
# define _ARM32_VCMPOP_MAP_le(W)    _ARM32_VCMPOP_MAP_LE(W)
# define _ARM32_VCMPOP_MAP_eq(W)    _ARM32_VCMPOP_MAP_EQ(W)
# define _ARM32_VCMPOP_MAP_ge(W)    _ARM32_VCMPOP_MAP_GE(W)
# define _ARM32_VCMPOP_MAP_gt(W)    _ARM32_VCMPOP_MAP_GT(W)

# define _ARM32_VCMPFOP_MAP(Op,W)   VIPM_GLUE(_ARM32_VCMPOP_MAP_,Op)(VIPM_GLUE(F,W))
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * vector not
 * vector and
 * vector and complement: BIC
 * vector or
 * vector or complement: ORN
 * vector xor
 * vector set all bits
 * vector clear all bits
 * vector bit select           : bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
 * vector bit count leading 0/1
 */

    /*
     * AND
     */

# define _ARM32_VAND_Q(W, DVRI,SVRI1,SVRI2) \
    "vand " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VAND_D(W, DVRI,SVRI1,SVRI2) \
    "vand " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

    /*
     * BIC : AND complement
     */

# define _ARM32_VBIC_Q(W, DVRI,SVRI1,SVRI2) \
    "vbic " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VBIC_D(W, DVRI,SVRI1,SVRI2) \
    "vbic " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

    /*
     * OR
     */

# define _ARM32_VORR_Q(W, DVRI,SVRI1,SVRI2) \
    "vorr " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VORR_D(W, DVRI,SVRI1,SVRI2) \
    "vorr " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

    /*
     * ORN : OR complement
     */

# define _ARM32_VORN_Q(W, DVRI,SVRI1,SVRI2) \
    "vorn " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VORN_D(W, DVRI,SVRI1,SVRI2) \
    "vorn " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

    /*
     * XOR
     */

# define _ARM32_VXOR_Q(W, DVRI,SVRI1,SVRI2) \
    "veor " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VXOR_D(W, DVRI,SVRI1,SVRI2) \
    "veor " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


    /*
     * Set/Clear all bits
     */

# define _ARM32_VBITSET_Q(DVRI) "vorn " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,DVRI)) " \n\t"
# define _ARM32_VBITSET_D(DVRI) "vorn " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,DVRI)) " \n\t"
# define _ARM32_VBITCLR_Q(DVRI) "vbic " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,DVRI)) " \n\t"
# define _ARM32_VBITCLR_D(DVRI) "vbic " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,DVRI)) " \n\t"


    /*
     * Bit select: bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
     * Bit set if true:
     * Bit set if false:
     */

# define _ARM32_VBITSEL_Q(W, DVRI,SVRI1,SVRI2) \
    "vbsl." VIPM_STR_C(_ARM32_VBSL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VBITSEL_D(W, DVRI,SVRI1,SVRI2) \
    "vbsl." VIPM_STR_C(_ARM32_VBSL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


# define _ARM32_VBITIFT_Q(W, DVRI,SVRI1,SVRI2) \
    "vbit." VIPM_STR_C(_ARM32_VBSL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VBITIFT_D(W, DVRI,SVRI1,SVRI2) \
    "vbit." VIPM_STR_C(_ARM32_VBSL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


# define _ARM32_VBITIFF_Q(W, DVRI,SVRI1,SVRI2) \
    "vbif." VIPM_STR_C(_ARM32_VBSL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VBITIFF_D(W, DVRI,SVRI1,SVRI2) \
    "vbif." VIPM_STR_C(_ARM32_VBSL_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


    /*
     * Count leading zero(0)/set(1) bits
     */

# define _ARM32_VCOUNTLEADZEROBITS_Q(W, DVRI,SVRI) \
    "vclz." VIPM_STR_C(_ARM32_VCLZ_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"

# define _ARM32_VCOUNTLEADZEROBITS_D(W, DVRI,SVRI) \
    "vclz." VIPM_STR_C(_ARM32_VCLZ_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"


# define _ARM32_VCOUNTLEADSETBITS_Q(W, DVRI,SVRI) \
    "vcls." VIPM_STR_C(_ARM32_VCLS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"

# define _ARM32_VCOUNTLEADSETBITS_D(W, DVRI,SVRI) \
    "vcls." VIPM_STR_C(_ARM32_VCLS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_VBSL_DATATYPE(W)        VIPM_GLUE(_ARM32_VBSL_DATATYPE_,W)()
# define _ARM32_VBSL_DATATYPE_8()       U8
# define _ARM32_VBSL_DATATYPE_16()      U16
# define _ARM32_VBSL_DATATYPE_32()      U32
# define _ARM32_VBSL_DATATYPE_64()      U64
# define _ARM32_VBSL_DATATYPE_S8()      U8
# define _ARM32_VBSL_DATATYPE_S16()     U16
# define _ARM32_VBSL_DATATYPE_S32()     U32
# define _ARM32_VBSL_DATATYPE_S64()     U64
# define _ARM32_VBSL_DATATYPE_U8()      U8
# define _ARM32_VBSL_DATATYPE_U16()     U16
# define _ARM32_VBSL_DATATYPE_U32()     U32
# define _ARM32_VBSL_DATATYPE_U64()     U64

# define _ARM32_VCLZ_DATATYPE(W)        VIPM_GLUE(_ARM32_VCLZ_DATATYPE_,W)()
# define _ARM32_VCLZ_DATATYPE_8()       I8
# define _ARM32_VCLZ_DATATYPE_16()      I16
# define _ARM32_VCLZ_DATATYPE_32()      I32
# define _ARM32_VCLZ_DATATYPE_64()      I64
# define _ARM32_VCLZ_DATATYPE_S8()      I8
# define _ARM32_VCLZ_DATATYPE_S16()     I16
# define _ARM32_VCLZ_DATATYPE_S32()     I32
# define _ARM32_VCLZ_DATATYPE_S64()     I64
# define _ARM32_VCLZ_DATATYPE_U8()      I8
# define _ARM32_VCLZ_DATATYPE_U16()     I16
# define _ARM32_VCLZ_DATATYPE_U32()     I32
# define _ARM32_VCLZ_DATATYPE_U64()     I64

# define _ARM32_VCLS_DATATYPE(W)        VIPM_GLUE(_ARM32_VCLS_DATATYPE_,W)()
# define _ARM32_VCLS_DATATYPE_8()       S8
# define _ARM32_VCLS_DATATYPE_16()      S16
# define _ARM32_VCLS_DATATYPE_32()      S32
# define _ARM32_VCLS_DATATYPE_64()      S64
# define _ARM32_VCLS_DATATYPE_S8()      S8
# define _ARM32_VCLS_DATATYPE_S16()     S16
# define _ARM32_VCLS_DATATYPE_S32()     S32
# define _ARM32_VCLS_DATATYPE_S64()     S64
# define _ARM32_VCLS_DATATYPE_U8()      S8
# define _ARM32_VCLS_DATATYPE_U16()     S16
# define _ARM32_VCLS_DATATYPE_U32()     S32
# define _ARM32_VCLS_DATATYPE_U64()     S64
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * vector shift left by immediate
 * vector shift left saturated by immediate
 * vector shift left long by immediate
 *
 * (negative shift-value shift right)
 * vector shift left by signed var
 * vector rounding shift left by signed
 * vector shift left saturated by signed
 * vector rounding shift left saturated by signed
 *
 * vector [rounding] shift right by immediate
 * vector [rounding] shift right narrow by immediate
 * vector saturating [rounding] shift right narrow by immediate
 */

    /*
     * VSHL by immediate
     */

# define _ARM32_VSHLI_Q(W, DVRI,SVRI,Imm) \
    "vshl." VIPM_STR_C(_ARM32_VSHI_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VSHLI_D(W, DVRI,SVRI,Imm) \
    "vshl." VIPM_STR_C(_ARM32_VSHI_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* shift saturated */
# define _ARM32_SAT_VSHLI_Q(W, DVRI,SVRI,Imm) \
    "vqshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_SAT_VSHLI_D(W, DVRI,SVRI,Imm) \
    "vqshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift saturated */
# define _ARM32_SAT_VSHLSI_Q(W, DVRI,SVRI,Imm) \
    "vqshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_SAT_VSHLSI_D(W, DVRI,SVRI,Imm) \
    "vqshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift saturated */
# define _ARM32_SAT_VSHLUI_Q(W, DVRI,SVRI,Imm) \
    "vqshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_SAT_VSHLUI_D(W, DVRI,SVRI,Imm) \
    "vqshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/*
 * shift long
 */

# define _ARM32_VSHLI_WIDEN_Q1D1(W, DVRI,SVRI,Imm) \
    "vshlL." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift long */
# define _ARM32_VSHLSI_WIDEN_Q1D1(W, DVRI,SVRI,Imm) \
    "vshlL." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift long */
# define _ARM32_VSHLUI_WIDEN_Q1D1(W, DVRI,SVRI,Imm) \
    "vshlL." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSHL by signed var (negative shift-value shift right)
     */

/* shift left */
# define _ARM32_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    "vshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    "vshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* signed shift left */
# define _ARM32_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "vshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "vshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* unsigned shift left */
# define _ARM32_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "vshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "vshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* shift left saturated */
# define _ARM32_SAT_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    "vqshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_SAT_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    "vqshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* signed shift left saturated */
# define _ARM32_SAT_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "vqshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_SAT_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "vqshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* unsigned shift left saturated */
# define _ARM32_SAT_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "vqshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_SAT_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "vqshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* rounding shift left */
# define _ARM32_RND_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    "vrshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_RND_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    "vrshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* signed rounding shift left */
# define _ARM32_RND_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "vrshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_RND_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "vrshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* unsigned rounding shift left */
# define _ARM32_RND_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "vrshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_RND_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "vrshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"


/* rounding shift left saturated */
# define _ARM32_SAT_RND_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    "vqrshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_SAT_RND_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    "vqrshl." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* signed rounding shift left saturated */
# define _ARM32_SAT_RND_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "vqrshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_SAT_RND_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "vqrshl." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* unsigned rounding shift left saturated */
# define _ARM32_SAT_RND_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "vqrshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) " \n\t"

# define _ARM32_SAT_RND_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "vqrshl." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSHR by immediate
     */

/* shift right */
# define _ARM32_VSHRI_Q(W, DVRI,SVRI,Imm) \
    "vshr." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VSHRI_D(W, DVRI,SVRI,Imm) \
    "vshr." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift right */
# define _ARM32_VSHRSI_Q(W, DVRI,SVRI,Imm) \
    "vshr." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VSHRSI_D(W, DVRI,SVRI,Imm) \
    "vshr." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift right */
# define _ARM32_VSHRUI_Q(W, DVRI,SVRI,Imm) \
    "vshr." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VSHRUI_D(W, DVRI,SVRI,Imm) \
    "vshr." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* rounding shift right */
# define _ARM32_RND_VSHRI_Q(W, DVRI,SVRI,Imm) \
    "vrshr." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RND_VSHRI_D(W, DVRI,SVRI,Imm) \
    "vrshr." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed rounding shift right */
# define _ARM32_RND_VSHRSI_Q(W, DVRI,SVRI,Imm) \
    "vrshr." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RND_VSHRSI_D(W, DVRI,SVRI,Imm) \
    "vrshr." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned rounding shift right */
# define _ARM32_RND_VSHRUI_Q(W, DVRI,SVRI,Imm) \
    "vrshr." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RND_VSHRUI_D(W, DVRI,SVRI,Imm) \
    "vrshr." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* [signed/unsigned] shift right narrow */
/* narrow: Q --> D */
# define _ARM32_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vshrn." VIPM_STR_C(_ARM32_VSHI_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
//# define _ARM32_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM32_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)
//# define _ARM32_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM32_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)


/* [signed/unsigned] rounding shift right narrow */
/* narrow: Q --> D */
# define _ARM32_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vrshrn." VIPM_STR_C(_ARM32_VSHI_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
//# define _ARM32_RND_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM32_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)
//# define _ARM32_RND_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM32_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)


/* [signed/unsigned] shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM32_SAT_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vqshrn." VIPM_STR_C(_ARM32_VSHI_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_SAT_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vqshrn." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_SAT_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vqshrn." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* [signed/unsigned] rounding shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM32_SAT_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vqrshrn." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_SAT_RND_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vqrshrn." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM32_SAT_RND_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "vqrshrn." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSHR and Accumulate by immediate
     */

/* shift right */
# define _ARM32_VSHRADDI_Q(W, DVRI,SVRI,Imm) \
    "vsra." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VSHRADDI_D(W, DVRI,SVRI,Imm) \
    "vsra." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift right */
# define _ARM32_VSHRADDSI_Q(W, DVRI,SVRI,Imm) \
    "vsra." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VSHRADDSI_D(W, DVRI,SVRI,Imm) \
    "vsra." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift right */
# define _ARM32_VSHRADDUI_Q(W, DVRI,SVRI,Imm) \
    "vsra." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VSHRADDUI_D(W, DVRI,SVRI,Imm) \
    "vsra." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* rounding shift right */
# define _ARM32_RND_VSHRADDI_Q(W, DVRI,SVRI,Imm) \
    "vrsra." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RND_VSHRADDI_D(W, DVRI,SVRI,Imm) \
    "vrsra." VIPM_STR_C(_ARM32_VSH_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed rounding shift right */
# define _ARM32_RND_VSHRADDSI_Q(W, DVRI,SVRI,Imm) \
    "vrsra." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RND_VSHRADDSI_D(W, DVRI,SVRI,Imm) \
    "vrsra." VIPM_STR_C(_ARM32_VSHS_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned rounding shift right */
# define _ARM32_RND_VSHRADDUI_Q(W, DVRI,SVRI,Imm) \
    "vrsra." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_RND_VSHRADDUI_D(W, DVRI,SVRI,Imm) \
    "vrsra." VIPM_STR_C(_ARM32_VSHU_DATATYPE(W)) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM32_VSHI_DATATYPE(W)        VIPM_GLUE(_ARM32_VSHI_DATATYPE_,W)()
# define _ARM32_VSHI_DATATYPE_8()       I8
# define _ARM32_VSHI_DATATYPE_16()      I16
# define _ARM32_VSHI_DATATYPE_32()      I32
# define _ARM32_VSHI_DATATYPE_64()      I64
# define _ARM32_VSHI_DATATYPE_S8()      I8
# define _ARM32_VSHI_DATATYPE_S16()     I16
# define _ARM32_VSHI_DATATYPE_S32()     I32
# define _ARM32_VSHI_DATATYPE_S64()     I64
# define _ARM32_VSHI_DATATYPE_U8()      I8
# define _ARM32_VSHI_DATATYPE_U16()     I16
# define _ARM32_VSHI_DATATYPE_U32()     I32
# define _ARM32_VSHI_DATATYPE_U64()     I64

# define _ARM32_VSH_DATATYPE(W)         VIPM_GLUE(_ARM32_VSH_DATATYPE_,W)()
# define _ARM32_VSH_DATATYPE_8()        U8
# define _ARM32_VSH_DATATYPE_16()       S16
# define _ARM32_VSH_DATATYPE_32()       S32
# define _ARM32_VSH_DATATYPE_64()       S64
# define _ARM32_VSH_DATATYPE_S8()       S8
# define _ARM32_VSH_DATATYPE_S16()      S16
# define _ARM32_VSH_DATATYPE_S32()      S32
# define _ARM32_VSH_DATATYPE_S64()      S64
# define _ARM32_VSH_DATATYPE_U8()       U8
# define _ARM32_VSH_DATATYPE_U16()      U16
# define _ARM32_VSH_DATATYPE_U32()      U32
# define _ARM32_VSH_DATATYPE_U64()      U64

# define _ARM32_VSHS_DATATYPE(W)        VIPM_GLUE(_ARM32_VSHS_DATATYPE_,W)()
# define _ARM32_VSHS_DATATYPE_8()       S8
# define _ARM32_VSHS_DATATYPE_16()      S16
# define _ARM32_VSHS_DATATYPE_32()      S32
# define _ARM32_VSHS_DATATYPE_64()      S64
# define _ARM32_VSHS_DATATYPE_S8()      S8
# define _ARM32_VSHS_DATATYPE_S16()     S16
# define _ARM32_VSHS_DATATYPE_S32()     S32
# define _ARM32_VSHS_DATATYPE_S64()     S64
# define _ARM32_VSHS_DATATYPE_U8()      S8
# define _ARM32_VSHS_DATATYPE_U16()     S16
# define _ARM32_VSHS_DATATYPE_U32()     S32
# define _ARM32_VSHS_DATATYPE_U64()     S64

# define _ARM32_VSHU_DATATYPE(W)        VIPM_GLUE(_ARM32_VSHU_DATATYPE_,W)()
# define _ARM32_VSHU_DATATYPE_8()       U8
# define _ARM32_VSHU_DATATYPE_16()      U16
# define _ARM32_VSHU_DATATYPE_32()      U32
# define _ARM32_VSHU_DATATYPE_64()      U64
# define _ARM32_VSHU_DATATYPE_S8()      U8
# define _ARM32_VSHU_DATATYPE_S16()     U16
# define _ARM32_VSHU_DATATYPE_S32()     U32
# define _ARM32_VSHU_DATATYPE_S64()     U64
# define _ARM32_VSHU_DATATYPE_U8()      U8
# define _ARM32_VSHU_DATATYPE_U16()     U16
# define _ARM32_VSHU_DATATYPE_U32()     U32
# define _ARM32_VSHU_DATATYPE_U64()     U64
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Generic data processing operations
 * @{
 *
 * vector element shift left by immediate
 * vector element shift right by immediate
 * vector element rotate left by immediate
 * vector element rotate right by immediate
 *
 * interleaves the elements of two vector (zip)
 * de-interleaves the elements of two vector (unzip)
 */

    /*
     * VEXTRACT
     */

# define _ARM32_VEXTRACT_Q(W,DVRI,SVRI1,SVRI2,Imm) \
    "vext." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI2)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM32_VEXTRACT_D(W,DVRI,SVRI1,SVRI2,Imm) \
    "vext." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI1)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI2)) ", #" VIPM_STR_C(Imm) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VZIP/VUZP (zip/unzip)
     */

/*
 * vector elements: {0,1,2,3,4,5,6,7}
 *
 * zip(s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    s1: {a0,b0,a1,b1,a2,b2,a3,b3};
 *    s2: {a4,b4,a5,b5,a6,b6,a7,b7};
 *
 * unzip(s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    s1: {a0,a2,a4,a6,b0,b2,b4,b6};
 *    s2: {a1,a3,a5,a7,b1,b3,b5,b7};
 */
# define _ARM32_VZIP_Q(W,DVRI,SVRI)     "vzip." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VZIP_D(W,DVRI,SVRI)     "vzip." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"
# define _ARM32_VUZP_Q(W,DVRI,SVRI)     "vuzp." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VUZP_D(W,DVRI,SVRI)     "vuzp." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VTRANSPOSE2x2
     */
# define _ARM32_VTRANSPOSE2x2_Q(W,DVRI,SVRI)  "vtrn." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VTRANSPOSE2x2_D(W,DVRI,SVRI)  "vtrn." VIPM_STR_C(W) " " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSWAP
     */
# define _ARM32_VSWAP_Q(DVRI,SVRI)      "vswp " VIPM_STR_C(VIPM_GLUE(q,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(q,SVRI)) " \n\t"
# define _ARM32_VSWAP_D(DVRI,SVRI)      "vswp " VIPM_STR_C(VIPM_GLUE(d,DVRI)) ", " VIPM_STR_C(VIPM_GLUE(d,SVRI)) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 *
 * vector load multiple N-element structures into one or more registers (with de-interleaving when N>1)
 * vector store multiple N-element structures from one or more registers (with interleaving when N>1)
 *
 * vector load single N-element structure into one or more registers (with de-interleaving when N>1)
 * (Multiple copies of one n-element structure are loaded into one ore more registers).
 */


/*
 * Push/Pop
 */

	/*
     * Push/Pop fdd registers
     */

# define _ARM32_VPUSH_ALLFDD()          "fstmfdd sp!,{d0,d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12,d13,d14,d15}\n\t"
/* @sa: ARM32 vector registers */
# define _ARM32_VPUSH_AVAFDD()          "fstmfdd sp!,{d8,d9,d10,d11,d12,d13,d14,d15}\n\t"
# define _ARM32_VPUSH_FDD1(VRI)         "fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}\n\t"
# define _ARM32_VPUSH_FDD2(VRI0,VRI1) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}\n\t"
# define _ARM32_VPUSH_FDD3(VRI0,VRI1,VRI2) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}\n\t"
# define _ARM32_VPUSH_FDD4(VRI0,VRI1,VRI2,VRI3) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}\n\t"
# define _ARM32_VPUSH_FDD5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "}\n\t"
# define _ARM32_VPUSH_FDD6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "}\n\t"
# define _ARM32_VPUSH_FDD7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "}\n\t"
# define _ARM32_VPUSH_FDD8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "}\n\t"
# define _ARM32_VPUSH_FDD9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "}\n\t"
# define _ARM32_VPUSH_FDD10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "}\n\t"
# define _ARM32_VPUSH_FDD11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "}\n\t"
# define _ARM32_VPUSH_FDD12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "}\n\t"
# define _ARM32_VPUSH_FDD13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "}\n\t"
# define _ARM32_VPUSH_FDD14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "}\n\t"
# define _ARM32_VPUSH_FDD15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14) \
	"fstmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "," VIPM_STR_C(VIPM_GLUE(d,VRI14)) "}\n\t"
# define _ARM32_VPUSH_FDD16(...)        _ARM32_VPUSH_ALLFDD()

# define _ARM32_VPOP_ALLFDD()           "fldmfdd sp!,{d0,d1,d2,d3,d4,d5,d6,d7,d8,d9,d10,d11,d12,d13,d14,d15}\n\t"
/* @sa: ARM32 vector registers */
# define _ARM32_VPOP_AVAFDD()           "fldmfdd sp!,{d8,d9,d10,d11,d12,d13,d14,d15}\n\t"
# define _ARM32_VPOP_FDD1(VRI)          "fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}\n\t"
# define _ARM32_VPOP_FDD2(VRI0,VRI1) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}\n\t"
# define _ARM32_VPOP_FDD3(VRI0,VRI1,VRI2) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}\n\t"
# define _ARM32_VPOP_FDD4(VRI0,VRI1,VRI2,VRI3) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}\n\t"
# define _ARM32_VPOP_FDD5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "}\n\t"
# define _ARM32_VPOP_FDD6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "}\n\t"
# define _ARM32_VPOP_FDD7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "}\n\t"
# define _ARM32_VPOP_FDD8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "}\n\t"
# define _ARM32_VPOP_FDD9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "}\n\t"
# define _ARM32_VPOP_FDD10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "}\n\t"
# define _ARM32_VPOP_FDD11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "}\n\t"
# define _ARM32_VPOP_FDD12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "}\n\t"
# define _ARM32_VPOP_FDD13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "}\n\t"
# define _ARM32_VPOP_FDD14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "}\n\t"
# define _ARM32_VPOP_FDD15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14) \
	"fldmfdd sp!,{" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "," VIPM_STR_C(VIPM_GLUE(d,VRI14)) "}\n\t"
# define _ARM32_VPOP_FDD16(...)         _ARM32_VPOP_ALLFDD()

	/*
     * Push/Pop vector registers
     */

# define _ARM32_VPUSH_ALL()             "vpush {d0-d15}\n\t vpush {d16-d31}\n\t"

# define _ARM32_VPUSH_D1(VRI)           "vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}\n\t"
# define _ARM32_VPUSH_D2(VRI0,VRI1) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}\n\t"
# define _ARM32_VPUSH_D3(VRI0,VRI1,VRI2) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}\n\t"
# define _ARM32_VPUSH_D4(VRI0,VRI1,VRI2,VRI3) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}\n\t"
# define _ARM32_VPUSH_D5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "}\n\t"
# define _ARM32_VPUSH_D6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "}\n\t"
# define _ARM32_VPUSH_D7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "}\n\t"
# define _ARM32_VPUSH_D8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "}\n\t"
# define _ARM32_VPUSH_D9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "}\n\t"
# define _ARM32_VPUSH_D10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "}\n\t"
# define _ARM32_VPUSH_D11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "}\n\t"
# define _ARM32_VPUSH_D12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "}\n\t"
# define _ARM32_VPUSH_D13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "}\n\t"
# define _ARM32_VPUSH_D14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "}\n\t"
# define _ARM32_VPUSH_D15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "," VIPM_STR_C(VIPM_GLUE(d,VRI14)) "}\n\t"
# define _ARM32_VPUSH_D16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "," VIPM_STR_C(VIPM_GLUE(d,VRI14)) "," VIPM_STR_C(VIPM_GLUE(d,VRI15)) "}\n\t"

# define _ARM32_VPUSH_Q1(VRI)             "vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI)) "}\n\t"
# define _ARM32_VPUSH_Q2(VRI0,VRI1) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "}\n\t"
# define _ARM32_VPUSH_Q3(VRI0,VRI1,VRI2) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "}\n\t"
# define _ARM32_VPUSH_Q4(VRI0,VRI1,VRI2,VRI3) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "}\n\t"
# define _ARM32_VPUSH_Q5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "}\n\t"
# define _ARM32_VPUSH_Q6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) "}\n\t"
# define _ARM32_VPUSH_Q7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) "," VIPM_STR_C(VIPM_GLUE(q,VRI6)) "}\n\t"
# define _ARM32_VPUSH_Q8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	"vpush {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) "," VIPM_STR_C(VIPM_GLUE(q,VRI6)) "," VIPM_STR_C(VIPM_GLUE(q,VRI7)) "}\n\t"

# define _ARM32_VPOP_ALL()              "vpop {d0-d15}\n\t vpop {d16-d31}\n\t"

# define _ARM32_VPOP_D1(VRI)            "vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}\n\t"
# define _ARM32_VPOP_D2(VRI0,VRI1) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}\n\t"
# define _ARM32_VPOP_D3(VRI0,VRI1,VRI2) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}\n\t"
# define _ARM32_VPOP_D4(VRI0,VRI1,VRI2,VRI3) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}\n\t"
# define _ARM32_VPOP_D5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "}\n\t"
# define _ARM32_VPOP_D6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "}\n\t"
# define _ARM32_VPOP_D7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "}\n\t"
# define _ARM32_VPOP_D8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "}\n\t"
# define _ARM32_VPOP_D9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "}\n\t"
# define _ARM32_VPOP_D10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "}\n\t"
# define _ARM32_VPOP_D11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "}\n\t"
# define _ARM32_VPOP_D12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "}\n\t"
# define _ARM32_VPOP_D13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "}\n\t"
# define _ARM32_VPOP_D14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "}\n\t"
# define _ARM32_VPOP_D15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "," VIPM_STR_C(VIPM_GLUE(d,VRI14)) "}\n\t"
# define _ARM32_VPOP_D16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "," VIPM_STR_C(VIPM_GLUE(d,VRI4)) "," VIPM_STR_C(VIPM_GLUE(d,VRI5)) "," VIPM_STR_C(VIPM_GLUE(d,VRI6)) "," VIPM_STR_C(VIPM_GLUE(d,VRI7)) "," VIPM_STR_C(VIPM_GLUE(d,VRI8)) "," VIPM_STR_C(VIPM_GLUE(d,VRI9)) "," VIPM_STR_C(VIPM_GLUE(d,VRI10)) "," VIPM_STR_C(VIPM_GLUE(d,VRI11)) "," VIPM_STR_C(VIPM_GLUE(d,VRI12)) "," VIPM_STR_C(VIPM_GLUE(d,VRI13)) "," VIPM_STR_C(VIPM_GLUE(d,VRI14)) "," VIPM_STR_C(VIPM_GLUE(d,VRI15)) "}\n\t"

# define _ARM32_VPOP_Q1(VRI)              "vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI)) "}\n\t"
# define _ARM32_VPOP_Q2(VRI0,VRI1) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "}\n\t"
# define _ARM32_VPOP_Q3(VRI0,VRI1,VRI2) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "}\n\t"
# define _ARM32_VPOP_Q4(VRI0,VRI1,VRI2,VRI3) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "}\n\t"
# define _ARM32_VPOP_Q5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "}\n\t"
# define _ARM32_VPOP_Q6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) "}\n\t"
# define _ARM32_VPOP_Q7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) "," VIPM_STR_C(VIPM_GLUE(q,VRI6)) "}\n\t"
# define _ARM32_VPOP_Q8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	"vpop {" VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) "," VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) "," VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) "," VIPM_STR_C(VIPM_GLUE(q,VRI6)) "," VIPM_STR_C(VIPM_GLUE(q,VRI7)) "}\n\t"


/*
 * Load/Store multiple N-element structures
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn) or (Qn,Qn+1)
     */

# define _ARM32_VLD1_D1(W, VRI,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD1_D2(W, VRI0,VRI1,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD1_D3(W, VRI0,VRI1,VRI2,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD1_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM32_VLD1_Q1(W, VRI,PRR) \
	_ARM32_VLD1_D2(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR)

# define _ARM32_VLD1_Q2(W, VRI0,VRI1,PRR) \
	_ARM32_VLD1_D4(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP1(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP1(VRI1), PRR)

# define _ARM32_VLD1_Q3(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VLD1_Q2_AIA(W, VRI0,VRI1,PRR)       \
	_ARM32_VLD1_Q1(W, VRI2,PRR)                \
	_ARM32_SUBI(PRR,PRR,32)

# define _ARM32_VLD1_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VLD1_Q2_AIA(W, VRI0,VRI1,PRR)            \
	_ARM32_VLD1_Q2(W, VRI2,VRI3,PRR)                \
	_ARM32_SUBI(PRR,PRR,32)


# define _ARM32_VST1_D1(W, VRI,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST1_D2(W, VRI0,VRI1,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST1_D3(W, VRI0,VRI1,VRI2,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST1_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM32_VST1_Q1(W, VRI,PRR) \
	_ARM32_VST1_D2(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR)

# define _ARM32_VST1_Q2(W, VRI0,VRI1,PRR) \
	_ARM32_VST1_D4(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP1(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP1(VRI1), PRR)

# define _ARM32_VST1_Q3(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VST1_Q2_AIA(W, VRI0,VRI1,PRR)       \
	_ARM32_VST1_Q1(W, VRI2,PRR)                \
	_ARM32_SUBI(PRR,PRR,32)

# define _ARM32_VST1_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VST1_Q2_AIA(W, VRI0,VRI1,PRR)            \
	_ARM32_VST1_Q2(W, VRI2,VRI3,PRR)                \
	_ARM32_SUBI(PRR,PRR,32)


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM32_VLD1_D1_AIA(W, VRI,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD1_D2_AIA(W, VRI0,VRI1,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD1_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD1_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]!) " \n\t"


# define _ARM32_VLD1_Q1_AIA(W, VRI,PRR) \
	_ARM32_VLD1_D2_AIA(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR)

# define _ARM32_VLD1_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VLD1_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP1(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP1(VRI1), PRR)

# define _ARM32_VLD1_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VLD1_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VLD1_Q1_AIA(W, VRI2,PRR)

# define _ARM32_VLD1_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VLD1_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VLD1_Q2_AIA(W, VRI2,VRI3,PRR)


# define _ARM32_VST1_D1_AIA(W, VRI,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST1_D2_AIA(W, VRI0,VRI1,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST1_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST1_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]!) " \n\t"


# define _ARM32_VST1_Q1_AIA(W, VRI,PRR) \
	_ARM32_VST1_D2_AIA(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR)

# define _ARM32_VST1_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VST1_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP1(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP1(VRI1), PRR)

# define _ARM32_VST1_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VST1_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VST1_Q1_AIA(W, VRI2,PRR)

# define _ARM32_VST1_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VST1_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VST1_Q2_AIA(W, VRI2,VRI3,PRR)


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1)
		 */

# define _ARM32_VLD1_D1_IA(W, VRI,PRR,ORR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD1_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD1_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD1_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"


# define _ARM32_VLD1_Q1_IA(W, VRI,PRR,ORR) \
	_ARM32_VLD1_D2_IA(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR,ORR)

# define _ARM32_VLD1_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	_ARM32_VLD1_D4_IA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP1(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP1(VRI1), PRR,ORR)

# define _ARM32_VLD1_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	_ARM32_VLD1_Q2_AIA(W, VRI0,VRI1,PRR)              \
	_ARM32_VLD1_Q1_IA(W, VRI2,PRR,ORR)                \
	_ARM32_SUBI(PRR,PRR,32)

# define _ARM32_VLD1_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM32_VLD1_Q2_AIA(W, VRI0,VRI1,PRR)                   \
	_ARM32_VLD1_Q2_IA(W, VRI2,VRI3,PRR,ORR)                \
	_ARM32_SUBI(PRR,PRR,32)


# define _ARM32_VST1_D1_IA(W, VRI,PRR,ORR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST1_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST1_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST1_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"


# define _ARM32_VST1_Q1_IA(W, VRI,PRR,ORR) \
	_ARM32_VST1_D2_IA(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR,ORR)

# define _ARM32_VST1_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	_ARM32_VST1_D4_IA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP1(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP1(VRI1), PRR,ORR)

# define _ARM32_VST1_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	_ARM32_VST1_Q2_AIA(W, VRI0,VRI1,PRR)              \
	_ARM32_VST1_Q1_IA(W, VRI2,PRR,ORR)                \
	_ARM32_SUBI(PRR,PRR,32)

# define _ARM32_VST1_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM32_VST1_Q2_AIA(W, VRI0,VRI1,PRR)                   \
	_ARM32_VST1_Q2_IA(W, VRI2,VRI3,PRR,ORR)                \
	_ARM32_SUBI(PRR,PRR,32)

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn,Qn+1)
     */

# define _ARM32_VLD2_D2(W, VRI0,VRI1,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD2_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD2_Q2(W, VRI0,VRI1,PRR) \
	_ARM32_VLD2_D2_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), PRR) \
	_ARM32_VLD2_D2(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), PRR)     \
	_ARM32_SUBI(PRR,PRR,16)

# define _ARM32_VLD2_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VLD2_Q2_AIA(W, VRI0,VRI1,PRR)                             \
	_ARM32_VLD2_D2_AIA(W, _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VLD2_D2(W, _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)     \
	_ARM32_SUBI(PRR,PRR,48)


# define _ARM32_VST2_D2(W, VRI0,VRI1,PRR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST2_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST2_Q2(W, VRI0,VRI1,PRR) \
	_ARM32_VST2_D2_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), PRR) \
	_ARM32_VST2_D2(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), PRR)     \
	_ARM32_SUBI(PRR,PRR,16)

# define _ARM32_VST2_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VST2_Q2_AIA(W, VRI0,VRI1,PRR)                             \
	_ARM32_VST2_D2_AIA(W, _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VST2_D2(W, _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)     \
	_ARM32_SUBI(PRR,PRR,48)


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM32_VLD2_D2_AIA(W, VRI0,VRI1,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD2_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD2_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VLD2_D2_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), PRR) \
	_ARM32_VLD2_D2_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), PRR)


# define _ARM32_VST2_D2_AIA(W, VRI0,VRI1,PRR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST2_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST2_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM32_VST2_D2_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), PRR) \
	_ARM32_VST2_D2_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), PRR)


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1)
		 */

# define _ARM32_VLD2_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD2_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD2_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	_ARM32_VLD2_D2_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), PRR)    \
	_ARM32_VLD2_D2_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,16)

# define _ARM32_VLD2_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM32_VLD2_Q2_AIA(W, VRI0,VRI1,PRR)                                \
	_ARM32_VLD2_D2_AIA(W, _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR)    \
	_ARM32_VLD2_D2_IA(W, _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,48)


# define _ARM32_VST2_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST2_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST2_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	_ARM32_VST2_D2_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), PRR)    \
	_ARM32_VST2_D2_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,16)

# define _ARM32_VST2_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM32_VST2_Q2_AIA(W, VRI0,VRI1,PRR)                                \
	_ARM32_VST2_D2_AIA(W, _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR)    \
	_ARM32_VST2_D2_IA(W, _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,48)

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM32_VLD3_D3(W, VRI0,VRI1,VRI2,PRR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD3_Q3(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VLD3_D3_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR) \
	_ARM32_VLD3_D3(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR)     \
	_ARM32_SUBI(PRR,PRR,24)


# define _ARM32_VST3_D3(W, VRI0,VRI1,VRI2,PRR) \
	"vst3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST3_Q3(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VST3_D3_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR) \
	_ARM32_VST3_D3(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR)     \
	_ARM32_SUBI(PRR,PRR,24)


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM32_VLD3_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD3_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VLD3_D3_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR) \
	_ARM32_VLD3_D3_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR)


# define _ARM32_VST3_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"vst3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST3_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VST3_D3_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR) \
	_ARM32_VST3_D3_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR)


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM32_VLD3_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD3_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	_ARM32_VLD3_D3_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR)    \
	_ARM32_VLD3_D3_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,24)


# define _ARM32_VST3_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"vst3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST3_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	_ARM32_VST3_D3_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR)    \
	_ARM32_VST3_D3_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,24)

/* ------------------------------------------------------------------------- */

	/*
	 * Load 4-element structures from memory (PRR: base address) into vector registers
	 * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
	 */

# define _ARM32_VLD4_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD4_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VLD4_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VLD4_D4(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)     \
	_ARM32_SUBI(PRR,PRR,32)


# define _ARM32_VST4_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST4_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VST4_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VST4_D4(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)     \
	_ARM32_SUBI(PRR,PRR,32)


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM32_VLD4_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD4_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VLD4_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VLD4_D4_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)


# define _ARM32_VST4_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST4_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VST4_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VST4_D4_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM32_VLD4_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD4_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM32_VLD4_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR)    \
	_ARM32_VLD4_D4_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,32)


# define _ARM32_VST4_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vst4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST4_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM32_VST4_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR)    \
	_ARM32_VST4_D4_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR,ORR) \
	_ARM32_SUBI(PRR,PRR,32)

/* ========================================================================= */

/*
 * Load (with duplicate) single N-element structure to all lanes (into all elements of vector registers)
 */

    /*
     * Load 1-element structure from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1)
	 * (VRI0, ...): (Qn)
     */

# define _ARM32_VLD1_DUP_D1(W, VRI,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD1_DUP_D2(W, VRI0,VRI1,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM32_VLD1_DUP_Q1(W, VRI,PRR) \
	_ARM32_VLD1_DUP_D2(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR)


		/*
		 * With auto-increment
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM32_VLD1_DUP_D1_AIA(W, VRI,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD1_DUP_D2_AIA(W, VRI0,VRI1,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "}, " VIPM_STR_C([%PRR]!) " \n\t"


# define _ARM32_VLD1_DUP_Q1_AIA(W, VRI,PRR) \
	_ARM32_VLD1_DUP_D2_AIA(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR)


		/*
		 * With offset update
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1)
		 * (VRI0, ...): (Qn)
		 */

# define _ARM32_VLD1_DUP_D1_IA(W, VRI,PRR,ORR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD1_DUP_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"


# define _ARM32_VLD1_DUP_Q1_IA(W, VRI,PRR,ORR) \
	_ARM32_VLD1_DUP_D2_IA(W, _ARM_QDMAP0(VRI), _ARM_QDMAP1(VRI), PRR,ORR)

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
     */

# define _ARM32_VLD2_DUP_D2(W, VRI0,VRI1,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD2_DUP_Q2(W, VRI0,VRI1,PRR) \
	_ARM32_VLD2_DUP_D2(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1),PRR) \
	_ARM32_VLD2_DUP_D2(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1),PRR)


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM32_VLD2_DUP_D2_AIA(W, VRI0,VRI1,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD2_DUP_Q2_AIA(W, VRI0,VRI1,PRR) \
	_ARM_VLD2_DUP_D4_AIA(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), PRR)


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
		 */

# define _ARM32_VLD2_DUP_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD2_DUP_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	_ARM32_VLD2_DUP_D2(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1),PRR) \
	_ARM32_VLD2_DUP_D2_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1),PRR,ORR)

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM32_VLD3_DUP_D3(W, VRI0,VRI1,VRI2,PRR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD3_DUP_Q3(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VLD3_DUP_D3(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR) \
	_ARM32_VLD3_DUP_D3(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR)


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM32_VLD3_DUP_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD3_DUP_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	_ARM32_VLD3_DUP_D3(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR) \
	_ARM32_VLD3_DUP_D3_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR)


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM32_VLD3_DUP_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD3_DUP_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	_ARM32_VLD3_DUP_D3(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), PRR) \
	_ARM32_VLD3_DUP_D3_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), PRR,ORR)

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
     */

# define _ARM32_VLD4_DUP_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VLD4_DUP_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VLD4_DUP_D4(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VLD4_DUP_D4(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM32_VLD4_DUP_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VLD4_DUP_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM32_VLD4_DUP_D4(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VLD4_DUP_D4_AIA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR)


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM32_VLD4_DUP_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VLD4_DUP_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM32_VLD4_DUP_D4(W, _ARM_QDMAP0(VRI0), _ARM_QDMAP0(VRI1), _ARM_QDMAP0(VRI2), _ARM_QDMAP0(VRI3), PRR) \
	_ARM32_VLD4_DUP_D4_IA(W, _ARM_QDMAP1(VRI0), _ARM_QDMAP1(VRI1), _ARM_QDMAP1(VRI2), _ARM_QDMAP1(VRI3), PRR,ORR)

/* ========================================================================= */

/*
 * Load/Store single N-element structure to/from one lane
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI): (Dn[L])
     */

# define _ARM32_VLD1_LANE_D1(W, L,VRI,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST1_LANE_D1(W, L,VRI,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM32_VLD1_LANE_D1_AIA(W, L,VRI,PRR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST1_LANE_D1_AIA(W, L,VRI,PRR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM32_VLD1_LANE_D1_IA(W, L,VRI,PRR,ORR) \
	"vld1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST1_LANE_D1_IA(W, L,VRI,PRR,ORR) \
	"vst1." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
     */

# define _ARM32_VLD2_LANE_D2(W, L,VRI0,VRI1,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST2_LANE_D2(W, L,VRI0,VRI1,PRR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM32_VLD2_LANE_D2_AIA(W, L,VRI0,VRI1,PRR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST2_LANE_D2_AIA(W, L,VRI0,VRI1,PRR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM32_VLD2_LANE_D2_IA(W, L,VRI0,VRI1,PRR,ORR) \
	"vld2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST2_LANE_D2_IA(W, L,VRI0,VRI1,PRR,ORR) \
	"vst2." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
     */

# define _ARM32_VLD3_LANE_D3(W, L,VRI0,VRI1,VRI2,PRR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST3_LANE_D3(W, L,VRI0,VRI1,VRI2,PRR) \
	"vst3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM32_VLD3_LANE_D3_AIA(W, L,VRI0,VRI1,VRI2,PRR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST3_LANE_D3_AIA(W, L,VRI0,VRI1,VRI2,PRR) \
	"vst3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM32_VLD3_LANE_D3_IA(W, L,VRI0,VRI1,VRI2,PRR,ORR) \
	"vld3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST3_LANE_D3_IA(W, L,VRI0,VRI1,VRI2,PRR,ORR) \
	"vst3." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
     * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
     */

# define _ARM32_VLD4_LANE_D4(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM32_VST4_LANE_D4(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[L]) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM32_VLD4_LANE_D4_AIA(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"

# define _ARM32_VST4_LANE_D4_AIA(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"vst4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[L]) "}, " VIPM_STR_C([%PRR]!) " \n\t"


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM32_VLD4_LANE_D4_IA(W, L,VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vld4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM32_VST4_LANE_D4_IA(W, L,VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"vst4." VIPM_STR_C(W) " {" VIPM_STR_C(VIPM_GLUE(d,VRI0)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI1)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI2)[L]) "," VIPM_STR_C(VIPM_GLUE(d,VRI3)[L]) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define __ARM32_VPRINT_LANE_D1(W,_Fmt,L,VRI) \
	"vmov." VIPM_STR_C(_ARM32_VGET_DATATYPE(W)) " r1, " VIPM_STR_C(VIPM_GLUE(d,VRI)[L]) " \n\t" \
    _ARM32_DPRINTFI(_Fmt)

# define __ARM32_VPRINTF32_LANE_D1(_Fmt,L,VRI) \
	_ARM32_VPUSH_Q1(0)                         \
	_ARM32_VMOV_D(0,VRI)                       \
	_ARM_VF32TOF64_D1S1(1,_ARM32_DSMAP(L,0))   \
	"fmrrd r2, r3, d1              \n\t"       \
    _ARM32_DPRINTFI(_Fmt)                      \
	_ARM32_VPOP_Q1(0)

/* @} */
/* @} */


/* ######################################################################### */

/* @{ */
#ifndef __VIPM_ARM_DEFS__
# define __VIPM_ARM_DEFS__

/*
 * @{
 */

/* @{ */
# define _ARM_WIDTHOF_PTR           _ARM32_WIDTHOF_PTR
# define _ARM_PTR_OFFSET(Offset)    _ARM32_PTR_OFFSET(Offset)
/* @} */


/* ========================================================================= */

/*
 * Move operations
 * @{
 */

# define _ARM_MOVI                  _ARM32_MOVI
# define _ARM_MOVI_UCF              _ARM32_MOVI_UCF
# define _ARM_MOVI_IF               _ARM32_MOVI_IF
# define _ARM_MOVI_UCF_IF           _ARM32_MOVI_UCF_IF

# define _ARM_MOV                   _ARM32_MOV
# define _ARM_MOV_UCF               _ARM32_MOV_UCF
# define _ARM_MOV_IF                _ARM32_MOV_IF
# define _ARM_MOV_UCF_IF            _ARM32_MOV_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 *
 * signed  : (x) --> (-2**(sat-1) <= x <= 2**(sat-1) -1)
 * unsigned: (x) --> (0 <= x <= 2**sat -1)
 * @{
 */

# define _ARM_SATURATE_S            _ARM32_SATURATE_S
# define _ARM_SATURATE_U            _ARM32_SATURATE_U

/* ------------------------------------------------------------------------- */

# define _ARM_RND_FTOSIEX           _ARM32_RND_FTOSIEX
# define _ARM_RND_FTOUIEX           _ARM32_RND_FTOUIEX

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * NEGATE: (a) --> (-a)
 * ADD : (a,b) --> (a + b)
 *		ADD[I]        : add
 *		ADD[I]_UCF    : add updating the condition flags
 *		ADD[I]_IF     : [conditionally] add
 *		ADD[I]_UCF_IF : [conditionally] add updating the condition flags
 * SUB : (a,b) --> (a - b)
 * RSUB: (a,b) --> (b - a)
 * MUL : (a,b) --> (a * b)
 */

/* ------------------------------------------------------------------------- */

    /*
     * NEGATE
	 * @{
     */
# define _ARM_NEGATE                _ARM32_NEGATE
# define _ARM_NEGATE_UCF            _ARM32_NEGATE_UCF
# define _ARM_NEGATE_IF             _ARM32_NEGATE_IF
# define _ARM_NEGATE_UCF_IF         _ARM32_NEGATE_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * ADD
	 * @{
     */
# define _ARM_ADDI                  _ARM32_ADDI
# define _ARM_ADDI_UCF              _ARM32_ADDI_UCF
# define _ARM_ADDI_IF               _ARM32_ADDI_IF
# define _ARM_ADDI_UCF_IF           _ARM32_ADDI_UCF_IF

# define _ARM_ADD                   _ARM32_ADD
# define _ARM_ADD_UCF               _ARM32_ADD_UCF
# define _ARM_ADD_IF                _ARM32_ADD_IF
# define _ARM_ADD_UCF_IF            _ARM32_ADD_UCF_IF
	/* @} */

    /*
     * ADDEX
	 * @{
     */
# define _ARM_ADDEX                 _ARM32_ADDEX
# define _ARM_ADDEX_UCF             _ARM32_ADDEX_UCF
# define _ARM_ADDEX_IF              _ARM32_ADDEX_IF
# define _ARM_ADDEX_UCF_IF          _ARM32_ADDEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * SUB
	 * @{
     */
# define _ARM_SUBI                  _ARM32_SUBI
# define _ARM_SUBI_UCF              _ARM32_SUBI_UCF
# define _ARM_SUBI_IF               _ARM32_SUBI_IF
# define _ARM_SUBI_UCF_IF           _ARM32_SUBI_UCF_IF

# define _ARM_SUB                   _ARM32_SUB
# define _ARM_SUB_UCF               _ARM32_SUB_UCF
# define _ARM_SUB_IF                _ARM32_SUB_IF
# define _ARM_SUB_UCF_IF            _ARM32_SUB_UCF_IF
	/* @} */

    /*
     * SUBEX
	 * @{
     */
# define _ARM_SUBEX                 _ARM32_SUBEX
# define _ARM_SUBEX_UCF             _ARM32_SUBEX_UCF
# define _ARM_SUBEX_IF              _ARM32_SUBEX_IF
# define _ARM_SUBEX_UCF_IF          _ARM32_SUBEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * RSUB
	 * @{
     */
# define _ARM_RSUBI                 _ARM32_RSUBI
# define _ARM_RSUBI_UCF             _ARM32_RSUBI_UCF
# define _ARM_RSUBI_IF              _ARM32_RSUBI_IF
# define _ARM_RSUBI_UCF_IF          _ARM32_RSUBI_UCF_IF

# define _ARM_RSUB                  _ARM32_RSUB
# define _ARM_RSUB_UCF              _ARM32_RSUB_UCF
# define _ARM_RSUB_IF               _ARM32_RSUB_IF
# define _ARM_RSUB_UCF_IF           _ARM32_RSUB_UCF_IF
	/* @} */

    /*
     * RSUBEX
	 * @{
     */
# define _ARM_RSUBEX                _ARM32_RSUBEX
# define _ARM_RSUBEX_UCF            _ARM32_RSUBEX_UCF
# define _ARM_RSUBEX_IF             _ARM32_RSUBEX_IF
# define _ARM_RSUBEX_UCF_IF         _ARM32_RSUBEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * MUL
	 * @{
     */
# define _ARM_MUL                   _ARM32_MUL
# define _ARM_MUL_UCF               _ARM32_MUL_UCF
# define _ARM_MUL_IF                _ARM32_MUL_IF
# define _ARM_MUL_UCF_IF            _ARM32_MUL_UCF_IF
	/* @} */

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

# define _ARM_CMPI                  _ARM32_CMPI
# define _ARM_CMPI_IF               _ARM32_CMPI_IF

# define _ARM_CMP                   _ARM32_CMP
# define _ARM_CMP_IF                _ARM32_CMP_IF

# define _ARM_CMPNEGI               _ARM32_CMPNEGI
# define _ARM_CMPNEGI_IF            _ARM32_CMPNEGI_IF

# define _ARM_CMPNEG                _ARM32_CMPNEG
# define _ARM_CMPNEG_IF             _ARM32_CMPNEG_IF

# define _ARM_UCF                   _ARM32_UCF

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * not                                                 : NOT[I]     : DRR = ~SRR
 * not updating the condition flags                    : NOT[I]_UCF : DRR = ~SRR
 * and                                                 : AND[I]     : DRR = SRR & N
 * and updating the condition flags                    : AND[I]_UCF : DRR = SRR & N
 * and complement (and * not)                          : ANT[I]     : DRR = SRR & ~N
 * and complement (and * not) updating the condition flags : ANT[I]_UCF : DRR = SRR & ~N
 * or                                                  : OR[I]      : DRR = SRR | N
 * or updating the condition flags                     : OR[I]_UCF  : DRR = SRR | N
 * xor                                                 : XOR[I]     : DRR = SRR ^ N
 * xor updating the condition flags                    : XOR[I]_UCF : DRR = SRR ^ N
 * set all bits                                        : BITSET     :
 * clear all bits                                      : BITCLR     :
 */

/* ------------------------------------------------------------------------- */

    /*
     * NOT
     */

# define _ARM_NOTI                  _ARM32_NOTI
# define _ARM_NOTI_UCF              _ARM32_NOTI_UCF
# define _ARM_NOTI_IF               _ARM32_NOTI_IF
# define _ARM_NOTI_UCF_IF           _ARM32_NOTI_UCF_IF

# define _ARM_NOT                   _ARM32_NOT
# define _ARM_NOT_UCF               _ARM32_NOT_UCF
# define _ARM_NOT_IF                _ARM32_NOT_IF
# define _ARM_NOT_UCF_IF            _ARM32_NOT_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * AND
     */

# define _ARM_ANDI                  _ARM32_ANDI
# define _ARM_ANDI_UCF              _ARM32_ANDI_UCF
# define _ARM_ANDI_IF               _ARM32_ANDI_IF
# define _ARM_ANDI_UCF_IF           _ARM32_ANDI_UCF_IF

# define _ARM_AND                   _ARM32_AND
# define _ARM_AND_UCF               _ARM32_AND_UCF
# define _ARM_AND_IF                _ARM32_AND_IF
# define _ARM_AND_UCF_IF            _ARM32_AND_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * BIC: AND * NOT
     */

# define _ARM_BICI                  _ARM32_BICI
# define _ARM_BICI_UCF              _ARM32_BICI_UCF
# define _ARM_BICI_IF               _ARM32_BICI_IF
# define _ARM_BICI_UCF_IF           _ARM32_BICI_UCF_IF

# define _ARM_BIC                   _ARM32_BIC
# define _ARM_BIC_UCF               _ARM32_BIC_UCF
# define _ARM_BIC_IF                _ARM32_BIC_IF
# define _ARM_BIC_UCF_IF            _ARM32_BIC_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * OR
     */

# define _ARM_ORRI                  _ARM32_ORRI
# define _ARM_ORRI_UCF              _ARM32_ORRI_UCF
# define _ARM_ORRI_IF               _ARM32_ORRI_IF
# define _ARM_ORRI_UCF_IF           _ARM32_ORRI_UCF_IF

# define _ARM_ORR                   _ARM32_ORR
# define _ARM_ORR_UCF               _ARM32_ORR_UCF
# define _ARM_ORR_IF                _ARM32_ORR_IF
# define _ARM_ORR_UCF_IF            _ARM32_ORR_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * XOR
     */

# define _ARM_XORI                  _ARM32_XORI
# define _ARM_XORI_UCF              _ARM32_XORI_UCF
# define _ARM_XORI_IF               _ARM32_XORI_IF
# define _ARM_XORI_UCF_IF           _ARM32_XORI_UCF_IF

# define _ARM_XOR                   _ARM32_XOR
# define _ARM_XOR_UCF               _ARM32_XOR_UCF
# define _ARM_XOR_IF                _ARM32_XOR_IF
# define _ARM_XOR_UCF_IF            _ARM32_XOR_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * rshift: DRR = SRR >> N
 * lshift: DRR = SRR << N
 */

# define _ARM_RSHIFTI               _ARM32_RSHIFTI
# define _ARM_RSHIFTI_UCF           _ARM32_RSHIFTI_UCF
# define _ARM_RSHIFT                _ARM32_RSHIFT
# define _ARM_RSHIFT_UCF            _ARM32_RSHIFT_UCF

# define _ARM_RSHIFTI_IF            _ARM32_RSHIFTI_IF
# define _ARM_RSHIFTI_UCF_IF        _ARM32_RSHIFTI_UCF_IF
# define _ARM_RSHIFT_IF             _ARM32_RSHIFT_IF
# define _ARM_RSHIFT_UCF_IF         _ARM32_RSHIFT_UCF_IF


# define _ARM_LSHIFTI               _ARM32_LSHIFTI
# define _ARM_LSHIFTI_UCF           _ARM32_LSHIFTI_UCF
# define _ARM_LSHIFT                _ARM32_LSHIFT
# define _ARM_LSHIFT_UCF            _ARM32_LSHIFT_UCF

# define _ARM_LSHIFTI_IF            _ARM32_LSHIFTI_IF
# define _ARM_LSHIFTI_UCF_IF        _ARM32_LSHIFTI_UCF_IF
# define _ARM_LSHIFT_IF             _ARM32_LSHIFT_IF
# define _ARM_LSHIFT_UCF_IF         _ARM32_LSHIFT_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 */

    /*
     * PUSH/POP
     */
# define _ARM_PUSH                  _ARM32_PUSH
# define _ARM_PUSH1                 _ARM32_PUSH1
# define _ARM_PUSH2                 _ARM32_PUSH2
# define _ARM_PUSH_ARGRR            _ARM32_PUSH_ARGRR
# define _ARM_PUSH_ALLRR            _ARM32_PUSH_ALLRR
# define _ARM_PUSH_ALLRR0           _ARM32_PUSH_ALLRR0
# define _ARM_PUSH_AVARR            _ARM32_PUSH_AVARR
# define _ARM_PUSH_AVARR0           _ARM32_PUSH_AVARR0

# define _ARM_POP                   _ARM32_POP
# define _ARM_POP1                  _ARM32_POP1
# define _ARM_POP2                  _ARM32_POP2
# define _ARM_POP_ARGRR             _ARM32_POP_ARGRR
# define _ARM_POP_ALLRR             _ARM32_POP_ALLRR
# define _ARM_POP_ALLRR0            _ARM32_POP_ALLRR0
# define _ARM_POP_AVARR             _ARM32_POP_AVARR
# define _ARM_POP_AVARR0            _ARM32_POP_AVARR0

/* ------------------------------------------------------------------------- */

    /*
     * Load
     */

# define _ARM_LOAD                  _ARM32_LOAD
# define _ARM_LOADO                 _ARM32_LOADO
# define _ARM_LOADOI                _ARM32_LOADOI

# define _ARM_LOAD_IF               _ARM32_LOAD_IF
# define _ARM_LOADO_IF              _ARM32_LOADO_IF
# define _ARM_LOADOI_IF             _ARM32_LOADOI_IF


/* slow operation (?) */
# define _ARM_LOAD_IB               _ARM32_LOAD_IB
# define _ARM_LOAD_DB               _ARM32_LOAD_DB
# define _ARM_LOAD_IA               _ARM32_LOAD_IA
# define _ARM_LOAD_DA               _ARM32_LOAD_DA
# define _ARM_LOADI_IB              _ARM32_LOADI_IB
# define _ARM_LOADI_DB              _ARM32_LOADI_DB
# define _ARM_LOADI_IA              _ARM32_LOADI_IA
# define _ARM_LOADI_DA              _ARM32_LOADI_DA

/* ------------------------------------------------------------------------- */

    /*
     * Store
     */

# define _ARM_STORE                 _ARM32_STORE
# define _ARM_STOREO                _ARM32_STOREO
# define _ARM_STOREOI               _ARM32_STOREOI

# define _ARM_STORE_IF              _ARM32_STORE_IF
# define _ARM_STOREO_IF             _ARM32_STOREO_IF
# define _ARM_STOREOI_IF            _ARM32_STOREOI_IF


/* slow operation (?) */
# define _ARM_STORE_IB              _ARM32_STORE_IB
# define _ARM_STORE_DB              _ARM32_STORE_DB
# define _ARM_STORE_IA              _ARM32_STORE_IA
# define _ARM_STORE_DA              _ARM32_STORE_DA
# define _ARM_STOREI_IB             _ARM32_STOREI_IB
# define _ARM_STOREI_DB             _ARM32_STOREI_DB
# define _ARM_STOREI_IA             _ARM32_STOREI_IA
# define _ARM_STOREI_DA             _ARM32_STOREI_DA

/* ------------------------------------------------------------------------- */

    /*
     * Prefetch Memory
     */

# define _ARM_PREFETCHI_CACHELINE   _ARM32_PREFETCHI_CACHELINE
# define _ARM_APREFETCHI_CACHELINE  _ARM32_APREFETCHI_CACHELINE
# define _ARM_PREFETCH_CACHELINE    _ARM32_PREFETCH_CACHELINE
# define _ARM_PREFETCHEX_CACHELINE  _ARM32_PREFETCHEX_CACHELINE

/* @} */


/* ========================================================================= */

/*
 * Flow control
 * @{
 */

# define _ARM_GOTO_IMM              _ARM32_GOTO_IMM
# define _ARM_GOTO_IF               _ARM32_GOTO_IF

/* ------------------------------------------------------------------------- */

    /*
     * Switch branch-table (ARM mode)
	 * @note: Imm = N, case <- {[0, N], default}
     */

# define _ARM_SWITCH                _ARM32_SWITCH

/* Usage sample:

ARM:
	_ARM_SWITCH([x], N, Ldefault)

	_ARM_GOTO_IMM(L0)
	_ARM_GOTO_IMM(L1)
	_ARM_GOTO_IMM(L2)
	_ARM_GOTO_IMM(L3)
	...
	_ARM_GOTO_IMM(Ln)

	L0:       stmt0 _ARM_GOTO_IMM(Lend)
	L1:       stmt1 _ARM_GOTO_IMM(Lend)
	L2:       stmt2 _ARM_GOTO_IMM(Lend)
	L3:       stmt3 _ARM_GOTO_IMM(Lend)
	...
	Ln:       stmtN _ARM_GOTO_IMM(Lend)
	Ldefault: stmtD _ARM_GOTO_IMM(Lend)
	Lend:

C:
	switch (x) {
	case 0:  stmt0 break;
	case 1:  stmt1 break;
	case 2:  stmt2 break;
	case 3:  stmt3 break;
	...
	case N:  stmtN break;
	default: stmtD break;
	}
 */

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define _ARM_DPRINTFI              _ARM32_DPRINTFI

# define __ARM_PRINTRR1             __ARM32_PRINTRR1
# define __ARM_PRINTRR2             __ARM32_PRINTRR2
# define __ARM_PRINTRR3             __ARM32_PRINTRR3

/* @} */
/* @} */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*
 * NEON
 * @{
 */

# define _ARM_DSMAP0                _ARM32_DSMAP0
# define _ARM_DSMAP1                _ARM32_DSMAP1


/* ************************************************************************* */

/*
 * Move operations
 * @{
 */

    /*
     * Duplicate/Set immediate value into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = Imm
     */
# define _ARM_VDUPI_Q               _ARM32_VDUPI_Q
# define _ARM_VDUPI_D               _ARM32_VDUPI_D
# define _ARM_VDUPFI_Q              _ARM32_VDUPFI_Q
# define _ARM_VDUPFI_D              _ARM32_VDUPFI_D
# define _ARM_VDUPFI_S              _ARM32_VDUPFI_S

# define _ARM_VMOV2I_Q              _ARM32_VMOV2I_Q
# define _ARM_VMOV2I_D              _ARM32_VMOV2I_D

/* ------------------------------------------------------------------------- */

    /*
     * Duplicate/Set [regular register] scalar into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = $RR
     */
# define _ARM_VDUPR_Q               _ARM32_VDUPR_Q
# define _ARM_VDUPR_D               _ARM32_VDUPR_D


    /*
     * Duplicate/Set [vector] scalar into every element of the destination vector register
	 * (dM[0],..., dM[N]) = dK[i], i <- {0,N}
     */
# define _ARM_VDUP_Q                _ARM32_VDUP_Q
# define _ARM_VDUP_D                _ARM32_VDUP_D


    /*
     * Set [regular register] scalar into one element of the destination vector register
	 * dM[k] = $RR, k <- {0,N}
	 * sM = $RR
	 */
# define _ARM_VSET1_Q               _ARM32_VSET1_Q
# define _ARM_VSET1_D               _ARM32_VSET1_D
# define _ARM_VSET1_S               _ARM32_VSET1_S

/* ------------------------------------------------------------------------- */

    /*
     * Get [regular register] scalar from one element of the source vector register
	 *
	 * $RR = dM[i], i <- {0,N}
	 * $RR1 = dM[0], $RR2 = dM[1]
	 * $RR = sM
     */
# define _ARM_VGET1_D               _ARM32_VGET1_D
# define _ARM_VGET1_D_IF            _ARM32_VGET1_D_IF
# define _ARM_VGET1_S               _ARM32_VGET1_S
# define _ARM_VGET1_S_IF            _ARM32_VGET1_S_IF

# define _ARM_VGET2_D               _ARM32_VGET2_D

/* ------------------------------------------------------------------------- */

    /*
     * Move between vector registers
	 */

/* Q --> Q */
/* D --> D */
/* S --> S */
# define _ARM_VMOV_Q                _ARM32_VMOV_Q
# define _ARM_VMOV_D                _ARM32_VMOV_D
# define _ARM_VMOV_S                _ARM32_VMOV_S

# define _ARM_VMOV                  _ARM32_VMOV

# define _ARM_VMVN_Q                _ARM32_VMVN_Q
# define _ARM_VMVN_D                _ARM32_VMVN_D

/* ------------------------------------------------------------------------- */

    /*
     * Move between ARM regular and SIMD and FP system registers
	 *
	 * VSR - vector system register: FPSCR, FPSID, FPEXC, ...
	 */

# define _ARM_VFPSRTORR             _ARM32_VFPSRTORR
# define _ARM_VFPGETCF              _ARM32_VFPGETCF
# define _ARM_VFPGETCF_IF           _ARM32_VFPGETCF_IF

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 * @{
 */

/* w - source data width
 *
 * widen:   D    --> (Q)   ;; uint8 --> int16, int16 --> int32, int32 --> int64
 * widenf:  S    --> (D)   ;; float16 --> float32, float32 --> float64
 *          D    --> (Q)
 * narrow: (Q)   -->  D    ;; int16 --> uint8, int32 --> int16, int64 --> int32
 * narrowf:(D)   -->  S    ;; float32 --> float16, float64 --> float32
 *         (Q)   -->  D
 */

# define _ARM_VWIDEN_Q1D1           _ARM32_VWIDEN_Q1D1
# define _ARM_VWIDENF_D1S1          _ARM32_VWIDENF_D1S1
# define _ARM_VWIDENF_Q1D1          _ARM32_VWIDENF_Q1D1

# define _ARM_VNARROW_D1Q1          _ARM32_VNARROW_D1Q1
# define _ARM_VNARROWF_S1D1         _ARM32_VNARROWF_S1D1
# define _ARM_VNARROWF_D1Q1         _ARM32_VNARROWF_D1Q1

/* narrow with saturate: Q --> D */
# define _ARM_VSATURATE_UU_D1Q1     _ARM32_VSATURATE_UU_D1Q1
# define _ARM_VSATURATE_SS_D1Q1     _ARM32_VSATURATE_SS_D1Q1
# define _ARM_VSATURATE_SU_D1Q1     _ARM32_VSATURATE_SU_D1Q1


    /*
     * vector convert signed/unsigned integer to floating point
     */
# define _ARM_VSITOFEX_Q            _ARM32_VSITOFEX_Q
# define _ARM_VSITOFEX_D            _ARM32_VSITOFEX_D
# define _ARM_VSITOFEX_S            _ARM32_VSITOFEX_S
# define _ARM_VUITOFEX_Q            _ARM32_VUITOFEX_Q
# define _ARM_VUITOFEX_D            _ARM32_VUITOFEX_D
# define _ARM_VUITOFEX_S            _ARM32_VUITOFEX_S

	/*
     * vector convert signed/unsigned integer to double
	 */
/* S --> D
   D --> Q
   Q --> (Q,Q) */
# define _ARM_VSITOF64_D1S1         _ARM32_VSITOF64_D1S1
# define _ARM_VSITOF64_Q1D1         _ARM32_VSITOF64_Q1D1
# define _ARM_VSITOF64_Q2Q1         _ARM32_VSITOF64_Q2Q1
# define _ARM_VUITOF64_D1S1         _ARM32_VUITOF64_D1S1
# define _ARM_VUITOF64_Q1D1         _ARM32_VUITOF64_Q1D1
# define _ARM_VUITOF64_Q2Q1         _ARM32_VUITOF64_Q2Q1

    /*
     * vector convert floating point to signed/unsigned integer with roundmode
     */
# define _ARM_RND_VFTOSIEX_S        _ARM32_RND_VFTOSIEX_S
# define _ARM_RND_VFTOSIEX_D        _ARM32_RND_VFTOSIEX_D
# define _ARM_RND_VFTOSIEX_Q        _ARM32_RND_VFTOSIEX_Q
# define _ARM_RND_VFTOUIEX_S        _ARM32_RND_VFTOUIEX_S
# define _ARM_RND_VFTOUIEX_D        _ARM32_RND_VFTOUIEX_D
# define _ARM_RND_VFTOUIEX_Q        _ARM32_RND_VFTOUIEX_Q

    /*
     * vector convert double to signed/unsigned integer with roundmode
     */
/* D --> S
   Q --> D
   (Q,Q) --> Q */
# define _ARM_RND_VF64TOSIEX_S1D1   _ARM32_RND_VF64TOSIEX_S1D1
# define _ARM_RND_VF64TOSIEX_D1Q1   _ARM32_RND_VF64TOSIEX_D1Q1
# define _ARM_RND_VF64TOSIEX_Q1Q2   _ARM32_RND_VF64TOSIEX_Q1Q2
# define _ARM_RND_VF64TOUIEX_S1D1   _ARM32_RND_VF64TOUIEX_S1D1
# define _ARM_RND_VF64TOUIEX_D1Q1   _ARM32_RND_VF64TOUIEX_D1Q1
# define _ARM_RND_VF64TOUIEX_Q1Q2   _ARM32_RND_VF64TOUIEX_Q1Q2

    /*
     * vector convert floating point to signed/unsigned [long] integer with roundmode
     */
# define _ARM_RND_VFTOSIEX          _ARM32_RND_VFTOSIEX
# define _ARM_RND_VFTOUIEX          _ARM32_RND_VFTOUIEX

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * :w  - data width
 * :ww - double data width
 * for instance: w = 8, ww = 16
 *               long operation: ww = w + w
 *               wide operation: ww = ww + w
 *
 * vector negate               : r[i]:w   = -a[i]:w
 *
 * vector add                           : r[i]:w   = a[i]:w + b[i]:w
 * vector add saturated                 : r[i]:w   = saturate(a[i]:w + b[i]:w)
 * vector add long                      : r[i]:ww  = a[i]:w + b[i]:w
 * vector add wide                      : r[i]:ww  = a[i]:ww + b[i]:w
 * vector halving add                   : r[i]:w   = (a[i]:w + b[i]:w) >> 1
 * vector rounding halving add          : r[i]:w   = (a[i]:w + b[i]:w + 1) >> 1
 *
 * vector sub                           : r[i]:w   = a[i]:w - b[i]:w
 * vector sub saturated                 : r[i]:w   = saturate(a[i]:w - b[i]:w)
 * vector sub long                      : r[i]:ww  = a[i]:w - b[i]:w
 * vector sub wide                      : r[i]:ww  = a[i]:ww - b[i]:w
 * vector halving sub                   : r[i]:w   = (a[i]:w - b[i]:w) >> 1
 *
 * vector mul                           : r[i]:w   = a[i]:w * b[i]:w
 * vector mul long                      : r[i]:ww  = a[i]:w * b[i]:w
 * vector mul long saturated            : r[i]:ww  = saturate(a[i]:w * b[i]:w)
 * vector mul accumulate                : r[i]:w  += a[i]:w * b[i]:w
 * vector mul accumulate long           : r[i]:ww += a[i]:w * b[i]:w
 * vector mul accumulate saturated long : r[i]:ww = saturate(r[i]:ww + (a[i]:w * b[i]:w))
 * vector mul subtract                  : r[i]:w  -= a[i]:w * b[i]:w
 * vector mul subtract long             : r[i]:ww -= a[i]:w * b[i]:w
 * vector mul subtract saturated long   : r[i]:ww = saturate(r[i]:ww - (a[i]:w * b[i]:w))
 *
 * vector doubling mul saturated with right shift 16/32          : r = saturate((a * b * 2) >>{U} 16{32})
 * vector rounding doubling mul saturated with right shift 16/32 : r = saturate(round((a * b * 2) >>{U} 16{32}))
 */


    /*
     * VNEGATE
     */

/* negate: Q --> Q
   negate: D --> D */
# define _ARM_VNEGATE_Q             _ARM32_VNEGATE_Q
# define _ARM_VNEGATE_D             _ARM32_VNEGATE_D

# define _ARM_VNEGATEF_Q            _ARM32_VNEGATEF_Q
# define _ARM_VNEGATEF_D            _ARM32_VNEGATEF_D

/* negate saturated: Q --> Q
   negate saturated: D --> D */
# define _ARM_SAT_VNEGATE_Q         _ARM32_SAT_VNEGATE_Q
# define _ARM_SAT_VNEGATE_D         _ARM32_SAT_VNEGATE_D

/* ------------------------------------------------------------------------- */

    /*
     * VADD
     */

/* add: (D,D) --> D
   add: (Q,Q) --> Q */
# define _ARM_VADD_Q                _ARM32_VADD_Q
# define _ARM_VADD_D                _ARM32_VADD_D

/* add< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM_VADDF_Q               _ARM32_VADDF_Q
# define _ARM_VADDF_D               _ARM32_VADDF_D
# define _ARM_VADDF_S               _ARM32_VADDF_S

/* add< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VADDF64_Q             _ARM32_VADDF64_Q
# define _ARM_VADDF64_D             _ARM32_VADDF64_D

/* add long: (D,D) --> Q */
# define _ARM_VADD_WIDEN_Q1D1D1     _ARM32_VADD_WIDEN_Q1D1D1

/* add wide: (Q,D) --> Q */
# define _ARM_VADD_WIDEN_Q1Q1D1     _ARM32_VADD_WIDEN_Q1Q1D1


/* add saturated: (Q,Q) --> Q
   add saturated: (D,D) --> D */
# define _ARM_SAT_VADD_Q            _ARM32_SAT_VADD_Q
# define _ARM_SAT_VADD_D            _ARM32_SAT_VADD_D

/* halving add */
# define _ARM_VHADD_Q               _ARM32_VHADD_Q
# define _ARM_VHADD_D               _ARM32_VHADD_D

/* rounding halving add */
# define _ARM_RND_VHADD_Q           _ARM32_RND_VHADD_Q
# define _ARM_RND_VHADD_D           _ARM32_RND_VHADD_D

/* add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_VADDH_NARROW_D1Q1Q1       _ARM32_VADDH_NARROW_D1Q1Q1

/* rounding add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_RND_VADDH_NARROW_D1Q1Q1   _ARM32_RND_VADDH_NARROW_D1Q1Q1

/* ------------------------------------------------------------------------- */

    /*
     * VSUB
     */

/* sub: (D,D) --> D
   sub: (Q,Q) --> Q */
# define _ARM_VSUB_Q                _ARM32_VSUB_Q
# define _ARM_VSUB_D                _ARM32_VSUB_D

/* sub< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM_VSUBF_Q               _ARM32_VSUBF_Q
# define _ARM_VSUBF_D               _ARM32_VSUBF_D
# define _ARM_VSUBF_S               _ARM32_VSUBF_S

/* sub< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VSUBF64_Q             _ARM32_VSUBF64_Q
# define _ARM_VSUBF64_D             _ARM32_VSUBF64_D

/* sub long: (D,D) --> Q */
# define _ARM_VSUB_WIDEN_Q1D1D1     _ARM32_VSUB_WIDEN_Q1D1D1

/* sub wide: (Q,D) --> Q */
# define _ARM_VSUB_WIDEN_Q1Q1D1     _ARM32_VSUB_WIDEN_Q1Q1D1


/* sub saturated: (Q,Q) --> Q
   sub saturated: (D,D) --> D */
# define _ARM_SAT_VSUB_Q            _ARM32_SAT_VSUB_Q
# define _ARM_SAT_VSUB_D            _ARM32_SAT_VSUB_D


/* halving sub */
# define _ARM_VHSUB_Q               _ARM32_VHSUB_Q
# define _ARM_VHSUB_D               _ARM32_VHSUB_D


/* sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_VSUBH_NARROW_D1Q1Q1       _ARM32_VSUBH_NARROW_D1Q1Q1

/* rounding sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_RND_VSUBH_NARROW_D1Q1Q1   _ARM32_RND_VSUBH_NARROW_D1Q1Q1

/* ------------------------------------------------------------------------- */

    /*
     * VMUL
     */

/* mul: (Q,Q) --> Q
   mul: (D,D) --> D */
# define _ARM_VMUL_Q                _ARM32_VMUL_Q
# define _ARM_VMUL_D                _ARM32_VMUL_D

/* mulS: (Q,D[L]) --> Q
   mulS: (D,D[L]) --> D */
# define _ARM_VMULS_Q               _ARM32_VMULS_Q
# define _ARM_VMULS_D               _ARM32_VMULS_D

/* mul< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM_VMULF_Q               _ARM32_VMULF_Q
# define _ARM_VMULF_D               _ARM32_VMULF_D
# define _ARM_VMULF_S               _ARM32_VMULF_S

/* mulS< float >
 * (q,d[L]) --> q
   (d,d[L]) --> d  */
# define _ARM_VMULSF_Q              _ARM32_VMULSF_Q
# define _ARM_VMULSF_D              _ARM32_VMULSF_D

/* mul< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VMULF64_Q             _ARM32_VMULF64_Q
# define _ARM_VMULF64_D             _ARM32_VMULF64_D

/* mul long: (D,D) --> Q
   mulS long: (D,D[L]) --> Q */
# define _ARM_VMUL_WIDEN_Q1D1D1     _ARM32_VMUL_WIDEN_Q1D1D1
# define _ARM_VMULS_WIDEN_Q1D1D1    _ARM32_VMULS_WIDEN_Q1D1D1

/* mul wide: (Q,D) --> Q
   mulS wide: (Q,D[L]) --> Q */
# define _ARM_VMUL_WIDEN_Q1Q1D1     _ARM32_VMUL_WIDEN_Q1Q1D1
# define _ARM_VMULS_WIDEN_Q1Q1D1    _ARM32_VMULS_WIDEN_Q1Q1D1


/* doubling mul saturated long: (D,D) --> Q; domain: {int16, int32} */
# define _ARM_SAT_VMUL2_WIDEN_Q1D1D1    _ARM32_SAT_VMUL2_WIDEN_Q1D1D1


/* doubling mul saturated, storing only high half
   with right shift 16/32 : r = saturate((a * b * 2) >> 16{32}) */
# define _ARM_SAT_VMUL2H_Q          _ARM32_SAT_VMUL2H_Q
# define _ARM_SAT_VMUL2H_D          _ARM32_SAT_VMUL2H_D

/* rounding doubling mul saturated, storing only high half
   with right shift 16/32: r = saturate(round((a * b * 2) >> 16{32})) */
# define _ARM_SAT_RND_VMUL2H_Q      _ARM32_SAT_RND_VMUL2H_Q
# define _ARM_SAT_RND_VMUL2H_D      _ARM32_SAT_RND_VMUL2H_D


/* mul accumulate/subtract: (Q,Q) --> Q
   mul accumulate/subtract: (D,D) --> D
   mulS accumulate/subtract: (Q,D[L]) --> Q
   mulS accumulate/subtract: (D,D[L]) --> D */
# define _ARM_VMUL_ACCUM_Q          _ARM32_VMUL_ACCUM_Q
# define _ARM_VMUL_ACCUM_D          _ARM32_VMUL_ACCUM_D
# define _ARM_VMULS_ACCUM_Q         _ARM32_VMULS_ACCUM_Q
# define _ARM_VMULS_ACCUM_D         _ARM32_VMULS_ACCUM_D
# define _ARM_VMUL_SUBTR_Q          _ARM32_VMUL_SUBTR_Q
# define _ARM_VMUL_SUBTR_D          _ARM32_VMUL_SUBTR_D
# define _ARM_VMULS_SUBTR_Q         _ARM32_VMULS_SUBTR_Q
# define _ARM_VMULS_SUBTR_D         _ARM32_VMULS_SUBTR_D

/* mul accumulate/subtract< float >
   (q,q) --> q
   (d,d) --> d
   (q,d[L]) --> q
   (d,d[L]) --> d */
# define _ARM_VMULF_ACCUM_Q         _ARM32_VMULF_ACCUM_Q
# define _ARM_VMULF_ACCUM_D         _ARM32_VMULF_ACCUM_D
# define _ARM_VMULSF_ACCUM_Q        _ARM32_VMULSF_ACCUM_Q
# define _ARM_VMULSF_ACCUM_D        _ARM32_VMULSF_ACCUM_D
# define _ARM_VMULF_SUBTR_Q         _ARM32_VMULF_SUBTR_Q
# define _ARM_VMULF_SUBTR_D         _ARM32_VMULF_SUBTR_D
# define _ARM_VMULSF_SUBTR_Q        _ARM32_VMULSF_SUBTR_Q
# define _ARM_VMULSF_SUBTR_D        _ARM32_VMULSF_SUBTR_D

/* mul accumulate/subtract< double >
   (q,q) --> q
   (d,d) --> d */
# define _ARM_VMULF64_ACCUM_Q       _ARM32_VMULF64_ACCUM_Q
# define _ARM_VMULF64_ACCUM_D       _ARM32_VMULF64_ACCUM_D
# define _ARM_VMULF64_SUBTR_Q       _ARM32_VMULF64_SUBTR_Q
# define _ARM_VMULF64_SUBTR_D       _ARM32_VMULF64_SUBTR_D


/* mul accumulate/subtract long: (D,D) --> Q
   mulS accumulate/subtract long: (D,D[L]) --> Q */
# define _ARM_VMUL_ACCUM_WIDEN_Q1D1D1   _ARM32_VMUL_ACCUM_WIDEN_Q1D1D1
# define _ARM_VMULS_ACCUM_WIDEN_Q1D1D1  _ARM32_VMULS_ACCUM_WIDEN_Q1D1D1
# define _ARM_VMUL_SUBTR_WIDEN_Q1D1D1   _ARM32_VMUL_SUBTR_WIDEN_Q1D1D1
# define _ARM_VMULS_SUBTR_WIDEN_Q1D1D1  _ARM32_VMULS_SUBTR_WIDEN_Q1D1D1


/* doubling mul accumulate/subtract saturated long: (D,D) --> Q; domain: {int16, int32} */
# define _ARM_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1  _ARM32_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1
# define _ARM_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1  _ARM32_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1


# define _ARM_VFMAEX                _ARM32_VFMAEX
# define _ARM_VFMSEX                _ARM32_VFMSEX

/* ------------------------------------------------------------------------- */

    /*
     * VDIV
     */

/* note: only for F32 */
# define _ARM_VDIVF_Q               _ARM32_VDIVF_Q
# define _ARM_VDIVF_D               _ARM32_VDIVF_D
# define _ARM_VDIVF_S               _ARM32_VDIVF_S

/* note: only for F64 */
# define _ARM_VDIVF64_Q             _ARM32_VDIVF64_Q
# define _ARM_VDIVF64_D             _ARM32_VDIVF64_D

/* @} */


/* ========================================================================= */

/*
 * Min/Max operations
 * @{
 *
 * r[i] = min(s1[i],s2[i])
 * r[i] = max(s1[i],s2[i])
 */

    /*
     * VMIN
     */

# define _ARM_VMIN_D                _ARM32_VMIN_D
# define _ARM_VMIN_Q                _ARM32_VMIN_Q

# define _ARM_VMINF_D               _ARM32_VMINF_D
# define _ARM_VMINF_Q               _ARM32_VMINF_Q

/* ------------------------------------------------------------------------- */

    /*
     * VMAX
     */

# define _ARM_VMAX_D                _ARM32_VMAX_D
# define _ARM_VMAX_Q                _ARM32_VMAX_Q

# define _ARM_VMAXF_D               _ARM32_VMAXF_D
# define _ARM_VMAXF_Q               _ARM32_VMAXF_Q

/* @} */


/* ========================================================================= */

/*
 * Folding operations: add, min, max
 * @{
 *
 * vector pairwise add [long] :
 *     (a,b) --> r : {r[i]} = {(a[2*i] + a[2*i + 1]),... (b[2*j] + b[2*j + 1])}
 *
 * vector pairwise add accumulate long :
 *     {r[i]} = {(r[i] + a[2*i] + a[2*i + 1]),... (r[j] + b[2*j] + b[2*j + 1])}
 *
 * vector add across vector :
 *     (a,b) --> r[0] : r[0] = (a[0] + a[1] + ... + a[n - 1])
 */

    /*
     * PADD, PADD long, PADD accumulate long, VADD, VADD long
     */

/* (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VPADD_Q               _ARM32_VPADD_Q
# define _ARM_VPADD_D               _ARM32_VPADD_D
/* q --> q(result in low half(d))
 * @note: high half can be modified
 */
# define _ARM_VPADD_Q1              _ARM32_VPADD_Q1

/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM_VPADD_WIDEN_Q         _ARM32_VPADD_WIDEN_Q
# define _ARM_VPADD_WIDEN_D         _ARM32_VPADD_WIDEN_D

/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM_VPADD_ACCUM_WIDEN_Q   _ARM32_VPADD_ACCUM_WIDEN_Q
# define _ARM_VPADD_ACCUM_WIDEN_D   _ARM32_VPADD_ACCUM_WIDEN_D

/* (q,q) --> q[0] */
/* (d,d) --> d[0] */
# define _ARM_VVADD_Q               _ARM32_VVADD_Q
# define _ARM_VVADD_D               _ARM32_VVADD_D

/* (q{dn,dn+1}) --> q[0] */
/* (hi-d,lo-d) --> d[0] */
# define _ARM_VVADD_WIDEN_Q         _ARM32_VVADD_WIDEN_Q
# define _ARM_VVADD_WIDEN_D         _ARM32_VVADD_WIDEN_D

/* ------------------------------------------------------------------------- */

    /*
     * PMIN, VMIN
     */

# define _ARM_VPMIN_Q               _ARM32_VPMIN_Q
# define _ARM_VPMIN_D               _ARM32_VPMIN_D
# define _ARM_VPMIN_Q1              _ARM32_VPMIN_Q1

/* pmin< float > */
# define _ARM_VPMINF_Q              _ARM32_VPMINF_Q
# define _ARM_VPMINF_D              _ARM32_VPMINF_D
# define _ARM_VPMINF_Q1             _ARM32_VPMINF_Q1

# define _ARM_VVMIN_Q               _ARM32_VVMIN_Q
# define _ARM_VVMIN_D               _ARM32_VVMIN_D

/* ------------------------------------------------------------------------- */

    /*
     * PMAX, VMAX
     */

# define _ARM_VPMAX_Q               _ARM32_VPMAX_Q
# define _ARM_VPMAX_D               _ARM32_VPMAX_D
# define _ARM_VPMAX_Q1              _ARM32_VPMAX_Q1

/* pmax< float > */
# define _ARM_VPMAXF_Q              _ARM32_VPMAXF_Q
# define _ARM_VPMAXF_D              _ARM32_VPMAXF_D
# define _ARM_VPMAXF_Q1             _ARM32_VPMAXF_Q1

# define _ARM_VVMAX_Q               _ARM32_VVMAX_Q
# define _ARM_VVMAX_D               _ARM32_VVMAX_D

/* ------------------------------------------------------------------------- */

    /*
     * VMINMAX
     */

# define _ARM_VVMINMAX_Q            _ARM32_VVMINMAX_Q
# define _ARM_VVMINMAX_D            _ARM32_VVMINMAX_D

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

    /*
     * Compare 2 vector registers
     * Op: LT, LE, EQ, GE, GT
     */

# define _ARM_VCMPI_Q               _ARM32_VCMPI_Q
# define _ARM_VCMPI_D               _ARM32_VCMPI_D

# define _ARM_VCMP_Q                _ARM32_VCMP_Q
# define _ARM_VCMP_D                _ARM32_VCMP_D


/*
 * float
 */

# define _ARM_VCMPFI_Q              _ARM32_VCMPFI_Q
# define _ARM_VCMPFI_D              _ARM32_VCMPFI_D

# define _ARM_VCMPF_Q               _ARM32_VCMPF_Q
# define _ARM_VCMPF_D               _ARM32_VCMPF_D


/*
 * Float-Point Subsystem
 *
 * 32-bit operations  -  S registers
 * 64-bit operations  -  D registers
 */

# define _ARM_VFPCMP32              _ARM32_VFPCMP32
# define _ARM_VFPCMP64              _ARM32_VFPCMP64
# define _ARM_VFPCMP32_IF           _ARM32_VFPCMP32_IF
# define _ARM_VFPCMP64_IF           _ARM32_VFPCMP64_IF

# define _ARM_VFPCMP32_UCF          _ARM32_VFPCMP32_UCF
# define _ARM_VFPCMP64_UCF          _ARM32_VFPCMP64_UCF
# define _ARM_VFPCMP32_UCF_IF       _ARM32_VFPCMP32_UCF_IF
# define _ARM_VFPCMP64_UCF_IF       _ARM32_VFPCMP64_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * vector not
 * vector and
 * vector and complement: BIC
 * vector or
 * vector or complement: ORN
 * vector xor
 * vector set all bits
 * vector clear all bits
 * vector bit select           : bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
 */

    /*
     * AND
     */

# define _ARM_VAND_Q                _ARM32_VAND_Q
# define _ARM_VAND_D                _ARM32_VAND_D

    /*
     * BIC : AND complement
     */

# define _ARM_VBIC_Q                _ARM32_VBIC_Q
# define _ARM_VBIC_D                _ARM32_VBIC_D

    /*
     * OR
     */

# define _ARM_VORR_Q                _ARM32_VORR_Q
# define _ARM_VORR_D                _ARM32_VORR_D

    /*
     * ORN : OR complement
     */

# define _ARM_VORN_Q                _ARM32_VORN_Q
# define _ARM_VORN_D                _ARM32_VORN_D

    /*
     * XOR
     */

# define _ARM_VXOR_Q                _ARM32_VXOR_Q
# define _ARM_VXOR_D                _ARM32_VXOR_D

    /*
     * Set/Clear all bits
     */

# define _ARM_VBITSET_Q             _ARM32_VBITSET_Q
# define _ARM_VBITSET_D             _ARM32_VBITSET_D
# define _ARM_VBITCLR_Q             _ARM32_VBITCLR_Q
# define _ARM_VBITCLR_D             _ARM32_VBITCLR_D


    /*
     * Bit select: bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
     * Bit set if true:
     * Bit set if false:
     */

# define _ARM_VBITSEL_Q             _ARM32_VBITSEL_Q
# define _ARM_VBITSEL_D             _ARM32_VBITSEL_D

# define _ARM_VBITIFT_Q             _ARM32_VBITIFT_Q
# define _ARM_VBITIFT_D             _ARM32_VBITIFT_D

# define _ARM_VBITIFF_Q             _ARM32_VBITIFF_Q
# define _ARM_VBITIFF_D             _ARM32_VBITIFF_D


    /*
     * Bit count leading 0/1:
     */

# define _ARM_VCOUNTLEADZEROBITS_Q  _ARM32_VCOUNTLEADZEROBITS_Q
# define _ARM_VCOUNTLEADZEROBITS_D  _ARM32_VCOUNTLEADZEROBITS_D

# define _ARM_VCOUNTLEADSETBITS_Q   _ARM32_VCOUNTLEADSETBITS_Q
# define _ARM_VCOUNTLEADSETBITS_D   _ARM32_VCOUNTLEADSETBITS_D

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * vector shift left by immediate
 * vector shift left saturated by immediate
 * vector shift left long by immediate
 *
 * (negative shift-value shift right)
 * vector shift left by signed var
 * vector rounding shift left by signed
 * vector shift left saturated by signed
 * vector rounding shift left saturated by signed
 *
 * vector [rounding] shift right by immediate
 * vector [rounding] shift right narrow by immediate
 * vector saturating [rounding] shift right narrow by immediate
 */

    /*
     * VSHL by immediate
     */

# define _ARM_VSHLI_Q               _ARM32_VSHLI_Q
# define _ARM_VSHLI_D               _ARM32_VSHLI_D


/* shift saturated */
# define _ARM_SAT_VSHLI_Q           _ARM32_SAT_VSHLI_Q
# define _ARM_SAT_VSHLI_D           _ARM32_SAT_VSHLI_D

/* signed shift saturated */
# define _ARM_SAT_VSHLSI_Q          _ARM32_SAT_VSHLSI_Q
# define _ARM_SAT_VSHLSI_D          _ARM32_SAT_VSHLSI_D

/* unsigned shift saturated */
# define _ARM_SAT_VSHLUI_Q          _ARM32_SAT_VSHLUI_Q
# define _ARM_SAT_VSHLUI_D          _ARM32_SAT_VSHLUI_D


/*
 * shift long
 */

# define _ARM_VSHLI_WIDEN_Q1D1      _ARM32_VSHLI_WIDEN_Q1D1

/* signed shift long */
# define _ARM_VSHLSI_WIDEN_Q1D1     _ARM32_VSHLSI_WIDEN_Q1D1

/* unsigned shift long */
# define _ARM_VSHLUI_WIDEN_Q1D1     _ARM32_VSHLUI_WIDEN_Q1D1

/* ------------------------------------------------------------------------- */

    /*
     * VSHL by signed var (negative shift-value shift right)
     */

/* shift left */
# define _ARM_VSHL_Q                _ARM32_VSHL_Q
# define _ARM_VSHL_D                _ARM32_VSHL_D

/* signed shift left */
# define _ARM_VSHLS_Q               _ARM32_VSHLS_Q
# define _ARM_VSHLS_D               _ARM32_VSHLS_D

/* unsigned shift left */
# define _ARM_VSHLU_Q               _ARM32_VSHLU_Q
# define _ARM_VSHLU_D               _ARM32_VSHLU_D


/* shift left saturated */
# define _ARM_SAT_VSHL_Q            _ARM32_SAT_VSHL_Q
# define _ARM_SAT_VSHL_D            _ARM32_SAT_VSHL_D

/* signed shift left saturated */
# define _ARM_SAT_VSHLS_Q           _ARM32_SAT_VSHLS_Q
# define _ARM_SAT_VSHLS_D           _ARM32_SAT_VSHLS_D

/* unsigned shift left saturated */
# define _ARM_SAT_VSHLU_Q           _ARM32_SAT_VSHLU_Q
# define _ARM_SAT_VSHLU_D           _ARM32_SAT_VSHLU_D


/* rounding shift left */
# define _ARM_RND_VSHL_Q            _ARM32_RND_VSHL_Q
# define _ARM_RND_VSHL_D            _ARM32_RND_VSHL_D

/* signed rounding shift left */
# define _ARM_RND_VSHLS_Q           _ARM32_RND_VSHLS_Q
# define _ARM_RND_VSHLS_D           _ARM32_RND_VSHLS_D

/* unsigned rounding shift left */
# define _ARM_RND_VSHLU_Q           _ARM32_RND_VSHLU_Q
# define _ARM_RND_VSHLU_D           _ARM32_RND_VSHLU_D


/* rounding shift left saturated */
# define _ARM_SAT_RND_VSHL_Q        _ARM32_SAT_RND_VSHL_Q
# define _ARM_SAT_RND_VSHL_D        _ARM32_SAT_RND_VSHL_D

/* signed rounding shift left saturated */
# define _ARM_SAT_RND_VSHLS_Q       _ARM32_SAT_RND_VSHLS_Q
# define _ARM_SAT_RND_VSHLS_D       _ARM32_SAT_RND_VSHLS_D

/* unsigned rounding shift left saturated */
# define _ARM_SAT_RND_VSHLU_Q       _ARM32_SAT_RND_VSHLU_Q
# define _ARM_SAT_RND_VSHLU_D       _ARM32_SAT_RND_VSHLU_D

/* ------------------------------------------------------------------------- */

    /*
     * VSHR by immediate
     */

/* shift right */
# define _ARM_VSHRI_Q               _ARM32_VSHRI_Q
# define _ARM_VSHRI_D               _ARM32_VSHRI_D

/* signed shift right */
# define _ARM_VSHRSI_Q              _ARM32_VSHRSI_Q
# define _ARM_VSHRSI_D              _ARM32_VSHRSI_D

/* unsigned shift right */
# define _ARM_VSHRUI_Q              _ARM32_VSHRUI_Q
# define _ARM_VSHRUI_D              _ARM32_VSHRUI_D


/* rounding shift right */
# define _ARM_RND_VSHRI_Q           _ARM32_RND_VSHRI_Q
# define _ARM_RND_VSHRI_D           _ARM32_RND_VSHRI_D

/* signed rounding shift right */
# define _ARM_RND_VSHRSI_Q          _ARM32_RND_VSHRSI_Q
# define _ARM_RND_VSHRSI_D          _ARM32_RND_VSHRSI_D

/* unsigned rounding shift right */
# define _ARM_RND_VSHRUI_Q          _ARM32_RND_VSHRUI_Q
# define _ARM_RND_VSHRUI_D          _ARM32_RND_VSHRUI_D


/* [signed/unsigned] shift right narrow */
/* narrow: Q --> D */
# define _ARM_VSHRI_NARROW_D1Q1     _ARM32_VSHRI_NARROW_D1Q1
//# define _ARM_VSHRSI_NARROW_D1Q1    _ARM32_VSHRSI_NARROW_D1Q1
//# define _ARM_VSHRUI_NARROW_D1Q1    _ARM32_VSHRUI_NARROW_D1Q1


/* [signed/unsigned] rounding shift right narrow */
/* narrow: Q --> D */
# define _ARM_RND_VSHRI_NARROW_D1Q1 _ARM32_RND_VSHRI_NARROW_D1Q1
//# define _ARM_RND_VSHRSI_NARROW_D1Q1    _ARM32_RND_VSHRSI_NARROW_D1Q1
//# define _ARM_RND_VSHRUI_NARROW_D1Q1    _ARM32_RND_VSHRUI_NARROW_D1Q1


/* [signed/unsigned] shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM_SAT_VSHRI_NARROW_D1Q1     _ARM32_SAT_VSHRI_NARROW_D1Q1
# define _ARM_SAT_VSHRSI_NARROW_D1Q1    _ARM32_SAT_VSHRSI_NARROW_D1Q1
# define _ARM_SAT_VSHRUI_NARROW_D1Q1    _ARM32_SAT_VSHRUI_NARROW_D1Q1


/* [signed/unsigned] rounding shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM_SAT_RND_VSHRI_NARROW_D1Q1     _ARM32_SAT_RND_VSHRI_NARROW_D1Q1
# define _ARM_SAT_RND_VSHRSI_NARROW_D1Q1    _ARM32_SAT_RND_VSHRSI_NARROW_D1Q1
# define _ARM_SAT_RND_VSHRUI_NARROW_D1Q1    _ARM32_SAT_RND_VSHRUI_NARROW_D1Q1

/* ------------------------------------------------------------------------- */

    /*
     * VSHR and Accumulate by immediate
     */

/* shift right */
# define _ARM_VSHRADDI_Q            _ARM32_VSHRADDI_Q
# define _ARM_VSHRADDI_D            _ARM32_VSHRADDI_D

/* signed shift right */
# define _ARM_VSHRADDSI_Q           _ARM32_VSHRADDSI_Q
# define _ARM_VSHRADDSI_D           _ARM32_VSHRADDSI_D

/* unsigned shift right */
# define _ARM_VSHRADDUI_Q           _ARM32_VSHRADDUI_Q
# define _ARM_VSHRADDUI_D           _ARM32_VSHRADDUI_D


/* rounding shift right */
# define _ARM_RND_VSHRADDI_Q        _ARM32_RND_VSHRADDI_Q
# define _ARM_RND_VSHRADDI_D        _ARM32_RND_VSHRADDI_D

/* signed rounding shift right */
# define _ARM_RND_VSHRADDSI_Q       _ARM32_RND_VSHRADDSI_Q
# define _ARM_RND_VSHRADDSI_D       _ARM32_RND_VSHRADDSI_D

/* unsigned rounding shift right */
# define _ARM_RND_VSHRADDUI_Q       _ARM32_RND_VSHRADDUI_Q
# define _ARM_RND_VSHRADDUI_D       _ARM32_RND_VSHRADDUI_D

/* @} */


/* ========================================================================= */

/*
 * Generic data processing operations
 * @{
 *
 * vector element shift left by immediate
 * vector element shift right by immediate
 * vector element rotate left by immediate
 * vector element rotate right by immediate
 *
 * interleaves the elements of two vector (zip)
 * de-interleaves the elements of two vector (unzip)
 */

    /*
     * VEXTRACT
     */

# define _ARM_VEXTRACT_Q            _ARM32_VEXTRACT_Q
# define _ARM_VEXTRACT_D            _ARM32_VEXTRACT_D

/* ------------------------------------------------------------------------- */

    /*
     * VZIP/VUZP (zip/unzip)
     */

/*
 * vector elements: {0,1,2,3,4,5,6,7}
 *
 * zip(s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    s1: {a0,b0,a1,b1,a2,b2,a3,b3};
 *    s2: {a4,b4,a5,b5,a6,b6,a7,b7};
 *
 * unzip(s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    s1: {a0,a2,a4,a6,b0,b2,b4,b6};
 *    s2: {a1,a3,a5,a7,b1,b3,b5,b7};
 */
# define _ARM_VZIP_Q                    _ARM32_VZIP_Q
# define _ARM_VZIP_D                    _ARM32_VZIP_D
# define _ARM_VUZP_Q                    _ARM32_VUZP_Q
# define _ARM_VUZP_D                    _ARM32_VUZP_D

/* ------------------------------------------------------------------------- */

    /*
     * VTRANSPOSE2x2
     */
# define _ARM_VTRANSPOSE2x2_Q           _ARM32_VTRANSPOSE2x2_Q
# define _ARM_VTRANSPOSE2x2_D           _ARM32_VTRANSPOSE2x2_D

/* ------------------------------------------------------------------------- */

    /*
     * VSWAP
     */
# define _ARM_VSWAP_Q                   _ARM32_VSWAP_Q
# define _ARM_VSWAP_D                   _ARM32_VSWAP_D

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 *
 * vector load multiple N-element structures into one or more registers (with de-interleaving when N>1)
 * vector store multiple N-element structures from one or more registers (with interleaving when N>1)
 *
 * vector load single N-element structure into one or more registers (with de-interleaving when N>1)
 * (Multiple copies of one n-element structure are loaded into one ore more registers).
 */


/*
 * Push/Pop
 */

	/*
     * Push/Pop fdd registers
     */

# define _ARM_VPUSH_ALLFDD          _ARM32_VPUSH_ALLFDD
# define _ARM_VPUSH_AVAFDD          _ARM32_VPUSH_AVAFDD
# define _ARM_VPUSH_FDD1            _ARM32_VPUSH_FDD1
# define _ARM_VPUSH_FDD2            _ARM32_VPUSH_FDD2
# define _ARM_VPUSH_FDD3            _ARM32_VPUSH_FDD3
# define _ARM_VPUSH_FDD4            _ARM32_VPUSH_FDD4
# define _ARM_VPUSH_FDD5            _ARM32_VPUSH_FDD5
# define _ARM_VPUSH_FDD6            _ARM32_VPUSH_FDD6
# define _ARM_VPUSH_FDD7            _ARM32_VPUSH_FDD7
# define _ARM_VPUSH_FDD8            _ARM32_VPUSH_FDD8
# define _ARM_VPUSH_FDD9            _ARM32_VPUSH_FDD9
# define _ARM_VPUSH_FDD10           _ARM32_VPUSH_FDD10
# define _ARM_VPUSH_FDD11           _ARM32_VPUSH_FDD11
# define _ARM_VPUSH_FDD12           _ARM32_VPUSH_FDD12
# define _ARM_VPUSH_FDD13           _ARM32_VPUSH_FDD13
# define _ARM_VPUSH_FDD14           _ARM32_VPUSH_FDD14
# define _ARM_VPUSH_FDD15           _ARM32_VPUSH_FDD15
# define _ARM_VPUSH_FDD16           _ARM32_VPUSH_FDD16

# define _ARM_VPOP_ALLFDD           _ARM32_VPOP_ALLFDD
# define _ARM_VPOP_AVAFDD           _ARM32_VPOP_AVAFDD
# define _ARM_VPOP_FDD1             _ARM32_VPOP_FDD1
# define _ARM_VPOP_FDD2             _ARM32_VPOP_FDD2
# define _ARM_VPOP_FDD3             _ARM32_VPOP_FDD3
# define _ARM_VPOP_FDD4             _ARM32_VPOP_FDD4
# define _ARM_VPOP_FDD5             _ARM32_VPOP_FDD5
# define _ARM_VPOP_FDD6             _ARM32_VPOP_FDD6
# define _ARM_VPOP_FDD7             _ARM32_VPOP_FDD7
# define _ARM_VPOP_FDD8             _ARM32_VPOP_FDD8
# define _ARM_VPOP_FDD9             _ARM32_VPOP_FDD9
# define _ARM_VPOP_FDD10            _ARM32_VPOP_FDD10
# define _ARM_VPOP_FDD11            _ARM32_VPOP_FDD11
# define _ARM_VPOP_FDD12            _ARM32_VPOP_FDD12
# define _ARM_VPOP_FDD13            _ARM32_VPOP_FDD13
# define _ARM_VPOP_FDD14            _ARM32_VPOP_FDD14
# define _ARM_VPOP_FDD15            _ARM32_VPOP_FDD15
# define _ARM_VPOP_FDD16            _ARM32_VPOP_FDD16

	/*
     * Push/Pop vector registers
     */

# define _ARM_VPUSH_ALL             _ARM32_VPUSH_ALL

# define _ARM_VPUSH_D1              _ARM32_VPUSH_D1
# define _ARM_VPUSH_D2              _ARM32_VPUSH_D2
# define _ARM_VPUSH_D3              _ARM32_VPUSH_D3
# define _ARM_VPUSH_D4              _ARM32_VPUSH_D4
# define _ARM_VPUSH_D5              _ARM32_VPUSH_D5
# define _ARM_VPUSH_D6              _ARM32_VPUSH_D6
# define _ARM_VPUSH_D7              _ARM32_VPUSH_D7
# define _ARM_VPUSH_D8              _ARM32_VPUSH_D8
# define _ARM_VPUSH_D9              _ARM32_VPUSH_D9
# define _ARM_VPUSH_D10             _ARM32_VPUSH_D10
# define _ARM_VPUSH_D11             _ARM32_VPUSH_D11
# define _ARM_VPUSH_D12             _ARM32_VPUSH_D12
# define _ARM_VPUSH_D13             _ARM32_VPUSH_D13
# define _ARM_VPUSH_D14             _ARM32_VPUSH_D14
# define _ARM_VPUSH_D15             _ARM32_VPUSH_D15
# define _ARM_VPUSH_D16             _ARM32_VPUSH_D16

# define _ARM_VPUSH_Q1              _ARM32_VPUSH_Q1
# define _ARM_VPUSH_Q2              _ARM32_VPUSH_Q2
# define _ARM_VPUSH_Q3              _ARM32_VPUSH_Q3
# define _ARM_VPUSH_Q4              _ARM32_VPUSH_Q4
# define _ARM_VPUSH_Q5              _ARM32_VPUSH_Q5
# define _ARM_VPUSH_Q6              _ARM32_VPUSH_Q6
# define _ARM_VPUSH_Q7              _ARM32_VPUSH_Q7
# define _ARM_VPUSH_Q8              _ARM32_VPUSH_Q8

# define _ARM_VPOP_ALL              _ARM32_VPOP_ALL

# define _ARM_VPOP_D1               _ARM32_VPOP_D1
# define _ARM_VPOP_D2               _ARM32_VPOP_D2
# define _ARM_VPOP_D3               _ARM32_VPOP_D3
# define _ARM_VPOP_D4               _ARM32_VPOP_D4
# define _ARM_VPOP_D5               _ARM32_VPOP_D5
# define _ARM_VPOP_D6               _ARM32_VPOP_D6
# define _ARM_VPOP_D7               _ARM32_VPOP_D7
# define _ARM_VPOP_D8               _ARM32_VPOP_D8
# define _ARM_VPOP_D9               _ARM32_VPOP_D9
# define _ARM_VPOP_D10              _ARM32_VPOP_D10
# define _ARM_VPOP_D11              _ARM32_VPOP_D11
# define _ARM_VPOP_D12              _ARM32_VPOP_D12
# define _ARM_VPOP_D13              _ARM32_VPOP_D13
# define _ARM_VPOP_D14              _ARM32_VPOP_D14
# define _ARM_VPOP_D15              _ARM32_VPOP_D15
# define _ARM_VPOP_D16              _ARM32_VPOP_D16

# define _ARM_VPOP_Q1               _ARM32_VPOP_Q1
# define _ARM_VPOP_Q2               _ARM32_VPOP_Q2
# define _ARM_VPOP_Q3               _ARM32_VPOP_Q3
# define _ARM_VPOP_Q4               _ARM32_VPOP_Q4
# define _ARM_VPOP_Q5               _ARM32_VPOP_Q5
# define _ARM_VPOP_Q6               _ARM32_VPOP_Q6
# define _ARM_VPOP_Q7               _ARM32_VPOP_Q7
# define _ARM_VPOP_Q8               _ARM32_VPOP_Q8

/*
 * Load/Store multiple N-element structures
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn) or (Qn,Qn+1)
     */

# define _ARM_VLD1_D1               _ARM32_VLD1_D1
# define _ARM_VLD1_D2               _ARM32_VLD1_D2
# define _ARM_VLD1_D3               _ARM32_VLD1_D3
# define _ARM_VLD1_D4               _ARM32_VLD1_D4

# define _ARM_VLD1_Q1               _ARM32_VLD1_Q1
# define _ARM_VLD1_Q2               _ARM32_VLD1_Q2
# define _ARM_VLD1_Q3               _ARM32_VLD1_Q3
# define _ARM_VLD1_Q4               _ARM32_VLD1_Q4


# define _ARM_VST1_D1               _ARM32_VST1_D1
# define _ARM_VST1_D2               _ARM32_VST1_D2
# define _ARM_VST1_D3               _ARM32_VST1_D3
# define _ARM_VST1_D4               _ARM32_VST1_D4

# define _ARM_VST1_Q1               _ARM32_VST1_Q1
# define _ARM_VST1_Q2               _ARM32_VST1_Q2
# define _ARM_VST1_Q3               _ARM32_VST1_Q3
# define _ARM_VST1_Q4               _ARM32_VST1_Q4


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD1_D1_AIA           _ARM32_VLD1_D1_AIA
# define _ARM_VLD1_D2_AIA           _ARM32_VLD1_D2_AIA
# define _ARM_VLD1_D3_AIA           _ARM32_VLD1_D3_AIA
# define _ARM_VLD1_D4_AIA           _ARM32_VLD1_D4_AIA

# define _ARM_VLD1_Q1_AIA           _ARM32_VLD1_Q1_AIA
# define _ARM_VLD1_Q2_AIA           _ARM32_VLD1_Q2_AIA
# define _ARM_VLD1_Q3_AIA           _ARM32_VLD1_Q3_AIA
# define _ARM_VLD1_Q4_AIA           _ARM32_VLD1_Q4_AIA


# define _ARM_VST1_D1_AIA           _ARM32_VST1_D1_AIA
# define _ARM_VST1_D2_AIA           _ARM32_VST1_D2_AIA
# define _ARM_VST1_D3_AIA           _ARM32_VST1_D3_AIA
# define _ARM_VST1_D4_AIA           _ARM32_VST1_D4_AIA

# define _ARM_VST1_Q1_AIA           _ARM32_VST1_Q1_AIA
# define _ARM_VST1_Q2_AIA           _ARM32_VST1_Q2_AIA
# define _ARM_VST1_Q3_AIA           _ARM32_VST1_Q3_AIA
# define _ARM_VST1_Q4_AIA           _ARM32_VST1_Q4_AIA


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1)
		 */

# define _ARM_VLD1_D1_IA            _ARM32_VLD1_D1_IA
# define _ARM_VLD1_D2_IA            _ARM32_VLD1_D2_IA
# define _ARM_VLD1_D3_IA            _ARM32_VLD1_D3_IA
# define _ARM_VLD1_D4_IA            _ARM32_VLD1_D4_IA

# define _ARM_VLD1_Q1_IA            _ARM32_VLD1_Q1_IA
# define _ARM_VLD1_Q2_IA            _ARM32_VLD1_Q2_IA
# define _ARM_VLD1_Q3_IA            _ARM32_VLD1_Q3_IA
# define _ARM_VLD1_Q4_IA            _ARM32_VLD1_Q4_IA


# define _ARM_VST1_D1_IA            _ARM32_VST1_D1_IA
# define _ARM_VST1_D2_IA            _ARM32_VST1_D2_IA
# define _ARM_VST1_D3_IA            _ARM32_VST1_D3_IA
# define _ARM_VST1_D4_IA            _ARM32_VST1_D4_IA

# define _ARM_VST1_Q1_IA            _ARM32_VST1_Q1_IA
# define _ARM_VST1_Q2_IA            _ARM32_VST1_Q2_IA
# define _ARM_VST1_Q3_IA            _ARM32_VST1_Q3_IA
# define _ARM_VST1_Q4_IA            _ARM32_VST1_Q4_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn,Qn+1)
     */

# define _ARM_VLD2_D2               _ARM32_VLD2_D2
# define _ARM_VLD2_D4               _ARM32_VLD2_D4

# define _ARM_VLD2_Q2               _ARM32_VLD2_Q2
# define _ARM_VLD2_Q4               _ARM32_VLD2_Q4


# define _ARM_VST2_D2               _ARM32_VST2_D2
# define _ARM_VST2_D4               _ARM32_VST2_D4

# define _ARM_VST2_Q2               _ARM32_VST2_Q2
# define _ARM_VST2_Q4               _ARM32_VST2_Q4


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD2_D2_AIA           _ARM32_VLD2_D2_AIA
# define _ARM_VLD2_D4_AIA           _ARM32_VLD2_D4_AIA

# define _ARM_VLD2_Q2_AIA           _ARM32_VLD2_Q2_AIA


# define _ARM_VST2_D2_AIA           _ARM32_VST2_D2_AIA
# define _ARM_VST2_D4_AIA           _ARM32_VST2_D4_AIA

# define _ARM_VST2_Q2_AIA           _ARM32_VST2_Q2_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1)
		 */

# define _ARM_VLD2_D2_IA            _ARM32_VLD2_D2_IA
# define _ARM_VLD2_D4_IA            _ARM32_VLD2_D4_IA

# define _ARM_VLD2_Q2_IA            _ARM32_VLD2_Q2_IA
# define _ARM_VLD2_Q4_IA            _ARM32_VLD2_Q4_IA


# define _ARM_VST2_D2_IA            _ARM32_VST2_D2_IA
# define _ARM_VST2_D4_IA            _ARM32_VST2_D4_IA

# define _ARM_VST2_Q2_IA            _ARM32_VST2_Q2_IA
# define _ARM_VST2_Q4_IA            _ARM32_VST2_Q4_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM_VLD3_D3               _ARM32_VLD3_D3
# define _ARM_VLD3_Q3               _ARM32_VLD3_Q3

# define _ARM_VST3_D3               _ARM32_VST3_D3
# define _ARM_VST3_Q3               _ARM32_VST3_Q3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM_VLD3_D3_AIA           _ARM32_VLD3_D3_AIA
# define _ARM_VLD3_Q3_AIA           _ARM32_VLD3_Q3_AIA

# define _ARM_VST3_D3_AIA           _ARM32_VST3_D3_AIA
# define _ARM_VST3_Q3_AIA           _ARM32_VST3_Q3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM_VLD3_D3_IA            _ARM32_VLD3_D3_IA
# define _ARM_VLD3_D3_IA            _ARM32_VLD3_D3_IA

# define _ARM_VST3_D3_IA            _ARM32_VST3_D3_IA
# define _ARM_VST3_Q3_IA            _ARM32_VST3_Q3_IA


/* ------------------------------------------------------------------------- */

	/*
	 * Load 4-element structures from memory (PRR: base address) into vector registers
	 * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
	 */

# define _ARM_VLD4_D4               _ARM32_VLD4_D4
# define _ARM_VLD4_Q4               _ARM32_VLD4_Q4

# define _ARM_VST4_D4               _ARM32_VST4_D4
# define _ARM_VST4_Q4               _ARM32_VST4_Q4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD4_D4_AIA           _ARM32_VLD4_D4_AIA
# define _ARM_VLD4_Q4_AIA           _ARM32_VLD4_Q4_AIA

# define _ARM_VST4_D4_AIA           _ARM32_VST4_D4_AIA
# define _ARM_VST4_Q4_AIA           _ARM32_VST4_Q4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM_VLD4_D4_IA            _ARM32_VLD4_D4_IA
# define _ARM_VLD4_Q4_IA            _ARM32_VLD4_Q4_IA

# define _ARM_VST4_D4_IA            _ARM32_VST4_D4_IA
# define _ARM_VST4_Q4_IA            _ARM32_VST4_Q4_IA

/* ========================================================================= */

/*
 * Load (with duplicate) single N-element structure to all lanes (into all elements of vector registers)
 */

    /*
     * Load 1-element structure from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1)
	 * (VRI0, ...): (Qn)
     */

# define _ARM_VLD1_DUP_D1           _ARM32_VLD1_DUP_D1
# define _ARM_VLD1_DUP_D2           _ARM32_VLD1_DUP_D2

# define _ARM_VLD1_DUP_Q1           _ARM32_VLD1_DUP_Q1


		/*
		 * With auto-increment
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD1_DUP_D1_AIA       _ARM32_VLD1_DUP_D1_AIA
# define _ARM_VLD1_DUP_D2_AIA       _ARM32_VLD1_DUP_D2_AIA

# define _ARM_VLD1_DUP_Q1_AIA       _ARM32_VLD1_DUP_Q1_AIA


		/*
		 * With offset update
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1)
		 * (VRI0, ...): (Qn)
		 */

# define _ARM_VLD1_DUP_D1_IA        _ARM32_VLD1_DUP_D1_IA
# define _ARM_VLD1_DUP_D2_IA        _ARM32_VLD1_DUP_D2_IA

# define _ARM_VLD1_DUP_Q1_IA        _ARM32_VLD1_DUP_Q1_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
     */

# define _ARM_VLD2_DUP_D2           _ARM32_VLD2_DUP_D2

# define _ARM_VLD2_DUP_Q2           _ARM32_VLD2_DUP_Q2


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD2_DUP_D2_AIA       _ARM32_VLD2_DUP_D2_AIA

# define _ARM_VLD2_DUP_Q2_AIA       _ARM32_VLD2_DUP_Q2_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
		 */

# define _ARM_VLD2_DUP_D2_IA        _ARM32_VLD2_DUP_D2_IA

# define _ARM_VLD2_DUP_Q2_IA        _ARM32_VLD2_DUP_Q2_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM_VLD3_DUP_D3           _ARM32_VLD3_DUP_D3

# define _ARM_VLD3_DUP_Q3           _ARM32_VLD3_DUP_Q3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM_VLD3_DUP_D3_AIA       _ARM32_VLD3_DUP_D3_AIA

# define _ARM_VLD3_DUP_Q3_AIA       _ARM32_VLD3_DUP_Q3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM_VLD3_DUP_D3_IA        _ARM32_VLD3_DUP_D3_IA

# define _ARM_VLD3_DUP_Q3_IA        _ARM32_VLD3_DUP_Q3_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
     */

# define _ARM_VLD4_DUP_D4           _ARM32_VLD4_DUP_D4

# define _ARM_VLD4_DUP_Q4           _ARM32_VLD4_DUP_Q4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD4_DUP_D4_AIA       _ARM32_VLD4_DUP_D4_AIA

# define _ARM_VLD4_DUP_Q4_AIA       _ARM32_VLD4_DUP_Q4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM_VLD4_DUP_D4_IA        _ARM32_VLD4_DUP_D4_IA

# define _ARM_VLD4_DUP_Q4_IA        _ARM32_VLD4_DUP_Q4_IA

/* ========================================================================= */

/*
 * Load/Store single N-element structure to/from one lane
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI): (Dn[L])
     */

# define _ARM_VLD1_LANE_D1          _ARM32_VLD1_LANE_D1

# define _ARM_VST1_LANE_D1          _ARM32_VST1_LANE_D1


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM_VLD1_LANE_D1_AIA      _ARM32_VLD1_LANE_D1_AIA

# define _ARM_VST1_LANE_D1_AIA      _ARM32_VST1_LANE_D1_AIA


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM_VLD1_LANE_D1_IA       _ARM32_VLD1_LANE_D1_IA

# define _ARM_VST1_LANE_D1_IA       _ARM32_VST1_LANE_D1_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
     */

# define _ARM_VLD2_LANE_D2          _ARM32_VLD2_LANE_D2

# define _ARM_VST2_LANE_D2          _ARM32_VST2_LANE_D2


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM_VLD2_LANE_D2_AIA      _ARM32_VLD2_LANE_D2_AIA

# define _ARM_VST2_LANE_D2_AIA      _ARM32_VST2_LANE_D2_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM_VLD2_LANE_D2_IA       _ARM32_VLD2_LANE_D2_IA

# define _ARM_VST2_LANE_D2_IA       _ARM32_VST2_LANE_D2_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
     */

# define _ARM_VLD3_LANE_D3          _ARM32_VLD3_LANE_D3

# define _ARM_VST3_LANE_D3          _ARM32_VST3_LANE_D3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM_VLD3_LANE_D3_AIA      _ARM32_VLD3_LANE_D3_AIA

# define _ARM_VST3_LANE_D3_AIA      _ARM32_VST3_LANE_D3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM_VLD3_LANE_D3_IA       _ARM32_VLD3_LANE_D3_IA

# define _ARM_VST3_LANE_D3_IA       _ARM32_VST3_LANE_D3_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
     * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
     */

# define _ARM_VLD4_LANE_D4          _ARM32_VLD4_LANE_D4

# define _ARM_VST4_LANE_D4          _ARM32_VST4_LANE_D4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM_VLD4_LANE_D4_AIA      _ARM32_VLD4_LANE_D4_AIA

# define _ARM_VST4_LANE_D4_AIA      _ARM32_VST4_LANE_D4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM_VLD4_LANE_D4_IA       _ARM32_VLD4_LANE_D4_IA

# define _ARM_VST4_LANE_D4_IA       _ARM32_VST4_LANE_D4_IA

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define __ARM_VPRINT_LANE_D1       __ARM32_VPRINT_LANE_D1
# define __ARM_VPRINTF32_LANE_D1    __ARM32_VPRINTF32_LANE_D1

/* @} */
/* @} */

#endif /* __VIPM_ARM_DEFS__ */
/* @} */

#endif /* !defined(__VIPM_ARM32_ASM_GCC__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/

