/* -*- Mode: c; tab-width: 4; -*-
 * -*- ex: ts=4 -*-
 */

/*
 * Copyright 2016 Seenaptec Ltd. All rights reserved.
 * SEENAPTEC PROPRIETARY/CONFIDENTIAL.
 *
 *
 * $Id$
 */

/*
 * Asm.gcc.h	(V.Sokur)
 * Vipm/machine/arm64/Asm.gcc.h
 */

/* ------------------------------------------------------------------------- */

#ifndef __VIPM_ARM64_ASM_GCC__
# define __VIPM_ARM64_ASM_GCC__

/******************************************************************************
 * %% BeginSection: includes
 */

/*
# ifdef __ARM_NEON__
#  include <arm_neon.h>
# endif
 */

# ifndef __VIPM_ARM_SDEFS__
#  include <Vipm/machine/arm/Sdefs.h>
# endif


/******************************************************************************
 * %% BeginSection: function definitions
 */

/*
 * @{
 */

/*
 * [gcc inline asm]
 *
 *
 * ARM registers:
 * Name  Alt.name  32-bit  Usage
 * r0    x0        w0      First function argument
 *                         Integer function result
 *                         Scratch register
 * r1    x1        w1      Second function argument
 *                         Scratch register
 * r2    x2        w2      Third function argument
 *                         Scratch register
 * r3    x3        w3      Fourth function argument
 *                         Scratch register
 * r4    x4        w4      Fifth function argument
 *                         Scratch register(?)
 * r5    x5        w5      Sixth function argument
 *                         Scratch register(?)
 * r6    x6        w6      Seventh function argument
 *                         Scratch register(?)
 * r7    x7        w7      Eighth function argument
 *                         Scratch register(?)
 * r8    xr/x8     w8      Register variable
 *                         Indirect function result
 * r9    x9        w9      Register variable
 * r10   x10       w10     Register variable
 * r11   x11       w11     Register variable
 * r12   x12       w12     Register variable
 * r13   x13       w13     Register variable
 * r14   x14       w14     Register variable
 * r15   x15       w15     Register variable
 * r16   ip0/x16   w16     Temporary workspace
 * r17   ip1/x17   w17     Temporary workspace
 * r18   pr/x18    w18     Platform register
 * r19   x19       w19     Register variable
 * r20   x20       w20     Register variable
 * r21   x21       w21     Register variable
 * r22   x22       w22     Register variable
 * r23   x23       w23     Register variable
 * r24   x24       w24     Register variable
 * r25   x25       w25     Register variable
 * r26   x26       w26     Register variable
 * r27   x27       w27     Register variable
 * r28   x28       w28     Register variable
 * r29   fp/x29    w29     Frame register
 * r30   lr/x30    w30     Link register
 * r31   sp/xzr    wzr     Stack pointer
 *                         Zero register
 *
 *
 * ARM64 128-bit vector/scalar registers:
 * AASname  ARM64name  ARM64scalar  Notes
 *   q0        v0          q0
 *   q1        v1          q1
 *   q2        v2          q2
 *   q3        v3          q3
 *   q4        v4          q4
 *   q5        v5          q5
 *   q6        v6          q6
 *   q7        v7          q7
 *   q8        v8          q8       low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q9        v9          q9       low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q10       v10         q10      low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q11       v11         q11      low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q12       v12         q12      low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q13       v13         q13      low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q14       v14         q14      low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q15       v15         q15      low half need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   q16       v16         q16
 *   q17       v17         q17
 *   q18       v18         q18
 *   q19       v19         q19
 *   q20       v20         q20
 *   q21       v21         q21
 *   q22       v22         q22
 *   q23       v23         q23
 *   q24       v24         q24
 *   q25       v25         q25
 *   q26       v26         q26
 *   q27       v27         q27
 *   q28       v28         q28
 *   q29       v29         q29
 *   q30       v30         q30
 *   q31       v31         q31
 *
 *
 * ARM64 64-bit vector/scalar registers:
 * AASname  ARM64name  ARM64scalar 128-bit map  Notes
 *   d0       v0[0]        d0         q0[0]
 *   d2       v1[0]        d1         q1[0]
 *   d4       v2[0]        d2         q2[0]
 *   d6       v3[0]        d3         q3[0]
 *   d8       v4[0]        d4         q4[0]
 *   d10      v5[0]        d5         q5[0]
 *   d12      v6[0]        d6         q6[0]
 *   d14      v7[0]        d7         q7[0]
 *   d16      v8[0]        d8         q8[0]     need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d18      v9[0]        d9         q9[0]     need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d20      v10[0]       d10        q10[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d22      v11[0]       d11        q11[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d24      v12[0]       d12        q12[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d26      v13[0]       d13        q13[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d28      v14[0]       d14        q14[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d30      v15[0]       d15        q15[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   d32      v16[0]       d16        q16[0]
 *   d34      v17[0]       d17        q17[0]
 *   d36      v18[0]       d18        q18[0]
 *   d38      v19[0]       d19        q19[0]
 *   d40      v20[0]       d20        q20[0]
 *   d42      v21[0]       d21        q21[0]
 *   d44      v22[0]       d22        q22[0]
 *   d46      v23[0]       d23        q23[0]
 *   d48      v24[0]       d24        q24[0]
 *   d50      v25[0]       d25        q25[0]
 *   d52      v26[0]       d26        q26[0]
 *   d54      v27[0]       d27        q27[0]
 *   d56      v28[0]       d28        q28[0]
 *   d58      v29[0]       d29        q29[0]
 *   d60      v30[0]       d30        q30[0]
 *   d62      v31[0]       d31        q31[0]
 *
 * @note: odd AAS 64-bit(d1,d3..d63) registers(which correspond to high halves of 128-bit registers)
 *        can be used only in such instructions:
 *        -- VZERO1, VGET1, VSET1;
 *        -- VMOV;
 *        -- all instructions with WIDEN/NARROW;
 *        -- VLD1{_LANE}_D1{_AIA|_IA}, VST1{_LANE}_D1{_AIA|_IA};
 *        -- VPRINT{F32}_LANE_D1.
 *
 *
 * ARM64 32-bit vector/scalar registers:
 * AASname  ARM64name  ARM64scalar  64-bit map  Notes
 *   s0                    s0         d0[0]
 *   s4                    s1         d2[0]
 *   s8                    s2         d4[0]
 *   s12                   s3         d6[0]
 *   s16                   s4         d8[0]
 *   s20                   s5         d10[0]
 *   s24                   s6         d12[0]
 *   s28                   s7         d14[0]
 *   s32                   s8         d16[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s36                   s9         d18[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s40                   s10        d20[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s44                   s11        d22[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s48                   s12        d24[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s52                   s13        d26[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s56                   s14        d28[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s60                   s15        d30[0]    need to be saved @sa: _ARM64_VPUSH/VPOP_AVA
 *   s64                   s16        d32[0]
 *   s68                   s17        d34[0]
 *   s72                   s18        d36[0]
 *   s76                   s19        d38[0]
 *   s80                   s20        d40[0]
 *   s84                   s21        d42[0]
 *   s88                   s22        d44[0]
 *   s92                   s23        d46[0]
 *   s96                   s24        d48[0]
 *   s100                  s25        d50[0]
 *   s104                  s26        d52[0]
 *   s108                  s27        d54[0]
 *   s112                  s28        d56[0]
 *   s116                  s29        d58[0]
 *   s120                  s30        d60[0]
 *   s124                  s31        d62[0]
 *
 * @note: else AAS 32-bit(s1,s2,s3,s5,s6,s7..s125,s126,s127) registers(which correspond to
 *        high 3 quaters of 128-bit registers) can be used only in such instructions:
 *        -- VZERO1, VGET1, VSET1;
 *        -- VMOV.
 *
 *
 * Suffix [A,O]DB - [auto,base offset] decrement before
 * Suffix [A,O]DA - [auto,base offset] decrement after
 * Suffix [A,O]IA - [auto,base offset] increment after
 * Suffix [A,O]IB - [auto,base offset] increment before
 *
 *
 * Vector registers:
 * H - 2 byte register // ISA64
 * S - 4 byte register // ISA64
 * D - 8 byte register
 * Q - 16 byte register
 *
 *
 * UCF    - [update] conditional flag
 *
 * RR     - regular register: 1 (numerated register), [src] (named register)
 * CRR    - counter (regular register)
 * DRR    - destination [data] (regular register)
 * SRR    - source [data] (regular register)
 * PRR    - data pointer (regular register)
 * LHSRR  - [data] regular register
 * RHSRR  - [data] regular register
 * VRI    - vector register index (Q or D)
 * DVRI   - destination [data] vector register index (Q or D)
 * SVRI   - source [data] vector register index (Q or D)
 * LHSVRI - [data] vector register index (Q or D)
 * RHSVRI - [data] vector register index (Q or D)
 * Imm    - immediate value: #K
 */

/* switch target register */
/* @note: all asm sources compile with option -ffixed-r15 */
# define _ARM64_SWTR32                      w15
# define _ARM64_SWTR                        x15

/* ------------------------------------------------------------------------- */

/*
 * @{
 */

# define _ARM64_RREG_SIZE_MOD(W)            VIPM_GLUE(_ARM64_RREG_SIZE_MOD_,W)()
/* @{ */
# define _ARM64_RREG_SIZE_MOD_8()           w
# define _ARM64_RREG_SIZE_MOD_16()          w
# define _ARM64_RREG_SIZE_MOD_32()          w
# define _ARM64_RREG_SIZE_MOD_64()          x

# define _ARM64_RREG_SIZE_MOD_I8()          _ARM64_RREG_SIZE_MOD_8()
# define _ARM64_RREG_SIZE_MOD_I16()         _ARM64_RREG_SIZE_MOD_16()
# define _ARM64_RREG_SIZE_MOD_I32()         _ARM64_RREG_SIZE_MOD_32()
# define _ARM64_RREG_SIZE_MOD_I64()         _ARM64_RREG_SIZE_MOD_64()
# define _ARM64_RREG_SIZE_MOD_S8()          _ARM64_RREG_SIZE_MOD_8()
# define _ARM64_RREG_SIZE_MOD_S16()         _ARM64_RREG_SIZE_MOD_16()
# define _ARM64_RREG_SIZE_MOD_S32()         _ARM64_RREG_SIZE_MOD_32()
# define _ARM64_RREG_SIZE_MOD_S64()         _ARM64_RREG_SIZE_MOD_64()
# define _ARM64_RREG_SIZE_MOD_U8()          _ARM64_RREG_SIZE_MOD_8()
# define _ARM64_RREG_SIZE_MOD_U16()         _ARM64_RREG_SIZE_MOD_16()
# define _ARM64_RREG_SIZE_MOD_U32()         _ARM64_RREG_SIZE_MOD_32()
# define _ARM64_RREG_SIZE_MOD_U64()         _ARM64_RREG_SIZE_MOD_64()
# define _ARM64_RREG_SIZE_MOD_F16()         _ARM64_RREG_SIZE_MOD_16()
# define _ARM64_RREG_SIZE_MOD_F32()         _ARM64_RREG_SIZE_MOD_32()
# define _ARM64_RREG_SIZE_MOD_F64()         _ARM64_RREG_SIZE_MOD_64()
/* @} */


# define _ARM64_VREG_SIZE_MOD(W)            VIPM_GLUE(_ARM64_VREG_SIZE_MOD_,W)()
/* @{ */
# define _ARM64_VREG_SIZE_MOD_8()           b
# define _ARM64_VREG_SIZE_MOD_16()          h
# define _ARM64_VREG_SIZE_MOD_32()          s
# define _ARM64_VREG_SIZE_MOD_64()          d
# define _ARM64_VREG_SIZE_MOD_128()         q

# define _ARM64_VREG_SIZE_MOD_I8()          _ARM64_VREG_SIZE_MOD_8()
# define _ARM64_VREG_SIZE_MOD_I16()         _ARM64_VREG_SIZE_MOD_16()
# define _ARM64_VREG_SIZE_MOD_I32()         _ARM64_VREG_SIZE_MOD_32()
# define _ARM64_VREG_SIZE_MOD_I64()         _ARM64_VREG_SIZE_MOD_64()
# define _ARM64_VREG_SIZE_MOD_I128()        _ARM64_VREG_SIZE_MOD_128()
# define _ARM64_VREG_SIZE_MOD_S8()          _ARM64_VREG_SIZE_MOD_8()
# define _ARM64_VREG_SIZE_MOD_S16()         _ARM64_VREG_SIZE_MOD_16()
# define _ARM64_VREG_SIZE_MOD_S32()         _ARM64_VREG_SIZE_MOD_32()
# define _ARM64_VREG_SIZE_MOD_S64()         _ARM64_VREG_SIZE_MOD_64()
# define _ARM64_VREG_SIZE_MOD_S128()        _ARM64_VREG_SIZE_MOD_128()
# define _ARM64_VREG_SIZE_MOD_U8()          _ARM64_VREG_SIZE_MOD_8()
# define _ARM64_VREG_SIZE_MOD_U16()         _ARM64_VREG_SIZE_MOD_16()
# define _ARM64_VREG_SIZE_MOD_U32()         _ARM64_VREG_SIZE_MOD_32()
# define _ARM64_VREG_SIZE_MOD_U64()         _ARM64_VREG_SIZE_MOD_64()
# define _ARM64_VREG_SIZE_MOD_U128()        _ARM64_VREG_SIZE_MOD_128()
# define _ARM64_VREG_SIZE_MOD_F16()         _ARM64_VREG_SIZE_MOD_16()
# define _ARM64_VREG_SIZE_MOD_F32()         _ARM64_VREG_SIZE_MOD_32()
# define _ARM64_VREG_SIZE_MOD_F64()         _ARM64_VREG_SIZE_MOD_64()
# define _ARM64_VREG_SIZE_MOD_F128()        _ARM64_VREG_SIZE_MOD_128()
/* @} */


# define _ARM64_WIDTHOF_PTR                 64
# define _ARM64_PTR_OFFSET(Offset)          VIPM_GLUE(_ARM64_PTR_OFFSET_,Offset)()
/* @{ */
# define _ARM64_PTR_OFFSET_0()              0
# define _ARM64_PTR_OFFSET_1()              8
# define _ARM64_PTR_OFFSET_2()              16
# define _ARM64_PTR_OFFSET_3()              24
# define _ARM64_PTR_OFFSET_4()              32
# define _ARM64_PTR_OFFSET_5()              40
# define _ARM64_PTR_OFFSET_6()              48
# define _ARM64_PTR_OFFSET_7()              56
# define _ARM64_PTR_OFFSET_8()              64
# define _ARM64_PTR_OFFSET_9()              72
# define _ARM64_PTR_OFFSET_10()             80
# define _ARM64_PTR_OFFSET_11()             88
# define _ARM64_PTR_OFFSET_12()             96
# define _ARM64_PTR_OFFSET_13()             104
# define _ARM64_PTR_OFFSET_14()             112
# define _ARM64_PTR_OFFSET_15()             120
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Move operations
 * @{
 */

# define _ARM64_MOVI(DRR,Imm)               "mov " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_MOVI_UCF(DRR,Imm)           "addS " VIPM_STR_C(%DRR) ", xzr, #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_MOVI_IF(_Cond,DRR,Imm)           \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_MOVI(DRR,Imm)                         \
"11001:"

# define _ARM64_MOVI_UCF_IF(_Cond,DRR,Imm)       \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_MOVI_UCF(DRR,Imm)                     \
"11001:"


# define _ARM64_MOV(DRR,SRR)                "mov " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM64_MOV_UCF(DRR,SRR)            "bicS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", xzr \n\t"

# define _ARM64_MOV_IF(_Cond,DRR,SRR)            \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_MOV(DRR,SRR)                          \
"11001:"

# define _ARM64_MOV_UCF_IF(_Cond,DRR,SRR)        \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_MOV_UCF(DRR,SRR)                      \
"11001:"

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 *
 * signed  : (x) --> (-2**(sat-1) <= x <= 2**(sat-1) -1)
 * unsigned: (x) --> (0 <= x <= 2**sat -1)
 * @{
 */

/* ------------------------------------------------------------------------- */

/* D,S - named variables [gcc] inline assembler */
# define _ARM64_RND_FTOSIEX(Wo,Wi,DRR,S,RndM)   "fcvt" VIPM_STR_C(RndM) "S " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(Wo)DRR) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(Wi)S) " \n\t"
# define _ARM64_RND_FTOUIEX(Wo,Wi,DRR,S,RndM)   "fcvt" VIPM_STR_C(RndM) "U " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(Wo)DRR) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(Wi)S) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * NEGATE: (a) --> (-a)
 * ADD : (a,b) --> (a + b)
 *		ADD[I]        : add
 *		ADD[I]_UCF    : add updating the condition flags
 *		ADD[I]_IF     : [conditionally] add
 *		ADD[I]_UCF_IF : [conditionally] add updating the condition flags
 * SUB : (a,b) --> (a - b)
 * RSUB: (a,b) --> (b - a)
 * MUL : (a,b) --> (a * b)
 */

/* ------------------------------------------------------------------------- */

    /*
     * NEGATE
	 * @{
     */

# define _ARM64_NEGATE(DRR,SRR)       "neg " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM64_NEGATE_UCF(DRR,SRR)   "negS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"

# define _ARM64_NEGATE_IF(_Cond, DRR,SRR)        \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_NEGATE(DRR,SRR)                       \
"11001:"

# define _ARM64_NEGATE_UCF_IF(_Cond, DRR,SRR)    \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_NEGATE_UCF(DRR,SRR)                   \
"11001:"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * ADD
	 * @{
     */

# define _ARM64_ADDI(DRR,SRR,Imm)       "add " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_ADDI_UCF(DRR,SRR,Imm)   "addS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_ADDI_IF(_Cond, DRR,SRR,Imm)      \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ADDI(DRR,SRR,Imm)                     \
"11001:"

# define _ARM64_ADDI_UCF_IF(_Cond, DRR,SRR,Imm)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ADDI_UCF(DRR,SRR,Imm)                 \
"11001:"


# define _ARM64_ADD(DRR,SRR1,SRR2)      "add " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_ADD_UCF(DRR,SRR1,SRR2)  "addS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM64_ADD_IF(_Cond, DRR,SRR1,SRR2)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ADD(DRR,SRR1,SRR2)                    \
"11001:"

# define _ARM64_ADD_UCF_IF(_Cond, DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ADD_UCF(DRR,SRR1,SRR2)                \
"11001:"

	/* @} */


    /*
     * ADDEX
	 * @{
     */

# define _ARM64_ADDEX(DRR,SRR1,SRR2,Ext)        "add " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"
# define _ARM64_ADDEX_UCF(DRR,SRR1,SRR2,Ext)    "addS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM64_ADDEX_IF(_Cond, DRR,SRR1,SRR2,Ext) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)   \
	_ARM64_ADDEX(DRR,SRR1,SRR2,Ext)                \
"11001:"

# define _ARM64_ADDEX_UCF_IF(_Cond, DRR,SRR1,SRR2,Ext) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)       \
	_ARM64_ADDEX_UCF(DRR,SRR1,SRR2,Ext)                \
"11001:"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * SUB
	 * @{
     */

# define _ARM64_SUBI(DRR,SRR,Imm)       "sub " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_SUBI_UCF(DRR,SRR,Imm)   "subS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_SUBI_IF(_Cond, DRR,SRR,Imm)      \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_SUBI(DRR,SRR,Imm)                     \
"11001:"

# define _ARM64_SUBI_UCF_IF(_Cond, DRR,SRR,Imm)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_SUBI_UCF(DRR,SRR,Imm)                 \
"11001:"


# define _ARM64_SUB(DRR,SRR1,SRR2)      "sub " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_SUB_UCF(DRR,SRR1,SRR2)  "subS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM64_SUB_IF(_Cond, DRR,SRR1,SRR2)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_SUB(DRR,SRR1,SRR2)                    \
"11001:"

# define _ARM64_SUB_UCF_IF(_Cond, DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_SUB_UCF(DRR,SRR1,SRR2)                \
"11001:"

	/* @} */


    /*
     * SUBEX
	 * @{
     */

# define _ARM64_SUBEX(DRR,SRR1,SRR2,Ext)        "sub " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"
# define _ARM64_SUBEX_UCF(DRR,SRR1,SRR2,Ext)    "subS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) ", " Ext " \n\t"

# define _ARM64_SUBEX_IF(_Cond, DRR,SRR1,SRR2,Ext) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)   \
	_ARM64_SUBEX(DRR,SRR1,SRR2,Ext)                \
"11001:"

# define _ARM64_SUBEX_UCF_IF(_Cond, DRR,SRR1,SRR2,Ext) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)       \
	_ARM64_SUBEX_UCF(DRR,SRR1,SRR2,Ext)                \
"11001:"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * RSUB
	 * @{
     */

# define _ARM64_RSUBI(DRR,SRR,Imm) \
	_ARM64_SUBI(DRR,SRR,Imm)       \
	_ARM64_NEGATE(DRR,DRR)
# define _ARM64_RSUBI_UCF(DRR,SRR,Imm) \
	_ARM64_SUBI(DRR,SRR,Imm)           \
	_ARM64_NEGATE_UCF(DRR,DRR)

# define _ARM64_RSUBI_IF(_Cond, DRR,SRR,Imm)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_RSUBI(DRR,SRR,Imm)                    \
"11001:"
# define _ARM64_RSUBI_UCF_IF(_Cond, DRR,SRR,Imm) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_RSUBI_UCF(DRR,SRR,Imm)                \
"11001:"


# define _ARM64_RSUB(DRR,SRR1,SRR2)                 _ARM64_SUB(DRR,SRR2,SRR1)
# define _ARM64_RSUB_UCF(DRR,SRR1,SRR2)             _ARM64_SUB_UCF(DRR,SRR2,SRR1)

# define _ARM64_RSUB_IF(_Cond, DRR,SRR1,SRR2)       _ARM64_SUB_IF(DRR,SRR2,SRR1)

# define _ARM64_RSUB_UCF_IF(_Cond, DRR,SRR1,SRR2)   _ARM64_SUB_UCF_IF(DRR,SRR2,SRR1)

	/* @} */


    /*
     * RSUBEX
	 * @{
     */

# define _ARM64_RSUBEX(DRR,SRR1,SRR2,Ext) \
	_ARM64_SUBEX(DRR,SRR1,SRR2,Ext)       \
	_ARM64_NEGATE(DRR,DRR)
# define _ARM64_RSUBEX_UCF(DRR,SRR1,SRR2,Ext) \
	_ARM64_SUBEX(DRR,SRR1,SRR2,Ext)           \
	_ARM64_NEGATE_UCF(DRR,DRR)

# define _ARM64_RSUBEX_IF(_Cond, DRR,SRR1,SRR2,Ext) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)    \
	_ARM64_RSUBEX(DRR,SRR1,SRR2,Ext)                \
"11001:"

# define _ARM64_RSUBEX_UCF_IF(_Cond, DRR,SRR1,SRR2,Ext) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)        \
	_ARM64_RSUBEX_UCF(DRR,SRR1,SRR2,Ext)                \
"11001:"

	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * MUL
	 * @{
     */

# define _ARM64_MUL(DRR,SRR1,SRR2)        "mul " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_MUL_UCF(DRR,SRR1,SRR2) \
	_ARM64_MUL(DRR,SRR1,SRR2)          \
	_ARM64_UCF(DRR)

# define _ARM64_MUL_IF(_Cond, DRR,SRR1,SRR2)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_MUL(DRR,SRR1,SRR2)                    \
"11001:"

# define _ARM64_MUL_UCF_IF(_Cond, DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_MUL_UCF(DRR,SRR1,SRR2)                \
"11001:"

	/* @} */

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

# define _ARM64_CMPI(LHSRR,Imm)             "cmp " VIPM_STR_C(%LHSRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_CMPI_IF(_Cond,LHSRR,Imm)         \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_CMPI(LHSRR,Imm)                       \
"11001:"

# define _ARM64_CMP(LHSRR,RHSRR)            "cmp " VIPM_STR_C(%LHSRR) ", " VIPM_STR_C(%RHSRR) " \n\t"
# define _ARM64_CMP_IF(_Cond,LHSRR,RHSRR)        \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_CMP(LHSRR,RHSRR)                      \
"11001:"


# define _ARM64_CMPNEGI(LHSRR,Imm)          "cmn " VIPM_STR_C(%LHSRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_CMPNEGI_IF(_Cond,LHSRR,Imm)      \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_CMPNEGI(LHSRR,Imm)                    \
"11001:"

# define _ARM64_CMPNEG(LHSRR,RHSRR)         "cmn " VIPM_STR_C(%LHSRR) ", " VIPM_STR_C(%RHSRR) " \n\t"
# define _ARM64_CMPNEG_IF(_Cond,LHSRR,RHSRR)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_CMPNEG(LHSRR,RHSRR)                   \
"11001:"


# define _ARM64_UCF(RR)                     "tst " VIPM_STR_C(%RR) ", " VIPM_STR_C(%RR) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * not                                                       : NOT[I]     : DRR = ~SRR
 * not (updating the condition flags)                        : NOT[I]_UCF : DRR = ~SRR
 * and                                                       : AND[I]     : DRR = SRR & N
 * and (updating the condition flags)                        : AND[I]_UCF : DRR = SRR & N
 * and complement (and * not)                                : BIC[I]     : DRR = SRR & ~N
 * and complement (and * not) (updating the condition flags) : BIC[I]_UCF : DRR = SRR & ~N
 * or                                                        : OR[I]      : DRR = SRR | N
 * or (updating the condition flags)                         : OR[I]_UCF  : DRR = SRR | N
 * xor                                                       : XOR[I]     : DRR = SRR ^ N
 * xor (updating the condition flags)                        : XOR[I]_UCF : DRR = SRR ^ N
 * set all bits                                              : BITSET     :
 * clear all bits                                            : BITCLR     :
 */

/* ------------------------------------------------------------------------- */

    /*
     * NOT
     */

# define _ARM64_NOTI(DRR,Imm)           "movn " VIPM_STR_C(%DRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_NOTI_UCF(DRR,Imm) \
	_ARM64_NOTI(DRR,Imm)          \
	_ARM64_UCF(DRR)
# define _ARM64_NOTI_IF(_Cond,DRR,Imm)           \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_NOTI(DRR,Imm)                         \
"11001:"
# define _ARM64_NOTI_UCF_IF(_Cond,DRR,Imm)       \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_NOTI_UCF(DRR,Imm)                     \
"11001:"

# define _ARM64_NOT(DRR,SRR)            "mvn " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) " \n\t"
# define _ARM64_NOT_UCF(DRR,SRR) \
	_ARM64_NOT(DRR,SRR)          \
	_ARM64_UCF(DRR)
# define _ARM64_NOT_IF(_Cond,DRR,SRR)            \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_NOT(DRR,SRR)                          \
"11001:"
# define _ARM64_NOT_UCF_IF(_Cond,DRR,SRR)        \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_NOT_UCF(DRR,SRR)                      \
"11001:"

/* ------------------------------------------------------------------------- */

    /*
     * AND
     */

# define _ARM64_ANDI(DRR,SRR,Imm)       "and " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_ANDI_UCF(DRR,SRR,Imm)   "andS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_ANDI_IF(_Cond, DRR,SRR,Imm)      \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ANDI(DRR,SRR,Imm)                     \
"11001:"

# define _ARM64_ANDI_UCF_IF(_Cond, DRR,SRR,Imm)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ANDI_UCF(DRR,SRR,Imm)                 \
"11001:"


# define _ARM64_AND(DRR,SRR1,SRR2)      "and " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_AND_UCF(DRR,SRR1,SRR2)  "andS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM64_AND_IF(_Cond, DRR,SRR1,SRR2)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_AND(DRR,SRR1,SRR2)                    \
"11001:"

# define _ARM64_AND_UCF_IF(_Cond, DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_AND_UCF(DRR,SRR1,SRR2)                \
"11001:"

/* ------------------------------------------------------------------------- */

    /*
     * BIC {* bit clear *}: AND * NOT
     */

# define _ARM64_BICI(DRR,SRR,Imm) \
	_ARM64_MOVI(DRR,Imm)          \
	_ARM64_BIC(DRR,SRR,DRR)
# define _ARM64_BICI_UCF(DRR,SRR,Imm) \
	_ARM64_MOVI(DRR,Imm)              \
	_ARM64_BIC_UCF(DRR,SRR,DRR)

# define _ARM64_BICI_IF(_Cond, DRR,SRR,Imm)      \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_BICI(DRR,SRR,Imm)                     \
"11001:"

# define _ARM64_BICI_UCF_IF(_Cond, DRR,SRR,Imm)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_BICI_UCF(DRR,SRR,Imm)                 \
"11001:"


# define _ARM64_BIC(DRR,SRR1,SRR2)      "bic " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_BIC_UCF(DRR,SRR1,SRR2)  "bicS " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"

# define _ARM64_BIC_IF(_Cond, DRR,SRR1,SRR2)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_BIC(DRR,SRR1,SRR2)                    \
"11001:"

# define _ARM64_BIC_UCF_IF(_Cond, DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_BIC_UCF(DRR,SRR1,SRR2)                \
"11001:"

/* ------------------------------------------------------------------------- */

    /*
     * OR
     */

# define _ARM64_ORRI(DRR,SRR,Imm)       "orr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_ORRI_UCF(DRR,SRR,Imm) \
	_ARM64_ORRI(DRR,SRR,Imm)          \
	_ARM64_UCF(DRR)

# define _ARM64_ORRI_IF(_Cond, DRR,SRR,Imm)      \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ORRI(DRR,SRR,Imm)                     \
"11001:"

# define _ARM64_ORRI_UCF_IF(_Cond, DRR,SRR,Imm)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ORRI_UCF(DRR,SRR,Imm)                 \
"11001:"


# define _ARM64_ORR(DRR,SRR1,SRR2)      "orr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_ORR_UCF(DRR,SRR1,SRR2) \
	_ARM64_ORR(DRR,SRR1,SRR2)          \
	_ARM64_UCF(DRR)

# define _ARM64_ORR_IF(_Cond, DRR,SRR1,SRR2)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ORR(DRR,SRR1,SRR2)                    \
"11001:"

# define _ARM64_ORR_UCF_IF(_Cond, DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_ORR_UCF(DRR,SRR1,SRR2)                \
"11001:"

/* ------------------------------------------------------------------------- */

    /*
     * XOR
     */

# define _ARM64_XORI(DRR,SRR,Imm)       "eor " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_XORI_UCF(DRR,SRR,Imm) \
	_ARM64_XORI(DRR,SRR,Imm)          \
	_ARM64_UCF(DRR)

# define _ARM64_XORI_IF(_Cond, DRR,SRR,Imm)      \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_XORI(DRR,SRR,Imm)                     \
"11001:"

# define _ARM64_XORI_UCF_IF(_Cond, DRR,SRR,Imm)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_XORI_UCF(DRR,SRR,Imm)                 \
"11001:"


# define _ARM64_XOR(DRR,SRR1,SRR2)      "eor " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_XOR_UCF(DRR,SRR1,SRR2) \
	_ARM64_XOR(DRR,SRR1,SRR2)          \
	_ARM64_UCF(DRR)

# define _ARM64_XOR_IF(_Cond, DRR,SRR1,SRR2)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_XOR(DRR,SRR1,SRR2)                    \
"11001:"

# define _ARM64_XOR_UCF_IF(_Cond, DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_XOR_UCF(DRR,SRR1,SRR2)                \
"11001:"

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * rshift: DRR = SRR >> N
 * lshift: DRR = SRR << N
 */

# define _ARM64_RSHIFTI(DRR,SRR,Imm)      "lsr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_RSHIFTI_UCF(DRR,SRR,Imm) \
	_ARM64_RSHIFTI(DRR,SRR,Imm)          \
	_ARM64_UCF(DRR)
# define _ARM64_RSHIFTI_IF(_Cond,DRR,SRR,Imm)    \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_RSHIFTI(DRR,SRR,Imm)                  \
"11001:"
# define _ARM64_RSHIFTI_UCF_IF(_Cond,DRR,SRR,Imm) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)  \
	_ARM64_RSHIFTI_UCF(DRR,SRR,Imm)               \
"11001:"

# define _ARM64_RSHIFT(DRR,SRR1,SRR2)     "lsr " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_RSHIFT_UCF(DRR,SRR1,SRR2) \
	_ARM64_RSHIFT(DRR,SRR1,SRR2)          \
	_ARM64_UCF(DRR)
# define _ARM64_RSHIFT_IF(_Cond,DRR,SRR1,SRR2)   \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_RSHIFT(DRR,SRR1,SRR2)                 \
"11001:"
# define _ARM64_RSHIFT_UCF_IF(_Cond,DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)   \
	_ARM64_RSHIFT_UCF(DRR,SRR1,SRR2)               \
"11001:"


# define _ARM64_LSHIFTI(DRR,SRR,Imm)      "lsl " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_LSHIFTI_UCF(DRR,SRR,Imm) \
	_ARM64_LSHIFTI(DRR,SRR,Imm)          \
	_ARM64_UCF(DRR)
# define _ARM64_LSHIFTI_IF(_Cond,DRR,SRR,Imm)    \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_LSHIFTI(DRR,SRR,Imm)                  \
"11001:"
# define _ARM64_LSHIFTI_UCF_IF(_Cond,DRR,SRR,Imm) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)  \
	_ARM64_LSHIFTI_UCF(DRR,SRR,Imm)               \
"11001:"

# define _ARM64_LSHIFT(DRR,SRR1,SRR2)     "lsl " VIPM_STR_C(%DRR) ", " VIPM_STR_C(%SRR1) ", " VIPM_STR_C(%SRR2) " \n\t"
# define _ARM64_LSHIFT_UCF(DRR,SRR1,SRR2) \
	_ARM64_LSHIFT(DRR,SRR1,SRR2)          \
	_ARM64_UCF(DRR)
# define _ARM64_LSHIFT_IF(_Cond,DRR,SRR1,SRR2)   \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_LSHIFT(DRR,SRR1,SRR2)                 \
"11001:"
# define _ARM64_LSHIFT_UCF_IF(_Cond,DRR,SRR1,SRR2) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f)   \
	_ARM64_LSHIFT_UCF(DRR,SRR1,SRR2)               \
"11001:"

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 */

    /*
     * PUSH/POP
     */
# define _ARM64_PUSH1(RR)               "str " VIPM_STR_C(%RR) ", [sp, -16]! \n\t"
# define _ARM64_PUSH2(RR1,RR2)          "stp " VIPM_STR_C(%RR1) ", " VIPM_STR_C(%RR2) ", [sp, -16]! \n\t"
# define _ARM64_PUSH_ARGRR()       \
	"stp  x0, x1, [sp, -64]! \n\t" \
	"stp  x2, x3, [sp,  16] \n\t"  \
	"stp  x4, x5, [sp,  32] \n\t"  \
	"stp  x6, x7, [sp,  48] \n\t"
# define _ARM64_PUSH_ALLRR()        \
	"stp  x0, x1, [sp, -256]! \n\t" \
	"stp  x2, x3, [sp,  16] \n\t"   \
	"stp  x4, x5, [sp,  32] \n\t"   \
	"stp  x6, x7, [sp,  48] \n\t"   \
	"stp  x8, x9, [sp,  64] \n\t"   \
	"stp x10,x11, [sp,  80] \n\t"   \
	"stp x12,x13, [sp,  96] \n\t"   \
	"stp x14,x15, [sp, 112] \n\t"   \
	"stp x16,x17, [sp, 128] \n\t"   \
	"stp x18,x19, [sp, 144] \n\t"   \
	"stp x20,x21, [sp, 160] \n\t"   \
	"stp x22,x23, [sp, 176] \n\t"   \
	"stp x24,x25, [sp, 192] \n\t"   \
	"stp x26,x27, [sp, 208] \n\t"   \
	"stp x28,x29, [sp, 224] \n\t"   \
	"str x30, [sp, 240] \n\t"
# define _ARM64_PUSH_ALLRR0()       \
	"stp  x1, x2, [sp, -240]! \n\t" \
	"stp  x3, x4, [sp,  16] \n\t"   \
	"stp  x5, x6, [sp,  32] \n\t"   \
	"stp  x7, x8, [sp,  48] \n\t"   \
	"stp  x9,x10, [sp,  64] \n\t"   \
	"stp x11,x12, [sp,  80] \n\t"   \
	"stp x13,x14, [sp,  96] \n\t"   \
	"stp x15,x16, [sp, 112] \n\t"   \
	"stp x17,x18, [sp, 128] \n\t"   \
	"stp x19,x20, [sp, 144] \n\t"   \
	"stp x21,x22, [sp, 160] \n\t"   \
	"stp x23,x24, [sp, 176] \n\t"   \
	"stp x25,x26, [sp, 192] \n\t"   \
	"stp x27,x28, [sp, 208] \n\t"   \
	"stp x29,x30, [sp, 224] \n\t"
# define _ARM64_PUSH_AVARR()        \
	"stp  x0, x1, [sp, -144]! \n\t" \
	"stp  x2, x3, [sp,  16] \n\t"   \
	"stp  x4, x5, [sp,  32] \n\t"   \
	"stp  x6, x7, [sp,  48] \n\t"   \
	"stp x19,x20, [sp,  64] \n\t"   \
	"stp x21,x22, [sp,  80] \n\t"   \
	"stp x23,x24, [sp,  96] \n\t"   \
	"stp x25,x26, [sp, 112] \n\t"   \
	"stp x27,x28, [sp, 128] \n\t"
# define _ARM64_PUSH_AVARR0()       \
	"stp  x1, x2, [sp, -144]! \n\t" \
	"stp  x3, x4, [sp,  16] \n\t"   \
	"stp  x5, x6, [sp,  32] \n\t"   \
	"stp  x7, x8, [sp,  48] \n\t"   \
	"stp x20,x21, [sp,  64] \n\t"   \
	"stp x22,x23, [sp,  80] \n\t"   \
	"stp x24,x25, [sp,  96] \n\t"   \
	"stp x26,x27, [sp, 112] \n\t"   \
	"str x28, [sp, 128] \n\t"
# define _ARM64_POP1(RR)                "ldr " VIPM_STR_C(%RR) ", [sp], 16 \n\t"
# define _ARM64_POP2(RR1,RR2)           "ldp " VIPM_STR_C(%RR1) ", " VIPM_STR_C(%RR2) ", [sp], 16 \n\t"
# define _ARM64_POP_ARGRR()       \
	"ldp  x2, x3, [sp,  16] \n\t" \
	"ldp  x4, x5, [sp,  32] \n\t" \
	"ldp  x6, x7, [sp,  48] \n\t" \
	"ldp  x0, x1, [sp], 64 \n\t"
# define _ARM64_POP_ALLRR()       \
	"ldp  x2, x3, [sp,  16] \n\t" \
	"ldp  x4, x5, [sp,  32] \n\t" \
	"ldp  x6, x7, [sp,  48] \n\t" \
	"ldp  x8, x9, [sp,  64] \n\t" \
	"ldp x10,x11, [sp,  80] \n\t" \
	"ldp x12,x13, [sp,  96] \n\t" \
	"ldp x14,x15, [sp, 112] \n\t" \
	"ldp x16,x17, [sp, 128] \n\t" \
	"ldp x18,x19, [sp, 144] \n\t" \
	"ldp x20,x21, [sp, 160] \n\t" \
	"ldp x22,x23, [sp, 176] \n\t" \
	"ldp x24,x25, [sp, 192] \n\t" \
	"ldp x26,x27, [sp, 208] \n\t" \
	"ldp x28,x29, [sp, 224] \n\t" \
	"ldr x30, [sp, 240] \n\t"     \
	"ldp  x0, x1, [sp], 256 \n\t"
# define _ARM64_POP_ALLRR0()      \
	"ldp  x3, x4, [sp,  16] \n\t" \
	"ldp  x5, x6, [sp,  32] \n\t" \
	"ldp  x7, x8, [sp,  48] \n\t" \
	"ldp  x9,x10, [sp,  64] \n\t" \
	"ldp x11,x12, [sp,  80] \n\t" \
	"ldp x13,x14, [sp,  96] \n\t" \
	"ldp x15,x16, [sp, 112] \n\t" \
	"ldp x17,x18, [sp, 128] \n\t" \
	"ldp x19,x20, [sp, 144] \n\t" \
	"ldp x21,x22, [sp, 160] \n\t" \
	"ldp x23,x24, [sp, 176] \n\t" \
	"ldp x25,x26, [sp, 192] \n\t" \
	"ldp x27,x28, [sp, 208] \n\t" \
	"ldp x29,x30, [sp, 224] \n\t" \
	"ldp  x1, x2, [sp], 240 \n\t"
# define _ARM64_POP_AVARR()       \
	"ldp  x2, x3, [sp,  16] \n\t" \
	"ldp  x4, x5, [sp,  32] \n\t" \
	"ldp  x6, x7, [sp,  48] \n\t" \
	"ldp x19,x20, [sp,  64] \n\t" \
	"ldp x21,x22, [sp,  80] \n\t" \
	"ldp x23,x24, [sp,  96] \n\t" \
	"ldp x25,x26, [sp, 112] \n\t" \
	"ldp x27,x28, [sp, 128] \n\t" \
	"ldp  x0, x1, [sp], 144 \n\t"
# define _ARM64_POP_AVARR0()      \
	"ldp  x3, x4, [sp,  16] \n\t" \
	"ldp  x5, x6, [sp,  32] \n\t" \
	"ldp  x7, x8, [sp,  48] \n\t" \
	"ldp x20,x21, [sp,  64] \n\t" \
	"ldp x22,x23, [sp,  80] \n\t" \
	"ldp x24,x25, [sp,  96] \n\t" \
	"ldp x26,x27, [sp, 112] \n\t" \
	"ldr x28, [sp, 128] \n\t"     \
	"ldp  x1, x2, [sp], 144 \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load
     */

# define _ARM64_LOAD(W, DRR,PRR)        VIPM_STR_C(_ARM64_LDR_CMD(W)) " " VIPM_STR_C(%_ARM64_LDR_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) "] \n\t"
# define _ARM64_LOADO(W, DRR,PRR,ORR)   VIPM_STR_C(_ARM64_LDR_CMD(W)) " " VIPM_STR_C(%_ARM64_LDR_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "] \n\t"
# define _ARM64_LOADOI(W, DRR,PRR,Off)  VIPM_STR_C(_ARM64_LDR_CMD(W)) " " VIPM_STR_C(%_ARM64_LDR_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "] \n\t"

# define _ARM64_LOAD_IF(W,_Cond, DRR,PRR)        \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_LOAD(W, DRR,PRR)                      \
"11001:"
# define _ARM64_LOADO_IF(W,_Cond, DRR,PRR,ORR)   \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_LOADO(W, DRR,PRR,ORR)                 \
"11001:"
# define _ARM64_LOADOI_IF(W,_Cond, DRR,PRR,Off)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_LOADOI(W, DRR,PRR,Off)                \
"11001:"


/* slow operation (?) */
# define _ARM64_LOAD_IB(W, DRR,PRR,ORR) \
	_ARM64_ADD(W, PRR,PRR,ORR)          \
	_ARM64_LOAD(W, DRR,PRR)
# define _ARM64_LOAD_DB(W, DRR,PRR,ORR) \
	_ARM64_SUB(W, PRR,PRR,ORR)          \
	_ARM64_LOAD(W, DRR,PRR)
# define _ARM64_LOAD_IA(W, DRR,PRR,ORR) \
	_ARM64_LOAD(W, DRR,PRR)             \
	_ARM64_ADD(W, PRR,PRR,ORR)
# define _ARM64_LOAD_DA(W, DRR,PRR,ORR) \
	_ARM64_LOAD(W, DRR,PRR)             \
	_ARM64_SUB(W, PRR,PRR,ORR)
# define _ARM64_LOADI_IB(W, DRR,PRR,Off)    VIPM_STR_C(_ARM64_LDR_CMD(W)) " " VIPM_STR_C(%_ARM64_LDR_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "]! \n\t"
# define _ARM64_LOADI_DB(W, DRR,PRR,Off)    VIPM_STR_C(_ARM64_LDR_CMD(W)) " " VIPM_STR_C(%_ARM64_LDR_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", #-" VIPM_STR_C(Off) "]! \n\t"
# define _ARM64_LOADI_IA(W, DRR,PRR,Off)    VIPM_STR_C(_ARM64_LDR_CMD(W)) " " VIPM_STR_C(%_ARM64_LDR_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) "], #" VIPM_STR_C(Off) " \n\t"
# define _ARM64_LOADI_DA(W, DRR,PRR,Off)    VIPM_STR_C(_ARM64_LDR_CMD(W)) " " VIPM_STR_C(%_ARM64_LDR_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) "], #-" VIPM_STR_C(Off) " \n\t"


/* @{ */
# define _ARM64_LDR_CMD(W)          VIPM_GLUE(_ARM64_LDR_CMD_,W)()
# define _ARM64_LDR_CMD_8()         ldrb
# define _ARM64_LDR_CMD_U8()        ldrb
# define _ARM64_LDR_CMD_S8()        ldrsb
# define _ARM64_LDR_CMD_I8()        ldrsb
# define _ARM64_LDR_CMD_16()        ldrsh
# define _ARM64_LDR_CMD_I16()       ldrsh
# define _ARM64_LDR_CMD_S16()       ldrsh
# define _ARM64_LDR_CMD_U16()       ldrh
# define _ARM64_LDR_CMD_32()        ldrsw
# define _ARM64_LDR_CMD_I32()       ldrsw
# define _ARM64_LDR_CMD_S32()       ldrsw
# define _ARM64_LDR_CMD_U32()       ldr
# define _ARM64_LDR_CMD_64()        ldr
# define _ARM64_LDR_CMD_I64()       ldr
# define _ARM64_LDR_CMD_S64()       ldr
# define _ARM64_LDR_CMD_U64()       ldr

# define _ARM64_LDR_RREG_SIZE_MOD(W)    VIPM_GLUE(_ARM64_LDR_RREG_SIZE_MOD_,W)()
# define _ARM64_LDR_RREG_SIZE_MOD_8()   w
# define _ARM64_LDR_RREG_SIZE_MOD_16()  w
# define _ARM64_LDR_RREG_SIZE_MOD_32()  x
# define _ARM64_LDR_RREG_SIZE_MOD_64()  x
# define _ARM64_LDR_RREG_SIZE_MOD_I8()  _ARM64_LDR_RREG_SIZE_MOD_8()
# define _ARM64_LDR_RREG_SIZE_MOD_I16() _ARM64_LDR_RREG_SIZE_MOD_16()
# define _ARM64_LDR_RREG_SIZE_MOD_I32() _ARM64_LDR_RREG_SIZE_MOD_32()
# define _ARM64_LDR_RREG_SIZE_MOD_I64() _ARM64_LDR_RREG_SIZE_MOD_64()
# define _ARM64_LDR_RREG_SIZE_MOD_S8()  _ARM64_LDR_RREG_SIZE_MOD_8()
# define _ARM64_LDR_RREG_SIZE_MOD_S16() _ARM64_LDR_RREG_SIZE_MOD_16()
# define _ARM64_LDR_RREG_SIZE_MOD_S32() _ARM64_LDR_RREG_SIZE_MOD_32()
# define _ARM64_LDR_RREG_SIZE_MOD_S64() _ARM64_LDR_RREG_SIZE_MOD_64()
# define _ARM64_LDR_RREG_SIZE_MOD_U8()  _ARM64_LDR_RREG_SIZE_MOD_8()
# define _ARM64_LDR_RREG_SIZE_MOD_U16() _ARM64_LDR_RREG_SIZE_MOD_16()
# define _ARM64_LDR_RREG_SIZE_MOD_U32() w
# define _ARM64_LDR_RREG_SIZE_MOD_U64() _ARM64_LDR_RREG_SIZE_MOD_64()
/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * Store
     */

# define _ARM64_STORE(W, DRR,PRR)       VIPM_STR_C(_ARM64_STR_CMD(W)) " " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) "] \n\t"
# define _ARM64_STOREO(W, DRR,PRR,ORR)  VIPM_STR_C(_ARM64_STR_CMD(W)) " " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "] \n\t"
# define _ARM64_STOREOI(W, DRR,PRR,Off) VIPM_STR_C(_ARM64_STR_CMD(W)) " " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "] \n\t"

# define _ARM64_STORE_IF(W,_Cond, DRR,PRR)       \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_STORE(W, DRR,PRR)                     \
"11001:"
# define _ARM64_STOREO_IF(W,_Cond, DRR,PRR,ORR)  \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_STOREO(W, DRR,PRR,ORR)                \
"11001:"
# define _ARM64_STOREOI_IF(W,_Cond, DRR,PRR,Off) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_STOREOI(W, DRR,PRR,Off)               \
"11001:"


/* slow operation (?) */
# define _ARM64_STORE_IB(W, DRR,PRR,ORR) \
	_ARM64_ADD(W, PRR,PRR,ORR)           \
	_ARM64_STORE(W, DRR,PRR)
# define _ARM64_STORE_DB(W, DRR,PRR,ORR) \
	_ARM64_SUB(W, PRR,PRR,ORR)           \
	_ARM64_STORE(W, DRR,PRR)
# define _ARM64_STORE_IA(W, DRR,PRR,ORR) \
	_ARM64_STORE(W, DRR,PRR)             \
	_ARM64_ADD(W, PRR,PRR,ORR)
# define _ARM64_STORE_DA(W, DRR,PRR,ORR) \
	_ARM64_STORE(W, DRR,PRR)             \
	_ARM64_SUB(W, PRR,PRR,ORR)
# define _ARM64_STOREI_IB(W, DRR,PRR,Off)   VIPM_STR_C(_ARM64_STR_CMD(W)) " " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Off) "]! \n\t"
# define _ARM64_STOREI_DB(W, DRR,PRR,Off)   VIPM_STR_C(_ARM64_STR_CMD(W)) " " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) ", #-" VIPM_STR_C(Off) "]! \n\t"
# define _ARM64_STOREI_IA(W, DRR,PRR,Off)   VIPM_STR_C(_ARM64_STR_CMD(W)) " " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) "], #" VIPM_STR_C(Off) " \n\t"
# define _ARM64_STOREI_DA(W, DRR,PRR,Off)   VIPM_STR_C(_ARM64_STR_CMD(W)) " " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)DRR) ", [" VIPM_STR_C(%PRR) "], #-" VIPM_STR_C(Off) " \n\t"


/* @{ */
# define _ARM64_STR_CMD(W)          VIPM_GLUE(_ARM64_STR_CMD_,W)()
# define _ARM64_STR_CMD_8()         strb
# define _ARM64_STR_CMD_U8()        strb
# define _ARM64_STR_CMD_S8()        strb
# define _ARM64_STR_CMD_I8()        strb
# define _ARM64_STR_CMD_16()        strh
# define _ARM64_STR_CMD_I16()       strh
# define _ARM64_STR_CMD_S16()       strh
# define _ARM64_STR_CMD_U16()       strh
# define _ARM64_STR_CMD_32()        str
# define _ARM64_STR_CMD_I32()       str
# define _ARM64_STR_CMD_S32()       str
# define _ARM64_STR_CMD_U32()       str
# define _ARM64_STR_CMD_64()        str
# define _ARM64_STR_CMD_I64()       str
# define _ARM64_STR_CMD_S64()       str
# define _ARM64_STR_CMD_U64()       str
/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * Prefetch Memory
     */

# define _ARM64_PREFETCHI_CACHELINE(_Type,_Target,_Policy,PRR,Offset) \
	"prfm " VIPM_STR_C(_ARM64_PRFM_TYPE(_Type)) VIPM_STR_C(_Target) VIPM_STR_C(_Policy) ", [" VIPM_STR_C(%PRR) ", #" VIPM_STR_C(Offset) "] \n\t"
# define _ARM64_APREFETCHI_CACHELINE(_Type,_Target,_Policy,PRR,Offset)  _ARM64_PREFETCHI_CACHELINE(_Type,_Target,_Policy,PRR,_ARM_PRFM_OFFSET(Offset))
# define _ARM64_PREFETCH_CACHELINE(_Type,_Target,_Policy,PRR,ORR) \
	"prfm " VIPM_STR_C(_ARM64_PRFM_TYPE(_Type)) VIPM_STR_C(_Target) VIPM_STR_C(_Policy) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) "] \n\t"
# define _ARM64_PREFETCHEX_CACHELINE(_Type,_Target,_Policy,PRR,ORR,Ext) \
	"prfm " VIPM_STR_C(_ARM64_PRFM_TYPE(_Type)) VIPM_STR_C(_Target) VIPM_STR_C(_Policy) ", [" VIPM_STR_C(%PRR) ", " VIPM_STR_C(%ORR) ", " Ext "] \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_PRFM_TYPE(_Type)    VIPM_GLUE(_ARM64_PRFM_TYPE_,_Type)
# define _ARM64_PRFM_TYPE_RD        PLD
# define _ARM64_PRFM_TYPE_RW        PLD
# define _ARM64_PRFM_TYPE_WR        PST
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Flow control
 * @{
 */

# define _ARM64_GOTO_IMM(Label)         "b " VIPM_STR_C(Label) " \n\t"
# define _ARM64_GOTO_IF(_Cond,Label)    "b." VIPM_STR_C(_ARM_COND_MAP(_Cond)) " " VIPM_STR_C(Label) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Switch jump-table
	 * @note: Imm = N, case <- {[0, N], default}
	 * @note: TRR - unnamed register
     */

# define _ARM64_SWITCHEX(TRR,RR,Imm,Label)                                           \
	_ARM64_CMPI(RR, Imm)                                                             \
	_ARM64_GOTO_IF(HI, Label)                                                        \
	"adr " VIPM_STR_C(TRR) ", 11011f \n\t"                                           \
	"add " VIPM_STR_C(TRR) ", " VIPM_STR_C(TRR) ", " VIPM_STR_C(%RR) ", lsl #2 \n\t" \
	"br " VIPM_STR_C(TRR) " \n\t"                                                    \
"11011:"

# define _ARM64_SWITCH(RR,Imm,Label) _ARM64_SWITCHEX(_ARM64_SWTR,RR,Imm,Label)

/* Usage sample:

ARM64:
	_ARM64_SWITCH([x], N, Ldefault)

	_ARM64_GOTO_IMM(L0)
	_ARM64_GOTO_IMM(L1)
	_ARM64_GOTO_IMM(L2)
	_ARM64_GOTO_IMM(L3)
	...
	_ARM64_GOTO_IMM(Ln)

	L0:       stmt0 _ARM64_GOTO_IMM(Lend)
	L1:       stmt1 _ARM64_GOTO_IMM(Lend)
	L2:       stmt2 _ARM64_GOTO_IMM(Lend)
	L3:       stmt3 _ARM64_GOTO_IMM(Lend)
	...
	Ln:       stmtN _ARM64_GOTO_IMM(Lend)
	Ldefault: stmtD _ARM64_GOTO_IMM(Lend)
	Lend:

C:
	switch (x) {
	case 0:  stmt0 break;
	case 1:  stmt1 break;
	case 2:  stmt2 break;
	case 3:  stmt3 break;
	...
	case N:  stmtN break;
	default: stmtD break;
	}
 */

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define _ARM64_DPRINTFI(_Str)          \
    "ldr x0,=10000f               \n\t" \
    "bl printf                    \n\t" \
    "b 10001f                     \n\t" \
	".ltorg                       \n\t" \
    "10000: .asciz " VIPM_STR_C(_Str) " \n\t" \
    ".align 4                     \n\t" \
    "10001:                       \n\t"


# define __ARM64_PRINTRR1(_Fmt,RR)      \
    "mov x1," VIPM_STR_C(%RR) "   \n\t" \
    _ARM64_DPRINTFI(_Fmt)

# define __ARM64_PRINTRR2(_Fmt,RR1,RR2) \
	_ARM64_PUSH2(RR1,RR2)               \
	"ldp x1, x2, [sp], 16         \n\t" \
    _ARM64_DPRINTFI(_Fmt)

# define __ARM64_PRINTRR3(_Fmt,RR1,RR2,RR3) \
	_ARM64_PUSH2(RR1,RR2)                   \
	_ARM64_PUSH1(RR3)                       \
	"ldr x3, [sp], 16                 \n\t" \
	"ldp x1, x2, [sp], 16             \n\t" \
    _ARM64_DPRINTFI(_Fmt)

/* @} */
/* @} */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*
 * NEON
 * @{
 */

# define _ARM64_QVR_FMT(W,Q)        VIPM_GLUE(v,Q)._ARM64_QWTMAP(W)

/* ------------------------------------------------------------------------- */

# define _ARM64_DVR_FMT(W,D)        VIPM_GLUE(_ARM64_DVR_FMT_,_ARM_WIDTHOF(W))(D)

/* @{ */
# define _ARM64_DVR_FMT_8(D)        VIPM_GLUE(v,_ARM64_DVLMAP(D))._ARM64_DWTMAP(8)
# define _ARM64_DVR_FMT_16(D)       VIPM_GLUE(v,_ARM64_DVLMAP(D))._ARM64_DWTMAP(16)
# define _ARM64_DVR_FMT_32(D)       VIPM_GLUE(v,_ARM64_DVLMAP(D))._ARM64_DWTMAP(32)
# define _ARM64_DVR_FMT_64(D)       VIPM_GLUE(d,_ARM64_DVLMAP(D))
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_XDVR_FMT(W,D)       VIPM_GLUE(_ARM64_XDVR_FMT_,_ARM_WIDTHOF(W))(D)

/* @{ */
# define _ARM64_XDVR_FMT_8(D)       VIPM_GLUE(v,_ARM64_DVMAP(D))._ARM64_XWTMAP(8,D)
# define _ARM64_XDVR_FMT_16(D)      VIPM_GLUE(v,_ARM64_DVMAP(D))._ARM64_XWTMAP(16,D)
# define _ARM64_XDVR_FMT_32(D)      VIPM_GLUE(v,_ARM64_DVMAP(D))._ARM64_XWTMAP(32,D)
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_SVR_FMT(W,S)        VIPM_GLUE(s,_ARM64_SVLMAP(S))

/* ------------------------------------------------------------------------- */

# define _ARM64_QSCL_FMT(W,L,Q)     VIPM_GLUE(v,Q)._ARM64_WTMAP(W)[L]
# define _ARM64_DSCL_FMT(W,L,D)     VIPM_GLUE(v,_ARM64_DVMAP(D))._ARM64_WTMAP(W)[_ARM64_LDVMAP(W,L,D)]
# define _ARM64_SSCL_FMT(W,L,S)     VIPM_GLUE(v,_ARM64_SVMAP(S))._ARM64_WTMAP(W)[_ARM64_LSVMAP(W,L,S)]

/* ------------------------------------------------------------------------- */

# define _ARM64_DVMAP(D)            VIPM_GLUE(_ARM64_DVMAP_,D)()

/* @{ */
# define _ARM64_DVMAP_0()           0
# define _ARM64_DVMAP_1()           0
# define _ARM64_DVMAP_2()           1
# define _ARM64_DVMAP_3()           1
# define _ARM64_DVMAP_4()           2
# define _ARM64_DVMAP_5()           2
# define _ARM64_DVMAP_6()           3
# define _ARM64_DVMAP_7()           3
# define _ARM64_DVMAP_8()           4
# define _ARM64_DVMAP_9()           4
# define _ARM64_DVMAP_10()          5
# define _ARM64_DVMAP_11()          5
# define _ARM64_DVMAP_12()          6
# define _ARM64_DVMAP_13()          6
# define _ARM64_DVMAP_14()          7
# define _ARM64_DVMAP_15()          7
# define _ARM64_DVMAP_16()          8
# define _ARM64_DVMAP_17()          8
# define _ARM64_DVMAP_18()          9
# define _ARM64_DVMAP_19()          9
# define _ARM64_DVMAP_20()          10
# define _ARM64_DVMAP_21()          10
# define _ARM64_DVMAP_22()          11
# define _ARM64_DVMAP_23()          11
# define _ARM64_DVMAP_24()          12
# define _ARM64_DVMAP_25()          12
# define _ARM64_DVMAP_26()          13
# define _ARM64_DVMAP_27()          13
# define _ARM64_DVMAP_28()          14
# define _ARM64_DVMAP_29()          14
# define _ARM64_DVMAP_30()          15
# define _ARM64_DVMAP_31()          15
# define _ARM64_DVMAP_32()          16
# define _ARM64_DVMAP_33()          16
# define _ARM64_DVMAP_34()          17
# define _ARM64_DVMAP_35()          17
# define _ARM64_DVMAP_36()          18
# define _ARM64_DVMAP_37()          18
# define _ARM64_DVMAP_38()          19
# define _ARM64_DVMAP_39()          19
# define _ARM64_DVMAP_40()          20
# define _ARM64_DVMAP_41()          20
# define _ARM64_DVMAP_42()          21
# define _ARM64_DVMAP_43()          21
# define _ARM64_DVMAP_44()          22
# define _ARM64_DVMAP_45()          22
# define _ARM64_DVMAP_46()          23
# define _ARM64_DVMAP_47()          23
# define _ARM64_DVMAP_48()          24
# define _ARM64_DVMAP_49()          24
# define _ARM64_DVMAP_50()          25
# define _ARM64_DVMAP_51()          25
# define _ARM64_DVMAP_52()          26
# define _ARM64_DVMAP_53()          26
# define _ARM64_DVMAP_54()          27
# define _ARM64_DVMAP_55()          27
# define _ARM64_DVMAP_56()          28
# define _ARM64_DVMAP_57()          28
# define _ARM64_DVMAP_58()          29
# define _ARM64_DVMAP_59()          29
# define _ARM64_DVMAP_60()          30
# define _ARM64_DVMAP_61()          30
# define _ARM64_DVMAP_62()          31
# define _ARM64_DVMAP_63()          31
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_DVLMAP(D)           VIPM_GLUE(_ARM64_DVLMAP_,D)()

/* @{ */
# define _ARM64_DVLMAP_0()          0
# define _ARM64_DVLMAP_1()          _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D1
# define _ARM64_DVLMAP_2()          1
# define _ARM64_DVLMAP_3()          _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D3
# define _ARM64_DVLMAP_4()          2
# define _ARM64_DVLMAP_5()          _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D5
# define _ARM64_DVLMAP_6()          3
# define _ARM64_DVLMAP_7()          _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D7
# define _ARM64_DVLMAP_8()          4
# define _ARM64_DVLMAP_9()          _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D9
# define _ARM64_DVLMAP_10()         5
# define _ARM64_DVLMAP_11()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D11
# define _ARM64_DVLMAP_12()         6
# define _ARM64_DVLMAP_13()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D13
# define _ARM64_DVLMAP_14()         7
# define _ARM64_DVLMAP_15()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D15
# define _ARM64_DVLMAP_16()         8
# define _ARM64_DVLMAP_17()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D17
# define _ARM64_DVLMAP_18()         9
# define _ARM64_DVLMAP_19()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D19
# define _ARM64_DVLMAP_20()         10
# define _ARM64_DVLMAP_21()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D21
# define _ARM64_DVLMAP_22()         11
# define _ARM64_DVLMAP_23()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D23
# define _ARM64_DVLMAP_24()         12
# define _ARM64_DVLMAP_25()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D25
# define _ARM64_DVLMAP_26()         13
# define _ARM64_DVLMAP_27()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D27
# define _ARM64_DVLMAP_28()         14
# define _ARM64_DVLMAP_29()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D29
# define _ARM64_DVLMAP_30()         15
# define _ARM64_DVLMAP_31()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D31
# define _ARM64_DVLMAP_32()         16
# define _ARM64_DVLMAP_33()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D33
# define _ARM64_DVLMAP_34()         17
# define _ARM64_DVLMAP_35()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D35
# define _ARM64_DVLMAP_36()         18
# define _ARM64_DVLMAP_37()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D37
# define _ARM64_DVLMAP_38()         19
# define _ARM64_DVLMAP_39()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D39
# define _ARM64_DVLMAP_40()         20
# define _ARM64_DVLMAP_41()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D41
# define _ARM64_DVLMAP_42()         21
# define _ARM64_DVLMAP_43()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D43
# define _ARM64_DVLMAP_44()         22
# define _ARM64_DVLMAP_45()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D45
# define _ARM64_DVLMAP_46()         23
# define _ARM64_DVLMAP_47()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D47
# define _ARM64_DVLMAP_48()         24
# define _ARM64_DVLMAP_49()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D49
# define _ARM64_DVLMAP_50()         25
# define _ARM64_DVLMAP_51()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D51
# define _ARM64_DVLMAP_52()         26
# define _ARM64_DVLMAP_53()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D53
# define _ARM64_DVLMAP_54()         27
# define _ARM64_DVLMAP_55()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D55
# define _ARM64_DVLMAP_56()         28
# define _ARM64_DVLMAP_57()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D57
# define _ARM64_DVLMAP_58()         29
# define _ARM64_DVLMAP_59()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D59
# define _ARM64_DVLMAP_60()         30
# define _ARM64_DVLMAP_61()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D61
# define _ARM64_DVLMAP_62()         31
# define _ARM64_DVLMAP_63()         _ARM64_VECTOR_HIGHHALF_ACCESS_ERROR_D63
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_SVMAP(S)            VIPM_GLUE(_ARM64_SVMAP_,S)()

/* @{ */
# define _ARM64_SVMAP_0()           0
# define _ARM64_SVMAP_1()           0
# define _ARM64_SVMAP_2()           0
# define _ARM64_SVMAP_3()           0
# define _ARM64_SVMAP_4()           1
# define _ARM64_SVMAP_5()           1
# define _ARM64_SVMAP_6()           1
# define _ARM64_SVMAP_7()           1
# define _ARM64_SVMAP_8()           2
# define _ARM64_SVMAP_9()           2
# define _ARM64_SVMAP_10()          2
# define _ARM64_SVMAP_11()          2
# define _ARM64_SVMAP_12()          3
# define _ARM64_SVMAP_13()          3
# define _ARM64_SVMAP_14()          3
# define _ARM64_SVMAP_15()          3
# define _ARM64_SVMAP_16()          4
# define _ARM64_SVMAP_17()          4
# define _ARM64_SVMAP_18()          4
# define _ARM64_SVMAP_19()          4
# define _ARM64_SVMAP_20()          5
# define _ARM64_SVMAP_21()          5
# define _ARM64_SVMAP_22()          5
# define _ARM64_SVMAP_23()          5
# define _ARM64_SVMAP_24()          6
# define _ARM64_SVMAP_25()          6
# define _ARM64_SVMAP_26()          6
# define _ARM64_SVMAP_27()          6
# define _ARM64_SVMAP_28()          7
# define _ARM64_SVMAP_29()          7
# define _ARM64_SVMAP_30()          7
# define _ARM64_SVMAP_31()          7
# define _ARM64_SVMAP_32()          8
# define _ARM64_SVMAP_33()          8
# define _ARM64_SVMAP_34()          8
# define _ARM64_SVMAP_35()          8
# define _ARM64_SVMAP_36()          9
# define _ARM64_SVMAP_37()          9
# define _ARM64_SVMAP_38()          9
# define _ARM64_SVMAP_39()          9
# define _ARM64_SVMAP_40()          10
# define _ARM64_SVMAP_41()          10
# define _ARM64_SVMAP_42()          10
# define _ARM64_SVMAP_43()          10
# define _ARM64_SVMAP_44()          11
# define _ARM64_SVMAP_45()          11
# define _ARM64_SVMAP_46()          11
# define _ARM64_SVMAP_47()          11
# define _ARM64_SVMAP_48()          12
# define _ARM64_SVMAP_49()          12
# define _ARM64_SVMAP_50()          12
# define _ARM64_SVMAP_51()          12
# define _ARM64_SVMAP_52()          13
# define _ARM64_SVMAP_53()          13
# define _ARM64_SVMAP_54()          13
# define _ARM64_SVMAP_55()          13
# define _ARM64_SVMAP_56()          14
# define _ARM64_SVMAP_57()          14
# define _ARM64_SVMAP_58()          14
# define _ARM64_SVMAP_59()          14
# define _ARM64_SVMAP_60()          15
# define _ARM64_SVMAP_61()          15
# define _ARM64_SVMAP_62()          15
# define _ARM64_SVMAP_63()          15
# define _ARM64_SVMAP_64()          16
# define _ARM64_SVMAP_65()          16
# define _ARM64_SVMAP_66()          16
# define _ARM64_SVMAP_67()          16
# define _ARM64_SVMAP_68()          17
# define _ARM64_SVMAP_69()          17
# define _ARM64_SVMAP_70()          17
# define _ARM64_SVMAP_71()          17
# define _ARM64_SVMAP_72()          18
# define _ARM64_SVMAP_73()          18
# define _ARM64_SVMAP_74()          18
# define _ARM64_SVMAP_75()          18
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_SVLMAP(S)           VIPM_GLUE(_ARM64_SVLMAP_,S)()

/* @{ */
# define _ARM64_SVLMAP_0()          0
# define _ARM64_SVLMAP_1()          _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S1
# define _ARM64_SVLMAP_2()          _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S2
# define _ARM64_SVLMAP_3()          _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S3
# define _ARM64_SVLMAP_4()          1
# define _ARM64_SVLMAP_5()          _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S5
# define _ARM64_SVLMAP_6()          _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S6
# define _ARM64_SVLMAP_7()          _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S7
# define _ARM64_SVLMAP_8()          2
# define _ARM64_SVLMAP_9()          _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S9
# define _ARM64_SVLMAP_10()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S10
# define _ARM64_SVLMAP_11()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S11
# define _ARM64_SVLMAP_12()         3
# define _ARM64_SVLMAP_13()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S13
# define _ARM64_SVLMAP_14()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S14
# define _ARM64_SVLMAP_15()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S15
# define _ARM64_SVLMAP_16()         4
# define _ARM64_SVLMAP_17()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S17
# define _ARM64_SVLMAP_18()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S18
# define _ARM64_SVLMAP_19()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S19
# define _ARM64_SVLMAP_20()         5
# define _ARM64_SVLMAP_21()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S21
# define _ARM64_SVLMAP_22()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S22
# define _ARM64_SVLMAP_23()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S23
# define _ARM64_SVLMAP_24()         6
# define _ARM64_SVLMAP_25()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S25
# define _ARM64_SVLMAP_26()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S26
# define _ARM64_SVLMAP_27()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S27
# define _ARM64_SVLMAP_28()         7
# define _ARM64_SVLMAP_29()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S29
# define _ARM64_SVLMAP_30()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S30
# define _ARM64_SVLMAP_31()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S31
# define _ARM64_SVLMAP_32()         8
# define _ARM64_SVLMAP_33()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S33
# define _ARM64_SVLMAP_34()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S34
# define _ARM64_SVLMAP_35()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S35
# define _ARM64_SVLMAP_36()         9
# define _ARM64_SVLMAP_37()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S37
# define _ARM64_SVLMAP_38()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S38
# define _ARM64_SVLMAP_39()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S39
# define _ARM64_SVLMAP_40()         10
# define _ARM64_SVLMAP_41()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S41
# define _ARM64_SVLMAP_42()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S42
# define _ARM64_SVLMAP_43()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S43
# define _ARM64_SVLMAP_44()         11
# define _ARM64_SVLMAP_45()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S45
# define _ARM64_SVLMAP_46()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S46
# define _ARM64_SVLMAP_47()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S47
# define _ARM64_SVLMAP_48()         12
# define _ARM64_SVLMAP_49()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S49
# define _ARM64_SVLMAP_50()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S50
# define _ARM64_SVLMAP_51()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S51
# define _ARM64_SVLMAP_52()         13
# define _ARM64_SVLMAP_53()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S53
# define _ARM64_SVLMAP_54()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S54
# define _ARM64_SVLMAP_55()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S55
# define _ARM64_SVLMAP_56()         14
# define _ARM64_SVLMAP_57()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S57
# define _ARM64_SVLMAP_58()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S58
# define _ARM64_SVLMAP_59()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S59
# define _ARM64_SVLMAP_60()         15
# define _ARM64_SVLMAP_61()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S61
# define _ARM64_SVLMAP_62()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S62
# define _ARM64_SVLMAP_63()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S63
# define _ARM64_SVLMAP_64()         16
# define _ARM64_SVLMAP_65()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S65
# define _ARM64_SVLMAP_66()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S66
# define _ARM64_SVLMAP_67()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S67
# define _ARM64_SVLMAP_68()         17
# define _ARM64_SVLMAP_69()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S69
# define _ARM64_SVLMAP_70()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S70
# define _ARM64_SVLMAP_71()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S71
# define _ARM64_SVLMAP_72()         18
# define _ARM64_SVLMAP_73()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S73
# define _ARM64_SVLMAP_74()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S74
# define _ARM64_SVLMAP_75()         _ARM64_VECTOR_HIGH3QUATERS_ACCESS_ERROR_S75
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_LDVMAP(W,L,D)       VIPM_GLUE4(_ARM64_LDVMAP_,_ARM_WIDTHOF(W),L,_ARM_IMOD2(D))()

/* @{ */
# define _ARM64_LDVMAP_800()        0
# define _ARM64_LDVMAP_810()        1
# define _ARM64_LDVMAP_820()        2
# define _ARM64_LDVMAP_830()        3
# define _ARM64_LDVMAP_840()        4
# define _ARM64_LDVMAP_850()        5
# define _ARM64_LDVMAP_860()        6
# define _ARM64_LDVMAP_870()        7
# define _ARM64_LDVMAP_801()        8
# define _ARM64_LDVMAP_811()        9
# define _ARM64_LDVMAP_821()        10
# define _ARM64_LDVMAP_831()        11
# define _ARM64_LDVMAP_841()        12
# define _ARM64_LDVMAP_851()        13
# define _ARM64_LDVMAP_861()        14
# define _ARM64_LDVMAP_871()        15
# define _ARM64_LDVMAP_1600()       0
# define _ARM64_LDVMAP_1610()       1
# define _ARM64_LDVMAP_1620()       2
# define _ARM64_LDVMAP_1630()       3
# define _ARM64_LDVMAP_1601()       4
# define _ARM64_LDVMAP_1611()       5
# define _ARM64_LDVMAP_1621()       6
# define _ARM64_LDVMAP_1631()       7
# define _ARM64_LDVMAP_3200()       0
# define _ARM64_LDVMAP_3210()       1
# define _ARM64_LDVMAP_3201()       2
# define _ARM64_LDVMAP_3211()       3
# define _ARM64_LDVMAP_6400()       0
# define _ARM64_LDVMAP_6401()       1
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_LSVMAP(W,L,S)       VIPM_GLUE4(_ARM64_LSVMAP_,W,L,_ARM_IMOD4(S))()

/* @{ */
# define _ARM64_LSVMAP_800()        0
# define _ARM64_LSVMAP_810()        1
# define _ARM64_LSVMAP_820()        2
# define _ARM64_LSVMAP_830()        3
# define _ARM64_LSVMAP_801()        4
# define _ARM64_LSVMAP_811()        5
# define _ARM64_LSVMAP_821()        6
# define _ARM64_LSVMAP_831()        7
# define _ARM64_LSVMAP_802()        8
# define _ARM64_LSVMAP_812()        9
# define _ARM64_LSVMAP_822()        10
# define _ARM64_LSVMAP_832()        11
# define _ARM64_LSVMAP_803()        12
# define _ARM64_LSVMAP_813()        13
# define _ARM64_LSVMAP_823()        14
# define _ARM64_LSVMAP_833()        15
# define _ARM64_LSVMAP_1600()       0
# define _ARM64_LSVMAP_1610()       1
# define _ARM64_LSVMAP_1601()       2
# define _ARM64_LSVMAP_1611()       3
# define _ARM64_LSVMAP_1602()       4
# define _ARM64_LSVMAP_1612()       5
# define _ARM64_LSVMAP_1603()       6
# define _ARM64_LSVMAP_1613()       7
# define _ARM64_LSVMAP_3200()       0
# define _ARM64_LSVMAP_3201()       1
# define _ARM64_LSVMAP_3202()       2
# define _ARM64_LSVMAP_3203()       3
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_QDMAP(Q)            VIPM_GLUE(_ARM64_QDMAP_,Q)()

/* @{ */
# define _ARM64_QDMAP_0()           0
# define _ARM64_QDMAP_1()           2
# define _ARM64_QDMAP_2()           4
# define _ARM64_QDMAP_3()           6
# define _ARM64_QDMAP_4()           8
# define _ARM64_QDMAP_5()           10
# define _ARM64_QDMAP_6()           12
# define _ARM64_QDMAP_7()           14
# define _ARM64_QDMAP_8()           16
# define _ARM64_QDMAP_9()           18
# define _ARM64_QDMAP_10()          20
# define _ARM64_QDMAP_11()          22
# define _ARM64_QDMAP_12()          24
# define _ARM64_QDMAP_13()          26
# define _ARM64_QDMAP_14()          28
# define _ARM64_QDMAP_15()          30
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_WTMAP(W)            VIPM_GLUE(_ARM64_WTMAP_,W)()

/* @{ */
# define _ARM64_WTMAP_8()           B
# define _ARM64_WTMAP_16()          H
# define _ARM64_WTMAP_32()          S
# define _ARM64_WTMAP_64()          D
# define _ARM64_WTMAP_S8()          B
# define _ARM64_WTMAP_S16()         H
# define _ARM64_WTMAP_S32()         S
# define _ARM64_WTMAP_S64()         D
# define _ARM64_WTMAP_U8()          B
# define _ARM64_WTMAP_U16()         H
# define _ARM64_WTMAP_U32()         S
# define _ARM64_WTMAP_U64()         D
# define _ARM64_WTMAP_F16()         H
# define _ARM64_WTMAP_F32()         S
# define _ARM64_WTMAP_F64()         D
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_DWTMAP(W)           VIPM_GLUE(_ARM64_DWTMAP_,W)()

/* @{ */
# define _ARM64_DWTMAP_8()          8B
# define _ARM64_DWTMAP_16()         4H
# define _ARM64_DWTMAP_32()         2S
# define _ARM64_DWTMAP_64()         1D
# define _ARM64_DWTMAP_S8()         8B
# define _ARM64_DWTMAP_S16()        4H
# define _ARM64_DWTMAP_S32()        2S
# define _ARM64_DWTMAP_S64()        1D
# define _ARM64_DWTMAP_U8()         8B
# define _ARM64_DWTMAP_U16()        4H
# define _ARM64_DWTMAP_U32()        2S
# define _ARM64_DWTMAP_U64()        1D
# define _ARM64_DWTMAP_F16()        4H
# define _ARM64_DWTMAP_F32()        2S
# define _ARM64_DWTMAP_F64()        1D
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_QWTMAP(W)           VIPM_GLUE(_ARM64_QWTMAP_,W)()

/* @{ */
# define _ARM64_QWTMAP_8()          16B
# define _ARM64_QWTMAP_16()         8H
# define _ARM64_QWTMAP_32()         4S
# define _ARM64_QWTMAP_64()         2D
# define _ARM64_QWTMAP_S8()         16B
# define _ARM64_QWTMAP_S16()        8H
# define _ARM64_QWTMAP_S32()        4S
# define _ARM64_QWTMAP_S64()        2D
# define _ARM64_QWTMAP_U8()         16B
# define _ARM64_QWTMAP_U16()        8H
# define _ARM64_QWTMAP_U32()        4S
# define _ARM64_QWTMAP_U64()        2D
# define _ARM64_QWTMAP_F16()        8H
# define _ARM64_QWTMAP_F32()        4S
# define _ARM64_QWTMAP_F64()        2D
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_XWTMAP(W,D)         VIPM_GLUE(_ARM64_XWTMAP_,_ARM_IMOD2(D))(W)

/* @{ */
# define _ARM64_XWTMAP_0(W)         _ARM64_DWTMAP(W)
# define _ARM64_XWTMAP_1(W)         _ARM64_QWTMAP(W)
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_VHALF_SUFFIX(D)     VIPM_GLUE(_ARM64_VHALF_SUFFIX_,_ARM_IMOD2(D))()

/* @{ */
# define _ARM64_VHALF_SUFFIX_0()
# define _ARM64_VHALF_SUFFIX_1()    2
/* @} */

/* ------------------------------------------------------------------------- */

# define _ARM64_IF_EQWI(W,Imm)      VIPM_GLUE4(_ARM64_IF_EQWI_,W,_,Imm)()

/* @{ */
# define _ARM64_IF_EQWI_0_0()       1
# define _ARM64_IF_EQWI_0_1()       0
# define _ARM64_IF_EQWI_0_2()       0
# define _ARM64_IF_EQWI_0_3()       0
# define _ARM64_IF_EQWI_0_4()       0
# define _ARM64_IF_EQWI_0_5()       0
# define _ARM64_IF_EQWI_0_6()       0
# define _ARM64_IF_EQWI_0_7()       0
# define _ARM64_IF_EQWI_0_8()       0
# define _ARM64_IF_EQWI_0_9()       0
# define _ARM64_IF_EQWI_0_10()      0
# define _ARM64_IF_EQWI_0_11()      0
# define _ARM64_IF_EQWI_0_12()      0
# define _ARM64_IF_EQWI_0_13()      0
# define _ARM64_IF_EQWI_0_14()      0
# define _ARM64_IF_EQWI_0_15()      0
# define _ARM64_IF_EQWI_0_16()      0
# define _ARM64_IF_EQWI_0_17()      0
# define _ARM64_IF_EQWI_0_18()      0
# define _ARM64_IF_EQWI_0_19()      0
# define _ARM64_IF_EQWI_0_20()      0
# define _ARM64_IF_EQWI_0_21()      0
# define _ARM64_IF_EQWI_0_22()      0
# define _ARM64_IF_EQWI_0_23()      0
# define _ARM64_IF_EQWI_0_24()      0
# define _ARM64_IF_EQWI_0_25()      0
# define _ARM64_IF_EQWI_0_26()      0
# define _ARM64_IF_EQWI_0_27()      0
# define _ARM64_IF_EQWI_0_28()      0
# define _ARM64_IF_EQWI_0_29()      0
# define _ARM64_IF_EQWI_0_30()      0
# define _ARM64_IF_EQWI_0_31()      0
# define _ARM64_IF_EQWI_0_32()      0
# define _ARM64_IF_EQWI_8_0()       0
# define _ARM64_IF_EQWI_8_1()       0
# define _ARM64_IF_EQWI_8_2()       0
# define _ARM64_IF_EQWI_8_3()       0
# define _ARM64_IF_EQWI_8_4()       0
# define _ARM64_IF_EQWI_8_5()       0
# define _ARM64_IF_EQWI_8_6()       0
# define _ARM64_IF_EQWI_8_7()       0
# define _ARM64_IF_EQWI_8_8()       1
# define _ARM64_IF_EQWI_8_9()       0
# define _ARM64_IF_EQWI_8_10()      0
# define _ARM64_IF_EQWI_8_11()      0
# define _ARM64_IF_EQWI_8_12()      0
# define _ARM64_IF_EQWI_8_13()      0
# define _ARM64_IF_EQWI_8_14()      0
# define _ARM64_IF_EQWI_8_15()      0
# define _ARM64_IF_EQWI_8_16()      0
# define _ARM64_IF_EQWI_8_17()      0
# define _ARM64_IF_EQWI_8_18()      0
# define _ARM64_IF_EQWI_8_19()      0
# define _ARM64_IF_EQWI_8_20()      0
# define _ARM64_IF_EQWI_8_21()      0
# define _ARM64_IF_EQWI_8_22()      0
# define _ARM64_IF_EQWI_8_23()      0
# define _ARM64_IF_EQWI_8_24()      0
# define _ARM64_IF_EQWI_8_25()      0
# define _ARM64_IF_EQWI_8_26()      0
# define _ARM64_IF_EQWI_8_27()      0
# define _ARM64_IF_EQWI_8_28()      0
# define _ARM64_IF_EQWI_8_29()      0
# define _ARM64_IF_EQWI_8_30()      0
# define _ARM64_IF_EQWI_8_31()      0
# define _ARM64_IF_EQWI_8_32()      0
# define _ARM64_IF_EQWI_16_0()      0
# define _ARM64_IF_EQWI_16_1()      0
# define _ARM64_IF_EQWI_16_2()      0
# define _ARM64_IF_EQWI_16_3()      0
# define _ARM64_IF_EQWI_16_4()      0
# define _ARM64_IF_EQWI_16_5()      0
# define _ARM64_IF_EQWI_16_6()      0
# define _ARM64_IF_EQWI_16_7()      0
# define _ARM64_IF_EQWI_16_8()      0
# define _ARM64_IF_EQWI_16_9()      0
# define _ARM64_IF_EQWI_16_10()     0
# define _ARM64_IF_EQWI_16_11()     0
# define _ARM64_IF_EQWI_16_12()     0
# define _ARM64_IF_EQWI_16_13()     0
# define _ARM64_IF_EQWI_16_14()     0
# define _ARM64_IF_EQWI_16_15()     0
# define _ARM64_IF_EQWI_16_16()     1
# define _ARM64_IF_EQWI_16_17()     0
# define _ARM64_IF_EQWI_16_18()     0
# define _ARM64_IF_EQWI_16_19()     0
# define _ARM64_IF_EQWI_16_20()     0
# define _ARM64_IF_EQWI_16_21()     0
# define _ARM64_IF_EQWI_16_22()     0
# define _ARM64_IF_EQWI_16_23()     0
# define _ARM64_IF_EQWI_16_24()     0
# define _ARM64_IF_EQWI_16_25()     0
# define _ARM64_IF_EQWI_16_26()     0
# define _ARM64_IF_EQWI_16_27()     0
# define _ARM64_IF_EQWI_16_28()     0
# define _ARM64_IF_EQWI_16_29()     0
# define _ARM64_IF_EQWI_16_30()     0
# define _ARM64_IF_EQWI_16_31()     0
# define _ARM64_IF_EQWI_16_32()     0
# define _ARM64_IF_EQWI_32_0()      0
# define _ARM64_IF_EQWI_32_1()      0
# define _ARM64_IF_EQWI_32_2()      0
# define _ARM64_IF_EQWI_32_3()      0
# define _ARM64_IF_EQWI_32_4()      0
# define _ARM64_IF_EQWI_32_5()      0
# define _ARM64_IF_EQWI_32_6()      0
# define _ARM64_IF_EQWI_32_7()      0
# define _ARM64_IF_EQWI_32_8()      0
# define _ARM64_IF_EQWI_32_9()      0
# define _ARM64_IF_EQWI_32_10()     0
# define _ARM64_IF_EQWI_32_11()     0
# define _ARM64_IF_EQWI_32_12()     0
# define _ARM64_IF_EQWI_32_13()     0
# define _ARM64_IF_EQWI_32_14()     0
# define _ARM64_IF_EQWI_32_15()     0
# define _ARM64_IF_EQWI_32_16()     0
# define _ARM64_IF_EQWI_32_17()     0
# define _ARM64_IF_EQWI_32_18()     0
# define _ARM64_IF_EQWI_32_19()     0
# define _ARM64_IF_EQWI_32_20()     0
# define _ARM64_IF_EQWI_32_21()     0
# define _ARM64_IF_EQWI_32_22()     0
# define _ARM64_IF_EQWI_32_23()     0
# define _ARM64_IF_EQWI_32_24()     0
# define _ARM64_IF_EQWI_32_25()     0
# define _ARM64_IF_EQWI_32_26()     0
# define _ARM64_IF_EQWI_32_27()     0
# define _ARM64_IF_EQWI_32_28()     0
# define _ARM64_IF_EQWI_32_29()     0
# define _ARM64_IF_EQWI_32_30()     0
# define _ARM64_IF_EQWI_32_31()     0
# define _ARM64_IF_EQWI_32_32()     1
# define _ARM64_IF_EQWI_64_0()      0
# define _ARM64_IF_EQWI_64_1()      0
# define _ARM64_IF_EQWI_64_2()      0
# define _ARM64_IF_EQWI_64_3()      0
# define _ARM64_IF_EQWI_64_4()      0
# define _ARM64_IF_EQWI_64_5()      0
# define _ARM64_IF_EQWI_64_6()      0
# define _ARM64_IF_EQWI_64_7()      0
# define _ARM64_IF_EQWI_64_8()      0
# define _ARM64_IF_EQWI_64_9()      0
# define _ARM64_IF_EQWI_64_10()     0
# define _ARM64_IF_EQWI_64_11()     0
# define _ARM64_IF_EQWI_64_12()     0
# define _ARM64_IF_EQWI_64_13()     0
# define _ARM64_IF_EQWI_64_14()     0
# define _ARM64_IF_EQWI_64_15()     0
# define _ARM64_IF_EQWI_64_16()     0
# define _ARM64_IF_EQWI_64_17()     0
# define _ARM64_IF_EQWI_64_18()     0
# define _ARM64_IF_EQWI_64_19()     0
# define _ARM64_IF_EQWI_64_20()     0
# define _ARM64_IF_EQWI_64_21()     0
# define _ARM64_IF_EQWI_64_22()     0
# define _ARM64_IF_EQWI_64_23()     0
# define _ARM64_IF_EQWI_64_24()     0
# define _ARM64_IF_EQWI_64_25()     0
# define _ARM64_IF_EQWI_64_26()     0
# define _ARM64_IF_EQWI_64_27()     0
# define _ARM64_IF_EQWI_64_28()     0
# define _ARM64_IF_EQWI_64_29()     0
# define _ARM64_IF_EQWI_64_30()     0
# define _ARM64_IF_EQWI_64_31()     0
# define _ARM64_IF_EQWI_64_32()     0
/* @} */


/* ************************************************************************* */

/*
 * Move operations
 * @{
 */

    /*
     * Duplicate/Set immediate value into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = Imm
     */
# define _ARM64_VDUPI_Q(W,VRI,Imm)      VIPM_GLUE(_ARM64_VDUPI_Q_,_ARM_DATAMOD(W))(W,VRI,Imm)
# define _ARM64_VDUPI_D(W,VRI,Imm)      VIPM_GLUE(_ARM64_VDUPI_D_,_ARM_DATAMOD(W))(W,VRI,Imm)
# define _ARM64_VDUPFI_Q(W,VRI,Imm)     "fmov " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_VDUPFI_D(W,VRI,Imm)     "fmov " VIPM_STR_C(_ARM64_DVR_FMT(W,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_VDUPFI_S(W,VRI,Imm)     "fmov " VIPM_STR_C(_ARM64_SVR_FMT(W,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VDUPIEX_Q(W,VRI,Imm,Ext)    "movI " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) ", #" VIPM_STR_C(Imm) ", " Ext " \n\t"
# define _ARM64_VDUPIEX_D(W,VRI,Imm,Ext)    "movI " VIPM_STR_C(_ARM64_DVR_FMT(W,VRI)) ", #" VIPM_STR_C(Imm) ", " Ext " \n\t"


    /*
     * Set zero value into one element of the destination vector register
	 * dM[k] = 0, k <- {0,N}
	 * sM = 0
	 */
# define _ARM64_VZERO1_Q(W,L,VRI)       "mov " VIPM_STR_C(_ARM64_QSCL_FMT(W,L,VRI)) ", " VIPM_STR_C(_ARM64_RREG_SIZE_MOD(W)zr) " \n\t"
# define _ARM64_VZERO1_D(W,L,VRI)       "mov " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,VRI)) ", " VIPM_STR_C(_ARM64_RREG_SIZE_MOD(W)zr) " \n\t"
# define _ARM64_VZERO1_S(VRI)           "mov " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,VRI)) ", wzr \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Duplicate/Set [regular register] scalar into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = $RR
     */
# define _ARM64_VDUPR_Q(W,DVRI,RR)      "dup " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)RR) " \n\t"
# define _ARM64_VDUPR_D(W,DVRI,RR)      "dup " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", %w" VIPM_STR_C(RR) " \n\t"


    /*
     * Duplicate/Set [vector] scalar into every element of the destination vector register
	 * (dM[0],..., dM[N]) = dK[i], i <- {0,N}
     */
# define _ARM64_VDUP_Q(W,DVRI,L,SVRI)   "dup " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI)) " \n\t"
# define _ARM64_VDUP_D(W,DVRI,L,SVRI)   "dup " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI)) " \n\t"


    /*
     * Set [regular register] scalar into one element of the destination vector register
	 * dM[k] = $RR, k <- {0,N}
	 * sM = $RR
	 */
# define _ARM64_VSET1_Q(W,L,VRI,RR)     "mov " VIPM_STR_C(_ARM64_QSCL_FMT(W,L,VRI)) ", " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)RR) " \n\t"
# define _ARM64_VSET1_D(W,L,VRI,RR)     "mov " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,VRI)) ", " VIPM_STR_C(%_ARM64_RREG_SIZE_MOD(W)RR) " \n\t"
# define _ARM64_VSET1_S(VRI,RR)         "mov " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,VRI)) ", %w" VIPM_STR_C(RR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Get [regular register] scalar from one element of the source vector register
	 *
	 * $RR = dM[i], i <- {0,N}
	 * $RR1 = dM[0], $RR2 = dM[1]
	 * $RR = sM
     */
# define _ARM64_VGET1_D(W,L,VRI,RR) \
	VIPM_STR_C(_ARM64_VGET_DATATYPE_PREFIX(W))"mov " VIPM_STR_C(%_ARM64_VGET_RREG_SIZE_MOD(W)RR) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,VRI)) " \n\t"
# define _ARM64_VGET1_D_IF(W,_Cond,L,VRI,RR)     \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_VGET1_D(W,L,VRI,RR)                   \
"11001:"
# define _ARM64_VGET1_S(VRI,RR)         "smov %x" VIPM_STR_C(RR) ", " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,VRI)) " \n\t"
# define _ARM64_VGET1_S_IF(_Cond,VRI,RR)         \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_VGET1_S(VRI,RR)                       \
"11001:"

/* ------------------------------------------------------------------------- */

    /*
     * Move between vector registers
	 */

/* Q --> Q */
/* D --> D */
/* S --> S */
# define _ARM64_VMOV_Q(DVRI,SVRI)       "mov " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI)) " \n\t"
# define _ARM64_VMOV_D(DVRI,SVRI)       "mov " VIPM_STR_C(_ARM64_DSCL_FMT(64,0,DVRI)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(64,0,SVRI)) " \n\t"
# define _ARM64_VMOV_S(DVRI,SVRI)       "mov " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,DVRI)) ", " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,SVRI)) " \n\t"

/* D,S - named variables [gcc] inline assembler */
# define _ARM64_VMOV(D,S)               "mov " VIPM_STR_C(%D) ", " VIPM_STR_C(%S) " \n\t"

# define _ARM64_VMVN_Q(DVRI,SVRI)       "mvn " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI)) " \n\t"
# define _ARM64_VMVN_D(DVRI,SVRI)       "mvn " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

/* impl.
 * @{
 */

    /*
     * Duplicate/Set immediate value into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = Imm
     */
# define _ARM64_VDUPI_Q_I(W,VRI,Imm)    "movI " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_VDUPI_Q_F(W,VRI,Imm)    _ARM64_VDUPFI_Q(W,VRI,Imm)
# define _ARM64_VDUPI_D_I(W,VRI,Imm)    "movI " VIPM_STR_C(_ARM64_DVR_FMT(W,VRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_VDUPI_D_F(W,VRI,Imm)    _ARM64_VDUPFI_D(W,VRI,Imm)

/* @} */

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VSETI_DATATYPE_PREFIX(W)    VIPM_GLUE(_ARM64_VSETI_DATATYPE_PREFIX_,W)()
# define _ARM64_VSETI_DATATYPE_PREFIX_8()
# define _ARM64_VSETI_DATATYPE_PREFIX_16()
# define _ARM64_VSETI_DATATYPE_PREFIX_32()
# define _ARM64_VSETI_DATATYPE_PREFIX_64()
# define _ARM64_VSETI_DATATYPE_PREFIX_S8()
# define _ARM64_VSETI_DATATYPE_PREFIX_S16()
# define _ARM64_VSETI_DATATYPE_PREFIX_S32()
# define _ARM64_VSETI_DATATYPE_PREFIX_S64()
# define _ARM64_VSETI_DATATYPE_PREFIX_U8()
# define _ARM64_VSETI_DATATYPE_PREFIX_U16()
# define _ARM64_VSETI_DATATYPE_PREFIX_U32()
# define _ARM64_VSETI_DATATYPE_PREFIX_U64()
# define _ARM64_VSETI_DATATYPE_PREFIX_F16() f
# define _ARM64_VSETI_DATATYPE_PREFIX_F32() f
# define _ARM64_VSETI_DATATYPE_PREFIX_F64() f

# define _ARM64_VGET_DATATYPE_PREFIX(W)     VIPM_GLUE(_ARM64_VGET_DATATYPE_PREFIX_,W)()
# define _ARM64_VGET_DATATYPE_PREFIX_8()    u
# define _ARM64_VGET_DATATYPE_PREFIX_16()   s
# define _ARM64_VGET_DATATYPE_PREFIX_32()   s
# define _ARM64_VGET_DATATYPE_PREFIX_64()   u
# define _ARM64_VGET_DATATYPE_PREFIX_S8()   s
# define _ARM64_VGET_DATATYPE_PREFIX_S16()  s
# define _ARM64_VGET_DATATYPE_PREFIX_S32()  s
# define _ARM64_VGET_DATATYPE_PREFIX_S64()  u
# define _ARM64_VGET_DATATYPE_PREFIX_U8()   u
# define _ARM64_VGET_DATATYPE_PREFIX_U16()  u
# define _ARM64_VGET_DATATYPE_PREFIX_U32()  u
# define _ARM64_VGET_DATATYPE_PREFIX_U64()  u
# define _ARM64_VGET_DATATYPE_PREFIX_F16()  f
# define _ARM64_VGET_DATATYPE_PREFIX_F32()  f
# define _ARM64_VGET_DATATYPE_PREFIX_F64()  f

# define _ARM64_VGET_RREG_SIZE_MOD(W)       VIPM_GLUE(_ARM64_VGET_RREG_SIZE_MOD_,W)()
/* @{ */
# define _ARM64_VGET_RREG_SIZE_MOD_8()      w
# define _ARM64_VGET_RREG_SIZE_MOD_16()     x
# define _ARM64_VGET_RREG_SIZE_MOD_32()     x
# define _ARM64_VGET_RREG_SIZE_MOD_64()     x

# define _ARM64_VGET_RREG_SIZE_MOD_I8()     _ARM64_VGET_RREG_SIZE_MOD_8()
# define _ARM64_VGET_RREG_SIZE_MOD_I16()    _ARM64_VGET_RREG_SIZE_MOD_16()
# define _ARM64_VGET_RREG_SIZE_MOD_I32()    _ARM64_VGET_RREG_SIZE_MOD_32()
# define _ARM64_VGET_RREG_SIZE_MOD_I64()    _ARM64_VGET_RREG_SIZE_MOD_64()
# define _ARM64_VGET_RREG_SIZE_MOD_S8()     x
# define _ARM64_VGET_RREG_SIZE_MOD_S16()    x
# define _ARM64_VGET_RREG_SIZE_MOD_S32()    x
# define _ARM64_VGET_RREG_SIZE_MOD_S64()    x
# define _ARM64_VGET_RREG_SIZE_MOD_U8()     w
# define _ARM64_VGET_RREG_SIZE_MOD_U16()    w
# define _ARM64_VGET_RREG_SIZE_MOD_U32()    w
# define _ARM64_VGET_RREG_SIZE_MOD_U64()    x
# define _ARM64_VGET_RREG_SIZE_MOD_F16()    _ARM64_VGET_RREG_SIZE_MOD_16()
# define _ARM64_VGET_RREG_SIZE_MOD_F32()    _ARM64_VGET_RREG_SIZE_MOD_32()
# define _ARM64_VGET_RREG_SIZE_MOD_F64()    _ARM64_VGET_RREG_SIZE_MOD_64()
/* @} */
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 * @{
 */

/* w - source data width
 *
 * widen:   D    --> (Q)   ;; uint8 --> int16, int16 --> int32, int32 --> int64
 * widenf:  S    --> (D)   ;; float16 --> float32, float32 --> float64
 *          D    --> (Q)
 * narrow: (Q)   -->  D    ;; int16 --> uint8, int32 --> int16, int64 --> int32
 * narrowf:(D)   -->  S    ;; float32 --> float16, float64 --> float32
 *         (Q)   -->  D
 */


# define _ARM64_VWIDEN_Q1D1(W,DVRI,SVRI) \
	VIPM_STR_C(_ARM64_VMOVL_DATATYPE_PREFIX(W))"xtL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VWIDENF_D1S1(W,DVRI,SVRI)   "fcvt " VIPM_STR_C(_ARM64_DVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VWIDENF_Q1D1(W,DVRI,SVRI) \
	"fcvtL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VNARROW_D1Q1(W,DVRI,SVRI) \
	"xtn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VNARROWF_S1D1(W,DVRI,SVRI)  "fcvt " VIPM_STR_C(_ARM64_SVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VNARROWF_D1Q1(W,DVRI,SVRI) \
	"fcvtN" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"


/* narrow with saturate: Q --> D */
# define _ARM64_VSATURATE_UU_D1Q1(W,DVRI,SVRI) \
	"uqxtn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VSATURATE_SS_D1Q1(W,DVRI,SVRI) \
	"sqxtn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VSATURATE_SU_D1Q1(W,DVRI,SVRI) \
	"sqxtun" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * vector convert signed/unsigned integer to floating point
     */
# define _ARM64_VSITOFEX_Q(W,DVRI,SVRI) "scvtf " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VSITOFEX_D(W,DVRI,SVRI) "scvtf " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VSITOFEX_S(W,DVRI,SVRI) "scvtf " VIPM_STR_C(_ARM64_SVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VUITOFEX_Q(W,DVRI,SVRI) "ucvtf " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VUITOFEX_D(W,DVRI,SVRI) "ucvtf " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VUITOFEX_S(W,DVRI,SVRI) "ucvtf " VIPM_STR_C(_ARM64_SVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(W,SVRI)) " \n\t"

	/*
     * vector convert signed/unsigned integer to double
	 * @note: SVRI must be 0-7 for Q, 0-15 for D, 0-31 for S (cause of S registers limit)
	 */
/* S --> D */
# define _ARM64_VSITOF64_D1S1(DVRI,SVRI) \
	"mov " VIPM_STR_C(_ARM64_SWTR32) ", " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,SVRI)) " \n\t" \
	"scvtf " VIPM_STR_C(_ARM64_DVR_FMT(64,DVRI)) ", " VIPM_STR_C(_ARM64_SWTR32) " \n\t"
/* D --> Q */
# define _ARM64_VSITOF64_Q1D1(DVRI,SVRI) \
    _ARM64_VWIDEN_Q1D1(S32,DVRI,SVRI)    \
    _ARM_VLITOF64_Q(DVRI,DVRI)
/* Q --> (Q,Q) */
# define _ARM64_VSITOF64_Q2Q1(DVRI0,DVRI1,SVRI) \
    _ARM_VWIDEN_Q2Q1(S32,DVRI0,DVRI1,SVRI)      \
    _ARM_VLITOF64_Q(DVRI0,DVRI0)                \
    _ARM_VLITOF64_Q(DVRI1,DVRI1)
/* S --> D */
# define _ARM64_VUITOF64_D1S1(DVRI,SVRI) \
	"mov " VIPM_STR_C(_ARM64_SWTR32) ", " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,SVRI)) " \n\t" \
	"ucvtf " VIPM_STR_C(_ARM64_DVR_FMT(64,DVRI)) ", " VIPM_STR_C(_ARM64_SWTR32) " \n\t"
/* D --> Q */
# define _ARM64_VUITOF64_Q1D1(DVRI,SVRI) \
    _ARM64_VWIDEN_Q1D1(U32,DVRI,SVRI)    \
    _ARM_VLUTOF64_Q(DVRI,DVRI)
/* Q --> (Q,Q) */
# define _ARM64_VUITOF64_Q2Q1(DVRI0,DVRI1,SVRI) \
    _ARM_VWIDEN_Q2Q1(U32,DVRI0,DVRI1,SVRI)      \
    _ARM_VLUTOF64_Q(DVRI0,DVRI0)                \
    _ARM_VLUTOF64_Q(DVRI1,DVRI1)

/* ------------------------------------------------------------------------- */

    /*
     * vector convert floating point to signed/unsigned integer with roundmode
     */
# define _ARM64_RND_VFTOSIEX_S(W,DVRI,SVRI,RndM)    "fcvt" VIPM_STR_C(RndM) "S " VIPM_STR_C(_ARM64_SVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_RND_VFTOSIEX_D(W,DVRI,SVRI,RndM) \
	"fcvt" VIPM_STR_C(RndM) "S " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_RND_VFTOSIEX_Q(W,DVRI,SVRI,RndM)    "fcvt" VIPM_STR_C(RndM) "S " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_RND_VFTOUIEX_S(W,DVRI,SVRI,RndM)    "fcvt" VIPM_STR_C(RndM) "U " VIPM_STR_C(_ARM64_SVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_RND_VFTOUIEX_D(W,DVRI,SVRI,RndM) \
	"fcvt" VIPM_STR_C(RndM) "U " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_RND_VFTOUIEX_Q(W,DVRI,SVRI,RndM)    "fcvt" VIPM_STR_C(RndM) "U " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

    /*
     * vector convert double to signed/unsigned integer with roundmode
	 * @note: DVRI must be 0-7 for Q, 0-15 for D, 0-31 for S (cause of S registers limit)
     */
/* D --> S */
# define _ARM64_RND_VF64TOSIEX_S1D1(DVRI,SVRI,RndM) \
	"fcvt" VIPM_STR_C(RndM) "S " VIPM_STR_C(_ARM64_SWTR32) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI)) " \n\t" \
	"mov " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,DVRI)) ", " VIPM_STR_C(_ARM64_SWTR32) " \n\t"
/* Q --> D */
# define _ARM64_RND_VF64TOSIEX_D1Q1(DVRI,SVRI,RndM) \
    _ARM64_RND_VFTOSIEX_Q(64,SVRI,SVRI,RndM)        \
    _ARM64_VSATURATE_SS_D1Q1(64,DVRI,SVRI)
/* (Q,Q) --> Q */
# define _ARM64_RND_VF64TOSIEX_Q1Q2(DVRI,SVRI0,SVRI1,RndM) \
    _ARM64_RND_VFTOSIEX_Q(64,SVRI0,SVRI0,RndM)             \
    _ARM64_RND_VFTOSIEX_Q(64,SVRI1,SVRI1,RndM)             \
    _ARM_VSATURATE_SS_Q1Q2(64,DVRI,SVRI0,SVRI1)
/* D --> S */
# define _ARM64_RND_VF64TOUIEX_S1D1(DVRI,SVRI,RndM) \
	"fcvt" VIPM_STR_C(RndM) "U " VIPM_STR_C(_ARM64_SWTR32) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI)) " \n\t" \
	"mov " VIPM_STR_C(_ARM64_SSCL_FMT(32,0,DVRI)) ", " VIPM_STR_C(_ARM64_SWTR32) " \n\t"
/* Q --> D */
# define _ARM64_RND_VF64TOUIEX_D1Q1(DVRI,SVRI,RndM) \
    _ARM64_RND_VFTOUIEX_Q(64,SVRI,SVRI,RndM)        \
    _ARM64_VSATURATE_UU_D1Q1(64,DVRI,SVRI)
/* (Q,Q) --> Q */
# define _ARM64_RND_VF64TOUIEX_Q1Q2(DVRI,SVRI0,SVRI1,RndM) \
    _ARM64_RND_VFTOUIEX_Q(64,SVRI0,SVRI0,RndM)             \
    _ARM64_RND_VFTOUIEX_Q(64,SVRI1,SVRI1,RndM)             \
    _ARM_VSATURATE_UU_Q1Q2(64,DVRI,SVRI0,SVRI1)


    /*
     * vector convert floating point to signed/unsigned [long] integer with roundmode
     */
/* D,S - named variables [gcc] inline assembler */
# define _ARM64_RND_VFTOSIEX(Wo,Wi,D,S,RndM)        "fcvt" VIPM_STR_C(RndM) "S " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(Wo)D) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(Wi)S) " \n\t"
# define _ARM64_RND_VFTOUIEX(Wo,Wi,D,S,RndM)        "fcvt" VIPM_STR_C(RndM) "U " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(Wo)D) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(Wi)S) " \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VMOVL_DATATYPE_PREFIX(W)    VIPM_GLUE(_ARM64_VMOVL_DATATYPE_PREFIX_,W)()
# define _ARM64_VMOVL_DATATYPE_PREFIX_8()   u
# define _ARM64_VMOVL_DATATYPE_PREFIX_16()  s
# define _ARM64_VMOVL_DATATYPE_PREFIX_32()  s
# define _ARM64_VMOVL_DATATYPE_PREFIX_64()  s
# define _ARM64_VMOVL_DATATYPE_PREFIX_S8()  s
# define _ARM64_VMOVL_DATATYPE_PREFIX_S16() s
# define _ARM64_VMOVL_DATATYPE_PREFIX_S32() s
# define _ARM64_VMOVL_DATATYPE_PREFIX_S64() s
# define _ARM64_VMOVL_DATATYPE_PREFIX_U8()  u
# define _ARM64_VMOVL_DATATYPE_PREFIX_U16() u
# define _ARM64_VMOVL_DATATYPE_PREFIX_U32() u
# define _ARM64_VMOVL_DATATYPE_PREFIX_U64() u
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * :w  - data width
 * :ww - double data width
 * for instance: w = 8, ww = 16
 *               long operation: ww = w + w
 *               wide operation: ww = ww + w
 *
 * vector negate               : r[i]:w   = -a[i]:w
 *
 * vector add                           : r[i]:w   = a[i]:w + b[i]:w
 * vector add saturated                 : r[i]:w   = saturate(a[i]:w + b[i]:w)
 * vector add long                      : r[i]:ww  = a[i]:w + b[i]:w
 * vector add wide                      : r[i]:ww  = a[i]:ww + b[i]:w
 * vector halving add                   : r[i]:w   = (a[i]:w + b[i]:w) >> 1
 * vector rounding halving add          : r[i]:w   = (a[i]:w + b[i]:w + 1) >> 1
 *
 * vector sub                           : r[i]:w   = a[i]:w - b[i]:w
 * vector sub saturated                 : r[i]:w   = saturate(a[i]:w - b[i]:w)
 * vector sub long                      : r[i]:ww  = a[i]:w - b[i]:w
 * vector sub wide                      : r[i]:ww  = a[i]:ww - b[i]:w
 * vector halving sub                   : r[i]:w   = (a[i]:w - b[i]:w) >> 1
 *
 * vector mul                           : r[i]:w   = a[i]:w * b[i]:w
 * vector mul long                      : r[i]:ww  = a[i]:w * b[i]:w
 * vector mul long saturated            : r[i]:ww  = saturate(a[i]:w * b[i]:w)
 * vector mul accumulate                : r[i]:w  += a[i]:w * b[i]:w
 * vector mul accumulate long           : r[i]:ww += a[i]:w * b[i]:w
 * vector mul accumulate saturated long : r[i]:ww = saturate(r[i]:ww + (a[i]:w * b[i]:w))
 * vector mul subtract                  : r[i]:w  -= a[i]:w * b[i]:w
 * vector mul subtract long             : r[i]:ww -= a[i]:w * b[i]:w
 * vector mul subtract saturated long   : r[i]:ww = saturate(r[i]:ww - (a[i]:w * b[i]:w))
 *
 * vector doubling mul saturated with right shift 16/32          : r = saturate((a * b * 2) >>{U} 16{32})
 * vector rounding doubling mul saturated with right shift 16/32 : r = saturate(round((a * b * 2) >>{U} 16{32}))
 */


	/*
     * VNEGATE
	 */

/* negate: Q --> Q */
/* negate: D --> D */
# define _ARM64_VNEGATE_Q(W, DVRI,SVRI) \
    "neg " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VNEGATE_D(W, DVRI,SVRI) \
    "neg " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

/* negate: Q --> Q */
/* negate: D --> D */
# define _ARM64_VNEGATEF_Q(W, DVRI,SVRI) \
    "fneg " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_VNEGATEF_D(W, DVRI,SVRI) \
    "fneg " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

/* negate saturated: Q --> Q */
/* negate saturated: D --> D */
# define _ARM64_SAT_VNEGATE_Q(W, DVRI,SVRI) \
    "sqneg " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"
# define _ARM64_SAT_VNEGATE_D(W, DVRI,SVRI) \
    "sqneg " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VADD
     */

/* add: (Q,Q) --> Q
   add: (D,D) --> D */
# define _ARM64_VADD_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"add " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VADD_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"add " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* add{float}
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM64_VADDF_Q(W, DVRI,SVRI1,SVRI2) \
    "fadd " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VADDF_D(W, DVRI,SVRI1,SVRI2) \
    "fadd " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VADDF_S(DVRI,SVRI1,SVRI2) \
    "fadd " VIPM_STR_C(_ARM64_SVR_FMT(32,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI1)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI2)) " \n\t"

/* add{double}
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM64_VADDF64_Q(DVRI,SVRI1,SVRI2) _ARM64_VADDF_Q(64,DVRI,SVRI1,SVRI2)
# define _ARM64_VADDF64_D(DVRI,SVRI1,SVRI2) _ARM64_VADDF_D(64,DVRI,SVRI1,SVRI2)

/* add long: (D,D) --> Q */
# define _ARM64_VADD_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"addL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"

/* add wide: (Q,D) --> Q */
# define _ARM64_VADD_WIDEN_Q1Q1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDW_DATATYPE_PREFIX(W))"addW" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"


/* add saturated: (Q,Q) --> Q
   add saturated: (D,D) --> D */
# define _ARM64_SAT_VADD_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qadd " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_SAT_VADD_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qadd " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* halving add */
# define _ARM64_VHADD_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VHADD_DATATYPE_PREFIX(W))"hadd " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VHADD_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VHADD_DATATYPE_PREFIX(W))"hadd " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* rounding halving add */
# define _ARM64_RND_VHADD_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VHADD_DATATYPE_PREFIX(W))"rhadd " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_RND_VHADD_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VHADD_DATATYPE_PREFIX(W))"rhadd " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM64_VADDH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "addhn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

/* rounding add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM64_RND_VADDH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "raddhn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSUB
     */

/* sub: (Q,Q) --> Q
   sub: (D,D) --> D */
# define _ARM64_VSUB_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"sub " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VSUB_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"sub " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* sub{float}
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM64_VSUBF_Q(W, DVRI,SVRI1,SVRI2) \
    "fsub " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VSUBF_D(W, DVRI,SVRI1,SVRI2) \
    "fsub " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VSUBF_S(DVRI,SVRI1,SVRI2) \
    "fsub " VIPM_STR_C(_ARM64_SVR_FMT(32,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI1)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI2)) " \n\t"

/* sub{double}
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM64_VSUBF64_Q(DVRI,SVRI1,SVRI2) _ARM64_VSUBF_Q(64,DVRI,SVRI1,SVRI2)
# define _ARM64_VSUBF64_D(DVRI,SVRI1,SVRI2) _ARM64_VSUBF_D(64,DVRI,SVRI1,SVRI2)

/* sub long: (D,D) --> Q */
# define _ARM64_VSUB_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"subL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"

/* sub wide: (Q,D) --> Q */
# define _ARM64_VSUB_WIDEN_Q1Q1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDW_DATATYPE_PREFIX(W))"subW" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"


/* sub saturated: (Q,Q) --> Q
   sub saturated: (D,D) --> D */
# define _ARM64_SAT_VSUB_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qsub " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_SAT_VSUB_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qsub " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* halving sub */
# define _ARM64_VHSUB_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VHADD_DATATYPE_PREFIX(W))"hsub " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VHSUB_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VHADD_DATATYPE_PREFIX(W))"hsub " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM64_VSUBH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "subhn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

/* rounding sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM64_RND_VSUBH_NARROW_D1Q1Q1(W, DVRI,SVRI1,SVRI2) \
    "rsubhn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VMUL
     */

/* mul: (Q,Q) --> Q
   mul: (D,D) --> D */
# define _ARM64_VMUL_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mul " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMUL_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mul " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* mulS: (Q,D[L]) --> Q
   mulS: (D,D[L]) --> D */
# define _ARM64_VMULS_Q(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mul " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"
# define _ARM64_VMULS_D(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mul " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

/* mul{float}
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM64_VMULF_Q(W, DVRI,SVRI1,SVRI2) \
    "fmul " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULF_D(W, DVRI,SVRI1,SVRI2) \
    "fmul " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULF_S(DVRI,SVRI1,SVRI2) \
    "fmul " VIPM_STR_C(_ARM64_SVR_FMT(32,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI1)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI2)) " \n\t"

/* mulS{float}
 * (q,d[L]) --> q
   (d,d[L]) --> d  */
# define _ARM64_VMULSF_Q(W, DVRI,SVRI1,L,SVRI2) \
	"fmul " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"
# define _ARM64_VMULSF_D(W, DVRI,SVRI1,L,SVRI2) \
	"fmul " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

/* mul{double}
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM64_VMULF64_Q(DVRI,SVRI1,SVRI2) _ARM64_VMULF_Q(64,DVRI,SVRI1,SVRI2)
# define _ARM64_VMULF64_D(DVRI,SVRI1,SVRI2) _ARM64_VMULF_D(64,DVRI,SVRI1,SVRI2)

/* mul long: (D,D) --> Q
   mulS long: (D,D[L]) --> Q */
# define _ARM64_VMUL_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"mulL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULS_WIDEN_Q1D1D1(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"mulL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI1)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

/* mul wide: (Q,D) --> Q
   mulS wide: (Q,D[L]) --> Q */
# define _ARM64_VMUL_WIDEN_Q1Q1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDW_DATATYPE_PREFIX(W))"mulW" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULS_WIDEN_Q1Q1D1(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADDW_DATATYPE_PREFIX(W))"mulW " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"


/* doubling mul saturated long: (D,D) --> Q; domain: {int16, int32} */
# define _ARM64_SAT_VMUL2_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"qdmulL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"


/* doubling mul saturated, storing only high half
   with right shift 16/32 : r = saturate((a * b * 2) >> 16{32}) */
# define _ARM64_SAT_VMUL2H_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qdmulh " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_SAT_VMUL2H_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qdmulh " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* rounding doubling mul saturated, storing only high half
   with right shift 16/32: r = saturate(round((a * b * 2) >> 16{32})) */
# define _ARM64_SAT_RND_VMUL2H_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qrdmulh " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_SAT_RND_VMUL2H_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VQADD_DATATYPE_PREFIX(W))"qrdmulh " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* mul accumulate/subtract: (Q,Q) --> Q
   mul accumulate/subtract: (D,D) --> D
   mulS accumulate/subtract: (Q,D[L]) --> Q
   mulS accumulate/subtract: (D,D[L]) --> D */
# define _ARM64_VMUL_ACCUM_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mla " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMUL_ACCUM_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mla " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULS_ACCUM_Q(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mla " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"
# define _ARM64_VMULS_ACCUM_D(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mla " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

# define _ARM64_VMUL_SUBTR_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mls " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMUL_SUBTR_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mls " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULS_SUBTR_Q(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mls " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"
# define _ARM64_VMULS_SUBTR_D(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"mls " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

/* mul accumulate/subtract{float}
   (q,q) --> q
   (d,d) --> d
   (q,d[L]) --> q
   (d,d[L]) --> d */
# define _ARM64_VMULF_ACCUM_Q(W, DVRI,SVRI1,SVRI2) \
    "fmla " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULF_ACCUM_D(W, DVRI,SVRI1,SVRI2) \
    "fmla " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULSF_ACCUM_Q(W, DVRI,SVRI1,L,SVRI2) \
    "fmla " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"
# define _ARM64_VMULSF_ACCUM_D(W, DVRI,SVRI1,L,SVRI2) \
    "fmla " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

# define _ARM64_VMULF_SUBTR_Q(W, DVRI,SVRI1,SVRI2) \
    "fmls " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULF_SUBTR_D(W, DVRI,SVRI1,SVRI2) \
    "fmls " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULSF_SUBTR_Q(W, DVRI,SVRI1,L,SVRI2) \
    "fmls " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"
# define _ARM64_VMULSF_SUBTR_D(W, DVRI,SVRI1,L,SVRI2) \
    "fmls " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

/* mul accumulate/subtract{double}
   (q,q) --> q
   (d,d) --> d */
# define _ARM64_VMULF64_ACCUM_Q(DVRI,SVRI1,SVRI2)   _ARM64_VMULF_ACCUM_Q(64, DVRI,SVRI1,SVRI2)
# define _ARM64_VMULF64_ACCUM_D(DVRI,SVRI1,SVRI2) \
    "fmadd " VIPM_STR_C(_ARM64_DVR_FMT(64,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI2)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,DVRI))" \n\t"

# define _ARM64_VMULF64_SUBTR_Q(DVRI,SVRI1,SVRI2)   _ARM64_VMULF_SUBTR_Q(64, DVRI,SVRI1,SVRI2)
# define _ARM64_VMULF64_SUBTR_D(DVRI,SVRI1,SVRI2) \
    "fmsub " VIPM_STR_C(_ARM64_DVR_FMT(64,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI2)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,DVRI))" \n\t"


/* mul accumulate/subtract long: (D,D) --> Q
   mulS accumulate/subtract long: (D,D[L]) --> Q */

# define _ARM64_VMUL_ACCUM_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"mlaL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULS_ACCUM_WIDEN_Q1D1D1(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"mlaL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI1)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2)) " \n\t"

# define _ARM64_VMUL_SUBTR_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"mlsL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMULS_SUBTR_WIDEN_Q1D1D1(W, DVRI,SVRI1,L,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"mlsL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI1)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,SVRI2))" \n\t"


/* doubling mul accumulate/subtract saturated long: (D,D) --> Q; domain: {int16, int32} */

# define _ARM64_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"qdmlaL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"qdmlsL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI2)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI2)) " \n\t"


/* A,X,Y - named variables [gcc] inline assembler */
# define _ARM64_VFMAEX(W,A,X,Y) \
	"fmadd " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)A) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)X) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)Y) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)A) " \n\t"
# define _ARM64_VFMSEX(W,A,X,Y) \
	"fmsub " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)A) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)X) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)Y) ", " VIPM_STR_C(%_ARM64_VREG_SIZE_MOD(W)A) " \n\t"


/* ------------------------------------------------------------------------- */

    /*
     * VDIV
     */

/* note: only for F32 */
# define _ARM64_VDIVF_Q(DVRI,SVRI1,SVRI2) \
    "fdiv " VIPM_STR_C(_ARM64_QVR_FMT(32,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(32,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(32,SVRI2)) " \n\t"
# define _ARM64_VDIVF_D(DVRI,SVRI1,SVRI2) \
    "fdiv " VIPM_STR_C(_ARM64_DVR_FMT(32,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(32,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(32,SVRI2)) " \n\t"
# define _ARM64_VDIVF_S(DVRI,SVRI1,SVRI2) \
    "fdiv " VIPM_STR_C(_ARM64_SVR_FMT(32,DVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI1)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,SVRI2)) " \n\t"

/* note: only for F64 */
# define _ARM64_VDIVF64_Q(DVRI,SVRI1,SVRI2) \
    "fdiv " VIPM_STR_C(_ARM64_QVR_FMT(64,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(64,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(64,SVRI2)) " \n\t"
# define _ARM64_VDIVF64_D(DVRI,SVRI1,SVRI2) \
    "fdiv " VIPM_STR_C(_ARM64_DVR_FMT(64,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VADD_DATATYPE_PREFIX(W)     VIPM_GLUE(_ARM64_VADD_DATATYPE_PREFIX_,W)()
# define _ARM64_VADD_DATATYPE_PREFIX_8()
# define _ARM64_VADD_DATATYPE_PREFIX_16()
# define _ARM64_VADD_DATATYPE_PREFIX_32()
# define _ARM64_VADD_DATATYPE_PREFIX_64()
# define _ARM64_VADD_DATATYPE_PREFIX_S8()
# define _ARM64_VADD_DATATYPE_PREFIX_S16()
# define _ARM64_VADD_DATATYPE_PREFIX_S32()
# define _ARM64_VADD_DATATYPE_PREFIX_S64()
# define _ARM64_VADD_DATATYPE_PREFIX_U8()
# define _ARM64_VADD_DATATYPE_PREFIX_U16()
# define _ARM64_VADD_DATATYPE_PREFIX_U32()
# define _ARM64_VADD_DATATYPE_PREFIX_U64()
# define _ARM64_VADD_DATATYPE_PREFIX_F16()  f
# define _ARM64_VADD_DATATYPE_PREFIX_F32()  f
# define _ARM64_VADD_DATATYPE_PREFIX_F64()  f

# define _ARM64_VQADD_DATATYPE_PREFIX(W)    VIPM_GLUE(_ARM64_VQADD_DATATYPE_PREFIX_,W)()
# define _ARM64_VQADD_DATATYPE_PREFIX_8()   u
# define _ARM64_VQADD_DATATYPE_PREFIX_16()  s
# define _ARM64_VQADD_DATATYPE_PREFIX_32()  s
# define _ARM64_VQADD_DATATYPE_PREFIX_64()  s
# define _ARM64_VQADD_DATATYPE_PREFIX_S8()  s
# define _ARM64_VQADD_DATATYPE_PREFIX_S16() s
# define _ARM64_VQADD_DATATYPE_PREFIX_S32() s
# define _ARM64_VQADD_DATATYPE_PREFIX_S64() s
# define _ARM64_VQADD_DATATYPE_PREFIX_U8()  u
# define _ARM64_VQADD_DATATYPE_PREFIX_U16() u
# define _ARM64_VQADD_DATATYPE_PREFIX_U32() u
# define _ARM64_VQADD_DATATYPE_PREFIX_U64() u

# define _ARM64_VADDL_DATATYPE_PREFIX(W)    VIPM_GLUE(_ARM64_VADDL_DATATYPE_PREFIX_,W)()
# define _ARM64_VADDL_DATATYPE_PREFIX_8()   u
# define _ARM64_VADDL_DATATYPE_PREFIX_16()  s
# define _ARM64_VADDL_DATATYPE_PREFIX_32()  s
# define _ARM64_VADDL_DATATYPE_PREFIX_64()  s
# define _ARM64_VADDL_DATATYPE_PREFIX_S8()  s
# define _ARM64_VADDL_DATATYPE_PREFIX_S16() s
# define _ARM64_VADDL_DATATYPE_PREFIX_S32() s
# define _ARM64_VADDL_DATATYPE_PREFIX_S64() s
# define _ARM64_VADDL_DATATYPE_PREFIX_U8()  u
# define _ARM64_VADDL_DATATYPE_PREFIX_U16() u
# define _ARM64_VADDL_DATATYPE_PREFIX_U32() u
# define _ARM64_VADDL_DATATYPE_PREFIX_U64() u

# define _ARM64_VADDW_DATATYPE_PREFIX(W)    VIPM_GLUE(_ARM64_VADDW_DATATYPE_PREFIX_,W)()
# define _ARM64_VADDW_DATATYPE_PREFIX_8()   u
# define _ARM64_VADDW_DATATYPE_PREFIX_16()  s
# define _ARM64_VADDW_DATATYPE_PREFIX_32()  s
# define _ARM64_VADDW_DATATYPE_PREFIX_64()  s
# define _ARM64_VADDW_DATATYPE_PREFIX_S8()  s
# define _ARM64_VADDW_DATATYPE_PREFIX_S16() s
# define _ARM64_VADDW_DATATYPE_PREFIX_S32() s
# define _ARM64_VADDW_DATATYPE_PREFIX_S64() s
# define _ARM64_VADDW_DATATYPE_PREFIX_U8()  u
# define _ARM64_VADDW_DATATYPE_PREFIX_U16() u
# define _ARM64_VADDW_DATATYPE_PREFIX_U32() u
# define _ARM64_VADDW_DATATYPE_PREFIX_U64() u

# define _ARM64_VHADD_DATATYPE_PREFIX(W)    VIPM_GLUE(_ARM64_VHADD_DATATYPE_PREFIX_,W)()
# define _ARM64_VHADD_DATATYPE_PREFIX_8()   u
# define _ARM64_VHADD_DATATYPE_PREFIX_16()  s
# define _ARM64_VHADD_DATATYPE_PREFIX_32()  s
# define _ARM64_VHADD_DATATYPE_PREFIX_64()  s
# define _ARM64_VHADD_DATATYPE_PREFIX_S8()  s
# define _ARM64_VHADD_DATATYPE_PREFIX_S16() s
# define _ARM64_VHADD_DATATYPE_PREFIX_S32() s
# define _ARM64_VHADD_DATATYPE_PREFIX_S64() s
# define _ARM64_VHADD_DATATYPE_PREFIX_U8()  u
# define _ARM64_VHADD_DATATYPE_PREFIX_U16() u
# define _ARM64_VHADD_DATATYPE_PREFIX_U32() u
# define _ARM64_VHADD_DATATYPE_PREFIX_U64() u
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Min/Max operations
 * @{
 *
 * r[i] = min(s1[i],s2[i])
 * r[i] = max(s1[i],s2[i])
 */

    /*
     * VMIN
     */

# define _ARM64_VMIN_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"min " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMIN_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"min " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VMINF_D(W, DVRI,SVRI1,SVRI2) \
    "fmin " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMINF_Q(W, DVRI,SVRI1,SVRI2) \
    "fmin " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VMAX
     */

# define _ARM64_VMAX_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"max " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMAX_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"max " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VMAXF_D(W, DVRI,SVRI1,SVRI2) \
    "fmax " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VMAXF_Q(W, DVRI,SVRI1,SVRI2) \
    "fmax " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VMAXMIN_DATATYPE_PREFIX(W)      VIPM_GLUE(_ARM64_VMAXMIN_DATATYPE_PREFIX_,W)()
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_8()     u
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_16()    s
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_32()    s
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_64()    s
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_S8()    s
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_S16()   s
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_S32()   s
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_S64()   s
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_U8()    u
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_U16()   u
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_U32()   u
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_U64()   u
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_F16()   f
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_F32()   f
# define _ARM64_VMAXMIN_DATATYPE_PREFIX_F64()   f
/* @} */

/* @} */

/* ========================================================================= */

/*
 * Folding operations: add, min, max
 * @{
 *
 * vector pairwise add [long] :
 *     (a,b) --> r : {r[i]} = {(a[2*i] + a[2*i + 1]),... (b[2*j] + b[2*j + 1])}
 *
 * vector pairwise add accumulate long :
 *     {r[i]} = {(r[i] + a[2*i] + a[2*i + 1]),... (r[j] + b[2*j] + b[2*j + 1])}
 *
 * vector add across vector :
 *     (a,b) --> r[0] : r[0] = (a[0] + a[1] + ... + a[n - 1])
 */

    /*
     * PADD, PADD long, PADD accumulate long, VADD, VADD long
     */

/* (q,q) --> q */
/* (d,d) --> d */
# define _ARM64_VPADD_Q(W, DVRI,SVRI1,SVRI2) \
	VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"addp " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPADD_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VADD_DATATYPE_PREFIX(W))"addp " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* q --> q(result in low half(d))
 * @note: high half can be modified
 */
# define _ARM64_VPADD_Q1(W, DVRI,SVRI)  _ARM64_VPADD_Q(W, DVRI,SVRI,SVRI)


/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM64_VPADD_WIDEN_Q(W, DVRI,SVRI) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"addLp " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VPADD_WIDEN_D(W, DVRI,SVRI) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"addLp " VIPM_STR_C(_ARM64_DVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"


/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM64_VPADD_ACCUM_WIDEN_Q(W, DVRI,SVRI) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"adaLp " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VPADD_ACCUM_WIDEN_D(W, DVRI,SVRI) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"adaLp " VIPM_STR_C(_ARM64_DVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"


/* (q,q) --> q[0] */
/* (d,d) --> d[0] */
# define _ARM64_VVADD_Q(W, DVRI,SVRI)   VIPM_GLUE(_ARM64_VVADD_Q_,_ARM_DATAMOD(W))(W, DVRI,SVRI)

# define _ARM64_VVADD_D(W, DVRI,SVRI)   VIPM_GLUE(_ARM64_VVADD_D_,_ARM_DATAMOD(W))(W, DVRI,SVRI)


/* (q{dn,dn+1}) --> q[0] */
/* (hi-d,lo-d) --> d[0] */
# define _ARM64_VVADD_WIDEN_Q(W, DVRI,SVRI) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"addLv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(_ARM_WMUL2(W)),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVADD_WIDEN_D(W, DVRI,SVRI) \
    VIPM_STR_C(_ARM64_VADDL_DATATYPE_PREFIX(W))"addLv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(_ARM_WMUL2(W)),_ARM64_DVLMAP(DVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * PMIN, VMIN
     */

# define _ARM64_VPMIN_Q(W, DVRI,SVRI1,SVRI2) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minp " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMIN_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minp " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMIN_Q1(W, DVRI,SVRI)  _ARM64_VPMIN_Q(W, DVRI,SVRI,SVRI)

/* pmin< float > */
# define _ARM64_VPMINF_Q(W, DVRI,SVRI1,SVRI2) \
	"fminp " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMINF_D(W, DVRI,SVRI1,SVRI2) \
    "fminp " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMINF_Q1(W, DVRI,SVRI)  _ARM64_VPMINF_Q(W, DVRI,SVRI,SVRI)


# define _ARM64_VVMIN_Q(W, DVRI,SVRI) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVMIN_D(W, DVRI,SVRI)   VIPM_GLUE(_ARM64_VVMIN_D_,_ARM_WIDTHOF(W))(W, DVRI,SVRI)

/* ------------------------------------------------------------------------- */

    /*
     * PMAX, VMAX
     */

# define _ARM64_VPMAX_Q(W, DVRI,SVRI1,SVRI2) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxp " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMAX_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxp " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMAX_Q1(W, DVRI,SVRI)  _ARM64_VPMAX_Q(W, DVRI,SVRI,SVRI)

/* pmax< float > */
# define _ARM64_VPMAXF_Q(W, DVRI,SVRI1,SVRI2) \
	"fmaxp " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMAXF_D(W, DVRI,SVRI1,SVRI2) \
    "fmaxp " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VPMAXF_Q1(W, DVRI,SVRI)  _ARM64_VPMAXF_Q(W, DVRI,SVRI,SVRI)


# define _ARM64_VVMAX_Q(W, DVRI,SVRI) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVMAX_D(W, DVRI,SVRI)   VIPM_GLUE(_ARM64_VVMAX_D_,_ARM_WIDTHOF(W))(W, DVRI,SVRI)

/* ------------------------------------------------------------------------- */

    /*
     * VMINMAX
	 * (Qs1,Qs2) -> (Qd1,Qd2)
	 * Qd1 = vmin(Qs1)
	 * Qd2 = vmax(Qs2)
     */

# define _ARM64_VVMINMAX_Q(W, DVRI1,DVRI2,SVRI1,SVRI2) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),DVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) " \n\t" \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),DVRI2)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VVMINMAX_D(W, DVRI1,DVRI2,SVRI1,SVRI2)  VIPM_GLUE(_ARM64_VVMINMAX_D_,_ARM_WIDTHOF(W))(W, DVRI1,DVRI2,SVRI1,SVRI2)

/* ------------------------------------------------------------------------- */

/* impl.
 * @{
 */

    /*
     * VADD
     */

# define _ARM64_VVADD_Q_I(W, DVRI,SVRI) \
	"addv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVADD_Q_F(W, DVRI,SVRI)                        \
	_ARM64_VPADD_Q(W,  DVRI,SVRI,SVRI)                         \
	_ARM64_VVADD_D_F(W, _ARM64_QDMAP(DVRI),_ARM64_QDMAP(DVRI))

# define _ARM64_VVADD_D_I(W, DVRI,SVRI) \
	"addv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVADD_D_F(W, DVRI,SVRI) VIPM_GLUE(_ARM64_VVADD_D_F,_ARM_WIDTHOF(W))(W, DVRI,SVRI)

# define _ARM64_VVADD_D_F32(W, DVRI,SVRI) \
	_ARM64_VPADD_D(W, DVRI,SVRI,SVRI)

# define _ARM64_VVADD_D_F64(W, DVRI,SVRI)

    /*
     * VMIN
     */

# define _ARM64_VVMIN_D_8(W, DVRI,SVRI) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVMIN_D_16(W, DVRI,SVRI) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVMIN_D_32(W, DVRI,SVRI) \
	_ARM64_VPMIN_D(W, DVRI,SVRI,SVRI)

# define _ARM64_VVMIN_D_64(W, DVRI,SVRI)

    /*
     * VMAX
     */

# define _ARM64_VVMAX_D_8(W, DVRI,SVRI) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVMAX_D_16(W, DVRI,SVRI) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VVMAX_D_32(W, DVRI,SVRI) \
	_ARM64_VPMAX_D(W, DVRI,SVRI,SVRI)

# define _ARM64_VVMAX_D_64(W, DVRI,SVRI)

    /*
     * VMINMAX
     */

# define _ARM64_VVMINMAX_D_8(W, DVRI1,DVRI2,SVRI1,SVRI2) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI1))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) " \n\t" \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI2))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VVMINMAX_D_16(W, DVRI1,DVRI2,SVRI1,SVRI2) \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"minv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI1))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) " \n\t" \
	VIPM_STR_C(_ARM64_VMAXMIN_DATATYPE_PREFIX(W))"maxv " VIPM_STR_C(VIPM_GLUE(_ARM64_WTMAP(W),_ARM64_DVLMAP(DVRI2))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VVMINMAX_D_32(W, DVRI1,DVRI2,SVRI1,SVRI2) \
	_ARM64_VPMIN_D(W, DVRI1,SVRI1,SVRI1)                  \
	_ARM64_VPMAX_D(W, DVRI2,SVRI2,SVRI2)

# define _ARM64_VVMINMAX_D_64(W, DVRI1,DVRI2,SVRI1,SVRI2)

/* @} */

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

    /*
     * Compare 2 vector registers
     * Op: LT, LE, EQ, GE, GT
     */

# define _ARM64_VCMPI_Q(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM64_VCMPIOP_MAP(_Op,W)) " " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VCMPI_D(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM64_VCMPIOP_MAP(_Op,W)) " " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM64_VCMP_Q(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM64_VCMPOP_MAP(_Op,W)) " " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,_ARM64_VCMP_ONEOFARGL(_Op, LHSVRI,RHSVRI))) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,_ARM64_VCMP_ONEOFARGR(_Op, LHSVRI,RHSVRI))) " \n\t"

# define _ARM64_VCMP_D(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM64_VCMPOP_MAP(_Op,W)) " " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,_ARM64_VCMP_ONEOFARGL(_Op, LHSVRI,RHSVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,_ARM64_VCMP_ONEOFARGR(_Op, LHSVRI,RHSVRI))) " \n\t"


	/*
	 * float
	 */

# define _ARM64_VCMPFI_Q(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM64_VCMPFIOP_MAP(_Op)) " " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VCMPFI_D(W,_Op, DVRI,LHSVRI,Imm) \
    "" VIPM_STR_C(_ARM64_VCMPFIOP_MAP(_Op)) " " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,LHSVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


# define _ARM64_VCMPF_Q(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM64_VCMPFOP_MAP(_Op)) " " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,_ARM64_VCMP_ONEOFARGL(_Op, LHSVRI,RHSVRI))) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,_ARM64_VCMP_ONEOFARGR(_Op, LHSVRI,RHSVRI))) " \n\t"

# define _ARM64_VCMPF_D(W,_Op, DVRI,LHSVRI,RHSVRI) \
    VIPM_STR_C(_ARM64_VCMPFOP_MAP(_Op)) " " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,_ARM64_VCMP_ONEOFARGL(_Op, LHSVRI,RHSVRI))) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,_ARM64_VCMP_ONEOFARGR(_Op, LHSVRI,RHSVRI))) " \n\t"


/*
 * Float-Point Subsystem
 *
 * 32-bit operations  -  S registers
 * 64-bit operations  -  D registers
 */

# define _ARM64_VFPCMP32(LHSVRI,RHSVRI)
# define _ARM64_VFPCMP64(LHSVRI,RHSVRI)
# define _ARM64_VFPCMP32_IF(_Cond,LHSVRI,RHSVRI) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_VFPCMP32(LHSVRI,RHSVRI)               \
"11001:"
# define _ARM64_VFPCMP64_IF(_Cond,LHSVRI,RHSVRI) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_VFPCMP64(LHSVRI,RHSVRI)               \
"11001:"

# define _ARM64_VFPCMP32_UCF(LHSVRI,RHSVRI)             "fcmp " VIPM_STR_C(_ARM64_SVR_FMT(32,LHSVRI)) ", " VIPM_STR_C(_ARM64_SVR_FMT(32,RHSVRI)) " \n\t"
# define _ARM64_VFPCMP64_UCF(LHSVRI,RHSVRI)             "fcmp " VIPM_STR_C(_ARM64_DVR_FMT(64,LHSVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(64,RHSVRI)) " \n\t"
# define _ARM64_VFPCMP32_UCF_IF(_Cond,LHSVRI,RHSVRI) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_VFPCMP32_UCF(LHSVRI,RHSVRI)               \
"11001:"
# define _ARM64_VFPCMP64_UCF_IF(_Cond,LHSVRI,RHSVRI) \
	_ARM64_GOTO_IF(_ARM_CCOND_MAP(_Cond),11001f) \
	_ARM64_VFPCMP64_UCF(LHSVRI,RHSVRI)               \
"11001:"

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VCMPOP_MAP(Op,W)    VIPM_GLUE3(_ARM64_VCMPOP_MAP_,Op,_ARM_DATAXMOD(W))()
# define _ARM64_VCMPOP_MAP_LTS()    cmgt
# define _ARM64_VCMPOP_MAP_LES()    cmge
# define _ARM64_VCMPOP_MAP_EQS()    cmeq
# define _ARM64_VCMPOP_MAP_GES()    cmge
# define _ARM64_VCMPOP_MAP_GTS()    cmgt
# define _ARM64_VCMPOP_MAP_ltS()    _ARM64_VCMPOP_MAP_LTS()
# define _ARM64_VCMPOP_MAP_leS()    _ARM64_VCMPOP_MAP_LES()
# define _ARM64_VCMPOP_MAP_eqS()    _ARM64_VCMPOP_MAP_EQS()
# define _ARM64_VCMPOP_MAP_geS()    _ARM64_VCMPOP_MAP_GES()
# define _ARM64_VCMPOP_MAP_gtS()    _ARM64_VCMPOP_MAP_GTS()
# define _ARM64_VCMPOP_MAP_LTU()    cmhi
# define _ARM64_VCMPOP_MAP_LEU()    cmhs
# define _ARM64_VCMPOP_MAP_EQU()    cmeq
# define _ARM64_VCMPOP_MAP_GEU()    cmhs
# define _ARM64_VCMPOP_MAP_GTU()    cmhi
# define _ARM64_VCMPOP_MAP_ltU()    _ARM64_VCMPOP_MAP_LTU()
# define _ARM64_VCMPOP_MAP_leU()    _ARM64_VCMPOP_MAP_LEU()
# define _ARM64_VCMPOP_MAP_eqU()    _ARM64_VCMPOP_MAP_EQU()
# define _ARM64_VCMPOP_MAP_geU()    _ARM64_VCMPOP_MAP_GEU()
# define _ARM64_VCMPOP_MAP_gtU()    _ARM64_VCMPOP_MAP_GTU()
# define _ARM64_VCMPOP_MAP_LTF()    fcmgt
# define _ARM64_VCMPOP_MAP_LEF()    fcmge
# define _ARM64_VCMPOP_MAP_EQF()    fcmeq
# define _ARM64_VCMPOP_MAP_GEF()    fcmge
# define _ARM64_VCMPOP_MAP_GTF()    fcmgt
# define _ARM64_VCMPOP_MAP_ltF()    _ARM64_VCMPOP_MAP_LTF()
# define _ARM64_VCMPOP_MAP_leF()    _ARM64_VCMPOP_MAP_LEF()
# define _ARM64_VCMPOP_MAP_eqF()    _ARM64_VCMPOP_MAP_EQF()
# define _ARM64_VCMPOP_MAP_geF()    _ARM64_VCMPOP_MAP_GEF()
# define _ARM64_VCMPOP_MAP_gtF()    _ARM64_VCMPOP_MAP_GTF()

# define _ARM64_VCMPIOP_MAP(Op,W)   VIPM_GLUE3(_ARM64_VCMPIOP_MAP_,Op,_ARM_DATAMOD(W))()
# define _ARM64_VCMPIOP_MAP_LTI()   cmlt
# define _ARM64_VCMPIOP_MAP_LEI()   cmle
# define _ARM64_VCMPIOP_MAP_EQI()   cmeq
# define _ARM64_VCMPIOP_MAP_GEI()   cmge
# define _ARM64_VCMPIOP_MAP_GTI()   cmgt
# define _ARM64_VCMPIOP_MAP_ltI()   _ARM64_VCMPIOP_MAP_LTI()
# define _ARM64_VCMPIOP_MAP_leI()   _ARM64_VCMPIOP_MAP_LEI()
# define _ARM64_VCMPIOP_MAP_eqI()   _ARM64_VCMPIOP_MAP_EQI()
# define _ARM64_VCMPIOP_MAP_geI()   _ARM64_VCMPIOP_MAP_GEI()
# define _ARM64_VCMPIOP_MAP_gtI()   _ARM64_VCMPIOP_MAP_GTI()
# define _ARM64_VCMPIOP_MAP_LTF()   fcmlt
# define _ARM64_VCMPIOP_MAP_LEF()   fcmle
# define _ARM64_VCMPIOP_MAP_EQF()   fcmeq
# define _ARM64_VCMPIOP_MAP_GEF()   fcmge
# define _ARM64_VCMPIOP_MAP_GTF()   fcmgt
# define _ARM64_VCMPIOP_MAP_ltF()   _ARM64_VCMPIOP_MAP_LTF()
# define _ARM64_VCMPIOP_MAP_leF()   _ARM64_VCMPIOP_MAP_LEF()
# define _ARM64_VCMPIOP_MAP_eqF()   _ARM64_VCMPIOP_MAP_EQF()
# define _ARM64_VCMPIOP_MAP_geF()   _ARM64_VCMPIOP_MAP_GEF()
# define _ARM64_VCMPIOP_MAP_gtF()   _ARM64_VCMPIOP_MAP_GTF()

# define _ARM64_VCMPFOP_MAP(Op)     VIPM_GLUE3(_ARM64_VCMPOP_MAP_,Op,F)()
# define _ARM64_VCMPFIOP_MAP(Op)    VIPM_GLUE3(_ARM64_VCMPIOP_MAP_,Op,F)()

# define _ARM64_VCMP_ONEOFARGL(Op,LHSVRI,RHSVRI)    VIPM_GLUE(_ARM64_VCMP_ONEOFARGL_,Op)(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGL_LT(LHSVRI,RHSVRI)    RHSVRI
# define _ARM64_VCMP_ONEOFARGL_LE(LHSVRI,RHSVRI)    RHSVRI
# define _ARM64_VCMP_ONEOFARGL_EQ(LHSVRI,RHSVRI)    LHSVRI
# define _ARM64_VCMP_ONEOFARGL_GE(LHSVRI,RHSVRI)    LHSVRI
# define _ARM64_VCMP_ONEOFARGL_GT(LHSVRI,RHSVRI)    LHSVRI
# define _ARM64_VCMP_ONEOFARGL_lt(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGL_LT(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGL_le(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGL_LE(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGL_eq(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGL_EQ(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGL_ge(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGL_GE(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGL_gt(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGL_GT(LHSVRI,RHSVRI)

# define _ARM64_VCMP_ONEOFARGR(Op,LHSVRI,RHSVRI)    VIPM_GLUE(_ARM64_VCMP_ONEOFARGR_,Op)(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGR_LT(LHSVRI,RHSVRI)    LHSVRI
# define _ARM64_VCMP_ONEOFARGR_LE(LHSVRI,RHSVRI)    LHSVRI
# define _ARM64_VCMP_ONEOFARGR_EQ(LHSVRI,RHSVRI)    RHSVRI
# define _ARM64_VCMP_ONEOFARGR_GE(LHSVRI,RHSVRI)    RHSVRI
# define _ARM64_VCMP_ONEOFARGR_GT(LHSVRI,RHSVRI)    RHSVRI
# define _ARM64_VCMP_ONEOFARGR_lt(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGR_LT(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGR_le(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGR_LE(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGR_eq(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGR_EQ(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGR_ge(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGR_GE(LHSVRI,RHSVRI)
# define _ARM64_VCMP_ONEOFARGR_gt(LHSVRI,RHSVRI)    _ARM64_VCMP_ONEOFARGR_GT(LHSVRI,RHSVRI)
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * vector not
 * vector and
 * vector and complement: BIC
 * vector or
 * vector or complement: ORN
 * vector xor
 * vector set all bits
 * vector clear all bits
 * vector bit select           : bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
 */

    /*
     * AND
     */

# define _ARM64_VAND_Q(W, DVRI,SVRI1,SVRI2) \
    "and " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VAND_D(W, DVRI,SVRI1,SVRI2) \
    "and " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"

    /*
     * BIC : AND complement
     */

# define _ARM64_VBIC_Q(W, DVRI,SVRI1,SVRI2) \
    "bic " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VBIC_D(W, DVRI,SVRI1,SVRI2) \
    "bic " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"

    /*
     * OR
     */

# define _ARM64_VORR_Q(W, DVRI,SVRI1,SVRI2) \
    "orr " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VORR_D(W, DVRI,SVRI1,SVRI2) \
    "orr " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"

    /*
     * ORN : OR complement
     */

# define _ARM64_VORN_Q(W, DVRI,SVRI1,SVRI2) \
    "orn " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VORN_D(W, DVRI,SVRI1,SVRI2) \
    "orn " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"

    /*
     * XOR
     */

# define _ARM64_VXOR_Q(W, DVRI,SVRI1,SVRI2) \
    "eor " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VXOR_D(W, DVRI,SVRI1,SVRI2) \
    "eor " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"


    /*
     * Set/Clear all bits
     */

# define _ARM64_VBITSET_Q(DVRI) "orn " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) " \n\t"
# define _ARM64_VBITSET_D(DVRI) "orn " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) " \n\t"
# define _ARM64_VBITCLR_Q(DVRI) "bic " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) " \n\t"
# define _ARM64_VBITCLR_D(DVRI) "bic " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) " \n\t"


    /*
     * Bit select: bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
     * Bit set if true:
     * Bit set if false:
     */

# define _ARM64_VBITSEL_Q(W, DVRI,SVRI1,SVRI2) \
    "bsl " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VBITSEL_D(W, DVRI,SVRI1,SVRI2) \
    "bsl " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"


# define _ARM64_VBITIFT_Q(W, DVRI,SVRI1,SVRI2) \
    "bit " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VBITIFT_D(W, DVRI,SVRI1,SVRI2) \
    "bit " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"


# define _ARM64_VBITIFF_Q(W, DVRI,SVRI1,SVRI2) \
    "bif " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) " \n\t"

# define _ARM64_VBITIFF_D(W, DVRI,SVRI1,SVRI2) \
    "bif " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) " \n\t"


    /*
     * Bit count leading 0/1:
     */

# define _ARM64_VCOUNTLEADZEROBITS_Q(W, DVRI,SVRI) \
    "clz " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VCOUNTLEADZEROBITS_D(W, DVRI,SVRI) \
    "clz " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"


# define _ARM64_VCOUNTLEADSETBITS_Q(W, DVRI,SVRI) \
    "cls " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) " \n\t"

# define _ARM64_VCOUNTLEADSETBITS_D(W, DVRI,SVRI) \
    "cls " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) " \n\t"

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * vector shift left by immediate
 * vector shift left saturated by immediate
 * vector shift left long by immediate
 *
 * (negative shift-value shift right)
 * vector shift left by signed var
 * vector rounding shift left by signed
 * vector shift left saturated by signed
 * vector rounding shift left saturated by signed
 *
 * vector [rounding] shift right by immediate
 * vector [rounding] shift right narrow by immediate
 * vector saturating [rounding] shift right narrow by immediate
 */

    /*
     * VSHL by immediate
     */

# define _ARM64_VSHLI_Q(W, DVRI,SVRI,Imm) \
    "shl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHLI_D(W, DVRI,SVRI,Imm) \
    "shl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* shift saturated */
# define _ARM64_SAT_VSHLI_Q(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_SAT_VSHLI_D(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift saturated */
# define _ARM64_SAT_VSHLSI_Q(W, DVRI,SVRI,Imm) \
    "sqshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_SAT_VSHLSI_D(W, DVRI,SVRI,Imm) \
    "sqshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift saturated */
# define _ARM64_SAT_VSHLUI_Q(W, DVRI,SVRI,Imm) \
    "uqshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_SAT_VSHLUI_D(W, DVRI,SVRI,Imm) \
    "uqshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/*
 * shift long
 */

# define _ARM64_VSHLI_WIDEN_Q1D1(W, DVRI,SVRI,Imm)      VIPM_GLUE(_ARM64_VSHLI_WIDEN_Q1D1_,_ARM64_IF_EQWI(_ARM_WIDTHOF(W),Imm))(W, DVRI,SVRI,Imm)

/* signed shift long */
# define _ARM64_VSHLSI_WIDEN_Q1D1(W, DVRI,SVRI,Imm)     VIPM_GLUE(_ARM64_VSHLSI_WIDEN_Q1D1_,_ARM64_IF_EQWI(_ARM_WIDTHOF(W),Imm))(W, DVRI,SVRI,Imm)

/* unsigned shift long */
# define _ARM64_VSHLUI_WIDEN_Q1D1(W, DVRI,SVRI,Imm)     VIPM_GLUE(_ARM64_VSHLUI_WIDEN_Q1D1_,_ARM64_IF_EQWI(_ARM_WIDTHOF(W),Imm))(W, DVRI,SVRI,Imm)

/* ------------------------------------------------------------------------- */

    /*
     * VSHL by signed var (negative shift-value shift right)
     */

/* shift left */
# define _ARM64_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"shl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"shl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* signed shift left */
# define _ARM64_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "sshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "sshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* unsigned shift left */
# define _ARM64_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "ushl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "ushl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* shift left saturated */
# define _ARM64_SAT_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_SAT_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* signed shift left saturated */
# define _ARM64_SAT_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "sqshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_SAT_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "sqshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* unsigned shift left saturated */
# define _ARM64_SAT_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "uqshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_SAT_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "uqshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* rounding shift left */
# define _ARM64_RND_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"rshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_RND_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"rshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* signed rounding shift left */
# define _ARM64_RND_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "srshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_RND_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "srshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* unsigned rounding shift left */
# define _ARM64_RND_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "urshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_RND_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "urshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"


/* rounding shift left saturated */
# define _ARM64_SAT_RND_VSHL_Q(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qrshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_SAT_RND_VSHL_D(W, DVRI,SVRI1,SVRI2) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qrshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* signed rounding shift left saturated */
# define _ARM64_SAT_RND_VSHLS_Q(W, DVRI,SVRI1,SVRI2) \
    "sqrshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_SAT_RND_VSHLS_D(W, DVRI,SVRI1,SVRI2) \
    "sqrshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* unsigned rounding shift left saturated */
# define _ARM64_SAT_RND_VSHLU_Q(W, DVRI,SVRI1,SVRI2) \
    "uqrshl " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"

# define _ARM64_SAT_RND_VSHLU_D(W, DVRI,SVRI1,SVRI2) \
    "uqrshl " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSHR by immediate
     */

/* shift right */
# define _ARM64_VSHRI_Q(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"shr " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHRI_D(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"shr " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift right */
# define _ARM64_VSHRSI_Q(W, DVRI,SVRI,Imm) \
    "sshr " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHRSI_D(W, DVRI,SVRI,Imm) \
    "sshr " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift right */
# define _ARM64_VSHRUI_Q(W, DVRI,SVRI,Imm) \
    "ushr " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHRUI_D(W, DVRI,SVRI,Imm) \
    "ushr " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* shift right */
# define _ARM64_RND_VSHRI_Q(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"rshr " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_RND_VSHRI_D(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"rshr " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift right */
# define _ARM64_RND_VSHRSI_Q(W, DVRI,SVRI,Imm) \
    "srshr " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_RND_VSHRSI_D(W, DVRI,SVRI,Imm) \
    "srshr " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift right */
# define _ARM64_RND_VSHRUI_Q(W, DVRI,SVRI,Imm) \
    "urshr " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_RND_VSHRUI_D(W, DVRI,SVRI,Imm) \
    "urshr " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* [signed/unsigned] shift right narrow */
/* narrow: Q --> D */
# define _ARM64_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)     VIPM_GLUE(_ARM64_VSHRI_NARROW_D1Q1_,_ARM64_IF_EQWI(0,Imm))(W, DVRI,SVRI,Imm)
//# define _ARM64_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM64_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)
//# define _ARM64_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM64_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)


/* [signed/unsigned] rounding shift right narrow */
/* narrow: Q --> D */
# define _ARM64_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm) VIPM_GLUE(_ARM64_RND_VSHRI_NARROW_D1Q1_,_ARM64_IF_EQWI(0,Imm))(W, DVRI,SVRI,Imm)
//# define _ARM64_RND_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM64_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)
//# define _ARM64_RND_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm)   _ARM64_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm)


/* [signed/unsigned] shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM64_SAT_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qshrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_SAT_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "sqshrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_SAT_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "uqshrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* [signed/unsigned] rounding shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM64_SAT_RND_VSHRI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"qrshrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_SAT_RND_VSHRSI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "sqrshrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"
# define _ARM64_SAT_RND_VSHRUI_NARROW_D1Q1(W, DVRI,SVRI,Imm) \
    "uqrshrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSHR and Accumulate by immediate
     */

/* shift right */
# define _ARM64_VSHRADDI_Q(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"sra " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHRADDI_D(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"sra " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift right */
# define _ARM64_VSHRADDSI_Q(W, DVRI,SVRI,Imm) \
    "ssra " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHRADDSI_D(W, DVRI,SVRI,Imm) \
    "ssra " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift right */
# define _ARM64_VSHRADDUI_Q(W, DVRI,SVRI,Imm) \
    "usra " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHRADDUI_D(W, DVRI,SVRI,Imm) \
    "usra " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"


/* rounding shift right */
# define _ARM64_RND_VSHRADDI_Q(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"rsra " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_RND_VSHRADDI_D(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"rsra " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed rounding shift right */
# define _ARM64_RND_VSHRADDSI_Q(W, DVRI,SVRI,Imm) \
    "srsra " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_RND_VSHRADDSI_D(W, DVRI,SVRI,Imm) \
    "srsra " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned rounding shift right */
# define _ARM64_RND_VSHRADDUI_Q(W, DVRI,SVRI,Imm) \
    "ursra " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_RND_VSHRADDUI_D(W, DVRI,SVRI,Imm) \
    "ursra " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* ------------------------------------------------------------------------- */

/* impl.
 * @{
 */

/*
 * shift long
 */

# define _ARM64_VSHLI_WIDEN_Q1D1_0(W, DVRI,SVRI,Imm) \
    VIPM_STR_C(_ARM64_VSH_DATATYPE_PREFIX(W))"shlL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

# define _ARM64_VSHLI_WIDEN_Q1D1_1(W, DVRI,SVRI,Imm) \
    "shlL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift long */
# define _ARM64_VSHLSI_WIDEN_Q1D1_0(W, DVRI,SVRI,Imm) \
    "sshlL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* signed shift long */
# define _ARM64_VSHLSI_WIDEN_Q1D1_1(W, DVRI,SVRI,Imm)       _ARM64_VSHLI_WIDEN_Q1D1_1(W, DVRI,SVRI,Imm)

/* unsigned shift long */
# define _ARM64_VSHLUI_WIDEN_Q1D1_0(W, DVRI,SVRI,Imm) \
    "ushlL" VIPM_STR_C(_ARM64_VHALF_SUFFIX(SVRI)) " " VIPM_STR_C(_ARM64_QVR_FMT(_ARM_WMUL2(W),DVRI)) ", " VIPM_STR_C(_ARM64_XDVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* unsigned shift long */
# define _ARM64_VSHLUI_WIDEN_Q1D1_1(W, DVRI,SVRI,Imm)       _ARM64_VSHLI_WIDEN_Q1D1_1(W, DVRI,SVRI,Imm)


/* [signed/unsigned] shift right narrow */
/* narrow: Q --> D */
# define _ARM64_VSHRI_NARROW_D1Q1_0(W, DVRI,SVRI,Imm) \
    "shrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* [signed/unsigned] shift right narrow */
/* narrow: Q --> D */
# define _ARM64_VSHRI_NARROW_D1Q1_1(W, DVRI,SVRI,Imm)       _ARM64_VNARROW_D1Q1(W, DVRI,SVRI)


/* [signed/unsigned] rounding shift right narrow */
/* narrow: Q --> D */
# define _ARM64_RND_VSHRI_NARROW_D1Q1_0(W, DVRI,SVRI,Imm) \
    "rshrn" VIPM_STR_C(_ARM64_VHALF_SUFFIX(DVRI)) " " VIPM_STR_C(_ARM64_XDVR_FMT(_ARM_WDIV2(W),DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI)) ", #" VIPM_STR_C(Imm) " \n\t"

/* [signed/unsigned] rounding shift right narrow */
/* narrow: Q --> D */
# define _ARM64_RND_VSHRI_NARROW_D1Q1_1(W, DVRI,SVRI,Imm)   _ARM64_VNARROW_D1Q1(W, DVRI,SVRI)

/* @} */

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VSH_DATATYPE_PREFIX(W)      VIPM_GLUE(_ARM64_VSH_DATATYPE_PREFIX_,W)()
# define _ARM64_VSH_DATATYPE_PREFIX_8()     u
# define _ARM64_VSH_DATATYPE_PREFIX_16()    s
# define _ARM64_VSH_DATATYPE_PREFIX_32()    s
# define _ARM64_VSH_DATATYPE_PREFIX_64()    s
# define _ARM64_VSH_DATATYPE_PREFIX_S8()    s
# define _ARM64_VSH_DATATYPE_PREFIX_S16()   s
# define _ARM64_VSH_DATATYPE_PREFIX_S32()   s
# define _ARM64_VSH_DATATYPE_PREFIX_S64()   s
# define _ARM64_VSH_DATATYPE_PREFIX_U8()    u
# define _ARM64_VSH_DATATYPE_PREFIX_U16()   u
# define _ARM64_VSH_DATATYPE_PREFIX_U32()   u
# define _ARM64_VSH_DATATYPE_PREFIX_U64()   u
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Generic data processing operations
 * @{
 *
 * vector element shift left by immediate
 * vector element shift right by immediate
 * vector element rotate left by immediate
 * vector element rotate right by immediate
 *
 * interleaves the elements of two vector (zip)
 * de-interleaves the elements of two vector (unzip)
 */

    /*
     * VEXTRACT
     */

# define _ARM64_VEXTRACT_Q(W,DVRI,SVRI1,SVRI2,Imm) \
    "ext " VIPM_STR_C(_ARM64_QVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(8,SVRI2)) ", #" VIPM_STR_C(_ARM64_VEXT_IMAP(W,Imm)) " \n\t"

# define _ARM64_VEXTRACT_D(W,DVRI,SVRI1,SVRI2,Imm) \
    "ext " VIPM_STR_C(_ARM64_DVR_FMT(8,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(8,SVRI2)) ", #" VIPM_STR_C(_ARM64_VEXT_IMAP(W,Imm)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VZIP/VUZP (zip/unzip)
     */

/*
 * vector elements: {0,1,2,3,4,5,6,7}
 *
 * zip1(d1,s1,s2):
 * zip2(d2,s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    d1: {a0,b0,a1,b1,a2,b2,a3,b3};
 *    d2: {a4,b4,a5,b5,a6,b6,a7,b7};
 *
 * unzip1(d1,s1,s2):
 * unzip2(d2,s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    d1: {a0,a2,a4,a6,b0,b2,b4,b6};
 *    d2: {a1,a3,a5,a7,b1,b3,b5,b7};
 */
# define _ARM64_VZIP1_Q(W,DVRI,SVRI1,SVRI2) "zip1 " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VZIP2_Q(W,DVRI,SVRI1,SVRI2) "zip2 " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VZIP1_D(W,DVRI,SVRI1,SVRI2) "zip1 " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VZIP2_D(W,DVRI,SVRI1,SVRI2) "zip2 " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VUZP1_Q(W,DVRI,SVRI1,SVRI2) "uzp1 " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VUZP2_Q(W,DVRI,SVRI1,SVRI2) "uzp2 " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VUZP1_D(W,DVRI,SVRI1,SVRI2) "uzp1 " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VUZP2_D(W,DVRI,SVRI1,SVRI2) "uzp2 " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VTRANSPOSE2x2
     */
# define _ARM64_VTRANSPOSE1_Q(W,DVRI,SVRI1,SVRI2)   "trn1 " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VTRANSPOSE2_Q(W,DVRI,SVRI1,SVRI2)   "trn2 " VIPM_STR_C(_ARM64_QVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VTRANSPOSE1_D(W,DVRI,SVRI1,SVRI2)   "trn1 " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"
# define _ARM64_VTRANSPOSE2_D(W,DVRI,SVRI1,SVRI2)   "trn2 " VIPM_STR_C(_ARM64_DVR_FMT(W,DVRI)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI1)) ", " VIPM_STR_C(_ARM64_DVR_FMT(W,SVRI2)) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * VSWAP
     */
# define _ARM64_VSWAP_Q(DVRI,SVRI)  \
	_ARM64_VXOR_Q(8,DVRI,DVRI,SVRI) \
	_ARM64_VXOR_Q(8,SVRI,SVRI,DVRI) \
	_ARM64_VXOR_Q(8,DVRI,DVRI,SVRI)
# define _ARM64_VSWAP_D(DVRI,SVRI)  \
	_ARM64_VXOR_D(8,DVRI,DVRI,SVRI) \
	_ARM64_VXOR_D(8,SVRI,SVRI,DVRI) \
	_ARM64_VXOR_D(8,DVRI,DVRI,SVRI)

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VEXT_IMAP(W,Imm)    VIPM_GLUE3(_ARM64_VEXT_IMAP_,W,Imm)()
# define _ARM64_VEXT_IMAP_80()      0
# define _ARM64_VEXT_IMAP_81()      1
# define _ARM64_VEXT_IMAP_82()      2
# define _ARM64_VEXT_IMAP_83()      3
# define _ARM64_VEXT_IMAP_84()      4
# define _ARM64_VEXT_IMAP_85()      5
# define _ARM64_VEXT_IMAP_86()      6
# define _ARM64_VEXT_IMAP_87()      7
# define _ARM64_VEXT_IMAP_88()      8
# define _ARM64_VEXT_IMAP_89()      9
# define _ARM64_VEXT_IMAP_810()     10
# define _ARM64_VEXT_IMAP_811()     11
# define _ARM64_VEXT_IMAP_812()     12
# define _ARM64_VEXT_IMAP_813()     13
# define _ARM64_VEXT_IMAP_814()     14
# define _ARM64_VEXT_IMAP_815()     15
# define _ARM64_VEXT_IMAP_160()     0
# define _ARM64_VEXT_IMAP_161()     2
# define _ARM64_VEXT_IMAP_162()     4
# define _ARM64_VEXT_IMAP_163()     6
# define _ARM64_VEXT_IMAP_164()     8
# define _ARM64_VEXT_IMAP_165()     10
# define _ARM64_VEXT_IMAP_166()     12
# define _ARM64_VEXT_IMAP_167()     14
# define _ARM64_VEXT_IMAP_320()     0
# define _ARM64_VEXT_IMAP_321()     4
# define _ARM64_VEXT_IMAP_322()     8
# define _ARM64_VEXT_IMAP_323()     12
# define _ARM64_VEXT_IMAP_640()     0
# define _ARM64_VEXT_IMAP_641()     8
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 *
 * vector load multiple N-element structures into one or more registers (with de-interleaving when N>1)
 * vector store multiple N-element structures from one or more registers (with interleaving when N>1)
 *
 * vector load single N-element structure into one or more registers (with de-interleaving when N>1)
 * (Multiple copies of one n-element structure are loaded into one ore more registers).
 */


/*
 * Push/Pop
 */

	/*
     * Push/Pop fdd registers
     */

# define _ARM64_VPUSH_ALLFDD()      \
	"stp  d0, d1, [sp, -256]! \n\t" \
	"stp  d2, d3, [sp,  16] \n\t"   \
	"stp  d4, d5, [sp,  32] \n\t"   \
	"stp  d6, d7, [sp,  48] \n\t"   \
	"stp  d8, d9, [sp,  64] \n\t"   \
	"stp d10,d11, [sp,  80] \n\t"   \
	"stp d12,d13, [sp,  96] \n\t"   \
	"stp d14,d15, [sp, 112] \n\t"   \
	"stp d16,d17, [sp, 128] \n\t"   \
	"stp d18,d19, [sp, 144] \n\t"   \
	"stp d20,d21, [sp, 160] \n\t"   \
	"stp d22,d23, [sp, 176] \n\t"   \
	"stp d24,d25, [sp, 192] \n\t"   \
	"stp d26,d27, [sp, 208] \n\t"   \
	"stp d28,d29, [sp, 224] \n\t"   \
	"stp d30,d31, [sp, 240] \n\t"
/* @sa: ARM64 vector registers */
# define _ARM64_VPUSH_AVAFDD()     \
	"stp  d8, d9, [sp, -64]! \n\t" \
	"stp d10,d11, [sp, 16] \n\t"   \
	"stp d12,d13, [sp, 32] \n\t"   \
	"stp d14,d15, [sp, 48] \n\t"
# define _ARM64_VPUSH_FDD1(VRI)         "str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI))) ", [sp, -16]! \n\t"
# define _ARM64_VPUSH_FDD2(VRI0,VRI1) \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -16]! \n\t"
# define _ARM64_VPUSH_FDD3(VRI0,VRI1,VRI2)                                                                                   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -32]! \n\t" \
	"str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) ", [sp, 16] \n\t"
# define _ARM64_VPUSH_FDD4(VRI0,VRI1,VRI2,VRI3)                                                                              \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -32]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"
# define _ARM64_VPUSH_FDD5(VRI0,VRI1,VRI2,VRI3,VRI4)                                                                         \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -48]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) ", [sp, 32] \n\t"
# define _ARM64_VPUSH_FDD6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5)                                                                    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -48]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"
# define _ARM64_VPUSH_FDD7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6)                                                               \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -64]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) ", [sp, 48] \n\t"
# define _ARM64_VPUSH_FDD8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7)                                                          \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -64]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"
# define _ARM64_VPUSH_FDD9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8)                                                     \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -80]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) ", [sp, 64] \n\t"
# define _ARM64_VPUSH_FDD10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9)                                               \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -80]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"
# define _ARM64_VPUSH_FDD11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10)                                         \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -96]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"   \
	"str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) ", [sp, 80] \n\t"
# define _ARM64_VPUSH_FDD12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11)                                   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -96]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t"
# define _ARM64_VPUSH_FDD13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12)                              \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -112]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t"  \
	"str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) ", [sp, 96] \n\t"
# define _ARM64_VPUSH_FDD14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13)                        \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -112]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t"  \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI13))) ", [sp, 96] \n\t"
# define _ARM64_VPUSH_FDD15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14)                  \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -128]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t"  \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI13))) ", [sp, 96] \n\t"  \
	"str " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI14))) ", [sp, 112] \n\t"
# define _ARM64_VPUSH_FDD16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15)            \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp, -128]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t"  \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI13))) ", [sp, 96] \n\t"  \
	"stp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI14))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI15))) ", [sp, 112] \n\t"

# define _ARM64_VPOP_ALLFDD()     \
	"ldp  d2, d3, [sp,  16] \n\t" \
	"ldp  d4, d5, [sp,  32] \n\t" \
	"ldp  d6, d7, [sp,  48] \n\t" \
	"ldp  d8, d9, [sp,  64] \n\t" \
	"ldp d10,d11, [sp,  80] \n\t" \
	"ldp d12,d13, [sp,  96] \n\t" \
	"ldp d14,d15, [sp, 112] \n\t" \
	"ldp d16,d17, [sp, 128] \n\t" \
	"ldp d18,d19, [sp, 144] \n\t" \
	"ldp d20,d21, [sp, 160] \n\t" \
	"ldp d22,d23, [sp, 176] \n\t" \
	"ldp d24,d25, [sp, 192] \n\t" \
	"ldp d26,d27, [sp, 208] \n\t" \
	"ldp d28,d29, [sp, 224] \n\t" \
	"ldp d30,d31, [sp, 240] \n\t" \
	"ldp  d0, d1, [sp], 256 \n\t"
/* @sa: ARM64 vector registers */
# define _ARM64_VPOP_AVAFDD()    \
	"ldp d10,d11, [sp, 16] \n\t" \
	"ldp d12,d13, [sp, 32] \n\t" \
	"ldp d14,d15, [sp, 48] \n\t" \
	"ldp  d8, d9, [sp], 64 \n\t"
# define _ARM64_VPOP_FDD1(VRI)         "ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI))) ", [sp], 16 \n\t"
# define _ARM64_VPOP_FDD2(VRI0,VRI1) \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 16 \n\t"
# define _ARM64_VPOP_FDD3(VRI0,VRI1,VRI2)                                 \
	"ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 32 \n\t"
# define _ARM64_VPOP_FDD4(VRI0,VRI1,VRI2,VRI3)                                                                             \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 32 \n\t"
# define _ARM64_VPOP_FDD5(VRI0,VRI1,VRI2,VRI3,VRI4)                                                                        \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) ", [sp, 32] \n\t"                                                  \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 48 \n\t"
# define _ARM64_VPOP_FDD6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5)                                                                   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 48 \n\t"
# define _ARM64_VPOP_FDD7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6)                                                              \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) ", [sp, 48] \n\t"                                                  \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 64 \n\t"
# define _ARM64_VPOP_FDD8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7)                                                         \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 64 \n\t"
# define _ARM64_VPOP_FDD9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8)                                                    \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) ", [sp, 64] \n\t"                                                  \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 80 \n\t"
# define _ARM64_VPOP_FDD10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9)                                              \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 80 \n\t"
# define _ARM64_VPOP_FDD11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10)                                        \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) ", [sp, 80] \n\t"                                                 \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 96 \n\t"
# define _ARM64_VPOP_FDD12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11)                                    \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 96 \n\t"
# define _ARM64_VPOP_FDD13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12)                              \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) ", [sp, 96] \n\t"                                                   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 112 \n\t"
# define _ARM64_VPOP_FDD14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13)                        \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI13))) ", [sp, 96] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 112 \n\t"
# define _ARM64_VPOP_FDD15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14)                  \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"   \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI13))) ", [sp, 96] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI14))) ", [sp, 112] \n\t"                                                  \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 128 \n\t"
# define _ARM64_VPOP_FDD16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15)             \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI2))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI3))) ", [sp, 16] \n\t"    \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI4))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI5))) ", [sp, 32] \n\t"    \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI6))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI7))) ", [sp, 48] \n\t"    \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI8))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI9))) ", [sp, 64] \n\t"    \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI10))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI11))) ", [sp, 80] \n\t"  \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI12))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI13))) ", [sp, 96] \n\t"  \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI14))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI15))) ", [sp, 112] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI0))) "," VIPM_STR_C(VIPM_GLUE(d,_ARM64_DVLMAP(VRI1))) ", [sp], 128 \n\t"

	/*
     * Push/Pop vector registers
     */

# define _ARM64_VPUSH_ALL()         \
	"stp  q0, q1, [sp, -512]! \n\t" \
	"stp  q2, q3, [sp,  32] \n\t"   \
	"stp  q4, q5, [sp,  64] \n\t"   \
	"stp  q6, q7, [sp,  96] \n\t"   \
	"stp  q8, q9, [sp, 128] \n\t"   \
	"stp q10,q11, [sp, 160] \n\t"   \
	"stp q12,q13, [sp, 192] \n\t"   \
	"stp q14,q15, [sp, 224] \n\t"   \
	"stp q16,q17, [sp, 256] \n\t"   \
	"stp q18,q19, [sp, 288] \n\t"   \
	"stp q20,q21, [sp, 320] \n\t"   \
	"stp q22,q23, [sp, 352] \n\t"   \
	"stp q24,q25, [sp, 384] \n\t"   \
	"stp q26,q27, [sp, 416] \n\t"   \
	"stp q28,q29, [sp, 448] \n\t"   \
	"stp q30,q31, [sp, 480] \n\t"

# define _ARM64_VPUSH_D1(VRI)           _ARM64_VPUSH_FDD1(VRI)
# define _ARM64_VPUSH_D2(VRI0,VRI1)     _ARM64_VPUSH_FDD2(VRI0,VRI1)
# define _ARM64_VPUSH_D3(VRI0,VRI1,VRI2) \
	_ARM64_VPUSH_FDD3(VRI0,VRI1,VRI2)
# define _ARM64_VPUSH_D4(VRI0,VRI1,VRI2,VRI3) \
	_ARM64_VPUSH_FDD4(VRI0,VRI1,VRI2,VRI3)
# define _ARM64_VPUSH_D5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	_ARM64_VPUSH_FDD5(VRI0,VRI1,VRI2,VRI3,VRI4)
# define _ARM64_VPUSH_D6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	_ARM64_VPUSH_FDD6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5)
# define _ARM64_VPUSH_D7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	_ARM64_VPUSH_FDD7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6)
# define _ARM64_VPUSH_D8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	_ARM64_VPUSH_FDD8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7)
# define _ARM64_VPUSH_D9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8) \
	_ARM64_VPUSH_FDD9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8)
# define _ARM64_VPUSH_D10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9) \
	_ARM64_VPUSH_FDD10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9)
# define _ARM64_VPUSH_D11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10) \
	_ARM64_VPUSH_FDD11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10)
# define _ARM64_VPUSH_D12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11) \
	_ARM64_VPUSH_FDD12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11)
# define _ARM64_VPUSH_D13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12) \
	_ARM64_VPUSH_FDD13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12)
# define _ARM64_VPUSH_D14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13) \
	_ARM64_VPUSH_FDD14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13)
# define _ARM64_VPUSH_D15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14) \
	_ARM64_VPUSH_FDD15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14)
# define _ARM64_VPUSH_D16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15) \
	_ARM64_VPUSH_FDD16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15)

# define _ARM64_VPUSH_Q1(VRI)             "str " VIPM_STR_C(VIPM_GLUE(q,VRI)) ", [sp, -16]! \n\t"
# define _ARM64_VPUSH_Q2(VRI0,VRI1) \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp, -32]! \n\t"
# define _ARM64_VPUSH_Q3(VRI0,VRI1,VRI2)                                                       \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp, -48]! \n\t" \
	"str " VIPM_STR_C(VIPM_GLUE(q,VRI2)) ", [sp, 32] \n\t"
# define _ARM64_VPUSH_Q4(VRI0,VRI1,VRI2,VRI3)                                                  \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp, -64]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t"
# define _ARM64_VPUSH_Q5(VRI0,VRI1,VRI2,VRI3,VRI4)                                             \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp, -80]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t"   \
	"str " VIPM_STR_C(VIPM_GLUE(q,VRI4)) ", [sp, 64] \n\t"
# define _ARM64_VPUSH_Q6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5)                                        \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp, -96]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t"   \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) ", [sp, 64] \n\t"
# define _ARM64_VPUSH_Q7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6)                                    \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp, -112]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) ", [sp, 64] \n\t"    \
	"str " VIPM_STR_C(VIPM_GLUE(q,VRI6)) ", [sp, 96] \n\t"
# define _ARM64_VPUSH_Q8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7)                               \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp, -128]! \n\t" \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) ", [sp, 64] \n\t"    \
	"stp " VIPM_STR_C(VIPM_GLUE(q,VRI6)) "," VIPM_STR_C(VIPM_GLUE(q,VRI7)) ", [sp, 96] \n\t"

# define _ARM64_VPOP_ALL()        \
	"ldp  q2, q3, [sp,  32] \n\t" \
	"ldp  q4, q5, [sp,  64] \n\t" \
	"ldp  q6, q7, [sp,  96] \n\t" \
	"ldp  q8, q9, [sp, 128] \n\t" \
	"ldp q10,q11, [sp, 160] \n\t" \
	"ldp q12,q13, [sp, 192] \n\t" \
	"ldp q14,q15, [sp, 224] \n\t" \
	"ldp q16,q17, [sp, 256] \n\t" \
	"ldp q18,q19, [sp, 288] \n\t" \
	"ldp q20,q21, [sp, 320] \n\t" \
	"ldp q22,q23, [sp, 352] \n\t" \
	"ldp q24,q25, [sp, 384] \n\t" \
	"ldp q26,q27, [sp, 416] \n\t" \
	"ldp q28,q29, [sp, 448] \n\t" \
	"ldp q30,q31, [sp, 480] \n\t" \
	"ldp  q0, q1, [sp], 512 \n\t"

# define _ARM64_VPOP_D1(VRI)            _ARM64_VPOP_FDD1(VRI)
# define _ARM64_VPOP_D2(VRI0,VRI1)      _ARM64_VPOP_FDD2(VRI0,VRI1)
# define _ARM64_VPOP_D3(VRI0,VRI1,VRI2) \
	_ARM64_VPOP_FDD3(VRI0,VRI1,VRI2)
# define _ARM64_VPOP_D4(VRI0,VRI1,VRI2,VRI3) \
	_ARM64_VPOP_FDD4(VRI0,VRI1,VRI2,VRI3)
# define _ARM64_VPOP_D5(VRI0,VRI1,VRI2,VRI3,VRI4) \
	_ARM64_VPOP_FDD5(VRI0,VRI1,VRI2,VRI3,VRI4)
# define _ARM64_VPOP_D6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5) \
	_ARM64_VPOP_FDD6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5)
# define _ARM64_VPOP_D7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6) \
	_ARM64_VPOP_FDD7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6)
# define _ARM64_VPOP_D8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7) \
	_ARM64_VPOP_FDD8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7)
# define _ARM64_VPOP_D9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8) \
	_ARM64_VPOP_FDD9(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8)
# define _ARM64_VPOP_D10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9) \
	_ARM64_VPOP_FDD10(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9)
# define _ARM64_VPOP_D11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10) \
	_ARM64_VPOP_FDD11(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10)
# define _ARM64_VPOP_D12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11) \
	_ARM64_VPOP_FDD12(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11)
# define _ARM64_VPOP_D13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12) \
	_ARM64_VPOP_FDD13(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12)
# define _ARM64_VPOP_D14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13) \
	_ARM64_VPOP_FDD14(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13)
# define _ARM64_VPOP_D15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14) \
	_ARM64_VPOP_FDD15(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14)
# define _ARM64_VPOP_D16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15) \
	_ARM64_VPOP_FDD16(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7,VRI8,VRI9,VRI10,VRI11,VRI12,VRI13,VRI14,VRI15)

# define _ARM64_VPOP_Q1(VRI)              "ldr " VIPM_STR_C(VIPM_GLUE(q,VRI)) ", [sp], 16 \n\t"
# define _ARM64_VPOP_Q2(VRI0,VRI1) \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp], 32 \n\t"
# define _ARM64_VPOP_Q3(VRI0,VRI1,VRI2)                    \
	"ldr " VIPM_STR_C(VIPM_GLUE(q,VRI2)) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp], 48 \n\t"
# define _ARM64_VPOP_Q4(VRI0,VRI1,VRI2,VRI3)                                                 \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp], 64 \n\t"
# define _ARM64_VPOP_Q5(VRI0,VRI1,VRI2,VRI3,VRI4)                                            \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(q,VRI4)) ", [sp, 64] \n\t"                                   \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp], 80 \n\t"
# define _ARM64_VPOP_Q6(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5)                                       \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) ", [sp, 64] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp], 96 \n\t"
# define _ARM64_VPOP_Q7(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6)                                  \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) ", [sp, 64] \n\t" \
	"ldr " VIPM_STR_C(VIPM_GLUE(q,VRI6)) ", [sp, 96] \n\t"                                   \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp], 112 \n\t"
# define _ARM64_VPOP_Q8(VRI0,VRI1,VRI2,VRI3,VRI4,VRI5,VRI6,VRI7)                             \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI2)) "," VIPM_STR_C(VIPM_GLUE(q,VRI3)) ", [sp, 32] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI4)) "," VIPM_STR_C(VIPM_GLUE(q,VRI5)) ", [sp, 64] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI6)) "," VIPM_STR_C(VIPM_GLUE(q,VRI7)) ", [sp, 96] \n\t" \
	"ldp " VIPM_STR_C(VIPM_GLUE(q,VRI0)) "," VIPM_STR_C(VIPM_GLUE(q,VRI1)) ", [sp], 128 \n\t"


/*
 * Load/Store multiple N-element structures
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn) or (Qn,Qn+1)
     */

# define _ARM64_VLD1_D1(W, VRI,PRR)             VIPM_GLUE(_ARM64_VLD1_D1_,_ARM_IMOD2(VRI))(W, VRI,PRR)

# define _ARM64_VLD1_D2(W, VRI0,VRI1,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD1_D3(W, VRI0,VRI1,VRI2,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD1_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM64_VLD1_Q1(W, VRI,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD1_Q2(W, VRI0,VRI1,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD1_Q3(W, VRI0,VRI1,VRI2,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD1_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM64_VST1_D1(W, VRI,PRR)             VIPM_GLUE(_ARM64_VST1_D1_,_ARM_IMOD2(VRI))(W, VRI,PRR)

# define _ARM64_VST1_D2(W, VRI0,VRI1,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST1_D3(W, VRI0,VRI1,VRI2,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST1_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM64_VST1_Q1(W, VRI,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST1_Q2(W, VRI0,VRI1,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST1_Q3(W, VRI0,VRI1,VRI2,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST1_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM64_VLD1_D1_AIA(W, VRI,PRR)         VIPM_GLUE(_ARM64_VLD1_D1_AIA_,_ARM_IMOD2(VRI))(W, VRI,PRR)

# define _ARM64_VLD1_D2_AIA(W, VRI0,VRI1,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #16 \n\t"

# define _ARM64_VLD1_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #24 \n\t"

# define _ARM64_VLD1_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"


# define _ARM64_VLD1_Q1_AIA(W, VRI,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", #16 \n\t"

# define _ARM64_VLD1_Q2_AIA(W, VRI0,VRI1,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"

# define _ARM64_VLD1_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #48 \n\t"

# define _ARM64_VLD1_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #64 \n\t"


# define _ARM64_VST1_D1_AIA(W, VRI,PRR)         VIPM_GLUE(_ARM64_VST1_D1_AIA_,_ARM_IMOD2(VRI))(W, VRI,PRR)

# define _ARM64_VST1_D2_AIA(W, VRI0,VRI1,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #16 \n\t"

# define _ARM64_VST1_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #24 \n\t"

# define _ARM64_VST1_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"


# define _ARM64_VST1_Q1_AIA(W, VRI,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", #16 \n\t"

# define _ARM64_VST1_Q2_AIA(W, VRI0,VRI1,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"

# define _ARM64_VST1_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #48 \n\t"

# define _ARM64_VST1_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #64 \n\t"


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1)
		 */

# define _ARM64_VLD1_D1_IA(W, VRI,PRR,ORR)      VIPM_GLUE(_ARM64_VLD1_D1_IA_,_ARM_IMOD2(VRI))(W, VRI,PRR,ORR)

# define _ARM64_VLD1_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD1_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD1_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"


# define _ARM64_VLD1_Q1_IA(W, VRI,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD1_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD1_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD1_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"


# define _ARM64_VST1_D1_IA(W, VRI,PRR,ORR)      VIPM_GLUE(_ARM64_VST1_D1_IA_,_ARM_IMOD2(VRI))(W, VRI,PRR,ORR)

# define _ARM64_VST1_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST1_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST1_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"


# define _ARM64_VST1_Q1_IA(W, VRI,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST1_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST1_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST1_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn,Qn+1)
     */

# define _ARM64_VLD2_D2(W, VRI0,VRI1,PRR) \
	"ld2 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD2_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM64_VLD2_D2_AIA(W, VRI0, VRI1, PRR)          \
	_ARM64_VLD2_D2(W, VRI2, VRI3, PRR)              \
	_ARM64_SUBI(PRR,PRR,16)

# define _ARM64_VLD2_Q2(W, VRI0,VRI1,PRR) \
	"ld2 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD2_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM64_VLD2_Q2_AIA(W, VRI0,VRI1,PRR)            \
	_ARM64_VLD2_Q2(W, VRI2,VRI3,PRR)                \
	_ARM64_SUBI(PRR,PRR,32)


# define _ARM64_VST2_D2(W, VRI0,VRI1,PRR) \
	"st2 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST2_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM64_VST2_D2_AIA(W, VRI0, VRI1, PRR)          \
	_ARM64_VST2_D2(W, VRI2, VRI3, PRR)              \
	_ARM64_SUBI(PRR,PRR,16)

# define _ARM64_VST2_Q2(W, VRI0,VRI1,PRR) \
	"st2 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST2_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM64_VST2_Q2_AIA(W, VRI0,VRI1,PRR)            \
	_ARM64_VST2_Q2(W, VRI2,VRI3,PRR)                \
	_ARM64_SUBI(PRR,PRR,32)


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM64_VLD2_D2_AIA(W, VRI0,VRI1,PRR) \
	"ld2 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #16 \n\t"

# define _ARM64_VLD2_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM64_VLD2_D2_AIA(W, VRI0, VRI1, PRR)          \
	_ARM64_VLD2_D2_AIA(W, VRI2, VRI3, PRR)

# define _ARM64_VLD2_Q2_AIA(W, VRI0,VRI1,PRR) \
	"ld2 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"


# define _ARM64_VST2_D2_AIA(W, VRI0,VRI1,PRR) \
	"st2 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #16 \n\t"

# define _ARM64_VST2_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	_ARM64_VST2_D2_AIA(W, VRI0, VRI1, PRR)          \
	_ARM64_VST2_D2_AIA(W, VRI2, VRI3, PRR)

# define _ARM64_VST2_Q2_AIA(W, VRI0,VRI1,PRR) \
	"st2 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1)
		 */

# define _ARM64_VLD2_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"ld2 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD2_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM64_VLD2_D2_AIA(W, VRI0, VRI1, PRR)                 \
	_ARM64_VLD2_D2_IA(W, VRI2, VRI3, PRR,ORR)              \
	_ARM64_SUBI(PRR,PRR,16)

# define _ARM64_VLD2_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	"ld2 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD2_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM64_VLD2_Q2_AIA(W, VRI0,VRI1,PRR)                   \
	_ARM64_VLD2_Q2_IA(W, VRI2,VRI3,PRR,ORR)                \
	_ARM64_SUBI(PRR,PRR,32)


# define _ARM64_VST2_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"st2 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST2_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM64_VST2_D2_AIA(W, VRI0, VRI1, PRR)                 \
	_ARM64_VST2_D2_IA(W, VRI2, VRI3, PRR,ORR)              \
	_ARM64_SUBI(PRR,PRR,16)

# define _ARM64_VST2_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	"st2 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST2_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	_ARM64_VST2_Q2_AIA(W, VRI0,VRI1,PRR)                   \
	_ARM64_VST2_Q2_IA(W, VRI2,VRI3,PRR,ORR)                \
	_ARM64_SUBI(PRR,PRR,32)

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM64_VLD3_D3(W, VRI0,VRI1,VRI2,PRR) \
	"ld3 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD3_Q3(W, VRI0,VRI1,VRI2,PRR) \
	"ld3 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM64_VST3_D3(W, VRI0,VRI1,VRI2,PRR) \
	"st3 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST3_Q3(W, VRI0,VRI1,VRI2,PRR) \
	"st3 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM64_VLD3_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"ld3 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #24 \n\t"

# define _ARM64_VLD3_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"ld3 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #48 \n\t"


# define _ARM64_VST3_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"st3 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #24 \n\t"

# define _ARM64_VST3_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"st3 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #48 \n\t"


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM64_VLD3_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"ld3 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD3_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"ld3 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"


# define _ARM64_VST3_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"st3 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST3_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"st3 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

	/*
	 * Load 4-element structures from memory (PRR: base address) into vector registers
	 * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
	 */

# define _ARM64_VLD4_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD4_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


# define _ARM64_VST4_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st4 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST4_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st4 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM64_VLD4_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"

# define _ARM64_VLD4_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #64 \n\t"


# define _ARM64_VST4_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st4 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #32 \n\t"

# define _ARM64_VST4_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"st4 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #64 \n\t"


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM64_VLD4_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"ld4 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD4_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"ld4 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"


# define _ARM64_VST4_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"st4 {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST4_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"st4 {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* ========================================================================= */

/*
 * Load (with duplicate) single N-element structure to all lanes (into all elements of vector registers)
 */

    /*
     * Load 1-element structure from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1)
	 * (VRI0, ...): (Qn)
     */

# define _ARM64_VLD1_DUP_D1(W, VRI,PRR) \
	"ld1r {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD1_DUP_D2(W, VRI0,VRI1,PRR)           _ARM64_VLD1_DUP_Q1(W, _ARM64_DVLMAP(VRI0),PRR)


# define _ARM64_VLD1_DUP_Q1(W, VRI,PRR) \
	"ld1r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1)
		 * (VRI0, ...): (Qn)
		 */

# define _ARM64_VLD1_DUP_D1_AIA(W, VRI,PRR) \
	"ld1r {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(1,W)) " \n\t"

# define _ARM64_VLD1_DUP_D2_AIA(W, VRI0,VRI1,PRR)       _ARM64_VLD1_DUP_Q1_AIA(W, _ARM64_DVLMAP(VRI0),PRR)


# define _ARM64_VLD1_DUP_Q1_AIA(W, VRI,PRR) \
	"ld1r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(1,W)) " \n\t"


		/*
		 * With offset update
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1)
		 * (VRI0, ...): (Qn)
		 */

# define _ARM64_VLD1_DUP_D1_IA(W, VRI,PRR,ORR) \
	"ld1r {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD1_DUP_D2_IA(W, VRI0,VRI1,PRR,ORR)    _ARM64_VLD1_DUP_Q1_IA(W, _ARM64_DVLMAP(VRI0),PRR,ORR)


# define _ARM64_VLD1_DUP_Q1_IA(W, VRI,PRR,ORR) \
	"ld1r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
     */

# define _ARM64_VLD2_DUP_D2(W, VRI0,VRI1,PRR) \
	"ld2r {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD2_DUP_Q2(W, VRI0,VRI1,PRR) \
	"ld2r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM64_VLD2_DUP_D2_AIA(W, VRI0,VRI1,PRR) \
	"ld2r {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(2,W)) " \n\t"

# define _ARM64_VLD2_DUP_Q2_AIA(W, VRI0,VRI1,PRR) \
	"ld2r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(2,W)) " \n\t"


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
		 */

# define _ARM64_VLD2_DUP_D2_IA(W, VRI0,VRI1,PRR,ORR) \
	"ld2r {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD2_DUP_Q2_IA(W, VRI0,VRI1,PRR,ORR) \
	"ld2r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) ", " VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM64_VLD3_DUP_D3(W, VRI0,VRI1,VRI2,PRR) \
	"ld3r {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD3_DUP_Q3(W, VRI0,VRI1,VRI2,PRR) \
	"ld3r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM64_VLD3_DUP_D3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"ld3r {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(3,W)) " \n\t"

# define _ARM64_VLD3_DUP_Q3_AIA(W, VRI0,VRI1,VRI2,PRR) \
	"ld3r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(3,W)) " \n\t"


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM64_VLD3_DUP_D3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"ld3r {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD3_DUP_Q3_IA(W, VRI0,VRI1,VRI2,PRR,ORR) \
	"ld3r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
     */

# define _ARM64_VLD4_DUP_D4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4r {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VLD4_DUP_Q4(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM64_VLD4_DUP_D4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4r {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(4,W)) " \n\t"

# define _ARM64_VLD4_DUP_Q4_AIA(W, VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(4,W)) " \n\t"


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM64_VLD4_DUP_D4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"ld4r {" VIPM_STR_C(_ARM64_DVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_DVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VLD4_DUP_Q4_IA(W, VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"ld4r {" VIPM_STR_C(_ARM64_QVR_FMT(W,VRI0)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI1)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI2)) "," VIPM_STR_C(_ARM64_QVR_FMT(W,VRI3)) "}, " VIPM_STR_C([%PRR]) "," VIPM_STR_C(%ORR) " \n\t"

/* ========================================================================= */

/*
 * Load/Store single N-element structure to/from one lane
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI): (Dn[L])
     */

# define _ARM64_VLD1_LANE_D1(W, L,VRI,PRR) \
	"ld1 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVMAP(VRI))._ARM64_WTMAP(W)) "}" VIPM_STR_C([_ARM64_LDVMAP(W,L,VRI)]) ", " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST1_LANE_D1(W, L,VRI,PRR) \
	"st1 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVMAP(VRI))._ARM64_WTMAP(W)) "}" VIPM_STR_C([_ARM64_LDVMAP(W,L,VRI)]) ", " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM64_VLD1_LANE_D1_AIA(W, L,VRI,PRR) \
	"ld1 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVMAP(VRI))._ARM64_WTMAP(W)) "}" VIPM_STR_C([_ARM64_LDVMAP(W,L,VRI)]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(1,W)) " \n\t"

# define _ARM64_VST1_LANE_D1_AIA(W, L,VRI,PRR) \
	"st1 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVMAP(VRI))._ARM64_WTMAP(W)) "}" VIPM_STR_C([_ARM64_LDVMAP(W,L,VRI)]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(1,W)) " \n\t"


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM64_VLD1_LANE_D1_IA(W, L,VRI,PRR,ORR) \
	"ld1 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVMAP(VRI))._ARM64_WTMAP(W)) "}" VIPM_STR_C([_ARM64_LDVMAP(W,L,VRI)]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST1_LANE_D1_IA(W, L,VRI,PRR,ORR) \
	"st1 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVMAP(VRI))._ARM64_WTMAP(W)) "}" VIPM_STR_C([_ARM64_LDVMAP(W,L,VRI)]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
     */

# define _ARM64_VLD2_LANE_D2(W, L,VRI0,VRI1,PRR) \
	"ld2 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST2_LANE_D2(W, L,VRI0,VRI1,PRR) \
	"st2 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM64_VLD2_LANE_D2_AIA(W, L,VRI0,VRI1,PRR) \
	"ld2 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(2,W)) " \n\t"

# define _ARM64_VST2_LANE_D2_AIA(W, L,VRI0,VRI1,PRR) \
	"st2 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(2,W)) " \n\t"


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM64_VLD2_LANE_D2_IA(W, L,VRI0,VRI1,PRR,ORR) \
	"ld2 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST2_LANE_D2_IA(W, L,VRI0,VRI1,PRR,ORR) \
	"st2 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
     */

# define _ARM64_VLD3_LANE_D3(W, L,VRI0,VRI1,VRI2,PRR) \
	"ld3 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST3_LANE_D3(W, L,VRI0,VRI1,VRI2,PRR) \
	"st3 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM64_VLD3_LANE_D3_AIA(W, L,VRI0,VRI1,VRI2,PRR) \
	"ld3 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(3,W)) " \n\t"

# define _ARM64_VST3_LANE_D3_AIA(W, L,VRI0,VRI1,VRI2,PRR) \
	"st3 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(3,W)) " \n\t"


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM64_VLD3_LANE_D3_IA(W, L,VRI0,VRI1,VRI2,PRR,ORR) \
	"ld3 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST3_LANE_D3_IA(W, L,VRI0,VRI1,VRI2,PRR,ORR) \
	"st3 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
     * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
     */

# define _ARM64_VLD4_LANE_D4(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI3))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) " \n\t"

# define _ARM64_VST4_LANE_D4(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"st4 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI3))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) " \n\t"


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM64_VLD4_LANE_D4_AIA(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"ld4 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI3))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(4,W)) " \n\t"

# define _ARM64_VST4_LANE_D4_AIA(W, L,VRI0,VRI1,VRI2,VRI3,PRR) \
	"st4 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI3))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", #" VIPM_STR_C(_ARM64_VXFER_INCSZ(4,W)) " \n\t"


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM64_VLD4_LANE_D4_IA(W, L,VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"ld4 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI3))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

# define _ARM64_VST4_LANE_D4_IA(W, L,VRI0,VRI1,VRI2,VRI3,PRR,ORR) \
	"st4 {" VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI0))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI1))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI2))._ARM64_WTMAP(W)) ", " VIPM_STR_C(VIPM_GLUE(v,_ARM64_DVLMAP(VRI3))._ARM64_WTMAP(W)) "}" VIPM_STR_C([L]) ", " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"

/* ------------------------------------------------------------------------- */

/* impl.
 * @{
 */

# define _ARM64_VLD1_D1_0(W, VRI,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"
# define _ARM64_VST1_D1_0(W, VRI,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) " \n\t"
# define _ARM64_VLD1_D1_1(W, VRI,PRR)           _ARM64_VLD1_LANE_D1(64, 0,VRI,PRR)
# define _ARM64_VST1_D1_1(W, VRI,PRR)           _ARM64_VST1_LANE_D1(64, 0,VRI,PRR)
/* With auto-increment */
# define _ARM64_VLD1_D1_AIA_0(W, VRI,PRR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", #8 \n\t"
# define _ARM64_VST1_D1_AIA_0(W, VRI,PRR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", #8 \n\t"
# define _ARM64_VLD1_D1_AIA_1(W, VRI,PRR)       _ARM64_VLD1_LANE_D1_AIA(64, 0,VRI,PRR)
# define _ARM64_VST1_D1_AIA_1(W, VRI,PRR)       _ARM64_VST1_LANE_D1_AIA(64, 0,VRI,PRR)
/* With offset update */
# define _ARM64_VLD1_D1_IA_0(W, VRI,PRR,ORR) \
	"ld1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"
# define _ARM64_VST1_D1_IA_0(W, VRI,PRR,ORR) \
	"st1 {" VIPM_STR_C(_ARM64_VXFER_DVR_FMT(W,VRI)) "}, " VIPM_STR_C([%PRR]) ", " VIPM_STR_C(%ORR) " \n\t"
# define _ARM64_VLD1_D1_IA_1(W, VRI,PRR,ORR)    _ARM64_VLD1_LANE_D1_IA(64, 0,VRI,PRR,ORR)
# define _ARM64_VST1_D1_IA_1(W, VRI,PRR,ORR)    _ARM64_VST1_LANE_D1_IA(64, 0,VRI,PRR,ORR)

/* @} */

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VXFER_DVR_FMT(W,D)      VIPM_GLUE(v,_ARM64_DVLMAP(D))._ARM64_DWTMAP(W)

# define _ARM64_VXFER_INCSZ(N,W)        VIPM_GLUE3(_ARM64_VXFER_INCSZ_,N,W)()
# define _ARM64_VXFER_INCSZ_18()        1
# define _ARM64_VXFER_INCSZ_28()        2
# define _ARM64_VXFER_INCSZ_38()        3
# define _ARM64_VXFER_INCSZ_48()        4
# define _ARM64_VXFER_INCSZ_116()       2
# define _ARM64_VXFER_INCSZ_216()       4
# define _ARM64_VXFER_INCSZ_316()       6
# define _ARM64_VXFER_INCSZ_416()       8
# define _ARM64_VXFER_INCSZ_132()       4
# define _ARM64_VXFER_INCSZ_232()       8
# define _ARM64_VXFER_INCSZ_332()       12
# define _ARM64_VXFER_INCSZ_432()       16
# define _ARM64_VXFER_INCSZ_164()       8
# define _ARM64_VXFER_INCSZ_264()       16
# define _ARM64_VXFER_INCSZ_364()       24
# define _ARM64_VXFER_INCSZ_464()       32
/* @} */

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define __ARM64_VPRINT_LANE_D1(W,_Fmt,L,VRI) \
	VIPM_STR_C(_ARM64_VGET_DATATYPE_PREFIX(W))"mov " VIPM_STR_C(_ARM64_VGET_RREG_SIZE_MOD(W)1) ", " VIPM_STR_C(_ARM64_DSCL_FMT(W,L,VRI)) " \n\t" \
    _ARM64_DPRINTFI(_Fmt)

# define __ARM64_VPRINTF32_LANE_D1(_Fmt,L,VRI) \
	_ARM64_VPUSH_Q1(0)                         \
	_ARM_VF32TOF64_Q1D1(0,VRI)                 \
	"fmov x1, " VIPM_STR_C(_ARM64_VPRINTF32_ARG2(L)) "\n\t" \
    _ARM64_DPRINTFI(_Fmt)                      \
	_ARM64_VPOP_Q1(0)

/* ------------------------------------------------------------------------- */

/* @{ */
# define _ARM64_VPRINTF32_ARG2(L)       VIPM_GLUE(_ARM64_VPRINTF32_ARG2_,L)()
# define _ARM64_VPRINTF32_ARG2_0()      d0
# define _ARM64_VPRINTF32_ARG2_1()      v0.d[1]
/* @} */

/* @} */
/* @} */


/* ######################################################################### */

/* @{ */
#ifndef __VIPM_ARM_DEFS__
# define __VIPM_ARM_DEFS__

/*
 * @{
 */

/* @{ */
# define _ARM_WIDTHOF_PTR           _ARM64_WIDTHOF_PTR
# define _ARM_PTR_OFFSET(Offset)    _ARM64_PTR_OFFSET(Offset)
/* @} */


/* ========================================================================= */

/*
 * Move operations
 * @{
 */

# define _ARM_MOVI                  _ARM64_MOVI
# define _ARM_MOVI_UCF              _ARM64_MOVI_UCF
# define _ARM_MOVI_IF               _ARM64_MOVI_IF
# define _ARM_MOVI_UCF_IF           _ARM64_MOVI_UCF_IF

# define _ARM_MOV                   _ARM64_MOV
# define _ARM_MOV_UCF               _ARM64_MOV_UCF
# define _ARM_MOV_IF                _ARM64_MOV_IF
# define _ARM_MOV_UCF_IF            _ARM64_MOV_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 *
 * signed  : (x) --> (-2**(sat-1) <= x <= 2**(sat-1) -1)
 * unsigned: (x) --> (0 <= x <= 2**sat -1)
 * @{
 */

/* ------------------------------------------------------------------------- */

# define _ARM_RND_FTOSIEX           _ARM64_RND_FTOSIEX
# define _ARM_RND_FTOUIEX           _ARM64_RND_FTOUIEX

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * NEGATE: (a) --> (-a)
 * ADD : (a,b) --> (a + b)
 *		ADD[I]        : add
 *		ADD[I]_UCF    : add updating the condition flags
 *		ADD[I]_IF     : [conditionally] add
 *		ADD[I]_UCF_IF : [conditionally] add updating the condition flags
 * SUB : (a,b) --> (a - b)
 * RSUB: (a,b) --> (b - a)
 * MUL : (a,b) --> (a * b)
 */

/* ------------------------------------------------------------------------- */

    /*
     * NEGATE
	 * @{
     */
# define _ARM_NEGATE                _ARM64_NEGATE
# define _ARM_NEGATE_UCF            _ARM64_NEGATE_UCF
# define _ARM_NEGATE_IF             _ARM64_NEGATE_IF
# define _ARM_NEGATE_UCF_IF         _ARM64_NEGATE_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * ADD
	 * @{
     */
# define _ARM_ADDI                  _ARM64_ADDI
# define _ARM_ADDI_UCF              _ARM64_ADDI_UCF
# define _ARM_ADDI_IF               _ARM64_ADDI_IF
# define _ARM_ADDI_UCF_IF           _ARM64_ADDI_UCF_IF

# define _ARM_ADD                   _ARM64_ADD
# define _ARM_ADD_UCF               _ARM64_ADD_UCF
# define _ARM_ADD_IF                _ARM64_ADD_IF
# define _ARM_ADD_UCF_IF            _ARM64_ADD_UCF_IF
	/* @} */

    /*
     * ADDEX
	 * @{
     */
# define _ARM_ADDEX                 _ARM64_ADDEX
# define _ARM_ADDEX_UCF             _ARM64_ADDEX_UCF
# define _ARM_ADDEX_IF              _ARM64_ADDEX_IF
# define _ARM_ADDEX_UCF_IF          _ARM64_ADDEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * SUB
	 * @{
     */
# define _ARM_SUBI                  _ARM64_SUBI
# define _ARM_SUBI_UCF              _ARM64_SUBI_UCF
# define _ARM_SUBI_IF               _ARM64_SUBI_IF
# define _ARM_SUBI_UCF_IF           _ARM64_SUBI_UCF_IF

# define _ARM_SUB                   _ARM64_SUB
# define _ARM_SUB_UCF               _ARM64_SUB_UCF
# define _ARM_SUB_IF                _ARM64_SUB_IF
# define _ARM_SUB_UCF_IF            _ARM64_SUB_UCF_IF
	/* @} */

    /*
     * SUBEX
	 * @{
     */
# define _ARM_SUBEX                 _ARM64_SUBEX
# define _ARM_SUBEX_UCF             _ARM64_SUBEX_UCF
# define _ARM_SUBEX_IF              _ARM64_SUBEX_IF
# define _ARM_SUBEX_UCF_IF          _ARM64_SUBEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * RSUB
	 * @{
     */
# define _ARM_RSUBI                 _ARM64_RSUBI
# define _ARM_RSUBI_UCF             _ARM64_RSUBI_UCF
# define _ARM_RSUBI_IF              _ARM64_RSUBI_IF
# define _ARM_RSUBI_UCF_IF          _ARM64_RSUBI_UCF_IF

# define _ARM_RSUB                  _ARM64_RSUB
# define _ARM_RSUB_UCF              _ARM64_RSUB_UCF
# define _ARM_RSUB_IF               _ARM64_RSUB_IF
# define _ARM_RSUB_UCF_IF           _ARM64_RSUB_UCF_IF
	/* @} */

    /*
     * RSUBEX
	 * @{
     */
# define _ARM_RSUBEX                _ARM64_RSUBEX
# define _ARM_RSUBEX_UCF            _ARM64_RSUBEX_UCF
# define _ARM_RSUBEX_IF             _ARM64_RSUBEX_IF
# define _ARM_RSUBEX_UCF_IF         _ARM64_RSUBEX_UCF_IF
	/* @} */

/* ------------------------------------------------------------------------- */

    /*
     * MUL
	 * @{
     */
# define _ARM_MUL                   _ARM64_MUL
# define _ARM_MUL_UCF               _ARM64_MUL_UCF
# define _ARM_MUL_IF                _ARM64_MUL_IF
# define _ARM_MUL_UCF_IF            _ARM64_MUL_UCF_IF
	/* @} */

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

# define _ARM_CMPI                  _ARM64_CMPI
# define _ARM_CMPI_IF               _ARM64_CMPI_IF

# define _ARM_CMP                   _ARM64_CMP
# define _ARM_CMP_IF                _ARM64_CMP_IF

# define _ARM_CMPNEGI               _ARM64_CMPNEGI
# define _ARM_CMPNEGI_IF            _ARM64_CMPNEGI_IF

# define _ARM_CMPNEG                _ARM64_CMPNEG
# define _ARM_CMPNEG_IF             _ARM64_CMPNEG_IF

# define _ARM_UCF                   _ARM64_UCF

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * not                                                 : NOT[I]     : DRR = ~SRR
 * not updating the condition flags                    : NOT[I]_UCF : DRR = ~SRR
 * and                                                 : AND[I]     : DRR = SRR & N
 * and updating the condition flags                    : AND[I]_UCF : DRR = SRR & N
 * and complement (and * not)                          : ANT[I]     : DRR = SRR & ~N
 * and complement (and * not) updating the condition flags : ANT[I]_UCF : DRR = SRR & ~N
 * or                                                  : OR[I]      : DRR = SRR | N
 * or updating the condition flags                     : OR[I]_UCF  : DRR = SRR | N
 * xor                                                 : XOR[I]     : DRR = SRR ^ N
 * xor updating the condition flags                    : XOR[I]_UCF : DRR = SRR ^ N
 * set all bits                                        : BITSET     :
 * clear all bits                                      : BITCLR     :
 */

/* ------------------------------------------------------------------------- */

    /*
     * NOT
     */

# define _ARM_NOTI                  _ARM64_NOTI
# define _ARM_NOTI_UCF              _ARM64_NOTI_UCF
# define _ARM_NOTI_IF               _ARM64_NOTI_IF
# define _ARM_NOTI_UCF_IF           _ARM64_NOTI_UCF_IF

# define _ARM_NOT                   _ARM64_NOT
# define _ARM_NOT_UCF               _ARM64_NOT_UCF
# define _ARM_NOT_IF                _ARM64_NOT_IF
# define _ARM_NOT_UCF_IF            _ARM64_NOT_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * AND
     */

# define _ARM_ANDI                  _ARM64_ANDI
# define _ARM_ANDI_UCF              _ARM64_ANDI_UCF
# define _ARM_ANDI_IF               _ARM64_ANDI_IF
# define _ARM_ANDI_UCF_IF           _ARM64_ANDI_UCF_IF

# define _ARM_AND                   _ARM64_AND
# define _ARM_AND_UCF               _ARM64_AND_UCF
# define _ARM_AND_IF                _ARM64_AND_IF
# define _ARM_AND_UCF_IF            _ARM64_AND_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * BIC: AND * NOT
     */

# define _ARM_BICI                  _ARM64_BICI
# define _ARM_BICI_UCF              _ARM64_BICI_UCF
# define _ARM_BICI_IF               _ARM64_BICI_IF
# define _ARM_BICI_UCF_IF           _ARM64_BICI_UCF_IF

# define _ARM_BIC                   _ARM64_BIC
# define _ARM_BIC_UCF               _ARM64_BIC_UCF
# define _ARM_BIC_IF                _ARM64_BIC_IF
# define _ARM_BIC_UCF_IF            _ARM64_BIC_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * OR
     */

# define _ARM_ORRI                  _ARM64_ORRI
# define _ARM_ORRI_UCF              _ARM64_ORRI_UCF
# define _ARM_ORRI_IF               _ARM64_ORRI_IF
# define _ARM_ORRI_UCF_IF           _ARM64_ORRI_UCF_IF

# define _ARM_ORR                   _ARM64_ORR
# define _ARM_ORR_UCF               _ARM64_ORR_UCF
# define _ARM_ORR_IF                _ARM64_ORR_IF
# define _ARM_ORR_UCF_IF            _ARM64_ORR_UCF_IF

/* ------------------------------------------------------------------------- */

    /*
     * XOR
     */

# define _ARM_XORI                  _ARM64_XORI
# define _ARM_XORI_UCF              _ARM64_XORI_UCF
# define _ARM_XORI_IF               _ARM64_XORI_IF
# define _ARM_XORI_UCF_IF           _ARM64_XORI_UCF_IF

# define _ARM_XOR                   _ARM64_XOR
# define _ARM_XOR_UCF               _ARM64_XOR_UCF
# define _ARM_XOR_IF                _ARM64_XOR_IF
# define _ARM_XOR_UCF_IF            _ARM64_XOR_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * rshift: DRR = SRR >> N
 * lshift: DRR = SRR << N
 */

# define _ARM_RSHIFTI               _ARM64_RSHIFTI
# define _ARM_RSHIFTI_UCF           _ARM64_RSHIFTI_UCF
# define _ARM_RSHIFT                _ARM64_RSHIFT
# define _ARM_RSHIFT_UCF            _ARM64_RSHIFT_UCF

# define _ARM_RSHIFTI_IF            _ARM64_RSHIFTI_IF
# define _ARM_RSHIFTI_UCF_IF        _ARM64_RSHIFTI_UCF_IF
# define _ARM_RSHIFT_IF             _ARM64_RSHIFT_IF
# define _ARM_RSHIFT_UCF_IF         _ARM64_RSHIFT_UCF_IF


# define _ARM_LSHIFTI               _ARM64_LSHIFTI
# define _ARM_LSHIFTI_UCF           _ARM64_LSHIFTI_UCF
# define _ARM_LSHIFT                _ARM64_LSHIFT
# define _ARM_LSHIFT_UCF            _ARM64_LSHIFT_UCF

# define _ARM_LSHIFTI_IF            _ARM64_LSHIFTI_IF
# define _ARM_LSHIFTI_UCF_IF        _ARM64_LSHIFTI_UCF_IF
# define _ARM_LSHIFT_IF             _ARM64_LSHIFT_IF
# define _ARM_LSHIFT_UCF_IF         _ARM64_LSHIFT_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 */

    /*
     * PUSH/POP
     */
# define _ARM_PUSH1                 _ARM64_PUSH1
# define _ARM_PUSH2                 _ARM64_PUSH2
# define _ARM_PUSH_ARGRR            _ARM64_PUSH_ARGRR
# define _ARM_PUSH_ALLRR            _ARM64_PUSH_ALLRR
# define _ARM_PUSH_ALLRR0           _ARM64_PUSH_ALLRR0
# define _ARM_PUSH_AVARR            _ARM64_PUSH_AVARR
# define _ARM_PUSH_AVARR0           _ARM64_PUSH_AVARR0

# define _ARM_POP1                  _ARM64_POP1
# define _ARM_POP2                  _ARM64_POP2
# define _ARM_POP_ARGRR             _ARM64_POP_ARGRR
# define _ARM_POP_ALLRR             _ARM64_POP_ALLRR
# define _ARM_POP_ALLRR0            _ARM64_POP_ALLRR0
# define _ARM_POP_AVARR             _ARM64_POP_AVARR
# define _ARM_POP_AVARR0            _ARM64_POP_AVARR0

/* ------------------------------------------------------------------------- */

    /*
     * Load
     */

# define _ARM_LOAD                  _ARM64_LOAD
# define _ARM_LOADO                 _ARM64_LOADO
# define _ARM_LOADOI                _ARM64_LOADOI

# define _ARM_LOAD_IF               _ARM64_LOAD_IF
# define _ARM_LOADO_IF              _ARM64_LOADO_IF
# define _ARM_LOADOI_IF             _ARM64_LOADOI_IF


/* slow operation (?) */
# define _ARM_LOAD_IB               _ARM64_LOAD_IB
# define _ARM_LOAD_DB               _ARM64_LOAD_DB
# define _ARM_LOAD_IA               _ARM64_LOAD_IA
# define _ARM_LOAD_DA               _ARM64_LOAD_DA
# define _ARM_LOADI_IB              _ARM64_LOADI_IB
# define _ARM_LOADI_DB              _ARM64_LOADI_DB
# define _ARM_LOADI_IA              _ARM64_LOADI_IA
# define _ARM_LOADI_DA              _ARM64_LOADI_DA

/* ------------------------------------------------------------------------- */

    /*
     * Store
     */

# define _ARM_STORE                 _ARM64_STORE
# define _ARM_STOREO                _ARM64_STOREO
# define _ARM_STOREOI               _ARM64_STOREOI

# define _ARM_STORE_IF              _ARM64_STORE_IF
# define _ARM_STOREO_IF             _ARM64_STOREO_IF
# define _ARM_STOREOI_IF            _ARM64_STOREOI_IF


/* slow operation (?) */
# define _ARM_STORE_IB              _ARM64_STORE_IB
# define _ARM_STORE_DB              _ARM64_STORE_DB
# define _ARM_STORE_IA              _ARM64_STORE_IA
# define _ARM_STORE_DA              _ARM64_STORE_DA
# define _ARM_STOREI_IB             _ARM64_STOREI_IB
# define _ARM_STOREI_DB             _ARM64_STOREI_DB
# define _ARM_STOREI_IA             _ARM64_STOREI_IA
# define _ARM_STOREI_DA             _ARM64_STOREI_DA

/* ------------------------------------------------------------------------- */

    /*
     * Prefetch Memory
     */

# define _ARM_PREFETCHI_CACHELINE   _ARM64_PREFETCHI_CACHELINE
# define _ARM_APREFETCHI_CACHELINE  _ARM64_APREFETCHI_CACHELINE
# define _ARM_PREFETCH_CACHELINE    _ARM64_PREFETCH_CACHELINE
# define _ARM_PREFETCHEX_CACHELINE  _ARM64_PREFETCHEX_CACHELINE

/* @} */


/* ========================================================================= */

/*
 * Flow control
 * @{
 */

# define _ARM_GOTO_IMM              _ARM64_GOTO_IMM
# define _ARM_GOTO_IF               _ARM64_GOTO_IF

/* ------------------------------------------------------------------------- */

    /*
     * Switch branch-table (ARM mode)
	 * @note: Imm = N, case <- {[0, N], default}
     */

# define _ARM_SWITCH                _ARM64_SWITCH

/* Usage sample:

ARM:
	_ARM_SWITCH([x], N, Ldefault)

	_ARM_GOTO_IMM(L0)
	_ARM_GOTO_IMM(L1)
	_ARM_GOTO_IMM(L2)
	_ARM_GOTO_IMM(L3)
	...
	_ARM_GOTO_IMM(Ln)

	L0:       stmt0 _ARM_GOTO_IMM(Lend)
	L1:       stmt1 _ARM_GOTO_IMM(Lend)
	L2:       stmt2 _ARM_GOTO_IMM(Lend)
	L3:       stmt3 _ARM_GOTO_IMM(Lend)
	...
	Ln:       stmtN _ARM_GOTO_IMM(Lend)
	Ldefault: stmtD _ARM_GOTO_IMM(Lend)
	Lend:

C:
	switch (x) {
	case 0:  stmt0 break;
	case 1:  stmt1 break;
	case 2:  stmt2 break;
	case 3:  stmt3 break;
	...
	case N:  stmtN break;
	default: stmtD break;
	}
 */

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define _ARM_DPRINTFI              _ARM64_DPRINTFI

# define __ARM_PRINTRR1             __ARM64_PRINTRR1
# define __ARM_PRINTRR2             __ARM64_PRINTRR2
# define __ARM_PRINTRR3             __ARM64_PRINTRR3

/* @} */
/* @} */


/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*
 * NEON
 * @{
 */


/* ************************************************************************* */

/*
 * Move operations
 * @{
 */

    /*
     * Duplicate/Set immediate value into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = Imm
     */
# define _ARM_VDUPI_Q               _ARM64_VDUPI_Q
# define _ARM_VDUPI_D               _ARM64_VDUPI_D
# define _ARM_VDUPFI_Q              _ARM64_VDUPFI_Q
# define _ARM_VDUPFI_D              _ARM64_VDUPFI_D
# define _ARM_VDUPFI_S              _ARM64_VDUPFI_S
# define _ARM_VDUPIEX_Q             _ARM64_VDUPIEX_Q
# define _ARM_VDUPIEX_D             _ARM64_VDUPIEX_D


    /*
     * Set zero value into one element of the destination vector register
	 * dM[k] = 0, k <- {0,N}
	 * sM = 0
	 */
# define _ARM_VZERO1_Q              _ARM64_VZERO1_Q
# define _ARM_VZERO1_D              _ARM64_VZERO1_D
# define _ARM_VZERO1_S              _ARM64_VZERO1_S

/* ------------------------------------------------------------------------- */

    /*
     * Duplicate/Set [regular register] scalar into every element of the destination vector register
	 *
	 * (dM[0],..., dM[N]) = $RR
     */
# define _ARM_VDUPR_Q               _ARM64_VDUPR_Q
# define _ARM_VDUPR_D               _ARM64_VDUPR_D


    /*
     * Duplicate/Set [vector] scalar into every element of the destination vector register
	 * (dM[0],..., dM[N]) = dK[i], i <- {0,N}
     */
# define _ARM_VDUP_Q                _ARM64_VDUP_Q
# define _ARM_VDUP_D                _ARM64_VDUP_D


    /*
     * Set [regular register] scalar into one element of the destination vector register
	 * dM[k] = $RR, k <- {0,N}
	 * sM = $RR
	 */
# define _ARM_VSET1_Q               _ARM64_VSET1_Q
# define _ARM_VSET1_D               _ARM64_VSET1_D
# define _ARM_VSET1_S               _ARM64_VSET1_S

/* ------------------------------------------------------------------------- */

    /*
     * Get [regular register] scalar from one element of the source vector register
	 *
	 * $RR = dM[i], i <- {0,N}
	 * $RR1 = dM[0], $RR2 = dM[1]
	 * $RR = sM
     */
# define _ARM_VGET1_D               _ARM64_VGET1_D
# define _ARM_VGET1_D_IF            _ARM64_VGET1_D_IF
# define _ARM_VGET1_S               _ARM64_VGET1_S
# define _ARM_VGET1_S_IF            _ARM64_VGET1_S_IF

/* ------------------------------------------------------------------------- */

    /*
     * Move between vector registers
	 */

/* Q --> Q */
/* D --> D */
/* S --> S */
# define _ARM_VMOV_Q                _ARM64_VMOV_Q
# define _ARM_VMOV_D                _ARM64_VMOV_D
# define _ARM_VMOV_S                _ARM64_VMOV_S

# define _ARM_VMOV                  _ARM64_VMOV

# define _ARM_VMVN_Q                _ARM64_VMVN_Q
# define _ARM_VMVN_D                _ARM64_VMVN_D

/* @} */


/* ========================================================================= */

/*
 * Converting/Saturating operations
 * @{
 */

/* w - source data width
 *
 * widen:   D    --> (Q)   ;; uint8 --> int16, int16 --> int32, int32 --> int64
 * widenf:  S    --> (D)   ;; float16 --> float32, float32 --> float64
 *          D    --> (Q)
 * narrow: (Q)   -->  D    ;; int16 --> uint8, int32 --> int16, int64 --> int32
 * narrowf:(D)   -->  S    ;; float32 --> float16, float64 --> float32
 *         (Q)   -->  D
 */

# define _ARM_VWIDEN_Q1D1           _ARM64_VWIDEN_Q1D1
# define _ARM_VWIDENF_D1S1          _ARM64_VWIDENF_D1S1
# define _ARM_VWIDENF_Q1D1          _ARM64_VWIDENF_Q1D1

# define _ARM_VNARROW_D1Q1          _ARM64_VNARROW_D1Q1
# define _ARM_VNARROWF_S1D1         _ARM64_VNARROWF_S1D1
# define _ARM_VNARROWF_D1Q1         _ARM64_VNARROWF_D1Q1

/* narrow with saturate: Q --> D */
# define _ARM_VSATURATE_UU_D1Q1     _ARM64_VSATURATE_UU_D1Q1
# define _ARM_VSATURATE_SS_D1Q1     _ARM64_VSATURATE_SS_D1Q1
# define _ARM_VSATURATE_SU_D1Q1     _ARM64_VSATURATE_SU_D1Q1


    /*
     * vector convert signed/unsigned integer to floating point
     */
# define _ARM_VSITOFEX_Q            _ARM64_VSITOFEX_Q
# define _ARM_VSITOFEX_D            _ARM64_VSITOFEX_D
# define _ARM_VSITOFEX_S            _ARM64_VSITOFEX_S
# define _ARM_VUITOFEX_Q            _ARM64_VUITOFEX_Q
# define _ARM_VUITOFEX_D            _ARM64_VUITOFEX_D
# define _ARM_VUITOFEX_S            _ARM64_VUITOFEX_S

    /*
     * vector convert signed/unsigned long integer to double
     */
# define _ARM_VLITOF64_Q(DVRI,SVRI) _ARM_VSITOFEX_Q(64,DVRI,SVRI)
# define _ARM_VLITOF64_D(DVRI,SVRI) _ARM_VSITOFEX_D(64,DVRI,SVRI)
# define _ARM_VLITOF64_S(DVRI,SVRI) _ARM_VSITOFEX_S(64,DVRI,SVRI)
# define _ARM_VLUTOF64_Q(DVRI,SVRI) _ARM_VUITOFEX_Q(64,DVRI,SVRI)
# define _ARM_VLUTOF64_D(DVRI,SVRI) _ARM_VUITOFEX_D(64,DVRI,SVRI)
# define _ARM_VLUTOF64_S(DVRI,SVRI) _ARM_VUITOFEX_S(64,DVRI,SVRI)

	/*
     * vector convert signed/unsigned integer to double
	 */
/* S --> D
   D --> Q
   Q --> (Q,Q) */
//# define _ARM_VSITOF64_D1S1         _ARM64_VSITOF64_D1S1
# define _ARM_VSITOF64_Q1D1         _ARM64_VSITOF64_Q1D1
# define _ARM_VSITOF64_Q2Q1         _ARM64_VSITOF64_Q2Q1
//# define _ARM_VUITOF64_D1S1         _ARM64_VUITOF64_D1S1
# define _ARM_VUITOF64_Q1D1         _ARM64_VUITOF64_Q1D1
# define _ARM_VUITOF64_Q2Q1         _ARM64_VUITOF64_Q2Q1

    /*
     * vector convert floating point to signed/unsigned integer with roundmode
     */
# define _ARM_RND_VFTOSIEX_S        _ARM64_RND_VFTOSIEX_S
# define _ARM_RND_VFTOSIEX_D        _ARM64_RND_VFTOSIEX_D
# define _ARM_RND_VFTOSIEX_Q        _ARM64_RND_VFTOSIEX_Q
# define _ARM_RND_VFTOUIEX_S        _ARM64_RND_VFTOUIEX_S
# define _ARM_RND_VFTOUIEX_D        _ARM64_RND_VFTOUIEX_D
# define _ARM_RND_VFTOUIEX_Q        _ARM64_RND_VFTOUIEX_Q

    /*
     * vector convert double to signed/unsigned integer with roundmode
     */
/* D --> S
   Q --> D
   (Q,Q) --> Q */
//# define _ARM_RND_VF64TOSIEX_S1D1   _ARM64_RND_VF64TOSIEX_S1D1
# define _ARM_RND_VF64TOSIEX_D1Q1   _ARM64_RND_VF64TOSIEX_D1Q1
# define _ARM_RND_VF64TOSIEX_Q1Q2   _ARM64_RND_VF64TOSIEX_Q1Q2
//# define _ARM_RND_VF64TOUIEX_S1D1   _ARM64_RND_VF64TOUIEX_S1D1
# define _ARM_RND_VF64TOUIEX_D1Q1   _ARM64_RND_VF64TOUIEX_D1Q1
# define _ARM_RND_VF64TOUIEX_Q1Q2   _ARM64_RND_VF64TOUIEX_Q1Q2

    /*
     * vector convert floating point to signed/unsigned [long] integer with roundmode
     */
# define _ARM_RND_VFTOSIEX          _ARM64_RND_VFTOSIEX
# define _ARM_RND_VFTOUIEX          _ARM64_RND_VFTOUIEX

    /*
     * vector convert truncate double to signed/unsigned long integer
     */
# define _ARM_VF64TOLI_S(DVRI,SVRI) _ARM_RND_VFTOSIEX_S(64,DVRI,SVRI,Z)
# define _ARM_VF64TOLI_D(DVRI,SVRI) _ARM_RND_VFTOSIEX_D(64,DVRI,SVRI,Z)
# define _ARM_VF64TOLI_Q(DVRI,SVRI) _ARM_RND_VFTOSIEX_Q(64,DVRI,SVRI,Z)
# define _ARM_VF64TOLU_S(DVRI,SVRI) _ARM_RND_VFTOUIEX_S(64,DVRI,SVRI,Z)
# define _ARM_VF64TOLU_D(DVRI,SVRI) _ARM_RND_VFTOUIEX_D(64,DVRI,SVRI,Z)
# define _ARM_VF64TOLU_Q(DVRI,SVRI) _ARM_RND_VFTOUIEX_Q(64,DVRI,SVRI,Z)

    /*
     * vector rounding convert float to signed/unsigned integer
     */
# define _ARM_RND_VFTOSI_D(DVRI,SVRI)   _ARM_RND_VFTOSIEX_D(32,DVRI,SVRI,A)
# define _ARM_RND_VFTOSI_Q(DVRI,SVRI)   _ARM_RND_VFTOSIEX_Q(32,DVRI,SVRI,A)
# define _ARM_RND_VFTOUI_D(DVRI,SVRI)   _ARM_RND_VFTOUIEX_D(32,DVRI,SVRI,A)
# define _ARM_RND_VFTOUI_Q(DVRI,SVRI)   _ARM_RND_VFTOUIEX_Q(32,DVRI,SVRI,A)

    /*
     * vector rounding convert double to signed/unsigned long integer
     */
# define _ARM_RND_VF64TOLI_S(DVRI,SVRI) _ARM_RND_VFTOSIEX_S(64,DVRI,SVRI,A)
# define _ARM_RND_VF64TOLI_D(DVRI,SVRI) _ARM_RND_VFTOSIEX_D(64,DVRI,SVRI,A)
# define _ARM_RND_VF64TOLI_Q(DVRI,SVRI) _ARM_RND_VFTOSIEX_Q(64,DVRI,SVRI,A)
# define _ARM_RND_VF64TOLU_S(DVRI,SVRI) _ARM_RND_VFTOUIEX_S(64,DVRI,SVRI,A)
# define _ARM_RND_VF64TOLU_D(DVRI,SVRI) _ARM_RND_VFTOUIEX_D(64,DVRI,SVRI,A)
# define _ARM_RND_VF64TOLU_Q(DVRI,SVRI) _ARM_RND_VFTOUIEX_Q(64,DVRI,SVRI,A)

/* @} */


/* ========================================================================= */

/*
 * Generic arithmetic operations
 * @{
 *
 * :w  - data width
 * :ww - double data width
 * for instance: w = 8, ww = 16
 *               long operation: ww = w + w
 *               wide operation: ww = ww + w
 *
 * vector negate               : r[i]:w   = -a[i]:w
 *
 * vector add                           : r[i]:w   = a[i]:w + b[i]:w
 * vector add saturated                 : r[i]:w   = saturate(a[i]:w + b[i]:w)
 * vector add long                      : r[i]:ww  = a[i]:w + b[i]:w
 * vector add wide                      : r[i]:ww  = a[i]:ww + b[i]:w
 * vector halving add                   : r[i]:w   = (a[i]:w + b[i]:w) >> 1
 * vector rounding halving add          : r[i]:w   = (a[i]:w + b[i]:w + 1) >> 1
 *
 * vector sub                           : r[i]:w   = a[i]:w - b[i]:w
 * vector sub saturated                 : r[i]:w   = saturate(a[i]:w - b[i]:w)
 * vector sub long                      : r[i]:ww  = a[i]:w - b[i]:w
 * vector sub wide                      : r[i]:ww  = a[i]:ww - b[i]:w
 * vector halving sub                   : r[i]:w   = (a[i]:w - b[i]:w) >> 1
 *
 * vector mul                           : r[i]:w   = a[i]:w * b[i]:w
 * vector mul long                      : r[i]:ww  = a[i]:w * b[i]:w
 * vector mul long saturated            : r[i]:ww  = saturate(a[i]:w * b[i]:w)
 * vector mul accumulate                : r[i]:w  += a[i]:w * b[i]:w
 * vector mul accumulate long           : r[i]:ww += a[i]:w * b[i]:w
 * vector mul accumulate saturated long : r[i]:ww = saturate(r[i]:ww + (a[i]:w * b[i]:w))
 * vector mul subtract                  : r[i]:w  -= a[i]:w * b[i]:w
 * vector mul subtract long             : r[i]:ww -= a[i]:w * b[i]:w
 * vector mul subtract saturated long   : r[i]:ww = saturate(r[i]:ww - (a[i]:w * b[i]:w))
 *
 * vector doubling mul saturated with right shift 16/32          : r = saturate((a * b * 2) >>{U} 16{32})
 * vector rounding doubling mul saturated with right shift 16/32 : r = saturate(round((a * b * 2) >>{U} 16{32}))
 */


    /*
     * VNEGATE
     */

/* negate: Q --> Q
   negate: D --> D */
# define _ARM_VNEGATE_Q             _ARM64_VNEGATE_Q
# define _ARM_VNEGATE_D             _ARM64_VNEGATE_D

# define _ARM_VNEGATEF_Q            _ARM64_VNEGATEF_Q
# define _ARM_VNEGATEF_D            _ARM64_VNEGATEF_D

/* negate saturated: Q --> Q
   negate saturated: D --> D */
# define _ARM_SAT_VNEGATE_Q         _ARM64_SAT_VNEGATE_Q
# define _ARM_SAT_VNEGATE_D         _ARM64_SAT_VNEGATE_D

/* ------------------------------------------------------------------------- */

    /*
     * VADD
     */

/* add: (D,D) --> D
   add: (Q,Q) --> Q */
# define _ARM_VADD_Q                _ARM64_VADD_Q
# define _ARM_VADD_D                _ARM64_VADD_D

/* add< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM_VADDF_Q               _ARM64_VADDF_Q
# define _ARM_VADDF_D               _ARM64_VADDF_D
# define _ARM_VADDF_S               _ARM64_VADDF_S

/* add< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VADDF64_Q             _ARM64_VADDF64_Q
# define _ARM_VADDF64_D             _ARM64_VADDF64_D

/* add long: (D,D) --> Q */
# define _ARM_VADD_WIDEN_Q1D1D1     _ARM64_VADD_WIDEN_Q1D1D1

/* add wide: (Q,D) --> Q */
# define _ARM_VADD_WIDEN_Q1Q1D1     _ARM64_VADD_WIDEN_Q1Q1D1


/* add saturated: (Q,Q) --> Q
   add saturated: (D,D) --> D */
# define _ARM_SAT_VADD_Q            _ARM64_SAT_VADD_Q
# define _ARM_SAT_VADD_D            _ARM64_SAT_VADD_D

/* halving add */
# define _ARM_VHADD_Q               _ARM64_VHADD_Q
# define _ARM_VHADD_D               _ARM64_VHADD_D

/* rounding halving add */
# define _ARM_RND_VHADD_Q           _ARM64_RND_VHADD_Q
# define _ARM_RND_VHADD_D           _ARM64_RND_VHADD_D

/* add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_VADDH_NARROW_D1Q1Q1       _ARM64_VADDH_NARROW_D1Q1Q1

/* rounding add narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_RND_VADDH_NARROW_D1Q1Q1   _ARM64_RND_VADDH_NARROW_D1Q1Q1

/* ------------------------------------------------------------------------- */

    /*
     * VSUB
     */

/* sub: (D,D) --> D
   sub: (Q,Q) --> Q */
# define _ARM_VSUB_Q                _ARM64_VSUB_Q
# define _ARM_VSUB_D                _ARM64_VSUB_D

/* sub< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM_VSUBF_Q               _ARM64_VSUBF_Q
# define _ARM_VSUBF_D               _ARM64_VSUBF_D
# define _ARM_VSUBF_S               _ARM64_VSUBF_S

/* sub< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VSUBF64_Q             _ARM64_VSUBF64_Q
# define _ARM_VSUBF64_D             _ARM64_VSUBF64_D

/* sub long: (D,D) --> Q */
# define _ARM_VSUB_WIDEN_Q1D1D1     _ARM64_VSUB_WIDEN_Q1D1D1

/* sub wide: (Q,D) --> Q */
# define _ARM_VSUB_WIDEN_Q1Q1D1     _ARM64_VSUB_WIDEN_Q1Q1D1


/* sub saturated: (Q,Q) --> Q
   sub saturated: (D,D) --> D */
# define _ARM_SAT_VSUB_Q            _ARM64_SAT_VSUB_Q
# define _ARM_SAT_VSUB_D            _ARM64_SAT_VSUB_D


/* halving sub */
# define _ARM_VHSUB_Q               _ARM64_VHSUB_Q
# define _ARM_VHSUB_D               _ARM64_VHSUB_D


/* sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_VSUBH_NARROW_D1Q1Q1       _ARM64_VSUBH_NARROW_D1Q1Q1

/* rounding sub narrow, selecting high half */
/* narrow: (Q,Q) --> D */
# define _ARM_RND_VSUBH_NARROW_D1Q1Q1   _ARM64_RND_VSUBH_NARROW_D1Q1Q1

/* ------------------------------------------------------------------------- */

    /*
     * VMUL
     */

/* mul: (Q,Q) --> Q
   mul: (D,D) --> D */
# define _ARM_VMUL_Q                _ARM64_VMUL_Q
# define _ARM_VMUL_D                _ARM64_VMUL_D

/* mulS: (Q,D[L]) --> Q
   mulS: (D,D[L]) --> D */
# define _ARM_VMULS_Q               _ARM64_VMULS_Q
# define _ARM_VMULS_D               _ARM64_VMULS_D

/* mul< float >
 * (q,q) --> q */
/* (d,d) --> d */
/* (s,s) --> s */
# define _ARM_VMULF_Q               _ARM64_VMULF_Q
# define _ARM_VMULF_D               _ARM64_VMULF_D
# define _ARM_VMULF_S               _ARM64_VMULF_S

/* mulS< float >
 * (q,d[L]) --> q
   (d,d[L]) --> d  */
# define _ARM_VMULSF_Q              _ARM64_VMULSF_Q
# define _ARM_VMULSF_D              _ARM64_VMULSF_D

/* mul< double >
 * (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VMULF64_Q             _ARM64_VMULF64_Q
# define _ARM_VMULF64_D             _ARM64_VMULF64_D

/* mul long: (D,D) --> Q
   mulS long: (D,D[L]) --> Q */
# define _ARM_VMUL_WIDEN_Q1D1D1     _ARM64_VMUL_WIDEN_Q1D1D1
# define _ARM_VMULS_WIDEN_Q1D1D1    _ARM64_VMULS_WIDEN_Q1D1D1

/* mul wide: (Q,D) --> Q
   mulS wide: (Q,D[L]) --> Q */
# define _ARM_VMUL_WIDEN_Q1Q1D1     _ARM64_VMUL_WIDEN_Q1Q1D1
# define _ARM_VMULS_WIDEN_Q1Q1D1    _ARM64_VMULS_WIDEN_Q1Q1D1


/* doubling mul saturated long: (D,D) --> Q; domain: {int16, int32} */
# define _ARM_SAT_VMUL2_WIDEN_Q1D1D1    _ARM64_SAT_VMUL2_WIDEN_Q1D1D1


/* doubling mul saturated, storing only high half
   with right shift 16/32 : r = saturate((a * b * 2) >> 16{32}) */
# define _ARM_SAT_VMUL2H_Q          _ARM64_SAT_VMUL2H_Q
# define _ARM_SAT_VMUL2H_D          _ARM64_SAT_VMUL2H_D

/* rounding doubling mul saturated, storing only high half
   with right shift 16/32: r = saturate(round((a * b * 2) >> 16{32})) */
# define _ARM_SAT_RND_VMUL2H_Q      _ARM64_SAT_RND_VMUL2H_Q
# define _ARM_SAT_RND_VMUL2H_D      _ARM64_SAT_RND_VMUL2H_D


/* mul accumulate/subtract: (Q,Q) --> Q
   mul accumulate/subtract: (D,D) --> D
   mulS accumulate/subtract: (Q,D[L]) --> Q
   mulS accumulate/subtract: (D,D[L]) --> D */
# define _ARM_VMUL_ACCUM_Q          _ARM64_VMUL_ACCUM_Q
# define _ARM_VMUL_ACCUM_D          _ARM64_VMUL_ACCUM_D
# define _ARM_VMULS_ACCUM_Q         _ARM64_VMULS_ACCUM_Q
# define _ARM_VMULS_ACCUM_D         _ARM64_VMULS_ACCUM_D
# define _ARM_VMUL_SUBTR_Q          _ARM64_VMUL_SUBTR_Q
# define _ARM_VMUL_SUBTR_D          _ARM64_VMUL_SUBTR_D
# define _ARM_VMULS_SUBTR_Q         _ARM64_VMULS_SUBTR_Q
# define _ARM_VMULS_SUBTR_D         _ARM64_VMULS_SUBTR_D

/* mul accumulate/subtract< float >
   (q,q) --> q
   (d,d) --> d
   (q,d[L]) --> q
   (d,d[L]) --> d */
# define _ARM_VMULF_ACCUM_Q         _ARM64_VMULF_ACCUM_Q
# define _ARM_VMULF_ACCUM_D         _ARM64_VMULF_ACCUM_D
# define _ARM_VMULSF_ACCUM_Q        _ARM64_VMULSF_ACCUM_Q
# define _ARM_VMULSF_ACCUM_D        _ARM64_VMULSF_ACCUM_D
# define _ARM_VMULF_SUBTR_Q         _ARM64_VMULF_SUBTR_Q
# define _ARM_VMULF_SUBTR_D         _ARM64_VMULF_SUBTR_D
# define _ARM_VMULSF_SUBTR_Q        _ARM64_VMULSF_SUBTR_Q
# define _ARM_VMULSF_SUBTR_D        _ARM64_VMULSF_SUBTR_D

/* mul accumulate/subtract< double >
   (q,q) --> q
   (d,d) --> d */
# define _ARM_VMULF64_ACCUM_Q       _ARM64_VMULF64_ACCUM_Q
# define _ARM_VMULF64_ACCUM_D       _ARM64_VMULF64_ACCUM_D
# define _ARM_VMULF64_SUBTR_Q       _ARM64_VMULF64_SUBTR_Q
# define _ARM_VMULF64_SUBTR_D       _ARM64_VMULF64_SUBTR_D


/* mul accumulate/subtract long: (D,D) --> Q
   mulS accumulate/subtract long: (D,D[L]) --> Q */
# define _ARM_VMUL_ACCUM_WIDEN_Q1D1D1   _ARM64_VMUL_ACCUM_WIDEN_Q1D1D1
# define _ARM_VMULS_ACCUM_WIDEN_Q1D1D1  _ARM64_VMULS_ACCUM_WIDEN_Q1D1D1
# define _ARM_VMUL_SUBTR_WIDEN_Q1D1D1   _ARM64_VMUL_SUBTR_WIDEN_Q1D1D1
# define _ARM_VMULS_SUBTR_WIDEN_Q1D1D1  _ARM64_VMULS_SUBTR_WIDEN_Q1D1D1


/* doubling mul accumulate/subtract saturated long: (D,D) --> Q; domain: {int16, int32} */
# define _ARM_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1  _ARM64_SAT_VMUL2_ACCUM_WIDEN_Q1D1D1
# define _ARM_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1  _ARM64_SAT_VMUL2_SUBTR_WIDEN_Q1D1D1


# define _ARM_VFMAEX                _ARM64_VFMAEX
# define _ARM_VFMSEX                _ARM64_VFMSEX

/* ------------------------------------------------------------------------- */

    /*
     * VDIV
     */

/* note: only for F32 */
# define _ARM_VDIVF_Q               _ARM64_VDIVF_Q
# define _ARM_VDIVF_D               _ARM64_VDIVF_D
# define _ARM_VDIVF_S               _ARM64_VDIVF_S

/* note: only for F64 */
# define _ARM_VDIVF64_Q             _ARM64_VDIVF64_Q
# define _ARM_VDIVF64_D             _ARM64_VDIVF64_D

/* @} */


/* ========================================================================= */

/*
 * Min/Max operations
 * @{
 *
 * r[i] = min(s1[i],s2[i])
 * r[i] = max(s1[i],s2[i])
 */

    /*
     * VMIN
     */

# define _ARM_VMIN_D                _ARM64_VMIN_D
# define _ARM_VMIN_Q                _ARM64_VMIN_Q

# define _ARM_VMINF_D               _ARM64_VMINF_D
# define _ARM_VMINF_Q               _ARM64_VMINF_Q

/* ------------------------------------------------------------------------- */

    /*
     * VMAX
     */

# define _ARM_VMAX_D                _ARM64_VMAX_D
# define _ARM_VMAX_Q                _ARM64_VMAX_Q

# define _ARM_VMAXF_D               _ARM64_VMAXF_D
# define _ARM_VMAXF_Q               _ARM64_VMAXF_Q

/* @} */


/* ========================================================================= */

/*
 * Folding operations: add, min, max
 * @{
 *
 * vector pairwise add [long] :
 *     (a,b) --> r : {r[i]} = {(a[2*i] + a[2*i + 1]),... (b[2*j] + b[2*j + 1])}
 *
 * vector pairwise add accumulate long :
 *     {r[i]} = {(r[i] + a[2*i] + a[2*i + 1]),... (r[j] + b[2*j] + b[2*j + 1])}
 *
 * vector add across vector :
 *     (a,b) --> r[0] : r[0] = (a[0] + a[1] + ... + a[n - 1])
 */

    /*
     * PADD, PADD long, PADD accumulate long, VADD, VADD long
     */

/* (q,q) --> q */
/* (d,d) --> d */
# define _ARM_VPADD_Q               _ARM64_VPADD_Q
# define _ARM_VPADD_D               _ARM64_VPADD_D
/* q --> q(result in low half(d))
 * @note: high half can be modified
 */
# define _ARM_VPADD_Q1              _ARM64_VPADD_Q1

/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM_VPADD_WIDEN_Q         _ARM64_VPADD_WIDEN_Q
# define _ARM_VPADD_WIDEN_D         _ARM64_VPADD_WIDEN_D

/* (q{dn,dn+1}) --> q */
/* (hi-d,lo-d) --> d */
# define _ARM_VPADD_ACCUM_WIDEN_Q   _ARM64_VPADD_ACCUM_WIDEN_Q
# define _ARM_VPADD_ACCUM_WIDEN_D   _ARM64_VPADD_ACCUM_WIDEN_D

/* (q,q) --> q[0] */
/* (d,d) --> d[0] */
# define _ARM_VVADD_Q               _ARM64_VVADD_Q
# define _ARM_VVADD_D               _ARM64_VVADD_D

/* (q{dn,dn+1}) --> q[0] */
/* (hi-d,lo-d) --> d[0] */
# define _ARM_VVADD_WIDEN_Q         _ARM64_VVADD_WIDEN_Q
# define _ARM_VVADD_WIDEN_D         _ARM64_VVADD_WIDEN_D

/* ------------------------------------------------------------------------- */

    /*
     * PMIN, VMIN
     */

# define _ARM_VPMIN_Q               _ARM64_VPMIN_Q
# define _ARM_VPMIN_D               _ARM64_VPMIN_D
# define _ARM_VPMIN_Q1              _ARM64_VPMIN_Q1

/* pmin< float > */
# define _ARM_VPMINF_Q              _ARM64_VPMINF_Q
# define _ARM_VPMINF_D              _ARM64_VPMINF_D
# define _ARM_VPMINF_Q1             _ARM64_VPMINF_Q1

# define _ARM_VVMIN_Q               _ARM64_VVMIN_Q
# define _ARM_VVMIN_D               _ARM64_VVMIN_D

/* ------------------------------------------------------------------------- */

    /*
     * PMAX, VMAX
     */

# define _ARM_VPMAX_Q               _ARM64_VPMAX_Q
# define _ARM_VPMAX_D               _ARM64_VPMAX_D
# define _ARM_VPMAX_Q1              _ARM64_VPMAX_Q1

/* pmax< float > */
# define _ARM_VPMAXF_Q              _ARM64_VPMAXF_Q
# define _ARM_VPMAXF_D              _ARM64_VPMAXF_D
# define _ARM_VPMAXF_Q1             _ARM64_VPMAXF_Q1

# define _ARM_VVMAX_Q               _ARM64_VVMAX_Q
# define _ARM_VVMAX_D               _ARM64_VVMAX_D

/* ------------------------------------------------------------------------- */

    /*
     * VMINMAX
     */

# define _ARM_VVMINMAX_Q            _ARM64_VVMINMAX_Q
# define _ARM_VVMINMAX_D            _ARM64_VVMINMAX_D

/* @} */


/* ========================================================================= */

/*
 * Comparing operations
 * @{
 */

    /*
     * Compare 2 vector registers
     * Op: LT, LE, EQ, GE, GT
     */

# define _ARM_VCMPI_Q               _ARM64_VCMPI_Q
# define _ARM_VCMPI_D               _ARM64_VCMPI_D

# define _ARM_VCMP_Q                _ARM64_VCMP_Q
# define _ARM_VCMP_D                _ARM64_VCMP_D


/*
 * float
 */

# define _ARM_VCMPFI_Q              _ARM64_VCMPFI_Q
# define _ARM_VCMPFI_D              _ARM64_VCMPFI_D

# define _ARM_VCMPF_Q               _ARM64_VCMPF_Q
# define _ARM_VCMPF_D               _ARM64_VCMPF_D


/*
 * Float-Point Subsystem
 *
 * 32-bit operations  -  S registers
 * 64-bit operations  -  D registers
 */

# define _ARM_VFPCMP32              _ARM64_VFPCMP32
# define _ARM_VFPCMP64              _ARM64_VFPCMP64
# define _ARM_VFPCMP32_IF           _ARM64_VFPCMP32_IF
# define _ARM_VFPCMP64_IF           _ARM64_VFPCMP64_IF

# define _ARM_VFPCMP32_UCF          _ARM64_VFPCMP32_UCF
# define _ARM_VFPCMP64_UCF          _ARM64_VFPCMP64_UCF
# define _ARM_VFPCMP32_UCF_IF       _ARM64_VFPCMP32_UCF_IF
# define _ARM_VFPCMP64_UCF_IF       _ARM64_VFPCMP64_UCF_IF

/* @} */


/* ========================================================================= */

/*
 * Logical bitwise operations
 * @{
 *
 * vector not
 * vector and
 * vector and complement: BIC
 * vector or
 * vector or complement: ORN
 * vector xor
 * vector set all bits
 * vector clear all bits
 * vector bit select           : bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
 */

    /*
     * AND
     */

# define _ARM_VAND_Q                _ARM64_VAND_Q
# define _ARM_VAND_D                _ARM64_VAND_D

    /*
     * BIC : AND complement
     */

# define _ARM_VBIC_Q                _ARM64_VBIC_Q
# define _ARM_VBIC_D                _ARM64_VBIC_D

    /*
     * OR
     */

# define _ARM_VORR_Q                _ARM64_VORR_Q
# define _ARM_VORR_D                _ARM64_VORR_D

    /*
     * ORN : OR complement
     */

# define _ARM_VORN_Q                _ARM64_VORN_Q
# define _ARM_VORN_D                _ARM64_VORN_D

    /*
     * XOR
     */

# define _ARM_VXOR_Q                _ARM64_VXOR_Q
# define _ARM_VXOR_D                _ARM64_VXOR_D

    /*
     * Set/Clear all bits
     */

# define _ARM_VBITSET_Q             _ARM64_VBITSET_Q
# define _ARM_VBITSET_D             _ARM64_VBITSET_D
# define _ARM_VBITCLR_Q             _ARM64_VBITCLR_Q
# define _ARM_VBITCLR_D             _ARM64_VBITCLR_D


    /*
     * Bit select: bit(i, dst) = (bit(i, dst) ? bit(i, src1) : bit(i, src2))
     * Bit set if true:
     * Bit set if false:
     */

# define _ARM_VBITSEL_Q             _ARM64_VBITSEL_Q
# define _ARM_VBITSEL_D             _ARM64_VBITSEL_D

# define _ARM_VBITIFT_Q             _ARM64_VBITIFT_Q
# define _ARM_VBITIFT_D             _ARM64_VBITIFT_D

# define _ARM_VBITIFF_Q             _ARM64_VBITIFF_Q
# define _ARM_VBITIFF_D             _ARM64_VBITIFF_D


    /*
     * Bit count leading 0/1:
     */

# define _ARM_VCOUNTLEADZEROBITS_Q  _ARM64_VCOUNTLEADZEROBITS_Q
# define _ARM_VCOUNTLEADZEROBITS_D  _ARM64_VCOUNTLEADZEROBITS_D

# define _ARM_VCOUNTLEADSETBITS_Q   _ARM64_VCOUNTLEADSETBITS_Q
# define _ARM_VCOUNTLEADSETBITS_D   _ARM64_VCOUNTLEADSETBITS_D

/* @} */


/* ========================================================================= */

/*
 * Shift operations
 * @{
 *
 * vector shift left by immediate
 * vector shift left saturated by immediate
 * vector shift left long by immediate
 *
 * (negative shift-value shift right)
 * vector shift left by signed var
 * vector rounding shift left by signed
 * vector shift left saturated by signed
 * vector rounding shift left saturated by signed
 *
 * vector [rounding] shift right by immediate
 * vector [rounding] shift right narrow by immediate
 * vector saturating [rounding] shift right narrow by immediate
 */

    /*
     * VSHL by immediate
     */

# define _ARM_VSHLI_Q               _ARM64_VSHLI_Q
# define _ARM_VSHLI_D               _ARM64_VSHLI_D


/* shift saturated */
# define _ARM_SAT_VSHLI_Q           _ARM64_SAT_VSHLI_Q
# define _ARM_SAT_VSHLI_D           _ARM64_SAT_VSHLI_D

/* signed shift saturated */
# define _ARM_SAT_VSHLSI_Q          _ARM64_SAT_VSHLSI_Q
# define _ARM_SAT_VSHLSI_D          _ARM64_SAT_VSHLSI_D

/* unsigned shift saturated */
# define _ARM_SAT_VSHLUI_Q          _ARM64_SAT_VSHLUI_Q
# define _ARM_SAT_VSHLUI_D          _ARM64_SAT_VSHLUI_D


/*
 * shift long
 */

# define _ARM_VSHLI_WIDEN_Q1D1      _ARM64_VSHLI_WIDEN_Q1D1

/* signed shift long */
# define _ARM_VSHLSI_WIDEN_Q1D1     _ARM64_VSHLSI_WIDEN_Q1D1

/* unsigned shift long */
# define _ARM_VSHLUI_WIDEN_Q1D1     _ARM64_VSHLUI_WIDEN_Q1D1

/* ------------------------------------------------------------------------- */

    /*
     * VSHL by signed var (negative shift-value shift right)
     */

/* shift left */
# define _ARM_VSHL_Q                _ARM64_VSHL_Q
# define _ARM_VSHL_D                _ARM64_VSHL_D

/* signed shift left */
# define _ARM_VSHLS_Q               _ARM64_VSHLS_Q
# define _ARM_VSHLS_D               _ARM64_VSHLS_D

/* unsigned shift left */
# define _ARM_VSHLU_Q               _ARM64_VSHLU_Q
# define _ARM_VSHLU_D               _ARM64_VSHLU_D


/* shift left saturated */
# define _ARM_SAT_VSHL_Q            _ARM64_SAT_VSHL_Q
# define _ARM_SAT_VSHL_D            _ARM64_SAT_VSHL_D

/* signed shift left saturated */
# define _ARM_SAT_VSHLS_Q           _ARM64_SAT_VSHLS_Q
# define _ARM_SAT_VSHLS_D           _ARM64_SAT_VSHLS_D

/* unsigned shift left saturated */
# define _ARM_SAT_VSHLU_Q           _ARM64_SAT_VSHLU_Q
# define _ARM_SAT_VSHLU_D           _ARM64_SAT_VSHLU_D


/* rounding shift left */
# define _ARM_RND_VSHL_Q            _ARM64_RND_VSHL_Q
# define _ARM_RND_VSHL_D            _ARM64_RND_VSHL_D

/* signed rounding shift left */
# define _ARM_RND_VSHLS_Q           _ARM64_RND_VSHLS_Q
# define _ARM_RND_VSHLS_D           _ARM64_RND_VSHLS_D

/* unsigned rounding shift left */
# define _ARM_RND_VSHLU_Q           _ARM64_RND_VSHLU_Q
# define _ARM_RND_VSHLU_D           _ARM64_RND_VSHLU_D


/* rounding shift left saturated */
# define _ARM_SAT_RND_VSHL_Q        _ARM64_SAT_RND_VSHL_Q
# define _ARM_SAT_RND_VSHL_D        _ARM64_SAT_RND_VSHL_D

/* signed rounding shift left saturated */
# define _ARM_SAT_RND_VSHLS_Q       _ARM64_SAT_RND_VSHLS_Q
# define _ARM_SAT_RND_VSHLS_D       _ARM64_SAT_RND_VSHLS_D

/* unsigned rounding shift left saturated */
# define _ARM_SAT_RND_VSHLU_Q       _ARM64_SAT_RND_VSHLU_Q
# define _ARM_SAT_RND_VSHLU_D       _ARM64_SAT_RND_VSHLU_D

/* ------------------------------------------------------------------------- */

    /*
     * VSHR by immediate
     */

/* shift right */
# define _ARM_VSHRI_Q               _ARM64_VSHRI_Q
# define _ARM_VSHRI_D               _ARM64_VSHRI_D

/* signed shift right */
# define _ARM_VSHRSI_Q              _ARM64_VSHRSI_Q
# define _ARM_VSHRSI_D              _ARM64_VSHRSI_D

/* unsigned shift right */
# define _ARM_VSHRUI_Q              _ARM64_VSHRUI_Q
# define _ARM_VSHRUI_D              _ARM64_VSHRUI_D


/* rounding shift right */
# define _ARM_RND_VSHRI_Q           _ARM64_RND_VSHRI_Q
# define _ARM_RND_VSHRI_D           _ARM64_RND_VSHRI_D

/* signed rounding shift right */
# define _ARM_RND_VSHRSI_Q          _ARM64_RND_VSHRSI_Q
# define _ARM_RND_VSHRSI_D          _ARM64_RND_VSHRSI_D

/* unsigned rounding shift right */
# define _ARM_RND_VSHRUI_Q          _ARM64_RND_VSHRUI_Q
# define _ARM_RND_VSHRUI_D          _ARM64_RND_VSHRUI_D


/* [signed/unsigned] shift right narrow */
/* narrow: Q --> D */
# define _ARM_VSHRI_NARROW_D1Q1     _ARM64_VSHRI_NARROW_D1Q1
//# define _ARM_VSHRSI_NARROW_D1Q1    _ARM64_VSHRSI_NARROW_D1Q1
//# define _ARM_VSHRUI_NARROW_D1Q1    _ARM64_VSHRUI_NARROW_D1Q1


/* [signed/unsigned] rounding shift right narrow */
/* narrow: Q --> D */
# define _ARM_RND_VSHRI_NARROW_D1Q1 _ARM64_RND_VSHRI_NARROW_D1Q1
//# define _ARM_RND_VSHRSI_NARROW_D1Q1    _ARM64_RND_VSHRSI_NARROW_D1Q1
//# define _ARM_RND_VSHRUI_NARROW_D1Q1    _ARM64_RND_VSHRUI_NARROW_D1Q1


/* [signed/unsigned] shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM_SAT_VSHRI_NARROW_D1Q1     _ARM64_SAT_VSHRI_NARROW_D1Q1
# define _ARM_SAT_VSHRSI_NARROW_D1Q1    _ARM64_SAT_VSHRSI_NARROW_D1Q1
# define _ARM_SAT_VSHRUI_NARROW_D1Q1    _ARM64_SAT_VSHRUI_NARROW_D1Q1


/* [signed/unsigned] rounding shift right narrow saturated */
/* narrow: Q --> D */
# define _ARM_SAT_RND_VSHRI_NARROW_D1Q1     _ARM64_SAT_RND_VSHRI_NARROW_D1Q1
# define _ARM_SAT_RND_VSHRSI_NARROW_D1Q1    _ARM64_SAT_RND_VSHRSI_NARROW_D1Q1
# define _ARM_SAT_RND_VSHRUI_NARROW_D1Q1    _ARM64_SAT_RND_VSHRUI_NARROW_D1Q1

/* ------------------------------------------------------------------------- */

    /*
     * VSHR and Accumulate by immediate
     */

/* shift right */
# define _ARM_VSHRADDI_Q            _ARM64_VSHRADDI_Q
# define _ARM_VSHRADDI_D            _ARM64_VSHRADDI_D

/* signed shift right */
# define _ARM_VSHRADDSI_Q           _ARM64_VSHRADDSI_Q
# define _ARM_VSHRADDSI_D           _ARM64_VSHRADDSI_D

/* unsigned shift right */
# define _ARM_VSHRADDUI_Q           _ARM64_VSHRADDUI_Q
# define _ARM_VSHRADDUI_D           _ARM64_VSHRADDUI_D


/* rounding shift right */
# define _ARM_RND_VSHRADDI_Q        _ARM64_RND_VSHRADDI_Q
# define _ARM_RND_VSHRADDI_D        _ARM64_RND_VSHRADDI_D

/* signed rounding shift right */
# define _ARM_RND_VSHRADDSI_Q       _ARM64_RND_VSHRADDSI_Q
# define _ARM_RND_VSHRADDSI_D       _ARM64_RND_VSHRADDSI_D

/* unsigned rounding shift right */
# define _ARM_RND_VSHRADDUI_Q       _ARM64_RND_VSHRADDUI_Q
# define _ARM_RND_VSHRADDUI_D       _ARM64_RND_VSHRADDUI_D

/* @} */


/* ========================================================================= */

/*
 * Generic data processing operations
 * @{
 *
 * vector element shift left by immediate
 * vector element shift right by immediate
 * vector element rotate left by immediate
 * vector element rotate right by immediate
 *
 * interleaves the elements of two vector (zip)
 * de-interleaves the elements of two vector (unzip)
 */

    /*
     * VEXTRACT
     */

# define _ARM_VEXTRACT_Q            _ARM64_VEXTRACT_Q
# define _ARM_VEXTRACT_D            _ARM64_VEXTRACT_D

/* ------------------------------------------------------------------------- */

    /*
     * VZIP/VUZP (zip/unzip)
     */

/*
 * vector elements: {0,1,2,3,4,5,6,7}
 *
 * zip1(d1,s1,s2):
 * zip2(d2,s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    d1: {a4,b4,a5,b5,a6,b6,a7,b7};
 *    d2: {a0,b0,a1,b1,a2,b2,a3,b3};
 *
 * unzip1(d1,s1,s2):
 * unzip2(d2,s1,s2):
 * input:
 *    s1: {a0,a1,a2,a3,a4,a5,a6,a7};
 *    s2: {b0,b1,b2,b3,b4,b5,b6,b7};
 * output:
 *    d1: {a0,a2,a4,a6,b0,b2,b4,b6};
 *    d2: {a1,a3,a5,a7,b1,b3,b5,b7};
 */
# define _ARM_VZIP1_Q                   _ARM64_VZIP1_Q
# define _ARM_VZIP2_Q                   _ARM64_VZIP2_Q
# define _ARM_VZIP1_D                   _ARM64_VZIP1_D
# define _ARM_VZIP2_D                   _ARM64_VZIP2_D
# define _ARM_VUZP1_Q                   _ARM64_VUZP1_Q
# define _ARM_VUZP2_Q                   _ARM64_VUZP2_Q
# define _ARM_VUZP1_D                   _ARM64_VUZP1_D
# define _ARM_VUZP2_D                   _ARM64_VUZP2_D

/* ------------------------------------------------------------------------- */

    /*
     * VTRANSPOSE2x2
     */
# define _ARM_VTRANSPOSE1_Q             _ARM64_VTRANSPOSE1_Q
# define _ARM_VTRANSPOSE2_Q             _ARM64_VTRANSPOSE2_Q
# define _ARM_VTRANSPOSE1_D             _ARM64_VTRANSPOSE1_D
# define _ARM_VTRANSPOSE2_D             _ARM64_VTRANSPOSE2_D

/* ------------------------------------------------------------------------- */

    /*
     * VSWAP
     */
# define _ARM_VSWAP_Q                   _ARM64_VSWAP_Q
# define _ARM_VSWAP_D                   _ARM64_VSWAP_D

/* @} */


/* ========================================================================= */

/*
 * Memory access
 * @{
 *
 * vector load multiple N-element structures into one or more registers (with de-interleaving when N>1)
 * vector store multiple N-element structures from one or more registers (with interleaving when N>1)
 *
 * vector load single N-element structure into one or more registers (with de-interleaving when N>1)
 * (Multiple copies of one n-element structure are loaded into one ore more registers).
 */


/*
 * Push/Pop
 */

	/*
     * Push/Pop fdd registers
     */

# define _ARM_VPUSH_ALLFDD          _ARM64_VPUSH_ALLFDD
# define _ARM_VPUSH_AVAFDD          _ARM64_VPUSH_AVAFDD
# define _ARM_VPUSH_FDD1            _ARM64_VPUSH_FDD1
# define _ARM_VPUSH_FDD2            _ARM64_VPUSH_FDD2
# define _ARM_VPUSH_FDD3            _ARM64_VPUSH_FDD3
# define _ARM_VPUSH_FDD4            _ARM64_VPUSH_FDD4
# define _ARM_VPUSH_FDD5            _ARM64_VPUSH_FDD5
# define _ARM_VPUSH_FDD6            _ARM64_VPUSH_FDD6
# define _ARM_VPUSH_FDD7            _ARM64_VPUSH_FDD7
# define _ARM_VPUSH_FDD8            _ARM64_VPUSH_FDD8
# define _ARM_VPUSH_FDD9            _ARM64_VPUSH_FDD9
# define _ARM_VPUSH_FDD10           _ARM64_VPUSH_FDD10
# define _ARM_VPUSH_FDD11           _ARM64_VPUSH_FDD11
# define _ARM_VPUSH_FDD12           _ARM64_VPUSH_FDD12
# define _ARM_VPUSH_FDD13           _ARM64_VPUSH_FDD13
# define _ARM_VPUSH_FDD14           _ARM64_VPUSH_FDD14
# define _ARM_VPUSH_FDD15           _ARM64_VPUSH_FDD15
# define _ARM_VPUSH_FDD16           _ARM64_VPUSH_FDD16

# define _ARM_VPOP_ALLFDD           _ARM64_VPOP_ALLFDD
# define _ARM_VPOP_AVAFDD           _ARM64_VPOP_AVAFDD
# define _ARM_VPOP_FDD1             _ARM64_VPOP_FDD1
# define _ARM_VPOP_FDD2             _ARM64_VPOP_FDD2
# define _ARM_VPOP_FDD3             _ARM64_VPOP_FDD3
# define _ARM_VPOP_FDD4             _ARM64_VPOP_FDD4
# define _ARM_VPOP_FDD5             _ARM64_VPOP_FDD5
# define _ARM_VPOP_FDD6             _ARM64_VPOP_FDD6
# define _ARM_VPOP_FDD7             _ARM64_VPOP_FDD7
# define _ARM_VPOP_FDD8             _ARM64_VPOP_FDD8
# define _ARM_VPOP_FDD9             _ARM64_VPOP_FDD9
# define _ARM_VPOP_FDD10            _ARM64_VPOP_FDD10
# define _ARM_VPOP_FDD11            _ARM64_VPOP_FDD11
# define _ARM_VPOP_FDD12            _ARM64_VPOP_FDD12
# define _ARM_VPOP_FDD13            _ARM64_VPOP_FDD13
# define _ARM_VPOP_FDD14            _ARM64_VPOP_FDD14
# define _ARM_VPOP_FDD15            _ARM64_VPOP_FDD15
# define _ARM_VPOP_FDD16            _ARM64_VPOP_FDD16

	/*
     * Push/Pop vector registers
     */

# define _ARM_VPUSH_ALL             _ARM64_VPUSH_ALL

# define _ARM_VPUSH_D1              _ARM64_VPUSH_D1
# define _ARM_VPUSH_D2              _ARM64_VPUSH_D2
# define _ARM_VPUSH_D3              _ARM64_VPUSH_D3
# define _ARM_VPUSH_D4              _ARM64_VPUSH_D4
# define _ARM_VPUSH_D5              _ARM64_VPUSH_D5
# define _ARM_VPUSH_D6              _ARM64_VPUSH_D6
# define _ARM_VPUSH_D7              _ARM64_VPUSH_D7
# define _ARM_VPUSH_D8              _ARM64_VPUSH_D8
# define _ARM_VPUSH_D9              _ARM64_VPUSH_D9
# define _ARM_VPUSH_D10             _ARM64_VPUSH_D10
# define _ARM_VPUSH_D11             _ARM64_VPUSH_D11
# define _ARM_VPUSH_D12             _ARM64_VPUSH_D12
# define _ARM_VPUSH_D13             _ARM64_VPUSH_D13
# define _ARM_VPUSH_D14             _ARM64_VPUSH_D14
# define _ARM_VPUSH_D15             _ARM64_VPUSH_D15
# define _ARM_VPUSH_D16             _ARM64_VPUSH_D16

# define _ARM_VPUSH_Q1              _ARM64_VPUSH_Q1
# define _ARM_VPUSH_Q2              _ARM64_VPUSH_Q2
# define _ARM_VPUSH_Q3              _ARM64_VPUSH_Q3
# define _ARM_VPUSH_Q4              _ARM64_VPUSH_Q4
# define _ARM_VPUSH_Q5              _ARM64_VPUSH_Q5
# define _ARM_VPUSH_Q6              _ARM64_VPUSH_Q6
# define _ARM_VPUSH_Q7              _ARM64_VPUSH_Q7
# define _ARM_VPUSH_Q8              _ARM64_VPUSH_Q8

# define _ARM_VPOP_ALL              _ARM64_VPOP_ALL

# define _ARM_VPOP_D1               _ARM64_VPOP_D1
# define _ARM_VPOP_D2               _ARM64_VPOP_D2
# define _ARM_VPOP_D3               _ARM64_VPOP_D3
# define _ARM_VPOP_D4               _ARM64_VPOP_D4
# define _ARM_VPOP_D5               _ARM64_VPOP_D5
# define _ARM_VPOP_D6               _ARM64_VPOP_D6
# define _ARM_VPOP_D7               _ARM64_VPOP_D7
# define _ARM_VPOP_D8               _ARM64_VPOP_D8
# define _ARM_VPOP_D9               _ARM64_VPOP_D9
# define _ARM_VPOP_D10              _ARM64_VPOP_D10
# define _ARM_VPOP_D11              _ARM64_VPOP_D11
# define _ARM_VPOP_D12              _ARM64_VPOP_D12
# define _ARM_VPOP_D13              _ARM64_VPOP_D13
# define _ARM_VPOP_D14              _ARM64_VPOP_D14
# define _ARM_VPOP_D15              _ARM64_VPOP_D15
# define _ARM_VPOP_D16              _ARM64_VPOP_D16

# define _ARM_VPOP_Q1               _ARM64_VPOP_Q1
# define _ARM_VPOP_Q2               _ARM64_VPOP_Q2
# define _ARM_VPOP_Q3               _ARM64_VPOP_Q3
# define _ARM_VPOP_Q4               _ARM64_VPOP_Q4
# define _ARM_VPOP_Q5               _ARM64_VPOP_Q5
# define _ARM_VPOP_Q6               _ARM64_VPOP_Q6
# define _ARM_VPOP_Q7               _ARM64_VPOP_Q7
# define _ARM_VPOP_Q8               _ARM64_VPOP_Q8

/*
 * Load/Store multiple N-element structures
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn) or (Qn,Qn+1)
     */

# define _ARM_VLD1_D1               _ARM64_VLD1_D1
# define _ARM_VLD1_D2               _ARM64_VLD1_D2
# define _ARM_VLD1_D3               _ARM64_VLD1_D3
# define _ARM_VLD1_D4               _ARM64_VLD1_D4

# define _ARM_VLD1_Q1               _ARM64_VLD1_Q1
# define _ARM_VLD1_Q2               _ARM64_VLD1_Q2
# define _ARM_VLD1_Q3               _ARM64_VLD1_Q3
# define _ARM_VLD1_Q4               _ARM64_VLD1_Q4


# define _ARM_VST1_D1               _ARM64_VST1_D1
# define _ARM_VST1_D2               _ARM64_VST1_D2
# define _ARM_VST1_D3               _ARM64_VST1_D3
# define _ARM_VST1_D4               _ARM64_VST1_D4

# define _ARM_VST1_Q1               _ARM64_VST1_Q1
# define _ARM_VST1_Q2               _ARM64_VST1_Q2
# define _ARM_VST1_Q3               _ARM64_VST1_Q3
# define _ARM_VST1_Q4               _ARM64_VST1_Q4


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD1_D1_AIA           _ARM64_VLD1_D1_AIA
# define _ARM_VLD1_D2_AIA           _ARM64_VLD1_D2_AIA
# define _ARM_VLD1_D3_AIA           _ARM64_VLD1_D3_AIA
# define _ARM_VLD1_D4_AIA           _ARM64_VLD1_D4_AIA

# define _ARM_VLD1_Q1_AIA           _ARM64_VLD1_Q1_AIA
# define _ARM_VLD1_Q2_AIA           _ARM64_VLD1_Q2_AIA
# define _ARM_VLD1_Q3_AIA           _ARM64_VLD1_Q3_AIA
# define _ARM_VLD1_Q4_AIA           _ARM64_VLD1_Q4_AIA


# define _ARM_VST1_D1_AIA           _ARM64_VST1_D1_AIA
# define _ARM_VST1_D2_AIA           _ARM64_VST1_D2_AIA
# define _ARM_VST1_D3_AIA           _ARM64_VST1_D3_AIA
# define _ARM_VST1_D4_AIA           _ARM64_VST1_D4_AIA

# define _ARM_VST1_Q1_AIA           _ARM64_VST1_Q1_AIA
# define _ARM_VST1_Q2_AIA           _ARM64_VST1_Q2_AIA
# define _ARM_VST1_Q3_AIA           _ARM64_VST1_Q3_AIA
# define _ARM_VST1_Q4_AIA           _ARM64_VST1_Q4_AIA


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1)
		 */

# define _ARM_VLD1_D1_IA            _ARM64_VLD1_D1_IA
# define _ARM_VLD1_D2_IA            _ARM64_VLD1_D2_IA
# define _ARM_VLD1_D3_IA            _ARM64_VLD1_D3_IA
# define _ARM_VLD1_D4_IA            _ARM64_VLD1_D4_IA

# define _ARM_VLD1_Q1_IA            _ARM64_VLD1_Q1_IA
# define _ARM_VLD1_Q2_IA            _ARM64_VLD1_Q2_IA
# define _ARM_VLD1_Q3_IA            _ARM64_VLD1_Q3_IA
# define _ARM_VLD1_Q4_IA            _ARM64_VLD1_Q4_IA


# define _ARM_VST1_D1_IA            _ARM64_VST1_D1_IA
# define _ARM_VST1_D2_IA            _ARM64_VST1_D2_IA
# define _ARM_VST1_D3_IA            _ARM64_VST1_D3_IA
# define _ARM_VST1_D4_IA            _ARM64_VST1_D4_IA

# define _ARM_VST1_Q1_IA            _ARM64_VST1_Q1_IA
# define _ARM_VST1_Q2_IA            _ARM64_VST1_Q2_IA
# define _ARM_VST1_Q3_IA            _ARM64_VST1_Q3_IA
# define _ARM_VST1_Q4_IA            _ARM64_VST1_Q4_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
	 * (VRI0, ...): (Qn,Qn+1)
     */

# define _ARM_VLD2_D2               _ARM64_VLD2_D2
# define _ARM_VLD2_D4               _ARM64_VLD2_D4

# define _ARM_VLD2_Q2               _ARM64_VLD2_Q2
# define _ARM_VLD2_Q4               _ARM64_VLD2_Q4


# define _ARM_VST2_D2               _ARM64_VST2_D2
# define _ARM_VST2_D4               _ARM64_VST2_D4

# define _ARM_VST2_Q2               _ARM64_VST2_Q2
# define _ARM_VST2_Q4               _ARM64_VST2_Q4


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD2_D2_AIA           _ARM64_VLD2_D2_AIA
# define _ARM_VLD2_D4_AIA           _ARM64_VLD2_D4_AIA

# define _ARM_VLD2_Q2_AIA           _ARM64_VLD2_Q2_AIA


# define _ARM_VST2_D2_AIA           _ARM64_VST2_D2_AIA
# define _ARM_VST2_D4_AIA           _ARM64_VST2_D4_AIA

# define _ARM_VST2_Q2_AIA           _ARM64_VST2_Q2_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1)
		 */

# define _ARM_VLD2_D2_IA            _ARM64_VLD2_D2_IA
# define _ARM_VLD2_D4_IA            _ARM64_VLD2_D4_IA

# define _ARM_VLD2_Q2_IA            _ARM64_VLD2_Q2_IA
# define _ARM_VLD2_Q4_IA            _ARM64_VLD2_Q4_IA


# define _ARM_VST2_D2_IA            _ARM64_VST2_D2_IA
# define _ARM_VST2_D4_IA            _ARM64_VST2_D4_IA

# define _ARM_VST2_Q2_IA            _ARM64_VST2_Q2_IA
# define _ARM_VST2_Q4_IA            _ARM64_VST2_Q4_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM_VLD3_D3               _ARM64_VLD3_D3
# define _ARM_VLD3_Q3               _ARM64_VLD3_Q3

# define _ARM_VST3_D3               _ARM64_VST3_D3
# define _ARM_VST3_Q3               _ARM64_VST3_Q3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM_VLD3_D3_AIA           _ARM64_VLD3_D3_AIA
# define _ARM_VLD3_Q3_AIA           _ARM64_VLD3_Q3_AIA

# define _ARM_VST3_D3_AIA           _ARM64_VST3_D3_AIA
# define _ARM_VST3_Q3_AIA           _ARM64_VST3_Q3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM_VLD3_D3_IA            _ARM64_VLD3_D3_IA
# define _ARM_VLD3_D3_IA            _ARM64_VLD3_D3_IA

# define _ARM_VST3_D3_IA            _ARM64_VST3_D3_IA
# define _ARM_VST3_Q3_IA            _ARM64_VST3_Q3_IA


/* ------------------------------------------------------------------------- */

	/*
	 * Load 4-element structures from memory (PRR: base address) into vector registers
	 * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
	 */

# define _ARM_VLD4_D4               _ARM64_VLD4_D4
# define _ARM_VLD4_Q4               _ARM64_VLD4_Q4

# define _ARM_VST4_D4               _ARM64_VST4_D4
# define _ARM_VST4_Q4               _ARM64_VST4_Q4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD4_D4_AIA           _ARM64_VLD4_D4_AIA
# define _ARM_VLD4_Q4_AIA           _ARM64_VLD4_Q4_AIA

# define _ARM_VST4_D4_AIA           _ARM64_VST4_D4_AIA
# define _ARM_VST4_Q4_AIA           _ARM64_VST4_Q4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM_VLD4_D4_IA            _ARM64_VLD4_D4_IA
# define _ARM_VLD4_Q4_IA            _ARM64_VLD4_Q4_IA

# define _ARM_VST4_D4_IA            _ARM64_VST4_D4_IA
# define _ARM_VST4_Q4_IA            _ARM64_VST4_Q4_IA

/* ========================================================================= */

/*
 * Load (with duplicate) single N-element structure to all lanes (into all elements of vector registers)
 */

    /*
     * Load 1-element structure from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, ...): (Dn) or (Dn,Dn+1)
	 * (VRI0, ...): (Qn)
     */

# define _ARM_VLD1_DUP_D1           _ARM64_VLD1_DUP_D1
# define _ARM_VLD1_DUP_D2           _ARM64_VLD1_DUP_D2

# define _ARM_VLD1_DUP_Q1           _ARM64_VLD1_DUP_Q1


		/*
		 * With auto-increment
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1) or (Dn,Dn+1,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn) or (Qn,Qn+1) or (Qn,Qn+1,Qn+2) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD1_DUP_D1_AIA       _ARM64_VLD1_DUP_D1_AIA
# define _ARM_VLD1_DUP_D2_AIA       _ARM64_VLD1_DUP_D2_AIA

# define _ARM_VLD1_DUP_Q1_AIA       _ARM64_VLD1_DUP_Q1_AIA


		/*
		 * With offset update
		 * Load 1-element structure from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn) or (Dn,Dn+1)
		 * (VRI0, ...): (Qn)
		 */

# define _ARM_VLD1_DUP_D1_IA        _ARM64_VLD1_DUP_D1_IA
# define _ARM_VLD1_DUP_D2_IA        _ARM64_VLD1_DUP_D2_IA

# define _ARM_VLD1_DUP_Q1_IA        _ARM64_VLD1_DUP_Q1_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
     */

# define _ARM_VLD2_DUP_D2           _ARM64_VLD2_DUP_D2

# define _ARM_VLD2_DUP_Q2           _ARM64_VLD2_DUP_Q2


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, ...): (Dn,Dn+1) or (Dn,Dn+2) or (Dn,Dn+1,Dn+2,Dn+3)
		 * (VRI0, ...): (Qn,Qn+1) or (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD2_DUP_D2_AIA       _ARM64_VLD2_DUP_D2_AIA

# define _ARM_VLD2_DUP_Q2_AIA       _ARM64_VLD2_DUP_Q2_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1): (Dn,Dn+1) or (Dn,Dn+2)
		 */

# define _ARM_VLD2_DUP_D2_IA        _ARM64_VLD2_DUP_D2_IA

# define _ARM_VLD2_DUP_Q2_IA        _ARM64_VLD2_DUP_Q2_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
     */

# define _ARM_VLD3_DUP_D3           _ARM64_VLD3_DUP_D3

# define _ARM_VLD3_DUP_Q3           _ARM64_VLD3_DUP_Q3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 * (VRI0, VRI1, VRI2): (Qn,Qn+1,Qn+2)
		 */

# define _ARM_VLD3_DUP_D3_AIA       _ARM64_VLD3_DUP_D3_AIA

# define _ARM_VLD3_DUP_Q3_AIA       _ARM64_VLD3_DUP_Q3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2): (Dn,Dn+1,Dn+2) or (Dn,Dn+2,Dn+4)
		 */

# define _ARM_VLD3_DUP_D3_IA        _ARM64_VLD3_DUP_D3_IA

# define _ARM_VLD3_DUP_Q3_IA        _ARM64_VLD3_DUP_Q3_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
	 *
	 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
     */

# define _ARM_VLD4_DUP_D4           _ARM64_VLD4_DUP_D4

# define _ARM_VLD4_DUP_Q4           _ARM64_VLD4_DUP_Q4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 * (VRI0, VRI1, VRI2, VRI3): (Qn,Qn+1,Qn+2,Qn+3)
		 */

# define _ARM_VLD4_DUP_D4_AIA       _ARM64_VLD4_DUP_D4_AIA

# define _ARM_VLD4_DUP_Q4_AIA       _ARM64_VLD4_DUP_Q4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (VRI0, VRI1, VRI2, VRI3): (Dn,Dn+1,Dn+2,Dn+3) or (Dn,Dn+2,Dn+4,Dn+6)
		 */

# define _ARM_VLD4_DUP_D4_IA        _ARM64_VLD4_DUP_D4_IA

# define _ARM_VLD4_DUP_Q4_IA        _ARM64_VLD4_DUP_Q4_IA

/* ========================================================================= */

/*
 * Load/Store single N-element structure to/from one lane
 */

	/*
     * Load 1-element structures from memory (PRR: base address) into vector registers
     * Store 1-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI): (Dn[L])
     */

# define _ARM_VLD1_LANE_D1          _ARM64_VLD1_LANE_D1

# define _ARM_VST1_LANE_D1          _ARM64_VST1_LANE_D1


		/*
		 * With auto-increment
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM_VLD1_LANE_D1_AIA      _ARM64_VLD1_LANE_D1_AIA

# define _ARM_VST1_LANE_D1_AIA      _ARM64_VST1_LANE_D1_AIA


		/*
		 * With offset update
		 * Load 1-element structures from memory (PRR: base address) into vector registers.
		 * Store 1-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI): (Dn[L])
		 */

# define _ARM_VLD1_LANE_D1_IA       _ARM64_VLD1_LANE_D1_IA

# define _ARM_VST1_LANE_D1_IA       _ARM64_VST1_LANE_D1_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 2-element structures from memory (PRR: base address) into vector registers
     * Store 2-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
     */

# define _ARM_VLD2_LANE_D2          _ARM64_VLD2_LANE_D2

# define _ARM_VST2_LANE_D2          _ARM64_VST2_LANE_D2


		/*
		 * With auto-increment
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM_VLD2_LANE_D2_AIA      _ARM64_VLD2_LANE_D2_AIA

# define _ARM_VST2_LANE_D2_AIA      _ARM64_VST2_LANE_D2_AIA


		/*
		 * With offset update
		 * Load 2-element structures from memory (PRR: base address) into vector registers.
		 * Store 2-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L,VRI0,VRI1): (Dn[L],Dn+1[L])
		 */

# define _ARM_VLD2_LANE_D2_IA       _ARM64_VLD2_LANE_D2_IA

# define _ARM_VST2_LANE_D2_IA       _ARM64_VST2_LANE_D2_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 3-element structures from memory (PRR: base address) into vector registers
     * Store 3-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
     */

# define _ARM_VLD3_LANE_D3          _ARM64_VLD3_LANE_D3

# define _ARM_VST3_LANE_D3          _ARM64_VST3_LANE_D3


		/*
		 * With auto-increment
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM_VLD3_LANE_D3_AIA      _ARM64_VLD3_LANE_D3_AIA

# define _ARM_VST3_LANE_D3_AIA      _ARM64_VST3_LANE_D3_AIA


		/*
		 * With offset update
		 * Load 3-element structures from memory (PRR: base address) into vector registers.
		 * Store 3-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2): (Dn[L],Dn+1[L],Dn+2[L])
		 */

# define _ARM_VLD3_LANE_D3_IA       _ARM64_VLD3_LANE_D3_IA

# define _ARM_VST3_LANE_D3_IA       _ARM64_VST3_LANE_D3_IA

/* ------------------------------------------------------------------------- */

    /*
     * Load 4-element structures from memory (PRR: base address) into vector registers
     * Store 4-element structures to memory (PRR: base address) from vector registers
	 *
	 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
     */

# define _ARM_VLD4_LANE_D4          _ARM64_VLD4_LANE_D4

# define _ARM_VST4_LANE_D4          _ARM64_VST4_LANE_D4


		/*
		 * With auto-increment
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + number of bytes transferred) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM_VLD4_LANE_D4_AIA      _ARM64_VLD4_LANE_D4_AIA

# define _ARM_VST4_LANE_D4_AIA      _ARM64_VST4_LANE_D4_AIA


		/*
		 * With offset update
		 * Load 4-element structures from memory (PRR: base address) into vector registers.
		 * Store 4-element structures to memory (PRR: base address) from vector registers.
		 *
		 * PRR register updated to (PRR + Offset-register) after all the loads/stores
		 * have taken place.
		 *
		 * (L, VRII0, VRII1, VRII2, VRII3): (Dn[L],Dn+1[L],Dn+2[L],Dn+3[L])
		 */

# define _ARM_VLD4_LANE_D4_IA       _ARM64_VLD4_LANE_D4_IA

# define _ARM_VST4_LANE_D4_IA       _ARM64_VST4_LANE_D4_IA

/* @} */


/* ========================================================================= */

/*
 * Print
 * @{
 */

# define __ARM_VPRINT_LANE_D1       __ARM64_VPRINT_LANE_D1
# define __ARM_VPRINTF32_LANE_D1    __ARM64_VPRINTF32_LANE_D1

/* @} */
/* @} */

#endif /* __VIPM_ARM_DEFS__ */
/* @} */

#endif /* !defined(__VIPM_ARM64_ASM_GCC__) */


/******************************************************************************
 *
 *
 *
 *****************************************************************************/
